proc extract {var bits} {
    upvar $var v

    set result [expr $v & ((1 << $bits) - 1)]
    set v [expr $v >> $bits]
    return $result
}



# -----------------------------------------------------------------
# igu_dump_fifo
#
# Dump up to 8 IGU fifo entries
# Dump the first 8 IGU errors or
# the last 8 IGU commands if IGU_REG_COMMAND_DEBUG is set.
#
# Parameters:        None
# Return value:      None
# -----------------------------------------------------------------
proc  igu_dump_fifo {} {
    global rdfFile
    
    # Parse the RDF file
    #get RDF file name according to DID
    get_rdf_file
    parse_rdf -f $rdfFile
        
    set chip_num [reg_read -n MISC_REG_CHIP_NUM]
    if {[is_e1x]} {
        error "Only supported on E2/E3"
    }
    if {[reg_read -n IGU_REG_ERROR_HANDLING_DATA_VALID] == 0} {
        error "IGU fifo is empty"
    }

    if {[set parseCommands [reg_read -n IGU_REG_COMMAND_DEBUG]]} {
        puts "Last IGU Commands: "
    } else {
        puts "First IGU Errors:"
    }
    # Read IGU fifo until it is empty
    while {[reg_read -n IGU_REG_ERROR_HANDLING_DATA_VALID]} {
        scan [reg_offset -n IGU_REG_ERROR_HANDLING_MEMORY] "0x%x-0x%x" lsb_addr msb_addr
        # Read WB register
        set lsb [reg read $lsb_addr]
        set msb [reg read $msb_addr]
        # Parse data
        set cmd_wr_n_rd [lindex {read write} [extract lsb 1]]
        set cmd_length [extract lsb 3]
        set cmd_func_err [extract lsb 1]
        set error_source [lindex {pxp grc attn <invalid>} [extract lsb 2]]
        set cmd_source [lindex {Chip Host} [extract lsb 1]]
        set cmd_fid [extract lsb 7]
        if {$parseCommands} {
            extract lsb 1
            set cmd_addr [format "0x%x" [extract lsb 15]]
        } else {
            set error_type_d [lindex {<invalid> "length >1"  "command to disable function" 
                "VF command to attn bits" "host tries to update prod" "during read from MIMD function"
                "segment not match" "attn prod upd" <invalid> "SB index not valid" "FID + vector num not found"
                "FID not match" "VF tries to access attn command"} [extract lsb 4]]
            set cmd_addr [format "0x%x" [extract lsb 13]]
        }
        set wr_data [format "0x%x" $msb]
        # decode cmd_fid
        if {$cmd_fid & 0x40} {
            set cmd_fid [format "PF %d" [expr $cmd_fid & 0x3F]]
        } else {
            set cmd_fid [format "VFID 0x%x" $cmd_fid]
        }
        puts " $cmd_wr_n_rd $cmd_length qwords addr=$cmd_addr data=$wr_data func_err=$cmd_func_err src=$cmd_source err_src=$error_source fid=$cmd_fid error=$error_type_d"
    }
}