proc set_duart {action {com {1}} {baud {115200}} {setting {8N1}}} {
	if {[string equal $action open]} {
		if {![regexp {\d+ \d+} "$com $baud"]} {
			puts "set_duart open must recieve a com port and baud (integers)"
			return -1
		}
		
		## For baud rate 115200, special DBU registers setting is required
		if {$baud == 115200} {
			puts "Setting serialport to baud rate 115200 .."
			serialport $action $com $baud $setting
			puts "Set variable ::sys(DUART_PORT) to com $com"
			set ::sys(DUART_PORT) $com
			## If serial open at baud 115200 failed, use default baud rate
			## to set DBU registers for 115200 prior to serialport open
			if {[catch {serialport redirect duart}]} {
				puts "Serialport redirect failed .. closing. Setting serialport to baud rate 19200 .."
				serialport close
				serialport $action $com 19200 $setting			
				if {[catch {serialport redirect duart}]} {
                                        ## If we got here, connection has failed in two baud rates.
                                        ## probably cable is disconnected.
					puts "Serialport redirect failed .. closing."
					serialport close
					puts "Did not succeed to open com $com with baud 19200"
					puts "Please validate that serial cable is connected and try again."
					return -1
				} else {
					## Set DBU registers for baud 115200
					puts "Setting DBU registers values for baud rate 115200 support"
					reg write 0x880c 0xd8006c -no_echo				
					after 500
					reg write 0x8808 0x7 -no_echo						
					after 500
					## Close the defualt baud serial port
					serialport close
					puts "Closing serialport at baud rate 19200 .."
					## Now set baud to 115200
					puts "Try again setting serialport to baud rate 115200 .."
					serialport $action $com $baud $setting
					if {[catch {serialport redirect duart}]} {
						puts "Setting serialport to baud rate 115200 failed !"
					} else {
						puts "Setting serialport to baud rate 115200 succeeded !!"
					}
				}
			} else {
				puts "Setting serialport to baud rate 115200 succeeded !!"	
			}
		} else {
			puts "Setting serialport to baud rate $baud .."
			serialport $action $com $baud $setting
			puts "Set variable ::sys(DUART_PORT) to com $com"
			set ::sys(DUART_PORT) $com
			## If setting baud rate failed, use 115200 baud rate
			## to set DBU registers for defualt setting
			if {[catch {serialport redirect duart}]} {
				puts "Serialport redirect failed .. closing. Trying baud rate 115200 .."
				serialport close
				serialport $action $com 115200 $setting				
				if {[catch {serialport redirect duart}]} {
                                        ## If we got here, connection has failed in two baud rates.
                                        ## probably cable is disconnected.
					puts "Serialport redirect failed .. closing."
					serialport close				
					puts "Did not succeed to open com $com with baud 115200"
					puts "Please validate that serial cable is connected and try again."
					return -1
				} else {
					## Set DBU registers to default setting
					puts "Setting DBU registers to default values"
					reg write 0x880c 0x516028b -no_echo				
					after 500
					reg write 0x8808 0x6 -no_echo						
					after 500
					## Close the defualt baud serial port
					serialport close
					puts "Closing serialport at baud rate 115200 .."				
					puts "Try again setting serialport to baud rate $baud .."
					## Now set baud to the requested baud rate
					serialport $action $com $baud $setting
					if {[catch {serialport redirect duart}]} {
						puts "Setting serialport to baud rate $baud failed !"
					} else {
						puts "Setting serialport to baud rate $baud succeeded !!"
					}
				}
			} else {
				puts "Setting serialport to baud rate $baud succeeded !!"
			}
			
		}
		set ::sys(DUART_PORT) $com
		
	} elseif {[string equal $action close]} {
		serialport $action
		set ::sys(DUART_PORT) 1
	} else {
		puts "unknown action for set_duart, allowed actions: open, close."
		return -1
	}
	return 0
}

#
# Redirect IO to a com port
#
proc redirect {{port 1}} {
    set ::sys(IO_PORT) $port
    #set ::sys(STAT_PORT) $port
    serialport open $port
    serialport redirect
}

add_help set_duart "Access chip via remote machine through serial port.\n"
