# This TCL script configure the debug bus/block in the chip.
# It is also used to read the data if working with the internal buffer or with the host's ram.
# Written by AmirF, Broadcom, December 2007

# In order to understand the code, it's best to read the DEBUG BUS USER MANUAL first


# NAMESPACE dbgTools
# Specify any dependency on other packages
package require Tcl

# Specify the package this module provides
package provide dbgTools 1.0


namespace eval dbgTools {
    
    # commands needed from the main namesapce
    namespace export setProfile dbgOn dbgEnd dbgOff getWrPtr dbgVer getStaticLineProfile staticDebug helpStaticDebug E1FixStaticDebug helpE1FixStaticDebug UARTChangePath kernelDebug
	variable first_write_by_name true
    # Global Variables in the dbgTools namespace
    # When dbg_no_pci equals 1, saving or getting data from the host's ram is impossible.
    variable dbg_no_pci 0
    variable current_path_num 0
    variable version {$Revision$}
	
	variable dumpString ""
	proc clearDumpString {} {
		variable dumpString
		set dumpString ""
	}
	
	# This variable saves for each configuration, the start offset and num registers for: dbgOn, dbgOff, cfg, and flush
	# for example, dump_counter_info(10,dbgOn, start) - start offset for configuration 10 for the dbgOn register
	
	variable dump_counter_info
	variable force_e3b false
	array set dump_counter_info {}
	
	variable dumph_current_config_number 0
	
	proc addDumpString {val} {
		variable dumpString
		set dumpString "[set dumpString][set val]"
	}
	
	proc getDumpString {} {
		variable dumpString
		return $dumpString
	}
	
	# write_by_name_counter - This is a counter that counter the number of 
	# registers written by the write_by_name_counter. It is used for
	# Kernel Debug feature for outputting sets of register writes for each
	# configuration.
	variable write_by_name_counter 0
	
	proc increaseWriteByNameCounter {} {
		variable write_by_name_counter
		incr write_by_name_counter
	}
	
	proc getWriteByNameCounter {} {
		variable write_by_name_counter
		return $write_by_name_counter
	}
	
	proc clearWriteByNameCounter {} {
		variable write_by_name_counter
		set write_by_name_counter 0
	}
	
	
	
	
	variable ME_register_address 0x450000
    ### CHIP AND REVISION 
    # useful to acknowledged the chip we worked with (E1 or E1H).
    # Correlation between chip num to everest versions.
    variable CHIPS
    array set CHIPS {0x164E 1 0x164F 1.5 0x164e 1 0x164f 1.5 0x1650 1.5 0x1662 2 0x1663 2 0x168e 3 0x168E 3 0x168d 3 0x168D 3 0x168a 3 0x168A 3 0x16a5 3 0x16A5 3 0x16ab 3 0x16AB 3}
	# The following is used to make the script behave as if working on a specific Chip.
	variable CHIP_ID
	array set CHIP_ID {1 0x164E 1.5 0x1650 2 0x1662 3 0x168E 3b 0x168E}
	variable CHIP_CSV_FOLDER
	array set CHIP_CSV_FOLDER {1 CSVE1 1.5 CSVE1H 2 CSVE2 3 CSVE3 3b CSVE3B}
    # MISC_REG_CHIP_NUM
    # MISC_REG_CHIP_REV - must be hardcoded since this will choose which rdf to parse
    variable chip_num_reg 0xa408
    variable PRAM_FW_FLAGS
    # Correlation between the flags in the fw version in the pram to the chip num
    array set PRAM_FW_FLAGS {0 1 1 1.5 2 2 3 3}
    # When in E2/E3, should data be dumped to the other path
	variable other_path 0
    # variable for the current csv folder initiliaze after recognizing the chip id
    variable csv_folder
    
    # A variable for knowing when the current debug block status {NONE CONFIGURED FLUSHED RUNNING DUMP_NO_PCI}
    variable DBG_STATUS NONE
    
    # A variable for knowing whether working in wrap or one-shot mode on the buffer (PCI or BUFFER) - important for dump procedures
    variable full_mode 0
    
    # a pointer for the memory that has been allocated for the debug data when working in PCI mode
    variable mem_addr 0
    
    # store the amount of memory allocated for debug data in KB
    variable pci_size 0
    
    # store to output mode (PCI, NETWORK or BUFFER) used in dbgEnd for knowing whether or not to output data and how
    variable OUTMODE -1
    
    # A variable for signing DUMP mode
    variable DUMP 0
    
    # A output file for dump data
    variable dumpFile "closed"
    
    # For future usage - different features between revisions
    # set chip_rev_reg 0xa40c
    variable chip_num [reg read $chip_num_reg]
    # For future usage
    #set chip_rev [reg read $chip_rev_reg]
    # adjusting to emulation and fpga revisions.
    # fpga ver ==> 15-2*$rev emul_ver ==> 14-2*$rev
    #if {$chip_rev> 4} {
    #    set chip_rev [expr {int((15-$chip_rev )/2)}]
    #}
    
    # chip_ver is either 1 or 1.5.
    variable chip_ver $CHIPS($chip_num)
    
    # check to see the driver is loaded on the machine
    # if the machine is working on windows or with serial connection the script assume the driver is loaded
    # (the NO_PCI variable exists only in versions newer than 0.23.2)
    if {[info exists ::sys(NO_PCI)]} {
       set dbg_no_pci $::sys(NO_PCI)
    } else {
       set dbg_no_pci [regexp {NO_PCI} $::tcl_prompt1]
    }

    # The script required that the driver will be running before sourcing it.
    # if the variable dbgTools_no_driver is set, the script will not perform cleaning
    if {($::tcl_platform(os)!="Windows NT")  && ($::tcl_platform(os)!="Linux") && ($::tcl_platform(os)!="VMkernel") && !($::current(DRV_STATE) == "RUNNING") &&   
        !$dbg_no_pci && ![info exists ::dbgTools_no_driver]} {
        puts "The script required that the driver will be running.\nPlease load the driver and source the script again."
        puts "If you want to work in advanced mode without the driver,\nplease write \"set dbgTools_no_driver 1\" before sourcing the script"
        return
    }

		
    # regutils add the interface which translate registers name to addresses
    # from E-diag 4.0.10 the regutils was wrapped into the E-diag and there is no need to source it.
	# it is also wrapped into the Linux version so we should also check if it's not Linux
    regexp {(\d+)\.(\d+)\.(\d+)} [version] verA verA verB verC
    if {(((![string equal $::tcl_platform(os) DOS] && ![string equal $::tcl_platform(os) Linux] && ![string equal $::tcl_platform(os) VMkernel])) || !($verA>=4 && ($verB>0 || $verC>=10))) && ![array exists ::regsOffset]} {
        
        namespace eval :: {
            catch {source regutils.tcl}
        }
    } 
    
    # an alias for easing the usage
    interp alias {} setp {} dbgTools::setProfile profiles.dbg
    variable PDs
    # an array of Physical design block and their indexes in the chip
    array set PDs {}
    variable blocksPD
    # An array that connect each block with its PD block number. e.g. blocksPD(PBF)==3
    array set blocksPD {}
    variable usedBlocks
    # An array for all the blocks that have been used in the last run (Used for cleaning between profiles)
    array set usedBlocks {}
    variable errors
    # An array of errors (There are no consts in TCL)
    array set errors {EOF -1, line_not_found -2}
    variable LINETYPE
    # An array of numbers for defining the possible line types of debug data (There are no consts in TCL)
    array set LINETYPE {64BIT,64GROUP 0 64BIT,32GROUP 1 64BIT,TWO32GROUP 2 24BIT,8BIT 3 24BIT,8BIT8BIT 4 \
                        24BIT,THREE8BIT 5 24BIT,16BIT 6 24BIT,8BIT16BIT 7 24BIT,16BIT8BIT 8 24BIT,24BIT 9 24BITSTART 3}
    variable FORMATS
    # A list of allowed formats 
    set FORMATS {NORMAL STORE SEM FMT PRINTF STORM}
    # An array with all the operators in E1H debug block trigger machine.
    variable OPS
    array set OPS { "=" 0 ">" 1 ">=" 2 "=>" 2 "=<" 3 "<=" 3 "<" 4}
    variable usedRegs
    # An array for all the regs that have been used in the last run (Used for cleaning between profiles)
    array set usedRegs {}
    variable rdfs_files
    variable sdfs_files
    # rdfs and sdf files name per chip.
    array set rdfs_files {1 everest.rdf 1.5 evrst15.rdf 2 everst2.rdf 3 everst3.rdf 3b everst3b.rdf}
    array set sdfs_files {1 everest.sdf 1.5 evrst15.sdf 2 everst2.sdf 3 everst3.sdf 3b everst3b.sdf}
    variable stormOffset
    # Addresses inside the fast memory used for cleaning all registers upon start
    # The filter registers are also used for configuring
    array set stormOffset {debug_active 0x18ac0 debug_mode 0x18b00 dbg_waitp_disable 0x18c00 dbg_mode3_src_disable 0x18c40 \
                            dbg_mode6_src_disable 0x18c80 filter_cidreg 0x18F00 filter_confreg 0x18F40 filter_eidmaskreg 0x18FC0}
    # Array to hold the chip struct for using triggers
    variable filedsInfo
    array set filedsInfo {}
    # Array which holds hard coded data of number of cycles in a chunk (16 lines) per mode
    variable CYCLES_PER_MODE
    array set CYCLES_PER_MODE {0 56 1 31}
    
    # Whether the trigger used
    variable trigger_used
    
}

# Set dbgTools to work on another chip
proc dbgTools::changeChip {chip_id} {
	variable ::dbg_chip_id
	variable CHIP_ID
	variable CHIPS
	variable csv_folder
	variable chip_num_reg
	
	set ::dbg_chip_id $CHIP_ID($chip_id)
	proc ::reg {args} {
		variable ::dbg_chip_id
		return $::dbg_chip_id
	}
	
	variable chip_num
	set chip_num [reg read $chip_num_reg]
	variable chip_ver
	set chip_ver $chip_id
	variable PDs
    array set PDs {}
    variable blocksPD
    array set blocksPD {}
    variable usedBlocks
    array set usedBlocks {}
	
	namespace eval :: {
            if {[parse_rdf -f $dbgTools::rdfs_files($dbgTools::chip_ver) -clr]!=0} {
                error "Problem parsing rdf file, make sure $dbgTools::rdfs_files($dbgTools::chip_ver) exist"
            }
        }
	
	# setting the relevant CSVs folders
	variable CHIP_CSV_FOLDER
	set csv_folder $CHIP_CSV_FOLDER($chip_id)

	puts "csv folder is $csv_folder"
	if {[parsePDblocks [format {%s/pdBlocks.txt} $csv_folder]]=="OK"} {
		puts "Parsed Blocks for chip"
		} else {
		error "Cannot parse Blocks for chip"
	}
	
}

proc	dbgTools::getDebugStatisticsForChip {} {
	variable blocksPD
	variable allBlocks
	
	set result {}
	
	set i 0
	
	foreach block $allBlocks {
	
		if {[regexp STORM $block]} {
			continue
		}
		
		set block_pd_num [set blocksPD([set block])]
		set block_num	[set i]
		incr i
		array unset curr_block_lines
		array set curr_block_lines {}
		getDebugLinesofBlock $block curr_block_lines
		set block_num_lines [set curr_block_lines(count)]
		set block_dbg_select [reg_offset -n [set block]_REG_DBG_SELECT]
		set block_dbg_byteen [reg_offset -n [set block]_REG_DBG_BYTE_ENABLE]
		if {[string equal $block_dbg_select ""] } {
			puts "Bad Block: $block"
			continue
		}
		lappend result [list $block $block_num $block_pd_num $block_num_lines $block_dbg_select $block_dbg_byteen $curr_block_lines(list)]
	}
	return $result
}


proc	dbgTools::getDebugStatistics {c_filename tcl_filename} {
	
	variable allDebugStatistics
	array set allDebugStatistics {}
	changeChip 1
	set allDebugStatistics(E1) [getDebugStatisticsForChip]
	changeChip 1.5
	set allDebugStatistics(E1H) [getDebugStatisticsForChip]
	changeChip 2
	set allDebugStatistics(E2) [getDebugStatisticsForChip]
	changeChip 3
	set allDebugStatistics(E3) [getDebugStatisticsForChip]
	changeChip 3b
	variable force_e3b
	set force_e3b true
	set allDebugStatistics(E3B) [getDebugStatisticsForChip]
	set force_e3b false
	
	set output_file [open $c_filename w]
	set output_tcl_file [open $tcl_filename w]
	puts $output_file {#ifndef _STATIC_DEBUG_DUMP_H}
	puts $output_file {#define _STATIC_DEBUG_DUMP_H}
	puts $output_file "\n"
	puts $output_file "/* This file was auto-generated. It contains all the needed meta data for"
	puts $output_file "* the Static Debug feature of Kernel Dump."
	puts $output_file "*/"
	puts $output_file "struct	static_debug_general_debug_info \{"
	puts $output_file "\tu32\tdbg_on_offset;"
	puts $output_file "\tu32\tclient_en_offset;"
	puts $output_file "\tu32\tframing_mode_offset;"
	puts $output_file "\tu32\ttdm_offset;"
	puts $output_file "\tu32\tfull_mode_offset;"
	puts $output_file "\tu32\tdata_out_lo_offset;"
	puts $output_file "\tu32\tdata_out_hi_offset;"
	puts $output_file "\tu32\tdata_out_meta_offset; /* bit 0 is frame, bit 4 is valid */"
	puts $output_file "\};\n\n"
	puts $output_file "struct	static_debug_block_info \{"
	puts $output_file "\tu32\tindex;"
	puts $output_file "\tu32\tpd_num;"
	puts $output_file "\tu32\tnum_dbg_lines;"
	puts $output_file "\tu32\tdbg_select_offset;"
	puts $output_file "\tu32\tdbg_byteen_offset;"
	puts $output_file "\};\n\n"
	

	puts $output_file "struct static_debug_output_record"
	puts $output_file "\{"
	puts $output_file "\tu32	block;"
	puts $output_file "\tu32	line_num;"
	puts $output_file "\tu32	lo;"
	puts $output_file "\tu32	hi;"
	puts $output_file "\tu32 meta;"
	puts $output_file "\};\n"
	

	puts $output_file "typedef enum	\{"
	puts $output_file "\tSD_CHIP_E1,"
	puts $output_file "\tSD_CHIP_E1H,"
	puts $output_file "\tSD_CHIP_E2,"
	puts $output_file "\tSD_CHIP_E3,"
	puts $output_file "\tSD_CHIP_E3B"
	puts $output_file "\}	SdChipType;\n\n"

	puts $output_file "\#ifdef STATIC_DEBUG_GENERATE"
	puts $output_file ""
	puts $output_file {#if !defined(SD_READ_REG)}
	puts $output_file {#define SD_READ_REG(_ctx,_size) (0)}
	puts $output_file {#endif}
	puts $output_file ""
	puts $output_file {#if !defined(SD_WRITE_REG)}
	puts $output_file {#define SD_WRITE_REG(_ctx,_addr,_val) /**/}
	puts $output_file {#endif}
	puts $output_file ""
	puts $output_file {#if !defined(SD_ADD_DATA_TO_DUMP)}
	puts $output_file {#define SD_ADD_DATA_TO_DUMP(_ctx, _record, _size) (0)}
	puts $output_file {#endif}
	puts $output_file "\nstatic const struct static_debug_general_debug_info debug_offsets = \{"
	puts $output_file "\t[reg_offset -n DBG_REG_DBG_BLOCK_ON],"
	puts $output_file "\t[reg_offset -n DBG_REG_CLIENT_ENABLE],"
	puts $output_file "\t[reg_offset -n DBG_REG_FRAMING_MODE],"
	puts $output_file "\t[reg_offset -n DBG_REG_TDM64_ENABLE],"
	puts $output_file "\t[reg_offset -n DBG_REG_FULL_MODE],"
	
	set start_off [get_start_offset [reg_offset -n DBG_REG_CALENDAR_OUT_DATA]]
	puts $output_file "\t[set start_off],"
	
	set next_off [format "0x%.8x" [expr {$start_off + 4}]]
	puts $output_file "\t[set next_off],"
	
	set next_off [format "0x%.8x" [expr {$next_off + 4}]]
	puts $output_file "\t[set next_off]"
	
	puts $output_file "\};\n"
	foreach chip {E1 E1H E2 E3 E3B} {
		set static_debug_accumulated_num_[set chip] 0
		puts	$output_file "static const struct	static_debug_block_info	[string tolower $chip]_blocks\[\] = \{";
		set curr_blocks $allDebugStatistics($chip)
		set counter 0
		set num_blocks [llength $curr_blocks]
		set add_comma ","
		set current_tcl_string ""
		foreach block $curr_blocks {
			if {[expr {$num_blocks - $counter}] == 1} {
				set add_comma ""
			}
			incr counter
			puts $output_file "\t\{ [lindex $block 1], [lindex $block 2], [lindex $block 3], [lindex $block 4], [lindex $block 5]\}[set add_comma]"
			lappend current_tcl_string [list [lindex $block 0] [lindex $block 1] [lindex $block 2] [lindex $block 3] [lindex $block 4] [lindex $block 5] [lindex $block 6]]
			incr static_debug_accumulated_num_[set chip] [lindex $block 3]
		}
		puts $output_tcl_file "set static_debug_meta_[set chip] \{[set current_tcl_string]\}"
		puts $output_file	"\};\n";
	}
	puts $output_file	"#ifndef RI_TYPE_STATIC_DEBUG"
	puts $output_file	"#define RI_TYPE_STATIC_DEBUG 3"
	puts $output_file	"#endif RI_TYPE_STATIC_DEBUG"
	
	puts $output_file	"\n/*	staticDebugDump: Adds static debug data to dump file."
	puts $output_file	"*	Returns the number of added bytes."
	puts $output_file	"*	Used Macros:"
	puts $output_file	"*	SD_READ_REG(<address_in_bytes>) - reads a register"
	puts $output_file	"*	SD_WRITE_REG(<address_in_bytes>,<value>) - writes a DWORD value to a register"
	puts $output_file	"*	SD_ADD_DATA_TO_DUMP(<pointer to data>,<size in bytes>) - adds data to dump file"
	puts $output_file	"*/\n"
	puts $output_file	"u32	staticDebugDump(void *ctx, SdChipType chip_type)"
	puts $output_file	"\{"
	puts $output_file	"\tu32 added_data = 0;"
	puts $output_file	"\tu32	block_index = 0;"
	puts $output_file	"\tu32 line_index = 0;"
	puts $output_file	"\tconst struct static_debug_block_info *p_curr_block = NULL;"
	puts $output_file	"\tstruct static_debug_output_record	output_data = \{0\};"
	puts $output_file	"\tu32 num_data_records = 0;"
	puts $output_file	"\tconst struct static_debug_block_info *p_block_data = NULL;"
	puts $output_file	"#define SD_CHECK_CASE(value,block_arr) \\"
	puts $output_file	"\tcase (value):	p_block_data = block_arr; \\"
	puts $output_file	"\tnum_data_records = (sizeof(block_arr)) / (sizeof((block_arr)\[0\])); \\"
	puts $output_file	"\tbreak\n"

	puts $output_file	"\tswitch (chip_type) \{"
	puts $output_file	"\t\tSD_CHECK_CASE(SD_CHIP_E1,e1_blocks);"
	puts $output_file	"\t\tSD_CHECK_CASE(SD_CHIP_E1H,e1h_blocks);"
	puts $output_file	"\t\tSD_CHECK_CASE(SD_CHIP_E2,e2_blocks);"
	puts $output_file	"\t\tSD_CHECK_CASE(SD_CHIP_E3,e3_blocks);"
	puts $output_file	"\t\tSD_CHECK_CASE(SD_CHIP_E3B,e3b_blocks);"
	puts $output_file	"\t\tdefault:	return 0;"
	puts $output_file	"\t\}"
	puts $output_file	"\n#undef SD_CHECK_CASE\n"
	puts $output_file	"\t// Turn off all blocks"
	puts $output_file	"\tfor (block_index=0; block_index<num_data_records; ++block_index)"
	puts $output_file	"\t\{"
		puts $output_file	"\t\tSD_WRITE_REG(ctx,p_block_data\[block_index\].dbg_byteen_offset,0);"
	puts $output_file	"\t\}\n"
	puts $output_file	"\t// Reset Debug Configuration"
	puts $output_file	"\tSD_WRITE_REG(ctx,debug_offsets.client_en_offset,0);"
	puts $output_file	"\tSD_WRITE_REG(ctx,debug_offsets.framing_mode_offset,1);"
	puts $output_file	"\tSD_WRITE_REG(ctx,debug_offsets.tdm_offset,0);"
	puts $output_file	"\tSD_WRITE_REG(ctx,debug_offsets.full_mode_offset,1);\n"

	

	puts $output_file	"\t// now, go over all the blocks"
	puts $output_file	"\tfor (block_index=0; block_index<num_data_records; ++block_index)"
	puts $output_file "\t\{"
		puts $output_file	"\t\tp_curr_block = &(p_block_data\[block_index\]);"
		puts $output_file	"\t\tSD_WRITE_REG(ctx,debug_offsets.client_en_offset,(1 << p_curr_block->pd_num));"
		puts $output_file	"\t\tSD_WRITE_REG(ctx,p_curr_block->dbg_byteen_offset,0xffffffff);"
		puts $output_file	"\t\tfor (line_index = 0; line_index < p_curr_block->num_dbg_lines; ++line_index)"
	puts $output_file "\t\t\{"
			puts $output_file	"\t\t\tSD_WRITE_REG(ctx,p_curr_block->dbg_select_offset,line_index);"
			puts $output_file	"\t\t\tSD_WRITE_REG(ctx,debug_offsets.dbg_on_offset,1);"
			puts $output_file "\t\t\toutput_data.lo = SD_READ_REG(ctx,debug_offsets.data_out_lo_offset);"
			puts $output_file "\t\t\toutput_data.hi = SD_READ_REG(ctx,debug_offsets.data_out_hi_offset);"
			puts $output_file	"\t\t\toutput_data.meta = SD_READ_REG(ctx,debug_offsets.data_out_meta_offset);"
			puts $output_file	"\t\t\tif ((output_data.meta >> 4) & 1)"
			puts $output_file	"\t\t\t\{"
				puts $output_file "\t\t\t\toutput_data.block = block_index;"
				puts $output_file "\t\t\t\toutput_data.line_num = line_index;"
				puts $output_file "\t\t\t\tif (!SD_ADD_DATA_TO_DUMP(ctx,&output_data,sizeof(output_data)))"
				puts $output_file "\t\t\t\t\{"
				puts $output_file "\t\t\t\t\treturn 0;"
				puts $output_file "\t\t\t\t\}"
				puts $output_file "\t\t\t\tadded_data += sizeof(output_data);"
			puts $output_file	"\t\t\t\}"
			
			puts $output_file	"\t\t\tSD_WRITE_REG(ctx,debug_offsets.dbg_on_offset,0);"
	puts $output_file "\t\t\}"
	puts $output_file	"\t\tSD_WRITE_REG(ctx,p_curr_block->dbg_byteen_offset,0);"
	puts $output_file "\t\}"
	puts $output_file "\treturn added_data;"
	puts $output_file "\}"
	close $output_tcl_file
	
	
	puts $output_file {#endif //_STATIC_DEBUG_GENERATED}
	puts $output_file ""
	foreach chip {E1 E1H E2 E3 E3B} {
		puts $output_file "\#define STATIC_DEBUG_MAX_SIZE_[set chip] (([set static_debug_accumulated_num_[set chip]]) * sizeof(struct static_debug_output_record))"
	}
	puts $output_file ""
	puts $output_file {#endif // STATIC_DEBUG_DUMP_H}
	
	close $output_file
	
	
	
	
	
}


# return the debug bus version according to the perforce
proc dbgTools::dbgVer {} {
    variable version
    return "DbgTools Version (perforce) $version"
}

# this subroutine return whatever or not to E1H debug block new options exists.
proc dbgTools::dbgE1H {} {
    variable chip_ver
    if {$chip_ver>1 && $chip_ver<2} {
        return 1
    } else {
        return 0
    }
}

proc dbgTools::dbgE2 {} {
    variable chip_ver
    if {$chip_ver==2} {
        return 1
    } else {
        return 0
    }
}

proc dbgTools::dbgE3 {} {
    variable chip_ver
    if {$chip_ver==3} {
        return 1
    } else {
        return 0
    }
}

proc dbgTools::dbgE3B {} {
	variable force_e3b
	variable chip_ver
	if {$force_e3b} {
		return true
	}
	set chip_rev [reg read 0xA40C]
	if {$chip_ver == 3} {
		 if {($chip_rev == 0xE) || ($chip_rev == 0xF) || ($chip_rev == 0)} {
			return false } else {
			return true
		}
	}

	return 0
}

namespace eval dbgTools {
    # setting the relevant CSVs folders
    if {[dbgE1H]} {
        set csv_folder {CSVE1H}
    } else {
		if {[dbgE2]} {
			set csv_folder {CSVE2}
                } else {
		    if {[dbgE3B]} {
			set csv_folder {CSVE3B}
			
                    } elseif {[dbgE3]} {
                        set csv_folder {CSVE3}
                    } else {
                        set csv_folder {CSVE1}
                    }                
		}
    }
}

# subroutine for getting the start offset of a register after reg_offset
proc dbgTools::get_start_offset {{offset}}  {
    
    # Parse out the starting offset
    return  [lindex [split $offset "-"] 0]                 
}

# subroutine for converting number (hex) in hex format to binary format
proc dbgTools::hex2bin hex {
    set t [list 0 0000 1 0001 2 0010 3 0011 4 0100 \
	    5 0101 6 0110 7 0111 8 1000 9 1001 \
	    a 1010 b 1011 c 1100 d 1101 e 1110 f 1111 \
	    A 1010 B 1011 C 1100 D 1101 E 1110 F 1111]
    return [string map -nocase $t $hex]
}


# this subroutine recieve size in bits and return the number of registers need to be read (goes to the closet power of 2)
proc dbgTools::sizeToRegNumber {size} { 

	set regNumber [expr {int(pow(2,ceil(log($size)/log(2))))}]
	if {$regNumber<32} {set regNumber 32}
	return [expr {int($regNumber/32)}]

}

# recieve a list build out of lines and their number (<data>@<line_number> ) and index (which index in the list to get)
# return the data into line and the line_number into line_number
proc dbgTools::getNextLine {lines index line line_number} {
    upvar $line lineR 
    upvar $line_number line_numberR
    set lineR [split [lindex $lines $index] @]
    set line_numberR [lindex $lineR 1]
    set lineR [lindex $lineR 0]
}

# A sub routine which write the Write pointer of the internal buffer.
proc dbgTools::getWrPtr {} {
    puts [reg_read -n DBG_REG_INTR_BUFFER_WR_PTR]
}

# subroutine replacing reg show. return the request amount of bytes in the closest multiple of 4. 
# addr is the address to read and count is the amounts in bytes
# (reg show only prints to the screen, while the script needs a function that will return the data) 
proc dbgTools::reg_show {{addr} {count 4}} {
	set i 0
	set ans ""
	while {$count>0} {
		if {$i%8!=0} {
			append ans [format " %.8x" [reg read $addr]]
		} else {
			if {$i!=0} {
				append ans " "
			}
			append ans [format "%.7x:%.8x" $addr [reg read $addr]]
		}
		set addr [format "0x%x" [expr {$addr+4}]] 
		incr i
		incr count -4
	}		
	return $ans
}

# subroutine replacing hmem show. return the request amount of bytes in the closest multiple of 4. 
# addr is the address to read and count is the amounts in bytes
# (hmem show only prints to the screen, while the script needs a function that will return the data) 
proc dbgTools::hmem_show {{addr} {count 4}} {
	set i 0
	set ans ""
	while {$count>0} {
		if {$i%8!=0} {
			append ans [format " %.8x" [hmem read $addr]]
		} else {
			if {$i!=0} {
				append ans " "
			}
			set addr_lsb [lindex $addr 0]
			if {[llength $addr]>1} {
				set addr_msb [lindex $addr 1]
			} else {
				set addr [list [expr $addr] 0]
				set addr_msb 0
			}
			append ans [format "%.7x:%.8x" [expr {$addr_msb<<32 + $addr_lsb}] [hmem read $addr]]
		}
		set addr [Add64Bits $addr 4 true] 
		incr i
		incr count -4
	}		
	return $ans
}

# subroutine for getting start offset of registers by name and not by address
proc dbgTools::getOffsetByName {reg_name} {
    variable DUMP
    
    
    if {$DUMP==2} {
        return "\[get_start_offset \[reg_offset -n $reg_name\]\]"
    }
    
    if {[regexp (^0x|^\\d+$) $reg_name]} {
        set addr $reg_name
    } else {
        set addr [get_start_offset [reg_offset -n $reg_name]]
    }
    if {$DUMP==1 || $DUMP==4} {
        return "$addr"
    } else {
        exit -1
    }
    
}

# subroutine for write to registers by name and not by address
# if DUMP isn't 0, the function will only write the registers and the data to a file and won't actually write to them.
# If DUMP ==1 then the registers is by address and if DUMP ==2 then the registers is by name.
proc dbgTools::writeByName {reg_name data} {
    variable DUMP
    variable dumpFile
    
	set is_first false
	if {[getWriteByNameCounter] == 0} {
		set is_first true
	}
	increaseWriteByNameCounter
	
    if {$DUMP==2} {
        puts $dumpFile "writeByName $reg_name [format "0x%.8x" $data]"
	    return
    }
	
	
    if {[regexp (^0x|^\\d+$) $reg_name]} {
        set addr $reg_name
    } else {
        set addr [get_start_offset [reg_offset -n $reg_name]]
    }
	if {$DUMP==4} {
		set addition ","
		if {$is_first} {
			set addition ""
		}
		set addr [format "0x%.8x" [expr $addr]]
		set data [format "0x%.8x" [expr $data]]
		addDumpString  "[set addition]\n\t\{ $addr, $data\}"
		return
	}
    if {$DUMP==1} {
        puts $dumpFile "reg write [format "0x%.8x" $addr] [format "0x%.8x" $data]"
    } else {
        reg write $addr $data
    }
}

# subroutine for reading from registers by name
# if DUMP isn't 0, the function will only write the registers to a file and won't actually read from them.
# If DUMP ==1 then the registers is by address and if DUMP ==2 then the registers is by name.
proc dbgTools::readByName {reg_name} {
    variable DUMP
    variable dumpFile
    
    if {$DUMP==2} {
        puts $dumpFile "readByName $reg_name"
	    return
    }
    
    if {[regexp (^0x|^\\d+$) $reg_name]} {
        set addr $reg_name
    } else {
        set addr [get_start_offset [reg_offset -n $reg_name]]
    }
    if {$DUMP==1} {
        puts $dumpFile "reg read [format "0x%.8x" $addr]"
    } else {
        return [reg read $addr]
    }
}



# this subroutine recieve the raw data in hex format from "reg_show" and the size of a line of data 
# and returns the data as a binary list of lines  
proc dbgTools::format_mem {data size} {

	set regNumber [sizeToRegNumber $size]
	# deleting addresses
        regsub -all \[\\da-fA-F\]+: $data {} data 
	# deleting dots
        regsub -all \\.+\\s $data {} data
	#transfering the data into binary format
        set data [hex2bin $data]
	set regList [split $data { }]
	if {$regNumber>1} {
                set resList ""
		set index 0	
		set listSize [llength $regList]
                # joining the registers to the requested line size
                # The data is sorted by lsb first so the joining is done backward
		for {set i 0} {$i<$listSize} {incr i $regNumber} {
			set tempStr ""
			for {set j [expr {$regNumber-1}]} {$j>=0} {incr j -1} {
				append tempStr [lindex $regList [expr {$i+$j}]]
			}
			lappend resList $tempStr
		}
		return $resList
	}
	return $regList
	
}


# subroutine for configuring the calendar of the debug block
# received a list of slots  
proc dbgTools::cfg_cal {{slots}} {
  
    for {set i 0} {$i<[llength $slots]} {incr i} {
        writeByName DBG_REG_CALENDAR_SLOT$i [lindex $slots $i]
    }
    
}

# subroutine for configuring a block outputs to the debug block
# it recieved the block name, the requested line number in the dbgmux, the needed byte enable and the needed shift
# it also recieved a list of registers and value needed to be set
# if the recieved block is storm only the registers in the reg list will be set
# in other cases also the DBG_SELECT (line in the dbgMux), the DBG_BYTE_ENABLE (which bytes to output) and the DBG_SHIFT
# will be set
proc dbgTools::cfg_block {{block_name} {lineNumber} {byteEn} {shift} {regList}} {
    variable usedRegs
    variable onRegs
    if {![regexp STORM $block_name]} {
        # workaround for e1 debug bus bug.
        # if you turn off a line it is important to disable the byte enable before changing a line
        if {!$byteEn} {
            writeByName [set block_name]_REG_DBG_BYTE_ENABLE $byteEn
        }
        
        writeByName [set block_name]_REG_DBG_SELECT $lineNumber
        
        if {$byteEn} {
            writeByName [set block_name]_REG_DBG_BYTE_ENABLE $byteEn
        }
        
        writeByName [set block_name]_REG_DBG_SHIFT $shift
        
    } else {
        if {[llength $regList]==0} {
            puts "You are using a new version of the script with older csv files.\nYou have to update to
            the newest csv or your configuration will fail."
            return -1
        }
        # between the version 1.09 (CVS) and version 1.10 (CVS) the storms CSVs files have been joined to 1 file so
        # working with old csv files can cause the script to fail so a warning is outputted in that cases
    }
    
    for {set i 0} {$i<[llength $regList]} {incr i 2} {
        if {[lindex $regList $i]!=""} {
            # if the value to set start with 'o', then the registers will be set only upon turning on the debug block.
	    if {[string index [lindex $regList [expr {$i+1}]] 0]=="o"} {
		append onRegs "[lindex $regList $i] [string range [lindex $regList [expr {$i+1}]] 1 end] "
	    } else {
		writeByName [lindex $regList $i] [lindex $regList [expr {$i+1}]]
		if {![regexp (^0x|^\\d+$) [lindex $regList $i]]} {
		    set usedRegs([lindex $regList $i]) [reset_value -n [lindex $regList $i]]
		} else {
		    set usedRegs([lindex $regList $i]) 0
		}
	    }
        }
    }
}


# subroutine for configuring nig as the output of the debug block
# recive an ethernet header as string in hex format (6 Bytes (12 chars) Destination, 6 Bytes Source, 2 Bytes Type)
proc dbgTools::cfg_header {{header} {clean 0}} {

    writeByName DBG_REG_ETHERNET_HDR0 0x[string range $header 0 7]
    writeByName DBG_REG_ETHERNET_HDR1 0x[string range $header 8 15]
    writeByName DBG_REG_ETHERNET_HDR2 0x[string range $header 16 23]
    writeByName DBG_REG_ETHERNET_HDR3 0x[string range $header 24 27]
    
    # DEBUG_TARGET==1 represents network
    if {$clean} {
	writeByName DBG_REG_DEBUG_TARGET 0
    } else {
	writeByName DBG_REG_DEBUG_TARGET 1
    }
    
    # OUTPUT_ENABLE==4 represents network
    if {$clean} {
	writeByName DBG_REG_OUTPUT_ENABLE 0
    } else {
	writeByName DBG_REG_OUTPUT_ENABLE 4
    }
	    

}

# subroutine for configuring trigger for the debug block
# recieve Data line (70 bits in HEX)
# recieve Bit Enable (70 bits in HEX)
# recieve type of trigger (start | stop | end)
# USED FOR THE OLD TRIGGERING MACHINE
proc dbgTools::cfg_trigger {data bitMask trigger_type} {
    
    # setting the pattern
    set addr [get_start_offset [reg_offset -n DBG_REG_EXPECTED_PATTERN]]
    writeByName $addr 0x[string range $data 10 17]
    incr addr 4
    writeByName $addr 0x[string range $data 2 9]
    incr addr 4
    writeByName $addr 0x[string range $data 0 1]
    
    # setting the bit mask
    set addr [get_start_offset [reg_offset -n DBG_REG_EXPECTED_PATTERN_BIT_MASK]]
    writeByName $addr 0x[format "%.8x" [expr 0xFFFFFFFF-0x[string range $bitMask 10 17]]]
    incr addr 4
    writeByName $addr 0x[format "%.8x" [expr 0xFFFFFFFF-0x[string range $bitMask 2 9]]]
    incr addr 4
    writeByName $addr 0x[format "%.2x" [expr 0x3F-0x[string range $bitMask 0 1]]]
    
    switch -- [string tolower $trigger_type] {
        "end"  { writeByName DBG_REG_PATTERN_RECOGNITION_STORAGE_MODE 1}
        "filter" { writeByName DBG_REG_PATTERN_RECOGNITION_FILTER 0 }
        "start" {}
    }
    
    # This register needs to be set to 0 for the pattern recognition will be regarded
    writeByName DBG_REG_PATTERN_RECOGNITION_DISABLE 0x0

}


# subroutine for configuring the debug block's NEW trigger machine window 
# pre - recieve number of pre chunks to record before trigger event (-7 to 0) or ^ which represent record from start.
# post - recieve number of cycles to recorded after trigger event. $ represent forever.
proc dbgTools::cfg_ntrig {pre post} {
    
    set preMode 1
    # $pre == ^ => record from turning on the dbg block
    if {$pre=="^"} {
        set preMode 0
    } elseif {$pre==0} {
        # $pre==0 means record from trigger
        set preMode 2
    }
    if {$post=="$"} {
        set post -1
    }
    writeByName DBG_REG_RCRD_ON_WINDOW_PRE_TRGR_EVNT_MODE $preMode
    if {$pre<0} {
        writeByName DBG_REG_RCRD_ON_WINDOW_PRE_NUM_CHUNKS [expr {0-$pre}]
    }
    writeByName DBG_REG_RCRD_ON_WINDOW_POST_TRGR_EVNT_MODE [expr {$post==0}]
    writeByName DBG_REG_RCRD_ON_WINDOW_POST_NUM_CYCLES $post
    writeByName DBG_REG_TRIGGER_ENABLE 1
        
}

# subroutine for configuring semi filter machine for the debug block (E1.5)
# recieve stormLetter
# recieve type of trigger (pre | post | con)
# semiFil - source filtering: semiFil == 1 all dra in # semiFil == 2 fic # semiFil == 4 pb
# sfilType - pre post con
# cid - the cid to filter accordingly (@ - don't filter according to cid).
# eid - the eid to mask accordingly (none - don't filter according to eid)
# eidMask - the eid mask
proc dbgTools::cfg_sFilter {stormLetter semiFil sfilType cid eid eidMask} {
    variable stormOffset
    variable usedRegs
    array set types {pre 1 post 2 con 3 none 0}
    # represent the cid +eid register ([23:0] cid [31:24] eid]
    set sfilData 0
    
    # represent the filter config register [1:0] - filter mode (0 - none, 01 - prior to trigger, 10 - post trigger,
    # 11 - constant) [2] - use recorded cid [3] - compare cid [4] - compare eid [5] - dra src [6] - compare dra src
    set sfilCon 0
    if {$cid=="dbg"} {
	incr sfilCon [expr 1<<2]
	incr sfilCon [expr 1<<3]
    } elseif {$cid!="@" } {
	incr sfilCon [expr 1<<3]
	incr sfilData $cid
    }
    if {$eid!="none"} {
	incr sfilData [expr $eid<<24]
	incr sfilCon [expr 1<<4]
    }
    
    incr sfilCon [expr {$types($sfilType)}]
    
    incr sfilCon [expr {($semiFil/4)<<5}]
    incr sfilCon [expr {(1-$semiFil%2)<<6}]
    set base_addr [get_start_offset [reg_offset -n [set stormLetter]SEM_REG_FAST_MEMORY]]
    writeByName [expr {$base_addr+$stormOffset(filter_cidreg)}] $sfilData
    writeByName [expr {$base_addr+$stormOffset(filter_confreg)}] $sfilCon
    # the eidMask is a separated register
    writeByName [expr {$base_addr+$stormOffset(filter_eidmaskreg)}] $eidMask
    
    # Adding the semi filter to usedRegs so it would be cleaned before next use.
    set usedRegs([expr {$base_addr+$stormOffset(filter_cidreg)}]) 0
    set usedRegs([expr {$base_addr+$stormOffset(filter_confreg)}]) 0
    set usedRegs([expr {$base_addr+$stormOffset(filter_eidmaskreg)}]) 0
}

# subroutine for configuring NEW filter mode for the debug block (E1.5)
# recieve type of trigger (pre | post | con)
proc dbgTools::cfg_filter {type} {
    
    array set types {pre 1 post 2 con 3 none 0}
    writeByName DBG_REG_FILTER_ENABLE $types($type)
}



# subroutine for configuring pci output for the debug data
# recieved the size of memory requested in KB
proc dbgTools::cfg_pci_dos {size} {
    
    variable mem_addr
	variable physical_mem_addr
	variable DUMP
	variable chip_ver
	variable ::current
	writeByName DBG_REG_PCI_LOGIC_ADDR 0
	if {$DUMP == 0} {
		# allocating memory with the ediag and saving the address for freeing the memory upon finishing
		set mem_addr [hmem palloc [expr {1024*$size}] "Debug Bus"]
		set physical_mem_addr [hmem paddr $mem_addr]
		set low_addr [lindex $physical_mem_addr 0]
		if {[llength $physical_mem_addr] > 1} {
			set high_addr [lindex $physical_mem_addr 1]
			} else {
			set high_addr 0
		}
		
		writeByName DBG_REG_PCI_EXT_BUFFER_STRT_ADDR_LSB $low_addr
		writeByName DBG_REG_PCI_EXT_BUFFER_STRT_ADDR_MSB $high_addr
		
		# Check if we need to update the pci func
		if {$chip_ver >= 2} {
		    set pci_func 0
		    if {[info exists ::current(PCI_FUNC)]} {
			set pci_func [expr $::current(PCI_FUNC)]
		    }
		    
		    writeByName DBG_REG_PCI_FUNC_NUM $pci_func
		}

		# size in the debug bus is saved in 256B packets 
		set size [expr $size*4]
		
		writeByName DBG_REG_PCI_EXT_BUFFER_SIZE $size
	}
    
    # DEBUG_TARGET==2 represents pci
    writeByName DBG_REG_DEBUG_TARGET 0x2
    # due to a bug in the chip this register must be set to 1
    writeByName DBG_REG_PCI_REQ_CREDIT 0x1
    
    # OUTPUT_ENABLE==3 represents pci
    writeByName DBG_REG_OUTPUT_ENABLE 3

}

# subroutine for configuring the debug block modes and clients
# client_enable is 9 bits register which each bit is used to allow a certain PD block to output data
# framing mode is the relavant mode of working (can be 0,1,2 - TDM32, 64BIT, 24BIT)
# when the framing mode is set to TDM32 you can change it to TDM64
# full mode - in internal buffer or pci determine whetever to work in wrap or in one-shot
#             in network determine whetever to limit the amount of data sent (not used - always working without a limit)
proc dbgTools::cfg_dbg {{client_enable} {framing_mode} {tdm64 0} {full_mode 0}} {
    
    writeByName DBG_REG_CLIENT_ENABLE $client_enable
    
    writeByName DBG_REG_FRAMING_MODE $framing_mode
    
    writeByName DBG_REG_TDM64_ENABLE $tdm64
    
    writeByName DBG_REG_FULL_MODE $full_mode
    
}

# subroutine for configurating the port when outputting to the network
proc dbgTools::cfg_port {port} {
    writeByName NIG_REG_EGRESS_DEBUG_PORT $port
}
# subroutine for configurating the path when outputting to the network
proc dbgTools::cfg_path {{chosen_path -1}} {
	variable other_path
	if {$chosen_path == -1} {
		set chosen_path $other_path
	}
	if {[dbgE2] || [dbgE3]} {
		writeByName NIG_REG_EGRESS_DEBUG_PATH $chosen_path
	}
}



# subroutine for configurating the pace (the gap in cycles between sampling)
proc dbgTools::cfg_pace {pace} {
    writeByName DBG_REG_CALENDAR_PACE $pace
}

# subrouting for adjusting the storm for the E2 Bug Fix
# The default is 8 but should be 16, and if working in TDM32 and all client's are storms
# then it should be 16.
proc dbgTools::cfg_full_buffer_threshold {is_only_storms_tdm32} {
    if {$is_only_storms_tdm32} {
        writeByName DBG_REG_FULL_BUFFER_THR 24
    } else {
        writeByName DBG_REG_FULL_BUFFER_THR 16
    }
}
# subroutine for configuring the storms' outputs to the debug block
# used only for cleaning old configuration upon loading the script
proc dbgTools::clean_storms {{storms}} {
 
   variable stormOffset
   foreach letter $storms {
       set addr [get_start_offset [reg_offset -n [set letter]SEM_REG_FAST_MEMORY]] 
       foreach field [array names stormOffset] {
              reg write [expr {$addr+$stormOffset($field)}] 0
       }
   }
}

proc dbgTools::dumpIntrBuffer {args} {
    puts "dumpIntrBuffer is now count to be deprecated, You can use dbgEnd <filename> instead."
    return
}

proc dbgTools::dumpRam {args} {
    puts "dumpRam is now count to be deprecated, You can use dbgEnd <filename> instead."
    return
}

proc dbgTools::dumpModeUsefulFunctions {} {
	variable DUMP
    variable dumpFile
	if {$DUMP == 4} {
		return
	}
	puts $dumpFile "proc reg_show \{\{addr\} \{count 4\}\} \{"
    puts $dumpFile [info body dbgTools::reg_show]
	puts $dumpFile "\}"
	
	puts $dumpFile "\n"
	
	puts $dumpFile "proc format_mem \{ data \} \{"
	puts $dumpFile "regsub -all \\\[\\\\da-fA-F\\\]+: \$data \{\} data"
	puts $dumpFile "regsub -all \\\\.+\\\\s \$data \{\} data"
	
	puts $dumpFile "set data \[hex2bin \$data\]"
	puts $dumpFile "set regList \[split \$data \{ \}\]"
	puts $dumpFile "set resList \"\""
	puts $dumpFile "set index 0"
	puts $dumpFile "set listSize \[llength \$regList\]"
	puts $dumpFile "for \{set i 0\} \{\$i<\$listSize\} \{incr i 4\} \{"
	puts $dumpFile "\tset tempStr \"\""
	puts $dumpFile "\tfor \{set j 3\} \{\$j>=0\} \{incr j -1\} \{"
	puts $dumpFile "\t\tappend tempStr \[lindex \$regList \[expr \{\$i+\$j\}\]\]"
	puts $dumpFile "\t\}"
	puts $dumpFile "\tlappend resList \$tempStr"
	puts $dumpFile "\}"
	puts $dumpFile "return \$resList"
	puts $dumpFile "\}"
	
	puts $dumpFile "\n"
	
	puts $dumpFile "proc hex2bin hex \{"
    puts $dumpFile [info body dbgTools::hex2bin]
	puts $dumpFile "\}"
	
	puts $dumpFile "\n"
}


proc dbgTools::dumpModeIntrBuf {} {
    variable DUMP
    variable dumpFile
    variable full_mode
    if {$DUMP} {
        puts $dumpFile "proc dumpBuffer {output_file} \{"
            flushNet
            
            puts $dumpFile "variable full_mode"
            puts $dumpFile "set full_mode $full_mode"
            puts $dumpFile "set addr [getOffsetByName DBG_REG_DBG_BLOCK_ON]"
            puts $dumpFile "if \{!\[reg read \$addr\]\} \{"
            puts $dumpFile "\tputs \"You can't read the interanl buffer, while the debug block is off. please turn it on first. use: \\\"dbgOn\\\"\""
            puts $dumpFile "\treturn -1"
            puts $dumpFile "\}"
            
            flushNet
            
            puts $dumpFile "set addr [getOffsetByName DBG_REG_INTR_BUFFER_WR_PTR]"
            puts $dumpFile "set wr_pointer \[reg read \$addr\]"
            
            puts $dumpFile "set wrapped \[reg read [getOffsetByName DBG_REG_WRAP_ON_INT_BUFFER]\]"
            
            puts $dumpFile "if \{(\$wr_pointer ==0) && (\$wrapped!=1)\} \{"
            puts $dumpFile "\tputs \"There is no data in the internal buffer to dump.\""
            puts $dumpFile "\treturn"
            puts $dumpFile "\}"
     
    
            puts $dumpFile "set size \[expr \{(\$wr_pointer+1)*16\}\]"
            
    
            puts $dumpFile "set outFile \[open \"\[set output_file\].buf\" w\]"
            puts $dumpFile "set addr [getOffsetByName DBG_REG_INTR_BUFFER]"
            puts $dumpFile "set data \[reg_show \$addr \$size\]"
    
            puts $dumpFile "if \{\$full_mode&&\$wrapped&&\$size!=2048\} \{"
            puts $dumpFile "\tset ans \[reg_show \[expr \{\$addr+\$size\}\] \[expr \{128*16-\$size\}\]\]"
            puts $dumpFile "\tappend ans \" \$data\""
            puts $dumpFile "\tset data \$ans"
            puts $dumpFile "\}"
    
    
            puts $dumpFile "set memList \[format_mem \$data\]"
            
            puts $dumpFile "for \{set i 0\} \{\$i<\[llength \$memList\]\} \{incr i\} \{"
            puts $dumpFile "puts \$outFile \[format \"%.8x\\t\[lindex \$memList \$i\]\" \$i\]"
            puts $dumpFile "\}"
            puts $dumpFile "close \$outFile"
            puts $dumpFile "puts \"Internal buffer has been dumped into \[set output_file\].buf\""
            
            puts $dumpFile "\}"
    } else {
        #nothing to do if not in dump mode.
        return
    }
    
}


# subroutine for dumping the debug block internal buffer to a file
# recieve only a name of a file
proc dbgTools::dumpIntrBuf {{outputFile}} {
    
    variable full_mode

    # Accessing the debug block's buffer when the block isn't turned on isn't possible
    set addr [get_start_offset [reg_offset -n DBG_REG_DBG_BLOCK_ON]]
    if {![reg read $addr]} {
        puts "You can't read the interanl buffer, while the debug block is off. please turn it on first.\
        use: \"dbgOn\""
	return -1
    }
    # Flushing the last data
    flushNet
        
    # reading the position of the write pointer
    set addr [get_start_offset [reg_offset -n DBG_REG_INTR_BUFFER_WR_PTR]]
    set wr_pointer [reg read $addr]
    # if the wr_pointer equals zero and the buffer haven't been wrapped there is not data on the buffer
    
    if {($wr_pointer ==0) && ([reg_read -n DBG_REG_WRAP_ON_INT_BUFFER]!=1)} {
        puts "There is no data in the internal buffer to dump."
        return
    }
    
    set size [expr {($wr_pointer+1)*16}]
    # opening the output file
    set outFile [open "[set outputFile].buf" w]
    #reading the data from the beginning of the buffer to the position of the write pointer
    set addr [get_start_offset [reg_offset -n DBG_REG_INTR_BUFFER]]
    set data [reg_show $addr $size]
    
    # if working in wrap mode and the buffer has been wrapped, then the data between the write pointer and the end of the buffer
    # should also be read and attached before the data read from the beginning of the buffer 
    if {$full_mode&&[reg_read -n DBG_REG_WRAP_ON_INT_BUFFER]&&$size!=2048} {
	set ans [reg_show [expr {$addr+$size}] [expr {128*16-$size}]]
	append ans " $data"
	set data $ans
    }
    # formating the memory
    set memList [format_mem $data 128]
    
    # outputting to the file
    for {set i 0} {$i<[llength $memList]} {incr i} {
    	puts $outFile [format "%.8x\t[lindex $memList $i]" $i]
    }
    
    puts -nonewline [close $outFile]
    puts "Internal buffer has been dumped into [set outputFile].buf"
}
# perform num_a - num_b (each is a list where num[0] is the lsb and num[1] is the msb
# if the result is only 32 bit, then a regular number will be returned
proc dbgTools::Sub64Bits {num_a num_b} {
	set lsb_a [lindex $num_a 0]
	set lsb_b [lindex $num_b 0]
	if {[llength $num_a] > 1} {
		set msb_a [lindex $num_a 1]
	} else {
		set msb_a 0
	}
	
	if {[llength $num_b] > 1} {
		set msb_b [lindex $num_b 1]
	} else {
		set msb_b 0
	}
	
	set lsb_diff [expr {$lsb_a - $lsb_b}]
	if {$lsb_b > $lsb_a} {
		set additional_msb_diff 1
	} else {
		set additional_msb_diff 0
	}
	set original_msb_diff [expr {$msb_a - $msb_b}]
	set msb_diff [expr {$msb_a - $msb_b - $additional_msb_diff}]
	if {$original_msb_diff == 0} {
		return $lsb_diff
		} else {
		return [list $lsb_diff $msb_diff]
		}
}
#perform num_a + num_b (each is a list where num[0] is the lsb and num[1] is the msb
# if the result is only 32 bit, then a regular number will be returned
proc dbgTools::Add64Bits {num_a num_b {formatted false}} {
	set lsb_a [lindex $num_a 0]
	set lsb_b [lindex $num_b 0]
	if {[llength $num_a] > 1} {
		set msb_a [lindex $num_a 1]
	} else {
		set msb_a 0
	}
	
	if {[llength $num_b] > 1} {
		set msb_b [lindex $num_b 1]
	} else {
		set msb_b 0
	}
	
	set res_lsb [expr [format "0x%x" [expr {$lsb_a + $lsb_b}]]]
	if {$res_lsb < $lsb_a} {
		set additional_msb 1
	} else {
		set additional_msb 0
	}
	
	set res_msb [expr {$msb_a + $msb_b + $additional_msb}]
	if {$formatted} {
		set res_msb [format "0x%x" $res_msb]
		set res_lsb [format "0x%x" $res_lsb]
		} 
	if {[expr $res_msb] == 0} {
		return $res_lsb
	} else {
		return [list $res_lsb $res_msb]
	}
}
#perform a comparison between num_a and num_b (each is a list where num[0] is the lsb and num[1] is the msb
# Result: 0 = equal, 1 - num_a is greater, -1 num_b is greater
proc dbgTools::Compare64Bits {num_a num_b} {
	set lsb_a [lindex $num_a 0]
	set lsb_b [lindex $num_b 0]
	if {[llength $num_a] > 1} {
		set msb_a [lindex $num_a 1]
	} else {
		set msb_a 0
	}
	
	if {[llength $num_b] > 1} {
		set msb_b [lindex $num_b 1]
	} else {
		set msb_b 0
	}
	
	if {($msb_a == $msb_b) && ($lsb_a == $lsb_b)} {
		return 0
	}
	if {($msb_a > $msb_b) || (($msb_a == $msb_b) && ($lsb_a > $lsb_b))} {
		return 1
	} else {
		return -1
	}
}

# subroutine for dumping the debug data from the host's ram to a file
# recieve only a name of a file
proc dbgTools::dumpExtBuf {{outputFile}} {
    
    variable full_mode
    variable pci_size
    variable mem_addr
    variable OUTMODE
	variable physical_mem_addr
    
    # insuring you are working in PCI mode
    if {$OUTMODE!="PCI"} {
        puts "You are not working in PCI mode or dumpExtBuf has been used already.\n dumpExtBuf is only for one use in pci mode."
        return
    }

    # Accessing the debug block's buffer when the block isn't turned on isn't possible
    if {![reg_read -n DBG_REG_DBG_BLOCK_ON]} {
        puts "You can't read the ram, while the debug block is off. please turn it on first:
        using \"dbgOn\""
	return -1
    }
    # upon Flush the debug bus will added the write pointer 256 although it might write only 128 B
    # (when working with TDM32 flush add only 8 lines) so diff is used not to read the lines which haven't been written
    # this is done by checking how much the read pointer has progress by the flush
    #set diff [reg_read -n DBG_REG_INTR_BUFFER_RD_PTR]

    # flushing the last data
    flushNet
    after 3000
    set rd_ptr [reg_read -n DBG_REG_INTR_BUFFER_RD_PTR]
    set wr_ptr [reg_read -n DBG_REG_INTR_BUFFER_WR_PTR]
    #if {(!$rd_ptr)&&($diff)} {set rd_ptr 0x7f}
	#after flush, we want to know how many lines were read , after the last output to pci (resolutio of 16 lines)
	# multiplying by 16 because 16 is the size of a line
    set diff [expr {(($rd_ptr)%16)*16}] 
    set addr [get_start_offset [reg_offset -n DBG_REG_EXT_BUFFER_WR_PTR]]
    set wr_pointer [list [reg read $addr] [reg read [expr {$addr+4}]]]
	
    
    # if the wr_pointer equals to the beginning of the memory allocated and the buffer haven't been wrapped
    # then there is not data on the buffer
    # upon reaching the end of the buffer (when working in one shot) the write pointer is set back to the beginning of memory allocated
    # So the script also checks the buffer haven't been overflowed
    if {([Compare64Bits $wr_pointer $physical_mem_addr] == 0) && ([reg_read -n DBG_REG_OVL_ON_EXT_BUFFER]!=1) && \
    ([reg_read -n DBG_REG_WRAP_ON_EXT_BUFFER]!=1)} {
        puts "There is no data in the ram to dump."
	hmem free $mem_addr
	set mem_addr 0
	set pci_size 0
        return
    }
    # set the needed data size to be read while considering diff
    set size [Sub64Bits [Sub64Bits $wr_pointer $physical_mem_addr] $diff]
    if {$size==0} {set size [expr {$pci_size*1024}]}
    # opening the output file
    set outFile [open "[set outputFile].pci" w]
    # reading the data from the beginning of the memory allocated to the position of the write pointer 
    set wrap_size 0
    set wrap_addr 0
    if {$full_mode && [reg_read -n DBG_REG_WRAP_ON_EXT_BUFFER] && ([Compare64Bits $physical_mem_addr $wr_pointer]==0)} {
        set wrap_size [Sub64Bits [expr {$pci_size*1024+$diff}] [Sub64Bits $wr_pointer $physical_mem_addr]]
	set wrap_addr [Sub64Bits $wr_pointer $diff]
    }
    # if working in wrap mode and the memory allocated has been wrapped, then the data between the write pointer
    # and the end of the memory allocated should also be read and attached before the data read from the beginning of the memory allocated 
    while {$size!=0 || $wrap_size!=0} {
	if {$wrap_size==0} {
	    set wrap_size $size
	    set size 0
	    set wrap_addr $mem_addr
	}
	if {[Compare64Bits $wrap_size 102400] == 1} {
	    set temp_size 102400
	} else {
	    set temp_size $wrap_size
	}
	set wrap_size [Sub64Bits $wrap_size $temp_size]
	set data [hmem_show $wrap_addr $temp_size]
	set wrap_addr [Add64Bits $wrap_addr $temp_size]
	# formatting the data
	set memList [format_mem $data 64]
    
	# outputting to the file
	for {set i 0} {$i<[llength $memList]} {incr i 2} {
		puts $outFile [format "%.8x\t[lindex $memList $i][lindex $memList [expr $i+1]]" [expr $i/2]]
	}
	}
	
    puts -nonewline [close $outFile]
    # releasing the memory
    hmem free $mem_addr
    set mem_addr 0
    set pci_size 0
    puts "Buffer on the ram has been dumped into [set outputFile].pci"
    
}

# subroutine for resetting the debug block
proc dbgTools::dbgReset {} {
    variable DBG_STATUS
    variable mem_addr
    variable pci_size
    variable DUMP
    variable dumpFile
    
    if {$DUMP} {
		if {$DUMP < 4} {
			puts $dumpFile "# resetting the debug block"
		}
    }
    # if memory was allocated in the last run, the script will release it
    if {([llength $mem_addr] > 1) || ($mem_addr)} {
        hmem free $mem_addr
        set mem_addr 0
    }
    set DBG_STATUS NONE
    
    # resetting the block (bit 10 in this register is responsive for resetting the debug block)
    set addr [get_start_offset [reg_offset -n MISC_REG_RESET_REG_2]]
	# do not reset the debug block for kernel debugging
	if {$DUMP < 4} { 
		writeByName MISC_REG_RESET_REG_2 [expr [reg read $addr] & 0xfffffbff]
	}
    if {$DUMP} {
		if {$DUMP < 4} {
			puts $dumpFile "# Finished resetting the debug block\n"
		}
    }
}

# subroutine for turning the debug block on
proc dbgTools::dbgOn {} {
    variable DBG_STATUS
    variable onRegs
    variable usedRegs
    variable DUMP
    
    # if the debug block has been flush it should be resetted before start working again
    if {!$DUMP && [string equal $DBG_STATUS FLUSHED]} {
	puts "After flushing you have to reset the debug block.\nPlease re-set the profile using setProfile."
        return -1
    } elseif {!$DUMP && [string equal $DBG_STATUS RUNNING]} {
        puts "You already turn the debug block on using dbgOn."
        return -1
    } elseif {!$DUMP && ![string equal $DBG_STATUS CONFIGURED]} {
	puts "You must set a profile before turning on the debug block."
        return -1
    } else {
	writeByName DBG_REG_DBG_BLOCK_ON 1
        set DBG_STATUS RUNNING
	foreach {reg val} $onRegs {
	    writeByName $reg $val
	}
    }
}


# subroutine for turning the debug block off 
proc dbgTools::dbgOff {} {
    variable onRegs
    variable DBG_STATUS
    variable DUMP
    if {!$DUMP && ![string equal $DBG_STATUS RUNNING]} {
	puts "The only usage of dbgOff is when the debug block is activate, press turn it on using dbgOn."
    } else {
        set DBG_STATUS CONFIGURED
	foreach {reg val} $onRegs {
	    if {![regexp (^0x|^\\d+$) $reg]} {
               set val [reset_value -n $reg]
            } else {
                set val 0
            }
            writeByName $reg $val
	}
	writeByName DBG_REG_DBG_BLOCK_ON 0
    }

}

# this function cleans the Semi debug fifo after use
# The fifo is been filled only when debug_active is 1,
# but even when debug_active is off, the semi empty the fifo when receiving grants from the Debug Block.
# The ensure enough grants will be recieved by the storms the debug block is set for a phony profile of all storms in wrap mode for 500 milis.
proc dbgTools::emptyFifo {} {
    
    variable blocksPD
    variable onRegs
    if {[llength $onRegs]==0} {
	return
    }
    dbgReset
    foreach {reg val} $onRegs {
	writeByName $reg 0
    }    
    cfg_cal {5 6 7 8 5 6 7 8}
    writeByName DBG_REG_CLIENT_ENABLE 0x1f0
    writeByName DBG_REG_FULL_MODE 1
    writeByName DBG_REG_DBG_BLOCK_ON 1
    after 500
    dbgReset
    
}

# subroutine for turning the debug block off, flushing last data
# and outputing the data into a file if working in PCI or BUFFER mode
proc dbgTools::dbgEnd {{outFile -1}} {
    variable OUTMODE
    variable DBG_STATUS
    variable trigger_used
    variable dbg_no_pci
        
    if {![string equal $DBG_STATUS RUNNING] && !($dbg_no_pci && [string equal $DBG_STATUS DUMP_NO_PCI])} {
    	puts "You can use dbgEnd only after you turn the block on using dbgOn."
        return -1
    }
    if {[string equal $OUTMODE NETWORK]} {
        flushNet
    } elseif {$outFile==-1} {
        puts "You must give a file name to dump the data into"
        return
    } elseif {[string equal $OUTMODE PCI]} {
        dumpExtBuf $outFile
    } elseif {[string equal $OUTMODE BUFFER]} {
        dumpIntrBuf $outFile
    }    
    # outputing the indirect state of working with the trigger
    if {$trigger_used} {
        puts "indirect0 was: [reg_read -n DBG_REG_TRIGGER_INDIRECT0_RECORDED_DATA]."
        puts "indirect1 was: [reg_read -n DBG_REG_TRIGGER_INDIRECT1_RECORDED_DATA]."
    }
        #DEBUG
        #set qwords [reg_read -n DBG_REG_NUM_OF_QWORDS_SENT0]
        #puts "num of qwords: $qwords, num of packets: [expr {$qwords/130}]"
        #
        #puts "wr_ptr: [reg_read -n DBG_REG_INTR_BUFFER_WR_PTR]."
        #puts "rd_ptr: [reg_read -n DBG_REG_INTR_BUFFER_RD_PTR]."
        #puts "buffer_full: [reg_read -n DBG_REG_FULL_ON_INT_BUFFER]."
        ###
        
    writeByName DBG_REG_DBG_BLOCK_ON 0
    # emptying the fifos.
    emptyFifo
    
    cleanCfg
}

# subroutine which get a block name and a line name in his debug mux and checks according in the block's file
# what is the line_number. return errors if file wasn't found or if line wasn't found, otherwise return a list which
# the first element is the line number, the second is the line_type according to the LINETYPE array and the rest of the list
# are pairs of registers name or address with the desire value
# the csv file is starting with group definition part.
# afterward comes the lines definition part.
# and afterward comes the special lines (sp_lines) part.
# lines are build from groups and there type is based on which groups they contain
# Special lines are used when their is additional configuration needed to be done
# They have the name of the line they refer to and also pairs of registers names and desire value
proc dbgTools::getLineOfBlock {{block} {lineName}} {
    
    variable errors
    variable LINETYPE
    variable csv_folder
	
	#Linux filename fix - change file to lowercase unless file contains STORM or SEM
	if {!([regexp STORM|SEM $block])} {
		set block [string tolower $block] }
    
    set letter ""
    set fileName [format {%s/%s.csv} $csv_folder $block]
    # the storms and the sems have only 1 file for all 4
    if {[regexp STORM|SEM $block]} {
        set letter [string index $block 0]
        set fileName [format {%s/%s.csv} $csv_folder [string range $block 1 end]]
    }
    
    
    
    array set groups {}
    array set lines {}
    
    # ensuring the file exists
    if {![file exist $fileName]} {
	puts "Couldn't find the file $fileName.\n Make sure the file exists."
        # between the version 1.09 (CVS) and version 1.10 (CVS) the storms and the sems CSVs files have been joined to 1 file so
        # working with old csv files can cause the script to fail so a warning is outputted in that cases
        if {$letter!=""} {
            puts "You are probably using old csv files with a new version of the script.\nPlease update the csv folder."
        }
	return $errors(EOF)
    }
    
    #opening the file
    set blkFile [open $fileName r]
    
    set line [gets $blkFile]
    # searching for the beginning of the lines definition part of the file
    # and saving the groups' name and size - will be use for determining the type of the line
    while {! [regexp "^Line Name" $line]} {
        set line [gets $blkFile]
        if {[regexp ",,#(.+),(\\d+),(\\d+)(,.*)?" $line groupName groupName end start]} {
	    set groups($groupName) [expr {$end-$start+1}]
	}  
    }
   
    set lineCount 0
    set regList ""

    # looking for the requested line
    # if reaching end of lines then the line will be searched in the special lines
    while {![regexp ^$lineName, $line] &&![regexp "#End_Of_Lines" $line]} {
        set line [gets $blkFile]
        incr lineCount
	regexp {^([^,]+),} $line name name
        set linesNum($name) $lineCount
        set lines($name) $line
	
    }
    # if the line hasn't been found search for it in the sp_lines
    if {![info exists lines($lineName)]} {
        # reaching the beginning of the sp_lines
        while {![eof $blkFile]&&![regexp Sp_Lines $line]} {
            set line [gets $blkFile]
        }
        
        if {[eof $blkFile]} {
	    close $blkFile
	    return $errors(line_not_found)
        }
        
        while {![regexp #End_Of_Sp_Lines $line]&&![regexp ^$lineName $line]&&![eof $blkFile]} {
                set line [gets $blkFile]
        }   
        
        if {![regexp ^$lineName $line]} {
	    close $blkFile
            return $errors(line_not_found)
        }
        
        set parms [split $line ,]
        set lineName [lindex $parms 1]

        set regList [lrange $parms 2 end]
    }
    set line $lines($lineName) 
    
    set ans $linesNum($lineName)
    set lineData [split $line ,]
    set line_type 0
    
    # determine the line type
    if {[lindex $lineData 1]!=64} {
        incr line_type 2
        for {set j 2} {$j<5} {incr j} {
            if {[lindex $lineData $j]!="none"} {
                if {$groups([lindex $lineData $j])==16} {incr line_type [expr {4+(4-$j)}]}
                if {$groups([lindex $lineData $j])==8} {incr line_type 1}
                if {$groups([lindex $lineData $j])==24} {incr line_type 7}  
	    }
        }
        
	
        switch -- $line_type {
            {3} {append ans " " $LINETYPE(24BIT,8BIT)}
            {4} {append ans " " $LINETYPE(24BIT,8BIT8BIT)}
            {5} {append ans " " $LINETYPE(24BIT,THREE8BIT)}
            {6} {append ans " " $LINETYPE(24BIT,16BIT)}
            {7} {append ans " " $LINETYPE(24BIT,8BIT16BIT)}
            {8} {append ans " " $LINETYPE(24BIT,16BIT8BIT)}
            {9} {append ans " " $LINETYPE(24BIT,24BIT)}
        }
    } else {
        for {set j 2} {$j<5} {incr j} {
            if {[lindex $lineData $j]!="none"} {
                if {$groups([lindex $lineData $j])==32} {incr line_type}
            }
        }
	
        switch -- $line_type {
            {0} {append ans " " $LINETYPE(64BIT,64GROUP)}
            {1} {append ans " " $LINETYPE(64BIT,32GROUP)}
            {2} {append ans " " $LINETYPE(64BIT,TWO32GROUP)}
        }
    }
    # in the storm and semis file the $ character is placeholder for the apporirate letter (e.g. $SEM should change into
    # XSEM if the request block was XSEM) that way the script can use only 1 file for all 4 blocks
    # since some of the storms registers aren't mapped in the everest rdf file, the addresses are saved in the end of the
    # CSV file. if there are registers in the regList, the script checks to see if the addresses are in the CSV file
    # or can be found in the everest RDF
    
    if {$regList!=""} {
        array set address {}
        while {![eof $blkFile]&&![regexp "Addresses" $line]} {
            set line [gets $blkFile]
        }
        while {![eof $blkFile]&&![regexp "#End_Of_Addresses" $line]} {
            set line [gets $blkFile]
            regexp ^(\[^,\]+),(\[^,\]+) $line reg reg num
            set address($reg) $num
        }
        
        for {set i 0} {$i<[llength $regList]} {incr i 2} {
            regsub \\$ [lindex $regList $i] $letter tmp
            set regList [lreplace $regList $i $i $tmp]
            if {[info exists address([lindex $regList $i])]} {
                set regList [lreplace $regList $i $i $address([lindex $regList $i])]
            }
            
        }
        
        append ans " " $regList
    }
    # closing the file
    close $blkFile
    return $ans
}

proc dbgTools::getDebugLinesofBlock {block block_lines} {
    upvar 1 $block_lines all_lines
    variable errors
    variable LINETYPE
    variable csv_folder
    variable line
    
    array set all_lines {}
	
	#Linux filename fix - change file to lowercase unless file contains STORM or SEM
	if {!([regexp STORM|SEM $block])} {
		set block [string tolower $block]
                }
    
    set letter ""
    set fileName [format {%s/%s.csv} $csv_folder $block]
    # the storms and the sems have only 1 file for all 4
    if {[regexp STORM|SEM $block]} {
        set letter [string index $block 0]
        set fileName [format {%s/%s.csv} $csv_folder [string range $block 1 end]]
    }
    
    # ensuring the file exists
    if {![file exist $fileName]} {
	puts "Couldn't find the file $fileName.\n Make sure the file exists."
        # between the version 1.09 (CVS) and version 1.10 (CVS) the storms and the sems CSVs files have been joined to 1 file so
        # working with old csv files can cause the script to fail so a warning is outputted in that cases
        if {$letter!=""} {
            puts "You are probably using old csv files with a new version of the script.\nPlease update the csv folder."
        }
	return $errors(EOF)
    }
    
    #opening the file
    set blkFile [open $fileName r]
    
    set line [gets $blkFile]
    # searching for the beginning of the lines definition part of the file
    # and saving the groups' name and size - will be use for determining the type of the line
    
    while {! [regexp "^Line Name" $line]} {
        set line [gets $blkFile]
	}  
    
   set lineCount 0
   
   set all_lines(0,name)	signature
   set all_lines(list)	[list signature]
    while {![regexp "#End_Of_Lines" $line]} {
        set line [gets $blkFile]
        incr lineCount
	regexp {^([^,]+),} $line name name
        set all_lines([set lineCount])  $line
        set all_lines([set lineCount],name) $name
		if {![regexp "#End_Of_Lines" $line]} {
			lappend all_lines(list)	$name
		}
    }
    set all_lines(count)  [set lineCount]
}

# subroutine which gets the profiles file and a profile name and some arguments.
# the possible arguments are: 
# -network h<num> - configurate the debug block to output data to the network and put apporiate header on the packets (-net is a synonym)
# -port0 - use port 0 when outputing data through the network (default is 1)
# -pci <size in KB> - configurate the debug block to output data to the host's memory through the pci, the size will be
#                     the upper limit of memory to be used in KB
# -wrap - will use the buffer (internal or ram) in cyclic way
# -grant - will lower backpressure on the storms
# -t start|end|filter <trigger_name> - set the trigger mechinsm of the debug block
# -dumpn? <filename> - will write the registers' configuration needed for setting the profile into
# -dumph index- will write the registers' configuration needed for setting the profile into the .h file. Configuration index is also given
# <filename> instead of configuring the chip (if written with n the dump will be by name).
# The next arguments are relevant only in E1H:
# -nt <window> <trigger name> - setting the new trigger machine with that trigger and in that window.
#   the <window> should hold pre value (-7 to 0 or ^) and post value (0- 2^31-1 or $) separated by '-'. e.g. -4-$.
#   ^ - represent record from dbgOn other values represent number of chunks to save prior to trigger event.
#   $ - represent forever other values represent number of cycles to record after trigger event.
# -f <pre|post|con> <filter name> - setting the new filter machine.
# -sf[stormLetter] - turn on the semiFilter in the relevant storm. if -cid[tucx] wasn't given the semi filter mode will be post
# and the cid will be taken from the debug block.
# -cid[stormLetter] <pre|post|con> <cid | @> - setting the semi filter mode and the cid to filter according to. @ - don't filter accroding to cid.
# -eid[stormLetter] <eid> <eidMask> - setting the semi filter to filter accroding to event id and mask.
# the subroutine get all relevant data about the requested profile from the profiles file into an array.
# it then recognizes the working mode and transfer the array to the appropriate function (setProfileTDM or
# setProfile64BIT).
# -eidonly[stormLetter] <eid> <eidMask> : equal to -sf[stormLetter] -cid[stormLetter] con @ -eid[stormLetter] <eid> <eidMask>
# if the profile setting had succeed it will configurate the network/pci configuration is needed.
# The sciprt assumes headers are in the beginning of the file, followed by triggers and ending with profiles.
proc dbgTools::setProfile {{input_file} {profile_name} {args}} {
    
    global tcl_platform
    variable pci_size
    variable OUTMODE
    variable dbg_no_pci
    variable dumpFile
    variable DUMP
    variable NTpre
    variable onRegs
    variable DBG_STATUS
    variable chip_ver
    variable full_mode
    variable FORMATS
    variable CYCLES_PER_MODE
    variable trigger_used
	variable pci_mode
	variable other_path
	
	set other_path 0
    
    if {$dumpFile!="" && $dumpFile!="closed"} {
        close $dumpFile
    }
    
    set DUMP 0
    set OUTMODE -1
    set tdm64 0
    set line_number 0 
    set mode -1
    set triggerName -1
    set ntrigName -1
    set headerNum -1
    set filterName -1
    # the variable represnt the filter indirects use (129 is none since the constraints is only for the first 128 cycles).
    set filterInd "129 129"
    # ethernet header
    set header ""
    # network port to output to
    set port 1
	if {[dbgE2] || [dbgE3]} {
		#in E2/E3, by default, when outputing data to network, it will be to port 0
		set port 0
	}
    # grant the storms even when the buffer is full
    set grant 0
    # memory size in the host's ram
    set pci_size 0
    # The pre mode the new trigger works in
    set NTpre none
    # Status of the semi filter of all 4 storms1
    array set semiFilters {TsemiFil 0 UsemiFil 0 CsemiFil 0 XsemiFil 0}
    set trigger_used 0
    #set semiFil 0
    
    # data and mask represent the trigger details - each supposed to be 70 bit in hexa format after configuration
    # -1 is reset value
    set data -1
    set mask -1 
    
	set pci_mode 0
	
    # working mode one shot (0) or wrap (1)
    set full_mode 0
    ####################
    # checking arguments
    ####################
    variable dump_counter_info
	variable dumph_current_config_number
	
	
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -regexp -- [string tolower [lindex $args $i]] {
            {^-net$}          -
            {^-network$}      {   if {$full_mode} {
                                    puts "Wrap isn't relevant for network, only for internal buffer"
                                    return
                                }
                                if {$pci_size} {
                                    puts "You can't use the pci and the network in the same time."
                                    return
                                }
                                incr i
				if {![regexp ^h(\\d+)$ [lindex $args $i] headerNum headerNum]} {
				    puts "After -network you must write the requested header from the profiles file. e.g. -network h5."
                                    return
				}
                            }
            {^-dumpn?$}         { set DUMP [expr [string equal -nocase {-dumpn} [lindex $args $i]]+1]
                               incr i
                              if {[lindex $args $i]=="" || [string equal [string index [lindex $args $i] 0] "-"]} {
                                puts "You must supply a file name to dump the registers configuration after -dump"
                                set DUMP 0
                                return
                              }
                              
                              set dumpFile [open [lindex $args $i] w]
                              if {$DUMP==2} {
                                puts $dumpFile "set DUMP 0"
                                puts $dumpFile "proc writeByName {reg_name data} \{"
                                puts $dumpFile [info body writeByName]
                                puts $dumpFile "\}"
                              }
                            }
			{^-dumph$}         { 
			
								set DUMP 4
								incr i
								variable dumph_current_config_number
								set dumph_current_config_number [lindex $args $i]
								# save the profile name - it will be used for parsing later
								set dump_counter_info([set dumph_current_config_number],profile_name) $profile_name
                            }
            {^-t$}            { incr i
                              if { ![regexp -nocase (start|end|filter) [lindex $args $i] type type]} {
                                puts "After -t you must write \"start\", \"end\" or \"filter\". e.g. -t start FIC0_EVENT5."
                                return
                              }
                              incr i
                              set triggerName [lindex $args $i]
                            }
			{^-pcidumph$}     { 
								set pci_mode 1
								set pci_size 1024
							  }
            {^-pci$}          { 
			
			      if {$dbg_no_pci} {
				    puts "You can't output to PCI, while working in no_pci state (usually serial)."
				    return
			      }
                                if {$headerNum!=-1} {
                                    puts "You can't use the pci and the network in the same time."
                                    return
                                }
                              incr i
			        if {![regexp ^(\\d+)$ [lindex $args $i] pci_size pci_size]} {
				    puts "After -pci you must write the maximum size to use on the host. e.g. -pci 1000."
                                    return
				}
                                if {$pci_size>524288} {
                                    puts "The debug bus can work with up to 512MB."
                                    set pci_size 0
                                    return
                                }
                                set pci_mode 1
                            }
	    {^-wrap$}         { if {$headerNum!=-1} {
                                puts "Wrap isn't relevant for network, only for internal buffer"
                                return
                              }
                              set full_mode 1
                            }
            {^-port0$}        { set port 0}
			{^-port1$}        { set port 1}
			{^-otherpath$}	  { set other_path 1 }
            {^-grant$}        { set grant 1}
            default         { if {([dbgE1H] || [dbgE2] || [dbgE3])} { 
                                switch -regexp -- [string tolower [lindex $args $i]] {
                                    {^-nt$}            { incr i
                                                        # check regexp
                                                        if { ![regexp -nocase "(\\^|0|-\[1-7\])-(0|\\d+|\\$)" [lindex $args $i] type NTpre NTpost]} {
                                                          puts "After -nt you must write \"(^|0|-\[1-7\])-(0|<num>|\$)\"."
                                                          puts "The first part defines when to start (^ - dbgOn, 0 - trigger event, -\[1-7\] - is the number of 256B chunks before the trigger event)."
                                                          puts "The last part defines when to stop (0 - trigger event,<num> - num cycles after trigger event, $ - dbgEnd only)."
                                                          puts "e.g. \"-nt -4-$ Sec_Isle\" - will set the trigger machine for tirgger Sec_isle."
                                                          puts "The debug block will save 1 KB of data pre the trigger event and will record until use of dbgEnd."
                                                          return
                                                        }
                                                        if {$NTpost!="$" && [expr {($NTpost/2)>2147483647}]} {
                                                          puts "The maximum limit on cycles after trigger event is 2^32-1."
                                                          puts "If you wish to record until dbgEnd, use $."
                                                                                          
                                                          return
                                                        }
                                                        if {$NTpre==$NTpost} {
                                                          puts "There is no reason in recording 0 cycles. recheck setting."
                                                          return
                                                        }
                                                        incr i
                                                        set ntrigName [lindex $args $i]
                                                      }
                                    {^-f$}            { incr i
                                                        if { ![regexp -nocase ^(pre|post|con)$ [lindex $args $i] filType filType]} {
                                                          puts "After -f you must write \"pre\", \"post\" or \"con\". e.g. -f post FIC0_EVENT5."
                                                          return
                                                        }
                                                        incr i
                                                        if {[lindex $args $i]=="" || [string equal [string index [lindex $args $i] 0] "-"]} {
                                                          puts "You must supply a filter name after -f"
                                                          return
                                                        }
                                                        set filterName [lindex $args $i]
                                                      }
									{^-eidonly[TUCXtucx]$} {
															regexp -- {-eidonly([TUCXtucx])} [lindex $args $i] all_match stormLetter
															set args [linsert $args [expr {$i + 1}] -sf[set stormLetter] -cid[set stormLetter] con @ -eid[set stormLetter]]
															continue
															}
                                    {^-sf[TUCXtucx]([fp])?$}
                                                      {   regexp -- {-sf([TUCXtucx])([fp])?} [lindex $args $i] stormLetter stormLetter type 
                                                          if {[string equal $type f]} {
                                                              set semiFil 2
                                                          } elseif {[string equal $type p]} {
                                                              set semiFil 4
                                                          } else {
                                                              set semiFil 1
                                                          }
                                                          set semiFilters([string toupper [set stormLetter]]semiFil) $semiFil
                                                      }
                                    {^-cid[TUCXtucx]$}	    { regexp {\-cid([TUCXtucx])} [lindex $args $i] stormLetter stormLetter
                                                          set stormLetter [string toupper $stormLetter]
                                                          
                                                          incr i
                                                        if { ![regexp -nocase (pre|post|con) [lindex $args $i] sfilType sfilType]} {
                                                          puts "After -cid$stormLetter you must write \"pre\", \"post\" or \"con\" and cid (@ - to avoid comparing cid)."
                                                          puts "e.g. -cid$stormLetter post 4."
                                                          return
                                                        }
                                                        
                                                        incr i
                                                        set cid [lindex $args $i]
                                                        if {$cid=="" || [string match [string index $cid 0] "-"]} {
                                                          puts "You must supply a cid number after -cid$stormLetter pre|post|con."
                                                          return
                                                        }
                                                        if {![regexp {^(0x[[:xdigit:]]+|@)$} $cid]} {
                                                          puts "The cid can only be a number in HEX or @. recheck -cid$stormLetter."
                                                          return
                                                        }
                                                        
                                                        set semiFilters([set stormLetter]sfilType) $sfilType
                                                        set semiFilters([set stormLetter]cid) $cid
                                                      }
                                    {^-eid[TUCXtucx]$}	    { regexp {\-eid([TUCXtucx])} [lindex $args $i] stormLetter stormLetter	
                                                        incr i
                                                        set stormLetter [string toupper $stormLetter]
                                                        set eid [lindex $args $i]
                                                        if {$eid=="" || [string equal [string index [lindex $args $i] 0] "-"]} {
                                                          puts "You must supply an event id and mask after -eid$stormLetter."
                                                          return
                                                        }
                                                        if {![regexp {^(0x[[:xdigit:]]+)$} $eid]} {
                                                          puts "The event id can only be a number in HEX with 0x. recheck -eid$stormLetter."
                                                          return
                                                        }
                                                        incr i
                                                        set eidMask [lindex $args $i]
                                                        if {$eidMask=="" || [string equal [string index [lindex $args $i] 0] "-"]} {
                                                          puts "You must supply a mask for the event id. e.g. -eid 0x20 0xff. recheck -eid$stormLetter."
                                                          return
                                                        }
                                                        if {![regexp {^(0x[[:xdigit:]]+)$} $eidMask]} {
                                                          puts "The event id mask can only be a number in HEX with 0x. recheck -eid$stormLetter."
                                                          return
                                                        }
                                                        set semiFilters([set stormLetter]eid) $eid
                                                        set semiFilters([set stormLetter]eidMask) $eidMask
                                                      }
                                    {^-(sfp|sff|eid|cid)$} { puts "You have choose a storm for [lindex $args $i]. recheck arg $i."
                                                              return
                                                              }
                                    default            { return "Unallowed arguments. Args can only be -network h(num), -port0, -port1, -otherpath, -wrap, -t, -pci, -dump and -grant.\nRecheck argument [expr {$i+1}]: [lindex $args $i]"}
                                }
                            } else {
                                return "Unallowed arguments. Args can only be -network h(num), -port0, -wrap, -t, -pci, -dump and -grant.\nHINT: Are you using E1H args on E1?\nRecheck argument [expr {$i+1}]: [lindex $args $i]"
                            }
                        }
        }
    }
    
    
    # ensuring the semifilter was configured in a useful way
    if {[dbgE1H] || [dbgE2] || [dbgE3]} { #check rev
    
        foreach stormLetter {X C T U} {
            if {([info exists semiFilters([set stormLetter]cid)] || [info exists semiFilters([set stormLetter]eid)])&& !$semiFilters([set stormLetter]semiFil)} {
                puts "You must use -sf\[TUCX\], -sf\[TUCX\]f or -sf\[TUCX\]p to work with the semi filter machine in [set stormLetter]SEM.\n-cid\[TUCX\] and -eid\[TUCX\] can only be used with it."
                return BAD
            }
            if {$semiFilters([set stormLetter]semiFil)==1 && ![info exists semiFilters([set stormLetter]eid)] && [info exists semiFilters([set stormLetter]cid)] && $semiFilters([set stormLetter]cid)=="@"} {
                puts -nonewline "The filter you have defined is meaningless, all cid, event id and src will pass. Recheck configuration for [set stormLetter]Semi"
                return BAD
            }   
        }
        
    }

    # if the block was already running, ensuring the storms fifo are running
    if {[string equal $DBG_STATUS RUNNING]} {
        emptyFifo
    }
    set onRegs {}
    
    # since dump affect the arrays which keeps the data of the last time, after finishing with dump and starting using the script as usual
    # it's best cleaning all the registers
    if {!$DUMP && $dumpFile!=""} {
          cleanOldCfg
          set dumpFile "closed";
    }
    # clean old conifiguration
    cleanCfg
    dbgReset 
    
    set DBG_STATUS NONE
        
    ###################################
    # opening input file (profile file)
    ###################################
    if {![file exists $input_file]} {
	puts "Can't open input file. It's doesn't exists.\n"
	return
    }
 
    set inFile [open $input_file r] 
    set line [gets $inFile]
    regsub \\s*#.+ $line {} line
    incr line_number
    
    # if working in wrap mode with the internal buffer and the storms this registers must be configurated to 0 
    # or the buffer will work as if it was in one-shot mode
    if {(!$pci_mode && $full_mode)||($grant)} {
        writeByName DBG_REG_NO_GRANT_ON_FULL 0
    }
    #################################################################################################
    ### in the following section the script will read the data from the file and save it as follows:
    ### the header into header variable
    ### old trigger data into slot, frame, data and slotM, frameM amd dataM
    ### new filter into filterLines
    ### new trigger into trigLines
    ### profile into lines
    ### The last 3 are parsed afterward.
    ##################################################################################################
    
    #########################    HEADER SECTION    ##########################
    
    # Skipping the header section in the profiles file
    # Also search for the relevant header if needed
    while {[regexp "^$|^\\s*#.+$|^HEADER\\d+\\s+\[0-9A-F\]+$" $line] && ![eof $inFile]} {
	    regexp "HEADER[set headerNum]\\s+(\[0-9A-F\]+)" $line header header
	    set line [gets $inFile]
	    incr line_number
	    regsub \\s*#.+ $line {} line
    }
    
    # getting network header from profiles.dbg if needed (different from initial value -1
    if {$headerNum!=-1} {
	if {$header==""} {
	    puts "The requested header couldn't be found, please recheck."
	    puts "Maybe you didn't put it in the header section?"
	    close $inFile
	    closeDump 
	    return
	} else {
	    if {![regexp  ^(\[A-F0-9\]{28})$ $header]} {
		puts "The header you requested is not properly defined, header should be 28 hexa digits. Recheck line $line_number."
		close $inFile
		closeDump
		return
	    }
	}
	incr full_mode
	# setting the mode to be constant send
    }
    regsub \\s*#.+ $line {} line
    
    #########################    OLD TRIGGER SECTION    ##########################
    
    # Skipping the trigger section in the profiles file
    # Also search for the needed trigger if needed
    while {[regexp "^$|^\\s*#.+$|^TRIGGER:\\s+\\w+$|^FILTER:\\s+(.+)$|^ENABLE:\\s+(.+)$|^ENDTRIGGER$" $line] && ![eof $inFile]} {
	    incr line_number
	    if {$triggerName!=-1} {
		if {[regexp "TRIGGER: [set triggerName]($|\\s*#\.+)" $line] } {
		    # Getting the filter
		    set line ""
		    while {$line=="" && ![eof $inFile]} {
			set line [gets $inFile]
			incr line_number
			regsub \\s*#.+ $line "" line
		    }
		    if {![regexp "^FILTER:\\s*(\[0-7\])-(\[0-7\])-(\[A-F0-9\]{16})\\s*$" $line slot slot Frame data] } {
			puts "The filter in the trigger is not properly defined, should be "
			puts "\"FILTER H-H-H{16}\". Recheck line $line_number."
			puts "So the first Hexa is the slot (0-7), the second is the frame (0-7)"
			puts "and the last 16 Hexas are the pattern compared."
			close $inFile
			closeDump
			return
		    }
		  
		    # Getting the mask
		    set line ""
		    while {$line=="" && ![eof $inFile]} {
			set line [gets $inFile]
			incr line_number
			regsub \\s*#.+ $line "" line
		    }
		    if {![regexp "^ENABLE:\\s*(\[0-7\])-(\[0-7\])-(\[A-Fa-f0-9\]{16})\\s*$" $line slotM slotM FrameM dataM] } {
		        puts "The bit enable in the trigger is not properly defined, should be"
			puts "\"ENABLE: H-H-H(16)\". Recheck line $line_number."
		        puts -nonewline "So the first Hexa is the slot's mask (0-7), the second is the frame's mask (0-7)"
		        puts "and the last 16 Hexas are mask for the pattern."
		        close $inFile
			closeDump
		        return
		    }
			  
		    set data [format "%.2x" [expr "0x$slot[format %x [expr $Frame<<1]]" >>1]]$data
		    set mask [format "%.2x" [expr "0x$slotM[format %x [expr $FrameM<<1]]" >>1]]$dataM
		  
		  
		    # Ensuring the existance of ENDTRIGGER
		    set line ""
		    while {$line=="" && ![eof $inFile]} {
			set line [gets $inFile]
			incr line_number
			regsub \\s*#.+ $line "" line
		    }
		    if {$line!="ENDTRIGGER"} {
			puts "Triggers must ended with \"ENDTRIGGER\". recheck line $line_number."
			close $inFile
			close $inFile
			closeDump
			return
		    }
		}
	    }
            set line [gets $inFile]
	    regsub \\s*#.+ $line {} line
    }
    
    # Setting the trigger from the profiles file
    if {$triggerName!=-1} {
	if {$mask==-1 || $data==-1} {
	    puts "The requested trigger couldn't be found, please recheck."
	    puts "Maybe you didn't put it in the trigger section?"
	    close $inFile
	    closeDump
	    return
	} else {
	    cfg_trigger $data $mask $type	
	}
    }
    
    #########################    NEW FILTER SECTION    ##########################
    
    # Skipping the new filter section in the profiles file
    # Also search for the needed filter if needed
    while {![regexp "^PROFILE:|^NTRIG:" $line] && ![eof $inFile]} {

	if {$filterName!=-1 && [regexp "^NFILTER:\\s+$filterName\\s*$" $line]} {
	    break
	}
	set line [gets $inFile]
        incr line_number
    }
    
    if {$filterName!=-1} {
	if {![regexp "^NFILTER:\\s+$filterName\\s*$" $line]} {
	    puts "Can't find the requested filter $filterName in the input file $input_file.\nRecheck filter name."
	    close $inFile
	    closeDump
	    return
	}
    
	# transferring all relevant data of the request profile into an array which each cell inside it
	# is a list of two objects the first is the line number the second is the data in that line
	# comments will be deleted, illegel lines will cause the script to stop
	while {![eof $inFile]&&![regexp ^ENDNFILTER $line]} {
	    
	    set line [gets $inFile]
	    incr line_number
	
	    if {[regexp ^\\s*#.+ $line]} {
		continue
	    }
	    if {[regexp "^(HEADER|TRIGGER:|NTRIG:|PROFILE:|NFILTER:)(.+)" $line line line]} {
		puts "New $line before this NFILTER ended.\nNfilters must be seprated by ENDNFILTER.\n\
		Recheck line $line_number."
		close $inFile
		closeDump
		return
	    }
	    if {$line==""} {
		puts "Empty Lines are not allowed inside an nfilter.\nRecheck line $line_number."
		close $inFile
		closeDump
		return
	    }
	    
	    regsub \\s*#(.+) $line {} line
	    lappend filterLines [set line]@$line_number
	}
	
	if {![regexp ^ENDNFILTER $line]} {
	    puts "Reached end of file before reaching ENDNFILTER.\nNfilters must ended with ENDNFILTER."
	    close $inFile
	    closeDump
	    return
	}
	
    }     

    #########################    NEW TRIGGER SECTION    ##########################
    # Skipping the New trigger section in the profiles file
    # Also search for the needed new trigger if needed
        
    #getting to the requested ntrigger 
    while {![regexp "^PROFILE:" $line] && ![eof $inFile]} {
        if {$ntrigName!=-1 && [regexp "^NTRIG:\\s+$ntrigName\\s*$" $line]} {
	    break
	}
	set line [gets $inFile]
        incr line_number
    }
    
    if {$ntrigName!=-1} {
	if {![regexp "^NTRIG:\\s+[set ntrigName]\\s*$" $line]} {
	    puts "Can't find the requested trigger $ntrigName in the input file $input_file.\nRecheck N_TRIGGER name."
	    close $inFile
	    closeDump
	    return
	}
    
	# transferring all relevant data of the request profile into an array which each cell inside it
	# is a list of two objects the first is the line number the second is the data in that line
	# comments will be deleted, illegel lines will cause the script to stop
	while {![eof $inFile]&&![regexp ^ENDNTRIG $line]} {
	    
	    set line [gets $inFile]
	    incr line_number
	
	    if {[regexp ^\\s*#.+ $line]} {
		continue
	    }
	    if {[regexp "^(HEADER|TRIGGER:|NTRIG:|PROFILE:|FILTER:)(.+)" $line line line]} {
		puts "New $line before this N_TRIGGER ended.\nNTRIGs must be seprated by ENDNTRIG.\n\
		Recheck line $line_number."
		close $inFile
		closeDump
		return
	    }
	    if {$line==""} {
		puts "Empty Lines are not allowed inside an N_TRIGGER.\nRecheck line $line_number."
		close $inFile
		closeDump
		return
	    }
	    
	    regsub \\s*#(.+) $line {} line
	    lappend trigLines [set line]@$line_number
	}
	
	if {![regexp ^ENDNTRIG $line]} {
	    puts "Reached end of file before reaching ENDNTRIG.\n N_TRIGGERS must ended with ENDNTRIG."
	    close $inFile
	    closeDump
	    return
	}
	
    }
    
    #########################    PROFILE SECTION    ##########################
    
    #getting to the requested profile
    while {![regexp "PROFILE:\\s+[set profile_name]($|\\s)(\\s?#.+)?" $line] && ![eof $inFile]} {
        set line [gets $inFile]
        incr line_number
    }
    
    if {[eof $inFile]} {
        puts "Can't find the requested profile $profile_name in the input file $input_file.\nRecheck profile name."
	close $inFile
	closeDump
	return
    }
    
    # transferring all relevant data of the request profile into an array which each cell inside it
    # is a list of two objects the first is the line number the second is the data in that line
    # comments will be deleted, illegel lines will cause the script to stop
    while {![eof $inFile]&&![regexp ^ENDPROFILE $line]} {
        
        set line [gets $inFile]
        incr line_number
    
        if {[regexp ^\\s*#.+ $line]} {
            continue
        }
	if {[regexp "PROFILE: (.+)(\\s?#.+)?" $line line line]} {
            puts "New profile before this profile ended.\nProfiles must be seprated by ENDPROFILE.\n\
            Recheck line $line_number."
	    close $inFile
	    closeDump
            return
	}
	if {$line==""} {
	    puts "Empty Lines are not allowed inside a profile.\nRecheck line $line_number."
	    close $inFile
	    closeDump
	    return
	}
        
        regsub \\s*#(.+) $line {} line
	lappend lines [set line]@$line_number
    }
    
    if {![regexp ^ENDPROFILE $line]} {
        puts "Reached end of file before reaching ENDPROFILE.\n Profiles must ended with ENDPROFILE."
	close $inFile
	closeDump
	return
    }
    close $inFile
    
    
    ####################################################
    # getting debugging mode and ensuring its validity
    ####################################################
    if {[llength $lines] ==0} {puts "Syntax error in line $line_number.\nThe profile is not well defined, reached end \
	of profile before getting all profile's data."
        return
    }
    
    getNextLine $lines 0 line line_number
    
    set parms [split $line { } ]
    if {[llength $parms]!=2} {
	puts "Too many/less arguments in line $line_number.\nFor comments use # ....\nOtherwise, should be MODE:\
	<mode_name>."
	
        closeDump
	return
    }
    
    
    if {!([lindex $parms 0]=="MODE:")} {
	puts "Syntax doens't fit requirements in line $line_number.\nThe Profile should begin with: \"MODE: \
	<mode_name>\"."
	closeDump
	return
    }
    
    switch -- [lindex $parms 1] {
	{TDM32} {set mode 0}
	{TDM64} {set mode 0
		 set tdm64 1}
	{64BIT} {set mode 1}
	{24BIT} {if {$ntrigName!=-1} {
                    puts "E1H Trigger Machine isn't usable in 24 bit mode."
		    closeDump
                    return
                }
                set mode 2}
	{default} {puts "Unkown mode: $line.\nYou should use only TDM32 | TDM64 | 24BIT | 64BIT"
                   closeDump
                   return}
    }
    if {[dbgE1H]} {
        ### hw bug workaround ##
        # since the pre windows doesn't work when working in one-shot mode on the internal buffer
        # the script change it to work in wrap mode but stops after (8-pre_chunks)*cycles_per_chunks
        if {!$full_mode && ![string equal $NTpre none] && $NTpre<0} {
            if {$NTpost==0} {
                set full_mode 1
            } elseif {[string equal $NTpost $]} {
                set full_mode 1
                set NTpost [expr {(8+$NTpre)*$CYCLES_PER_MODE($mode)}]
            } else {
                puts "In E1H there is a bug when working with a pre window and one-shot mode on the internal buffer."
                puts "The script can solve the problem for you only when you haven't defined post_cycles."
                puts "In case you did need post cycles, you need to find another way."
                return BAD
            }
        }
    }
    
    set lines [lreplace $lines 0 0]
    getNextLine $lines 0 line line_number
    
    ##########################################################
    # getting the pace of sampling and ensuring its vaildity
    ##########################################################
    if {[regexp "^PACE:" $line]} {
        if {[regexp "^PACE:\\s+(\\d{1,3})$" $line pace pace]} {
            if {$pace>255||$pace<1} {
               puts "PACE: should be a value between 1 to 255.\nRecheck line $line_number"
	       closeDump
               return
            }
            cfg_pace $pace
        } else {puts "PACE: should be a value between 1 to 255.\nRecheck line $line_number"
		closeDump
                return
        }
        set lines [lreplace $lines 0 0]
	getNextLine $lines 0 line line_number
    
    }
    ######################################################################
    # ensuring format exists in the profile, it is needed only for parsing
    # but this script is responsive for checking the profile legitimacy
    ######################################################################
    if {[regexp "^FORMAT:\\s+(.+)" $line fmt fmt]} {
        if {[lsearch $FORMATS $fmt]==-1} {
            puts "FORMAT: can only be NORMAL, FMT, SEM, STORE or PRINTF. Recheck line $line_number."
	    closeDump
           return
        }
        if {$fmt=="STORE"} {
            set lines [lreplace $lines 0 0]
            getNextLine $lines 0 line line_number
            if {![regexp "^STOREADDRESS:\\s+(\\d+)$" $line]} {
                puts "After FORMAT: STORE, you must give STOREADDRESS, for proper parsing.\nRecheck line $line_number"
		closeDump
                return
            }
        }
            
        set lines [lreplace $lines 0 0]
        getNextLine $lines 0 line line_number
        
    } else { puts "FORMAT: is not an optional field, missing before BLOCKS: in line $line_number"
	    closeDump
           return}
     
    # checking the blocks configuration exists
    if {$line!="BLOCKS:"} {
        puts "Syntax error in line $line_number.\nThe profile is not well defined, The next line (excluding comments) after mode must be \
	      \"Blocks: (# optional comment)\""
	return
    }
    set lines [lreplace $lines 0 0]
    
    
    # trageting the relevant subroutine according to the mode
    set res 0
    switch -- $mode {
	{0} { set res [setProfileTDM lines $full_mode $tdm64]}	    
	{1} { set res [setProfile64BIT lines $full_mode]}
	{2} { set res [setProfile24BIT lines $full_mode]}
    }
    
    if {$res=="GOOD"} {
		PrintDebugInfo
        # configurating network after profile configuration is done
        if {$headerNum!=-1} {
            cfg_header $header
            cfg_port $port
			cfg_path
			PrintNetDebugInfo $port
            set OUTMODE "NETWORK"
        }
        if {$pci_size!=0} {
            cfg_pci_dos $pci_size
            set OUTMODE "PCI"
        }
        if {$OUTMODE==-1} {
            set OUTMODE "BUFFER"
        }
	# new filter configuration
        if {[info exists filterLines]} {
	    if {$mode==2} {
		puts "You can't use the new filter machine (E1.5) with 24 bit mode.\nEither use a different mode or use the old filter."
		closeDump
		return 
	    }
            # setting the filter (return the cycle each indirect was used, 129 is never).
            # setFilter configure all the constraints
	    set filterInd [setFilter $filterLines $mode $tdm64]
	    # Ensuring the filter is appropriate
            # setting it went well and if indirect were use then the filter must be in post mode
            # And if the mode isn't con ensuring the trigger exists
            if {$filterInd == "BAD"} {
		dbgReset
		closeDump
                return [cleanCfg]
	    } elseif {![string equal $filterInd "129 129"] && $filType!="post"} {
		puts "You can't use indirect in the filter unless you use post mode."
                dbgReset
		closeDump
		return [cleanCfg]
	    }
	    
	    if {$filType!="con" && ![info exists trigLines]} {
		puts "You can use the filter in pre|post mode only when you define an ntrigger."
                dbgReset
		closeDump
		return [cleanCfg]
	    }
            # turning the filter on and setting mode
            cfg_filter $filType

	}
	# Setting the semifil
        foreach stormLetter {X C T U} {
            if {$semiFilters([set stormLetter]semiFil)} {
                if {![info exists semiFilters([set stormLetter]cid)]} {
                    set semiFilters([set stormLetter]cid) dbg
                    set semiFilters([set stormLetter]sfilType) post
                    # Adding to setNTrig to check indirect 0 was initiliazed
                    set filterInd [lreplace $filterInd 0 0 1]
                } 
                if {![info exists semiFilters([set stormLetter]eid)]} {
                    set semiFilters([set stormLetter]eid) none
                    set semiFilters([set stormLetter]eidMask) 0
                } 
                if {![string equal $semiFilters([set stormLetter]sfilType) con] && ![info exists trigLines]} {
                    puts "You have to defined a ntrigger in order to use semi filter which depend on it."
                    dbgReset
                    closeDump
                    return [cleanCfg]
                }
                cfg_sFilter $stormLetter $semiFilters([set stormLetter]semiFil) $semiFilters([set stormLetter]sfilType) $semiFilters([set stormLetter]cid) $semiFilters([set stormLetter]eid) $semiFilters([set stormLetter]eidMask)
            }
        }
        # new trigger configuration
	if {[info exists trigLines]} {
	    if {$mode==2} {
		puts "You can't use the new trigger machine (E1.5) with 24 bit mode.\nEither use a different mode or use the old trigger."
		closeDump
		return -1
	    }
            # setting the NEW trigger 
            # setNTrig configure all the constraints
	    if {![string equal [setNTrig $trigLines $mode $filterInd $tdm64] "GOOD"]} {
		dbgReset
		closeDump
                return [cleanCfg]
	    }
	    # if the trigger configuration pass well, then turn on the trigger and set its windows
            cfg_ntrig $NTpre $NTpost
	}
        # Adding dbgOn, dbgOff and flushnet to dumped file
			variable dumph_current_config_number
			variable dump_counter_info			
        if {$DUMP} {
			if {$DUMP < 4} {
				puts $dumpFile "\n#To activate the debug block:"
				puts $dumpFile "proc dbgOn {} \{"
			} else {
				set dump_counter_info([set dumph_current_config_number],cfg,count) [expr {[getWriteByNameCounter] - [set dump_counter_info([set dumph_current_config_number],cfg,start)]}]
				set dump_counter_info([set dumph_current_config_number],dbgOn,start) [getWriteByNameCounter]
			}
            dbgOn
			if {$DUMP < 4} {
				puts $dumpFile "\}"
				puts $dumpFile "proc dbgOff {} \{"
			} else {
				set dump_counter_info([set dumph_current_config_number],dbgOn,count) [expr {[getWriteByNameCounter] - [set dump_counter_info([set dumph_current_config_number],dbgOn,start)]}]
				set dump_counter_info([set dumph_current_config_number],dbgOff,start) [getWriteByNameCounter]
			}
            dbgOff
			if {$DUMP < 4} {
            puts $dumpFile "\}"
            puts $dumpFile "\n#To flush the debug block at the end of the work:"
            puts $dumpFile "proc netFlush {} \{"
			} else {
				set dump_counter_info([set dumph_current_config_number],dbgOff,count) [expr {[getWriteByNameCounter] - [set dump_counter_info([set dumph_current_config_number],dbgOff,start)]}]
				set dump_counter_info([set dumph_current_config_number],flush,start) [getWriteByNameCounter]
			}
            flushNet
			if {$DUMP < 4} {
            puts $dumpFile "\}"
            puts $dumpFile "\n#To read the Debug Block's buffer at the end:"
            dumpModeIntrBuf
			dumpModeUsefulFunctions
            closeDump
			} else {
				set dump_counter_info([set dumph_current_config_number],flush,count) [expr {[getWriteByNameCounter] - [set dump_counter_info([set dumph_current_config_number],flush,start)]}]
			}
            set DBG_STATUS DUMP_NO_PCI
            puts "The registers configuration for $profile_name including the cmd line args has been dumped to the requested file."
        } else {
            if {[info exists trigLines]} {
                set trigger_used 1
            }
            set DBG_STATUS CONFIGURED
            puts "Debug block is now configurate to profile $profile_name.\nIn order to start recording use dbgOn, to end use dbgEnd <filename>\n(when working with the network, there is no need for a filename)."

        }
    } elseif {$DUMP} {
		if {$DUMP < 4} {
			puts "The profile failed, first verify the profile without -dump and afterward try again"
			closeDump
		}
    }
    
}

# subroutine which gets an array with all relavent data about the profile. The subroutine verify that all the configurated
# blocks and calender is correct (which mean only up to one block for each pd block is configurated. The lines and the requested shifts
# are appropriate for TDM mode. All slots have been set with a configurated block. All the syntax is correct and in
# the right order). And if so, the debug block and the blocks will be configurated according to the profile.
proc dbgTools::setProfileTDM {_lines full_mode {tdm64 0}} {
    
    variable errors
    variable LINETYPE
    variable SLOTS
    variable blocksPD
    variable usedBlocks
    
    variable is_only_storms
    set is_only_storms true
    array set usedPD {}
    # upper is use when in tdm64
    # if upper is larger then 0 it means that output to the upper 32 bits has already been set
    set upper 0
    set SLOTS ""
    set tdmSlot ""
    
    
    upvar $_lines lines
    
    set clientEnable 0
    set slotsLeft 8
    
    
    
    set i 0
    set line ""
    set line_number 0
    
    getNextLine $lines $i line line_number
    
    ##############################
    # reading blocks configuration
    ##############################
    while {$line!="ENDBLOCKS"} {
        set 64block 0
        # 64block is for the block which is currently being checked
        # 1 if it was configurate with 64:
        # 2 if it was configurate with UPPER:
        # o if it was a block configurate only for the lower bits
	set parms [split $line { }]
    
        if {[llength $parms]!=2 && [llength $parms]!=3} {
                    puts "Too many/less arguments in line $line_number.\nFor comments use # ....\nOtherwise, should\
                    be <block_name>  <line_name> <byte shift | optional>.\nNote: A 64 bits line which contain a 64 bit group can't be use in TDM32 mode.\n\
		    A 64 bits line which contain two 32 bits group can be shifted only by 4.\nA 64 bits line which contain a single 32 bits line can't be shifted at all.\n\
                    if using A 24BIT line, only the first group is used, the allowed shifted are 1,2 in TDM32 mode\n and 4,5,6 in TDM64 depending on the excat groups's size and\
		    order in the line."
                    return
        }
        
        set blk_name [lindex $parms 0]
        set line_name [lindex $parms 1]
	set shift 0
        
        #Check if the block is a storm
        if {![regexp STORM $blk_name]} {
            set is_only_storms false
        }
        
        # if  64: or UPPER: precede the block name, that a tdm64 configuration
        # 64: means both groups in a 64bit line should be sampled. the upper group (after the shift) will be sampled
        # every cycle and the lower one will be sample according to the calendar
        # UPPER: means only the upper group (after the shift) will be sampled every cycle
        # This part will check the legitimacy of the syntax for TDM64
        if {[regexp ^(64|UPPER):(.+)$ $blk_name name type name]} {
            if {!$tdm64} {
                puts "You can't set block for upper bits with TDM32. You have to use TDM64.\nRecheck line $line_number."
                cleanCfg
                return
            }
            if {$upper>0} {
                puts "you can set only one block for the upper 32 bits. You are trying to configure 2 blocks.\nRecheck line $line_number."
                cleanCfg
                return
            }
            if {[regexp STORM $blk_name]} {
                puts "The storm can't output in 64 bits and can't be use in the upper 32 bits.\nRecheck line $line_number"
                return [cleanCfg]
            }
            incr upper
            
            if {$type==64} {
                incr 64block
                set tdmSlot $blk_name
            } else {
                incr 64block 2
                set tdmSlot $name
            }
                
		regsub (64:|UPPER:) $blk_name {} blk_name
        }

	if {[llength $parms]==3} {
            if {![regexp ^\[0-7\]$|^$  [lindex $parms 2]]} {
                puts "Shift can be only from 0 to 7.\nRecheck line $line_number."
                return
            }
            if {[lindex $parms 2]!=""} {
                set shift [lindex $parms 2]
            }
	}
        if {![info exists blocksPD($blk_name)]} {
            puts "Can't find a block named: $blk_name.\nRecheck block's name."
	    cleanCfg    
	    return
        }
        if {[info exists usedBlocks($blk_name)]} {
                puts "You are trying to configure the same block for 2 different lines.\nIncorrect usage - each block can\
                output only from one line"
                return [cleanCfg]

        }
	# getting the line information using getLineOfBlock
        # signature is constant and doesn't depend on the CSVs.
        if {$line_name!="SIGNATURE"} {
            set lineCheck [getLineOfBlock $blk_name $line_name]
        } else {
            set lineCheck "0 $LINETYPE(64BIT,TWO32GROUP)"
        }
		
	# the first parameters is the line number
        set blk_line [lindex $lineCheck 0]
        # the second is the type
	set line_type [lindex $lineCheck 1]
        # the rest is the list of pairs of registers and their desire value
        set regList [lrange $lineCheck 2 end]
        
        if {$blk_line==$errors(EOF)} {
            return [cleanCfg]
        }
	
        if {$blk_line==$errors(line_not_found)} {
            puts "Unknown line name in block $blk_name in line $line_number.\nRecheck line name."
	    return [cleanCfg]
        }
        # if usedPD(<block_num> equals 2 then only an upper block is coming from that PD.
	# if it equals 1 then a lower block comes from that PD (and mabye also an upper one).
	# Since configuring two upper blocks is already limited  (see above) then the following check is relevant only
	# for lower blocks.
        if {[info exists usedPD($blocksPD($blk_name))] && ($usedPD($blocksPD($blk_name)) !=2) && $64block!=2} {
            puts "You are configuring two blocks from the same PD block or the same STORM twice in line $line_number.\n\
            Incorrect usage - their data will overlap.\nRecheck Profile configuration."
	    return [cleanCfg]
	}  
        # if UPPER: was used another block from the same PD block can be configurated (so mark the PD as used for upper)
        # otherwise, mark that PD block as used 
        if {$64block!=2} {     
            set usedPD($blocksPD($blk_name)) 1
        } elseif {![info exists usedPd($blocksPD($blk_name))]} {
	    set usedPD($blocksPD($blk_name)) 2
	}
        # mark the block as used for ensuring it won't be configurate twice and also for
        # ensuring every block used in the calendar is configurated
        # the formula is used because blocks used with UPPER: should not be used in the calendar
        set usedBlocks($blk_name) [expr {1-$64block}]
        if {[regexp STORM $blk_name]} {
                if {$shift} {
                    puts "You can't shift data from the storms.\nRecheck line $line_number."
                    return [cleanCfg]
                }
        }
        # getByteEnable checks the legitimacy of the line for the proper mode and return the byte enable that
        # should be set
        set byteEnable [getByteEnable $line_type $shift $64block $line_number]
        # if there was an error getByteEnable output a warning message and return -1
        if {$byteEnable==-1} {
            return [cleanCfg]
        }
        # if all checks passes configurate the block using cfg_block
        cfg_block $blk_name $blk_line $byteEnable $shift $regList
        
        # i is used to run over the array
	incr i
        if {$i==[llength $lines]} {
            puts "Syntax error in line $line_number.\nThe profile is not well defined, reached end \
            of profile before getting all profile's data."
            return [cleanCfg]        
        }
        # get next line
        getNextLine $lines $i line line_number
	
    }
    
    if {$tdm64&&!$upper} {
        puts "You choose to work in TDM64 but haven't configurate the upper 32 bit of data\n(using UPPER: or 64:). \
        You should either change to TDM32\nor change the configuration."
        return [cleanCfg]
    }
    
    
    # get next line
    incr i
    getNextLine $lines $i line line_number
    
    
    #We now know all the relevant blocks, so we'll configure the full buffer threshold
    set is_tdm_32 1
    if {$tdm64} {
        set is_tdm_32 0
    }
    if {$is_tdm_32 && $is_only_storms} {
        cfg_full_buffer_threshold true
    } else {
        cfg_full_buffer_threshold false
    }
    set slots {}
    
    
    if {![regexp "^CALENDAR:\\s*$" $line]} {
       puts "Syntax error in line $line_number.\nShould be \"CALENDAR:\"."
    }
    # get next line
    incr i
    getNextLine $lines $i line line_number
    
    ########################
    # calendar configuration
    ########################
    while {$line!="ENDCALENDAR"} {
        # A slot can be define as EMPTY or with a block which has been configurated
	if {[regexp "^SLOT\\s+(\[a-zA-z0-9\]+)\\s*$" $line blk_name blk_name]} {
	    if {$blk_name=="EMPTY"} {
		lappend SLOTS "EMPTY"
		append slots "15 "
		incr slotsLeft -1
	    } elseif {![info exists blocksPD($blk_name)]} {
		   puts "Unknown block name: $blk_name in line $line_number.\nRecheck block's name."
                   return [cleanCfg]
	    } elseif {![info exists usedBlocks($blk_name)]} {
		puts "You scheduled to take from a block, you didn't config before in line $line_number.\nRecheck profile\
		configuration"
		return [cleanCfg]
	    } elseif {$usedBlocks($blk_name)<0} {
                puts "You scheduled to take from a block, you didn't config before in line $line_number.\nRecheck profile\
		configuration\nNOTE: When using UPPER the block shouldn't be configurated in the calendar."
		return [cleanCfg]
            } else {
		append slots "[set blocksPD($blk_name)] "
		lappend SLOTS "$blk_name"
		incr slotsLeft -1
	    }
	} else {puts "Syntax error in line $line_number.\nExpected SLOT <block name | EMPTY>."
                return [cleanCfg]
                }
	incr i
	getNextLine $lines $i line line_number
    }
        
    if {$slotsLeft-$tdm64!=0} {
          puts "You have to define exactly 8 slots for TDM32 and excatly 7 for TDM64.\nYou defined [expr 8-$slotsLeft], \
          please recheck the calendar config in line $line_number."
          return
    }
    # All is ready to be configurate
    
    # configurating the calendar
    cfg_cal $slots 
    
    # calculating the needed client enable
    for {set i 0} {$i<9} {incr i} {
            if {[info exists usedPD($i)]} {
                incr clientEnable [expr int(pow(2,$i))]
            }
    }
    
    #configurating the debug block (clientEnable framingMode (TDM is 0) tdm64 full_mode)
    cfg_dbg $clientEnable 0 $tdm64 $full_mode
    if {$tdm64} {
        lappend SLOTS $tdmSlot
    }
    return "GOOD"
}
    
# check the validity of the requested line in the requested mode
# upon error return -1 otherwise the appropriate byteEnable
proc dbgTools::getByteEnable {line_type shift 64block line_number} {
    variable LINETYPE
    
    if {$line_type==$LINETYPE(64BIT,64GROUP)} {
        if {$64block==1} {
            return [expr 0xff*1]
        } elseif {$64block==2} {
            set tmp [expr {int(15*pow(2,$shift+4))}]
            return [expr {$tmp%256+int($tmp/256)}]
        }
        set tmp [expr {int(15*pow(2,$shift))}]
        return [expr {$tmp%256+int($tmp/256)}]
  
        #puts "You can't use 64BIT line with only 64BIT group in TDM mode.\nRecheck line $line_number."
        #return -1
    }
    # $64block==1 - 64: was used $64block==2 - UPPER: was used
    if {$64block==1} {
        if {$line_type!=$LINETYPE(64BIT,TWO32GROUP)} {
            puts "Only line with two 32BITS groups are allowed in TDM64 mode as a full block (64:)"
            return -1
        }
        if {[expr $shift%4]!=0} {
            puts "You can only shift a two 32BITS groups line by 4.\nRecheck line $line_number."
            return -1
        }
        return 0xff
    }
    if {$64block==2} {
        set AllowedShift 4
        set errorType "for upper 32 bits"
    } else {
        set AllowedShift 0
        set errorType "for lower 32 bits"
    }
    
    if {$line_type==$LINETYPE(64BIT,32GROUP)} {
        if {$shift!=$AllowedShift} {
            puts "Your shifting in line $line_number is wrong $errorType.\nYou can only shift this line by $AllowedShift.\nRecheck line $line_number."
            return -1
        } else { return 0xf; }
    }
    if {$line_type==$LINETYPE(64BIT,TWO32GROUP)} {
        if {[expr $shift%4]!=0} {
            puts "You can only shift a two 32BITS groups line by 4.\nRecheck line $line_number."
            return -1
        } else {
            set tmp [expr int(15*pow(2,$shift+$AllowedShift))]
            return [expr $tmp%256+int($tmp/256)]
        }
    }
    if {[expr $line_type%3]==0} {
        if {$shift!=$AllowedShift} {
            puts "You can shift line $line_number only by $AllowedShift.\nRecheck that line."
            return -1
        }
        return [expr int(pow(2,int($line_type/3)))-1]
    }
    if {$line_type==$LINETYPE(24BIT,THREE8BIT)} {
        if {$shift!=$AllowedShift && $shift!=$AllowedShift+1 && $shift!=$AllowedShift+2} {
            set as $AllowedShift
            puts "You can shift line $line_number only by [set as], [expr $as+1] or [expr $as+2].\nRecheck that line."
            return -1
        }
        return [expr int(pow(2,$shift-$AllowedShift))]
    }
    if {$line_type==$LINETYPE(24BIT,8BIT8BIT)||$line_type==$LINETYPE(24BIT,16BIT8BIT)} {
        if {$shift!=$AllowedShift && $shift!=$AllowedShift+1} {
            set as $AllowedShift
            puts "You can shift line $line_number only by [set as] or [expr $as+1].\nRecheck that line."
            return -1
        }
        if {$line_type==$LINETYPE(24BIT,8BIT8BIT)} {
            return [expr int(pow(2,$shift-$AllowedShift))]
        } else { return [expr int(pow(6,$shift-$AllowedShift))] }
    }
    if {$line_type==$LINETYPE(24BIT,8BIT16BIT)} {
        if {$shift!=$AllowedShift  && $shift!=$AllowedShift+2} {
            set as $AllowedShift
            puts "You can shift line $line_number only by [set as] or [expr $as+2].\nRecheck that line."
            return -1
        }
        return [expr {3+int(($shift-$AllowedShift)/2)}]
    }
        
}

proc dbgTools::set64BITMode {blk_name} {
    variable errors
    variable LINETYPE
    variable blocksPD
    variable usedBlocks
    variable SLOTS
    
    set clientEnable [expr {int(pow(2,$blocksPD($blk_name)))}]
    #the last argument for cfg_dbg is full mode (1 is for wrap and 0 is for one shot)
    cfg_dbg $clientEnable 1 0 1
    set SLOTS $blk_name
}

proc dbgTools::setTDM64Mode {blk_name} {
    variable errors
    variable LINETYPE
    variable blocksPD
    variable usedBlocks
    variable SLOTS
    
    set clientEnable [expr {int(pow(2,$blocksPD($blk_name)))}]
    #the last argument for cfg_dbg is full mode (1 is for wrap and 0 is for one shot)
    cfg_dbg $clientEnable 1 0 1
    set SLOTS $blk_name
    #incomplete function
}


proc dbgTools::simpleDbgBlockSwitch {isOn} {
    writeByName DBG_REG_DBG_BLOCK_ON $isOn
}

proc dbgTools::setLineOfBlock {blkName lineNumber {byteen 0xff}} {
    set usedBlocks($blkName) 1
    cfg_block $blkName $lineNumber $byteen 0 {}
}
proc dbgTools::helpStaticDebug {args} {
    puts "For using StaticDebug:"
    puts "staticDebug <Block Name> <Output Data> <optional: OVERRIDE>"
    puts "For example, staticDebug PBF pbf.txt OVERRIDE"
    puts " if using the OVERRIDE, it overrides older profiles set by the setProfile method and not used by dbgOn and dbgEnd."
}

proc dbgTools::helpE1FixStaticDebug {args} {
    		puts "You can use the E1FixStaticDebug function as follows (before going on to the next block): "
		puts "From the block you've just output data, select a line that has at least 1 byte that will be 0 (constantly from now on)"
		puts "run: E1FixStaticDebug <block_name> <line number> <byte enable of the specified zero byte>"
}

proc dbgTools::E1FixStaticDebug {blk_name line_number byte_en} {
   setLineOfBlock $blk_name $line_number $byte_en
   puts "Done with fix."
}

proc dbgTools::staticDebug {blk_name file_name {override no}} {
    variable DBG_STATUS
     if {[string equal $DBG_STATUS RUNNING]} {
            puts "Debug Block is currently already running. Try DbgEnd first."
            return -1
        }
        
    if {![string equal $override OVERRIDE]} {
        if {[string equal $DBG_STATUS CONFIGURED]} {
    	puts "You have used setProfile but didn't used dbgOn. Debug Block is already configured to some profile."
            puts "You can call staticDebug with the additional override parameter set to OVERRIDE."
            helpStaticDebug
            return -1
        }
    } else {
        puts "Performing Override..."
        cleanCfg
    }
    array set bl_lines {}
    getDebugLinesofBlock $blk_name bl_lines
    set after_last_lines [set bl_lines(count)]
    puts "Setting Debug Block to 64 bit mode"
    set64BITMode $blk_name
    set dump_file [open $file_name "w"]
    puts $dump_file "BLOCK $blk_name"
    set start_off [get_start_offset [reg_offset -n DBG_REG_CALENDAR_OUT_DATA]]
    set num_regs 2
    set valid_reg_offset [expr {2 * 4}]
    puts "Turning on Debug Block"
    simpleDbgBlockSwitch 1
    for {set i 1} {$i < $after_last_lines} {incr i} {
        
        setLineOfBlock $blk_name $i
        #for WideBus, read first offset
        set low [reg read $start_off]
        set high [reg read [expr {$start_off + 4}]]
        set valid_reg [reg read [expr {$start_off + $valid_reg_offset}]]
        set frame_bit [expr {$valid_reg & 0x1}]
        set is_line_valid [expr {$valid_reg & 0x10}]
        if {$is_line_valid > 0} {
            puts $dump_file "$i [set bl_lines($i,name)]"
        
            puts $dump_file "Reg 1: $low"
            puts $dump_file "Reg 2: $high"
            puts $dump_file "Frame Bit: $frame_bit"
            puts $dump_file ""
        }
        
    }
    puts "Turning off Debug Block..."
    
    simpleDbgBlockSwitch 0
    cleanOldCfg
    puts "Done."
    variable chip_ver
    if {$chip_ver == 1} {
        puts "Warning: On E1, Due to a bug in the Debug Block, you cannot use the Static Debug again on another block in the same PD."
		puts "You can use the E1FixStaticDebug function as follows (before going on to the next block): "
		puts "From the block you've just output data, select a line that has at least 1 byte that will be 0 (constantly from now on)"
		puts "run: E1FixStaticDebug <block_name> <line number> <byte enable of the specified zero byte"
    }
    close $dump_file
}
# subroutine which gets an array with all relavent data about the profile. The subroutine verify that all the configurated
# blocks and calender is correct (which mean only one block is configurated. The line is appropriate for 64BIT mode.
# All the syntax is correct and in the right order). And if so, the debug block and the block will be configurated
# according to the profile.
proc dbgTools::setProfile64BIT {_lines full_mode} {
    
    variable errors
    variable LINETYPE
    variable blocksPD
    variable usedBlocks
    variable SLOTS
    upvar $_lines lines
    
    set i 0
    set line ""
    set line_number 0
    
    cfg_full_buffer_threshold false
    getNextLine $lines $i line line_number
    
    set parms [split $line { }]
    
    if {[llength $parms]!=2} {
                puts "Too many/less arguments in line $line_number for requested mode.\nFor comments use # ....\nOtherwise,\
                should be <block_name>  <line_name>."
        return
    }
    
    
    set blk_name [lindex $parms 0]
    set line_name [lindex $parms 1]
    set regList ""
    
    if {![info exists blocksPD($blk_name)]} {
        puts "Can't find a block named: $blk_name.\nRecheck block's name."
        return
    }
    # Signature is constant and doesn't depend on the CSVs.
    if {![regexp STORM $line]} {
        if {$line_name!="SIGNATURE"} {
            set lineCheck [getLineOfBlock $blk_name $line_name]
        } else {
            set lineCheck "0 $LINETYPE(64BIT,64GROUP)"
        }
        set blk_line [lindex $lineCheck 0]
	set line_type [lindex $lineCheck 1]
        set regList [lrange $lineCheck 2 end]
        
    } else {
        puts "Storms works with the debug block only in TDM mode, and output only 32 bit a cycle.\n\
        you can't use them in other modes. Please fix your profile.\n"
        return
    }
    
    if {$blk_line==$errors(EOF)} {return}
    if {$blk_line==$errors(line_not_found)} {
        puts "Unknown line name in block $blk_name in line $line_number.\nRecheck line name."  
	return
    }
    
    if {$line_type!=$LINETYPE(64BIT,64GROUP)} {
	    puts "Only 64 bits lines with a single 64 bit group are allowed.\nRecheck setting in line $line_number.\n"
	    return
    }

    set usedBlocks($blk_name) 1
    cfg_block $blk_name $blk_line 0xff 0 $regList
    incr i
    getNextLine $lines $i line line_number
    
    if {$line!="ENDBLOCKS"} {
        
        puts "In 64bit mode, you can configurated only one block. Afterward, there should be \"ENDBLOCKS\".\
        \nRecheck line $line_number."
        return [cleanCfg]
    } 
    
    set clientEnable [expr {int(pow(2,$blocksPD($blk_name)))}]
    cfg_dbg $clientEnable 1 0 $full_mode
    set SLOTS $blk_name
    return GOOD
}

# subroutine which gets an array with all relavent data about the profile. The subroutine verify that all the configurated
# blocks and calender is correct (which mean up to three blocks are configurated. The lines are appropriate for 24BIT mode.
# All the syntax is correct and in the right order). And if so, the debug block and the block will be configurated
# according to the profile.
proc dbgTools::setProfile24BIT {_lines full_mode} {
    
    variable errors
    variable LINETYPE
    variable blocksPD
    variable usedBlocks
    array set usedPD {}
    # array used for calculating the client enable
    array set usedBytes {}
    # In 24 BIT mode only line with 24 bit are allowed
    # shifting by different numbers then thus will shift bytes to bytes which are not been samples
    # e.g. shift byte 0 by 4, will put it as byte 4 which doesn't get sampled at all in 24 bit
    # (the check is done only if byte was set a enabled)
    array set allowedShift {0 {6 7} 1 {1 7} 2 {1 2}}
    
    set clientEnable 0
    upvar $_lines lines
    
    set i 0
    set line ""
    set line_number 0
    cfg_full_buffer_threshold false
    getNextLine $lines $i line line_number
    
    # checking and configurating the blocks
    # The syntax expected is <block_name>  <line_name> <Byte_Enable> <Shift |optional>
    while {$line!="ENDBLOCKS"} {
	
	set shift 0
	set parms [split $line { }]
        
	if {[llength $parms]!=4 && [llength $parms]!=3} {
		    puts "Too many/less arguments in line $line_number for the requested mode.\nFor comments use # ....\nOtherwise,\
		    should be <block_name>  <line_name> <Byte_Enable> <Shift | optional>."
	    return
	}
    
    
	set blk_name [lindex $parms 0]
	set line_name [lindex $parms 1]
	set regList ""
	
	if {![regexp ^\[01\]{3}$ [lindex $parms 2] byteEnable]} {
	    puts "The byte enable should be build out of three 1 or 0.\nRecheck line $line_number.\nRemember the syntax is <block_name>  <line_name> <Byte_Enable> <Shift>."
	    return [cleanCfg]
	}
	
	if {[llength $parms]==4} {
	    if {![regexp ^\[0-7\]$|^$  [lindex $parms 3]]} {
		puts "Shift can be only from 0 to 7.\nRecheck line $line_number."
		return
	    }
	    if {[lindex $parms 3]!=""} {
		set shift [lindex $parms 3]
	    }
	}
	# Configurating a line with no byte enable is useless
	if {$byteEnable=="000"} {
	    puts "There is no use of configuring a block but setting its byte enable to 0 on all bytes"
	    return [cleanCfg]
	}
        
        if {![info exists blocksPD($blk_name)]} {
            puts "Can't find a block named: $blk_name.\nRecheck block's name."
	    cleanCfg    
	    return
        }
        if {[info exists usedBlocks($blk_name)]} {
                puts "You are trying to configure the same block for 2 different lines.\nIncorrect usage - each block can\
                output only from one line"
                return [cleanCfg]
        }
	        
        
	if {![regexp STORM $line]} {
	    # getting the line information using getLineOfBlock	
	    set lineCheck [getLineOfBlock $blk_name $line_name]
	    # the first parameters is the line number
	    set blk_line [lindex $lineCheck 0]
	    # the second is the type
	    set line_type [lindex $lineCheck 1]
	    # the rest is the list of pairs of registers and their desire value
	    set regList [lrange $lineCheck 2 end]
	    
	} else {
	    puts "Storms works with the debug block only in TDM mode, and output only 32 bit a cycle.\n\
	    you can't use them in other modes. Please fix your profile."
	    return
	}
	# The getLineOfBlock prints the eof error to the screen
	if {$blk_line==$errors(EOF)} {return}
	if {$blk_line==$errors(line_not_found)} {
	    puts "Unknown line name in block $blk_name in line $line_number.\nRecheck line name."  
	    return
	}
        # all allowed linetypes for 24 BIT mode values are higher then 24BITSTART
	if {$line_type<$LINETYPE(24BITSTART)} {
		puts "Only 24 bits lines are allowed in 24BIT mode.\nRecheck setting in line $line_number."
		return
	}
	set byteEn_dec 0
        
        ###########################################################################
        # Checking the configured bytes doesn't overlap and have resaonable shifts.
        ###########################################################################
	
        for {set j 0} {$j<3} {incr j} {
	    if {[string index $byteEnable [expr 2-$j]]} {
		if {$shift&&[lsearch -exact $allowedShift($j) $shift]==-1} {
		    puts "Shifting Byte $j by $shift is useless. It won't output data like that.\nRecheck setting in line $line_number"
		    return [cleanCfg]
		}
		set byte [expr ($j-$shift+8)%8]
		if {[info exists usedBytes($byte)]} {
		    puts "You have configurated two blocks to output on the same byte.\nThey will overlap each other.\nRecheck setting in line $line_number."
		    return [cleanCfg]
		}
		set usedBytes($byte) 1
		incr byteEn_dec [expr int(pow(2,$j))]
	    }
	}
        ##################################################
        # Checking non-existing bytes haven't been enabled
        ##################################################
        
	switch -- $line_type {
	    {$LINETYPE(8BIT)} {
				if {$byteEn_dec!=1} {
				    puts "You have configurate a line with only 1 valid byte to output more than 1 byte.\n\
				    Recheck line $line_type."
				    return [cleanCfg]
				}
	    		    }
	    {$LINETYPE(8BIT8BIT)} --
	    {$LINETPYE(16BIT)} {
				if {$byteEn_dec>3} {
				    puts "You have configurate a line with only 2 valid byte to output none existing bytes.\n\
				    Recheck line $line_type."
				    return [cleanCfg]
				}
	    }
	}
        ################################################
        # Saving the relevant blocks for later cleaning
        ################################################
	set usedBlocks($blk_name) 1
	set usedPD($blocksPD($blk_name)) 1
	
        # configuring the bloc
        cfg_block $blk_name $blk_line $byteEn_dec $shift $regList
        incr i
	getNextLine $lines $i line line_number
	
    }
    
    # calculating the needed client enable
    for {set i 0} {$i<9} {incr i} {
            if {[info exists usedPD($i)]} {
                incr clientEnable [expr int(pow(2,$i))]
            }
    }
    
    cfg_dbg $clientEnable 2 0 $full_mode
    return GOOD
}


# subroutine which gets an array with all relavent data about the ntrigger. The subroutine verify that all the configurated
# states and sets is correct - trigger is reachable, use of indirect is valid, 
# in the requested profile. 
proc dbgTools::setNTrig {lines mode filter_ind {tdm64 0}} {
    
    # hold the block in each slot
    variable SLOTS
    # An array of operations to their numbers    
    variable OPS
    
    # holds all the data needed between states:
    # indirect uses by the filter are counted as in state 3.
    # template - meaning
    # <state>,use,<indirect_num> - the cycle state <state> use indirect <indirect_num>
    # <stateA>,from,<stateB> - from stateB it is possible to reach stateA
    # <state>,rec<indirect_num>,cycle - the cycle in state <state> record indirect <indirect_num>
    # <state>,rec<indirect_num>,mask - the mask, state <state> use to record indirect <indirect_num>
    # <state>,rec<indirect_num>,shift - the shift, state <state> use to record indirect <indirect_num>
    # <state>,ml - the message length in the state - only if message length was defined
    array set state_d {}

    set i 0
    set line ""
    set line_number 0
    # goto is always bigger than the highest reachable state  
    set goto 1
    set trigReach 0
    # set_info holds all the data for a set during it configuration
    # the constraints are order first by there cycle and afterward by index in that cycle
    # template - meaning
    # <cycle_num> - the number of constraint on that cycle
    # <cycle_num>,ind<index> - whether constraint <index> in cycle <cycle> use indirects
    # <cycle_num>,mask<index> - exists only for equals constraints, hold the mask
    # <cycle_num>,must<index> - whether or not the constraint is must
    # <cycle_num>,fr<index> - exists only for equals constraints, hold the frame to compare to
    # <cycle_num>,fr_mask<index> - exists only for equals constraints, hold the frame mask
    # <cycle_num>,val<index> - The val to compare to.
    # <cycle_num>,cyclic<index> - exist only in non-equals operator, whether or not the operator is cyclic
    # <cycle_num>,range<index> - exist only in non-equals operator, the range for the constraint (5 lsb are the size, [9:5] bits are the offset).
    # <cycle_num>,op<index>  - the operator constraint <index> in <cycle> uses.
    array set setInfo {}
    
    # adding filter indirects into state_d
    for {set j 0} {$j<2} {incr j} {
        if {[lindex $filter_ind $j]!=129} {
            set state_d(3,use,$j) [lindex $filter_ind $j]
        }
    }
        
    
    # first we scans the trigger by States
    for {set state 0} {$state<$goto} {incr state} {
        # getting the state relevant block and validating the syntax
	set tdm64block 0
        getNextLine $lines $i line line_number
        
        if {![regexp {S(\d):\s+BLOCK:\s+([0-9_A-Z:]+)\s*(\s+ML:\s+(\d+))?$} $line blk_name num blk_name ml ml]} {
            puts "Trigger must be build of states, by using\n \"S[set state]: BLOCK: <block_name> \[ML: <message length> - opt\]\".\nrecheck line $line_number"			
            return "BAD"
        }
        if {$num!=$state} {
            puts "Trigger must be build of orderly sorted states.\nCurrently expecting state $state, recheck line $line_number"
            return "BAD"
        }
	
        # parsing the header of a set by parseFilTrigHeader
        if {[string equal [parseFilTrigHeader $blk_name ml $tdm64 tdm64block line line_number $lines i TRIGGER_STATE $state] "BAD"]} {
            return "BAD"
        }
        
        if {$ml!=""} {
            set state_d($state,ml) $ml
        } else {
            set ml 128
        }

	# Scans the sets
        # Assuming use of two sets, might be disabled later
        writeByName DBG_REG_TRIGGER_STATE_USE_BOTH_SETS_$state 1
        # parsing the sets
        for {set d_set 0} {$d_set<2} {incr d_set} {
            
            if {![regexp "^\\s*SET(\\d):\\s*(\\s+COUNT:\\s+(\\d+))?$" $line num num count count]} {
                puts "State must be build of orderly sets.\nCurrently expecting set $d_set, recheck line $line_number"
                return "BAD"
            }
			
            if {$num!=$d_set} {
                puts "Currently expecting set [set d_set], recieved set [set num]. recheck line $line_number"
                return "BAD"
            }
			
            if {[info exists count] && $count !=""} {
                writeByName DBG_REG_TRIGGER_STATE_SET_COUNT_[expr {$state*2+$d_set}] $count
                unset count
            }
	    incr i
	    getNextLine $lines $i line line_number
            # using parseSet to parse all constraints and prepare setInfo
            set usedInd [parseSet $mode setInfo TRIGGER $tdm64block line line_number $lines i $ml]
            if {[string equal $usedInd BAD]} {
                return BAD
            }
            # add the indirect use by the parsed set to the state_d data base 
            for {set j 0} {$j<2} {incr j} {
                if {[lindex $usedInd $j]!=129 &&
                    (![info exists state_d($state,use,$j)] || $state_d($state,use,$j)>[lindex $usedInd $j])} {
                        set state_d($state,use,$j) [lindex $usedInd $j]
                }
            }
               
            # Ensure goto exist and set goto to the maximum reachable state for now
	    if {[regexp "^\\s*GOTO: (S\[0-2\]|TRIG)$" $line temp temp]} {
		if {$temp=="TRIG"} {
		    set trigReach 1
		    set set_goto 3
		} else {
		    set set_goto [string index $temp 1]
		    if {$set_goto>[expr {$goto-1}]} {
			set goto [expr {$set_goto+1}]
		    }   
		}
		
		incr i
		getNextLine $lines $i line line_number
		set state_d($set_goto,from$state) 1
		writeByName DBG_REG_TRIGGER_STATE_SET_NXT_STATE_[expr {$state*2+$d_set}] $set_goto
	    } else {
		    puts "Set must end with GOTO: <S<state>|TRIG>."
                    puts "Recheck line $line_number."
		    return "BAD"
	    }

	    if {![regexp {^\s*ENDSET} $line]} {
		puts "A Set must end with ENDSET, recheck line $line_number."
		return "BAD"
	    }

	    # configuring the set using setInfo
            cfg_set setInfo TRIGGER_STATE_SET $state $d_set

	    incr i
            getNextLine $lines $i line line_number
	    if {![regexp "^\\s*SET[expr {$d_set+1}]:" $line]} {
		if {$d_set==0} {
                    writeByName DBG_REG_TRIGGER_STATE_USE_BOTH_SETS_$state 0
                }
		set d_set 2
	    }
	    unset setInfo	
        }
	# taking care of Indirect save 
        while {[regexp "^\\s*SAVE(\\d):\\s+(.+)(\\s+SHIFT: (\\d))?\\s*$" $line temp indNum fieldDesc iShift iShift]} {
            set indOff {}
            set size {}
            if {[parseFieldDesc $fieldDesc indOff size $line_number]!={OK}} {
                return BAD
            }
            if {[info exists state_d($state,rec$indNum,cycle)]} {
                puts "You already defined a save for indirect $indNum in state $state."
                puts "Recheck line $line_number."
                return "BAD"
            }
            if {![expr {($indOff%8==0)&($size%8==0)}]} {
                puts "You can only save fields which are align to bytes."
		puts "Recheck line $line_number."
                return "BAD"
            }
            # The cycle registers in the debug block takes the lsb as whetever to look on the lower or upper 32bit in the cycle.
            # and on the rest of the bits as the number. e.g. if cycle equals 9 then in cycle 4 (counting from 0) look on the upper 32 bit.
            if {$mode==1} {
               set cycle [expr {($indOff/64)*2+(($indOff/32)%2==1)}]
            } else {
                set cycle [expr {($indOff/32)*2}]
            }

            if {$cycle>=[expr {$ml*2}]} {
                puts "You requested to save data from an offset which doesn't exist in the message length you defined."
                puts "(when working with the frame, the top cycle for saving is 128). $ml"
                puts "Recheck line $line_number."
                return BAD
            }
            if {[info exists state_d($state,use,$indNum)]} {
                if {$state_d($state,use,$indNum)<=[expr {$cycle+1}]} {
                    puts "If you record a value in a state you can use it only in cycles afterward (and at least 2 cycles after recording)."
                    puts "Recheck state $state, line $line_number."
                    return "BAD"
                } else {
                    unset state_d($state,use,$indNum)
                }
            }
            set state_d($state,rec$indNum,cycle) $cycle
            set indOff [expr {$indOff%32}]
            if {[expr {(($indOff+$size)>32)| (($indOff+$size)==0 )}]} {
                puts "The requested field to save lays in two continuous cycles."
                puts "Recheck line $line_number."
                return "BAD"
            }
	    if {[string equal $iShift ""]} {
		set iShift 0
	    } elseif {$iShift>3} {
		puts "You can only shift the indirect value by 0 to 3."
		puts "Recheck line $line_number."
		return "BAD"
	    }

            set state_d($state,rec$indNum,shift) $iShift
            set mask [expr {pow(2,$size/8)-1}]
            set mask [expr {~(int($mask)<<[expr {$indOff/8}])}]
            set state_d($state,rec$indNum,mask) $mask
	    incr i
            getNextLine $lines $i line line_number
	}
        
        # checking the state was ended properly
	if {![regexp {^\s*ENDS} $line]} {
	    puts "A State must end with ENDS, recheck line $line_number.\n"
	    return "BAD"
	}
        incr i
        getNextLine $lines $i line line_number
	
	
	
    }
    # checking the new trigger was ended properly
    if {$line!="ENDNTRIG"} {
	if {[regexp "^S\\d:" $line]} {
	    puts "You can have only states which previous states can reach."
	    return "BAD"
	} else {
	    puts "A N_TRIGGER must end with ENDNTRIG (no spaces before), recheck line $line_number"
	    return "BAD"
	}
    }
    # ensuring it is possible to reach the trigger
    if {!$trigReach} {
	puts "The n_trigger you have defined never create trigger event, recheck it."
	return "BAD"
    }
    #### check indirect values are always initiailized before used.
    if {[array names state_d 0,use]!=""} {
        puts "State 0 can only use indirect that was recorded in state 0. recheck state 0."
        return "BAD"
    }
    for {set state 1} {$state<4} {incr state} {
        if {[info exists state_d($state,from0)]} {
            set min 0
        } elseif {[info exists state_d($state,from1)]} {
            set min 1
        } elseif {[info exists state_d($state,from2)]} {
            set min 2
        } else {
		#no error message since checking if states are reachable occurred earlier. This is only for indirect - when it was saved and when it is used.
           continue
        }

        foreach used [array names state_d $state,use*] {
            regexp "$state,use,(\\d+)" $used ind ind
            if {$state!=3} {
                set whenComment "state $state"
            } else {
                set whenComment {the filter}
            }
            
            if {![info exists state_d($min,rec[set ind],shift)] && ![info exists state_d(0,rec[set ind],shift)]} {
		if {$state!=3} {
		    puts "You request to use indirect register $ind in state $state, although it was never\
		    initialized in the this state or in the state prior to it."
		    return "BAD"
		} else {
		    puts "You request to use indirect register $ind in the filter, although it was never\
		    initialized."
		    return "BAD"
		}
            }
            if {$state_d($state,use,$ind)<2} {
                foreach fromState "0 $min" {
                    if {[info exists state_d($fromState,rec[set ind],shift)] && [info exists state_d($fromState,ml)] \
                        && [expr {(int($state_d($fromState,rec[set ind],cycle)/2)+1)==$state_d($fromState,ml)}]} {
                        puts "WARNING: you record indirect $ind in state $fromState in the last cycle of the"
                        puts "message and use it directly in the first cycle of $whenComment."
                        puts "Since the Indirect registers are ready to use only one cycle"
                        puts "after recording, the value in the first comparsion will be wrong."
                        break
                    }
                }
            }
        }
    }
    # configure INDIRECT!!!
    set recState0 0
    set recState1 0

    for {set j 0} {$j<3} {incr j} {
	if {[array names state_d [set j],rec0,shift]!=""} {
	    incr recState0 [expr {int(pow(2, $j ))}]
	}
	if {[array names state_d [set j],rec1,shift]!=""} {
	    incr recState1 [expr {int(pow(2, $j ))}]
	}
    }
    writeByName DBG_REG_TRIGGER_INDIRECT0_STATE $recState0
    writeByName DBG_REG_TRIGGER_INDIRECT1_STATE $recState1
    foreach used [array names state_d *,rec?,shift] {
	regexp "(\\d+),rec(\\d+)" $used st st ind
    	writeByName DBG_REG_TRIGGER_INDIRECT[set ind]_OFFSET_$st $state_d([set st],rec[set ind],cycle)
	writeByName DBG_REG_TRIGGER_INDIRECT[set ind]_SHIFT_$st $state_d([set st],rec[set ind],shift)
	writeByName DBG_REG_TRIGGER_INDIRECT[set ind]_MASK_$st $state_d([set st],rec[set ind],mask)
    }
    
    return "GOOD"
}

proc dbgTools::setFilter {lines mode {tdm64 0}} {
    
    variable SLOTS
    variable OPS
    
    set i 0
    set line ""
    set line_number 0
    set tdm64block 0
    # same as in setNTrig
    array set setInfo {}
    

    getNextLine $lines $i line line_number
        
    if {![regexp {BLOCK:\s+([0-9_A-Z]+)(\s+ML:\s+(\d+))?$} $line blk_name blk_name ml ml]} {
        puts "Filter must be start with the relevant block, by using\n \"BLOCK: <block_name> \[ML: <message length> - opt\]\".\nrecheck line $line_number"			
        return "BAD"
    }
    # parsing the header of a set by parseFilTrigHeader
    if {[parseFilTrigHeader $blk_name ml $tdm64 tdm64block line line_number $lines i FILTER]} {
        return "BAD"
    }

    if {$ml==""} {
        set ml 128
    }
    
    set constraint 0
    
    # using parseSet to parse all constraints and prepare setInfo
    set usedInd [parseSet $mode setInfo FILTER $tdm64block line line_number $lines i $ml]
    if {[string equal $usedInd BAD]} {
        return BAD
    }
    # checking the filter ended properly
    if {$line!="ENDNFILTER"} {
    	puts "A Filter must end with ENDFILTER, recheck line $line_number."
        return "BAD"
    }
    # configuring the set using cfg_set
    cfg_set setInfo FILTER
    
    #  return if the indirect registers were used
    return $usedInd
}
# Subrouting which recieve the header of the filter or the new trigger and confiure registers in the debug block
proc dbgTools::parseFilTrigHeader {blk_name _ml tdm64 _tdm64block _line _line_number lines _index type {state ""}} {
    
    variable blocksPD
    variable SLOTS
    upvar $_line line
    upvar $_line_number line_number
    upvar $_tdm64block tdm64block
    upvar $_index i
    upvar $_ml ml
    
    # adjusing the template of the register name (differance between filter and trigger)
    if {[string equal $state ""]} {
	set underScoreState ""
    } else {
	set underScoreState _$state
    }
    # Ensuring the requested block exists
    if {![info exists blocksPD($blk_name)] } {
        if {!([regexp "^(64):(.+)" $blk_name blk blk blk] && [info exists blocksPD($blk)])} { 
            puts "Unknown block $blk_name in line $line_number.\n"
            return "BAD"
        }
    }
    set blkSlots 0
            
    # SLOTS is the slots list in TDM mode
    # blkSlots is 8 bit - each bit represent one slot
    # if the the bit is 1 the set will treat data from that slot
    for {set slot 0} {$slot<[llength $SLOTS]} {incr slot} {
        if {[lindex $SLOTS $slot]==$blk_name} {
            incr blkSlots [expr {int(pow(2,$slot))}]
        }
    }
    # Ensure the block was used in the profile
    if {$blkSlots==0} {
        puts "You have'nt configured block $blk_name to output in this profile.\nrecheck line $line_number.\n"
        return "BAD"
    }
   
    # The upper slot in TDM need VALID_SEL to be turn on
    if {$tdm64} {
        writeByName DBG_REG_[set type]_VALID_SEL$underScoreState [expr {int($blkSlots/128)}]
        
        set tdm64block [expr {int($blkSlots/128)}]
        set blkSlots [expr {$blkSlots%128}]
        
    }
    
    writeByName DBG_REG_[set type]_SLOTS$underScoreState $blkSlots
    incr i
    getNextLine $lines $i line line_number
    # checking whetever to work with message length or with frame and configure appropriately 
    if {$ml!=""} {
        if {$ml >129 || $ml <1} {
            puts "When defining a message length, it must be between 1 to 128\n"
            return "BAD"
        }
        writeByName DBG_REG_[set type]_MSG_LENGTH$underScoreState [expr {$ml-1}]
    } else {
        writeByName DBG_REG_[set type]_MSG_LENGTH_ENABLE$underScoreState 0
    }
    
    return 0

}

# a procedure which recieve a field descriptor and separate it into offset and size
# filed descriptor can be one of 2 options:
# 1. FIELD: <fieldname> ?OFF: <offset>? ?SIZE: <size>? - those who wrapped in ? are optional. the offset is relative to the field
#    offset in everest.sdf and the size is absolute.
# 2. OFF: <offset> SIZE: <size> - both are mandatory and absolute.
# the results are return to the variables names given in _offset and _size
# line number is used only for error_report
# the procedure used fieldsInfo which is created by parseSdf
proc dbgTools::parseFieldDesc {fieldDesc _offset _size line_number} {
    variable fieldsInfo
    variable chip_mc_ver
    variable sdfs_files
    upvar $_offset offset $_size size
    
    set currField [lindex $fieldDesc 0]
    if {[string equal $currField FIELD:]} {
        if {![info exists fieldsInfo] && ![string equal [parseSDF $sdfs_files($chip_mc_ver)] "OK"]} {
            puts "Can't find $sdfs_files($chip_mc_ver)."
            puts "After fixing the problem. re-try to set the profile and new trigger."
            return BAD
        }
        set field [lindex $fieldDesc 1]
        if {![info exists fieldsInfo($field)]} {
            puts "The field \"$field\" you ask for in line $line_number doesn't exist in the sdf file."
            puts "Recheck the line or switch the sdf file (you have to re-source the script afterward)"
            return BAD
        }
        regexp {(\d+):(\d+)} $fieldsInfo($field) fend fend fstart
        set offset $fstart
        set size [expr {$fend-$fstart+1}]
        set roffset 0
        for {set i 2} {$i<[llength $fieldDesc]} {incr i} {
            if {[string equal [lindex $fieldDesc $i] OFF:]} {
                incr i
                if {![regexp {(\d+|0x[A-Fa-f0-9]+)} [lindex $fieldDesc $i] roffset]} {
                    puts "The <offset> must be a number in decimal or hexa format (0x...). Recheck line $line_number."
                    return BAD
                }
                if {$roffset>=$size} {
                    puts "You can't give relative offset for a field which is bigger the the field size."
                    puts "Recheck line $line_number."
                    return BAD
                }
                continue
            }
            if {[string equal [lindex $fieldDesc $i] SIZE:]} {
                incr i
                if {![regexp {(\d+|0x[A-Fa-f0-9]+)} [lindex $fieldDesc $i] rsize]} {
                    puts "The <size> must be a number in decimal or hexa format (0x...). Recheck line $line_number."
                    return BAD
                }
                continue
            }
            puts "Unknown arguments in the field descriptor in line $line_number. recheck argument $i: [lindex $fieldDesc $i]."
            return BAD
        }
        
        if {[info exists rsize]} {
            if {[expr {$rsize+$roffset}]>$size} {
                puts "You give a <size> which with the relative offset (default 0) is bigger than the field's size."
                puts "Recheck line $line_number."
                return BAD
            }
            set size $rsize
        } else {
            incr size -$roffset
        }
        set offset [expr {$offset+$roffset}]
    } elseif {[string equal $currField OFF:]} {
        if {![regexp {(\d+|0x[A-Fa-f0-9]+)} [lindex $fieldDesc 1] offset]} {
            puts "The <offset> must be a number in decimal or hexa format (0x...). Recheck line $line_number."
            return BAD
        }
        if {![string equal [lindex $fieldDesc 2] SIZE:]} {
            puts "When working without FIELD: in the field descriptor, you must give \"SIZE: <size>\" after the offset."
            puts "Recheck line $line_number."
            return BAD
        }
        if {![regexp {(\d+|0x[A-Fa-f0-9]+)} [lindex $fieldDesc 3] size]} {
            puts "The <size> must be a number in decimal or hexa format (0x...). Recheck line $line_number."
            return BAD
        }
        if {[llength $fieldDesc]>4} {
            puts "unknown arguments in field descriptor \"[lrange $fieldDesc 4 end]\". Recheck line $line_number."
            return BAD
        }
    } else {
        puts "Wrong syntax on line $line_number."
        puts {The field descriptor must be either "FIELD: <field name> [OFF: <relative offset> -opt] [SIZE: <size> -opt]" or "OFF: <absolute offset> SIZE: <size>".}
        puts "Recieved $fieldDesc."
        return BAD
    }
    return OK
}

# This function is used for kernelDebug: It recieves a configuration file (profile_file) and the names of 3 outputs
# and genereates the 3 files:
#.h file with typedefs of structs
#.c file with the arrays of registers to write for each configuration
#.txt file - meta data for parsing kernel debug data.
proc dbgTools::kernelDebug {profile_file out_h_file out_c_file out_txt_file} {
	set t_stamp [format 0x%x [clock seconds]]
	set src_file [open $profile_file r]
	set handling_chip 0
	set current_chip_num 0
	set current_chip_name 0
	set all_used_chips ""
	set profile_counter_per_chip 0
	variable dump_counter_info
	while {![eof $src_file]} {
	set next_line [gets $src_file]
	#Remarks
	if {[regexp {^#} $next_line]} {
		continue
	}
	# go over all the lines and activate all the profiles for all chips
	if {$handling_chip} {
		if {[regexp "^END E(.)+_CONFIGS$" $next_line matched chip_num]} {
			set handling_chip 0
			set profile_counter_for_[set current_chip_name] $profile_counter_per_chip
			array set counter_info_for_[set current_chip_name] {}
			array set counter_info_for_[set current_chip_name] [array get dump_counter_info]
			array set dump_counter_info {}
			set register_string_[set current_chip_name] [getDumpString]
			clearDumpString
		} else {
			if {[regexp "setp" $next_line]} {
			# ignore everything before setp
				set len [string length $next_line]
				set setp_loc [string first "setp" $next_line]
				set next_line [string range $next_line $setp_loc [expr {$len - 1}]]
				eval "[set next_line] -dumph $profile_counter_per_chip -pcidumph"
				incr profile_counter_per_chip
			} else {
				puts "Error line: $next_line"
			}
		}
	
	} else {
		if {[regexp "^E(.+)_CONFIGS:$" $next_line matched chip_num]} {
			set current_chip_name "E[set chip_num]"
			if {[string equal $chip_num 1h] || [string equal $chip_num 1H]} {
				set current_chip_num 1.5
			} else {
				set current_chip_num $chip_num
			}
			set handling_chip 1
			changeChip $current_chip_num
			clearWriteByNameCounter
			clearDumpString
			set all_used_chips "[set all_used_chips] $current_chip_name"
			set profile_counter_per_chip 0
		}
	}
	}
	close $src_file

	set tgt_h [open $out_h_file w]
	set tgt_txt [open $out_txt_file w]
	set tgt_c [open $out_c_file w]
	puts $tgt_h "\#ifndef _DBUS_ST_H"
	puts $tgt_h "\#define _DBUS_ST_H\n"
	puts $tgt_h "\#include \"bcmtype.h\"\n\n"
	puts $tgt_h "typedef struct _dbg_reg_write"
	puts $tgt_h "\{"
	puts $tgt_h "\tu32_t addr;"
	puts $tgt_h "\tu32_t value;"
	puts $tgt_h "\} dbg_reg_write;\n\n"
	
	puts $tgt_h "typedef struct _dbg_register_set_write"
	puts $tgt_h "\{"
	puts $tgt_h "\tdbg_reg_write *p_reg;"
	puts $tgt_h "\tu32_t count;"
	puts $tgt_h "\}	dbg_register_set_write;\n\n"

	puts $tgt_h "typedef struct _dbg_reg_group"
	puts $tgt_h "\{"
	puts $tgt_h "\tdbg_register_set_write dbgCfg;"
	puts $tgt_h "\tdbg_register_set_write dbgOn;"
	puts $tgt_h "\tdbg_register_set_write dbgOff;"
	puts $tgt_h "\tdbg_register_set_write dbgFlush;"
	puts $tgt_h "\} dbg_reg_group;\n\n"
	
	puts $tgt_h "typedef struct _dbg_driver_end_read_regs"
	puts $tgt_h "\{"
	puts $tgt_h "\tu32_t dbg_block_on;"
	puts $tgt_h "\tu32_t intr_buffer_read_ptr;"
	puts $tgt_h "\tu32_t intr_buffer_wr_ptr;"
	puts $tgt_h "\tu32_t ext_buffer_wr_ptr_lsb;"
	puts $tgt_h "\tu32_t ext_buffer_wr_ptr_msb;"
	puts $tgt_h "\tu32_t dbg_ovl_on_ext_buffer;"
	puts $tgt_h "\tu32_t dbg_wrap_ext;"
	puts $tgt_h "\} dbg_driver_end_read_regs;\n\n"

	puts $tgt_h "typedef struct _dbg_driver_fill_regs"
	puts $tgt_h "\{"
	puts $tgt_h "\tu32_t ext_buffer_start_addr_lsb;"
	puts $tgt_h "\tu32_t ext_buffer_start_addr_msb;"
	puts $tgt_h "\tu32_t ext_buffer_size; // in 256 byte blocks"
	puts $tgt_h "\tu32_t pci_func_num; // not for E1 "
	puts $tgt_h "\} dbg_driver_fill_regs;\n\n"
	
	puts $tgt_h "typedef struct _dbg_general_info"
	puts $tgt_h "\{"
	puts $tgt_h "\tu32_t timestamp;"
	puts $tgt_h "\tu32_t chip_num;"
	puts $tgt_h "\tu32_t chosen_config;"
	puts $tgt_h "\tu32_t path_num;"
	puts $tgt_h "\} dbg_general_info;\n\n"
	
	puts $tgt_h "typedef struct _dbg_dump_hdr"
	puts $tgt_h "\{"
	puts $tgt_h "\tu32_t header_length; // will hold sizeof(dbg_dump_hdr)"
	puts $tgt_h "\tdbg_general_info info;"
	puts $tgt_h "\tdbg_driver_fill_regs driver_filled_info;"
	puts $tgt_h "\tdbg_driver_end_read_regs driver_read_regs;"
	puts $tgt_h "\} dbg_dump_hdr;\n\n"
	
	
	puts $tgt_h "\#define DBG_E1\t0"
	puts $tgt_h "\#define DBG_E1H\t1"
	puts $tgt_h "\#define DBG_E2\t2"
	puts $tgt_h "\#define DBG_E3\t4"
	
	
	
	array set CHIP_TO_NUM { E1 0 E1H 1 E2 2 E3 4}
	
	puts $tgt_txt "TIMESTAMP [expr $t_stamp]"
	puts $tgt_c "\#include \"$out_h_file\"\n\n"
	
	
	foreach chip_name $all_used_chips {
			if {[string equal E1 $chip_name]} {
				set pci_func_num_reg 0
			} else {
				set pci_func_num_reg [reg_offset -n DBG_REG_PCI_FUNC_NUM]
			}
			puts "Handling chip $chip_name"
			puts "Number Profiles:  [set profile_counter_for_[set chip_name]]"
			set all_regs_name "dbg_bus_all_regs_[set chip_name]"
			array set cur_info [array get counter_info_for_[set chip_name]]
			puts $tgt_h "extern dbg_general_info dbg_bus_general_info_[set chip_name];"
			puts $tgt_c "dbg_general_info dbg_bus_general_info_[set chip_name] = \{ $t_stamp , $CHIP_TO_NUM($chip_name) , 0 \};\n"
			puts $tgt_h "extern dbg_driver_fill_regs dbg_bus_driver_fill_regs_[set chip_name];"
			puts $tgt_c "dbg_driver_fill_regs dbg_bus_driver_fill_regs_[set chip_name] = \{ [get_start_offset [reg_offset -n DBG_REG_PCI_EXT_BUFFER_STRT_ADDR_LSB]] , [get_start_offset [reg_offset -n DBG_REG_PCI_EXT_BUFFER_STRT_ADDR_MSB]] , [get_start_offset [reg_offset -n DBG_REG_PCI_EXT_BUFFER_SIZE]] , $pci_func_num_reg\};\n"
			
			set ext_buff_wr_ptr_addr_lsb [get_start_offset [reg_offset -n DBG_REG_EXT_BUFFER_WR_PTR]]
			set ext_buff_wr_ptr_addr_msb [format "0x%.8x" [expr {$ext_buff_wr_ptr_addr_lsb + 4}]]
			puts $tgt_h "extern dbg_driver_end_read_regs dbg_bus_driver_end_read_regs_[set chip_name];"
			puts $tgt_c "dbg_driver_end_read_regs dbg_bus_driver_end_read_regs_[set chip_name] = \n\t\{ [get_start_offset [reg_offset -n DBG_REG_DBG_BLOCK_ON]] , [get_start_offset [reg_offset -n DBG_REG_INTR_BUFFER_RD_PTR]] , [get_start_offset [reg_offset -n DBG_REG_INTR_BUFFER_WR_PTR]] ,\n\t $ext_buff_wr_ptr_addr_lsb , $ext_buff_wr_ptr_addr_msb , [get_start_offset [reg_offset -n DBG_REG_OVL_ON_EXT_BUFFER]] ,\n\t [get_start_offset [reg_offset -n DBG_REG_WRAP_ON_EXT_BUFFER]]\};\n"
			
			puts $tgt_h "extern dbg_reg_write [set all_regs_name]\[\];"
			puts $tgt_c "dbg_reg_write [set all_regs_name]\[\] = \{ [set register_string_[set chip_name]] \};\n"
			puts $tgt_h "extern dbg_reg_group dbg_bus_configs_[set chip_name]\[[set profile_counter_for_[set chip_name]]\];\n"
			puts $tgt_c "dbg_reg_group dbg_bus_configs_[set chip_name]\[[set profile_counter_for_[set chip_name]]\] = \{"
			for {set i 0} {$i < [set profile_counter_for_[set chip_name]]} {incr i} {
				set addition ","
				if {$i == [expr {[set profile_counter_for_[set chip_name]] - 1}]} {
					set addition ""
				}
				puts $tgt_c "\t\{"
				puts $tgt_c "\t\t\{ [set all_regs_name] + [set cur_info([set i],cfg,start)] , [set cur_info([set i],cfg,count)] \},"
				puts $tgt_c "\t\t\{ [set all_regs_name] + [set cur_info([set i],dbgOn,start)] , [set cur_info([set i],dbgOn,count)] \},"
				puts $tgt_c "\t\t\{ [set all_regs_name] + [set cur_info([set i],dbgOff,start)] , [set cur_info([set i],dbgOff,count)] \},"
				puts $tgt_c "\t\t\{ [set all_regs_name] + [set cur_info([set i],flush,start)] , [set cur_info([set i],flush,count)] \}"
				puts $tgt_c "\t\}[set addition]"
				puts $tgt_txt "[set chip_name],[set i] [set cur_info([set i],profile_name)]"
			}
			puts $tgt_c "\};\n\n\n"
	}
	puts $tgt_h "\n\n\#endif //_DBUS_ST_H"
	close $tgt_h
	close $tgt_c
	close $tgt_txt
}


#this proceduer recieves:
#mode -the framing mode
# _setInfo - name of the array to puts the set data
# type - FILTER OR TRIGGER
#tdm64block - whetever the block working on is tdm64
#_line - name of the variable holding current line
#_line_number - name of the variable holding current line's number
#lines - the actual trigger lines
#_index - name of variable holding the line index in lines
#ml - the length of the message
proc dbgTools::parseSet {mode _setInfo type tdm64block _line _line_number lines _index ml} {
    
    variable OPS
    variable fieldsInfo
    upvar $_setInfo setInfo
    upvar $_line line
    upvar $_line_number line_number
    upvar $_index i
    
    set USED {129 129}

    set constraint 0
    # The loop passes each constraint, verify its syntax.
    # Saves it in set_info as cycle, operator, mask or range, frame and frame_mask, must or optional, value to compare,
    # using indirect or not and whether cyclic operator.
    # The loop also try to merge equals constraints in the same cycle.
    # split fields that cross 32 bit to 2.
    # And make sure there are at most 4 constraints.
    while {[regexp {^\s*(M|O|V)} $line]} {
        set indUse 0
        if {[regexp {^V FIC: ([^\s]+)\s*$} $line field field]} {
            if {![info exists fieldsInfo] && ![string equal [parseSDF $sdfs_files($chip_mc_ver)] "OK"]} {
                puts "Can't find $sdfs_files($chip_mc_ver)."
                puts "After fixing the problem. re-try to set the profile and new trigger."
                return BAD
            }
            if {![info exists fieldsInfo($field)]} {
                puts "The fic \"$field\" you ask for in line $line_number doesn't exist in the sdf file."
                puts "Recheck the line or switch the sdf file (you have to re-source the script afterward)"
                return BAD
            }
            set must 1
            set val 0
            set op =
            regexp {(\d+):(\d+)} $fieldsInfo($field) fend fend fstart
            #THE Fic spans for fend-fstart+1 bits, however, a trailer is appended and we want to check the
            # appropriate bit in the header
            # division and multiplication by 128 is for getting the result as a multiply (4*32)
            set offset [expr {int(ceil(($fend-$fstart+1)/128)*128)+28}]
            set size 4
            set fr 1
        } elseif {![regexp {^\s*(M|O([1-9])?)\s+VAL:\s+([iI][01]|\d+|0[xX][A-Fa-f0-9]+)\s+(=|(>|<|>=|<=|=<|=>)c?)\s+(.+?)(\s+F:\s+([01]))?\s*$} \
         $line must must dc val op dc fieldDesc fr fr]} {
            puts "Wrong syntax on line $line_number.\nExpecting \"(M|O?\[1-9\]?) VAL: <num> <operator> <field descriptor> ?F: <0|1>?\""
            puts "Or \"V FIC: <field name>\", recieved $line\n"
            return "BAD"
        } else {
            if {![string equal [parseFieldDesc $fieldDesc offset size $line_number] OK]} {
                return BAD
            }
        }
        if {[string length $must]==2} {
            if {![string equal $op =]} {
                puts "Merging optional constraints is possible only when working with eqaul operator.\n"
                puts "Recheck line $line_number."
                return BAD
            }
            set must [expr [string index $must 1]+1]
        } else {
            set must [string equal $must M]
        }

        if {$mode==1} {
            set cycle [expr {($offset/64)*2+int($offset/32)%2}]
        } else {
            set cycle [expr {($offset/32)*2+$tdm64block}]
        }
        set offset [expr {int($offset)%32}]
        
        #### limit the top allowed cycle in the filter
	if {[string equal $type FILTER]} {
	    if {$cycle>7} {
		puts "The filter's conditions can only be used on the first four cycles of the messages."
		return BAD
	    }
	}
        #cycle is saved as (num_cycles << 2) + (0 or 1)
        if {$cycle>=[expr {$ml*2}]} {
            puts "You requested to compare data from an offset which doesn't exist in the message length you defined."
            puts "Recheck line $line_number."
            return BAD
        }
        
        if {![string match [string tolower [string index $val 0]] "i"]} {
            if {$val>=[expr {pow(2,$size)}]} {
                puts "The value is bigger than the field can hold.\nrecheck line $line_number\n"
                return "BAD"
            }
            if {[expr {$offset+$size}]>64} {
                puts "Values above 64 bit are not supported by the script"
                return "BAD"
            }
            if {[expr {$offset+$size}]>32} {
                #cycle is saved as (num_cycles << 2) + (0 or 1)
                # so in mode 1 (64 bit), adding 1 will change the lsb (upper or lower) and will possibly overflow
                # to correctly change to real cycle number.
                # and you can ponder yourself what will happen in tdm
                
                set secondC [expr {$cycle+2-($mode==1)}] 
                set secondV [expr {$val>>(32-$offset)}]
                set secondS [expr {$size-(32-$offset)}]
                set val [expr {$val%int(pow(2,32-$offset))}]
                set size [expr {32-$offset}]
            }
        } else {
            if {[expr {$offset+$size}]>32} {
                puts "when using Indirect values the compared field can't cross 32 bit structures."
                return "BAD"
            }
            set indUse 1
            set val [string index $val 1]
            if {[lindex $USED $val]>$cycle} {
                set USED [lreplace $USED $val $val $cycle]
            }
        
        }
        
        while {$size} {
            set newConstraint 1
            if {![info exists setInfo($cycle)]} {
                set setInfo($cycle) 0
            }
            set index $setInfo($cycle)
            set setInfo($cycle,ind$index) $indUse		
            if {[string equal $op "="]} {
                if {$size==32} {
                    set mask 0
                } else {
                    set mask [expr {~(int(pow(2,$size )-1)<<$offset)}]
                }
                
                set val [expr {$val<<$offset}]
                # The loop passes all the constraint who uses equals operator and try to merge the current constraint with
                # one of them if possible
                # The merging is done only for Must constraints
                for {set k 0} {$k<$setInfo($cycle) && $must} {incr k} {
                    
                    if {[info exists setInfo($cycle,mask$k)]} {
                        #mask will not show bits in places where the mask is 1
                        if {$setInfo($cycle,must$k)==$must} {
                            if {([expr {$setInfo($cycle,mask$k)|$mask}]==0xffffffff &&
                                !$indUse) || ($indUse && $setInfo($cycle,ind$k) && $val==$setInfo($cycle,val$k))} {
                                if {($fr=={} && $setInfo($cycle,fr_mask$k)==1) ||
                                    ($fr==$setInfo($cycle,fr$k) && $setInfo($cycle,fr_mask$k)==0)} {
    
                                    set mask [expr {$setInfo($cycle,mask$k)&$mask}]
                                    set setInfo($cycle,val$k) [expr {$setInfo($cycle,val$k)|$val}]
                                    
                                    set index $k
                                    set k $setInfo($cycle)
                                    set newConstraint 0
                                }
                            } 
                            if {$must>1 && $newConstraint} {
                                puts "The optional group [expr $must-1] you asked to merge is impossible."
                                puts "Recheck line $line_number."
                                return BAD
                            }
                        }
                    }
                    
                }
                
                set setInfo($cycle,mask$index) $mask
                set setInfo($cycle,cyclic$index) 0
                if {$fr==""} {
                    set setInfo($cycle,fr_mask$index) 1
                    set setInfo($cycle,fr$index) 0
                } else {
                    if {$indUse} {
                        puts "You can't set the frame when working with an indirect value."
                        puts "Recheck line $line_number."
                        return "BAD"
                    }
                    set setInfo($cycle,fr_mask$index) 0
                    set setInfo($cycle,fr$index) $fr
                }
            } else {
                if {$fr!=""} {
                    puts "Frame can be compare only when using equal, recheck $line_number."
                    return "BAD"
                }
                set setInfo($cycle,range$index) [expr {$size-1+($offset<<5)}]
                if {[string index $op end]=="c"} {
                    if {$size!=32} {
                        puts "When working with cyclic comparsion, you can only work with 32 bits fields."
                        puts "Recheck line $line_number."
                        return BAD
                    }
                    set setInfo($cycle,cyclic$index) 1
                    set op [string range $op 0 end-1]
                } else {
                    set setInfo($cycle,cyclic$index) 0
                }
            }
            if {$newConstraint} {
                
                incr constraint
                incr setInfo($cycle)
                set setInfo($cycle,val$index) $val
                set setInfo($cycle,op$index) $OPS($op)
                set setInfo($cycle,must$index) $must
            }
            
            if {$constraint==5} {
                puts "You have defined too many constraints for one set.\nUpto line $line_number, the following cycles were used:"
                puts "For equals:"
                foreach used [array names setInfo *,mask*] {
                    regexp "(\\d+)" $used ucycle ucycle
                    puts "cycle: $ucycle"
                }
                puts "For others:"
                foreach used [array names setInfo *,range*] {
                    regexp "(\\d+)" $used ucycle ucycle
                    puts "cycle: $ucycle"
                }
                return "BAD"
            }
            if {[info exists secondV]} {
                set size $secondS
                set val $secondV
                set cycle $secondC
				# and don't forget, no longer need offset
				set offset 0
                unset secondV
            } else {
                set size 0
            }
            
        }
        incr i
        getNextLine $lines $i line line_number
    }
    # retern the lowest cycle the indirect have been used.
    return $USED
}
# this procedure recieves info about a set and configure it
# _setInfo - name of the array to puts the set data
#type - TRIGGER_STATE_SET or FILTER
# state and d_set relevant only for TRIGGER
proc dbgTools::cfg_set {_setInfo type {state 0} {d_set 0}} {
    upvar $_setInfo setInfo
    
    # configure set
    set currCon [expr {$state*8+$d_set*4}]
    set indrct INDRCT
    
    # Since the registers name convention in the trigger and filter has differences.
    if {[string equal $type FILTER]} {
        set indrct INDIRECT
    }
    # configure equal constraints
    foreach used [array names setInfo *,mask*] {
        regexp "(\\d+),mask(\\d+)" $used off off index
                
        if {$setInfo($off,ind$index)} {
            writeByName DBG_REG_[set type]_CNSTR_[set indrct]_$currCon [expr {$setInfo($off,val$index)+1}]
        } else {
            writeByName DBG_REG_[set type]_CNSTR_DATA_$currCon $setInfo($off,val$index)
        }
        
        
        writeByName DBG_REG_[set type]_CNSTR_DATA_MASK_$currCon $setInfo($off,mask$index)
        writeByName DBG_REG_[set type]_CNSTR_FRAME_$currCon $setInfo($off,fr$index)
        writeByName DBG_REG_[set type]_CNSTR_FRAME_MASK_$currCon $setInfo($off,fr_mask$index)
        writeByName DBG_REG_[set type]_CNSTR_OPRTN_$currCon $setInfo($off,op$index)
        
        writeByName DBG_REG_[set type]_CNSTR_MUST_$currCon [expr $setInfo($off,must$index)==1]
        writeByName DBG_REG_[set type]_CNSTR_OFFSET_$currCon $off
        incr currCon 
    }
    # configure others constraints
    foreach used [array names setInfo *,range*] {
        regexp "(\\d+),range(\\d+)" $used off off index
        
        if {$setInfo($off,ind$index)} {
            writeByName DBG_REG_[set type]_CNSTR_[set indrct]_$currCon [expr {[string index $setInfo($off,val$index) 0]+1}]
        } else {
            writeByName DBG_REG_[set type]_CNSTR_DATA_$currCon $setInfo($off,val$index)
        }
        
        writeByName DBG_REG_[set type]_CNSTR_OPRTN_$currCon $setInfo($off,op$index)
        writeByName DBG_REG_[set type]_CNSTR_CYCLIC_$currCon $setInfo($off,cyclic$index) 
        writeByName DBG_REG_[set type]_CNSTR_MUST_$currCon $setInfo($off,must$index)
        writeByName DBG_REG_[set type]_CNSTR_OFFSET_$currCon $off
        writeByName DBG_REG_[set type]_CNSTR_RANGE_$currCon $setInfo($off,range$index)
        incr currCon 
    }
    
}

# subroutine for flushing the last data into the network, afterward the debug block must be resetted inorder to continue
# (which mean resetting a profile).
proc dbgTools::flushNet {} {
 
    variable DBG_STATUS
    variable onRegs
    
    foreach {reg val} $onRegs {
        if {![regexp (^0x|^\\d+$) $reg]} {
            set val [reset_value -n $reg]
        } else {
            set val 0
        }
        writeByName $reg $val
    }
    writeByName DBG_REG_CLIENT_ENABLE 0
    after 100
    set DBG_STATUS FLUSHED
    writeByName DBG_REG_CPU_TIMEOUT 1
}

# subroutine for clearing the data from all configurated blocks.
# used upon errors in the profiles (syntax errors or logical errors) and before setting a new profile.
proc dbgTools::cleanCfg {} {
 
    variable usedBlocks
    variable usedRegs
    variable blocksPD
    variable DUMP
    variable dumpFile
	variable dumph_current_config_number
	variable dump_counter_info
    if {$DUMP} {
		if {$DUMP < 4} {
			puts $dumpFile "# Starting cleaning stage"
		} else {
			set dump_counter_info([set dumph_current_config_number],cfg,start) [getWriteByNameCounter]
		}
    }
    foreach blk [array names usedBlocks] {
        if {![regexp "STORM" $blk]} {
          cfg_block $blk 0 0 0 ""
        } else {
            clean_storms [string index $blk 0]
        } 
    }
    foreach reg [array names usedRegs] {
        writeByName $reg $usedRegs($reg)
    }
    cfg_header [format "%.28d" 0] 1
    cfg_port 0
	cfg_path 0
    cfg_pace 1
    array unset usedBlocks
    array set usedBlocks {}
    if {$DUMP} {
		if {$DUMP < 4} {
			puts $dumpFile "# Finished cleaning stage\n"
		}
    }
        
}

# subroutine for leaving dump mode
proc dbgTools::closeDump {} {
    variable dumpFile
    variable DUMP
    
    if {$DUMP} {
        set DUMP 0
        if {$dumpFile !="" && ![string equal $dumpFile closed]} {
            close $dumpFile
        }
        set dumpFile closed
    }
}

# subroutine for clearing all blocks.
# used upon loading.
proc dbgTools::cleanOldCfg {} {
    
    variable blocksPD

    foreach blk [array names blocksPD] {
        if {[regexp "STORM" $blk]} {
            clean_storms [string index $blk 0]
        } else {cfg_block $blk 0 0 0 ""}
    }

    cfg_port 0
	cfg_path 0
	

    
}
# subroutine for getting all structs meta data.
proc dbgTools::parseSDF {sdf_file} {
    variable fieldsInfo
    
    # opening input file
    if {![file exists $sdf_file]} {
	puts "Can't open sdf file $sdf_file. It's doesn't exists.\nThe script won't run without it"
	return -1
    }
    set SDF [open $sdf_file r]
    while {![eof $SDF]} {
        set line [gets $SDF]
        if {[llength $line]!=2} {
            continue
        }
        set fieldsInfo([lindex $line 0]) [lindex $line 1]
    }
    close $SDF
    return OK
}

# subroutine for getting all physical design blocks and their sub-blocks.
proc dbgTools::parsePDblocks {{pd_input}} {

    variable blocksPD
    variable PDs
	variable mapPDToBlocks
	variable allBlocks
    
    # opening input file
    if {![file exists $pd_input]} {
	puts "Can't open pd input file $pd_input. It's doesn't exists.\n The script won't run without it"
	return -1
    }
    set pd_name {}
    set pd_num {}
	array unset mapPDToBlocks
	array set mapPDToBlocks {}
	set allBlocks {}
    set pdFile [open $pd_input r]
    while {![eof $pdFile]} {
        set tmpStr [gets $pdFile]
        
        if {[regexp "^PD(\\d): (\[a-zA-z0-9\]+)" $tmpStr tmpStr pd_num pd_name]} {
            set PDs($pd_num) [string toupper $pd_name]
			set mapPDToBlocks($pd_num) {}
        } elseif {[regexp "^(\[a-zA-Z0-9_\]+)" $tmpStr tmpStr tmpStr]} {
            set blocksPD([string toupper $tmpStr]) $pd_num
			lappend mapPDToBlocks($pd_num) [string toupper $tmpStr]
			lappend allBlocks [string toupper $tmpStr]
        } 
    }
    close $pdFile
    return OK
}
# subroutine for Static Debug. It returns an array defining a profile for a specified line.
proc dbgTools::getStaticLineProfile {profile_name line_name block_name line_size is_tdm res_variable} {
    upvar 1 $res_variable res
    
    set res "PROFILE: $profile_name"
    variable add_to_block_name ""
    variable tdm_string "64BIT"
    if {$is_tdm == 1} {
        set tdm_string "TDM64"
        set add_to_block_name "64:"
    } else {
        if {$line_size == 24} {
            set tdm_string "24BIT"
        }
    }
    
    set res "[set res]\nMODE: $tdm_string"
    set res "[set res]\nBLOCKS:"
    set res "[set res]\n[set add_to_block_name][set block_name] [set line_name]"
    set res "[set res]\nENDBLOCKS"
    if {$is_tdm == 1} {
        set res "[set res]\nCALENDAR:"
        for {set i 0} {$i < 7} {incr i} {
            set res "[set res]\nSLOT [set block_name]"
        }
        set res "[set res]\nENDCALENDAR"
    }
    set res "[set res]\nENDPROFILE"
    return $res
}

proc dbgTools::GetCurrentPathNum {} {
	variable current_path_num
	variable dbg_no_pci
	variable ME_register_address
	if {$dbg_no_pci} {
		return $current_path_num
	}
	
	#otherwise, pci
	if {![dbgE2] && ![dbgE3]} {
        return 0
    } else {
	    set me_reg [reg read $ME_register_address]
        set abs_pf_num [expr {$me_reg >> 16}]
        return [expr {$abs_pf_num & 1}]
    }
}
# subrouting for describing which path and port were chosen for debugging with network
proc dbgTools::PrintNetDebugInfo {port} {
	variable other_path
	set current_path [GetCurrentPathNum]
	if {$other_path} {
		set chosen_path [expr {1 - $current_path}]
	} else {
		set chosen_path $current_path
	}
	set path_addition ""
	if {[dbgE2] || [dbgE3]} {
		set path_addition "path $chosen_path and "
	}
	puts "You have chosen to dump the debug data to network to $path_addition port $port"
}

# subrouting for describing which path and port were chosen for debugging with network
proc dbgTools::PrintDebugInfo {} {
	if {[dbgE2] || [dbgE3]} {
		set current_path [GetCurrentPathNum]
		puts "You have chosen to record debug data from path $current_path"
	}
}
# subroutine for changing the path when working with UART
proc dbgTools::UARTChangePath {new_path_num} {
	variable current_path_num
	set current_path_num $new_path_num
    set config_reg [reg_offset -n DBU_REG_DBU_VFID_CFG]
    set config_value [expr {(1 << 31) | ($new_path_num << 20)}]
    reg write $config_reg $config_value
    after 200
}

# subroutine for adding help
proc dbgTools::addHelpsetProfile {} {
    set setProfileHelp "setProfile <profiles file> <profile name> <args> \n"
    append setProfileHelp "configurate the requested profile into the debug bus.\n"
    append setProfileHelp "the possible arguments are:\n-network h<header number> - header with that number will be taken from the profiles file\n"
    append setProfileHelp "-pci <memory size in KB) - will set the debug bus to output to the host memory.\nThe data size will be up to the defined size\n"
    append setProfileHelp "-port0 - output to the network will be sent on port 0.\n-wrap - set the internal buffer in cyclic mode.\n"
	append setProfileHelp "-port1 - output to the network will be sent on port1.\n-otherpath - output to the network will be sent on other path.\n"
    append setProfileHelp "-dump <filename> - will write the needed registers' configuration to a file, instead of configuring the chip\n"
    append setProfileHelp "-t start|end|filter <triger_name> - will set the debug block to work with that trigger.\n"
    append setProfileHelp "-grant - reduced chances the storm will get into waitp, but can lead to old data lost.\n"
    append setProfileHelp "E1H options:\n"
    append setProfileHelp "-nt <window> <new trigger name> - will configure the Debug block new trigger to the requested\n"
    append setProfileHelp "trigger, with the requested window.\n"
    append setProfileHelp "-f <pre|con|post> <filter name> - will configure the Debug block filter machine to requested filter\n"
    append setProfileHelp {-sf[TUCX], -eid[TUCX], -cid[TUCX] - are used to configure the semi filter - for usage check the user manual}
	append setProfileHelp "\n"
	append setProfileHelp {-eidonly[TUCX] <event id> <event mask> is a shortcut. It can be used instead of writing the entire -sf[TUCX] -cid con @ -eid[TUCX] <event> <event mask>}
    append setProfileHelp "\nNOTE: You can either use network or pci, not both.\n"
    append setProfileHelp "NOTE: Pci mode is implemented only in DOS / Linux.\n"
    add_help setProfile $setProfileHelp
    
    set dbgEndHelp  "dbgEnd <output file>\n"
    append dbgEndHelp "Will flush the last data, and if working with the buffer or with the ram, will dump the internal buffer/Ram into a file, with the extension .buf/.pci\n"
    add_help dbgEnd $dbgEndHelp
    
    puts "In order to configurate the debug bus you need to write:"
    puts "setProfile <profiles file> <profile name> <args>"
    puts "(setp <profile name> <args>, if you use profiles.dbg)"
    puts "The possible arguments are:"
    puts "-network h<header number> | -port0 | -port1 | -otherpath|-pci <memory size in KB> | -grant"
    puts "-port0 - output to the network will be sent on port 0.\n-wrap - set the internal buffer in cyclic mode."
	puts "-port1 - output to the network will be sent on port1.\n"
	puts "-otherpath - output to the network will be sent on other path.\n"
    puts "-pci <memory size in KB) - will set the debug bus to output to the host memory."
    puts "-dump <filename> - will write the needed registers' configuration to a file, instead of configuring the chip"
    puts "-t start|end|filter <triger_name> - will set the debug block to work with that trigger."
    puts "-grant - reduced chances the storm will get into waitp, but could lead to old data lost."
    puts "The data size will be up to the defined size."
    puts "NOTE: You can either use network or pci, not both."
    puts "NOTE: pci is implemented only in DOS / Linux."
    puts "e.g. setProfile profiles.dbg EXAMPLE1 (-network h1 | -pci 1000)"
    puts "You should start the debug bus with dbgOn"
    puts "You should finishing working with dbgEnd <filename>"
    puts "If working with the internal buffer or with the pci,"
    puts "the data will be dump into the file (The file extension will be .buf | .pci)."
    puts "If working with the network, you should make sure you catch the data on the\nother side."
    puts "and dbgEnd will flush the last packet to the net." 
}



#### MAIN ########
namespace eval dbgTools {
    # adding the interface which combine registers name to their addresses
    if {![array exist ::regsOffset]} {
        namespace eval :: {
            if {[parse_rdf -f $dbgTools::rdfs_files($dbgTools::chip_ver)]!=0} {
                error "Problem parsing rdf file, make sure $dbgTools::rdfs_files($dbgTools::chip_ver) exist"
            }
        }
    }    
    # checking the microcode version in E1H
    set pram_addr [get_start_offset [reg_offset -n XSEM_REG_PRAM]]
    reg read $pram_addr
    set flags [reg read [expr {$pram_addr+4}]]
    set chip_mc_ver $PRAM_FW_FLAGS([expr {($flags>>4)&3}])
    
    # getting the pd blocks and their sub-blocks, cleaning old configuration that might exists and adding help/
    if {[parsePDblocks [format {%s/pdBlocks.txt} $csv_folder]]=="OK"} {
           addHelpsetProfile
            if {![info exists ::dbgTools_no_driver]} {
                cleanOldCfg
            } else {
                puts "You choose to work in advanced mode with no driver."
                puts "Make sure all the blocks you are working with are initilized before setting a profile"
                puts "Otherwise, the computer would halt"
                unset dbgTools_no_driver
            }
            namespace eval :: {
                namespace import -force dbgTools::*
            } 
    } else {
        puts {After fixing the problem, re-source the script.}
        namespace eval :: {
            namespace delete dbgTools
        }
    }
	
	#Adding new remark for Linux users - Linux has Bnx2x drivers and cannot use the standard driver load and driver state
	if {[string equal $::tcl_platform(os) "Linux"]} {
		puts "In Linux, you have to make sure that the driver is running. From now on, the script asuumes it's on - NO CHECK IS BEING PERFORMED!!!!"
	}
}
