#!/bin/sh
# -*- tcl -*-
# The next line is executed by /bin/sh, but not tcl \

source regutils.tcl
parse_rdf -f everest.rdf

# subroutine for getting the start offset of a register after reg_offset
proc get_start_offset {{offset}}  {
    
    # Parse out the starting offset
    return  [lindex [split $offset "-"] 0]                 
}

# subroutine for write to registers by name and not by address
proc  writeByName {reg_name data} {
    if {[regexp (^0x|^\\d+$) $reg_name]} {
        set addr $reg_name
    } else {
        set addr [get_start_offset [reg_offset -n $reg_name]]
    }
    reg write $addr $data
}

proc  disEnFic {value} {
    foreach letter {X C T U} {
        writeByName "[set letter]SEM_REGISTERS_FIC0_DISABLE" $value
        writeByName "[set letter]SEM_REGISTERS_FIC1_DISABLE" $value
    }

}

proc  dumpPram {pram_file int_file letter} {
    
    set i 0
    set addr [get_start_offset [reg_offset -n "[set letter]SEM_REGISTERS_PRAM"]]
    while {$i<[expr pow(2,16)] && ! [eof $pram_file]} {
        set line [gets $pram_file]
        if {$line==""} {
            continue
        }
        writeByName $addr "0x$line"
        incr addr 4
        incr i
    }
    puts "[set letter]STORM Pram Switching is done..."
    set i 0
    set addr [get_start_offset [reg_offset -n "[set letter]SEM_REGISTERS_INT_TABLE"]]

    while {$i<[expr pow(2,9)] && ! [eof $int_file]} {
        set line [gets $int_file]
        if {$line==""} {
            continue
        }
        writeByName $addr "0x$line"
        incr addr 4
        incr i
    }
    puts "[set letter]STORM Int Table Switching is done..."
    
}

proc  fileExist {dir} {
    global Pram_template
    global Int_template
    foreach letter {X C T U} {
        if {![file exists [format "%s%s%s" $dir $letter $Pram_template]] ||
        ![file exists [format "%s%s%s" $dir $letter $Int_template]]} {
            return false
        }
    }
    return true
}
proc  isReady {} {
    set state true
    foreach letter {X C T U} {
        set thread_list 0xffff
        if {$letter=="T"} {
            set thread_list 0xff
        }
        if {![reg_read -n "[set letter]SEM_REGISTERS_SLEEP_THREADS_VALID"]==0 ||\
        ![reg_read -n "[set letter]SEM_REGISTERS_SET0_THREAD_EMPTY"]==1 ||\
        ![reg_read -n "[set letter]SEM_REGISTERS_SET1_THREAD_EMPTY"]==1 ||\
        ![reg_read -n "[set letter]SEM_REGISTERS_THREADS_LIST"]==$thread_list} {
            set state false
            break;
        }
    }
    if {!$state} {    
        return false
    }
    return true
}

proc pramSw {{dir "pram\\"}} {
    global tcl_platform
    global current
    global Pram_template
    global Int_template
    
    puts "stopping stats.."
    puts -nonewline [driver disable_stat]
    if {$current(TOTAL_DEV)>1} {
	    device [expr $current(DEV)%2+1]
	    puts -nonewline [driver disable_stat]
	    device [expr $current(DEV)%2+1]
    }
    
    after 2000
    disEnFic 1

    after 1000
    set state [isReady]
    
    if {!$state} {
        puts "The passive buffer can't be emptied, try to reduce load from the chip and try again later."
    } else {
        
        if {![fileExist $dir]} {
            puts "You must have $Pram_template and $Int_template files for all 4 storms in $dir.\nSomething is missing, recheck."
        } else {
            foreach letter {X C T U} {
                set pram [open [format "%s%s%s" $dir $letter $Pram_template]]
                set int [open [format "%s%s%s" $dir $letter $Int_template]]
                puts "Switching [set letter]storm pram and int table..."
                dumpPram $pram $int $letter
                close $pram
                close $int
            }
        }
    }
    disEnFic 0
    puts "resuming stats.."	
    puts -nonewline [driver enable_stat]
    if {$current(TOTAL_DEV)>1} {
	    device [expr $current(DEV)%2+1]
	    puts -nonewline [driver enable_stat]
	    device [expr $current(DEV)%2+1]
    }
}

set Pram_template "Storm.bin"
set Int_template "Storm.int"


puts "Use pramSw <rel path - optional (with ending \\) > to switch the pram"
puts "Relative path default value is \"pram\\\""
puts "if you want to write \\, you have to use \\\\ for it"
