# eewrite.tcl
set help_info "Read/Write a byte on given address in the external I2C NVRAM\n"
append help_info "Usage: ee read <addr>\n"
append help_info "       Returns single byte for a given address\n"
append help_info "Usage: ee write <addr> <val>\n"
append help_info "       Set value on address in external EEPROM address\n"
add_help ee $help_info

global isolate_epio
proc ee_isolate {enable} {
	global isolate_epio
	if {[info exists isolate_epio] == 0} {
		set isolate_epio [nvm cfg 223=]
		if {($isolate_epio < 9) || ($isolate_epio > 40) } {
			puts "Invalid isolate epio pin (nvm 223 = $isolate_epio)"
			return
		}
		set isolate_epio [expr $isolate_epio - 9]
	}

	if {($isolate_epio < 0) || ($isolate_epio > 31) } {
		return
	}

	epio write $isolate_epio [expr !$enable]
}
#
proc ee_init { } {
	ee_write 1 0xfe
	ee_write 0 0x03
	ee_write 0xc9 0x00
	ee_write 0xc8 0x04
}

proc ee_reset_smbus {} {
	# MCP_A_REG_MCPR_SMBUS_CONFIG reset
	reg write 0x388000 0x80000000

	# MCP_A_REG_MCPR_SMBUS_CONFIG enable
	reg write 0x388000 0x40000000

	# Flush SMBUS TX FIFO
	reg write 0x38800c 0x40000000

	# Enable All SMBUS events
	reg write 0x388038 0xffffffff
}
 

# ee_write enables writing byte through I2C SMBUS
proc ee_write {addr byte} {
	ee_isolate 1
	# Fresh start
	ee_reset_smbus

	# Write I2C address
	reg write 0x388040 0xa0

	# Write requested address 
	reg write 0x388040 [expr $addr >> 8] 
	reg write 0x388040 [expr $addr & 0xff]

	# Set last byte, and place requested value
	reg write 0x388040 [expr 0x80000000 | $byte]

	# Write Byte operation
	reg write 0x388030 0x80000600
	delay ms 5
	ee_isolate 0
}

proc ee_read {addr} {
	ee_isolate 1
	# Fresh start
	ee_reset_smbus
	
	# Write I2C address
	reg write 0x388040 0xa0

	# Write requested address for read operation
	reg write 0x388040 [expr $addr >> 8]
	reg write 0x388040 [expr 0x80000000 | $addr & 0xff]

	#  Write Byte operation
	reg write 0x388030 0x80000600
	delay ms 5

	ee_reset_smbus
	# Set write address of 0xa1 where the response await
	reg write 0x388040 0x000000a1

	# Read single byte
	reg write 0x388030 0x80000801
	delay ms 5
	set val [expr [reg read 0x388044] & 0xff]
	
	ee_isolate 0
	return $val
}

proc ee {args} {
	if {[llength $args] < 2 } {
		puts "Not enough arguments. Use \"help ee\"."
		return
	}
	set cmd [lindex $args 0]
	if {$cmd == "read"} {
		if {[llength $args] != 2 } {
			puts "Not enough arguments. Use \"help ee\"."
			return 0
		}
		set hex_val [format %x [ee_read [lindex $args 1]]]
		return 0x$hex_val
	} else {
		 if {$cmd == "write"} {
			if {[llength $args] != 3 } {
				puts "Not enough arguments. Use \"help ee\"."
				return 0
			}
			return [ee_write [lindex $args 1] [lindex $args 2] ]
		} else {
			puts "Invalid command.  Use \"help ee\"."
		}
	}
}
