set help_info "Display / Set EPIO\n"
append help_info "Usage: epio read \[num\]\n"
append help_info "       Display EPIOs I/O value/s\n"
append help_info "Usage: epio write <num> <val>\n"
append help_info "       Set specific EPIO output to 1 or 0\n"
append help_info "       <num> - 0..31\n"
append help_info "       <val> - 0..1\n"
add_help epio $help_info

proc epio {args} { 
  reg write 0xa9a0 [expr [reg read 0xa9a0] | 1]
  set epio_oe [reg read 0x800c8]
  set epio_i [reg read 0x800c0]
  set epio_o [reg read 0x800c4]
  set indent 1s
  set lfmt "format \"\%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent\""
  set number_h $lfmt
  set number_l $lfmt
  set oe $lfmt
  set io $lfmt
  set cmd [lindex $args 0]
  if {$cmd == "read"} {
	set epio_num [lindex $args 1]
	if {$epio_num != ""} {
	  if {$epio_num > 31} {
		puts "epio num not in range 0..31. Use \"help epio\"."
		return
	  }
      if {$epio_oe & [expr 1<<$epio_num]} {
		   set val [expr ($epio_o>>$epio_num) & 1]
		   puts "O"
	   } else {
		   set val [expr ($epio_i>>$epio_num) & 1]
		   puts "I"
	   }
	   return $val
	} else {
       for {set i 0} {$i<32} {incr i} {
		  set number_l "$number_l [expr $i % 10]"
          if {[expr ($epio_oe>>$i)&1]} {
             set oe "$oe O"
             set val [expr ($epio_o>>$i)&1];
          } else {
             set oe "$oe I"
             set val [expr ($epio_i>>$i)&1];
          }
          set io "$io $val"
       }
	   puts "EPIOs:"
	   puts "0                   1                   2                   3"
	   puts "[eval $number_l]"
       puts "[eval $oe]"
       puts "[eval $io]"
	}
  } elseif {$cmd == "write"} {
	set epio_num [lindex $args 1]
    set epio_val [lindex $args 2]
	if {[llength $args] < 3 } {
       puts "Not enough arguments. Use \"help epio\"."
	   return
	}
	if {$epio_num > 31} {
		puts "epio num not in range 0..31. Use \"help epio\"."
		return
	}
	if {$epio_val > 1} {
	    puts "epio val not in range 0..1. Use \"help epio\"."
		return
	}
  
    reg write 0x800c8 [expr $epio_oe | (1<<$epio_num)]
    reg write 0x800c4 [expr ($epio_o & ~ (1<<$epio_num))| ($epio_val<<$epio_num)]
#    puts "Setting EPIO$epio_num output val to $epio_val"
  } else {
    puts "Invalid command $cmd. Use \"help epio\"."
	return;
  }
}
