##########################################################
# idle_chk.tcl                                           #
#                                                        #
# Copyright(c) 2011-2014 Broadcom Corporation            #
# Copyright(c) 2014-2015 QLogic Corporation              #
# Proprietary and Confidential Information.              #
#                                                        #
# WARNING: DO NOT CHANGE THIS SCRIPT MANUALLY !          #
# SCRIPT IS CREATED AUTOMATICALY VIA create_idle_chk.tcl #
#                                                        #
# Module Description:                                    #
# The script tests the value of important registers,     #
# to verify the chip is at IDLE state.                   #
##########################################################
global errors
global warnings
global total
global dumpMode
global outfileName
global outfile
# Check if condition exists, if so print a relevant message according to its sevirity
proc CONDITION_CHK {{val} {val1} {val2} {condition} {severity} {fail_msg}} {	
	global errors
	global warnings
	global total
	global outfileName
	global outfile	
	
	incr total
	
	if {[expr $condition]} {		
		switch $severity { 
			IDLE_CHK_ERROR
			{
				if {$outfileName != ""} { 
					puts $outfile "idle_chk. Error (level 1)  : $fail_msg"
				} else {
					puts "idle_chk. Error (level 1)  : $fail_msg"
				}
				incr errors				
			}
			IDLE_CHK_ERROR_NO_TRAFFIC
			{
				if {$outfileName != ""} { 
					puts $outfile "idle_chk. Error if no traffic (level 2)  : $fail_msg"
				} else {
					puts "idle_chk. Error if no traffic (level 2)  : $fail_msg"
				}
				incr errors							
			}
			IDLE_CHK_WARNING
			{
				if {$outfileName != ""} {
					puts $outfile "idle_chk. Warning (level 3): $fail_msg" 
				} else {
					puts "idle_chk. Warning (level 3): $fail_msg" 
				}
				
				incr warnings				
			}
			
			default
			{
				if {$outfileName != ""} {
					puts $outfile "Unknown severity level"
					puts $outfile "severity=$severity fail_msg=$fail_msg"
				} else {
					puts "Unknown severity level"
					puts "severity=$severity fail_msg=$fail_msg"
				}
			}
		} 
	}
}

# Check if test should be performed on this chip, according to chip_mask
proc CHIP_MASK_CHK {chip_mask} {		
	
	# get chip number for MISC register
	set chip_num [reg_read -n MISC_REG_CHIP_NUM]
	set chip_rev [reg_read -n MISC_REG_CHIP_REV]
	set chip_metal [reg_read -n MISC_REG_CHIP_METAL]
	
	if {(($chip_rev == 1) || ($chip_rev == 0xC) || ($chip_rev == 0xD)) &&
	    (($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770))} {
		set is_e3_b0 1
	} else {
		set is_e3_b0 0
	}
	
        switch $chip_mask {		
		"0x1" { if { $chip_num == 5710} {return 1}}
                "0x2" { if {($chip_num == 5711) || ($chip_num == 5712)} {return 1}}
                "0x3" { if {($chip_num == 5710) || ($chip_num == 5711) || ($chip_num == 5712)} {return 1}}
		"0x4" { if {($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731)} {return 1}}
		"0x5" { if {($chip_num == 5710) || ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731)} {return 1}}
		"0x6" { if {($chip_num == 5711) || ($chip_num == 5712) || ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731)} {return 1}}
		"0x7" { if {($chip_num == 5710) || ($chip_num == 5711) || ($chip_num == 5712) ||
			    ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731)} {return 1}}
		"0x8" { if {(($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)) && !$is_e3_b0 } {return 1}}
		"0x9" { if {(($chip_num == 5710) || ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)) && !$is_e3_b0} {return 1}}
		"0xA" { if {(($chip_num == 5711) || ($chip_num == 5712) || ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)) && !$is_e3_b0} {return 1}}
		"0xB" { if {(($chip_num == 5710) || ($chip_num == 5711) || ($chip_num == 5712) ||
			    ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)) && !$is_e3_b0} {return 1}}
		"0xC" { if {(($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) ||
			    ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)) && !$is_e3_b0} {return 1}}
		"0xD" { if {(($chip_num == 5710) || ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) ||
			    ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)) && !$is_e3_b0} {return 1}}
		"0xE" { if {(($chip_num == 5711) || ($chip_num == 5712) ||
			    ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) ||
			    ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)) && !$is_e3_b0} {return 1}}
		"0xF" { if {(($chip_num == 5710) || ($chip_num == 5711) || ($chip_num == 5712) ||
			    ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) ||
			    ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)) && !$is_e3_b0} {return 1}}
		
		# Support new bit 4 - E3 B0    
		"0x10" { if {$is_e3_b0} {return 1}}
		"0x11" { if {($chip_num == 5710) || $is_e3_b0} {return 1}}
		"0x12" { if {($chip_num == 5711) || ($chip_num == 5712) || $is_e3_b0} {return 1}}
		"0x13" { if {($chip_num == 5710) || ($chip_num == 5711) || ($chip_num == 5712) || $is_e3_b0} {return 1}}
		"0x14" { if {($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) || $is_e3_b0} {return 1}}
		"0x15" { if {($chip_num == 5710) || ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) || $is_e3_b0 } {return 1}}
		"0x16" { if {($chip_num == 5711) || ($chip_num == 5712) || ($chip_num == 5713) || ($chip_num == 5714) ||
			    ($chip_num == 5730) || ($chip_num == 5731) || $is_e3_b0 } {return 1}}
		"0x17" { if {($chip_num == 5710) || ($chip_num == 5711) || ($chip_num == 5712) ||
			    ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) || $is_e3_b0} {return 1}}
		"0x18" { if {($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770) } {return 1}}
		"0x19" { if {($chip_num == 5710) || ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)} {return 1}}
		"0x1A" { if {($chip_num == 5711) || ($chip_num == 5712) ||
			     ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)} {return 1}}
		"0x1B" { if {($chip_num == 5710) || ($chip_num == 5711) || ($chip_num == 5712) ||
			    ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)} {return 1}}
		"0x1C" { if {($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) ||
			    ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)} {return 1}}
		"0x1D" { if {($chip_num == 5710) || ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) ||
			     ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)} {return 1}}
		"0x1E" { if {($chip_num == 5711) || ($chip_num == 5712) ||
			    ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) ||
			    ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)} {return 1}}
		"0x1F" { if {($chip_num == 5710) || ($chip_num == 5711) || ($chip_num == 5712) ||
			    ($chip_num == 5713) || ($chip_num == 5714) || ($chip_num == 5730) || ($chip_num == 5731) ||
			    ($chip_num == 5773) || ($chip_num == 5774) || ($chip_num == 5770)} {return 1}}	     
		
		default	{ puts "Unknown chip_mask" }		
	}
	
	return 0	
}


# Check if this register's block is in reset
proc IS_BLOCK_IN_RESET {{reg}} {
	
	set block_name [lindex [split $reg "_"] 0]
	
	if {$block_name == "PGLUE"} {set block_name "PGLUE_B"}
		
	if {[is_block_in_reset $block_name]} {
		puts "Warning: block $block_name is in reset state, skipping this idle_chk.."
		return 1
	} else {
		return 0
	}
}

# IDLE_CHK_1: read one reg and check the condition
proc IDLE_CHK_1 {{chip_mask} {reg} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	#Incase <reg> includes register+offset split it into <reg> and <offset>
	if {[lsearch $reg "+"] != -1} {
		set offset [lindex [split $reg "+"] 1]
		set reg [lindex [split $reg "+"] 0]
		#remove reduandant space and brackets
		regsub { } $reg {} reg		
	} else {
		set offset 0
	}
		
	#Incase of offset value e.g. 0x2114, read directly from offset
	if {[lsearch $reg "0x*"] != -1} {
		set start_offset $reg		
	} else {
		#otherwise get the register offset using reg_offset utility
		#incase of offset range (xxx-yyy), use the first offset
		set start_offset [lindex [split [reg_offset -n $reg] "-"] 0]
	}	
	set val [reg read [expr ($start_offset + $offset)]]
	
	# Incase of dumpMode print the register, offset and value without condition check
	if {$dumpMode && ($outfileName != "")} {                
		puts $outfile "idle_chk_1 dump $reg: offset = [format "0x%x" [expr ($start_offset+$offset)]] val = [format "0x%x" $val]"		
		return
	}
	
	CONDITION_CHK $val 0 0 $condition $severity "$fail_msg, Value is [format 0x%x $val]"
}
	

# IDLE_CHK_2: loop to read one reg and check the condition 
proc IDLE_CHK_2 {{chip_mask} {reg} {loop} {inc} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	#Incase <reg> includes register+offset split it into <reg> and <offset>
	if {[lsearch $reg "+"] != -1} {
		set offset [lindex [split $reg "+"] 1]
		set reg [lindex [split $reg "+"] 0]
		#remove reduandant space
		regsub { } $reg {} reg
	} else {
		set offset 0
	}
	
	#Incase of offset value e.g. 0x2114, read directly from offset
	if {[lsearch $reg "0x*"] != -1} {		
		set start_offset $reg
	} else {
		#Other wise use the utility reg_read -n <register_name>
		set start_offset [lindex [split [reg_offset -n $reg] "-"] 0]
	}	
	
	incr start_offset $offset
	for {set i 0} {$i < $loop} {incr i} {		 
		set val [reg read [expr ($start_offset + $i*$inc)]]
		
		# Incase of dumpMode print the register, offset and value without condition check
		if {$dumpMode && ($outfileName != "")} {
			puts $outfile "idle_chk_2 dump $reg: offset = [format "0x%x" [expr ($start_offset + $i*$inc)]] val = [format "0x%x" $val]"			
		} else {
			CONDITION_CHK $val 0 0 $condition $severity "$reg+$i*$inc $fail_msg, Value is [format 0x%x $val]"
		}
	} 
}

# IDLE_CHK_3: read two regs and check the condition
proc IDLE_CHK_3 {{chip_mask} {reg1} {reg2} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg1] || [IS_BLOCK_IN_RESET $reg2]} {
		return
	}
	
	set val1 [reg_read -n $reg1]
	set val2 [reg_read -n $reg2]
	
	# Incase of dumpMode print the register, offset and value without condition check
	if {$dumpMode && ($outfileName != "")} {
		puts $outfile  "idle_chk_3 dump $reg1: offset = [format "0x%x" [reg_offset -n $reg1]] val1 = [format "0x%x" $val1]"
		puts $outfile  "idle_chk_3 dump $reg2: offset = [format "0x%x" [reg_offset -n $reg2]] val2 = [format "0x%x" $val2]"
		return
	}
	CONDITION_CHK 0 $val1 $val2 $condition $severity "$fail_msg, Values are [format 0x%x $val1] [format 0x%x $val2]"
}

# IDLE_CHK_4: read one reg and check according to CID_CAM
proc IDLE_CHK_4 {{chip_mask} {reg1} {reg2} {inc} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg1] || [IS_BLOCK_IN_RESET $reg2]} {
		return
	}
	
	set offset1 [lindex [split [reg_offset -n $reg1] "-"] 0]
	set offset2 [lindex [split [reg_offset -n $reg2] "-"] 0]	
			
	# Get the loop size
	set count [reg_count -n $reg1]
	set count [expr ($count >> 2)]
	
	#debug print
	#puts "IDLE_CHK_4 count=$count offset1=[format "0x%x" $offset1] offset2=[format "0x%x" $offset2]"
	
	for {set i 0} {$i < $count} {incr i} {				
		set val1 [reg read [expr ($offset1 + $i*$inc)]]
		set val2 [expr [reg read [expr ($offset2 + $i*$inc)]] >> 1]
		
		# Incase of dumpMode print the register, offset and value without condition check
		if {$dumpMode && ($outfileName != "")} {
			puts $outfile "idle_chk_4 dump $reg1: offset = [format "0x%x" [expr ($offset1 + $i*$inc)]] val = [format "0x%x" $val1]"
			puts $outfile "idle_chk_4 dump $reg2: offset = [format "0x%x" [expr ($offset2 + $i*$inc)]] val = [format "0x%x" $val2]"			
		} else {
			CONDITION_CHK 0 $val1 $val2 $condition $severity "$fail_msg - LCID $i CID_CAM [format 0x%x $val2] Value is [format 0x%x $val1]"
		}
	}
}


# IDLE_CHK_5: read one reg and check according to another reg */
proc IDLE_CHK_5 {{chip_mask} {reg} {reg1} {reg2} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg1] || [IS_BLOCK_IN_RESET $reg2] || [IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	# Incase of dumpMode print the register, offset and value without condition check
	if {$dumpMode && ($outfileName != "")} {
		puts $outfile "idle_chk_5 dump $reg:  offset = [format "0x%x" [reg_offset -n $reg]]  val =  [format "0x%x" [reg_read -n $reg]]"
		puts $outfile "idle_chk_5 dump $reg1: offset = [format "0x%x" [reg_offset -n $reg1]] val1 = [format "0x%x" [reg_read -n $reg1]]"
		puts $outfile "idle_chk_5 dump $reg2: offset = [format "0x%x" [reg_offset -n $reg2]] val2 = [format "0x%x" [reg_read -n $reg2]]"			
	} else {
		if {![reg_read -n $reg]} {
			IDLE_CHK_3 $chip_mask $reg1 $reg2 $condition $severity $fail_msg
		}
	}
}

# IDLE_CHK_6: read wide-bus reg and check sub-fields
proc IDLE_CHK_6 {{chip_mask} {reg} {loop} {inc} {severity}} {
	
	global dumpMode
	global outfileName
	global outfile	
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	set offset [lindex [split [reg_offset -n $reg] "-"] 0]	
	for {set i 0} {$i < $loop} {incr  i} {		    
		set val1 [reg read [expr ($offset + $i*$inc)]]
		set val2 [reg read [expr ($offset + $i*$inc + 4)]]
		#QM_REG_PTRTBL - check that
		#bits[53:30] read pointer == bits[29:6] write pointer
		set rd_ptr [expr ($val1 & 0x3FFFFFC0) >> 6]
		set wr_ptr [expr (((($val1 & 0xC0000000) >> 30) & 0x3) | (($val2 & 0x3FFFFF) << 2))]
		# Incase of dumpMode print the register, offset and value without condition check
		if {$dumpMode && ($outfileName != "")} {
			puts $outfile "idle_chk_6 dump $reg: offset = [format "0x%x" [expr ($offset + $i*$inc)]] val = [format "0x%x" $val1]"
			puts $outfile "idle_chk_6 dump $reg: offset = [format "0x%x" [expr ($offset + $i*$inc + 4)]] val = [format "0x%x" $val2]"						
		} else {
			CONDITION_CHK 0 $rd_ptr $wr_ptr "($rd_ptr != $wr_ptr)" $severity "QM: PTRTBL entry $i, rd_ptr is not equal to wr_ptr. Values are [format 0x%x $rd_ptr] [format 0x%x $wr_ptr]"
		}			
		
		#QM_REG_PTRTBL - check that:
		#bits[5:4] read bank0 == bits[1:0] write bank0
		set rd_bank [expr (($val1 & 0x30) >> 4)]
	        set wr_bank [expr ($val1 & 0x03)]
		if {!$dumpMode } {		
			CONDITION_CHK 0 $rd_bank $wr_bank "($rd_bank != $wr_bank)" $severity "QM: PTRTBL entry $i, rd_bank is not equal to wr_bank. Values are [format 0x%x $rd_bank] [format 0x%x $wr_bank]"
		}			
	} 
}

# IDLE_CHK_7:  Verify activity count value is as expected to this connType
proc IDLE_CHK_7 {{chip_mask} {ac_reg} {cfc_info_reg} {cfc_cid_cam} {inc} {condition} {severity} {fail_msg}} {

	global dumpMode
	global outfileName
	global outfile	
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $ac_reg] || [IS_BLOCK_IN_RESET $cfc_info_reg]} {
		return
	}
	
	set ac_offset [lindex [split [reg_offset -n $ac_reg] "-"] 0]
	set cfc_info_offset [lindex [split [reg_offset -n $cfc_info_reg] "-"] 0]
	set cfc_cid_cam_offset [lindex [split [reg_offset -n $cfc_cid_cam] "-"] 0]	
	
	# Get the loop size
	set count [reg_count -n $cfc_info_reg]
	set count [expr {$count >> 4}]
	
	#debug print
	#puts "IDLE_CHK_7 count=$count offset1=[format "0x%x" $offset1] offset2=[format "0x%x" $offset2]"
	
	for {set i 0} {$i < $count} {incr  i} {
		# For dump mode only
		if {$dumpMode && ($outfileName != "")} {
			set tmp_offset [expr {$ac_offset + $i*4}]
			puts $outfile "idle_chk_7 dump $ac_reg: offset = [format "0x%x" $tmp_offset] val = [format "0x%x" [reg read $tmp_offset]]"
			set tmp_offset [expr {$cfc_info_offset + $i*$inc}]
			puts $outfile "idle_chk_7 dump $cfc_info_reg: offset = [format "0x%x" $tmp_offset] val = [format "0x%x" [reg read $tmp_offset]]"
			set tmp_offset [expr {$cfc_info_offset + $i*$inc + 4}]
			puts $outfile "idle_chk_7 dump $cfc_info_reg: offset = [format "0x%x" $tmp_offset] val = [format "0x%x" [reg read $tmp_offset]]"
			set tmp_offset [expr {$cfc_info_offset + $i*$inc + 8}]
			puts $outfile "idle_chk_7 dump $cfc_info_reg: offset = [format "0x%x" $tmp_offset] val = [format "0x%x" [reg read $tmp_offset]]"			
		}
		
		# read AC value
		# val2 holds the AC value
		set val2 [reg read [expr {$ac_offset + $i*4}]]		
		
		set chip_num [reg_read -n MISC_REG_CHIP_NUM]   
		# in E2 the CFC_REG_INFO_RAM was changed to 128 bits width
		# and the connection type location was changed to bits 92-89
		if {($chip_num == 0x1662) || ($chip_num == 0x1663) || ($chip_num == 0x1651) || ($chip_num == 0x1652)} {			
			set tmp_val [reg read [expr {$cfc_info_offset +$i*$inc}]]									
			set tmp_val [reg read [expr {$cfc_info_offset +$i*$inc + 4}]]			
			set tmp_val [reg read [expr {$cfc_info_offset + $i*$inc + 8}]]
			reg read [expr {$cfc_info_offset + $i*$inc + 12}]
			# val1 holds the connn_type value
			set val1 [expr {$tmp_val & 0x1E000000} >> 25]			
		} else {			
			set tmp_val [reg read [expr {$cfc_info_offset +$i*$inc}]]									
			set tmp_val [reg read [expr {$cfc_info_offset +$i*$inc + 4}]]			
			set tmp_val [reg read [expr {$cfc_info_offset + $i*$inc + 8}]]
			reg read [expr {$cfc_info_offset + $i*$inc + 12}]
			# val1 holds the connn_type value (bits 70-67 in CFC_REG_INFO_RAM)
			set val1 [expr {$tmp_val & 0x00000078} >> 3]
		}
		
		# read CID_CAM of this lcid entry
		# val holds the cid_cam value
		set val [reg read [expr {$cfc_cid_cam_offset + $i*4}]]		
				
		if {!$dumpMode } {		
			# check AC only if CID_CAM bit 0 is valid
			if {[expr {$val & 0x1}] == 0x1} {
				CONDITION_CHK $val $val1 $val2 $condition $severity "$fail_msg. Activity counter value is [format 0x%x $val2]. LCID $i CID_CAM $val"								
			}				
		}		
	}
}

proc get_vq_hoq {vq} {
    global dumpMode
    global outfileName
    global outfile
    # Only supported since E2
    set chip_num [reg_read -n MISC_REG_CHIP_NUM]
    if {[is_e1x]} {
        return "-"
    }

    # read HOQ at specified VQ
    reg_write -n PXP2_REG_RQ_HOQ_RAM_RD_REQ -v $vq
    for {set i 0} {$i < 200} {incr i} {
        if {[reg_read -n PXP2_REG_RQ_HOQ_RAM_RD_STATUS]} break
    }
    if {($i == 200) && !$dumpMode} {
        return "ERROR: Timeout while reading HOQ at VQ $vq"
    }
    set clients {"USDM" "CSDM" "XSDM" "TSDM" "PBF" "QM" "Timers" "Searcher" "CDU_RD" "DMAE" "USDM_DP" "HC" "CDU_WR" "DBG" "_INVALID_" "_INVALID_"}
    set rd0 [reg_read -n PXP2_REG_RQ_HOQ_RAM_DATA_RD_0]
    set rd1 [reg_read -n PXP2_REG_RQ_HOQ_RAM_DATA_RD_1]
    set rd2 [reg_read -n PXP2_REG_RQ_HOQ_RAM_DATA_RD_2]
    set rd3 [reg_read -n PXP2_REG_RQ_HOQ_RAM_DATA_RD_3]
    set read_bit [lsearch {1 2 3 5 6 7 8 9 10 11 28} [expr $vq]]
    set is_write [expr (1 << $vq) & 0x60200000]
    if {!$is_write && ($read_bit == -1 || ([reg_read -n PXP2_REG_RQ_CMG_VQ_RDWR_N] & (1 << $read_bit)))} {
        set rq_type "READ"
    } else {
        set rq_type "WRITE"
    }

    # Incase of dumpMode print the register values
    if {$dumpMode && ($outfileName != "")} {                
	puts $outfile "idle_chk_8 dump PXP2_REG_RQ_HOQ_RAM_DATA_RD_0: val = [format "0x%x" $rd0]"
	puts $outfile "idle_chk_8 dump PXP2_REG_RQ_HOQ_RAM_DATA_RD_1: val = [format "0x%x" $rd1]"
	puts $outfile "idle_chk_8 dump PXP2_REG_RQ_HOQ_RAM_DATA_RD_2: val = [format "0x%x" $rd2]"
	puts $outfile "idle_chk_8 dump PXP2_REG_RQ_HOQ_RAM_DATA_RD_3: val = [format "0x%x" $rd3]"		
    }

    set rq_id [expr $rd0 >> 16]
    set len [expr $rd0 & 0xffff]
    set addr_lo $rd1
    set addr_hi $rd2
    set flags [lindex {"None" "RO" "NS" "RO+NS"} [expr $rd3 & 3]]
    set client [lindex $clients [expr ($rd3 >> 2) & 0xf]]
    set done [expr ($rd3 >> 6) & 1]
    set pfid [expr ($rd3 >> 8) & 7]
    set vfvalid [expr ($rd3 >> 11) & 1]
    set vfid [expr ($rd3 >> 12) & 0x3f]
    set atc [expr ($rd3 >> 18) & 0x7]
    if {$vfvalid} {
        set func [format "VFID %d (PF %d)" $vfid $pfid]
    } else {
        set func [format "PF %d" $pfid]
    }

    return [format "  Request at VQ%d is %s %d bytes to/from address 0x%x_%08x by %s: rq_id=%d done=%d func=%s atc=%d flags=%s." \
        $vq $rq_type $len $addr_hi $addr_lo $client $rq_id $done $func $atc $flags]

}

# IDLE_CHK_8: check PXP VQ occupancy according to condition
proc IDLE_CHK_8 {{chip_mask} {reg} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	#Incase <reg> includes register+offset split it into <reg> and <offset>
	if {[lsearch $reg "+"] != -1} {
		set offset [lindex [split $reg "+"] 1]
		set reg [lindex [split $reg "+"] 0]
		#remove reduandant space and brackets
		regsub { } $reg {} reg		
	} else {
		set offset 0
	}
		
	#Incase of offset value e.g. 0x2114, read directly from offset
	if {[lsearch $reg "0x*"] != -1} {
		set start_offset $reg		
	} else {
		#otherwise get the register offset using reg_offset utility
		#incase of offset range (xxx-yyy), use the first offset
		set start_offset [lindex [split [reg_offset -n $reg] "-"] 0]
	}	
	set val [reg read [expr ($start_offset + $offset)]]
	
	# Incase of dumpMode print the register, offset and value without condition check
	if {$dumpMode && ($outfileName != "")} {                
		puts $outfile "idle_chk_8 dump $reg: offset = [format "0x%x" [expr ($start_offset+$offset)]] val = [format "0x%x" $val]"		
	}

	if {(([expr $condition]) || ($dumpMode && ($outfileName != "")))} {
	    if {[scan $reg "PXP2_REG_RQ_VQ%d_ENTRY_CNT" vq] == 1} {
	        set msg [get_vq_hoq $vq]
	    } else {
	        set msg "  < ERROR: cannot extract VQ number from register $reg >"
	    }
	    if {$dumpMode && ($outfileName != "")} {
			return
	    }
            CONDITION_CHK 0 0 0 1 $severity "$fail_msg, Value is [format 0x%x $val]\n$msg"
        }
}

proc dump_vqs {} {
    for {set i 0} {$i < 32} {incr i} {
        set msg [get_vq_hoq $i]
        if {[string first "r/w 0 bytes" $msg] == -1} {
            puts $msg
        }
    }
}

# Disable timer scan at a given port
proc disable_timer {port} {
	
    set chip_num [reg_read -n MISC_REG_CHIP_NUM]
    
    # for E2 we need to disable TM_REG_EN_LINEAR0_TIMER
    if {[is_e2]} {
	if {![is_4port_mode]} {
	    set port 0	
        }	
    } 
    
    # skip this setting in E2, since these registers do not exist    
    if {[reg_read -n [format "TM_REG_EN_LINEAR%d_TIMER" $port]] == 0} {
	# timer not active
	return 0;
    }
    # disable timer
    reg write [reg_offset -n [format "TM_REG_EN_LINEAR%d_TIMER" $port]] 0
    # poll until end of scan
    for {set i 0} {$i < 200} {incr i} {
	if {[reg_read -n [format "TM_REG_LIN%d_SCAN_ON" $port]] == 0} {
		# scan completed - finish
	        return 1
	}        
    }
    puts "timeout while waiting for timers scan at port $port to complete!"
    return 1
}


# Enable timer scan at a given port
proc enable_timer {port} {    
    set chip_num [reg_read -n MISC_REG_CHIP_NUM]
    
    # for E2 we need to disable TM_REG_EN_LINEAR0_TIMER
    if {[is_e2]} {
	set port 0	
    } 
	
    reg write [reg_offset -n [format "TM_REG_EN_LINEAR%d_TIMER" $port]] 1    
}


# Check existance of assertions
proc check_assertions {} {
    set IS_E1H 0
    if {$::sys(NO_PCI) == 0} {
	if {$::current(DID) > 5710} {set IS_E1H 1}
    } else {
	#Incase of -no_pci mode read directly from MISC_REG_CHIP_NUM
	if {[reg read 0xA408] > 5710} {set IS_E1H 1}	
    }
    
    set STORM_ASSERT_LIST_INDEX_OFFSET 0x2000    
    

    set totalAsserts 0
    
    foreach instance { X T C U } {
        set stormAsserts 0
        # Get the base offset of the fast memory in SEMI
        set ramName [format "%sSEM_REG_FAST_MEMORY" $instance]
        set ramOffset [reg_offset -n $ramName]
        set ramOffset [lindex [split $ramOffset "-"] 0]
        
        # Get the last assert index number:	
	if {$::sys(NO_PCI)} {
		set lastIndexOffset $STORM_ASSERT_LIST_INDEX_OFFSET		
	} else {
		set lastIndexOffset [assert last_index_offset $instance]		
	}
        incr lastIndexOffset $ramOffset
        set lastIndex [reg read $lastIndexOffset]
        
        # Check assertions
        if {$lastIndex > 0} {
            set stormAsserts [expr {$lastIndex / 2}]
            puts [format "Warning! found %d assertions in %sstorm" $stormAsserts $instance]
            incr totalAsserts $stormAsserts
        }
    }
    return $totalAsserts
}


proc idle_chk {args} {
	global errors
	global warnings
	global total
	global dumpMode
	global outfileName
	global outfile
	global rdfFile
	set errors 0
	set warnings 0
	set noWarning 0
	set ignoreAssert 0
	set total 0
	set dumpMode 0
	set outfileName ""

	# Parse the options
	for {set i 0} {$i < [llength $args]} {incr i} {
		switch -- [lindex $args $i] {
		    "-ignoreAssert"  { set ignoreAssert 1 }
		    "-noWarn"        { set noWarning 1}
		    "-out"           { incr i; set outfileName [lindex $args $i]}
		    "-dumpMode"      { set dumpMode 1}
		}
	}
	if {$outfileName != ""} {
	       set outfile [open $outfileName a]
	}
	if {$dumpMode && ($outfileName == "" )} {
	       puts "Usage: -dumpMode -out filename "
	       return 0
	}

	# Parse the RDF file
	#get RDF file name according to DID
	get_rdf_file
	parse_rdf -f $rdfFile

	# Disable timers
	set enableTimer0 [disable_timer 0]
	set enableTimer1 [disable_timer 1]
	# Read register 0x2114 val and check if condition on val exist
	IDLE_CHK_1 0x3 0x2114 "((\$val & 0x0FF010) != 0)" IDLE_CHK_ERROR "PCIE: ucorr_err_status is not 0"
	# Read register 0x2114 val and check if condition on val exist
	IDLE_CHK_1 0x3 0x2114 "((\$val & 0x100000) != 0)" IDLE_CHK_WARNING "PCIE: ucorr_err_status - Unsupported request error"
	# Read register 0x2120 val and check if condition on val exist
	IDLE_CHK_1 0x3 0x2120 "(((\$val & 0x31C1) != 0x2000) && ((\$val & 0x31C1) != 0))" IDLE_CHK_WARNING "PCIE: corr_err_status is not 0x2000"
	# Read register 0x2814 val and check if condition on val exist
	IDLE_CHK_1 0x3 0x2814 "((\$val & ~0x40100) != 0)" IDLE_CHK_ERROR "PCIE: attentions register is not 0x40100"
	# Read register 0x281c val and check if condition on val exist
	IDLE_CHK_1 0x2 0x281c "((\$val & ~0x40040100) != 0)" IDLE_CHK_ERROR "PCIE: attentions register is not 0x40040100"
	# Read register 0x2820 val and check if condition on val exist
	IDLE_CHK_1 0x2 0x2820 "((\$val & ~0x40040100) != 0)" IDLE_CHK_ERROR "PCIE: attentions register is not 0x40040100"
	# Read register PXP2_REG_PGL_EXP_ROM2 val and check if condition on val exist
	IDLE_CHK_1 0x3 PXP2_REG_PGL_EXP_ROM2 "(\$val != 0xffffffff)" IDLE_CHK_WARNING "PXP2: There are outstanding read requests. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register 0x212c val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x3 0x212c 4 4 "((\$val != 0) && (\$errors > 0))" IDLE_CHK_WARNING "PCIE: error packet header is not 0"
	# Read register 0x2104 val and check if condition on val exist
	IDLE_CHK_1 0x1C 0x2104 "((\$val & 0x0FD010) != 0)" IDLE_CHK_ERROR "PCIE: ucorr_err_status is not 0"
	# Read register 0x2104 val and check if condition on val exist
	IDLE_CHK_1 0x1C 0x2104 "((\$val & 0x100000) != 0)" IDLE_CHK_WARNING "PCIE: ucorr_err_status - Unsupported request error"
	# Read register 0x2104 val and check if condition on val exist
	IDLE_CHK_1 0x1C 0x2104 "((\$val & 0x2000) != 0)" IDLE_CHK_WARNING "PCIE: ucorr_err_status - Flow Control Protocol Error"
	# Read register 0x2110 val and check if condition on val exist
	IDLE_CHK_1 0x1C 0x2110 "(((\$val & 0x31C1) != 0x2000) && ((\$val & 0x31C1) != 0))" IDLE_CHK_WARNING "PCIE: corr_err_status is not 0x2000"
	# Read register 0x2814 val and check if condition on val exist
	IDLE_CHK_1 0x1C 0x2814 "((\$val & 0x2000000) != 0)" IDLE_CHK_WARNING "PCIE: TTX_BRIDGE_FORWARD_ERR - Received master request while BME was 0."
	# Read register 0x2814 val and check if condition on val exist
	IDLE_CHK_1 0x1C 0x2814 "((\$val & ~0x2040902) != 0)" IDLE_CHK_ERROR "PCIE: Func 0 1: attentions register is not 0x2040902"
	# Read register 0x2854 val and check if condition on val exist
	IDLE_CHK_1 0x1C 0x2854 "((\$val & ~0x10240902) != 0)" IDLE_CHK_ERROR "PCIE: Func 2 3 4: attentions register is not 0x10240902"
	# Read register 0x285c val and check if condition on val exist
	IDLE_CHK_1 0x1C 0x285c "((\$val & ~0x10240902) != 0)" IDLE_CHK_ERROR "PCIE: Func 5 6 7: attentions register is not 0x10240902"
	# Read register 0x3040 val and check if condition on val exist
	IDLE_CHK_1 0x18 0x3040 "((\$val & 0x2) != 0)" IDLE_CHK_ERROR "PCIE: Overflow in DLP2TLP buffer"
	# Read register PXP2_REG_PGL_EXP_ROM2 val and check if condition on val exist
	IDLE_CHK_1 0x1C PXP2_REG_PGL_EXP_ROM2 "(\$val != 0xffffffff)" IDLE_CHK_WARNING "PXP2: There are outstanding read requests for tags 0-31. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register 0x211c val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x1C 0x211c 4 4 "((\$val != 0) && (\$errors > 0))" IDLE_CHK_WARNING "PCIE: error packet header is not 0"
	# Read register PGLUE_B_REG_INCORRECT_RCV_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_INCORRECT_RCV_DETAILS "(\$val != 0)" IDLE_CHK_ERROR "PGLUE_B: Packet received from PCIe not according to the rules."
	# Read register PGLUE_B_REG_WAS_ERROR_VF_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_WAS_ERROR_VF_31_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 0-31 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_VF_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_WAS_ERROR_VF_63_32 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 32-63 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_VF_95_64 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_WAS_ERROR_VF_95_64 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 64-95 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_VF_127_96 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_WAS_ERROR_VF_127_96 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 96-127 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_PF_7_0 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_WAS_ERROR_PF_7_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for PFs 0-7 is not 0"
	# Read register PGLUE_B_REG_RX_ERR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_RX_ERR_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Completion received with error. (2:0) - PFID. (3) - VF_VALID. (9:4) - VFID. (11:10) - Error code : 0 - Completion Timeout ; 1 - Unsupported Request; 2 - Completer Abort. (12) - valid bit."
	# Read register PGLUE_B_REG_RX_TCPL_ERR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_RX_TCPL_ERR_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: ATS TCPL received with error. (2:0) - PFID. (3) - VF_VALID. (9:4) - VFID. (11:10) - Error code : 0 - Completion Timeout ; 1 - Unsupported Request; 2 - Completer Abort. (16:12) - OTB Entry ID. (17) - valid bit."
	# Read register PGLUE_B_REG_TX_ERR_WR_ADD_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_TX_ERR_WR_ADD_31_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master write. Address(31:0) is not 0"
	# Read register PGLUE_B_REG_TX_ERR_WR_ADD_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_TX_ERR_WR_ADD_63_32 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master write. Address(63:32) is not 0"
	# Read register PGLUE_B_REG_TX_ERR_WR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_TX_ERR_WR_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master write. Error details register is not 0. (4:0) VQID. (23:21) - PFID. (24) - VF_VALID. (30:25) - VFID."
	# Read register PGLUE_B_REG_TX_ERR_WR_DETAILS2 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_TX_ERR_WR_DETAILS2 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master write. Error details 2nd register is not 0. (21) - was_error set; (22) - BME cleared; (23) - FID_enable cleared; (24) - VF with parent PF FLR_request or IOV_disable_request."
	# Read register PGLUE_B_REG_TX_ERR_RD_ADD_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_TX_ERR_RD_ADD_31_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE: Error in master read address(31:0) is not 0"
	# Read register PGLUE_B_REG_TX_ERR_RD_ADD_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_TX_ERR_RD_ADD_63_32 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master read address(63:32) is not 0"
	# Read register PGLUE_B_REG_TX_ERR_RD_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_TX_ERR_RD_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master read Error details register is not 0. (4:0) VQID. (23:21) - PFID. (24) - VF_VALID. (30:25) - VFID."
	# Read register PGLUE_B_REG_TX_ERR_RD_DETAILS2 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_TX_ERR_RD_DETAILS2 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master read Error details 2nd register is not 0. (21) - was_error set; (22) - BME cleared; (23) - FID_enable cleared; (24) - VF with parent PF FLR_request or IOV_disable_request."
	# Read register PGLUE_B_REG_VF_LENGTH_VIOLATION_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_VF_LENGTH_VIOLATION_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Target VF length violation access."
	# Read register PGLUE_B_REG_VF_GRC_SPACE_VIOLATION_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_VF_GRC_SPACE_VIOLATION_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Target VF GRC space access failed permission check."
	# Read register PGLUE_B_REG_TAGS_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_TAGS_63_32 "(\$val != 0xffffffff)" IDLE_CHK_WARNING "PGLUE_B: There are outstanding read requests for tags 32-63. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register PXP_REG_HST_VF_DISABLED_ERROR_VALID val1 and register PXP_REG_HST_VF_DISABLED_ERROR_DATA val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1C PXP_REG_HST_VF_DISABLED_ERROR_VALID PXP_REG_HST_VF_DISABLED_ERROR_DATA "(\$val != 0)" IDLE_CHK_WARNING "PXP: Access to disabled VF took place"
	# Read register PXP_REG_HST_PER_VIOLATION_VALID val and check if condition on val exist
	IDLE_CHK_1 0x1C PXP_REG_HST_PER_VIOLATION_VALID "(\$val != 0)" IDLE_CHK_WARNING "PXP: Zone A permission violation occurred"
	# Read register PXP_REG_HST_INCORRECT_ACCESS_VALID val and check if condition on val exist
	IDLE_CHK_1 0x1C PXP_REG_HST_INCORRECT_ACCESS_VALID "(\$val != 0)" IDLE_CHK_WARNING "PXP: Incorrect transaction took place"
	# Read register PXP2_REG_RD_CPL_ERR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x1C PXP2_REG_RD_CPL_ERR_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PXP2: Completion received with error. Error details register is not 0. (15:0) - ECHO. (28:16) - Sub Request length plus start_offset_2_0 minus 1."
	# Read register PXP2_REG_RD_CPL_ERR_DETAILS2 val and check if condition on val exist
	IDLE_CHK_1 0x1C PXP2_REG_RD_CPL_ERR_DETAILS2 "(\$val != 0)" IDLE_CHK_WARNING "PXP2: Completion received with error. Error details 2nd register is not 0. (4:0) - VQ ID. (8:5) - client ID. (9) - valid bit."
	# Read register PXP2_REG_RQ_VQ0_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ0_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ0 is not empty"
	# Read register PXP2_REG_RQ_VQ1_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ1_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ1 is not empty"
	# Read register PXP2_REG_RQ_VQ2_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ2_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ2 is not empty"
	# Read register PXP2_REG_RQ_VQ3_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ3_ENTRY_CNT "(\$val > 2)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ3 is not empty"
	# Read register PXP2_REG_RQ_VQ4_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ4_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ4 is not empty"
	# Read register PXP2_REG_RQ_VQ5_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ5_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ5 is not empty"
	# Read register PXP2_REG_RQ_VQ6_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ6_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ6 is not empty"
	# Read register PXP2_REG_RQ_VQ7_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ7_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ7 is not empty"
	# Read register PXP2_REG_RQ_VQ8_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ8_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ8 is not empty"
	# Read register PXP2_REG_RQ_VQ9_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ9_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ9 is not empty"
	# Read register PXP2_REG_RQ_VQ10_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ10_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ10 is not empty"
	# Read register PXP2_REG_RQ_VQ11_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ11_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ11 is not empty"
	# Read register PXP2_REG_RQ_VQ12_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ12_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ12 is not empty"
	# Read register PXP2_REG_RQ_VQ13_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ13_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ13 is not empty"
	# Read register PXP2_REG_RQ_VQ14_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ14_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ14 is not empty"
	# Read register PXP2_REG_RQ_VQ15_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ15_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ15 is not empty"
	# Read register PXP2_REG_RQ_VQ16_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ16_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ16 is not empty"
	# Read register PXP2_REG_RQ_VQ17_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ17_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ17 is not empty"
	# Read register PXP2_REG_RQ_VQ18_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ18_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ18 is not empty"
	# Read register PXP2_REG_RQ_VQ19_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ19_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ19 is not empty"
	# Read register PXP2_REG_RQ_VQ20_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ20_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ20 is not empty"
	# Read register PXP2_REG_RQ_VQ21_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ21_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ21 is not empty"
	# Read register PXP2_REG_RQ_VQ22_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ22_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ22 is not empty"
	# Read register PXP2_REG_RQ_VQ23_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ23_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ23 is not empty"
	# Read register PXP2_REG_RQ_VQ24_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ24_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ24 is not empty"
	# Read register PXP2_REG_RQ_VQ25_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ25_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ25 is not empty"
	# Read register PXP2_REG_RQ_VQ26_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ26_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ26 is not empty"
	# Read register PXP2_REG_RQ_VQ27_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ27_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ27 is not empty"
	# Read register PXP2_REG_RQ_VQ28_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ28_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ28 is not empty"
	# Read register PXP2_REG_RQ_VQ29_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ29_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ29 is not empty"
	# Read register PXP2_REG_RQ_VQ30_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ30_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ30 is not empty"
	# Read register PXP2_REG_RQ_VQ31_ENTRY_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_VQ31_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: VQ31 is not empty"
	# Read register PXP2_REG_RQ_UFIFO_NUM_OF_ENTRY val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_UFIFO_NUM_OF_ENTRY "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: rq_ufifo_num_of_entry is not 0"
	# Read register PXP2_REG_RQ_RBC_DONE val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_RBC_DONE "(\$val != 1)" IDLE_CHK_ERROR "PXP2: rq_rbc_done is not 1"
	# Read register PXP2_REG_RQ_CFG_DONE val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RQ_CFG_DONE "(\$val != 1)" IDLE_CHK_ERROR "PXP2: rq_cfg_done is not 1"
	# Read register PXP2_REG_PSWRQ_BW_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x3 PXP2_REG_PSWRQ_BW_CREDIT "(\$val != 0x1B)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: rq_read_credit and rq_write_credit are not 3"
	# Read register PXP2_REG_RD_START_INIT val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RD_START_INIT "(\$val != 1)" IDLE_CHK_ERROR "PXP2: rd_start_init is not 1"
	# Read register PXP2_REG_RD_INIT_DONE val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RD_INIT_DONE "(\$val != 1)" IDLE_CHK_ERROR "PXP2: rd_init_done is not 1"
	# Read register PXP2_REG_RD_SR_CNT val1 and register PXP2_REG_RD_SR_NUM_CFG val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1F PXP2_REG_RD_SR_CNT PXP2_REG_RD_SR_NUM_CFG "(\$val1 != (\$val2-1))" IDLE_CHK_WARNING "PXP2: rd_sr_cnt is not equal to rd_sr_num_cfg"
	# Read register PXP2_REG_RD_BLK_CNT val1 and register PXP2_REG_RD_BLK_NUM_CFG val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1F PXP2_REG_RD_BLK_CNT PXP2_REG_RD_BLK_NUM_CFG "(\$val1 != \$val2)" IDLE_CHK_WARNING "PXP2: rd_blk_cnt is not equal to rd_blk_num_cfg"
	# Read register PXP2_REG_RD_SR_CNT val1 and register PXP2_REG_RD_SR_NUM_CFG val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1F PXP2_REG_RD_SR_CNT PXP2_REG_RD_SR_NUM_CFG "(\$val1 < (\$val2-3))" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: There are more than two unused SRs"
	# Read register PXP2_REG_RD_BLK_CNT val1 and register PXP2_REG_RD_BLK_NUM_CFG val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1F PXP2_REG_RD_BLK_CNT PXP2_REG_RD_BLK_NUM_CFG "(\$val1 < (\$val2-2))" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: There are more than two unused blocks"
	# Read register PXP2_REG_RD_PORT_IS_IDLE_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RD_PORT_IS_IDLE_0 "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: P0 All delivery ports are not idle"
	# Read register PXP2_REG_RD_PORT_IS_IDLE_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RD_PORT_IS_IDLE_1 "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: P1 All delivery ports are not idle"
	# Read register PXP2_REG_RD_ALMOST_FULL_0 val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x1F PXP2_REG_RD_ALMOST_FULL_0 11 4 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: rd_almost_full is not 0"
	# Read register PXP2_REG_RD_DISABLE_INPUTS  val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_RD_DISABLE_INPUTS  "(\$val != 0)" IDLE_CHK_ERROR "PXP2: PSWRD inputs are disabled"
	# Read register PXP2_REG_HST_HEADER_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_HST_HEADER_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: HST header FIFO status is not 0"
	# Read register PXP2_REG_HST_DATA_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_HST_DATA_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: HST data FIFO status is not 0"
	# Read register PXP2_REG_PGL_WRITE_BLOCKED val and check if condition on val exist
	IDLE_CHK_1 0x3 PXP2_REG_PGL_WRITE_BLOCKED "(\$val != 0)" IDLE_CHK_ERROR "PXP2: pgl_write_blocked is not 0"
	# Read register PXP2_REG_PGL_READ_BLOCKED val and check if condition on val exist
	IDLE_CHK_1 0x3 PXP2_REG_PGL_READ_BLOCKED "(\$val != 0)" IDLE_CHK_ERROR "PXP2: pgl_read_blocked is not 0"
	# Read register PXP2_REG_PGL_WRITE_BLOCKED val and check if condition on val exist
	IDLE_CHK_1 0x1C PXP2_REG_PGL_WRITE_BLOCKED "(\$val != 0)" IDLE_CHK_WARNING "PXP2: pgl_write_blocked is not 0"
	# Read register PXP2_REG_PGL_READ_BLOCKED val and check if condition on val exist
	IDLE_CHK_1 0x1C PXP2_REG_PGL_READ_BLOCKED "(\$val != 0)" IDLE_CHK_WARNING "PXP2: pgl_read_blocked is not 0"
	# Read register PXP2_REG_PGL_TXW_CDTS val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP2_REG_PGL_TXW_CDTS "(((\$val >> 17) & 1) != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: There is data which is ready"
	# Read register PXP_REG_HST_ARB_IS_IDLE val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP_REG_HST_ARB_IS_IDLE "(\$val != 1)" IDLE_CHK_WARNING "PXP: HST arbiter is not idle"
	# Read register PXP_REG_HST_CLIENTS_WAITING_TO_ARB val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP_REG_HST_CLIENTS_WAITING_TO_ARB "(\$val != 0)" IDLE_CHK_WARNING "PXP: HST one of the clients is waiting for delivery"
	# Read register PXP_REG_HST_DISCARD_INTERNAL_WRITES_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x1E PXP_REG_HST_DISCARD_INTERNAL_WRITES_STATUS "(\$val != 0)" IDLE_CHK_WARNING "PXP: HST Close the gates: Discarding internal writes"
	# Read register PXP_REG_HST_DISCARD_DOORBELLS_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x1E PXP_REG_HST_DISCARD_DOORBELLS_STATUS "(\$val != 0)" IDLE_CHK_WARNING "PXP: HST Close the gates: Discarding doorbells"
	# Read register PXP2_REG_RQ_GARB val and check if condition on val exist
	IDLE_CHK_1 0x1C PXP2_REG_RQ_GARB "((\$val & 0x1000) != 0)" IDLE_CHK_WARNING "PXP2: PSWRQ Close the gates is asserted. Check AEU AFTER_INVERT registers for parity errors."
	# Read register DMAE_REG_GO_C0 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C0 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 0 go is not 0"
	# Read register DMAE_REG_GO_C1 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C1 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 1 go is not 0"
	# Read register DMAE_REG_GO_C2 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C2 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 2 go is not 0"
	# Read register DMAE_REG_GO_C3 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C3 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 3 go is not 0"
	# Read register DMAE_REG_GO_C4 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C4 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 4 go is not 0"
	# Read register DMAE_REG_GO_C5 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C5 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 5 go is not 0"
	# Read register DMAE_REG_GO_C6 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C6 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 6 go is not 0"
	# Read register DMAE_REG_GO_C7 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C7 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 7 go is not 0"
	# Read register DMAE_REG_GO_C8 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C8 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 8 go is not 0"
	# Read register DMAE_REG_GO_C9 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C9 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 9 go is not 0"
	# Read register DMAE_REG_GO_C10 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C10 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 10 go is not 0"
	# Read register DMAE_REG_GO_C11 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C11 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 11 go is not 0"
	# Read register DMAE_REG_GO_C12 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C12 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 12 go is not 0"
	# Read register DMAE_REG_GO_C13 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C13 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 13 go is not 0"
	# Read register DMAE_REG_GO_C14 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C14 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 14 go is not 0"
	# Read register DMAE_REG_GO_C15 val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_GO_C15 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 15 go is not 0"
	# Read register CFC_REG_ERROR_VECTOR val and check if condition on val exist
	IDLE_CHK_1 0x1F CFC_REG_ERROR_VECTOR "(\$val != 0)" IDLE_CHK_ERROR "CFC: error vector is not 0"
	# Read register CFC_REG_NUM_LCIDS_ARRIVING val and check if condition on val exist
	IDLE_CHK_1 0x1F CFC_REG_NUM_LCIDS_ARRIVING "(\$val != 0)" IDLE_CHK_ERROR "CFC: number of arriving LCIDs is not 0"
	# Read register CFC_REG_NUM_LCIDS_ALLOC val and check if condition on val exist
	IDLE_CHK_1 0x1F CFC_REG_NUM_LCIDS_ALLOC "(\$val != 0)" IDLE_CHK_ERROR "CFC: number of alloc LCIDs is not 0"
	# Read register CFC_REG_NUM_LCIDS_LEAVING val and check if condition on val exist
	IDLE_CHK_1 0x1F CFC_REG_NUM_LCIDS_LEAVING "(\$val != 0)" IDLE_CHK_ERROR "CFC: number of leaving LCIDs is not 0"
	# Check if AC (ACTIVITY COUNTERS) value is allowed on this connection Type
	IDLE_CHK_7 0x1F CFC_REG_ACTIVITY_COUNTER CFC_REG_INFO_RAM CFC_REG_CID_CAM 16 "((\$val1 == 0) && (\$val2 != 0) && (\$val2 != 2))" IDLE_CHK_ERROR_NO_TRAFFIC "CFC: AC is neither 0 nor 2 on connType 0 (ETH)"
	# Check if AC (ACTIVITY COUNTERS) value is allowed on this connection Type
	IDLE_CHK_7 0x1F CFC_REG_ACTIVITY_COUNTER CFC_REG_INFO_RAM CFC_REG_CID_CAM 16 "((\$val1 == 1) && (\$val2 != 0))" IDLE_CHK_ERROR_NO_TRAFFIC "CFC: AC is not 0 on connType 1 (TOE)"
	# Check if AC (ACTIVITY COUNTERS) value is allowed on this connection Type
	IDLE_CHK_7 0x1F CFC_REG_ACTIVITY_COUNTER CFC_REG_INFO_RAM CFC_REG_CID_CAM 16 "((\$val1 == 3) && (\$val2 != 0))" IDLE_CHK_ERROR_NO_TRAFFIC "CFC: AC is not 0 on connType 3 (iSCSI)"
	# Check if AC (ACTIVITY COUNTERS) value is allowed on this connection Type
	IDLE_CHK_7 0x1F CFC_REG_ACTIVITY_COUNTER CFC_REG_INFO_RAM CFC_REG_CID_CAM 16 "((\$val1 == 4) && (\$val2 != 0))" IDLE_CHK_ERROR_NO_TRAFFIC "CFC: AC is not 0 on connType 4 (FCoE)"
	# Read register QM_REG_QTASKCTR_0 val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x1F QM_REG_QTASKCTR_0 64 4 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Queue is not empty"
	# Read register QM_REG_VOQCREDIT_0 val1 and register QM_REG_VOQINITCREDIT_0 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0xF QM_REG_VOQCREDIT_0 QM_REG_VOQINITCREDIT_0 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: VOQ_0, VOQ credit is not equal to initial credit"
	# Read register QM_REG_VOQCREDIT_1 val1 and register QM_REG_VOQINITCREDIT_1 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0xF QM_REG_VOQCREDIT_1 QM_REG_VOQINITCREDIT_1 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: VOQ_1, VOQ credit is not equal to initial credit"
	# Read register QM_REG_VOQCREDIT_4 val1 and register QM_REG_VOQINITCREDIT_4 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0xF QM_REG_VOQCREDIT_4 QM_REG_VOQINITCREDIT_4 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: VOQ_4, VOQ credit is not equal to initial credit"
	# Read register QM_REG_PORT0BYTECRD val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3 QM_REG_PORT0BYTECRD QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: P0 Byte credit is not equal to initial credit"
	# Read register QM_REG_PORT1BYTECRD val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3 QM_REG_PORT1BYTECRD QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: P1 Byte credit is not equal to initial credit"
	# Read register CCM_REG_CAM_OCCUP val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_CAM_OCCUP "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "CCM: XX protection CAM is not empty"
	# Read register TCM_REG_CAM_OCCUP val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_CAM_OCCUP "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TCM: XX protection CAM is not empty"
	# Read register UCM_REG_CAM_OCCUP val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_CAM_OCCUP "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "UCM: XX protection CAM is not empty"
	# Read register XCM_REG_CAM_OCCUP val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_CAM_OCCUP "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "XCM: XX protection CAM is not empty"
	# Read register BRB1_REG_NUM_OF_FULL_BLOCKS val and check if condition on val exist
	IDLE_CHK_1 0x1F BRB1_REG_NUM_OF_FULL_BLOCKS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB1: BRB is not empty"
	# Read register CSEM_REG_SLEEP_THREADS_VALID val and check if condition on val exist
	IDLE_CHK_1 0x1F CSEM_REG_SLEEP_THREADS_VALID "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "CSEM: There are sleeping threads"
	# Read register TSEM_REG_SLEEP_THREADS_VALID val and check if condition on val exist
	IDLE_CHK_1 0x1F TSEM_REG_SLEEP_THREADS_VALID "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: There are sleeping threads"
	# Read register USEM_REG_SLEEP_THREADS_VALID val and check if condition on val exist
	IDLE_CHK_1 0x1F USEM_REG_SLEEP_THREADS_VALID "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: There are sleeping threads"
	# Read register XSEM_REG_SLEEP_THREADS_VALID val and check if condition on val exist
	IDLE_CHK_1 0x1F XSEM_REG_SLEEP_THREADS_VALID "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: There are sleeping threads"
	# Read register CSEM_REG_SLOW_EXT_STORE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F CSEM_REG_SLOW_EXT_STORE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "CSEM: External store FIFO is not empty"
	# Read register TSEM_REG_SLOW_EXT_STORE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F TSEM_REG_SLOW_EXT_STORE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: External store FIFO is not empty"
	# Read register USEM_REG_SLOW_EXT_STORE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F USEM_REG_SLOW_EXT_STORE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: External store FIFO is not empty"
	# Read register XSEM_REG_SLOW_EXT_STORE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F XSEM_REG_SLOW_EXT_STORE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: External store FIFO is not empty"
	# Read register CSDM_REG_SYNC_PARSER_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F CSDM_REG_SYNC_PARSER_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "CSDM: Parser serial FIFO is not empty"
	# Read register TSDM_REG_SYNC_PARSER_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F TSDM_REG_SYNC_PARSER_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSDM: Parser serial FIFO is not empty"
	# Read register USDM_REG_SYNC_PARSER_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F USDM_REG_SYNC_PARSER_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USDM: Parser serial FIFO is not empty"
	# Read register XSDM_REG_SYNC_PARSER_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F XSDM_REG_SYNC_PARSER_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSDM: Parser serial FIFO is not empty"
	# Read register CSDM_REG_SYNC_SYNC_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F CSDM_REG_SYNC_SYNC_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "CSDM: Parser SYNC serial FIFO is not empty"
	# Read register TSDM_REG_SYNC_SYNC_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F TSDM_REG_SYNC_SYNC_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSDM: Parser SYNC serial FIFO is not empty"
	# Read register USDM_REG_SYNC_SYNC_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F USDM_REG_SYNC_SYNC_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USDM: Parser SYNC serial FIFO is not empty"
	# Read register XSDM_REG_SYNC_SYNC_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F XSDM_REG_SYNC_SYNC_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSDM: Parser SYNC serial FIFO is not empty"
	# Read register CSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F CSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "CSDM: pxp_ctrl rd_data fifo is not empty in sdm_dma_rsp block"
	# Read register TSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F TSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSDM: pxp_ctrl rd_data fifo is not empty in sdm_dma_rsp block"
	# Read register USDM_REG_RSP_PXP_CTRL_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F USDM_REG_RSP_PXP_CTRL_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USDM: pxp_ctrl rd_data fifo is not empty in sdm_dma_rsp block"
	# Read register XSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F XSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSDM: pxp_ctrl rd_data fifo is not empty in sdm_dma_rsp block"
	# Read register DORQ_REG_DQ_FILL_LVLF val and check if condition on val exist
	IDLE_CHK_1 0x1F DORQ_REG_DQ_FILL_LVLF "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DORQ: DORQ queue is not empty"
	# Read register CFC_REG_CFC_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F CFC_REG_CFC_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "CFC: Interrupt status is not 0"
	# Read register CDU_REG_CDU_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F CDU_REG_CDU_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "CDU: Interrupt status is not 0"
	# Read register CCM_REG_CCM_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_CCM_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "CCM: Interrupt status is not 0"
	# Read register TCM_REG_TCM_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_TCM_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "TCM: Interrupt status is not 0"
	# Read register UCM_REG_UCM_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_UCM_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "UCM: Interrupt status is not 0"
	# Read register XCM_REG_XCM_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_XCM_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "XCM: Interrupt status is not 0"
	# Read register PBF_REG_PBF_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0xF PBF_REG_PBF_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "PBF: Interrupt status is not 0"
	# Read register TM_REG_TM_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F TM_REG_TM_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "TIMERS: Interrupt status is not 0"
	# Read register DORQ_REG_DORQ_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F DORQ_REG_DORQ_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "DORQ: Interrupt status is not 0"
	# Read register SRC_REG_SRC_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F SRC_REG_SRC_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "SRCH: Interrupt status is not 0"
	# Read register PRS_REG_PRS_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F PRS_REG_PRS_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "PRS: Interrupt status is not 0"
	# Read register BRB1_REG_BRB1_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F BRB1_REG_BRB1_INT_STS "((\$val & ~0xFC00) != 0)" IDLE_CHK_ERROR "BRB1: Interrupt status is not 0"
	# Read register XPB_REG_PB_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F XPB_REG_PB_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "XPB: Interrupt status is not 0"
	# Read register UPB_REG_PB_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F UPB_REG_PB_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "UPB: Interrupt status is not 0"
	# Read register PXP2_REG_PXP2_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1 PXP2_REG_PXP2_INT_STS "(\$val != 0)" IDLE_CHK_WARNING "PXP2: Interrupt status 0 is not 0"
	# Read register PXP2_REG_PXP2_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1E PXP2_REG_PXP2_INT_STS_0 "(\$val != 0)" IDLE_CHK_WARNING "PXP2: Interrupt status 0 is not 0"
	# Read register PXP2_REG_PXP2_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1E PXP2_REG_PXP2_INT_STS_1 "(\$val != 0)" IDLE_CHK_WARNING "PXP2: Interrupt status 1 is not 0"
	# Read register QM_REG_QM_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_QM_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "QM: Interrupt status is not 0"
	# Read register PXP_REG_PXP_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP_REG_PXP_INT_STS_0 "(\$val != 0)" IDLE_CHK_WARNING "PXP: P0 Interrupt status is not 0"
	# Read register PXP_REG_PXP_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP_REG_PXP_INT_STS_1 "(\$val != 0)" IDLE_CHK_WARNING "PXP: P1 Interrupt status is not 0"
	# Read register PGLUE_B_REG_PGLUE_B_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_PGLUE_B_INT_STS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Interrupt status is not 0"
	# Read register DORQ_REG_RSPA_CRD_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F DORQ_REG_RSPA_CRD_CNT "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "DORQ: Credit to XCM is not full"
	# Read register DORQ_REG_RSPB_CRD_CNT val and check if condition on val exist
	IDLE_CHK_1 0x1F DORQ_REG_RSPB_CRD_CNT "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "DORQ: Credit to UCM is not full"
	# Read register QM_REG_VOQCRDERRREG val and check if condition on val exist
	IDLE_CHK_1 0x3 QM_REG_VOQCRDERRREG "(\$val != 0)" IDLE_CHK_ERROR "QM: Credit error register is not 0 (byte or credit overflow/underflow)"
	# Read register DORQ_REG_DQ_FULL_ST val and check if condition on val exist
	IDLE_CHK_1 0x1F DORQ_REG_DQ_FULL_ST "(\$val != 0)" IDLE_CHK_ERROR "DORQ: DORQ queue is full"
	# Read register MISC_REG_AEU_AFTER_INVERT_1_FUNC_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F MISC_REG_AEU_AFTER_INVERT_1_FUNC_0 "((\$val & ~0xCFFC) != 0)" IDLE_CHK_WARNING "AEU: P0 AFTER_INVERT_1 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_2_FUNC_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F MISC_REG_AEU_AFTER_INVERT_2_FUNC_0 "(\$val != 0)" IDLE_CHK_ERROR "AEU: P0 AFTER_INVERT_2 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_3_FUNC_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F MISC_REG_AEU_AFTER_INVERT_3_FUNC_0 "((\$val & ~0xFFFF0000) != 0)" IDLE_CHK_ERROR "AEU: P0 AFTER_INVERT_3 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_4_FUNC_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F MISC_REG_AEU_AFTER_INVERT_4_FUNC_0 "((\$val & ~0x801FFFFF) != 0)" IDLE_CHK_ERROR "AEU: P0 AFTER_INVERT_4 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_1_FUNC_1 val and check if condition on val exist
	IDLE_CHK_1 0x3 MISC_REG_AEU_AFTER_INVERT_1_FUNC_1 "((\$val & ~0xCFFC) != 0)" IDLE_CHK_WARNING "AEU: P1 AFTER_INVERT_1 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_2_FUNC_1 val and check if condition on val exist
	IDLE_CHK_1 0x3 MISC_REG_AEU_AFTER_INVERT_2_FUNC_1 "(\$val != 0)" IDLE_CHK_ERROR "AEU: P1 AFTER_INVERT_2 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_3_FUNC_1 val and check if condition on val exist
	IDLE_CHK_1 0x3 MISC_REG_AEU_AFTER_INVERT_3_FUNC_1 "((\$val & ~0xFFFF0000) != 0)" IDLE_CHK_ERROR "AEU: P1 AFTER_INVERT_3 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_4_FUNC_1 val and check if condition on val exist
	IDLE_CHK_1 0x3 MISC_REG_AEU_AFTER_INVERT_4_FUNC_1 "((\$val & ~0x801FFFFF) != 0)" IDLE_CHK_ERROR "AEU: P1 AFTER_INVERT_4 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_1_MCP val and check if condition on val exist
	IDLE_CHK_1 0x1F MISC_REG_AEU_AFTER_INVERT_1_MCP "((\$val & ~0xCFFC) != 0)" IDLE_CHK_WARNING "AEU: MCP AFTER_INVERT_1 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_2_MCP val and check if condition on val exist
	IDLE_CHK_1 0x1F MISC_REG_AEU_AFTER_INVERT_2_MCP "(\$val != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_2 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_3_MCP val and check if condition on val exist
	IDLE_CHK_1 0x1F MISC_REG_AEU_AFTER_INVERT_3_MCP "((\$val & ~0xFFFF0000) != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_3 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_4_MCP val and check if condition on val exist
	IDLE_CHK_1 0x1F MISC_REG_AEU_AFTER_INVERT_4_MCP "((\$val & ~0x801FFFFF) != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_4 is not 0"
	# If register PBF_REG_DISABLE_NEW_TASK_PROC_P0 is valid, read PBF_REG_P0_CREDIT val1 and register PBF_REG_P0_INIT_CRD val2 and check if codition on val1,val2 exist
	IDLE_CHK_5 0xF PBF_REG_DISABLE_NEW_TASK_PROC_P0 PBF_REG_P0_CREDIT PBF_REG_P0_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: P0 credit is not equal to init_crd"
	# If register PBF_REG_DISABLE_NEW_TASK_PROC_P1 is valid, read PBF_REG_P1_CREDIT val1 and register PBF_REG_P1_INIT_CRD val2 and check if codition on val1,val2 exist
	IDLE_CHK_5 0xF PBF_REG_DISABLE_NEW_TASK_PROC_P1 PBF_REG_P1_CREDIT PBF_REG_P1_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: P1 credit is not equal to init_crd"
	# Read register PBF_REG_P4_CREDIT val1 and register PBF_REG_P4_INIT_CRD val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0xF PBF_REG_P4_CREDIT PBF_REG_P4_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: P4 credit is not equal to init_crd"
	# If register PBF_REG_DISABLE_NEW_TASK_PROC_Q0 is valid, read PBF_REG_CREDIT_Q0 val1 and register PBF_REG_INIT_CRD_Q0 val2 and check if codition on val1,val2 exist
	IDLE_CHK_5 0x10 PBF_REG_DISABLE_NEW_TASK_PROC_Q0 PBF_REG_CREDIT_Q0 PBF_REG_INIT_CRD_Q0 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q0 credit is not equal to init_crd"
	# If register PBF_REG_DISABLE_NEW_TASK_PROC_Q1 is valid, read PBF_REG_CREDIT_Q1 val1 and register PBF_REG_INIT_CRD_Q1 val2 and check if codition on val1,val2 exist
	IDLE_CHK_5 0x10 PBF_REG_DISABLE_NEW_TASK_PROC_Q1 PBF_REG_CREDIT_Q1 PBF_REG_INIT_CRD_Q1 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q1 credit is not equal to init_crd"
	# If register PBF_REG_DISABLE_NEW_TASK_PROC_Q2 is valid, read PBF_REG_CREDIT_Q2 val1 and register PBF_REG_INIT_CRD_Q2 val2 and check if codition on val1,val2 exist
	IDLE_CHK_5 0x10 PBF_REG_DISABLE_NEW_TASK_PROC_Q2 PBF_REG_CREDIT_Q2 PBF_REG_INIT_CRD_Q2 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q2 credit is not equal to init_crd"
	# If register PBF_REG_DISABLE_NEW_TASK_PROC_Q3 is valid, read PBF_REG_CREDIT_Q3 val1 and register PBF_REG_INIT_CRD_Q3 val2 and check if codition on val1,val2 exist
	IDLE_CHK_5 0x10 PBF_REG_DISABLE_NEW_TASK_PROC_Q3 PBF_REG_CREDIT_Q3 PBF_REG_INIT_CRD_Q3 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q3 credit is not equal to init_crd"
	# If register PBF_REG_DISABLE_NEW_TASK_PROC_Q4 is valid, read PBF_REG_CREDIT_Q4 val1 and register PBF_REG_INIT_CRD_Q4 val2 and check if codition on val1,val2 exist
	IDLE_CHK_5 0x10 PBF_REG_DISABLE_NEW_TASK_PROC_Q4 PBF_REG_CREDIT_Q4 PBF_REG_INIT_CRD_Q4 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q4 credit is not equal to init_crd"
	# If register PBF_REG_DISABLE_NEW_TASK_PROC_Q5 is valid, read PBF_REG_CREDIT_Q5 val1 and register PBF_REG_INIT_CRD_Q5 val2 and check if codition on val1,val2 exist
	IDLE_CHK_5 0x10 PBF_REG_DISABLE_NEW_TASK_PROC_Q5 PBF_REG_CREDIT_Q5 PBF_REG_INIT_CRD_Q5 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q5 credit is not equal to init_crd"
	# Read register PBF_REG_CREDIT_LB_Q val1 and register PBF_REG_INIT_CRD_LB_Q val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 PBF_REG_CREDIT_LB_Q PBF_REG_INIT_CRD_LB_Q "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: LB Q credit is not equal to init_crd"
	# Read register PBF_REG_P0_TASK_CNT val and check if condition on val exist
	IDLE_CHK_1 0xF PBF_REG_P0_TASK_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: P0 task_cnt is not 0"
	# Read register PBF_REG_P1_TASK_CNT val and check if condition on val exist
	IDLE_CHK_1 0xF PBF_REG_P1_TASK_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: P1 task_cnt is not 0"
	# Read register PBF_REG_P4_TASK_CNT val and check if condition on val exist
	IDLE_CHK_1 0xF PBF_REG_P4_TASK_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: P4 task_cnt is not 0"
	# Read register PBF_REG_TASK_CNT_Q0 val and check if condition on val exist
	IDLE_CHK_1 0x10 PBF_REG_TASK_CNT_Q0 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q0 task_cnt is not 0"
	# Read register PBF_REG_TASK_CNT_Q1 val and check if condition on val exist
	IDLE_CHK_1 0x10 PBF_REG_TASK_CNT_Q1 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q1 task_cnt is not 0"
	# Read register PBF_REG_TASK_CNT_Q2 val and check if condition on val exist
	IDLE_CHK_1 0x10 PBF_REG_TASK_CNT_Q2 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q2 task_cnt is not 0"
	# Read register PBF_REG_TASK_CNT_Q3 val and check if condition on val exist
	IDLE_CHK_1 0x10 PBF_REG_TASK_CNT_Q3 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q3 task_cnt is not 0"
	# Read register PBF_REG_TASK_CNT_Q4 val and check if condition on val exist
	IDLE_CHK_1 0x10 PBF_REG_TASK_CNT_Q4 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q4 task_cnt is not 0"
	# Read register PBF_REG_TASK_CNT_Q5 val and check if condition on val exist
	IDLE_CHK_1 0x10 PBF_REG_TASK_CNT_Q5 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Q5 task_cnt is not 0"
	# Read register PBF_REG_TASK_CNT_LB_Q val and check if condition on val exist
	IDLE_CHK_1 0x10 PBF_REG_TASK_CNT_LB_Q "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: LB Q task_cnt is not 0"
	# Read register XCM_REG_CFC_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_CFC_INIT_CRD "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XCM: CFC_INIT_CRD is not 1"
	# Read register UCM_REG_CFC_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_CFC_INIT_CRD "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "UCM: CFC_INIT_CRD is not 1"
	# Read register TCM_REG_CFC_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_CFC_INIT_CRD "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TCM: CFC_INIT_CRD is not 1"
	# Read register CCM_REG_CFC_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_CFC_INIT_CRD "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "CCM: CFC_INIT_CRD is not 1"
	# Read register XCM_REG_XQM_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_XQM_INIT_CRD "(\$val != 32)" IDLE_CHK_ERROR_NO_TRAFFIC "XCM: XQM_INIT_CRD is not 32"
	# Read register UCM_REG_UQM_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_UQM_INIT_CRD "(\$val != 32)" IDLE_CHK_ERROR_NO_TRAFFIC "UCM: UQM_INIT_CRD is not 32"
	# Read register TCM_REG_TQM_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_TQM_INIT_CRD "(\$val != 32)" IDLE_CHK_ERROR_NO_TRAFFIC "TCM: TQM_INIT_CRD is not 32"
	# Read register CCM_REG_CQM_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_CQM_INIT_CRD "(\$val != 32)" IDLE_CHK_ERROR_NO_TRAFFIC "CCM: CQM_INIT_CRD is not 32"
	# Read register XCM_REG_TM_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_TM_INIT_CRD "(\$val != 4)" IDLE_CHK_ERROR_NO_TRAFFIC "XCM: TM_INIT_CRD is not 4"
	# Read register UCM_REG_TM_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_TM_INIT_CRD "(\$val != 4)" IDLE_CHK_ERROR_NO_TRAFFIC "UCM: TM_INIT_CRD is not 4"
	# Read register XCM_REG_FIC0_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_FIC0_INIT_CRD "(\$val != 64)" IDLE_CHK_WARNING "XCM: FIC0_INIT_CRD is not 64"
	# Read register UCM_REG_FIC0_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_FIC0_INIT_CRD "(\$val != 64)" IDLE_CHK_ERROR_NO_TRAFFIC "UCM: FIC0_INIT_CRD is not 64"
	# Read register TCM_REG_FIC0_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_FIC0_INIT_CRD "(\$val != 64)" IDLE_CHK_ERROR_NO_TRAFFIC "TCM: FIC0_INIT_CRD is not 64"
	# Read register CCM_REG_FIC0_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_FIC0_INIT_CRD "(\$val != 64)" IDLE_CHK_ERROR_NO_TRAFFIC "CCM: FIC0_INIT_CRD is not 64"
	# Read register XCM_REG_FIC1_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_FIC1_INIT_CRD "(\$val != 64)" IDLE_CHK_ERROR_NO_TRAFFIC "XCM: FIC1_INIT_CRD is not 64"
	# Read register UCM_REG_FIC1_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_FIC1_INIT_CRD "(\$val != 64)" IDLE_CHK_ERROR_NO_TRAFFIC "UCM: FIC1_INIT_CRD is not 64"
	# Read register TCM_REG_FIC1_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_FIC1_INIT_CRD "(\$val != 64)" IDLE_CHK_ERROR_NO_TRAFFIC "TCM: FIC1_INIT_CRD is not 64"
	# Read register CCM_REG_FIC1_INIT_CRD val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_FIC1_INIT_CRD "(\$val != 64)" IDLE_CHK_ERROR_NO_TRAFFIC "CCM: FIC1_INIT_CRD is not 64"
	# Read register XCM_REG_XX_FREE val and check if condition on val exist
	IDLE_CHK_1 0x1 XCM_REG_XX_FREE "(\$val != 31)" IDLE_CHK_ERROR_NO_TRAFFIC "XCM: XX_FREE differs from expected 31"
	# Read register XCM_REG_XX_FREE val and check if condition on val exist
	IDLE_CHK_1 0x1E XCM_REG_XX_FREE "(\$val != 32)" IDLE_CHK_ERROR_NO_TRAFFIC "XCM: XX_FREE differs from expected 32"
	# Read register UCM_REG_XX_FREE val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_XX_FREE "(\$val != 27)" IDLE_CHK_ERROR_NO_TRAFFIC "UCM: XX_FREE differs from expected 27"
	# Read register TCM_REG_XX_FREE val and check if condition on val exist
	IDLE_CHK_1 0x7 TCM_REG_XX_FREE "(\$val != 32)" IDLE_CHK_ERROR_NO_TRAFFIC "TCM: XX_FREE differs from expected 32"
	# Read register TCM_REG_XX_FREE val and check if condition on val exist
	IDLE_CHK_1 0x18 TCM_REG_XX_FREE "(\$val != 29)" IDLE_CHK_ERROR_NO_TRAFFIC "TCM: XX_FREE differs from expected 29"
	# Read register CCM_REG_XX_FREE val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_XX_FREE "(\$val != 24)" IDLE_CHK_ERROR_NO_TRAFFIC "CCM: XX_FREE differs from expected 24"
	# Read register "XSEM_REG_FAST_MEMORY + 0x18000" val and check if condition on val exist
	IDLE_CHK_1 0x1F "XSEM_REG_FAST_MEMORY + 0x18000" "(\$val !=  0)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: FOC0 credit less than initial credit"
	# Read register "XSEM_REG_FAST_MEMORY + 0x18040" val and check if condition on val exist
	IDLE_CHK_1 0x1F "XSEM_REG_FAST_MEMORY + 0x18040" "(\$val !=  24)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: FOC1 credit less than initial credit"
	# Read register "XSEM_REG_FAST_MEMORY + 0x18080" val and check if condition on val exist
	IDLE_CHK_1 0x1F "XSEM_REG_FAST_MEMORY + 0x18080" "(\$val !=  12)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: FOC2 credit less than initial credit"
	# Read register "USEM_REG_FAST_MEMORY + 0x18000" val and check if condition on val exist
	IDLE_CHK_1 0x1F "USEM_REG_FAST_MEMORY + 0x18000" "(\$val != 26)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: FOC0 credit less than initial credit"
	# Read register "USEM_REG_FAST_MEMORY + 0x18040" val and check if condition on val exist
	IDLE_CHK_1 0x1F "USEM_REG_FAST_MEMORY + 0x18040" "(\$val != 78)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: FOC1 credit less than initial credit"
	# Read register "USEM_REG_FAST_MEMORY + 0x18080" val and check if condition on val exist
	IDLE_CHK_1 0x1F "USEM_REG_FAST_MEMORY + 0x18080" "(\$val != 16)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: FOC2 credit less than initial credit"
	# Read register "USEM_REG_FAST_MEMORY + 0x180C0" val and check if condition on val exist
	IDLE_CHK_1 0x1F "USEM_REG_FAST_MEMORY + 0x180C0" "(\$val != 32)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: FOC3 credit less than initial credit"
	# Read register "TSEM_REG_FAST_MEMORY + 0x18000" val and check if condition on val exist
	IDLE_CHK_1 0x1F "TSEM_REG_FAST_MEMORY + 0x18000" "(\$val != 52)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: FOC0 credit less than initial credit"
	# Read register "TSEM_REG_FAST_MEMORY + 0x18040" val and check if condition on val exist
	IDLE_CHK_1 0x1F "TSEM_REG_FAST_MEMORY + 0x18040" "(\$val != 24)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: FOC1 credit less than initial credit"
	# Read register "TSEM_REG_FAST_MEMORY + 0x18080" val and check if condition on val exist
	IDLE_CHK_1 0x1F "TSEM_REG_FAST_MEMORY + 0x18080" "(\$val != 12)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: FOC2 credit less than initial credit"
	# Read register "TSEM_REG_FAST_MEMORY + 0x180C0" val and check if condition on val exist
	IDLE_CHK_1 0x1F "TSEM_REG_FAST_MEMORY + 0x180C0" "(\$val != 32)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: FOC3 credit less than initial credit"
	# Read register "CSEM_REG_FAST_MEMORY + 0x18000" val and check if condition on val exist
	IDLE_CHK_1 0x1F "CSEM_REG_FAST_MEMORY + 0x18000" "(\$val != 16)" IDLE_CHK_ERROR_NO_TRAFFIC "CSEM: FOC0 credit less than initial credit"
	# Read register "CSEM_REG_FAST_MEMORY + 0x18040" val and check if condition on val exist
	IDLE_CHK_1 0x1F "CSEM_REG_FAST_MEMORY + 0x18040" "(\$val != 18)" IDLE_CHK_ERROR_NO_TRAFFIC "CSEM: FOC1 credit less than initial credit"
	# Read register "CSEM_REG_FAST_MEMORY + 0x18080" val and check if condition on val exist
	IDLE_CHK_1 0x1F "CSEM_REG_FAST_MEMORY + 0x18080" "(\$val != 48)" IDLE_CHK_ERROR_NO_TRAFFIC "CSEM: FOC2 credit less than initial credit"
	# Read register "CSEM_REG_FAST_MEMORY + 0x180C0" val and check if condition on val exist
	IDLE_CHK_1 0x1F "CSEM_REG_FAST_MEMORY + 0x180C0" "(\$val != 14)" IDLE_CHK_ERROR_NO_TRAFFIC "CSEM: FOC3 credit less than initial credit"
	# Read register PRS_REG_TSDM_CURRENT_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x1F PRS_REG_TSDM_CURRENT_CREDIT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PRS: TSDM current credit is not 0"
	# Read register PRS_REG_TCM_CURRENT_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x1F PRS_REG_TCM_CURRENT_CREDIT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PRS: TCM current credit is not 0"
	# Read register PRS_REG_CFC_LD_CURRENT_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x1F PRS_REG_CFC_LD_CURRENT_CREDIT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PRS: CFC_LD current credit is not 0"
	# Read register PRS_REG_CFC_SEARCH_CURRENT_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x1F PRS_REG_CFC_SEARCH_CURRENT_CREDIT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PRS: CFC_SEARCH current credit is not 0"
	# Read register PRS_REG_SRC_CURRENT_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x1F PRS_REG_SRC_CURRENT_CREDIT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PRS: SRCH current credit is not 0"
	# Read register PRS_REG_PENDING_BRB_PRS_RQ val and check if condition on val exist
	IDLE_CHK_1 0x1F PRS_REG_PENDING_BRB_PRS_RQ "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PRS: PENDING_BRB_PRS_RQ is not 0"
	# Read register PRS_REG_PENDING_BRB_CAC0_RQ val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x1F PRS_REG_PENDING_BRB_CAC0_RQ 5 4 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PRS: PENDING_BRB_CAC_RQ is not 0"
	# Read register PRS_REG_SERIAL_NUM_STATUS_LSB val and check if condition on val exist
	IDLE_CHK_1 0x1F PRS_REG_SERIAL_NUM_STATUS_LSB "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PRS: SERIAL_NUM_STATUS_LSB is not 0"
	# Read register PRS_REG_SERIAL_NUM_STATUS_MSB val and check if condition on val exist
	IDLE_CHK_1 0x1F PRS_REG_SERIAL_NUM_STATUS_MSB "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PRS: SERIAL_NUM_STATUS_MSB is not 0"
	# Read register CDU_REG_ERROR_DATA val and check if condition on val exist
	IDLE_CHK_1 0x1F CDU_REG_ERROR_DATA "(\$val != 0)" IDLE_CHK_ERROR "CDU: ERROR_DATA is not 0"
	# Read register CCM_REG_STORM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_STORM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "CCM: STORM declared message length unequal to actual"
	# Read register CCM_REG_CSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_CSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "CCM: CSDM declared message length unequal to actual"
	# Read register CCM_REG_TSEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_TSEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "CCM: TSEM declared message length unequal to actual"
	# Read register CCM_REG_XSEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_XSEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "CCM: XSEM declared message length unequal to actual"
	# Read register CCM_REG_USEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_USEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "CCM: USEM declared message length unequal to actual"
	# Read register CCM_REG_PBF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F CCM_REG_PBF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "CCM: PBF declared message length unequal to actual"
	# Read register TCM_REG_STORM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_STORM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "TCM: STORM declared message length unequal to actual"
	# Read register TCM_REG_TSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_TSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "TCM: TSDM declared message length unequal to actual"
	# Read register TCM_REG_PRS_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_PRS_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "TCM: PRS declared message length unequal to actual"
	# Read register TCM_REG_PBF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_PBF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "TCM: PBF declared message length unequal to actual"
	# Read register TCM_REG_USEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_USEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "TCM: USEM declared message length unequal to actual"
	# Read register TCM_REG_CSEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F TCM_REG_CSEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "TCM: CSEM declared message length unequal to actual"
	# Read register UCM_REG_STORM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_STORM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: STORM declared message length unequal to actual"
	# Read register UCM_REG_USDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_USDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: USDM declared message length unequal to actual"
	# Read register UCM_REG_TSEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_TSEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: TSEM declared message length unequal to actual"
	# Read register UCM_REG_CSEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_CSEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: CSEM declared message length unequal to actual"
	# Read register UCM_REG_XSEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_XSEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: XSEM declared message length unequal to actual"
	# Read register UCM_REG_DORQ_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F UCM_REG_DORQ_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: DORQ declared message length unequal to actual"
	# Read register XCM_REG_STORM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_STORM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: STORM declared message length unequal to actual"
	# Read register XCM_REG_XSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_XSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: XSDM declared message length unequal to actual"
	# Read register XCM_REG_TSEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_TSEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: TSEM declared message length unequal to actual"
	# Read register XCM_REG_CSEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_CSEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: CSEM declared message length unequal to actual"
	# Read register XCM_REG_USEM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_USEM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: USEM declared message length unequal to actual"
	# Read register XCM_REG_DORQ_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_DORQ_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: DORQ declared message length unequal to actual"
	# Read register XCM_REG_PBF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_PBF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: PBF declared message length unequal to actual"
	# Read register XCM_REG_NIG0_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_NIG0_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: NIG0 declared message length unequal to actual"
	# Read register XCM_REG_NIG1_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x1F XCM_REG_NIG1_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: NIG1 declared message length unequal to actual"
	# Read register QM_REG_XQM_WRC_FIFOLVL val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_XQM_WRC_FIFOLVL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: XQM wrc_fifolvl is not 0"
	# Read register QM_REG_UQM_WRC_FIFOLVL val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_UQM_WRC_FIFOLVL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: UQM wrc_fifolvl is not 0"
	# Read register QM_REG_TQM_WRC_FIFOLVL val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_TQM_WRC_FIFOLVL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: TQM wrc_fifolvl is not 0"
	# Read register QM_REG_CQM_WRC_FIFOLVL val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_CQM_WRC_FIFOLVL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: CQM wrc_fifolvl is not 0"
	# Read register QM_REG_QSTATUS_LOW val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_QSTATUS_LOW "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: QSTATUS_LOW is not 0"
	# Read register QM_REG_QSTATUS_HIGH val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_QSTATUS_HIGH "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: QSTATUS_HIGH is not 0"
	# Read register QM_REG_PAUSESTATE0 val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_PAUSESTATE0 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: PAUSESTATE0 is not 0"
	# Read register QM_REG_PAUSESTATE1 val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_PAUSESTATE1 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: PAUSESTATE1 is not 0"
	# Read register QM_REG_OVFQNUM val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_OVFQNUM "(\$val != 0)" IDLE_CHK_ERROR "QM: OVFQNUM is not 0"
	# Read register QM_REG_OVFERROR val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_OVFERROR "(\$val != 0)" IDLE_CHK_ERROR "QM: OVFERROR is not 0"
	# Special check for QM PTRTBL
	IDLE_CHK_6 0x1F QM_REG_PTRTBL 64 8 IDLE_CHK_ERROR_NO_TRAFFIC
	# Read register BRB1_REG_BRB1_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F BRB1_REG_BRB1_PRTY_STS "((\$val & ~ 0x8) != 0)" IDLE_CHK_WARNING "BRB1: parity status is not 0"
	# Read register CDU_REG_CDU_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F CDU_REG_CDU_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "CDU: parity status is not 0"
	# Read register CFC_REG_CFC_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F CFC_REG_CFC_PRTY_STS "((\$val  & ~0x2)!= 0)" IDLE_CHK_WARNING "CFC: parity status is not 0"
	# Read register CSDM_REG_CSDM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F CSDM_REG_CSDM_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "CSDM: parity status is not 0"
	# Read register DBG_REG_DBG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3 DBG_REG_DBG_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "DBG: parity status is not 0"
	# Read register DMAE_REG_DMAE_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F DMAE_REG_DMAE_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "DMAE: parity status is not 0"
	# Read register DORQ_REG_DORQ_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F DORQ_REG_DORQ_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "DORQ: parity status is not 0"
	# Read register TCM_REG_TCM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1 TCM_REG_TCM_PRTY_STS "((\$val & ~0x3ffc0) != 0)" IDLE_CHK_WARNING "TCM: parity status is not 0"
	# Read register TCM_REG_TCM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1E TCM_REG_TCM_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "TCM: parity status is not 0"
	# Read register CCM_REG_CCM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1 CCM_REG_CCM_PRTY_STS "((\$val & ~0x3ffc0) != 0)" IDLE_CHK_WARNING "CCM: parity status is not 0"
	# Read register CCM_REG_CCM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1E CCM_REG_CCM_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "CCM: parity status is not 0"
	# Read register UCM_REG_UCM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1 UCM_REG_UCM_PRTY_STS "((\$val & ~0x3ffc0) != 0)" IDLE_CHK_WARNING "UCM: parity status is not 0"
	# Read register UCM_REG_UCM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1E UCM_REG_UCM_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "UCM: parity status is not 0"
	# Read register XCM_REG_XCM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1 XCM_REG_XCM_PRTY_STS "((\$val & ~0x3ffc0) != 0)" IDLE_CHK_WARNING "XCM: parity status is not 0"
	# Read register XCM_REG_XCM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1E XCM_REG_XCM_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "XCM: parity status is not 0"
	# Read register HC_REG_HC_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1 HC_REG_HC_PRTY_STS "((\$val& ~0x1) != 0)" IDLE_CHK_WARNING "HC: parity status is not 0"
	# Read register MISC_REG_MISC_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1 MISC_REG_MISC_PRTY_STS "((\$val& ~0x1) != 0)" IDLE_CHK_WARNING "MISC: parity status is not 0"
	# Read register PRS_REG_PRS_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F PRS_REG_PRS_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "PRS: parity status is not 0"
	# Read register PXP_REG_PXP_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F PXP_REG_PXP_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "PXP: parity status is not 0"
	# Read register QM_REG_QM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F QM_REG_QM_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "QM: parity status is not 0"
	# Read register SRC_REG_SRC_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1 SRC_REG_SRC_PRTY_STS "((\$val & ~0x4) != 0)" IDLE_CHK_WARNING "SRCH: parity status is not 0"
	# Read register TSDM_REG_TSDM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F TSDM_REG_TSDM_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "TSDM: parity status is not 0"
	# Read register USDM_REG_USDM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F USDM_REG_USDM_PRTY_STS "((\$val & ~0x20) != 0)" IDLE_CHK_WARNING "USDM: parity status is not 0"
	# Read register XSDM_REG_XSDM_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F XSDM_REG_XSDM_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "XSDM: parity status is not 0"
	# Read register XPB_REG_PB_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F XPB_REG_PB_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "XPB: parity status is not 0"
	# Read register UPB_REG_PB_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1F UPB_REG_PB_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "UPB: parity status is not 0"
	# Read register CSEM_REG_CSEM_PRTY_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F CSEM_REG_CSEM_PRTY_STS_0 "(\$val != 0)" IDLE_CHK_WARNING "CSEM: parity status 0 is not 0"
	# Read register PXP2_REG_PXP2_PRTY_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1 PXP2_REG_PXP2_PRTY_STS_0 "((\$val & ~0xfff40020) != 0) " IDLE_CHK_WARNING "PXP2: parity status 0 is not 0"
	# Read register PXP2_REG_PXP2_PRTY_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1E PXP2_REG_PXP2_PRTY_STS_0 "((\$val & ~0x20) != 0)" IDLE_CHK_WARNING "PXP2: parity status 0 is not 0"
	# Read register TSEM_REG_TSEM_PRTY_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F TSEM_REG_TSEM_PRTY_STS_0 "(\$val != 0)" IDLE_CHK_WARNING "TSEM: parity status 0 is not 0"
	# Read register USEM_REG_USEM_PRTY_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F USEM_REG_USEM_PRTY_STS_0 "(\$val != 0)" IDLE_CHK_WARNING "USEM: parity status 0 is not 0"
	# Read register XSEM_REG_XSEM_PRTY_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F XSEM_REG_XSEM_PRTY_STS_0 "(\$val != 0)" IDLE_CHK_WARNING "XSEM: parity status 0 is not 0"
	# Read register CSEM_REG_CSEM_PRTY_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F CSEM_REG_CSEM_PRTY_STS_1 "(\$val != 0)" IDLE_CHK_WARNING "CSEM: parity status 1 is not 0"
	# Read register PXP2_REG_PXP2_PRTY_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1 PXP2_REG_PXP2_PRTY_STS_1 "((\$val & ~0x20) != 0)" IDLE_CHK_WARNING "PXP2: parity status 1 is not 0"
	# Read register PXP2_REG_PXP2_PRTY_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1E PXP2_REG_PXP2_PRTY_STS_1 "(\$val != 0)" IDLE_CHK_WARNING "PXP2: parity status 1 is not 0"
	# Read register TSEM_REG_TSEM_PRTY_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F TSEM_REG_TSEM_PRTY_STS_1 "(\$val != 0)" IDLE_CHK_WARNING "TSEM: parity status 1 is not 0"
	# Read register USEM_REG_USEM_PRTY_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F USEM_REG_USEM_PRTY_STS_1 "(\$val != 0)" IDLE_CHK_WARNING "USEM: parity status 1 is not 0"
	# Read register XSEM_REG_XSEM_PRTY_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F XSEM_REG_XSEM_PRTY_STS_1 "(\$val != 0)" IDLE_CHK_WARNING "XSEM: parity status 1 is not 0"
	# Read register PGLUE_B_REG_PGLUE_B_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_PGLUE_B_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: parity status is not 0"
	# Read register QM_REG_QTASKCTR_EXT_A_0 val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x2 QM_REG_QTASKCTR_EXT_A_0 64 4 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Q_EXT_A (upper 64 queues), Queue is not empty"
	# Read register QM_REG_QSTATUS_LOW_EXT_A val and check if condition on val exist
	IDLE_CHK_1 0x2 QM_REG_QSTATUS_LOW_EXT_A "(\$val != 0)" IDLE_CHK_ERROR "QM: QSTATUS_LOW_EXT_A is not 0"
	# Read register QM_REG_QSTATUS_HIGH_EXT_A val and check if condition on val exist
	IDLE_CHK_1 0x2 QM_REG_QSTATUS_HIGH_EXT_A "(\$val != 0)" IDLE_CHK_ERROR "QM: QSTATUS_HIGH_EXT_A is not 0"
	# Read register QM_REG_PAUSESTATE2 val and check if condition on val exist
	IDLE_CHK_1 0x1E QM_REG_PAUSESTATE2 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: PAUSESTATE2 is not 0"
	# Read register QM_REG_PAUSESTATE3 val and check if condition on val exist
	IDLE_CHK_1 0x1E QM_REG_PAUSESTATE3 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: PAUSESTATE3 is not 0"
	# Read register QM_REG_PAUSESTATE4 val and check if condition on val exist
	IDLE_CHK_1 0x2 QM_REG_PAUSESTATE4 "(\$val != 0)" IDLE_CHK_ERROR "QM: PAUSESTATE4 is not 0"
	# Read register QM_REG_PAUSESTATE5 val and check if condition on val exist
	IDLE_CHK_1 0x2 QM_REG_PAUSESTATE5 "(\$val != 0)" IDLE_CHK_ERROR "QM: PAUSESTATE5 is not 0"
	# Read register QM_REG_PAUSESTATE6 val and check if condition on val exist
	IDLE_CHK_1 0x2 QM_REG_PAUSESTATE6 "(\$val != 0)" IDLE_CHK_ERROR "QM: PAUSESTATE6 is not 0"
	# Read register QM_REG_PAUSESTATE7 val and check if condition on val exist
	IDLE_CHK_1 0x2 QM_REG_PAUSESTATE7 "(\$val != 0)" IDLE_CHK_ERROR "QM: PAUSESTATE7 is not 0"
	# Special check for QM PTRTBL
	IDLE_CHK_6 0x2 QM_REG_PTRTBL_EXT_A 64 8 IDLE_CHK_ERROR_NO_TRAFFIC
	# Read register MISC_REG_AEU_SYS_KILL_OCCURRED val and check if condition on val exist
	IDLE_CHK_1 0x1E MISC_REG_AEU_SYS_KILL_OCCURRED "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occurd;"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1E MISC_REG_AEU_SYS_KILL_STATUS_0 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occurd; status_0 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1E MISC_REG_AEU_SYS_KILL_STATUS_1 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occurd; status_1 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_2 val and check if condition on val exist
	IDLE_CHK_1 0x1E MISC_REG_AEU_SYS_KILL_STATUS_2 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occurd; status_2 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_3 val and check if condition on val exist
	IDLE_CHK_1 0x1E MISC_REG_AEU_SYS_KILL_STATUS_3 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occurd; status_3 register"
	# Read register MISC_REG_PCIE_HOT_RESET val and check if condition on val exist
	IDLE_CHK_1 0x1E MISC_REG_PCIE_HOT_RESET "(\$val != 0)" IDLE_CHK_WARNING "MISC: pcie_rst_b was asserted without perst assertion"
	# Read register NIG_REG_NIG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_NIG_INT_STS_0 "((\$val & ~0x300) != 0)" IDLE_CHK_ERROR "NIG: interrupt 0 is active"
	# Read register NIG_REG_NIG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_NIG_INT_STS_0 "(\$val == 0x300)" IDLE_CHK_WARNING "NIG: Access to BMAC while not active. If tested on FPGA, ignore this warning."
	# Read register NIG_REG_NIG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_NIG_INT_STS_1 "((\$val & 0x783FF03) != 0)" IDLE_CHK_ERROR "NIG: interrupt 1 is active"
	# Read register NIG_REG_NIG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_NIG_INT_STS_1 "((\$val & ~0x783FF0F) != 0)" IDLE_CHK_WARNING "NIG: port cos was paused too long"
	# Read register NIG_REG_NIG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_NIG_INT_STS_1 "((\$val & 0xC) != 0)" IDLE_CHK_WARNING "NIG: Got packets w/o Outer-VLAN in MF mode"
	# Read register NIG_REG_NIG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x2 NIG_REG_NIG_PRTY_STS "((\$val & ~0xFFC00000) != 0)" IDLE_CHK_ERROR "NIG: parity interrupt is active"
	# Read register NIG_REG_NIG_PRTY_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1C NIG_REG_NIG_PRTY_STS_0 "((\$val & ~0xFFC00000) != 0)" IDLE_CHK_ERROR "NIG: parity 0 interrupt is active"
	# Read register NIG_REG_NIG_PRTY_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x4 NIG_REG_NIG_PRTY_STS_1 "((\$val & 0xff) != 0)" IDLE_CHK_ERROR "NIG: parity 1 interrupt is active"
	# Read register NIG_REG_NIG_PRTY_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x18 NIG_REG_NIG_PRTY_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "NIG: parity 1 interrupt is active"
	# Read register TSEM_REG_TSEM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F TSEM_REG_TSEM_INT_STS_0 "((\$val & ~0x10000000) != 0)" IDLE_CHK_WARNING "TSEM: interrupt 0 is active"
	# Read register TSEM_REG_TSEM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F TSEM_REG_TSEM_INT_STS_0 "(\$val  == 0x10000000)" IDLE_CHK_WARNING "TSEM: interrupt 0 is active"
	# Read register TSEM_REG_TSEM_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F TSEM_REG_TSEM_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "TSEM: interrupt 1 is active"
	# Read register CSEM_REG_CSEM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F CSEM_REG_CSEM_INT_STS_0 "((\$val & ~0x10000000) != 0)" IDLE_CHK_WARNING "CSEM: interrupt 0 is active"
	# Read register CSEM_REG_CSEM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F CSEM_REG_CSEM_INT_STS_0 "(\$val  == 0x10000000)" IDLE_CHK_WARNING "CSEM: interrupt 0 is active"
	# Read register CSEM_REG_CSEM_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F CSEM_REG_CSEM_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "CSEM: interrupt 1 is active"
	# Read register USEM_REG_USEM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F USEM_REG_USEM_INT_STS_0 "((\$val & ~0x10000000) != 0)" IDLE_CHK_WARNING "USEM: interrupt 0 is active"
	# Read register USEM_REG_USEM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F USEM_REG_USEM_INT_STS_0 "(\$val  == 0x10000000)" IDLE_CHK_WARNING "USEM: interrupt 0 is active"
	# Read register USEM_REG_USEM_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F USEM_REG_USEM_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "USEM: interrupt 1 is active"
	# Read register XSEM_REG_XSEM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F XSEM_REG_XSEM_INT_STS_0 "((\$val & ~0x10000000) != 0)" IDLE_CHK_WARNING "XSEM: interrupt 0 is active"
	# Read register XSEM_REG_XSEM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F XSEM_REG_XSEM_INT_STS_0 "(\$val  == 0x10000000)" IDLE_CHK_WARNING "XSEM: interrupt 0 is active"
	# Read register XSEM_REG_XSEM_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F XSEM_REG_XSEM_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "XSEM: interrupt 1 is active"
	# Read register TSDM_REG_TSDM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F TSDM_REG_TSDM_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "TSDM: interrupt 0 is active"
	# Read register TSDM_REG_TSDM_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F TSDM_REG_TSDM_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "TSDM: interrupt 0 is active"
	# Read register CSDM_REG_CSDM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F CSDM_REG_CSDM_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "CSDM: interrupt 0 is active"
	# Read register CSDM_REG_CSDM_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F CSDM_REG_CSDM_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "CSDM: interrupt 0 is active"
	# Read register USDM_REG_USDM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F USDM_REG_USDM_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "USDM: interrupt 0 is active"
	# Read register USDM_REG_USDM_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F USDM_REG_USDM_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "USDM: interrupt 0 is active"
	# Read register XSDM_REG_XSDM_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x1F XSDM_REG_XSDM_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "XSDM: interrupt 0 is active"
	# Read register XSDM_REG_XSDM_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x1F XSDM_REG_XSDM_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "XSDM: interrupt 0 is active"
	# Read register HC_REG_HC_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x2 HC_REG_HC_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "HC: parity status is not 0"
	# Read register MISC_REG_MISC_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1E MISC_REG_MISC_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "MISC: parity status is not 0"
	# Read register SRC_REG_SRC_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1E SRC_REG_SRC_PRTY_STS "(\$val  != 0)" IDLE_CHK_WARNING "SRCH: parity status is not 0"
	# Read register QM_REG_BYTECRD0 val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0xC QM_REG_BYTECRD0 QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Byte credit 0 is not equal to initial credit"
	# Read register QM_REG_BYTECRD1 val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0xC QM_REG_BYTECRD1 QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Byte credit 1 is not equal to initial credit"
	# Read register QM_REG_BYTECRD2 val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0xC QM_REG_BYTECRD2 QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Byte credit 2 is not equal to initial credit"
	# Read register QM_REG_VOQCRDERRREG val and check if condition on val exist
	IDLE_CHK_1 0x1C QM_REG_VOQCRDERRREG "((\$val & 0xFFFF) != 0)" IDLE_CHK_ERROR "QM: VOQ credit error register is not 0 (VOQ credit overflow/underflow)"
	# Read register QM_REG_BYTECRDERRREG val and check if condition on val exist
	IDLE_CHK_1 0x1C QM_REG_BYTECRDERRREG "((\$val & 0xFFF) != 0)" IDLE_CHK_ERROR "QM: Byte credit error register is not 0 (Byte credit overflow/underflow)"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_FLR_REQUEST_VF_31_0 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 31-0"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_FLR_REQUEST_VF_63_32 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 63-32"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_95_64 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_FLR_REQUEST_VF_95_64 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 95-64"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_127_96 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_FLR_REQUEST_VF_127_96 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 127-96"
	# Read register PGLUE_B_REG_FLR_REQUEST_PF_7_0 val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_FLR_REQUEST_PF_7_0 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for PF addresses 7-0"
	# Read register PGLUE_B_REG_SR_IOV_DISABLED_REQUEST val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_SR_IOV_DISABLED_REQUEST "(\$val  != 0)" IDLE_CHK_WARNING "PGL: SR-IOV disable request is set "
	# Read register PGLUE_B_REG_CFG_SPACE_A_REQUEST val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_CFG_SPACE_A_REQUEST "(\$val  != 0)" IDLE_CHK_WARNING "PGL: Cfg-Space A request is set"
	# Read register PGLUE_B_REG_CFG_SPACE_B_REQUEST val and check if condition on val exist
	IDLE_CHK_1 0x1C PGLUE_B_REG_CFG_SPACE_B_REQUEST "(\$val  != 0)" IDLE_CHK_WARNING "PGL: Cfg-Space B request is set"
	# Read register IGU_REG_ERROR_HANDLING_DATA_VALID val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_ERROR_HANDLING_DATA_VALID "(\$val  != 0)" IDLE_CHK_WARNING "IGU: some unauthorized commands arrived to the IGU. Use igu_dump_fifo utility for more details."
	# Read register IGU_REG_ATTN_WRITE_DONE_PENDING val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_ATTN_WRITE_DONE_PENDING "(\$val  != 0)" IDLE_CHK_WARNING "IGU attention message write done pending is not empty"
	# Read register IGU_REG_WRITE_DONE_PENDING val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_WRITE_DONE_PENDING "(\$val  != 0)" IDLE_CHK_WARNING "IGU MSI/MSIX message write done pending is not empty"
	# Read register IGU_REG_IGU_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_IGU_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "IGU: parity status is not 0"
	# Read register MISC_REG_GRC_TIMEOUT_ATTN val1 and register MISC_REG_AEU_AFTER_INVERT_4_FUNC_0 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1E MISC_REG_GRC_TIMEOUT_ATTN MISC_REG_AEU_AFTER_INVERT_4_FUNC_0 "((\$val & 0x4000000) != 0)" IDLE_CHK_ERROR "MISC_REG_GRC_TIMEOUT_ATTN: GRC timeout attention parameters (FUNC_0)."
	# Read register MISC_REG_GRC_TIMEOUT_ATTN_FULL_FID val1 and register MISC_REG_AEU_AFTER_INVERT_4_FUNC_0 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1C MISC_REG_GRC_TIMEOUT_ATTN_FULL_FID MISC_REG_AEU_AFTER_INVERT_4_FUNC_0 "((\$val & 0x4000000) != 0)" IDLE_CHK_ERROR "MISC_REG_GRC_TIMEOUT_ATTN_FULL_FID: GRC timeout attention FID (FUNC_0)."
	# Read register MISC_REG_GRC_TIMEOUT_ATTN val1 and register MISC_REG_AEU_AFTER_INVERT_4_FUNC_1 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1E MISC_REG_GRC_TIMEOUT_ATTN MISC_REG_AEU_AFTER_INVERT_4_FUNC_1 "((\$val & 0x4000000) != 0)" IDLE_CHK_ERROR "MISC_REG_GRC_TIMEOUT_ATTN: GRC timeout attention parameters (FUNC_1)."
	# Read register MISC_REG_GRC_TIMEOUT_ATTN_FULL_FID val1 and register MISC_REG_AEU_AFTER_INVERT_4_FUNC_1 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1C MISC_REG_GRC_TIMEOUT_ATTN_FULL_FID MISC_REG_AEU_AFTER_INVERT_4_FUNC_1 "((\$val & 0x4000000) != 0)" IDLE_CHK_ERROR "MISC_REG_GRC_TIMEOUT_ATTN_FULL_FID: GRC timeout attention FID (FUNC_1)."
	# Read register MISC_REG_GRC_TIMEOUT_ATTN val1 and register MISC_REG_AEU_AFTER_INVERT_4_MCP val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1E MISC_REG_GRC_TIMEOUT_ATTN MISC_REG_AEU_AFTER_INVERT_4_MCP "((\$val & 0x4000000) != 0)" IDLE_CHK_ERROR "MISC_REG_GRC_TIMEOUT_ATTN: GRC timeout attention parameters (MCP)."
	# Read register MISC_REG_GRC_TIMEOUT_ATTN_FULL_FID val1 and register MISC_REG_AEU_AFTER_INVERT_4_MCP val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x1C MISC_REG_GRC_TIMEOUT_ATTN_FULL_FID MISC_REG_AEU_AFTER_INVERT_4_MCP "((\$val & 0x4000000) != 0)" IDLE_CHK_ERROR "MISC_REG_GRC_TIMEOUT_ATTN_FULL_FID: GRC timeout attention FID (MCP)."
	# Read register IGU_REG_SILENT_DROP val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_SILENT_DROP "(\$val  != 0)" IDLE_CHK_ERROR "Some messages were not executed in the IGU."
	# Read register PXP2_REG_PSWRQ_BW_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x1C PXP2_REG_PSWRQ_BW_CREDIT "(\$val != 0x2D)" IDLE_CHK_ERROR "PXP2: rq_read_credit and rq_write_credit are not 5"
	# Read register IGU_REG_SB_CTRL_FSM val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_SB_CTRL_FSM "(\$val  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. SB_CTRL_FSM should be zero in idle state"
	# Read register IGU_REG_INT_HANDLE_FSM val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_INT_HANDLE_FSM "(\$val  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. INT_HANDLE_FSM should be zero in idle state"
	# Read register IGU_REG_ATTN_FSM val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_ATTN_FSM "((\$val & ~0x2)  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. SB_ATTN_FSMshould be zeroor two in idle state"
	# Read register IGU_REG_CTRL_FSM val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_CTRL_FSM "((\$val & ~0x1)  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. SB_CTRL_FSM should be zero in idle state"
	# Read register IGU_REG_PXP_ARB_FSM val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_PXP_ARB_FSM "((\$val & ~0x1)  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. SB_ARB_FSM should be zero in idle state"
	# Read register IGU_REG_PENDING_BITS_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x1C IGU_REG_PENDING_BITS_STATUS "(\$val  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. There are pending write done"
	# Read register QM_REG_VOQCREDIT_0 val1 and register QM_REG_VOQINITCREDIT_0 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_VOQCREDIT_0 QM_REG_VOQINITCREDIT_0 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: VOQ_0, VOQ credit is not equal to initial credit"
	# Read register QM_REG_VOQCREDIT_1 val1 and register QM_REG_VOQINITCREDIT_1 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_VOQCREDIT_1 QM_REG_VOQINITCREDIT_1 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: VOQ_1, VOQ credit is not equal to initial credit"
	# Read register QM_REG_VOQCREDIT_2 val1 and register QM_REG_VOQINITCREDIT_2 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_VOQCREDIT_2 QM_REG_VOQINITCREDIT_2 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: VOQ_2, VOQ credit is not equal to initial credit"
	# Read register QM_REG_VOQCREDIT_3 val1 and register QM_REG_VOQINITCREDIT_3 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_VOQCREDIT_3 QM_REG_VOQINITCREDIT_3 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: VOQ_3, VOQ credit is not equal to initial credit"
	# Read register QM_REG_VOQCREDIT_4 val1 and register QM_REG_VOQINITCREDIT_4 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_VOQCREDIT_4 QM_REG_VOQINITCREDIT_4 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: VOQ_4, VOQ credit is not equal to initial credit"
	# Read register QM_REG_VOQCREDIT_5 val1 and register QM_REG_VOQINITCREDIT_5 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_VOQCREDIT_5 QM_REG_VOQINITCREDIT_5 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: VOQ_5, VOQ credit is not equal to initial credit"
	# Read register QM_REG_VOQCREDIT_6 val1 and register QM_REG_VOQINITCREDIT_6 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_VOQCREDIT_6 QM_REG_VOQINITCREDIT_6 "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: VOQ_6 (LB VOQ), VOQ credit is not equal to initial credit"
	# Read register QM_REG_BYTECRD0 val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_BYTECRD0 QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Byte credit 0 is not equal to initial credit"
	# Read register QM_REG_BYTECRD1 val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_BYTECRD1 QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Byte credit 1 is not equal to initial credit"
	# Read register QM_REG_BYTECRD2 val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_BYTECRD2 QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Byte credit 2 is not equal to initial credit"
	# Read register QM_REG_BYTECRD3 val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_BYTECRD3 QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Byte credit 3 is not equal to initial credit"
	# Read register QM_REG_BYTECRD4 val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_BYTECRD4 QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Byte credit 4 is not equal to initial credit"
	# Read register QM_REG_BYTECRD5 val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_BYTECRD5 QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Byte credit 5 is not equal to initial credit"
	# Read register QM_REG_BYTECRD6 val1 and register QM_REG_BYTECRDINITVAL val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x10 QM_REG_BYTECRD6 QM_REG_BYTECRDINITVAL "(\$val1 != \$val2)" IDLE_CHK_ERROR_NO_TRAFFIC "QM: Byte credit 6 is not equal to initial credit"
	# Read register QM_REG_FWVOQ0TOHWVOQ val and check if condition on val exist
	IDLE_CHK_1 0x10 QM_REG_FWVOQ0TOHWVOQ "(\$val == 0x7)" IDLE_CHK_ERROR "QM: FwVoq0 is mapped to HwVoq7 (non-TX HwVoq)"
	# Read register QM_REG_FWVOQ1TOHWVOQ val and check if condition on val exist
	IDLE_CHK_1 0x10 QM_REG_FWVOQ1TOHWVOQ "(\$val == 0x7)" IDLE_CHK_ERROR "QM: FwVoq1 is mapped to HwVoq7 (non-TX HwVoq)"
	# Read register QM_REG_FWVOQ2TOHWVOQ val and check if condition on val exist
	IDLE_CHK_1 0x10 QM_REG_FWVOQ2TOHWVOQ "(\$val == 0x7)" IDLE_CHK_ERROR "QM: FwVoq2 is mapped to HwVoq7 (non-TX HwVoq)"
	# Read register QM_REG_FWVOQ3TOHWVOQ val and check if condition on val exist
	IDLE_CHK_1 0x10 QM_REG_FWVOQ3TOHWVOQ "(\$val == 0x7)" IDLE_CHK_ERROR "QM: FwVoq3 is mapped to HwVoq7 (non-TX HwVoq)"
	# Read register QM_REG_FWVOQ4TOHWVOQ val and check if condition on val exist
	IDLE_CHK_1 0x10 QM_REG_FWVOQ4TOHWVOQ "(\$val == 0x7)" IDLE_CHK_ERROR "QM: FwVoq4 is mapped to HwVoq7 (non-TX HwVoq)"
	# Read register QM_REG_FWVOQ5TOHWVOQ val and check if condition on val exist
	IDLE_CHK_1 0x10 QM_REG_FWVOQ5TOHWVOQ "(\$val == 0x7)" IDLE_CHK_ERROR "QM: FwVoq5 is mapped to HwVoq7 (non-TX HwVoq)"
	# Read register QM_REG_FWVOQ6TOHWVOQ val and check if condition on val exist
	IDLE_CHK_1 0x10 QM_REG_FWVOQ6TOHWVOQ "(\$val == 0x7)" IDLE_CHK_ERROR "QM: FwVoq6 is mapped to HwVoq7 (non-TX HwVoq)"
	# Read register QM_REG_FWVOQ7TOHWVOQ val and check if condition on val exist
	IDLE_CHK_1 0x10 QM_REG_FWVOQ7TOHWVOQ "(\$val == 0x7)" IDLE_CHK_ERROR "QM: FwVoq7 is mapped to HwVoq7 (non-TX HwVoq)"
	# Read register NIG_REG_INGRESS_EOP_PORT0_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_INGRESS_EOP_PORT0_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: Port 0 EOP FIFO is not empty."
	# Read register NIG_REG_INGRESS_EOP_PORT1_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_INGRESS_EOP_PORT1_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: Port 1 EOP FIFO is not empty."
	# Read register NIG_REG_INGRESS_EOP_LB_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_INGRESS_EOP_LB_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: LB EOP FIFO is not empty."
	# Read register NIG_REG_INGRESS_RMP0_DSCR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_INGRESS_RMP0_DSCR_EMPTY "(\$val != 1)" IDLE_CHK_WARNING "NIG: Port 0 RX MCP descriptor FIFO is not empty."
	# Read register NIG_REG_INGRESS_RMP1_DSCR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_INGRESS_RMP1_DSCR_EMPTY "(\$val != 1)" IDLE_CHK_WARNING "NIG: Port 1 RX MCP descriptor FIFO is not empty."
	# Read register NIG_REG_INGRESS_LB_PBF_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_INGRESS_LB_PBF_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: PBF LB FIFO is not empty."
	# Read register NIG_REG_EGRESS_MNG0_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_EGRESS_MNG0_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_WARNING "NIG: Port 0 TX MCP FIFO is not empty."
	# Read register NIG_REG_EGRESS_MNG1_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_EGRESS_MNG1_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_WARNING "NIG: Port 1 TX MCP FIFO is not empty."
	# Read register NIG_REG_EGRESS_DEBUG_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_EGRESS_DEBUG_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: Debug FIFO is not empty."
	# Read register NIG_REG_EGRESS_DELAY0_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_EGRESS_DELAY0_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: PBF IF0 FIFO is not empty."
	# Read register NIG_REG_EGRESS_DELAY1_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_EGRESS_DELAY1_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: PBF IF1 FIFO is not empty."
	# Read register NIG_REG_LLH0_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_LLH0_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG:  Port 0 RX LLH FIFO is not empty."
	# Read register NIG_REG_LLH1_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1F NIG_REG_LLH1_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG:  Port 1 RX LLH FIFO is not empty."
	# Read register NIG_REG_P0_TX_MNG_HOST_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1C NIG_REG_P0_TX_MNG_HOST_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_WARNING "NIG: Port 0 TX MCP FIFO for traffic going to the host is not empty."
	# Read register NIG_REG_P1_TX_MNG_HOST_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1C NIG_REG_P1_TX_MNG_HOST_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_WARNING "NIG: Port 1 TX MCP FIFO for traffic going to the host is not empty."
	# Read register NIG_REG_P0_TLLH_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1C NIG_REG_P0_TLLH_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG:  Port 0 TX LLH FIFO is not empty."
	# Read register NIG_REG_P1_TLLH_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1C NIG_REG_P1_TLLH_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG:  Port 1 TX LLH FIFO is not empty."
	# Read register NIG_REG_P0_HBUF_DSCR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1C NIG_REG_P0_HBUF_DSCR_EMPTY "(\$val != 1)" IDLE_CHK_WARNING "NIG: Port 0 RX MCP descriptor FIFO for traffic from the host is not empty."
	# Read register NIG_REG_P1_HBUF_DSCR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x1C NIG_REG_P1_HBUF_DSCR_EMPTY "(\$val != 1)" IDLE_CHK_WARNING "NIG: Port 1 RX MCP descriptor FIFO for traffic from the host is not empty."
	# Read register NIG_REG_P0_RX_MACFIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x18 NIG_REG_P0_RX_MACFIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: Port 0 RX MAC interface FIFO is not empty."
	# Read register NIG_REG_P1_RX_MACFIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x18 NIG_REG_P1_RX_MACFIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: Port 1 RX MAC interface FIFO is not empty."
	# Read register NIG_REG_P0_TX_MACFIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x18 NIG_REG_P0_TX_MACFIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: Port 0 TX MAC interface FIFO is not empty."
	# Read register NIG_REG_P1_TX_MACFIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x18 NIG_REG_P1_TX_MACFIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: Port 1 TX MAC interface FIFO is not empty."
	# Read register NIG_REG_EGRESS_DELAY2_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x10 NIG_REG_EGRESS_DELAY2_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: PBF IF2 FIFO is not empty."
	# Read register NIG_REG_EGRESS_DELAY3_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x10 NIG_REG_EGRESS_DELAY3_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: PBF IF3 FIFO is not empty."
	# Read register NIG_REG_EGRESS_DELAY4_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x10 NIG_REG_EGRESS_DELAY4_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: PBF IF4 FIFO is not empty."
	# Read register NIG_REG_EGRESS_DELAY5_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x10 NIG_REG_EGRESS_DELAY5_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: PBF IF5 FIFO is not empty."

	# Check assertions
	if {$ignoreAssert != 1} {
	    # Check assertions only if device is not hidden
	    if {$::sys(NO_PCI) == 0} {
	       if {$::current(DID)} {
	           incr errors [check_assertions]
	       }
	    }
	}
	if {$errors == 0} {
		if {$outfileName != ""} {
		       puts $outfile "Idle_chk completed successfully. (with warnings: $warnings of $total checks)"
		} else {
		       puts "Idle_chk completed successfully. (with warnings: $warnings of $total checks)"
		}
	} else {
		if {$outfileName != ""} {
		       puts $outfile "Idle_chk failed !!! (with errors: $errors warnings: $warnings of $total checks)"
		} else {
		       puts "Idle_chk failed !!! (with errors: $errors warnings: $warnings of $total checks)"
		}
	}
	if {$outfileName != ""} {
	    close $outfile
	}
	# Re-enable timers
	if {$enableTimer0} {
	       enable_timer 0
	}
	if {$enableTimer1} {
	       enable_timer 1
	}
	return $errors
}
