###########################################################################
#
# Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved
# Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
# Proprietary and Confidential Information.
#
# Module Description:
# This TCL script loads all needed link information needed for debug purposes
#
# Written by Yaniv Rosner, QLogic, Jul 2008                                        
#
###########################################################################

# Specify any dependency on other packages
package require Tcl

# Specify the package this module provides
package provide _access_phy 1.0
# -----------------------------------------------------------------
# PORT MODE
#
# Returns: E1X - 2 
#          E2/E3  - 2 / 4 
# -----------------------------------------------------------------

namespace eval _access_phy {
    # Export the package
    namespace export access_phy
}

proc _access_phy::PORT_MODE {} {
    set NIG_REG_PORT4MODE_EN      0x18274
    set MISC_REG_PORT4MODE_EN_OVWR      0xa720
    if {([is_e2] && [reg read $NIG_REG_PORT4MODE_EN]) || ([is_e3] && ([reg read $MISC_REG_PORT4MODE_EN_OVWR] == 3))} {
        return 4
    } else {
        return 2
    }
}

# -----------------------------------------------------------------
# PORT -
#
# Returns PORT of the pf_num
# -----------------------------------------------------------------
proc _access_phy::PORT {pf_num} {
    if {([is_e2] || [is_e3])} {
        if {[PORT_MODE] == 4} {
            return [expr ($pf_num >> 1) & 1]
        } else {
            return 0
        }
    } else {
        return [expr $pf_num & 1]
    }
}

# -----------------------------------------------------------------
# PATH -
#
# Returns PATH of the pf_num
# -----------------------------------------------------------------
proc _access_phy::PATH {pf_num} {

    if {([is_e2] || [is_e3])} {
         if {[PORT_MODE] == 4} {
            return [expr $pf_num & 1]
         } else {
            return [expr ($pf_num >> 1) & 1]
         }
    } else {
            return 0
    }
}
# -----------------------------------------------------------------
# set_device
#
# Goto the appropriate device based on the chip id, path and port
# -----------------------------------------------------------------
proc _access_phy::set_cur_device {pf_num} {

    set base_dev [expr $::current(DEV) - $::current(PCI_FUNC)]
   # puts "set_device $base_dev $pf_num"
    if {([is_e2] || [is_e3])} {
        device [expr $base_dev + [PATH $pf_num] + (2*[PORT $pf_num])] -no_display
    } else {
        device [expr $base_dev + [PORT $pf_num]] -no_display
    }
}

proc _access_phy::speedadv { speed_adv_1_10 speed_adv_2_5 } {
    set speed_str ""
    if [expr $speed_adv_1_10 & {1<<7}] {
        set speed_str "10G"        
    }
     if [expr $speed_adv_2_5 & {1<<0}] {
        set speed_str "2.5G $speed_str"      
    }
    if [expr $speed_adv_1_10 & {1<<5}] {
        set speed_str "1G $speed_str"     
    }   
    return $speed_str
}

proc _access_phy::far_end_capable_of_cl73 {gp_status} {
   set  MDIO_GP_STATUS_TOP_AN_STATUS1_CL73_MR_LP_NP_AN_ABLE 0x0010
   set far_end_capable_of_cl73 [expr $gp_status & $MDIO_GP_STATUS_TOP_AN_STATUS1_CL73_MR_LP_NP_AN_ABLE]
   if [expr $far_end_capable_of_cl73] {
        return "Yes"
   }
   return "No"
}
proc _access_phy::cl37_complete {gp_status} {
   set MDIO_GP_STATUS_TOP_AN_STATUS1_CL37_AUTONEG_COMPLETE  0x0002  
   set cl73_complete [expr $gp_status & $MDIO_GP_STATUS_TOP_AN_STATUS1_CL37_AUTONEG_COMPLETE]
   if [expr $cl73_complete] {
        return "Yes"
   }
   return "No"
}

proc _access_phy::cl73_complete {gp_status} {
   set MDIO_GP_STATUS_TOP_AN_STATUS1_CL73_AUTONEG_COMPLETE  0x0001
     
   set cl73_complete [expr $gp_status & $MDIO_GP_STATUS_TOP_AN_STATUS1_CL73_AUTONEG_COMPLETE]
   if [expr $cl73_complete] {
        return "Yes"
   }
   return "No"
}

proc _access_phy::lp_capable_bam {gp_status} {
   set  CL73_LP_NP_BAM_ABLE 0x0020
   set  lp_capable_bam [expr $gp_status & $CL73_LP_NP_BAM_ABLE]
   if [expr $lp_capable_bam] {
        return "Yes"
   }
   return "No"
   
}
proc _access_phy::is_bmac {gp_status} {
    set speed [expr $gp_status & 0x3f00]
    if {$speed == 0x600 || $speed == 0x700 || $speed == 0xe00} {
        return 1
    }
    return 0
}

proc _access_phy::gpstatus { gp_status } {
    set int_name "XGXS"
    if {[is_e3]} {
    set int_name "WC"
    }

    set link_up [expr $gp_status & {1<<2}]
    if [expr !$link_up] {
        return "$int_name link down"
    }
    set speed [expr $gp_status & 0x3f00]

    if {$speed == 0x000} {
    return "$int_name link up 10M"
    }    
    if {$speed == 0x100} {
    return "$int_name link up 100M"
    }
    if {$speed == 0x200} {
    return "$int_name link up 1G"
    }
    if {$speed == 0x300} {
    return "$int_name link up 2.5G"
    }
    if {$speed == 0x600} {
    return "$int_name link up 10G-HIG"        
    }
    if {$speed == 0x700} {
    return "$int_name link up 10G-CX4"        
    }
    if {$speed == 0xd00} {
    return "$int_name link up 1G-KX"        
    }
    if {$speed == 0xe00} {
    return "$int_name link up 10G-KX4"        
    }

    return "$int_name unknown status"
}
proc _access_phy::linkspeed_8073 { link_status } {
    set result "External link down"
        set speed_10000 [expr $link_status & {1<<2}]
        set link_10000_down [expr $link_status & {1<<15}]
        set speed_2500 [expr $link_status & {1<<1}]
        set link_2500_down [expr $link_status & {1<<14}]
        set speed_1000 [expr $link_status & {1<<0}]
        set link_1000_down [expr $link_status & {1<<13}]
        if [expr $speed_10000 && {!$link_10000_down }] {
            set result "Ext link up 10G"
            return $result 
        }
        if [expr $speed_2500 && {!$link_2500_down}] {
            set result "Ext link up 2.5G"
            return $result 
        }
        if [expr $speed_1000 && {!$link_1000_down}] {
            set result "Ext link up 1G"
            return $result 
        }
         
        return $result           
}

proc _access_phy::get_rx_flow_ctrl {port gp_status} {

    set link_up [expr $gp_status & {1<<2}]
    if {[expr !$link_up] || ($gp_status == 0xFFFF)} {
        return No
    }
    if {[is_e2]} {
            set cur_dev "$::current(DEV)"
            device [expr (($cur_dev-1) & ~1) + 1 + $port] -no_display
            set port 0
    }
            
    if {[is_bmac $gp_status]} {
        if {[is_e1x]} {
            set rx_control_reg 0x108
        } else {
            set rx_control_reg 0x1d0
        }
    
        if {[expr [reg read [expr $rx_control_reg + 0x10c00 + $port*0x400]] & 0x20]} {                
             set res Yes
        } else {
             set res No
        }        
    } else {
         if {[expr [reg read [expr 0x80c8 + $port*0x400]] & 0x4]} {
            set res Yes
        } else {
            set res No
        }
    }
     if {[is_e2]} {
            device $cur_dev -no_display
    }
    return $res
}

proc _access_phy::get_tx_flow_ctrl {port gp_status} {

    set link_up [expr $gp_status & {1<<2}]
    if {[expr !$link_up] || ($gp_status == 0xFFFF)} {
        return No
    }
     if {[is_e2]} {
            set cur_dev "$::current(DEV)"
            device [expr (($cur_dev-1) & ~1) + 1 + $port] -no_display
            set port 0
    }
    if {[is_bmac $gp_status]} {
        if {[is_e1x]} {
            set tx_control_reg 0x38
        } else {
            set tx_control_reg 0xe0
        }
    
        if {[expr [reg read [expr $tx_control_reg + 0x10c00 + $port*0x400]] & 0x800000]} {                
             set res Yes
        } else {
             set res No
        }        
    } else {
         if {[expr [reg read [expr 0x80bc + $port*0x400]] & (3<<3)]} {
            set res Yes
        } else {
            set res No
        }
    }  
     if {[is_e2]} {
            device $cur_dev -no_display
    }     
    return $res
}

proc _access_phy::get_remote_fault_e1xe2 {port} {
    set XMAC_REG_LSS_VAL 0x58
    set XMAC_RX_LSS_STATUS_LOCAL_FAULT_STATUS 1
    set XMAC_RX_LSS_STATUS_REMOTE_FAULT_STATUS 2
    set res "N/A"
    set LSS_REG 0x10df0
    set BMAC_RESET_MASK 0x1
    if {[is_e2]} {
            set cur_dev "$::current(DEV)"
            device [expr (($cur_dev-1) & ~1) + 1 + $port] -no_display
    } else {
       set BMAC_RESET_MASK 0x2
       if {$port} {
         set LSS_REG 0x11218
       } else {
         set LSS_REG 0x10a18
       }
    }
    
    set bmac_out_of_reset [expr [reg read 0xa590] & $BMAC_RESET_MASK]
    if {$bmac_out_of_reset} {
    set lss_val [reg read $LSS_REG]
        if {$lss_val == 0x3} { 
           set res "Local/Remote Fault !!"
        }  else { if {$lss_val == 0x2} {
                  set res "Remote Fault !!"       
        } else  { if {$lss_val == 0x1} {
                 set res "Local Fault !!"     
        } else {
           set res "None"
        }}}
    }   
    if {[is_e2]} {
        device $cur_dev -no_display
    }     
    return $res
}

proc _access_phy::ld_extphy {phy_index port dev addr args} {
    if {[is_e2]} {
        set cur_dev "$::current(DEV)"
        device [expr (($cur_dev-1) & ~1) + 1 + $port] -no_display
        set port 0
    }
        
    if {$phy_index == 1} {
        if {$args == ""} {
            set res [extphy $port $dev $addr]
        } else {
            set res [extphy $port $dev $addr $args]
        }
    } else {
        if {$args == ""} {
            set res [extphy2 $port $dev $addr]
        } else {
            set res [extphy2 $port $dev $addr $args]
        }
    }
    if {[is_e2]} {
        device $cur_dev -no_display
    }
    return $res
}

proc _access_phy::get_84833_link_att {att phy_index port} {
	if {$att == "Copper"} {
        set legacy_status  [ld_extphy $phy_index $port 7 0xfff5] 
		set pmd_signal     [ld_extphy $phy_index $port 1 0xa811]
        set link10G        [expr $pmd_signal & (1<<11)]
        if {$link10G} {
            set result "\"Link up 10G\""
        } else {
            # check legacy link speed
            set legacy_status_link [expr $legacy_status & (1<<11)]
            if {$legacy_status_link} {
                set legacy_speed [expr $legacy_status & (3<<9)]
                if {$legacy_speed == (0<<9)} {
                    set result "\"Link up 10Mb\"" 
                } else {
                    if {$legacy_speed == (1<<9)} {
                        set result "\"Link up 100Mb\""
                    } else {
                        if {$legacy_speed == (2<<9)} {
                            set result "\"Link up 1Gb\""
                        } else {
                            set result "\"Unknown legacy link speed\""
                        }
                    }
                }
            } else {
                set result "\"Ext link down\""
            }
        }
		return $result
	} else { if {$att == "Fiber"} {
        set fiber_link     [expr [ld_extphy $phy_index $port 0x1e 0x4095] & (1<<14)]
        if {$fiber_link} {
            set fiber_link "\"Link up 10G\""
        } else {
            set fiber_link "\"Link down\""
        }
		return $fiber_link
	} else { if {$att == "LinkReg"} {
	    set legacy_status  [ld_extphy $phy_index $port 7 0xfff5] 
		set pmd_signal     [ld_extphy $phy_index $port 1 0xa811]
		return "\"$pmd_signal/$legacy_status\""
	}}}
	return "\"N/A\""
}
proc _access_phy::get_fwver {phy_index port} {
    variable ext_phy_name  
    set shmem2_base [reg read 0xa460]
    if {[is_e2]} {
        set cur_dev "$::current(DEV)"
        device [expr (($cur_dev-1) & ~1) + 1 + $port] -no_display
        if {$port == 1} {
            set shmem2_base [reg read [expr $shmem2_base + 14*4]]
        }
        set port 0
    }
    set shmem_base  [reg read 0xa2b4]
    if {$phy_index == 1} {
            set fw_ver [reg read [expr $shmem_base + 0x664 + $port*16 + 12]]
    } else {
            set fw_ver [reg read [expr $shmem2_base + 8 + 4*$port]]
    }
    if {$ext_phy_name == 7101} {
        return [format "%c%c%c%c" ($fw_ver & 0xFF) (($fw_ver & 0x00FF) >> 8) (($fw_ver & 0x0000FF) >> 16) (($fw_ver & 0x000000FF) >> 24)]
    } else {
    if {$ext_phy_name == 8481 || $ext_phy_name == 84823 || $ext_phy_name == 84833} {
        return [format "%2x.%2x" [hex32 [expr (($fw_ver & 0xF80) >> 7)]] [hex32 [expr ($fw_ver & 0x7F)]]]
    } else {
        return [format "%2x.%2x" [hex32 [expr ($fw_ver & 0xFFFF0000) >> 16]] [hex32 [expr ($fw_ver & 0xFFFF)]]]
    }}
}
# this function handles external phy register dump. Also include
# XGXS for e1x and e2, WC for e3
proc _access_phy::linkdump_e1xe2e3 {{phy_index 1}} {
    variable ext_phy_name     
            
    set indent 20s
    set labels [format "%-$indent %-$indent %-$indent" "Item" "Port0" "Port1"]
    set no_dev_2 0
    set cur_dev "$::current(DEV)"        


    if {[is_e2]} {            
        device [expr (($cur_dev-1) & ~1) + 1] -no_display
        set gp_status0 [phy 0 0x812b]
        set shmem_base [reg read 0xa2b4]
        set link_status0 [reg read  [expr $shmem_base + 0x664]]
        set populate_phy_info [ld_extphy $phy_index 0 1 1]
        set no_dev_2 [catch {device [expr (($cur_dev-1) & ~1) + 2] -no_display}]
        if {$no_dev_2} {
            set gp_status1 "invalid"
            set link_status1 "invalid"
        } else {            
            set gp_status1 [phy 0 0x812b] 
            set shmem_base [reg read 0xa2b4]
            set link_status1 [reg read  [expr $shmem_base + 0x664]]
        }
        device $cur_dev -no_display
    } elseif {[is_e1x]} {
        set gp_status0 [phy 0 0x812b]
        set gp_status1 [phy 1 0x812b]
        set shmem_base [reg read 0xa2b4]
        set populate_phy_info [ld_extphy $phy_index 0 1 1]
        set link_status0 [reg read  [expr $shmem_base + 0x664]]
        set link_status1 [reg read  [expr $shmem_base + 0x664 + 16]]
    } elseif {[is_e3]} {
             set rc [linkdump_e3_wc]
             return $rc
    }
        
       
    puts "$labels"
    puts "----------------------------------------------------------"

        if {[string compare $ext_phy_name "n/a"] != 0 } {
            # fw_version
            set fw_ver0 [get_fwver $phy_index 0]
            set fw_ver1 [get_fwver $phy_index 1]

            set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name fw_version" "$fw_ver0" "$fw_ver1" ]
            puts "$line"

            # link status ( Read twice )dr
            if {$ext_phy_name == 8726 || $ext_phy_name == 8706} {
                set rx_sd0          [ld_extphy $phy_index 0 1 0x000a]
                set pcs_status0     [ld_extphy $phy_index 0 3 0x0020]
                set an_link_status0 [ld_extphy $phy_index 0 7 0x8304]
                set an_link_status0 [ld_extphy $phy_index 0 7 0x8304]
                set link_speed0_1G  [expr ($an_link_status0 & (1<<1))]
                set link_status0    [expr ($rx_sd0 & $pcs_status0 & 0x1) || $link_speed0_1G ]                
                if {$link_status0 && $an_link_status0 != 0xffff} {
                    if {$link_speed0_1G} {                    
                        set result0 "Ext link up 1Gb"
                    } else {
                        set result0 "Ext link up 10Gb" 
                    }
                } else {
                     set result0 "Ext link down" 
                }

                set rx_sd1          [ld_extphy $phy_index 1 1 0x000a]
                set pcs_status1     [ld_extphy $phy_index 1 3 0x0020]
                set an_link_status1 [ld_extphy $phy_index 1 7 0x8304]
                set an_link_status1 [ld_extphy $phy_index 1 7 0x8304]
                set link_speed1_1G  [expr ($an_link_status1 & (1<<1))]
                set link_status1    [expr ($rx_sd1 & $pcs_status1 & 0x1) || $link_speed1_1G ]   
                 if {$link_status1 && $an_link_status1 != 0xffff} {
                    if {$link_speed1_1G} {                    
                        set result1 "Ext link up 1Gb"
                    } else {
                        set result1 "Ext link up 10Gb" 
                    }
                } else {
                     set result1 "Ext link down" 
                }

                set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name rx_sd" "$rx_sd0 " "$rx_sd1 " ]
                puts "$line"
                set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name pcs_status" "$pcs_status0 " "$pcs_status1 " ]
                puts "$line"
                set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name an_link_status" "$an_link_status0 " "$an_link_status1 " ]
                puts "$line"
                set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name link status" "$result0 " "$result1 " ]
                puts "$line"                
            } else {
                if {$ext_phy_name == 8481 || $ext_phy_name == 84823 || $ext_phy_name == 84833 } {
					for {set port 0} {$port < 2} {incr port} {
						set copper_link($port) [string trim [get_84833_link_att Copper $phy_index $port] \"]
						set fiber_link($port) [string trim [get_84833_link_att Fiber $phy_index $port] \"]
						set link_regs($port) [string trim [get_84833_link_att LinkReg $phy_index $port] \"]
					}
                    set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name Copper link" "$copper_link(0)" "$copper_link(1)" ]
                    puts "$line"
                    set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name Fiber link" "$fiber_link(0)" "$fiber_link(1)" ]
                    puts "$line"
                    set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name pmd/legacy" "$link_regs(0) " "$link_regs(1)" ]
                    puts "$line"                
                } else {                           
                    set link_status0 [ld_extphy $phy_index 0 1 1]
                    set link_status0 [ld_extphy $phy_index 0 1 1]
                    set link_status1 [ld_extphy $phy_index 1 1 1]
                    set link_status1 [ld_extphy $phy_index 1 1 1]
                    set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name link <1,1>" "$link_status0 " "$link_status1 " ]
                    puts "$line"
                }   
            }
           
            # speed advertisment
            set speed_adv_1_10_0 [ld_extphy $phy_index 0 7 0x11]
            set speed_adv_1_10_1 [ld_extphy $phy_index 1 7 0x11]
    
            if {$ext_phy_name == 8073 } {        
                set speed_adv_25_0 [ld_extphy $phy_index 0 7 0x8329]
                set speed_adv_25_1 [ld_extphy $phy_index 1 7 0x8329]
            } else {
                set speed_adv_25_0 0
                set speed_adv_25_1 0
            }
      
            set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name speed adv" "[speedadv $speed_adv_1_10_0 $speed_adv_25_0]" "[speedadv $speed_adv_1_10_1 $speed_adv_25_1]" ]
            puts "$line"
    
             # speed advertisment
            set an_status0 [ld_extphy $phy_index 0 7 1]
            set an_status1 [ld_extphy $phy_index 1 7 1]
            set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name an status" "$an_status0" "$an_status1" ]
            puts "$line"
    
            # link speed
            if {($ext_phy_name == 8073) || ($ext_phy_name == 8727)} {        
                set link_speed0 [ld_extphy $phy_index 0 1 0xc820]
                set link_speed1 [ld_extphy $phy_index 1 1 0xc820]   
                set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name link speed" "$link_speed0" "$link_speed1" ]
                puts "$line"
                set line [format "%-$indent %-$indent %-$indent" "" "[linkspeed_8073 $link_speed0]" "[linkspeed_8073 $link_speed1]" ]
                puts "$line"
            } else {
                if {$ext_phy_name == 8726 } {
                    if {$link_status0} {                
                        if {$link_speed0_1G} {
                            set result0 "Ext link up 1G"
                        } else {
                            set result0 "Ext link up 10G"   
                        }                    
                    } else {
                        set result0 "Ext link down"   
                    }
             
                    if {$link_status1} {                
                        if {$link_speed1_1G} {
                            set result1 "Ext link up 1G"
                        } else {
                            set result1 "Ext link up 10G"   
                        }                    
                    } else {
                        set result1 "Ext link down"   
                    }
                    set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name link speed" "$result0" "$result1" ]
                }
            }
    
            # flow control
            set fc_ld0 [ld_extphy $phy_index 0 7 0x10]
            set fc_ld1 [ld_extphy $phy_index 1 7 0x10]
            set fc_lp0 [ld_extphy $phy_index 0 7 0x13]
            set fc_lp1 [ld_extphy $phy_index 1 7 0x13]
            set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name fc (ld/lp)" "$fc_ld0/$fc_lp0" "$fc_ld1/$fc_lp1" ]
            puts "$line"
            
            if {$ext_phy_name == 8073 } {   
                set fc_cl37_ld0 [ld_extphy $phy_index 0 7 0xffe4]
                set fc_cl37_ld1 [ld_extphy $phy_index 1 7 0xffe4]
                set fc_cl37_lp0 [ld_extphy $phy_index 0 7 0xffe5]
                set fc_cl37_lp1 [ld_extphy $phy_index 1 7 0xffe5]   
                set line [format "%-$indent %-$indent %-$indent" "$ext_phy_name CL37 fc (ld/lp)" "$fc_cl37_ld0/$fc_cl37_lp0" "$fc_cl37_ld1/$fc_cl37_lp1" ]
                puts "$line"
            }
        } else {
                set cl37_complete_0 [cl37_complete $gp_status0]
                if {$no_dev_2} {
                    set cl37_complete_1 "invalid"
                } else {                
                    set cl37_complete_1 [cl37_complete $gp_status1]
                }
                set line [format "%-$indent %-$indent %-$indent" "CL37 complete" "$cl37_complete_0" "$cl37_complete_1" ]
                puts "$line"
        
                set cl73_complete_0 [cl73_complete $gp_status0]
                if {$no_dev_2} {
                    set cl73_complete_1 "invalid"
                } else {                
                    set cl73_complete_1 [cl73_complete $gp_status1]
                }
                set line [format "%-$indent %-$indent %-$indent" "CL73 complete" "$cl73_complete_0" "$cl73_complete_1" ]
                puts "$line"
        
                set far_end_capable_of_cl73_0 [far_end_capable_of_cl73 $gp_status0]
                if {$no_dev_2} {
                    set far_end_capable_of_cl73_1 "invalid"
                } else {                
                    set far_end_capable_of_cl73_1 [far_end_capable_of_cl73 $gp_status1]
                }
                set line [format "%-$indent %-$indent %-$indent" "LP capable of CL73" "$far_end_capable_of_cl73_0" "$far_end_capable_of_cl73_1" ]
                puts "$line"

                set lp_capable_bam_0 [lp_capable_bam $gp_status0]
                if {$no_dev_2} {
                    set lp_capable_bam_1 "invalid"
                } else {
                    set lp_capable_bam_1 [lp_capable_bam $gp_status1]
                }
                set line [format "%-$indent %-$indent %-$indent" "LP support BAM->CL37" "$lp_capable_bam_0" "$lp_capable_bam_1" ]
                puts "$line"

                set pd_10g_0 [expr [phy 0 0x8130] & 1<<15]
                set pd_10g_1 [expr [phy 1 0x8130] & 1<<15]
                if {$pd_10g_0} {
                    set pd_10g_0 "Yes"
                } else {
                    set pd_10g_0 "No"
                }
                 if {$pd_10g_1} {
                    set pd_10g_1 "Yes"
                } else {
                    set pd_10g_1 "No"
                }
                set line [format "%-$indent %-$indent %-$indent" "10G Parallel Detect" "$pd_10g_0" "$pd_10g_1" ]
                puts "$line"
                

                
        }

        set rx_flow_ctrl0 [get_rx_flow_ctrl 0 $gp_status0]
        if {$no_dev_2} {
            set rx_flow_ctrl1 "invalid"
        } else {        
            set rx_flow_ctrl1 [get_rx_flow_ctrl 1 $gp_status1]
        }
        set tx_flow_ctrl0 [get_tx_flow_ctrl 0 $gp_status0]
        if {$no_dev_2} {
            set tx_flow_ctrl1 "invalid"
        } else {        
            set tx_flow_ctrl1 [get_tx_flow_ctrl 1 $gp_status1]
        }
        
        set line [format "%-$indent %-$indent %-$indent" "RX/TX flow control" "$rx_flow_ctrl0/$tx_flow_ctrl0" "$rx_flow_ctrl1/$tx_flow_ctrl1" ]
        puts "$line"
        
        # gp_status       
        set line [format "%-$indent %-$indent %-$indent" "gp_status" "$gp_status0" "$gp_status1" ]
        puts "$line"
        if {$no_dev_2} {
            set line [format "%-$indent %-$indent %-$indent" "" "[gpstatus $gp_status0]" "invalid" ]
        } else {        
            set line [format "%-$indent %-$indent %-$indent" "" "[gpstatus $gp_status0]" "[gpstatus $gp_status1]" ]        
        }
        puts "$line"

    set fault0 [get_remote_fault_e1xe2 0]
    set fault1 [get_remote_fault_e1xe2 1]
    set line [format "%-$indent %-$indent %-$indent" "Link Fault" "$fault0" "$fault1" ]
        puts "$line"
        set line [format "%-$indent %-$indent %-$indent" "link_status" "$link_status0" "$link_status1" ]
        puts "$line"

        device $cur_dev -no_display
}
proc _access_phy::get_e3_link_speed {lane} {

    set speed_reg [expr ($lane>>1) + 0x81d2]
    set speed_lane_offset [expr (8*($lane & 1))]
    set speed [expr ([wc_phy 0 3 $speed_reg] >> $speed_lane_offset) & 0x3f]

    if {$speed == 0x000} {return "10M";}
    if {$speed == 0x001} {return "100M";}

    if {$speed == 0x002} {
            if {[expr [wc_phy $lane 3 0x8300] & 1]} {
                return "1G-SFI";
            } else {
                return "1G-SGMII";
            }
    }

    if {$speed == 0x003} {return "2.5G";}

    if {$speed == 0x00d} {return "1G-KX";}

    if {$speed == 0x00e} {return "10G-KX4";}

    if {$speed == 0x00f} {return "10G-KR";}

    if {$speed == 0x01b} {return "10G-XFI";}

    if {$speed == 0x01f} {return "10G-SFI";}

    if {$speed == 0x01e} {return "20G-DXGXS";}

    if {$speed == 0x039} {return "20G-KR2";}

    return "Unknown"

    }

proc _access_phy::get_link_e3 {lane} {
    set link_speed [get_e3_link_speed $lane]
    if {[string match "*20G-D*" $link_speed]} {
        set link_up [expr ([wc_phy $lane 3 0x1] & 0x4) >> 2]
    } else {
        set link_reg_val [wc_phy 0 3 0x81d1]
        set link_status [expr ($link_reg_val >> (12+$lane)) & 0x1]
        set link_KR [expr ($link_reg_val >> (8+$lane)) & 0x1]
		set link_an [expr ($link_reg_val >> (0+$lane)) & 0x1]
        set link_up [expr $link_status | $link_KR | $link_an]
    }
    if {$link_up} {
       return "\"Link Up $link_speed\""
    } else {
       return "\"Link Down\""
    } 
}

proc _access_phy::XMAC_BASE {lane} {
    if {[PORT $lane]} {
        return 0x163800
    } else {
            return 0x163000
        }
}

proc _access_phy::get_fc_xmac_e3 {lane} {
set XMAC_REG_PAUSE_CTRL 0x68
    set XMAC_PAUSE_CTRL_REG_RX_PAUSE_EN              [expr 0x1<<18]
    set XMAC_PAUSE_CTRL_REG_TX_PAUSE_EN              [expr 0x1<<17]
set XMAC_REG_PFC_CTRL_HI 0x74
    set XMAC_PFC_CTRL_HI_REG_RX_PFC_EN               [expr 0x1<<4]
    set XMAC_PFC_CTRL_HI_REG_TX_PFC_EN               [expr 0x1<<5]
    set pause_val [reg read [expr [XMAC_BASE $lane] + $XMAC_REG_PAUSE_CTRL]]
    
    if {[expr $pause_val & ($XMAC_PAUSE_CTRL_REG_RX_PAUSE_EN | $XMAC_PAUSE_CTRL_REG_TX_PAUSE_EN)] == [expr $XMAC_PAUSE_CTRL_REG_RX_PAUSE_EN | $XMAC_PAUSE_CTRL_REG_TX_PAUSE_EN]} {
       return "\"RX/TX pause\""
    }
    if {[expr $pause_val & ($XMAC_PAUSE_CTRL_REG_RX_PAUSE_EN)]} {
       return "\"RX pause\""
    }
    if {[expr $pause_val & ($XMAC_PAUSE_CTRL_REG_TX_PAUSE_EN)]} {
       return "\"TX pause\""
    }
    set pfc_val [reg read [expr [XMAC_BASE $lane] + $XMAC_REG_PFC_CTRL_HI]]
    if {[expr $pause_val & ($XMAC_PFC_CTRL_HI_REG_RX_PFC_EN | $XMAC_PFC_CTRL_HI_REG_TX_PFC_EN)]} {
       return "\"RX/TX PFC\""
    }
    if {[expr $pause_val & ($XMAC_PFC_CTRL_HI_REG_RX_PFC_EN)]} {
       return "\"RX PFC\""
    }
    if {[expr $pause_val & ($XMAC_PFC_CTRL_HI_REG_TX_PFC_EN)]} {
       return "\"TX PFC\""
    }
    return "\"None\""
}

proc _access_phy::UMAC_BASE {lane} {
    if {[PORT $lane]} {
        return 0x160400
    } else {
            return 0x160000
        }
}

proc _access_phy::get_fc_umac_e3 {lane} {
set UMAC_REG_COMMAND_CONFIG   0x8
    set UMAC_COMMAND_CONFIG_REG_PAUSE_IGNORE     [expr 0x1<<8]
    set UMAC_COMMAND_CONFIG_REG_IGNORE_TX_PAUSE  [expr 0x1<<28]
     set pause_val [reg read [expr [UMAC_BASE $lane] + $UMAC_REG_COMMAND_CONFIG]]
     if {[expr ($pause_val & ($UMAC_COMMAND_CONFIG_REG_PAUSE_IGNORE | $UMAC_COMMAND_CONFIG_REG_IGNORE_TX_PAUSE))] == [expr ($UMAC_COMMAND_CONFIG_REG_PAUSE_IGNORE | $UMAC_COMMAND_CONFIG_REG_IGNORE_TX_PAUSE)]} {
        return "\"None\""
     }
     if {[expr $pause_val & $UMAC_COMMAND_CONFIG_REG_PAUSE_IGNORE]} {
        return "\"TX pause\""
     }
     if {[expr $pause_val & $UMAC_COMMAND_CONFIG_REG_IGNORE_TX_PAUSE]} {
        return "\"RX pause\""
     }
     return "\"RX/TX pause\""
}

proc _access_phy::get_fc_e3 {lane link} {
    if {[string match "*Up*" $link]} {
        if {[string match "*10M*" $link] || [string match "*100M*" $link] || [string match "*1G*" $link]} {
            return [get_fc_umac_e3 $lane]
        } else {
            return [get_fc_xmac_e3 $lane]
        }
    } else {
        return "\"N/A\""
        
    }
}

proc _access_phy::get_module_e3 {lane link} {
    if {[string match "*SFI*" [get_e3_link_speed $lane]]}  {
        set module [expr ([wc_phy 0 3 0x81f2] >> ($lane << 2)) & 0xf]
        if {$module} {
            return "\"DAC($module)\""
        } else {
            return "\"OPTIC($module)\""
        }
    } else {
        return "\"N/A\""
    }
}

proc _access_phy::get_bam_on_kr_e3 {lane link} {
	if {[string match "*KR*" $link]}  {
		set bam_on_kr [expr [wc_phy $lane 3 0x8350] & 1]
		set cl73_bam [expr [wc_phy $lane 3 0x8372] & (1<<15)]
		if {$bam_on_kr} {
			return "\"BAM on KR\""
		} elseif {$cl73_bam} {
			return "\"CL73 BAM\""
		} else {
			return "\"None\""
		}
	} else {
		return "\"N/A\""
	}
}

proc _access_phy::get_remote_fault_e3 {lane link} {
    set XMAC_REG_LSS_VAL 0x58
    set LOCAL_FAULT 1
    set REMOTE_FAULT 2

    if {[string match "*Up*" $link] == 0} {
        return "\"N/A\""
    }
     
    set xmac_out_of_reset [expr [reg read 0xa590] & (1<<22)]
    if {$xmac_out_of_reset} {
        set lss_val [reg read [expr [XMAC_BASE $lane] + $XMAC_REG_LSS_VAL]]
    #   puts "$lane $lss_val [hex32 [expr [XMAC_BASE $lane] + $XMAC_REG_LSS_VAL]]"
        if {[expr $lss_val & ($LOCAL_FAULT | $REMOTE_FAULT)] == [expr ($LOCAL_FAULT | $REMOTE_FAULT)]} {
            return "\"Local/Remote Fault!\""
        } else {
            if {[expr $lss_val & $LOCAL_FAULT]} {
                 return "\"Local Fault !!\""
            } else {
              if {[expr $lss_val & $REMOTE_FAULT]} {
                 return "\"Remote Fault !!\""
            } else {
               return "\"None\""
            }
          }
        }       
    } else {
       return "\"N/A\""
    }
}

proc _access_phy::get_54616_link {port} {
	set legacy_status [ld_extphy 1 $port 0 0x19]
	set link_up [expr $legacy_status & (1<<2)]
	if {$link_up} {
		set legacy_speed [expr $legacy_status & (7<<8)]
		if {$legacy_speed == (7<<8)} {
			return "\"Link up 1G-FD\""
		}
		if {$legacy_speed == (6<<8)} {
			return "\"Link up 1G-HD\""
		}
		if {$legacy_speed == (5<<8)} {
			return "\"Link up 100M-FD\""
		}
		if {$legacy_speed == (3<<8)} {
			return "\"Link up 100M-HD\""
		}
		if {$legacy_speed == (2<<8)} {
			return "\"Link up 10M-FD\""
		}
		if {$legacy_speed == (1<<8)} {
			return "\"Link up 10M-HD\""
		}
		return "\"N/A\""
	}
	return "\"Link down\""
}
#this function handles E3 with WC only
proc _access_phy::linkdump_e3_wc {{phy_index 1}} {
    variable ext_phy_name     
    set indent 20s
    set cur_dev "$::current(DEV)"  

    #populate external phy information
    #ld_extphy $phy_index 0 1 1
    set shmem_base [reg read 0xa2b4]
    set port_layout [expr ([reg read [expr $shmem_base + 48]] & 0x0F000000) >> 24]

    set ACTUAL_LANE [list 0 1 2 3]
    if {$port_layout < 2} {
	if {$port_layout == 1} {
            set ACTUAL_LANE [list 2 2 0 0]
	}
        set lfmt "format \"\%-$indent \%-$indent \%-$indent\""
        set linc 2
        set labels [eval $lfmt "Item" "Port0" "Port1"]
        set headline "------------------------------------------------------"
    } else {
		if {$port_layout == 3} {
			set ACTUAL_LANE [list 1 0 3 2]
		}
		if {$port_layout == 4} {
			set ACTUAL_LANE [list 2 3 0 1]
		}
		if {$port_layout == 5} {
			set ACTUAL_LANE [list 3 2 1 0]
		}
        set lfmt "format \"\%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\""
        set linc 1
        set labels [eval $lfmt "Item" "Port0" "Port1" "Port2" "Port3"]
        set headline "----------------------------------------------------------------------------------------------"
    }
    if {[is_one_port_board]} {        
        set lfmt "format \"\%-$indent \%-$indent \""
        set linc 4
        set labels [eval $lfmt "Item" "Port0"]
        set headline "-------------------------------"
    }
    
    puts $labels
    puts $headline
    set link_line "$lfmt \"Warpcore Link\""
    set fc_line "$lfmt \"Flow Control\""
    set module_line "$lfmt \"Module type\""
    set bam_on_kr_line "$lfmt \"Bam Over KR\""
    set remote_fault_line "$lfmt \"Link Fault\""
    set link_status_line "$lfmt \"Link Status\""
    
    for {set lane_idx 0} {$lane_idx < 4} {set lane_idx [expr $lane_idx + $linc]} {
	set lane [lindex $ACTUAL_LANE $lane_idx]
        set_cur_device $lane_idx

        set link [get_link_e3 $lane]
        set link_line "$link_line $link"

        set fc [get_fc_e3 $lane $link]
        set fc_line "$fc_line $fc"

        set module [get_module_e3 $lane $link ]
        set module_line "$module_line $module"

        set bam_on_kr [get_bam_on_kr_e3 $lane $link]
        set bam_on_kr_line "$bam_on_kr_line $bam_on_kr"

        set remote_fault [get_remote_fault_e3 $lane $link]
        set remote_fault_line "$remote_fault_line $remote_fault"

        set shmem_base [reg read 0xa2b4]
	set port [PORT $lane]
        set link_status [reg read  [expr $shmem_base + 0x664 + 16*$port]]
        set link_status_line "$link_status_line $link_status"


    }
    puts "Warpcore Section:"
    puts "[eval $link_line]"
    puts "[eval $fc_line]"
    puts "[eval $module_line]"      
    puts "[eval $bam_on_kr_line]"
    puts "[eval $remote_fault_line]"
    puts "[eval $link_status_line]"


    #############  BCM84833 #############
    set bcm84833_exist 0
    set bcm84833_copper_link "$lfmt \"Copper Link\""
    set bcm84833_fiber_link "$lfmt \"Fiber Link\""
    set bcm84833_link_regs "$lfmt \"PMD/Legacy\""
    set bcm84833_fc        "$lfmt \"Flow Control LD/LP\""
    for {set lane_idx 0} {$lane_idx < 4} {set lane_idx [expr $lane_idx + $linc]} {
	#	set lane [lindex $ACTUAL_LANE $lane_idx]
        set lane $lane_idx
        set_cur_device $lane_idx

        set port [PORT $lane]
        mii_init $port 1
        if {$ext_phy_name == 84833 || $ext_phy_name == 84834} {
            set bcm84833_exist 1
            set copper_link [get_84833_link_att Copper 1 $port]
            set fiber_link [get_84833_link_att Fiber 1 $port]
            set link_regs [get_84833_link_att LinkReg 1 $port]

            set fc_ld [ld_extphy 1 $port 7 0x10]
            set fc_lp [ld_extphy 1 $port 7 0x13]
            set fc "$fc_ld/$fc_lp"
	} else {
            set copper_link "\"-\""
            set fiber_link "\"-\""
            set link_regs "\"-\""
            set fc "\"-\""
	}
        set bcm84833_copper_link "$bcm84833_copper_link $copper_link"
	set bcm84833_fiber_link "$bcm84833_fiber_link $fiber_link"
	set bcm84833_link_regs "$bcm84833_link_regs $link_regs"
	set bcm84833_fc "$bcm84833_fc $fc"
    }
    if {$bcm84833_exist} {
	puts "\nBCM84833 Section:"
	puts "[eval $bcm84833_copper_link]"
        puts "[eval $bcm84833_fiber_link]"
        puts "[eval $bcm84833_link_regs]"
	puts "[eval $bcm84833_fc]"
    }       

    #############  BCM54616 #############
    set bcm54616_exist 0
    set bcm54616_link "$lfmt \"Link\""
    set bcm54616_fc   "$lfmt \"Flow Control LD/LP\""
    for {set lane_idx 0} {$lane_idx < 4} {set lane_idx [expr $lane_idx + $linc]} {
	#	set lane [lindex $ACTUAL_LANE $lane_idx]
	set lane $lane_idx
        set_cur_device $lane_idx
		
	set port [PORT $lane]
        mii_init $port 1
			
	if {($ext_phy_name == 54616) || ($ext_phy_name == 54618)} {
	    set bcm54616_exist 1
            set link [get_54616_link $port]
            set fc_ld [ld_extphy 1 $port 0 4]
            set fc_lp [ld_extphy 1 $port 0 5]
            set fc "\"$fc_ld/$fc_lp\""
	} else {
            set link "\"-\""
            set fc "\"-\""
	}

	set bcm54616_link "$bcm54616_link $link"
	set bcm54616_fc   "$bcm54616_fc $fc"
    }
    if {$bcm54616_exist} {
	puts "\nBCM54616/8 Section:"
	puts "[eval $bcm54616_link]"
	puts "[eval $bcm54616_fc]"
    }       
    #END
    device $cur_dev -no_display
    
}

proc _access_phy::linkdump {{phy_index 1}} {
    puts "\nLink Dump:\n"
    set rc [linkdump_e1xe2e3 $phy_index]
    
    puts "\nNote: The values above represent the actual PHY/MAC status"
    return $rc
}
