# NAMESPACE nvmcfgT
# Specify any dependency on other packages
package require Tcl

# Specify the package this module provides
package provide nvmcfgT 1.0

interp alias {} dump_all_cfg {}  nvm cfg -dump
interp alias {} dump_def_cfg {}  nvmcfgT::nvm_default_dump
interp alias {} swap_ports_cfg {} nvmcfgT::swap_ports_cfg

namespace eval nvmcfgT {
# assuming meta data exists
# used for special announcements when a field is changed.
    variable PRINT
    set PRINT 1
    variable funcs
    variable ME_register_address 0x450000
    variable chip_num_reg   0xa408
    variable announcement
    variable is_default_dump false
    variable ignore_vpd_string IS_VPD_IMAGE
    variable ignore_macp_string IS_MAC_PARTITION
    variable is_vpd_string is_vpd
    variable is_macp_string is_macp
    variable is_shared_macp_string is_shared_macp
    variable no_field_is_ignored false
    variable do_not_show_shared false
    variable do_not_show_per_port false
    variable do_not_show_per_function false
    variable no_group_name "<No Group>"
    variable is_swap_ports_cfg 0

}

proc nvmcfgT::getDeviceID {} {
        global ::current
     
        set curr_did $::current(DID)
        set curr_did [expr {$curr_did}]
        return [format "0x%x" $curr_did]
}

proc nvmcfgT::isVpdField {id} {
	variable is_vpd_string
	variable nvm_cfg
	if {[info exists nvm_cfg([set id],[set is_vpd_string])]} {
		return true
	} else {
		return false
	}
}

proc nvmcfgT::isMACPField {id} {
	variable is_macp_string
	variable nvm_cfg
	if {[info exists nvm_cfg([set id],[set is_macp_string])]} {
		return true
	} else {
		return false
	}
}

proc nvmcfgT::isSharedMACPField {id} {
	variable is_shared_macp_string
	variable nvm_cfg
	if {[info exists nvm_cfg([set id],[set is_shared_macp_string])]} {
		return true
	} else {
		return false
	}
}
#getAllGroups gets all the groups
#limit_id_list is a subsset of the list that is relevant
proc nvmcfgT::getAllGroups {{limit_id_list {}} {always_get false}} {
	variable IDs
	variable no_group_name
	set result {}
	if {[llength $limit_id_list] == 0} {
		set limit_id_list $IDs
	}
	foreach id $limit_id_list {
		if {!$always_get} {
			if {[isFieldIgnored $id]} {
				continue
			}
		}
		
		if {[isGroupDefined $id]} {
			lappend result [getGroup $id]
			}
	}
	set groups [lsort -unique $result]
	set no_grouped [getGroupIds $no_group_name $limit_id_list]
	
	#if there are IDs with no groups
	if {[llength $no_grouped] > 0} {
		lappend groups $no_group_name
	}
	
	return $groups
}

proc nvmcfgT::isPrefix {text sub_text} {
	set shorter_length [string length $sub_text]
	if {$shorter_length == 0} {
	#empty string do not count
		return false
	}
	if {[string length $text] < $shorter_length} {
		return false
	}
	set sub_str_text [string range $text 0 [expr  {$shorter_length - 1}]]
	if {[string compare $sub_str_text $sub_text] == 0} {
		return true
	} else {
		return false
	}
}

proc nvmcfgT::getGroupIds {group_name {limit_id_list {}} {show_ignored false}} {
	variable nvm_cfg
	variable IDs
	if {[llength $limit_id_list] == 0} {
		set limit_id_list $IDs
	}
	set result {}
	foreach id $limit_id_list {
		#is there such an ID
		if {[lsearch $IDs $id] < 0} {
			continue
		}
		if {[isFieldIgnored $id]} {
			if {!$show_ignored} {
				continue
			}
		}
		if {[string equal $group_name [getGroup $id]]} {
			lappend result $id
		}
	}
	
	return $result
}

proc nvmcfgT::getGroup {id} {
		variable nvm_cfg
		variable no_group_name
		if {[info exists nvm_cfg([set id],group_name)]} {
			return [set nvm_cfg([set id],group_name)]
		} else {
			return $no_group_name
		}
}

proc nvmcfgT::isGroupDefined {id} {
		variable nvm_cfg
		return [info exists nvm_cfg([set id],group_name)]
}


proc nvmcfgT::isFieldIgnored {id} {
	variable no_field_is_ignored

	variable ignore_vpd_string
	variable ignore_macp_string
        variable is_default_dump
	
	variable IDs
	variable nvm_cfg
        variable nvm_config
	set svid_idx [find_id_for_field {Subsystem Vendor ID}]
	set svid_val [get_formatted_val $svid_idx]

   set handover_lldp [find_id_for_field {Handover LLDP to BMC}]	
		
	# Show Bypass slicer offset tuning ID option only for specific OEM
	if {$id == $handover_lldp} {		
		if {[string equal $svid_val 1028]} {
			return false
		} else {
			return true
		}
	}
	
        set bypass_slice_idx [find_id_for_field {Bypass slicer offset tuning}]	
		
	# Show Bypass slicer offset tuning ID option only for specific OEM
	if {$id == $bypass_slice_idx} {		
		if {[string equal $svid_val 1014]} {
			return false
		} else {
			return true
		}
	}
	
	set prevent_ocbb_idx [find_id_for_field {Prevent OCBB}]	
		
	# Show OCBB enable option only for specific OEM
	if {$id == $prevent_ocbb_idx} {		
		if {[string equal $svid_val 103c]} {
			return false
		} else {
			return true
		}
	}

	set dci_support_idx [find_id_for_field {DCi support}]	
		
	# Show DCi support option only for specific OEM
	if {$id == $dci_support_idx} {		
		if {[string equal $svid_val 103c]} {
			return false
		} else {
			return true
		}
	}

        # for default dump, we'll ignore ext image fields
        if {$is_default_dump} {
                if {[isExtImageField $id]} {
                        return true
                }
        }

	if {$no_field_is_ignored} {
		return false
	}

        if {[info exists ::current($ignore_vpd_string)] && $::current($ignore_vpd_string) == 1 && [isVpdField $id]} {
            return true
	}
        if {[info exists ::current($ignore_macp_string)] && $::current($ignore_macp_string) == 0 && [isMACPField $id]} {
	    return true
	}

	return false
}

proc nvmcfgT::isExtImageField {id} {
	if {[isMACPField $id] } {
		return true
	}

        if {[isSharedMACPField $id]} {
	return true
        }

        return false
}

proc nvmcfgT::find_id_for_field {name} {
    variable IDs
    variable nvm_cfg
    foreach curr_id $IDs {
        if {[string equal $name $nvm_cfg([set curr_id],name)]} {
            return $curr_id
        }
    }
    return -1
}

proc nvmcfgT::nvm_cfg_no_print {args} {
    variable PRINT
    set PRINT 0
    set ans [nvm_cfg $args]
    set PRINT 1
    return $ans
}

proc nvmcfgT::isE2 {} {
    variable chip_num_reg
    variable portMax

    if {$::current(VIRTUAL_NVM_MODE) == 1} {
	if {$::current(CHIP_NUM) == "578XX"} {
	    return false
	}
	return true
    }

    set chip_num_val [reg read $chip_num_reg]
    if {$chip_num_val >= 0x1651} {
        return true
    } else {
        return false
    }
}
proc nvmcfgT::isShared {id} {
	variable nvm_cfg
	return $nvm_cfg([set id],shared)
}

proc nvmcfgT::isPerFunc {id} {
	variable nvm_cfg
	if {[isShared $id]} {
		return false
	} else {
		return $nvm_cfg([set id],per_func)
	}
}

proc nvmcfgT::isPerPort {id} {
	variable nvm_cfg
	if {[isShared $id]} {
		return false
	}
	if {[isPerFunc $id]} {
		return false
	} else {
		return true
	}
}


proc nvmcfgT::showPerFunctionFields {} {
	variable do_not_show_per_function
	variable is_default_dump
	if {$do_not_show_per_function} {
		return false
	} else {
		if {$is_default_dump} {
			return false
		} else {
			return true
		}
	}
}


proc nvmcfgT::showPerPortFields {} {
	variable do_not_show_per_port

	if {$do_not_show_per_port} {
		return false
	} else {
		return true
	}
}

proc nvmcfgT::showSharedFields {} {
	variable do_not_show_shared
	if {$do_not_show_shared} {
		return false
	} else {
		return true
	}
}

proc nvmcfgT::showField {id} {	
	
	if {[isFieldIgnored $id]} {
		return false
	}
	if {[isShared $id]} {
		return [showSharedFields]
	}
	if {[isPerFunc $id]} {
		return [showPerFunctionFields]
	}
	return [showPerPortFields]
}

proc nvmcfgT::GetDefaultMaxNumOfPorts {board} {
    set hide_port_result [get_default_NumPorts $board]
    if {$hide_port_result == 1} {
        return 1
    } else {
        set port_mode_result [get_default_port_mode $board]
        if {$port_mode_result == -1} {
            return 2
        } else {
            return $port_mode_result
        }
    }
}

proc nvmcfgT::GetMaxNumOfPorts {} {
    variable nvm_cfg
    variable nvm_config

    set port_hide_id [find_id_for_field {Hide port1}]
    set port_hide_val [get_formatted_val $port_hide_id]
    if {[string equal $port_hide_val Disabled]} {
        if {[is_e3]} {
                # per CQ62980 for 57811 single port
                if {$::current(TOTAL_DEV) == 1} {
                        return 1
                }
                # Check if next device is on the same board (having the same bus number)
                set cur_dev $::current(DEV)
                set cur_bus $::current(BUS)
                if {$cur_dev < $::current(TOTAL_DEV)} {
                        device [expr {$cur_dev+1}] -no_display      
                        set next_bus $::current(BUS)
                } else {
                        #mark as invalid
                        set next_bus -1
                }
                device $cur_dev -no_display
                if {$next_bus!=$cur_bus} {                         
                        return 1
                }
                
                set port_layout [get_formatted_val [find_id_for_field {Port Layout}]]
                if {[string match 2P* $port_layout]} {
                    return 2;
                } else {
                    if {[string match 4P* $port_layout]} {
                        return 4
                    } else {
                        return 0
                    }
                }
         } else {
           # In 577xx, we support only 2 port mode
           return 2
         }
    } else {
       return 1
    }
}



#To read the current Path Num, simply read the absolute PF num (16 MSBs of ME Register)
# and then take the lsb of the result
proc nvmcfgT::GetCurrentPathNum {} {
    variable ME_register_address
    if {![isE2]} {
        return 0
    } else {
	if {$::sys(NO_PCI) || ($::current(VIRTUAL_NVM_MODE) == 1)} {
	    return [expr {$::current(DEV) & 1}]
	} else {
	    return [expr {([reg read $ME_register_address] >> 16) & 1}]
	}        
    }
}

proc nvmcfgT::isFuncZero {} {
    variable ::current
    variable ME_register_address

    if {$::current(FUNC)} {
        return false
    }

    if {$::sys(NO_PCI) || ($::current(VIRTUAL_NVM_MODE) == 1)} {
	if {![isE2]} {
	    return [expr {$::current(DEV) == 1}]
	} else {
	    return [expr {($::current(DEV) == 1) || ($::current(DEV) == 2)}]
	}
    } else {
	set me_reg [reg read $ME_register_address]
	set abs_pf_num [expr {$me_reg >> 16}]
	if {$abs_pf_num == 0} {
	    return true
	} else {
	    return false
	}
    }
}

proc nvmcfgT::gotoFuncZero {} {
    variable ::current
    while {![isFuncZero]} {
        set curr_dev $::current(DEV)
        set next_to_try [expr {$curr_dev - 1}]
        #if cannot keep going to prev device
        if {[catch {device $next_to_try -no_display}]} {
            return
        }
        if {[isFuncZero]} {
            return
        }
    }
}

proc nvmcfgT::CreateString {char iter} {
	set result ""
	for {set i 0} {$i < $iter} {incr i} {
		append result $char
	}
	return $result
}

proc nvmcfgT::CreateHeadline {text {additional_first_char {}} {full_line true}} {
set half_length [expr [string length $text] / 2]
set num_spaces [expr {40 - $half_length}]
set result [CreateString " " $num_spaces]
append result "[set additional_first_char][set text]\n"

if {$full_line} {
	set num_spaces 0
	set num_lines [expr {80 - [string length $additional_first_char]}]
} else {
	set num_spaces [expr {40 - $half_length}]
	set num_lines [string length $text]
	
}
append result [set additional_first_char][CreateString " " $num_spaces]
append result [CreateString "-" $num_lines]
append result "\n"
return $result
}

proc nvmcfgT::GroupSelection {} {

	variable no_group_name
	set available_groups [getAllGroups]

	if {[llength $available_groups] == 0} {
		return $no_group_name
	}
	set selection [CreateHeadline "NVRAM Configuration Groups:" {} false]
	for {set i 1} {$i <= [llength $available_groups]} {incr i} {
		append selection "$i .\t[lindex $available_groups [expr {$i - 1}]]\n"
	}
	
	append selection "\n\nSelect: (q to quit)"
	
	while {1} {
		puts $selection
		set user_selection [gets stdin]
		if {[string equal $user_selection "q"] || [string equal $user_selection "Q"]} {
			return -1
		}
		if {[string equal $user_selection ""]} {
			return 0
		}
		if {![string is double $user_selection]} {
		# if not a number, maybe the group name

			set user_selection [GetGroupIndex $user_selection]

		} else {
		# menu is 1 to n+1 and not 0 to n
			set user_selection [expr {$user_selection - 1}]
		}

		if {($user_selection < 0) || ($user_selection >= [llength $available_groups])} {
			puts "Bad Choice."
			continue
		}
		
		return $user_selection
	}
}

proc nvmcfgT::DoesGroupNumberExist {number} {
	set available_groups [getAllGroups]
	if {($number < 0) || ($number >= [llength $available_groups])} {
		return false
	} else {
		return true
	}
}

proc nvmcfgT::GetGroupIndex {name} {
	set available_groups [getAllGroups]
	for {set i 0} {$i < [llength $available_groups]} {incr i} {
		if {[string equal [lindex $available_groups $i] $name]} {
			return $i
		}
	}
	return -1
}

proc nvmcfgT::GetNextGroupIndex {index} {
	incr index
	if {$index < 0} {
		return 0
	}
	set available_groups [getAllGroups]
	if {$index >= [llength $getAllGroups]} {
		return 0
	}
	return $index
}

# The main function,
# when args is empty work in interactive mode, which means:
# display menu, recieved commnads to do (<id>=<value>, cancel or save)
# otherwise, for every <id>=<value> set the value, for every <id>= return id s'value
# and if received a board name as the first and only argument it will set default value for the same board
proc nvmcfgT::nvm_cfg {args} {
    variable PRINT
    # When nvm_cfg has no args display menu
    variable IDs
    variable boards
    variable announcements
    variable nvm_config
    variable shmem_config
    variable mac_partition
    variable cfg_ext_shared_block
    variable nvm_cfg
    variable is_swap_ports_cfg
    
    set announcements {}
    set batch [expr !$PRINT]
    set run true
    set save false
    set ans ""
    set dirty false
    set drv_state [is_driver_loaded]
    if {$drv_state!=0} {
        error "WARNING: nvm cfg command can not be performed while driver is running\nplease unload device $drv_state driver before using nvm_cfg"
    }
    #nvm_load retuns a list {nvm_config shmem_config}
    set config_list [nvm_load]
    set nvm_config [lindex $config_list 0]
    set shmem_config [lindex $config_list 1]
    set mac_partition [lindex $config_list 2]
    set cfg_ext_shared_block [lindex $config_list 3]
    
    set group_list [nvmcfgT::getAllGroups]
    set group_num [llength $group_list]
    set current_group_index 0
    if {$group_num > 1} {
	set enable_group_commands 1
    } else {
	set enable_group_commands 0
    }
    set first_run true
    set show_menu true
    filter_lfa
    while {$run} {
        if {[string equal $args {}]} {
            # interactive mode
			if {$enable_group_commands} {
				if {$first_run} {
					set current_group_index [GroupSelection]
					set first_run false
				}
				if {$current_group_index == -1} {
					set current_group_index 0
					set show_menu false
					
				}
				if {$show_menu} {
					display_menu false [lindex $group_list $current_group_index]	
				} else {
					display_menu false false true
				}
			} else {
				if {$show_menu} {
					display_menu
				} else {
					display_menu false false true
				}
			}
		   if {$show_menu} {
			set current_group_index [expr {($current_group_index + 1) % $group_num}]
			} else {
				set show_menu true
			}
           set commands [gets stdin]
        } else {
		set first_run false
		set batch 1
		set save true
		set run false
		set commands $args
                set show_offset_in_nvm 0
                set show_offset_in_shmem 0
                
                # Check if -offset option was requested
                if {[string first "-offset" $commands] != -1} {                        
                        set show_offset_in_nvm 1                        
                        # Check if -shmem option was requested as well
                        if {[string first "-shmem" $commands] != -1} {
                                set show_offset_in_shmem 1
                                # remove -offset and -shmem flags
                                set commands [string replace $commands 0 14]                                
                        } else {
                                # remove -offset flag
                                set commands [string replace $commands 0 7]  
                        }                        
                } elseif {[string first "-shmem" $commands] != -1} {
                        set show_offset_in_shmem 1
                        # remove -shmem flag
                        set commands [string replace $commands 0 6] 
                }
        }
        for {set i 0} {$i<[llength $commands]} {incr i} {            
            if {[regexp {^(\d+)=(.*)$} [lindex $commands $i] id id data]} {
				if {[isFieldIgnored $id]} {                                 				
					puts "Field [set id] is ignored."
                                        continue
                                } elseif {![does_id_exist_for_chip $id [getDeviceID]]} {
                                        puts "Field [set id] doesn't exist for this device."
                                        continue                               
				} else {
					if {![string equal $data {}]} {
						set res [set_feature_val $id $data]
						if {!$dirty} {
							set dirty $res
						}
					} else {
						set value [get_formatted_val $id false]
						if {![string equal $value false]} {
							lappend ans $value
						}
                                                
                                                set size [expr abs($nvm_cfg($id,elementSize))]
                                                set elements $nvm_cfg($id,numOfElements)
                                                set size [expr {$elements * $size}]
                                                #puts "call nvm_show_offset with offset=$nvm_cfg($id,offset) size=$size"
                                                if {$show_offset_in_nvm} {
                                                        nvm_show_offset $nvm_cfg($id,offset) $size
                                                }
                                                if {$show_offset_in_shmem} {
                                                        shmem_show_offset $nvm_cfg($id,offset) $size
                                                }
					}
				}
            } elseif {[lsearch -exact $boards [lindex $commands $i]]!=-1} {
                if {[string equal $args {}] || [llength $commands]>1} {
                    puts "You can only set default values without the menu and as only argument"
                    puts "Ignoring the default request"
                    continue
                }
                return [set_default_board [lindex $commands $i]]
            } elseif {[string equal [lindex $commands 0] -dump]} {
                if {[string equal $args {}] || [llength $commands]>2} {
                    puts "You can only use dump filename without the menu and as only argument"
                    puts "Ignoring the dump request"
                    continue
                }
                set filename [lindex $args 1]
                if {[string equal $filename ""]} {
                    error "When using -dump you must give a filename."                    
                }

                #mark that we are in special NVM mode (to enable getting/setting nvm values of all 8 functions, even if we are in SF mode
                set ::current(IS_SPECIAL_NVM_MODE) 1

                dump_menu $filename

                #disable special NVM mode
                set ::current(IS_SPECIAL_NVM_MODE) 0
		
                return
            } elseif {[isPrefix save [lindex $commands $i]]} {
                set save true
                set run false                
                break
            } elseif {[isPrefix cancel [lindex $commands $i]]} {
                set run false                
                break
            } elseif { $enable_group_commands} {
				if {[isPrefix "list-groups" [lindex $commands $i]]} {
					
						set tmp_group_selection [GroupSelection]
						if {$tmp_group_selection == -1} {
							set show_menu false
						} else {
							set current_group_index $tmp_group_selection
						}
						break
						
				} elseif {[isPrefix group [lindex $commands $i]]} {
						incr i
						if {[llength $commands] < 2} {
							puts "After group, you must enter a group name or number. \n You can see the full list of groups by using the list-groups command."
							set show_menu false
							set i [llength $commands]
							break
						}
						set length_first_token [expr {[string length [lindex $commands 0]] + 1}]
						set group_name [string range $commands $length_first_token [string length $commands]]
						if {![string is integer $group_name]} {
							set current_group_index [GetGroupIndex $group_name]
						} else {
							#groups are actually 0-n and not 1-n+1
							set current_group_index [expr {$group_name - 1}]
						}
						if {![DoesGroupNumberExist $current_group_index]} {
							puts "No such group: [lindex $commands $i]\n You can see the full list of groups by using the list-groups command."
							set show_menu false
							set current_group_index 0
						}
						set i [llength $commands]
						
				} else {
					set split_string [split [lindex $commands $i] -]
					if {[llength $split_string] == 2} {
						set id_info_first [lindex $split_string 0]
						set id_info_second [lindex $split_string 1]
						if {((![string is integer $id_info_first]) || ((![string is integer $id_info_second]) && !($id_info_second == {})))} {
							puts "unknown argument [lindex $commands $i] : you can use <id>- or <id>-<id>."
							set show_menu false
						} else {
							if {$id_info_second == {}} {
								set id_info_second $id_info_first
							}
							set show_menu false
							ShowGroupInfo $id_info_first $id_info_second
							incr i
						}
					} else {
					                puts "unknown argument - [lindex $commands $i], ignored."
							puts "The allowed arguments are <id>=, <id>=<value> or <board type>"							
							set show_menu false
					
					}

				}
			
			} else {
                puts "unknown argument - [lindex $commands $i], ignored."
                puts "The allowed arguments are <id>=, <id>=<value> or <board type>"                
            }

        }
    }
    if {$save && $dirty} {
        if {!$batch} {
            puts "Config saved."
        }
        if {![string equal $announcements {}] &&
            ! $is_swap_ports_cfg} {
            puts -nonewline $announcements
        }
	nvm_store $nvm_config $shmem_config $mac_partition $cfg_ext_shared_block
    } else {
        if {!$batch} {
            puts "Config not saved."
        }        
        nvm_free_structs $nvm_config $shmem_config $mac_partition $cfg_ext_shared_block
    }
    set announcements {}
    if {![string equal $args {}]} {
        return $ans
    }        
    
    return
}

proc nvmcfgT::ShowGroupInfo {first last} {
	variable IDs
	if {$first > $last} {
			set temp $first
			set first $last
			set last $temp
	}
	set limiting_list {}
	set not_exists_text ""
	for {} {$first <= $last} {incr first} {
		if {[lsearch $IDs $first] > -1} {
			lappend limiting_list $first
		} else {
			append not_exists_text "ID $first Does not exist.\n"
		}
	}
	
	
	set result ""
	if {[llength $limiting_list] > 0} {
		set relevant_groups [getAllGroups $limiting_list true]
		foreach group $relevant_groups {
			set relevant_ids [getGroupIds $group $limiting_list true]
			foreach id $relevant_ids {
				if {[isFieldIgnored $id]} {
					append result "ID $id Does not exist.\n"
				} else {
                                        append result [CreateHeadline "Group: $group (Group [expr {[GetGroupIndex $group] + 1}])"]
					append result [get_feature_line $id true]
                                        append result "\n\n"
				}
			}			
		}
	}
	append result $not_exists_text
	catch {puts $result}

}


proc nvmcfgT::nvm_default_dump {output board} {
    variable default_dump_params
    variable is_default_dump
	variable no_field_is_ignored
	set old_no_field_ignored_value $no_field_is_ignored
	set no_field_is_ignored true
    array set default_dump_params {}
    set chip [get_default_device_id $board]
    if {$chip == -1} {
        puts "Error: Cannot determine DeviceID for Board $board"
        return
    }

    set total_dev [get_default_NumPorts $board]
    if {$total_dev == -1} {
        puts "Error: Cannot determine Num Ports for Board $board"
        return
    }
    set funcs $total_dev
    set default_dump_params(BOARD) $board
    set default_dump_params(DID) $chip
    set default_dump_params(TOTAL_DEV) $total_dev
    set default_dump_params(FUNCS) $funcs
    set is_default_dump true
    dump_menu $output true default_dump_params
    set is_default_dump false
    set no_field_is_ignored $old_no_field_ignored_value
    return
}

proc nvmcfgT::isFieldExcluded {id {did -1}} {
	variable nvm_cfg
	if {$did == -1} {
		set did [getDeviceID]
	}
	variable ChipIdToChipNum
	if {($did=="false") || ($did==0)} {
		set chip_num false
	} else {
		set did [format "0x%x" [expr $did]]
		set chip_num $ChipIdToChipNum($did)
	}
	if {[info exists nvm_cfg($id,exChipNumAttrs)]} {
		if {([lsearch $nvm_cfg($id,exChipNumAttrs) $chip_num]!=-1)} {
			return 1;
		}

		for {set i 0} {$i<[llength $nvm_cfg($id,exChipNumAttrs)]} {incr i} {	   
			set tcl_var nvmcfg_ignore_[lindex $nvm_cfg($id,exChipNumAttrs) $i]
			if {([info exists ::current(nvmcfg_ignore_[lindex $nvm_cfg($id,exChipNumAttrs) $i])])} {
				if {$::current(nvmcfg_ignore_[lindex $nvm_cfg($id,exChipNumAttrs) $i]) == 1} {
					return 1;
				}
			}
		}
	}

	return 0;
}

# check the defaults for a specic board for requested ids
# first look for the default in that board's array (id as key)
# otherwise checks if the value exists per port (id,port as key)
# otherwise checks if the value exists in the default's array
# otherwise checks if the value exists per port in the default's array (id,port as key)
# Returned Result: 0 - for success, otherwise, last ID with error
proc nvmcfgT::nvm_chk {board {fix false}} {


    variable nvm_cfg
    variable portMax
    variable funcs
    variable chkIDs
    variable boards
    variable nvm_config
    variable shmem_config
    variable mac_partition
    variable cfg_ext_shared_block
    
    set errorStr {}
	set returned_result 0
    if {[lsearch -exact $boards $board]==-1} {
        error "Unknown board to check."
    }
    upvar #0 nvmcfgT::$board boardD
    upvar #0 nvmcfgT::default defaultD
    is_driver_loaded
    set curr_device $::current(DEV)
    if {[string equal $fix -fix]} {
        set fix true
    } elseif {![string equal $fix false]} {
        puts "unknown argument: $fix"
    }
    # setting the port to be port0
    gotoFuncZero
    set port0 $::current(DEV)
    device $curr_device -no_display

    # nvm load returns a lst {nvm_config shmem_config}
    set config_list [nvm_load]
    set nvm_config [lindex $config_list 0]
    set shmem_config [lindex $config_list 1]
    set mac_partition [lindex $config_list 2]
    set cfg_ext_shared_block [lindex $config_list 3]
    
    set portMax [GetMaxNumOfPorts]

    for {set port_num 0} {$port_num<$portMax && $port_num<$funcs} {incr port_num} {
        set dirty false
        device [expr $port0+$port_num] -no_display
        if {[isE2]} {
            ## even numbered devices should be in port 0
            ## odd numbered devices should be in port 1
            if {$port_num > 0 && [GetCurrentPathNum]!=[expr {$port_num & 1}]} {
                continue
            }

        } else {
            # in E1H the lsb is the port num
            if {$port_num > 0 && [expr $::current(FUNC)&1]==0} {
                continue
            }
        }
        #nvm_load retuns a list {nvm_config shmem_config}
	set config_list [nvm_load]
	set nvm_config [lindex $config_list 0]
	set shmem_config [lindex $config_list 1]
	set mac_partition [lindex $config_list 2]
        set cfg_ext_shared_block [lindex $config_list 3]

        for {set i 0} {$i<[llength $chkIDs]} {incr i} {
            set id [lindex $chkIDs $i]
			if {[isFieldIgnored $id]} {
				continue
			}
			if {[isExtImageField $id]} {
				continue
			}
            set defVal {}
            regexp {^(\d+)} $id idNoV idNoV
            if {[isFieldExcluded $idNoV]} {
                continue
            }
            if {$nvm_cfg($idNoV,shared) && $port_num>0} {
                continue
            }
			set no_default false
			set defVal [verify_default_value $id $board $port_num no_default]

			if {$no_default} {
				error "couldn't find a value to verify for id $id , port $port_num"
			}

            if {[string equal $id $idNoV]} {
                set formattedDefVal [format_values $id [lreverse [checkUserInput $id $defVal]]]
                if {[string equal -nocase $formattedDefVal $defVal]} {
                    set prefixD ""
                    set prefixE ""
                } else {
                    set prefixD [set defVal]=
                    set prefixE [get_formatted_val $id false]=
                }
                if {![string equal [get_formatted_val $id] $formattedDefVal]} {
                    if {$nvm_cfg($id,shared)} {
                        append errorStr "ID $id: \"$nvm_cfg($id,name)\" mismatch:\n"
                    } else {
                        append errorStr "ID $id for port $port_num: \"$nvm_cfg($id,name)\" mismatch:\n"
                    }
					set returned_result $id
                    append errorStr "   Expected [set prefixD]$formattedDefVal\n"
                    append errorStr "   Existing [set prefixE][get_formatted_val $id]\n"
                }
                if {$fix} {
                    set_feature_val $id $defVal
                    set dirty true
                }
            } else {
                if {[string first $defVal [get_formatted_val $idNoV]]!=0} {
                    if {$nvm_cfg($idNoV,shared)} {
                        append errorStr "ID $idNoV: \"$nvm_cfg($idNoV,name)\" mismatch\n"
                    } else {
                        append errorStr "ID $idNoV for port $port_num: \"$nvm_cfg($idNoV,name)\" mismatch\n"
                    }
                    set returned_result $id
                    append errorStr "   Expected prefix $defVal\n"
                    append errorStr "   Exsiting prefix [string range [get_formatted_val $idNoV] 0 [expr [string length $defVal]-1]]\n"
                }
            }

        }
        if {$fix && $dirty} {
            nvm_store $nvm_config $shmem_config $mac_partition $cfg_ext_shared_block	
        }
    }
    device $curr_device -no_display
   
    
    if {[string equal $errorStr {}]} {
        puts "Nvm chk hasn't found any errors in the configuration"
		return $returned_result
    } else {
        puts "The following errors were found in the configuration:\n$errorStr"
		return $returned_result
    }
}

# this sub routine displays the menu
# it stops upon recieving 'q'
proc nvmcfgT::display_menu {{show_all true} {group_id false} {show_only_options false}} {
    variable IDs
    set lineCount 0
	set groups_list [getAllGroups]
	set group_num [llength $groups_list]
	if {!$show_only_options} {
		if {$show_all} {
			set list_of_IDs $IDs
		} else {
			set list_of_IDs [getGroupIds $group_id]
			if {[llength $list_of_IDs] == 0} {
				puts "Bad Group $group_id"
			}
			append	menuD [CreateHeadline "Group: $group_id (Group [expr {[GetGroupIndex $group_id] + 1}])"]
			
			set lineCount 2
		}
	
	
	
		for {set i 0} {$i<[llength $list_of_IDs]} {incr i} {
			if {[isFieldIgnored [lindex $list_of_IDs $i]]} {
				continue
			}
			set next_item [get_feature_line [lindex $list_of_IDs $i]]
			set next_lCount [regexp -all {\n} $next_item]
			if {$next_lCount>=22} {
				error "Item [lindex $list_of_IDs $i] is too big to fit in one screen."
			} elseif {[expr {$lineCount+$next_lCount}]<22} {
				append menuD $next_item
				incr lineCount $next_lCount
			} else {
				append menuD "more.., 'q' to quit"
				puts -nonewline $menuD
				set menuD $next_item
				set lineCount $next_lCount
				set ch [getc]
				puts {}
				if {[string equal $ch "q"]} {
					set menuD {}
					break
				}
			}
		}
	}
    append menuD { Choice (<option>=<value>, save, cancel}
	if {$group_num > 1} {
		append menuD {, group <group name>, list-groups, <ID>-<optional ID>)}
	} else {
		append menuD {)}
	}
	
    puts $menuD

}

proc nvmcfgT::dump_menu {output {defaultdump false} {defdumpparams none}} {
    variable IDs
    variable funcs

    upvar $defdumpparams def_params
    set outFile [open $output w]

	if {$defaultdump} {
		set board_name [set def_params(BOARD)]
	} else {
		set board_name "<board name>"
	}


    set manuf_ver_idx [find_id_for_field {Manufacture kit version}]

    puts $outFile "#######################################################"
    puts $outFile "# DO NOT EDIT!!! THIS FILE IS AUTOMATICALLY GENERATED."
    puts $outFile "#"
    puts $outFile "#Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved"
    puts $outFile "#Copyright(c) 2014-2015 QLogic Corporation, all rights reserved"
    puts $outFile "#Proprietary and Confidential Information."
    puts $outFile "#"
    puts $outFile "#This source file is a property of QLogic Corporation, and"
    puts $outFile "#may not be copied or distributed in any isomorphic form without"
    puts $outFile "#the prior written consent of QLogic Corporation."
    puts $outFile "#"
    puts $outFile "#Engineer:"
    if {$::sys(IS_EDEBUG) == 0} {
	puts $outFile "#Creation Date: [clock format [clock seconds] -format {%b. %d, %Y %I:%M:%S %p}]"
    }
    puts $outFile "#Description: This file contains the NVRAM configuration for $board_name"
    puts $outFile "#             To incorporate these settings into the board's NVRAM, simply"
    puts $outFile "#             source this file in Ediag as follows"
    puts $outFile "#                 source <config_file>"
    puts $outFile "#"
    puts $outFile "#Design Name: $board_name"
    puts $outFile "#Diagnostic Tool Version: [version]"
    puts $outFile "#Software Release:"
    puts $outFile "#Customer Name:"
    puts $outFile "#Revision History:"
    puts $outFile "#"
    puts $outFile "#Manufacture kit version: [get_formatted_val $manuf_ver_idx]"
    puts $outFile "#Additional Comments:"
    puts $outFile "\n\n\n"
    if {$defaultdump} {
        set curr_device 1
        set total_dev $def_params(TOTAL_DEV)
        set funcs $def_params(FUNCS)
        } else {
        set curr_device $::current(DEV)
        set total_dev $::current(TOTAL_DEV)
    }

    

    for {set device_base $curr_device} {$device_base <= $total_dev} {incr device_base $funcs} {
	
        if {!$defaultdump} {
            device $device_base -no_display
        }
        set shared {}
        set perPorts {}
        set perDevMenu {}

	set perFunctions {}
        #funcs should be updated in print_configuration
        if {[catch {print_configuration $device_base shared perPorts perFunctions perDevMenu $defaultdump def_params} errMsg]} {
            close $outFile
            error $errMsg
        }

        for {set i 0} {$i<[llength $perDevMenu]} {incr i} {
            puts $outFile "################\n### Device [expr $i+$device_base] ###\n################\n"
            puts $outFile [lindex $perDevMenu $i]
            puts $outFile "\n\n"
        }

        puts $outFile "##############\n### Shared ###\n##############\n"
        puts $outFile "if {\[catch {set x \$::current(IS_SPECIAL_NVM_MODE)}\]} {} else {set ::current(IS_SPECIAL_NVM_MODE) 1}"                 
        puts $outFile "if {\[catch {set x \$::current(IS_SPECIAL_NVM_MODE)}\]} {device [expr {$device_base-$curr_device+1}]} else {set ::current(SPECIAL_NVM_MODE_FUNC_NUM) 0}"        
        puts $outFile "nvm cfg \\"
        puts $outFile $shared
        puts $outFile "\n\n"
	set max_count [llength $perFunctions]
	if {[llength $perPorts] > $max_count} {
		set max_count [llength $perPorts]
	}
        for {set i 0} {$i<[llength $perFunctions]} {incr i} {
            puts $outFile "################\n### Device [expr $i+$device_base] ###\n################\n"
            
            if {$i} {
                 puts $outFile "if {\[catch {set x \$::current(IS_SPECIAL_NVM_MODE)}\]} {device [expr {$i+$device_base-$curr_device+1}]} else {set ::current(SPECIAL_NVM_MODE_FUNC_NUM) [expr $i]}"                
            }
            puts $outFile "nvm cfg \\"
	    set curr_line ""
	    if {$i < [llength $perPorts] } {
		set curr_line [lindex $perPorts $i]
	    }

	    if {$i < [llength $perFunctions] } {
		set curr_line "[set curr_line][lindex $perFunctions $i]"
	    }
	    puts $outFile $curr_line
            puts $outFile "\n\n"
        }
    }
    if {!$defaultdump} {
        device $curr_device -no_display
    }

    puts $outFile "if {\[catch {set x \$::current(IS_SPECIAL_NVM_MODE)}\]} {} else {set ::current(IS_SPECIAL_NVM_MODE) 0}"
    puts $outFile "device 1"
    close $outFile

}

proc nvmcfgT::initGroupDisplay {} {
	variable group_display_info
	if {[info exists group_display_info]} {
		array unset group_display_info
	}
	
	array set group_display_info {}
	set group_display_info(groups) {}

}

proc nvmcfgT::addIdInfo {id data} {
	variable group_display_info
	if {$data == {}} {
		return
	} else {
		set group_name [getGroup $id]
		if {[lsearch $group_display_info(groups) $group_name] == -1} {
			lappend group_display_info(groups) $group_name
		}
	}
	append group_display_info($group_name) $data
}

proc nvmcfgT::printGroupDisplay {} {
		variable group_display_info
		set groups_list $group_display_info(groups)
		if {[llength $groups_list] == 0} {
			return {}
		}
		
		foreach group_name $groups_list {
			append result [CreateHeadline "Group: $group_name (Group [expr {[GetGroupIndex $group_name] + 1}])"]
			append result $group_display_info($group_name)
			append result "\n\n"
		}
		return $result
}

proc nvmcfgT::print_configuration {dev sharedVar perPortsVar perFunctionVar perDevMenuVar {def_dump false} {def_params_var none}} {
    upvar $sharedVar shared
    upvar $perPortsVar perPorts
    upvar $perFunctionVar perFunction
    upvar $perDevMenuVar perDevMenu
    upvar $def_params_var def_params

    variable do_not_show_shared
    variable do_not_show_per_port
    variable do_not_show_per_function

    set old_show_shared $do_not_show_shared
    set old_show_per_port $do_not_show_per_port
    set old_show_per_function $do_not_show_per_function


    variable IDs
    variable nvm_cfg
    variable portMax
    variable funcs
    variable noDefault
    variable ::current
    variable nvm_config
    variable shmem_config
    variable mac_partition
    variable cfg_ext_shared_block
    

    # setting the port to be port0
    if {!$def_dump} {
        set cur_dev $::current(DEV)
        gotoFuncZero
        set port0 $::current(DEV)
        device $cur_dev -no_display
        # counting the functions
        set error_dev [is_driver_loaded]
        if {$error_dev} {
            error "NONE OF THE DEVICES IS ALLOWED TO BE WITH DRIVER LOADED.\nRecheck device $error_dev."
        }
	
       	# Setting portMax
	set portMax [GetMaxNumOfPorts]
    } else {
        set port0 1
        set portMax [GetDefaultMaxNumOfPorts $def_params(BOARD)]
        set funcs $portMax
    }

    set funcs 8

    for {set port_num 0} {$port_num<$funcs} {incr port_num} {
	
	if {$port_num >= $portMax} {
		set do_not_show_per_port true
	}
	if {$port_num > 0} {
		set do_not_show_shared true
	}
        set currPort {}
	set currFunc {}
        if {!$def_dump} {
            set ::current(SPECIAL_NVM_MODE_FUNC_NUM) $port_num
            set curr_func $::current(FUNC)
            if {[isE2]} {
                ## even numbered devices should be in port 0
                ## odd numbered devices should be in port 1                
                if {$port_num > 0 && [expr $::current(SPECIAL_NVM_MODE_FUNC_NUM) & 1]!=[expr {$port_num & 1}]} {                    
                    continue
                }
            } else {
                # in E1H the lsb is the port num
                if {$port_num > 0 && [expr $::current(SPECIAL_NVM_MODE_FUNC_NUM)&1]==0} {
                    continue
                }
            }
        } else {
            set curr_func $port_num
        }
		initGroupDisplay
        set menuD {}
        if {!$def_dump} {
            #nvm_load retuns a list {nvm_config shmem_config ...}
	    set config_list [nvm_load]
	    set nvm_config [lindex $config_list 0]
            set shmem_config [lindex $config_list 1]
	    set mac_partition [lindex $config_list 2]
	    set cfg_ext_shared_block [lindex $config_list 3]
        }
	
	set any_id_shown false
        for {set i 0} {$i<[llength $IDs]} {incr i} {
            #puts $i
	    if {![showField [lindex $IDs $i]]} {
		continue
	    }
	    if {[isFieldIgnored [lindex $IDs $i]]} {
		continue
	    }
	    set any_id_shown true
		set curr_id [lindex $IDs $i]
            if {!$def_dump} {
		addIdInfo $curr_id [get_feature_line $curr_id]
            } else {
		addIdInfo $curr_id [get_default_feature_line $curr_id $def_params(DID) $def_params(BOARD) $port_num]
            }
        }
	#if now id was shown, skip this device
	if {![set any_id_shown]} {                
		continue
	}
	set menuD [printGroupDisplay]
        set menuD #[string map {"\n" "\n#"} $menuD]
        lappend perDevMenu $menuD
	set any_id_shown false
        for {set i 0} {$i<[llength $IDs]} {incr i} {
		if {![showField [lindex $IDs $i]]} {
		continue
		}
		if {[isFieldIgnored [lindex $IDs $i]]} {
			continue
		}
            set id [lindex $IDs $i]
            if {!$def_dump} {
                set dev_id [getDeviceID]
            } else {
                set dev_id $def_params(DID)
            }
            if {$nvm_cfg($id,shared) && $port_num>0 || [lsearch -exact $noDefault $id]>-1 || \
                ([isFieldExcluded $id $dev_id])} {
                continue
            }
            if {!$def_dump} {
                set formatted_val [get_formatted_val $id false]
		set any_id_shown true
            } else {
                set no_def_var false
				if {$no_def_var} {
					continue
				}
		set any_id_shown true
                set formatted_val [get_default_val $id $def_params(BOARD) $port_num no_def_var false]
            }
            if {$nvm_cfg($id,shared)} {
                append shared "{$id=[set formatted_val]}\\\n"
            } else {
		if {[isPerFunc $id]} {
			append currFunc "{$id=[set formatted_val]}\\\n"
		} else {
			append currPort "{$id=[set formatted_val]}\\\n"
		}
            }
        }
	if {[set any_id_shown]} {
		lappend perPorts $currPort
		lappend perFunction $currFunc
	}
    }


    set do_not_show_shared $old_show_shared
    set do_not_show_per_port $old_show_per_port
    set do_not_show_per_function $old_show_per_function
       
}

# retruned an aligned line for the requested id
proc nvmcfgT::get_feature_line {id {show_not_applicable false} {force_not_applicable false}} {
        variable nvm_cfg
	set is_applicable true
	if {$force_not_applicable} {
		set is_applicable false
	}
        if {[isFieldExcluded $id]} {
           set is_applicable false 
        }
        
	if {!$is_applicable && !$show_not_applicable} {
		return {}
	}
        set line [format "%3d" $id]:

        if {$nvm_cfg($id,shared)} {
            append line { [S] }
        } else {
		set appended 0
		if {[info exists nvm_cfg($id,per_func)]} {
			if {$nvm_cfg($id,per_func)} {
				append line { [F] }
				set appended 1
			}

		}
		if {$appended == 0} {
			append line { [P] }
		}
        }
        append line "[align_feature_desc $id]: "
	if {$is_applicable} {
		append line [get_formatted_val $id]
	} else {
		append line "NOT APPLICABLE"
	}
        append line "\n"
        return $line
}

# retruned the default aligned line for the requested id
proc nvmcfgT::get_default_feature_line {id device_id board port_num} {
    variable nvm_cfg
    if {![does_id_exist_for_chip $id $device_id]} {
        return {}
    }
    set line [format "%2d" $id]:

    if {$nvm_cfg($id,shared)} {
        append line { [S] }
    } else {
        set appended 0
		if {[info exists nvm_cfg($id,per_func)]} {
			if {$nvm_cfg($id,per_func)} {
				append line { [F] }
				set appended 1
			}

		}
		if {$appended == 0} {
			append line { [P] }
		}
    }
    append line "[align_feature_desc $id]: "
    variable is_default_value
    append line [get_default_val $id $board $port_num is_no_default_value true]
    append line "\n"
    if {!$is_no_default_value} {
        return $line
    } else {
        return {}
    }
}

# transfer an allowed list to a string with all values and their enums
proc nvmcfgT::allowedListToString {id} {
    variable nvm_cfg
    upvar #0 nvmcfgT::$nvm_cfg($id,allowedList) aList
    set ans ""
    set keys [lsort -integer [array names aList]]
    set formats [stringFormatToList $id]
    if {[regexp {%\d*x} [lindex $formats 0]]} {
        set base %x
    } else {
        set base %d
    }
    for {set i 0} {$i<[llength $keys]} {incr i} {

        if {[string equal $nvm_cfg($id,listType) "BitField"]} {
            append ans [format "%s(0x%x)" $aList([lindex $keys $i]) [lindex $keys $i]]
        } else {
            append ans [format "%s($base)" $aList([lindex $keys $i]) [lindex $keys $i]]
        }
        if {[expr {$i+1}]!=[llength $keys]} {
            append ans ", "
        }
    }
    append ans "\}"
    return $ans
}

# align the feature description to fit the screen
proc nvmcfgT::align_feature_desc {id} {
    variable nvm_cfg
    set ans [align_text $nvm_cfg($id,name) "        " 47]
    if {[info exists nvm_cfg($id,allowedList)]} {

        set aList [allowedListToString $id]

        set lineSize [string length [string trimright $ans " "]]
        if {[expr [string length $aList]+$lineSize+2<48]} {
             set ans [format "%s \{%s" [string trimright $ans " "] [align_text $aList "        " [expr 46-$lineSize]]]
        } else {
            set ans [format "%s\n        \{%s" [string trimright $ans] [align_text $aList "         " 47]]
        }

    }
    return $ans
}
# returned the id's value, if verbose is true return the string from the allowed list (if exists)
# otherwise, return the numeric value
# (if the value is a string, it will be return as is.)
proc nvmcfgT::get_formatted_val {id {verbose true}} {
    variable IDs
    variable nvm_cfg
    variable nvm_config
    
    if {![does_id_exist_for_chip $id [getDeviceID]]} {
        puts "Unknown ID - $id, ignoring $id=."
        return false
    }

    set values [lreverse [get_elements $id]]
    return [format_values $id $values $verbose]
}

# Making sure that the device ID is not in the excluded list for the specified ID.
proc nvmcfgT::does_id_exist_for_chip {id chip} {
    variable IDs
    variable nvm_cfg

    if {[lsearch -exact $IDs $id]==-1 || ([isFieldExcluded $id $chip])} {
        return false
    }

    return true

}
#Return the default value for Number of ports
proc nvmcfgT::get_default_NumPorts {board} {
    variable IDs
    variable nvm_cfg
    variable no_def_var
    foreach curr_id $IDs {
        if {[string equal "Hide port1" $nvm_cfg([set curr_id],name)]} {
            set result [get_default_val $curr_id $board 0 no_def_var]
            if {$no_def_var} {
                return -1
            } else {
                if {[string equal "Disabled" $result]} {
                    return 2
                } else {
                    return 1
                }
            }
        }
    }
    return -1
}

proc nvmcfgT::get_default_port_mode {board} {
    variable IDs
    variable nvm_cfg
    variable no_def_var
    foreach curr_id $IDs {
        if {[is_e3]} {
            if {[string equal "Port Layout" $nvm_cfg([set curr_id],name)]} {
                set result [get_default_val $curr_id $board 0 no_def_var]
                if {[string match 2P* $result]} {
                    return 2;
                } else {
                    if {[string match 4P* $result]} {
                        return 4;
                    } else {
                        return 0;
                    }
                }
            }
        } else {
            if {[string equal "Port Mode" $nvm_cfg([set curr_id],name)]} {
                set result [get_default_val $curr_id $board 0 no_def_var]
                if {$no_def_var} {
                    return -1
                } else {
                    return $result
                }
            }
        }
    }
    return -1
}

#Return the sub-vendor Vendor id
proc nvmcfgT::filter_lfa {} {
    variable IDs
    variable nvm_cfg
    variable no_def_var
    set curr_id [find_id_for_field {Subsystem Vendor ID}]
    set val [get_formatted_val $curr_id]
    if {[string equal $val 1137]} {
       set ::current(nvmcfg_ignore_LFA) 0
    } else {
       set ::current(nvmcfg_ignore_LFA) 1
    }
}


#Return the default value for device ID for the specified Board
proc nvmcfgT::get_default_device_id {board} {
    variable IDs
    variable nvm_cfg
    variable no_def_var
    foreach curr_id $IDs {
        if {[string equal "Vendor Device ID" $nvm_cfg([set curr_id],name)]} {
            set result "0x[get_default_val $curr_id $board 0 no_def_var]"
            if {$no_def_var} {
                return -1
            } else {
                return $result
            }
        }
    }
    return -1
}
# Verifies that a specific board and port have a default val for the chosen ID. (no_def_var is appropriately
# updated.)
# The returned value is the default val
proc nvmcfgT::verify_default_value {id board port_num no_def_var} {
	variable IDs
    variable nvm_cfg
    variable boards



    upvar #0 nvmcfgT::$board boardD
    upvar #0 nvmcfgT::default defaultD
    upvar 1 $no_def_var no_default

	set no_default false

    if {[lsearch -exact $boards $board]==-1} {
        set no_default true
        error "Unknown board to check."
	}

	if {[info exist boardD($id)]} {
        return $boardD($id)
    } elseif {[info exists boardD($id,$port_num)]} {
        return $boardD($id,$port_num)
    } elseif {[info exist defaultD($id)]} {
        return $defaultD($id)
    } elseif {[info exists defaultD($id,$port_num)]} {
        return $defaultD($id,$port_num)
    } else {
        set no_default true
        return false
    }
}

# returned the id's default value, if verbose is true return the string from the allowed list (if exists)
# otherwise, return the numeric value
# (if the value is a string, it will be return as is.)
proc nvmcfgT::get_default_val {id board port_num no_def_var {verbose true}} {
    variable IDs
    variable nvm_cfg
    variable boards
    variable default_dump_params
    set did false
    if {[info exists default_dump_params(DID)]} {
	set did $default_dump_params(DID)
    }

    upvar #0 nvmcfgT::$board boardD
    upvar #0 nvmcfgT::default defaultD
    upvar 1 $no_def_var no_default

    set no_default false

    set defVal [verify_default_value $id $board $port_num no_default]

    if {[lsearch -exact $boards $board]==-1} {
        set no_default true
        error "Unknown board to check."
	}

    if {$no_default} {
        return false
    }

    set values [checkUserInput $id $defVal $did]
    if {[string equal $values false]} {
	set no_default true
        return false
    }
    return [format_values $id [lreverse $values] $verbose]
}

proc nvmcfgT::updateNumOfElements {id} {
    variable nvm_cfg
    set mac_size [getMacSize $id]
    if {$mac_size > 0} {
        if {$nvm_cfg([set id],numOfElements) == 6} {
            set nvm_cfg([set id],numOfElements) $mac_size
        }
    }
}

proc nvmcfgT::format_values {id values {verbose true}} {
    variable nvm_cfg
    variable IDs


    updateNumOfElements $id
    set formatVal [string trim $nvm_cfg($id,stringFormat) "^$"]

    regsub -all {([(]\?:.+?[)]\?)} $formatVal {} formatVal
    set formats [stringFormatToList $id]
    if {[string equal [lindex $formats 0] "string"]} {
        set val ""
        set values [lreverse $values]
		if {![is_string_array $id]} {
			for {set i 0} {$i<$nvm_cfg($id,numOfElements) && [lindex $values $i]!=0} {incr i} {
				append val [format %c [lindex $values $i]]
			}
			regsub {[(][^)]+[)]} $formatVal $val formatVal
		} else {
		# array of strings
		set val ""
		for {set str_num 0} {$str_num < $nvm_cfg($id,numOfElements)} {incr str_num} {

			if {$str_num > 0} {
				append val ":"
			}
			set curr_values [lindex $values $str_num]
			set i 0
			for {} {$i< [llength $curr_values] && [lindex $curr_values $i]!=0} {incr i} {
				append val [format %c [lindex $curr_values $i]]
			}
			if {$i == 0} {
				append val ""
			}

			set formatVal $val
		}
		}
    } elseif {[llength $formats]==[llength $values]} {
        if {[info exist nvm_cfg($id,allowedList)] && [string equal $nvm_cfg($id,listType) "Enum"] && $verbose} {
            variable $nvm_cfg($id,allowedList)
            set formatVal ""
            for {set i 0} {$i<[llength $formats]} {incr i} {
                if {[info exist $nvm_cfg($id,allowedList)([lindex $values $i])]} {
                    append formatVal [subst $$nvm_cfg($id,allowedList)([lindex $values $i])]
                } else {
                    append formatVal [format "Unknown (0x%X)" [lindex $values $i]]
                }
                if {[expr $i+1<[llength $formats]]} {
                    append formatVal :
                }
            }
        } else {
            for {set i 0} {$i<[llength $formats]} {incr i} {
                regsub {[(][^)]+[)]} $formatVal [format [string tolower [lindex $formats $i]] [lindex $values $i]] formatVal
            }
        }
    } else {
        error "The string format define different amount of elements ([llength $formats]) than the feature holds [llength $values].[lindex $formats 0]"
    }
    return $formatVal
}

# recieve the id and data enter by the user
# verify the validity of the data and if so set the data
proc nvmcfgT::set_feature_val {id data} {
	variable nvm_config
	variable shmem_config
	
	if {[isFieldIgnored $id]} {
		return false
	}
	set values [checkUserInput $id $data]
	if {[string equal $values "false"]} {
	    return false
	} else {
	    set_elements $id $values
	    return true
	}
}

# verify the user input by checking the following:
# The id is known.
# The data fits the regualr expression (string format)
# if an allowed list exists ensure data is a member of it.
# ensure data doesn't exceed the field
# ensure data is a multiple of <multiple of> if exists
proc nvmcfgT::checkUserInput {id data {given_did false}} {
    variable nvm_cfg
    variable IDs
    variable noDefault

    updateNumOfElements $id
    
    # if ID is not in the IDs list return false.
    # This call must be early in the function because otherwise function will
    # exit with error other than return false, and then if a nvm_cfg script
    # will include one ID not in the list other assignments will not take place.
    # It is important for us that other assignments will take place because if
    # script was generated before we removed options from nvm cfg it wouldn't
    # pass
    if {[lsearch -exact $IDs $id]==-1 || ([isFieldExcluded $id $given_did])} {
        puts "Unknown ID - $id, ignoring $id=$data."
        return false
    }

	if {[isFieldIgnored $id]} {
		puts "Field $id is currently ignored."
		return false
	}

    upvar #0 nvmcfgT::default defaults
    set regEx $nvm_cfg($id,stringFormat)
    set formats [stringFormatToList $id]
    set elements $nvm_cfg($id,numOfElements)

    if {[string equal $given_did false]} {
            set given_did [getDeviceID]
    }
    if {[string equal [lindex $formats 0] "string"]} {
        set  elements 1
    }

    if {[regexp $regEx $data]} {
        for {set i 0} {$i<$elements} {incr i} {
            lappend vars val($i)
        }

        eval [subst {regexp \{$regEx\} \{$data\} dc $vars}]

        if {[string equal [lindex $formats 0] "string"]} {
            set i 0
            for {} {$i<[string length $val(0)]} {incr i} {
               lappend ans [scan [string index $val(0) $i] %c]
            }
            if {$i!=$nvm_cfg($id,numOfElements)} {
                lappend ans 0
            }
            return $ans
        } else {
            for {set i 0} {$i<$elements} {incr i} {

                set value [scan $val($i) [string tolower [lindex $formats $i]]]
                lappend ans $value
                if {[info exist nvm_cfg($id,allowedList)] && [string equal $nvm_cfg($id,listType) "Enum"]} {
                    variable $nvm_cfg($id,allowedList)
                    upvar #0 nvmcfgT::$nvm_cfg($id,allowedList) aList
                    if {![info exist $nvm_cfg($id,allowedList)($value)]} {
                        puts "The value for id $id, is illegal, the value must be member of the allowed list."
                        puts "The list: \{[align_text $aList "        " 69]"
                        puts "Your value was: $val($i)"
                        return false
                    } else {
                        continue
                    }
                }
                if {$value>=[expr pow(2,$nvm_cfg($id,elementSize))]} {
                    puts "The value of element $i for id $id, is illegal, the value must be less than [expr int(pow(2,$nvm_cfg($id,elementSize)))]."
                    return false
                }
                #special handling - TMP - fields 93/95 should be able to receive values 0..64
                if {($id == 93 || $id == 95) && $value>64} {
                    puts "The value of element $i for id $id, is illegal, the value must be less than 65."
                    return false
                }
                if {[info exists nvm_cfg($id,multipleOf)] && [expr {$value % $nvm_cfg($id,multipleOf)}]} {
                    puts "All elements given in the value of id $id, must be multiple of $nvm_cfg($id,multipleOf)."
                    puts "Recheck element $i."
                    return false
                }

            }
            return [lreverse $ans]
        }
    } else {
        puts "The data you enter for id $id, is not in the correct string format."
        if {[lsearch -exact $noDefault $id]!=-1} {
            if {[string equal $nvm_cfg($id,size) mac]} {
                puts "Example: 00:10:18:AB:CD:EF."
            } else {
                puts "Expected string in this format: $regEx."
            }
        } else {
            puts "Example: $defaults($id)."
        }
        puts "The data was: $data."
        return false
    }
}
# split the string format to a list
# used by scan for each data element
proc nvmcfgT::stringFormatToList {id} {
    variable nvm_cfg
    set withStr false
    regsub -all {\?:} $nvm_cfg($id,stringFormat) {~@~} stringFormat
    set formats [split  $stringFormat ":"]
    for {set i 0} {$i<[llength $formats]} {incr i} {
        if {$withStr} {
            # String array
			set nvm_cfg([set id],is_string_array) true
        }
        if {[regexp {[(]\[0-9a-fA-F\]} [lindex $formats $i]]} {
            if {[regexp {[(]\[0-9a-fA-F\][\{](?:\d+,)?(\d+)[\}]} [lindex $formats $i] align align]} {
                lappend ans "%0[set align]x"
            } else {
                lappend ans "%x"
            }
        } elseif {[regexp {[(]\\d} [lindex $formats $i]]} {
            if {[regexp {[(]\\d[\{](?:\d+,)?(\d+)[\}][)]} [lindex $formats $i] align align]} {
                lappend ans "%0[set align]d"
            } else {
                lappend ans "%d"
            }
        } elseif {[regexp {[(]\.} [lindex $formats $i]]} {
            if {$i} {
                # String array
		set nvm_cfg([set id],is_string_array) true
            }
            set withStr true
            lappend ans "string"
        } else {
            error "Unknown stringFormat $nvm_cfg($id,stringFormat) element $i $formats"
        }
    }
    return $ans
}

#returns 0 is non-MAC
proc nvmcfgT::getMacSize {id} {
    variable nvm_cfg
    if {[string equal $nvm_cfg([set id],size) mac] == 0} {
        return 0
    }
    set sform $nvm_cfg($id,stringFormat)
    set sform [split $sform :]
    return [expr {[llength $sform] / 2}]
}

# Recieve a list with the data of the elements  and id
# set the elements for that id
proc nvmcfgT::set_elements {id elems} {
    variable nvm_cfg
    variable nvm_config
    variable shmem_config
    variable announcements
    set setElems 0
    set offset $nvm_cfg($id,offset)
    set size [expr abs($nvm_cfg($id,elementSize))]
    updateNumOfElements $id
    set elements $nvm_cfg($id,numOfElements)
    set isMac [string equal $nvm_cfg($id,size) mac]
    if {$isMac} {
        if {[getMacSize $id] != 8} {
      
            if {[getMacSize $id] != 6} {
                puts "ERROR: field $id contains [getMacSize $id] elements. Defaulting to 6."
            }
            set temp [lrange $elems 0 3]
            set elems [linsert $temp 0 [lindex $elems 4] [lindex $elems 5] 0 0]
            set elements 8
        } else {            
            set temp [lrange $elems 0 3]
            set elems [linsert $temp 0 [lindex $elems 4] [lindex $elems 5] [lindex $elems 6] [lindex $elems 7]]
            set elements 8
        }
        
    } elseif {![string equal [stringFormatToList $id] {string}] && $size<32 && [expr $size*$elements>32]} {
        # field which are larger then 32 bits and are build from elements which are smaller then 32 bits
        # are array which must be written in Big endianity
        for {set i 0} {$i<[llength $elems]-1} {incr i [expr 32/$size]} {
            eval [subst {lappend temp [lreverse [lrange $elems $i [expr $i+32/$size-1]]]}]
        }
        set elems $temp
    }

    while {$elements!=$setElems && $setElems !=[llength $elems]} {
        nvm_set_bits $nvm_config $shmem_config $offset $size [lindex $elems $setElems]
        incr offset $size

        incr setElems
    }
    if {[info exist nvm_cfg($id,announce)]} {
        append announcements $nvm_cfg($id,announce)
        append announcements "\n"
    }
}
proc nvmcfgT::is_string_array {id} {

	variable nvm_cfg
	if {[catch {set l $nvm_cfg([set id],stringFormat)}]} {

		return false
	}
	set l [split $l :]
	if {[llength $l] < 2} {

		return false
	}
	set first_item [lindex $l 0]
	if {[regexp {\.\{} $first_item]} {
		return true
	} else {

		return false
	}
}

proc nvmcfgT::read_byte_list {offset size} {
	variable nvm_config

	set result {}
	for {set read_size 0} {$read_size < $size } { incr read_size 8} {
		set current_offset [expr {$offset + $read_size}]
		lappend result [nvm_get_bits $nvm_config $current_offset 8]
	}

	return $result
}

# Return a list with the data of the elements
proc nvmcfgT::get_elements {id} {
    variable nvm_cfg
    variable nvm_config
    
    set readElems 0
    set offset $nvm_cfg($id,offset)
    set size [expr abs($nvm_cfg($id,elementSize))]
    updateNumOfElements $id
    set elements $nvm_cfg($id,numOfElements)
    set isMac [string equal $nvm_cfg($id,size) mac]
    if {$isMac} {
        set elements 8
    }
	set string_array false
	if {[is_string_array $id]} {
		set string_array true
	}
    while {$elements!=$readElems} {
		if {$string_array} {
			set val [read_byte_list $offset $size]
		} else {
			set val [nvm_get_bits $nvm_config $offset $size]
		}
        lappend ans $val
        incr offset $size

        incr readElems
    }

    if {$isMac} {
        set temp $ans
        set ans [lrange $ans 4 end]
        set ans [linsert $ans end [lindex $temp 0] [lindex $temp 1]]
        if {[getMacSize $id] == 8} {
            set ans [linsert $ans end [lindex $temp 2] [lindex $temp 3]]
        }
    } elseif {![string equal [stringFormatToList $id] {string}] && $size<32 && [expr $size*$elements>32]} {
        # field which are larger then 32 bits and are build from elements which are smaller then 32 bits
        # are array which must be read in Big endianity
        for {set i 0} {$i<[llength $ans]-1} {incr i [expr 32/$size]} {
            eval [subst {lappend temp [lreverse [lrange $ans $i [expr $i+32/$size-1]]]}]
        }
        set ans $temp
    }
    return $ans
}

# align the text to lines of size
# the firstLine can have different length
# the prefix is added to all lines below the first
proc nvmcfgT::align_text {text prefix size {firstLine -1}} {
    set col 0
    set aligned ""
    if {$firstLine!=-1} {
        set lineSize $firstLine
    } else {
        set lineSize $size
    }

    for {set i 0} {$i<[llength $text]} {incr i} {
        if {$col+[string length [lindex $text $i]]<$lineSize} {
            append aligned [lindex $text $i]
            incr col [string length [lindex $text $i]]
        } else {
            if {[string length [lindex $text $i]]>$size} {
                error "You enter strings which are larger than the line size: [lindex $text $i]."
            }
            set lineSize $size
            append aligned [format "\n%s%s" $prefix [lindex $text $i]]
            set col [string length [lindex $text $i]]
        }
        if {$col<$size} {
            append aligned " "
            incr col 1
        }
    }
    for {set i 0} {$i<[expr $size-$col]} {incr i} {
        append aligned " "
    }
    return $aligned
}

# reverse a list
proc nvmcfgT::lreverse {originList} {
    for {set i [expr [llength $originList]-1]} {$i>=0} {incr i -1} {
        lappend ans [lindex $originList $i]
    }
    return $ans
}
# set the defaults for a specic board
# first look for the default in that board's array (id as key)
# otherwise checks if the value exists per port (id,port as key)
# otherwise checks if the value exists in the default's array
# otherwise checks if the value exists per port in the default's array (id,port as key)
proc nvmcfgT::set_default_board {board} {
    variable IDs
    variable nvm_cfg
    variable portMax
    variable funcs
    variable noDefault
    variable nvm_config
    variable shmem_config
    variable mac_partition
    variable cfg_ext_shared_block
    
    upvar #0 nvmcfgT::$board boardD
    upvar #0 nvmcfgT::default defaultD
    set curr_device $::current(DEV)

    # setting the port to be port0
    gotoFuncZero
    set port0 $::current(DEV)
    device $curr_device -no_display
    set portMax [GetDefaultMaxNumOfPorts $board]
    for {set port_num 0} {$port_num<$funcs} {incr port_num} {
        device [expr $port0+$port_num] -no_display

        if {[isE2]} {
            ## even numbered devices should be in port 0
            ## odd numbered devices should be in port 1
            if {$port_num > 0 && [GetCurrentPathNum]!=[expr {$port_num & 1}]} {
                continue
            }

        } else {
            # in E1H the lsb is the port num
            if {$port_num > 0 && [expr $::current(FUNC)&1]==0} {
                continue
            }
        }
        #nvm_load retuns a list {nvm_config shmem_config}
        set config_list [nvm_load]
        set nvm_config [lindex $config_list 0]
        set shmem_config [lindex $config_list 1]
        set mac_partition [lindex $config_list 2]
        set cfg_ext_shared_block [lindex $config_list 3]

        for {set i 0} {$i<[llength $IDs]} {incr i} {
            set id [lindex $IDs $i]
            if {(![isExtImageField $id]) && ($port_num>=$portMax)} {
                continue
            }
            if {$nvm_cfg($id,shared) && $port_num>0 || [lsearch -exact $noDefault $id]>-1 || \
                ([isFieldExcluded $id])} {
                continue
            }
            set no_default false
            set defVal [verify_default_value $id $board $port_num no_default]

            if {$no_default} {
                error "couldn't find a value to verify for id $id , port $port_num"
            }
            set_feature_val $id $defVal
        }
        nvm_store $nvm_config $shmem_config $mac_partition $cfg_ext_shared_block
    }
    device $curr_device -no_display
}

# return 0 if all devices for that card are not loaded
# else return first device with driver loaded in that card
proc nvmcfgT::is_driver_loaded {} {
    if {$::current(VIRTUAL_NVM_MODE) == "1"} {
	variable funcs
	set funcs $::current(TOTAL_DEV)
	return 0
    }

    set curr_device $::current(DEV)
    gotoFuncZero
    set dev $::current(DEV)
    set ans none
    variable funcs
    set funcs 0
    while {1} {
        incr funcs
        if {[string equal $::current(DRV_STATE) "RUNNING"]} {
            device $curr_device -no_display
            return $dev
        }
        if {$dev==$::current(TOTAL_DEV)} {
            device $curr_device -no_display
            return 0
        }
        incr dev
        if {[catch {device $dev -no_display}]} {
            continue
        }
        if {[isFuncZero]} {
            device $curr_device -no_display
            return 0
        }
    }

}

proc nvmcfgT::show_boards {} {
        variable boards
        
        puts "Supported boards are:\n $boards"
}


proc nvmcfgT::swap_ports_cfg {args}  {
    variable IDs
    variable nvm_cfg
    variable is_swap_ports_cfg
    variable nvm_config
    
    # By default do not swap the MAC addresses
    set swap_macs 0
    set from_4ports_cfg {-1 -1 -1 -1}
    set to_4ports_cfg   {-1 -1 -1 -1}
            
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {                
                "-from"      { incr i; set from_4ports_cfg [lindex $args $i]}
                "-to"        { incr i; set to_4ports_cfg [lindex $args $i]}
                "-swap_macs" { set swap_macs 1}               
                "-help"      { swap_ports_cfg_usage; return -1 }
                default      { swap_ports_cfg_usage; return -1 }
         }
    }   
    
    # Use the swap_ports_cfg flag to silent the announcements
    # during the swap.
    set is_swap_ports_cfg 1
    
    # Functions per ports:
    set funcsPerPortList(0) {0 4}
    set funcsPerPortList(1) {1 5}
    set funcsPerPortList(2) {2 6}
    set funcsPerPortList(3) {3 7}
 
    set perFuncIDsList {}
    set perPortIDsList {}
    
    set gotPerPortVals 0
    set dev_id [getDeviceID]
    
    set config_list [nvm_load]
    set nvm_config [lindex $config_list 0]
    
    # Check what is the number of ports
    if {[is_4port_mode]} {	
        set maxPort 4
	# Check 4 ports configuration exists and ports are between 0 - 3
	foreach i $from_4ports_cfg j $to_4ports_cfg {		
		if {($i==-1) || ($j==-1)} {
			puts "Error: The port configuration is missing !"
			swap_ports_cfg_usage
			return
		}
		if {($i<0) || ($i>3) || ($j<0) || ($j>3)} {
			puts "Error: Port numbers should be between 0-3"			
			return
		}		
	}
		
	# Check there are no duplicate ports in 4 ports configuration
	foreach i $from_4ports_cfg {
		set idx [lindex $from_4ports_cfg $i]
		for {set j [expr {$idx+1}]} {$j < [llength $from_4ports_cfg]} {incr j} { 			
			if { $i == [lindex $from_4ports_cfg $j]} {
				puts "Error: The four port configuration includes port $i in more than one instance !"			
				return
			}
		}
	}
	
	# Check there are no duplicate ports in 4 ports configuration
	foreach i $to_4ports_cfg {
		set idx [lindex $to_4ports_cfg $i]
		for {set j [expr {$idx+1}]} {$j < [llength $to_4ports_cfg]} {incr j} { 			
			if { $i == [lindex $to_4ports_cfg $j]} {
				puts "Error: The four port configuration includes port $i in more than one instance !"			
				return
			}
		}
	}
	
    } else {
        set maxPort 2
	# Check 4 ports configuration doesn't exist
	foreach i $from_4ports_cfg j $to_4ports_cfg {		
		if {($i!=-1) || ($j!=-1)} {
			puts "Error: The port configuration is not required !"
			swap_ports_cfg_usage
			return
		}
	}
    }
    
    # Go through all "nvm cfg" IDs:
    # Save the per func IDs in perFuncIDsList
    # Save the per port IDs in perPortIDsList
    for {set i 0} {$i<[llength $IDs]} {incr i} {
        set id [lindex $IDs $i]
            
        # Continue over fields which are ignored or
        # excluded per this device
        if {[isFieldIgnored $id]} {
                continue
	}           
              
        if {[isFieldExcluded $id $dev_id]} {
                continue
        }            
 
        set mac_size [getMacSize $id]         
        if {[isPerFunc $id]} {                
                if {!$mac_size} {                        
                        lappend perFuncIDsList $id
                } elseif {$swap_macs} {
                        # Include MAC fields only if swap_macs is on
                        lappend perFuncIDsList $id
                }
        } elseif {[isPerPort $id]} {
                if {!$mac_size} {         
                        lappend perPortIDsList $id
                } elseif {$swap_macs} {
                        # Include MAC fields only if swap_macs is on
                        lappend perPortIDsList $id
                }
        }
    }    
    puts "list of per port IDs: $perPortIDsList"
    puts "list of per func IDs: $perFuncIDsList"
    
    # Work in special NVM mode, since we need to support
    # hidden functions as well.
    set ::current(IS_SPECIAL_NVM_MODE) 1
    set ::current(SPECIAL_NVM_MODE_FUNC_NUM) 0   
    
    
    puts "\nReading current per port configuration .."    
    # Go through all ports, read the current perPort values
    for {set port 0} {$port < $maxPort} {incr port} {
	       
        set ::current(SPECIAL_NVM_MODE_FUNC_NUM) $port        
        
	set cmd "nvm cfg " 
        # Go through all "nvm cfg" per port IDs
        for {set i 0} {$i < [llength $perPortIDsList] } {incr i} {
            set id [lindex $perPortIDsList $i]
            append cmd "$id= "             
        }	
	
	# Save the nvm cfg per port ID result in perPortValsList($port) list	
        set perPortValsList($port) [eval $cmd]	                 
    }
   
    #puts "list of per port values (port 0): $perPortValsList(0)"
    #puts "list of per port values (port 1): $perPortValsList(1)"    
   
    puts "Reading current per function configuration .."    
    # Go through all functions (including hidden functions)
    # Read the per function values    
    for {set func 0} {$func < 8} {incr func} {
	   
        set ::current(SPECIAL_NVM_MODE_FUNC_NUM) $func        
        
	set cmd "nvm cfg " 
        # Go through all "nvm cfg" per func IDs
        for {set i 0} {$i < [llength $perFuncIDsList] } {incr i} {
            set id [lindex $perFuncIDsList $i]
            append cmd "$id= "            
        }
		
	# Save the nvm cfg per func ID result in perFuncValsList($func) list           
        set perFuncValsList($func) [eval $cmd]        
    }
        
    #puts "list of per func values (func 0): $perFuncValsList(0)"
    #puts "list of per func values (func 1): $perFuncValsList(1)"
       
    puts "\nSwap per port configuration:"
    for {set port 0} {$port < $maxPort} {incr port} {
        
        set ::current(SPECIAL_NVM_MODE_FUNC_NUM) $port        
        
	# In 2 port mode, swap between port 0 and port 1
	if {$maxPort == 2} {
		if {$port == 0} {
		        set swap_port [expr {$port+1}]
		} else {
		        set swap_port [expr {$port-1}]
		}
	} else {
		set idx [lsearch $from_4ports_cfg $port]
		set swap_port [lindex $to_4ports_cfg $idx]		
	}
        
        puts "Swap port $port configuration with port $swap_port configuration .."        
        
        # Go through all "nvm cfg" per port IDs
        # Configure the per port id with the swap_port value
	set cmd "nvm cfg "
        for {set i 0} {$i < [llength $perPortIDsList] } {incr i} {            
            set id [lindex $perPortIDsList $i]
            set val [lindex $perPortValsList($swap_port) $i]                     
            append cmd "$id=$val "
        }
	#Perform the "nvm cfg" command
	eval $cmd
    }
            
    puts "\nSwap per function configuration:"
    for {set func 0} {$func < 8} {incr func} {
        
        set ::current(SPECIAL_NVM_MODE_FUNC_NUM) $func        
        
	# In 2 port mode, swap between odd functions and even functions
	if {$maxPort == 2} {
		if {[expr {$func%2}] == 0} {
		        set swap_func [expr {$func+1}]
		} else {
		        set swap_func [expr {$func-1}]
		}
	} else {
		set port [expr {$func%4}]
		# Find the port index in the "from_4port_cfg" list
		set port_idx [lsearch $from_4ports_cfg $port]
		# Find the matching port in the "to_4ports_cfg"
		set swap_port [lindex $to_4ports_cfg $port_idx]
		
		# Find the function index in the relevant funcPerPortList
		set func_idx [lsearch $funcsPerPortList($port) $func]
		# Find the matching func in the swap port funcsPerPortList
		set swap_func [lindex $funcsPerPortList($swap_port) $func_idx]
	}
        
        puts "Swap function $func configuration with function $swap_func configuration .."        
        
        # Go through all "nvm cfg" per port IDs
        # Configure the per port id with the swap_port value
	set cmd "nvm cfg "
        for {set i 0} {$i < [llength $perFuncIDsList] } {incr i} {            
            set id [lindex $perFuncIDsList $i]
            set val [lindex $perFuncValsList($swap_func) $i]                     
            lappend cmd "$id=$val"
        }
	#Perform the "nvm cfg" command
	eval $cmd
    }
    
    puts "\nPort swap completed successfully. Please power-cycle the machine !!"
    
    # Return back to regular NVM mode
    set ::current(IS_SPECIAL_NVM_MODE) 0
    
    # Turn off the swap_ports_cfg flag
    set is_swap_ports_cfg 0
}

# -----------------------------------------------
# This proc prints out the swap_ports_cfg usage 
# -----------------------------------------------
proc swap_ports_cfg_usage {}  {
    
    puts stdout "Usage (for 2 ports):   swap_ports_cfg "
    puts stdout "Optional:              -swap_macs (by default don't swap macs)"
    puts stdout "Usage (for 4 ports):   swap_ports_cfg "
    puts stdout "                       -from <from 4port cfg e.g. {0 1 2 3}>"
    puts stdout "                       -to   <to 4 port cfg e.g. {1 0 3 2}"    
    puts stdout "Optional:              -swap_macs (by default don't swap macs)"
    
}
