###########################################################################
#
# Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved
# Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
# Proprietary and Confidential Information.
#
# Module Description:
# This TCL script mii access via mdio clause45 for debug purposes
#
# Written by Yaniv Rosner, QLogic, Jul 2008                                        
#
###########################################################################

# Specify any dependency on other packages
package require Tcl

# Specify the package this module provides
package provide _access_phy 1.0

namespace eval _access_phy {
    
    variable ext_phy_addr   0 
    variable ext_phy_type   0
    variable ext_phy_name   0
    variable ext_emac_base  0
    variable int_emac_base  0
    variable int_phy_addr   0
    variable emac_base      0
    variable phy_addr       0
    variable single_port_device 0
    variable mdc_mdio_access 0
    variable restore_drv_intr 0
    # Export the package
    namespace export access_phy

    # Alias names
    interp alias {} wc_phy {}    "_access_phy::wc_phy"
    interp alias {} phy {}    "_access_phy::phy"
    interp alias {} extphy {} "_access_phy::extphy"
    interp alias {} extphy2 {} "_access_phy::extphy2"
    interp alias {} linkdump {} "_access_phy::linkdump"   
    interp alias {} read_sfp_eeprom {} "_access_phy::read_sfp_eeprom"   
    interp alias {} phy_dump {} "_access_phy::phy_dump"   
    interp alias {} extphy_dump {} "_access_phy::extphy_dump"   
    interp alias {} gphy_dump {} "_access_phy::gphy_dump"   
    interp alias {} dump_regs {} "_access_phy::dump_regs"   
    interp alias {} wc_phy_dump {} "_access_phy::wc_phy_dump"
    interp alias {} autogreeen_stats {} "_access_phy::autogreeen_stats"
    interp alias {} wc_dump {} "_access_phy::wc_dump"
}


proc _access_phy::phy_help {} {
    
    set help_info "Read/Write WARPCORE register\n"
    append help_info "Usage: wc_phy <port> <devad> <reg> \[value\]\n" 
    add_help wc_phy $help_info

    set help_info "Read/Write PHY register\n"
    append help_info "Usage: phy <port> <reg> \[value\]\n" 
    add_help phy $help_info

    set help_info "Read/Write external PHY register\n"
    append help_info "Usage: extphy <port> <dev> <reg> \[value\]\n" 
    append help_info "       On GPHY, <port> is 1; <dev> can be 0 (1KB-T), 1(1KB-X), or 7(EEE).\n"
    add_help extphy $help_info        

    set help_info "Read/Write external PHY2 register\n"
    append help_info "Usage: extphy2 <port> <dev> <reg> \[value\]\n" 
    add_help extphy $help_info       
    
    set help_info "Dumps XGXS phy bank registers\n"
    append help_info "Usage: phy_dump <port> <bank>\n" 
    add_help phy_dump $help_info 

    set help_info "Dumps external phy registers\n"
    append help_info "Usage: extphy_dump \[phy\]\n" 
    add_help extphy_dump $help_info 

    set help_info "Read from SFP+ module's EEPROM\n"
    append help_info "Usage: read_sfp_eeprom <port> <addr> <byte_cnt> \[phy_index\]\n" 
    add_help read_sfp_eeprom $help_info

    set help_info "Dumps relevant link information for debug\n"
    append help_info "Usage: linkdump \[phy_index\]\n" 
    add_help linkdump $help_info

    set help_info "Displays statistics for Auto-grEEEn\n"
    append help_info "Usage: autogreeen_stats\n" 
    add_help autogreeen_stats $help_info

    set help_info "Dumps Warpcore phy bank registers\n"
    append help_info "Usage: wc_phy_dump <lane>\n" 
    add_help wc_phy_dump $help_info 

    set help_info "Dumps relevant Warpcore registers\n"
    append help_info "Usage: wc_dump <lane>\n" 
    add_help wc_dump $help_info 

}

proc _access_phy::populate_shmem_vars {port phy_index} {
    variable ext_phy_addr
    variable int_phy_addr
    variable ext_phy_type
    variable single_port_device
    variable mdc_mdio_access
    set dev_info_offset 8
    set port_hw_config 44
    set port_hw_config_size 400    
    set shared_feature_config_size 4
    set port_feature_config_size 116    
    set shmem_base [reg read 0xa2b4]   
    set port_feature_config_offset [expr {$port_hw_config + $port_hw_config_size*2 + $shared_feature_config_size + $port_feature_config_size*$port}]
    set link_config_offset [expr {$port_feature_config_offset + 40}]    
    set config2 [reg read [expr {$shmem_base + $dev_info_offset + 4 + 20}]]
    #puts "config2 = $config2"
   # puts "[expr [expr $config2 & 0x38000000] >> 3]"
    if {$phy_index < 2} {
        set external_phy_config_offset [expr {$dev_info_offset +  $port_hw_config + $port_hw_config_size*$port + 384}]    
        set mdc_mdio_access [expr {$config2 & 0x1C000000}]
    } else {
        set external_phy_config_offset [expr {$dev_info_offset +  $port_hw_config + $port_hw_config_size*$port + 360}]    
        set mdc_mdio_access [expr [expr $config2 & 0xE0000000 ] >> 3];
    }
  
    set external_phy_config [reg read [expr {$shmem_base + $external_phy_config_offset} ]]    
    set ext_phy_type [expr {$external_phy_config & 0xff00}]
    set ext_phy_addr [expr {$external_phy_config & 0xff} ]
    # puts "phy type = $ext_phy_type, phy addr = $ext_phy_addr offset: [expr {$shmem_base + $external_phy_config_offset} ]\n"
    set device_id [expr [expr [reg read [expr $shmem_base + $port_hw_config + 12]] & 0xffff0000] >> 16]
    if {($device_id == 0x1022) || ($device_id == 0x1020) || ($device_id == 0x1015) || ($device_id == 0x1100)} {
        set single_port_device 1
    } else {
        set single_port_device 0
    }
    
    set PORT_FEATURE_CONNECTED_SWITCH_MASK          0x03000000
    set PORT_FEATURE_CON_SWITCH_1G_SWITCH           0x00000000
    set PORT_FEATURE_CON_SWITCH_10G_SWITCH          0x01000000
    if {[is_e3] == 0} {
        set switch_cfg [expr [reg read [expr {$shmem_base + $link_config_offset} ]] & $PORT_FEATURE_CONNECTED_SWITCH_MASK]
        if {$switch_cfg} {
            # XGXS - 10G SWITCH
            set int_phy_addr [reg read [expr 0x10340 + {$port*0x18}]]        
        } else {    
            # SERDES - 1G SWITCH 
            set int_phy_addr [reg read [expr 0x10374 + {$port*0x10}]]
        }
    }
  # puts "xx $external_phy_config_offset"
}

proc _access_phy::mii_init {port {phy_index 1}} {
    variable ext_emac_base
    variable ext_phy_addr
    variable ext_phy_name
    variable ext_phy_type
    variable int_emac_base
    variable int_phy_addr
    variable mdc_mdio_access

    set result [_access_phy::populate_shmem_vars $port $phy_index]
    #calculate emac_base
	set EXT_PHY_TYPE_DIRECT  0x0000
    set EXT_PHY_TYPE_BCM8072 0x0200
    set EXT_PHY_TYPE_BCM8073 0x0300
    set EXT_PHY_TYPE_BCM8705 0x0400
    set EXT_PHY_TYPE_BCM8706 0x0500
    set EXT_PHY_TYPE_BCM8726 0x0600
    set EXT_PHY_TYPE_BCM8481 0x0700
    set	EXT_PHY_TYPE_SFX7101 0x0800
    set	EXT_PHY_TYPE_BCM8727 0x0900
    set EXT_PHY_TYPE_BCM8727_NOC 0x0A00
    set EXT_PHY_TYPE_BCM84823 0x0B00
    set EXT_PHY_TYPE_BCM8727_SMD 0x0C00
    set EXT_PHY_TYPE_BCM84833 0x0D00
    set EXT_PHY_TYPE_BCM54618SE 0x0E00
    set EXT_PHY_TYPE_BCM54616 0x1000
	set EXT_PHY_TYPE_BCM84834 0x1100

    #default ext emac
    if {$port == 0 } {
        set ext_emac_base 0x8000
    } else {
        set ext_emac_base 0x8400
    }		 
 
    set ext_phy_name n/a
    if {$ext_phy_type == $EXT_PHY_TYPE_BCM8072} {   
         set ext_phy_name 8072
         set ext_emac_base 0x8000
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM8073} {
        set ext_phy_name 8073
        if {$port == 0 } {
            set ext_emac_base 0x8400
        } else {
            set ext_emac_base 0x8000
        }
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM8705 } {
        set ext_phy_name 8705
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM8706 } {
         set ext_phy_name 8706
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM8726 } {
        set port_swap [reg read 0x10394]
        if {$port_swap } {
            set ext_emac_base 0x8000
        } else {
            set ext_emac_base 0x8400
        }
         set ext_phy_name 8726
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM8727 || $ext_phy_type == $EXT_PHY_TYPE_BCM8727_NOC} {
        set port_swap [reg read 0x10394]
        if {$port_swap} {
            set ext_emac_base 0x8000
        } else {
            set ext_emac_base 0x8400
        }
         set ext_phy_name 8727
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_SFX7101 } {
        set ext_phy_name 7101
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM8481} {
        set ext_phy_name 8481
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM84823} {
        set ext_phy_name 84823
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM84833} {
        set ext_phy_name 84833
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM54618SE} {
        set ext_phy_name 54618
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_DIRECT} {
		#wc is the only PHY on E3. No exteranl PHY
        set ext_phy_name wc
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM54616} {
        set ext_phy_name 54616
    } elseif {$ext_phy_type == $EXT_PHY_TYPE_BCM84834} {
		set ext_phy_name 84834
	}

  #  puts $mdc_mdio_access
    if {$mdc_mdio_access > 0} {
        if {$mdc_mdio_access == 0x04000000} {
            set ext_emac_base 0x8000
        }
        if {$mdc_mdio_access == 0x08000000} {
            set ext_emac_base 0x8400
        } 
        if {$mdc_mdio_access == 0x0C000000} {
            set ext_emac_base [expr 0x8000 + 0x400*$port]
        }
        if {$mdc_mdio_access == 0x10000000} {
            set ext_emac_base [expr 0x8400 - 0x400*$port]
        }
    }
  #  puts $ext_emac_base
    if {$port == 0 } {
        set int_emac_base 0x8000
    } else {
        set int_emac_base 0x8400
    }
}

# MDIO lock
#   SIDE EFFECT: Once lock is acquired, interrupt to the driver will
#   be disabled. Need free_mdio_lock proc to re-enable it.
proc _access_phy::acq_mdio_lock {{to_msec 1000}} {
    variable restore_drv_intr
    # DRIVER_CONTROL_13 is reserved for this.
    # Bit 0 is the MDIO client.
    if {[reg read 0xa3fc] & 1} {
        error "MDIO lock already acquired."
    }
    # Disable interrupt
    set restore_drv_intr 0

    if {$::sys(IS_EDEBUG) != 1} {
       if {$::current(DRV_STATE) == "RUNNING"} {
           driver intr 0
            set restore_drv_intr 1
        }
    }
    for {set idx 0} {$idx < $to_msec} {incr idx} {
        reg write 0xa3fc 1
        if {[reg read 0xa3fc] & 1} {
            return
        }
        after 1
    }
    # Can't get the lock. Re-enable the interrupt.
    if {$restore_drv_intr} {
        driver intr 1
    }
    error "MDIO lock timeout."
}

proc _access_phy::free_mdio_lock {} {
    variable restore_drv_intr
    # DRIVER_CONTROL_13 is reserved for this.
    # Bit 0 is the MDIO client.
    if {[reg read 0xa3f8] & 1} {
        reg write 0xa3f8 1
    }
    # Re-enable the interrupt only in case driver was enabled before.
    if {$restore_drv_intr} {
        if {$::current(DRV_STATE) == "RUNNING"} {
            driver intr 1
        }
    }
}


# Clause 45 read
proc _access_phy::do_miiread {dev reg} {

    variable emac_base
    variable phy_addr
    
    set start_bit [expr 1 << 29]
    set mid_bit [expr 3 << 26]
    
    acq_mdio_lock

    set err [catch {
        set mdio_mode [reg read [ expr $emac_base + 0x00b4]]
        # CL45, CLK 2.5MHz
        reg write [expr {$emac_base + 0x00b4}] 0x80310000
 
        set tmp [expr {$start_bit | ($phy_addr << 21) | ($dev << 16) | $reg}]
        reg write [expr {$emac_base + 0x00ac}] $tmp
        set result [reg read [expr {$emac_base + 0x00ac}]]
        while {$result & $start_bit} {
            set result [reg read [expr {$emac_base + 0x00ac}]]
        }
        set tmp [expr {$start_bit | $mid_bit | ($phy_addr << 21) | ($dev << 16) | $reg}]
        reg write [expr {$emac_base + 0x00ac}] $tmp
        set result [reg read [expr {$emac_base + 0x00ac}]]
        while {$result & $start_bit} {
            set result [reg read [expr {$emac_base + 0x00ac}]]
        }

        reg write [expr {$emac_base +0x00b4}] $mdio_mode
        #puts "miiread = Phy_addr:$phy_addr  Device_addr:$dev Reg_addr:$reg  Value:$result"
    }]

    free_mdio_lock
    if {$err} {
        # Throw the error after freeing the MDIO lock.
        error $::errorInfo
    }

    return [format 0x%04x [expr {$result & 0xffff}]]
}

proc _access_phy::wc_read { dev reg} {

    set emac_base 0x8000
    
    set start_bit [expr 1 << 29]
    set mid_bit [expr 3 << 26]

    # Supply MDIO clock (used in E3 B0).
    reg write [expr {$emac_base + 0x00b0}] 0x2

    set mdio_mode [reg read [ expr $emac_base + 0x00b4]]
    # CL45, CLK 2.5MHz
    
    reg write [expr {$emac_base + 0x00b4}] 0x804a0000
 
    set tmp [expr {$start_bit | (1 << 21) | ($dev << 16) | $reg}]
    
    
    reg write [expr {$emac_base + 0x00ac}] $tmp
    set result [reg read [expr {$emac_base + 0x00ac}]]
    while {$result & $start_bit} {
        set result [reg read [expr {$emac_base + 0x00ac}]]
    }
    
    set tmp [expr {$start_bit | $mid_bit | (1 << 21) | ($dev << 16) | $reg}]
    
    reg write [expr {$emac_base + 0x00ac}] $tmp
    set result [reg read [expr {$emac_base + 0x00ac}]]
    while {$result & $start_bit} {
        set result [reg read [expr {$emac_base + 0x00ac}]]
    }

    reg write [expr {$emac_base +0x00b4}] $mdio_mode
    #puts "miiread = Phy_addr:1  Device_addr:$dev Reg_addr:$reg  Value:$result"

    # Remove MDIO clock (used in E3 B0).
    reg write [expr {$emac_base + 0x00b0}] 0x0

    return [format 0x%04x [expr {$result & 0xffff}]]
}

proc _access_phy::wc_write { dev reg cur_val } {

    set emac_base 0x8000
    
    set start_bit [expr 1 << 29]
    set mid_bit [expr 1 << 26]
    
    # Supply MDIO clock (used in E3 B0).
    reg write [expr {$emac_base + 0x00b0}] 0x2
   
    set mdio_mode [reg read [ expr {$emac_base + 0x00b4}]]
    # CL45, CLK 2.5MHz
    reg write [expr {$emac_base + 0x00b4}] 0x804a0000
 
    set tmp [expr {$start_bit | (1 << 21) | ($dev << 16) | $reg}]
    reg write [expr {$emac_base + 0x00ac}] $tmp
    set result [reg read [expr {$emac_base + 0x00ac}]]
    while {$result & $start_bit} {
        set result [reg read [expr {$emac_base + 0x00ac}]]
    }
    set tmp [expr {$start_bit | $mid_bit | (1 << 21) | ($dev << 16) | $cur_val}]
    reg write [expr {$emac_base + 0x00ac}] $tmp
    set result [reg read [expr {$emac_base + 0x00ac}]]
    while {$result & $start_bit} {
        set result [reg read [expr {$emac_base + 0x00ac}]]
    }
    #puts "miiwrite= Phy_addr:$phy_addr  Device_addr:$dev Reg_addr:$reg  WriteValue:$cur_val"
    set result 1
   
    reg write [expr {$emac_base +0x00b4}] $mdio_mode
    #puts "Write Verification - Read after write: $result"

    # Remove MDIO clock (used in E3 B0).
    reg write [expr {$emac_base + 0x00b0}] 0x0

    return $result
}

# Clause 45 write
proc _access_phy::do_miiwrite { dev reg cur_val } {

    variable phy_addr
    variable emac_base
    
    set start_bit [expr 1 << 29]
    set mid_bit [expr 1 << 26]
    
    acq_mdio_lock
   
    set err [catch {
        set mdio_mode [reg read [ expr {$emac_base + 0x00b4}]]
        # CL45, CLK 2.5MHz
        reg write [expr {$emac_base + 0x00b4}] 0x80310000
 
        set tmp [expr {$start_bit | ($phy_addr << 21) | ($dev << 16) | $reg}]
        reg write [expr {$emac_base + 0x00ac}] $tmp
        set result [reg read [expr {$emac_base + 0x00ac}]]
        while {$result & $start_bit} {
            set result [reg read [expr {$emac_base + 0x00ac}]]
        }
        set tmp [expr {$start_bit | $mid_bit | ($phy_addr << 21) | ($dev << 16) | $cur_val}]
        reg write [expr {$emac_base + 0x00ac}] $tmp
        set result [reg read [expr {$emac_base + 0x00ac}]]
        while {$result & $start_bit} {
            set result [reg read [expr {$emac_base + 0x00ac}]]
        }
        #puts "miiwrite= Phy_addr:$phy_addr  Device_addr:$dev Reg_addr:$reg  WriteValue:$cur_val"
        set result 1
   
        reg write [expr {$emac_base +0x00b4}] $mdio_mode
        #puts "Write Verification - Read after write: $result"
    }]

    free_mdio_lock

    if {$err} {
        # Throw the error after freeing the MDIO lock.
        error $::errorInfo
    }

    return $result
}


#############################################################################
# Clause 22 access (for E3 SGMII/GPHY)
# For GPHY, we have shadow registers and 1000BT and 1000BX registers.
# We need access to these registers to be atomic. Thus, a lock is needed
# in gphy_dump and extphy commands, not in the actual CL22.
#############################################################################
proc _access_phy::gphy_dump {} {
    variable emac_base
    variable phy_addr
    variable ext_emac_base
    variable ext_phy_addr


    set port [lookup_port]
    _access_phy::mii_init $port 1
    set emac_base $ext_emac_base
    set phy_addr $ext_phy_addr   

    acq_mdio_lock

    set err [catch {
        puts "1000Base-T"
        set x [cl22_1c 0x1f]
        cl22_1c 0x1f [expr $x & 0xfffe]
        cl22_show 

        puts "1000Base-X"
        set x [cl22_1c 0x1f]
        cl22_1c 0x1f [expr $x | 1]
        cl22_show 

        # Dump 0x18 shadows
        puts "Shadow 0x18"
        set stop_point [expr 0x8 - 1]
        for {set idx 0} {$idx < 0x8} {incr idx} {
            set val [cl22_18 $idx]
            puts -nonewline "[format %04x $val] "
            if {($idx & $stop_point) == $stop_point} {puts ""}
        }
        if {($idx & $stop_point) != 0} {puts ""}

        # Dump 0x1c shadows
        puts "Shadow 0x1C"
        set stop_point [expr 0x8 - 1]
        for {set idx 0} {$idx < 0x20} {incr idx} {
            set val [cl22_1c $idx]
            puts -nonewline "[format %04x $val] "
            if {($idx & $stop_point) == $stop_point} {puts ""}
        }
        if {($idx & $stop_point) != 0} {puts ""}
    }]

    free_mdio_lock
    if {$err} {
        # Throw the error after freeing the MDIO lock.
        error $::errorInfo
    }
}


proc _access_phy::cl22_1c {shadow {data "none"}} {
    cl22 0x1c [expr $shadow << 10]
    if {$data == "none"} {
        set val [cl22 0x1c]
        return $val
    } else {
        set data [expr $data & 0x3ff]
        cl22 0x1c [expr ($shadow << 10) | (1<<15) | $data] 
        return 0
    }
}

proc _access_phy::cl22_exp {expansion {data "none"}} {
    set expansion [expr $expansion & 0xff]
    cl22 0x17 [expr 0xf00 | $expansion]
    if {$data == "none"} {
        set val [cl22 0x15]
        return $val
    } else {
        cl22 0x15 $data
        return 0
    }
}
proc _access_phy::cl22_18 {shadow {data "none"}} {
    set shadow [expr $shadow & 0x7]
    if {$data == "none"} {
        cl22 0x18 [expr ($shadow << 12) | 0x7]
        set val [cl22 0x18]
        return $val
    } else {
        if {$shadow == 7} {
            set data [expr ($data & 0x1f8) | (1<<15)]
        } else {
            set data [expr $data & 0xfff8]
        }
        cl22 0x18 [expr $shadow | $data] 
        return 0
    }
}

proc _access_phy::cl22_show {} {
    set stop_point [expr 0x8 - 1]
    for {set idx 0} {$idx < 0x20} {incr idx} {
        set val [cl22 $idx]
        puts -nonewline "[format %04x $val] "
        #puts -nonewline "[format %04x $idx]: [format %04x $val]  "
        if {($idx & $stop_point) == $stop_point} {puts ""}
    }
    if {($idx & $stop_point) != 0} {puts ""}
}

proc _access_phy::cl22_dev7 {offset {data ""}} {
    cl22 0xd 7
    cl22 0xe $offset
    if {$data == ""} {
        cl22 0xd 0xc007
        return [cl22 0xe]
    } else {
        cl22 0xd 0x4007
        cl22 0xe $data
        return 0
    }
}


proc _access_phy::cl22 {offset {data ""}} {
    variable emac_base
    variable phy_addr

    set mode_offset [expr $emac_base + 0x00b4]
    set mdio_comm [expr $emac_base + 0x00ac]

    # Keep the mode
    set mode [reg read $mode_offset]
    # Switch to CL22 mode
    reg write $mode_offset [expr $mode & ~(1<<31)]

    if {$data == ""} {
        set cmd [expr 2<<26]
        set data 0
    } else {
        set cmd [expr 1<<26]
    }
    set comm [expr (1<<29) | $cmd | ($phy_addr << 21) | ($offset << 16) | $data]
    reg write $mdio_comm $comm
    for {set idx 0} {$idx < 10000} {incr idx} {
        set val [reg read $mdio_comm]
        if {($val & (1<<29)) == 0} {
            # Restore mode
            reg write $mode_offset $mode
            return [format 0x%x [expr $val & 0xffff]]
        }
    }
    # Restore mode
    reg write $mode_offset $mode

    error "cl22 timeout"
}
#############################################################################

proc _access_phy::extphy2 {port dev reg args} {
    variable ext_emac_base
    variable ext_phy_addr
    variable emac_base
    variable phy_addr
    variable ext_phy_name    
    variable single_port_device
    if {($single_port_device == 1) && ($port == 1)} {
        return 0
    }
    _access_phy::mii_init $port 2
    set emac_base $ext_emac_base
    set phy_addr $ext_phy_addr   
    if {$args == ""} {
        return [_access_phy::do_miiread $dev $reg]
    } else {
        return [_access_phy::do_miiwrite $dev $reg $args]
    }
}

proc _access_phy::extphy {port dev reg args} {
    variable ext_emac_base
    variable ext_phy_addr
    variable emac_base
    variable phy_addr
    variable ext_phy_name    
    variable single_port_device
    variable ext_phy_name
    if {($single_port_device == 1) && ($port == 1)} {
        return 0
    }
    _access_phy::mii_init $port 1
    set emac_base $ext_emac_base
    set phy_addr $ext_phy_addr   
    if {($ext_phy_name == 54616) || ($ext_phy_name == 54618)} {
        acq_mdio_lock
        set err [catch {
            if {$dev == 0} {
                set x [cl22_1c 0x1f]
                cl22_1c 0x1f [expr $x & 0xfffe]
                set result [_access_phy::cl22 $reg $args]
            } elseif {$dev == 1} {
                set x [cl22_1c 0x1f]
                cl22_1c 0x1f [expr $x | 1]
                set result [_access_phy::cl22 $reg $args]
            } elseif {$dev == 7} {
                set result [_access_phy::cl22_dev7 $reg $args]
            } else {
                error "<dev> needs to be 0, 1, or 7."
            }
        }]

        free_mdio_lock
        if {$err} {
            # Throw the error after freeing the MDIO lock.
            error $::errorInfo
        }
        return $result
    } else {
        if {$args == ""} {
            return [_access_phy::do_miiread $dev $reg]
        } else {
            return [_access_phy::do_miiwrite $dev $reg $args]
        }
    }
}

proc _access_phy::wc_phy {lane devad reg args } {

    set _access_phy::emac_base 0x8000
    set _access_phy::phy_addr 1  

    acq_mdio_lock

    set err [catch {
        _access_phy::wc_write $devad 0xffde $lane
        if {$args == ""} {        
            set result [_access_phy::wc_read  $devad $reg]
        } else {        
            set result [_access_phy::wc_write  $devad $reg $args]
        }
    }]

    free_mdio_lock
    if {$err} {
        # Throw the error after freeing the MDIO lock.
        error $::errorInfo
    }
    return $result
}

proc _access_phy::phy {port reg args} {
    variable int_emac_base
    variable int_phy_addr
    variable emac_base
    variable phy_addr    

    variable single_port_device
    if {($single_port_device == 1) && ($port == 1)} {
        return 0
    }
    set port4mode [reg read 0xa750]

    _access_phy::mii_init [expr $port & 1] 0
    
    set emac_base $int_emac_base
    set phy_addr $int_phy_addr
    set devad [reg read 0x1033c]
    if {$port4mode} {
        _access_phy::do_miiwrite $devad 0xffde [expr 0x0800 + $port]
       # puts "setting aer to [ hex16 [expr 0x0800 + $port ]]"
    }
    if {$args == ""} {        
        return [_access_phy::do_miiread $devad $reg]
    } else {        
        return [_access_phy::do_miiwrite $devad $reg $args]
    }
}

proc _access_phy::sfp_eeprom_8726 {port addr byte_cnt write} {
   
    if {$byte_cnt > 16} {
        puts "EEPROM access size should be up to 16 bytes"
        return -1
    }
  	
    # Set the read command byte count    
    set temp [_access_phy::extphy $port 1 0x8002 [expr 0xa000 | $byte_cnt]]

    # Set the read command address 
    set temp [_access_phy::extphy $port 1 0x8003 $addr]

     # Activate read command 
    set temp [_access_phy::extphy $port 1 0x8000 0x2c0f]  
   
    # Wait appropriate time for two-wire command to finish before polling the status register 
    after 1

    for {set i 0} {$i<100} {incr i} {
        set status [expr [_access_phy::extphy $port 1 0x8000] & 0xc]
	# Wait up to 500us for command complete status */
        if {$status == 0x4 } {
            break
        }
        delay us 5		
    }

    if {$status != 0x4 } {
	puts "Got bad status $status when reading from SFP+ EEPROM"
	return -1
    }
    
    # Wait for idle	
     for {set i 0} {$i<100} {incr i} {
        set status [expr [_access_phy::extphy $port 1 0x8000] & 0xc]
	# Wait up to 500us for command complete status */
        if {$status == 0x0 } {
            return 0
        }
        after 1	
    }
    puts "Failed to reach IDLE state"
    return 0
}



proc _access_phy::sfp_eeprom_8727 {port addr byte_cnt write} {
   
    if {$byte_cnt > 16 || $byte_cnt < 2} {
        puts "EEPROM access size should be between 2 to 16 bytes"
        return
    }
    # Need to read from 1.8000 to clear it */
    set temp [_access_phy::extphy $port 1 0x8000]
  	
    # Set the read command byte count    
    set temp [_access_phy::extphy $port 1 0x8002 $byte_cnt]

    # Set the read command address 
    set temp [_access_phy::extphy $port 1 0x8003 $addr]

     # Set the read command address 
    set temp [_access_phy::extphy $port 1 0x8004 0x8007]
    	
    # Activate read / write command */
    set command [expr 0x8002 | ($write << 5)]
    set temp [_access_phy::extphy $port 1 0x8000 $command]
	
    # Wait appropriate time for two-wire command to finish before polling the status register 
    after 1

    for {set i 0} {$i<100} {incr i} {
        set status [expr [_access_phy::extphy $port 1 0x8000] & 0xc]
	# Wait up to 500us for command complete status */
        if {$status == 0x4 } {
            break
        }
        delay us 5		
    }

    if {$status != 0x4 } {
	puts "Got bad status $status when reading from SFP+ EEPROM"
	return 
    }
    
    # Wait for idle	
     for {set i 0} {$i<100} {incr i} {
        set status [expr [_access_phy::extphy $port 1 0x8000] & 0xc]
	# Wait up to 500us for command complete status */
        if {$status == 0x0 } {
            return 
        }
        after 1	
    }
    puts "Failed to reach IDLE state"
}

proc _access_phy::bsc_mod_sel {device} {
	set r(GEN_PURP_HWG)  0xA9A0
	set r(EPIO_OEN)      0x800c8
    set r(EPIO_OUT)      0x800c4

	set cur_dev "$::current(DEV)"
	set_cur_device $device
        set shmem_base [reg read 0xa2b4]
	set_cur_device 0
	set epios_val [reg read [expr $shmem_base + 0x2c]]
	
	set epio_oen_val [reg read $r(EPIO_OEN)]
	set epio_out_val [reg read $r(EPIO_OUT)]

	set_cur_device $device
        if {($::current(CHIP_NUM)==57800) || ($::current(CHIP_NUM)==57840)} {
           set port [expr ($::current(PCI_FUNC) & 2) >> 1]
        } else {
           set port 0
        }
        set shmem_epio_val [reg read [expr $shmem_base + 0x74 + $port*400]]
	set bit(0) [expr (($shmem_epio_val & 0x3000000) >> 24) & 1]
	set bit(1) [expr (($shmem_epio_val & 0x3000000) >> 25) & 1]

        set epio(0) [expr ($epios_val & 0x3f)]
	set epio(1) [expr (($epios_val >> 6) & 0x3f)]
	for {set i 0} {$i<2} {incr i} {
		if {$epio($i)} {
			 set epio_adj [expr $epio($i) - 9]
			 set epio_oen_val [expr $epio_oen_val | (1<<$epio_adj)]
			 set epio_out_val [expr $epio_out_val & ~(1<<$epio_adj) | ($bit($i) << $epio_adj)]
		}
	}

	#puts "EPIO$epio0<$bit0>, EPIO$epio1<$bit1>"
	# change IO into EPIO
	set gen_purp_hwg [reg read $r(GEN_PURP_HWG)]
	reg write $r(GEN_PURP_HWG) [expr ($gen_purp_hwg & 0xFFFFFFFE) | (1 << 0)]
	

    # set up the EPIO output enable
	reg write $r(EPIO_OEN) $epio_oen_val
	# set up the EPIO output
	reg write $r(EPIO_OUT) $epio_out_val
	device $cur_dev -no_display
}

proc _access_phy::sfp_eeprom_warpcore {port sl_addr xfer_cnt} {
	set r(IMC_COMMAND)   0x85900
    set r(IMC_SLAVE)     0x85904
    set r(IMC_TMG0)      0x85914
    set r(IMC_TMG1)      0x85918
    set r(IMC_TMG2)      0x8591c
    set r(IMC_DATA0)     0x85920
    set r(IMC_DATA1)     0x85924
    set r(IMC_DATA2)     0x85928
    set r(IMC_DATA3)     0x8592c

	set sl_devid 0xa0
	set lc_addr  0x0
	set write_op 2
	set read_op  1
	set EEPROM_TIMEOUT 1000
	# set up the EPIO
	bsc_mod_sel $port

	variable eeprom_data


	puts "
Reading $xfer_cnt bytes from
  Module $port 
  Device $sl_devid 
  starting at address $sl_addr
"
	#Enable the engine
	reg write $r(IMC_COMMAND) [expr (1 << 31)]
	
	# Program Slave Device ID
	reg write $r(IMC_SLAVE) [expr ($sl_devid << 16) | ($sl_addr << 0)]
	
	# Send a 0-byte write to update the address pointer
	reg write $r(IMC_COMMAND) [expr (1 << 31) | ($write_op << 28) | ($lc_addr << 8) | (0 << 0)]
	
	# poll for operation to complete
	set rdval [reg read $r(IMC_COMMAND)]
	
	for {set wait_time 0} {$wait_time<$EEPROM_TIMEOUT} {incr wait_time} {
		if {[expr ($rdval & 0x00030000) >> 16] == 1} {
			break
		}
		delay ms 1
		set rdval [reg read $r(IMC_COMMAND)]
	}
	if {$wait_time == $EEPROM_TIMEOUT} {
		puts "EEPROM timed out !!"
		return -1
	}
	
	# Start Transfer
	reg write $r(IMC_COMMAND) [expr (1 << 31) | ($read_op << 28) | ($lc_addr << 8) |  ($xfer_cnt << 0)]
	
	# poll for operation to complete
	set rdval [reg read $r(IMC_COMMAND)]
	while { [expr ($rdval & 0x00030000) >> 16] != 1 } {
		# puts "Polling for second completion $rdval"
		set rdval [reg read $r(IMC_COMMAND)]
	}

	set eeprom_data(0) [reg read $r(IMC_DATA0)]
	
	set eeprom_data(1) [reg read $r(IMC_DATA1)]
	set eeprom_data(2) [reg read $r(IMC_DATA2)]
	set eeprom_data(3) [reg read $r(IMC_DATA3)]
	return 0
  
}
proc _access_phy::read_sfp_eeprom {port addr byte_cnt {phy_index 1}} {
     variable ext_phy_name    
    variable eeprom_data

    set result [mii_init $port $phy_index]
    if {$ext_phy_name == 8727} {
         set result [sfp_eeprom_8727 $port $addr $byte_cnt 0]     
         set base_output_addr 0x8007   
    } elseif {$ext_phy_name == 8726 || $ext_phy_name == 8706 } {
	     set result [sfp_eeprom_8726 $port $addr $byte_cnt 0]        
         set base_output_addr 0xc820  
         if {$result == -1} {
             return
        }
	} elseif {[is_e3]} {
			set result [sfp_eeprom_warpcore $port $addr $byte_cnt]
			if {$result != 0} {
				return
			}
	} else {
        puts "Invalid phy type for this operation"
        return 
    }
   
    for {set i 0} {$i < $byte_cnt} {incr i} {
		 if {[is_e3]} {

			set byte [expr $eeprom_data([expr $i/4]) >> (8*($i%4))]
		 } else {
            set byte [_access_phy::extphy $port 1 [expr $base_output_addr + $i ]]
		 }
         set byte [expr $byte & 0xff]
         #set str $str$byte  
         #puts $byte
         lappend hex_str [format "0x%x" $byte]
         append ascii_str [format "%c" $byte]
    }
    puts "hex: $hex_str"
    puts "ascii: $ascii_str"
}


proc _access_phy::write_sfp_eeprom {port addr byte_cnt mylist} {
    for {set i 0} {$i < $byte_cnt} {incr i} {
        set byte [lindex $mylist $i]
        set result [_access_phy::extphy $port 1 [expr 0x8007 + $i ] $byte]
    }
    set result [sfp_eeprom $port $addr $byte_cnt 1]
}

proc _access_phy::phy_dump {port bank} {

    set bank [expr $bank & 0xfff0]
    set line [format "Port: %s      Bank: 0x%x" "$port" "$bank" ]
    puts "$line"
    for {set i 0} {$i<4} {incr i} {
        set i0 [expr $i*4 + 0]
        set val0 [phy $port [expr $bank + $i0]]
        set i1 [expr $i*4 + 1]
        set val1 [phy $port [expr $bank + $i1]]
        set i2 [expr $i*4 + 2]
        set val2 [phy $port [expr $bank + $i2]]
        set i3 [expr $i*4 + 3]
        set val3 [phy $port [expr $bank + $i3]]
        set line [format "0x%x: %s    0x%x: %s    0x%x: %s    0x%x: %s" "$i0" "$val0" "$i1" "$val1" "$i2" "$val2" "$i3" "$val3"]
        puts "$line"
    }
}


######################################################################
# Dumping of relevant PHY registers. 
######################################################################
proc _access_phy::extphy_dump {{phy "auto"}} {
    set port [lookup_port]
    set id [extphy $port 1 3]
    if {$phy == "auto"} {
        if {($id == 0x5cb0) || ($id == 0x5cb3) || ($id == 0x5cb4)} {
            set phy 84833
        } elseif {$id == 0x6036} {
            set id [extphy 0 1 0xc802]
            if {$id == 0x8073} {
                set phy 8073
            } else {
                set phy 8727
            }
        } elseif {$id == 0x6035} {
            set phy 8706
        } elseif {$id == 0x5cd5} {
            set phy 54618
        } elseif {($id & 0xfff0) == 0x5d10} {
            set phy 54616
        } elseif {$id == 0x5c62} {
            set phy 84823
        } elseif {$id == 0xffff} {
            error "ExtPHY either in reset or doesn't exist."
        } else {
            error "Unknown PHY: ID=$id"
        }
        puts "Detected $phy."
    }

    if {($phy==54616) || ($phy==54618)} {
        gphy_dump
        return
    }
    # The offsets are relative to 84823, 84833 needs adjustments.
    set reglist {}
    if {($phy == 84823) || ($phy == 84833)} {
        lappend reglist {1 0xa82c} {1 0xa82d} {1 0xa82e} {1 0xa83b}
        lappend reglist {1 0xa82f} {1 0xa830} {1 0xa831} {1 0xa83c}
        lappend reglist {1 0xa832} {1 0xa833} {1 0xa834} {1 0xa83d}
        lappend reglist {1 0xa835} {1 0xa836} {1 0xa837} {0 0}
        lappend reglist {1 0xa838} {1 0xa839} {1 0xa83a}
        lappend reglist {7 0xffe0} {7 0xffe1} {30 0x2} {30 0x3}
        lappend reglist {7 0xffe4} {7 0xffe5} {7 0xffe6}
        lappend reglist {7 0xffe9}
        #lappend reglist {7 0xffec} {7 0xffed} {7 0xffee} {7 0xffef} 
        #lappend reglist {7 0xfff0} {7 0xfff1} {7 0xfff2} {7 0xfff3} 
        #lappend reglist {7 0xfff4} {7 0xfff5} {7 0xfff6} {7 0xfff7} 
        lappend reglist {7 0xfff9}
        #lappend reglist {7 0xfffc} {7 0xfffd} {7 0xfffe} {7 0xffff} 
        lappend reglist {7 0} {7 1} {7 0x20} {7 0x3c}
        lappend reglist {30 0x400d} {30 0x401a} {30 0x4110} {30 0x4111}
        lappend reglist {30 0x4112} {30 0x4113} {30 0x4131} {30 0x412e}
        lappend reglist {30 0x4083} {30 0x4085}
        lappend reglist {1 0} {1 0xa811}
        lappend reglist {1 0xa83b}
        #set reglist {{0x1e 0x400d} {0x1e 0x4083} {0x1e 0x4085} }
    } elseif {($phy==8727) || ($phy==8706)} {
        lappend reglist {1 0x0000} {1 0x0001} {1 0x0007} {1 0x000a}
        lappend reglist {3 0x0020} {0 0} {0 0} {0 0}
        lappend reglist {4 0x0001} {4 0x0018} {1 0x8005} {0 0}
        lappend reglist {1 0x9000} {1 0x9003} {0 0} {0 0}
        lappend reglist {1 0x9001} {1 0x9004} {1 0x9002} {1 0x9005}
        lappend reglist {1 0xc800} {1 0xc802} {1 0xc808} {1 0xc80e}
        lappend reglist {1 0xc820} {1 0xcd04} {0 0} {0 0}
        lappend reglist {1 0xca10} {1 0xca13} {1 0xca19} {1 0xca1a}
        lappend reglist {1 0xca1c} {0 0} {0 0} {0 0}
        lappend reglist {7 0x8304} {7 0xffe0} {7 0xffe4} {7 0xffe5}
    } elseif {($phy==8073)} {
        lappend reglist {1 0x0000} {1 0x0001} {1 0x0007} {1 0x0009}
        lappend reglist {1 0x000a} {1 0x0096} {1 0x0097} {1 0x00a0}
        lappend reglist {1 0x00a1} {0 0} {0 0} {0 0}
        lappend reglist {3 0x0020} {0 0} {0 0} {0 0}
        lappend reglist {7 0x0000} {7 0x0001} {7 0x0011} {7 0x0014}
        lappend reglist {7 0x0030} {0 0} {0 0} {0 0}
        lappend reglist {1 0x9000} {1 0x9003} {0 0} {0 0}
        lappend reglist {1 0x9001} {1 0x9004} {1 0x9002} {1 0x9005}
        lappend reglist {1 0xc802} {1 0xc808} {1 0xc80e} {1 0xc820}
        lappend reglist {1 0xca10} {1 0xca13} {1 0xca19} {1 0xca1a}
        lappend reglist {1 0xca1c} {1 0xca1d} {1 0xca46} {0 0}
        lappend reglist {4 0x80A1} {4 0x80A7} {4 0x80e0} {4 0x80fa}
        lappend reglist {7 0x8300} {7 0x8304} {7 0x812b} {7 0x8370}
        lappend reglist {7 0x8350} {7 0x8329} {7 0x8372} {0 0}
        lappend reglist {7 0xffe0} {7 0xffe4} {7 0xffe5} {0 0}
    }

    for {set idx 0} {$idx < [llength $reglist]} {incr idx} {
        set item [lindex $reglist $idx]
        set devad [lindex $item 0]
        set offset [lindex $item 1]
        if {($::current(CHIP_NUM)==57711) || ($::current(CHIP_NUM)==57710)} {
            set port [expr $::current(PCI_FUNC) & 1]
        } elseif {($::current(CHIP_NUM)==57800) || ($::current(CHIP_NUM)==57840)} {
            # Port 1 for functions 2, 3, 6, & 7
            set port [expr ($::current(PCI_FUNC) & 2) >> 1]
        } else {
            set port 0
        }

        if {($devad==0) && ($offset==0)} {
            # Skip showing this. Do nothing.
        } else {
             set val [extphy $port $devad $offset]
             puts -nonewline "[format %02x $devad].[format %04x $offset]: [format %04x $val]  "
        }
        if {($idx & 3) == 3} {puts ""}
    }
    if {($idx & 3) != 0} {puts ""}
    if {$::sys(IS_EDEBUG) != 1} { driver link_state }

}


proc _access_phy::wc_dump_bank {lane devad bank} {

    set bank [expr $bank & 0xfff0]
    set line [format "lane: %s Devad:%x  Bank: 0x%x" "$lane" "$devad" "$bank" ]
    puts "$line"
    for {set i 0} {$i<4} {incr i} {
        set i0 [expr $i*4 + 0]
        set val0 [wc_phy $lane $devad [expr $bank + $i0]]
        set i1 [expr $i*4 + 1]
        set val1 [wc_phy $lane $devad [expr $bank + $i1]]
        set i2 [expr $i*4 + 2]
        set val2 [wc_phy $lane $devad [expr $bank + $i2]]
        set i3 [expr $i*4 + 3]
        set val3 [wc_phy $lane $devad [expr $bank + $i3]]
        set line [format "0x%x: %s    0x%x: %s    0x%x: %s    0x%x: %s" "$i0" "$val0" "$i1" "$val1" "$i2" "$val2" "$i3" "$val3"]
        puts "$line"
    }
}

proc _access_phy::dump_regs {port} {
    #log open $my_log
    phy_dump $port 0x0000
    phy_dump $port 0x0010
    phy_dump $port 0x0030
    phy_dump $port 0x8050
    phy_dump $port 0x8060
    phy_dump $port 0x80B0
    phy_dump $port 0x8120
    phy_dump $port 0x8130
    phy_dump $port 0x8300
    phy_dump $port 0x8320
    phy_dump $port 0x8330
    phy_dump $port 0x8350
    phy_dump $port 0x8370
    phy_dump $port 0xFFE0
   # log close
   # puts "Log file: \"$my_log\""
}

proc _access_phy::wc_phy_dump {lane} {
	
	# Global 
	wc_dump_bank 0 3 0x8000
	wc_dump_bank 0 3 0x8010
	wc_dump_bank 0 3 0x8050
	wc_dump_bank 0 3 0x8060
	wc_dump_bank 0 3 0x8070
	wc_dump_bank 0 3 0x8080
	wc_dump_bank 0 3 0x8090
	wc_dump_bank 0 3 0x80b0
	wc_dump_bank 0 3 0x80c0
	wc_dump_bank 0 3 0x80d0
	wc_dump_bank 0 3 0x80e0
	wc_dump_bank 0 3 0x8100
	wc_dump_bank 0 3 0x8110
	wc_dump_bank 0 3 0x8120
	wc_dump_bank 0 3 0x8130
	wc_dump_bank 0 3 0x8140
	wc_dump_bank 0 3 0x8150
	wc_dump_bank 0 3 0x8190
	wc_dump_bank 0 3 0x81d0
	wc_dump_bank 0 3 0x81e0
	wc_dump_bank 0 3 0x81f0
	wc_dump_bank 0 3 0xffc0
	wc_dump_bank 0 3 0xffd0
	wc_dump_bank 0 3 0xffe0

	wc_dump_bank $lane 3 0x00
	wc_dump_bank $lane 3 0x10
	wc_dump_bank $lane 3 0x20
	wc_dump_bank $lane 3 0x30
	wc_dump_bank $lane 3 0x82e0
	wc_dump_bank $lane 3 0x82f0
	wc_dump_bank $lane 3 0x8300
	wc_dump_bank $lane 3 0x8310
	wc_dump_bank $lane 3 0x8320
	wc_dump_bank $lane 3 0x8340
	wc_dump_bank $lane 3 0x8350
	wc_dump_bank $lane 3 0x8360
	wc_dump_bank $lane 3 0x8370
	wc_dump_bank $lane 3 0x8380
	wc_dump_bank $lane 3 0x8390
	wc_dump_bank $lane 3 0x83a0
	wc_dump_bank $lane 3 0x83b0
	wc_dump_bank $lane 3 0x83c0
	wc_dump_bank $lane 3 0x8420
	wc_dump_bank $lane 3 0x8430
	wc_dump_bank $lane 3 0x8440
	wc_dump_bank $lane 3 0x8450

	wc_dump_bank $lane 1 0
	wc_dump_bank $lane 1 0x90
	wc_dump_bank $lane 1 0xa0

	wc_dump_bank $lane 7 0x0
	wc_dump_bank $lane 7 0x10
	wc_dump_bank $lane 7 0x30
}

######################################################################
# Dumping of relevant WC registers. 
######################################################################

proc _access_phy::wc_dump {lane} {
    set reglist { {3 0x816b "RX lane swap"}
                  {3 0x8169 "TX lane swap"}
		  {3 0x8000 "XgxsBlk0 xgxsControl"}
		  {3 0x8345 "digital5_misc6"} 
                  {3 0x800e "misc_ctrl_1"} 
                  {3 0x8300 "srds_dgtl_ctrl1kx1"} 
                  {3 0xffe0 "combo_ieee0_miictrl"} 
                  {3 0xffde "AER"} 
                  {3 0x80b9 "Rx0_AnaRxCtrl1G"} 
                  {3 0x83c2 "Rx66_SCW0"} 
                  {3 0x83c3 "Rx66_SCW1"} 
                  {3 0x83c4 "Rx66_SCW2"} 
                  {3 0x83c5 "Rx66_SCW3"} 
                  {3 0x83c6 "Rx66_SCW0_mask"} 
                  {3 0x83c7 "Rx66_SCW1_mask"} 
                  {3 0x83c8 "Rx66_SCW2_mask"} 
                  {3 0x83c9 "Rx66_SCW3_mask"} 
                  {3 0x8308 "srds_dgtl_misc1"} 
                  {3 0x833c "digital4_misc3"} 
                  {3 0x0011 "AN_ieee1_adv1"} 
                  {3 0x0000 "ieee0_miictrl"} 
                  {3 0x0001 "ieee0Blk_MIIStat"} 
                  {3 0x8329 "digital3_up1"} 
                  {3 0x8349 "digital5_misc7"} 
                  {3 0x8015 "xgxsblk1_ln_ctrl0"} 
                  {3 0x8016 "xgxsblk1_ln_ctrl1"} 
                  {3 0x0096 "ieee9_10gbkr_pmd_r150"} 
                  {3 0x82e3 "cl72_usr0_misc1_ctrl"} 
                  {3 0x0007 "ieee0_an_np"} 
                  {3 0x8309 "srds_dgtl_misc2"} 
                  {3 0x83b0 "tx66_ctrl"} 
                  {3 0x83c0 "rx66_ctrl"} 
                  {3 0x821e "dsc2b0_misc_ctrl0"} 
                  {3 0x8302 "srds_dgtl_ctrl1kx3"} 
                  {3 0x8400 "cl45_fx100_ctrl1"} 
                  {3 0x8402 "cl45_fx100_ctrl3"} 
                  {3 0x833c "cl45_dgtl4_misc3"} 
                  {3 0x8301 "srds_dgtl_ctrl1kx2"} 
                  {3 0x812b "gp_an_status1"} 
                  {3 0x8010 "xgxsblk1_deskew"} 
                  {3 0x8017 "xgxsblk1_ln_ctrl2"}
		  {3 0x8141 "xgxs x2 ctrl2"} 
                  {3 0x834d "dgtl5_lnk_status"} 
                  {3 0x8343 "dgtl5_speed"} 
                  {7 0x0000 "mdio_an_ctrl"} 
                  {7 0x0001 "mdio_an_status"} 
                  {7 0x0011 "mdio_an_local_adv1"} 
                  {7 0x0014 "mdio_an_lp_adv1"} 
              } 
    for {set idx 0} {$idx < [llength $reglist]} {incr idx} {
        set item [lindex $reglist $idx]
        set devad [lindex $item 0]
        set offset [lindex $item 1]
        set name [lindex $item 2]
        set val [wc_phy $lane $devad $offset]
        puts -nonewline "[format %01x $devad].[format %04x $offset]: [format %04x $val] ([format %22s $name])  "
        if {($idx & 1) == 1} {puts ""}
    }
    if {($idx & 1) != 0} {puts ""}
}

proc _access_phy::lookup_port {} {
    if {($::current(CHIP_NUM) == 57712) || 
            ($::current(CHIP_NUM) == "57712E") || 
            ($::current(CHIP_NUM) == 57810) ||
            ($::current(CHIP_NUM) == 57811)} {
        return 0
    } elseif {($::current(CHIP_NUM)==57800) || ($::current(CHIP_NUM)==57840)} {
        if {$::current(PCI_FUNC) & 2} {
            return 1
        } else {
            return 0
        }
    } else {
        # E1/E1H
        if {$::current(PCI_FUNC) & 1} {
            return 1
        } else {
            return 0
        }
    }
}

proc _access_phy::autogreeen_stats {} {
     variable ext_phy_name    

    set port [lookup_port]
    # Assume port 0 for now. This gets tricky on quad port ext PHY design.
    mii_init $port 1
    if {$ext_phy_name == 84833} {

        #set read clear
        set read_clear [extphy $port 1 0xa8f5]
        set read_clear [expr {$read_clear | 0x1}]
        extphy $port 1 0xa8f5 $read_clear
    
        set tx_hi_val [extphy $port 1 0xa8f6]
        set tx_lo_val [extphy $port 1 0xa8f7]
    
        set rx_hi_val [extphy $port 1 0xa8fa]
        set rx_lo_val [extphy $port 1 0xa8fb]
    
        puts "Auto-grEEEn stats:"
        puts "TX Count: (hi)$tx_hi_val - (lo)$tx_lo_val"
        puts "RX Count: (hi)$rx_hi_val - (lo)$rx_lo_val"

	set ret_list "$tx_hi_val $tx_lo_val $rx_hi_val $rx_lo_val"
	
    } elseif {$ext_phy_name == 54618} {
        
        set lpi_count_val [extphy $port 7 0x803f]

        puts "Auto-grEEEn stats:"
        puts "LPI Mode Counter: $lpi_count_val"
	
	set ret_list "$lpi_count_val"

    } else {
        puts "Phy not supported"
        return
    }
    return $ret_list
}

# Add help information to the help menu
_access_phy::phy_help
