

# This script provides information about the status of the PCIE link by reading
# different registers from the PCIE core and from the PXP.
# Some of the parameters that are extracted here are: Link Width, Link Speed, Max Read
# Request Size, Max Payload Size and BusMasterEnable bit


## Function's arguments:
## file_name - name of log file for the script's output. If none is attached the output will be sent to STDOUT
## if_uart - if the script is being run through uart, this argument should be 1, otherwise the script will not work
 
proc pstat { {file_name ""} {if_uart 0} } {

##Include the addresses of the registers

    set addr_file [open pstat_rg.txt "r"]
	while {[gets $addr_file curr_addr] >= 0} {
		set cur_addr_split [split $curr_addr " "]
		if {[lindex $cur_addr_split 2] == "1"} {
			if {[is_e3]} {
				set [lindex $cur_addr_split 0] [lindex $cur_addr_split 1]
			}
		} elseif {[lindex $cur_addr_split 2] == "0"} {
			if {[is_e2]} {
				set [lindex $cur_addr_split 0] [lindex $cur_addr_split 1]
			}
		} else {
			set [lindex $cur_addr_split 0] [lindex $cur_addr_split 1]
		}
	}
	close $addr_file

    global rdfFile

    #Parse the appropriate RDF file according to DID
    get_rdf_file
    parse_rdf -f $rdfFile

	#Find out what is the current chip_num
	set chip_n [reg_read -n MISC_REG_CHIP_NUM]
	
	
	if {$file_name != ""} {
		set out_file [open $file_name "w"]
		puts $out_file [format "GENERAL SETTINGS:"]

	} else {
		puts stdout [format "GENERAL SETTINGS:"]
	}
	set level [reg dread $link_capability_addr]

	set mlw [expr (($level & 0x1f0)>>4)]
	set mls [expr ($level & 0xf)]
	if {$file_name != ""} {
	puts $out_file [format "MAX LINK WIDTH declared: 0x%x MAX LINK SPEED declared: 0x%d" $mlw $mls ]
	} else {
		puts stdout [format "MAX LINK WIDTH declared: 0x%x MAX LINK SPEED declared: 0x%d" $mlw $mls ]
	}
	set level [reg dread $link_status_control_addr]

	set mlwa [expr (($level & 0x1f00000)>>20)]
	set mlsa [expr (($level & 0xf0000)>>16)]

	if {$file_name != ""} {
	puts $out_file [format "MAX LINK WIDTH actual  : 0x%x MAX LINK SPEED actual  : 0x%x" $mlwa $mlsa ]
	} else {
		puts stdout [format "MAX LINK WIDTH actual  : 0x%x MAX LINK SPEED actual  : 0x%x" $mlwa $mlsa ]
	}
	set level [reg dread $status_command_addr]
	if {$file_name != ""} {
	puts $out_file [format "PCI status command is 0x%x (a value of 0x00100446 or a subset of these bits being set is OK) " $level ]
	} else {
		puts stdout [format "PCI status command is 0x%x (a value of 0x00100446 or a subset of these bits being set is OK) " $level ]
	}
	set level [reg dread $device_status_control_addr]

	set mrrs   [expr (($level & 0x00007000)>>12)]
	set mps    [expr (($level & 0x000000e0)>>5)]
	set status [expr (($level & 0x003f0000)>>16)]



	if {$file_name != ""} {
	puts $out_file [format "MRRS: 0x%x MPS: 0x%x" $mrrs $mps]
	puts $out_file [format "NO_PEND,VAUX,UNSUP,FATAL,NON_FATAL,CORR 0x%x (usually 0x9) " $status ]
	puts $out_file  [format "FLOW CONTROL:"]

	} else {
		puts stdout [format "MRRS: 0x%x MPS: 0x%x" $mrrs $mps]
		puts stdout [format "NO_PEND,VAUX,UNSUP,FATAL,NON_FATAL,CORR 0x%x (usually 0x9) " $status ]

		puts stdout [format "FLOW CONTROL:"]
	}





	set level1 [reg dread $tl_hdr_fc_st_addr]

	set cplh_avail [expr (($level1 & 0xff0000)>>16)]
	set ph_avail   [expr (($level1 & 0xff00)>>8)]
	set nph_avail  [expr (($level1 & 0xff))]

	set level2 [reg dread $tl_dat_fc_st_addr]

	set cpld_avail [expr (($level2 & 0xfff0000)>>16)]
	set pd_avail   [expr (($level2 & 0xfff))]

	
	if {$file_name != ""} {
		puts $out_file [format "RC available credits: NPH 0x%x PH 0x%x PD 0x%x CPLH 0x%x CPLD 0x%x " $nph_avail $ph_avail 		$pd_avail $cplh_avail $cpld_avail ]
	} else {
		puts stdout [format "RC available credits: NPH 0x%x PH 0x%x PD 0x%x CPLH 0x%x CPLD 0x%x " $nph_avail $ph_avail 		$pd_avail $cplh_avail $cpld_avail ]
	}



	set level1 [reg dread $tl_tgt_crdt_st_addr]
	set ep_pd_avail   [expr (($level1 & 0x007f00)>>8)]
	set ep_ph_avail   [expr (($level1 & 0x7f))]
	set ep_nph_avail  [expr (($level1 & 0x10000)>>16)]

	
	if {$file_name != ""} {
		puts $out_file [format "EP available credits: NPH 0x%x PH 0x%x PD 0x%x " $ep_nph_avail $ep_ph_avail $ep_pd_avail ]
	} else {
		puts stdout [format "EP available credits: NPH 0x%x PH 0x%x PD 0x%x " $ep_nph_avail $ep_ph_avail $ep_pd_avail ]
	}
	set level1 [reg dread $tl_crdt_alloc_st_addr]
	set pd_alloc   [expr (($level1 & 0x000000ff000000)>>24)]
	set ph_alloc   [expr (($level1 & 0xff0000)>>16)]
	set nph_alloc  [expr (($level1 & 0xff))]
	if {$file_name != ""} {
		puts $out_file [format "EP allocated counter: NPH 0x%x PH 0x%x PD 0x%x " $nph_alloc $ph_alloc $pd_alloc ]
		puts $out_file [format "read/write from 0x2994 to see that EP FC counters are advancing"]
		puts $out_file [format "read from 0x2988,0x298c to see that RC FC counters are advancing"]
	} else { 
		puts stdout [format "EP allocated counter: NPH 0x%x PH 0x%x PD 0x%x " $nph_alloc $ph_alloc $pd_alloc ]
		puts stdout [format "read/write from 0x2994 to see that EP FC counters are advancing"]
		puts stdout [format "read from 0x2988,0x298c to see that RC FC counters are advancing"]
	}


	set level [reg_read -n PXP2_REG_PGL_TXR_CDTS]
	
	if {$file_name != ""} {
		puts $out_file [format "PGLC read credits (read_pending, nph(4:0)): 0x%x" $level]
	} else {
		puts stdout [format "PGLC read credits (read_pending, nph(4:0)): 0x%x" $level]
	}
	set level [reg_read -n PXP2_REG_PGL_TXW_CDTS]
	
	if {$file_name != ""} {
		puts $out_file [format "PGLC write credits (data_ready,ph(4:0),pd(11:0)): 0x%x" $level]
	} else {
		puts stdout [format "PGLC write credits (data_ready,ph(4:0),pd(11:0)): 0x%x" $level]
	}
	set level  [reg_read -n PXP2_REG_PGL_EXP_ROM2]
	set level2 [reg_read -n PXP2_REG_PGL_MOT]

	 if {![is_e1x]} { 
	
		set level3 [reg_read -n PGLUE_B_REG_TAGS_63_32]
		
		if {$file_name != ""} {
			puts $out_file [format "PGLC tag vector for tags 0-31: 0x%x  PGLC tags vector for tags 32-63: 0x%x PGLC status:  0x%x " $level $level3 $level2]
		} else {
			puts stdout [format "PGLC tag vector for tags 0-31: 0x%x  PGLC tags vector for tags 32-63: 0x%x PGLC status:  0x%x " $level $level3 $level2]
		}
	} else {
		
		if {$file_name != ""} {
			puts $out_file [format "PGLC tag vector: 0x%x  PGLC status:  0x%x " $level $level2]
		} else {
			puts stdout [format "PGLC tag vector: 0x%x  PGLC status:  0x%x " $level $level2]
		}
	}

	## Read the 'ME register' by reading from the DQ BAR space - only if not working through UART
	if {$if_uart==0} {
		set level [bar 1 read 0x0 4]

		if {$file_name != ""} {
			puts $out_file "ME register's value is: $level"
		} else {
			puts stdout "ME register's value is: $level"
		}
	}
	

	set level [reg dread $device_vendor_id_addr]
	set  did [expr (($level & 0xFFFF0000)>>16)]
	set vid [expr $level & 0xFFFF]

	

	if {$file_name != ""} {
		puts $out_file [format "Device ID is: 0x%x, Vendor ID is: 0x%x" $did $vid]
	} else {
		puts stdout [format "Device ID is: 0x%x, Vendor ID is: 0x%x" $did $vid]
	}

	set level1 [reg dread $bar_1_addr]
	set level2 [reg dread $bar_2_addr]
	set level3 [reg dread $bar_3_addr]
	set level4 [reg dread $bar_4_addr]
	set level5 [reg dread $bar_5_addr]
	set level6 [reg dread $bar_6_addr]

	
	if {$file_name != ""} {
	puts $out_file [format "PF bar1-6 addresses are: 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x" $level1 $level2 $level3 $level4 $level5 $level6]
	} else {
		puts stdout [format "PF bar1-6 addresses are: 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x" $level1 $level2 $level3 $level4 $level5 $level6]
	}


	if {![is_e1x]} { 

		set level1 [reg dread $header_log1_addr]
		set level2 [reg dread $header_log2_addr]
		set level3 [reg dread $header_log3_addr]
		set level4 [reg dread $header_log4_addr]

		
		if {$file_name != ""} {
			puts $out_file [format "header log registers 1-4 are: 0x%x 0x%x 0x%x 0x%x" $level1 $level2 $level3 $level4]
		} else {
			puts stdout [format "header log registers 1-4 are: 0x%x 0x%x 0x%x 0x%x" $level1 $level2 $level3 $level4]
		}



		set level0 [reg dread $vf_bar0_addr]
		set level1 [reg dread $vf_bar1_addr]
		set level2 [reg dread $vf_bar2_addr]
		set level3 [reg dread $vf_bar3_addr]
		set level4 [reg dread $vf_bar4_addr]
		set level5 [reg dread $vf_bar5_addr]

		
		if {$file_name != ""} {
			puts $out_file [format "VF bar 0-5 addresses are: 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x" $level0 $level1 $level2 $level3 $level4 $level5]
		} else {
			puts stdout [format "VF bar 0-5 addresses are: 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x" $level0 $level1 $level2 $level3 $level4 $level5]
		}

		set level [reg dread $msix_cap_addr]
		
		if {$file_name != ""} {
		puts $out_file [format "MSIX_CAP register is 0x%x" $level]
		} else {
			puts stdout [format "MSIX_CAP register is 0x%x" $level]
		}
		set level [reg dread $lat_min_grant_int_pin_int_line_addr]
		
		if {$file_name != ""} {
			puts $out_file [format "INT line is 0x%x " $level]

			puts $out_file [format "SRIOV data:"]
		} else {
			puts stdout [format "INT line is 0x%x " $level]

			puts stdout [format "SRIOV data:"]
		}
		set level [reg dread $sriov_control_addr]
		
		if {$file_name != ""} {
			puts $out_file [format "sriov control register is 0x%x " $level]
		} else {
			puts stdout [format "sriov control register is 0x%x " $level]
		}
		set level [reg dread $sriov_numvf_addr]
		
		if {$file_name != ""} {
			puts $out_file [format "num VF is 0x%x " $level]
		} else {
			puts stdout [format "num VF is 0x%x " $level]
		}
		set level [reg dread $sriov_vfoffset_addr]
		
		if {$file_name != ""} {
			puts $out_file [format "VF offset is 0x%x " $level]
		} else {
			puts stdout [format "VF offset is 0x%x " $level]
		}
		set level [reg dread $ats_control_addr]
		
		if {$file_name != ""} {
			puts $out_file [format "ATS control register is 0x%x" $level]
		} else {
			puts stdout [format "ATS control register is 0x%x" $level]
		}
	}




	

	set level [reg dread $pm_csr_addr]
	set pme   [expr (($level & 0x8000)>>15)]
	set pm    [expr (($level & 0x3))]

	
	if {$file_name != ""} {
		puts $out_file [format "POWER MANAGEMENT:"]

		puts $out_file [format "PME status, PM status is: 0x%x 0x%x (normal power mode is: 0x0,0x0) " $pme $pm ]
	} else {
		puts stdout [format "POWER MANAGEMENT:"]
		puts stdout [format "PME status, PM status is: 0x%x 0x%x (normal power mode is: 0x0,0x0) " $pme $pm ]
	}
	set level [reg dread $config_3_addr]
	set power_present [expr (($level & 0xc0000000))]

	
	if {$file_name != ""} {
		puts $out_file [format "{Main Power,Vaux Power} = 0x%x (should be 0xc0000000)" $power_present ]
	} else {
		puts stdout [format "{Main Power,Vaux Power} = 0x%x (should be 0xc0000000)" $power_present ]
	}
	set level [reg dread $pm_status_0_addr]
	
	if {$file_name != ""} {
		puts $out_file [format "PM_STATUS_0 (addr 0x2878) : 0x%x (should be: 0x1000202)" $level ]
	} else {
		puts stdout [format "PM_STATUS_0 (addr 0x2878) : 0x%x (should be: 0x1000202)" $level ]
	}
	set level [reg dread $pm_status_1_addr]
	
	if {$file_name != ""} {
		puts $out_file [format "PM_STATUS_1 (addr 0x287c) : 0x%x (0x2d - PME en, 0x24 - PME dis)" $level ]
	} else {
		puts stdout [format "PM_STATUS_1 (addr 0x287c) : 0x%x (0x2d - PME en, 0x24 - PME dis)" $level ]
	}
	set level [reg_read -n PXP2_REG_PGL_PM_STATUS]
	
	if {$file_name != ""} {
		puts $out_file [format "{LINK_L23,PM_STATE1,PM_STATE0} : 0x%x" $level ]
	} else {
		puts stdout [format "{LINK_L23,PM_STATE1,PM_STATE0} : 0x%x" $level ]
	}

	set level [reg dread $link_status_control_addr]
	set aspm  [expr (($level & 0x3))]
	set ckc  [expr (($level & 0x40)>>6)]



	if {$file_name != ""} {
		puts $out_file [format "Common Clock, ASPM mode (3 is full): 0x%x 0x%x" $ckc $aspm ]

		puts $out_file [format "ERRORS:"]
	} else {
		puts stdout [format "Common Clock, ASPM mode (3 is full): 0x%x 0x%x" $ckc $aspm ]

		puts stdout [format "ERRORS:"]
	}



	set level  [reg dread $tl_control_5_addr]
	
	if {$file_name != ""} {
		puts $out_file [format "PCIE  ATTN      : 0x%x " $level ]
	} else {
		puts stdout [format "PCIE  ATTN      : 0x%x " $level ]
	}

	if {[is_e3]} {
	set level [reg dread $uc_err_status_addr]
	} else {
		set level [reg dread 0x2114]
	}
	
	if {$file_name != ""} {
		puts $out_file [format "UCORR ERR STATUS: 0x%x" $level]
	} else {
		puts stdout [format "UCORR ERR STATUS: 0x%x" $level]
	}

	if {[is_e3]} {
	set level [reg dread $corr_err_status_addr]
	} else {
		set level [reg dread 0x2120]
	}
	
	if {$file_name != ""} {
		puts $out_file [format "CORR  ERR STATUS: 0x%x" $level]
	} else {
		puts stdout [format "CORR  ERR STATUS: 0x%x" $level]
	}
	set level [reg dread $dlattn_vec_addr]
	
	if {$file_name != ""} {
		puts $out_file [format "DLL   ERR STATUS: 0x%x (0x20 means many corr errs)" $level]
	} else {
		puts stdout [format "DLL   ERR STATUS: 0x%x (0x20 means many corr errs)" $level]
	}
	set level [reg dread $phy_err_attn_vec_addr]
	
	if {$file_name != ""} {
		puts $out_file [format "PHY   ERR STATUS: 0x%x" $level]
	} else {
		puts stdout [format "PHY   ERR STATUS: 0x%x" $level]
	}
	set level [reg_read -n PXP2_REG_PXP2_INT_STS_0]
	
	if {$file_name != ""} {
		puts $out_file [format "PXP   INT STATUS: 0x%x" $level]
	} else {
		puts stdout [format "PXP   INT STATUS: 0x%x" $level]
	}



	


	set level1 [reg dread $phy_ltssm_hist_0_addr]
	set level2 [reg dread $phy_ltssm_hist_1_addr]
	set level3 [reg dread $phy_ltssm_hist_2_addr]
	if {[is_e3]} {
		set level4 [reg dread $phy_ltssm_hist_3_addr]
	}
	

	if {[is_e3]} {
		if {$file_name != ""} {
			puts $out_file [format "DEBUG:"]
			puts $out_file [format "LTSSM states {hist_0, hist_1, hist_2, hist_3} were: 0x%x 0x%x 0x%x 0x%x. (L0 decoding: 0x00)" $level1 $level2 $level3 $level4]
		} else {
			puts stdout [format "DEBUG:"]
			puts stdout [format "LTSSM states {hist_0, hist_1, hist_2, hist_3} were: 0x%x 0x%x 0x%x 0x%x. (L0 decoding: 0x00)" $level1 $level2 $level3 $level4]
		}
	} else {
		if {$file_name != ""} {
			puts $out_file [format "DEBUG:"]
			puts $out_file [format "LTSSM states {hist_0, hist_1, hist_2} were: 0x%x 0x%x 0x%x. (L0 decoding: 0x00)" $level1 $level2 $level3]
			puts $out_file [format "serdes bug scenario : 81828309_83098081_82830980"]
		} else {
			puts stdout [format "DEBUG:"]
			puts stdout [format "LTSSM states {hist_0, hist_1, hist_2} were: 0x%x 0x%x 0x%x. (L0 decoding: 0x00)" $level1 $level2 $level3]
			puts stdout [format "serdes bug scenario : 81828309_83098081_82830980"]
		}
	}

	set level1 [reg dread $recovery_hist_0_addr]
	if {[is_e3]} {
		set level2 [reg dread $recovery_hist_1_addr]
	}
		
	if {[is_e3]} {
		if {$file_name != ""} {
			puts $out_file [format "DEBUG:"]	
			puts $out_file [format "Recovery history {hist_0, hist_1} were: 0x%x 0x%x. " $level1 $level2]
		} else {
			puts stdout [format "DEBUG:"]
			puts stdout [format "Recovery history {hist_0, hist_1} were: 0x%x 0x%x. " $level1 $level2]
		}
	} else {
		if {$file_name != ""} {
			puts $out_file [format "DEBUG:"]	
			puts $out_file [format "Recovery history {hist_0} was: 0x%x . " $level1]
		} else {
			puts stdout [format "DEBUG:"]	
			puts stdout [format "Recovery history {hist_0} was: 0x%x . " $level1]
		}
	}
		
		
		
	set level [reg dread $dl_status_addr]
	
	if {$file_name != ""} {
		puts $out_file [format "DLL STATUS: {DL_INIT,DL_AC,PHYUP,2'b00,RPLY_AFULL,TOTAL_ERRORS(9:0)} 0x%x" $level]
	} else {
		puts stdout [format "DLL STATUS: {DL_INIT,DL_AC,PHYUP,2'b00,RPLY_AFULL,TOTAL_ERRORS(9:0)} 0x%x" $level]
	}

	set level1 [reg dread $tl_smlogic_st_addr]

	
	if {$file_name != ""} {
		puts $out_file [format "TL state machines: 0x%x (should be 0x0 or 0x5 (0x5 means NP machine is busy)) " $level1 ]
	} else {
		puts stdout [format "TL state machines: 0x%x (should be 0x0 or 0x5 (0x5 means NP machine is busy)) " $level1 ]
	}


	#decoding for bits[3:0] of this register:
	#           NP_IDLE          = 4'h0,
	#           NP_CFG_WT_END    = 4'h1,
	#           NP_CFG_RD_DAT    = 4'h2,
	#           NP_CFG_WR_DAT    = 4'h3,
	#           NP_CFG_WT_ACK    = 4'h4,
	#           NP_MEM_RD_DAT    = 4'h5,
	#           NP_MEMR_WT_END   = 4'h6,
	#           NP_WT_CPL_GEN    = 4'h7,
	#           NP_IGNORE_DAT    = 4'h8,
	#           NP_NFC_WT_END    = 4'h9;



	set level0 [reg dread  $phy_dbg_0_addr]
	set level1 [reg dread  $phy_dbg_1_addr]
	set level2 [reg dread  $phy_dbg_2_addr]
	set level3 [reg dread  $phy_dbg_3_addr]
	set level4 [reg dread  $phy_dbg_4_addr]
	set level5 [reg dread  $phy_dbg_5_addr]
	set level6 [reg dread  $phy_dbg_6_addr]
	set level7 [reg dread  $phy_dbg_7_addr]
	set level8 [reg dread  $phy_dbg_8_addr]
	set level9 [reg dread  $phy_dbg_9_addr]
	set level10 [reg dread $phy_dbg_10_addr]


	
	if {$file_name != ""} {
		puts $out_file [format "PCIE PHY DEBUG 0-10 : 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x " $level0 	$level1 	$level2 $level3 $level4 $level5 $level6 $level7 $level8 $level9 $level10  ]
		close $out_file
	} else {
		puts stdout [format "PCIE PHY DEBUG 0-10 : 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x " $level0 	$level1 	$level2 $level3 $level4 $level5 $level6 $level7 $level8 $level9 $level10  ]
	}









}
