###########################################################################
#
# Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved
# Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
# Proprietary and Confidential Information.
#
# This source file is the property of QLogic Corporation, and
# may not be copied or distributed in any isomorphic form without 
# the prior written consent of QLogic Corporation. 
#
# Name:        qstat.tcl
#
# Description: 
#    This is a script which displays the statistics                                        
#
###########################################################################

proc qstat { args } {
    
    set noPause 0
        
    set statList [list "Txed Packets" \
                       "Txed Unicast Packets" \
                       "Txed Multicast Packets" \
                       "Txed Broadcast Packets" \
                       "Txed Octets" \
                       "Rxed Packets" \
                       "Rxed Unicast Packets" \
                       "Rxed Multicast Packets" \
                       "Rxed Broadcast Packets" \
                       "Rxed Octets" \
                       "d3StatsFCSErrors" \
                       "d3StatsIntlMacTxErrors" \
                       "d3StatsCarrierSenseErrors" \
                       "d3StatsAlignmentErrors" \
                       "d3StatsSingleCollisionFrames" \
                       "d3StatsMultipleCollisionFrames" \
                       "d3StatsDeferredTransmissions" \
                       "d3StatsExcessiveCol" \
                       "d3StatsLateCollisions" \
                       "etherStatsCollisions" \
                       "IfInFramesL2FilterDiscards" \
                       "IfInTTL0Discards" \
                       "IfInxxOverflowDiscards" \
                       "IfInMBUFDiscards" \
                       "IfInErrors" \
                       "IfInNoBrbBuffer" \
                       "IfHCInFalseCarrierErrors" \
                       "IfHCOutDiscards" \
                       "OutXonSent" \
		       "OutXoffSent" \
	               "XonPauseFramesReceived" \
		       "XoffPauseFramesReceived" \
                       ]
    
    # Parse the options
    for {set i 0} {$i < [llength $args]} {incr i} {
	switch -- [lindex $args $i] {	    
	    "-no_pause"        { set noPause 1}
            default { puts "Usage: qstat"; puts "Optional: -no_pause"; return 1}
	}
    }
    
    # Read statistics from driver
    driver stat hiStats lowStats
    
    if {$noPause} {       
        puts "[format "%-35s:" [lindex $statList 0]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutPkts) $lowStats(l2_chip_stats_IfHCOutPkts)]"
            
        puts "[format "%-35s:" [lindex $statList 1]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutUcastPkts) $lowStats(l2_chip_stats_IfHCOutUcastPkts)]"
        
        puts "[format "%-35s:" [lindex $statList 2]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutMulticastPkts) $lowStats(l2_chip_stats_IfHCOutMulticastPkts)]"
        
        puts "[format "%-35s:" [lindex $statList 3]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutBroadcastPkts) $lowStats(l2_chip_stats_IfHCOutBroadcastPkts)]"
        
        # Tx Octets
        puts "[format "%-35s:" [lindex $statList 4]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutOctets) $lowStats(l2_chip_stats_IfHCOutOctets)]"
    
        # Rx Packets
        puts "[format "%-35s:" [lindex $statList 5]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCInPkts) $lowStats(l2_chip_stats_IfHCInPkts)]"        
        puts "[format "%-35s:" [lindex $statList 6]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCInUcastPkts) $lowStats(l2_chip_stats_IfHCInUcastPkts)]"        
        puts "[format "%-35s:" [lindex $statList 7]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCInMulticastPkts) $lowStats(l2_chip_stats_IfHCInMulticastPkts)]"        
        puts "[format "%-35s:" [lindex $statList 8]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCInBroadcastPkts) $lowStats(l2_chip_stats_IfHCInBroadcastPkts)]"
        
        # Rx Octets
        puts "[format "%-35s:" [lindex $statList 9]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCInOctets) $lowStats(l2_chip_stats_IfHCInOctets)]"
        
        # Dot3StatsFCSErrors
        puts "[format "%-35s:" [lindex $statList 10]] [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsFCSErrors) $lowStats(l2_chip_stats_Dot3StatsFCSErrors)]"
        
        # Dot3statsInternalMacTransmitErrors
        puts "[format "%-35s:" [lindex $statList 11]] [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsInternalMacTransmitErrors) $lowStats(l2_chip_stats_Dot3StatsInternalMacTransmitErrors)]"
        
        # Dot3StatsCarrierSenseErrors
        puts "[format "%-35s:" [lindex $statList 12]] [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsCarrierSenseErrors) $lowStats(l2_chip_stats_Dot3StatsCarrierSenseErrors)]"
        
        # Dot3StatsAlignmentError`s
        puts "[format "%-35s:" [lindex $statList 13]] [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsAlignmentErrors) $lowStats(l2_chip_stats_Dot3StatsAlignmentErrors)]"
        
        # Dot3StatsSingleCollisionFrames
        puts "[format "%-35s:" [lindex $statList 14]] [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsSingleCollisionFrames) $lowStats(l2_chip_stats_Dot3StatsSingleCollisionFrames)]"
        
        # Dot3StatsMultipleCollisionFrames
        puts "[format "%-35s:" [lindex $statList 15]] [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsMultipleCollisionFrames) $lowStats(l2_chip_stats_Dot3StatsMultipleCollisionFrames)]"
        
        # Dot3StatsDeferredTransmissions
        puts "[format "%-35s:" [lindex $statList 16]] [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsDeferredTransmissions) $lowStats(l2_chip_stats_Dot3StatsDeferredTransmissions)]"
        
        # Dot3StatsExcessiveCollisions
        puts "[format "%-35s:" [lindex $statList 17]] [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsExcessiveCollisions) $lowStats(l2_chip_stats_Dot3StatsExcessiveCollisions)]"
        
        # Dot3StatsLateCollisions
        puts "[format "%-35s:" [lindex $statList 18]] [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsLateCollisions) $lowStats(l2_chip_stats_Dot3StatsLateCollisions)]"
        
        # EtherStatsCollisions
        puts "[format "%-35s:" [lindex $statList 19]] [disp64 unsigned $hiStats(l2_chip_stats_EtherStatsCollisions) $lowStats(l2_chip_stats_EtherStatsCollisions)]"
        
        # IfInFramesL2FilterDiscards
        puts "[format "%-35s:" [lindex $statList 20]] [disp64 unsigned $hiStats(l2_chip_stats_IfInFramesL2FilterDiscards) $lowStats(l2_chip_stats_IfInFramesL2FilterDiscards)]"
        
        # IfInTTL0Discards    
        puts "[format "%-35s:" [lindex $statList 21]] [disp64 unsigned $hiStats(l2_chip_stats_IfInTTL0Discards) $lowStats(l2_chip_stats_IfInTTL0Discards)]"
        
        #IfInxxOverflowDiscards    
        puts "[format "%-35s:" [lindex $statList 22]] [disp64 unsigned $hiStats(l2_chip_stats_IfInxxOverflowDiscards) $lowStats(l2_chip_stats_IfInxxOverflowDiscards)]"
        
        #IfInMBUFDiscards
        puts "[format "%-35s:" [lindex $statList 23]] [disp64 unsigned $hiStats(l2_chip_stats_IfInMBUFDiscards) $lowStats(l2_chip_stats_IfInMBUFDiscards)]"
        
        #IfInErrors
        puts "[format "%-35s:" [lindex $statList 24]] [disp64 unsigned $hiStats(l2_chip_stats_IfInErrors) $lowStats(l2_chip_stats_IfInErrors)]"
        
        #IfInNoBrbBuffer
        puts "[format "%-35s:" [lindex $statList 25]] [disp64 unsigned $hiStats(l2_chip_stats_IfInNoBrbBuffer) $lowStats(l2_chip_stats_IfInNoBrbBuffer)]"
        
        # IfHCInFalseCarrierErrors
        puts "[format "%-35s:" [lindex $statList 26]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCInFalseCarrierErrors) $lowStats(l2_chip_stats_IfHCInFalseCarrierErrors)]"
        
        #IfHCOutDiscards
        puts "[format "%-35s:" [lindex $statList 27]] [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutDiscards) $lowStats(l2_chip_stats_IfHCOutDiscards)]"
        
        #OutXonSent
        puts "[format "%-35s:" [lindex $statList 28]] [disp64 unsigned $hiStats(l2_chip_stats_OutXonSent) $lowStats(l2_chip_stats_OutXonSent)]"
    
        #OutXoffSent
        puts "[format "%-35s:" [lindex $statList 29]] [disp64 unsigned $hiStats(l2_chip_stats_OutXoffSent) $lowStats(l2_chip_stats_OutXoffSent)]"
     
        #XonPauseFramesReceived	
        puts "[format "%-35s:" [lindex $statList 30]] [disp64 unsigned $hiStats(l2_chip_stats_XonPauseFramesReceived) $lowStats(l2_chip_stats_XonPauseFramesReceived)]"
     
        #XoffPauseFramesReceived	
        puts "[format "%-35s:" [lindex $statList 31]] [disp64 unsigned $hiStats(l2_chip_stats_XoffPauseFramesReceived) $lowStats(l2_chip_stats_XoffPauseFramesReceived)]"
       
    } else {   
        append result [format "%-35s:" [lindex $statList 0]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutPkts) $lowStats(l2_chip_stats_IfHCOutPkts)]]
            
        append result [format "%-35s:" [lindex $statList 1]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutUcastPkts) $lowStats(l2_chip_stats_IfHCOutUcastPkts)]]
        
        append result [format "%-35s:" [lindex $statList 2]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutMulticastPkts) $lowStats(l2_chip_stats_IfHCOutMulticastPkts)]]
        
        append result [format "%-35s:" [lindex $statList 3]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutBroadcastPkts) $lowStats(l2_chip_stats_IfHCOutBroadcastPkts)]]
        
        # Tx Octets
        append result [format "%-35s:" [lindex $statList 4]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutOctets) $lowStats(l2_chip_stats_IfHCOutOctets)]]
    
        # Rx Packets
        append result [format "%-35s:" [lindex $statList 5]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCInPkts) $lowStats(l2_chip_stats_IfHCInPkts)]]
        
        append result [format "%-35s:" [lindex $statList 6]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCInUcastPkts) $lowStats(l2_chip_stats_IfHCInUcastPkts)]]
        
        append result [format "%-35s:" [lindex $statList 7]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCInMulticastPkts) $lowStats(l2_chip_stats_IfHCInMulticastPkts)]]
        
        append result [format "%-35s:" [lindex $statList 8]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCInBroadcastPkts) $lowStats(l2_chip_stats_IfHCInBroadcastPkts)]]
        
        # Rx Octets
        append result [format "%-35s:" [lindex $statList 9]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCInOctets) $lowStats(l2_chip_stats_IfHCInOctets)]]
        
        # Dot3StatsFCSErrors
        append result [format "%-35s:" [lindex $statList 10]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsFCSErrors) $lowStats(l2_chip_stats_Dot3StatsFCSErrors)]]
        
        # Dot3statsInternalMacTransmitErrors
        append result [format "%-35s:" [lindex $statList 11]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsInternalMacTransmitErrors) $lowStats(l2_chip_stats_Dot3StatsInternalMacTransmitErrors)]]                                                             
        
        # Dot3StatsCarrierSenseErrors
        append result [format "%-35s:" [lindex $statList 12]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsCarrierSenseErrors) $lowStats(l2_chip_stats_Dot3StatsCarrierSenseErrors)]]
        
        # Dot3StatsAlignmentError`s
        append result [format "%-35s:" [lindex $statList 13]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsAlignmentErrors) $lowStats(l2_chip_stats_Dot3StatsAlignmentErrors)]]
        
        # Dot3StatsSingleCollisionFrames
        append result [format "%-35s:" [lindex $statList 14]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsSingleCollisionFrames) $lowStats(l2_chip_stats_Dot3StatsSingleCollisionFrames)]]
        
        # Dot3StatsMultipleCollisionFrames
        append result [format "%-35s:" [lindex $statList 15]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsMultipleCollisionFrames) $lowStats(l2_chip_stats_Dot3StatsMultipleCollisionFrames)]]
        
        # Dot3StatsDeferredTransmissions
        append result [format "%-35s:" [lindex $statList 16]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsDeferredTransmissions) $lowStats(l2_chip_stats_Dot3StatsDeferredTransmissions)]]
        
        # Dot3StatsExcessiveCollisions
        append result [format "%-35s:" [lindex $statList 17]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsExcessiveCollisions) $lowStats(l2_chip_stats_Dot3StatsExcessiveCollisions)]]
        
        # Dot3StatsLateCollisions
        append result [format "%-35s:" [lindex $statList 18]]    
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_Dot3StatsLateCollisions) $lowStats(l2_chip_stats_Dot3StatsLateCollisions)]]
        
        # EtherStatsCollisions
        append result [format "%-35s:" [lindex $statList 19]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_EtherStatsCollisions) $lowStats(l2_chip_stats_EtherStatsCollisions)]]
        
        # IfInFramesL2FilterDiscards
        append result [format "%-35s:" [lindex $statList 20]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfInFramesL2FilterDiscards) $lowStats(l2_chip_stats_IfInFramesL2FilterDiscards)]]
        
        # IfInTTL0Discards    
        append result [format "%-35s:" [lindex $statList 21]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfInTTL0Discards) $lowStats(l2_chip_stats_IfInTTL0Discards)]]
        
        #IfInxxOverflowDiscards    
        append result [format "%-35s:" [lindex $statList 22]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfInxxOverflowDiscards) $lowStats(l2_chip_stats_IfInxxOverflowDiscards)]]
        
        #IfInMBUFDiscards
        append result [format "%-35s:" [lindex $statList 23]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfInMBUFDiscards) $lowStats(l2_chip_stats_IfInMBUFDiscards)]]
        
        #IfInErrors
        append result [format "%-35s:" [lindex $statList 24]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfInErrors) $lowStats(l2_chip_stats_IfInErrors)]]
        
        #IfInNoBrbBuffer
        append result [format "%-35s:" [lindex $statList 25]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfInNoBrbBuffer) $lowStats(l2_chip_stats_IfInNoBrbBuffer)]]
        
        # IfHCInFalseCarrierErrors
        append result [format "%-35s:" [lindex $statList 26]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCInFalseCarrierErrors) $lowStats(l2_chip_stats_IfHCInFalseCarrierErrors)]]
        
        #IfHCOutDiscards
        append result [format "%-35s:" [lindex $statList 27]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_IfHCOutDiscards) $lowStats(l2_chip_stats_IfHCOutDiscards)]]   
        #OutXonSent
        append result [format "%-35s:" [lindex $statList 28]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_OutXonSent) $lowStats(l2_chip_stats_OutXonSent)]]   
    
        #OutXoffSent
        append result [format "%-35s:" [lindex $statList 29]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_OutXoffSent) $lowStats(l2_chip_stats_OutXoffSent)]]    	    
     
        #XonPauseFramesReceived	
        append result [format "%-35s:" [lindex $statList 30]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_XonPauseFramesReceived) $lowStats(l2_chip_stats_XonPauseFramesReceived)]]    	    
     
        #XoffPauseFramesReceived	
        append result [format "%-35s:" [lindex $statList 31]]        
        append result [format "%25s\n" [disp64 unsigned $hiStats(l2_chip_stats_XoffPauseFramesReceived) $lowStats(l2_chip_stats_XoffPauseFramesReceived)]]    	        
    
        return $result
    }
}

# -----------------------------------------------------------------
# Help info on "qstat" command
# -----------------------------------------------------------------
proc qstat_help {args} {
    set help_info     "Display L2 chip statistics\n"
    append help_info  "Usage: qstat\n"    
    add_help qstat $help_info
}

qstat_help