###########################################################################
#
# Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved
# Proprietary and Confidential Information.
#
# Module Description:
# This script includes impotant utilities such as:
# parse_rdf, reg_read, reg_offset, reg_size, reg_type, reg_info
# read_regs, get_block_registers
##########################################################################

#Globals:

#Blocks table is an array of block name & block offset
global blocksTable

#Blocks to skip
global skipBlocks

#Registeers to skip
global exceptionList

# Regs information in 4 arrays
global list regsName {}
global list regsOffset {}
global list regsSize {}
global list regsCount {}
global list regsType {}
global list regsResetVal {}
global list regsSplitVal {}
global list regsPhaseVal {}
global emacOrgVal
global chip_num
global org_ump_en
global org_lldp_en

# Global register file
global rdfFile

# -----------------------------------------------------------------
# Disbale UMP 
# -----------------------------------------------------------------
proc disable_ump {} {
   global org_ump_en
   global org_lldp_en
    
   # Disable UMP to avoid failure incase NCSI is running
   set offset [reg_offset -n "MISC_REG_RESET_REG_1"]
   set startOffset [lindex [split $offset "-"] 0]
   set reset_0 [reg read $startOffset]
   if {[expr {$reset_0 & 0x80} ] == 0x80} {
       if {$::current(FUNC) & 0x01} {
          set org_ump_en [reg_read -n NIG_REG_EGRESS_UMP1_IN_EN]
          debug_print DEBUG "Disable NIG_REG_EGRESS_UMP1_IN_EN"
          reg_write -n NIG_REG_EGRESS_UMP1_IN_EN -v 0
          
          set org_lldp_en [reg_read -n NIG_REG_P1_LLH_MCP_MASK_C_LLDP]
          debug_print DEBUG "Disable NIG_REG_P1_LLH_MCP_MASK_C_LLDP"
          reg_write -n NIG_REG_P1_LLH_MCP_MASK_C_LLDP -v 0
       } else {
         
          set org_ump_en [reg_read -n NIG_REG_EGRESS_UMP0_IN_EN]
          debug_print DEBUG "Disable NIG_REG_EGRESS_UMP0_IN_EN"
          reg_write -n NIG_REG_EGRESS_UMP0_IN_EN -v 0
          
          set org_lldp_en [reg_read -n NIG_REG_P0_LLH_MCP_MASK_C_LLDP]
          debug_print DEBUG "Disable NIG_REG_P0_LLH_MCP_MASK_C_LLDP"
          reg_write -n NIG_REG_P0_LLH_MCP_MASK_C_LLDP -v 0
      }    
   }
}

# -----------------------------------------------------------------
# Enable UMP 
# -----------------------------------------------------------------
proc enable_ump {} {
   global org_ump_en
   global org_lldp_en
    
   if {$::current(FUNC) & 0x01} {
        debug_print DEBUG "Restoring orginal value of NIG_REG_EGRESS_UMP1_IN_EN"
        reg_write -n NIG_REG_EGRESS_UMP1_IN_EN -v $org_ump_en
        debug_print DEBUG "Restoring orginal value of NIG_REG_P1_LLH_MCP_MASK_C_LLDP"
        reg_write -n NIG_REG_P1_LLH_MCP_MASK_C_LLDP -v $org_lldp_en
   } else {       
        debug_print DEBUG "Restoring original value of NIG_REG_EGRESS_UMP0_IN_EN"
        reg_write -n NIG_REG_EGRESS_UMP0_IN_EN -v $org_ump_en
        debug_print DEBUG "Restoring orginal value of NIG_REG_P0_LLH_MCP_MASK_C_LLDP"
        reg_write -n NIG_REG_P0_LLH_MCP_MASK_C_LLDP -v $org_lldp_en
   }
    
}

# -----------------------------------------------------------------
# Save MSI/MSI-X configuration from IGU
# -----------------------------------------------------------------
proc save_msix_table {buf_addr len} {
    if {![is_e1x]} {
        set offset [get_start_offset [reg_offset -n "IGU_REG_MSIX_MEMORY"]]
    } else {
        set offset [get_start_offset [reg_offset -n "HC_REG_MAIN_MEMORY"]]
    }

    for {set i 0} {$i < $len} {set i [expr ($i + 4)]} {
        hmem write [add_to_addr $buf_addr $i] [reg iread [expr ($offset + $i)]]
    }
}

# -----------------------------------------------------------------
# Restore MSI/MSI-X configuration in IGU
# -----------------------------------------------------------------
proc restore_msix_table {buf_addr len} {
    if {![is_e1x]} {
        set offset [get_start_offset [reg_offset -n "IGU_REG_MSIX_MEMORY"]]
    } else {
        set offset [get_start_offset [reg_offset -n "HC_REG_MAIN_MEMORY"]]
    }

    for {set i 0} {$i < $len} {set i [expr ($i + 4)]} {
        reg iwrite [expr ($offset + $i)] [hmem read [add_to_addr $buf_addr $i]]
    }
}

# -----------------------------------------------------------------
# Help info on "reset_chip" command
# -----------------------------------------------------------------
proc reset_chip_help {args} {
    set help_info     "Reset the chip using the MISC block\n"
    append help_info  "Usage: reset_chip\n"    
    add_help reset_chip $help_info
}

# -----------------------------------------------------------------
# The proc resets all the blocks in the chip using a MISC reset register
# -----------------------------------------------------------------
proc reset_chip { {noMcpReset 0}} {
   global regsOffset
   global rdfFile
    
   #get RDF file name according to DID
   get_rdf_file          
   parse_rdf -f $rdfFile

   #halt mcp before resetting the chip, to validate we won't reset during DMA transaction
   mcp halt
   after 50    
        
   # Reset the chip
   set offset [reg_offset -n "MISC_REG_RESET_REG_1"]
   set startOffset [lindex [split $offset "-"] 0]
   #puts "reset_chip reset also PXP and PXPV"
   #reg write [expr ($startOffset+8)] 0xFFFFFFFF
   reg write [expr ($startOffset+8)] 0xF3FFFFFF
   
   set offset [reg_offset -n "MISC_REG_RESET_REG_2"]
   set startOffset [lindex [split $offset "-"] 0]
   if {[is_e3]} {
      # Note: XMAC0,1 are not reset (bits 22,23)      
      # ELINK will take these blocks out of reset
      # Per CQ61526 do not attempt to access BMAC/EMAC
      # reset bits in E3 (these blocks don't exist)
      reg write [expr ($startOffset+8)] 0x3301400      
   } else {
      reg write [expr ($startOffset+8)] 0x330140F
   }

   # Config the MISC block, remove chip from reset
   set offset [reg_offset -n "MISC_REG_RESET_REG_1"]
   set startOffset [get_start_offset $offset]
   reg write [expr ($startOffset+4)] 0xFFFFFFFF

   set offset [reg_offset -n "MISC_REG_RESET_REG_2"]
   set startOffset [get_start_offset $offset]
   if {[is_e3]} {
      # Per CQ61526 do not attempt to access BMAC/EMAC
      # reset bits in E3 (these blocks don't exist)
      reg write [expr ($startOffset+4)] 0x330FFF0
   } else {
      reg write [expr ($startOffset+4)] 0x330FFFF
   }
     
   # MCP should be reset after resetting the chip
   if {!$noMcpReset} {
      puts "Resetting MCP .."
      mcp reset
      after 300
   }
}

# -----------------------------------------------------------------
# The proc implements hard reset to the chip
# -----------------------------------------------------------------
proc hard_reset_chip {args} {
    global regsOffset
    global rdfFile
    
    #get RDF file name according to DID
    get_rdf_file  
    parse_rdf -f $rdfFile                
    
        
    # Hard-Reset the chip
    set offset [reg_offset -n "MISC_REG_IPOR_CMD_REG"]    
    reg write $offset 1    
}

# -----------------------------------------------------------------
# This proc parses the start offset out of an offset field
# (startOffset - endOffset)
# -----------------------------------------------------------------
proc get_start_offset {{offset}}  {
    
    # Parse out the starting offset
    return  [lindex [split $offset "-"] 0]                 
}


    
# -----------------------------------------------------------------
# Help info on "parse_rdf" command
# -----------------------------------------------------------------
proc parse_rdf_help {args} {
    set help_info     "Parse the registers file into global arrays e.g. regsOffset\n"
    append help_info  "Usage: parse_rdf -f <RDF file> -b <Block name>\n"
    append help_info  "optional:        -skip <Block to skip> -e < ending block>"
    add_help parse_rdf $help_info
}

# -----------------------------------------------------------------
# The proc parses the EVEREST RDF file which contains a list of
# [name, offset, access_type, count, reset_value] for each register.
# Saves the registers offset, dataWidth in regsInfo global array.
# -----------------------------------------------------------------
proc parse_rdf {args} {
    global regsName
    global regsOffset 
    global regsSize
    global regsCount
    global regsType   
    global regsResetVal
    global regsSplitVal
    global regsPhase    
    global skipBlocks
    global chip_num
    
    set skipBlocks {  MCP }
    set testBlock ""
    set rdfFile ""
    set doParsing 0
    set clrFlag   0
  
        
    # if no arguments, print the usage
    if {[llength $args] == 0} {
        parse_rdf_usage
        return -1
    }

    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {        
        switch -- [lindex $args $i] {
            "-f"         { incr i; set rdfFile [lindex $args $i] }
            "-skip"      { incr i; lappend skipBlocks [lindex $args $i] }
            "-b"         { incr i; set testBlock [lindex $args $i] }
            "-e"         { incr i; set endBlock [lindex $args $i] }
            "-clr"       { set clrFlag 1 }
            "-help"      { parse_rdf_usage; return -1 }
            default      { parse_rdf_usage; return -1 }
        }
    }
    
    # If clear flag was set, then clear the regsOffset array
    if { $clrFlag } {
         un_parse_rdf
    } else {
        set last_chip_num 0
        if {[info exists chip_num]} {
            set last_chip_num $chip_num
        }
        set chip_num [reg read 0xA408]
                
        if {$last_chip_num != $chip_num} {
            # if different chip, and already parsed rdf, then un_parse it.
            if {[info exists regsName]} {
               un_parse_rdf
            }
        } elseif { [llength $regsName] > 0} {
            #if RDF file allready parsed, return            
            return 0
        }
    }
    
    # Verify that the rdf file exists
    if { ! [file exist $rdfFile]} {
        puts "The $rdfFile does not exists"
        puts "Usage: parse_rdf -f <rdfFile>"
        return -1
    }
    
    puts "Parsing $rdfFile ..."
    # split the skipBlocks list into seperate blocks
    set skipBlocks [split $skipBlocks]
    
    # build the blocks table (block name, block offset)
    # from the rdf file.
    build_blocks_table $rdfFile    

    # Open the rdf file for reading
    set infile [open $rdfFile r]
        
    # Go through each line in the rdf file
    while { [eof $infile] != 1 } {
        set regName ""
        set splitVal 0
        set phase "port0"
        
        # parse the read line into fields
        set line [gets $infile]        
        scan $line %s%s%s%s%s%s%s%s%s regName offset accessType dataWidth count addrWidth resetVal splitVal phase        
                   
        #skip remark lines and empty lines
        if { [skip_line $regName]} {
              continue
        }
                     
        # Check if line includes only the block name,
        if { [string first ":" $regName] != -1 } {
            
            # Get the next block name and offset
            set blockName [lindex [split $regName ":"] 0]
            set blockOffset [block_offset -b $blockName]
            if {$blockOffset == -1} {
                puts "ERROR: block $blockName offset was not found !!"
                break
            }
            
            # If the -b option is set, check if the block name equals the
            # requested block to test.
            if { ($testBlock != "") && ($testBlock == $blockName)} {
                set doParsing 1
            # If the -b option is not set, check if we do not have to skip 
            # this block cause it is in the skipBlocks list.
            } elseif { ($testBlock == "") && ([lsearch -exact $skipBlocks $blockName] == -1)} {
                set doParsing 1
            # Need also the MISC block registers
            } elseif { $blockName == "MISC" } {
                set doParsing 1
            } else {
                set doParsing 0
            }
            
            # Skip the block name line.
            continue
        }             
        
        # If we need to skip this block, continue to next block
        if {$doParsing == 0} {
            continue
        }
                       
        # Check if the offset field is a range (startOffset - endOffset)
        # If so, parse the start offset and end offset.
        if { ([string first "-" $offset] != -1) } {
            set startOffset [lindex [split $offset "-"] 0]
            set endOffset   [lindex [split $offset "-"] 1]
            
        # Or several registers (count > 1)
        }  elseif { $count > 1 } {            
            # For WB regs, calc the reg reserved width (the upper 2^x)
            if {$dataWidth > 32} {
                set power [expr (log($dataWidth)/log(2))]
                # round up the power
                if {$power != int($power)} {
                    set power [expr (int($power))]
                    incr power                 
                }
                set width [expr (int(pow(2,$power)))]
                set noWords [expr ($width/32)]
            } else  {
                set noWords 1
            }
            set startOffset $offset
            set endOffset   [expr ($startOffset + (($count-1) * 4))]
        } else  {
        # Or a single register
                set startOffset $offset
                set endOffset   $startOffset
        }

        # Add to offsets the block's offset
        set startOffset [format "0\x%08x" [incr startOffset $blockOffset]]
        set endOffset   [format "0\x%08x" [incr endOffset $blockOffset]]                                    

        # Save the register's offset and dataWidth in regsInfo array
        if {$endOffset == $startOffset} {
            lappend regsOffset $startOffset         
        } else  {
            # incase of a range of offsets
            lappend regsOffset "$startOffset-$endOffset"            
        }        
        
        lappend regsName $regName        
        lappend regsSize $dataWidth
        lappend regsType $accessType
        lappend regsSplitVal $splitVal
        lappend regsPhase $phase
        lappend regsResetVal $resetVal
        lappend regsCount $count
    }
    close $infile
    puts "Completed parsing $rdfFile successfully"
    
    return 0
}

# -----------------------------------------------------------------
# This proc prints out the deletes the lists used by parse_rdf
# -----------------------------------------------------------------
proc un_parse_rdf {}  {
   global regsName
   global regsOffset 
   global regsSize
   global regsCount
   global regsType   
   global regsResetVal
   global regsSplitVal
   global regsPhase    
      
   set regsName {}   
   set regsOffset {}   
   set regsSize {}   
   set regsType {}   
   set regsSplitVal {}   
   set regsPhase {}   
   set regsResetVal {}   
   set regsCount {}  
    
}

# -----------------------------------------------------------------
# This proc prints out the parse_rdf proc usage information
# -----------------------------------------------------------------
proc parse_rdf_usage {}  {
    
    puts stdout "Usage:   parse_rdf -f     <rdf file name>"
    puts stdout "                   -skip  <skip blocks>  "
    puts stdout "                   -b     <test only this block>    "
    puts stdout "                   -e     <end block>    "
    puts stdout "                   -clr                  "
    
}

# -----------------------------------------------------------------
# This proc builds the blocks array: block name, block offset
# -----------------------------------------------------------------
proc build_blocks_table {filename}  {

    #use the global array blocksTable 
    global blocksTable
                    
    # Open the rdf file for reading
    set infile [open $filename r]

    # Go through the rdf file, until the first block name:
    while { [eof $infile] != 1 } {
        # Parse the first line into fields      
        set line [gets $infile]
        scan $line "%s%s" blockName blockOffset

        #skip the remark lines and empty lines
        if { [skip_line $blockName] } {
            continue
        }
 
        # When reaching the first block name: exit
        if {[string first ":" $blockName] != -1} {
           close $infile
           return
        }

        # Save the blockName and blockOffset in the blocksTable array
        set blocksTable($blockName) $blockOffset        
    }
     close $infile
}

# -----------------------------------------------------------------
# Help info on "block_offset" command
# -----------------------------------------------------------------
proc block_offset_help {args} {
       
    set help_info     "Returns the block's offset from BAR by its name\n"
    append help_info  "Usage: block_offset -b <blockname>\n"    
    add_help block_offset $help_info
}

# -----------------------------------------------------------------
# This proc returns the block's offset
# -----------------------------------------------------------------
proc block_offset {args}  {
    #use the global array blocksTable
    global blocksTable

    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: block_offset -b <blockName>"  
        return -1
    }   
    
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-b"      { incr i; set blockName [lindex $args $i] }
            "-help"   { puts "Usage: block_offset -b <blockName>"; return -1 }
            default   { puts "Usage: block_offset -b <blockName>"; return -1 }           
        }
    }
   
    # Search for the matching name in blocksTable
    foreach name [array names blocksTable] {
        # found the matching block in blocksTable, return its offset
        if {[string compare $name $blockName] == 0} {
          return $blocksTable($name)
        }
    }
    
    #if we got here, it means we did not find a matching block
    puts "The block $blockName does not exist in blocks table"
    return -1        
}

# -----------------------------------------------------------------
# This proc checks if we need to skip this line cause its a remark
# (starts with #) or an empty line
# -----------------------------------------------------------------
proc skip_line {regName} {
    if { [string equal -length 1 $regName "#"] == 1 } {
        #puts "skiping $regName"
        return 1;
    }
    if { [string length $regName] == 0 } {
        return 1;
    }

    return 0;
}

# -----------------------------------------------------------------
# Help info on "reg_read" command
# -----------------------------------------------------------------
proc reg_read_help {args} {    
    set help_info     "Reads a register's value by its name\n"
    append help_info  "Usage: reg_read -n <regName>\n"
    add_help reg_read $help_info
}

# -----------------------------------------------------------------
# This proc returns the register's read value (by its name)
# -----------------------------------------------------------------
proc reg_read {args} {

    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: reg_read -n <regName>"  
        return -1
    }
    
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"      { incr i; set regName [lindex $args $i] }
            "-help"   { puts "Usage: reg_read -n <regName>"; return -1 }
            default   { puts "Usage: reg_read -n <regName>"; return -1 }
        }
    }
    
    #get the register's offset
    set offset [reg_offset -n $regName]
    if {$offset != -1} {
      if {$::sys(NO_PCI) == 0} {      
        # if device is valid, perform direct reg access, if device is hidden perform it inderectly
        if {$::current(DID)}  {            
            return [reg dread $offset]
        } else {             
            return [reg iread $offset]    
        }
      } else {
         # in NO_PCI mode always perform direct reg access
         return [reg dread $offset]
      }      
    } else  {        
        return -1
    }
}

#-----------------------------------------------------------------
# This proc writes value to the register (by its name)
# -----------------------------------------------------------------
proc reg_write {args} {

    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: reg_write -n <regName> -v <value>"  
        return -1
    }
    
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"      { incr i; set regName [lindex $args $i] }
            "-v"      { incr i; set value [lindex $args $i] }
            "-help"   { puts "Usage: reg_write -n <regName> -v <value>"; return -1 }
            default   { puts "Usage: reg_write -n <regName> -v <value>"; return -1 }
        }
    }
    
    #get the register's offset
    set offset [reg_offset -n $regName]
    if {$offset != -1} {
        return [reg dwrite $offset $value]
    } else  {
        return -1
    }
}

# -----------------------------------------------------------------
# Help info on "reg_offset" command
# -----------------------------------------------------------------
proc reg_offset_help {args} {
    set help_info     "Get the register's offset address from BAR by its name\n"
    append help_info  "Usage: reg_offset -n <regName>\n"
    add_help reg_offset $help_info
}

# -----------------------------------------------------------------
# This proc returns the registers offset (by its name)
# -----------------------------------------------------------------
proc reg_offset {args} {
    #use the global array regsOffset
    global regsOffset
    global regsName

    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: reg_offset -n <regName>"  
        return -1
    }
    
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"      { incr i; set regName [lindex $args $i] }
            "-help"   { puts "Usage: reg_offset -n <regName>"; return -1 }
            default   { puts "Usage: reg_offset -n <regName>"; return -1 }
        }
    }     
    
    set idx [lsearch -exact $regsName $regName]
    return [lindex $regsOffset $idx]     
}

# -----------------------------------------------------------------
# Help info on "reg_size" command
# -----------------------------------------------------------------
proc reg_size_help {args} {
    set help_info     "Get the register's size (in bits) by its name\n"
    append help_info  "Usage: reg_size -n <regName>\n"
    add_help reg_size $help_info
}

# -----------------------------------------------------------------
# This proc returns the registers size (by its name)
# -----------------------------------------------------------------
proc reg_size {args} {
    #use the global array regsSize
    global regsSize
    global regsName

    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: reg_size -n <regName>"  
        return -1
    }
    
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"      { incr i; set regName [lindex $args $i] }
            "-help"   { puts "Usage: reg_size -n <regName>"; return -1 }
            default   { puts "Usage: reg_size -n <regName>"; return -1 }
        }
    }

    set idx [lsearch -exact $regsName $regName]
    return [lindex $regsSize $idx]                
}

# -----------------------------------------------------------------
# Help info on "reg_size" command
# -----------------------------------------------------------------
proc reg_count_help {args} {
    set help_info     "Get the register's count by its name (2^count is the total register's array size)\n"
    append help_info  "Usage: reg_count -n <regName>\n"
    add_help reg_count $help_info
}

# -----------------------------------------------------------------
# This proc returns the registers size (by its name)
# -----------------------------------------------------------------
proc reg_count {args} {
    #use the global array regsSize
    global regsCount
    global regsName

    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: reg_count -n <regName>"  
        return -1
    }
    
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"      { incr i; set regName [lindex $args $i] }
            "-help"   { puts "Usage: reg_count -n <regName>"; return -1 }
            default   { puts "Usage: reg_count -n <regName>"; return -1 }
        }
    }

    set idx [lsearch -exact $regsName $regName]
    return [lindex $regsCount $idx]     
              
}

# -----------------------------------------------------------------
# Help info on "reg_type" command
# -----------------------------------------------------------------
proc reg_type_help {args} {
    set help_info     "Get the register's access type (RW/R/ST/WB) by its name\n"
    append help_info  "Usage: reg_type -n <regName>\n"
    append help_info  "RW - read write register\n"
    append help_info  "R  - read only register\n"
    append help_info  "ST - statistics register, clears on second read\n"
    append help_info  "RC - read clears register, clears on second readn"
    append help_info  "WB - wide bus register (> 32 bit)\n"
    add_help reg_type $help_info
}

# -----------------------------------------------------------------
# This proc returns the registers type (by its name)
# -----------------------------------------------------------------
proc reg_type {args} {
    #use the global array regsType
    global regsType
    global regsName

    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: reg_type -n <regName>"  
        return -1
    }
        
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"      { incr i; set regName [lindex $args $i] }
            "-help"   { puts "Usage: reg_type <regName>"; return -1 }
            default   { puts "Usage: reg_type <regName>"; return -1 }
        }
    }    
     
    set idx [lsearch -exact $regsName $regName]
    return [lindex $regsType $idx] 
              
}

# -----------------------------------------------------------------
# Help info on "reset_value" command
# -----------------------------------------------------------------
proc reset_value_help {args} {
    set help_info     "Get the register's reset value by its name\n"
    append help_info  "Usage: reset_value -n <regName>\n"
    add_help reset_value $help_info
}

# -----------------------------------------------------------------
# This proc returns the registers type (by its name)
# -----------------------------------------------------------------
proc reset_value {args} {
    #use the global array regsType
    global regsResetVal
    global regsName

    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: reset_value -n <regName>"  
        return -1
    }
     
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"      { incr i; set regName [lindex $args $i] }
            "-help"   { puts "Usage: reset_value <regName>"; return -1 }
            default   { puts "Usage: reset_value <regName>"; return -1 }
        }
    }

    set idx [lsearch -exact $regsName $regName]
    return [lindex $regsResetVal $idx]           
}

# -----------------------------------------------------------------
# Help info on "split_value" command
# -----------------------------------------------------------------
proc split_value_help {args} {
    set help_info     "Get the register's split value by its name\n"
    append help_info  "Usage: split_value -n <regName>\n"
    add_help split_value $help_info
}

# -----------------------------------------------------------------
# This proc returns the registers type (by its name)
# -----------------------------------------------------------------
proc split_value {args} {
    
    global regsSplitVal
    global regsName

    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: split_value -n <regName>"  
        return -1
    }
     
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"      { incr i; set regName [lindex $args $i] }
            "-help"   { puts "Usage: split_value <regName>"; return -1 }
            default   { puts "Usage: split_value <regName>"; return -1 }
        }
    }    
    
    set idx [lsearch -exact $regsName $regName]
    return [lindex $regsSplitVal $idx] 
             
}

# -----------------------------------------------------------------
# Help info on "phase_value" command
# -----------------------------------------------------------------
proc phase_value_help {args} {
    set help_info     "Get the register's phase value by its name\n"
    append help_info  "Usage: phase_value -n <regName>\n"
    add_help phase_value $help_info
}

# -----------------------------------------------------------------
# This proc returns the registers type (by its name)
# -----------------------------------------------------------------
proc phase_value {args} {
    
    global regsPhase
    global regsName

    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: phase_value -n <regName>"  
        return -1
    }
     
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"      { incr i; set regName [lindex $args $i] }
            "-help"   { puts "Usage: phase_value <regName>"; return -1 }
            default   { puts "Usage: phase_value <regName>"; return -1 }
        }
    }    
    
    set idx [lsearch -exact $regsName $regName]
    return [lindex $regsPhase $idx]
             
}

# -----------------------------------------------------------------
# Help info on "reg_info" command
# -----------------------------------------------------------------
proc reg_info_help {args} {
    set help_info     "Get the register's information (offset, size, type) by its name\n"
    append help_info  "Usage: reg_info -n <regName>\n"
    add_help reg_info $help_info
}

# -----------------------------------------------------------------
# This proc returns all the registers information (by its name)
# -----------------------------------------------------------------
proc reg_info {args} {
    # if no arguments, print the usage
    if {[llength $args] == 0} {
        puts "Usage: reg_info -n <regName>"  
        return -1
    }
    
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"      { incr i; set regName [lindex $args $i] }
            "-help"   { puts "Usage: reg_info -n <regName>"; return -1 }
            default   { puts "Usage: reg_info -n <regName>"; return -1 }
        }
    }
    
    puts "offset: [reg_offset -n $regName] size: [reg_size -n $regName] type: [reg_type -n $regName]"
}



# -----------------------------------------------------------------
# Help info on "read_regs" command
# -----------------------------------------------------------------
proc read_regs_help {args} {
    set help_info     "Read all the registers values and print them into a file\n"
    append help_info  "Usage: read_regs -out <outfile>"
    append help_info  "optional:        -all -b <blockName> -noPrint\n\n"
    append help_info  "Notes: \n"
    append help_info  "-all reads all registers\n"
    append help_info  "-b <blockname> reads only this block registers\n"
    append help_info  "-noPrint does not print any extra print-outs\n"    
    add_help read_regs $help_info
}


# -----------------------------------------------------------------
# This proc checks if block is in reset 
# -----------------------------------------------------------------
proc is_block_in_reset {blockName} {
    # Read the MISC Reset register 1
    set offset [reg_offset -n "MISC_REG_RESET_REG_1"]
    set startOffset [lindex [split $offset "-"] 0]    
    set resetReg1 [reg read $startOffset]
            
    # Read the MISC Reset register 2
    set offset [reg_offset -n "MISC_REG_RESET_REG_2"]
    set startOffset [lindex [split $offset "-"] 0]    
    set resetReg2 [reg read $startOffset]
                  
    switch $blockName {
        "BRB1"  { if {[expr ($resetReg1 & 0x1)] != 0x1} {return 1} }
        "PRS"   { if {[expr ($resetReg1 & 0x2)] != 0x2} {return 1} }
        "SRC"   { if {[expr ($resetReg1 & 0x4)] != 0x4} {return 1} }
        "TSDM"  { if {[expr ($resetReg1 & 0x8)] != 0x8} {return 1} }
        "TSEM"  { if {[expr ($resetReg1 & 0x10)] != 0x10} {return 1} }
        "TCM"   { if {[expr ($resetReg1 & 0x20)] != 0x20} {return 1} }
        "RBCR"  { if {[expr ($resetReg1 & 0x40)] != 0x40} {return 1} }
        "NIG"   { if {[expr ($resetReg1 & 0x80)] != 0x80} {return 1} }
        "USDM"  { if {[expr ($resetReg1 & 0x100)] != 0x100} {return 1} }
        "UCM"   { if {[expr ($resetReg1 & 0x200)] != 0x200} {return 1} }
        "USEM"  { if {[expr ($resetReg1 & 0x400)] != 0x400} {return 1} }        
        "UPB"   { if {[expr ($resetReg1 & 0x800)] != 0x800} {return 1} }
        "CCM"   { if {[expr ($resetReg1 & 0x1000)] != 0x1000} {return 1} }        
        "CSEM"  { if {[expr ($resetReg1 & 0x2000)] != 0x2000} {return 1} }
        "CSDM"  { if {[expr ($resetReg1 & 0x4000)] != 0x4000} {return 1} }            
        "RBCU"  { if {[expr ($resetReg1 & 0x8000)] != 0x8000} {return 1} }
        "PBF"   { if {[expr ($resetReg1 & 0x10000)] != 0x10000} {return 1} }
        "QM"    { if {[expr ($resetReg1 & 0x20000)] != 0x20000} {return 1} }
        "TM"    { if {[expr ($resetReg1 & 0x40000)] != 0x40000} {return 1} }
        "DORQ"  { if {[expr ($resetReg1 & 0x80000)] != 0x80000} {return 1} }
        "XCM"   { if {[expr ($resetReg1 & 0x100000)] != 0x100000} {return 1} }
        "XSDM"  { if {[expr ($resetReg1 & 0x200000)] != 0x200000} {return 1} }
        "XSEM"  { if {[expr ($resetReg1 & 0x400000)] != 0x400000} {return 1} }
        "RBCT"  { if {[expr ($resetReg1 & 0x800000)] != 0x800000} {return 1} }       
        "CDU"   { if {[expr ($resetReg1 & 0x1000000)] != 0x1000000} {return 1} }
        "CFC"   { if {[expr ($resetReg1 & 0x2000000)] != 0x2000000} {return 1} }
        "PXP"   { if {[expr ($resetReg1 & 0x4000000)] != 0x4000000} {return 1} }
        "PXPV"  { if {[expr ($resetReg1 & 0x8000000)] != 0x8000000} {return 1} }
        "RBCP"  { if {[expr ($resetReg1 & 0x10000000)] != 0x10000000} {return 1} }
        "HC"    { if {[expr ($resetReg1 & 0x20000000)] != 0x20000000} {return 1} }
        "DMAE"  { if {[expr ($resetReg1 & 0x40000000)] != 0x40000000} {return 1} }
        "BMAC"  { if {[expr ($resetReg2 & 0x3)] != 0x3} {return 1} }
        "EMAC"  { if {[expr ($resetReg2 & 0xC)] != 0xC} {return 1} }
        "UMAC0"  { if {[expr ($resetReg2 & 0x100000)] != 0x100000} {return 1} }
        "UMAC1"  { if {[expr ($resetReg2 & 0x200000)] != 0x200000} {return 1} }
        "XMAC0"  { if {[expr ($resetReg2 & 0x400000)] != 0x400000} {return 1} }
        "XMAC1"  { if {[expr ($resetReg2 & 0x800000)] != 0x800000} {return 1} }
        "MSTAT0"  { if {[expr ($resetReg2 & 0x1000000)] != 0x1000000} {return 1} }
        "MSTAT1"  { if {[expr ($resetReg2 & 0x2000000)] != 0x2000000} {return 1} } 
    }
    
    return 0
}    
# -----------------------------------------------------------------
# This proc reads all the registers of the RDF file and prints
# the register's name, offset, size, type and read value in an output file.
# The outfput file name is the same as the RDF file with the extention
# ".out" instead of ".rdf"
# -----------------------------------------------------------------
proc read_regs {args} {
    
    set outfileName ""
    set testBlock ""
    set noPrint 0
    set did 0
    set splitRegs 0
     
    # use globals
    global regsOffset
    global regsName
    global rdfFile
    
    set exceptionReadList [list DORQ_REG_DQ_FIFO_FULL_CLR         NIG_REG_DEBUG_PACKET_LB           NIG_REG_DEBUG_PACKET_PORT0      NIG_REG_DEBUG_PACKET_PORT1 \
                                NIG_REG_INGRESS_EOP_PORT0_FIFO    NIG_REG_INGRESS_EOP_PORT1_FIFO    NIG_REG_INGRESS_EOP_LB_FIFO      \
                                NIG_REG_INGRESS_BMAC0_MEM         NIG_REG_INGRESS_BMAC1_MEM         CFC_REG_STRING_CAM              CFC_REG_LINK_LIST \
                                PXP2_REG_PGL_EXP_ROM0             PXP2_REG_PGL_EXP_ROM1             PXP2_REG_PGL_EXP_ROM2           PXP2_REG_RQ_DRAM_ALIGN_RD \
                                HC_REG_COMMAND_REG                HC_REG_PBA_COMMAND                HC_REG_MAIN_MEMORY              HC_REG_STATISTIC_COUNTERS \
                                CCM_REG_CP_MSG                    TCM_REG_CP_MSG                    UCM_REG_CP_MSG                  XCM_REG_CP_MSG \
                                CSDM_REG_CM_QUEUE                 TSDM_REG_CM_QUEUE                 USDM_REG_CM_QUEUE               XSDM_REG_CM_QUEUE \
                                CSDM_REG_INP_QUEUE                USDM_REG_INP_QUEUE                XSDM_REG_INP_QUEUE              TSDM_REG_INP_QUEUE \
                                CSDM_REG_OPERATION_GEN            TSDM_REG_OPERATION_GEN            USDM_REG_OPERATION_GEN          XSDM_REG_OPERATION_GEN \
                                CSDM_REG_TIMERS                   TSDM_REG_TIMERS                   USDM_REG_TIMERS                 XSDM_REG_TIMERS \
                                CSDM_REG_TIMERS_ADDR_EMPTY        TSDM_REG_TIMERS_ADDR_EMPTY        USDM_REG_TIMERS_ADDR_EMPTY      XSDM_REG_TIMERS_ADDR_EMPTY \
                                CSDM_REG_TIMERS_ADDR_FULL         TSDM_REG_TIMERS_ADDR_FULL         USDM_REG_TIMERS_ADDR_FULL       XSDM_REG_TIMERS_ADDR_FULL \
                                ATC_REG_ATC_SPA_ARRAY_ACCESS      ATC_REG_ATC_GPA_ARRAY_ACCESS_STATE ATC_REG_ATC_GPA_ARRAY_ACCESS_W3 NIG_REG_INGRESS_MNG0_FIFO \
                                ATC_REG_ATC_GPA_ARRAY_ACCESS_W2   ATC_REG_ATC_GPA_ARRAY_ACCESS_W1   ATC_REG_ATC_GPA_ARRAY_ACCESS_W0  NIG_REG_INGRESS_MNG1_FIFO \
                                CSEM_REG_SEM_FAST_VFC_DATA_RD     XSEM_REG_SEM_FAST_VFC_DATA_RD     TSEM_REG_SEM_FAST_VFC_DATA_RD   USEM_REG_SEM_FAST_VFC_DATA_RD \
                                CSEM_REG_FIC0_FIFO                TSEM_REG_FIC0_FIFO                USEM_REG_FIC0_FIFO              XSEM_REG_FIC0_FIFO \
                                CSEM_REG_FIC1_FIFO                USEM_REG_FIC1_FIFO                XSEM_REG_FIC1_FIFO              TSEM_REG_FIC1_FIFO \
                                CSEM_REG_FIN_FIFO                 TSEM_REG_FIN_FIFO                 USEM_REG_FIN_FIFO               XSEM_REG_FIN_FIFO \
                                CSEM_REG_WB_MSB                   TSEM_REG_WB_MSB                   USEM_REG_WB_MSB                 XSEM_REG_WB_MSB \
                                CSEM_REG_SEM_FAST_PAS_REG_SET0    TSEM_REG_SEM_FAST_PAS_REG_SET0    USEM_REG_SEM_FAST_PAS_REG_SET0  XSEM_REG_SEM_FAST_PAS_REG_SET0 \
                                CSEM_REG_SEM_FAST_PAS_REG_SET1    TSEM_REG_SEM_FAST_PAS_REG_SET1    USEM_REG_SEM_FAST_PAS_REG_SET1  XSEM_REG_SEM_FAST_PAS_REG_SET1 \
                                CSEM_REG_VFC_SW_RST               TSEM_REG_VFC_SW_RST               USEM_REG_VFC_SW_RST             XSEM_REG_VFC_SW_RST \
                                IGU_REG_ERROR_HANDLING_DATA_VALID IGU_REG_ERROR_HANDLING_MEMORY \
				MISC_REG_SHIFT1_CMD               MISC_REG_SHIFT1_DATA              MISC_REG_SHIFT2_CMD             MISC_REG_SHIFT2_DATA ]
   
   
    # if no arguments, print the usage
    if {[llength $args] == 0} {
        read_regs_usage  
        return -1
    }
    
    
        
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {                
                "-b"         { incr i; set testBlock [lindex $args $i]}
                "-out"       { incr i; set outfileName [lindex $args $i]}
                "-silent"    { set noPrint 1}
                "-split_regs" { set splitRegs 1} 
                "-help"      { read_regs_usage; return -1 }
                default      { read_regs_usage; return -1 }
         }
    }   
  
    
    # Open the output file
    if {$outfileName != ""} {        
        set outfile [open "$outfileName" a]
    }
    
    
    get_rdf_file
    # Parse the appropriate RDF file
    parse_rdf -f $rdfFile
    
    # If block in reset print error and return 1
    if { $testBlock != "" } {           
        if {[is_block_in_reset $testBlock] == 1} {
            puts "ERROR: Can not read from block $testBlock. Block is in reset"
            # Close the output file
            if {$outfileName != ""} {                
                close $outfile
            }
            return 1
        }
    }
        
    set tmpList [lsort $regsName]
    
    # Go through each register in the regsOffset array   
    for {set idx 0} {$idx < [llength $regsName]} {incr idx} {
        
        set name [lindex $tmpList $idx]                
        
        # Set the "testBlock_REG" temporary string
        set tmpStr $testBlock
        append tmpStr "_REG*"       
      
        
        # If the -b option is set, skip the register if its not in the tested block
        if { ($testBlock != "") && ![string match $tmpStr $name]} {
            continue    
        }                     
               
        # skip registers that are in the exceptionReadList
        if { [lsearch -exact $exceptionReadList $name] != -1 } {
            continue;
        }
                
        # skip special registers
        if { [ string match "*UNUSED*" $name ]} {
            continue;
        }       
        if { [ string match "*INT_STS*" $name ]} {
            continue;
        }
        if { [ string match "*INT_STS_CLR*" $name ]} {
            continue;
        }
        if { [ string match "*INT_STS_WR*" $name ]} {
            continue;
        }
        if { [ string match "*PRTY_STS*" $name ]} {
            continue;
        }         
        if { [ string match "*PRTY_MASK*" $name ]} {
            continue;
        }
        if { [ string match "*INT_MASK*" $name ]} {
            continue;
        }
        if {[ string match "PBF_REG_EVEN_TXP_MEM_*" $name ]} {
            continue;
        }
    
        if {[string match "PBF_REG_ODD_TXP_MEM_*" $name ]} {
            continue;
        }    
	
	# incase of E3 4-domain emulation skip DBU and MSTAT1 blocks
        # skip the relevant MAC block according to BOND_ID register
        if { [is_e3]  && [is_sim] } {
            set val [reg_read -n MISC_REG_BOND_ID]
            # incase Bit#2 or Bit#3 is set its a 4-domain compile
            if {([expr {$val & 0x4}] == 0x4) || ([expr {$val & 0x8}] == 0x8)} {
               if {[string match "DBU_REG*" $name] || [string match "MSTAT1_REG*" $name]} {
                  continue;
               }
            }          
            # incase Bit#2 is set, there is no XMAC block
            if {[expr {$val & 0x4}] == 0x4} {
               if {[string match "XMAC0_REG*" $name] || [string match "XMAC1_REG*" $name]} {
                  continue;
               }
            }
            # incase Bit#3 is set, there is no UMAC block
            if {[expr {$val & 0x8}] == 0x8} {
               if {[string match "UMAC0_REG*" $name] || [string match "UMAC1_REG*" $name]} {
                  continue;
               }
            }           
        }               
    
        set offset [reg_offset -n $name]
        
        # if offset includes a range, read only the startOffset
        if { ([string first "-" $offset] != -1) } {
            set offset [lindex [split $offset "-"] 0]
        }
                
        # Get the register's information (size, type)
        set size [reg_size -n $name]
        set type [reg_type -n $name]
        set dataWidth  [reg_size -n $name]
        set splitVal [split_value -n $name]
        
        # Incase user requests to show only split registers
        # skip this register if it's splitVal is 0
        if {$splitRegs && ($splitVal==0)} {
            continue
        }
        
        # For WB regs, calc the reg reserved width (the upper 2^x)
        if {$dataWidth > 32} {
            set power [expr (log($dataWidth)/log(2))]
            #round up the power value
            if { $power != [expr (int($power))]} {
                set power [expr (int($power))]
                incr power
            }
            set width [expr (int(pow(2,$power)))]
            set noWords [expr ($width/32)]
        } else  {
            set noWords 1
        }
        
        # Incase of WB go through all its words (32bit each)
        for {set i 0} {$i < $noWords} {incr i} {
            set wordOffset [expr ($offset + $i*4)]
            set value [reg dread $wordOffset]              
        
            # print the registers name, information and read value to output file
            if {$outfileName != ""} {                
                puts $outfile "$name \t[format "0x%x" $wordOffset] \t$size \t$type \t$value"
            }
        
            # print information to screen also if required
            if {$noPrint != 1} {
                puts "$name \t[format "0x%x" $wordOffset] \t$size \t$type \t$value"            
            }
        }        
    }
   
    if {$outfileName != ""} {        
        close $outfile        
        return 0
    }
}

# -----------------------------------------------------------------
# This proc prints out the read_regs proc usage information
# -----------------------------------------------------------------
proc read_regs_usage {}  {
    
    puts stdout "Usage:   read_regs "
    puts stdout "                   -b <blockName>      :read only this block registers"
    puts stdout "                   -silent             :dont print to screen"    
    puts stdout "                   -out <filename>     :output to this file"
    
}

# -----------------------------------------------------------------
# This procedure performs read_regs on all blocks
# -----------------------------------------------------------------
proc read_regs_all {{args}} {
        global blocksTable
        global regsOffset
        global rdfFile
        
        set silent 0        
        set did 0
        set outfileName ""
        set splitRegs 0
        
        if {[info exists ::sys(NO_PCI)]} {
            set read_via_no_pci $::sys(NO_PCI)
        } else {
            set read_via_no_pci [regexp {NO_PCI} $::tcl_prompt1]
        }                                    
        
        # Parse the options.
        for {set i 0} {$i < [llength $args]} {incr i} {
            switch -- [lindex $args $i] {
                "-out"       { incr i; set outfileName [lindex $args $i]}
                "-silent"    { set silent 1}
                "-split_regs"    { set splitRegs 1}
                "-help"      { read_regs_all_usage; return -1 }                
                default      { read_regs_all_usage; return -1 }
            }
        }        
        
        
        
        #get RDF file name according to DID
        get_rdf_file
        # Parse the appropriate RDF file
        parse_rdf -f $rdfFile                 
                
        foreach blockName [lsort [array names blocksTable]] {
                # Skip these blocks from testing                
                set skipBlocks { AEU DBG MCP PXPCS EMAC0 EMAC1}

                if { [lsearch -exact $skipBlocks $blockName] != -1 } {
                        continue
                }                                        
                if {$silent} {
                     if {$splitRegs} {
                        read_regs -b $blockName -out $outfileName -silent -split_regs
                     } else {
                        read_regs -b $blockName -out $outfileName -silent
                     }
                } else {
                     if {$splitRegs} {
                        read_regs -b $blockName -out $outfileName -split_regs
                     } else {
                        read_regs -b $blockName -out $outfileName
                     }
                }
                
        }
}

# -----------------------------------------------------------------
# This proc prints out the read_regs proc usage information
# -----------------------------------------------------------------
proc read_regs_all_usage {}  {
    
    puts stdout "Usage:   read_regs_all                 :read all registers"    
    puts stdout "                   -silent             :dont print to screen"
    puts stdout "                   -split_regs         :read only split registers" 
    puts stdout "                   -out <filename>     :output to this file"
    
}

# -----------------------------------------------------------------
# Checks if running on FPGA or emulation
# -----------------------------------------------------------------
proc is_sim {} {
    global rdfFile
    set isSim 0

    #get RDF file name according to DID
    get_rdf_file
    # Parse the appropriate RDF file
    parse_rdf -f $rdfFile
               
    set chipRev [reg_read -n "MISC_REG_CHIP_REV"]
    
    if {$chipRev > 5} {
        set isSim 1
    } 
    
    return $isSim
}

# -----------------------------------------------------------------
# Checks if running on FPGA
# -----------------------------------------------------------------
proc is_fpga {} {
    global rdfFile
    set isFpga 0

    #get RDF file name according to DID
    get_rdf_file
    # Parse the appropriate RDF file
    parse_rdf -f $rdfFile
               
    set chipRev [reg_read -n "MISC_REG_CHIP_REV"]
    
    if {[is_sim] && (($chipRev & 0x1) == 0x1)} {
        set isFpga 1
    } 
    
    return $isFpga
}

# -----------------------------------------------------------------
# Checks if running on Emulation
# -----------------------------------------------------------------
proc is_emul {} {
    global rdfFile
    set isEmul 0

    #get RDF file name according to DID
    get_rdf_file
    # Parse the appropriate RDF file
    parse_rdf -f $rdfFile
               
    set chipRev [reg_read -n "MISC_REG_CHIP_REV"]
    
    # if simulation and bit 0 is 0
    if {([is_sim] && (($chipRev & 0x1) == 0x0))} {
        set isEmul 1
    } 
    
    return $isEmul
}



# -----------------------------------------------------------------
# Checks if running on FPGA
# -----------------------------------------------------------------
proc chip_rev {} {
    global regsOffset
    global rdfFile
    
    #get RDF file name according to DID
    get_rdf_file
    # Parse the appropriate RDF file
    parse_rdf -f $rdfFile
               
    set chipRev [reg_read -n "MISC_REG_CHIP_REV"]  
        
    return $chipRev
}


# -----------------------------------------------------------------
# Set the RX strip CRC ON (value=1) or OFF (value=0)
# -----------------------------------------------------------------
proc rx_strip_crc {port is10Gmac value} {
    global regsOffset
    global rdfFile
     
    set NIG_REG_INGRESS_BMAC0_MEM   0x10C00
    set NIG_REG_INGRESS_BMAC1_MEM   0x11000
    
    set chip_num [reg_read -n MISC_REG_CHIP_NUM]
    
    # for E2 we need to use BMAC2
    # for E3 do not set this register
    if {[is_e2]} {
      set BIGMAC_REGISTER_RX_CONTROL	  [expr  (0x3A<<3)]	
    } elseif {![is_e3]} {      
      set BIGMAC_REGISTER_RX_CONTROL	  [expr (0x21<<3)]	
    }
   
    if {($value != 0) && ($value != 1)} {
        puts "rx_strip_crc: value should be 0 or 1" 
        return 1
    }
    
    
    #get RDF file name according to DID
    get_rdf_file
    # Parse the appropriate RDF file
    parse_rdf -f $rdfFile
               
    
    if {$is10Gmac} {
        # Implement rxStripCRc in BMAC
        if {$port%2 == 0} {
            set bmacAddr $NIG_REG_INGRESS_BMAC0_MEM        
        } elseif {$port%2 == 1} {
            set bmacAddr $NIG_REG_INGRESS_BMAC1_MEM
        } else {
            puts "rx_strip_crc: Unknown port number"            
            return 1
        }
        
        # Read the original value of RX control register
        if {![is_e3]} {
            set addr [expr ($bmacAddr+$BIGMAC_REGISTER_RX_CONTROL)]
            set orgValue1 [reg read $addr]
            # Register is WB, thus requires two reads
            set orgValue2 [reg read [expr ($addr+4)]]
            
            if {$value} {
                # Set the StripCrc bit 2 
                reg write $addr [expr $orgValue1 | 0x4]            
            } else {
                # Clear the StripCrc bit 2 
                reg write $addr [expr $orgValue1 & 0xFFFFFFFB]            
            }
            # Register is WB, thus requires two writes
            reg write [expr ($addr+4)] [expr $orgValue2]
        } else {
            set orgValue1 [reg_read -n XMAC0_REG_RX_CTRL]
            set orgValue2 [reg_read -n XMAC1_REG_RX_CTRL]
            if {$value} {
               if {$port} {
                  reg_write -n XMAC1_REG_RX_CTRL -v [expr {$orgValue2 | 0x4}]                                 
               } else {                  
                  reg_write -n XMAC0_REG_RX_CTRL -v [expr {$orgValue1 | 0x4}]
               }               
            } else {
               if {$port} {                  
                  reg_write -n XMAC1_REG_RX_CTRL -v [expr {$orgValue2 & ~0x4}]                  
               } else {
                  reg_write -n XMAC0_REG_RX_CTRL -v [expr {$orgValue1 & ~0x4}]
               }

            }
            
        }
        
    } else {    
         # Implementation for EMAC
         if {![is_e3]} {
            if {$port%2 == 0} {
               set addr [reg_offset -n "NIG_REG_NIG_INGRESS_EMAC0_NO_CRC"]
            } elseif {$port%2 == 1} {
               set addr [reg_offset -n "NIG_REG_NIG_INGRESS_EMAC1_NO_CRC"]
            } else {
               puts "rx_strip_crc: Unknown port number"            
               return 1
            }
            # Set the nig register no_crc according to the rxStripCRc value
            reg write $addr $value   
         } else {
            # In E3, implementation for UMAC0,1
            if {$port%2 == 0} {
               set orgValue [reg_read -n UMAC0_REG_COMMAND_CONFIG]                         
               if {$value} {
                  # Clear the ForwardCrc bit 6 
                  reg_write -n UMAC0_REG_COMMAND_CONFIG -v [expr {$orgValue & 0xFFFFFFBF}]             
               } else {
                  # Set the ForwardCrc bit 6 
                  reg_write -n UMAC0_REG_COMMAND_CONFIG -v [expr {$orgValue | 0x40}]           
               }
            } elseif {$port%2 == 1} {
               set orgValue [reg_read -n UMAC1_REG_COMMAND_CONFIG]                         
               if {$value} {
                  # Clear the ForwardCrc bit 6 
                  reg_write -n UMAC1_REG_COMMAND_CONFIG -v [expr {$orgValue & 0xFFFFFFBF}]             
               } else {
                  # Set the ForwardCrc bit 6 
                  reg_write -n UMAC1_REG_COMMAND_CONFIG -v [expr {$orgValue | 0x40}]           
               }
            } else {
               puts "rx_strip_crc: Unknown port number"            
               return 1
            }   
         }           
   }  
    
    return 0
}

# -----------------------------------------------------------------
# Set the TX append CRC ON (value=1) or OFF (value=0)
# -----------------------------------------------------------------
proc tx_append_crc {port is10Gmac value} {
    global regsOffset
    global rdfFile
     
    set NIG_REG_INGRESS_BMAC0_MEM   0x10C00
    set NIG_REG_INGRESS_BMAC1_MEM   0x11000
    
    set chip_num [reg_read -n MISC_REG_CHIP_NUM]
    
    # for E2 we need to use BMAC2
    if {[is_e2]} {
	set BIGMAC_REGISTER_TX_CONTROL	  [expr (0x1C<<3)]	
    } else {
        set BIGMAC_REGISTER_TX_CONTROL	  [expr (0x7<<3)]
    }    
   
    if {($value != 0) && ($value != 1)} {
        puts "txAppendCrc: value should be 0 or 1" 
        return 1
    }
    
    #get RDF file name according to DID
    get_rdf_file
    # Parse the appropriate RDF file
    parse_rdf -f $rdfFile
              
    
    if {$is10Gmac} {
        # Implement txAppendCrc in BMAC    
        if {$port == 0} {
            set bmacAddr $NIG_REG_INGRESS_BMAC0_MEM        
        } elseif {$port == 1} {
            set bmacAddr $NIG_REG_INGRESS_BMAC1_MEM
        } else {
            puts "txAppendCrc: Unknown port number"            
            return 1
        }
        
        # Read the original value of TX control register
        set addr [expr ($bmacAddr+$BIGMAC_REGISTER_TX_CONTROL)]
        set orgValue1 [reg read $addr]
        # Register is WB, thus requirs two reads
        set orgValue2 [reg read [expr ($addr+4)]]
        
        if {$value} {
            # Set the txCRCmode bits 3,2 to 00 CRCAPPEND mode
            reg write $addr [expr $orgValue1 & 0xFFFFFFF3]
        } else {
            # Set the txCRCmode bits 3,2 to 01 CRCKEEP mode
            reg write $addr [expr $orgValue1 & 0xFFFFFFF7]
            reg write $addr [expr $orgValue1 | 0x4]
        }
        #Register is WB, this requires two writes
        reg write [expr ($addr+4)] $orgValue2
    } else {          
        # Implementation for EMAC
        if {$port == 0} {
            set addr [reg_offset -n "NIG_REG_NIG_EGRESS_EMAC0_NO_CRC"]
        } elseif {$port == 1} {
            set addr [reg_offset -n "NIG_REG_NIG_EGRESS_EMAC1_NO_CRC"]
        } else {
            puts "txAppendCrc: Unknown port number"            
            return 1
        }        
    
        # If requested tx_append_crc value is 0, then set the EGRESS_EMAC0_NO_CRC to 1
        if {$value == 0 } {        
            reg write $addr 1
        } else {
            reg write $addr 0
        }
    }
    return 0
}

# -----------------------------------------------------------------
# This proc checks if we need to skip this register
# cause it's offset is in the exception list.
# -----------------------------------------------------------------
proc skip_register {name} {
    global exceptionList
        
    # skip registers that are in the exception list
    if { [lsearch -exact $exceptionList $name] != -1 } {
        return 1;
    }
    
    # skip registers named "UNUSED*"
    if { [ string match "*UNUSED*" $name ]} {
        return 1;
    } 
          
    # These registers do not have read access (advise by Igor)
    if { [ string match "*CM_REG_CP_MSG" $name ]} {
        return 1;
    }
    
    if { [ string match "*INT_STS*" $name ]} {
        return 1;
    }

    if { [ string match "*PRTY_STS*" $name ]} {
        return 1;
    }

    
    if { [ string match "*FIN_COMMAND" $name ]} {
        return 1;
    }   
   
    # Skip this registers cause reset value changes if read after
    # passive_buffer, used only for debugging (advised by Irena)
    if { [ string match "*SEM_REG_WB_MSB" $name ]} {
        return 1;
    }     

    # Skip this registers as their value changes when accessing other
    # registers, used only for debugging (advised by Irena)
    if { [ string match "*SDM_REG_TIMERS_ADDR_EMPTY" $name ]} {
        return 1;
    }
    
    if { [ string match "*SDM_REG_TIMERS_ADDR_FULL" $name ]} {
        return 1;
    }
    
    
    
    # Skip this register as it resets internally when activated (according to Igor)
    if { [string match "DMAE_REG_GO_C*" $name ]} {
        return 1;
    }
    
    # Skip this registers cause they are outside the NIG
    if {[string match "NIG_REG_XGXS0_STATUS_SPEED_*" $name ]} {
        return 1;
    }
        
    if {[string match "NIG_REG_XGXS1_STATUS_SPEED_*" $name ]} {
        return 1;
    }
    
    if {[string match "NIG_REG_SERDES0_STATUS_SPEED_*" $name ]} {
        return 1;    
    }
    
    if {[string match "NIG_REG_SERDES1_STATUS_SPEED_*" $name ]} {
        return 1;
    }
    
    # Skip these registers cause soft_reset does not reset them
    if {[string match "BRB1_REG_NUM_OF_FULL_CYCLES_*" $name ]} {
        return 1;
    }
    
    # Skip these MISC special registers
    if {[string match "MISC_REG_RESET_REG_*" $name ]} {
        return 1;
    }
    
    if {[string match "MISC_REG_DRIVER_CONTROL_*" $name ]} {
        return 1;
    }
    
    if {[string match "MISC_REG_PLL_MAIN_CTRL_*" $name ]} {
        return 1;
    }
    
    if {[string match "MISC_REG_PLL_MAIN_OTP_*" $name ]} {
        return 1;
    }
    
    if {[string match "MISC_REG_*_PLL_LOCK" $name ]} {
        return 1;
    }
    
    if {[string match "MISC_REG_AEU_AFTER_INVERT_*" $name ]} {
        return 1;
    }
    
    # Skip these register cause they can not be checked after XX_TABLE registers
    if {[string match "*_REG_LL_SZ_STICKY" $name ]} {
        return 1;
    }
    
    # Skip PGLUE_B error registers per CQ72256
    if {[string match "PGLUE_B_REG_TX_ERR_*" $name ]} {
        return 1;
    }
        
    return 0;
}

# -----------------------------------------------------------------
# This proc builds the exception list which contains the registers
# which should be skipped during testing.
# -----------------------------------------------------------------
proc build_exception_list {}  {
    global exceptionList
    
    # The registers we need to skip:
    
    lappend exceptionList BRB1_REG_SOFT_RESET
    
    # Skipped since this register can not be written twice
    lappend exceptionList BRB1_REG_FREE_LIST_PRS_CRDT
    
    # Skipped because mismatch in RAM size in RDF file
    lappend exceptionList BRB1_REG_LL_RAM
    
    # Skip cause different reset value in everest1.5
    #lappend exceptionList BRB1_REG_HEADER_SIZE 

    
    # Skip these configuration registers
    lappend exceptionList NIG_REG_BMAC0_REGS_OUT_EN
    lappend exceptionList NIG_REG_BMAC1_REGS_OUT_EN

    # Skip these WO registers  (advised by Irena)
    lappend exceptionList NIG_REG_DEBUG_PACKET_LB
    lappend exceptionList NIG_REG_DEBUG_PACKET_PORT0
    lappend exceptionList NIG_REG_DEBUG_PACKET_PORT1

    # Skip these memories since they are fifos (can access only offset 0)
    lappend exceptionList NIG_REG_INGRESS_EOP_PORT0_FIFO
    lappend exceptionList NIG_REG_INGRESS_EOP_PORT1_FIFO
    lappend exceptionList NIG_REG_INGRESS_EOP_LB_FIFO

        
    # Skip these registers cause there are outside the NIG
    lappend exceptionList NIG_REG_SERDES0_STATUS_TXPLL_LOCK
    lappend exceptionList NIG_REG_SERDES0_STATUS_MAC_COL
    lappend exceptionList NIG_REG_XGXS0_STATUS_TXPLL_LOCK
    lappend exceptionList NIG_REG_XGXS0_STATUS_MAC_COL
    lappend exceptionList NIG_REG_XGXS1_STATUS_TXPLL_LOCK
    lappend exceptionList NIG_REG_XGXS1_STATUS_MAC_COL
    lappend exceptionList NIG_REG_SERDES1_STATUS_TXPLL_LOCK
    lappend exceptionList NIG_REG_SERDES1_STATUS_MAC_COL
    lappend exceptionList NIG_REG_EMAC0_STATUS_MISC_MI_INT
    lappend exceptionList NIG_REG_EMAC0_STATUS_MISC_MI_COMPLETE
    lappend exceptionList NIG_REG_EMAC0_STATUS_MISC_CFG_CHANGE
    lappend exceptionList NIG_REG_EMAC0_STATUS_MISC_LINK_STATUS
    lappend exceptionList NIG_REG_EMAC0_STATUS_MISC_LINK_CHANGE
    lappend exceptionList NIG_REG_EMAC0_STATUS_MISC_ATTN
    lappend exceptionList NIG_REG_SERDES0_STATUS_MAC_CRS
    lappend exceptionList NIG_REG_SERDES0_STATUS_AUTONEG_COMPLETE
    lappend exceptionList NIG_REG_SERDES0_STATUS_FIBER_RXACT
    lappend exceptionList NIG_REG_SERDES0_STATUS_LINK_STATUS
    lappend exceptionList NIG_REG_SERDES0_STATUS_MR_PAGE_RX
    lappend exceptionList NIG_REG_SERDES0_STATUS_CL73_AN_COMPLETE
    lappend exceptionList NIG_REG_SERDES0_STATUS_CL73_MR_PAGE_RX
    lappend exceptionList NIG_REG_SERDES0_STATUS_RX_SIGDET
    lappend exceptionList NIG_REG_XGXS0_STATUS_REMOTEMDIOREQ
    lappend exceptionList NIG_REG_XGXS0_STATUS_LINK10G
    lappend exceptionList NIG_REG_XGXS0_STATUS_AUTONEG_COMPLETE
    lappend exceptionList NIG_REG_XGXS0_STATUS_FIBER_RXACT
    lappend exceptionList NIG_REG_XGXS0_STATUS_LINK_STATUS
    lappend exceptionList NIG_REG_XGXS0_STATUS_MR_PAGE_RX
    lappend exceptionList NIG_REG_XGXS0_STATUS_CL73_AN_COMPLETE
    lappend exceptionList NIG_REG_XGXS0_STATUS_CL73_MR_PAGE_RX
    lappend exceptionList NIG_REG_XGXS0_STATUS_RX_SIGDET
    lappend exceptionList NIG_REG_XGXS0_STATUS_MAC_CRS
    lappend exceptionList NIG_REG_EMAC1_STATUS_MISC_MI_INT
    lappend exceptionList NIG_REG_EMAC1_STATUS_MISC_MI_COMPLETE
    lappend exceptionList NIG_REG_EMAC1_STATUS_MISC_CFG_CHANGE
    lappend exceptionList NIG_REG_EMAC1_STATUS_MISC_LINK_STATUS
    lappend exceptionList NIG_REG_EMAC1_STATUS_MISC_LINK_CHANGE
    lappend exceptionList NIG_REG_EMAC1_STATUS_MISC_ATTN
    lappend exceptionList NIG_REG_SERDES1_STATUS_MAC_CRS
    lappend exceptionList NIG_REG_SERDES1_STATUS_AUTONEG_COMPLETE
    lappend exceptionList NIG_REG_SERDES1_STATUS_FIBER_RXACT
    lappend exceptionList NIG_REG_SERDES1_STATUS_LINK_STATUS
    lappend exceptionList NIG_REG_SERDES1_STATUS_AUTONEG_COMPLETE
    lappend exceptionList NIG_REG_SERDES1_STATUS_FIBER_RXACT
    lappend exceptionList NIG_REG_SERDES1_STATUS_LINK_STATUS
    lappend exceptionList NIG_REG_SERDES1_STATUS_MR_PAGE_RX
    lappend exceptionList NIG_REG_SERDES1_STATUS_CL73_AN_COMPLETE
    lappend exceptionList NIG_REG_SERDES1_STATUS_CL73_MR_PAGE_RX
    lappend exceptionList NIG_REG_SERDES1_STATUS_RX_SIGDET
    lappend exceptionList NIG_REG_XGXS1_STATUS_REMOTEMDIOREQ
    lappend exceptionList NIG_REG_XGXS1_STATUS_LINK10G
    lappend exceptionList NIG_REG_XGXS1_STATUS_AUTONEG_COMPLETE
    lappend exceptionList NIG_REG_XGXS1_STATUS_FIBER_RXACT    
    lappend exceptionList NIG_REG_XGXS1_STATUS_LINK_STATUS
    lappend exceptionList NIG_REG_XGXS1_STATUS_MR_PAGE_RX
    lappend exceptionList NIG_REG_XGXS1_STATUS_CL73_AN_COMPLETE
    lappend exceptionList NIG_REG_XGXS1_STATUS_CL73_MR_PAGE_RX
    lappend exceptionList NIG_REG_XGXS1_STATUS_RX_SIGDET
    lappend exceptionList NIG_REG_XGXS1_STATUS_MAC_CRS    
    lappend exceptionList NIG_REG_INGRESS_BMAC0_MEM
    lappend exceptionList NIG_REG_INGRESS_BMAC1_MEM
    lappend exceptionList NIG_REG_LED_STATUS_ACTIVE_P1
    
    #skip this register
    lappend exceptionList NIG_REG_LATCH_STATUS_0
    lappend exceptionList NIG_REG_LATCH_STATUS_1    
    lappend exceptionList NIG_REG_PORT4MODE_EN
    lappend exceptionList NIG_REG_LED_STATUS_ACTIVE_P0
    lappend exceptionList NIG_REG_XGXS0_SPEED_10000_LR_SR
    lappend exceptionList NIG_REG_XGXS0_SPEED_10000_XFI
    lappend exceptionList NIG_REG_XGXS0_SPEED_20000_CX4                           
    lappend exceptionList NIG_REG_XGXS0_SPEED_20000
    lappend exceptionList NIG_REG_XGXS1_SPEED_10000_LR_SR
    lappend exceptionList NIG_REG_XGXS1_SPEED_10000_XFI
    lappend exceptionList NIG_REG_XGXS1_SPEED_20000_CX4                           
    lappend exceptionList NIG_REG_XGXS1_SPEED_20000
    
     # Skip these registers in E3 B0
    lappend exceptionList NIG_REG_P0_LLH_PTP_MCP_BUF_SEQID        
    lappend exceptionList NIG_REG_P0_LLH_PTP_HOST_BUF_SEQID       
    lappend exceptionList NIG_REG_P1_LLH_PTP_MCP_BUF_SEQID        
    lappend exceptionList NIG_REG_P1_LLH_PTP_HOST_BUF_SEQID       
    lappend exceptionList NIG_REG_P0_TLLH_PTP_BUF_SEQID   
    lappend exceptionList NIG_REG_P1_TLLH_PTP_BUF_SEQID
    lappend exceptionList NIG_REG_P0_RX_PTP_TS_MSB_ERR
    lappend exceptionList NIG_REG_P0_TX_PTP_TS_MSB_ERR
    lappend exceptionList NIG_REG_P1_RX_PTP_TS_MSB_ERR
    lappend exceptionList NIG_REG_P1_TX_PTP_TS_MSB_ERR
    lappend exceptionList NIG_REG_NIG_TSGEN_INT_MASK
    lappend exceptionList NIG_REG_TSGEN_CTRL
    lappend exceptionList NIG_REG_TSGEN_TSIO_OUTPUT_CONTROL
    lappend exceptionList NIG_REG_TSGEN_DRIFT_T0
    lappend exceptionList NIG_REG_TSGEN_TSIO_INCTL_T0
    lappend exceptionList NIG_REG_TSGEN_TSIO_OUTCTL_T0
    lappend exceptionList NIG_REG_TSGEN_DRIFT_T1
    lappend exceptionList NIG_REG_TSGEN_TSIO_INCTL_T1
    lappend exceptionList NIG_REG_TSGEN_TSIO_OUTCTL_T1
    lappend exceptionList NIG_REG_XGXS0_LINK10G_DXGXS1
    lappend exceptionList NIG_REG_XGXS1_LINK10G_DXGXS1
    lappend exceptionList NIG_REG_TSGEN_FREECOUNT
    
    lappend exceptionList PRS_REG_INFO_VECTOR
    
    #skip cause differnt reset value in everest1.5
    #lappend exceptionList PRS_REG_INITIAL_HEADER_SIZE 

    
    # This register should be skipped as it changes the registers
    # GRC mode (advised by Ofer)
    lappend exceptionList PXP_REG_HST_GRC_SWAP_MODE
    lappend exceptionList PXP_REG_HST_DISABLE_INPUTS
    lappend exceptionList PXP_REG_HST_ARB_IS_IDLE
    lappend exceptionList PXP_REG_HST_CLIENTS_WAITING_TO_ARB
    lappend exceptionList PXP_REG_DBGSYN_STATUS
    
    lappend exceptionList PXP_REG_HST_ZONE_PERM_TABLE_INIT
    lappend exceptionList PXP_REG_HST_ZONE_PERM_TABLE_INIT_DONE
    lappend exceptionList PXP_REG_PXP_PRTY_MASK
    lappend exceptionList PGLUE_B_REG_PGLUE_B_PRTY_MASK
    
    #Skip cause "driver load" changes their value, fails A15 test
    lappend exceptionList PXP_REG_PXP_INT_MASK_0
    lappend exceptionList PXP_REG_PXP_INT_MASK_1
    
    lappend exceptionList PXP_REG_HST_PORT4MODE_EN
    
    #Skip these registers to avoid failure incase another application
    #accessed invalid address (since PXP block is nor reset via reset_chip)
    lappend exceptionList PXP_REG_HST_INCORRECT_ACCESS_DATA
    lappend exceptionList PXP_REG_HST_INCORRECT_ACCESS_VALID
    lappend exceptionList PXP_REG_HST_INCORRECT_ACCESS_ADDRESS


    #The following PXP2 registers should be skipped (Ofer)
    #Read only debug registers with incorrect reset value
    lappend exceptionList PXP2_REG_RD_PORT_IS_IDLE_0 
    lappend exceptionList PXP2_REG_RD_PORT_IS_IDLE_1
    #Registers that affect blocks initialization process
    lappend exceptionList PXP2_REG_RQ_RBC_DONE
    lappend exceptionList PXP2_REG_RD_START_INIT
    #Registers that disable inputs or affect target access
    lappend exceptionList PXP2_REG_HST_DISABLE_INPUTS_PCI 
    lappend exceptionList PXP2_REG_PGL_CONTROL0 
    lappend exceptionList PXP2_REG_PGL_CONTROL1
    #Debug only registers in PGLUE
    lappend exceptionList PXP2_REG_PGL_DQ_PREFIX2
    lappend exceptionList PXP2_REG_PGL_PCIE_REPLAY_TM
    lappend exceptionList PXP2_REG_PGL_DEBUG 
    lappend exceptionList PXP2_REG_PGL_PCIE     
    lappend exceptionList PXP2_REG_PGL_TXR_CDTS                                 
    lappend exceptionList PXP2_REG_PGL_TXW_CDTS                                 
    lappend exceptionList PXP2_REG_PGL_MOT        
    lappend exceptionList PXP2_REG_PGL_CORE_DEBUG
    lappend exceptionList PXP2_REG_PGL_EXP_ROM0
    lappend exceptionList PXP2_REG_PGL_EXP_ROM1
    lappend exceptionList PXP2_REG_PGL_EXP_ROM2
    lappend exceptionList PXP2_REG_RQ_CMG_GARB_HOQ_RDWR_N        
    lappend exceptionList PXP2_REG_RQ_GARB_CTRL   
    lappend exceptionList PXP2_REG_RQ_GARB_SBMT
            
    #This register does not have read access
    lappend exceptionList DORQ_REG_DQ_FIFO_FULL_CLR
    lappend exceptionList DORQ_REG_DQ_FIFO
    lappend exceptionList DORQ_REG_DQ_FILL_LVL_MAX
    lappend exceptionList DORQ_REG_DQ_FIFO_FULL_TH
          
    lappend exceptionList UCM_REG_UQM_INIT_CRD

    # These registers require special initialization (according to Yuval)
    lappend exceptionList CFC_REG_CID_CAM
    lappend exceptionList CFC_REG_INFO_RAM
    lappend exceptionList CFC_REG_STRING_CAM
    lappend exceptionList CFC_REG_LINK_LIST

    # These registers require special configuration for testing
    # the HC block (advise by Yariv)
    lappend exceptionList HC_REG_CONFIG_0
    lappend exceptionList HC_REG_CONFIG_1
    lappend exceptionList HC_REG_COMMAND_REG
    lappend exceptionList HC_REG_STATISTIC_COUNTERS

    # Skip these registers as there 12 LSB bits are fixed to zero
    #lappend exceptionList QM_REG_BASEADDR

    # Skip these registers as they are Write-Only
    lappend exceptionList QM_REG_UPDATECREDIT
    
    # Skip this register as its access type is different for each bit
    lappend exceptionList DBU_REG_DBU_CMD

    # Skip these MISC special registers
    lappend exceptionList MISC_REG_IPOR_CMD_REG
    lappend exceptionList MISC_REG_UNPREPPARED                                                    
    lappend exceptionList MISC_REG_SPIO
    lappend exceptionList MISC_REG_SPIO_INT
    lappend exceptionList MISC_REG_GPIO
    lappend exceptionList MISC_REG_GPIO_INT
    lappend exceptionList MISC_REG_SHIFT1_DATA
    lappend exceptionList MISC_REG_SHIFT1_CMD
    lappend exceptionList MISC_REG_SHIFT2_DATA
    lappend exceptionList MISC_REG_SHIFT2_CMD
    lappend exceptionList MISC_REG_MAIN_SEQ_BYP_SEL
    lappend exceptionList MISC_REG_MAIN_SEQ_BYP_VAL
    lappend exceptionList MISC_REG_ISOLATION_LOGIC
    lappend exceptionList MISC_REG_UNCOND_ENTER_PLAY_DEAD
    lappend exceptionList MISC_REG_CHIP_SILENT_REV
    lappend exceptionList MISC_REG_LINK_HOLDOFF_REQ
    lappend exceptionList MISC_REG_SHARED_MEM_ADDR   
    lappend exceptionList MISC_REG_CHIP_REV
    lappend exceptionList MISC_REG_VAUX_PRESENT    
    lappend exceptionList MISC_REG_PARITY_MODE
    lappend exceptionList MISC_REG_NIG_WOL_P0
    lappend exceptionList MISC_REG_NIG_WOL_P1
    # This register's reset value can be either 0 or 1 - thus not tested
    lappend exceptionList MISC_REG_AEU_GENERAL_ATTN_21    
        
    # Exceptions added for chip rev A0
    lappend exceptionList MISC_REG_AEU_ENABLE4_MCP_OUT_0
    lappend exceptionList MISC_REG_AEU_ENABLE4_MCP_OUT_1
    lappend exceptionList MISC_REG_AEU_EVENT_ENABLE
    lappend exceptionList MISC_REG_AEU_MASK_ATTN_MCP
    lappend exceptionList MISC_REG_GRC_TIMEOUT_EN
    lappend exceptionList MISC_REG_LCPLL_CTRL_REG_2
    lappend exceptionList MISC_REG_LCPLL_MISC_LOCK
    # Exceptions added for chip rev A0
    lappend exceptionList MISC_REG_LCPLL_STATE  
    lappend exceptionList MISC_REG_PLL_MAIN_MISC_LOCK
    lappend exceptionList MISC_REG_FUNCTION_HIDE
    # This register differs for each chip rev
    lappend exceptionList MISC_REG_CHIP_METAL
    # Exceptions added for chip rev B0
    lappend exceptionList MISC_REG_SW_TIMER_10US_RESOLUTION  


    #Skip these registers cause they are used to configure DBG block
    lappend exceptionList DBG_REG_IMMEDIATE_ACK
    lappend exceptionList DBG_REG_DBG_BLOCK_ON


    #Skip this memory register, test it in the memtest script
    lappend exceptionList DBG_REG_CALENDAR_OUT_DATA
    lappend exceptionList DBG_REG_EXT_BUFFER_RD_PTR
    lappend exceptionList DBG_REG_EXT_BUFFER_WR_PTR
    
    #Skip these registers 
    lappend exceptionList DMAE_REG_DMAE_CPT_MBIST1_CNTRL_CMD
    lappend exceptionList DMAE_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD
    lappend exceptionList DMAE_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD
    lappend exceptionList DMAE_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD
    lappend exceptionList DMAE_REG_CPU_MBIST_MEMCTRL_3_CNTRL_CMD
    lappend exceptionList DMAE_REG_CPU_MBIST_MEMCTRL_4_CNTRL_CMD
    lappend exceptionList DMAE_REG_CPU_MBIST_MEMCTRL_5_CNTRL_CMD
    lappend exceptionList DMAE_REG_CPU_MBIST_MEMCTRL_6_CNTRL_CMD
    lappend exceptionList DMAE_REG_CPU_MBIST_MEMCTRL_7_CNTRL_CMD
    lappend exceptionList DBG_REG_DBG_NM_MBIST1_CNTRL_CMD    
    lappend exceptionList PBF_REG_PBF_TX_MBIST1_CNTRL_CMD
    lappend exceptionList USDM_REG_USDM_ULP_MBIST1_CNTRL_CMD
    lappend exceptionList TSDM_REG_TSDM_RX_MBIST1_CNTRL_CMD  
    lappend exceptionList PGLUE_B_REG_PORT4MODE_EN      
    
    #write clear register
    lappend exceptionList CDU_REG_ERROR_DATA

    #IGU ignore registers
    lappend exceptionList IGU_REG_ERROR_HANDLING_MEMORY
    lappend exceptionList IGU_REG_ATTN_MSG_ADDR_L         
    lappend exceptionList IGU_REG_COMMAND_REG_32MSB_DATA
    lappend exceptionList IGU_REG_COMMAN_REG_32LSB_DATA
    lappend exceptionList IGU_REG_COMMAND_REG_CTRL
    lappend exceptionList IGU_REG_RESET_MEMORIES
    lappend exceptionList IGU_REG_ATTENTION_SIGNAL_P0_STATUS
    lappend exceptionList IGU_REG_ATTENTION_SIGNAL_P1_STATUS
    lappend exceptionList IGU_REG_CTRL_FSM
    lappend exceptionList IGU_REG_PXP_ARB_FSM
    
    lappend exceptionList ATC_REG_ATC_INIT_ARRAY
    lappend exceptionList ATC_REG_ATC_INIT_DONE
    lappend exceptionList ATC_REG_ATC_EMPTY_REG

    #SEMI ignore registers semifast registers
    lappend exceptionList CSEM_REG_SEM_FAST_REAL_TIME_CNT1
    lappend exceptionList CSEM_REG_SEM_FAST_REAL_TIME_CNT2
    lappend exceptionList CSEM_REG_SEM_FAST_REAL_TIME_CNT3
    lappend exceptionList CSEM_REG_SEM_FAST_REAL_TIME_CNT4
    lappend exceptionList CSEM_REG_SEM_FAST_SEM_FAST_INT_MASK
    lappend exceptionList CSEM_REG_SEM_FAST_SEM_FAST_PRTY_MASK
    lappend exceptionList CSEM_REG_SEM_FAST_IDLE_CNT0 
    lappend exceptionList CSEM_REG_SEM_FAST_IDLE_CNT1
    lappend exceptionList CSEM_REG_SEM_FAST_IDLE_CNT2
    lappend exceptionList CSEM_REG_SEM_FAST_IDLE_CNT3
    lappend exceptionList CSEM_REG_SEM_FAST_CAM_VALUE_MSB
    lappend exceptionList CSEM_REG_SEM_FAST_PARITY_RST
    lappend exceptionList CSEM_REG_SEM_FAST_ERROR_RST
    lappend exceptionList CSEM_REG_SEM_FAST_VFC_DATA_RD
    lappend exceptionList CSEM_REG_SEM_FAST_VFC_DATA_WR
    lappend exceptionList CSEM_REG_SEM_FAST_VFC_ADDR 
    lappend exceptionList CSEM_REG_SEM_FAST_PAS_REG_SET0  
    lappend exceptionList CSEM_REG_SEM_FAST_PAS_REG_SET1  
    lappend exceptionList CSEM_REG_SEM_FAST_MAX_HANDLER_CYCLES
    lappend exceptionList CSEM_REG_SEM_FAST_STORE_HALT_CYCLES
    lappend exceptionList CSEM_REG_SEM_FAST_IDLE_CNT4


    lappend exceptionList TSEM_REG_SEM_FAST_REAL_TIME_CNT1
    lappend exceptionList TSEM_REG_SEM_FAST_REAL_TIME_CNT2
    lappend exceptionList TSEM_REG_SEM_FAST_REAL_TIME_CNT3
    lappend exceptionList TSEM_REG_SEM_FAST_REAL_TIME_CNT4
    lappend exceptionList TSEM_REG_SEM_FAST_SEM_FAST_INT_MASK
    lappend exceptionList TSEM_REG_SEM_FAST_SEM_FAST_PRTY_MASK 
    lappend exceptionList TSEM_REG_SEM_FAST_IDLE_CNT0 
    lappend exceptionList TSEM_REG_SEM_FAST_IDLE_CNT1
    lappend exceptionList TSEM_REG_SEM_FAST_IDLE_CNT2
    lappend exceptionList TSEM_REG_SEM_FAST_IDLE_CNT3
    lappend exceptionList TSEM_REG_SEM_FAST_CAM_VALUE_MSB
    lappend exceptionList TSEM_REG_SEM_FAST_PARITY_RST
    lappend exceptionList TSEM_REG_SEM_FAST_ERROR_RST
    lappend exceptionList TSEM_REG_SEM_FAST_VFC_DATA_RD
    lappend exceptionList TSEM_REG_SEM_FAST_VFC_DATA_WR
    lappend exceptionList TSEM_REG_SEM_FAST_VFC_ADDR
    lappend exceptionList TSEM_REG_SEM_FAST_PAS_REG_SET0  
    lappend exceptionList TSEM_REG_SEM_FAST_PAS_REG_SET1  
    lappend exceptionList TSEM_REG_SEM_FAST_MAX_HANDLER_CYCLES
    lappend exceptionList TSEM_REG_SEM_FAST_STORE_HALT_CYCLES
    lappend exceptionList TSEM_REG_SEM_FAST_IDLE_CNT4


    lappend exceptionList XSEM_REG_SEM_FAST_REAL_TIME_CNT1
    lappend exceptionList XSEM_REG_SEM_FAST_REAL_TIME_CNT2
    lappend exceptionList XSEM_REG_SEM_FAST_REAL_TIME_CNT3
    lappend exceptionList XSEM_REG_SEM_FAST_REAL_TIME_CNT4
    lappend exceptionList XSEM_REG_SEM_FAST_SEM_FAST_INT_MASK
    lappend exceptionList XSEM_REG_SEM_FAST_SEM_FAST_PRTY_MASK 
    lappend exceptionList XSEM_REG_SEM_FAST_IDLE_CNT0 
    lappend exceptionList XSEM_REG_SEM_FAST_IDLE_CNT1
    lappend exceptionList XSEM_REG_SEM_FAST_IDLE_CNT2
    lappend exceptionList XSEM_REG_SEM_FAST_IDLE_CNT3
    lappend exceptionList XSEM_REG_SEM_FAST_CAM_VALUE_MSB
    lappend exceptionList XSEM_REG_SEM_FAST_PARITY_RST
    lappend exceptionList XSEM_REG_SEM_FAST_ERROR_RST
    lappend exceptionList XSEM_REG_SEM_FAST_VFC_DATA_RD
    lappend exceptionList XSEM_REG_SEM_FAST_VFC_DATA_WR
    lappend exceptionList XSEM_REG_SEM_FAST_VFC_ADDR 
    lappend exceptionList XSEM_REG_SEM_FAST_PAS_REG_SET0  
    lappend exceptionList XSEM_REG_SEM_FAST_PAS_REG_SET1  
    lappend exceptionList XSEM_REG_SEM_FAST_MAX_HANDLER_CYCLES
    lappend exceptionList XSEM_REG_SEM_FAST_STORE_HALT_CYCLES
    lappend exceptionList XSEM_REG_SEM_FAST_IDLE_CNT4

    lappend exceptionList USEM_REG_SEM_FAST_REAL_TIME_CNT1
    lappend exceptionList USEM_REG_SEM_FAST_REAL_TIME_CNT2
    lappend exceptionList USEM_REG_SEM_FAST_REAL_TIME_CNT3
    lappend exceptionList USEM_REG_SEM_FAST_REAL_TIME_CNT4
    lappend exceptionList USEM_REG_SEM_FAST_SEM_FAST_INT_MASK
    lappend exceptionList USEM_REG_SEM_FAST_SEM_FAST_PRTY_MASK 
    lappend exceptionList USEM_REG_SEM_FAST_IDLE_CNT0 
    lappend exceptionList USEM_REG_SEM_FAST_IDLE_CNT1
    lappend exceptionList USEM_REG_SEM_FAST_IDLE_CNT2
    lappend exceptionList USEM_REG_SEM_FAST_IDLE_CNT3
    lappend exceptionList USEM_REG_SEM_FAST_CAM_VALUE_MSB
    lappend exceptionList USEM_REG_SEM_FAST_PARITY_RST
    lappend exceptionList USEM_REG_SEM_FAST_ERROR_RST
    lappend exceptionList USEM_REG_SEM_FAST_VFC_DATA_RD
    lappend exceptionList USEM_REG_SEM_FAST_VFC_DATA_WR
    lappend exceptionList USEM_REG_SEM_FAST_VFC_ADDR
    lappend exceptionList USEM_REG_SEM_FAST_PAS_REG_SET0  
    lappend exceptionList USEM_REG_SEM_FAST_PAS_REG_SET1  
    lappend exceptionList USEM_REG_SEM_FAST_MAX_HANDLER_CYCLES
    lappend exceptionList USEM_REG_SEM_FAST_STORE_HALT_CYCLES
    lappend exceptionList USEM_REG_SEM_FAST_IDLE_CNT4

    # SEMI ignore registers VFC registers
    lappend exceptionList TSEM_REG_VFC_VFC_PRTY_MASK
    lappend exceptionList TSEM_REG_VFC_VFC_INT_MASK
    lappend exceptionList TSEM_REG_VFC_INTERRUPT_IND
    lappend exceptionList TSEM_REG_VFC_MEMORIES_RST
    lappend exceptionList TSEM_REG_VFC_PARITY_IND
    lappend exceptionList TSEM_REG_VFC_DEBUG_DATA
    lappend exceptionList TSEM_REG_VFC_INDICATIONS1
    lappend exceptionList TSEM_REG_VFC_INDICATIONS2 

    lappend exceptionList XSEM_REG_VFC_VFC_PRTY_MASK
    lappend exceptionList XSEM_REG_VFC_VFC_INT_MASK
    lappend exceptionList XSEM_REG_VFC_INTERRUPT_IND
    lappend exceptionList XSEM_REG_VFC_MEMORIES_RST
    lappend exceptionList XSEM_REG_VFC_PARITY_IND 
    lappend exceptionList XSEM_REG_VFC_DEBUG_DATA
    lappend exceptionList XSEM_REG_VFC_INDICATIONS1
    lappend exceptionList XSEM_REG_VFC_INDICATIONS2 

    # NIG ignore registers 
    lappend exceptionList NIG_REG_TIMER1_COUNTER
    
    # Ignore PGLUE_B registers
    lappend exceptionList PGLUE_B_REG_SHADOW_BME_PF_7_0
    lappend exceptionList PGLUE_B_REG_INTERNAL_PFID_ENABLE_TARGET_WRITE
    lappend exceptionList PGLUE_B_REG_INTERNAL_PFID_ENABLE_TARGET_READ 
    lappend exceptionList PGLUE_B_REG_PCIE_ERR_STATUS
    lappend exceptionList PGLUE_B_REG_SHADOW_BME_PF_7_0
    
    # Exclude PGLUE_B registers which change and not reset by driver unload
    # TBD solve this issue
    lappend exceptionList PGLUE_B_REG_CSDM_START_OFFSET_A
    lappend exceptionList PGLUE_B_REG_CSDM_START_OFFSET_B
    lappend exceptionList PGLUE_B_REG_CSDM_VF_SHIFT_B
    lappend exceptionList PGLUE_B_REG_CSDM_ZONE_A_SIZE_PF
    lappend exceptionList PGLUE_B_REG_TSDM_START_OFFSET_A
    lappend exceptionList PGLUE_B_REG_TSDM_START_OFFSET_B
    lappend exceptionList PGLUE_B_REG_TSDM_VF_SHIFT_B
    lappend exceptionList PGLUE_B_REG_TSDM_ZONE_A_SIZE_PF
    lappend exceptionList PGLUE_B_REG_USDM_INB_INT_A_0
    lappend exceptionList PGLUE_B_REG_USDM_INB_INT_A_1
    lappend exceptionList PGLUE_B_REG_USDM_INB_INT_A_2
    lappend exceptionList PGLUE_B_REG_USDM_INB_INT_A_3
    lappend exceptionList PGLUE_B_REG_USDM_INB_INT_A_4
    lappend exceptionList PGLUE_B_REG_USDM_INB_INT_A_5
    lappend exceptionList PGLUE_B_REG_USDM_INB_INT_A_6
    lappend exceptionList PGLUE_B_REG_USDM_INB_INT_A_PF_ENABLE
    lappend exceptionList PGLUE_B_REG_USDM_INB_INT_A_VF_ENABLE
    lappend exceptionList PGLUE_B_REG_USDM_START_OFFSET_A
    lappend exceptionList PGLUE_B_REG_USDM_START_OFFSET_B
    lappend exceptionList PGLUE_B_REG_USDM_VF_SHIFT_B
    lappend exceptionList PGLUE_B_REG_USDM_ZONE_A_SIZE_PF
    lappend exceptionList PGLUE_B_REG_XSDM_START_OFFSET_A
    lappend exceptionList PGLUE_B_REG_XSDM_START_OFFSET_B
    lappend exceptionList PGLUE_B_REG_XSDM_VF_SHIFT_B
    lappend exceptionList PGLUE_B_REG_XSDM_ZONE_A_SIZE_PF
    lappend exceptionList PGLUE_B_REG_CSDM_INB_INT_B_VF
    lappend exceptionList PGLUE_B_REG_CSDM_INB_INT_B_VF_ENABLE
    lappend exceptionList PGLUE_B_REG_SHADOW_BME_PF_7_0_CLR
    lappend exceptionList PGLUE_B_REG_SHADOW_BME_VF_127_96_CLR 
    lappend exceptionList PGLUE_B_REG_SHADOW_BME_VF_31_0_CLR
    lappend exceptionList PGLUE_B_REG_SHADOW_BME_VF_63_32_CLR
    lappend exceptionList PGLUE_B_REG_SHADOW_BME_VF_95_64_CLR
    lappend exceptionList PGLUE_B_REG_INTERNAL_VFID_ENABLE_31_0_VALUE
    lappend exceptionList PGLUE_B_REG_INTERNAL_VFID_ENABLE_63_32_VALUE
    lappend exceptionList PGLUE_B_REG_INTERNAL_VFID_ENABLE_95_64_VALUE
    lappend exceptionList PGLUE_B_REG_INTERNAL_VFID_ENABLE_127_96_VALUE
    
   # Exclude PGLUE_B registers - MBIST and TM
   lappend exceptionList PGLUE_B_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD        
   lappend exceptionList PGLUE_B_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD        
   lappend exceptionList PGLUE_B_REG_PGL_TM0
   lappend exceptionList PGLUE_B_REG_INTERNAL_PFID_ENABLE_VALUE
   lappend exceptionList PGLUE_B_REG_INTERNAL_PFID_ENABLE_MASTER    
   
   # Exclude UMAC/XMAC registers - some of their bits are invalid
   lappend exceptionList UMAC0_REG_UMAC_TIMESTAMP_ADJUST
   lappend exceptionList UMAC0_REG_COMMAND_CONFIG
   lappend exceptionList UMAC0_REG_MAC_MODE
   lappend exceptionList UMAC0_REG_MAC_PFC_CTRL
   lappend exceptionList UMAC0_REG_MAC_PFC_REFRESH_CTRL
   lappend exceptionList UMAC0_REG_RXFIFO_STAT
   lappend exceptionList UMAC0_REG_SFD_OFFSET
   lappend exceptionList UMAC0_REG_TS_STATUS_CNTRL
   lappend exceptionList UMAC0_REG_TX_TS_DATA
   lappend exceptionList UMAC0_REG_TXFIFO_STAT
   lappend exceptionList UMAC0_REG_TX_TS_SEQ_ID  
   lappend exceptionList UMAC1_REG_UMAC_TIMESTAMP_ADJUST
   lappend exceptionList UMAC1_REG_COMMAND_CONFIG
   lappend exceptionList UMAC1_REG_MAC_MODE
   lappend exceptionList UMAC1_REG_MAC_PFC_CTRL
   lappend exceptionList UMAC1_REG_MAC_PFC_REFRESH_CTRL
   lappend exceptionList UMAC1_REG_RXFIFO_STAT
   lappend exceptionList UMAC1_REG_SFD_OFFSET
   lappend exceptionList UMAC1_REG_TS_STATUS_CNTRL
   lappend exceptionList UMAC1_REG_TX_TS_DATA
   lappend exceptionList UMAC1_REG_TXFIFO_STAT
   lappend exceptionList UMAC1_REG_TX_TS_SEQ_ID
   
   lappend exceptionList XMAC0_REG_FIFO_STATUS
   lappend exceptionList XMAC0_REG_RX_LSS_STATUS
   lappend exceptionList XMAC0_REG_SCH_CTRL 
   lappend exceptionList XMAC0_REG_LH_HDR_0_SCH_IDLE_HI
   lappend exceptionList XMAC0_REG_LH_HDR_0_SCH_IDLE_LO
   lappend exceptionList XMAC0_REG_LH_HDR_1_HI
   lappend exceptionList XMAC0_REG_LH_HDR_1_LO
   lappend exceptionList XMAC0_REG_LH_HDR_2_HI
   lappend exceptionList XMAC0_REG_LH_HDR_2_LO
   lappend exceptionList XMAC0_REG_LH_HDR_3_HI
   lappend exceptionList XMAC0_REG_LH_HDR_3_LO
   lappend exceptionList XMAC0_REG_SEQUENCE_ID
   lappend exceptionList XMAC0_REG_TIME_STAMP
   lappend exceptionList XMAC0_REG_WB_LH_HDR_0_SCH_IDLE
   lappend exceptionList XMAC0_REG_WB_LH_HDR_1
   lappend exceptionList XMAC0_REG_WB_LH_HDR_2
   lappend exceptionList XMAC0_REG_WB_LH_HDR_3
   lappend exceptionList XMAC0_REG_WB_OSTS_TIMESTAMP_ADJUST
   lappend exceptionList XMAC0_REG_ENTRY_COUNT
   lappend exceptionList XMAC0_REG_WB_PFC_DA
   lappend exceptionList XMAC0_REG_WB_RX_MAC_SA
   lappend exceptionList XMAC0_REG_WB_TX_MAC_SA
   lappend exceptionList XMAC0_REG_TX_TIMESTAMP_FIFO_DATA_HI
   lappend exceptionList XMAC1_REG_FIFO_STATUS
   lappend exceptionList XMAC1_REG_RX_LSS_STATUS
   lappend exceptionList XMAC1_REG_SCH_CTRL 
   lappend exceptionList XMAC1_REG_LH_HDR_0_SCH_IDLE_HI
   lappend exceptionList XMAC1_REG_LH_HDR_0_SCH_IDLE_LO
   lappend exceptionList XMAC1_REG_LH_HDR_1_HI
   lappend exceptionList XMAC1_REG_LH_HDR_1_LO
   lappend exceptionList XMAC1_REG_LH_HDR_2_HI
   lappend exceptionList XMAC1_REG_LH_HDR_2_LO
   lappend exceptionList XMAC1_REG_LH_HDR_3_HI
   lappend exceptionList XMAC1_REG_LH_HDR_3_LO
   lappend exceptionList XMAC1_REG_SEQUENCE_ID
   lappend exceptionList XMAC1_REG_TIME_STAMP
   lappend exceptionList XMAC1_REG_WB_LH_HDR_0_SCH_IDLE
   lappend exceptionList XMAC1_REG_WB_LH_HDR_1
   lappend exceptionList XMAC1_REG_WB_LH_HDR_2
   lappend exceptionList XMAC1_REG_WB_LH_HDR_3
   lappend exceptionList XMAC1_REG_WB_OSTS_TIMESTAMP_ADJUST
   lappend exceptionList XMAC1_REG_ENTRY_COUNT
   lappend exceptionList XMAC1_REG_WB_PFC_DA
   lappend exceptionList XMAC1_REG_WB_RX_MAC_SA
   lappend exceptionList XMAC1_REG_WB_TX_MAC_SA
   lappend exceptionList XMAC1_REG_TX_TIMESTAMP_FIFO_DATA_HI     

   if {[is_e3]} {
      lappend exceptionList PBF_REG_CPU_MBIST_MEMCTRL_0_STATUS_0
      lappend exceptionList PBF_REG_CPU_MBIST_MEMCTRL_1_STATUS_0
                
      lappend exceptionList USDM_REG_CPU_MBIST_MEMCTRL_0_STATUS_0
      lappend exceptionList USDM_REG_CPU_MBIST_MEMCTRL_1_STATUS_0
                
      lappend exceptionList TSDM_REG_CPU_MBIST_MEMCTRL_0_STATUS_0
      lappend exceptionList TSDM_REG_CPU_MBIST_MEMCTRL_1_STATUS_0
      
      lappend exceptionList PBF_REG_CPU_MBIST_MEMCTRL_2_STATUS_0
      lappend exceptionList USDM_REG_CPU_MBIST_MEMCTRL_2_STATUS_0   
      
      if {[is_emul]} {
         # skip this register/memory on emulation due to forces in palladium
         lappend exceptionList IGU_REG_MAPPING_MEMORY 
      }
   }
}
# -----------------------------------------------------------------
# This proc gets the mask (number of 1's) according to the reg dataWidth
# e.g. if dataWidth=2 then mask is 0x11, is dataWidth=8 then mask is 0x11111111
# -----------------------------------------------------------------
proc get_mask {{dataWidth}}  {
    set mask 1
    set multiple 1
    
    # incase of a register > 32 bit, mask if of the remainder
    if {$dataWidth > 32} {
        set size [expr ($dataWidth % 32)]
    } else  {
        set size $dataWidth
    }
    
    # If register size is a multiple of 32 (%32 equals 0),
    # then last word mask is 0xFFFFFFFF
    if {$size == 0} {
        return 0xFFFFFFFF
    }
        
    for {set i 0} {$i < [expr ($size - 1)]} {incr i} {
        set multiple [expr $multiple * 2]
        set mask [expr ($mask + $multiple)]
    }
    set mask [format "0\x%08x" $mask]
    return $mask
}

# -----------------------------------------------------------------
# This proc checks is the machine on which test is running is a 
# ZLR machine, by writing/reading from several WB registers and
# incase of failure assumes this is a ZLR machine.
# -----------------------------------------------------------------
proc is_zlr {} {
    
    global regsOffset
    global rdfFile
   
    #Initialize the error counter
    set errCnt 0
    
    puts "Checking if ZLR machine .."
    
    #get RDF file name according to DID
    get_rdf_file
    # Parse the appropriate RDF file
    parse_rdf -f $rdfFile
    
                
    
    # Get chip out of reset, if it is in reset state
    if { [is_block_in_reset CSDM] } {       
        reset_chip 1
    }
    
    # List of wide-bus registers
    set wbRegs {CSDM_REG_TIMERS CSDM_REG_CM_QUEUE \
                CSDM_REG_INP_QUEUE SRC_REG_FIRSTFREE0}
    
    
    # Run through the list of WB register                  
    for {set idx 0} {$idx < [llength $wbRegs]} {incr idx} {
        set name [lindex $wbRegs $idx]        
        set dataWidth  [reg_size -n $name]
        set offset     [get_start_offset [reg_offset -n $name]]
                
        # Save the number of words (32 bit) of the register
        set noWords [expr ($dataWidth/32)]
        if { [expr ($dataWidth % 32) != 0] } {
            incr noWords
        }
        
        #puts "DEBUG: Testing $name dataWidth $dataWidth noWords $noWords"
       
        # Save the register's original value:
        # each 32-bit word of the register's org value is saved in array orgValue
        for {set i 0} {$i < $noWords} {incr i} {
            set wordOffset [expr ($offset + $i*4)]
            set orgValue($i,value) [reg dread $wordOffset]
                     
            # Create the mask according to the register's dataWidth
            if {$i < [expr ($noWords-1)]} {
               set mask($i,value) 0xFFFFFFFF            
            } else  {
               # The last part of the register's mask is according to remaining
               # data width e.g 100 bit register, mask is for remaining 4 bits (0x1111)
               set mask($i,value) [get_mask $dataWidth]             
            }         
        }              
        
        # Write to the WB register words the value 0x55AA55AA
        for {set i 0} {$i < $noWords} {incr i} {
            set wordOffset [expr ($offset + $i*4)]
            if {$i < [expr ($noWords-1)]} {
                set writtenValue($i,value) 0x55AA55AA
                reg write $wordOffset 0x55AA55AA
                #puts "DEBUG: Write to [format "0x%x" $wordOffset] value=0x55AA55AA"
            } else  {
                # write to the remaining part of the register
                # only to the valid bits (according to mask)
                set writtenWord [expr (0x55AA55AA & $mask($i,value))]
                set writtenWord [expr ($orgValue($i,value) | $writtenWord)]
                set writtenValue($i,value) $writtenWord
                reg write $wordOffset $writtenWord
                #puts "DEBUG: Write to [format "0x%x" $wordOffset] value=[format "0x%x" $writtenWord]"
            }
        }

        # Read the WB register words
        for {set i 0} {$i < $noWords} {incr i} {
            set wordOffset [expr ($offset + $i*4)]
            set readValue($i,value) [reg read $wordOffset]
        
            # save in temporary variables the current register's word (32bit)
            # readValue, orgValue and writtenValue
            set readWord     $readValue($i,value)            
            set writtenWord  $writtenValue($i,value)
            if {$readWord != $writtenWord} {
                #puts "DEBUG: Read from [format "0x%x" $wordOffset] value=[format "0x%x" $readWord]"
                incr errCnt 
            } 
        }
    
        # Restore the original value of the register's words
        for {set i 0} {$i < $noWords} {incr i} {
            set wordOffset [expr ($offset + $i*4)]        
            reg write  $wordOffset $orgValue($i,value)
        }             
    }
    
    #Reset the chip at the end of this check
    puts "Resetting chip .."
    reset_chip
    
    #Incase errCnt as the number of the tested WB registers,
    #assume the failures are cause this is a ZLR machine    
    if {$errCnt > 3} {
        #puts "DEBUG: errCnt=$errCnt assuming ZLR machine"
        return 1
    } else {
        return 0
    }  
}

# -----------------------------------------------------------------
# This proc builds the exception list which contains the registers
# which should be skipped during testing.
# -----------------------------------------------------------------
proc build_zlr_exception_list {}  {
    global zlrExceptionList
    
    # The registers we need to skip:    
    lappend zlrExceptionList CCM_REG_CAM_OCCUP_ST
    lappend zlrExceptionList CCM_REG_GLB_CNT_STICKY
    lappend zlrExceptionList UCM_REG_CAM_OCCUP_ST
    lappend zlrExceptionList UCM_REG_GLB_CNT_STICKY
    lappend zlrExceptionList CFC_REG_TASK_COUNTER
    lappend zlrExceptionList DORQ_REG_DQ_FILL_LVL_MAX    
    lappend zlrExceptionList CSDM_REG_STATISTICS
    lappend zlrExceptionList USDM_REG_STATISTICS
    lappend zlrExceptionList XSDM_REG_STATISTICS
    
}

# -----------------------------------------------------------------
# This proc checks if we need to test this register
# returns 1 - incase we need to test this register
# returns 0 - incase there is no need to test this register
# -----------------------------------------------------------------    
proc check_split_reg {{skipCommonReg} {splitVal} {port} {curFunc}} {
             
    
    switch $splitVal {
            "0" {               
                # Incase of common register (splitVal==0), run test only on Function 0
                # Testing common register on other functions, if skipCommonReg
                # flag is not set.
                if {($curFunc==0) || !$skipCommonReg } {
                    return 1
                }
            }
            "4" {
                # Incase of split4 register type, verify current function is related
                # to the register's port (mentioned in the phase field).
                # if port=0 check the register if current device is 0,2,4 or 6
                # if port=1 check the register if current device is 1,3,5 or 7
                if {($port==0) && ([expr $curFunc % 2] == 0)} {
                    return 1
                } elseif {($port==1) && ([expr $curFunc % 2] == 1)} {
                    return 1                    
                }
            }                
            "8" {
                #Incase of split 8 register type, run test on each function
                return 1
            }
            default {
                # Do not check unknown split register
                return 0
            }
    }
    #puts "DEBUG: Skip register from testing (splitVal $splitVal port $port curFunc $curFunc)"
    return 0
}

#----------------------------------------------------------------------------
# This utility is used for stopping the STORMs
#----------------------------------------------------------------------------
proc set_waitp {} {

    reg write 0x1B8A80 1
    reg write 0x238A80 1
    reg write 0x2B8A80 1
    reg write 0x338A80 1
}

# -----------------------------------------------------------------
# Help info on "set_waitp" command
# -----------------------------------------------------------------
proc set_waitp_help {args} {
    set help_info     "Stop the STORMs\n"
    append help_info  "Usage: set_waitp\n"    
    add_help set_waitp $help_info
}

#----------------------------------------------------------------------------
# Get RDF file according to the supplied environment variable
#----------------------------------------------------------------------------
proc get_rdf_file_of_chip_num_and_rev {{chip_num} {chip_rev}} {
   if {$chip_num == 5710} {
      return "everest.rdf"        
   } elseif {($chip_num == 5711) || ($chip_num == 5712)} {
      return "evrst15.rdf"                 
   } elseif {[is_e2]} {
      return "everst2.rdf"
   } elseif {[is_e3]} {
         #On E3 A0 Emulation chip rev is 0xE on FPGA 0xF
         #On E3 A0 ASIC chip rev should be 0
      if {($chip_rev == 0xE) || ($chip_rev == 0xF) || ($chip_rev == 0)} {
         set rdfFile "everst3.rdf"
      } else {
         #On E3 B0 Emulation chip rev is 0xC on FPGA 0xD
         #On E3 B0 ASIC chip rev should be 1
         set rdfFile "everst3b.rdf"
      }
   } else {
      return ""
   }               
}

#----------------------------------------------------------------------------
# Get RDF file according This utility is used for stopping the STORMs
#----------------------------------------------------------------------------
proc get_rdf_file {} {
   global rdfFile
    
   set chip_rev [reg read 0xA40C]
    
   if {$::sys(NO_PCI) == 0} {
      # first try to get rdf file according to $::current(DID)       
      set rdfFile [get_rdf_file_of_chip_num_and_rev $::current(DID) $chip_rev]
      # if device is not valid, try to get DID from the default value $::current(DEFAULT_DID) 
      if {$rdfFile == ""} {            
         set rdfFile [get_rdf_file_of_of_chip_num_and_rev $::current(DEFAULT_DID) $chip_rev]
      }        
              
   } else {
      set chip_num [reg read 0xA408]      
      set rdfFile [get_rdf_file_of_chip_num_and_rev $chip_num $chip_rev]                 
   }
   
   if {$rdfFile == ""} {   
         puts "ERROR: Unknown DID. Cannot select RDF file"
   }
}

#----------------------------------------------------------------------------
# Check if we are in E2 4 port mode
#----------------------------------------------------------------------------
proc is_4port_mode {} {
   global rdfFile
   
   #get RDF file name according to DID
   get_rdf_file          
   parse_rdf -f $rdfFile
   
   if {$::sys(NO_PCI) == 0} {
       if {[is_e1x]} {
          # if not E2/E3, not 4 port mode
          return 0
         }
   } else {
      #incase of NO_PCI mode
      set chip_num [reg read 0xA408]
      if {[is_e1x]} {
         # if not E2/E3, not 4 port mode
         return 0
      }      
   }   

   # ovwr_flag is determined by MISC_REG_PORT4MODE_EN_OVWR bit 0
   set ovwr_flag [reg_read -n MISC_REG_PORT4MODE_EN_OVWR]
             
   if {$ovwr_flag == 0} {
      # if overwrite flag is not set, read MISC_REG_PORT4MODE_EN
      if {[reg_read -n MISC_REG_PORT4MODE_EN] == 0x1} {
         return 1
      } else {
         return 0
      }
   } else {
      # if overwrite flag is set, read MISC_REG_PORT4MODE_EN_OVWR bit 1         
      return [expr {([reg_read -n MISC_REG_PORT4MODE_EN_OVWR] >> 1) & 1}]           
   }
     
}

#----------------------------------------------------------------------------
# Check if chip is E2
#----------------------------------------------------------------------------
proc is_e2 {} {
   if {$::sys(NO_PCI) == 0} {
      if {($::current(CHIP_NUM) == "57712") || ($::current(CHIP_NUM) == "57712E") ||
          ($::current(CHIP_NUM) == "57713") || ($::current(CHIP_NUM) == "57713E") } {
         return 1
      } else {
         return 0
      }
   } else {
      #incase of NO_PCI mode
      set chip_num [reg read 0xA408]
      if {($chip_num == 5713) || ($chip_num == 5714) ||
         ($chip_num == 5730) || ($chip_num == 5731)} {         
         return 1
      } else {
         return 0
      }
   }   
}

#----------------------------------------------------------------------------
# Check if chip is E3
#----------------------------------------------------------------------------
proc is_e3 {} {
   if {$::sys(NO_PCI) == 0} {
      if {($::current(CHIP_NUM) == "57810") || ($::current(CHIP_NUM) == "57840") || ($::current(CHIP_NUM) == "57800") || ($::current(CHIP_NUM) == "57811")} {
         return 1
      } else {
         return 0
      }
   } else {
      #incase of NO_PCI mode
      set chip_num [reg read 0xA408]
      if {($chip_num == 0x168d) || ($chip_num == 0x168e) || ($chip_num == 0x168a) ||
          ($chip_num == 0x16a5) || ($chip_num == 0x16ae) || ($chip_num == 0x16ab) ||
          ($chip_num == 0x16a9) || ($chip_num == 0x16af) || ($chip_num == 0x16ad) ||
          ($chip_num == 0x163d) || ($chip_num == 0x163e) || ($chip_num == 0x163f) ||
          ($chip_num == 0x16a1) || ($chip_num == 0x16a2)} {         
         return 1
      } else {
         return 0
      }
   }   
}

#----------------------------------------------------------------------------
# Check if chip is E1/E1H
#----------------------------------------------------------------------------
proc is_e1x {} {
   if {$::sys(NO_PCI) == 0} {
      if {($::current(CHIP_NUM) == "57710") || ($::current(CHIP_NUM) == "57711") || ($::current(CHIP_NUM) == "57711E")} {
         return 1
      } else {
         return 0
      }
   } else {
      #incase of NO_PCI mode
      set chip_num [reg read 0xA408]
      if {($chip_num == 0x164e) || ($chip_num == 0x164f) || ($chip_num == 0x1650)} {         
         return 1
      } else {
         return 0
      }
   }   
}

#----------------------------------------------------------------------------
# Check if board is 1-port board.
#----------------------------------------------------------------------------
proc is_one_port_board {} {
    
    # if total devices is 1, it means board is 1-port board
    if {$::current(TOTAL_DEV) == 1} {
        return 1
    }
    
    # Save the cuurent device and bus numbers
    set cur_dev $::current(DEV)
    set cur_bus $::current(BUS)
    set cur_func $::current(PCI_FUNC)
         
    # find next device
    if {$cur_dev < $::current(TOTAL_DEV)} {
        set next_dev [expr {$cur_dev + 1}]
    } else {
        # mark next_dev as invalid
        set next_dev 0
    }
    
    # find previous device   
    set prev_dev [expr {$cur_dev - 1}]     
        
    if {$next_dev > 0} {    
        device $next_dev -no_display      
        set next_bus $::current(BUS)
        set next_func $::current(PCI_FUNC)
    } else {
        set next_bus -1
    }
    
    if {$prev_dev > 0} {
        device $prev_dev -no_display      
        set prev_bus $::current(BUS)
        set prev_func $::current(PCI_FUNC)
    } else {
        set prev_bus -1
    }
    
    #return to current device
    device $cur_dev -no_display
     
    # if cur bus equals either next/prev bus then it is a 2-port board
    if {(($cur_bus == $next_bus) && ($next_func == $cur_func+1)) ||
         (($cur_bus == $prev_bus) && ($prev_func == $cur_func-1))} {        
        return 0
    } else {
        puts "Found 1-port board"        
        return 1
    }    
}

# Add help information on all importatnt commands in this script
parse_rdf_help
block_offset_help
reg_read_help
reg_offset_help
reg_size_help
reg_type_help
reset_chip_help
reset_value_help
read_regs_help
set_waitp_help
