###########################################################################
#
# Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
# Proprietary and Confidential Information.
#
# Module Description:
# This TCL script is for parsing shmem mailbox and checking if driver is loaded
###########################################################################
#
# Copyright(c) 2014 QLogic Corporation, all rights reserved
# Proprietary and Confidential Information.
#
# Module Description:
# This TCL script is for parsing shmem mailbox and checking if driver is loaded
# and if PMF.
#
###########################################################################

# Specify any dependency on other packages
package require Tcl
# Specify the package this module provides
package provide _show_pmf 1.0

namespace eval _show_pmf {
    
    # Export the package
    namespace export show_pmf
   
    interp alias {} show_pmf {}    "_show_pmf::show_pmf"
}
  

# -----------------------------------------------------------------
# get_shared_mem -
#
# Returns the shmem_base address according on the PF
# -----------------------------------------------------------------
proc _show_pmf::get_shared_mem {pf_num} {
    set MISC_REG_SHARED_MEM_ADDR  0xa2b4
    set_device $pf_num   
    set SharedMem   [reg read $MISC_REG_SHARED_MEM_ADDR]
    return $SharedMem
}


# -----------------------------------------------------------------
# PORT -
#
# Returns PORT of the pf_num
# -----------------------------------------------------------------
proc _show_pmf::PORT {pf_num} {
    if {([is_e2] || [is_e3])} {
        if {[is_4port_mode]} {
            return [expr ($pf_num >> 1) & 1]
        } else {
            return 0
        }
    } else {
        return [expr ($pf_num & 1)]
    }
}

# -----------------------------------------------------------------
# PATH -
#
# Returns PATH of the pf_num
# -----------------------------------------------------------------
proc _show_pmf::PATH {pf_num} {
    if {([is_e2] || [is_e3])} {
        return [expr ($pf_num & 1)]
    } else {
        return 0
    }
}

# -----------------------------------------------------------------
# set_device
#
# Goto the appropriate device based on the chip id, path and port
# -----------------------------------------------------------------
proc _show_pmf::set_device {pf_num} {

    set base_dev [expr $::current(DEV) - $::current(PCI_FUNC)]    

    if {([is_e2] || [is_e3])} {
        device [expr $base_dev + [PATH $pf_num]] -no_display
    } else {
        device [expr $base_dev + [PORT $pf_num]] -no_display
    }
}

proc _show_pmf::SHMEM_FUNC {pf_num} {
    if {[is_e1x]} {
        return $pf_num
    } else {
        return [expr $pf_num >> 1]
    }
}
# -----------------------------------------------------------------
# get_drv_func_mb_addr 
#
# Returns the drv_func_mb address according to path and pf_num
# -----------------------------------------------------------------
proc _show_pmf::get_drv_func_mb_addr {pf_num} {
    set drv_func_mbOffset    0x0684
    set drv_func_mbSize      44
    set SharedMem            [get_shared_mem $pf_num]
    set drv_func_mb_addr     [expr ($SharedMem + $drv_func_mbOffset + ([SHMEM_FUNC $pf_num] * $drv_func_mbSize))]
    #puts "drv_func_mb_addr (pf $pf_num) =  [format 0x%x $drv_func_mb_addr]"
    return $drv_func_mb_addr
}

# -----------------------------------------------------------------
# is_PMF
#
# Checks is driver status is PMF from the mailbox
# -----------------------------------------------------------------
proc _show_pmf::is_PMF {pf_num} {
    set drv_statusOffset     0x20
    set DRV_STATUS_PMF       0x00000001
    set mb_addr [get_drv_func_mb_addr $pf_num]
    set val [reg read [expr $mb_addr + $drv_statusOffset]]
    
    if {$val == $DRV_STATUS_PMF} {
        return 1
    } else {
        return 0
    }            
}


# -----------------------------------------------------------------
# is_PMF
#
# Checks is driver status is PMF from the mailbox
# -----------------------------------------------------------------

proc _show_pmf::get_last_command {pf_num} {

    set mb_addr [get_drv_func_mb_addr $pf_num]

    set val [expr [reg read $mb_addr] & 0xffff0000]
    
    switch [format "0x%08x" $val] {
        "0x10000000"    {return "DRV_MSG_CODE_LOAD_REQ"}
        "0x11000000"    {return "DRV_MSG_CODE_LOAD_DONE"}
        "0x20000000"    {return "DRV_MSG_CODE_UNLOAD_REQ_WOL_EN"}
        "0x20010000"    {return "DRV_MSG_CODE_UNLOAD_REQ_WOL_DIS"}
        "0x20020000"    {return "DRV_MSG_CODE_UNLOAD_REQ_WOL_MCP"}
        "0x21000000"    {return "DRV_MSG_CODE_UNLOAD_DONE"}
        "0x30000000"    {return "DRV_MSG_CODE_DCC_OK"}
        "0x31000000"    {return "DRV_MSG_CODE_DCC_FAILURE"}
        "0x50000000"    {return "DRV_MSG_CODE_DIAG_ENTER_REQ"}
        "0x60000000"    {return "DRV_MSG_CODE_DIAG_EXIT_REQ"}
        "0x70000000"    {return "DRV_MSG_CODE_VALIDATE_KEY"}
        "0x80000000"    {return "DRV_MSG_CODE_GET_CURR_KEY"}
        "0x81000000"    {return "DRV_MSG_CODE_GET_UPGRADE_KEY"}
        "0x82000000"    {return "DRV_MSG_CODE_GET_MANUF_KEY"}
        "0x90000000"    {return "DRV_MSG_CODE_LOAD_L2B_PRAM"}	
        "0xa0000000"    {return "DRV_MSG_CODE_VRFY_FIRST_PHY_OPT_MDL"}
        "0x00050006"    {return "REQ_BC_VER_4_VRFY_FIRST_PHY_OPT_MDL"}
        "0xa1000000"    {return "DRV_MSG_CODE_VRFY_SPECIFIC_PHY_OPT_MDL"}
        "0x00050234"    {return "REQ_BC_VER_4_VRFY_SPECIFIC_PHY_OPT_MDL"}
        "0xa2000000"    {return "DRV_MSG_CODE_VRFY_VNTAG_SUPPORTED"}
        "0x00070002"    {return "REQ_BC_VER_4_VRFY_VNTAG_SUPPORTED"}
        "0x00070014"    {return "REQ_BC_VER_4_SFP_TX_DISABLE_SUPPORTED"}
        "0x00070201"    {return "REQ_BC_VER_4_PFC_STATS_SUPPORTED"}
        "0xb0000000"    {return "DRV_MSG_CODE_DCBX_ADMIN_PMF_MSG"}
        "0xb2000000"    {return "DRV_MSG_CODE_DCBX_PMF_DRV_OK"}
        "0xc0000000"    {return "DRV_MSG_CODE_VF_DISABLED_DONE"}
        "0xd0000000"    {return "DRV_MSG_CODE_VNTAG_DRIVER_SETMAC"}
        "0xd1000000"    {return "DRV_MSG_CODE_VNTAG_LISTGET_ACK"}
        "0xd2000000"    {return "DRV_MSG_CODE_VNTAG_LISTSET_ACK"}
        "0xd3000000"    {return "DRV_MSG_CODE_VNTAG_STATSGET_ACK"}
        "0xd4000000"    {return "DRV_MSG_CODE_VNTAG_VIFSET_ACK"}
        "0xd8000000"    {return "DRV_MSG_CODE_DRV_INFO_ACK"}
        "0xd9000000"    {return "DRV_MSG_CODE_DRV_INFO_NACK"}
        "0xe0000000"    {return "DRV_MSG_CODE_SET_MF_BW"}
        "0x00060202"    {return "REQ_BC_VER_4_SET_MF_BW"}
        "0xe1000000"    {return "DRV_MSG_CODE_SET_MF_BW_ACK"}
        "0x01000000"    {return "DRV_MSG_CODE_LINK_STATUS_CHANGED"}
        "0xff010000"    {return "BIOS_MSG_CODE_LIC_CHALLENGE"}
        "0xff020000"    {return "BIOS_MSG_CODE_LIC_RESPONSE"}
        "0xff030000"    {return "BIOS_MSG_CODE_VIRT_MAC_PRIM"}
        "0xff040000"    {return "BIOS_MSG_CODE_VIRT_MAC_ISCSI"}
        "0x00000000"    {return "EMPTY"}
        default         {return "UNKNOWN COMMAND"}
    }    
}


proc _show_pmf::show_pmf {} {    
    
    set PF_NUM_MAX  8
    set cur_dev $::current(DEV)
    
    for {set pf_num 0} {$pf_num < $PF_NUM_MAX} {incr pf_num} {
        
        set is_pmf [is_PMF $pf_num]
        if {$is_pmf} {
            puts "PF=$pf_num: last_cmd=[get_last_command $pf_num], is PMF"
        } else {
            puts "PF=$pf_num: last_cmd=[get_last_command $pf_num], is not PMF"
        }
    }
   
    #return to original device
    device $cur_dev -no_display

}
