###########################################################################
#
# Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved
# Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
# Proprietary and Confidential Information.
#
# Module Description:
# configure vnic properties
#                                          
#
###########################################################################

global mcp_avail 
global NO_MCP_WA_CFG_SET_ADDR           
global NO_MCP_WA_CFG_SET_MAGIC          
global NO_MCP_WA_MULTI_VNIC_MODE        
global NO_MCP_WA_VNICS_PER_PORT_BASE    
global NO_MCP_WA_OVLAN_BASE             
global NO_MCP_WA_FORCE_5710_MODE        
global NO_MCP_WA_VALID_LIC_ADDR
global NO_MCP_WA_VALID_LIC_MAGIC
global NO_MCP_WA_TOE_LIC
global NO_MCP_WA_ISCSI_LIC
global NO_MCP_WA_RDMA_LIC

# -----------------------------------------------------------------
# Help info on "read_regs" command
# -----------------------------------------------------------------
proc vnic_cfg_help {args} {
    set help_info     "Configure vnic related features\n"
    append help_info  "Usage: vnic_cfg -mode <s/m/clp/5710> -vpp <port=vnicsPerPort...> -ovlan <funcId=ovlan funcId=ovlan...>\n"
    append help_info  "                -mac_lsb 0x<lsb> -config <funcId=ethernet/iscsi/rdma ...>\n"
    append help_info  "Notes: \n"
    append help_info  "-mode    single function or multiple function or clp mode or \"force 5710\" mode\n"
    append help_info  "-vpp     #vnics per port \"portId=#vnics portId=#vnics>\"\n"
    append help_info  "-ovlan   outer-vlans per funcId \"funcId=ovlan funcId=ovlan>\"\n"
    append help_info  "-num_vfs number of vfs per pf (optional, if not marked value will be 0)\n"
    append help_info  "-config  set function type (ethernet/rdma/iscsi) \"funcId=type funcId=type\"\n"
    append help_info  "-mac_lsb mac address LSB for all functions \"0x9a\"\n"
    append help_info  "-rdf     filename for rdf. default is everest.rdf\n"
    append help_info  "-display display the current configuration. \n\n"
    append help_info  "-min  set function min bandwidth \"funcId=value funcId=value\"\n"
    append help_info  "-max  set function max bandwidth \"funcId=value funcId=value\"\n"
    append help_info  "Example: vnic_cfg -mode m -vpp \"0=3 1=2\" -ovlan \"0=500 1=700 2=300\" -config \"0=ethernet 1=iscsi\" -min \"0=1 2=2 4=4\"\n"
    add_help vnic_cfg $help_info
}


# -----------------------------------------------------------------
# This proc sets the function mode single / multi
# -----------------------------------------------------------------
proc cfg_func_mode {funcMode} {
    global mcp_avail
    global NO_MCP_WA_MULTI_VNIC_MODE
    global NO_MCP_WA_FORCE_5710_MODE
    global MCP_PORT_MF_CFG_ADDR
    global MCP_PORT_MF_CFG_SIZE

    if {$funcMode == "m"} {
        puts "MULTI"
        set val 1
    } elseif {$funcMode == "s"} {
        puts "SINGLE"
        set val 0
    } elseif {$funcMode == "5710"} {
        puts "FORCE 5710"
        set val 2        
    } else {
        puts "Error: Function mode must be single (s) or multi (m)"
        return -1
    }
    if {$mcp_avail == 0} {
        if {$val == 2} {
            reg write $NO_MCP_WA_FORCE_5710_MODE 1        
        } else {
            reg write $NO_MCP_WA_MULTI_VNIC_MODE $val
            reg write $NO_MCP_WA_FORCE_5710_MODE 0        
        }               
    } else {
        if {$val == 2} {
            puts "Error: FORCE 5710 mode is not supported when MCP is up"
            return -1
        } elseif {$val == 1} {
            # in mf_mode, set per port e1hov tag value to some non default value
            for {set port 0} {$port < 2} {incr port} {                
                reg write [expr $MCP_PORT_MF_CFG_ADDR + $MCP_PORT_MF_CFG_SIZE*$port] [expr 0x10 + $port]
            }
        }
    }
}

# -----------------------------------------------------------------
# This proc sets the vnics-per-port setting. vnicsPerPort is expected to look
# like pairs separated by spaces, each pair is two numbers separated by '='
# -----------------------------------------------------------------
proc cfg_vpp {vnicsPerPort} {
    global mcp_avail
    global NO_MCP_WA_VNICS_PER_PORT_BASE
    global funcMode

    global MCP_FUNC_MF_CFG_SIZE    
    global MCP_FUNC_MF_CFG_CONFIG_ADDR
    global MCP_FUNC_MF_CFG_PFALLOCATION_ADDR
    global MCP_FUNC_MF_CFG_CONFIG_HIDE
    global MCP_FUNC_MF_CFG_CONFIG_DISABLED
    global MCP_FUNC_MF_CFG_CONFIG_DEFAULT 

    global num_vfs  

    # default values for vpps... 
    if {$funcMode == "m"} {
        set vpps(0) 4
        set vpps(1) 4
    } else {
        set vpps(0) 1
        set vpps(1) 1
    }
    
    
    set vppPairs [split $vnicsPerPort " "]
    if { [llength $vppPairs] > 2 } {
        puts "Error: only two ports!"
        return -1;
    }

    # Make sure we got a valid pairs... 
    foreach pair $vppPairs {
        set pairVal [split $pair "="]

        if { [llength $pairVal] != 2 } {
            puts "Error: bad pair ($pair)"
            return -1;
        }

        set portId  [lindex $pairVal 0]
        set vpp     [lindex $pairVal 1]

        if {$portId > 1} {
            puts "Error: PortID must be between 0-1 ($pair)"
            return -1;
        }

        if {$funcMode == "s"} {
            if {$vpp != 1} {
                puts "Error: Single Function mode, vnic per port can only be one"
                return -1;
            }
        } else {
            if {($vpp > 4) || ($vpp < 1)} {
                puts "Error: Between 1-4 vnics aloud per port"
                return -1;
            }
        }
        set vpps($portId) $vpp
    }

    foreach { port } [lsort [array names vpps]] {
        puts "Setting port $port with #Vnics per port: $vpps($port)"
        if {$mcp_avail == 0} {
            reg write [expr $NO_MCP_WA_VNICS_PER_PORT_BASE + $port*4] $vpps($port)
        } else {
            # set config per funtion according to vpps:            
            for {set func $port} {$func < 8} {incr func 2} {                
                set fconfig [reg read [expr ($MCP_FUNC_MF_CFG_CONFIG_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)]]
                if {$func < [expr $vpps($port) * 2]} {
                    set fconfig [expr (($fconfig & ~($MCP_FUNC_MF_CFG_CONFIG_HIDE | $MCP_FUNC_MF_CFG_CONFIG_DISABLED)) | $MCP_FUNC_MF_CFG_CONFIG_DEFAULT)]            
                } else {                
                    set fconfig [expr (($fconfig & ~$MCP_FUNC_MF_CFG_CONFIG_DEFAULT) | $MCP_FUNC_MF_CFG_CONFIG_HIDE)]
                }
                reg write [expr ($MCP_FUNC_MF_CFG_CONFIG_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)] $fconfig
                reg write [expr ($MCP_FUNC_MF_CFG_PFALLOCATION_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)] $num_vfs		
            }
        }
    }

    

}

# -----------------------------------------------------------------
# This proc sets the function configuration (ethernet/iscsi/rdma)
# funcConfig is a list of pairs separated by spaces, each pair is 
# in the form of number=type 
# -----------------------------------------------------------------
proc cfg_config {funcConfig} {
    global MCP_FUNC_MF_CFG_CONFIG_ETH
    global MCP_FUNC_MF_CFG_CONFIG_ETH_RDMA
    global MCP_FUNC_MF_CFG_CONFIG_ISCSI
    global MCP_FUNC_MF_CFG_CONFIG_FCOE
    global MCP_FUNC_MF_CFG_CONFIG_ADDR
    global MCP_FUNC_MF_CFG_SIZE
    global MCP_FUNC_MF_CFG_CONFIG_MASK

    # default mode for functions is ethernet
    array set config {}
    for {set i 0} {$i < 8} {incr i} {
        set config($i) $MCP_FUNC_MF_CFG_CONFIG_ETH
    }

    # parse input variable funcConfig
    foreach pair [split $funcConfig " "]  {
        set pairVal [split $pair "="]
        if { [llength $pairVal] != 2 } {
            error "cfg_config: bad pair ($pair) should be num=type"
        }
        set funcId [lindex $pairVal 0]
        set funcType [lindex $pairVal 1]
        if {$funcId > 7} {
            error "cfg_config: ($pair) funcID must be between 0-7"
        }
        switch -- "$funcType" {
            "ethernet"  { set cfg $MCP_FUNC_MF_CFG_CONFIG_ETH }
            "rdma"      { set cfg $MCP_FUNC_MF_CFG_CONFIG_ETH_RDMA }
            "iscsi"     { set cfg $MCP_FUNC_MF_CFG_CONFIG_ISCSI }
            "fcoe"      { set cfg $MCP_FUNC_MF_CFG_CONFIG_FCOE }
            default     { error "cfg_config: {$pair} invalid type: must be ehternet rdma or iscsi" }
        }
        set config($funcId) $cfg
    }

    # set function configuration
    for {set i 0} {$i < 8} {incr i} {
        set reg_addr [expr ($MCP_FUNC_MF_CFG_CONFIG_ADDR + $i * $MCP_FUNC_MF_CFG_SIZE)]
        set reg_value [expr ([reg read $reg_addr] & ~ $MCP_FUNC_MF_CFG_CONFIG_MASK) | $config($i)]
        reg write $reg_addr $reg_value
    }
}


# -----------------------------------------------------------------
# This proc sets the min configuration for function
# minConfig is a list of pairs separated by spaces, each pair is 
# in the form of number=number 
# -----------------------------------------------------------------
proc cfg_min {minConfig} {
    global MCP_FUNC_MF_CFG_CONFIG_ADDR
    global MCP_FUNC_MF_CFG_SIZE
    global MCP_FUNC_MF_CFG_MIN_BW_MASK
    global MCP_FUNC_MF_CFG_MIN_BW_SHIFT

    # default mode for functions is 0
    array set config {}
    for {set i 0} {$i < 8} {incr i} {
        set config($i) 0
    }

    # parse input variable minConfig
    foreach pair [split $minConfig " "]  {
        set pairVal [split $pair "="]
        if { [llength $pairVal] != 2 } {
            error "cfg_config: bad pair ($pair) should be num=minVal"
        }
        set funcId [lindex $pairVal 0]
        set minVal [lindex $pairVal 1]
        if {$funcId > 7} {
            error "cfg_config: ($pair) funcID must be between 0-7"
        }
        if {$minVal>100} {
            error "cfg_config: ($pair) minVal must be smaller than 100"
        }
        set config($funcId) $minVal
    }

    # set function configuration
    for {set i 0} {$i < 8} {incr i} {
        set reg_addr [expr ($MCP_FUNC_MF_CFG_CONFIG_ADDR + $i * $MCP_FUNC_MF_CFG_SIZE)]
        set reg_value [expr ([reg read $reg_addr] & ~ $MCP_FUNC_MF_CFG_MIN_BW_MASK) | ($config($i) << $MCP_FUNC_MF_CFG_MIN_BW_SHIFT)]
        reg write $reg_addr $reg_value
    }
}

# -----------------------------------------------------------------
# This proc sets the max configuration for function
# minConfig is a list of pairs separated by spaces, each pair is 
# in the form of number=number 
# -----------------------------------------------------------------
proc cfg_max {maxConfig} {
    global MCP_FUNC_MF_CFG_CONFIG_ADDR
    global MCP_FUNC_MF_CFG_SIZE
    global MCP_FUNC_MF_CFG_MAX_BW_MASK
    global MCP_FUNC_MF_CFG_MAX_BW_SHIFT

    # default max for functions is 100
    array set config {}
    for {set i 0} {$i < 8} {incr i} {
        set config($i) 100
    }

    # parse input variable minConfig
    foreach pair [split $maxConfig " "]  {
        set pairVal [split $pair "="]
        if { [llength $pairVal] != 2 } {
            error "cfg_config: bad pair ($pair) should be num=minVal"
        }
        set funcId [lindex $pairVal 0]
        set maxVal [lindex $pairVal 1]
        if {$funcId > 7} {
            error "cfg_config: ($pair) funcID must be between 0-7"
        }
        if {$maxVal>200} {
            error "cfg_config: ($pair) maxVal must be smaller than 200"
        }
        set config($funcId) $maxVal
    }

    # set function configuration
    for {set i 0} {$i < 8} {incr i} {
        set reg_addr [expr ($MCP_FUNC_MF_CFG_CONFIG_ADDR + $i * $MCP_FUNC_MF_CFG_SIZE)]
        set reg_value [expr ([reg read $reg_addr] & ~ $MCP_FUNC_MF_CFG_MAX_BW_MASK) | ($config($i) << $MCP_FUNC_MF_CFG_MAX_BW_SHIFT)]
        reg write $reg_addr $reg_value
    }
}


# -----------------------------------------------------------------
# This proc sets the outer-vlan setting. outerVlan is expected to look
# like pairs separated by spaces, each pair is two numbers separated by '='
# -----------------------------------------------------------------
proc cfg_ovlan {outerVlan} {
    global mcp_avail
    global NO_MCP_WA_OVLAN_BASE
    global funcMode
    
    global MCP_FUNC_MF_CFG_OVLAN_ADDR
    global MCP_FUNC_MF_CFG_OVLAN_INVALID
    global MCP_FUNC_MF_CFG_CONFIG_ADDR
    global MCP_FUNC_MF_CFG_SIZE
    global MCP_FUNC_MF_CFG_CONFIG_HIDE

     # default values for ovlans... 
    if {$funcMode == "m"} {
        for {set i 0} {$i < 8} {incr i} {
            set ovlans($i) [expr $i+ 1]
        }
    } else {
        for {set i 0} {$i < 8} {incr i} {
            set ovlans($i) $MCP_FUNC_MF_CFG_OVLAN_INVALID
        }
    }

    set ovlanPairs [split $outerVlan " "]
    if { [llength $ovlanPairs] > 8 } {
        puts "Error: Only up to 8 vnics allowed"
        return -1;
    }

    # Make sure we got a valid pairs... 
    foreach pair $ovlanPairs {
        set pairVal [split $pair "="]
        if { [llength $pairVal] != 2 } {
            puts "Error: bad pair ($pair)"
            return -1;
        }
        set funcId [lindex $pairVal 0]
        set ovlan [lindex $pairVal 1]
        if {$funcId > 7} {
            puts "Error: funcID must be between 0-7 ($pair)"
            return -1;
        }
        set ovlans($funcId) $ovlan
    }
    
    #write 'em
    foreach { func } [lsort [array names ovlans]] {        
        if {$mcp_avail == 0} {
            reg write [expr ($NO_MCP_WA_OVLAN_BASE + 0x$func*4)] $ovlans($func)
        } else {
            # disable ovlan on hidden function
            set fconfig [reg read [expr ($MCP_FUNC_MF_CFG_CONFIG_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)]]
            if {$fconfig & $MCP_FUNC_MF_CFG_CONFIG_HIDE} {
                set ovlans($func) $MCP_FUNC_MF_CFG_OVLAN_INVALID
            }
            reg write [expr ($MCP_FUNC_MF_CFG_OVLAN_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)] $ovlans($func)
        }
        puts "Setting func $func with outervlan $ovlans($func)"
    }

}

# -----------------------------------------------------------------
# This proc sets the mac addresses setting. macAddrLsb is 1 byte
# formatted 0x<LSB>
# -----------------------------------------------------------------
proc cfg_mac {macAddrLsb} {
    global mcp_avail
        
    global MCP_FUNC_MF_CFG_MACUPPER_ADDR
    global MCP_FUNC_MF_CFG_MACLOWER_ADDR
    global MCP_FUNC_MF_CFG_MACUPPER_INVALID
    global MCP_FUNC_MF_CFG_MACLOWER_INVALID
    global MCP_FUNC_MF_CFG_MACUPPER_DEFAULT
    global MCP_FUNC_MF_CFG_MACLOWER_DEFAULT
    global MCP_FUNC_MF_CFG_CONFIG_ADDR
    global MCP_FUNC_MF_CFG_OVLAN_ADDR
    global MCP_FUNC_MF_CFG_OVLAN_INVALID
    global MCP_FUNC_MF_CFG_SIZE
    global MCP_FUNC_MF_CFG_CONFIG_HIDE

    if {$macAddrLsb == -1} {
        # Use mac address from NVM
        if {[scan [nvm cfg 1=] "%x:%x:%x:%x:%x:%x" dmac0 dmac1 dmac2 dmac3 dmac4 dmac5] != 6} {
            error "cfg_mac: Cannot read MAC address from device $::current(DEV)"
        } else {
            set defMacHi 0x20
            set defMacLo [expr $dmac2 << 24 | $dmac3 << 16 | $dmac4 << 8 | $dmac5]
        }
    } elseif { $macAddrLsb > 0xff } {
        puts "Error: bad macAddrLsb ($macAddrLsb)"
        return -1;
    } else {
        set defMacHi $MCP_FUNC_MF_CFG_MACUPPER_DEFAULT
        set defMacLo [expr $MCP_FUNC_MF_CFG_MACLOWER_DEFAULT | $macAddrLsb << 8]
    }

     # set maclower per function
    for {set func 0} {$func < 8} {incr func} {        
        set fconfig [reg read [expr ($MCP_FUNC_MF_CFG_CONFIG_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)]]
        set ovlan [reg read [expr ($MCP_FUNC_MF_CFG_OVLAN_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)]]
        if {$fconfig & $MCP_FUNC_MF_CFG_CONFIG_HIDE || $ovlan == $MCP_FUNC_MF_CFG_OVLAN_INVALID} {
            # disable mac on hidden function, or if this is signle function mode
            set maclower $MCP_FUNC_MF_CFG_MACLOWER_INVALID
            set macupper $MCP_FUNC_MF_CFG_MACUPPER_INVALID
        } else {        
            set lsb2 [expr $func + 1]
            set maclower $defMacLo
            set macupper [expr $defMacHi + $func]
        }

        #write 'em
        puts "Setting func $func with mac: [hex16 $macupper] [hex32 $maclower]"
        reg write [expr ($MCP_FUNC_MF_CFG_MACLOWER_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)] $maclower
        reg write [expr ($MCP_FUNC_MF_CFG_MACUPPER_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)] $macupper
    }
}

# -----------------------------------------------------------------
# Check on what func mode we work on
# -----------------------------------------------------------------
proc cfg_check {} {
    global mcp_avail
    global NO_MCP_WA_MULTI_VNIC_MODE
    global NO_MCP_WA_FORCE_5710_MODE
    global NO_MCP_WA_CFG_SET_ADDR           
    global NO_MCP_WA_CFG_SET_MAGIC          

    global MCP_FUNC_MF_CFG_OVLAN_ADDR
    global MCP_FUNC_MF_CFG_SIZE
    global MCP_FUNC_MF_CFG_OVLAN_INVALID
    
    if {$mcp_avail == 0} {
        if {[reg read $NO_MCP_WA_CFG_SET_ADDR] != $NO_MCP_WA_CFG_SET_MAGIC} {
            puts "ERROR: mode isn't set yet..."
            return 0
        }
        if {[reg read $NO_MCP_WA_FORCE_5710_MODE]} {
            return "5710"
        } elseif {[reg read $NO_MCP_WA_MULTI_VNIC_MODE]} {
            return "m"
        } else {
            return "s"
        }
    } else {
        # Read ovlan of func 0:        
        set func 0
        set ovlan0 [reg read [expr ($MCP_FUNC_MF_CFG_OVLAN_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)]]       
        
        # Check which mode we're in
        # Assumption: function 0 is always enumerated!
        if {$ovlan0 == $MCP_FUNC_MF_CFG_OVLAN_INVALID} {
            return "s"
        } else {
            return "m"
        }        
    }
}

# -----------------------------------------------------------------
# Display the current configuration 
# -----------------------------------------------------------------
proc cfg_display {} {
    global mcp_avail
    global NO_MCP_WA_MULTI_VNIC_MODE
    global NO_MCP_WA_FORCE_5710_MODE
    global NO_MCP_WA_CFG_SET_ADDR           
    global NO_MCP_WA_CFG_SET_MAGIC          
    global NO_MCP_WA_VNICS_PER_PORT_BASE
    global NO_MCP_WA_OVLAN_BASE
    global NO_MCP_WA_VALID_LIC_ADDR         
    global NO_MCP_WA_VALID_LIC_MAGIC        
    global NO_MCP_WA_TOE_LIC                
    global NO_MCP_WA_ISCSI_LIC               
    global NO_MCP_WA_RDMA_LIC  

    global MCP_CLP_MB_ADDR
    global MCP_CLP_MB_NO_CLP
    global MCP_FUNC_MF_CFG_CONFIG_ADDR
    global MCP_FUNC_MF_CFG_CONFIG_HIDE
    global MCP_FUNC_MF_CFG_CONFIG_ETH
    global MCP_FUNC_MF_CFG_CONFIG_ETH_RDMA
    global MCP_FUNC_MF_CFG_CONFIG_ISCSI
    global MCP_FUNC_MF_CFG_CONFIG_FCOE

    global MCP_FUNC_MF_CFG_CONFIG_DISABLED
    global MCP_FUNC_MF_CFG_CONFIG_DEFAULT
    global MCP_FUNC_MF_CFG_CONFIG_MASK
    global MCP_FUNC_MF_CFG_OVLAN_ADDR
    global MCP_FUNC_MF_CFG_OVLAN_INVALID
    global MCP_FUNC_MF_CFG_MACUPPER_ADDR
    global MCP_FUNC_MF_CFG_MACLOWER_ADDR
    global MCP_FUNC_MF_CFG_PFALLOCATION_ADDR
    global MCP_FUNC_MF_CFG_SIZE

    global MCP_FUNC_MF_CFG_MIN_BW_MASK   
    global MCP_FUNC_MF_CFG_MIN_BW_SHIFT  
    global MCP_FUNC_MF_CFG_MIN_BW_DEFAULT
    global MCP_FUNC_MF_CFG_MAX_BW_MASK   
    global MCP_FUNC_MF_CFG_MAX_BW_SHIFT  
    global MCP_FUNC_MF_CFG_MAX_BW_DEFAULT

    if {$mcp_avail == 0} {
        if {[reg read $NO_MCP_WA_CFG_SET_ADDR] != $NO_MCP_WA_CFG_SET_MAGIC} {
            puts "ERROR: mode isn't set yet..."
            return 0
        }
        # Check which mode we're in
        if {[reg read $NO_MCP_WA_FORCE_5710_MODE]} {
            puts "MODE: 5710"
        } elseif {[reg read $NO_MCP_WA_MULTI_VNIC_MODE]} {
            puts "MODE: multi-function-mode"
        } else {
            puts "MODE: single-function-mode"
        }
        # Check how many vnics-per-port we have
        for {set port 0} {$port < 2} {incr port} {
            puts "Port $port is configured with [reg read [expr $NO_MCP_WA_VNICS_PER_PORT_BASE + $port*4]] vnics per port"            
        }
        # check outervlan values
        for {set func 0} {$func < 8} {incr func} {
            puts "Func $func is configured with ovlan [reg read [expr ($NO_MCP_WA_OVLAN_BASE + 0x$func*4)]]"
        }
        #check if license is set, if so display otherwise just notify there is no license
        for {set port 0} {$port < 2} {incr port} {
            if {[reg read $NO_MCP_WA_VALID_LIC_ADDR] != $NO_MCP_WA_VALID_LIC_MAGIC} {
                puts "No licensing info for port $port"
            } else {
                puts "Licensing info for port $port:"
                puts "Max Toe Conn (65535 for unlimited)  : [reg read [expr $NO_MCP_WA_TOE_LIC + $port*4]]"
                puts "Max iScsi Conn (65535 for unlimited): [reg read [expr $NO_MCP_WA_ISCSI_LIC + $port*4]]"
                puts "Max Rdma Conn (65535 for unlimited) : [reg read [expr $NO_MCP_WA_RDMA_LIC + $port*4]]"
            }
        }
    } else {
        puts "MCP up detected!"

        # Read CLP mode
        set clp_mb [reg read $MCP_CLP_MB_ADDR]
        if {$clp_mb & $MCP_CLP_MB_NO_CLP} {
            puts "CLP MODE OFF detected"
        } else {
            puts "CLP MODE ON detected"
        }   

        # Read information per function:
        for {set func 0} {$func < 8} {incr func} {
            set ovlans($func) [reg read [expr ($MCP_FUNC_MF_CFG_OVLAN_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)]]
            set config($func) [reg read [expr ($MCP_FUNC_MF_CFG_CONFIG_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)]]	    
            set maclower($func) [reg read [expr ($MCP_FUNC_MF_CFG_MACLOWER_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)]]
            set macupper($func) [reg read [expr ($MCP_FUNC_MF_CFG_MACUPPER_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)]]
            set vfs($func) [reg read [expr ($MCP_FUNC_MF_CFG_PFALLOCATION_ADDR + 0x$func*$MCP_FUNC_MF_CFG_SIZE)]]
        }

        # Check which mode we're in
        # Assumption: function 0 is always enumerated!
        if {$ovlans(0) == $MCP_FUNC_MF_CFG_OVLAN_INVALID} {
            puts "MODE: single-function-mode"
        } else {
            puts "MODE: multi-function-mode"
        }   

        # Check how many vnics-per-port we have
        for {set port 0} {$port < 2} {incr port} {
            set vpp 4
            for {set func $port} {$func < 8} {incr func 2} {
                if {$config($func) & $MCP_FUNC_MF_CFG_CONFIG_HIDE} {
                    #function is hidden
                    incr vpp -1
                }
            }
            puts "Port $port is configured with $vpp vnics per port"            
        }

        # dump per function info 
        for {set func 0} {$func < 8} {incr func} {
            set conf [expr $config($func)]
            set conf_value [expr $conf & $MCP_FUNC_MF_CFG_CONFIG_MASK]
            set cfg_str "<INVALID>"
            if {$conf_value == "$MCP_FUNC_MF_CFG_CONFIG_ETH"} {
                set cfg_str "ETH     "
            }
            if {$conf_value == $MCP_FUNC_MF_CFG_CONFIG_ETH_RDMA} {
                set cfg_str "ETH_RDMA"
            }
            if {$conf_value == $MCP_FUNC_MF_CFG_CONFIG_ISCSI} {
                set cfg_str "ISCSI   "
            }
            if {$conf_value == $MCP_FUNC_MF_CFG_CONFIG_FCOE} {
                set cfg_str "FCOE    "
            }
            if {$conf & $MCP_FUNC_MF_CFG_CONFIG_DISABLED} {
                set cfg_str "$cfg_str;DISABLED"
            }
            if {$conf & $MCP_FUNC_MF_CFG_CONFIG_HIDE} {
                set cfg_str "HIDDEN  "
            }
            set mac [format "%02X:%02X:%02X:%02X:%02X:%02X" [expr $macupper($func) >> 8] [expr $macupper($func) & 0xff] \
                [expr $maclower($func) >> 24 & 0xff] [expr $maclower($func) >> 16 & 0xff]   \
                [expr $maclower($func) >> 8 & 0xff] [expr $maclower($func) & 0xff] ]
            set bw_lo [expr (($conf & $MCP_FUNC_MF_CFG_MIN_BW_MASK) >> $MCP_FUNC_MF_CFG_MIN_BW_SHIFT) & 0xff]   
            set bw_hi [expr (($conf & $MCP_FUNC_MF_CFG_MAX_BW_MASK) >> $MCP_FUNC_MF_CFG_MAX_BW_SHIFT) & 0xff]
            set num_vfs [expr ($vfs($func) & 0xff)]	    
            puts "Func $func: $cfg_str ovlan=$ovlans($func) mac=$mac BW=($bw_hi,$bw_lo) vfs=$num_vfs" 
        }

        #BUGBUG - TBA licnence/bw/etc ....
        return 0
    }
}

# -----------------------------------------------------------------
# Configure licenseing issues... 
# -----------------------------------------------------------------
proc cfg_license {license} {
    global mcp_avail
    global NO_MCP_WA_VALID_LIC_ADDR         
    global NO_MCP_WA_VALID_LIC_MAGIC        
    global NO_MCP_WA_TOE_LIC                
    global NO_MCP_WA_ISCSI_LIC               
    global NO_MCP_WA_RDMA_LIC               

    set toeCons 2000
    set iscsiCons 2
    set rdmaCons 2

    set toeConsSet 0
    set iscsiConsSet 0
    set rdmaConsSet 0

    if {$mcp_avail == 0} {
        puts "license is $license"

        # parse the license string
        set licensePairs [split $license " "]

        set portId  [lindex $licensePairs 0]

        foreach pair $licensePairs {
            set licenseVal [split $pair "="]
    
            if { [llength $licenseVal] == 2 } {
                set proto  [lindex $licenseVal 0]
                set num    [lindex $licenseVal 1]    
    
                if {$num > 0xffff} {
                    puts "Error: #connections can't be larger than 0xffff"
                    return -1
                }
    
                if {$proto == "toe"} {
                    if {$toeConsSet} {
                        puts "Error: Toe connections given more than once\n"
                        return -1
                    }
                   set toeCons $num
                   set toeConsSet 1
                } elseif {$proto == "iscsi"} {
                    if {$iscsiConsSet} {
                        puts "Error: iScsi connections given more than once\n"
                        return -1
                    }
                    set iscsiCons $num
                    set iscsiConsSet 1
                } elseif {$proto == "rdma"} {
                    if {$rdmaConsSet} {
                        puts "Error: Rdma connections given more than once\n"
                        return -1
                    }
                    set rdmaCons $num
                    set rdmaConsSet 1
                } else {
                    puts "Error: unknown protocol: $proto"
                    return -1
                }
            }
        }
       
        reg write [expr $NO_MCP_WA_TOE_LIC + $portId*4]        $toeCons
        reg write [expr $NO_MCP_WA_ISCSI_LIC + $portId*4]      $iscsiCons
        reg write [expr $NO_MCP_WA_RDMA_LIC + $portId*4]       $rdmaCons
        reg write [expr $NO_MCP_WA_VALID_LIC_ADDR + $portId*4] $NO_MCP_WA_VALID_LIC_MAGIC
    } else {
        puts "ERROR: Script doesn't work with MCP yet..."
        return 0
    }

}

# -----------------------------------------------------------------
# Main procedure for this file: vnic_cfg.
# -----------------------------------------------------------------
proc vnic_cfg {args} {
    global rdfFile
    global mcp_avail
    global NO_MCP_WA_CFG_SET_ADDR           
    global NO_MCP_WA_CFG_SET_MAGIC          
    global NO_MCP_WA_MULTI_VNIC_MODE        
    global NO_MCP_WA_VNICS_PER_PORT_BASE    
    global NO_MCP_WA_OVLAN_BASE             
    global NO_MCP_WA_FORCE_5710_MODE
    global NO_MCP_WA_VALID_LIC_ADDR         
    global NO_MCP_WA_VALID_LIC_MAGIC        
    global NO_MCP_WA_TOE_LIC                
    global NO_MCP_WA_ISCSI_LIC               
    global NO_MCP_WA_RDMA_LIC   

    global MCP_SHMEM_BASE
    global MCP_CLP_MB_ADDR
    global MCP_CLP_MB_NO_CLP
    global MCP_PORT_MF_CFG_ADDR
    global MCP_FUNC_MF_CFG_CONFIG_ADDR
    global MCP_FUNC_MF_CFG_PFALLOCATION_ADDR
    global MCP_FUNC_MF_CFG_CONFIG_HIDE
    global MCP_FUNC_MF_CFG_CONFIG_ETH
    global MCP_FUNC_MF_CFG_CONFIG_ETH_RDMA
    global MCP_FUNC_MF_CFG_CONFIG_ISCSI
    global MCP_FUNC_MF_CFG_CONFIG_FCOE
    global MCP_FUNC_MF_CFG_CONFIG_DISABLED
    global MCP_FUNC_MF_CFG_CONFIG_DEFAULT
    global MCP_FUNC_MF_CFG_CONFIG_MASK
    global MCP_FUNC_MF_CFG_OVLAN_ADDR
    global MCP_FUNC_MF_CFG_OVLAN_INVALID
    global MCP_FUNC_MF_CFG_MACUPPER_OFFSET  
    global MCP_FUNC_MF_CFG_MACUPPER_ADDR
    global MCP_FUNC_MF_CFG_MACUPPER_DEFAULT
    global MCP_FUNC_MF_CFG_MACUPPER_INVALID
    global MCP_FUNC_MF_CFG_MACLOWER_OFFSET  
    global MCP_FUNC_MF_CFG_MACLOWER_ADDR
    global MCP_FUNC_MF_CFG_MACLOWER_DEFAULT 
    global MCP_FUNC_MF_CFG_MACLOWER_INVALID
    global MCP_PORT_MF_CFG_SIZE
    global MCP_FUNC_MF_CFG_SIZE
    global MCP_FUNC_MF_CFG_MIN_BW_MASK   
    global MCP_FUNC_MF_CFG_MIN_BW_SHIFT  
    global MCP_FUNC_MF_CFG_MIN_BW_DEFAULT
    global MCP_FUNC_MF_CFG_MAX_BW_MASK   
    global MCP_FUNC_MF_CFG_MAX_BW_SHIFT  
    global MCP_FUNC_MF_CFG_MAX_BW_DEFAULT
    global MCP_FUNC_MF_CFG_PROTOCOL_MASK

    global funcMode
    global num_vfs

    set NO_MCP_WA_CFG_SET_ADDR           0xA0000
    set NO_MCP_WA_CFG_SET_MAGIC          0x88AA55FF
    set NO_MCP_WA_MULTI_VNIC_MODE        0xA0004
    set NO_MCP_WA_VNICS_PER_PORT_BASE    0xA0008 
    set NO_MCP_WA_OVLAN_BASE             0xA0010
    set NO_MCP_WA_FORCE_5710_MODE        0xA0030
    set NO_MCP_WA_VALID_LIC_ADDR         0xA0040
    set NO_MCP_WA_VALID_LIC_MAGIC        0xCCAAFFEE
    set NO_MCP_WA_TOE_LIC                0xA0048
    set NO_MCP_WA_ISCSI_LIC              0xA0050
    set NO_MCP_WA_RDMA_LIC               0xA0058

    # hard coded offsets within shmem structure - to be asserted statically in driver
    set MCP_SHMEM_BASE_DEFAULT           0xaf900
    set MCP_MF_CFG_OFFSET                0x7e4

    set MCP_MF_CFG_CLP_MB_OFFSSET        0
    set MCP_MF_CFG_PORT_CONFIG_OFFSET    4
    set MCP_CLP_MB_NO_CLP                0x80000000

    set MCP_MF_CFG_FUNC_MF_CFG_OFFSET    36

    set MCP_FUNC_MF_CFG_CONFIG_OFFSET    0
    set MCP_FUNC_MF_CFG_CONFIG_HIDE      0x1
    set MCP_FUNC_MF_CFG_CONFIG_FCOE      0x0
    set MCP_FUNC_MF_CFG_CONFIG_ETH       0x2
    set MCP_FUNC_MF_CFG_CONFIG_ETH_RDMA  0x4
    set MCP_FUNC_MF_CFG_CONFIG_ISCSI     0x6
    set MCP_FUNC_MF_CFG_CONFIG_DISABLED  0x8
    set MCP_FUNC_MF_CFG_CONFIG_DEFAULT   0x4
    set MCP_FUNC_MF_CFG_CONFIG_MASK      0x6
    set MCP_FUNC_MF_CFG_PROTOCOL_MASK    0x7
    set MCP_FUNC_MF_CFG_MACUPPER_OFFSET  4
    set MCP_FUNC_MF_CFG_MACUPPER_DEFAULT 0x00000050
    set MCP_FUNC_MF_CFG_MACUPPER_INVALID 0x0000ffff
    set MCP_FUNC_MF_CFG_MACLOWER_OFFSET  8
    set MCP_FUNC_MF_CFG_MACLOWER_DEFAULT 0xc22c0000
    set MCP_FUNC_MF_CFG_MACLOWER_INVALID 0xffffffff
    set MCP_FUNC_MF_CFG_PFALLOCATION_OFFSET  20

    set MCP_FUNC_MF_CFG_MIN_BW_MASK          0x00ff0000
    set MCP_FUNC_MF_CFG_MIN_BW_SHIFT         16
    set MCP_FUNC_MF_CFG_MIN_BW_DEFAULT       0x00000000
    set MCP_FUNC_MF_CFG_MAX_BW_MASK          0xff000000
    set MCP_FUNC_MF_CFG_MAX_BW_SHIFT         24
    set MCP_FUNC_MF_CFG_MAX_BW_DEFAULT       0x64000000

    set MCP_FUNC_MF_CFG_OVLAN_OFFSET     12
    set MCP_FUNC_MF_CFG_OVLAN_INVALID    0xffff

    set MCP_FUNC_MF_CFG_SIZE             24
    set MCP_PORT_MF_CFG_SIZE             16
    

    set funcMode "m"
    set vnicsPerPort ""
    set outerVlan ""
    set funcConfig ""
	set minConfig ""
	set maxConfig ""
    set macAddrLsb -1
    
    #get RDF file name according to DID
    get_rdf_file
    
    # if no arguments, print the usage
    if {[llength $args] == 0} {
        vnic_cfg_usage  
        return -1
    }
    set display 0
    set check 0	
    set has_license 0
    set num_vfs 0
    
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
                "-mode"         { incr i; set funcMode [lindex $args $i]}
                "-vpp"          { incr i; set vnicsPerPort [lindex $args $i]}
                "-ovlan"        { incr i; set outerVlan [lindex $args $i]}
                "-mac_lsb"      { incr i; set macAddrLsb [lindex $args $i]}
                "-config"       { incr i; set funcConfig [lindex $args $i]}
                "-min"          { incr i; set minConfig [lindex $args $i]}
                "-max"          { incr i; set maxConfig [lindex $args $i]}
                "-rdf"          { incr i; set rdfFile [lindex $args $i]}
                "-license"      { incr i; set license [lindex $args $i]; set has_license 1}
                "-num_vfs"      { incr i; set num_vfs [lindex $args $i]}
                "-check"        { set check 1 }
                "-display"      { set display 1 }
                "-help"         { vnic_cfg_usage; return -1 }
                default         { vnic_cfg_usage; return -1 }
         }
    }   
        
    #  if there is MCP - script acts differentaly...

    parse_rdf -f $rdfFile
    set offset [reg_offset -n "MISC_REG_SHARED_MEM_ADDR"]
    set MCP_SHMEM_BASE [reg read $offset]
    set offset [reg_offset -n "MISC_REG_GENERIC_CR_0"]
    set MCP_SHMEM2_BASE [reg read $offset]
    if {($MCP_SHMEM2_BASE >= 0x3a0000) && ($MCP_SHMEM2_BASE < 0x3c8000) && ([reg read $MCP_SHMEM2_BASE] > 8)} {
        set mcp_avail 1
        set MF_CFG_BASE [reg read [expr $MCP_SHMEM2_BASE + 16]]
    } elseif {($MCP_SHMEM_BASE >= 0xa0000) && ($MCP_SHMEM_BASE < 0xc0000)} {
        set mcp_avail 1                
        set MF_CFG_BASE [expr $MCP_SHMEM_BASE + $MCP_MF_CFG_OFFSET]
    } else {
        set mcp_avail 0
        set MF_CFG_BASE [expr $MCP_SHMEM_BASE_DEFAULT + $MCP_MF_CFG_OFFSET]
    }    
    
    set MCP_CLP_MB_ADDR                 [expr $MF_CFG_BASE + $MCP_MF_CFG_CLP_MB_OFFSSET]
    set MCP_PORT_MF_CFG_ADDR            [expr $MF_CFG_BASE + $MCP_MF_CFG_PORT_CONFIG_OFFSET]
    set MCP_FUNC_MF_CFG_CONFIG_ADDR     [expr $MF_CFG_BASE + $MCP_MF_CFG_FUNC_MF_CFG_OFFSET + $MCP_FUNC_MF_CFG_CONFIG_OFFSET]
    set MCP_FUNC_MF_CFG_PFALLOCATION_ADDR     [expr $MF_CFG_BASE + $MCP_MF_CFG_FUNC_MF_CFG_OFFSET + $MCP_FUNC_MF_CFG_PFALLOCATION_OFFSET]
    set MCP_FUNC_MF_CFG_OVLAN_ADDR      [expr $MF_CFG_BASE + $MCP_MF_CFG_FUNC_MF_CFG_OFFSET + $MCP_FUNC_MF_CFG_OVLAN_OFFSET]        
    set MCP_FUNC_MF_CFG_MACUPPER_ADDR   [expr $MF_CFG_BASE + $MCP_MF_CFG_FUNC_MF_CFG_OFFSET + $MCP_FUNC_MF_CFG_MACUPPER_OFFSET]        
    set MCP_FUNC_MF_CFG_MACLOWER_ADDR   [expr $MF_CFG_BASE + $MCP_MF_CFG_FUNC_MF_CFG_OFFSET + $MCP_FUNC_MF_CFG_MACLOWER_OFFSET]        

    if {$display} {
       cfg_display
       return 0
    }

    if {$check} {
        return [cfg_check]
    }

    if {$mcp_avail} {
        if {$funcMode == "clp"} {            
            # RMW clp mb
            reg write $MCP_CLP_MB_ADDR [expr ([reg read $MCP_CLP_MB_ADDR] & ~$MCP_CLP_MB_NO_CLP)]    
            puts "CLP mode turned ON"
            return 0;
        } else {
            # RMW clp mb
            reg write $MCP_CLP_MB_ADDR [expr ([reg read $MCP_CLP_MB_ADDR] | $MCP_CLP_MB_NO_CLP)]    
        }        
    }

    cfg_func_mode $funcMode
    cfg_vpp $vnicsPerPort
    cfg_ovlan $outerVlan
    cfg_config $funcConfig
    cfg_min $minConfig
    cfg_max $maxConfig

    if {$mcp_avail} {
        cfg_mac $macAddrLsb
    }

    if {$has_license} {
        cfg_license $license
    }

    if {$mcp_avail == 0} {
        reg write $NO_MCP_WA_CFG_SET_ADDR $NO_MCP_WA_CFG_SET_MAGIC
    }
    
}

# -----------------------------------------------------------------
# This proc prints out the vnic_cfg proc usage information
# -----------------------------------------------------------------
proc vnic_cfg_usage {}  {
    
    puts stdout "Usage: vnic_cfg  -mode    <s/m/clp/5710>        :set single or multi func mode or CLP mode or force 5710 mode"
    puts stdout "                 -vpp     <Id=#vnics Id=#vnics> :set # vnics per port"
    puts stdout "                 -ovlan   <Id=ovlan Id=vlan...> :set outervlan per funcId. "
    puts stdout "                 -config  <Id=type Id=type...>  :set function type (ethernet/rdma/iscsi/fcoe)"
    puts stdout "                 -min     <Id=value Id=value...>:set function min bandwitch"
    puts stdout "                 -max     <Id=value Id=value...>:set function max bandwitch"
    puts stdout "                 -mac_lsb <0x<lsb>>             :set mac lsb for all funcs. "
    puts stdout "                 -rdf     <rdf file>            :default is everest.rdf. "
    puts stdout "                 -display                       :display the current configuration. " 
    
}

# -----------------------------------------------------------------
# This proc changes board mode to NPAR like performed by MBA/CCM
# -----------------------------------------------------------------
proc set_npar_mode {{fcoe0 -1} {fcoe1 -1}} {
    #check inputs are valid
    if {($fcoe0 > 7) || ($fcoe1 > 7)} {
        puts "fcoe functions must be between 0 to 7"
        return -1
    }
    if {($fcoe0 > 0) && ($fcoe1 > 0)} {
        set fcoe0_port [expr $fcoe0 & 1]
        set fcoe1_port [expr $fcoe1 & 1]
        if {$fcoe0_port == $fcoe1_port} {
            puts "two fcoe functions must be from different ports"
            return -1
        }
    }

    #change mode to NPAR
    nvm cfg 73=3  

    set ::current(IS_SPECIAL_NVM_MODE) 1
    set ::current(SPECIAL_NVM_MODE_FUNC_NUM) 0

    #read MAC from NVRAM, to decide which MAC addresses to write to the new image
    scan [nvm cfg 1=] "%x:%x:%x:%x:%x:%x" dmac0 dmac1 dmac2 dmac3 dmac4 dmac5

    for {set i 0} {$i < 2} {incr i} {
        set ::current(SPECIAL_NVM_MODE_FUNC_NUM) $i
        nvm cfg 115=0       
    }

    for {set i 0} {$i < 8} {incr i} {
        set func_cfg 0x3
        set ::current(SPECIAL_NVM_MODE_FUNC_NUM) $i

        # mark FCoE enabled according to input to this function (disabled by default)
        if {($fcoe0 == $i) || ($fcoe1 == $i)} {
            set func_cfg [expr $func_cfg | 0x8]
        }

        # mark iSCSI offload enabled only for functions 0/1
        if {$i <= 1} {
            set func_cfg [expr $func_cfg | 0x4]
        }

        nvm cfg 118=[format %x $func_cfg] 119=0 120=100 121=[format %02x $dmac0]:[format %02x $dmac1]:[format %02x $dmac2]:[format %02x $dmac3]:[format %02x $dmac4]:[format %02x [expr $dmac5 + 2*$i]] 122=[format %02x $dmac0]:[format %02x $dmac1]:[format %02x $dmac2]:[format %02x $dmac3]:[format %02x $dmac4]:[format %02x [expr $dmac5 + 2*$i + 1]] 123=[format %02x $dmac0]:[format %02x $dmac1]:[format %02x $dmac2]:[format %02x $dmac3]:[format %02x $dmac4]:[format %02x [expr $dmac5 + 2*$i + 1]] 136=10:00:[format %02x $dmac0]:[format %02x $dmac1]:[format %02x $dmac2]:[format %02x $dmac3]:[format %02x $dmac4]:[format %02x [expr $dmac5 + 2*$i + 1]] 137=20:00:[format %02x $dmac0]:[format %02x $dmac1]:[format %02x $dmac2]:[format %02x $dmac3]:[format %02x $dmac4]:[format %02x [expr $dmac5 + 2*$i + 1]]       
    }

    set ::current(IS_SPECIAL_NVM_MODE) 0
}

# -----------------------------------------------------------------
# This proc Enables SRIOV mode and set #of VF according to NPAR/SF mode & #of ports
# -----------------------------------------------------------------
proc en_sriov {} {

    #Enable SRIOV mode
    nvm cfg 91=1
    #set the IS_SPECIAL_NVM_MODE to 1 in order to write on other functions regardless of MF support
    set ::current(IS_SPECIAL_NVM_MODE) 1
    set num_of_ports [_access_phy::PORT_MODE]
    set npar_mode [nvm cfg 73=]

    #In case of SF num of VF per PF in SF mode dependes on the 2/4 port mode configuration.
    #Number of allowed VF is 64 per engine, hence if 2 port mode allow 64 per function if 4 port allow 32 per function
    if {$npar_mode == 1} {
            puts "SF configuration is presernt"
            for {set i 0} {$i < 8} {incr i} {
                set ::current(SPECIAL_NVM_MODE_FUNC_NUM) $i        
                if {$num_of_ports == 2} {
                        nvm cfg 190=64	
                } elseif {$num_of_ports == 4} {
                        nvm cfg 190=32
                } else {
                        puts "Error # of ports not supported"
                }
            }    
    #In switch independent MF mode we need to set nvm cfg 191 to 16.
    } elseif {$npar_mode == 3} {
        nvm cfg 190=16
        puts "switch independent MF configuration is present"
    }
                
    set ::current(IS_SPECIAL_NVM_MODE) 0
}

# -----------------------------------------------------------------
# This proc checks NPAR1.5
# -----------------------------------------------------------------
proc check_npar1_5 { {args}} {
    
    set stride 8
        
    # Parse the options.
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
                "-stride"       { incr i; set stride [lindex $args $i]}               
                default         { "check_npar1_5 -stride 8 or 16"; return -1 }
         }
    }   
    
    if {($stride != 8) && ($stride != 16)} {
        puts "Stride should be 8 or 16"
        return -1
    }
    
    # Difference between MAC address should be +4 (when stride is 8)
    # or +8 (when stride is 16)
    set mac_diff [expr {$stride / 2}]
    
    set ::current(IS_SPECIAL_NVM_MODE) 1   

    #read MAC from NVRAM, to decide which MAC addresses to write to the new image
    scan [nvm cfg 1=] "%x:%x:%x:%x:%x:%x" prim_mac0 prim_mac1 prim_mac2 prim_mac3 prim_mac4 prim_mac5
    
    set prim_mac_lo [expr {($prim_mac3 << 16) | ($prim_mac4 << 8) | $prim_mac5}]  
    
    for {set i 0} {$i < 8} {incr i} {    
            
        set ::current(SPECIAL_NVM_MODE_FUNC_NUM) $i
        scan [nvm cfg 121=] "%x:%x:%x:%x:%x:%x" mac0 mac1 mac2 mac3 mac4 mac5
        
        if {($mac0 != $prim_mac0) || ($mac1 != $prim_mac1) || ($mac2 != $prim_mac2)} {
            puts "MAC address prefix at pf# $i is different \
            [format "0x%X" $mac0] : [format "0x%X" $mac1] : [format "0x%X" $mac2] \
             than expected: [format "0x%X" $prim_mac0] : [format "0x%X" $prim_mac1] : [format "0x%x" $prim_mac2]"
            set ::current(IS_SPECIAL_NVM_MODE) 0
            return -1
        }
        
        set mac_lo [expr {($mac3 << 16) | ($mac4 << 8) | $mac5}]

        #Verify MAC address is primary_mac + (func%2)*mac_diff + (func/2)
        # func  mac
        #  0    primary_mac
        #  1    primary_mac+4
        #  2    primary_mac+1
        #  3    primary_mac+5
        #  4    primary_mac+2
        #  5    primary_mac+6
        #  6    primary_mac+3
        #  7    primary_mac+7        
        if {$mac_lo != [expr {$prim_mac_lo + ($i%2) * $mac_diff + $i/2}]} {
                puts "PF# $i: Low MAC address is different [format "0x%X" $mac_lo] \
                than expected [format "0x%X" [expr {$prim_mac_lo + ($i%2) * $mac_diff + $i/2}]]"
                set ::current(IS_SPECIAL_NVM_MODE) 0
                return -1
        }                        
    }

    set ::current(SPECIAL_NVM_MODE_FUNC_NUM) 0
    set ::current(IS_SPECIAL_NVM_MODE) 0
    puts "NPAR1.5 check completed successfully !"
    return 0
}

# Add help information on all importatnt commands in this script
vnic_cfg_help

