######################################################################
# Switch between 1G and 10G AN
# Driver on both sides needs to be loaded
######################################################################
proc igxg_an {iterations {to_sec 10}} {
    for {set i 0} {$i < $iterations} {incr i} {
        puts "$i, 1000M"
        # Disable 10G
        extphy 1 7 0x20 0x1
        extphy 1 7 0 0x3200
        set x [wait_for_link $to_sec]
        if {$x != "1000M"} {
            error "Got $x instead of 1000M"
        }
        delay ms 1000
        puts "Passed: got $x"

        puts "$i, 10G"
        # Enable 10G
        extphy 1 7 0x20 0x1001
        extphy 1 7 0 0x3200
        set x [wait_for_link $to_sec]
        if {$x != "10G"} {
            error "Got $x instead of 10G"
        }
        puts "Passed: got $x"
        #delay ms 1000
    }
}
proc wait_for_link {to_sec} {
    set to_msec [expr $to_sec * 1000]
    set interval 500
    for {set idx 0} {$idx < $to_msec} {incr idx $interval} {
        set x [report_link]
        if {($x != "DOWN") && ($x != "UNKNOWN")} {
            puts "Got link in $idx msec"
            return $x
        }
        delay ms $interval
    }
    return $x
}
proc report_link {} {
    set x [extphy 1 7 0xffe1]
    if {($x & (1 << 2)) == 0} {
        return "DOWN"
    }
    set x [extphy 1 1 0xa811]
    if {$x & (1 << 11)} {
        return "10G"
    }
    set x [extphy 1 7 0xfff9]
    set x [expr $x & (7 << 8)]
    if {$x == (7 << 8)} {
        return "1000M"
    }
    return "UNKNOWN"
}
######################################################################
# lb_tst: Run extlb_test at various speeds. 
# It's recommended to specify the slave port since we have seen
# duplex mismatch causing packet drops, especially noticeable at 10M.
######################################################################
proc lb_tst {iterations {slave 0} {speed_list "all"}} {
    if {$speed_list == "all"} {set speed_list {10M 100M 1000M 10G}}
    for {set i 0} {$i < $iterations} {incr i} {
        foreach speed $speed_list {
            puts "$i, $speed"
            if {$slave == 0} {
                extlb_test -case 3 -subcase A -link_speed $speed
            } else {
                extlb_test -case 3 -subcase A -hlb $slave -link_speed $speed
            }
        }
    }
}


######################################################################
# link_test: Test various link speed, no traffic.
# If "dev_s" is left as default, it's assumed that user has already
# enabled the slave port using "dev <n> hlb_on" prior to running this.
######################################################################
proc link_test {dev_a {dev_s 0} {iterations 1} {speed_list "all"} {to_sec 10}} {

    # Find speed
    #if {$speed_list == "all"} {set speed_list {10G}}
    if {$speed_list == "all"} {set speed_list {10M 100M 1000M 10G}}

    set to_msec [expr $to_sec * 1000]
    set interval 500

    device $dev_a -no_display
    if {$::current(DRV_STATE) == "RUNNING"} {driver unload}
    if {$dev_s != 0} {
        device $dev_s -no_display
        if {$::current(DRV_STATE) == "RUNNING"} {driver unload}
    }

    for {set i 0} {($iterations == 0) || ($i < $iterations)} {incr i} {
        puts "Iteration $i"
        # Iterate all speeds in $speed_list
        foreach speed $speed_list {
            puts "testing $speed"
            device $dev_a -no_display
            #device $dev_a
            if {$speed == "10G"} { 
                puts "lmdev set_speed AUTONEG"
                lmdev set_speed AUTONEG 
            } else { 
                puts "lmdev set_speed $speed"
                lmdev set_speed $speed 
            }
            driver load
            if {$dev_s != 0} {
                device $dev_s -no_display
                #device $dev_s
                driver load
            }
            # Wait for link up
            for {set idx 0} {$idx < $to_msec} {incr idx $interval} {
                set link_state [driver link_state]
                if {[lindex $link_state 3] == "UP"} {
                    puts "link up within $idx msec."
                    break
                }
                delay ms $interval
            }
            if {$idx >= $to_msec} {
                error "Device $dev_s sees no link, timed out! Got $link_state"
            }
            # Check speed
            set final_spd [lindex $link_state 1]
            set sb_result $speed
            append sb_result BPS

            if {$sb_result != $final_spd} {
                error "Device $dev_s sees wrong speed, got $final_spd, sb $sb_result"
            }
            # Check the first port
            device $dev_a -no_display
            #device $dev_a
            set link_state [driver link_state]
            if {[lindex $link_state 3] != "UP"} {
                error "First port ($dev_a) sees link down"
            }
            set final_spd [lindex $link_state 1]
            if {$sb_result != $final_spd} {
                error "First port ($dev_a) wrong speed, got $final_spd, sb $sb_result"
            }
            driver unload
            if {$dev_s != 0} {
                device $dev_s -no_display
                #device $dev_s
                driver unload
            }
            delay ms 1000
        }
    }
}

######################################################################
# By default, the stat update is disabled when the driver is loaded.
# Enable statistics update while driver is loaded.
######################################################################
proc stat_update {} {
    driver enable_stat
    delay ms 500
    driver disable_stat
}

proc intrcnt {{iterations 10}} {
    #for {set idx 0} {$idx < $iterations} {incr idx} {
    #    set x [driver intrcnt]
    #    puts $x
    #    delay ms 1000
    #}
}

######################################################################
# Dump NIG block info, along with link state of the current function.
######################################################################
proc dump_nig {} {
    show 0x10320 0x20
    show 0x10680 0x20
    show 0x18000 0x20
    show 0x10480 0x30
    link_state
}

proc disable_stat {} {
    set curr $::current(DEV)
    set first $curr
    if {($curr & 1) == 0} { incr first -1 }
    set second [expr $first + 1]
    device $first -no_display
    driver disable_stat
    device $second -no_display
    driver disable_stat
    device $curr -no_display
}
######################################################################
proc intrcnt {} {
    set curr $::current(DEV)
    set first $curr
    if {($curr & 1) == 0} { incr first -1 }
    set second [expr $first + 1]
    device $first -no_display
    puts -nonewline "$first: "
    puts [driver intrcnt]
    device $second -no_display
    puts -nonewline "$second: "
    puts [driver intrcnt]
    device $curr -no_display
}

proc link_state {} {
    set curr $::current(DEV)
    set first $curr
    if {($curr & 1) == 0} { incr first -1 }
    set second [expr $first + 1]
    device $first -no_display
    puts -nonewline $first
    puts [driver link_state]
    device $second -no_display
    puts -nonewline $second
    puts [driver link_state]
    device $curr -no_display
}

proc show {offset length} {
    set curr $::current(DEV)
    set first $curr
    if {($curr & 1) == 0} { incr first -1 }
    set second [expr $first + 1]
    device $first -no_display
    reg show $offset $length
    device $second -no_display
    reg show $offset $length
    device $curr -no_display
}

proc test_id {{iterations 1000}} {
    for {set idx 0} {$idx < $iterations} {incr idx} {
        set val [extphy 0 7 3]
        if {$val != 0xbff0} {
            error "Got [format 0x%x $val]"
        }
    }
    puts "Pass $iterations"
}


######################################################################
# time_link: Report the amount of time required to establish link
# between two ports, at the interval of 0.5 sec. The will involve 
# loading/unloading driver of the current port.
# Assumption: The link partner is the "other" port of the same chip,
# and the driver on that port is already loaded.
######################################################################
proc time_link {{iterations 10} {limit_sec 50}} {
    set delayms 500
    set limit [expr 1000 / $delayms * $limit_sec]
    set port [expr ($::current(DEV) & 1) ^ 1]

    for {set idx 0} {$idx < $iterations} {incr idx} {
        puts -nonewline "Iteration $idx: "
        flush stdout   
        driver load
        for {set cnt 0} {$cnt < $limit} {incr cnt} {
            set link_val [extphy $port 1 0xa811]
            #puts "[format 0x%x $link_val]"
            if { ($link_val & (1 << 11)) } {
                puts "up in [expr $cnt * 0.5] second(s)"
                break
            }
            puts -nonewline "."
            #phy_dump
            flush stdout
            delay ms $delayms            
        }
        if {$cnt >= $limit} {error "Link up takes over [expr $limit * 0.5] seconds."}
        #phy_dump
        driver unload
        delay ms 1000
    }
}

######################################################################
# time_link: Report the amount of time required to establish link
# between two ports, at the interval of 0.5 sec. The will involve 
# restarting autoneg from the current port.
# Assumption: The link partner is the "other" port of the same chip,
# and the driver on that port is already loaded.
######################################################################
proc time_an {{iterations 10} {limit_sec 50}} {
    set delayms 500
    set limit [expr 1000 / $delayms * $limit_sec]
    set port [expr ($::current(DEV) & 1) ^ 1]

    puts "Timing the re-AN"
    for {set idx 0} {$idx < $iterations} {incr idx} {
        puts -nonewline "Iteration $idx: "
        flush stdout
        set val [extphy $port 7 0]
        extphy $port 7 0 [expr $val | (1 << 9)]
        for {set cnt 0} {$cnt < $limit} {incr cnt} {
            set link_val [extphy $port 1 0xa811]
            #puts "[format 0x%x $link_val]"
            if { ($link_val & (1 << 11)) } {
                puts "up in [expr $cnt * 0.5] second(s)"
                break
            }
            puts -nonewline "."
            #phy_dump
            flush stdout
            delay ms $delayms            
        }
        if {$cnt >= $limit} {error "Link up takes over [expr $limit * 0.5] seconds."}
        #phy_dump
        delay ms 1000
    }
}


######################################################################
# Dumping of relevant PHY registers. If "led" is specified in "part"
# argument, it'll dump the LED related registers instead. This is only
# used for 84823 and 84833Ax.
######################################################################
proc phy_dump { {part ""} } {

    set id [extphy 0 1 3]
    set phy 0
    if {$id == 0x5cb0} {
        set phy 84833
    }
    # The offsets are relative to 84823, 84833 needs adjustments.
    set reglist {}
    if {$part == "led"} {
        lappend reglist {1 0xa82c} {1 0xa82d} {1 0xa82e} {1 0xa83b}
        lappend reglist {1 0xa82f} {1 0xa830} {1 0xa831} {1 0xa83c}
        lappend reglist {1 0xa832} {1 0xa833} {1 0xa834} {1 0xa83d}
        lappend reglist {1 0xa835} {1 0xa836} {1 0xa837} {0 0}
        lappend reglist {1 0xa838} {1 0xa839} {1 0xa83a}
    } else {
        lappend reglist {7 0xffe0} {7 0xffe1} {30 0x2} {30 0x3} 
        lappend reglist {7 0xffe4} {7 0xffe5} {7 0xffe6} 
        lappend reglist {7 0xffe9} 
        #lappend reglist {7 0xffec} {7 0xffed} {7 0xffee} {7 0xffef} 
        #lappend reglist {7 0xfff0} {7 0xfff1} {7 0xfff2} {7 0xfff3} 
        #lappend reglist {7 0xfff4} {7 0xfff5} {7 0xfff6} {7 0xfff7} 
        lappend reglist {7 0xfff9} 
        #lappend reglist {7 0xfffc} {7 0xfffd} {7 0xfffe} {7 0xffff} 
        lappend reglist {7 0} {7 1} {7 0x20} {7 0x3c}
        lappend reglist {30 0x400d} {30 0x401a} {30 0x4110} {30 0x4111} 
        lappend reglist {30 0x4112} {30 0x4113} {30 0x4131} {30 0x412e} 
        lappend reglist {30 0x4083} {30 0x4085} 
        lappend reglist {1 0} {1 0xa811}
        lappend reglist {1 0xa83b}
        #set reglist {{0x1e 0x400d} {0x1e 0x4083} {0x1e 0x4085} }
    }

    for {set idx 0} {$idx < [llength $reglist]} {incr idx} {
        set item [lindex $reglist $idx]
        set devad [lindex $item 0]
        set offset [lindex $item 1]
        set port $::current(PCI_FUNC)
        if {$phy==84833} {
            if {$offset == 0x401a} {
                incr offset 3
            } elseif {($offset >= 0xa811) && ($offset <= 0xa83d)} {
                incr offset -1
            }
        }
        set val [extphy $port $devad $offset]
        puts -nonewline "[format %02x $devad].[format %04x $offset]: [format %04x $val]  "
        if {($idx & 3) == 3} {puts ""}
    }
    if {($idx & 3) != 0} {puts ""}
}


proc reg_dump {} {
    puts [driver link_state]
    puts "MAC registers"
    set mac_list {0xa590 0x10058 0x102e8 0x102e0 0x8000 0x80c8 0x80bc 
        0x1003c 0x809c 0x10044 0x100ac 0x10110 0x100e0 0x100a4 0x10118
        0x10120 0x100e8}
    foreach item $mac_list {
        puts "$item: [reg read $item]"
    }
    puts "internal PHY"
    set phy_list {0x8300 0xffe0 0x80f0}
    foreach item $phy_list {
        puts "$item: [phy 0 $item]"
    }
}

if {[file exists l2lb.tcl]} {
    source l2lb.tcl
}

proc get_scree { } {
	puts "1"
	gets stdin
	puts "2"
	gets stdin
	puts "done"
}


proc reset_wc { } {
    reg write 0xac30 0xe
    delay ms 1
    reg write 0xac30 0xf
    delay ms 1
    reg write 0xac30 0x3ff
    # Set Rx lane swap
    wc_phy 1 3 0x816b 0x1b
    wc_phy 1 3 0x8169 0xe4
    ## Set Tx lane swap
    #wc_phy 1 3 0x8169 0x1b
    #wc_phy 1 3 0x816b 0xe4
}
proc force_10G { } {
    
    #DISABLE SEQUENCER
    puts "wc_phy 1 3 0x8000 0x42f"
    gets stdin
    wc_phy 1 3 0x8000 0x42f
    wc_dump   

    #RX LANE SWAPPING
    puts "wc_phy 1 3 0x816b 0x1b"
    gets stdin
    wc_phy 1 3 0x816b 0x1b
    wc_dump

    #TX LANE SWAPPING
    puts "wc_phy 1 3 0x8169 0xe4"
    gets stdin
    wc_phy 1 3 0x8169 0xe4 
    wc_dump

    #MDIO_AER_BLOCK_AER_REG
    #puts "wc_phy 1 3 0xffde 0x3800"
    #gets stdin
    #wc_phy 1 3 0xffde 0x3800
    #wc_dump
    
    #MDIO_WC_CL45_XGXSBLK1_LANECTRL0
    puts "wc_phy 1 3 0x8015 0x0"
    gets stdin
    wc_phy 1 3 0x8015 0x0
    wc_dump
    
    #MDIO_WC_CL45_XGXSBLK1_LANECTRL1
    puts "wc_phy 1 3 0x8016 0xFFFF"
    gets stdin
    wc_phy 1 3 0x8016 0xFFFF
    wc_dump    
    #MDIO_WC_CL45_PMD_IEEE9BLK_TENGBASE_KR_PMD_CONTROL_REGISTER_150
    puts "wc_phy 1 3 0x0096 0x0"
    gets stdin
    wc_phy 1 3 0x0096 0x0
    wc_dump
    #MDIO_WC_CL45_CL72_USERB0_CL72_MISC1_CONTROL
    puts "wc_phy 1 3 0x82e3 0xB800"
    gets stdin
    wc_phy 1 3 0x82e3 0xB800
    wc_dump
    
    #MDIO_WC_CL45_IEEE0BLK_MIICNTL
    puts "wc_phy 1 3 0x0000 0x2040"
    gets stdin
    wc_phy 1 3 0x0000 0x2040
    wc_dump
    
    #MDIO_WC_CL45_IEEE0BLK_AUTONEGNP <-- didn't stick
    puts "wc_phy 1 3 0x0007 0xb"
    gets stdin
    wc_phy 1 3 0x0007 0xb
    wc_dump
    
    #MDIO_WC_CL45_SERDESDIGITAL_MISC2
    puts "wc_phy 1 3 0x8309 0x30"
    gets stdin
    wc_phy 1 3 0x8309 0x30
    wc_dump
    
    #MDIO_WC_CL45_TX66_CONTROL
    puts "wc_phy 1 3 0x83b0 0x9"
    gets stdin
    wc_phy 1 3 0x83b0 0x9
    wc_dump
    
    #MDIO_WC_CL45_RX66_CONTROL
    puts "wc_phy 1 3 0x83c0 0xf9"
    gets stdin
    wc_phy 1 3 0x83c0 0xf9
    wc_dump
    
    #MDIO_WC_CL45_IEEE0BLK_MIICNTL <-- read modify write??
    puts "wc_phy 1 3 0x0000 0x4000"
    gets stdin
    wc_phy 1 3 0x0000 0x4000
    wc_dump

    #MDIO_WC_CL45_IEEE0BLK_MIICNTL
    puts "wc_phy 1 3 0x0000 0x0"
    gets stdin
    wc_phy 1 3 0x0000 0x0
    wc_dump

    #ENABLE SEQUENCER
    puts "wc_phy 1 3 0x8000 0x242f"
    gets stdin
    wc_phy 1 3 0x8000 0x242f
    wc_dump 
}


proc loopback10G { } {
    
    #MDIO_WC_CL45_IEEE0BLK_MIICNTL
    set val [wc_phy 1 3 0x0]

    puts "old val: $val"

    set val [expr $val | 0x4000]
    
    puts "new val: $val"

    #MDIO_WC_CL45_IEEE0BLK_MIICNTL  
    wc_phy 1 3 0x0 $val
    
    #MDIO_WC_CL45_AERBLK_AER
    wc_phy 1 3 0xffde 0x800
    
    #MDIO_WC_CL45_IEEE0BLK_MIICNTL
    set val [wc_phy 1 3 0x0]

    puts "old val: $val"

    set val [expr $val | 0x1]
    
    puts "new val: $val"

    #MDIO_WC_CL45_IEEE0BLK_MIICNTL  
    wc_phy 1 3 0x0 $val
}

proc force_link {} {
    wc_phy 1 3 0x800e 1
    wc_phy 1 3 0x8000 0x0c2f

    # set to forced 10G XFI
    set x [wc_phy 1 3 0x8308]
    set x [expr $x & ~0x3f]
    wc_phy 1 3 0x8308 [expr $x | 0x05] 
    set x [wc_phy 1 3 0x833c]
    wc_phy 1 3 0x833c [expr $x | 0x80] 
    ## Flip polarity on Tx0
    #wc_phy 1 3 0x8061 0x20
    #wc_phy 1 1 7 0xb
    #wc_phy 1 3 0x07 0x0

    wc_phy 1 3 0x8000 0x2c2f
}


#if {[file exists l2utils.tcl]} {
#    source l2utils.tcl
#}

