###########################################################################
#
# Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved
# Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
# Proprietary and Confidential Information.
#
# Module Description:
# This TCL script for debugging WoL
#
# Written by Yaniv Rosner, QLogic, Jun 2011
#
###########################################################################

# Specify any dependency on other packages
package require Tcl


proc set_device {path port} {
   set base_dev [expr $::current(DEV) - $::current(PCI_FUNC)]
   set rest_devices [expr $::current(TOTAL_DEV) - $base_dev + 1]
   if {([is_e2] || [is_e3])} {
       set new_device [expr $base_dev + $path + $port*2]
       puts "set device $new_device"
       device $new_device -no_display      
   } else {
       device [expr $base_dev + $port] -no_display
   }
   return $rest_devices
}

proc yesno {val} {
    if {$val} {
        return YES
    } else {
        return "NO "
    }
}
proc wol_dump_dev {path port} {
    #  1. Check PME enabled
    set cur_dev $::current(DEV)    
    set rest_devices [set_device $path $port]
    
    if {[is_4port_mode]} {
        set next_func 4
    } else {
        set next_func 2
    }    
    
    set emac_base [expr 0x8000 + $port*0x400]

    set magic_packet_reg [reg read $emac_base]
    set magic_packet_regonition_enabled [expr $magic_packet_reg & 0x40000]
    set magic_packet_received [expr $magic_packet_reg & 0x80000]

    set shmem_base [reg read 0xa2b4]

    set link_reg [reg read [expr $shmem_base + $port*0x10+ 0x664]]
    set link_up [expr $link_reg & 1]

    set link_str "Unknown"
    if {$link_up} {
        set link_speed [expr ($link_reg & 0x1e) >> 1 ]
        if {$link_speed == 0} { set link_str "UP - 10M" }
        if {$link_speed == 1} { set link_str "UP - 10M" }
        if {$link_speed == 2} { set link_str "UP - 10M" }
        if {$link_speed == 3} { set link_str "UP - 100M" }
        if {$link_speed == 4} { set link_str "UP - 100M" }
        if {$link_speed == 5} { set link_str "UP - 100M" }
        if {$link_speed == 6} { set link_str "UP - 1000M" }
        if {$link_speed == 7} { set link_str "UP - 1000M" }
        if {$link_speed == 8} { set link_str "UP - 2500M" }
        if {$link_speed == 9} { set link_str "UP - 2500M" }
        if {$link_speed == 10} { set link_str "UP - 10G" }
        if {$link_speed == 11} { set link_str "UP - 20G" }
    } else {
        set link_str "DOWN"
    }

    set wol_enabled_in_nvram [expr [reg read [expr $shmem_base + 856 + $port*116]] & 0x01000000]
    for {set pf 0} {$pf < 5} {incr pf} {
        lappend mac_hi [format %04x [reg read [expr $emac_base + 0x10 + 4*$pf*2]]]
        lappend mac_lo [format %08x [reg read [expr $emac_base + 0x10 + 4*($pf*2 + 1)]]]
    }

    set func 0    
    
    for {set dev 0} {$dev <= $rest_devices} {incr dev} {
       set prev_pci_func $::current(PCI_FUNC)
       set mf_wol_reg($func) [reg read [expr $emac_base + 0x1e0]]
       set multi_function_wol_en_pf($func) [expr $mf_wol_reg($func) & (1<<$func)];
       set pme_reg($func) [reg read 0x204c]
       set pme_enabled($func) [expr $pme_reg($func) & 0x100]
       set pme_status_asserted($func) [expr $pme_reg($func) & 0x8000]

       set acpi_enable($func) [reg read [expr $port*0x44 + 0x10138]]

       set acpi_crc_0($func)  [reg read [expr $port*0x44 + 0x1015c]]
       set acpi_crc_1($func)  [reg read [expr $port*0x44 + 0x10160]]
       set acpi_crc_2($func)  [reg read [expr $port*0x44 + 0x10164]]
       set acpi_crc_3($func)  [reg read [expr $port*0x44 + 0x10168]]
       set acpi_crc_4($func)  [reg read [expr $port*0x44 + 0x1016c]]
       set acpi_crc_5($func)  [reg read [expr $port*0x44 + 0x10170]]
       set acpi_crc_6($func)  [reg read [expr $port*0x44 + 0x10174]]
       set acpi_crc_7($func)  [reg read [expr $port*0x44 + 0x10178]]

       set acpi_len_0($func)  [reg read [expr $port*0x44 + 0x1013c]]
       set acpi_len_1($func)  [reg read [expr $port*0x44 + 0x10140]]
       set acpi_len_2($func)  [reg read [expr $port*0x44 + 0x10144]]
       set acpi_len_3($func)  [reg read [expr $port*0x44 + 0x10148]]
       set acpi_len_4($func)  [reg read [expr $port*0x44 + 0x1014c]]
       set acpi_len_5($func)  [reg read [expr $port*0x44 + 0x10150]]
       set acpi_len_6($func)  [reg read [expr $port*0x44 + 0x10154]]
       set acpi_len_7($func)  [reg read [expr $port*0x44 + 0x10158]]

       incr func;
       
       if {[expr $::current(DEV) + $next_func] > $rest_devices} {
            break;
       }
       
       puts "Move to next device [expr $::current(DEV) + $next_func]"
       device [expr $::current(DEV) + $next_func] -no_display
       if {$prev_pci_func >= $::current(PCI_FUNC)} {
            break;
        }
    }
    set max_func $func
    puts "device \[$path:$port\]:"
    puts "LINK"
    puts "   $link_str"
    puts "WOL"
    puts "   Enabled in nvram: [yesno $wol_enabled_in_nvram]"
    puts "MAGIC PACKET"
    puts "   Reg 0x8000 val: $magic_packet_reg"
    puts "   Recognition enabled: [yesno $magic_packet_regonition_enabled]"
    puts "   Received: [yesno $magic_packet_received]"
    puts "PME"
    for {set func 0} {$func < $max_func} {incr func} {
       puts "   pf$func: Reg 0x204c val: $pme_reg($func)"
       puts "        Enabled: [yesno $pme_enabled($func)]"
       puts "        Status_asserted: [yesno $pme_status_asserted($func)]"
    }
    puts "MAC ADDR LISTEN"
    for {set func 0} {$func < $max_func} {incr func} {
       puts "   pf$func: [yesno $multi_function_wol_en_pf($func)] [lindex $mac_hi $func][lindex $mac_lo $func]"
    }

    puts "ACPI pattern"
    for {set func 0} {$func < $max_func} {incr func} {
        puts "   ACPI pattern Enabled             [yesno $acpi_enable($func)]"
        puts "   ACPI pattern 0-7 Length in bytes [format "0x%02x 0x%02x 0x%02x 0x%02x\r\n                                    0x%02x 0x%02x 0x%02x 0x%02x" $acpi_len_0($func) $acpi_len_1($func) $acpi_len_2($func) $acpi_len_3($func) $acpi_len_4($func) $acpi_len_5($func) $acpi_len_6($func) $acpi_len_7($func)]"
        puts "   ACPI pattern 0-7 CRC32           [format "0x%08x 0x%08x 0x%08x 0x%08x\r\n                                    0x%08x 0x%08x 0x%08x 0x%08x" $acpi_crc_0($func) $acpi_crc_1($func) $acpi_crc_2($func) $acpi_crc_3($func) $acpi_crc_4($func) $acpi_crc_5($func) $acpi_crc_6($func) $acpi_crc_7($func)]"
    }   
    
    device $cur_dev -no_display
}

proc get_max_path_num {} {    
    
    if {![is_e1x]} {                
        if {$::current(CHIP_NUM) == 57811} {
            return 1
        } else {                
            return 2
        } 
    } else {
        # In 577xx, we support only 1 path
        return 1
    }    
}



proc wol_dump {} {
    
    if {[is_4port_mode]} {
        set max_port 2
    } else {
        set max_port 1
    }
    
    if {[is_one_port_board]} {
        set max_port 1
    }
    
    set max_path [get_max_path_num]
    
    for {set path 0} {$path < $max_path} {incr path} {
        for {set port 0} {$port < $max_port} {incr port} {
            puts "\nRead WOL dump for path $path port $port:"
            wol_dump_dev $path $port         
           
        }
    }      
    
    puts "\nRead linkdump:"
    linkdump
           
    puts "\nRead MCP trace:"
    mcp trace
}
