/** @file
Header file for compression routine

Copyright (c) 2006 - 2014, Intel Corporation. All rights reserved.<BR>
This program and the accompanying materials                          
are licensed and made available under the terms and conditions of the BSD License         
which accompanies this distribution.  The full text of the license may be found at        
http://opensource.org/licenses/bsd-license.php                                            
                                                                                          
THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,                     
WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.             

**/

#ifndef _EFI_DECOMPRESS_H
#define _EFI_DECOMPRESS_H

#include <stdlib.h>
#include "bcmtype.h"

#define RETURN_SUCCESS               0

#ifdef UEFI
#include "efierr.h"
#else
//
// EFI Error Codes common to all execution phases
//

typedef u32_t RETURN_STATUS;
#define MAX_BIT 0x80000000
typedef u32_t EFI_STATUS;
typedef u64_t PHYSICAL_ADDRESS;

///
/// Set the upper bit to indicate EFI Error.
///
#define ENCODE_ERROR(a)              (MAX_BIT | (a))

#define ENCODE_WARNING(a)            (a)
#define RETURN_ERROR(a)              ((a) < 0)

#define RETURN_LOAD_ERROR            ENCODE_ERROR (1)
#define RETURN_INVALID_PARAMETER     ENCODE_ERROR (2)
#define RETURN_UNSUPPORTED           ENCODE_ERROR (3)
#define RETURN_BAD_BUFFER_SIZE       ENCODE_ERROR (4)
#define RETURN_BUFFER_TOO_SMALL      ENCODE_ERROR (5)
#define RETURN_NOT_READY             ENCODE_ERROR (6)
#define RETURN_DEVICE_ERROR          ENCODE_ERROR (7)
#define RETURN_WRITE_PROTECTED       ENCODE_ERROR (8)
#define RETURN_OUT_OF_RESOURCES      ENCODE_ERROR (9)
#define RETURN_VOLUME_CORRUPTED      ENCODE_ERROR (10)
#define RETURN_VOLUME_FULL           ENCODE_ERROR (11)
#define RETURN_NO_MEDIA              ENCODE_ERROR (12)
#define RETURN_MEDIA_CHANGED         ENCODE_ERROR (13)
#define RETURN_NOT_FOUND             ENCODE_ERROR (14)
#define RETURN_ACCESS_DENIED         ENCODE_ERROR (15)
#define RETURN_NO_RESPONSE           ENCODE_ERROR (16)
#define RETURN_NO_MAPPING            ENCODE_ERROR (17)
#define RETURN_TIMEOUT               ENCODE_ERROR (18)
#define RETURN_NOT_STARTED           ENCODE_ERROR (19)
#define RETURN_ALREADY_STARTED       ENCODE_ERROR (20)
#define RETURN_ABORTED               ENCODE_ERROR (21)
#define RETURN_ICMP_ERROR            ENCODE_ERROR (22)
#define RETURN_TFTP_ERROR            ENCODE_ERROR (23)
#define RETURN_PROTOCOL_ERROR        ENCODE_ERROR (24)
#define RETURN_INCOMPATIBLE_VERSION  ENCODE_ERROR (25)
#define RETURN_SECURITY_VIOLATION    ENCODE_ERROR (26)
#define RETURN_CRC_ERROR             ENCODE_ERROR (27)
#define RETURN_END_OF_MEDIA          ENCODE_ERROR (28)
#define RETURN_END_OF_FILE           ENCODE_ERROR (31)

#define RETURN_WARN_UNKNOWN_GLYPH    ENCODE_WARNING (1)
#define RETURN_WARN_DELETE_FAILURE   ENCODE_WARNING (2)
#define RETURN_WARN_WRITE_FAILURE    ENCODE_WARNING (3)
#define RETURN_WARN_BUFFER_TOO_SMALL ENCODE_WARNING (4)

//
// Enumeration of EFI_STATUS.
// 

#define EFI_SUCCESS               RETURN_SUCCESS              
#define EFI_LOAD_ERROR            RETURN_LOAD_ERROR           
#define EFI_INVALID_PARAMETER     RETURN_INVALID_PARAMETER    
#define EFI_UNSUPPORTED           RETURN_UNSUPPORTED          
#define EFI_BAD_BUFFER_SIZE       RETURN_BAD_BUFFER_SIZE      
#define EFI_BUFFER_TOO_SMALL      RETURN_BUFFER_TOO_SMALL     
#define EFI_NOT_READY             RETURN_NOT_READY            
#define EFI_DEVICE_ERROR          RETURN_DEVICE_ERROR         
#define EFI_WRITE_PROTECTED       RETURN_WRITE_PROTECTED      
#define EFI_OUT_OF_RESOURCES      RETURN_OUT_OF_RESOURCES     
#define EFI_VOLUME_CORRUPTED      RETURN_VOLUME_CORRUPTED     
#define EFI_VOLUME_FULL           RETURN_VOLUME_FULL          
#define EFI_NO_MEDIA              RETURN_NO_MEDIA             
#define EFI_MEDIA_CHANGED         RETURN_MEDIA_CHANGED        
#define EFI_NOT_FOUND             RETURN_NOT_FOUND            
#define EFI_ACCESS_DENIED         RETURN_ACCESS_DENIED        
#define EFI_NO_RESPONSE           RETURN_NO_RESPONSE          
#define EFI_NO_MAPPING            RETURN_NO_MAPPING           
#define EFI_TIMEOUT               RETURN_TIMEOUT              
#define EFI_NOT_STARTED           RETURN_NOT_STARTED          
#define EFI_ALREADY_STARTED       RETURN_ALREADY_STARTED      
#define EFI_ABORTED               RETURN_ABORTED              
#define EFI_ICMP_ERROR            RETURN_ICMP_ERROR           
#define EFI_TFTP_ERROR            RETURN_TFTP_ERROR           
#define EFI_PROTOCOL_ERROR        RETURN_PROTOCOL_ERROR       
#define EFI_INCOMPATIBLE_VERSION  RETURN_INCOMPATIBLE_VERSION 
#define EFI_SECURITY_VIOLATION    RETURN_SECURITY_VIOLATION   
#define EFI_CRC_ERROR             RETURN_CRC_ERROR   
#define EFI_END_OF_MEDIA          RETURN_END_OF_MEDIA
#define EFI_END_OF_FILE           RETURN_END_OF_FILE

#define EFI_WARN_UNKNOWN_GLYPH    RETURN_WARN_UNKNOWN_GLYPH   
#define EFI_WARN_DELETE_FAILURE   RETURN_WARN_DELETE_FAILURE  
#define EFI_WARN_WRITE_FAILURE    RETURN_WARN_WRITE_FAILURE   
#define EFI_WARN_BUFFER_TOO_SMALL RETURN_WARN_BUFFER_TOO_SMALL

#endif //UEFI

#ifndef UINT8_MAX
# define UINT8_MAX		(255)
#endif

//
// Define macros to build data structure signatures from characters.
//
#define SIGNATURE_16(A, B)        ((A) | (B << 8))
#define SIGNATURE_32(A, B, C, D)  (SIGNATURE_16 (A, B) | (SIGNATURE_16 (C, D) << 16))
#define SIGNATURE_64(A, B, C, D, E, F, G, H) \
    (SIGNATURE_32 (A, B, C, D) | ((u64_t) (SIGNATURE_32 (E, F, G, H)) << 32))

#define EFI_IMAGE_NUMBER_OF_DIRECTORY_ENTRIES 16
//
// EXE file formats
//
#define EFI_IMAGE_DOS_SIGNATURE     SIGNATURE_16('M', 'Z')
#define EFI_IMAGE_NT_SIGNATURE      SIGNATURE_32('P', 'E', '\0', '\0')

///
/// PE images can start with an optional DOS header, so if an image is run
/// under DOS it can print an error message.
///
typedef struct {
  u16_t  e_magic;    ///< Magic number.
  u16_t  e_cblp;     ///< Bytes on last page of file.
  u16_t  e_cp;       ///< Pages in file.
  u16_t  e_crlc;     ///< Relocations.
  u16_t  e_cparhdr;  ///< Size of header in paragraphs.
  u16_t  e_minalloc; ///< Minimum extra paragraphs needed.
  u16_t  e_maxalloc; ///< Maximum extra paragraphs needed.
  u16_t  e_ss;       ///< Initial (relative) SS value.
  u16_t  e_sp;       ///< Initial SP value.
  u16_t  e_csum;     ///< Checksum.
  u16_t  e_ip;       ///< Initial IP value.
  u16_t  e_cs;       ///< Initial (relative) CS value.
  u16_t  e_lfarlc;   ///< File address of relocation table.
  u16_t  e_ovno;     ///< Overlay number.
  u16_t  e_res[4];   ///< Reserved words.
  u16_t  e_oemid;    ///< OEM identifier (for e_oeminfo).
  u16_t  e_oeminfo;  ///< OEM information; e_oemid specific.
  u16_t  e_res2[10]; ///< Reserved words.
  u32_t  e_lfanew;   ///< File address of new exe header.
} EFI_IMAGE_DOS_HEADER;

///
/// COFF File Header (Object and Image).
///
typedef struct {
  u16_t  Machine;
  u16_t  NumberOfSections;
  u32_t  TimeDateStamp;
  u32_t  PointerToSymbolTable;
  u32_t  NumberOfSymbols;
  u16_t  SizeOfOptionalHeader;
  u16_t  Characteristics;
} EFI_IMAGE_FILE_HEADER;
///
/// Size of EFI_IMAGE_FILE_HEADER.
///
#define EFI_IMAGE_SIZEOF_FILE_HEADER        20

///
/// Header Data Directories.
///
typedef struct {
  u32_t  VirtualAddress;
  u32_t  Size;
} EFI_IMAGE_DATA_DIRECTORY;

///
/// EFI_IMAGE_NT_OPTIONAL_HDR32_MAGIC means PE32 and 
/// EFI_IMAGE_OPTIONAL_HEADER32 must be used. The data structures only vary
/// after NT additional fields.
///
#define EFI_IMAGE_NT_OPTIONAL_HDR32_MAGIC 0x10b

///
/// Optional Header Standard Fields for PE32.
///
typedef struct {
  ///
  /// Standard fields.
  ///
  u16_t                    Magic;
  u8_t                     MajorLinkerVersion;
  u8_t                     MinorLinkerVersion;
  u32_t                    SizeOfCode;
  u32_t                    SizeOfInitializedData;
  u32_t                    SizeOfUninitializedData;
  u32_t                    AddressOfEntryPoint;
  u32_t                    BaseOfCode;
  u32_t                    BaseOfData;  ///< PE32 contains this additional field, which is absent in PE32+.
  ///
  /// Optional Header Windows-Specific Fields.
  ///
  u32_t                    ImageBase;
  u32_t                    SectionAlignment;
  u32_t                    FileAlignment;
  u16_t                    MajorOperatingSystemVersion;
  u16_t                    MinorOperatingSystemVersion;
  u16_t                    MajorImageVersion;
  u16_t                    MinorImageVersion;
  u16_t                    MajorSubsystemVersion;
  u16_t                    MinorSubsystemVersion;
  u32_t                    Win32VersionValue;
  u32_t                    SizeOfImage;
  u32_t                    SizeOfHeaders;
  u32_t                    CheckSum;
  u16_t                    Subsystem;
  u16_t                    DllCharacteristics;
  u32_t                    SizeOfStackReserve;
  u32_t                    SizeOfStackCommit;
  u32_t                    SizeOfHeapReserve;
  u32_t                    SizeOfHeapCommit;
  u32_t                    LoaderFlags;
  u32_t                    NumberOfRvaAndSizes;
  EFI_IMAGE_DATA_DIRECTORY  DataDirectory[EFI_IMAGE_NUMBER_OF_DIRECTORY_ENTRIES];
} EFI_IMAGE_OPTIONAL_HEADER32;

///
/// EFI_IMAGE_NT_OPTIONAL_HDR64_MAGIC means PE32+ and 
/// EFI_IMAGE_OPTIONAL_HEADER64 must be used. The data structures only vary
/// after NT additional fields.
///
#define EFI_IMAGE_NT_OPTIONAL_HDR64_MAGIC 0x20b

///
/// Optional Header Standard Fields for PE32+.
///
typedef struct {
  ///
  /// Standard fields.
  ///
  u16_t                    Magic;
  u8_t                     MajorLinkerVersion;
  u8_t                     MinorLinkerVersion;
  u32_t                    SizeOfCode;
  u32_t                    SizeOfInitializedData;
  u32_t                    SizeOfUninitializedData;
  u32_t                    AddressOfEntryPoint;
  u32_t                    BaseOfCode;
  ///
  /// Optional Header Windows-Specific Fields.
  ///
  u64_t                    ImageBase;
  u32_t                    SectionAlignment;
  u32_t                    FileAlignment;
  u16_t                    MajorOperatingSystemVersion;
  u16_t                    MinorOperatingSystemVersion;
  u16_t                    MajorImageVersion;
  u16_t                    MinorImageVersion;
  u16_t                    MajorSubsystemVersion;
  u16_t                    MinorSubsystemVersion;
  u32_t                    Win32VersionValue;
  u32_t                    SizeOfImage;
  u32_t                    SizeOfHeaders;
  u32_t                    CheckSum;
  u16_t                    Subsystem;
  u16_t                    DllCharacteristics;
  u64_t                    SizeOfStackReserve;
  u64_t                    SizeOfStackCommit;
  u64_t                    SizeOfHeapReserve;
  u64_t                    SizeOfHeapCommit;
  u32_t                    LoaderFlags;
  u32_t                    NumberOfRvaAndSizes;
  EFI_IMAGE_DATA_DIRECTORY  DataDirectory[EFI_IMAGE_NUMBER_OF_DIRECTORY_ENTRIES];
} EFI_IMAGE_OPTIONAL_HEADER64;

typedef struct {
  u32_t                      Signature;
  EFI_IMAGE_FILE_HEADER       FileHeader;
  EFI_IMAGE_OPTIONAL_HEADER32 OptionalHeader;
} EFI_IMAGE_NT_HEADERS32;

typedef struct {
  u32_t                      Signature;
  EFI_IMAGE_FILE_HEADER       FileHeader;
  EFI_IMAGE_OPTIONAL_HEADER64 OptionalHeader;
} EFI_IMAGE_NT_HEADERS64;

///
/// Header format for TE images, defined in the PI Specification, 1.0.
///
typedef struct {
  u16_t                    Signature;            ///< The signature for TE format = "VZ".
  u16_t                    Machine;              ///< From the original file header.
  u8_t                     NumberOfSections;     ///< From the original file header.
  u8_t                     Subsystem;            ///< From original optional header.
  u16_t                    StrippedSize;         ///< Number of bytes we removed from the header.
  u32_t                    AddressOfEntryPoint;  ///< Offset to entry point -- from original optional header.
  u32_t                    BaseOfCode;           ///< From original image -- required for ITP debug.
  u64_t                    ImageBase;            ///< From original file header.
  EFI_IMAGE_DATA_DIRECTORY  DataDirectory[2];     ///< Only base relocation and debug directory.
} EFI_TE_IMAGE_HEADER;

///
/// Union of PE32, PE32+, and TE headers.
///
typedef union {
  EFI_IMAGE_NT_HEADERS32   Pe32;
  EFI_IMAGE_NT_HEADERS64   Pe32Plus;
  EFI_TE_IMAGE_HEADER      Te;
} EFI_IMAGE_OPTIONAL_HEADER_UNION;


EFI_STATUS
EfiGetInfo (
  IN      void *Source,
  IN      u32_t  SrcSize,
  OUT     u32_t  *DstSize,
  OUT     u32_t  *ScratchSize
  );
/**

Routine Description:

  The implementation Efi Decompress GetInfo().

Arguments:

  Source      - The source buffer containing the compressed data.
  SrcSize     - The size of source buffer
  DstSize     - The size of destination buffer.
  ScratchSize - The size of scratch buffer.

Returns:

  EFI_SUCCESS           - The size of destination buffer and the size of scratch buffer are successull retrieved.
  EFI_INVALID_PARAMETER - The source data is corrupted

**/

EFI_STATUS
EfiDecompress (
  IN      void *Source,
  IN      u32_t  SrcSize,
  IN OUT  void *Destination,
  IN      u32_t  DstSize,
  IN OUT  void *Scratch,
  IN      u32_t  ScratchSize
  );
/**

Routine Description:

  The implementation of Efi Decompress().

Arguments:

  Source      - The source buffer containing the compressed data.
  SrcSize     - The size of source buffer
  Destination - The destination buffer to store the decompressed data
  DstSize     - The size of destination buffer.
  Scratch     - The buffer used internally by the decompress routine. This  buffer is needed to store intermediate data.
  ScratchSize - The size of scratch buffer.

Returns:

  EFI_SUCCESS           - Decompression is successfull
  EFI_INVALID_PARAMETER - The source data is corrupted

**/

EFI_STATUS
TianoGetInfo (
  IN      void *Source,
  IN      u32_t  SrcSize,
  OUT     u32_t  *DstSize,
  OUT     u32_t  *ScratchSize
  );
/**

Routine Description:

  The implementation Tiano Decompress GetInfo().

Arguments:

  Source      - The source buffer containing the compressed data.
  SrcSize     - The size of source buffer
  DstSize     - The size of destination buffer.
  ScratchSize - The size of scratch buffer.

Returns:

  EFI_SUCCESS           - The size of destination buffer and the size of scratch buffer are successull retrieved.
  EFI_INVALID_PARAMETER - The source data is corrupted

**/

EFI_STATUS
TianoDecompress (
  IN      void *Source,
  IN      u32_t  SrcSize,
  IN OUT  void *Destination,
  IN      u32_t  DstSize,
  IN OUT  void *Scratch,
  IN      u32_t  ScratchSize
  );
/**

Routine Description:

  The implementation of Tiano Decompress().

Arguments:

  Source      - The source buffer containing the compressed data.
  SrcSize     - The size of source buffer
  Destination - The destination buffer to store the decompressed data
  DstSize     - The size of destination buffer.
  Scratch     - The buffer used internally by the decompress routine. This  buffer is needed to store intermediate data.
  ScratchSize - The size of scratch buffer.

Returns:

  EFI_SUCCESS           - Decompression is successfull
  EFI_INVALID_PARAMETER - The source data is corrupted

**/

typedef
EFI_STATUS
(*GETINFO_FUNCTION) (
  IN      void *Source,
  IN      u32_t  SrcSize,
  OUT     u32_t  *DstSize,
  OUT     u32_t  *ScratchSize
  );

typedef
EFI_STATUS
(*DECOMPRESS_FUNCTION) (
  IN      void *Source,
  IN      u32_t  SrcSize,
  IN OUT  void *Destination,
  IN      u32_t  DstSize,
  IN OUT  void *Scratch,
  IN      u32_t  ScratchSize
  );

EFI_STATUS
Extract (
  IN      void *Source,
  IN      u32_t  SrcSize,
     OUT  void **Destination,
     OUT  u32_t  *DstSize,
  IN      u32_t   Algorithm
  );

#endif
