
#ifndef __COMMON_NVM_H__
#define __COMMON_NVM_H__

#include "tcl.h"
#include "bcmtype.h"
#include "license.h"
#include "mcp_shmem.h"
#ifndef FWUPGTOOL
#include "append.h"
#else
#include "../57710h/append.h"
#endif // !FWUPGTOOL
#include "tcl_al_common.h"

#ifdef EDEBUG
#include "dbg_vpd.h"
#include "edebug_types.h"
#include "dg_nvm_struct.h"
#else

#ifndef FWUPGTOOL
#include "man_types.h"
#else
#include "nvm_hlp.h"
#endif // !FWUPGTOOL

#include "dg_vpd.h"
#include "diag_types.h"
#include "dg_nvm_struct.h"
#endif

typedef struct nvm_cfg nvm_cfg_t;

#ifndef FWUPGTOOL
extern Tcl_Interp *globalInterp;
#endif // !FWUPGTOOL

#ifndef EDEBUG
typedef diag_status_t common_nvm_status_t;
typedef diag_virt_addr_t common_nvm_virt_addr_t;
#ifndef FWUPGTOOL
typedef diag_dev_info_t common_nvm_dev_info_t;
typedef dev_stat_t common_nvm_dev_stat_t;
#endif // !FWUPGTOOL
#define COMMON_NVM_CHAR_PP char **
#define COMMON_NVM_CUR_DEV XDIAG_CUR_DEV
#define COMMON_NVM_CUR_DEV_IF XDIAG_CUR_DEV_IF
#define COMMON_NVM_CUR_DEV_IS_VALID XDIAG_CUR_DEV_IS_VALID
#define COMMON_NVM_PATH_NUM DG_PATH_NUM
#define common_nvm_cur_dev xdiag_cur_dev

#else
typedef dbg_status_t common_nvm_status_t;
typedef dbg_virt_addr_t common_nvm_virt_addr_t;
typedef dbg_dev_info_t common_nvm_dev_info_t;
typedef pci_info_t common_nvm_dev_stat_t;
#define COMMON_NVM_CHAR_PP const char **
#define COMMON_NVM_CUR_DEV XDBG_CUR_DEV
#define COMMON_NVM_CUR_DEV_IF XDBG_CUR_DEV_IF
#define COMMON_NVM_CUR_DEV_IS_VALID XDBG_CUR_DEV_IS_VALID
#define COMMON_NVM_PATH_NUM DBG_PATH_NUM
#define common_nvm_cur_dev xdbg_cur_dev
#endif //EDEBUG

#ifdef FWUPGTOOL
u32_t COMMON_NVM_IS_E1x(void);
u32_t COMMON_NVM_IS_E1(void);
u32_t COMMON_NVM_IS_E1H(void);
u32_t COMMON_NVM_IS_E2(void);
u32_t COMMON_NVM_IS_E3(void);
u32_t COMMON_NVM_FUNC_NUM(void);
u32_t COMMON_NVM_PATH_NUM(void);
common_nvm_status_t common_nvm_select_other_func(void);

// Debug version will have _DEBUG set
#if defined( _DEBUG )
#define FWUPGNX2_LOGMSG  FwUpgNx2LogMsg
#else
#define FWUPGNX2_LOGMSG
#endif /* _DEBUG */

#endif // FWUPGTOOL

common_nvm_status_t common_nvm_is_4port(int *result);

typedef enum {
        EDEBUG_GRC_DUMP_MODE = 0,
        EDEBUG_NO_PCI_MODE,
        EDEBUG_ENG_MODE,
        EDEBUG_NO_INIT_MODE,
	EDEBUG_ENG_NO_UART_MODE,
	EDEBUG_VIRTUAL_NVM_MODE
} edebug_eng_mode_t;

typedef enum {
	PORT_MODE_2_PORTS,
	PORT_MODE_4_PORTS,
	PORT_MODE_INVALID
} port_mode_t;

typedef enum {
	CHIP_TYPE_E1,
	CHIP_TYPE_E1X,
	CHIP_TYPE_E2,
	CHIP_TYPE_E3,
	CHIP_TYPE_INVALID
} chip_type_t;

typedef struct {
	port_mode_t port_mode;
	chip_type_t chip_type;
	int nvm_size;
	FILE *fd;
	u8 *nvm_buff;
} virtual_nvm_t;


#define COMMON_NVM_STATUS_SUCCESS             0
#define COMMON_NVM_STATUS_FAILURE            -1     /* General failure */
#define COMMON_NVM_STATUS_RESOURCE           -2     /* Memory failure  */
#define COMMON_NVM_STATUS_ACCESS             -3     /* Access failure  */
#define COMMON_NVM_STATUS_INVALID_PARAMETER  -4     /* Invalid parameters  */
#define COMMON_NVM_STATUS_PENDING            -5	  /* Pending for l2pkt loopback to complete */
#define COMMON_NVM_STATUS_NVM_INVAL_DIR      -6     /* Invalid directory index */
#define COMMON_NVM_STATUS_NVM_BAD_VALUE      -7     /* Bogus value */
#define COMMON_NVM_STATUS_TIMEOUT            -8     /* Timeout */
#define COMMON_NVM_STATUS_CON_CLOSED         -9     /* Connection closed */
#define COMMON_NVM_STATUS_ABORTED            -10    /* Packet aborted */
#define COMMON_NVM_STATUS_UPLOAD_IN_PROGRESS -11    /* Upload in progress */


#define CRC_TRAILER    0x43474c51 /* QLGC in ascii */
#define NVM_LITTLE_ENDIAN_FLAG "-l"
#define NEW_NCSI_START_VERSION  10007
#define IMAGE_HDR_TYPE_ISCSI_BOOT_CFG_PRG_LGCY     0x70636269
#define IMAGE_HDR_TYPE_ISCSI_BOOT_CPRG_LGCY        0x6c706269

#define PRG_NVM_LITTLE_ENDIAN    (0<<0)
#define PRG_NVM_BIG_ENDIAN       (1<<0)
#define PRG_NVM_APPEND_CRC       (1<<1)
#define PRG_NVM_MAC_PRG          (1<<2)
#define PRG_NVM_LIC_PRG          (1<<3)
#define PRG_NVM_SKIP_CFG_PRG     (1<<4)
#define PRG_NVM_IBCFG            (1<<5)
#define PRESERVE_NVM_EC          (1<<6)
#define PRESERVE_NVM_SN          (1<<7)
#define PRG_HANDLE_MACP          (1<<8)
#define PRG_NVM_EXTRA_PADDING    (1<<9)
#define PRG_NVM_FEBCFG           (1<<10)
#define PRG_NVM_NIV              (1<<11)


#define NVM_ATMEL_PAGE_SIZE      264

#define IMG_TYPE_ATMEL       1
#define IMG_TYPE_ST          2

#define NVM_CFG_KEY_DEFAULT      "default"
#define NVM_CFG_KEY_CANCEL       "cancel"
#define NVM_CFG_KEY_SAVE         "save"

#define NVM_FILL_PTTN_INC        "inc"
#define NVM_FILL_PTTN_ADDR       "addr"

#define NVM_ERASE_VALUE          0

#define NVM_IMAGE_NAME_BC1             		"BC1"
#define NVM_IMAGE_NAME_BC2             		"BC2"
#define NVM_IMAGE_NAME_MBA             		"MBA"
#define NVM_IMAGE_NAME_L2RXP           		"L2_RXP"
#define NVM_IMAGE_NAME_IPMI            		"IPMI"
#define NVM_IMAGE_NAME_ISCSI_BOOT      		"ISCSI_B"
#define NVM_IMAGE_NAME_ISCSI_BOOT_IPV6 		"ISCSI_B_IPV6"
#define NVM_IMAGE_NAME_ISCSI_BOOT_IPV4N6 	"ISCSI_B_IPV4N6"
#define NVM_IMAGE_NAME_ISCSI_BOOT_CFG  		"I_B_CFG"
#define NVM_IMAGE_NAME_ISCSI_BOOT_CFG_V2 	"I_B_CFG_V2"
#define NVM_IMAGE_NAME_ISCSI_BOOT_CFG2 		"I_B_CFG2"
#define NVM_IMAGE_NAME_ISCSI_BOOT_CFG2_V2 	"I_B_CFG2_V2"
#define NVM_IMAGE_NAME_ISCSI_BOOT_CFG3 		"I_B_CFG3"
#define NVM_IMAGE_NAME_ISCSI_BOOT_CFG4 		"I_B_CFG4"
#define NVM_IMAGE_NAME_ISCSI_CPRG      		"I_B_CPRG"
#define NVM_IMAGE_NAME_NCSI_CMN        		"NCSI_CMN"
#define NVM_IMAGE_NAME_NCSI_ELIB       		"NCSI_ELIB"
#define NVM_IMAGE_NAME_L2T             		"L2T"
#define NVM_IMAGE_NAME_L2C             		"L2C"
#define NVM_IMAGE_NAME_L2X             		"L2X"
#define NVM_IMAGE_NAME_L2U             		"L2U"
#define NVM_IMAGE_NAME_L2TI            		"L2TI"
#define NVM_IMAGE_NAME_L2CI            		"L2CI"
#define NVM_IMAGE_NAME_L2XI            		"L2XI"
#define NVM_IMAGE_NAME_L2UI            		"L2UI"
#define NVM_IMAGE_NAME_MODULES_PN      		"MODULES_PN"
#define NVM_IMAGE_NAME_FCOE_BOOT      		"FCOE_B"
#define NVM_IMAGE_NAME_FCOE_BOOT_CFG  		"FEB_CFG"
#define NVM_IMAGE_NAME_FCOE_BOOT_CFG2 		"FEB_CFG2"
#define NVM_IMAGE_NAME_FCOE_BOOT_CFG3 		"FEB_CFG3"
#define NVM_IMAGE_NAME_FCOE_BOOT_CFG4 		"FEB_CFG4"
#define NVM_IMAGE_NAME_FCOE_BOOT_CPRG 		"FEB_CPRG"
#define NVM_IMAGE_NAME_FCOE_BOOT_CPRG_LGCY 	"FEB_CPRG_LGCY"
#define NVM_IMAGE_NAME_FCOE_BOOT_CPRG_EVRST	"FEB_CPRG"
#define NVM_IMAGE_NAME_NIC_PARTITION_CFG    "NIC_PRTION"
#define NVM_IMAGE_NAME_VPD                  "VPD"
#define NVM_IMAGE_NAME_E3_WC                "E3_WC"
#define NVM_IMAGE_NAME_E3_PCIE              "E3_PCIE"
#define NVM_IMAGE_NAME_NIV_CFG              "NIV"
#define NVM_IMAGE_NAME_NIV_PROFILES_CFG     "NIVPROFILES"
#define NVM_IMAGE_NAME_BOOT_CFG_SHADOW      "BOOT_CFG_SHADOW"
#define NVM_IMAGE_NAME_CFG_EXTENDED_SHARED  "CFG_SHARED"
#define NVM_IMAGE_NAME_SWIM1       	        "SWIM1"
#define NVM_IMAGE_NAME_SWIM2                "SWIM2"
#define NVM_IMAGE_NAME_SWIM3                "SWIM3"
#define NVM_IMAGE_NAME_SWIM4                "SWIM4"
#define NVM_IMAGE_NAME_SWIM5                "SWIM5"
#define NVM_IMAGE_NAME_SWIM6                "SWIM6"
#define NVM_IMAGE_NAME_SWIM7                "SWIM7"
#define NVM_IMAGE_NAME_SWIM8                "SWIM8"

// change this value if num of swim changes, otherwise it might cause severe bugs!!!!
#define NVM_NUM_OF_SWIMS 8

#define NVM_IMAGE_NAME_SWIM1_B     	        "SWIM1B"
#define NVM_IMAGE_NAME_SWIM2_B              "SWIM2B"
#define NVM_IMAGE_NAME_SWIM3_B              "SWIM3B"
#define NVM_IMAGE_NAME_SWIM4_B              "SWIM4B"
#define NVM_IMAGE_NAME_SWIM5_B              "SWIM5B"
#define NVM_IMAGE_NAME_SWIM6_B              "SWIM6B"
#define NVM_IMAGE_NAME_SWIM7_B              "SWIM7B"
#define NVM_IMAGE_NAME_SWIM8_B              "SWIM8B"

#define NVM_IMAGE_NAME_MFW1            		"MFW1"
#define NVM_IMAGE_NAME_MFW2            		"MFW2"
#define NVM_IMAGE_NAME_MFW2_A         	    "MFW2_A"
#define NVM_IMAGE_NAME_OCNVM         	    "OCNVM"
#define NVM_IMAGE_NAME_E3_WCV2              "E3_WC_V2"
#define NVM_IMAGE_NAME_E3_PCIEV2            "E3_PCIE_V2"
#define NVM_IMAGE_NAME_CCM                  "CCM"
#define NVM_IMAGE_NAME_HW_SET               "HW_SET"
#define NVM_IMAGE_NAME_USR_BLK              "USR_BLK"
#define NVM_IMAGE_NAME_ISCSI_PERS           "ISCSI_PERS"
#define NVM_IMAGE_NAME_BDN                  "BDN"
#define NVM_IMAGE_NAME_MDUMP                "MDUMP"

#define MAX_VER_STR_SIZE                         50
#define UPDATE_CFG_MAX_ERR_MSG_LEN               80

#define EXTRACT_CFG_INIT_VALUE 0x999999
#define EXTRACT_CFG_VAL_VALUE  1
#define EXTRACT_CFG_VAL_STRING                   0


#define MANUF_INFO_PCI_VENDOR_ID(id)         (((id)>>16)&0xffff)
#define MANUF_INFO_PCI_DEVICE_ID(id)         ((id)&0xffff)
#define MANUF_INFO_PCI_ID(v,d)               (((u32_t)(d) & 0xffff) | \
                                             (((u32_t)(v)<<16) & 0xffff0000))
#define MANUF_INFO_PCI_SUBSYS_DEVICE_ID(id)  (((id)>>16)&0xffff)
#define MANUF_INFO_PCI_SUBSYS_VENDOR_ID(id)  ((id)&0xffff)
#define MANUF_INFO_PCI_SUBSYS_ID(v,d)        (((u32_t)(v) & 0xffff) | \
                                             (((u32_t)(d)<<16) & 0xffff0000))
#define PRIM_MAC           0
#define DUMMY_MAC          1
#define BACKUP_MAC         2

typedef enum {
	MBA_MBA_LEGACY_IDX=0,
	MBA_MBA_PCI3CLP_IDX,
	MBA_MBA_PCI3_IDX,
	MBA_FCODE_IDX,
	EFI_X86_IDX,
	EFI_IPF_IDX,
	EFI_EBC_IDX,
	EFI_X64_IDX
} mba_image_t;

typedef enum {
    NVM_NOT_ENOUGH_ROOM=0,
    NVM_ENOUGH_ROOM,
    NVM_ENOUGH_ROOM_EXCEPT_2_SWIM_GROUPS
} nvm_enough_room_result_t;

enum delete_mfw_cmd {
    DELETE_MFW_CMD_DONT_DELETE=0,
    DELETE_MFW_CMD_DELETE_ALL_MFWS=1,
    DELETE_MFW_CMD_ALL_MFWS_BESIDES_MFW1=2
};

#define ROMIMG_NUM_MAX 8

typedef enum {
    DEF_CFG_IDX_DID_STRING,
    DEF_CFG_IDX_PWR_CONSUMED,
    DEF_CFG_IDX_PWR_DISSIPATED,
    DEF_CFG_IDX_MAX
} def_cfg_idx_t;

typedef struct _default_id_table_t
{
    u32_t id;
    char *dev_info[DEF_CFG_IDX_MAX];
} default_id_table_t;

#define NVM_IMAGE_NAME_BOOTSTRAP                "BOOTSTRAP"
#define NVM_IMAGE_NAME_DIRECTORY                "DIRECTORY"
#define NVM_IMAGE_NAME_HW_INFO                  "HW_INFO"
#define NVM_IMAGE_NAME_FEATURE_INFO             "FEAT_INFO"
#define NVM_IMAGE_NAME_PATH1_FEATURE_INFO       "FEAT_INFO_2"
#define NVM_IMAGE_NAME_PATH1_HW_INFO            "HW_INFO_2"
#define NVM_IMAGE_NAME_MANUF_KEY_INFO_P0        "MKEY_INFO_P0"
#define NVM_IMAGE_NAME_UPGRADE_KEY_INFO_P0      "UKEY_INFO_P0"
#define NVM_IMAGE_NAME_MANUF_KEY_INFO_P1        "MKEY_INFO_P1"
#define NVM_IMAGE_NAME_UPGRADE_KEY_INFO_P1      "UKEY_INFO_P1"

#define BC_VERSION_OF_SHMEM_OFFSET_FIX_MAJOR 5
#define BC_VERSION_OF_SHMEM_OFFSET_FIX_MINOR 2
#define BC_VERSION_OF_SHMEM_OFFSET_FIX_BUILD 5
#define MISC_REG_SHARED_MEM_OLD_VAL 0xaf900


enum image_map_idx_t
{
        IMAGE_MAP_IDX_BC1,
        IMAGE_MAP_IDX_BC2,
        IMAGE_MAP_IDX_MBA,
        IMAGE_MAP_IDX_NCSI_CMN,
        IMAGE_MAP_IDX_NCSI_ELIB,
        IMAGE_MAP_IDX_L2T,
        IMAGE_MAP_IDX_L2C,
        IMAGE_MAP_IDX_L2X,
        IMAGE_MAP_IDX_L2U,
        IMAGE_MAP_IDX_IPMI,
	IMAGE_MAP_IDX_MODULES_PN,
        IMAGE_MAP_IDX_ISCSI_BOOT,
        IMAGE_MAP_IDX_ISCSI_BOOT_CFG,
        IMAGE_MAP_IDX_ISCSI_BOOT_CFG2,
        IMAGE_MAP_IDX_ISCSI_BOOT_CPRG,
        IMAGE_MAP_IDX_FCOE_BOOT,
        IMAGE_MAP_IDX_FCOE_BOOT_CFG,
        IMAGE_MAP_IDX_FCOE_BOOT_CFG2,
        IMAGE_MAP_IDX_FCOE_BOOT_CFG3,
        IMAGE_MAP_IDX_FCOE_BOOT_CFG4,
        IMAGE_MAP_IDX_FCOE_BOOT_CPRG,
	IMAGE_MAP_IDX_NICP_CFG,
	IMAGE_MAP_IDX_VPD,
	IMAGE_MAP_IDX_E3_WC,
	IMAGE_MAP_IDX_E3_PCIE,
	IMAGE_MAP_IDX_NIV_CFG,
	IMAGE_MAP_IDX_NIV_PROFILES_CFG,
	IMAGE_MAP_IDX_BOOT_CFG_SHADOW,
	IMAGE_MAP_IDX_CFG_EXTEBDED_SHARED,
	IMAGE_MAP_IDX_ISCSI_BOOT_CFG3,
	IMAGE_MAP_IDX_ISCSI_BOOT_CFG4,
	IMAGE_MAP_IDX_MFW1,
	IMAGE_MAP_IDX_MFW2,
	IMAGE_MAP_IDX_SWIM1,
	IMAGE_MAP_IDX_SWIM2,
	IMAGE_MAP_IDX_SWIM3,
	IMAGE_MAP_IDX_SWIM4,
	IMAGE_MAP_IDX_SWIM5,
	IMAGE_MAP_IDX_SWIM6,
	IMAGE_MAP_IDX_SWIM7,
	IMAGE_MAP_IDX_SWIM8,
	IMAGE_MAP_IDX_SWIM1_B,
	IMAGE_MAP_IDX_SWIM2_B,
	IMAGE_MAP_IDX_SWIM3_B,
	IMAGE_MAP_IDX_SWIM4_B,
	IMAGE_MAP_IDX_SWIM5_B,
	IMAGE_MAP_IDX_SWIM6_B,
	IMAGE_MAP_IDX_SWIM7_B,
	IMAGE_MAP_IDX_SWIM8_B,
	IMAGE_MAP_IDX_MFW2_A,
	IMAGE_MAP_IDX_OCNVM,
	IMAGE_MAP_IDX_E3_WCV2,
	IMAGE_MAP_IDX_E3_PCIEV2,
	IMAGE_MAP_IDX_CCM,
	IMAGE_MAP_IDX_HW_SET,
	IMAGE_MAP_IDX_USR_BLK,
	IMAGE_MAP_IDX_ISCSI_PERS,
	IMAGE_MAP_IDX_BDN,
	IMAGE_MAP_IDX_MDUMP
        };

typedef struct _image_map_t
{
    enum image_map_idx_t image_idx;
    char 		*name;
    char 		*opt;
    u32_t 		agent;
    u32_t 		type;
    u32_t 		hdr_type;
} image_map_t;

typedef enum {
        IDX_DUMMY = 0,
        IDX_L2_MAC_ADDRESS,
        IDX_POWER_DISS_COMMON,
        IDX_POWER_MAN_SCALE,
        IDX_POWER_DISSIPATED,
        IDX_POWER_CONSUMED,
        IDX_VID,
        IDX_VDID,
        IDX_SVID,
        IDX_SDID,
        //  IDX_BOARD_TYPE, - removed
        IDX_PROD_NAME = 11,
        IDX_PART_NUM,
        IDX_ENGR_CHNG,
        IDX_SERIAL_NUM,
        IDX_MANUF_ID,
        IDX_LED_MODE,
        IDX_UMP_NUM,
        IDX_BAR_SIZE,
        IDX_WOL_MAGIC_ENABLE,
        IDX_WOL_WINDOWS_DEFAULT,
        IDX_WOL_ACPI_UPON_MGMT,
        IDX_MBA_ENABLE,
        IDX_MBA_BOOT_AGENT,
        IDX_MBA_BOOT_TYPE,
        IDX_MBA_DELAY,
        IDX_MBA_HOTKEY,
        IDX_MBA_PROMPT,
        IDX_EXP_ROM_SIZE,
        IDX_MFW_ENABLE,
        IDX_MFW_TYPE_SELECT,
        IDX_AN_TYPE,
        IDX_AN_GRACE_PER,
        IDX_PCI_GEN2_ENABLE,
        IDX_CORE_CLOCK_CHANGE,
        IDX_SPEED_CAP_MASK_D0,
        IDX_SPEED_CAP_MASK_D3,
        IDX_ISCSI_MAC,
        IDX_RDMA_MAC,
        IDX_SERDES_EXT_PHY_TYPE,
        IDX_SERDES_EXT_PHY_ADDR,
        IDX_XGXS_EXT_PHY_TYPE,
        IDX_XGXS_EXT_PHY_ADDR,
        IDX_SERDES_BPLANE_FORCED_COEF,
        IDX_XGXS_BPLANE_FORCED_TX_COEF,
        IDX_XGXS_BPLANE_FORCED_RX_COEF,
        IDX_SERDES_BPLANE_COEF,
        IDX_XGXS_BPLANE_TX_COEF,
        IDX_XGXS_BPLANE_RX_COEF,
        IDX_LANE_SWAP_CONFIGURATION,
        IDX_PORT_SWAP,
        IDX_BCM_LOOPBACK_MODE_EN,
        IDX_MBA_VLAN_ENABLE,
        IDX_MBA_VLAN_VALUE,
        IDX_UMP_USE_RMII,                /* Teton II only */
        IDX_SPARE_NUM,
        IDX_10G_OVERCLK,
        IDX_SWITCH_CON,
        IDX_FLOW_CTL,
        IDX_MFW_WOL_10G_OVERCLK,
        IDX_MFW_WOL_SWITCH_CON,
        IDX_MFW_WOL_FLOW_CTL,
        IDX_SMBUS_ADDR,
        IDX_SMBUS_ENABLE,
        IDX_RESET_MCP_ON_CORE_RESET,
        IDX_BOARD_VERSION,
        IDX_BEACON_WOL_EN,
        IDX_MDIO_VOLTAGE,
        IDX_MBA_LINK_SPEED,
        IDX_FORCE_SF_MODE=73,
        IDX_SECOND_EXT_PHY_TYPE=98,
#ifdef FWUPGTOOL
    IDX_PHY_SELECTION=104,
#endif // FWUPGTOOL
        IDX_DCBX_ENABLE=105,
        IDX_MBA_BOOT_RETRY=107,
#ifdef FWUPGTOOL
		IDX_SECOND_EXT_PHY_SPEED_CAP_MASK_D0,
		IDX_SECOND_EXT_PHY_SPEED_CAP_MASK_D3, // 109
#endif // FWUPGTOOL
		IDX_NIV_IMAGE_VERSION=138,
		IDX_IFFE=202,
		IDX_REV_ID_CTRL=232,
        IDX_CONFIG_MAX
} nvm_cfg_idx_t;

#define DATA_TYPE_SHARED_SECRET        0
#define DATA_TYPE_MANUF_KEY            1
#define DATA_TYPE_UPGRADE_KEY          2
#define DATA_TYPE_CURR_KEY             3     /* the key in shmem */
#define DATA_TYPE_MAX                  4

#ifndef FWUPGTOOL
// For FWUPG, this has been defined in nvm.c for NX2 1G...
void swap_buffer_if_big_endian_host( u32_t *pBuf, u32_t uLen );
#endif // !FWUPGTOOL
u8_t common_nvm_is_oem_npar_device(u16_t vendor_id, u16_t sub_vendor_id, u16_t sub_device_id);
u32_t common_nvm_program_nvm(u8_t *buf, u32_t byte_cnt, u32_t offset,
                         u32_t flag, u32_t *total_byte_ptr);
u32_t common_nvm_xlate_flash_virt_addr (u32_t phy_addr);
u32_t common_nvm_xlate_flash_phy_addr (u32_t virt_addr);
common_nvm_status_t common_nvm_fix_shmem_offset_for_older_versions(void);

common_nvm_status_t common_nvm_read_nvram(
    u32_t   addr,
    u32_t * val_p,
    u32_t   byte_cnt
    );

common_nvm_status_t common_nvm_read_NVM_or_image(
#define READ_VERSION_FROM_NVM 0
#define READ_VERSION_FROM_IMAGE 1
		u8_t                    read_flag,
		common_nvm_virt_addr_t  base_addr,
		u32_t                   offset,
		u32_t *                 val_p,
		u32_t                   byte_cnt
		);

common_nvm_status_t common_nvm_write_nvram(
    u32_t addr,
    u32_t * val_p,
    u32_t   byte_cnt
    );

common_nvm_status_t common_nvm_update_mba_hdr  (
        u8_t *buffer,
        u16_t vendor_id,
        u16_t device_id
        );

common_nvm_status_t common_nvm_nvm_find_dir_entry(
        IN   u32_t image_type,
        OUT  u32_t *entry_idx,
		OUT  u32_t *start_address
        );

u32_t common_nvm_find_empty_entry(
    OUT   u32_t *entry_idx
    );

nvm_enough_room_result_t enough_room(
    u32_t is_e1x,
    u32_t image_type,
    u32_t byte_cnt,
    u32_t *image_types_in_input_mfw,
    u32_t input_mfw_images_found);

common_nvm_status_t common_nvm_find_room(
    u32_t image_type,
    u32_t byte_cnt,
    extended_dir_image_t *extended_dir_p,
    u32_t forbidden_nvm_addr,
    u32_t forbidden_nvm_size,
    u32_t *nvm_addr_ptr
);

common_nvm_status_t common_nvm_nvm_get_dir_info(
    u32_t entry_idx,
    u32_t *nvm_addr_ptr,
    u32_t *byte_cnt_ptr,
    u32_t *sram_addr_ptr,
    u32_t *image_type_ptr,
    u32_t *agent_ptr
        );

u32_t common_nvm_update_dir_entry(
    u32_t entry_idx,
    u32_t nvm_addr,
    u32_t byte_cnt,
    u32_t sram_addr,
    u32_t image_type,
    u32_t agent_type
    );

u32_t common_nvm_fw_ver_code(
        u32_t nvm_addr
        );

u32_t common_nvm_image_table_size();

common_nvm_status_t common_nvm_image_version_validation(
	const char *image_name,
	char *old_version,
	char *new_version,
	u8_t *same_image
);

common_nvm_status_t common_nvm_image_mba_version_validation (
	common_nvm_virt_addr_t file_buf,
	common_nvm_virt_addr_t nvm_addr,
	u8_t *same_image
);

common_nvm_status_t common_nvm_get_romimg_vers(
	u8_t read_source,
	common_nvm_virt_addr_t addr,
	u32_t *vers,
	char **type_str
);

void common_nvm_fw_ver_string(
    common_nvm_virt_addr_t	nvm_addr,
    const char 			*name,
    char       			*str_ptr,
    u8_t                read_source
    );

common_nvm_status_t common_nvm_update_bootstrap (
    u32_t sram_start_addr,
    u32_t byte_cnt,
    u32_t nvm_start_addr
    );

common_nvm_status_t common_nvm_read_nvm_cfg(
    nvm_cfg_t *cfg_ptr,
    u8_t path_num,
    u8_t port_num,
    media_vpd_t *vpd_block_ptr,
    dir_t *dir_block_ptr,
    nvm_nic_part_cfg_t *mac_partition_block_ptr,
    u32_t *nvm_cfg_extended_shared_image_ptr
    );

common_nvm_status_t common_shmem_load_cfg(
    nvm_cfg_t *shmem_cfg_ptr
    );

common_nvm_status_t common_nvm_load_cfg(
    nvm_cfg_t *cfg_ptr,
    nvm_cfg_t *shmem_cfg_ptr,
    nvm_nic_part_cfg_t *mac_partition_ptr,
    u32_t *nvm_cfg_extended_shared_ptr
    );

void common_nvm_load_shmem(
    u32_t shmem_base,
    u8_t port_num,
    nvm_cfg_t *shmem_cfg_ptr
    );

void common_nvm_store_shmem(
    u32_t shmem_base,
    u8_t update_port_cfg,
    u8_t port_num,
    nvm_cfg_t *shmem_cfg_ptr
    );

common_nvm_status_t common_nvm_program_nvm_cfg(
    nvm_cfg_t *cfg_ptr,
    u8_t path_num,
    u8_t update_port_cfg,
    manuf_info_t *manuf_block_ptr,
    feature_info_t *feature_block_ptr,
    dir_t *dir_block_ptr,
    media_vpd_t *vpd_block_ptr,
    nvm_nic_part_cfg_t *mac_partition_ptr,
    u32_t *cfg_extended_shared_block_ptr
    );

common_nvm_status_t common_nvm_store_cfg(
    nvm_cfg_t *cfg_ptr,
    nvm_cfg_t *shmem_cfg_ptr,
    nvm_nic_part_cfg_t *mac_partition_ptr,
    u32_t *cfg_extended_shared_block_ptr
    );

void common_nvm_compute_crc(
        u8_t *buf,
        u32_t byte_cnt,
        u32_t big_endian,
        u32_t *crc_ptr
        );

common_nvm_status_t common_nvm_nvm_program_image (
    u8_t *buf,
    u32_t byte_cnt,
    u32_t code_image
    );

common_nvm_status_t common_nvm_prog_image_by_idx (
    u8_t *buf,
    u32_t byte_cnt,
    u32_t image_table_idx,
	u8_t validate_versions,
	u8_t mbi_upgrade
	);

common_nvm_status_t common_nvm_dump_nvm_into_buf (
    u8_t *buf,
    u32_t big_endian,
    u32_t byte_cnt,
    u32_t offset,
    u32_t *byte_dumped
	);

common_nvm_status_t common_nvm_validate_extended_dir_byte_cnt();

common_nvm_status_t common_nvm_prog_image_in_extended_dir (
    u8_t *buf,
    u32_t byte_cnt,
    u32_t image_table_idx,
    u8_t validate_versions,
    u8_t silent,
    u8_t remove_swims,
	u8_t mbi_upgrade
    );

u32_t common_nvm_nvm_size(void);

common_nvm_status_t common_nvm_compute_image_crc (
        u32_t start_nvm_addr,
        u32_t byte_cnt,
        u32_t *computed,
        u32_t *stored
        );

/**
 * Returns DIAG_STATUS_SUCCESS if file_image is
 * a legacy iSCSI image.
 */
common_nvm_status_t common_nvm_nvm_is_legacy_iscsi_image(
    image_header_t * file_image_p
    );

/*
 *------------------------------------------------------------------------
 * dg_nvm_fill_mac_partition_info --
 *
 *     Fills input struct with MAC partition info read from NVM image,
 *     or zeros if such image does not exist.
 *
 * Results:
 *     DIAG_STATUS_SUCCESS if operation succeeded (even if image does
 *     not exist)
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_nvm_fill_mac_partition_info (nvm_nic_part_cfg_t *mac_partition_block, u32_t *result);

common_nvm_status_t common_nvm_nvm_fill_nvm_cfg_extended_shared(u32_t *nvm_cfg_extended_shared_image, u32_t max_image_size);

/*
 *------------------------------------------------------------------------

 * dg_program_mac_partition_to_nvm --
 *
 *     Checks if user changed MAC partition configuration. If so, save it to
 *     MAC partition image in extended dir. If such an image didn't exist,
 *     create it.
 *
 * Results:
 *     DIAG_STATUS_SUCCESS if operation succeeded.
 *
 * Side effects:
 *     Creation of MAC partition image in extended dir if it didn't exist
 *     and user changed MAC partition configuration.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_program_mac_partition_to_nvm(nvm_cfg_t *nvm_cfg, nvm_nic_part_cfg_t *mac_partition_ptr);
common_nvm_status_t common_nvm_create_mac_partition_image(void);

common_nvm_status_t common_nvm_program_extended_nvm_cfg_shared_block(nvm_cfg_t *nvm_cfg, u32_t *nvm_cfg_extended_shared_block);

#ifndef FWUPGTOOL
#ifndef WIN_DIAG
/**   Update device information of all devices and environment MAC addr.
 *    Used after "nvm prg" "nvm fill" "nvm cfg"
 *
 *    For FWUPG tool, we don't issue the MCP reset and update the device information.
 **/
void common_nvm_refresh_devices();
#endif // WIN_DIAG
#endif // !FWUPGTOOL


common_nvm_status_t common_nvm_nvm_parse_extended_dir_image(
		IN u32_t offset,
		IN u32_t length,
		IN extended_dir_image_t *extended_dir_ptr);

/*
 *------------------------------------------------------------------------
 * dg_handle_mac_partition_data --
 *
 *     If user requested to remember old mac partition values, nvm data will
 *     hold data according to the table below:
 *
 *     original nvm     new nvm
 *     included         included
 *     mac_p image?     mac_p image?   logics
 *     ------------     -----------    ------
 *     No               No             no mac_p imahe
 *     No               No             no mac_p imahe
 *     Yes              No             original mac_p image with empty MACs
 *     Yes              Yes            original mac_p image with new MACs
 *
 * Input:
 *     macp_image_found - Whether mac_p existed in source NVM
 *     macp_image - values of mac+p image if existed in source NVM
 *
 *
 * Output:
 *      None, besides "side effect" of new NVM holding right mac_p image
 *
 * Results:
 *     DIAG_STATUS_SUCCESS if operation succeeded.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_handle_mac_partition_data(u32_t orig_macp_image_found,
                           nvm_nic_part_cfg_t orig_macp_image);

/*
 *------------------------------------------------------------------------
 * common_nvm_nvm_find_image_in_extended_dir --
 *
 *     Checks if an image suitable to input type exists in extended dir.
 *     If yes, returns its parameters.
 *     Function can handle scenario that extended dir image itself doesn't
 *     exist.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_nvm_find_image_in_extended_dir(u32_t *found_image, u32_t image_type, u32_t *nvm_addr, u32_t *byte_cnt, u32_t *entry_idx);

/*
 *------------------------------------------------------------------------
 * common_nvm_find_entry_in_image_table --
 *
 *     Returns index of image in global image table.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_find_entry_in_image_table(u32_t image_type, u32_t *index);

common_nvm_status_t common_nvm_get_image_in_extended_dir_info(char *image_name, u32_t *offset, u32_t *length, u32_t *image_table_idx);

common_nvm_status_t common_nvm_delete_vpd_image();

common_nvm_status_t common_nvm_delete_ccm_image();

common_nvm_status_t common_nvm_delete_niv_images();

common_nvm_status_t common_nvm_delete_bc_and_ncsi_images();

common_nvm_status_t common_nvm_delete_mfw_and_swim_images(enum delete_mfw_cmd, u32_t delete_swims);

common_nvm_status_t common_nvm_delete_unused_swim_images(u32_t *used_group);

common_nvm_status_t common_nvm_delete_swim_group_b_images();

common_nvm_status_t common_nvm_read_mfw_version(char *mfw_version);

common_nvm_status_t common_nvm_read_mba_efi_versions(char *mba_version, char *efi_version);

common_nvm_status_t common_nvm_nvm_find_extended_dir_entry(
        IN   u32_t 					image_type,
	IN   extended_dir_image_t 	                *extended_dir_p,
        OUT  u32_t 					*entry_idx,
        OUT  u32_t 					*start_address,
        OUT  u32_t                                      *sram_addr,
	OUT  u32_t 				        *byte_cnt,
	OUT  u32_t                                      *agent
        );

common_nvm_status_t common_nvm_nvm_delete_extended_dir_entry(
		IN extended_dir_image_t *extended_dir_p,
		IN u32_t entry_idx,
		IN u32_t nvm_offset);

common_nvm_status_t common_nvm_get_nvm_offset_for_mac_addr(IN u32_t hwkey_used);
common_nvm_status_t common_nvm_get_nvm_offset_for_license(u32_t license_type, u32_t *nvm_addr_p, u32_t *blk_start_p);
common_nvm_status_t common_nvm_get_key_offset_from_shmem(u32_t *key_offset_p);
common_nvm_status_t common_nvm_get_nvm_offset_for_pci_id( u32_t *pci_id_p, u32_t *pci_sub_id_p);

/* new functions implemented due to common_nvm layer */
void common_nvm_generate_hex_int_object(Tcl_Obj **obj_ptr, u32_t val);
void common_nvm_set_hex_int_object(Tcl_Obj **obj_ptr, u32_t val);
u32_t common_nvm_get_total_br_dev(void);
void *common_nvm_memset(void  *buf, u8_t ch, u32_t buf_size);
u32_t common_nvm_number_of_everest_functions(void);
u32_t common_nvm_tcl_al_get_no_pci_mode(void);
#ifndef FWUPGTOOL
// Not applicable to FWUPG...
u32_t common_nvm_tcl_al_get_virtual_nvm_mode(void);
#endif // !FWUPGTOOL
void common_nvm_vpd_set_defaults(vpd_info_t *vpd_info, u32_t mask);
u32_t common_nvm_set_vpd_info(vpd_info_t *vpd_info, media_vpd_t *vpd_e);
#ifndef FWUPGTOOL
common_nvm_status_t common_nvm_dev_read_nvram(IN dev_if_t *cur_dev, IN u32_t addr, OUT u32_t *val_p, IN u32_t byte_cnt);
common_nvm_status_t common_nvm_dev_write_nvram(IN dev_if_t *cur_dev, IN u32_t addr, IN u32_t *val_p, IN u32_t byte_cnt);
#endif // !FWUPGTOOL
void common_nvm_init_npar_mac_addr(u32_t pf, int is_4port, u32_t is_nicp1_5, u32_t wwn_prefix, nvm_nic_part_cfg_t *npar_block_ptr);
void add_shadow_dell_ncsi_image();
void add_bdn_image();
common_nvm_status_t common_nvm_init_npar_nvm_image(int is_e3);
u8_t common_nvm_get_duart_port(void);
common_nvm_status_t common_nvm_duart_nvm_read(IN u8_t io_port, IN u32_t addr, OUT u32_t *buf, IN u32_t buf_sz);
common_nvm_status_t common_nvm_duart_nvm_write(IN u8_t io_port, IN u32_t addr, IN u32_t *buf, IN u32_t buf_sz);
void common_nvm_get_vpd_info(vpd_info_t *vpd_info, media_vpd_t *vpd_e, u8_t vpd_check);
void common_nvm_swap_bytes(u32_t  *buf, u32_t  buf_size);
u32_t common_nvm_le32_to_cpu(u32_t val);
common_nvm_virt_addr_t common_nvm_alloc_virt(IN u32_t sz);
void common_nvm_free_virt(IN common_nvm_virt_addr_t virt_addr);
common_nvm_status_t common_nvm_get_port_num(u32_t *result);
common_nvm_status_t common_nvm_get_path_num(u32_t *result);
common_nvm_status_t common_nvm_get_func_num(u32_t *result);
void *common_nvm_memcpy(IN void *dest, IN const void *src, IN u32_t bnum);
common_nvm_status_t common_nvm_select_other_func(void);
#ifndef FWUPGTOOL
void common_nvm_get_info (OUT common_nvm_dev_stat_t * dev_stat_p);
u32_t common_nvm_get_func_num_from_dev_info (common_nvm_dev_info_t *p_dev_info);
char *common_nvm_get_chip_num(common_nvm_dev_stat_t *dev_stat);
u32_t common_nvm_nvm_size_from_dev_info(common_nvm_dev_info_t *p_dev_info);
#endif
common_nvm_status_t common_nvm_reg_read(IN u32_t addr, OUT u32_t *val32_p);
common_nvm_status_t common_nvm_reg_write(IN u32_t addr, IN u32_t val32);
#ifndef FWUPGTOOL
u32_t common_nvm_duart_get_nvm_size(void);
common_nvm_status_t common_nvm_duart_get_chip_id(u32_t *chip_id_p);
u32_t is_vpd_image_exist(void);
#else
u32_t dg_number_of_everest_functions(void);
void dg_os_swap_bytes(u32_t *buf, u32_t buf_size);
u32_t is_vpd_image_exist(NVM_INTERFACE iNvm, u32_t* offset, u32_t* length);
#endif // !FWUPGTOOL

common_nvm_status_t common_nvm_get_nvm_offset( u32_t offset, u32_t *nvm_offset_ptr);
common_nvm_status_t common_nvm_get_shmem_offset( u32_t offset, u32_t *shmem_addr_ptr);

#ifdef EDEBUG

/* special handling to enable DbgMessage in commonal nvm layer */
void DbgMessageXX(void *dummy, int level, char *msg, ...);
#define DbgMessage              DbgMessageXX
#define DbgBreakMsg(_m)         DbgMessageXX(NULL, FATAL, _m)
#define DbgBreak()              DbgMessageXX(NULL, FATAL, "DbgBreak occured!!!\n")

#define CP_NVM                  0x00000200
#define CP_ALL                  0xffffff00
#define LV_VERBOSE              0x04
#define LV_INFORM               0x03
#define LV_WARN                 0x02
#define LV_FATAL                0x01
#define VERBOSEnv               (CP_NVM | LV_VERBOSE)
#define INFORMnv                (CP_NVM | LV_INFORM)
#define WARNnv                  (CP_NVM | LV_WARN)
#define WARN                    (CP_ALL | LV_WARN)
#define INFORM                  (CP_ALL | LV_INFORM)
#define VERBOSE                 (CP_ALL | LV_VERBOSE)
#define FATAL                   (CP_ALL | LV_FATAL)

/* functions which need specific implementation in eDebug */
common_nvm_status_t tcl_al_log_command(IN char* opString, IN int objc, IN Tcl_Obj* CONST objv[]);
void print_msg(IN char *msg, IN u32_t color);
int get_vaddr(IN  Tcl_Interp *interp, IN Tcl_Obj *CONST objv[], IN u32_t arg_idx, OUT u32_t *low_addr, OUT u32_t *high_addr);

#endif

#endif /* __COMMON_NVM_H__ */

