/****************************************************************************
 * Copyright(c) 2001-2014 Broadcom Corporation, all rights reserved
 * Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
 * Proprietary and Confidential Information.
 *
 * This source file is the property of QLogic Corporation, and
 * may not be copied or distributed in any isomorphic form without
 * the prior written consent of QLogic Corporation.
 *
 * Name:        decode_license.c
 *
 * Description: Decode structures in license.h 
 *
 * Created:     File generated from \work\vdiag\license.h in 3/26/2010 dlau
 *     !!! Majority of the code is script generated. 
 *    Avoid modifying this file manually. Consult David Lau for help. 
 *
 ****************************************************************************/

#include  <stdio.h>
#include  <ctype.h>
#include  <stdlib.h>
#include  <string.h>
#include "tcl.h"
#include "bcmtype.h"
#include "license.h"


u32_t indent_level;

#ifndef INDENT_STRUCT
#define INDENT_STRUCT(tab_cnt) \
{\
	u32_t tab_index;\
	for (tab_index = 0; tab_index < tab_cnt; tab_index++) SPECIAL_DBG_TAB; \
}

#define STRUCT_PRINT(x)      INDENT_STRUCT(indent_level); SPECIAL_DBG_PRINT(x);
#endif // INDENT_STRUCT

static char disp_str[1000];
#ifndef SPECIAL_DBG_PRINT
#define SPECIAL_DBG_PRINT(x)   \
{\
	sprintf x;\
	Tcl_AppendStringsToObj( rc_ptr, disp_str, NULL); \
}
#endif // SPECIAL_DBG_PRINT
#ifndef SPECIAL_DBG_TAB
#define SPECIAL_DBG_TAB  Tcl_AppendStringsToObj( rc_ptr, "\t", NULL) 
#endif // SPECIAL_DBG_TAB


/*****************************************************************************

FUNCTION:       DecodeLicenseKey

DESCRIPTION:    This routine decode the structure/union license_key_t

IN PARAMETERS:  pointer to license_key_t

OUT PARAMETERS: none

RETURNS:        none

*****************************************************************************/
void DecodeLicenseKey(Tcl_Obj* rc_ptr, const license_key_t * data)
{
	u32_t index;
	indent_level += 1;  // increase indentation for each level of structure decoding

    for (index = 0; index < sizeof(license_key_t)/sizeof(u32_t); index ++)
    {
        ((u32_t *)data)[index] ^= FW_ENCODE_32BIT_PATTERN;
    }
	STRUCT_PRINT((disp_str,  "digest[0 .. %d] = ", (HASH_VALUE_SIZE) - 1));
	SPECIAL_DBG_PRINT((disp_str,  " (combine u8[] to u32_t[])"));
	for (index = 0; index < (sizeof(data->digest)+sizeof(u32_t)-1)/sizeof(u32_t); index++)  // array (round to word) 
	{
		if ((index & 0x3) ==0){ SPECIAL_DBG_PRINT((disp_str,  "\n")); STRUCT_PRINT((disp_str,  "\t"));}
		SPECIAL_DBG_PRINT((disp_str,  " 0x%08x", ((u32_t *)data->digest)[index]));
	}
	SPECIAL_DBG_PRINT((disp_str,  "\n"));

	STRUCT_PRINT((disp_str,  "\t\""));
	for (index = 0; ((HASH_VALUE_SIZE) & 0x3) ==0 &&index < (HASH_VALUE_SIZE)/sizeof(u32_t); index++)  // array 
	{
		u32_t temp;
		temp = ((u32_t *)data->digest)[index];
		SPECIAL_DBG_PRINT((disp_str,  "%c%c%c%c", (char)(temp>>24), (char)(temp>>16), (char)(temp>>8), (char)temp));
	}
	SPECIAL_DBG_PRINT((disp_str,  "\"\n"));

	STRUCT_PRINT((disp_str,  "oem_id = 0x%x\n", (u32_t)(data->oem_id)));
	if ((data->oem_id & 0xFFFFFFFF) == OEM_ID_BRCM) 
	{
		STRUCT_PRINT((disp_str,  "\t\t(ID_BRCM)\n")); 
	}
	else if ((data->oem_id & 0xFFFFFFFF) == OEM_ID_HPQ) 
	{
		STRUCT_PRINT((disp_str,  "\t\t(ID_HPQ)\n")); 
	}
	STRUCT_PRINT((disp_str,  "dword_length = 0x%x\n", (u32_t)(data->dword_length)));
	STRUCT_PRINT((disp_str,  "version = 0x%x\n", (u32_t)(data->version)));
	if ((data->version & 0xFFFFFFFF) == VERSION_CURRENT) 
	{
		STRUCT_PRINT((disp_str,  "\t\t(CURRENT)\n")); 
	}
	STRUCT_PRINT((disp_str,  "key_type = 0x%x\n", (u32_t)(data->key_type)));
	if ((data->key_type & 0xFFFFFFFF) == KEY_TYPE_ENUM_BCM5706) 
	{
		STRUCT_PRINT((disp_str,  "\t\t(TYPE_ENUM_BCM5706)\n")); 
	}
	else if ((data->key_type & 0xFFFFFFFF) == KEY_TYPE_ENUM_BCM5710) 
	{
		STRUCT_PRINT((disp_str,  "\t\t(TYPE_ENUM_BCM5710)\n")); 
	}
	STRUCT_PRINT((disp_str,  "max_toe_conn = 0x%x\n", (u32_t)(data->max_toe_conn)));
	if ((data->max_toe_conn & 0xFFFFFFFF) == CONN_UNLIMITED) 
	{
		STRUCT_PRINT((disp_str,  "\t\t(UNLIMITED)\n")); 
	}
	STRUCT_PRINT((disp_str,  "capability = 0x%x\n", (u32_t)(data->capability)));
	STRUCT_PRINT((disp_str,  "\tRESERVED             = 0x%x \n", (u32_t)(data->capability & CAP_RESERVED)>>0));
	STRUCT_PRINT((disp_str,  "\tUSER_RDMA            = 0x%x \n", (u32_t)(data->capability & CAP_USER_RDMA)>>1));
	STRUCT_PRINT((disp_str,  "\tTOE                  = 0x%x \n", (u32_t)(data->capability & CAP_TOE)>>2));
	STRUCT_PRINT((disp_str,  "\tISCSI_INIT           = 0x%x \n", (u32_t)(data->capability & CAP_ISCSI_INIT)>>3));
	STRUCT_PRINT((disp_str,  "\tISCSI_TRGT           = 0x%x \n", (u32_t)(data->capability & CAP_ISCSI_TRGT)>>4));
	STRUCT_PRINT((disp_str,  "\tISER_INIT            = 0x%x \n", (u32_t)(data->capability & CAP_ISER_INIT)>>5));
	STRUCT_PRINT((disp_str,  "\tISER_TRGT            = 0x%x \n", (u32_t)(data->capability & CAP_ISER_TRGT)>>6));
	STRUCT_PRINT((disp_str,  "\tISCSI_BOOT           = 0x%x \n", (u32_t)(data->capability & CAP_ISCSI_BOOT)>>7));
	STRUCT_PRINT((disp_str,  "\tISCSI_FULL_ACCL      = 0x%x \n", (u32_t)(data->capability & CAP_ISCSI_FULL_ACCL)>>8));
	STRUCT_PRINT((disp_str,  "\tISCSI_HDR_DGST       = 0x%x \n", (u32_t)(data->capability & CAP_ISCSI_HDR_DGST)>>9));
	STRUCT_PRINT((disp_str,  "\tISCSI_BODY_DGST      = 0x%x \n", (u32_t)(data->capability & CAP_ISCSI_BODY_DGST)>>10));
	STRUCT_PRINT((disp_str,  "\tSERDES_2_5G          = 0x%x \n", (u32_t)(data->capability & CAP_SERDES_2_5G)>>11));
	STRUCT_PRINT((disp_str,  "\tCAP_EVRST_RSVD       = 0x%x \n", (u32_t)(data->capability & CAP_EVRST_RSVD)>>11));
	STRUCT_PRINT((disp_str,  "max_um_rdma_conn = 0x%x\n", (u32_t)(data->max_um_rdma_conn)));
	STRUCT_PRINT((disp_str,  "reserved = 0x%x\n", (u32_t)(data->reserved)));
	STRUCT_PRINT((disp_str,  "max_iscsi_trgt_conn = 0x%x\n", (u32_t)(data->max_iscsi_trgt_conn)));
	STRUCT_PRINT((disp_str,  "max_iscsi_init_conn = 0x%x\n", (u32_t)(data->max_iscsi_init_conn)));
	STRUCT_PRINT((disp_str,  "max_iser_trgt_conn = 0x%x\n", (u32_t)(data->max_iser_trgt_conn)));
	STRUCT_PRINT((disp_str,  "max_iser_init_conn = 0x%x\n", (u32_t)(data->max_iser_init_conn)));
	STRUCT_PRINT((disp_str,  "max_fcoe_trgt_conn = 0x%x\n", (u32_t)(data->max_fcoe_trgt_conn)));
	STRUCT_PRINT((disp_str,  "max_fcoe_init_conn = 0x%x\n", (u32_t)(data->max_fcoe_init_conn)));
	STRUCT_PRINT((disp_str,  "reserved_a[0 .. %d] = ", (2) - 1));
	for (index = 0; index < (2); index++)  // u32 array 
	{
		if ((index & 0x3) ==0){ SPECIAL_DBG_PRINT((disp_str,  "\n")); STRUCT_PRINT((disp_str,  "\t"));}
		SPECIAL_DBG_PRINT((disp_str,  " 0x%08x", ((u32_t *)data->reserved_a)[index]));
	}
	SPECIAL_DBG_PRINT((disp_str,  "\n"));
	STRUCT_PRINT((disp_str,  "sn = 0x%x\n", (u32_t)(data->sn)));
	STRUCT_PRINT((disp_str,  "expiration = 0x%x\n", (u32_t)(data->expiration)));
	if ((data->expiration & 0xFFFFFFFF) == EXPIRATION_NEVER) 
	{
		STRUCT_PRINT((disp_str,  "\t\t(NEVER)\n")); 
	}
	STRUCT_PRINT((disp_str,  "reserved_b = 0x%x\n", (u32_t)(data->reserved_b)));
	indent_level = indent_level == 0 ? 0: indent_level-1;  // decrease indentation when exiting each level of structure decoding

}// DecodeLicenseKeyL




// ??????? move following declaration to header file
extern void DecodeLicenseKey(Tcl_Obj* rc_ptr, const license_key_t * data);
