/*************************************************************************
 * Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved.
 * Copyright(c) 2014-2015 QLogic Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of QLogic Corporation, and
 * may not be copied or distributed in any isomorphic form without
 * the prior written consent of QLogic Corporation.
 *
 *
 *  Name: tcl_fmodules.c
 *
 *  Description: This file handles the allowed modules & part numbers text file
 ****************************************************************************/

#ifdef EDEBUG
#include <stdio.h>
#ifndef LINUX
#if !defined(UEFI) && !defined(__SunOS) && !defined(FREEBSD)
#include <io.h>
#endif
#else
#include <unistd.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "man_types.h"
#include "tcl_fmodules.h"
#include <tcl_util.h>
#include "dbg_os_utils.h"
#include "common_nvm.h"
#include "utils.h"
#ifdef UEFI
#include <sys/unistd.h>
#endif
#else // NOT EDEBUG

#include <stdio.h>
#ifndef USER_LINUX
#ifndef UEFI
#include <io.h>
#endif
#else
#include <unistd.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tcl_fmodules.h"
#include "tcl_util.h"
#include "dg_os_util.h"
#include "debug.h"
#include "common_nvm.h"
#include "utils.h"
#ifdef UEFI
#include <sys/unistd.h>
#endif

#endif 
/* Set the bit(s), bit_mask - is a mask with the desired bits set */
#include <stdio.h>
#define MODULE_INFO_CTRL_FLAG_SET(val, bit_mask)    val |= (bit_mask)
/* Clear the bit(s) */
#define MODULE_INFO_CTRL_FLAG_CLEAR(val, bit_mask)  val = val & (~bit_mask)

#define MFW_VER_OPT_MDL_VER_1 0x00050234
#define MFW_VER_OPT_MDL_VER_2 0x00070a26
#define MAX_TOKEN_STR_LEN 30
static int tcl_al_get_mfw_supported_fmodule_ver(void)
{
    char mfw_version[20];

    u32 mfw_ver_dword;	
    unsigned int mfw_ver[4] = {0,0,0,0};
    common_nvm_read_mfw_version(&mfw_version[0]);

    if (sscanf(mfw_version, "%d.%d.%d.%d", &mfw_ver[0], &mfw_ver[1], &mfw_ver[2], &mfw_ver[3]) != 4) {

        sscanf(mfw_version, "%d.%d.%d", &mfw_ver[0],&mfw_ver[1], &mfw_ver[2]);
    }
    mfw_ver_dword = (mfw_ver[2] & 0xff) | ((mfw_ver[1] & 0xff) << 8)  | ((mfw_ver[0] & 0xff) << 16);
    // printf("mfw_version %s 0x%x: mfw_ver[0] %d ,mfw_ver[1] %d, mfw_ver[2] %d, mfw_ver[3] %d\n", mfw_version, mfw_ver_dword, mfw_ver[0], mfw_ver[1], mfw_ver[2], mfw_ver[3]);
    if (mfw_ver_dword > MFW_VER_OPT_MDL_VER_2) {
        return MODULE_IMAGE_VERSION_2;
    } else if (mfw_ver_dword > MFW_VER_OPT_MDL_VER_1) {
        return MODULE_IMAGE_VERSION_1;
    } else {
        return 0;
    }
}
static int tcl_al_parse_module_line(u32 format_version, char *line, char *vendor_name_ptr, char *vendor_oui_ptr, char *vendor_pn_ptr, char *branding);
/*
 * read_fmac: 
 * read the allowed optic modules information
 * 
 * input: specify the name of the file that contains the modules information
 * return: NULL on error; otherwise, a handle used for other operations.
 */
void * tcl_al_read_fmodules(char *filename, u32_t *byte_read)
{
    module_image_t *module_image_ptr = NULL;
    FILE *fd;
    char line[1024];
	char err_str[80];
    char *cptr = NULL;
    u32_t val32, module_idx=0, i=0, format_version=0, len=0;
	u8_t is_format_ver=0;
	char tmp_str[1024];
	char pad_char=' ';
	char vendor_name[MAX_TOKEN_STR_LEN];
	char vendor_oui_str[MAX_TOKEN_STR_LEN];
	char part_number[MAX_TOKEN_STR_LEN];
    char branding[MAX_TOKEN_STR_LEN];
	u32_t vendor_oui[SFF_VENDOR_OUI_LEN];
	module_info_t 	modules[MAX_VENDOR_MODULES];
	
	

#if !defined (UEFI) && !defined (WIN_DIAG)
    val32 = access(filename, R_OK | W_OK | F_OK);
    if (val32 == -1)
    {
        goto handle_error;
    }
#endif
	
    fd = fopen(filename, "r");
    if (fd == NULL)
    {
        goto handle_error;
    }

	/* Initialize local variables */
	vendor_name[0]='\0';
	vendor_oui[0]='\0';
	part_number[0]='\0';
    branding[0]='\0';
	*byte_read = 0;

    while (!feof(fd))
    {
        if (fgets(line, 1024, fd))
        {
			/* Skip comment lines */
			if (line[0] == '#') continue;
			
			/* Skip lines which do not include '=' or ',' */
            if ((strchr(line, '=') == NULL) && (strchr(line, ',') == NULL)) continue;
			
			/* Update fmodules_ptr->revision according to REVISION field in input file */
            if (strstr(line, REVISION))
            {
				/* Get the value after the '=' char */
                int mfw_module_ver_support = tcl_al_get_mfw_supported_fmodule_ver();
				cptr = strchr(line, '=');
				for (cptr++; *cptr != ' '; cptr++);
				val32 = strtoul(&cptr[1], &cptr, 16);
				format_version = val32;

                
				if ((format_version != MODULE_IMAGE_VERSION_1) &&  (format_version != MODULE_IMAGE_VERSION_2))  {
					PRINT_MSG ("Module file revision is not supported\n", ESC_COLOR_RED);
					goto handle_error;
				}
                if (format_version > mfw_module_ver_support )  {
                    PRINT_MSG ("Module file revision is not supported by MFW\n", ESC_COLOR_RED);
                    goto handle_error;
                }

				is_format_ver=1;
				//printf("info: format_version=%d \n", format_version);
            }
            else
            { 
				if (is_format_ver) {
					
					/* Parse the rest of file 
					   Each line should include: vendor_name , vendor_oui , part_number 
					*/
					if (tcl_al_parse_module_line(format_version, line, vendor_name, vendor_oui_str, part_number, branding) == TCL_OK) {

						if (strlen(vendor_name) > SFF_VENDOR_NAME_LEN) {
							sprintf (err_str, "Module file error: vendor_name string length is longer than %d\n", SFF_VENDOR_NAME_LEN);
							PRINT_MSG (err_str, ESC_COLOR_RED);
							goto handle_error;
						}

						if (strlen(part_number) > SFF_VENDOR_PN_LEN) {
							sprintf (err_str, "Module file error: vendor_pn string length is longer than %d\n", SFF_VENDOR_PN_LEN);
							PRINT_MSG (err_str, ESC_COLOR_RED);
							goto handle_error;
						}
					
						strncpy(modules[module_idx].vendor_name, vendor_name, SFF_VENDOR_NAME_LEN);

						if (vendor_oui_str[0] != '*') {
							sscanf(vendor_oui_str, "%2x-%2x-%2x", &vendor_oui[0], &vendor_oui[1], &vendor_oui[2]);

							/* Verify that vendor OUI byte value < 0xFF */
							for (i=0; i<3; i++) {
								if (vendor_oui[i] > 0xFF) {
									PRINT_MSG ("Module file error: vendor_oui incorrect format. Should be XX-XX-XX (in hex)\n", ESC_COLOR_RED);
									goto handle_error;
								}
							}
							modules[module_idx].vendor_oui[0] = (u8_t) vendor_oui[0];
							modules[module_idx].vendor_oui[1] = (u8_t) vendor_oui[1];
							modules[module_idx].vendor_oui[2] = (u8_t) vendor_oui[2];
						} else {
							modules[module_idx].vendor_oui[0] = 0;
							modules[module_idx].vendor_oui[1] = 0;
							modules[module_idx].vendor_oui[2] = 0;
						}

						strncpy(modules[module_idx].vendor_pn, part_number, SFF_VENDOR_PN_LEN);
						
						/*
						printf("info: module_idx=%d vendor_name=[%s] vendor_oui=%2x%2x%2x, vendor_pn=[%s]\n", 
																	module_idx, modules[module_idx].vendor_name, 
																	modules[module_idx].vendor_oui[0], modules[module_idx].vendor_oui[1], 
																	modules[module_idx].vendor_oui[2], modules[module_idx].vendor_pn);
						*/

						/* Set the ctrl_flags field */
						/* Set flag to 0 - if name begins with '*' */
						/* Set flag to 1 - otherwise */
						modules[module_idx].ctrl_flags = 0;
						if (vendor_name[0] != '*') {
							MODULE_INFO_CTRL_FLAG_SET(modules[module_idx].ctrl_flags, MODULE_INFO_FLAG_CHECK_VENDOR_NAME);
						} else {
							MODULE_INFO_CTRL_FLAG_CLEAR(modules[module_idx].ctrl_flags, MODULE_INFO_FLAG_CHECK_VENDOR_NAME);
						}
						if (vendor_oui_str[0] != '*') {
							MODULE_INFO_CTRL_FLAG_SET(modules[module_idx].ctrl_flags, MODULE_INFO_FLAG_CHECK_VENDOR_OUI);
						} else {
							MODULE_INFO_CTRL_FLAG_CLEAR(modules[module_idx].ctrl_flags, MODULE_INFO_FLAG_CHECK_VENDOR_OUI);
						}
						if (part_number[0] != '*') {
							MODULE_INFO_CTRL_FLAG_SET(modules[module_idx].ctrl_flags, MODULE_INFO_FLAG_CHECK_VENDOR_PN);
						} else {
							MODULE_INFO_CTRL_FLAG_CLEAR(modules[module_idx].ctrl_flags, MODULE_INFO_FLAG_CHECK_VENDOR_PN);
						}
			if (format_version > 1) {
				
			    if (!strcmp(branding, "Yes")) {
				MODULE_INFO_CTRL_FLAG_SET(modules[module_idx].ctrl_flags, MODULE_INFO_FLAG_CHECK_BRANDING);
                            } else if (!strcmp(branding, "No")) {
                                MODULE_INFO_CTRL_FLAG_CLEAR(modules[module_idx].ctrl_flags, MODULE_INFO_FLAG_CHECK_BRANDING);
                            } else {
                                sprintf (err_str, "Module file error: Incorrect Branding string %s\n", branding);
							    PRINT_MSG (err_str, ESC_COLOR_RED);
							    goto handle_error;
                            }
                        } else {
                            MODULE_INFO_CTRL_FLAG_CLEAR(modules[module_idx].ctrl_flags, MODULE_INFO_FLAG_CHECK_BRANDING);
                        }

						
						/*printf("info: module_idx %d ctrl_flags=0x%x\n", module_idx, modules[module_idx].ctrl_flags);*/
						
                     
						module_idx ++;
						if (module_idx > MAX_VENDOR_MODULES) {
							PRINT_MSG ("Module file exceeded number of supported modules\n", ESC_COLOR_RED);
							goto handle_error;
						}
					
					} else {
						sprintf (tmp_str, "Module file incorrect format: in line [%s]\n", line);
						PRINT_MSG (tmp_str, ESC_COLOR_RED);
						PRINT_MSG ("Module file incorrect format: should include vendor_name, vendor_oui(xx-xx-xx) , part_number\n", ESC_COLOR_RED);
						goto handle_error;
					}
				}
			}
		}
	}

	/* Allocate memory for the parsed file structure module_image_t */    
	module_image_ptr = (module_image_t *) dg_os_alloc_virt(sizeof(module_image_t)+ (sizeof(module_info_t) * module_idx));
	
    if ( module_image_ptr == NULL ) {
        PRINT_MSG("Memory allocation failed!\n", ESC_COLOR_RED);
        goto handle_error;
    }

	/* Set the image format revision */
    module_image_ptr->format_version = format_version;
	

    /* Save the number of modules parsed from modules file */
	module_image_ptr->no_modules = module_idx++;

	for (module_idx=0; module_idx < module_image_ptr->no_modules; module_idx++ ) {
		/* Set the modules info array */

		/* Set the ctrl_flags field */
		module_image_ptr->modules[module_idx].ctrl_flags = modules[module_idx].ctrl_flags;

		/* Set the vendor_name field */
		/* if vendor_name is exactly in the length of SFF_VENDOR_NAME_LEN there will be no NULL terminator at the end
		   Therefore, limit str len to SFF_VENDOR_NAME_LEN */
		strncpy(module_image_ptr->modules[module_idx].vendor_name, modules[module_idx].vendor_name, SFF_VENDOR_NAME_LEN);
		len=min(strlen(modules[module_idx].vendor_name), SFF_VENDOR_NAME_LEN);

		/* Pad vendor name on the right with ASCII spaces */
		for (i=0; i<(SFF_VENDOR_NAME_LEN-len); i++) {
			module_image_ptr->modules[module_idx].vendor_name[len+i]=pad_char;
		}

		/* Set the vendor_oui field bytes */
		module_image_ptr->modules[module_idx].vendor_oui[0] = modules[module_idx].vendor_oui[0];
		module_image_ptr->modules[module_idx].vendor_oui[1] = modules[module_idx].vendor_oui[1];
		module_image_ptr->modules[module_idx].vendor_oui[2] = modules[module_idx].vendor_oui[2];

		/* Set the vendor_pn field */
		/* if vendor_name is exactly in the length of SFF_VENDOR_PN_LEN there will be no NULL terminator at the end
		   Therefore, limit str len to SFF_VENDOR_PN_LEN */
		strncpy(module_image_ptr->modules[module_idx].vendor_pn, modules[module_idx].vendor_pn, 16);
		len=min(strlen(modules[module_idx].vendor_pn), SFF_VENDOR_PN_LEN);		

		/* Pad vendor OUI on the right with ASCII spaces */
		for (i=0; i<(SFF_VENDOR_NAME_LEN-len); i++) {
			module_image_ptr->modules[module_idx].vendor_pn[len+i]=pad_char;
		}
	}
	/* Calculate the number of bytes in image */
	*byte_read = sizeof(module_image_t)+ (sizeof(module_info_t) * (module_idx-1));
	
	fclose(fd);
    return (void *)module_image_ptr;

handle_error:
#ifdef EDEBUG
    if (module_image_ptr) free((char *)module_image_ptr);
#else
    if (module_image_ptr) dg_os_free_virt((int_ptr_t)module_image_ptr);
#endif
    return NULL;

}


/*
 * tcl_al_write_fmodules: 
 * read the nvm image MODULES_PN, parse it and output to file
 * 
 * input: specify the name of the output file 
 * return: 
 */

static void tcl_al_write_fmodule_hdr_ver1(FILE *fd)
{
		fprintf(fd, "##################################################\n");
		fprintf(fd, "#           APPROVED SFP+ MODULES LIST           #\n");
		fprintf(fd, "##################################################\n");
		
		fprintf(fd, "# Revision for future enhancements. Currently, only revision 1 is supported\n");
		fprintf(fd, "revision = 1\n");
		
		fprintf(fd, "# Each Comment line starts with \"#\" \n");
		fprintf(fd, "# Enter the following 3 required fields separated by coma \",\" \n");
		
		fprintf(fd, "# 1) Vendor Name [module eeprom Address A0h; 0x14h-0x23h]\n");
		fprintf(fd, "# 2) OUI: 3 hex single bytes (dual nibble) numbers in xx-xx-xx format; [module eeprom Address A0h; 0x25h-0x27h]\n");
		fprintf(fd, "# 3) Vendor Part Number [module eeprom Address A0h; 0x28h-0x37h]\n");
		fprintf(fd, "#\n");
		fprintf(fd, "# It is possible to replace a parameter with \"*\" to indicate any value is\n");
		fprintf(fd, "# acceptable\n");
		fprintf(fd, "#\n");
		fprintf(fd, "#examples:\n");
		fprintf(fd, "# Full description:\n");
		fprintf(fd, "#JDSU, 00-01-9c, PLRXPLSCS4321N\n");
		fprintf(fd, "# The same without explicitly stating the OUI:\n");
		fprintf(fd, "#JDSU, *, PLRXPLSCS4321N\n");
		fprintf(fd, "# All JDSU modules are acceptable:\n");
		fprintf(fd, "#JDSU, *, *\n");
}

static void tcl_al_write_fmodule_hdr_ver2(FILE *fd)
{
    fprintf(fd, "##################################################\n");
    fprintf(fd, "#           APPROVED SFP+ MODULES LIST           #\n");
    fprintf(fd, "##################################################\n");

    fprintf(fd, "# Revision for future enhancements. Currently, only revisions 1, 2 are supported\n");
    fprintf(fd, "revision = 2\n");

    fprintf(fd, "# Each Comment line starts with \"#\" \n");
    fprintf(fd, "# Enter the following 4 required fields separated by coma \",\" \n");

    fprintf(fd, "# 1) Vendor Name [module eeprom Address A0h; 0x14h-0x23h]\n");
    fprintf(fd, "# 2) OUI: 3 hex single bytes (dual nibble) numbers in xx-xx-xx format; [module eeprom Address A0h; 0x25h-0x27h]\n");
    fprintf(fd, "# 3) Vendor Part Number [module eeprom Address A0h; 0x28h-0x37h]\n");
    fprintf(fd, "# 4) Branding check [<Yes/No>] - Valid for version 2 and above only.\n");
    fprintf(fd, "#\n");
    fprintf(fd, "# It is possible to replace a parameter with \"*\" to indicate any value is\n");
    fprintf(fd, "# acceptable\n");
    fprintf(fd, "#\n");
    fprintf(fd, "#examples:\n");
    fprintf(fd, "# Full description:\n");
    fprintf(fd, "#JDSU, 00-01-9c, PLRXPLSCS4321N, No\n");
    fprintf(fd, "# The same without explicitly stating the OUI:\n");
    fprintf(fd, "#JDSU, *, PLRXPLSCS4321N, No\n");
    fprintf(fd, "# All JDSU modules are acceptable:\n");
    fprintf(fd, "#JDSU, *, *, No\n");
    fprintf(fd, "# All branded modules are acceptable:\n");
    fprintf(fd, "#*, *, *, Yes\n");
}

int tcl_al_write_fmodules(char *filename, u8_t *buf)
{
	FILE *fd=NULL;
	module_image_t *module_image_ptr = NULL;
	u32_t i=0, module_idx=0;
	u32_t file_size=0;
	char pad_char=' ';
	char vendor_name[SFF_VENDOR_NAME_LEN+1];
	char vendor_pn[SFF_VENDOR_PN_LEN+1];
	char *brand_str;
    char yes_str[]=" , Yes";
    char no_str[]=" , No";
    char null_str[]="";
    
    fd = fopen(filename, "a");
    if (fd)
	{
		/* If file already exists, exit do not allow to overwrite the file */
		fseek (fd, 0L, SEEK_END);
		file_size = ftell (fd);
		fseek (fd, 0L, SEEK_SET);      
		if (file_size > 0) {
			PRINT_MSG ("Parsing modules image failed ! output file allready exists\n", ESC_COLOR_RED);
			return TCL_ERROR;
		}
		module_image_ptr = (module_image_t *) buf;
       if (htonl(module_image_ptr->format_version) == MODULE_IMAGE_VERSION_1) {
           tcl_al_write_fmodule_hdr_ver1(fd);
       } else {
           tcl_al_write_fmodule_hdr_ver2(fd);
       }

		printf("info: modules image revision = %d\n", htonl(module_image_ptr->format_version));
		printf("info: module image includes %d modules \n", htonl(module_image_ptr->no_modules));
        
		
		for (module_idx=0; module_idx < htonl(module_image_ptr->no_modules); module_idx++) {
			// Ignore padding at the end of vendor name 
			i=SFF_VENDOR_NAME_LEN-1;
			while ((i > 0) && (module_image_ptr->modules[module_idx].vendor_name[i] == pad_char)) { i--; }
			vendor_name[i+1]= '\0';			

			// Copy vendor_name
			while (i > 0) {
				vendor_name[i] = module_image_ptr->modules[module_idx].vendor_name[i];
				i--;
			}
			vendor_name[i] = module_image_ptr->modules[module_idx].vendor_name[i];
			
            
			// Ignore padding at the end of vendor name 
			i=SFF_VENDOR_PN_LEN-1;
			while ((i > 0) && (module_image_ptr->modules[module_idx].vendor_pn[i]==pad_char)) { i--; }
			vendor_pn[i+1]= '\0';

			// Copy vendor_name
			while (i > 0) {
				vendor_pn[i] = module_image_ptr->modules[module_idx].vendor_pn[i];
				i--;
			}
			vendor_pn[i] = module_image_ptr->modules[module_idx].vendor_pn[i];

            if (htonl(module_image_ptr->format_version) == MODULE_IMAGE_VERSION_1) {
                brand_str = null_str;
            } else {
                if (htonl(module_image_ptr->modules[module_idx].ctrl_flags) & MODULE_INFO_FLAG_CHECK_BRANDING) {
                    brand_str = yes_str;
                } else {
                    brand_str = no_str;
                }
            }

			if ((module_image_ptr->modules[module_idx].vendor_oui[0] == 0) &&
				(module_image_ptr->modules[module_idx].vendor_oui[1] == 0) &&
				(module_image_ptr->modules[module_idx].vendor_oui[2] == 0)) {
				fprintf(fd, "%s , * , %s%s\n", 	vendor_name, vendor_pn, brand_str);
			} else {
				fprintf(fd, "%s , %02x-%02x-%02x , %s%s\n", 	vendor_name, 
														module_image_ptr->modules[module_idx].vendor_oui[0], 
														module_image_ptr->modules[module_idx].vendor_oui[1], 
														module_image_ptr->modules[module_idx].vendor_oui[2],
														vendor_pn, brand_str);
			}
		
		}
		
		printf("Module image was parsed succesfully into %s\n", filename);
		
		  
        fclose(fd);
		return TCL_OK;
	} else {
		return TCL_ERROR;
	}

}

/* tcl_al_parse_module_line 
 * 
 * Parse the line into three fields: vendor_name, vendor_oui, vendor_pn
 * 
 * Ignore white spaces at the begining and ending of field if exit
 *
*/
static int tcl_al_parse_module_line(u32 format_version, char *line, char *vendor_name_ptr, char *vendor_oui_ptr, char *vendor_pn_ptr, char *branding)
{
	   char *token_ptr;
	   char module_line[1024];

	   // Find first occurance of the token ','
	   strcpy(module_line, line);
	   token_ptr = strtok( module_line, ",");
	   if (token_ptr == NULL || (strlen(token_ptr) >= MAX_TOKEN_STR_LEN)) {
		   return TCL_ERROR;
	   }
	  
	   remove_white_spaces(token_ptr, strlen(token_ptr), vendor_name_ptr);
	
	   // Find next occurance of the token ','
	   token_ptr = strtok( NULL, "," );
	   if (token_ptr == NULL) {
		   return TCL_ERROR;
	   }
	  
	   remove_white_spaces(token_ptr, strlen(token_ptr), vendor_oui_ptr);

       if (format_version == MODULE_IMAGE_VERSION_1) {     
           token_ptr = strtok(NULL, " \r\t\n"); 
           remove_white_spaces(token_ptr, strlen(token_ptr), vendor_pn_ptr);
       } else if (format_version == MODULE_IMAGE_VERSION_2) {
           token_ptr = strtok( NULL, "," );
           remove_white_spaces(token_ptr, strlen(token_ptr), vendor_pn_ptr);
           token_ptr = strtok(NULL, " \r\t\n"); 
           remove_white_spaces(token_ptr, strlen(token_ptr), branding);
       }
	  
	   return TCL_OK;
}

