#ifndef __BROADCOM_XDIAG_COMMON_TYPES_H__
#define __BROADCOM_XDIAG_COMMON_TYPES_H__

#ifdef USER_LINUX
#include <sys/types.h>
#elif defined(__LINUX)
#include <linux/types.h>
#include <asm/byteorder.h>
#endif
#include "bcmtype.h"
#include "listq.h"
#include "lm_defs.h"
#include "os_types.h"
#include "debug.h"

/* Number of line per one DOS display */
#define LINE_CNT            18

/* QLogic Vendor ID */
#define BROADCOM_VID        0x14e4 
#define DEVICE_ID_57710     0x164e
#define DEVICE_ID_57711     0x164f
#define DEVICE_ID_57711E    0x1650


#define DIAG_STRERROR(err_code) diag_stat_str[-((s32_t)err_code)]

extern char * diag_stat_str[];

/* TBD: Check if we can change negative error codes to positive */
typedef u32_t diag_status_t;
#define DIAG_STATUS_SUCCESS             0
#define DIAG_STATUS_FAILURE            -1     /* General failure */
#define DIAG_STATUS_RESOURCE           -2     /* Memory failure  */
#define DIAG_STATUS_ACCESS             -3     /* Access failure  */
#define DIAG_STATUS_INVALID_PARAMETER  -4     /* Invalid parameters  */
#define DIAG_STATUS_PENDING            -5	  /* Pending for l2pkt loopback to complete */
#define DIAG_STATUS_NVM_INVAL_DIR      -6     /* Invalid directory index */
#define DIAG_STATUS_NVM_BAD_VALUE      -7     /* Bogus value */
#define DIAG_STATUS_TIMEOUT            -8     /* Timeout */
#define DIAG_STATUS_CON_CLOSED         -9     /* Connection closed */
#define DIAG_STATUS_ABORTED            -10    /* Packet aborted */
#define DIAG_STATUS_UPLOAD_IN_PROGRESS -11    /* Upload in progress */

typedef struct _dg_l4_app_buf_t {
    u32_t query_buf_sz;    /* Size of the query buffer for this AB */
    s_list_t l4buf_list;   /* List of dg_l4buf_t */
} dg_l4_app_buf_t;

typedef struct _dg_l4buf_t {
    s_list_entry_t   link;
    u32_t            flags;
    u32_t            nbl_id;
    u32_t            l4buf_id;
    diag_virt_addr_t frags_virt_base;
    diag_phys_addr_t frags_phys_base;
    lm_frag_list_t   frag_list;
} dg_l4buf_t;

typedef struct _dg_tcp_state_info_t {
    u8_t fin_received   : 1;
    u8_t rst_received   : 1;
    u8_t urg            : 1;
    u8_t rt_tout        : 1;
    u8_t ka_tout        : 1;
    u8_t upld_requested : 1;
} dg_tcp_state_info_t;

#endif /* __BROADCOM_XDIAG_COMMON_TYPES_H__ */
