/****************************************************************************
 * Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved
 * Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
 * Proprietary and Confidential Information.
 *
 * This source file is the property of QLogic Corporation, and
 * may not be copied or distributed in any isomorphic form without
 * the prior written consent of QLogic Corporation.
 *
 * Name:        errcode.h
 *
 * Description: Defines the error codes 
 *
 ****************************************************************************/

#include "tcl.h"
#include "bcmtype.h"

#ifndef ERRCODE_H
#define ERRCODE_H

extern u8_t *errcode[];

typedef enum _errcode_t {
	ERR_NONE=0,		    /* 00: No error */
	ERR_REG_TEST,		    /* 01: Register test fails */
	ERR_PCICFG_TEST,	    /* 02: PCI config test fails */
	ERR_INTR_TEST,		    /* 03: Interrupt test fails */
	ERR_PCIE_LINK_TEST,	    /* 04: PCIE link test fails */
	ERR_MSI_TEST,		    /* 05: MSI test fails */
	ERR_BIST_TEST,		    /* 06: Memory BIST test fails */
	ERR_NETLINK_TEST,	    /* 07: Network link test fails */   
	ERR_CAM_TEST=28,	    /* 28: CAM test fails */
	ERR_TIMER_TEST,		    /* 29: Timer test fails */   
	ERR_NVRAM_TEST=32,	    /* 32: NVRAM test fails */
	ERR_RESET_TEST,		    /* 33: Reset test fails */
	ERR_DMAE_TEST,		    /* 34: DMA engine test fails */   
	ERR_MACLB_TEST=38,	    /* 38: MAC loopback test fails */
	ERR_PHYLB_TEST,		    /* 39: PHY loopback test fails */
	ERR_EXTLB_TEST,		    /* 40: External loopback test fails */
	ERR_LSO_TEST,		    /* 41: LSO traffic test fails */
	ERR_STAT_TEST,		    /* 42: Statistic test fails */
	ERR_RPC_TEST,		    /* 43: RPC test fails */
	ERR_BAD_PASSWD,		    /* 44: Invalid password */
	ERR_BAD_ARGS,		    /* 45: Invalid command switch(es) */
	ERR_NO_DEV_FOUND,	    /* 46: Operation(s) skipped, no device found */
	ERR_NOT_AUTH,		    /* 47: Not authorized to perform the operation(s) */
	ERR_ONE_DEV_ONLY,	    /* 48: Specify one device, no more, no less */
	ERR_MAC_REDUND,		    /* 49: Redundant MAC address input methods */
	ERR_FMAC_BAD_FILE,	    /* 50: Bad MAC address range file (not exist, bad format) */
	ERR_FMAC_RUN_OUT,	    /* 51: Used up MAC addr in the range file */
	ERR_FMAC_UPDATE,	    /* 52: Fail to update the MAC address range file */
	ERR_PROMPT_MAC,		    /* 53: Fail to input MAC address */
	ERR_MAC_TOO_DIFF ,	    /* 54: Primary and iSCSI MAC are not consecutive */
	ERR_SAME_MAC_ADDR,	    /* 55: Both primary and iSCSI MAC addr are the same */
	ERR_PRG_NVM_IMAGE,	    /* 56: Fail to program NVRAM image */
	ERR_PRG_PRIM_MAC_ADDR,	    /* 57: Fail to program primary MAC address */
	ERR_PRG_ISCSI_MAC_ADDR,	    /* 58: Fail to program iSCSI MAC address */
	ERR_SEL_DEV,		    /* 59: Fail to select the device for operation(s) */
	ERR_PRG_BC,		    /* 60: Fail to program boot code */
	ERR_PRG_MFW,		    /* 61: Fail to program IPMI/UMP firmware */
	ERR_PRG_MBA,		    /* 62: Fail to program MBA image */
	ERR_CFG_WOL,		    /* 63: Fail to configure WOL */
	ERR_CFG_MFW,		    /* 64: Fail to configure IPMI/UMP firmware */
	ERR_CFG_MBA,		    /* 65: Fail to configure MBA image */
	ERR_PN_NOT_FOUND,	    /* 66: Fail to extract part number from device */
	ERR_PN_BAD_INPUT,	    /* 67: Fail to capture part number from input */
	ERR_PN_MISMATCH,	    /* 68: Part number does not match */
	ERR_CFG_MISMATCH,	    /* 69: Configuration mismatch */
	ERR_RESTORE_PCI,	    /* 70: Fail to restore pci info to the next board */
	ERR_USER_ABORT,		    /* 71: User abort */
	ERR_PRG_MKEY,		    /* 72: Fail to program manufacturing license key */
	ERR_PRG_UKEY,		    /* 73: Fail to program upgrade license key */
	ERR_CHKKEY_SS,		    /* 74: Fail to validate the SS portion of the key */
	ERR_CHKKEY_MK,		    /* 75: Fail to validate the manufacturing license key */
	ERR_CHKKEY_UK,		    /* 76: Fail to validate the upgrade license key */
	ERR_NVRAM_CRC,		    /* 77: CRC check fails on one of the NVRAM blocks */
	ERR_CFG_MBA_SPD,	    /* 78: Fail to configure MBA link speed */
	ERR_CFG_MBA_PROT,	    /* 79: Fail to configure MBA protocol parameter */
	ERR_CFG_MBA_VLAN,	    /* 80: Fail to configure MBA VLAN */
	ERR_CFG_MBA_VLAN_VAL,	    /* 81: Fail to configure MBA VLAN value */   
	ERR_PRG_MBI,    	    /* 82: Fail to program MBI image */
	ERR_MSIX_TEST=90,	    /* 90: MSIX test fails */ 
	ERR_UMP_EXTLB_TEST=99,		/* 99: UMP external loopbak test fails */
	ERR_REG_BRB1_TEST=101,	    /* 101: BRB1 Register test fails (for Everest only) */
	ERR_REG_CCM_TEST,	    /* 102: CCM  Register test fails (for Everest only) */
	ERR_REG_CDU_TEST,	    /* 103: CDU  Register test fails (for Everest only) */
	ERR_REG_CFC_TEST,	    /* 104: CFC  Register test fails (for Everest only) */
	ERR_REG_CSDM_TEST,	    /* 105: CSDM Register test fails (for Everest only) */
	ERR_REG_CSEM_TEST,	    /* 106: CSEM Register test fails (for Everest only) */
	ERR_REG_DBG_TEST,	    /* 107: DBG  Register test fails (for Everest only) */
	ERR_REG_DMAE_TEST,	    /* 108: DMAE Register test fails (for Everest only) */
	ERR_REG_DORQ_TEST,	    /* 109: DORQ Register test fails (for Everest only) */
	ERR_REG_HC_TEST,	    /* 110: HC   Register test fails (for Everest only) */
	ERR_REG_MISC_TEST,	    /* 111: MISC Register test fails (for Everest only) */
	ERR_REG_NIG_TEST,	    /* 112: NIG  Register test fails (for Everest only) */
	ERR_REG_PBF_TEST,	    /* 113: PBF  Register test fails (for Everest only) */
	ERR_REG_PRS_TEST,	    /* 114: PRS  Register test fails (for Everest only) */
	ERR_REG_PXP_TEST,	    /* 115: PXP  Register test fails (for Everest only) */
	ERR_REG_QM_TEST,	    /* 116: QM   Register test fails (for Everest only) */
	ERR_REG_SRC_TEST,	    /* 117: SRC  Register test fails (for Everest only) */
	ERR_REG_TCM_TEST,	    /* 118: TCM  Register test fails (for Everest only) */
	ERR_REG_TM_TEST,	    /* 119: TM   Register test fails (for Everest only) */
	ERR_REG_TSDM_TEST,	    /* 120: TSDM Register test fails (for Everest only) */
	ERR_REG_TSEM_TEST,	    /* 121: TSEM Register test fails (for Everest only) */
	ERR_REG_UCM_TEST,	    /* 122: UCM  Register test fails (for Everest only) */
	ERR_REG_UPB_TEST,	    /* 123: UPB  Register test fails (for Everest only) */
	ERR_REG_USDM_TEST,	    /* 124: USDM Register test fails (for Everest only) */
	ERR_REG_USEM_TEST,	    /* 125: USEM Register test fails (for Everest only) */
	ERR_REG_XCM_TEST,	    /* 126: XCM  Register test fails (for Everest only) */
	ERR_REG_XPB_TEST,	    /* 127: XPB  Register test fails (for Everest only) */
	ERR_REG_XSDM_TEST,	    /* 128: XSDM Register test fails (for Everest only) */
	ERR_REG_XSEM_TEST,	    /* 129: XSEM Register test fails (for Everest only) */
	ERR_REG_EMAC_TEST,	    /* 130: EMAC Register test fails (for Everest only) */
	ERR_REG_MCP_TEST,	    /* 131: MCP  Register test fails (for Everest only) */      
	ERR_MEM_BRB1_TEST,	    /* 132: BRB1 Memory test fails (for Everest only) */
	ERR_MEM_CCM_TEST,	    /* 133: CCM  Memory test fails (for Everest only) */
	ERR_MEM_CDU_TEST,	    /* 134: CDU  Memory test fails (for Everest only) */
	ERR_MEM_CFC_TEST,	    /* 135: CFC  Memory test fails (for Everest only) */
	ERR_MEM_CSDM_TEST,	    /* 136: CSDM Memory test fails (for Everest only) */
	ERR_MEM_CSEM_TEST,	    /* 137: CSEM Memory test fails (for Everest only) */
	ERR_MEM_DBG_TEST,	    /* 138: DBG  Memory test fails (for Everest only) */
	ERR_MEM_DMAE_TEST,	    /* 139: DMAE Memory test fails (for Everest only) */
	ERR_MEM_DORQ_TEST,	    /* 140: DORQ Memory test fails (for Everest only) */
	ERR_MEM_HC_TEST,	    /* 141: HC   Memory test fails (for Everest only) */
	ERR_MEM_NIG_TEST,	    /* 142: NIG  Memory test fails (for Everest only) */
	ERR_MEM_PBF_TEST,	    /* 143: PBF  Memory test fails (for Everest only) */
	ERR_MEM_PXP_TEST,	    /* 144: PXP  Memory test fails (for Everest only) */
	ERR_MEM_QM_TEST,	    /* 145: QM   Memory test fails (for Everest only) */
	ERR_MEM_SRC_TEST,	    /* 146: SRC  Memory test fails (for Everest only) */
	ERR_MEM_TCM_TEST,	    /* 147: TCM  Memory test fails (for Everest only) */
	ERR_MEM_TM_TEST,	    /* 148: TM   Memory test fails (for Everest only) */
	ERR_MEM_TSDM_TEST,	    /* 149: TSDM Memory test fails (for Everest only) */
	ERR_MEM_TSEM_TEST,	    /* 150: TSEM Memory test fails (for Everest only) */
	ERR_MEM_UCM_TEST,	    /* 151: UCM  Memory test fails (for Everest only) */
	ERR_MEM_UPB_TEST,	    /* 152: UPB  Memory test fails (for Everest only) */
	ERR_MEM_USDM_TEST,	    /* 153: USDM Memory test fails (for Everest only) */
	ERR_MEM_USEM_TEST,	    /* 154: USEM Memory test fails (for Everest only) */
	ERR_MEM_XCM_TEST,	    /* 155: XCM  Memory test fails (for Everest only) */
	ERR_MEM_XPB_TEST,	    /* 156: XPB  Memory test fails (for Everest only) */ 
	ERR_MEM_XSDM_TEST,	    /* 157: XSDM Memory test fails (for Everest only) */
	ERR_MEM_XSEM_TEST,	    /* 158: XSEM Memory test fails (for Everest only) */       
	ERR_CAM_SEARCH_TEST,	    /* 159: CAM test fails (for Everest only) */   
	ERR_GRC_TEST,		    /* 160: GRC test fails (for Everest only) */
	ERR_DBG_PXP_TEST,	    /* 161: Debug bus w out to PXPX test fails (for Everest only) */
	ERR_SERDES_REG_TEST,	    /* 162: Serdes Register test fails (for Everest only) */
	ERR_XGXS_REG_TEST,	    /* 163: XGXS Register test fails (for Everest only) */
	ERR_SERDES_INTLB_TEST,	    /* 164: Serdes Internal loopback test fails (for Everest only) */
	ERR_XGXS_INTLB_TEST,	    /* 165: XGXS Internal loopback test fails (for Everest only) */            
	ERR_TOE_TEST,		    /* 166: TOE test fails (for Everest only) */
	ERR_EXTPHY_TEST,	    /* 167: External PHY loopback test fails (for Everest only) */
	ERR_SERDES_EXTLB_TEST,	    /* 168: Serdes External loopback test fails (for Everest only) */
	ERR_XGXS_EXTLB_TEST,	    /* 169: XGXS External loopback test fails (for Everest only) */
	ERR_EXTLB_CASE1_TEST,	    /* 170: External loopback case1 test fails (for Everest only) */
	ERR_EXTLB_CASE2_TEST,	    /* 171: External loopback case2 test fails (for Everest only) */
	ERR_EXTLB_CASE3_TEST,	    /* 172: External loopback case3 test fails (for Everest only) */
	ERR_EXTLB_CASE4_TEST,	    /* 173: External loopback case4 test fails (for Everest only) */
	ERR_EXTLB_CASE5_TEST,	    /* 174: External loopback case5 test fails (for Everest only) */
	ERR_EXTLB_CASE6_TEST,	    /* 175: External loopback case6 test fails (for Everest only) */
	ERR_EXTLB_CASE7_TEST,	    /* 176: External loopback case7 test fails (for Everest only) */
	ERR_EXTLB_CASE8_TEST,	    /* 177: External loopback case8 test fails (for Everest only) */
	ERR_EXTLB_CASE9_TEST,	    /* 178: External loopback case9 test fails (for Everest only) */
	ERR_EXTLB_CASE10_TEST,	    /* 179: External loopback case10 test fails (for Everest only) */
	ERR_EXTLB_CASE11_TEST,	    /* 180: External loopback case11 test fails (for Everest only) */
	ERR_EXTLB_CASE12_TEST,	    /* 181: External loopback case12 test fails (for Everest only) */
	ERR_EXTLB_CASE13_TEST,	    /* 182: External loopback case13 test fails (for Everest only) */
	ERR_EXTLB_CASE14_TEST,	    /* 183: External loopback case14 test fails (for Everest only) */
	ERR_EXTLB_CASE15_TEST,	    /* 184: External loopback case15 test fails (for Everest only) */
	ERR_EXTLB_CASE16_TEST,	    /* 185: External loopback case16 test fails (for Everest only) */
	ERR_EXTLB_CASE17_TEST,	    /* 186: External loopback case17 test fails (for Everest only) */
	ERR_EXTLB_CASE18_TEST,	    /* 187: External loopback case18 test fails (for Everest only) */       
	ERR_EXTLSO_BASIC_TEST,	    /* 188: External LSO basic test fails (for Everest only) */     
	ERR_EXTLSO_EXTND_TEST,	    /* 189: External LSO extended test fails (for Everest only) */  
	ERR_EXTLSO_SNAP_TEST,	    /* 190: External LSO snap test fails (for Everest only) */              
	ERR_EXTLSO_SNAP_EXTND,	    /* 191: External LSO snap extended test fails (for Everest only) */   
	ERR_PRG_RDMA_MAC_ADDR,	    /* 192: Fail to program iSCSI MAC address (for Everest only) */   
	ERR_CFG_SERIAL_NO,	    /* 193: Fail to configure serial number (for Everest only) */
	ERR_PRG_L2B,		    /* 194: Fail to program L2B image (for Everest only) */ 
	ERR_PRG_IB,		    /* 195: Fail to program iSCSI boot image (for Everest only) */     
	ERR_REG_IGU_TEST,	     /* 196: IGU Register test fails (for Everest2 only) */
	ERR_REG_PGLUE_B_TEST,	     /* 197: PGLUE_B Register test fails (for Everest2 only) */
	ERR_REG_ATC_TEST,	     /* 198: ATC Register test fails (for Everest2 only) */
	ERR_MEM_IGU_TEST,	     /* 199: IGU Memory test fails (for Everest2 only) */
	ERR_MEM_ATC_TEST,	     /* 201: ATC Memory test fails (for Everest2 only) */
	ERR_CFG_MBA_BOOT_RETRY,	     /* 202: Fail to configure MBA boot retry count */
	ERR_MEM_VFC_TEST,	     /* 203: VFC Memory test fails (for Everest2 only) */
	ERR_REG_XMAC0_TEST,	     /* 204: XMAC Register test fails (for Everest2 only) */
	ERR_REG_XMAC1_TEST,	     /* 205: XMAC Register test fails (for Everest2 only) */
	ERR_REG_UMAC0_TEST,	     /* 206: UMAC0 Register test fails (for Everest2 only) */
	ERR_REG_UMAC1_TEST,	     /* 207: UMAC1 Register test fails (for Everest2 only) */
	ERR_REG_MSTAT0_TEST,	     /* 208: MSTAT0 Register test fails (for Everest2 only) */
	ERR_REG_MSTAT1_TEST,	     /* 209: MSTAT1 Register test fails (for Everest2 only) */
	ERR_EXTLB2_TEST,	     /* 207: Second External loopback test fails (for dual-media only) */
	ERR_VFC_CAM_TEST,	     /* 208: VFC CMA test fails (for Everest2 only) */
	ERR_INVALID_STRIDE,	     /* 209: Primary LAN MAC address does not align properly */
	ERR_PRG_FCOE_MAC_ADDR,	     /* 210: Fail to program FCOE MAC address */
	ERR_PROMPT_SN,		     /* 211: Fail to input serial number */
	ERR_PCIECHK,		     /* 212: Failed the PCIE speed/width expected values */
	ERR_BLAST_BAD_FILE,	     /* 213: Failed to parse blast text file */
	ERR_BLAST,	     	     /* 214: Blast test failed */
	ERR_LED_CHK,	     	 /* 215: Led check failed */
	MAX_ERR
} errcode_t;


#endif /* ERRCODE_H */

