/******************************************************************************/
/* Module Description:                                                        */
/*    Some useful string processing routines.                                 */
/*                                                                            */
/******************************************************************************/

#ifndef PARSER_H
#define PARSER_H

#ifdef USER_LINUX
#include <sys/types.h>
#endif
#include "bcmtype.h"



#ifdef __cplusplus
extern "C" {
#endif

// String functions:
//
// Input:
//    char *pBuf -- Input string pointer.
//    u32_t *pCnt -- Number of chars processed.  (Optional).
//
// Return:
//    Pointer to the end of the scan which the same as pBuf + *pCnt.
char *find_word(char *pBuf, u32_t *pCnt);
char *next_word(char *pBuf, u32_t *pCnt);
char *find_space(char *pBuf, u32_t *pCnt);


// String to integer functions.
//
// Input:
//    char *pBuf -- Input string.
//    u32_t *pRetval -- Result of the conversion.
//    u32_t *pCnt -- Number of characters scanned (optional).
//
// Return:
//    Pointer to the end of the scan which the same as pBuf + *pCnt.
char *integer_from_string10(char *pBuf, u32_t *pRetval,
    u32_t *pCnt);
char *integer_from_string16(char *pBuf, u32_t *pRetval,
    u32_t *pCnt);
char *integer_from_string(char *pBuf, u32_t *pRetval, 
    u32_t *pCnt);
char *integer_from_string_ex(char *pBuf, u32_t *pRetval, 
    u32_t *pCnt, u32_t Default);


// String to IP address.
//
// Input:
//    char *pBuf -- Input IP address string in nnn.nnn.nnn.nnn format.
//    u32_t *pIpAddress -- Ip address as 32-bit integer.
//    unsinged long *pCnt -- Number of characters scanned (optional).
//
// Return:
//    Pointer to the end of the scan which the same as pBuf + *pCnt.
char *ipaddress_from_string(char *pBuf, u32_t *pIpAddress, 
    u32_t *pCnt);


char *parse_var_value(char* pBuf, char** pVar, u32_t* pValue);
char *find_right_substring(char* pBuf, char* pSubString);

#ifdef __cplusplus
}
#endif



#endif /* PARSER_H */
