/****************************************************************************
 * Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved
 * Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
 * Proprietary and Confidential Information.
 *
 * This source file is the property of QLogic Corporation, and
 * may not be copied or distributed in any isomorphic form without
 * the prior written consent of QLogic Corporation.
 *
 * Name:        sync.h
 *
 * Description: synchronization primitives
 *
 ****************************************************************************/
#ifndef _SYNC_H_
#define _SYNC_H_

#include "os_util.h"

#if ! ( defined(USER_LINUX) || defined(__LINUX)  || defined(UEFI))

#define INTERRUPT_FLAG_MASK 0x20

/**
 *
 */
#define LOCK_TYPE1   0x1
#define LOCK_TYPE2   0x2

extern u32_t lock_mask;

/* asm macros */
#define CHECKED_LOCK(lock_type) _asm { pushfd }; _asm { cli }; \
                                if ( lock_mask & (lock_type) ) { \
                                    DbgBreakMsg("Double lock");   \
                                } else { \
                                    lock_mask |= (lock_type); \
                                }
#define CHECKED_UNLOCK(lock_type) if ( ! (lock_mask & (lock_type) ) ) { \
                                    DbgBreakMsg("Illegal unlock");   \
                                } else { \
                                    lock_mask &= ~(lock_type); \
                                } \
                                _asm { popfd }


#define LOCK()   _asm { pushfd };	_asm { cli }
#define UNLOCK() _asm { popfd }

#else /* non-DOS */

#define LOCK()    os_if_lock()
#define UNLOCK()  os_if_unlock()

#endif 

#endif /* _SYNC_H_ */

