/******************************************************************************/
/* The information contained in this file is confidential and proprietary to  */
/* QLogic Corporation.  No part of this file may be reproduced or             */
/* distributed, in any form or by any means for any purpose, without the      */
/* express written permission of QLogic Corporation.                          */
/*                                                                            */
/* (c) COPYRIGHT 2011-2014 Broadcom Corporation, ALL RIGHTS RESERVED.         */
/* (c) COPYRIGHT 2014-2015 QLogic Corporation, ALL RIGHTS RESERVED.           */
/*                                                                            */
/*                                                                            */
/* Module Description:                                                        */
/*                                                                            */
/*                                                                            */
/******************************************************************************/

#if DBG

#include "debug.h"
#include <stdio.h>
#include <stdarg.h>
#include "prints.h"
#include "tcl_io.h"
#include "tcl_log.h"

extern FILE *g_logfile_ptr;

/******************************************************************************/
/* Macros for processing variable number of arguments. */
/******************************************************************************/

#ifndef _VA_LIST_DEFINED
#define _VA_LIST_DEFINED

#ifndef UEFI
typedef char *va_list;
#endif

#ifndef _INTSIZEOF
#define _INTSIZEOF(n)   ( (sizeof(n) + sizeof(int) - 1) & ~(sizeof(int) - 1) )
#endif

#ifndef va_start
#define va_start(ap,v)  ( ap = (va_list)&v + _INTSIZEOF(v) )
#endif
#ifndef va_arg
#define va_arg(ap,t)    ( *(t *)((ap += _INTSIZEOF(t)) - _INTSIZEOF(t)) )
#endif
#ifndef va_end
#define va_end(ap)      ( ap = (va_list)0 )
#endif
#endif /* _VA_LIST_DEFINED */


/****************************************************************************
 * console buffer
 ****************************************************************************/
#if ! ( (defined USER_LINUX) || (defined WIN_DIAG))
static port_buffer_t console_buffer;
extern u32_t intr_recur;
extern void tcl_io_puts(u8_t *buf, u16_t len);

void 
console_init( 
    void 
    )
{
    console_buffer.head = 0;
    console_buffer.tail = 0;
    console_buffer.uberr = 0;
    console_buffer.ubflags = 0;
    *(u32_t*)&console_buffer.ubmagic[0] = 0x6e614646;
}


static u16_t 
console_append( 
    u8_t* msg, 
    u16_t msg_len 
    )
{
    u16_t room;
    port_buffer_t* wb;

    wb = &console_buffer;
    for( ; (msg_len != 0) && !UbFull(wb); )
    {
        room = UbRoom(wb);
        if( room != 0 )
        {
            u16_t n;
            n = (room > msg_len) ? msg_len : room;
            msg_len -= n;
            for( ; n > 0; --n )
            {
                wb->ubuf[wb->head] = *msg;
                wb->head = UbPlus(wb->head);
                ++msg;
            }
        }
    }

    return msg_len;
}


u32_t
console_print( 
    void * console_buffer_p
    )
{
    port_buffer_t* wb;

    /* console_buffer_p may be NULL only during cleanup, hence not in the interrupt context */
    if (console_buffer_p) {
        wb = (port_buffer_t*)console_buffer_p;
    } else {
        wb = &console_buffer;
    }

    while( ! UbEmpty(wb) )
    {
        tcl_io_puts((u8_t*)(&(wb->ubuf[wb->tail])),1);	
        wb->tail = UbPlus( wb->tail );
    }

    return 1;
} 

void console_cleanup(void)
{
    console_print(&console_buffer);
}

#endif

#ifndef USER_LINUX
/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
STATIC long
debug_ltoh(
    unsigned long val,
    long width,
    char fill_char,
    char *dst_buf,
    unsigned long buf_len)
{
    unsigned long digit;
    unsigned long len;
    long ret_len;
    char c;

    if(buf_len == 0)
    {
        ret_len = 0;
        return ret_len;
    }

    if(width == 0)
    {
        width++;
    }

    len = 0;

    for(; ;)
    {
        if(len >= buf_len)
        {
            len = 0;
            break;
        }

        digit = val % 16;
        val = val/16;

        if(digit < 10)
        {
            *dst_buf = '0' + (char) digit;
        }
        else
        {
            *dst_buf = 'a' + (char) (digit-10);
        }

        dst_buf++;

        len++;
        width--;

        if(val == 0)
        {
            break;
        }
    }

    while(width > 0)
    {
        if(len >= buf_len)
        {
            len = 0;
            break;
        }

        *dst_buf = fill_char;
        dst_buf++;
        len++;
        width--;
    }

    ret_len = len;

    if(len)
    {
        /* Reverse the digits. */
        digit = 1;
        while(digit < len)
        {
            c = *(dst_buf-digit);
            *(dst_buf-digit) = *(dst_buf-len);
            *(dst_buf-len) = c;

            len--;
            digit++;
        }
    }

    return ret_len;
} /* debug_ltoh */



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
STATIC long
debug_ltod(
    long val,
    long width,
    char fill_char,
    char *dst_buf,
    unsigned long buf_len)
{
    unsigned long digit;
    unsigned long len;
    long ret_len;
    char c;
    unsigned char sign = 0;

    if(buf_len == 0)
    {
        ret_len = 0;

        return ret_len;
    }

    if(width == 0)
    {
        width++;
    }

    len = 0;

    if ( val < 0 ) {
        *dst_buf = '-';
        dst_buf++;
        len++;
        width--;
        sign = 1;
        val = -val;
    }

    for(; ;)
    {
        if(len >= buf_len)
        {
            len = 0;
            break;
        }

        digit = val % 10;
        val = val/10;

        *dst_buf = '0' + (char) digit;
        dst_buf++;

        len++;
        width--;

        if(val == 0)
        {
            break;
        }
    }

    while(width > 0)
    {
        if(len >= buf_len)
        {
            len = 0;
            break;
        }

        *dst_buf = fill_char;
        dst_buf++;
        len++;
        width--;
    }

    ret_len = len;
    if(len)
    {
        len -= sign;
        /* Reverse the digits. */
        digit = 1;
        while(digit < len)
        {
            c = *(dst_buf - digit);
            *(dst_buf - digit) = *(dst_buf - len);
            *(dst_buf - len) = c;

            len--;
            digit++;
        }
    }

    

    return ret_len;
} /* debug_ltod */



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
STATIC long
debug_atol(
    char *buf,
    unsigned long *ret_val)
{
    unsigned long result;
    long len;

    result = 0;
    len = 0;

    while(*buf >= '0' && *buf <= '9')
    {
        result = result * 10 + *buf-'0';
        buf++;
        len++;
    }

    *ret_val = result;

    return len;
} /* debug_atol */



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
STATIC long
debug_strcpy(
    char *src_buf,
    char *dst_buf,
    unsigned long buf_len)
{
    long len;

    len = 0;
    while(*src_buf && (len < (long) buf_len))
    {
        *dst_buf = *src_buf;
        dst_buf++; src_buf++;
        len++;
    }

    /* Return an error if buf_len is too short. */
    if(*src_buf)
    {
        len = -1;
    }

    return len;
} /* debug_strcpy */


/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
STATIC u32_t
debug_sprintf(
    char *buf,
    unsigned long buf_len,
    char *format,
    va_list args)
{
    u32_t len;
    long ret_len;

    len = 0;
    while(*format && len < buf_len)
    {
        if(*format == '%')
        {
            format++;

            switch(*format)
            {
                case '%':
                    *buf = '%'; buf++; len++;
                    break;

                case 's':
                case 'S':
                {
                    char *str_arg;
                    char *null_string="(null)";

                    str_arg = va_arg(args, char *);
                    if ( ! str_arg ) {
                        str_arg = null_string;
                    }

                    ret_len = debug_strcpy(str_arg, buf, buf_len - len);
                    if(ret_len == -1)
                    {
                        len = 0;
                    }
                    else
                    {
                        buf += ret_len;
                        len += ret_len;
                    }

                    break;
                }

                case '0': case '1': case '2': case '3': case '4':
                case '5': case '6': case '7': case '8': case '9':
                case 'd': case 'D':
                case 'x': case 'X':
                case 'p':
                {
                    unsigned long int_arg;
                    unsigned long width;
                    char fill_char;

                    if(*format == '0')
                    {
                        fill_char = '0';
                    }
                    else
                    {
                        fill_char = ' ';
                    }
                    width = 0;

                    if(*format >= '0' && *format <= '9')
                    {
                        ret_len = debug_atol(format, &width);
                        format += ret_len;
                    }

                    if(*format == 'd' || *format == 'D')
                    {
                        int_arg = va_arg(args, long);
                        ret_len = debug_ltod(int_arg, width, fill_char, buf,
                            buf_len - len);
                    }
                    else if(*format == 'x' || *format == 'X' || *format == 'p')
                    {
                        int_arg = va_arg(args, unsigned long);
                        ret_len = debug_ltoh(int_arg, width, fill_char, buf,
                            buf_len - len);
                    }
                    else
                    {
                        ret_len = -1;
                    }

                    if(ret_len == -1)
                    {
                        len = 0;
                    }
                    else
                    {
                        buf += ret_len;
                        len += ret_len;
                    }

                    break;
                }

                default:
                    break;
            }
        }
        else
        {
            *buf = *format;
            buf++;
            len++;
        }

        format++;

        /* Error encountered. */
        if(len == 0)
        {
            break;
        }
    }

    *buf = 0;

    return len;
} /* debug_printf */
#endif


void __cdecl
debug_msgx(
    unsigned long level,
    char *msg,
    ...)
{
    u32_t len;
    char buf[256];
    va_list args;
    
    if(LOG_MSG(level))
    {
        va_start(args, msg);
#ifndef USER_LINUX
        len = debug_sprintf(buf, sizeof(buf), msg, args);
#else
	len = vsnprintf(buf, sizeof(buf), msg, args);
#endif
        if(len && MSG_LEVEL(level))
        {         
#if ! ((defined USER_LINUX) || (defined WIN_DIAG))
            console_append(buf, len);
#else	
	    printf("%s", buf);
#endif
	   	
        }
        va_end(args);
#if ! ((defined USER_LINUX) || (defined WIN_DIAG))
        if ( intr_recur == 0 ) {
            console_print(NULL);
        }
#endif
    }
} /* debug_msgx */


void __cdecl
log_msgx(   
    char *msg,
    ...)
{
	va_list args;
	char buf[256];
	u32_t len;

	va_start(args, msg);
#ifndef USER_LINUX
        len = debug_sprintf(buf, sizeof(buf), msg, args);
#else
	len = vsnprintf(buf, sizeof(buf), msg, args);
#endif

	/* Record DEBUG BREAK to log file */	    
	if (g_logfile_ptr != NULL) {    		    
		tcl_log_fputs(buf, g_logfile_ptr, len);
	}
}



#endif
