
/*************************************************************************
 * Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved.
 * Copyright(c) 2014-2015 QLogic Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of QLogic Corporation, and
 * may not be copied or distributed in any isomorphic form without
 * the prior written consent of QLogic Corporation.
 *
 *
 *  Name: DISPUTIL.C
 *
 *  Description: 
 * 
 ************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#if !defined(USER_LINUX) && !defined(UEFI)
#include <conio.h>
#endif
#include <string.h>
#include "tcl.h"
#include "util.h"
#include "disputil.h"
#include "dg_os_util.h"
#include "esc_seq.h"

/* global vars */
u8_t cur_disp_col = 0xFF; 


/*
 *------------------------------------------------------------------------
 * disp_32bit_array --
 *     Display an array of 32-bit values on screen. The parameter 
 *     "start_addr" must be a multiple of four.
 *
 * Results:
 *     None.
 *
 * Side effects:
 *     None -- the array is not modified at all.
 *------------------------------------------------------------------------
 */


extern void tcl_io_puts(char *buf, u16_t len);
extern void tcl_io_flush(void);
extern int tcl_io_getch(void);
extern void os_if_print(char *buf, u16_t len); 

#ifndef USER_LINUX
void
disp_32bit_array ( u32_t *array, int_ptr_t start_addr, u32_t count, u8_t swap )
{
    volatile u32_t *ptr = (volatile u32_t *)array;
    u32_t value32, i, j, bytecount;
    char *print_buf, *print_cursor;
    u32_t print_buf_size;
    u8_t ms4b_zero = ((start_addr & 0xf0000000) == 0);

    start_addr &= 0xfffffffc;
    bytecount = count * sizeof(u32_t);
    /* Need to add 2 for storing blank spaces in case 
       (start_addr % BYTES_PER_32BIT_LINE != 0)          */
    print_buf_size = (bytecount/BYTES_PER_32BIT_LINE+2) * CHARS_PER_LINE;
    print_buf = (char *) dg_os_alloc_virt (print_buf_size);
    print_cursor = print_buf;

    for (i=0; i<bytecount; i+=sizeof(u32_t), ptr++)
    {
        if (i==0)
        {
            if (ms4b_zero)
            {
                print_cursor += sprintf(print_cursor, "%0.7x:", start_addr&0xfffffe0);
            }
            else
            {
                print_cursor += sprintf(print_cursor, "%0.8x:\n", start_addr&0xff000000);
                print_cursor += sprintf(print_cursor, "+%0.6x:", start_addr&0xfffffe);
            }
            for (j=start_addr&0xffffffe0; j<start_addr; j+=4)
            {
                print_cursor += sprintf(print_cursor, "........ ", 9*(start_addr % BYTES_PER_32BIT_LINE)/sizeof(u32_t), "");
            }
        }
        else if ((start_addr+i) % BYTES_PER_32BIT_LINE == 0)
        {
            if (ms4b_zero)
            {
                print_cursor += sprintf(print_cursor, "\n%0.7x:", (start_addr+i) & 0xfffffe0);
            }
            else
            {
                print_cursor += sprintf(print_cursor, "\n+%0.6x:", (start_addr+i)&0xfffffe);
            }
        }
        else 
        {
            print_cursor += sprintf(print_cursor, " ");
        }

        value32 = (swap & DISP_BYTE_SWAP) ? BYTE_SWAP(*ptr) : *ptr;
        print_cursor += sprintf(print_cursor, "%08x", value32);
    }
    print_cursor += sprintf(print_cursor, "\n");
    printf(print_buf);
    fflush(stdout);
    dg_os_free_virt((diag_virt_addr_t)print_buf);
}

#else 
void
disp_32bit_array ( u32_t *array, int_ptr_t start_addr, u32_t count, u8_t swap )
{
    volatile u32_t *ptr = (volatile u32_t *)array;
    u32_t value32, i, j, bytecount;
    char *print_buf, *print_cursor;
    u32_t print_buf_size;
    u8_t ms4b_zero = ((start_addr & 0xf0000000) == 0);

    start_addr &= 0xfffffffc;
    bytecount = count * sizeof(u32_t);
    /* Need to add 2 for storing blank spaces in case 
       (start_addr % BYTES_PER_32BIT_LINE != 0)          */
    print_buf_size = (bytecount/BYTES_PER_32BIT_LINE+2) * CHARS_PER_LINE;
    print_buf = (char *) dg_os_alloc_virt (print_buf_size);
    print_cursor = print_buf;

    for (i=0; i<bytecount; i+=sizeof(u32_t), ptr++)
    {
        if (i==0)
        {
            if (ms4b_zero)
            {
                print_cursor += sprintf(print_cursor, "%07lx:", start_addr&0xfffffe0);
            }
            else
            {
                print_cursor += sprintf(print_cursor, "%08lx:\n", start_addr&0xff000000);
                print_cursor += sprintf(print_cursor, "+%06lx:", start_addr&0xfffffe);
            }
            for (j=start_addr&0xffffffe0; j<start_addr; j+=4)
            {
                print_cursor += sprintf(print_cursor, "........ ");
            }
        }
        else if ((start_addr+i) % BYTES_PER_32BIT_LINE == 0)
        {
            if (ms4b_zero)
            {
                print_cursor += sprintf(print_cursor, "\n%07lx:", (start_addr+i) & 0xfffffe0);
            }
            else
            {
                print_cursor += sprintf(print_cursor, "\n+%06lx:", (start_addr+i)&0xfffffe);
            }
        }
        else 
        {
            print_cursor += sprintf(print_cursor, " ");
        }

        value32 = (swap & DISP_BYTE_SWAP) ? BYTE_SWAP(*ptr) : *ptr;
        print_cursor += sprintf(print_cursor, "%08x", value32);
    }
    print_cursor += sprintf(print_cursor, "\n");
    printf(print_buf);
    fflush(stdout);
    dg_os_free_virt((diag_virt_addr_t)print_buf);
}
#endif
/*
 *------------------------------------------------------------------------
 * disp_8bit_array --
 *
 *     Display an array of 8-bit values along with ASCII on screen.
 *
 * Results:
 *     None.
 *
 * Side effects:
 *     None -- the array is not modified at all.
 *------------------------------------------------------------------------
 */
#define FOUR_BYTE_SPACING 1
void
disp_8bit_array ( u8_t *array, int_ptr_t start_addr, u32_t count)
{
    volatile u8_t *ptr, *ascii_ptr;
    u32_t bytes_skipped, hex_skipped=0;
    int_ptr_t curr_addr, end_addr;
    char *print_buf, *print_cursor;
    u32_t print_buf_size, j;

    curr_addr = (int_ptr_t)start_addr;
    end_addr = (int_ptr_t)start_addr+count;
    print_buf_size = (count/BYTES_PER_8BIT_LINE+1) * CHARS_PER_LINE;
    print_buf = (char *) dg_os_alloc_virt (print_buf_size);
    print_cursor = print_buf;

    for (ptr=(volatile u8_t *)array; curr_addr<end_addr; )
    {
        ascii_ptr = ptr;
        print_cursor += sprintf(print_cursor, "%08lx:", (curr_addr) & 0xfffffff0);
        if (curr_addr==start_addr)
        {
            bytes_skipped = start_addr % BYTES_PER_8BIT_LINE;
#if FOUR_BYTE_SPACING
            for (j=0; j<bytes_skipped; j++)
            {
                print_cursor += sprintf(print_cursor, " ..");
                if (j%SEGMENTS_PER_8BIT_LINE == (SEGMENTS_PER_8BIT_LINE-1)) 
                    print_cursor += sprintf(print_cursor, " ");
            }
            //print_cursor += sprintf(print_cursor, "%*s", 
            //       3*bytes_skipped + ((bytes_skipped+SEGMENTS_PER_8BIT_LINE-1)/SEGMENTS_PER_8BIT_LINE),
            //       "");
#else
            for (j=0; j<bytes_skipped; j++)
                print_cursor += sprintf(print_cursor, " ..");
            //print_cursor += sprintf(print_cursor, "%*s", 3*bytes_skipped, "");
            if (curr_addr % BYTES_PER_8BIT_LINE != 0) print_cursor += sprintf(print_cursor, " ");
#endif
        }
        else
        {
            bytes_skipped = 0;
        }

        /* Dump hex byte by byte */
        do
        {
#if FOUR_BYTE_SPACING
            /* Add a space every four bytes */
            if ((curr_addr) % BYTES_PER_8BIT_SEGMENT == 0)
#else
            if ((curr_addr) % BYTES_PER_8BIT_LINE == 0)
#endif
                print_cursor += sprintf(print_cursor, " ");
            print_cursor += sprintf(print_cursor, "%02x ", *ptr);
            curr_addr++; ptr++;
            if (curr_addr==end_addr)
            {
                hex_skipped = BYTES_PER_8BIT_LINE - (curr_addr) % BYTES_PER_8BIT_LINE;
                hex_skipped %= BYTES_PER_8BIT_LINE;
                break;
            }
            else
            {
                hex_skipped = 0;
            }
        } while ((curr_addr) % BYTES_PER_8BIT_LINE != 0);

        /* Dump corresponding ASCII char byte by byte */
#if FOUR_BYTE_SPACING
        print_cursor += sprintf(print_cursor, "%*s", 3*hex_skipped+(hex_skipped/SEGMENTS_PER_8BIT_LINE), "");
#else
        print_cursor += sprintf(print_cursor, "%*s", 3*hex_skipped, "");
#endif
        print_cursor += sprintf(print_cursor, " ");
        print_cursor += sprintf(print_cursor, "%*s", bytes_skipped, "");
        while (ascii_ptr != ptr)
        {
            print_cursor += sprintf(print_cursor, "%c", isprint(*ascii_ptr) ? *ascii_ptr : '.');
            ascii_ptr++;
        }
        print_cursor += sprintf(print_cursor, "\n");
    }
    printf(print_buf);
    fflush(stdout);
    dg_os_free_virt((diag_virt_addr_t)print_buf);
}

/*
 *------------------------------------------------------------------------
 * disp_pause --
 *
 *     Display a pause message and wait for user input (a keystroke).
 *     Neither the message nor the user keystroke will be logged.
 *
 * Returns:
 *     The key pressed
 *
 * Side effects:
 *     None
 *------------------------------------------------------------------------
 */
int
disp_pause ( char *msg)
{
    int key_pressed;
    tcl_io_puts(msg, strlen(msg) + 1);
    tcl_io_flush();
    key_pressed = tcl_io_getch();
    tcl_io_puts("\n", 2);

    return key_pressed;
}
 
/*
 *------------------------------------------------------------------------
 * sc_change_color 
 *
 * Description:
 *     Change the color, used to show ERROR / PASS return value of tests
 * 
 * Returns:
 *     A standard Tcl result.
 *------------------------------------------------------------------------
*/
void sc_change_color(u8_t color, u8_t fore_or_back)
{
    if (cur_disp_col != color)
    {
        dg_os_change_color(color, fore_or_back);
        cur_disp_col = color;
    }
}


void sc_restore_color(void)
{
    if (cur_disp_col != 0xFF)
    {
        dg_os_restore_color();
        cur_disp_col = 0xFF;
    }    
}

void sc_restore_cursor(void)
{
	os_if_restore_cursor();
}

void sc_save_cursor(void)
{
	os_if_save_cursor();
}

void sc_print(char *buf, u16_t len)
{
	os_if_print(buf, len);
}

#ifdef TEST_MAIN
#define MAX_8BIT_VAL 255
static u8_t test_string[MAX_8BIT_VAL+1];

int main (int argc, char *argv[])
{
    u32_t i;
    for (i=0; i<MAX_8BIT_VAL+1; i++) test_string[i] = i;

#if 0
    for (i=0; i<BYTES_PER_32BIT_LINE; i+=sizeof(u32_t))
    {
        disp_32bit_array ( (u32_t *)test_string, i, 
                           (MAX_8BIT_VAL+1)/sizeof(u32_t), 0 );
        disp_pause("pause...");
    }
#endif

    for (i=0; i<BYTES_PER_8BIT_LINE; i++)
    {
        disp_8bit_array ( (u8_t *)test_string, i, MAX_8BIT_VAL+1 );
        disp_pause("pause...");
    }
    return 0;
}
#endif


