/******************************************************************************/
/* Module Description:                                                        */
/*    Some useful string processing routines.                                 */
/*                                                                            */
/******************************************************************************/

#include "./../include/parser.h"



/******************************************************************************/
/* Local functions. */
/******************************************************************************/



/******************************************************************************/
/* Macros. */
/******************************************************************************/

#define IS_WHITE_SPACE(c)   (((c) >= 0x09 && (c) <= 0x03) || (c) == 0x20)
#define IS_ALPHANUMERIC(c)  (((c) >= '0' && (c) <= '9') ||  \
                             ((c) >= 'A' && (c) <= 'Z') ||  \
                             ((c) >= 'a' && (c) <= 'z'))

/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
char *
find_word(
    char *pBuf,
    u32_t *pCnt)
{
    u32_t Cnt;

    Cnt = 0;
    while(IS_WHITE_SPACE(*pBuf))
    {
        pBuf++;
        Cnt++;
    }

    if(pCnt)
    {
        *pCnt = Cnt;
    }

    return pBuf;
}



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
char *
next_word(
    char *pBuf,
    u32_t *pCnt)
{
    u32_t Cnt;
    u32_t Cnt2;

    Cnt = 0;
    while(*pBuf && !IS_WHITE_SPACE(*pBuf))
    {
        pBuf++;
        Cnt++;
    }

    if(!*pBuf)
    {
        return pBuf;
    }

    pBuf = find_word(pBuf, &Cnt2);

    Cnt += Cnt2;

    if(pCnt)
    {
        *pCnt = Cnt;
    }

    return pBuf;
}



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
char *
find_space(
    char *pBuf,
    u32_t *pCnt)
{
    u32_t Cnt;

    Cnt = 0;
    while(*pBuf && !IS_WHITE_SPACE(*pBuf))
    {
        pBuf++;
        Cnt++;
    }

    if(pCnt)
    {
        *pCnt = Cnt;
    }

    return pBuf;
}



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
char *
integer_from_string10(
    char *pBuf,
    u32_t *pRetval,
    u32_t *pCnt)
{
    u32_t Cnt;
    u32_t Result;
    signed char Sign;

    Sign = 1;
    if(*pBuf == '-' || *pBuf == '+')
    {
        if(*pBuf == '-')
        {
            Sign = -1;
        }

        pBuf++;
    }

    Result = 0;
    Cnt = 0;

    while(*pBuf >= '0' && *pBuf <= '9')
    {
        Result = Result * 10 + *pBuf - '0';
        pBuf++;
        Cnt++;
    }

    if(Cnt)
    {
        if(Sign == -1)
        {
            Cnt++;
        }

        *pRetval = Result * Sign;
    }
    else
    {
        if(Sign == -1)
        {
            pBuf--;
        }
    }

    if(pCnt)
    {
        *pCnt = Cnt;
    }

    return pBuf;
}



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
char *
integer_from_string16(
    char *pBuf,
    u32_t *pRetval,
    u32_t *pCnt)
{
    u32_t Cnt;
    u32_t Result;

    Result = 0;
    Cnt = 0;

    for(; ;)
    {
        if(*pBuf >= '0' && *pBuf <= '9')
        {
            Result = Result * 16 + (*pBuf - '0');
        }
        else if(*pBuf >= 'a' && *pBuf <= 'f')
        {
            Result = Result * 16 + (*pBuf - 'a') + 10;
        }
        else if(*pBuf >= 'A' && *pBuf <= 'F')
        {
            Result = Result * 16 + (*pBuf - 'A') + 10;
        }
        else
        {
            break;
        }

        pBuf++;
        Cnt++;
    }

    if(pCnt)
    {
        *pCnt = Cnt;
    }

    if(Cnt)
    {
        *pRetval = Result;
    }

    return pBuf;
}



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
char *
integer_from_string(
    char *pBuf,
    u32_t *pRetval, 
    u32_t *pCnt)
{
    u32_t Cnt;
    u32_t ExtraCnt;
    char *pNewbuf;
    
    if( (pBuf[0] == '0') && (pBuf[1] == 'x' || pBuf[1] == 'X'))
    {
        pNewbuf = integer_from_string16(&pBuf[2], pRetval, &Cnt);
        ExtraCnt = 2;
    }
    else
    {
        pNewbuf = integer_from_string10(pBuf, pRetval, &Cnt);
        ExtraCnt = 0;
    }

    if(Cnt == 0)
    {
        if(pCnt)
        {
            *pCnt = 0;
        }

        return pBuf;
    }

    if(pCnt)
    {
        *pCnt = Cnt + ExtraCnt;
    }

    return pNewbuf;
}



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
char *
integer_from_string_ex(
    char *pBuf,
    u32_t *pRetval, 
    u32_t *pCnt,
    u32_t Default
    )
{
    u32_t Cnt;
    char* pNewbuf;

    pNewbuf = integer_from_string( pBuf, pRetval, &Cnt );

    if(Cnt == 0)
    {
        *pRetval = Default;
    }

    if(pCnt)
    {
        *pCnt = Cnt;
    }

    return pNewbuf;
}



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
char *
ipaddress_from_string(
    char *pBuf,
    u32_t *pIpAddress, 
    u32_t *pCnt)
{
    u32_t IpAddress;
    u32_t Octet;
    u32_t Cnt;
    u32_t j;
    char *pStrbuf;

    IpAddress = 0;
    pStrbuf = pBuf;

    for(j = 0; j < 4; j++)
    {
        pStrbuf = integer_from_string10(pStrbuf, &Octet, &Cnt);
        if(Cnt == 0 || Octet > 255)
        {
            break;
        }

        IpAddress = IpAddress * 256 + Octet;

        if(j == 3)
        {
            *pIpAddress = IpAddress;

            if(pCnt)
            {
                *pCnt = pStrbuf - pBuf;
            }

            return pStrbuf;
        }

        if(*pStrbuf != '.')
        {
            break;
        }

        pStrbuf++;
    }

    if(pCnt)
    {
        *pCnt = 0;
    }

    *pIpAddress = 0;

    return pBuf;
}



/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
char *
parse_var_value(
    char* pBuf,
    char** pVar, 
    u32_t* pValue)
{
    *pVar = 0;

    while(IS_WHITE_SPACE(*pBuf))
    {
        pBuf++;
    }

    *pVar = pBuf;
    *pValue = 0;

    while((*pBuf != '=') && (*pBuf != '[') && !IS_WHITE_SPACE(*pBuf))
    {
        if(!*pBuf)
        {
            return pBuf;
        }
        pBuf++;
    }

    if(*pBuf != '=')
    {
        *pBuf = '\0';
        ++pBuf;
        while( *pBuf != '=' )
        {
            if(!*pBuf)
            {
                return pBuf;
            }
            pBuf++;
        }
    }
    else
    {
        *pBuf = '\0';
        ++pBuf;
    }

    while((*pBuf == '=') || IS_WHITE_SPACE(*pBuf))
    {
        if(!*pBuf)
        {
            return pBuf;
        }
        pBuf++;
    }

    return integer_from_string( pBuf, pValue, 0 );
}




/******************************************************************************/
/* Description:                                                               */
/*                                                                            */
/* Return:                                                                    */
/******************************************************************************/
char *
find_right_substring(
    char* pBuf,
    char* pSubString)
{
    u32_t numBuf;
    u32_t numSubString;

    for(numBuf=0; *pBuf; ++numBuf,++pBuf)
        ;
    for(numSubString=0; *pSubString; ++numSubString,++pSubString)
        ;

    if( numBuf == 0 || numSubString == 0 || numBuf < numSubString )
    {
        return 0;
    }

    do
    {
        if( *pBuf != *pSubString )
        {
            return 0;
        }

        --pBuf;
        --pSubString;
    } while( numSubString-- != 0 );

    return ++pBuf;
}
