/****************************************************************************
 * Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved
 * Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
 * Proprietary and Confidential Information.
 *
 * This source file is the property of QLogic Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of QLogic Corporation. 
 *
 * Name:        prints.c
 *
 * Description: printf replacement functions of the stdio for output
 *              redirection.
 * 
 *
 ****************************************************************************/
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#ifdef USER_LINUX
#include <sys/types.h>
#endif
#include "bcmtype.h"
#include "prints.h"
#include "uart.h"
#include "tcl.h"


extern uart_info_t uart_info;


#if 0
/* modal print function */
int vbprintf(int verbMode, U8 *fmt, ...)
{
   va_list arglist;
   int rc;
   
   va_start(arglist, fmt);
   rc = vsprintf(toeGlobal.print_buf, fmt, arglist);

   if (toeGlobal.verbose_mode & V_CONSOLE)
         printf(toeGlobal.print_buf);

   /* implement other print utilities here such as warning,
      informational, etc. 
    */      

   va_end(arglist);
   return(rc);
}    
#endif

/* replacement for printf () */
int tPrintf(const char *fmt, ...)
{
   va_list arglist;
   int rc;
   Tcl_Channel outChannel;
  
   va_start(arglist, fmt);

   /* Format the buffer */
   rc = vsprintf(uart_info.print_buf, fmt, arglist);

   if (uart_info.verbose_mode & V_BUFFERED)
   {
      strcat(uart_info.auxprint_buf, uart_info.print_buf);    
   }

   /* Get stdout channel */
   outChannel = Tcl_GetStdChannel( TCL_STDOUT );
   Tcl_WriteChars(outChannel, uart_info.print_buf, -1);

   va_end(arglist);
   return(rc);
}   

void buffer_printf_start()
{
   uart_info.auxprint_buf[0] = 0;  
   uart_info.verbose_mode |= V_BUFFERED;
}   

void buffer_printf_stop()
{
   uart_info.verbose_mode &= ~V_BUFFERED;
}   
