#include "stdio.h"
#include "man_types.h"
#include "edebug_types.h"
#include "man_types.h"
#include "dbg_man.h"
#include "hw_access.h"
#include "tcl_al.h"

extern edebug_eng_mode_t eng_mode;

dbg_status_t dbg_dev_init_dispatch(dbg_dev_info_t * dev_info_p)
{
    if (IS_EVEREST(dev_info_p->pci_info)) {
        if (0 == strcmp("XXXX", dev_info_p->pci_info.eth_name)) {
            return DBG_STATUS_FAILURE;
        }
        if (ETH_D0 != dev_info_p->pci_info.eth_state) {
            return DBG_STATUS_FAILURE;
        }
        return everest_dev_if_init_dispatch(dev_info_p->br_idx,
                                            dev_info_p->pci_info,
                                            dev_info_p->drv_dev_if_p);
    }

    return DBG_STATUS_INVALID_PARAMETER;
}

dbg_status_t dbg_select_dev_num(
    u32_t dev_num
    )
{
    /* in non pci mode nothing should be done here */
    if ((tcl_al_get_no_pci_mode() == 1) || (eng_mode == EDEBUG_VIRTUAL_NVM_MODE)) {
            xdbg_cur_dev = (dbg_dev_hdl_t)BR_DEV_IDX(dev_num);
            return DBG_STATUS_SUCCESS;
    }

    /* select device <dev_num> and verify it is present */
    if ( BR_PCI_DEV_PRESENT(dev_num) ) {
        dbg_dev_info_t * dev_info_p = BR_DEV_INFO(dev_num);
        xdbg_cur_dev = (dbg_dev_hdl_t)BR_DEV_IDX(dev_num);

        /* Set "setdut" address to the address of the currently selected device */
        dbg_pcicfg_setdut(dev_info_p->pci_info.pci_domain,
						  dev_info_p->pci_info.pci_bus,
                          dev_info_p->pci_info.pci_device,
                          dev_info_p->pci_info.pci_func);
        return DBG_STATUS_SUCCESS;
    } else {
        return DBG_STATUS_FAILURE;
    }
}

pci_info_t * dbg_get_br_pci_info(
    int dev_idx
    )
{
    if ( BR_PCI_DEV_PRESENT(dev_idx) ) {
        return &(BR_DEV_INFO(dev_idx)->pci_info);
    } else {
        return NULL;
    }
}

dbg_status_t dbg_dev_reg_read(
    dev_if_t * cur_dev,
    u32_t addr,
    u32_t * val_p)
{
    return cur_dev->dev_if_reg_read(cur_dev->os_params, addr, val_p);
}

dbg_status_t dbg_dev_reg_dread(
    dev_if_t * cur_dev,
    u32_t addr,
    u32_t * val_p)
{
    return cur_dev->dev_if_reg_read(cur_dev->os_params, addr, val_p);
}

dbg_status_t dbg_dev_reg_write(
    dev_if_t * cur_dev,
    u32_t addr,
    u32_t val)
{
    return cur_dev->dev_if_reg_write(cur_dev->os_params, addr, val);
}

dbg_status_t dbg_dev_reg_dwrite(
    dev_if_t * cur_dev,
    u32_t addr,
    u32_t val)
{
    return cur_dev->dev_if_reg_write(cur_dev->os_params, addr, val);
}

dbg_status_t dbg_dev_read_nvram(
    IN    dev_if_t * cur_dev,
    IN    u32_t      addr,
    OUT   u32_t    * val_p,
    IN    u32_t      byte_cnt
    )
{
    return cur_dev->dev_if_nv_read(cur_dev->os_params, addr, val_p, byte_cnt);
}

dbg_status_t dbg_dev_write_nvram(
    IN    dev_if_t * cur_dev,
    IN    u32_t      addr,
    IN    u32_t    * val_p,
    IN    u32_t      byte_cnt
    )
{
    return cur_dev->dev_if_nv_write(cur_dev->os_params, addr, val_p, byte_cnt);
}

dbg_status_t dbg_dev_stats_show(
    dev_if_t * cur_dev)
{
    return cur_dev->dev_if_stats_show(cur_dev->os_params);
}

/*
 *************************************************************************
 Get the latest assert index offset.

 Inputs:
 dev		- device handler
 instance	- x (for xsem) or u (for usem) or t (for tsem) or c (for csem)

 Returns:
 DIAG_STATUS_SUCCESS or DIAG_STATUS_FAILURE
 The returned index offset is in index_offset_ptr
 *********************************************************************
 */
dbg_status_t dbg_dev_assert_get_last_assert_index_offset(
	IN  dev_if_t *  cur_dev,			/* device handler */
	IN  char*		instance,			/* x (for xsem) or u (for usem) or t (for tsem) or c (for csem) */
	OUT u32_t*		index_offset_ptr	/* latest assert index return value */
	)
{
	/* Get the assert index offset address */
	if (strcmp (instance, "X") == 0) {
		*index_offset_ptr = 0x2000 /*XSTORM_ASSERT_LIST_INDEX_OFFSET*/;
	} else if (strcmp (instance, "T") == 0) {
		*index_offset_ptr = 0x2000 /*TSTORM_ASSERT_LIST_INDEX_OFFSET*/;
	} else if (strcmp (instance, "U") == 0) {
		*index_offset_ptr = 0x2000 /*USTORM_ASSERT_LIST_INDEX_OFFSET*/;
	} else if (strcmp (instance, "C") == 0) {
		*index_offset_ptr = 0x2000 /*CSTORM_ASSERT_LIST_INDEX_OFFSET*/;
	} else {
		return DBG_STATUS_FAILURE;
	}

	return DBG_STATUS_SUCCESS;
}
