#ifndef _DBG_DEV_IF_H_
#define _DBG_DEV_IF_H_

#include "edebug_types.h"
#include "dev_if.h"
#include "edebug_if.h"

/* FW declaration */
struct _dev_if_t;

/* opaque handles */
/* Used as device identifier when diag calls driver (dev_if* calls) */
typedef void* dev_hdl_t;

/* read 32 bits value from given grc addr */
typedef dbg_status_t (* dev_if_reg_read_t)(      
	IN  struct os_params    os_params,
	IN  u32_t		addr,
	OUT u32_t*		value
	);

/* write 32 bits value from given grc addr */
typedef dbg_status_t (* dev_if_reg_write_t)(       
	IN  struct os_params	os_params,
	IN  u32_t		addr,
	IN  u32_t		value
	);

/* gets more info for the device */
typedef dbg_status_t (* dev_if_get_dev_info_t)(      
	IN  struct os_params    os_params,	
	OUT  pci_info_t         *p_pci_info
	);

/* OS depndent cleanup */
typedef dbg_status_t (* dev_if_cleanup_t)(       
	IN  struct _dev_if_t	*dev_if	
	);

/* read the given number of bytes from the given nvm addr */
typedef dbg_status_t (* dev_if_nv_read_t)(
        IN struct os_params     os_params,
        IN  u32_t		addr,
	OUT u32_t*		value,
        IN  u32_t               size
	);

/* write the given number of bytes to the given nvm addr */
typedef dbg_status_t (* dev_if_nv_write_t)(
        IN struct os_params     os_params,
	IN  u32_t		addr,
	IN  u32_t*		value,
        IN  u32_t       size
	);

/* show NIC statistcis */
typedef dbg_status_t (* dev_if_stats_show_t)(
	IN struct os_params 	os_params
	);


typedef struct _dev_if_t
{
	/* parameters needed by the OS to perform dispatch table ops */
	struct os_params   os_params;

	/* Dispatch table */ 
	dev_if_reg_read_t   		dev_if_reg_read;
	dev_if_reg_write_t  		dev_if_reg_write;
	dev_if_get_dev_info_t		dev_if_get_dev_info;
	dev_if_cleanup_t      		dev_if_cleanup;
	dev_if_nv_read_t    		dev_if_nv_read;
	dev_if_nv_write_t   		dev_if_nv_write;
	dev_if_stats_show_t 		dev_if_stats_show;
} dev_if_t;

#endif

