#ifndef __DBG_HW_ACCESS_LAYER_H__
#define __DBG_HW_ACCESS_LAYER_H__

#include "man_types.h"
#include "57712_reg.h"


#define CHIP_IS_E1(_p)   (_p.device_id == CHIP_NUM_5710)
						 
#define CHIP_IS_E1H(_p)  ((_p.device_id == CHIP_NUM_5711) || (_p.device_id == CHIP_NUM_5711E))

#define CHIP_IS_E1X(_p)   ((CHIP_IS_E1(_p) || CHIP_IS_E1H(_p))

#define CHIP_IS_E2(_p) 	 ((_p.device_id == CHIP_NUM_5712) ||			\
                          (_p.device_id == CHIP_NUM_5712E)||			\
                          (_p.device_id == CHIP_NUM_5713) ||			\
                          (_p.device_id == CHIP_NUM_5713E))

#define CHIP_IS_E3(_p)   ((_p.device_id == CHIP_NUM_57800) ||			\
                         (_p.device_id == CHIP_NUM_57810) ||			\
                         (_p.device_id == CHIP_NUM_57840_OBSOLETE) ||		\
                         (_p.device_id == CHIP_NUM_57840_MF_OBSOLETE) ||	\
                         (_p.device_id == CHIP_NUM_57840_4_10) ||		\
                         (_p.device_id == CHIP_NUM_57840_2_20) ||		\
                         (_p.device_id == CHIP_NUM_57800_MF) ||			\
                         (_p.device_id == CHIP_NUM_57810_MF) ||			\
                         (_p.device_id == CHIP_NUM_57840_MF) ||			\
                         (_p.device_id == CHIP_NUM_57811) || \
                         (_p.device_id == CHIP_NUM_57811_MF) || \
                         (_p.device_id == CHIP_NUM_57811_VF))

        
//((strcmp(_p->board_type, "57712") == 0) || (strcmp(_p->board_type, "57712E") == 0))
#define CHIP_REV(_p)                    ((_p) & 0x0000f000)
        #define CHIP_REV_Ax                 0x00000000
        #define CHIP_REV_Bx                 0x00001000
        #define CHIP_REV_Cx                 0x00002000
        #define CHIP_REV_FPGA               0x0000f000
        #define CHIP_REV_EMUL               0x0000e000

#define CHIP_REV_IS_ASIC(_p)                (CHIP_REV(_p) <= 0x00005000)
#define CHIP_REV_IS_FPGA(_p)                ((CHIP_REV(_p) > 0x00005000) &&  (CHIP_REV(_p) & 0x00001000))
#define CHIP_REV_IS_EMUL(_p)                ((CHIP_REV(_p) > 0x00005000) &&  !((CHIP_REV(_p) & 0x00001000)))

#define CHIP_METAL(_p)                  ((_p) & 0x00000ff0)

#define PCI_CFG_SRIOV_BASE              0x1C0
#define SRIOV_NUM_VFS_OFFSET            0x10
#define SRIOV_FIRST_VF_STRIDE_OFFSET    0x14

#define PCIE_LANE_SPEED_2_5G            1
#define PCIE_LANE_SPEED_5G              2
#define PCIE_LANE_SPEED_8G              3

// In E2 chip rev A0 the PCI LANE speed are different (ERR 8)
#define PCIE_LANE_SPEED_2_5G_E2_A0      0
#define PCIE_LANE_SPEED_5G_E2_A0        1

#define CHIP_NUM_5710               0x164e
#define CHIP_NUM_5711               0x164f
#define CHIP_NUM_5711E              0x1650
#define CHIP_NUM_5712               0x1662
#define CHIP_NUM_5712E              0x1663
#define CHIP_NUM_5713               0x1651
#define CHIP_NUM_5713E              0x1652
#define CHIP_NUM_57840_OBSOLETE     0x168d
#define CHIP_NUM_57810              0x168e
#define CHIP_NUM_57800              0x168a
#define CHIP_NUM_57800_MF           0x16a5
#define CHIP_NUM_57840_MF_OBSOLETE  0x16ae
#define CHIP_NUM_57810_MF           0x16ab
#define CHIP_NUM_57811              0x163d
#define CHIP_NUM_57811_MF           0x163e
#define CHIP_NUM_57811_VF           0x163f
#define CHIP_NUM_57840_4_10         0x16a1
#define CHIP_NUM_57840_2_20         0x16a2
#define CHIP_NUM_57840_MF           0x16a4
#define CHIP_NUM_57840_VF           0x16ad

/* needed because in winediag this paramtere is known in the end of the process */
#define DEVICE_ID_STILL_UNKNOWN		0xFF00 
#define BROADCOM_VID				0x14e4

// #define PORT_FEATURE_EN_SIZE_MASK		    0x07000000
// #define PORT_FEATURE_EN_SIZE_SHIFT		    24
// #define PORT_FEATURE_WOL_ENABLED		    0x01000000
// #define PORT_FEATURE_MBA_ENABLED		    0x02000000
// #define PORT_FEATURE_MFW_ENABLED		    0x04000000


#define PORT_FEATURE_MBA_BOOT_AGENT_TYPE_MASK       0x00000007
#define PORT_FEATURE_MBA_BOOT_AGENT_TYPE_SHIFT      0
#define PORT_FEATURE_MBA_BOOT_AGENT_TYPE_PXE        0x00000000
#define PORT_FEATURE_MBA_BOOT_AGENT_TYPE_RPL        0x00000001
#define PORT_FEATURE_MBA_BOOT_AGENT_TYPE_BOOTP      0x00000002
#define PORT_FEATURE_MBA_BOOT_AGENT_TYPE_ISCSIB     0x00000003
#define PORT_FEATURE_MBA_BOOT_AGENT_TYPE_FCOE_BOOT  0x00000004
#define PORT_FEATURE_MBA_BOOT_AGENT_TYPE_NONE       0x00000007

#define SERIAL_NUMBER_OFFSET 0xC

#define NVRAM_1MB_SIZE              0x20000  // 1M bit in bytes
#define NVRAM_PAGE_SIZE             256


#define DEVICE_VENDOR_ID_REGISTER 						0x2000
#define BAR_ME_REGISTER 								0x450000
#define ME_REG_ABS_PF_NUM_SHIFT                         16
#define ME_REG_ABS_PF_NUM                               (7L<<ME_REG_ABS_PF_NUM_SHIFT) /* Absolute PF Num */
#define ME_REG_PF_NUM_SHIFT				0
#define ME_REG_PF_NUM   				(7L<<ME_REG_PF_NUM_SHIFT) /* Relative PF Num */
#define OFFSETOF_SHMEM2_OTHER_SHMEM_BASE_ADDR           0x34     
#define DBU_REG_DBU_VFID_CFG_OFFSET                     0x8818



/**
 * Description:
 *      Initialized a device dispatch table.
 *      Device information struct contains a pinter to the dispatch table,
 *      which is updated.
 * 
 * Returns:
 *      DBG_STATUS_SUCCESS if all was okay,
 *      DBG_STATUS_FAILURE otherwise
 */ 
dbg_status_t dbg_dev_init_dispatch(
    INOUT dbg_dev_info_t * dev_info_p   /* device information struct */
    );

dbg_status_t dbg_dev_assert_get_last_assert_index_offset(
	IN  dev_if_t *  cur_dev,			/* device handler */		
	IN  char*		instance,			/* x (for xsem) or u (for usem) or t (for tsem) or c (for csem) */
	OUT u32_t*		index_offset_ptr	/* latest assert index return value */
	);

#endif
