#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "57712_reg.h"
#include "edebug_types.h"
#include "bcmtype.h"
#include "man_types.h"
#include "hw_access.h"
#include "license.h"
#include "mcp_shmem.h"
#include "pcics_reg_driver.h"
#include "util.h"
#include "tcl.h"
#include "tclDecls.h"
#include "tcl_al.h"
#include "tcl_al_common.h"

extern Tcl_Interp *globalInterp;
extern edebug_eng_mode_t eng_mode;
extern virtual_nvm_t virtual_nvm;

#define SHMEM_READ(index, _addr,_p_val)    \
        if ((BR_DEV_INFO(index)->pci_info.shmem_base)) { \
                dbg_reg_read((BR_DEV_INFO(index)->pci_info.shmem_base+_addr), (_p_val)); \
        } \
        else { \
                /* shmem is not valid, return zero */ \
                *_p_val = 0; \
        }


#define MAX_READ_RETRIES 5
#define WAIT_MS 		 500

u8 dbg_dev_get_abs_func_num(u32_t num_paths, u32_t index)
{
        u32 val;

        /* first select the device */
        dbg_select_dev_num(index);

        dbg_reg_read(BAR_ME_REGISTER, &val);
        if (num_paths == 1) {
                return (u8)((val & ME_REG_PF_NUM) >> ME_REG_PF_NUM_SHIFT);
        } else {
                return (u8)((val & ME_REG_ABS_PF_NUM) >> ME_REG_ABS_PF_NUM_SHIFT);
        }
}

dbg_status_t dbg_dev_get_shmem_base(u32_t index)
{
        u32_t val, i;
        dbg_status_t status;
        
        dbg_dev_info_t *cur_dev_p = BR_DEV_INFO(index);     

        /* first select the device */
        dbg_select_dev_num(index);

        if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
                return DBG_STATUS_SUCCESS;
        }

        /* get shmem1 base addr */
        for (i=0; i<MAX_READ_RETRIES; i++) {
            status=dbg_reg_read(MISC_REG_SHARED_MEM_ADDR, &val);   
            if (status==DBG_STATUS_SUCCESS) break;
            dbg_msleep(WAIT_MS);
        }
        if (status!=DBG_STATUS_SUCCESS) {
            return DBG_STATUS_FAILURE;
        }        

        if (val == 0xFFFFFFFF) {
                BR_DEV_INFO(index)->hidden = 1;
        }
        else {
                BR_DEV_INFO(index)->pci_info.shmem_base = val;               
        }        
        
        /* get shmem2 base addr */  
        for (i=0; i<MAX_READ_RETRIES; i++) {
            
            status=dbg_reg_read((DBG_PATH_NUM(*cur_dev_p) ? MISC_REG_GENERIC_CR_1 : MISC_REG_GENERIC_CR_0), &val);   
            if (status==DBG_STATUS_SUCCESS) break;
            dbg_msleep(WAIT_MS);
        }
        
        if (status!=DBG_STATUS_SUCCESS) {
            printf("Error - Failed to read shmem base2 !\n");
            return DBG_STATUS_FAILURE;
        }

        if (val == 0xFFFFFFFF) {
                BR_DEV_INFO(index)->hidden = 1;
        }
        else {
                BR_DEV_INFO(index)->pci_info.shmem_base2 = val;
        }
        
        /* get mf_cfg base */
        for (i=0; i<MAX_READ_RETRIES; i++) {
            status = shmem2_read(OFFSETOF(shmem2_region_t, mf_cfg_addr), &val);
            if (status==DBG_STATUS_SUCCESS) break;
            dbg_msleep(WAIT_MS);           
        }

        if (status != DBG_STATUS_SUCCESS) {
                printf("Error - Failed to read mf_cfg value\n");
                return DBG_STATUS_FAILURE;
        }
        else {
                BR_DEV_INFO(index)->pci_info.mf_cfg_base = val;
        }

        return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_dev_get_board_type(u32_t index)
{
        u32_t val;
        u32 num_paths;

        dbg_select_dev_num(index);

        if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
                switch (virtual_nvm.chip_type) {
                case CHIP_TYPE_E1:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57710");
                        break;
                case CHIP_TYPE_E1X:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57711");
                        break;
                case CHIP_TYPE_E2:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57712");
                        break;
                case CHIP_TYPE_E3:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "578XX");
                        break;
                }

                return DBG_STATUS_SUCCESS;
        }

        if ((tcl_al_get_no_pci_mode() == 1) || (!XDBG_CUR_DEV_IS_HIDDEN)) {
                dbg_reg_read(MISC_REG_CHIP_NUM, &val);
                switch (val) {
                case CHIP_NUM_5710:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57710");
                        num_paths = 1;
                        break;
                case CHIP_NUM_5711:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57711");
                        num_paths = 1;
                        break;
                case CHIP_NUM_5711E:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57711E");
                        num_paths = 1;
                        break;    
                case CHIP_NUM_5712:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57712");
                        num_paths = 2;
                        break;    
                case CHIP_NUM_5712E:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57712E");
                        num_paths = 2;
                        break;    
                case CHIP_NUM_5713:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57713");
                        num_paths = 1;
                        break;    
                case CHIP_NUM_5713E:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57713E");
                        num_paths = 1;
                        break;    
                case CHIP_NUM_57800:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57800");
                        num_paths = 2;
                        break;    
                case CHIP_NUM_57810:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57810");
                        num_paths = 2;
                        break;
                case CHIP_NUM_57840_4_10:
                case CHIP_NUM_57840_2_20:
                case CHIP_NUM_57840_OBSOLETE:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57840");
                        num_paths = 2;
                        break;
                case CHIP_NUM_57800_MF:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57800_MF");
                        num_paths = 2;
                        break;    
                case CHIP_NUM_57810_MF:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57810_MF");
                        num_paths = 2;
                        break;
                case CHIP_NUM_57840_MF:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57840_MF");
                        num_paths = 2;
                        break;
                case CHIP_NUM_57840_MF_OBSOLETE:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57840_MF");
                        num_paths = 2;
                        break;
                case CHIP_NUM_57811:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57811");
                        num_paths = 1;
                        break;
                case CHIP_NUM_57811_MF:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57811_MF");
                        num_paths = 1;
                        break;
                case CHIP_NUM_57811_VF:
                        sprintf(BR_DEV_INFO(index)->pci_info.asic_rev, "%s", "57811_VF");
                        num_paths = 1;
                        break;
                default:
                        printf("dbg_dev_get_board_type-Unknown chip number: 0x%x\n", val);
                        return DBG_STATUS_FAILURE;
                }  
                
                dbg_set_num_paths(num_paths);                    
        }
        
        return DBG_STATUS_SUCCESS;        
}

dbg_status_t dbg_dev_get_revision_type(u32_t  index)
{
        u32_t val, offset = 0, silent_rev;
		char *rev_id_ctrl;

        dbg_select_dev_num(index);

        if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
                return DBG_STATUS_SUCCESS;
        }

        if ((tcl_al_get_no_pci_mode() == 1) || (!XDBG_CUR_DEV_IS_HIDDEN)) {
                dbg_reg_read(MISC_REG_CHIP_REV, &val);

                if (CHIP_REV_IS_ASIC(val << 12)) {

                        switch (CHIP_REV(val << 12)) {   
                        case CHIP_REV_Ax:
                                offset = sprintf(BR_DEV_INFO(index)->pci_info.revision, "%s", "A");
                                break;
                        case CHIP_REV_Bx:
                                offset = sprintf(BR_DEV_INFO(index)->pci_info.revision, "%s", "B");
                                break;
                        case CHIP_REV_Cx:
                                offset = sprintf(BR_DEV_INFO(index)->pci_info.revision, "%s", "C");
                                break;
                        default:
                                printf("dbg_dev_get_revision_type-Unknown chip number: 0x%x\n", val);
                                return DBG_STATUS_FAILURE;
                        }
                }
                else if (CHIP_REV_IS_FPGA(val << 12)) {
                        offset = sprintf(BR_DEV_INFO(index)->pci_info.revision, "%s", "FPGA");
                }
                else if (CHIP_REV_IS_EMUL(val << 12)) {
                        offset = sprintf(BR_DEV_INFO(index)->pci_info.revision, "%s", "EMUL");
                }
                else
                {
                        printf("dbg_dev_get_revision_type-Unknown chip number: 0x%x\n", val);
                        return DBG_STATUS_FAILURE;
                }

                dbg_reg_read(MISC_REG_CHIP_METAL, &val);
                sprintf(BR_DEV_INFO(index)->pci_info.revision + offset, "%d", val); 

				dbg_reg_read(MISC_REG_CHIP_TEST_REG, &val);
				silent_rev = val & 0xff;
				if (silent_rev) {
					/* Support E3 57840 B1 */
					if ((BR_DEV_INFO(index)->pci_info.device_id == CHIP_NUM_57840_OBSOLETE) ||
						(BR_DEV_INFO(index)->pci_info.device_id == CHIP_NUM_57840_4_10) ||
						(BR_DEV_INFO(index)->pci_info.device_id == CHIP_NUM_57840_2_20) ||
						(BR_DEV_INFO(index)->pci_info.device_id == CHIP_NUM_57840_MF) ||
						(BR_DEV_INFO(index)->pci_info.device_id == CHIP_NUM_57840_VF)) {
						sprintf(BR_DEV_INFO(index)->pci_info.revision + offset, "%d", silent_rev);
					}
				}
        }
	
        return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_dev_get_bc_ver(u32_t  index)
{
        u32_t val, offset;

        dbg_select_dev_num(index);

        if ((tcl_al_get_no_pci_mode() == 1) || (!XDBG_CUR_DEV_IS_HIDDEN)) {
                offset = OFFSETOF(shmem_region_t, dev_info.bc_rev);               
                SHMEM_READ(index, offset, &val);
                sprintf(BR_DEV_INFO(index)->pci_info.bc_ver, "%d.%d.%d.%d", 
						(val & 0xFF000000) >> 24,
						(val & 0x00FF0000) >> 16,
						(val & 0x0000FF00) >> 8,
						(val & 0x000000FF));

                 /* check if bc_ver is 0xdeaddead*/
                if (0 == strcmp(BR_DEV_INFO(index)->pci_info.bc_ver, "222.173.222.173")) {
                        strcpy(BR_DEV_INFO(index)->pci_info.bc_ver, "0xdeaddead");
                }
        }

        return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_dev_get_feature_support(u32_t  index)
{
        u32_t val, mba_config, mba_features, offset;
        u32_t mfw_enable, pxe_enable, wol_enable, port_id;

        dbg_select_dev_num(index);

        if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
                return DBG_STATUS_SUCCESS;
        }

        if (!XDBG_CUR_DEV_IS_HIDDEN) {
                offset = OFFSETOF(shmem_region_t, dev_info.port_feature_config[BR_DEV_PORT_ID(index)].config);               
                SHMEM_READ(index, offset, &val);
                mba_config = (val & PORT_FEATURE_EN_SIZE_MASK) >> PORT_FEATURE_EN_SIZE_SHIFT;
                
                offset = OFFSETOF(shmem_region_t,dev_info.port_feature_config[BR_DEV_PORT_ID(index)].mba_config);                               
                SHMEM_READ(index, offset, &val);
                mba_features = (val & PORT_FEATURE_MBA_BOOT_AGENT_TYPE_MASK) >> PORT_FEATURE_MBA_BOOT_AGENT_TYPE_SHIFT;

                 /* MFW enable */
                BR_DEV_INFO(index)->pci_info.mfw_enable = mba_config & (PORT_FEATURE_MFW_ENABLED >> PORT_FEATURE_EN_SIZE_SHIFT) ? 1 : 0;
                /* MBA enable */
                BR_DEV_INFO(index)->pci_info.pxe_enable = mba_config & (PORT_FEATURE_MBA_ENABLED >> PORT_FEATURE_EN_SIZE_SHIFT) ? 1 : 0;
                /* WoL enable */
                BR_DEV_INFO(index)->pci_info.wol_enable = mba_config & (PORT_FEATURE_WOL_ENABLED >> PORT_FEATURE_EN_SIZE_SHIFT) ? 1 : 0;

                /* MBA boot agent */
                if (!(BR_DEV_INFO(index)->pci_info.pxe_enable)) {                        
                        BR_DEV_INFO(index)->pci_info.pxe_agent = 0;
                } else {
                        switch (mba_features ) {
                        case PORT_FEATURE_MBA_BOOT_AGENT_TYPE_PXE:			
                                BR_DEV_INFO(index)->pci_info.pxe_agent = 'p';
                                break;
                        case PORT_FEATURE_MBA_BOOT_AGENT_TYPE_RPL:			
                                BR_DEV_INFO(index)->pci_info.pxe_agent = 'r';
                                break;
                        case PORT_FEATURE_MBA_BOOT_AGENT_TYPE_BOOTP:			
                                BR_DEV_INFO(index)->pci_info.pxe_agent = 'b';
                                break;
                        case PORT_FEATURE_MBA_BOOT_AGENT_TYPE_ISCSIB:			
                                BR_DEV_INFO(index)->pci_info.pxe_agent = 'i';
                                break;
                        case PORT_FEATURE_MBA_BOOT_AGENT_TYPE_FCOE_BOOT:			
                                BR_DEV_INFO(index)->pci_info.pxe_agent = 'f';
                                break;
                        case PORT_FEATURE_MBA_BOOT_AGENT_TYPE_NONE:			
                                BR_DEV_INFO(index)->pci_info.pxe_agent = 'n';
                                break;
                        default:			
                            BR_DEV_INFO(index)->pci_info.pxe_agent= 0;
                        }
                }

                offset = OFFSETOF(shmem_region_t,dev_info.port_feature_config[BR_DEV_PORT_ID(index)].link_config);               
                SHMEM_READ(index, offset, &val);
              
                switch (val & PORT_FEATURE_LINK_SPEED_MASK ) {
                case PORT_FEATURE_LINK_SPEED_10M_FULL:
                    sprintf(BR_DEV_INFO(index)->pci_info.pxe_speed, "10FD");
                    break;
                case PORT_FEATURE_LINK_SPEED_10M_HALF:
                    sprintf(BR_DEV_INFO(index)->pci_info.pxe_speed, "10HD");
                    break;
                case PORT_FEATURE_LINK_SPEED_100M_HALF:
                    sprintf(BR_DEV_INFO(index)->pci_info.pxe_speed, "100HD");
                    break;
                case PORT_FEATURE_LINK_SPEED_100M_FULL:
                    sprintf(BR_DEV_INFO(index)->pci_info.pxe_speed, "100FD");
                    break;
                case PORT_FEATURE_LINK_SPEED_1G:
                    sprintf(BR_DEV_INFO(index)->pci_info.pxe_speed, "1GFD");
                    break;
                case PORT_FEATURE_LINK_SPEED_2_5G:
                    sprintf(BR_DEV_INFO(index)->pci_info.pxe_speed, "2.5GFD");
                    break;
                case PORT_FEATURE_LINK_SPEED_10G_CX4:
                    sprintf(BR_DEV_INFO(index)->pci_info.pxe_speed, "10G");
                    break;             
                case PORT_FEATURE_LINK_SPEED_AUTO:
                    sprintf(BR_DEV_INFO(index)->pci_info.pxe_speed, "auto");
                    break;
                default:
                    sprintf(BR_DEV_INFO(index)->pci_info.pxe_speed, "unknown");
                    break;
                }    

        }
	
        return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_dev_get_serial_number(u32_t  index)
{
        u32_t isZero, i, val, mba_config, mba_features;
        u32_t mfw_enable, pxe_enable, wol_enable;

        
        dbg_select_dev_num(index);        

        if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
                return DBG_STATUS_SUCCESS;
        }

        if (!XDBG_CUR_DEV_IS_HIDDEN) {
                SHMEM_READ(index, SERIAL_NUMBER_OFFSET, &val);
                BR_DEV_INFO(index)->pci_info.serial_number[0] = (u8_t) val;
                BR_DEV_INFO(index)->pci_info.serial_number[1] = (u8_t) (val >> 8);
                BR_DEV_INFO(index)->pci_info.serial_number[2] = (u8_t) (val >> 16);
                BR_DEV_INFO(index)->pci_info.serial_number[3] = (u8_t) (val >> 24);
        
                SHMEM_READ(index, SERIAL_NUMBER_OFFSET+4, &val);
                BR_DEV_INFO(index)->pci_info.serial_number[4] = (u8_t) val;
                BR_DEV_INFO(index)->pci_info.serial_number[5] = (u8_t) (val >> 8);
                BR_DEV_INFO(index)->pci_info.serial_number[6] = (u8_t) (val >> 16);
                BR_DEV_INFO(index)->pci_info.serial_number[7] = (u8_t) (val >> 24);
        
                SHMEM_READ(index, SERIAL_NUMBER_OFFSET+8, &val);
                BR_DEV_INFO(index)->pci_info.serial_number[8] = (u8_t) val;
                BR_DEV_INFO(index)->pci_info.serial_number[9] = (u8_t) (val >> 8);
                BR_DEV_INFO(index)->pci_info.serial_number[10] = (u8_t) (val >> 16);
                BR_DEV_INFO(index)->pci_info.serial_number[11] = (u8_t) (val >> 24);
        
                SHMEM_READ(index, SERIAL_NUMBER_OFFSET+12, &val);
                BR_DEV_INFO(index)->pci_info.serial_number[12] = (u8_t) val;
                BR_DEV_INFO(index)->pci_info.serial_number[13] = (u8_t) (val >> 8);
                BR_DEV_INFO(index)->pci_info.serial_number[14] = (u8_t) (val >> 16);
                BR_DEV_INFO(index)->pci_info.serial_number[15] = (u8_t) (val >> 24);
        
                BR_DEV_INFO(index)->pci_info.serial_number[16]='\0';

                /* check if serial number is zero (mainly due to wrong SHMEM)
                   If so, write "invalid" in serial number */
                for (i = 0; i < 16; i++) {
                        if (BR_DEV_INFO(index)->pci_info.serial_number[i])
                                break;
                }
                if (i == 16) {
                        sprintf(BR_DEV_INFO(index)->pci_info.serial_number, "invalid");
                }
        }
	
        return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_dev_get_mac_address(u32_t index, u32_t *is_bad_mf_mac)
{
        u32_t val, offset, mf_mode, nvm_addr, extended_dir_size_in_nvm, type;
        u32_t mac0=0, mac1=0, mac2=0, mac3=0, mac4=0, mac5=0, code_start;
        u32 mac[2];
        extended_dir_image_t extended_dir_in_nvm;
        
        *is_bad_mf_mac = 0;

        dbg_select_dev_num(index);

        if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
                common_nvm_read_nvram(NVM_OFFSET(feature_info.shared_feature_config.config), &mf_mode, sizeof(mf_mode));

                switch (mf_mode & SHARED_FEAT_CFG_FORCE_SF_MODE_MASK) {
                case SHARED_FEAT_CFG_FORCE_SF_MODE_MF_ALLOWED:
                case SHARED_FEAT_CFG_FORCE_SF_MODE_FORCED_SF:
                case SHARED_FEAT_CFG_FORCE_SF_MODE_SPIO4:
                        offset = NVM_OFFSET_PATH(manuf_info.port_hw_config[BR_DEV_INFO(index)->pci_info.port_id].mac_upper,
                                                 BR_DEV_INFO(index)->pci_info.path_num);
                        common_nvm_read_nvram(offset, (u32_t *)mac, 8);
                        mac0 = (mac[0] & 0x0000FF00) >> 8;
                        mac1 = (mac[0] & 0x000000FF);
                        mac2 = (mac[1] & 0xFF000000) >> 24;
                        mac3 = (mac[1] & 0x00FF0000) >> 16;
                        mac4 = (mac[1] & 0x0000FF00) >> 8;
                        mac5 = (mac[1] & 0x000000FF);
                        break;
                case SHARED_FEAT_CFG_FORCE_SF_MODE_SWITCH_INDEPT:
                case SHARED_FEAT_CFG_FORCE_SF_MODE_UFP_MODE:
                        // Find MAC from NPAR image
                        if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &extended_dir_size_in_nvm, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS) {                    // Parse the extended DIR
                                if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
                                        if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, extended_dir_size_in_nvm, &extended_dir_in_nvm) == COMMON_NVM_STATUS_SUCCESS ) {        
                                                if (common_nvm_nvm_find_extended_dir_entry(CODE_IMAGE_TYPE_NIC_PARTITION_CFG, &extended_dir_in_nvm,
                                                                                           NULL, &code_start, NULL, NULL, NULL) == COMMON_NVM_STATUS_SUCCESS) {
                                                        common_nvm_read_nvram(code_start + OFFSETOF(nvm_nic_part_cfg_t, func_cfg[index].net_mac_addr_upper), (u32_t *)mac, 8);
                                                        mac0 = (mac[0] & 0x0000FF00) >> 8;
                                                        mac1 = (mac[0] & 0x000000FF);
                                                        mac2 = (mac[1] & 0xFF000000) >> 24;
                                                        mac3 = (mac[1] & 0x00FF0000) >> 16;
                                                        mac4 = (mac[1] & 0x0000FF00) >> 8;
                                                        mac5 = (mac[1] & 0x000000FF);
                                                }
                                        }                                       
                                 }
                        }
                        break;
              
                }
        } else {

                if ((tcl_al_get_no_pci_mode() == 1) && XDBG_CUR_DEV_IS_HIDDEN) {
                        return DBG_STATUS_SUCCESS;
                }

                /* if mf_mode - read MACs from mf_cfg, otherwise read from port_hw_config */
                offset = OFFSETOF(shmem_region_t, dev_info.shared_feature_config.config);
                SHMEM_READ(index, offset, &val);
                switch (val & SHARED_FEAT_CFG_FORCE_SF_MODE_MASK) {                       
                case SHARED_FEAT_CFG_FORCE_SF_MODE_SWITCH_INDEPT:
                case SHARED_FEAT_CFG_FORCE_SF_MODE_MF_ALLOWED:
                case SHARED_FEAT_CFG_FORCE_SF_MODE_UFP_MODE:  
                        offset = OFFSETOF(mf_cfg_t, func_mf_config[index].mac_upper);
                        mf_cfg_read(offset, &val);
                        mac0 = (val & 0x0000FF00) >> 8;
                        mac1 = (val & 0x000000FF);

                        offset = OFFSETOF(mf_cfg_t, func_mf_config[index].mac_lower);
                        mf_cfg_read(offset, &val);
                        mac2 = (val & 0xFF000000) >> 24;
                        mac3 = (val & 0x00FF0000) >> 16;
                        mac4 = (val & 0x0000FF00) >> 8;
                        mac5 = (val & 0x000000FF);
                        
                        /* if MAC read is all 1s, read MAC from port_hw_config and mark this port is in sf_mode */                
                        if ((val != 0xFFFFFFFF) || (mac0 != 0xFF) || (mac1 != 0xFF)) {                        
                                break;                        
                        } else {                                      
                                *is_bad_mf_mac = 1;
                        }
                case SHARED_FEAT_CFG_FORCE_SF_MODE_FORCED_SF:
                        offset = OFFSETOF(shmem_region_t, dev_info.port_hw_config[BR_DEV_PORT_ID(index)].mac_upper);
                        SHMEM_READ(index, offset, &val);                
                        mac0 = (val & 0x0000FF00) >> 8;
                        mac1 = (val & 0x000000FF);

                        offset = OFFSETOF(shmem_region_t, dev_info.port_hw_config[BR_DEV_PORT_ID(index)].mac_lower);               
                        SHMEM_READ(index, offset, &val);
                        mac2 = (val & 0xFF000000) >> 24;
                        mac3 = (val & 0x00FF0000) >> 16;
                        mac4 = (val & 0x0000FF00) >> 8;
                        mac5 = (val & 0x000000FF);
                        break;
                default:
                        printf("unknown mf mode: unmasked 0x%x masked 0x%x \n",  val, val & SHARED_FEAT_CFG_FORCE_SF_MODE_MASK);
                        break;      
                }    
        }

        sprintf(BR_DEV_INFO(index)->pci_info.mac_addr, "%02x:%02x:%02x:%02x:%02x:%02x", mac0, mac1, mac2, mac3, mac4, mac5);

        return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_is_4port(int	*result)
{
	u32_t val32;
	dbg_dev_info_t *cur_dev_p = XDBG_CUR_DEV;

        if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
                if (virtual_nvm.port_mode == PORT_MODE_4_PORTS) {
                        *result = 1;
                } else {
                        *result = 0;
                }

                return DBG_STATUS_SUCCESS;
        }

	if (is_e1x(globalInterp)) {
		*result = 0;
		return DBG_STATUS_SUCCESS;
	}
	
	if (dbg_reg_read(MISC_REG_PORT4MODE_EN_OVWR, &val32) != DBG_STATUS_SUCCESS)
		return DBG_STATUS_FAILURE;	

	if ((val32 & 0x1) == 0)
	{
		if (dbg_reg_read(MISC_REG_PORT4MODE_EN, &val32) != DBG_STATUS_SUCCESS)
			return DBG_STATUS_FAILURE;		
        *result = (val32 & 0x1);

	} else {
        *result = (val32 >> 1) & 0x1;
    }
	
	return DBG_STATUS_SUCCESS;
}

/*
*------------------------------------------------------------------------
* dg_number_of_everest_functions --
*
* Result:
*     number of functions (1 or 2)
*
* please note:
* ***********
*  this function assumes that all everest functions in this machine of the same device, are following device ids. 
*  therefore,  the second everest function should be 1 following or 1 previous to the current device.
*  if this assumption is not current,  than also function dg_select_other_func should be corrected.
*------------------------------------------------------------------------
*/
u32_t dbg_number_of_everest_functions(void){

	u32_t bus, dev;
	int is_4port = 0;
	bus =  dbg_pci_devices[(int_ptr_t)xdbg_cur_dev].pci_info.pci_bus;
	dev =  dbg_pci_devices[(int_ptr_t)xdbg_cur_dev].pci_info.pci_device;

	/* Added check for E2 - 4 port mode. */
	if (dbg_is_4port(&is_4port) == DBG_STATUS_SUCCESS)
	{
		if (is_4port) return 4;
	}

	if (( dbg_pci_devices[(int_ptr_t)xdbg_cur_dev+1].pci_info.pci_bus == bus )&&
		( dbg_pci_devices[(int_ptr_t)xdbg_cur_dev+1].pci_info.pci_device == dev )
		)
		return 2;

	if (( dbg_pci_devices[(int_ptr_t)xdbg_cur_dev-1].pci_info.pci_bus == bus )&&
		( dbg_pci_devices[(int_ptr_t)xdbg_cur_dev-1].pci_info.pci_device == dev )
		)
		return 2;

	return 1;
}

void dbg_get_info (
    OUT    pci_info_t * dev_stat_p
    )
{
        memcpy(dev_stat_p, &(XDBG_CUR_DEV->pci_info), sizeof(pci_info_t));    
}

/**
* Description:
*      Returns the port number.
* 
* Returns:
*      DIAG_STATUS_SUCCESS on success.
*	   result contains the check result.
*/ 
dbg_status_t dbg_get_port_num(u32_t *result)
{
	dbg_dev_info_t *cur_dev_p = XDBG_CUR_DEV;
	u32_t pcie_func = DBG_FUNC_NUM(*cur_dev_p);
	if (!is_e1x(globalInterp)) {
		int port4 = 0;
		dbg_status_t tmp_result = dbg_is_4port(&port4);
		if (tmp_result != DBG_STATUS_SUCCESS)
			return tmp_result;
		*result = port4 ? ((pcie_func >> 1) & 1) : 0;
		return DBG_STATUS_SUCCESS;
	}

	else // not E2
	{
		*result = (pcie_func & 1);
		return DBG_STATUS_SUCCESS;
	}

}


/**
* Description:
*      Returns the function number.
* 
* Returns:
*      DIAG_STATUS_SUCCESS on success.
*	   result contains the result.
*/ 
dbg_status_t dbg_get_func_num(u32_t *result)
{
	dbg_dev_info_t *cur_dev_p = XDBG_CUR_DEV;        
     
        *result = DBG_FUNC_NUM(*cur_dev_p);        
        	
	return DBG_STATUS_SUCCESS;
}

/**
* Description:
*      Returns the path number.
* 
* Returns:
*      DIAG_STATUS_SUCCESS on success.
*	   result contains the result.
*/ 
dbg_status_t dbg_get_path_num(u32_t *result)
{
	dbg_dev_info_t *cur_dev_p = XDBG_CUR_DEV;        
                  		
	*result = DBG_PATH_NUM(*cur_dev_p);	
        	
	return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_select_dev(dbg_dev_hdl_t dev) 
{
	// Error if requested dev is larger than total number of pci devices found in system
    if ( (int_ptr_t)dev >= dbg_get_total_dev() ) {
        return DBG_STATUS_FAILURE;
    } else {        
        xdbg_cur_dev = dev;
		// Set cur_dut_addr bus and device & funcion to the address of the currently selected device 
		dbg_pcicfg_setdut(XDBG_CUR_DEV->pci_info.pci_domain,
						  XDBG_CUR_DEV->pci_info.pci_bus,
						  XDBG_CUR_DEV->pci_info.pci_device,
                          XDBG_CUR_DEV->pci_info.pci_func);
    }
    return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_select_other_func(void)
{
    int pci_arr_idx = (int_ptr_t)xdbg_cur_dev; /* Index of the current device */
    dbg_dev_info_t * dev_info_p = XDBG_CUR_DEV;
    u32_t domain, func, bus, dev, search_func,mod_func;
    pci_info_t * pci_info_p;
	int is_4port = 0;
	dbg_status_t stat = dbg_is_4port(&is_4port);
	int change;
	if (stat != DBG_STATUS_SUCCESS)
		return stat;

    /* Cur. device PCI address */
    domain = dev_info_p->pci_info.pci_domain;
    bus = dev_info_p->pci_info.pci_bus;
    dev = dev_info_p->pci_info.pci_device;
    func = dev_info_p->pci_info.pci_func;

    /* Other function should have the same bus and device. 
       The function number should be (func + 1) mod 2 */
    search_func = (func + 1) % 2;

    /* Calculate the array index of the "other" function */

	if (is_4port)
	{
		mod_func = func % 4;
		if (mod_func < 3)
			change = 1; // valid functions in this mode are 0-3
		else // we're in func 3 (or 7), and we need to go to func 0 (or 4)
			change = -3;
	}
	else 
	{	// not 4 port
		change = (int)1 - 2*func; 
	}

    pci_arr_idx += change;

    pci_info_p = &(dbg_pci_devices[pci_arr_idx].pci_info);	
  
    /* Select the device.  */	
    return dbg_select_dev((dbg_dev_hdl_t)((int_ptr_t)pci_arr_idx));
}

void dbg_unselect_dev(void)
{
    xdbg_cur_dev = BAD_DEVICE_HNDL;
}     
     

