#include <stdlib.h>
#include <string.h>
#include "dbg_man.h"
#include "man_types.h"
#include "edebug_types.h"
#include "bcmtype.h"
#include "dbg_man.h"
#include "os_types.h"
#include "dbg_os_utils.h"
#include "tcl_al.h"

static s32_t total_br_dev = -1;
static u32_t num_paths = (u32_t)-1;
static u32_t num_ports_per_path = (u32_t)-1;

extern edebug_eng_mode_t eng_mode;
extern virtual_nvm_t virtual_nvm;

void dbg_set_total_br_dev(u32_t no_br_dev);

dbg_dev_hdl_t xdbg_cur_dev = BAD_DEVICE_HNDL;  /* Currently selected device */

dbg_dev_info_t *dbg_pci_devices;

/* Array of indices of supported QLogic devices in pci_devices array */
dbg_dev_hdl_t *br_devices; 

dbg_status_t init_global_arrays()
{ 
        u32_t i;
	u32_t no_total_dev=0;

	/* First scan the PCI to get the total_dev */
	no_total_dev = dbg_get_total_dev();	

	/* Allocate memory for dbg_pci_devices - all pci devices found in system  */
	dbg_pci_devices = (dbg_dev_info_t *) (malloc( (no_total_dev+1) * sizeof(dbg_dev_info_t)));
	if ( dbg_pci_devices == NULL ) {
		   return DBG_STATUS_RESOURCE;
	}
	/* Clear the memory allocated for dbg_pci_devices */
	memset(dbg_pci_devices, 0, (no_total_dev + 1) * sizeof(dbg_dev_info_t));
	 
	/* Allocate memory for br_devices - all QLogic devices found in system */
	br_devices = (dbg_dev_hdl_t *) (malloc( no_total_dev * sizeof(dbg_dev_hdl_t)));
	if ( br_devices == NULL ) {
		free((char *)(dbg_pci_devices));
		return DBG_STATUS_RESOURCE;
	}
	/* Clear the memory allocated for br_devices */
	memset(br_devices, 0, no_total_dev * sizeof(dbg_dev_hdl_t));
  
	/* Clear the array of QLogic devices by setting index to total_dev */
	for ( i = 0; i < no_total_dev; i++ ) {
                CLEAR_BR_INFO(i, no_total_dev);
	}
	
        return DBG_STATUS_SUCCESS;
}

u32_t dbg_get_total_dev(void)
{
	u32_t no_total_dev, num_paths, num_ports_per_path, num_devices, num_devices_not_hidden;

	if ((tcl_al_get_no_pci_mode()) || (eng_mode == EDEBUG_VIRTUAL_NVM_MODE)) {
		dbg_non_pci_get_num_of_devices(&num_paths, &num_ports_per_path, &num_devices, &num_devices_not_hidden, NULL);
		dbg_set_total_br_dev(num_devices);
		return num_devices;
	}
	else {
		dbg_os_get_total_dev(&no_total_dev);
		return no_total_dev;
	}
}

u32_t dbg_get_total_br_dev(void)
{	
	/* if total_br_dev is not already known (== -1) then need to scan the PCI */
	if (total_br_dev == -1) {
		/* if we are in non PCI mode we will not reach here
		   because total_br_dev will be set by dbg_get_total_dev */
		dbg_pci_scan();
	}
	
	return total_br_dev;
}

void dbg_set_num_paths(u32_t in_num_paths)
{
	num_paths = in_num_paths;
}

u32_t dbg_get_num_paths(void)
{	
	return num_paths;
}

void dbg_set_num_ports_per_path(u32_t in_num_ports_per_path)
{
	num_ports_per_path = in_num_ports_per_path;
}

u32_t dbg_get_num_ports_per_path(void)
{	
	return num_ports_per_path;
}

void dbg_set_total_br_dev(u32_t no_br_dev)
{
	total_br_dev = no_br_dev;
}

dbg_status_t dbg_cleanup(
    void
    )
{
	u32_t i;
	dbg_status_t res = DBG_STATUS_SUCCESS;
	u32_t no_total_br_dev=0;

	no_total_br_dev = dbg_get_total_br_dev();


	/* Clean up br_devices and pci_devices arrays */
	for ( i = 0; i < no_total_br_dev; i++ ) {
		if (dbg_cleanup_one_br_device(i) != DBG_STATUS_SUCCESS ) {
			res = DBG_STATUS_FAILURE;
		}
	}

	/* Free allocated memories for pci devices and QLogic's pci devices arrays */
	free((char *)(dbg_pci_devices));
	free((char *)(br_devices));

	if ((eng_mode == EDEBUG_VIRTUAL_NVM_MODE) && (virtual_nvm.fd)) {
		fclose(virtual_nvm.fd);
	}	

	return res;
}

dbg_status_t dbg_cleanup_one_device(
	dbg_dev_info_t * dev_info
	)
{
	dbg_status_t res = DBG_STATUS_SUCCESS;


	if (dev_info->drv_dev_if_p != NULL) {
		if (dev_info->drv_dev_if_p->dev_if_cleanup != NULL) {
			if (dev_info->drv_dev_if_p->dev_if_cleanup((struct _dev_if_t *)(dev_info->drv_dev_if_p)) != DBG_STATUS_SUCCESS) {
				res = DBG_STATUS_FAILURE;
			} 
		}
         
		free((char *)(dev_info->drv_dev_if_p));
	
		dev_info->drv_dev_if_p = NULL;
	}

	return res;
}

dbg_status_t dbg_cleanup_one_br_device (
    IN   u32_t   dev_idx    /* device index */
    )
{
	dbg_status_t res = DBG_STATUS_SUCCESS;
	dbg_dev_info_t * cur_info = BR_DEV_INFO(dev_idx);
	u32_t no_total_dev = dbg_get_total_dev();

	/* clear device by setting its index to total_dev */
	CLEAR_BR_INFO(dev_idx, no_total_dev);

	if (!cur_info->hidden) {
		res = dbg_cleanup_one_device(cur_info);
	}

	return res;
}
