#include "man_types.h"
#include "edebug_types.h"
#include "bcmtype.h"
#include "57712_reg.h"
#include "license.h"
#include "mcp_shmem.h"
#include "util.h"
#include "grc_addr.h"
#include "misc_bits.h"

/*
 *-------------------------------------------------------------------------------
 * dg_reset_mcp --
 *
 *	Reset MCP and wait for MCP to come up by checkeing the share memory validity
 *-------------------------------------------------------------------------------
 */
dbg_status_t dbg_reset_mcp(void)
{

	u32_t shmem;
	u32_t val=0;
	u32_t cnt=0;
	u32_t validity_offset;
	u8 is_slow_device = 0;

	/* Get shmem offset */
	dbg_reg_read(MISC_REG_SHARED_MEM_ADDR, &shmem);
	validity_offset = OFFSETOF(shmem_region_t, validity_map[0]);

	/* Clear validity map flags */
	if( shmem > 0 ) {
		dbg_reg_write(shmem + validity_offset, 0, 1);    //wait_for_echo is set to 1
	}


	 /* Reset the MCP */
	dbg_reg_dwrite(GRCBASE_MISC + MISC_REGISTERS_RESET_REG_2_CLEAR,
                  MISC_REGISTERS_RESET_REG_2_RST_MCP_N_RESET_REG_HARD_CORE  |
                  MISC_REGISTERS_RESET_REG_2_RST_MCP_N_HARD_CORE_RST_B      |
                  MISC_REGISTERS_RESET_REG_2_RST_MCP_N_RESET_CMN_CPU        |
                  MISC_REGISTERS_RESET_REG_2_RST_MCP_N_RESET_CMN_CORE       );
	dbg_msleep(100);

	/* Get shmem offset */
	dbg_reg_read(MISC_REG_SHARED_MEM_ADDR, &shmem);
	if( shmem == 0 ) {
		return DBG_STATUS_FAILURE;
	}

	/* Wait for MCP to come up */
	for(cnt = 0; cnt < (10 + is_slow_device*300); cnt++)
	{
		// TBD: its best to check validity map of last port. currently checks on port 0.
		dbg_reg_read(shmem + validity_offset, &val );

		// check that shared memory is valid.
		if((val & (SHR_MEM_VALIDITY_DEV_INFO | SHR_MEM_VALIDITY_MB)) == (SHR_MEM_VALIDITY_DEV_INFO|SHR_MEM_VALIDITY_MB))
			break;

		dbg_msleep(100);
	}

	// Check that shared memory is valid. This indicates that MCP is up.
	if((val & (SHR_MEM_VALIDITY_DEV_INFO | SHR_MEM_VALIDITY_MB)) != (SHR_MEM_VALIDITY_DEV_INFO|SHR_MEM_VALIDITY_MB))
	{
		return DBG_STATUS_FAILURE;
	}

	return DBG_STATUS_SUCCESS;
}
