#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include "tcl.h"
#include "bcmtype.h"
#include "edebug_types.h"
#include "dbg_man.h"
#include "dbg_vpd.h"
#include "common_nvm.h"
#include "hw_access.h"
#include "dbg_serport.h"
#include "os_utils.h"
#include "tcl_al.h"

extern edebug_eng_mode_t eng_mode;

void common_nvm_generate_hex_int_object(Tcl_Obj **obj_ptr, u32_t val)
{
        char tmp_str[256];        
        sprintf(tmp_str, "0x%x", val);
	*obj_ptr = Tcl_NewStringObj(tmp_str, strlen(tmp_str));
}

void common_nvm_set_hex_int_object(Tcl_Obj **obj_ptr, u32_t val)
{
        char tmp_str[256];        
        sprintf(tmp_str, "0x%x", val);
	Tcl_SetStringObj(*obj_ptr, tmp_str, strlen(tmp_str));
}

u32_t common_nvm_get_total_br_dev(void)
{
        return dbg_get_total_br_dev();
}

void *common_nvm_memset(void  *buf, u8_t ch, u32_t buf_size)
{
        return memset(buf, ch, buf_size);
}

u32_t common_nvm_number_of_everest_functions(void)
{
        return dbg_number_of_everest_functions();
}

u32_t common_nvm_tcl_al_get_no_pci_mode(void)
{
        return tcl_al_get_no_pci_mode();
}

u32_t common_nvm_tcl_al_get_virtual_nvm_mode(void)
{
        return (eng_mode == EDEBUG_VIRTUAL_NVM_MODE);
}

void common_nvm_vpd_set_defaults(vpd_info_t *vpd_info, u32_t mask)
{
        dbg_vpd_set_defaults(vpd_info, mask);
}

u32_t common_nvm_set_vpd_info(vpd_info_t *vpd_info, media_vpd_t *vpd_e)
{
        return dbg_set_vpd_info(vpd_info, vpd_e);
}

common_nvm_status_t common_nvm_dev_read_nvram(IN dev_if_t *cur_dev, IN u32_t addr, OUT u32_t *val_p, IN u32_t byte_cnt)
{
        return dbg_dev_read_nvram(cur_dev, addr, val_p, byte_cnt);
}

common_nvm_status_t common_nvm_dev_write_nvram(IN dev_if_t *cur_dev, IN u32_t addr, IN u32_t *val_p, IN u32_t byte_cnt)
{
        return dbg_dev_write_nvram(cur_dev, addr, val_p, byte_cnt);
}

common_nvm_status_t common_nvm_duart_nvm_read(u8_t io_port, u32_t addr, u32_t *buf, u32_t buf_sz)
{
        return dbg_duart_nvm_read(io_port, addr, buf, buf_sz);        
}

u8_t common_nvm_get_duart_port(void)
{
        return dbg_get_duart_port();
}

common_nvm_status_t common_nvm_duart_nvm_write(IN u8_t io_port, IN u32_t addr, IN u32_t *buf, IN u32_t buf_sz)
{
        return dbg_duart_nvm_write(io_port, addr, buf, buf_sz);
}

void common_nvm_get_vpd_info(vpd_info_t *vpd_info, media_vpd_t *vpd_e, u8_t vpd_check)
{
        dbg_get_vpd_info(vpd_info, vpd_e, vpd_check);
}

void common_nvm_swap_bytes(u32_t  *buf, u32_t  buf_size)
{
        dbg_os_swap_bytes(buf, buf_size);
}

void common_nvm_get_info (OUT common_nvm_dev_stat_t * dev_stat_p)
{
        dbg_get_info(dev_stat_p);
}

u32_t common_nvm_le32_to_cpu(u32_t val)
{
        return os_if_le32_to_cpu(val);
}

common_nvm_virt_addr_t common_nvm_alloc_virt(IN u32_t sz)
{
        return (common_nvm_virt_addr_t)(uintptr_t)malloc(sz);
}

void common_nvm_free_virt(IN common_nvm_virt_addr_t virt_addr)
{
        free((char *)(uintptr_t)virt_addr);
}

u32_t common_nvm_get_func_num_from_dev_info (common_nvm_dev_info_t *p_dev_info)
{
        return p_dev_info->pci_info.pci_func;
}

char *common_nvm_get_chip_num(common_nvm_dev_stat_t *dev_stat)
{
        return dev_stat->asic_rev;
}

common_nvm_status_t common_nvm_get_path_num(u32_t *result)
{
        return dbg_get_path_num(result);
}

common_nvm_status_t common_nvm_get_port_num(u32_t *result)
{
        return dbg_get_port_num(result);
}

common_nvm_status_t common_nvm_get_func_num(u32_t *result)
{
        return dbg_get_func_num(result);
}

void *common_nvm_memcpy(IN void *dest, IN const void *src, IN u32_t bnum)
{
        return memcpy(dest, src, bnum);
}

common_nvm_status_t common_nvm_select_other_func(void)
{
        return dbg_select_other_func();
}

u32_t common_nvm_nvm_size_from_dev_info(common_nvm_dev_info_t *p_dev_info)
{
        return p_dev_info->drv_dev_if_p->os_params.geeprom_length;
}

common_nvm_status_t common_nvm_reg_read(IN u32_t addr, OUT u32_t *val32_p)
{
        return dbg_reg_read(addr, val32_p);
}

common_nvm_status_t common_nvm_reg_write(IN u32_t addr, IN u32_t val32)
{
        return dbg_reg_write(addr, val32, 0);
}

void DbgMessageXX(void *dummy, int level, char *msg, ...)
{
         if (level == FATAL) {
            va_list args;
            char new_msg[200];
            va_start(args, msg);
            vsprintf(new_msg, msg, args);
            printf(new_msg);        
    }
}

common_nvm_status_t tcl_al_log_command(IN char* opString, IN int objc, IN Tcl_Obj* CONST objv[])
{
    /* not really implemented, dummy */
    return DBG_STATUS_SUCCESS;
}

void print_msg(IN char *msg, IN u32_t color)
{
    printf(msg);
}

u32_t common_nvm_duart_get_nvm_size(void)
{
        return dbg_duart_get_nvm_size();
}

common_nvm_status_t common_nvm_duart_get_chip_id(u32_t *chip_id_p)
{
        return dbg_duart_get_chip_id(chip_id_p);
}



