#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "57712_reg.h"
#include "edebug_types.h"
#include "bcmtype.h"
#include "man_types.h"
#include "hw_access.h"
#include "license.h"
#include "mcp_shmem.h"
#include "pcics_reg_driver.h"
#include "util.h"
#include "tcl_al.h"
#include "nvm_map.h"
#include "common_nvm.h"

#define MAX_READ_RETRIES 5
#define WAIT_MS          500

extern edebug_eng_mode_t eng_mode;
extern virtual_nvm_t virtual_nvm;

static dbg_status_t non_pci_is_mf_mode(u32_t *is_mf_mode)
{
        u32_t shmem_base, offset, mf_mode, i, value;
        dbg_status_t status;

        if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
                common_nvm_read_nvram(NVM_OFFSET(feature_info.shared_feature_config.config), &mf_mode, sizeof(mf_mode));
        } else {
                for (i=0; i<MAX_READ_RETRIES; i++) {
                    status=dbg_reg_read(MISC_REG_SHARED_MEM_ADDR, &shmem_base);
                    if (status==DBG_STATUS_SUCCESS) break;
                    dbg_msleep(WAIT_MS);
                }
                if (status!=DBG_STATUS_SUCCESS) {
                    return DBG_STATUS_FAILURE;
                }

                offset = OFFSETOF(shmem_region_t, dev_info.shared_feature_config.config);

                for (i=0; i<MAX_READ_RETRIES; i++) {
                    status=dbg_reg_read(shmem_base+offset, &mf_mode);
                    if (status==DBG_STATUS_SUCCESS) break;
                    dbg_msleep(WAIT_MS);
                }
                if (status!=DBG_STATUS_SUCCESS) {
                            printf("Error - Failed to read MF mode !\n");
                    return DBG_STATUS_FAILURE;
                }
        }

        switch (mf_mode & SHARED_FEAT_CFG_FORCE_SF_MODE_MASK) {               
        case SHARED_FEAT_CFG_FORCE_SF_MODE_FORCED_SF:
                *is_mf_mode=0;
                break;                
        case SHARED_FEAT_CFG_FORCE_SF_MODE_MF_ALLOWED:
        case SHARED_FEAT_CFG_FORCE_SF_MODE_SWITCH_INDEPT:
        case SHARED_FEAT_CFG_FORCE_SF_MODE_UFP_MODE:
                *is_mf_mode=1;
                break;
        default:
                *is_mf_mode=0;
                break;
        }
        return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_non_pci_get_num_of_devices(u32_t *num_paths, u32_t *num_ports_per_path,
                                            u32_t *num_total_devices, u32_t *num_devices_not_hidden, u32_t *did)
{
        u32_t val, four_port_reg_val, shmem_base, offset, i, is_mf_mode=0;
        dbg_status_t status;

        if (eng_mode != EDEBUG_VIRTUAL_NVM_MODE) {        
                for (i=0; i<MAX_READ_RETRIES; i++) {
                    status=dbg_reg_read(MISC_REG_CHIP_NUM, &val);
                    if (status==DBG_STATUS_SUCCESS) break;
                    dbg_msleep(WAIT_MS);
                }
                if (status!=DBG_STATUS_SUCCESS) {
                    return DBG_STATUS_FAILURE;
                }
        } else {
                switch (virtual_nvm.chip_type) {
                case 	CHIP_TYPE_E1:
                        val = CHIP_NUM_5710;
                case CHIP_TYPE_E1X:
                        val = CHIP_NUM_5711;
                case CHIP_TYPE_E2:
                        val = CHIP_NUM_5712;
                case CHIP_TYPE_E3:
                        val = CHIP_NUM_57800;
                }             
        }            

        status = non_pci_is_mf_mode(&is_mf_mode);
        if (status != DBG_STATUS_SUCCESS) {
            printf("Error - Failed to check if MF mode !\n");
            return DBG_STATUS_FAILURE;
        }        

        switch (val) {
        case CHIP_NUM_5710:
        case CHIP_NUM_5711:
                *num_paths = 1;
                *num_ports_per_path = 2;
                *num_total_devices = 2;
                *num_devices_not_hidden = 2;
                break;             
        case CHIP_NUM_5711E:
                *num_paths = 1;
                *num_ports_per_path = 2;
                *num_total_devices = 8;
                if (is_mf_mode) {
                        *num_devices_not_hidden = 8;
                } else {
                        *num_devices_not_hidden = 2;
                }
                break;    
        case CHIP_NUM_5712:
        case CHIP_NUM_5712E:
        case CHIP_NUM_5713:
        case CHIP_NUM_5713E:                
                *num_paths = 2;
                *num_ports_per_path = 1;
                *num_total_devices = 8;
                if (is_mf_mode) {
                        *num_devices_not_hidden = 8;
                } else {
                        *num_devices_not_hidden = 2;
                }
                break;              
        case CHIP_NUM_57800:
        case CHIP_NUM_57810:
        case CHIP_NUM_57840_4_10:
        case CHIP_NUM_57840_2_20:
        case CHIP_NUM_57800_MF:
        case CHIP_NUM_57840_MF:
        case CHIP_NUM_57810_MF:
        case CHIP_NUM_57840_OBSOLETE:
        case CHIP_NUM_57840_MF_OBSOLETE:
                if (eng_mode != EDEBUG_VIRTUAL_NVM_MODE) {
                        for (i=0; i<MAX_READ_RETRIES; i++) {
                            status=dbg_reg_read(MISC_REG_PORT4MODE_EN_OVWR, &four_port_reg_val);
                            if (status==DBG_STATUS_SUCCESS) break;
                            dbg_msleep(WAIT_MS);
                        }
                        if (status!=DBG_STATUS_SUCCESS) {
        				    printf("Error - Failed to read 4-port mode register !\n");
                            return DBG_STATUS_FAILURE;
                        }
                        if (four_port_reg_val & 0x2) {
                                *num_ports_per_path = 2;
                        } else {
                                *num_ports_per_path = 1;
                        }
                } else {
                        if (virtual_nvm.port_mode == PORT_MODE_4_PORTS) {
                                *num_ports_per_path = 2;
                        } else {
                                *num_ports_per_path = 1;
                        }
                }

                *num_paths = 2;
                *num_total_devices = 8;               
                
                if (is_mf_mode) {
                        *num_devices_not_hidden = 8;
                } else {
                        *num_devices_not_hidden = 2*(*num_ports_per_path);
                }

                break;            
        case CHIP_NUM_57811:       
        case CHIP_NUM_57811_MF:
        case CHIP_NUM_57811_VF:
                if (eng_mode != EDEBUG_VIRTUAL_NVM_MODE) {      
                        for (i=0; i<MAX_READ_RETRIES; i++) {
                            status=dbg_reg_read(MISC_REG_PORT4MODE_EN_OVWR, &four_port_reg_val);
                            if (status==DBG_STATUS_SUCCESS) break;
                            dbg_msleep(WAIT_MS);
                        }
                        if (status!=DBG_STATUS_SUCCESS) {
        				    printf("Error - Failed to read 4-port mode register !\n");
                            return DBG_STATUS_FAILURE;
                        }
                        if (four_port_reg_val & 0x2) {
                                *num_ports_per_path = 2;
                        } else {
                                *num_ports_per_path = 1;
                        }
                } else {
                         if (virtual_nvm.port_mode == PORT_MODE_4_PORTS) {
                                *num_ports_per_path = 2;
                        } else {
                                *num_ports_per_path = 1;
                        }
                }

                *num_paths = 1;
                *num_total_devices = 4;               
                
                if (is_mf_mode) {
                        *num_devices_not_hidden = 4;
                } else {
                        *num_devices_not_hidden = 2*(*num_ports_per_path);
                }

                break;            
        default:                
                printf("dbg_non_pci_get_num_of_devices-Unknown chip number: 0x%x\n", val);
                return DBG_STATUS_FAILURE;
        }   

        if (did) {
                *did = val;
        }

        dbg_set_num_paths(*num_paths);        
                            
        return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_pci_scan(void) 
{
        pci_info_t *info_ar;
        u32_t no_total_br_dev = 0;
        u32_t no_total_dev = 0;
        u32_t i;
        dbg_status_t res;                   
        u32_t num_paths, num_ports_per_path, num_devices, num_devices_not_hidden;

        no_total_dev = dbg_get_total_dev();
        
        /* Allocate memory for info_ar */
        info_ar = (pci_info_t *) (malloc(no_total_dev * sizeof(pci_info_t)));
        if (!info_ar) {
                return DBG_STATUS_RESOURCE;
        }
        
        /* Clear the memory allocated for info_ar */
        memset(info_ar, 0, no_total_dev * sizeof(pci_info_t));

        for (i = 0 ; i < no_total_dev; i++) {
            strcpy((char *)(&(info_ar[i].mac_addr)), "XX:XX:XX:XX:XX:XX");
            strcpy((char *)(&(info_ar[i].asic_rev)), "XXXXXX");
            strcpy((char *)(&(info_ar[i].revision)), "XX");
            strcpy((char *)(&(info_ar[i].serial_number)), "XXXXXXXXXXXXXXX");	
            strcpy((char *)(&(info_ar[i].bc_ver)), "XXXXXXXX");	
            strcpy((char *)(&(info_ar[i].fw_ver)), "XXXXXXXX");	
            strcpy((char *)(&(info_ar[i].eth_name)), "XXXX");	
            strcpy((char *)(&(info_ar[i].driver_name)), "XXXXX");	
            strcpy((char *)(&(info_ar[i].driver_ver)), "XXXXXXX");	
        }
        
        /* Scan the PCI to copy pci_info into info_ar */
        res = dbg_os_pci_scan(info_ar);
        if (res != DBG_STATUS_SUCCESS) {
                return res;
        }
                
        /* Simplistic inplementation - no resize in pci_devices is supported */
        /* Update the pci_device array and create a driver instance for supported QLogic devices */
        for ( i = 0; i < no_total_dev; i++ ) {
                memcpy(&(dbg_pci_devices[i].pci_info), &(info_ar[i]) , sizeof(pci_info_t));
                dbg_pci_devices[i].present = 1;
                dbg_pci_devices[i].hidden = 0;	  	      
                // dbg_pci_devices[i].dg_drv_dev = NULL;
                  
                if ( info_ar[i].vendor_id == BROADCOM_VID ) { 
                        
                        /* Allocate the dispatch table. If fails - return DIAG_STATUS_RESOURCE, it's a fatal error */
                        dbg_pci_devices[i].drv_dev_if_p = (struct _dev_if_t *)malloc(sizeof(dev_if_t));
                        if ( dbg_pci_devices[i].drv_dev_if_p == NULL ) {
                                return DBG_STATUS_RESOURCE;
                        }
                
                        /* Remember QLogic device */
                        br_devices[no_total_br_dev] = (dbg_dev_hdl_t)((int_ptr_t)i);
                        dbg_pci_devices[i].br_idx = no_total_br_dev;                                    
                        no_total_br_dev++;
                
                        /* Initialize the driver */
                        res = dbg_dev_init_dispatch( &(dbg_pci_devices[i]));
                        if ( res != DBG_STATUS_SUCCESS) {
                                if (res == DBG_STATUS_INVALID_PARAMETER) {
                                        /* Device isn't supported. Do not include this device to "device" list and continue */                                        
                                        free((char *)(dbg_pci_devices[i].drv_dev_if_p));
                                        no_total_br_dev--;                                          
                                } else {
                                        /* Init failed, but include this device in "device" list as hidden device */                                        		
                                        free((char *)(dbg_pci_devices[i].drv_dev_if_p));
                                        dbg_pci_devices[i].hidden = 1;					
                                }
                        }  
                        /* if initializtion succeeded, perform more init operations */
                        else {
                                /* if more info can be obtained, do it */
                                if (dbg_pci_devices[i].drv_dev_if_p->dev_if_get_dev_info != NULL) {
                                        dbg_pci_devices[i].drv_dev_if_p->dev_if_get_dev_info(dbg_pci_devices[i].drv_dev_if_p->os_params, 
                                                                                            &(dbg_pci_devices[i].pci_info));
                                }
                        }
                } 
        }

        if (no_total_dev) {
            dbg_select_dev_num(0);
        }
        
        dbg_non_pci_get_num_of_devices(&num_paths, &num_ports_per_path, &num_devices, &num_devices_not_hidden, NULL);
                
        for (i = 0; i < no_total_br_dev; i++) {                
                BR_DEV_INFO(i)->pci_info.pci_func = dbg_dev_get_abs_func_num(num_paths, i);
                if (num_ports_per_path == 1) {
                        BR_DEV_INFO(i)->pci_info.port_id = 0;
                }
                else if (num_paths == 1) {
                        BR_DEV_INFO(i)->pci_info.port_id = BR_DEV_INFO(i)->pci_info.pci_func & 0x1;
                }
                else {
                        BR_DEV_INFO(i)->pci_info.port_id = ((BR_DEV_INFO(i)->pci_info.pci_func & 0x2) != 0);
                }
                
                dbg_dev_get_shmem_base(i);
                dbg_dev_get_board_type(i);
                dbg_dev_get_bc_ver(i);
                dbg_dev_get_revision_type(i);
                dbg_dev_get_feature_support(i);
                dbg_dev_get_serial_number(i);
        }     
            

        dbg_set_total_br_dev(no_total_br_dev);

        free((char *)(info_ar));
        
        return DBG_STATUS_SUCCESS;
}

dbg_status_t dbg_non_pci_scan(void) 
{
        u32_t num_paths, num_ports_per_path, num_devices, num_devices_not_hidden, i, is_bad_mf_mac, did;
        dbg_status_t res;

        res = dbg_non_pci_get_num_of_devices(&num_paths, &num_ports_per_path, &num_devices, &num_devices_not_hidden, &did);
        if (res != DBG_STATUS_SUCCESS) {
		        printf("Error - Failed to get number of devices !\n");
                return res;
        }

        printf("Scanning devices, Please wait ..\n");
       
        for (i = 0; i < num_devices; i++) {
                /* in non pci mode br_devices and dbg_pci_devices includes same devices */
                br_devices[i]=(dbg_dev_hdl_t)((int_ptr_t)i);

                BR_DEV_INFO(i)->pci_info.device_id = did;

                if (i >= num_devices_not_hidden) {
                        BR_DEV_INFO(i)->hidden = 1;                        
                }

                BR_DEV_INFO(i)->pci_info.path_num = i % num_paths;
                BR_DEV_INFO(i)->pci_info.device_num_in_path = i / (num_paths);
                BR_DEV_INFO(i)->pci_info.device_num_in_port = i / (num_paths*num_ports_per_path);
                BR_DEV_INFO(i)->pci_info.device_num = i;
                BR_DEV_INFO(i)->pci_info.pci_func = i; // for correct nvram access

                if (num_ports_per_path == 1) {
                        BR_DEV_INFO(i)->pci_info.port_id = 0;
                }
                else if (num_paths == 1) {
                        BR_DEV_INFO(i)->pci_info.port_id = i & 0x1;
                }
                else {
                        BR_DEV_INFO(i)->pci_info.port_id = ((i & 0x2) != 0);
                } 
                
                
                dbg_dev_get_shmem_base(i);        
                dbg_dev_get_board_type(i); 
                dbg_dev_get_revision_type(i);    
                dbg_dev_get_feature_support(i); 
                dbg_dev_get_serial_number(i);                         
                dbg_dev_get_mac_address(i, &is_bad_mf_mac);
                
                /* if mac is bad, then mark upper functions as hidden */                
                if (is_bad_mf_mac && (i >= num_paths*num_ports_per_path)) {
                        BR_DEV_INFO(i)->hidden = 1;
                }                
        }     

        return DBG_STATUS_SUCCESS;
}
