#include "man_types.h"
#include "dbg_man.h"
#include "dbg_os_utils.h"
#include "hw_access.h"

/* Currently set device address for "pcicfg" commands */
static struct {
    u16_t domain;
    u8_t bus;
    u8_t device;
    u8_t function;
} cur_dut_addr = {0, 0, 0};

void dbg_pcicfg_setdut(
    u16_t domain,
    u8_t  bus,
    u8_t  device,
    u8_t  function
    )
{
        cur_dut_addr.domain = domain;
        cur_dut_addr.bus = bus;
        cur_dut_addr.device = device;
        cur_dut_addr.function = function;
}

void dbg_pcicfg_getdut(
    u16_t *domain,
    u8_t *bus,
    u8_t *device,
    u8_t *function
    )
{
        *domain = cur_dut_addr.domain;
        *bus = cur_dut_addr.bus;
        *device = cur_dut_addr.device;
        *function = cur_dut_addr.function;
}

dbg_status_t dbg_pcicfg_read(
    u32_t addr,
    u32_t * ret_val_p
    )
{
        return dbg_os_pcicfg_read(cur_dut_addr.domain, cur_dut_addr.bus, cur_dut_addr.device, cur_dut_addr.function, addr, ret_val_p);
}

dbg_status_t dbg_pcicfg_process_link_control_reg(pci_info_t  * pci_info_ar, u32_t i, u32_t ret_val)
{
        u32_t val;
        /* PCIE lane number */
        sprintf(pci_info_ar[i].pcie_lane_width, "PCIE-%d", (ret_val & 0x3f00000) >> 20);

        /* PCIE speed */
        dbg_reg_read(MISC_REG_CHIP_METAL, &val);
        if (CHIP_IS_E2(pci_info_ar[i]) && (val==0)) {
                /* In E2 chip rev A0 [ERR8] pci speeds bits are '0' for 2.5G, '1' for 5G */
                switch ((ret_val & 0xf0000) >> 16) {
                case PCIE_LANE_SPEED_2_5G_E2_A0:
                        sprintf(pci_info_ar[i].pcie_lane_speed, "%s", "2.5");
                        break;
                case PCIE_LANE_SPEED_5G_E2_A0:
                        sprintf(pci_info_ar[i].pcie_lane_speed, "%s", "5.0");
                        break;
                default:
                        sprintf(pci_info_ar[i].pcie_lane_speed, "(unknown)");
                }
        } else {
                switch ((ret_val & 0xf0000) >> 16) {
                case PCIE_LANE_SPEED_2_5G:
                        sprintf(pci_info_ar[i].pcie_lane_speed, "%s", "2.5");
                        break;
                case PCIE_LANE_SPEED_5G:
                        sprintf(pci_info_ar[i].pcie_lane_speed, "%s", "5.0");
                        break;
                case PCIE_LANE_SPEED_8G:
                        sprintf(pci_info_ar[i].pcie_lane_speed, "%s", "8.0");
                        break;
                default:
                        sprintf(pci_info_ar[i].pcie_lane_speed, "(unknown)");
                }
        }
}

