#ifdef UEFI
#include <sys/types.h>
#include <machine/endian.h>
#endif

#include <stdio.h>
#include <string.h>
#include "man_types.h"
#include "edebug_types.h"
#include "dbg_vpd.h"
#ifndef FWUPGTOOL
#include "dbg_os_utils.h"
#endif // !FWUPGTOOL


void
dbg_vpd_set_defaults(
        vpd_info_t *vpd_info,
        u32_t def_mask
        )
{
  if (def_mask & SET_DEF_PRODUCT_NAME)
    strcpy(vpd_info->product_name,DEFAULT_PRODUCT_STRING);

  if (def_mask & SET_DEF_PART_NUMBER)
    strcpy(vpd_info->part_number,DEFAULT_PART_NUMBER);

  if (def_mask & SET_DEF_EC_NUMBER)
    strcpy(vpd_info->ec_number,DEFAULT_ENG_CHANGE_NUM);

  if (def_mask & SET_DEF_SERIAL_NUMBER)
    strcpy(vpd_info->serial_number,DEFAULT_SERIAL_NUM);

  if (def_mask & SET_DEF_MANUFACT_ID)
    strcpy(vpd_info->manufacturing_id,DEFAULT_MANUFACTURING_ID);

  if (def_mask & SET_DEF_ASSET_TAG)
    strcpy(vpd_info->asset_tag,DEFAULT_ASSET_TAG_ID);

  if (def_mask & SET_DEF_V0)
    strcpy(vpd_info->v0,"");

  if (def_mask & SET_DEF_V1)
    strcpy(vpd_info->v1,"");
}

static u32_t
vpd_check_region(
        u8_t id,
        vpd_region_t *region,
        u16_t len
        )
{
    return (region->id == id) && (region->len <= len);
}

static u32_t
vpd_check_field(
        vpd_info_t *vpd_info,
        u8_t **ptr,
        char *id,
        u8_t len,
        char *field,
        u8_t tag
        )
{
    vpd_field_t *fld = (vpd_field_t*) *ptr;

    if (id[0] == fld->name[0] && (id[1] == fld->name[1]))
    {
        if (fld->len > len)
        {
            printf("** Error: The max length for field %c%c is %d,\nbut found %d instead. Truncated\n",id[0],id[1],len, fld->len);
            fld->len = len;
        }
        strncpy(field, (char*)(&fld->data), fld->len);
        *ptr += fld->len + FIELD_HEADER_LEN;
        return VPD_ACCESS_SUCCESS;
    }
    return VPD_ACCESS_FAILURE;
}

void dbg_get_vpd_info(
        vpd_info_t *vpd_info,
        media_vpd_t *vpd_e,
        u8_t        vpd_check
        )
{
    vpd_region_t *region = (vpd_region_t *) &vpd_e->vpd_r;
    vpd_region_t *rRegion;
    vpd_region_t *wRegion = (vpd_region_t *) &vpd_e->vpd_w;
    u8_t *ptr = (u8_t *)&vpd_e->vpd_r;

#ifndef FWUPGTOOL
    memset((u8_t *)vpd_info,0,sizeof(vpd_info_t));
#else
    memset((u8_t *)vpd_info,0,sizeof(vpd_info_t));
#endif // !FWUPGTOOL
    if (vpd_check_region(TAG_DATA_TYPE_LR_STRING | TAG_LARGE_RESOURCE, region, MAX_PRODUCT_NAME_LENGTH))
    {
        strncpy(vpd_info->product_name, (char*)(&region->data), region->len > MAX_PRODUCT_NAME_LENGTH ? MAX_PRODUCT_NAME_LENGTH : region->len);
        /* check read only region */
        rRegion = (vpd_region_t *) (ptr + region->len + REGION_HEADER_LEN);
        if (vpd_check_region(TAG_DATA_TYPE_LR_VPD_R | TAG_LARGE_RESOURCE,
                             rRegion, (u16_t) (MAX_VPD_R_LENGTH -
                             ((u32_t)(unsigned long)rRegion -
                             (u32_t)(unsigned long)region) - REGION_HEADER_LEN)))
        {
            ptr = &rRegion->data;
            while (ptr < (u8_t*) wRegion)
            {
                if (vpd_check_field(vpd_info, &ptr, "PN", MAX_PART_NUMBER_LENGTH, vpd_info->part_number, SET_DEF_PART_NUMBER) &&
                    vpd_check_field(vpd_info, &ptr, "EC", MAX_EC_NUMBER_LENGTH, vpd_info->ec_number, SET_DEF_EC_NUMBER) &&
                    vpd_check_field(vpd_info, &ptr, "SN", MAX_SERIAL_NUMBER_LENGTH, vpd_info->serial_number, SET_DEF_SERIAL_NUMBER) &&
                    vpd_check_field(vpd_info, &ptr, "MN", MAX_MANUFACT_ID_LENGTH, vpd_info->manufacturing_id, SET_DEF_MANUFACT_ID) &&
                    vpd_check_field(vpd_info, &ptr, "V0", MAX_VENDDOR_SPECIFIC_V0, vpd_info->v0, SET_DEF_V0))
                {
                    if ((ptr[0] == 'R') && (ptr[1] == 'V'))
                    {
                        u8_t * ptr2 = (u8_t *) vpd_e;
                        u8_t sum=0;

                        while (ptr2<=(ptr+3))
                            sum += *ptr2++;
                        if (sum)
                        {
                            /* Calculate the old way to maintain compatibility */
                            for (ptr2=(u8_t*)vpd_e; ptr2 != (u8_t*)rRegion; ptr2++)
                            {
                                sum -= *ptr2;
                            }
                            if (sum)
                            {
                                printf("** Error: Invalid checksum in RV field\n");
                                dbg_vpd_set_defaults(vpd_info,SET_DEF_ALL_VPD_R);
                            }
                        }
                    }
                    else
                    {
                        printf("** Error: unknow field %c%c found\n",ptr[0],ptr[1]);
                        dbg_vpd_set_defaults(vpd_info,SET_DEF_ALL_VPD_R);
                    }
                    break;
                }
            }
        }

#if (VPD_WRITE_SUPPORTED)
        /* check read/write region */
        /* len = MAX_VPD_W_LENGTH -  One byte for End Tag and 3 bytes for VPD-W tag */
        if (vpd_check_region(TAG_DATA_TYPE_LR_VPD_W | TAG_LARGE_RESOURCE, wRegion, MAX_VPD_W_LENGTH-4))
        {
            ptr = &wRegion->data;
            while (((u32_t)ptr - (u32_t) wRegion) < (MAX_VPD_W_LENGTH-1))
            {
                if (vpd_check_field(vpd_info, &ptr, "YA", MAX_ASSET_TAG_LENGTH, vpd_info->asset_tag, SET_DEF_ASSET_TAG) &&
                    vpd_check_field(vpd_info, &ptr, "V1", MAX_VENDDOR_SPECIFIC_V1, vpd_info->v1, SET_DEF_V1))
                {
                    if (!(((ptr[0] == 'R') && (ptr[1] == 'W')) || (ptr[0] == 'Y')))
                    {
                        printf("** Error: Unknow field %c%c found\n",ptr[0],ptr[1]);
                        dg_vpd_set_defaults(vpd_info,SET_DEF_ALL_VPD_W);
                        break;
                    }
                    ptr += FIELD_HEADER_LEN + ptr[2];
                }
            }
        }
#endif
    }
    else
    {
        if (vpd_check) {
            printf("** Error: No valid VPD header found\n");
        }
        vpd_info->product_name[0] = '\0';
        vpd_info->part_number[0] = '\0';
        vpd_info->ec_number[0] = '\0';
        vpd_info->serial_number[0] = '\0';
        vpd_info->manufacturing_id[0] = '\0';
        vpd_info->asset_tag[0] = '\0';
        vpd_info->v0[0] = '\0';
        vpd_info->v1[0] = '\0';
        //dg_vpd_set_defaults(vpd_info,SET_DEF_ALL_PARMS);
    }
}


u32_t dbg_set_vpd_info(
        vpd_info_t *vpd_info,
        media_vpd_t *vpd_e
        )
{
    u8_t *ptr;
#ifndef FWUPGTOOL
    int len_remaining;
    u16_t field_len;
#else
    size_t len_remaining;
    size_t field_len; // to eliminate compile warnings
#endif // !FWUPGTOOL
    u8_t *checksum_start;
    u8_t checksum;

    /* Unused space is set to zeros */
#ifndef FWUPGTOOL
    memset((u8_t *)vpd_e,0,sizeof(media_vpd_t));
#else
    memset((u8_t *)vpd_e,0,sizeof(media_vpd_t));
#endif // !FWUPGTOOL

    ptr = (u8_t *)&vpd_e->vpd_r;
    checksum_start = ptr;       // checksum calculation should start at the begining of vpd_r

    len_remaining = MAX_VPD_R_LENGTH;
    /* Product Name */
    *ptr++ = (TAG_LARGE_RESOURCE | TAG_DATA_TYPE_LR_STRING);
    field_len = strlen(vpd_info->product_name)+1;
    //*(u16_t *)ptr = field_len; // Alignment problem on IA64
#ifndef FWUPGTOOL
    memcpy(ptr, &field_len, sizeof(u16_t));
#else
    memcpy(ptr, &field_len, sizeof(u16_t));
#endif // !FWUPGTOOL
    ptr+=2;
    strncpy((char *)ptr,vpd_info->product_name,field_len);
    ptr += field_len;
    len_remaining -= (field_len+3);

    /****************  VPD-R *************/
    *ptr++ = (TAG_LARGE_RESOURCE | TAG_DATA_TYPE_LR_VPD_R);
    len_remaining -= 3;
#if (VPD_WRITE_SUPPORTED == 0)
    /* Subtract one more for the END TAG */
    len_remaining--;
#endif
    //*(u16_t *)ptr = len_remaining; // Alignment problem on IA64
#ifndef FWUPGTOOL
    memcpy(ptr, &len_remaining, sizeof(u16_t));
#else
    memcpy(ptr, &len_remaining, sizeof(u16_t));
#endif // !FWUPGTOOL
    ptr +=2;

    /* Part Number - PN */
    strcpy((char *)ptr,"PN");
    ptr += 2;
    field_len = strlen(vpd_info->part_number);
    *ptr++ = (u8_t )field_len;
    strcpy((char *)ptr,vpd_info->part_number);
    ptr += field_len;
    len_remaining -= (field_len + 3);

    /* Engineering Change Level - EC */
    strcpy((char *)ptr,"EC");
    ptr += 2;
    field_len = strlen(vpd_info->ec_number);
    *ptr++ = (u8_t )field_len;
    strcpy((char *)ptr,vpd_info->ec_number);
    ptr += field_len;
    len_remaining -= (field_len + 3);
    /* Serial Number - SN */
    strcpy((char *)ptr,"SN");
    ptr += 2;
    field_len = strlen(vpd_info->serial_number);
    *ptr++ = (u8_t )field_len;
    strcpy((char *)ptr,vpd_info->serial_number);
    ptr += field_len;
    len_remaining -= (field_len + 3);

    /* Manufacturing ID - MN */
    strcpy((char *)ptr,"MN");
    ptr += 2;
    field_len = strlen(vpd_info->manufacturing_id);
    *ptr++ = (u8_t )field_len;
    strcpy((char *)ptr,vpd_info->manufacturing_id);
    ptr += field_len;
    len_remaining -= (field_len + 3);

    if (strlen(vpd_info->v0))
    {
        /* Manufacturing ID - MN */
        strcpy((char *)ptr,"V0");
        ptr += 2;
        field_len = strlen(vpd_info->v0);
        *ptr++ = (u8_t )field_len;
        strcpy((char *)ptr,vpd_info->v0);
        ptr += field_len;
        len_remaining -= (field_len + 3);
    }
    /* Reserved and Checksum - RV */
    strcpy((char *)ptr,"RV");
    ptr += 2;
    field_len = (len_remaining - 3);
#if (VPD_WRITE_SUPPORTED == 0)
    /* End Tag */
    vpd_e->vpd_r.data[MAX_VPD_R_LENGTH-1] = (TAG_DATA_TYPE_SR_END << 3);
#endif
    *ptr++ = (u8_t )field_len;
    checksum = 0;
    while (checksum_start < ptr)
    {
        checksum += *checksum_start++;
    }
    *ptr = -checksum;

#if (VPD_WRITE_SUPPORTED)
    /****************  VPD-W *************/
    ptr = (u8_t *)&vpd_e->vpd_w;
    len_remaining = MAX_VPD_W_LENGTH-1;  /* 1 byte for End Tag */
    *ptr++ = (TAG_LARGE_RESOURCE | TAG_DATA_TYPE_LR_VPD_W);
    len_remaining -= 3;
    //*(u16_t *)ptr = len_remaining;  // Alignment problem on IA64
#ifndef FWUPGTOOL
    memcpy(ptr, &len_remaining,  sizeof(u16_t));
#else
    memcpy(ptr, &len_remaining,  sizeof(u16_t));
#endif // !FWUPGTOOL
    ptr +=2;

    /* Asset Tag - YA */
    strcpy((char *)ptr,"YA");
    ptr += 2;
    field_len = strlen(vpd_info->asset_tag);
    *ptr++ = field_len;
    strcpy((char *)ptr,vpd_info->asset_tag);
    ptr += field_len;
    len_remaining -= (field_len + 3);

    /* Vendor specific - V1 */
    if (strlen(vpd_info->v1))
    {
        strcpy((char *)ptr,"V1");
        ptr += 2;
        field_len = strlen(vpd_info->v1);
        *ptr++ = field_len;
        strcpy((char *)ptr,vpd_info->v1);
        ptr += field_len;
        len_remaining -= (field_len + 3);
    }
    /* R/W fields - RW */
    strcpy((char *)ptr,"RW");
    ptr += 2;
    len_remaining -= 3;
    field_len = len_remaining;
    *ptr++ = field_len;

    /* End Tag */
    vpd_e->vpd_w.data[MAX_VPD_W_LENGTH-1] = (TAG_DATA_TYPE_SR_END << 3);
#endif

    return VPD_ACCESS_SUCCESS;
}

