
/********************************************************************
 * Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved.
 * Copyright(c) 2014-2015 QLogic Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of QLogic Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of QLogic Corporation. 
 *
 *
 *  Name: E X P R O M . H
 *
 *  Description: This file contains structure and macro definitions
 *  relating to PCI ROM header stuffs. 
 *
 ********************************************************************/

#ifndef __DBG_EXPROM_H__
#define __DBG_EXPROM_H__

#include "bcmtype.h"

typedef struct _exp_rom_hdr_t
{
#define ROM_HEADER_SIG		0x0AA55
  u16_t Signature;
  u8_t  Size;
  u8_t  Entry[4];
  u8_t  Cksum;
  u16_t VendorOffset;             /* Offset to vendor_data_t structure */
  u8_t  reserved1[12];
  u16_t ROMIDoffset;
  u16_t PCIdsOffset;
  u16_t PnPehOffset;              /* Offset to pci_rom_hdr_t structure */
  u8_t  reserved2[4];
} exp_rom_hdr_t;



//#pragma pack(1)
#pragma pack(push,1)

typedef struct _s_vendor_data_t {
#define VF_LOM            	        0x01  	/* This is a LOM image.  Runtime code */
									        /* is included in the memory defined */
									        /* by the option ROM header. */
#define VF_DISABLE_PROT_CFG         0x02  	/* This disables protocol configuration  */

#define VF_FORCE_PFN      	        0x04  	/* Force PXE image to use a given PFN */

#define VF_DIRS_FOLLOWED        	0x100

#define VF_NX2_E2_SUPPORT_ONLY      0x1000  /* Option ROM with E2 support only */
#define VF_NX2_E3_SUPPORT_ONLY      0x2000  /* Option ROM with E3 support only */
#define VF_NX2_E4_SUPPORT_ONLY      0x4000  /* Option ROM with E4 support only */
#define VF_NX2_SUPPORT_ALL_DEVICES  0x0000  /* Option ROM with support to all devices */
#define VF_NX2_DEV_SUPPORT_MASK     0xFF00


    u16_t Flags;               /* See VF_xxxx equates */

    u32_t UNDIAddr;            /* This field, when stored in the */
                                /* ROM, contains the 16bit offset of */
                                /* the UNDI driver image from the */
                                /* beginning of the ROM.  During */
                                /* initialization this field is */
                                /* changed to the physical address */
                                /* of the UNDI driver image. */

    u16_t UNDIDataImageSize;   /* Number of bytes of initialized */
                                /* data in the UNDI driver image. */

    u16_t UNDICodeImageSize;   /* Number of bytes of initialized */
                                /* code in the UNDI driver image. */

    u16_t UNDIDataSegSize;     /* Number of bytes of memory needed */
                                /* for the data segment. */

    u16_t UNDICodeSegSize;     /* Number of bytes of memory needed */
                                /* for the code segment. */

    u32_t BaseAddr;            /* These fields contain the same kind */
    u16_t BaseDataImageSize;   /* of information about the base-code */
    u16_t BaseCodeImageSize;   /* runtime image as the above fields */
    u16_t BaseDataSegSize;     /* contain about the UNDI driver. */
    u16_t BaseCodeSegSize;
    u16_t InitCodeSize;
/* Since Watcom compiler pads 16 to 32 bit, so OFFSET macro doest not work */
#define PXEVERSION_OFFSET   0x1c    
    u32_t PxeVersion;
	u16_t forced_pfn;
	u16_t attachDirOffset;     /* Offset from beginning of VendorData structure */
} vendor_data_t;

#define MAX_ATTACH_DIRS 4

typedef struct attach_image_t {
    u16_t dirs, maxDirs;
    code_entry_t dir[MAX_ATTACH_DIRS];
    /* Note the code_entry_t.nvm_start_addr will be offset from
       MBA beginning instead of abusolute NVM address */
} attach_image_t;

#pragma pack(pop)

typedef struct _pci_rom_hdr_t
{
  u8_t  Signature[4]; 
  u16_t VendorID;
  u16_t DeviceID;
  u16_t VP;
  u16_t StructLength;
  u8_t  StructRev;
  u8_t  BaseClass;
  u8_t  SubClass;
  u8_t  Interface;
  u16_t ImageLength;
  u16_t ImageRev;
  u8_t  CodeType;
  u8_t  Indicator;
  u8_t  reserved[2];
} pci_rom_hdr_t;

typedef struct _pci30_rom_hdr_t
{
  u8_t  Signature[4]; 
  u16_t VendorID;
  u16_t DeviceID;
  u16_t VP;
  u16_t StructLength;
  u8_t  StructRev; /*PCI30 or not*/
  u8_t  BaseClass;
  u8_t  SubClass;
  u8_t  Interface;
  u16_t ImageLength;
  u16_t ImageRev;
  u8_t  CodeType;
  u8_t  Indicator;
  u16_t RunTimeImgLen;
  u16_t CfgCodeHdr;
  u16_t DmtfEntry;
} pci30_rom_hdr_t;

#define PCIR_OFFSET(f)                  ((u32_t)((int_ptr_t) &(((pci30_rom_hdr_t *)0)->f)))

typedef struct 
{
  u16_t          Signature;              // 0xaa55
  u16_t          InitializationSize;
  u32_t          EfiSignature;           // 0x0EF1
  u16_t          EfiSubsystem;
  u16_t          EfiMachineType;
  u16_t          CompressionType;
  u8_t           Reserved[8];
  u16_t          EfiImageHeaderOffset;
  u16_t          PcirOffset;
  u16_t          PCIR4alig;
  u32_t          PCIRSignature;
  u16_t          VendorID;
  u16_t          DeviceID;
  u16_t          Reserved1;
  u16_t          PCIRLen;
  u8_t           PCIRRev;
} efi_pci_expansion_rom_header_t;

#define EFI_PCI_EXPANSION_ROM_HEADER_EFISIGNATURE 0x0EF1
#define PCI_CODE_TYPE_EFI_IMAGE                   0x03
#define EFI_PCI_EXPANSION_ROM_HEADER_COMPRESSED   0x0001

#define PCI_DATA_STRUC_SIG	"PCIR"

#define PCI_CODE_X86		0x00

#define PCI_IND_LAST_IMAGE	0x80

#endif /* __DBG_EXPROM_H__ */

