#ifndef __DBG_NVM_H__
#define __DBG_NVM_H__

#include "bcmtype.h"
#include "dbg_vpd.h"
#include "edebug_types.h"
#include "append.h"
#include "mcp_shmem.h"
#include "dg_nvm_struct.h"


typedef struct nvm_cfg nvm_cfg_t;


#define NVM_LITTLE_ENDIAN_FLAG "-l"
#define NEW_NCSI_START_VERSION  10007
#define IMAGE_HDR_TYPE_ISCSI_BOOT_CFG_PRG_LGCY     0x70636269
#define IMAGE_HDR_TYPE_ISCSI_BOOT_CPRG_LGCY        0x6c706269

#define PRG_NVM_LITTLE_ENDIAN    (0<<0)
#define PRG_NVM_BIG_ENDIAN       (1<<0)
#define PRG_NVM_APPEND_CRC       (1<<1)
#define PRG_NVM_MAC_PRG          (1<<2)
#define PRG_NVM_LIC_PRG          (1<<3)
#define PRG_NVM_SKIP_CFG_PRG     (1<<4)
#define PRG_NVM_IBCFG            (1<<5)
#define PRESERVE_NVM_EC          (1<<6)
#define PRESERVE_NVM_SN          (1<<7)
#define PRG_HANDLE_MACP          (1<<8)


#define NVM_ATMEL_PAGE_SIZE      264

#define IMG_TYPE_ATMEL       1
#define IMG_TYPE_ST          2

#define NVM_CFG_KEY_DEFAULT      "default"
#define NVM_CFG_KEY_CANCEL       "cancel"
#define NVM_CFG_KEY_SAVE         "save"

#define NVM_FILL_PTTN_INC        "inc"
#define NVM_FILL_PTTN_ADDR       "addr"

#define NVM_ERASE_VALUE          0

#define NVM_IMAGE_NAME_BC1             		"BC1"
#define NVM_IMAGE_NAME_BC2             		"BC2"
#define NVM_IMAGE_NAME_MBA             		"MBA"
#define NVM_IMAGE_NAME_L2RXP           		"L2_RXP"
#define NVM_IMAGE_NAME_IPMI            		"IPMI"
#define NVM_IMAGE_NAME_ISCSI_BOOT      		"ISCSI_B"
#define NVM_IMAGE_NAME_ISCSI_BOOT_IPV6 		"ISCSI_B_IPV6"
#define NVM_IMAGE_NAME_ISCSI_BOOT_IPV4N6 	"ISCSI_B_IPV4N6"
#define NVM_IMAGE_NAME_ISCSI_BOOT_CFG  		"I_B_CFG"
#define NVM_IMAGE_NAME_ISCSI_BOOT_CFG_V2 	"I_B_CFG_V2"
#define NVM_IMAGE_NAME_ISCSI_BOOT_CFG2 		"I_B_CFG2"
#define NVM_IMAGE_NAME_ISCSI_BOOT_CFG2_V2 	"I_B_CFG2_V2"
#define NVM_IMAGE_NAME_ISCSI_CPRG      		"I_B_CPRG"
#define NVM_IMAGE_NAME_NCSI_CMN        		"NCSI_CMN"
#define NVM_IMAGE_NAME_NCSI_ELIB       		"NCSI_ELIB"
#define NVM_IMAGE_NAME_L2T             		"L2T"
#define NVM_IMAGE_NAME_L2C             		"L2C"
#define NVM_IMAGE_NAME_L2X             		"L2X"
#define NVM_IMAGE_NAME_L2U             		"L2U"
#define NVM_IMAGE_NAME_L2TI            		"L2TI"
#define NVM_IMAGE_NAME_L2CI            		"L2CI"
#define NVM_IMAGE_NAME_L2XI            		"L2XI"
#define NVM_IMAGE_NAME_L2UI            		"L2UI"
#define NVM_IMAGE_NAME_MODULES_PN      		"MODULES_PN"
#define NVM_IMAGE_NAME_FCOE_BOOT      		"FCOE_B"
#define NVM_IMAGE_NAME_FCOE_BOOT_CFG  		"FEB_CFG"
#define NVM_IMAGE_NAME_FCOE_BOOT_CFG2 		"FEB_CFG2"
#define NVM_IMAGE_NAME_FCOE_BOOT_CPRG 		"FEB_CPRG"
#define NVM_IMAGE_NAME_FCOE_BOOT_CPRG_LGCY 	"FEB_CPRG_LGCY"
#define NVM_IMAGE_NAME_FCOE_BOOT_CPRG_EVRST	"FEB_CPRG"
#define NVM_IMAGE_NAME_NIC_PARTITION_CFG        "NIC_PRTION"
#define NVM_IMAGE_NAME_E3_WC                    "E3_WC"   
#define NVM_IMAGE_NAME_E3_PCIE                  "E3_PCIE" 
#define NVM_IMAGE_NAME_SWIM1                    "SWIM1"
#define NVM_IMAGE_NAME_SWIM2                    "SWIM2"
#define NVM_IMAGE_NAME_SWIM3                    "SWIM3"
#define NVM_IMAGE_NAME_SWIM4                    "SWIM4"
#define NVM_IMAGE_NAME_SWIM5                    "SWIM5"
#define NVM_IMAGE_NAME_SWIM6                    "SWIM6"
#define NVM_IMAGE_NAME_SWIM7                    "SWIM7"
#define NVM_IMAGE_NAME_SWIM8                    "SWIM8"
#define NVM_IMAGE_NAME_CCM                      "CCM"

#define MAX_VER_STR_SIZE                         50
#define UPDATE_CFG_MAX_ERR_MSG_LEN               80

#define EXTRACT_CFG_INIT_VALUE 0x999999
#define EXTRACT_CFG_VAL_VALUE  1
#define EXTRACT_CFG_VAL_STRING                   0


#define MANUF_INFO_PCI_VENDOR_ID(id)         (((id)>>16)&0xffff)
#define MANUF_INFO_PCI_DEVICE_ID(id)         ((id)&0xffff)
#define MANUF_INFO_PCI_ID(v,d)               (((u32_t)(d) & 0xffff) | \
                                             (((u32_t)(v)<<16) & 0xffff0000))
#define MANUF_INFO_PCI_SUBSYS_DEVICE_ID(id)  (((id)>>16)&0xffff)
#define MANUF_INFO_PCI_SUBSYS_VENDOR_ID(id)  ((id)&0xffff)
#define MANUF_INFO_PCI_SUBSYS_ID(v,d)        (((u32_t)(v) & 0xffff) | \
                                             (((u32_t)(d)<<16) & 0xffff0000))

typedef enum {
	MBA_MBA_LEGACY_IDX=0,
	MBA_MBA_PCI3CLP_IDX,
	MBA_MBA_PCI3_IDX,
	MBA_FCODE_IDX,
	EFI_X86_IDX,
	EFI_IPF_IDX,
	EFI_EBC_IDX,
	EFI_X64_IDX
} mba_image_t;

#define ROMIMG_NUM_MAX 8

typedef struct _image_map_t
{
    char *name;
    char *opt;
    u32_t agent;
    u32_t type;
    u32_t hdr_type;
} image_map_t;

typedef enum {
    DEF_CFG_IDX_DID_STRING,
    DEF_CFG_IDX_PWR_CONSUMED,
    DEF_CFG_IDX_PWR_DISSIPATED,
    DEF_CFG_IDX_MAX
} def_cfg_idx_t;

typedef struct _default_id_table_t
{
    u32_t id;
    char *dev_info[DEF_CFG_IDX_MAX];
} default_id_table_t;

#define NVM_IMAGE_NAME_BOOTSTRAP                "BOOTSTRAP"
#define NVM_IMAGE_NAME_DIRECTORY                "DIRECTORY"
#define NVM_IMAGE_NAME_HW_INFO                  "HW_INFO"
#define NVM_IMAGE_NAME_FEATURE_INFO             "FEAT_INFO"
#define NVM_IMAGE_NAME_PATH1_FEATURE_INFO       "FEAT_INFO_2"
#define NVM_IMAGE_NAME_MANUF_KEY_INFO_P0        "MKEY_INFO_P0"
#define NVM_IMAGE_NAME_UPGRADE_KEY_INFO_P0      "UKEY_INFO_P0"
#define NVM_IMAGE_NAME_MANUF_KEY_INFO_P1        "MKEY_INFO_P1"
#define NVM_IMAGE_NAME_UPGRADE_KEY_INFO_P1      "UKEY_INFO_P1"


enum image_map_idx_t
{
    IMAGE_MAP_IDX_BC1,
    IMAGE_MAP_IDX_BC2,
    IMAGE_MAP_IDX_MBA,
	IMAGE_MAP_IDX_NCSI_CMN,
	IMAGE_MAP_IDX_NCSI_ELIB,
    IMAGE_MAP_IDX_L2T,
    IMAGE_MAP_IDX_L2C,
    IMAGE_MAP_IDX_L2X,
    IMAGE_MAP_IDX_L2U,
    IMAGE_MAP_IDX_IPMI,
    IMAGE_MAP_IDX_UMP,
    IMAGE_MAP_IDX_ISCSI_BOOT,
    IMAGE_MAP_IDX_ISCSI_BOOT_CFG,
    IMAGE_MAP_IDX_ISCSI_BOOT_CFG2,
    IMAGE_MAP_IDX_ISCSI_BOOT_CPRG,
    IMAGE_MAP_IDX_FCOE_BOOT,
    IMAGE_MAP_IDX_FCOE_BOOT_CFG,
    IMAGE_MAP_IDX_FCOE_BOOT_CFG2,
    IMAGE_MAP_IDX_FCOE_BOOT_CPRG
};

typedef enum {
    IDX_DUMMY = 0,
    IDX_L2_MAC_ADDRESS,
    IDX_POWER_DISS_COMMON,
    IDX_POWER_MAN_SCALE,
    IDX_POWER_DISSIPATED,
    IDX_POWER_CONSUMED,
    IDX_VID,
    IDX_VDID,
    IDX_SVID,
    IDX_SDID,
//  IDX_BOARD_TYPE, - removed
    IDX_PROD_NAME = 11,
    IDX_PART_NUM,
    IDX_ENGR_CHNG,
    IDX_SERIAL_NUM,
    IDX_MANUF_ID,
    IDX_LED_MODE,
    IDX_UMP_NUM,
    IDX_BAR_SIZE,
    IDX_WOL_MAGIC_ENABLE,
    IDX_WOL_WINDOWS_DEFAULT,
    IDX_WOL_ACPI_UPON_MGMT,
    IDX_MBA_ENABLE,
    IDX_MBA_BOOT_AGENT,
    IDX_MBA_BOOT_TYPE,
    IDX_MBA_DELAY,
    IDX_MBA_HOTKEY,
    IDX_MBA_PROMPT,
    IDX_EXP_ROM_SIZE,
    IDX_MFW_ENABLE,
    IDX_MFW_TYPE_SELECT, 
    IDX_AN_TYPE,
    IDX_AN_GRACE_PER,
    IDX_PCI_GEN2_ENABLE,
    IDX_CORE_CLOCK_CHANGE,
    IDX_SPEED_CAP_MASK_D0,
    IDX_SPEED_CAP_MASK_D3,
    IDX_ISCSI_MAC,
    IDX_RDMA_MAC,
    IDX_SERDES_EXT_PHY_TYPE,
    IDX_SERDES_EXT_PHY_ADDR,
    IDX_XGXS_EXT_PHY_TYPE,
    IDX_XGXS_EXT_PHY_ADDR,
    IDX_SERDES_BPLANE_FORCED_COEF,
    IDX_XGXS_BPLANE_FORCED_TX_COEF,
    IDX_XGXS_BPLANE_FORCED_RX_COEF,
    IDX_SERDES_BPLANE_COEF,
    IDX_XGXS_BPLANE_TX_COEF,
    IDX_XGXS_BPLANE_RX_COEF,
    IDX_LANE_SWAP_CONFIGURATION,
    IDX_PORT_SWAP,
    IDX_BCM_LOOPBACK_MODE_EN,
    IDX_MBA_VLAN_ENABLE,
    IDX_MBA_VLAN_VALUE,
    IDX_UMP_USE_RMII,                /* Teton II only */
    IDX_SPARE_NUM,
    IDX_10G_OVERCLK,
    IDX_SWITCH_CON,
    IDX_FLOW_CTL,
    IDX_MFW_WOL_10G_OVERCLK,
    IDX_MFW_WOL_SWITCH_CON,
    IDX_MFW_WOL_FLOW_CTL,
    IDX_SMBUS_ADDR,
    IDX_SMBUS_ENABLE,
    IDX_RESET_MCP_ON_CORE_RESET,
    IDX_BOARD_VERSION,
    IDX_BEACON_WOL_EN,
    IDX_MDIO_VOLTAGE,
    IDX_MBA_LINK_SPEED,
	IDX_MBA_BOOT_RETRY=107,
    IDX_CONFIG_MAX
} nvm_cfg_idx_t;

#define DATA_TYPE_SHARED_SECRET        0
#define DATA_TYPE_MANUF_KEY            1
#define DATA_TYPE_UPGRADE_KEY          2
#define DATA_TYPE_CURR_KEY             3     /* the key in shmem */
#define DATA_TYPE_MAX                  4

u32_t dbg_program_nvm(u8_t *buf, u32_t byte_cnt, u32_t offset, 
                         u32_t flag, u32_t *total_byte_ptr);

u32_t dbg_xlate_flash_virt_addr (u32_t phy_addr);
u32_t dbg_xlate_flash_phy_addr (u32_t virt_addr);

dbg_status_t dbg_read_nvram(
    u32_t   addr,
    u32_t * val_p,
    u32_t   byte_cnt
    );

dbg_status_t dbg_read_NVM_or_image(
#define READ_VERSION_FROM_NVM 0
#define READ_VERSION_FROM_IMAGE 1
		u8_t               read_flag,
		dbg_virt_addr_t   base_addr,
		u32_t              offset,
		u32_t *            val_p,
		u32_t              byte_cnt
		);

dbg_status_t dbg_write_nvram(
    u32_t addr,
    u32_t * val_p,
    u32_t   byte_cnt
    );

dbg_status_t dbg_update_mba_hdr  (
        u8_t *buffer,
        u16_t vendor_id,
        u16_t device_id
        );

dbg_status_t dbg_nvm_find_dir_entry(
        IN   u32_t image_type, 
        OUT  u32_t *entry_idx,
		OUT  u32_t *start_address
        );

u32_t dbg_find_empty_entry(
    OUT   u32_t *entry_idx
    ); 

u32_t dbg_find_room(
    u32_t image_type, 
    u32_t byte_cnt, 
	extended_dir_image_t *extended_dir_p,
    u32_t *nvm_addr_ptr
    );

dbg_status_t dbg_nvm_get_dir_info(
    u32_t entry_idx,
    u32_t *nvm_addr_ptr,
    u32_t *byte_cnt_ptr,
    u32_t *sram_addr_ptr,
    u32_t *image_type_ptr,
    u32_t *agent_ptr
        );

u32_t dbg_update_dir_entry(
    u32_t entry_idx, 
    u32_t nvm_addr, 
    u32_t byte_cnt, 
    u32_t sram_addr,
    u32_t image_type, 
    u32_t agent_type
    );

u32_t dbg_fw_ver_code(
        u32_t nvm_addr
        );

u32_t dbg_image_table_size();

dbg_status_t dbg_image_version_validation(
	const char *image_name, 
	char *old_version,
	char *new_version
);

dbg_status_t dbg_image_mba_version_validation (
	dbg_virt_addr_t file_buf, 
	dbg_virt_addr_t nvm_addr
);

dbg_status_t dbg_get_romimg_vers(
	u8_t read_source, 
	dbg_virt_addr_t addr, 
	u32_t *vers, 
	char **type_str
);

void dbg_fw_ver_string(
    dbg_virt_addr_t	nvm_addr, 
    const char 			*name,
    char       			*str_ptr,
    u8_t                read_source
    );

dbg_status_t dbg_update_bootstrap (
    u32_t sram_start_addr,
    u32_t byte_cnt, 
    u32_t nvm_start_addr
    );

dbg_status_t dbg_load_cfg(
    nvm_cfg_t *cfg_ptr
    );



dbg_status_t dbg_store_cfg(
    nvm_cfg_t *cfg_ptr
    );

void dbg_compute_crc(
        u8_t *buf, 
        u32_t byte_cnt, 
        u32_t big_endian, 
        u32_t *crc_ptr
        );

dbg_status_t dbg_nvm_program_image (
    u8_t *buf, 
    u32_t byte_cnt,
    u32_t code_image
    );

dbg_status_t dbg_prog_image_by_idx (
    u8_t *buf, 
    u32_t byte_cnt,
    u32_t image_table_idx,
	u8_t validate_versions
	);

dbg_status_t dbg_nvm_dump_nvm (
    u8_t *buf,
    u32_t big_endian,
    u32_t byte_cnt,
    u32_t offset,
    u32_t *byte_dumped
	);

dbg_status_t dbg_prog_image_in_extended_dir (    
    u8_t *buf,
    u32_t byte_cnt,
	u32_t image_table_idx, 
    u8_t validate_versions
    );

u32_t dbg_nvm_size(void);

dbg_status_t dbg_compute_image_crc (
        u32_t start_nvm_addr, 
        u32_t byte_cnt, 
        u32_t *computed, 
        u32_t *stored
        );

/**
 * Returns DIAG_STATUS_SUCCESS if file_image is
 * a legacy iSCSI image.
 */ 
dbg_status_t dbg_nvm_is_legacy_iscsi_image(
    image_header_t * file_image_p
    );

/*
 *------------------------------------------------------------------------
 * dg_nvm_fill_mac_partition_info --
 *
 *     Fills input struct with MAC partition info read from NVM image,
 *     or zeros if such image does not exist.
 *
 * Results:
 *     DIAG_STATUS_SUCCESS if operation succeeded (even if image does
 *     not exist)
 *------------------------------------------------------------------------
 */
dbg_status_t dbg_nvm_fill_mac_partition_info (nvm_nic_part_cfg_t *mac_partition_block, u32_t *result);

/*
 *------------------------------------------------------------------------
 * dg_program_mac_partition_to_nvm --
 *
 *     Checks if user changed MAC partition configuration. If so, save it to
 *     MAC partition image in extended dir. If such an image didn't exist,
 *     create it.
 *
 * Results:
 *     DIAG_STATUS_SUCCESS if operation succeeded.
 * 
 * Side effects:
 *     Creation of MAC partition image in extended dir if it didn't exist
 *     and user changed MAC partition configuration.
 *------------------------------------------------------------------------
 */
dbg_status_t dbg_program_mac_partition_to_nvm(nvm_cfg_t *nvm_cfg);

#ifndef FWUPGTOOL
#ifndef WIN_DIAG
/**   Update device information of all devices and environment MAC addr.
 *    Used after "nvm prg" "nvm fill" "nvm cfg" 
 *
 *    For FWUPG tool, we don't issue the MCP reset and update the device information.
 **/
void dbg_refresh_devices();
#endif // WIN_DIAG
#endif // !FWUPGTOOL


dbg_status_t dbg_nvm_parse_extended_dir_image(
		IN u32_t offset, 
		IN u32_t length,
		IN extended_dir_image_t *extended_dir_ptr);

/*
 *------------------------------------------------------------------------
 * dg_handle_mac_partition_data --
 *
 *     If user requested to remember old mac partition values, nvm data will
 *     hold data according to the table below:
 * 
 *     original nvm     new nvm 
 *     included         included     
 *     mac_p image?     mac_p image?   logics
 *     ------------     -----------    ------
 *     No               No             no mac_p imahe
 *     No               No             no mac_p imahe
 *     Yes              No             original mac_p image with empty MACs
 *     Yes              Yes            original mac_p image with new MACs
 * 
 * Input:
 *     macp_image_found - Whether mac_p existed in source NVM
 *     macp_image - values of mac+p image if existed in source NVM
 * 
 * 
 * Output:
 *      None, besides "side effect" of new NVM holding right mac_p image
 *
 * Results:
 *     DIAG_STATUS_SUCCESS if operation succeeded.
 * 
 *------------------------------------------------------------------------
 */
dbg_status_t dbg_handle_mac_partition_data(u32_t orig_macp_image_found, 
                           nvm_nic_part_cfg_t orig_macp_image);

/*
 *------------------------------------------------------------------------
 * dg_program_mac_partition_to_nvm --
 *
 *     Checks if an image suitable to input type exists in extended dir.
 *     If yes, returns its parameters.
 *     Function can handle scenario that extended dir image itself doesn't
 *     exist.
 * 
 * Input:
 *     image_type - type to search in extended dir.
 * 
 * Output:
 *     found_image - TRUE if found image in extended dir.
 *     nvm_addr - address if the image (if found)
 *     byte_cnt - byte count of image (if found)
 *     entry_idx - entry in extended dir (if found) *  
 *
 * Results:
 *     DIAG_STATUS_SUCCESS if operation succeeded.
 *     found_image is TRUE if found
 * 
 *------------------------------------------------------------------------
 */
dbg_status_t dbg_nvm_find_image_in_extended_dir(u32_t *found_image, u32_t image_type, u32_t *nvm_addr, u32_t *byte_cnt, u32_t *entry_idx);

/*
 *------------------------------------------------------------------------
 * dg_find_entry_in_image_table --
 *
 *     Returns index of image in global image table.
 * 
 * Input:
 *     image_type - type to search in the table.
 * 
 * Output:
 *     Index - index of image in the table if found.  
 *
 * Results:
 *     DIAG_STATUS_SUCCESS if found image in table. * 
 *------------------------------------------------------------------------
 */
dbg_status_t dbg_find_entry_in_image_table(u32_t image_type, u32_t *index);	   

dbg_status_t dbg_nvm_find_extended_dir_entry(
        IN   u32_t 					image_type,
		IN   extended_dir_image_t 	*extended_dir_p,
        OUT  u32_t 					*entry_idx,
        OUT  u32_t 					*start_address, 
		OUT  u32_t 					*byte_cnt   
        );

dbg_status_t dbg_nvm_delete_extended_dir_entry(
		IN extended_dir_image_t *extended_dir_p, 
		IN u32_t entry_idx, 
		IN u32_t nvm_offset);

dbg_status_t dbg_get_nvm_offset_for_mac_addr(IN u32_t hwkey_used);	   
dbg_status_t dbg_get_nvm_offset_for_license(u32_t license_type, u32_t *nvm_addr_p, u32_t *blk_start_p);
dbg_status_t dbg_get_key_offset_from_shmem(u32_t *key_offset_p);
dbg_status_t dbg_get_nvm_offset_for_pci_id( u32_t *pci_id_p, u32_t *pci_sub_id_p);

#endif /* __DG_NVM_H__ */
