
/********************************************************************
 * Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved
 * Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
 * Contains proprietary and confidential information.
 *
 * This source file is the property of QLogic Corporation, and
 * may not be copied or distributed in any isomorphic form without 
 * the prior written consent of QLogic Corporation. 
 *
 *
 *  Name: V P D . H
 *
 *  Description: This file contains macros and data structure 
 *  definitions pertaining to VPD.
 * 
 *  AUTHOR: Kevin Tran
 *
 *  REVISION HISTORY:   
 *           Kevin Tran      01/04/2001     Created
 *           Stephen Keung   09/29/2003     Modified for Teton
 *
 *******************************************************************/


#ifndef __DBG_VPD_H__
#define __DBG_VPD_H__

#ifdef USER_LINUX
#include <sys/types.h>
#endif
#include "bcmtype.h"
#ifdef UEFI
#include <sys/types.h>
#include <machine/endian.h>
#endif
#include "license.h"
#include "nvm_map.h"

/* Status from other functions */
#define VPD_ACCESS_SUCCESS        0
#define VPD_ACCESS_FAILURE        -1

/* VPD-W is not supported for Teton */
#define VPD_WRITE_SUPPORTED       0

/* The following values are defined based on PCI 2.2 specification */
#define TAG_LARGE_RESOURCE     0x80
#define TAG_SMALL_RESOURCE     0x00

#define TAG_DATA_TYPE_LR_STRING   0x2
#define TAG_DATA_TYPE_LR_VPD_R    0x10
#define TAG_DATA_TYPE_LR_VPD_W    0x11
#define TAG_DATA_TYPE_SR_END      0xf 

#define DEFAULT_PRODUCT_STRING   "QLogic BCM57710 NetXtreme II 10 GigE"
#define DEFAULT_PART_NUMBER      "BCM957710A1"
#define DEFAULT_ENG_CHANGE_NUM   "220197-2"
#define DEFAULT_SERIAL_NUM       "0123456789"
#define DEFAULT_MANUFACTURING_ID "14e4"

#define DEFAULT_ASSET_TAG_ID     "XYZ01234567"

#define MAX_PRODUCT_NAME_LENGTH    48
#define MAX_PART_NUMBER_LENGTH     16
#define MAX_EC_NUMBER_LENGTH       10
#define MAX_SERIAL_NUMBER_LENGTH   16
#define MAX_MANUFACT_ID_LENGTH     4
#define MAX_ASSET_TAG_LENGTH       16
#define MAX_VENDDOR_SPECIFIC_V0    16
#define MAX_VENDDOR_SPECIFIC_V1    16

#pragma pack(push,1)

#define REGION_HEADER_LEN  3
typedef struct _vpd_region_t
{
	u8_t id;
    u16_t len;
    u8_t data;
} vpd_region_t;

#define FIELD_HEADER_LEN  3
typedef struct _vpd_field_t
{
	u8_t name[2];
    u8_t len;
    u8_t data;
} vpd_field_t;

#pragma pack(pop)

typedef struct _vpd_info_t
{
  char product_name[MAX_PRODUCT_NAME_LENGTH+1];
  char part_number[MAX_PART_NUMBER_LENGTH+1];
  char ec_number[MAX_EC_NUMBER_LENGTH+1];
  char serial_number[MAX_SERIAL_NUMBER_LENGTH+1]; // if modified - modify 'dg_program_nvm' function accordingly.
  char manufacturing_id[MAX_MANUFACT_ID_LENGTH+1];
  char asset_tag[MAX_ASSET_TAG_LENGTH+1];
  char v0[MAX_VENDDOR_SPECIFIC_V0+1];
  char v1[MAX_VENDDOR_SPECIFIC_V1+1];
} vpd_info_t;

#define SET_DEF_PRODUCT_NAME  (1 << 0)
#define SET_DEF_PART_NUMBER   (1 << 1)
#define SET_DEF_EC_NUMBER     (1 << 2)
#define SET_DEF_SERIAL_NUMBER (1 << 3)
#define SET_DEF_MANUFACT_ID   (1 << 4)
#define SET_DEF_ASSET_TAG     (1 << 5)
#define SET_DEF_V0			  (1 << 6)
#define SET_DEF_V1			  (1 << 7)

#define SET_DEF_ALL_VPD_R  (SET_DEF_PART_NUMBER | SET_DEF_EC_NUMBER | SET_DEF_SERIAL_NUMBER | SET_DEF_MANUFACT_ID | SET_DEF_V0)

#define SET_DEF_ALL_VPD_W  (SET_DEF_ASSET_TAG | SET_DEF_V1)

#if (VPD_WRITE_SUPPORTED)
#define SET_DEF_ALL_PARMS  (SET_DEF_PRODUCT_NAME | SET_DEF_ALL_VPD_R | SET_DEF_ALL_VPD_W)
#else
#define SET_DEF_ALL_PARMS  (SET_DEF_PRODUCT_NAME | SET_DEF_ALL_VPD_R)
#endif

void dbg_vpd_set_defaults(vpd_info_t *vpd_info, u32_t mask);
u32_t dbg_set_vpd_info(vpd_info_t *vpd_info, media_vpd_t *vpd_e);
void dbg_get_vpd_info(vpd_info_t *vpd_info, media_vpd_t *vpd_e, u8_t vpd_check);

#endif /* __DBG_VPD_H__ */

