#ifndef __DBG_MANAGEMENT_TYPES_H__
#define __DBG_MANAGEMENT_TYPES_H__

#ifdef USER_LINUX
#include <sys/types.h>
#endif

#include "dev_if.h"
#include "os_types.h"


#ifndef MAX_PATH_NUM
#define MAX_PATH_NUM 2
#endif

#define DEVICE_ID_E1                  5710
#define DEVICE_ID_E1_5                5711
#define DEVICE_ID_E1_5E               5712

#define DBG_IS_E1x(dbg_dev)	  ((((dbg_dev).pci_info.device_id == DEVICE_ID_E1) || ((dbg_dev).pci_info.device_id == DEVICE_ID_E1_5) || ((dbg_dev).pci_info.device_id == DEVICE_ID_E1_5E)) ? 1 : 0)
#define DBG_FUNC_NUM(dbg_dev) ((atoi(Tcl_GetVar2(globalInterp, TCL_ENV, ENV_IS_SPECIAL_NVM_MODE, TCL_GLOBAL_ONLY))) ?  \
				                (atoi(Tcl_GetVar2(globalInterp, TCL_ENV, ENV_SPECIAL_NVM_MODE_FUNC_NUM, TCL_GLOBAL_ONLY))) : \
                                ((dbg_dev).pci_info.pci_func))
#define DBG_PATH_NUM(dbg_dev) ((!DBG_IS_E1x(dbg_dev)) ? (DBG_FUNC_NUM(dbg_dev) & 1) : 0)


#define XDBG_CUR_DEV_IS_VALID           (xdbg_cur_dev != BAD_DEVICE_HNDL)
#define XDBG_CUR_DEV_IS_HIDDEN	        PCI_DEV_INFO(xdbg_cur_dev)->hidden
#define XDBG_CUR_DEV_IF                 PCI_DEV_INFO(xdbg_cur_dev)->drv_dev_if_p
#define XDBG_CUR_DEV                    PCI_DEV_INFO(xdbg_cur_dev)
#define CLEAR_BR_INFO(i, empty_val)     (br_devices[i] = (dbg_dev_hdl_t) ((int_ptr_t) empty_val))
#define BR_DEV_IDX(i)                   ((int_ptr_t)(br_devices[i]))
#define DBG_DEV_INDEX(i)                PF_IDX_BY_HDL((dbg_dev_hdl_t)BR_DEV_IDX(i))
#define BR_DEV_INFO(i)                  PCI_DEV_INFO((dbg_dev_hdl_t)BR_DEV_IDX(i))
#define BR_DEV_PORT_ID(i)               (BR_DEV_INFO(i)->pci_info.port_id)
#define BR_PCI_DEV_HIDDEN(i) 	        (BR_DEV_INFO(i)->hidden != 0)
#define PF_IDX_BY_HDL(hdl)              ((int_ptr_t)(hdl) & 0xffff)
#define IS_VF_HDL(hdl)                  (((int_ptr_t)(hdl) & 0xffff0000) && (hdl != BAD_DEVICE_HNDL))
#define VF_IDX_BY_HDL(hdl)              ((((int_ptr_t)(hdl) >> 16) & 0xffff) - 1)
#define BR_PCI_DEV_PRESENT(i) 	        (BR_DEV_INFO(i)->present == 1)
#define BR_PCI_DEV_VALID(i) 	        (BR_PCI_DEV_PRESENT(i) && !BR_PCI_DEV_HIDDEN(i))
#define BAD_DEVICE_HNDL                 (dbg_dev_hdl_t)((int_ptr_t)0xffffffff)

typedef void* dbg_dev_hdl_t;

typedef struct _dbg_dev_info {
    struct _dev_if_t *drv_dev_if_p;   /* Driver device interface, if NULL - means that the device
                                           driver hasn't been initialized */
    //BW void* dg_drv_dev;   /* diag Driver device interface, if NULL - means that basc device driver hasn't been initialized */
    pci_info_t  pci_info; /* Partial PCI info needed in the management level */
    u8_t        present;  /* Device present flag: 1 if present, 0 - if not. */
    u8_t 	hidden;  /* Device hidden flag: 1 if hidden, 0 - if not. */
    //BW dev_stat_t  dev_stat; /* Device info */
    int         br_idx;   /* Index in br_devices[] array corresponding to this device */
    //BW struct _diag_dev_info *vf_devices;
    u16_t  vfs_num;  /* Actual number of VFs */
    //BW struct _diag_dev_info *parent_pf; /* Parent PF info */
} dbg_dev_info_t;

extern dbg_dev_info_t *dbg_pci_devices;         /* array that will contain all PCI devices found */
extern dbg_dev_hdl_t *br_devices; 		/* Array of indices of supported QLogic devices in pci_devices array */
extern dbg_dev_hdl_t xdbg_cur_dev; 		/* Currently selected QLogic device ("device" command) */

static __inline dbg_dev_info_t* PCI_DEV_INFO(dbg_dev_hdl_t hdl)
{
	dbg_dev_info_t *parent_pf_dev = &(dbg_pci_devices[PF_IDX_BY_HDL(hdl)]);

//BW
#if 0

	if (IS_VF_HDL(hdl)) {
		u32_t vf_idx = VF_IDX_BY_HDL(hdl);
		DbgBreakIf(parent_pf_dev->vf_devices == NULL);
		DbgBreakIf(parent_pf_dev->vfs_num <= vf_idx);
		DbgBreakIf(VF_DEV_INFO(parent_pf_dev, vf_idx)->drv_dev_if_p == NULL);

		// return VF_DEV_INFO(parent_pf_dev, vf_idx);
                return NULL;
	} else {
		return parent_pf_dev;
	}

#endif

        return parent_pf_dev;
}

#endif
