#ifndef __DBG_OS_UART_H__
#define __DBG_OS_UART_H__

#include "disputil.h"

/* valid ports are from 1 to PORT_MAX */
#define COM_PORT_MAX 4

/* status */
typedef u8_t ua_status_t;

/* error code */
#define UA_STATUS_OK                0
#define UA_STATUS_FAILURE           1
#define UA_STATUS_NODEV             0xFF
#define UA_STATUS_PORT_INUSE        0xFE
#define UA_STATUS_BAD_BAUD          0xFD
#define UA_STATUS_BAD_DATABIT       0xFC
#define UA_STATUS_BAD_PARITY        0xFB
#define UA_STATUS_BAD_STOPBIT       0xFA
#define UA_STATUS_WRITE_TIMEOUT     0xF9
#define UA_STATUS_BAD_INPUT         0xF8

/* handle */
typedef u32_t ua_handle_t;
#define UA_INVALID_HANDLE           0xFF

/* open mode */
#define UAM_BINARY                  0
#define UAM_TEXT                    1

/* default values */
#define UA_DEFAULT_BAUD             19200
#define UA_DEFAULT_PORT             1

#define MAX_UART_HANDLES            99

typedef struct {
    /* All fields are initialized in init_global_var() function */
    u32_t verbose_mode;
    ua_handle_t  uart_handles[MAX_UART_HANDLES];	/* to record each opened port's handle*/
	ua_handle_t  uart_io_handle;					/* to store i/o port's handle for later 
											 * easy access. it can be eliminated by 
											 * always getting system variable value 
											 * and then index into uart_handle array. 
	                                         */
    u8_t  duart_port;                       /* DUART port */
    char  print_buf[MAX_PRINTF_BUFFER];
    char  auxprint_buf[MAX_AUXPRINTF_BUFFER];
} uart_info_t;

#endif /* __DBG_OS_UART_H__ */
