#ifndef __EDEBUG_OS_UTILS_H__
#define __EDEBUG_OS_UTILS_H__

#include <stdio.h>
#include "edebug_types.h"
#include "util.h"

#define MAX_DOMAIN_NUM    0x10000
#define MAX_BUS_NUM       256
#define MAX_DEV_NUM       32
#define MAX_FUNC          8

dbg_status_t os_if_get_total_dev(
    OUT   u32_t          * dev_found_p       /* number of found devices */
    );

void os_if_change_color(
    IN    u8_t color,        /* Color to set */
    IN    u8_t fore_or_back  /* Forground or background */
    );

dbg_status_t os_if_alloc_pcontig(
     IN    u32_t           nbytes,              /* the size of memory in bytes */
     OUT   dbg_virt_addr_t   * virt_addr_p,    /* pointer to a virtual address pointer */
     OUT   dbg_phys_addr_t   * phy_addr_p      /* physical address */
     );

void os_if_restore_color(void);

char getch(void);
char os_if_getch(void);

/**
 * Turns 32-bit value to the appropriate CPU
 * representation.
 *
 * @param val
 *
 * @return u32_t
 */
static __inline u32_t os_if_le32_to_cpu(
	IN   u32_t val
	)
{
#ifdef EDEBUG_BIG_ENDIAN
	return BYTE_SWAP(val);
#else
	return val;
#endif
}

/**
 * Turns 32-bit value to little endian
 * representation.
 *
 * @param val
 *
 * @return u32_t
 */
static __inline u32_t os_if_cpu_to_le32(
	IN   u32_t val
	)
{
#ifdef EDEBUG_BIG_ENDIAN
	return BYTE_SWAP(val);
#else
	return val;
#endif
}

/**
 * Turns 16-bit value to the appropriate CPU
 * representation.
 *
 * @param val
 *
 * @return u16_t
 */
static __inline u16_t os_if_le16_to_cpu(
	IN   u16_t val
	)
{
#ifdef EDEBUG_BIG_ENDIAN
	return BYTE_SWAP16(val);
#else
	return val;
#endif
}

/**
 * Turns 16-bit value to little endian
 * representation.
 *
 * @param val
 *
 * @return u32_t
 */
static __inline u16_t os_if_cpu_to_le16(
	IN   u16_t val
	)
{
#ifdef __BIG_ENDIAN
	return BYTE_SWAP16(val);
#else
	return val;
#endif
}

/**
 * Turns 32-bit value to the appropriate CPU
 * representation.
 *
 * @param val
 *
 * @return u32_t
 */
static __inline u32_t os_if_be32_to_cpu(
	IN   u32_t val
	)
{
#ifndef EDEBUG_BIG_ENDIAN
	return BYTE_SWAP(val);
#else
	return val;
#endif
}

/**
 * Turns 32-bit value to little endian
 * representation.
 *
 * @param val
 *
 * @return u32_t
 */
static __inline u32_t os_if_cpu_to_be32(
	IN   u32_t val
	)
{
#ifndef EDEBUG_BIG_ENDIAN
	return BYTE_SWAP(val);
#else
	return val;
#endif
}

/**
 * Turns 16-bit value to the appropriate CPU
 * representation.
 *
 * @param val
 *
 * @return u16_t
 */
static __inline u16_t os_if_be16_to_cpu(
	IN   u16_t val
	)
{
#ifndef EDEBUG_BIG_ENDIAN
	return BYTE_SWAP16(val);
#else
	return val;
#endif
}

/**
 * Turns 16-bit value to little endian
 * representation.
 *
 * @param val
 *
 * @return u32_t
 */
static __inline u16_t os_if_cpu_to_be16(
	IN   u16_t val
	)
{
#ifndef __BIG_ENDIAN
	return BYTE_SWAP16(val);
#else
	return val;
#endif
}

FILE* os_if_log_fopen(const char *file_name);
#endif
