#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "man_types.h"
#include "tcl.h"
#include "tclDecls.h"
#include "tcl_al.h"
#include "dbg_os_utils.h"
#include "sync.h"
#include "dbg_init.h"
#include "dbg_man.h"
#include "tcl_al_cmd_handlers.h"
#include "tcl_al_common.h"
#include "tcl_io.h"
#include "tcl_version.h"
#include "tcl_help.h"
#include "57712_reg.h"
#include "common_nvm.h"

#define MAX_UART_RETRIES 3
#define WAIT_MS          50

/* tcl script folder definitions */
#define SCRIPTS_FOLDER_OPTION_1 "./../../common/scripts"
#define SCRIPTS_FOLDER_OPTION_2 "./scripts"
#define E3_SCRIPTS_FOLDER_OPTION_1 "./../../common/scripts/e3_scripts"
#define E3_SCRIPTS_FOLDER_OPTION_2 "./scripts/e3_scripts"
#define COMMON_TCL_FOLDER_OPTION_1 "./../lib/tcl8.5"
#define COMMON_TCL_FOLDER_OPTION_2 "./src/edebug/lib/tcl8.5"
#define COMMON_TCL_FOLDER_OPTION_3 "./scripts/tcl8.5"

extern u8_t is_logfile;
extern char *logfile_name;

typedef enum {
        DIAG_TCL_SCRIPTS_FOLDER,
        COMMON_TCL_SCRIPTS_FOLDER
} tcl_folder_location_t;


static char	help_info[MAX_USAGE_LEN];
extern int diag_argc;
extern char **diag_argv;
extern Tcl_Interp *globalInterp;
static u32_t currCmdId = 0;
u32_t uart_com = (u32_t)-1;
u32_t uart_baud = (u32_t)-1;
static char script_to_run[100]="";
static u8_t ctl_c_is_pressed = 0;
static u32_t no_pci_mode = 0;
static u32_t special_log_mode = 0;
virtual_nvm_t virtual_nvm;
edebug_eng_mode_t eng_mode;
extern u8_t g_list;

char tcl_io_getch(void);
static void getCmdHistory(Tcl_Interp *interp, u32_t dir, u32_t *currCmdId, char *retCmd);
static int tcl_al_main_loop();
static int tcl_al_open_com_port(Tcl_Interp *interp, int com_num, int baud);
void tcl_al_set_no_pci_mode(IN   Tcl_Interp * interp, IN   u8_t         no_pci);
void tcl_al_set_special_log_mode(IN   Tcl_Interp * interp);
dbg_status_t tcl_al_set_uart_com(char *com);
dbg_status_t tcl_al_set_uart_baud(char *baud);
dbg_status_t tcl_al_set_virtual_nvm_chip_type(char *char_chip_type);
void tcl_al_set_script_to_run(char *script_name);

/* fw declaration */
u32_t ctl_c_handler(void * params);
dbg_status_t tcl_al_register_api(Tcl_Interp* interp);
dbg_status_t tcl_al_load_scripts(Tcl_Interp *interp);

static void init_global_vars(
    IN   Tcl_Interp* interp
    )
{
    /* set edebug flag */
    Tcl_SetVar2(interp, TCL_SYS_ENV, ENV_IS_EDEBUG, "1", TCL_GLOBAL_ONLY);

    /* Set the array display type to "by dwords" */
    Tcl_SetVar2(interp, TCL_SYS_ENV, ENV_DISP_BYTE, "0", TCL_GLOBAL_ONLY);

    /* Set I/O and DUART serial ports to COM0 */
    Tcl_SetVar2(interp, TCL_SYS_ENV, ENV_DUART_PORT, "0", TCL_GLOBAL_ONLY);


    /* special nvm mode is off by default */
    Tcl_SetVar2(interp, TCL_ENV, ENV_IS_SPECIAL_NVM_MODE, "0", TCL_GLOBAL_ONLY);
    Tcl_SetVar2(interp, TCL_ENV, ENV_SPECIAL_NVM_MODE_FUNC_NUM, "0", TCL_GLOBAL_ONLY);

    #ifdef _WINDOWS
    Tcl_SetVar2(interp, TCL_SYS_ENV, ENV_IS_WINDOWS, "1", TCL_GLOBAL_ONLY);
    #else
    Tcl_SetVar2(interp, TCL_SYS_ENV, ENV_IS_WINDOWS, "0", TCL_GLOBAL_ONLY);
    #endif

    if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
            Tcl_SetVar2(interp, TCL_ENV, ENV_VIRTUAL_NVM_MODE, "1", TCL_GLOBAL_ONLY);
    } else {
            Tcl_SetVar2(interp, TCL_ENV, ENV_VIRTUAL_NVM_MODE, "0", TCL_GLOBAL_ONLY);
    }

    /* By default PCI mode is selected */
    if ( ! Tcl_GetVar2(interp, TCL_SYS_ENV, ENV_NO_PCI, TCL_GLOBAL_ONLY) ) {
        tcl_al_set_no_pci_mode(interp, 0);
    }
}

static int tcl_api_cleanup(
    IN   Tcl_Interp * interp
    )
{
        int stat = TCL_OK;

    if (tcl_al_log_cleanup(interp) != TCL_OK) {
		return TCL_ERROR;
	}

	if (tcl_al_reg_cleanup(interp) != TCL_OK) {
		stat = TCL_ERROR;
	}
        if (tcl_al_device_cleanup(interp) != TCL_OK) {
		stat = TCL_ERROR;
	}

        return stat;
}

static int tcl_al_interactive_get_com_num()
{
       char c = 0;
       int com_num;

       while ((c < 48 || c > 57) && (c != 13) && (c != 10)) {
               printf("\n\n\n"
               "*****************************************************\n"
               "        type com port number to use (one digit)      \n"
               "        default value (empty input) = com 1          \n"
               "*****************************************************\n");
               c = getch();
               if ((c == 13) || (c == 10)) {
                       com_num = 1;
               }
               else {
                       com_num = c - 48;
               }
       }

       return com_num;
}


static int tcl_al_interactive_get_baud_rate()
{
        char c = 0;
        int baud_rate;

        while ((c < 49 || c > 55) && (c != 13) && (c != 10)) {
              printf("\n\n\n"
               "*****************************************************\n"
               "     type baud rate to use:                          \n"
               "     1.  1200                                        \n"
               "     2.  4800                                        \n"
               "     3.  9600                                        \n"
               "     4.  19200                                       \n"
               "     5.  38400                                       \n"
               "     6.  57600                                       \n"
               "     7.  115200                                      \n"
               "                                                     \n"
               "     default value (empty input) = baud rate 115200  \n"
               "*****************************************************\n");
               c = getch();
               switch (c) {
                case 49:
                       baud_rate = 1200;
                       break;
                case 50:
                       baud_rate = 4800;
                       break;
                case 51:
                       baud_rate = 9600;
                       break;
                case 52:
                       baud_rate = 19200;
                       break;
                case 53:
                       baud_rate = 38400;
                       break;
                case 54:
                       baud_rate = 57600;
                       break;
                case 13:
                case 10:
                case 55:
                       baud_rate = 115200;
                       break;
               }
      }

        return baud_rate;
}


int Tcl_AppInit(Tcl_Interp *interp)
{
        int result, code;
        int n_opt;
        Tcl_Command cmd;
        dbg_status_t res;
        dbg_dev_info_t *cur_dev_p;
        char c = 0;
        int i, com_num, baud_rate;
        char return_string[20], cmd_str[256];
        char mac_partition_str[10], sf_mode_str[10];
        u32 port_mode;

        result = Tcl_Init(interp);

        /* parse eDebug command line options */
        res = tcl_al_parse_params(interp, diag_argc, diag_argv, &eng_mode);
        if ( res != DBG_STATUS_SUCCESS )  {
                return TCL_ERROR;
        }

        if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
                if (virtual_nvm.chip_type == CHIP_TYPE_INVALID) {
                        printf("chip_type must be assigned in virtual nvm mode\n");
                        return TCL_ERROR;
                }
                if ((virtual_nvm.nvm_size == 0) && (!virtual_nvm.fd)) {
                        printf("nvm_size or nvm_image must be set in virtual_nvm mode\n");
                        return TCL_ERROR;
                }
        }

        res = tcl_al_register_api(interp);
        if ( res != DBG_STATUS_SUCCESS )  {
                return TCL_ERROR;
        }

        res = tcl_al_load_scripts(interp);
        if ( res != DBG_STATUS_SUCCESS )  {
                return TCL_ERROR;
        }

        res = tcl_al_dbg_init(interp, eng_mode);
        if ( res != DBG_STATUS_SUCCESS )  {
                return TCL_ERROR;
        }

        if (eng_mode != EDEBUG_NO_INIT_MODE) {

                /* show list of valid devices */
                Tcl_Eval(interp, "device");
                res = TCL_ERROR;

                /* choose first valid device */
                for (i = 1; i <= dbg_get_total_br_dev(); i++) {
                        sprintf(cmd_str, "device %d", i);
                        res = Tcl_Eval(interp, cmd_str);
                        if (res == TCL_OK) {
                                break;
                        }
                }

                cur_dev_p = XDBG_CUR_DEV;
                
        } else {
                set_tcl_prompt(interp, eng_mode);
        }
		if (g_list) {
			Tcl_Eval(interp, "exit");
		}

        if (eng_mode == EDEBUG_GRC_DUMP_MODE) {
                if (diag_argc == 1) {
                        code = Tcl_Eval(interp, "grcDump grc_dump.bin\n");
                        if (code == 0) {
                                printf("====================================\n");
                                printf("grcDump output saved to grc_dump.bin\n");
                                printf("====================================\n");
                        }
                } else {
                        sprintf(cmd_str, "%s", "grcDump");
                        for (n_opt = 1; (n_opt < diag_argc) && (diag_argv[n_opt] != NULL) ; n_opt++)
                        {
                                sprintf(cmd_str, "%s %s", cmd_str, diag_argv[n_opt]);
                        }

                        code = Tcl_Eval(interp, cmd_str);
                        if (code == 0) {
                                printf("==========================================\n");
                                printf("grcDump output saved to %s\n", diag_argv[1]);
                                printf("==========================================\n");
                        }
                }
                if (code != 0) {
                        printf("==========================================\n");
                        printf("grcDump failed. Script returned status %d\n", code);
                        printf("==========================================\n");
                }
                sprintf(return_string, "exit %d\n", code);
                Tcl_Eval(interp, return_string);
        }

        if ((res != TCL_OK) && (eng_mode != EDEBUG_NO_INIT_MODE) && (eng_mode != EDEBUG_ENG_NO_UART_MODE)) {
                sc_restore_color();
                while (c != 89 && c != 121 && c!= 78 && c!= 110) {
                        printf("\n\n\n"
                               "*****************************************************\n"
                               "                 no devices were found.              \n"
                               "      Do you want to load in no_pci mode? [Y/N]      \n"
                               "*****************************************************\n");

                        c = getch();
                }

                switch (c) {
                /* typed N */
                case 78:
                case 110:
                        printf("ok... exiting from edebug. good bye\n");
                        Tcl_Eval(interp, "exit");

                /* typed Y */
                case 89:
                case 121:
                      com_num = tcl_al_interactive_get_com_num();
                      baud_rate = tcl_al_interactive_get_baud_rate();
                      uart_com = com_num;
                      uart_baud = baud_rate;
                      tcl_al_reinit_non_pci_mode(interp, com_num, baud_rate);
                }
        }

        if (strlen(script_to_run)) {
                code = Tcl_EvalFile(interp, script_to_run);
                sprintf(return_string, "exit %d\n", code);
                Tcl_Eval(interp, return_string);
        }

        /* internal main_loop should not be called in windows */
        #ifndef _WINDOWS
        return tcl_al_main_loop(interp);
        #else
        return TCL_OK;
        #endif
}

dbg_status_t tcl_al_dbg_init(Tcl_Interp* in_interp, edebug_eng_mode_t eng_mode)
{
        dbg_status_t res;
        u32 port_mode;

        printf("******************************************************************************\n"
               "          edebug -- QLogic Everest debugging tool, version %s\n"
               "******************************************************************************\n",
               VERSION_STRING);

        globalInterp = in_interp;

        /* Attach Ctl-C handler */
        res = dbg_os_ctl_c_attach(ctl_c_handler);
        if (res != DBG_STATUS_SUCCESS) {
                return TCL_ERROR;
        }

        /* Register a cleanup callbacks */
        Tcl_CreateExitHandler(tcl_al_dbg_cleanup, (ClientData)in_interp);

        if (eng_mode == EDEBUG_VIRTUAL_NVM_MODE) {
                virtual_nvm.nvm_buff = malloc(virtual_nvm.nvm_size);
                if (!virtual_nvm.nvm_buff) {
                        printf("failed to allocate buffer in nvm size %d\n", virtual_nvm.nvm_size);
                        return DBG_STATUS_RESOURCE;
                }
                if (virtual_nvm.fd) {
                        fread(virtual_nvm.nvm_buff, virtual_nvm.nvm_size, 1, virtual_nvm.fd);
                        if (virtual_nvm.chip_type == CHIP_TYPE_E3) {
                                common_nvm_read_nvram(NVM_OFFSET(manuf_info.shared_hw_config.wc_lane_config), &port_mode, sizeof(port_mode));
                                switch (port_mode & SHARED_HW_CFG_E3_PORT_LAYOUT_MASK) {
                                case SHARED_HW_CFG_E3_PORT_LAYOUT_2P_01:
                                case SHARED_HW_CFG_E3_PORT_LAYOUT_2P_01_SIG:
                                case SHARED_HW_CFG_E3_PORT_LAYOUT_2P_10:
                                        virtual_nvm.port_mode = PORT_MODE_2_PORTS;
                                        break;
                                case SHARED_HW_CFG_E3_PORT_LAYOUT_4P_0123:
                                case SHARED_HW_CFG_E3_PORT_LAYOUT_4P_1032:
                                case SHARED_HW_CFG_E3_PORT_LAYOUT_4P_2301:
                                case SHARED_HW_CFG_E3_PORT_LAYOUT_4P_3210:
                                        virtual_nvm.port_mode = PORT_MODE_4_PORTS;
                                        break;
                                default:
                                        virtual_nvm.port_mode = PORT_MODE_INVALID;
                                        break;
                                }
                        } else {
                                virtual_nvm.port_mode = PORT_MODE_2_PORTS;
                        }
                } else {
                        memset(virtual_nvm.nvm_buff, 0, virtual_nvm.nvm_size);
                }
        }

         /* TCL globals initialization */
        init_global_vars(in_interp);

        /* init global variables and arrays */
        res = init_global_arrays();
		if ( res != DBG_STATUS_SUCCESS )  {
                return res;
        }

        if ((eng_mode == EDEBUG_ENG_MODE) || (eng_mode == EDEBUG_ENG_NO_UART_MODE)) {
                res = dbg_pci_scan();
                if ( res != DBG_STATUS_SUCCESS ) {
                        return res;
                }
        }
        else if ((eng_mode == EDEBUG_NO_PCI_MODE) || (eng_mode == EDEBUG_VIRTUAL_NVM_MODE)) {
                res = dbg_non_pci_scan();
                if ( res != DBG_STATUS_SUCCESS ) {
                        return res;
                }

        }

        return DBG_STATUS_SUCCESS;
}

u32_t ctl_c_handler(void * params)
{
        /* We don't want Ctl-C handler to be reentrant */
        LOCK();
        if ( ctl_c_is_pressed ) {
        UNLOCK();
        return 0;
        }
        ctl_c_is_pressed = 1;
        UNLOCK();

        tcl_al_dbg_cleanup(globalInterp);

        return 1;
}

void tcl_al_dbg_cleanup(
    IN   ClientData   clientData
    )
{
        Tcl_Interp *interp = (Tcl_Interp *)clientData;

        if (tcl_api_cleanup(interp) != TCL_OK ) {
                printf("BW error\n");
        }

        if (dbg_cleanup() != DBG_STATUS_SUCCESS ) {
                printf("BW error\n");
        }

        /* Disconnect SIGIO handler (needed for Linux MC_DIAG and
                                 Linux DbgBreak handling) */
        //BW dg_os_async_detach();

        sc_restore_color();

        return;
}

static __inline void tcl_al_reg_new_cmd(
	Tcl_Interp     *interp,
	char           *cmd_name,
	Tcl_ObjCmdProc *obj_cmd_proc,
	help_callback_t help_func
	)
{
	u32_t help_len = 0;

	/* Register the command */
	Tcl_CreateObjCommand(interp, cmd_name, obj_cmd_proc, (ClientData)NULL,
			     (Tcl_CmdDeleteProc *)NULL);

        /* Calculate the usage string length of this command */
        help_func(help_info, &help_len);
        /* Add the usage information of this command to the help */
        tcl_al_add_usage_info(cmd_name, help_func, help_len);
}

dbg_status_t tcl_al_register_api(
	Tcl_Interp* interp
	)
{
	tcl_al_reg_new_cmd(interp, DEVICE_CMD, tcl_al_device_cmd,
			   tcl_al_device_usage);

	tcl_al_reg_new_cmd(interp, "dev", tcl_al_device_cmd,
			   tcl_al_device_usage);

        tcl_al_reg_new_cmd(interp, VERSION_CMD, tcl_al_version_cmd,
			   tcl_al_version_usage);

        tcl_al_reg_new_cmd(interp, HELP_CMD, tcl_al_help_cmd,
			   tcl_al_help_usage);

	tcl_al_reg_new_cmd(interp, NVM_CMD, tcl_al_nvm_cmd,
			   tcl_al_nvm_usage);

        /*** Register the "hex16" "hex32" commands to Tcl ***/
	tcl_al_reg_new_cmd(interp, HEX_CMD"16", tcl_al_hex16_cmd,
			   tcl_al_hex_usage);

	Tcl_CreateObjCommand(interp, HEX_CMD"32", tcl_al_hex32_cmd,
						(ClientData)NULL,
						(Tcl_CmdDeleteProc *)NULL);

        /*** Register the "nvm_get_bits" command to Tcl ***/
	Tcl_CreateObjCommand(interp, NVM_GET_BITS, tcl_nvm_get_bits,
						(ClientData)NULL,
						(Tcl_CmdDeleteProc *)NULL);
	/*** Register the "nvm_set_bits" command to Tcl ***/
	Tcl_CreateObjCommand(interp, NVM_SET_BITS, tcl_nvm_set_bits,
						(ClientData)NULL,
						(Tcl_CmdDeleteProc *)NULL);
	/*** Register the "nvm_load" command to Tcl ***/
	Tcl_CreateObjCommand(interp, NVM_LOAD, tcl_nvm_load,
						(ClientData)NULL,
						(Tcl_CmdDeleteProc *)NULL);
	/*** Register the "nvm_store" command to Tcl ***/
	Tcl_CreateObjCommand(interp, NVM_STORE, tcl_nvm_store,
						(ClientData)NULL,
						(Tcl_CmdDeleteProc *)NULL);
	 /*** Register the "nvm_free_structs" command to Tcl ***/
	Tcl_CreateObjCommand(interp, NVM_FREE_STRUCTS, tcl_nvm_free_structs,
						(ClientData)NULL,
						(Tcl_CmdDeleteProc *)NULL);
	/*** Register the "nvm_show_offset" command to Tcl ***/
	Tcl_CreateObjCommand(interp, NVM_SHOW_OFFSET, tcl_nvm_show_nvm_offset,
						(ClientData)NULL,
						(Tcl_CmdDeleteProc *)NULL);

        /*** Register the "getc" command to Tcl ***/
        Tcl_CreateObjCommand(interp, GETC, tcl_getc,
                             (ClientData)NULL,
                             (Tcl_CmdDeleteProc *)NULL);

        /*** Register the "log" command to Tcl ***/
	tcl_al_reg_new_cmd(interp, LOG_CMD, tcl_al_log_cmd,
						tcl_al_log_usage);

        if (eng_mode != EDEBUG_VIRTUAL_NVM_MODE) {
        	tcl_al_reg_new_cmd(interp, REG_CMD, tcl_al_reg_cmd,
        			   tcl_al_reg_usage);

        	tcl_al_reg_new_cmd(interp, PCICFG_CMD, tcl_al_pcicfg_cmd,
        			   tcl_al_pcicfg_usage);

        	tcl_al_reg_new_cmd(interp, ASSERT_CMD, tcl_al_assert_cmd,
        			   tcl_al_assert_usage);

        	tcl_al_reg_new_cmd(interp, ADD_HELP_CMD, tcl_al_add_help_cmd,
        			   tcl_al_add_help_usage);

        	tcl_al_reg_new_cmd(interp, MCP_CMD, tcl_al_mcp_cmd,
        			   tcl_al_mcp_usage);

        	tcl_al_reg_new_cmd(interp, SERPORT_CMD, tcl_al_serport_cmd,
        			   tcl_al_serport_usage);

        	tcl_al_reg_new_cmd(interp, STATS_CMD, tcl_al_stats_cmd,
        			   tcl_al_stats_usage);

        	tcl_al_reg_new_cmd(interp, VPD_CMD, tcl_al_vpd_cmd,
        			   tcl_al_vpd_usage);

        	/*** Register the "elink" command ***/
        	tcl_al_reg_new_cmd(interp, ELINK_CMD, tcl_al_elink_cmd,
        			   tcl_al_elink_usage);

        	/* Register the "hmem" command to Tcl - supported only in windows*/
        	tcl_al_reg_new_cmd(interp, HMEM_CMD, tcl_al_hmem_cmd,
        			   tcl_al_hmem_usage);


        	/*** Register the "shmem_show_offset" command to Tcl ***/
        	Tcl_CreateObjCommand(interp, SHMEM_SHOW_OFFSET, tcl_nvm_show_shmem_offset,
        						(ClientData)NULL,
        						(Tcl_CmdDeleteProc *)NULL);
        }

	return DBG_STATUS_SUCCESS;
}

dbg_status_t tcl_al_load_script(
    Tcl_Interp                  *interp,
    tcl_folder_location_t       folder_location,
    char *                      script
    )
{
        u32_t res = 0;
        char full_script_name[260];

        if (folder_location == DIAG_TCL_SCRIPTS_FOLDER) {
                sprintf(full_script_name, "%s/%s", SCRIPTS_FOLDER_OPTION_1, script);
                res = Tcl_EvalFile(interp, full_script_name);
                sprintf(full_script_name, "%s/%s", SCRIPTS_FOLDER_OPTION_2, script);
                res &= Tcl_EvalFile(interp, full_script_name);
        }
        else if (folder_location == COMMON_TCL_SCRIPTS_FOLDER) {
                sprintf(full_script_name, "%s/%s", COMMON_TCL_FOLDER_OPTION_1, script);
                res = Tcl_EvalFile(interp, full_script_name);
                sprintf(full_script_name, "%s/%s", COMMON_TCL_FOLDER_OPTION_2, script);
                res &= Tcl_EvalFile(interp, full_script_name);
                sprintf(full_script_name, "%s/%s", COMMON_TCL_FOLDER_OPTION_3, script);
                res &= Tcl_EvalFile(interp, full_script_name);
        }


        if ((res == 1) && (eng_mode != EDEBUG_VIRTUAL_NVM_MODE)) {
                printf("warning: could not automatically source %s\n", script);
        }

        return DBG_STATUS_SUCCESS;
}

dbg_status_t tcl_al_load_scripts(
    Tcl_Interp *interp		/* Interpreter to source rc file into. */
    )
{
        /* load diag tcl scripts */
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "regutils.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "idle_chk.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "GRC_dump.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "qstat.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "linkdump.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "phy.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "wol_dump.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "SERIAL.TCL");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "nvm_cfg.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "nvm_meta.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "IguDump.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "vnic_cfg.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "profiler.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "profileCfg.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "pstat.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "stdout.tcl");
        tcl_al_load_script(interp, DIAG_TCL_SCRIPTS_FOLDER, "epio.tcl");

        /* open com port if needed only once */
        if (tcl_al_get_no_pci_mode() == 1) {
                if (tcl_al_open_com_port(interp, uart_com, uart_baud) != DBG_STATUS_SUCCESS) {
                        Tcl_Eval(interp, "exit");
                        return DBG_STATUS_FAILURE;
                }
                printf("Opened com port %d at baud %d successfully!. \n", uart_com, uart_baud);
        }

		/* open commonal tcl scripts */
        tcl_al_load_script(interp, COMMON_TCL_SCRIPTS_FOLDER, "auto.tcl");
        tcl_al_load_script(interp, COMMON_TCL_SCRIPTS_FOLDER, "history.tcl");

        return DBG_STATUS_SUCCESS;
}

void tcl_al_set_special_log_mode(
        IN   Tcl_Interp * interp
    )
{
        char cmd[64];

        special_log_mode = 1;

        /* clear output file */
        sprintf(cmd, "rm -f %s\n", SPECIAL_LOG_FILE);
        system(cmd);
}

dbg_status_t tcl_al_set_virtual_nvm_chip_type(char *char_chip_type)
{
        if ((strcmp(char_chip_type, "E1") == 0) || (strcmp(char_chip_type, "e1") == 0)) {
                virtual_nvm.chip_type = CHIP_TYPE_E1;
        } else if ((strcmp(char_chip_type, "E1x") == 0) || (strcmp(char_chip_type, "e1x") == 0)) {
                virtual_nvm.chip_type = CHIP_TYPE_E1X;
        } else if ((strcmp(char_chip_type, "E2") == 0) || (strcmp(char_chip_type, "e2") == 0)) {
                virtual_nvm.chip_type = CHIP_TYPE_E2;
        } else if ((strcmp(char_chip_type, "E3") == 0) || (strcmp(char_chip_type, "e3") == 0)) {
                virtual_nvm.chip_type = CHIP_TYPE_E3;
        } else {
                return DBG_STATUS_FAILURE;
        }

        return DBG_STATUS_SUCCESS;
}

dbg_status_t tcl_al_set_virtual_nvm_size(char *char_nvm_size)
{
        u32 val;
        if (sscanf(char_nvm_size, "%d", &val) != 1) {
                return DBG_STATUS_FAILURE;
        }
        virtual_nvm.nvm_size = val;
        return DBG_STATUS_SUCCESS;
}

void tcl_al_set_no_pci_mode(
    IN   Tcl_Interp * interp,
    IN   u8_t         no_pci
    )
{
        char buf[3];

        sprintf(buf, "%d", no_pci);
        Tcl_SetVar2(interp, TCL_SYS_ENV, ENV_NO_PCI, buf, TCL_GLOBAL_ONLY);
        no_pci_mode = no_pci;
}

u32_t tcl_al_get_no_pci_mode(
    )
{
        return no_pci_mode;

}

u32_t tcl_al_get_special_log_mode(
    )
{
        return special_log_mode;

}

/******************************************************************************
 * getCmdHistory --
 *
 *     Gets the command history, uses the "history" TCL command
 *
 * Inputs:	dir			- 0 incase of arrow down, 1 incase of arrow up
 *			*currCmdId	- pointer to the current command Id
 *          *retCmd		- pointer to the TCL command to use
 *
 * Results:
 *     none
 ******************************************************************************/

static void getCmdHistory(Tcl_Interp *interp, u32_t dir, u32_t *currCmdId, char *retCmd)
{
        Tcl_Obj *pobj;
        const char *str;
        u32_t rc, latest;

        /* get the current+1 cmd index */
        pobj = Tcl_NewObj();
        Tcl_SetStringObj(pobj, "history nextid", strlen("history nextid"));
        rc = Tcl_EvalObjEx(interp, pobj, TCL_LEAVE_ERR_MSG);
        str = Tcl_GetStringResult(interp);
        latest = atoi(str);
        latest--;

        /* arrow down */
        if (dir == 0)
        {
                //printf("latest=%d curr=%d\n", latest, *currCmdId);
                if (*currCmdId < (latest)) {
                        (*currCmdId)++;
                }

        	/* use TCL command "history event" to get the requested previous command */
                sprintf(retCmd, "history event %i", *currCmdId);
                rc = strlen(retCmd);
                pobj = Tcl_NewObj();
                Tcl_SetStringObj(pobj, retCmd, strlen(retCmd));
                rc = Tcl_EvalObjEx(interp, pobj, TCL_LEAVE_ERR_MSG);
                str = Tcl_GetStringResult(interp);
                if (rc)
                        *retCmd = (char)0;
                else
                        strncpy(retCmd, str, 160);
        }

        /* arrow up */
        else
        {
                if (*currCmdId > 1) {
                        (*currCmdId)--;
                } else if (*currCmdId == 0){
                        *currCmdId = latest;
                }

                /* use TCL command "history event" to get the requested previous command */
                sprintf(retCmd, "history event %i\n", *currCmdId);
                rc = strlen(retCmd);
                pobj = Tcl_NewObj();
                Tcl_SetStringObj(pobj, retCmd, strlen(retCmd));
                rc = Tcl_EvalObjEx(interp, pobj, TCL_LEAVE_ERR_MSG);
                str = Tcl_GetStringResult(interp);
                if (rc)
                        *retCmd = (char)0;
                else
                        strncpy(retCmd, str, 160);
        }
}

static int tcl_al_main_loop(Tcl_Interp *interp)
{
        int length, i, chars_read = 0;
        Tcl_Obj *commandPtr = NULL;
        Tcl_Obj *resultPtr;
        Tcl_Obj *promptCmdPtr;
        Tcl_Channel inChannel, outChannel;
        char myChar, escChar, oldEscChar;
        char history_command[160];
        char *tmp_str;
        char orig_str[160];
        int show_prompt = 1;
        int chars_to_clear = 0;
        int orig_chars_to_clear;
        int tmpVal, commandSent;
        FILE *fd;

        outChannel = Tcl_GetStdChannel(TCL_STDOUT);
        inChannel = Tcl_GetStdChannel(TCL_STDIN);
        commandPtr = Tcl_NewObj();

        while (1) {
                /* show edebug prompt if needed */
                if (show_prompt) {
                        promptCmdPtr = Tcl_GetVar2Ex(interp, "tcl_prompt1", NULL, TCL_GLOBAL_ONLY);
                        Tcl_EvalObjEx(interp, promptCmdPtr, 0);
                        Tcl_Flush(outChannel);
                        show_prompt = 0;
                }

                /* read next char */
                myChar = tcl_io_getch();

                switch (myChar) {
                case ESC_KEY:
                        /* rememeber old command to know how many bytes should be cleared */
                        tmp_str = Tcl_GetString(commandPtr);

                        /* in linux escape characters are 3 chars long, 3rd is the relevant one */
                        escChar = tcl_io_getch();
                        escChar = tcl_io_getch();

                        switch (escChar) {
                        case ARROW_UP:
                                 /* generate new command */
                                commandPtr = Tcl_NewObj();

                                /* get relevant history command */
                                getCmdHistory(interp, 1, &currCmdId, history_command);

                                /* if last command was also history command, clear it from screen */
                                if (oldEscChar == ARROW_UP || oldEscChar == ARROW_DOWN) {
                                        for (i = 0; i < strlen(tmp_str); i++) {
                                                fprintf(stdout, "\b \b");
                                        }
                                        fflush(stdout);
                                }

                                /* print new history command */
                                printf(history_command);
                                chars_read = strlen(history_command);

                                /* add new command to tcl object */
                                if (chars_read > 0) {
                                           Tcl_AppendToObj(commandPtr, history_command, chars_read);
                                }

                                break;

                        case ARROW_DOWN:
                                 /* generate new command */
                                commandPtr = Tcl_NewObj();

                                /* get relevant history command */
                                getCmdHistory(interp, 0, &currCmdId, history_command);

                                /* if last command was also history command, clear it from screen */
                                if (oldEscChar == ARROW_UP || oldEscChar == ARROW_DOWN) {
                                        for (i = 0; i < strlen(tmp_str); i++) {
                                                fprintf(stdout, "\b \b");
                                        }
                                        fflush(stdout);
                                }

                                /* print new history command */
                                printf(history_command);
                                chars_read = strlen(history_command);

                                /* add new command to tcl object */
                                Tcl_AppendToObj(commandPtr, history_command, chars_read);

                                break;

                        case ARROW_LEFT:
                                if (chars_read > 0 && (chars_read - chars_to_clear > 0)) {
                                             fprintf(stdout, "\b");
                                             chars_to_clear++;
                                }

                                break;


                        case ARROW_RIGHT:
                                 if (chars_to_clear == 0) {
                                         printf(" ");
                                         Tcl_AppendToObj(commandPtr, " ", 1);
                                         chars_read++;
                                 }
                                 else {
                                         printf("%c", tmp_str[strlen(tmp_str)-chars_to_clear]);
                                         chars_to_clear--;
                                 }

                                break;

                        case DELETE_KEY:
                                /* should read one more dummy esc char in case of delete */
                                escChar = tcl_io_getch();

                                if (chars_to_clear) {
                                        /* move one char to the left all chars after the cursor*/
                                        for (i = 0; i < strlen(tmp_str); i++) {
                                              if (i >= (strlen(tmp_str) - chars_to_clear)) {
                                                      tmp_str[i]=tmp_str[i+1];
                                              }
                                        }

                                        /* update tcl object according to that change */
                                        commandPtr = Tcl_NewObj();
                                        Tcl_AppendToObj(commandPtr, tmp_str, strlen(tmp_str));

                                        /* go to beginning of the line */
                                        if (strlen(tmp_str) - chars_to_clear != -1) {
                                                for (i = 0; i <= strlen(tmp_str) - chars_to_clear; i++) {
                                                	fprintf(stdout, "\b");
                                                }
                                        }

                                        /* print the new command */
                                        fprintf(stdout, "%s", tmp_str);
                                        fprintf(stdout, " \b");

                                        /* update cursor location */
                                        chars_to_clear--;
                                        chars_read--;

                                        /* go to the right location of the cursor */
                                        for (i = 0; i < chars_to_clear; i++) {
                                             fprintf(stdout, "\b");
                                        }
                                }

                                break;
                        }


                        /* remember old esc char for consecutive history commands */
                        oldEscChar =  escChar;

                        break;

                case BACK_SPACE:
                        /* if there are bytes to clear, clear them from screen
                           and for tcl object */
                         if (chars_read > 0) {
                                     if (chars_to_clear == 0) {
                                             /* if we are at the end of the line, simply remove last char */
											 fprintf(stdout, "\b \b");
                                             tmp_str = Tcl_GetString(commandPtr);
                                             commandPtr = Tcl_NewObj();
                                             Tcl_AppendToObj(commandPtr, tmp_str, strlen(tmp_str)-1);
                                             chars_read--;
                                     }
                                     else {
                                             if (chars_read > chars_to_clear) {
                                                     /* move one char to the left all chars after the cursor*/
                                                     for (i = 0; i < strlen(tmp_str); i++) {
                                                             if (i >= (strlen(tmp_str) - chars_to_clear - 1)) {
                                                                     tmp_str[i]=tmp_str[i+1];
                                                             }
                                                     }

                                                     /* update tcl object after the change */
                                                     commandPtr = Tcl_NewObj();
                                                     Tcl_AppendToObj(commandPtr, tmp_str, strlen(tmp_str));

                                                     /* go to beginning of the line */
                                                     for (i = 0; i <= strlen(tmp_str) - chars_to_clear; i++) {
                                                             fprintf(stdout, "\b");
                                                     }

                                                     /* print new object to command line */
                                                     fprintf(stdout, "%s", tmp_str);
                                                     fprintf(stdout, " \b");

                                                     /* move cursor to right location*/
                                                     for (i = 0; i < chars_to_clear; i++) {
                                                             fprintf(stdout, "\b");
                                                     }

                                                     /* update cursor location */
                                                     chars_read--;
                                             }
                                     }
                        }

                        break;

                case ENTER:
                        /* if object is valid, perform the command */
                        if (strlen(Tcl_GetString(commandPtr))) {
                                Tcl_AppendToObj(commandPtr, "\n", 1);
                                TclObjCommandComplete(commandPtr);
                                fprintf(stdout, "\n");

                                /* Record the whole command to the log file */
                                if (is_logfile) {
                                    fd = os_if_log_fopen(logfile_name);
                                    fprintf(fd, "%s", Tcl_GetString(commandPtr));
                                    fclose(fd);
                                }

                                Tcl_RecordAndEval(interp, Tcl_GetString(commandPtr), 0);
                                oldEscChar = 0;
                                commandSent = 1;
                        }
                        else {
                                fprintf(stdout, "\n");
                        }

                        /* clear variables for next read */
                        chars_read = 0;
                        chars_to_clear = 0;
                        show_prompt = 1;

                        break;

                default:
                        /* read next char, print it to screen*/
                        fprintf(stdout, "%c", myChar);
                        fflush(stdout);

                        if (chars_to_clear > 0) {
                                /* rememeber old command to know how many bytes should be cleared */
                                tmp_str = Tcl_GetString(commandPtr);
                                tmp_str[strlen(tmp_str)-chars_to_clear]=myChar;
                                commandPtr = Tcl_NewObj();
                                Tcl_AppendToObj(commandPtr, tmp_str, strlen(tmp_str));
                                chars_to_clear--;
                        }
                        else {
                                Tcl_AppendToObj(commandPtr, &myChar, 1);
                                chars_read++;
                        }

                        break;
                }

                /* get result from interpreter */
                resultPtr = Tcl_GetObjResult(interp);
                Tcl_GetStringFromObj(resultPtr, &length);

                if (commandSent) {
                        /* empty history index after operation performed */
                        currCmdId = 0;
                }

                /* show result to user */
                if (length > 0 && outChannel && show_prompt && commandSent) {
                        Tcl_WriteObj(outChannel, resultPtr);
                        Tcl_WriteChars(outChannel, "\n", 1);
                        TclObjCommandComplete(commandPtr);
                        commandPtr = Tcl_NewObj();
                }
                /* clear command even if interpreter resonse is empty */
                else if (show_prompt) {
                        commandPtr = Tcl_NewObj();
                }
                commandSent = 0;
        }

        return TCL_OK;
}

static int tcl_al_open_com_port(Tcl_Interp *interp, int com_num, int baud)
{
        int my_com_num = 0;
        int my_baud = 19200;
        char cmd[64];
        u32_t val, i;
        dbg_status_t status;
        int res;

        /* build the command */
        if (com_num == -1) {
                my_com_num = tcl_al_interactive_get_com_num();
                uart_com = my_com_num;
        }
        else {
                my_com_num = com_num;
        }

        if (baud == -1) {
                my_baud = tcl_al_interactive_get_baud_rate();
                uart_baud = my_baud;
        }
        else {
                my_baud = baud;
        }

        /* try to open com port and verify according to a register */
        sprintf(cmd, "set_duart open %d %d", my_com_num, my_baud);

        if ( Tcl_Eval(interp, cmd) != TCL_OK) {
                printf("Failed to use set_duart open.\n");
                return TCL_ERROR;
        } else {
                Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), &res);
        }

        if (res != DBG_STATUS_SUCCESS) {
                printf("Error - set_duart open failed. No serial connection.\n");
                return -1;
        }


        for (i=0; i<MAX_UART_RETRIES; i++) {
                status = dbg_reg_read(MISC_REG_SHARED_MEM_ADDR, &val);
                if (status == DBG_STATUS_SUCCESS) {
                        return 0;
                }
                dbg_msleep(WAIT_MS);
        }

        printf("Error - Failed to read shmem base, close DUART and try again ...\n", status, val);
        sprintf(cmd, "set_duart close %d", my_com_num);
        Tcl_Eval(interp, cmd);

        printf("Did not succeed to open com %d with baud %d for %d times\n", my_com_num, my_baud, MAX_UART_RETRIES);
        printf("Consider changing these values. Exit and try again\n");

        /* update that there are no devices at all */
        Tcl_SetVar2(interp, TCL_ENV, ENV_TOTAL_DEV, "NONE", TCL_GLOBAL_ONLY);
        Tcl_SetVar2(interp, TCL_ENV, ENV_DEV, "0", TCL_GLOBAL_ONLY);
        Tcl_SetVar2(interp, TCL_ENV, ENV_BUS, "0", TCL_GLOBAL_ONLY);

        return -1;
}

dbg_status_t tcl_al_set_uart_com(char *com)
{
        if (1 != sscanf(com, "%d", &uart_com)) {
                return DBG_STATUS_FAILURE;
        }

        return DBG_STATUS_SUCCESS;
}

void tcl_al_set_script_to_run(char *script_name)
{
        strncpy(script_to_run, script_name, 100);
}

dbg_status_t tcl_al_set_uart_baud(char *baud)
{
        if (1 != sscanf(baud, "%d", &uart_baud)) {
                return DBG_STATUS_FAILURE;
        }

        return DBG_STATUS_SUCCESS;
}

void tcl_al_reinit_non_pci_mode(Tcl_Interp *interp, int com_num, int baud_rate)
{
        /* set no pci mode in interpreter */
        tcl_al_set_no_pci_mode(interp, 1);

        /* open com port */
        if (tcl_al_open_com_port(interp, com_num, baud_rate) == DBG_STATUS_SUCCESS) {
                /* should reinit global arrays, now that no_pci mode is set */
                init_global_arrays();

                /* perform non_pci scan */
                dbg_non_pci_scan();

                /* show list of valid devices */
                Tcl_Eval(interp, "device");

                /* choose first valid device */
                Tcl_Eval(interp, "device 1");
        }
}


