#ifndef __DBG_TCL_COMMAND_HANDLERS_H__
#define __DBG_TCL_COMMAND_HANDLERS_H__

/**
 * Description:
 *      Implementation of the "hex16" "hex32" commands.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_hex16_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    );

int tcl_al_hex32_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    );

/**
 * Description:
 *      Add help and usage information on the "hex" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_hex_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);

/**
 * Description:
 *      A function cleaning up "hex" internals.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_hex_cleanup(
    IN   Tcl_Interp *interp
    );

/**
 * Description:
 *      Implementation of the "device" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_device_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    );

/**
 * Description:
 *      Implementation of the "hmem" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_hmem_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    );

/**
 * Description:
 *      A function cleaning up "delay" internals.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_mcp_cmd(
    IN  ClientData client_data, /* Not used. */
    IN  Tcl_Interp* interp,     /* current interpreter */
    IN  int objc,               /* number of arguments */
    IN  Tcl_Obj* CONST objv[]   /* argument strings */
    );

/**
 * Description:
 *      Implementation of the "reg" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_reg_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    );

int tcl_al_assert_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    );

/**
 * Description:
 *      Implementation of the "serport" command.
 * Returns:
 *      TCL_OK if all was okay
 */ 
int
tcl_al_serport_cmd(
    ClientData client_data, /* Not used. */
    Tcl_Interp *interp,     /* current interpreter */
    int objc,               /* number of arguments */
    Tcl_Obj *CONST objv[]   /* argument strings */
        );

int tcl_al_add_help_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    );

/**
 * Description:
 *     Implements "stats" command.
 * 
 * Returns:
 *     A standard Tcl result.
 */ 
int tcl_al_stats_cmd (
    IN    ClientData client_data, /* Not used. */
    IN    Tcl_Interp *interp,     /* current interpreter */
    IN    int objc,               /* number of arguments */
    IN    Tcl_Obj *CONST objv[]   /* argument strings */
    );

int tcl_al_vpd_cmd (
    IN    ClientData client_data, /* Not used. */
    IN    Tcl_Interp *interp,     /* current interpreter */
    IN    int objc,               /* number of arguments */
    IN    Tcl_Obj *CONST objv[]   /* argument strings */
    );

/**
 * Description:
 *      Implementation of the "version" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_version_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    );

/**
 * Description:
 *      Implementation of the "help" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_help_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    );

/**
 * Description:
 *      Implementation of the "nvm" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_nvm_cmd(
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp *interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj *CONST objv[]   /* argument strings */
    );

int tcl_nvm_set_bits(
	IN	 ClientData client_data,
	IN	 Tcl_Interp *interp,
    IN	 int objc,
	IN	 Tcl_Obj *CONST objv[]
	);

int tcl_nvm_get_bits(
	IN	 ClientData client_data, 
	IN	 Tcl_Interp *interp,
    IN	 int objc,
	IN	 Tcl_Obj *CONST objv[]
	);

int tcl_getc(
	IN	ClientData client_data,
	IN	Tcl_Interp *interp,
	IN  int objc, 
	IN	Tcl_Obj *CONST objv[]
	);

int tcl_nvm_load(
	IN	 ClientData client_data,
	IN	 Tcl_Interp *interp,
    IN   int objc, 
	IN	 Tcl_Obj *CONST objv[]
	);

int tcl_nvm_store(
	IN	 ClientData client_data,
	IN	 Tcl_Interp *interp,
    IN   int objc, 
	IN	 Tcl_Obj *CONST objv[]
	);

int tcl_nvm_free_structs(
	IN	 ClientData client_data,
	IN	 Tcl_Interp *interp,
    IN   int objc, 
	IN	 Tcl_Obj *CONST objv[]
	);

int tcl_nvm_show_nvm_offset(
	IN	 ClientData client_data,
	IN	 Tcl_Interp *interp,
    IN   int objc, 
	IN	 Tcl_Obj *CONST objv[]
	);
	
int tcl_nvm_show_shmem_offset(
	IN	 ClientData client_data,
	IN	 Tcl_Interp *interp,
    IN   int objc, 
	IN	 Tcl_Obj *CONST objv[]
	);

/**
 * Description:
 *      Add help and usage information on the "nvm" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_nvm_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);

/**
 * Description:
 *      Add help and usage information on the "device" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_device_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);

/**
 * Description:
 *      Add help and usage information on the "reg" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_reg_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);

int tcl_al_stats_usage(
	IN	char *buf,			//Pointer to the buffer of the command usage
	IN	u32_t *length			//Pointer to the length result of the command usage
	);

int tcl_al_vpd_usage(
	IN	char *buf,			//Pointer to the buffer of the command usage
	IN	u32_t *length			//Pointer to the length result of the command usage
	);

/**
 * Description:
 *      Add help and usage information on the "assert" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_assert_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);

/**
 * Description:
 *      Add help and usage information on the "add_help" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_add_help_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);

/**
 * Description:
 *      Add help and usage information on the "hmem" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_hmem_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);

/**
 * Description:
 *      Add help and usage information on the "version" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_version_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);

/**
 * Description:
 *      Add help and usage information on the "mcp" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_mcp_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);

/*
 *------------------------------------------------------------------------
 * tcl_al_serport_usage --
 *     CB for "help" command to display help
 *
 * Results:
 *     None.
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
int tcl_al_serport_usage(    
    char *buf,
    u32_t *len      
    );
 
/**
 * Description:    
 *      Add help and usage information on the "help" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_help_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);
  /**
 * Description:
 *      Implementation of the "elink" command.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_elink_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    );

/**
 * Description:
 *      Add help and usage information on the "elink"
 * command. Returns: 
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int	tcl_al_elink_usage (
	IN char *buf,		/* Usage char buffer */
    IN u32_t *length	/* Usage char buffer length */
	);

/**
 * Description:
 *      A function cleaning up "elink" internals.
 * Returns:
 *      TCL_OK if all was okay
 *      TCL_ERROR otherwise
 */ 
int tcl_al_elink_cleanup(
    IN   Tcl_Interp *interp      /* current interpreter */
    );

/**
 * Description:
 *     Implements "pcicfg" command.
 * 
 * Returns:
 *     A standard Tcl result.
 */ 
int tcl_al_pcicfg_cmd (
    IN    ClientData client_data, /* Not used. */
    IN    Tcl_Interp *interp,     /* current interpreter */
    IN    int objc,               /* number of arguments */
    IN    Tcl_Obj *CONST objv[]   /* argument strings */
    );

/*
 *------------------------------------------------------------------------
 * tcl_al_pcicfg_usage --
 *     Add usage information on "pcicfg" command 
 *------------------------------------------------------------------------
 */
int tcl_al_pcicfg_usage(
	IN	char *buf,				//Pointer to the buffer of the command usage
	IN	u32_t *length			//Pointer to the length result of the command usage
	);


int tcl_al_log_cmd(
    ClientData client_data, /* Not used. */
    Tcl_Interp *interp,     /* current interpreter */
    int objc,               /* number of arguments */
    Tcl_Obj *CONST objv[]   /* argument strings */
    );

int tcl_al_log_usage(
	IN	char *buf,				//Pointer to the buffer of the command usage
	IN	u32_t *length			//Pointer to the length result of the command usage
	);

int tcl_al_log_cleanup(
    Tcl_Interp *interp
    );

#endif
