#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tcl.h"
#include "tclDecls.h"
#include "tcl_al.h"
#include "man_types.h"
#include "tcl_al_common.h"
#include "util.h"
#include "disputil.h"
#include "dbg_man.h"
#include "license.h"
#include "mcp_shmem.h"
#include "clc.h"
struct elink_params link_params;
struct elink_vars link_vars;

u32 elink_cb_reg_read(struct elink_dev *cb, u32 reg_addr)
{
	u32_t val, retries = 10000;
	dbg_status_t rc;
	while (--retries > 0) {
		rc = dbg_reg_read(reg_addr, &val);
		if (rc == 0)
			break;
	}
	if (rc) {
		printf("FAILED TO READ FROM DEVICE !!!!\n");
	} else {
	//	printf("elink_cb_reg_read addr 0x%x, val 0x%x\n", reg_addr, val);
	}
	return (u32)val;
}

void elink_cb_reg_write(struct elink_dev *cb, u32 reg_addr, u32 val)
{
	dbg_status_t rc;
	u32_t retries = 10000;
	while (--retries > 0) {
		rc = dbg_reg_write((u32_t)reg_addr, (u32_t)val, 0);
		if (rc == 0)
			break;
	}
	if (rc) {
		printf("FAILED TO READ FROM DEVICE !!!!\n");
	} else {
//		printf("elink_cb_reg_write addr 0x%x, val 0x%x\n", reg_addr, val);
	}
}

/* wb_write - pointer to 2 32 bits vars to be passed to the DMAE*/
void elink_cb_reg_wb_write(struct elink_dev *cb, u32 offset,
			   u32 *wb_write, u16 len)
{
	elink_cb_reg_write(cb, offset, wb_write[0]);
	elink_cb_reg_write(cb, offset, wb_write[1]);
}
void elink_cb_reg_wb_read(struct elink_dev *cb, u32 offset,
			  u32 *wb_write, u16 len)
{
	wb_write[0] = elink_cb_reg_read(cb, offset);
	wb_write[1] = elink_cb_reg_read(cb, offset + 4);
}

/* mode - 0( LOW ) /1(HIGH)*/
u8 elink_cb_gpio_write(struct elink_dev *cb,
		       u16 gpio_num,
		       u8 mode, u8 port)
{
	printf("Unhandled GPIO write function !!! gpio_num %d, mode %d, port %d\n", gpio_num, mode, port);
	return 0;
}


u32 elink_cb_gpio_read(struct elink_dev *cb, u16 gpio_num, u8 port)
{
	printf("Unhandled GPIO read function !!! gpio_num %d, port %d\n", gpio_num, port);
	return 0;
}
u8 elink_cb_gpio_int_write(struct elink_dev *cb,
			   u16 gpio_num,
			   u8 mode, u8 port)
{
	printf("Unhandled GPIO int write function !!! gpio_num %d, port %d\n", gpio_num, port);
	return 0;
}

u32 elink_cb_fw_command(struct elink_dev *cb, u32 command, u32 param)
{
	return 0;
}

/* Delay */
void elink_cb_udelay(struct elink_dev *cb, u32 usec)
{
	dbg_usleep((u32_t)usec);
}

void elink_cb_event_log_unqual_io_module(struct elink_dev *cb, const char* vendor_name, const char* vendor_pn)
{
	return;
}
void elink_cb_event_log_over_current(struct elink_dev *cb)
{
	return;
}
void elink_cb_event_log(struct elink_dev *cb)
{
	return;
}

u8 elink_cb_path_id(struct elink_dev *cb)
{
	u32_t path;
	dbg_get_path_num(&path);
	return (u8)path;
}

/* Debug prints */
void elink_cb_dbg(struct elink_dev *cb, char *fmt)
{
	printf(fmt);
}
void elink_cb_dbg1(struct elink_dev *cb, char *fmt, u32 arg1)
{
	printf(fmt, arg1);
}
void elink_cb_dbg2(struct elink_dev *cb, char *fmt, u32 arg1, u32 arg2)
{
	printf(fmt, arg1, arg2);
}
void elink_cb_dbg3(struct elink_dev *cb, char *fmt, u32 arg1, u32 arg2, u32 arg3)
{
	printf(fmt, arg1, arg2, arg3);
}

void link_init_wrapper(unsigned int i_loopback_mode,
                       unsigned int i_duplex,
                       unsigned int i_speed,
		       unsigned int i_speed_cap_mask)
{
	int stat;
	u32_t chip_num, path;
	u32 shmem_base[2], shmem2_base[2];
	u32 other_shmem, other_shmem2;
	dbg_get_path_num(&path);

	if (dbg_reg_read(/*MISC_REG_CHIP_NUM*/ 0xa408, &chip_num) != DBG_STATUS_SUCCESS ) {
		printf("Failed to read chip number from UART");
		return;
        }
	dbg_get_port_num((u32_t*)&link_params.port);
	printf("port = 0x%x\n", link_params.port);
	link_params.loopback_mode = (u8)i_loopback_mode;
        link_params.req_duplex[0] = (u16)i_duplex;
	link_params.req_flow_ctrl[0] = 0x300;
	link_params.req_line_speed[0] = (u16)i_speed;
	dbg_reg_read(/*MISC_REG_SHARED_MEM_ADDR */ 0xa2b4, (u32_t*)&link_params.shmem_base);
	dbg_reg_read(/* MISC_REG_GENERIC_CR_0 */ 0xa460 + path*4, (u32_t*)&link_params.shmem2_base);

	dbg_reg_read((u32_t)link_params.shmem2_base + OFFSETOF(shmem2_region_t,other_shmem_base_addr), (u32_t*)&other_shmem);
	dbg_reg_read((u32_t)link_params.shmem2_base + OFFSETOF(shmem2_region_t,other_shmem2_base_addr), (u32_t*)&other_shmem2);
	if (path == 0) {
		shmem_base[0] = link_params.shmem_base;
		shmem2_base[0] = link_params.shmem2_base;
		shmem_base[1] = other_shmem;
		shmem2_base[1] = other_shmem2;
	} else {
		shmem_base[1] = link_params.shmem_base;
		shmem2_base[1] = link_params.shmem2_base;
		shmem_base[0] = other_shmem;
		shmem2_base[0] = other_shmem2;
	}
	printf("shmem[0]=0x%x, shmem2[0]=0x%x, shmem[1]=0x%x, shmem2[1]=0x%x\n", shmem_base[0],shmem2_base[0], shmem_base[1], shmem2_base[1]);
        link_params.speed_cap_mask[0] = i_speed_cap_mask;
	link_params.switch_cfg = 0x01000000; // PORT_FEATURE_CON_SWITCH_10G_SWITCH
	link_params.lane_config = 0;
	link_params.chip_id = (chip_num<<16); //
	link_params.feature_config_flags = 0;
        link_params.hw_led_mode = 1;
	link_params.multi_phy_config = 0;
        link_params.cb = NULL;
	link_params.req_fc_auto_adv = 0x300;


	/* Initialize EMAC register for MDC/MDIO accesss (Usually been done by bootcode) */
	elink_cb_reg_write(NULL, 0xa598, 3<<2);
	elink_cb_reg_write(NULL, 0xa594, 3<<2);

	elink_common_init_phy(NULL, shmem_base, shmem2_base, (u32)link_params.chip_id, 0);

	stat =  elink_phy_probe(&link_params);
	if (stat!=0) {
		printf("Error in elink_phy_probe\n");
		return;
	}
	printf("elink_phy_probe completed successully\n");
	if (elink_link_reset(&link_params, &link_vars, 0) != 0) {
		printf("Error in elink_phy_init\n");
		return;
	}
	printf("elink_link_reset completed successully\n");
	if (elink_phy_init(&link_params, &link_vars) != 0) {
		printf("Error in elink_phy_init\n");
		return;
	}
	printf("elink_phy_init completed successully\n");
}

unsigned int link_update_wrapper(void) {
	u32_t timeout = 10;
	u32_t nig_status;
	u32_t reg = 0x10328 + link_params.port*4;
	while (timeout > 0 ) {
		dbg_reg_read(reg, &nig_status);
		if (nig_status & 0x3c8019) {
			printf("nig_status = 0x%x reg 0x%x\n", nig_status, reg);
			if (elink_link_update(&link_params, &link_vars) != 0) {
				printf("Error in elink_link_update\n");
				return 0;
			}
		}
		dbg_msleep(5);
		timeout-=5;
	}

	printf("link_update_wrapper: link is %s\n", (link_vars.link_up) ? "up" : "down");

	return link_vars.link_up;
}

void link_reset(void) {
	if (elink_link_reset(&link_params, &link_vars, 0) != 0) {
		printf("Error in elink_phy_init\n");
		return;
	}
}

//static void usage_loopback(Tcl_Interp *interp)
//{
//   Tcl_WrongNumArgs(interp, 0, NULL, 	"elink loopback <loopback_type>\n");
//}

static void usage_init(Tcl_Interp *interp)
{
    Tcl_WrongNumArgs(interp, 0, NULL, 	"elink init <speed> [duplex] [loopback]");
}

int tcl_al_elink_cmd (
    IN   ClientData client_data, /* Not used. */
    IN   Tcl_Interp* interp,     /* current interpreter */
    IN   int objc,               /* number of arguments */
    IN   Tcl_Obj* CONST objv[]   /* argument strings */
    )
{
    static char *option_names[] = { "init", "check_link",  (char *)NULL };
    static char *speed_names[] = {"10M", "100M", "1000M", "10G", "20G" ,"AUTONEG", (char *)NULL};
    static char *duplex_names[] = {"FULL", "HALF"};
    static char *loopback_names[] = {"XMAC_LB", "UMAC_LB", "EXT_PHY_LB"};

    enum options {ELINK_INIT, ELINK_CHECKLINK};
    enum speed {SPEED_10M, SPEED_100M, SPEED_1000M, SPEED_10G, SPEED_20G ,SPEED_AUTONEG};
    enum duplex {FULL, HALF};
    enum loopback {XMAC, UMAC, PHY};
    int cmd_idx, speed_idx, duplex_idx, loopback_idx;
    u32 my_speed, my_duplex, my_loopback;

    tcl_al_log_command(ELINK_CMD, objc, objv);

    /**
     * Get op. type - find the op. index in option_names array
     */
    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 0, NULL, "elink init|check_link");
        return TCL_ERROR;
    }
    if (Tcl_GetIndexFromObj(interp, objv[1], (const char**)option_names, "option", TCL_EXACT,
                            &cmd_idx) != TCL_OK) {
        //usage(interp);
        return TCL_ERROR;
    }


    switch (cmd_idx) {
    case ELINK_INIT:
	/**
	 * Check the number of arguments - must be 4 (elink init
	 * <speed> <speed_cap_mask> <serdes_if> <duplex>
	 * <loopback_mode>)
	 */

	    if ( objc < 3 ) {
		    usage_init(interp);
		    return TCL_ERROR;
	    }

	    /* Get the speed */
	    if (Tcl_GetIndexFromObj(interp, objv[2], (const char**)speed_names, "speed", TCL_EXACT,
				    &speed_idx) != TCL_OK) {
		    usage_init(interp);
		    return TCL_ERROR;
	    }
	    switch (speed_idx) {
	    case SPEED_10M:
		    my_speed = ELINK_SPEED_10;
		    break;
	    case SPEED_100M:
		    my_speed = ELINK_SPEED_100;
		    break;
	    case SPEED_1000M:
		    my_speed = ELINK_SPEED_1000;
		    break;
	    case SPEED_10G:
		    my_speed = ELINK_SPEED_10000;
		    break;
	    case SPEED_20G:
		    my_speed = ELINK_SPEED_20000;
		    break;
	    case SPEED_AUTONEG:
		    my_speed = ELINK_SPEED_AUTO_NEG;
		    break;
	    default:
		    //usage(interp);
		    return TCL_ERROR;
	    }
	    /* Get the speed */
	    if ( objc >= 4 ) {
		    if (Tcl_GetIndexFromObj(interp, objv[3], (const char**)duplex_names, "duplex", TCL_EXACT,
					    &duplex_idx) != TCL_OK) {
			    usage_init(interp);
			    return TCL_ERROR;
		    }
	    } else
		    duplex_idx = FULL;
	    switch (duplex_idx) {
	    case FULL:
		    my_duplex = DUPLEX_FULL;
		    break;
	    case HALF:
		    my_duplex = DUPLEX_HALF;
		    break;
	    default:
		    // usage(interp);
		    return TCL_ERROR;
	    }

	    if (objc >=5 ) {
		    if (Tcl_GetIndexFromObj(interp, objv[4], (const char**)loopback_names, "loopback", TCL_EXACT,
					    &loopback_idx) != TCL_OK) {
			    usage_init(interp);
			    return TCL_ERROR;
		    }
	    } else
		    loopback_idx = 0;

	    switch (loopback_idx) {
	    case XMAC:
		    my_loopback = ELINK_LOOPBACK_XMAC;
		    break;
	    case UMAC:
		    my_loopback = ELINK_LOOPBACK_UMAC;
		    break;
	    case PHY:
		    my_loopback = ELINK_LOOPBACK_XGXS;
		    break;
	    default:
		    my_loopback = ELINK_LOOPBACK_NONE;
		    break;
	    }
	    link_init_wrapper(my_loopback, my_duplex, my_speed, 0xffffffff);
	    link_update_wrapper();
	    break;
    case ELINK_CHECKLINK:
	    link_update_wrapper();
	    break;
    default:
	    break;
    }

    return TCL_OK;

}

int tcl_al_elink_cleanup(
    IN   Tcl_Interp *interp
    )
{
    return Tcl_DeleteCommand(interp, ELINK_CMD);
}

/*
 *------------------------------------------------------------------------
 * tcl_al_bits_usage --
 *     Add usage information on "bits" command
 *------------------------------------------------------------------------
 */
int tcl_al_elink_usage(
	IN	char *buf,				//Pointer to the buffer of the command usage
	IN	u32_t *length			//Pointer to the length result of the command usage
	)
{
	u32_t offset = 0;
	*length = 0;

	*length += sprintf( &buf[offset+*length], "Test link over UART\n\n");
	*length += sprintf( &buf[offset+*length], "Usage: elink init <speed> [duplex] [loopback_mode]\n" );
	*length += sprintf( &buf[offset+*length], "       elink check_link\n" );

    return TCL_OK;
}
