#include <string.h>
#include <time.h>
#include <sys/types.h>

#include "tcl.h"
#include "bcmtype.h"
#include "tcl_al_cmd_handlers.h"
#include "tcl_al.h"

#define MAX_FILE_NAME_LEN 30

u8_t is_logfile = 0;
char *logfile_name;


// Prototypes
static u32_t openfile_cmd (ClientData client_data, Tcl_Interp *interp, 
							       int objc, Tcl_Obj *CONST objv[]);
static u32_t closefile_cmd (ClientData client_data, Tcl_Interp *interp, 
									int objc, Tcl_Obj *CONST objv[]);

/**
 *     Implements the Tcl "log" command.
 */
int tcl_al_log_cmd(client_data, interp, objc, objv)
    ClientData client_data; /* Not used. */
    Tcl_Interp *interp;     /* current interpreter */
    int objc;               /* number of arguments */
    Tcl_Obj *CONST objv[];  /* argument strings */
{
    static char *option_names[] = {"open", "close", (char *)NULL };
    enum options {OPEN, CLOSE};
    int index;   
	
    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "option= open|close");
        return TCL_ERROR;
    }

    if (Tcl_GetIndexFromObj(interp, objv[1], (const char**)option_names, "option", TCL_EXACT, 
                            &index) != TCL_OK) {
        return TCL_ERROR;
    }
	// Handle the sub-command accordingly
    switch ((enum options)index) {
        case OPEN:
            return openfile_cmd(client_data, interp, objc-1, objv+1);
            break;
        case CLOSE:
            return closefile_cmd(client_data, interp, objc-1, objv+1);
            break;       
        default:
            return TCL_ERROR;
    }
}

/**
 *     Implements the Tcl "log open" command.
 */
static u32_t openfile_cmd (ClientData client_data, Tcl_Interp *interp, 
									int objc, Tcl_Obj *CONST objv[])
{
    Tcl_Obj *result_ptr = NULL;
    u32_t len, index;    
    char *option_str;
    static char logFile[MAX_FILE_NAME_LEN];
    time_t time_now;    
    char    tmp_str[80];
   
    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "?filename?");
        return TCL_ERROR;
    }

    result_ptr = Tcl_GetObjResult(interp);

    logfile_name = Tcl_GetStringFromObj(objv[1], (int *)&len);

    if (is_logfile != 0)
    {
      printf("A log file is already opened, close it first.\n");
      return(TCL_ERROR);
    }

	// open the log file
    sprintf(tmp_str, "stdout off %s", logfile_name);    
    Tcl_Eval(interp, tmp_str);
    is_logfile=1;

    // set the tcl prompt to use the "__puts" command
    set_tcl_prompt(interp, EDEBUG_ENG_MODE);   
  
	// TBD: add the current time to the log file    
    
	return TCL_OK;
	
}


/**
 *     Implements the Tcl "log close" command.
 */
static u32_t closefile_cmd (ClientData client_data, Tcl_Interp *interp, 
									int objc, Tcl_Obj *CONST objv[])
{
    Tcl_Obj *result_ptr = NULL;

    if (objc !=1) {
        Tcl_WrongNumArgs(interp, 1, objv, "too many arguments");
        return TCL_ERROR;
    }

    result_ptr = Tcl_GetObjResult(interp);   
    Tcl_Eval(interp, "stdout on"); 
    is_logfile = 0;

    // set the tcl prompt to use the "puts" command
    set_tcl_prompt(interp, EDEBUG_ENG_MODE);
    
    return TCL_OK;
}


/**
 * Add "log" command cleanup
 */
int tcl_al_log_cleanup(interp)
    Tcl_Interp *interp;     
{   
    /* De-register the command from Tcl */
    Tcl_DeleteCommand(interp, LOG_CMD);
    return TCL_OK;
}

/**
 * Add usage information on "log" command
 */
int tcl_al_log_usage(
	IN	char *buf,				//Pointer to the buffer of the command usage
	IN	u32_t *length			//Pointer to the length result of the command usage
	)
{	
	u32_t offset = 0;
	*length = 0;

	*length += sprintf( &buf[offset+*length], "Opens a log file and starts logging the display to this file\n\n");    
    *length += sprintf( &buf[offset+*length], "Usage: log open <logfile>\n" );
    *length += sprintf( &buf[offset+*length], "       log close\n" );    
    	  
    return TCL_OK;
}
