/****************************************************************************
* Copyright(c) 2001-2014 Broadcom Corporation, all rights reserved
* Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
* Proprietary and Confidential Information.
*
* This source file is the property of QLogic Corporation, and
* may not be copied or distributed in any isomorphic form without 
* the prior written consent of QLogic Corporation. 
*
* FILE NAME:       tcl_mcp.h
*
* DESCRIPTION: 
*
* CONVENTIONS:
*
* AUTHOR:          David Lau
*
* CREATION DATE:     
*
* REVISION HISTORY
*
*   When        Who     What
*   -------------------------------------------------------------------------
****************************************************************************/
#ifndef __BROADCOM_XDBG_TCL_MCP_H__
#define __BROADCOM_XDBG_TCL_MCP_H__


#include "ncsi_basic_types.h"
#include "ncsi_cmds.h"
#include "multi_thread_def.h"

#define CHIP_NUM_57712              0x1662
#define  EVEREST_CHIP_NUM   CHIP_NUM_57712

typedef u32_t  nig_grc_reg_t;

typedef struct
{
    nig_grc_reg_t LoAddr;
    nig_grc_reg_t HiAddr;
}nig_dest_mac_addr_t;

typedef  struct _nig_reg_t{
   nig_dest_mac_addr_t const * DEST_MAC;
   nig_grc_reg_t        MCP_MASK;
   nig_grc_reg_t        VLAN_ID[NCSI_VLAN_TAG_COUNT];
   nig_grc_reg_t        EGRESS_FIFO_EMPTY;
   nig_grc_reg_t        EGRESS_IN_EN;
   nig_grc_reg_t        INGRESS_IN_EN;  
   nig_grc_reg_t        INGRESS_OUT_EN;  
   nig_grc_reg_t        BRB_MCP_MASK;
#if (EVEREST_CHIP_NUM != CHIP_NUM_57710)
   nig_grc_reg_t        MCP_MASK_MF;
   nig_grc_reg_t        BRB_MCP_MASK_MF;
#endif
#if (EVEREST_CHIP_NUM == CHIP_NUM_57712)
    // Following is only available for new features in E2 (e.g. Host-BMC interface)
   nig_grc_reg_t        HOST_FWD_MCP_MASK;
   nig_grc_reg_t        HOST_FWD_MCP_MASK_EXT;
   nig_grc_reg_t        HOST_EGRESS_FIFO_EMPTY;
   nig_grc_reg_t        BRB_MCP_MASK_A; // for MAC4 and MAC5
   nig_grc_reg_t        MCP_MASK_A;   // for MAC4 and MAC5
   nig_grc_reg_t        HOST_FWD_NTWK_DRV_MASK;
   nig_grc_reg_t        HOST_FWD_NTWK_MCP_MASK;
   nig_grc_reg_t        HOST_DONT_FWD_NTWK_DRV_MASK;
   nig_grc_reg_t        HOST_DONT_FWD_NTWK_MCP_MASK;
   nig_grc_reg_t        TX_MNG_DESTINATION;         
   nig_grc_reg_t        TX_MNG_HOST_CTRL;           
   nig_grc_reg_t        TX_MNG_HOST_OUTER_VLAN_MSB; 
   nig_grc_reg_t        TX_MNG_HOST_FIFO_TM;        
   nig_grc_reg_t        TX_MNG_HOST_ENABLE;         
   nig_grc_reg_t        TX_HOST_MNG_ENABLE;   
   nig_grc_reg_t        GLOBAL_ERROR_MASK;
   nig_grc_reg_t        FIFO_EMPTY;
   nig_grc_reg_t        FIFO_FULL;
   nig_grc_reg_t        RSLT_BUF_FULL;
   nig_grc_reg_t        TLLH_MNG_PKT_RCVD;
   nig_grc_reg_t        TLLH_MNG_OCTETS_RCVD;
   nig_grc_reg_t        HBUF_PACKET_DISCARD;
   nig_grc_reg_t        MNG_HOST_PACKET_OUT;
   nig_grc_reg_t        MNG_HOST_OCTET_OUT;
#endif
   nig_grc_reg_t        MNG_PACKET_INP; 
   nig_grc_reg_t        MNG_OCTET_INP;  
   nig_grc_reg_t        MNG_DISCARD;    
   nig_grc_reg_t        MNG_PACKET_OUT; 
   nig_grc_reg_t        MNG_OCTET_OUT;  
}nig_reg_t;


// Compute channel ID based on port and path ID
#define COMPUTE_CHNL(path, port) (is_e1x(globalInterp) ? (port): ((port) * 2 + (path)))

extern void DecodeMcpGrcRegDetail(Tcl_Obj* rc_ptr, u32_t addr, u32_t data, u32_t help);
extern void DecodeMcpFioRegDetail(Tcl_Obj* rc_ptr, u32_t addr, u32_t data, u32_t help);
extern void DecodeMfCfg(Tcl_Obj* rc_ptr, const struct mf_cfg * data);
extern void DecodeShmemRegion(Tcl_Obj* rc_ptr, const struct shmem_region * data);
extern void DecodeShmem2Region(Tcl_Obj* rc_ptr, const struct shmem2_region * data);
extern void DecodeRetainAddr(Tcl_Obj* rc_ptr, u32_t path, const u32_t * data);
extern void DecodeNcsiChannelData(Tcl_Obj* rc_ptr, const NcsiChannelData_t * data);
extern void DecodeTemperature(Tcl_Obj* rc_ptr, const struct shmem2_region * data);
extern void DecodeNcsiPersistentData(Tcl_Obj* rc_ptr, const NcsiPersistentData_t * data);
extern void DecodeNcsiCmdPkt(Tcl_Obj* rc_ptr, const NcsiRmiiCmdPkt_t * data);
extern void DecodeNcsiRmiiCmdRspPkt(Tcl_Obj* rc_ptr, const NcsiRmiiCmdRspPkt_t * data);
extern void DecodeScheduler(Tcl_Obj* rc_ptr, const struct multi_thread_t * data, const u32 stack_depth[]);

#endif
