
#ifndef __BROADCOM_EDBG_TCL_UTIL_H__
#define __BROADCOM_EDBG_TCL_UTIL_H__

#ifndef FWUPGTOOL
#include "os_types.h"
#endif // !FWUPGTOOL
#include "esc_seq.h"
#include "errcode.h"
#include "man_types.h"
#include "common_nvm.h"
#include "tcl_al.h"

// #include "tcl_al_if.h"

int print_dbg(char *fmt, ...);
#define printf print_dbg

#define MAX_PRINTF_BUFFER       16384

/* With the format XX:XX:XX:XX:XX:XX, mac address requires 17 bytes. */
#define MAC_ADDR_SIZE_IN_HEX          17

/* MAC addresses types */
#define ADDR_TYPE_PRIMARY              0
#define ADDR_TYPE_ISCSI                1
#define ADDR_TYPE_RDMA                 2

/* Images types */
#define BC_IMAGE					   0
#define BC1_IMAGE					   1	
#define BC2_IMAGE					   2
#define IPMI_IMAGE					   3
#define UMP_IMAGE					   4
#define MBA_IMAGE					   5
#define IB_IMAGE					   6
#define L2B_IMAGE					   7
#define IB_IPV6_IMAGE				   8
#define IB_IPV4N6_IMAGE				   9
#define NCSI_IMAGE				   	   10
#define MBI_IMAGE					   11

/* NVM CFG options types */
#define MAC_OPTION						IDX_L2_MAC_ADDRESS
#define SERIAL_NUMBER_OPTION			IDX_SERIAL_NUM
#define WOL_OPTION						IDX_WOL_MAGIC_ENABLE
#define MBA_OPTION						IDX_MBA_ENABLE
#define MBA_PROTOCOL_OPTION				IDX_MBA_BOOT_AGENT
#define MFW_OPTION						IDX_MFW_ENABLE
#define ISCSI_MAC_OPTION				IDX_ISCSI_MAC
#define RDMA_MAC_OPTION					IDX_RDMA_MAC
#define MBA_VLAN_OPTION					IDX_MBA_VLAN_ENABLE
#define MBA_VLAN_VAL_OPTION				IDX_MBA_VLAN_VALUE
#define PN_OPTION						IDX_PART_NUM
#define MBA_BOOT_RETRY_OPTION			IDX_MBA_BOOT_RETRY
#define DCBX_OPTION						IDX_DCBX_ENABLE
#define IFFE_OPTION						IDX_IFFE
#define REV_ID_CTRL_OPTION				IDX_REV_ID_CTRL

#ifndef FWUPGTOOL

#endif // !FWUPGTOOL

/**
 * Description:
 *      Sets xDiag's prompt.  
 * 
 * Returns:
 *      void
 */ 
void set_tcl_prompt(
    IN   Tcl_Interp* interp,   /* current interpreter */
    IN   edebug_eng_mode_t eng_mode
    );

/**
 * Description:
 * Convert the lower three bytes of the MAC address to hex number 
 */
u32_t mac_to_hex (
	IN char *mac_str
	);

/**
 * Description:
 * Check that mac & iscsi mac addresses are not idential.
 * Check also that the mac addresses are sequential. 
 */
u32_t check_mac_addr(
	IN Tcl_Interp *interp,		 /* Tcl interperter */
	IN char *mac_addr_p,		 /* pointer to the primary MAC address string */
	IN char *iscsi_mac_addr_p,   /* pointer to the iscsi MAC address string */
	OUT errcode_t* err_code		 /* the returned error code */
	);

void generate_new_mac(
	IN	char *src_mac_str, 
	OUT char *new_mac_str, 
	IN	u8_t incr);

/**
 * Description:
 * Program the requested MAC addresses (primary,iScsi,rdma) on the requested 
 * device using the prg_mac_addr utility.
 */
u32_t program_mac_addresses(
	IN Tcl_Interp *interp,			/* Tcl interperter */
	IN char *mac_addr_str,			/* pointer to the primary MAC address string */
	IN char *iscsi_mac_addr_str,	/* pointer to the iscsi MAC address string */
	IN char *rdma_mac_addr_str,		/* pointer to the rdma MAC address string */
	IN u8_t dev_idx,				/* device number */
	OUT errcode_t* err_code			/* the returned error code */
	);
 
/**
 * Description:
 * Program the requested MAC address (primary/iScsi/rdma) on the requested 
 * device using "nvm cfg" command options.
 */
u32_t prg_mac_addr(
	IN Tcl_Interp *interp,		/* Tcl interperter */
	IN char *mac_addr_str,		/* pointer to the input MAC address string */
	IN u8_t mac_addr_type,		/* ADDR_TYPE_PRIMARY or ADDR_TYPE_ISCSI or ADDR_TYPE_RDMA */
	IN u8_t dev_idx				/* device number */
	);

/**
 * Description: 
 * upgrade nvm bootcode file 
 */
u32_t nvm_upgrade(
	IN Tcl_Interp *interp,		/* Tcl interperter */
	IN u8_t image_type,			/* BC or BC1 or BC2 */
	IN char *image_name,		/* boot code file name */
	IN u8_t force_upgrade,		/* Use -F or not */
	IN u8_t ibc,				/* Incase of Iscsi boot code, program configuration block0 or not*/
	IN u8_t ibc2,				/* Incase of Iscsi boot code, program configuration block1 or not */
	IN u8_t ibp  				/* Incase of Iscsi boot code, program configuration utility or not */
	);



/**
 * Description: 
 * Configure nvm option
 */
u32_t nvm_cfg(
	IN Tcl_Interp *interp,		/* Tcl interperter */
	IN u8_t option_num,			/* Option number of the option to configure */
	IN u8_t option_value		/* Option value to configure */
	);

/**
 * Description: 
 * Get nvm option
 */
int nvm_cfg_get(
    IN Tcl_Interp *interp,
    IN u8_t option_num   
    );

/**
 * Description: 
 * Configure nvm option of serial number
 */
u32_t nvm_cfg_serial_no(
	IN Tcl_Interp *interp,			/* Tcl interperter */
	IN char *serial_no_p			/* Pointer to serial number string */
	);

/**
 * Description: 
 * Program the NVM
 * Configure nvm option of serial number
 */ 
u32_t nvm_prg(Tcl_Interp *interp, 
			  IN char *file_name_p, 
			  IN u8_t match_dev_id, 
			  IN u8_t use_raw_img, 
			  IN u8_t preserve_ec, 
			  IN u8_t preserve_sn
			  );

/**
 * Description:
 * Chcek the NVM 
 */
u32_t nvm_crc(
	IN Tcl_Interp *interp,			/* Tcl interperter */
	IN char *block_name_p			/* pointer to block name to check */
	);


/**
 * Description: 
 * Execute the "device" command 
 */
u32_t execute_device (
	IN Tcl_Interp *interp,		/* Tcl interperter*/
	IN u8_t dev_idx				/* device number */
	);

/**
 * Description: 
 * Show the devices
 */
u32_t show_devices (
	IN Tcl_Interp *interp		/* Tcl interperter*/	
	);

u8_t is_zlr_machine (
    IN Tcl_Interp *interp
    );

u8_t is_fpga (
    IN Tcl_Interp *interp
    );

u8_t is_emul (
    IN Tcl_Interp *interp
    );

int split_dev(
    IN const char   *dev_str,
    OUT u32_t       *pf_dev_num, 
    OUT u32_t       *vf_dev_num);

u8_t is_e2 (
    IN Tcl_Interp *interp
    );

u8_t is_e1x (
    IN Tcl_Interp *interp
    );

u8_t is_e3 (
    IN Tcl_Interp *interp
    );

u8_t is_rev_b (
    IN Tcl_Interp *interp
    );

u8_t is_e3_57840 (
    IN Tcl_Interp *interp
    );
u8_t is_e1x (
    IN Tcl_Interp *interp
    );
u8_t is_e1 (
    IN Tcl_Interp *interp
    );
u8_t is_e1h (
    IN Tcl_Interp *interp
    );

void remove_white_spaces (char *token_ptr, 
			  u32_t field_len, 
			  char *field_ptr);

#endif /*__BROADCOM_EDBG_TCL_UTIL_H__*/
