#include <string.h>
#include <stdlib.h>
#include "tcl.h"
#include "tclDecls.h"
#include "tcl_al.h"
#include "bcmtype.h"
#include "tcl_al_cmd_handlers.h"
#include "tcl_al.h"
#include "tcl_version.h"

/*
 *------------------------------------------------------------------------
 * Description:
 *     Implements "version" command.
 * 
 * Returns:
 *     A standard Tcl result.
 *------------------------------------------------------------------------
 */
int tcl_al_version_cmd (
    IN    ClientData client_data, /* Not used. */
    IN    Tcl_Interp *interp,     /* current interpreter */
    IN    int objc,               /* number of arguments */
    IN    Tcl_Obj *CONST objv[]   /* argument strings */
    )
{	
        if (objc > 1) {
                Tcl_WrongNumArgs(interp, 1, objv, ":too many args");
                return TCL_ERROR;
        }

        Tcl_SetObjResult(interp, Tcl_NewStringObj("eDebug version "VERSION_STRING"", -1)); 

        return TCL_OK;
}

/*
 *------------------------------------------------------------------------
 * Description:
 *     Clean up the "version" command.
 * 
 * Returns:
 *     A standard Tcl result.
 *------------------------------------------------------------------------
 */
int tcl_al_version_cleanup(
    IN   Tcl_Interp *interp
    )
{

    Tcl_DeleteCommand(interp, VERSION_CMD);	

    return TCL_OK;
}

/*
 *------------------------------------------------------------------------
 * tcl_al_version_usage --
 *     Add usage and help information on "version" command 
 *------------------------------------------------------------------------
 */
int tcl_al_version_usage(
	IN	char *buf,				//Pointer to the buffer of the command usage
	IN	u32_t *length			        //Pointer to the length result of the command usage
	)
{	
	u32_t offset = 0;
	*length = 0;

	*length += sprintf( &buf[offset+*length], "Get the eDebug version number\n\n");
        *length += sprintf( &buf[offset+*length], "Usage: version \n" );
	  
        return TCL_OK;
}





