#include <string.h>
#include "man_types.h"
#include "edebug_types.h"
#include "tcl.h"
#include "tcl_al.h"
#include "tcl_version.h"

extern virtual_nvm_t virtual_nvm;

u8_t g_usb = 0;
u8_t g_list = 0;

dbg_status_t tcl_al_parse_params(Tcl_Interp* interp, int diag_argc, char **diag_argv, edebug_eng_mode_t *eng_mode)
{

	u32_t n_opt;
	
	*eng_mode = EDEBUG_GRC_DUMP_MODE;	

	for (n_opt = 1; (n_opt < diag_argc) && (diag_argv[n_opt] != NULL) ; n_opt++)
	{
		if (0 == strcmp(diag_argv[n_opt], "-special_log")) {
			tcl_al_set_special_log_mode(interp);
		}
		else if (0 == strcmp(diag_argv[n_opt], "-no_pci")) {
			tcl_al_set_no_pci_mode(interp, 1);
			*eng_mode = EDEBUG_NO_PCI_MODE;
		}
		else if (0 == strcmp(diag_argv[n_opt], "-usb")) {
			tcl_al_set_no_pci_mode(interp, 1);
			*eng_mode = EDEBUG_NO_PCI_MODE;
			g_usb = 1;
		}
		else if (0 == strcmp(diag_argv[n_opt], "-vnvm")) {
			*eng_mode = EDEBUG_VIRTUAL_NVM_MODE;

			// reset virtual_nvm values
			virtual_nvm.chip_type = CHIP_TYPE_INVALID;
			virtual_nvm.port_mode = PORT_MODE_INVALID;
			virtual_nvm.fd = NULL;
			virtual_nvm.nvm_size = 0;
		}
		else if (0 == strcmp(diag_argv[n_opt], "-no_uart")) {
			*eng_mode = EDEBUG_ENG_NO_UART_MODE;
		}
		else if (0 == strcmp(diag_argv[n_opt], "-b10eng")) {
			if (*eng_mode == EDEBUG_GRC_DUMP_MODE) {
				*eng_mode = EDEBUG_ENG_MODE;
			}			
		}
		else if (0 == strcmp(diag_argv[n_opt], "-noinit")) {
			*eng_mode = EDEBUG_NO_INIT_MODE;
		}
		else if (0 == strcmp(diag_argv[n_opt], "-version")) {
			printf(VERSION_STRING);
			Tcl_Eval(interp, "exit");
		}		
		else if (0 == strcmp(diag_argv[n_opt], "-com")) {
			if (tcl_al_get_no_pci_mode() != 1) {
				printf("-com must appear in -no_pci mode\n");
				return DBG_STATUS_FAILURE;
			}
			if (n_opt+1 < diag_argc) {
				if (tcl_al_set_uart_com(diag_argv[n_opt+1])) {
					printf("param after -com must be numeric\n");
					return DBG_STATUS_FAILURE;
				}
				n_opt++;
			}
			else {
				printf("command line parameters are invliad, number must appear after -com\n");
				return DBG_STATUS_FAILURE;
			}
		}
		else if (0 == strcmp(diag_argv[n_opt], "-baud")) {
			if (tcl_al_get_no_pci_mode() != 1) {
				printf("-baud must appear in -no_pci mode\n");
				return DBG_STATUS_FAILURE;
			}
			if (n_opt+1 < diag_argc) {
				if (tcl_al_set_uart_baud(diag_argv[n_opt+1])) {
					printf("param after -baud must be numeric\n");
					return DBG_STATUS_FAILURE;

				}
				n_opt++;
			}
			else {
				printf("command line parameters are invliad, number must appear after -baud\n");
				return DBG_STATUS_FAILURE;
			}
		}
		else if (0 == strcmp(diag_argv[n_opt], "-chip")) {
			if (*eng_mode != EDEBUG_VIRTUAL_NVM_MODE) {
				printf("-chip must appear in virtual nvm mode\n");
				return DBG_STATUS_FAILURE;
			}
			if (n_opt+1 < diag_argc) {
				if (tcl_al_set_virtual_nvm_chip_type(diag_argv[n_opt+1])) {
					printf("param after -chip must be valid chip type\n");
					return DBG_STATUS_FAILURE;

				}
				n_opt++;				
			}
			else {
				printf("command line parameters are invliad, value must appear after -chip\n");
				return DBG_STATUS_FAILURE;
			}
		}		
		else if (0 == strcmp(diag_argv[n_opt], "-nvm_size")) {
			if (*eng_mode != EDEBUG_VIRTUAL_NVM_MODE) {
				printf("-nvm_size must appear in virtual nvm mode\n");
				return DBG_STATUS_FAILURE;
			}
			if (n_opt+1 < diag_argc) {
				if (tcl_al_set_virtual_nvm_size(diag_argv[n_opt+1])) {
					printf("param after -nvm_size must be valid port mode\n");
					return DBG_STATUS_FAILURE;

				}
				n_opt++;				
			}
			else {
				printf("command line parameters are invalid, valid size (in bytes) must appear after -nvm_size\n");
				return DBG_STATUS_FAILURE;
			}
		}
		else if (0 == strcmp(diag_argv[n_opt], "-img")) {
			if (*eng_mode != EDEBUG_VIRTUAL_NVM_MODE) {
				printf("-img must appear in virtual nvm mode\n");
				return DBG_STATUS_FAILURE;
			}
			if (n_opt+1 < diag_argc) {
				virtual_nvm.fd = fopen(diag_argv[n_opt+1], "rb+");
				if (!virtual_nvm.fd) {
					printf("could not find file named %s\n", diag_argv[n_opt+1]);
					return DBG_STATUS_FAILURE;
				}
				fseek (virtual_nvm.fd, 0L, SEEK_END);
				virtual_nvm.nvm_size = ftell(virtual_nvm.fd);				
				fseek (virtual_nvm.fd, 0L, SEEK_SET);
				n_opt++;				
			}
			else {
				printf("command line parameters are invalid, file name must appear after -img\n");
				return DBG_STATUS_FAILURE;
			}
		}
		else if (0 == strcmp(diag_argv[n_opt], "-rc")) {
			if (n_opt+1 < diag_argc) {
				tcl_al_set_script_to_run(diag_argv[n_opt+1]);
				n_opt++;
			}
			else {
				printf("command line parameters are invliad, script must appear after -rc\n");
				return DBG_STATUS_FAILURE;
			}
		}
		else if (0 == strcmp(diag_argv[n_opt], "-list")) {
			g_list = 1;
		}
		else if (eng_mode != EDEBUG_GRC_DUMP_MODE) {
			printf("command line parameters are invliad, wrong param: %s\n", diag_argv[n_opt]);
			return DBG_STATUS_FAILURE;
		}
	}

	return DBG_STATUS_SUCCESS;
}
