#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <net/if.h>
#include "asm_i386_types.h"
#include "usr_include_linux_ethtool.h"
#include "usr_include_linux_sockios.h"
#include "util.h"
#include "os_types.h"
#include "edebug_types.h"
#include "dev_if.h"
#include "edebug_if.h"
#include "os_util_ioctl.h"

#define VMFS_VMKDRIVER_DIR  "/vmfs/devices/char/vmkdriver"

int bnx2x_fd = -1;

static dbg_status_t vmware_open_ioctl_socket(struct os_params *p_os_params)
{
        int fd;
        struct ifreq ifr;
        int err;
	struct ethtool_drvinfo drvinfo;
        // struct ethtool_command ecmd;
        char full_eth_name[30];
        char intf[256];

        /* Setup our control structures. */
        snprintf( intf, sizeof( intf ), "%s/%s", VMFS_VMKDRIVER_DIR, p_os_params->eth_name);
        fd = open( intf, O_RDWR );

        if (fd < 0) {
		printf("Cannot get control socket\n");
		return DBG_STATUS_FAILURE;
        }           

        memset(&ifr, 0, sizeof(ifr));
	strcpy(ifr.ifr_name, p_os_params->eth_name);
	drvinfo.cmd = ETHTOOL_GDRVINFO;
	ifr.ifr_data = (caddr_t)&drvinfo;
	err = ioctl(fd, SIOCETHTOOL, &ifr);
	if (err < 0) {
		printf("Cannot get driver information\n");
		return DBG_STATUS_FAILURE;
	}

	if (p_os_params->geeprom_length <= 0)
		p_os_params->geeprom_length = drvinfo.eedump_len;

	if (drvinfo.eedump_len < p_os_params->geeprom_length)
                p_os_params->geeprom_length = drvinfo.eedump_len;      

	p_os_params->fd_ethtool = fd;

	return DBG_STATUS_SUCCESS;
}

/* read 32 bits value from given grc addr */
dbg_status_t vmware_reg_read(
    IN  struct os_params   	os_params,
    IN  u32_t       	  	addr,
    OUT u32_t*      		value
    )
{
        struct bnx2x_ioctl_cmd ioctl_cmd;
        struct bnx2x_ioctl_req req;

        /* build the command */
        req.cmd = BNX2X_VMWARE_CIM_CMD_REG_READ;
        req.cmd_req.reg_read.reg_offset = addr;
        sprintf(ioctl_cmd.eth_name, "%s", os_params.eth_name);
        ioctl_cmd.p_data = &req;

        /* send the command to the driver */
        ioctl(os_params.fd, BNX2X_IOCTL_NUM, &ioctl_cmd);

        /* parse response from the driver */
        *value = req.cmd_req.reg_read.reg_value;

        return DBG_STATUS_SUCCESS;
}

/* write 32 bits value to given grc addr */
dbg_status_t vmware_reg_write(
    IN  struct os_params   	os_params,
    IN  u32_t       		addr,
    IN  u32_t       		value
    )
{
        struct bnx2x_ioctl_cmd ioctl_cmd;
        struct bnx2x_ioctl_req req;

        /* build the command */
        req.cmd = BNX2X_VMWARE_CIM_CMD_REG_WRITE;
        req.cmd_req.reg_write.reg_offset = addr;
        req.cmd_req.reg_write.reg_value = value;

        sprintf(ioctl_cmd.eth_name, "%s", os_params.eth_name);
        ioctl_cmd.p_data = &req;

        /* send the command to the driver */
        ioctl(os_params.fd, BNX2X_IOCTL_NUM, &ioctl_cmd);

        return DBG_STATUS_SUCCESS;
}

dbg_status_t vmware_cleanup(
    IN  struct _dev_if_t   	*dev_if
    ) 
{  
	return DBG_STATUS_SUCCESS;
}

dbg_status_t vmware_nv_read(
    IN struct os_params 	os_params,
    IN  u32_t			addr,
    OUT u32_t*			value,
    IN  u32_t       		size
    )
{
	struct ifreq ifr;
	struct ethtool_eeprom *eeprom;
        int err;

	 /* Setup our control structures. */
	memset(&ifr, 0, sizeof(ifr));
	strcpy(ifr.ifr_name, os_params.eth_name);

	eeprom = calloc(1, sizeof(*eeprom) + size);
	if (!eeprom) {
		printf("Cannot allocate memory for EEPROM data");
		return DBG_STATUS_FAILURE;
	}

	eeprom->cmd = ETHTOOL_GEEPROM;
	eeprom->len = size;
	eeprom->offset = addr;
	ifr.ifr_data = (caddr_t)(eeprom);

	err = ioctl(os_params.fd_ethtool, SIOCETHTOOL, &ifr);
        if (err < 0) {
                printf("Cannot get EEPROM data\n");
                return DBG_STATUS_FAILURE;
        }

	memcpy(value, eeprom->data, size);

	free(eeprom);

	return DBG_STATUS_SUCCESS;
}

#define VMWARE_MAX_NVRAM_OPERATION_SIZE 1024

/* write 32 bits value to given grc addr */
dbg_status_t vmware_nv_write(
    IN struct os_params     	os_params,
    IN  u32_t			addr,
    IN  u32_t*			value,
    IN  u32_t			size
    )
{
	struct ifreq ifr;
	struct ethtool_eeprom *eeprom;
        int err, i;
	u32_t *tmpArray;
	u32_t offset = 0;
	u32_t remaining;

        if (!size) {
		return DBG_STATUS_SUCCESS;
	}

	 /* Setup our control structures. */
	memset(&ifr, 0, sizeof(ifr));
	strcpy(ifr.ifr_name, os_params.eth_name);

	eeprom = calloc(1, sizeof(*eeprom) + VMWARE_MAX_NVRAM_OPERATION_SIZE);
	if (!eeprom) {
		printf("Cannot allocate memory for EEPROM data");
		return DBG_STATUS_FAILURE;
	}

	eeprom->cmd = ETHTOOL_SEEPROM;
	eeprom->magic = 0;

	/* cq70289 - perform nvram iterations in small chunks due to VMWARE
	 * heartbit mechanism
	 */
	while (offset < size) {
		remaining = MIN(size - offset, VMWARE_MAX_NVRAM_OPERATION_SIZE);
		eeprom->offset = addr + offset;
		eeprom->len = remaining;
		memcpy(eeprom->data, (u8_t *)value + offset, remaining);

		/* byte swapping should be performed because bnx2x expects to receive the bytes in byte swapped order */
		tmpArray = (u32_t *)(eeprom->data);
		for (i = 0; i < (remaining/4); i++) {
			tmpArray[i] = ntohl(tmpArray[i]);
		}	

		ifr.ifr_data = (caddr_t)eeprom;
	
		err = ioctl(os_params.fd_ethtool, SIOCETHTOOL, &ifr);
		if (err < 0) {
			printf("Cannot set EEPROM data");
			return DBG_STATUS_FAILURE;
		}

		offset += remaining;
	}

	free(eeprom);	

	return DBG_STATUS_SUCCESS;
}

/* get NIC statistics via ethtool */
dbg_status_t vmware_stats_show(
	IN  struct os_params	os_params)
{
	
	struct ifreq ifr;
	struct ethtool_drvinfo drvinfo;
	struct ethtool_gstrings *strings;
	struct ethtool_stats *stats;
	unsigned int n_stats, sz_str, sz_stats, i;
	int fd, err;
	char intf[256];

	/* setup ifr control structure */
	memset(&ifr, 0, sizeof(ifr));
	strcpy(ifr.ifr_name, os_params.eth_name);

	/* prepare and send first ioctl to get the number of statistics */
	drvinfo.cmd = ETHTOOL_GDRVINFO;
	ifr.ifr_data = (caddr_t)&drvinfo;

	err = ioctl(os_params.fd_ethtool, SIOCETHTOOL, &ifr);
	if (err < 0) {
		printf("Cannot get NIC statistics (first ioctl failed)\n");
		return DBG_STATUS_FAILURE;
	}
	n_stats = drvinfo.n_stats;
	if (n_stats < 1) {
		printf("Cannot get NIC statistics (n_stats < 1)\n");
		return DBG_STATUS_FAILURE;
	}

	/* allocate memory for the statistics */
	sz_str = n_stats * ETH_GSTRING_LEN;
	sz_stats = n_stats * sizeof(u64);
	strings = calloc(1, sz_str + sizeof(struct ethtool_gstrings));
	stats = calloc(1, sz_stats + sizeof(struct ethtool_stats));
	if (!strings || !stats) {
		printf("Cannot get NIC statistics (memory allocation failed)\n");
		return DBG_STATUS_FAILURE;
	}

	/* prepare and send second ioctl to get the statistics strings */
	strings->cmd = ETHTOOL_GSTRINGS;
	strings->string_set = ETH_SS_STATS;
	strings->len = n_stats;
	ifr.ifr_data = (caddr_t) strings;
	err = ioctl(os_params.fd_ethtool, SIOCETHTOOL, &ifr);
	if (err < 0) {
		printf("Cannot get NIC statistics (strings ioctl failed)\n");
		free(strings);
		free(stats);
		return DBG_STATUS_FAILURE;
	}

	/* prepare and send third ioctl to get the statistics counters */
	stats->cmd = ETHTOOL_GSTATS;
	stats->n_stats = n_stats;
	ifr.ifr_data = (caddr_t) stats;
	err = ioctl(os_params.fd_ethtool, SIOCETHTOOL, &ifr);
	if (err < 0) {
		printf("Cannot get NIC statistics (stats ioctl failed)\n");
		free(strings);
		free(stats);
		return DBG_STATUS_FAILURE;
	}

	/* pretty-print the statistics per-driver */
	printf("NIC statistics:\n");
	for (i = 0; i < n_stats; i++) {
		printf("	 %.*s: %llu\n",
			ETH_GSTRING_LEN,
			&strings->data[i * ETH_GSTRING_LEN],
			stats->data[i]);
	}

	free(strings);
	free(stats);
	close(fd);

	return DBG_STATUS_SUCCESS;
}

/* initialize Everest's dispatch table  */
dbg_status_t everest_dev_if_init_dispatch(
	IN  int 	br_idx,
	IN  pci_info_t  	pci_info,
	OUT dev_if_t*   	dif 		/* dispatch table to be initialized */
	)
{
	/* init OS dependent parameters from pci_info */
	sprintf(dif->os_params.eth_name, "%s", pci_info.eth_name);
	dif->os_params.fd = get_bnx2x_fd(pci_info.eth_name);
	dif->os_params.geeprom_length = -1;

	/* perform one time initializations */
	vmware_open_ioctl_socket(&(dif->os_params));

	/* Dispatch table */ 
	dif->dev_if_reg_read = vmware_reg_read;
	dif->dev_if_reg_write = vmware_reg_write;
	dif->dev_if_cleanup = vmware_cleanup;
	dif->dev_if_nv_read = vmware_nv_read;
	dif->dev_if_nv_write = vmware_nv_write;
	dif->dev_if_get_dev_info = NULL;
	dif->dev_if_stats_show = vmware_stats_show;

	return DBG_STATUS_SUCCESS;
}

