/****************************************************************************
 * Copyright(c) 2009-2015 Broadcom Corporation, all rights reserved
 * Proprietary and Confidential Information.
 *
 * This source file is the property of Broadcom Corporation, and
 * may not be copied or distributed in any isomorphic form without
 * the prior written consent of Broadcom Corporation.
 *
 * Name: bdn.h
 *
 * Description: BDN definitions
 *
 * Author: Yaniv Rosner
 *
 ****************************************************************************/

#ifndef BDN_H
#define BDN_H

typedef u32 bdn_cfg;
#define BDN_CFG_SIZE_MASK		0x0000ffff
#define BDN_CFG_SIZE_OFFSET		0
#define BDN_CFG_STATE_MASK		0x00ff0000
#define BDN_CFG_STATE_OFFSET		16
	#define BDN_CFG_STATE_ACTIVE		0x00010000
	#define BDN_CFG_STATE_PENDING		0x00020000
	#define BDN_CFG_STATE_DEFAULT		0x00040000

#define SERVER_DESCRIPTION_MAX_LENGTH 64
struct server_descrip {
	u8_t len;
	u8_t str[SERVER_DESCRIPTION_MAX_LENGTH];
};

#define SERVER_UUID_LENGTH	 36
struct server_uuid {
	u8_t len;
	u8_t str[SERVER_UUID_LENGTH];
};

#define GENERALIZED_TIME_MAX_SIZE 16 /* YYYYMMDDhhmmss.s */
struct generalized_time {
	u8_t len;
	u8_t time_str[GENERALIZED_TIME_MAX_SIZE];
};
#define CONNECTION_ID_LENGTH	16

#define SLOT_TYPE_NUM_MAX_LENGTH 32
struct slot_type_num {
	u8_t len;
	u8_t str[SLOT_TYPE_NUM_MAX_LENGTH];
};

#define ILO_MGMT_MAX_NUM_OF_ADDR 3
#define ILO_MGMT_ADDR_MAX_LENGTH 16
struct iLO_mgmt_addr {
	u8_t num_of_add;
	u8_t len_of_add[ILO_MGMT_MAX_NUM_OF_ADDR];
	u8_t str[ILO_MGMT_MAX_NUM_OF_ADDR][ILO_MGMT_ADDR_MAX_LENGTH];
};
struct base_dev_next_os {
	u8_t reset_2_factory;
	u8_t res[3];
};

struct base_dev_now {
	u8_t one_view_config;
	u8_t res[3];
};

struct base_dev {
	struct base_dev_now now;
	struct base_dev_next_os next_os;
};

struct server_info {
	u8_t optional_bitmap;
#define SERVER_INFO_ILO_MGMT_VLAN_PRESENT	(1<<0)
#define SERVER_INFO_DATA_TIMESTAMP_PRESENT	(1<<1)
	struct server_descrip server_descrip;
	struct server_uuid server_uuid;
	struct slot_type_num slot_type_num;
	struct iLO_mgmt_addr iLO_mgmt_addr;
	u16_t iLO_mgmt_vlan;
	struct generalized_time data_timestamp;
};

struct Codec_Info {
	u32_t version;	/* Version of Codec */

	enum codec_location {
		e_None		= 0,
		e_Here		= 1,
		e_FRU_EEPROM	= 2
	} loc; /* DEFAULT e-None */
	u16_t total_size;	/* total size of Codec in bytes.Max size 32K (64K?)*/
	u16_t num_msg_segments;	/* number of segments = total size / max message payload size */
};

#define MAX_CODEC_SIZE		0x8000
#define CODEC_SEGMENT_SIZE	0x400
#define NUMBER_OF_SEGMENTS (MAX_CODEC_SIZE/CODEC_SEGMENT_SIZE)
struct codec_t {
	u8_t data[NUMBER_OF_SEGMENTS][CODEC_SEGMENT_SIZE];
};

struct bdn_netport_now {
	bdn_cfg hdr;
	u8 enable_port;
	u8 enable_diag;
	u8 num_pfs_min_bw;
	u8 num_pfs_max_bw;
	u8 min_bw[E2_FUNC_MAX]; /* 4 PFs in 2 port mode / 2 PFs in 4 port mode */
	u8 max_bw[E2_FUNC_MAX]; /* 4 PFs in 2 port mode / 2 PFs in 4 port mode */
};

struct bdn_netport_on_port_reset {
	bdn_cfg hdr;
	u32 link_config;
/* Same definitions as in PORT_HW_CFG_SPEED_CAPABILITY_D3_MASK */
#define BDN_LINK_CONFIG_ADVERTISED_SPEED_MASK	0x0000ffff
#define BDN_LINK_CONFIG_ADVERTISED_SPEED_SHIFT	0
/* Same definitions as PORT_FEATURE_FLOW_CONTROL_MASK */
#define BDN_LINK_CONFIG_FLOW_CONTROL_MASK	0x00070000
#define BDN_LINK_CONFIG_FLOW_CONTROL_SHIFT	16

#define BDN_LINK_CONFIG_PFC_ENABLED_MASK	0x00080000
#define BDN_LINK_CONFIG_PFC_ENABLED_SHIFT	19

#define BDN_LINK_CONFIG_EEE_ENABLED_MASK	0x00100000
#define BDN_LINK_CONFIG_EEE_ENABLED_SHIFT	20
};

struct bdn_netport_next_os_boot {
	bdn_cfg hdr;
	u8   num_pfs;
	u8   num_vf_per_pf[E2_FUNC_MAX];
};

struct bdn_netport_diag_ctrl {
	bdn_cfg hdr;
	u8  port_reset;
	u8  local_loopback;
	u8  remote_loopback;
	u8  rsrv[1];
};

struct bdn_netport {
	struct bdn_netport_now now;
	struct bdn_netport_on_port_reset on_port_reset; /* On Port Reset */
	struct bdn_netport_next_os_boot next_os_boot; /* Next OS Boot */
	struct bdn_netport_diag_ctrl diag_ctrl;
};

#define CONNECTION_ID_LEN 16
struct bdn_flexfunc_now {
	bdn_cfg hdr;
	u8 connection_id[CONNECTION_ID_LEN];
	u8 fnic_enabled;
	u8 rsrv[3];
};

struct bdn_flexfunc_next_os_boot {
	bdn_cfg hdr;
	u32 optional_bitmap;
#define FLEXFUNC_CFG_NEXT_OS_C2S_PCP_MAP_PRESENT	(1<<0)
#define FLEXFUNC_CFG_NEXT_OS_PORT_ASSIGNMENT_PRESENT	(1<<1)
#define FLEXFUNC_CFG_NEXT_OS_EMB_LAG_PRESENT		(1<<2)
	u8 mac_addr[6];
	u8 func_type;
	u8 boot_mode;
	u8 c_2_s_pcp_map[9]; /*  Maps O/S C-VLAN PCP value to S-VLAN PCP value for TX
			      * -- items 1-8 correspond to the O/S C-VLAN PCP values 0-7
			      * -- item 9 is the default if no C-VLAN present
			      * -- Values in these 9 bytes are the expected S-PCP values.
			      * -- If NetDev-Device-Module: EVB-Support.c-2-s-pcp-map = FALSE, then
			      * --     all values must be identical
			      * -- else, it supports C-PCP -> S-PCP mapping
			      */
	u16 mtu_size;
	u8 rsrv[1];
};

struct bdn_flex_func_diag_ctrl {
	bdn_cfg hdr;
	u8  enable_wol;
	u8  rsrv[3];
};

struct bdn_flex_func {
	struct bdn_flexfunc_now now;
	struct bdn_flexfunc_next_os_boot next_os_boot; /* Next OS Boot */
	struct bdn_flex_func_diag_ctrl diag_ctrl;
};

struct bdn_fc_npiv {
	struct bdn_fc_npiv_tbl now; /* Next device level reset */
};

struct bdn_iscsi_initiator_cfg {
	u32 optional_bitmap;
#define ISCSI_INITIATOR_ROUTE_PRESENT	(1<<0)
#define ISCSI_INITIATOR_PRIMARY_DNS_PRESENT	(1<<1)
#define ISCSI_INITIATOR_SECONDARY_DNS_PRESENT	(1<<2)
	u8 name[232];
	u8 ip_add[16];
	u8 netmask[16];
	u8 route[16];
	u8 primary_dns[16];
	u8 secondary_dns[16];
};

struct bdn_iscsi_target_params {
	u32 optional_bitmap;
#define ISCSI_TARGET_LLMNR_ENABLE_PRESENT	(1<<0)
#define ISCSI_TARGET_ROUTE_ADV_ENABLE_PRESENT	(1<<1)
#define ISCSI_TARGET_IPV2_PRESENT	(1<<2)
	u8 name[232];
	u32 lun;
	u8 ip_addr[16];
	u32 tcp_port;
	u8 ip_addr_2[16];
	u32 tcp_port_2;
	u32 llmnr_en;
	u32 route_adv_en;
};

struct bdn_iscsi_authentication {
	u32 optional_bitmap;
#define ISCSI_AUTH_CHAP_USERNAME_PRESENT	(1<<0)
#define ISCSI_AUTH_CHAP_SECRET_PRESENT	(1<<1)
#define ISCSI_AUTH_MUTUAL_USERNAME_PRESENT	(1<<2)
#define ISCSI_AUTH_MUTUAL_SECRET_PRESENT	(1<<3)
	u32 auth_meth;
	u8 username[232];
	u8 secret[16];
	u32 secret_len;
	u8 mutual_username[232];
	u8 mutual_secret[16];
	u32 mutual_secret_len;
};

struct bdn_iscsi_boot_cfg {
	u32 optional_bitmap;
#define ISCSI_CFG_CVID_PRESENT	(1<<0)
#define ISCSI_CFG_DNS_VIA_DHCP_PRESENT	(1<<1)
#define ISCSI_CFG_TARGET_INFO_DHCP_PRESENT	(1<<2)
#define ISCSI_CFG_INITIATOR_PRESENT	(1<<3)
#define ISCSI_CFG_TARGET_PRESENT	(1<<4)
#define ISCSI_CFG_DHCP_VENDOR_ID_PRESENT	(1<<5)
#define ISCSI_CFG_AUTH_PRESENT	(1<<6)
#define ISCSI_AUTH_HEADER_DIGEST_FLAG_PRESENT	(1<<7)
#define ISCSI_AUTH_DATA_DIGEST_FLAG_PRESENT	(1<<8)
	bdn_cfg hdr;
	u32 cvid;
	u32 ip_add_type;
	u32 dns_via_dhcp;
	u32 target_via_dhcp;
	u8 dhcp_vendor_id[32];
	u32 head_digest_flag_en;
	u32 data_digest_flag_en;
};

struct bdn_iscsi_boot_next_dlr {
	struct bdn_iscsi_boot_cfg cfg; /* Next device level reset */
	struct bdn_iscsi_initiator_cfg initiator_cfg;
	struct bdn_iscsi_target_params target_params;
	struct bdn_iscsi_authentication authentication;
};

struct bdn_iscsi_boot {
	struct bdn_iscsi_boot_next_dlr next_dlr; /* Next device level reset */
};

#define FCOE_TARGETS_WWPN_SIZE 8
#define FCOE_TARGETS_LUN_SIZE 8

struct bdn_fcoe_targets {
	u8 wwpn[FCOE_TARGETS_WWPN_SIZE];
	u8 lun_id[FCOE_TARGETS_LUN_SIZE];
};

struct bdn_fcoe_boot_cfg {
		u32 optional_bitmap;
#define FCOE_CFG_CVID_PRESENT	(1<<0)
#define FCOE_BASE_WWNN_PRESENT	(1<<1)
#define FCOE_WWPN_PRESENT	(1<<2)
	bdn_cfg hdr;
	u32 cvid;
	u8 base_wwnn[FCOE_TARGETS_WWPN_SIZE]; /* Host World wide name*/
	u8 wwpn[FCOE_TARGETS_WWPN_SIZE]; /* base wwpn */
	u32 num_of_fcoe_targets;
};

struct bdn_fcoe_boot_next_dlr {
	struct bdn_fcoe_boot_cfg cfg; /* Next device level reset */
	struct bdn_fcoe_targets fcoe_targets[8];
};

struct bdn_fcoe_boot {
	struct bdn_fcoe_boot_next_dlr next_dlr; /* Next device level reset */
};
#ifndef PF_NUM_MAX
#define PF_NUM_MAX 8
#endif
struct bdn_ncsi_next_dlr {
	bdn_cfg hdr;
	u32 ncsi_scid;
};

struct bdn_ncsi {
	struct bdn_ncsi_next_dlr bdn_ncsi_next_dlr;
};

// Rapid-Response-Poll-Module definition
#define MAX_CCT_SIZE			64
#define VALID_RPP_SIGNATURE		0x69616853
struct rpp_sum_stc {
	u32 hash;
	u32 num;
};

struct digest_stc {
	u8 type;
	u8 inst;
	u16 digest;
};

struct bdn_rapid_respon_stc {
	u32 sig;
	struct rpp_sum_stc rpp_sum;
	u8 num_of_cct;
	u8 res[3];
	struct digest_stc cct[MAX_CCT_SIZE];
#ifdef CONFIG_SUPPORT_SCI
	struct digest_stc sci[MAX_CCT_SIZE];
#endif
};

struct bdn {
	u32 size;
	struct base_dev	base_dev;
	struct server_info server_info;
	struct bdn_netport netport[PORT_MAX * NVM_PATH_MAX];
	struct bdn_flex_func flexfunc[PF_NUM_MAX];
	struct bdn_fc_npiv fc_npiv[PORT_MAX * NVM_PATH_MAX];
	struct bdn_iscsi_boot iscsi_boot[PORT_MAX * NVM_PATH_MAX];
	struct bdn_fcoe_boot fcoe_boot[PORT_MAX * NVM_PATH_MAX];
	struct bdn_ncsi ncsi;
	struct bdn_rapid_respon_stc rpp_stc;
	u32 crc;
};

union bdn_image {
	struct bdn bdn;
	u8_t rsrv[0x9000];
};

/* Expected BDN size is basically the offset of rsrv within the bdn structure */
#define BDN_SIZE (sizeof(struct bdn))
#define BDN_SERVER_INFO()		  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, server_info))
#define BDN_BASE_DEV_NEXT_OS()		   (ASN1_P->bdn_addr + OFFSETOF(struct bdn, base_dev.next_os))
#define BDN_BASE_DEV_NOW()			  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, base_dev.now))
/* #define BDN_CODEC_INFO()		  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, codec_info)) */
/* #define BDN_CODEC_DATA()		  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, codec)) */
#define BDN_NETPORT_NOW(papo)		  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, netport[(papo.path << 1) + papo.port].now))
#define BDN_NETPORT_ON_PORT_RESET(papo)   (ASN1_P->bdn_addr + OFFSETOF(struct bdn, netport[(papo.path << 1) + papo.port].on_port_reset))
#define BDN_NETPORT_NEXT_OS_BOOT(papo)	  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, netport[(papo.path << 1) + papo.port].next_os_boot))
#define BDN_NETPORT_DIAG_CTRL(papo)	  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, netport[(papo.path << 1) + papo.port].diag_ctrl))
#define BDN_FLEXFUNC_NOW(pf_num)	  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, flexfunc[pf_num].now))
#define BDN_FLEXFUNC_NEXT_OS_BOOT(pf_num) (ASN1_P->bdn_addr + OFFSETOF(struct bdn, flexfunc[pf_num].next_os_boot))
#define BDN_FLEXFUNC_DIAG_CTRL(pf_num)	  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, flexfunc[pf_num].diag_ctrl))
#define BDN_FC_NPIV_NOW(papo)		  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, fc_npiv[(papo.path << 1) + papo.port].now))
#define BDN_FC_NPIV_NOW_CFG(papo)		  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, fc_npiv[(papo.path << 1) + papo.port].now.fc_npiv_cfg))
#define BDN_FC_NPIV_NOW_SETTINGS(papo, idx)		  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, fc_npiv[(papo.path << 1) + papo.port].now.settings[idx]))
#define BDN_ISCSI_BOOT_NEXT_DLR_CFG(papo)	  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, iscsi_boot[(papo.path << 1) + papo.port].next_dlr.cfg))
#define BDN_ISCSI_BOOT_NEXT_DLR_INITIATOR(papo)	  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, iscsi_boot[(papo.path << 1) + papo.port].next_dlr.initiator_cfg))
#define BDN_ISCSI_BOOT_NEXT_DLR_TARGET(papo)	  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, iscsi_boot[(papo.path << 1) + papo.port].next_dlr.target_params))
#define BDN_ISCSI_BOOT_NEXT_DLR_AUTHENTICATION(papo)	  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, iscsi_boot[(papo.path << 1) + papo.port].next_dlr.authentication))
#define BDN_FCOE_BOOT_NEXT_DLR_CFG(papo)	  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, fcoe_boot[(papo.path << 1) + papo.port].next_dlr.cfg))
#define BDN_FCOE_BOOT_NEXT_DLR_TARGET(papo, idx)	 (ASN1_P->bdn_addr + OFFSETOF(struct bdn, fcoe_boot[(papo.path << 1) + papo.port].next_dlr.fcoe_targets[idx]))
#define BDN_NCSI_NEXT_DLR()	  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, ncsi.bdn_ncsi_next_dlr))
#define BDN_RAPID_RESPONSE		  (ASN1_P->bdn_addr + OFFSETOF(struct bdn, rpp_stc))
#define BDN_CRC()			  (ASN1_P->bdn_addr + sizeof(union bdn_image))

#endif /* BDN_H */
