
/*
	This header file is for monolithical image data structure 
*/

#ifndef __MONO_IMAGE_H__
#define __MONO_IMAGE_H__

#define	MAX_REGIONS	 20
#define	MAX_CARDTYPE 100

//Monolithical region definition for E3
#define IMAGE_HDR_TYPE_MBA                      0x61626d00			//MBA, it maybe pxe only, pxe + efi, or pxe + efi + fcode
#define IMAGE_HDR_TYPE_MFW                      0x3177666d          //use MFW1 region code to represent whole MFW image
#define IMAGE_HDR_TYPE_L2                       0x74326c00          //use l2t region code to represent whole L2 image
#define IMAGE_HDR_TYPE_CCM                      0x6d636300          //ccm
#define IMAGE_HDR_TYPE_PHY_FW                   0x79687000          //ext phy fw      
#define IMAGE_HDR_TYPE_ISCSI					0x62690000          //iscsi boot
#define IMAGE_HDR_TYPE_FCOE				        0x62656600			//fcoe

/*Bit Mask Bit Patterns in PCI ID Matrix

Product Name	Support Bit Mask	phy fw2 84834	phy fw1  84833	fcoe	iscsi	840( pxe + uefi )	810 ( pxe + uefi )	l2	ccm	mfw	Catalogory
Benavides			0F																						x				x	  x	 x	    1
Bastrop				0F																						x				x	  x	 x	    1
Bailey				0F																						x				x	  x	 x		1
Bayview				0F																						x				x	  x	 x		1
Sundown				8F									x													x				x	  x	 x      2
Bronco				37												   x	   x			x		                        x	  x	 x	    3
Breezy				37												   x	   x			x								x	  x	 x		3
Baird				37												   x	   x			x								x     x	 x		3
Brownwood			37												   x	   x	        x								x	  x	 x		3
Benavides 2			6F												   x	   x							x				x	  x	 x		3
Bastrop2/Barracude	6F												   x	   x							x				x	  x	 x		3
Bailey 2			6F												   x	   x							x				x	  x	 x		3
Bayview 2			6F												   x	   x							x				x	  x	 x		3
Bryan				EF									x			   x	   x							x				x	  x	 x		4
*/

#define	SUPPORTED_PCI_ID_BENAVIDES_BIT_MASK		0x0F
#define	SUPPORTED_PCI_ID_BASTROP_BIT_MASK		0x0F
#define	SUPPORTED_PCI_ID_BAILEY_BIT_MASK		0x0F
#define SUPPORTED_PCI_ID_BAYVIEW_BIT_MASK		0x0F
#define	SUPPORTED_PCI_ID_SUNDOW_BIT_MASK		0x8F
#define	SUPPORTED_PCI_ID_BRONCO_BIT_MASK		0x37
#define	SUPPORTED_PCI_ID_BREEZY_BIT_MASK		0x37
#define	SUPPORTED_PCI_ID_BAIRD_BIT_MASK			0x37
#define	SUPPORTED_PCI_ID_BROWNWOOD_BIT_MASK		0x37
#define	SUPPORTED_PCI_ID_BENAVIDES2_BIT_MASK	0x6F
#define	SUPPORTED_PCI_ID_BARRACUDA_BIT_MASK		0x6F
#define	SUPPORTED_PCI_ID_BAILEY2_BIT_MASK		0x6F
#define	SUPPORTED_PCI_ID_BAYVIEW2_BIT_MASK		0x6F
#define	SUPPORTED_PCI_ID_BRYAN_BIT_MASK			0xEF



/*****************************************************************************/
typedef struct
{
    u8_t  signature[4];				
#define ILT_SIGNATURE_2     "QILT"
    u16_t tableVersion;				//initial  version  0001
    u16_t tableLength;				//Lengthe of Image metadata ( size of header + size of image entry * no of images + size of cardtype list)
    u16_t tableChecksum;			//16 bit 2's complete add-zero checksum over the entire image including the header
    u16_t numOfEntries;				//No. of image entries(Total number of standalone images)
    u8_t  cardListExists;			//0 - PCI ID matrix not present, 1 - PCI ID matrix present

    u8_t  versionMajor;				//MBI version major
    u8_t  versionMinor;				//MBI version minor
	u8_t  versionSubMinor;			//MBI version subminor
	u16_t rev;
    u8_t  build_month;				//Month(1 - 12)
    u8_t  build_date;				//Day(1-31) 
    u16_t build_year;				//Year(2015)
	u8_t  reserved[10];
} ILT_Header_t;

typedef struct
{
	u32_t regionType;				//To identify the image referenced by this entry
    u16_t offsetLo;					// Offset from the beginning of the image file
    u16_t offsetHi;
    u16_t sizeLo;					//Length of the image in no. of bytes
    u16_t sizeHi;
	u32_t rev;
    u8_t  chipRev;					//current we set it to zero
    u8_t  reserved[15];			
} ILT_Region_t;

typedef struct 
{
    u16_t noCardTypeList;			//No of entries for adapter to supported.
    u16_t reserved0;
    u32_t reserved1;
} ILT_CardType_Header_t;

typedef struct 
{
    u16_t vendorID;					
    u16_t deviceID;
    u16_t subSystemVID;
    u16_t subSystemDID;
	u32_t SupportBitMask;
	u16_t ChipVersion;
	u16_t Rev;
} ILT_CardType_Entry_t;

#endif //__MONO_IMAGE_H__

