
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "os_utils.h"

dbg_status_t os_if_map_iomem(u32_t             nbytes,
                             dbg_phys_addr_t   phy_addr,
                             dbg_virt_addr_t * virt_addr)
{
    dbg_phys_addr_t page_size = getpagesize();
    dbg_phys_addr_t page_offset;

    /* If page_size is not a power of 2 we are in trouble! */
    dbg_phys_addr_t page_mask = page_size - 1;
    int dev_mem_fd2;

    page_offset = phy_addr & page_mask;
    nbytes += page_offset;

    dev_mem_fd2 = open("/dev/mem", O_RDWR);
    if (dev_mem_fd2 == -1)
    {
        printf("ERROR - /dev/mem could not be opened\n");
        return DBG_STATUS_FAILURE;
    }

    if (mlockall(MCL_FUTURE) == -1)
    {
        printf("ERROR - mlockall for futures failed\n");
        return DBG_STATUS_FAILURE;
    }

    *virt_addr = (dbg_virt_addr_t)mmap(NULL, nbytes,
                                       (PROT_READ | PROT_WRITE),
                                       MAP_SHARED,
                                       dev_mem_fd2,
                                       (phy_addr - page_offset));
    if (*virt_addr == (dbg_virt_addr_t)MAP_FAILED)
    {
        printf("ERROR - mmap failed (%d)\n", errno);
        munlockall();
        return DBG_STATUS_FAILURE;
    }

    if ((*virt_addr) & page_mask)
    {
        printf("ERROR - (*virt_addr) & page_mask != 0 (%llx & %llx)\n",
               *virt_addr, page_mask);
    }

    *virt_addr += page_offset;

    return DBG_STATUS_SUCCESS;
}


dbg_status_t os_if_unmap_iomem(dbg_virt_addr_t virt_addr,
                               u32_t           nbytes)
{
    dbg_virt_addr_t page_size = getpagesize();
    dbg_virt_addr_t page_offset;

    /* If page_size is not a power of 2 we are in trouble! */
    dbg_virt_addr_t page_mask = page_size - 1;

    page_offset = virt_addr & page_mask;
    nbytes += page_offset;

    if (munmap((void*)(virt_addr & (~page_mask)), (int)nbytes))
    {
        printf("ERROR - munmap failed (%d)\n", errno);
        return DBG_STATUS_FAILURE;
    }

    munlockall();

    return DBG_STATUS_SUCCESS;
}

dbg_status_t os_if_alloc_pcontig(
     u32_t             nbytes,
     dbg_virt_addr_t * virt_addr_p, /* User space virtual address */
     dbg_phys_addr_t * phy_addr_p   /* PCI bus address */
     )
{
    printf("physical memory allocation is not supported in edebug\n");
    return DBG_STATUS_FAILURE;
}

dbg_status_t os_if_free_pcontig(
    dbg_virt_addr_t virt_addr
    )
{
    return DBG_STATUS_SUCCESS;
}

