#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "man_types.h"
#include "os_utils.h"
#include "edebug_types.h"
#include "pcics_reg_driver.h"
#include "hw_access.h"

static int total_pci_dev = -1;

static int get_dev_info(char       *dev_line,
                        pci_info_t *pci_info)
{
    char selector[32];
    char sel[32];
    char chip[33];
    char tmp[32];
    char *token;
    u32_t devid;
    char line[200];
    char cmd[128];
    FILE* pipe;
    u32_t bxe_inst;

    sscanf(dev_line, "%s %s %s %s %s %s", sel, tmp, tmp, chip, tmp, tmp);
    strcpy(selector, sel);

    pci_info->pci_domain = 0;

    token = strtok(sel, "@");
    sscanf(token, "%s", pci_info->eth_name); /* "drv#" */
    token = strtok(NULL, ":");
    token = strtok(NULL, ":");
    sscanf(token, "%d", &pci_info->pci_bus);
    token = strtok(NULL, ":");
    sscanf(token, "%d", &pci_info->pci_device);
    token = strtok(NULL, ":");
    sscanf(token, "%d", &pci_info->pci_func);

    token = pci_info->eth_name;
    while (!isdigit(*token)) token++;
    *tmp = *token;
    *token = 0;
    strcpy(pci_info->driver_name, pci_info->eth_name); /* "drv" */
    *token = *tmp;

    token = strtok(chip, "=");
    token = strtok(NULL, "\0");
    sscanf(token, "%x", &devid);
    pci_info->vendor_id = (devid & 0xffff);
    pci_info->device_id = ((devid >> 16) & 0xffff);

    if (!IS_EVEREST((*pci_info))) {
        return FALSE;
    }

    printf("QLogic 10Gb device found: %04x:%02x.%02x %04x:%04x\n",
           pci_info->pci_bus,
           pci_info->pci_device,
           pci_info->pci_func,
           pci_info->vendor_id,
           pci_info->device_id);

    pci_info->irq = 0; /* XXX */
    pci_info->eth_state = ETH_D0;

    pipe = popen("pciconf -lb", "r");
    if (!pipe) {
        printf("ERROR: Failed to open pciconf pipe!\n");
        return FALSE;
    }

    while (fgets(line, sizeof(line), pipe)) {				
        if (strstr(line, selector) == NULL) {
            continue;
        }

        fgets(line, sizeof(line), pipe);

        if (strstr(line, "type Memory") != NULL) {
            sscanf(line, "%s %s %s %s %s %s %s %s %llx, %s %d, %s",
                   tmp, tmp, tmp, tmp, tmp, tmp, tmp, tmp,
                   &pci_info->region0_base_addr, tmp,
                   &pci_info->region0_size, tmp);
            //printf("BAR0: 0x%016llx (%d)\n", pci_info->region0_base_addr, pci_info->region0_size);

            fgets(line, sizeof(line), pipe);

            fgets(line, sizeof(line), pipe);
            sscanf(line, "%s %s %s %s %s %s %s %s %llx, %s %d, %s",
                   tmp, tmp, tmp, tmp, tmp, tmp, tmp, tmp,
                   &pci_info->region2_base_addr, tmp,
                   &pci_info->region2_size, tmp);
            //printf("BAR2: 0x%016llx (%d)\n", pci_info->region2_base_addr, pci_info->region2_size);

        } else if (strstr(line, "type Prefetchable Memory") != NULL) {
            sscanf(line, "%s %s %s %s %s %s %s %s %s %llx, %s %d, %s",
                   tmp, tmp, tmp, tmp, tmp, tmp, tmp, tmp, tmp,
                   &pci_info->region0_base_addr, tmp,
                   &pci_info->region0_size, tmp);
            //printf("BAR0: 0x%016llx (%d)\n", pci_info->region0_base_addr, pci_info->region0_size);

            fgets(line, sizeof(line), pipe);

            fgets(line, sizeof(line), pipe);
            sscanf(line, "%s %s %s %s %s %s %s %s %s %llx, %s %d, %s",
                   tmp, tmp, tmp, tmp, tmp, tmp, tmp, tmp, tmp,
                   &pci_info->region2_base_addr, tmp,
                   &pci_info->region2_size, tmp);
            //printf("BAR2: 0x%016llx (%d)\n", pci_info->region2_base_addr, pci_info->region2_size);
        } else {
            printf("ERROR: Unable to parse pciconf output for BARs!\n");
            return FALSE;
        }
    }

    pclose(pipe);

    /* if driver not loaded then bail */
    if (strcmp(pci_info->driver_name, "bxe") != 0) {
        return TRUE;
    }

    /* get the driver instance */
    sscanf(pci_info->eth_name, "%3s%d", tmp, &bxe_inst);

    sprintf(cmd, "sysctl dev.bxe.%d.version", bxe_inst);
    pipe = popen(cmd, "r");
    if (!pipe) {
        printf("ERROR: Failed to open sysctl pipe!\n");
        return TRUE;
    }

    if (!fgets(line, sizeof(line), pipe)) {
        printf("WARNING: %s interface missing 'version' sysctl\n",
               pci_info->eth_name);
        pclose(pipe);
        return TRUE;
    }

    sscanf(line, "%s %s", tmp, pci_info->driver_ver);

    pclose(pipe);

    sprintf(cmd, "sysctl dev.bxe.%d.bc_version", bxe_inst);
    pipe = popen(cmd, "r");
    if (!pipe) {
        printf("ERROR: Failed to open sysctl pipe!\n");
        return TRUE;
    }

    if (!fgets(line, sizeof(line), pipe)) {
        printf("WARNING: %s interface missing 'bc_version' sysctl\n",
               pci_info->eth_name);
        pclose(pipe);
        return TRUE;
    }

    sscanf(line, "%s %s", tmp, pci_info->fw_ver);

    pclose(pipe);

    sprintf(cmd, "sysctl dev.bxe.%d.mac_addr", bxe_inst);
    pipe = popen(cmd, "r");
    if (!pipe) {
        printf("ERROR: Failed to open sysctl pipe!\n");
        return TRUE;
    }

    if (!fgets(line, sizeof(line), pipe))
    {
        printf("WARNING: %s interface missing 'mac_addr' sysctl\n",
               pci_info->eth_name);
        pclose(pipe);
        return TRUE;
    }

    sscanf(line, "%s %s", tmp, pci_info->mac_addr);

    pclose(pipe);

    return TRUE;
}

dbg_status_t os_if_get_total_dev(u32_t * dev_found_p)
{
    char line[200];
    FILE* pipe;
    u32_t result = 0;

    if (total_pci_dev >= 0) {
        *dev_found_p = total_pci_dev;
        return DBG_STATUS_SUCCESS;
    }

    /* get number of devices from pciconf output */
    pipe = popen("pciconf -l", "r");
    if (!pipe) {
        printf("ERROR: Failed to open pciconf pipe!\n");
        perror("popen");
        return DBG_STATUS_FAILURE;
    }

    /* count number of lines from pciconf output for number of devices */
    while (fgets(line, sizeof(line), pipe)) {
        result++;
    }

    *dev_found_p  = result;
    total_pci_dev = result;

    pclose(pipe);

    return DBG_STATUS_SUCCESS;
}

dbg_status_t os_if_pci_scan(pci_info_t * pci_info_ar)
{
    char line[200];
    char cmd[60];
    char *token;
    FILE *pipe;
    u32_t i, num_everest, ret_val;

    pipe = popen("pciconf -l", "r");
    if (!pipe) {
        printf("ERROR: Failed to open pciconf pipe!\n");
        return DBG_STATUS_FAILURE;
    }

    i = 0;
    num_everest = 0;

    while (fgets(line, sizeof(line), pipe)) {				
        if (get_dev_info(line, &pci_info_ar[i]) == TRUE) {
            num_everest++;
        }
        i++;
    }

    pclose(pipe);

    dbg_set_total_br_dev(num_everest);

    for (i = 0; i < dbg_get_total_dev(); i++)
    {
        if (IS_EVEREST(pci_info_ar[i]))
        {
            /* set current DUT that pci config space approach will be right */
            dbg_pcicfg_setdut(pci_info_ar[i].pci_domain,
                              pci_info_ar[i].pci_bus,
                              pci_info_ar[i].pci_device,
                              pci_info_ar[i].pci_func);

            /* read and parse link_control register */
            dbg_pcicfg_read(PCICFG_LINK_CONTROL, &ret_val);
            dbg_pcicfg_process_link_control_reg(pci_info_ar, i, ret_val);

            /* read and parse SRIOV info in config space */
            dbg_pcicfg_read((PCI_CFG_SRIOV_BASE + SRIOV_NUM_VFS_OFFSET),
                            &ret_val);

            if (ret_val)
            {
                pci_info_ar[i].total_vfs = ret_val & 0xFF;
                dbg_pcicfg_read((PCI_CFG_SRIOV_BASE + SRIOV_FIRST_VF_STRIDE_OFFSET),
                                &ret_val);
                pci_info_ar[i].first_vf_offset = ret_val & 0xFF;
                pci_info_ar[i].vf_stride = (ret_val & 0xFF00) >> 16;
            }

            /* read and parse SVID info from config space */
            dbg_pcicfg_read(PCICFG_SUBSYSTEM_VENDOR_ID_OFFSET, &ret_val);
            pci_info_ar[i].svid = (u32_t)((u16_t) ret_val);
        }
    }

    return DBG_STATUS_SUCCESS;
}

dbg_status_t os_if_read_configuration_dword(
    u8_t    domain,
    u16_t   bus,
    u8_t    device,
    u8_t    function,
    u32_t   register_number,        /* register offset */
    u32_t * dword_read              /* output buffer */
    )
{
    char cmd[60];
    char line[200];
    FILE *pipe;
    char tmp[32];

    *dword_read = 0;

    sprintf(cmd, "pciconf -r pci%d:%d:%d 0x%x",
            bus, device, function, register_number);

    pipe = popen(cmd, "r");
    if (!pipe) {
        printf("ERROR: Failed to open pciconf pipe!\n");
        return DBG_STATUS_FAILURE;
    }

    if (!fgets(line, sizeof(line), pipe)) {
        printf("ERROR: Failed to get config dword\n");
        pclose(pipe);
        return DBG_STATUS_FAILURE;
    }

    sscanf(line, "%s %s", tmp, tmp);
    *dword_read = strtoul(tmp, NULL, 16);

    pclose(pipe);

//printf("pcicfg read: %d:%d:%d (0x%04x = 0x%08x)\n", bus, device, function, register_number, *dword_read);

    return DBG_STATUS_SUCCESS;
}

