
#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include "man_types.h"
#include "edebug_types.h"
#include "bcmtype.h"
#include "tcl_io.h"

struct termios oldterm1;
struct termios newterm1;

static void set_unbuffered(void)
{
    tcgetattr(STDIN_FILENO, &oldterm1);

    /*
     * For some reason the VMIN/VTIME are defaulting to 4/0
     * respectively on Solaris. This really jacks with user
     * input. Changing back to 1/0 default.
     */
    oldterm1.c_cc[VMIN]  = 1;
    oldterm1.c_cc[VTIME] = 0;

    newterm1 = oldterm1;
    newterm1.c_lflag &= ~(ICANON | ECHO);

    tcsetattr(STDIN_FILENO, TCSANOW, &newterm1);
}

static void set_buffered(void)
{
    tcsetattr(STDIN_FILENO, TCSANOW, &oldterm1);
}

void os_if_change_color(u8_t color,        /* Color to set */
                        u8_t fore_or_back) /* Forground or background */
{
    fprintf(stdout, "\x1b\[%u%um", fore_or_back, color);
    fflush(stdout);
}

void os_if_restore_color(void)
{
    fprintf(stdout, "\x1b\[0m");
    fflush(stdout);
}

char getch(void)
{
    return tcl_io_getch();
}

char os_if_getch(void)
{
    int ch = -1;

    set_unbuffered();

    while (ch == -1)
    {
        ch = getchar();
    }

    set_buffered();

    return ch;
}

void os_if_save_cursor(void)
{
    fprintf(stdout, "\x1b\[s");
    fflush(stdout);
}

void os_if_restore_cursor(void)
{
    fprintf(stdout, "\x1b\[u");
    fflush(stdout);
}

void os_if_print(char *buf, u16_t len)
{
    fprintf(stdout, "%s", buf);
    fflush(stdout);
}

FILE* os_if_log_fopen(const char *file_name)
{
    return fopen(file_name, "a");
}

