#include <stdio.h>
#include <string.h>
#include <libdevinfo.h>
#include <sys/pci.h>
#include "man_types.h"
#include "os_utils.h"
#include "edebug_types.h"
#include "pcics_reg_driver.h"
#include "hw_access.h"

typedef struct _di_walk_data
{
    u32_t        dev_cnt;
    u32_t        num_everest;
    pci_info_t * pci_info;
} di_walk_data_t;

static u32_t total_pci_dev = (u32_t)-1;

static int cnt_dev_nodes(di_node_t node, void * arg)
{
    di_walk_data_t * p_di = (di_walk_data_t *)arg;
    p_di->dev_cnt++;
    return DI_WALK_CONTINUE;
}

static u32_t dbg_dev_info = 1;

static void get_more_dev_info(pci_info_t * pci_info)
{
    char cmd[128];
    char dladm_line[256];
    char kstat_line[256];
    FILE * dladm_pipe;
    FILE * kstat_pipe;
    char dl_ifc[32];
    char tmp[32];
    char ifc[32];
    u32_t ifc_inst;
    u32_t bus = (u32_t)-1, dev = (u32_t)-1, func = (u32_t)-1;
    int i, index = -1;
    int s11 = 0;

    dladm_pipe = popen("dladm show-dev 2>&1", "r");
    if (!dladm_pipe)
    {
        printf("ERROR - failed to open dladm pipe!\n");
        return;
    }

    fgets(dladm_line, sizeof(dladm_line), dladm_pipe);

    if (strcmp(dladm_line, "dladm: insufficient privileges") == 0)
    {
        printf("ERROR - must run as root\n");
        pclose(dladm_pipe);
        return;
    }

    if ((strcmp(dladm_line, "dladm: unknown subcommand 'show-dev'\n") == 0) ||
        (strcmp(dladm_line, "Unrecognized subcommand: 'show-dev'\n") == 0))
    {
        pclose(dladm_pipe);
        dladm_pipe = popen("dladm show-phys", "r");
        if (!dladm_pipe)
        {
            printf("ERROR - failed to open dladm pipe!\n");
            return;
        }
    }

    /* for each interface on the system... */
    while (fgets(dladm_line, sizeof(dladm_line), dladm_pipe))
    {
        if (strncmp(dladm_line, "LINK", 4) == 0)
        {
            s11 = 1;
            continue;
        }

        *dl_ifc = 0; /* just in case */
        if (s11)
        {
            /* grab DEVICE column because of interface renaming */
            sscanf(dladm_line, "%s %s %s %s %s %s",
                   tmp, tmp, tmp, tmp, tmp, dl_ifc);
        }
        else /* s10 */
        {
            sscanf(dladm_line, "%s %s %s %s %s %s %s %s",
                   dl_ifc, tmp, tmp, tmp, tmp, tmp, tmp, tmp);
        }

        *ifc = 0; /* just in case */
        sscanf(dl_ifc, "%4s%d", ifc, &ifc_inst);

        if (strcmp(ifc, "bnxe") != 0)
        {
            continue;
        }

        sprintf(cmd, "kstat -p -m %s -i %d -n stats -s devBDF", ifc, ifc_inst);
        kstat_pipe = popen(cmd, "r");
        if (!kstat_pipe)
        {
            printf("ERROR - failed to open kstat pipe!\n");
            continue;
        }

        if (!fgets(kstat_line, sizeof(kstat_line), kstat_pipe))
        {
            printf("WARNING - %s not supported (missing 'devBDF' kstat)\n",
                   dl_ifc);
            pclose(kstat_pipe);
            continue;
        }

        sscanf(kstat_line, "%s %d:%d:%d", tmp, &bus, &dev, &func);

        pclose(kstat_pipe);

        if (bus != -1 && dev != -1 && func != -1)
        {
            /* find BDF in pci_info array */
            for (i = 0; i < total_pci_dev; i++)
            {
                if ((pci_info[i].pci_bus == bus) &&
                    (pci_info[i].pci_device == dev) &&
                    (pci_info[i].pci_func == func))
                {
                    index = i;
                    break;
                }
            }
        }

        if (index == -1)
        {
            continue;
        }

        if (dbg_dev_info) {
            printf("-----\n     bdf: %04x:%02x.%02x\n",
                   bus, dev, func);
        }

        strcpy(pci_info[i].driver_name, ifc);
        strcpy(pci_info[i].eth_name, dl_ifc);

        sprintf(cmd, "kstat -p -m %s -i %d -n stats -s version", ifc, ifc_inst);
        kstat_pipe = popen(cmd, "r");
        if (!kstat_pipe)
        {
            printf("ERROR - failed to open kstat pipe!\n");
            continue;
        }

        if (!fgets(kstat_line, sizeof(kstat_line), kstat_pipe))
        {
            printf("WARNING - %s interface missing 'version' kstat\n", dl_ifc);
            pclose(kstat_pipe);
            continue;
        }

        sscanf(kstat_line, "%s %s", tmp, pci_info[i].driver_ver);

        if (dbg_dev_info) {
            printf("drv vers: %s\n", pci_info[i].driver_ver);
        }

        pclose(kstat_pipe);

        sprintf(cmd, "kstat -p -m %s -i %d -n stats -s versionBC", ifc, ifc_inst);
        kstat_pipe = popen(cmd, "r");
        if (!kstat_pipe)
        {
            printf("ERROR - failed to open kstat pipe!\n");
            continue;
        }

        if (!fgets(kstat_line, sizeof(kstat_line), kstat_pipe))
        {
            printf("WARNING - %s interface missing 'versionBC' kstat\n", dl_ifc);
            pclose(kstat_pipe);
            continue;
        }

        sscanf(kstat_line, "%s %s", tmp, pci_info[i].fw_ver);

        if (dbg_dev_info) {
            printf(" fw vers: %s\n", pci_info[i].fw_ver);
        }

        pclose(kstat_pipe);

        sprintf(cmd, "kstat -p -m %s -i %d -n stats -s macAddr", ifc, ifc_inst);
        kstat_pipe = popen(cmd, "r");
        if (!kstat_pipe)
        {
            printf("ERROR - failed to open kstat pipe!\n");
            continue;
        }

        if (!fgets(kstat_line, sizeof(kstat_line), kstat_pipe))
        {
            printf("WARNING - %s interface missing 'macAddr' kstat\n", dl_ifc);
            pclose(kstat_pipe);
            continue;
        }

        sscanf(kstat_line, "%s %s", tmp, pci_info[i].mac_addr);

        if (dbg_dev_info) {
            printf("     mac: %s\n-----\n", pci_info[i].mac_addr);
        }

        pclose(kstat_pipe);
    }

    pclose(dladm_pipe);
}

static int get_dev_info(di_node_t node, void * arg)
{
    di_walk_data_t * p_di = (di_walk_data_t *)arg;
    int num_ints;
    int * p_ints;
    int i = p_di->dev_cnt;
    u64_t addr;
#ifdef SPARCV9
    char line[200];
    char cmd[60];
    char device_bdf[20];
    char kstat_bdf[20];
    char tmp[32];
    char devid[32];
    char instance[32];
    FILE *pipe;
    FILE *pipe2;
    char *token;
#endif

    if ((num_ints = di_prop_lookup_ints(DDI_DEV_T_ANY, node,
                                        "reg", &p_ints)) > 0)
    {
        p_di->pci_info[i].pci_bus    = PCI_REG_BUS_G(p_ints[0]);
        p_di->pci_info[i].pci_device = PCI_REG_DEV_G(p_ints[0]);
        p_di->pci_info[i].pci_func   = PCI_REG_FUNC_G(p_ints[0]);
    }

#ifdef SPARCV9

    /*
     * For some reason the vendor-id and device-id fields are't exposed on
     * SPARC. Instead we'll use kstat to get these values.
     */

    sprintf(device_bdf, "%04x:%02x:%02x",
            p_di->pci_info[i].pci_bus,
            p_di->pci_info[i].pci_device,
            p_di->pci_info[i].pci_func);

    sprintf(cmd, "kstat -p -m bnxe -n stats -s devBDF");
    pipe = popen(cmd, "r");
    if (!pipe) {
        printf("ERROR - failed to open kstat pipe!\n");
        return DI_WALK_CONTINUE;
    }

    while (fgets(line, sizeof(line), pipe)) {
        sscanf(line, "%s %s", tmp, kstat_bdf);
        if (strcmp(device_bdf, kstat_bdf) != 0) {
            continue;
        }

        token = strtok(line, ":");
        sscanf(token, "%s", tmp);
        token = strtok(NULL, ":");
        sscanf(token, "%s", instance);

        sprintf(cmd, "kstat -p -m bnxe -i %s -n stats -s devID", instance);

        pipe2 = popen(cmd, "r");
        if (!pipe2) {
            printf("ERROR - failed to open kstat pipe!\n");
            continue;
        }

        if (fgets(line, sizeof(line), pipe2)) {
            sscanf(line, "%s %s", tmp, devid);

            token = strtok(devid, ":");
            p_di->pci_info[i].vendor_id = strtoul(token, NULL, 16);
            token = strtok(NULL, ":");
            p_di->pci_info[i].device_id = strtoul(token, NULL, 16);
        }

        pclose(pipe2);
        break;
    }

    pclose(pipe);

#else /* X86 */

    if ((num_ints = di_prop_lookup_ints(DDI_DEV_T_ANY, node,
                                        "vendor-id", &p_ints)) > 0)
    {
        p_di->pci_info[i].vendor_id = p_ints[0];
    }

    if ((num_ints = di_prop_lookup_ints(DDI_DEV_T_ANY, node,
                                        "device-id", &p_ints)) > 0)
    {
        p_di->pci_info[i].device_id = p_ints[0];
    }

#endif

    if (!IS_EVEREST(p_di->pci_info[i]))
    {
        p_di->dev_cnt++;
        return DI_WALK_CONTINUE;
    }

    p_di->num_everest++;

    printf("QLogic 10Gb device found: %04x:%02x.%02x %04x:%04x\n",
           p_di->pci_info[i].pci_bus,
           p_di->pci_info[i].pci_device,
           p_di->pci_info[i].pci_func,
           p_di->pci_info[i].vendor_id,
           p_di->pci_info[i].device_id);

    p_di->pci_info[i].irq = 0; /* XXX */
    p_di->pci_info[i].eth_state = ETH_D0;

    if ((num_ints = di_prop_lookup_ints(DDI_DEV_T_ANY, node,
                                        "assigned-addresses", &p_ints)) > 0)
    {
        if (num_ints >= 5) /* 5 ints per base addr specification */
        {
            /* BAR0 */
            addr = (((u64_t)p_ints[3] << 32) | (p_ints[2]));
            //printf("BAR0: 0x%016llx (%d)\n", addr, p_ints[4]);
            p_di->pci_info[i].region0_base_addr = addr;
            p_di->pci_info[i].region0_size      = p_ints[4];

            /* BAR1 */
            //addr = (((u64_t)p_ints[8] << 32) | (p_ints[7]));
            //printf("BAR1: 0x%016llx (%d)\n", addr, p_ints[9]);

            /* BAR2 */
            addr = (((u64_t)p_ints[13] << 32) | (p_ints[12]));
            //printf("BAR2: 0x%016llx (%d)\n", addr, p_ints[14]);
            p_di->pci_info[i].region2_base_addr = addr;
            p_di->pci_info[i].region2_size      = p_ints[14];
        }
    }

    p_di->dev_cnt++;
    return DI_WALK_CONTINUE;
}

dbg_status_t os_if_get_total_dev(
    u32_t * dev_found_p /* number of found devices */
    )
{
    di_node_t root_node;
    di_walk_data_t di;

    if ((root_node = di_init("/", DINFOCPYALL)) == DI_NODE_NIL)
    {
        printf("ERROR - di_init() failed\n");
        return DBG_STATUS_FAILURE;
    }

    memset(&di, 0, sizeof(di));
    di_walk_node(root_node, DI_WALK_CLDFIRST, &di, cnt_dev_nodes);

    *dev_found_p  = di.dev_cnt;
    total_pci_dev = di.dev_cnt;

    di_fini(root_node);

    return DBG_STATUS_SUCCESS;
}

dbg_status_t os_if_pci_scan(
    pci_info_t * pci_info_ar /* array to return the PCI devices info */
    )
{
    di_node_t root_node;
    di_walk_data_t di;
    u32_t i, ret_val;

    if ((root_node = di_init("/", DINFOCPYALL)) == DI_NODE_NIL)
    {
        printf("ERROR - di_init() failed\n");
        return DBG_STATUS_FAILURE;
    }

    memset(&di, 0, sizeof(di));
    di.pci_info = pci_info_ar;
    di_walk_node(root_node, DI_WALK_CLDFIRST, &di, get_dev_info);

    di_fini(root_node);

    dbg_set_total_br_dev(di.num_everest);

    get_more_dev_info(pci_info_ar);

    for (i = 0; i < dbg_get_total_dev(); i++)
    {
        if (IS_EVEREST(pci_info_ar[i]))
        {
            /* set current DUT that pci config space approach will be right */
            dbg_pcicfg_setdut(pci_info_ar[i].pci_domain,
                              pci_info_ar[i].pci_bus,
                              pci_info_ar[i].pci_device,
                              pci_info_ar[i].pci_func);

            /* read and parse link_control register */
            dbg_pcicfg_read(PCICFG_LINK_CONTROL, &ret_val);
            dbg_pcicfg_process_link_control_reg(pci_info_ar, i, ret_val);

            /* read and parse SRIOV info in config space */
            dbg_pcicfg_read((PCI_CFG_SRIOV_BASE + SRIOV_NUM_VFS_OFFSET),
                            &ret_val);

            if (ret_val)
            {
                pci_info_ar[i].total_vfs = ret_val & 0xFF;
                dbg_pcicfg_read((PCI_CFG_SRIOV_BASE + SRIOV_FIRST_VF_STRIDE_OFFSET),
                                &ret_val);
                pci_info_ar[i].first_vf_offset = ret_val & 0xFF;
                pci_info_ar[i].vf_stride = (ret_val & 0xFF00) >> 16;
            }

            /* read and parse SVID info from config space */
            dbg_pcicfg_read(PCICFG_SUBSYSTEM_VENDOR_ID_OFFSET, &ret_val);
            pci_info_ar[i].svid = (u32_t)((u16_t) ret_val);
        }
    }

    return DBG_STATUS_SUCCESS;
}

dbg_status_t os_if_read_configuration_dword(
    u8_t    domain,
    u16_t   bus,
    u8_t    device,
    u8_t    function,
    u32_t   register_number,        /* register offset */
    u32_t * dword_read              /* output buffer */
    )
{
    char line[200];
    char cmd[60];
    char device_bdf[20];
    char kstat_bdf[20];
    FILE *pipe;
    char tmp[32];
    char ifname[32];
    char instance[32];
    char *token;

    /* XXX
     * On Linux they use the 'lspci' command here which has the very
     * nice feature of dumping the entire PCI config space for a device.
     * We don't have that on Solaris so instead we have a two step process.
     * First we find the interface name from the BDF and then we send down
     * and ioctl to the driver to get the PCI config word. This sucks
     * because the ioctls are directed through the GLD interface when
     * opening /dev/bnxe#. The GLD layer will start the L2 interface, drop
     * down the ioctl command, stop the L2 interface, and return the result.
     * The GLD start/stop routines are very heavy and repeated calls here
     * will *really* slow down edebug. This code needs to be changed to
     * send the ioctl directly to the driver via the cb_ops ioctl as the
     * driver is already attached.
     */

    *dword_read = 0;

    sprintf(device_bdf, "%04x:%02x:%02x", bus, device, function);

    sprintf(cmd, "kstat -p -m bnxe -n stats -s devBDF");
    pipe = popen(cmd, "r");
    if (!pipe) {
        printf("ERROR - failed to open kstat pipe!\n");
        return DBG_STATUS_FAILURE;
    }

    /* parse each line in the output to find index in pci_info_ar */
    while (fgets(line, sizeof(line), pipe)) {
        sscanf(line, "%s %s", tmp, kstat_bdf);
        if (strcmp(device_bdf, kstat_bdf) != 0) {
            continue;
        }

        token = strtok(line, ":");
        sscanf(token, "%s", ifname);
        token = strtok(NULL, ":");
        sscanf(token, "%s", instance);
        strcat(ifname, instance);

        solaris_pcicfg_read_ifname(ifname, register_number, dword_read);
printf("pcicfg read: %s %s (0x%04x = 0x%08x)\n", ifname, device_bdf, register_number, *dword_read);

        break;
    }

    pclose(pipe);

    return DBG_STATUS_SUCCESS;
}

