#ifndef __DBG_OS_UTIL_IOCTL_H__
#define __DBG_OS_UTIL_IOCTL_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "edebug_types.h"
#include "bcmtype.h"
#include "common_os_utils.h"


#define BNX2X_IOCTL_NUM 0x89F0

#define BNX2X_VMWARE_CIM_CMD_ENABLE_NIC		0x0001
#define BNX2X_VMWARE_CIM_CMD_DISABLE_NIC	0x0002
#define BNX2X_VMWARE_CIM_CMD_REG_READ		0x0003
#define BNX2X_VMWARE_CIM_CMD_REG_WRITE		0x0004
#define BNX2X_VMWARE_CIM_CMD_GET_NIC_PARAM	0x0005
#define BNX2X_VMWARE_CIM_CMD_GET_NIC_STATUS	0x0006

extern int bnx2x_fd;

struct bnx2x_ioctl_cmd {
        char eth_name[16];
        void *p_data;
};

struct bnx2x_ioctl_reg_read_req {
	u32 reg_offset;
	u32 reg_value;
} __attribute__((packed));

struct bnx2x_ioctl_reg_write_req {
	u32 reg_offset;
	u32 reg_value;
} __attribute__((packed));

struct bnx2x_ioctl_get_nic_param_req {
	u32 version;
	u32 mtu;
	u8  current_mac_addr[8];
} __attribute__((packed));

struct bnx2x_ioctl_get_nic_status_req
{
	u32 nic_status; /*  1: Up, 0: Down */
} __attribute__((packed));

struct bnx2x_ioctl_req {
	u32 cmd;
	union {
		/* no struct for reset_nic command */
		struct bnx2x_ioctl_reg_read_req reg_read;
		struct bnx2x_ioctl_reg_write_req reg_write;
		struct bnx2x_ioctl_get_nic_param_req get_nic_param;
		struct bnx2x_ioctl_get_nic_status_req get_nic_status;
	} cmd_req;
} __attribute__((packed));

static inline int get_bnx2x_fd (char *eth_name)
{
        char full_eth_name[200];
        int fd;

        sprintf(full_eth_name, "%s%s", "/dev/", eth_name);
        fd = open (full_eth_name, O_RDWR);
        if (fd == -1) {
            sprintf(full_eth_name, "%s%s", "/vmfs/devices/char/vmkdriver/", eth_name);
            fd = open (full_eth_name, O_RDWR);
            if (fd == -1) {
                printf("error opening driver named %s\n", full_eth_name);
            }
        }

        return fd;
}
#endif /* __DBG_OS_UTIL_IOCTL_H__ */
