#include <stdio.h>
#include <string.h>
#include "man_types.h"
#include "os_utils.h"
#include "edebug_types.h"
#include "pcics_reg_driver.h"
#include "hw_access.h"

static s32_t total_pci_dev = -1;
static s32_t total_interfaces = -1;

static dbg_status_t parse_lspci_info(
    OUT   pci_info_t  * pci_info_ar
    );

static dbg_status_t parse_esxcfgnics_ethtool_info(
    OUT   pci_info_t  * pci_info_ar
    );

static dbg_status_t parse_ethtool_info(
    OUT   pci_info_t  * pci_info_ar,
    IN    u32_t         index
    );


#define PROC_PCI_DEVICES_FILE           "/proc/bus/pci/devices"
#define PROC_PCI_DEVICES_MAX_LINE_LEN   400
#define DEVICE_FOLDER_MAX_LEN           200
#define FILE_NAME_MAX_LEN               220
#define PCI_BDF_BUS_MASK                0xFF00
#define PCI_BDF_BUS_SHIFT               8
#define PCI_BDF_DEVICE_MASK             0xF8
#define PCI_BDF_DEVICE_SHIFT            5
#define PCI_BDF_FUNC_MASK               0x7
#define PCI_BDF_FUNC_SHIFT              0


dbg_status_t os_if_get_total_dev(
    OUT   u32_t       * dev_found_p          /* number of found devices */
    )
{
        char line[200];
        FILE* pipe;
        u32_t result = 0;

        if (total_pci_dev >= 0) {
		*dev_found_p = total_pci_dev;
		return DBG_STATUS_SUCCESS;
	}

        /* get number of devices from lspci output */
        pipe = popen("lspci -n", "r");
        if (!pipe) {
                printf("BW error!\n");
                return DBG_STATUS_FAILURE;
        }

        /* count number of lines in lspci to get num of devices */
        while (fgets(line, sizeof(line), pipe)) {
                result++;
        }

        *dev_found_p = result;
        total_pci_dev = *dev_found_p;

        pclose(pipe);

	return DBG_STATUS_SUCCESS;
}

dbg_status_t os_if_pci_scan(
    OUT   pci_info_t  * pci_info_ar         /* array to return the PCI devices info */
    )
{
        dbg_status_t status = DBG_STATUS_SUCCESS;
        u32_t i, ret_val;
        char tmp[40]="xxxxxxxxxxxxxxxxxxxx";

        status = parse_lspci_info(pci_info_ar);
        if (DBG_STATUS_SUCCESS != status) {
                return status;
        }

        status = parse_esxcfgnics_ethtool_info(pci_info_ar);
        if (DBG_STATUS_SUCCESS != status) {
                return status;
        }

        for (i = 0; i < dbg_get_total_dev(); i++) {

                if (IS_EVEREST(pci_info_ar[i])) {
                        /* set current DUT that pci config space approach will be right */
                        dbg_pcicfg_setdut(pci_info_ar[i].pci_domain, pci_info_ar[i].pci_bus, pci_info_ar[i].pci_device, pci_info_ar[i].pci_func);

                        /* read and parse link_control register */
                        dbg_pcicfg_read(PCICFG_LINK_CONTROL, &ret_val);
                        dbg_pcicfg_process_link_control_reg(pci_info_ar, i, ret_val);

                        /* read and parse SRIOV info in config space */
                        dbg_pcicfg_read(PCI_CFG_SRIOV_BASE + SRIOV_NUM_VFS_OFFSET, &ret_val);

                        if (ret_val) {
                                pci_info_ar[i].total_vfs = ret_val & 0xFF;
                                dbg_pcicfg_read (PCI_CFG_SRIOV_BASE + SRIOV_FIRST_VF_STRIDE_OFFSET, &ret_val);
                                pci_info_ar[i].first_vf_offset = ret_val & 0xFF;
                                pci_info_ar[i].vf_stride = (ret_val & 0xFF00) >> 16;
                        }

			/* read and parse SVID info from config space */
			dbg_pcicfg_read(PCICFG_SUBSYSTEM_VENDOR_ID_OFFSET, &ret_val);
			pci_info_ar[i].svid = (u32_t)((u16_t) ret_val);
                }
        }

        return DBG_STATUS_SUCCESS;
}

static dbg_status_t parse_lspci_info(
    OUT   pci_info_t  * pci_info_ar
    )
{
        char line[200];
        char cmd[60];
        char *token;
        FILE *pipe;
        u32_t i, tmp_u32, line_num, region0, region2, tmp2, br_devices = 0;
        char tmp_str[60];
        int lspci_format_hex = 0;

        /* BDF and vendor/device details are retrieved through lspci */
        pipe = popen("lspci -n", "r");
        if (!pipe) {
                printf("error opening lspci -n!\n");
                return DBG_STATUS_FAILURE;
        }

        i = 0;

	/* parse each line in the output according to lspci format */
	while (fgets(line, sizeof(line), pipe)) {

         pci_info_ar[i].pci_domain = 0;

		/* lspci -n can have to formats, depends whether domain is 0 */
		/* first try to read in the format where domain num is not 0 */
                if (lspci_format_hex ||
                    (7 != (sscanf(line, "%d:%d:%d.%d Class %x: %x:%x",
                                 &(pci_info_ar[i].pci_domain),
                                 &(pci_info_ar[i].pci_bus),
                                 &(pci_info_ar[i].pci_device),
                                 &(pci_info_ar[i].pci_func),
                                 &tmp_u32,
                                 &(pci_info_ar[i].vendor_id),
                                 &(pci_info_ar[i].device_id))))) {

                        /* if reached here coulnd't read in first format */
                        /* try second format */
                        if (6 != (sscanf(line, "%x:%x.%x %x: %x:%x",
                                         &(pci_info_ar[i].pci_bus),
                                         &(pci_info_ar[i].pci_device),
                                         &(pci_info_ar[i].pci_func),
                                         &tmp_u32,
                                         &(pci_info_ar[i].vendor_id),
                                         &(pci_info_ar[i].device_id)))) {
                                /* if reached here coulnd't read in second format */
                                /* try third format */
                                if (6 != (sscanf(line, "%x:%x.%x Class %x: %x:%x",
                                                 &(pci_info_ar[i].pci_bus),
                                                 &(pci_info_ar[i].pci_device),
                                                 &(pci_info_ar[i].pci_func),
                                                 &tmp_u32,
                                                 &(pci_info_ar[i].vendor_id),
                                                 &(pci_info_ar[i].device_id)))) {
                                        if (7 != (sscanf(line, "%x:%x:%x.%x Class %x: %x:%x",
                                                  &(pci_info_ar[i].pci_domain),
                                                  &(pci_info_ar[i].pci_bus),
                                                  &(pci_info_ar[i].pci_device),
                                                  &(pci_info_ar[i].pci_func),
                                                  &tmp_u32,
                                                  &(pci_info_ar[i].vendor_id),
                                                  &(pci_info_ar[i].device_id)))) {
                                                        /* if reached here couldn't read in both format */
                                                        /* ERROR */
                                                        printf("error parsing lspci -n!\n");
                                                        return DBG_STATUS_FAILURE;
                                        } else {
                                                lspci_format_hex = 1;
                                        }
                                }
                        }
                }
             
                i++;
        }

        pclose(pipe);

        for (i = 0; i < total_pci_dev; i++) {
                /* for each EVEREST device - parse verbose lspci for more data */
                if (IS_EVEREST(pci_info_ar[i])) {
                        br_devices++;
                }
        }

        dbg_set_total_br_dev(br_devices);

        return DBG_STATUS_SUCCESS;
}

static dbg_status_t parse_esxcfgnics_ethtool_info(
    OUT   pci_info_t  * pci_info_ar
    )
{
        char line[200];
        char eth_name[20], pci_address[60];
        char *token;
        FILE *pipe;
        u32_t i;
        u32_t pci_domain, pci_bus, pci_device, pci_func, index;
        dbg_status_t status;

        /* get all valid interfaces through esxcfg-nics cmd */
        pipe= popen("esxcfg-nics -l", "r");
        if (!pipe) {
                printf("error opening esxcfg-nics\n");
                return DBG_STATUS_FAILURE;
        }

        /* first line is the header. Jump over it */
        fgets(line, sizeof(line), pipe);

        /* parse each line in the output to find interface names */
        while (fgets(line, sizeof(line), pipe)) {

                        /* first word is the interface name */
                        token=strtok(line, " ");
                        sprintf(eth_name, "%s", token);

                        /* second word is PCI address */
                        token=strtok(NULL, " ");

                        pci_domain = pci_bus = pci_device = pci_func = 0;

                        if (sscanf(token, "%4x:%2x:%2x.%2x", &pci_domain, &pci_bus, &pci_device, &pci_func) != 4)
                                sscanf(token, "%2x:%2x.%2x", &pci_bus, &pci_device, &pci_func);

                        /* search if BDF is QLogic device according to what we know so far */
                        index = -1;
                        for (i = 0; i < total_pci_dev; i++) {
                                if (IS_EVEREST(pci_info_ar[i]) &&
                                    pci_info_ar[i].pci_domain == pci_domain &&
                                    pci_info_ar[i].pci_bus == pci_bus &&
                                    pci_info_ar[i].pci_device == pci_device &&
                                    pci_info_ar[i].pci_func == pci_func) {
                                        index = i;
                                        break;
                                }
                        }

                        /* if found device in array, insert additional data to it from esxcfg-nics and ethtool */
                        if (index != -1 ) {
                                sprintf(pci_info_ar[index].eth_name, "%s", eth_name);

                                /* third word is driver name */
                                token=strtok(NULL, " ");
                                sprintf(pci_info_ar[index].driver_name, "%s", token);

                                /* seventh word is MAC address */
                                token=strtok(NULL, " ");
                                token=strtok(NULL, " ");
                                token=strtok(NULL, " ");
                                token=strtok(NULL, " ");
                                sprintf(pci_info_ar[index].mac_addr, "%s", token);

                                /* every device in esxcfg-nics is already loaded */
                                pci_info_ar[index].eth_state = ETH_D0;

                                 /* parse restults from ethtool */
                                status = parse_ethtool_info(pci_info_ar, index);
                                if (DBG_STATUS_SUCCESS != status) {
                                        pclose(pipe);
                                        return status;
                                }
                        }
        }

        pclose(pipe);

        return DBG_STATUS_SUCCESS;
}

static dbg_status_t parse_ethtool_info(
    OUT   pci_info_t  * pci_info_ar,
    IN    u32_t         index
    )
{
        char line[200];
        char cmd[60];
        char *token;
        FILE *pipe;
        u32_t i;

        /* build ethtool get info command for the input interface */
        sprintf(cmd, "ethtool -i %s", pci_info_ar[index].eth_name);
        pipe= popen(cmd, "r");
        if (!pipe) {
                printf("error opening ethtool -i for interface %s\n", pci_info_ar[index].eth_name);
                return DBG_STATUS_FAILURE;
        }

        /* parse each line in the output */
        while (fgets(line, sizeof(line), pipe)) {
                token=strtok(line, " ");

                if (0 == strcmp("version:", token)) {
                        token=strtok(NULL, " ");
                        sscanf(token, "%s", &(pci_info_ar[index].driver_ver));
                }

                if (0 == strcmp("firmware-version:", token)) {
                        char fw_ver[512];
                        int fw_ver_offset = 0;

                        token=strtok(NULL, " ");
                        while (token) {
                                strcpy(fw_ver + fw_ver_offset, token);
                                fw_ver_offset += strlen(token);
                                fw_ver[fw_ver_offset]='_';
                                fw_ver_offset++;
                                token=strtok(NULL, " ");
                        }

                        if (fw_ver_offset) {
                                sscanf(fw_ver, "%s", &(pci_info_ar[index].fw_ver));
                        } else {
                                sscanf("invalid", "%s", &(pci_info_ar[index].fw_ver));
                        }
                }
        }

        pclose(pipe);

        return DBG_STATUS_SUCCESS;
}

dbg_status_t os_if_read_configuration_dword(
    IN    u16_t   domain,
    IN    u16_t   bus,
    IN    u8_t    device,
    IN    u8_t    function,
    IN    u32_t   register_number,        /* register offset */
    OUT   u32_t * dword_read              /* output buffer */
    )
{
        char line[200];
        char cmd[60];
        char device_bdf_opt_1[20];
        char device_bdf_opt_2[20];
        char device_bdf_opt_3[20];
        char device_bdf_opt_4[20];
        char device_bdf_opt_5[20];
	char device_bdf_opt_6[20];
        char reg_offset_line_start[20];
        char *token;
        FILE *pipe;
        u32_t i, result = 0;
        u8_t tmp;

        /* send lspci -d command to get all pcicfg data of all pci devices */
        pipe= popen("lspci -d 2> /dev/null", "r");
        if (!pipe) {
                printf("error openeing lspci -xxx\n");
                return DBG_STATUS_FAILURE;
        }

        /* validate that lspci works, otherwise run "lspci -xxx" */
        fgets(line, sizeof(line), pipe);
	
        if ((line[0] >= '0') &&  (line[0] <= '9')) {
                fseek(pipe, 0, SEEK_SET);
        } else {
                pipe= popen("lspci -xxx 2> /dev/null", "r");
        }        

        sprintf(device_bdf_opt_1, "%d:%d:%d.%d", domain, bus, device, function);
        sprintf(device_bdf_opt_2, "%d:%d.%d", bus, device, function);
        sprintf(device_bdf_opt_3, "%03d:%03d:%02d.%d", 0, bus, device, function);
        sprintf(device_bdf_opt_4, "%02x:%02x.%d", bus, device, function);
        sprintf(device_bdf_opt_5, "%02x:%02x:%02x.%d", 0, bus, device, function);
	sprintf(device_bdf_opt_6, "%04x:%02x:%02x.%d", domain, bus, device, function);
        sprintf(reg_offset_line_start, "%x", register_number & 0xF0);

        /* parse each line in the output */
        while (fgets(line, sizeof(line), pipe)) {
		token=strtok(line, " ");
		
                /* search if line starts with the device we are looking for */
                if ((0 == strcmp(device_bdf_opt_1, token)) ||
                    (0 == strcmp(device_bdf_opt_2, token)) ||
                    (0 == strcmp(device_bdf_opt_3, token)) ||
                    (0 == strcmp(device_bdf_opt_4, token)) || 
                    (0 == strcmp(device_bdf_opt_5, token)) ||
		    (0 == strcmp(device_bdf_opt_6, token))) {

                        /* if we're here - we found the device we're looking for */
                        while (fgets(line, sizeof(line), pipe)) {
                                token=strtok(line, ":");

                                /* search for the line with the address we're looking for */
                                if (0 == strcmp(reg_offset_line_start, token)) {
                                        /* found relevant line */
                                        if (register_number & 0xF > 0xC) {
                                                printf("error - not supported\n");
                                                pclose(pipe);
                                                return DBG_STATUS_FAILURE;
                                        }
                                        for (i = 0; i < (register_number & 0xF); i++) {
                                                token=strtok(NULL, " ");
                                        }
                                        token=strtok(NULL, " ");
                                        sscanf(token, "%x", &tmp);
                                        result = tmp ;
                                        token=strtok(NULL, " ");
                                        sscanf(token, "%x", &tmp);
                                        result |= tmp << 8;
                                        token=strtok(NULL, " ");
                                        sscanf(token, "%x", &tmp);
                                        result |= tmp << 16;
                                        token=strtok(NULL, " ");
                                        sscanf(token, "%x", &tmp);
                                        result |= tmp << 24;

                                        *dword_read = result;
                                        pclose(pipe);
                                        return DBG_STATUS_SUCCESS;
                                }
                        }
                } 
        }

        pclose(pipe);
        return DBG_STATUS_FAILURE;
}

