#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include "bcmtype.h"
#include "edebug_types.h"
#include "dbg_os_utils.h"
#include "os_utils.h"
#include "mem_track.h"
#include "dbg_hmem_def.h"
#include "man_types.h"
#include "dev_info.h"
#include "license.h"
#include "shmem.h"
#include "dbg_shmem.h"

/* externals */
extern int dev_mem_fd;

/* Macros */
#define DEREF_MEM_ENTRY(e) \
          (mem_entry_t *) (((u8_t *) (e)) - sizeof(mem_entry_t))

dbg_status_t os_if_map_iomem (
        u32_t         nbytes,
        dbg_phys_addr_t   phy_addr,
        dbg_virt_addr_t * virt_addr
        )
{   
    return DBG_STATUS_SUCCESS;
}


dbg_status_t os_if_unmap_iomem (
    dbg_virt_addr_t virt_addr,
    u32_t            nbytes
    )
{
    return DBG_STATUS_SUCCESS;
}

dbg_status_t os_if_alloc_pcontig(
     IN    u32_t           nbytes,
     OUT   dbg_virt_addr_t   * virt_addr_p,  /* User space virtual address */
     OUT   dbg_phys_addr_t   * phy_addr_p   /* PCI bus address */
        )
{	
    b10_phys_buffer_param_t param = {0};
    b10_phys_buffer_data_t result = {0};
    BOOL        success = FALSE;
    u32_t       ioctl_bytes_written = 0;
    dev_if_t   *cur_dev = dbg_pci_devices[PF_IDX_BY_HDL(xdbg_cur_dev)].drv_dev_if_p;

    param.opcode = B10_IOC_PHYS_BUFFER_ALLOC;
    param.param.size = nbytes;

    success = DeviceIoControl(
        (HANDLE)(cur_dev->os_params.dev),
        B10_IOC_PHYS_BUFFER,
        &param,
        sizeof(param),
        &result,                        
        sizeof(result),                 
        &ioctl_bytes_written,           
        NULL);                          

    if (!success) 
    {
        printf("os_if_alloc_pcontig: IOCTL failed.\n");
        return DBG_STATUS_FAILURE;
    }
    
    *virt_addr_p = result.vaddr;
    *phy_addr_p = result.paddr;

    return DBG_STATUS_SUCCESS;
}


dbg_status_t os_if_free_pcontig(
    IN    dbg_virt_addr_t     virt_addr
    )
{   
    b10_phys_buffer_param_t param = {0};
    BOOL        success = FALSE;
    u32_t       ioctl_bytes_written = 0;
    dev_if_t    *cur_dev = dbg_pci_devices[PF_IDX_BY_HDL(xdbg_cur_dev)].drv_dev_if_p;

    param.opcode = B10_IOC_PHYS_BUFFER_FREE;
    param.param.vaddr = virt_addr;

    success = DeviceIoControl(
        (HANDLE)(cur_dev->os_params.dev),
        B10_IOC_PHYS_BUFFER,
        &param,
        sizeof(param),
        NULL,                       // v   OUT
        0,                          // v
        &ioctl_bytes_written,       // v   OUT
        NULL);                      // v

    if (!success) 
    {
        printf("os_if_free_pcontig: IOCTL failed.\n");
        return DBG_STATUS_FAILURE;
    }

    return DBG_STATUS_SUCCESS;
}

