#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <windows.h>
#include <initguid.h>
#include <setupapi.h>
#include <devguid.h>
#include <regstr.h>
#include "edebug_types.h"
#include "os_utils.h"
#include "edebug_types.h"
#include "pcics_reg_driver.h"
#include "hw_access.h"
#include "setupapi_if.h"

static s32_t total_pci_dev = -1;
static s32_t total_interfaces = -1;
static u32_t is_setupapi_initialized = 0;

/* holders for setupapi.dll */
static setupapi_if_t setupapi_if;
HMODULE	setupapi_module = NULL;

#define PROC_PCI_DEVICES_MAX_LINE_LEN   400
#define DEVICE_FOLDER_MAX_LEN           200
#define FILE_NAME_MAX_LEN               220
#define PCI_BDF_BUS_MASK                0xFF00
#define PCI_BDF_BUS_SHIFT               8
#define PCI_BDF_DEVICE_MASK             0xF8
#define PCI_BDF_DEVICE_SHIFT            5
#define PCI_BDF_FUNC_MASK               0x7
#define PCI_BDF_FUNC_SHIFT              0

/* functions prototypes */
static void extract_location_info(
						   IN char* location_info,
						   OUT u8_t* device,
						   OUT u8_t* function,
						   OUT u8_t* bus
						   );

static dbg_status_t pci_scan (IN pci_info_t  * pci_info_ar, OUT u32_t *p_num_devices)
{	
	DWORD idx = 0, dwSize, i;
	ULONG dev_if_detail_size = 0;
	HDEVINFO dev_info_set = NULL;	
	SP_DEVICE_INTERFACE_DETAIL_DATA *dev_if_detail = NULL;
	SP_DEVINFO_DATA devinfo_data;	
	SP_DEVICE_INTERFACE_DATA dev_if_data;		
	BOOL success;	
	CONST GUID InterfaceClassGuid = GUID_DEVINTERFACE_B10BDRV;
	char dev_desc[MAX_PATH];
	char location_info [200];
	pci_info_t tmp_pci_info;

	/* get set of supported devices according to GUID */
	dev_info_set = setupapi_if.pSetupDiGetClassDevs(&GUID_DEVINTERFACE_B10BDRV, NULL, NULL, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);	
	if (dev_info_set == INVALID_HANDLE_VALUE)
	{
		printf("error in response of pSetupDiGetClassDevs\n");
		return DBG_STATUS_FAILURE;
	}

	/* iterate through generated devices set */
	for(idx = 0; ; idx++)
	{
		/* clear data of the device */
		memset( &dev_if_data, 0, sizeof( dev_if_data ) );
		dev_if_data.cbSize = sizeof(dev_if_data);

		/* get device according to the index */
		success = setupapi_if.pSetupDiEnumDeviceInterfaces( dev_info_set, NULL, &GUID_DEVINTERFACE_B10BDRV, idx, &dev_if_data );		
		if(!success)
		{
			break; 
		}

		/* get size of device details */
		success = setupapi_if.pSetupDiGetDeviceInterfaceDetail(
		 		dev_info_set,
	 			&dev_if_data,
 				NULL,
				0,
				&dev_if_detail_size,
				NULL);

		if(!success)
		{
			/* due to unknown reason this function returns failure even though its response is valid */
			// printf("error in response of pSetupDiGetDeviceInterfaceDetail\n");			
		}	

		/* allocate memory for device datails */
		dev_if_detail = (SP_DEVICE_INTERFACE_DETAIL_DATA *) malloc(dev_if_detail_size);
		if(dev_if_detail == NULL)
		{
			printf("error allocating memory for dev_if_detail\n");
			return DBG_STATUS_FAILURE;
		}
		memset( dev_if_detail, 0, dev_if_detail_size ) ;
		dev_if_detail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
		devinfo_data.cbSize = sizeof(SP_DEVINFO_DATA);

		/* get device details */
		success = setupapi_if.pSetupDiGetDeviceInterfaceDetail(
			dev_info_set,
			&dev_if_data,
			dev_if_detail,
			dev_if_detail_size,
			NULL,
			&devinfo_data);
		if(!success)
		{
			printf("error in response of pSetupDiGetDeviceInterfaceDetail\n");
			return DBG_STATUS_FAILURE;
		}

		if (pci_info_ar != NULL) {			
			memcpy(pci_info_ar[idx].eth_name, dev_if_detail->DevicePath, strlen(dev_if_detail->DevicePath));

			/* all devices found are QLogic and in valid state */
			pci_info_ar[idx].vendor_id = BROADCOM_VID;
			pci_info_ar[idx].device_id = DEVICE_ID_STILL_UNKNOWN;
			pci_info_ar[idx].eth_state = ETH_D0;

			/* get location info from registry */
			setupapi_if.pSetupDiGetDeviceRegistryProperty(dev_info_set, &devinfo_data, SPDRP_LOCATION_INFORMATION, NULL, 
													   location_info, sizeof(location_info), NULL);
			extract_location_info(location_info, &(pci_info_ar[idx].pci_device), &(pci_info_ar[idx].pci_func), &(pci_info_ar[idx].pci_bus));

			/* get driver friendly name from registry */
			setupapi_if.pSetupDiGetDeviceRegistryProperty(dev_info_set, &devinfo_data, SPDRP_DEVICEDESC, NULL, 
														  &(pci_info_ar[idx].driver_name), sizeof(pci_info_ar[idx].driver_name), NULL);			

		}	
	}

	/* free allocated memory and lists */
	success = setupapi_if.pSetupDiDestroyDeviceInfoList(dev_info_set);
	if (dev_if_detail) free(dev_if_detail);
	
	/* perform bubble sort to sort device according to pci BDF */
	if ((pci_info_ar != NULL) && idx)
	{
		while (idx-1)
		{
			for (i = 0; i < (idx-1); i++)
			{
				if ((pci_info_ar[i].pci_bus > pci_info_ar[i+1].pci_bus) ||
					((pci_info_ar[i].pci_bus == pci_info_ar[i+1].pci_bus) && (pci_info_ar[i].pci_device > pci_info_ar[i+1].pci_device)) || 
					((pci_info_ar[i].pci_bus == pci_info_ar[i+1].pci_bus) && (pci_info_ar[i].pci_device == pci_info_ar[i+1].pci_device) && (pci_info_ar[i].pci_func > pci_info_ar[i+1].pci_func)))
				{
					memcpy(&tmp_pci_info, &pci_info_ar[i], sizeof(pci_info_t));
					memcpy(&pci_info_ar[i], &pci_info_ar[i+1], sizeof(pci_info_t));
					memcpy(&pci_info_ar[i+1], &tmp_pci_info, sizeof(pci_info_t));
				}
			}
			idx--;
		}
	}	

	/* if needed, update number of devices found */
	if (p_num_devices != NULL) {
		*p_num_devices = idx;
	}
	
	return DBG_STATUS_SUCCESS;
}

static dbg_status_t init_setuap_api(void)
{	
	char buf[260];
	FARPROC	func = NULL;	

	if (!GetSystemDirectory( buf, sizeof( buf ) ))
	{
		printf("error: can not get system dir\n");
		return DBG_STATUS_FAILURE;
	}
	
	strcat( buf, "\\Setupapi.dll" );	
	
	setupapi_module = LoadLibraryEx( buf, NULL, 0 );
	if ( setupapi_module == NULL ) 
	{		
		printf("error: can not load %s\n", buf);
		return DBG_STATUS_FAILURE;
	}	

	func = GetProcAddress( setupapi_module, "SetupDiEnumDeviceInterfaces" );
	if ( func == NULL ) 
	{
		printf("error: can not get from dll func SetupDiEnumDeviceInterfaces\n");
		return DBG_STATUS_FAILURE;
	}
	setupapi_if.pSetupDiEnumDeviceInterfaces = ( SETUPDIENUMDEVICEINTERFACES )func;
	
	func = GetProcAddress( setupapi_module, "SetupDiGetClassDevsA" );
	if ( func == NULL ) 
	{
		printf("error: can not get from dll func SetupDiGetClassDevsA\n");
		return DBG_STATUS_FAILURE;
	}	
	setupapi_if.pSetupDiGetClassDevs = ( SETUPDIGETCLASSDEVS) func;	

	func = GetProcAddress( setupapi_module, "SetupDiGetDeviceInterfaceDetailA" );
	if ( func == NULL ) 
	{
		printf("error: can not get from dll func SetupDiGetDeviceInterfaceDetailA\n");
		return DBG_STATUS_FAILURE;
	}
	setupapi_if.pSetupDiGetDeviceInterfaceDetail = ( SETUPDIGETDEVICEINTERFACEDETAIL) func;	

	func = GetProcAddress( setupapi_module, "SetupDiGetDeviceRegistryPropertyA" );
	if ( func == NULL ) 
	{
		printf("error: can not get from dll func SetupDiGetDeviceRegistryPropertyA\n");
		return DBG_STATUS_FAILURE;
	}
	setupapi_if.pSetupDiGetDeviceRegistryProperty = ( SETUPDIGETDEVICEREGISTRYPROPERTY )func;

	func = GetProcAddress( setupapi_module, "SetupDiDestroyDeviceInfoList" );
	if ( func == NULL ) 
	{
		printf("error: can not get from dll func SetupDiDestroyDeviceInfoList\n");
		return DBG_STATUS_FAILURE;
	}
	setupapi_if.pSetupDiDestroyDeviceInfoList = ( SETUPDIDESTROYDEVICEINFOLIST )func;

	func = GetProcAddress( setupapi_module, "SetupDiEnumDeviceInfo" );
	if ( func == NULL ) 
	{
		printf("error: can not get from dll func SetupDiEnumDeviceInfo\n");
		return DBG_STATUS_FAILURE;
	}
	setupapi_if.pSetupDiEnumDeviceInfo = ( SETUPDIENUMDEVICEINFO )func;	

	return DBG_STATUS_SUCCESS;
}

static void extract_location_info(
						   IN char* location_info,
						   OUT u8_t* device,
						   OUT u8_t* function,
						   OUT u8_t* bus
						   )
{    
    char location_info_str[200];
	char* token;
	char last_token[200];
	char seps[]   = " ,\t\n";	

	strcpy(last_token,"");
    // save the location_info in temporary string
    strcpy(location_info_str, location_info);

	// "PCI bus 2, device 0, function 1"
	token = strtok( location_info_str, seps );
	while (token != NULL) {
		if (strcmp(last_token,"device")==0){
		    *device = atoi(token);
		}
		if (strcmp(last_token,"function")==0){
		   *function = atoi(token);
		}
		if (strcmp(last_token,"bus")==0){
		    *bus = atoi(token);
		}
		strcpy(last_token,token);
		token = strtok( NULL, seps );
	}	
}	

dbg_status_t os_if_get_total_dev(
    OUT   u32_t       * dev_found_p          /* number of found devices */
    )
{
	dbg_status_t status;
	if (total_pci_dev == -1) {
		if (is_setupapi_initialized == 0) {
			status = init_setuap_api();
			if (status != DBG_STATUS_SUCCESS) {
				return status;
			}

			is_setupapi_initialized = 1;

			status = pci_scan(NULL, dev_found_p);
			if (status != DBG_STATUS_SUCCESS) {
				return status;
			}	

			total_pci_dev = *dev_found_p;

		}
	}
	else {
		*dev_found_p = total_pci_dev;
		return DBG_STATUS_SUCCESS;
	}
	
	return DBG_STATUS_SUCCESS;	
}

dbg_status_t os_if_pci_scan(
    OUT   pci_info_t  * pci_info_ar         /* array to return the PCI devices info */ 
    )
{	
	return pci_scan(pci_info_ar, NULL);      
}

static dbg_status_t parse_lspci_info(
    OUT   pci_info_t  * pci_info_ar          
    )
{
  
        return DBG_STATUS_SUCCESS;
}

static dbg_status_t parse_ifconfig_ethtool_info(
    OUT   pci_info_t  * pci_info_ar          
    )
{
       

        return DBG_STATUS_SUCCESS;
}

static dbg_status_t parse_ethtool_info(
    OUT   pci_info_t  * pci_info_ar,
    IN    char    * if_name,  
    IN    char    * mac_addr       
    )
{
      
               
        return DBG_STATUS_SUCCESS;
}

dbg_status_t os_if_read_configuration_dword(
    IN    u16_t   domain,
    IN    u16_t   bus,                 
    IN    u8_t    device,             
    IN    u8_t    function,    
    IN    u32_t   register_number,        /* register offset */
    OUT   u32_t * dword_read              /* output buffer */
    )
{
	dev_if_t * cur_dev = XDBG_CUR_DEV_IF;
	windows_pcicfg_read(cur_dev->os_params, register_number, dword_read);
	return DBG_STATUS_SUCCESS;
}

