# This TCL script loads all needed memories from a Everest by using the 
# GRC through the X-diag interface.
# Written by AmirF, QLogic, Oct 2006

# Due to a bug in tcl 8.3, The script keeps opening and closing the output file
# This is done to prevent the appearance  of "more..., 'q' for quit"

# This script uses the regsUtils.tcl script to parse the RDF into
# variable arrays: regsOffset regsType regsSize and regsResetVal

namespace eval GRC_dump {
    
    namespace export grcDump addrDump loadRegs freeRegs Exists define_assert translateLinuxFile changeRdfDir linuxAddrDump dumpFromLinuxFile linuxLoadRegs
    variable GRC_dump_ver {$Revision: #62 $}
    
    # const declaration
	variable is_linux_dump false
	variable has_used_storm_letters false
    variable extension_data_string ""
    variable current_chip_revision ""
    variable chip_num_reg 0xa408
    variable chip_rev_reg 0xa40c
    variable chip_metal_reg 0xa404
    variable pcicfg_offset 0x2000
    variable ME_register_address 0x450000
    variable waitPstatus "false"
	variable auto_split 0
    variable line_size
    variable number_of_lines
    variable ram_size
    variable ctx_type_size
    variable ram_offset
    variable passive_buffer_size
    variable qm_size
    variable Cam_offset
    variable activeIor
    variable brb1_ram_size
    variable pcieCore
    variable pcieCoreRelated
    variable semiStats
    variable MCP_scpd
    variable MCP_regs
    variable CHIPS
	variable preset_lists_init false
    variable PRAM_FW_FLAGS
    variable rdfs_files
    variable scriptInc
    variable dump_no_pci
    variable chip_mc_ver
    variable pcicfg_reads
    variable HCRelated
    variable int_table_regs
    variable withHC
    variable withInterruptTable
    variable idle_chk_list
    variable idleChkDelay
    variable register_information
    variable current_waitp_status
	variable ignore_mc_chip
	set ignore_mc_chip false
	variable default_rdf_dir
	set default_rdf_dir	""
    global   assertOffsets
    global   assertFirstJump
    global   assertJump
    global   assertChip
    variable chip_ver
    variable redirected_output
    variable no_asserts
    variable used_chip
    variable outFileName
    variable crash_text
    variable current_chosen_path
    variable path_to_device
    array set path_to_device {}
    #current state can be reg, cam, or idle
    variable current_state
    variable all_states
    set crash_text "crash"
    set no_crash_text "nocrash"
    set no_asserts ""
    set current_state reg
	variable const_igu 0
	variable const_vfc	1
	variable const_split 2
	variable const_static_debug 3
	variable const_xstorm 0
	variable const_tstorm 1
	variable const_ustorm 2
	variable const_cstorm 3
	variable const_max_vfc_write_size 2
	variable const_other_block 4
    set all_states {E1 E1H E2 E3 E3B0 crash nocrash}
    variable timer_info
    variable is_partial_dump
    set is_partial_dump false
    variable is_with_idle
    set is_with_idle true
    variable dump_chip_ver
    variable old_version
    set old_version true
    variable chosen_dev_num
    variable vfc_sizes
    array set vfc_sizes {alu_num_lines 12 cam_num_lines 272 ram_num_lines 765}
    variable dev_info_list
	variable addr_dump_properties
	array set addr_dump_properties {}
    variable with_cfc_string_cam
    variable current_page_mode_state
    set current_page_mode_state false
    variable current_page_mode_value
    set current_page_mode_value 0
    array set pagemode {}
    set pagemode(e2,read) {}
    set pagemode(e2,write) {}
    set pagemode(e2,values) {}
    set pagemode(e3,read) {}
    set pagemode(e3,write) {}
    set pagemode(e3,values) {}
   
    variable mcp_scratchpad
	array set mcp_scratchpad {}
    variable const_ri_e1 	[expr 0x1]
    variable const_ri_e1h 	[expr 0x2]
    variable const_ri_e2 	[expr 0x4]
    variable const_ri_e3 	[expr 0x8]
	variable const_ri_e3b0 	[expr 0x10]
    variable const_ri_nocrash	[expr 0x100]
    variable const_ri_p0_dump	[expr 0x200]
    variable const_ri_p1_dump	[expr 0x400]
    variable const_ri_split_4   [expr 0x1000]
    variable const_ri_split_8   [expr 0x2000]
    variable vfc_status_bits
    array set vfc_status_bits {IS_READY 0 IS_BUSY 1 IS_ONGOING_SEND 2}
    
    
    set assertFirstJump 0x20
    set assertJump      0x10
    set idle_chk_list ""
    set idleChkDelay 50
    
    set int_table_regs SEM_REG_INT_TABLE
    variable no_qstat 0
    variable FIRST_RUN 1
    variable resetRegs {MISC_REG_RESET_REG_1 MISC_REG_RESET_REG_2}
    variable resetRegsVals
    array set resetRegsVals {MISC_REG_RESET_REG_1 0xffffffff MISC_REG_RESET_REG_2 0x0}
    # array which holds registers' values needed to simulate the chip for dumping address (such as chip version, mcp signature and so on)
    variable neededRegs
	
    array set neededRegs [list E1,[expr {$chip_num_reg*1}] 0x164e E1H,[expr {$chip_num_reg*1}] 0x1650 E2,[expr {$chip_num_reg*1}] 0x1662 E3,[expr {$chip_num_reg*1}] 0x168A]
	array set neededRegs [list E1,[expr {$chip_rev_reg*1}] 0 E1H,[expr {$chip_rev_reg*1}] 0 E2,[expr {$chip_rev_reg*1}] 0 E3,[expr {$chip_rev_reg*1}] 0]
    variable chipsNames {e1 e1h e2 e3}
    array set CHIPS {0x164e 1 0x164E 1 0x164f 1.5 0x164F 1.5 0x1650 1.5 0x1662 2 0x1663 2 0x1651 2 0x1652 2 0x168E 3 0x168e 3 0x168A 3 0x168a 3 0x168D 3 0x168d 3}
	variable e_blocks
	set e_blocks {ATC BRB1 CCM CDU CFC CSDM DMAE DORQ MISC NIG PBF PGLUE_B PRS PXP2 PXP QM SRC TCM TM TSDM UCM USDM XCM XSDM IGU XSEM TSEM USEM CSEM IGU}
    array set rdfs_files {1 everest.rdf 1.5 evrst15.rdf 2 everst2.rdf 3 everst3.rdf 3b0 everst3b.rdf}
	variable page_mode_ids 
	array set page_mode_ids {tcm_st	1 brb 2}
	variable current_page_mode_id
	
	
	variable pfc_regs
	set pfc_regs {
{ MAC_Ctrl  0x00000 }
{ MAC__Ctrl    0x00006 }
{ MAC_TxPPPCtrl   0x00020 }
{ MAC_RxCtrl      0x0003a }
{ MAC_TxCtrl      0x0001c }       
{ MAC_GRxPP 0x00051 }
{ MAC_GTxPP 0x00024 }
{ MAC__COS0_XOFF_CNT 0x00009 }
{ MAC__COS1_XOFF_CNT 0x0000a }
{ MAC__COS2_XOFF_CNT 0x0000b }
{ MAC__COS3_XOFF_CNT 0x0000c }
{ MAC__COS4_XOFF_CNT 0x0000d }
{ MAC__COS5_XOFF_CNT 0x0000e }
{ MAC__COS6_XOFF_CNT 0x0000f }
{ MAC__COS7_XOFF_CNT 0x00010 }
{ MAC__COS8_XOFF_CNT 0x00011 }
{ MAC__COS9_XOFF_CNT 0x00012 }
{ MAC__COS10_XOFF_CNT      0x00013 }
{ MAC__COS11_XOFF_CNT      0x00014 }
{ MAC__COS12_XOFF_CNT      0x00015 }
{ MAC__COS13_XOFF_CNT      0x00016 }
{ MAC__COS14_XOFF_CNT      0x00017 }
{ MAC__COS15_XOFF_CNT      0x00018 }
{ MAC_GRxPF 0x00050 }
{ MAC_GTxPF 0x00023 }
{ MAC_Ctrl  0x00000 }
{ MAC_XGXS_Ctrl   0x00001 }
{ MAC_XGXS_Stat   0x00002 }
{ MAC_TxMuxCtrl   0x00003 }
{ MAC_RxMuxCtrl   0x00004 }
{ MAC_CntMaxSz    0x00005 }
{ MAC__Ctrl    0x00006 }
{ MAC__DA      0x00007 }
{ MAC__FIELD   0x00008 }
{ MAC__COS0_XOFF_CNT 0x00009 }
{ MAC__COS1_XOFF_CNT 0x0000a }
{ MAC__COS2_XOFF_CNT 0x0000b }
{ MAC__COS3_XOFF_CNT 0x0000c }
{ MAC__COS4_XOFF_CNT 0x0000d }
{ MAC__COS5_XOFF_CNT 0x0000e }
{ MAC__COS6_XOFF_CNT 0x0000f }
{ MAC__COS7_XOFF_CNT 0x00010 }
{ MAC__COS8_XOFF_CNT 0x00011 }
{ MAC__COS9_XOFF_CNT 0x00012 }
{ MAC__COS10_XOFF_CNT      0x00013 }
{ MAC__COS11_XOFF_CNT      0x00014 }
{ MAC__COS12_XOFF_CNT      0x00015 }
{ MAC__COS13_XOFF_CNT      0x00016 }
{ MAC__COS14_XOFF_CNT      0x00017 }
{ MAC__COS15_XOFF_CNT      0x00018 }
{ MAC_HCFC_CTRL   0x00019 }
{ MAC_HCFC_STATUS 0x0001a }
{ MAC_CoreSpare0  0x0001b }
{ MAC_TxCtrl      0x0001c }
{ MAC_TxMACSA     0x0001d }
{ MAC_TxMaxSz     0x0001e }
{ MAC_TxPseThr    0x0001f }
{ MAC_TxPPPCtrl   0x00020 }
{ MAC_TxSpare0    0x00021 }
{ MAC_GTxPOK      0x00022 }
{ MAC_GTxPF 0x00023 }
{ MAC_GTxPP 0x00024 }
{ MAC_GTxFCS      0x00025 }
{ MAC_GTxUCA      0x00026 }
{ MAC_GTxMCA      0x00027 }
{ MAC_GTxBCA      0x00028 }
{ MAC_GTxOvr      0x00029 }
{ MAC_GTxFrg      0x0002a }
{ MAC_GTxPkt      0x0002b }
{ MAC_GTx64 0x0002c }
{ MAC_GTx127      0x0002d }
{ MAC_GTx255      0x0002e }
{ MAC_GTx511      0x0002f }
{ MAC_GTx1023     0x00030 }
{ MAC_GTx1518     0x00031 }
{ MAC_GTx2047     0x00032 }
{ MAC_GTx4095     0x00033 }
{ MAC_GTx9216     0x00034 }
{ MAC_GTx16383    0x00035 }
{ MAC_GTxMAX      0x00036 }
{ MAC_GTxUfl      0x00037 }
{ MAC_GTxErr      0x00038 }
{ MAC_GTxByt      0x00039 }
{ MAC_RxCtrl      0x0003a }
{ MAC_RxMACSA     0x0003b }
{ MAC_RxMaxSz     0x0003c }
{ MAC_RxLSSCtrl   0x0003d }
{ MAC_RxLSSStat   0x0003e }
{ MAC_RxSpare0    0x0003f }
{ MAC_GRx64 0x00040 }
{ MAC_GRx127      0x00041 }
{ MAC_GRx255      0x00042 }
{ MAC_GRx511      0x00043 }
{ MAC_GRx1023     0x00044 }
{ MAC_GRx1518     0x00045 }
{ MAC_GRx2047     0x00046 }
{ MAC_GRx4095     0x00047 }
{ MAC_GRx9216     0x00048 }
{ MAC_GRx16383    0x00049 }
{ MAC_GRxMax      0x0004a }
{ MAC_GRxPkt      0x0004b }
{ MAC_GRxFCS      0x0004c }
{ MAC_GRxUCA      0x0004d }
{ MAC_GRxMCA      0x0004e }
{ MAC_GRxBCA      0x0004f }
{ MAC_GRxPF 0x00050 }
{ MAC_GRxPP 0x00051 }
{ MAC_GRxUO 0x00052 }
{ MAC_GRxJbr      0x00053 }
{ MAC_GRxOvr      0x00054 }
{ MAC_GRxCF 0x00055 }
{ MAC_GRxFLR      0x00056 }
{ MAC_GRxPOK      0x00057 }
{ MAC_GRxMEG      0x00058 }
{ MAC_GRxMEB      0x00059 }
{ MAC_GRxByt      0x0005a }
{ MAC_GRxUnd      0x0005b }
{ MAC_GRxFrg      0x0005c }
{ MAC_GRxErrByt   0x0005d }
{ MAC_GRxFrErr    0x0005e }
{ MAC_GRxIPJ      0x0005f }
{ MAC_TxLlfcCtrl  0x00060 }
{ MAC_TxLlfcMsgFlds     0x00061 }
{ MAC_RxLlfcMsgFlds     0x00062 }
{ MAC_RxLlfcMsgCnt      0x00063 }
{ MAC_TxTimeStampFIFORead     0x00064 }
{ MAC_TxTimeStampFIFOStatus   0x00065 }
}

	variable PB_Blocks {XPB UPB}
}



#functions for Groups

proc GRC_dump::isEmptyGroup {g} {
	if {[llength $g] == 0} {
		return true
	} else {
	return false
	}
}

proc GRC_dump::getSubGroup {group index} {
	set result_group {}
	set curr 1
	foreach item $group {
		if {[expr {$curr & $index}] > 0} {
			lappend result_group $item
}
	set curr [expr {$curr << 1}]
	}
	return $result_group
}

proc GRC_dump::getAllSubGroups {group} {
	set result_group {}
	set sgroup [llength $group]
	set num_groups [expr {1 << $sgroup}]
	for {set i 0} {$i < $num_groups} {incr i} {
		lappend result_group [getSubGroup $group $i]
	}
	return $result_group
}



proc GRC_dump::maskMCP {maskOn} {
	set mcp_parity_bits [expr {0xf0000000}]
	set non_mcp_parity [expr {0x0fffffff}]
	
	set reg_list {}
	for {set f 0} {$f < 2} {incr f} {
		for {set i 0} {$i < 8} {incr i} {
			lappend reg_list [reg_offset -n MISC_REG_AEU_ENABLE4_FUNC_[set f]_OUT_[set i]]
		}
	}
	#set reg_list [list 0xa078 0xa0f8 0xa108 0xa118 0xa1a8 0xa1b8]
	
	if {![set maskOn]} {
		puts "Clearing MCP Latched Attention"
		reg_write -n MISC_REG_AEU_CLR_LATCH_SIGNAL -v 0x780
	}
	
	foreach r $reg_list {
		# read modify write
		set old [reg read $r]
		if {$maskOn} {
			set old [expr {$old & $non_mcp_parity}]
		} else {
			set old [expr {$old | $mcp_parity_bits}]
			
		}
		reg write $r $old
	}
}


proc GRC_dump::changeRdfDir {new_dir} {
	variable default_rdf_dir
	set default_rdf_dir $new_dir
}

proc GRC_dump::rdfParse {rdf_file} {
	
	variable rdf_file_param
	set rdf_file_param $rdf_file
	
	namespace eval :: {
		variable GRC_dump::default_rdf_dir
		variable GRC_dump::rdf_file_param
		parse_rdf -clr -f [set GRC_dump::default_rdf_dir][set GRC_dump::rdf_file_param]
	}
}

proc GRC_dump::getPageModeId {name} {
	variable page_mode_ids
	
	return $page_mode_ids([set name])
}

proc GRC_dump::getCurrentPageModeId {} {
	variable current_page_mode_id
	return $current_page_mode_id
}

proc GRC_dump::setCurrentPageModeId {name} {
	variable current_page_mode_id
	set current_page_mode_id [getPageModeId $name]
}

proc GRC_dump::getMcpVer {} {
set shmem_offset [reg_read -n MISC_REG_SHARED_MEM_ADDR]
if {[expr {$shmem_offset}] == 0} {
	return "0.0.0"
}

set version_offset [expr {$shmem_offset + 8}]
set version_val [reg read $version_offset]
set high [expr {$version_val >> 24}]
set mid [expr {($version_val >> 16) & 0xff}]
set low [expr {($version_val >> 8) & 0xff}]
return "[set high].[set mid].[set low]"
}



proc GRC_dump::pfcAddressCalc {offset} {
	return [format "0x%x" [expr {0x10c00 + 8 * $offset}]]
}

# this subroutine added the data from the PFC
proc GRC_dump::pfcDumpStats {device } {
    
    variable outFile
	variable pfc_regs
    setFile
    
    variable dump_no_pci
    variable ::current
    set base 0
    if {$dump_no_pci} {
        set base [expr ($device-1)<<22]
        set func_num [expr {$device -1 }]
    } else {
        device $device -no_display
        set func_num [expr {$::current(FUNC)}]
    }

	    puts $outFile "PRINTING MEM:\tBMAC_STATS_$func_num"
        puts $outFile "***************************************************************"
        foreach couple $pfc_regs {
			set name [lindex $couple 0]
			set offset [lindex $couple 1]
			set address [pfcAddressCalc $offset]
			puts $outFile "[set name] : [reg read $address]"
		}
        puts $outFile "END\n" 
		close $outFile
}


proc GRC_dump::blockMaskingInit {} {
	variable block_masking_values
	catch {array unset block_masking_values}
	array set block_masking_values {}
}

proc GRC_dump::maskBlock {block_name addition} {

	variable block_masking_values
	
	set val_reg_name [set block_name]_REG_[set block_name]_PRTY_STS[set addition]
	set mask_reg_name [set block_name]_REG_[set block_name]_PRTY_MASK[set addition]

	
	set flag [catch {set old_mask [reg_read -n $mask_reg_name]}]        
	if {$flag || [lindex $old_mask 0] == "expected"} {
		return
	}
	
	maskParityMaskReg $mask_reg_name
	set flag [catch {set old_val [reg_read -n $val_reg_name]}]

	
	
	set block_masking_values([set block_name],[set addition],val) $old_val
	set block_masking_values([set block_name],[set addition],mask) $old_val
	
}

proc	GRC_dump::maskParityMaskReg {reg_name} {
	
	set size [reg_size -n $reg_name]
	
	if {$size >= 32} {
		set value 0xffffffff
	} else {
		set value [expr {(1 << $size) - 1}]
	}
	
	reg_write -n $reg_name -v $value
}

proc GRC_dump::maskPBBlock {block_name} {

	variable block_masking_values
	
	set val_reg_name [set block_name]_REG_PB_PRTY_STS
	set mask_reg_name [set block_name]_REG_PB_PRTY_MASK

	set addition {}
	set flag [catch {set old_mask [reg_read -n $mask_reg_name]}]
	if {$flag} {
		return
	}
	
	maskParityMaskReg $mask_reg_name
	set flag [catch {set old_val [reg_read -n $val_reg_name]}]

	
	
	set block_masking_values([set block_name],[set addition],val) $old_val
	set block_masking_values([set block_name],[set addition],mask) $old_mask
	
}

# this function assumed that block was masked
proc GRC_dump::restoreBlockMasking {block_name addition} {

	variable block_masking_values
	set clear_reg_name [set block_name]_REG_[set block_name]_PRTY_STS_CLR[set addition]
	set mask_reg_name [set block_name]_REG_[set block_name]_PRTY_MASK[set addition]
	set val_reg_name [set block_name]_REG_[set block_name]_PRTY_STS_WR[set addition]
	set parity_reg_name [set block_name]_REG_[set block_name]_PRTY_STS[set addition]

	#clearing the parity
	set curr_val [reg_read -n $clear_reg_name]
	
	#return mask value
	reg_write -n $mask_reg_name -v $block_masking_values([set block_name],[set addition],mask)
	
	return $curr_val
}

proc GRC_dump::restorePBBlockMasking {block_name} {
	variable block_masking_values
	set clear_reg_name [set block_name]_REG_PB_PRTY_STS_CLR
	set mask_reg_name [set block_name]_REG_PB_PRTY_MASK
	set val_reg_name [set block_name]_REG_PB_PRTY_STS_WR
	set parity_reg_name [set block_name]_REG_PB_PRTY_STS

	#clearing the parity
	set curr_val [reg_read -n $clear_reg_name]
	
	set addition {}
	reg_write -n $mask_reg_name -v $block_masking_values([set block_name],[set addition],mask)
	
	return $curr_val
}

proc GRC_dump::wasBlockMasked {block_name addition} {
	variable block_masking_values
	if {[info exists block_masking_values([set block_name],[set addition],val)]} {
		return true
	} else {
		return false
	}
}

proc GRC_dump::restoreAllBlocksMasking {} {
    variable e_blocks
	variable outFile
    setFile
    
	#Clear sem fast parity (will still read data from sem slow)
	foreach sem_block {CSEM USEM XSEM TSEM} {
	reg_write -n [set sem_block]_REG_SEM_FAST_PARITY_RST -v 1
	}
    puts $outFile "PRINTING BLOCK_PARITY"
    puts $outFile "***************************************************************"
	
    foreach block $e_blocks {
		foreach addition {_1 _0 {}} {
		
			# if block was masked
			if {[wasBlockMasked $block $addition]} {
				set curr_val [restoreBlockMasking $block $addition]
				if {[expr $curr_val]!= 0} {
					puts $outFile "[set block]_REG_[set block]_PRTY_STS[set addition] is $curr_val"
				}					
			
			}
		
		}
	}
	
	variable PB_Blocks
	
	foreach block $PB_Blocks {
		if {[wasBlockMasked $block {}]} {
			set curr_val [restorePBBlockMasking $block]
			if {[expr $curr_val]!= 0} {
					puts $outFile "[set block]_REG_PB_PRTY_STS is $curr_val"
				}					
		}
	}
	
    puts $outFile "END OF BLOCK_PARITY\n"

    close $outFile
	
	blockMaskingInit
}

proc GRC_dump::maskAllBlocks {} {
	variable e_blocks
	
	blockMaskingInit
	
	foreach block $e_blocks {
		foreach addition {_0 _1 {}} {
			maskBlock $block $addition
		}
	}
	
	maskPBBlocks
}

proc GRC_dump::maskPBBlocks {} {
	variable PB_Blocks
	
	foreach block $PB_Blocks {
		maskPBBlock $block
	}
}
# Upating the offsets of fast-mem registers according to the .rdf file
proc GRC_dump::UpdateFastMemOffsets {stormLetter} {
    variable waitP_offset
    variable active_reg_set
    variable Cam_offset
    variable chip_ver
    variable chip_mc_ver
    variable semiStats
    variable ram_offset
    variable activeIor
    array set Cam_offset {}
    array set semiStats {alignlast_pram_address 8}
    array set ram_offset {}
    array set activeIor {size 4096 line_size 32}
    
    
    
    
    set fast_mem_base [get_start_offset [reg_offset -n [set stormLetter]SEM_REG_FAST_MEMORY]]
    
    set activeIor(0)  [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_PAS_REG_SET0 $fast_mem_base]
    set activeIor(1)  [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_PAS_REG_SET1 $fast_mem_base]
    
    set ram_offset(0) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_INT_RAM0 $fast_mem_base]
    set ram_offset(1) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_INT_RAM1 $fast_mem_base]
    
    set waitP_offset [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_WAITP $fast_mem_base]
    set active_reg_set [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_ACTIVE_REG_SET $fast_mem_base]

    set semiStats(last_pram_address) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_PRAM_LAST_ADDR $fast_mem_base]
    set semiStats(waitp) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_WAITP $fast_mem_base]
    set semiStats(idle_cycles) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_IDLE_CYCLES $fast_mem_base]
    set semiStats(store_halt_cycles) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_STORE_HALT_CYCLES $fast_mem_base]
    set semiStats(fin_halt_cycles) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_FIN_HALT_CYCLES $fast_mem_base]
    set semiStats(partial_halt_cycles) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_PARTIAL_HALT_CYCLES $fast_mem_base]

    set Cam_offset(lsb) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_CAM_VALUE_MSB $fast_mem_base]
    set Cam_offset(msb) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_CAM_RD_DATA_MSB $fast_mem_base]
    if {$chip_ver>1 && $chip_mc_ver>1} {
        set Cam_offset(middle) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_CAM_RD_DATA_MIDDLE $fast_mem_base]
        set Cam_offset(mode) [GetFastMemOffset $stormLetter SEM_REG_SEM_FAST_CAM_MODE $fast_mem_base]
    } else {
        # Fitting the cam mode back to E1
            if {[info exists Cam_offset(middle)]} {
                unset Cam_offset(middle)
                unset Cam_offset(mode)
            }
    }    
    
}

proc GRC_dump::GetFastMemOffset {stormLetter regName fastmemBase} {
    set addr [get_start_offset [reg_offset -n [set stormLetter][set regName]]]
    return [format "0x%x" [expr {$addr - $fastmemBase}]]
}

proc GRC_dump::Log2Calc {num} {
	return [expr {int(log($num)/log(2))}]
}
proc GRC_dump::E2TCMStormContextSetPage {first_lcid} {
        markCamArea page_write
	reg_write -n TCM_REG_L1ST_PAGE -v $first_lcid
        markCamArea end
}
proc GRC_dump::E2TCMIsPageMode {} {
	return [reg_read -n TCM_REG_L1ST_PAGE_MODE]
}


proc GRC_dump::E2TCMStormBitsForLcid {} {
	set def_lcid 256
	set res 0
	if {[E2TCMIsPageMode]} {
		set res [reg_read -n TCM_REG_L1ST_PAGE_SIZE]
	} else {
		set res $def_lcid
	}
	return [Log2Calc $res]
}

proc GRC_dump::E2TCMNextRegisterOffsetSize {} {
#Adding 2 to the shift since each register is 4 bytes in size
		return [expr {1 << (2 + [E2TCMStormBitsForLcid])}]
}

proc GRC_dump::E2TCMGetRelativeLcid {lcid} {
	if {[E2TCMIsPageMode]} {
		return [expr {$lcid % [reg_read -n TCM_REG_L1ST_PAGE_SIZE]}]
		} else {
		return $lcid
		}
}

proc GRC_dump::E2TCMAdjustPage {lcid} {
	if {[E2TCMIsPageMode]} {
		set page_num [expr {$lcid / [reg_read -n TCM_REG_L1ST_PAGE_SIZE]}]
		E2TCMStormContextSetPage [expr {$page_num * [reg_read -n TCM_REG_L1ST_PAGE_SIZE]}]
	}
}

proc GRC_dump::E2TCMAdjustBackPage {} {
	if {[E2TCMIsPageMode]} {
		E2TCMStormContextSetPage 0
		}
}
proc GRC_dump::E2TCMNumLines {} {
	variable line_size
	if {[E2TCMIsPageMode]} {
		return [expr {([reg_read -n TCM_REG_L1ST_CTX_SIZE] * 32) / ( [set line_size(st_ctx)]) }]
	} else {
	return 8
	}
}


proc GRC_dump::Exists {func_name} {
    if {[catch {namespace origin $func_name}]} {
        return false
    }
    return true
}

proc GRC_dump::pci_cfg {{op} {address}} {

    variable pcicfg_offset
    if {[string equal $op read] || [string equal $op dread]} {
        return [reg read [expr $address + $pcicfg_offset]]
    }
}
proc GRC_dump::define_assert {chip} {
    global   assertJump
    global   assertFirstJump
    global   assertOffsets
    global   assertChip
    set assertChip [string toupper $chip]
    array set assertOffsets {E1,X 0x1000 E1,U 0x1000 E1,T 0x1000 E1,C 0x1000 E1H,X 0x9000 E1H,T 0xa000 E1H,U 0x8960 E1H,C 0x7000}    
    catch {rename ::assert old_assert}
  
    proc ::assert {func storm args} {
	variable assertChip
        variable assertOffsets
	variable assertFirstJump
	variable assertJump
        if {[string equal last_index_offset $func]} {
            return [set assertOffsets([set assertChip],[set storm])]
        }
        if {[string equal get_offset $func]} {
            return [expr [assert last_index_offset $storm] + $assertFirstJump + $assertJump * $args]
        }
    }
}

proc GRC_dump::unload_assert {} {
    catch {rename ::assert not_useful}
    catch {rename old_assert ::assert}
}


proc GRC_dump::getRdfFile {args} {
	global rdfFile
	
	get_rdf_file
	
	return $rdfFile
}
	

proc GRC_dump::prepare {} {
    variable scriptInc
    variable dump_no_pci
    variable CHIPS
    variable PRAM_FW_FLAGS
    variable ram_offset
    variable ram_size
    variable Cam_size
    variable Cam_offset
    variable nomcp 0
    variable chip_mc_ver
    variable chip_ver
    variable rdfs_files
    variable no_qstat
    variable chip_num_reg
    variable chip_rev_reg
    global   assertOffsets
    
    global regsOffset
    
    ### CHIP AND REVISION 
    # MISC_REG_CHIP_NUM
    # MISC_REG_CHIP_REV - must be hardcoded since this will choose which rdf to parse
    
    # Correlation between the flags in the fw version in the pram to the chip num
    array set PRAM_FW_FLAGS {0 1 1 1.5 2 2 3 3}
    
    
    # Starting ediag version 4.0.10 the scripts were included inside it
    # regutils was changed in ver 4.0.15
    regexp {(\d+)\.(\d+)\.(\d+)} [version] verA verA verB verC
    if {![info exists scriptInc]} {
        if {([string equal $::tcl_platform(os) DOS] || [string equal $::tcl_platform(os) Linux]) && $verA>=4 && ($verB>0 || $verC>=10)} {
            if {$verA>=4 && ($verB>0 || $verC>=15)} {
                set scriptInc 2
            } else {
                set scriptInc 1
            }
        } else {
            set scriptInc 0
        }
    }
    
    set dump_no_pci 0
    
    if {[info exists ::sys(NO_PCI)]} {
        set dump_no_pci $::sys(NO_PCI)
    } else {
        set dump_no_pci [regexp {NO_PCI} $::tcl_prompt1]
    }
    

    # checking whether the chip has changed, and if so replacing the rdf file.
    set last_chip_ver 0
    if {[info exists chip_ver]} {
            set last_chip_ver $chip_ver
    }
    variable chip_num [reg read $chip_num_reg]
    set chip_ver $CHIPS($chip_num)
    
    variable chip_rev [reg read $chip_rev_reg]
    # since the mcp doesn't have code for E1.5 for now, it is excluded from the dump
    if {($chip_rev>5 && ![expr {$chip_rev&1}])} {
        set nomcp 1
    }
    if {$chip_ver >= 2 } {
        set nomcp 0
    }
    
    if {![array exist ::regsOffset] || $last_chip_ver !=$chip_ver} {
		set rdf_file [getRdfFile]
        rdfParse $rdf_file
    }
    variable neededRegs
    #adding dummy registers for addr dumping
    set neededRegs([expr [get_start_offset [reg_offset -n MISC_REG_RESET_REG_1]]*1]) 0xffffffff
    set neededRegs([expr [get_start_offset [reg_offset -n MISC_REG_RESET_REG_2]]*1]) 0xffffffff
    set neededRegs(E1H,[expr [get_start_offset [reg_offset -n XSEM_REG_PRAM]]+4])  0x10

}

proc GRC_dump::const_declare {user_chip_mc_ver} {

    variable line_size
    variable number_of_lines
    variable ram_size
    variable ctx_type_size
    variable ram_offset
    variable passive_buffer_size
    variable qm_size
    variable Cam_offset
    variable activeIor
    variable brb1_ram_size
    variable pcieCore
    variable pcieCoreRelated
    variable semiStats
    variable MCP_scpd
    variable MCP_regs
    variable CHIPS
    variable PRAM_FW_FLAGS
    variable rdfs_files
    variable HCRelated
    variable HCNonSplit
    variable chip_ver
    variable chip_mc_ver
    variable pcicfg_reads
    variable chip_used_num
    variable chip_num
    variable neededRegs	
    set chip_used_num [set CHIPS([format 0x%x $chip_num])]
    
    array set line_size {ccm_ag_ctx 320 tcm_ag_ctx 416 xcm_ag_ctx 256 ucm_ag_ctx 256 st_ctx 128}
    array set number_of_lines {ccm_ag_ctx 1 tcm_ag_ctx 1 ucm_ag_ctx 1 xcm_ag_ctx 4 tcm_st_ctx 8 ccm_st_ctx 8 xcm_st_ctx 16 ucm_st_ctx 16} 
    
    array set ram_size {0 180224 1 2048 line_size 64}
    array set ctx_type_size {line_size 32 type_count 8}

    array set passive_buffer_size {T 32768 C 65536 X 65536 U 65536}
    array set qm_size {bigram 49152 pointertable 4096 line_size 30}
    
    variable Cam_size 192
    
    array set brb1_ram_size {bigram 1048576 bigram_part_size 262144 big_num_parts 4 llram 16384}
    array set pcicfg_reads {decname0 PCIE_REG_PCIER_CFG_MSIX_CONTROL addr0 0xa0 size0 32 \
                            decname1 PCIE_REG_PCIER_CFG_MSI_CONTROL addr1 0x58 size1 32}
    # pcieCore include data which should be dump to pcieCore.txt in hex form. the data should be read for both functions.
    # it is build out of address and sizes in bytes to read. Used in pcieDump
    array set pcieCore {addr0 0x2000 size0 104 addr1 0x20a0 size1 220 addr2 0x2400 size2 340 addr3 0x2800 size3 412 addr4 0x3000 size4 1148 addr5 0x3800 size5 1324 addr6 0x2068 size6 56 addr7 0x217c size7 644}
    # pcieCoreRelated holds registers who need to be read and parsed. the data is added to the dumpFile.
    # it is build out of registers name (where ABSOLUTE represent a registers which are not in everest.rdf), the name they should appear and the offset.
    # used in add_pcieCore_regs
    array set pcieCoreRelated {dec_name0 CFG_MISC_STATUS mem_name0 ABSOLUTE offset0 0x206c dec_name1 COMMAND_AND_STATUS mem_name1 ABSOLUTE offset1 0x2004}
    
    # HCRelated is similiar to pcieCore but since here the registers are in everest.rdf the offset isn't needed
    # used in add_hc_regs
    array set HCRelated {dec_name0 "HC_INT_MASK" mem_name0 HC_REG_INT_MASK dec_name1 "HC_INTERRUPT_A" mem_name1 HC_REG_INTERRUPT_A \
                                dec_name2 "HC_INTERRUPT_B" mem_name2 HC_REG_INTERRUPT_B \
                                dec_name3 "HC_ATTN_MSG0_ADDR_L" mem_name3 HC_REG_ATTN_MSG0_ADDR_L \
                                dec_name4 "HC_ATTN_MSG0_ADDR_H" mem_name4 HC_REG_ATTN_MSG0_ADDR_H \
                                dec_name5 "HC_ATTN_MSG1_ADDR_L" mem_name5 HC_REG_ATTN_MSG1_ADDR_L \
                                dec_name6 "HC_ATTN_MSG1_ADDR_H" mem_name6 HC_REG_ATTN_MSG1_ADDR_H \
                                dec_name7 "HC_ATTN_NUM_P0" mem_name7 HC_REG_ATTN_NUM_P0 \
                                dec_name8 "HC_ATTN_NUM_P1" mem_name8 HC_REG_ATTN_NUM_P1 \
                                dec_name9 "HC_LEADING_EDGE_0" mem_name9 HC_REG_LEADING_EDGE_0 \
                                dec_name10 "HC_TRAILING_EDGE_0" mem_name10 HC_REG_TRAILING_EDGE_0 \
                                dec_name11 "HC_LEADING_EDGE_1" mem_name11 HC_REG_LEADING_EDGE_1 \
                                dec_name12 "HC_TRAILING_EDGE_1" mem_name12 HC_REG_TRAILING_EDGE_1 \
                                dec_name13 "HC_ATTN_IDX" mem_name13 HC_REG_ATTN_IDX \
                                dec_name14 "HC_ATTN_BIT" mem_name14 HC_REG_ATTN_BIT \
                                dec_name15 "HC_P0_PROD_CONS" mem_name15 HC_REG_P0_PROD_CONS \
                                dec_name16 "HC_P1_PROD_CONS" mem_name16 HC_REG_P1_PROD_CONS \
                                dec_name17 "HC_MAIN_MEMORY" mem_name17 HC_REG_MAIN_MEMORY \
                                }
    
    array set HCNonSplit {dec_name0 "HC_CONFIG_0" mem_name0 HC_REG_CONFIG_0 \
                                dec_name1 "HC_CONFIG_1" mem_name1 HC_REG_CONFIG_1}
                                
    
    # All the stats releted variables are used in the procedure collect_stats
    # semiStats hold address of ST registers in the Sem_fast, if an alignment is needed there must exist align<name> in the array.
    array set semiStats {last_pram_address 0x18A00 alignlast_pram_address 8 waitp 0x18A80 idle_cycles 0x18200 store_halt_cycles 0x18240 fin_halt_cycles 0x18280 partial_halt_cycles 0x182c0}
    # The following lists holds all the ST registers names in the relevant blocks.
    # They should be built as stats<block_name>
    variable statsCFC {CFC_REG_NUM_LCIDS_EMPTY CFC_REG_NUM_LCIDS_ARRIVING CFC_REG_NUM_LCIDS_INSIDE CFC_REG_NUM_LCIDS_INA \
                        CFC_REG_NUM_LCIDS_INAS CFC_REG_NUM_LCIDS_INANS CFC_REG_NUM_LCIDS_LEAVING CFC_REG_NUM_LCIDS_IO \
                        CFC_REG_NUM_LCIDS_ALLOC CFC_REG_ERROR_DATA1 CFC_REG_ERROR_DATA2 CFC_REG_ERROR_DATA3 \
                        CFC_REG_LOAD_CONTEXT_HITS_0 CFC_REG_LOAD_CONTEXT_HITS_1 CFC_REG_LOAD_CONTEXT_HITS_2 \
                        CFC_REG_LOAD_CONTEXT_HITS_3 CFC_REG_LOAD_CONTEXT_HITS_4 CFC_REG_LOAD_CONTEXT_HITS_5 \
                        CFC_REG_LOAD_CONTEXT_HITS_6 CFC_REG_LOAD_CONTEXT_HITS_7 CFC_REG_LOAD_CONTEXT_MISSES_0 \
                        CFC_REG_LOAD_CONTEXT_MISSES_1 CFC_REG_LOAD_CONTEXT_MISSES_2 CFC_REG_LOAD_CONTEXT_MISSES_3 \
                        CFC_REG_LOAD_CONTEXT_MISSES_4 CFC_REG_LOAD_CONTEXT_MISSES_5 CFC_REG_LOAD_CONTEXT_MISSES_6 \
                        CFC_REG_LOAD_CONTEXT_MISSES_7 CFC_REG_RFE_SEARCH_HITS CFC_REG_RFE_SEARCH_MISSES \
                        CFC_REG_CDU_WRITE_BACKS CFC_REG_TASK_COUNTER CFC_REG_LC_BLOCKED}
    variable statsCDU {CDU_REG_ERROR_DATA}
    variable statsSRC {SRC_REG_COUNTFREE0 SRC_REG_COUNTFREE1 SRC_REG_NUMIPV4CONN0 SRC_REG_NUMIPV4CONN1 \
                    SRC_REG_NUMIPV6CONN0 SRC_REG_NUMIPV6CONN1}
	if {$chip_used_num == 2} {
		variable statsSRC
		set statsSRC {SRC_REG_COUNTFREE0 SRC_REG_NUMIPV4CONN0 SRC_REG_NUMIPV6CONN0}
					}
    variable statsTCM {TCM_REG_XX_OVFL_CNTR}
    variable statsUSDM {USDM_REG_NUM_OF_Q9_CMD}
    variable statsQM {QM_REG_QLEVELMHVAL QM_REG_MHQNUMSEL}
    variable statsXSEM {XSEM_REG_SLEEP_THREADS_VALID XSEM_REG_THREAD_RDY XSEM_REG_THREAD_ERROR \
                    XSEM_REG_THREADS_LIST}
    variable statsTSEM {TSEM_REG_SLEEP_THREADS_VALID TSEM_REG_THREAD_RDY TSEM_REG_THREAD_ERROR \
                    TSEM_REG_THREADS_LIST}
    variable statsCSEM {CSEM_REG_SLEEP_THREADS_VALID CSEM_REG_THREAD_RDY CSEM_REG_THREAD_ERROR \
                    CSEM_REG_THREADS_LIST}
    variable statsUSEM {USEM_REG_SLEEP_THREADS_VALID USEM_REG_THREAD_RDY USEM_REG_THREAD_ERROR \
                    USEM_REG_THREADS_LIST}
    # statsByName hold the names of the blocks which from ST should be read.
    # should be build of blocks who has a stats<block_name> list of stats regs.
    variable statsByName {CFC CDU SRC TCM USDM QM XSEM USEM TSEM CSEM}
    array set MCP_scpd {trcb_signature_addr 0xf100 trcb_signature 0x54524342 trcb_start_ptr 0xf104 trcb_start 0xf108 trcb_offset_zero 0x800f108 trcb_size 2040 \
            scratchpad_addr 0xa0000 scratchpad_size 524288 shared_mem_offset 0xf900 shared_mem_size 14336 shmem_addr_in_shmem2 0x34 dcbx_lldp_addr_in_shmem2 40 \
			dcbx_lldp_local_addr_in_shmem2 44 dcbx_lldp_remote_addr_in_shmem2 48 dcbx_lldp_params_size_bytes 40 dcbx_admin_mib_size_bytes 92 dcbx_remote_mib_size_bytes 100 \
			dcbx_local_mib_size_bytes 100 }
    array set MCP_regs {CPU_MODE 0x85000 CPU_STATE 0x85004 CPU_EVENT_MASK 0x85008 CPU_PROGRAM_COUNTER 0x8501C CPU_INSTRUCTION 0x85020 \
                CPU_DATA_ACCESS 0x85024 CPU_INTERRUPT_ENABLE 0x85028 CPU_INTERRUPT_VECTOR 0x8502C CPU_INTERRPUT_SAVED_PC 0x85030 \
                CPU_HW_BREAKPOINT 0x85034 CPU_LAST_BRANCH_ADDR 0x85048 CPU_REG_FILE 0x85200 sizeCPU_REG_FILE 1024}
    
    # adding to needRegs MCP scratch pad registers
    variable neededRegs                                              
    set neededRegs([expr {$MCP_scpd(scratchpad_addr)+$MCP_scpd(trcb_signature_addr)}]) $MCP_scpd(trcb_signature)
    set neededRegs([expr {$MCP_scpd(scratchpad_addr)+$MCP_scpd(trcb_start_ptr)}]) $MCP_scpd(trcb_offset_zero) 
    
    
    variable lastIndex 0
    
    
    
    if {$user_chip_mc_ver==0} {
        # checking the microcode version in E1H
        set pram_addr [get_start_offset [reg_offset -n XSEM_REG_PRAM]]
        reg read $pram_addr
        set flags [reg read [expr {$pram_addr+4}]]
        set chip_mc_ver $PRAM_FW_FLAGS([expr {($flags>>4)&3}])
        
    } else {
        set chip_mc_ver $user_chip_mc_ver
    }
	variable ignore_mc_chip
	if {$ignore_mc_chip} {
		set chip_mc_ver $chip_ver
	}
    
    if {$chip_ver>1} {
        # fitting the Mcp to E1H
        # The mcp is affected by which microcde version is used, and should always be treated according to the used hardware
        set MCP_scpd(scratchpad_size) 1048576
        set MCP_scpd(shared_mem_size) 17984
    }
    
    if {$chip_ver>1 && $chip_mc_ver>1} {
        set ram_size(0) 540672

        set Cam_size 256
        
    } else {
        if {[info exists Cam_offset(middle)]} {
            # Fitting the cam mode back to E1
            unset Cam_offset(middle)
            unset Cam_offset(mode)
        }
    }
    
    if {$chip_ver >= 2} {
        set Cam_size 256
	set ram_size(0) 524288
        set ram_size(1) 180224
	set Cam_size 128
        set MCP_scpd(scratchpad_size) 1310720
        set MCP_scpd(scratchpad_addr) 0x3a0000
        #temporary solution
        set chip_mc_ver $chip_ver
	#/temporary solution
        array unset HCNonSplit
        array unset HCRelated
        
        set MCP_scpd(shared_mem_size) 19136
        

        
        set igu_config [reg_read -n IGU_REG_BLOCK_CONFIGURATION]
        set add_string "_E2_MODE"
        if {[expr {$igu_config & 2}] > 0} {
            set add_string "_BACKWARD_MODE"
        }
        
        set neededRegs([expr {[reg_offset -n TCM_REG_L1ST_PAGE_MODE]}]) 1
        set neededRegs([expr {[reg_offset -n TCM_REG_L1ST_PAGE_SIZE]}]) 0x80
        set neededRegs([expr {[reg_offset -n TCM_REG_L1ST_CTX_SIZE]}]) 36
		
        array set HCNonSplit "  dec_name0 IGU_CONFIG mem_name0 IGU_REG_BLOCK_CONFIGURATION \
                                dec_name1 IGU_MAPPING_MEMORY mem_name1 IGU_REG_MAPPING_MEMORY \
                                dec_name2 IGU_SB_CONS_PROD[set add_string] mem_name2 IGU_REG_PROD_CONS_MEMORY \
                                dec_name3 IGU_SB_MSIX_MEMORY mem_name3 IGU_REG_MSIX_MEMORY \
								dec_name4 IGU_PENDING_BITS_STATUS mem_name4 IGU_REG_PENDING_BITS_STATUS \
								dec_name5 IGU_WRITE_DONE_PENDING mem_name5 IGU_REG_WRITE_DONE_PENDING \
								dec_name6 IGU_ATTN_MSG_PENDING mem_name6 IGU_REG_ATTN_MSG_PENDING \
								dec_name7 IGU_ATTN_WRITE_DONE_PENDING mem_name7 IGU_REG_ATTN_WRITE_DONE_PENDING \
								dec_name8 IGU_STATISTIC_NUM_MESSAGE_SENT mem_name8 IGU_REG_STATISTIC_NUM_MESSAGE_SENT \
								dec_name9 IGU_ERROR_HANDLING_DATA_VALID mem_name9 IGU_REG_ERROR_HANDLING_DATA_VALID \
								dec_name10 IGU_SILENT_DROP mem_name10 IGU_REG_SILENT_DROP "
        
        array set HCRelated " dec_name0 IGU_PF_CONFIGURATION mem_name0 IGU_REG_PF_CONFIGURATION \
                              dec_name1 IGU_SB_INT_BEFORE_MASK_LSB mem_name1 IGU_REG_SB_INT_BEFORE_MASK_LSB \
                              dec_name2 IGU_SB_INT_BEFORE_MASK_MSB mem_name2 IGU_REG_SB_INT_BEFORE_MASK_MSB \
                              dec_name3 IGU_SB_MASK_LSB mem_name3 IGU_REG_SB_MASK_LSB \
                              dec_name4 IGU_SB_MASK_LSB mem_name4 IGU_REG_SB_MASK_MSB \
							  dec_name5 IGU_PCI_PF_MSIX_EN mem_name5 IGU_REG_PCI_PF_MSIX_EN \
							  dec_name6 IGU_PCI_PF_MSIX_FUNC_MASK mem_name6 IGU_REG_PCI_PF_MSIX_FUNC_MASK \
							  dec_name7 IGU_PCI_PF_MSI_EN mem_name7 IGU_REG_PCI_PF_MSI_EN "
        
    if {$chip_ver > 2} {
        array set brb1_ram_size {bigram 2097152 bigram_part_size 524288 big_num_parts 4 llram 32768}
}
    }
    
}

proc GRC_dump::check_reset {} {
    variable resetRegs
    variable resetRegsVals
    variable outFile
    setFile
    foreach reg $resetRegs {
        regexp {_REG_(.+)$} $reg declared_name declared_name
        puts $outFile "PRINTING MEM:\t$declared_name"
        puts $outFile "***************************************************************"
        read_component BATCH $reg $declared_name 32
        puts $outFile "END\n"
        set addr [get_start_offset [reg_offset -n $reg]]
        reg write [expr $addr+4] $resetRegsVals($reg)
    }
    
    close $outFile
}


# -----------------------------------------------------------------
# This proc parses the start offset out of an offset field
# (startOffset - endOffset)
# -----------------------------------------------------------------
proc GRC_dump::get_start_offset {{offset}}  {
    
    # Parse out the starting offset
    return  [lindex [split $offset "-"] 0]                 
}

# subroutine for converting number (hex) in hex format to binary format
proc GRC_dump::hex2bin hex {
    set t [list 0 0000 1 0001 2 0010 3 0011 4 0100 \
	    5 0101 6 0110 7 0111 8 1000 9 1001 \
	    a 1010 b 1011 c 1100 d 1101 e 1110 f 1111 \
	    A 1010 B 1011 C 1100 D 1101 E 1110 F 1111]
    return [string map -nocase $t $hex]
}

# subroutine replacing reg show. return the request amount of bytes in the closest multiple of 4. 
# addr is the address to read and count is the amounts in bytes
proc GRC_dump::reg_show {{addr} {count 4}} {
	set i 0
	set ans ""
	while {$count>0} {
		if {$i%8!=0} {
			append ans [format " %.8x" [reg read $addr]]
		} else {
			if {$i!=0} {
				append ans " "
			}
			append ans [format "%.7x:%.8x" $addr [reg read $addr]]
		}
		set addr [format "0x%x" [expr {$addr+4}]] 
		incr i
		incr count -4
	}		
	return $ans
}

# subroutine for activating/releasing (1/0) the wait_p on all storms 
# onOff should equal 1 in order to activate the WaitP (which mean stop the storms)
# and 0 to stop it (which mean reactivate the storms)
proc GRC_dump::stormWaitP {onOff {storm_letters {T U C X}}} {
	variable waitP_offset
	variable waitPstatus
	variable has_used_storm_letters
	variable used_storm_letters
	if {$has_used_storm_letters} {
		set storm_letters $used_storm_letters
	}
	if {$waitPstatus == $onOff} { return }
	set mem_name "SEM_REG_FAST_MEMORY"
	if {$onOff == "true"} {
	    set onOff 1
            set waitPstatus true
	} elseif {$onOff =="false"} {
	    set onOff 0
            set waitPstatus false
	} else { return "SYNTAX: stormWaitP <true|false> only.\n" }
	if {!$has_used_storm_letters} {
		foreach stormLetter $storm_letters {
                UpdateFastMemOffsets $stormLetter
		set x [read_component "ONLINE" "$stormLetter$mem_name" "Stopping [set stormLetter]storm" 32 $waitP_offset 1]
		set addr [get_start_offset [reg_offset -n "$stormLetter$mem_name"]]
		set addr [format "0x%x" [expr {$addr+$waitP_offset}]]
		if {$x==!$onOff} {
			reg write $addr $onOff		
		}
	}
        after 1000
	 }
        
        
}
# this subroutine recieve size in bits and return the number of registers need to be read (goes to the closet power of 2)
proc GRC_dump::sizeToRegNumber {size} { 

	set regNumber [expr {int(pow(2,ceil(log($size)/log(2))))}]
	if {$regNumber<32} {set regNumber 32}
	return [expr {int($regNumber/32)}]

}


# this subroutine recieve a cid, the register name, amount of bits to read and offset (optional) 
# and either return the entire context memory for that cid or part of it (if amount and offset are declared)
# mode can be either BATCH/ONLINE (ONLINE is for future use and hasn't been implemented yet)
# lcid is the lcid for which the context is needed
# mem_name is the name of the block in the everest.rdf 
# amount is in bits and will be used in ONLINE mode only
# offset will be used in ONLINE mode only
# In order to get a register YY in lcid ZZ you need to define the offset 0xYYZZ*4
proc GRC_dump::read_context_by_lcid {{mode} {lcid} {mem_name} {amount 0} {offset 0}} {

	variable line_size
	variable number_of_lines 
	variable special_mode
	variable chip_used_num
	set special_mode false
	variable special_mode_addition
	
#first, let's handle the specific TCM ST Context case
	if {$chip_used_num > 1.5} {
		if {[expr ([regexp TCM $mem_name] && [regexp ST $mem_name])]} {
			set special_mode true
			E2TCMAdjustPage $lcid
			set lcid [E2TCMGetRelativeLcid $lcid]
			#update the number of lines
			set number_of_lines(tcm_st_ctx) [E2TCMNumLines]
			set special_mode_addition [E2TCMNextRegisterOffsetSize]
		}
	}
	if {$mode=="BATCH"} {
		set addr [get_start_offset [reg_offset -n $mem_name]]
		set addr [format "0x%x" [expr {$addr+$lcid*4}]]
		if {[regexp AG $mem_name]} {
			if {[regexp CCM $mem_name]} {
				set size $line_size(ccm_ag_ctx)
			} elseif {[regexp TCM $mem_name]} {
			 	set size $line_size(tcm_ag_ctx) 
			} elseif {[regexp UCM $mem_name]} {
			 	set size $line_size(ucm_ag_ctx) 
			} elseif {[regexp XCM $mem_name]} {
			 	set size $line_size(xcm_ag_ctx) 
			} else { return "INVALID memory name" }
			if {[regexp XCM $mem_name]} {
				set linesToRead $number_of_lines(xcm_ag_ctx)
			} elseif {[regexp UCM $mem_name]} {
				set linesToRead $number_of_lines(ucm_ag_ctx)
			} elseif {[regexp TCM $mem_name]} {
				set linesToRead $number_of_lines(tcm_ag_ctx)
			} elseif {[regexp CCM $mem_name]} {
				set linesToRead $number_of_lines(ccm_ag_ctx)
			}
		} elseif {[regexp ST $mem_name]} { 
			set size $line_size(st_ctx)
			if {[regexp XCM $mem_name]} {
				set linesToRead $number_of_lines(xcm_st_ctx)
			} elseif {[regexp TCM $mem_name]} {
				set linesToRead $number_of_lines(tcm_st_ctx)
			} elseif {[regexp UCM $mem_name]} {
				set linesToRead $number_of_lines(ucm_st_ctx)
			} elseif {[regexp CCM $mem_name]} {
				set linesToRead $number_of_lines(ccm_st_ctx)
			} else {return "INVALID memory name"}	
		} else {return "INVALID memory name"}
		# in the future, when ONLINE will also be implemented, this will have to be changed
		set addr [format "0x%x" [expr {$addr+$offset}]]
		set result ""
		for {set i 0} {$i<[expr {$linesToRead*$size/32}]} {incr i} {
                        if {$special_mode} {
                            markCamArea page_read
                        }
			append result "[reg_show $addr 4] "
                        
			if {$special_mode} {
                        markCamArea page_write
			set addr [format "0x%x" [expr {$addr+$special_mode_addition}]]
			} else {
			set addr [format "0x%x" [expr {$addr+0x400}]]			
			}
		}
		set result [string trimright $result { }]
		
		if {$special_mode} {
			E2TCMAdjustBackPage
			}
		return [format_mem $result $size]
	}
}
		
# this subroutine uses read_context_by_cid and read the context of all the connections
# it's recieve mem_name - the block from whom to read and declared name to print it			
proc GRC_dump::read_context {{stormLetter} {ctxType} {connectionNumber 256}} {
	variable outFile
	set ctxType [string toupper $ctxType]
	if {$stormLetter!="T" && $stormLetter!="U" && $stormLetter!="X" && $stormLetter!="C"} {
	    return "Storm letter can be only T, U, C, or X"
	}
	variable is_linux_dump
	if {$is_linux_dump} {
		if {$ctxType == "STORM" && $stormLetter=="T" && $connectionNumber < 256} {
			return
		}
	}
	
	
	
	setFile
	set mem_name "[string toupper $stormLetter]CM_REG_[set ctxType]_CTX"
	if {$ctxType == "STORM"} {
	    set declared_name "[set stormLetter]cm ST context"
	} elseif {$ctxType=="AG"} {
	    set declared_name "[set stormLetter]cm AG context"
	} else { return "context Type can be only STORM or AG" }
	puts "Reading $declared_name"
	puts $outFile "PRINTING MEM:\t$declared_name"
	puts $outFile "***************************************************************"
	for {set i 0} {$i<$connectionNumber} {incr i} {
		add_mem_to_file [read_context_by_lcid BATCH $i $mem_name] $declared_name $i
	}
	puts $outFile "END\n"
        close $outFile
}


# this subroutine recieve the raw data in hex format from "reg_show" and the size of a line of data 
# and returns the data in a binary list of lines  
proc GRC_dump::format_mem {data size {outformat bin}} {

	set regNumber [sizeToRegNumber $size]
	regsub -all {[0-9xXA-Fa-f]+:} $data {} data
	regsub -all \\.+\\s $data {} data
	if {[string equal $outformat "bin"]} {
            set data [hex2bin $data]
        }
	set regList [split $data { }]
	if {$regNumber>1} {
		set index 0	
		set listSize [llength $regList]
		for {set i 0} {$i<$listSize} {incr i $regNumber} {
			set tempStr ""
			for {set j [expr {$regNumber-1}]} {$j>=0} {incr j -1} {
				append tempStr [lindex $regList [expr {$i+$j}]]
			}
			lappend resList $tempStr
		}
		return $resList
	}
	return $regList
	
}


# this subroutine check which IOR is active for the requested storm
# and returns the data in a binary list  
proc GRC_dump::active_ior {{storm_name}} {

	variable activeIor
	variable active_reg_set
	if {$storm_name!="Tstorm" && $storm_name!="Ustorm" && $storm_name!="Xstorm" && $storm_name!="Cstorm"} {
	    return "Storm_name can be only Tstorm, Ustorm, Cstorm, or Xstorm"
	}
	set declared_name "$storm_name active IOR set"
	set mem_name "[string toupper [string index $storm_name 0]]SEM_REG_FAST_MEMORY"
	set result [read_component "ONLINE" $mem_name "$storm_name active set check" 1 $active_reg_set 1]
	array set activeIor "$storm_name $result"
	set result [expr {1-$result}]
	return $result
	
}

proc GRC_dump::printILTClients {device} {
    variable dump_no_pci
    variable ::current
    variable ilt_clients
    set base 0
    
    if {$dump_no_pci} {
        set base [expr ($device-1)<<22]
        set func_num [expr $device - 1]
    } else {
        device $device -no_display
        set func_num [expr {$::current(FUNC)}]
    }
    set curr_f [format "%d" $func_num]
    foreach client $ilt_clients {
        
        printMemoryToFile "PXP2_REG_RQ_[set client]_FIRST_ILT" "ILT_NEW_MODE_FIRST_FUNC[set curr_f]_[set client]" $base
        printMemoryToFile "PXP2_REG_RQ_[set client]_LAST_ILT" "ILT_NEW_MODE_LAST_FUNC[set curr_f]_[set client]" $base
    }
    
    
    
    
}

proc GRC_dump::addILT {} {
    variable chip_ver
    variable ilt_clients
    
    set ilt_clients {CDU TM QM SRC XSDM TSDM USDM USDMDP CSDM DBG}
    
    
    if {$chip_ver > 1} {
        set isE1 false
    } else {
        set isE1 true
    }
    if {$isE1} {
        printMemoryToFile PXP2_REG_RQ_ONCHIP_AT ILT_MEMORY
    } else {
        printMemoryToFile PXP2_REG_RQ_ONCHIP_AT_B0 ILT_MEMORY
    }
    
    foreach client $ilt_clients {
        printMemoryToFile "PXP2_REG_PSWRQ_[set client]0_L2P" "ILT_OLD_MODE_P0_[set client]"
        printMemoryToFile "PXP2_REG_PSWRQ_[set client]1_L2P" "ILT_OLD_MODE_P1_[set client]"
        printMemoryToFile "PXP2_REG_RQ_[set client]_P_SIZE" "ILT_PAGE_SIZE_[set client]"
    }
    
    
    
    if {![set isE1]} {
        printMemoryToFile "PXP2_REG_RQ_ILT_MODE" "ILT_IS_NEW_MODE"
        run_over_devices printILTClients
    }
}

# this subroutine added the data from the HC to the output file
proc GRC_dump::add_split_fullregs {device} {
    
 
    variable HCRelated
    variable dump_no_pci
    variable device_read_base
    variable outFile
    
    puts "Reading Split Registers for device $device . . . "
    
    set base 0
    if {$dump_no_pci} {
        set base [expr ($device-1)<<22]
        set func_num [expr $device - 1]
	set device_read_base $base
	catch {rename ::reg ::uartRegSave}
	proc	::reg {act addr args} {
		variable GRC_dump::device_read_base
		set new_addr [expr {$GRC_dump::device_read_base + $addr}]
		return [::uartRegSave $act $new_addr $args]
	}
    } else {
        device $device -no_display
        set func_num [expr {$::current(FUNC)}]
    }
    
    setFile
    puts $outFile "\n\n\nSPLIT REGISTERS(DEV $func_num):"
    puts $outFile       "*******************************"
    
    close $outFile
                if {[catch {read_regs_all -silent -out $GRC_dump::outFileName -split_regs} errMsg]} {
                    if {[regexp "^invalid command name" $errMsg]} {
                        puts "Your regutils.tcl version isn't updated.\nYou must use a version from 18/09/2007 or later."
                    } else {
                        puts $errMsg
                    }
                }
		
	
		
    setFile
    puts $outFile "\n\n\nEND OF SPLIT REGISTERS\n\n"
    close $outFile
    if {$dump_no_pci} {
	catch {rename ::reg {}}
	catch {rename ::uartRegSave ::reg}
    }
}

# this subroutine write which ior is active for each storm in the output file.
proc GRC_dump::add_active_iors {} {
	
	variable outFile
	variable waitPstatus
        
        setFile
	if {$waitPstatus == "false"} { return -1}
	
	puts $outFile "PRINTING ACTIVE IOR SETS:"
	puts $outFile "***************************************************************"
	foreach stormLetter {T U C X} {
                UpdateFastMemOffsets $stormLetter
		set storm_name "[set stormLetter]storm"
		puts $outFile "$storm_name active IOR set: [active_ior $storm_name]"
	}
	puts $outFile "END\n"
        close $outFile
}


# this subroutine recieve mode (BATCH/ONLINE), mem name and offset in it and read the requested data
# if mode is BATCH the data is written to the file in the requested line size under the declared_name
# otherwise the data is return in a binary list 
# mode (BATCH - to a file / ONLINE to recieve the data directly)
# mem_name - the block name
# declared_name - the name to print underneath in the file
# amount - the amount of data to read in bits
# offset - offset from the block's start in bytes
# size - the requested line size in the output - by defualt will be taken from the everest.rdf
# outformat - the format to output the data binary / hexa (bin/hex)
proc GRC_dump::read_component {{mode} {mem_name} {declared_name} {amount} {offset 0} {size 0} {outformat bin}} {
	
	puts "Reading $declared_name"
	variable outFile
        if {[string match $mem_name "ABSOLUTE"]} {
            set addr 0
        } else {
            set addr [get_start_offset [reg_offset -n $mem_name]]
        }
	if {$size==0} {
		set size [reg_size -n $mem_name]
	}
	set left 0		
	set byteAmount [expr int(ceil($amount./8))]
	set addr [format "0x%x" [expr {$addr+$offset}]]
	if {$mode=="BATCH"} {
		if {$byteAmount>6496} {
			set left [expr {$byteAmount-6496}]
			set byteAmount 6496
		}
		set memList [format_mem [reg_show $addr $byteAmount] $size]	
		add_mem_to_file $memList $declared_name
		while {$left!=0} {
			set addr [format "0x%x" [expr {$addr+6496}]]
			if {$left>6496} {
				set left [expr {$left-6496}]
			} else { 
				set byteAmount $left
				set left 0
			}
			unset memList
			set memList [format_mem [reg_show $addr $byteAmount] $size]
			add_mem_to_file $memList $declared_name 1
		}
	} elseif {$mode=="ONLINE"} {
		if {$byteAmount>6496} {
			set left [expr {$byteAmount-6496}]
			set byteAmount 6496
		}
		if {$amount<32} {
			set memList [format_mem [reg_show $addr $byteAmount] $size]
			set result [string range [lindex $memList 0] [expr {[string length [lindex $memList 0]]-$amount}] end]
			return $result
		} else {
			set memList [format_mem [reg_show $addr $byteAmount] $size]
			while {$left!=0} {
				set addr [format "0x%x" [expr {$addr+6496}]]
				if {$left>6496} {
					set left [expr {$left-6496}]
				} else { 
					set amount $left
					set left 0
				}
				set memList2 [format_mem [reg_show $addr $amount] $size]	
				lappend memList $memList2
			}
		}	
		return $memList	
	}
}	

		
# this subroutine recieve the a binary list and puts it into the request output file
# memlist - the data
# declared_name - the title in the output file
# outFile - the output file
# toAppend - whatever to put a new title or continue the last one
proc GRC_dump::add_mem_to_file {{memList} {declared_name} {toAppend 0}} {
		
    variable lastIndex
    variable outFile
    
    if {!($toAppend)} {
            set lastIndex 0
    }
    for {set i 0} {$i<[llength $memList]} {incr i} {
            puts $outFile [format "%.8x\t[lindex $memList $i]" [expr $i+$lastIndex]]            
    }
    incr lastIndex [llength $memList]
    
}

# this subroutine added the Cid Cam to the output file
# it can recieve a number of connetion to add - default is all
proc GRC_dump::add_cfc_cid_cam {{connectionNumber 256}} {

    variable outFile
    setFile
    set declared_name "CFC CID CAM"
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    read_component BATCH CFC_REG_CID_CAM $declared_name [expr {$connectionNumber*32}]
    puts $outFile "END\n"

    close $outFile
}

# this subroutine added the Cid String Cam to the output file
proc GRC_dump::add_cfc_string_cam {} {

    variable outFile
    setFile
    set declared_name "CFC STRING CAM"
    puts $outFile "PRINTING MEM:\t$declared_name"
	set mem_name CFC_REG_STRING_CAM
    puts $outFile "***************************************************************"
    set offsets [split [reg_offset -n $mem_name] "-"]
    if {[llength $offsets]>1} {
        set size [expr ([lindex $offsets 1]-[lindex $offsets 0])*8+32]
    } else {
        set size 32
    }
    read_component BATCH $mem_name $declared_name $size
    puts $outFile "END\n"

    close $outFile
	
}

# this subroutine added the Cid info RAM to the output file
# it can recieve a number of connetion to add - default is all
proc GRC_dump::add_cfc_info_ram {{connectionNumber 256}} {
    
    variable outFile
    setFile
	
    set declared_name "CFC info RAM"
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    read_component BATCH CFC_REG_INFO_RAM $declared_name [expr {$connectionNumber*128}]
    puts $outFile "END\n"
    close $outFile
}

# this subroutine added the Cid activity counter RAM to the output file
# it can recieve a number of connetion to add - default is all
proc GRC_dump::add_cfc_ac_ram {{connectionNumber 256}} {
    
    variable outFile
    setFile
	
    set declared_name "CFC activity counter RAM"
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    read_component BATCH CFC_REG_ACTIVITY_COUNTER $declared_name [expr {$connectionNumber*32}]
    puts $outFile "END\n"
    close $outFile
}

proc GRC_dump::find_size {mem_name} {
    set offsets [split [reg_offset -n $mem_name] "-"]
    if {[llength $offsets]>1} {
        set size [expr {([lindex $offsets 1]-[lindex $offsets 0])*8+32}]
    } else {
        set size 32
    }
    return $size
}

proc GRC_dump::add_pxp_data {} {
    variable outFile
    
    if {[IsE2]} {
        setFile
        set mem_name PXP_REG_HST_ZONE_PERMISSION_TABLE
        set declared_name PXP_PERMISSION_TABLE
        puts $outFile "PRINTING MEM:\t$declared_name"
        puts $outFile "***************************************************************"
        set size [find_size $mem_name]
        read_component BATCH  $mem_name $declared_name $size 
        puts $outFile "END\n"
        close $outFile
    }
}

# this subroutine added the QM RAMs to the output file
proc GRC_dump::add_qm_mem {} {
    
    variable outFile
    variable qm_size
    setFile
	
    set declared_name "QM big ram low"
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    read_component BATCH QM_REG_BIGRAMLOW $declared_name $qm_size(bigram)
	
    puts $outFile "END\n"
    close $outFile
    setFile
    set declared_name "QM big ram high"
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    read_component BATCH QM_REG_BIGRAMHIGH $declared_name $qm_size(bigram)
	
    puts $outFile "END\n"
    
    set declared_name "QM pointer table"
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    read_component BATCH QM_REG_PTRTBL $declared_name $qm_size(pointertable)
    puts $outFile "END\n"
    close $outFile

}
# this subroutine added the SEM CAM to the output file
proc GRC_dump::add_sem_cam {storm} {

    variable outFile
    variable Cam_size
    variable Cam_offset
    setFile
    markCamArea start
    set declared_name [format {%sSEM CAM} $storm]
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    
    puts "Reading $declared_name"
    set mem_name [format {%sSEM_REG_FAST_MEMORY} $storm]
    UpdateFastMemOffsets $storm
    set addr [get_start_offset [reg_offset -n $mem_name]]	
    set addr_lsb [expr {$addr+$Cam_offset(lsb)}]
    if {[info exists Cam_offset(middle)]} {
        set addr_middle [expr {$addr+$Cam_offset(middle)}]
        reg write [expr {$addr+$Cam_offset(mode)}] 1
    }
    set addr_msb [expr {$addr+$Cam_offset(msb)}]
    
    for {set i 0} {$i<$Cam_size} {incr i} {
            set memList [format_mem [reg_show $addr_lsb 4] 32]	
            add_mem_to_file $memList $declared_name $i
            #puts $outFile "read from address $addr_msb data $memList"
            
            if {[info exists addr_middle]} {
                set memList [format_mem [reg_show $addr_middle 4] 32]	
                add_mem_to_file $memList $declared_name 1
            }
            
            set memList [format_mem [reg_show $addr_msb 4] 32]	
            add_mem_to_file $memList $declared_name 1
            
            #puts $outFile "read from address $addr_rd_msb data $memList"
            incr addr_lsb 4
            
            #incr addr_rd_msb 4

    }
    markCamArea end
    puts $outFile "END\n"
    close $outFile
}


# this subroutine added the request storm RAM to the output file
# it can recieve a storm letter and ram number to know which ram to add
proc GRC_dump::add_storm_ram {stormLetter ram_num} {
    
    variable outFile
    setFile
    variable ram_size
    variable ram_offset
    UpdateFastMemOffsets $stormLetter
    set mem_name "[set stormLetter]SEM_REG_FAST_MEMORY"
    set declared_name "[set stormLetter]storm RAM $ram_num" 
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    read_component BATCH  $mem_name $declared_name $ram_size($ram_num) $ram_offset($ram_num) $ram_size(line_size)
    puts $outFile "END\n"
    close $outFile
}

    
proc GRC_dump::add_brb_bram {} {
    
    variable outFile
    variable brb1_ram_size
    setFile
	
    set mem_name "BRB1_REG_BIG_RAM"
    set declared_name "BRB1 BIGRAM" 
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    for {set i 0} {$i < $brb1_ram_size(big_num_parts)} {incr i 1} {
		puts "Reading part $i of BRB BIG_RAM:"
		reg_write -n BRB1_REG_BIG_RAM_INDIRECT -v $i
		read_component BATCH  $mem_name $declared_name $brb1_ram_size(bigram_part_size)
		puts "Done with part $i of BRB BIG_RAM."
	}
    puts $outFile "END\n"
    
    set mem_name "BRB1_REG_LL_RAM"
    set declared_name "BRB1 LLRAM" 
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    read_component BATCH  $mem_name $declared_name $brb1_ram_size(llram)
    puts $outFile "END\n"
	
	set mem_name "BRB1_REG_FREE_LIST_PRS_CRDT"
    set declared_name "BRB1 FREE LIST PRS CREDIT" 
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
	set offsets [split [reg_offset -n $mem_name] "-"]
    if {[llength $offsets]>1} {
        set size [expr ([lindex $offsets 1]-[lindex $offsets 0])*8+32]
    } else {
		set size 32
	}
    read_component BATCH  $mem_name $declared_name $size
	
    puts $outFile "END\n"
	
    close $outFile
}

    
# this subroutine added the request storm passive buffer to the output file
# it can recieve a storm letter to know which passive buffer to add
proc GRC_dump::add_storm_passive_buffer {stormLetter} {
    
    variable outFile
    setFile
    variable passive_buffer_size
    set mem_name "[set stormLetter]SEM_REG_PASSIVE_BUFFER"
    set declared_name "[set stormLetter]storm passive buffer"
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    read_component BATCH  $mem_name $declared_name $passive_buffer_size($stormLetter)
    puts $outFile "END\n"
    close $outFile
}

# this subroutine added the request storm ior sets to the output file
# it can recieve a storm letter to know which ior sets to add
proc GRC_dump::add_storm_iors {stormLetter} {
    
    
    variable outFile
    variable activeIor
    UpdateFastMemOffsets $stormLetter
    setFile
    set act $activeIor([set stormLetter]storm)
    set mem_name "[set stormLetter]SEM_REG_FAST_MEMORY"
    set declared_name "[set stormLetter]storm IOR [expr 1-$act]"
    set iorOffset $activeIor([expr (1-$act)])
    
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    
    read_component BATCH  $mem_name $declared_name $activeIor(size) $iorOffset $activeIor(line_size)
    puts $outFile "END\n"
    
    set iorOffset $activeIor($act)
    set declared_name "[set stormLetter]storm IOR $act"
    
    puts $outFile "PRINTING MEM:\t$declared_name"
    puts $outFile "***************************************************************"
    
    read_component BATCH  $mem_name $declared_name $activeIor(size) $iorOffset $activeIor(line_size)
    
    puts $outFile "END\n"
    close $outFile
}

# this subroutine read the size of a storm context by type. and write it to the file in bytes.
proc GRC_dump::add_storms_ctx_sizes {{storm_letter}} {

	variable outFile
        variable ctx_type_size
	setFile
	if {$storm_letter!="T" && $storm_letter!="U" && $storm_letter!="X" && $storm_letter!="C"} {
	    return "Storm_letter can be only T, U, C or X"
	}
	puts "Reading [set storm_letter]storm context size per type"
        puts $outFile "PRINTING [set storm_letter]cm ST context TYPE SIZE:"
	puts $outFile "***************************************************************"
	set declared_name "[set storm_letter]cm TYPE SIZE"
	set mem_name "[set storm_letter]CM_REG_N_SM_CTX_LD_"
	for {set i 0} {$i<$ctx_type_size(type_count)} {incr i} {
            set addr [get_start_offset [reg_offset -n $mem_name$i]]
            puts $outFile "$i\t[expr [reg read $addr]*128]"
        }
	puts $outFile "END\n"
	close $outFile
	
}


# this subroutine added the data from the pcieCore to the output file
proc GRC_dump::add_pcieCore_regs {device } {
    
    variable outFile
    setFile
    variable pcieCoreRelated
    variable dump_no_pci
    variable ::current
    set base 0
    if {$dump_no_pci} {
        set base [expr ($device-1)<<22]
        set func_num [expr {$device -1 }]
    } else {
        device $device -no_display
        set func_num [expr {$::current(FUNC)}]
    }
    
    for {set i 0} {$i<[expr [array size pcieCoreRelated]/3]} {incr i} {
        set mem_name $pcieCoreRelated(mem_name$i)
        set declared_name $pcieCoreRelated(dec_name$i)
        puts $outFile "PRINTING MEM:\t[set declared_name]_$func_num"
        puts $outFile "***************************************************************"
        set size 32
        read_component BATCH  $mem_name $declared_name $size [expr $pcieCoreRelated(offset$i)+$base] 32
        puts $outFile "END\n"    
    }
    close $outFile
    
}

proc GRC_dump::run_over_devices {func_to_execute {mod_num 1} {msg none}} {
    #Stage 1:   Set funcs to hold number of functions
    #           and func0 to be the first function of current port
    
    variable dump_no_pci
    variable chip_ver
    variable chip_mc_ver
    variable funcs
    variable func0
    variable last_func_for_path
    set devCount $::current(TOTAL_DEV)
    #set funcs 1
    #set func0 1
    
    if {$dump_no_pci} {
        set last_func [expr {$funcs + $func0}]
    } else {
        set last_func $last_func_for_path
    }
    set counter 0
    for {set d $func0} {$d<[expr $last_func]} {set d [NextDeviceNum $d]} {
        if {$dump_no_pci} {
            set path_num [expr {$d-$func0}]
        } else {
            set path_num [expr {$::current(FUNC)}]
        }

        if {![string equal $msg none]} {
            puts "$msg from device $d (function $path_num)"    
        }
        if {[expr {$counter % $mod_num}]==0} {
            catch {$func_to_execute [expr $d]}
        }
        incr counter
    }
}

#this routing adds the data from all the non split registers of the HC block
proc GRC_dump::add_hc_nosplit_regs {} {
    variable HCNonSplit
    variable outFile
    
    setFile

    for {set i 0} {$i < [expr [array size HCNonSplit] / 2]} {incr i} {
        set mem_name $HCNonSplit(mem_name$i)
        set declared_name $HCNonSplit(dec_name$i)
        puts $outFile "PRINTING MEM:\t$declared_name"
        puts $outFile "***************************************************************"
        set offsets [split [reg_offset -n $mem_name] "-"]
        if {[llength $offsets]>1} {
            set size [expr ([lindex $offsets 1]-[lindex $offsets 0])*8+32]
        } else {
	set size 32
	}


        read_component BATCH  $mem_name $declared_name $size 
        puts $outFile "END\n"    
        
    }
    
    close $outFile
}
# this subroutine added the data from the HC to the output file (all = split and no split)
proc GRC_dump::add_hc_regs {} {

    variable withHC
    variable chip_ver
    set modulus 2
    if {$chip_ver >= 2} {
        set modulus 1
    }
    if {!$withHC} {
		return
		}
		
    #step 1: output all the non split regs
    add_hc_nosplit_regs
    
    #step 2: go over all the devices and output the split regs and pcicfg reads
    run_over_devices add_pcicfg_reads
    run_over_devices add_hc_split_regs $modulus
   
   #step 3: 
}

proc GRC_dump::add_vlan {} {

	set modulus 2
	puts "Reading VLAN information"
	run_over_devices addOuterVlan $modulus
	run_over_devices addLLH $modulus

}
# this subroutine added the data from the HC to the output file
proc GRC_dump::add_hc_split_regs {device} {
    
    variable outFile
    variable HCRelated
    variable dump_no_pci
    variable ::current

    set base 0
    if {$dump_no_pci} {
        set base [expr ($device-1)<<22]
        set func_num [expr $device - 1]
    } else {
        device $device -no_display
        set func_num [expr {$::current(FUNC)}]
    }
    
    setFile
    for {set i 0} {$i<[expr [array size HCRelated]/2]} {incr i} {
        set mem_name $HCRelated(mem_name$i)
        set declared_name $HCRelated(dec_name$i)
        set declared_name "[set declared_name]_[expr $func_num]"
        
        puts $outFile "PRINTING MEM:\t[set declared_name]"
        puts $outFile "***************************************************************"
        set size 32
        set offsets [split [reg_offset -n $mem_name] "-"]
        if {[llength $offsets]>1} {
            set size [expr ([lindex $offsets 1]-[lindex $offsets 0])*8+32]
        }

        read_component BATCH  $mem_name $declared_name $size $base
        puts $outFile "END\n"    
    }
    close $outFile
}

proc GRC_dump::printMemoryToFile {mem_name declared_name {base 0}} {
    variable outFile
    setFile
    
    puts $outFile "PRINTING MEM:\t[set declared_name]"
    puts $outFile "***************************************************************"
    set size 32
    set offsets [split [reg_offset -n $mem_name] "-"]
    if {[llength $offsets]>1} {
        set size [expr ([lindex $offsets 1]-[lindex $offsets 0])*8+32]
    }

    read_component BATCH  $mem_name $declared_name $size $base
    puts $outFile "END\n"
    close $outFile
}

#this subrouting adds the data from the special pcicfg_reads. It
proc GRC_dump::add_pcicfg_reads {device} {
    
    variable outFile
    variable pcicfg_reads
    variable pcicfg_offset
    variable dump_no_pci
    set base 0
    if {$dump_no_pci} {
        set base [expr ($device-1)<<22]
        set func_num [expr $device - 1]
    } else {
        device $device -no_display
        set func_num [expr{$::current(FUNC)}]
    }
    setFile
    for {set i 0} {$i<[expr [array size pcicfg_reads] / 3]} {incr i} {
            set offset_addr $pcicfg_reads(addr$i)
            set offset_addr [expr $offset_addr + $pcicfg_offset]
            set declared_name $pcicfg_reads(decname$i)
            set size $pcicfg_reads(size$i)
            set declared_name "[set declared_name]_$func_num"
            puts $outFile "PRINTING MEM:\t[set declared_name]"
            puts $outFile "***************************************************************"
            read_component BATCH ABSOLUTE $declared_name $size [expr $offset_addr + $base] 32
            puts $outFile "END\n"    
    }
    close $outFile
}

proc GRC_dump::regToStr {reg {bytes 4}} {
    set ans ""
    for {set i [expr {($bytes-1)}]} {$i>=0} {incr i -1} {
        set char [expr {($reg>>(8*$i))&0xff}]
        if {$char} {
            append ans [format "%c" $char]
        } else {
            append ans " "
        }
    }

    return $ans
}

proc GRC_dump::add_mcp_tracebuffer {path_num} {
	variable MCP_scpd
    variable MCP_regs
    variable outFile
    variable mcp_scratchpad
    variable dump_only_path	
	 set mcp_scpd_base $MCP_scpd(scratchpad_addr)	
   #reading the trace buffer only for first path
    if {$path_num == 0 || $dump_only_path > -1} {
        
     set trcb_start [expr {$mcp_scpd_base+$MCP_scpd(trcb_start)}]
        if {[reg read [expr {$mcp_scpd_base+$MCP_scpd(trcb_signature_addr)}]]==$MCP_scpd(trcb_signature)} {
            set trcb_start_ptr [reg read [expr {$mcp_scpd_base+$MCP_scpd(trcb_start_ptr)}]]
            if {$trcb_start_ptr>=$MCP_scpd(trcb_offset_zero) && $trcb_start_ptr<[expr {$MCP_scpd(trcb_offset_zero)+$MCP_scpd(trcb_size)}]} {
                puts "Reading MCP TRACEBUFFER"
                puts $outFile "PRINTING MEM:\tMCP_TRACE_BUFFER"
                puts $outFile "***************************************************************"
                set diff [expr {$trcb_start_ptr-$MCP_scpd(trcb_offset_zero)}]
                set sepReg [reg read [expr {$trcb_start+$diff-$diff%4}]]
                puts -nonewline $outFile [regToStr [expr {$sepReg}] [expr {4-$diff%4}]]
                for {set i [expr {$trcb_start+$diff-$diff%4+4}]} {$i<[expr {$trcb_start+$MCP_scpd(trcb_size)}]} {incr i 4} {
                    puts -nonewline $outFile [regToStr [reg read $i]]
                }
                for {set i [expr {$trcb_start}]} {$i<[expr {$trcb_start+$diff-$diff%4}]} {incr i 4} {
                    puts -nonewline $outFile [regToStr [reg read $i]]
                }
                puts $outFile [regToStr [expr {$sepReg>>((4-$diff%4)*8)}] [expr {$diff%4}]]
                puts $outFile "END\n"  
                #puts $moreFile "more prevent"
        } else {
                puts "Can't read MCP TRACEBUFFER since the pointer is out of bounds."
            }
        } else {
            puts "Can't read MCP TRACEBUFFER since the signature doesn't exist."
        }
	}
}


proc GRC_dump::add_mcp {{MCPFULL 0} {path_num 0}} {
    variable MCP_scpd
    variable MCP_regs
    variable outFile
    variable mcp_scratchpad
	variable waitPstatus
	variable chip_ver
	set old_wpstatus [set waitPstatus]
	# we're updating this so that the MCP will be counted as OFFLINE ONLY
	set waitPstatus true
    
    setFile
    set mcp_scpd_base $MCP_scpd(scratchpad_addr)
    #reading the trace buffer only for first path
    add_mcp_tracebuffer $path_num
    if {$path_num == 0} {
    puts $outFile "PRINTING MEM:\tMCP_SHARED_MEM"
    } else {
        puts $outFile "PRINTING MEM:\tMCP_SHARED_MEM_PATH_1"
    }
    puts $outFile "***************************************************************"
    read_component BATCH ABSOLUTE {MCP SHARED MEM} $MCP_scpd(shared_mem_size) [expr {$mcp_scpd_base+$MCP_scpd(shared_mem_offset)}] 32

    puts $outFile "END\n"
    
	if {$chip_ver >= 2} {
		set shmem2_addr [reg_read -n MISC_REG_GENERIC_CR_0]
		if {$shmem2_addr != 0} {
			set shmem2_size [reg read $shmem2_addr]
			if {$shmem2_size >= 4} { # must contain at least a DWORD for the size
				puts $outFile "PRINTING MEM:\tMCP_SHARED_MEM_2"
				puts $outFile "***************************************************************"
				read_component BATCH ABSOLUTE {MCP SHARED MEM 2} [expr {$shmem2_size * 8}] $shmem2_addr 32
				puts $outFile "END\n"
			}
			
			set dcbx_offset_addr $MCP_scpd(dcbx_lldp_addr_in_shmem2)
			set dcbx_params_addr [reg read [expr {$dcbx_offset_addr + $shmem2_addr}]]
			
			set dcbx_local_mib_offset_addr $MCP_scpd(dcbx_lldp_local_addr_in_shmem2)
			set dcbx_local_mib_addr [reg read [expr {$dcbx_local_mib_offset_addr + $shmem2_addr}]]
			
			set dcbx_remote_mib_offset_addr $MCP_scpd(dcbx_lldp_remote_addr_in_shmem2)
			set dcbx_remote_mib_addr [reg read [expr {$dcbx_remote_mib_offset_addr + $shmem2_addr}]]
			
			if {$dcbx_params_addr != 0} {
				set params_size_bytes_per_port $MCP_scpd(dcbx_lldp_params_size_bytes)
				set admin_size_bytes_per_port $MCP_scpd(dcbx_admin_mib_size_bytes)
				
				set curr_addr $dcbx_params_addr
				puts $outFile "PRINTING MEM:\tMCP_DCBX_PARAMS_0"
				puts $outFile "***************************************************************"
				read_component BATCH ABSOLUTE {MCP DCBX PARAMS 0} [expr {$params_size_bytes_per_port * 8}] $curr_addr 32
				puts $outFile "END\n"
				
				incr curr_addr $params_size_bytes_per_port
				
				puts $outFile "PRINTING MEM:\tMCP_DCBX_PARAMS_1"
				puts $outFile "***************************************************************"
				read_component BATCH ABSOLUTE {MCP DCBX PARAMS 1} [expr {$params_size_bytes_per_port * 8}] $curr_addr 32
				puts $outFile "END\n"
				
				incr curr_addr $params_size_bytes_per_port
				
				puts $outFile "PRINTING MEM:\tMCP_DCBX_ADMIN_MIB_0"
				puts $outFile "***************************************************************"
				read_component BATCH ABSOLUTE {MCP DCBX ADMIN MIB 0} [expr {$admin_size_bytes_per_port * 8}] $curr_addr 32
				puts $outFile "END\n"
				
				incr curr_addr $admin_size_bytes_per_port
				
				puts $outFile "PRINTING MEM:\tMCP_DCBX_ADMIN_MIB_1"
				puts $outFile "***************************************************************"
				read_component BATCH ABSOLUTE {MCP DCBX ADMIN MIB 1} [expr {$admin_size_bytes_per_port * 8}] $curr_addr 32
				puts $outFile "END\n"
			}
			
			if {$dcbx_local_mib_addr != 0} {
				set local_mib_size_bytes_per_port $MCP_scpd(dcbx_local_mib_size_bytes)
				
				set curr_addr $dcbx_local_mib_addr
				puts $outFile "PRINTING MEM:\tMCP_DCBX_LOCAL_MIB_0"
				puts $outFile "***************************************************************"
				read_component BATCH ABSOLUTE {MCP DCBX LOCAL MIB 0} [expr {$local_mib_size_bytes_per_port * 8}] $curr_addr 32
				puts $outFile "END\n"
				
				incr curr_addr $local_mib_size_bytes_per_port
				
				puts $outFile "PRINTING MEM:\tMCP_DCBX_LOCAL_MIB_1"
				puts $outFile "***************************************************************"
				read_component BATCH ABSOLUTE {MCP DCBX LOCAL MIB 1} [expr {$local_mib_size_bytes_per_port * 8}] $curr_addr 32
				puts $outFile "END\n"
			}
			
			if {$dcbx_remote_mib_addr != 0} {
				set remote_mib_size_bytes_per_port $MCP_scpd(dcbx_remote_mib_size_bytes)
				
				set curr_addr $dcbx_remote_mib_addr
				puts $outFile "PRINTING MEM:\tMCP_DCBX_REMOTE_MIB_0"
				puts $outFile "***************************************************************"
				read_component BATCH ABSOLUTE {MCP DCBX REMOTE MIB 0} [expr {$remote_mib_size_bytes_per_port * 8}] $curr_addr 32
				puts $outFile "END\n"
				
				incr curr_addr $remote_mib_size_bytes_per_port
				
				puts $outFile "PRINTING MEM:\tMCP_DCBX_REMOTE_MIB_1"
				puts $outFile "***************************************************************"
				read_component BATCH ABSOLUTE {MCP DCBX REMOTE MIB 1} [expr {$remote_mib_size_bytes_per_port * 8}] $curr_addr 32
				puts $outFile "END\n"
			}			
			
			
			
		}
		
		
		
		
		
	}
		

    
    foreach reg [array names MCP_regs] {
        if {[string equal -length 4 size $reg]} {
            continue
        }
        if {[info exist MCP_regs(size$reg)]} {
            set size [expr {$MCP_regs(size$reg)/32}]
        } else {
            set size 1
        }
        
        puts $outFile "PRINTING MEM:\tMCP_$reg"
        puts $outFile "***************************************************************"
        for {set i 0} {$i<$size} {incr i} {
            add_mem_to_file [format_mem [reg_show [expr {$MCP_regs($reg)+$i*4}]] 32] {MCP REGS} $i
        }
        puts $outFile "END\n"  
    }
    if {$MCPFULL} {
        puts $outFile "PRINTING MEM:\tMCP_SCRATCHPAD"
        puts $outFile "***************************************************************"
        read_component BATCH ABSOLUTE {MCP SCRATCHPAD} $MCP_scpd(scratchpad_size) $mcp_scpd_base 32 
        puts $outFile "END\n"  
    }
    close $outFile
	set waitPstatus [set old_wpstatus]
}       

proc GRC_dump::pcieDump {device} {
    variable outFile
    variable pcieCore
    variable dump_no_pci
    setFile
    set base 0
    if {$dump_no_pci} {
        set base [expr {($device-1)<<22}]
    } else {
        device $device -no_display
    }
    puts $outFile "Device $device:"
    for {set i 0} {$i<[expr [array size pcieCore]/2]} {incr i} {
        set currDump [format_mem [reg_show [expr {$base+$pcieCore(addr$i)}] $pcieCore(size$i)] 32 hex]
        puts $outFile "Addresses [format "0x%x-0x%x" $pcieCore(addr$i) [expr {$pcieCore(addr$i)+$pcieCore(size$i)-4}]]"
        for {set j 0} {$j<[llength $currDump]} {incr j} {
            if {[expr {$j%8==0}]} {
                puts -nonewline $outFile [format %.8x: [expr {$pcieCore(addr$i)+$j*4}]]
            }
            puts -nonewline $outFile "[lindex $currDump $j]"
            if {[expr {($j+1)%8==0}]} {
                puts $outFile {}
            } else {
		puts -nonewline $outFile { }
	    }		
        }
        puts $outFile {}    
    }
    puts $outFile "END\n"
    close $outFile
}
proc GRC_dump::dump_igu_error_handling {} {
	
	if {[reg_read -n IGU_REG_ERROR_HANDLING_DATA_VALID] == 0} {
			return
		}
	
	
	 if {[reg_read -n IGU_REG_COMMAND_DEBUG]} {
		set data_type "IGU_LAST_COMMANDS"
	} else {
		set data_type "IGU_FIRST_ERRORS"
	}

	set counter 0
	
	while {[reg_read -n IGU_REG_ERROR_HANDLING_DATA_VALID]} {
		printMemoryToFile IGU_REG_ERROR_HANDLING_MEMORY "[set data_type]_[set counter]"
		incr counter
	}	
	 
}

proc GRC_dump::dump_vfc {storm} {
    variable outFile
    variable vfc_sizes
    if {![VFCIsAlive $storm]} {
        puts "Cannot Read from VFC."
        return
    }
    setFile
    puts $outFile "PRINTING [set storm]Storm VFC RAM"
    puts $outFile "***************************************************************"
    set curr_line_num 0
    for {set i 0} {$i < $vfc_sizes(ram_num_lines)} {incr i} {
        set result [VFCReadRamLine $storm $i]    
        set result_as_string [RegListToBinary $result 2]
        for {set t 0} {$t < [llength $result_as_string]} {incr t} {
            puts $outFile [format "%.8x\t%s" $curr_line_num [lindex $result_as_string $t]]
            incr curr_line_num
        }
    }
    puts $outFile "END\n"
    puts $outFile "PRINTING [set storm]Storm VFC CAM"
    puts $outFile "***************************************************************"
    set curr_line_num 0
    for {set i 0} {$i < $vfc_sizes(cam_num_lines)} {incr i} {
        set result [VFCCAMRead $storm $i]    
        set result_as_string [RegListToBinary $result 2]
        for {set t 0} {$t < [llength $result_as_string]} {incr t} {
            puts $outFile [format "%.8x\t%s" $curr_line_num [lindex $result_as_string $t]]
            incr curr_line_num
        }
    }
    puts $outFile "END\n"
    puts $outFile "PRINTING [set storm]Storm VFC ALU_Vectors"
    puts $outFile "***************************************************************"
    set curr_line_num 0
    for {set i 0} {$i < $vfc_sizes(alu_num_lines)} {incr i} {
        set result [VFCReadALUVector $storm $i]
        set result_as_string [RegListToBinary $result 2]
        for {set t 0} {$t < [llength $result_as_string]} {incr t} {
            puts $outFile [format "%.8x\t%s" $curr_line_num [lindex $result_as_string $t]]
            incr curr_line_num
        }
    }
    puts $outFile "END\n"
    close $outFile
    
}

proc GRC_dump::read_reg_data {reg_name} {
    set addresses [split [reg_offset -n $reg_name] "-"]
    set start_offset [expr [lindex $addresses 0]]
    set size 1
    if {[llength $addresses] > 1} {
        set end_offset [expr [lindex $addresses 1]]
        set size_calc [expr {$end_offset + 1 - $start_offset + 4 - 1}]
        set size [expr {$size_calc / 4}]
    }
    
    set data 0
    for {set i 0} {$i < $size} {incr i} {
        set data [expr {$data << 32}]
        set data [expr {$data + [reg read [expr {$i * 4 + $start_offset}]]}]
    }
    return [format "0x%x" $data]
}

proc GRC_dump::collect_stats {noQstat} {
    variable semiStats
    variable statsByName
    
    set stats_data {}
    
    append stats_data "SEMI STATISTICS:\n"
    foreach letter "X T U C" {
        UpdateFastMemOffsets $letter
        append stats_data [format "Stats for %sStorm\n" $letter]
        set addr [get_start_offset [reg_offset -n [format "%sSEM_REG_FAST_MEMORY" $letter]]]
        foreach stat [array names semiStats] {
            if {[regexp ^align $stat]} {
                continue
            }
            if {[info exists semiStats(align$stat)]} {
                set align .$semiStats(align$stat)
            } else {
                set align {}
            }
            append stats_data [format "%s: 0x%[set align]x\n" $stat [reg read [expr {$addr+$semiStats($stat)}]]]
        }
        append stats_data "\n"
    }
    append stats_data "STATISTICS FROM BLOCKS:\n"
    foreach block $statsByName {
        variable [subst stats$block]
	set reg_list [subst $[subst stats$block]]
        append stats_data "$block STATISTICS:\n"
        for {set i 0} {$i<[llength $reg_list]} {incr i} {
            set reg [lindex $reg_list $i]
            regexp "_REG_(.+)$" $reg reg_name reg_name
            append stats_data [format "%s: 0x%x\n" $reg_name [read_reg_data $reg]]
        }
        append stats_data "\n"
    }

    if {!$noQstat} {
        if {[catch {set qStats [qstat]} msgerr]} {
            puts "Some problem using qstat. The error was: $msgerr"
        } else {
            append stats_data "Q_STATISTICS:\n"
            append stats_data $qStats
            append stats_data "\n"
        }
    }

    return $stats_data
}

# this function dumps the data into the requested output file. and can recieve another variables 
# args which can be "-n numberOfConnections" | "-withior"  | "-withiornos" | "-noram" | "-nopb" | "-noqm".
# if "-withior" / "-withiornos" is use, the ior will be sampled during the dump (NOTE: in order to read the iors
# the storms must be stoppen for few seconds - the difference between the 2 is that withiornos (withior no stop) will return the storms after sampling)

proc GRC_dump::grcDump {{outputFile} {args}} {
    
	variable activeIor
	variable outFileName
	variable outFile
	variable noIdleChk
	variable no_parity_mask
	set no_parity_mask false
	
	
        variable dump_no_pci
        variable scriptInc
        variable FIRST_RUN
        variable semiStats
        variable no_qstat
        variable nomcp
        variable chip_ver
        variable chip_mc_ver
        variable user_chip_mc_ver 0
	variable withHC
        variable withInterruptTable
        variable is_partial_dump
        variable no_vfc
        variable last_func_for_path
        variable funcs
        variable func0
        variable noilt
		variable dump
		variable dump_chosen_path
        global current
        variable with_cfc_string_cam
        variable dump_only_path -1
        set with_cfc_string_cam false
        set noilt false
        set no_vfc true
	set IOR 0
        set FULL 1
        set MCPFULL 1
        set noram false
        set noqm false
	set nopb false
        set nobrb false
	set SPLITFULL 0
        set noPcieCore false
        set noIdleChk false
        set numOfConnectios 256
        set storms "T U C X"
        set devCount $::current(TOTAL_DEV)
        set UserNoMcp 0
        set funcs $devCount
        set func0 $::current(DEV)
	set withHC true
        set withInterruptTable true
        set noPstat false
		set withIguErrorFifo false
		set withBmac false

        
        prepare
        
        set outFileName $outputFile
	

	
        for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {
            "-n"            { incr i
                               if {[lindex $args $i]<257&&[lindex $args $i]>0} {
                                        set numOfConnectios [lindex $args $i]
                                } else {return "Number of connections must be between 1 to 256."}
                            }
            "-withior"      { set IOR 1}
            "-withiornos"   { set IOR 2}
			"-withigufifo"  { set withIguErrorFifo true }
			"-withbmac"		{ set withBmac true }
			"-crash"		{ 
								set withIguErrorFifo true
								set IOR 2
								set no_vfc false
								set MCPFULL 1
								set withBmac true
								set FULL 1
								set SPLITFULL 1
							}
            "-noram"        { set noram true }
            "-nopb"         { set nopb true }
            "-noqm"         { set noqm true }
            "-nobrb"        { set nobrb true }
	    "-nohc"	    { set withHC false }
            "-withvfc"	    { set no_vfc false }
            "-storms"       { incr i
                              set storms [lindex $args $i]
                              if {![regexp {^[TUCX](\s[TUCX])*$} $storms]} {
                                return "Unknown storm requested. Known Storms are T, U, C and X"
                              }
                            }
            "-nofull"         { set FULL 0
                            }
            "-mcpfull"      { set MCPFULL 1}
			"-nomcpfull"      { set MCPFULL 0}
            "-nomcp"        { set UserNoMcp 1}
	    "-splitfull"    { set FULL 1
				set SPLITFULL 1
			}
			"-noparitymasking" {set no_parity_mask true}
            "-chip"         { incr i
                              if {[regexp {^E(1|1.5|2|3)$} [lindex $args $i] ch_ver ch_ver]} {
                                set user_chip_mc_ver $ch_ver
                                if {$user_chip_mc_ver>$chip_ver} {
                                    puts "You can't dump memories in E1.5/E2 structs from E1."
                                    return -1
                                }
                              } else {
                                puts "unknown chip: [lindex $args $i], allowed versions are E1 and E1.5 only."
                                return -1
                              }
                            }
			"-ignoremcchip" { 
								variable ignore_mc_chip
								set ignore_mc_chip true
							}
            "-noint"        {set withInterruptTable false}
            "-mini"        { set UserNoMcp 1
                              set FULL 0
                              set noqm true
                              set nobrb true
                              set noPcieCore true
                              set withHC false
                              set withInterruptTable false
                              set noIdleChk true}
            "-nopciecore"   { set noPcieCore true}
            "-noidlechk"    { set noIdleChk true }
            "-nopstat"      { set noPstat true }
            "-noqstat"      { set no_qstat 1 }
            "-nostrcam"     { set with_cfc_string_cam false }
            "-noilt"        { set noilt true }
            "-dumppath"     { incr i
                              set dump_only_path [lindex $args $i]
                            }
            default         { error "unallowed arguments - [lindex $args $i]. use help grcDump to recieved allowed arguments list."}
            }
        }

        
        # qstat is not supported in edebug (it requires driver envolvement)
        if {$::sys(IS_EDEBUG) == 1} {
            set no_qstat 1
        }
        
        if {[info exists dump_chosen_path]} {
			set dump_only_path $dump_chosen_path
		}
        const_declare $user_chip_mc_ver
	if {[string equal $chip_mc_ver "1.5"]} { prepareIdleChk E1H } else { prepareIdleChk E1 }

    
        if {$chip_ver < 2} {
            set max_paths 1
        } else {
            set max_paths 2
        }

        if {$dump_only_path >= $max_paths} {
            return "You asked to dump path $dump_only_path which doesn't exist for E$chip_ver"
        }
        

        set first_run true
        if {!$dump_no_pci} {
            set original_device $::current(DEV)
        }
	
	#masking MCP parity
	if {$MCPFULL} {
		if {$chip_ver >= 2} {
			for {set current_chosen_path 0} {$current_chosen_path < $max_paths} {incr current_chosen_path} {
		
				if {!$dump_no_pci} {
					set current_path [GetCurrentPathNum]
					if {$current_path != $current_chosen_path} {
						set other_dev [FindFunctionFromOtherPath]
						if {!$other_dev} {
							continue
						}
						#otherwise, change to the other function
						device $other_dev -no_display
					}
				} else {
					if {$chip_ver >= 2} {
						UARTSetPath $current_chosen_path
					}
				}
				puts "Masking MCP Parity for path $current_chosen_path"
				maskMCP true
				
			}
		}  else {
			puts "Masking MCP Parity."
			maskMCP true
		}
	}
			
	if {!$dump_no_pci} {
            device $original_device -no_display
        }	else {
            if {$chip_ver >= 2} {
                UARTUnsetPath
            }
	 }
	    
	
	
	
	
	
	
        for {set current_chosen_path 0} {$current_chosen_path < $max_paths} {incr current_chosen_path} {
            
            if {$dump_only_path > -1} {
                if {$current_chosen_path != $dump_only_path} {
                    continue
                }
            }
            if {!$dump_no_pci} {
                set current_path [GetCurrentPathNum]
                if {$current_path != $current_chosen_path} {
                    set other_dev [FindFunctionFromOtherPath]
                    if {!$other_dev} {
                        continue
                    }
                    #otherwise, change to the other function
                    device $other_dev -no_display
                }
            } else {
                if {$chip_ver >= 2} {
                    UARTSetPath $current_chosen_path
                }
            }
            
        
        
            setFile w $current_chosen_path $first_run
            set first_run false
            if {!$noIdleChk} {
                puts "Dumping 2 Idle Checks"
                add_idle_chk
            }
            puts "Done with Idle Checks."
            
			if {!$no_parity_mask} {
					puts "Masking parity for all blocks."
					maskAllBlocks
			}
            if {!$noPstat} {
                puts "Dumping PStat"
                add_pstat
                puts "Done with PStat"
            }
            puts "Dumping the memories as E$chip_mc_ver."
        
            if {!$no_vfc} {
                if {[string equal $chip_mc_ver "2"] || [string equal $chip_mc_ver "3"]} {         
                    foreach storm_letter {X T} {
                        puts "Dumping VFC Data for [set storm_letter]Storm."
                        dump_vfc $storm_letter
                        puts "Done with VFC for [set storm_letter]Storm."            
                    }
                }
            }
        
			if {$withIguErrorFifo} {
				if {[string equal $chip_mc_ver "2"] || [string equal $chip_mc_ver "3"]} {         
					puts "Dumping IGU Error Handling Memory."
					dump_igu_error_handling
					puts "Done with IGU Error Handling Memory."
				}
			}
        
        
            puts "checking reset regs"
            check_reset
            # Mcp scrachpad data + some configuration data
            if {!$nomcp && !$UserNoMcp} {
                UpdateMCPConstants $chip_ver $current_chosen_path
                add_mcp $MCPFULL $current_chosen_path
            }
        
            puts "Collecting stats"
            # collecting stats
            set stats_data [collect_stats [expr {$dump_no_pci | $no_qstat}]]
	    addExtensionOutput
	    
            # check to see if ior should be checked
            
            if {!$dump_no_pci} {
                set dev $::current(DEV)
                set funcs 0
                # switch to function 0 of the device
                set func0 [FindCurrentPathFunc0]
                set d $func0
                set last_func_for_path [expr {$d + 1}]
                for {} {$d<=$devCount} {set d [NextDeviceNum $d]} {
                    
                    set device_change_error [catch {device $d -no_display}]
                    if {$d!=$func0 && [expr {$::current(FUNC)}]==0} {
                        break
                    }
                    if {!$device_change_error} {
                        incr funcs
                        set last_func_for_path [expr {$d + 1}]
                    }
                }
                
                device $dev -no_display
            } else {
                set func0 1
                set dev 1
                if {$chip_ver>1 && $chip_mc_ver>1} {
                    set funcs 8
                    
                } else {
                    set funcs 2
                }
                if {$chip_ver >= 2} {
                    # for E2, only 4 functions per path
                    set funcs 4
                }
            }
			if {!$is_partial_dump} {
                if {$IOR} {
                    append stats_data "\nPRAM during WAITP:\n"
                    if {!$dump_no_pci} {
                        for {set d $func0} {$d<$last_func_for_path} {set d [NextDeviceNum $d]} {
                            device $d -no_display
                            stats disable
                        }
                    }
                    stormWaitP true
                }
            #ending partial dump
            }
            #PXP
            add_pxp_data
            
            # CFC 
            add_cfc_cid_cam $numOfConnectios
            if {$with_cfc_string_cam} {
                add_cfc_string_cam
            }
            
            add_cfc_ac_ram $numOfConnectios
            add_cfc_info_ram $numOfConnectios
            
            #ILT
            if {!$noilt} {
                addILT    
            }
            
            
            #QM memories
            if {!$noqm} {
                    add_qm_mem
            }
    	
            #interrupt table - The Linux team requested on 8/2014 not to dump the interrupt tables
#            getInterruptTables
            
            #tsemi cam
            add_sem_cam T
    
            #brb1 ram
            if {!$nobrb} {
                add_brb_bram
            }

            # ACTIVE/PASSIVE IOR
            if {!$is_partial_dump} {
                if {$IOR} {
                    add_active_iors
                }
            }
            set storms [split $storms " "]
            foreach stormLetter $storms {
                
                # STORM RAM
                if {$noram == "false"} {
                    add_storm_ram $stormLetter 0
                    if {$chip_mc_ver >= 2} {
                    add_storm_ram $stormLetter 1
                    }
                }
                
                # STORM CONTEXT SIZE OF EACH TYPE              
                add_storms_ctx_sizes $stormLetter
    
                # STORM CONTEXTS
                read_context $stormLetter STORM $numOfConnectios
                read_context $stormLetter AG $numOfConnectios
    
                # STORM PASSIVE BUFFER
                if {$nopb == "false"} {
                    add_storm_passive_buffer $stormLetter
                }
    
                if {!$is_partial_dump} {
                    # STORM IOR SETS
                    if {$IOR} {
                            UpdateFastMemOffsets $stormLetter
                            add_storm_iors $stormLetter
                            set addr [get_start_offset [reg_offset -n [format "%sSEM_REG_FAST_MEMORY" $stormLetter]]]
                            append stats_data [format "%sstorm_pram_address:\t%.8x\n" $stormLetter [reg read [expr {$addr+$semiStats(last_pram_address)}]]]
                    }
                }
            }
            # release the Waitp - but only for the -withiornos
            if {!$is_partial_dump} {
                if {$IOR > 1} {
                    stormWaitP false
                    append stats_data "PRAM after WAITP:\n"
                    foreach stormLetter $storms {
                        UpdateFastMemOffsets $stormLetter
                        set addr [get_start_offset [reg_offset -n [format "%sSEM_REG_FAST_MEMORY" $stormLetter]]]
                        append stats_data [format "%sstorm_pram_address:\t%.8x\n" $stormLetter [reg read [expr {$addr+$semiStats(last_pram_address)}]]]
                    }
                    puts "Since you asked to dump the IORs, the script disabled the statistics and"
                    puts "turn on the waitp signal for the process. The waitp had been turn off now"
                    puts "but the statistics will stay disabled."
                }
            }
                
            ## pcieCore and hc dump 
        
            # reading by devices
            if {$dump_no_pci} {
                set last_func_for_path [expr {$func0 + $funcs}]
            }
        
            for {set d $func0} {$d<$last_func_for_path} {set d [NextDeviceNum $d]} {
                if {$dump_no_pci} {
                    set func_num [expr {$d - $func0}]
                } else {
                    set func_num [expr {$::current(FUNC)}]
                }
                puts "reading pciecore registers from device $d (function [expr $func_num])"
                catch {add_pcieCore_regs $d}
            }
        
			if {$withBmac} {
				if {$chip_ver >= 2} {
					for {set d $func0} {$d<$last_func_for_path} {set d [NextDeviceNum $d]} {
						if {$dump_no_pci} {
							set func_num [expr {$d - $func0}]
						} else {
							set func_num [expr {$::current(FUNC)}]
						}
						puts "reading BMAC Stats registers from device $d (function [expr $func_num])"
						catch {pfcDumpStats $d}
					}
				}
			}
		
            add_hc_regs
			add_vlan
            setFile
            puts $outFile "\n\n\nEND OF MEMORIES\n\n\n"
            puts $outFile "STATISTICS:"
            puts $outFile $stats_data
        
            puts $outFile "END OF STATISTICS\n\n\n"
        
            if {!$noPcieCore} {
                puts $outFile "PCIECORE:"
                close $outFile
                for {set d $func0} {$d<$last_func_for_path} {set d [NextDeviceNum $d]} {
                    puts "Reading pciecore from device $d (function [expr $d-$func0])"
                    catch {pcieDump $d}
                }
                setFile
                puts $outFile "END OF PCIECORE"
            
            
            }    
            # return to the original device
            if {!$dump_no_pci} { 
                device $dev -no_display
            }
        
            ## full dump of all the registers
            if {$FULL} {
                puts "Reading All Registers"
                puts $outFile "\n\n\nALL REGISTERS:"
                close $outFile
                if {[catch {read_regs_all -silent -out $GRC_dump::outFileName} errMsg]} {
                    if {[regexp "^invalid command name" $errMsg]} {
                        puts "Your regutils.tcl version isn't updated.\nYou must use a version from 18/09/2007 or later."
                    } else {
                        puts $errMsg
                    }
                }
		
	
		
            setFile
            puts $outFile "\n\n\nEND OF ALL REGISTERS\n\n"
	    close $outFile
	    if {$SPLITFULL} {
			markCamArea start_split
	    	run_over_devices add_split_fullregs
			markCamArea stop_split
	    }
            }

		
			#restoring parity
			if {!$no_parity_mask} {
				puts "Restoring parity"
				restoreAllBlocksMasking
			}
			
        }


        if {!$dump_no_pci} {
            device $original_device -no_display
        } else {
            if {$chip_ver >= 2} {
                UARTUnsetPath
            }
        }
	
	#masking MCP parity
	if {$MCPFULL} {
		if {$chip_ver >= 2} {
			for {set current_chosen_path 0} {$current_chosen_path < $max_paths} {incr current_chosen_path} {
		
				if {!$dump_no_pci} {
					set current_path [GetCurrentPathNum]
					if {$current_path != $current_chosen_path} {
						set other_dev [FindFunctionFromOtherPath]
						if {!$other_dev} {
							continue
						}
						#otherwise, change to the other function
						device $other_dev -no_display
					}
				} else {
					if {$chip_ver >= 2} {
						UARTSetPath $current_chosen_path
					}
				}
				maskMCP false
				puts "Unamsked MCP Parity for path $current_chosen_path"
			}
		} else {
			maskMCP false
			puts "Unmasked MCP Parity."
		}
		
	}
	
	return "DONE"

}

proc	GRC_dump::getPresetArgs {index} {

	variable preset_lists
	
	variable preset_lists_init
	if {!$preset_lists_init} {
		set preset_lists {}
		foreach storm {C T X U} {
			lappend preset_lists "-nofull -nomcp -noqm -nohc -noilt -noint -storms \{[set storm]\} -n 1"
			lappend preset_lists "-nofull -nomcp -noqm -nobrb -nohc -noilt -noint -storms \{[set storm]\} -noram -withiornos"
			lappend preset_lists "-nomcp -nobrb -noint -storms \{[set storm]\} -noram"
			
		}
		lappend preset_lists "-nofull -mcpfull -noqm -nobrb -nohc -noilt -storms \{C\} -noram -n 1 -noint -nopciecore -nostrcam -noidlechk"
		set preset_lists_init true
	}
	
	if {$index < 0} {
		return [llength $preset_lists]
	} else {
		return [lindex $preset_lists $index]
	}
}


proc	GRC_dump::runPresets {chip {b0_rev false} chip_index} {

	variable addr_dump_properties
	variable rdf_file_to_use
	catch {unset addr_dump_properties}
	array set addr_dump_properties {}
	set addr_dump_properties(curr_chip_index) $chip_index
	variable dumpChip
	variable neededRegs
	variable chip_rev_reg
	variable current_chip_revision
	variable rdfs_files
	array set mapChipToRdf {E1 everest.rdf E1H evrst15.rdf E2 everst2.rdf E3 everst3.rdf E3B0 everst3b.rdf}
	global	rdfFile
	set dumpChip [string toupper $chip]
	if {$b0_rev} {
		
		set current_chip_revision B0
		set neededRegs(E3,[expr {$chip_rev_reg}]) 1
		set rdf_file_to_use everst3b.rdf
		parse_rdf -f everst3b.rdf -clr
	} else {
		set current_chip_revision {}
		puts "DEBUG:got here"
		puts "DEBUG:[set dumpChip] [set mapChipToRdf([set dumpChip])]"
		set rdf_file_to_use [set mapChipToRdf([set dumpChip])]
		parse_rdf -f [set mapChipToRdf([set dumpChip])] -clr
	}
	
	set num_presets [getPresetArgs -1]
	for {set i 0} {$i < $num_presets} {incr i} {
			set addr_dump_properties(curr_preset) $i
			emulateGrcDump [getPresetArgs $i]
	}
}

proc GRC_dump::str_rep {str f_char n_char} {
	set result $str
	set loc [string first $f_char $result]
	while {$loc > 0} {
		set result [string replace $result $loc $loc $n_char]
		set loc [string first $f_char $result]
	}
	return $result
}


proc GRC_dump::generateLinuxFiles {c_file_name tcl_file_name t_stamp} {

	# Step 1: TCL File
	set tcl_file [open $tcl_file_name "w"]
	variable register_information
	puts $tcl_file "array set ri \{\}\n"
	puts "Checkpoint 1"
	foreach name [array names register_information] {
		
	
		if {![regexp list, $name]} {
			continue
		}
		if {[regexp cam $name]} {
			set outCamList {}
			puts "DEBUG: special print $name"
			GRC_dump::arrangeCamList $register_information($name) outCamList
			puts $tcl_file "set ri($name) \{[lsort -unique $outCamList]\}"
			continue
		}
		
		if {[regexp reg $name] || [regexp idle $name]} {
			#set output_list [lsort -unique -integer $register_information($name)]
			#puts "DEBUG: printing $name"
			#set output_list [SimpleArrangeList $output_list]
			#puts $tcl_file "set ri($name) \{$output_list\}"
			
			continue
		}
	}
	puts $tcl_file "set preset_lists \{\}"
	puts "Checkpoint 2"
	set num_presets [getPresetArgs -1]
	for {set i 0} {$i < $num_presets} {incr i} {
			puts $tcl_file "lappend preset_lists \{[getPresetArgs $i]\}"
	}
	
	set dump_version 0
	
	puts $tcl_file "set time_stamp $t_stamp"
	
	puts "Checkpoint 3"
	# Step 2: C File
	 
    variable const_ri_e1
    variable const_ri_e1h
    variable const_ri_e2
    variable const_ri_e3
	variable const_ri_e3b0
    variable const_ri_p0_dump
    variable const_ri_p1_dump
	set c_file [open $c_file_name "w"]
	set c_file_name_define [str_rep $c_file_name . _]
	
	
	puts $c_file "/* bnx2x_dump.h: QLogic Everest network driver."
 puts $c_file "*"
 puts $c_file "* Copyright (c) 2012 Broadcom Corporation"
 puts $c_file "* Copyright (c) 2014-2015 QLogic Corporation"
 puts $c_file "*"
 puts $c_file "* This program is free software; you can redistribute it and/or modify"
 puts $c_file "* it under the terms of the GNU General Public License as published by"
 puts $c_file "* the Free Software Foundation."
 puts $c_file "*"
 puts $c_file "* Maintained by: Ariel Elior <ariel.elior@qlogic.com>"
 puts $c_file "* Written by: Gil Berkovski"
 puts $c_file "*/\n"
 
 
 
	puts $c_file "\#ifndef [string toupper $c_file_name_define]"
	puts $c_file "\#define [string toupper $c_file_name_define]\n"
	
	puts $c_file "\#ifndef ARRAY_SIZE"
	puts $c_file "\#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof(arr\[0\]))"
	puts $c_file "\#endif"
	
	puts $c_file "\n/* WaitP Definitions */"
	puts $c_file {#define DRV_DUMP_XSTORM_WAITP_ADDRESS    0x2b8a80}
	puts $c_file {#define DRV_DUMP_TSTORM_WAITP_ADDRESS    0x1b8a80}
	puts $c_file {#define DRV_DUMP_USTORM_WAITP_ADDRESS    0x338a80}
	puts $c_file {#define DRV_DUMP_CSTORM_WAITP_ADDRESS    0x238a80}
	
		
	set num_presets 13
	set num_chips 5
	
	
	puts $c_file "\n\n/* Possible Chips */"
	puts $c_file "\#define DUMP_CHIP_E1 $const_ri_e1"
	puts $c_file "\#define DUMP_CHIP_E1H $const_ri_e1h"
	puts $c_file "\#define DUMP_CHIP_E2 $const_ri_e2"
	puts $c_file "\#define DUMP_CHIP_E3A0 $const_ri_e3"
	puts $c_file "\#define DUMP_CHIP_E3B0 $const_ri_e3b0"
	
	puts $c_file "\#define DUMP_PATH_0 $const_ri_p0_dump"
	puts $c_file "\#define DUMP_PATH_1 $const_ri_p1_dump"
	puts $c_file "\#define NUM_PRESETS $num_presets"
	puts $c_file "\#define NUM_CHIPS $num_chips\n"
	
	
	
	puts $c_file "struct	dump_header \{"
	puts $c_file "\tu32 header_size; /* Size in DWORDs excluding this field */"
	
	puts $c_file "\tu32 version;"
	
	puts $c_file "\tu32 preset;"
	
	puts $c_file "\tu32	dump_meta_data; /* OR of CHIP and PATH. */"
	
	puts $c_file "\};\n"
	
	puts $c_file "\#define  BNX2X_DUMP_VERSION [set t_stamp]"

	puts $c_file "struct reg_addr \{"
    puts $c_file "\tu32 addr;"
    puts $c_file "\tu32 size;"
    puts $c_file "\tu32 chips;"
	puts $c_file "\tu32 presets;"
    puts $c_file "\};\n"

    puts $c_file "struct wreg_addr \{"
    puts $c_file "\tu32 addr;"
    puts $c_file "\tu32 size;"
    puts $c_file "\tu32 read_regs_count;"
    puts $c_file "\tconst u32 *read_regs;"
    puts $c_file "\tu32 chips;"
	puts $c_file "\tu32 presets;"
    puts $c_file "\};\n"
	

puts "Checkpoint 4"
puts	$c_file "\#define PAGE_MODE_VALUES_E2 2"

puts	$c_file "\#define PAGE_READ_REGS_E2 1"

puts	$c_file "\#define PAGE_WRITE_REGS_E2 1"

puts	$c_file "static const u32 page_vals_e2\[\] = \{ 0,  128\};"

puts	$c_file "static const u32 page_write_regs_e2\[\] = \{328476\};"

puts	$c_file "static const struct reg_addr page_read_regs_e2\[\] = \{"
puts	$c_file "\t\{0x58000, 4608, DUMP_CHIP_E2, 0x30\}"
puts	$c_file "\};\n"


puts	$c_file "\#define PAGE_MODE_VALUES_E3 2"

puts	$c_file "\#define PAGE_READ_REGS_E3 1"

puts	$c_file "\#define PAGE_WRITE_REGS_E3 1"

puts	$c_file "static const u32 page_vals_e3\[\] = \{ 0,  128\};"

puts	$c_file "static const u32 page_write_regs_e3\[\] = \{328476\};"

puts	$c_file "static const struct reg_addr page_read_regs_e3\[\] = \{"
puts	$c_file "\t\{0x58000, 4608, DUMP_CHIP_E3A0 | DUMP_CHIP_E3B0, 0x30\}"
puts	$c_file "\};\n"
	
	puts "Checkpoint 5"
	for {set i 0} {$i < $num_chips} {incr i} {
		for {set j 0} {$j < $num_presets} {incr j} {
			set num_regs_[set i]_[set j] 0
		}
	}
	
	puts "Checkpoint 6"
	
	set properties [list curr_chip_index curr_preset]
	puts "DEBUG: a"
	set l [splitRegList $register_information(list,reg) $properties]
	puts "DEBUG: b"
	puts $c_file "static const struct reg_addr reg_addrs\[\] = \{"
	set tcl_regs_list ""
	puts "DEBUG: c"
	set lenl [llength $l]
	puts "DEBUG: d"
	for {set i 0} {$i < $lenl} {incr i} {
		set addition ","
		if {$i == [expr {$lenl - 1}]} {
			set addition ""
		}
		set curr_record [lindex $l $i]
		set write_addr [format "0x%x" [lindex $curr_record 0]]
		set write_presets [format "0x%x" [lindex [lindex $curr_record 2] 1]]
		set write_chips [format "0x%x" [lindex [lindex $curr_record 2] 0]]
		puts $c_file "\t\{ $write_addr, [lindex $curr_record 1], $write_chips, $write_presets\}[set addition]"
		lappend tcl_regs_list "[list $write_addr [lindex $curr_record 1] $write_chips $write_presets]"
		for {set t 0} {$t < $num_chips} {incr t} {
			set curr_chip [expr {1 << $t}]
			if {[expr {$curr_chip & $write_chips}] == 0} {
				continue
			}
			for {set j 0} {$j < $num_presets} {incr j} {
				set curr_preset [expr {1 << $j}]
				if {[expr {$curr_preset & $write_presets}] == 0} {
				continue
				}
				puts "num_regs_[set t]_[set j] before is [set num_regs_[set t]_[set j]]"
				incr num_regs_[set t]_[set j] [expr [lindex $curr_record 1]]
				puts "num_regs_[set t]_[set j] after reg_addrs is [set num_regs_[set t]_[set j]]"
				
			}
		}
	}
	puts $c_file "\};\n"
	puts $tcl_file "set ri_regs \[list $tcl_regs_list]"
	
	puts $c_file "\#define REGS_COUNT ARRAY_SIZE(reg_addrs)\n"
	puts "DEBUG: e"
	set l [splitRegList $register_information(list,idle) $properties]
	puts "DEBUG: f"
	puts $c_file "static const struct reg_addr idle_reg_addrs\[\] = \{"
	set tcl_regs_list ""
	puts "DEBUG: g"
	set lenl [llength $l]
	puts "DEBUG: h"
	for {set i 0} {$i < $lenl} {incr i} {
		set addition ","
		if {$i == [expr {$lenl - 1}]} {
			set addition ""
		}
		set curr_record [lindex $l $i]
		set write_addr [format "0x%x" [lindex $curr_record 0]]
		set write_presets [format "0x%x" [lindex [lindex $curr_record 2] 1]]
		set write_chips [format "0x%x" [lindex [lindex $curr_record 2] 0]]
		puts $c_file "\t\{ $write_addr, [lindex $curr_record 1], $write_chips, $write_presets\}[set addition]"
		lappend tcl_regs_list "[list $write_addr [lindex $curr_record 1] $write_chips $write_presets]"
		
		for {set t 0} {$t < $num_chips} {incr t} {
			set curr_chip [expr {1 << $t}]
			if {[expr {$curr_chip & $write_chips}] == 0} {
				continue
			}
			for {set j 0} {$j < $num_presets} {incr j} {
				set curr_preset [expr {1 << $j}]
				if {[expr {$curr_preset & $write_presets}] == 0} {
				continue
				}
				puts "num_regs_[set t]_[set j] before idle_reg_addrs is [set num_regs_[set t]_[set j]]"
				incr num_regs_[set t]_[set j] [expr [lindex $curr_record 1]]
				puts "num_regs_[set t]_[set j] after idle_reg_addrs is [set num_regs_[set t]_[set j]]"
				
			}
		}
	}
	puts $c_file "\};\n"
	puts $tcl_file "set ri_idle_regs \[list $tcl_regs_list]"
	puts $c_file "\#define IDLE_REGS_COUNT ARRAY_SIZE(idle_reg_addrs)\n"
	close $tcl_file
	set checked_chips_list {E1 E1H E2 E3 E3B0}
	for {set k 0} {$k < [llength $checked_chips_list]} {incr k} {
		set chip [lindex $checked_chips_list $k]
		variable out_cam_list
		set out_cam_list {}
		GRC_dump::arrangeCamList $register_information(list,[set chip],cam,0) outCamList
		set outCamList [lsort -unique $outCamList]
		set curr_rec [lindex $outCamList 0]
		set reg_read_list [lindex $curr_rec 3]
		set reg_read_list_length [llength $reg_read_list]
		set addition "static const u32 read_reg_[string tolower $chip] \[\]= \{\n\t"
		set is_first true
		foreach creg $reg_read_list {
			if {$is_first} {
				set is_first false
				append addition ""
			} else {
				append addition ", "
			}
			append addition [format "0x%x" $creg]
		}
		
		append addition "\};"
		puts $c_file $addition
		puts $c_file ""
		set write_addr [format "0x%x" [lindex $curr_rec 0]]
		set chosen_chip [format "0x%x" [set register_information(total,curr_chip_index)]]
		set chosen_preset [format "0x%x" [set register_information(total,curr_preset)]]
		
		puts $c_file "static const struct wreg_addr wreg_addr_[string tolower $chip] = \{\n\t$write_addr, [lindex $curr_rec 1], [llength $reg_read_list], read_reg_[string tolower $chip], $chosen_chip, $chosen_preset\};\n"
		for {set j 0} {$j < $num_presets} {incr j} {
			puts "num_regs_[set k]_[set j] before cam regs is [set num_regs_[set k]_[set j]]"
			incr num_regs_[set k]_[set j] [expr {[lindex $curr_rec 1] * (1 + $reg_read_list_length)}]
			puts "num_regs_[set k]_[set j] after cam regs is [set num_regs_[set k]_[set j]]"
			if {$k > 1} {
				if {([expr {1 << $j}] == [expr {0x10}]) || ([expr {1 << $j}] == [expr {0x20}])} {
					puts "num_regs_[set k]_[set j] before page regs is [set num_regs_[set k]_[set j]]"
					incr num_regs_[set k]_[set j] [expr {2 * 4608}]
					puts "num_regs_[set k]_[set j] after page regs is [set num_regs_[set k]_[set j]]"
					
				}
			}
		}
		
	}
	
	puts $c_file "static const unsigned int dump_num_registers\[NUM_CHIPS\]\[NUM_PRESETS\] = \{"
	
	for {set i 0} {$i < $num_chips} {incr i} {
		set result ""
		append result "\t\{"
		set append_c ""
		if {$i > 0} {
			set append_c ", "
		}
		set append_l ""
		for {set j 0} {$j < $num_presets} {incr j} {
			if {$j > 0} {
				if {$j == 10} {
				set append_l ",\n\t\t"
				} else {
					set append_l ", "
				}
			}
			append result "[set append_l][set num_regs_[set i]_[set j]]"
		}
		append result "\}"
		puts $c_file "[set append_c][set result]"
	}
	puts $c_file "\};"
			
			
	
	
	puts $c_file "\#endif"
	close $c_file

}



proc GRC_dump::linuxAddrDump {c_file_name tcl_file_name t_stamp} {
    variable const_ri_e1
    variable const_ri_e1h
    variable const_ri_e2
    variable const_ri_e3
	variable const_ri_e3b0
	variable addr_dump_properties
	
	set chip_index 0
	variable current_chosen_chip_value
	set current_chosen_chip_value $const_ri_e1
	runPresets e1 false $chip_index
	incr chip_index
	set current_chosen_chip_value $const_ri_e1h
	set addr_dump_properties(curr_chip_index) 1
	runPresets e1h false $chip_index
	incr chip_index
	set current_chosen_chip_value $const_ri_e2
	set addr_dump_properties(curr_chip_index) 2
	runPresets e2 false $chip_index
	incr chip_index
	set current_chosen_chip_value $const_ri_e3
	set addr_dump_properties(curr_chip_index) 3
	runPresets e3 false $chip_index
	incr chip_index
	set current_chosen_chip_value $const_ri_e3b0
	set addr_dump_properties(curr_chip_index) 4
	runPresets e3 true $chip_index
	
	generateLinuxFiles $c_file_name $tcl_file_name $t_stamp
}

proc	GRC_dump::getPropertyValue {address property_list} {
	variable register_information
	set result {}
	foreach property $property_list {
		lappend result $register_information([expr $address],[set property])
	}
	return $result
}

proc GRC_dump::comparePropertiesList {l1 l2} {
	if {[llength $l1] != [llength $l2]} {
		error "Cannot compare lists of different sizes"
	}

	set size [llength $l1]
	for {set i 0} {$i < $size} {incr i} {
		if {[lindex $l1 $i] != [lindex $l2 $i]} {
			return false
		}
	}
	
	return true
}

proc	GRC_dump::splitRegList {reg_list property_list} {
	set reg_list [lsort -unique -integer $reg_list]
	set result {}
	set curr_property_values {}
	foreach prop $property_list {
		lappend curr_property_values 0
	}
	
	set curr_start 0
	set curr_size 0
	set next_exp 0
	set len_reg_list [llength $reg_list]
	for {set i 0} {$i < $len_reg_list} {incr i} {
		set addr [lindex $reg_list $i]
		if {$curr_size == 0} {
			set curr_start $addr
			set curr_size 1
			set next_exp [expr {$addr + 4}]
		
			set curr_property_values [getPropertyValue $curr_start $property_list]
		} else {
			if {$next_exp == $addr} {
				if {[comparePropertiesList $curr_property_values [getPropertyValue $addr $property_list]]} {
					incr curr_size
					incr next_exp 4
					continue
				}
			}
		
			lappend result [list $curr_start $curr_size $curr_property_values]
			set curr_start $addr
			set curr_size 1
			set next_exp [expr {$addr + 4}]
			set curr_property_values [getPropertyValue $addr $property_list]
		}
	}
	
	if {$curr_size > 0} {
		lappend result [list $curr_start $curr_size $curr_property_values]
	}
}

proc	GRC_dump::emulateGrcDump {grc_arg_line} {
variable addr_dump_properties
variable unique_sort_counter
set unique_sort_counter 0
# Step 1: Overload functions
        
		proc add_idle_chk {} {
                #get the idle_chk regs into a new list
                markCamArea idle
                
                idle_chk -dumpMode -out %idle_chk.tmp
                
                markCamArea end
        }
		
        proc UpdateMCPConstants {dump_chip path_num} {
            reg_read -n MISC_REG_SHARED_MEM_ADDR
            if {$dump_chip >= 2} {
                reg_read -n MISC_REG_GENERIC_CR_0
            }
        }
		
		proc ::get_rdf_file {} {
							
				global rdfFile
				variable GRC_dump::rdf_file_to_use
				set rdfFile $GRC_dump::rdf_file_to_use
				
				
			}
		proc markCamArea {state} {
            variable currList
            variable current_state
            variable current_page_mode_state
			variable auto_split
            
            if {[string equal $state start]} {
                set currList cam_regs_list
                set current_state cam
            } elseif {[string equal $state end]} {
                set currList regs_list
                set current_state reg
                set current_page_mode_state false
            } elseif {[string equal $state idle]} {
                set currList  idle_chk_list
                set current_state idle
            } elseif {[string equal $state page_read]} {
                set current_page_mode_state read
            } elseif {[string equal $state page_write]} {
                set current_page_mode_state write
            } elseif {[string equal $state start_split]} {
				set auto_split 1
			} elseif {[string equal $state stop_split]} {
				set auto_split 0
			} else {
                error "Unknown state - $state"
            }
        }
		
		# As explained above the procedure imitate reg command
        # it save all the registers' adresses to the variable defined in currList
        # When not in cam list, it is the regs_list, otherwise it is the cam_regs_list
        # It the register exist in neededRegs, it will return the value defined there,
        # otherwise it will be 0
        proc ::reg {command addr {data {}}} {
            global GRC_dump::neededRegs
            global GRC_dump::dumpChip
			upvar #0 GRC_dump::addr_dump_properties	dump_properties
			upvar #0 GRC_dump::current_chip_revision current_chip_rev
            set lower_case_chip [string tolower $GRC_dump::dumpChip]
            upvar #0 GRC_dump::current_page_mode_state page_state
            upvar #0 GRC_dump::pagemode page_mode
            upvar #0 GRC_dump::register_information reg_info
            upvar #0 GRC_dump::waitPstatus wp_status
            upvar #0 GRC_dump::crash_text ctext
            upvar #0 GRC_dump::current_state cstate
            upvar #0 GRC_dump::no_crash_text noctext
			upvar #0 GRC_dump::auto_split is_split
			upvar #0 GRC_dump::current_chosen_chip_value current_chip_value
			upvar #0 GRC_dump::unique_sort_counter unique_counter
            if {[string equal $page_state write]} {
                if {[string equal $command write] || [string equal $command dwrite]} {
                    lappend page_mode([set lower_case_chip],write) [expr $addr&0x3fffff]
                    lappend page_mode([set lower_case_chip],values) [expr $data]
                    return 0
                }
            }
            #puts "$command $addr"
            if {[string equal $command read] || [string equal $command dread]} {
				
				incr unique_counter
                #add address which require answers
                if {[string equal $page_state read]} {
                    lappend page_mode([set lower_case_chip],read) [expr $addr&0x3fffff]
                    return 0
                }
                set temp_addr [expr $addr&0x3fffff]

                set reg_info([set temp_addr]) 1
				if {[info exists reg_info([set temp_addr],used_chips)]} {
					set old_val_used_chips $reg_info([set temp_addr],used_chips)
				} else {
					set old_val_used_chips 0
				}
				lappend reg_info(list,[set cstate]) $temp_addr
				if {$unique_counter > 600} {
					set reg_info(list,[set cstate]) [lsort -unique -integer $reg_info(list,[set cstate])]
					set unique_counter 0
				}
				set new_val_used_chips [expr {$current_chip_value | $old_val_used_chips}]
				set reg_info([set temp_addr],used_chips) $new_val_used_chips
				
				foreach name [array names dump_properties] {
					set old_value_for_name 0
					set old_total_value 0
					if {[info exists reg_info([set temp_addr],[set name])]} {
						set old_value_for_name [set reg_info([set temp_addr],[set name])]
					}
					if {[info exists reg_info(total,[set name])]} {
						set old_total_value [set reg_info(total,[set name])]
					}
					
					set new_addition_for_name [expr {1 << $dump_properties($name)}]
					set reg_info([set temp_addr],[set name]) [expr {$old_value_for_name | $new_addition_for_name}]
					set reg_info(total,[set name]) [expr {$old_total_value | $new_addition_for_name}]
				}
				if {![info exists reg_info(list,[set GRC_dump::dumpChip][set current_chip_rev],[set cstate],[set dump_properties(curr_preset)])]} {
					set reg_info(list,[set GRC_dump::dumpChip][set current_chip_rev],[set cstate],[set dump_properties(curr_preset)]) {}
				}
				lappend reg_info(list,[set GRC_dump::dumpChip][set current_chip_rev],[set cstate],[set dump_properties(curr_preset)]) $temp_addr
				
				set reg_info([set temp_addr],[set GRC_dump::dumpChip][set current_chip_rev],[set cstate],) 1
                set chosen_device [expr {$addr >> 22}]
                #if it's an even numbered device, mark it as split 4 (we can't know if it's 4 or 8)
                #if it's an odd numbered device, mark it as split 8
                #this is only when device number is greater than 0
				
                if {![info exists reg_info([set temp_addr],[string toupper [set GRC_dump::dumpChip][set current_chip_rev]],last_dev)]} {
					 set reg_info([set temp_addr],[string toupper [set GRC_dump::dumpChip][set current_chip_rev]],last_dev) $chosen_device
				} else {
					if {$reg_info([set temp_addr],[string toupper [set GRC_dump::dumpChip][set current_chip_rev]],last_dev) != $chosen_device} {
						set reg_info([set temp_addr],[string toupper [set GRC_dump::dumpChip][set current_chip_rev]],split) 1
					}
				}

                set reg_info([set temp_addr],[set GRC_dump::dumpChip][set current_chip_rev]) 1
                # save read into appropriate list
                
                if {$wp_status} {
                    set reg_info([set temp_addr],[set ctext]) 1
                } else {
                    set reg_info([set temp_addr],[set noctext]) 1
                }
            
                set addr [expr $temp_addr*1]
                if {[info exists neededRegs($addr)]} {
                    return $neededRegs($addr)
                }
                if {[info exists neededRegs($dumpChip,$addr)]} {
                    return $neededRegs([string toupper $dumpChip],$addr)
                }
                    
                return 0
            }
        }
		
		puts "DEBUG:DEBUG[set grc_arg_line]DEBUG[reg read 0xa408]"
		 set tmp_error [catch {eval grcDump %grcTemp.txt $grc_arg_line} err]
        if {$tmp_error} {
            variable ::errorInfo
            puts "calling stack:\n [set ::errorInfo]\nend of calling stack"
            error $err
        }

	return $tmp_error
}

# The following section are procedures added to give GRC_dump the ability to dump all address space it reads
# to be used by the driver. The driver before reaching a BSOD, will read all those address to a buffer in the host memory.
# Later on, using the windows debugger the data can be outputted to a binary file.
# The script also holds the ability to read the binary file and change reg command so it will imitate the chip.

# The function change the reg commnad to only save the addresses read from.
# Then it activate grcDump, and from that the address apaces to read are created.
# Since grcDump is affected by some registers in the chip such as chip_ver and mcp signature, the reg command
# take this registers from the neededRegs, if a registers doesn't exist there, then it will return 0.
# The function recieved 2 paramaters:
# chip name - affect the sizes of dumped regions
# args - arguments which are pass to the grcDump command, allow the user to choose what will be dumped
proc GRC_dump::addrDump {chip args} {
    variable register_information
	variable current_chip_revision
    variable create_c_file
    variable current_state
    variable timer_info
    set current_state reg
    variable waitPstatus
	variable extension_data
    set waitPstatus false
	variable neededRegs
	variable chip_rev_reg
    set remark_line "# Original Args were: $args"
	
	proc restoreAllBlocksMasking {} {
		puts "Skipping."
	}
	
	proc maskAllBlocks {} {
		puts "Skipping."
	}
	
	
    if {[string equal $chip all]} {
		array unset extension_data
		array set extension_data {}
        array set timer_info {}
        array set register_information {}
        set create_c_file false
        #define_assert E1
        eval addrDump e1 $args
        #define_assert E1H
        eval addrDump e1h $args
        #define_assert E2
        eval addrDump e2 $args
        #define_assert E3
		
        eval addrDump e3 $args
		
		#setting for E3B0 and running
		set current_chip_revision B0
		set neededRegs(E3,[expr {$chip_rev_reg}]) 1
		parse_rdf -f everst3b.rdf -clr
        eval addrDump e3 $args
        set create_c_file true
    }
    variable regs_list
    variable cam_regs_list
    variable idle_chk_list
    variable dumpChip
    variable currList regs_list
    variable chipsNames
    variable GRC_dump_ver
    variable const_ri_e1
    variable const_ri_e1h
    variable const_ri_e2
    variable const_ri_e3
	variable const_ri_e3b0
    variable const_ri_nocrash
    variable const_ri_p0_dump
    variable const_ri_p1_dump
    variable pagemode
    
    if {!$create_c_file} {
        set regs_list {}
        set cam_regs_list {}
        set idle_chk_list {}
        if {[lsearch $chipsNames [string tolower $chip]]==-1} {
            error "Chip must be from the following group: $chipsNames."
        }
    
        set chip [string toupper $chip]
        set dumpChip $chip
        catch {rename ::reg regSave}
 
        #let's make add_idle_chk disapper for now and save the registers used
        catch {rename add_idle_chk add_idle_chk_save}
        prepareIdleChk $chip
        proc add_idle_chk {} {
                #get the idle_chk regs into a new list
                markCamArea idle
                
                idle_chk -dumpMode -out %idle_chk.tmp
                
                markCamArea end
        }
        
       
        # since the cam has special reading method, (read a certain register and two const registers for each line),
        # There should be a way to know when in cam area.
        # When not in addrDump or loadRegs, this procedure is an empty procedure
        proc markCamArea {state} {
            variable currList
            variable current_state
            variable current_page_mode_state
			variable auto_split
            
            if {[string equal $state start]} {
                set currList cam_regs_list
                set current_state cam
            } elseif {[string equal $state end]} {
                set currList regs_list
                set current_state reg
                set current_page_mode_state false
            } elseif {[string equal $state idle]} {
                set currList  idle_chk_list
                set current_state idle
            } elseif {[string equal $state page_read]} {
                set current_page_mode_state read
            } elseif {[string equal $state page_write]} {
                set current_page_mode_state write
            } elseif {[string equal $state start_split]} {
				set auto_split 1
			} elseif {[string equal $state stop_split]} {
				set auto_split 0
			} else {
                error "Unknown state - $state"
            }
        }
        
        #We cannot perform the logic of finding the MCP SHMEM Addresses, so we'll replace
        #UpdateMCPConstants with our version, which simply reads 2 registers that may be needed
        
        catch {rename oldUpdateMcp {}}
        
        catch {rename UpdateMCPConstants oldUpdateMcp}
        
        proc UpdateMCPConstants {dump_chip path_num} {
            reg_read -n MISC_REG_SHARED_MEM_ADDR
            if {$dump_chip >= 2} {
                reg_read -n MISC_REG_GENERIC_CR_0
            }
        }
        # As explained above the procedure imitate reg command
        # it save all the registers' adresses to the variable defined in currList
        # When not in cam list, it is the regs_list, otherwise it is the cam_regs_list
        # It the register exist in neededRegs, it will return the value defined there,
        # otherwise it will be 0
        proc ::reg {command addr {data {}}} {
            global GRC_dump::neededRegs
            global GRC_dump::dumpChip
			upvar #0 GRC_dump::current_chip_revision current_chip_rev
            set lower_case_chip [string tolower $GRC_dump::dumpChip]
            upvar #0 GRC_dump::current_page_mode_state page_state
            upvar #0 GRC_dump::pagemode page_mode
            
            upvar #0 GRC_dump::register_information reg_info
            upvar #0 GRC_dump::$GRC_dump::currList regs_list
            upvar #0 GRC_dump::waitPstatus wp_status
            upvar #0 GRC_dump::crash_text ctext
            upvar #0 GRC_dump::current_state cstate
            upvar #0 GRC_dump::no_crash_text noctext
			upvar #0 GRC_dump::auto_split is_split
            if {[string equal $page_state write]} {
                if {[string equal $command write] || [string equal $command dwrite]} {
                    lappend page_mode([set lower_case_chip],write) [expr $addr&0x3fffff]
                    lappend page_mode([set lower_case_chip],values) [expr $data]
                    return 0
                }
            }
            #puts "$command $addr"
            if {[string equal $command read] || [string equal $command dread]} {
                #add address which require answers
                if {[string equal $page_state read]} {
                    lappend page_mode([set lower_case_chip],read) [expr $addr&0x3fffff]
                    return 0
                }
                lappend regs_list [expr $addr&0x3fffff]
                set temp_addr [expr $addr&0x3fffff]
                set reg_info([set temp_addr]) 1

                set chosen_device [expr {$addr >> 22}]
                #if it's an even numbered device, mark it as split 4 (we can't know if it's 4 or 8)
                #if it's an odd numbered device, mark it as split 8
                #this is only when device number is greater than 0
				if {$is_split} {
					set reg_info([set temp_addr],[string toupper [set GRC_dump::dumpChip][set current_chip_rev]],split) 1
				}
                if {![info exists reg_info([set temp_addr],[string toupper [set GRC_dump::dumpChip][set current_chip_rev]],last_dev)]} {
					 set reg_info([set temp_addr],[string toupper [set GRC_dump::dumpChip][set current_chip_rev]],last_dev) $chosen_device
				} else {
					if {$reg_info([set temp_addr],[string toupper [set GRC_dump::dumpChip][set current_chip_rev]],last_dev) != $chosen_device} {
						set reg_info([set temp_addr],[string toupper [set GRC_dump::dumpChip][set current_chip_rev]],split) 1
					}
				}
                if {[expr {$chosen_device & 1 > 0}]} {
                    set reg_info([set temp_addr],split) 8
                } else {
                    if {![info exists reg_info([set temp_addr],split)] && [expr {$chosen_device > 0}]} {
                        set reg_info([set temp_addr],split) 4
                    }
                }
                set reg_info([set temp_addr],[set GRC_dump::dumpChip][set current_chip_rev]) 1
                set reg_info([set temp_addr][set current_chip_rev],[set cstate]) 1
                # save read into appropriate list
                if {![info exists reg_info(all,$cstate)]} {
                    set reg_info(all,$cstate)) {}
                }
                lappend reg_info(all,$cstate) $temp_addr
                if {![info exists reg_info([set GRC_dump::dumpChip][set current_chip_rev],$cstate)]} {
                    set reg_info([set GRC_dump::dumpChip][set current_chip_rev],$cstate)) {}
                }
                lappend reg_info([set GRC_dump::dumpChip][set current_chip_rev],$cstate) [set temp_addr]
                
                if {$wp_status} {
                    set reg_info([set temp_addr],[set ctext]) 1
                } else {
                    set reg_info([set temp_addr],[set noctext]) 1
                }
            
                set addr [expr $temp_addr*1]
                if {[info exists neededRegs($addr)]} {
                    return $neededRegs($addr)
                }
                if {[info exists neededRegs($dumpChip,$addr)]} {
                    return $neededRegs([string toupper $dumpChip],$addr)
                }
                    
                return 0
            }
        }
    
        # activating the grc dump to a temporary file with the given arguments
        set tmp_error [catch {eval grcDump %grcTemp.txt $args} err]
        if {$tmp_error} {
            variable ::errorInfo
            puts "calling stack:\n [set ::errorInfo]\nend of calling stack"
            error $err
        }
		variable dumpChip
		set lower_case_chip [string tolower $dumpChip]
		
		set additionals {""}
		
		#extension data
		if {[string equal $lower_case_chip e2] || [string equal $lower_case_chip e3]} {
			if {[string equal $lower_case_chip e3]} {
				set additionals {"" "b0"}
			}
			foreach ad $additionals {
				set extension_data([set lower_case_chip][set ad],vfc) [addrDumpHelperVfcStruct [set lower_case_chip][set ad] x true]
				append extension_data([set lower_case_chip][set ad],vfc) "\n"
				append extension_data([set lower_case_chip][set ad],vfc) [addrDumpHelperVfcStruct [set lower_case_chip][set ad] t true]
				append extension_data([set lower_case_chip][set ad],vfc) "\n"
				append extension_data([set lower_case_chip][set ad],vfc) [addrDumpHelperVfcReader [set lower_case_chip][set ad] t true]
				append extension_data([set lower_case_chip][set ad],vfc) "\n"
				append extension_data([set lower_case_chip][set ad],vfc) [addrDumpHelperVfcReader [set lower_case_chip][set ad] x true]
				append extension_data([set lower_case_chip][set ad],vfc) "\n"
			
				set extension_data([set lower_case_chip][set ad],igu) [addrDumpHelperIguStruct [set lower_case_chip][set ad] true]
				append extension_data([set lower_case_chip],igu) "\n"
			}
		} else {
			set extension_data([set lower_case_chip],igu) [addrDumpHelperIguStruct $lower_case_chip]
			append extension_data([set lower_case_chip],igu) "\n"
			set extension_data([set lower_case_chip],vfc) [addrDumpHelperVfcStruct $lower_case_chip x]
			append extension_data([set lower_case_chip],vfc) "\n"
			append extension_data([set lower_case_chip],vfc) [addrDumpHelperVfcStruct $lower_case_chip t]
			append extension_data([set lower_case_chip],vfc) "\n"
		}
		
        # return reg command to normal
        rename ::reg {}
        rename regSave ::reg

        # return add_idle_chk
        catch {rename add_idle_chk {}}
        catch {rename add_idle_chk_save add_idle_chk}
        
        # return UpdateMCPConstants
        catch {rename UpdateMCPConstants {}}
        catch {rename oldUpdateMcp UpdateMCPConstants}
        
        # deleting markCamArea
        proc markCamArea {state} {}
        
        if {$tmp_error} {
            variable ::errorInfo
            puts "calling stack:\n [set ::errorInfo]\nend of calling stack"
            error $err
        }
        
        
        
        set timer_info([string tolower $chip][string tolower $current_chip_revision],tcl) "set timer_status_regs \[list [reg_offset -n TM_REG_EN_LINEAR0_TIMER ] [reg_offset -n TM_REG_EN_LINEAR1_TIMER ]\]\n"
        append timer_info([string tolower $chip][string tolower $current_chip_revision],tcl) "set timer_scan_regs \[list [reg_offset -n TM_REG_LIN0_SCAN_ON ] [reg_offset -n TM_REG_LIN1_SCAN_ON ]\]\n"
        set timer_info([string tolower $chip][string tolower $current_chip_revision],c) "\n"
        #set timer_info([string tolower $chip],c) "/* The following structs contain information for the idle_chk script. "
        #append timer_info([string tolower $chip],c) "\nThe status regs will be used to read status at first / write new status."
        #append timer_info([string tolower $chip],c) "\nThe scan regs will be used to read and compare to 0 after disabling (writing status to 0).*/\n"
        set define_string "TIMER_REGS_COUNT_[string toupper $chip][string toupper $current_chip_revision]"
        append timer_info([string tolower $chip][string tolower $current_chip_revision],c) {#define }
        append timer_info([string tolower $chip[string tolower $current_chip_revision]],c) "$define_string 2\n"
        
        append timer_info([string tolower $chip][string tolower $current_chip_revision],c) "\nstatic const u32 timer_status_regs_[string tolower $chip][string tolower $current_chip_revision]\[\] = \{ [format "0x%x" [expr [reg_offset -n TM_REG_EN_LINEAR0_TIMER ]]], [format "0x%x" [expr [reg_offset -n TM_REG_EN_LINEAR1_TIMER ]]]\};"
        append timer_info([string tolower $chip][string tolower $current_chip_revision],c) "\nstatic const u32 timer_scan_regs_[string tolower $chip][string tolower $current_chip_revision]\[\] = \{ [format "0x%x" [expr [reg_offset -n TM_REG_LIN0_SCAN_ON]]], [format "0x%x" [expr [reg_offset -n TM_REG_LIN1_SCAN_ON]]]\};"


            puts "[set chip][string tolower $current_chip_revision] done"
            return
    }
    puts "Preparing lists"
    #preparing and sorting lists of registers for .c and .tcl files
    set tmp_idle_regs [lsort -unique -integer $register_information(all,idle)]
    set tmp_reg_regs [lsort -unique -integer $register_information(all,reg)]
	set all_idle_regs {}
	set all_reg_regs {}
	set addr_dump_chips {e1 e1h e2 e3 e3b0}
	set all_split_regs {}
    foreach ch $addr_dump_chips {
		set [set ch]_idle_regs {}
		set [set ch]_reg_regs {}
		createChipList $tmp_idle_regs [string toupper $ch]  [set ch]_idle_regs register_information
		createChipList $tmp_reg_regs [string toupper $ch] [set ch]_reg_regs register_information
		set [set ch]_reg_split_regs [getSplitRegisterList $tmp_reg_regs register_information [string toupper $ch]]
		foreach curr_reg [set [set ch]_reg_split_regs] {
			lappend all_split_regs $curr_reg
		}
		
		eval lappend all_idle_regs [set [set ch]_idle_regs]
		eval lappend all_reg_regs [set [set ch]_reg_regs]
	}

    #set e1_idle_regs [lsort -unique -integer $register_information(E1,idle)]
    #set e1h_idle_regs [lsort -unique -integer $register_information(E1H,idle)]
    #set e1_reg_regs [lsort -unique -integer $register_information(E1,reg)]
    #set e1h_reg_regs [lsort -unique -integer $register_information(E1H,reg)]
    set e1_wreg_regs $register_information(E1,cam)
    set e1h_wreg_regs $register_information(E1H,cam)
    set e2_wreg_regs $register_information(E2,cam)
    set e3_wreg_regs $register_information(E2,cam)
	set e3b0_wreg_regs $register_information(E2,cam)
    set all_idle_regs [lsort -unique -integer $all_idle_regs]
    set all_reg_regs [lsort -unique -integer $all_reg_regs]
    puts "done."
	variable arranged_lists
    array set arranged_lists {}
    
    puts "dividing lists: crash and no crash (for tcl)"
    foreach ch $addr_dump_chips {
        foreach type {idle reg} {
            set tmp {}
            set curr_list [subst $[set ch]_[set type]_regs]
            crashDivide $curr_list tmp register_information
            arrangeList tmp register_information arranged_lists($ch,$type,nocrash)
        }
    }
    
    foreach ch $addr_dump_chips {
        set tmp {}
        set curr_list [subst $[set ch]_wreg_regs]
        crashDivide $curr_list tmp register_information
        arrangeCamList $tmp arranged_lists($ch,cam,nocrash)
    }
    
    
    
    
    puts "arranging lists"
    # arranging all the addresses into appropriate format
	foreach ch $addr_dump_chips {
		arrangeList [set ch]_idle_regs register_information arranged_lists([set ch],idle)
		arrangeList [set ch]_reg_regs register_information arranged_lists([set ch],reg)
		set [set ch]_arranged_split_regs [SimpleArrangeList [set [set ch]_reg_split_regs]]
		arrangeCamList [set [set ch]_wreg_regs] arranged_lists([set ch],cam)
		set arranged_lists([set ch],cam) [lsort -unique $arranged_lists([set ch],cam)]
	}
    arrangeSplitList all_split_regs register_information register_information(split_regs_arranged)
    arrangeList all_idle_regs register_information arranged_lists(all,idle)
    arrangeList all_reg_regs register_information arranged_lists(all,reg)
    set pagemode(e3,read) [SimpleArrangeList $pagemode(e3,read)]
    set pagemode(e3,write) [lsort -integer -unique $pagemode(e3,write)]
    set pagemode(e3,values) [lsort -integer -unique $pagemode(e3,values)]
	set pagemode(e3b0,read) [set pagemode(e3,read)]
    set pagemode(e3b0,write) [set pagemode(e3,write)]
    set pagemode(e3b0,values) [set pagemode(e3,values)]
    set pagemode(e2,read) [SimpleArrangeList $pagemode(e2,read)]
    set pagemode(e2,write) [lsort -integer -unique $pagemode(e2,write)]
    set pagemode(e2,values) [lsort -integer -unique $pagemode(e2,values)]
    
    puts "done."
    

    # lappend fidle_list [list [format "0x%.8x" $startReg] $size]
    
    set current_list [set arranged_lists(all,reg)]
    set regCount [llength $current_list]
    # formatting the adresses spaces to c syntax
    set reg_array "static const struct reg_addr reg_addrs\[\] = \{"
    set num_in_line 1
    set current_num_in_line $num_in_line
    for {set i 0} {$i < $regCount} {incr i} {
        if {$i > 0} {
            append reg_array ","
            if {$current_num_in_line != $num_in_line} {
                append reg_array "\n\t"
            }
        }
        if {$current_num_in_line == $num_in_line} {
            set current_num_in_line 0
            append reg_array "\n\t"
        } else {
            incr current_num_in_line
        }
        
        set curr_item [lindex $current_list $i]
        set curr_addr [lindex $curr_item 0]
        
        set curr_size [lindex $curr_item 1]
        set current_info [getCRegisterInfo $curr_addr register_information]
        set curr_addr [format "0x%x" $curr_addr]
        append reg_array "\{ $curr_addr, $curr_size, $current_info\}"
    }
    append reg_array "\n\};"
	
	set current_list [set register_information(split_regs_arranged)]
    set regCount [llength $current_list]
	set split_reg_array "static const struct reg_addr split_reg_addrs\[\] = \{"
    set num_in_line 1
    set current_num_in_line $num_in_line
    for {set i 0} {$i < $regCount} {incr i} {
        if {$i > 0} {
            append split_reg_array ","
            if {$current_num_in_line != $num_in_line} {
                append split_reg_array "\n\t"
            }
        }
        if {$current_num_in_line == $num_in_line} {
            set current_num_in_line 0
            append split_reg_array "\n\t"
        } else {
            incr current_num_in_line
        }
        
        set curr_item [lindex $current_list $i]
        set curr_addr [lindex $curr_item 0]
        
        set curr_size [lindex $curr_item 1]
        set current_info [getCRegisterInfo $curr_addr register_information]
        set curr_addr [format "0x%x" $curr_addr]
        append split_reg_array "\{ $curr_addr, $curr_size, $current_info\}"
    }
    append split_reg_array "\n\};"
    set current_list [set arranged_lists(all,idle)]
    set idleCount [llength $current_list]
    # formatting the adresses spaces to c syntax
    set idle_array "static const struct reg_addr idle_addrs\[\] = \{"
    set num_in_line 1
    set current_num_in_line $num_in_line
    for {set i 0} {$i < $idleCount} {incr i} {
        if {$i > 0} {
            append idle_array ","
            if {$current_num_in_line != $num_in_line} {
                append idle_array "\n\t"
            }
        }
        if {$current_num_in_line == $num_in_line} {
            set current_num_in_line 0
            append idle_array "\n\t"
        } else {
            incr current_num_in_line
        }
        
        set curr_item [lindex $current_list $i]
        set curr_addr [lindex $curr_item 0]
        set curr_size [lindex $curr_item 1]
        set current_info [getCRegisterInfo $curr_addr register_information]
        set curr_addr [format "0x%x" $curr_addr]
        append idle_array "\{ $curr_addr, $curr_size, $current_info\}"
    }
    append idle_array "\n\};"
    
    
    variable wreg_array_e1
    variable wreg_array_e1h
    variable wreg_array_e2
    variable wreg_array_e3
	variable wreg_array_e3b0
    variable const_arrays_e1
    variable const_arrays_e1h
    variable const_arrays_e2
    variable const_arrays_e3
	variable const_arrays_e3b0
    variable wregCount_e1
    variable wregCount_e1h
    variable wregCount_e2
    variable wregCount_e3
	variable wregCount_e3b0
    #now, for the cam registers
    foreach ch $addr_dump_chips {
	#puts "DEBUG: chip is $ch"
        set current_list  [set arranged_lists($ch,cam)]
        set wregCount [llength $current_list]
        set wregCount_$ch $wregCount
        set define_string "WREGS_COUNT_[string toupper $ch]"
        set wreg_array_$ch "static const struct wreg_addr wreg_addrs_$ch\[\] = \{"
        set const_arrays_$ch {}
        set const_array_index_$ch 0
        set num_in_line 3
        set current_num_in_line $num_in_line
        set is_first true
        set info [getCamCRegisterInfo $register_information([string toupper $ch],cam) $ch register_information]
        foreach wreg_addr_$ch $current_list {
            if {$is_first} {
            set is_first false
            } else {
                append wreg_addr_$ch ", "
            }
            if {$num_in_line == $current_num_in_line} {
                append wreg_array_$ch "\n\t"
                set current_num_in_line 0
            } else {
                incr current_num_in_line
            }
            set curr_item [set wreg_addr_$ch]
            set curr_addr [lindex $curr_item 0]
            set curr_size [lindex $curr_item 1]
            set curr_const_size [lindex $curr_item 2]
            set curr_const_list [lindex $curr_item 3]
            set current_info $info
            set tmp_addr $curr_addr
            set curr_addr [format "0x%x" $tmp_addr]
            append wreg_array_$ch "\{ $curr_addr, $curr_size, $curr_const_size, read_reg_[set ch]_[set const_array_index_$ch], $current_info\}"
            set tmp_const "\{ "
            for {set i 0} {$i < [llength $curr_const_list]} {incr i} {
                if {$i > 0} {
                    append tmp_const ", "
                }
                append tmp_const [format "0x%x" [lindex $curr_const_list $i]]
            }
            append tmp_const "\};\n"
            append const_arrays_$ch "static const u32 read_reg_[set ch]_[set const_array_index_$ch]\[\] = $tmp_const"
            incr const_array_index_$ch
       }
       append wreg_array_$ch "\n\};\n"
    }    
    
    # writing the data to h file
    set cfile [open "hw_dump.h" w]
    puts $cfile {#ifndef __HW_DUMP_H}
    puts $cfile {#define __HW_DUMP_H}
    
    puts $cfile "\n\n/* e_dump.h: QLogic Everest network driver."
    puts $cfile "*"
    puts $cfile "*Copyright (c) 2011-2014 Broadcom Corporation"
    puts $cfile "*Copyright (c) 2014-2015 QLogic Corporation"
    puts $cfile "*/\n\n\n"
    
	puts $cfile "/*"
	puts $cfile "ARRSIZE:"
	puts $cfile "used to calcualte item count of an array"
	puts $cfile "this macro is used to prevent compile warning for unreferenced parametes"
	puts $cfile "*/"
	puts $cfile {#ifndef ARRSIZE}
	puts $cfile {#define ARRSIZE(a)				(sizeof(a)/sizeof((a)[0]))}
	puts $cfile {#endif // ARRSIZE}
	puts $cfile "\n\n"


    puts $cfile "/*definitions */"
    puts $cfile {#define DRV_DUMP_XSTORM_WAITP_ADDRESS    0x2b8a80}
    puts $cfile {#define DRV_DUMP_TSTORM_WAITP_ADDRESS    0x1b8a80}
    puts $cfile {#define DRV_DUMP_USTORM_WAITP_ADDRESS    0x338a80}
    puts $cfile {#define DRV_DUMP_CSTORM_WAITP_ADDRESS    0x238a80}
    puts $cfile {#define DRV_DUMP_TSTORM_CAM_MODE         0x1B1440}
    puts $cfile {}
    puts $cfile {#define DRV_DUMP_MAX_TIMER_PENDING      200}
    puts $cfile {#define DRV_DUMP_MAX_HEADER             1000}
    puts $cfile {#define DRV_DUMP_TIMER_SCAN_DONT_CARE   0xFF}

	
	
    variable const_ri_e3b0

    set define_string {#define RI_E1			}
    append define_string [format "0x%x" $const_ri_e1]
    puts $cfile $define_string
    set define_string {#define RI_E1H			}
    append define_string [format "0x%x" $const_ri_e1h]
    puts $cfile $define_string
    set define_string {#define RI_E2			}
    append define_string [format "0x%x" $const_ri_e2]
    puts $cfile $define_string
    set define_string {#define RI_E3			}
    append define_string [format "0x%x" $const_ri_e3]
    puts $cfile $define_string
	set define_string {#define RI_E3B0			}
    append define_string [format "0x%x" $const_ri_e3b0]
    puts $cfile $define_string
    set define_string {#define RI_ONLINE 		}
    append define_string [format "0x%x" $const_ri_nocrash]
    puts $cfile $define_string
	set define_string {#define RI_OFFLINE			0x0}
    puts $cfile $define_string
    
    set define_string {#define RI_PATH0_DUMP 		}
    append define_string [format "0x%x" $const_ri_p0_dump]
    puts $cfile $define_string
    set define_string {#define RI_PATH1_DUMP 		}
    append define_string [format "0x%x" $const_ri_p1_dump]
    puts $cfile $define_string
    puts {}
	
	set def_start {#define}
	variable const_xstorm
	variable const_tstorm
	variable const_ustorm
	variable const_cstorm
	variable const_other_block
	variable const_igu
	variable const_vfc
	variable const_split
	variable const_static_debug
	variable const_max_vfc_write_size
	puts $cfile "\n$def_start MAX_VFC_WRITE_SIZE $const_max_vfc_write_size"
	puts $cfile "\n"
	puts $cfile "/* Data Source - For Extension Headers */"
	puts $cfile "$def_start RI_SRC_XSTORM $const_xstorm"
	puts $cfile "$def_start RI_SRC_TSTORM $const_tstorm"
	puts $cfile "$def_start RI_SRC_USTORM $const_ustorm"
	puts $cfile "$def_start RI_SRC_CSTORM $const_cstorm"
	puts $cfile "$def_start RI_OTHER_BLOCK $const_other_block"
	puts $cfile "\n"
	puts $cfile "/* Data Types */"
	puts $cfile "$def_start RI_TYPE_IGU $const_igu"
	puts $cfile "$def_start RI_TYPE_VFC $const_vfc"
	puts $cfile "$def_start RI_TYPE_SPLIT $const_split"
	puts $cfile "\n"
	puts $cfile "/* VFC Status flag */"
	puts $cfile "$def_start RI_VFC_IS_READY 1"
	puts $cfile "$def_start RI_VFC_IS_BUSY 2"
	puts $cfile "$def_start RI_VFC_IS_ONGOING_SEND 4"

	set group_of_chips {}
	foreach ch $addr_dump_chips {
		lappend group_of_chips [string toupper $ch]
	}
	
	set def_word {#define}
	
	foreach onl_offl {ONLINE OFFLINE} {
		foreach chosen_chip_group [getAllSubGroups $group_of_chips] {
			if {[isEmptyGroup $chosen_chip_group]}	{
				continue
			}
			set app_chip_names ""
			set app_chip_consts "("
			foreach ch $chosen_chip_group {
				append app_chip_names $ch
				append app_chip_consts "RI_$ch | "
			}
			append app_chip_consts "RI_[set onl_offl])"
			set def_line "$def_word RI_[set app_chip_names]_[set onl_offl]"
			set len [string length $def_line]
			set curr_tab [expr {($len + 7) / 8}]
			set needed_tabs [expr {6 - $curr_tab}]
			set added_tabs ""
			for {set i 0} {$i < $needed_tabs} {incr i} {
				append added_tabs "\t"
			}
			puts $cfile "[set def_line][set added_tabs]$app_chip_consts"
		}
	}
	
	set names_of_all_groups ""
	foreach ch $addr_dump_chips {
		append names_of_all_groups [string toupper $ch]
	}
	
	foreach onl_offl {ONLINE OFFLINE} {
		puts $cfile "$def_word RI_ALL_[set onl_offl]\tRI_[set names_of_all_groups]_[set onl_offl]"
	}

	puts $cfile ""
	puts $cfile "/* Macros for handling the MCP Tracebuffer */"
	puts $cfile {#define DBG_DMP_TRACE_BUFFER_SIZE 0x800}
	puts $cfile "$def_word DBG_DMP_TRACE_BUFFER_OFFSET(shmem0_offset) \\"
	puts $cfile "\t\t((shmem0_offset) - DBG_DMP_TRACE_BUFFER_SIZE)"
    puts $cfile "\n\n/* This struct holds a signature to ensure the dump returned from the driver"
    puts $cfile "   match the meta data file inserted to grc_dump.tcl"
    puts $cfile "   The signature is time stamp, diag version and grc_dump version */\n\n"
    
    puts $cfile "\n\nstruct hd_param \{"
    puts $cfile "\tu32 time_stamp;"
    puts $cfile "\tu32 diag_ver;"
    puts $cfile "\tu32 grc_dump_ver;"
    puts $cfile "\};\n"

    puts $cfile "struct dump_hdr \{"
    puts $cfile "\tu32  hdr_size;	/* in dwords, excluding this field */"
    puts $cfile "\tstruct hd_param	hd_param_all;"
    puts $cfile "\tu32  x_storm_wait_p_status;"
    puts $cfile "\tu32  t_storm_wait_p_status;"
    puts $cfile "\tu32  u_storm_wait_p_status;"
    puts $cfile "\tu32  c_storm_wait_p_status;"
    puts $cfile "\tu16  info;"
    puts $cfile "\tu8   idle_chk;"
    puts $cfile "\tu8   reserved;"
    puts $cfile "\};\n\n\n"

	puts $cfile {#define HDR_SIGNATURE 0x11ff22ee}
	
	puts $cfile "\n\nstruct extension_hdr \{"
	puts $cfile "\tu32  hdr_signature;"
	puts $cfile "\tu32  hdr_size;	/* in dwords, excluding this field */"
	puts $cfile "\tu32  data_size;	/* in dwords */"
	puts $cfile "\tu32  additional_data;"
	puts $cfile "\tu32  data_source; /* storm number or none if from a block. */"
	puts $cfile "\tu32  data_type;"
	puts $cfile "\tu32  reserved;"
	puts $cfile "\tu32  error;"
	puts $cfile "\};\n"

	puts $cfile "struct igu_data \{"
	puts $cfile "\tu32  valid;	/* is valid for the current chip */"
	puts $cfile "\tu32  max_size;	/* max read iterations */"
	puts $cfile "\tu32  is_last_commands;	/* read this address and put the value in the additional_data field */"
	puts $cfile "\tu32  is_data_valid;	/* non-zero for valid */"
	puts $cfile "\tu32  data\[2\];"
	puts $cfile "\};\n"
	
	puts $cfile "struct vfc_general \{"
	puts $cfile "\tu32  valid;	/* is valid for the current chip */"
	puts $cfile "\tu32  vfc_status;"
	puts $cfile "\tu32  vfc_address;"
	puts $cfile "\tu32  vfc_data_write;"
	puts $cfile "\tu32  vfc_data_read;"
	puts $cfile "\};\n"
	
	puts $cfile "struct vfc_read_write_vector \{"
	puts $cfile "u32 read_size;"
	puts $cfile "u32 address_value;"
	puts $cfile "u32 write_value_num_valid;"
	puts $cfile "u32 write_value\[MAX_VFC_WRITE_SIZE\];"
	puts $cfile "\};\n"

	puts $cfile "struct vfc_read_task \{"
	puts $cfile "struct vfc_read_write_vector *read_write_vectors;"
	puts $cfile "u32 array_size;"
	puts $cfile "\};\n"

    puts $cfile "struct reg_addr \{"
    puts $cfile "\tu32 addr;"
    puts $cfile "\tu32 size;"
    puts $cfile "\tu16 info;"
    puts $cfile "\};\n"

    puts $cfile "struct wreg_addr \{"
    puts $cfile "\tu32 addr;"
    puts $cfile "\tu32 size;"
    puts $cfile "\tu32 read_regs_count;"
    puts $cfile "\tconst u32 *read_regs;"
    puts $cfile "\tu16 info;"
    puts $cfile "\};\n\n\n"


    # adding const data for the header (used to ensure using the right meta data with the binary data)
    # the header included time stamp, diag version and grc_dump version
    regexp {(\d+).(\d+).(\d+)} [version] verA verA verB verC
    set diag_ver [format "0x%x" [expr {($verA<<16)+($verB<<8)+($verC)}]]
    set t_stamp [format 0x%x [clock seconds]]
    regexp {#(\d+)} $GRC_dump_ver grc_dump_ver grc_dump_ver
    set grc_dump_ver [format "0x%x" $grc_dump_ver]

    
    puts $cfile "/* generated file from GRC_dump.tcl\n\tThe grcDump args were $args\n\tThe chip was [string tolower $chip]*/\n\n"
    #puts $cfile {#include "hw_dump_reg_st.h"}
    set  define_string {#define REGS_COUNT}
    append define_string " ARRSIZE(reg_addrs)"
    puts $cfile $reg_array
    puts $cfile "\n"
	puts $cfile $define_string
    puts $cfile "\n\n"
    set  define_string {#define IDLE_REGS_COUNT}
    append define_string " ARRSIZE(idle_addrs)"
	puts $cfile $idle_array
    puts $cfile "\n"
    puts $cfile $define_string
	puts $cfile "\n\n"
	set  define_string {#define SPLIT_REGS_COUNT}
    append define_string " ARRSIZE(idle_reg_addrs)"
    puts $cfile $split_reg_array
	puts $cfile "\n"
    puts $cfile $define_string
	puts $cfile "\n\n"
	

    
	foreach ch $addr_dump_chips {
		set  define_string {#define WREGS_COUNT_}
		append define_string [string toupper $ch]
		append define_string " [set wregCount_[set ch]]"
    puts $cfile $define_string
		puts $cfile "[set const_arrays_[set ch]]\n"
		puts $cfile [set wreg_array_[set ch]]
    puts $cfile "\n"
	}

    puts $cfile "/* The struct hold a signature to ensure the dump returned from the driver\n\tmatch the meta data file inserted to grc_dump.tcl"
    puts $cfile "\tThe signature is time stamp,diag version and grc_dump version*/\n"
    puts $cfile "static const struct hd_param hd_param_[string tolower $chip] = {$t_stamp, $diag_ver, $grc_dump_ver};\n"
    puts $cfile "\n"
	foreach ch $addr_dump_chips {
		puts $cfile $timer_info([set ch],c)
 puts $cfile "\n"
	puts $cfile $extension_data([set ch],vfc)
    puts $cfile "\n"
	puts $cfile $extension_data([set ch],igu)
    puts $cfile "\n"
	}
    
    foreach ch $addr_dump_chips {
        set define_string {#define PAGE_MODE_VALUES_}
        set page_read_regs_define {#define PAGE_READ_REGS_}
        set page_write_regs_define {#define PAGE_WRITE_REGS_}
        set values_count 0
        
        set read_regs [list [list 0 0]]
        set values [list 0]
        set write_regs [list 0]
        set change_count false
        if {[info exists pagemode([set ch],values)]} {
            set values $pagemode([set ch],values)
            set read_regs $pagemode([set ch],read)
            set write_regs $pagemode([set ch],write)
            set values_count [llength $values]
            set change_count true
        }
        
        set read_count 0
        set write_count 0
        set read_array_size 1
        set write_array_size 1
        set values_array_size [llength $values]
        if {$change_count} {
            set read_count [llength $read_regs]
            set read_array_size "PAGE_READ_REGS_[string toupper $ch]"
            set write_count [llength $write_regs]
            set write_array_size "PAGE_WRITE_REGS_[string toupper $ch]"
            set values_array_size "PAGE_MODE_VALUES_[string toupper $ch]"
        }
        
        append define_string "[string toupper $ch] $values_count\n"
        append page_read_regs_define "[string toupper $ch] $read_count"
        append page_write_regs_define "[string toupper $ch] $write_count"
        puts $cfile $define_string
        puts $cfile ""
        puts $cfile $page_read_regs_define
        puts $cfile ""
        puts $cfile $page_write_regs_define
        puts $cfile ""
        
        set curr_str "static const u32 page_vals_$ch\[\] = \{\n"
        set comma_append ""
        foreach val $values {
            append curr_str "[set comma_append][set val]"
            set comma_append ","
        }
        append curr_str "\n\};\n"
        puts $cfile $curr_str
        
        set curr_str "static const u32 page_write_regs_$ch\[\] = \{\n\t"
        set comma_append ""
        foreach wr $write_regs {
            append curr_str "[set comma_append][set wr]"
            set comma_append ","
        }
        append curr_str "\n\};\n"
        puts $cfile $curr_str
        
        set curr_str "static const struct reg_addr page_read_regs_$ch\[\] = \{\n\t"
        set comma_append ""
        foreach rr $read_regs {
            append curr_str "[set comma_append]\{[format "0x%x" [lindex $rr 0]], [lindex $rr 1], RI_[string toupper $ch]_ONLINE\}"
            set comma_append ","
        }
        append curr_str "\n\};\n"
        puts $cfile $curr_str
        
    }
    
    #puts $cfile "\nu32_t timer_scan_regs_count_[string tolower $chip] = 2;"
    puts $cfile "\n"
    puts $cfile {#endif}
    close $cfile

    # saving the same data in the tcl file
    set tclfile [open "regs.tcl" w]
    puts $tclfile $remark_line
	foreach ch $addr_dump_chips {
    puts $tclfile "set reg_addrs_[set ch] \[list $arranged_lists([set ch],reg)\]"
    puts $tclfile "set wreg_addrs_[set ch] \[list $arranged_lists([set ch],cam)\]"
    puts $tclfile "set idlereg_addrs_[set ch] \[list $arranged_lists([set ch],idle)\]"
    puts $tclfile "set reg_addrs_[set ch]_nocrash \[list $arranged_lists([set ch],reg,nocrash)\]"
    puts $tclfile "set wreg_addrs_[set ch]_nocrash \[list $arranged_lists([set ch],cam,nocrash)\]"
    puts $tclfile "set idlereg_addrs_[set ch]_nocrash \[list $arranged_lists([set ch],idle,nocrash)\]"
	puts $tclfile "set splitreg_addrs_[set ch] \[list [set [set ch]_arranged_split_regs]\]"
	puts $tclfile "set splitreg_addrs_[set ch]_nocrash \[list [set [set ch]_arranged_split_regs]\]"
	
	}
    puts $tclfile "array set pagemode \{[array get pagemode]\}"
    puts $tclfile "set time_stamp $t_stamp\n"
    puts $tclfile "set diag_ver $diag_ver\n"
    puts $tclfile "set grc_dump_ver $grc_dump_ver\n"
	foreach ch $addr_dump_chips {
		puts $tclfile $timer_info([set ch],tcl)
	}

    close $tclfile
    unset regs_list
    unset cam_regs_list
    unset idle_chk_list
    unset timer_info
  #  unset register_information
}



proc GRC_dump::addrDumpHelperIguStruct {chip {is_valid false}} {

	set valid_addr 0
	set data_lsb 0
	set data_msb 0
	set is_last 0
	set is_valid_data 0
	set max_size 0
	if {$is_valid} {
		set valid_addr 1
		set max_size 10
		set data_addr [split [reg_offset -n IGU_REG_ERROR_HANDLING_MEMORY] -]
		set data_lsb [lindex $data_addr 0]
		set data_msb [lindex $data_addr 1]
		set is_last [format "0x%x" [reg_offset -n IGU_REG_COMMAND_DEBUG]]
		set is_valid_data [format "0x%x" [reg_offset -n IGU_REG_ERROR_HANDLING_DATA_VALID]]
		set is_valid_data [format "0x%x" [reg_offset -n IGU_REG_ERROR_HANDLING_DATA_VALID]]
	}
	
	set result "static struct igu_data igu_address_[set chip] = \{ "
	append result "[set valid_addr]"
	append result ", [set max_size]"
	append result ", [set is_last]"
	append result ", [set is_valid_data]"
	append result ", \{ [set data_lsb], [set data_msb]\}\};"
	return $result
}

proc GRC_dump::addrDumpHelperVfcStruct {chip storm_lower_letter {is_valid false}} {
	set storm_upper_letter [string toupper $storm_lower_letter]
	
	set vfc_status 0
	set vfc_address 0
	set vfc_data_write 0
	set vfc_data_read 0
	set vfc_valid 0
	if {$is_valid} {
		set vfc_valid 1
		set vfc_status [format "0x%x" [reg_offset -n [set storm_upper_letter]SEM_REG_SEM_FAST_VFC_STATUS ]]
		set vfc_data_write [format "0x%x" [reg_offset -n [set storm_upper_letter]SEM_REG_SEM_FAST_VFC_DATA_WR ]]
		set vfc_data_read [format "0x%x" [reg_offset -n [set storm_upper_letter]SEM_REG_SEM_FAST_VFC_DATA_RD ]]
		set vfc_address [format "0x%x" [reg_offset -n [set storm_upper_letter]SEM_REG_SEM_FAST_VFC_ADDR ]]
	}
	set result "static struct vfc_general vfc_general_[set storm_lower_letter]_[set chip] = \{ "
	append result "[set vfc_valid]"
	append result ", [set vfc_status]"
	append result ", [set vfc_address]"
	append result ", [set vfc_data_write]"
	append result ", [set vfc_data_read]\};"
	
	return $result
}


proc GRC_dump::getSplitRegisterList {all_reg_list register_information curr_ch_rev} {
	upvar #1 $register_information rinfo

	set result {}
	foreach reg_addr $all_reg_list {
		if {[info exists rinfo([set reg_addr],[set curr_ch_rev],split)]} {
			lappend result $reg_addr
		}
	}

	return $result
}

proc GRC_dump::addrDumpHelperBuildRWVector {read_size address_value data_values num_valid_data_values max_data_values} {
	set result "\{ $read_size, $address_value, $num_valid_data_values, \{"
	for {set i 0} {$i < $max_data_values} {incr i} {
		if {$i > 0} {
			append result ","
			}
		if {$i < $num_valid_data_values} {
			append result [lindex $data_values $i]
			} else {
			append result "0"
			}
	}
	append result  "\}"
	
	append result "\}"

}

proc GRC_dump::addrDumpHelperVfcReader {chip storm_lower_letter {is_valid false}} {

		if {![set is_valid]} {
		return ""
	}
	variable const_max_vfc_write_size
	variable vfc_sizes
	
	#first, the RAM
	set result [list]
	for {set i 0} {$i < $vfc_sizes(ram_num_lines)} {incr i} {
		set address_value [VFCBuildAddressRegValue 0 $i 2]
		lappend result [addrDumpHelperBuildRWVector 4 $address_value [list 0] 0 $const_max_vfc_write_size]
	}
	
	#now, the CAM
	for {set i 0} {$i < $vfc_sizes(cam_num_lines)} {incr i} {
		set address_value [VFCBuildAddressRegValue 14 0 4]
		set data_values [list 0 $i]
		lappend result [addrDumpHelperBuildRWVector 6 $address_value $data_values 2 $const_max_vfc_write_size]
	}
	
	#last, the ALU Vectors
	for {set i 0} {$i < $vfc_sizes(alu_num_lines)} {incr i} {
		set address_value [VFCBuildAddressRegValue 6 0 4]
		set data_values  [VFCSingleALUCommandBuilder $i 5]
		lappend result [addrDumpHelperBuildRWVector 10 $address_value $data_values 1 $const_max_vfc_write_size]
	}
	
	set result_text "static struct vfc_read_write_vector vfc_vectors_[set storm_lower_letter]_[set chip]\[[llength $result]\] = \{"
	for {set i 0} {$i < [llength $result]} {incr i} {
		append result_text "\n\t"
		if {$i > 0} {
			append result_text ", "
			}
		append result_text [lindex $result $i]
	}
	append result_text "\};\n\n"
	
	append result_text "static struct vfc_read_task vfc_task_[set storm_lower_letter]_[set chip] = \{"
	append result_text "vfc_vectors_[set storm_lower_letter]_[set chip], [llength $result]\};"
	
	return $result_text
}

# empty procedure invoke when using normal grc dump
proc GRC_dump::markCamArea {state} {
}

# used to read data from the binary file
# recieves 1 argumetns
# in - the file channel to read from
# size - how many registers to read
# return a list of registers in decimal
proc GRC_dump::getNextReg {in {size 1}} {
	variable current_location
	set ans {}
    for {set i 0} {$i<$size} {incr i} {
	incr current_location 4
        set temp [read $in 4]
        if {[eof $in]} {
			set curr_addr [format "0x%x" $current_location]
            error "File size doesn't fit meta data or not aligned to dword ($curr_addr)"
        }
        if {![binary scan $temp i val]} {
            error "Can't translate binary to integer"
        }
        lappend ans $val
    }
    return $ans
}


proc GRC_dump::readExtensionHeader {input_ch} {
	variable extensionHeader
	array unset extensionHeader
	array set extensionHeader {}
	variable current_location
	#puts "DEBUG: Trying to read extension header from $current_location"
	set result 0
	set failure [catch {set result [getNextReg $input_ch]}]
	if {$failure} {
		#puts "DEBUG: false"
		return false
	}
	#puts "DEBUG: Result is $result"
	if {$result != [expr 0x11ff22ee]} {
		return false
	}
	set extensionHeader(hdr_signature) [format "0x%x" $result]
	set extensionHeader(hdr_size) [getNextReg $input_ch]
	#puts "DEBUG: read size is $extensionHeader(hdr_size)"
	#puts "DEBUG: hdr_signature is $extensionHeader(hdr_signature)"
	set extensionHeader(data_size) [getNextReg $input_ch]
	#puts "DEBUG: data_size is $extensionHeader(data_size)"
	set extensionHeader(additional_data) [getNextReg $input_ch]
	#puts "DEBUG: additional_data is $extensionHeader(additional_data)"
	set extensionHeader(data_source) [getNextReg $input_ch]
	#puts "DEBUG: data_source is $extensionHeader(data_source)"
	set extensionHeader(data_type) [getNextReg $input_ch]
	#puts "DEBUG: data_type is $extensionHeader(data_type)"
	set extensionHeader(reserved) [getNextReg $input_ch]
	#puts "DEBUG: reserved is $extensionHeader(reserved)"
	set extensionHeader(err) [getNextReg $input_ch]
	#puts "DEBUG: err is $extensionHeader(err)"
	
	set extensionHeader(data) {}
	for {set i 0} {$i < $extensionHeader(data_size)} {incr i 1} {
		lappend extensionHeader(data) [getNextReg $input_ch]
	}
	#after 1000
	return true
}
proc GRC_dump::dwordToBits {dw} {
	set result ""
	for {set i 0} {$i < 32} {incr i} {
		set current [expr {$dw % 2}]
		set dw [expr {$dw / 2}]
		set result "[set current][set result]"
	}
	return $result
}

proc GRC_dump::convertToLine {index lowReg hiReg} {
	set result [format "%.8x" $index]
	set result "[set result]\t[dwordToBits $hiReg][dwordToBits $lowReg]"
	
	return $result
}
proc GRC_dump::getSeparator {} {
return "***************************************************************"
}

proc GRC_dump::getIguExtensionHeaderString {type data} {
	if {$type} {
		set data_type "IGU_LAST_COMMANDS"
	} else {
		set data_type "IGU_FIRST_ERRORS"
	}
	set i 0
	set result ""
	while {[llength $data]} {
		set first [lindex $data 0]
		set second [lindex $data 1]
		set curr_length [llength $data]
		set data [lrange $data 2 [expr {$curr_length - 1}]]
		
		set current_result "PRINTING MEM:\t[set data_type]_[set i]\n[getSeparator]\n[convertToLine 0 $first $second]\nEND\n\n"
		set result "[set result]\n[set current_result]\n"
		incr i
	}
	
	
	return $result
}

proc GRC_dump::getVFCExtensionHeaderString {storm data} {
	set expected_size [expr {765*4 + 272*6+12*10}]
	if {[llength $data] != $expected_size} {
		puts "Error handling VFC Extension Header for [set storm]Storm - size is [llength $data]"
		return ""
	}
	
	set current_data_index 0
	set current_line_index 0
	#RAM
	#puts "DEBUG: extracting $storm STORM VFC RAM"
	set ram_result "PRINTING [set storm]Storm VFC RAM\n[getSeparator]\n"
	for {set i 0} {$i < 765} {incr i} {
		
		for {set j 0} {$j < 2} {incr j} {
			set first [lindex $data $current_data_index]
			incr current_data_index
			set second [lindex $data $current_data_index]
			incr current_data_index
			set ram_result "[set ram_result][convertToLine $current_line_index $first $second]\n"
			incr current_line_index
		}
	}
	set ram_result "[set ram_result]END\n\n"
	
	#puts "DEBUG: extracting $storm STORM VFC CAM"
	#CAM
	set current_line_index 0
	set cam_result "PRINTING [set storm]Storm VFC CAM\n[getSeparator]\n"
	for {set i 0} {$i < 272} {incr i} {
		
		for {set j 0} {$j < 3} {incr j} {
			set first [lindex $data $current_data_index]
			incr current_data_index
			set second [lindex $data $current_data_index]
			incr current_data_index
			set cam_result "[set cam_result][convertToLine $current_line_index $first $second]\n"
			incr current_line_index
		}
	}
	set cam_result "[set cam_result]END\n\n"
	
	#puts "DEBUG: extracting $storm STORM VFC ALU"
	#ALU Vectors
	set current_line_index 0
	set alu_result "PRINTING [set storm]Storm VFC ALU_Vectors\n[getSeparator]\n"
	for {set i 0} {$i < 12} {incr i} {
		for {set j 0} {$j < 5} {incr j} {
			set first [lindex $data $current_data_index]
			incr current_data_index
			set second [lindex $data $current_data_index]
			incr current_data_index
			set alu_result "[set alu_result][convertToLine $current_line_index $first $second]\n"
			incr current_line_index
		}
	}
	set alu_result "[set alu_result]END\n\n"

	return "[set ram_result][set cam_result][set alu_result]"
	
	
	
}


proc	GRC_dump::readStaticDebugData {chip data} {
	source static_debug_dump.tcl
	set curr_list [set static_debug_meta_[string toupper $chip]]
		
	set data_length [llength $data]
	set result ""
	for {set curr_place 0} {$curr_place < $data_length} {incr curr_place 5} {
		set curr_block_num [lindex $data $curr_place]
		set curr_line_num [lindex $data [expr {$curr_place + 1}]]
		set lo [lindex $data [expr {$curr_place + 2}]]
		set hi [lindex $data [expr {$curr_place + 3}]]
		set meta [lindex $data [expr {$curr_place + 4}]]
		
		set curr_record [lindex $curr_list $curr_block_num]
		set curr_block_name [lindex $curr_record 0]
		set curr_block_lines [lindex $curr_record 6]
		set curr_line_name [lindex $curr_block_lines $curr_line_num]
		
		append result "\nBlock:$curr_block_name\tLine:$curr_line_name\n"
		append result "lo: [format "0x%.8x" [expr $lo]]\n"
		append result "hi: [format "0x%.8x" [expr $hi]]\n"
		append result "meta: [format "0x%.8x" [expr {($meta >> 4) & 1}]]\n"
	}
	
	return "PRINTING MEM:	STATIC_DEBUG\n*************************************************[set result]\nEND\n\n"
	
}


proc GRC_dump::readSplitRegistersExtensionHeader {pf_id data} {
	variable split_regs
	variable split_addrs
	variable pf_list
	#puts "DEBUG: in read split registers"
	set curr_data_pos 0
	lappend pf_list [expr $pf_id]
	foreach split_addr $split_addrs {
        set addr [expr [lindex $split_addr 0]*1]
        set size [lindex $split_addr 1]
        for {set i 0} {$i<$size} {incr i} {
            set split_regs([expr $pf_id],[expr $addr]) [lindex $data $curr_data_pos]
            incr addr 4
			incr curr_data_pos
        }
    }
}

proc GRC_dump::getExtensionHeaderString {} {
	variable extensionHeader
	
	if {$extensionHeader(data_type) == 0} {
		#IGU
		#puts "DEBUG: Adding IGU Extension Header"
		return [getIguExtensionHeaderString $extensionHeader(additional_data) $extensionHeader(data)]
	} else {
		#VFC
		
		if {$extensionHeader(data_type) == 1} {
			#puts "DEBUG: Adding VFC Extension Header"
			set source_storm T
			if {$extensionHeader(data_source) == 0} {
				set source_storm X
			}
			return [getVFCExtensionHeaderString $source_storm $extensionHeader(data)]
		} else {
			variable static_debug_chip_name
			if {$extensionHeader(data_type) == 3} {
				return [readStaticDebugData $static_debug_chip_name $extensionHeader(data)]
			
			} else {
		#split
			#puts "DEBUG: Adding SPLIT Extension Header"
			readSplitRegistersExtensionHeader $extensionHeader(data_source) $extensionHeader(data)
			return ""
			}
		}
	}
}

# The procedure loads the binary file to the memory and changes command needed to used it
# input - the binary file received from the driver
# meta - the tcl file which hold the meta-data
# see more data above
proc GRC_dump::linuxLoadRegs {input meta {force false}} {
    variable cam_mode
    set cam_mode false
    linuxLoadRegsFile $input $meta $force
    linuxDefineLoadProcs
}


# this procedure should be activate when recieving binary data from the driver crush
# the procedure recieves 2 mandatory arguments and 1 optional
# input - the binary file name
# meta - the meta data file name (a tcl file created during addrDump)
# force - whether to ignore the header or not, default is false
# the procedure loads all registers to an array and change the reg command to read from it
# the procedure holds a special index in the array for the cam, which hold the registers address and the last adress read from
proc GRC_dump::linuxLoadRegsFile {input meta {force false}} {
	variable current_location
	set current_location 0
    variable regs
    variable idle_0_regs
    variable idle_1_regs
    variable rest_regs
    variable additional
    variable timer_scan_regs
    variable timer_status_regs
    variable waitP_offset
    variable CHIPS
    variable rdfs_files
    variable chip_num_reg
    variable used_chip
    variable const_ri_p0_dump
    variable const_ri_p1_dump
    variable is_partial_dump
    set is_partial_dump false
    variable is_with_idle
    set is_with_idle true
    variable dump_chip_ver
    variable old_version
    variable pagemode_regs
    array set pagemode_regs {}
    set old_version true
    variable chip_ver
	variable chip_mc_ver
    variable const_ri_e1
    variable const_ri_e1h
    variable const_ri_e2
    variable const_ri_e3
	variable const_ri_e3b0
    variable const_ri_nocrash
    
    variable reg_addrs
    variable wreg_addrs
    variable idlereg_addrs
    variable text_crash
    variable text_chip
    set text_crash {}
    set text_chip {}
    array set additional {}
    array set regs {}
    set input_channel [open $input r]
    fconfigure $input_channel -translation binary
    source $meta
    variable dump_chosen_path
	variable chosen_preset_args
    set dump_chosen_path 0
    set header_size [getNextReg $input_channel]
	if {!$header_size} {
		error "Invalid data in dump file - header size must be non-zero";
	}
    set header_bin [getNextReg $input_channel $header_size]
	set is_with_idle false
    if {!$force} {
        #ensure the meta file is equal for the h file in the driver
        if {[expr [lindex $header_bin 0]-$time_stamp!=0]} {
            close $input_channel
            error "The time stamp in the supplied meta data ($time_stamp) doesn't match the time stamp in the dumped file ([lindex $header_bin 0])\n"
        }
		variable preset_number
		variable use_all false
		set preset_number [expr {[lindex $header_bin 1] - 1}]
		if {$preset_number > 20 } {
			set use_all true
			set preset_number 0
		}
        #set preset_number [expr {round(log([lindex $header_bin 1]/log(2)))}]
		set preset_bin [expr {1 << $preset_number}]
		variable chosen_preset_args
		if {$use_all} {
			set chosen_preset_args "-mcpfull -withiornos"
		} else {
			set chosen_preset_args [getPresetArgs $preset_number]
		}
		variable used_storm_letters
		variable has_used_storm_letters
		set has_used_storm_letters true
		set used_storm_letters {}
		if {$use_all} {
			set used_storm_letters {X T C U}
		} else {
			foreach lett {X T C U} {
				if {[regexp "\{$lett\}" $chosen_preset_args]} {
					lappend used_storm_letters $lett
				}
			}
		}
		puts "Choseen preset is $preset_number : [getPresetArgs $preset_number]"
		set specific_meta_data [lindex $header_bin 2]
		
		
		if {[expr $const_ri_p1_dump & $specific_meta_data] != 0} {
			set dump_chosen_path 1
			puts "Data is from Path 1"
		} else {
			puts "Data is from path 0"
		}
		
            
        proc rdfParse {args} {
			variable rdf_file_to_use
			namespace eval :: {
				variable GRC_dump::default_rdf_dir
				variable GRC_dump::rdf_file_to_use
				parse_rdf -clr -f [set GRC_dump::default_rdf_dir][set GRC_dump::rdf_file_to_use]
			}
		}
			
		proc ::get_rdf_file {} {
							
			global rdfFile
			variable GRC_dump::rdf_file_to_use
			set rdfFile $GRC_dump::rdf_file_to_use
		}
		
		set chosen_chip 0
		variable rdf_file_to_use
		variable static_debug_chip_name
		if {[expr $const_ri_e1h & $specific_meta_data] != 0} {
			set dump_chip_ver e1h
			set linux_chip_ver E1H
			set rdf_file_to_use evrst15.rdf
			set chosen_chip $const_ri_e1h
			rdfParse evrst15.rdf
			set static_debug_chip_name e1h
			set text_chip "_e1h"
		} else {
			if {[expr $const_ri_e2 & $specific_meta_data] != 0} {
				set dump_chip_ver e2
				set linux_chip_ver E2
				set rdf_file_to_use everst2.rdf
				set chosen_chip $const_ri_e2
				rdfParse everst2.rdf
				set static_debug_chip_name e2
				set text_chip "_e2"
			} else {
				if {[expr $const_ri_e3 & $specific_meta_data] != 0} {
					set dump_chip_ver e3
					set linux_chip_ver E3
					set rdf_file_to_use everst3.rdf
					rdfParse everst3.rdf
					set text_chip "_e3"
					set static_debug_chip_name e3
					set chosen_chip $const_ri_e3
				} else {
					if {[expr $const_ri_e3b0 & $specific_meta_data] != 0} {
						set dump_chip_ver e3
						set linux_chip_ver E3B0
						set rdf_file_to_use everst3b.rdf
						set chosen_chip $const_ri_e3b0
						rdfParse everst3b.rdf
						set text_chip "_e3b0"
						set static_debug_chip_name e3b
					} else {
						set dump_chip_ver e1
						set linux_chip_ver E1
						set chosen_chip $const_ri_e1
						set rdf_file_to_use everest.rdf
						rdfParse everest.rdf
						set text_chip "_e1"
						set static_debug_chip_name e1
					}
				}
			}                         
        }

		puts "Data is from [string toupper $dump_chip_ver]"
		puts "Data contains idle checks"
		set is_with_idle true

		variable split_addrs
		variable split_regs
		array set split_regs {}
		variable pf_list
		set pf_list {}		
		set split_addrs {}

		set wreg_addrs [set ri(list,[set linux_chip_ver],cam,[set preset_number])]
		
		variable load_page_values
		variable load_page_reads
		variable load_page_writes
		set load_page_values {}
		set load_page_reads {}
		set load_page_writes {}
    }
	variable use_all
	if {$use_all} {
		set first_preset_number 0
		set last_preset_number 12
	} else {
		variable preset_number
		set first_preset_number $preset_number
		set last_preset_number $preset_number
	}
	for {set curr_pre $first_preset_number} {$curr_pre <= $last_preset_number} {incr curr_pre} {
	if {($last_preset_number > $first_preset_number) && ([regexp withior [getPresetArgs $curr_pre]]) } {
		continue
	}
	set wreg_addrs [set ri(list,[set linux_chip_ver],cam,[set curr_pre])]
	set preset_bin [expr {1 << $curr_pre}]
	
	if {$is_with_idle} {    
        for {set iteration 0} {$iteration < 1} {incr iteration} {
			
            foreach reg_addr $ri_idle_regs {
                set addr [expr [lindex $reg_addr 0]*1]
                set size [lindex $reg_addr 1]
				set chips [lindex $reg_addr 2]
				set preset [lindex $reg_addr 3]
				if {[expr {$preset_bin & $preset}] == 0} {
					continue
				}
				
				if {[expr {$chosen_chip & $chips}] == 0} {
					continue
				}
				
                for {set i 0} {$i<$size} {incr i} {
                    set regs($addr) [getNextReg $input_channel]
					incr addr 4
                }
            }
            array set idle_[set iteration]_regs [array get regs]
            foreach offset [array names additional] {
                set idle_[set iteration]_regs([expr $offset]) [set additional($offset)]				
            }
            unset regs
            array set regs {}
        }
    }
	
	
	puts "Reading Regular Addresses"
	variable reg_regs
	array set reg_regs {}
	array set regs [array get reg_regs]
    foreach reg_addr $ri_regs {
        set addr [expr [lindex $reg_addr 0]*1]
        set size [lindex $reg_addr 1]
		set chips [lindex $reg_addr 2]
				set preset [lindex $reg_addr 3]
				if {[expr {$preset_bin & $preset}] == 0} {
					continue
				}
				
				if {[expr {$chosen_chip & $chips}] == 0} {
					continue
				}
        for {set i 0} {$i<$size} {incr i} {
			if {[expr $addr] == [expr 0xa408]} {
				variable current_location
			}
			
            set regs($addr) [getNextReg $input_channel]
            incr addr 4
        }
    }
	variable current_location
	set current_location_hex [format "0x%x" $current_location]
	puts "Reading CAM Addresses"
		foreach wreg_addr $wreg_addrs {
			set addr [expr [lindex $wreg_addr 0]*1]
			set size [lindex $wreg_addr 1]
			set num_consts [lindex $wreg_addr 2]
			
			set const_regs [lindex $wreg_addr 3]
			for {set i 0} {$i<$size} {incr i} {
				set regs($addr) [getNextReg $input_channel]
				for {set j 0} {$j<$num_consts} {incr j} {
					set regs([expr [lindex $const_regs $j]*1],$addr) [getNextReg $input_channel]
				}
				incr addr 4
			}
		}
		array set reg_regs [array get regs]
		array set pagemode {e3,values {0 128} e3,read {{360448 4608}} e3b0,write 328476 e3b0,values {0 128} e3b0,read {{360448 4608}} e2,write 328476 e3,write 328476 e2,values {0 128} e2,read {{360448 4608}}}
	variable load_page_values
            variable load_page_reads
            variable load_page_writes
            if {($chosen_chip > 2) && ($preset_bin == [expr 0x10] || $preset_bin == [expr 0x20])} {
                set load_page_values $pagemode([set dump_chip_ver],values)
                set load_page_reads $pagemode([set dump_chip_ver],read)
                set load_page_writes $pagemode([set dump_chip_ver],write)
            } else {
                set load_page_values {}
                set load_page_reads {}
                set load_page_writes {}
            }
	foreach value $load_page_values {
        foreach couple $load_page_reads {
            set curr_start [expr {[lindex $couple 0]}]
            set curr_size [lindex $couple 1]
            for {set i 0} {$i < $curr_size} {incr i} {
                set addr [expr {$curr_start + $i * 4}]
                set pagemode_regs([set addr],[set value]) [getNextReg $input_channel]
            }
        }
    }
	}
	
	
	
    #parse the rdf
    
    set read_reg [set regs([expr $chip_num_reg])]
    
    set used_chip [set CHIPS([format 0x%x $read_reg])]
    
        set rdf_file_to_parse [set GRC_dump::rdfs_files([set GRC_dump::used_chip])]
    rdfParse $rdf_file_to_parse
    
    # hard-coded giving the waitp their values before the crash
    foreach stormLetter {X T U C} index {3 4 5 6} {
		set chip_ver $used_chip
		set chip_mc_ver $used_chip
        UpdateFastMemOffsets $stormLetter
        set addr [get_start_offset [reg_offset -n [format "%sSEM_REG_FAST_MEMORY" $stormLetter]]]
        set regs([expr {$addr+$waitP_offset}]) [lindex $header_bin $index]
    }
    array set rest_regs [array get regs]
    
    variable extension_data_string
    set extension_data_string ""
    while {[readExtensionHeader $input_channel]} {
	set extension_data_string "[set extension_data_string][getExtensionHeaderString]"
    }
     
     
    close $input_channel
}



# this procedure should be activate when recieving binary data from the driver crush
# the procedure recieves 2 mandatory arguments and 1 optional
# input - the binary file name
# meta - the meta data file name (a tcl file created during addrDump)
# force - whether to ignore the header or not, default is false
# the procedure loads all registers to an array and change the reg command to read from it
# the procedure holds a special index in the array for the cam, which hold the registers address and the last adress read from
proc GRC_dump::loadRegsFile {input meta {force false}} {
	variable current_location
	set current_location 0
    variable regs
    variable idle_0_regs
    variable idle_1_regs
    variable rest_regs
    variable additional
    variable timer_scan_regs
    variable timer_status_regs
    variable waitP_offset
    variable CHIPS
    variable rdfs_files
    variable chip_num_reg
    variable used_chip
    variable const_ri_p0_dump
    variable const_ri_p1_dump
    variable is_partial_dump
    set is_partial_dump false
    variable is_with_idle
    set is_with_idle true
    variable dump_chip_ver
    variable old_version
    variable pagemode_regs
    array set pagemode_regs {}
    set old_version true
    variable chip_ver
	variable chip_mc_ver
    variable const_ri_e1
    variable const_ri_e1h
    variable const_ri_e2
    variable const_ri_e3
	variable const_ri_e3b0
    variable const_ri_nocrash
    
    variable reg_addrs
    variable wreg_addrs
    variable idlereg_addrs
    variable text_crash
    variable text_chip
    set text_crash {}
    set text_chip {}
    array set additional {}
    array set regs {}
    set input_channel [open $input r]
    fconfigure $input_channel -translation binary
    source $meta
    variable dump_chosen_path
    set dump_chosen_path 0
    set header_size [getNextReg $input_channel]
    set header_bin [getNextReg $input_channel $header_size]
    if {!$force} {
        #ensure the meta file is equal for the h file in the driver
        if {[expr [lindex $header_bin 0]-$time_stamp!=0]} {
            close $input_channel
            error "The meta data supllied doesn't fit the dump time stamp\n"
        }
        if {[expr [lindex $header_bin 1]-$diag_ver!=0]} {
            close $input_channel
            error "The meta data supllied doesn't fit the dump diag verion\n"
        }
        if {[expr [lindex $header_bin 2]-$grc_dump_ver!=0]} {
            close $input_channel
            error "The meta data supllied doesn't fit the dump grc_dump version\n"
        }
        
        #check if new version or old version
        if {$header_size > 7} {
            set old_version false

            if {[expr $const_ri_nocrash & [lindex $header_bin 7]] != 0} {
                set is_partial_dump true
                puts "Data contains partial dump only"
                set text_crash "_nocrash"
            }
            if {[expr $const_ri_p1_dump & [lindex $header_bin 7]] != 0} {
                set dump_chosen_path 1
                puts "Data is from Path 1"
            }
            
            proc rdfParse {args} {
				variable rdf_file_to_use
				namespace eval :: {
					variable GRC_dump::default_rdf_dir
					variable GRC_dump::rdf_file_to_use
					parse_rdf -clr -f [set GRC_dump::default_rdf_dir][set GRC_dump::rdf_file_to_use]
				}
			}
			
			proc ::get_rdf_file {} {
							
				global rdfFile
				variable GRC_dump::rdf_file_to_use
				set rdfFile $GRC_dump::rdf_file_to_use
				
				
			}
            variable rdf_file_to_use
			variable static_debug_chip_name
            if {[expr $const_ri_e1h & [lindex $header_bin 7]] != 0} {
                set dump_chip_ver e1h
				set rdf_file_to_use evrst15.rdf
                rdfParse evrst15.rdf
				set static_debug_chip_name e1h
                set text_chip "_e1h"
            } else {
                if {[expr $const_ri_e2 & [lindex $header_bin 7]] != 0} {
                    set dump_chip_ver e2
					set rdf_file_to_use everst2.rdf
                    rdfParse everst2.rdf
					set static_debug_chip_name e2
                    set text_chip "_e2"
                } else {
                    if {[expr $const_ri_e3 & [lindex $header_bin 7]] != 0} {
                        set dump_chip_ver e3
						set rdf_file_to_use everst3.rdf
                        rdfParse everst3.rdf
                        set text_chip "_e3"
						set static_debug_chip_name e3
                    } else {
						if {[expr $const_ri_e3b0 & [lindex $header_bin 7]] != 0} {
							set dump_chip_ver e3
							set rdf_file_to_use everst3b.rdf
							rdfParse everst3b.rdf
							set text_chip "_e3b0"
							set static_debug_chip_name e3b
						} else {
                    set dump_chip_ver e1
						set rdf_file_to_use everest.rdf
						rdfParse everest.rdf
                    set text_chip "_e1"
					set static_debug_chip_name e1
                }
            }
            }
            }
            
            puts "Data is from [string toupper $dump_chip_ver]"
            if {[expr [lindex $header_bin 7] & 0xff0000] != 0} {
                puts "Data contains idle checks"
                set is_with_idle true
            } else {
                puts "Data doesn't contain idle checks"
                set is_with_idle false
            }
            variable split_addrs
			variable split_regs
			array set split_regs {}
			variable pf_list
			set pf_list {}
			
			set split_addrs [set splitreg_addrs[set text_chip][set text_crash]]
            set reg_addrs [set reg_addrs[set text_chip][set text_crash]]
            set wreg_addrs [set wreg_addrs[set text_chip][set text_crash]]
            set idlereg_addrs [set idlereg_addrs[set text_chip][set text_crash]]
            
            variable load_page_values
            variable load_page_reads
            variable load_page_writes
            if {[info exists pagemode($dump_chip_ver,values)]} {
                set load_page_values $pagemode([set dump_chip_ver],values)
                set load_page_reads $pagemode([set dump_chip_ver],read)
                set load_page_writes $pagemode([set dump_chip_ver],write)
            } else {
                set load_page_values {}
                set load_page_reads {}
                set load_page_writes {}
            }
        }
    }

    if {$is_with_idle} {    
        #read additonal status and scan reg - these will be later added to the idle_check registers
        foreach status_addr $timer_status_regs {
            set additional($status_addr) [getNextReg $input_channel]
        }
        
        foreach status_addr $timer_scan_regs {
            set additional($status_addr) [getNextReg $input_channel]
        }
        
        for {set iteration 0} {$iteration < 2} {incr iteration} {
            foreach reg_addr $idlereg_addrs {
                set addr [expr [lindex $reg_addr 0]*1]
                set size [lindex $reg_addr 1]
                for {set i 0} {$i<$size} {incr i} {
                    set regs($addr) [getNextReg $input_channel]
                incr addr 4
                }
                
            }
            array set idle_[set iteration]_regs [array get regs]
            foreach offset [array names additional] {
                set idle_[set iteration]_regs([expr $offset]) [set additional($offset)]
            }
            unset regs
            array set regs {}
        }
    
    }
	 variable mcp_scratchpad
	for {set i 0} {$i < 0x800} {incr i 4} {
		set mcp_scratchpad([set i]) [getNextReg $input_channel]
	}
	
	proc add_mcp_tracebuffer {path_num} {
	variable MCP_scpd
    variable MCP_regs
    variable outFile
    variable mcp_scratchpad

   set mcp_scpd_base $MCP_scpd(scratchpad_addr)	
   set start_ptr_addr [expr {$MCP_scpd(trcb_start_ptr) - $MCP_scpd(trcb_signature_addr)}]
   #reading the trace buffer only for first path
     set trcb_start [expr {$mcp_scpd_base+$MCP_scpd(trcb_start)}]
	 set local_trcb_start [expr {$MCP_scpd(trcb_start) - $MCP_scpd(trcb_signature_addr)}]
        if {[set mcp_scratchpad(0)]==$MCP_scpd(trcb_signature)} {
            set trcb_start_ptr [set mcp_scratchpad($start_ptr_addr)]
			set local_trcb_start_ptr [expr {$trcb_start_ptr - $MCP_scpd(trcb_signature_addr)}]
            if {$trcb_start_ptr>=$MCP_scpd(trcb_offset_zero) && $trcb_start_ptr<[expr {$MCP_scpd(trcb_offset_zero)+$MCP_scpd(trcb_size)}]} {
                puts "Reading MCP TRACEBUFFER"
                puts $outFile "PRINTING MEM:\tMCP_TRACE_BUFFER"
                puts $outFile "***************************************************************"
                set diff [expr {$trcb_start_ptr-$MCP_scpd(trcb_offset_zero)}]
                set sepReg [set mcp_scratchpad([expr {$local_trcb_start+$diff-$diff%4}])]
                puts -nonewline $outFile [regToStr [expr {$sepReg}] [expr {4-$diff%4}]]
                for {set i [expr {$local_trcb_start+$diff-$diff%4+4}]} {$i<[expr {$local_trcb_start+$MCP_scpd(trcb_size)}]} {incr i 4} {
                    puts -nonewline $outFile [regToStr [set mcp_scratchpad([expr $i])]]
                }
                for {set i [expr {$local_trcb_start}]} {$i<[expr {$local_trcb_start+$diff-$diff%4}]} {incr i 4} {
                    puts -nonewline $outFile [regToStr [set mcp_scratchpad([expr $i])]]
                }
                puts $outFile [regToStr [expr {$sepReg>>((4-$diff%4)*8)}] [expr {$diff%4}]]
                puts $outFile "END\n"  
                #puts $moreFile "more prevent"
        } else {
                puts "Can't read MCP TRACEBUFFER since the pointer is out of bounds."
            }
        } else {
            puts "Can't read MCP TRACEBUFFER since the signature doesn't exist."
        }
	
}
	
	
	
    foreach reg_addr $reg_addrs {
        set addr [expr [lindex $reg_addr 0]*1]
        set size [lindex $reg_addr 1]
        for {set i 0} {$i<$size} {incr i} {
            set regs($addr) [getNextReg $input_channel]
            incr addr 4
        }
    }
	if {!$is_partial_dump} {
		foreach wreg_addr $wreg_addrs {
			set addr [expr [lindex $wreg_addr 0]*1]
			set size [lindex $wreg_addr 1]
			set num_consts [lindex $wreg_addr 2]
			set const_regs [lindex $wreg_addr 3]
			for {set i 0} {$i<$size} {incr i} {
				set regs($addr) [getNextReg $input_channel]
				for {set j 0} {$j<$num_consts} {incr j} {
					set regs([expr [lindex $const_regs $j]*1],$addr) [getNextReg $input_channel]
				}
				incr addr 4
			}
		}
	}
    foreach value $load_page_values {
        foreach couple $load_page_reads {
            set curr_start [expr {[lindex $couple 0]}]
            set curr_size [lindex $couple 1]
            for {set i 0} {$i < $curr_size} {incr i} {
                set addr [expr {$curr_start + $i * 4}]
                set pagemode_regs([set addr],[set value]) [getNextReg $input_channel]
            }
        }
    }
    #parse the rdf
    
    set read_reg [set regs([expr $chip_num_reg])]
    
    set used_chip [set CHIPS([format 0x%x $read_reg])]
    
        set rdf_file_to_parse [set GRC_dump::rdfs_files([set GRC_dump::used_chip])]
    rdfParse $rdf_file_to_parse
    
    # hard-coded giving the waitp their values before the crash
    foreach stormLetter {X T U C} index {3 4 5 6} {
		set chip_ver $used_chip
		set chip_mc_ver $used_chip
        UpdateFastMemOffsets $stormLetter
        set addr [get_start_offset [reg_offset -n [format "%sSEM_REG_FAST_MEMORY" $stormLetter]]]
        set regs([expr {$addr+$waitP_offset}]) [lindex $header_bin $index]
    }
    array set rest_regs [array get regs]
    
    variable extension_data_string
    set extension_data_string ""
    while {[readExtensionHeader $input_channel]} {
	set extension_data_string "[set extension_data_string][getExtensionHeaderString]"
    }
     
     
    close $input_channel
}
#This procedure returns a string of the form a.b.c where a.b.c. is the version
# of the firmware. The value is read from the first address of the XSTorm
# PRAM. The fw version is the 3 LSBs of the read result in reverse order.
# The 4th part is engineering and will only be shown if greater than 0.
proc GRC_dump::getFwVer {} {
	variable fwver
	array set fwver {}
	set fwver_addr [get_start_offset [reg_offset -n XSEM_REG_PRAM]]
	set fwver_value [reg read $fwver_addr]
	for {set i 0} {$i < 4} {set i [expr {$i + 1}]} {
		set fwver{$i} [expr {$fwver_value % 0x100}]
		set fwver_value [expr {$fwver_value / 0x100}]
		}
        if {[set fwver{3}] > 0} {
            return "[set fwver{0}].[set fwver{1}].[set fwver{2}].[set fwver{3}]"
        } else {
            return "[set fwver{0}].[set fwver{1}].[set fwver{2}]"
        }
}
#This procedure outputs the interrupt table
proc GRC_dump::getInterruptTables {} {
    variable withInterruptTable
    variable int_table_regs
    variable outFile
	variable chip_used_num
	
    setFile
    
    if {$withInterruptTable} {
        foreach letter "X T U C" {
            set mem_name "[set letter][set int_table_regs]"
            set declared_name [set mem_name]
            puts $outFile "PRINTING MEM:\t$declared_name"
            puts $outFile "***************************************************************"
            set offsets [split [reg_offset -n $mem_name] "-"]
            if {[llength $offsets]>1} {
                set size [expr ([lindex $offsets 1]-[lindex $offsets 0])*8+32]
            } else {
                set size 32
            }
            read_component BATCH  $mem_name $declared_name $size 
            puts $outFile "END\n"    
        }
    }
    
    close $outFile
}
proc GRC_dump::addExtensionOutput {} {
	variable extension_data_string
	variable outFile
	setFile
	puts $outFile $extension_data_string
	close $outFile
}





# This procedure define markCamArea and reg after finishing loading the bianry file to the memory
# reg is changed to read from the memory and not from the chip
# markCamArea is changed to set a global variable to know when to treat data as cam
proc GRC_dump::defineLoadProcs {} {
    
	variable current_pf_valid
	variable current_pf
	set current_pf_valid false;
    proc markCamArea {state} {
        variable cam_mode
        variable current_page_mode_state
        if {[string equal $state start]} {
            set cam_mode true
            set current_page_mode_state false
        } elseif {[string equal $state end]} {
            set cam_mode false
            set current_page_mode_state false
        } elseif {[string equal $state page_read]} {
            set current_page_mode_state read
        } elseif {[string equal $state page_write]} {
            set current_page_mode_state write
		} elseif {[string equal $state start_split]} {
		} elseif {[string equal $state stop_split]} {
        } else {
            error "Unknown state - $state"
        }
    }
	
    proc restoreAllBlocksMasking {} {
		puts "Skipping."
	}
	
	proc maskAllBlocks {} {
		puts "Skipping."
	}
	proc run_over_devices {func_to_execute {mod_num 1} {msg none}} {
		variable pf_list
		variable current_pf_valid
		variable current_pf
		set current_pf_valid true
		foreach pf $pf_list {
			set current_pf [expr $pf]
			$func_to_execute [expr {$pf + 1}]
		}
		set current_pf_valid false
	}
	
    catch {rename ::reg regSave}
    proc ::reg {command addr {data {}}} {
        global GRC_dump::regs
        global GRC_dump::cam_mode
        global GRC_dump::last_reg
        variable GRC_dump::pagemode_regs
        variable GRC_dump::current_page_mode_state
        variable GRC_dump::current_page_mode_value
		variable GRC_dump::split_regs
		variable GRC_dump::current_pf_valid
		variable GRC_dump::current_pf
        
        if {[string equal $GRC_dump::current_page_mode_state write]} {
            if {[string equal $command write] || [string equal $command dwrite]} {
            set GRC_dump::current_page_mode_value [expr $data]
            return
            }
        }
        if {[string equal $command read] || [string equal $command dread]} {
            set addr [expr $addr&0x3fffff]
            if {[string equal $GRC_dump::current_page_mode_state read]} {
                return $GRC_dump::pagemode_regs([set addr],[set GRC_dump::current_page_mode_value])
            }
                
            if {$cam_mode} {
                if {[info exists regs($addr)]} {
                    set last_reg $addr
                    return [format "0x%x" $regs($addr)]
                } elseif {[info exists regs($addr,$last_reg)]} {
                    return [format "0x%x" $regs($addr,$last_reg)]
                } else {
                    error "Request to read a registers not in registers file, not allowed, recheck address [format "%.8x" $addr] or change input file"
                }
            } else {
				if {$GRC_dump::current_pf_valid} {
					set curr_pf [expr $GRC_dump::current_pf]
					if {[info exists split_regs([set curr_pf],[expr $addr])]} {
						return [format "0x%x" $split_regs([set curr_pf],[expr $addr])]
					}
				}
                if {[info exists regs($addr)]} {
						return [format "0x%x" $regs($addr)]
                } else {
                    error "Request to read a registers not in registers file, not allowed, recheck address [format "%.8x" $addr] or change input file"
                }
            }
        }
    }
    
    catch {rename add_idle_chk old_add_idle_chk}
    variable is_with_idle
    if  {$is_with_idle} {
        proc add_idle_chk {} {
            global GRC_dump::regs
			global GRC_dump::noIdleChk
            variable rest_regs
            variable idle_0_regs
            variable idle_1_regs
            
			if {$GRC_dump::noIdleChk} {
				puts "Skipping Idle Check . . ."
				unset regs
				array set regs [array get rest_regs]	
			}
            unset regs
            array set regs [array get idle_0_regs]
            print_out_idle_chk 1
            unset regs
            array set regs [array get rest_regs]
            
        }
    } else {
        proc add_idle_chk {} {
            global GRC_dump::regs
            variable rest_regs
            variable idle_0_regs
            variable idle_1_regs
            
            puts "Idle Check is not used"
            array unset regs
            array set regs [array get rest_regs]
        }
    }
     
}



	
# This procedure define markCamArea and reg after finishing loading the bianry file to the memory
# reg is changed to read from the memory and not from the chip
# markCamArea is changed to set a global variable to know when to treat data as cam
proc GRC_dump::defineLoadProcs {} {
    
	variable current_pf_valid
	variable current_pf
	set current_pf_valid false;
    proc markCamArea {state} {
        variable cam_mode
        variable current_page_mode_state
        if {[string equal $state start]} {
            set cam_mode true
            set current_page_mode_state false
        } elseif {[string equal $state end]} {
            set cam_mode false
            set current_page_mode_state false
        } elseif {[string equal $state page_read]} {
            set current_page_mode_state read
        } elseif {[string equal $state page_write]} {
            set current_page_mode_state write
		} elseif {[string equal $state start_split]} {
		} elseif {[string equal $state stop_split]} {
        } else {
            error "Unknown state - $state"
        }
    }
	
    proc restoreAllBlocksMasking {} {
		puts "Skipping."
	}
	
	proc maskAllBlocks {} {
		puts "Skipping."
	}
	proc run_over_devices {func_to_execute {mod_num 1} {msg none}} {
		variable pf_list
		variable current_pf_valid
		variable current_pf
		set current_pf_valid true
		foreach pf $pf_list {
			set current_pf [expr $pf]
			$func_to_execute [expr {$pf + 1}]
		}
		set current_pf_valid false
	}
	
    catch {rename ::reg regSave}
    proc ::reg {command addr {data {}}} {
        global GRC_dump::regs
        global GRC_dump::cam_mode
        global GRC_dump::last_reg
        variable GRC_dump::pagemode_regs
        variable GRC_dump::current_page_mode_state
        variable GRC_dump::current_page_mode_value
		variable GRC_dump::split_regs
		variable GRC_dump::current_pf_valid
		variable GRC_dump::current_pf
        
        if {[string equal $GRC_dump::current_page_mode_state write]} {
            if {[string equal $command write] || [string equal $command dwrite]} {
            set GRC_dump::current_page_mode_value [expr $data]
            return
            }
        }
        if {[string equal $command read] || [string equal $command dread]} {
            set addr [expr $addr&0x3fffff]
            if {[string equal $GRC_dump::current_page_mode_state read]} {
                return $GRC_dump::pagemode_regs([set addr],[set GRC_dump::current_page_mode_value])
            }
                
            if {$cam_mode} {
                if {[info exists regs($addr)]} {
                    set last_reg $addr
                    return [format "0x%x" $regs($addr)]
                } elseif {[info exists regs($addr,$last_reg)]} {
                    return [format "0x%x" $regs($addr,$last_reg)]
                } else {
                    error "Request to read a registers not in registers file, not allowed, recheck address [format "%.8x" $addr] or change input file"
                }
            } else {
				if {$GRC_dump::current_pf_valid} {
					set curr_pf [expr $GRC_dump::current_pf]
					if {[info exists split_regs([set curr_pf],[expr $addr])]} {
						return [format "0x%x" $split_regs([set curr_pf],[expr $addr])]
					}
				}
                if {[info exists regs($addr)]} {
						return [format "0x%x" $regs($addr)]
                } else {
                    error "Request to read a registers not in registers file, not allowed, recheck address [format "%.8x" $addr] or change input file"
                }
            }
        }
    }
    
    catch {rename add_idle_chk old_add_idle_chk}
    variable is_with_idle
    if  {$is_with_idle} {
        proc add_idle_chk {} {
            global GRC_dump::regs
			global GRC_dump::noIdleChk
            variable rest_regs
            variable idle_0_regs
            variable idle_1_regs
            
			if {$GRC_dump::noIdleChk} {
				puts "Skipping Idle Check . . ."
				unset regs
				array set regs [array get rest_regs]	
			}
            unset regs
            array set regs [array get idle_0_regs]
            print_out_idle_chk 1
            unset regs
            array set regs [array get idle_1_regs]
            print_out_idle_chk 2
            unset regs
            array set regs [array get rest_regs]
            
        }
    } else {
        proc add_idle_chk {} {
            global GRC_dump::regs
            variable rest_regs
            variable idle_0_regs
            variable idle_1_regs
            
            puts "Idle Check is not used"
            array unset regs
            array set regs [array get rest_regs]
        }
    }
     
}
# The procedure loads the binary file to the memory and changes command needed to used it
# input - the binary file received from the driver
# meta - the tcl file which hold the meta-data
# see more data above
proc GRC_dump::loadRegs {input meta {force false}} {
    variable cam_mode
    set cam_mode false
    loadRegsFile $input $meta $force
    defineLoadProcs
}

# release the diag from imitating the chip and return to normal mode
# includes clean the memory, and restoring reg and markCamArea command
proc GRC_dump::freeRegs {} {
    catch {rename regSave ::reg}
    variable regs
    proc markCamArea {state} {}
    unset regs {}
}

# auxilary procedure use for comparing lists
proc GRC_dump::lequal {listA listB} {
    if {[llength $listA] ==[llength $listB]} {
        foreach e1 $listA e2 $listB {
            if {![string equal $e1 $e2]} {
                return false;
            }
        }
        return true;
    }
    return false;
}

# this subroutine is used to set a new output file 
proc GRC_dump::setFile {{mode a} {path_num 0} {is_first_path true}} {
    variable outFileName
    variable outFile
    variable chip_mc_ver
    variable GRC_dump_ver
    if {![string equal $mode a]} {
        set fwversion [getFwVer]
		set mcpversion [getMcpVer]
        if {$is_first_path} {
            set outFile [open $outFileName w]
        } else {
            set outFile [open $outFileName a]
            puts $outFile "-------------------------- PATH_SEPARATOR ------------------------------"
        }
        puts $outFile "MEMORY DATA from GRC Chip: E$chip_mc_ver Path: $path_num GRC_DUMP version: $GRC_dump_ver FW Version: $fwversion MFW_Version: $mcpversion"
        close $outFile
    } else {
        set outFile [open $outFileName a]
    }
}

proc GRC_dump::addHelpGrcDump {} {
    set grcDumpHelp "grcDump <output file> <args> \n"
    append grcDumpHelp "dump the requested memory data into the output file.\n"
    append grcDumpHelp "the possible arguments are:\n-n number of connection (1 to 256). default 256.\n"
    append grcDumpHelp "-noram or -nopb to skip the ram or the passive buffer.\n"
    append grcDumpHelp "-withior to include the iors (requiring stopping the storms)\n"
    append grcDumpHelp "-withiornos - the same as -withior but returning the storms to running state after sampling.\n"
    append grcDumpHelp "-noqm to skip the qm meories\n"
    append grcDumpHelp "-nobrb to skip the brb meories\n"
    append grcDumpHelp "-nomcp to skip the mcp meories\n"
    append grcDumpHelp "-splitfull to dump split registers.\n"
    append grcDumpHelp "-storms <storm list> - which storms' memory to dump.\n"
    append grcDumpHelp "-mcpfull - to dump all the scarchpad from the mcp.\n"
	append grcDumpHelp "-nomcpfull - to skip dumping all the scarchpad from the mcp.\n"
    append grcDumpHelp "-nofull - NOT to dump all the reigsters in the chip in the end of the dump.\n"
    append grcDumpHelp "-nopciecore - to skip reading the pciecore.\n"
    append grcDumpHelp "-mini - dump only microcode related memories (RAM, PB and contexts).\n"
    append grcDumpHelp "-nohc - to skip HC block information.\n"
    append grcDumpHelp "-noint - to skip Storms' Interrupt Tables.\n"
    append grcDumpHelp "-noidlechk - to skip idle checks.\n"
	append grcDumpHelp "-noparitymasking - to skip parity masking.\n"
    append grcDumpHelp "-dumppath <path num> - to dump data only from the specified path.\n"
    append grcDumpHelp "-noilt - to skip ILT information.\n"
	append grcDumpHelp "-withvfc - to dump also VFC information. (RAM, CAM, ALU Vectors)\n"
	append grcDumpHelp "-withigufifo - to dump also IGU Error Handling Memory.\n"
	append grcDumpHelp "-withbmac - to dump also PFC related registers (BMAC).\n"
	append grcDumpHelp "-crash - recommended after chip crashed.\n"
    append grcDumpHelp "e.g. grcDump ever_grc.txt -n 256 -noram -withior -storms \"X T C\"\n"
    add_help grcDump $grcDumpHelp
}
proc GRC_dump::GetSingleIdleChk {} {
    variable outFileName
    variable filename
    if {[string equal $::tcl_platform(os) DOS]} {
        set filename "idlechk.tmp"
    } else {
        set filename "[set outFileName].idle_chk_temp.txt"    
    }
    
    set tmp_fd [open $filename "w"]
    variable no_asserts
    close $tmp_fd
    idle_chk -out $filename $no_asserts
    set fd [open $filename r]
    set read_res [read $fd]
    close $fd
    return $read_res
}
proc GRC_dump::GetPStat {} {
    variable outFileName
    variable filename
    if {[string equal $::tcl_platform(os) DOS]} {
        set filename "pst.tmp"
    } else {
        set filename "[set outFileName].pst.txt"    
    }
    
    set tmp_fd [open $filename "w"]
    close $tmp_fd
    set err_pstat [catch "pstat $filename true"]
    if {$err_pstat} {
        return "ERROR READING PSTAT"
    }
    set fd [open $filename r]
    set read_res [read $fd]
    close $fd
    return $read_res
}




proc GRC_dump::print_out_idle_chk {iteration} {
    variable outFile
    setFile
    
    puts $outFile "PRINTING IDLE_CHK_[set iteration]"
    puts $outFile "***************************************************************"
    puts $outFile [GetSingleIdleChk]
    puts $outFile "END\n"

    close $outFile
}

proc GRC_dump::add_idle_chk {} {
    variable idleChkDelay
    print_out_idle_chk 1
    after $idleChkDelay
    print_out_idle_chk 2
}

proc GRC_dump::prepareIdleChk {chip } {
    global assertChip
    variable no_asserts
    set assertChip $chip
    namespace eval :: {
        global assertChip
        if {![Exists assert]} {
            set no_asserts "-ignoreAssert"
        }
    }
}

proc GRC_dump::add_pstat {} {
	variable outFile
	setFile
	
    puts $outFile "PRINTING PSTAT"
    puts $outFile "***************************************************************"
    puts $outFile [GetPStat]
    puts $outFile "END\n"

    close $outFile

}

proc GRC_dump::formatList {li {group_size 1} {left_close "\{"} {right_close "\}"} {separator ","} {rec_sep "\n "}} {

    if {[llength $li] == 0} { return "$left_close $right_close" }
    if {$group_size > [llength $li]} {set group_size [llength $li]}
    set result [set left_close]
    set real_length [llength [set li]]
    set num_groups [expr $real_length / $group_size]
    if {$group_size == [llength $li]} {
        set result "$left_close"
        for {set i 0} {$i < $group_size} {incr i} {
            set result "$result [lindex $li $i]"
            if {$i < [expr $group_size - 1]} {
                set result "$result $separator"
            }
        }
        
        return "$result $right_close"
    }
    for {set i 0} {$i < [llength $li]} {set i [expr $i + $group_size]} {
        set base [expr $i]
        set subList [lrange $li $base [expr $base + $group_size - 1]]
        set result "[set result][set left_close][formatList $subList $group_size {} {} "$separator"][set right_close]"
        if {[expr $i + $group_size] < [llength $li]} {
            set result "[set result][set separator][set rec_sep]"
        }
    }
    set result "[set result][set right_close]"
    return $result
}

proc GRC_dump::appendRegs {} {
  
}

proc GRC_dump::compareSplitState {reg1 reg2 register_info} {
	upvar 1 $register_info rinfo
	upvar #0 GRC_dump::all_states state_list
	
	foreach state $state_list {
		set s1 [info exists rinfo($reg1,[string toupper $state],split)]
		set s2 [info exists rinfo($reg2,[string toupper $state],split)]
		if {$s1 != $s2} {
			return false
		}
	}
	return true
}

proc GRC_dump::compareState {reg1 reg2 register_info} {
    upvar #0 GRC_dump::all_states state_list
    upvar 1 $register_info rinfo
    if {![info exists rinfo($reg1)]} {
        return false
    }
    if {![info exists rinfo($reg2)]} {
        return false
    }    
    for {set i 0} {$i < [llength $state_list]} {incr i} {
        set curr_state [lindex $state_list $i]
        set state1 [info exists rinfo($reg1,$curr_state)]
        set state2 [info exists rinfo($reg2,$curr_state)]
        if {[set state1] != [set state2]} {
            return false
        }
    }
    return true
}

proc GRC_dump::createChipList {orig_list chip tgt_list register_information} {
    upvar #1 $tgt_list tg_list
    upvar #1 $register_information rinfo
    set tg_list {}
    foreach regi $orig_list {
        if {[info exists rinfo($regi,[string toupper $chip])]} {
            lappend tg_list $regi
        }
    }
}
proc GRC_dump::getRegisterInfo {reg register_info} {
    upvar 1 $register_info rinfo
    upvar #0 GRC_dump::all_states states
    set result ""
    for {set i 0} {$i < [llength $states]} {incr i} {
        if {[info exists rinfo($reg,[lindex $states $i])]} {
            lappend result [lindex $states $i]
        }
    }
    return "$result"
}

proc GRC_dump::getCRegisterInfo {reg register_info} {
    upvar 1 $register_info rinfo
    variable chip_text
    variable crash_text
    if {[info exists rinfo($reg,E1)] && [info exists rinfo($reg,E1H)] && [info exists rinfo($reg,E2)] && [info exists rinfo($reg,E3)]  && [info exists rinfo($reg,E3B0)]} {
        set chip_text "ALL"
    } else {
		set chip_text ""
		foreach ch {E1 E1H E2 E3 E3B0} {
			if {[info exists rinfo([set reg],[set ch])]} {
				set chip_text "[set chip_text][set ch]"
			}				
		}
    }
    if {[info exists rinfo($reg,nocrash)]} {
        set crash_text "ONLINE"
    } else {
        set crash_text "OFFLINE"
    }
    return "RI_[set chip_text]_[set crash_text]"
}

proc GRC_dump::getCamCRegisterInfo {reglist chipname register_info} {
    upvar 1 $register_info rinfo
    variable chip_text
    set chip_text [string toupper $chipname]
    set crash_only false
    foreach reg $reglist {
        if {![info exists rinfo($reg,nocrash)]} {
            set crash_only true
        }
    }
    variable crash_text
    if {$crash_only} {
        set crash_text "OFFLINE"
    } else {
        set crash_text "ONLINE"
    }
    return "RI_[set chip_text]_[set crash_text]"
}

proc GRC_dump::crashDivide {reglist no_crash_list register_info} {
    upvar 1 $register_info rinfo
    upvar 1 $no_crash_list noclist
    
    set noclist {}
    for {set i 0} {$i < [llength $reglist]} {incr i} {
        set curr_reg [lindex $reglist $i]
        if {[info exists rinfo($curr_reg,nocrash)]} {
            lappend noclist $curr_reg
        }
    }
}

proc GRC_dump::findConstRegList {readList returnedList} {
    upvar 1 $returnedList rlist
    set rlist {}
    if {![llength $readList]} {
        return [set rlist]
    }
    set first_reg [lindex $readList 0]
    set expected_reg [expr {$first_reg + 4}]
    for {set i 1} {$i < [llength $readList]} {incr i} {
        set current_reg [expr [lindex $readList $i]]
        if {$current_reg == $expected_reg} {
            return $rlist
        }
        lappend rlist [expr [set current_reg]]
    }
    return $rlist
}

proc GRC_dump::arrangeCamList {readlist retlist} {
    upvar 1 $retlist rlist
    set rlist {}
    set i 0
    if {![llength $readlist]} {
        return $rlist
    }
    
    variable const_regs
    set curr_count 1
    set first_reg [expr [lindex $readlist $i]]
    findConstRegList $readlist const_regs
    set const_count [llength $const_regs]
    set expected_reg [expr {$first_reg + 4}]
    incr i [expr {$const_count + 1}]
    while {$i < [llength $readlist]} {
        set start_const_index [expr {1 + $i}]
        if {$expected_reg == [lindex $readlist $i] && [compareLists $const_regs [lrange $readlist $start_const_index [expr {$start_const_index + $const_count - 1}]]]} {
            incr expected_reg 4
            incr i [expr {$const_count + 1}]
            incr curr_count
        } else {
            lappend rlist [list $first_reg $curr_count $const_count $const_regs]
            set curr_count 1
            set first_reg [expr [lindex $readlist $i]]
            findConstRegList [lrange $readlist $i [llength $readlist]] const_regs
            set const_count [llength $const_regs]
            set expected_reg [expr {$first_reg + 4}]
            incr i [expr {$const_count + 1}]
        }
    }
    lappend rlist [list $first_reg $curr_count $const_count $const_regs]
    return 
    
    
    
    
}

proc GRC_dump::arrangeSplitList {all_split register_info outl} {
    upvar 1 $all_split split_list
    upvar 1 $outl outlist
    upvar 1 $register_info rinfo
    set outlist {}
    
    set lis [lsort -increasing -integer -unique $split_list]
    if {![llength $lis]} {
        return
    }
    set current_reg_addr [expr [lindex $lis 0]]
    set current_size 1
    set expected_reg_addr [expr {$current_reg_addr + 4}]
    
    #otherwise, let's start arranging
    for {set i 1} {$i < [llength $lis]} {incr i} {
        set next_reg_addr [lindex $lis $i]
        if {[expr [expr {$next_reg_addr}] == $expected_reg_addr] && [compareSplitState $current_reg_addr $next_reg_addr rinfo] && $current_size < 32768} {
                incr current_size 1
                incr expected_reg_addr 4
           } else {
                lappend outlist [list [set current_reg_addr] [set current_size]]
                set current_size 1
                set current_reg_addr [set next_reg_addr]
                set expected_reg_addr [expr {$current_reg_addr + 4}]
        }
    }
    lappend outlist [list [set current_reg_addr] [set current_size]]
}



proc GRC_dump::arrangeList {l register_info  outl} {
    upvar 1 $l templis
    upvar 1 $outl outlist
    upvar 1 $register_info rinfo
    set outlist {}
    
    set lis [lsort -increasing -integer -unique $templis]
    if {![llength $lis]} {
        return
    }
    set current_reg_addr [expr [lindex $lis 0]]
    set current_size 1
    set expected_reg_addr [expr {$current_reg_addr + 4}]
    
    #otherwise, let's start arranging
    for {set i 1} {$i < [llength $lis]} {incr i} {
        set next_reg_addr [lindex $lis $i]
        if {[expr [expr {$next_reg_addr}] == $expected_reg_addr] && [compareState $current_reg_addr $next_reg_addr rinfo] && $current_size < 32768} {
                incr current_size 1
                incr expected_reg_addr 4
            } else {
                lappend outlist [list [set current_reg_addr] [set current_size]]
                set current_size 1
                set current_reg_addr [set next_reg_addr]
                set expected_reg_addr [expr {$current_reg_addr + 4}]
            }
    }
    lappend outlist [list [set current_reg_addr] [set current_size]]
}



proc GRC_dump::compareLists {l1 l2} {
    set len1 [llength $l1]
    set len2 [llength $l2]
    if {$len1 != $len2} {
        return false
    }
    
    for {set i 0} {$i < $len1} {incr i} {
        if {[lindex $l1 $i] != [lindex $l2 $i]} {
            return false
        }
    }
    return true
}

proc GRC_dump::translateLinuxFile {input_file output_file args} {
    set ifile [open $input_file r]
    set ofile [open $output_file w]
    fconfigure $ofile -translation binary    
	set line [gets $ifile]
	while {![regexp {^0+.*:.*} $line]} {
		set line [gets $ifile]
	}
    while {true} {
	
	for {set i 1} {$i < [llength $line]} {incr i} {
            set current [lindex $line $i]
			set current [string trim $current]
            if {![regexp 0x [set current]]} {
                set current_tmp 0x[set current]
                set current [set current_tmp]
            }
            puts -nonewline $ofile [binary format c [expr $current]]
        }
	if {[eof $ifile]} {
		close $ifile
                close $ofile
                puts "Done"
		return
	}
	set line [gets $ifile]
    }
}

proc GRC_dump::VFCReadStatus {storm_letter} {
    #Status is 3 bits: b0 - is response ready, b1 - is busy, b2 - is ongoing send
    return [reg_read -n [set storm_letter]SEM_REG_SEM_FAST_VFC_STATUS]
}

proc GRC_dump::VFCCheckStatus {storm_letter query} {
    variable vfc_status_bits
    set status [VFCReadStatus $storm_letter]
    set bit_num $vfc_status_bits($query)
    set mask [expr {(1 << $bit_num)}]
    if {[expr {(($status & $mask) >> $bit_num)}] == 1} {
        return true
    } else {
        return false
    }
}

proc GRC_dump::VFCCheckAllStatusReset {storm_letter} {
    if {![VFCCheckStatus $storm_letter IS_READY] && ![VFCCheckStatus $storm_letter IS_BUSY] && ![VFCCheckStatus $storm_letter IS_ONGOING_SEND]} {
            return true
    } else {
        return false
    }
}

proc GRC_dump::VFCReadDataReg {storm_letter num_regs {tries 10000}} {
    set reg_name "[set storm_letter]SEM_REG_SEM_FAST_VFC_DATA_RD"
    set result {}
    for {set i 0} {$i < $num_regs} {incr i} {
        #wait for ready
        set counter 0
        while {![VFCCheckStatus $storm_letter IS_READY]} {
            after 1
            incr counter
            if {$counter == $tries} {
                puts "VFC Read FAILED!!!"
                return
            }
        }        
        lappend result [reg_read -n $reg_name]
    }
    return $result
}

proc GRC_dump::VFCReadALUVector {storm_letter vec_num} {
    set to_write [VFCSingleALUCommandBuilder $vec_num 5]
    VFCWriteDataReg $storm_letter $to_write
    VFCWriteAddressReg $storm_letter 6 0 4
    return [VFCReadDataReg $storm_letter 10]
}

proc GRC_dump::VFCSingleALUCommandBuilder {vec_num length_qwords {cycles_gap 0}} {
    #valid bit is b0
    #vec_num should start in b8
    #length should start in b16
    #gap should start in b28
    
    return [expr {(($cycles_gap << 28) | ($length_qwords << 16) | ($vec_num << 8) | 1)}]
}

proc GRC_dump::VFCWriteRamLine {storm_letter ram_line_num list_of_vals} {
    if {[llength $list_of_vals] == 4} {
        VFCWriteDataReg $storm_letter $list_of_vals
        VFCWriteAddressReg $storm_letter 1 $ram_line_num 2
    } else {
        puts "Error: RAM Line in VFC is 128 bit."
    }
}




proc GRC_dump::VFCReadRamLine {storm_letter ram_line_num} {
    #wait until can send command
    while {![VFCCheckAllStatusReset $storm_letter]} {
        after 1
    }
    VFCWriteAddressReg $storm_letter 0 $ram_line_num 2
    return [VFCReadDataReg $storm_letter 4]
}

proc GRC_dump::VFCCAMRead {storm_letter address {gap_cycles 0}} {
    #64 bit command: Address starts at bit 32
    # Gap Cycles start at bit 60
    set second_dword [expr {(($gap_cycles << 28) | $address)}]
    set to_write [list 0 $second_dword]
    
    VFCWriteDataReg $storm_letter $to_write
    VFCWriteAddressReg $storm_letter 14 0 4
    
    return [VFCReadDataReg $storm_letter 6]
}


proc GRC_dump::VFCWriteDataReg {storm_letter list_of_vals} {
    #wait until can send command
    while {![VFCCheckAllStatusReset $storm_letter]} {
        after 1
    }
    set reg_name "[set storm_letter]SEM_REG_SEM_FAST_VFC_DATA_WR"
    set length [llength $list_of_vals]
    
    for {set i 0} {$i < $length} {incr i} {
        reg_write -n $reg_name -v [lindex $list_of_vals $i]
    }
    
}

proc GRC_dump::VFCBuildAddressRegValue {op_code address num_bits_op_code} {
	return [expr {($address << $num_bits_op_code) | $op_code}]
}

proc GRC_dump::VFCWriteAddressReg {storm_letter op_code address num_bits_op_code} {
    set reg_name "[set storm_letter]SEM_REG_SEM_FAST_VFC_ADDR"
    
    set val_to_write [expr {($address << $num_bits_op_code) | $op_code}]
    reg_write -n $reg_name -v $val_to_write
}

proc GRC_dump::VFCIsAlive {storm_letter {num_tries 10000}} {
    set i 0
    while {![VFCCheckAllStatusReset $storm_letter]} {
        after 1
        incr i
        if {$i == $num_tries} {
            return false
        }
    }
    return true
}

proc GRC_dump::RegListToBinary {original_list_of_regs {num_regs_in_line 10}} {
    set curr_pos 0
    set total_length [llength $original_list_of_regs]
    set final_result {}
    while {$curr_pos != $total_length} {
        set list_of_regs [lrange $original_list_of_regs $curr_pos [expr {$curr_pos + $num_regs_in_line - 1}]]
        incr curr_pos [llength $list_of_regs]
    
        set last [expr {[llength $list_of_regs] - 1}]
        set result ""
    
        for {set i $last} {$i > -1} {incr i -1} {
            set current_hex_reg [format "%.8x" [expr [lindex $list_of_regs $i]]]
            set current_hex_string [hex2bin $current_hex_reg]
            set result "[set result][set current_hex_string]"
        }
        lappend final_result $result
    }
    return $final_result
}


proc GRC_dump::NextDeviceNum {curr_num} {
    variable chip_ver
    variable dump_no_pci
    
    set orig_dev $curr_num
    set path_num [GetCurrentPathNum]

    if {[info exists dump_no_pci]} {
        if {$dump_no_pci} {
            return [expr {$curr_num + 1}]
        }
    }
    
    while {1} {
        incr curr_num
        if {[catch {device $curr_num -no_display}]} {
            catch {device $orig_dev -no_display}
            return [expr $curr_num]
        }
        #otherwise, we should check if it's the same path
        set curr_path [GetCurrentPathNum]
        if {$path_num == $curr_path} {
            catch {device $orig_dev -no_display}
            return [expr $curr_num]
        }
        
    }
    
}

proc GRC_dump::GetAbsolutePfNum {} {
    variable chip_ver
    variable ME_register_address
    if {![IsE2]} {
        return [expr {$::current(FUNC)}]
    } else {
        set me_reg [reg read $ME_register_address]
        return [expr {$me_reg >> 16}]
    }
}

proc GRC_dump::IsE2 {} {
    variable chip_ver
    if {[info exists chip_ver]} {
        if {$chip_ver >= 2} {
            return true
        } else {
            return false
        }
    }
    if {[reg read 0xa408] >= 0x1651} {
        return true
    } else {
        return false
    }
}

proc GRC_dump::GetCurrentPathNum {} {
    
    if {![IsE2]} {
        return 0
    } else {
        return [expr {[GetAbsolutePfNum] & 1}]
    }
}

proc GRC_dump::UARTSetPath {new_path_num} {
    set config_reg [reg_offset -n DBU_REG_DBU_VFID_CFG]
    #set config_reg 0x8818
    set config_value [expr {(1 << 31) | ($new_path_num << 20)}]
    reg write $config_reg $config_value
    after 200
}

proc GRC_dump::UARTUnsetPath {} {
    
    set config_reg [reg_offset -n DBU_REG_DBU_VFID_CFG]
    #set config_reg 0x8818
    reg write $config_reg  0
    after 200
}
proc GRC_dump::FindCurrentPathFunc0 {} {
    if {[expr {$::current(FUNC)}]==0} {
        return $::current(DEV)
    }
    # in edebug ::current(FUNC) holds PCI function, so function 1 is first one in path 1
    if {($::sys(IS_EDEBUG) == 1) && ([expr {$::current(FUNC)}]==1)} {
        return $::current(DEV)
    }
    set orig_dev $::current(DEV)
    set path_num [GetCurrentPathNum]
    set next_dev_to_try $orig_dev
    while {[GetAbsolutePfNum] > 0} {
        incr next_dev_to_try -1
        device $next_dev_to_try -no_display
        if {[GetCurrentPathNum] == $path_num} {
            if {[expr {$::current(FUNC)}] == 0} {
                device $orig_dev -no_display
                return $next_dev_to_try
            }
        }
   }
}

proc GRC_dump::IsHiddenDevice {} {
    if {$::current(DID) == 0} {
        return true
    } else {
        return false
    }
}

proc GRC_dump::FindFunctionFromOtherPath {} {
    set orig_device $::current(DEV)
    set f0 [FindCurrentPathFunc0]
    device $f0 -no_display
    set path_num [GetCurrentPathNum]
    
    #try 1 ahead and 1 below
    set f_ahead [expr {$f0 + 1}]
    set f_below [expr {$f0 - 1}]
    
    foreach d [list $f_ahead $f_below] {
        if {![catch {device $d -no_display}]} {
            if {[IsHiddenDevice]} {
                continue
            }
            set curr_path [GetCurrentPathNum]
            if {$curr_path != $path_num} {
                device $orig_device -no_display
                return $d
            }
        }
    }
    return false
}

proc GRC_dump::DoesRegisterExist {reg_name} {
	set flag [catch {set reg_cand [reg_offset -n $reg_name]}]
	if {$flag} { return false }
	if {[string length $reg_cand] == 0} {return false }
	return true
}
 #this subroutine added the data from the PFC
proc GRC_dump::addOuterVlan {dev} {

    variable outFile
	
	if {![DoesRegisterExist NIG_REG_LLH0_FUNC_EN]} { return }

    setFile

	
    variable dump_no_pci
    variable ::current
    set base 0

    if {$dump_no_pci} {
        set base [expr ($dev-1)<<22]
        set func_num [expr {$dev -1 }]
    } else {
        device $dev -no_display
        set func_num [expr {$::current(FUNC)}]
    }

	for {set portn 0} {$portn < 2} {incr portn} {
	    puts $outFile "PRINTING MEM:\tMAC_VLAN_[set func_num]_PORT_[set portn]"
        puts $outFile "***************************************************************"
		puts $outFile "[HexFormat 0]\t[BinFormat [reg_read -n NIG_REG_LLH[set portn]_FUNC_EN]]"
		puts $outFile "[HexFormat 1]\t[BinFormat [reg_read -n NIG_REG_LLH[set portn]_FUNC_VLAN_ID]]"
        puts $outFile "END\n" 
	}
	close $outFile
}


proc GRC_dump::HexFormat {num} {
	return [format "%.8x" $num]
}

proc GRC_dump::BinFormat {num} {
	set formatted [HexFormat $num]
	return [hex2bin $formatted]
}

proc GRC_dump::addLLH {dev} {

	variable outFile
	
	if {![DoesRegisterExist NIG_REG_LLH0_FUNC_MEM_ENABLE]} { return }
	
	    setFile
    
	
    variable dump_no_pci
    variable ::current
    set base 0
	
    if {$dump_no_pci} {
        set base [expr ($dev-1)<<22]
        set func_num [expr {$dev -1 }]
    } else {
		
        device $dev -no_display
		
        set func_num [expr {$::current(FUNC)}]
    }
	
	set count 8
	set num_registers 2
	
	set base0	[get_start_offset [reg_offset -n NIG_REG_LLH0_FUNC_MEM]]
	set base1	[get_start_offset [reg_offset -n NIG_REG_LLH1_FUNC_MEM]]
	set base_en0	[get_start_offset [reg_offset -n NIG_REG_LLH0_FUNC_MEM_ENABLE]]
	set base_en1	[get_start_offset [reg_offset -n NIG_REG_LLH1_FUNC_MEM_ENABLE]]
	
	set add_mem2 [DoesRegisterExist NIG_REG_P0_LLH_FUNC_MEM2_ENABLE]
	
	set current_line 0
	for {set portn 0} {$portn < 2} {incr portn} {
	
	    puts $outFile "PRINTING MEM:\tMAC_MEMORY_[set func_num]_PORT_[set portn]"
        puts $outFile "***************************************************************"
		
		for {set i 0} {$i < $count} {incr i} {
	
			
			puts $outFile "[HexFormat $current_line]\t[BinFormat [reg read [expr {[set base_en[set portn]] + 4 * $i}]]]"
			incr current_line
			for {set j 0} {$j < $num_registers} {incr j} {
					puts $outFile "[HexFormat $current_line]\t[BinFormat [reg read [expr {[set base[set portn]] + 4 * $i * $num_registers + 4 * $j}]]]"
					incr current_line
			}
		}
		if {$add_mem2} {
	
			set base[set portn] [get_start_offset [reg_offset -n NIG_REG_P[set portn]_LLH_FUNC_MEM2]]
			set base_en[set portn]	[get_start_offset [reg_offset -n NIG_REG_P[set portn]_LLH_FUNC_MEM2_ENABLE]]
	
			for {set i 0} {$i < $count} {incr i} {
				
				puts $outFile "[HexFormat $current_line]\t[BinFormat [reg read [expr {[set base_en[set portn]] + 4 * $i}]]]"
				incr current_line
				for {set j 0} {$j < $num_registers} {incr j} {
					puts $outFile "[HexFormat $current_line]\t[BinFormat [reg read [expr {[set base[set portn]] + 4 * $i * $num_registers + 4 * $j}]]]"
					incr current_line
				}
			}
		}
	
        puts $outFile "END\n" 
	}
	
	close $outFile

}


proc GRC_dump::UpdateMCPConstants {dump_chip path_num} {
    variable MCP_scpd

    if {$path_num > 0} {
        set MCP_scpd(shared_mem_size) 14752
    }
    #at this point, scratchpad_addr / scratchpad_size / shared_mem_size / trcb_signature, are already ok
    
    set curr_path_shmem_addr [reg_read -n MISC_REG_SHARED_MEM_ADDR]
    if {$path_num == 0} {
        set path0_shmem_addr $curr_path_shmem_addr
		set shmem2_region_base [reg_read -n MISC_REG_GENERIC_CR_0]
    } else {
        set shmem2_region_base [reg_read -n MISC_REG_GENERIC_CR_1]
        # the field other_shmem_base_addr is offset 13 * uint32 from the beginning of shmem 2
        set path0_shmem_addr [reg read [expr {$shmem2_region_base + $MCP_scpd(shmem_addr_in_shmem2)}]]
    }
    set curr_path_shmem_offset [expr {$curr_path_shmem_addr - $MCP_scpd(scratchpad_addr)}]
    set path0_shmem_offset [expr {$path0_shmem_addr - $MCP_scpd(scratchpad_addr)}]
    
    
    
    #at this point, scratchpad_addr / scratchpad_size / shared_mem_size / trcb_signature, are already ok
    
    
    
    
    set MCP_scpd(trcb_signature_addr) [format "0x%x" [expr {$path0_shmem_offset - 0x800}]]
    set MCP_scpd(shared_mem_offset) [format "0x%x" $curr_path_shmem_offset]
    set MCP_scpd(trcb_start_ptr) [format "0x%x" [expr {$MCP_scpd(trcb_signature_addr) + 4}]]
    set MCP_scpd(trcb_start) [format "0x%x" [expr {$MCP_scpd(trcb_start_ptr) + 4}]]
    set MCP_scpd(trcb_offset_zero) [format "0x%x" [expr {$MCP_scpd(trcb_start) + 0x8000000}]]
    
    set neededRegs([expr {$MCP_scpd(scratchpad_addr)+$MCP_scpd(trcb_signature_addr)}]) $MCP_scpd(trcb_signature)
    set neededRegs([expr {$MCP_scpd(scratchpad_addr)+$MCP_scpd(trcb_start_ptr)}]) $MCP_scpd(trcb_offset_zero)
}

proc GRC_dump::SimpleArrangeList {orig_list} {
    set tmp_list [lsort -unique -integer $orig_list]
    set result {}
    set first -1
    set counter -1
    set expected -1
    for {set i 0} {$i < [llength $tmp_list]} {incr i} {
        if {$counter == -1} {
            set counter 1
            set first [expr {[lindex $tmp_list $i]}]
            set expected [expr {$first + 4}]
            continue
        }
        set current [expr {[lindex $tmp_list $i]}]
        if {$expected == $current} {
            incr counter
            incr expected 4
            continue
        } else {
            lappend result [list $first $counter]
            set counter -1
            set first $current
            set expected [expr {$first + 4}]
            continue
        }
        
    }
    if {$counter > -1} {
        lappend result [list $first $counter]
    }
	
	return $result
}

proc GRC_dump::dumpFromLinuxFile {linux_file meta_data_filename output_file_name is_ethtool_d  {temp_file_name temp.txt}} {
	variable is_linux_dump
	set is_linux_dump true
	if {$is_ethtool_d} 	{
		puts "Converting text file to bin file . . ."
		translateLinuxFile $linux_file $temp_file_name
		linuxLoadRegs $temp_file_name $meta_data_filename
		grcDump $output_file_name 
	} else {
		linuxLoadRegs $linux_file $meta_data_filename
		variable chosen_preset_args
		eval "grcDump $output_file_name $chosen_preset_args"
	}
}
##### MAIN ############
namespace eval GRC_dump {
    set fileStatus "false"
    addHelpGrcDump
}

namespace eval :: {
    namespace import -force GRC_dump::*
}
