#ifdef UEFI
#include <sys/types.h>
#include <machine/endian.h>
#endif
#ifdef FWUPGTOOL
#include <stdio.h>
#ifdef LINUX
#include <netinet/in.h> // Linux only
#else
#include <Winsock2.h>
#endif // LINUX
#endif // FWUPGTOOL

#include <string.h>
#include <stdlib.h>
#include <stdint.h>

#ifdef FWUPGTOOL
#include "tcl.h"
#include "tclInt.h"
#endif // FWUPGTOOL

#include "man_types.h"
#include "tcl_util.h"
#include "bdn.h"
#ifdef EDEBUG
#include "tcl_al_common.h"
#include "edebug_types.h"
#include "bcmtype.h"
#include "util.h"
#include "dbg_mcp.h"
#include "dbg_exprom.h"
#include "dbg_fw_img.h"
#include "dbg_man.h"
#include "dbg_os_utils.h"
#include "dbg_serport.h"
#include "hw_access.h"
#include "tcl.h"
#include "tclDecls.h"
#include "tcl_al.h"
extern Tcl_Interp *interp;
#else
#ifndef FWUPGTOOL
#include "dg_serport.h"
#include "dg_mcp.h"
#include "append.h"
#else
#include "../57710h/append.h"
#include "../57710h/util.h"
#define __USER_MODE_DEBUG // for debug.h
#endif // !FWUPGTOOL
#include "debug.h"
#include "dg_exprom.h"
#ifndef FWUPGTOOL
#include "lm5710.h"
#include "dg_fw_img.h"
#include "net_util.h"
#include "diag_man.h"
#include "dg_ipmi.h"
#include "hw_access.h"
#else
#include "dg_fw_img.h"
#endif // !FWUPGTOOL

#ifndef FWUPGTOOL
#include "dg_os_util.h"
#include "esc_seq.h"
#endif // !FWUPGTOOL

#endif // EDEBUG

#include "common_nvm.h"
#include "Decompress.h"

#include "nvm_map.h"
#ifndef FWUPGTOOL
#include "dev_if.h"
#endif // !FWUPGTOOL

#ifdef UEFI
#include "debug.h"
#endif
#include "shmem.h"

#ifdef FWUPGTOOL
// defined in .../577xx/drivers/Diag/edebug/common/include/util.h
#define PTR_SUB(_a, _b)     ((u8_t *) (_a) - (u8_t *) (_b))
#define OFFSETOF(_s, _m)    ((u32_t) PTR_SUB(&((_s *) 0)->_m, (u8_t *) 0))
#define MISC_REG_SHARED_MEM_ADDR    0xa2b4
#define MISC_REG_GENERIC_CR_0       0xa460
#define MISC_REG_GENERIC_CR_1       0xa464
#define MAX_PATH_NUM                2
#endif


#include "crctab.h"
#include "license.h"
#include "mcp_shmem.h"
#include "id.h"


#ifdef FWUPGTOOL
#include "status.h"
#include "nvm_hlp.h"
#include "toe_env.h"
#ifdef LINUX
#include "linux/BMAPI.h"
#endif

/*
// copied from ...\nx2\577xx\drivers\Common\include\utils.h
ARRSIZE:
used to calcualte item count of an array
this macro is used to prevent compile warning for unreferenced parametes
*/
#ifndef ARRSIZE
#define ARRSIZE(a)                   (sizeof(a)/sizeof((a)[0]))
#endif // ARRSIZE

#endif // FWUPGTOOL



#define BACKWARD_COMPATIBLE_VERSION                 "4.2.0"
#define E2_A0_NON_SUPPORTED_VER                     "6.4.0"
#define AUTO_MODE 8
#define AUTO_MODE_MASK 0x8

#ifdef FWUPGTOOL
#define TOE_STATUS_SUCCESS 0
// TODO:: Have to re-design the interface between FwUpg application
// and FwUpg library and see how to avoid using the global variable
// such as gpINx2Nvm!!!!!
NVM_INTERFACE    *gpINx2Nvm;
#endif // FWUPGTOOL

/* External functions */
#ifdef FWUPGTOOL
extern NVM_INTERFACE nvm_interface(Tcl_Interp *interp);
extern void printfWrapper(char *fmt, ...);
#endif // FWUPGTOOL

// used for shadow SWIMs feature
extern u32_t new_mfw_major, new_mfw_minor, new_mfw_build, new_mfw_drop;
extern u32_t running_bc_major, running_bc_minor, running_bc_build, running_bc_drop;

// used for virtual nvm feature
extern virtual_nvm_t virtual_nvm;

#ifndef MAX
#define MAX(a,b) (((a) > (b)) ? (a) : (b))
#endif

image_map_t image_table[] = {
	{ IMAGE_MAP_IDX_BC1, NVM_IMAGE_NAME_BC1, "-bc1", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_BC1, IMAGE_HDR_TYPE_BC1 },  /*0*/
	{ IMAGE_MAP_IDX_BC2, NVM_IMAGE_NAME_BC2, "-bc2", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_BC2, IMAGE_HDR_TYPE_BC2 },
	{ IMAGE_MAP_IDX_MBA, NVM_IMAGE_NAME_MBA, "-mba", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_MBA, IMAGE_HDR_TYPE_MBA },

	{ IMAGE_MAP_IDX_NCSI_CMN, NVM_IMAGE_NAME_NCSI_CMN, "-ncsi",  CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_NC_SI_CMN, IMAGE_HDR_TYPE_NCSI_CMN },
	{ IMAGE_MAP_IDX_NCSI_ELIB, NVM_IMAGE_NAME_NCSI_ELIB, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_NC_SI_EVEREST, IMAGE_HDR_TYPE_NCSI_LIB_EV },

	{ IMAGE_MAP_IDX_L2T, NVM_IMAGE_NAME_L2T, "-l2t", CODE_IMAGE_AGENT_TSTORM, CODE_IMAGE_TYPE_L2T, IMAGE_HDR_TYPE_L2T },
	{ IMAGE_MAP_IDX_L2C, NVM_IMAGE_NAME_L2C, "-l2c", CODE_IMAGE_AGENT_CSTORM, CODE_IMAGE_TYPE_L2C, IMAGE_HDR_TYPE_L2C },
	{ IMAGE_MAP_IDX_L2X, NVM_IMAGE_NAME_L2X, "-l2x", CODE_IMAGE_AGENT_XSTORM, CODE_IMAGE_TYPE_L2X, IMAGE_HDR_TYPE_L2X },
	{ IMAGE_MAP_IDX_L2U, NVM_IMAGE_NAME_L2U, "-l2u", CODE_IMAGE_AGENT_USTORM, CODE_IMAGE_TYPE_L2U, IMAGE_HDR_TYPE_L2U },

	{ IMAGE_MAP_IDX_IPMI, NVM_IMAGE_NAME_IPMI, "-ipmi", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_IPMI, IMAGE_HDR_TYPE_IPMI },
	{ IMAGE_MAP_IDX_MODULES_PN, NVM_IMAGE_NAME_MODULES_PN, "-m_pn", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_MODULES_PN, IMAGE_HDR_TYPE_MODULES_PN },

	{ IMAGE_MAP_IDX_ISCSI_BOOT, NVM_IMAGE_NAME_ISCSI_BOOT, "-ib", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_ISCSI_BOOT, IMAGE_HDR_TYPE_ISCSI_BOOT },
	{ IMAGE_MAP_IDX_ISCSI_BOOT_CFG, NVM_IMAGE_NAME_ISCSI_BOOT_CFG, "", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_ISCSI_BOOT_CFG, IMAGE_HDR_TYPE_ISCSI_BOOT_CFG },
	{ IMAGE_MAP_IDX_ISCSI_BOOT_CFG2, NVM_IMAGE_NAME_ISCSI_BOOT_CFG2, "", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_ISCSI_BOOT_CFG2, IMAGE_HDR_TYPE_ISCSI_BOOT_CFG },
	{ IMAGE_MAP_IDX_ISCSI_BOOT_CPRG, NVM_IMAGE_NAME_ISCSI_CPRG, "-ibpe", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_ISCSI_BOOT_CPRG, IMAGE_HDR_TYPE_ISCSI_BOOT_CPRG },

	{ IMAGE_MAP_IDX_FCOE_BOOT, NVM_IMAGE_NAME_FCOE_BOOT, "-feb", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_FCOE_BOOT, IMAGE_HDR_TYPE_FCOE_BOOT }, /*15*/
	{ IMAGE_MAP_IDX_FCOE_BOOT_CFG, NVM_IMAGE_NAME_FCOE_BOOT_CFG, "", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_FCOE_BOOT_CFG, IMAGE_HDR_TYPE_FCOE_BOOT_CFG },
	{ IMAGE_MAP_IDX_FCOE_BOOT_CFG2, NVM_IMAGE_NAME_FCOE_BOOT_CFG2, "", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_FCOE_BOOT_CFG2, IMAGE_HDR_TYPE_FCOE_BOOT_CFG },
	{ IMAGE_MAP_IDX_FCOE_BOOT_CFG3, NVM_IMAGE_NAME_FCOE_BOOT_CFG3, "", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_FCOE_BOOT_CFG3, IMAGE_HDR_TYPE_FCOE_BOOT_CFG },
	{ IMAGE_MAP_IDX_FCOE_BOOT_CFG4, NVM_IMAGE_NAME_FCOE_BOOT_CFG4, "", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_FCOE_BOOT_CFG4, IMAGE_HDR_TYPE_FCOE_BOOT_CFG },
	{ IMAGE_MAP_IDX_FCOE_BOOT_CPRG, NVM_IMAGE_NAME_FCOE_BOOT_CPRG_EVRST, "", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_FCOE_BOOT_CPRG_EVRST, IMAGE_HDR_TYPE_FCOE_BOOT_CPRG_EVRST },
	{ IMAGE_MAP_IDX_NICP_CFG, NVM_IMAGE_NAME_NIC_PARTITION_CFG, "-nicp", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_NIC_PARTITION_CFG, IMAGE_HDR_TYPE_NIC_PARTITION_CFG },
	{ IMAGE_MAP_IDX_VPD, NVM_IMAGE_NAME_VPD, "-vpd", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_VPD, IMAGE_HDR_TYPE_VPD_TABLE },
	{ IMAGE_MAP_IDX_E3_WC, NVM_IMAGE_NAME_E3_WC, "", CODE_IMAGE_AGENT_E3WC, CODE_IMAGE_TYPE_E3_WC, IMAGE_HDR_TYPE_E3_WC },
	{ IMAGE_MAP_IDX_E3_PCIE, NVM_IMAGE_NAME_E3_PCIE, "", CODE_IMAGE_AGENT_E3PCIE, CODE_IMAGE_TYPE_E3_PCIE, IMAGE_HDR_TYPE_E3_PCIE },
	{ IMAGE_MAP_IDX_NIV_CFG, NVM_IMAGE_NAME_NIV_CFG, "-niv", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_VNTAG_DATA, IMAGE_HDR_TYPE_NIV_CFG },
	{ IMAGE_MAP_IDX_NIV_PROFILES_CFG, NVM_IMAGE_NAME_NIV_PROFILES_CFG, "-nivprofiles", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_VNTAG_PROFILES_DATA, IMAGE_HDR_TYPE_NIV_PROFILES_CFG },
	{ IMAGE_MAP_IDX_BOOT_CFG_SHADOW, NVM_IMAGE_NAME_BOOT_CFG_SHADOW, "-bootsh", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_BOOT_CFG_SHADOW, IMAGE_HDR_TYPE_BOOT_CFG_SHADOW },
	{ IMAGE_MAP_IDX_CFG_EXTEBDED_SHARED, NVM_IMAGE_NAME_CFG_EXTENDED_SHARED, "-cfg_extended_shared", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, IMAGE_HDR_TYPE_CFG_EXTENDED_SHARED },
	{ IMAGE_MAP_IDX_ISCSI_BOOT_CFG3, NVM_IMAGE_NAME_ISCSI_BOOT_CFG3, "", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_ISCSI_BOOT_CFG3, IMAGE_HDR_TYPE_ISCSI_BOOT_CFG },
	{ IMAGE_MAP_IDX_ISCSI_BOOT_CFG4, NVM_IMAGE_NAME_ISCSI_BOOT_CFG4, "", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_ISCSI_BOOT_CFG4, IMAGE_HDR_TYPE_ISCSI_BOOT_CFG },
	{ IMAGE_MAP_IDX_MFW1, NVM_IMAGE_NAME_MFW1, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_MFW1, IMAGE_HDR_TYPE_MFW1 },
	{ IMAGE_MAP_IDX_MFW2, NVM_IMAGE_NAME_MFW2, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_MFW2, IMAGE_HDR_TYPE_MFW2 },
	{ IMAGE_MAP_IDX_SWIM1, NVM_IMAGE_NAME_SWIM1, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM1, IMAGE_HDR_TYPE_SWIM1 },
	{ IMAGE_MAP_IDX_SWIM2, NVM_IMAGE_NAME_SWIM2, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM2, IMAGE_HDR_TYPE_SWIM2 },
	{ IMAGE_MAP_IDX_SWIM3, NVM_IMAGE_NAME_SWIM3, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM3, IMAGE_HDR_TYPE_SWIM3 },
	{ IMAGE_MAP_IDX_SWIM4, NVM_IMAGE_NAME_SWIM4, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM4, IMAGE_HDR_TYPE_SWIM4 },
	{ IMAGE_MAP_IDX_SWIM5, NVM_IMAGE_NAME_SWIM5, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM5, IMAGE_HDR_TYPE_SWIM5 },
	{ IMAGE_MAP_IDX_SWIM6, NVM_IMAGE_NAME_SWIM6, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM6, IMAGE_HDR_TYPE_SWIM6 },
	{ IMAGE_MAP_IDX_SWIM7, NVM_IMAGE_NAME_SWIM7, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM7, IMAGE_HDR_TYPE_SWIM7 },
	{ IMAGE_MAP_IDX_SWIM8, NVM_IMAGE_NAME_SWIM8, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM8, IMAGE_HDR_TYPE_SWIM8 },
	{ IMAGE_MAP_IDX_SWIM1_B, NVM_IMAGE_NAME_SWIM1_B, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM1_B, IMAGE_HDR_TYPE_SWIM1_B },
	{ IMAGE_MAP_IDX_SWIM2_B, NVM_IMAGE_NAME_SWIM2_B, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM2_B, IMAGE_HDR_TYPE_SWIM2_B },
	{ IMAGE_MAP_IDX_SWIM3_B, NVM_IMAGE_NAME_SWIM3_B, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM3_B, IMAGE_HDR_TYPE_SWIM3_B },
	{ IMAGE_MAP_IDX_SWIM4_B, NVM_IMAGE_NAME_SWIM4_B, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM4_B, IMAGE_HDR_TYPE_SWIM4_B },
	{ IMAGE_MAP_IDX_SWIM5_B, NVM_IMAGE_NAME_SWIM5_B, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM5_B, IMAGE_HDR_TYPE_SWIM5_B },
	{ IMAGE_MAP_IDX_SWIM6_B, NVM_IMAGE_NAME_SWIM6_B, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM6_B, IMAGE_HDR_TYPE_SWIM6_B },
	{ IMAGE_MAP_IDX_SWIM7_B, NVM_IMAGE_NAME_SWIM7_B, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM7_B, IMAGE_HDR_TYPE_SWIM7_B },
	{ IMAGE_MAP_IDX_SWIM8_B, NVM_IMAGE_NAME_SWIM8_B, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_SWIM8_B, IMAGE_HDR_TYPE_SWIM8_B },
	{ IMAGE_MAP_IDX_MFW2_A, NVM_IMAGE_NAME_MFW2_A, "", CODE_IMAGE_AGENT_MCP, CODE_IMAGE_TYPE_MFW2_A, IMAGE_HDR_TYPE_MFW2_A },
	{ IMAGE_MAP_IDX_OCNVM, NVM_IMAGE_NAME_OCNVM, "", CODE_IMAGE_AGENT_NONE, CODE_IMAGE_TYPE_OCNVM, IMAGE_HDR_TYPE_OCNVM },
	{ IMAGE_MAP_IDX_E3_WCV2, NVM_IMAGE_NAME_E3_WCV2, "", CODE_IMAGE_AGENT_E3WC, CODE_IMAGE_TYPE_E3_WCV2, IMAGE_HDR_TYPE_E3_WCV2 },
	{ IMAGE_MAP_IDX_E3_PCIEV2, NVM_IMAGE_NAME_E3_PCIEV2, "", CODE_IMAGE_AGENT_E3PCIE, CODE_IMAGE_TYPE_E3_PCIEV2, IMAGE_HDR_TYPE_E3_PCIEV2 },
	{ IMAGE_MAP_IDX_CCM, NVM_IMAGE_NAME_CCM, "-ccm", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_CCM, IMAGE_HDR_TYPE_CCM },
	{ IMAGE_MAP_IDX_HW_SET, NVM_IMAGE_NAME_HW_SET, "-hw_set", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_HW_SET, IMAGE_HDR_TYPE_HW_SET },
	{ IMAGE_MAP_IDX_USR_BLK, NVM_IMAGE_NAME_USR_BLK, "-usrblk", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_USR_BLK, IMAGE_HDR_TYPE_USR_BLK },
	{ IMAGE_MAP_IDX_ISCSI_PERS, NVM_IMAGE_NAME_ISCSI_PERS, "-ipers", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_ISCSI_PERS, IMAGE_HDR_TYPE_ISCSI_PERS },
	{ IMAGE_MAP_IDX_BDN, NVM_IMAGE_NAME_BDN, "-bdn", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_BDN, IMAGE_HDR_TYPE_BDN },
	{ IMAGE_MAP_IDX_MDUMP, NVM_IMAGE_NAME_MDUMP, "-mdump", CODE_IMAGE_AGENT_HOST, CODE_IMAGE_TYPE_MDUMP, IMAGE_HDR_TYPE_MDUMP },
};




// Global NVM config and SHMEM config
nvm_cfg_t nvm_config;
nvm_cfg_t shmem_config;

// for virtual nvm mode
u8_t *virtual_nvm_buff;

/* Static functions    */
static common_nvm_status_t common_nvm_keep_mac(u32_t mac, u32_t byte_cnt, u8_t *buf, u8_t big_endian);
static common_nvm_status_t common_nvm_keep_license(u32_t port_no, u32_t byte_cnt, u8_t *buf, u8_t big_endian);
static common_nvm_status_t common_nvm_keep_nvm_config(u32_t byte_cnt, u8_t *buf, u8_t big_endian);
static common_nvm_status_t common_nvm_keep_iscsi_fcoe_boot_config(u32_t byte_cnt, u8_t *buf, u8_t big_endia, u8_t is_iscsi);
//static common_nvm_status_t common_nvm_is_4port(int *result);


#ifdef FWUPGTOOL

#ifdef LINUX
void os_if_save_cursor(void) {
	fprintf(stdout, "\x1b\[s");
	fflush(stdout);
}

void os_if_restore_cursor(void) {
	fprintf(stdout, "\x1b\[u");
	fflush(stdout);
}

void os_if_print(char *buf, u16_t len) {
	fprintf(stdout, "%s", buf);
	fflush(stdout);
}
#else // Windows
int last_msg_len = 0;

void os_if_save_cursor(void) {
	last_msg_len = 0;
}

void os_if_restore_cursor(void) {
	int i;
	for (i = 0; i < last_msg_len; i++) {
		fprintf(stdout, "\b");
	}
	fflush(stdout);
}

void os_if_print(char *buf, u16_t len) {
	last_msg_len = len;
	fprintf(stdout, "%s", buf);
	fflush(stdout);
}
#endif

// These functions are extracted from disputil.c in
// \\server\main\nx2\577xx\drivers\Diag\common\util rev#58
// in Perforce and modified FwUpg Tool...
void sc_restore_cursor(void) {
	os_if_restore_cursor();
}

void sc_save_cursor(void) {
	os_if_save_cursor();
}

void sc_print(char *buf, u16_t len) {
	os_if_print(buf, len);
}



// These functions are similar to functions defined in man_types.h
// \\server\main\nx2\577xx\drivers\Diag\common\diag_manager\include rev#34
// in Perforce and modified FwUpg Tool...
u32_t
COMMON_NVM_IS_E1x() {
	u32_t is_e1x = 0;

//    printf( "COMMON_NVM_IS_E1x: m_current_dev_index = 0x%x, m_chip_rev_id = 0x%x\n",
//        gpINx2Nvm->m_current_dev_index, gpINx2Nvm->m_chip_rev_id );

	switch (EV_ASIC_REV(gpINx2Nvm->m_chip_rev_id)) {
	case CHIP_NUM_5710:
	case CHIP_NUM_5711:
	case CHIP_NUM_5711E:
		is_e1x = 1;
		break;

	default:
		is_e1x = 0;
	}

//    printf( "COMMON_NVM_IS_E1x: return is_e1x = %d\n", is_e1x );

	return is_e1x;
}

u32_t
COMMON_NVM_IS_E2() {
	u32_t is_e2 = 0;

//    printf( "COMMON_NVM_IS_E2: m_current_dev_index = 0x%x, m_chip_rev_id = 0x%x\n",
//        gpINx2Nvm->m_current_dev_index, gpINx2Nvm->m_chip_rev_id );

	// The ediag uses device_id, but we use chip_rev_id...
	switch (EV_ASIC_REV(gpINx2Nvm->m_chip_rev_id)) {
	case CHIP_NUM_5712:     // 57712 (SERDES)
	case CHIP_NUM_5712E:    // 57712E (SERDES)
	case CHIP_NUM_5713:     // 57713 (10G Base-T)
	case CHIP_NUM_5713E:    // 57713E (10G Base-T)
		is_e2 = 1;
		break;

	default:
		is_e2 = 0;
	}

//    printf( "COMMON_NVM_IS_E2: return is_e2 = %d\n", is_e2 );

	return is_e2;
}

u32_t
COMMON_NVM_IS_E3() {
	u32_t is_e3 = 0;

//    printf( "COMMON_NVM_IS_E2: m_current_dev_index = 0x%x, m_chip_rev_id = 0x%x\n",
//        gpINx2Nvm->m_current_dev_index, gpINx2Nvm->m_chip_rev_id );

	// The ediag uses device_id, but we use chip_rev_id...
	switch (EV_ASIC_REV(gpINx2Nvm->m_chip_rev_id)) {
	case CHIP_NUM_57800:    // 57800 (Serdes 2x10G +2x1G)
	case CHIP_NUM_57840_4_10:    // 57840 (Serdes 4x10G)
	case CHIP_NUM_57840_2_20:    // 57840 (Serdes 2x20G)
	case CHIP_NUM_57840_OBSOLETE:    // 57840 (Serdes 4x10G or 2x20G) - old DID
	case CHIP_NUM_57810:    // 57810 (Serdes/Copper 2x10G)
	case CHIP_NUM_57811:    //  57811 single port flavor
		is_e3 = 1;
		break;

	default:
		is_e3 = 0;
	}

//    printf( "COMMON_NVM_IS_E3: return is_e3 = %d\n", is_e3 );

	return is_e3;
}

u32_t
COMMON_NVM_FUNC_NUM() {
	int function_no;

	// ENV_FUNC_NUM is set in SetSelectedNIC() in Bnx2Helper.cpp...
	function_no = atoi(Tcl_GetVar2(gpINx2Nvm->m_tcl_interp, TCL_ENV, ENV_FUNC_NUM, 0));
//    printf( "COMMON_NVM_FUNC_NUM: function_no = 0x%x\n", function_no );

	return function_no;
}

u32_t
COMMON_NVM_PATH_NUM() {
	if (COMMON_NVM_IS_E1x()) return 0; // always use path0 E1 and E1_5 NICs
	else {
		if (COMMON_NVM_FUNC_NUM() & 0x1) {
			return 1; // function 1, 3, 5, 7
		} else {
			return 0; // function 0, 2, 4, 6
		}
	}
}

#endif

/*******************************************
 *
 *               Interface functions
 *
 *******************************************/
common_nvm_status_t common_nvm_read_nvram(
	u32_t   addr,
	u32_t *val_p,
	u32_t   byte_cnt
	) {

#ifndef FWUPGTOOL
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;
	dev_if_t *cur_dev = NULL;
	u32_t i, data32, *ptr32;

	ptr32 = val_p;

	if (!common_nvm_tcl_al_get_virtual_nvm_mode()) {
		cur_dev = COMMON_NVM_CUR_DEV_IF;
	}

	/* Read the block */
	if (common_nvm_tcl_al_get_virtual_nvm_mode()) {
		memcpy(val_p, &virtual_nvm.nvm_buff[addr], byte_cnt);
	} else if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
		if (!COMMON_NVM_CUR_DEV_IS_VALID) {
			return COMMON_NVM_STATUS_FAILURE;
		} else {
			status = common_nvm_dev_read_nvram(cur_dev, addr, val_p, byte_cnt);
		}
	} else {

		u8_t duart_port = common_nvm_get_duart_port();
		status = common_nvm_duart_nvm_read(duart_port, addr, val_p, byte_cnt);
	}

	if (status == COMMON_NVM_STATUS_SUCCESS) {
		/* Translate to driver format */
		for (i = 0; i < byte_cnt; i += 4, ptr32++) {
			data32 = ntohl(*ptr32);
			*ptr32 = data32;
		}
	} else {
		DbgMessage(NULL, FATAL, "common_nvm_read_nvram failed\n");
	}

#else
	common_nvm_status_t status;
	u8_t *g_buf = NULL;
	int toe_status;

	if (byte_cnt == sizeof(u32_t)) {
		toe_status = gpINx2Nvm->m_td_read_nv_mem32(gpINx2Nvm->m_current_dev_index, addr, val_p, g_buf);
	} else {
		toe_status = gpINx2Nvm->m_td_read_nv_mem_blk(gpINx2Nvm->m_current_dev_index, addr, byte_cnt, val_p, g_buf);
	}

	if (toe_status == TOE_STATUS_SUCCESS) {
		status = COMMON_NVM_STATUS_SUCCESS;
	} else {
		status = COMMON_NVM_STATUS_FAILURE;
	}
#endif // !FWUPGTOOL

	return status;
}

common_nvm_status_t common_nvm_nvm_is_legacy_iscsi_image(
	image_header_t *file_image_p
	) {
	switch (file_image_p->type) {
	case IMAGE_HDR_TYPE_ISCSI_BOOT_CFG_PRG_LGCY:
	case IMAGE_HDR_TYPE_ISCSI_BOOT_CPRG_LGCY:
		return COMMON_NVM_STATUS_SUCCESS;
	default:
		return COMMON_NVM_STATUS_FAILURE;
	}
}


common_nvm_status_t common_nvm_write_nvram(
	u32_t   addr,
	u32_t *val_p,
	u32_t   byte_cnt
	) {

#ifndef FWUPGTOOL
	dev_if_t *cur_dev = COMMON_NVM_CUR_DEV_IF;
	u32_t data32, *ptr32, i;
#else
	u8_t *g_buf = NULL;
	int toe_status;
#ifdef LINUX
	u32_t data32, i;
#endif
	u32_t *ptr32;
#endif // !FWUPGTOOL
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;
#ifndef FWUPGTOOL
	u32_t flash_strap, flash_vendor, val32;
#endif
	ptr32 = val_p;

#ifndef FWUPGTOOL

	/* Convert to network order for driver */
	for (i = 0; i < byte_cnt; i += 4, ptr32++) {
		data32 = htonl(*ptr32);
		*ptr32 = data32;
	}

	if (common_nvm_tcl_al_get_virtual_nvm_mode()) {
		memcpy(&virtual_nvm.nvm_buff[addr], val_p, byte_cnt);
		if (virtual_nvm.fd) {
			fseek(virtual_nvm.fd, 0, SEEK_SET);
			fwrite(virtual_nvm.nvm_buff, 1, virtual_nvm.nvm_size, virtual_nvm.fd);
		}
	} else if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
		if (!COMMON_NVM_CUR_DEV_IS_VALID) {
			return COMMON_NVM_STATUS_FAILURE;
		} else {
			/* Special handling for "Auto" mode write access */
			// Read flash strap from MCP_REG_MCPR_NVM_RECONFIG bits [3:0]
			common_nvm_reg_read(MCP_REG_MCPR_NVM_RECONFIG, &flash_strap);
			flash_strap = (flash_strap & MCPR_NVM_RECONFIG_ORIG_STRAP_VALUE);

			// Read flash vendor from MCP_REG_MCPR_NVM_CFG4 bit 3
			common_nvm_reg_read(MCP_REG_MCPR_NVM_CFG4, &flash_vendor);
			flash_vendor = (flash_vendor & MCPR_NVM_CFG4_FLASH_VENDOR);
			DbgMessage(NULL, INFORM, "flash_strap is 0x%x flash_vendor is 0x%x\n", flash_strap, flash_vendor);

#ifndef EDEBUG
			// Incase on new mode "auto mode":
			if ((flash_strap & AUTO_MODE_MASK) == AUTO_MODE) {
				// For ST change CSB_W bits [15:8] to 0x40
				if (flash_vendor == MCPR_NVM_CFG4_FLASH_VENDOR_ST) {
					DbgMessage(NULL, INFORM, "Detected Auto mode in ST flash\n");
					common_nvm_reg_read(MCP_REG_MCPR_NVM_CFG2, &val32);
					common_nvm_reg_write(MCP_REG_MCPR_NVM_CFG2, ((val32 & 0xFFFF00FF) | 0x4000));
				}

				// For Atmel Auto mode:
				if (flash_vendor == MCPR_NVM_CFG4_FLASH_VENDOR_ATMEL) {
					DbgMessage(NULL, INFORM, "Detected Auto mode in ATMEL flash\n");

					if (is_e3(globalInterp) && is_rev_b(globalInterp)) {
						DbgMessage(NULL, INFORM, "Detected E3B0 device\n");
						// For E3B0, clear bit 10
						common_nvm_reg_read(MCP_REG_MCPR_NVM_CFG4, &val32);
						common_nvm_reg_write(MCP_REG_MCPR_NVM_CFG4, (val32 & ~0x400));

						// For E3B0, change STATUS_CMD bits [23:16] to 0xD7 and CSB_W bits[15:8] to 0x40
						common_nvm_reg_read(MCP_REG_MCPR_NVM_CFG2, &val32);
						common_nvm_reg_write(MCP_REG_MCPR_NVM_CFG2, ((val32 & 0xFF0000FF) | 0xD74000));
					} else {
						// For E3A0 and below, change bits [23:16] to 0x57 and bits [15:8] to 0x40
						common_nvm_reg_read(MCP_REG_MCPR_NVM_CFG2, &val32);
						common_nvm_reg_write(MCP_REG_MCPR_NVM_CFG2, ((val32 & 0xFF0000FF) | 0x574000));
					}
				}
			}
			/* End of special handling for Auto mode write access */
#endif

			status = common_nvm_dev_write_nvram(cur_dev, addr, val_p, byte_cnt);
		}
	} else {
		u8_t duart_port = common_nvm_get_duart_port();
		status = common_nvm_duart_nvm_write(duart_port, addr, val_p, byte_cnt);
	}

	/* Restore the original buffer */
	ptr32 = val_p;
	for (i = 0; i < byte_cnt; i += 4, ptr32++) {
		data32 = ntohl(*ptr32);
		*ptr32 = data32;
	}

#else
#ifdef LINUX
	/* Convert to host order for driver */
	for (i = 0; i < byte_cnt; i += 4, ptr32++) {
		data32 = ntohl(*ptr32);
		*ptr32 = data32;
	}
#endif

	if (byte_cnt == sizeof(u32_t)) {
		toe_status = gpINx2Nvm->m_td_write_nv_mem32(gpINx2Nvm->m_current_dev_index, addr, *val_p, g_buf);
	} else {
		toe_status = gpINx2Nvm->m_td_write_nv_mem_blk(gpINx2Nvm->m_current_dev_index, addr, byte_cnt, val_p, g_buf);
	}

	if (toe_status == TOE_STATUS_SUCCESS) {
		status = COMMON_NVM_STATUS_SUCCESS;
	} else {
		status = COMMON_NVM_STATUS_FAILURE;
	}

#ifdef LINUX
	/* Restore the original buffer */
	ptr32 = val_p;
	for (i = 0; i < byte_cnt; i += 4, ptr32++) {
		data32 = htonl(*ptr32);
		*ptr32 = data32;
	}
#endif

#endif // !FWUPGTOOL

	return status;
}

#ifdef FWUPGTOOL
/*
	This function is copied from dg_os_mem.c since the module
	is not included in makeing the FWUPG.
*/

void common_nvm_os_swap_bytes(
	u32_t  *buf,
	u32_t  buf_size
	) {
	u32_t i;

	for (i = 0; i < buf_size / 4; i++) {
		buf[i] = htonl(buf[i]);
	}

	return;
}

/*
 *--------------------------------------------------------------------------
 * common_nvm_write_mac_partition_data --
 *
 * This utility is used to write the provided NIC-partition block into NVRAM
 *
 *--------------------------------------------------------------------------
 */
static common_nvm_status_t common_nvm_write_mac_partition_data(
	u32_t orig_macp_image_found,
	nvm_nic_part_cfg_t orig_macp_image) {
	u32_t index_image_table;
	common_nvm_status_t status;

	/* find image index in image_table */
	status = common_nvm_find_entry_in_image_table(CODE_IMAGE_TYPE_NIC_PARTITION_CFG, &index_image_table);
	if (status != COMMON_NVM_STATUS_SUCCESS) return COMMON_NVM_STATUS_FAILURE;

	// We need this because data in orig_macp_image has to be
	// in LE format before calling common_nvm_prog_image_in_extended_dir()
	// Data in orig_macp_image were read from NVRAM in LE format for LE machine
	// and in BE format for BE machine...
	swap_buffer_if_big_endian_host((u32_t *)&orig_macp_image,
								   (sizeof(nvm_nic_part_cfg_t) / sizeof(u32_t)));

	/* program the provided image */
	status = common_nvm_prog_image_in_extended_dir((u8_t *)&orig_macp_image,
												   sizeof(nvm_nic_part_cfg_t), index_image_table, FALSE, TRUE, FALSE);

	return status;
}
#endif // FWUPGTOOL

/*
 *------------------------------------------------------------------------
 * swap_buffer_if_big_endian_host --
 *
 *------------------------------------------------------------------------
 */
void swap_buffer_if_big_endian_host(u32_t *pBuf, u32_t uLen) {
	while (uLen--) {
		*pBuf = BMAPI_BENDIAN_SWAP(*pBuf);
		pBuf++;
	}
}

/*
 *------------------------------------------------------------------------
 * Check if specific OEM NPAR1.5 device ID
 *
 *------------------------------------------------------------------------
 */

u8_t common_nvm_is_oem_npar_device(u16_t vendor_id, u16_t sub_vendor_id, u16_t sub_device_id) {
	u32_t status, nvm_addr, entry_idx, found_image, byte_cnt, manufacture_data_offset, manufacture_data = 0;

	if ((vendor_id == BROADCOM_VID) &&
		(sub_vendor_id == OEM_ID_2) &&
		((sub_device_id == 0x193a) || (sub_device_id == 0x1930) ||
		 (sub_device_id == 0x1931) || (sub_device_id == 0x18d3) ||
		 (sub_device_id == 0x3382) || (sub_device_id == 0x339d) ||
		 (sub_device_id == 0x1916) || (sub_device_id == 0x1917) ||
		 (sub_device_id == 0x1932) || (sub_device_id == 0x1933) ||
		 (sub_device_id == 0x1798) || (sub_device_id == 0x17a5) ||
		 (sub_device_id == 0x22fa))) {
		return 1;
	} else if ((vendor_id == BROADCOM_VID) &&
			   (sub_vendor_id == OEM_ID_3) &&
			   ((sub_device_id == 0x04d9) || (sub_device_id == 0x049f))) {
		// per CQ74871, CQ75271
		return 1;
	} else {
		/* find manufacture_data in nvram */
		status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &nvm_addr, &byte_cnt, &entry_idx);

		if ((status == COMMON_NVM_STATUS_SUCCESS) && found_image) {
			manufacture_data_offset = OFFSETOF(struct extended_dev_info_shared_cfg, manufacture_data);

			if (byte_cnt > manufacture_data_offset) {
				common_nvm_read_nvram(nvm_addr + manufacture_data_offset, &manufacture_data, sizeof(u32_t));
			} else {
				manufacture_data = 0;
				printf("warning: nvm cfg option 236 - Number of ISCSI/FCOE cfg images was not configured.\n");
			}

			/* Check if num_iscsi_fcoe_cfgs in nvm cfg is 4 */
			if ((manufacture_data & EXTENDED_DEV_INFO_SHARED_CFG_NUM_ISCSI_FCOE_CFGS_MASK) ==
					EXTENDED_DEV_INFO_SHARED_CFG_NUM_ISCSI_FCOE_CFGS_4) {
				return 1;
			} else {
				return 0;
			}
		} else {
			return 0;
		}
	}

}

/*
 *------------------------------------------------------------------------
 * program_nvm --
 *
 *     Program buffer content into NVM. If the offset starts at zero, this
 *     function will preserve the MAC addresses (L2 and iSCSI) that are
 *     already in the flash.
 *
 * Results:
 *     0 if success, -1 for failure/abort.
 *
 * Output:
 *     The 'total_byte_ptr' contains the amount of data programmed into NVM
 *     (may be greater than 'byte_cnt' if PRG_NVM_APPEND_CRC flag is set).
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_program_nvm(
	u8_t *buf,
	u32_t byte_cnt,
	u32_t offset,
	u32_t flag,
	u32_t *total_byte_ptr
	) {
	u32_t idx, *ptr32, *start_ptr, data32, crc32, nvm_offset;
	common_nvm_status_t status;
	u8_t port_no;
	u32_t save_val;
	media_vpd_t old_vpd;
	media_vpd_t new_vpd;
	vpd_info_t old_vpd_struct_format; // as defined in dg_vpd.h
	vpd_info_t new_vpd_struct_format; // as defined in dg_vpd.h
	u32_t need_manuf_crc = 0;
	u32_t need_feature_crc = 0;
	u8_t big_endian   = (flag & PRG_NVM_BIG_ENDIAN) ? 1 : 0;
	u8_t cal_crc      = (flag & PRG_NVM_APPEND_CRC) ? 1 : 0;
	u8_t mac_prg      = (flag & PRG_NVM_MAC_PRG) ? 1 : 0;
	u8_t lic_prg      = (flag & PRG_NVM_LIC_PRG) ? 1 : 0;
	u8_t skip_cfg_prg = (flag & PRG_NVM_SKIP_CFG_PRG) ? 1 : 0;
	u8_t ibcfg_prg    = (flag & PRG_NVM_IBCFG) ? 1 : 0;
	u8_t febcfg_prg   = (flag & PRG_NVM_FEBCFG) ? 1 : 0;
	u32_t preserve_ec = (flag & PRESERVE_NVM_EC) ? 1 : 0;
	u32_t preserve_sn = (flag & PRESERVE_NVM_SN) ? 1 : 0;
	u32_t handle_macp = (flag & PRG_HANDLE_MACP) ? 1 : 0;
	u32_t extra_padding = (flag & PRG_NVM_EXTRA_PADDING) ? 4 : 0;
	u32_t alignment_mask = 0xfffffffc;
	u8_t vpd_check    = 0;
	u32_t num_paths = 1;
	u32_t curr_path = 0;
	u32_t macp_image_found;
	nvm_nic_part_cfg_t macp_image;
	u32_t block_size = 1024;
	u8_t *tmp_buf = NULL;
#ifndef FWUPGTOOL
	char msg[80] = "\0";
#endif

#ifndef FWUPGTOOL
	if (!is_e1x(globalInterp)) num_paths = 2;
#else
	if (!COMMON_NVM_IS_E1x()) num_paths = 2;
#endif


	/* Because we are rounding up to 4 or 8-byte alignment,
	 * we need to pad extra bytes with 0's, rather
	 * than leaving junk characters. */
	tmp_buf = (u8_t *)(uintptr_t)common_nvm_alloc_virt(byte_cnt + 4 + extra_padding);

	if (tmp_buf == NULL) {
		DbgMessage(NULL, FATAL, "memory allocation error\n");
		return COMMON_NVM_STATUS_RESOURCE;
	}
	memset(tmp_buf, 0, byte_cnt + 4 + extra_padding);
	memcpy(tmp_buf, buf, byte_cnt);
	/* Modify the pointer and byte count */
	buf = tmp_buf;
	/* Round up to the next 4 or 8-byte multiple */
	if (extra_padding) alignment_mask = 0xfffffff8;
	byte_cnt = (byte_cnt + 3 + extra_padding) & alignment_mask;

	/* When programing from offset 0, preserve or force some fields
		   e.g. MAC address, Serial number (SN), Engineer Change (EC) */
	if (offset == 0) {
		/* if one of the option '-skip_ec' or '-skip_sn' is set
	  preserve these fields EC /SN in VPD. */
		if (preserve_ec || preserve_sn) {
			/* VPD serial number */
			nvm_offset = NVM_OFFSET(vpd);
			if (byte_cnt > nvm_offset) {  //TBD: consider a less harsh condition as if(nvm_offset+SN_offset_in_vpd).
										  //Note: all vpd with the saved SN will be extracted back to a buffer
										  //other fields might change their offsets too ! if SN size has changed. everything will be shifted inside.

				//extract old vpd
				status = common_nvm_read_nvram(nvm_offset, (u32_t *)(&old_vpd), sizeof(media_vpd_t));
				if (status != COMMON_NVM_STATUS_SUCCESS) {
					goto common_nvm_program_nvm_cleanup;
				}
				// Get VPD info from board into vpd_struct_format
				// no need to check the old VPD (vpd_check=0), since it could be an empty board on first burn.
				common_nvm_get_vpd_info(&old_vpd_struct_format, &old_vpd, vpd_check);

				//extract new vpd requested (including perhaps a different SN which should be ignored ! )
				memcpy(&new_vpd, &(buf[nvm_offset]), sizeof(media_vpd_t));
				if (big_endian) common_nvm_swap_bytes((u32_t *)(&new_vpd), sizeof(media_vpd_t));
				// Get VPD info from image to burn into vpd_struct_format
				// Check the VPD is valid.
				vpd_check = 1;
				common_nvm_get_vpd_info(&new_vpd_struct_format, &new_vpd, vpd_check);

				// Incase -skip_sn option was set
				// save the old serial number if it is valid
				if (preserve_sn) {
					if (old_vpd_struct_format.serial_number[0] != '\0') {
						memcpy(
							&(new_vpd_struct_format.serial_number),
							&(old_vpd_struct_format.serial_number),
							(sizeof(u8_t) * (MAX_SERIAL_NUMBER_LENGTH + 1))  // according to 'vpd_info_t' struct.
							);
					}
				}

				// Incase -skip_ec option was set
				// save the old engineer change number if it is valid
				if (preserve_ec) {
					if (old_vpd_struct_format.ec_number[0] != '\0') {
						memcpy(
							&(new_vpd_struct_format.ec_number),
							&(old_vpd_struct_format.ec_number),
							(sizeof(u8_t) * (MAX_EC_NUMBER_LENGTH + 1))  // according to 'vpd_info_t' struct.
							);
					}
				}

				// set new vpd back with old SN inside.
				common_nvm_set_vpd_info(&new_vpd_struct_format, &new_vpd); // create the vpd buffer again !  other fields might have changed position
				if (big_endian) common_nvm_swap_bytes((u32_t *)(&new_vpd), sizeof(media_vpd_t));
				memcpy(&(buf[nvm_offset]), &new_vpd, sizeof(media_vpd_t));
			}
		}

		// Incase -skip_config option, keep both the manufacturer and feature information.
		// Untouch the MAC information as it will be dealt with in 'if (mac_prg == 0)' block.
		if (skip_cfg_prg != 0) {
			status = common_nvm_keep_nvm_config(byte_cnt, buf, big_endian);
			if (status != COMMON_NVM_STATUS_SUCCESS) {
				goto common_nvm_program_nvm_cleanup;
			}
			need_feature_crc = 1;
			need_manuf_crc = 1;
		}

		// Incase -force_mac was not set, keep the current MAC address
		if (mac_prg == 0) {
			/* remember if before programing new nvm the mac partition
			   image existed, and if so: store its values */
			if (handle_macp) {
				common_nvm_nvm_fill_mac_partition_info(&macp_image, &macp_image_found);
			}

			for (curr_path = 0; curr_path < num_paths; ++curr_path) {
				// Go through the devices ports
				for (port_no = 0; port_no < PORT_MAX; port_no++) {
					// Keep the primary MAC
					status = common_nvm_keep_mac(NVM_OFFSET_PATH(manuf_info.port_hw_config[port_no].mac_lower, curr_path), byte_cnt, buf, big_endian);
					if (status != COMMON_NVM_STATUS_SUCCESS) {
						goto common_nvm_program_nvm_cleanup;
					}

					// Keep the iSCSI MAC
					status = common_nvm_keep_mac(NVM_OFFSET_PATH(manuf_info.port_hw_config[port_no].iscsi_mac_lower, curr_path), byte_cnt, buf, big_endian);
					if (status != COMMON_NVM_STATUS_SUCCESS) {
						goto common_nvm_program_nvm_cleanup;
					}


					// Keep the backup MAC
					status = common_nvm_keep_mac(NVM_OFFSET_PATH(manuf_info.port_hw_config[port_no].backup_mac_lower, curr_path), byte_cnt, buf, big_endian);
					if (status != COMMON_NVM_STATUS_SUCCESS) {
						goto common_nvm_program_nvm_cleanup;
					}
				}
			}
			need_manuf_crc = 1;
		}

		// Incase -skip_lic was set (and not -force_lic), keep the current license keys and do not update them
		if (lic_prg == 0) {
			// Go through the devices ports
			for (port_no = 0; port_no < PORT_MAX; port_no++) {
				// Keep the manufacture and upgrade license keys
				status = common_nvm_keep_license(port_no, byte_cnt, buf, big_endian);
				if (status != COMMON_NVM_STATUS_SUCCESS) {
					goto common_nvm_program_nvm_cleanup;
				}
			}
		}

		// do not force the iscsi boot cfg (or cfg2) block update. Feature Description: CQ#30356
		if (ibcfg_prg == 0) {
			status = common_nvm_keep_iscsi_fcoe_boot_config(byte_cnt, buf, big_endian, 1);
			if (status != COMMON_NVM_STATUS_SUCCESS) {
				goto common_nvm_program_nvm_cleanup;
			}
		}

		// do not force the feb boot cfg (or cfg2) block update. CQ#58431
		if (febcfg_prg == 0) {
			status = common_nvm_keep_iscsi_fcoe_boot_config(byte_cnt, buf, big_endian, 0);
			if (status != COMMON_NVM_STATUS_SUCCESS) {
				goto common_nvm_program_nvm_cleanup;
			}
		}

		for (curr_path = 0; curr_path < num_paths; ++curr_path) {
			/* Recompute the CRC for the manuf block */
			nvm_offset = NVM_OFFSET_PATH(manuf_info.crc, curr_path);
			if (byte_cnt > nvm_offset) {
				/* This programming has long enough buffer to hold the CRC.
							* So, perform CRC check here. */
				crc32 = 0xffffffff;
				common_nvm_compute_crc((u8_t *)&(buf[NVM_OFFSET_PATH(manuf_info, curr_path)]), sizeof(manuf_info_t) - 4, big_endian, &crc32);
#ifdef FWUPGTOOL
				crc32 = BMAPI_BENDIAN_SWAP(crc32);
#endif
				crc32 = ~htonl(crc32);
				ptr32 = (u32_t *)(&(buf[nvm_offset]));
				*ptr32 = (big_endian) ? htonl(crc32) : crc32;
				need_manuf_crc = 0;
			}
		}
	}

#ifndef FWUPGTOOL
	if (handle_macp && (mac_prg == 0)) {
#else
	if ((handle_macp && (mac_prg == 0)) ||
		(skip_cfg_prg != 0)) {
#endif // !FWUPGTOOL
		/* remember if before programing new nvm the mac partition
		   image existed, and if so: store its values.
		   (required here incase offset != 0 CQ48188, CQ52144) */

		/* search and save the NIC partition block */
		common_nvm_nvm_fill_mac_partition_info(&macp_image, &macp_image_found);
	}

	start_ptr =(u32_t *)buf;

	if (cal_crc) {
		crc32 = 0xffffffff;
		common_nvm_compute_crc((u8_t *)buf, byte_cnt, big_endian, &crc32);
	}

	/* Read buffer */
	if (total_byte_ptr) *total_byte_ptr = 0;


	/*adjust complete buffer data at once */
	ptr32 =(u32_t *)buf;
	for (idx = 0; idx < byte_cnt; idx += 4, ptr32++) {
		data32 = (big_endian) ? ntohl(*ptr32): *ptr32;
		*ptr32 = data32 ;
	}

	/* Save the original value as it may be part of the next data block */
	save_val = *ptr32;
	if (cal_crc) {
		*ptr32 = ~BYTE_SWAP(crc32); /* write CRC at the end of the buffer. make sure buffer had 4 empty locations at the end */
		byte_cnt += 4;
	}

#ifndef FWUPGTOOL
	sc_save_cursor();
#endif

	if (byte_cnt > block_size) {
		// Write to NVM in 1K byte blocks and show progress
		for (idx = 0; idx <= byte_cnt - block_size; idx += block_size) {
			status = common_nvm_write_nvram(offset + idx, (u32_t *)(buf + idx), block_size);
			if (status != COMMON_NVM_STATUS_SUCCESS) {
				return COMMON_NVM_STATUS_FAILURE;
			} else {
#ifndef FWUPGTOOL
				// in FWUPG tool, we does not display the text message of progress...
				sprintf(msg, "Programmed %02d percent (out of %d bytes)", (idx * 100 / byte_cnt), byte_cnt);
				sc_print(msg, sizeof(msg));
				sc_restore_cursor();
#endif
			}

		}
		// write the leftover of the last block to NVM
		status = common_nvm_write_nvram(offset + idx, (u32_t *)(buf + idx), byte_cnt % block_size);
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			return COMMON_NVM_STATUS_FAILURE;
		} else {
#ifndef FWUPGTOOL
			sprintf(msg, "Programmed 100 percent (out of %d bytes)\n", byte_cnt);
			sc_print(msg, sizeof(msg));
#endif
		}

	} else {

		status = common_nvm_write_nvram(offset, start_ptr, byte_cnt);
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			return COMMON_NVM_STATUS_FAILURE;
		}
	}



	for (curr_path = 0; curr_path < num_paths; ++curr_path) {
		if (need_manuf_crc) {
			/* Programming buffer wasn't long enough to have CRC. So, compute
					* CRC here instead. */
			common_nvm_compute_image_crc(NVM_OFFSET_PATH(manuf_info, curr_path), sizeof(manuf_info_t), &crc32, NULL);
			common_nvm_write_nvram(NVM_OFFSET_PATH(manuf_info.crc, curr_path), &crc32, sizeof(crc32));
		}

		if (need_feature_crc) {
			/* Programming buffer wasn't long enough to have CRC. So, compute
			* CRC here instead. */
			common_nvm_compute_image_crc(NVM_OFFSET_PATH(feature_info, curr_path), sizeof(feature_info_t), &crc32, NULL);
			common_nvm_write_nvram(NVM_OFFSET_PATH(feature_info.crc, curr_path), &crc32, sizeof(crc32));
		}
	}
	/* Restore the original value as it may be part of the next data block */
	*ptr32 = save_val;

	/* Restore the original buffer as it might have been swapped from above */
	ptr32 = start_ptr;

	/* Restore the byte count that was used during endianess adjustments */
	if (cal_crc) {
		byte_cnt -= 4;
	}
	for (idx = 0; idx < byte_cnt; idx += 4, ptr32++) {
		data32 = (big_endian) ? ntohl(*ptr32): *ptr32;
		*ptr32 = data32 ;
	}


	/* Restore the real byte count for correct reporting */
	if (cal_crc) {
		byte_cnt += 4;
	}

	if (status != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "NVM write failed.");
		status = COMMON_NVM_STATUS_FAILURE;
		goto common_nvm_program_nvm_cleanup;
	} else {
		if (total_byte_ptr) *total_byte_ptr += byte_cnt;

		if (cal_crc) byte_cnt -= 4;
	}

	/* if -force_mac was not set, perform special handling for mac partition
	   image */
#ifdef FWUPGTOOL
	if (skip_cfg_prg != 0) {
		// 'config' is NOT specified in 'restorenvram' command...
		// The NIC-partition block will be preserved...
		// But, don't write it if mac-part is not previously existing.
		if (macp_image_found) if (common_nvm_write_mac_partition_data(macp_image_found, macp_image) !=
								  COMMON_NVM_STATUS_SUCCESS) return COMMON_NVM_STATUS_FAILURE;

	} else if (handle_macp && (mac_prg == 0)) {
#else

	if (handle_macp && (mac_prg == 0)) {
#endif
	// With 'config', but no 'mac' option...
	// Handle the same way as ediag does...
	common_nvm_handle_mac_partition_data(macp_image_found, macp_image);
}

/*
printf("program_nvm: buf=%p; byte_cnt=%d; offset=%x; bytes_prg=%d\n",
		buf, byte_cnt, offset, byte_cnt);
*/

status = COMMON_NVM_STATUS_SUCCESS;

common_nvm_program_nvm_cleanup:
common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)tmp_buf);
tmp_buf = NULL;

return status;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_update_mba_hdr --
 *
 *     Update MBA header in buffer before programming into NVM
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     The buffer array content is changed.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_update_mba_hdr(
											  u8_t * buffer,
											  u16_t vendor_id,
											  u16_t device_id
											  ) {
	u32_t mba_img_len, rom_len, i, chksum, hdr_vendor_id, hdr_device_id;
	exp_rom_hdr_t *exp_rom_ptr;
	pci_rom_hdr_t *pci_rom_ptr;
	u8_t last_image = FALSE;
	u8_t code_type;
	u8_t *ptr8;

	ptr8 = buffer;

	while (!last_image) {
		exp_rom_ptr =(exp_rom_hdr_t *)ptr8;

		/* Sanity check */
		if (BMAPI_BENDIAN_SWAP_S(exp_rom_ptr->Signature) != ROM_HEADER_SIG) {

			printf("Error: Invalid Expansion ROM (incorrect header signature).\n");
			return COMMON_NVM_STATUS_FAILURE;
		}

		mba_img_len = exp_rom_ptr->Size * 512;

		if (BMAPI_BENDIAN_SWAP_S(exp_rom_ptr->PCIdsOffset) > 0) {
			pci_rom_ptr =(pci_rom_hdr_t *)( ptr8 + BMAPI_BENDIAN_SWAP_S(exp_rom_ptr->PCIdsOffset));

			rom_len = BMAPI_BENDIAN_SWAP_S(pci_rom_ptr->ImageLength)* 0x200;
			last_image = pci_rom_ptr->Indicator & PCI_IND_LAST_IMAGE;
			code_type = pci_rom_ptr->CodeType;

			hdr_vendor_id = BMAPI_BENDIAN_SWAP_S(pci_rom_ptr->VendorID);
#ifndef EDEBUG
			hdr_device_id = (BMAPI_BENDIAN_SWAP_S(pci_rom_ptr->DeviceID)) << 16;
#else
			hdr_device_id = BMAPI_BENDIAN_SWAP_S(pci_rom_ptr->DeviceID);
#endif

			/* Validate vendor/device id of image */
			if ((hdr_vendor_id != BROADCOM_VID) ||
				((hdr_device_id != CHIP_NUM_5710) && (hdr_device_id != CHIP_NUM_5711) &&
				 (hdr_device_id != CHIP_NUM_5711E) &&
				 (hdr_device_id != CHIP_NUM_5712) && (hdr_device_id != CHIP_NUM_5712E) &&
				 (hdr_device_id != CHIP_NUM_5713) && (hdr_device_id != CHIP_NUM_5713E) &&
				 (hdr_device_id != CHIP_NUM_57800) && (hdr_device_id != CHIP_NUM_57810) &&
				 (hdr_device_id != CHIP_NUM_57800_MF) && (hdr_device_id != CHIP_NUM_57840_MF) &&
				 (hdr_device_id != CHIP_NUM_57810_MF) &&
				 (hdr_device_id != CHIP_NUM_57840_4_10) && (hdr_device_id != CHIP_NUM_57840_2_20) &&
				 (hdr_device_id != CHIP_NUM_57840_OBSOLETE) && (hdr_device_id != CHIP_NUM_57840_MF_OBSOLETE) &&
				 (hdr_device_id != CHIP_NUM_57811) && (hdr_device_id != CHIP_NUM_57811_MF) &&
				 (hdr_device_id != CHIP_NUM_57811_VF))) {

				printf("Error: Unknown image with DID 0x%04x & VID 0x%04x\n", pci_rom_ptr->DeviceID, pci_rom_ptr->VendorID);
				return COMMON_NVM_STATUS_FAILURE;
			}

			if ((BMAPI_BENDIAN_SWAP_S(pci_rom_ptr->VendorID) != vendor_id) ||
				(BMAPI_BENDIAN_SWAP_S(pci_rom_ptr->DeviceID) != device_id)) {
				printf("Updating PCI ROM header with Vendor ID = 0x%04x Device ID = 0x%04x\n",
					   vendor_id, device_id);
				pci_rom_ptr->VendorID = BMAPI_BENDIAN_SWAP_S(vendor_id);
				pci_rom_ptr->DeviceID = BMAPI_BENDIAN_SWAP_S(device_id);
				if (code_type == PCI_CODE_X86) {
					/* Recalculate the checksum */
					chksum = 0;
					exp_rom_ptr->Cksum = 0;
					for (i = 0; i < mba_img_len; i++) {
						chksum += ptr8[i];
					}

					exp_rom_ptr->Cksum = (0 - chksum) & 0xff; // exp_rom_ptr->Cksum is u8_t
				}
			}
		} else {
			printf("Error: ROM hdr exists, but no ROM image\n");
			return COMMON_NVM_STATUS_FAILURE;
		}
		ptr8 += rom_len;
	}
	return COMMON_NVM_STATUS_SUCCESS;
}


/*
 *------------------------------------------------------------------------
 * nvm_find_dir_entry --
 *
 *     Find the entry from the directory structure
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if success
 *
 * Side effects:
 *     Entry index is based on the image type (one-to-one mapping)
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_nvm_find_dir_entry(
												  IN   u32_t image_type,
												  OUT  u32_t *entry_idx,
												  OUT  u32_t *start_address   // if NULL,  do not assign into this output
												  ) {
	u32_t idx, value;
	common_nvm_status_t status, rc;

	if (image_type >= CODE_IMAGE_TYPE_MAX) {
		if ((image_type == CODE_IMAGE_TYPE_BC1) && (start_address != NULL)) {
			// extract start address from bootstrap
			common_nvm_status_t res = common_nvm_read_nvram(NVM_OFFSET(bootstrap.code_start_addr), start_address, sizeof(*start_address));
			if (res != COMMON_NVM_STATUS_SUCCESS) {
				printf("warning: could not read BC1 previous NVM start address from bootstrap. version validation might fail.");
			}
		}
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* Find dir entry (0-15) for images in dir
	   (not in the new extended dir) */
	rc = COMMON_NVM_STATUS_FAILURE;
	*entry_idx = CODE_ENTRY_MAX;

	for (idx = 0; idx < CODE_ENTRY_MAX; idx++) {
		status = common_nvm_read_nvram(NVM_OFFSET(dir.code[idx].code_attribute), &value, sizeof(value));
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			break;
		} else if ((value & CODE_IMAGE_LENGTH_MASK) == 0) {
			continue;
		} else if ((value & CODE_IMAGE_TYPE_MASK) == image_type) {
			*entry_idx = idx;
			if (start_address != NULL) {
				u32_t dummy_code_len, dummy_type;
				common_nvm_status_t res = common_nvm_nvm_get_dir_info(idx, start_address, &dummy_code_len, NULL, &dummy_type, NULL);
				if (res != COMMON_NVM_STATUS_SUCCESS) {
					printf("warning: could not read image previous NVM start address from bootstrap. version validation might fail.");
				}
			}
			rc = COMMON_NVM_STATUS_SUCCESS;
			break;
		}
	}
	return rc;
}

/*
 *------------------------------------------------------------------------
 * find_empty_entry --
 *
 *     Find an empty entry from the directory structure
 *
 * Results:
 *     0 if success
 *
 * Side effects:
 *     None
 *------------------------------------------------------------------------
 */
u32_t common_nvm_find_empty_entry(
								  u32_t * entry_idx
								  ) {
	u32_t rc, idx, value;
	common_nvm_status_t status;

	rc = -1;
	for (idx = 0; idx < CODE_ENTRY_MAX; idx++) {
		status = common_nvm_read_nvram(NVM_OFFSET(dir.code[idx].code_attribute),
									   &value, sizeof(value));
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			break;
		} else if ((value & CODE_IMAGE_LENGTH_MASK) == 0) {
			*entry_idx = idx;
			rc = 0;
			break;
		}
	}
	return rc;
}

/*
 *------------------------------------------------------------------------
 * nvm_get_dir_info --
 *
 *     Get information about the directory entry
 *
 * Results:
 *     0 if success, -1 if invalid entry_idx or NVM read fails
 *
 * Side effects:
 *     None
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_nvm_get_dir_info(
												u32_t entry_idx,
												u32_t *nvm_addr_ptr,
												u32_t *byte_cnt_ptr,
												u32_t *sram_addr_ptr,
												u32_t *image_type_ptr,
												u32_t *agent_ptr
												) {
	u32_t entry_addr, value32;
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;

	/* Go to the entry. */
	if (entry_idx >= CODE_ENTRY_MAX) {
		return COMMON_NVM_STATUS_NVM_INVAL_DIR;
	}
	entry_addr = NVM_OFFSET(dir.code[entry_idx]);
	//entry_addr += entry_idx * sizeof(code_entry_t);

	/* Check the address */
	if (entry_addr & 0x3) {
		DbgMessage(NULL, FATAL, "Illegal entry address (0x%x) for entry %d\n", entry_addr, entry_idx);
#ifndef FWUPGTOOL
		DbgBreak();
#else
		return COMMON_NVM_STATUS_FAILURE;
#endif // !FWUPGTOOL
	}

	if (sram_addr_ptr) {
		status |= common_nvm_read_nvram(entry_addr, sram_addr_ptr, sizeof(u32_t));
	}
	if (nvm_addr_ptr) {
		status |= common_nvm_read_nvram(entry_addr + 8, nvm_addr_ptr, sizeof(u32_t));
	}

	status |= common_nvm_read_nvram(entry_addr + 4, &value32, sizeof(value32));
	if (status != COMMON_NVM_STATUS_SUCCESS) {
		return COMMON_NVM_STATUS_FAILURE;
	}

	if (agent_ptr) *agent_ptr = value32 & CODE_IMAGE_AGENT_TYPE_MASK;
	if (image_type_ptr) *image_type_ptr = value32 & CODE_IMAGE_TYPE_MASK;
	if (byte_cnt_ptr) *byte_cnt_ptr = value32 & CODE_IMAGE_LENGTH_MASK;


	/* Final screening */
	value32 &= CODE_IMAGE_LENGTH_MASK;

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * update_dir_entry --
 *
 *     Update the entry into the directory structure
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if success
 *
 * Side effects:
 *     Content in NVM will be updated.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_update_dir_entry(
												u32_t entry_idx,
												u32_t nvm_addr,
												u32_t byte_cnt,
												u32_t sram_addr,
												u32_t image_type,
												u32_t agent_type
												) {
	u32_t entry_addr;
	u32_t entry_info[3];
	//u32_t sram_start_addr = sram_addr, code_attrib;
	common_nvm_status_t status = 0;
	u32_t stored, computed;

	/* Go to the entry. */
	if (entry_idx >= CODE_ENTRY_MAX) {
		DbgMessage(NULL, FATAL, "Invalid NVM directory index.\n");
		return COMMON_NVM_STATUS_FAILURE;
	}
	entry_addr = NVM_OFFSET(dir.code[entry_idx]);
	//entry_addr += entry_idx * sizeof(code_entry_t);

	entry_info[0]= sram_addr;
	entry_info[1]= (image_type & CODE_IMAGE_TYPE_MASK) |
	(agent_type & CODE_IMAGE_AGENT_TYPE_MASK) |
	(byte_cnt & CODE_IMAGE_LENGTH_MASK);
	entry_info[2]= nvm_addr;


	status = common_nvm_write_nvram(entry_addr, entry_info, 12);
	if (status != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "NVM write failed.\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* Update the CRC at the end of the directory block */
	/* Check directory block */
	status = common_nvm_compute_image_crc(NVM_OFFSET(dir), sizeof(dir_t),
										  &computed, &stored);
	if (computed != stored) {
		status |= common_nvm_write_nvram(NVM_OFFSET(dir.crc), &computed, sizeof(computed));
	}
	if (status != 0) {
		DbgMessage(NULL, FATAL, "Update directory CRC failed.");
		return COMMON_NVM_STATUS_FAILURE;
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

common_nvm_status_t common_nvm_read_NVM_or_image(
												 u8_t                read_flag,
												 common_nvm_virt_addr_t    base_addr,
												 u32_t               offset,
												 u32_t *val_p,
												 u32_t               byte_cnt
												 ) {
	if (read_flag == READ_VERSION_FROM_NVM) {
		return common_nvm_read_nvram((u32_t)(base_addr + offset), val_p, byte_cnt);

	} else if (read_flag == READ_VERSION_FROM_IMAGE) {
		u32_t *image =(u32_t *)(uintptr_t)base_addr;
		u32_t i;

		for (i = 0; i < byte_cnt / sizeof(u32_t); i++) {
			// Image file is in LE format, swap them to BE format in BE machine...
			val_p[i]= BMAPI_BENDIAN_SWAP(image[(offset / 4) + i]);
		}
		return COMMON_NVM_STATUS_SUCCESS;
	} else {
		DbgMessage(NULL, FATAL, "Invalid version read flag\n");
#ifndef FWUPGTOOL
		DbgBreak();
#endif // !FWUPGTOOL
		return COMMON_NVM_STATUS_FAILURE; // just to avoid compilation warning
	}
}


#ifndef FWUPGTOOL
#define CHECK_VERSION_SCAN_RESULT(num_of_params) \
    if (result != num_of_params) {\
    DbgMessage(NULL, FATAL, "invalid version format for this image type: %s\n",image_name);\
        DbgBreak();\
    }
#else
#define CHECK_VERSION_SCAN_RESULT(num_of_params) \
    if (result != num_of_params) {\
        printf ("File has an invalid version string. Upgrade aborted for image: %s\n",image_name);\
        return COMMON_NVM_STATUS_FAILURE;\
    }
#endif // !FWUPGTOOL

common_nvm_status_t common_nvm_image_version_validation(
														const char *image_name,
														char *old_version,
														char *new_version,
														u8_t *same_image
														) {

	u64_t old_ver_val=0;
	u64_t new_ver_val=0;

	u32_t result;
#ifndef FWUPGTOOL
	u32_t tempOld[4]={ 0, 0, 0, 0 };
	u32_t tempNew[4]={ 0, 0, 0, 0 };
	int backwardVer[3]={ 0, 0, 0 };         //localy used with sscanf so can be int (instead of u32_t)
	u64_t backward_ver;
	common_nvm_dev_stat_t dev_stat;
#else
	// to eliminate the compile warnings
	int tempOld[4]={ 0, 0, 0, 0 };
	int tempNew[4]={ 0, 0, 0, 0 };
	int backwardVer[3]={ 0, 0, 0 };
	u64_t backward_ver;
	char *chip_rev;
	u32_t device_id;
	Tcl_Obj *obj;
	*same_image = 0;
#endif // !FWUPGTOOL

	if (strcmp(image_name, NVM_IMAGE_NAME_IPMI) == 0) {
		if (old_version[5] == 'D')  // special patch for 'D' prefix to the version number
			result = sscanf(old_version, "IPMI D%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		else result = sscanf(old_version, "IPMI %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(3)
		if (new_version[5] == 'D')  // special patch for 'D' prefix to the version number
			result = sscanf(new_version, "IPMI D%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		else result = sscanf(new_version, "IPMI %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		CHECK_VERSION_SCAN_RESULT(3)
	} else if (strcmp(image_name, NVM_IMAGE_NAME_L2T) == 0) {
		result = sscanf(old_version, "L2T %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(3)
		result = sscanf(new_version, "L2T %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		CHECK_VERSION_SCAN_RESULT(3)
	} else if (strcmp(image_name, NVM_IMAGE_NAME_L2U) == 0) {
		result = sscanf(old_version, "L2U %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(3)
		result = sscanf(new_version, "L2U %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		CHECK_VERSION_SCAN_RESULT(3)
	} else if (strcmp(image_name, NVM_IMAGE_NAME_L2C) == 0) {
		result = sscanf(old_version, "L2C %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(3)
		result = sscanf(new_version, "L2C %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		CHECK_VERSION_SCAN_RESULT(3)
	} else if (strcmp(image_name, NVM_IMAGE_NAME_L2X) == 0) {
		result = sscanf(old_version, "L2X %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(3)
		result = sscanf(new_version, "L2X %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		CHECK_VERSION_SCAN_RESULT(3)
	} else if ((strcmp(image_name, NVM_IMAGE_NAME_NCSI_CMN) == 0) || (strcmp(image_name, NVM_IMAGE_NAME_NCSI_ELIB) == 0)) {
		char dummy;
		result = sscanf(old_version, "NCSI%c %x.%x.%x", &dummy, &tempOld[0], &tempOld[1], &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(4)
		result = sscanf(new_version, "NCSI%c %x.%x.%x", &dummy, &tempNew[0], &tempNew[1], &tempNew[2]);
		CHECK_VERSION_SCAN_RESULT(4)
	} else if (strcmp(image_name, NVM_IMAGE_NAME_BC1) == 0) {
		if (old_version[0] == 's') { // special patch for 's2p' version
			result = sscanf(old_version, "s2p %x.%x.%x.", &tempOld[0], &tempOld[1], &tempOld[2]);
		} else if (old_version[4] == 'D') { // special patch for 'D' prefix to the version number
			if (old_version[0] == 'B') result = sscanf(old_version, "BC1 D%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			else result = sscanf(old_version, "bc1 D%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		} else if ((old_version[0] == 'M') && (old_version[1] == 'F') && (old_version[2] == 'W')) {
			// special check incase MFW image exist
			result = sscanf(old_version, "MFW1 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
			if (result != 4) {
				result = sscanf(old_version, "MFW1 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			}
		} else {
			if (old_version[0] == 'B') result = sscanf(old_version, "BC1 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			else result = sscanf(old_version, "bc1 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		}
		CHECK_VERSION_SCAN_RESULT(3)
		if (new_version[4] == 'D') { // special patch for 'D' prefix to the version number
			if (new_version[0] == 'B') result = sscanf(new_version, "BC1 D%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			else result = sscanf(new_version, "bc1 D%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		} else {
			if (new_version[0] == 'B') result = sscanf(new_version, "BC1 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			else result = sscanf(new_version, "bc1 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		}
		CHECK_VERSION_SCAN_RESULT(3)
	} else if (strcmp(image_name, NVM_IMAGE_NAME_BC2) == 0) {
		if (old_version[4] == 'D') { // special patch for 'D' prefix to the version number
			if (old_version[0] == 'B') result = sscanf(old_version, "BC2 D%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			else result = sscanf(old_version, "bc2 D%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		} else {
			if (old_version[0] == 'B') result = sscanf(old_version, "BC2 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			else result = sscanf(old_version, "bc2 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		}
		CHECK_VERSION_SCAN_RESULT(3)
		if (new_version[4] == 'D') { // special patch for 'D' prefix to the version number
			if (new_version[0] == 'B') result = sscanf(new_version, "BC2 D%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			else result = sscanf(new_version, "bc2 D%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		} else {
			if (new_version[0] == 'B') result = sscanf(new_version, "BC2 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			else result = sscanf(new_version, "bc2 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		}
		CHECK_VERSION_SCAN_RESULT(3)
	} else if (strcmp(image_name, NVM_IMAGE_NAME_SWIM1) == 0) {
		result = sscanf(old_version, "SWIM1 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "SWIM1 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "SWIM1 %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "SWIM1 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
	} else if (strcmp(image_name, NVM_IMAGE_NAME_SWIM2) == 0) {
		result = sscanf(old_version, "SWIM2 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "SWIM2 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "SWIM2 %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "SWIM2 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
	} else if (strcmp(image_name, NVM_IMAGE_NAME_SWIM3) == 0) {
		result = sscanf(old_version, "SWIM3 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "SWIM3 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "SWIM3 %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "SWIM3 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
	} else if (strcmp(image_name, NVM_IMAGE_NAME_SWIM4) == 0) {
		result = sscanf(old_version, "SWIM4 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "SWIM4 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "SWIM4 %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "SWIM4 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
	} else if (strcmp(image_name, NVM_IMAGE_NAME_SWIM5) == 0) {
		result = sscanf(old_version, "SWIM5 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "SWIM5 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "SWIM5 %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "SWIM5 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
	} else if (strcmp(image_name, NVM_IMAGE_NAME_SWIM6) == 0) {
		result = sscanf(old_version, "SWIM6 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "SWIM6 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "SWIM6 %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "SWIM6 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
	} else if (strcmp(image_name, NVM_IMAGE_NAME_SWIM7) == 0) {
		result = sscanf(old_version, "SWIM7 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "SWIM7 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "SWIM7 %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "SWIM7 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
	} else if (strcmp(image_name, NVM_IMAGE_NAME_SWIM8) == 0) {
		result = sscanf(old_version, "SWIM8 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "SWIM8 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "SWIM8 %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "SWIM8 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
	} else if ((strcmp(image_name, NVM_IMAGE_NAME_ISCSI_BOOT) == 0) ||
			   (strcmp(image_name, NVM_IMAGE_NAME_ISCSI_CPRG) == 0) ||
			   (strcmp(image_name, NVM_IMAGE_NAME_FCOE_BOOT) == 0)) {
		if (old_version[0] == 'v') {
			result = sscanf(old_version, "v%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		} else {
			// CQ53066 Accept old FCOE versions which are in format x.x.x (without 'v' prefix)
			result = sscanf(old_version, "%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
		}
		CHECK_VERSION_SCAN_RESULT(3)
		if (new_version[0] == 'v') {
			result = sscanf(new_version, "v%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		} else {
			// CQ53066 Accept old FCOE versions which are in format x.x.x (without 'v' prefix)
			result = sscanf(new_version, "%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
		}
		CHECK_VERSION_SCAN_RESULT(3)
	} else if (
			   (strcmp(image_name, NVM_IMAGE_NAME_ISCSI_BOOT_CFG) == 0) ||
			   (strcmp(image_name, NVM_IMAGE_NAME_ISCSI_BOOT_CFG2) == 0) ||
			   (strcmp(image_name, NVM_IMAGE_NAME_ISCSI_BOOT_CFG3) == 0) ||
			   (strcmp(image_name, NVM_IMAGE_NAME_ISCSI_BOOT_CFG4) == 0) ||
			   (strcmp(image_name, NVM_IMAGE_NAME_FCOE_BOOT_CFG) == 0)) {
		result = sscanf(old_version, "v%x.%x", &tempOld[1], &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(2)
		result = sscanf(new_version, "v%x.%x", &tempNew[1], &tempNew[2]);
		CHECK_VERSION_SCAN_RESULT(2)
		tempOld[0]= tempNew[0]= 0;
	} else if (strcmp(image_name, NVM_IMAGE_NAME_NIC_PARTITION_CFG) == 0) {
		result = sscanf(old_version, "Rev%x", &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(1)
		result = sscanf(new_version, "Rev%x", &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(1)
		tempOld[0]= tempNew[0]= tempOld[1]= tempNew[1]= 0;
	} else if (strcmp(image_name, NVM_IMAGE_NAME_NIV_CFG) == 0) {
		result = sscanf(old_version, "Rev%x", &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(1)
		result = sscanf(new_version, "Rev%x", &tempOld[2]);
		CHECK_VERSION_SCAN_RESULT(1)
		tempOld[0]= tempNew[0]= tempOld[1]= tempNew[1]= 0;
	} else if (strcmp(image_name, NVM_IMAGE_NAME_CFG_EXTENDED_SHARED) == 0) {
		result = sscanf(old_version, "N/A");
		CHECK_VERSION_SCAN_RESULT(1)
		result = sscanf(new_version, "N/A");
		CHECK_VERSION_SCAN_RESULT(1)
		tempOld[0]= tempNew[0]= tempOld[1]= tempNew[1]= 0;
	} else if (strcmp(image_name, NVM_IMAGE_NAME_MBA) == 0) {
		// Do nothing.
		// MBA version check has different version validation proc
		// since it might include combined images
	} else if (strcmp(image_name, NVM_IMAGE_NAME_MFW1) == 0) {
		result = sscanf(old_version, "MFW1 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "MFW1 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "MFW1 %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "MFW1 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
	} else if ((strcmp(image_name, NVM_IMAGE_NAME_MFW2) == 0) || (strcmp(image_name, NVM_IMAGE_NAME_MFW2_A) == 0)) {
		result = sscanf(old_version, "MFW2 %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "MFW2 %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "MFW2 %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "MFW2 %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
	} else if ((strcmp(image_name, NVM_IMAGE_NAME_E3_WC) == 0) ||
			   (strcmp(image_name, NVM_IMAGE_NAME_E3_WCV2) == 0)) {
		// For now, don't check E3_WC version since is N/A
		return COMMON_NVM_STATUS_SUCCESS;
	} else if ((strcmp(image_name, NVM_IMAGE_NAME_E3_PCIE) == 0) ||
			   (strcmp(image_name, NVM_IMAGE_NAME_E3_PCIEV2) == 0)) {
		// For now, don't check E3_PCIE version since is N/A
		return COMMON_NVM_STATUS_SUCCESS;
	} else if (strcmp(image_name, NVM_IMAGE_NAME_OCNVM) == 0) {
		// For now, don't check OCNVM version since is N/A.
		// Change here is version is defined later
		result = sscanf(old_version, "OCNVM %x.%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2], &tempOld[3]);
		if (result != 4) {
			result = sscanf(old_version, "OCNVM %x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		result = sscanf(new_version, "OCNVM %x.%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2], &tempNew[3]);
		if (result != 4) {
			result = sscanf(new_version, "OCNVM %x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		}
		// return COMMON_NVM_STATUS_SUCCESS;
	} else if (strcmp(image_name, NVM_IMAGE_NAME_CCM) == 0) {
		if (old_version[0] == 'v') {
			// Support new CCM version numbering
			result = sscanf(old_version, "v%x.%x.%x", &tempOld[0], &tempOld[1], &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		} else {
			// Support old CCM version numbering
			result = sscanf(old_version, "Rev%x", &tempOld[2]);
			CHECK_VERSION_SCAN_RESULT(1)
			tempOld[0]= tempOld[1]= 0;
		}

		if (new_version[0] == 'v') {
			// Support new CCM version numbering
			result = sscanf(new_version, "v%x.%x.%x", &tempNew[0], &tempNew[1], &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(3)
		} else {
			// Support old CCM version numbering
			result = sscanf(new_version, "Rev%x", &tempNew[2]);
			CHECK_VERSION_SCAN_RESULT(1)
			tempNew[0]= tempNew[1]= 0;
		}
	}


	/*
	debug code:
	printf ("old_version = %s, new_version = %s,  scanf result = %x, \n",old_version,new_version,result);
	for (result=0;result<3;result++){
		printf ("tempOld[%x] = %x,  ",result,tempOld[result]);
		printf ("tempNew[%x] = %x,  ",result,tempNew[result]);
	}
	*/

	old_ver_val = tempOld[3]+ ((u64_t)tempOld[2]<<8) + ((u64_t)tempOld[1]<<16) + ((u64_t)tempOld[0]<<24);
	new_ver_val = tempNew[3]+ ((u64_t)tempNew[2]<<8) + ((u64_t)tempNew[1]<<16) + ((u64_t)tempNew[0]<<24);

	/* Version BACKWARD_COMPATIBLE_VERSION 4.2.0 and above do not support chip revision A0 */
	sscanf(BACKWARD_COMPATIBLE_VERSION, "%d.%d.%d", &backwardVer[0], &backwardVer[1], &backwardVer[2]);
	backward_ver = backwardVer[2]+ ((u64_t)backwardVer[1]<<16) + ((u64_t)backwardVer[0]<<32);
#ifndef FWUPGTOOL
	common_nvm_get_info(&dev_stat);

	if (is_e1(globalInterp) && (new_ver_val >= backward_ver) && (strcmp(dev_stat.asic_rev, "A0") == 0)) {
		printf("ERROR: Bootcode version 4.2.0 and above do not support chip revision A0\n");
		printf("Upgrade aborted for image %s\n", image_name);
		return COMMON_NVM_STATUS_FAILURE;
	}
#else
	// The ENV_CHIP_REV was set in Bnx2Helper::SetSelectedNIC()
	chip_rev = Tcl_GetVar2(gpINx2Nvm->m_tcl_interp, TCL_ENV, ENV_CHIP_REV, 0);
	obj = Tcl_GetVar2Ex(gpINx2Nvm->m_tcl_interp, TCL_ENV, ENV_DID, 0);
	Tcl_GetIntFromObj(gpINx2Nvm->m_tcl_interp, obj, (int *)&device_id);

	if ((device_id == DEVICE_ID_E1) && (new_ver_val >= backward_ver) && (strcmp(chip_rev, "A0") == 0)) {
		printf("ERROR: Bootcode version 4.2.0 and above do not support chip revision A0\n");
		printf("Upgrade aborted for image %s\n", image_name);
		return COMMON_NVM_STATUS_FAILURE;
	}
#endif // !FWUPGTOOL

	/* Version E2_A0_NON_SUPPORTED_VER 6.4.0 and above is not supported by on E2 chip rev A0 */
	sscanf(E2_A0_NON_SUPPORTED_VER, "%d.%d.%d", &backwardVer[0], &backwardVer[1], &backwardVer[2]);
	backward_ver = backwardVer[2]+ ((u64_t)backwardVer[1]<<16) + ((u64_t)backwardVer[0]<<32);
#ifndef FWUPGTOOL
	common_nvm_get_info(&dev_stat);

	if (is_e2(globalInterp) &&
		(new_ver_val >= backward_ver) && (strcmp(dev_stat.asic_rev, "A0") == 0)) {
		printf("ERROR: Bootcode version 6.4.0 and above do not support 57712 chip rev A0\n");
		printf("Upgrade aborted for image %s\n", image_name);
		return COMMON_NVM_STATUS_FAILURE;
	}
#else
	// The ENV_CHIP_REV was set in Bnx2Helper::SetSelectedNIC()
	chip_rev = Tcl_GetVar2(gpINx2Nvm->m_tcl_interp, TCL_ENV, ENV_CHIP_REV, 0);
	obj = Tcl_GetVar2Ex(gpINx2Nvm->m_tcl_interp, TCL_ENV, ENV_DID, 0);
	Tcl_GetIntFromObj(gpINx2Nvm->m_tcl_interp, obj, (int *)&device_id);

	if (((device_id == DEVICE_ID_E2_5712) || (device_id == DEVICE_ID_E2_5712E) ||
		 (device_id == DEVICE_ID_E2_5713) || (device_id == DEVICE_ID_E2_5713E)) &&
		(new_ver_val >= backward_ver) && (strcmp(chip_rev, "A0") == 0)) {
		printf("ERROR: Bootcode version 6.4.0 and above do not support 57712 chip rev A0\n");
		printf("Upgrade aborted for image %s\n", image_name);
		return COMMON_NVM_STATUS_FAILURE;
	}

#endif // !FWUPGTOOL

	if (old_ver_val < new_ver_val) {
		// version is good - newer than previous version
		return COMMON_NVM_STATUS_SUCCESS;
	}

	if (old_ver_val == new_ver_val) {
		printf("Given version(%s) is same as NVRAM version(%s). \nUpgrade aborted for image %s.\n", new_version, old_version, image_name);
		*same_image = 1;
	} else {
		printf("Given version(%s) is older than NVRAM version(%s). \nUpgrade aborted for image %s.\n", new_version, old_version, image_name);
	}

	printf("Please use \"-F\" option to force upgrade.\n");
	return COMMON_NVM_STATUS_FAILURE;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_image_mba_version_validation --
 *
 *     Special handling for MBA and its combined MBA/EFI/Fcode images:
 *     Checks all images versions and verify upgrade is indeed requested.
 *     If user attemps to downgrade versions, show warning message.
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if success
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_image_mba_version_validation(common_nvm_virt_addr_t nvm_addr, common_nvm_virt_addr_t file_buf, u8_t *same_image) {
	u32_t file_vers[ROMIMG_NUM_MAX];                // This array stores the images in input file
	u32_t nvm_vers[ROMIMG_NUM_MAX];                 // This array stores the images already in NVM
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;
	u32_t i = 0, eq_cnt = 0, first_filled_element = 0;
	char  *type_str[ROMIMG_NUM_MAX]={ 0 };
	*same_image = 0;

	// Initialize the output parsed arrays
	for (i = 0; i < ROMIMG_NUM_MAX; i++) {
		file_vers[i]= 0;
		nvm_vers[i]= 0;
	}

	// Get the combined images versions in file, output in file_vers
	status = common_nvm_get_romimg_vers(READ_VERSION_FROM_IMAGE, file_buf, file_vers, type_str);

	// Get the combined images versions in NVM, output in nvm_vers
	status |= common_nvm_get_romimg_vers(READ_VERSION_FROM_NVM, nvm_addr, nvm_vers, type_str);

	/*for(i=0; i<ROMIMG_NUM_MAX; i++)
	{
			printf("==%s: file ver 0x%x, nvm ver 0x%x==\n", type_str[i], file_vers[i], nvm_vers[i]);
	}*/

	if (status != COMMON_NVM_STATUS_SUCCESS) {
		printf("MBA upgrade error.\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	// Compare versions of each image in file (file_vers) and in NVM (nvm_vers)
	for (i = 0; i < ROMIMG_NUM_MAX; i++) {
		// If versions in file and in NVM match, increase eq_cnt counter
		if (file_vers[i] == nvm_vers[i]) {
			eq_cnt++;
			if ((file_vers[i] != 0) && (first_filled_element == 0)) {
				first_filled_element = i;
			}
		}

		// If file includes lower version than in NVM, refuse to downgrade
		if (file_vers[i] < nvm_vers[i]) {
			if (file_vers[i] != 0) {
				printf("Given version (%d.%d.%d) is older than NVRAM version(%d.%d.%d). Upgrade aborted for image %s.\n",
					   (file_vers[i] & 0xff000) >> 12,
					   (file_vers[i] & 0x0f00) >> 8,
					   file_vers[i] & 0xff,
					   (nvm_vers[i] & 0xff000) >> 12,
					   (nvm_vers[i] & 0x0f00) >> 8,
					   nvm_vers[i] & 0xff,
					   type_str[i]);
				printf("Please use \"-F\" option to force downgrade.\n");
				status = COMMON_NVM_STATUS_FAILURE;  // new MBA image has less component
				break;
			} else {
				printf("This \"nvm upgrade\" operation involves removal of %s.\n", type_str[i]);
				printf("Please use \"-F\" option to force downgrade.\n");
				status = COMMON_NVM_STATUS_FAILURE;  /* new MBA image has less component */
				break;
			}
		}

		// If all images in file include equal version as in NVM, refuse to program
		if (eq_cnt == ROMIMG_NUM_MAX) {
			*same_image = 1;
			printf("Given version (%d.%d.%d) is same as NVRAM version(%d.%d.%d). Upgrade aborted for image %s.\n",
				   (file_vers[first_filled_element] & 0xff000) >> 12,
				   (file_vers[first_filled_element] & 0x0f00) >> 8,
				   file_vers[first_filled_element] & 0xff,
				   (nvm_vers[first_filled_element] & 0xff000) >> 12,
				   (nvm_vers[first_filled_element] & 0x0f00) >> 8,
				   nvm_vers[first_filled_element] & 0xff,
				   type_str[first_filled_element]);

			printf("Please use \"-F\" option to force upgrade.\n");
			status = COMMON_NVM_STATUS_FAILURE;  /* new MBA image has less component */
			break;
		}
	}

	return status;
}

/*
 *     Checks for new version scheme in EFI image
 *
 * Returns:
 *     COMMON_NVM_STATUS_SUCCESS if success + new version scheme value
 */
static common_nvm_status_t get_new_efi_version(u8_t source, common_nvm_virt_addr_t addr, u32_t start_addr, u32_t code_img_len, u32_t *version) {
	u8_t *p_efi = NULL;
	u8_t *p_src = NULL;
	u32_t compression_type, offset, signature, efi_signature, val;
	void *Scratch = NULL;
	EFI_IMAGE_DOS_HEADER *dos_hdr;
	EFI_IMAGE_OPTIONAL_HEADER_UNION *opt_uni;
	EFI_IMAGE_OPTIONAL_HEADER32 *opt_h32;
	EFI_IMAGE_OPTIONAL_HEADER64 *opt_h64;
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;
 
	// This is EFI - try to read EFI_PCI_EXPANSION_ROM_HEADER signatures
	status = common_nvm_read_NVM_or_image(source, addr, start_addr, &signature, sizeof(signature));
	status |= common_nvm_read_NVM_or_image(source, addr, start_addr + 4, &efi_signature, sizeof(efi_signature));
	if (status != COMMON_NVM_STATUS_SUCCESS) {
		goto cleanup;
	}
	// Signature should be 0xAA55 and EFI signature should be 0x0EF1
	if (((signature & 0xffff) != 0xAA55) || (efi_signature != 0x0EF1)) {
		printf("invalid EFI image\n");
		status = COMMON_NVM_STATUS_FAILURE;
		goto cleanup;
	}
	// This is EFI_PCI_EXPANSION_ROM_HEADER - check compression
	status = common_nvm_read_NVM_or_image(source, addr, start_addr + 12, &compression_type, sizeof(compression_type));
	// Find EFI offset
	status |= common_nvm_read_NVM_or_image(source, addr, start_addr + 20, &offset, sizeof(offset));
	offset = (offset >> 16) & 0xffff;
	if (status != COMMON_NVM_STATUS_SUCCESS) {
		goto cleanup;
	}
	if (source == READ_VERSION_FROM_NVM) {
		// Reading from NVM - p_image is NULL
		p_src = (u8_t *)malloc(code_img_len);
		if (p_src == NULL) {
			printf("Can't allocate memory!\n");
			status = COMMON_NVM_STATUS_FAILURE;
			goto cleanup;
		}
		// Get EFI image into p_src
		status = common_nvm_read_NVM_or_image(source, addr, start_addr + offset, (u32_t *)p_src, code_img_len);
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			goto cleanup;
		}
	} else {
		p_src = (u8_t *)(uintptr_t)addr + start_addr + offset;
	}
	if ((compression_type & 0xffff) != 0) {
		// EFI image is compressed! uncompress it!
		u32_t  DstSize, ScratchSize;
		if (EfiGetInfo((void *)p_src, code_img_len, &DstSize, &ScratchSize) != RETURN_SUCCESS) {
			printf("Failed to uncompress EFI!\n");
			status = COMMON_NVM_STATUS_FAILURE;
			goto cleanup;
		}
		p_efi = (u8_t *)malloc(DstSize);
		if (p_efi == NULL) {
			printf("Can't allocate memory!\n");
			status = COMMON_NVM_STATUS_FAILURE;
			goto cleanup;
		}
		Scratch = malloc(ScratchSize);
		if (Scratch == NULL) {
			printf("Can't allocate memory!\n");
			status = COMMON_NVM_STATUS_FAILURE;
			goto cleanup;
		}
		if (EfiDecompress((void *)p_src, code_img_len, (void *)p_efi, DstSize, Scratch, ScratchSize) != RETURN_SUCCESS) {
			printf("Failed to uncompress EFI!\n");
			status = COMMON_NVM_STATUS_FAILURE;
			goto cleanup;
		}
	} else {
		p_efi = p_src;
	}
	// EFI image is now uncompressed
	/* Check for "MZ" header signature */
	dos_hdr = (EFI_IMAGE_DOS_HEADER*)p_efi;
	if (dos_hdr->e_magic != EFI_IMAGE_DOS_SIGNATURE) {
		printf("invalid EFI image\n");
		status = COMMON_NVM_STATUS_FAILURE;
		goto cleanup;
	}
	opt_uni = (EFI_IMAGE_OPTIONAL_HEADER_UNION*)(((u8*)dos_hdr) + dos_hdr->e_lfanew);
	/* Check for "PE__" header signature */
	if (opt_uni->Pe32.Signature != EFI_IMAGE_NT_SIGNATURE) {
		printf("invalid EFI image\n");
		status = COMMON_NVM_STATUS_FAILURE;
		goto cleanup;
	}
	opt_h32 = (EFI_IMAGE_OPTIONAL_HEADER32*)(((u8*)opt_uni) + sizeof(opt_uni->Pe32.Signature) + EFI_IMAGE_SIZEOF_FILE_HEADER);
	switch (opt_h32->Magic) {
	case EFI_IMAGE_NT_OPTIONAL_HDR32_MAGIC:
		/* Valid PE32 image */
		val = opt_h32->MajorImageVersion;
		*version = opt_h32->MinorImageVersion;
		break;
	case EFI_IMAGE_NT_OPTIONAL_HDR64_MAGIC:
		/* Valid PE64 image */
		opt_h64 = (EFI_IMAGE_OPTIONAL_HEADER64*)opt_h32;
		val = opt_h64->MajorImageVersion;
		*version = opt_h64->MinorImageVersion;
		break;
	default:
		printf("invalid EFI image\n");
		status = COMMON_NVM_STATUS_FAILURE;
		break;
	}

cleanup:
	if ((p_efi != NULL) && ((compression_type & 0xffff) != 0)) {
		free(p_efi);
	}
	if ((p_src != NULL) && (source == READ_VERSION_FROM_NVM)){
		free(p_src);
	}
	if (Scratch != NULL) {
		free(Scratch);
	}

	return status;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_get_romimg_vers --
 *
 *     Special handling for MBA and its combined MBA/EFI/Fcode images:
 *     Parse the image versions read from the input addr.
 *
 * Inputs:
 * read_source - Can be either from NVM or File
 *               READ_VERSION_FROM_NVM or READ_VERSION_FROM_IMAGE
 * addr        - NVM address of file buffer address
 * Outputs:
 * vers        - array with all images versions
 * type_str    - array with all images types
 *
 * Note: The output arrays indexes are defined in mba_image_t
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if success
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_get_romimg_vers(u8_t read_source, common_nvm_virt_addr_t addr, u32_t *vers, char **type_str) {

	u32_t idx, code_type, indicator, code_img_len, pcir_rev, dmtf_clp, pcir_start;
	u32_t val, val2, efi_type, start_addr = 0;
	u32_t cti_offset, img_len_offset, rev_offset, dtmf_offset, version = 0;
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;

	dtmf_offset         = PCIR_OFFSET(CfgCodeHdr);      /*0x18*/
	cti_offset          = PCIR_OFFSET(CodeType);        /*0x14*/
	img_len_offset      = PCIR_OFFSET(ImageLength);     /*0x10*/
	rev_offset          = PCIR_OFFSET(StructRev);       /*0xc*/


	// Run till max allowed combined images
	for (idx = 0; idx < ROMIMG_NUM_MAX; idx++) {
		// Find the offset of where the PCI structure starts
		status = common_nvm_read_NVM_or_image(read_source, addr, (start_addr + dtmf_offset), &pcir_start, sizeof(pcir_start));
		pcir_start &= 0xff;

		// Read EFI machine type
		status |= common_nvm_read_NVM_or_image(read_source, addr, (start_addr + 0x8), &efi_type, sizeof(efi_type));

		// Get code type (MBA/Fcode/EFI) and indicator (for following images)
		status |= common_nvm_read_NVM_or_image(read_source, addr, (start_addr + pcir_start + cti_offset), &val2, sizeof(val2));
		code_type = val2&0xff;
		indicator = (val2>>8)&0x80; // bit 7 clear means another image follows

		// Image length and Revision Level of Code/Data
		// Version value is at start+0x10 within PCI structure
		status |= common_nvm_read_NVM_or_image(read_source, addr, (start_addr + pcir_start + img_len_offset), &val, sizeof(val));
		// length is in units of 0x200 bytes
		code_img_len = (val&0xffff)*0x200;

		// pci data structure revision and class code
		status |= common_nvm_read_NVM_or_image(read_source, addr, (start_addr + pcir_start + rev_offset), &pcir_rev, sizeof(pcir_rev));
		pcir_rev = pcir_rev&0xff;
		if (pcir_rev == 0x03) {
			// DMTF CLP Entry Point
			status |= common_nvm_read_NVM_or_image(read_source, addr, (start_addr + pcir_start + dtmf_offset), &dmtf_clp, sizeof(dmtf_clp));
		}

		if (status != COMMON_NVM_STATUS_SUCCESS) {
			return status;
		}

		if (code_type == 0x0) { // MBA
			switch (pcir_rev) {
			case 0x00: // legacy
				vers[MBA_MBA_LEGACY_IDX]= (val >> 16) & 0xffff;
				type_str[MBA_MBA_LEGACY_IDX]= "legacy MBA";
				break;
			case 0x03: // PCI F/W 3.0
				dmtf_clp = (dmtf_clp>>16)&0xffff;
				if (dmtf_clp != 0) {
					vers[MBA_MBA_PCI3CLP_IDX]= (val >> 16) & 0xffff;
					type_str[MBA_MBA_PCI3CLP_IDX]= "PCI30_CLP MBA";
				} else {
					vers[MBA_MBA_PCI3_IDX]= (val >> 16) & 0xffff;
					type_str[MBA_MBA_PCI3_IDX]= "PCI30 MBA";
				}
				break;
			default:
				printf("invalid MBA type");
				break;
			}
		} else if (code_type == 0x1) { // Fcode
			vers[MBA_FCODE_IDX]= (val >> 16) & 0xffff;
			type_str[MBA_FCODE_IDX]= "FCODE";
		} else if (code_type == 0x03) { // EFI
										//vers[MBA_EFI_IDX] = (val >> 16) & 0xffff;
			status = get_new_efi_version(read_source, addr, start_addr, code_img_len, &version);
			if (status != COMMON_NVM_STATUS_SUCCESS)
				return status;
			if (!version) {
				/* No new version scheme use old version scheme */
				version = (val >> 16) & 0xffff;
			}
			efi_type = (efi_type >> 16) & 0xffff;
			switch (efi_type) {
			case 0x014c: // x86(IA-32)
				vers[EFI_X86_IDX]= version;
				type_str[EFI_X86_IDX]= "EFI x86";
				break;
			case 0x200: // IPF
				vers[EFI_IPF_IDX]= version;
				type_str[EFI_IPF_IDX]= "EFI IPF";
				break;
			case 0xEBC: // EBC
				vers[EFI_EBC_IDX]= version;
				type_str[EFI_EBC_IDX]= "EFI EBC";
				break;
			case 0x8664: // x64
				vers[EFI_X64_IDX]= version;
				type_str[EFI_X64_IDX]= "EFI x64";
				break;
			default:
				printf("invalid EFI type");
				break;
			}
		}
		if (indicator == 0x80) break;
		else {
			start_addr += code_img_len;
		}
	}

	return status;
}


/*
 *------------------------------------------------------------------------
 * common_nvm_fw_ver_string --
 *
 *     Returns the version string of a firmware image.
 * Input:
 *     nvm_addr is the "virtual" one in case of the buffered flash.
 *     ptr must have a preallocated storage (32 bytes minimum).
 * Output:
 *     ptr will contain a NULL terminated string (up to 31 char + NULL).
 *     If no string found or error, a NULL string is returned.
 *------------------------------------------------------------------------
 */
void common_nvm_fw_ver_string(
							  common_nvm_virt_addr_t nvm_addr,
							  const char *name,
							  char *str_ptr,
							  u8_t read_source
							  ) {
	u32_t  idx, val, ret_val, offset, str_idx, nvm_idx;
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;

	str_ptr[0]= 0;
	str_ptr[MAX_VER_STR_SIZE - 1]= 0;

	if (strcmp(name, NVM_IMAGE_NAME_MBA) == 0) {
		u32_t   vers[ROMIMG_NUM_MAX], str_len;
		char *type_str[ROMIMG_NUM_MAX]={ 0 };
		char    ver_str[MAX_VER_STR_SIZE];

		for (idx = 0; idx < ROMIMG_NUM_MAX; idx++) {
			vers[idx]= 0;
		}

		status = common_nvm_get_romimg_vers(read_source, nvm_addr, vers, type_str);

		/*for(idx=0; idx<ROMIMG_NUM_MAX; idx++) {
			printf("**%s: file ver 0x%x**\n", type_str[i], vers[i]);
		}*/

		for (idx = 0; idx < ROMIMG_NUM_MAX; idx++) {
			if (vers[idx] != 0) {
				sprintf(ver_str, "%s %d.%d.%d;", type_str[idx], (vers[idx] & 0xff000) >> 12, (vers[idx] & 0x0f00) >> 8, vers[idx] & 0xff);
				strcat(str_ptr, ver_str);
			}
		}
		/* Take out last ; */
		str_len = strlen(str_ptr);
		str_ptr[str_len - 1]= '\0';
	} else if (
			   (strcmp(name, NVM_IMAGE_NAME_ISCSI_BOOT) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_ISCSI_BOOT_IPV6) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_ISCSI_BOOT_IPV4N6) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_ISCSI_CPRG) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_FCOE_BOOT) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_FCOE_BOOT_CPRG_EVRST) == 0)) {

		/* Retrieve version string for iSCSI boot image or iSCSI configuration program */
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0x4, &val, sizeof(val));
		offset = val & 0x3;  /* version can start at offset */
		nvm_addr += (val & 0xfffc);

		for (str_idx = 0, nvm_idx = 0; str_idx < MAX_VER_STR_SIZE - 4; nvm_idx += 4) {
			status |= common_nvm_read_NVM_or_image(read_source, nvm_addr, nvm_idx, &ret_val, sizeof(ret_val));
#ifndef FWUPGTOOL
			val = common_nvm_le32_to_cpu(ret_val);
#else
			val = BMAPI_BENDIAN_SWAP(ret_val);
#endif // !FWUPGTOOL
			if ((offset != 0) && (nvm_idx < offset)) {
				/* Handle offset on the first dword read */
				switch (offset) {
				case 1:
					sprintf(&str_ptr[str_idx], "%c%c%c", BYTE_1(val), BYTE_2(val), BYTE_3(val));
					str_idx += 3;
					break;
				case 2:
					sprintf(&str_ptr[str_idx], "%c%c", BYTE_2(val), BYTE_3(val));
					str_idx += 2;
					break;
				case 3:
					sprintf(&str_ptr[str_idx], "%c", BYTE_3(val));
					str_idx++;
					break;
				}
			} else {
				sprintf(&str_ptr[str_idx], "%c%c%c%c", BYTE_0(val), BYTE_1(val), BYTE_2(val), BYTE_3(val));
				str_idx +=4;
			}
		}

	} else if (
			   (strcmp(name, NVM_IMAGE_NAME_ISCSI_BOOT_CFG) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_ISCSI_BOOT_CFG2) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_ISCSI_BOOT_CFG3) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_ISCSI_BOOT_CFG4) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_FCOE_BOOT_CFG) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_FCOE_BOOT_CFG2) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_FCOE_BOOT_CFG3) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_FCOE_BOOT_CFG4) == 0)) {
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0, &val, sizeof(val));
		sprintf(str_ptr, "v%d.%d", BYTE_3(val), BYTE_2(val));

	} else if (strcmp(name, NVM_IMAGE_NAME_NIC_PARTITION_CFG) == 0) {
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0, &val, sizeof(val));
		sprintf(str_ptr, "Rev%d", BYTE_0(val));

	} else if (strcmp(name, NVM_IMAGE_NAME_NIV_CFG) == 0) {
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0, &val, sizeof(val));
		sprintf(str_ptr, "Rev%d", BYTE_0(val));

	} else if (strcmp(name, NVM_IMAGE_NAME_NIV_PROFILES_CFG) == 0) {
		// Do not show a version string in case of NIVPROFILES image
		sprintf(str_ptr, "N/A");

	} else if (strcmp(name, NVM_IMAGE_NAME_BOOT_CFG_SHADOW) == 0) {
		// Do not show a version string in case of BOOT_CFG_SHADOW image
		sprintf(str_ptr, "N/A");

	} else if (strcmp(name, NVM_IMAGE_NAME_BDN) == 0) {
		// Do not show a version string in case of BDN image
		sprintf(str_ptr, "N/A");

	} else if (strcmp(name, NVM_IMAGE_NAME_MDUMP) == 0) {
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 4, &val, sizeof(val));
		sprintf(str_ptr, "v%d.%d.%d.%d", BYTE_3(val), BYTE_2(val), BYTE_1(val), BYTE_0(val));

	} else if ((strcmp(name, NVM_IMAGE_NAME_L2T) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_L2C) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_L2X) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_L2U) == 0)) {
		u8_t client;
		u8_t primary;
		u16_t patch;

		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0, &ret_val, sizeof(u32_t));
#ifndef FWUPGTOOL
		val = common_nvm_le32_to_cpu(ret_val);
#else
		val = BMAPI_BENDIAN_SWAP(ret_val);
#endif
		client = BYTE_0(val);
		primary = BYTE_1(val);
		patch = WORD_MSB(val);
		sprintf(str_ptr, "%s %d.%d.%d", name, client, primary, patch);

	} else if (strcmp(name, NVM_IMAGE_NAME_MODULES_PN) == 0) {
		/* First dword in nvm image of MODULES_PN is the format_revision */
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0, &val, sizeof(u32_t));
		sprintf(str_ptr, "Rev %d", val);

	} else if (strcmp(name, NVM_IMAGE_NAME_VPD) == 0) {
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0, &val, sizeof(u32_t));
		sprintf(str_ptr, "VPD %d", val);

	} else if (strcmp(name, NVM_IMAGE_NAME_NCSI_CMN) == 0) {
		/* ncsi fw_version name can be 16 bytes long */
		for (idx = 0; idx < 16; idx += sizeof(u32_t)) {
			status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0x10 + idx, &ret_val, sizeof(u32_t));
			val = htonl(ret_val);
			sprintf(&str_ptr[idx], "%c%c%c%c", BYTE_0(val), BYTE_1(val), BYTE_2(val), BYTE_3(val));
		}
	} else if ((strcmp(name, NVM_IMAGE_NAME_E3_WC) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_E3_PCIE) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_E3_WCV2) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_E3_PCIEV2) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_CFG_EXTENDED_SHARED) == 0) ||
			   (strcmp(name, NVM_IMAGE_NAME_USR_BLK) == 0)) {
		sprintf(str_ptr, "N/A");

	} else if (strcmp(name, NVM_IMAGE_NAME_OCNVM) == 0) {
		/* For now, don't check OCNVM version since is N/A.
		 Change here is version is defined later*/
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 4, &val, sizeof(u32_t));
		sprintf(str_ptr, "OCNVM %d.%d.%d", htons(WORD_LSB(val)), BYTE_2(val), BYTE_3(val));

	} else if (strcmp(name, NVM_IMAGE_NAME_CCM) == 0) {
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 16, &val, sizeof(u32_t));
		sprintf(str_ptr, "v%d.%d.%d", BYTE_0(val), BYTE_1(val), BYTE_2(val));

	} else if (strcmp(name, NVM_IMAGE_NAME_HW_SET) == 0) {
		/* First dword in nvm image is the format_revision */
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0, &val, sizeof(u32_t));
		sprintf(str_ptr, "Rev %d", val);
	} else if (strcmp(name, NVM_IMAGE_NAME_ISCSI_PERS) == 0) {
		/* version is upper byte in first DWORD */
		status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0, &val, sizeof(u32_t));
		sprintf(str_ptr, "Rev %d", BYTE_0(val));
	} else {
		/* Retrieve version string for all other firmware images */
		for (idx = 0; idx < 16; idx += sizeof(u32_t)) {
			status = common_nvm_read_NVM_or_image(read_source, nvm_addr, 0x10 + idx, &ret_val, sizeof(u32_t));
			val = htonl(ret_val);
			sprintf(&str_ptr[idx], "%c%c%c%c", BYTE_0(val), BYTE_1(val), BYTE_2(val), BYTE_3(val));
		}
	}

	/* Reject the built string if there is a failure in NVM accesses. */
	if (status != COMMON_NVM_STATUS_SUCCESS) {
		str_ptr[0]= 0;
	}
	return;
}



/*
 *------------------------------------------------------------------------
 * common_nvm_fw_ver_code --
 *
 * Returns a 32-bit version code of a firmware image. On error, -1 is
 * returned.
 *
 * The nvm_addr is the "virtual" one in case of the buffered flash.
 *------------------------------------------------------------------------
 */
u32_t common_nvm_fw_ver_code(
							 u32_t nvm_addr
							 ) {
	common_nvm_status_t status;
	u32_t data;

	status = common_nvm_read_nvram(nvm_addr + 0x1c, &data, sizeof(data));
	return(status == COMMON_NVM_STATUS_SUCCESS) ? data : -1;
}

u32_t common_nvm_image_table_size() {
	return sizeof(image_table)/sizeof(image_map_t);
}

/*
 *------------------------------------------------------------------------
 * update_bootstrap --
 *
 *     Update bootstrap header in NVM
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_update_bootstrap(
												u32_t sram_start_addr,
												u32_t byte_cnt,
												u32_t nvm_start_addr
												) {
	u32_t crc = 0xffffffff, buf[5];
	common_nvm_status_t status;


	/* Update bootstrap */
	// buf[0] includes the magic number (unless byte_cnt is 0)
	buf[0]= (byte_cnt==0) ? 0 : NVM_MAGIC_VALUE;
	buf[1]= sram_start_addr;
	buf[2]= byte_cnt/4;
	buf[3]= common_nvm_xlate_flash_phy_addr(nvm_start_addr);

	common_nvm_compute_crc((u8_t *)buf, 16, 0, &crc);

	buf[4]= ~BYTE_SWAP(crc);

	status = common_nvm_write_nvram(NVM_OFFSET(bootstrap.magic_value), buf, 20);
	if (status != COMMON_NVM_STATUS_SUCCESS) return status;

	DbgMessage(NULL, INFORMnv, "update bootstrap: start=%x; byte_cnt=%d; crc=0x%x\n", nvm_start_addr, byte_cnt, ~htonl(crc));

	return status;
}







/*******************************************
 *
 *               Static functions
 *
 *******************************************/
/*
 *------------------------------------------------------------------------
 * compute_crc --
 *     Compute CRC of the buffer array
 *     crc_ptr is both input and output parameter; as input, it is an
 *     intermediate crc value (useful for computing crc block by block),
 *     must be initialized to 0xffffffff if used for the first block;
 *     as output, it will contain the resulting crc.
 *
 * Results:
 *     0 if success
 *
 * Side effects:
 *     None
 *------------------------------------------------------------------------
 */
void common_nvm_compute_crc(
							u8_t * buf,
							u32_t byte_cnt,
							u32_t big_endian,
							u32_t *crc_ptr
							) {
	u32_t crc, data32, idx, *ptr32;
	crc = *crc_ptr;
	for (idx = 0, ptr32 = (u32_t *)buf; idx < byte_cnt; idx += 4, ptr32++) {
		data32 = (big_endian) ? htonl(*ptr32): *ptr32;
		crc = UPD32CRC32(data32, crc);
	}
	*crc_ptr = crc;
}

/*
 *------------------------------------------------------------------------
 * compute_image_crc --
 *
 *     Read image from NVM and compute CRC; also capture the last dword
 *     as the stored CRC value.
 *
 * Results:
 *     0 for success, -1 for any failure.
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_compute_image_crc(
												 u32_t start_nvm_addr,
												 u32_t byte_cnt,
												 u32_t *computed,
												 u32_t *stored
												 ) {
	common_nvm_status_t status;
	int rc;
	u8_t *buf;
	u32_t *ptr32, big_endian, crc32;
#ifndef FWUPGTOOL
	u8_t all_zero, need_fix=0;
	u32_t idx, port_no;
#endif

	buf =(u8_t *)(uintptr_t)common_nvm_alloc_virt(byte_cnt);

	if (buf == NULL) {
		DbgMessage(NULL, FATAL, "memory allocation error\n");
		return COMMON_NVM_STATUS_RESOURCE;
	}

	rc = 0;

	if (byte_cnt & 0x3) {
		rc = -1;
	}

	ptr32 =(u32_t *)buf;

#ifndef FWUPGTOOL
	all_zero = 1;

	for (idx = 0; ((idx < byte_cnt) && (rc == 0)); idx += 4, ptr32++) {
		status = common_nvm_read_nvram(start_nvm_addr + idx, ptr32, sizeof(u32_t));
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			rc = -1;
		}
		if (all_zero && (*ptr32 != 0)) {
			all_zero = 0;
		}
	}
#else
	// if rc != 0, we don't need to read from NVRAM at all
	if (rc == 0) {
		status = common_nvm_read_nvram(start_nvm_addr, ptr32, byte_cnt);

		if (status != COMMON_NVM_STATUS_SUCCESS) {
			rc = -1;
		}

#if 0
		// For FWUPGTOOL, we don't need to check for all_zero condition
		// because we do not write to NVRAM on this condition...
		for ( idx = 0; idx < byte_cnt; idx+=4, ptr32++ ) {
			if (all_zero && (*ptr32 != 0)) {
				all_zero = 0;
				break;
			}
		}
#endif
	}
#endif // !FWUPGTOOL

#ifndef FWUPGTOOL
	// For FWUPGTOOL, we don't need to check for all_zero condition
	// because we do not write to NVRAM on this condition...
	/* Silently correct the MKEY_INFO and UKEY_INFO blocks */
	for (port_no = 0; port_no < PORT_MAX; port_no++) {
		if ((start_nvm_addr == NVM_OFFSET(manuf_key_info[port_no])) ||
			(start_nvm_addr == NVM_OFFSET(upgrade_key_info[port_no]))) {
			if (all_zero) {
				printf("INFORM: MKEY_INFO/UKEY_INFO all zeros. Updating CRC ..\n");
				need_fix = 1;
			}
		}
	}
#endif // !FWUPGTOOL

	big_endian = 0;
	if (rc == 0) {
		crc32 = 0xffffffff;
		common_nvm_compute_crc(buf, byte_cnt - 4, big_endian, &crc32);
#ifndef FWUPGTOOL
		crc32 = os_if_cpu_to_le32(~htonl(crc32));
		if (need_fix) {
			common_nvm_write_nvram(start_nvm_addr + byte_cnt - 4, &crc32, sizeof(u32_t));
			*(u32_t *)(&buf[byte_cnt - 4]) = crc32;
		}
#else
		crc32 = BMAPI_BENDIAN_SWAP(~htonl(crc32));
#endif
		if (computed) *computed = crc32;
		if (stored) *stored = *(u32_t *)(&buf[byte_cnt - 4]);
	}

	common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)buf);

	return rc == 0 ? COMMON_NVM_STATUS_SUCCESS : COMMON_NVM_STATUS_FAILURE;
}


/*
 *------------------------------------------------------------------------
 * find_room --
 *
 *     Find storage area (and move images around if necessary) for
 *     programming an image.
 *
 * Input:
 *    The byte_cnt does not include the 4-byte CRC.
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if success.
 *
 * Side effects:
 *     Directory entries in NVM may be updated, and NVM images may be
 *     moved around.
 *
 * To do:
 *     Add algorithm to truly find room and juggle images to make things
 *     fit, if possible. Make sure to include 4 bytes of CRC into
 *     consideration.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_find_room(
										 u32_t image_type,
										 u32_t byte_cnt,
										 extended_dir_image_t *extended_dir_p,
										 u32_t forbidden_nvm_addr,
										 u32_t forbidden_nvm_size,
										 u32_t *nvm_addr_ptr
										 ) {

	u32_t total_size, i;
	common_nvm_status_t res;

	int is_evst1x = 0;
	u32_t image_is_in_extended_dir=0;
	u32_t extended_dir_entry=0;


#ifndef FWUPGTOOL
	if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
		is_evst1x = is_e1x(globalInterp);
	} else {
		u32_t device_id;
		common_nvm_duart_get_chip_id(&device_id);
		if ((device_id == DEVICE_ID_E1) ||
			(device_id == DEVICE_ID_E1_5) ||
			(device_id == DEVICE_ID_E1_5E)) is_evst1x = 1;
	}
#else
	is_evst1x = COMMON_NVM_IS_E1x();
#endif
	byte_cnt += 4;

	/* If NVM size does not have room for this image, return failure */
	if (enough_room(is_evst1x, image_type, byte_cnt, NULL, 0) != NVM_ENOUGH_ROOM) {
		DbgMessage(NULL, FATAL, "ERROR: common_nvm_find_room(): not enough room in NVM for image_type 0x%x !!\n ", image_type);
		*nvm_addr_ptr = 0xffffffff;
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* First, nvm_addr_ptr points to end of NVM reserved area for bootstrap, license ect. */
	*nvm_addr_ptr = (!is_evst1x) ? (sizeof(nvm_image_t) + sizeof(path1_nvm_image_t)) : sizeof(nvm_image_t);
	total_size = common_nvm_nvm_size();

	/* Run as long as nvm_addr_ptr is less than NVM total size */
	while (*nvm_addr_ptr < total_size) {

		u32_t move_start, move_len;
		u32_t code_start, code_len, type=0xFF;
		int entry, idx;

		move_start = total_size;
		move_len = 0;
		entry = -2;
		image_is_in_extended_dir = 0;
		extended_dir_entry=0;

		if ((*nvm_addr_ptr >= forbidden_nvm_addr) && (*nvm_addr_ptr <= forbidden_nvm_addr + forbidden_nvm_size)) {
			*nvm_addr_ptr = forbidden_nvm_addr + forbidden_nvm_size;
		}
		/*
		printf("1) Upgrade image_type 0x%x, nvm_addr_ptr=0x%x code_start=0x%x, move_start=0x%x\n",
			   image_type, *nvm_addr_ptr, code_start, move_start);
		*/


		/* Go through directory entries (0-15) */
		for (idx = -1; idx < CODE_ENTRY_MAX; idx++) {
			if (idx == -1) {

				if (image_type == CODE_IMAGE_TYPE_BC1) {
					/* Don't include this if we are looking for room for bc1 */
					continue;
				}
				res = common_nvm_read_nvram(NVM_OFFSET(bootstrap.code_start_addr),
											&code_start, sizeof(code_start));
				if (res != COMMON_NVM_STATUS_SUCCESS) {
					return res;
				}
				code_start = common_nvm_xlate_flash_virt_addr(code_start);
				if (code_start == 0xffffffff) {
					DbgMessage(NULL, FATAL, "common_nvm_xlate_flash_virt_addr failed\n");
					return COMMON_NVM_STATUS_FAILURE;
				}
				res = common_nvm_read_nvram(NVM_OFFSET(bootstrap.code_len),
											&code_len, sizeof(code_len));
				if (res != COMMON_NVM_STATUS_SUCCESS) {
					return res;
				}
				code_len *= sizeof(u32_t);
			} else {
				/* Read this directory entry info (code_start, code_len) */
				res = common_nvm_nvm_get_dir_info(idx, &code_start, &code_len, NULL, &type,
												  NULL);
				if (res != COMMON_NVM_STATUS_SUCCESS) {
					return res;
				}

				/*
				printf("2) Dir entry %d nvm_addr_ptr=0x%x code_start=0x%x, move_start=0x%x\n",
					   idx, *nvm_addr_ptr, code_start, move_start);
				*/

				/* special handling for extended dir: if it size remained the same, simply return same address */
				if ((type == CODE_IMAGE_TYPE_EXTENDED_DIR) && (image_type == type) && (code_len == (sizeof(extended_dir_image_t) + 4))) {
					*nvm_addr_ptr = code_start;
					// printf("14) Found room for image size 0x%x at *nvm_addr_ptr 0x%x!! \n", byte_cnt, *nvm_addr_ptr);
					return COMMON_NVM_STATUS_SUCCESS;
				}

				/* If extended dir image exist, go through its images information:
				   Set the move_start, move_len accoring to current image info in extended dir,
				   since we might need to move it if there are gaps in NVM addresses.
				*/
				if ((idx == CODE_ENTRY_EXTENDED_DIR_IDX) && (type == CODE_IMAGE_TYPE_EXTENDED_DIR)) {
					/* Check the extended dir image code start */
					if (code_len && (*nvm_addr_ptr <= code_start) &&
						(move_start >= code_start)) {
						/*
						printf ("3) Extended dir image nvm_addr_ptr 0x%x code_start 0x%x move_start 0x%x\n",
									*nvm_addr_ptr, code_start, move_start);
						*/
						move_start = code_start;
						move_len = code_len;
						entry = CODE_ENTRY_EXTENDED_DIR_IDX;
						if (*nvm_addr_ptr == code_start) break;
					}

					/* Check the images inside the extended dir image */
					if (common_nvm_nvm_parse_extended_dir_image(code_start, code_len, extended_dir_p) != COMMON_NVM_STATUS_SUCCESS) {
						DbgMessage(NULL, FATAL, "Failed to parse extended dir image !! \n");
						return COMMON_NVM_STATUS_FAILURE;
					}


					for (i = 0; i < extended_dir_p->no_images; i++) {
						code_start = extended_dir_p->extended_dir_images[i].nvm_start_addr;
						code_len = (extended_dir_p->extended_dir_images[i].code_attribute & CODE_IMAGE_LENGTH_MASK);
						type = (extended_dir_p->extended_dir_images[i].code_attribute & CODE_IMAGE_TYPE_MASK);

						/* If there is an image with the same type of image we intend to upgrade
						   Then, this place is free */
						if (image_type == type) {
							continue;
						}


						if (code_len && (*nvm_addr_ptr <= code_start) &&
							(move_start >= code_start)) {
							/*
							printf ("4) Extended dir entry %d nvm_addr_ptr 0x%x, code_start 0x%x move_start 0x%x\n",
										i, *nvm_addr_ptr, code_start, move_start);
							*/
							move_start = code_start;
							move_len = code_len;
							entry = CODE_ENTRY_EXTENDED_DIR_IDX;
							extended_dir_entry = i;
							image_is_in_extended_dir=1;

							// There is an image at this place, so it can't be used.
							if (*nvm_addr_ptr == code_start) break;

						}
					}

				}

				/* If there is an image with the same type of image we intend to upgrade
				   Then, this place is free */
				if (image_type == type) {
					continue;
				}
			}

			/* This check was added for the last entry in extended_dir.
			   If there is an image with the same type of image we intend
			   to upgrade. Then, this place is free. */
			if (image_type == type) {
				continue;
			}

			/* If there is an image in this directory entry  */
			if (code_len && (*nvm_addr_ptr <= code_start) &&
				(move_start >= code_start)) {
				/*
				printf ("5) Dir entry %d nvm_addr_ptr 0x%x  code_start 0x%x move_start 0x%x \n",
													idx,  *nvm_addr_ptr, code_start, move_start);
				*/

				/* move_start holds the nvm_offset of the image in this directory entry */
				move_start = code_start;
				/* move_len holds the size of image in this directory entry */
				move_len = code_len;
				entry = idx;

				// There is an image at this location, so it can't be used
				if (*nvm_addr_ptr == code_start) {
					break;
				}
			}
		} // go through dir entries (0-15)


		/* Do we have enough space now? */
		/* move_start holds the next image location */
		/* nvm_addr_ptr holds the current location in NVM */
		if ((move_start - *nvm_addr_ptr) >= byte_cnt) {
			//printf("6) Found room for image size 0x%x at *nvm_addr_ptr 0x%x!! \n", byte_cnt, *nvm_addr_ptr);
			return COMMON_NVM_STATUS_SUCCESS;  /* yes, use this space */
		}

		/* Check if there is a gap between images, then need to move data */
		if (move_start != *nvm_addr_ptr) {
			if (entry != -2) {  /* valid entry */
				u32_t *ptr32;
				u32_t *buffer =(u32_t *)(uintptr_t)common_nvm_alloc_virt(move_len);
				if (buffer == NULL) {
					DbgMessage(NULL, FATAL, "memory allocation error\n");
					return COMMON_NVM_STATUS_RESOURCE;
				}
				//printf ("----> 7) There is a gap !! Moving image from offset %08X to %08X\n", move_start, *nvm_addr_ptr);
				ptr32 = buffer;
#ifndef FWUPGTOOL
				for (idx = 0; idx < (int)move_len; idx += 4, ptr32++) {
					res = common_nvm_read_nvram(move_start + idx, ptr32, sizeof(u32_t));
					if (res != COMMON_NVM_STATUS_SUCCESS) {
						return res;
					}
				}
#else
				res = common_nvm_read_nvram(move_start, (u32_t *)buffer, move_len);
				if (res != COMMON_NVM_STATUS_SUCCESS) {
					common_nvm_free_virt((common_nvm_virt_addr_t)buffer);
					return res;
				}
#endif // !FWUPGTOOL
				DbgMessage(NULL, VERBOSEnv, "Writing data to %08X...\n", *nvm_addr_ptr);
				res = common_nvm_write_nvram(*nvm_addr_ptr, buffer, move_len);
				if (res != COMMON_NVM_STATUS_SUCCESS) {
					return res;
				}
				DbgMessage(NULL, VERBOSEnv, "Updating directory...\n");
				/* update dir */
				if (entry == -1) { /* boot code */
					bootstrap_region_t boot;
#ifndef FWUPGTOOL
					u32_t *ptr32 =(u32_t *)&boot;

					for (idx = 0; idx < sizeof(bootstrap_region_t); idx += 4, ptr32++) {
						res = common_nvm_read_nvram(idx, ptr32, sizeof(u32_t));
						if (res != COMMON_NVM_STATUS_SUCCESS) {
							common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)buffer);
							return res;
						}
					}
#else
					res = common_nvm_read_nvram(0, (u32_t *)&boot, sizeof(bootstrap_region_t));
					if (res != COMMON_NVM_STATUS_SUCCESS) {
						common_nvm_free_virt((common_nvm_virt_addr_t)buffer);
						return res;
					}
#endif // !FWUPGTOOL
					common_nvm_update_bootstrap(boot.sram_start_addr, boot.code_len * sizeof(u32_t), *nvm_addr_ptr);
				} else {
					u32_t byte_cnt, sram_addr, image_type, agent_type, nvm_addr, byte_prg, extended_dir_image_type;

					res = common_nvm_nvm_get_dir_info(entry, NULL, &byte_cnt, &sram_addr, &image_type, &agent_type);
					if (res != COMMON_NVM_STATUS_SUCCESS) {
						common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)buffer);
						return res;
					}

					// If we are moving an image from the extended dir, we need to update the extended_dir_image
					if (image_is_in_extended_dir) {

						/* Update the new nvm_addr of this image in extended dir */
						extended_dir_p->extended_dir_images[extended_dir_entry].nvm_start_addr = *nvm_addr_ptr;
						extended_dir_image_type = extended_dir_p->extended_dir_images[extended_dir_entry].code_attribute & CODE_IMAGE_TYPE_MASK;
						/*
						  printf("8) Extended dir entry %d Image type 0x%x new location is 0x%x \n",
						  extended_dir_entry, extended_dir_image_type, *nvm_addr_ptr);
						*/

						// Update bootstrap, incase we are moving MFW image
						if (extended_dir_image_type == CODE_IMAGE_TYPE_MFW1) {
							bootstrap_region_t boot;

							u32_t *ptr32 =(u32_t *)&boot;

							for (idx = 0; idx < sizeof(bootstrap_region_t); idx += 4, ptr32++) {
								res = common_nvm_read_nvram(idx, ptr32, sizeof(u32_t));
								if (res != COMMON_NVM_STATUS_SUCCESS) {
									common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)buffer);
									return res;
								}
							}

							//printf("9) Update bootstrap with size 0x%x and nvm_addr 0x%x\n", boot.code_len*sizeof(u32_t), *nvm_addr_ptr);
							common_nvm_update_bootstrap(boot.sram_start_addr, boot.code_len * sizeof(u32_t), *nvm_addr_ptr);
						}


						res = common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &byte_cnt, &sram_addr, &image_type, &agent_type);
						if (res != COMMON_NVM_STATUS_SUCCESS) {
							common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)buffer);
							return res;
						}
						/* Program the updated extended dir image
						   Note: without the last 4 bytes for CRC, need to re-calculate CRC after the changes */
						if (common_nvm_program_nvm((u8_t *)extended_dir_p, byte_cnt - 4, nvm_addr,
												   PRG_NVM_LITTLE_ENDIAN | PRG_NVM_APPEND_CRC,
												   &byte_prg) != COMMON_NVM_STATUS_SUCCESS) {
							DbgMessage(NULL, FATAL, "Programming extended dir image into NVM failed !!");
							common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)buffer);
							return COMMON_NVM_STATUS_FAILURE;
						}

					} else {
						common_nvm_update_dir_entry(entry, *nvm_addr_ptr, byte_cnt, sram_addr, image_type, agent_type);
					}
				}
				common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)buffer);
			}
		}

		/* nvm_addr_ptr is increased by the length of the image
		   in this directoy entry */
		*nvm_addr_ptr += move_len;
	}

	return COMMON_NVM_STATUS_FAILURE;
}




/* ************************************************************* */
/* **************** NVM config utility functions *************** */
/* ************************************************************* */

/*
 *------------------------------------------------------------------------
 * common_nvm_load_shmem --
 *
 *     This function reads the current shmem dev_info into shmem_cfg_ptr.
 *     This function is used by common_nvm_load_cfg().
 *
 *------------------------------------------------------------------------
 */
void common_nvm_load_shmem(
						   u32_t shmem_base,
						   u8_t port_num,
						   nvm_cfg_t *shmem_cfg_ptr
						   ) {
	u32_t *shmem_ptr;
	u32_t i, addr, value;
#ifdef FWUPGTOOL
	NVM_INTERFACE iNvm = nvm_interface(gpINx2Nvm->m_tcl_interp);
	u32_t dev_index = iNvm.m_current_dev_index;
#endif

	/* Read from shmem the manufacture block */
	shmem_ptr =(u32_t *)&(shmem_cfg_ptr->shared_manuf);
	addr = OFFSETOF(struct shmem_region, dev_info.shared_hw_config);

	for (i = 0; i < sizeof(shared_hw_cfg_t) / 4; i++) {
#ifndef FWUPGTOOL
		common_nvm_reg_read(shmem_base + addr + i * 4, &value);
#else
		iNvm.m_td_read_reg32(dev_index, (shmem_base + addr + i * 4), &value);
#endif
		*(shmem_ptr + i) = value;
	}

	/* Read from shmem the feature block */
	shmem_ptr =(u32_t *)&(shmem_cfg_ptr->shared_feat);
	addr = OFFSETOF(struct shmem_region, dev_info.shared_feature_config);
	for (i = 0; i < sizeof(shared_feat_cfg_t) / 4; i++) {
#ifndef FWUPGTOOL
		common_nvm_reg_read(shmem_base + addr + i * 4, &value);
#else
		iNvm.m_td_read_reg32(dev_index, (shmem_base + addr + i * 4), &value);
#endif
		*(shmem_ptr + i) = value;
	}

	/* Read from shmem the port hw config block */
	shmem_ptr =(u32_t *)&(shmem_cfg_ptr->port_manuf);
	addr = OFFSETOF(struct shmem_region, dev_info.port_hw_config[port_num]);
	for (i = 0; i < sizeof(port_hw_cfg_t) / 4; i++) {
#ifndef FWUPGTOOL
		common_nvm_reg_read(shmem_base + addr + i * 4, &value);
#else
		iNvm.m_td_read_reg32(dev_index, (shmem_base + addr + i * 4), &value);
#endif
		*(shmem_ptr + i) = value;
	}

	/* Read from shmem the port hw config block */
	shmem_ptr =(u32_t *)&(shmem_cfg_ptr->port_feat);
	addr = OFFSETOF(struct shmem_region, dev_info.port_feature_config[port_num]);
	for (i = 0; i < sizeof(port_feat_cfg_t) / 4; i++) {
#ifndef FWUPGTOOL
		common_nvm_reg_read(shmem_base + addr + i * 4, &value);
#else
		iNvm.m_td_read_reg32(dev_index, (shmem_base + addr + i * 4), &value);
#endif
		*(shmem_ptr + i) = value;
	}

}

/*
 *------------------------------------------------------------------------
 * common_nvm_read_nvm_cfg --
 *
 *     This function reads the NVM configuration and requested images.
 *     This function is used by common_nvm_load_cfg().
 *
 * Results:
 *     0 if success, -1 for failure/abort.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_read_nvm_cfg(
											nvm_cfg_t * cfg_ptr,
											u8_t path_num,
											u8_t port_num,
											media_vpd_t *vpd_block_ptr,
											dir_t *dir_block_ptr,
											nvm_nic_part_cfg_t *mac_partition_block_ptr,
											u32_t *nvm_cfg_extended_shared_image_ptr
											) {

	common_nvm_status_t stat = COMMON_NVM_STATUS_SUCCESS;

	/* Read the manuf block from NVM */
	stat |= common_nvm_read_nvram(NVM_OFFSET_PATH(manuf_info.shared_hw_config, path_num), (u32_t *)&cfg_ptr->shared_manuf,
								  sizeof(shared_hw_cfg_t));

	stat |= common_nvm_read_nvram(NVM_OFFSET_PATH(manuf_info.port_hw_config[port_num], path_num), (u32_t *)&cfg_ptr->port_manuf,
								  sizeof(port_hw_cfg_t));

	/* Read the feature block */
	stat |= common_nvm_read_nvram(NVM_OFFSET_PATH(feature_info.shared_feature_config, path_num), (u32_t *)&cfg_ptr->shared_feat,
								  sizeof(shared_feat_cfg_t));

	stat |= common_nvm_read_nvram(NVM_OFFSET_PATH(feature_info.port_feature_config[port_num], path_num), (u32_t *)&cfg_ptr->port_feat,
								  sizeof(port_feat_cfg_t));

	/* Read the VPD block */
	stat |= common_nvm_read_nvram(NVM_OFFSET(vpd), (u32_t *)vpd_block_ptr, sizeof(media_vpd_t));

	/* Read the dir block */
	stat |= common_nvm_read_nvram(NVM_OFFSET(dir.code[0]), (u32_t *)dir_block_ptr, sizeof(dir_t));

	/* Read the mac_partition block from its image, if exists */
	stat |= common_nvm_nvm_fill_mac_partition_info(mac_partition_block_ptr, NULL);

	/* Read the nvm cfg extended shared block from its image, if exists */
	stat |= common_nvm_nvm_fill_nvm_cfg_extended_shared(nvm_cfg_extended_shared_image_ptr, NVM_CFG_EXTENDED_SHARED_IMAGE_MAX_SIZE);

	return stat;
}

/*
 *------------------------------------------------------------------------
 * common_shmem_load_cfg --
 *
 *     This function reads shmem content into an input buffer.
 *
 * Results:
 *     0 if success, -1 for failure/abort.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_shmem_load_cfg(
										  nvm_cfg_t * shmem_cfg_ptr
										  ) {
	u8_t path_num, port_num;
	u32_t shmem_base=0, shmem2_base=0, tmp_val;
	common_nvm_status_t stat;
#ifndef FWUPGTOOL
	common_nvm_dev_info_t *cur_dev_p = COMMON_NVM_CUR_DEV;
	path_num = COMMON_NVM_PATH_NUM(*cur_dev_p);
#else
	NVM_INTERFACE iNvm = nvm_interface(gpINx2Nvm->m_tcl_interp);
	u32_t dev_index = iNvm.m_current_dev_index;

	path_num =(u8_t)COMMON_NVM_PATH_NUM();
#endif // !FWUPGTOOL

	stat = common_nvm_get_port_num(&tmp_val);
	if (stat != COMMON_NVM_STATUS_SUCCESS) return stat;
	port_num =(u8_t)tmp_val;

	if (!common_nvm_tcl_al_get_virtual_nvm_mode() && shmem_cfg_ptr != NULL) {

		/* Get shmem base address of path0 and path 1*/
#ifndef FWUPGTOOL
		if (path_num == 0) {
			common_nvm_reg_read(MISC_REG_SHARED_MEM_ADDR, &shmem_base);
		} else {
			if (!is_e1x(globalInterp)) {
				common_nvm_reg_read(MISC_REG_GENERIC_CR_0, &shmem2_base);
				if (shmem2_base != 0) {
					/* Path 1 shmem_base*/
					common_nvm_reg_read(shmem2_base + OFFSETOF(shmem2_region_t, other_shmem_base_addr), &shmem_base);
				}
			}
		}
#else
		if (path_num == 0) {
			iNvm.m_td_read_reg32(dev_index, MISC_REG_SHARED_MEM_ADDR, &shmem_base);
		} else {
			if (!COMMON_NVM_IS_E1x()) {
				iNvm.m_td_read_reg32(dev_index, MISC_REG_GENERIC_CR_0, &shmem2_base);
				if (shmem2_base != 0) {
					/* Path 1 shmem_base*/
					iNvm.m_td_read_reg32(dev_index, (shmem2_base + OFFSETOF(shmem2_region_t, other_shmem_base_addr)), &shmem_base);
				}
			}
		}
#endif

		/* Read from shmem the manufacture and feature information */
		if (shmem_base != 0) {
			common_nvm_load_shmem(shmem_base, port_num, shmem_cfg_ptr);
		}
	}
	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_load_cfg --
 *
 *     This function implements the "nvm_load" command.
 *     It loads the current NVM configuration into cfg_ptr.
 *     Also loads the current shmem configuration to shmem_cfg_ptr
 *
 * Results:
 *     0 if success, -1 for failure/abort.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_load_cfg(
										nvm_cfg_t * cfg_ptr,
										nvm_cfg_t * shmem_cfg_ptr,
										nvm_nic_part_cfg_t * mac_partition_ptr,
										u32_t * nvm_cfg_extended_shared_ptr
										) {

	media_vpd_t vpd_block;
	dir_t dir_block;
	nvm_nic_part_cfg_t mac_partition_block;
	u32_t nvm_cfg_extended_shared_image[NVM_CFG_EXTENDED_SHARED_IMAGE_MAX_SIZE];
	common_nvm_status_t stat = COMMON_NVM_STATUS_SUCCESS;
	u8_t port_num;
	u8_t path_num;
	u32_t tmp_val, dev_func_num;


#ifndef FWUPGTOOL
	common_nvm_dev_info_t *cur_dev_p = COMMON_NVM_CUR_DEV;
	path_num = COMMON_NVM_PATH_NUM(*cur_dev_p);
#else
	u32_t   idx;
	u32_t   *ptr32;
	NVM_INTERFACE iNvm = nvm_interface(gpINx2Nvm->m_tcl_interp);
	u32_t dev_index = iNvm.m_current_dev_index;

	path_num =(u8_t)COMMON_NVM_PATH_NUM();
#endif // !FWUPGTOOL

	stat = common_nvm_get_port_num(&tmp_val);
	common_nvm_get_func_num(&dev_func_num);
	if (stat != COMMON_NVM_STATUS_SUCCESS) return stat;
	port_num =(u8_t)tmp_val;

	/* Read NVM configuration and requested images from NVRAM */
	stat = common_nvm_read_nvm_cfg(cfg_ptr, path_num, port_num, &vpd_block, &dir_block,
								   &mac_partition_block, nvm_cfg_extended_shared_image);

	if (stat != COMMON_NVM_STATUS_SUCCESS) {
		return stat;
	}

	/* save a copy of read image (or zeros if not exists in NVM)*/
	memcpy(mac_partition_ptr, &mac_partition_block, sizeof(nvm_nic_part_cfg_t));

	/* save a copy of read image (or zeros if not exists in NVM)*/
	common_nvm_memset(nvm_cfg_extended_shared_ptr, 0, NVM_CFG_EXTENDED_SHARED_IMAGE_MAX_SIZE);
	memcpy(nvm_cfg_extended_shared_ptr, nvm_cfg_extended_shared_image, NVM_CFG_EXTENDED_SHARED_IMAGE_MAX_SIZE);

	memset(cfg_ptr->spare_num, 0, MAX_SPARE_NUM_LENGTH);

#ifdef FWUPGTOOL
	// To support the BE host
	ptr32 =(u32_t *)dir_block.spare_part_num;
	for (idx = 0; idx < ARRSIZE(dir_block.spare_part_num); idx += 4, ptr32++) {
		*ptr32 = BMAPI_BENDIAN_SWAP(*ptr32); // convert string in buffer to BE for BE host
	}
//    printf( "common_nvm_load_cfg: spare_part_num = %s\n", dir_block.spare_part_num );
#endif

	strncpy((char *)(cfg_ptr->spare_num), (char *)(dir_block.spare_part_num), ARRSIZE(cfg_ptr->spare_num));

	/* VPD info */
	common_nvm_get_vpd_info(&cfg_ptr->vpd_info, &vpd_block, 0);
	strncpy((char *)(cfg_ptr->shared_manuf.part_num), cfg_ptr->vpd_info.part_number, ARRSIZE(cfg_ptr->shared_manuf.part_num));

	/* MAC partition info */
	common_nvm_memcpy(&cfg_ptr->macp_gbl, &mac_partition_block.global_cfg, sizeof(struct macp_gbl_cfg));
	common_nvm_memcpy(&cfg_ptr->macp_port, &mac_partition_block.port_cfg[path_num][port_num], sizeof(struct macp_port_cfg));
	common_nvm_memcpy(&cfg_ptr->macp_func, &mac_partition_block.func_cfg[dev_func_num], sizeof(struct macp_func_cfg));

	/* nvm cfg extended shared config */
	common_nvm_memcpy(&cfg_ptr->extended_dev_info_shared, nvm_cfg_extended_shared_image, sizeof(struct extended_dev_info_shared_cfg));

	/* Fill up the shmem_cfg_ptr struct with shmem content. */
	stat = common_shmem_load_cfg(shmem_cfg_ptr);
	if (stat != COMMON_NVM_STATUS_SUCCESS) return stat;

	/* Returns status */
	return COMMON_NVM_STATUS_SUCCESS;
}
#ifndef FWUPGTOOL
/*
*------------------------------------------------------------------------
* common_nvm_get_nvm_offset --
*
* Returns the NVM offset of a requested option within nvm_cfg at "offset"
*------------------------------------------------------------------------
*/
common_nvm_status_t common_nvm_get_nvm_offset(
											  u32_t offset,   /* offset in bits */
											  u32_t *nvm_offset_ptr
											  ) {
	u32_t port_n, offset_in_bytes,  dev_func_num;
	u8_t port_num, path_num;
	u32_t macp_nvm_addr, cfg_ext_shared_nvm_addr;
	u32_t found_image, byte_cnt, entry_idx;
	common_nvm_status_t status;

#ifndef FWUPGTOOL
	// Get current path number
	common_nvm_dev_info_t *cur_dev_p = COMMON_NVM_CUR_DEV;
	path_num = COMMON_NVM_PATH_NUM(*cur_dev_p);
#else
	u32_t   idx;
	u32_t   *ptr32;
	NVM_INTERFACE iNvm = nvm_interface(gpINx2Nvm->m_tcl_interp);
	u32_t dev_index = iNvm.m_current_dev_index;
	path_num =(u8_t)COMMON_NVM_PATH_NUM();
#endif // !FWUPGTOOL

	*nvm_offset_ptr = 0;

	// Get current port number
	if (common_nvm_get_port_num(&port_n) != COMMON_NVM_STATUS_SUCCESS) {
		return COMMON_NVM_STATUS_FAILURE;
	}
	port_num =(u8_t)port_n;

	// Get current func number
	common_nvm_get_func_num(&dev_func_num);

	// Convert input offset in bits to offset in bytes
	offset_in_bytes = offset/8;


	// Convert the offset in nvm_cfg struct to real offset in NVM
	// ----------------------------------------------------------
	if ((offset_in_bytes >= NVM_CFG_OFFSET(shared_manuf)) && (offset_in_bytes < NVM_CFG_OFFSET(port_manuf))) {
		// Option is with in manuf_info.shared_hw_config in NVM
		*nvm_offset_ptr = offset_in_bytes - NVM_CFG_OFFSET(shared_manuf)+ NVM_OFFSET_PATH(manuf_info.shared_hw_config, path_num);
		printf("Please note that is you change this value in NVM, you must change the other path location in NVM as well !\n");

	} else if ((offset_in_bytes >=  NVM_CFG_OFFSET(port_manuf)) && (offset_in_bytes < NVM_CFG_OFFSET(shared_feat))) {
		// Option is with in manuf_info.port_hw_config[port] in NVM
		*nvm_offset_ptr = offset_in_bytes - NVM_CFG_OFFSET(port_manuf)+ NVM_OFFSET_PATH(manuf_info.port_hw_config[port_num], path_num);

	} else if ((offset_in_bytes >=  NVM_CFG_OFFSET(shared_feat)) && (offset_in_bytes < NVM_CFG_OFFSET(port_feat))) {
		// Option is with in feature_info.shared_feature_config in NVM
		*nvm_offset_ptr = offset_in_bytes - NVM_CFG_OFFSET(shared_feat)+ NVM_OFFSET_PATH(feature_info.shared_feature_config, path_num);
		printf("Please note that is you change this value in NVM, you must change the other path location in NVM as well !\n");

	} else if ((offset_in_bytes >=  NVM_CFG_OFFSET(port_feat)) && (offset_in_bytes <  NVM_CFG_OFFSET(vpd_info))) {
		// Option is with in manuf_info.port_feature_config[port] in NVM
		*nvm_offset_ptr = offset_in_bytes - NVM_CFG_OFFSET(port_feat)+ NVM_OFFSET_PATH(feature_info.port_feature_config[port_num], path_num);

	} else if ((offset_in_bytes >= NVM_CFG_OFFSET(vpd_info)) && (offset_in_bytes < NVM_CFG_OFFSET(macp_gbl))) {
		// Option is with in feature_info.port_feature_config[port] in NVM
		printf("VPD options offset is not available.\n");
		return COMMON_NVM_STATUS_FAILURE;

	} else if ((offset_in_bytes >=  NVM_CFG_OFFSET(macp_gbl)) && (offset_in_bytes < NVM_CFG_OFFSET(niv_gbl))) {
		// search for NIC_PRTION image in extended dir
		status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_NIC_PARTITION_CFG, &macp_nvm_addr, &byte_cnt, &entry_idx);

		if ((status != COMMON_NVM_STATUS_SUCCESS) || !found_image) {
			printf("NIC_PRTION image does not exist in NVM !\n");
			return COMMON_NVM_STATUS_FAILURE;

		} else {

			if ((offset_in_bytes >=  NVM_CFG_OFFSET(macp_gbl)) && (offset_in_bytes < NVM_CFG_OFFSET(macp_port))) {
				// MACP global configuration
				*nvm_offset_ptr = offset_in_bytes - NVM_CFG_OFFSET(macp_gbl)+ macp_nvm_addr;

			} else if ((offset_in_bytes >=  NVM_CFG_OFFSET(macp_port)) && (offset_in_bytes < NVM_CFG_OFFSET(macp_func))) {
				// MACP per port configuration, add offset of the port in port_cfg[NVM_PATH_MAX][PORT_MAX]
				*nvm_offset_ptr = offset_in_bytes - NVM_CFG_OFFSET(macp_port)+ macp_nvm_addr +
				sizeof(struct macp_gbl_cfg) + (port_num * sizeof(struct macp_port_cfg));

				if (path_num > 0) {*nvm_offset_ptr += PORT_MAX * sizeof(struct macp_port_cfg);}

			} else {
				// MACP per func configuration, add offset of the func in func_cfg[E1H_FUNC_MAX]
				*nvm_offset_ptr = offset_in_bytes - NVM_CFG_OFFSET(macp_func)+ macp_nvm_addr +
				sizeof(struct macp_gbl_cfg) + (PORT_MAX * NVM_PATH_MAX * sizeof(struct macp_port_cfg)) +
				(dev_func_num * sizeof(struct macp_func_cfg));

			}
		}
	} else {
		// search for CFG_EXTENDED_SHARED image in extended dir
		status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &cfg_ext_shared_nvm_addr, &byte_cnt, &entry_idx);

		if ((status != COMMON_NVM_STATUS_SUCCESS) || !found_image) {
			printf("CFG_SHARED image does not exist in NVM ! \n");
			return COMMON_NVM_STATUS_FAILURE;
		} else {
			if ((offset_in_bytes - NVM_CFG_OFFSET(extended_dev_info_shared)) > byte_cnt) {
				printf("Offset ecxeeds CFG_SHARED image size ! \n");
				return COMMON_NVM_STATUS_FAILURE;
			}
			*nvm_offset_ptr = offset_in_bytes - NVM_CFG_OFFSET(extended_dev_info_shared)+ cfg_ext_shared_nvm_addr;
		}
	}
	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_get_shmem_offset --
 *
 * Returns the SHMEM offset of a requested option within nvm_cfg at "offset"
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_get_shmem_offset(
												u32_t offset,   /* offset in bits */
												u32_t *shmem_addr_ptr
												) {
	u32_t port_n, offset_in_bytes;
	u8_t port_num, path_num;
	u32_t shmem_base=0, shmem2_base=0;

#ifndef FWUPGTOOL
	// Get current path number
	common_nvm_dev_info_t *cur_dev_p = COMMON_NVM_CUR_DEV;
	path_num = COMMON_NVM_PATH_NUM(*cur_dev_p);
#else
	u32_t   idx;
	u32_t   *ptr32;
	NVM_INTERFACE iNvm = nvm_interface(gpINx2Nvm->m_tcl_interp);
	u32_t dev_index = iNvm.m_current_dev_index;
	path_num =(u8_t)COMMON_NVM_PATH_NUM();
#endif // !FWUPGTOOL

	/* Get shmem base address of path0 and path 1*/
	if (path_num == 0) {
#ifndef FWUPGTOOL
		common_nvm_reg_read(MISC_REG_SHARED_MEM_ADDR, &shmem_base);
#else
		iNvm.m_td_read_reg32(dev_index, MISC_REG_SHARED_MEM_ADDR, &shmem_base);
#endif
	} else {
#ifndef FWUPGTOOL
		if (!is_e1x(globalInterp)) {
			common_nvm_reg_read(MISC_REG_GENERIC_CR_0, &shmem2_base);
#else
		if (!COMMON_NVM_IS_E1x()) {
			iNvm.m_td_read_reg32(dev_index, MISC_REG_GENERIC_CR_0, &shmem2_base);
#endif
		if (shmem2_base != 0) {
			/* Path 1 shmem_base*/
#ifndef FWUPGTOOL
			common_nvm_reg_read(shmem2_base + OFFSETOF(shmem2_region_t, other_shmem_base_addr), &shmem_base);
#else
			iNvm.m_td_read_reg32(dev_index, (shmem2_base + OFFSETOF(shmem2_region_t, other_shmem_base_addr)), &shmem_base);
#endif
		}
	}
}

if (shmem_base == 0) {
	printf("Failed to read shmem_base address !\n");
	return COMMON_NVM_STATUS_FAILURE;
}

// Get current port number
if (common_nvm_get_port_num(&port_n) != COMMON_NVM_STATUS_SUCCESS) {
	return COMMON_NVM_STATUS_FAILURE;
}
port_num =(u8_t)port_n;


// Convert input offset in bits to offset in bytes
offset_in_bytes = offset/8;

// Convert the offset in nvm_cfg struct to shmem address
// ----------------------------------------------------------
if ((offset_in_bytes >= NVM_CFG_OFFSET(shared_manuf)) && (offset_in_bytes < NVM_CFG_OFFSET(port_manuf))) {
	*shmem_addr_ptr = offset_in_bytes - NVM_CFG_OFFSET(shared_manuf)+ OFFSETOF(struct shmem_region, dev_info.shared_hw_config);

} else if ((offset_in_bytes >=  NVM_CFG_OFFSET(port_manuf)) && (offset_in_bytes < NVM_CFG_OFFSET(shared_feat))) {
	*shmem_addr_ptr = offset_in_bytes - NVM_CFG_OFFSET(port_manuf)+ OFFSETOF(struct shmem_region, dev_info.port_hw_config[port_num]);

} else if ((offset_in_bytes >=  NVM_CFG_OFFSET(shared_feat)) && (offset_in_bytes < NVM_CFG_OFFSET(port_feat))) {
	*shmem_addr_ptr =  offset_in_bytes - NVM_CFG_OFFSET(shared_feat)+ OFFSETOF(struct shmem_region, dev_info.shared_feature_config);

} else if ((offset_in_bytes >=  NVM_CFG_OFFSET(port_feat)) && (offset_in_bytes <  NVM_CFG_OFFSET(vpd_info))) {
	*shmem_addr_ptr = offset_in_bytes - NVM_CFG_OFFSET(port_feat)+ OFFSETOF(struct shmem_region, dev_info.port_feature_config[port_num]);

} else {
	return COMMON_NVM_STATUS_FAILURE;
}
*shmem_addr_ptr += shmem_base;

return COMMON_NVM_STATUS_SUCCESS;
}
#endif // !FWUPGTOOL

/*
 *------------------------------------------------------------------------
 * common_nvm_store_shmem --
 *
 *     This function writes the requested shmem_cfg_ptr into shmem.
 *     This function is used by common_nvm_store_cfg().
 *
 *------------------------------------------------------------------------
 */
void common_nvm_store_shmem(
							u32_t shmem_base,
							u8_t update_port_cfg,
							u8_t port_num,
							nvm_cfg_t *shmem_cfg_ptr
							) {
	u32_t *shmem_ptr;
	u32_t i, addr;
#ifdef FWUPGTOOL
	NVM_INTERFACE iNvm = nvm_interface(gpINx2Nvm->m_tcl_interp);
	u32_t dev_index = iNvm.m_current_dev_index;
#endif // FWUPGTOOL

	/* Update the shared_hw_config in shmem_dev_info*/
	shmem_ptr =(u32_t *)&shmem_cfg_ptr->shared_manuf;
	addr = OFFSETOF(struct shmem_region, dev_info.shared_hw_config);
	for (i = 0; i < sizeof(shared_hw_cfg_t) / 4; i++) {
#ifndef FWUPGTOOL
		common_nvm_reg_write(shmem_base + addr + i * 4, *(shmem_ptr + i));
#else
		iNvm.m_td_write_reg32(dev_index, (shmem_base + addr + i * 4), *(shmem_ptr + i));
#endif
	}

	/* Update the shared_feat_config in shmem_dev_info*/
	shmem_ptr =(u32_t *)&shmem_cfg_ptr->shared_feat;
	addr = OFFSETOF(struct shmem_region, dev_info.shared_feature_config);
	for (i = 0; i < sizeof(shared_feat_cfg_t) / 4; i++) {
#ifndef FWUPGTOOL
		common_nvm_reg_write(shmem_base + addr + i * 4, *(shmem_ptr + i));
#else
		iNvm.m_td_write_reg32(dev_index, (shmem_base + addr + i * 4), *(shmem_ptr + i));
#endif
	}

	if (update_port_cfg) {
		/* Update the port_hw_config in shmem_dev_info*/
		shmem_ptr =(u32_t *)&shmem_cfg_ptr->port_manuf;
		addr = OFFSETOF(struct shmem_region, dev_info.port_hw_config[port_num]);
		for (i = 0; i < sizeof(port_hw_cfg_t) / 4; i++) {
#ifndef FWUPGTOOL
			common_nvm_reg_write(shmem_base + addr + i * 4, *(shmem_ptr + i));
#else
			iNvm.m_td_write_reg32(dev_index, (shmem_base + addr + i * 4), *(shmem_ptr + i));
#endif
		}

		/* Update the port_feat_config in shmem_dev_info*/
		shmem_ptr =(u32_t *)&shmem_cfg_ptr->port_feat;
		addr = OFFSETOF(struct shmem_region, dev_info.port_feature_config[port_num]);
		for (i = 0; i < sizeof(port_feat_cfg_t) / 4; i++) {
#ifndef FWUPGTOOL
			common_nvm_reg_write(shmem_base + addr + i * 4, *(shmem_ptr + i));
#else
			iNvm.m_td_write_reg32(dev_index, (shmem_base + addr + i * 4), *(shmem_ptr + i));
#endif
		}
	}

}


/*
 *------------------------------------------------------------------------
 * common_nvm_program_nvm_cfg --
 *
 *     This function writes the NVM configuration and requested images.
 *     This function is used by common_nvm_store_cfg().
 *
 * Results:
 *     0 if success, -1 for failure/abort.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_program_nvm_cfg(
											   nvm_cfg_t * cfg_ptr,
											   u8_t path_num,
											   u8_t update_port_cfg,
											   manuf_info_t *manuf_block_ptr,
											   feature_info_t *feature_block_ptr,
											   dir_t *dir_block_ptr,
											   media_vpd_t *vpd_block_ptr,
											   nvm_nic_part_cfg_t *mac_partition_ptr,
											   u32_t *cfg_extended_shared_block_ptr
											   ) {
	u32_t value32, nvm_addr;
	common_nvm_status_t stat = COMMON_NVM_STATUS_SUCCESS;

	/* Update manufacturing block */
	nvm_addr = NVM_OFFSET_PATH(manuf_info.length, path_num); /* first field */
	stat |= common_nvm_program_nvm((u8_t *)manuf_block_ptr, sizeof(manuf_info_t) - 4,
								   nvm_addr, PRG_NVM_APPEND_CRC, &value32);

	/* Update feature block */
	nvm_addr = NVM_OFFSET_PATH(feature_info, path_num); /* first field */
	stat |= common_nvm_program_nvm((u8_t *)feature_block_ptr, sizeof(feature_info_t) - 4,
								   nvm_addr, PRG_NVM_APPEND_CRC, &value32);

	if (update_port_cfg) {
		/* Update directory block */
		nvm_addr = NVM_OFFSET(dir.code[0]); /* first field */
		stat = common_nvm_program_nvm((u8_t *)dir_block_ptr, sizeof(dir_t) - 4,
									  nvm_addr, PRG_NVM_APPEND_CRC, &value32);
		/* Update VPD block */
		nvm_addr = NVM_OFFSET(vpd); /* first field */
		stat |= common_nvm_program_nvm((u8_t *)vpd_block_ptr, sizeof(media_vpd_t),  nvm_addr, 0, &value32);

		/* Update MAC partition image block */
		stat |= common_nvm_program_mac_partition_to_nvm(cfg_ptr, mac_partition_ptr);

		/* update extended nvm_cfg shared block */
		stat |= common_nvm_program_extended_nvm_cfg_shared_block(cfg_ptr, cfg_extended_shared_block_ptr);
	}

	return stat;

}


/*
 *------------------------------------------------------------------------
 * common_nvm_store_cfg --
 *
 *     This function implements the "nvm_store" command.
 *     It stores the requested cfg_ptr into NVM.
 *     Also stores the shmem_cfg_ptr into shmem.
 *
 * Results:
 *     0 if success, -1 for failure/abort.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_store_cfg(
										 nvm_cfg_t * cfg_ptr,
										 nvm_cfg_t * shmem_cfg_ptr,
										 nvm_nic_part_cfg_t * mac_partition_ptr,
										 u32_t * cfg_extended_shared_block_ptr
										 ) {
	manuf_info_t manuf_block;
	feature_info_t feature_block;
	media_vpd_t vpd_block;
	dir_t dir_block;
	u8_t port_num;
	common_nvm_status_t stat;
	u32_t port_n;
	u8_t path_num;
	u8_t total_path_num;
	u8_t cur_path_num;
	u32_t shmem_base[MAX_PATH_NUM]={ 0, 0 };
	u32_t shmem2_base[MAX_PATH_NUM]={ 0, 0 };
	u8_t  update_port_cfg=0;

#ifndef FWUPGTOOL
	common_nvm_dev_info_t *cur_dev_p = COMMON_NVM_CUR_DEV;
	stat = common_nvm_get_port_num(&port_n);
	if (stat != 0) return stat;

	port_num =(u8_t)port_n;
	path_num = COMMON_NVM_PATH_NUM(*cur_dev_p);
	total_path_num = is_e1x(globalInterp)? 1 : 2;

#else
	u32_t   idx;
	u32_t   *ptr32;
	NVM_INTERFACE iNvm = nvm_interface(gpINx2Nvm->m_tcl_interp);
	u32_t dev_index = iNvm.m_current_dev_index;

	stat = common_nvm_get_port_num(&port_n);
	if (stat != 0) return stat;

	port_num =(u8_t)port_n;
	path_num = COMMON_NVM_PATH_NUM();
	total_path_num = COMMON_NVM_IS_E1x()? 1 : 2;
#endif // !FWUPGTOOL

#ifndef FWUPGTOOL
	/* Get shmem base address of path0 and path 1*/
	if (!common_nvm_tcl_al_get_virtual_nvm_mode()) {
		common_nvm_reg_read(MISC_REG_SHARED_MEM_ADDR, &shmem_base[0]);
	}
	if (!is_e1x(globalInterp)) {
		if (!common_nvm_tcl_al_get_virtual_nvm_mode()) {
			common_nvm_reg_read(MISC_REG_GENERIC_CR_0, &shmem2_base[0]);
		}
#else
	/* Get shmem base address of path0 and path 1*/
	iNvm.m_td_read_reg32(dev_index, MISC_REG_SHARED_MEM_ADDR, &shmem_base[0]);
	if (!COMMON_NVM_IS_E1x()) {
		iNvm.m_td_read_reg32(dev_index, MISC_REG_GENERIC_CR_0, &shmem2_base[0]);
#endif // !FWUPG_TOOL

	/* Path 1 shmem_base and shmem2_base*/
	if (!common_nvm_tcl_al_get_virtual_nvm_mode() && (shmem2_base[0] != 0)) {
#ifndef FWUPGTOOL
		common_nvm_reg_read(shmem2_base[0] + OFFSETOF(shmem2_region_t, other_shmem_base_addr), &shmem_base[1]);
		common_nvm_reg_read(MISC_REG_GENERIC_CR_1, &shmem2_base[1]);
#else
		iNvm.m_td_read_reg32(dev_index, (shmem2_base[0] + OFFSETOF(shmem2_region_t, other_shmem_base_addr)), &shmem_base[1]);
		iNvm.m_td_read_reg32(dev_index, MISC_REG_GENERIC_CR_1, &shmem2_base[1]);
#endif
	}
}

for (cur_path_num = 0; cur_path_num < total_path_num; ++cur_path_num) {
	// vpd / dir - out of the loop - not duplicated
	// if store_path_number == current_poth_number - store both port and shared
	// otherwise, only shared

	// Check if we should update port configuration or not
	if (cur_path_num == path_num) {
		update_port_cfg=1;
	} else {
		update_port_cfg=0;
	}

	if (update_port_cfg) {
		/* This condition is only to make sure
		   That the VPD and dir are only read once. */

		/* Doing read-modify-write. */
		common_nvm_read_nvram(NVM_OFFSET(dir.code[0]), (u32_t *)&dir_block, sizeof(dir_t));

		/* Doing read-modify-write. */
		common_nvm_read_nvram(NVM_OFFSET(vpd), (u32_t *)&vpd_block, sizeof(media_vpd_t));
	}

	/* Doing read-modify-write. */
	common_nvm_read_nvram(NVM_OFFSET_PATH(feature_info.shared_feature_config.config, cur_path_num), (u32_t *)&feature_block, sizeof(feature_info_t));

	/* Doing read-modify-write. */
	common_nvm_read_nvram(NVM_OFFSET_PATH(manuf_info.length, cur_path_num), (u32_t *)&manuf_block, sizeof(manuf_info_t));

	/* Assign the format revision */
	//feature_block.shared_feature_config.format_rev = SHARED_FEATURE_CFG_FORMAT_REV_ID;
	/* AMIR: TO CHECK: what is this*/
	manuf_block.length |= MANUF_INFO_FORMAT_REV_ID;

	/* Part number */
	strncpy((char *)(cfg_ptr->shared_manuf.part_num), (char *)cfg_ptr->vpd_info.part_number, ARRSIZE(cfg_ptr->shared_manuf.part_num));
	strncpy((char *)(shmem_cfg_ptr->shared_manuf.part_num), (char *)cfg_ptr->shared_manuf.part_num, ARRSIZE(cfg_ptr->shared_manuf.part_num));
#ifdef FWUPGTOOL
	// All strings saved in NVRAM are in LE format...
	swap_buffer_if_big_endian_host((u32_t *)cfg_ptr->shared_manuf.part_num, ARRSIZE(cfg_ptr->vpd_info.part_number) / sizeof(u32_t));
	swap_buffer_if_big_endian_host((u32_t *)shmem_cfg_ptr->shared_manuf.part_num, ARRSIZE(cfg_ptr->shared_manuf.part_num) / sizeof(u32_t));
#endif

	/************ Common manufactoring information ************/
	manuf_block.length = MANUF_INFO_FORMAT_REV_ID | MANUF_INFO_LENGTH;

	/**************** Common manufactoring information *******************/
	common_nvm_memcpy(&manuf_block.shared_hw_config, &cfg_ptr->shared_manuf, sizeof(shared_hw_cfg_t));


	/*********** Shared Feature Configuration ********************************/
	common_nvm_memcpy(&feature_block.shared_feature_config, &cfg_ptr->shared_feat, sizeof(shared_feat_cfg_t));


	if (update_port_cfg) {
		/* These 2 writes should only occur in the
		   specified path. The vpd info setting, should also
		   occur only once.*/

		/**************** Per port manufactoring information ************************/
		common_nvm_memcpy(&manuf_block.port_hw_config[port_num], &cfg_ptr->port_manuf, sizeof(port_hw_cfg_t));

		/*********** Per port Feature Configuration ********************************/
		common_nvm_memcpy(&feature_block.port_feature_config[port_num], &cfg_ptr->port_feat, sizeof(port_feat_cfg_t));

		/* VPD info */
		common_nvm_set_vpd_info(&cfg_ptr->vpd_info, &vpd_block);
		/* Spare number */
		strncpy((char *)(dir_block.spare_part_num), (char *)(cfg_ptr->spare_num),
				ARRSIZE(dir_block.spare_part_num));

#ifdef FWUPGTOOL
		// The dir_block.spare_part_num needs to be convert to LE format before
		// saving to the NVRAM for the BE host.
		ptr32 =(u32_t *)dir_block.spare_part_num;
		for (idx = 0; idx < ARRSIZE(dir_block.spare_part_num); idx += 4, ptr32++) {
			*ptr32 = BMAPI_BENDIAN_SWAP(*ptr32); // convert string in buffer to BE for BE host
		}
#endif
	}

	stat = common_nvm_program_nvm_cfg(cfg_ptr, cur_path_num, update_port_cfg,
									  &manuf_block, &feature_block,
									  &dir_block, &vpd_block,
									  mac_partition_ptr,
									  cfg_extended_shared_block_ptr);

	if ((stat == COMMON_NVM_STATUS_SUCCESS) && (shmem_base[path_num] != 0)) {
		/* Update manufacture & feature blocks in shmem */
		common_nvm_store_shmem(shmem_base[cur_path_num], update_port_cfg, port_num, shmem_cfg_ptr);
	}
}


/* Returns status */
return stat;
}



#ifndef FWUPGTOOL
/*
 *------------------------------------------------------------------------
 * nvm_program_image --
 *
 *     Program an image and update the directory, based on code_image
 *     defined in md_rgn.h.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     Error message (if any) will be appended to the result object in
 *     interp.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_nvm_program_image(
												 u8_t * buf,
												 u32_t byte_cnt,
												 u32_t code_image
												 ) {
	u32_t idx, rc;

	for (idx = 0; idx < sizeof(image_table) / sizeof(image_map_t); idx++) {
		if (image_table[idx].type == code_image) {
			rc = common_nvm_prog_image_by_idx(buf, byte_cnt, idx, 0, 0);
			return rc;
		}
	}
	return COMMON_NVM_STATUS_FAILURE;
}
#endif // !FWUPGTOOL

static __inline void _common_nvm_nvm_copy_fw_data(
												  IN    DIR_ENTRY *pdir,     /* directory entry */
												  IN    u8_t      *fw_img,   /* fw image source */
												  INOUT u8_t      *buf,      /* destination buffer */
												  IN    u32_t       sram_addr /* SRAM address of the "text" section */
												  ) {
	u32_t pdir_addr = BMAPI_BENDIAN_SWAP(pdir->address);
	u32_t pdir_offset = BMAPI_BENDIAN_SWAP(pdir->offset);
	u32_t pdir_length = BMAPI_BENDIAN_SWAP(pdir->length);

	if ((pdir_addr >= sram_addr) && (pdir_offset > 0)) {
		u32_t addr = pdir_addr - sram_addr;
		memcpy(buf + addr, fw_img  + pdir_offset, pdir_length);
	}
}

#ifdef FWUPGTOOL
// defined in bnx2x_reg.h, 57710_reg.h, 57711_reg.h
#define TSEM_REG_PRAM                        0x1c0000
#define CSEM_REG_PRAM                        0x240000
#define XSEM_REG_PRAM                        0x2c0000
#define USEM_REG_PRAM                        0x340000
#endif

common_nvm_status_t common_nvm_perform_image_manipulation(u8_t * *buf,
														  u8_t * *newbuf_ptr,
														  u32_t image_type,
														  u32_t agent_type,
														  u32_t *sram_addr,
														  u32_t *byte_cnt) {
	FWIMG_HDR fwimg_hdr;
	u32_t max_addr, max_addr_length;
	DIR_ENTRY *dir_entry_ptr;
	u32_t data32;
	u32_t rc = 0;
	u32_t cnt;
#ifndef FWUPGTOOL
	common_nvm_dev_info_t *cur_dev_p = COMMON_NVM_CUR_DEV;
#else
	u32_t port_num;
	u32_t path_num;
	u32_t   pci_id_offset;
#endif


	common_nvm_memcpy(&fwimg_hdr, *buf, sizeof(FWIMG_HDR));

	if (BMAPI_BENDIAN_SWAP(fwimg_hdr.magic) == FW_IMG_MAGIC) {
		*sram_addr = BMAPI_BENDIAN_SWAP(fwimg_hdr.text.address);

		max_addr = 0;
		max_addr_length = 0;
		for (dir_entry_ptr = &(fwimg_hdr.text); dir_entry_ptr <= &(fwimg_hdr.sdata); dir_entry_ptr++) {
			if (BMAPI_BENDIAN_SWAP(dir_entry_ptr->address) > max_addr) {
				max_addr = BMAPI_BENDIAN_SWAP(dir_entry_ptr->address);
				max_addr_length = BMAPI_BENDIAN_SWAP(dir_entry_ptr->length);
			}
		}
		cnt = max_addr - *sram_addr + max_addr_length;

		// Allocate cnt + 4 bytes for CRC
		*newbuf_ptr =(u8_t *)(uintptr_t)common_nvm_alloc_virt(cnt + 4);

		if (newbuf_ptr == NULL) {
			DbgMessage(NULL, FATAL, "memory allocation error\n");
			return COMMON_NVM_STATUS_RESOURCE;
		}
		common_nvm_memset(*newbuf_ptr, 0, cnt);

		_common_nvm_nvm_copy_fw_data(&(fwimg_hdr.text), *buf, *newbuf_ptr, *sram_addr);
		_common_nvm_nvm_copy_fw_data(&(fwimg_hdr.ronly_data), *buf, *newbuf_ptr, *sram_addr);
		_common_nvm_nvm_copy_fw_data(&(fwimg_hdr.data), *buf, *newbuf_ptr, *sram_addr);
		_common_nvm_nvm_copy_fw_data(&(fwimg_hdr.sbss), *buf, *newbuf_ptr, *sram_addr);
		_common_nvm_nvm_copy_fw_data(&(fwimg_hdr.bss), *buf, *newbuf_ptr, *sram_addr);


		if (BMAPI_BENDIAN_SWAP(fwimg_hdr.major_ver) >= 1) {
			_common_nvm_nvm_copy_fw_data(&(fwimg_hdr.sdata), *buf, *newbuf_ptr, *sram_addr);
		}

		*buf = *newbuf_ptr;
		*byte_cnt = cnt;
	} else {
		// Set default address according to the HOST
		switch (agent_type) {
		case CODE_IMAGE_AGENT_TSTORM:
			*sram_addr = TSEM_REG_PRAM;
			break;
		case CODE_IMAGE_AGENT_CSTORM:
			*sram_addr = CSEM_REG_PRAM;
			break;
		case CODE_IMAGE_AGENT_XSTORM:
			*sram_addr = XSEM_REG_PRAM;
			break;
		case CODE_IMAGE_AGENT_USTORM:
			*sram_addr = USEM_REG_PRAM;
			break;
		default:
			*sram_addr = NVM_DEFAULT_SRAM_ADDR;
		}
	}

	/* Handle image-specific things */
	/* CQ#51037: E3_WC and E3_PCIE fw should have sram address as 0 */
	if ((image_type == CODE_IMAGE_TYPE_MODULES_PN) ||
		(image_type == CODE_IMAGE_TYPE_E3_WC) ||
		(image_type == CODE_IMAGE_TYPE_E3_PCIE) ||
		(image_type == CODE_IMAGE_TYPE_E3_WCV2) ||
		(image_type == CODE_IMAGE_TYPE_E3_PCIEV2) ||
		(image_type == CODE_IMAGE_TYPE_OCNVM)) {
		*sram_addr = 0;
	}

	if (image_type == CODE_IMAGE_TYPE_MBA) {
		u16_t device_id, vendor_id;

		*sram_addr = 0;
		/* Update VID and DID*/
#ifndef FWUPGTOOL
		rc = common_nvm_read_nvram(NVM_OFFSET(manuf_info.port_hw_config[common_nvm_get_func_num_from_dev_info(cur_dev_p)].pci_id),
								   &data32, sizeof(data32));
#else
		// To get the correct PCI ID for the function, we need to use both
		// the port_num and path_num to find the PCI ID offset in NVRAM.
		if (common_nvm_get_port_num(&port_num) != COMMON_NVM_STATUS_SUCCESS) {
			return COMMON_NVM_STATUS_FAILURE;
		}

		path_num = COMMON_NVM_PATH_NUM();

		pci_id_offset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port_num].pci_id, path_num);
		rc = common_nvm_read_nvram(pci_id_offset, &data32, sizeof(u32_t));
#endif // !FWUPGTOOL
		if (rc != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Reading VID/DID from NVM failed.");
#ifdef FWUPGTOOL
			return COMMON_NVM_STATUS_FAILURE; // should not continue
#endif // FWUPGTOOL
		}

		device_id =(u16_t)MANUF_INFO_PCI_DEVICE_ID(data32);
		vendor_id =(u16_t)MANUF_INFO_PCI_VENDOR_ID(data32);
		rc = common_nvm_update_mba_hdr(*buf, vendor_id, device_id);
		if (rc != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Not a valid NetXtreme II MBA image.");
		}
	}

	return(rc == 0) ? COMMON_NVM_STATUS_SUCCESS : COMMON_NVM_STATUS_FAILURE;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_is_bcv_image_type --
 *
 *     Check if image is BCV image, per CQ69138 "Show HOSTBCV in nvm dir
 *     'agent' column for BCV image"
 *
 * Results:
 *     TRUE/FALSE
 *
 *------------------------------------------------------------------------
 */

u8_t common_nvm_is_bcv_image_type(u8_t * buf) {
	FWIMG_HDR fwimg_hdr;
	DIR_ENTRY tmp_dir_entry;

	common_nvm_memcpy(&fwimg_hdr, buf, sizeof(FWIMG_HDR));
	tmp_dir_entry = fwimg_hdr.text;
	if ((BMAPI_BENDIAN_SWAP(tmp_dir_entry.offset)
		 & BCV_IMAGE_TYPE_MASK)
		== BCV_IMAGE_TYPE) {
		return TRUE;
	} else {
		return FALSE;
	}

}

/*
 *------------------------------------------------------------------------
 * prog_image_by_idx --
 *
 *     Program an image and update the directory, based on the info from the idx.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     Error message (if any) will be appended to the result object in
 *     interp.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_prog_image_by_idx(
												 u8_t * buf,
												 u32_t byte_cnt,
												 u32_t image_table_idx,
												 u8_t validate_versions,
												 u8_t mbi_upgrade
												 ) {
	u32_t data32, entry_idx, ib_cprg_entry_idx;
	u32_t rc;
	u32_t image_type, agent_type, sram_addr, byte_prg, nvm_addr, old_nvm_addr, curr_func, max_funcs;
	u8_t *newbuf_ptr =  NULL;
	u8_t same_image = 0;
#ifndef FWUPGTOOL
	common_nvm_dev_info_t *cur_dev_p = COMMON_NVM_CUR_DEV;
#else
	// 57710 use ENV_PORT0 to determine port.
	// ENV_PORT0 was set up in SetSelectedNIC() of Bnx2Helper.cpp
	u8_t is_port0 = atoi(Tcl_GetVar2(gpINx2Nvm->m_tcl_interp, TCL_ENV, ENV_PORT0, 0));
	u8_t port_id = is_port0 ? 0:1; // set the port ID
	NVM_INTERFACE iNvm = nvm_interface(gpINx2Nvm->m_tcl_interp);
#endif // !FWUPGTOOL
	char old_version[MAX_VER_STR_SIZE]="N/A";
	char new_version[MAX_VER_STR_SIZE]="N/A";
	u8_t image_existed;
	nvm_cfg_t cfg, shmem_cfg;
	nvm_nic_part_cfg_t mac_partition_cfg;
	u32_t cfg_extended_shared_block[NVM_CFG_EXTENDED_SHARED_IMAGE_MAX_SIZE];
	extended_dir_image_t extended_dir;
	int tempVer[3]={ 0, 0, 0 };


	do {
		image_type = image_table[image_table_idx].type;

		/* Special treatment for iSCSI boot data section - if was called with explict option and not in the
		   cumulative image (-ib) */
		if (image_type == CODE_IMAGE_TYPE_ISCSI_BOOT_CFG) {
#ifndef FWUPGTOOL
			if (!COMMON_NVM_CUR_DEV_IS_VALID) {
				return COMMON_NVM_STATUS_FAILURE;
			}

			/* Select ISCSI boot data according to the currently selected device */
			if (common_nvm_get_func_num_from_dev_info(cur_dev_p) == 1) {
				image_type = CODE_IMAGE_TYPE_ISCSI_BOOT_CFG2;
			}
#else
			/* Select ISCSI boot data according to the currently selected device */
			if (port_id == 1) {
				image_type = CODE_IMAGE_TYPE_ISCSI_BOOT_CFG2;
			}
#endif // !FWUPGTOOL
		}

		DbgMessage(NULL, INFORM, "Programming image with type 0x%x, size 0x%x\n", image_type, byte_cnt);

		agent_type = image_table[image_table_idx].agent;

		/* CQ69138 Show HOSTBCV in nvm dir 'agent' column for BCV image */
		if (image_type == CODE_IMAGE_TYPE_ISCSI_BOOT) {
			if (common_nvm_is_bcv_image_type(buf)) {
				agent_type = CODE_IMAGE_AGENT_HOST_BCV;
			}
		}

		rc = common_nvm_perform_image_manipulation(&buf, &newbuf_ptr, image_type, agent_type, &sram_addr, &byte_cnt);
		if (rc != COMMON_NVM_STATUS_SUCCESS) {
			break;
		}

		/* Find an existing matching entry or empty one if it does not exist. */
		rc = common_nvm_nvm_find_dir_entry(image_type, &entry_idx, &old_nvm_addr);

		if (rc != COMMON_NVM_STATUS_SUCCESS) {
			if (image_type == CODE_IMAGE_TYPE_BC1) {
				// extract old version only for BC case - was returned by dg_nvm_find_dir_entry although failed.
				common_nvm_fw_ver_string(old_nvm_addr, image_table[image_table_idx].name, old_version, READ_VERSION_FROM_NVM);
			}
			rc = common_nvm_find_empty_entry(&entry_idx);
			if (rc != 0) {
				DbgMessage(NULL, FATAL, "No directory entry left.");
				break;
			}
			image_existed=0;
		} else {
			image_existed=1;
			common_nvm_fw_ver_string((common_nvm_virt_addr_t)old_nvm_addr, image_table[image_table_idx].name, old_version, READ_VERSION_FROM_NVM);
		}

		common_nvm_fw_ver_string((common_nvm_virt_addr_t)(uintptr_t)buf, image_table[image_table_idx].name, new_version, READ_VERSION_FROM_IMAGE);

		if (image_existed || ((image_type == CODE_IMAGE_TYPE_BC1) && (old_nvm_addr != 0))) {
			// if bootstrap was 0 - we assumed that it was erased by filling nvm with 0's and not 0 address
			// of the BC1 image.   if such BC1 address can exist, further implementation to this code
			// is needed - for example validating the BC1 size in bootstrap to be 0 too.
			if (validate_versions) {
				if (image_type != CODE_IMAGE_TYPE_MBA) {
					rc = common_nvm_image_version_validation(image_table[image_table_idx].name, old_version, new_version, &same_image);
				} else {
					rc = common_nvm_image_mba_version_validation(old_nvm_addr, (common_nvm_virt_addr_t)(uintptr_t)buf, &same_image);
				}
				if (rc != COMMON_NVM_STATUS_SUCCESS) {
#ifdef FWUPGTOOL
					HANDLE_ERROR(iNvm, TRUE, FWUPG_FW_IMAGE_WRONG_VERSION);
#endif
					if (mbi_upgrade && same_image) {
						/* Don't fail MBI upgrade if a single image is the same */
						rc = COMMON_NVM_STATUS_SUCCESS;
					}
					break;
				}

				printf("Upgrading %s image: from ver %s to ver %s\n", image_table[image_table_idx].name, old_version, new_version);

			} else {
				printf("Forced upgrading %s image: from ver %s to ver %s\n", image_table[image_table_idx].name, old_version, new_version);
			}
		} else {
			printf("Upgrading %s image: first time image, ver is %s\n", image_table[image_table_idx].name, new_version);
		}

		/* If programing BC1, delete MFW+SWIM images (if exist in extended dir) */
		if (image_type == CODE_IMAGE_TYPE_BC1) {
			common_nvm_delete_mfw_and_swim_images(DELETE_MFW_CMD_DELETE_ALL_MFWS, 1);
		}

		/* If programing ISCSI_B version 7.0.x and above, delete I_B_CPRG (if exist) */
		if (image_type == CODE_IMAGE_TYPE_ISCSI_BOOT)  {
			sscanf(new_version, "v%d.%d.%d", &tempVer[0], &tempVer[1], &tempVer[2]);
			if (tempVer[0] >= 7) {
				if (common_nvm_nvm_find_dir_entry(CODE_IMAGE_TYPE_ISCSI_BOOT_CPRG, &ib_cprg_entry_idx, NULL) == COMMON_NVM_STATUS_SUCCESS) {

					// CQ60109 request: Erase the ISCSI_BOOT_CPRG entry incase of ISCSI_BOOT ver 7.0.0 and above
					printf("Remove ISCSI_BOOT_CPRG image.\n");
					common_nvm_update_dir_entry(ib_cprg_entry_idx, NVM_ERASE_VALUE, NVM_ERASE_VALUE,
												NVM_ERASE_VALUE, NVM_ERASE_VALUE, NVM_ERASE_VALUE);

				}
			}

		}

		/* NOTE: The common_nvm_find_room() should be after common_nvm_image_version_validation(), since this procedure also writes to NVM
		   and if version_validation is not successful, we break after corrupting the NVM */
		rc = common_nvm_find_room(image_type, byte_cnt, &extended_dir, 0, 0, &nvm_addr);
		if (rc != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Not enough space in NVM.");
			break;
		}

		// Make the data in buf to be in BE if the host is BE before calling program_nvm()
		swap_buffer_if_big_endian_host((u32_t *)buf, (byte_cnt / sizeof(u32_t)));
		rc = common_nvm_program_nvm(buf, byte_cnt, nvm_addr,
									PRG_NVM_LITTLE_ENDIAN | PRG_NVM_APPEND_CRC,
									&byte_prg);
		if (rc != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Programming NVM failed.");
			break;
		}

		/* Handle image-specific things */
		if (image_type == CODE_IMAGE_TYPE_BC1) {
			data32 = common_nvm_fw_ver_code(nvm_addr);

			/* Update bootstrap */
			rc = common_nvm_update_bootstrap(sram_addr, byte_prg, nvm_addr);
			if (rc != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Bootstrap update in NVM failed.");
				break;
			}

		} else {
			if (image_type == CODE_IMAGE_TYPE_MBA) {
				/* Handle MBA specific stuffs. */
				/*nvm_cfg_t cfg;*/
				u32_t bitshift;
				char        valstr[8];

				for (bitshift = 10; bitshift < 24; bitshift++) {
					if (byte_cnt <= (u32_t)(1 << bitshift)) {
						/* Adjust to the configuration value */
						bitshift -= 10;
						sprintf(valstr, "%d", bitshift);
						break;
					}
				}
				max_funcs = common_nvm_number_of_everest_functions();

				for (curr_func = 0; curr_func < max_funcs; ++curr_func) {
					if (common_nvm_load_cfg(&cfg, &shmem_cfg, &mac_partition_cfg,
											&cfg_extended_shared_block[0]) != COMMON_NVM_STATUS_SUCCESS) {
						DbgMessage(NULL, FATAL, "dg_load_cfg failed\n");
						return rc;
					}

					// TODO - Add validation of the values (bitshift)
					cfg.port_feat.mba_config &= ~PORT_FEATURE_MBA_EXP_ROM_SIZE_MASK;
					cfg.port_feat.mba_config |= bitshift << PORT_FEATURE_MBA_EXP_ROM_SIZE_SHIFT;

					if ((rc = common_nvm_store_cfg(&cfg, &shmem_cfg, &mac_partition_cfg, &cfg_extended_shared_block[0]))
						!= COMMON_NVM_STATUS_SUCCESS) {
						DbgMessage(NULL, FATAL, "dg_store_cfg failed\n");
						return rc;
					}

					if (common_nvm_number_of_everest_functions() > 1) {
						if ((rc = common_nvm_select_other_func()) != COMMON_NVM_STATUS_SUCCESS) {
							DbgMessage(NULL, FATAL, "Failed to select another function\n");
							return rc;
						}
					}
				}
			}

			rc = common_nvm_update_dir_entry(entry_idx, nvm_addr,
											 byte_prg, sram_addr, image_type, agent_type);
			if (rc != 0) {
				DbgMessage(NULL, FATAL, "Directory update in NVM failed\n");
				break;
			}
		}

		if ((image_type == CODE_IMAGE_TYPE_BC2) || (image_type == CODE_IMAGE_TYPE_BC1)) {
			DbgMessage(NULL, INFORMnv, "The upgrade won't take effect until the next chip reset.");
		}

		/* Clean up if needed */
		if (newbuf_ptr != NULL) {
			common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)newbuf_ptr);
		}

		rc = 0;
	} while (0);

	return(rc == 0) ? COMMON_NVM_STATUS_SUCCESS : COMMON_NVM_STATUS_FAILURE;
}


/*
 *------------------------------------------------------------------------
 * common_nvm_dump_nvm_into_buf --
 *
 *     Dump NVM content into a byte array
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_dump_nvm_into_buf(
												 u8_t * buf,
												 u32_t big_endian,
												 u32_t byte_cnt,
												 u32_t offset,
												 u32_t *byte_dumped
												 ) {
	u32_t start, end,  idx, data, *ptr32;
	common_nvm_status_t status=(common_nvm_status_t)COMMON_NVM_STATUS_FAILURE;


	/* Make sure offset and length are 4-byte aligned */
	start = offset & 0xfffffffc;
	end = (offset+byte_cnt+3) & 0xfffffffc;

	/* Read NVM content and write to file */
	ptr32 =(u32_t *)buf;
	if (byte_dumped != NULL) *byte_dumped = 0;

#ifndef FWUPGTOOL
	for (idx = start; idx < end; idx += 4, ptr32++) {
		status = common_nvm_read_nvram(idx, &data, sizeof(data));
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			printf("NVM read failed.");
			break;
		}
		*ptr32 = (big_endian) ? htonl(data): data;
		if (byte_dumped != NULL) *byte_dumped += 4;
		/*if (toe_kbhit())
		{
			break;
		}*/
	}
#else

	// read all data from NVRAM to buffer

	status = common_nvm_read_nvram(start, (u32_t *)buf, byte_cnt);

	if (status != COMMON_NVM_STATUS_SUCCESS) {
		printf("NVM read failed.");
	} else {
		// swap each data word in the buffer if necessary

		for (idx = start; idx < end; idx += 4, ptr32++) {
			data = *ptr32;
			*ptr32 = (big_endian) ? htonl(data): data;
		}

		if (byte_dumped) {
			*byte_dumped = byte_cnt;
		}
	}

#endif // !FWUPGTOOL

	if (idx >= end) {
		status = COMMON_NVM_STATUS_SUCCESS;
	} else {
		/* User abort, report it out. */
		printf("User abort\n");
	}

	//printf("nvm_dump_nvm: buf=%p; len=%d; offset=%x; byte_dumped=%d\n",
	//        buf, byte_cnt, offset, *byte_dumped);
	//toe_flush();

	return status;
}

#define MACP_BW_MAX_DEFAULT_VAL         100

#define DELL_SDID_1 0x1006
#define DELL_SDID_2 0x1007
#define DELL_SDID_3 0x1008
#define DELL_SDID_4 0x1014

void add_shadow_dell_ncsi_image() {
	u32_t shmem_base, val, addr, found_image, nvm_addr, byte_cnt, entry_idx, status, index_image_table;
	u32_t svid, sdid;
	shadow_image_t shadow_image;

	if (common_nvm_tcl_al_get_virtual_nvm_mode()) {
		return;
	}

	/* read subsystem vendor ID from shmem, and proceed only if it matches Dell */
	common_nvm_reg_read(MISC_REG_SHARED_MEM_ADDR, &shmem_base);
	addr = OFFSETOF(struct shmem_region, dev_info.port_hw_config[0].pci_sub_id);
	common_nvm_reg_read(shmem_base + addr, &val);
	svid = (val & PORT_HW_CFG_PCI_SUBSYS_VENDOR_ID_MASK) >> PORT_HW_CFG_PCI_SUBSYS_VENDOR_ID_SHIFT;
	sdid = (val & PORT_HW_CFG_PCI_SUBSYS_DEVICE_ID_MASK) >> PORT_HW_CFG_PCI_SUBSYS_DEVICE_ID_SHIFT;

	if ((svid == OEM_ID_1) ||
		((svid == BROADCOM_VID) &&
		 ((sdid == DELL_SDID_1) ||
		  (sdid == DELL_SDID_2) ||
		  (sdid == DELL_SDID_3) ||
		  (sdid == DELL_SDID_4)))) {
		/* if image already exists - do nothing */
		status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_BOOT_CFG_SHADOW, &nvm_addr, &byte_cnt, &entry_idx);
		if ((status ==  COMMON_NVM_STATUS_SUCCESS) && (found_image)) {
			return;
		}
		status = common_nvm_find_entry_in_image_table(CODE_IMAGE_TYPE_BOOT_CFG_SHADOW, &index_image_table);
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			return;
		}

		/* if reached here - should program new empty shadow image to nvram */
		memset(&shadow_image, 0, sizeof(shadow_image));
		common_nvm_prog_image_in_extended_dir((u8_t *)&shadow_image, sizeof(shadow_image), index_image_table, FALSE, FALSE, FALSE, FALSE);
	}
}


void add_bdn_image() {
	u32_t found_image, nvm_addr, byte_cnt, entry_idx, status, index_image_table;
	common_nvm_dev_stat_t dev_stat;
	union bdn_image bdn_image;

	/* Get vendor ID from NVM, and proceed only if it matches OEM_ID_2 */
	common_nvm_get_info(&dev_stat);
	if (dev_stat.svid != OEM_ID_2) {
		return;
	}

	/* if image already exists - do nothing */
	status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_BDN, &nvm_addr, &byte_cnt, &entry_idx);
	if ((status ==  COMMON_NVM_STATUS_SUCCESS) && (found_image) && (sizeof(union bdn_image) <= byte_cnt)) {
		return;
	}
	status = common_nvm_find_entry_in_image_table(CODE_IMAGE_TYPE_BDN, &index_image_table);
	if (status != COMMON_NVM_STATUS_SUCCESS) {
		return;
	}

	/* if reached here - should program new empty shadow image to nvram */
	memset(&bdn_image, 0, sizeof(union bdn_image));
	common_nvm_prog_image_in_extended_dir((u8_t *)&bdn_image, sizeof(union bdn_image), index_image_table, FALSE, FALSE, FALSE, FALSE);
}


void common_nvm_init_npar_mac_addr(u32_t pf, int is_4port, u32_t is_nicp1_5, u32_t wwn_prefix, nvm_nic_part_cfg_t *npar_block_ptr) {
	u32_t nvm_offset, mac_addr_upper, mac_addr_lower, iscsi_mac_addr_upper, iscsi_mac_addr_lower;
	u32_t path, port;
	u32_t mac_diff=4;

	path = pf & 1;
	port = is_4port ? ((pf >> 1) & 1) : 0;

	nvm_offset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port].mac_upper, path);
	common_nvm_read_nvram(nvm_offset, &mac_addr_upper, 4);
	npar_block_ptr->func_cfg[pf].net_mac_addr_upper = mac_addr_upper;

	nvm_offset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port].mac_lower, path);
	common_nvm_read_nvram(nvm_offset, &mac_addr_lower, 4);
	npar_block_ptr->func_cfg[pf].net_mac_addr_lower = mac_addr_lower;

	if (is_nicp1_5) {
		/* In npar1_5 PF has only one MAC address
		   Note: -num_mac (Mac addresses per port) should be 4 */
		mac_diff=1;
	}

	if (!is_4port) {
		npar_block_ptr->func_cfg[pf].net_mac_addr_lower += (pf >> 1)*mac_diff;
	} else {
		npar_block_ptr->func_cfg[pf].net_mac_addr_lower += (pf >> 2)*mac_diff*2;
	}

	/* set iscsi mac addr according to SF iscsi MAC from NVM */
	nvm_offset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port].iscsi_mac_upper, path);
	common_nvm_read_nvram(nvm_offset, &iscsi_mac_addr_upper, 4);
	npar_block_ptr->func_cfg[pf].iscsi_mac_addr_upper = iscsi_mac_addr_upper;

	nvm_offset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port].iscsi_mac_lower, path);
	common_nvm_read_nvram(nvm_offset, &iscsi_mac_addr_lower, 4);
	npar_block_ptr->func_cfg[pf].iscsi_mac_addr_lower = iscsi_mac_addr_lower;

	if (!is_4port) {
		npar_block_ptr->func_cfg[pf].iscsi_mac_addr_lower += (pf >> 1)*mac_diff;
	} else {
		npar_block_ptr->func_cfg[pf].iscsi_mac_addr_lower += (pf >> 2)*mac_diff*2;
	}

	if (is_nicp1_5) {
		/* In npar1_5 PF has only one MAC address
		   Note: -num_mac (Mac addresses per port) should be 4 */
		npar_block_ptr->func_cfg[pf].iscsi_mac_addr_lower = npar_block_ptr->func_cfg[pf].net_mac_addr_lower;
		npar_block_ptr->func_cfg[pf].iscsi_mac_addr_upper = npar_block_ptr->func_cfg[pf].net_mac_addr_upper;
	}

	/* fcoe fip mac addr is equal to iscsi MAC */
	npar_block_ptr->func_cfg[pf].fcoe_mac_addr_upper = npar_block_ptr->func_cfg[pf].iscsi_mac_addr_upper;
	npar_block_ptr->func_cfg[pf].fcoe_mac_addr_lower = npar_block_ptr->func_cfg[pf].iscsi_mac_addr_lower;

	/* fcoe wwn node mac addr */
	npar_block_ptr->func_cfg[pf].fcoe_node_wwn_upper = npar_block_ptr->func_cfg[pf].fcoe_mac_addr_upper;
	npar_block_ptr->func_cfg[pf].fcoe_node_wwn_lower = npar_block_ptr->func_cfg[pf].fcoe_mac_addr_lower;

	if ((wwn_prefix & EXTENDED_DEV_INFO_SHARED_CFG_WWN_NODE_PREFIX0_MASK) ||
		(wwn_prefix & EXTENDED_DEV_INFO_SHARED_CFG_WWN_NODE_PREFIX1_MASK)) {

		npar_block_ptr->func_cfg[pf].fcoe_node_wwn_upper |= (wwn_prefix & EXTENDED_DEV_INFO_SHARED_CFG_WWN_NODE_PREFIX0_MASK)
		>> EXTENDED_DEV_INFO_SHARED_CFG_WWN_NODE_PREFIX0_SHIFT
		<< 16;
		npar_block_ptr->func_cfg[pf].fcoe_node_wwn_upper |= (wwn_prefix & EXTENDED_DEV_INFO_SHARED_CFG_WWN_NODE_PREFIX1_MASK)
		>> EXTENDED_DEV_INFO_SHARED_CFG_WWN_NODE_PREFIX1_SHIFT
		<< 24;
	} else {
		/* Per CQ76134 if wwn_prefix is 0, set default wwn prefix 10:00 */
		npar_block_ptr->func_cfg[pf].fcoe_node_wwn_upper |= 10 << 24;
	}

	/* fcoe wwn port mac addr */
	npar_block_ptr->func_cfg[pf].fcoe_port_wwn_upper = npar_block_ptr->func_cfg[pf].fcoe_mac_addr_upper;
	npar_block_ptr->func_cfg[pf].fcoe_port_wwn_lower = npar_block_ptr->func_cfg[pf].fcoe_mac_addr_lower;

	if ((wwn_prefix & EXTENDED_DEV_INFO_SHARED_CFG_WWN_PORT_PREFIX0_MASK) ||
		(wwn_prefix & EXTENDED_DEV_INFO_SHARED_CFG_WWN_PORT_PREFIX1_MASK)) {

		npar_block_ptr->func_cfg[pf].fcoe_port_wwn_upper |= (wwn_prefix & EXTENDED_DEV_INFO_SHARED_CFG_WWN_PORT_PREFIX0_MASK)
		>> EXTENDED_DEV_INFO_SHARED_CFG_WWN_PORT_PREFIX0_SHIFT
		<< 16;
		npar_block_ptr->func_cfg[pf].fcoe_port_wwn_upper |= (wwn_prefix & EXTENDED_DEV_INFO_SHARED_CFG_WWN_PORT_PREFIX1_MASK)
		>> EXTENDED_DEV_INFO_SHARED_CFG_WWN_PORT_PREFIX1_SHIFT
		<< 24;
	} else {
		/* Per CQ76134 if wwn_prefix is 0, set default wwn prefix 20:00 */
		npar_block_ptr->func_cfg[pf].fcoe_port_wwn_upper |= 20 << 24;
	}
}

int common_nvm_is_l2_mac_initialized() {
	u32_t sf_mac_upper, sf_mac_lower, tmp_nvm_addr, pf, path, port, num_ports;
	int is_4port;

	common_nvm_is_4port(&is_4port);
	num_ports = is_4port ? 4 : 2;

	for (pf = 0; pf < num_ports; pf++) {
		/* calculate path and port */
		path = pf & 1;
		port = is_4port ? ((pf >> 1) & 1) : 0;

		/* check if SF L2 MAC addr of that port is initialized */
		tmp_nvm_addr = NVM_OFFSET_PATH(manuf_info.port_hw_config[port].mac_upper, path);
		common_nvm_read_nvram(tmp_nvm_addr, &sf_mac_upper, 4);
		tmp_nvm_addr = NVM_OFFSET_PATH(manuf_info.port_hw_config[port].mac_lower, path);
		common_nvm_read_nvram(tmp_nvm_addr, &sf_mac_lower, 4);
		if ((sf_mac_upper == 0) && (sf_mac_lower == 0)) {
			return 0;
		}
	}

	return 1;
}

common_nvm_status_t common_nvm_init_npar_nvm_image(int is_e3) {
	u32_t status, path, port, pf, num_ports, expected_val, wwn_prefix = 0, crc32, wwn_prefix_offset, link_config, default_cfg;
	u32_t nvm_addr1, entry_idx1, found_image1, byte_cnt1, nvm_addr2, entry_idx2, found_image2, byte_cnt2;
	int is_4port;
	nvm_nic_part_cfg_t npar_block;

	/* TBD: SUPPORT NPAR1.5 at init ?*/
	u32_t is_nicp1_5=0;

	/* search for NICP image in extended dir */
	status = common_nvm_nvm_find_image_in_extended_dir(&found_image1, CODE_IMAGE_TYPE_NIC_PARTITION_CFG, &nvm_addr1, &byte_cnt1, &entry_idx1);

	if ((status != COMMON_NVM_STATUS_SUCCESS) || !found_image1) {
		return COMMON_NVM_STATUS_SUCCESS;
	}

	/* read whole NICP info image*/
	common_nvm_read_nvram(nvm_addr1,
						  (u32_t *)(&npar_block),
						  sizeof(nvm_nic_part_cfg_t));

	/* can set NICP image values only if similar size remains */
	if (byte_cnt1 != (sizeof(nvm_nic_part_cfg_t) + 4)) {
		return COMMON_NVM_STATUS_SUCCESS;
	}

	/* read whole NICP info image*/
	common_nvm_read_nvram(nvm_addr1,
						  (u32_t *)(&npar_block),
						  sizeof(nvm_nic_part_cfg_t));

	/* find wwn prefix in nvram */
	status = common_nvm_nvm_find_image_in_extended_dir(&found_image2, CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &nvm_addr2, &byte_cnt2, &entry_idx2);
	if ((status == COMMON_NVM_STATUS_SUCCESS) && found_image2) {
		wwn_prefix_offset = OFFSETOF(struct extended_dev_info_shared_cfg, wwn_prefix);
		if (byte_cnt2 >= wwn_prefix_offset) {
			common_nvm_read_nvram(nvm_addr2 + wwn_prefix_offset, &wwn_prefix, sizeof(u32_t));
		}
	}

	/* init values only if all function in image are disabled */
	for (pf = 0; pf < E1H_FUNC_MAX; pf++) {
		if (npar_block.func_cfg[pf].func_cfg & MACP_FUNC_CFG_FLAGS_MASK) {
			return COMMON_NVM_STATUS_SUCCESS;
		}
	}

	/* check global field value */
	if (!npar_block.global_cfg.gbl_cfg) {
		expected_val = (MACP_GBL_CFG_GBL_CFG_ENABLED | MAC_PARTITION_FORMAT_VERSION);
		DbgMessage(NULL, WARN, "change npar image - global_cfg - 0x0 --> 0x%x\n", expected_val);
		npar_block.global_cfg.gbl_cfg = expected_val;
	}

	if (!common_nvm_is_l2_mac_initialized()) {
		DbgMessage(NULL, WARN, "not all SF L2 MAC addresses are initialized, not configuring NPAR block\n");
		return COMMON_NVM_STATUS_SUCCESS;
	}

	/* if reached here - change per function values */
	DbgMessage(NULL, WARN, "change npar image - set per function fields to default\n");
	for (pf = 0; pf < E1H_FUNC_MAX; pf++) {
		/* calculate path and port */
		path = pf & 1;
		common_nvm_is_4port(&is_4port);
		port = is_4port ? ((pf >> 1) & 1) : 0;
		num_ports = is_4port ? 4 : 2;

		/* set port config only for lower functions */
		if (pf < num_ports) {
			common_nvm_read_nvram(NVM_OFFSET_PATH(feature_info.port_feature_config[port].link_config, path),
								  &link_config,
								  sizeof(u32_t));
			npar_block.port_cfg[path][port].port_cfg |= (link_config & PORT_FEATURE_FLOW_CONTROL_MASK)
			>> PORT_FEATURE_FLOW_CONTROL_SHIFT
			<< MACP_PORT_CFG_FLOW_CTRL_SHIFT;
		} else if (is_e3) {
			/* if port is 1G, don't enable upper functions */
			common_nvm_read_nvram(NVM_OFFSET_PATH(manuf_info.port_hw_config[port].default_cfg, path),
								  &default_cfg,
								  sizeof(u32_t));
			if ((default_cfg & PORT_HW_CFG_NET_SERDES_IF_MASK) == PORT_HW_CFG_NET_SERDES_IF_SGMII) {
				continue;
			}
		}

		/* set function config it as follows: PF_0: fcoe, iscsi, l2. Others: l2 only */
		if (pf < num_ports) {
			expected_val = (MACP_FUNC_CFG_FLAGS_ENABLED | MACP_FUNC_CFG_FLAGS_ETHERNET | MACP_FUNC_CFG_FLAGS_ISCSI_OFFLOAD);
		} else {
			expected_val = (MACP_FUNC_CFG_FLAGS_ENABLED | MACP_FUNC_CFG_FLAGS_ETHERNET);
		}
		npar_block.func_cfg[pf].func_cfg = expected_val;

		/* set max BW */
		npar_block.func_cfg[pf].func_cfg |= (MACP_BW_MAX_DEFAULT_VAL << MACP_FUNC_CFG_BW_MAX_SHIFT);

		/* set net mac addr according to SF L2 MAC from NVM */
		common_nvm_init_npar_mac_addr(pf, is_4port, is_nicp1_5, wwn_prefix, &npar_block);

	}

	/* write manipulated image */
	status = common_nvm_write_nvram(nvm_addr1, (u32_t *)(&npar_block), sizeof(nvm_nic_part_cfg_t));

	/* fix crc */
	if (status == COMMON_NVM_STATUS_SUCCESS) {
		common_nvm_compute_image_crc(nvm_addr1, sizeof(nvm_nic_part_cfg_t) + 4, &crc32, NULL);
		common_nvm_write_nvram(nvm_addr1 + sizeof(nvm_nic_part_cfg_t), &crc32, sizeof(crc32));
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * enough_room --
 *
 *     Determine whether the NVM part has enough room to store an image.
 *
 * Input:
 *    - The byte_cnt already includes the 4-byte CRC.
 *    - Images included in the MFW (if relevant), for proper calculation
 *
 * Results:
 *     According to nvm_enough_room_result_t
 *
 * Side effects:
 *     None.
 *
 *------------------------------------------------------------------------
 */
nvm_enough_room_result_t enough_room(
									 u32_t is_evst1x,
									 u32_t image_type,
									 u32_t byte_cnt,
									 u32_t *image_types_in_input_mfw,
									 u32_t input_mfw_images_found
									 ) {
	u32_t total, nvm_size, val32, idx, i, j;
	extended_dir_image_t extended_dir;
	u32_t bootstrap_byte_cnt=0, bc2_byte_cnt=0;
	u32_t ncsi_byte_cnt=0;
	u32_t mfw2_byte_cnt=0;
	u32_t swim_byte_cnt=0;
	u32_t code_start, code_len, code_type;
	u32_t mfw1_exist=0;
	u32_t size_of_images_to_be_replaced = 0;
	u32_t index_image_table;
	common_nvm_status_t status;

	nvm_size = common_nvm_nvm_size();
	/* Leave 2K for image manipulation */
	nvm_size -= 2048;

	/* Include the CRC before check for NVM storage availability */
	total = sizeof(nvm_image_t) + byte_cnt;
	if (!is_evst1x) {total += sizeof(path1_nvm_image_t);}

	DbgMessage(NULL, WARN, "enough_room: 1) image byte_cnt 0x%x, nvm_image_t 0x%x path1_nvm_image_t 0x%x total = 0x%x\n",
			   byte_cnt, sizeof(nvm_image_t), sizeof(path1_nvm_image_t), total);

	/* Get byte count from bc1 */
	if (image_type != CODE_IMAGE_TYPE_BC1) {
		common_nvm_read_nvram(NVM_OFFSET(bootstrap.code_len), &val32, sizeof(val32));
		bootstrap_byte_cnt = val32 * sizeof(u32_t);
		total += bootstrap_byte_cnt;
		DbgMessage(NULL, WARN, "enough_room: 2) bootstrap byte_cnt 0x%x, total = 0x%x\n", bootstrap_byte_cnt, total);
	}

	/* Traverse directory and gather up sizes of all images */
	for (idx = 0; idx < CODE_ENTRY_MAX; idx++) {
		common_nvm_read_nvram(NVM_OFFSET(dir.code[idx].code_attribute), &val32, sizeof(val32));
		code_type = val32 & CODE_IMAGE_TYPE_MASK;
		code_len = val32 & CODE_IMAGE_LENGTH_MASK;

		if (val32 != 0) {
			// If BC2, NCSI images exist save their byte count
			if (code_type == CODE_IMAGE_TYPE_BC2) {
				bc2_byte_cnt = code_len;
			} else if (code_type == CODE_IMAGE_TYPE_NC_SI_CMN) {
				ncsi_byte_cnt += code_len;
			} else if (code_type == CODE_IMAGE_TYPE_NC_SI_EVEREST) {
				ncsi_byte_cnt += code_len;
			} else {
				// do nothing
			}
		}

		/* Add the extended dir image total bytes count */
		if ((idx == CODE_ENTRY_EXTENDED_DIR_IDX) && ((val32 & CODE_IMAGE_TYPE_MASK) == CODE_IMAGE_TYPE_EXTENDED_DIR)) {
			total += (val32 & CODE_IMAGE_LENGTH_MASK);


			if (common_nvm_nvm_get_dir_info(idx, &code_start, &code_len, NULL, NULL, NULL) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Failed to parse extended dir image !! \n");
				return NVM_NOT_ENOUGH_ROOM;
			}
			if (common_nvm_nvm_parse_extended_dir_image(code_start, code_len, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Failed to parse extended dir image !! \n");
				return NVM_NOT_ENOUGH_ROOM;
			}

			total += (extended_dir.total_byte_cnt);

			DbgMessage(NULL, WARN, "enough_room: 4) extended_dir total byte_cnt 0x%x, total = 0x%x\n",
					   (extended_dir.total_byte_cnt), total);

			/* Save MFW1, MFW2, SWIM byte count if exist */
			for (i = 0; i < extended_dir.no_images; i++) {
				code_len = (extended_dir.extended_dir_images[i].code_attribute & CODE_IMAGE_LENGTH_MASK);
				code_type = (extended_dir.extended_dir_images[i].code_attribute & CODE_IMAGE_TYPE_MASK);
				if (code_type == CODE_IMAGE_TYPE_MFW1) {
					// do nothing, byte_cnt was already taken from bootstrap.code_len
					mfw1_exist=1;
				} else {
					if ((code_type == CODE_IMAGE_TYPE_MFW2) || (code_type == CODE_IMAGE_TYPE_MFW2_A)) {
						mfw2_byte_cnt += code_len;
					}
					if ((code_type == CODE_IMAGE_TYPE_SWIM1) || (code_type == CODE_IMAGE_TYPE_SWIM2) ||
						(code_type == CODE_IMAGE_TYPE_SWIM3) || (code_type == CODE_IMAGE_TYPE_SWIM4) ||
						(code_type == CODE_IMAGE_TYPE_SWIM5) || (code_type == CODE_IMAGE_TYPE_SWIM6) ||
						(code_type == CODE_IMAGE_TYPE_SWIM7) || (code_type == CODE_IMAGE_TYPE_SWIM8) ||
						(code_type == CODE_IMAGE_TYPE_SWIM1_B) || (code_type == CODE_IMAGE_TYPE_SWIM2_B) ||
						(code_type == CODE_IMAGE_TYPE_SWIM3_B) || (code_type == CODE_IMAGE_TYPE_SWIM4_B) ||
						(code_type == CODE_IMAGE_TYPE_SWIM5_B) || (code_type == CODE_IMAGE_TYPE_SWIM6_B) ||
						(code_type == CODE_IMAGE_TYPE_SWIM7_B) || (code_type == CODE_IMAGE_TYPE_SWIM8_B)) {
						swim_byte_cnt += code_len;
					}
					/* Incase this entry has the same image type we are upgrading,
						then delete it's code length since this image will be over-written */
					if (image_type == code_type) {
						if (total > code_len) {
							total -= code_len;
							DbgMessage(NULL, WARN, "enough_room: 5) Reduce image_type 0x%x byte_cnt 0x%x, total = 0x%x\n",
									   image_type, code_len, total);
						}
					}
				}

				/* during MFW burn - ignore non MFW/SWIM images that also exist in the new image to be burned,
				   as they will be replaced anyway, for exmaple: E3_WC, E#_PCIE, OCNVM etc. */
				// Note: code assumes MFW1 is always the first image in MCP package
				if ((image_type == CODE_IMAGE_TYPE_MFW1) && (image_types_in_input_mfw != NULL)) {
					status = common_nvm_find_entry_in_image_table(code_type, &index_image_table);
					if (status != COMMON_NVM_STATUS_SUCCESS) {
						DbgMessage(NULL, INFORM, "Could not find entry in image table !! \n");
						continue;
					}
					if ((image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_BC1) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_BC2) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_NCSI_CMN) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_NCSI_LIB_XI) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_NCSI_LIB_EV) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_SWIM1) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_SWIM2) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_SWIM3) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_SWIM4) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_SWIM5) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_SWIM6) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_SWIM7) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_SWIM8) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_MFW1) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_MFW2) &&
						(image_table[index_image_table].hdr_type != IMAGE_HDR_TYPE_MFW2_A)) {
						/* search for those images in the structure representing new image */
						for (j = 0; j < input_mfw_images_found; j++) {
							if (image_table[index_image_table].hdr_type == image_types_in_input_mfw[j]) {
								size_of_images_to_be_replaced += code_len;
							}
						}
					}
				}
			}
		} else {
			if (image_type != (val32 & CODE_IMAGE_TYPE_MASK)) {
				total += (val32 & CODE_IMAGE_LENGTH_MASK);

				if ((val32 & CODE_IMAGE_LENGTH_MASK) > 0) {
					DbgMessage(NULL, WARN, "enough_room: 3) image idx=%d byte_cnt 0x%x, total = 0x%x\n",
							   idx, (val32 & CODE_IMAGE_LENGTH_MASK), total);
				}
			} else {
				DbgMessage(NULL, WARN, "Found the same image type code_len (0x%x) do not include in total (0x%x)\n",
						   code_len, total);
			}
		}
	}

	if (mfw1_exist) {
		// MFW1 size was taken twice (from bootstarp and in extended_dir.byte_cnt)
		total -= bootstrap_byte_cnt;
		DbgMessage(NULL, WARN, "Reduce MFW1 byte_cnt 0x%x, total = 0x%x\n", bootstrap_byte_cnt, total);
	}

	// If checking room for MFW1,2+SWIM images reduce the BC1,2+NCSI they will be deleted
	// Also reduce the MFW1,2 and images to be replaced
	// reduce SWIM images only if there isn't place for both instances (shadow feature)
	if (image_type == CODE_IMAGE_TYPE_MFW1) {
		if (total < (size_of_images_to_be_replaced + bootstrap_byte_cnt + bc2_byte_cnt + ncsi_byte_cnt + mfw2_byte_cnt + swim_byte_cnt)) {
			DbgMessage(NULL, FATAL, "enough room total bytes calculation - internal error!\n");
			return NVM_NOT_ENOUGH_ROOM;
		}

		total -= size_of_images_to_be_replaced;
		total -= (bootstrap_byte_cnt+bc2_byte_cnt+ncsi_byte_cnt);
		total -= mfw2_byte_cnt;

		DbgMessage(NULL, WARN, "enough_room: 7) After reducing images to replace total 0x%x\n", total);


		if (image_types_in_input_mfw != NULL) {
			// if both SWIM instances can remain return NVM_ENOUGH_ROOM
			if (total <= nvm_size) {
				DbgMessage(NULL, WARN, "enough_room returned NVM_ENOUGH_ROOM\n");
				return NVM_ENOUGH_ROOM;
				// if only new instances of swims can remain return NVM_ENOUGH_ROOM_EXCEPT_2_SWIM_GROUPS
			} else if ((total - swim_byte_cnt) <= nvm_size) {
				DbgMessage(NULL, WARN, "enough_room returned NVM_ENOUGH_ROOM_EXCEPT_2_SWIM_GROUPS\n");
				return NVM_ENOUGH_ROOM_EXCEPT_2_SWIM_GROUPS;
				// else - return NVM_NOT_ENOUGH_ROOM
			} else {
				DbgMessage(NULL, WARN, "enough_room returned NVM_NOT_ENOUGH_ROOM\n");
				return NVM_NOT_ENOUGH_ROOM;
			}
		}
	}

	DbgMessage(NULL, WARN, "Image size 0x%x, required size 0x%x available 0x%x\n", byte_cnt, total, common_nvm_nvm_size());

	if (total <= common_nvm_nvm_size()) {
		return NVM_ENOUGH_ROOM;
	} else {
		return NVM_NOT_ENOUGH_ROOM;
	}
}

#ifdef FWUPGTOOL
// from ...\nx2\577xx\hsi\hw\include\57712_reg.h in Perforce...
#define MISC_REG_PORT4MODE_EN_OVWR  0xa720UL
#define MISC_REG_PORT4MODE_EN       0xa750UL
#endif // FWUPGTOOL

/*
 *------------------------------------------------------------------------
 * common_nvm_is_4port --
 *
 *     Find out if this is a 4port board
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_is_4port(int *result) {
#ifdef FWUPGTOOL
	// The code is following the same logic in dg_is_4port()
	// in dg_device.c, rev#68, under .../servers/main/577xx/drivers/
	// common/diag/diag_manager in perforce.
	NVM_INTERFACE iNvm = nvm_interface(gpINx2Nvm->m_tcl_interp);
	u32_t val32, dev_index = iNvm.m_current_dev_index;

	if (COMMON_NVM_IS_E1x()) {
		*result = 0;
		return COMMON_NVM_STATUS_SUCCESS;
	}

	if (iNvm.m_td_read_reg32(dev_index, MISC_REG_PORT4MODE_EN_OVWR, &val32) != TCL_OK) return COMMON_NVM_STATUS_FAILURE;

	if ((val32 & 0x1) == 0) {
		if (iNvm.m_td_read_reg32(dev_index, MISC_REG_PORT4MODE_EN, &val32) != TCL_OK) return COMMON_NVM_STATUS_FAILURE;

		*result = ( val32 & 1 );
	} else {
		*result = (( val32 >> 1 ) & 1 );
	}

	return COMMON_NVM_STATUS_SUCCESS;
#else
#ifdef EDEBUG
	return dbg_is_4port(result);
#else
	return dg_is_4port(result);
#endif // EDEBUG
#endif // FWUPGTOOL
}


/*
 *------------------------------------------------------------------------
 * common_nvm_xlate_flash_phy_addr --
 *
 *     Find the NVM physical address for ATMEL buffered flash
 *
 * Results:
 *     Physical address of the flash
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
u32_t common_nvm_xlate_flash_phy_addr(
									  u32_t virt_addr
									  ) {
#if 0
	/* There is no use to translate virt addr to phy addr.
		   We are using virt addr always, even with ATMEL nvm type */
	diag_status_t status;
	u32_t value32;

	status = common_nvm_reg_read(0x86418, &value32);

	if ((value32 & 0xff000000) == 0x57000000) {
		/* ATMEL buffered flash - translate address */
		return(u32_t)((virt_addr/NVM_ATMEL_PAGE_SIZE)*512 +
					  (virt_addr % NVM_ATMEL_PAGE_SIZE));
	} else {
		/* Non-buffered flash, no translation needed. */
		return virt_addr;
	}
#else
	// does not apply to 57710
	return virt_addr;
#endif // !FWUPGTOOL
}

/*
 *------------------------------------------------------------------------
 * common_nvm_xlate_flash_virt_addr --
 *
 *     Find the NVM virtual address for ATMEL buffered flash
 *
 * Results:
 *     Virtual address of the flash - returns a wrong virtual address if
 *     the input is an invalid physical one (e.g. 268).
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
u32_t common_nvm_xlate_flash_virt_addr(
									   u32_t phy_addr
									   ) {

#if 0
	diag_status_t status;
	u32_t value32;

	/* There is no use to translate phy_addr to virt_addr.
		   We are using virt addr always, even with ATMEL nvm type */
	status = common_nvm_reg_read(0x86418, &value32);

	if ((value32 & 0xff000000) == 0x57000000) {
		/* ATMEL buffered flash - translate address */
		return(u32_t)((phy_addr/512)*NVM_ATMEL_PAGE_SIZE +
					  (phy_addr % 512));
	} else {
		/* Non-buffered flash, no translation needed. */
		return phy_addr;
	}
#else
	/* Non-buffered flash, no translation needed. */
	return phy_addr;
#endif

}

u32_t common_nvm_nvm_size(void) {
#ifndef FWUPGTOOL
	common_nvm_dev_info_t *cur_dev_p;

	if (common_nvm_tcl_al_get_virtual_nvm_mode()) {
		return virtual_nvm.nvm_size;
	}

	if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
		cur_dev_p= COMMON_NVM_CUR_DEV;
		return common_nvm_nvm_size_from_dev_info(cur_dev_p);
	} else {
		return common_nvm_duart_get_nvm_size();
	}
#else
	// retrieve the nvm_size from ENV_NVM_SIZE TCL environment variable
	// This is what has been done in 5706/5708/5709

	Tcl_Obj *tcl_obj;
	u32_t size = 0;

	tcl_obj = Tcl_GetVar2Ex(gpINx2Nvm->m_tcl_interp, TCL_ENV, ENV_NVM_SIZE,  0);
	Tcl_GetLongFromObj(gpINx2Nvm->m_tcl_interp, tcl_obj, (long *)&size);
	return size;
#endif // !FWUPGTOOL
}

#ifndef FWUPGTOOL
#ifndef WIN_DIAG
// For FWUPG tool, we don't issue the MCP reset and update the device information.
/*
 *------------------------------------------------------------------------
 * common_nvm_refresh_devices --
 *
 *     Updated device information of all devices and enviromnet MAC addr.
 *     Used after "nvm prg" "nvm fill" "nvm cfg"
 *
 *------------------------------------------------------------------------
 */
void common_nvm_refresh_devices(void) {
#ifdef EDEBUG
	/* nothing should be done in edebug in that function */
	return;
#else
	u32_t dev_num;
	diag_dev_info_t *cur_dev_p = XDIAG_CUR_DEV;
	dev_stat_t *p_env_struct;
	u32_t no_total_br_dev=0;

	no_total_br_dev = dg_get_total_br_dev();


	/* Reset the MCP */
	dg_reset_mcp();


	/* Update device information of all devices on this board only */
	for (dev_num = 0; (u32_t)dev_num < no_total_br_dev; dev_num++) {

		if (BR_PCI_DEV_PRESENT(dev_num) && !BR_PCI_DEV_HIDDEN(dev_num)) {
			diag_dev_info_t *dev_info_p = BR_DEV_INFO(dev_num);

			/* Update information of devices on this board only
			   (with the same domain and bus number) */
			if ((dev_info_p->pci_info.domain == cur_dev_p->pci_info.domain) &&
				(dev_info_p->pci_info.bus_num ==   cur_dev_p->pci_info.bus_num)) {

				/* Set "setdut" address to the address of the currently selected device */
				dg_pcicfg_setdut(dev_info_p->pci_info.domain,
								 dev_info_p->pci_info.bus_num, (dev_info_p->pci_info.device << 3) | dev_info_p->pci_info.func);

				dg_dev_update_dev_info((dev_if_t *)dev_info_p->drv_dev_if_p);

				/* Update environment struct so that MAC address will be updated also */
				p_env_struct = dg_get_br_dev_stat(dev_num);
				//printf("p_env_struct->mac_addr %12s\n", p_env_struct->mac_addr);
			}
		}
	}

	/* return to current device */
	dg_pcicfg_setdut(cur_dev_p->pci_info.domain,
					 cur_dev_p->pci_info.bus_num, (cur_dev_p->pci_info.device << 3) | cur_dev_p->pci_info.func);
#endif // EDEBUG
}
#endif
#endif // !FWUPGTOOL

/*
 *------------------------------------------------------------------------
 * common_nvm_keep_mac --
 *
 *     This utility is used to keep the current MAC address
 *     incase of "nvm prg" was called without -force_mac option.
 *
 *------------------------------------------------------------------------
 */
static common_nvm_status_t common_nvm_keep_mac(u32_t nvm_offset, u32_t byte_cnt, u8_t *buf, u8_t big_endian) {
	u32_t data_h, data_l;
	u32_t *ptr32;
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;

	if (byte_cnt > nvm_offset) {
		/* There could be a potential problem when the amount of
		* data to be programmed fall within the skipped range.
		* For example, the amount falls right up updating the
		* upper mac.
		*/

		status = common_nvm_read_nvram(nvm_offset, &data_l, sizeof(data_l));
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			return status;
		}
		status = common_nvm_read_nvram(nvm_offset - 4, &data_h, sizeof(data_l));
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			return status;
		}

		ptr32 =(u32_t *)(&(buf[nvm_offset]));

		/* Initialize MAC address to zeroes if it's all F's. */
		if ((data_l == 0xFFFFFFFF) && (data_h == 0xFFFFFFFF)) {
			*ptr32 = 0;
			*(ptr32-1) = 0;
		} else {
			/* Otherwise, keep the MAC address */
			*ptr32 = (big_endian) ? htonl(data_l): data_l;
			DbgMessage(NULL, INFORM, "Keep lower MAC address %x\n", data_l);
			*(ptr32-1) = (big_endian) ? htonl(data_h): data_h;
			DbgMessage(NULL, INFORM, "Keep upper MAC address %x\n", data_h);
		}
	}
	return COMMON_NVM_STATUS_SUCCESS;
}


/*
 *------------------------------------------------------------------------
 * common_nvm_keep_license --
 *
 *     This utility is used to keep the current license key
 *     incase of "nvm prg" was called without -force_lic option.
 *
 *------------------------------------------------------------------------
 */
static common_nvm_status_t common_nvm_keep_license(u32_t port_no, u32_t byte_cnt, u8_t *buf, u8_t big_endian) {
	u32_t nvm_offset, data32, crc32, idx;
	u32_t *ptr32;
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;

#ifndef FWUPGTOOL
	/* Starting E2 there is one license per board, the port0 license */
	if (!is_e1x(globalInterp)) port_no = 0;
#else
	if (!COMMON_NVM_IS_E1x()) port_no = 0;
#endif

	// Keep the upgrade license key
	nvm_offset = NVM_OFFSET(upgrade_key_info[port_no].upgrade_key);
	if (byte_cnt > nvm_offset) {
		ptr32 =(u32_t *)&buf[nvm_offset];
		for (idx = 0; idx < sizeof(license_key_t) * UPGRADE_KEY_COUNT; idx += 4, ptr32++) {
			status = common_nvm_read_nvram(nvm_offset + idx, &data32, sizeof(data32));
			if (status != COMMON_NVM_STATUS_SUCCESS) {
				return status;
			}
			*ptr32 = (big_endian) ? htonl(data32): data32;
		}
	}

	// CRC of the upgrade license key block
	nvm_offset = NVM_OFFSET(upgrade_key_info[port_no].crc);
	if (byte_cnt > nvm_offset) {
		// This programming has long enough buffer to hold the CRC. So, perform CRC check here.
		crc32 = 0xffffffff;
		common_nvm_compute_crc(&buf[NVM_OFFSET(upgrade_key_info[port_no])], sizeof(upgrade_key_info_t) - 4, big_endian, &crc32);
#ifdef FWUPGTOOL
		crc32 = BMAPI_BENDIAN_SWAP(crc32);
#endif
		crc32 = ~htonl(crc32);
		ptr32 =(u32_t *)&buf[nvm_offset];
		*ptr32 = (big_endian) ? htonl(crc32): crc32;
	}

	// Keep the manufacture license key */
	nvm_offset = NVM_OFFSET(manuf_key_info[port_no].manuf_key);
	if (byte_cnt > nvm_offset) {
		ptr32 =(u32_t *)&buf[nvm_offset];
		for (idx = 0; idx < sizeof(license_key_t); idx += 4, ptr32++) {
			status = common_nvm_read_nvram(nvm_offset + idx, &data32, sizeof(data32));
			if (status != COMMON_NVM_STATUS_SUCCESS) {
				return status;
			}
			*ptr32 = (big_endian) ? htonl(data32): data32;
		}
	}

	// CRC of the manuf license key block */
	nvm_offset = NVM_OFFSET(manuf_key_info[port_no].crc);
	if (byte_cnt > nvm_offset) {
		// This programming has long enough buffer to hold the CRC. So, perform CRC check here.
		crc32 = 0xffffffff;
		common_nvm_compute_crc(&buf[NVM_OFFSET(manuf_key_info[port_no])], sizeof(manuf_key_info_t) - 4, big_endian, &crc32);
#ifdef FWUPGTOOL
		crc32 = BMAPI_BENDIAN_SWAP(crc32);
#endif
		crc32 = ~htonl(crc32);
		ptr32 =(u32_t *)&buf[nvm_offset];
		*ptr32 = (big_endian) ? htonl(crc32): crc32;
	}

	return status;
}

#ifndef FWUPGTOOL
/*
 *------------------------------------------------------------------------
 * dg_get_nvm_offset_for_mac_addr --
 *
 *     This utility is used to get the MAC address from NVM
 *     used by tcl_license.c get_mac_addr() utility.
 *
 *------------------------------------------------------------------------
 */
u32_t common_nvm_get_nvm_offset_for_mac_addr(u32_t mac_type) {
	u32_t port_num, nvm_offset;

	/* Starting E2 there is one license per board, the port0 license */
	if (!is_e1x(globalInterp)) {
		port_num = 0;
	} else {
		common_nvm_get_port_num(&port_num);
	}

	if (mac_type == DUMMY_MAC) {
		nvm_offset = NVM_OFFSET(upgrade_key_info[port_num].hwkey_mac);

	} else if (mac_type == BACKUP_MAC) {
		nvm_offset = NVM_OFFSET(manuf_info.port_hw_config[port_num].backup_mac_upper);

	} else {
		nvm_offset = NVM_OFFSET(manuf_info.port_hw_config[port_num].mac_upper);
	}

	return nvm_offset;
}


/*
 *------------------------------------------------------------------------
 * dg_get_nvm_offset_for_license --
 *
 *     This utility is used to get the NVM address of the requested
 *     license type.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_get_nvm_offset_for_license(u32_t license_type, u32_t *nvm_addr_p, u32_t *blk_start_p) {
	u32_t port_num;

	/* Starting E2 there is one license per board, the port0 license */
	if (!is_e1x(globalInterp)) {
		port_num = 0;
	} else {
		common_nvm_get_port_num(&port_num);
	}

	switch (license_type) {
	case DATA_TYPE_SHARED_SECRET:
		*nvm_addr_p = NVM_OFFSET(manuf_key_info[port_num].shared_secret);
		*blk_start_p = NVM_OFFSET(manuf_key_info[port_num]);
		break;
	case DATA_TYPE_MANUF_KEY:
		*nvm_addr_p = NVM_OFFSET(manuf_key_info[port_num].manuf_key);
		*blk_start_p = NVM_OFFSET(manuf_key_info[port_num]);
		break;
	case DATA_TYPE_UPGRADE_KEY:
		*nvm_addr_p = NVM_OFFSET(upgrade_key_info[port_num].upgrade_key);
		*blk_start_p = NVM_OFFSET(upgrade_key_info[port_num]);
		break;
	default:
		DbgMessage(NULL, FATAL, "Unexpected license type %d\n", license_type);
		return COMMON_NVM_STATUS_FAILURE;
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * dg_get_nvm_offset_for_pci_id --
 *
 *     This utility is used to get the NVM offset of pci_id and pci_sub_id
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_get_nvm_offset_for_pci_id(u32_t * pci_id_p, u32_t * pci_sub_id_p) {
	u32_t port_num;

	/* Starting E2 there is one license per board, the port0 license */
	if (!is_e1x(globalInterp)) {
		port_num = 0;
	} else {
		common_nvm_get_port_num(&port_num);
	}

	*pci_id_p = NVM_OFFSET(manuf_info.port_hw_config[port_num].pci_id);
	*pci_sub_id_p = NVM_OFFSET(manuf_info.port_hw_config[port_num].pci_sub_id);

	return COMMON_NVM_STATUS_SUCCESS;
}


/*
 *------------------------------------------------------------------------
 * dg_get_key_offset_from_shmem --
 *
 *     This utility is used to get the key_offset in shmem of the current
 *     license.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_get_key_offset_from_shmem(u32_t * key_offset_p) {
	u32_t port_num;
	u32_t shmem;

	/* Starting E2 there is one license per board, the port0 license */
	if (!is_e1x(globalInterp)) {
		port_num = 0;
	} else {
		common_nvm_get_port_num(&port_num);
	}

	/* Read the license from the shared memory */
	common_nvm_reg_read(MISC_REG_SHARED_MEM_ADDR, &shmem);
	if (shmem == 0) {
		DbgMessage(NULL, FATAL, "No shared memory address");
		return COMMON_NVM_STATUS_FAILURE;
	}

	*key_offset_p =(u32_t)((int_ptr_t)&(((shmem_region_t *)((int_ptr_t)shmem))->drv_lic_key[port_num]));

	return COMMON_NVM_STATUS_SUCCESS;
}
#endif


/*
 *------------------------------------------------------------------------
 * common_nvm_keep_nvm_config --
 *
 *     This utility is used to keep the current nvm configuration
 *     incase of "nvm prg" was called with -skip_config option.
 *
 *------------------------------------------------------------------------
 */
static common_nvm_status_t common_nvm_keep_nvm_config(u32_t byte_cnt, u8_t *buf, u8_t big_endian) {
	u32_t *ptr_mac_lower, *ptr_iscsi_mac_lower, *ptr_rdma_mac_lower;
	u32_t nvm_offset, idx, data32, port_no;
	u32_t *ptr32;
	u32_t curr_path;
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;
	u32_t num_paths = 1;
#ifndef FWUPGTOOL
	if (!is_e1x(globalInterp)) num_paths = 2;
#else
	if (!COMMON_NVM_IS_E1x()) num_paths = 2;
#endif

	for (curr_path = 0; curr_path < num_paths; ++curr_path) {
		// First do the necessary things to preseve the feature block configurations.
		nvm_offset = NVM_OFFSET_PATH(feature_info, curr_path);
		if (byte_cnt > nvm_offset) {
			ptr32 =(u32_t *)&buf[nvm_offset];
			for (idx = 0; idx < sizeof(feature_info_t); idx += 4, ptr32++) {
				status = common_nvm_read_nvram(nvm_offset + idx, &data32, sizeof(data32));
				if (status != COMMON_NVM_STATUS_SUCCESS) {
					return status;
				}
				*ptr32 = (big_endian) ? htonl(data32): data32;
			}

		}

		// Secondly do the necessary things to preseve the manufacture shared_hw_config block
		// configurations.
		nvm_offset = NVM_OFFSET_PATH(manuf_info.shared_hw_config, curr_path);
		if (byte_cnt > nvm_offset) {
			ptr32 =(u32_t *)&buf[nvm_offset];
			for (idx = 0; idx < sizeof(shared_hw_cfg_t); idx += 4, ptr32++) {
				status = common_nvm_read_nvram(nvm_offset + idx, &data32, sizeof(data32));
				if (status != COMMON_NVM_STATUS_SUCCESS) {
					return status;
				}
				*ptr32 = (big_endian) ? htonl(data32): data32;
			}

		}

		// Third do the necessary things to preseve the manufacture port_hw_cfg_t block
		// configurations.
		for (port_no = 0; port_no < PORT_MAX; port_no++) {
			nvm_offset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port_no], curr_path);
			if (byte_cnt > nvm_offset) {
				ptr32 =(u32_t *)&buf[nvm_offset];
				for (idx = 0; idx < sizeof(port_hw_cfg_t); idx += 4, ptr32++) {
					ptr_mac_lower       =(u32_t *)&buf[NVM_OFFSET_PATH(manuf_info.port_hw_config[port_no].mac_lower, curr_path)];
					ptr_iscsi_mac_lower =(u32_t *)&buf[NVM_OFFSET_PATH(manuf_info.port_hw_config[port_no].iscsi_mac_lower, curr_path)];
					ptr_rdma_mac_lower  =(u32_t *)&buf[NVM_OFFSET_PATH(manuf_info.port_hw_config[port_no].rdma_mac_lower, curr_path)];

					// Keep the mac_lower, mac_upper, iscsi_mac_lower, iscsi_mac_upper
					if ((ptr_mac_lower == ptr32) || (ptr_mac_lower - 1 == ptr32) ||
						(ptr_iscsi_mac_lower == ptr32) || (ptr_iscsi_mac_lower - 1 == ptr32) ||
						(ptr_rdma_mac_lower == ptr32) || (ptr_rdma_mac_lower - 1 == ptr32)) continue;

					status = common_nvm_read_nvram(nvm_offset + idx, &data32, sizeof(data32));
					if (status != COMMON_NVM_STATUS_SUCCESS) return status;

					*ptr32 = (big_endian) ? htonl(data32): data32;
				}
			}
		}
	}

	return status;
}


/*
 *------------------------------------------------------------------------
 * common_nvm_keep_iscsi_fcoe_boot_config --
 *
 *     This utility is used to keep the current iscsi boot configuration
 *     I_B_CFG / I_B_CFG2 / I_B_CFG3 / I_B_CFG4
 *
 *     This utility is used to keep the current fcoe boot configuration
 *     FEB_CFG / FEB_CFG2 / FEB_CFG3 / FEB_CFG4
 *
 *------------------------------------------------------------------------
 */
static common_nvm_status_t common_nvm_keep_iscsi_fcoe_boot_config(u32_t byte_cnt, u8_t *buf, u8_t big_endian, u8_t is_iscsi) {
	u32_t entry_idx=0, entry_idx_in_buf=0, idx=0, image_idx=0, nvm_addr, data32;
	u32_t entry_addr, start=(u32_t)-1, type=(u32_t)-1, len=(u32_t)-1, temp, i;
	u32_t code_start, code_type, code_len;
	u32_t *p32_1, *p32_2, value, no_funcs, func;
	u32_t image_in_extended_dir=1, no_images;
	u32_t extended_dir_addr_in_buf=0, extended_dir_entry_addr;
	extended_dir_image_t extended_dir_in_nvm, extended_dir_in_buf;
	u32_t extended_dir_size_in_nvm;


	no_funcs = common_nvm_number_of_everest_functions();

	for (func = 0; func < no_funcs; func++) {
		// Set image index according to the function number
		switch (func) {
		case 0:
			image_idx = (is_iscsi) ? IMAGE_MAP_IDX_ISCSI_BOOT_CFG : IMAGE_MAP_IDX_FCOE_BOOT_CFG;
			image_in_extended_dir = (is_iscsi) ? 0 : 1;
			break;
		case 1:
			image_idx = (is_iscsi) ? IMAGE_MAP_IDX_ISCSI_BOOT_CFG2 : IMAGE_MAP_IDX_FCOE_BOOT_CFG2;
			image_in_extended_dir = (is_iscsi) ? 0 : 1;
			break;
		case 2:
			image_idx = (is_iscsi) ? IMAGE_MAP_IDX_ISCSI_BOOT_CFG3 : IMAGE_MAP_IDX_FCOE_BOOT_CFG3;
			image_in_extended_dir = (is_iscsi) ? 1 : 1;
			break;
		case 3:
			image_idx = (is_iscsi) ? IMAGE_MAP_IDX_ISCSI_BOOT_CFG4 : IMAGE_MAP_IDX_FCOE_BOOT_CFG4;
			image_in_extended_dir = (is_iscsi) ? 1 : 1;
			break;
		default:
			image_idx = (is_iscsi) ? IMAGE_MAP_IDX_ISCSI_BOOT_CFG : IMAGE_MAP_IDX_FCOE_BOOT_CFG;
			image_in_extended_dir = (is_iscsi) ? 0 : 1;
			break;
		}


		if (!image_in_extended_dir) {
			// Find an existing matching entry for this image in NVM DIR
			if (common_nvm_nvm_find_dir_entry(image_table[image_idx].type, &entry_idx, &nvm_addr) == COMMON_NVM_STATUS_SUCCESS) {
				// Get the information on this found entry
				common_nvm_nvm_get_dir_info(entry_idx, &code_start, &code_len, NULL, &code_type, NULL);
			} else {
				// There is no such image in NVM, so exit without doing anything
				return COMMON_NVM_STATUS_SUCCESS;
			}

			// Find an existing matching entry in file image bufer
			for (idx = 0; idx < CODE_ENTRY_MAX; idx++) {
				value = *(u32_t *)&buf[NVM_OFFSET(dir.code[idx].code_attribute)];
				if (big_endian) value = ntohl(value);

				if ((value & CODE_IMAGE_LENGTH_MASK) == 0) {
					continue;
				} else if ((value & CODE_IMAGE_TYPE_MASK) == image_table[image_idx].type) {
					// Found a matching image entry in image buffer to be programed
					entry_idx_in_buf = idx;
					DbgMessage(NULL, WARN, "Found %s for func %d in image buffer entry_idx %d\n",
							   image_table[image_idx].name, func, entry_idx_in_buf);
					break;
				}
			}

			if (idx == CODE_ENTRY_MAX) {
				DbgMessage(NULL, WARN, "The %s is in NVRAM but not in the image file\n", image_table[image_idx].name);
				// There is no such image in file image, so exit without doing anything
				return COMMON_NVM_STATUS_SUCCESS;

			} else {
				// Get image information (start, len) from file image buffer
				entry_addr = NVM_OFFSET(dir.code[entry_idx_in_buf]);
				if (byte_cnt > entry_addr + 12) {
					p32_1 =(u32_t *)&buf[entry_addr + 4];
					p32_2 =(u32_t *)&buf[entry_addr + 8];
					if (big_endian) {
						temp = ntohl(*p32_1);
						type = ntohl(*p32_1)& CODE_IMAGE_TYPE_MASK;
						len = ntohl(*p32_1)& CODE_IMAGE_LENGTH_MASK;
						start = ntohl(*p32_2);
					} else {
						temp = *p32_1;
						type = (*p32_1) & CODE_IMAGE_TYPE_MASK;
						len = (*p32_1) & CODE_IMAGE_LENGTH_MASK;
						start = *p32_2;
					}
				}

				// Replace the image in file image buffer with NVM content, dword by dword
				if ((byte_cnt > start + len) &&
					(type == image_table[image_idx].type) &&
					(len == code_len)) {

					u32_t word_num;
					// Replace file content with NVM content dword by dword
					DbgMessage(NULL, WARN, "Replace %s for func %d file content with NVM content\n", image_table[image_idx].name, func);
					word_num = (code_len+3)/4;

					for (i = 0; i < word_num; i++) {
						p32_1 =(u32_t *)&buf[start + i * 4];
						if (common_nvm_read_nvram(code_start + i * 4, &data32, sizeof(u32_t)) != COMMON_NVM_STATUS_SUCCESS) {
							return COMMON_NVM_STATUS_FAILURE;
						}
						*p32_1 = (big_endian) ? htonl(data32): data32;
					}
				} else {
					return COMMON_NVM_STATUS_SUCCESS;
				}
			}

		} else { // Image is in extended dir

			// Find an existing matching entry for this image in NVM EXTENDED DIR
			if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &extended_dir_size_in_nvm, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS) {

				// Parse the extended DIR
				if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
					if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, extended_dir_size_in_nvm, &extended_dir_in_nvm) != COMMON_NVM_STATUS_SUCCESS) {
						// Failed to parse extended dir image, then exit
						return COMMON_NVM_STATUS_FAILURE;
					}
					if (common_nvm_nvm_find_extended_dir_entry(image_table[image_idx].type, &extended_dir_in_nvm,
															   &entry_idx, &code_start, NULL, &code_len, NULL) != COMMON_NVM_STATUS_SUCCESS) {
						// There is no such image in NVM, so exit without doing anything
						return COMMON_NVM_STATUS_SUCCESS;
					}
				} else {
					// There is no extended dir, so exit without doing anything
					return COMMON_NVM_STATUS_SUCCESS;
				}
			} else {
				// There is no extended dir, so exit without doing anything
				return COMMON_NVM_STATUS_SUCCESS;
			}

			// Find an existing matching entry in file image bufer
			// First search for the extended dir in image file buffer
			for (idx = 0; idx < CODE_ENTRY_MAX; idx++) {
				value = *(u32_t *)&buf[NVM_OFFSET(dir.code[idx].code_attribute)];
				if (big_endian) value = ntohl(value);

				if ((value & CODE_IMAGE_LENGTH_MASK) == 0) {
					continue;
				} else if ((value & CODE_IMAGE_TYPE_MASK) == CODE_IMAGE_TYPE_EXTENDED_DIR) {
					// Found extended dir entry in image buffer to be programed
					extended_dir_entry_addr = NVM_OFFSET(dir.code[idx])+8;
					DbgMessage(NULL, WARN, "Found extended DIR in buf entry addr 0x%x\n", extended_dir_entry_addr);

					if (byte_cnt > extended_dir_entry_addr + 12) {
						p32_1 =(u32_t *)&buf[extended_dir_entry_addr];
						if (big_endian) {
							extended_dir_addr_in_buf = ntohl(*p32_1);
						} else {
							extended_dir_addr_in_buf = *p32_1;
						}
					}
					DbgMessage(NULL, WARN, "Extended DIR in buf address is 0x%x\n", extended_dir_addr_in_buf);
					break;
				}
			}

			if ((idx == CODE_ENTRY_MAX) || (extended_dir_addr_in_buf == 0)) {
				DbgMessage(NULL, WARN, "There is no extended dir in the image file\n");
			} else {
				// Copy extended DIR in buffer to extended_dir structure
				common_nvm_memcpy(&extended_dir_in_buf, (u32_t *)&buf[extended_dir_addr_in_buf], sizeof(extended_dir_image_t));

				no_images = htonl(extended_dir_in_buf.no_images);
				DbgMessage(NULL, WARN, "No of images in image file extended DIR is %d\n", no_images);

				// Find the image in the file's extended dir
				for (idx = 0; idx < no_images; idx++) {
					// Get image code_type and len
					value = extended_dir_in_buf.extended_dir_images[idx].code_attribute;
					code_type = htonl(value)& CODE_IMAGE_TYPE_MASK;
					len = htonl(value)& CODE_IMAGE_LENGTH_MASK;

					// Get image start address
					value = extended_dir_in_buf.extended_dir_images[idx].nvm_start_addr;
					start =  htonl(value);

					if (code_type == image_table[image_idx].type) {
						DbgMessage(NULL, WARN, "Found %s at start address 0x%x length 0x%x\n", image_table[image_idx].name, start, len);
						break;
					}
				}

				// If there is no such image in image file buffer, then exit without doing anything
				if (idx == no_images) {
					DbgMessage(NULL, WARN, "There is no %s in the image file\n", image_table[image_idx].name);
					return COMMON_NVM_STATUS_SUCCESS;
				}

				// Preserve the FCOE_BOOT_CFG image in NVM, by copying
				// it from NVM to file image buffer to the correct location in buffer
				if ((byte_cnt > start + len) && (len == code_len)) {
					u32_t word_num;

					// Replace file content with NVM content dword by dword
					DbgMessage(NULL, WARN, "Replace %s for func %d file content with NVM content\n", image_table[image_idx].name, func);
					word_num = (code_len+3)/4;

					for (i = 0; i < word_num; i++) {
						p32_1 =(u32_t *)&buf[start + i * 4];
						if (common_nvm_read_nvram(code_start + i * 4, &data32, sizeof(u32_t)) != COMMON_NVM_STATUS_SUCCESS) {
							return COMMON_NVM_STATUS_FAILURE;
						}
						*p32_1 = (big_endian) ? htonl(data32): data32;
					}
				} else {
					return COMMON_NVM_STATUS_SUCCESS;
				}
			}
		}
	}
	return COMMON_NVM_STATUS_SUCCESS;
}



/*
*------------------------------------------------------------------------
* common_nvm_nvm_parse_extended_dir_image --
*
*     Parse the the extended dir image into structure
*
*------------------------------------------------------------------------
*/
common_nvm_status_t common_nvm_nvm_parse_extended_dir_image(u32_t nvm_offset,
															u32_t image_length,
															extended_dir_image_t *extended_dir_p) {
	u8_t big_endian=1;                      /* TBD: verify big endian  */
	u32_t no_images=0;
	u8_t *buf=NULL;
	u32_t idx=0, byte_dumped=0;
	u32_t val;


	/* Allocate memory buffer for dumping the extended dir image */
	buf =(u8_t *)(uintptr_t)common_nvm_alloc_virt(image_length);

	if (buf == NULL) {
		DbgMessage(NULL, FATAL, "No available memory for dumping extended dir image !!\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* Dump the extended dir image in NVM into buffer */
	if (common_nvm_dump_nvm_into_buf(buf, big_endian, image_length, nvm_offset, &byte_dumped) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Failed to read extended dir image from NVM !!");
		if (buf) common_nvm_free_virt((int_ptr_t)buf);
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* Get the no_images in extended_dir from the first dword in buf */
	no_images = htonl((u32_t)*((u32_t *)buf));
	DbgMessage(NULL, INFORM, "Parsed extended dir no_images=%d!!\n", no_images);

	/* The first byte includes the number of images in the extended dir */
	if (no_images > MAX_IMAGES_IN_EXTENDED_DIR) {
		DbgMessage(NULL, FATAL, "Number of images in extended dir is invalid %d !!", no_images);
		if (buf) common_nvm_free_virt((int_ptr_t)buf);
		return COMMON_NVM_STATUS_FAILURE;
	}


	/* Copy dumped buffer into extended_dir_structure */
	common_nvm_memcpy(extended_dir_p, (u32_t *)buf, sizeof(extended_dir_image_t));

	/* Swap bytes of extended_dir info */
	extended_dir_p->no_images = htonl(extended_dir_p->no_images);
	extended_dir_p->total_byte_cnt = htonl(extended_dir_p->total_byte_cnt);

	for (idx = 0; idx < no_images; idx++) {
		val = extended_dir_p->extended_dir_images[idx].sram_start_addr;
		extended_dir_p->extended_dir_images[idx].sram_start_addr = htonl(val);

		val = extended_dir_p->extended_dir_images[idx].code_attribute;
		extended_dir_p->extended_dir_images[idx].code_attribute = htonl(val);

		val = extended_dir_p->extended_dir_images[idx].nvm_start_addr;
		extended_dir_p->extended_dir_images[idx].nvm_start_addr = htonl(val);

		/*
		DbgMessage(NULL, INFORM, "Extended dir idx %d: nvm_offset=0x%x byte_cnt=0x%x \n", idx,
					extended_dir_p->extended_dir_images[idx].nvm_start_addr,
					extended_dir_p->extended_dir_images[idx].code_attribute & CODE_IMAGE_LENGTH_MASK);
		*/
	}

	/* Free the temporary buf used */
	common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)buf);

	return COMMON_NVM_STATUS_SUCCESS;
}


/*
*------------------------------------------------------------------------
* common_nvm_nvm_update_extended_dir_image --
*
*     Update the extended dir image with the latest new image programmed
*     information (image type, byte_cnt, nvm_offset)
*
*------------------------------------------------------------------------
*/
common_nvm_status_t common_nvm_nvm_update_extended_dir_image(extended_dir_image_t * extended_dir_p,
															 u32_t image_type,
															 u32_t byte_cnt,
															 u32_t nvm_addr,
															 u32_t sram_addr,
															 u32_t old_byte_cnt,
															 u32_t agent) {
	u32_t no_images=0, total_byte_cnt=0, byte_prg=0, entry_idx=0;
	u32_t nvm_offset=0;
	u32_t image_existed;

	if (extended_dir_p == NULL) {
		DbgMessage(NULL, FATAL, "Invalid extended directory struct pointer !!\n");
		return COMMON_NVM_STATUS_FAILURE;
	}


	if (common_nvm_nvm_find_extended_dir_entry(image_type, extended_dir_p, NULL, NULL, NULL, NULL, NULL) == COMMON_NVM_STATUS_SUCCESS) {
		image_existed = 1;
		no_images = extended_dir_p->no_images;
	} else {
		image_existed = 0;
		no_images = extended_dir_p->no_images + 1;
	}

	if (no_images < 1) {
		DbgMessage(NULL, FATAL, "Can't update extended dir. Number of images < 1 !!\n");
		return COMMON_NVM_STATUS_FAILURE;
	}
	if (common_nvm_nvm_find_extended_dir_entry(image_type, extended_dir_p, &entry_idx, NULL, NULL, NULL, NULL) != COMMON_NVM_STATUS_SUCCESS) {
		entry_idx=no_images-1;
	}

	if (common_nvm_find_room(CODE_IMAGE_TYPE_EXTENDED_DIR, sizeof(extended_dir_image_t),
							 extended_dir_p, nvm_addr, byte_cnt, &nvm_offset) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Not enough space in NVM !!\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* this can't be done before find_room, as it will revert the change! */
	if (!image_existed) {
		extended_dir_p->no_images++;
	}
	/* Round up to 4 byte count */
	byte_cnt = ((byte_cnt + 3) & 0xfffffffc);
	nvm_addr = (nvm_addr + 3) & 0xfffffffc;

	/* Update extended dir struct total_byte_cnt with this image byte_cnt */
	total_byte_cnt = extended_dir_p->total_byte_cnt;

	/* If image existed, remove its old byte cnt */
	/* If image did not exist, old_byte_cnt is 0 */
	total_byte_cnt -= old_byte_cnt;

	/* Total byte cnt includes also extended dir image size
	   So that next image won't overwite the extended dir image */
	extended_dir_p->total_byte_cnt = total_byte_cnt + byte_cnt;

	extended_dir_p->extended_dir_images[entry_idx].sram_start_addr = sram_addr;
	extended_dir_p->extended_dir_images[entry_idx].nvm_start_addr = nvm_addr;
	extended_dir_p->extended_dir_images[entry_idx].code_attribute = (image_type & CODE_IMAGE_TYPE_MASK);
	extended_dir_p->extended_dir_images[entry_idx].code_attribute |= (byte_cnt & CODE_IMAGE_LENGTH_MASK);
	extended_dir_p->extended_dir_images[entry_idx].code_attribute |= (agent & CODE_IMAGE_AGENT_TYPE_MASK);

	/* Program the updated extended dir image */
	if (common_nvm_program_nvm((u8_t *)extended_dir_p, sizeof(extended_dir_image_t), nvm_offset,
							   PRG_NVM_LITTLE_ENDIAN | PRG_NVM_APPEND_CRC,
							   &byte_prg) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Programming extended dir image into NVM failed !!");
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* Update the extended dir entry with new info */
	if (common_nvm_update_dir_entry(CODE_ENTRY_EXTENDED_DIR_IDX, nvm_offset, byte_prg,
									0, CODE_IMAGE_TYPE_EXTENDED_DIR, CODE_IMAGE_AGENT_HOST) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Failed to update extended dir entry in NVM !!\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	return COMMON_NVM_STATUS_SUCCESS;
}


/*
*------------------------------------------------------------------------
* common_nvm_nvm_delete_extended_dir_entry --
*
*     Delete extended_dir entry
*     Reduce the no_images, and total_byte_cnt accordingly
*
*------------------------------------------------------------------------
*/
common_nvm_status_t common_nvm_nvm_delete_extended_dir_entry(extended_dir_image_t * extended_dir_p,
															 u32_t entry_idx, u32_t nvm_offset) {
	u32_t no_images=0, byte_prg=0;


	if (extended_dir_p == NULL) {
		DbgMessage(NULL, FATAL, "Invalid extended directory struct pointer !!\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	no_images = extended_dir_p->no_images;

	if (no_images < 1) {
		DbgMessage(NULL, FATAL, "Can't delete exteded dir entry. No images < 1 !!\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	if (no_images == 1) {
		if (common_nvm_update_dir_entry(CODE_ENTRY_EXTENDED_DIR_IDX, NVM_ERASE_VALUE, NVM_ERASE_VALUE,
										NVM_ERASE_VALUE, NVM_ERASE_VALUE, NVM_ERASE_VALUE) != COMMON_NVM_STATUS_SUCCESS) {

			DbgMessage(NULL, FATAL, "Failed to erase the extended dir entry in directory.\n");
			return COMMON_NVM_STATUS_FAILURE;
		}
		return COMMON_NVM_STATUS_SUCCESS;
	}

	if (common_nvm_find_room(CODE_IMAGE_TYPE_EXTENDED_DIR, sizeof(extended_dir_image_t), extended_dir_p, 0, 0, &nvm_offset) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Failed to find room for empty EXTENDED_DIR !!\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* reduce the total byte count with this image byte count */
	extended_dir_p->total_byte_cnt -= (extended_dir_p->extended_dir_images[entry_idx].code_attribute & CODE_IMAGE_LENGTH_MASK);

	/* delete this entry by overwritting it with last entry info */
	extended_dir_p->extended_dir_images[entry_idx].sram_start_addr = extended_dir_p->extended_dir_images[no_images - 1].sram_start_addr;
	extended_dir_p->extended_dir_images[entry_idx].code_attribute = extended_dir_p->extended_dir_images[no_images - 1].code_attribute;
	extended_dir_p->extended_dir_images[entry_idx].nvm_start_addr = extended_dir_p->extended_dir_images[no_images - 1].nvm_start_addr;

	/* delete the last entry */
	extended_dir_p->extended_dir_images[no_images - 1].sram_start_addr = 0;
	extended_dir_p->extended_dir_images[no_images - 1].code_attribute = 0;
	extended_dir_p->extended_dir_images[no_images - 1].nvm_start_addr = 0;

	/* reduce the no_images*/
	no_images --;
	extended_dir_p->no_images = no_images;

	/* Program the updated extended dir image */
	if (common_nvm_program_nvm((u8_t *)extended_dir_p, sizeof(extended_dir_image_t), nvm_offset,
							   PRG_NVM_LITTLE_ENDIAN | PRG_NVM_APPEND_CRC,
							   &byte_prg) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Programming extended dir image into NVM failed !!");

		return COMMON_NVM_STATUS_FAILURE;
	}

	/* Update the extended dir entry with new extended_dir image size */
	if (common_nvm_update_dir_entry(CODE_ENTRY_EXTENDED_DIR_IDX, nvm_offset, byte_prg,
									0, CODE_IMAGE_TYPE_EXTENDED_DIR, CODE_IMAGE_AGENT_HOST) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Failed to update extended dir entry in NVM !!\n");
		common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)extended_dir_p);
		return COMMON_NVM_STATUS_FAILURE;
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_nvm_find_extended_dir_entry --
 *
 *     Search if this image type already exist in exended
 *     If exist, return the entry in extended directory structure
 *	   Else return entry_idx -1
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS or COMMON_NVM_STATUS_FAILURE
 *     entry_idx if found, -1 if not found
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_nvm_find_extended_dir_entry(
														   IN   u32_t                      image_type,
														   IN   extended_dir_image_t       *extended_dir_p,
														   OUT  u32_t                      *entry_idx,
														   OUT  u32_t                      *start_address,   // if NULL,  do not assign into this output
														   OUT  u32_t                      *sram_addr,
														   OUT  u32_t                      *byte_cnt,
														   OUT  u32_t                      *agent
														   ) {
	u32_t idx;

	if (entry_idx != NULL) {
		*entry_idx = -1;
	}

	/* Verify the extended directory pointer is valid */
	if (extended_dir_p == NULL) {
		DbgMessage(NULL, FATAL, "Error: extended dir pointer does not exist !\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* Go through the entries in extended dir structure */
	for (idx = 0; idx < extended_dir_p->no_images; idx++) {
		if ((extended_dir_p->extended_dir_images[idx].code_attribute & CODE_IMAGE_TYPE_MASK) == image_type) {

			/* Found a matching image type in extended dir */
			/* Return it's entry index */
			if (entry_idx != NULL) {
				*entry_idx = idx;
			}

			/* Return the nvm offset of this matching image */
			if (start_address != NULL) {
				*start_address = extended_dir_p->extended_dir_images[idx].nvm_start_addr;
			}

			/* Return the sram offset of this matching image */
			if (sram_addr != NULL) {
				*sram_addr = extended_dir_p->extended_dir_images[idx].sram_start_addr;
			}

			/* Return the byte_cnt of this matching image */
			if (byte_cnt != NULL) {
				*byte_cnt = (extended_dir_p->extended_dir_images[idx].code_attribute & CODE_IMAGE_LENGTH_MASK);
			}
			if (agent != NULL) {
				*agent = (extended_dir_p->extended_dir_images[idx].code_attribute & CODE_IMAGE_AGENT_TYPE_MASK);
			}
			return COMMON_NVM_STATUS_SUCCESS;
		}
	}
	return COMMON_NVM_STATUS_FAILURE;
}


/*
 *------------------------------------------------------------------------
 * common_nvm_validate_extended_dir_byte_cnt --
 *
 *     Checks that byte_cnt field in extended dir is not corrupted (due to bugs
 *     in older versions). If corrupted, value is fixed and notice is shown
 *     to user.
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if operation succeeded. *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_validate_extended_dir_byte_cnt() {
	extended_dir_image_t extended_dir;
	u32_t offset, length, type, i, total_byte_cnt = 0, byte_prg;

	/* Clear the extended_dir */
	memset(&extended_dir, 0, sizeof(extended_dir_image_t));

	/* Check if extended dir already exists */
	if ((common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &offset, &length, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS)
		&& (type == CODE_IMAGE_TYPE_EXTENDED_DIR)) {

		/* If extended dir image already exist, parse it into extended_dir_image struct */
		if (common_nvm_nvm_parse_extended_dir_image(offset, length, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Failed to parse extended dir image !! \n");
			return COMMON_NVM_STATUS_FAILURE;
		}

		for (i = 0; i < extended_dir.no_images; i++) {
			total_byte_cnt += extended_dir.extended_dir_images[i].code_attribute & CODE_IMAGE_LENGTH_MASK;
		}

		if (total_byte_cnt != extended_dir.total_byte_cnt) {
			printf("Fixing extended dir byte_cnt value from 0x%x to 0x%x.\n",
				   extended_dir.total_byte_cnt,
				   total_byte_cnt);

			extended_dir.total_byte_cnt = total_byte_cnt;

			/* Program the updated extended dir image */
			if (common_nvm_program_nvm((u8_t *)&extended_dir, sizeof(extended_dir_image_t), offset,
									   PRG_NVM_LITTLE_ENDIAN | PRG_NVM_APPEND_CRC,
									   &byte_prg) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Programming extended dir image into NVM failed !!");
				return COMMON_NVM_STATUS_FAILURE;
			}
		}
	}

	return COMMON_NVM_STATUS_SUCCESS;
}


/*
 *------------------------------------------------------------------------
 * prog_image_in_extended_dir --
 *
 *     Program a new type image and update the extended dir image
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     Error message (if any) will be appended to the result object in
 *     interp.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_prog_image_in_extended_dir(
														  u8_t * buf,
														  u32_t byte_cnt,
														  u32_t image_table_idx,
														  u8_t validate_versions,
														  u8_t silent,
														  u8_t remove_swims,
														  u8_t mbi_upgrade
														  ) {
	u32_t entry_idx;
	u32_t byte_prg, nvm_addr, extra_flag = 0;
	u32_t old_nvm_addr, old_byte_cnt;
	u32_t extended_dir_image_size;
	extended_dir_image_t extended_dir;
	u32_t sram_addr, offset, length, type, image_type, image_type1=0, image_type2=0, image_type3=0, agent_type;
	u8_t *newbuf_ptr = NULL;
	int is_4port = 0;
	u8_t same_image = 0;

#ifndef FWUPGTOOL
	common_nvm_dev_info_t *cur_dev_p = COMMON_NVM_CUR_DEV;
	common_nvm_dev_stat_t dev_stat;
#else
	// 57710 use ENV_PORT0 to determine port.
	// ENV_PORT0 was set up in SetSelectedNIC() of Bnx2Helper.cpp
	u8_t is_port0 = atoi(Tcl_GetVar2(gpINx2Nvm->m_tcl_interp, TCL_ENV, ENV_PORT0, 0));
	u8_t port_id = is_port0 ? 0:1; // set the port ID
	u32_t device_id;
	Tcl_Obj *obj;
#endif // !FWUPGTOOL

	char old_version[64]="N/A";
	char new_version[64]="N/A";
	u8_t image_existed=0;
	common_nvm_status_t rc=0;
	u32_t pci_id_offset, pci_sub_id_offset, val32;
	u16_t device_id, vendor_id, sub_vendor_id, sub_device_id;

	/* Get the sub_vendor_id from NVM configuration,
   Required later to decide if to prgram OCNVM image */
	common_nvm_get_nvm_offset_for_pci_id(&pci_id_offset, &pci_sub_id_offset);
	if (common_nvm_read_nvram(pci_sub_id_offset, &val32, sizeof(u32_t)) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Reading VID from NVM failed.", NULL);
		return COMMON_NVM_STATUS_FAILURE;
	}
	sub_vendor_id =(u16_t)MANUF_INFO_PCI_SUBSYS_VENDOR_ID(val32);
	sub_device_id =(u16_t)MANUF_INFO_PCI_SUBSYS_DEVICE_ID(val32);

	if (common_nvm_read_nvram(pci_id_offset, &val32, sizeof(u32_t)) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Reading VID/DID from NVM failed.", NULL);
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* Update VID and DID*/
	device_id =(u16_t)MANUF_INFO_PCI_DEVICE_ID(val32);
	vendor_id =(u16_t)MANUF_INFO_PCI_VENDOR_ID(val32);

	/* Clear the extended_dir */
	memset(&extended_dir, 0, sizeof(extended_dir_image_t));

	image_type = image_table[image_table_idx].type;
	agent_type = image_table[image_table_idx].agent;    /*e3 temp*/

	/* Select FCOE boot data according to the currently selected device */
	if (image_type == CODE_IMAGE_TYPE_FCOE_BOOT_CFG) {
#ifndef FWUPGTOOL
		if (!COMMON_NVM_CUR_DEV_IS_VALID) {
			return COMMON_NVM_STATUS_FAILURE;
		}

		if (common_nvm_get_func_num_from_dev_info(cur_dev_p) == 1) {
			image_type = CODE_IMAGE_TYPE_FCOE_BOOT_CFG2;
			image_type1 = CODE_IMAGE_TYPE_FCOE_BOOT_CFG;
		} else {
			image_type1 = CODE_IMAGE_TYPE_FCOE_BOOT_CFG2;
		}

		common_nvm_get_info(&dev_stat);

		common_nvm_is_4port(&is_4port);
		if ((is_4port) && is_e3_57840(globalInterp)) {
			image_type2= CODE_IMAGE_TYPE_FCOE_BOOT_CFG3;
			image_type3= CODE_IMAGE_TYPE_FCOE_BOOT_CFG4;
		}
#else
		if (port_id == 1) {
			image_type = CODE_IMAGE_TYPE_FCOE_BOOT_CFG2;
			image_type1 = CODE_IMAGE_TYPE_FCOE_BOOT_CFG;
		} else {
			image_type1 = CODE_IMAGE_TYPE_FCOE_BOOT_CFG2;
		}

		// The ENV_CHIP_REV was set in Bnx2Helper::SetSelectedNIC()
		obj = Tcl_GetVar2Ex(gpINx2Nvm->m_tcl_interp, TCL_ENV, ENV_DID, 0);
		Tcl_GetIntFromObj(gpINx2Nvm->m_tcl_interp, obj, (int *)&device_id);

		common_nvm_is_4port(&is_4port);
		if ((is_4port) &&
			((device_id == DEVICE_ID_E3_57840) ||
			 (device_id == DEVICE_ID_E3_57840_4_10) ||
			 (device_id == DEVICE_ID_E3_57840_2_20) ||
			 (device_id == DEVICE_ID_E3_57840_MF))) {
			image_type2= CODE_IMAGE_TYPE_FCOE_BOOT_CFG3;
			image_type3= CODE_IMAGE_TYPE_FCOE_BOOT_CFG4;
		}
#endif // !FWUPGTOOL
		/* Check if OEM NPAR1.5 device ID */
		if (common_nvm_is_oem_npar_device(vendor_id, sub_vendor_id, sub_device_id)) {
			image_type2= CODE_IMAGE_TYPE_FCOE_BOOT_CFG3;
			image_type3= CODE_IMAGE_TYPE_FCOE_BOOT_CFG4;
		}
	}

	if (image_type == CODE_IMAGE_TYPE_E3_PCIEV2) {
		extra_flag = PRG_NVM_EXTRA_PADDING;
	}
	while (1) {
		DbgMessage(NULL, INFORM, "Programming image with type 0x%x, size 0x%x\n", image_type, byte_cnt);

		old_nvm_addr=0;
		old_byte_cnt=0;
		extended_dir_image_size=0;
		offset=0;
		length=0;
		type=0;
		image_existed=0;

		/* Check if extended dir already exists */
		if ((common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &offset, &length, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS)
			&& (type == CODE_IMAGE_TYPE_EXTENDED_DIR)) {
			/* If extended dir image already exist, parse it into extended_dir_image struct */
			if (common_nvm_nvm_parse_extended_dir_image(offset, length, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Failed to parse extended dir image !! \n");
				return COMMON_NVM_STATUS_FAILURE;
			}

		} else {

			/* Program an empty EXTENDED_DIR image, to save room for EXTENDED_DIR image */
			if (common_nvm_find_room(CODE_IMAGE_TYPE_EXTENDED_DIR, sizeof(extended_dir_image_t),
									 &extended_dir, 0, 0, &offset) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Failed to find room for empty EXTENDED_DIR !!\n");

				return COMMON_NVM_STATUS_FAILURE;
			}

			/* Program the updated extended dir image */
			if (common_nvm_program_nvm((u8_t *)&extended_dir, sizeof(extended_dir_image_t), offset,
									   PRG_NVM_LITTLE_ENDIAN | PRG_NVM_APPEND_CRC,
									   &byte_prg) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Programming extended dir image into NVM failed !!");
				return COMMON_NVM_STATUS_FAILURE;
			}

			/* Update the extended dir entry with new info */
			if (common_nvm_update_dir_entry(CODE_ENTRY_EXTENDED_DIR_IDX, offset, byte_prg,
											0, CODE_IMAGE_TYPE_EXTENDED_DIR, CODE_IMAGE_AGENT_HOST) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Failed to update extended dir entry in NVM !!\n");
				return COMMON_NVM_STATUS_FAILURE;
			}
		}

		/* If extended dir already exists, return the entry_idx of this image (if exists)
		   and the old_nvm_addr - NVM offset of the image with this type (if exists)  */
		if (common_nvm_nvm_find_extended_dir_entry(image_type, &extended_dir,
												   &entry_idx, &old_nvm_addr, NULL, &old_byte_cnt, NULL) == COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, INFORM, "Found a matching image type on entry_idx %d !!\n", entry_idx);
			image_existed=1;
		} else {
			image_existed=0;
		}

		/* CQ69138 Show HOSTBCV in nvm dir 'agent' column for BCV image */
		if (image_type == CODE_IMAGE_TYPE_FCOE_BOOT) {
			if (common_nvm_is_bcv_image_type(buf)) {
				agent_type = CODE_IMAGE_AGENT_HOST_BCV;
			}
		}

		common_nvm_perform_image_manipulation(&buf, &newbuf_ptr, image_type, agent_type, &sram_addr, &byte_cnt);

		common_nvm_fw_ver_string((common_nvm_virt_addr_t)(uintptr_t)buf, image_table[image_table_idx].name, &new_version[0], READ_VERSION_FROM_IMAGE);

		/* store version values of new MFW to be burned */
		// Note: code assumes MFW1 is always the first image in MCP package
		if (image_type == CODE_IMAGE_TYPE_MFW1) {
			if (sscanf(new_version, "MFW1 %d.%d.%d", &new_mfw_major, &new_mfw_minor, &new_mfw_build) != 3) {
				sscanf(new_version, "MFW1 %d.%d.%d.%d", &new_mfw_major, &new_mfw_minor, &new_mfw_build, &new_mfw_drop);
			}
		}
		/* If image exists in NVM and version validation required (-F option was not used)
		   then validate that the version in file (new_version) is indeed newer than
		   the version in NVM (old_version) */
		if (image_existed) {
			common_nvm_fw_ver_string((common_nvm_virt_addr_t)old_nvm_addr,
									 image_table[image_table_idx].name,
									 &old_version[0],
									 READ_VERSION_FROM_NVM);
			if (validate_versions) {
				if (common_nvm_image_version_validation(image_table[image_table_idx].name, old_version, new_version, &same_image)
					!= COMMON_NVM_STATUS_SUCCESS) {

					if (image_type2 &&
						((image_type == CODE_IMAGE_TYPE_FCOE_BOOT_CFG) ||
						 (image_type == CODE_IMAGE_TYPE_FCOE_BOOT_CFG2))) {
						/* If we need to upgrade FEB_CFG3,4
						   but FEB_CFG is the same version
						   continue to next image without failure */
						goto next_image;
					} else {
						/* Image is the same version */
						if (mbi_upgrade && same_image) {
							/* Don't fail MBI upgrade if a single image is the same */
							return COMMON_NVM_STATUS_SUCCESS;
						}
						/* Non MBI upgrade - fail upgrade */
						return COMMON_NVM_STATUS_FAILURE;
					}
				}
				printf("Upgrading %s image: from ver %s to ver %s\n",
					   image_table[image_table_idx].name,
					   old_version,
					   new_version);
			} else {
				if (!silent) {
					printf("Forced upgrading %s image: from ver %s to ver %s\n",
						   image_table[image_table_idx].name,
						   old_version,
						   new_version);
				}
			}
		} else {
			printf("Upgrading %s image: to version %s\n", image_table[image_table_idx].name, new_version);
		}

		if (image_type == CODE_IMAGE_TYPE_MFW1) {
			// delete MFWs beside MFW1
			common_nvm_delete_mfw_and_swim_images(DELETE_MFW_CMD_ALL_MFWS_BESIDES_MFW1, remove_swims);
		}

		/* NOTE: The common_nvm_find_room() should be after common_nvm_image_version_validation(), since this procedure also writes to NVM
				and if version_validation is not successful, we break after corrupting the NVM */

		/* Find place in NVM for the new image */
		if (common_nvm_find_room(image_type, byte_cnt, &extended_dir, 0, 0, &nvm_addr) != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Not enough space in NVM !!\n");
			return COMMON_NVM_STATUS_FAILURE;
		}

		/* Program the new image */
		// Make the data in buf to be in BE if the host is BE before calling program_nvm()
		swap_buffer_if_big_endian_host((u32_t *)buf, (byte_cnt / sizeof(u32_t)));
		if (common_nvm_program_nvm(buf, byte_cnt, nvm_addr,
								   PRG_NVM_LITTLE_ENDIAN | PRG_NVM_APPEND_CRC | extra_flag,
								   &byte_prg) != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Programming NVM failed !!\n");
			return COMMON_NVM_STATUS_FAILURE;
		}


		// Recover the byte order in buf if they are re-used again,
		// such as for CODE_IMAGE_TYPE_FCOE_BOOT_CFG/2/3/4
		if ((image_type == CODE_IMAGE_TYPE_FCOE_BOOT_CFG) ||
			(image_type == CODE_IMAGE_TYPE_FCOE_BOOT_CFG2) ||
			(image_type == CODE_IMAGE_TYPE_FCOE_BOOT_CFG3) ||
			(image_type == CODE_IMAGE_TYPE_FCOE_BOOT_CFG4)) swap_buffer_if_big_endian_host((u32_t *)buf, (byte_cnt / sizeof(u32_t)));

		if (image_type == CODE_IMAGE_TYPE_MFW1) {
			/* Update bootstrap */
			rc = common_nvm_update_bootstrap(sram_addr, byte_prg, nvm_addr);
			if (rc != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Bootstrap update in NVM failed.");
				return COMMON_NVM_STATUS_FAILURE;
			}
		}

		/* Update the extended dir image with the new image information */
		if (common_nvm_nvm_update_extended_dir_image(&extended_dir, image_type, byte_prg, nvm_addr, sram_addr, old_byte_cnt, agent_type) != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Failed to update extended dir image !!\n");
			return COMMON_NVM_STATUS_FAILURE;
		}
	next_image:
		if (image_type1) {
			image_type = image_type1;
			image_type1 = 0;
		} else if (image_type2) {
			image_type = image_type2;
			image_type2 = 0;
		} else if (image_type3) {
			image_type = image_type3;
			image_type3 = 0;
		} else {
			break;
		}
	} //while(1)

	/* Clean up if needed */
	if (newbuf_ptr != NULL) {
		common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)newbuf_ptr);
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_nvm_fill_mac_partition_info --
 *
 *     Fills input struct with MAC partition info read from NVM image,
 *     or zeros if such image does not exist.
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if operation succeeded (even if image does
 *     not exist)
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_nvm_fill_mac_partition_info(nvm_nic_part_cfg_t * mac_partition_block, u32_t * result) {
	u32_t byte_cnt, nvm_addr, entry_idx, status, found_image;

	/* search for MAC partition image in extended dir */
	status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_NIC_PARTITION_CFG, &nvm_addr, &byte_cnt, &entry_idx);

	if (status != COMMON_NVM_STATUS_SUCCESS) {
		if (result) {
			*result = FALSE;
		}
		return status;
	}

	/* if couldn't find image, return default struct */
	if (!found_image) {
		common_nvm_memset(mac_partition_block, 0, sizeof(nvm_nic_part_cfg_t));

		/* write format version */
		mac_partition_block->global_cfg.gbl_cfg = (MAC_PARTITION_FORMAT_VERSION & MACP_GBL_CFG_FORMAT_VER_MASK) >> MACP_GBL_CFG_FORMAT_VER_SHIFT;

		if (result) {
			*result = FALSE;
		}
		return COMMON_NVM_STATUS_SUCCESS;
	}

	/* read whole MAC partition info image*/
	common_nvm_read_nvram(nvm_addr,
						  (u32_t *)(mac_partition_block),
						  sizeof(nvm_nic_part_cfg_t));


	if (result) {
		*result = TRUE;
	}
	return COMMON_NVM_STATUS_SUCCESS;
}


common_nvm_status_t common_nvm_nvm_fill_nvm_cfg_extended_shared(u32_t * nvm_cfg_extended_shared_image, u32_t max_image_size) {
	u32_t byte_cnt, nvm_addr, entry_idx, status, found_image;

	common_nvm_memset(nvm_cfg_extended_shared_image, 0, max_image_size);

	/* search for image in extended dir */
	status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &nvm_addr, &byte_cnt, &entry_idx);
	if (status != COMMON_NVM_STATUS_SUCCESS) {
		return status;
	}

	/* if couldn't find image, return default struct */
	if (!found_image) {
		return COMMON_NVM_STATUS_SUCCESS;
	}

	/* if image is above maximal size, error */
	if (byte_cnt > max_image_size) {
		printf("nvm cfg extended shared image is larger than maximum defined size\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* read whole info image (ignore crc) */
	common_nvm_read_nvram(nvm_addr,
						  nvm_cfg_extended_shared_image,
						  byte_cnt - 4);

	return COMMON_NVM_STATUS_SUCCESS;
}


/*
 *------------------------------------------------------------------------
 * common_nvm_program_mac_partition_to_nvm --
 *
 *     Checks if user changed MAC partition configuration. If so, save it to
 *     MAC partition image in extended dir. If such an image didn't exist,
 *     create it.
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if operation succeeded.
 *
 * Side effects:
 *     Creation of MAC partition image in extended dir if it didn't exist
 *     and user changed MAC partition configuration.
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_program_mac_partition_to_nvm(nvm_cfg_t * nvm_cfg, nvm_nic_part_cfg_t * mac_partition_ptr) {
	u32_t is_dirty = FALSE;
	u8_t *mac_partition_buf;
	u8_t *mac_partition_copy_buf;
	u32_t found_image, byte_cnt, nvm_addr, entry_idx, status, port_num, func_num, path_num, i, index_image_table;
	u8_t *image;
	nvm_nic_part_cfg_t *nvm_nic_part_cfg;

	common_nvm_get_port_num(&port_num);
	common_nvm_get_func_num(&func_num);
	common_nvm_get_path_num(&path_num);

	/* check if mac_partition image was changed */
	mac_partition_buf =(u8_t *)&nvm_cfg->macp_gbl;
	mac_partition_copy_buf =(u8_t *)&mac_partition_ptr->global_cfg;
	for (i = 0; i < sizeof(struct macp_gbl_cfg); i++) {
		if (mac_partition_buf[i] != mac_partition_copy_buf[i]) {
			is_dirty = TRUE;
			break;
		}
	}
	if (!is_dirty) {
		mac_partition_buf =(u8_t *)&nvm_cfg->macp_port;
		mac_partition_copy_buf =(u8_t *)&mac_partition_ptr->port_cfg[path_num][port_num];
		for (i = 0; i < sizeof(struct macp_port_cfg); i++) {
			if (mac_partition_buf[i] != mac_partition_copy_buf[i]) {
				is_dirty = TRUE;
				break;
			}
		}
	}

	if (!is_dirty) {
		mac_partition_buf =(u8_t *)&nvm_cfg->macp_func;
		mac_partition_copy_buf =(u8_t *)&mac_partition_ptr->func_cfg[func_num];
		for (i = 0; i < sizeof(struct macp_func_cfg); i++) {
			if (mac_partition_buf[i] != mac_partition_copy_buf[i]) {
				is_dirty = TRUE;
				break;
			}
		}
	}

	if (is_dirty) {
		image =(u8_t *)(uintptr_t)common_nvm_alloc_virt(sizeof(nvm_nic_part_cfg_t));
		if (image == NULL) {
			DbgMessage(NULL, FATAL, "No available memory for dumping mac partion image !!\n");
			return COMMON_NVM_STATUS_FAILURE;
		}
		nvm_nic_part_cfg =(nvm_nic_part_cfg_t *)image;

		/* serach for MAC partition image in extended dir */
		status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_NIC_PARTITION_CFG, &nvm_addr, &byte_cnt, &entry_idx);

		if (status != COMMON_NVM_STATUS_SUCCESS) {
			/* Free the allocated image buffer */
			common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)image);
			return status;
		}

		if (found_image) {
			/* Read current mac partition image from NVM */
			common_nvm_read_nvram(nvm_addr,
								  (u32_t *)image,
								  sizeof(nvm_nic_part_cfg_t));
		} else {
			/* could not find mac_partition image in extended dir. */
			common_nvm_memset(image, 0, sizeof(nvm_nic_part_cfg_t));
		}

		/* modify mac partition image according to changes performed through nvm cfg */
		common_nvm_memcpy(&(nvm_nic_part_cfg->global_cfg), &nvm_cfg->macp_gbl, sizeof(struct macp_gbl_cfg));
		common_nvm_memcpy(&(nvm_nic_part_cfg->port_cfg[path_num][port_num]), &nvm_cfg->macp_port, sizeof(struct macp_port_cfg));
		common_nvm_memcpy(&(nvm_nic_part_cfg->func_cfg[func_num]), &nvm_cfg->macp_func, sizeof(struct macp_func_cfg));

		/* find image index in image_table */
		status = common_nvm_find_entry_in_image_table(CODE_IMAGE_TYPE_NIC_PARTITION_CFG, &index_image_table);
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			/* Free the allocated image buffer */
			common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)image);
			return status;
		}
		// We need this because data in image has to be
		// in LE format before calling common_nvm_prog_image_in_extended_dir()
		// Data in image were read from NVRAM in LE format for LE machine
		// and in BE format for BE machine...
		swap_buffer_if_big_endian_host((u32_t *)image,
									   (sizeof(nvm_nic_part_cfg_t) / sizeof(u32_t)));

		/* program modified image */
		/* Do not validate versions, silent mode */
		common_nvm_prog_image_in_extended_dir(image, sizeof(nvm_nic_part_cfg_t), index_image_table, FALSE, TRUE, FALSE, FALSE);

		/* Free the allocated image buffer */
		common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)image);
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_create_mac_partition_image --
 *
 *     Create NPAR1.5 image if it doesn't exist
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if operation succeeded.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_create_mac_partition_image(void) {

	u32_t found_image, byte_cnt, nvm_addr, entry_idx, status, index_image_table;
	u8_t *image;

	image =(u8_t *)(uintptr_t)common_nvm_alloc_virt(sizeof(nvm_nic_part_cfg_t));
	if (image == NULL) {
		DbgMessage(NULL, FATAL, "No available memory for mac partion image !!\n");
		return COMMON_NVM_STATUS_FAILURE;
	}
	common_nvm_memset(image, 0, sizeof(nvm_nic_part_cfg_t));

	/* Search for MAC partition image in extended dir */
	status = common_nvm_nvm_find_image_in_extended_dir(&found_image,
													   CODE_IMAGE_TYPE_NIC_PARTITION_CFG,
													   &nvm_addr,
													   &byte_cnt,
													   &entry_idx);

	/* In case of failure, free the allocated image buffer and exit */
	if (status != COMMON_NVM_STATUS_SUCCESS) {
		common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)image);
		return status;
	}

	/* If image already exist, exit with doing nothing */
	if (found_image) {
		return status;
	}

	/* Find image index in image_table */
	status = common_nvm_find_entry_in_image_table(CODE_IMAGE_TYPE_NIC_PARTITION_CFG, &index_image_table);
	if (status != COMMON_NVM_STATUS_SUCCESS) {
		/* Free the allocated image buffer */
		common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)image);
		return status;
	}

	/* Image has to be in LE format */
	swap_buffer_if_big_endian_host((u32_t *)image, (sizeof(nvm_nic_part_cfg_t) / sizeof(u32_t)));

	/* Program image, do not validate versions (silent mode) */
	common_nvm_prog_image_in_extended_dir(image, sizeof(nvm_nic_part_cfg_t), index_image_table, FALSE, TRUE, FALSE, FALSE);

	/* Free the allocated image buffer */
	common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)image);

	return COMMON_NVM_STATUS_SUCCESS;
}

common_nvm_status_t common_nvm_program_extended_nvm_cfg_shared_block(nvm_cfg_t * nvm_cfg, u32_t * nvm_cfg_extended_shared_block) {
	u32_t is_dirty = FALSE;
	u8_t *nvm_cfg_extended_shared_buf;
	u8_t *nvm_cfg_extended_shared_copy_buf;
	u8_t *image;
	u32_t found_image, byte_cnt, nvm_addr, entry_idx, status, i, index_image_table, new_byte_cnt;

	/* find image index in image_table */
	status = common_nvm_find_entry_in_image_table(CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &index_image_table);
	if (status != COMMON_NVM_STATUS_SUCCESS) return status;

	/* serach for relevant image in extended dir */
	status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &nvm_addr, &byte_cnt, &entry_idx);
	if (status != COMMON_NVM_STATUS_SUCCESS) return status;

	if (found_image) {
		/* remove crc from length calculation */
		byte_cnt -= 4;
	} else {
		byte_cnt = 0;
	}

	/* new image size is MAX between current image to values supported by this ediag version */
	new_byte_cnt = MAX(byte_cnt, sizeof(struct extended_dev_info_shared_cfg));
	if (byte_cnt < sizeof(struct extended_dev_info_shared_cfg)) {
		is_dirty = TRUE;
	}

	if (!is_dirty) {
		/* check if image was changed */
		nvm_cfg_extended_shared_buf =(u8_t *)&nvm_cfg->extended_dev_info_shared;
		nvm_cfg_extended_shared_copy_buf =(u8_t *)nvm_cfg_extended_shared_block;
		for (i = 0; i < sizeof(struct extended_dev_info_shared_cfg); i++) {
			if (nvm_cfg_extended_shared_buf[i] != nvm_cfg_extended_shared_copy_buf[i]) {
				is_dirty = TRUE;
				break;
			}
		}
	}

	if (is_dirty) {
		image =(u8_t *)(uintptr_t)common_nvm_alloc_virt(new_byte_cnt);
		if (image == NULL) {
			DbgMessage(NULL, FATAL, "No available memory for extended_dev_info_shared_cfg !!\n");
			return COMMON_NVM_STATUS_FAILURE;
		}

		common_nvm_memset(image, 0, new_byte_cnt);
		if (found_image) {
			/* Read current NIV image from NVM */
			common_nvm_read_nvram(nvm_addr, (u32_t *)image, byte_cnt);
		}
		/* modify NIV image according to changes performed through nvm cfg */
		common_nvm_memcpy(image, &nvm_cfg->extended_dev_info_shared, sizeof(struct extended_dev_info_shared_cfg));

		// The data in image has to be in LE format before
		// calling common_nvm_prog_image_in_extended_dir()
		swap_buffer_if_big_endian_host((u32_t *)image,
									   (new_byte_cnt / sizeof(u32_t)));

		/* program modified image */
		common_nvm_prog_image_in_extended_dir(image, new_byte_cnt, index_image_table, FALSE, TRUE, FALSE, FALSE);

		/* Free the allocated image buffer */
		common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)image);
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_nvm_find_image_in_extended_dir --
 *
 *     Checks if an image suitable to input type exists in extended dir.
 *     If yes, returns its parameters.
 *     Function can handle scenario that extended dir image itself doesn't
 *     exist.
 *
 * Input:
 *     image_type - type to search in extended dir.
 *
 * Output:
 *     found_image - TRUE if found image in extended dir.
 *     nvm_addr - address if the image (if found)
 *     byte_cnt - byte count of image (if found)
 *     entry_idx - entry in extended dir (if found)
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if operation succeeded.
 *     found_image is TRUE if found
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_nvm_find_image_in_extended_dir(u32_t * found_image, u32_t image_type, u32_t *nvm_addr, u32_t *byte_cnt, u32_t *entry_idx) {
	extended_dir_image_t extended_dir;
	u32_t type, status;

	/* Check if extended dir exist */
	if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, nvm_addr, byte_cnt, NULL, &type, NULL) != COMMON_NVM_STATUS_SUCCESS ||
		type != CODE_IMAGE_TYPE_EXTENDED_DIR) {
		/* could not find extended dir image. Simply return mac_partition_block as all zeros */
		DbgMessage(NULL, INFORM, "could not find extended dir image\n");
		*found_image = 0;
		return COMMON_NVM_STATUS_SUCCESS;
	}

	/* If reached here - extended dir image exist.
	   parse the extended dir image into extended_dir_image struct */
	if (common_nvm_nvm_parse_extended_dir_image(*nvm_addr, *byte_cnt, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Could not parse extended dir image, even though it exists\n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* check if mac_partition image exists in extended dir */
	status = common_nvm_nvm_find_extended_dir_entry(image_type, &extended_dir, entry_idx, nvm_addr, NULL, byte_cnt, NULL);
	if (status != COMMON_NVM_STATUS_SUCCESS) {
		/* could not find requested image in extended dir. Simply return mac_partition_block as all zeros */
		DbgMessage(NULL, INFORM, "could not find requested image in extended dir\n");
		*found_image = 0;
		return COMMON_NVM_STATUS_SUCCESS;
	}

	*found_image = 1;
	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_handle_mac_partition_data --
 *
 *     If user requested to remember old mac partition values, nvm data will
 *     hold data according to the table below:
 *
 *     original nvm     new nvm
 *     included         included
 *     mac_p image?     mac_p image?   logics
 *     ------------     -----------    ------
 *     No               No             no mac_p image
 *     No               No             no mac_p image
 *     Yes              No             original mac_p image with empty MACs
 *     Yes              Yes            original mac_p image with new MACs
 *
 * Input:
 *     macp_image_found - Whether mac_p existed in source NVM
 *     macp_image - values of mac_p image if existed in source NVM
 *
 *
 * Output:
 *      None, besides "side effect" of new NVM holding right mac_p image
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if operation succeeded.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_handle_mac_partition_data(u32_t orig_macp_image_found,
														 nvm_nic_part_cfg_t orig_macp_image) {
	u32_t new_macp_image_found;
	nvm_nic_part_cfg_t new_macp_image;
	u32_t i, index_image_table;
	common_nvm_status_t status;
	int is_4port;
	u32_t wwn_prefix=0, wwn_prefix_offset;
	u32_t nvm_addr, entry_idx, found_image, byte_cnt;
	u32_t is_nicp1_5=0;

	common_nvm_is_4port(&is_4port);

	common_nvm_nvm_fill_mac_partition_info(&new_macp_image, &new_macp_image_found);

	/* special handling should take effect only if new nvm includes
	   nicp image (according to the table above) */
	if (new_macp_image_found) {
		/* if orig nvm had no mac_p image, set mac addr according to SF L2 MAC from NVM */
		if (!orig_macp_image_found) {

			/* find wwn prefix in nvram */
			status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &nvm_addr, &byte_cnt, &entry_idx);
			if ((status == COMMON_NVM_STATUS_SUCCESS) && found_image) {
				wwn_prefix_offset = OFFSETOF(struct extended_dev_info_shared_cfg, wwn_prefix);
				if (byte_cnt >= wwn_prefix_offset) {
					common_nvm_read_nvram(nvm_addr + wwn_prefix_offset, &wwn_prefix, sizeof(u32_t));
				}
			}

			/* Set mac addr according to SF L2 MAC from NVM */
			if (common_nvm_is_l2_mac_initialized()) {
				for (i = 0; i < E1H_FUNC_MAX; i++) {
					common_nvm_init_npar_mac_addr(i, is_4port, is_nicp1_5, wwn_prefix, &new_macp_image);
				}
			}
		}
		/* if orig nvm had mac_p image, take MAC addresses from that image */
		else {
			for (i = 0; i < E1H_FUNC_MAX; i++) {
				new_macp_image.func_cfg[i].net_mac_addr_upper = orig_macp_image.func_cfg[i].net_mac_addr_upper;
				new_macp_image.func_cfg[i].net_mac_addr_lower = orig_macp_image.func_cfg[i].net_mac_addr_lower;
				new_macp_image.func_cfg[i].iscsi_mac_addr_upper = orig_macp_image.func_cfg[i].iscsi_mac_addr_upper;
				new_macp_image.func_cfg[i].iscsi_mac_addr_lower = orig_macp_image.func_cfg[i].iscsi_mac_addr_lower;
				new_macp_image.func_cfg[i].fcoe_mac_addr_upper = orig_macp_image.func_cfg[i].fcoe_mac_addr_upper;
				new_macp_image.func_cfg[i].fcoe_mac_addr_lower = orig_macp_image.func_cfg[i].fcoe_mac_addr_lower;
				new_macp_image.func_cfg[i].fcoe_node_wwn_upper = orig_macp_image.func_cfg[i].fcoe_node_wwn_upper;
				new_macp_image.func_cfg[i].fcoe_node_wwn_lower = orig_macp_image.func_cfg[i].fcoe_node_wwn_lower;
				new_macp_image.func_cfg[i].fcoe_port_wwn_upper = orig_macp_image.func_cfg[i].fcoe_port_wwn_upper;
				new_macp_image.func_cfg[i].fcoe_port_wwn_lower = orig_macp_image.func_cfg[i].fcoe_port_wwn_lower;
			}
		}

		/* find image index in image_table */
		status = common_nvm_find_entry_in_image_table(CODE_IMAGE_TYPE_NIC_PARTITION_CFG, &index_image_table);
		if (status != COMMON_NVM_STATUS_SUCCESS) return status;

		// The data in image has to be in LE format
		// before calling common_nvm_prog_image_in_extended_dir()
		swap_buffer_if_big_endian_host((u32_t *)&new_macp_image,
									   (sizeof(nvm_nic_part_cfg_t) / sizeof(u32_t)));

		/* program modified image */
		/* Do not validate versions, silent mode */
		common_nvm_prog_image_in_extended_dir((u8_t *)&new_macp_image, sizeof(nvm_nic_part_cfg_t), index_image_table, FALSE, TRUE, FALSE, FALSE);
	}

	return COMMON_NVM_STATUS_SUCCESS;
}


/*
 *------------------------------------------------------------------------
 * common_nvm_find_entry_in_image_table --
 *
 *     Returns index of image in global image table.
 *
 * Input:
 *     image_type - type to search in the table.
 *
 * Output:
 *     Index - index of image in the table if found.
 *
 * Results:
 *     COMMON_NVM_STATUS_SUCCESS if found image in table. *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_find_entry_in_image_table(u32_t image_type, u32_t *index) {
	u32_t i;
	u32_t image_table_size = common_nvm_image_table_size();

	for (i = 0; i < image_table_size; i++) {
		if (image_table[i].type == image_type) {
			*index = i;
			return COMMON_NVM_STATUS_SUCCESS;
		}
	}

	/* if reached here couldn't find image in image_table */
	return COMMON_NVM_STATUS_FAILURE;
}


/*
 *------------------------------------------------------------------------
 * common_nvm_get_image_in_extended_dir_info --
 *
 *     Returns NVM offset and length of image by it's image name
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_get_image_in_extended_dir_info(char *image_name, u32_t *offset, u32_t *length, u32_t *image_table_idx) {
	u32_t idx, nvm_addr, byte_cnt, type, entry_idx;
	extended_dir_image_t extended_dir;

	for (idx = 0; idx < common_nvm_image_table_size(); idx++) {
		if (strstr(image_table[idx].name, image_name)) {
			*image_table_idx=idx;
			break;
		}
	}

	/* Get the length of the VPD image from the extended dir */
	if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &byte_cnt, NULL, &type, NULL)
		== COMMON_NVM_STATUS_SUCCESS) {

		if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
			/* If extended dir image exist, parse the extended dir image into structure
			   parse it into extended_dir_image struct */
			if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, byte_cnt, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Failed to parse extended dir image");
				return COMMON_NVM_STATUS_FAILURE;
			}

			/* Search for this image in extended dir */
			if (common_nvm_nvm_find_extended_dir_entry(image_table[idx].type, &extended_dir,
													   &entry_idx, offset, NULL, length, NULL) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, WARN, "%s image does not exist in extended dir", image_name);
				return COMMON_NVM_STATUS_FAILURE;
			}

		} else {
			DbgMessage(NULL, WARN, "Extended dir does not exist");
			return COMMON_NVM_STATUS_FAILURE;
		}
	} else {
		DbgMessage(NULL, WARN, "Extended dir does not exist");
		return COMMON_NVM_STATUS_FAILURE;
	}

	DbgMessage(NULL, INFORM, "%s image offset=0x%x length=0x%x\n", image_name, *offset, *length);
	return COMMON_NVM_STATUS_SUCCESS;
}


/*
 *------------------------------------------------------------------------
 * common_nvm_delete_vpd_image --
 *
 *     Delete VPD image from NVM
 *     Returns image length.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_delete_vpd_image() {
	u32_t idx, nvm_addr, byte_cnt, type, entry_idx, offset, length;
	extended_dir_image_t extended_dir;

	for (idx = 0; idx < common_nvm_image_table_size(); idx++) {
		if (strstr(image_table[idx].name, "VPD")) {
			break;
		}
	}

	/* Get the length of the VPD image from the extended dir */
	if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &byte_cnt, NULL, &type, NULL)
		== COMMON_NVM_STATUS_SUCCESS) {

		if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
			/* If extended dir image exist, parse the extended dir image into structure
			   parse it into extended_dir_image struct */
			if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, byte_cnt, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "Failed to parse extended dir image");
				return COMMON_NVM_STATUS_FAILURE;
			}

			/* Search for this image in extended dir */
			if (common_nvm_nvm_find_extended_dir_entry(image_table[idx].type, &extended_dir,
													   &entry_idx, &offset, NULL, &length, NULL) != COMMON_NVM_STATUS_SUCCESS) {
				DbgMessage(NULL, FATAL, "VPD image does not exist in extended dir");
				return COMMON_NVM_STATUS_FAILURE;
			}

		} else {
			DbgMessage(NULL, FATAL, "Extended dir does not exist");
			return COMMON_NVM_STATUS_FAILURE;
		}
	} else {
		DbgMessage(NULL, FATAL, "Extended dir does not exist");
		return COMMON_NVM_STATUS_FAILURE;
	}


	/* Delete the VPD extended dir entry */
	if (common_nvm_nvm_delete_extended_dir_entry(&extended_dir, entry_idx, nvm_addr) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Failed to delete VPD extended dir entry !! \n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_delete_ccm_image --
 *
 *     Delete CCM image from NVM if it exists.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_delete_ccm_image() {
	u32_t idx, nvm_addr, byte_cnt, type, entry_idx, offset, length;
	extended_dir_image_t extended_dir;

	for (idx = 0; idx < common_nvm_image_table_size(); idx++) {
		if (strstr(image_table[idx].name, "CCM")) {
			break;
		}
	}

	if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &byte_cnt, NULL, &type, NULL)
		== COMMON_NVM_STATUS_SUCCESS) {
		/* If extended dir image exist, parse it */
		if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, byte_cnt, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Failed to parse extended dir image");
			return COMMON_NVM_STATUS_FAILURE;
		}

		/* Search for CCM image in extended dir */
		if (common_nvm_nvm_find_extended_dir_entry(image_table[idx].type, &extended_dir,
												   &entry_idx, &offset, NULL, &length, NULL) != COMMON_NVM_STATUS_SUCCESS) {
			/* If CCM image doesn't exist, exit without doing anything */
			return COMMON_NVM_STATUS_SUCCESS;
		}

	} else {
		/* If extended dir doesn't exist, exit without doing anything */
		return COMMON_NVM_STATUS_SUCCESS;
	}


	/* Delete the CCM extended dir entry */
	if (common_nvm_nvm_delete_extended_dir_entry(&extended_dir, entry_idx, nvm_addr) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Failed to delete CCM extended dir entry !! \n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	return COMMON_NVM_STATUS_SUCCESS;
}


/*
 *------------------------------------------------------------------------
 * common_nvm_delete_niv_images --
 *
 *     Delete CCM image from NVM if it exists.
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_delete_niv_images() {
	u32_t idx, nvm_addr, byte_cnt, type, entry_idx, offset, length;
	extended_dir_image_t extended_dir;

	/* Check if extended dir exist */
	if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &byte_cnt, NULL, &type, NULL)
		== COMMON_NVM_STATUS_SUCCESS) {

		/* If extended dir image exist, parse it */
		if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, byte_cnt, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Failed to parse extended dir image");
			return COMMON_NVM_STATUS_FAILURE;
		}


	} else {
		/* If extended dir doesn't exist, exit without doing anything */
		return COMMON_NVM_STATUS_SUCCESS;
	}

	/* Search for NIV image in extended dir */
	for (idx = 0; idx < common_nvm_image_table_size(); idx++) {
		if (strstr(image_table[idx].name, "NIV")) {
			break;
		}
	}

	if (common_nvm_nvm_find_extended_dir_entry(image_table[idx].type, &extended_dir,
											   &entry_idx, &offset, NULL, &length, NULL) != COMMON_NVM_STATUS_SUCCESS) {
		/* If NIV image doesn't exist, exit without doing anything */
		return COMMON_NVM_STATUS_SUCCESS;
	}

	/* Delete the NIV extended dir entry */
	if (common_nvm_nvm_delete_extended_dir_entry(&extended_dir, entry_idx, nvm_addr) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Failed to delete NIV image !! \n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	/* Search for NIVPROFILES image in extended dir */
	for (idx = 0; idx < common_nvm_image_table_size(); idx++) {
		if (strstr(image_table[idx].name, "NIVPROFILES")) {
			break;
		}
	}

	/* Search for NIV image in extended dir */
	if (common_nvm_nvm_find_extended_dir_entry(image_table[idx].type, &extended_dir,
											   &entry_idx, &offset, NULL, &length, NULL) != COMMON_NVM_STATUS_SUCCESS) {
		/* If NIV image doesn't exist, exit without doing anything */
		return COMMON_NVM_STATUS_SUCCESS;
	}

	/* Delete the NIVPROFILES extended dir entry */
	if (common_nvm_nvm_delete_extended_dir_entry(&extended_dir, entry_idx, nvm_addr) != COMMON_NVM_STATUS_SUCCESS) {
		DbgMessage(NULL, FATAL, "Failed to delete NIVPROFILES image !! \n");
		return COMMON_NVM_STATUS_FAILURE;
	}

	return COMMON_NVM_STATUS_SUCCESS;
}
common_nvm_status_t common_nvm_fix_shmem_offset_for_older_versions(void) {
// 4/15/2011 by Jason:
// Johnny Ho decided that we don't implement the following for FwUpg tool.
// In that case, The MISC_REG_SHARED_MEM_ADDR is not manipulated with the
// new bootcode's Shared Memory Base.  A 'Cold Reboot' is required in order
// for the new bootcode, v5.2.5 or older, to update the MISC_REG_SHARED_MEM_ADDR
// correctly.  Johnny will put in a caveat on the release/CD.
#ifndef FWUPGTOOL
	common_nvm_status_t status;
	u32_t found_image, nvm_addr, byte_cnt, entry_idx, major, minor, build;
	char *ver_str;
	char ver_str_raw[64 + 3]; /* `+3' if for dword alignment */
	char tmp_str[64];

	/* Align ver_str to 32-bit */
	ver_str =(char *)ALIGN_UP4_PTR(ver_str_raw);

	/* search if there is MFW or BC */
	status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_MFW1, &nvm_addr, &byte_cnt, &entry_idx);

	/* Continue only if no MFW */
	if ((status != COMMON_NVM_STATUS_SUCCESS) || (!found_image)) {
		status |= common_nvm_read_nvram(0xc, &nvm_addr, sizeof(u32_t));

		if (status == COMMON_NVM_STATUS_SUCCESS) {
			nvm_addr = common_nvm_xlate_flash_virt_addr(nvm_addr);
			common_nvm_fw_ver_string(nvm_addr, image_table[0].name, ver_str, READ_VERSION_FROM_NVM);
			if (4 == sscanf(ver_str, "%s %d.%d.%d", tmp_str, &major, &minor, &build)) {
				/* check if version is below expected version */
				if ((major < BC_VERSION_OF_SHMEM_OFFSET_FIX_MAJOR) ||
					((major == BC_VERSION_OF_SHMEM_OFFSET_FIX_MAJOR) && (minor < BC_VERSION_OF_SHMEM_OFFSET_FIX_MINOR)) ||
					((major == BC_VERSION_OF_SHMEM_OFFSET_FIX_MAJOR) && (minor == BC_VERSION_OF_SHMEM_OFFSET_FIX_MINOR) && (build < BC_VERSION_OF_SHMEM_OFFSET_FIX_BUILD))) {
					/* fix shmem offset register to predefined value */
					common_nvm_reg_write(MISC_REG_SHARED_MEM_ADDR, MISC_REG_SHARED_MEM_OLD_VAL);
					common_nvm_reg_write(MISC_REG_GENERIC_CR_0, 0);
#ifndef EDEBUG
					dg_pci_scan();
					dg_setup_all();
#endif
				}
			}
		}
	}
#endif // !FWUPGTOOL

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_delete_bc_and_ncsi_images --
 *
 *     Delete BC1,2+NCSI images
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_delete_bc_and_ncsi_images() {
	u32_t entry_idx, tmp_nvm_addr;

	common_nvm_nvm_find_dir_entry(CODE_IMAGE_TYPE_BC2, &entry_idx, &tmp_nvm_addr);
	if (entry_idx < CODE_ENTRY_MAX) {
		printf("Deleting old BC image ..\n");
		common_nvm_update_bootstrap(0, 0, 0);
		common_nvm_update_dir_entry(entry_idx, NVM_ERASE_VALUE, NVM_ERASE_VALUE,
									NVM_ERASE_VALUE, NVM_ERASE_VALUE, NVM_ERASE_VALUE);
	}
	common_nvm_nvm_find_dir_entry(CODE_IMAGE_TYPE_NC_SI_CMN, &entry_idx, &tmp_nvm_addr);
	if (entry_idx < CODE_ENTRY_MAX) {
		printf("Deleting old NCSI image ..\n");
		common_nvm_update_dir_entry(entry_idx, NVM_ERASE_VALUE, NVM_ERASE_VALUE,
									NVM_ERASE_VALUE, NVM_ERASE_VALUE, NVM_ERASE_VALUE);
	}
	common_nvm_nvm_find_dir_entry(CODE_IMAGE_TYPE_NC_SI_EVEREST, &entry_idx, &tmp_nvm_addr);
	if (entry_idx < CODE_ENTRY_MAX) {
		common_nvm_update_dir_entry(entry_idx, NVM_ERASE_VALUE, NVM_ERASE_VALUE,
									NVM_ERASE_VALUE, NVM_ERASE_VALUE, NVM_ERASE_VALUE);
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_delete_mfw_and_swim_images --
 *
 *     Delete MFW1,2+SWIM images
 *     if delete_mfw==0 don't delete MFWs
 *     if delete_mfw==1 delete all MFWs
 *     if delete_mfw==2 delete all MFWs beside MFW1
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_delete_mfw_and_swim_images(enum delete_mfw_cmd cmd, u32_t delete_swims) {
	extended_dir_image_t extended_dir;
	u32_t i, entry_idx=0, byte_cnt=0, found_image=0, offset=0, extended_dir_addr=0, cnt=0, type=0;
	u32_t mfw_code_images[]={ CODE_IMAGE_TYPE_MFW1, CODE_IMAGE_TYPE_MFW2, CODE_IMAGE_TYPE_MFW2_A };
	u32_t swim_code_images[]={ CODE_IMAGE_TYPE_SWIM1, CODE_IMAGE_TYPE_SWIM2, CODE_IMAGE_TYPE_SWIM3, CODE_IMAGE_TYPE_SWIM4,
		CODE_IMAGE_TYPE_SWIM5, CODE_IMAGE_TYPE_SWIM6, CODE_IMAGE_TYPE_SWIM7, CODE_IMAGE_TYPE_SWIM8,
		CODE_IMAGE_TYPE_SWIM1_B, CODE_IMAGE_TYPE_SWIM2_B, CODE_IMAGE_TYPE_SWIM3_B, CODE_IMAGE_TYPE_SWIM4_B,
		CODE_IMAGE_TYPE_SWIM5_B, CODE_IMAGE_TYPE_SWIM6_B, CODE_IMAGE_TYPE_SWIM7_B, CODE_IMAGE_TYPE_SWIM8_B };

	// Check if extended DIR exists
	if ((common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &extended_dir_addr, &byte_cnt, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS) &&
		(type == CODE_IMAGE_TYPE_EXTENDED_DIR)) {
		if (common_nvm_nvm_parse_extended_dir_image(extended_dir_addr, byte_cnt, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
			DbgMessage(NULL, FATAL, "Failed to parse extended dir image !! \n");
			return COMMON_NVM_STATUS_FAILURE;
		}
		if (cmd != DELETE_MFW_CMD_DONT_DELETE) {
			/* delete MFW1 only if cmd==DELETE_MFW_CMD_DELETE_ALL_MFWS */
			i = 0;
			if (cmd == DELETE_MFW_CMD_DELETE_ALL_MFWS) {
				i = 0;
			} else if (cmd == DELETE_MFW_CMD_ALL_MFWS_BESIDES_MFW1) {
				i = 1;
			}
			for (; i < (sizeof(mfw_code_images) / sizeof(u32_t)); i++) {
				common_nvm_nvm_find_image_in_extended_dir(&found_image, mfw_code_images[i], &offset, &cnt, &entry_idx);
				if (found_image) {
					common_nvm_nvm_delete_extended_dir_entry(&extended_dir, entry_idx, extended_dir_addr);
				}
			}
		}

		/* delete SWIMs if needed */
		if (delete_swims) {
			for (i = 0; i < (sizeof(swim_code_images) / sizeof(u32_t)); i++) {
				common_nvm_nvm_find_image_in_extended_dir(&found_image, swim_code_images[i], &offset, &cnt, &entry_idx);
				if (found_image) {
					common_nvm_nvm_delete_extended_dir_entry(&extended_dir, entry_idx, extended_dir_addr);
				}
			}
		}
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_delete_unused_swim_images --
 *
 *     Delete unused SWIM images (SWIMs not currently running)
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_delete_unused_swim_images(u32_t * used_group) {
	extended_dir_image_t extended_dir;
	u32_t i, result, entry_idx=0, byte_cnt=0, found_image=0, offset=0, extended_dir_addr=0, cnt=0, type=0, bc_ver, shmem_base;
	u32_t image_major=0, image_minor=0, image_build=0, image_drop=0;
	char ver_str[64];
	u32_t swim_code_images[]={ CODE_IMAGE_TYPE_SWIM1, CODE_IMAGE_TYPE_SWIM2, CODE_IMAGE_TYPE_SWIM3, CODE_IMAGE_TYPE_SWIM4,
		CODE_IMAGE_TYPE_SWIM5, CODE_IMAGE_TYPE_SWIM6, CODE_IMAGE_TYPE_SWIM7, CODE_IMAGE_TYPE_SWIM8,
		CODE_IMAGE_TYPE_SWIM1_B, CODE_IMAGE_TYPE_SWIM2_B, CODE_IMAGE_TYPE_SWIM3_B, CODE_IMAGE_TYPE_SWIM4_B,
		CODE_IMAGE_TYPE_SWIM5_B, CODE_IMAGE_TYPE_SWIM6_B, CODE_IMAGE_TYPE_SWIM7_B, CODE_IMAGE_TYPE_SWIM8_B };
	char swim_image_scanf_pattern[][64]={ "SWIM1 %d.%d.%d", "SWIM2 %d.%d.%d", "SWIM3 %d.%d.%d", "SWIM4 %d.%d.%d",
		"SWIM5 %d.%d.%d", "SWIM6 %d.%d.%d", "SWIM7 %d.%d.%d", "SWIM8 %d.%d.%d",
		"SWIM1 %d.%d.%d", "SWIM2 %d.%d.%d", "SWIM3 %d.%d.%d", "SWIM4 %d.%d.%d",
		"SWIM5 %d.%d.%d", "SWIM6 %d.%d.%d", "SWIM7 %d.%d.%d", "SWIM8 %d.%d.%d" };
	char swim_image_extended_pattern[][64]={ "SWIM1 %d.%d.%d.%d", "SWIM2 %d.%d.%d.%d", "SWIM3 %d.%d.%d.%d", "SWIM4 %d.%d.%d.%d",
		"SWIM5 %d.%d.%d.%d", "SWIM6 %d.%d.%d.%d", "SWIM7 %d.%d.%d.%d", "SWIM8 %d.%d.%d.%d",
		"SWIM1 %d.%d.%d.%d", "SWIM2 %d.%d.%d.%d", "SWIM3 %d.%d.%d.%d", "SWIM4 %d.%d.%d.%d",
		"SWIM5 %d.%d.%d.%d", "SWIM6 %d.%d.%d.%d", "SWIM7 %d.%d.%d.%d", "SWIM8 %d.%d.%d.%d" };
	char swim_images_name[][64]={ NVM_IMAGE_NAME_SWIM1, NVM_IMAGE_NAME_SWIM2, NVM_IMAGE_NAME_SWIM3, NVM_IMAGE_NAME_SWIM4,
		NVM_IMAGE_NAME_SWIM5, NVM_IMAGE_NAME_SWIM6, NVM_IMAGE_NAME_SWIM7, NVM_IMAGE_NAME_SWIM8,
		NVM_IMAGE_NAME_SWIM1_B, NVM_IMAGE_NAME_SWIM2_B, NVM_IMAGE_NAME_SWIM3_B, NVM_IMAGE_NAME_SWIM4_B,
		NVM_IMAGE_NAME_SWIM5_B, NVM_IMAGE_NAME_SWIM6_B, NVM_IMAGE_NAME_SWIM7_B, NVM_IMAGE_NAME_SWIM8_B };

#ifdef FWUPGTOOL
//    NVM_INTERFACE iNvm = nvm_interface( gpINx2Nvm->m_tcl_interp );
//    u32_t dev_index = iNvm.m_current_dev_index;
	u32_t dev_index = gpINx2Nvm->m_current_dev_index;
#endif

	*used_group = 0;

	if (common_nvm_tcl_al_get_virtual_nvm_mode()) {
		char old_version[64];
		u32 nvm_addr, extended_dir_size_in_nvm, type, code_start;
		extended_dir_image_t extended_dir_in_nvm;

		// Find an existing matching entry for this image in NVM EXTENDED DIR
		if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &extended_dir_size_in_nvm, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS) {                    // Parse the extended DIR
			if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
				if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, extended_dir_size_in_nvm, &extended_dir_in_nvm) == COMMON_NVM_STATUS_SUCCESS) {
					if (common_nvm_nvm_find_extended_dir_entry(CODE_IMAGE_TYPE_MFW1, &extended_dir_in_nvm,
															   NULL, &code_start, NULL, NULL, NULL) == COMMON_NVM_STATUS_SUCCESS) {
						common_nvm_fw_ver_string(code_start, NVM_IMAGE_NAME_MFW1, old_version, READ_VERSION_FROM_NVM);
						sscanf(old_version, "MFW1 %d.%d.%d", &running_bc_major, &running_bc_minor, &running_bc_build);
					}
				}
			}
		}
	} else {
		// read current running version from shmem
#ifndef FWUPGTOOL
		common_nvm_reg_read(MISC_REG_SHARED_MEM_ADDR, &shmem_base);
#else
		//    iNvm.m_td_read_reg32( dev_index, MISC_REG_SHARED_MEM_ADDR, &shmem_base );
		gpINx2Nvm->m_td_read_reg32(dev_index, MISC_REG_SHARED_MEM_ADDR, &shmem_base);
#endif
		if (shmem_base == 0) {
			printf("No shared memory address");
			return COMMON_NVM_STATUS_FAILURE;
		}

#ifndef FWUPGTOOL
		common_nvm_reg_read(shmem_base + OFFSETOF(shmem_region_t, dev_info.bc_rev), &bc_ver);
#else
		//    iNvm.m_td_read_reg32( dev_index, ( shmem_base + OFFSETOF(shmem_region_t, dev_info.bc_rev)), &bc_ver );
		gpINx2Nvm->m_td_read_reg32(dev_index, (shmem_base + OFFSETOF(shmem_region_t, dev_info.bc_rev)), &bc_ver);
#endif
		running_bc_major = (bc_ver & 0xFF000000) >> 24;
		running_bc_minor = (bc_ver & 0x00FF0000) >> 16;
		running_bc_build = (bc_ver & 0x0000FF00) >> 8;
		running_bc_drop = (bc_ver & 0x000000FF);
	}

	// Check if extended DIR exists
	if ((common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &extended_dir_addr, &byte_cnt, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS) &&
		(type == CODE_IMAGE_TYPE_EXTENDED_DIR)) {
		if (common_nvm_nvm_parse_extended_dir_image(extended_dir_addr, byte_cnt, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
			return COMMON_NVM_STATUS_SUCCESS;
		}

		for (i = 0; i < (sizeof(swim_code_images) / sizeof(u32_t)); i++) {
			common_nvm_nvm_find_image_in_extended_dir(&found_image, swim_code_images[i], &offset, &cnt, &entry_idx);
			if (found_image) {
				common_nvm_fw_ver_string(offset, swim_images_name[i], ver_str, READ_VERSION_FROM_NVM);
				result = sscanf(ver_str, swim_image_scanf_pattern[i], &image_major, &image_minor, &image_build);
				if (result != 3) {
					result = sscanf(ver_str, swim_image_extended_pattern[i], &image_major, &image_minor, &image_build, &image_drop);
				}
				if ((image_major != running_bc_major) || (image_minor != running_bc_minor) ||
					(image_build != running_bc_build) || (image_drop != running_bc_drop)) {
					common_nvm_nvm_delete_extended_dir_entry(&extended_dir, entry_idx, extended_dir_addr);
				} else if (swim_code_images[i] == CODE_IMAGE_TYPE_SWIM1) {
					// found matching SWIM1 image, mark group A is used
					*used_group = 1;
				} else if (swim_code_images[i] == CODE_IMAGE_TYPE_SWIM1_B) {
					// found matching SWIM1_B image, mark group B is used
					*used_group = 2;
				}
			}
		}
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_delete_swim_group_b_images --
 *
 *     Delete all SWIMs of group B
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_delete_swim_group_b_images() {
	extended_dir_image_t extended_dir;
	u32_t i, entry_idx=0, byte_cnt=0, found_image=0, offset=0, extended_dir_addr=0, cnt=0, type=0;
	u32_t swim_group_b_code_images[]={ CODE_IMAGE_TYPE_SWIM1_B, CODE_IMAGE_TYPE_SWIM2_B, CODE_IMAGE_TYPE_SWIM3_B, CODE_IMAGE_TYPE_SWIM4_B,
		CODE_IMAGE_TYPE_SWIM5_B, CODE_IMAGE_TYPE_SWIM6_B, CODE_IMAGE_TYPE_SWIM7_B, CODE_IMAGE_TYPE_SWIM8_B };

	// Check if extended DIR exists
	if ((common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &extended_dir_addr, &byte_cnt, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS) &&
		(type == CODE_IMAGE_TYPE_EXTENDED_DIR)) {
		if (common_nvm_nvm_parse_extended_dir_image(extended_dir_addr, byte_cnt, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
			return COMMON_NVM_STATUS_SUCCESS;
		}

		for (i = 0; i < (sizeof(swim_group_b_code_images) / sizeof(u32_t)); i++) {
			common_nvm_nvm_find_image_in_extended_dir(&found_image, swim_group_b_code_images[i], &offset, &cnt, &entry_idx);
			if (found_image) {
				common_nvm_nvm_delete_extended_dir_entry(&extended_dir, entry_idx, extended_dir_addr);
			}
		}
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_read_mfw_version --
 *
 *     Read MFW version
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_read_mfw_version(char *mfw_version) {
	common_nvm_status_t status;
	u32_t found_image, nvm_addr, byte_cnt, entry_idx;
	char tmp_str[MAX_VER_STR_SIZE];

	*mfw_version = '\0';

	status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_MFW1, &nvm_addr, &byte_cnt, &entry_idx);

	if ((status == COMMON_NVM_STATUS_SUCCESS) && found_image) {

		common_nvm_fw_ver_string((common_nvm_virt_addr_t)nvm_addr, NVM_IMAGE_NAME_MFW1, &tmp_str[0], READ_VERSION_FROM_NVM);

		sscanf(&tmp_str[0], "MFW1 %s", mfw_version);

	} else {
		return COMMON_NVM_STATUS_FAILURE;
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_read_mba_version --
 *
 *     Read MBA version
 *
 *------------------------------------------------------------------------
 */
common_nvm_status_t common_nvm_read_mba_efi_versions(char *mba_version, char *efi_version) {
	common_nvm_status_t status;
	u32_t nvm_addr, entry_idx, i;

	u32_t nvm_vers[ROMIMG_NUM_MAX];
	char  *type_str[ROMIMG_NUM_MAX]={ 0 };

	*mba_version = '\0';

	status = common_nvm_nvm_find_dir_entry(CODE_IMAGE_TYPE_MBA, &entry_idx, &nvm_addr);

	if ((status == COMMON_NVM_STATUS_SUCCESS) && (entry_idx != CODE_ENTRY_MAX)) {

		// Get the combined mba images versions in NVM, output in nvm_vers
		common_nvm_get_romimg_vers(READ_VERSION_FROM_NVM, nvm_addr, nvm_vers, type_str);

		for (i = 0; i <= MBA_MBA_PCI3_IDX; i++) {
			if (type_str[i] != '\0') {
				sprintf(mba_version, "%d.%d.%d", (nvm_vers[i] & 0xff000) >> 12, (nvm_vers[i] & 0x0f00) >> 8, nvm_vers[i] & 0xff);
			}
		}

		for (i = EFI_X86_IDX; i <= EFI_X64_IDX; i++) {
			if (type_str[i] != '\0') {
				sprintf(efi_version, "%d.%d.%d", (nvm_vers[i] & 0xff000) >> 12, (nvm_vers[i] & 0x0f00) >> 8, nvm_vers[i] & 0xff);
			}
		}

	} else {
		return COMMON_NVM_STATUS_FAILURE;
	}

	return COMMON_NVM_STATUS_SUCCESS;
}

#ifdef FWUPGTOOL
u32_t
nvm_find_extended_dir_entry(
							NVM_INTERFACE iNvm,
							u32_t image_type,
							u32_t *entry_idx
							) {
	u32_t found_image, nvm_addr, byte_cnt;

	if ((common_nvm_nvm_find_image_in_extended_dir(&found_image, image_type, &nvm_addr, &byte_cnt, entry_idx) != COMMON_NVM_STATUS_SUCCESS) ||
		(!found_image)) {
		*entry_idx = -1;
		return -1;
	}

	return 0;
}

int
dg_find_image_type(NVM_INTERFACE iNvm, char *opt_str, U32 file_image_type, U32 *pType) {
	U32 idx;

	if ((opt_str != NULL) && (file_image_type == 0)) {
		// Use opt_str to find the image_type...
		for (idx = 0; idx < sizeof(image_table) / sizeof(image_map_t); idx++) {
			if (strcmp(opt_str, image_table[idx].opt) == 0) {
				/* Found a match */
				*pType = image_table[idx].type;
				return 1;
			}
		}
	} else if ((opt_str == NULL) && (file_image_type != 0)) {
		// Use file_image_type to find the image_type...
		for (idx = 0; idx < sizeof(image_table) / sizeof(image_map_t); idx++) {
//            printf( "dg_find_image_type: idx = %d, file_image_type = 0x%x, hdr_type = 0x%x\n",
//                idx, file_image_type, image_table[idx].hdr_type );
			if (file_image_type == image_table[idx].hdr_type) {
				*pType = image_table[idx].type;
				return 1;
			}
		}
	}

	return 0;
}


int
isDirEntryThereInFile(char *mpFwFileData, U32 uEntry) {
	dir_t *pDir =(dir_t *)&( mpFwFileData[NVM_OFFSET(dir)]);
	code_entry_t *pEntry;
	U32 image_type, code_start;
	int i;
	U32 idx;
	extended_dir_image_t *extended_dir_p;

	for (i = 0; i < CODE_ENTRY_MAX; i++) {
		pEntry = &(pDir->code[i]);
		image_type = pEntry->code_attribute & CODE_IMAGE_TYPE_MASK;
		if (image_type == uEntry) return 1;

		if (image_type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
			// Extended DIR exists in mpFwFileData
			code_start = pEntry->nvm_start_addr;
			extended_dir_p =(extended_dir_image_t *)&mpFwFileData[code_start];
			for (idx = 0; idx < extended_dir_p->no_images; idx++) {
				image_type = extended_dir_p->extended_dir_images[idx].code_attribute & CODE_IMAGE_TYPE_MASK;
				if (image_type == uEntry) return 1;
			}
		}
	}

	return 0;
}


int
dg_doMBAMatch(NVM_INTERFACE iNvm, char *mpFwFileData) {
	int bMBAInDir = 0;
	U32 idx =(U32)-1;

	bMBAInDir = ( common_nvm_nvm_find_dir_entry(CODE_IMAGE_TYPE_MBA, &idx, NULL)== COMMON_NVM_STATUS_SUCCESS );
	return (bMBAInDir == isDirEntryThereInFile(mpFwFileData, CODE_IMAGE_TYPE_MBA));
}


int
dg_doNCSIMatch(NVM_INTERFACE iNvm, char *mpFwFileData) {
	int bNCSIInDir = 0;
	U32 idx =(U32)-1;

	bNCSIInDir = ( common_nvm_nvm_find_dir_entry(CODE_IMAGE_TYPE_NC_SI_CMN, &idx, NULL)== COMMON_NVM_STATUS_SUCCESS );
	return (bNCSIInDir == isDirEntryThereInFile(mpFwFileData, CODE_IMAGE_TYPE_NC_SI_CMN));
}

int
dg_doFCOEMatch(NVM_INTERFACE iNvm, char *mpFwFileData) {
	int bFCOEInDir = 0;
	int bFCOEInFile = 0;
	U32 idx =(U32)-1;
	U32 found_image, nvm_addr, byte_cnt;

	found_image = 0;
	bFCOEInDir = 0;
	if (common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_FCOE_BOOT,
												  &nvm_addr, &byte_cnt, &idx) ==  COMMON_NVM_STATUS_SUCCESS) {
		if (found_image) bFCOEInDir = 1;
	}

	bFCOEInFile = isDirEntryThereInFile(mpFwFileData, CODE_IMAGE_TYPE_FCOE_BOOT);

	if (bFCOEInDir != bFCOEInFile) return 0;

	found_image = 0;
	bFCOEInDir = 0;
	if (common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_FCOE_BOOT_CFG,
												  &nvm_addr, &byte_cnt, &idx) ==  COMMON_NVM_STATUS_SUCCESS) {
		if (found_image) bFCOEInDir = 1;
	}

	bFCOEInFile = isDirEntryThereInFile(mpFwFileData, CODE_IMAGE_TYPE_FCOE_BOOT_CFG);

	if (bFCOEInDir != bFCOEInFile) return 0;

	found_image = 0;
	bFCOEInDir = 0;
	if (common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_FCOE_BOOT_CFG2,
												  &nvm_addr, &byte_cnt, &idx) ==  COMMON_NVM_STATUS_SUCCESS) {
		if (found_image) bFCOEInDir = 1;
	}

	bFCOEInFile = isDirEntryThereInFile(mpFwFileData, CODE_IMAGE_TYPE_FCOE_BOOT_CFG2);

	if (bFCOEInDir != bFCOEInFile) return 0;

	found_image = 0;
	bFCOEInDir = 0;
	if (common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_FCOE_BOOT_CFG3,
												  &nvm_addr, &byte_cnt, &idx) == COMMON_NVM_STATUS_SUCCESS) {
		if (found_image) bFCOEInDir = 1;
	}

	bFCOEInFile = isDirEntryThereInFile(mpFwFileData, CODE_IMAGE_TYPE_FCOE_BOOT_CFG3);

	if (bFCOEInDir != bFCOEInFile) return 0;

	found_image = 0;
	bFCOEInDir = 0;
	if (common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_FCOE_BOOT_CFG4,
												  &nvm_addr, &byte_cnt, &idx) == COMMON_NVM_STATUS_SUCCESS) {
		if (found_image) bFCOEInDir = 1;
	}

	bFCOEInFile = isDirEntryThereInFile(mpFwFileData, CODE_IMAGE_TYPE_FCOE_BOOT_CFG4);

	if (bFCOEInDir != bFCOEInFile) return 0;

#if 0
	// FCOE_BOOT_CPRG_EVRST is not in FCoE boot image file and will never be in NVRAM.
	// CCE is the tool to configure the FCoE configuration now.
	bFCOEInDir = ( common_nvm_nvm_find_dir_entry( CODE_IMAGE_TYPE_FCOE_BOOT_CPRG_EVRST, &idx, NULL ) == COMMON_NVM_STATUS_SUCCESS );
	bFCOEInFile = isDirEntryThereInFile( mpFwFileData, CODE_IMAGE_TYPE_FCOE_BOOT_CPRG_EVRST );

	if ( bFCOEInDir != bFCOEInFile )
	return 0;
#endif

	return 1;
}

//
// id: to identify which item in the NVRAM to retrieve
// pOffset: location for returned offset
//
int
get_57710_NVM_data(NVM_INTERFACE iNvm, U32 id, char *pBuf, U32 *pData) {
	U32 uOffset;
	U32 func_num, port_num, path_num, port_n;
	common_nvm_status_t stat;

	*pData = 0; // default value returned if error...

	func_num = COMMON_NVM_FUNC_NUM();
	stat = common_nvm_get_port_num(&port_n);
	if (stat != 0) return -1;

	port_num =(u8_t)port_n;
	path_num = COMMON_NVM_PATH_NUM();

	switch (id) {
	case NVM_57710_MANUF_PORTCFG_PCIID:
		uOffset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port_num].pci_id, path_num);
//            printf( "get_57710_NVM_data: pci_id offset = 0x%x\n", uOffset );
		break;
	case NVM_57710_MANUF_PORTCFG_PCISUBID:
		uOffset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port_num].pci_sub_id, path_num);
//            printf( "get_57710_NVM_data: pci_sub_id offset = 0x%x\n", uOffset );
		break;
	case NVM_57710_MANUF_PORTCFG_EXTPHYCONFIG:
		uOffset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port_num].external_phy_config, path_num);
//            printf( "get_57710_NVM_data: external_phy_config offset = 0x%x\n", uOffset );
		break;
	case NVM_57710_MANUF_PORTCFG_MF_PCIID:
		uOffset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port_num].mf_pci_id, path_num);
		break;
	case NVM_57710_FEAT_SHARE_CONFIG:
		uOffset = NVM_OFFSET_PATH(feature_info.shared_feature_config, path_num);
		break;
	default:
		return -1; // failed, unknown id
	}

	if (pBuf) {
		// retrieve data from provided buffer...
		*pData = *((U32 *)&( pBuf[uOffset]));
	} else {
		// retrieve data from NVRAM...
		if (common_nvm_read_nvram(uOffset, pData, sizeof(u32_t)) != COMMON_NVM_STATUS_SUCCESS) return -1;
	}

	switch (id) {
	case NVM_57710_MANUF_PORTCFG_EXTPHYCONFIG:
		*pData &= PORT_HW_CFG_XGXS_EXT_PHY_TYPE_MASK;
		if (*pData == PORT_HW_CFG_XGXS_EXT_PHY_TYPE_BCM8727_NOC) *pData = PORT_HW_CFG_XGXS_EXT_PHY_TYPE_BCM8727;
		break;
	default:
		break; // do nothing...
	}

	return 0; // successful
}

//
// id: to identify which item in the NVRAM to retrieve
// pOffset: location for returned offset
//
int
get_57710_NVM_offset(NVM_INTERFACE iNvm, U32 id, U32 *pOffset) {
	U32 func_num, port_num, path_num, port_n;
	common_nvm_status_t stat;

	*pOffset = 0; // default value returned if error...

	func_num = COMMON_NVM_FUNC_NUM();
	stat = common_nvm_get_port_num(&port_n);
	if (stat != 0) return -1; // error

	port_num =(u8_t)port_n;
	path_num = COMMON_NVM_PATH_NUM();

	switch (id) {
	case NVM_57710_MANUF_PORTCFG_PCIID:
		*pOffset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port_num].pci_id, path_num);
//            printf( "get_57710_NVM_data: pci_id offset = 0x%x\n", *pOffset );
		break;
	case NVM_57710_MANUF_PORTCFG_PCISUBID:
		*pOffset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port_num].pci_sub_id, path_num);
//            printf( "get_57710_NVM_data: pci_sub_id offset = 0x%x\n", *pOffset );
		break;
	case NVM_57710_MANUF_PORTCFG_EXTPHYCONFIG:
		*pOffset = NVM_OFFSET_PATH(manuf_info.port_hw_config[port_num].external_phy_config, path_num);
//            printf( "get_57710_NVM_data: external_phy_config offset = 0x%x\n", *pOffset );
		break;
	default:
		return -1; // failed, unknown id
	}

	return 0; // successful
}

#define MISC_REGISTERS_SHARED_MEM_ADDR   0xa2b4
#define SHMEM_OFFSET(f)     ((u32_t)((int_ptr_t)(&(((shmem_region_t *)0)->f))))
int
get_57710_ext_phy_type(NVM_INTERFACE iNvm, U32 port_id, U32 *pValue) {
	U32             shmem_base, ext_phy_type;
	U32             uOffset = 0;
	U32             dev_index = iNvm.m_current_dev_index;

	*pValue = 0; // default value returned if error...

//    printf( "get_57710_ext_phy_type: dev_index = 0x%x, port_id = 0x%x\n", dev_index, port_id );

	// read External PHY type from Shared Memory using BMAPI API...
	if (iNvm.m_td_read_reg32(dev_index, MISC_REGISTERS_SHARED_MEM_ADDR, &shmem_base) != TCL_OK) {
		return -1; // Failed to read external PHY type
	}

	uOffset = SHMEM_OFFSET(dev_info.port_hw_config[port_id].external_phy_config);
	shmem_base += uOffset;

	// read External PHY type from Shared Memory using BMAPI API...
	if (iNvm.m_td_read_reg32(dev_index, shmem_base, &ext_phy_type) != TCL_OK) {
		// ext_phy_type will be set to 0 by default.
		printf("Failed to read external PHY type.\n");
		return -1; // Failed to read external PHY type
	}

	ext_phy_type &= PORT_HW_CFG_XGXS_EXT_PHY_TYPE_MASK;
	if (ext_phy_type == PORT_HW_CFG_XGXS_EXT_PHY_TYPE_BCM8727_NOC) ext_phy_type = PORT_HW_CFG_XGXS_EXT_PHY_TYPE_BCM8727;

	*pValue = ext_phy_type;
	return 0; // successful
}


int
check_57710_ext_phy_type(NVM_INTERFACE iNvm, U32 port_id, U32 ext_phy_type_to_check) {
	U32             shmem_base, ext_phy_type;
	U32             uOffset = 0;
	U32             dev_index = iNvm.m_current_dev_index;

	// read External PHY type from Shared Memory using BMAPI API...
	if (iNvm.m_td_read_reg32(dev_index, MISC_REGISTERS_SHARED_MEM_ADDR, &shmem_base) != TCL_OK) {
		return 0; // Failed to read external PHY type, no match
	}

	uOffset = SHMEM_OFFSET(dev_info.port_hw_config[port_id].external_phy_config);

	// read External PHY type from Shared Memory using BMAPI API...
	if (iNvm.m_td_read_reg32(dev_index, (shmem_base + uOffset), &ext_phy_type) != TCL_OK) {
		// ext_phy_type will be set to 0 by default.
		printf("Failed to read external PHY type.\n");
		return 0; // Failed to read external PHY type, no match
	}

	ext_phy_type &= PORT_HW_CFG_XGXS_EXT_PHY_TYPE_MASK;
	if (ext_phy_type == PORT_HW_CFG_XGXS_EXT_PHY_TYPE_BCM8727_NOC) ext_phy_type = PORT_HW_CFG_XGXS_EXT_PHY_TYPE_BCM8727;

	if (ext_phy_type == ext_phy_type_to_check) return 1;

	uOffset = SHMEM_OFFSET(dev_info.port_hw_config[port_id].external_phy_config2);

	// read External PHY2 type from Shared Memory using BMAPI API...
	if (iNvm.m_td_read_reg32(dev_index, (shmem_base + uOffset), &ext_phy_type) != TCL_OK) {
		// ext_phy_type will be set to 0 by default.
		printf("Failed to read external PHY2 type.\n");
		return 0; // Failed to read external PHY type, no match
	}

	ext_phy_type &= PORT_HW_CFG_XGXS_EXT_PHY_TYPE_MASK;
	if (ext_phy_type == PORT_HW_CFG_XGXS_EXT_PHY_TYPE_BCM8727_NOC) ext_phy_type = PORT_HW_CFG_XGXS_EXT_PHY_TYPE_BCM8727;

	if (ext_phy_type == ext_phy_type_to_check) return 1;

	return 0; // no match
}

// This function is modified from dg_number_of_everest_functions() in dg_device.c
// under ...\drivers\Diag\common\diag_manager directory

/*
*------------------------------------------------------------------------
* dg_number_of_everest_functions --
*
* The following logic is based on dg_number_of_everest_functions(), rev#68,
* in dg_device.c under .../servers/main/577xx/drivers/common/diag/diag_manager
* in Perforce.
*
* Result:
*     number of ports (1 or 2 or 4)
*
* please note:
* ***********
*  this function assumes that all everest functions in this machine of the same device, are following device ids.
*  therefore,  the second everest function should be 1 following or 1 previous to the current device.
*  if this assumption is not current,  than also function dg_select_other_func should be corrected.
*------------------------------------------------------------------------
*/
u32_t dg_number_of_everest_functions(void) {
	u32_t   port_cnt;
	int     is_4port=0;

	/* Added check for E3 - 4 port mode. */
	if (common_nvm_is_4port(&is_4port) == COMMON_NVM_STATUS_SUCCESS) if (is_4port) return 4;

	// The number of everest functions is calculated based on the same algorithm here.
	// It is implemented in Bnx2Helper::SetSelectedNIC() in Bnx2Helper.cpp and
	// the ENV_PORT_CNT_57710 is set within the function.
	port_cnt = atoi(Tcl_GetVar2(gpINx2Nvm->m_tcl_interp, TCL_ENV, ENV_PORT_CNT, 0));
	return port_cnt;
}

/*
	This function is copied from dg_os_mem.c since the module
	is not included in makeing the FWUPG.
*/

void dg_os_swap_bytes(
					  u32_t  * buf,
					  u32_t  buf_size
					  ) {
	u32_t i;

	for (i = 0; i < buf_size / 4; i++) {
		buf[i]= htonl(buf[i]);
	}

	return;
}

#endif // FWUPGTOOL
