/*************************************************************************
 * Copyright(c) 2011-2014 Broadcom Corporation, all rights reserved.
 * Copyright(c) 2014-2015 QLogic Corporation, all rights reserved.
 * Contains proprietary and confidential information.
 *
 * This source file is the property of QLogic Corporation, and
 * may not be copied or distributed in any isomorphic form without
 * the prior written consent of QLogic Corporation.
 *
 *
 *  Name: NVM.C
 *
 *  Description:
 *
 ************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdint.h>
#include "tcl.h"
#include "tclDecls.h"
#include "tcl_al_common.h"
#include "tcl_al.h"
#include "disputil.h"
#include "tcl_fmodules.h"
#include "tcl_vpd.h"
#include "tcl_io.h"
#include "tcl_hw_set.h"
#include "tcl_util.h"
#include "pcics_reg_driver.h"

#ifndef FWUPGTOOL
#include "hw_access.h"
#else
// from ...\servers\main\nx2\577xx\drivers\Diag\common\diag_device\include\hw_access.h
#define NVRAM_1MB_SIZE              0x20000  // 1M bit in bytes
#define NVRAM_PAGE_SIZE             256
#endif

#ifndef FWUPGTOOL
#include "util.h"
#include "esc_seq.h"
#include "tcl_serport.h"
#else
#include "../57710h/util.h"
#endif // !FWUPGTOOL

#ifndef FWUPGTOOL
#include "bcmtype.h"
#endif // !FWUPGTOOL

#ifdef EDEBUG
#include "edebug_types.h"
#include "man_types.h"
#include "license.h"
#include "nvm_map.h"
#include "invm_cfg.h"
#include "dbg_exprom.h"
#include "dbg_man.h"
#include "util.h"
#include "dbg_os_utils.h"
#include "57712_reg.h"
#else
#include "dg_ipmi.h"
#include "dg_exprom.h"
#include "dg_hmem_def.h"
#include "dg_mcp.h"
#ifndef FWUPGTOOL
#include "append.h"
#else
#include "../57710h/append.h"
#endif // !FWUPGTOOL
#include "dg_vpd.h"
#include "tcl_util.h"
#include "tcl_hostmem.h"
#include "invm_cfg.h"
#include "net_util.h"
#ifdef FWUPGTOOL
#ifndef LINUX
#include <Winsock2.h>
#else
#include "linux/BMAPI.h"
#endif // !LINUX
#define __USER_MODE_DEBUG // for debug.h
#endif // FWUPGTOOL
#include "debug.h"
#ifndef FWUPGTOOL
#include "dg_os_util.h"
#endif // !FWUPGTOOL
#ifdef UEFI
#include <sys/stat.h>
#include <efi.h>
#include <efiapi.h>
#include <efilib.h>
#endif // UEFI
#endif // EDEBUG

#include "common_nvm.h"

#include "id.h"



#ifdef FWUPGTOOL
// from ...\nx2\577xx\hsi\hw\include\57712_reg.h in Perforce...
#define MISC_REG_GENERIC_POR_0  0xa470UL
#define OFFSETOF(_s, _m)    ((u32_t) PTR_SUB(&((_s *) 0)->_m, (u8_t *) 0))
#endif // FWUPGTOOL



#ifdef FWUPGTOOL
#include "nvm_hlp.h"
#endif // FWUPGTOOL

#ifdef WIN_DIAG
#define Tcl_SetIntObjDispType(tmp_obj, disp_type) Tcl_SetObjDisp2Hex(tmp_obj,#disp_type)
#endif //WIN_DIAG

#define CFG_DISP_BREAK_CHAR  'q'  /* A character to be pressed to break "nvm cfg" */
#define NPAR1_5_SUPPORT_VER   "7.a.0.0"
#define AFEX_REMOVED_VER     "7.12.0.0"

#include "mono_image.h"
#include "checksum.h"
#ifndef EDEBUG
#include "dg_ext_phy_fw.h"
#endif // EDEBUG

/* global vars */
#ifndef FWUPGTOOL
static u32_t old_offset = 0;
static u32_t old_length = 4;
u32_t g_no_ctrl_keys = FALSE;
#else
extern NVM_INTERFACE *gpINx2Nvm;
#endif // !FWUPGTOOL

//global definitions for SWIM shadow feature
u32_t new_mfw_major = 0, new_mfw_minor = 0, new_mfw_build = 0, new_mfw_drop = 0;
u32_t running_bc_major = 0, running_bc_minor = 0, running_bc_build = 0, running_bc_drop = 0;

/* externals */
#ifdef EDEBUG
extern u32_t uart_com;
extern u32_t uart_baud;
#endif

static char *exec_env_name[] = {
	/* Make sure to include all CODE_IMAGE_AGENT_* from md_rgn.h. */
	"NONE",
	"HOST",
	"MCP",
	"TSTORM",
	"CSTORM",
	"XSTORM",
	"USTORM",
	"WC",
	"PCIEuc",
	"HOSTBCV"
};

#ifndef FWUPGTOOL
int tcl_nvm_set_bits(ClientData client_data, Tcl_Interp *interp,
					 int objc, Tcl_Obj *CONST objv[]);
int tcl_nvm_get_bits(ClientData client_data, Tcl_Interp *interp,
					 int objc, Tcl_Obj *CONST objv[]);
int tcl_nvm_load(ClientData client_data, Tcl_Interp *interp,
				 int objc, Tcl_Obj *CONST objv[]);
int tcl_nvm_store(ClientData client_data, Tcl_Interp *interp,
				  int objc, Tcl_Obj *CONST objv[]);
int tcl_nvm_free_structs(ClientData client_data, Tcl_Interp *interp,
						 int objc, Tcl_Obj *CONST objv[]);
int tcl_nvm_show_nvm_offset(ClientData client_data, Tcl_Interp *interp,
							int objc, Tcl_Obj *CONST objv[]);
int tcl_nvm_show_shmem_offset(ClientData client_data, Tcl_Interp *interp,
							  int objc, Tcl_Obj *CONST objv[]);
#endif // !FWUPGTOOL


static u32_t get_file_size(char *filename, u32_t *size);
static u32_t read_bin_file(Tcl_Interp *interp, char *filename, u8_t *buf,
						   u32_t size, u32_t *byte_read);
static u32_t write_bin_file(Tcl_Interp *interp, char *filename, u8_t *buf,
							u32_t size, u32_t *byte_written);
static u32_t print_mba_ver_string(char *str_ptr, char *buf);


static u32_t atmel_8B_append(
	Tcl_Interp *interp,
	u8_t *buf,
	u32_t length,
	u32_t *byte_dumped
	);

static u32_t atmel_8B_strip(
	Tcl_Interp *interp,
	u8_t *buf,
	u32_t length,
	u32_t *bytes_written
	);


#ifndef FWUPGTOOL
static int check_vid_did(Tcl_Interp *interp, u32_t pci_id, u32_t pci_sub_id);
#endif // !FWUPGTOOL

extern image_map_t image_table[];


#ifdef FWUPGTOOL
int tcl_al_nvm_dir_cmd(ClientData client_data, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
#endif // FWUPGTOOL

/* nvm prg command options */
static char      *nvmprg_option_names[] = { "-mem", "-l", "-force_mac", "-force_lic", "-skip_lic", "-skip_config", "-force_ibcfg", "-force_febcfg", "-force_niv",\
		"-skip_sn", "-skip_ec", "-no_idmatch", "-idmatch", "-raw", "-skip_options", "-iffe", (char *)NULL };
enum nvmprg_options {
	OPTMem, OPTLittle, OPTFmac, OPTFlic, OPTSkiplic, OPTSkipcfg, OPTFibcfg, OPTFfebcfg, OPTFniv,\
		OPTSkipsn, OPTSkipec, OPTNoidmatch, OPTIdmatch, OPTRaw, OPTSkipOptions, OPTIffe, NO_MATCH
};

typedef enum {
	IB_OPTION_NONE,
	IB_OPTION_IPV4,
	IB_OPTION_IPV6,
	IB_OPTION_BOTH
} IB_OPTION_T;

typedef struct _nvm_upgrade_params_t {
	common_nvm_dev_stat_t dev_stat;
	char            *option;
	u8_t            *buf;
	u32_t           byte_read;
	u32_t           *image_types_in_input_mfw;
	u32_t           input_mfw_images_found;
	u8_t            iscsi_ib_option;
	u8_t            ib_opt;
	u8_t            validate_versions;
	u8_t            forced_cfg;
	u8_t            forced_prg;
	u16_t           device_id;
	u16_t           vendor_id;
	u16_t           sub_vendor_id;
	u16_t           sub_device_id;
	void            *modules_buf;
	void            *vpd_buf;
	void            *hw_set_buf;
	u8_t            feb_forced_cfg;
	u8_t            feb_forced_prg_evrst;
	u8_t            mbi_upgrade;
	extended_dir_image_t extended_dir;
	char            mfw_version[20];
	char            mba_version[20];
	char            efi_version[20];
} nvm_upgrade_params_t;

/* External functions */
#ifdef FWUPGTOOL
extern NVM_INTERFACE nvm_interface(Tcl_Interp *interp);
extern void printfWrapper(char *fmt, ...);
extern common_nvm_status_t common_nvm_get_port_num(u32_t *port_num);
extern u32_t common_nvm_number_of_everest_functions(void);
#endif // FWUPGTOOL

#ifndef FWUPGTOOL

/*
 *------------------------------------------------------------------------
 * tcl_al_nvm_usage --
 *     Add usage information on "nvm" command
 *------------------------------------------------------------------------
 */
int tcl_al_nvm_usage(
	IN      char  *buf,                //Pointer to the buffer of the command usage
	IN      u32_t *len                     //Pointer to the length result of the command usage
	) {
	u32_t offset = 0;
	u32_t i;

	*len = 0;
	*len += sprintf(&buf[offset + *len], "%-*s Access non-volatile memory.\n\n",
					MAX_CMD_LENGTH, NVM_CMD);

	*len += sprintf(&buf[offset + *len], "Usage: nvm read <addr> [bytecount]\n");
	*len += sprintf(&buf[offset + *len], "       nvm write <addr> <value32 | byte_list>\n");
	*len += sprintf(&buf[offset + *len], "       nvm show [<addr> [bytecount]]\n");
	*len += sprintf(&buf[offset + *len], "       nvm fill <addr> <bytecount> <value32|"NVM_FILL_PTTN_INC"|"NVM_FILL_PTTN_ADDR">\n");
	*len += sprintf(&buf[offset + *len], "           <addr>     : memory offset\n");
	*len += sprintf(&buf[offset + *len], "           <value32>  : a 32-bit value\n");
	*len += sprintf(&buf[offset + *len], "           "NVM_FILL_PTTN_INC"      : for dword incremental pattern\n");
	*len += sprintf(&buf[offset + *len], "           "NVM_FILL_PTTN_ADDR"     : for address pattern\n");
	*len += sprintf(&buf[offset + *len], "           <byte_list>: a Tcl list of byte values\n");
	*len += sprintf(&buf[offset + *len], "           <bytecount>: number of bytes\n");
	*len += sprintf(&buf[offset + *len], "       nvm upgrade [-F] <image_type> <filename>\n");
	*len += sprintf(&buf[offset + *len], "           <image_type> : -bc, -l2, -mfw, -mbi");
	for (i = 0; i < common_nvm_image_table_size(); i++) {
		if (strcmp(image_table[i].opt, "") != 0) {
			*len += sprintf(&buf[offset + *len], ", %s", image_table[i].opt);
		}
	}
	*len += sprintf(&buf[offset + *len], ",-ib_ipv6, -ib_ipv4n6\n");
	*len += sprintf(&buf[offset + *len], "           <filename>   : binary file\n");
	*len += sprintf(&buf[offset + *len], "       nvm cfg [item=choice]         : Sets item to requested choice value\n");
	*len += sprintf(&buf[offset + *len], "       nvm cfg {item=string}         : Sets item to requested string\n");
	*len += sprintf(&buf[offset + *len], "       nvm cfg [item=]               : Gets the current item value\n");
	*len += sprintf(&buf[offset + *len], "       nvm cfg -offset [item=]       : Gets the requested item offset in NVM\n");
	*len += sprintf(&buf[offset + *len], "       nvm cfg -shmem [item=]        : Gets the requested item offset in SHMEM\n");
	*len += sprintf(&buf[offset + *len], "       nvm cfg [board name]          : Sets default NVM configuration\n");
	*len += sprintf(&buf[offset + *len], "                                     : according to this board type\n");
	*len += sprintf(&buf[offset + *len], "       nvm cfg -dump <filename>      : Dump NVM configuration to file\n");
	*len += sprintf(&buf[offset + *len], "       nvm chk <board name> [-fix]   : Compare NVM configuration to the default\n");
	*len += sprintf(&buf[offset + *len], "                                     : setting of this board type.\n");
	*len += sprintf(&buf[offset + *len], "                                     : [-fix] fixes any differences found.\n");
	*len += sprintf(&buf[offset + *len], "       nvm prg [options] <offset> [<filename>|<hostaddr>] [size]\n");
	*len += sprintf(&buf[offset + *len], "           option -mem               : input from host memory\n");
	*len += sprintf(&buf[offset + *len], "           option "NVM_LITTLE_ENDIAN_FLAG"                 : little endian file format\n");
	*len += sprintf(&buf[offset + *len], "           option -force_mac         : force to update MAC addresses\n");
	*len += sprintf(&buf[offset + *len], "                                     : by default, MAC address is preserved\n");
	*len += sprintf(&buf[offset + *len], "           option -force_lic         : force to update license blocks\n");
	*len += sprintf(&buf[offset + *len], "                                     : by default, license is preserved\n");
	*len += sprintf(&buf[offset + *len], "           option -skip_lic          : skip updating license blocks\n");
	*len += sprintf(&buf[offset + *len], "           option -skip_ec           : preserve EC field in VPD block\n");
	*len += sprintf(&buf[offset + *len], "           option -skip_sn           : preserve SN field in VPD block\n");
	*len += sprintf(&buf[offset + *len], "           option -raw               : program the complete raw image\n");
	*len += sprintf(&buf[offset + *len], "           option -skip_config       : skip update params configured by nvm cfg\n");
	*len += sprintf(&buf[offset + *len], "           option -force_ibcfg       : force to update iscsi boot cfg block\n");
	*len += sprintf(&buf[offset + *len], "           option -force_febcfg      : force to update fcoe boot cfg block\n");
	*len += sprintf(&buf[offset + *len], "           option -no_idmatch        : skip device ID check\n");
	*len += sprintf(&buf[offset + *len], "           option -idmatch           : match device VID,DID,SSID and SVID\n");
	*len += sprintf(&buf[offset + *len], "           <offset>                  : where in NVM to start programming\n");
	*len += sprintf(&buf[offset + *len], "           [<filename>|<hostaddr>]   : binary file or host memory address\n");
	*len += sprintf(&buf[offset + *len], "           <size>                    : the number of bytes read from host memory\n");
	*len += sprintf(&buf[offset + *len], "       nvm dump ["NVM_LITTLE_ENDIAN_FLAG"] [<offset> <length>] <filename>\n");
	*len += sprintf(&buf[offset + *len], "           ["NVM_LITTLE_ENDIAN_FLAG"]         : little endian format\n");
	*len += sprintf(&buf[offset + *len], "           <offset>     : where in NVM to start dumping\n");
	*len += sprintf(&buf[offset + *len], "           <length>     : bytes to dump (multiple of 4)\n");
	*len += sprintf(&buf[offset + *len], "           <filename>   : output binary file\n");
	*len += sprintf(&buf[offset + *len], "       nvm dir [[-delete] entry] # Display NVM directory of images\n");
	*len += sprintf(&buf[offset + *len], "       nvm crc [[-fix] entry] # Display NVM directory of images and check CRCs, correct if '-fix' is included\n");
	*len += sprintf(&buf[offset + *len], "Note: \"nvm show\" takes previous addr+bytecount as new addr\n");
	*len += sprintf(&buf[offset + *len], "Examples:nvm read 0x0 0x100\n");
	*len += sprintf(&buf[offset + *len], "         nvm write 0x0 0xffff\n");
	*len += sprintf(&buf[offset + *len], "         nvm write 0x0 {0x01 0x02}\n");
	*len += sprintf(&buf[offset + *len], "         nvm show 0x0 0x100\n");
	*len += sprintf(&buf[offset + *len], "         nvm fill 0x0 0x100 0xfeed\n");
	*len += sprintf(&buf[offset + *len], "         nvm upgrade -mfw mf712v64.20\n");
	*len += sprintf(&buf[offset + *len], "         nvm upgrade -ib iscsi.bin\n");
	*len += sprintf(&buf[offset + *len], "         nvm upgrade -feb fcoeb.bin\n");
	*len += sprintf(&buf[offset + *len], "         nvm upgrade -m_pn modules.txt\n");
	*len += sprintf(&buf[offset + *len], "         nvm upgrade -vpd vpd.txt\n");
	*len += sprintf(&buf[offset + *len], "         nvm cfg 1=00:10:18:01:02:03 17=1\n");
	*len += sprintf(&buf[offset + *len], "         nvm cfg 2=\n");
	*len += sprintf(&buf[offset + *len], "         nvm cfg -offset 2=\n");
	*len += sprintf(&buf[offset + *len], "         nvm cfg -offset -shmem 2=\n");
	*len += sprintf(&buf[offset + *len], "         nvm cfg {11=PCIe 2-P 10Gbe-SFP+ adapter}\n");
	*len += sprintf(&buf[offset + *len], "         nvm cfg A1020G\n");
	*len += sprintf(&buf[offset + *len], "         nvm cfg -dump tmp.txt\n");
	*len += sprintf(&buf[offset + *len], "         nvm chk A1020G\n");
	*len += sprintf(&buf[offset + *len], "         nvm chk A1020G -fix\n");
	*len += sprintf(&buf[offset + *len], "         nvm prg 0 image.bin\n");
	*len += sprintf(&buf[offset + *len], "         nvm dump 0 0x100 output.bin\n");
	*len += sprintf(&buf[offset + *len], "         nvm dump output.bin\n");
	*len += sprintf(&buf[offset + *len], "         nvm dump -m_pn test.txt\n");
	*len += sprintf(&buf[offset + *len], "         nvm dump -vpd vpd_dump.txt\n");
	*len += sprintf(&buf[offset + *len], "         nvm dir bc1\n");
	*len += sprintf(&buf[offset + *len], "         nvm dir -delete mba\n");
	*len += sprintf(&buf[offset + *len], "         nvm crc -fix bc1\n");

	return COMMON_NVM_STATUS_SUCCESS;
}

/*
 *------------------------------------------------------------------------
 * tcl_al_update_isnpar_env_vals --
 *
 * Results:
 *     None
 *
 * Side effects:
 *     updates IS_NIC_PRTITION tcl env vars according to existance of their NVM images
 *------------------------------------------------------------------------
 */
void tcl_al_update_isnpar_env_vals(Tcl_Interp *interp) {
	u32_t status, found_image, nvm_addr, byte_cnt, entry_idx;

	/* those tcl env vars are relevant for 57712 and 5778x only */
	if (!is_e1x(interp)) {
		/* first clear vals */
		Tcl_SetVar2(interp, TCL_ENV, ENV_IS_MAC_PARTITION, "0", TCL_GLOBAL_ONLY);

		if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
			/* set IS_NPAR to 1 if NIC_PRTION image exists */
			status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_NIC_PARTITION_CFG, &nvm_addr, &byte_cnt, &entry_idx);
			if ((status ==  COMMON_NVM_STATUS_SUCCESS) && (found_image)) {
				Tcl_SetVar2(interp, TCL_ENV, ENV_IS_MAC_PARTITION, "1", TCL_GLOBAL_ONLY);
			}
		} else {
			/* in serial mode assume both exists, to minimize nvm opertaions */
			Tcl_SetVar2(interp, TCL_ENV, ENV_IS_MAC_PARTITION, "1", TCL_GLOBAL_ONLY);
		}
	}
}

/*
 *------------------------------------------------------------------------
 * toe_fini_nvm_cmd --
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
int tcl_al_nvm_cleanup(Tcl_Interp *interp) {
	/* De-register the command from Tcl */
	Tcl_DeleteCommand(interp, NVM_CMD);
	return TCL_OK;
}

/*
 *------------------------------------------------------------------------
 * tcl_nvm_store --
 *      Store nvm_config and shmem_config in NVM
 *
 * Results:
 *     A standard Tcl result
 *
 *------------------------------------------------------------------------
 */
int tcl_nvm_store(ClientData client_data, Tcl_Interp *interp,
				  int objc, Tcl_Obj *CONST objv[]) {
	nvm_cfg_t *nvm_config_p = NULL, *shmem_config_p = NULL;
	nvm_nic_part_cfg_t *mac_partition_p = NULL;
	u32_t *cfg_ext_shared_block_p = NULL;
	int rc;
	u32_t low_addr, high_addr;


	tcl_al_log_command(NVM_STORE, objc, objv);

	if (objc != 5) {
		Tcl_WrongNumArgs(interp, objc, objv, "should include <nvm_config> <shmem_config> <mac_partition> <cfg_ext_shared>");
		return TCL_ERROR;
	}

	get_vaddr(interp, objv, 1, &low_addr, &high_addr);
	nvm_config_p = (nvm_cfg_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));

	get_vaddr(interp, objv, 2, &low_addr, &high_addr);
	shmem_config_p = (nvm_cfg_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));

	get_vaddr(interp, objv, 3, &low_addr, &high_addr);
	mac_partition_p = (nvm_nic_part_cfg_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));

	get_vaddr(interp, objv, 4, &low_addr, &high_addr);
	cfg_ext_shared_block_p = (u32_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));

	if (common_nvm_store_cfg(nvm_config_p, shmem_config_p, mac_partition_p, cfg_ext_shared_block_p)
		== COMMON_NVM_STATUS_SUCCESS) {
		add_shadow_dell_ncsi_image();
		add_bdn_image();
#ifndef FWUPGTOOL
		// For FWUPG tool, we don't issue the MCP reset and update the device information.
#ifndef WIN_DIAG
		// Update device information of all devices and environment MAC addr.
		common_nvm_refresh_devices();
#endif
#endif // !FWUPGTOOL
		rc = TCL_OK;
	} else {
		Tcl_AppendResult(interp, "Problem storing nvm_cfg\n", NULL);
		rc = TCL_ERROR;
	}

	common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)nvm_config_p);
	common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)shmem_config_p);
	common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)mac_partition_p);
	common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)cfg_ext_shared_block_p);
	return rc;
}

/*
 *------------------------------------------------------------------------
 * tcl_nvm_free_structs --
 *      Free the nvm_config, shmem_config and other structures
 *
 * Results:
 *     A standard Tcl result
 *
 *------------------------------------------------------------------------
 */
int tcl_nvm_free_structs(ClientData client_data, Tcl_Interp *interp,
						 int objc, Tcl_Obj *CONST objv[]) {
	nvm_cfg_t *nvm_config_p = NULL, *shmem_config_p = NULL;
	nvm_nic_part_cfg_t *mac_partition_p = NULL;
	u32_t *cfg_ext_shared_block_p = NULL;
	u32_t low_addr, high_addr;


	tcl_al_log_command(NVM_FREE_STRUCTS, objc, objv);

	if (objc != 5) {
		Tcl_WrongNumArgs(interp, objc, objv, "should include <nvm_config> <shmem_config> <mac_partition> <cfg_ext_shared>");
		return TCL_ERROR;
	}

	get_vaddr(interp, objv, 1, &low_addr, &high_addr);
	nvm_config_p = (nvm_cfg_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));

	get_vaddr(interp, objv, 2, &low_addr, &high_addr);
	shmem_config_p = (nvm_cfg_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));

	get_vaddr(interp, objv, 3, &low_addr, &high_addr);
	mac_partition_p = (nvm_nic_part_cfg_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));

	get_vaddr(interp, objv, 4, &low_addr, &high_addr);
	cfg_ext_shared_block_p = (u32_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));

	if (nvm_config_p != NULL) {
		common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)nvm_config_p);
	}
	if (shmem_config_p != NULL) {
		common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)shmem_config_p);
	}
	if (mac_partition_p != NULL) {
		common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)mac_partition_p);
	}
	if (cfg_ext_shared_block_p != NULL) {
		common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)cfg_ext_shared_block_p);
	}
	return TCL_OK;
}

/*
 *------------------------------------------------------------------------
 * tcl_getc --
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     get a char from the user and set it as tcl result object.
 *------------------------------------------------------------------------
 */
int tcl_getc(ClientData client_data, Tcl_Interp *interp,
			 int objc, Tcl_Obj *CONST objv[]) {
	char ans[2];

	tcl_al_log_command(GETC, objc, objv);

	if (objc != 1) {
		Tcl_WrongNumArgs(interp, objc, objv, ":too many args");
		return TCL_ERROR;
	}
	ans[0] = tcl_io_getch();
	ans[1] = 0;
	Tcl_AppendResult(interp, ans, NULL);
	return TCL_OK;
}

/*
 *------------------------------------------------------------------------------------
 * tcl_nvm_load --
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     load the nvm_config, shmem_config structs from the nvram
 *     load also mac_partition and cfg_extended_shared_block
 *------------------------------------------------------------------------------------
 */
int tcl_nvm_load(ClientData client_data, Tcl_Interp *interp,
				 int objc, Tcl_Obj *CONST objv[]) {
	Tcl_Obj * result_ptr,*objPtr;
	Tcl_Obj * nvmConfigObj[2],*shmemConfigObj[2];
	Tcl_Obj * macPartitionObj[2],*cfgExtSharedObj[2];
	nvm_cfg_t * nvm_config_p,*shmem_config_p;
	nvm_nic_part_cfg_t *mac_partition_cfg_p;
	u32_t *cfg_ext_shared_block_p;


	tcl_al_log_command(NVM_LOAD, objc, objv);

	if (objc != 1) {
		Tcl_WrongNumArgs(interp, objc, objv, ":too many args");
		return TCL_ERROR;
	}

	result_ptr = Tcl_GetObjResult(interp);

	// Allocate memory for the nvm_config struct
	nvm_config_p = (nvm_cfg_t *)(uintptr_t)common_nvm_alloc_virt(sizeof(nvm_cfg_t));
	if (nvm_config_p == NULL) {
		DbgBreakMsg("tcl_nvm_load: Memory allocation for nvm_config failed\n");
		return TCL_ERROR;
	}
	memset(nvm_config_p, 0, sizeof(nvm_cfg_t));

	// Allocate memory for the shmem_config struct
	shmem_config_p = (nvm_cfg_t *)(uintptr_t)common_nvm_alloc_virt(sizeof(nvm_cfg_t));
	if (shmem_config_p == NULL) {
		DbgBreakMsg("tcl_nvm_load: Memory allocation for shmem_config failed\n");
		return TCL_ERROR;
	}
	memset(shmem_config_p, 0, sizeof(nvm_cfg_t));

	// Allocate memory for the mac_partition_cfg struct
	mac_partition_cfg_p = (nvm_nic_part_cfg_t *)(uintptr_t)common_nvm_alloc_virt(sizeof(nvm_nic_part_cfg_t));
	if (mac_partition_cfg_p == NULL) {
		DbgBreakMsg("tcl_nvm_load: Memory allocation for mac_partition failed\n");
		return TCL_ERROR;
	}
	memset(mac_partition_cfg_p, 0, sizeof(nvm_nic_part_cfg_t));

	// Allocate memory for the ext_cfg_shared_block struct
	cfg_ext_shared_block_p = (u32_t *)(uintptr_t)common_nvm_alloc_virt(4 * NVM_CFG_EXTENDED_SHARED_IMAGE_MAX_SIZE);
	if (cfg_ext_shared_block_p == NULL) {
		DbgBreakMsg("tcl_nvm_load: Memory allocation for ext_cfg_shared_block failed\n");
		return TCL_ERROR;
	}
	memset(cfg_ext_shared_block_p, 0, 4 * NVM_CFG_EXTENDED_SHARED_IMAGE_MAX_SIZE);

	// load the NVM configuration from NVM to allocated structures
	if (common_nvm_load_cfg(nvm_config_p, shmem_config_p, mac_partition_cfg_p,
							cfg_ext_shared_block_p) == COMMON_NVM_STATUS_SUCCESS) {

		// Add nvmConfigObj to result list
		nvmConfigObj[0] = Tcl_NewIntObj(U64_LO((int_ptr_t)nvm_config_p));
		nvmConfigObj[1] = Tcl_NewIntObj(U64_HI((int_ptr_t)nvm_config_p));
		common_nvm_set_hex_int_object(&nvmConfigObj[0], U64_LO((int_ptr_t)nvm_config_p));
		common_nvm_set_hex_int_object(&nvmConfigObj[1], U64_HI((int_ptr_t)nvm_config_p));
		objPtr = Tcl_NewListObj(2, nvmConfigObj);
		Tcl_ListObjAppendElement(interp, result_ptr, objPtr);

		// Add shmemConfigObj to result list
		shmemConfigObj[0] = Tcl_NewIntObj(U64_LO((int_ptr_t)shmem_config_p));  // lo 32-bits
		shmemConfigObj[1] = Tcl_NewIntObj(U64_HI((int_ptr_t)shmem_config_p));  // hi 32-bits
		common_nvm_set_hex_int_object(&shmemConfigObj[0], U64_LO((int_ptr_t)shmem_config_p));
		common_nvm_set_hex_int_object(&shmemConfigObj[1], U64_HI((int_ptr_t)shmem_config_p));
		objPtr = Tcl_NewListObj(2, shmemConfigObj);
		Tcl_ListObjAppendElement(interp, result_ptr, objPtr);

		// Add macPartitionObj to result list
		macPartitionObj[0] = Tcl_NewIntObj(U64_LO((int_ptr_t)mac_partition_cfg_p));  // lo 32-bits
		macPartitionObj[1] = Tcl_NewIntObj(U64_HI((int_ptr_t)mac_partition_cfg_p));  // hi 32-bits
		common_nvm_set_hex_int_object(&macPartitionObj[0], U64_LO((int_ptr_t)mac_partition_cfg_p));
		common_nvm_set_hex_int_object(&macPartitionObj[1], U64_HI((int_ptr_t)mac_partition_cfg_p));
		objPtr = Tcl_NewListObj(2, macPartitionObj);
		Tcl_ListObjAppendElement(interp, result_ptr, objPtr);

		// Add cfgExtSharedObj to result list
		cfgExtSharedObj[0] = Tcl_NewIntObj(U64_LO((int_ptr_t)cfg_ext_shared_block_p));  // lo 32-bits
		cfgExtSharedObj[1] = Tcl_NewIntObj(U64_HI((int_ptr_t)cfg_ext_shared_block_p));  // hi 32-bits
		common_nvm_set_hex_int_object(&cfgExtSharedObj[0], U64_LO((int_ptr_t)cfg_ext_shared_block_p));
		common_nvm_set_hex_int_object(&cfgExtSharedObj[1], U64_HI((int_ptr_t)cfg_ext_shared_block_p));
		objPtr = Tcl_NewListObj(2, cfgExtSharedObj);
		Tcl_ListObjAppendElement(interp, result_ptr, objPtr);

		return TCL_OK;
	} else {
		Tcl_AppendResult(interp, "Problem loading nvm_cfg\n", NULL);
		return TCL_ERROR;
	}
}

/*
 *------------------------------------------------------------------------
 * tcl_nvm_set_bits --
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     Receive 3 variables from the user (offset, size, value)
 *     If all variables are numerical and the value can fit in the size given
 *     Will store the the value to that bits in the nvm_config struct.
 *------------------------------------------------------------------------
 */
int tcl_nvm_set_bits(ClientData client_data, Tcl_Interp *interp,
					 int objc, Tcl_Obj *CONST objv[]) {
	u64_t offset, mask, shift;
	u64_t size  = 0;
	u64_t value = 0;
	int int_buf;

	u32_t *nvmAsArr = NULL;
	u32_t *shmemArr = NULL;
	nvm_cfg_t *nvm_config_p = NULL;
	nvm_cfg_t *shmem_config_p = NULL;
	u32_t low_addr, high_addr;

	tcl_al_log_command(NVM_SET_BITS, objc, objv);

	if (objc != 6) {
		Tcl_WrongNumArgs(interp, objc, objv, "<nvm_config_ptr> <shmem_config_ptr> <offset> <size> <value>");
		return TCL_ERROR;
	}

	get_vaddr(interp, objv, 1, &low_addr, &high_addr);
	nvm_config_p = (nvm_cfg_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));

	get_vaddr(interp, objv, 2, &low_addr, &high_addr);
	shmem_config_p = (nvm_cfg_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));


	if (Tcl_GetIntFromObj(interp, objv[3], &int_buf) != TCL_OK) {
		printf(Tcl_GetStringFromObj(objv[1], NULL));
		Tcl_AppendResult(interp, "Offset must be a integer", NULL);
		Tcl_AppendResult(interp, Tcl_GetStringFromObj(objv[1], NULL), "\n", NULL);
		return TCL_ERROR;
	}
	offset = int_buf;

	if (Tcl_GetIntFromObj(interp, objv[4], &int_buf) != TCL_OK || int_buf < 1) {
		Tcl_AppendResult(interp, "Size must be a integer", NULL);
		Tcl_AppendResult(interp, Tcl_GetStringFromObj(objv[2], NULL), "\n", NULL);
		return TCL_ERROR;
	}
	size = int_buf;

	if (Tcl_GetIntFromObj(interp, objv[5], &int_buf) != TCL_OK) {
		Tcl_AppendResult(interp, "Value must be a integer", NULL);
		Tcl_AppendResult(interp, Tcl_GetStringFromObj(objv[3], NULL), "\n", NULL);
		return TCL_ERROR;
	}
	value = int_buf;

	if (offset % 32 + size > 32) {
		Tcl_AppendResult(interp, "Offset mod 32 + size must be smaller equal to 32.\n", NULL);
		return TCL_ERROR;
	}
	if (value >= ((u64_t)1 << size)) {
		Tcl_AppendResult(interp, "Value must fit the field's size : ", NULL);
		Tcl_AppendResult(interp, Tcl_GetStringFromObj(objv[3], NULL), "\n", NULL);

		return TCL_ERROR;
	}
	shift = offset % 32;
	mask = (u64_t)1 << size;
	mask = (mask - 1) << shift;

	nvmAsArr = (u32_t *)nvm_config_p;
	nvmAsArr[offset / 32] &= ~mask;
	nvmAsArr[offset / 32] |= value << shift;

	shmemArr = (u32_t *)shmem_config_p;
	shmemArr[offset / 32] &= ~mask;
	shmemArr[offset / 32] |= value << shift;

	return TCL_OK;
}

/*
 *------------------------------------------------------------------------
 * tcl_nvm_get_bits --
 *
 * Receive 2 variables from the user (offset, size)
 * Returns the read value from this offset
 *------------------------------------------------------------------------
 */
int tcl_nvm_get_bits(ClientData client_data, Tcl_Interp *interp,
					 int objc, Tcl_Obj *CONST objv[]) {
	u32_t offset, size, shift, value;
	u64_t mask;
	Tcl_Obj *result_ptr = NULL;
	u32_t *nvmAsArr = NULL;
	nvm_cfg_t *nvm_config_p = NULL;
	u32_t low_addr, high_addr;

	tcl_al_log_command(NVM_GET_BITS, objc, objv);

	if (objc != 4) {
		Tcl_WrongNumArgs(interp, objc, objv, "<nvm_config_ptr> <offset> <size>");
		return TCL_ERROR;
	}

	get_vaddr(interp, objv, 1, &low_addr, &high_addr);
	nvm_config_p = (nvm_cfg_t *)((int_ptr_t)LONG_ADDR(low_addr, high_addr));


	if (Tcl_GetIntFromObj(interp, objv[2], (int *)&offset) != TCL_OK) {
		Tcl_AppendResult(interp, "Offset must be a integer", NULL);
		Tcl_AppendResult(interp, Tcl_GetStringFromObj(objv[1], NULL), "\n", NULL);
		return TCL_ERROR;
	}
	if (Tcl_GetIntFromObj(interp, objv[3], (int *)&size) != TCL_OK || size < 1) {
		Tcl_AppendResult(interp, "Size must be a integer", NULL);
		Tcl_AppendResult(interp, Tcl_GetStringFromObj(objv[2], NULL), "\n", NULL);
		return TCL_ERROR;
	}


	if (offset % 32 + size > 32) {
		Tcl_AppendResult(interp, "Offset mod 32 + size must be smaller equal to 32.\n", NULL);
		nvmAsArr = (u32_t *)nvm_config_p;
		value = nvmAsArr[offset / 32];
		result_ptr = Tcl_NewObj();
		Tcl_SetIntObj(result_ptr, value);
		Tcl_SetObjResult(interp, result_ptr);
		return TCL_OK;
		return TCL_ERROR;
	}
	shift = offset % 32;
	mask = 1 << size;

	mask = (mask - 1) << shift;

	nvmAsArr = (u32_t *)nvm_config_p;
	value = nvmAsArr[offset / 32];

	if (size < 32) {
		value &= mask;
	}
	value = value >> shift;

	result_ptr = Tcl_NewObj();
	Tcl_SetIntObj(result_ptr, value);
	Tcl_SetObjResult(interp, result_ptr);
	return TCL_OK;
}

/*
 *------------------------------------------------------------------------
 * tcl_nvm_show_nvm_offset --
 *
 * Show NVM offset of nvm_cfg option
 *------------------------------------------------------------------------
 */
int tcl_nvm_show_nvm_offset(ClientData client_data, Tcl_Interp *interp,
							int objc, Tcl_Obj *CONST objv[]) {
	u32_t offset, size, bytecount, shift;
	u64_t mask;
	u32_t nvm_offset, i, tmp_addr;
	u32_t *ptr;

	tcl_al_log_command(NVM_SHOW_OFFSET, objc, objv);

	if (objc != 3) {
		Tcl_WrongNumArgs(interp, objc, objv, "<offset> <size>");
		return TCL_ERROR;
	}

	if (Tcl_GetIntFromObj(interp, objv[1], (int *)&offset) != TCL_OK) {
		Tcl_AppendResult(interp, "Offset must be a integer", NULL);
		Tcl_AppendResult(interp, Tcl_GetStringFromObj(objv[1], NULL), "\n", NULL);
		return TCL_ERROR;
	}
	if (Tcl_GetIntFromObj(interp, objv[2], (int *)&size) != TCL_OK || (size < 1)) {
		Tcl_AppendResult(interp, "Size must be a integer", NULL);
		Tcl_AppendResult(interp, Tcl_GetStringFromObj(objv[2], NULL), "\n", NULL);
		return TCL_ERROR;
	}

	if (common_nvm_get_nvm_offset(offset, &nvm_offset) == COMMON_NVM_STATUS_SUCCESS) {

		nvm_offset = (nvm_offset / 4) * 4; // address should be a multiple of 4
		bytecount = (size - 1) / 8 + 1;      // convert size in bits to byte count
		bytecount = (bytecount + 3) & 0xfffffffc;; // show at least one dword (4 bytes)

		ptr = (u32_t *)Tcl_Alloc(bytecount);
		for (i = 0, tmp_addr = nvm_offset; i < (u32_t)bytecount / 4; i++, tmp_addr += 4) {
			if (common_nvm_read_nvram(tmp_addr, &ptr[i], sizeof(u32_t)) != COMMON_NVM_STATUS_SUCCESS) {
				Tcl_SetObjResult(interp, Tcl_NewStringObj("Bad device or address", 0));
				Tcl_Free((char *)ptr);
				return TCL_ERROR;
			}
		}

		// Convert size and offset into mask
		shift = offset % 32;
		mask = (u64_t)1 << size;
		mask = (mask - 1) << shift;

		printf("NVM offset: 0x%08x Mask: 0x%Lx (size=%d offset=%d in bits)\n", nvm_offset, mask, size, shift);
		disp_32bit_array(ptr, nvm_offset, bytecount / 4, 0);

		Tcl_Free((char *)ptr);
		return TCL_OK;

	} else {
		Tcl_AppendResult(interp, "Problem getting nvm offset.\n", NULL);
		return TCL_ERROR;
	}
}

/*
 *------------------------------------------------------------------------
 * tcl_nvm_show_shmem_offset --
 *
 * Show offset in shmem of nvm_cfg option
 *------------------------------------------------------------------------
 */
int tcl_nvm_show_shmem_offset(ClientData client_data, Tcl_Interp *interp,
							  int objc, Tcl_Obj *CONST objv[]) {
	u32_t offset, size, shmem_addr, bytecount;
	u32_t i, tmp_addr, *ptr;

	tcl_al_log_command(NVM_SHOW_OFFSET, objc, objv);

	if (objc != 3) {
		Tcl_WrongNumArgs(interp, objc, objv, "<offset> <size>");
		return TCL_ERROR;
	}

	if (Tcl_GetIntFromObj(interp, objv[1], (int *)&offset) != TCL_OK) {
		Tcl_AppendResult(interp, "Offset must be a integer", NULL);
		Tcl_AppendResult(interp, Tcl_GetStringFromObj(objv[1], NULL), "\n", NULL);
		return TCL_ERROR;
	}
	if (Tcl_GetIntFromObj(interp, objv[2], (int *)&size) != TCL_OK || (size < 1)) {
		Tcl_AppendResult(interp, "Size must be a integer", NULL);
		Tcl_AppendResult(interp, Tcl_GetStringFromObj(objv[2], NULL), "\n", NULL);
		return TCL_ERROR;
	}

	if (common_nvm_get_shmem_offset(offset, &shmem_addr) == COMMON_NVM_STATUS_SUCCESS) {

		shmem_addr = (shmem_addr / 4) * 4; // address should be a multiple of 4
		bytecount = (size - 1) / 8 + 1;      // convert size in bits to byte count
		bytecount = (bytecount + 3) & 0xfffffffc;; // show at least one dword (4 bytes)

		ptr = (u32_t *)Tcl_Alloc(bytecount);
		for (i = 0, tmp_addr = shmem_addr; i < (u32_t)bytecount / 4; i++, tmp_addr += 4) {
			if (common_nvm_reg_read(tmp_addr, &ptr[i]) != COMMON_NVM_STATUS_SUCCESS) {
				Tcl_SetObjResult(interp, Tcl_NewStringObj("Bad device or address", 0));
				Tcl_Free((char *)ptr);
				return TCL_ERROR;
			}
		}

		printf("\nShmem address: 0x%08x \n", shmem_addr);
		disp_32bit_array(ptr, shmem_addr, bytecount / 4, 0);
		Tcl_Free((char *)ptr);
		return TCL_OK;

	} else {
		Tcl_AppendResult(interp, "This nvm cfg option is not relevant in shmem.\n", NULL);
		return TCL_ERROR;
	}
}

/*
 *------------------------------------------------------------------------
 * toe_nvm_read_cmd --
 *
 *     Implements the new Tcl "nvm read" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
static int tcl_al_nvm_read_cmd(ClientData client_data, Tcl_Interp *interp,
							   int objc, Tcl_Obj *CONST objv[]) {
	Tcl_Obj *result_ptr = NULL, *tmp_obj;
	int addr;
	common_nvm_status_t status;
	u32_t value;
	int count, ret;

	if (objc < 2 || objc > 3) {
		Tcl_WrongNumArgs(interp, 1, objv, "addr [nbytes]");
		return TCL_ERROR;
	}

	result_ptr = Tcl_GetObjResult(interp);

	/* Get register offset */
	if (Tcl_GetIntFromObj(interp, objv[1], (int *)&addr) == TCL_ERROR) {
		Tcl_WrongNumArgs(interp, 1, objv, "option=read <addr>");
		return TCL_ERROR;
	}

	/* Verify <addr> field is larger than zero (CQ# 32209)*/
	if (addr < 0) {
		Tcl_AppendStringsToObj(result_ptr, "Bad address, must be larger than 0",
							   (char *)NULL);
		return TCL_ERROR;
	}

	/* Verify <addr> field is not larger than nvm size */
	if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
		if ((u32_t)addr > common_nvm_nvm_size()) {
			Tcl_AppendStringsToObj(result_ptr, "Bad address, larger than NVM size",
								   (char *)NULL);
			return TCL_ERROR;
		}
	}

	/* Process the command */
	if (objc == 2) {
		DbgMessage(NULL, VERBOSEnv, "Before nvm_read\n");
		/* Display only one dword */
		status = common_nvm_read_nvram(addr & 0xfffffffc, &value, sizeof(value));
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			Tcl_AppendStringsToObj(result_ptr, "Bad device or address",
								   (char *)NULL);
			return TCL_ERROR;
		}
		DbgMessage(NULL, VERBOSEnv, "Setting the result\n");
		common_nvm_set_hex_int_object(&result_ptr, value);

	} else if (objc == 3) {
		int i, j;
		/* Display an array of bytes */
		if (Tcl_GetIntFromObj(interp, objv[2], &count) == TCL_ERROR) {
			return TCL_ERROR;
		}
		tmp_obj = Tcl_NewIntObj(addr);
		ret = Tcl_ConvertToType(interp, tmp_obj, Tcl_GetObjType("string"));
		Tcl_AppendToObj(tmp_obj, ":", 1);
		ret = Tcl_ListObjAppendElement(interp, result_ptr, tmp_obj);
		for (i = 0; i < count;) {
			status = common_nvm_read_nvram(addr & 0xfffffffc, &value, sizeof(value));
			for (j = addr & 0x3; j < 4; j++) {
				common_nvm_generate_hex_int_object(&tmp_obj, value >> (3 - j) * 8 & 0xff);

				Tcl_ListObjAppendElement(interp, result_ptr, tmp_obj);

				addr++;
				if (++i >= count) {
					break;
				}
			}
		}
	}
	return TCL_OK;
}

/*
*------------------------------------------------------------------------
* check_vid_did --
*
*     validate vendor id and device id
*
* Results:
*     None.
*
* Side effects:
*     None.
*------------------------------------------------------------------------
*/
static int check_vid_did(Tcl_Interp *interp, u32_t pci_id, u32_t pci_sub_id) {
	u16 image_vid;
	u16 image_did;
	u16 image_svid;
	u16 image_sdid;
	Tcl_Obj *obj;
	u32_t dev_vid;
	u32_t dev_did;
	u32_t dev_svid;
	u32_t dev_sdid;

	pci_id = ntohl(pci_id);
	image_vid = (pci_id & PORT_HW_CFG_PCI_VENDOR_ID_MASK) >> 16;
	image_did = (pci_id & PORT_HW_CFG_PCI_DEVICE_ID_MASK);


	pci_sub_id = ntohl(pci_sub_id);
	image_sdid = (pci_sub_id & PORT_HW_CFG_PCI_SUBSYS_DEVICE_ID_MASK) >> 16;
	image_svid = (pci_sub_id & PORT_HW_CFG_PCI_SUBSYS_VENDOR_ID_MASK);

	/* Get the device's VID, DID, SDID, SVID */
	obj = Tcl_GetVar2Ex(interp, TCL_ENV, ENV_VID, 0);
	Tcl_GetIntFromObj(interp, obj, (int *)&dev_vid);

	obj = Tcl_GetVar2Ex(interp, TCL_ENV, ENV_DID, 0);
	Tcl_GetIntFromObj(interp, obj, (int *)&dev_did);

	obj = Tcl_GetVar2Ex(interp, TCL_ENV, ENV_SVID, 0);
	Tcl_GetIntFromObj(interp, obj, (int *)&dev_svid);

	obj = Tcl_GetVar2Ex(interp, TCL_ENV, ENV_SDID, 0);
	Tcl_GetIntFromObj(interp, obj, (int *)&dev_sdid);

	if ((image_vid != dev_vid) ||
		(image_did != dev_did) ||
		(image_svid != dev_svid) ||
		(image_sdid != dev_sdid)) {

		if (image_vid != dev_vid) printf("Vendor ID mismatch in image. expected:0x%x, value on image:0x%x\n", dev_vid, image_vid);

		if (image_did != dev_did) printf("Device ID mismatch in image. expected:0x%x, value on image:0x%x\n", dev_did, image_did);

		if (image_svid != dev_svid) printf("Sub-vendor ID mismatch in image. expected:0x%x,  value on image:0x%x\n", dev_svid, image_svid);

		if (image_sdid != dev_sdid) printf("Sub-device ID mismatch in image. expected:0x%x,  value on image:0x%x\n", dev_sdid, image_sdid);

		return TCL_ERROR;
	}

	return TCL_OK;
}
#endif // !FWUPGTOOL

/*
 *------------------------------------------------------------------------
 * tcl_al_nvm_prg_cmd --
 *
 *     Implements the new Tcl "nvm prg" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
#ifndef FWUPGTOOL
static int
#else
int
#endif // !FWUPGTOOL
tcl_al_nvm_prg_cmd(ClientData client_data, Tcl_Interp *interp,
				   int objc, Tcl_Obj *CONST objv[]) {
	u32_t   offset = (u32_t)-1, flag = PRG_NVM_BIG_ENDIAN, prg_raw = 0, skip_options = 0;
	char    *filename = NULL, *ptr;
	u8_t    *buf = NULL;
	u32_t   file_size, nvm_size, byte_read, bytes_written, tmp_val;
	int     tcl_status, final_status = TCL_ERROR;
#ifndef FWUPGTOOL
	u32_t   pci_id, pci_sub_id; // vars for did vid validation
	u32_t   pci_id_offset, pci_sub_id_offset;
#endif // !FWUPGTOOL
	u8_t    id_match_enabled = 0, iffe_enabled = 0;
#ifndef FWUPGTOOL
	u32_t   n_devices = 2, dev_idx = 1, cur_dev_idx = 1;
#endif // !FWUPGTOOL
	u32_t   byte_cnt = 0, lic_opt_override = 0;
	u8_t    from_hmem = 0;
	int     index;
	u32_t   low_addr_val = (u32_t)-1;
#ifndef FWUPGTOOL
	u32_t   high_addr_val = 0;
#endif // !FWUPGTOOL
	char tmp_str[256] = "";
	char options_to_skip_str[256] = "";
	char *token_ptr = NULL, *res_ptr = NULL;
	int options_to_skip[256];
	char *cmd_for_dev[MAX_FUNC];
	char tmp_cmd[256] = "nvm cfg ";
	u32_t i = 0, j = 0, cur_bus = 0, dev_n = 0, bus = 0, idx = 0;
	char delimiters[] = " ,\t\n";
	bootstrap_region_t bootstrap;
	u32_t crc32, in_crc, trailer;

#ifdef FWUPGTOOL
	NVM_INTERFACE iNvm = nvm_interface(interp); // in FwUpgNx2.cpp
#endif // FWUPGTOOL

	// Verify the command includes input arguments
	if (objc == 1) {
		Tcl_WrongNumArgs(interp, 1, objv, "?options? offset <file|mem_addr> ?size?");
		return TCL_ERROR;
	}


#ifndef FWUPGTOOL
	// For FWUPG tool, the driver is always running and we don't need to check it.

	// incase of PCI mode, go through devices and verify driver is not loaded
	if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
		// Get the total number of devices, and current device index
		n_devices = common_nvm_get_total_br_dev();
		cur_dev_idx = atoi(Tcl_GetVar2(interp, TCL_ENV, ENV_DEV, TCL_GLOBAL_ONLY));

		// Go through the devices and check that the driver is not running
		for (dev_idx = 1; dev_idx <= n_devices; dev_idx++) {
			if (BR_PCI_DEV_PRESENT(dev_idx - 1)) {
				execute_device(interp, dev_idx);

				if (strcmp(Tcl_GetVar2(interp, TCL_ENV, ENV_DRV_STATE, TCL_GLOBAL_ONLY), "RUNNING") == 0) {
					print_msg("WARNING: nvm prg command can not be performed while driver is running\n", ESC_COLOR_RED);
					printf("please unload device %d driver before using nvm prg\n", dev_idx);
					return TCL_ERROR;
				}
			}
		}
		// Go back to current device
		execute_device(interp, cur_dev_idx);
	}

#endif // !FWUPGTOOL

	// Skip the cmd name
	objc--, objv++;

	// Parse the "nvm prg" command options
	while (objc > 0) {
		ptr = Tcl_GetStringFromObj(*objv, NULL);
		if (ptr[0] == '-') {
			// Get the enum of the option
			if (Tcl_GetIndexFromObj(interp, *objv, (COMMON_NVM_CHAR_PP)nvmprg_option_names, "option", TCL_EXACT, &index) != TCL_OK) {
				return TCL_ERROR;
			}

			switch ((enum nvmprg_options)index) {
				// Parse the '-mem' option
			case OPTMem:
				// Since the content resides in host memory, pass on its order info.
#if defined(LITTLE_ENDIAN)
				flag &= ~PRG_NVM_BIG_ENDIAN;
#else
				flag |= PRG_NVM_BIG_ENDIAN;
#endif
				from_hmem = 1;
				break;

				// Parse the '-l' option
			case OPTLittle:
				// Set the flag to little endian
				flag &= ~PRG_NVM_BIG_ENDIAN;
				break;

				// Parse the '-force_mac' option
			case OPTFmac:
				// Set the flag to force MAC programming
				flag |= PRG_NVM_MAC_PRG;
				break;

				// Parse the '-force_lic' option
			case OPTFlic:
				flag |= PRG_NVM_LIC_PRG;
				lic_opt_override = 1;
				break;

				// Parse the 'skip_lic' option
			case OPTSkiplic:
				flag &= ~PRG_NVM_LIC_PRG;
				lic_opt_override = 1;
				break;

				// Parse the 'skip_config' option
			case OPTSkipcfg:
				flag |= PRG_NVM_SKIP_CFG_PRG;
				break;

				// Parse the 'force_ibcfg' option
			case OPTFibcfg:
				flag |= PRG_NVM_IBCFG;
				break;

				// Parse the 'force_febcfg' option
			case OPTFfebcfg:
				flag |= PRG_NVM_FEBCFG;
				break;

				// Parse the 'force_niv' option
			case OPTFniv:
				flag |= PRG_NVM_NIV;
				break;

				// Parse the '-skip_sn' option
			case OPTSkipsn:
				// Set the flag to skip serial number programming
				flag |= PRESERVE_NVM_SN;
				break;

				// Parse the '-skip_ec' option
			case OPTSkipec:
				// Set the flag to skip enginerring change programming
				flag |= PRESERVE_NVM_EC;
				break;

				// Parse the '-no_idmatch' options
			case OPTNoidmatch:
				id_match_enabled = 0;
				break;

				// Parse the '-idmatch' option
			case OPTIdmatch:
				id_match_enabled = 1;
				break;

			case OPTRaw:
				/* modify flag after parameter parsing to
				 * turn on -force_mac, -force_smbus, -force_ibcfg, -force_febcfg, -force_lic
				 * turn off -skip_ec, -skip_sn, -skip_lic, -skip_config*/
				prg_raw = 1;
				break;

				// Parse the 'skip_options' option
			case OPTSkipOptions:
				skip_options = 1;
				// skip the "-skip_options"
				objc--, objv++;
				ptr = Tcl_GetStringFromObj(*objv, NULL);
				if ((ptr[0] == '-') || (strchr(ptr, '.') != NULL)) {
					Tcl_AppendResult(interp, "No options specified for -skip_options.\n", NULL);
					return TCL_ERROR;
				} else {
					strcpy(options_to_skip_str, ptr);
				}
				break;

			case OPTIffe:
				iffe_enabled = 1;
				break;

			default:
				Tcl_AppendResult(interp, "Unknown option\n", NULL);
				return TCL_ERROR;
			}
		}
		// Read the offset value if it's not yet initialized
		else if (offset == -1) {
			if (Tcl_GetIntFromObj(interp, *objv, (int *)&offset) == TCL_ERROR) {
				Tcl_AppendResult(interp, "Offset: expect an integer, but got ", NULL);
				Tcl_AppendResult(interp, Tcl_GetStringFromObj(*objv, NULL), "\n", NULL);
				return TCL_ERROR;
			}
			offset &= 0xfffffffc;
		}
		// Read the filname/hmem_addr if it's not yet initialized
		else if ((filename == NULL) && (low_addr_val == -1)) {
			if (from_hmem) {
#ifndef FWUPGTOOL
#ifndef WIN_DIAG
				if ((get_vaddr(interp, objv, 0, &low_addr_val, &high_addr_val)) != TCL_OK) return TCL_ERROR;
#else
				Tcl_SetObjResult(interp, Tcl_NewStringObj("nvm prg -mem option is not supported in winediag", -1));
#endif
#else
				Tcl_SetObjResult(interp, Tcl_NewStringObj("nvm prg -mem option is not supported in fwupg tool", -1));
#endif // !FWUPGTOOL
			} else {
				filename = Tcl_GetStringFromObj(*objv, NULL);
				DbgMessage(NULL, INFORM, "tcl_al_nvm_prg_cmd filename %s\n", filename);
			}
		} else {
			// Get the optional size input argument
			if (Tcl_GetIntFromObj(interp, *objv, (int *)&byte_cnt) == TCL_ERROR) {
				Tcl_AppendResult(interp, "Size: expect an integer, but got ", NULL);
				Tcl_AppendResult(interp, Tcl_GetStringFromObj(*objv, NULL), "\n", NULL);
				return TCL_ERROR;
			}
			offset &= 0xfffffffc;
		}
		// Go to next field
		objc--, objv++;
	}

	// If no filename nor hmem address, show error message
	if ((filename == NULL) && (low_addr_val == -1)) {
		Tcl_AppendResult(interp, "Neither memory location nor file name specified.\n", NULL);
		return TCL_ERROR;
	}

	// Implement the programming to NVM
	do {
		//Read from file
		if (filename != NULL) {
			tcl_status = get_file_size(filename, &file_size);
			if (tcl_status != TCL_OK) {
				Tcl_AppendResult(interp, "File does not exist.\n", NULL);
				break;
			}

			// Check that image file size is not larger than NVM size
			nvm_size = common_nvm_nvm_size();

			if ((file_size > nvm_size) &&
				// for ATMEL - for every 256 bytes add another 8 bytes.
				(file_size != (nvm_size + (nvm_size / 256) * 8))) {
				Tcl_AppendResult(interp, "Failed to program NVM!! Image file size is larger than NVM size!!", NULL);
				break;
			}

			// Allocate buffer according to the file_size, round up for multiple of 4 bytes
			// plus 4 bytes saved for CRC.
			buf = (u8_t *)Tcl_Alloc(((file_size + 3) & 0xfffffffc) + 4);
			if (buf == NULL) {
				Tcl_AppendResult(interp, "No more host memory.\n", NULL);
				break;
			}
			// Clear the buffer
			common_nvm_memset(buf, 0, ((file_size + 3) & 0xfffffffc) + 4);

			tcl_status = read_bin_file(interp, filename, buf, file_size, &byte_read);
			if (tcl_status != TCL_OK) {
				Tcl_AppendResult(interp, "Can't read file.\n", NULL);
				break;
			}

			// Strip 8 extra bytes at end of each block if image was created -a option (Atmel)
			if (atmel_8B_strip(interp, buf, byte_read, &bytes_written) == TCL_ERROR) {
				Tcl_Free((char *)buf);
				Tcl_AppendResult(interp, "Failed to strip 8 extra bytes per block\n", NULL);
				break;
			} else byte_read = bytes_written;

			if (byte_cnt == 0) {
				byte_cnt = byte_read;
			} else {
				byte_cnt = (byte_cnt < byte_read) ? byte_cnt : byte_read;
			}

			/* Check if trailer exist in the last dword of the input file */
			memcpy((u8_t *)&trailer, (u8_t *)&buf[byte_cnt - 4], sizeof(u32_t));
			if (trailer == CRC_TRAILER) {
				/* Compute CRC */
				crc32 = 0xffffffff;
				common_nvm_compute_crc((u8_t *)buf, byte_cnt - 8, PRG_NVM_LITTLE_ENDIAN, &crc32);
				memcpy((u8_t *)&in_crc, (u8_t *)&buf[byte_cnt - 8], sizeof(u32_t));

				if (in_crc != crc32) {
					printf("CRC check failed !! in_crc 0x%x computed 0x%x byte_cnt %d\n", in_crc, crc32, byte_cnt - 8);
					return COMMON_NVM_STATUS_FAILURE;
				} else {
					printf("CRC check pass \n");
					/* Don't program CRC and trailer */
					byte_cnt -= 8;
				}
			}
		} else {  // Read from host memory
#ifndef FWUPGTOOL
			buf = (u8_t *)Tcl_Alloc(byte_cnt + 4);
			if (buf == NULL) {
				Tcl_AppendResult(interp, "No more host memory.\n", NULL);
				break;
			}

			// Copy from host memory to local buf
			memcpy(buf, (u8_t *)((int_ptr_t)LONG_ADDR(low_addr_val, high_addr_val)), byte_cnt);
#else
			return TCL_ERROR; // FWUPG does not support read from host memory...
#endif
		}

		/* Verify that the image to be programed has MFW/BC in bootstrap */
		if (offset == 0) {
			memcpy((u8_t *)&bootstrap, buf, sizeof(bootstrap));
			if ((ntohl(bootstrap.magic_value) != NVM_MAGIC_VALUE) ||
				(bootstrap.code_start_addr == 0) || (bootstrap.code_len == 0))	{
				Tcl_AppendResult(interp, "Check image includes BC/MFW failed.\n", NULL);
				return TCL_ERROR;
			}
		}
#ifndef FWUPGTOOL
		// This has done by FWUPG tool
		// Check VID DID
		if (id_match_enabled) {
			for (dev_idx = 1; dev_idx <= n_devices; dev_idx++) {
				if (BR_PCI_DEV_PRESENT(dev_idx - 1)) {
					DbgMessage(NULL, INFORM, "tcl_al_nvm_prg_cmd check VID DID ON dev_idx %d\n", dev_idx);
					common_nvm_get_nvm_offset_for_pci_id(&pci_id_offset, &pci_sub_id_offset);
					pci_id = *((u32_t *)&buf[pci_id_offset]);
					pci_sub_id = *((u32_t *)&buf[pci_sub_id_offset]);

					if (check_vid_did(interp, pci_id, pci_sub_id) != TCL_OK) {
						Tcl_AppendResult(interp, "Check VID/DID failed.\n", NULL);
						return TCL_ERROR;
					}
				}
			}
		}
#endif // !FWUPGTOOL
		if (prg_raw) {
			/* Incase use requested "raw" nvm programming:
			 * turn on -force_mac, -force_ibcfg, -force_febcfg, -force_lic
			 * turn off -skip_ec, -skip_sn, -skip_lic, -skip_config*/
			flag |= PRG_NVM_MAC_PRG;
			flag |= PRG_NVM_IBCFG;
			flag |= PRG_NVM_FEBCFG;
			flag |= PRG_NVM_LIC_PRG;
			flag |= PRG_NVM_NIV;

			flag &= ~PRESERVE_NVM_EC;
			flag &= ~PRESERVE_NVM_SN;
			flag &= ~PRG_NVM_SKIP_CFG_PRG;
		}

		common_nvm_read_nvram(NVM_OFFSET(bootstrap.magic_value), &tmp_val, sizeof(u32_t));
		if (tmp_val !=  NVM_MAGIC_VALUE) {
			printf("Detected a blank board.\n");
		}

		/* from prg command we need to set the flag of special handling of mac
		   partition image. Special handling means remembering old MAC
		   addresses if requested by user (not using -force_mac flag )*/
		flag |= PRG_HANDLE_MACP;

		if (skip_options) {
			// Do not enable -skip_options in case of -raw
			if (prg_raw) {
				print_msg("The -skip_options can not be used along with -raw mode.\n.", ESC_COLOR_RED);
				return TCL_ERROR;
			}

			// Save the options numbers user wants to skip
			strcpy(tmp_str, options_to_skip_str);
			token_ptr = strtok(options_to_skip_str, delimiters);
			options_to_skip[i++] = atoi(token_ptr);
			if ((*token_ptr < '1') || (*token_ptr > '9')) {
				print_msg("Invalid character in NVM configuration options to skip\n.", ESC_COLOR_RED);
				return TCL_ERROR;
			}
			while (token_ptr != NULL) {
				token_ptr = strtok(NULL, delimiters);
				if (token_ptr == NULL) break;
				if ((*token_ptr < '1') || (*token_ptr > '9')) {
					print_msg("Invalid character in NVM configuration options to skip.\n", ESC_COLOR_RED);
					return TCL_ERROR;
				}
				options_to_skip[i++] = atoi(token_ptr);
			}
			printf("Skip NVM configuration options %s while programming ..\n", tmp_str);

			cur_bus = atoi(Tcl_GetVar2(interp, TCL_ENV, ENV_BUS, TCL_GLOBAL_ONLY));
			cur_dev_idx = atoi(Tcl_GetVar2(interp, TCL_ENV, ENV_DEV, TCL_GLOBAL_ONLY));

			// Go through all the devices with the same bus number
			for (dev_n = cur_dev_idx; dev_n <= common_nvm_get_total_br_dev(); dev_n++) {
				execute_device(interp, dev_n);
				bus = atoi(Tcl_GetVar2(interp, TCL_ENV, ENV_BUS, TCL_GLOBAL_ONLY));
				if (bus == cur_bus) {
					strcpy(tmp_cmd, "nvm cfg ");
					// Save the original values of the options to skip
					for (j = 0; j < i; j++) {
						nvm_cfg_get(interp, options_to_skip[j]);
						res_ptr = Tcl_GetString(Tcl_GetObjResult(interp));
						if ((strstr(res_ptr, "no such element in array") != NULL) ||
							(strstr(res_ptr, "Does not exist") != NULL)) {
							print_msg("Attempt to skip an invalid option in NVM configuration.\n", ESC_COLOR_RED);
							return TCL_ERROR;
						}
						sprintf(tmp_str,  "%d=%s ", options_to_skip[j], res_ptr);
						strcat(tmp_cmd, tmp_str);
					}
					// Save the command in the cmd_for_dev[] array
					cmd_for_dev[idx] = (char *)(uintptr_t)common_nvm_alloc_virt(strlen(tmp_cmd));
					strcpy(cmd_for_dev[idx], tmp_cmd);
					DbgMessage(NULL, WARN, "command for dev %d = %s\n", dev_n, cmd_for_dev[idx]);
					idx++;
				} else {
					break;
				}
			}
			// Return to the current device
			execute_device(interp, cur_dev_idx);

		}

#ifdef FWUPGTOOL
#if defined(BIG_ENDIAN) && !defined(BCM_LITTLE_ENDIAN_HOST)
		// Data in buf is in BE, but the host is BE too...
		// The following setting will make the program_nvm(0
		// to work correctly in a BE host.
		flag &= ~PRG_NVM_BIG_ENDIAN;
#endif
#endif

		// Program NVM
		Tcl_AppendResult(interp, "Programming NVM ...\n", NULL);
		tcl_status = (common_nvm_program_nvm(buf,
											 byte_cnt,
											 offset,
											 flag,
											 NULL) == COMMON_NVM_STATUS_SUCCESS) ? TCL_OK : TCL_ERROR;
		if (tcl_status != TCL_OK) {
			Tcl_AppendResult(interp, "Programming NVM failed.\n", NULL);
			break;
		} else {
			add_shadow_dell_ncsi_image();
			add_bdn_image();

			// program ended succesfully - check if there is a need to manually fix shmem base register
			common_nvm_fix_shmem_offset_for_older_versions();

			// To avoid failure when upgrading from BC5.0.13 to BC6.2.10
			// "trick" the BC to think there was a POR reset
#ifndef FWUPGTOOL
			common_nvm_reg_write(MISC_REG_GENERIC_POR_0, 1);
#else
			if (iNvm.m_td_write_reg32(iNvm.m_current_dev_index, MISC_REG_GENERIC_POR_0, 1) != TCL_OK) {
				// final_status is initialized to TCL_ERROR...
				Tcl_AppendResult(interp, "Failed to set MISC_REG_GENERIC_POR_0 register.\n", NULL);
				break;
			}
#endif // !FWUPGTOOL
			Tcl_AppendResult(interp, "info burned into nvm successfully.\n", NULL);
		}

		final_status = TCL_OK;
#ifndef FWUPGTOOL
		if (iffe_enabled) {
			/* Disable IFFE before running nictest, to allow testing all 4 ports */
			print_msg("Disable IFFE feature.\n", ESC_COLOR_CYAN);
			nvm_cfg(interp, IFFE_OPTION, 0);
		}

		// For FWUPG tool, we don't issue the MCP reset and update the device information.
#ifndef WIN_DIAG
		if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
			//  Update device information of all devices and environment MAC addr.
			common_nvm_refresh_devices();
		} else {
			Tcl_AppendResult(interp, "Please power-cycle the target machine.\n", NULL);
		}
#endif
#endif // !FWUPGTOOL

	} while (0);

	/* Check if VPD image exist and set the IS_VPD environment variable */
	sprintf(tmp_str, "%d", is_vpd_image_exist());
	Tcl_SetVar2(interp, TCL_ENV, ENV_IS_VPD, tmp_str, TCL_GLOBAL_ONLY);

	if (buf != NULL) {
		Tcl_Free((char *)buf);
	}

	if (skip_options) {
		idx = 0;
		// Go through all the devices with the same bus number
		for (dev_n = cur_dev_idx; dev_n <= common_nvm_get_total_br_dev(); dev_n++) {
			execute_device(interp, dev_n);
			bus = atoi(Tcl_GetVar2(interp, TCL_ENV, ENV_BUS, TCL_GLOBAL_ONLY));
			if (bus == cur_bus) {
				DbgMessage(NULL, WARN, "Run %s on device %d\n", cmd_for_dev[idx], dev_n);

				// Run the "nvm cfg" command to restore the original options that were skipped.
				Tcl_Eval(interp, cmd_for_dev[idx]);
				common_nvm_free_virt((common_nvm_virt_addr_t)(uintptr_t)cmd_for_dev[idx]);
				idx++;
			} else {
				break;
			}
		}
		execute_device(interp, cur_dev_idx);
	}
	return final_status;
}

#ifndef FWUPGTOOL
/*
 *------------------------------------------------------------------------
 * toe_nvm_show_cmd --
 *
 *     Implements the new Tcl "nvm show" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
static int
tcl_al_nvm_show_cmd(ClientData client_data, Tcl_Interp *interp,
					int objc, Tcl_Obj *CONST objv[]) {
	u32_t *ptr, maxcount, i, tmp_addr;
	int addr, bytecount;
	common_nvm_status_t status;
	Tcl_Obj *result_ptr = NULL;

	if (objc < 1 || objc > 3) {
		Tcl_WrongNumArgs(interp, 0, objv, "[[<addr> [<nbytes>]]]");
		return TCL_ERROR;
	}

	switch (objc) {
	case 1:
		addr = (old_offset + old_length) & 0xfffffffc;
		bytecount = old_length;
		break;
	case 2:
		if (Tcl_GetIntFromObj(interp, objv[1], (int *)&addr) == TCL_ERROR) {
			Tcl_WrongNumArgs(interp, 2, objv, "[[<addr> [<nbytes>]]]");
			return TCL_ERROR;
		}
		bytecount = 4;
		break;
	case 3:
		if (Tcl_GetIntFromObj(interp, objv[1], (int *)&addr) == TCL_ERROR) {
			Tcl_WrongNumArgs(interp, 2, objv, "[[<addr> [<nbytes>]]]");
			return TCL_ERROR;
		}
		if (Tcl_GetIntFromObj(interp, objv[2], (int *)&bytecount) == TCL_ERROR) {
			Tcl_WrongNumArgs(interp, 3, objv, "[[<addr> [<nbytes>]]]");
			return TCL_ERROR;
		}
		addr &= 0xfffffffc;

		/* Set the cap on how much to dump */
		maxcount = MAX_PRINTF_BUFFER / CHARS_PER_LINE * 32;
		if (DISPLAY_IN_BYTES(interp)) maxcount /= 2;

		if ((u32_t)bytecount > maxcount) {
			printf("Bytecount exceeded, only 0x%x bytes will be displayed\n", maxcount);
			bytecount = maxcount;
		}
		bytecount = (bytecount + 3) & 0xfffffffc;
		break;
	default:
		Tcl_SetObjResult(interp, Tcl_NewStringObj("Uncaught error", 0));
		return TCL_ERROR;
	}

	/* Verify <addr> field is larger than zero (CQ# 32209)*/
	result_ptr = Tcl_GetObjResult(interp);
	if (addr < 0) {
		Tcl_AppendStringsToObj(result_ptr, "Bad address, must be larger than 0",
							   (char *)NULL);
		return TCL_ERROR;
	}
	/* Verify <byte_count> field is larger than zero (CQ# 32209)*/
	if (bytecount < 0) {
		Tcl_AppendStringsToObj(result_ptr, "Bad nbytes, must be larger than 0",
							   (char *)NULL);
		return TCL_ERROR;
	}

	ptr = (u32_t *)Tcl_Alloc(bytecount);
	for (i = 0, tmp_addr = addr; i < (u32_t)bytecount / 4; i++, tmp_addr += 4) {
		status = common_nvm_read_nvram(tmp_addr, &ptr[i], sizeof(u32_t));
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			Tcl_SetObjResult(interp, Tcl_NewStringObj("Bad device or address", 0));
			Tcl_Free((char *)ptr);
			return TCL_ERROR;
		}
	}

	if (DISPLAY_IN_BYTES(interp)) {
		disp_8bit_array((u8_t *)ptr, addr, bytecount);
	} else {
		disp_32bit_array(ptr, addr, bytecount / 4, 0);
	}
	Tcl_Free((char *)ptr);

	/* Store the address and length for the next command */
	old_offset = addr;
	old_length = bytecount;

	/* Remove the list from result_ptr */
	Tcl_SetObjResult(interp, Tcl_NewStringObj("", 0));

	return TCL_OK;
}
#endif // !FWUPGTOOL

#ifndef FWUPGTOOL
/*
 *------------------------------------------------------------------------
 * toe_nvm_write_cmd --
 *
 *     Implements the new Tcl "nvm write" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
static int
tcl_al_nvm_write_cmd(ClientData client_data, Tcl_Interp *interp,
					 int objc, Tcl_Obj *CONST objv[]) {
	u32_t value32;
	u8_t value8;
	int i, j, count = 0, addr;
	Tcl_Obj * result_ptr,**vector;
	common_nvm_status_t status;

	if (objc != 3) {
		Tcl_WrongNumArgs(interp, objc, objv, "<addr> <value32>|<byte list>");
		return TCL_ERROR;
	}

	result_ptr = Tcl_GetObjResult(interp);


	/* Get register offset */
	if (Tcl_GetIntFromObj(interp, objv[1], (int *)&addr) == TCL_ERROR) {
		goto error;
	}

	/* Verify <addr> field is larger than zero (CQ# 32209)*/
	if (addr < 0) {
		Tcl_AppendStringsToObj(result_ptr, "Bad address, must be larger than 0",
							   (char *)NULL);
		return TCL_ERROR;
	}

	/* Verify <addr> field is not larger than nvm size */
	if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
		if ((u32_t)addr > common_nvm_nvm_size()) {
			Tcl_AppendStringsToObj(result_ptr, "Bad address, larger than NVM size",
								   (char *)NULL);
			return TCL_ERROR;
		}
	}

	if (Tcl_GetIntFromObj(interp, objv[2], (int *)&value32) == TCL_OK) {
		/* Write only one dword */
		status = common_nvm_write_nvram(addr & 0xfffffffc, &value32, sizeof(value32));
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			Tcl_AppendStringsToObj(result_ptr, "Bad device or address",
								   (char *)NULL);
			goto error;
		} else {
			DbgMessage(NULL, VERBOSEnv,
					   "common_nvm_write_nvram returned success\n");
		}
	} else if (Tcl_ListObjGetElements(interp, objv[2], &count, &vector) == TCL_OK
			   && (count > 1)) {
		/* objv[2] is a list; write one or more dwords */
		u32_t mask[4] = { 0xffffff, 0xff00ffff, 0xffff00ff, 0xffffff00 };
		for (i = 0; i < count;) {
			u32_t tmp_addr = addr & 0xfffffffc;
			/* Doing read-modify-write for every byte */
			status = common_nvm_read_nvram(tmp_addr, &value32, sizeof(value32));
			if (status != COMMON_NVM_STATUS_SUCCESS) break;   /* the subsequent read & write would fail anyway */

			for (j = addr & 0x3; j < 4 && i < count; i++) {
				if (strchr(Tcl_GetString(vector[i]), ':')) {
					continue;    /* Skip anything with ':' */
				}
				if (Tcl_GetIntFromObj(interp, vector[i], (int *)&value8)
					== TCL_OK) {
					value32 = (value32 & mask[j]) | (value8 << (3 - j) * 8);
				}
				j++;
				addr++;
				/* Don't do write if it's NaN */
			}
			status = common_nvm_write_nvram(tmp_addr, &value32, sizeof(value32));
		}
	} else {
		goto error;
	}

	Tcl_SetIntObj(result_ptr, 1);
	return TCL_OK;

error:
	Tcl_SetIntObj(result_ptr, 0
				 );
	return TCL_ERROR;
}
#endif // !FWUPGTOOL

#ifndef FWUPGTOOL
/*
 *------------------------------------------------------------------------
 * toe_nvm_fill_cmd --
 *
 *     Implements the new Tcl "nvm fill" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
static int
tcl_al_nvm_fill_cmd(ClientData client_data, Tcl_Interp *interp,
					int objc, Tcl_Obj *CONST objv[]) {
	u32_t byte_prg, *ptr32;
	u8_t *buf;
	char *argv3;
	int idx;
	int addr, byte_cnt = 0, value32;
	Tcl_Obj *result_ptr;
	common_nvm_status_t status;
	char tmp_str1[20], tmp_str2[20];
	enum {
		PTTN_INC, PTTN_ADDR, PTTN_FIX
	} pattern;

	if (objc != 4) {
		Tcl_WrongNumArgs(interp, 1, objv,
						 "addr bytecount dword_pttrn");
		return TCL_ERROR;
	}

	result_ptr = Tcl_GetObjResult(interp);


	/* Get memory offset */
	if (Tcl_GetIntFromObj(interp, objv[1], (int *)&addr) == TCL_ERROR) {
		Tcl_AppendStringsToObj(result_ptr, ": Bad address",
							   (char *)NULL);
		return TCL_ERROR;
	}

	if (Tcl_GetIntFromObj(interp, objv[2], (int *)&byte_cnt) == TCL_ERROR) {
		Tcl_AppendStringsToObj(result_ptr, ": Bad byte_cnt",
							   (char *)NULL);
		return TCL_ERROR;
	}

	argv3 = Tcl_GetString(objv[3]);
	if (strstr(argv3, NVM_FILL_PTTN_INC)) {
		pattern = PTTN_INC;
	} else if (strstr(argv3, NVM_FILL_PTTN_ADDR)) {
		pattern = PTTN_ADDR;
	} else if (Tcl_GetIntFromObj(interp, objv[3], (int *)&value32) == TCL_OK) {
		pattern = PTTN_FIX;
	} else {
		Tcl_AppendStringsToObj(result_ptr, ": Bad value or pattern",
							   (char *)NULL);
		return TCL_ERROR;
	}

	/* Verify that byte cnt and address are larger than 0 */
	if (byte_cnt < 0) {
		Tcl_AppendStringsToObj(result_ptr, "Bad byte_cnt, must be larger then 0",
							   (char *)NULL);
		return TCL_ERROR;
	}

	/* Check that requested byte_cnt does not exceed the NVM size */
	if ((u32_t)(addr + byte_cnt) > common_nvm_nvm_size()) {
		Tcl_AppendStringsToObj(result_ptr, "Bad address or byte_cnt, attemp to access address larger than NVM size",
							   (char *)NULL);
		return TCL_ERROR;
	}

	if (addr < 0) {
		Tcl_AppendStringsToObj(result_ptr, "Bad address, must be larger then 0",
							   (char *)NULL);
		return TCL_ERROR;
	}

	/* Actual filling of memory */
	byte_cnt = ((byte_cnt + 3) / 4) * 4;
	addr &= 0xfffffffc;

	buf = (u8_t *)Tcl_Alloc(byte_cnt);
	ptr32 = (u32_t *)buf;
	for (idx = 0; idx < byte_cnt; idx += 4, ptr32++) {
		switch (pattern) {
		case PTTN_ADDR:
			value32 = addr + idx;
			break;
		case PTTN_INC:
			value32 = idx / 4;
			break;
		case PTTN_FIX:
		default:
			/* Do nothing, value32 already has the correct value */
			break;
		}
		*ptr32 = value32;
	}

	sprintf(tmp_str1, "0x%08x", addr);
	sprintf(tmp_str2, "0x%08x", addr + byte_cnt);
	Tcl_AppendStringsToObj(result_ptr, "Programming from: ", tmp_str1, " to ", tmp_str2, "\n", NULL);
	tcl_io_flush();
	status = common_nvm_program_nvm(buf, byte_cnt, addr,
									PRG_NVM_LITTLE_ENDIAN | PRG_NVM_MAC_PRG | PRG_NVM_LIC_PRG |
										PRG_NVM_IBCFG |  PRG_NVM_FEBCFG | PRG_NVM_NIV,
									&byte_prg);
#ifndef FWUPGTOOL
	// For FWUPG tool, we don't issue the MCP reset and update the device information.
#ifndef WIN_DIAG
	// Update device information of all devices and environment MAC addr.
	if (status == COMMON_NVM_STATUS_SUCCESS) common_nvm_refresh_devices();
#endif
#endif // !FWUPGTOOL
	Tcl_Free((char *)buf);

	/* Check if fill completes */
	if ((status != COMMON_NVM_STATUS_SUCCESS) || (byte_cnt != byte_prg)) {
		Tcl_AppendStringsToObj(result_ptr, "Fill fails",
							   (char *)NULL);
	} else {
		Tcl_AppendStringsToObj(result_ptr, "done",
							   (char *)NULL);
	}
	return status;
}
#endif // !FWUPGTOOL


static int validate_chip(Tcl_Interp *interp, u32_t image_info) {
	u32_t type_required;
	u32_t type_of_image = image_info & IMAGE_INFO_CHIP_MASK;
	u32_t device_id;
	Tcl_Obj *objptr;

	if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
		/* extract device id */
		objptr = Tcl_GetVar2Ex(interp, TCL_ENV, ENV_DID, 0);
		Tcl_GetIntFromObj(interp, objptr, (int *)&device_id);

		if (device_id == 0) {
			/* extract device id from default_did env var */
			objptr = Tcl_GetVar2Ex(interp, TCL_ENV, "DEFAULT_DID", 0);
			Tcl_GetIntFromObj(interp, objptr, (int *)&device_id);
		}
	} else {
		get_serport_chip_id(&device_id);
	}

	if (is_e1(interp)) {
		type_required = IMAGE_INFO_CHIP_57710;
	} else if (is_e1h(interp)) {
		type_required = IMAGE_INFO_CHIP_57711;
	} else if (is_e2(interp)) {
		type_required = IMAGE_INFO_CHIP_57712;
	} else if (is_e3(interp)) {
		type_required = IMAGE_INFO_CHIP_57840;
	} else {
		Tcl_AppendResult(interp, "Unknown chip (", Tcl_GetVar2(interp, TCL_ENV, ENV_DID, 0), ").\n", NULL);
		return TCL_ERROR;
	}

	if (type_of_image == 0) {
		// image does not include chip information at all
		// allow upgrade for backward compatibility of old images but issue a warning
		//printf ("Warning: Image header does not include chip number.\n");
		return TCL_OK;
	} else {
		// Check that type_of_image and type required bitwise match
		if ((type_of_image & type_required) == 0) {
			printf("Warning: File includes image not for device %s (will be skipped).\n", Tcl_GetVar2(interp, TCL_ENV, ENV_CHIP_NUM, 0));
			return TCL_ERROR;
		}

	}
	return TCL_OK;
}

/**
 * validate_image_type -
 * verify image type is as expecetd for this option (e.g. BC type for "-bc" option)
 *
 * image_type should be in little endian format.
 *
 */
static int validate_image_type(Tcl_Interp *interp, char *option, u32_t image_type) {

	if (strncmp(option, "-bc", 3) == 0) {
		/* Check if the image is for boot code in general */
		if ((image_type != IMAGE_HDR_TYPE_BC1) &&
			(image_type != IMAGE_HDR_TYPE_BC2) &&
			(image_type != IMAGE_HDR_TYPE_E3_WC) &&
			(image_type != IMAGE_HDR_TYPE_E3_PCIE) &&
			(image_type != IMAGE_HDR_TYPE_E3_WCV2) &&
			(image_type != IMAGE_HDR_TYPE_E3_PCIEV2) &&
			(image_type != IMAGE_HDR_TYPE_OCNVM)) goto error_type;
	} else if (strcmp(option, "-ipmi") == 0) {
		/* Check if the image is for IPMI firmware in general */
		if (image_type != IMAGE_HDR_TYPE_IPMI) goto error_type;
	} else if (strcmp(option, "-ncsi") == 0) {
		/* Check if the image is for NCSI firmware in general */
		if ((image_type != IMAGE_HDR_TYPE_NCSI_CMN) &&
			(image_type != IMAGE_HDR_TYPE_NCSI_LIB_XI) &&
			(image_type != IMAGE_HDR_TYPE_NCSI_LIB_EV)) goto error_type;
	} else if ((strncmp(option, "-ib", 3) == 0) || (strncmp(option, "-ib_ipv6", 8) == 0) ||
			   (strncmp(option, "-ib_ipv4n6", 10) == 0)) {
		/* Check if the image is for iSCSI firmware in general */
		if ((image_type != IMAGE_HDR_TYPE_ISCSI_BOOT) &&
			(image_type != IMAGE_HDR_TYPE_ISCSI_BOOT_IPV6) &&
			(image_type != IMAGE_HDR_TYPE_ISCSI_BOOT_IPV4N6) &&
			(image_type != IMAGE_HDR_TYPE_ISCSI_BOOT_CFG_V2) &&
			(image_type != IMAGE_HDR_TYPE_ISCSI_BOOT_CFG) &&
			(image_type != IMAGE_HDR_TYPE_ISCSI_BOOT_CPRG) &&
			(image_type != IMAGE_HDR_TYPE_ISCSI_BOOT_CFG_PRG_LGCY) &&
			(image_type != IMAGE_HDR_TYPE_ISCSI_BOOT_CPRG_LGCY)) goto error_type;
	} else if (strcmp(option, "-mba") == 0) {
		/* Check if the image is for MBA */
		if (image_type != IMAGE_HDR_TYPE_MBA) goto error_type;
	} else if (strcmp(option, "-feb") == 0) {
		/* Check FCOE BOOT image type */
		if ((image_type != IMAGE_HDR_TYPE_FCOE_BOOT) &&
			(image_type != IMAGE_HDR_TYPE_FCOE_BOOT_CFG) &&
			(image_type != IMAGE_HDR_TYPE_FCOE_BOOT_CPRG) &&
			(image_type != IMAGE_HDR_TYPE_FCOE_BOOT_CPRG_LGCY) &&
			(image_type != IMAGE_HDR_TYPE_FCOE_BOOT_CPRG_EVRST)) goto error_type;
	} else if (strcmp(option, "-nicp") == 0) {
		/* Check FCOE BOOT image type */
		if (image_type != IMAGE_HDR_TYPE_NIC_PARTITION_CFG) goto error_type;
	} else if (strncmp(option, "-mfw", 4) == 0) {
		/* Check MFWx, SWIMx image type */
		if ((image_type != IMAGE_HDR_TYPE_MFW1) &&
			(image_type != IMAGE_HDR_TYPE_MFW2) &&
			(image_type != IMAGE_HDR_TYPE_MFW2_A) &&
			(image_type != IMAGE_HDR_TYPE_SWIM1) &&
			(image_type != IMAGE_HDR_TYPE_SWIM2) &&
			(image_type != IMAGE_HDR_TYPE_SWIM3) &&
			(image_type != IMAGE_HDR_TYPE_SWIM4) &&
			(image_type != IMAGE_HDR_TYPE_SWIM5) &&
			(image_type != IMAGE_HDR_TYPE_SWIM6) &&
			(image_type != IMAGE_HDR_TYPE_SWIM7) &&
			(image_type != IMAGE_HDR_TYPE_SWIM8) &&
			(image_type != IMAGE_HDR_TYPE_E3_WC) &&
			(image_type != IMAGE_HDR_TYPE_E3_PCIE) &&
			(image_type != IMAGE_HDR_TYPE_E3_WCV2) &&
			(image_type != IMAGE_HDR_TYPE_E3_PCIEV2) &&
			(image_type != IMAGE_HDR_TYPE_OCNVM) &&
			(image_type != IMAGE_HDR_TYPE_MDUMP)) goto error_type;
	} else if (strcmp(option, "-ccm") == 0) {
		if (image_type != IMAGE_HDR_TYPE_CCM) goto error_type;
	} else if (strcmp(option, "-bootsh") == 0) {
		if (image_type != IMAGE_HDR_TYPE_BOOT_CFG_SHADOW) goto error_type;
	} else if (strcmp(option, "-ipers") == 0) {
		if (image_type != IMAGE_HDR_TYPE_ISCSI_PERS) goto error_type;
	} else if (strcmp(option, "-mdump") == 0) {
		if (image_type != IMAGE_HDR_TYPE_MDUMP) goto error_type;
	} else {
		/* Flag an error, this file image is not intended
		 * other purposes.
		 */
		goto error_type;
	}
	DbgMessage(NULL, INFORM, "INFO: validate_image_type 0x%x succeeded\n", image_type);
	return TCL_OK;

error_type:
	DbgMessage(NULL, FATAL, "validate_image_type 0x%x for option %s failed !!\n", image_type, option);
	Tcl_AppendResult(interp, "Improper file format, ", "Not intended for ", option, " option.\n", NULL);
	return TCL_ERROR;
}

/*
 *------------------------------------------------------------------------
* verify_NCSI_images --
*
*     checks that common and everest NCSI images exist in the image.
*
* Results:
*     A standard Tcl result
*
* Side effects:
*     None.
*------------------------------------------------------------------------
*/
static int verify_NCSI_images(
	Tcl_Interp *interp,
	u32_t byte_read,
	u8_t *buf
	) {

	image_header_t *file_image;
	u32_t idx, total = 0;
	u32_t image_tbl_sz = common_nvm_image_table_size();

	u8_t common_exists = 0;
	u8_t elib_exists = 0;
	u8_t xlib_exists = 0;

	while (total < byte_read) {
		file_image = (image_header_t *)&buf[total];
		if (BMAPI_BENDIAN_SWAP(file_image->magic) != FILE_MAGIC) {
			Tcl_AppendResult(interp, "Improper file format, ",
							 "missing magic value.\n", NULL);
			return TCL_ERROR;
		}

		if ((BMAPI_BENDIAN_SWAP(file_image->version) == FORMAT_VERSION_1) ||
			(BMAPI_BENDIAN_SWAP(file_image->version) == FORMAT_VERSION_2)) {
			for (idx = 0; idx < image_tbl_sz; idx++) {

				if (BMAPI_BENDIAN_SWAP(file_image->type) == IMAGE_HDR_TYPE_NCSI_CMN) {
					common_exists = 1;
					break;
				}
				if (BMAPI_BENDIAN_SWAP(file_image->type) == IMAGE_HDR_TYPE_NCSI_LIB_XI) {
					xlib_exists = 1;
					break;
				}
				if (BMAPI_BENDIAN_SWAP(file_image->type) == IMAGE_HDR_TYPE_NCSI_LIB_EV) {
					elib_exists = 1;
					break;
				}
			}
			if (idx == image_tbl_sz) {
				Tcl_AppendResult(interp, "Improper file format, ",
								 "incorrect file type.\n", NULL);
				return TCL_ERROR;
			}
		} else {
			Tcl_AppendResult(interp, "Improper file format, ",
							 "incorrect format version.\n", NULL);
			return TCL_ERROR;
		}

		total += sizeof(image_header_t) +  BMAPI_BENDIAN_SWAP(file_image->byte_cnt);

	}
	printf("NCSI images found:  ");
	if (common_exists) printf("common,  ");
	if (elib_exists) printf("everest lib,  ");
	if (xlib_exists) printf("xinan lib (will be skipped)  ");
	printf("\n");

	if (common_exists == 0) {
		Tcl_AppendResult(interp, "NCSI common is missing.",
						 "cannot burn image on Everest NVM.\n", NULL);
		return TCL_ERROR;
	}
	return TCL_OK;
}

/*
*------------------------------------------------------------------------
* verify_mba_image --
*
*     checks that MBA image includes signature
*
* Results:
*     A standard Tcl result
*
* Side effects:
*     None.
*------------------------------------------------------------------------
*/
static int verify_mba_image(
	Tcl_Interp *interp,
	u8_t *buf
	) {
	exp_rom_hdr_t *exp_rom_ptr;
	vendor_data_t *v_data_ptr;
	u16_t           flags;

	exp_rom_ptr = (exp_rom_hdr_t *)buf;
	v_data_ptr = (vendor_data_t *)(&buf[BMAPI_BENDIAN_SWAP_S(exp_rom_ptr->VendorOffset)]);

	/* Check for header signature */
	if (BMAPI_BENDIAN_SWAP_S(exp_rom_ptr->Signature) != ROM_HEADER_SIG) {
		return TCL_ERROR;
	}

	/* validate image is for this chip */
	flags = BMAPI_LENDIAN_SWAP_S(v_data_ptr->Flags);

	DbgMessage(NULL, WARN, "verify_mba_image: v_data_ptr->Flags =0x%x, flags =0x%x\n", v_data_ptr->Flags, flags);

	if (flags & VF_LOM) {

		if (is_e1x(interp)) {
			if ((flags & VF_NX2_DEV_SUPPORT_MASK) == VF_NX2_SUPPORT_ALL_DEVICES) {
				return TCL_OK;
			} else {
				Tcl_AppendResult(interp, "This MBA image is not for this device type.\n", NULL);
				return TCL_ERROR;
			}
		}

		if (is_e2(interp)) {
			/* If flags show E2 support or all devices support, return success */
			if (((flags & VF_NX2_E2_SUPPORT_ONLY) == VF_NX2_E2_SUPPORT_ONLY) ||
				((flags & VF_NX2_DEV_SUPPORT_MASK) == VF_NX2_SUPPORT_ALL_DEVICES)) {
				return TCL_OK;
			} else {
				Tcl_AppendResult(interp, "This MBA image is not for this device type.\n", NULL);
				return TCL_ERROR;
			}
		}

		if (is_e3(interp)) {
			/* If flags show E3 support or all devices support, return success */
			if (((flags & VF_NX2_E3_SUPPORT_ONLY) == VF_NX2_E3_SUPPORT_ONLY) ||
				((flags & VF_NX2_DEV_SUPPORT_MASK) == VF_NX2_SUPPORT_ALL_DEVICES)) {
				return TCL_OK;
			} else {
				Tcl_AppendResult(interp, "This MBA image is not for this device type.\n", NULL);
				return TCL_ERROR;
			}
		}
	}

	return TCL_OK;
}

/*
*------------------------------------------------------------------------
* get_mbi_version --
*
*     Read MBI image version and date from NVM
*
* Results:
*     A standard nvm result
*
* Side effects:
*     None.
*------------------------------------------------------------------------
*/
static common_nvm_status_t get_mbi_version(Tcl_Interp *interp, u32_t *mbi_version, u32_t *mbi_date) {
	u32_t mbi_version_offset, mbi_date_offset;
	u32_t found_image, nvm_addr, byte_cnt, entry_idx;
	common_nvm_status_t status;

	/* find mbi_version and mbi_date in nvram */
	status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &nvm_addr, &byte_cnt, &entry_idx);

	if ((status == COMMON_NVM_STATUS_SUCCESS) && found_image) {
		mbi_date_offset = OFFSETOF(struct extended_dev_info_shared_cfg, mbi_date);
		mbi_version_offset = OFFSETOF(struct extended_dev_info_shared_cfg, mbi_version);

		if (byte_cnt > mbi_date_offset) {
			common_nvm_read_nvram(nvm_addr + mbi_date_offset, mbi_date, sizeof(u32_t));
			common_nvm_read_nvram(nvm_addr + mbi_version_offset, mbi_version, sizeof(u32_t));
		} else {
			*mbi_date = 0;
			*mbi_version = 0;
			printf("MBI version and date were not configured. Setting to default.\n");
			Tcl_Eval(interp, "nvm cfg 244=0");
			Tcl_Eval(interp, "nvm cfg 243=00:00:00");
		}
	}
	return status;
}

/*
*------------------------------------------------------------------------
* set_mbi_version --
*
*     Write MBI image version and date to NVM
*
* Results:
*     A standard nvm result
*
* Side effects:
*     MBI version and date are changed in NVM.
*------------------------------------------------------------------------
*/
static common_nvm_status_t set_mbi_version(Tcl_Interp *interp, u32_t mbi_version, u32_t mbi_date) {
	u32_t mbi_version_offset, mbi_date_offset;
	u32_t found_image, nvm_addr, byte_cnt, entry_idx;
	common_nvm_status_t status;

	/* find mbi_version and mbi_date in nvram */
	status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &nvm_addr, &byte_cnt, &entry_idx);

	if ((status == COMMON_NVM_STATUS_SUCCESS) && found_image) {
		mbi_date_offset = OFFSETOF(struct extended_dev_info_shared_cfg, mbi_date);
		mbi_version_offset = OFFSETOF(struct extended_dev_info_shared_cfg, mbi_version);

		if (byte_cnt > mbi_date_offset) {
			common_nvm_write_nvram(nvm_addr + mbi_date_offset, &mbi_date, sizeof(u32_t));
			common_nvm_write_nvram(nvm_addr + mbi_version_offset, &mbi_version, sizeof(u32_t));
		} else {
			char buffer[30];
			printf("MBI version and date were not configured. Setting new values.\n");
			sprintf(buffer, "nvm cfg 244=0x%X", mbi_date);
			Tcl_Eval(interp, buffer);
			sprintf(buffer, "nvm cfg 243=%02d:%02d:%02d", ((mbi_version >> 24) & 0xff), ((mbi_version >> 16) & 0xff), ((mbi_version >> 8) & 0xff));
			Tcl_Eval(interp, buffer);
		}
	}
	return status;
}

/*
*------------------------------------------------------------------------
* verify_mbi_image --
*
*     checks that MBI image includes signature
*
* Results:
*     A standard Tcl result
*
* Side effects:
*     None.
*------------------------------------------------------------------------
*/
static int verify_mbi_image(Tcl_Interp *interp, nvm_upgrade_params_t *upgrade_params_ptr, u32_t *bitmask) {
	ILT_Header_t *ILT_Header_ptr;
	ILT_CardType_Entry_t *ILT_CardType_Entry_ptr;
	ILT_CardType_Header_t *ILT_CardType_Header_ptr;
	u32_t i;
	u8_t chip_rev;
	char err[100];
	u32_t mbi_version, mbi_date;

	*bitmask = 0;
	ILT_Header_ptr = (ILT_Header_t *)upgrade_params_ptr->buf;
	/* Check for header signature */
	if (strncmp((char *)ILT_Header_ptr->signature, "QILT", 4) != 0) {
		Tcl_AppendResult(interp, "This is not monolithic image.\n", NULL);
		return TCL_ERROR;
	}
	/* Check for header version */
	if (ILT_Header_ptr->tableVersion != 1) {
		sprintf(err, "This monolithic image version %d is not supported.\n", ILT_Header_ptr->tableVersion);
		Tcl_AppendResult(interp, err, NULL);
		return TCL_ERROR;
	}
	/* cardListExists != 0 */
	if (ILT_Header_ptr->cardListExists == 0) {
		Tcl_AppendResult(interp, "This monolithic image does not have supported PCI card list.\n", NULL);
		return TCL_ERROR;
	}
	/* numOfEntries > 0 */
	if (ILT_Header_ptr->numOfEntries == 0) {
		Tcl_AppendResult(interp, "This monolithic image does not have any images.\n", NULL);
		return TCL_ERROR;
	}
	/* if upgrade is not forced, validate that MBI version is newer */
	if (upgrade_params_ptr->validate_versions) {
		if (get_mbi_version(interp, &mbi_version, &mbi_date) != COMMON_NVM_STATUS_SUCCESS) {
			Tcl_AppendResult(interp, "Can't read MBI version from nvm.\n", NULL);
			return TCL_ERROR;
		}
		i = (ILT_Header_ptr->versionMajor << 24) + (ILT_Header_ptr->versionMinor << 16) + (ILT_Header_ptr->versionSubMinor << 8);
		if (mbi_version >= i) {
			Tcl_AppendResult(interp, "Given MBI version is same or newer than NVRAM version. Use -F to force upgrade.\n", NULL);
			return TCL_ERROR;
		}
	}
	/*	tableChecksum is correct */
	if (!HLPR_Validate_Sum16((u16_t *)ILT_Header_ptr, upgrade_params_ptr->byte_read / 2)) {
		sprintf(err, "This monolithic image checksum (0x%X) is wrong.\n", ILT_Header_ptr->tableChecksum);
		Tcl_AppendResult(interp, err, NULL);
		return TCL_ERROR;
	}

	ILT_CardType_Header_ptr = (ILT_CardType_Header_t *)(upgrade_params_ptr->buf + sizeof(ILT_Header_t) + (ILT_Header_ptr->numOfEntries * sizeof(ILT_Region_t)));
	/* NoPCI_IDEntries > 0 */
	if (ILT_CardType_Header_ptr->noCardTypeList == 0) {
		Tcl_AppendResult(interp, "This monolithic image does not support any PCI cards.\n", NULL);
		return TCL_ERROR;
	}

	/* Match card and revision */
	common_nvm_reg_read(MISC_REG_CHIP_REV, &i);
	chip_rev = (u8_t)(i & 0xff);
	ILT_CardType_Entry_ptr = (ILT_CardType_Entry_t *)((u8_t *)ILT_CardType_Header_ptr + sizeof(ILT_CardType_Header_t));
	for (i = 0; i < ILT_CardType_Header_ptr->noCardTypeList; i++) {
		if ((ILT_CardType_Entry_ptr->vendorID == upgrade_params_ptr->vendor_id) &&
			(ILT_CardType_Entry_ptr->deviceID == upgrade_params_ptr->device_id) &&
			(ILT_CardType_Entry_ptr->subSystemVID == upgrade_params_ptr->sub_vendor_id) &&
			(ILT_CardType_Entry_ptr->subSystemDID == upgrade_params_ptr->sub_device_id) &&
			((ILT_CardType_Entry_ptr->Rev == chip_rev) || (ILT_CardType_Entry_ptr->Rev == 0))) {
			*bitmask = ILT_CardType_Entry_ptr->SupportBitMask;
			return TCL_OK;
		}
		ILT_CardType_Entry_ptr++;
	}

	Tcl_AppendResult(interp, "This monolithic image does not support this machine's card.\n", NULL);
	return TCL_ERROR;
}

#ifndef FWUPGTOOL
// These functions were available in nvm.c.
// Therefore, exclude them for FWUPG Tool.
int
ib_migrate_cfgblk_v2_to_v1(
	pNVM_ISCSI_CFG_BLOCK_V2 iblk_v2,
	pNVM_ISCSI_CFG_BLOCK iblk_v1) {
	int ipv6;
	int len;

	memset(iblk_v1, 0, sizeof(NVM_ISCSI_CFG_BLOCK));

	ipv6 = (iblk_v2->gen.ctrl_flags & NVM_ISCSI_FLAGS_IPV6_SUPPORT) ? 1 : 0;

	/* Migrate general block */
	iblk_v1->gen.signature = HOST_TO_NET16(NVM_ISCSI_BLOCK_SIGNATURE);
	iblk_v1->gen.version = NVM_ISCSI_FORMAT_VERSION;
	iblk_v1->gen.ctrl_flags = iblk_v2->gen.ctrl_flags;

	/* Version 1 Cfg block doesn't support IPV6 */
	iblk_v1->gen.ctrl_flags &= ~NVM_ISCSI_FLAGS_IPV6_SUPPORT;

	strcpy(iblk_v1->gen.dhcp_vendor_id, iblk_v2->gen.dhcp_vendor_id);
	iblk_v1->gen.link_up_delay = iblk_v2->gen.link_up_delay;
	iblk_v1->gen.lun_busy_retry_count = iblk_v2->gen.lun_busy_retry_count;

	/* Migrate Secondary block */
	*(&iblk_v1->secondary_device) = *(&iblk_v2->secondary_device);

	/* Migrate Initiator block */
	if (!ipv6) {
		iblk_v1->initiator.ip_addr = iblk_v2->initiator.ip_addr.ipv4;
		iblk_v1->initiator.subnet_mask = iblk_v2->initiator.subnet_mask.ipv4;
		iblk_v1->initiator.gateway = iblk_v2->initiator.gateway.ipv4;
		iblk_v1->initiator.primary_dns = iblk_v2->initiator.primary_dns.ipv4;
		iblk_v1->initiator.secondary_dns = iblk_v2->initiator.secondary_dns.ipv4;
	}

	strcpy(iblk_v1->initiator.iscsi_name, iblk_v2->initiator.iscsi_name);

	if ((len = strlen(iblk_v2->initiator.chap_id)) > ISCSI_MAX_CHAP_ID_SLENGTH) len = ISCSI_MAX_CHAP_ID_SLENGTH;

	strncpy(iblk_v1->initiator.chap_id, iblk_v2->initiator.chap_id, len);
	strcpy(iblk_v1->initiator.chap_password, iblk_v2->initiator.chap_password);

	/* Migrate Target1 block */
	iblk_v1->target1.ctrl_flags = iblk_v2->target1.ctrl_flags;
	iblk_v1->target1.tcp_port = iblk_v2->target1.tcp_port;
	if (!ipv6) {
		iblk_v1->target1.ip_addr = iblk_v2->target1.ip_addr.ipv4;
	}

	iblk_v1->target1.boot_lun = iblk_v2->target1.boot_lun;

	if ((len = strlen(iblk_v2->target1.chap_id)) > ISCSI_MAX_CHAP_ID_SLENGTH) len = ISCSI_MAX_CHAP_ID_SLENGTH;
	strncpy(iblk_v1->target1.chap_id, iblk_v2->target1.chap_id, len);
	strcpy(iblk_v1->target1.chap_password, iblk_v2->target1.chap_password);
	strcpy(iblk_v1->target1.iscsi_name, iblk_v2->target1.iscsi_name);

	/* Migrate Target2 block */
	iblk_v1->target2.ctrl_flags = iblk_v2->target2.ctrl_flags;
	iblk_v1->target2.tcp_port = iblk_v2->target2.tcp_port;
	if (!ipv6) {
		iblk_v1->target2.ip_addr = iblk_v2->target2.ip_addr.ipv4;
	}

	iblk_v1->target2.boot_lun = iblk_v2->target2.boot_lun;
	if ((len = strlen(iblk_v2->target2.chap_id)) > ISCSI_MAX_CHAP_ID_SLENGTH) len = ISCSI_MAX_CHAP_ID_SLENGTH;
	strncpy(iblk_v1->target2.chap_id, iblk_v2->target2.chap_id, len);
	strcpy(iblk_v1->target2.chap_password, iblk_v2->target2.chap_password);
	strcpy(iblk_v1->target2.iscsi_name, iblk_v2->target2.iscsi_name);

	return 0;
}


int
ib_migrate_cfgblk_v1_to_v2(
	pNVM_ISCSI_CFG_BLOCK iblk_v1,
	pNVM_ISCSI_CFG_BLOCK_V2 iblk_v2,
	int ipv6) {
	memset(iblk_v2, 0, sizeof(NVM_ISCSI_CFG_BLOCK_V2));

	/* Migrate general block */
	iblk_v2->gen.signature = HOST_TO_NET16(NVM_ISCSI_BLOCK_SIGNATURE);
	iblk_v2->gen.version = NVM_ISCSI_FORMAT_VERSION_2;
	iblk_v2->gen.ctrl_flags = iblk_v1->gen.ctrl_flags;
	if (ipv6) iblk_v2->gen.ctrl_flags |= NVM_ISCSI_FLAGS_IPV6_SUPPORT;

	strcpy(iblk_v2->gen.dhcp_vendor_id, iblk_v1->gen.dhcp_vendor_id);
	iblk_v2->gen.link_up_delay = iblk_v1->gen.link_up_delay;
	iblk_v2->gen.lun_busy_retry_count = iblk_v1->gen.lun_busy_retry_count;

	/* Migrate Secondary block */
	*(&iblk_v2->secondary_device) = *(&iblk_v1->secondary_device);

	/* Migrate Initiator block */
	if (!ipv6) {
		iblk_v2->initiator.ip_addr.ipv4 = iblk_v1->initiator.ip_addr;
		iblk_v2->initiator.subnet_mask.ipv4 = iblk_v1->initiator.subnet_mask;
		iblk_v2->initiator.gateway.ipv4 = iblk_v1->initiator.gateway;
		iblk_v2->initiator.primary_dns.ipv4 = iblk_v1->initiator.primary_dns;
		iblk_v2->initiator.secondary_dns.ipv4 = iblk_v1->initiator.secondary_dns;
	} else {
		iblk_v2->initiator.subnet_mask.ipv6.addr8[0] = 64;
	}

	strcpy(iblk_v2->initiator.iscsi_name, iblk_v1->initiator.iscsi_name);
	strcpy(iblk_v2->initiator.chap_id, iblk_v1->initiator.chap_id);
	strcpy(iblk_v2->initiator.chap_password, iblk_v1->initiator.chap_password);

	/* Migrate Target1 block */
	iblk_v2->target1.ctrl_flags = iblk_v1->target1.ctrl_flags;
	iblk_v2->target1.tcp_port = iblk_v1->target1.tcp_port;
	if (!ipv6) iblk_v2->target1.ip_addr.ipv4 = iblk_v1->target1.ip_addr;
	iblk_v2->target1.boot_lun = iblk_v1->target1.boot_lun;
	strcpy(iblk_v2->target1.chap_id, iblk_v1->target1.chap_id);
	strcpy(iblk_v2->target1.chap_password, iblk_v1->target1.chap_password);
	strcpy(iblk_v2->target1.iscsi_name, iblk_v1->target1.iscsi_name);

	/* Migrate Target2 block */
	iblk_v2->target2.ctrl_flags = iblk_v1->target2.ctrl_flags;
	iblk_v2->target2.tcp_port = iblk_v1->target2.tcp_port;
	if (!ipv6) iblk_v2->target2.ip_addr.ipv4 = iblk_v1->target2.ip_addr;
	iblk_v2->target2.boot_lun = iblk_v1->target2.boot_lun;
	strcpy(iblk_v2->target2.chap_id, iblk_v1->target2.chap_id);
	strcpy(iblk_v2->target2.chap_password, iblk_v1->target2.chap_password);
	strcpy(iblk_v2->target2.iscsi_name, iblk_v1->target2.iscsi_name);

	return 0;
}

int
ib_migrate_v2_cfgblk(
	pNVM_ISCSI_CFG_BLOCK_V2 iblk_v2,
	int ipv6_to_ipv4) {
	if (ipv6_to_ipv4) iblk_v2->gen.ctrl_flags &= ~NVM_ISCSI_FLAGS_IPV6_SUPPORT;
	else iblk_v2->gen.ctrl_flags |= NVM_ISCSI_FLAGS_IPV6_SUPPORT;

	/*
	 * IPv6 address format is not compatiable with IPv4 and vice versa.
	 * Let's clear them.
	 */
	memset(&iblk_v2->initiator.ip_addr, 0, sizeof(IP_ADDR));
	memset(&iblk_v2->initiator.subnet_mask, 0, sizeof(IP_ADDR));
	if (!ipv6_to_ipv4) iblk_v2->initiator.subnet_mask.ipv6.addr8[0] = 64;

	memset(&iblk_v2->initiator.gateway, 0, sizeof(IP_ADDR));
	memset(&iblk_v2->initiator.primary_dns, 0, sizeof(IP_ADDR));
	memset(&iblk_v2->initiator.secondary_dns, 0, sizeof(IP_ADDR));
	memset(&iblk_v2->target1.ip_addr, 0, sizeof(IP_ADDR));
	memset(&iblk_v2->target2.ip_addr, 0, sizeof(IP_ADDR));

	return 0;
}
#endif

common_nvm_status_t
handle_iscsi_upgrade(Tcl_Interp *interp, u8_t *buf, u32_t idx, image_header_t *file_image,
					 u32_t byte_read, u8_t *iscsi_boot_new_fmt, u8_t ib_opt,
					 u8_t validate_versions, u8_t forced_prg, u8_t forced_cfg, u8_t *skip, u8_t mbi_upgrade) {
	common_nvm_status_t status = COMMON_NVM_STATUS_SUCCESS;
	u8_t skipit = FALSE;
	u32_t val32;
	u32_t pci_id_offset, pci_sub_id_offset;
	u16_t device_id, vendor_id, sub_vendor_id, sub_device_id;
	u8_t same_image = 0;

	*skip = FALSE;

	/* Below FORMAT_VERSION_3 header doesn't inlcude CRC in header */
	/* In format version 1,2 image starts after byte_cnt field */


	/* Get the sub_vendor_id from NVM configuration,
	   Required later to decide if to prgram OCNVM image */
	common_nvm_get_nvm_offset_for_pci_id(&pci_id_offset, &pci_sub_id_offset);
	if (common_nvm_read_nvram(pci_sub_id_offset, &val32, sizeof(u32_t)) != COMMON_NVM_STATUS_SUCCESS) {
		Tcl_AppendResult(interp, "Reading VID from NVM failed.\n", NULL);
		return TCL_ERROR;
	}
	sub_vendor_id = (u16_t)MANUF_INFO_PCI_SUBSYS_VENDOR_ID(val32);
	sub_device_id = (u16_t)MANUF_INFO_PCI_SUBSYS_DEVICE_ID(val32);

	if (common_nvm_read_nvram(pci_id_offset, &val32, sizeof(u32_t)) != COMMON_NVM_STATUS_SUCCESS) {
		Tcl_AppendResult(interp, "Reading VID/DID from NVM failed.\n", NULL);
		return TCL_ERROR;
	}

	/* Update VID and DID*/
	device_id = (u16_t)MANUF_INFO_PCI_DEVICE_ID(val32);
	vendor_id = (u16_t)MANUF_INFO_PCI_VENDOR_ID(val32);

	/* Check if we need to skip */
	if (idx == IMAGE_MAP_IDX_ISCSI_BOOT) {
		char old_version[64];
		char new_version[64];
		u32_t rc, entry_idx, old_nvm_addr;

		if (validate_versions) {
			rc = common_nvm_nvm_find_dir_entry(image_table[idx].type, &entry_idx, &old_nvm_addr);
			if (rc == COMMON_NVM_STATUS_SUCCESS) {
				common_nvm_fw_ver_string(old_nvm_addr, image_table[idx].name, old_version, READ_VERSION_FROM_NVM);
				common_nvm_fw_ver_string((common_nvm_virt_addr_t)(uintptr_t)buf, image_table[idx].name, new_version, READ_VERSION_FROM_IMAGE);
				rc = common_nvm_image_version_validation(image_table[idx].name, old_version, new_version, &same_image);
				if (rc != COMMON_NVM_STATUS_SUCCESS) if (mbi_upgrade & same_image) {
						/* Don't fail MBI upgrade if a single image is the same */
						rc = COMMON_NVM_STATUS_SUCCESS;
					}
				return rc;
			}
		}

		/* Both -ib, -ib_ipv4n6, and -ib_ipv6 are at this idx. */
		if ((file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT) &&
			(ib_opt != IB_OPTION_IPV4)) {
			skipit = TRUE;
		} else if ((file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_IPV6) &&
				   (ib_opt != IB_OPTION_IPV6)) {
			skipit = TRUE;
		} else if ((file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_IPV4N6) &&
				   (ib_opt != IB_OPTION_BOTH)) {
			skipit = TRUE;
		}

		/* Determine if this is new iSCSI driver format so as to check if
		   configuration block needs to be converted between v.1 and v.2 */
		if (skipit == FALSE) {
			u32_t format_ver;

			format_ver = *((u32_t *)((((char *)(&file_image->byte_cnt)) + 4) + 0x0c));
			if (((format_ver & 0xffff) == 0x86dd) || ((format_ver & 0xffff) == 0x0800) ||
				((format_ver & 0xffff) == 0xc0b0)) {
				*iscsi_boot_new_fmt = TRUE;
			}
		}
	} else if ((idx == IMAGE_MAP_IDX_ISCSI_BOOT_CFG) || (idx == IMAGE_MAP_IDX_ISCSI_BOOT_CFG2)) {
		u32_t match = TRUE, status;
		u32_t sram_addr, byte_cnt, nvm_addr, entry_idx, agent, val, image_idx;

		if (forced_cfg) {
			if (common_nvm_nvm_find_dir_entry(image_table[IMAGE_MAP_IDX_ISCSI_BOOT].type, &entry_idx, NULL) == 0) {
				status = common_nvm_nvm_get_dir_info(entry_idx, &nvm_addr, &byte_cnt,
													 &sram_addr, NULL, &agent);
				if ((status == 0) && (nvm_addr != NVM_ERASE_VALUE)) {
					if (common_nvm_read_nvram(nvm_addr + 0x0c, &val, 4) == 0) {
						/* ib_opt is called by value parameter. It won't get changed after return.
						   Just change it wiithin this function only for "if (match)" logic below */
						switch (val & 0xffff) {
						case 0x86dd:
							*iscsi_boot_new_fmt = TRUE;
							ib_opt = IB_OPTION_IPV6;
							break;
						case 0x0800:
							*iscsi_boot_new_fmt = TRUE;
							ib_opt = IB_OPTION_IPV4;
							break;
						case 0xc0b0:
							*iscsi_boot_new_fmt = TRUE;
							ib_opt = IB_OPTION_BOTH;
							break;
						default:
							*iscsi_boot_new_fmt = FALSE;
							ib_opt = IB_OPTION_IPV4;
						}
					}
				}
			}
			skipit = TRUE;
		} else {
			/* Both -ib, -ib_ipv4n6, and -ib_ipv6 configuration are at this idx. */
			if (file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_CFG) {
				if ((ib_opt != IB_OPTION_IPV4) || (*iscsi_boot_new_fmt)) {
					match = FALSE;
				}
				skipit = TRUE;
			} else if ((file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_CFG_V2) &&
					   (!*iscsi_boot_new_fmt)) {
				match = FALSE;
				skipit = TRUE;
			} else if (common_nvm_nvm_find_dir_entry(image_table[idx].type, &val32, NULL) != -1) {
				skipit = TRUE;
			}
		}

		/* Check if need to migrate */
		if (match) {
			u32_t end_image_idx = 0, no_funcs;
			u32_t upgrade_cfg34 = 0;
			u32_t offset, length, image_table_idx;


			// Check if need to upgrade I_B_CFG2 incase this is a dual port board
			// TBD: Though common_nvm_number_of_everest_functions returns 2 (dual port board) or 1 (single port board)
			// better change the name of function or use a different one which returns no_ports

			no_funcs = common_nvm_number_of_everest_functions();
			if (no_funcs == 1) {
				end_image_idx = IMAGE_MAP_IDX_ISCSI_BOOT_CFG;
			} else if (common_nvm_is_oem_npar_device(vendor_id, sub_vendor_id, sub_device_id)) {
				end_image_idx = IMAGE_MAP_IDX_ISCSI_BOOT_CFG2;
				upgrade_cfg34 = 1; // MF_ALLOWED: reserve cfg3 and cfg4 for next function (2 & 3).
			} else if (no_funcs == 2) {
				end_image_idx = IMAGE_MAP_IDX_ISCSI_BOOT_CFG2;
			} else if (no_funcs == 4) {
				end_image_idx = IMAGE_MAP_IDX_ISCSI_BOOT_CFG2;
				upgrade_cfg34 = 1; // need to upgrade IMAGE_MAP_IDX_ISCSI_BOOT_CFG3,4 as well
			}

			/* Update for single or dual functions */
			for (image_idx = IMAGE_MAP_IDX_ISCSI_BOOT_CFG; image_idx <= end_image_idx; image_idx++) {
				/* Check if conversion between v.1 and v.2 is needed
				   Also check if v.2 ipv4<-->ipv6 conversion is needed */
				if (common_nvm_nvm_find_dir_entry(image_table[image_idx].type, &entry_idx, NULL) == 0) {
					status = common_nvm_nvm_get_dir_info(entry_idx, &nvm_addr, &byte_cnt,
														 &sram_addr, NULL, &agent);
					if ((status == 0) && (nvm_addr != NVM_ERASE_VALUE)) {
						if (common_nvm_read_nvram(nvm_addr, &val, 4) == 0) {
							/* New driver format, old cfg format. Convert v.1->v.2 */
							if (*iscsi_boot_new_fmt == TRUE) {
								if ((val & 0xffff0000) == 0x01000000) {
									NVM_ISCSI_CFG_BLOCK_V2 iblk_v2;
									NVM_ISCSI_CFG_BLOCK iblk;

									common_nvm_read_nvram(nvm_addr,
														  (u32_t *)&iblk,
														  sizeof(NVM_ISCSI_CFG_BLOCK) - 4);
									ib_migrate_cfgblk_v1_to_v2(
										&iblk,
										&iblk_v2,
										(ib_opt == IB_OPTION_IPV6));
									status = common_nvm_prog_image_by_idx((u8_t *)&iblk_v2,
																		  sizeof(NVM_ISCSI_CFG_BLOCK_V2) - 4,
																		  image_idx,
																		  FALSE,
																		  mbi_upgrade);

									skipit = TRUE;
								}
								/* Check if ipv4<-->ipv6 conversion is needed */
								else {
									if (common_nvm_read_nvram(nvm_addr + 4, &val, 4) == 0) {
										NVM_ISCSI_CFG_BLOCK_V2 iblk_v2;
										int ipv6_to_ipv4 = -1;

										/* Convert v.2 ipv6->ipv4 */
										if ((val & NVM_ISCSI_FLAGS_IPV6_SUPPORT) && (ib_opt == IB_OPTION_IPV4)) {
											ipv6_to_ipv4 = TRUE;
										}
										/* Convert v.2 ipv4->ipv6 */
										else if (!(val & NVM_ISCSI_FLAGS_IPV6_SUPPORT) && ((ib_opt == IB_OPTION_IPV6))) {
											ipv6_to_ipv4 = FALSE;
										}
										if (ipv6_to_ipv4 != -1) {
											common_nvm_read_nvram(nvm_addr,
																  (u32_t *)&iblk_v2,
																  sizeof(NVM_ISCSI_CFG_BLOCK_V2) - 4);
											ib_migrate_v2_cfgblk(
												&iblk_v2,
												ipv6_to_ipv4);
											status = common_nvm_prog_image_by_idx((u8_t *)&iblk_v2,
																				  sizeof(NVM_ISCSI_CFG_BLOCK_V2) - 4,
																				  image_idx,
																				  FALSE,
																				  mbi_upgrade);
											skipit = TRUE;
										}
									}
								}
							}
							/* Old driver format, new cfg format. Convert v.2->v.1 */
							else if ((*iscsi_boot_new_fmt == FALSE) && ((val & 0xffff0000) == 0x02000000)) {
								NVM_ISCSI_CFG_BLOCK iblk_v1;
								NVM_ISCSI_CFG_BLOCK_V2 iblk_v2;

								common_nvm_read_nvram(nvm_addr,
													  (u32_t *)&iblk_v2,
													  sizeof(NVM_ISCSI_CFG_BLOCK_V2) - 4);
								ib_migrate_cfgblk_v2_to_v1(
									&iblk_v2,
									&iblk_v1);
								status = common_nvm_prog_image_by_idx((u8_t *)&iblk_v1,
																	  sizeof(NVM_ISCSI_CFG_BLOCK) - 4,
																	  image_idx,
																	  FALSE,
																	  mbi_upgrade);
								skipit = TRUE;
							}
						}
					}

					/* If I_B_CFG exists but I_B_CFG3 doesn't exist and upgrade_cfg3,4 is set,
					   then create I_B_CFG3,4*/
					if (common_nvm_get_image_in_extended_dir_info(NVM_IMAGE_NAME_ISCSI_BOOT_CFG3, &offset, &length, &image_table_idx) !=
							COMMON_NVM_STATUS_SUCCESS) {
						if (upgrade_cfg34) {
							if (*iscsi_boot_new_fmt) {
								pNVM_ISCSI_CFG_BLOCK_V2 p_iblk_v2;
								p_iblk_v2 = (pNVM_ISCSI_CFG_BLOCK_V2)buf;
								if (ib_opt == IB_OPTION_IPV6) {
									p_iblk_v2->gen.ctrl_flags |= NVM_ISCSI_FLAGS_IPV6_SUPPORT;
								} else {
									p_iblk_v2->gen.ctrl_flags &= ~NVM_ISCSI_FLAGS_IPV6_SUPPORT;
								}
							}
							common_nvm_prog_image_in_extended_dir(buf,
																  byte_read,
																  IMAGE_MAP_IDX_ISCSI_BOOT_CFG3,
																  FALSE,
																  FALSE,
																  FALSE,
																  mbi_upgrade);
							common_nvm_prog_image_in_extended_dir(buf,
																  byte_read,
																  IMAGE_MAP_IDX_ISCSI_BOOT_CFG4,
																  FALSE,
																  FALSE,
																  FALSE,
																  mbi_upgrade);
							upgrade_cfg34 = 0;
						}
					}
				}
				/* No cfg file in function, create one for it */
				else {
					if (*iscsi_boot_new_fmt) {
						pNVM_ISCSI_CFG_BLOCK_V2 p_iblk_v2;
						p_iblk_v2 = (pNVM_ISCSI_CFG_BLOCK_V2)buf;
						if (ib_opt == IB_OPTION_IPV6) {
							p_iblk_v2->gen.ctrl_flags |= NVM_ISCSI_FLAGS_IPV6_SUPPORT;
						} else {
							p_iblk_v2->gen.ctrl_flags &= ~NVM_ISCSI_FLAGS_IPV6_SUPPORT;
						}
					}

					status = common_nvm_prog_image_by_idx(buf,
														  byte_read,
														  image_idx,
														  FALSE,
														  mbi_upgrade);
					if (upgrade_cfg34) {
						common_nvm_prog_image_in_extended_dir(buf,
															  byte_read,
															  IMAGE_MAP_IDX_ISCSI_BOOT_CFG3,
															  FALSE,
															  FALSE,
															  FALSE,
															  mbi_upgrade);
						common_nvm_prog_image_in_extended_dir(buf,
															  byte_read,
															  IMAGE_MAP_IDX_ISCSI_BOOT_CFG4,
															  FALSE,
															  FALSE,
															  FALSE,
															  mbi_upgrade);
						upgrade_cfg34 = 0;
					}
					skipit = TRUE;
				}
			}
		}
	} else if (idx == IMAGE_MAP_IDX_ISCSI_BOOT_CPRG) {
		/* Per Kevin Tran: always program IB_CPRG when upgrade/downgrade ISCSI_B.
		   Because ISCSI_B and IB_CPRG come hand-in-hand */
		skipit = FALSE;
	} else if (file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_CPRG_LGCY) {
		skipit = TRUE;
	}

	*skip = skipit;
	return status;
}

/*
 *------------------------------------------------------------------------
 * common_nvm_check_rev_id_ctrl --
 *
 *     Check nvm cfg option 233: revision ID control
 *     0 - Preserve revision ID after MFW upgrade
 *     1 - Show actual revision ID (do nothing)
 *     2 - Force B0
 *     3 - Force B1
 *
 *------------------------------------------------------------------------
 */
void common_nvm_check_rev_id_ctrl(Tcl_Interp *interp) {
	u32_t val;
	u8_t pci_rev_id;
	char *rev_id_ctrl;

	nvm_cfg_get(interp, IDX_REV_ID_CTRL);
	rev_id_ctrl = Tcl_GetString(Tcl_GetObjResult(interp));

	/* Read revision ID from PCI configuration space */
	common_nvm_reg_read(PCICFG_OFFSET + PCICFG_REVISION_ID_OFFSET, &val);
	pci_rev_id = val & 0xf;
	DbgMessage(NULL, WARN, "common_nvm_check_rev_id_ctrl: rev_id_ctrl %s pci_rev_id 0x%X\n", rev_id_ctrl, pci_rev_id);

	/* Preserve revision ID*/
	if (strcmp(rev_id_ctrl, "0") == 0) {
		DbgMessage(NULL, WARN, "Preserve pci revision ID\n");
#ifndef EDEBUG
		if (!dg_mcp_exist()) {
			/* If MFW doesn't exist (first time MFW upgrade) do nothing */
			DbgMessage(NULL, WARN, "MFW doesn't exist\n");
		} else {
#endif
			if (pci_rev_id == 0) {
				DbgMessage(NULL, WARN, "Force B0\n");
				/* If previous revision ID, 0 force B0*/
				nvm_cfg(interp, REV_ID_CTRL_OPTION, 2);
			} else {
				DbgMessage(NULL, WARN, "Force B1\n");
				/* else force B1*/
				nvm_cfg(interp, REV_ID_CTRL_OPTION, 3);
			}
#ifndef EDEBUG
		}
#endif
	} else if (strcmp(rev_id_ctrl, "1") == 0) {
		DbgMessage(NULL, WARN, "Display actual revision ID\n");
		nvm_cfg(interp, REV_ID_CTRL_OPTION, 3);
	}
}

/*
 *------------------------------------------------------------------------
 * upgrade_usage --
 *
 *     Show information on using "nvm upgrade" command.
 *
 * Results:
 *     A standard TCL result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
int upgrade_usage(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {
	u32_t idx, cnt = 0;
	char  err_msg[512];

	cnt += sprintf(&err_msg[cnt], "[-F] -bc|-l2|-mfw|-mbi");
	for (idx = 0; idx < common_nvm_image_table_size(); idx++) {
		if (image_table[idx].opt[0] != '\0') {
			cnt += sprintf(&err_msg[cnt], "|%s", image_table[idx].opt);
			if (strcmp(image_table[idx].opt, "-mba") == 0) {
				cnt += sprintf(&err_msg[cnt], "[-d -p [-n <pfn_val>]");
			} else if (strcmp(image_table[idx].opt, "-feb") == 0) {
				cnt += sprintf(&err_msg[cnt], "[-c|-p|-cp]");
			}
		}
	}
	cnt += sprintf(&err_msg[cnt], "|-ib_ipv6|-ib_ipv4n6");
	cnt += sprintf(&err_msg[cnt], " filename");
	Tcl_WrongNumArgs(interp, 1, objv, err_msg);
	return TCL_ERROR;
}

/*
 *------------------------------------------------------------------------
 * upgrade_image --
 *
 *     Implements "nvm upgrade" command for one image
 *
 * Results:
 *     A standard TCL result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
int upgrade_image(ClientData client_data, Tcl_Interp *interp, int objc,
				  Tcl_Obj *CONST objv[], nvm_upgrade_params_t *upgrade_params_ptr) {
	Tcl_Obj       *objptr;
	u64_t         new_ver_val = 0, npar1_5_support_ver_val = 0, afex_removed_ver_val = 0;
	u32_t         image_tbl_sz = common_nvm_image_table_size();
	u32_t         idx, i, j;
	u32_t         cur_device_id = 0;
	u32_t         swim_group = 0;
	u32_t         isE3B0 = 0;
	u32_t         extended_cfg_nvm_addr, swim_shadow_supported = 0;
	u32_t         image_table_idx;
	u32_t         image_hdr_size;
	u32_t         nvm_addr;
	u32_t         image_type = (u32_t)-1;
	u32_t         byte_cnt;
	u32_t         total;
	u32_t         type;
	u8_t          image_for_extended_dir = 0, delete_swims = 0;
	u8_t          iscsi_boot_new_fmt = FALSE;
	u8_t          skip = 0;
	u8_t          *image_ptr;
	char          *sf_mode;
	char          buffer[80];
	int           final_status = TCL_OK;
	int           version[4] = { 0, 0, 0, 0 };  //localy used with sscanf so can be int (instead of u32_t)
	int           new_mfw_ver[4] = { 0, 0, 0, 0 };
	nvm_enough_room_result_t res = NVM_NOT_ENOUGH_ROOM;
	common_nvm_status_t status = (common_nvm_status_t)COMMON_NVM_STATUS_SUCCESS;
	const char    *str_p;
	image_header_t *file_image = NULL;

	do {
		/* Program appended images */
		total = 0;
		while (total < upgrade_params_ptr->byte_read) {
			skip = 0;
			file_image = (image_header_t *)&upgrade_params_ptr->buf[total];

			// We should check file_image->magic first before checking
			// file_image->type to determine image_for_extended_dir.

			/* Appended image */
			if (BMAPI_BENDIAN_SWAP(file_image->magic) == FILE_MAGIC) {
				// Swap the whold file_image struct before checking
				// Since the file_image is not written to NVRAM, this portion
				// is swapped and left in buf[], please review and see if
				// there's any downside.
				// If we want to keep the content in buf[] intact, then each
				// field within file_image has to be swapped individually.
				// Let's swap the image_header_t in each block in buf since
				// they are not programmed into NVRAM...
				// All fields in image_header_t do not need to be swapped.
				swap_buffer_if_big_endian_host((u32_t *)file_image,
											   ((sizeof(image_header_t) - 4) / sizeof(u32_t)));

				if ((file_image->type == IMAGE_HDR_TYPE_FCOE_BOOT) ||
					(file_image->type == IMAGE_HDR_TYPE_FCOE_BOOT_CFG) ||
					(file_image->type == IMAGE_HDR_TYPE_FCOE_BOOT_CPRG_LGCY) ||
					(file_image->type == IMAGE_HDR_TYPE_FCOE_BOOT_CPRG_EVRST) ||
					(file_image->type == IMAGE_HDR_TYPE_NIC_PARTITION_CFG) ||
					(file_image->type == IMAGE_HDR_TYPE_VPD_TABLE) ||
					(file_image->type == IMAGE_HDR_TYPE_E3_WC) ||
					(file_image->type == IMAGE_HDR_TYPE_E3_PCIE) ||
					(file_image->type == IMAGE_HDR_TYPE_E3_WCV2) ||
					(file_image->type == IMAGE_HDR_TYPE_E3_PCIEV2) ||
					(file_image->type == IMAGE_HDR_TYPE_NIV_CFG) ||
					(file_image->type == IMAGE_HDR_TYPE_MFW1) ||
					(file_image->type == IMAGE_HDR_TYPE_MFW2) ||
					(file_image->type == IMAGE_HDR_TYPE_MFW2_A) ||
					(file_image->type == IMAGE_HDR_TYPE_SWIM1) ||
					(file_image->type == IMAGE_HDR_TYPE_SWIM2) ||
					(file_image->type == IMAGE_HDR_TYPE_SWIM3) ||
					(file_image->type == IMAGE_HDR_TYPE_SWIM4) ||
					(file_image->type == IMAGE_HDR_TYPE_SWIM5) ||
					(file_image->type == IMAGE_HDR_TYPE_SWIM6) ||
					(file_image->type == IMAGE_HDR_TYPE_SWIM7) ||
					(file_image->type == IMAGE_HDR_TYPE_SWIM8) ||
					(file_image->type == IMAGE_HDR_TYPE_OCNVM) ||
					(file_image->type == IMAGE_HDR_TYPE_BOOT_CFG_SHADOW) ||
					(file_image->type == IMAGE_HDR_TYPE_CCM) ||
					(file_image->type == IMAGE_HDR_TYPE_ISCSI_PERS) ||
					(file_image->type == IMAGE_HDR_TYPE_MDUMP)) {
					//printf("EXT_DIR: Image name 0x%x, size %d \n", file_image->type, file_image->byte_cnt);
					image_for_extended_dir = 1;
				} else {
					//printf("REG_IMG: Image name 0x%x, size %d \n", file_image->type, file_image->byte_cnt);
					image_for_extended_dir = 0;
				}

				/* Supported formats are <= 2 */
				if ((file_image->version == FORMAT_VERSION_1) ||
					(file_image->version == FORMAT_VERSION_2)) {
					/* In format version 1,2 image starts after byte_cnt field */
					image_ptr = (u8_t *)(((char *)&file_image->byte_cnt) + 4);
					image_hdr_size = sizeof(image_header_t);

					if (file_image->version == LATEST_FORMAT_VERSION) {
						// Verify image is for this board chip number, if not continue to next image
						if (validate_chip(interp, file_image->image_info) != TCL_OK) {
							total += image_hdr_size + file_image->byte_cnt;
							continue;
						}

						// Verify image type is for this option
						if (validate_image_type(interp, upgrade_params_ptr->option, file_image->type) != TCL_OK) {
							return TCL_ERROR;
						}
					}

					/* TBD check is this is still necessary */
					/* in case of "-ib" option, skip legacy images (all images but
					   iSCSI boot driver and iSCSI configuration images */
					if ((common_nvm_nvm_is_legacy_iscsi_image(file_image) == COMMON_NVM_STATUS_SUCCESS) ||
						(upgrade_params_ptr->iscsi_ib_option)) {
						if (((file_image->type != IMAGE_HDR_TYPE_ISCSI_BOOT) &&
							 (file_image->type != IMAGE_HDR_TYPE_ISCSI_BOOT_IPV6) &&
							 (file_image->type != IMAGE_HDR_TYPE_ISCSI_BOOT_IPV4N6) &&
							 (file_image->type != IMAGE_HDR_TYPE_ISCSI_BOOT_CPRG) &&
							 (file_image->type != IMAGE_HDR_TYPE_ISCSI_BOOT_CFG_V2) &&
							 (file_image->type != IMAGE_HDR_TYPE_ISCSI_BOOT_CFG)) ||
							(((file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT) && (upgrade_params_ptr->ib_opt != IB_OPTION_IPV4)) ||
							 ((file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_IPV6) && (upgrade_params_ptr->ib_opt != IB_OPTION_IPV6)) ||
							 ((file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_IPV4N6) && (upgrade_params_ptr->ib_opt != IB_OPTION_BOTH)))) {
							total += image_hdr_size + file_image->byte_cnt;
							continue;
						} else if ((upgrade_params_ptr->ib_opt == IB_OPTION_NONE) &&
								   ((file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT) ||
									(file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_IPV4N6) ||
									(file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_IPV6))) {
							total += image_hdr_size + file_image->byte_cnt;
							continue;
						} else if ((upgrade_params_ptr->forced_cfg == 0) && (upgrade_params_ptr->ib_opt == IB_OPTION_NONE) &&
								   ((file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_CFG) ||
									(file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_CFG_V2))) {
							total += image_hdr_size + file_image->byte_cnt;
							continue;
						} else if ((upgrade_params_ptr->forced_prg == 0) && (upgrade_params_ptr->ib_opt == IB_OPTION_NONE) &&
								   (file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_CPRG)) {
							total += image_hdr_size + file_image->byte_cnt;
							continue;
						}
					}

					// Get the image index in image_table
					for (idx = 0; idx < image_tbl_sz; idx++) {
						/* Single out ipv6 iscsi driver and ipv6 iscsi cfg for special handling */
						if ((file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_IPV6) ||
							(file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_IPV4N6)) {
							if (IMAGE_HDR_TYPE_ISCSI_BOOT == image_table[idx].hdr_type) {
								image_type = IMAGE_HDR_TYPE_ISCSI_BOOT;
								break;
							}
						} else if (file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_CFG_V2) {
							if (IMAGE_HDR_TYPE_ISCSI_BOOT_CFG == image_table[idx].hdr_type) {
								image_type = IMAGE_HDR_TYPE_ISCSI_BOOT_CFG;
								break;
							}
						} else if (file_image->type == image_table[idx].hdr_type) {
							/* Assign the image type */
							image_type = file_image->type;
							break;
						}
					}

					/* Did not find this file type in image table */
					if (idx == image_tbl_sz) {
						/* incase of combined FCOE image, ignore FCOE_CPRG and FCOE_LGCY images */
						if ((strcmp(upgrade_params_ptr->option, "-feb") == 0) &&
							((file_image->type == IMAGE_HDR_TYPE_FCOE_BOOT_CPRG) ||
							 (file_image->type == IMAGE_HDR_TYPE_FCOE_BOOT_CPRG_LGCY))) {
							printf("skipping FCOE CPRG/CPRG_LGCY block image\n");
							total += image_hdr_size + file_image->byte_cnt;
							continue;
						} else {
							Tcl_AppendResult(interp, "Improper file format, ",
											 "unknown file type.\n", NULL);
							final_status = TCL_ERROR;
							break;
						}
					}

					status = handle_iscsi_upgrade(interp, image_ptr, idx, file_image, file_image->byte_cnt, &iscsi_boot_new_fmt,
												  upgrade_params_ptr->ib_opt, upgrade_params_ptr->validate_versions,
												  upgrade_params_ptr->forced_prg, upgrade_params_ptr->forced_cfg, &skip, upgrade_params_ptr->mbi_upgrade);
					if (status != COMMON_NVM_STATUS_SUCCESS) {
						Tcl_AppendResult(interp, "Image programming failed !!\n", NULL);
						final_status = TCL_ERROR;
						break;
					}

					// incase of I_B_CPRG force to program, without checking version validation
					if (file_image->type == IMAGE_HDR_TYPE_ISCSI_BOOT_CPRG) {
						upgrade_params_ptr->validate_versions = 0;
					}

					// For -feb, if -c option isn't set, skip FCOE_BOOT_CFG is if already exists
					if (!upgrade_params_ptr->feb_forced_cfg && (file_image->type == IMAGE_HDR_TYPE_FCOE_BOOT_CFG)) {

						/* Search for FCOE_BOOT_CFG image in extended dir */
						if (common_nvm_get_image_in_extended_dir_info(NVM_IMAGE_NAME_FCOE_BOOT_CFG,
																	  &nvm_addr,
																	  &byte_cnt,
																	  &image_table_idx) ==
								COMMON_NVM_STATUS_SUCCESS) {
							/* FEB_CFG already exist:*/

							/* Check if NPAR1.5 device ID */
							if (common_nvm_is_oem_npar_device(upgrade_params_ptr->vendor_id,
															  upgrade_params_ptr->sub_vendor_id,
															  upgrade_params_ptr->sub_device_id)) {

								/* MF_ALLOWED/BD/NPAR1.5 mode */
								if (common_nvm_get_image_in_extended_dir_info(NVM_IMAGE_NAME_FCOE_BOOT_CFG3,
																			  &nvm_addr,
																			  &byte_cnt,
																			  &image_table_idx) !=
										COMMON_NVM_STATUS_SUCCESS) {
									/* FEB_CFG3,4 are missing, add  them and skip CFG, CFG2 */
									skip = 1;
									common_nvm_prog_image_in_extended_dir(image_ptr,
																		  file_image->byte_cnt,
																		  IMAGE_MAP_IDX_FCOE_BOOT_CFG3,
																		  upgrade_params_ptr->validate_versions,
																		  FALSE,
																		  delete_swims,
																		  upgrade_params_ptr->mbi_upgrade);
									common_nvm_prog_image_in_extended_dir(image_ptr,
																		  file_image->byte_cnt,
																		  IMAGE_MAP_IDX_FCOE_BOOT_CFG4,
																		  upgrade_params_ptr->validate_versions,
																		  FALSE,
																		  delete_swims,
																		  upgrade_params_ptr->mbi_upgrade);
								} else {
									/* FEB_CFG3 exist, then skip programing FEB_CFG files */
									printf("skipping FCOE boot config block\n");
									skip = 1;
								}
							} else {
								/* FEB_CFG exist and not MF_ALLOWED mode, then skip FEB_CFG files*/
								printf("skipping FCOE boot config block\n");
								skip = 1;
							}
						}
					}

					// For -feb, if -p option isn't set, skip FCOE_BOOT_CPRG_EVRST
					if (!upgrade_params_ptr->feb_forced_prg_evrst && (file_image->type == IMAGE_HDR_TYPE_FCOE_BOOT_CPRG_EVRST)) {
						printf("skipping FCOE boot config program everest block\n");
						skip = 1;
					}

#ifndef FWUPGTOOL
					isE3B0 = is_e3(interp) && is_rev_b(interp);
#else
					chip_rev = Tcl_GetVar2(interp, TCL_ENV, ENV_CHIP_REV, 0);
					isE3B0 = (COMMON_NVM_IS_E3() && (strncmp(chip_rev, "B", 1) == 0));
#endif // !FWUPGTOOL
					//printf("isE3B0 is %d\n", isE3B0);

					if ((file_image->type == IMAGE_HDR_TYPE_E3_WC) || (file_image->type == IMAGE_HDR_TYPE_E3_PCIE)) {
						skip = isE3B0 ? 1 : 0;
						//printf("E3_WCpcie skip is %d\n", skip);
					} else if ((file_image->type == IMAGE_HDR_TYPE_E3_WCV2) || (file_image->type == IMAGE_HDR_TYPE_E3_PCIEV2)) {
						skip = isE3B0 ? 0 : 1;
						//printf("E3_WCpcie_V2 skip is %d\n", skip);
					}

					/* Check if OCNVM image needs to be programed according to
					   sub-vendor ID in NVM and not from PCI config space.
					   (per SYSOP request to avoid reboot when programing an empty board)*/
					if ((file_image->type == IMAGE_HDR_TYPE_OCNVM) &&
						(upgrade_params_ptr->sub_vendor_id != OEM_ID_2)) {
						skip = 1;
					}

					// Program image if decided not to skip it (in handle_iscsi_upgrade())
					if (!image_for_extended_dir) {
						status = (skip) ? COMMON_NVM_STATUS_SUCCESS :
										  common_nvm_prog_image_by_idx(image_ptr,
																	   file_image->byte_cnt,
																	   idx,
																	   upgrade_params_ptr->validate_versions,
																	   upgrade_params_ptr->mbi_upgrade);
					} else {
						// If programming new MFW image (BC+NCSI), delete the old BC image
						// Note: code assumes MFW1 is always the first image in MCP package
						if (image_table[idx].type == CODE_IMAGE_TYPE_MFW1) {

							/* if upgrading MFW, validate byte_cnt field in extened dir is correct */
							common_nvm_validate_extended_dir_byte_cnt();

							/* MFW does not support 57712 chip rev A0 */
#ifndef FWUPGTOOL
							common_nvm_get_info(&upgrade_params_ptr->dev_stat);

							if ((strcmp(upgrade_params_ptr->dev_stat.asic_rev, "A0") == 0) && (is_e2(interp))) {
								Tcl_AppendResult(interp, "ERROR: MFW image does not support 57712 chip rev A0.\n",
												 "Upgrade aborted for image MFW.\n", NULL);
								return TCL_ERROR;
							}
#else
							// The ENV_CHIP_REV was set in Bnx2Helper::SetSelectedNIC()
							chip_rev = Tcl_GetVar2(interp, TCL_ENV, ENV_CHIP_REV, 0);

							if ((strcmp(chip_rev, "A0") == 0) && COMMON_NVM_IS_E2()) {
								Tcl_AppendResult(interp, "ERROR: MFW image does not support 57712 chip rev A0.\n",
												 "Upgrade aborted for image MFW.\n", NULL);
								return TCL_ERROR;
							}
#endif // !FWUPGTOOL

							//check if shadow swim feature is supported according to nvm cfg
							swim_shadow_supported = 0;
							if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &byte_cnt, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS) {
								if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
									if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, byte_cnt, &upgrade_params_ptr->extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
										Tcl_AppendResult(interp, "Failed to parse extended dir image\n", NULL);
										return TCL_ERROR;
									}

									/* Search for CFG_EXTENDED_SHARED image in extended dir */
									if (common_nvm_nvm_find_extended_dir_entry(CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &upgrade_params_ptr->extended_dir,
																			   NULL, &extended_cfg_nvm_addr, NULL, NULL, NULL) == COMMON_NVM_STATUS_SUCCESS) {
										common_nvm_read_nvram(extended_cfg_nvm_addr + OFFSETOF(struct extended_dev_info_shared_cfg, board_cfg),
															  &swim_shadow_supported, sizeof(u32_t));

										swim_shadow_supported &= EXTENDED_DEV_INFO_SHARED_CFG_SHADOW_SWIM_MASK;
									}
								}
							}

							// delete SWIMs that are not currently used (due to shadow feature) and find swim group currently used
							common_nvm_delete_unused_swim_images(&swim_group);

							// Note: code assumes MFW1 is always the first image in MCP package
#ifndef FWUPGTOOL
							res = enough_room(is_e1x(interp), CODE_IMAGE_TYPE_MFW1, upgrade_params_ptr->byte_read, upgrade_params_ptr->image_types_in_input_mfw, upgrade_params_ptr->input_mfw_images_found);
#else
							res = enough_room(COMMON_NVM_IS_E1x(), CODE_IMAGE_TYPE_MFW1, upgrade_params_ptr->byte_read, upgrade_params_ptr->image_types_in_input_mfw, upgrade_params_ptr->input_mfw_images_found);
#endif // !FWUPGTOOL
							delete_swims = ((res == NVM_ENOUGH_ROOM_EXCEPT_2_SWIM_GROUPS) || !swim_shadow_supported);
							if (res != NVM_NOT_ENOUGH_ROOM) {
								// If there is enough room for MFW+SWIM images, delete BC1, NCSI images
								common_nvm_delete_bc_and_ncsi_images();
							} else {
								Tcl_AppendResult(interp, "ERROR: There is not enough room in NVM for MFW and SWIM images\n",
												 "Upgrade aborted for image MFW.\n", NULL);
								return TCL_ERROR;
							}
						}

						// delete SWIMs of group B if:
						// 1. burning image of same number
						// 2. burning old MFW image that doesn't support shadow feature
						// 3. shadow is not supported according to NVM
						if ((image_table[idx].type == CODE_IMAGE_TYPE_SWIM1) &&
							(((running_bc_major == new_mfw_major) &&
							  (running_bc_minor == new_mfw_minor) &&
							  (running_bc_build == new_mfw_build) &&
							  (new_mfw_build != 0) &&
							  (swim_group == 2)) ||
							 (!swim_shadow_supported) ||
							 ((((new_mfw_major & 0xFF) << 16) | ((new_mfw_minor & 0xFF) << 8) | (new_mfw_build & 0xFF)) <
								  (((MFW_SHADOW_FEATURE_SUPPORTED_MAJOR & 0xFF) << 16) | ((MFW_SHADOW_FEATURE_SUPPORTED_MINOR & 0xFF) << 8) | (MFW_SHADOW_FEATURE_SUPPORTED_BUILD & 0xFF))))) {
							common_nvm_delete_swim_group_b_images();
						}

						// burn SWIMs to group B if:
						// 1. current SWIMs of group A are running.
						// 2. There is a place for both SWIMs instances
						// 3. Asked to burn a version different than the one currently running
						// 4. MFW version to be burned supporting shadow feature
						// 5. shadow feature is supported
						if ((swim_group == 1) && (res == NVM_ENOUGH_ROOM) && (swim_shadow_supported) &&
							((image_table[idx].type == CODE_IMAGE_TYPE_SWIM1) ||
							 (image_table[idx].type == CODE_IMAGE_TYPE_SWIM2) ||
							 (image_table[idx].type == CODE_IMAGE_TYPE_SWIM3) ||
							 (image_table[idx].type == CODE_IMAGE_TYPE_SWIM4) ||
							 (image_table[idx].type == CODE_IMAGE_TYPE_SWIM5) ||
							 (image_table[idx].type == CODE_IMAGE_TYPE_SWIM6) ||
							 (image_table[idx].type == CODE_IMAGE_TYPE_SWIM7) ||
							 (image_table[idx].type == CODE_IMAGE_TYPE_SWIM8)) &&
							((running_bc_major != new_mfw_major) ||
							 (running_bc_minor != new_mfw_minor) ||
							 (running_bc_build != new_mfw_build)) &&
							((((new_mfw_major & 0xFF) << 16) | ((new_mfw_minor & 0xFF) << 8) | (new_mfw_build & 0xFF)) >=
								 (((MFW_SHADOW_FEATURE_SUPPORTED_MAJOR & 0xFF) << 16) | ((MFW_SHADOW_FEATURE_SUPPORTED_MINOR & 0xFF) << 8) | (MFW_SHADOW_FEATURE_SUPPORTED_BUILD & 0xFF)))) {
							status = (skip) ? COMMON_NVM_STATUS_SUCCESS :
											  common_nvm_prog_image_in_extended_dir(image_ptr,
																					file_image->byte_cnt,
																					idx + NVM_NUM_OF_SWIMS,
																					upgrade_params_ptr->validate_versions,
																					FALSE,
																					delete_swims,
																					upgrade_params_ptr->mbi_upgrade);
						} else {
							status = (skip) ? COMMON_NVM_STATUS_SUCCESS :
											  common_nvm_prog_image_in_extended_dir(image_ptr,
																					file_image->byte_cnt,
																					idx,
																					upgrade_params_ptr->validate_versions,
																					FALSE,
																					delete_swims,
																					upgrade_params_ptr->mbi_upgrade);
						}
					}

					// for MFW validate version only for first image, to avoid the need to have all images with higher version
					// number than the old one (as some of the images doesn't increment each time, such as OCNVM)
					// Note: code assumes MFW1 is always the first image in MCP package
					if (image_table[idx].type == CODE_IMAGE_TYPE_MFW1) {
						if (!upgrade_params_ptr->mbi_upgrade) {
							/* In case of MBI upgrade, check each single image and skip if it is the same */
							upgrade_params_ptr->validate_versions = 0;
						}
						add_shadow_dell_ncsi_image();
						add_bdn_image();
					}

					if (status != COMMON_NVM_STATUS_SUCCESS) {
						Tcl_AppendResult(interp, "Image programming failed !!\n", NULL);
						final_status = TCL_ERROR;
						break;
					}

					total += image_hdr_size + file_image->byte_cnt;
				} else {
					Tcl_AppendResult(interp, "Improper file format, ",
									 "incorrect format version.\n", NULL);
					final_status = TCL_ERROR;
					break;
				}
			} /* if (appended_image_file) */
			else {
				/* Single image: no "append" header */
				objptr = Tcl_GetVar2Ex(interp, TCL_ENV, ENV_DID, 0);
				Tcl_GetIntFromObj(interp, objptr, (int *)&cur_device_id);

				/* For everest devices, all images (except mba, m_pn) should include append header*/
				if ((is_e1x(globalInterp)) &&
					(strcmp(upgrade_params_ptr->option, "-mba") != 0) && (strcmp(upgrade_params_ptr->option, "-m_pn") != 0) &&
					(strcmp(upgrade_params_ptr->option, "-vpd") != 0) && (strcmp(upgrade_params_ptr->option, "-hw_set") != 0)) {
					Tcl_AppendResult(interp, "Improper file format, does not include 'append' header \n ",
									 "Not intended for 5710, 5711 or 5712.\n", NULL);
					final_status = TCL_ERROR;
					break;
				}

				// Check in case if "option" is meant for multi-image file
				if ((strcmp(upgrade_params_ptr->option, "-bc") == 0) ||
					(strcmp(upgrade_params_ptr->option, "-ipmi") == 0) ||
					(strcmp(upgrade_params_ptr->option, "-ib") == 0) ||
					(strcmp(upgrade_params_ptr->option, "-ib_ipv6") == 0) ||
					(strcmp(upgrade_params_ptr->option, "-ib_ipv4n6") == 0) ||
					(strcmp(upgrade_params_ptr->option, "-ncsi") == 0) ||
					(strcmp(upgrade_params_ptr->option, "-l2t") == 0) ||
					(strcmp(upgrade_params_ptr->option, "-l2c") == 0) ||
					(strcmp(upgrade_params_ptr->option, "-l2x") == 0) ||
					(strcmp(upgrade_params_ptr->option, "-l2u") == 0) ||
					(strcmp(upgrade_params_ptr->option, "-feb") == 0)) {

					Tcl_AppendResult(interp, "Improper file format, ",
									 "Not intended for ", upgrade_params_ptr->option, " option.\n", NULL);
					final_status = TCL_ERROR;
					break;
				}

				/* Figure out which image to upgrade */
				for (idx = 0; idx < image_tbl_sz; idx++) {
					/* Since I cannot create new entry in image_table for
					   -ib_ipv6 or -ib_ipv4n6 due to image_type exhaustion,
					   just force it to use -ib. This is ugly */
					if ((strcmp(upgrade_params_ptr->option, "-ib_ipv6") == 0) || (strcmp(upgrade_params_ptr->option, "-ib_ipv4n6") == 0)) {
						if (strcmp("-ib", image_table[idx].opt) == 0) {
							image_type = image_table[idx].type;
							break;
						}
					} else if (strcmp(upgrade_params_ptr->option, image_table[idx].opt) == 0) {
						/* Assign the image type */
						image_type = image_table[idx].type;
						break;
					}
				}

				/* Incase of MBA image, accept image without header (CQ#34211) */
				if ((image_type != CODE_IMAGE_TYPE_MBA) &&
					(strcmp(upgrade_params_ptr->option, "-m_pn") != 0) &&
					(strcmp(upgrade_params_ptr->option, "-vpd") != 0) &&
					(strcmp(upgrade_params_ptr->option, "-hw_set") != 0) &&
					(strcmp(upgrade_params_ptr->option, "-bootsh") != 0)) {
					Tcl_AppendResult(interp, "Warning: Image is not wrapped with 'append' header.\n", NULL);
				}

				if (idx >= image_tbl_sz) {
					return upgrade_usage(interp, objc, objv);
				}

				status = handle_iscsi_upgrade(interp, upgrade_params_ptr->buf, idx, file_image, file_image->byte_cnt,
											  &iscsi_boot_new_fmt, upgrade_params_ptr->ib_opt, upgrade_params_ptr->validate_versions,
											  upgrade_params_ptr->forced_prg, upgrade_params_ptr->forced_cfg, &skip, upgrade_params_ptr->mbi_upgrade);
				if (status != COMMON_NVM_STATUS_SUCCESS) {
					Tcl_AppendResult(interp, "Image programming failed !!\n", NULL);
					final_status = TCL_ERROR;
					break;
				}

				// Incase of I_B_CFG iscsi configuration image and flag force_cfg (-ib, -ibc) is on
				// Incase of I_B_CPRG iscsi utility program and flag force_prg (-ibpe) is on

				// Program image
				if (strcmp(upgrade_params_ptr->option, "-m_pn") == 0) {
					module_image_t *modules_p;
					/* No need to validate versions incase of -m_pn option
					   for programming modules image */
					upgrade_params_ptr->validate_versions = 0;
					/* Swap bytes of modules buffer */
					modules_p = (module_image_t *)upgrade_params_ptr->modules_buf;

					/* swap all but the flags (first 32 bits): */
					for (i = 0; i < BMAPI_BENDIAN_SWAP(modules_p->no_modules); i++) {
						/* the module_info_t is a multiplied of u32_t*/
						for (j = sizeof(u32_t) / sizeof(u32_t); j < sizeof(module_info_t) / sizeof(u32_t); j++) {
							// Swap to LE format for both LE and BE machine
							((u32_t *)&(modules_p->modules[i]))[j] = BYTE_SWAP(((u32_t *)&(modules_p->modules[i]))[j]);
						}
					}
					status = common_nvm_prog_image_by_idx(upgrade_params_ptr->modules_buf,
														  upgrade_params_ptr->byte_read,
														  idx,
														  upgrade_params_ptr->validate_versions,
														  upgrade_params_ptr->mbi_upgrade);
				} else if (strcmp(upgrade_params_ptr->option, "-vpd") == 0) {
					/* No need to validate versions incase of -vpd option
					   for programming modules image */
					upgrade_params_ptr->validate_versions = 0;
					status = common_nvm_prog_image_in_extended_dir((u8_t *)upgrade_params_ptr->vpd_buf,
																   upgrade_params_ptr->byte_read,
																   idx,
																   upgrade_params_ptr->validate_versions,
																   FALSE,
																   FALSE,
																   upgrade_params_ptr->mbi_upgrade);
					Tcl_SetVar2(interp, TCL_ENV, ENV_IS_VPD, "1", TCL_GLOBAL_ONLY);
				} else if (strcmp(upgrade_params_ptr->option, "-hw_set") == 0) {
					// No need to validate versions incase of -hw_set option
					upgrade_params_ptr->validate_versions = 0;
					status = common_nvm_prog_image_in_extended_dir((u8_t *)upgrade_params_ptr->hw_set_buf,
																   upgrade_params_ptr->byte_read,
																   idx,
																   upgrade_params_ptr->validate_versions,
																   FALSE,
																   FALSE,
																   upgrade_params_ptr->mbi_upgrade);
				} else {
					status = (skip) ? COMMON_NVM_STATUS_SUCCESS :
									  common_nvm_prog_image_by_idx(upgrade_params_ptr->buf,
																   upgrade_params_ptr->byte_read,
																   idx,
																   upgrade_params_ptr->validate_versions,
																   upgrade_params_ptr->mbi_upgrade);
				}
				total += sizeof(image_header_t) + upgrade_params_ptr->byte_read;
			}
		}
		if ((total < upgrade_params_ptr->byte_read) || (status == COMMON_NVM_STATUS_FAILURE)) {
			/* Encountered error */
			final_status = TCL_ERROR;
			break;         /* Break out of the do-while loop */
		} else {
			// program ended succesfully - check if there is a need to manually fix shmem base register
			common_nvm_fix_shmem_offset_for_older_versions();

			if ((strcmp(upgrade_params_ptr->option, "-mfw") == 0) || (strcmp(upgrade_params_ptr->option, "-nicp") == 0)) {
#ifndef FWUPGTOOL
				common_nvm_init_npar_nvm_image(is_e3(interp));
#else
				common_nvm_init_npar_nvm_image(COMMON_NVM_IS_E3());
#endif // !FWUPGTOOL
			}

			// To avoid failure when upgrading from BC5.0.13 to BC6.2.10
			// "trick" the BC to think there was a POR reset
#ifndef FWUPGTOOL
			if (!common_nvm_tcl_al_get_virtual_nvm_mode()) {
				common_nvm_reg_write(MISC_REG_GENERIC_POR_0, 1);
			}
#else
			if (iNvm.m_td_write_reg32(iNvm.m_current_dev_index, MISC_REG_GENERIC_POR_0, 1) != TCL_OK) {
				// final_status is initialized to TCL_ERROR...
				Tcl_AppendResult(interp, "Failed to set MISC_REG_GENERIC_POR_0 register. \n", NULL);
				break;
			}
#endif // !FWUPGTOOL

			if ((strcmp(upgrade_params_ptr->option, "-mfw") == 0)) {
				sscanf(NPAR1_5_SUPPORT_VER, "%x.%x.%x.%x", &version[0], &version[1],
					   &version[2], &version[3]);

				npar1_5_support_ver_val = version[3] + ((u64_t)version[2] << 8) +
					((u64_t)version[1] << 16) + ((u64_t)version[0] << 24);

				sscanf(AFEX_REMOVED_VER, "%x.%x.%x.%x", &version[0], &version[1], &version[2], &version[3]);

				afex_removed_ver_val = version[3] + ((u64_t)version[2] << 8) +
					((u64_t)version[1] << 16) + ((u64_t)version[0] << 24);

				common_nvm_read_mfw_version(&upgrade_params_ptr->mfw_version[0]);

				if (sscanf(upgrade_params_ptr->mfw_version, "%x.%x.%x.%x", &new_mfw_ver[0], &new_mfw_ver[1],
						   &new_mfw_ver[2], &new_mfw_ver[3]) != 4) {

					sscanf(upgrade_params_ptr->mfw_version, "%x.%x.%x", &new_mfw_ver[0], &new_mfw_ver[1], &new_mfw_ver[2]);
				}

				new_ver_val = new_mfw_ver[3] + ((u64_t)new_mfw_ver[2] << 8) +
					((u64_t)new_mfw_ver[1] << 16) + ((u64_t)new_mfw_ver[0] << 24);

				/* Create empty NICP image incase upgrading to MFW 7.a.x (for specific OEM only) */
				if ((new_ver_val >= npar1_5_support_ver_val) &&
					((upgrade_params_ptr->dev_stat.svid == OEM_ID_2) || (upgrade_params_ptr->dev_stat.svid == OEM_ID_3))) {
					common_nvm_create_mac_partition_image();
				}

				/* Delete NIV images */
				if (new_ver_val >= afex_removed_ver_val) {
					common_nvm_delete_niv_images();
					nvm_cfg_get(interp, IDX_FORCE_SF_MODE);
					sf_mode = Tcl_GetString(Tcl_GetObjResult(interp));
					if (strcmp(sf_mode, "4") == 0) {
						nvm_cfg(interp, IDX_FORCE_SF_MODE, 1);
					}
				}
			}

			if (!common_nvm_tcl_al_get_virtual_nvm_mode()) {
				/* Check nvm cfg option 232: revision ID control upon MFW upgrade*/
				if ((strcmp(upgrade_params_ptr->option, "-mfw") == 0) && is_e3(interp) && !is_e3_57840(interp)) {
					common_nvm_check_rev_id_ctrl(interp);
				}
			}

			final_status = TCL_OK;
			Tcl_AppendResult(interp, "Info burned into nvm successfully.\n\n", NULL);
		}
	} while (0);

	if (status != COMMON_NVM_STATUS_SUCCESS) {
		final_status = TCL_ERROR;
	}

#ifndef FWUPGTOOL
	// For FWUPG tool, we don't issue the MCP reset and update the device information.

	if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
		// Update device information of all devices and environment MAC addr.
		common_nvm_refresh_devices();
	} else {
		Tcl_AppendResult(interp, "Please power-cycle the target machine.\n", NULL);
	}
#endif

	if (final_status == TCL_OK) {
		common_nvm_get_info(&upgrade_params_ptr->dev_stat);
		/* Update VPD V3, V6 fields with MFW, MBA versions (for specific OEM only) */
		if (upgrade_params_ptr->dev_stat.svid == OEM_ID_2) {
			if (strcmp(upgrade_params_ptr->option, "-mfw") == 0) {
				/* Update VPD V3 only if it already exists */
				printf("Update VPD V3 field, only if exists ..\n");
				if (Tcl_Eval(interp, "vpd read V3") == TCL_OK) {
					str_p = Tcl_GetStringResult(interp);
					if ((strcmp(str_p, "Unknown VPD field\n") != 0) &&
						(strcmp(str_p, "Requested tag is not found in VPD info!\n") != 0)) {
						if (common_nvm_read_mfw_version(&upgrade_params_ptr->mfw_version[0]) == COMMON_NVM_STATUS_SUCCESS) {
							sprintf(&buffer[0], "Programming VPD V3 field (%s)\n", &upgrade_params_ptr->mfw_version[0]);
							print_msg(buffer, ESC_COLOR_CYAN);
							sprintf(&buffer[0], "vpd write V3 %s", &upgrade_params_ptr->mfw_version[0]);
							if (Tcl_Eval(interp,  buffer) != TCL_OK) {
								print_msg("Failed to program VPD V3 field\n", ESC_COLOR_RED);
								return TCL_ERROR;
							}
						}
					}
				}
			}

			/* Update VPD V6 field with MBA version (for specific OEM only) */
			if (strcmp(upgrade_params_ptr->option, "-mba") == 0) {
				/* Update VPD V6 only if it already exists */
				printf("Update VPD V6 field, only if exists ..\n");
				if (Tcl_Eval(interp, "vpd read V6") == TCL_OK) {
					str_p = Tcl_GetStringResult(interp);
					if ((strcmp(str_p, "Unknown VPD field\n") != 0) &&
						(strcmp(str_p, "Requested tag is not found in VPD info!\n") != 0)) {
						if (common_nvm_read_mba_efi_versions(&upgrade_params_ptr->mba_version[0], &upgrade_params_ptr->efi_version[0]) == COMMON_NVM_STATUS_SUCCESS) {
							sprintf(&buffer[0], "Programming VPD V6 field (%s)\n", &upgrade_params_ptr->mba_version[0]);
							print_msg(buffer, ESC_COLOR_CYAN);
							sprintf(&buffer[0], "vpd write V6 %s", &upgrade_params_ptr->mba_version[0]);
							if (Tcl_Eval(interp,  buffer) != TCL_OK) {
								print_msg("Failed to program VPD V6 field\n", ESC_COLOR_RED);
								return TCL_ERROR;
							}

							/* Update VPD V1 with EFI version */
							if (upgrade_params_ptr->efi_version[0] != '\0') {
								sprintf(&buffer[0], "Programming VPD V1 field (%s)\n", &upgrade_params_ptr->efi_version[0]);
								print_msg(buffer, ESC_COLOR_CYAN);
								sprintf(&buffer[0], "vpd write V1 %s", &upgrade_params_ptr->efi_version[0]);
								if (Tcl_Eval(interp,  buffer) != TCL_OK) {
									print_msg("Failed to program VPD V1 field\n", ESC_COLOR_RED);
									return TCL_ERROR;
								}
							}
						}
					}
				}
			}
		}
	}

	return final_status;
}

/*
 *------------------------------------------------------------------------
 * parse_upgrade_cmd --
 *
 *     Parse the "nvm upgrade" command.
 *
 * Results:
 *     A standard diag result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
int parse_upgrade_cmd(ClientData client_data, Tcl_Interp *interp, int objc,
					  Tcl_Obj *CONST objv[], nvm_upgrade_params_t *upgrade_params_ptr) {
	exp_rom_hdr_t   *exp_rom_ptr;
	vendor_data_t   *v_data_ptr;
	image_header_t  *file_image = NULL;
	u32_t           idx, i;
	u32_t           pci_id_offset, pci_sub_id_offset;
	u32_t           val32, crc32, in_crc;
	u32_t 			trailer;
	u32_t 			bitmask;
	u32_t           file_size;
	u32_t           total;
	u16_t           pfn_val = 0;
	u8_t            bypass_dev_check = 0, forced_pfn = 0;
	char            *filename = NULL;
	char            *ptr;
	int             final_status = TCL_OK;

#ifndef FWUPGTOOL
	u32_t n_devices = 2, dev_idx = 1, cur_dev_idx = 1;

	common_nvm_memset(&upgrade_params_ptr->extended_dir, 0, sizeof(extended_dir_image_t));

	if (common_nvm_tcl_al_get_no_pci_mode() == 0) {
		// incase of PCI mode, go through devices and verify driver is not loaded
		n_devices = common_nvm_get_total_br_dev();
		cur_dev_idx = atoi(Tcl_GetVar2(interp, TCL_ENV, ENV_DEV, TCL_GLOBAL_ONLY));

		for (dev_idx = 1; dev_idx <= n_devices; dev_idx++) {
			if (BR_PCI_DEV_VALID(dev_idx - 1)) {
				execute_device(interp, dev_idx);

				if (strcmp(Tcl_GetVar2(interp, TCL_ENV, ENV_DRV_STATE, TCL_GLOBAL_ONLY), "RUNNING") == 0) {
					print_msg("WARNING: nvm upgrade command can not be performed while driver is running\n", ESC_COLOR_RED);
					printf("please unload device %d driver before using nvm upgrade\n", dev_idx);
					return TCL_ERROR;
				}
			}
		}

		// Go back to current device
		execute_device(interp, cur_dev_idx);
	}

#endif // !FWUPGTOOL

#ifdef FWUPGTOOL
	char *chip_rev;
	u32_t func_num;
	u32_t port_num;
	u32_t path_num;
	Tcl_Obj *obj;
	NVM_INTERFACE iNvm = nvm_interface(interp); // in wrapper.cpp

	if (common_nvm_get_port_num(&port_num) != COMMON_NVM_STATUS_SUCCESS) {
		HANDLE_ERROR(iNvm, TRUE, FWUPG_ADAPTER_NOT_FOUND);
		Tcl_AppendResult(interp, "Failed to get port number of current device.\n", NULL);
		return DIAG_STATUS_FAILURE;
	}

	func_num = COMMON_NVM_FUNC_NUM();
	path_num = COMMON_NVM_PATH_NUM();

#endif // FWUPGTOOL
	if ((objc < 3)) {
		return upgrade_usage(interp, objc, objv);
	}

	/* Get the sub_vendor_id from NVM configuration,
	Required later to decide if to prgram OCNVM image */
	common_nvm_get_nvm_offset_for_pci_id(&pci_id_offset, &pci_sub_id_offset);
	if (common_nvm_read_nvram(pci_sub_id_offset, &val32, sizeof(u32_t)) != COMMON_NVM_STATUS_SUCCESS) {
		Tcl_AppendResult(interp, "Reading VID from NVM failed.\n", NULL);
		return TCL_ERROR;
	}
	upgrade_params_ptr->sub_vendor_id = (u16_t)MANUF_INFO_PCI_SUBSYS_VENDOR_ID(val32);
	upgrade_params_ptr->sub_device_id = (u16_t)MANUF_INFO_PCI_SUBSYS_DEVICE_ID(val32);

	if (common_nvm_read_nvram(pci_id_offset, &val32, sizeof(u32_t)) != COMMON_NVM_STATUS_SUCCESS) {
		Tcl_AppendResult(interp, "Reading VID/DID from NVM failed.\n", NULL);
		return TCL_ERROR;
	}

	/* Update VID and DID*/
	upgrade_params_ptr->device_id = (u16_t)MANUF_INFO_PCI_DEVICE_ID(val32);
	upgrade_params_ptr->vendor_id = (u16_t)MANUF_INFO_PCI_VENDOR_ID(val32);

	do {
		for (idx = 1; idx < (u32_t)objc; idx++) {
			upgrade_params_ptr->option = Tcl_GetStringFromObj(objv[idx], NULL);

			if (strcmp(upgrade_params_ptr->option, "-F") == 0) {
				// there is '-F' flag
				upgrade_params_ptr->validate_versions = 0;
				if (objc < 4) {
					return upgrade_usage(interp, objc, objv);
				}
			} else if (upgrade_params_ptr->option[0] == '-') {
				// no '-F' flag
				idx++;

				// check if option is a known image type option
				for (i = 0; i < common_nvm_image_table_size(); i++) {
					if ((strcmp(upgrade_params_ptr->option, "-bc") == 0) || (strcmp(upgrade_params_ptr->option, "-ib") == 0) ||
						(strcmp(upgrade_params_ptr->option, "-ib_ipv4n6") == 0) || (strcmp(upgrade_params_ptr->option, "-ib_ipv6") == 0) ||
						(strcmp(upgrade_params_ptr->option, "-l2") == 0) || (strcmp(upgrade_params_ptr->option, "-mfw") == 0) ||
						(strcmp(upgrade_params_ptr->option, "-mbi") == 0) || (strcmp(upgrade_params_ptr->option, image_table[i].opt) == 0)) {
						// Found option in image type table options
						break;
					}
				}

				// did not find this option in image table, thus not valid option
				if (i == common_nvm_image_table_size()) {
					return upgrade_usage(interp, objc, objv);
				}

				if (strncmp(upgrade_params_ptr->option, "-ib", 3) == 0) {
					upgrade_params_ptr->iscsi_ib_option = 1;

					if (strcmp(upgrade_params_ptr->option, "-ib_ipv6") == 0) upgrade_params_ptr->ib_opt = IB_OPTION_IPV6;
					else if (strcmp(upgrade_params_ptr->option, "-ib_ipv4n6") == 0) upgrade_params_ptr->ib_opt = IB_OPTION_BOTH;
					else if (strcmp(upgrade_params_ptr->option, "-ib") == 0) upgrade_params_ptr->ib_opt = IB_OPTION_IPV4;

					/* Check for additional options */
					if ((strcmp(upgrade_params_ptr->option, "-ib") == 0) || (strcmp(upgrade_params_ptr->option, "-ibc") == 0)) {
						upgrade_params_ptr->forced_cfg = 1;
					} else if (strcmp(upgrade_params_ptr->option, "-ibpe") == 0) {
						upgrade_params_ptr->forced_prg = 1;
					} else {
						// Do nothing;
					}
				}

				/* Parse mba additional options '-d' '-p' '-n' */
				if (strcmp(upgrade_params_ptr->option, "-mba") == 0) {
					while (1) {
						ptr = Tcl_GetStringFromObj(objv[idx], NULL);
						if (ptr == NULL) {
							return upgrade_usage(interp, objc, objv);
						} else if (strcmp(ptr, "-d") == 0) {
							idx++;
							bypass_dev_check = 1;
						} else if (strcmp(ptr, "-p") == 0) {
							idx++;
							forced_pfn |= 0x1;
						} else if (strcmp(ptr, "-n") == 0) {
							idx++;
							if (Tcl_GetIntFromObj(interp, objv[idx], (int *)&val32) == TCL_ERROR) {
								return upgrade_usage(interp, objc, objv);
							}
							pfn_val = (u16_t)val32;
							forced_pfn |= 0x2;
							idx++;
						} else {
							/* Missing file name */
							if (idx == (u32_t)objc) {
								return upgrade_usage(interp, objc, objv);
							} else {
								break;
							}
						}
					}
				}

				/* Parse fcoe additional options */
				if (strcmp(upgrade_params_ptr->option, "-feb") == 0) {
					while (1) {
						/* Check for additional options after -feb*/
						ptr = Tcl_GetStringFromObj(objv[idx], NULL);
						if (ptr == NULL) {
							return upgrade_usage(interp, objc, objv);
						} else {
							if (strcmp(ptr, "-c") == 0) {
								idx++;
								upgrade_params_ptr->feb_forced_cfg = 1;
							} else if (strcmp(ptr, "-p") == 0) {
								idx++;
								upgrade_params_ptr->feb_forced_prg_evrst = 1;
							} else if (strcmp(ptr, "-cp") == 0) {
								idx++;
								upgrade_params_ptr->feb_forced_cfg = 1;
								upgrade_params_ptr->feb_forced_prg_evrst = 1;
							} else {
								/* Missing file name */
								if (idx == (u32_t)objc) {
									return upgrade_usage(interp, objc, objv);
								} else {
									break;
								}
							}
						}
					}
				}
			} else {
				return upgrade_usage(interp, objc, objv);
			}

			/* Find the file name */
			filename = Tcl_GetStringFromObj(objv[idx], NULL);
			if (filename == NULL) {
				return upgrade_usage(interp, objc, objv);
			}
		}

		/* Open the file and find out the size */
		if (get_file_size(filename, &file_size) != TCL_OK) {
			Tcl_AppendResult(interp, "File does not exist.\n", NULL);
			final_status = TCL_ERROR;
			break;
		}

		/* Allocate buffer according to the file_size, round up for multiple of 4 bytes
		 plus 4 bytes saved for CRC. */
		upgrade_params_ptr->buf = (u8_t *)Tcl_Alloc(((file_size + 3) & 0xfffffffc) + 4);
		if (upgrade_params_ptr->buf == NULL) {
			Tcl_AppendResult(interp, "No more host memory.\n", NULL);
			final_status = TCL_ERROR;
			break;
		}

		/* Clear the buffer */
		common_nvm_memset(upgrade_params_ptr->buf, 0, ((file_size + 3) & 0xfffffffc) + 4);

		if (strcmp(upgrade_params_ptr->option, "-m_pn") == 0) {
			/* Parse the modules & part numbers text file */
			upgrade_params_ptr->modules_buf = tcl_al_read_fmodules(filename, &upgrade_params_ptr->byte_read);
			if (upgrade_params_ptr->modules_buf == NULL) {
				Tcl_AppendResult(interp, "File not found or Improper file format\n", NULL);
				final_status = TCL_ERROR;
				break;
			}
		} else if (strcmp(upgrade_params_ptr->option, "-vpd") == 0) {
			/* Parse the VPD info text file */
			upgrade_params_ptr->vpd_buf = tcl_al_read_vpd_text_file(filename, &upgrade_params_ptr->byte_read);
			if (upgrade_params_ptr->vpd_buf == NULL) {
				Tcl_AppendResult(interp, "File not found or Improper file format\n", NULL);
				final_status = TCL_ERROR;
				break;
			}
		} else if (strcmp(upgrade_params_ptr->option, "-hw_set") == 0) {
			/* Parse the HW_SET text file */
			upgrade_params_ptr->hw_set_buf = tcl_al_read_hw_set_text_file(filename, &upgrade_params_ptr->byte_read);
			if (upgrade_params_ptr->hw_set_buf == NULL) {
				Tcl_AppendResult(interp, "File not found or Improper file format\n", NULL);
				final_status = TCL_ERROR;
				break;
			}
		} else {
			/* Parse the image to be upgraded bin file */
			if (read_bin_file(interp, filename, upgrade_params_ptr->buf, file_size, &upgrade_params_ptr->byte_read) != TCL_OK) {
				Tcl_AppendResult(interp, "Can't read file.\n", NULL);
				final_status = TCL_ERROR;
				break;
			}

			/* Check if trailer exist in the last dword of the input file */
			memcpy((u8_t *)&in_crc,  (u8_t *)&upgrade_params_ptr->buf[upgrade_params_ptr->byte_read - 8], sizeof(u32_t));
			memcpy((u8_t *)&trailer, (u8_t *)&upgrade_params_ptr->buf[upgrade_params_ptr->byte_read - 4], sizeof(u32_t));

			if (trailer == CRC_TRAILER) {
				memset(&upgrade_params_ptr->buf[upgrade_params_ptr->byte_read - 8], 0, 8);
				/* Compute CRC */
				crc32 = 0xffffffff;
				common_nvm_compute_crc((u8_t *)upgrade_params_ptr->buf, upgrade_params_ptr->byte_read - 8, PRG_NVM_LITTLE_ENDIAN, &crc32);

				if (in_crc != crc32) {
					printf("CRC check failed !! in_crc 0x%x computed 0x%x upgrade_params_ptr->byte_read %d\n", in_crc, crc32, upgrade_params_ptr->byte_read);
					return TCL_ERROR;
				} else {
					printf("CRC check pass \n");
					/* Don't program CRC and trailer */
					upgrade_params_ptr->byte_read -= 8;
				}
			} else {
				/* Do nothing until CRC check is forced again.
				   This feature was deffered. */
				/* 
			 if (upgrade_params_ptr->validate_versions) {    
				Tcl_AppendResult(interp, "ERROR: Missing CRC trailer in FW component! \n", 
										 "Please upgrade to T7.12 or above component or use -F to force upgrade.", NULL); 
				final_status = TCL_ERROR; 
				break; 
			} 
			*/
			}
		}

		/* iSCSI boot code upgrade is not supported in NO_PCI mode */
		if (strncmp(upgrade_params_ptr->option, "-ib", 3) == 0) {
			upgrade_params_ptr->iscsi_ib_option = 1;

#ifndef FWUPGTOOL
			// This does not apply to FWUPG tool.
			if (common_nvm_tcl_al_get_no_pci_mode() == 1) {
				printf("iSCSI boot code upgrade is disabled in \"no PCI\" mode\n");
				return TCL_ERROR;
			}
#endif // !FWUPGTOOL
		}

		if (strcmp(upgrade_params_ptr->option, "-ncsi") == 0) {
			// verify NCSI image completion
			if (verify_NCSI_images(interp, upgrade_params_ptr->byte_read, upgrade_params_ptr->buf) != TCL_OK) {
				Tcl_AppendResult(interp, "Improper file format, ",
								 "Not intended for -ncsi option.\n", NULL);
				final_status = TCL_ERROR;
				break;
			}
		}

		if (strcmp(upgrade_params_ptr->option, "-mba") == 0) {
			exp_rom_ptr = (exp_rom_hdr_t *)upgrade_params_ptr->buf;
			v_data_ptr = (vendor_data_t *)&upgrade_params_ptr->buf[BMAPI_BENDIAN_SWAP_S(exp_rom_ptr->VendorOffset)];

			// verify MBA image by checking signature exists
			if (verify_mba_image(interp, upgrade_params_ptr->buf) != TCL_OK) {
				Tcl_AppendResult(interp, "Improper file format, ",
								 "Not intended for -mba option.\n", NULL);
				final_status = TCL_ERROR;
				break;
			}

			if (forced_pfn) {
				if (forced_pfn & 0x1) {
					v_data_ptr->Flags |= VF_FORCE_PFN;
				}
				if (forced_pfn & 0x2) {
					v_data_ptr->forced_pfn = pfn_val;
				}
			}

			if (BMAPI_LENDIAN_SWAP_S(v_data_ptr->Flags) & VF_DIRS_FOLLOWED)  {
				attach_image_t *attach_img_ptr;
				attach_img_ptr = (attach_image_t *)((u8_t *)v_data_ptr +
													BMAPI_LENDIAN_SWAP_S(v_data_ptr->attachDirOffset));

				if (BMAPI_LENDIAN_SWAP_S(attach_img_ptr->dirs)) {
					/* CCM is present in MBA image, need to delete
					   stand alone CCM image from NVM (if exists) */
					common_nvm_delete_ccm_image();
				}
			}

			if (bypass_dev_check == 0) {
				/* Check the image and update the IDs */
				if (common_nvm_update_mba_hdr(upgrade_params_ptr->buf, upgrade_params_ptr->vendor_id, upgrade_params_ptr->device_id) != TCL_OK) {
					Tcl_AppendResult(interp, "Not a valid NetXtreme II MBA image.\n", NULL);
					break;
				}
			}
		}

		if (strcmp(upgrade_params_ptr->option, "-mbi") == 0) {
			u8_t upgrade_failed = 0;
			u8_t validate_versions = upgrade_params_ptr->validate_versions;
			ILT_Region_t *ILT_Region_ptr;
			u32_t offset, size;
			upgrade_params_ptr->mbi_upgrade = 1;
			/* verify MBI image */
			if (verify_mbi_image(interp, upgrade_params_ptr, &bitmask) != TCL_OK) {
				Tcl_AppendResult(interp, "Improper file format, ",
								 "Not intended for -mbi option.\n", NULL);
				final_status = TCL_ERROR;
				break;
			}
			ILT_Region_ptr = (ILT_Region_t *)(upgrade_params_ptr->buf + sizeof(ILT_Header_t));
			while (bitmask != 0) {
				if ((bitmask & 0x1) == 0x1) {
					u8_t phy_fw_upgrade = 0;
					char buffer[100];
					u8_t is_driver_loaded = 0;
					size = (ILT_Region_ptr->sizeHi << 16) + ILT_Region_ptr->sizeLo;
					offset = (ILT_Region_ptr->offsetHi << 16) + ILT_Region_ptr->offsetLo;
					ptr = upgrade_params_ptr->option;
					switch (ILT_Region_ptr->regionType) {
					case IMAGE_HDR_TYPE_MBA:
						upgrade_params_ptr->option = "-mba";
						break;
					case IMAGE_HDR_TYPE_MFW:
						upgrade_params_ptr->option = "-mfw";
						break;
					case IMAGE_HDR_TYPE_L2:
						upgrade_params_ptr->option = "-l2";
						break;
					case IMAGE_HDR_TYPE_CCM:
						upgrade_params_ptr->option = "-ccm";
						break;
					case IMAGE_HDR_TYPE_PHY_FW:
#ifdef EDEBUG
						printf("Phy firmware upgrade is not supported in ediag ...\n");
#else
						phy_fw_upgrade = 1;
						// Check driver state
						is_driver_loaded = (strcmp(Tcl_GetVar2(interp, TCL_ENV, ENV_DRV_STATE, TCL_GLOBAL_ONLY), "RUNNING") == 0) ? 1 : 0;

						// Incase -F flag and driver is not running, load the driver
						if (!is_driver_loaded) {
							printf("Loading driver ...\n");
							sprintf(&buffer[0], "driver load");
							Tcl_Eval(interp,  buffer);
						}
						if (dg_ext_phy_fw_upgrade_from_image(upgrade_params_ptr->buf + offset, size) != DIAG_STATUS_SUCCESS) {
							/* Print TCL result to synchronize Tcl_AppendResult and printf */
							printf((char *)Tcl_GetStringResult(interp));
							Tcl_ResetResult(interp);
							/* Try again once! and continue */
							upgrade_params_ptr->validate_versions = validate_versions;
							if (dg_ext_phy_fw_upgrade_from_image(upgrade_params_ptr->buf + offset, size) != DIAG_STATUS_SUCCESS) {
								upgrade_failed = 1;
							}
						}
						if (!is_driver_loaded) {
							printf("Unloading driver ...\n");
							sprintf(buffer, "driver unload");
							Tcl_Eval(interp,  buffer);
						}
#endif // EDEBUG
						break;
					case IMAGE_HDR_TYPE_ISCSI:
						upgrade_params_ptr->iscsi_ib_option = 1;
						upgrade_params_ptr->forced_cfg = 1;
						upgrade_params_ptr->ib_opt = IB_OPTION_BOTH;
						upgrade_params_ptr->option = "-ib_ipv4n6";
						break;
					case IMAGE_HDR_TYPE_FCOE:
						upgrade_params_ptr->option = "-feb";
						break;
					default:
						sprintf(buffer, "Unknown region type 0x%X. Ignoring current image.\n", ILT_Region_ptr->regionType);
						Tcl_AppendResult(interp, buffer, NULL);
						break;
					}
					if (phy_fw_upgrade == 0) {
						upgrade_params_ptr->buf += offset;
						val32 = upgrade_params_ptr->byte_read;
						upgrade_params_ptr->byte_read = size;
						printf("Upgrading image %s\n", &upgrade_params_ptr->option[1]);
						if (upgrade_image(client_data, interp, objc, objv, upgrade_params_ptr) != TCL_OK) {
							/* Print TCL result to synchronize Tcl_AppendResult and printf */
							printf((char *)Tcl_GetStringResult(interp));
							Tcl_ResetResult(interp);
							/* Try again once! and continue */
							upgrade_params_ptr->validate_versions = validate_versions;
							if (upgrade_image(client_data, interp, objc, objv, upgrade_params_ptr) != TCL_OK) {
								upgrade_failed = 1;
							}
						}
						upgrade_params_ptr->buf -= offset;
						upgrade_params_ptr->byte_read = val32;
						upgrade_params_ptr->iscsi_ib_option = 0;
						upgrade_params_ptr->forced_cfg = 0;
						upgrade_params_ptr->ib_opt = IB_OPTION_NONE;
					}
					upgrade_params_ptr->option = ptr;
					upgrade_params_ptr->validate_versions = validate_versions;
					/* Print TCL result to synchronize Tcl_AppendResult and printf */
					printf((char *)Tcl_GetStringResult(interp));
					Tcl_ResetResult(interp);
				}
				ILT_Region_ptr++;
				bitmask = bitmask >> 1;
			}
			if (upgrade_failed == 1) {
				final_status = TCL_ERROR;
				if (set_mbi_version(interp, 0, 0) != COMMON_NVM_STATUS_SUCCESS) {
					printf("Failed to write MBI version to NVM.\n");
					return TCL_ERROR;
				}
			} else {
				final_status = TCL_OK;
				if (set_mbi_version(interp, BYTE_SWAP(*(u32_t *)(upgrade_params_ptr->buf + OFFSETOF(ILT_Header_t, versionMajor))),
									BYTE_SWAP(*(u32_t *)(upgrade_params_ptr->buf + OFFSETOF(ILT_Header_t, build_month)))) != COMMON_NVM_STATUS_SUCCESS) {
					printf("Failed to write MBI version to NVM.\n");
					return TCL_ERROR;
				}
			}
			break;
		}

		/* find all non MFW and SWIM images in the MFW binary file, to perform correct calculation of empty nvm space */
		/* For exmaple: E3_PCIE, E#_WC, OCNVM etc. */
		if (strcmp(upgrade_params_ptr->option, "-mfw") == 0) {
			upgrade_params_ptr->image_types_in_input_mfw = (u32_t *)(uintptr_t)common_nvm_alloc_virt(common_nvm_image_table_size());
			if (upgrade_params_ptr->image_types_in_input_mfw == NULL) {
				DbgMessage(NULL, FATAL, "memory allocation error\n");
				return TCL_ERROR;
			}
			common_nvm_memset(upgrade_params_ptr->image_types_in_input_mfw, 0, common_nvm_image_table_size());
			total = 0;
			upgrade_params_ptr->input_mfw_images_found = 0;
			while (total < upgrade_params_ptr->byte_read) {
				file_image = (image_header_t *)&upgrade_params_ptr->buf[total];

				/* validate that first image in MFW is MFW1, as it is an assumption all over the code */
				// Need to swap each field within file_image individually...
				if ((total == 0) && (BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_MFW1)) {
					Tcl_AppendResult(interp, "MFW package must start with MFW1 image.\n", NULL);
					final_status = TCL_ERROR;
					break;
				}

				if ((BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_BC1) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_BC2) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_NCSI_CMN) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_NCSI_LIB_XI) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_NCSI_LIB_EV) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_SWIM1) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_SWIM2) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_SWIM3) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_SWIM4) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_SWIM5) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_SWIM6) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_SWIM7) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_SWIM8) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_MFW1) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_MFW2) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_MFW2_A) &&
					(BMAPI_BENDIAN_SWAP(file_image->type) != IMAGE_HDR_TYPE_MDUMP)) {
					/* if reached here - found a non MFW/SWIM image in the new binary */
					upgrade_params_ptr->image_types_in_input_mfw[upgrade_params_ptr->input_mfw_images_found] = BMAPI_BENDIAN_SWAP(file_image->type);
					upgrade_params_ptr->input_mfw_images_found++;
				}

				total += sizeof(image_header_t) + BMAPI_BENDIAN_SWAP(file_image->byte_cnt);
			}
		}
	} while (0);

	return final_status;
}

/*
 *------------------------------------------------------------------------
 * tcl_al_nvm_upgrade_cmd --
 *
 *     Implements the new Tcl "nvm upgrade" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
#ifndef FWUPGTOOL
static int
#else
int
#endif // !FWUPGTOOL
tcl_al_nvm_upgrade_cmd(ClientData client_data, Tcl_Interp *interp,
					   int objc, Tcl_Obj *CONST objv[]) {
	int tcl_status = TCL_OK;
	nvm_upgrade_params_t nvm_upgrade_params;
	memset(&nvm_upgrade_params, 0, sizeof(nvm_upgrade_params));
	nvm_upgrade_params.validate_versions = 1;

	tcl_status = parse_upgrade_cmd(client_data, interp, objc, objv, &nvm_upgrade_params);
	if (tcl_status != TCL_OK) {
		goto CLEANUP;
	}

	/* MBI upgrade is called from parse_upgrade_cmd - skip upgrade_image */
	if (nvm_upgrade_params.mbi_upgrade == 0) {
		tcl_status = upgrade_image(client_data, interp, objc, objv, &nvm_upgrade_params);
		if (tcl_status != TCL_OK) {
			goto CLEANUP;
		}
		/* Successful non MBI upgrade - MBI version should be removed */
		set_mbi_version(interp, 0, 0);
	}

CLEANUP:
	if (nvm_upgrade_params.buf != NULL) {
		Tcl_Free((char *)nvm_upgrade_params.buf);
	}

#ifdef FWUPGTOOL
	if (nvm_upgrade_params.modules_buf) free(nvm_upgrade_params.modules_buf); // for -m_pn case.
#endif // FWUPGTOOL

	return tcl_status;
}

/*
 *------------------------------------------------------------------------
 * toe_nvm_dump_cmd --
 *
 *     Implements the new Tcl "nvm dump" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
#ifndef FWUPGTOOL
static int
#else
int
#endif // !FWUPGTOOL
tcl_al_nvm_dump_cmd(ClientData client_data, Tcl_Interp *interp,
					int objc, Tcl_Obj *CONST objv[]) {
	u32_t offset_arg, len_arg, file_arg, length, offset /*,crc32, trailer=CRC_TRAILER*/;
	char *filename;
	u8_t *buf = NULL;
	int tcl_status, final_status = TCL_ERROR;
	u32_t byte_dumped, byte_written;
	char *option;
	int i;
	int real_options_idx[3];
	int num_of_real_options = 0;
	u8_t big_endian = 1, img_type = IMG_TYPE_ST;
	u8_t parse_modules_image = 0, parse_mdump_image = 0;
	u8_t parse_vpd_image = 0, parse_hw_set_image = 0;
	u32_t idx = 0, type = 0, entry_idx, image_idx;

	for (i = 1; i < objc; i++) {
		option = Tcl_GetStringFromObj(objv[i], NULL);
		if (option[0] == '-') {
			if (strcmp(option, "-l") == 0) {
				//LITTLE ENDIAN
				big_endian = 0;
			} else if (strcmp(option, "-b") == 0) {
				//BIG ENDIAN		-		*** default
				big_endian = 1;
			} else if (strcmp(option, "-s") == 0) {
				//ST FLASH TYPE		-		*** default
				img_type = IMG_TYPE_ST;
			} else if (strcmp(option, "-a") == 0) {
				//ATMEL FLASH TYPE
				img_type = IMG_TYPE_ATMEL;
			} else if (strcmp(option, "-m_pn") == 0) {
				parse_modules_image = 1;
			} else if (strcmp(option, "-vpd") == 0) {
				parse_vpd_image = 1;
			} else if (strcmp(option, "-hw_set") == 0) {
				parse_hw_set_image = 1;
			} else if (strcmp(option, "-mdump") == 0) {
				parse_mdump_image = 1;
			} else {
				goto dump_usage;
			}
		} else {
			//real option (filename offset length)
			real_options_idx[num_of_real_options++] = i;
		}
	}

	// real options (not flags can be 1 or 3 only)
	if (num_of_real_options == 1) {
		// nvm dump filename
		offset_arg = -1;
		len_arg = -1;
		file_arg = real_options_idx[0];
	} else if (num_of_real_options == 3) {
		// nvm dump offset length filename
		offset_arg      = real_options_idx[0];
		len_arg         = real_options_idx[1];
		file_arg        = real_options_idx[2];
	} else {
		goto dump_usage;
	}

	do {
		if (len_arg == -1) {
			if ((!parse_modules_image) && (!parse_vpd_image) &&
				(!parse_hw_set_image) && (!parse_mdump_image)) {
				length = common_nvm_nvm_size();
			} else if (parse_modules_image) {
				/* Incase of -m_pn option get the length of the MODULES_PN image */
				for (idx = 0; idx < common_nvm_image_table_size(); idx++) {
					if (strstr(image_table[idx].name, "MODULES_PN")) {
						break;
					}
				}
				if (idx < common_nvm_image_table_size()) {
					common_nvm_nvm_find_dir_entry(image_table[idx].type, &entry_idx, NULL);
					common_nvm_nvm_get_dir_info(entry_idx, &offset, &length, NULL, &type, NULL);
				} else {
#ifndef FWUPGTOOL
					print_msg("Dump -m_pn option failed! image MODULES_PN does not exist\n", ESC_COLOR_RED);
#endif
					return TCL_ERROR;
				}
			}  else if (parse_vpd_image) {
				/* Incase of -vpd option get the NVM offset and length of the VPD image */
				if (common_nvm_get_image_in_extended_dir_info(NVM_IMAGE_NAME_VPD, &offset, &length, &image_idx) !=
						COMMON_NVM_STATUS_SUCCESS) {
					Tcl_AppendResult(interp, "Can't dump VPD image, does not exist.\n", NULL);
					return TCL_ERROR;
				}

			} else if (parse_hw_set_image) {
				/* Incase of -hw_set option get the NVM offset and length of the HW_SET image */
				if (common_nvm_get_image_in_extended_dir_info(NVM_IMAGE_NAME_HW_SET, &offset, &length, &image_idx) !=
						COMMON_NVM_STATUS_SUCCESS) {
					Tcl_AppendResult(interp, "Can't dump HW_SWT image, does not exist.\n", NULL);
					return TCL_ERROR;
				}
			} else if (parse_mdump_image) {
				/* Incase of -mdump option get the NVM offset and length of the MDUMP image */
				if (common_nvm_get_image_in_extended_dir_info(NVM_IMAGE_NAME_MDUMP, &offset, &length, &image_idx) !=
						COMMON_NVM_STATUS_SUCCESS) {
					Tcl_AppendResult(interp, "Can't dump MDUMP image, does not exist.\n", NULL);
					return TCL_ERROR;
				}
			}

		} else {
			if (Tcl_GetIntFromObj(interp, objv[len_arg],
								  (int *)&length) == TCL_ERROR) {
				break;
			}
			length &= 0xfffffffc;
		}

		if (offset_arg == -1) {
			// Incase of dumping image modules offset was set above.
			// Otherwise, dump NVM from offset 0
			if ((!parse_modules_image) && (!parse_vpd_image) && (!parse_hw_set_image) && (!parse_mdump_image)) {
				offset = 0;
			}
		} else {
			if (Tcl_GetIntFromObj(interp, objv[offset_arg],
								  (int *)&offset) == TCL_ERROR) {
				break;
			}
			offset &= 0xfffffffc;
		}

		filename = Tcl_GetStringFromObj(objv[file_arg], NULL);
		if (filename == NULL) break;

		buf = (u8_t *)Tcl_Alloc(
			(img_type == IMG_TYPE_ST) ?
			length
			:
			// for ATMEL - for every 256 bytes add another 8 bytes.
			(length + (length / 256 * 8))
			);

		/* Append CRC feature was deffered to T7.12b */
		/*
		// Allocate 8 more bytes for CRC and TRAILER 
		buf = (u8_t *)Tcl_Alloc(
							   (img_type == IMG_TYPE_ST) ?
							   length + 8  
							   :
							   // for ATMEL - for every 256 bytes add another 8 bytes.
							   ((length+8) + ((length+8) / 256 * 8))
							   );
		*/
		if (buf == NULL) {
			Tcl_AppendResult(interp, "No more host memory.\n", NULL);
			break;
		}

		if (common_nvm_dump_nvm_into_buf(buf, big_endian, length, offset, &byte_dumped) != COMMON_NVM_STATUS_SUCCESS) {
			Tcl_AppendResult(interp, "Dump from NVM failed.\n", NULL);
			break;
		}

		if (img_type == IMG_TYPE_ATMEL) {
			if (atmel_8B_append(interp, buf, length, &byte_dumped) == TCL_ERROR) {
				final_status = TCL_ERROR;
			}
		}

		if (parse_vpd_image) {
			tcl_status = tcl_al_write_vpd_text_file(filename, buf, length);
		} else if (parse_modules_image) {
			tcl_status = tcl_al_write_fmodules(filename, buf);
		} else if (parse_hw_set_image) {
			tcl_status = tcl_al_write_hw_set_text_file(filename, buf);
		} else {
			/* Append CRC feature was deffered to T7.12b */
			/*
			// Add CRC and TRAILER at the end of the dumped buffer
			crc32 = 0xffffffff;
			common_nvm_compute_crc((u8_t *)buf, byte_dumped-8, PRG_NVM_LITTLE_ENDIAN, &crc32);

			memcpy((u8_t *) &buf[byte_dumped-8], (u8_t *) &crc32, sizeof(u32_t));
	
			memcpy((u8_t *) &buf[byte_dumped-4], (u8_t *) &trailer, sizeof(u32_t));
			
			printf("Append CRC 0x%x to end of file (cnt %d).\n", crc32, byte_dumped-8); 
			*/

			tcl_status = write_bin_file(interp, filename, buf, byte_dumped, &byte_written);
		}
		if (tcl_status != TCL_OK) {
			Tcl_AppendResult(interp, "Can't write to file, already exists?\n", NULL);
			break;
		}

		final_status = TCL_OK;
	} while (0);

	if (buf != NULL) Tcl_Free((char *)buf);
	return final_status;

dump_usage:
	Tcl_WrongNumArgs(interp, 1, objv,
					 "?"NVM_LITTLE_ENDIAN_FLAG"? offset length filename");
	return TCL_ERROR;
}


/* ************************************************************* */
/* ************* NVM programming utility functions ************* */
/* ************************************************************* */

/*
 *------------------------------------------------------------------------
 * Append8bytes --
 *
 *     For Atmel flash and 5710, append 8 bytes at end of each block
 *     to make block size of 264
 *
 * Results:
 *     Pass/Fail
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */


static u32_t
atmel_8B_append(
	Tcl_Interp *interp,
	u8_t *buf,
	u32_t length,
	u32_t *byte_dumped) {
	u32_t byte_len, i, buffer_size, nvm_size;
	u8_t * t_buf,*s_ptr,*d_ptr;

	t_buf = (u8_t *)Tcl_Alloc(length);
	if (t_buf == NULL) {
		Tcl_AppendResult(interp, "Append8bytes: No more host memory.\n", NULL);
		return TCL_ERROR;
	}
	memcpy(t_buf, buf, length);
	s_ptr = t_buf;
	d_ptr = buf;

	byte_len = *byte_dumped;
	*byte_dumped = 0;

	/* For NVM 16M use buffer size of 512 bytes, other wise buffer size is 256Bytes*/
	nvm_size = common_nvm_nvm_size() & 0xfffff000;
	if (nvm_size >= NVRAM_1MB_SIZE * 16) {
		buffer_size = 512;
	} else {
		buffer_size = 256;
	}

	DbgMessage(NULL, WARN, "Append bytes for every %d (0x%x) bytes\n", buffer_size, buffer_size);

	/* Append 8 bytes for every buffer */
	for (i = 0; i < byte_len; i += buffer_size) {
		if ((i + buffer_size) > byte_len) {
			memcpy(d_ptr, s_ptr, byte_len - i);
			*byte_dumped += byte_len - i;
		} else {
			memcpy(d_ptr, s_ptr, buffer_size);
			s_ptr += buffer_size;
			d_ptr += buffer_size;
			*((u32_t *)d_ptr) = 0x57105710;
			d_ptr += 4;
			*((u32_t *)d_ptr) = 0x57105710;
			d_ptr += 4;
			*byte_dumped += (buffer_size + 8);
			/* For 16M NVM should append 16 bytes */
			if (buffer_size == 512) {
				*((u32_t *)d_ptr) = 0x57105710;
				d_ptr += 4;
				*((u32_t *)d_ptr) = 0x57105710;
				d_ptr += 4;
				*byte_dumped += 8;
			}
		}
	}
	Tcl_Free((char *)t_buf);
	return TCL_OK;
}

/*
 *------------------------------------------------------------------------
 * Strip8bytes --
 *
 *     For Atmel flash and 5710, strip 8 bytes at end of each block.
 *
 * Results:
 *     Pass/Fail
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
static u32_t
atmel_8B_strip(
	Tcl_Interp *interp,
	u8_t *buf,
	u32_t length,
	u32_t *bytes_written) {
	u32_t i, total, buffer_size, nvm_size;
	u8_t * t_buf,*s_ptr,*d_ptr;

	t_buf = (u8_t *)Tcl_Alloc(length);
	if (t_buf == NULL) {
		Tcl_AppendResult(interp, "Strip8bytes: No more host memory.\n", NULL);
		return TCL_ERROR;
	}
	memcpy(t_buf, buf, length);
	s_ptr = t_buf;
	d_ptr = buf;
	total = 0;

	/* For NVM 16M use buffer size of 512 bytes, other wise buffer size is 256Bytes*/
	nvm_size = common_nvm_nvm_size() & 0xfffff000;
	if (nvm_size >= NVRAM_1MB_SIZE * 16) {
		buffer_size = 512;
	} else {
		buffer_size = 256;
	}

	for (i = 0; i < length; i += buffer_size) {
		if ((i + buffer_size) > length) {
			memcpy(d_ptr, s_ptr, length - i);
			total += (length - i);
		} else {
			memcpy(d_ptr, s_ptr, buffer_size);
			s_ptr += buffer_size;
			d_ptr += buffer_size;
			total += buffer_size;
			if (*((u32_t *)s_ptr) == 0x57105710) {
				s_ptr += 4;
				i += 4;
			}
			if (*((u32_t *)s_ptr) == 0x57105710) {
				s_ptr += 4;
				i += 4;
			}
			/* For 16M NVM should append 16 bytes */
			if (buffer_size == 512) {
				if (*((u32_t *)s_ptr) == 0x57105710) {
					s_ptr += 4;
					i += 4;
				}
				if (*((u32_t *)s_ptr) == 0x57105710) {
					s_ptr += 4;
					i += 4;
				}
			}
		}
	}
	*bytes_written = total;
	Tcl_Free((char *)t_buf);
	return TCL_OK;
}

/*
 *------------------------------------------------------------------------
 * toe_nvm_dir_cmd --
 *
 *     Implements the new Tcl "nvm dir" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
#ifndef FWUPGTOOL
static int
#else
int
#endif // !FWUPGTOOL
tcl_al_nvm_dir_cmd(ClientData client_data, Tcl_Interp *interp,
				   int objc, Tcl_Obj *CONST objv[]) {
	int rc = TCL_ERROR;
	int status = COMMON_NVM_STATUS_FAILURE;
	char *fmt_str, *name = NULL, *ptr, *cpuName = NULL,  *image_name, *ver_str;
	char ver_str_raw[64 + 3]; /* `+3' if for dword alignment */
	u32_t sram_addr, byte_cnt, nvm_addr, idx, tmp_addr;
	u32_t entry_idx;
	u32_t name_idx, is_delete, agent, type;
	u32_t mbi_version, mbi_date;
	u8_t image_in_extended_dir = 0;
	extended_dir_image_t extended_dir;
	u32_t no_images = 0;
	u32_t found_image;
	u32_t length = 0;
	char buf[4096];
	Tcl_Channel outChannel;
#ifdef EDEBUG
	FILE *fd;
#endif

	/* Clear the memory allocated for extended_dir */
	common_nvm_memset(&extended_dir, 0, sizeof(extended_dir_image_t));

	/* Align ver_str to 32-bit */
	ver_str = (char *)ALIGN_UP4_PTR(ver_str_raw);

	outChannel = Tcl_GetStdChannel(TCL_STDOUT);

	if (objc == 1) {
		/* Interactive display all entries */
		fmt_str = "%11s  0x%08X  0x%08X  0x%08X  %6s  %18s\n";
		if ((get_mbi_version(interp, &mbi_version, &mbi_date) == COMMON_NVM_STATUS_SUCCESS) && (mbi_version != 0) && (mbi_date != 0)) {
			length += sprintf(&buf[length], "MBI version: %d.%d.%d    MBI date: %d/%d/%d\n", ((mbi_version >> 24) & 0xff), ((mbi_version >> 16) & 0xff), ((mbi_version >> 8) & 0xff), ((mbi_date >> 24) & 0xff), ((mbi_date >> 16) & 0xff), BYTE_SWAP16(mbi_date & 0xffff));
			length += sprintf(&buf[length], "------------------------------------------------------------------------------\n");
		}
		length += sprintf(&buf[length], "%11s  %10s  %10s  %10s  %6s  %18s\n",
						  "Image", "SRAM Addr", "NVM Offset", "Byte Cnt",
						  "CPU", "Version");
		length += sprintf(&buf[length], "------------------------------------------------------------------------------\n");
		//DRAW_LINE('-');

		/* search for MFW image in extended dir */
		status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_MFW1, &nvm_addr, &byte_cnt, &entry_idx);

		/* Show BC1 information from bootstrap if MFW1 */
		if ((status != COMMON_NVM_STATUS_SUCCESS) || (!found_image)) {

			status = common_nvm_read_nvram(0x4, &sram_addr, sizeof(u32_t));
			status |= common_nvm_read_nvram(0x8, &byte_cnt, sizeof(u32_t));
			status |= common_nvm_read_nvram(0xc, &nvm_addr, sizeof(u32_t));
			if (status != COMMON_NVM_STATUS_SUCCESS) {
				Tcl_AppendResult(interp, "Failed to read from NVRAM\n", NULL);
				return TCL_ERROR;
			}
			if (byte_cnt != 0) {
				byte_cnt *= 4;
				nvm_addr = common_nvm_xlate_flash_virt_addr(nvm_addr);
#ifdef FWUPGTOOL
				// FWUPGTOOL requires this to work on the PPC64 system.
				common_nvm_fw_ver_string((diag_virt_addr_t)nvm_addr, image_table[0].name, ver_str, READ_VERSION_FROM_NVM);
#else
				common_nvm_fw_ver_string(nvm_addr, image_table[0].name, ver_str, READ_VERSION_FROM_NVM);
#endif
				length += sprintf(&buf[length], fmt_str, image_table[0].name, sram_addr, nvm_addr, byte_cnt, "MCP", ver_str);
			}
		}

		/* Get dir info on images in dir (idx 1-15) */
		for (idx = 1; idx < (CODE_ENTRY_MAX - 1); idx++) {
			status = common_nvm_nvm_find_dir_entry(image_table[idx].type, &entry_idx, NULL);
			if (status != 0) continue;

			status = common_nvm_nvm_get_dir_info(entry_idx, &nvm_addr, &byte_cnt,
												 &sram_addr, NULL, &agent);
			if ((status != 0) || (nvm_addr == NVM_ERASE_VALUE)) continue;

			if (strcmp(image_table[idx].name, NVM_IMAGE_NAME_MBA) == 0) {

				length += sprintf(&buf[length], "%11s  0x%08X  0x%08X  0x%08X  %6s", image_table[idx].name, sram_addr, nvm_addr,
								  byte_cnt & CODE_IMAGE_LENGTH_MASK,
								  exec_env_name[agent >> 24]);

#ifdef FWUPGTOOL
				// FWUPGTOOL requires this to work on the PPC64 system.
				common_nvm_fw_ver_string((diag_virt_addr_t)nvm_addr, image_table[idx].name, ver_str, READ_VERSION_FROM_NVM);
#else
				common_nvm_fw_ver_string(nvm_addr, image_table[idx].name, ver_str, READ_VERSION_FROM_NVM);
#endif
				length += print_mba_ver_string(ver_str, &buf[length]);
			} else {
#ifdef FWUPGTOOL
				// FWUPGTOOL requires this to work on the PPC64 system.
				common_nvm_fw_ver_string((diag_virt_addr_t)nvm_addr, image_table[idx].name, ver_str, READ_VERSION_FROM_NVM);
#else
				common_nvm_fw_ver_string(nvm_addr, image_table[idx].name, ver_str, READ_VERSION_FROM_NVM);
#endif
				image_name = image_table[idx].name;

				// Starting from NCSI version 1.0.7, there is only one single image
				// for NCSI. The name of the image is "NCSI".
				if (strcmp(image_table[idx].name, NVM_IMAGE_NAME_NCSI_CMN) == 0) {
					u32_t v_1, v_2, v_3;
					u8_t v_str[20];

					sscanf(ver_str, "%s %x.%x.%x", v_str, &v_1, &v_2, &v_3);
					if (v_1 * 10000 + v_2 * 100 + v_3 >= NEW_NCSI_START_VERSION) {
						image_name = "NCSI";
					}
				} else if (strcmp(image_table[idx].name, NVM_IMAGE_NAME_ISCSI_BOOT) == 0) {
					u32_t val;
					u8_t tmp_ver_str[64];

					memcpy(tmp_ver_str, ver_str, 20);
					memset(ver_str, 0, 64);
					memcpy(ver_str, "ipv4", 5);
					common_nvm_read_nvram(nvm_addr + 0x0c, (u32_t *)&val, 4);
					if ((val & 0xffff) == 0x86dd) ver_str[3] = '6';
					else if ((val & 0xffff) == 0xc0b0) memcpy(&ver_str[4], "_ipv6", 5);
					memcpy((u8_t *)(ver_str + strlen(ver_str)), " ", 1);
					memcpy((u8_t *)(ver_str + strlen(ver_str)), tmp_ver_str, 20);
				}

				length += sprintf(&buf[length], fmt_str, image_name, sram_addr, nvm_addr, byte_cnt, exec_env_name[agent >> 24], ver_str);
			}
		}


		/* Check if extended dir exist */
		if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &byte_cnt, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS) {
			agent = CODE_IMAGE_AGENT_HOST;

			if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
				/* If extended dir image exist, parse the extended dir image into structure
						   parse it into extended_dir_image struct */
				if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, byte_cnt, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
					Tcl_AppendResult(interp, "Failed to parse extended dir image\n", NULL);
					return TCL_ERROR;
				}

				/* Get extended dir info on images in extended dir (idx 16 and above) */
				for (idx = CODE_ENTRY_MAX - 1; idx < common_nvm_image_table_size(); idx++) {
					/* Search for this image type in extended dir */
					/* If found, get its info (nvm_addr, byte_cnt) */
					status = common_nvm_nvm_find_extended_dir_entry(image_table[idx].type, &extended_dir, &entry_idx, &nvm_addr, &sram_addr, &byte_cnt, &agent);
					if (status != COMMON_NVM_STATUS_SUCCESS) continue;

					/* Get this image version and name */
					common_nvm_fw_ver_string(nvm_addr, image_table[idx].name, ver_str, READ_VERSION_FROM_NVM);
					image_name = image_table[idx].name;

					/* print the info into screen */
					length += sprintf(&buf[length], fmt_str, image_name, sram_addr, nvm_addr, byte_cnt, exec_env_name[agent >> 24], ver_str);
				}
			}
		}

#ifndef FWUPGTOOL
		Tcl_WriteChars(outChannel, buf, -1);
		Tcl_Flush(outChannel);
#else
		// printf will be called as printfWrapper() in wrapper.cpp
		// which will output the buf to both stdout and log file.
		printf("%s", buf);
#endif

#ifdef EDEBUG
		if (tcl_al_get_special_log_mode()) {
			fd = fopen(SPECIAL_LOG_FILE, "a");
			if (fd) {
				fseek(fd, 0L, SEEK_END);
				fprintf(fd, "\n\nnvm dir output\n");
				fprintf(fd, "==============\n\n");
				fprintf(fd, "%s", buf);
				fprintf(fd, "\n\n");
			}
			fclose(fd);
		}
#endif

		rc = TCL_OK;
	} else if ((objc == 2) || (objc == 3)) {
		name_idx = 1;
		is_delete = 0;
		if (objc == 3) {
			if (strcmp(Tcl_GetString(objv[1]), "-delete") != 0) {
				goto nvm_dir_err;
			}
			name_idx++;
			is_delete = 1;
		}

		/* Find the right entry */
		name = Tcl_GetString(objv[name_idx]);
		if (strcmp(name, "MBI") == 0) {
			if ((get_mbi_version(interp, &mbi_version, &mbi_date) == COMMON_NVM_STATUS_SUCCESS) && (mbi_version != 0) && (mbi_date != 0)) {
				printf("MBI version: %d.%d.%d    MBI date: %d/%d/%d\n", ((mbi_version >> 24) & 0xff), ((mbi_version >> 16) & 0xff), ((mbi_version >> 8) & 0xff), ((mbi_date >> 24) & 0xff), ((mbi_date >> 16) & 0xff), BYTE_SWAP16(mbi_date & 0xffff));
			} else {
				printf("No MBI version\n");
			}
			return TCL_OK;
		}
		for (ptr = name; *ptr != '\0'; ptr++) *ptr = toupper(*ptr);
		for (idx = 0; idx < common_nvm_image_table_size(); idx++) {
			if (strstr(image_table[idx].name, name)) {
				name = image_table[idx].name;
				break;
			}
		}

		if (idx < (CODE_ENTRY_MAX - 1)) {
			/* Found the requested image name in image_table
			   Now, get the matching entry idx */
			status = common_nvm_nvm_find_dir_entry(image_table[idx].type, &entry_idx, NULL);
		} else {
			/* Check if extended dir exist */
			if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &byte_cnt, NULL, &type, NULL) == COMMON_NVM_STATUS_SUCCESS) {
				agent = CODE_IMAGE_AGENT_HOST;

				if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
					/* If extended dir image exist, parse the extended dir image into structure
					   parse it into extended_dir_image struct */
					if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, byte_cnt, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
						Tcl_AppendResult(interp, "Failed to parse extended dir image\n", NULL);
						return TCL_ERROR;
					}

					/* Search for this image in extended dir */
					status = common_nvm_nvm_find_extended_dir_entry(image_table[idx].type, &extended_dir,
																	&entry_idx, &tmp_addr, &sram_addr, &byte_cnt, &agent);

					if (status == COMMON_NVM_STATUS_SUCCESS) image_in_extended_dir = 1;
				}
			}
		}


		/* Determine what to do: retrive or delete */
		if (is_delete) {

			/* Handle deletion of image from dir */
			if (status == COMMON_NVM_STATUS_SUCCESS) {
				if (!image_in_extended_dir) {
					/* Erase the entry */
					status = common_nvm_update_dir_entry(entry_idx, NVM_ERASE_VALUE, NVM_ERASE_VALUE,
														 NVM_ERASE_VALUE, NVM_ERASE_VALUE, NVM_ERASE_VALUE);
					if (status != 0) {
						Tcl_AppendResult(interp, "Failed to update directory entry.\n", NULL);
						rc = TCL_ERROR;
					} else {
						rc = TCL_OK;
					}
				} else {
					no_images = extended_dir.no_images;
					/* If there are images left */
					if (no_images > 0) {

						/* if there are images left in extended dir, just delete the requested extended dir entry */
						status = common_nvm_nvm_delete_extended_dir_entry(&extended_dir, entry_idx, nvm_addr);
						if (status != 0) {
							Tcl_AppendResult(interp, "Failed to delete extended dir entry !!\n", NULL);
							rc = TCL_ERROR;
						} else {
							if (strcmp(image_table[idx].name, NVM_IMAGE_NAME_VPD) == 0) {
								// If VPD image was deleted update the $::sys(IS_VPD_IMAGE) accordingly
								Tcl_SetVar2(interp, TCL_ENV, ENV_IS_VPD, "0", TCL_GLOBAL_ONLY);
							}
							if (strcmp(image_table[idx].name, NVM_IMAGE_NAME_MFW1) == 0) {
								// If MFW image deleted, set bootstrap to zero as well.
								common_nvm_update_bootstrap(0, 0, 0);
							}
							rc = TCL_OK;
						}
					}

				}
			} else if (strcmp(name, image_table[0].name) == 0) {
				/* bootcode phase 1, zero out the length */
				// Fix for CQ#32763 Everest::A1::eDiag:: eDiag does not allow "-nvm dir -delete" of BC1
				// Replcaed the nvm write with update bootstrap
				//common_nvm_write_nvram(NVM_OFFSET(bootstrap.code_len), 0, sizeof(u32_t));
				common_nvm_update_bootstrap(0, 0, 0);
			} else {
				Tcl_AppendResult(interp, "No entry to delete from NVM.\n", NULL);
				rc = TCL_ERROR;
			}
		} else {
			/* Retrieve */
			if (status == COMMON_NVM_STATUS_SUCCESS) {
				if (!image_in_extended_dir) {
					status = common_nvm_nvm_get_dir_info(entry_idx, &nvm_addr, &byte_cnt,
														 &sram_addr, NULL, &agent);
					if ((status != 0) || (nvm_addr == NVM_ERASE_VALUE)) {
						Tcl_AppendResult(interp, "Failed to get directory entry.\n", NULL);
						rc = TCL_ERROR;
					} else {
						cpuName = exec_env_name[agent >> 24];
#ifdef FWUPGTOOL
						// FWUPGTOOL requires this to work on the PPC64 system.
						common_nvm_fw_ver_string((diag_virt_addr_t)nvm_addr, name, ver_str, READ_VERSION_FROM_NVM);
#else
						common_nvm_fw_ver_string(nvm_addr, name, ver_str, READ_VERSION_FROM_NVM);
#endif
						rc = TCL_OK;
					}
				} else {
					/* Get this image version and name */
					common_nvm_fw_ver_string(tmp_addr, name, ver_str, READ_VERSION_FROM_NVM);
					rc = TCL_OK;
				}
			} else if (strcmp(name, image_table[0].name) == 0) {
				/* Bootcode phase 1, get from bootstrap location */
				status = 0;
				status |= common_nvm_read_nvram(NVM_OFFSET(bootstrap.sram_start_addr), &sram_addr, sizeof(sram_addr));
				status |= common_nvm_read_nvram(NVM_OFFSET(bootstrap.code_len), &byte_cnt, sizeof(byte_cnt));
				status |= common_nvm_read_nvram(NVM_OFFSET(bootstrap.code_start_addr), &nvm_addr, sizeof(nvm_addr));

				/* Return error incase read failed or all bootstrap params are zero */
				if ((status != 0) || ((sram_addr == 0) && (byte_cnt == 0) && (nvm_addr == 0))) {
					Tcl_AppendResult(interp, "Failed to get bootstrap entry.\n", NULL);
					rc = TCL_ERROR;
				} else {
					byte_cnt *= 4;
					nvm_addr = common_nvm_xlate_flash_virt_addr(nvm_addr);
					cpuName = "MCP";
					agent = CODE_IMAGE_AGENT_MCP;
#ifdef FWUPGTOOL
					// FWUPGTOOL requires this to work on the PPC64 system.
					common_nvm_fw_ver_string((diag_virt_addr_t)nvm_addr, name, ver_str, READ_VERSION_FROM_NVM);
#else
					common_nvm_fw_ver_string(nvm_addr, name, ver_str, READ_VERSION_FROM_NVM);
#endif
					rc = TCL_OK;
				}
			} else {
				Tcl_AppendResult(interp, "Invalid image name: ", name, "\n", NULL);
				rc = TCL_ERROR;
			}

			/* If no error, build the list object. */
			if (rc == TCL_OK) {
				cpuName = exec_env_name[agent >> 24];
				if (image_in_extended_dir) {
					sprintf(buf, "%s 0x%X 0x%X 0x%X %s %s\n", name, sram_addr, tmp_addr, byte_cnt, cpuName, ver_str);
				} else {
					sprintf(buf, "%s 0x%X 0x%X 0x%X %s %s\n", name, sram_addr, nvm_addr, byte_cnt, cpuName, ver_str);
				}
#ifndef FWUPGTOOL
				Tcl_WriteChars(outChannel, buf, -1);
				Tcl_Flush(outChannel);
#else
				// printf will be called as printfWrapper() in wrapper.cpp
				// which will output the buf to both stdout and log file.
				printf("%s", buf);
#endif // !FWUPGTOOL

			}
		}
	} else {
		goto nvm_dir_err;
	}

	return TCL_OK;
nvm_dir_err:
	Tcl_WrongNumArgs(interp, 1, objv, "?delete? entry?");
	return TCL_ERROR;
}


#ifdef FWUPGTOOL

/*
 *------------------------------------------------------------------------
 * tcl_al_nvm_crc_check --
 *
 *     Implements the new Tcl "nvm crc" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */

int
tcl_al_nvm_crc_check(Tcl_Interp *interp) {
	typedef struct _block_entry_t {
		u8_t *name;
		u32_t offset;
		u32_t byte_cnt;
		u8_t  silent;
	} block_entry_t;

	block_entry_t block_table[] =
	{
		{ (u8_t *)NVM_IMAGE_NAME_BOOTSTRAP, NVM_OFFSET(bootstrap.magic_value),
			sizeof(bootstrap_region_t), 0 },
		{ (u8_t *)NVM_IMAGE_NAME_DIRECTORY, NVM_OFFSET(dir),
			sizeof(dir_t), 0 },
		{ (u8_t *)NVM_IMAGE_NAME_HW_INFO, NVM_OFFSET(manuf_info.length),
			sizeof(manuf_info_t), 0 },
		{ (u8_t *)NVM_IMAGE_NAME_FEATURE_INFO, NVM_OFFSET(feature_info),
			sizeof(feature_info_t), 0 },
		// This is done in tcl_al_nvm_crc_cmd...
		{ (u8_t *)NVM_IMAGE_NAME_MANUF_KEY_INFO_P0, NVM_OFFSET(manuf_key_info[0]),
			sizeof(manuf_key_info_t), 1 },
		{ (u8_t *)NVM_IMAGE_NAME_MANUF_KEY_INFO_P1, NVM_OFFSET(manuf_key_info[1]),
			sizeof(manuf_key_info_t), 1 },
		{ (u8_t *)NVM_IMAGE_NAME_UPGRADE_KEY_INFO_P0, NVM_OFFSET(upgrade_key_info[0]),
			sizeof(upgrade_key_info_t), 1 },
		{ (u8_t *)NVM_IMAGE_NAME_UPGRADE_KEY_INFO_P1, NVM_OFFSET(upgrade_key_info[1]),
			sizeof(upgrade_key_info_t), 1 }
	};

	u32_t status, offset, length, computed, stored;
	u32_t entry_idx, value32;
	int rc;
	u8_t * fmt_str,*err_msg;
	u32_t idx;


	fmt_str = (u8_t *)"%15s  0x%08X  0x%08X  0x%08X  0x%08X  %s\n";
	printf("%15s  %10s  %10s  %10s  %10s  %s\n", "Region", "Start",
		   "Length", "Content", "Computed", "Result");
	DRAW_LINE('-');

	err_msg = (u8_t *)"Failed to access NVM.";

	do {
		rc = TCL_OK;

		for (idx = 0; idx < sizeof(block_table) / sizeof(block_entry_t); idx++) {
			status = common_nvm_compute_image_crc(block_table[idx].offset,
												  block_table[idx].byte_cnt,
												  &computed, &stored);
			if (status != 0) {
				break;
			}

			block_table[idx].silent = 0;
			if (!block_table[idx].silent || (computed != stored)) {
				printf((char *)fmt_str, block_table[idx].name, block_table[idx].offset,
					   block_table[idx].byte_cnt, stored, computed,
					   (computed == stored) ? "OK" : "Mismatch");
			}
		}
		if (idx < sizeof(block_table) / sizeof(block_entry_t)) {
			break;
		}

		/* Check BC1 */
		status = 0;
		status |= common_nvm_read_nvram(0xc, &offset, sizeof(offset));
		status |= common_nvm_read_nvram(0x8, &length, sizeof(length));
		if (status != COMMON_NVM_STATUS_SUCCESS) {
			break;
		}


		if (length == 0) {
			/* BC1 legnth 0, bogus, skip */
		} else {
			offset = common_nvm_xlate_flash_virt_addr(offset);
			length *= 4;
			status = common_nvm_compute_image_crc(offset, length, &computed,
												  &stored);
			if (status != 0) break;

			printf((char *)fmt_str, image_table[0].name, offset, length, stored,
				   computed, (computed == stored) ? "OK" : "Mismatch");
		}

		/* Traverse directory to check other images*/
		for (idx = 1; idx < common_nvm_image_table_size(); idx++) {
			if (common_nvm_nvm_find_dir_entry(image_table[idx].type, &entry_idx, NULL) == -1) {
				continue;
			}
			status = common_nvm_nvm_get_dir_info(entry_idx, &offset, &length,
												 NULL, NULL, NULL);
			if (status != 0) {
				break;
			} else if (length == 0) {
				continue;
			}
			common_nvm_compute_image_crc(offset, length, &computed, &stored);
			printf((char *)fmt_str, image_table[idx].name, offset, length,
				   stored, computed, (computed == stored) ? "OK" : "Mismatch");
		}


		if (status == COMMON_NVM_STATUS_SUCCESS) {
			Tcl_ResetResult(interp);
			/* Let rc be returned */
		}

	} while (0);

	common_nvm_read_nvram(NVM_OFFSET(vpd), &value32, sizeof(value32));
	if ((value32 == 0) || (value32 == 0xffffffff)) {
		u32_t nvm_addr;
		vpd_info_t vpd_info;
		media_vpd_t vpd_block;
		common_nvm_status_t stat;

		common_nvm_vpd_set_defaults(&vpd_info, SET_DEF_ALL_PARMS);
		nvm_addr = NVM_OFFSET(vpd); /* first field */
		common_nvm_set_vpd_info(&vpd_info, &vpd_block);
		stat = common_nvm_program_nvm((u8_t *)&vpd_block, sizeof(media_vpd_t),
									  nvm_addr, 0, &value32);
#ifndef FWUPGTOOL
		// For FWUPG tool, we don't issue the MCP reset and update the device information.
#ifndef WIN_DIAG
		// Update device information of all devices and environment MAC addr.
		if (stat == COMMON_NVM_STATUS_SUCCESS) {
			common_nvm_refresh_devices();
			Tcl_AppendResult(interp, "Default VPD info programmed to NVM.\n", (char *)NULL);
		} else rc = TCL_ERROR;
#endif
#endif // !FWUPGTOOL

	}

	if (rc == TCL_ERROR) Tcl_AppendResult(interp, err_msg, (char *)NULL);
	return rc;
}
#endif // FWUPGTOOL


/*
 *------------------------------------------------------------------------
 * toe_nvm_crc_cmd --
 *
 *     Implements the new Tcl "nvm crc" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
#ifndef FWUPGTOOL
static int
#else
int
#endif // !FWUPGTOOL
tcl_al_nvm_crc_cmd(
	ClientData client_data,
	Tcl_Interp *interp,
	int objc,
	Tcl_Obj *CONST objv[]
	) {
	typedef struct _block_entry_t {
		char *name;
		u32_t offset;
		u32_t byte_cnt;
		u8_t  silent;
	} block_entry_t;

	block_entry_t block_table[] = {
		{ NVM_IMAGE_NAME_BOOTSTRAP, NVM_OFFSET(bootstrap.magic_value),
			sizeof(bootstrap_region_t), 0 },
		{ NVM_IMAGE_NAME_DIRECTORY, NVM_OFFSET(dir),
			sizeof(dir_t), 0 },
		{ NVM_IMAGE_NAME_HW_INFO, NVM_OFFSET(manuf_info.length),
			sizeof(manuf_info_t), 0 },
		{ NVM_IMAGE_NAME_FEATURE_INFO, NVM_OFFSET(feature_info),
			sizeof(feature_info_t), 0 },
		{ NVM_IMAGE_NAME_PATH1_HW_INFO, PATH1_NVM_OFFSET(manuf_info),
			sizeof(manuf_info_t), 0 },
		{ NVM_IMAGE_NAME_PATH1_FEATURE_INFO, PATH1_NVM_OFFSET(feature_info),
			sizeof(feature_info_t), 0 },
		{ NVM_IMAGE_NAME_MANUF_KEY_INFO_P0, NVM_OFFSET(manuf_key_info[0]),
			sizeof(manuf_key_info_t), 1 },
		{ NVM_IMAGE_NAME_MANUF_KEY_INFO_P1, NVM_OFFSET(manuf_key_info[1]),
			sizeof(manuf_key_info_t), 1 },
		{ NVM_IMAGE_NAME_UPGRADE_KEY_INFO_P0, NVM_OFFSET(upgrade_key_info[0]),
			sizeof(upgrade_key_info_t), 1 },
		{ NVM_IMAGE_NAME_UPGRADE_KEY_INFO_P1, NVM_OFFSET(upgrade_key_info[1]),
			sizeof(upgrade_key_info_t), 1 }
	};

	u32_t status, offset, length, computed, stored, type;
	u32_t name_idx, idx, need_fix, entry_idx, value32, is_evst1x = 0;
	int rc;
	char *fmt_str, *name, *err_msg;
	Tcl_Obj * result_obj,*tmp_obj;
	extended_dir_image_t extended_dir;
	u32_t total_func, found_image, nvm_addr, byte_cnt;

#ifndef FWUPGTOOL
	is_evst1x = is_e1x(globalInterp);
#else
	NVM_INTERFACE iNvm = nvm_interface(interp); // in wrapper.cpp

	is_evst1x = COMMON_NVM_IS_E1x();
#endif // !FWUPGTOOL

	common_nvm_memset(&extended_dir, 0, sizeof(extended_dir_image_t));

	if (objc == 1) {
		fmt_str = "%15s  0x%08X  0x%08X  0x%08X  0x%08X  %s\n";
		printf("%15s  %10s  %10s  %10s  %10s  %s\n", "Region", "Start",
			   "Length", "Content", "Computed", "Result");
		DRAW_LINE('-');

		err_msg = "Failed to access NVM.";

		do {
			rc = TCL_OK;

			for (idx = 0; idx < sizeof(block_table) / sizeof(block_entry_t); idx++) {
				status = common_nvm_compute_image_crc(block_table[idx].offset,
													  block_table[idx].byte_cnt,
													  &computed, &stored);
				if (status != 0) {
					break;
				}

				if (is_evst1x) {
					// If not E2, don't show FEAT_INFO_2, HW_INFO_2
					if (strcmp(block_table[idx].name, NVM_IMAGE_NAME_PATH1_FEATURE_INFO) == 0) {
						continue;
					}
					if (strcmp(block_table[idx].name, NVM_IMAGE_NAME_PATH1_HW_INFO) == 0) {
						continue;
					}
				}

				// Get the number of functions of this board
				total_func = common_nvm_number_of_everest_functions();
				if ((total_func < 2) || (!is_evst1x)) {
					// If there is no second func or if E2, don't show MKEY_INFO_P1, UKEY_INFO_P1
					if ((strcmp(block_table[idx].name, NVM_IMAGE_NAME_MANUF_KEY_INFO_P1) == 0) ||
						(strcmp(block_table[idx].name, NVM_IMAGE_NAME_UPGRADE_KEY_INFO_P1) == 0)) {
						continue;
					}
				}

				if (!block_table[idx].silent || (computed != stored)) {
					printf(fmt_str, block_table[idx].name, block_table[idx].offset,
						   block_table[idx].byte_cnt, stored, computed,
						   (computed == stored) ? "OK" : "Mismatch");
				}
			}
			if (idx < sizeof(block_table) / sizeof(block_entry_t)) {
				break;
			}

			/* Check BC1 */
			/* search for MFW image in extended dir */
			status = common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_MFW1, &nvm_addr, &byte_cnt, &entry_idx);

			/* Show BC1 information from bootstrap if MFW1 */
			if ((status != COMMON_NVM_STATUS_SUCCESS) || (!found_image)) {
				status = 0;
				status |= common_nvm_read_nvram(0xc, &offset, sizeof(offset));
				status |= common_nvm_read_nvram(0x8, &length, sizeof(length));
				if (status != COMMON_NVM_STATUS_SUCCESS) {
					break;
				}

				if (length == 0) {
					/* BC1 with length 0, bogus, skip */
				} else {
					offset = common_nvm_xlate_flash_virt_addr(offset);
					length *= 4;
					status = common_nvm_compute_image_crc(offset, length, &computed,
														  &stored);
					if (status != 0) break;

					printf(fmt_str, image_table[0].name, offset, length, stored,
						   computed, (computed == stored) ? "OK" : "Mismatch");
				}
			}

			/* Traverse directory to check other images*/
			for (idx = 1; idx < (CODE_ENTRY_MAX - 1); idx++) {
				if (common_nvm_nvm_find_dir_entry(image_table[idx].type, &entry_idx, NULL) == -1) {
					continue;
				}
				status = common_nvm_nvm_get_dir_info(entry_idx, &offset, &length,
													 NULL, NULL, NULL);
				if (status != 0) {
					break;
				}

				common_nvm_compute_image_crc(offset, length, &computed, &stored);
				printf(fmt_str, image_table[idx].name, offset, length,
					   stored, computed, (computed == stored) ? "OK" : "Mismatch");
			}

			/* Check if extended dir exist */
			status = common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &offset, &length, NULL, &type, NULL);
			if (status == COMMON_NVM_STATUS_SUCCESS) {
				if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
					/* Calculate extended_dir_image crc */
					common_nvm_compute_image_crc(offset, length, &computed, &stored);
					printf(fmt_str, "EXTENDED_DIR", offset, length,
						   stored, computed, (computed == stored) ? "OK" : "Mismatch");

					/* If extended dir image exist, parse the extended dir image into structure
					   parse it into extended_dir_image struct */
					if (common_nvm_nvm_parse_extended_dir_image(offset, length, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
						Tcl_AppendResult(interp, "Failed to parse extended dir image.\n", NULL);
						return TCL_ERROR;
					}

					/* Get extended dir info on images in extended dir (idx 16 and above) */
					for (idx = (CODE_ENTRY_MAX - 1); idx < common_nvm_image_table_size(); idx++) {
						/* Search for this image type in extended dir */
						/* If found, get its info (nvm_addr, byte_cnt) */
						status = common_nvm_nvm_find_extended_dir_entry(image_table[idx].type, &extended_dir, &entry_idx, &offset, NULL, &length, NULL);
						if (status != COMMON_NVM_STATUS_SUCCESS) continue;

						common_nvm_compute_image_crc(offset, length, &computed, &stored);

						// Do not attempt to compute CRC for NIVPROFILES image
						// Temporary do not check CRC for BDN image
						if ((image_table[idx].type == CODE_IMAGE_VNTAG_PROFILES_DATA) ||
							(image_table[idx].type == CODE_IMAGE_TYPE_ISCSI_PERS) ||
							(image_table[idx].type == CODE_IMAGE_TYPE_BDN) ||
							(image_table[idx].type == CODE_IMAGE_TYPE_MDUMP)) {
							printf(fmt_str, image_table[idx].name, offset, length,
								   stored, computed, "N/A");
							continue;
						}

						printf(fmt_str, image_table[idx].name, offset, length,
							   stored, computed, (computed == stored) ? "OK" : "Mismatch");
					}
				}
			}

			if (status == COMMON_NVM_STATUS_SUCCESS) {
				Tcl_ResetResult(interp);
				/* Let rc be returned */
			}

		} while (0);

	} else if ((objc == 2) || (objc == 3)) {
		name_idx = 1;
		need_fix = 0;
		if (objc == 3) {
			if (strcmp(Tcl_GetString(objv[1]), "-fix") != 0) {
				goto nvm_crc_err;
			}
			name_idx++;
			need_fix = 1;
		}
		name = Tcl_GetString(objv[name_idx]);
		for (idx = 0; idx < strlen(name); idx++) name[idx] = toupper(name[idx]);

		err_msg = "Failed to access NVM.";
		offset = -1;
		/* Get the entry: obtain the NVM addr and image length */
		for (idx = 0; idx < sizeof(block_table) / sizeof(block_entry_t); idx++) {
			if (strstr(block_table[idx].name, name) != NULL) {
				offset = block_table[idx].offset;
				length = block_table[idx].byte_cnt;
				break;
			}
		}

		if (idx < sizeof(block_table) / sizeof(block_entry_t)) {
			/* Entry already found, skip all "else if". */
		} else if (strstr(image_table[0].name, name) != NULL) {
			status = 0;
			status |= common_nvm_read_nvram(0xc, &offset, sizeof(offset));
			status |= common_nvm_read_nvram(0x8, &length, sizeof(length));
			if (status == COMMON_NVM_STATUS_SUCCESS) {
				offset = common_nvm_xlate_flash_virt_addr(offset);
				length *= 4;
			} else {
				offset = -1;
			}
		} else {
			for (idx = 1; idx < (CODE_ENTRY_MAX - 1); idx++) {
				if (strstr(image_table[idx].name, name)) {
					break;
				}
			}
			if (idx < (CODE_ENTRY_MAX - 2)) {
				common_nvm_nvm_find_dir_entry(image_table[idx].type, &entry_idx, NULL);
				status = common_nvm_nvm_get_dir_info(entry_idx, &offset, &length,
													 NULL, NULL, NULL);
				if (status != COMMON_NVM_STATUS_SUCCESS) offset = -1;
			} else {
				/* If image is not found in dir, look at extended dir */
				/* Check if extended dir exist */
				status = common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &offset, &length, NULL, &type, NULL);
				if (status == COMMON_NVM_STATUS_SUCCESS) {
					if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
						/* If extended dir image exist, parse the extended dir image into structure
				   parse it into extended_dir_image struct */
						if (common_nvm_nvm_parse_extended_dir_image(offset, length, &extended_dir) != COMMON_NVM_STATUS_SUCCESS) {
							Tcl_AppendResult(interp, "Failed to parse extended dir image.\n", NULL);
							return TCL_ERROR;
						}

						/* Get extended dir info on images in extended dir (idx 16 and above) */
						for (idx = (CODE_ENTRY_MAX - 1); idx < common_nvm_image_table_size(); idx++) {
							/* Search for this image type in extended dir */
							if (strstr(image_table[idx].name, name)) {
								break;
							}
						}

						if (idx < common_nvm_image_table_size()) {
							status = common_nvm_nvm_find_extended_dir_entry(image_table[idx].type, &extended_dir, &entry_idx, &offset, NULL, &length, NULL);
							if (status != COMMON_NVM_STATUS_SUCCESS) offset = -1;
						} else {
							err_msg = "No such entry found.";
							offset = -1;
						}
					}
				} else {
					err_msg = "No such entry found.";
					offset = -1;
				}
			}
		}

		do {
			rc = TCL_ERROR;
			if (offset == -1) break;
			err_msg = "Failed to access NVM.";

			/* Run through buffer to compute CRC and check against result */
			status = common_nvm_compute_image_crc(offset, length, &computed, &stored);
			if (status != 0) break;

			/* Construct the result list */
			result_obj = Tcl_NewObj();
			if (computed == stored) {
				/* Return "OK" and the CRC value */
				Tcl_ListObjAppendElement(interp, result_obj, Tcl_NewStringObj("OK", -1));
				tmp_obj = Tcl_NewIntObj(computed);
#ifndef FWUPGTOOL
				common_nvm_generate_hex_int_object(&tmp_obj, computed);

#else
				iNvm.m_Tcl_SetIntObjDispType(tmp_obj, 1);
#endif // !FWUPGTOOL
				Tcl_ListObjAppendElement(interp, result_obj, tmp_obj);
			} else {
				if (need_fix) {
					/* Update NVM and return "Fixed" */
					status = common_nvm_write_nvram(offset + length - 4, &computed, sizeof(computed));
					if (status != 0) break;
					Tcl_ListObjAppendElement(interp, result_obj, Tcl_NewStringObj("Fixed", -1));
					tmp_obj = Tcl_NewIntObj(computed);
#ifndef FWUPGTOOL
					common_nvm_generate_hex_int_object(&tmp_obj, computed);
#else
					iNvm.m_Tcl_SetIntObjDispType(tmp_obj, 1);
#endif // !FWUPGTOOL
					Tcl_ListObjAppendElement(interp, result_obj, tmp_obj);
				} else {
					/* Return expected and stored results */
					tmp_obj = Tcl_NewIntObj(computed);
#ifndef FWUPGTOOL
					common_nvm_generate_hex_int_object(&tmp_obj, computed);

#else
					iNvm.m_Tcl_SetIntObjDispType(tmp_obj, 1);
#endif // !FWUPGTOOL
					Tcl_ListObjAppendElement(interp, result_obj, tmp_obj);
					tmp_obj = Tcl_NewIntObj(stored);
#ifndef FWUPGTOOL
					common_nvm_generate_hex_int_object(&tmp_obj, stored);

#else
					iNvm.m_Tcl_SetIntObjDispType(tmp_obj, 1);
#endif // !FWUPGTOOL
					Tcl_ListObjAppendElement(interp, result_obj, tmp_obj);
				}
			}
			Tcl_SetObjResult(interp, result_obj);
			rc = TCL_OK;
		} while (0);

	} else {
		/* Wrong argument count */
		goto nvm_crc_err;
	}


	common_nvm_read_nvram(NVM_OFFSET(vpd), &value32, sizeof(value32));
	if ((value32 == 0) || (value32 == 0xffffffff)) {
		u32_t nvm_addr;
		vpd_info_t vpd_info;
		media_vpd_t vpd_block;
		common_nvm_status_t stat;

		common_nvm_vpd_set_defaults(&vpd_info, SET_DEF_ALL_PARMS);
		nvm_addr = NVM_OFFSET(vpd); /* first field */
		common_nvm_set_vpd_info(&vpd_info, &vpd_block);
		stat = common_nvm_program_nvm((u8_t *)&vpd_block, sizeof(media_vpd_t),
									  nvm_addr, 0, &value32);
#ifndef FWUPGTOOL
		// For FWUPG tool, we don't issue the MCP reset and update the device information.
#ifndef WIN_DIAG
		// Update device information of all devices and environment MAC addr.
		if (stat == COMMON_NVM_STATUS_SUCCESS) {
			common_nvm_refresh_devices();
			Tcl_AppendResult(interp, "Default VPD info programmed to NVM.\n", (char *)NULL);
		} else rc = TCL_ERROR;
#endif
#endif // !FWUPGTOOL
	}

	if (rc == TCL_ERROR) Tcl_AppendResult(interp, err_msg, (char *)NULL);
	return rc;
nvm_crc_err:
	Tcl_WrongNumArgs(interp, 1, objv, "\?\?-fix\? entry\?");
	return TCL_ERROR;
}

/*
 *------------------------------------------------------------------------
 * tcl_al_nvm_usrblk_cmd --
 *
 *     Implements the new Tcl "nvm usrblk" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
int
tcl_al_nvm_usrblk_cmd(ClientData client_data,
					  Tcl_Interp *interp,
					  int objc,
					  Tcl_Obj *CONST objv[]) {
	u32_t usr_blk_size = 0, rc, size, nvm_addr, usr_blk_nvm_addr;
	u32_t idx, image_table_idx = 54, byte_cnt;
	u32_t found_image, entry_idx;
	extended_dir_image_t extended_dir;
	u8_t *buf = NULL;
	char *filename = NULL, *option = NULL;

	if (objc != 3) {
		Tcl_WrongNumArgs(interp, 1, objv, "-s <size> or -f <file name>");
		return TCL_ERROR;
	}

	for (idx = 1; idx < (u32_t)objc; idx++) {
		option = Tcl_GetStringFromObj(objv[idx++], NULL);

		/* Incase USR_BLK size */
		if (strcmp(option, "-s") == 0) {
			if (Tcl_GetIntFromObj(interp, objv[idx], (int *)&usr_blk_size)
				== TCL_ERROR) {
				return TCL_ERROR;
			}
		}

		/* Incase USR_BLK file name */
		else if (strcmp(option, "-f") == 0) {
			filename = Tcl_GetStringFromObj(objv[idx], NULL);
			if (filename == NULL) {
				return TCL_ERROR;
			} else {
				/* Open the file and find out the size */
				if (get_file_size(filename, &usr_blk_size) != TCL_OK) {
					Tcl_AppendResult(interp, "File does not exist.\n", NULL);
					return TCL_ERROR;
				}
			}
		} else {
			Tcl_WrongNumArgs(interp, 1, objv, "-s <size> or -f <file name>");
			return TCL_ERROR;
		}
	}

	if (usr_blk_size & 0x3) {
		Tcl_AppendResult(interp, "Byte count must be a multiple of four bytes.\n", NULL);
		return TCL_ERROR;
	}

	/* Allocate the buffer */
	buf = (u8_t *)Tcl_Alloc(usr_blk_size);
	common_nvm_memset(buf, 0, usr_blk_size);

	/* In case user provided a USR_BLK file, try to read it */
	if (filename != NULL) {
		if (read_bin_file(interp, filename, buf, usr_blk_size, NULL) != TCL_OK) {
			Tcl_AppendResult(interp, "Can't read file.\n", NULL);
			return TCL_ERROR;
		}
	}

	/* Find user block in NVRAM in extended dir */
	common_nvm_nvm_find_image_in_extended_dir(&found_image,
											  CODE_IMAGE_TYPE_USR_BLK,
											  &usr_blk_nvm_addr,
											  &size,
											  &entry_idx);

	if (found_image) {
		size -= 4;
		if (size < usr_blk_size) {
			/* Collect the user block content */
			if (common_nvm_read_nvram(usr_blk_nvm_addr, (u32_t *)buf, size) != COMMON_NVM_STATUS_SUCCESS) {
				Tcl_AppendResult(interp, "Error reading user block from NVRAM.\n", NULL);
				rc = TCL_ERROR;
				goto usrblk_cleanup;
			}

			/* Parse extended dir */
			common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &byte_cnt, NULL, NULL, NULL);

			common_nvm_nvm_parse_extended_dir_image(nvm_addr, byte_cnt, &extended_dir);

			/* Delete user block */
			if (common_nvm_nvm_delete_extended_dir_entry(&extended_dir, entry_idx, nvm_addr)
				!= COMMON_NVM_STATUS_SUCCESS) {
				Tcl_AppendResult(interp, "Error deleting user block from NVRAM.\n", NULL);
				rc = TCL_ERROR;
				goto usrblk_cleanup;
			}
		} else {
			Tcl_AppendResult(interp, "A bigger user block exists.\n", NULL);
			rc = TCL_ERROR;
			goto usrblk_cleanup;
		}
	}

	/* Get the user block index in image_table */
	for (idx = 0; idx < common_nvm_image_table_size(); idx++) {
		if (strstr(image_table[idx].name, NVM_IMAGE_NAME_USR_BLK)) {
			image_table_idx = idx;
			break;
		}
	}

	/* Program user block */
	if (common_nvm_prog_image_in_extended_dir(buf,
											  usr_blk_size,
											  image_table_idx,
											  0,
											  0,
											  0,
											  FALSE) != COMMON_NVM_STATUS_SUCCESS) {
		Tcl_AppendResult(interp, "Failed to program user block !!\n");
		return TCL_ERROR;
	}

	rc = TCL_OK;

usrblk_cleanup:
	if (buf) {
		Tcl_Free((char *)buf);
		buf = NULL;
	}

	return rc;
}

/* ************************************************************* */
/* ************* NVM programming utility functions ************* */
/* ************************************************************* */


/*
 *------------------------------------------------------------------------
 * get_file_size --
 *     Get size of a file
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
static u32_t
get_file_size(
	char *filename,
	u32_t *size
	) {
	Tcl_Interp *interp = Tcl_CreateInterp();
	u32_t status;
#ifdef DOS
	Tcl_Obj *obj_array[3];
#else
	char tmp_str[256];
#endif

	if (size == NULL) return TCL_ERROR;

#ifdef DOS
	obj_array[0] = Tcl_NewStringObj("file", -1);
	obj_array[1] = Tcl_NewStringObj("size", -1);
	obj_array[2] = Tcl_NewStringObj(filename, -1);
	status = Tcl_FileObjCmd(NULL, interp, 3, obj_array);
#else
	sprintf(tmp_str, "file size %s\n", filename);
	status = Tcl_Eval(interp, tmp_str);
#endif
	if (status != TCL_OK) return status;

	/* Get the result */
	status = Tcl_GetIntFromObj(interp, Tcl_GetObjResult(interp), (int *)size);
	if (status != TCL_OK) return status;

#ifdef DOS
	/* Free the objects */
	Tcl_DecrRefCount(obj_array[0]);
	Tcl_DecrRefCount(obj_array[1]);
	Tcl_DecrRefCount(obj_array[2]);
#endif
	Tcl_DeleteInterp(interp);

	return TCL_OK;
}

/*
 *------------------------------------------------------------------------
 * read_bin_file --
 *     Read file content into a pre-allocated byte array.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
static u32_t
read_bin_file(
	Tcl_Interp *interp,
	char *filename,
	u8_t *buf,
	u32_t size,
	u32_t *byte_read
	) {
	Tcl_Obj *tclobj = Tcl_NewObj();
	Tcl_Channel chan = NULL;
	int tcl_status, final_status = TCL_ERROR;
	u32_t cnt;

	if (byte_read != NULL) *byte_read = 0;

	do {
		/* Open the file */
		chan = Tcl_OpenFileChannel(interp, filename, "r", 0);
		if (chan == NULL) break;

		tcl_status = Tcl_SetChannelOption(interp, chan, "-translation", "binary");
		if (tcl_status != TCL_OK) break;

		tcl_status = Tcl_SetChannelOption(interp, chan, "-encoding", "binary");
		if (tcl_status != TCL_OK) break;

		cnt = Tcl_ReadChars(chan, tclobj, size, 0);
		if (byte_read != NULL) *byte_read = (cnt == -1) ? 0 : cnt;
		if (cnt == -1) break;

		memcpy(buf, Tcl_GetByteArrayFromObj(tclobj, NULL), cnt);

		final_status = TCL_OK;
	} while (0);

	//printf("read_bin_file: file read %d bytes; buf=%p\n", cnt, buf);

	if (chan != NULL) Tcl_Close(interp, chan);

	Tcl_DecrRefCount(tclobj);
	tclobj = NULL;
	return final_status;
}

/*
 *------------------------------------------------------------------------
 * write_bin_file --
 *     Write content from a byte array to a file.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
static u32_t
write_bin_file(
	Tcl_Interp *interp,
	char *filename,
	u8_t *buf,
	u32_t size,
	u32_t *byte_written
	) {
	Tcl_Channel chan = NULL;
	int tcl_status, final_status = TCL_ERROR;
	u32_t cnt;
	Tcl_Obj *obj_ptr;

	if (byte_written != NULL) *byte_written = 0;

	do {
		/* Check if it exists */
		tcl_status = get_file_size(filename, &cnt); /* cnt serves as a dummy */
		if (tcl_status == TCL_OK) break;

		/* Open the file */
#ifndef UEFI
		chan = Tcl_OpenFileChannel(interp, filename, "w", 0);
#else
		chan = Tcl_OpenFileChannel(interp, filename, "w", S_IRWXU);
#endif
		if (chan == NULL) break;

		tcl_status = Tcl_SetChannelOption(interp, chan, "-translation", "binary");
		if (tcl_status != TCL_OK) break;

		tcl_status = Tcl_SetChannelOption(interp, chan, "-encoding", "binary");
		if (tcl_status != TCL_OK) break;

		obj_ptr = Tcl_NewByteArrayObj(buf, size);
		cnt = Tcl_WriteObj(chan, obj_ptr);
		if (byte_written != NULL) *byte_written = (cnt == -1) ? 0 : cnt;
		if (cnt == -1) break;

		final_status = TCL_OK;

	} while (0);

	if (chan != NULL) Tcl_Close(interp, chan);

#ifndef FWUPGTOOL
	printf("write_bin_file: file write %d bytes; buf=%p\n", cnt, buf);
	tcl_io_flush();
#else
	gpINx2Nvm->m_toe_flush();
#endif // !FWUPGTOOL

	return final_status;
}

static u32_t
print_mba_ver_string(
	char *str_ptr,
	char *buf
	) {
	char *temp_ptr, ver_ptr[32];
	u8_t i, isfirst = 1;
	u32_t len;
	u32_t ver_len = 0;

	for (i = 0; i < ROMIMG_NUM_MAX; i++) {
		temp_ptr = strstr(str_ptr, ";");
		if (temp_ptr == NULL) {
			if (isfirst) {
				ver_len += sprintf(buf + ver_len, "%20s\n", str_ptr);
				isfirst = 0;
			} else ver_len += sprintf(buf + ver_len, "%74s\n", str_ptr);
			break;
		} else {
			len = (u32_t)(temp_ptr - str_ptr);
			strncpy(ver_ptr, str_ptr, len);
			ver_ptr[len] = '\0';
			if (isfirst) {
				ver_len += sprintf(buf + ver_len, "%20s\n", ver_ptr);
				isfirst = 0;
			} else ver_len += sprintf(buf + ver_len, "%74s\n", ver_ptr);
		}
		str_ptr = temp_ptr + 1;
	}
	return ver_len;
}

/* Check if VPD image exists */
u32_t is_vpd_image_exist(void) {

	u32_t idx = 0, type = 0, entry_idx;
	u32_t nvm_addr, length, byte_cnt, offset;
	extended_dir_image_t extended_dir;

	/* Get the VPD image index in image_table[] */
	for (idx = 0; idx < common_nvm_image_table_size(); idx++) {
		if (strstr(image_table[idx].name, "VPD")) {
			break;
		}
	}

	if (idx < common_nvm_image_table_size()) {
		/* Get the length of the VPD image from the extended dir */
		if (common_nvm_nvm_get_dir_info(CODE_ENTRY_EXTENDED_DIR_IDX, &nvm_addr, &byte_cnt, NULL, &type, NULL)
			== COMMON_NVM_STATUS_SUCCESS) {

			if (type == CODE_IMAGE_TYPE_EXTENDED_DIR) {
				/* If extended dir image exist, parse the extended dir image into structure
				   parse it into extended_dir_image struct */
				if (common_nvm_nvm_parse_extended_dir_image(nvm_addr, byte_cnt, &extended_dir)
					!= COMMON_NVM_STATUS_SUCCESS) {
					/* Failed to parse the extended dir */
					return FALSE;
				}

				/* Search for this image in extended dir */
				if (common_nvm_nvm_find_extended_dir_entry(image_table[idx].type, &extended_dir, &entry_idx, &offset, NULL, &length, NULL)
					!= COMMON_NVM_STATUS_SUCCESS) {
					/* VPD image does not exist in the extended dir */
					return FALSE;
				} else {
					return TRUE;
				}

			} else {
				/* Extended dir does not exist */
				return FALSE;
			}
		}
	}
	return FALSE;
}

#ifndef FWUPGTOOL
/* nvm command */
/*
 *------------------------------------------------------------------------
 * tcl_al_nvm_cmd --
 *
 *     Implements the new Tcl "nvm" command.
 *
 * Results:
 *     A standard Tcl result
 *
 * Side effects:
 *     None.
 *------------------------------------------------------------------------
 */
int
tcl_al_nvm_cmd(client_data, interp, objc, objv)
ClientData client_data; /* Not used. */
Tcl_Interp *interp;     /* current interpreter */
int objc;               /* number of arguments */
Tcl_Obj *CONST objv[];  /* argument strings */
{
	static char *option_names[] = { "read", "disp", "write", "show", "fill",
		"upgrade", "cfg", "prg", "dump", "dir",
		"crc", "chk", "usrblk",
		(char *)NULL };
	enum options {
		READ, DISPLAY, WRITE, SHOW, FILL, UPGRADE, CFG, PRG, DUMP,
		DIRECTORY, CRC, CHK, USRBLK
	};
	int index;
	/* used to create the tcl command for nvm cfg */
	int        iter;
	int status;
	char buffer[1024];
	Tcl_Obj *strObj = Tcl_NewObj();
	g_no_ctrl_keys = FALSE;

	if (objc < 2) {
		Tcl_WrongNumArgs(interp, 1, objv, "option=(read|write|show|fill|upgrade|cfg|prg|dump|dir|usrblk) arg...");
		return TCL_ERROR;
	}

	if (Tcl_GetIndexFromObj(interp, objv[1], (COMMON_NVM_CHAR_PP)option_names, "option", TCL_EXACT,
							&index) != TCL_OK) {
		return TCL_ERROR;
	}
	if (IS_VF_HDL(common_nvm_cur_dev)) {
		printf("nvm operations are not valid regarding VF devices");
		return TCL_ERROR;
	}

	/* before each nvm operation, update IS_NIV and IS_NPAR env vars if needed */
	tcl_al_update_isnpar_env_vals(interp);

	switch ((enum options)index) {
	case READ:
		status = tcl_al_nvm_read_cmd(client_data, interp, objc - 1, objv + 1);
#ifdef EDEBUG
		if (status && (common_nvm_tcl_al_get_no_pci_mode() != 0)) {
			tcl_al_reinit_non_pci_mode(interp, uart_com, uart_baud);
			printf("\n\n serial connection was broken. Recovered... \n\n");
			status = tcl_al_nvm_read_cmd(client_data, interp, objc - 1, objv + 1);
		}
#endif
		break;
	case DISPLAY:
	case SHOW:
		status = tcl_al_nvm_show_cmd(client_data, interp, objc - 1, objv + 1);
#ifdef EDEBUG
		if (status && (common_nvm_tcl_al_get_no_pci_mode() != 0)) {
			tcl_al_reinit_non_pci_mode(interp, uart_com, uart_baud);
			printf("\n\n serial connection was broken. Recovered... \n\n");
			status = tcl_al_nvm_show_cmd(client_data, interp, objc - 1, objv + 1);
		}
#endif
		break;
	case WRITE:
		status = tcl_al_nvm_write_cmd(client_data, interp, objc - 1, objv + 1);
#ifdef EDEBUG
		if (status && (common_nvm_tcl_al_get_no_pci_mode() != 0)) {
			tcl_al_reinit_non_pci_mode(interp, uart_com, uart_baud);
			printf("\n\n serial connection was broken. Recovered... \n\n");
			status = tcl_al_nvm_write_cmd(client_data, interp, objc - 1, objv + 1);
		}
#endif
		break;
	case FILL:
		status = tcl_al_nvm_fill_cmd(client_data, interp, objc - 1, objv + 1);
#ifdef EDEBUG
		if (status && (common_nvm_tcl_al_get_no_pci_mode() != 0)) {
			tcl_al_reinit_non_pci_mode(interp, uart_com, uart_baud);
			printf("\n\n serial connection was broken. Recovered... \n\n");
			status = tcl_al_nvm_fill_cmd(client_data, interp, objc - 1, objv + 1);
		}
#endif
		break;
	case DUMP:
		status = tcl_al_nvm_dump_cmd(client_data, interp, objc - 1, objv + 1);
#ifdef EDEBUG
		if (status && (common_nvm_tcl_al_get_no_pci_mode() != 0)) {
			tcl_al_reinit_non_pci_mode(interp, uart_com, uart_baud);
			printf("\n\n serial connection was broken. Recovered... \n\n");
			status = tcl_al_nvm_dump_cmd(client_data, interp, objc - 1, objv + 1);
		}
#endif
		break;
	case UPGRADE:
		status = tcl_al_nvm_upgrade_cmd(client_data, interp, objc - 1, objv + 1);
#ifdef EDEBUG
		if (status && (common_nvm_tcl_al_get_no_pci_mode() != 0)) {
			tcl_al_reinit_non_pci_mode(interp, uart_com, uart_baud);
			printf("\n\n serial connection was broken. Recovered... \n\n");
			status = tcl_al_nvm_upgrade_cmd(client_data, interp, objc - 1, objv + 1);
		}
#endif
		break;
	case DIRECTORY:
		status = tcl_al_nvm_dir_cmd(client_data, interp, objc - 1, objv + 1);
#ifdef EDEBUG
		if (status && (common_nvm_tcl_al_get_no_pci_mode() != 0)) {
			tcl_al_reinit_non_pci_mode(interp, uart_com, uart_baud);
			printf("\n\n serial connection was broken. Recovered... \n\n");
			status = tcl_al_nvm_dir_cmd(client_data, interp, objc - 1, objv + 1);
		}
#endif
		break;
	case CFG:
		/* Disable supporting control keys such as F3, Arrow up
		   during "nvm cfg" parsing cause of a bug in "history" */
		g_no_ctrl_keys = TRUE;

		for (iter = 2; iter < objc; iter++) {
			Tcl_AppendToObj(strObj, "{", 1);
			Tcl_AppendObjToObj(strObj, objv[iter]);
			Tcl_AppendToObj(strObj, "} ", 2);
		}
		sprintf(buffer, "%s %s", NVM_CFG, Tcl_GetStringFromObj(strObj, NULL));
		status = Tcl_Eval(interp, buffer);
		break;

	case CHK:
		for (iter = 2; iter < objc; iter++) {
			Tcl_AppendToObj(strObj, "{", 1);
			Tcl_AppendObjToObj(strObj, objv[iter]);
			Tcl_AppendToObj(strObj, "} ", 2);
		}
		sprintf(buffer, "%s %s", NVM_CHK, Tcl_GetStringFromObj(strObj, NULL));
		status = Tcl_Eval(interp, buffer);
		break;
	case CRC:
		status = tcl_al_nvm_crc_cmd(client_data, interp, objc - 1, objv + 1);
#ifdef EDEBUG
		if (status && (common_nvm_tcl_al_get_no_pci_mode() != 0)) {
			tcl_al_reinit_non_pci_mode(interp, uart_com, uart_baud);
			printf("\n\n serial connection was broken. Recovered... \n\n");
			status = tcl_al_nvm_crc_cmd(client_data, interp, objc - 1, objv + 1);
		}
#endif
		break;
	case PRG:
		status = tcl_al_nvm_prg_cmd(client_data, interp, objc - 1, objv + 1);
#ifdef EDEBUG
		if (status && (common_nvm_tcl_al_get_no_pci_mode() != 0)) {
			tcl_al_reinit_non_pci_mode(interp, uart_com, uart_baud);
			printf("\n\n serial connection was broken. Recovered... \n\n");
			status = tcl_al_nvm_prg_cmd(client_data, interp, objc - 1, objv + 1);
		}
#endif
		break;

	case USRBLK:
		status = tcl_al_nvm_usrblk_cmd(client_data, interp, objc - 1, objv + 1);
		break;
	default:
		status = TCL_ERROR;
		break;
	}

	/* after each nvm operation, update IS_NIV and IS_NPAR env vars if needed */
	tcl_al_update_isnpar_env_vals(interp);

	return status;
}
#endif // !FWUPGTOOL


#ifdef FWUPGTOOL

// These functions are used by FWUPG tool.

// UMP is no longer supported in BCM57710.
// util_hasUMP_57710() will always return 0.
int
util_hasUMP_57710(NVM_INTERFACE iNvm) {
	int         iRet = 0;

	return (iRet);
}


// util_hasIPMI_57710() returns 1 if the NIC contains IPMI firmware;
// otherwise, it returns 0.
int
util_hasIPMI_57710(NVM_INTERFACE iNvm) {
	U32         idx;

	if (common_nvm_nvm_find_dir_entry(CODE_IMAGE_TYPE_IPMI, &idx, NULL) == COMMON_NVM_STATUS_SUCCESS) return 1;

	return 0;
}


// util_hasMFW_57710() returns 1 if the NIC contains MFW firmware;
// otherwise, it returns 0.
int
util_hasMFW_57710(NVM_INTERFACE iNvm) {
	u32_t       found_image = 0, offset = 0, cnt = 0, entry_idx;

//    printf( "util_hasMFW_57710: check for MFW1\n" );
// MFW is in NVRAM only when both MFW1 and MFW2 are existing in the directory.
	if ((common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_MFW1, &offset, &cnt, &entry_idx)
		 != COMMON_NVM_STATUS_SUCCESS) || (found_image != 1)) return 0;
//    printf( "util_hasMFW_57710: has MFW1\n" );
	if ((common_nvm_nvm_find_image_in_extended_dir(&found_image, CODE_IMAGE_TYPE_MFW2, &offset, &cnt, &entry_idx)
		 != COMMON_NVM_STATUS_SUCCESS) || (found_image != 1)) return 0;
//    printf( "util_hasMFW_57710: has MFW1 and MFW2\n" );

	return 1;
}


// util_hasNCSI_57710() returns 1 if the NIC contains NCSI firmware;
// otherwise, it returns 0.
int
util_hasNCSI_57710(NVM_INTERFACE iNvm) {
	U32         idx;

	if (common_nvm_nvm_find_dir_entry(CODE_IMAGE_TYPE_NC_SI_CMN, &idx, NULL) == COMMON_NVM_STATUS_SUCCESS) return 1;

	// NCSI firmware is now included in MFW firmware
	// Let's check for MFW firmware too.
	if (util_hasMFW_57710(iNvm)) return 1;

	return 0;
}


// util_hasPXE_57710() returns 1 if the NIC contains PXE firmware;
// otherwise, it returns 0.
int
util_hasPXE_57710(NVM_INTERFACE iNvm) {
	U32         idx;

	if (common_nvm_nvm_find_dir_entry(CODE_IMAGE_TYPE_MBA, &idx, NULL) == COMMON_NVM_STATUS_SUCCESS) return 1;

	return 0;
}


/*
	set_wol_cfg --

	Change the state of WOL configuration setting in the NVRAM.
	This function was mainly added for windows firmware upgrade tool.

	Results:
		Will return TCL_ERROR in case of invalid device selection.
		Will return TCL_OK only if successfully changed the WOL setting.

	Side effects:
		None.
 */
int set_wol_cfg_57710(NVM_INTERFACE iNvm, int nEnable) {
	// Currently, the nvm_cfg_t structures are identical for 5706/5708 and 57710
	nvm_cfg_t cfg;
	nvm_cfg_t shmem_cfg;
	nvm_nic_part_cfg_t mac_partition_cfg;
	u32_t nvm_cfg_extended_shared[NVM_CFG_EXTENDED_SHARED_IMAGE_MAX_SIZE];

	common_nvm_load_cfg(&cfg, &shmem_cfg, &mac_partition_cfg, nvm_cfg_extended_shared);

	if (nEnable) {
		cfg.port_feat.config |= PORT_FEATURE_WOL_ENABLED;
	} else {
		cfg.port_feat.config &= ~PORT_FEATURE_WOL_ENABLED;
	}

	if (common_nvm_store_cfg(&cfg, &shmem_cfg, &mac_partition_cfg, nvm_cfg_extended_shared) != COMMON_NVM_STATUS_SUCCESS) {
		printf("Error, new config updates not saved.\n");
		return TCL_ERROR;
	}

	iNvm.m_tm_exec_cmd(iNvm.m_tcl_interp, (u8_t *)"_reset", TM_CURR_DEVICE);

	return TCL_OK;
}

static int
is_valid_option_value_57710(u32_t param_index, u32_t param_value) {
	switch (param_index) {
	case IDX_LED_MODE: // 16
		param_value = (param_value << SHARED_HW_CFG_LED_MODE_SHIFT);
		param_value = (param_value & SHARED_HW_CFG_LED_MODE_MASK);
		switch (param_value) {
		case SHARED_HW_CFG_LED_MAC1:
		case SHARED_HW_CFG_LED_PHY1:
		case SHARED_HW_CFG_LED_PHY2:
		case SHARED_HW_CFG_LED_PHY3:
		case SHARED_HW_CFG_LED_MAC2:
		case SHARED_HW_CFG_LED_PHY4:
		case SHARED_HW_CFG_LED_PHY5:
		case SHARED_HW_CFG_LED_PHY6:
		case SHARED_HW_CFG_LED_MAC3:
		case SHARED_HW_CFG_LED_PHY7:
		case SHARED_HW_CFG_LED_PHY9:
		case SHARED_HW_CFG_LED_PHY11:
		case SHARED_HW_CFG_LED_MAC4:
		case SHARED_HW_CFG_LED_PHY8:
		case SHARED_HW_CFG_LED_EXTPHY1:
			break;
		default:
			return 0;
		}
		break;

	case IDX_SPEED_CAP_MASK_D3: // 36
		param_value = (param_value << PORT_HW_CFG_SPEED_CAPABILITY_D3_SHIFT);
		param_value = (param_value & PORT_HW_CFG_SPEED_CAPABILITY_D3_MASK);

		if ((param_value & PORT_HW_CFG_SPEED_CAPABILITY_D3_RESERVED) != 0) return 0; // reserve bits were set...
		break;

	case IDX_MFW_WOL_10G_OVERCLK: // 59, Mfw Wol Link Speed
		param_value = (param_value << PORT_FEATURE_LINK_SPEED_SHIFT);
		param_value = (param_value & PORT_FEATURE_LINK_SPEED_MASK);

		switch (param_value) {
		case PORT_FEATURE_LINK_SPEED_AUTO:
		case PORT_FEATURE_LINK_SPEED_10M_FULL:
		case PORT_FEATURE_LINK_SPEED_10M_HALF:
		case PORT_FEATURE_LINK_SPEED_100M_HALF:
		case PORT_FEATURE_LINK_SPEED_100M_FULL:
		case PORT_FEATURE_LINK_SPEED_1G:
		case PORT_FEATURE_LINK_SPEED_2_5G:
		case PORT_FEATURE_LINK_SPEED_10G_CX4:
			break;
		default:
			return 0;
		}
		break;

	case IDX_PHY_SELECTION: // 104
		param_value = (param_value << PORT_HW_CFG_PHY_SELECTION_SHIFT);
		param_value = (param_value & PORT_HW_CFG_PHY_SELECTION_MASK);

		switch (param_value) {
		case PORT_HW_CFG_PHY_SELECTION_HARDWARE_DEFAULT:
		case PORT_HW_CFG_PHY_SELECTION_FIRST_PHY:
		case PORT_HW_CFG_PHY_SELECTION_SECOND_PHY:
		case PORT_HW_CFG_PHY_SELECTION_FIRST_PHY_PRIORITY:
		case PORT_HW_CFG_PHY_SELECTION_SECOND_PHY_PRIORITY:
			break;
		default:
			return 0;
		}
		break;

	case IDX_SECOND_EXT_PHY_SPEED_CAP_MASK_D3: // 109
		param_value = (param_value << PORT_HW_CFG_SPEED_CAPABILITY2_D3_SHIFT);
		param_value = (param_value & PORT_HW_CFG_SPEED_CAPABILITY2_D3_MASK);

		if ((param_value & 0x0000F000) != 0) return 0; // reserve bits were set...
		break;

	default:
		printf("Unsupported Parameter\n");
		return 0;
	}

	return 1;
}


static int
update_cfg_value_57710(nvm_cfg_t *pCfg, u32_t param_index, u32_t param_value) {
//    printf( "CFG_HIDDEN_OPTION received, param_index = 0x%x, param_value = 0x%x\n",
//        param_index, param_value );

	if (!is_valid_option_value_57710(param_index, param_value)) {
		printf("Error: invalid param_value = 0x%x for param_index = %d\n", param_value, param_index);
		return TCL_ERROR;
	}

	/* Scan the string and store scanned value, based on index value,
	 * into the structure*/
	switch (param_index) {
	case IDX_LED_MODE: // 16
//            printf( "IDX_LED_MODE Parameter, value = 0x%x\n", param_value );
		pCfg->shared_manuf.config &= ~SHARED_HW_CFG_LED_MODE_MASK;
		pCfg->shared_manuf.config |= (param_value << SHARED_HW_CFG_LED_MODE_SHIFT);
		break;
	case IDX_SPEED_CAP_MASK_D3: // 36
//            printf( "IDX_SPEED_CAP_MASK_D3 Parameter, value = 0x%x\n", param_value );
		pCfg->port_manuf.speed_capability_mask &= ~PORT_HW_CFG_SPEED_CAPABILITY_D3_MASK;
		pCfg->port_manuf.speed_capability_mask |= (param_value << PORT_HW_CFG_SPEED_CAPABILITY_D3_SHIFT);
		break;
	case IDX_MFW_WOL_10G_OVERCLK: // 59, Mfw Wol Link Speed
//            printf( "IDX_MFW_WOL_10G_OVERCLK Parameter, value = 0x%x\n", param_value );
		pCfg->port_feat.mfw_wol_link_cfg &= ~PORT_FEATURE_LINK_SPEED_MASK; // clean up the old setting.
		pCfg->port_feat.mfw_wol_link_cfg |= (param_value << PORT_FEATURE_LINK_SPEED_SHIFT);
		break;
	case IDX_PHY_SELECTION: // 104
//            printf( "IDX_PHY_SELECTION Parameter, value = 0x%x\n", param_value );
		pCfg->port_manuf.multi_phy_config &= ~PORT_HW_CFG_PHY_SELECTION_MASK;
		pCfg->port_manuf.multi_phy_config |= (param_value << PORT_HW_CFG_PHY_SELECTION_SHIFT);
		break;
	case IDX_SECOND_EXT_PHY_SPEED_CAP_MASK_D3: // 109
//            printf( "IDX_SECOND_EXT_PHY_SPEED_CAP_MASK_D3 Parameter, value = 0x%x\n", param_value );
		pCfg->port_manuf.speed_capability_mask2 &= ~PORT_HW_CFG_SPEED_CAPABILITY2_D3_MASK;
		pCfg->port_manuf.speed_capability_mask2 |= (param_value << PORT_HW_CFG_SPEED_CAPABILITY2_D3_SHIFT);
		break;
	default:
		printf("Unsupported Parameter\n");
		return TCL_ERROR;
	}

	return TCL_OK;
}


static int
show_cfg_value_57710(nvm_cfg_t *pCfg, u32_t param_index) {
	U32 param_value;

//    printf( "show_cfg_value_57710: param_index = 0x%x\n", param_index );

	/* Scan the string and store scanned value, based on index value,
	 * into the structure*/
	switch (param_index) {
	case IDX_LED_MODE: // 16
		param_value = (pCfg->shared_manuf.config & SHARED_HW_CFG_LED_MODE_MASK);
		param_value = (param_value >> SHARED_HW_CFG_LED_MODE_SHIFT);
		break;
	case IDX_SPEED_CAP_MASK_D3: // 36
		param_value = (pCfg->port_manuf.speed_capability_mask & PORT_HW_CFG_SPEED_CAPABILITY_D3_MASK);
		param_value = (param_value >> PORT_HW_CFG_SPEED_CAPABILITY_D3_SHIFT);
		break;
	case IDX_MFW_WOL_10G_OVERCLK: // 59, Mfw Wol Link Speed
		param_value = (pCfg->port_feat.mfw_wol_link_cfg & PORT_FEATURE_LINK_SPEED_MASK);
		param_value = (param_value >> PORT_FEATURE_LINK_SPEED_SHIFT);
		break;
	case IDX_PHY_SELECTION: // 104
		param_value = (pCfg->port_manuf.multi_phy_config & PORT_HW_CFG_PHY_SELECTION_MASK);
		param_value = (param_value >> PORT_HW_CFG_PHY_SELECTION_SHIFT);
		break;
	case IDX_SECOND_EXT_PHY_SPEED_CAP_MASK_D3: // 109
		param_value = (pCfg->port_manuf.speed_capability_mask2 & PORT_HW_CFG_SPEED_CAPABILITY2_D3_MASK);
		param_value = (param_value >> PORT_HW_CFG_SPEED_CAPABILITY2_D3_SHIFT);
		break;
	default:
		printf("Unsupported Parameter\n");
		return TCL_ERROR;
	}

	printf("Option #%d: value = 0x%x\n", param_index, param_value);
	return TCL_OK;
}

/*
	set_nvm_cfg_57710 --

	Change the requested configuration setting in the NVRAM.
	This function was mainly added for windows firmware upgrade tool.

	Results:
		Will return TCL_ERROR in case of invalid device selection.
		Will return TCL_OK only if successfully changed the NVRAM setting.

	Side effects:
		None.
 */
int set_nvm_cfg_57710(NVM_INTERFACE iNvm, U32 *pCfgData) {

	FWUPG_CFG_DATA_T  *pCfg;
	nvm_cfg_t         cfg;
	nvm_cfg_t         shmem_cfg;
	Tcl_Interp        *interp = iNvm.m_tcl_interp;
	int               iRet = TCL_OK;
	u32_t               found_image = 0, nvm_addr = 0, byte_cnt = 0, entry_idx;
	nvm_nic_part_cfg_t mac_partition_cfg;
	u32_t nvm_cfg_extended_shared_block[NVM_CFG_EXTENDED_SHARED_IMAGE_MAX_SIZE];

	pCfg = (FWUPG_CFG_DATA_T *)pCfgData;

	common_nvm_load_cfg(&cfg, &shmem_cfg, &mac_partition_cfg, nvm_cfg_extended_shared_block);

	switch (pCfg->option_flag & 0xFFFFFFFF) {
	case CFG_MAC_OPTION:
		*((char *)&cfg.port_manuf.mac_upper + 0) = pCfg->mac_addr[1];
		*((char *)&cfg.port_manuf.mac_upper + 1) = pCfg->mac_addr[0];
		*((char *)&cfg.port_manuf.mac_lower + 0) = pCfg->mac_addr[5];
		*((char *)&cfg.port_manuf.mac_lower + 1) = pCfg->mac_addr[4];
		*((char *)&cfg.port_manuf.mac_lower + 2) = pCfg->mac_addr[3];
		*((char *)&cfg.port_manuf.mac_lower + 3) = pCfg->mac_addr[2];

		break;

	case CFG_BAKMAC_OPTION:
		*((char *)&cfg.port_manuf.backup_mac_upper + 0) = pCfg->mac_addr[1];
		*((char *)&cfg.port_manuf.backup_mac_upper + 1) = pCfg->mac_addr[0];
		*((char *)&cfg.port_manuf.backup_mac_lower + 0) = pCfg->mac_addr[5];
		*((char *)&cfg.port_manuf.backup_mac_lower + 1) = pCfg->mac_addr[4];
		*((char *)&cfg.port_manuf.backup_mac_lower + 2) = pCfg->mac_addr[3];
		*((char *)&cfg.port_manuf.backup_mac_lower + 3) = pCfg->mac_addr[2];

		break;

	case CFG_ISCSI_MAC_OPTION:
		*((char *)&cfg.port_manuf.iscsi_mac_upper + 0) = pCfg->mac_addr[1];
		*((char *)&cfg.port_manuf.iscsi_mac_upper + 1) = pCfg->mac_addr[0];
		*((char *)&cfg.port_manuf.iscsi_mac_lower + 0) = pCfg->mac_addr[5];
		*((char *)&cfg.port_manuf.iscsi_mac_lower + 1) = pCfg->mac_addr[4];
		*((char *)&cfg.port_manuf.iscsi_mac_lower + 2) = pCfg->mac_addr[3];
		*((char *)&cfg.port_manuf.iscsi_mac_lower + 3) = pCfg->mac_addr[2];

		break;

	case CFG_MBA_OPTION:
		if (pCfg->param_value) {
			cfg.port_feat.config |= PORT_FEATURE_MBA_ENABLED;
		} else {
			cfg.port_feat.config &= ~PORT_FEATURE_MBA_ENABLED;
		}

		break;

	case CFG_DCBX_OPTION:
		if (pCfg->param_value) {
			cfg.port_feat.config |= PORT_FEAT_CFG_DCBX_ENABLED;
		} else {
			cfg.port_feat.config &= ~PORT_FEAT_CFG_DCBX_ENABLED;
		}

		break;

	case CFG_IPMI_OPTION:
	case CFG_UMP_OPTION:
	case CFG_MGMT_OPTION:
		if (pCfg->param_value) {
			cfg.port_feat.config |= PORT_FEATURE_MFW_ENABLED;
		} else {
			cfg.port_feat.config &= ~PORT_FEATURE_MFW_ENABLED;
		}

		break;

	case CFG_GEN2_OPTION:
		if (pCfg->param_value) {
			cfg.shared_manuf.config2 |= SHARED_HW_CFG_PCIE_GEN2_ENABLED;
		} else {
			cfg.shared_manuf.config2 &= ~SHARED_HW_CFG_PCIE_GEN2_ENABLED;
		}
		break;

	case CFG_WOL_OPTION:
		if (pCfg->param_value) {
			cfg.port_feat.config |= PORT_FEATURE_WOL_ENABLED;
		} else {
			cfg.port_feat.config &= ~PORT_FEATURE_WOL_ENABLED;
		}

		break;

	case CFG_MFW_WOL_SPEED_OPTION:
		cfg.port_feat.mfw_wol_link_cfg &= ~PORT_FEATURE_LINK_SPEED_MASK; // clean up the old setting.
		cfg.port_feat.mfw_wol_link_cfg |= (pCfg->param_value << PORT_FEATURE_LINK_SPEED_SHIFT); // set to the new value

		break;

	case CFG_MBA_BOOT_PROTO_OPTION:
		cfg.port_feat.mba_config &= ~PORT_FEATURE_MBA_BOOT_AGENT_TYPE_MASK; // clean up the old setting.
		cfg.port_feat.mba_config |= (pCfg->param_value << PORT_FEATURE_MBA_BOOT_AGENT_TYPE_SHIFT); // set to the new value

		break;

	case CFG_OPT_MDL_ENFRCMNT_OPTION:
		cfg.port_feat.config &= ~PORT_FEAT_CFG_OPT_MDL_ENFRCMNT_MASK; // clean up the old setting.
		cfg.port_feat.config |= (pCfg->param_value << PORT_FEAT_CFG_OPT_MDL_ENFRCMNT_SHIFT); // set to the new value
		break;

	case CFG_SHADOW_SWIM_OPTION:
		// We have to make sure the CODE_IMAGE_NVM_CFG_EXTENDED_SHARED entry
		// is existing in CODE_IMAGE_TYPE_EXTENDED_DIR directory...

		/* search for image in extended dir */
		if ((common_nvm_nvm_find_image_in_extended_dir(&found_image,
													   CODE_IMAGE_NVM_CFG_EXTENDED_SHARED, &nvm_addr, &byte_cnt, &entry_idx) !=
				 COMMON_NVM_STATUS_SUCCESS) ||
			(!found_image)) {
			Tcl_AppendResult(interp, NVM_IMAGE_NAME_CFG_EXTENDED_SHARED,
							 " entry has to be in NVRAM in order to set the feature.\n", NULL);
			HANDLE_ERROR(iNvm, TRUE, FWUPG_DIR_NOT_FOUND);
			return TCL_ERROR; // Can't set the value...
		}

		cfg.extended_dev_info_shared.board_cfg &= ~EXTENDED_DEV_INFO_SHARED_CFG_SHADOW_SWIM_MASK; // clean up the old setting.
		cfg.extended_dev_info_shared.board_cfg |= (pCfg->param_value << EXTENDED_DEV_INFO_SHARED_CFG_SHADOW_SWIM_SHIFT); // set to the new value

		break;

	case CFG_PREVENT_FCOE_OPTION:
		if (pCfg->param_value) cfg.shared_feat.config |= SHARED_FEAT_CFG_PREVENT_FCOE;
		else cfg.shared_feat.config &= ~SHARED_FEAT_CFG_PREVENT_FCOE;

		break;

	case CFG_HIDDEN_OPTION:
		iRet = update_cfg_value_57710(&cfg, pCfg->param_index, pCfg->param_value);

		break;

	default:
		break;
	}

	if (iRet == TCL_OK) common_nvm_store_cfg(&cfg, &shmem_cfg, &mac_partition_cfg, &nvm_cfg_extended_shared_block[0]);
//    iNvm.m_tm_exec_cmd(iNvm.m_tcl_interp, "_reset", TM_CURR_DEVICE);

	return iRet;
}


/*
	show_nvm_cfg_57710 --

	Display the configuration setting in the NVRAM.
	This function was mainly added for firmware upgrade tool.

	Results:
		Will return TCL_ERROR in case of invalid device selection.
		Will return TCL_OK only if successfully changed the NVRAM setting.

	Side effects:
		None.
 */
int show_nvm_cfg_57710(NVM_INTERFACE iNvm, U32 *pCfgData) {
	nvm_cfg_t           cfg;
	nvm_cfg_t           shmem_cfg;
	u32_t               uRet;
	U32                 show_param;
	U8                  * pMU,*pML;
	U32                 option_idx;
	int                 iRet = TCL_OK;
	u32_t               vpd_offset, vpd_length;
	u32_t               byte_dumped;
	u8_t                *vpd_buf = NULL;
	nvm_nic_part_cfg_t  mac_partition_cfg;
	u32_t nvm_cfg_extended_shared[NVM_CFG_EXTENDED_SHARED_IMAGE_MAX_SIZE];

	uRet = common_nvm_load_cfg(&cfg, &shmem_cfg, &mac_partition_cfg, nvm_cfg_extended_shared);

	if (uRet != TCL_OK) {
		return uRet;
	}

	show_param = ((FWUPG_CFG_DATA_T *)pCfgData)->param_value;

	if (show_param != CFG_SHOW_PARAM_NONE) {
		// for "cfg -show <param>" command
		switch (show_param) {
		case CFG_SHOW_PARAM_BAKMAC:
			// Both cfg.port_manuf.backup_mac_upper and cfg.port_manuf.backup_mac_lower
			// are in Little Endian format.
			pMU = (U8 *)&cfg.port_manuf.backup_mac_upper;
			pML = (U8 *)&cfg.port_manuf.backup_mac_lower;
			printf("Backup MAC Address: %2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X\n",
				   *(pMU + 1), *(pMU), *(pML + 3), *(pML + 2), *(pML + 1), *(pML));
			break;
		case CFG_SHOW_PARAM_ISCSI_MAC:
			// Both cfg.port_manuf.iscsi_mac_upper and cfg.port_manuf.iscsi_mac_lower
			// are in Little Endian format.
			pMU = (U8 *)&cfg.port_manuf.iscsi_mac_upper;
			pML = (U8 *)&cfg.port_manuf.iscsi_mac_lower;
			printf("iSCSI MAC Address: %2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X\n",
				   *(pMU + 1), *(pMU), *(pML + 3), *(pML + 2), *(pML + 1), *(pML));
			break;
		case CFG_SHOW_PARAM_MBA_BOOT:
			printf("MBA Boot Protocol: %d\n", (cfg.port_feat.mba_config & PORT_FEATURE_MBA_BOOT_AGENT_TYPE_MASK));
			break;
		case CFG_SHOW_PARAM_MFW_WOL_SPD:
			printf("MFW WOL Speed: %d\n", ((cfg.port_feat.mfw_wol_link_cfg & PORT_FEATURE_LINK_SPEED_MASK) >> PORT_FEATURE_LINK_SPEED_SHIFT));
			break;
		case CFG_SHOW_PARAM_HIDDEN:
			option_idx = ((FWUPG_CFG_DATA_T *)pCfgData)->param_index;
			iRet = show_cfg_value_57710(&cfg, option_idx);
			break;
		case CFG_SHOW_PARAM_OPT_MDL_ENFRCMNT:
			printf("Optic Module Verdor Enforcement: %d\n", ((cfg.port_feat.config & PORT_FEAT_CFG_OPT_MDL_ENFRCMNT_MASK) >> PORT_FEAT_CFG_OPT_MDL_ENFRCMNT_SHIFT));
			break;
		case CFG_SHOW_PARAM_PREVENT_FCOE:
			printf("Prevent FCoE: %d\n",
				   ((cfg.shared_feat.config & SHARED_FEAT_CFG_PREVENT_FCOE) >> 12));
			break;
		default:
			printf("Invalid option specified.\n");
			return TCL_ERROR;
		}

		return iRet;
	}

	// for "cfg -show" command...

	if (util_hasPXE_57710(iNvm)) {
		if (cfg.port_feat.config & PORT_FEATURE_MBA_ENABLED) {
			printf("mba/pxe: 1\n");
		} else {
			printf("mba/pxe: 0\n");
		}
	} else {
		printf("mba/pxe firmware not found in the NVRAM\n");
	}

	if (cfg.port_feat.config & PORT_FEAT_CFG_DCBX_ENABLED) {
		printf("dcbx: 1\n");
	} else {
		printf("dcbx: 0\n");
	}

	if (util_hasIPMI_57710(iNvm) ||
		util_hasUMP_57710(iNvm) ||
		util_hasNCSI_57710(iNvm) ||
		util_hasMFW_57710(iNvm)) {
		if (cfg.port_feat.config & PORT_FEATURE_MFW_ENABLED) {
			printf("mgmt(mfw/ncsi/ump/ipmi): 1\n");
		} else {
			printf("mgmt(mfw/ncsi/ump/ipmi): 0\n");
		}
	} else {
		printf("management firmware not found in the NVRAM\n");
	}

	if (cfg.port_feat.config & PORT_FEATURE_WOL_ENABLED) {
		printf("wol: 1\n");
	} else {
		printf("wol: 0\n");
	}

	if (is_vpd_image_exist(iNvm, &vpd_offset, &vpd_length)) {
		vpd_buf = (u8_t *)Tcl_Alloc(vpd_length);

		if (vpd_buf == NULL) {
			printf("VPD (V0): failed to allocate memory \n");
			return TCL_ERROR;
		}

		if (common_nvm_dump_nvm_into_buf(vpd_buf, 0, vpd_length - 4, vpd_offset + 4, &byte_dumped) != COMMON_NVM_STATUS_SUCCESS) {
			printf("VPD (V0): failed to dump nvm \n");
			return TCL_ERROR;
		}

		dg_show_vpd_info(vpd_buf, vpd_length, "V0");
	} else if (cfg.vpd_info.v0[0]) {
		printf("VPD (V0): %s\n", cfg.vpd_info.v0);
	}

	// If CODE_IMAGE_NVM_CFG_EXTENDED_SHARED is not existing,
	// cfg.extended_dev_info_shared.board_cfg is set to 0 in
	// common_nvm_load_cfg()->common_nvm_read_nvm_cfg()->
	// common_nvm_nvm_fill_nvm_cfg_extended_shared()
	if (cfg.extended_dev_info_shared.board_cfg &
			EXTENDED_DEV_INFO_SHARED_CFG_SHADOW_SWIM_ENABLED) {
		printf("shadow_swim: 1\n");
	} else {
		printf("shadow_swim: 0\n");
	}

	return TCL_OK;
}

#endif // FWUPGTOOL


#ifdef WIN_DIAG
#undef Tcl_SetIntObjDispType
#endif //WIN_DIAG

