/****************************************************************************
 * Copyright(c) 2001-2014 Broadcom Corporation, all rights reserved
 * Copyright(c) 2014-2015 QLogic Corporation, all rights reserved
 * Proprietary and Confidential Information.
 *
 * This source file is the property of QLogic Corporation, and
 * may not be copied or distributed in any isomorphic form without
 * the prior written consent of QLogic Corporation.
 *
 * Name:        decode_grc.c
 *
 * Description: Decode GRC registers in 57712_reg.h.
 *
 * Created:     8/31/2011 dlau
 *     !!! Majority of the code is script generated. 
 *    Avoid modifying this file manually. Consult David Lau for help. 
 *
 ****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "tcl.h"
#include "grc_addr.h"
#include "bcmtype.h"

#ifndef LITTLE_ENDIAN
#define LITTLE_ENDIAN
#endif

#include "57712_reg.h"


/*****************************************************************************

FUNCTION:       DecodeGrc

DESCRIPTION:    This routine decode the GRCs defined in 57712_reg.h

IN PARAMETERS:  grc address and data

OUT PARAMETERS: none

RETURNS:        none

*****************************************************************************/
void DecodeGrc(Tcl_Obj* rc_ptr, u32_t addr, u32_t data)
{
	static char  disp_str[1500];
	
	switch (addr)
	{
		case 0x110000UL: // (ATC_REG_ATC_NUM_SETS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nATC_REG_ATC_NUM_SETS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Defines the number of sets - 3 - 256 ;2- 128; 1- 64; 0- 32\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110004UL: // (ATC_REG_ATC_1_WAY: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_1_WAY (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set the ATC will use only one way per set\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110008UL: // (ATC_REG_ATC_FULL_REG: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nATC_REG_ATC_FULL_REG (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: SPA Done FIFO full bit; RCPL FIFO full bit; TCPL FIFO full\n     bit; IREQ full bit; PLKP FIFO full bit; MLKP FIFO full bit; OTB full \n    bit; OIB full bit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11000cUL: // (ATC_REG_ATC_EMPTY_REG: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nATC_REG_ATC_EMPTY_REG (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: SPA Done FIFO empty bit; RCPL FIFO empty bit; TCPL FIFO empty\n     bit; IREQ empty bit; PLKP FIFO empty bit; MLKP FIFO empty bit; OTB empty\n     bit; OIB empty bit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110010UL: // (ATC_REG_ATC_WAIT_IF_MISS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_WAIT_IF_MISS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: WaitIfMiss configuration bit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110014UL: // (ATC_REG_ATC_WAIT_IF_PENDING: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_WAIT_IF_PENDING (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: WaitTransPending cofiguration bit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110018UL: // (ATC_REG_ATC_STALL_SEQ_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STALL_SEQ_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall (\n    on the second event). The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq\n    ;3-Tcpl;4-SPA-Done;5-Rcpl. The order of the bits inside the vector is \n    the same. Stall is taking place when there is a sequence of two events\n     X and than Y; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall\n     is not available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11001cUL: // (ATC_REG_ATC_STALL_SEQ_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STALL_SEQ_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall (\n    on the second event). The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq\n    ;3-Tcpl;4-SPA-Done;5-Rcpl. The order of the bits inside the vector is \n    the same. Stall is taking place when there is a sequence of two events\n     X and than Y; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall\n     is not available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110020UL: // (ATC_REG_ATC_STALL_SEQ_2: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STALL_SEQ_2 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall (\n    on the second event). The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq\n    ;3-Tcpl;4-SPA-Done;5-Rcpl. The order of the bits inside the vector is \n    the same. Stall is taking place when there is a sequence of two events\n     X and than Y; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall\n     is not available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110024UL: // (ATC_REG_ATC_STALL_SEQ_3: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STALL_SEQ_3 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall (\n    on the second event). The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq\n    ;3-Tcpl;4-SPA-Done;5-Rcpl. The order of the bits inside the vector is \n    the same. Stall is taking place when there is a sequence of two events\n     X and than Y; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall\n     is not available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110028UL: // (ATC_REG_ATC_STALL_SEQ_4: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STALL_SEQ_4 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall (\n    on the second event). The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq\n    ;3-Tcpl;4-SPA-Done;5-Rcpl. The order of the bits inside the vector is \n    the same. Stall is taking place when there is a sequence of two events\n     X and than Y; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall\n     is not available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11002cUL: // (ATC_REG_ATC_STALL_SEQ_5: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STALL_SEQ_5 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall (\n    on the second event). The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq\n    ;3-Tcpl;4-SPA-Done;5-Rcpl. The order of the bits inside the vector is \n    the same. Stall is taking place when there is a sequence of two events\n     X and than Y; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall\n     is not available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110030UL: // (ATC_REG_ATC_SET_STALL_SEQ_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_SET_STALL_SEQ_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall in\n     case of 2 consecutive accesses to the same set (on the second event). \n    The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq;3-Tcpl;4-SPA-Done;5-Rcpl\n    . The order of the bits inside the vector is the same. Stall is taking\n     place when there is a sequence of two events X and than Y to the same\n     set; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall is not\n     available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110034UL: // (ATC_REG_ATC_SET_STALL_SEQ_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_SET_STALL_SEQ_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall in\n     case of 2 consecutive accesses to the same set (on the second event). \n    The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq;3-Tcpl;4-SPA-Done;5-Rcpl\n    . The order of the bits inside the vector is the same. Stall is taking\n     place when there is a sequence of two events X and than Y to the same\n     set; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall is not\n     available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110038UL: // (ATC_REG_ATC_SET_STALL_SEQ_2: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_SET_STALL_SEQ_2 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall in\n     case of 2 consecutive accesses to the same set (on the second event). \n    The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq;3-Tcpl;4-SPA-Done;5-Rcpl\n    . The order of the bits inside the vector is the same. Stall is taking\n     place when there is a sequence of two events X and than Y to the same\n     set; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall is not\n     available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11003cUL: // (ATC_REG_ATC_SET_STALL_SEQ_3: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_SET_STALL_SEQ_3 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall in\n     case of 2 consecutive accesses to the same set (on the second event). \n    The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq;3-Tcpl;4-SPA-Done;5-Rcpl\n    . The order of the bits inside the vector is the same. Stall is taking\n     place when there is a sequence of two events X and than Y to the same\n     set; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall is not\n     available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110040UL: // (ATC_REG_ATC_SET_STALL_SEQ_4: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_SET_STALL_SEQ_4 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall in\n     case of 2 consecutive accesses to the same set (on the second event). \n    The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq;3-Tcpl;4-SPA-Done;5-Rcpl\n    . The order of the bits inside the vector is the same. Stall is taking\n     place when there is a sequence of two events X and than Y to the same\n     set; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall is not\n     available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110044UL: // (ATC_REG_ATC_SET_STALL_SEQ_5: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_SET_STALL_SEQ_5 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Indicates the B2B event sequnece which will cause stall in\n     case of 2 consecutive accesses to the same set (on the second event). \n    The order of the vectors is 0-Mlkp;1-Plkp;2-Ireq;3-Tcpl;4-SPA-Done;5-Rcpl\n    . The order of the bits inside the vector is the same. Stall is taking\n     place when there is a sequence of two events X and than Y to the same\n     set; and atc_stall_seq[X][Y] is set to 1. Comment for E2: stall is not\n     available for lookups due to the bounded latency restriction\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110048UL: // (ATC_REG_ATC_DISABLE_BYPASS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DISABLE_BYPASS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: disables the bypass on the GPA table\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11004cUL: // (ATC_REG_ATC_ISSUE_4_CYCLES: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ISSUE_4_CYCLES (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Issue event once in four cycles (instead of 2)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110054UL: // (ATC_REG_ATC_IREQ_ALMOST_FULL_THR: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nATC_REG_ATC_IREQ_ALMOST_FULL_THR (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: defines the IFIFO almost full threshold. Its \n    size can't be bigger than the Ireq FIFO size\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110058UL: // (ATC_REG_ATC_PIGGYBACKED_TREQ_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_PIGGYBACKED_TREQ_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Piggybacked treq issue enabled\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11005cUL: // (ATC_REG_ATC_WAIT_RESP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_WAIT_RESP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Allows the ATC to return Wait response\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110060UL: // (ATC_REG_ATC_TREQ_CREDITS: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_TREQ_CREDITS (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Number of credits for the treq interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110064UL: // (ATC_REG_ATC_ARBITER_PRIO_MLKP: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ARBITER_PRIO_MLKP (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: MLKP prio\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110068UL: // (ATC_REG_ATC_ARBITER_PRIO_PLKP: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ARBITER_PRIO_PLKP (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: PLKP prio\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11006cUL: // (ATC_REG_ATC_ARBITER_PRIO_IREQ: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ARBITER_PRIO_IREQ (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: IREQ prio\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110070UL: // (ATC_REG_ATC_ARBITER_PRIO_TCPL: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ARBITER_PRIO_TCPL (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TCPL prio\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110074UL: // (ATC_REG_ATC_ARBITER_PRIO_SPAD: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ARBITER_PRIO_SPAD (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: SPAD prio\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110078UL: // (ATC_REG_ATC_ARBITER_PRIO_RCPL: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ARBITER_PRIO_RCPL (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: RCPL prio\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11007cUL: // (ATC_REG_ATC_OTB_MAX_ENTRY: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nATC_REG_ATC_OTB_MAX_ENTRY (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Defines the number of entries in the OTB when 31 indicates\n     32 entries (entries count begins in 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110080UL: // (ATC_REG_ATC_CHECK_TAGS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_CHECK_TAGS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CheckTags configuration bit - when set the available NPH \n    credits is checked before issuing TREQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110084UL: // (ATC_REG_ATC_TAG_THR: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nATC_REG_ATC_TAG_THR (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TAG threshold - for the checkTags feature\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110088UL: // (ATC_REG_ATC_ICPL_CREDIT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ICPL_CREDIT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Credit value for the ICPL interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11008cUL: // (ATC_REG_ATC_DIS_MLKP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DIS_MLKP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables the main lookup interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110090UL: // (ATC_REG_ATC_DIS_PLKP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DIS_PLKP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables the pre lookup interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110094UL: // (ATC_REG_ATC_DIS_IREQ: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DIS_IREQ (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables the invalidation request interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110098UL: // (ATC_REG_ATC_DIS_TCPL: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DIS_TCPL (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables the translation completion interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11009cUL: // (ATC_REG_ATC_DIS_SPAD: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DIS_SPAD (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables the spa done interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100a0UL: // (ATC_REG_ATC_DIS_RCPL: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DIS_RCPL (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables the Read Completion interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100a4UL: // (ATC_REG_ATC_DIS_LKPRES: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DIS_LKPRES (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables the lookup response interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100a8UL: // (ATC_REG_ATC_DIS_TREQ: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DIS_TREQ (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables the translation request interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100acUL: // (ATC_REG_ATC_DIS_ICPL: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DIS_ICPL (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables the invalidation completion interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100b0UL: // (ATC_REG_ATC_SCRUB_CYC: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nATC_REG_ATC_SCRUB_CYC (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Number of cycles between one scrub event to another\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100b4UL: // (ATC_REG_ATC_SCRUB_DIS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_SCRUB_DIS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable bit for the scrubbing event of the GPA table\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100b8UL: // (ATC_REG_ATC_INIT_ARRAY: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_INIT_ARRAY (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Initiate the ATC array - reset all the valid bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100bcUL: // (ATC_REG_ATC_INIT_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_INIT_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: ATC initalization done\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100c0UL: // (ATC_REG_ATC_STAT_MLKP_HITS: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_MLKP_HITS (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of hits for Main-lookups in the ATC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100c4UL: // (ATC_REG_ATC_STAT_MLKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_MLKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of Main lookups in the ATC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100c8UL: // (ATC_REG_ATC_STAT_PLKP_TREQ: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_PLKP_TREQ (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of treqs issued due to pre-lookup\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100ccUL: // (ATC_REG_ATC_STAT_PLKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_PLKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of Pre Lookps in the ATC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100d0UL: // (ATC_REG_ATC_STAT_EVICT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_EVICT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of evictions out of the ATC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100d4UL: // (ATC_REG_ATC_STAT_INV_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_INV_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of invalidations handled by the ATC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100d8UL: // (ATC_REG_ATC_STAT_TREQ_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_TREQ_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of translation requests issued by the ATC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100dcUL: // (ATC_REG_ATC_STAT_ACTIVE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_ACTIVE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When this signal is set the statistics count is on\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100e0UL: // (ATC_REG_ATC_STAT_USDM_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_USDM_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100e4UL: // (ATC_REG_ATC_STAT_USDM_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_USDM_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100e8UL: // (ATC_REG_ATC_STAT_CSDM_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_CSDM_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100ecUL: // (ATC_REG_ATC_STAT_CSDM_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_CSDM_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100f0UL: // (ATC_REG_ATC_STAT_XSDM_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_XSDM_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100f4UL: // (ATC_REG_ATC_STAT_XSDM_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_XSDM_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100f8UL: // (ATC_REG_ATC_STAT_TSDM_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_TSDM_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1100fcUL: // (ATC_REG_ATC_STAT_TSDM_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_TSDM_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110100UL: // (ATC_REG_ATC_STAT_PBF_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_PBF_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110104UL: // (ATC_REG_ATC_STAT_PBF_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_PBF_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110108UL: // (ATC_REG_ATC_STAT_QM_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_QM_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11010cUL: // (ATC_REG_ATC_STAT_QM_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_QM_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110110UL: // (ATC_REG_ATC_STAT_TM_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_TM_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110114UL: // (ATC_REG_ATC_STAT_TM_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_TM_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110118UL: // (ATC_REG_ATC_STAT_SRC_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_SRC_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11011cUL: // (ATC_REG_ATC_STAT_SRC_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_SRC_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110120UL: // (ATC_REG_ATC_STAT_CDURD_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_CDURD_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110124UL: // (ATC_REG_ATC_STAT_CDURD_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_CDURD_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110128UL: // (ATC_REG_ATC_STAT_DMAE_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_DMAE_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11012cUL: // (ATC_REG_ATC_STAT_DMAE_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_DMAE_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110130UL: // (ATC_REG_ATC_STAT_USDM_DP_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_USDM_DP_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110134UL: // (ATC_REG_ATC_STAT_USDM_DP_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_USDM_DP_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110138UL: // (ATC_REG_ATC_STAT_HC_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_HC_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11013cUL: // (ATC_REG_ATC_STAT_HC_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_HC_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110140UL: // (ATC_REG_ATC_STAT_CDUWR_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_CDUWR_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110144UL: // (ATC_REG_ATC_STAT_CDUWR_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_CDUWR_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110148UL: // (ATC_REG_ATC_STAT_DBG_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_DBG_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11014cUL: // (ATC_REG_ATC_STAT_DBG_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_DBG_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110150UL: // (ATC_REG_ATC_STAT_L2P_LKP_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_L2P_LKP_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110154UL: // (ATC_REG_ATC_STAT_L2P_HIT_NUM: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_STAT_L2P_HIT_NUM (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Count lookups and hit for the different clients\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110158UL: // (ATC_REG_ATC_GPA_HASH_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_GPA_HASH_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: enable the use of a hash function for the GPA table; instead\n     of the lsb bits of the address\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11015cUL: // (ATC_REG_ATC_GPA_HASH_CRC: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_GPA_HASH_CRC (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: relevant only if hash_en is set. selects the CRC as hash \n    function for the GPA table; If reset use xor of the FID LS bits with the\n     relevant bits out of the GPA as hash function \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110160UL: // (ATC_REG_ATC_TCPL_LOG_ON_ERROR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nATC_REG_ATC_TCPL_LOG_ON_ERROR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: In case of TCPL with error log the relevant data. The seperation\n     for the different errors is: BME clear [0]; Unsupported request [1]; \n    Completer abort/completion timeout [2]; Both R & W bits are reset [3]; \n    Other [4]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110164UL: // (ATC_REG_ATC_TCPL_DIS_ON_ERROR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nATC_REG_ATC_TCPL_DIS_ON_ERROR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: In case of TCPL with error disable the ATC.  The seperation\n     for the different errors is: BME clear [0]; Unsupported request [1]; \n    Completer abort/completion timeout [2]; Both R & W bits are reset [3]; \n    Other [4]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110168UL: // (ATC_REG_ATC_TCPL_ERR_LOG: ACCESS:R  DataWidth:0x16 ) 
			sprintf(disp_str, "\nATC_REG_ATC_TCPL_ERR_LOG (ACCESS:R  DataWidth:0x16 )" \
				"\nDescription: Data belongs to an erroneous TCPL: [9:0] Func (VF_Valid;VFID\n    ;PFID);[10] U bit; [11] W bit; [12] R bit; [13] NS bit; [18:14] OTBEntryID\n    ;[21:19] Error code          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11016cUL: // (ATC_REG_ATC_TCPL_ERR_ADDR_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_TCPL_ERR_ADDR_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Data belongs to an erroneous TCPL: [31:0]-bits [31:0] of \n    the address\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110170UL: // (ATC_REG_ATC_TCPL_ERR_ADDR_MSB: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nATC_REG_ATC_TCPL_ERR_ADDR_MSB (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Data belongs to an erroneous TCPL: [19:0]-bits [51:32] of\n     the address\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110174UL: // (ATC_REG_ATC_TCPL_ERR_LOG_VALID: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_TCPL_ERR_LOG_VALID (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indicates valid data at the tcpl error log registers\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110178UL: // (ATC_REG_ATC_ARRAY_ACCESS_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ARRAY_ACCESS_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Allows GRC access to the GPA and SPA table\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11017cUL: // (ATC_REG_ATC_DURING_FLI: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DURING_FLI (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication that the ATC currently handles FLI\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110180UL: // (ATC_REG_ATC_DURING_INV: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DURING_INV (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication that the ATC currently handles Any type of invalidation\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110184UL: // (ATC_REG_ATC_FLI_DONE_VF_31_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_FLI_DONE_VF_31_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Indicates the end of FLI flow for VF 31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110188UL: // (ATC_REG_ATC_FLI_DONE_VF_63_32: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_FLI_DONE_VF_63_32 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Indicates the end of FLI flow for VF 63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11018cUL: // (ATC_REG_ATC_FLI_DONE_PF_7_0: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nATC_REG_ATC_FLI_DONE_PF_7_0 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Indicates the end of FLI flow for PF 7-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110190UL: // (ATC_REG_ATC_FLI_DONE_CLR_VF_31_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_FLI_DONE_CLR_VF_31_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Clears the FLI done indication for VF bits 31-0 accordingly\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110194UL: // (ATC_REG_ATC_FLI_DONE_CLR_VF_63_32: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_ATC_FLI_DONE_CLR_VF_63_32 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Clears the FLI done indication for VFbits 63-32 accordingly\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110198UL: // (ATC_REG_ATC_FLI_DONE_CLR_PF_7_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nATC_REG_ATC_FLI_DONE_CLR_PF_7_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Clears the FLI done indication for PF bits 7-0 accordingly\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11019cUL: // (ATC_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nATC_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PXP\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1101a0UL: // (ATC_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nATC_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PXP\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select and before the shift)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1101a4UL: // (ATC_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nATC_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PXP\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1101a8UL: // (ATC_REG_DBGSYN_ALMOST_FULL_THR: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nATC_REG_DBGSYN_ALMOST_FULL_THR (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Debug only: If more than this Number of entries are occupied\n     in the dbgsyn clock synchronization FIFO; it does not enable writing \n    to the fifo. This value is based on implementation and should not be changed\n    . Its value can't be bigger than the set dbg FIFO size\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1101acUL: // (ATC_REG_ATC_ALLOW_LOW_REP_HIGH: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ALLOW_LOW_REP_HIGH (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When set low priority lookup can replace high priority entry\n    ; iff the set is full with high prio entries\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1101b0UL: // (ATC_REG_ATC_DIS_IREQ_EVENT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_DIS_IREQ_EVENT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When set Ireq event won't be selected by the ATC arbiter\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1101b4UL: // (ATC_REG_ATC_ECO_RESERVED: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nATC_REG_ATC_ECO_RESERVED (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For future ECOs implementation\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1101b8UL: // (ATC_REG_ATC_TM: ACCESS:RW  DataWidth:0x1e  Multi Field Register  ) 
			sprintf(disp_str, "\nATC_REG_ATC_TM (ACCESS:RW  DataWidth:0x1e  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATC_GPA_STATE_TM :"
				, (u32_t)((data & ATC_ATC_TM_REG_ATC_GPA_STATE_TM) >> 25)
				, "\n\tATC_GPA_DATA_W3_TM :"
				, (u32_t)((data & ATC_ATC_TM_REG_ATC_GPA_DATA_W3_TM) >> 20)
				, "\n\tATC_GPA_DATA_W2_TM :"
				, (u32_t)((data & ATC_ATC_TM_REG_ATC_GPA_DATA_W2_TM) >> 15)
				, "\n\tATC_GPA_DATA_W1_TM :"
				, (u32_t)((data & ATC_ATC_TM_REG_ATC_GPA_DATA_W1_TM) >> 10)
				, "\n\tATC_GPA_DATA_W0_TM :"
				, (u32_t)((data & ATC_ATC_TM_REG_ATC_GPA_DATA_W0_TM) >> 5)
				, "\n\tATC_SPA_TABLE_TM :"
				, (u32_t)((data & ATC_ATC_TM_REG_ATC_SPA_TABLE_TM) >> 0));
			break;
		case 0x1101bcUL: // (ATC_REG_ATC_INT_STS: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_INT_STS (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATC_IREQ_LESS_THAN_STU :"
				, (u32_t)((data & ATC_ATC_INT_STS_REG_ATC_IREQ_LESS_THAN_STU) >> 5)
				, "\n\tATC_TCPL_ERROR :"
				, (u32_t)((data & ATC_ATC_INT_STS_REG_ATC_TCPL_ERROR) >> 4)
				, "\n\tATC_RCPL_TO_EMPTY_CNT :"
				, (u32_t)((data & ATC_ATC_INT_STS_REG_ATC_RCPL_TO_EMPTY_CNT) >> 3)
				, "\n\tATC_GPA_MULTIPLE_HITS :"
				, (u32_t)((data & ATC_ATC_INT_STS_REG_ATC_GPA_MULTIPLE_HITS) >> 2)
				, "\n\tATC_TCPL_TO_NOT_PEND :"
				, (u32_t)((data & ATC_ATC_INT_STS_REG_ATC_TCPL_TO_NOT_PEND) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & ATC_ATC_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1101c0UL: // (ATC_REG_ATC_INT_STS_CLR: ACCESS:RC  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_INT_STS_CLR (ACCESS:RC  DataWidth:0x6 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATC_IREQ_LESS_THAN_STU :"
				, (u32_t)((data & ATC_ATC_INT_STS_CLR_REG_ATC_IREQ_LESS_THAN_STU) >> 5)
				, "\n\tATC_TCPL_ERROR :"
				, (u32_t)((data & ATC_ATC_INT_STS_CLR_REG_ATC_TCPL_ERROR) >> 4)
				, "\n\tATC_RCPL_TO_EMPTY_CNT :"
				, (u32_t)((data & ATC_ATC_INT_STS_CLR_REG_ATC_RCPL_TO_EMPTY_CNT) >> 3)
				, "\n\tATC_GPA_MULTIPLE_HITS :"
				, (u32_t)((data & ATC_ATC_INT_STS_CLR_REG_ATC_GPA_MULTIPLE_HITS) >> 2)
				, "\n\tATC_TCPL_TO_NOT_PEND :"
				, (u32_t)((data & ATC_ATC_INT_STS_CLR_REG_ATC_TCPL_TO_NOT_PEND) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & ATC_ATC_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1101c4UL: // (ATC_REG_ATC_INT_STS_WR: ACCESS:WR  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_INT_STS_WR (ACCESS:WR  DataWidth:0x6 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATC_IREQ_LESS_THAN_STU :"
				, (u32_t)((data & ATC_ATC_INT_STS_WR_REG_ATC_IREQ_LESS_THAN_STU) >> 5)
				, "\n\tATC_TCPL_ERROR :"
				, (u32_t)((data & ATC_ATC_INT_STS_WR_REG_ATC_TCPL_ERROR) >> 4)
				, "\n\tATC_RCPL_TO_EMPTY_CNT :"
				, (u32_t)((data & ATC_ATC_INT_STS_WR_REG_ATC_RCPL_TO_EMPTY_CNT) >> 3)
				, "\n\tATC_GPA_MULTIPLE_HITS :"
				, (u32_t)((data & ATC_ATC_INT_STS_WR_REG_ATC_GPA_MULTIPLE_HITS) >> 2)
				, "\n\tATC_TCPL_TO_NOT_PEND :"
				, (u32_t)((data & ATC_ATC_INT_STS_WR_REG_ATC_TCPL_TO_NOT_PEND) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & ATC_ATC_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1101c8UL: // (ATC_REG_ATC_INT_MASK: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nATC_REG_ATC_INT_MASK (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATC_IREQ_LESS_THAN_STU :"
				, (u32_t)((data & ATC_ATC_INT_MASK_REG_ATC_IREQ_LESS_THAN_STU) >> 5)
				, "\n\tATC_TCPL_ERROR :"
				, (u32_t)((data & ATC_ATC_INT_MASK_REG_ATC_TCPL_ERROR) >> 4)
				, "\n\tATC_RCPL_TO_EMPTY_CNT :"
				, (u32_t)((data & ATC_ATC_INT_MASK_REG_ATC_RCPL_TO_EMPTY_CNT) >> 3)
				, "\n\tATC_GPA_MULTIPLE_HITS :"
				, (u32_t)((data & ATC_ATC_INT_MASK_REG_ATC_GPA_MULTIPLE_HITS) >> 2)
				, "\n\tATC_TCPL_TO_NOT_PEND :"
				, (u32_t)((data & ATC_ATC_INT_MASK_REG_ATC_TCPL_TO_NOT_PEND) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & ATC_ATC_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1101ccUL: // (ATC_REG_ATC_PRTY_STS: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nATC_REG_ATC_PRTY_STS (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTCPL_FIFO :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_REG_TCPL_FIFO) >> 4)
				, "\n\tSPA_TABLE :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_REG_SPA_TABLE) >> 3)
				, "\n\tIREQ_FIFO :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_REG_IREQ_FIFO) >> 2)
				, "\n\tGPA_TABLE :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_REG_GPA_TABLE) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x1101d0UL: // (ATC_REG_ATC_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x5 ) 
			sprintf(disp_str, "\nATC_REG_ATC_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x5 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTCPL_FIFO :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_CLR_REG_TCPL_FIFO) >> 4)
				, "\n\tSPA_TABLE :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_CLR_REG_SPA_TABLE) >> 3)
				, "\n\tIREQ_FIFO :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_CLR_REG_IREQ_FIFO) >> 2)
				, "\n\tGPA_TABLE :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_CLR_REG_GPA_TABLE) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x1101d4UL: // (ATC_REG_ATC_PRTY_STS_WR: ACCESS:WR  DataWidth:0x5 ) 
			sprintf(disp_str, "\nATC_REG_ATC_PRTY_STS_WR (ACCESS:WR  DataWidth:0x5 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTCPL_FIFO :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_WR_REG_TCPL_FIFO) >> 4)
				, "\n\tSPA_TABLE :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_WR_REG_SPA_TABLE) >> 3)
				, "\n\tIREQ_FIFO :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_WR_REG_IREQ_FIFO) >> 2)
				, "\n\tGPA_TABLE :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_WR_REG_GPA_TABLE) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & ATC_ATC_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x1101d8UL: // (ATC_REG_ATC_PRTY_MASK: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nATC_REG_ATC_PRTY_MASK (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTCPL_FIFO :"
				, (u32_t)((data & ATC_ATC_PRTY_MASK_REG_TCPL_FIFO) >> 4)
				, "\n\tSPA_TABLE :"
				, (u32_t)((data & ATC_ATC_PRTY_MASK_REG_SPA_TABLE) >> 3)
				, "\n\tIREQ_FIFO :"
				, (u32_t)((data & ATC_ATC_PRTY_MASK_REG_IREQ_FIFO) >> 2)
				, "\n\tGPA_TABLE :"
				, (u32_t)((data & ATC_ATC_PRTY_MASK_REG_GPA_TABLE) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & ATC_ATC_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x1101dcUL: // (ATC_REG_ATC_IREQ_FIFO_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nATC_REG_ATC_IREQ_FIFO_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits of GPA state array\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110200UL: // (ATC_REG_DBG_OUT_DATA_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_DBG_OUT_DATA_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 32 lsb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110204UL: // (ATC_REG_DBG_OUT_DATA_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nATC_REG_DBG_OUT_DATA_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 32 msb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110208UL: // (ATC_REG_DBG_OUT_FRAME: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nATC_REG_DBG_OUT_FRAME (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 4 frame bits that goes to the DBG block. Bit0\n     is the frame of data byte0; Bit1 is the frame of data byte1; Bit2 is \n    the frame of data byte2; Bit3 is the frame of data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11020cUL: // (ATC_REG_DBG_OUT_VALID: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nATC_REG_DBG_OUT_VALID (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 4 valid bits that goes to the DBG block. Bit0\n     validtes data byte0; Bit1 validates data byte1; Bit2 validates data byte2\n    ; Bit3 validates data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110800UL: // (ATC_REG_ATC_GPA_ARRAY_ACCESS_STATE: ACCESS:WB  DataWidth:0x34 ) 
			sprintf(disp_str, "\nATC_REG_ATC_GPA_ARRAY_ACCESS_STATE (ACCESS:WB  DataWidth:0x34 )" \
				"\nDescription: Access the state fields of the GPA table; format is: W3 - {\n    par - [51]; NS bit - [50]; W bit - [49]; R bit - [48]; U bit - [47]; Priority\n     bit - [46]; PLRU - [45];  R-counter - [44:42]; transpend bit - [41]; \n    invpend bit [40]; valid bit[39]}; W2 - {par - [38]; NS bit - [37]; W bit\n     - [36]; R bit - [35]; U bit - [34]; Priority bit - [33]; PLRU - [32];  \n    R-counter - [31:29]; transpend bit - [28]; invpend bit [27]; valid bit\n    [26]}; W1 -  {par - [25]; NS bit - [24]; W bit - [23]; R bit - [22]; U\n     bit - [21]; Priority bit - [20]; PLRU - [19];  R-counter - [18:16]; transpend\n     bit - [15]; invpend bit [14]; valid bit[13]}; W0 -  {par - [12]; NS bit\n     - [11]; W bit - [10]; R bit - [9]; U bit - [8]; Priority bit - [7]; PLRU\n     - [6];  R-counter - [5:3]; transpend bit - [2]; invpend bit [1]; valid\n     bit[0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x111000UL: // (ATC_REG_ATC_GPA_ARRAY_ACCESS_W0: ACCESS:WB  DataWidth:0x3e ) 
			sprintf(disp_str, "\nATC_REG_ATC_GPA_ARRAY_ACCESS_W0 (ACCESS:WB  DataWidth:0x3e )" \
				"\nDescription: Access the GPA table way 0; format is: GPA - [51:0]; VF_Valid\n    -[52]; VFID-[58:53]; PFID-[61:59];Parity[62]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x111800UL: // (ATC_REG_ATC_GPA_ARRAY_ACCESS_W1: ACCESS:WB  DataWidth:0x3e ) 
			sprintf(disp_str, "\nATC_REG_ATC_GPA_ARRAY_ACCESS_W1 (ACCESS:WB  DataWidth:0x3e )" \
				"\nDescription: Access the GPA table way 1; format is: GPA - [51:0]; VF_Valid\n    -[52]; VFID-[58:53]; PFID-[61:59];Parity[62]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x112000UL: // (ATC_REG_ATC_GPA_ARRAY_ACCESS_W2: ACCESS:WB  DataWidth:0x3e ) 
			sprintf(disp_str, "\nATC_REG_ATC_GPA_ARRAY_ACCESS_W2 (ACCESS:WB  DataWidth:0x3e )" \
				"\nDescription: Access the GPA table way 2; format is: GPA - [51:0]; VF_Valid\n    -[52]; VFID-[58:53]; PFID-[61:59];Parity[62]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x112800UL: // (ATC_REG_ATC_GPA_ARRAY_ACCESS_W3: ACCESS:WB  DataWidth:0x3e ) 
			sprintf(disp_str, "\nATC_REG_ATC_GPA_ARRAY_ACCESS_W3 (ACCESS:WB  DataWidth:0x3e )" \
				"\nDescription: Access the GPA table way 3; format is: GPA - [51:0]; VF_Valid\n    -[52]; VFID-[58:53]; PFID-[61:59];Parity[62]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x114000UL: // (ATC_REG_ATC_SPA_ARRAY_ACCESS: ACCESS:WB  DataWidth:0x34 ) 
			sprintf(disp_str, "\nATC_REG_ATC_SPA_ARRAY_ACCESS (ACCESS:WB  DataWidth:0x34 )" \
				"\nDescription: Debug access to the SPA array\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1101e0UL: // (ATC_REG_ATC_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nATC_REG_ATC_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x110210UL: // (ATC_REG_ATC_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nATC_REG_ATC_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x113000UL: // (ATC_REG_ATC_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nATC_REG_ATC_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x116000UL: // (ATC_REG_ATC_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nATC_REG_ATC_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60004UL: // (BRB1_REG_READ_FREE_BLOCK_ERROR_0: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_FREE_BLOCK_ERROR_0 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 0: attempt to read from free block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60008UL: // (BRB1_REG_READ_FREE_BLOCK_ERROR_10: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_FREE_BLOCK_ERROR_10 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 10: attempt to read from free block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6000cUL: // (BRB1_REG_READ_FREE_BLOCK_ERROR_11: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_FREE_BLOCK_ERROR_11 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 11: attempt to read from free block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60010UL: // (BRB1_REG_READ_FREE_BLOCK_ERROR_12: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_FREE_BLOCK_ERROR_12 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 12: attempt to read from free block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60014UL: // (BRB1_REG_READ_FREE_BLOCK_ERROR_13: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_FREE_BLOCK_ERROR_13 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 13: attempt to read from free block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60018UL: // (BRB1_REG_READ_FREE_BLOCK_ERROR_14: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_FREE_BLOCK_ERROR_14 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 14: attempt to read from free block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6001cUL: // (BRB1_REG_READ_FREE_BLOCK_ERROR_2: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_FREE_BLOCK_ERROR_2 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 2: attempt to read from free block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60020UL: // (BRB1_REG_READ_FREE_BLOCK_ERROR_3: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_FREE_BLOCK_ERROR_3 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 3: attempt to read from free block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60024UL: // (BRB1_REG_READ_FREE_BLOCK_ERROR_4: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_FREE_BLOCK_ERROR_4 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 4: attempt to read from free block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60028UL: // (BRB1_REG_READ_LENGTH_ERROR_0: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_LENGTH_ERROR_0 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 0: attempt to read more data than in packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6002cUL: // (BRB1_REG_READ_LENGTH_ERROR_10: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_LENGTH_ERROR_10 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 10: attempt to read more data than in packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60030UL: // (BRB1_REG_READ_LENGTH_ERROR_11: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_LENGTH_ERROR_11 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 11: attempt to read more data than in packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60034UL: // (BRB1_REG_READ_LENGTH_ERROR_12: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_LENGTH_ERROR_12 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 12: attempt to read more data than in packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60038UL: // (BRB1_REG_READ_LENGTH_ERROR_13: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_LENGTH_ERROR_13 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 13: attempt to read more data than in packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6003cUL: // (BRB1_REG_READ_LENGTH_ERROR_14: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_LENGTH_ERROR_14 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 14: attempt to read more data than in packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60040UL: // (BRB1_REG_READ_LENGTH_ERROR_2: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_LENGTH_ERROR_2 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 2: attempt to read more data than in packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60044UL: // (BRB1_REG_READ_LENGTH_ERROR_3: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_LENGTH_ERROR_3 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 3: attempt to read more data than in packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60048UL: // (BRB1_REG_READ_LENGTH_ERROR_4: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_LENGTH_ERROR_4 (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Read client 4: attempt to read more data than in packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6004cUL: // (BRB1_REG_STOP_INTERFACE_ON_READ_FREE_BLOCK_ERROR: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nBRB1_REG_STOP_INTERFACE_ON_READ_FREE_BLOCK_ERROR (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: stop interface on read from free block. Bit 0 (LSB) for client\n     0. Bit 1 for client 1 logical 0. Bit 2 for client 1 logical 1. Bit 3 \n    for client 1 logical 2. Bit 4 for client 1 logical 3. Bit 5 for client\n     1 logical 4. Bit 6 for client 2. Bit 7 for client 3. Bit 8 for client\n     4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60050UL: // (BRB1_REG_STOP_INTERFACE_ON_READ_LENGTH_ERROR: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nBRB1_REG_STOP_INTERFACE_ON_READ_LENGTH_ERROR (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: stop interface on read more data than in packet. Bit 0 (LSB\n    ) for client 0. Bit 1 for client 1 logical 0. Bit 2 for client 1 logical\n     1. Bit 3 for client 1 logical 2. Bit 4 for client 1 logical 3. Bit 5 \n    for client 1 logical 4. Bit 6 for client 2. Bit 7 for client 3. Bit 8 \n    for client 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60054UL: // (BRB1_REG_READ_PRIORITY_0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_PRIORITY_0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Read client 0: Read priority \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60058UL: // (BRB1_REG_READ_PRIORITY_1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_PRIORITY_1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Read client 1: Read priority \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6005cUL: // (BRB1_REG_READ_PRIORITY_2: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_PRIORITY_2 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Read client 2: Read priority \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60060UL: // (BRB1_REG_READ_PRIORITY_3: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_PRIORITY_3 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Read client 3: Read priority \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60064UL: // (BRB1_REG_READ_PRIORITY_4: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nBRB1_REG_READ_PRIORITY_4 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Read client 4: Read priority\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60068UL: // (BRB1_REG_PAUSE_LOW_THRESHOLD_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_LOW_THRESHOLD_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Write client 0: Assert pause threshold. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6006cUL: // (BRB1_REG_PAUSE_LOW_THRESHOLD_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_LOW_THRESHOLD_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Write client 0: Assert pause threshold. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60070UL: // (BRB1_REG_PAUSE_LOW_THRESHOLD_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_LOW_THRESHOLD_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Write client 0: Assert pause threshold. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60074UL: // (BRB1_REG_PAUSE_LOW_THRESHOLD_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_LOW_THRESHOLD_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Write client 0: Assert pause threshold. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60078UL: // (BRB1_REG_PAUSE_HIGH_THRESHOLD_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_HIGH_THRESHOLD_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Write client 0: De-assert pause threshold. Not Functional\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6007cUL: // (BRB1_REG_PAUSE_HIGH_THRESHOLD_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_HIGH_THRESHOLD_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Write client 0: De-assert pause threshold. Not Functional\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60080UL: // (BRB1_REG_PAUSE_HIGH_THRESHOLD_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_HIGH_THRESHOLD_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Write client 0: De-assert pause threshold. Not Functional\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60084UL: // (BRB1_REG_PAUSE_HIGH_THRESHOLD_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_HIGH_THRESHOLD_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Write client 0: De-assert pause threshold. Not Functional\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60088UL: // (BRB1_REG_BIG_RAM_INDIRECT: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nBRB1_REG_BIG_RAM_INDIRECT (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Indirect memory address\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6008cUL: // (BRB1_REG_NUM_OF_FREE_BLOCKS: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FREE_BLOCKS (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of free 256-byte blocks.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60090UL: // (BRB1_REG_NUM_OF_FULL_BLOCKS: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_BLOCKS (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of full blocks.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60094UL: // (BRB1_REG_PORT_NUM_OCC_BLOCKS_0: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_NUM_OCC_BLOCKS_0 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of full blocks occpied by port.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60098UL: // (BRB1_REG_PORT_NUM_OCC_BLOCKS_1: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_NUM_OCC_BLOCKS_1 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of full blocks occpied by port.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6009cUL: // (BRB1_REG_PORT_NUM_OCC_BLOCKS_2: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_NUM_OCC_BLOCKS_2 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of full blocks occpied by port.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600a0UL: // (BRB1_REG_PORT_NUM_OCC_BLOCKS_3: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_NUM_OCC_BLOCKS_3 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of full blocks occpied by port.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600a4UL: // (BRB1_REG_PORT_NUM_OCC_BLOCKS_4: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_NUM_OCC_BLOCKS_4 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of full blocks occpied by port.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600a8UL: // (BRB1_REG_FULL_WRC_RC: ACCESS:R  DataWidth:0xf ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_WRC_RC (ACCESS:R  DataWidth:0xf )" \
				"\nDescription: Full on read clients USDM_DP USDM_CTRL TSDM WRC4 WRC1 WRC0\n    . See remark 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600acUL: // (BRB1_REG_PRS_CRDT: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PRS_CRDT (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Parser credit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600b0UL: // (BRB1_REG_PEND_REQ: ACCESS:R  DataWidth:0x12 ) 
			sprintf(disp_str, "\nBRB1_REG_PEND_REQ (ACCESS:R  DataWidth:0x12 )" \
				"\nDescription: Pending requests. 2 bits per each client. Order is from msb\n     to lsb USDM_DP; USDM_CTRL; TSDM_CTRL; CAC; PRS.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600b4UL: // (BRB1_REG_MAXIMUM_NUMBER_OF_FULL_BLOCKS: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAXIMUM_NUMBER_OF_FULL_BLOCKS (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: The maximum number of occupied block in the time interval\n     between two RBC reads. This field is cleared-on-read.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600b8UL: // (BRB1_REG_NUM_OF_PAUSE_CYCLES_0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_PAUSE_CYCLES_0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the pause signal towards MAC #0\n     was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600bcUL: // (BRB1_REG_NUM_OF_PAUSE_CYCLES_1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_PAUSE_CYCLES_1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the pause signal towards MAC #0\n     was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600c0UL: // (BRB1_REG_NUM_OF_PAUSE_CYCLES_2: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_PAUSE_CYCLES_2 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the pause signal towards MAC #0\n     was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600c4UL: // (BRB1_REG_NUM_OF_PAUSE_CYCLES_3: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_PAUSE_CYCLES_3 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the pause signal towards MAC #0\n     was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600c8UL: // (BRB1_REG_NUM_OF_FULL_CYCLES_0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_CYCLES_0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the write_full signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600ccUL: // (BRB1_REG_NUM_OF_FULL_CYCLES_1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_CYCLES_1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the write_full signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600d0UL: // (BRB1_REG_NUM_OF_FULL_CYCLES_2: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_CYCLES_2 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the write_full signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600d4UL: // (BRB1_REG_NUM_OF_FULL_CYCLES_3: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_CYCLES_3 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the write_full signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600d8UL: // (BRB1_REG_NUM_OF_FULL_CYCLES_4: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_CYCLES_4 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the write_full signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600dcUL: // (BRB1_REG_SOFT_RESET: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_SOFT_RESET (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Reset the design by software.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600e0UL: // (BRB1_REG_PORT_0_GUARANTIED: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_0_GUARANTIED (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of blocks guarantied for port 0. Not Functional\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600e4UL: // (BRB1_REG_PORT_1_GUARANTIED: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_1_GUARANTIED (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of blocks guarantied for port 1. Not Functional\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600e8UL: // (BRB1_REG_PORT_2_GUARANTIED: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_2_GUARANTIED (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of blocks guarantied for port 2. Not Functional\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600ecUL: // (BRB1_REG_PORT_3_GUARANTIED: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_3_GUARANTIED (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of blocks guarantied for port 3. Not Functional\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600f0UL: // (BRB1_REG_PORT_4_GUARANTIED: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_4_GUARANTIED (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of blocks guarantied for port 4. Not Functional\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600f8UL: // (BRB1_REG_DELTA: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_DELTA (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The histeresis value. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x600fcUL: // (BRB1_REG_INPUT_BUFFER_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nBRB1_REG_INPUT_BUFFER_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Number of free entries in the buffer of each write port.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60100UL: // (BRB1_REG_M_SP_RAM_BL_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nBRB1_REG_M_SP_RAM_BL_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of BL RAM memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60104UL: // (BRB1_REG_M_SP_RAM_BR_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nBRB1_REG_M_SP_RAM_BR_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of BR RAM memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60108UL: // (BRB1_REG_LRAM_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nBRB1_REG_LRAM_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TMA port register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6010cUL: // (BRB1_REG_LRAM_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nBRB1_REG_LRAM_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TMB port register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60110UL: // (BRB1_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nBRB1_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from BRB1\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60114UL: // (BRB1_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nBRB1_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from BRB1\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60118UL: // (BRB1_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nBRB1_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from BRB1\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6011cUL: // (BRB1_REG_BRB1_INT_STS: ACCESS:R  DataWidth:0x13 ) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_INT_STS (ACCESS:R  DataWidth:0x13 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_LEN_ERR_4 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_LEN_ERR_4) >> 18)
				, "\n\tRD_LEN_ERR_3 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_LEN_ERR_3) >> 17)
				, "\n\tRD_LEN_ERR_2 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_LEN_ERR_2) >> 16)
				, "\n\tRD_LEN_ERR_14 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_LEN_ERR_14) >> 15)
				, "\n\tRD_LEN_ERR_13 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_LEN_ERR_13) >> 14)
				, "\n\tRD_LEN_ERR_12 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_LEN_ERR_12) >> 13)
				, "\n\tRD_LEN_ERR_11 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_LEN_ERR_11) >> 12)
				, "\n\tRD_LEN_ERR_10 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_LEN_ERR_10) >> 11)
				, "\n\tRD_LEN_ERR_0 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_LEN_ERR_0) >> 10)
				, "\n\tRD_FREE_BLK_ERR_4 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_FREE_BLK_ERR_4) >> 9)
				, "\n\tRD_FREE_BLK_ERR_3 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_FREE_BLK_ERR_3) >> 8)
				, "\n\tRD_FREE_BLK_ERR_2 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_FREE_BLK_ERR_2) >> 7)
				, "\n\tRD_FREE_BLK_ERR_14 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_FREE_BLK_ERR_14) >> 6)
				, "\n\tRD_FREE_BLK_ERR_13 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_FREE_BLK_ERR_13) >> 5)
				, "\n\tRD_FREE_BLK_ERR_12 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_FREE_BLK_ERR_12) >> 4)
				, "\n\tRD_FREE_BLK_ERR_11 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_FREE_BLK_ERR_11) >> 3)
				, "\n\tRD_FREE_BLK_ERR_10 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_FREE_BLK_ERR_10) >> 2)
				, "\n\tRD_FREE_BLK_ERR_0 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_RD_FREE_BLK_ERR_0) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x60120UL: // (BRB1_REG_BRB1_INT_STS_CLR: ACCESS:RC  DataWidth:0x13 ) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_INT_STS_CLR (ACCESS:RC  DataWidth:0x13 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_LEN_ERR_4 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_LEN_ERR_4) >> 18)
				, "\n\tRD_LEN_ERR_3 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_LEN_ERR_3) >> 17)
				, "\n\tRD_LEN_ERR_2 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_LEN_ERR_2) >> 16)
				, "\n\tRD_LEN_ERR_14 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_LEN_ERR_14) >> 15)
				, "\n\tRD_LEN_ERR_13 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_LEN_ERR_13) >> 14)
				, "\n\tRD_LEN_ERR_12 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_LEN_ERR_12) >> 13)
				, "\n\tRD_LEN_ERR_11 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_LEN_ERR_11) >> 12)
				, "\n\tRD_LEN_ERR_10 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_LEN_ERR_10) >> 11)
				, "\n\tRD_LEN_ERR_0 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_LEN_ERR_0) >> 10)
				, "\n\tRD_FREE_BLK_ERR_4 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_FREE_BLK_ERR_4) >> 9)
				, "\n\tRD_FREE_BLK_ERR_3 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_FREE_BLK_ERR_3) >> 8)
				, "\n\tRD_FREE_BLK_ERR_2 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_FREE_BLK_ERR_2) >> 7)
				, "\n\tRD_FREE_BLK_ERR_14 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_FREE_BLK_ERR_14) >> 6)
				, "\n\tRD_FREE_BLK_ERR_13 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_FREE_BLK_ERR_13) >> 5)
				, "\n\tRD_FREE_BLK_ERR_12 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_FREE_BLK_ERR_12) >> 4)
				, "\n\tRD_FREE_BLK_ERR_11 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_FREE_BLK_ERR_11) >> 3)
				, "\n\tRD_FREE_BLK_ERR_10 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_FREE_BLK_ERR_10) >> 2)
				, "\n\tRD_FREE_BLK_ERR_0 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_RD_FREE_BLK_ERR_0) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x60124UL: // (BRB1_REG_BRB1_INT_STS_WR: ACCESS:WR  DataWidth:0x13 ) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_INT_STS_WR (ACCESS:WR  DataWidth:0x13 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_LEN_ERR_4 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_LEN_ERR_4) >> 18)
				, "\n\tRD_LEN_ERR_3 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_LEN_ERR_3) >> 17)
				, "\n\tRD_LEN_ERR_2 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_LEN_ERR_2) >> 16)
				, "\n\tRD_LEN_ERR_14 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_LEN_ERR_14) >> 15)
				, "\n\tRD_LEN_ERR_13 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_LEN_ERR_13) >> 14)
				, "\n\tRD_LEN_ERR_12 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_LEN_ERR_12) >> 13)
				, "\n\tRD_LEN_ERR_11 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_LEN_ERR_11) >> 12)
				, "\n\tRD_LEN_ERR_10 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_LEN_ERR_10) >> 11)
				, "\n\tRD_LEN_ERR_0 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_LEN_ERR_0) >> 10)
				, "\n\tRD_FREE_BLK_ERR_4 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_FREE_BLK_ERR_4) >> 9)
				, "\n\tRD_FREE_BLK_ERR_3 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_FREE_BLK_ERR_3) >> 8)
				, "\n\tRD_FREE_BLK_ERR_2 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_FREE_BLK_ERR_2) >> 7)
				, "\n\tRD_FREE_BLK_ERR_14 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_FREE_BLK_ERR_14) >> 6)
				, "\n\tRD_FREE_BLK_ERR_13 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_FREE_BLK_ERR_13) >> 5)
				, "\n\tRD_FREE_BLK_ERR_12 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_FREE_BLK_ERR_12) >> 4)
				, "\n\tRD_FREE_BLK_ERR_11 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_FREE_BLK_ERR_11) >> 3)
				, "\n\tRD_FREE_BLK_ERR_10 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_FREE_BLK_ERR_10) >> 2)
				, "\n\tRD_FREE_BLK_ERR_0 :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_RD_FREE_BLK_ERR_0) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & BRB1_BRB1_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x60128UL: // (BRB1_REG_BRB1_INT_MASK: ACCESS:RW  DataWidth:0x13 ) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_INT_MASK (ACCESS:RW  DataWidth:0x13 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_LEN_ERR_4 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_LEN_ERR_4) >> 18)
				, "\n\tRD_LEN_ERR_3 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_LEN_ERR_3) >> 17)
				, "\n\tRD_LEN_ERR_2 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_LEN_ERR_2) >> 16)
				, "\n\tRD_LEN_ERR_14 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_LEN_ERR_14) >> 15)
				, "\n\tRD_LEN_ERR_13 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_LEN_ERR_13) >> 14)
				, "\n\tRD_LEN_ERR_12 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_LEN_ERR_12) >> 13)
				, "\n\tRD_LEN_ERR_11 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_LEN_ERR_11) >> 12)
				, "\n\tRD_LEN_ERR_10 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_LEN_ERR_10) >> 11)
				, "\n\tRD_LEN_ERR_0 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_LEN_ERR_0) >> 10)
				, "\n\tRD_FREE_BLK_ERR_4 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_FREE_BLK_ERR_4) >> 9)
				, "\n\tRD_FREE_BLK_ERR_3 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_FREE_BLK_ERR_3) >> 8)
				, "\n\tRD_FREE_BLK_ERR_2 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_FREE_BLK_ERR_2) >> 7)
				, "\n\tRD_FREE_BLK_ERR_14 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_FREE_BLK_ERR_14) >> 6)
				, "\n\tRD_FREE_BLK_ERR_13 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_FREE_BLK_ERR_13) >> 5)
				, "\n\tRD_FREE_BLK_ERR_12 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_FREE_BLK_ERR_12) >> 4)
				, "\n\tRD_FREE_BLK_ERR_11 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_FREE_BLK_ERR_11) >> 3)
				, "\n\tRD_FREE_BLK_ERR_10 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_FREE_BLK_ERR_10) >> 2)
				, "\n\tRD_FREE_BLK_ERR_0 :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_RD_FREE_BLK_ERR_0) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & BRB1_BRB1_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x6012cUL: // (BRB1_REG_BRB1_PRTY_STS: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_PRTY_STS (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLRAM :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_REG_LRAM) >> 3)
				, "\n\tBRAM_BR :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_REG_BRAM_BR) >> 2)
				, "\n\tBRAM_BL :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_REG_BRAM_BL) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x60130UL: // (BRB1_REG_BRB1_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x4 ) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x4 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLRAM :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_CLR_REG_LRAM) >> 3)
				, "\n\tBRAM_BR :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_CLR_REG_BRAM_BR) >> 2)
				, "\n\tBRAM_BL :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_CLR_REG_BRAM_BL) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x60134UL: // (BRB1_REG_BRB1_PRTY_STS_WR: ACCESS:WR  DataWidth:0x4 ) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_PRTY_STS_WR (ACCESS:WR  DataWidth:0x4 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLRAM :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_WR_REG_LRAM) >> 3)
				, "\n\tBRAM_BR :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_WR_REG_BRAM_BR) >> 2)
				, "\n\tBRAM_BL :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_WR_REG_BRAM_BL) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & BRB1_BRB1_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x60138UL: // (BRB1_REG_BRB1_PRTY_MASK: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_PRTY_MASK (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLRAM :"
				, (u32_t)((data & BRB1_BRB1_PRTY_MASK_REG_LRAM) >> 3)
				, "\n\tBRAM_BR :"
				, (u32_t)((data & BRB1_BRB1_PRTY_MASK_REG_BRAM_BR) >> 2)
				, "\n\tBRAM_BL :"
				, (u32_t)((data & BRB1_BRB1_PRTY_MASK_REG_BRAM_BL) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & BRB1_BRB1_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x6013cUL: // (BRB1_REG_HIGH_LLFC_LOW_THRESHOLD_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_HIGH_LLFC_LOW_THRESHOLD_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks below which the High_llfc signal\n     to interface #n is asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60140UL: // (BRB1_REG_HIGH_LLFC_LOW_THRESHOLD_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_HIGH_LLFC_LOW_THRESHOLD_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks below which the High_llfc signal\n     to interface #n is asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60144UL: // (BRB1_REG_HIGH_LLFC_LOW_THRESHOLD_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_HIGH_LLFC_LOW_THRESHOLD_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks below which the High_llfc signal\n     to interface #n is asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60148UL: // (BRB1_REG_HIGH_LLFC_LOW_THRESHOLD_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_HIGH_LLFC_LOW_THRESHOLD_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks below which the High_llfc signal\n     to interface #n is asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6014cUL: // (BRB1_REG_HIGH_LLFC_HIGH_THRESHOLD_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_HIGH_LLFC_HIGH_THRESHOLD_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks above which the High_llfc signal\n     to interface #n is de-asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60150UL: // (BRB1_REG_HIGH_LLFC_HIGH_THRESHOLD_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_HIGH_LLFC_HIGH_THRESHOLD_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks above which the High_llfc signal\n     to interface #n is de-asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60154UL: // (BRB1_REG_HIGH_LLFC_HIGH_THRESHOLD_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_HIGH_LLFC_HIGH_THRESHOLD_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks above which the High_llfc signal\n     to interface #n is de-asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60158UL: // (BRB1_REG_HIGH_LLFC_HIGH_THRESHOLD_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_HIGH_LLFC_HIGH_THRESHOLD_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks above which the High_llfc signal\n     to interface #n is de-asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6015cUL: // (BRB1_REG_LOW_LLFC_LOW_THRESHOLD_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_LOW_LLFC_LOW_THRESHOLD_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks below which the Low_llfc signal\n     to interface #n is asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60160UL: // (BRB1_REG_LOW_LLFC_LOW_THRESHOLD_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_LOW_LLFC_LOW_THRESHOLD_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks below which the Low_llfc signal\n     to interface #n is asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60164UL: // (BRB1_REG_LOW_LLFC_LOW_THRESHOLD_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_LOW_LLFC_LOW_THRESHOLD_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks below which the Low_llfc signal\n     to interface #n is asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60168UL: // (BRB1_REG_LOW_LLFC_LOW_THRESHOLD_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_LOW_LLFC_LOW_THRESHOLD_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks below which the Low_llfc signal\n     to interface #n is asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6016cUL: // (BRB1_REG_LOW_LLFC_HIGH_THRESHOLD_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_LOW_LLFC_HIGH_THRESHOLD_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks above which the Low_llfc signal\n     to interface #n is de-asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60170UL: // (BRB1_REG_LOW_LLFC_HIGH_THRESHOLD_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_LOW_LLFC_HIGH_THRESHOLD_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks above which the Low_llfc signal\n     to interface #n is de-asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60174UL: // (BRB1_REG_LOW_LLFC_HIGH_THRESHOLD_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_LOW_LLFC_HIGH_THRESHOLD_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks above which the Low_llfc signal\n     to interface #n is de-asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60178UL: // (BRB1_REG_LOW_LLFC_HIGH_THRESHOLD_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nBRB1_REG_LOW_LLFC_HIGH_THRESHOLD_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The number of free blocks above which the Low_llfc signal\n     to interface #n is de-asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6017cUL: // (BRB1_REG_NUM_OF_LLFC_HIGH_CYCLES_0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_LLFC_HIGH_CYCLES_0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the llfc high signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60180UL: // (BRB1_REG_NUM_OF_LLFC_HIGH_CYCLES_1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_LLFC_HIGH_CYCLES_1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the llfc high signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60184UL: // (BRB1_REG_NUM_OF_LLFC_HIGH_CYCLES_2: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_LLFC_HIGH_CYCLES_2 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the llfc high signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60188UL: // (BRB1_REG_NUM_OF_LLFC_HIGH_CYCLES_3: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_LLFC_HIGH_CYCLES_3 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the llfc high signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6018cUL: // (BRB1_REG_NUM_OF_LLFC_LOW_CYCLES_0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_LLFC_LOW_CYCLES_0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the llfc low signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60190UL: // (BRB1_REG_NUM_OF_LLFC_LOW_CYCLES_1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_LLFC_LOW_CYCLES_1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the llfc low signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60194UL: // (BRB1_REG_NUM_OF_LLFC_LOW_CYCLES_2: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_LLFC_LOW_CYCLES_2 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the llfc low signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60198UL: // (BRB1_REG_NUM_OF_LLFC_LOW_CYCLES_3: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_LLFC_LOW_CYCLES_3 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the llfc low signal towards MAC\n     #0 was asserted. Not Functional\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6019cUL: // (BRB1_REG_M_SP_RAM_BM_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nBRB1_REG_M_SP_RAM_BM_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of BM RAM memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601a0UL: // (BRB1_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nBRB1_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO; when bit equal to 1: b0 -disable for\n     parser IF bug 1 fix; b1- enable  for state machine bug 1 fix; b2- disable\n     for  SDM Ifs bug1 fix; b3 - disable for bug2 fix\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601acUL: // (BRB1_REG_NUM_OF_PAUSE_0_CYCLES_0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_PAUSE_0_CYCLES_0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the pause signal towards MAC port\n     class 0 was asserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601b0UL: // (BRB1_REG_NUM_OF_PAUSE_1_CYCLES_0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_PAUSE_1_CYCLES_0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the pause signal towards MAC port\n     class 1 was asserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601b4UL: // (BRB1_REG_NUM_OF_FULL_0_CYCLES_0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_0_CYCLES_0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the full signal towards MAC port\n     class 0 was asserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601b8UL: // (BRB1_REG_NUM_OF_FULL_1_CYCLES_0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_1_CYCLES_0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the full signal towards MAC port\n     class 1 was asserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601bcUL: // (BRB1_REG_NUM_OF_FULL_LB_CYCLES: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_LB_CYCLES (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the full signal towards LB port\n     was asserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601c0UL: // (BRB1_REG_PAUSE_0_XOFF_THRESHOLD_0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_0_XOFF_THRESHOLD_0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks below which the pause signal to\n     class 0 is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601c4UL: // (BRB1_REG_PAUSE_0_XON_THRESHOLD_0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_0_XON_THRESHOLD_0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks above which the pause signal to\n     class 0 is de-asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601c8UL: // (BRB1_REG_PAUSE_1_XOFF_THRESHOLD_0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_1_XOFF_THRESHOLD_0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks below which the pause signal to\n     class 1 is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601ccUL: // (BRB1_REG_PAUSE_1_XON_THRESHOLD_0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_1_XON_THRESHOLD_0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks above which the pause signal to\n     class 1 is de-asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601d0UL: // (BRB1_REG_FULL_0_XOFF_THRESHOLD_0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_0_XOFF_THRESHOLD_0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks below which the full signal to \n    class 0 is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601d4UL: // (BRB1_REG_FULL_0_XON_THRESHOLD_0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_0_XON_THRESHOLD_0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks above which the full signal to \n    class 0 is de-asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601d8UL: // (BRB1_REG_FULL_1_XOFF_THRESHOLD_0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_1_XOFF_THRESHOLD_0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks below which the full signal to \n    class 1 is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601dcUL: // (BRB1_REG_FULL_1_XON_THRESHOLD_0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_1_XON_THRESHOLD_0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks above which the full signal to \n    class 1 is de-asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601e0UL: // (BRB1_REG_FULL_LB_XOFF_THRESHOLD: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_LB_XOFF_THRESHOLD (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks below which the full signal to \n    the LB port is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601e4UL: // (BRB1_REG_FULL_LB_XON_THRESHOLD: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_LB_XON_THRESHOLD (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks above which the full signal to \n    the LB port is de-asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601e8UL: // (BRB1_REG_MAC_GUARANTIED_0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAC_GUARANTIED_0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of blocks guarantied for the MAC port. The register\n     is applicable only when per_class_guaranty_mode  is reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601ecUL: // (BRB1_REG_LB_GUARANTIED: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_LB_GUARANTIED (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of blocks guarantied for the LB port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601f0UL: // (BRB1_REG_BRB_EMPTY_THRESHOLD: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_BRB_EMPTY_THRESHOLD (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks above which the brb_empty signal\n     to TSEMI is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60210UL: // (BRB1_REG_NUM_OF_PAUSE_0_CYCLES_1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_PAUSE_0_CYCLES_1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the pause signal towards MAC port\n     class 0 was asserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60214UL: // (BRB1_REG_NUM_OF_PAUSE_1_CYCLES_1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_PAUSE_1_CYCLES_1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the pause signal towards MAC port\n     class 1 was asserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60218UL: // (BRB1_REG_NUM_OF_FULL_0_CYCLES_1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_0_CYCLES_1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the full signal towards MAC port\n     class 0 was asserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6021cUL: // (BRB1_REG_NUM_OF_FULL_1_CYCLES_1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nBRB1_REG_NUM_OF_FULL_1_CYCLES_1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles that the full signal towards MAC port\n     class 1 was asserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60220UL: // (BRB1_REG_PAUSE_0_XOFF_THRESHOLD_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_0_XOFF_THRESHOLD_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks below which the pause signal to\n     class 0 is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60224UL: // (BRB1_REG_PAUSE_0_XON_THRESHOLD_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_0_XON_THRESHOLD_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks above which the pause signal to\n     class 0 is de-asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60228UL: // (BRB1_REG_PAUSE_1_XOFF_THRESHOLD_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_1_XOFF_THRESHOLD_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks below which the pause signal to\n     class 1 is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6022cUL: // (BRB1_REG_PAUSE_1_XON_THRESHOLD_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PAUSE_1_XON_THRESHOLD_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks above which the pause signal to\n     class 1 is de-asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60230UL: // (BRB1_REG_FULL_0_XOFF_THRESHOLD_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_0_XOFF_THRESHOLD_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks below which the full signal to \n    class 0 is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60234UL: // (BRB1_REG_FULL_0_XON_THRESHOLD_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_0_XON_THRESHOLD_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks above which the full signal to \n    class 0 is de-asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60238UL: // (BRB1_REG_FULL_1_XOFF_THRESHOLD_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_1_XOFF_THRESHOLD_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks below which the full signal to \n    class 1 is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6023cUL: // (BRB1_REG_FULL_1_XON_THRESHOLD_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FULL_1_XON_THRESHOLD_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of free blocks above which the full signal to \n    class 1 is de-asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60240UL: // (BRB1_REG_MAC_GUARANTIED_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAC_GUARANTIED_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of blocks guarantied for the MAC port. The register\n     is applicable only when per_class_guaranty_mode  is reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60244UL: // (BRB1_REG_MAC_0_CLASS_0_GUARANTIED: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAC_0_CLASS_0_GUARANTIED (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of blocks guarantied for class 0 in MAC 0. The\n     register is applicable only when per_class_guaranty_mode is set. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60248UL: // (BRB1_REG_MAC_0_CLASS_1_GUARANTIED: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAC_0_CLASS_1_GUARANTIED (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of blocks guarantied for class 1 in MAC 0. The\n     register is applicable only when per_class_guaranty_mode is set. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6024cUL: // (BRB1_REG_MAC_1_CLASS_0_GUARANTIED: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAC_1_CLASS_0_GUARANTIED (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of blocks guarantied for class 0in MAC1.The register\n     is applicable only when per_class_guaranty_mode is set. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60250UL: // (BRB1_REG_MAC_1_CLASS_1_GUARANTIED: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAC_1_CLASS_1_GUARANTIED (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The number of blocks guarantied for class 1 in MAC 1. The\n     register is applicable only when per_class_guaranty_mode is set. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60254UL: // (BRB1_REG_MAC_0_CLASS_0_GUARANTIED_HYST: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAC_0_CLASS_0_GUARANTIED_HYST (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The hysteresis on the guarantied buffer space for class 0\n     in MAC 1 before signaling XON. The register is applicable only when per_class_guaranty_mode\n     is set. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60258UL: // (BRB1_REG_MAC_0_CLASS_1_GUARANTIED_HYST: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAC_0_CLASS_1_GUARANTIED_HYST (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The hysteresis on the guarantied buffer space for class 1in\n     MAC 0 before signaling XON. The register is applicable only when per_class_guaranty_mode\n     is set. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6025cUL: // (BRB1_REG_MAC_1_CLASS_0_GUARANTIED_HYST: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAC_1_CLASS_0_GUARANTIED_HYST (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The hysteresis on the guarantied buffer space for class 0\n     in MAC 1 before signaling XON. The register is applicable only when per_class_guaranty_mode\n     is set. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60260UL: // (BRB1_REG_MAC_1_CLASS_1_GUARANTIED_HYST: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_MAC_1_CLASS_1_GUARANTIED_HYST (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The hysteresis on the guarantied buffer space for class 1\n     in MAC 1 before signaling XON. The register is applicable only when per_class_guaranty_mode\n     is set. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60264UL: // (BRB1_REG_LB_GUARANTIED_HYST: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_LB_GUARANTIED_HYST (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: The hysteresis on the guarantied buffer space for the Lb \n    port before signaling XON. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60268UL: // (BRB1_REG_PER_CLASS_GUARANTY_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_PER_CLASS_GUARANTY_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates if to use per-class guaranty mode (new mode) or\n     per-MAC guaranty mode (backwards-compatible mode). 0=per-MAC guaranty\n     mode (BC mode). 1=per-class guaranty mode (new mode).  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6027cUL: // (BRB1_REG_EMPTY_BACKWARD_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nBRB1_REG_EMPTY_BACKWARD_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When reset then brb_empty signal will be set according to\n     new E3_B0 formula. When set it will be used an old formula\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60280UL: // (BRB1_REG_SF_MODE_EN: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nBRB1_REG_SF_MODE_EN (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Store and forward enable mode per port (b0 - mac port 0; \n    b1 - mac port 1; b4 - LB port ). When it is set then NPM notification \n    to parser will be transmitted after EOP inputs to BRB. When it is reset\n     then it will be transmitted after header_size bytes or EOP inputs to \n    BRTB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60200UL: // (BRB1_REG_FREE_LIST_PRS_CRDT: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_FREE_LIST_PRS_CRDT (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: At address BRB1_IND_FREE_LIST_PRS_CRDT initialize free head\n    . At address BRB1_IND_FREE_LIST_PRS_CRDT+1 initialize free tail. At address\n     BRB1_IND_FREE_LIST_PRS_CRDT+2 initialize parser initial credit. Warning\n     - following reset the first rbc access to this reg must be write; there\n     can be no more rbc writes after the first one; there can be any number\n     of rbc read following the first write; rbc access not following these\n     rules will result in hang condition.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x6026cUL: // (BRB1_REG_PORT_0_CLASS_0_NUM_OCC_BLOCKS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_0_CLASS_0_NUM_OCC_BLOCKS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: The number of full blocks occpied by port0 class0 .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60270UL: // (BRB1_REG_PORT_0_CLASS_1_NUM_OCC_BLOCKS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_0_CLASS_1_NUM_OCC_BLOCKS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: The number of full blocks occpied by port0 class0 .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60274UL: // (BRB1_REG_PORT_1_CLASS_0_NUM_OCC_BLOCKS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_1_CLASS_0_NUM_OCC_BLOCKS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: The number of full blocks occpied by port0 class0 .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60278UL: // (BRB1_REG_PORT_1_CLASS_1_NUM_OCC_BLOCKS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nBRB1_REG_PORT_1_CLASS_1_NUM_OCC_BLOCKS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: The number of full blocks occpied by port0 class0 .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x61000UL: // (BRB1_REG_LL_RAM: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nBRB1_REG_LL_RAM (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: LL RAM data.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x70000UL: // (BRB1_REG_BIG_RAM: ACCESS:WB  DataWidth:0x100 ) 
			sprintf(disp_str, "\nBRB1_REG_BIG_RAM (ACCESS:WB  DataWidth:0x100 )" \
				"\nDescription: Big RAM data; addressing uses 2 IND bits in item 33 as most\n     significant bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601a4UL: // (BRB1_REG_BRB1_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x601f4UL: // (BRB1_REG_BRB1_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x60284UL: // (BRB1_REG_BRB1_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x62000UL: // (BRB1_REG_BRB1_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nBRB1_REG_BRB1_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0000UL: // (CCM_REG_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. Initialises specific states and statuses. To \n    initialise the state - write 1 into register; to enable working after \n    that - write 0. For debug purposes only.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0004UL: // (CCM_REG_CCM_STORM0_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CCM_STORM0_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - STORM 0 Interface enable. If 0 - the acknowledge input\n     is disregarded; valid is deasserted; all other signals are treated as\n     usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0008UL: // (CCM_REG_CCM_STORM1_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CCM_STORM1_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - STORM 1 Interface enable. If 0 - the acknowledge input\n     is disregarded; valid is deasserted; all other signals are treated as\n     usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd000cUL: // (CCM_REG_CCM_CQM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CCM_CQM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - QM Interface enable. If 0 - the acknowledge input is\n     disregarded; valid is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0010UL: // (CCM_REG_STORM_CCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_STORM_CCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: STORM - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0014UL: // (CCM_REG_CQM_CCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CQM_CCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: QM - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0018UL: // (CCM_REG_CSDM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CSDM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input SDM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd001cUL: // (CCM_REG_TSEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_TSEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input tsem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0020UL: // (CCM_REG_XSEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_XSEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input xsem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0024UL: // (CCM_REG_USEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_USEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input usem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0028UL: // (CCM_REG_PBF_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_PBF_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input pbf Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd002cUL: // (CCM_REG_CDU_AG_WR_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CDU_AG_WR_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU AG write Interface enable. If 0 - the request and valid\n     input are disregarded; all other signals are treated as usual; if 1 - \n    normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0030UL: // (CCM_REG_CDU_AG_RD_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CDU_AG_RD_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU AG read Interface enable. If 0 - the request input is\n     disregarded; valid output is deasserted; all other signals are treated\n     as usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0034UL: // (CCM_REG_CDU_SM_WR_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CDU_SM_WR_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU STORM write Interface enable. If 0 - the request and \n    valid input is disregarded; all other signals are treated as usual; if\n     1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0038UL: // (CCM_REG_CDU_SM_RD_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CDU_SM_RD_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU STORM read Interface enable. If 0 - the request input\n     is disregarded; valid output is deasserted; all other signals are treated\n     as usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd003cUL: // (CCM_REG_CCM_CFC_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CCM_CFC_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - CFC Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0040UL: // (CCM_REG_XX_MAX_LL_SZ: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCCM_REG_XX_MAX_LL_SZ (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Maximum link list size per connection in the XX protection\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0044UL: // (CCM_REG_XX_OVFL_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCCM_REG_XX_OVFL_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID; sent to the STORM in case of XX overflow.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0048UL: // (CCM_REG_XX_MAX_NUM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCCM_REG_XX_MAX_NUM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The maximum number of connections in the XX protection.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd004cUL: // (CCM_REG_N_SM_CTX_LD_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0050UL: // (CCM_REG_N_SM_CTX_LD_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0054UL: // (CCM_REG_N_SM_CTX_LD_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0058UL: // (CCM_REG_N_SM_CTX_LD_3: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_3 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd005cUL: // (CCM_REG_N_SM_CTX_LD_4: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_4 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0060UL: // (CCM_REG_N_SM_CTX_LD_5: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_5 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0064UL: // (CCM_REG_N_SM_CTX_LD_6: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_6 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0068UL: // (CCM_REG_N_SM_CTX_LD_7: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_7 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd006cUL: // (CCM_REG_N_SM_CTX_LD_8: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_8 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0070UL: // (CCM_REG_N_SM_CTX_LD_9: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_9 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0074UL: // (CCM_REG_N_SM_CTX_LD_10: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_10 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0078UL: // (CCM_REG_N_SM_CTX_LD_11: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_11 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd007cUL: // (CCM_REG_N_SM_CTX_LD_12: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_12 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0080UL: // (CCM_REG_N_SM_CTX_LD_13: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_13 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0084UL: // (CCM_REG_N_SM_CTX_LD_14: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_14 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0088UL: // (CCM_REG_N_SM_CTX_LD_15: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_N_SM_CTX_LD_15 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs(128 bits); loaded from the\n     STORM context and sent to STORM; for a specific connection type. The \n    double REG-pairs are used in order to align to STORM context row size \n    of 128 bits. The offset of these data in the STORM context is always 0\n    . Index _(0..15) stands for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd008cUL: // (CCM_REG_CQM_CCM_HDR_P: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nCCM_REG_CQM_CCM_HDR_P (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header value for QM request (primary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0090UL: // (CCM_REG_CQM_CCM_HDR_S: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nCCM_REG_CQM_CCM_HDR_S (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header value for QM request (secondary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0094UL: // (CCM_REG_ERR_CCM_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nCCM_REG_ERR_CCM_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header for QM formatting in case of an error in the\n     QM inputs.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0098UL: // (CCM_REG_ERR_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCCM_REG_ERR_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID in case the input message ErrorFlg is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd009cUL: // (CCM_REG_STORM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_STORM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the STORM input in the WRR (Weighted Round \n    robin) mechanism. 0 stands for weight 8 (the most prioritised); 1 stands\n     for weight 1(least prioritised); 2 stands for weight 2 (more prioritised\n    );etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00a0UL: // (CCM_REG_TSEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_TSEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input tsem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00a4UL: // (CCM_REG_XSEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_XSEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input xsem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00a8UL: // (CCM_REG_USEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_USEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input usem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00acUL: // (CCM_REG_PBF_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_PBF_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input pbf in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00b0UL: // (CCM_REG_CP_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_CP_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the CP input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00b4UL: // (CCM_REG_CSDM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_CSDM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the SDM input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00b8UL: // (CCM_REG_CQM_P_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_CQM_P_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the QM (primary) input in the WRR mechanism\n    . 0 stands for weight 8 (the most prioritised); 1 stands for weight 1(\n    least prioritised); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00bcUL: // (CCM_REG_CQM_S_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_CQM_S_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the QM (secondary) input in the WRR mechanism\n    . 0 stands for weight 8 (the most prioritised); 1 stands for weight 1(\n    least prioritised); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00c0UL: // (CCM_REG_CCM_CQM_USE_Q: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CCM_CQM_USE_Q (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set the Q index; received from the QM is inserted to event\n     ID. Otherwise 0 is inserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00c4UL: // (CCM_REG_CCM_REG0_SZ: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_CCM_REG0_SZ (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The size of AG context region 0 in REG-pairs. Designates \n    the MS REG-pair number (e.g. if region 0 is 6 REG-pairs; the value should\n     be 5). Is used to determine the number of the AG context REG-pairs written\n     back; when the input message Reg1WbFlg isn't set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00c8UL: // (CCM_REG_CNT_AUX1_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_CNT_AUX1_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Auxillary counter flag Q number 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00ccUL: // (CCM_REG_CNT_AUX2_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_CNT_AUX2_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Auxillary counter flag Q number 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00d0UL: // (CCM_REG_CNT_AUX3_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_CNT_AUX3_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Auxillary counter flag Q number 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00d4UL: // (CCM_REG_CNT_AUX4_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_CNT_AUX4_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Auxillary counter flag Q number 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00d8UL: // (CCM_REG_CNT_AUX5_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_CNT_AUX5_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Auxillary counter flag Q number 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00dcUL: // (CCM_REG_CNT_AUX6_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_CNT_AUX6_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Auxillary counter flag Q number 6.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00e0UL: // (CCM_REG_SLOW_PATH_INV_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_SLOW_PATH_INV_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Slow path invalid counter flag Q number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00e4UL: // (CCM_REG_RST_RCVD_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_RST_RCVD_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: RST received flag Q number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00e8UL: // (CCM_REG_AUX1_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_AUX1_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Aux 1 flag Q number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00ecUL: // (CCM_REG_AUX2_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_AUX2_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Aux 2 flag Q number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00f0UL: // (CCM_REG_URSE_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_URSE_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: ULP Rx SE counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00f4UL: // (CCM_REG_URINV_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_URINV_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: ULP Rx Invalide counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00f8UL: // (CCM_REG_REL_SEQ_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_REL_SEQ_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Release sequence decision Q index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00fcUL: // (CCM_REG_HQ_PROD_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_HQ_PROD_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: HQ producer index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0100UL: // (CCM_REG_ORQ_UR_CONS_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_ORQ_UR_CONS_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: ORQ ULP Rx consumer index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0104UL: // (CCM_REG_RQ_UR_CONS_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_RQ_UR_CONS_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: RQ consumer updated by ULP Rx index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0108UL: // (CCM_REG_INV_DONE_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_INV_DONE_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: General flags index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd010cUL: // (CCM_REG_AUX1_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_AUX1_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Auxillary flag Q number 1 index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0110UL: // (CCM_REG_AUX2_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_AUX2_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Auxillary flag Q number 2 index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd014cUL: // (CCM_REG_SM_CTX0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCCM_REG_SM_CTX0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Debug only. TM bits of STORM context.LSB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0150UL: // (CCM_REG_AG_CTX0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCCM_REG_AG_CTX0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Debug only. TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0154UL: // (CCM_REG_AG_CTX1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCCM_REG_AG_CTX1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Debug only. TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0158UL: // (CCM_REG_AG_CTX2_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCCM_REG_AG_CTX2_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Debug only. TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd015cUL: // (CCM_REG_GR_ARB_TYPE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_GR_ARB_TYPE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Arbitration between Input Arbiter groups: 0 - fair Round-\n    Robin; 1 - strict priority defined by ~ccm_registers_gr_ag_pr.gr_ag_pr\n    ; ~ccm_registers_gr_ld0_pr.gr_ld0_pr and ~ccm_registers_gr_ld1_pr.gr_ld1_pr\n    . Groups are according to channels and outputs to STORM: aggregation; \n    load FIC0; load FIC1 and store.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0160UL: // (CCM_REG_GR_AG_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_GR_AG_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Aggregation channel group priority. The lowest priority is\n     0; the highest priority is 3. It is supposed; that the Store channel \n    priority is the compliment to 4 of the rest priorities - Aggregation channel\n    ; Load (FIC0) channel and Load (FIC1).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0164UL: // (CCM_REG_GR_LD0_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_GR_LD0_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Load (FIC0) channel group priority. The lowest priority is\n     0; the highest priority is 3. It is supposed; that the Store channel \n    priority is the compliment to 4 of the rest priorities - Aggregation channel\n    ; Load (FIC0) channel and Load (FIC1).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0168UL: // (CCM_REG_GR_LD1_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_GR_LD1_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Load (FIC1) channel group priority. The lowest priority is\n     0; the highest priority is 3. It is supposed; that the Store channel \n    priority is the compliment to 4 of the rest priorities - Aggregation channel\n    ; Load (FIC0) channel and Load (FIC1).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd016cUL: // (CCM_REG_STORM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_STORM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the STORM interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0170UL: // (CCM_REG_CSDM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CSDM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the SDM interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0174UL: // (CCM_REG_TSEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_TSEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the tsem interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0178UL: // (CCM_REG_XSEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_XSEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the xsem interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd017cUL: // (CCM_REG_USEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_USEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when message length mismatch (relative to last indication\n    ) at the usem interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0180UL: // (CCM_REG_PBF_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_PBF_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the pbf interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0184UL: // (CCM_REG_XX_FREE: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nCCM_REG_XX_FREE (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: Used to read the value of XX protection Free counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0188UL: // (CCM_REG_CAM_OCCUP: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCCM_REG_CAM_OCCUP (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Used to read the value of the XX protection CAM occupancy\n     counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd018cUL: // (CCM_REG_UNLOCK_MISS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_UNLOCK_MISS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Error; indicating the LCID to be unlocked from XX protection\n     doesn't exist in LCID CAM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0190UL: // (CCM_REG_CQM_GLB_USE_CNTR: ACCESS:R  DataWidth:0x1a ) 
			sprintf(disp_str, "\nCCM_REG_CQM_GLB_USE_CNTR (ACCESS:R  DataWidth:0x1a )" \
				"\nDescription: Used to read the QM global usage counter - the counter of\n     QM requests still pending in the QM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0194UL: // (CCM_REG_CP_BUF_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_CP_BUF_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication of CP buffer is empty.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0198UL: // (CCM_REG_CP_BUF_STATUS: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCCM_REG_CP_BUF_STATUS (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: CP buffer status.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd019cUL: // (CCM_REG_XX_OVFL_CNTR: ACCESS:ST  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCCM_REG_XX_OVFL_CNTR (ACCESS:ST  DataWidth:0x10 )" \
				"\nDescription: Counter of XX 0verflow occurencies.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01a0UL: // (CCM_REG_STORM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_STORM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the STORM input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01a4UL: // (CCM_REG_CSDM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_CSDM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input SDM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01a8UL: // (CCM_REG_TSEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_TSEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input tsem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01acUL: // (CCM_REG_XSEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_XSEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input xsem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01b0UL: // (CCM_REG_USEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_USEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input usem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01b4UL: // (CCM_REG_PBF_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_PBF_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input pbf.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01b8UL: // (CCM_REG_CP_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_CP_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the CP input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01bcUL: // (CCM_REG_CQM_P_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_CQM_P_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the QM input (primary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01c0UL: // (CCM_REG_CQM_S_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_CQM_S_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the QM input (secondary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01c4UL: // (CCM_REG_STORM_OUT_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_STORM_OUT_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the output messages at FIC0 and FIC1 interfaces\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01c8UL: // (CCM_REG_CQM_OUT_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCCM_REG_CQM_OUT_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the output QM commands.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01ccUL: // (CCM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCCM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from CCM\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01d0UL: // (CCM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCCM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from CCM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select; before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01d4UL: // (CCM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCCM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from CCM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01d8UL: // (CCM_REG_CCM_INT_STS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nCCM_REG_CCM_INT_STS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_CP0_ERR) >> 8)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_PBF_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_USEM_ERR) >> 6)
				, "\n\tXSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_XSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_TSEM_ERR) >> 4)
				, "\n\tCSDM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_CSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CCM_CCM_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xd01dcUL: // (CCM_REG_CCM_INT_STS_CLR: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nCCM_REG_CCM_INT_STS_CLR (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_CP0_ERR) >> 8)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_PBF_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_USEM_ERR) >> 6)
				, "\n\tXSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_XSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_TSEM_ERR) >> 4)
				, "\n\tCSDM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_CSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CCM_CCM_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xd01e0UL: // (CCM_REG_CCM_INT_STS_WR: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nCCM_REG_CCM_INT_STS_WR (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_CP0_ERR) >> 8)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_PBF_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_USEM_ERR) >> 6)
				, "\n\tXSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_XSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_TSEM_ERR) >> 4)
				, "\n\tCSDM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_CSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CCM_CCM_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xd01e4UL: // (CCM_REG_CCM_INT_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nCCM_REG_CCM_INT_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_CP0_ERR) >> 8)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_PBF_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_USEM_ERR) >> 6)
				, "\n\tXSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_XSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_TSEM_ERR) >> 4)
				, "\n\tCSDM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_CSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CCM_CCM_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xd01e8UL: // (CCM_REG_CCM_PRTY_STS: ACCESS:R  DataWidth:0x1b ) 
			sprintf(disp_str, "\nCCM_REG_CCM_PRTY_STS (ACCESS:R  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_CP0_PRTY) >> 24)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_PBF_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_USEM_PRTY) >> 22)
				, "\n\tXSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_XSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_TSEM_PRTY) >> 20)
				, "\n\tCSDM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_CSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0xd01ecUL: // (CCM_REG_CCM_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x1b ) 
			sprintf(disp_str, "\nCCM_REG_CCM_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_CP0_PRTY) >> 24)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_PBF_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_USEM_PRTY) >> 22)
				, "\n\tXSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_XSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_TSEM_PRTY) >> 20)
				, "\n\tCSDM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_CSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0xd01f0UL: // (CCM_REG_CCM_PRTY_STS_WR: ACCESS:WR  DataWidth:0x1b ) 
			sprintf(disp_str, "\nCCM_REG_CCM_PRTY_STS_WR (ACCESS:WR  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_CP0_PRTY) >> 24)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_PBF_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_USEM_PRTY) >> 22)
				, "\n\tXSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_XSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_TSEM_PRTY) >> 20)
				, "\n\tCSDM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_CSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CCM_CCM_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0xd01f4UL: // (CCM_REG_CCM_PRTY_MASK: ACCESS:RW  DataWidth:0x1b ) 
			sprintf(disp_str, "\nCCM_REG_CCM_PRTY_MASK (ACCESS:RW  DataWidth:0x1b )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_CP0_PRTY) >> 24)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_PBF_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_USEM_PRTY) >> 22)
				, "\n\tXSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_XSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_TSEM_PRTY) >> 20)
				, "\n\tCSDM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_CSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CCM_CCM_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0xd01f8UL: // (CCM_REG_SM_CTX1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCCM_REG_SM_CTX1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Debug only. TM bits of STORM context. MSB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01fcUL: // (CCM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCCM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: chicken bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0800UL: // (CCM_REG_IS_UM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_IS_UM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Debug only. TM bits of input stage UM buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0804UL: // (CCM_REG_IS_STORM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_IS_STORM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Debug only. TM bits of input stage STORM buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0808UL: // (CCM_REG_PM_RAM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_PM_RAM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Debug only. TM bits of Pending Messages RAM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd080cUL: // (CCM_REG_IS_USEM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_IS_USEM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Debug only. TM bits of USEM input stage..\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0818UL: // (CCM_REG_UM_FIC1_FORCE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCCM_REG_UM_FIC1_FORCE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0-messages unlocked from Pending messages RAM go to the FIC\n     for which they were designated in input message; 1-messages unlocked \n    from Pending messages RAM are forced to FIC1 whether they were destined\n     to FIC0 or FIC1 in original message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0114UL: // (CCM_REG_QOS_PHYS_QNUM0_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_QOS_PHYS_QNUM0_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 0 with QOS equal\n     0 port index 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0118UL: // (CCM_REG_QOS_PHYS_QNUM0_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_QOS_PHYS_QNUM0_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 0 with QOS equal\n     0 port index 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd011cUL: // (CCM_REG_QOS_PHYS_QNUM1_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_QOS_PHYS_QNUM1_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 0 with QOS equal\n     1 port index 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0120UL: // (CCM_REG_QOS_PHYS_QNUM1_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_QOS_PHYS_QNUM1_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 0 with QOS equal\n     1 per port index 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0124UL: // (CCM_REG_QOS_PHYS_QNUM2_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_QOS_PHYS_QNUM2_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 0 with QOS equal\n     2 port index 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0128UL: // (CCM_REG_QOS_PHYS_QNUM2_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_QOS_PHYS_QNUM2_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 0 with QOS equal\n     2 port index 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd012cUL: // (CCM_REG_QOS_PHYS_QNUM3_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_QOS_PHYS_QNUM3_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 0 with QOS equal\n     3 port index 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0130UL: // (CCM_REG_QOS_PHYS_QNUM3_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_QOS_PHYS_QNUM3_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 0 with QOS equal\n     3 port index 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0134UL: // (CCM_REG_PHYS_QNUM1_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_PHYS_QNUM1_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 1 per port index\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0138UL: // (CCM_REG_PHYS_QNUM1_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_PHYS_QNUM1_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 1 per port index\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd013cUL: // (CCM_REG_PHYS_QNUM2_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_PHYS_QNUM2_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 2 per port index\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0140UL: // (CCM_REG_PHYS_QNUM2_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_PHYS_QNUM2_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 2 per port index\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0144UL: // (CCM_REG_PHYS_QNUM3_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_PHYS_QNUM3_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 3 per port index\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0148UL: // (CCM_REG_PHYS_QNUM3_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nCCM_REG_PHYS_QNUM3_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number of queue number 3 per port index\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0200UL: // (CCM_REG_CAM_OCCUP_ST: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCCM_REG_CAM_OCCUP_ST (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: XX protection LCID CAM occupancy sticky status. The write\n     to the register is performed by the XX internal circuitry.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0204UL: // (CCM_REG_CFC_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCCM_REG_CFC_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CFC output initial credit. Max credit available - 15.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 1 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0208UL: // (CCM_REG_CP_MSG: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCCM_REG_CP_MSG (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Indirect access to write the CP message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd020cUL: // (CCM_REG_CQM_INIT_CRD: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCCM_REG_CQM_INIT_CRD (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: QM output initial credit. Max credit available - 32. Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 32 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0210UL: // (CCM_REG_FIC0_INIT_CRD: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCCM_REG_FIC0_INIT_CRD (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: FIC0 output initial credit. Max credit available - 255. Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 64 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0214UL: // (CCM_REG_FIC1_INIT_CRD: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCCM_REG_FIC1_INIT_CRD (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: FIC1 output initial credit. Max credit available - 255.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 64 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0218UL: // (CCM_REG_GLB_CNT_STICKY: ACCESS:RW  DataWidth:0x1a ) 
			sprintf(disp_str, "\nCCM_REG_GLB_CNT_STICKY (ACCESS:RW  DataWidth:0x1a )" \
				"\nDescription: QM global usage counter maximum sticky value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd021cUL: // (CCM_REG_LL_SZ_STICKY: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCCM_REG_LL_SZ_STICKY (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: XX protection Link List maximum value ever reached sticky\n     value for any connection. The write to the register is performed by the\n     XX internal circuitry.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0220UL: // (CCM_REG_XX_INIT_CRD: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCCM_REG_XX_INIT_CRD (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Initial value for the credit counter; responsible for fulfilling\n     of the Input Stage XX protection buffer by the XX protection pending \n    messages. Max credit available - 127. Write writes the initial credit \n    value; read returns the current value of the credit counter. Must be initialized\n     to maximum XX protected message size - 2 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0224UL: // (CCM_REG_XX_MSG_NUM: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nCCM_REG_XX_MSG_NUM (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: The maximum number of pending messages; which may be stored\n     in XX protection. At write the ~ccm_registers_xx_free.xx_free counter\n     is also written.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0228UL: // (CCM_REG_LCID_CAM_0: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_0 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd022cUL: // (CCM_REG_LCID_CAM_1: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_1 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0230UL: // (CCM_REG_LCID_CAM_2: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_2 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0234UL: // (CCM_REG_LCID_CAM_3: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_3 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0238UL: // (CCM_REG_LCID_CAM_4: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_4 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd023cUL: // (CCM_REG_LCID_CAM_5: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_5 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0240UL: // (CCM_REG_LCID_CAM_6: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_6 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0244UL: // (CCM_REG_LCID_CAM_7: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_7 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0248UL: // (CCM_REG_LCID_CAM_8: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_8 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd024cUL: // (CCM_REG_LCID_CAM_9: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_9 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0250UL: // (CCM_REG_LCID_CAM_10: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_10 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0254UL: // (CCM_REG_LCID_CAM_11: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_11 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0258UL: // (CCM_REG_LCID_CAM_12: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_12 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd025cUL: // (CCM_REG_LCID_CAM_13: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_13 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0260UL: // (CCM_REG_LCID_CAM_14: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_14 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0264UL: // (CCM_REG_LCID_CAM_15: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_15 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0268UL: // (CCM_REG_LCID_CAM_16: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_16 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd026cUL: // (CCM_REG_LCID_CAM_17: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCCM_REG_LCID_CAM_17 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0280UL: // (CCM_REG_XX_TABLE: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nCCM_REG_XX_TABLE (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: Indirect access to the XX table of the XX protection mechanism\n    . The fields are: [5:0] - tail pointer;[11:6] - Link List size;[17:12] - \n    header pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0300UL: // (CCM_REG_XX_DESCR_TABLE: ACCESS:RW  DataWidth:0x13 ) 
			sprintf(disp_str, "\nCCM_REG_XX_DESCR_TABLE (ACCESS:RW  DataWidth:0x13 )" \
				"\nDescription: Indirect access to the descriptor table of the XX protection\n     mechanism. The fields are: [5:0] - message length; [12:6] - message pointer\n    ;[18:13] - next pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0400UL: // (CCM_REG_XX_PEND_MSG: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCCM_REG_XX_PEND_MSG (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only. Indirect access to the Pending messages RAM of\n     the XX protection mechanism. Bits [7:1] stand for row number; bit [0] \n    stand for one of 2 REGs in the row.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4000UL: // (CCM_REG_AG_CTX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCCM_REG_AG_CTX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indirect access to AG context with 32-bits granularity. The\n     bits [11:8] of the address should be the offset within the accessed LCID\n     context; the bits [7:0] are the accessed LCID.Example: to write to REG3\n     LCID100. The RBC address should be 12'h364.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd8000UL: // (CCM_REG_STORM_CTX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCCM_REG_STORM_CTX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indirect access to STORM context with 32-bits granularity\n    . The bits [12:8] of the address should be the offset within the accessed\n     LCID context; the bits [7:0] are the accessed LCID.Example: to write \n    to REG10 LCID100. The RBC address should be 13'ha64.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0810UL: // (CCM_REG_DT_TM: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCCM_REG_DT_TM (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Debug only. TM bits of Descriptor table.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0270UL: // (CCM_REG_CCM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCCM_REG_CCM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd081cUL: // (CCM_REG_CCM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCCM_REG_CCM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101000UL: // (CDU_REG_CDU_CHK_MASK0: ACCESS:RW  DataWidth:0x18  Multi Field Register  ) 
			sprintf(disp_str, "\nCDU_REG_CDU_CHK_MASK0 (ACCESS:RW  DataWidth:0x18  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCHECK_EN3 :"
				, (u32_t)((data & CDU_CDU_CHK_MASK0_REG_CHECK_EN3) >> 18)
				, "\n\tCHECK_EN2 :"
				, (u32_t)((data & CDU_CDU_CHK_MASK0_REG_CHECK_EN2) >> 12)
				, "\n\tCHECK_EN1 :"
				, (u32_t)((data & CDU_CDU_CHK_MASK0_REG_CHECK_EN1) >> 6)
				, "\n\tCHECK_EN0 :"
				, (u32_t)((data & CDU_CDU_CHK_MASK0_REG_CHECK_EN0) >> 0));
			break;
		case 0x101004UL: // (CDU_REG_CDU_CHK_MASK1: ACCESS:RW  DataWidth:0x18  Multi Field Register  ) 
			sprintf(disp_str, "\nCDU_REG_CDU_CHK_MASK1 (ACCESS:RW  DataWidth:0x18  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCHECK_EN7 :"
				, (u32_t)((data & CDU_CDU_CHK_MASK1_REG_CHECK_EN7) >> 18)
				, "\n\tCHECK_EN6 :"
				, (u32_t)((data & CDU_CDU_CHK_MASK1_REG_CHECK_EN6) >> 12)
				, "\n\tCHECK_EN5 :"
				, (u32_t)((data & CDU_CDU_CHK_MASK1_REG_CHECK_EN5) >> 6)
				, "\n\tCHECK_EN4 :"
				, (u32_t)((data & CDU_CDU_CHK_MASK1_REG_CHECK_EN4) >> 0));
			break;
		case 0x101008UL: // (CDU_REG_CDU_CONTROL0: ACCESS:RW  DataWidth:0x9  Multi Field Register  ) 
			sprintf(disp_str, "\nCDU_REG_CDU_CONTROL0 (ACCESS:RW  DataWidth:0x9  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMASK_PCIE :"
				, (u32_t)((data & CDU_CDU_CONTROL0_REG_MASK_PCIE) >> 8)
				, "\n\tDISABLE_OUTPUTS :"
				, (u32_t)((data & CDU_CDU_CONTROL0_REG_DISABLE_OUTPUTS) >> 7)
				, "\n\tDISABLE_INPUTS :"
				, (u32_t)((data & CDU_CDU_CONTROL0_REG_DISABLE_INPUTS) >> 6)
				, "\n\tLDRESP_ADDR_DIS :"
				, (u32_t)((data & CDU_CDU_CONTROL0_REG_LDRESP_ADDR_DIS) >> 5)
				, "\n\tMASK_ECC :"
				, (u32_t)((data & CDU_CDU_CONTROL0_REG_MASK_ECC) >> 4)
				, "\n\tDISABLE_DIVIDER :"
				, (u32_t)((data & CDU_CDU_CONTROL0_REG_DISABLE_DIVIDER) >> 3)
				, "\n\tMATT_SP :"
				, (u32_t)((data & CDU_CDU_CONTROL0_REG_MATT_SP) >> 2)
				, "\n\tL1TT_SP :"
				, (u32_t)((data & CDU_CDU_CONTROL0_REG_L1TT_SP) >> 1)
				, "\n\tPXP_ACTIVE :"
				, (u32_t)((data & CDU_CDU_CONTROL0_REG_PXP_ACTIVE) >> 0));
			break;
		case 0x10100cUL: // (CDU_REG_CDU_CONTROL1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nCDU_REG_CDU_CONTROL1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWBL1REQ_INTERGAP :"
				, (u32_t)((data & CDU_CDU_CONTROL1_REG_WBL1REQ_INTERGAP) >> 22)
				, "\n\tWBDPRELAX :"
				, (u32_t)((data & CDU_CDU_CONTROL1_REG_WBDPRELAX) >> 21)
				, "\n\tLDL1REQ_INTERGAP :"
				, (u32_t)((data & CDU_CDU_CONTROL1_REG_LDL1REQ_INTERGAP) >> 16)
				, "\n\tLDDPRELAX :"
				, (u32_t)((data & CDU_CDU_CONTROL1_REG_LDDPRELAX) >> 15)
				, "\n\tWBDP_RDTHRESH :"
				, (u32_t)((data & CDU_CDU_CONTROL1_REG_WBDP_RDTHRESH) >> 10)
				, "\n\tWBBUF_THRESH :"
				, (u32_t)((data & CDU_CDU_CONTROL1_REG_WBBUF_THRESH) >> 5)
				, "\n\tLDBUF_THRESH :"
				, (u32_t)((data & CDU_CDU_CONTROL1_REG_LDBUF_THRESH) >> 0));
			break;
		case 0x101010UL: // (CDU_REG_CDU_DEBUG: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nCDU_REG_CDU_DEBUG (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPXP_INIT_WBCREDIT_SET :"
				, (u32_t)((data & CDU_CDU_DEBUG_REG_PXP_INIT_WBCREDIT_SET) >> 9)
				, "\n\tPXP_INIT_WBCREDIT :"
				, (u32_t)((data & CDU_CDU_DEBUG_REG_PXP_INIT_WBCREDIT) >> 6)
				, "\n\tPXP_INIT_LDCREDIT_SET :"
				, (u32_t)((data & CDU_CDU_DEBUG_REG_PXP_INIT_LDCREDIT_SET) >> 5)
				, "\n\tPXP_INIT_LDCREDIT :"
				, (u32_t)((data & CDU_CDU_DEBUG_REG_PXP_INIT_LDCREDIT) >> 2)
				, "\n\tDISABLE_MERGE :"
				, (u32_t)((data & CDU_CDU_DEBUG_REG_DISABLE_MERGE) >> 1)
				, "\n\tMASK_LD_EOP_ERR :"
				, (u32_t)((data & CDU_CDU_DEBUG_REG_MASK_LD_EOP_ERR) >> 0));
			break;
		case 0x101018UL: // (CDU_REG_LD_L1_NUM_ERROR_DATA: ACCESS:RC  DataWidth:0x14 ) 
			sprintf(disp_str, "\nCDU_REG_LD_L1_NUM_ERROR_DATA (ACCESS:RC  DataWidth:0x14 )" \
				"\nDescription: logging of error data in case of a CDU load error:{iqcc_l1_regions\n    ;iqcc_l1_type;iqcc_l1_lcid}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10101cUL: // (CDU_REG_WB_L1_NUM_ERROR_DATA: ACCESS:RC  DataWidth:0x14 ) 
			sprintf(disp_str, "\nCDU_REG_WB_L1_NUM_ERROR_DATA (ACCESS:RC  DataWidth:0x14 )" \
				"\nDescription: logging of error data in case of a CDU load error:{iqcc_l1_regions\n    ;iqcc_l1_type;iqcc_l1_lcid}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101020UL: // (CDU_REG_CDU_GLOBAL_PARAMS: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nCDU_REG_CDU_GLOBAL_PARAMS (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tNCIB :"
				, (u32_t)((data & CDU_CDU_GLOBAL_PARAMS_REG_NCIB) >> 24)
				, "\n\tBLOCK_WASTE :"
				, (u32_t)((data & CDU_CDU_GLOBAL_PARAMS_REG_BLOCK_WASTE) >> 12));
			break;
		case 0x101024UL: // (CDU_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCDU_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: For dbgmux usage (debug data that goes from CDU to the DBG\n     block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101028UL: // (CDU_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCDU_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: For dbgmux usage (debug data that goes from CDU to the DBG\n     block) - for enabling bytes in the selected line (after the select and\n     before the shift)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10102cUL: // (CDU_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCDU_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: For dbgmux usage (debug data that goes from CDU to the DBG\n     block) - for circular right shifting of the selected line (after the \n    enabling)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101030UL: // (CDU_REG_CDU_INT_STS: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nCDU_REG_CDU_INT_STS (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWBOFFSET_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_REG_WBOFFSET_ERROR) >> 6)
				, "\n\tLDOFFSET_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_REG_LDOFFSET_ERROR) >> 5)
				, "\n\tBVALID_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_REG_BVALID_ERROR) >> 4)
				, "\n\tWB_L1_NUM_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_REG_WB_L1_NUM_ERROR) >> 3)
				, "\n\tLD_L1_NUM_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_REG_LD_L1_NUM_ERROR) >> 2)
				, "\n\tEOP_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_REG_EOP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x101034UL: // (CDU_REG_CDU_INT_STS_CLR: ACCESS:RC  DataWidth:0x7 ) 
			sprintf(disp_str, "\nCDU_REG_CDU_INT_STS_CLR (ACCESS:RC  DataWidth:0x7 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWBOFFSET_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_CLR_REG_WBOFFSET_ERROR) >> 6)
				, "\n\tLDOFFSET_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_CLR_REG_LDOFFSET_ERROR) >> 5)
				, "\n\tBVALID_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_CLR_REG_BVALID_ERROR) >> 4)
				, "\n\tWB_L1_NUM_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_CLR_REG_WB_L1_NUM_ERROR) >> 3)
				, "\n\tLD_L1_NUM_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_CLR_REG_LD_L1_NUM_ERROR) >> 2)
				, "\n\tEOP_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_CLR_REG_EOP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x101038UL: // (CDU_REG_CDU_INT_STS_WR: ACCESS:WR  DataWidth:0x7 ) 
			sprintf(disp_str, "\nCDU_REG_CDU_INT_STS_WR (ACCESS:WR  DataWidth:0x7 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWBOFFSET_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_WR_REG_WBOFFSET_ERROR) >> 6)
				, "\n\tLDOFFSET_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_WR_REG_LDOFFSET_ERROR) >> 5)
				, "\n\tBVALID_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_WR_REG_BVALID_ERROR) >> 4)
				, "\n\tWB_L1_NUM_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_WR_REG_WB_L1_NUM_ERROR) >> 3)
				, "\n\tLD_L1_NUM_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_WR_REG_LD_L1_NUM_ERROR) >> 2)
				, "\n\tEOP_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_WR_REG_EOP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x10103cUL: // (CDU_REG_CDU_INT_MASK: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nCDU_REG_CDU_INT_MASK (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWBOFFSET_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_MASK_REG_WBOFFSET_ERROR) >> 6)
				, "\n\tLDOFFSET_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_MASK_REG_LDOFFSET_ERROR) >> 5)
				, "\n\tBVALID_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_MASK_REG_BVALID_ERROR) >> 4)
				, "\n\tWB_L1_NUM_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_MASK_REG_WB_L1_NUM_ERROR) >> 3)
				, "\n\tLD_L1_NUM_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_MASK_REG_LD_L1_NUM_ERROR) >> 2)
				, "\n\tEOP_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_MASK_REG_EOP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CDU_CDU_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x101040UL: // (CDU_REG_CDU_PRTY_STS: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCDU_REG_CDU_PRTY_STS (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tL1TT_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_REG_L1TT_PAR_ERR) >> 4)
				, "\n\tMATT_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_REG_MATT_PAR_ERR) >> 3)
				, "\n\tCDULDBUF_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_REG_CDULDBUF_PAR_ERR) >> 2)
				, "\n\tCDUWBBUF_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_REG_CDUWBBUF_PAR_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x101044UL: // (CDU_REG_CDU_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCDU_REG_CDU_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x5 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tL1TT_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_CLR_REG_L1TT_PAR_ERR) >> 4)
				, "\n\tMATT_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_CLR_REG_MATT_PAR_ERR) >> 3)
				, "\n\tCDULDBUF_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_CLR_REG_CDULDBUF_PAR_ERR) >> 2)
				, "\n\tCDUWBBUF_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_CLR_REG_CDUWBBUF_PAR_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x101048UL: // (CDU_REG_CDU_PRTY_STS_WR: ACCESS:WR  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCDU_REG_CDU_PRTY_STS_WR (ACCESS:WR  DataWidth:0x5 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tL1TT_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_WR_REG_L1TT_PAR_ERR) >> 4)
				, "\n\tMATT_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_WR_REG_MATT_PAR_ERR) >> 3)
				, "\n\tCDULDBUF_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_WR_REG_CDULDBUF_PAR_ERR) >> 2)
				, "\n\tCDUWBBUF_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_WR_REG_CDUWBBUF_PAR_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CDU_CDU_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x10104cUL: // (CDU_REG_CDU_PRTY_MASK: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCDU_REG_CDU_PRTY_MASK (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tL1TT_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_MASK_REG_L1TT_PAR_ERR) >> 4)
				, "\n\tMATT_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_MASK_REG_MATT_PAR_ERR) >> 3)
				, "\n\tCDULDBUF_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_MASK_REG_CDULDBUF_PAR_ERR) >> 2)
				, "\n\tCDUWBBUF_PAR_ERR :"
				, (u32_t)((data & CDU_CDU_PRTY_MASK_REG_CDUWBBUF_PAR_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CDU_CDU_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x101050UL: // (CDU_REG_MF_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCDU_REG_MF_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: when this bit is set the CDU operates in multifunction mode\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101054UL: // (CDU_REG_MEM_TM_PORTS: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nCDU_REG_MEM_TM_PORTS (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: TM Bits of memories: matt bits 1:0; l1tt_low0 bits 3:2; l1tt_low1\n     bits 5:4; l1tt_high0 bits 7:6; l1tt_high1 bits 9:8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101058UL: // (CDU_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCDU_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: eco reserved register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10105cUL: // (CDU_REG_CDU_ATC_FLAGS: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nCDU_REG_CDU_ATC_FLAGS (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATC_FLAGS_LD :"
				, (u32_t)((data & CDU_CDU_ATC_FLAGS_REG_ATC_FLAGS_LD) >> 3)
				, "\n\tATC_FLAGS_WB :"
				, (u32_t)((data & CDU_CDU_ATC_FLAGS_REG_ATC_FLAGS_WB) >> 0));
			break;
		case 0x101014UL: // (CDU_REG_ERROR_DATA: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCDU_REG_ERROR_DATA (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: logging of error data in case of a CDU load error: {expected_cid\n    [15:0];expected_type[2:0];expected_region[2:0];active_error;type_error\n    ;actual_active;actual_compressed_context}; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101100UL: // (CDU_REG_MATT: ACCESS:WB  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCDU_REG_MATT (ACCESS:WB  DataWidth:0x18 )" \
				"\nDescription: MATT ram access. each entry has the following format:{RegionLength\n    [11:0];RegionOffset[11:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101800UL: // (CDU_REG_L1TT: ACCESS:WB  DataWidth:0xd8 ) 
			sprintf(disp_str, "\nCDU_REG_L1TT (ACCESS:WB  DataWidth:0xd8 )" \
				"\nDescription: L1TT ram access. each entry has the following format : {mrege_regions\n    [7:0];offset12[5:0]...offset0[5:0];length12[5:0]...length0[5:0];id12[3\n    :0]...id0[3:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101060UL: // (CDU_REG_CDU_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCDU_REG_CDU_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101200UL: // (CDU_REG_CDU_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCDU_REG_CDU_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104000UL: // (CFC_REG_NUM_LCIDS_EMPTY: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCFC_REG_NUM_LCIDS_EMPTY (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Number of Empty LCIDs in Link List Block (not allocated)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104004UL: // (CFC_REG_NUM_LCIDS_ARRIVING: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCFC_REG_NUM_LCIDS_ARRIVING (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Number of Arriving LCIDs in Link List Block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104008UL: // (CFC_REG_NUM_LCIDS_INSIDE: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCFC_REG_NUM_LCIDS_INSIDE (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Number of Inside LCIDs in Link List Block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10400cUL: // (CFC_REG_NUM_LCIDS_INA: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCFC_REG_NUM_LCIDS_INA (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Number of Inside not active LCIDs in Link List Block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104010UL: // (CFC_REG_NUM_LCIDS_INAS: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCFC_REG_NUM_LCIDS_INAS (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Number of Inside not active latency sensitive LCIDs in Link\n     List Block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104014UL: // (CFC_REG_NUM_LCIDS_INANS: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCFC_REG_NUM_LCIDS_INANS (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Number of Inside not active not sensitive LCIDs in Link List\n     Block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104018UL: // (CFC_REG_NUM_LCIDS_LEAVING: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCFC_REG_NUM_LCIDS_LEAVING (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Number of Leaving LCIDs in Link List Block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10401cUL: // (CFC_REG_NUM_LCIDS_IO: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCFC_REG_NUM_LCIDS_IO (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Number of Inside and Outside LCIDs in Link List Block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104020UL: // (CFC_REG_NUM_LCIDS_ALLOC: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCFC_REG_NUM_LCIDS_ALLOC (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Number of allocated LCIDs which are at empty state\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104024UL: // (CFC_REG_LC_BLOCKED: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LC_BLOCKED (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: statistics register that counts cycles in which load context\n     requests were blocked\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104028UL: // (CFC_REG_CONTROL0: ACCESS:RW  DataWidth:0xf  Multi Field Register  ) 
			sprintf(disp_str, "\nCFC_REG_CONTROL0 (ACCESS:RW  DataWidth:0xf  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCCAM_SCRUB_MISS_EN :"
				, (u32_t)((data & CFC_CONTROL0_REG_CCAM_SCRUB_MISS_EN) >> 14)
				, "\n\tCCAM_SCRUB_HIT_EN :"
				, (u32_t)((data & CFC_CONTROL0_REG_CCAM_SCRUB_HIT_EN) >> 13)
				, "\n\tSCAM_SCRUB_MISS_EN :"
				, (u32_t)((data & CFC_CONTROL0_REG_SCAM_SCRUB_MISS_EN) >> 12)
				, "\n\tSCAM_SCRUB_HIT_EN :"
				, (u32_t)((data & CFC_CONTROL0_REG_SCAM_SCRUB_HIT_EN) >> 11)
				, "\n\tNLOE :"
				, (u32_t)((data & CFC_CONTROL0_REG_NLOE) >> 10)
				, "\n\tCID_CAM_DISABLE :"
				, (u32_t)((data & CFC_CONTROL0_REG_CID_CAM_DISABLE) >> 9)
				, "\n\tSTRING_CAM_DISABLE :"
				, (u32_t)((data & CFC_CONTROL0_REG_STRING_CAM_DISABLE) >> 8)
				, "\n\tWB_THRESHOLD :"
				, (u32_t)((data & CFC_CONTROL0_REG_WB_THRESHOLD) >> 0));
			break;
		case 0x10402cUL: // (CFC_REG_MASK_REQUESTS: ACCESS:RW  DataWidth:0x7  Multi Field Register  ) 
			sprintf(disp_str, "\nCFC_REG_MASK_REQUESTS (ACCESS:RW  DataWidth:0x7  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMASK_CDUWBRESP :"
				, (u32_t)((data & CFC_MASK_REQUESTS_REG_MASK_CDUWBRESP) >> 6)
				, "\n\tMASK_CDULDRESP :"
				, (u32_t)((data & CFC_MASK_REQUESTS_REG_MASK_CDULDRESP) >> 5)
				, "\n\tMASK_INACTIVATE :"
				, (u32_t)((data & CFC_MASK_REQUESTS_REG_MASK_INACTIVATE) >> 4)
				, "\n\tMASK_WB :"
				, (u32_t)((data & CFC_MASK_REQUESTS_REG_MASK_WB) >> 3)
				, "\n\tMASK_UPDATE :"
				, (u32_t)((data & CFC_MASK_REQUESTS_REG_MASK_UPDATE) >> 2)
				, "\n\tMASK_SEARCH :"
				, (u32_t)((data & CFC_MASK_REQUESTS_REG_MASK_SEARCH) >> 1)
				, "\n\tMASK_LCREQ :"
				, (u32_t)((data & CFC_MASK_REQUESTS_REG_MASK_LCREQ) >> 0));
			break;
		case 0x104030UL: // (CFC_REG_ERROR_DATA1: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nCFC_REG_ERROR_DATA1 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: when the CFC detects an internal error it updates these fields\n    . {error_id[4:0];req_incval[3:0];ac_error_client[3:0];ac_error_lcid;error_id\n    [3:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104034UL: // (CFC_REG_ERROR_DATA2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_ERROR_DATA2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: {req_cid;req_taskid[7:4];req_type}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104038UL: // (CFC_REG_ERROR_DATA3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_ERROR_DATA3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: when the CFC detects an internal error it updates these fields\n    . {req_regions;req_lcid;curr_lcid;ac_lcid}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10403cUL: // (CFC_REG_ERROR_VECTOR: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCFC_REG_ERROR_VECTOR (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: CFC error vector. when the CFC detects an internal error \n    it will set one of these bits. the bit description can be found in CFC\n     specifications\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104040UL: // (CFC_REG_ERROR_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCFC_REG_ERROR_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: masking for error logging. if a bit in this field is set \n    then the corresponding bit in #cfc_registers_cfc_error_vector.error_vector\n     will not be set\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104044UL: // (CFC_REG_DISABLE_ON_ERROR: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCFC_REG_DISABLE_ON_ERROR (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: indicates per error (in #cfc_registers_cfc_error_vector.cfc_error\n     vector) whether the cfc should be disabled upon it\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104048UL: // (CFC_REG_ARBITERS_REG: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nCFC_REG_ARBITERS_REG (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSP_AC_INC :"
				, (u32_t)((data & CFC_ARBITERS_REG_REG_SP_AC_INC) >> 5)
				, "\n\tSP_AC_DEC :"
				, (u32_t)((data & CFC_ARBITERS_REG_REG_SP_AC_DEC) >> 4)
				, "\n\tSP_MARB_RR3 :"
				, (u32_t)((data & CFC_ARBITERS_REG_REG_SP_MARB_RR3) >> 3)
				, "\n\tSP_MARB_RR2 :"
				, (u32_t)((data & CFC_ARBITERS_REG_REG_SP_MARB_RR2) >> 2)
				, "\n\tSP_LCARB :"
				, (u32_t)((data & CFC_ARBITERS_REG_REG_SP_LCARB) >> 1)
				, "\n\tSP_MARB_RR1 :"
				, (u32_t)((data & CFC_ARBITERS_REG_REG_SP_MARB_RR1) >> 0));
			break;
		case 0x10404cUL: // (CFC_REG_INIT_REG: ACCESS:RW  DataWidth:0xb  Multi Field Register  ) 
			sprintf(disp_str, "\nCFC_REG_INIT_REG (ACCESS:RW  DataWidth:0xb  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCAM_INIT :"
				, (u32_t)((data & CFC_INIT_REG_REG_CAM_INIT) >> 10)
				, "\n\tLL_INIT :"
				, (u32_t)((data & CFC_INIT_REG_REG_LL_INIT) >> 9)
				, "\n\tLL_INIT_LAST_LCID :"
				, (u32_t)((data & CFC_INIT_REG_REG_LL_INIT_LAST_LCID) >> 1)
				, "\n\tAC_INIT :"
				, (u32_t)((data & CFC_INIT_REG_REG_AC_INIT) >> 0));
			break;
		case 0x104050UL: // (CFC_REG_DEBUG0: ACCESS:RW  DataWidth:0x1e  Multi Field Register  ) 
			sprintf(disp_str, "\nCFC_REG_DEBUG0 (ACCESS:RW  DataWidth:0x1e  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tIO_THRESHOLD :"
				, (u32_t)((data & CFC_DEBUG0_REG_IO_THRESHOLD) >> 27)
				, "\n\tINAS_THRESHOLD :"
				, (u32_t)((data & CFC_DEBUG0_REG_INAS_THRESHOLD) >> 24)
				, "\n\tINANS_THRESHOLD :"
				, (u32_t)((data & CFC_DEBUG0_REG_INANS_THRESHOLD) >> 21)
				, "\n\tINA_THRESHOLD :"
				, (u32_t)((data & CFC_DEBUG0_REG_INA_THRESHOLD) >> 18)
				, "\n\tE_THRESHOLD :"
				, (u32_t)((data & CFC_DEBUG0_REG_E_THRESHOLD) >> 15)
				, "\n\tAC_GRANT_PERIOD :"
				, (u32_t)((data & CFC_DEBUG0_REG_AC_GRANT_PERIOD) >> 11)
				, "\n\tDELAY_CAM_RESP :"
				, (u32_t)((data & CFC_DEBUG0_REG_DELAY_CAM_RESP) >> 10)
				, "\n\tAC_COUNTER_ZERO :"
				, (u32_t)((data & CFC_DEBUG0_REG_AC_COUNTER_ZERO) >> 2)
				, "\n\tDISABLE_OUTPUTS :"
				, (u32_t)((data & CFC_DEBUG0_REG_DISABLE_OUTPUTS) >> 1)
				, "\n\tDISABLE_INPUTS :"
				, (u32_t)((data & CFC_DEBUG0_REG_DISABLE_INPUTS) >> 0));
			break;
		case 0x104054UL: // (CFC_REG_DEBUG1: ACCESS:RW  DataWidth:0xd  Multi Field Register  ) 
			sprintf(disp_str, "\nCFC_REG_DEBUG1 (ACCESS:RW  DataWidth:0xd  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUPD_CANCEL_DIS :"
				, (u32_t)((data & CFC_DEBUG1_REG_UPD_CANCEL_DIS) >> 12)
				, "\n\tEN_ON_INT_CLR :"
				, (u32_t)((data & CFC_DEBUG1_REG_EN_ON_INT_CLR) >> 11)
				, "\n\tSW_RESET :"
				, (u32_t)((data & CFC_DEBUG1_REG_SW_RESET) >> 10)
				, "\n\tTYPE_FROM_REQ2 :"
				, (u32_t)((data & CFC_DEBUG1_REG_TYPE_FROM_REQ2) >> 9)
				, "\n\tTYPE_FROM_REQ1 :"
				, (u32_t)((data & CFC_DEBUG1_REG_TYPE_FROM_REQ1) >> 8)
				, "\n\tWVAL_AC :"
				, (u32_t)((data & CFC_DEBUG1_REG_WVAL_AC) >> 6)
				, "\n\tMY_VAL_AC :"
				, (u32_t)((data & CFC_DEBUG1_REG_MY_VAL_AC) >> 5)
				, "\n\tWRITE_AC :"
				, (u32_t)((data & CFC_DEBUG1_REG_WRITE_AC) >> 4)
				, "\n\tMARB_THRESHOLD :"
				, (u32_t)((data & CFC_DEBUG1_REG_MARB_THRESHOLD) >> 0));
			break;
		case 0x104058UL: // (CFC_REG_INTERFACES: ACCESS:RW  DataWidth:0x1a  Multi Field Register  ) 
			sprintf(disp_str, "\nCFC_REG_INTERFACES (ACCESS:RW  DataWidth:0x1a  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLRESP6_CREDIT_SET :"
				, (u32_t)((data & CFC_INTERFACES_REG_LRESP6_CREDIT_SET) >> 25)
				, "\n\tLRESP6_CREDIT :"
				, (u32_t)((data & CFC_INTERFACES_REG_LRESP6_CREDIT) >> 22)
				, "\n\tCDUWB_CREDIT_SET :"
				, (u32_t)((data & CFC_INTERFACES_REG_CDUWB_CREDIT_SET) >> 21)
				, "\n\tCDUWB_CREDIT :"
				, (u32_t)((data & CFC_INTERFACES_REG_CDUWB_CREDIT) >> 17)
				, "\n\tCDULD_CREDIT_SET :"
				, (u32_t)((data & CFC_INTERFACES_REG_CDULD_CREDIT_SET) >> 16)
				, "\n\tCDULD_CREDIT :"
				, (u32_t)((data & CFC_INTERFACES_REG_CDULD_CREDIT) >> 12)
				, "\n\tSEARCH_CREDIT_SET :"
				, (u32_t)((data & CFC_INTERFACES_REG_SEARCH_CREDIT_SET) >> 11)
				, "\n\tSEARCH_CREDIT :"
				, (u32_t)((data & CFC_INTERFACES_REG_SEARCH_CREDIT) >> 8)
				, "\n\tPRSRESP_CREDIT_SET :"
				, (u32_t)((data & CFC_INTERFACES_REG_PRSRESP_CREDIT_SET) >> 7)
				, "\n\tPRSRESP_CREDIT :"
				, (u32_t)((data & CFC_INTERFACES_REG_PRSRESP_CREDIT) >> 4)
				, "\n\tLRESP_CREDIT_SET :"
				, (u32_t)((data & CFC_INTERFACES_REG_LRESP_CREDIT_SET) >> 3)
				, "\n\tLRESP_CREDIT :"
				, (u32_t)((data & CFC_INTERFACES_REG_LRESP_CREDIT) >> 0));
			break;
		case 0x10405cUL: // (CFC_REG_CID_CAM_CONTROL: ACCESS:RW  DataWidth:0x16  Multi Field Register  ) 
			sprintf(disp_str, "\nCFC_REG_CID_CAM_CONTROL (ACCESS:RW  DataWidth:0x16  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCAM_125MHZ :"
				, (u32_t)((data & CFC_CID_CAM_CONTROL_REG_CAM_125MHZ) >> 21)
				, "\n\tCCAM_SEARCH :"
				, (u32_t)((data & CFC_CID_CAM_CONTROL_REG_CCAM_SEARCH) >> 20)
				, "\n\tCCAM_MASK_VECTOR :"
				, (u32_t)((data & CFC_CID_CAM_CONTROL_REG_CCAM_MASK_VECTOR) >> 0));
			break;
		case 0x104060UL: // (CFC_REG_SCAM_MASK_VECTOR0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_MASK_VECTOR0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: string cam mask[31:0]. this mask is used for searches. 0 \n    means ignore corresponding bit in search. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104064UL: // (CFC_REG_SCAM_MASK_VECTOR1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_MASK_VECTOR1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: string cam mask[63:32]. this mask is used for searches. 0\n     means ignore corresponding bit in search. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104068UL: // (CFC_REG_SCAM_MASK_VECTOR2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_MASK_VECTOR2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: string cam mask[95:64]. this mask is used for searches. 0\n     means ignore corresponding bit in search. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10406cUL: // (CFC_REG_SCAM_SEARCH: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_SEARCH (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: when this bit is set writing to the scam will cause a search\n     operation on the written item (written using ~cfc_registers_lcid_string_cam\n    .string_cam interface. the write can be to any address\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104070UL: // (CFC_REG_SEARCH_RESULT: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nCFC_REG_SEARCH_RESULT (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: {HIT;LCID}. HIT - if set then previous CAM seach item (either\n     CCAM or SCAM) was found. LCID contains the result in case CAM search \n    item (either CCAM or SCAM) was found\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104074UL: // (CFC_REG_LL_INIT_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_LL_INIT_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: indication the initializing the link list by the hardware\n     was done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104078UL: // (CFC_REG_AC_INIT_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_AC_INIT_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: indication the initializing the activity counter by the hardware\n     was done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10407cUL: // (CFC_REG_CAM_INIT_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_CAM_INIT_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: indication the initializing the cams by the hardware was \n    done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104080UL: // (CFC_REG_TASK_COUNTER: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_TASK_COUNTER (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts number of tasks done by the cfc\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104084UL: // (CFC_REG_LCREQ_WEIGHTS: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCFC_REG_LCREQ_WEIGHTS (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: {weight_load_client7[2:0] to weight_load_client0[2:0]}. this\n     field allows changing the priorities of the weighted-round-robin arbiter\n     which selects which CFC load client should be served next\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104088UL: // (CFC_REG_LOAD_CONTEXT_HITS_0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_HITS_0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context hits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10408cUL: // (CFC_REG_LOAD_CONTEXT_HITS_1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_HITS_1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context hits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104090UL: // (CFC_REG_LOAD_CONTEXT_HITS_2: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_HITS_2 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context hits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104094UL: // (CFC_REG_LOAD_CONTEXT_HITS_3: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_HITS_3 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context hits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104098UL: // (CFC_REG_LOAD_CONTEXT_HITS_4: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_HITS_4 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context hits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10409cUL: // (CFC_REG_LOAD_CONTEXT_HITS_5: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_HITS_5 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context hits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040a0UL: // (CFC_REG_LOAD_CONTEXT_HITS_6: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_HITS_6 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context hits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040a4UL: // (CFC_REG_LOAD_CONTEXT_HITS_7: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_HITS_7 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context hits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040a8UL: // (CFC_REG_LOAD_CONTEXT_MISSES_0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_MISSES_0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context misses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040acUL: // (CFC_REG_LOAD_CONTEXT_MISSES_1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_MISSES_1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context misses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040b0UL: // (CFC_REG_LOAD_CONTEXT_MISSES_2: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_MISSES_2 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context misses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040b4UL: // (CFC_REG_LOAD_CONTEXT_MISSES_3: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_MISSES_3 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context misses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040b8UL: // (CFC_REG_LOAD_CONTEXT_MISSES_4: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_MISSES_4 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context misses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040bcUL: // (CFC_REG_LOAD_CONTEXT_MISSES_5: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_MISSES_5 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context misses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040c0UL: // (CFC_REG_LOAD_CONTEXT_MISSES_6: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_MISSES_6 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context misses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040c4UL: // (CFC_REG_LOAD_CONTEXT_MISSES_7: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_LOAD_CONTEXT_MISSES_7 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts for each client the number of load context misses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040c8UL: // (CFC_REG_RFE_SEARCH_HITS: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_RFE_SEARCH_HITS (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts the number of RFE serach hits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040ccUL: // (CFC_REG_RFE_SEARCH_MISSES: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_RFE_SEARCH_MISSES (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts the number of RFE serach misses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040d0UL: // (CFC_REG_CDU_WRITE_BACKS: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_CDU_WRITE_BACKS (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: counts the number of CDU write backs submitted by CFC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040d4UL: // (CFC_REG_CID_CAM_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nCFC_REG_CID_CAM_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM bits of  cid cam memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040d8UL: // (CFC_REG_STRING_CAM_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nCFC_REG_STRING_CAM_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM bits of string cam memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040dcUL: // (CFC_REG_LRAM_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCFC_REG_LRAM_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of LCID ram memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040e0UL: // (CFC_REG_LL1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCFC_REG_LL1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of LinkList1 memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040e4UL: // (CFC_REG_LL2_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCFC_REG_LL2_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of LinkList2 memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040e8UL: // (CFC_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCFC_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: For dbgmux usage (debug data that goes from CFC to the DBG\n     block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040ecUL: // (CFC_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCFC_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: For dbgmux usage (debug data that goes from CFC to the DBG\n     block) - for enabling bytes in the selected line (after the select and\n     before the shift)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040f0UL: // (CFC_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCFC_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: For dbgmux usage (debug data that goes from CFC to the DBG\n     block) - for circular right shifting of the selected line (after the \n    enabling)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040f4UL: // (CFC_REG_CCAM_BIST_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_CCAM_BIST_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CID CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040f8UL: // (CFC_REG_SCAM_BIST_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_BIST_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: STRING CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040fcUL: // (CFC_REG_CFC_INT_STS: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCFC_REG_CFC_INT_STS (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEXE_ERROR :"
				, (u32_t)((data & CFC_CFC_INT_STS_REG_EXE_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CFC_CFC_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x104100UL: // (CFC_REG_CFC_INT_STS_CLR: ACCESS:RC  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCFC_REG_CFC_INT_STS_CLR (ACCESS:RC  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEXE_ERROR :"
				, (u32_t)((data & CFC_CFC_INT_STS_CLR_REG_EXE_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CFC_CFC_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x104104UL: // (CFC_REG_CFC_INT_STS_WR: ACCESS:WR  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCFC_REG_CFC_INT_STS_WR (ACCESS:WR  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEXE_ERROR :"
				, (u32_t)((data & CFC_CFC_INT_STS_WR_REG_EXE_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CFC_CFC_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x104108UL: // (CFC_REG_CFC_INT_MASK: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCFC_REG_CFC_INT_MASK (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEXE_ERROR :"
				, (u32_t)((data & CFC_CFC_INT_MASK_REG_EXE_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CFC_CFC_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x10410cUL: // (CFC_REG_CFC_PRTY_STS: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCFC_REG_CFC_PRTY_STS (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSCAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_REG_SCAM_PAR_ERR) >> 5)
				, "\n\tCCAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_REG_CCAM_PAR_ERR) >> 4)
				, "\n\tAC_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_REG_AC_PAR_ERR) >> 3)
				, "\n\tINFORAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_REG_INFORAM_PAR_ERR) >> 2)
				, "\n\tLL_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_REG_LL_PAR_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x104110UL: // (CFC_REG_CFC_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCFC_REG_CFC_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x6 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSCAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_CLR_REG_SCAM_PAR_ERR) >> 5)
				, "\n\tCCAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_CLR_REG_CCAM_PAR_ERR) >> 4)
				, "\n\tAC_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_CLR_REG_AC_PAR_ERR) >> 3)
				, "\n\tINFORAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_CLR_REG_INFORAM_PAR_ERR) >> 2)
				, "\n\tLL_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_CLR_REG_LL_PAR_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x104114UL: // (CFC_REG_CFC_PRTY_STS_WR: ACCESS:WR  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCFC_REG_CFC_PRTY_STS_WR (ACCESS:WR  DataWidth:0x6 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSCAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_WR_REG_SCAM_PAR_ERR) >> 5)
				, "\n\tCCAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_WR_REG_CCAM_PAR_ERR) >> 4)
				, "\n\tAC_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_WR_REG_AC_PAR_ERR) >> 3)
				, "\n\tINFORAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_WR_REG_INFORAM_PAR_ERR) >> 2)
				, "\n\tLL_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_WR_REG_LL_PAR_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CFC_CFC_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x104118UL: // (CFC_REG_CFC_PRTY_MASK: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCFC_REG_CFC_PRTY_MASK (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSCAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_MASK_REG_SCAM_PAR_ERR) >> 5)
				, "\n\tCCAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_MASK_REG_CCAM_PAR_ERR) >> 4)
				, "\n\tAC_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_MASK_REG_AC_PAR_ERR) >> 3)
				, "\n\tINFORAM_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_MASK_REG_INFORAM_PAR_ERR) >> 2)
				, "\n\tLL_PAR_ERR :"
				, (u32_t)((data & CFC_CFC_PRTY_MASK_REG_LL_PAR_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CFC_CFC_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x10412cUL: // (CFC_REG_EXEC_ERROR_PF: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCFC_REG_EXEC_ERROR_PF (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Debug Only. The PF that generated the last execution error\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107000UL: // (CFC_REG_TM_AC: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCFC_REG_TM_AC (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: tm bits for ram_256x14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107004UL: // (CFC_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCFC_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: eco reserved.  bit0: when set use new equation for last_free_lcid\n     using threshold values. Bit1: when clear (default) use the LL_IO pop \n    PFID to recover the correct Not-Empty PF counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107008UL: // (CFC_REG_TM_LL_RAM_PREV: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCFC_REG_TM_LL_RAM_PREV (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: tm bits for ll_ram_prev\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10700cUL: // (CFC_REG_TM_LL_RAM_NEXT: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCFC_REG_TM_LL_RAM_NEXT (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: tm bits for rll_ram_next\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107010UL: // (CFC_REG_DORQ_MASK_VALERR: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_DORQ_MASK_VALERR (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates if DORQ Validation Error (CDU Error#2) is masked\n     (independent of cfc_error_mask)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107014UL: // (CFC_REG_DORQ_MASK_PCIERR: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_DORQ_MASK_PCIERR (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates if DORQ PCIe Error (CDU Error#1) is masked (independent\n     of cfc_error_mask)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107018UL: // (CFC_REG_SCAM_MASK_VECTOR3: ACCESS:RW  DataWidth:0xf ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_MASK_VECTOR3 (ACCESS:RW  DataWidth:0xf )" \
				"\nDescription: string cam mask[110:96]. this mask is used for searches. \n    0 means ignore corresponding bit in search. (110:99 - Vlan; 98:96-pfid\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10701cUL: // (CFC_REG_INTERFACES2: ACCESS:RW  DataWidth:0xb  Multi Field Register  ) 
			sprintf(disp_str, "\nCFC_REG_INTERFACES2 (ACCESS:RW  DataWidth:0xb  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCDULD_CREDIT_SET_E3 :"
				, (u32_t)((data & CFC_INTERFACES2_REG_CDULD_CREDIT_SET_E3) >> 10)
				, "\n\tCDULD_CREDIT_E3 :"
				, (u32_t)((data & CFC_INTERFACES2_REG_CDULD_CREDIT_E3) >> 5)
				, "\n\tLRESP8_CREDIT_SET :"
				, (u32_t)((data & CFC_INTERFACES2_REG_LRESP8_CREDIT_SET) >> 4)
				, "\n\tLRESP8_CREDIT :"
				, (u32_t)((data & CFC_INTERFACES2_REG_LRESP8_CREDIT) >> 0));
			break;
		case 0x10411cUL: // (CFC_REG_NEMPTY_SIZE_PF: ACCESS:RW  DataWidth:0x9  SPLIT:8 ) 
			sprintf(disp_str, "\nCFC_REG_NEMPTY_SIZE_PF (ACCESS:RW  DataWidth:0x9  SPLIT:8 )" \
				"\nDescription: the size of the Not Empty Link List is set accordingly per\n     PF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104120UL: // (CFC_REG_NUM_LCIDS_INSIDE_PF: ACCESS:RW  DataWidth:0x9  SPLIT:8 ) 
			sprintf(disp_str, "\nCFC_REG_NUM_LCIDS_INSIDE_PF (ACCESS:RW  DataWidth:0x9  SPLIT:8 )" \
				"\nDescription: Number of Inside LCIDs in Link List Block per PF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104124UL: // (CFC_REG_WEAK_ENABLE_PF: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nCFC_REG_WEAK_ENABLE_PF (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: This bit when clear will cause a load-cancel response to \n    a load request for PF and set an execution error. Set processes load requests\n     normally.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104128UL: // (CFC_REG_STRONG_ENABLE_PF: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nCFC_REG_STRONG_ENABLE_PF (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: This bit when clear will cause a CFC execution error (weak_enable\n     will override to force load-cancel) to a search or load request for PF\n    . The PFID that caused the execution error will be stored (exec_error_pf\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104130UL: // (CFC_REG_SREQ_FULL_STICKY: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_SREQ_FULL_STICKY (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The Interface to Searcher Request Queue has reached the maximum\n     value (4)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104134UL: // (CFC_REG_PRSRESP_FULL_STICKY: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_PRSRESP_FULL_STICKY (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The Interface to Parser Response Queue has reached the maximum\n     value (6)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104138UL: // (CFC_REG_DISABLE_ROBUSTWB_PF: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nCFC_REG_DISABLE_ROBUSTWB_PF (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Disable Robust WB change: When an inactivate request is processed\n     do not move the LCID to Inactive state if any of the regions are in error\n     state\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10413cUL: // (CFC_REG_CCAM_BIST_DBG_SEL: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCFC_REG_CCAM_BIST_DBG_SEL (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: This registers selects the type of data present on bist_status\n     bus \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104140UL: // (CFC_REG_CCAM_BIST_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_CCAM_BIST_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bist status from CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104144UL: // (CFC_REG_SCAM_BIST_DBG_SEL: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_BIST_DBG_SEL (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: This registers selects the type of data present on bist_status\n     bus \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104148UL: // (CFC_REG_SCAM_BIST_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_BIST_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bist status from CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10414cUL: // (CFC_REG_DBG_OUT_DATA_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_DBG_OUT_DATA_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from CFC\n     to the DBG block) - The 32 lsb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104150UL: // (CFC_REG_DBG_OUT_DATA_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_DBG_OUT_DATA_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from CFC\n     to the DBG block) - The 32 msb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104154UL: // (CFC_REG_DBG_OUT_FRAME: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCFC_REG_DBG_OUT_FRAME (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from CFC\n     to the DBG block) - The 4 frame bits that goes to the DBG block. Bit0\n     is the frame of data byte0; Bit1 is the frame of  data byte1; Bit2 is\n     the frame of data byte2; Bit3 is the frame of  data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104158UL: // (CFC_REG_DBG_OUT_VALID: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCFC_REG_DBG_OUT_VALID (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from CFC\n     to the DBG block) - The 4 valid bits that goes to the DBG block. Bit0\n     validtes data byte0; Bit1 validates data byte1; Bit2 validates data byte2\n    ; Bit3 validates data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104200UL: // (CFC_REG_CCAM_BIST_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_CCAM_BIST_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: CID CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104204UL: // (CFC_REG_CCAM_BIST_GO: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_CCAM_BIST_GO (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: CID CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104208UL: // (CFC_REG_CCAM_S2_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_CCAM_S2_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: CID CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10420cUL: // (CFC_REG_CCAM_S3_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_CCAM_S3_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: CID CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104210UL: // (CFC_REG_CCAM_S5_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_CCAM_S5_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: CID CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104214UL: // (CFC_REG_CCAM_S6_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_CCAM_S6_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: CID CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104218UL: // (CFC_REG_CCAM_S8_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_CCAM_S8_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: CID CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10421cUL: // (CFC_REG_EMPTY_HEAD: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nCFC_REG_EMPTY_HEAD (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: the head of the empty Link List is set accordingly  pfid;\n    head\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104224UL: // (CFC_REG_EMPTY_TAIL: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nCFC_REG_EMPTY_TAIL (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: the tail of the empty Link List is set accordingly  pfid;\n    tail\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104228UL: // (CFC_REG_SCAM_BIST_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_BIST_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: STRING CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10422cUL: // (CFC_REG_SCAM_BIST_GO: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_BIST_GO (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: STRING CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104230UL: // (CFC_REG_SCAM_S2_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_S2_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: STRING CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104234UL: // (CFC_REG_SCAM_S3_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_S3_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: STRING CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104238UL: // (CFC_REG_SCAM_S5_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_S5_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: STRING CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10423cUL: // (CFC_REG_SCAM_S6_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_S6_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: STRING CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104240UL: // (CFC_REG_SCAM_S8_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCFC_REG_SCAM_S8_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: STRING CAM BIST\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104400UL: // (CFC_REG_ACTIVITY_COUNTER: ACCESS:RW  DataWidth:0xd ) 
			sprintf(disp_str, "\nCFC_REG_ACTIVITY_COUNTER (ACCESS:RW  DataWidth:0xd )" \
				"\nDescription: activity counter ram access\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104800UL: // (CFC_REG_CID_CAM: ACCESS:RW  DataWidth:0x15 ) 
			sprintf(disp_str, "\nCFC_REG_CID_CAM (ACCESS:RW  DataWidth:0x15 )" \
				"\nDescription: CID cam  access (21:1 - Data;Valid - 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104c00UL: // (CFC_REG_LINK_LIST: ACCESS:RW  DataWidth:0x16 ) 
			sprintf(disp_str, "\nCFC_REG_LINK_LIST (ACCESS:RW  DataWidth:0x16 )" \
				"\nDescription: Link List ram access; data = {prev_pfid;prev_lcid;next_pfid\n    ;next_lcid}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105000UL: // (CFC_REG_INFO_RAM: ACCESS:WB  DataWidth:0x61 ) 
			sprintf(disp_str, "\nCFC_REG_INFO_RAM (ACCESS:WB  DataWidth:0x61 )" \
				"\nDescription: LCID info ram  access = {96-vpf;95:93-pfid;92:89-type;88:\n    85-action;84-paddrv;83:20-paddr;19:4-rstates;3-lsf;2:0-lstate}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106000UL: // (CFC_REG_STRING_CAM: ACCESS:WB  DataWidth:0x70 ) 
			sprintf(disp_str, "\nCFC_REG_STRING_CAM (ACCESS:WB  DataWidth:0x70 )" \
				"\nDescription: String cam access (111:100 - Vlan; 99:97-pfid; 96:1 - Data\n    ; 0 - Valid)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10415cUL: // (CFC_REG_CFC_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCFC_REG_CFC_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104244UL: // (CFC_REG_CFC_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCFC_REG_CFC_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107020UL: // (CFC_REG_CFC_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCFC_REG_CFC_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2000UL: // (CSDM_REG_TIMER_TICK: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_TIMER_TICK (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Tick for timer counter. Applicable only when ~csdm_registers_timer_tick_enable\n    .timer_tick_enable =1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2004UL: // (CSDM_REG_TIMERS_TICK_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_TIMERS_TICK_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for tick counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2008UL: // (CSDM_REG_CFC_RSP_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nCSDM_REG_CFC_RSP_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the  internal RAM for the cfc_rsp lcid\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc200cUL: // (CSDM_REG_CMP_COUNTER_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nCSDM_REG_CMP_COUNTER_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal  RAM for the completion\n     counters.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2010UL: // (CSDM_REG_Q_COUNTER_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nCSDM_REG_Q_COUNTER_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal RAM for queue counters\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2014UL: // (CSDM_REG_PCK_END_MSG_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nCSDM_REG_PCK_END_MSG_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal RAM for the packet end \n    message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2018UL: // (CSDM_REG_COUNTERS_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_COUNTERS_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates if the 204 statistics counters should stop counting\n     when reaching an all-ones value or should wrap-around 0=stop counting\n     1=wrap-around.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc201cUL: // (CSDM_REG_CMP_COUNTER_MAX0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCSDM_REG_CMP_COUNTER_MAX0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2020UL: // (CSDM_REG_CMP_COUNTER_MAX1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCSDM_REG_CMP_COUNTER_MAX1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2024UL: // (CSDM_REG_CMP_COUNTER_MAX2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCSDM_REG_CMP_COUNTER_MAX2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2028UL: // (CSDM_REG_CMP_COUNTER_MAX3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCSDM_REG_CMP_COUNTER_MAX3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc202cUL: // (CSDM_REG_BRB1_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCSDM_REG_BRB1_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from BRB1 in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2030UL: // (CSDM_REG_PXP_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCSDM_REG_PXP_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from pxp in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2034UL: // (CSDM_REG_PB_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCSDM_REG_PB_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from PB in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2038UL: // (CSDM_REG_AGG_INT_EVENT_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc203cUL: // (CSDM_REG_AGG_INT_EVENT_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2040UL: // (CSDM_REG_AGG_INT_EVENT_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2044UL: // (CSDM_REG_AGG_INT_EVENT_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2048UL: // (CSDM_REG_AGG_INT_EVENT_4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc204cUL: // (CSDM_REG_AGG_INT_EVENT_5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2050UL: // (CSDM_REG_AGG_INT_EVENT_6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2054UL: // (CSDM_REG_AGG_INT_EVENT_7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2058UL: // (CSDM_REG_AGG_INT_EVENT_8: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_8 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc205cUL: // (CSDM_REG_AGG_INT_EVENT_9: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_9 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2060UL: // (CSDM_REG_AGG_INT_EVENT_10: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_10 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2064UL: // (CSDM_REG_AGG_INT_EVENT_11: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_11 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2068UL: // (CSDM_REG_AGG_INT_EVENT_12: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_12 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc206cUL: // (CSDM_REG_AGG_INT_EVENT_13: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_13 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2070UL: // (CSDM_REG_AGG_INT_EVENT_14: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_14 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2074UL: // (CSDM_REG_AGG_INT_EVENT_15: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_15 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2078UL: // (CSDM_REG_AGG_INT_EVENT_16: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_16 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc207cUL: // (CSDM_REG_AGG_INT_EVENT_17: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_17 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2080UL: // (CSDM_REG_AGG_INT_EVENT_18: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_18 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2084UL: // (CSDM_REG_AGG_INT_EVENT_19: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_19 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2088UL: // (CSDM_REG_AGG_INT_EVENT_20: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_20 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc208cUL: // (CSDM_REG_AGG_INT_EVENT_21: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_21 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2090UL: // (CSDM_REG_AGG_INT_EVENT_22: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_22 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2094UL: // (CSDM_REG_AGG_INT_EVENT_23: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_23 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2098UL: // (CSDM_REG_AGG_INT_EVENT_24: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_24 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc209cUL: // (CSDM_REG_AGG_INT_EVENT_25: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_25 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20a0UL: // (CSDM_REG_AGG_INT_EVENT_26: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_26 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20a4UL: // (CSDM_REG_AGG_INT_EVENT_27: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_27 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20a8UL: // (CSDM_REG_AGG_INT_EVENT_28: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_28 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20acUL: // (CSDM_REG_AGG_INT_EVENT_29: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_29 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20b0UL: // (CSDM_REG_AGG_INT_EVENT_30: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_30 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20b4UL: // (CSDM_REG_AGG_INT_EVENT_31: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_EVENT_31 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20b8UL: // (CSDM_REG_AGG_INT_T_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20bcUL: // (CSDM_REG_AGG_INT_T_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20c0UL: // (CSDM_REG_AGG_INT_T_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20c4UL: // (CSDM_REG_AGG_INT_T_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20c8UL: // (CSDM_REG_AGG_INT_T_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20ccUL: // (CSDM_REG_AGG_INT_T_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20d0UL: // (CSDM_REG_AGG_INT_T_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20d4UL: // (CSDM_REG_AGG_INT_T_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20d8UL: // (CSDM_REG_AGG_INT_T_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20dcUL: // (CSDM_REG_AGG_INT_T_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20e0UL: // (CSDM_REG_AGG_INT_T_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20e4UL: // (CSDM_REG_AGG_INT_T_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20e8UL: // (CSDM_REG_AGG_INT_T_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20ecUL: // (CSDM_REG_AGG_INT_T_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20f0UL: // (CSDM_REG_AGG_INT_T_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20f4UL: // (CSDM_REG_AGG_INT_T_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20f8UL: // (CSDM_REG_AGG_INT_T_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc20fcUL: // (CSDM_REG_AGG_INT_T_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2100UL: // (CSDM_REG_AGG_INT_T_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2104UL: // (CSDM_REG_AGG_INT_T_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2108UL: // (CSDM_REG_AGG_INT_T_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc210cUL: // (CSDM_REG_AGG_INT_T_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2110UL: // (CSDM_REG_AGG_INT_T_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2114UL: // (CSDM_REG_AGG_INT_T_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2118UL: // (CSDM_REG_AGG_INT_T_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc211cUL: // (CSDM_REG_AGG_INT_T_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2120UL: // (CSDM_REG_AGG_INT_T_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2124UL: // (CSDM_REG_AGG_INT_T_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2128UL: // (CSDM_REG_AGG_INT_T_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc212cUL: // (CSDM_REG_AGG_INT_T_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2130UL: // (CSDM_REG_AGG_INT_T_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2134UL: // (CSDM_REG_AGG_INT_T_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_T_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2138UL: // (CSDM_REG_AGG_INT_FIC_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc213cUL: // (CSDM_REG_AGG_INT_FIC_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2140UL: // (CSDM_REG_AGG_INT_FIC_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2144UL: // (CSDM_REG_AGG_INT_FIC_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2148UL: // (CSDM_REG_AGG_INT_FIC_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc214cUL: // (CSDM_REG_AGG_INT_FIC_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2150UL: // (CSDM_REG_AGG_INT_FIC_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2154UL: // (CSDM_REG_AGG_INT_FIC_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2158UL: // (CSDM_REG_AGG_INT_FIC_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc215cUL: // (CSDM_REG_AGG_INT_FIC_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2160UL: // (CSDM_REG_AGG_INT_FIC_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2164UL: // (CSDM_REG_AGG_INT_FIC_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2168UL: // (CSDM_REG_AGG_INT_FIC_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc216cUL: // (CSDM_REG_AGG_INT_FIC_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2170UL: // (CSDM_REG_AGG_INT_FIC_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2174UL: // (CSDM_REG_AGG_INT_FIC_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2178UL: // (CSDM_REG_AGG_INT_FIC_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc217cUL: // (CSDM_REG_AGG_INT_FIC_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2180UL: // (CSDM_REG_AGG_INT_FIC_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2184UL: // (CSDM_REG_AGG_INT_FIC_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2188UL: // (CSDM_REG_AGG_INT_FIC_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc218cUL: // (CSDM_REG_AGG_INT_FIC_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2190UL: // (CSDM_REG_AGG_INT_FIC_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2194UL: // (CSDM_REG_AGG_INT_FIC_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2198UL: // (CSDM_REG_AGG_INT_FIC_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc219cUL: // (CSDM_REG_AGG_INT_FIC_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21a0UL: // (CSDM_REG_AGG_INT_FIC_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21a4UL: // (CSDM_REG_AGG_INT_FIC_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21a8UL: // (CSDM_REG_AGG_INT_FIC_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21acUL: // (CSDM_REG_AGG_INT_FIC_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21b0UL: // (CSDM_REG_AGG_INT_FIC_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21b4UL: // (CSDM_REG_AGG_INT_FIC_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_FIC_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21b8UL: // (CSDM_REG_AGG_INT_MODE_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21bcUL: // (CSDM_REG_AGG_INT_MODE_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21c0UL: // (CSDM_REG_AGG_INT_MODE_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21c4UL: // (CSDM_REG_AGG_INT_MODE_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21c8UL: // (CSDM_REG_AGG_INT_MODE_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21ccUL: // (CSDM_REG_AGG_INT_MODE_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21d0UL: // (CSDM_REG_AGG_INT_MODE_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21d4UL: // (CSDM_REG_AGG_INT_MODE_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21d8UL: // (CSDM_REG_AGG_INT_MODE_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21dcUL: // (CSDM_REG_AGG_INT_MODE_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21e0UL: // (CSDM_REG_AGG_INT_MODE_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21e4UL: // (CSDM_REG_AGG_INT_MODE_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21e8UL: // (CSDM_REG_AGG_INT_MODE_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21ecUL: // (CSDM_REG_AGG_INT_MODE_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21f0UL: // (CSDM_REG_AGG_INT_MODE_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21f4UL: // (CSDM_REG_AGG_INT_MODE_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21f8UL: // (CSDM_REG_AGG_INT_MODE_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (16) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc21fcUL: // (CSDM_REG_AGG_INT_MODE_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (17)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2200UL: // (CSDM_REG_AGG_INT_MODE_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2204UL: // (CSDM_REG_AGG_INT_MODE_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2208UL: // (CSDM_REG_AGG_INT_MODE_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc220cUL: // (CSDM_REG_AGG_INT_MODE_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2210UL: // (CSDM_REG_AGG_INT_MODE_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2214UL: // (CSDM_REG_AGG_INT_MODE_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2218UL: // (CSDM_REG_AGG_INT_MODE_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc221cUL: // (CSDM_REG_AGG_INT_MODE_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2220UL: // (CSDM_REG_AGG_INT_MODE_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2224UL: // (CSDM_REG_AGG_INT_MODE_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2228UL: // (CSDM_REG_AGG_INT_MODE_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc222cUL: // (CSDM_REG_AGG_INT_MODE_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2230UL: // (CSDM_REG_AGG_INT_MODE_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2234UL: // (CSDM_REG_AGG_INT_MODE_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_AGG_INT_MODE_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2238UL: // (CSDM_REG_ENABLE_IN1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nCSDM_REG_ENABLE_IN1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tQM_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_QM_EXT_WR_FULL_IN_EN) >> 26)
				, "\n\tCFC_PB_ACK_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_CFC_PB_ACK_IN_EN) >> 25)
				, "\n\tCFC_ACDEC_ACK_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_CFC_ACDEC_ACK_IN_EN) >> 24)
				, "\n\tCFC_ACINC_ACK_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_CFC_ACINC_ACK_IN_EN) >> 23)
				, "\n\tCFC_LOAD_RSP_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_CFC_LOAD_RSP_IN_EN) >> 22)
				, "\n\tCFC_LOAD_ACK_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_CFC_LOAD_ACK_IN_EN) >> 21)
				, "\n\tPXP_REQ_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PXP_REQ_IN_EN) >> 20)
				, "\n\tSDM_WAKE_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_SDM_WAKE_IN_EN) >> 19)
				, "\n\tPRS_MSG_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PRS_MSG_IN_EN) >> 18)
				, "\n\tPB_DATA_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PB_DATA_IN_EN) >> 17)
				, "\n\tBRB1_DP_DATA_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_BRB1_DP_DATA_IN_EN) >> 16)
				, "\n\tBRB1_CTRL_DATA_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_BRB1_CTRL_DATA_IN_EN) >> 15)
				, "\n\tPXP_DP_ACK_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PXP_DP_ACK_IN_EN) >> 14)
				, "\n\tPXP_CTRL_ACK_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PXP_CTRL_ACK_IN_EN) >> 13)
				, "\n\tPXP_DP_DATA_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PXP_DP_DATA_IN_EN) >> 12)
				, "\n\tPXP_INT_DONE_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PXP_INT_DONE_IN_EN) >> 11)
				, "\n\tPXP_CTRL_DATA_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PXP_CTRL_DATA_IN_EN) >> 10)
				, "\n\tPXP_DP_FULL_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PXP_DP_FULL_IN_EN) >> 9)
				, "\n\tPXP_CTRL_FULL_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PXP_CTRL_FULL_IN_EN) >> 8)
				, "\n\tPXP_DP_DONE_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PXP_DP_DONE_IN_EN) >> 7)
				, "\n\tPXP_CTRL_DONE_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PXP_CTRL_DONE_IN_EN) >> 6)
				, "\n\tPAS_BUF_FULL_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PAS_BUF_FULL_IN_EN) >> 5)
				, "\n\tPAS_BUF_DONE_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_PAS_BUF_DONE_IN_EN) >> 4)
				, "\n\tINT_RAM_FULL_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_INT_RAM_FULL_IN_EN) >> 3)
				, "\n\tINT_RAM_DONE_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_INT_RAM_DONE_IN_EN) >> 2)
				, "\n\tINT_RAM_DATA_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_INT_RAM_DATA_IN_EN) >> 1)
				, "\n\tEXT_STORE_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN1_REG_EXT_STORE_IN_EN) >> 0));
			break;
		case 0xc223cUL: // (CSDM_REG_ENABLE_IN2: ACCESS:RW  DataWidth:0x7  Multi Field Register  ) 
			sprintf(disp_str, "\nCSDM_REG_ENABLE_IN2 (ACCESS:RW  DataWidth:0x7  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_REQ_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN2_REG_DORQ_REQ_IN_EN) >> 6)
				, "\n\tPB_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN2_REG_PB_EXT_WR_FULL_IN_EN) >> 5)
				, "\n\tPBF_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN2_REG_PBF_EXT_WR_FULL_IN_EN) >> 4)
				, "\n\tPB_FULL_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN2_REG_PB_FULL_IN_EN) >> 3)
				, "\n\tPB_STATUS_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN2_REG_PB_STATUS_IN_EN) >> 2)
				, "\n\tCM_ACK_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN2_REG_CM_ACK_IN_EN) >> 1)
				, "\n\tSDM_ACK_IN_EN :"
				, (u32_t)((data & CSDM_ENABLE_IN2_REG_SDM_ACK_IN_EN) >> 0));
			break;
		case 0xc2240UL: // (CSDM_REG_ENABLE_OUT1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nCSDM_REG_ENABLE_OUT1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPBF_EXT_WR_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PBF_EXT_WR_OUT_EN) >> 26)
				, "\n\tPB_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PB_OUT_EN) >> 25)
				, "\n\tCFC_SDM_ACK_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_CFC_SDM_ACK_OUT_EN) >> 24)
				, "\n\tCM_MSG_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_CM_MSG_OUT_EN) >> 23)
				, "\n\tPXP_REQ_DONE_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PXP_REQ_DONE_OUT_EN) >> 22)
				, "\n\tEXT_FULL_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_EXT_FULL_OUT_EN) >> 21)
				, "\n\tPXP_CTRL_FULL_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PXP_CTRL_FULL_OUT_EN) >> 20)
				, "\n\tPB_FULL_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PB_FULL_OUT_EN) >> 19)
				, "\n\tBRB1_DP_FULL_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_BRB1_DP_FULL_OUT_EN) >> 18)
				, "\n\tBRB1_CTRL_FULL_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_BRB1_CTRL_FULL_OUT_EN) >> 17)
				, "\n\tPXP_DP_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PXP_DP_OUT_EN) >> 16)
				, "\n\tPXP_CTRL_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PXP_CTRL_OUT_EN) >> 15)
				, "\n\tPXP_ASYNC_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PXP_ASYNC_OUT_EN) >> 14)
				, "\n\tPAS_BUF_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PAS_BUF_OUT_EN) >> 13)
				, "\n\tINT_RAM_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_INT_RAM_OUT_EN) >> 12)
				, "\n\tPRS_ACK_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PRS_ACK_OUT_EN) >> 11)
				, "\n\tPRS_SYNC_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PRS_SYNC_OUT_EN) >> 10)
				, "\n\tBRB1_DP_REQ_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_BRB1_DP_REQ_OUT_EN) >> 9)
				, "\n\tBRB1_CTRL_REQ_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_BRB1_CTRL_REQ_OUT_EN) >> 8)
				, "\n\tPXP_DP_REQ_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PXP_DP_REQ_OUT_EN) >> 7)
				, "\n\tPXP_CTRL_REQ_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PXP_CTRL_REQ_OUT_EN) >> 6)
				, "\n\tCFC_PB_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_CFC_PB_OUT_EN) >> 5)
				, "\n\tCFC_ACDEC_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_CFC_ACDEC_OUT_EN) >> 4)
				, "\n\tCFC_ACINC_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_CFC_ACINC_OUT_EN) >> 3)
				, "\n\tCFC_LOAD_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_CFC_LOAD_OUT_EN) >> 2)
				, "\n\tTHREADREADY_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_THREADREADY_OUT_EN) >> 1)
				, "\n\tPXP_INT_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT1_REG_PXP_INT_OUT_EN) >> 0));
			break;
		case 0xc2244UL: // (CSDM_REG_ENABLE_OUT2: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nCSDM_REG_ENABLE_OUT2 (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_REQ_DONE_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT2_REG_DORQ_REQ_DONE_OUT_EN) >> 5)
				, "\n\tVFPF_ERR_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT2_REG_VFPF_ERR_OUT_EN) >> 4)
				, "\n\tSDM_EXT_WR_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT2_REG_SDM_EXT_WR_OUT_EN) >> 3)
				, "\n\tQM_EXT_WR_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT2_REG_QM_EXT_WR_OUT_EN) >> 2)
				, "\n\tDQ_EXT_WR_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT2_REG_DQ_EXT_WR_OUT_EN) >> 1)
				, "\n\tPB_EXT_WR_OUT_EN :"
				, (u32_t)((data & CSDM_ENABLE_OUT2_REG_PB_EXT_WR_OUT_EN) >> 0));
			break;
		case 0xc2248UL: // (CSDM_REG_NUM_OF_Q0_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q0_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc224cUL: // (CSDM_REG_NUM_OF_Q1_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q1_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2250UL: // (CSDM_REG_NUM_OF_Q3_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q3_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2254UL: // (CSDM_REG_NUM_OF_Q4_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q4_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2258UL: // (CSDM_REG_NUM_OF_Q5_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q5_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc225cUL: // (CSDM_REG_NUM_OF_Q6_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q6_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2260UL: // (CSDM_REG_NUM_OF_Q7_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q7_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2264UL: // (CSDM_REG_NUM_OF_Q8_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q8_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2268UL: // (CSDM_REG_NUM_OF_Q9_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q9_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc226cUL: // (CSDM_REG_NUM_OF_Q10_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q10_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2270UL: // (CSDM_REG_NUM_OF_Q11_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_Q11_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2274UL: // (CSDM_REG_NUM_OF_PKT_END_MSG: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_PKT_END_MSG (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of packet end messages received from the parser\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2278UL: // (CSDM_REG_NUM_OF_PXP_ASYNC_REQ: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_PXP_ASYNC_REQ (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of requests received from the pxp async if\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc227cUL: // (CSDM_REG_NUM_OF_ACK_AFTER_PLACE: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_NUM_OF_ACK_AFTER_PLACE (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of ACK after placement messages received \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2280UL: // (CSDM_REG_STATISTICS_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCSDM_REG_STATISTICS_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM bits for statistics sram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2284UL: // (CSDM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from CSDM\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2288UL: // (CSDM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from CSDM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc228cUL: // (CSDM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSDM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from CSDM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2290UL: // (CSDM_REG_CSDM_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc2294UL: // (CSDM_REG_CSDM_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc2298UL: // (CSDM_REG_CSDM_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc229cUL: // (CSDM_REG_CSDM_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc22a0UL: // (CSDM_REG_CSDM_INT_STS_1: ACCESS:R  DataWidth:0xe ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_INT_STS_1 (ACCESS:R  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0xc22a4UL: // (CSDM_REG_CSDM_INT_STS_CLR_1: ACCESS:RC  DataWidth:0xe ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_CLR_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0xc22a8UL: // (CSDM_REG_CSDM_INT_STS_WR_1: ACCESS:WR  DataWidth:0xe ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_INT_STS_WR_1 (ACCESS:WR  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_STS_WR_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0xc22acUL: // (CSDM_REG_CSDM_INT_MASK_1: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_INT_MASK_1 (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & CSDM_CSDM_INT_MASK_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0xc22b0UL: // (CSDM_REG_CSDM_PRTY_STS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_PRTY_STS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0xc22b4UL: // (CSDM_REG_CSDM_PRTY_STS_CLR: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_PRTY_STS_CLR (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0xc22b8UL: // (CSDM_REG_CSDM_PRTY_STS_WR: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_PRTY_STS_WR (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CSDM_CSDM_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0xc22bcUL: // (CSDM_REG_CSDM_PRTY_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_PRTY_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CSDM_CSDM_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0xc22c0UL: // (CSDM_REG_CM_QUEUE_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_CM_QUEUE_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits  CM_QUEUE\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc22c4UL: // (CSDM_REG_INP_QUEUE_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_INP_QUEUE_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits  INP_QUEUE\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc22c8UL: // (CSDM_REG_FIFOS_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCSDM_REG_FIFOS_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits  fifos: PXP_CTRL[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc22ccUL: // (CSDM_REG_TIMERS_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSDM_REG_TIMERS_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits for timers sram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc22d0UL: // (CSDM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSDM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc22d8UL: // (CSDM_REG_CMP_COUNTER_MAX4: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCSDM_REG_CMP_COUNTER_MAX4 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc22dcUL: // (CSDM_REG_CMP_COUNTER_MAX5: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCSDM_REG_CMP_COUNTER_MAX5 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc22e0UL: // (CSDM_REG_CMP_COUNTER_MAX6: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCSDM_REG_CMP_COUNTER_MAX6 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc22e4UL: // (CSDM_REG_CMP_COUNTER_MAX7: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCSDM_REG_CMP_COUNTER_MAX7 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2400UL: // (CSDM_REG_AGGREG_INTERRUPT_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_AGGREG_INTERRUPT_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: lsb register of aggregated interrupt in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2404UL: // (CSDM_REG_AGGREG_INTERRUPT_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_AGGREG_INTERRUPT_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: msb register of aggregated interrupt  in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2408UL: // (CSDM_REG_ASYNC_HOST_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_ASYNC_HOST_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: async fifo empty in sdm_async block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc240cUL: // (CSDM_REG_ASYNC_HOST_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_ASYNC_HOST_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: async fifo full in sdm_async block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2410UL: // (CSDM_REG_CFC_LOAD_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_CFC_LOAD_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2414UL: // (CSDM_REG_CFC_LOAD_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_CFC_LOAD_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load pending fifo full in sdm_cfc block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2418UL: // (CSDM_REG_CFC_LOAD_RSP_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_CFC_LOAD_RSP_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load rsp fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc241cUL: // (CSDM_REG_CFC_LOAD_RSP_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_CFC_LOAD_RSP_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load rsp fifo full in sdm_cfcblock\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2420UL: // (CSDM_REG_CM_DELAY_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_CM_DELAY_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm delay fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2424UL: // (CSDM_REG_CM_DELAY_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_CM_DELAY_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm delay fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2428UL: // (CSDM_REG_CM_QUEUE_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_CM_QUEUE_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm queue fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc242cUL: // (CSDM_REG_CM_QUEUE_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_CM_QUEUE_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm queue fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2430UL: // (CSDM_REG_DELAY_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DELAY_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: delay FIFO  empty in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2434UL: // (CSDM_REG_DELAY_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DELAY_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: delay FIFO  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2438UL: // (CSDM_REG_DST_BRB1_CTRL_SRC_ADDR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_BRB1_CTRL_SRC_ADDR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src address fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc243cUL: // (CSDM_REG_DST_BRB1_CTRL_SRC_ADDR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_BRB1_CTRL_SRC_ADDR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src address fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2440UL: // (CSDM_REG_DST_BRB1_CTRL_SRC_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_BRB1_CTRL_SRC_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2444UL: // (CSDM_REG_DST_BRB1_CTRL_SRC_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_BRB1_CTRL_SRC_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2448UL: // (CSDM_REG_DST_INT_RAM_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_INT_RAM_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc244cUL: // (CSDM_REG_DST_INT_RAM_WAIT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_INT_RAM_WAIT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram_wait fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2450UL: // (CSDM_REG_DST_INT_RAM_WAIT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_INT_RAM_WAIT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram_wait fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2454UL: // (CSDM_REG_DST_NONE_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_NONE_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: none pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2458UL: // (CSDM_REG_DST_NONE_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_NONE_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: none pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc245cUL: // (CSDM_REG_DST_PAS_BUF_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PAS_BUF_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2460UL: // (CSDM_REG_DST_PAS_BUF_WAIT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PAS_BUF_WAIT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf_wait fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2464UL: // (CSDM_REG_DST_PAS_BUF_WAIT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PAS_BUF_WAIT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf_wait fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2468UL: // (CSDM_REG_DST_PB_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PB_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc246cUL: // (CSDM_REG_DST_PB_IMMED_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PB_IMMED_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb immediate fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2470UL: // (CSDM_REG_DST_PB_IMMED_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PB_IMMED_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb immediate fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2474UL: // (CSDM_REG_DST_PXP_CTRL_DST_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_CTRL_DST_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_dst pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2478UL: // (CSDM_REG_DST_PXP_CTRL_DST_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_CTRL_DST_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_dst pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc247cUL: // (CSDM_REG_DST_PXP_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2480UL: // (CSDM_REG_DST_PXP_CTRL_IMMED_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_CTRL_IMMED_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl immediate fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2484UL: // (CSDM_REG_DST_PXP_CTRL_IMMED_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_CTRL_IMMED_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl immediate fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2488UL: // (CSDM_REG_DST_PXP_CTRL_LINK_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_CTRL_LINK_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl link list empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc248cUL: // (CSDM_REG_DST_PXP_CTRL_LINK_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_CTRL_LINK_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl link list full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2490UL: // (CSDM_REG_DST_PXP_CTRL_SRC_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_CTRL_SRC_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_src pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2494UL: // (CSDM_REG_DST_PXP_CTRL_SRC_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_CTRL_SRC_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_src pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2498UL: // (CSDM_REG_DST_PXP_DP_DST_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_DP_DST_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp_dst pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc249cUL: // (CSDM_REG_DST_PXP_DP_DST_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_DP_DST_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp_dst pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24a0UL: // (CSDM_REG_DST_PXP_DP_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_DP_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24a4UL: // (CSDM_REG_DST_PXP_DP_LINK_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_DP_LINK_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp link list empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24a8UL: // (CSDM_REG_DST_PXP_DP_LINK_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_DST_PXP_DP_LINK_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp link list full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24acUL: // (CSDM_REG_INIT_CREDIT_CFC_ACDEC: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCSDM_REG_INIT_CREDIT_CFC_ACDEC (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     activity counters interface without receiving any ACK\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24b0UL: // (CSDM_REG_INIT_CREDIT_CFC_ACINC: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCSDM_REG_INIT_CREDIT_CFC_ACINC (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     activity counters interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24b4UL: // (CSDM_REG_INIT_CREDIT_CFC_LOAD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCSDM_REG_INIT_CREDIT_CFC_LOAD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     load interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24b8UL: // (CSDM_REG_INIT_CREDIT_CM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCSDM_REG_INIT_CREDIT_CM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of cycles that can be sent to the CM  \n    interface without receiving any ACK in CM block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24bcUL: // (CSDM_REG_INIT_CREDIT_PXP_CTRL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nCSDM_REG_INIT_CREDIT_PXP_CTRL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the pxp\n     control interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24c0UL: // (CSDM_REG_INT_RAM_RR_REQ: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCSDM_REG_INT_RAM_RR_REQ (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: round robin for int_ram arbiter: b0-pas_buf; b1-int_ram;b2\n    -pxp_dp;b3-pxp_ctrl;b4-brb1_ctrl;b5-brb1_dp;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24c4UL: // (CSDM_REG_OPERATION_GEN: ACCESS:W  DataWidth:0x11 ) 
			sprintf(disp_str, "\nCSDM_REG_OPERATION_GEN (ACCESS:W  DataWidth:0x11 )" \
				"\nDescription: Generate an operation after completion; bit-16 is AggVectIdx_valid\n    ; bits 15:8 are AggVectIdx; bits 7:5 are the TRIG and bits 4:0 are the\n     T124Param[4:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24c8UL: // (CSDM_REG_PB_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_PB_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: UPB IF  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24ccUL: // (CSDM_REG_PBF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_PBF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF if  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24d0UL: // (CSDM_REG_PXP_DELAY_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_PXP_DELAY_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp switch delay fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24d4UL: // (CSDM_REG_PXP_DELAY_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_PXP_DELAY_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp switch delay fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24d8UL: // (CSDM_REG_QM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_QM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: QM IF  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24dcUL: // (CSDM_REG_QUEUE_EMPTY: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nCSDM_REG_QUEUE_EMPTY (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Input queue fifo empty in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24e0UL: // (CSDM_REG_QUEUE_FULL: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nCSDM_REG_QUEUE_FULL (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Input queue fifo full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24e4UL: // (CSDM_REG_RR_CNT_COUNTERS_STATUS: ACCESS:R  DataWidth:0x15 ) 
			sprintf(disp_str, "\nCSDM_REG_RR_CNT_COUNTERS_STATUS (ACCESS:R  DataWidth:0x15 )" \
				"\nDescription: round robin for all completion counters\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24e8UL: // (CSDM_REG_RR_COMPLETE_REQ: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nCSDM_REG_RR_COMPLETE_REQ (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: round robin for all completion requests  in sdm_cm block: \n    b0-async b1-nop;b2-pxp_int; b3-timers;b4-dma;b5-grc;b6-rbc\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24ecUL: // (CSDM_REG_RR_PTR_REQ: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nCSDM_REG_RR_PTR_REQ (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: round robin for cm pointer: b0-async; b1-dma_dp; b2 - dma_ctrl\n    ; b3-cfc; b4-nop; b5-timers; b6-pxp_int\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24f0UL: // (CSDM_REG_RSP_BRB1_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24f4UL: // (CSDM_REG_RSP_BRB1_CTRL_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_CTRL_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24f8UL: // (CSDM_REG_RSP_BRB1_CTRL_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_CTRL_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc24fcUL: // (CSDM_REG_RSP_BRB1_CTRL_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_CTRL_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2500UL: // (CSDM_REG_RSP_BRB1_CTRL_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_CTRL_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2504UL: // (CSDM_REG_RSP_BRB1_DP_DST_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_DP_DST_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending dst fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2508UL: // (CSDM_REG_RSP_BRB1_DP_DST_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_DP_DST_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending dst fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc250cUL: // (CSDM_REG_RSP_BRB1_DP_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_DP_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2510UL: // (CSDM_REG_RSP_BRB1_DP_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_DP_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2514UL: // (CSDM_REG_RSP_BRB1_DP_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_DP_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2518UL: // (CSDM_REG_RSP_BRB1_DP_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_DP_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc251cUL: // (CSDM_REG_RSP_BRB1_DP_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_BRB1_DP_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2520UL: // (CSDM_REG_RSP_INT_RAM_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_INT_RAM_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2524UL: // (CSDM_REG_RSP_INT_RAM_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_INT_RAM_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2528UL: // (CSDM_REG_RSP_INT_RAM_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_INT_RAM_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc252cUL: // (CSDM_REG_RSP_INT_RAM_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_INT_RAM_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2530UL: // (CSDM_REG_RSP_PB_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_PB_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2534UL: // (CSDM_REG_RSP_PB_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_PB_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2538UL: // (CSDM_REG_RSP_PB_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_PB_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc253cUL: // (CSDM_REG_RSP_PB_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_PB_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2540UL: // (CSDM_REG_RSP_PB_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_PB_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2544UL: // (CSDM_REG_RSP_PXP_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_PXP_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2548UL: // (CSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc254cUL: // (CSDM_REG_RSP_PXP_CTRL_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_RSP_PXP_CTRL_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2550UL: // (CSDM_REG_SYNC_PARSER_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_SYNC_PARSER_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser fifo  empty in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2554UL: // (CSDM_REG_SYNC_PARSER_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_SYNC_PARSER_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser fifo  full in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2558UL: // (CSDM_REG_SYNC_SYNC_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_SYNC_SYNC_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser serial fifo  empty in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc255cUL: // (CSDM_REG_SYNC_SYNC_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_SYNC_SYNC_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser serial fifo  full in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2560UL: // (CSDM_REG_TIMERS_ADDR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_TIMERS_ADDR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: address FIFO  empty in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2564UL: // (CSDM_REG_TIMERS_ADDR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_TIMERS_ADDR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: address FIFO  full in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2568UL: // (CSDM_REG_TIMERS_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_TIMERS_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pending FIFO  empty in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc256cUL: // (CSDM_REG_TIMERS_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSDM_REG_TIMERS_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pending FIFO  full in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2600UL: // (CSDM_REG_STATISTICS: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSDM_REG_STATISTICS (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Statistics memory. Each read from RBC resets the corresponding\n     statistic counter\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2800UL: // (CSDM_REG_TIMERS: ACCESS:WB  DataWidth:0x34 ) 
			sprintf(disp_str, "\nCSDM_REG_TIMERS (ACCESS:WB  DataWidth:0x34 )" \
				"\nDescription: Debug only. Timers memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc3000UL: // (CSDM_REG_CM_QUEUE: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nCSDM_REG_CM_QUEUE (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Debug only. CM queue memory. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc3800UL: // (CSDM_REG_INP_QUEUE: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nCSDM_REG_INP_QUEUE (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Debug only. Input queue memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc22d4UL: // (CSDM_REG_CSDM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc22e8UL: // (CSDM_REG_CSDM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2570UL: // (CSDM_REG_CSDM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc2700UL: // (CSDM_REG_CSDM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSDM_REG_CSDM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200000UL: // (CSEM_REG_MSG_NUM_FIC0: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCSEM_REG_MSG_NUM_FIC0 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that entered \n    through FIC0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200004UL: // (CSEM_REG_MSG_NUM_FIC1: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCSEM_REG_MSG_NUM_FIC1 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that entered \n    through FIC1 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200008UL: // (CSEM_REG_MSG_NUM_FOC0: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCSEM_REG_MSG_NUM_FOC0 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20000cUL: // (CSEM_REG_MSG_NUM_FOC1: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCSEM_REG_MSG_NUM_FOC1 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC1 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200010UL: // (CSEM_REG_MSG_NUM_FOC2: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCSEM_REG_MSG_NUM_FOC2 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC2 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200014UL: // (CSEM_REG_MSG_NUM_FOC3: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCSEM_REG_MSG_NUM_FOC3 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200018UL: // (CSEM_REG_THREAD_INTER_CNT_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_THREAD_INTER_CNT_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for start count of counter ~csem_registers_thread_inter_cnt\n    .thread_inter_cnt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20001cUL: // (CSEM_REG_THREAD_INTER_CNT: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nCSEM_REG_THREAD_INTER_CNT (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Maximum value of threads interrupt counter; when it gets \n    this value then interrupt to will be send if thread active from previous\n     maximum value of this counter. This register may be used only when ~csem_registers_thread_inter_cnt_enable\n    .thread_inter_cnt_enable =1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200020UL: // (CSEM_REG_ARB_ELEMENT0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_ARB_ELEMENT0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 0. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200024UL: // (CSEM_REG_ARB_ELEMENT1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_ARB_ELEMENT1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 1. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~csem_registers_arb_element0.arb_element0\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200028UL: // (CSEM_REG_ARB_ELEMENT2: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_ARB_ELEMENT2 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 2. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~csem_registers_arb_element0.arb_element0\n     and ~csem_registers_arb_element1.arb_element1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20002cUL: // (CSEM_REG_ARB_ELEMENT3: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_ARB_ELEMENT3 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 3. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2.Could not be equal to register ~csem_registers_arb_element0.arb_element0\n     and ~csem_registers_arb_element1.arb_element1 and ~csem_registers_arb_element2\n    .arb_element2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200030UL: // (CSEM_REG_ARB_ELEMENT4: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_ARB_ELEMENT4 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 4. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~csem_registers_arb_element0.arb_element0\n     and ~csem_registers_arb_element1.arb_element1 and ~csem_registers_arb_element2\n    .arb_element2 and ~csem_registers_arb_element3.arb_element3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200038UL: // (CSEM_REG_TS_0_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_0_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20003cUL: // (CSEM_REG_TS_1_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_1_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200040UL: // (CSEM_REG_TS_2_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_2_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200044UL: // (CSEM_REG_TS_3_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_3_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200048UL: // (CSEM_REG_TS_4_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_4_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20004cUL: // (CSEM_REG_TS_5_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_5_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200050UL: // (CSEM_REG_TS_6_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_6_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200054UL: // (CSEM_REG_TS_7_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_7_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200058UL: // (CSEM_REG_TS_8_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_8_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20005cUL: // (CSEM_REG_TS_9_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_9_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200060UL: // (CSEM_REG_TS_10_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_10_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200064UL: // (CSEM_REG_TS_11_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_11_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200068UL: // (CSEM_REG_TS_12_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_12_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20006cUL: // (CSEM_REG_TS_13_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_13_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200070UL: // (CSEM_REG_TS_14_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_14_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200074UL: // (CSEM_REG_TS_15_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_15_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200078UL: // (CSEM_REG_TS_16_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_16_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20007cUL: // (CSEM_REG_TS_17_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_17_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200080UL: // (CSEM_REG_TS_18_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_18_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200084UL: // (CSEM_REG_TS_19_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_TS_19_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200088UL: // (CSEM_REG_FIC0_MIN_MSG_LINES: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC0_MIN_MSG_LINES (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The minimum number of cycles in a message from FIC0 interfaces\n     after which the message can be sent to the passive register_file. It \n    must be always equal to 2; other way it may cause to deadlock in a chip\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20008cUL: // (CSEM_REG_FIC1_MIN_MSG_LINES: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC1_MIN_MSG_LINES (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The minimum number of cycles in a message from FIC1 interfaces\n     after which the message can be sent to the passive register_file. It \n    must be always equal to 2; other way it may cause to deadlock in a chip\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200090UL: // (CSEM_REG_PASSIVE_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_PASSIVE_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of free entries in the sync FIFO between the external\n     HW and the passive buffer; below which the PassiveFull is asserted \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200094UL: // (CSEM_REG_SYNC_DRA_WR_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_SYNC_DRA_WR_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full for sync dra_wr fifo (data from DRA to STORM)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200098UL: // (CSEM_REG_SYNC_RAM_WR_ALM_FULL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCSEM_REG_SYNC_RAM_WR_ALM_FULL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Almost full for sync ram_wr fifo (data from EXT_IF to STORM\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20009cUL: // (CSEM_REG_DBG_ALM_FULL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCSEM_REG_DBG_ALM_FULL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Almost full for slow debug fifo \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000a0UL: // (CSEM_REG_EXCEPTION_INT: ACCESS:RW  DataWidth:0xf ) 
			sprintf(disp_str, "\nCSEM_REG_EXCEPTION_INT (ACCESS:RW  DataWidth:0xf )" \
				"\nDescription: The PRAM address for the interrupt in a case the event ID\n     is bigger then the INT table size. This register is always NA; because\n     this feature is removed\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000a4UL: // (CSEM_REG_ENABLE_IN: ACCESS:RW  DataWidth:0xf  Multi Field Register  ) 
			sprintf(disp_str, "\nCSEM_REG_ENABLE_IN (ACCESS:RW  DataWidth:0xf  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFPF_ERROR_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_VFPF_ERROR_ENABLE_IN) >> 14)
				, "\n\tWAITP_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_WAITP_ENABLE_IN) >> 13)
				, "\n\tFOC3_ACK_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_FOC3_ACK_ENABLE_IN) >> 12)
				, "\n\tFOC2_ACK_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_FOC2_ACK_ENABLE_IN) >> 11)
				, "\n\tFOC1_ACK_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_FOC1_ACK_ENABLE_IN) >> 10)
				, "\n\tFOC0_ACK_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_FOC0_ACK_ENABLE_IN) >> 9)
				, "\n\tRAM1_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_RAM1_ENABLE_IN) >> 8)
				, "\n\tRAM0_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_RAM0_ENABLE_IN) >> 7)
				, "\n\tEXT_FULL_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_EXT_FULL_ENABLE_IN) >> 6)
				, "\n\tEXT_RD_DATA_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_EXT_RD_DATA_ENABLE_IN) >> 5)
				, "\n\tTHREAD_RDY_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_THREAD_RDY_ENABLE_IN) >> 4)
				, "\n\tGENERAL_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_GENERAL_ENABLE_IN) >> 3)
				, "\n\tPASSIVE_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_PASSIVE_ENABLE_IN) >> 2)
				, "\n\tFIC1_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_FIC1_ENABLE_IN) >> 1)
				, "\n\tFIC0_ENABLE_IN :"
				, (u32_t)((data & CSEM_ENABLE_IN_REG_FIC0_ENABLE_IN) >> 0));
			break;
		case 0x2000a8UL: // (CSEM_REG_ENABLE_OUT: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nCSEM_REG_ENABLE_OUT (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWAITP_ENABLE_OUT :"
				, (u32_t)((data & CSEM_ENABLE_OUT_REG_WAITP_ENABLE_OUT) >> 9)
				, "\n\tRAM1_ENABLE_OUT :"
				, (u32_t)((data & CSEM_ENABLE_OUT_REG_RAM1_ENABLE_OUT) >> 8)
				, "\n\tRAM0_ENABLE_OUT :"
				, (u32_t)((data & CSEM_ENABLE_OUT_REG_RAM0_ENABLE_OUT) >> 7)
				, "\n\tPASSIVE_ENABLE_OUT :"
				, (u32_t)((data & CSEM_ENABLE_OUT_REG_PASSIVE_ENABLE_OUT) >> 6)
				, "\n\tFOC3_ENABLE_OUT :"
				, (u32_t)((data & CSEM_ENABLE_OUT_REG_FOC3_ENABLE_OUT) >> 5)
				, "\n\tFOC2_ENABLE_OUT :"
				, (u32_t)((data & CSEM_ENABLE_OUT_REG_FOC2_ENABLE_OUT) >> 4)
				, "\n\tFOC1_ENABLE_OUT :"
				, (u32_t)((data & CSEM_ENABLE_OUT_REG_FOC1_ENABLE_OUT) >> 3)
				, "\n\tFOC0_ENABLE_OUT :"
				, (u32_t)((data & CSEM_ENABLE_OUT_REG_FOC0_ENABLE_OUT) >> 2)
				, "\n\tEXT_WR_REQ_ENABLE_OUT :"
				, (u32_t)((data & CSEM_ENABLE_OUT_REG_EXT_WR_REQ_ENABLE_OUT) >> 1)
				, "\n\tEXT_RD_REQ_ENABLE_OUT :"
				, (u32_t)((data & CSEM_ENABLE_OUT_REG_EXT_RD_REQ_ENABLE_OUT) >> 0));
			break;
		case 0x2000acUL: // (CSEM_REG_STORM0_H_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_STORM0_H_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits for storm0_h memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000b0UL: // (CSEM_REG_STORM1_H_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_STORM1_H_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm1_h memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000b4UL: // (CSEM_REG_STORM0_L_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_STORM0_L_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm0_l memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000b8UL: // (CSEM_REG_STORM1_L_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_STORM1_L_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm1_l memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000bcUL: // (CSEM_REG_CAM_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nCSEM_REG_CAM_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM bits  for cam\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000c0UL: // (CSEM_REG_RAM0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_RAM0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory ram0_0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000c4UL: // (CSEM_REG_PAS_BUF_LSB_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_PAS_BUF_LSB_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_lsb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000c8UL: // (CSEM_REG_PAS_BUF_LSB_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_PAS_BUF_LSB_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_lsb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000ccUL: // (CSEM_REG_PAS_BUF_MSB_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_PAS_BUF_MSB_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_msb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000d0UL: // (CSEM_REG_PAS_BUF_MSB_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_PAS_BUF_MSB_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_msb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000d4UL: // (CSEM_REG_INT_TABLE_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_INT_TABLE_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory int_table\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000d8UL: // (CSEM_REG_CLEAR_WAITP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_CLEAR_WAITP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Write 1 to this register will disable waitp from this storm\n     to other storms\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000dcUL: // (CSEM_REG_SLOW_DBG_MODE: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DBG_MODE (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: debug mode for slow debug bus. Applicable only when ~csem_registers_slow_dbg_active\n    .slow_dbg_active =1. If mode =0 thread number; pram address and DRA WR\n     data selected; if mode =1 fin command and DRA RD ; if mode =2 pram address\n     and thread number and fin command and released thread from STORM; if \n    mode =3 STORE data to SDM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000e0UL: // (CSEM_REG_SLOW_DBG_ACTIVE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DBG_ACTIVE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug mode is active\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000e4UL: // (CSEM_REG_DBG_MSG_SRC: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_DBG_MSG_SRC (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~csem_registers_slow_dbg_mode.slow_dbg_mode\n     =0. If =0only FIC-s output to debug bus; 1=both FIC-s and passive buffer\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000e8UL: // (CSEM_REG_DBG_MODE0_CFG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_DBG_MODE0_CFG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~csem_registers_slow_dbg_mode.slow_dbg_mode\n     =0. If =0 all the message output to debug bus; 1=partial message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000ecUL: // (CSEM_REG_DBG_MODE0_CFG_CYCLE: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_DBG_MODE0_CFG_CYCLE (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Applicable only when ~csem_registers_dbg_mode0_cfg.dbg_mode0_cfg\n     =1. If =1 the additional cycles to extract to the debug bus.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000f0UL: // (CSEM_REG_DBG_MODE1_CFG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_DBG_MODE1_CFG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~csem_registers_slow_dbg_mode.slow_dbg_mode\n     =1. If=0 output to debug bus without the data; 1=with the data.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000f4UL: // (CSEM_REG_DBG_EACH_CYLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_DBG_EACH_CYLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If=0 output every cycle full indication or thread status; \n    1= output only when there is a change.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000f8UL: // (CSEM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSEM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from USEMI\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000fcUL: // (CSEM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSEM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only.For dbgmux usage (debug data that goes from USEM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200100UL: // (CSEM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nCSEM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only.For dbgmux usage (debug data that goes from USEM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200104UL: // (CSEM_REG_CSEM_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x200108UL: // (CSEM_REG_CSEM_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x20010cUL: // (CSEM_REG_CSEM_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x200110UL: // (CSEM_REG_CSEM_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x200114UL: // (CSEM_REG_CSEM_INT_STS_1: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_INT_STS_1 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x200118UL: // (CSEM_REG_CSEM_INT_STS_CLR_1: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_CLR_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x20011cUL: // (CSEM_REG_CSEM_INT_STS_WR_1: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_INT_STS_WR_1 (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_STS_WR_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x200120UL: // (CSEM_REG_CSEM_INT_MASK_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_INT_MASK_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & CSEM_CSEM_INT_MASK_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x200124UL: // (CSEM_REG_CSEM_PRTY_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_PRTY_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_0_REG_PARITY) >> 0));
			break;
		case 0x200128UL: // (CSEM_REG_CSEM_PRTY_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_PRTY_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_0_REG_PARITY) >> 0));
			break;
		case 0x20012cUL: // (CSEM_REG_CSEM_PRTY_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_PRTY_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_0_REG_PARITY) >> 0));
			break;
		case 0x200130UL: // (CSEM_REG_CSEM_PRTY_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_PRTY_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_0_REG_PARITY) >> 0));
			break;
		case 0x200134UL: // (CSEM_REG_CSEM_PRTY_STS_1: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_PRTY_STS_1 (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Parity register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x200138UL: // (CSEM_REG_CSEM_PRTY_STS_CLR_1: ACCESS:RC  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_PRTY_STS_CLR_1 (ACCESS:RC  DataWidth:0x5 )" \
				"\nDescription: Parity register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_CLR_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x20013cUL: // (CSEM_REG_CSEM_PRTY_STS_WR_1: ACCESS:WR  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_PRTY_STS_WR_1 (ACCESS:WR  DataWidth:0x5 )" \
				"\nDescription: Parity register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_STS_WR_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x200140UL: // (CSEM_REG_CSEM_PRTY_MASK_1: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_PRTY_MASK_1 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Parity mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & CSEM_CSEM_PRTY_MASK_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x20014cUL: // (CSEM_REG_RAM0_TM1: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_RAM0_TM1 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory ram0_1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200150UL: // (CSEM_REG_RAM0_TM2: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_RAM0_TM2 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory ram0_2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2003a8UL: // (CSEM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSEM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2003acUL: // (CSEM_REG_FIFOS_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nCSEM_REG_FIFOS_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM bits  for  FIC0_LSB [1:0]; FIC0_MSB[3:2]; FIC1_LSB[5:4\n    ]; FIC1_MSB[7:6]; DBG_LSB[9:8];DBG_MSB[11:10]; EXT_PAS[13:12]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200200UL: // (CSEM_REG_ARBITER_REQUEST: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_ARBITER_REQUEST (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last request: 0-fic0; 1-fic1; 2-priority0; 3-\n    priority1; 4-priority2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200204UL: // (CSEM_REG_ARBITER_SELECT: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_ARBITER_SELECT (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last selection: 0-fic0; 1-fic1; 2-priority0; \n    3-priority1; 4-priority2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200208UL: // (CSEM_REG_ARBITER_SLOT: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_ARBITER_SLOT (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last slot\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20020cUL: // (CSEM_REG_DBG_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_DBG_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG IF is full in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200210UL: // (CSEM_REG_DRA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_DRA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: This register is active when FIN FIO  is empty and DRA RD\n     FIFO is empty\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200214UL: // (CSEM_REG_EXT_PAS_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_EXT_PAS_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_PAS FIFO empty in sem_slow\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200218UL: // (CSEM_REG_EXT_PAS_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_EXT_PAS_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_PAS FIFO Full in sem_slow\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20021cUL: // (CSEM_REG_EXT_STORE_FREE_ENTRIES: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCSEM_REG_EXT_STORE_FREE_ENTRIES (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Number of free entries in the external STORE sync FIFO.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200220UL: // (CSEM_REG_EXT_STORE_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_EXT_STORE_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE IF is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200224UL: // (CSEM_REG_FIC0_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC0_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from FIC0  May be updated during \n    run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200228UL: // (CSEM_REG_FIC0_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC0_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO empty in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20022cUL: // (CSEM_REG_FIC0_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC0_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO Full in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200230UL: // (CSEM_REG_FIC0_LENGTH: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC0_LENGTH (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Length from FIC0. Active only with ~csem_registers_fic0_length_error\n    .fic0_length_error interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200234UL: // (CSEM_REG_FIC1_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC1_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from FIC1  May be updated during \n    run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200238UL: // (CSEM_REG_FIC1_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC1_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO empty in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20023cUL: // (CSEM_REG_FIC1_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC1_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO Full in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200240UL: // (CSEM_REG_FIC1_LENGTH: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC1_LENGTH (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Length from FIC1. Active only with ~csem_registers_fic1_length_error\n    .fic1_length_error interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200244UL: // (CSEM_REG_GPI_DATA: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nCSEM_REG_GPI_DATA (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: GPI signals that are inputs to SEMI\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200248UL: // (CSEM_REG_NUM_OF_THREADS: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nCSEM_REG_NUM_OF_THREADS (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: The number of threads currently active \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20024cUL: // (CSEM_REG_PAS_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_PAS_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from the passive buffer  May be updated\n     during run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200250UL: // (CSEM_REG_PAS_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_PAS_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Full from passive buffer asserted toward SDM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200254UL: // (CSEM_REG_RAM0_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_RAM0_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RAM0 IF is full in sem_slow_ls_ram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200258UL: // (CSEM_REG_RAM1_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_RAM1_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RAM1 IF is full in sem_slow_ls_ram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20025cUL: // (CSEM_REG_SET0_THREAD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SET0_THREAD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET0_THREAD fifo is empty in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200260UL: // (CSEM_REG_SET0_THREAD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SET0_THREAD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET0_THREAD fifo is full in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200264UL: // (CSEM_REG_SET1_THREAD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SET1_THREAD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET1_THREAD fifo is empty in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200268UL: // (CSEM_REG_SET1_THREAD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SET1_THREAD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET1_THREAD fifo is full in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20026cUL: // (CSEM_REG_SLEEP_THREADS_VALID: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nCSEM_REG_SLEEP_THREADS_VALID (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Valid sleeping threads indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200270UL: // (CSEM_REG_SLOW_DBG_ALM_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DBG_ALM_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is almost empty in sem_slow_ls_dbg (31 entry inside\n     fifo)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200274UL: // (CSEM_REG_SLOW_DBG_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DBG_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is almost empty in sem_slow_ls_dbg according to \n    configuration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200278UL: // (CSEM_REG_SLOW_DBG_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DBG_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is empty in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20027cUL: // (CSEM_REG_SLOW_DBG_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DBG_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is full in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200280UL: // (CSEM_REG_SLOW_DRA_FIN_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DRA_FIN_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIN fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200284UL: // (CSEM_REG_SLOW_DRA_FIN_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DRA_FIN_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIN fifo is full in sem_slow_dra_sync (never may be active\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200288UL: // (CSEM_REG_SLOW_DRA_INT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DRA_INT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20028cUL: // (CSEM_REG_SLOW_DRA_INT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DRA_INT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt fifo is full in sem_slow_dra_int\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200290UL: // (CSEM_REG_SLOW_DRA_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DRA_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_RD pop fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200294UL: // (CSEM_REG_SLOW_DRA_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DRA_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_RD pop fifo is full in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200298UL: // (CSEM_REG_SLOW_DRA_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DRA_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_WR push fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20029cUL: // (CSEM_REG_SLOW_DRA_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_DRA_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_WR push fifo is full in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002a0UL: // (CSEM_REG_SLOW_EXT_STORE_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_EXT_STORE_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002a4UL: // (CSEM_REG_SLOW_EXT_STORE_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_EXT_STORE_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002a8UL: // (CSEM_REG_SLOW_RAM0_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_RAM0_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM0 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002acUL: // (CSEM_REG_SLOW_RAM0_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_RAM0_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM0 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002b0UL: // (CSEM_REG_SLOW_RAM0_WR_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_RAM0_WR_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is almost full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002b4UL: // (CSEM_REG_SLOW_RAM0_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_RAM0_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM0 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002b8UL: // (CSEM_REG_SLOW_RAM0_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_RAM0_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM0 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002bcUL: // (CSEM_REG_SLOW_RAM1_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_RAM1_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM1 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002c0UL: // (CSEM_REG_SLOW_RAM1_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_RAM1_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM1 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002c4UL: // (CSEM_REG_SLOW_RAM1_WR_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_RAM1_WR_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is almost full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002c8UL: // (CSEM_REG_SLOW_RAM1_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_RAM1_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002ccUL: // (CSEM_REG_SLOW_RAM1_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SLOW_RAM1_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002d0UL: // (CSEM_REG_SYNC_DBG_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SYNC_DBG_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FAST SYNC FIFO is empty in sem_slow_ls_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002d4UL: // (CSEM_REG_SYNC_DBG_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nCSEM_REG_SYNC_DBG_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FAST SYNC FIFO is full in sem_slow_ls_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002d8UL: // (CSEM_REG_THREAD_ERROR: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nCSEM_REG_THREAD_ERROR (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Thread error indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002dcUL: // (CSEM_REG_THREAD_OVERRUN_NUM: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nCSEM_REG_THREAD_OVERRUN_NUM (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Threads are sleeping in passive buffer more than ~csem_registers_thread_inter_cnt\n    .thread_inter_cnt number of cycles\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002e0UL: // (CSEM_REG_THREAD_RDY: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nCSEM_REG_THREAD_RDY (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Thread ready indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002e4UL: // (CSEM_REG_THREADS_LIST: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nCSEM_REG_THREADS_LIST (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: List of free threads . There is a bit per thread.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002e8UL: // (CSEM_REG_WB_MSB: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nCSEM_REG_WB_MSB (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Reset value of this register is right when was not read to\n     ~csem_registers_fic0_fifo.fic0_fifo or ~csem_registers_fic1_fifo.fic1_fifo\n     or ~csem_registers_passive_buffer.passive_buffer. For read from ~csem_registers_passive_buffer\n    .passive_buffer :b0- parity0; b1 parity1. For  read from ~csem_registers_fic0_fifo\n    .fic0_fifo and ~csem_registers_fic1_fifo.fic1_fifo :b1=0 data from ~csem_registers_fic0_fifo\n    .fic0_fifo and ~csem_registers_fic1_fifo.fic1_fifo is valid; b1 =1 ~csem_registers_fic0_fifo\n    .fic0_fifo and ~csem_registers_fic1_fifo.fic1_fifo is empty and data from\n     it must be equal to 0; b0 - parity from ~csem_registers_fic0_fifo.fic0_fifo\n     and ~csem_registers_fic1_fifo.fic1_fifo\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200300UL: // (CSEM_REG_FIC0_FIFO: ACCESS:WB_R  DataWidth:0x80 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC0_FIFO (ACCESS:WB_R  DataWidth:0x80 )" \
				"\nDescription: Debug only. FIC0_fifo: b[127:0] data; b128-parity;b129=1- \n    fifo empty;b129=0-data is valid\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200320UL: // (CSEM_REG_FIC1_FIFO: ACCESS:WB_R  DataWidth:0x80 ) 
			sprintf(disp_str, "\nCSEM_REG_FIC1_FIFO (ACCESS:WB_R  DataWidth:0x80 )" \
				"\nDescription: Debug only. FIC1_fifo read for debugging mode; b[127:0] data\n    ; b128-parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200340UL: // (CSEM_REG_FIN_COMMAND: ACCESS:WB_R  DataWidth:0x6d ) 
			sprintf(disp_str, "\nCSEM_REG_FIN_COMMAND (ACCESS:WB_R  DataWidth:0x6d )" \
				"\nDescription: last fin command that was read from fifo. Its spelling in\n     ~csem_registers_fin_fifo.fin_fifo register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200360UL: // (CSEM_REG_FIN_FIFO: ACCESS:WB_R  DataWidth:0x6d ) 
			sprintf(disp_str, "\nCSEM_REG_FIN_FIFO (ACCESS:WB_R  DataWidth:0x6d )" \
				"\nDescription: Debug only. FIn FIFO.  [5:0]   start_rp_foc3; [11:6]    start_rp_foc2\n    ;  [17:12]   start_rp_foc1; [23:18]   start_rp_foc0;  [29:24]   end_rp_foc3\n    ; [35:30]   end_rp_foc2; [41:36]   end_rp_foc1; [47:42]   end_rp_foc0; [\n    53:48]   lowest rp; [59:54]   highest rp; [65:60]   store start rp; [71\n    :66]   store end rp; [77:72]   load start rp; [83:78]   load end rp; [\n    85:84]   priority; [101:86]  pram address; [102] pas; [103] foc3; [104\n    ] foc2; [105] foc1; [106] foc0; [107] release; [108] fin; [109]=1- fin\n     fifo is empty; 0 - data[108:0] is valid\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200380UL: // (CSEM_REG_VFPF_ERR_NUM: ACCESS:W  DataWidth:0x7 ) 
			sprintf(disp_str, "\nCSEM_REG_VFPF_ERR_NUM (ACCESS:W  DataWidth:0x7 )" \
				"\nDescription: VF or PF ID for reset error bit. Values 0-63 reset error \n    bit for 64 VF; values 64-67 reset error for 4 PF; values 68-127 are not\n     valid.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200388UL: // (CSEM_REG_VF_ERR_VECTOR_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_VF_ERR_VECTOR_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: VF/PF error bitmap vector [0-31]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200390UL: // (CSEM_REG_VF_ERR_VECTOR_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_VF_ERR_VECTOR_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: VF/PF error bitmap vector [32-63]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200398UL: // (CSEM_REG_PF_ERR_VECTOR: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nCSEM_REG_PF_ERR_VECTOR (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: VF/PF error bitmap vector [0-7]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2003a0UL: // (CSEM_REG_THREAD_SET_NUM: ACCESS:W  DataWidth:0x5 ) 
			sprintf(disp_str, "\nCSEM_REG_THREAD_SET_NUM (ACCESS:W  DataWidth:0x5 )" \
				"\nDescription: Thread ID. Write thread ID will set ready indication for \n    this thread ID\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200400UL: // (CSEM_REG_INT_TABLE: ACCESS:RW  DataWidth:0xf ) 
			sprintf(disp_str, "\nCSEM_REG_INT_TABLE (ACCESS:RW  DataWidth:0xf )" \
				"\nDescription: Interrupt table  Read and write access to it is not possible\n     in the middle of the work\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202000UL: // (CSEM_REG_PASSIVE_BUFFER: ACCESS:WB  DataWidth:0x80 ) 
			sprintf(disp_str, "\nCSEM_REG_PASSIVE_BUFFER (ACCESS:WB  DataWidth:0x80 )" \
				"\nDescription: Debug only. Passive buffer memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x204000UL: // (CSEM_REG_PASSIVE_BUFFER_MSB: ACCESS:WB  DataWidth:0x80 ) 
			sprintf(disp_str, "\nCSEM_REG_PASSIVE_BUFFER_MSB (ACCESS:WB  DataWidth:0x80 )" \
				"\nDescription: Debug only. Passive buffer memory MSB that starts from row\n     512 of passive buffer till row 639\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x220000UL: // (CSEM_REG_FAST_MEMORY: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nCSEM_REG_FAST_MEMORY (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This address space contains all registers and memories that\n     are placed in SEM_FAST block. The SEM_FAST registers are described in\n     appendix B. In order to access the SEM_FAST registers the base address\n     CSEM_REGISTERS_FAST_MEMORY (Offset: 0x220000) should be added to each\n     SEM_FAST register offset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x240000UL: // (CSEM_REG_PRAM: ACCESS:WB  DataWidth:0x2e ) 
			sprintf(disp_str, "\nCSEM_REG_PRAM (ACCESS:WB  DataWidth:0x2e )" \
				"\nDescription: pram memory. B45 is parity; b[44:0] - data.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200144UL: // (CSEM_REG_CSEM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200154UL: // (CSEM_REG_CSEM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002ecUL: // (CSEM_REG_CSEM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200384UL: // (CSEM_REG_CSEM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20038cUL: // (CSEM_REG_CSEM_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200394UL: // (CSEM_REG_CSEM_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20039cUL: // (CSEM_REG_CSEM_UNUSED_EMPTY_6: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_6 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2003a4UL: // (CSEM_REG_CSEM_UNUSED_EMPTY_7: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_7 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2003b0UL: // (CSEM_REG_CSEM_UNUSED_EMPTY_8: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_8 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200800UL: // (CSEM_REG_CSEM_UNUSED_EMPTY_9: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_9 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x204800UL: // (CSEM_REG_CSEM_UNUSED_EMPTY_10: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nCSEM_REG_CSEM_UNUSED_EMPTY_10 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc000UL: // (DBG_REG_CLIENT_ENABLE: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nDBG_REG_CLIENT_ENABLE (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: debug only:                            These bits are enables\n     to client interfaces as follows: 0b000000001 - rx; 0b000000010 - nm; \n    0b000000100 - ulp; 0b000001000 - tx; 0b000010000 - cpt; 0b000100000 - \n    usem; 0b001000000 - xsem; 0b010000000 - tsem; 0b100000000 - csem;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc004UL: // (DBG_REG_OUTPUT_ENABLE: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_OUTPUT_ENABLE (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tNIG_ENABLE :"
				, (u32_t)((data & DBG_OUTPUT_ENABLE_REG_NIG_ENABLE) >> 2)
				, "\n\tPCI_DATA_ENABLE :"
				, (u32_t)((data & DBG_OUTPUT_ENABLE_REG_PCI_DATA_ENABLE) >> 1)
				, "\n\tPCI_REQ_ENABLE :"
				, (u32_t)((data & DBG_OUTPUT_ENABLE_REG_PCI_REQ_ENABLE) >> 0));
			break;
		case 0xc008UL: // (DBG_REG_CALENDAR_SLOT0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_SLOT0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: debug only:                            These bits are a client\n     index for slot 0 in calendar as follows: 0b0000      -     rx;          \n    0b0001      -     nm;          0b0010      -     ulp;          0b0011      -     \n    tx;          0b0100      -     cpt;          0b0101      -     usem;          \n    0b0110      -     xsem;          0b0111      -     tsem;          0b1000\n          -     csem;     0b1001      -     cpu;       0b1010      -     joint\n     HW;      0b1111      -     None;         \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc00cUL: // (DBG_REG_CALENDAR_SLOT1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_SLOT1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: debug only:                            These bits are a client\n     index for slot 1 in calendar as follows: 0b0000      -     rx;          \n    0b0001      -     nm;          0b0010      -     ulp;          0b0011      -     \n    tx;          0b0100      -     cpt;          0b0101      -     usem;          \n    0b0110      -     xsem;          0b0111      -     tsem;          0b1000\n          -     csem;     0b1001      -     cpu;       0b1010      -     joint\n     HW;      0b1111      -     None; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc010UL: // (DBG_REG_CALENDAR_SLOT2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_SLOT2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: debug only:                            These bits are a client\n     index for slot 2 in calendar as follows: 0b0000      -     rx;          \n    0b0001      -     nm;          0b0010      -     ulp;          0b0011      -     \n    tx;          0b0100      -     cpt;          0b0101      -     usem;          \n    0b0110      -     xsem;          0b0111      -     tsem;          0b1000\n          -     csem;     0b1001      -     cpu;       0b1010      -     joint\n     HW;      0b1111      -     None; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc014UL: // (DBG_REG_CALENDAR_SLOT3: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_SLOT3 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: debug only:                            These bits are a client\n     index for slot 3 in calendar as follows: 0b0000      -     rx;          \n    0b0001      -     nm;          0b0010      -     ulp;          0b0011      -     \n    tx;          0b0100      -     cpt;          0b0101      -     usem;          \n    0b0110      -     xsem;          0b0111      -     tsem;          0b1000\n          -     csem;     0b1001      -     cpu;       0b1010      -     joint\n     HW;      0b1111      -     None; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc018UL: // (DBG_REG_CALENDAR_SLOT4: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_SLOT4 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: debug only:                            These bits are a client\n     index for slot 4 in calendar as follows: 0b0000      -     rx;          \n    0b0001      -     nm;          0b0010      -     ulp;          0b0011      -     \n    tx;          0b0100      -     cpt;          0b0101      -     usem;          \n    0b0110      -     xsem;          0b0111      -     tsem;          0b1000\n          -     csem;     0b1001      -     cpu;       0b1010      -     joint\n     HW;      0b1111      -     None; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc01cUL: // (DBG_REG_CALENDAR_SLOT5: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_SLOT5 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: debug only:                            These bits are a client\n     index for slot 5 in calendar as follows: 0b0000      -     rx;          \n    0b0001      -     nm;          0b0010      -     ulp;          0b0011      -     \n    tx;          0b0100      -     cpt;          0b0101      -     usem;          \n    0b0110      -     xsem;          0b0111      -     tsem;          0b1000\n          -     csem;     0b1001      -     cpu;       0b1010      -     joint\n     HW;      0b1111      -     None; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc020UL: // (DBG_REG_CALENDAR_SLOT6: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_SLOT6 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: debug only:                            These bits are a client\n     index for slot 6 in calendar as follows: 0b0000      -     rx;          \n    0b0001      -     nm;          0b0010      -     ulp;          0b0011      -     \n    tx;          0b0100      -     cpt;          0b0101      -     usem;          \n    0b0110      -     xsem;          0b0111      -     tsem;          0b1000\n          -     csem;     0b1001      -     cpu;       0b1010      -     joint\n     HW;      0b1111      -     None; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc024UL: // (DBG_REG_CALENDAR_SLOT7: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_SLOT7 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: debug only:                            These bits are a client\n     index for slot 7 in calendar as follows: 0b0000      -     rx;          \n    0b0001      -     nm;          0b0010      -     ulp;          0b0011      -     \n    tx;          0b0100      -     cpt;          0b0101      -     usem;          \n    0b0110      -     xsem;          0b0111      -     tsem;          0b1000\n          -     csem;     0b1001      -     cpu;       0b1010      -     joint\n     HW;      0b1111      -     None; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc028UL: // (DBG_REG_CALENDAR_PACE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_PACE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: debug only:                            This bit indicates\n     the calendar pacing which is the number of cycles the calendar stays \n    on the same slot before moving to the next slot to support lower rates\n     (During the number of cycles configured in the #dbg_registers_calendar_pace\n     only one cycle can be valid)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc02cUL: // (DBG_REG_FRAMING_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_FRAMING_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: debug only:                            These bits indicate\n     the framing mode:                           0  -  tdm (32/64) mode;            \n    1  -  64 bits mode;                   2  -  24 bits mode;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc030UL: // (DBG_REG_DEBUG_TARGET: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_DEBUG_TARGET (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: debug only:                            These bits indicates\n     the target of the debug data:                    0 - internal buffer;                    \n    1 - NIG;                                  2 - PCI;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc034UL: // (DBG_REG_FULL_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FULL_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicates\n     whether data will be wrapped (oldest data is thrown) or overflowed-one\n     shot (newest data is thrown) as follows:                                (\n    a) When ~dbg_registers_debug_target=2/0 (PCI/internal buffer):              \n    1 - wrap;                                 0 - One Shot;                       (\n    b) When ~dbg_registers_debug_target=1 (NIG):                                     \n    1 - constant send;                   0 - One Shot;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc038UL: // (DBG_REG_ETHERNET_HDR0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_ETHERNET_HDR0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the ethernet header: bytes 0:3; The ethernet header is 14\n     bytes length which are being added at the beginning of the packet in \n    network order\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc03cUL: // (DBG_REG_ETHERNET_HDR1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_ETHERNET_HDR1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the ethernet header: bytes 4:7; The ethernet header is 14\n     bytes length which are being added at the beginning of the packet in \n    network order\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc040UL: // (DBG_REG_ETHERNET_HDR2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_ETHERNET_HDR2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the ethernet header: bytes 8:11; The ethernet header is \n    14 bytes length which are being added at the beginning of the packet in\n     network order\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc044UL: // (DBG_REG_ETHERNET_HDR3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nDBG_REG_ETHERNET_HDR3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the ethernet header: bytes 12:13; The ethernet header is\n     14 bytes length which are being added at the beginning of the packet \n    in network order\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc048UL: // (DBG_REG_PCI_EXT_BUFFER_STRT_ADDR_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_PCI_EXT_BUFFER_STRT_ADDR_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: debug only:                            LSB of external PCI\n     buffer start address; MUST be configured BEFORE pci_req_credit is configured\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc04cUL: // (DBG_REG_PCI_EXT_BUFFER_STRT_ADDR_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_PCI_EXT_BUFFER_STRT_ADDR_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: debug only:                            MSB of external PCI\n     buffer start address; MUST be configured BEFORE pci_req_credit is configured\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc058UL: // (DBG_REG_PCI_VQ_ID: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_PCI_VQ_ID (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: debug only:                            This bit is a handle\n     given to the PCI block to refer to this request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc05cUL: // (DBG_REG_CPU_DEBUG_FRAME: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_DEBUG_FRAME (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicate \n    the frame signal of the debug data that arrives from the CPU\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc060UL: // (DBG_REG_CPU_TIMEOUT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_TIMEOUT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            Timeout operation \n    initiated by the CPU; prior to initiating a timeout event all inputs must\n     be disabled; Timeout signal must stay high until all data was fully sent\n     to nig or pci and the internal buffer is empty\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc064UL: // (DBG_REG_DBG_BLOCK_ON: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_DBG_BLOCK_ON (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit enables the\n     operation of the debug block; This bit should be set upon completion \n    of all required configuration for the dbg block and shouldn't be reset\n     during all operational phase of the block;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc068UL: // (DBG_REG_NUM_OF_QWORDS_SENT0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_NUM_OF_QWORDS_SENT0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: debug only:                            These bits represent\n     the total number of qwords sent from the dbg block to output interface\n     (NIG/PCI) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc06cUL: // (DBG_REG_TDM64_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TDM64_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicates\n     tdm working mode as follows:                     (a) 1 -  enables tdm64\n     mechanism which allows 64 bit of data. In this case the 32 lsb are chosen\n     by the tdm mechanism using slots 0..6 and the 32 msb are taken from the\n     joint HW and are marked as slot number 7;                 (b) 0 - disables\n     the tdm64 mechanism hence allowing 32 bits of data only; the tdm operation\n     mode allows only the 32 lsb of data. In this case the data is chosen \n    by the tdm mechanism using slots 0..7  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc070UL: // (DBG_REG_IMMEDIATE_ACK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_IMMEDIATE_ACK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicates\n     that an immediate ack should be sent to the CPU upon writing to ~dbg_registers_cpu_debug_data\n     and an interrupt will be sent when a new write can be issued (after the\n     data was actually taken); If 0 then the ack will be delayed until a new\n     write can be issued by the CPU (after the data was actually taken)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc074UL: // (DBG_REG_NO_GRANT_ON_FULL: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_NO_GRANT_ON_FULL (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicate \n    whether grant will be issued by the dbg block towards the storms in case\n     the internal buffer is almost full as follows:                                   (\n    a) 1 -  no grants will be made to the storms when the internal buffer \n    is almost full. When the buffer will be partialy freed (enough for a complete\n     data chunk) then grant is resumed;  (b) 0 - grant is supplied every time\n     the matching storms's slot is chosen disregarding the volume status of\n     the internal buffer. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc078UL: // (DBG_REG_FULL_BUFFER_THR: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nDBG_REG_FULL_BUFFER_THR (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the internal buffer almost full threshold used for deciding\n     when ~dbg_registers_dbg_buffer_full output should go high/low; holds \n    the number of 128 bit free lines in the internal buffer under which the\n     full would go high; not applicable when ~dbg_registers_debug_target=0\n     (internal buffer) and ~dbg_registers_full_mode=1 (wrap);\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc07cUL: // (DBG_REG_PCI_LOGIC_ADDR: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_PCI_LOGIC_ADDR (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicates\n     logical/physical address in PCI request as follows:                 (\n    a) 1 - logical address;           (b) 0 - physical address;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc080UL: // (DBG_REG_PATTERN_RECOGNITION_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_PATTERN_RECOGNITION_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            For pattern recognition\n     usage: This bit indicates whether the pattern recognition feature is \n    disabled/enabled as follows:   (a) 1 - disabled;                     (\n    b) 0 - enabled;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc084UL: // (DBG_REG_PATTERN_RECOGNITION_STORAGE_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_PATTERN_RECOGNITION_STORAGE_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            For pattern recognition\n     usage: This bit indicates the trigger behavior of the pattern recognition\n     feature as follows:  (a) 1 - stop debug data storgae when the expected\n     pattern is initially recognized;                (b) 0 - start debug data\n     storage when the expected pattern is initially recognized\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc088UL: // (DBG_REG_PATTERN_RECOGNITION_FILTER: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_PATTERN_RECOGNITION_FILTER (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only:                            For pattern recognition\n     usage:  This bit indicates whether data is continously stored in the \n    dbg block until/from pattern recognition initial event; or stored only\n     in cycles of a pattern recognition event occurence as follows:             (\n    a) 1 - enable continuously data storage after/before first occurence of\n     pattern recognition;                           (b) 0 - enable data storage\n     only in cycles of a pttern recognition event occurence\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc08cUL: // (DBG_REG_DBG_INT_STS: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_DBG_INT_STS (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCPU_DATA_TAKEN_INTR :"
				, (u32_t)((data & DBG_DBG_INT_STS_REG_CPU_DATA_TAKEN_INTR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DBG_DBG_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc090UL: // (DBG_REG_DBG_INT_STS_CLR: ACCESS:RC  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_DBG_INT_STS_CLR (ACCESS:RC  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCPU_DATA_TAKEN_INTR :"
				, (u32_t)((data & DBG_DBG_INT_STS_CLR_REG_CPU_DATA_TAKEN_INTR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DBG_DBG_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc094UL: // (DBG_REG_DBG_INT_STS_WR: ACCESS:WR  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_DBG_INT_STS_WR (ACCESS:WR  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCPU_DATA_TAKEN_INTR :"
				, (u32_t)((data & DBG_DBG_INT_STS_WR_REG_CPU_DATA_TAKEN_INTR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DBG_DBG_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc098UL: // (DBG_REG_DBG_INT_MASK: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_DBG_INT_MASK (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCPU_DATA_TAKEN_INTR :"
				, (u32_t)((data & DBG_DBG_INT_MASK_REG_CPU_DATA_TAKEN_INTR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DBG_DBG_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc09cUL: // (DBG_REG_DBG_PRTY_STS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_DBG_PRTY_STS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & DBG_DBG_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0xc0a0UL: // (DBG_REG_DBG_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_DBG_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & DBG_DBG_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0xc0a4UL: // (DBG_REG_DBG_PRTY_STS_WR: ACCESS:WR  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_DBG_PRTY_STS_WR (ACCESS:WR  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & DBG_DBG_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0xc0a8UL: // (DBG_REG_DBG_PRTY_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_DBG_PRTY_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & DBG_DBG_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0xd010UL: // (DBG_REG_TRIGGER_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 0 - trigger machine is off (all data will bypass the \n    triggering machine) in this mode trigger_event is never asserted. (b) \n    1 - trigger machine is on; before AND/OR upon trigger_event assertion \n    data will be recorded according to the configuration of the recording \n    mode before/upon triggering event: rcrd_on_window_pre_trgr_evnt_mode  & \n    rcrd_on_window_post_trgr_evnt_mode\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd014UL: // (DBG_REG_TRIGGER_INTERLEAVED_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INTERLEAVED_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 0 - triggering interleaved messages is disbaled.             (\n    b) 1 - triggering interleaved messages is enabled; will be used for triggering\n     on recorded handler messages.                  NOTE: triggering is possible\n     on one level depth of interleaved messages; i.e. if message B is interleaved\n     within message A then it is ok; However if message C is interleaved within\n     message B and message B is interleaved within message A this scenario\n     is NOT supported\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd018UL: // (DBG_REG_TRIGGER_STATE_SLOTS_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SLOTS_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Valid slots for triggering machine in relevant state. Data\n     from clients which are not in those slots will be dropped (not compared\n    ). b0 - slot0; b1 - slot1;. . .;b7 - slot7;         NOTE:  1. NA when \n    framing mode is tdm64 and trigger_state_valid_seli=1; in that case the\n     only client is the JointHW (upper 32 bits of the calendar) and hence \n    the value of trigger_state_slotsi  is disregarded. Each valid cycle will\n     be compared.                    2. all set bits MUST contain the same\n     client \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd01cUL: // (DBG_REG_TRIGGER_STATE_SLOTS_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SLOTS_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Valid slots for triggering machine in relevant state. Data\n     from clients which are not in those slots will be dropped (not compared\n    ). b0 - slot0; b1 - slot1;. . .;b7 - slot7;         NOTE:  1. NA when \n    framing mode is tdm64 and trigger_state_valid_seli=1; in that case the\n     only client is the JointHW (upper 32 bits of the calendar) and hence \n    the value of trigger_state_slotsi  is disregarded. Each valid cycle will\n     be compared.                    2. all set bits MUST contain the same\n     client \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd020UL: // (DBG_REG_TRIGGER_STATE_SLOTS_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SLOTS_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Valid slots for triggering machine in relevant state. Data\n     from clients which are not in those slots will be dropped (not compared\n    ). b0 - slot0; b1 - slot1;. . .;b7 - slot7;         NOTE:  1. NA when \n    framing mode is tdm64 and trigger_state_valid_seli=1; in that case the\n     only client is the JointHW (upper 32 bits of the calendar) and hence \n    the value of trigger_state_slotsi  is disregarded. Each valid cycle will\n     be compared.                    2. all set bits MUST contain the same\n     client \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd024UL: // (DBG_REG_TRIGGER_STATE_VALID_SEL_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_VALID_SEL_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When working in tdm64 mode there are 2 valid signals as follows\n    :               (1) Valid[3] - validates data[63:32]              (2) \n    Valid[0] - validates data[31:0]          (a) 1 - use valid[3] (compared\n     data is data[63:32] in that case);    (b) 0 - use valid[0] (compared \n    data is data[31:0] in that case).      NOTE:                               \n    1. in each state the triggering machine compares the constraints to single\n     data source only (which means that only data[31:0] / data[63:32] will\n     be compared in each state)               2. if 64 bit framing mode OR\n     tdm32 framing mode then this bit is not relevant; in that case only valid\n    [0] & data[31:0]/data[63:0] are referred.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd028UL: // (DBG_REG_TRIGGER_STATE_VALID_SEL_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_VALID_SEL_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When working in tdm64 mode there are 2 valid signals as follows\n    :               (1) Valid[3] - validates data[63:32]              (2) \n    Valid[0] - validates data[31:0]          (a) 1 - use valid[3] (compared\n     data is data[63:32] in that case);    (b) 0 - use valid[0] (compared \n    data is data[31:0] in that case).      NOTE:                               \n    1. in each state the triggering machine compares the constraints to single\n     data source only (which means that only data[31:0] / data[63:32] will\n     be compared in each state)               2. if 64 bit framing mode OR\n     tdm32 framing mode then this bit is not relevant; in that case only valid\n    [0] & data[31:0]/data[63:0] are referred.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd02cUL: // (DBG_REG_TRIGGER_STATE_VALID_SEL_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_VALID_SEL_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When working in tdm64 mode there are 2 valid signals as follows\n    :               (1) Valid[3] - validates data[63:32]              (2) \n    Valid[0] - validates data[31:0]          (a) 1 - use valid[3] (compared\n     data is data[63:32] in that case);    (b) 0 - use valid[0] (compared \n    data is data[31:0] in that case).      NOTE:                               \n    1. in each state the triggering machine compares the constraints to single\n     data source only (which means that only data[31:0] / data[63:32] will\n     be compared in each state)               2. if 64 bit framing mode OR\n     tdm32 framing mode then this bit is not relevant; in that case only valid\n    [0] & data[31:0]/data[63:0] are referred.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd030UL: // (DBG_REG_TRIGGER_STATE_USE_BOTH_SETS_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_USE_BOTH_SETS_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - use both constraint set0 and constraint set1 in relevant\n     state.                                   (b) 0 - use only constraint \n    set0 in relevant state.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd034UL: // (DBG_REG_TRIGGER_STATE_USE_BOTH_SETS_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_USE_BOTH_SETS_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - use both constraint set0 and constraint set1 in relevant\n     state.                                   (b) 0 - use only constraint \n    set0 in relevant state.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd038UL: // (DBG_REG_TRIGGER_STATE_USE_BOTH_SETS_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_USE_BOTH_SETS_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - use both constraint set0 and constraint set1 in relevant\n     state.                                   (b) 0 - use only constraint \n    set0 in relevant state.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd03cUL: // (DBG_REG_TRIGGER_STATE_SET_NXT_STATE_0: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_NXT_STATE_0 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Next state in the fsm triggering machine if the referred \n    constraints set in the specified state are met. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd040UL: // (DBG_REG_TRIGGER_STATE_SET_NXT_STATE_1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_NXT_STATE_1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Next state in the fsm triggering machine if the referred \n    constraints set in the specified state are met. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd044UL: // (DBG_REG_TRIGGER_STATE_SET_NXT_STATE_2: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_NXT_STATE_2 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Next state in the fsm triggering machine if the referred \n    constraints set in the specified state are met. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd048UL: // (DBG_REG_TRIGGER_STATE_SET_NXT_STATE_3: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_NXT_STATE_3 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Next state in the fsm triggering machine if the referred \n    constraints set in the specified state are met. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd04cUL: // (DBG_REG_TRIGGER_STATE_SET_NXT_STATE_4: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_NXT_STATE_4 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Next state in the fsm triggering machine if the referred \n    constraints set in the specified state are met. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd050UL: // (DBG_REG_TRIGGER_STATE_SET_NXT_STATE_5: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_NXT_STATE_5 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Next state in the fsm triggering machine if the referred \n    constraints set in the specified state are met. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd054UL: // (DBG_REG_TRIGGER_STATE_SET_COUNT_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_COUNT_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Number of times that the referred constraints set should \n    be met prior to recognition (moving to next state).           NOTE: value\n     of 0 is NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd058UL: // (DBG_REG_TRIGGER_STATE_SET_COUNT_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_COUNT_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Number of times that the referred constraints set should \n    be met prior to recognition (moving to next state).           NOTE: value\n     of 0 is NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd05cUL: // (DBG_REG_TRIGGER_STATE_SET_COUNT_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_COUNT_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Number of times that the referred constraints set should \n    be met prior to recognition (moving to next state).           NOTE: value\n     of 0 is NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd060UL: // (DBG_REG_TRIGGER_STATE_SET_COUNT_3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_COUNT_3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Number of times that the referred constraints set should \n    be met prior to recognition (moving to next state).           NOTE: value\n     of 0 is NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd064UL: // (DBG_REG_TRIGGER_STATE_SET_COUNT_4: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_COUNT_4 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Number of times that the referred constraints set should \n    be met prior to recognition (moving to next state).           NOTE: value\n     of 0 is NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd068UL: // (DBG_REG_TRIGGER_STATE_SET_COUNT_5: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_COUNT_5 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Number of times that the referred constraints set should \n    be met prior to recognition (moving to next state).           NOTE: value\n     of 0 is NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd06cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd070UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd074UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd078UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd07cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd080UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd084UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd088UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd08cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_8: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_8 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd090UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_9: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_9 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd094UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_10: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_10 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd098UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_11: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_11 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd09cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_12: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_12 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0a0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_13: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_13 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0a4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_14: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_14 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0a8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_15: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_15 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0acUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_16: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_16 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0b0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_17: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_17 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0b4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_18: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_18 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0b8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_19: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_19 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0bcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_20: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_20 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0c0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_21: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_21 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0c4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_22: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_22 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0c8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_23: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_23 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The data that need to be compared. The 32 bit vector is determined\n     as follows:                        (a) data[31:0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                         OR                       (b) data[63:32\n    ] - if trigger_state_set_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0ccUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0d0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0d4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0d8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0dcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0e0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0e4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0e8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0ecUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0f0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0f4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0f8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd0fcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd100UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd104UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd108UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd10cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd110UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd114UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd118UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd11cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd120UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd124UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd128UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The frame that need to be compared. The 1 bit vector is determined\n     as follows:                    (a) frame[0] - if trigger_state_set_cnstr_offseti\n    [0] = 0                OR                  (b) frame[3] - if trigger_state_set_cnstr_offseti\n    [0] = 1             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd12cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd130UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd134UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd138UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd13cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd140UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd144UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd148UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd14cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_8: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_8 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd150UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_9: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_9 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd154UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_10: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_10 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd158UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_11: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_11 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd15cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_12: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_12 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd160UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_13: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_13 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd164UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_14: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_14 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd168UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_15: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_15 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd16cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_16: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_16 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd170UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_17: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_17 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd174UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_18: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_18 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd178UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_19: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_19 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd17cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_20: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_20 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd180UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_21: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_21 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd184UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_22: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_22 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd188UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_23: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_DATA_MASK_23 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd18cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd190UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd194UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd198UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd19cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1a0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1a4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1a8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1acUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1b0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1b4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1b8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1bcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1c0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1c4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1c8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1ccUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1d0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1d4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1d8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1dcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1e0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1e4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1e8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_FRAME_MASK_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 - the frame is compared; the mask is valid only for the equal operation\n     (trigger_state_set_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1ecUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1f0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1f4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_2: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_2 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1f8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_3: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_3 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd1fcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_4: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_4 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd200UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_5: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_5 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd204UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_6: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_6 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd208UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_7: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_7 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd20cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_8: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_8 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd210UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_9: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_9 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd214UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_10: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_10 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd218UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_11: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_11 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd21cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_12: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_12 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd220UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_13: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_13 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd224UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_14: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_14 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd228UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_15: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_15 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd22cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_16: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_16 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd230UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_17: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_17 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd234UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_18: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_18 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd238UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_19: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_19 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd23cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_20: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_20 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd240UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_21: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_21 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd244UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_22: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_22 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd248UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_23: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OPRTN_23 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     trigger_state_set_cnstr_datai and the actual data as follows: (a) 000\n     - equal;     (b) 001 - smaller than (<);     (c) 010 - smaller or equal\n     (<=);     (d) 011 greater or equal (>=);     (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd24cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_0: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_0 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_0 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_0_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_0) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_0 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_0_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_0) >> 0));
			break;
		case 0xd250UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_1: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_1 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_1 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_1_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_1) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_1 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_1_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_1) >> 0));
			break;
		case 0xd254UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_2: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_2 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_2 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_2_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_2) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_2 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_2_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_2) >> 0));
			break;
		case 0xd258UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_3: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_3 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_3 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_3_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_3) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_3 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_3_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_3) >> 0));
			break;
		case 0xd25cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_4: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_4 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_4 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_4_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_4) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_4 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_4_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_4) >> 0));
			break;
		case 0xd260UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_5: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_5 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_5 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_5_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_5) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_5 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_5_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_5) >> 0));
			break;
		case 0xd264UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_6: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_6 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_6 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_6_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_6) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_6 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_6_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_6) >> 0));
			break;
		case 0xd268UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_7: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_7 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_7 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_7_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_7) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_7 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_7_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_7) >> 0));
			break;
		case 0xd26cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_8: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_8 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_8 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_8_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_8) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_8 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_8_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_8) >> 0));
			break;
		case 0xd270UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_9: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_9 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_9 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_9_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_9) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_9 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_9_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_9) >> 0));
			break;
		case 0xd274UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_10: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_10 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_10 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_10_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_10) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_10 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_10_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_10) >> 0));
			break;
		case 0xd278UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_11: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_11 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_11 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_11_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_11) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_11 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_11_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_11) >> 0));
			break;
		case 0xd27cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_12: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_12 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_12 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_12_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_12) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_12 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_12_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_12) >> 0));
			break;
		case 0xd280UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_13: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_13 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_13 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_13_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_13) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_13 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_13_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_13) >> 0));
			break;
		case 0xd284UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_14: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_14 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_14 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_14_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_14) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_14 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_14_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_14) >> 0));
			break;
		case 0xd288UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_15: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_15 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_15 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_15_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_15) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_15 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_15_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_15) >> 0));
			break;
		case 0xd28cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_16: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_16 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_16 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_16_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_16) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_16 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_16_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_16) >> 0));
			break;
		case 0xd290UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_17: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_17 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_17 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_17_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_17) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_17 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_17_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_17) >> 0));
			break;
		case 0xd294UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_18: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_18 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_18 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_18_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_18) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_18 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_18_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_18) >> 0));
			break;
		case 0xd298UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_19: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_19 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_19 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_19_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_19) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_19 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_19_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_19) >> 0));
			break;
		case 0xd29cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_20: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_20 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_20 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_20_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_20) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_20 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_20_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_20) >> 0));
			break;
		case 0xd2a0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_21: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_21 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_21 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_21_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_21) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_21 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_21_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_21) >> 0));
			break;
		case 0xd2a4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_22: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_22 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_22 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_22_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_22) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_22 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_22_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_22) >> 0));
			break;
		case 0xd2a8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_23: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_RANGE_23 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_LSB_23 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_23_REG_TRIGGER_STATE_SET_CNSTR_RANGE_LSB_23) >> 5)
				, "\n\tTRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_23 :"
				, (u32_t)((data & DBG_TRIGGER_STATE_SET_CNSTR_RANGE_23_REG_TRIGGER_STATE_SET_CNSTR_RANGE_WIDTH_23) >> 0));
			break;
		case 0xd2acUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2b0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2b4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2b8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2bcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2c0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2c4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2c8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2ccUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_8: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_8 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2d0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_9: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_9 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2d4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_10: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_10 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2d8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_11: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_11 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2dcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_12: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_12 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2e0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_13: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_13 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2e4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_14: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_14 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2e8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_15: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_15 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2ecUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_16: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_16 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2f0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_17: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_17 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2f4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_18: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_18 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2f8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_19: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_19 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd2fcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_20: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_20 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd300UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_21: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_21 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd304UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_22: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_22 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd308UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_23: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_OFFSET_23 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    trigger_state_set_cnstr_offseti / 2 cycles after start of message.          \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd30cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd310UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd314UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd318UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd31cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd320UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd324UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd328UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd32cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd330UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd334UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd338UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd33cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd340UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd344UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd348UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd34cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd350UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd354UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd358UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd35cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd360UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd364UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd368UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_MUST_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector must exist as part of the message\n    .                             (b) 0: the above value vector is not mandatory\n     for the message; However at least one of those (low) constraints must\n     exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd36cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_0: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_0 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd370UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd374UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_2: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_2 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd378UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_3: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_3 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd37cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_4: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_4 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd380UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_5: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_5 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd384UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_6: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_6 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd388UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_7: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_7 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd38cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_8: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_8 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd390UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_9: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_9 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd394UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_10: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_10 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd398UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_11: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_11 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd39cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_12: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_12 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3a0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_13: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_13 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3a4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_14: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_14 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3a8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_15: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_15 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3acUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_16: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_16 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3b0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_17: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_17 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3b4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_18: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_18 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3b8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_19: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_19 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3bcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_20: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_20 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3c0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_21: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_21 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3c4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_22: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_22 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3c8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_23: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_INDRCT_23 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     trigger_state_set_cnstr_datai.               (b) 01: indirect: use the\n     recorded value from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).          (c) 10: indirect: use the recorded value from of fsm triggering\n     machine (trigger_indirect1_recorded_data).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3ccUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3d0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3d4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3d8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3dcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3e0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3e4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3e8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3ecUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3f0UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3f4UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3f8UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd3fcUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd400UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd404UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd408UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd40cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd410UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd414UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd418UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd41cUL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd420UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd424UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd428UL: // (DBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_SET_CNSTR_CYCLIC_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (trigger_state_set_cnstr_oprtni > 0)                                        (\n    a) 0 - regular comparsion (not cyclic): a>b if a-b >0                (\n    b) 1 - cyclic comparison: a>b if a-b<2^31-1 (half the range of 32 bit)                                    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd42cUL: // (DBG_REG_TRIGGER_STATE_MSG_LENGTH_ENABLE_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_MSG_LENGTH_ENABLE_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: use trigger_state_msg_lengthi  to determine message\n     boundary.   (b) 0: use trigger_state_valid_seli to determine which frame\n     (frame[0]/frame[3]) signals message boundary (end of message) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd430UL: // (DBG_REG_TRIGGER_STATE_MSG_LENGTH_ENABLE_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_MSG_LENGTH_ENABLE_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: use trigger_state_msg_lengthi  to determine message\n     boundary.   (b) 0: use trigger_state_valid_seli to determine which frame\n     (frame[0]/frame[3]) signals message boundary (end of message) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd434UL: // (DBG_REG_TRIGGER_STATE_MSG_LENGTH_ENABLE_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_MSG_LENGTH_ENABLE_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: use trigger_state_msg_lengthi  to determine message\n     boundary.   (b) 0: use trigger_state_valid_seli to determine which frame\n     (frame[0]/frame[3]) signals message boundary (end of message) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd438UL: // (DBG_REG_TRIGGER_STATE_MSG_LENGTH_0: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_MSG_LENGTH_0 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Message length-1  in terms of numbers of cycles.                  \n    NOTE:                                     (a) if for example trigger_state_msg_lengthi\n    =0 then Message length = 1 cycle. (b) if for example trigger_state_msg_lengthi\n    =1 then Message length = 2 cycles. etc.                               (\n    c) Applicable only when trigger_state_msg_length_eni = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd43cUL: // (DBG_REG_TRIGGER_STATE_MSG_LENGTH_1: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_MSG_LENGTH_1 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Message length-1  in terms of numbers of cycles.                  \n    NOTE:                                     (a) if for example trigger_state_msg_lengthi\n    =0 then Message length = 1 cycle. (b) if for example trigger_state_msg_lengthi\n    =1 then Message length = 2 cycles. etc.                               (\n    c) Applicable only when trigger_state_msg_length_eni = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd440UL: // (DBG_REG_TRIGGER_STATE_MSG_LENGTH_2: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_STATE_MSG_LENGTH_2 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Message length-1  in terms of numbers of cycles.                  \n    NOTE:                                     (a) if for example trigger_state_msg_lengthi\n    =0 then Message length = 1 cycle. (b) if for example trigger_state_msg_lengthi\n    =1 then Message length = 2 cycles. etc.                               (\n    c) Applicable only when trigger_state_msg_length_eni = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd444UL: // (DBG_REG_TRIGGER_INDIRECT0_STATE: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_STATE (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: If set then record data in relevant state;       If clear\n     then do not record data in relevant state;          b0: state0; b1: state1\n    ; b2: state2;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd448UL: // (DBG_REG_TRIGGER_INDIRECT0_OFFSET_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_OFFSET_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The offset in relevant state (fsm triggering machine) from\n     beginning of message to the data that should be recorded for indirect\n     value usage.               If set of constraints appear more than once\n     (trigger_state_set_counti >1) then cycle trigger_indirect0_offseti from\n     the last message will be recorded.                                 NOTE\n    : even offsets are for the 32 lsb as uneven offsets are for the 32 msb\n    . For example offset=0 is for cycle 0 for the 32 lsb; offset=1 is for \n    cycle 0 for the 32 msb; offset=2N is for cycle N for the 32 lsb; offset\n    =2N+1 is for cycle N for the 32 msb;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd44cUL: // (DBG_REG_TRIGGER_INDIRECT0_OFFSET_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_OFFSET_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The offset in relevant state (fsm triggering machine) from\n     beginning of message to the data that should be recorded for indirect\n     value usage.               If set of constraints appear more than once\n     (trigger_state_set_counti >1) then cycle trigger_indirect0_offseti from\n     the last message will be recorded.                                 NOTE\n    : even offsets are for the 32 lsb as uneven offsets are for the 32 msb\n    . For example offset=0 is for cycle 0 for the 32 lsb; offset=1 is for \n    cycle 0 for the 32 msb; offset=2N is for cycle N for the 32 lsb; offset\n    =2N+1 is for cycle N for the 32 msb;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd450UL: // (DBG_REG_TRIGGER_INDIRECT0_OFFSET_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_OFFSET_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The offset in relevant state (fsm triggering machine) from\n     beginning of message to the data that should be recorded for indirect\n     value usage.               If set of constraints appear more than once\n     (trigger_state_set_counti >1) then cycle trigger_indirect0_offseti from\n     the last message will be recorded.                                 NOTE\n    : even offsets are for the 32 lsb as uneven offsets are for the 32 msb\n    . For example offset=0 is for cycle 0 for the 32 lsb; offset=1 is for \n    cycle 0 for the 32 msb; offset=2N is for cycle N for the 32 lsb; offset\n    =2N+1 is for cycle N for the 32 msb;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd454UL: // (DBG_REG_TRIGGER_INDIRECT0_SHIFT_0: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_SHIFT_0 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Shift vector (byte resolution) for the data trigger_indirect0_recorded_data\n      The shift is implemented after the recording (after the registering) \n    of the indirect register is implemented.      The comparison with the \n    actual coming data is implemented on the shifted data. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd458UL: // (DBG_REG_TRIGGER_INDIRECT0_SHIFT_1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_SHIFT_1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Shift vector (byte resolution) for the data trigger_indirect0_recorded_data\n      The shift is implemented after the recording (after the registering) \n    of the indirect register is implemented.      The comparison with the \n    actual coming data is implemented on the shifted data. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd45cUL: // (DBG_REG_TRIGGER_INDIRECT0_SHIFT_2: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_SHIFT_2 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Shift vector (byte resolution) for the data trigger_indirect0_recorded_data\n      The shift is implemented after the recording (after the registering) \n    of the indirect register is implemented.      The comparison with the \n    actual coming data is implemented on the shifted data. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd460UL: // (DBG_REG_TRIGGER_INDIRECT0_MASK_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_MASK_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: If set then the relevant byte will be zeroed; if clear then\n     the relevant byte will be registered with its exact data.                   \n    NOTE:                                    (a) Mask is implemented prior\n     to registering the recorded data to trigger_indirect0_recorded_data;                                             (\n    b) The mask is implemented in byte resolution.                     (c) \n    b0: byte0; b1: byte1; b2: byte2; b3: byte2                  (d) useful\n     when trigger_state_set_cnstr_oprtni > 0 (>/</=</=>)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd464UL: // (DBG_REG_TRIGGER_INDIRECT0_MASK_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_MASK_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: If set then the relevant byte will be zeroed; if clear then\n     the relevant byte will be registered with its exact data.                   \n    NOTE:                                    (a) Mask is implemented prior\n     to registering the recorded data to trigger_indirect0_recorded_data;                                             (\n    b) The mask is implemented in byte resolution.                     (c) \n    b0: byte0; b1: byte1; b2: byte2; b3: byte2                  (d) useful\n     when trigger_state_set_cnstr_oprtni > 0 (>/</=</=>)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd468UL: // (DBG_REG_TRIGGER_INDIRECT0_MASK_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_MASK_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: If set then the relevant byte will be zeroed; if clear then\n     the relevant byte will be registered with its exact data.                   \n    NOTE:                                    (a) Mask is implemented prior\n     to registering the recorded data to trigger_indirect0_recorded_data;                                             (\n    b) The mask is implemented in byte resolution.                     (c) \n    b0: byte0; b1: byte1; b2: byte2; b3: byte2                  (d) useful\n     when trigger_state_set_cnstr_oprtni > 0 (>/</=</=>)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd46cUL: // (DBG_REG_TRIGGER_INDIRECT1_STATE: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_STATE (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: If set then record data in relevant state;       If clear\n     then do not record data in relevant state;          b0: state0; b1: state1\n    ; b2: state2;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd470UL: // (DBG_REG_TRIGGER_INDIRECT1_OFFSET_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_OFFSET_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The offset in relevant state (fsm triggering machine) from\n     beginning of message to the data that should be recorded for indirect\n     value usage.               If set of constraints appear more than once\n     (trigger_state_set_counti >1) then cycle trigger_indirect1_offseti from\n     the last message will be recorded.                                 NOTE\n    : even offsets are for the 32 lsb as uneven offsets are for the 32 msb\n    . For example offset=0 is for cycle 0 for the 32 lsb; offset=1 is for \n    cycle 0 for the 32 msb; offset=2N is for cycle N for the 32 lsb; offset\n    =2N+1 is for cycle N for the 32 msb;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd474UL: // (DBG_REG_TRIGGER_INDIRECT1_OFFSET_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_OFFSET_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The offset in relevant state (fsm triggering machine) from\n     beginning of message to the data that should be recorded for indirect\n     value usage.               If set of constraints appear more than once\n     (trigger_state_set_counti >1) then cycle trigger_indirect1_offseti from\n     the last message will be recorded.                                 NOTE\n    : even offsets are for the 32 lsb as uneven offsets are for the 32 msb\n    . For example offset=0 is for cycle 0 for the 32 lsb; offset=1 is for \n    cycle 0 for the 32 msb; offset=2N is for cycle N for the 32 lsb; offset\n    =2N+1 is for cycle N for the 32 msb;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd478UL: // (DBG_REG_TRIGGER_INDIRECT1_OFFSET_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_OFFSET_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The offset in relevant state (fsm triggering machine) from\n     beginning of message to the data that should be recorded for indirect\n     value usage.               If set of constraints appear more than once\n     (trigger_state_set_counti >1) then cycle trigger_indirect1_offseti from\n     the last message will be recorded.                                 NOTE\n    : even offsets are for the 32 lsb as uneven offsets are for the 32 msb\n    . For example offset=0 is for cycle 0 for the 32 lsb; offset=1 is for \n    cycle 0 for the 32 msb; offset=2N is for cycle N for the 32 lsb; offset\n    =2N+1 is for cycle N for the 32 msb;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd47cUL: // (DBG_REG_TRIGGER_INDIRECT1_SHIFT_0: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_SHIFT_0 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Shift vector (byte resolution) for the data trigger_indirect1_recorded_data\n      The shift is implemented after the recording (after the registering) \n    of the indirect register is implemented.      The comparison with the \n    actual coming data is implemented on the shifted data. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd480UL: // (DBG_REG_TRIGGER_INDIRECT1_SHIFT_1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_SHIFT_1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Shift vector (byte resolution) for the data trigger_indirect1_recorded_data\n      The shift is implemented after the recording (after the registering) \n    of the indirect register is implemented.      The comparison with the \n    actual coming data is implemented on the shifted data. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd484UL: // (DBG_REG_TRIGGER_INDIRECT1_SHIFT_2: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_SHIFT_2 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Shift vector (byte resolution) for the data trigger_indirect1_recorded_data\n      The shift is implemented after the recording (after the registering) \n    of the indirect register is implemented.      The comparison with the \n    actual coming data is implemented on the shifted data. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd488UL: // (DBG_REG_TRIGGER_INDIRECT1_MASK_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_MASK_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: If set then the relevant byte will be zeroed; if clear then\n     the relevant byte will be registered with its exact data.                   \n    NOTE:                                    (a) Mask is implemented prior\n     to registering the recorded data to trigger_indirect1_recorded_data;                                             (\n    b) The mask is implemented in byte resolution.                     (c) \n    b0: byte0; b1: byte1; b2: byte2; b3: byte2                  (d) useful\n     when trigger_state_set_cnstr_oprtni > 0 (>/</=</=>)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd48cUL: // (DBG_REG_TRIGGER_INDIRECT1_MASK_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_MASK_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: If set then the relevant byte will be zeroed; if clear then\n     the relevant byte will be registered with its exact data.                   \n    NOTE:                                    (a) Mask is implemented prior\n     to registering the recorded data to trigger_indirect1_recorded_data;                                             (\n    b) The mask is implemented in byte resolution.                     (c) \n    b0: byte0; b1: byte1; b2: byte2; b3: byte2                  (d) useful\n     when trigger_state_set_cnstr_oprtni > 0 (>/</=</=>)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd490UL: // (DBG_REG_TRIGGER_INDIRECT1_MASK_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_MASK_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: If set then the relevant byte will be zeroed; if clear then\n     the relevant byte will be registered with its exact data.                   \n    NOTE:                                    (a) Mask is implemented prior\n     to registering the recorded data to trigger_indirect1_recorded_data;                                             (\n    b) The mask is implemented in byte resolution.                     (c) \n    b0: byte0; b1: byte1; b2: byte2; b3: byte2                  (d) useful\n     when trigger_state_set_cnstr_oprtni > 0 (>/</=</=>)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd494UL: // (DBG_REG_FILTER_ENABLE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_ENABLE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00 - Filter off; in that case all data should be transmitted\n     to the internal buffer without any filtering implemented (data should\n     bypass filtering machine).                            (b) 01 - Filter\n     on prior (in time domain) to trigger_event (asserted by the triggering\n     machine block) only; When off (after trigger event) - data should be \n    transmitted to the internal buffer without any filtering. in this mode\n     trigger_enable must be set.                         (c) 10 - Filter on\n     upon trigger_event (asserted by the triggering machine) only. When off\n     (before trigger event) - data should be transmitted to the internal buffer\n     without any filtering.  in this mode trigger_enable must be set.                    (\n    d) 11 - Filter on - constant filtering; in this case the triggering event\n     (asserted by the triggering machine) is irrelevant.             \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd498UL: // (DBG_REG_FILTER_SLOTS: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_SLOTS (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Valid slots for filtering machine. Data from clients which\n     are not in those slots will be dropped (not compared).    b0 - slot0; \n    b1 - slot1;. . .;b7 - slot7;              NOTE:                                 (\n    1) NA when framing mode is tdm64 and filter_valid_sel=1; in that case \n    the only client is the JointHW (upper 32 bits of the calendar) and hence\n     the value of filter_slots is disregarded. Each valid cycle will be compared\n    .                             (2) all set bits MUST contain the same client\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd49cUL: // (DBG_REG_FILTER_VALID_SEL: ACCESS:RW  DataWidth:0x1  Description: When working in tdm64 mode there are 2 valid signals as follows:                                 (1) Valid[3] - validates data[63:32]                           (2) Valid[0] - validates data[31:0]                              ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_VALID_SEL (ACCESS:RW  DataWidth:0x1  Description: When working in tdm64 mode there are 2 valid signals as follows:                                 (1) Valid[3] - validates data[63:32]                           (2) Valid[0] - validates data[31:0]                              )" \
				"\nDescription:                            (a) 1 - use valid[3] (compared\n     data is data[63:32] in that case)                                   (\n    b) 0 - use valid[0] (compared data is data[31:0] in that case).                                            \n    NOTE:                                   (1) The filtering machine compares\n     the constraints to single data source only (which means that only data\n    [31:0] / data[63:32] will be compared in each state)                            (\n    2) if 64 bit framing mode OR tdm32 framing mode then this bit is not relevant\n    ; in that case only valid[0] & data[31:0]/data[63:0] are referred.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4a0UL: // (DBG_REG_FILTER_CNSTR_DATA_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_DATA_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The value that need to be compared.                            (\n    a) data[31:0] - if filter_cnstr_offseti[0] = 0;      OR                                        (\n    b) data[63:32] - if filter_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4a4UL: // (DBG_REG_FILTER_CNSTR_DATA_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_DATA_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The value that need to be compared.                            (\n    a) data[31:0] - if filter_cnstr_offseti[0] = 0;      OR                                        (\n    b) data[63:32] - if filter_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4a8UL: // (DBG_REG_FILTER_CNSTR_DATA_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_DATA_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The value that need to be compared.                            (\n    a) data[31:0] - if filter_cnstr_offseti[0] = 0;      OR                                        (\n    b) data[63:32] - if filter_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4acUL: // (DBG_REG_FILTER_CNSTR_DATA_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_DATA_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The value that need to be compared.                            (\n    a) data[31:0] - if filter_cnstr_offseti[0] = 0;      OR                                        (\n    b) data[63:32] - if filter_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4b0UL: // (DBG_REG_FILTER_CNSTR_FRAME_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_FRAME_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The value that need to be compared.                            (\n    a) frame[0] - if filter_cnstr_offseti[0] = 0;          OR                                       (\n    b) frame[3] - if filter_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4b4UL: // (DBG_REG_FILTER_CNSTR_FRAME_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_FRAME_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The value that need to be compared.                            (\n    a) frame[0] - if filter_cnstr_offseti[0] = 0;          OR                                       (\n    b) frame[3] - if filter_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4b8UL: // (DBG_REG_FILTER_CNSTR_FRAME_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_FRAME_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The value that need to be compared.                            (\n    a) frame[0] - if filter_cnstr_offseti[0] = 0;          OR                                       (\n    b) frame[3] - if filter_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4bcUL: // (DBG_REG_FILTER_CNSTR_FRAME_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_FRAME_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The value that need to be compared.                            (\n    a) frame[0] - if filter_cnstr_offseti[0] = 0;          OR                                       (\n    b) frame[3] - if filter_cnstr_offseti[0] = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4c0UL: // (DBG_REG_FILTER_CNSTR_DATA_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_DATA_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (filter_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4c4UL: // (DBG_REG_FILTER_CNSTR_DATA_MASK_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_DATA_MASK_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (filter_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4c8UL: // (DBG_REG_FILTER_CNSTR_DATA_MASK_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_DATA_MASK_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (filter_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4ccUL: // (DBG_REG_FILTER_CNSTR_DATA_MASK_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_DATA_MASK_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: If specific bit is 1 then the matched bit in the above data\n     vector is masked (not compared); the mask is valid only for the equal\n     operation (filter_cnstr_oprtni=000); i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4d0UL: // (DBG_REG_FILTER_CNSTR_FRAME_MASK_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_FRAME_MASK_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 -  the frame is compared;  NOTE:                                The\n     mask is valid only for the equal operation (trigger_cnstr_oprtni=000); \n    i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4d4UL: // (DBG_REG_FILTER_CNSTR_FRAME_MASK_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_FRAME_MASK_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 -  the frame is compared;  NOTE:                                The\n     mask is valid only for the equal operation (trigger_cnstr_oprtni=000); \n    i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4d8UL: // (DBG_REG_FILTER_CNSTR_FRAME_MASK_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_FRAME_MASK_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 -  the frame is compared;  NOTE:                                The\n     mask is valid only for the equal operation (trigger_cnstr_oprtni=000); \n    i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4dcUL: // (DBG_REG_FILTER_CNSTR_FRAME_MASK_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_FRAME_MASK_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1 - the frame is masked (not compared);                            (\n    b) 0 -  the frame is compared;  NOTE:                                The\n     mask is valid only for the equal operation (trigger_cnstr_oprtni=000); \n    i.e. not valid for </<=/>=/>.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4e0UL: // (DBG_REG_FILTER_CNSTR_OFFSET_0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_OFFSET_0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    filter_cnstr_offseti / 2  cycles after start of message (0..3 cycles --> \n    valid values: 0..7; The filtering is implemented according to the data\n     on the first 4 cycles only)                   NOTE:                                   (\n    a) even though the comparison is for the first 4 cycles only; the messages\n     length is up to 128 cycles.                 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4e4UL: // (DBG_REG_FILTER_CNSTR_OFFSET_1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_OFFSET_1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    filter_cnstr_offseti / 2  cycles after start of message (0..3 cycles --> \n    valid values: 0..7; The filtering is implemented according to the data\n     on the first 4 cycles only)                   NOTE:                                   (\n    a) even though the comparison is for the first 4 cycles only; the messages\n     length is up to 128 cycles.                 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4e8UL: // (DBG_REG_FILTER_CNSTR_OFFSET_2: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_OFFSET_2 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    filter_cnstr_offseti / 2  cycles after start of message (0..3 cycles --> \n    valid values: 0..7; The filtering is implemented according to the data\n     on the first 4 cycles only)                   NOTE:                                   (\n    a) even though the comparison is for the first 4 cycles only; the messages\n     length is up to 128 cycles.                 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4ecUL: // (DBG_REG_FILTER_CNSTR_OFFSET_3: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_OFFSET_3 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The above value vector (data & frame) should be compared \n    filter_cnstr_offseti / 2  cycles after start of message (0..3 cycles --> \n    valid values: 0..7; The filtering is implemented according to the data\n     on the first 4 cycles only)                   NOTE:                                   (\n    a) even though the comparison is for the first 4 cycles only; the messages\n     length is up to 128 cycles.                 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4f0UL: // (DBG_REG_FILTER_CNSTR_OPRTN_0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_OPRTN_0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     fliter_cnstr_datai and the actual data as follows: (a) 000 - equal; (\n    b) 001 - smaller than (<); (c)010 - smaller or equal (<=); (d) 011 greater\n     or equal (>=); (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4f4UL: // (DBG_REG_FILTER_CNSTR_OPRTN_1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_OPRTN_1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     fliter_cnstr_datai and the actual data as follows: (a) 000 - equal; (\n    b) 001 - smaller than (<); (c)010 - smaller or equal (<=); (d) 011 greater\n     or equal (>=); (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4f8UL: // (DBG_REG_FILTER_CNSTR_OPRTN_2: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_OPRTN_2 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     fliter_cnstr_datai and the actual data as follows: (a) 000 - equal; (\n    b) 001 - smaller than (<); (c)010 - smaller or equal (<=); (d) 011 greater\n     or equal (>=); (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd4fcUL: // (DBG_REG_FILTER_CNSTR_OPRTN_3: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_OPRTN_3 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The comparison operation that should be implemented between\n     fliter_cnstr_datai and the actual data as follows: (a) 000 - equal; (\n    b) 001 - smaller than (<); (c)010 - smaller or equal (<=); (d) 011 greater\n     or equal (>=); (e) 100 = greater than (>); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd500UL: // (DBG_REG_FILTER_CNSTR_RANGE_0: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_RANGE_0 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tFILTER_CNSTR_RANGE_LSB_0 :"
				, (u32_t)((data & DBG_FILTER_CNSTR_RANGE_0_REG_FILTER_CNSTR_RANGE_LSB_0) >> 5)
				, "\n\tFILTER_CNSTR_RANGE_WIDTH_0 :"
				, (u32_t)((data & DBG_FILTER_CNSTR_RANGE_0_REG_FILTER_CNSTR_RANGE_WIDTH_0) >> 0));
			break;
		case 0xd504UL: // (DBG_REG_FILTER_CNSTR_RANGE_1: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_RANGE_1 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tFILTER_CNSTR_RANGE_LSB_1 :"
				, (u32_t)((data & DBG_FILTER_CNSTR_RANGE_1_REG_FILTER_CNSTR_RANGE_LSB_1) >> 5)
				, "\n\tFILTER_CNSTR_RANGE_WIDTH_1 :"
				, (u32_t)((data & DBG_FILTER_CNSTR_RANGE_1_REG_FILTER_CNSTR_RANGE_WIDTH_1) >> 0));
			break;
		case 0xd508UL: // (DBG_REG_FILTER_CNSTR_RANGE_2: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_RANGE_2 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tFILTER_CNSTR_RANGE_LSB_2 :"
				, (u32_t)((data & DBG_FILTER_CNSTR_RANGE_2_REG_FILTER_CNSTR_RANGE_LSB_2) >> 5)
				, "\n\tFILTER_CNSTR_RANGE_WIDTH_2 :"
				, (u32_t)((data & DBG_FILTER_CNSTR_RANGE_2_REG_FILTER_CNSTR_RANGE_WIDTH_2) >> 0));
			break;
		case 0xd50cUL: // (DBG_REG_FILTER_CNSTR_RANGE_3: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_RANGE_3 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tFILTER_CNSTR_RANGE_LSB_3 :"
				, (u32_t)((data & DBG_FILTER_CNSTR_RANGE_3_REG_FILTER_CNSTR_RANGE_LSB_3) >> 5)
				, "\n\tFILTER_CNSTR_RANGE_WIDTH_3 :"
				, (u32_t)((data & DBG_FILTER_CNSTR_RANGE_3_REG_FILTER_CNSTR_RANGE_WIDTH_3) >> 0));
			break;
		case 0xd510UL: // (DBG_REG_FILTER_CNSTR_MUST_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_MUST_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector & frame must exist as part of\n     the message.                               (b) 0: the above data vector\n     & vector is not mandatory for the message; However at least one of those\n     (low) constraints must exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd514UL: // (DBG_REG_FILTER_CNSTR_MUST_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_MUST_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector & frame must exist as part of\n     the message.                               (b) 0: the above data vector\n     & vector is not mandatory for the message; However at least one of those\n     (low) constraints must exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd518UL: // (DBG_REG_FILTER_CNSTR_MUST_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_MUST_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector & frame must exist as part of\n     the message.                               (b) 0: the above data vector\n     & vector is not mandatory for the message; However at least one of those\n     (low) constraints must exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd51cUL: // (DBG_REG_FILTER_CNSTR_MUST_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_MUST_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: the above data vector & frame must exist as part of\n     the message.                               (b) 0: the above data vector\n     & vector is not mandatory for the message; However at least one of those\n     (low) constraints must exist as part of the message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd520UL: // (DBG_REG_FILTER_CNSTR_INDIRECT_0: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_INDIRECT_0 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     filter_cnstr_datai.                     (b) 01: indirect: use the recorded\n     value 0 from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).                                          (c) 10: indirect: use the \n    recorded value 1 from of fsm triggering machine (trigger_indirect1_recorded_data\n    ).                                          NOTE: if  filter_enable=11\n     OR filter_enable=01 then filter_cnstr_indirecti MUST be all 0 (need to\n     filter prior to triggering machine event and hence cannot use the recorded\n     data from state 0 in triggering machine) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd524UL: // (DBG_REG_FILTER_CNSTR_INDIRECT_1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_INDIRECT_1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     filter_cnstr_datai.                     (b) 01: indirect: use the recorded\n     value 0 from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).                                          (c) 10: indirect: use the \n    recorded value 1 from of fsm triggering machine (trigger_indirect1_recorded_data\n    ).                                          NOTE: if  filter_enable=11\n     OR filter_enable=01 then filter_cnstr_indirecti MUST be all 0 (need to\n     filter prior to triggering machine event and hence cannot use the recorded\n     data from state 0 in triggering machine) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd528UL: // (DBG_REG_FILTER_CNSTR_INDIRECT_2: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_INDIRECT_2 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     filter_cnstr_datai.                     (b) 01: indirect: use the recorded\n     value 0 from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).                                          (c) 10: indirect: use the \n    recorded value 1 from of fsm triggering machine (trigger_indirect1_recorded_data\n    ).                                          NOTE: if  filter_enable=11\n     OR filter_enable=01 then filter_cnstr_indirecti MUST be all 0 (need to\n     filter prior to triggering machine event and hence cannot use the recorded\n     data from state 0 in triggering machine) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd52cUL: // (DBG_REG_FILTER_CNSTR_INDIRECT_3: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_INDIRECT_3 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: (a) 00: direct: use the value which was configured in the\n     filter_cnstr_datai.                     (b) 01: indirect: use the recorded\n     value 0 from of fsm triggering machine (trigger_indirect0_recorded_data\n    ).                                          (c) 10: indirect: use the \n    recorded value 1 from of fsm triggering machine (trigger_indirect1_recorded_data\n    ).                                          NOTE: if  filter_enable=11\n     OR filter_enable=01 then filter_cnstr_indirecti MUST be all 0 (need to\n     filter prior to triggering machine event and hence cannot use the recorded\n     data from state 0 in triggering machine) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd530UL: // (DBG_REG_FILTER_CNSTR_CYCLIC_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_CYCLIC_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (filter_cnstr_oprtni > 0)             (a) 0 - regular comparsion\n     (not cyclic): a>b if a-b >0                (b) 1 - cyclic comparison: \n    a>b if a-b<2^31-1 (half the range of 32 bit)     \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd534UL: // (DBG_REG_FILTER_CNSTR_CYCLIC_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_CYCLIC_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (filter_cnstr_oprtni > 0)             (a) 0 - regular comparsion\n     (not cyclic): a>b if a-b >0                (b) 1 - cyclic comparison: \n    a>b if a-b<2^31-1 (half the range of 32 bit)     \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd538UL: // (DBG_REG_FILTER_CNSTR_CYCLIC_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_CYCLIC_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (filter_cnstr_oprtni > 0)             (a) 0 - regular comparsion\n     (not cyclic): a>b if a-b >0                (b) 1 - cyclic comparison: \n    a>b if a-b<2^31-1 (half the range of 32 bit)     \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd53cUL: // (DBG_REG_FILTER_CNSTR_CYCLIC_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_CNSTR_CYCLIC_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: refers the comparison which is implemented in case the operation\n     is NOT equal (filter_cnstr_oprtni > 0)             (a) 0 - regular comparsion\n     (not cyclic): a>b if a-b >0                (b) 1 - cyclic comparison: \n    a>b if a-b<2^31-1 (half the range of 32 bit)     \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd540UL: // (DBG_REG_FILTER_MSG_LENGTH_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_MSG_LENGTH_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: (a) 1: use filter_msg_length  to determine message boundary\n    . (b) 0: use filter_valid_sel           to determine which frame (frame\n    [0]/frame[3]) signals message boundary (end of message) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd544UL: // (DBG_REG_FILTER_MSG_LENGTH: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nDBG_REG_FILTER_MSG_LENGTH (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Message length-1  in terms of numbers of cycles.                  \n    NOTE:                                     (a) if for example filter_msg_length\n    =0 then Message length = 1 cycle.        (b) if for example filter_msg_lengthi\n    =1 then Message length = 2 cycles. etc.   (c) Applicable only when filter_msg_length_en\n     = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd548UL: // (DBG_REG_RCRD_ON_WINDOW_PRE_TRGR_EVNT_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_RCRD_ON_WINDOW_PRE_TRGR_EVNT_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Recording mode prior to trigger event:                                    (\n    a) 00 - record from time=0;     (b) 01 - record rcrd_on_window_pre_num_chunks\n     chunks (16 lines chunks within the internal buffer) to internal buffer\n     prior to triggering event;                                    (c) 10 - \n    Don't record prior to triggering event (drop data).     NOTE: applicable\n     only if trigger_enable=1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd54cUL: // (DBG_REG_RCRD_ON_WINDOW_POST_TRGR_EVNT_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_RCRD_ON_WINDOW_POST_TRGR_EVNT_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Recording mode upon trigger event:                                   (\n    a) 0- enable recording data upon triggering event; in that case record\n     for rcrd_on_window_post_num_cycles valid cycles upon the event;                                   (\n    b) 1 - disable recording data upon triggering event.             NOTE:                                  \n    applicable only if trigger_enable=1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd550UL: // (DBG_REG_RCRD_ON_WINDOW_PRE_NUM_CHUNKS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_RCRD_ON_WINDOW_PRE_NUM_CHUNKS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Number of chunks (chunk = 16 lines of 128 bit each within\n     the internal buffer) that should be recorded to the internal buffer prior\n     to triggering event.         NOTE:                                  (\n    1) applicable only when rcrd_on_window_pre_trgr_evnt_mode=01;                              (\n    2) valid values are 1..7;          (3) the data that will be stored in\n     the internal buffer is the most recent data prior to the triggering event\n    .                      (4) rcrd_on_window_pre_num_chunks represents the\n     maximum number of chunks that will be written to the internal buffer; \n    if from since time=0 until triggering event the amount of driven data \n    is smaller then the amount of the above value the amount of data stored\n     in the internal buffer will be smaller then the above value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd554UL: // (DBG_REG_RCRD_ON_WINDOW_POST_NUM_CYCLES: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_RCRD_ON_WINDOW_POST_NUM_CYCLES (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Number of valid cycles that should be recorded upon triggering\n     event.                      NOTE:                                   (\n    1) applicable only when rcrd_on_window_post_trgr_evnt_mode=0;                              (\n    2) value of 0xffffffff (maximum value) result in recording of unlimited\n     amount of cycles (infinite amount of cycles).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd558UL: // (DBG_REG_PCI_FUNC_NUM: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDBG_REG_PCI_FUNC_NUM (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: pci function number; for pci request interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd55cUL: // (DBG_REG_DBG_NM_MBIST1_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_DBG_NM_MBIST1_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd560UL: // (DBG_REG_NM_CLK_MBIST1_CNTRL_DBG_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_NM_CLK_MBIST1_CNTRL_DBG_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd564UL: // (DBG_REG_NM_CLK_MBIST1_CNTRL_DBG_STATUS_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_NM_CLK_MBIST1_CNTRL_DBG_STATUS_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd568UL: // (DBG_REG_NM_CLK_CP_MBIST1_CNTRL_DBG_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_NM_CLK_CP_MBIST1_CNTRL_DBG_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd56cUL: // (DBG_REG_INTERNAL_BUFFER_LSB_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_INTERNAL_BUFFER_LSB_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: tm port for the internal buffer lsb memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd570UL: // (DBG_REG_INTERNAL_BUFFER_MSB_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_INTERNAL_BUFFER_MSB_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: tm port for the internal buffer msb memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd574UL: // (DBG_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDBG_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: eco reserved register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd578UL: // (DBG_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd57cUL: // (DBG_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd580UL: // (DBG_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd584UL: // (DBG_REG_CPU_MBIST_MEMCTRL_3_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_3_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd588UL: // (DBG_REG_CPU_MBIST_MEMCTRL_4_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_4_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd58cUL: // (DBG_REG_CPU_MBIST_MEMCTRL_5_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_5_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd590UL: // (DBG_REG_CPU_MBIST_MEMCTRL_6_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_6_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd594UL: // (DBG_REG_CPU_MBIST_MEMCTRL_7_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_7_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd598UL: // (DBG_REG_CPU_MBIST_MEMCTRL_8_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_8_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd59cUL: // (DBG_REG_CPU_MBIST_MEMCTRL_0_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_0_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5a0UL: // (DBG_REG_CPU_MBIST_MEMCTRL_1_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_1_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5a4UL: // (DBG_REG_CPU_MBIST_MEMCTRL_2_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_2_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5a8UL: // (DBG_REG_CPU_MBIST_MEMCTRL_3_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_3_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5acUL: // (DBG_REG_CPU_MBIST_MEMCTRL_4_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_4_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5b0UL: // (DBG_REG_CPU_MBIST_MEMCTRL_5_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_5_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5b4UL: // (DBG_REG_CPU_MBIST_MEMCTRL_6_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_6_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5b8UL: // (DBG_REG_CPU_MBIST_MEMCTRL_7_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_7_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5bcUL: // (DBG_REG_CPU_MBIST_MEMCTRL_8_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_8_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5c0UL: // (DBG_REG_CPU_MBIST_MEMCTRL_9_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_9_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5c4UL: // (DBG_REG_CPU_MBIST_MEMCTRL_9_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_9_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5c8UL: // (DBG_REG_CPU_MBIST_MEMCTRL_10_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_10_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5ccUL: // (DBG_REG_CPU_MBIST_MEMCTRL_11_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_11_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5d0UL: // (DBG_REG_CPU_MBIST_MEMCTRL_10_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_10_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5d4UL: // (DBG_REG_CPU_MBIST_MEMCTRL_11_STATUS_0: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_MBIST_MEMCTRL_11_STATUS_0 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc100UL: // (DBG_REG_CALENDAR_OUT_DATA_LOW: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_OUT_DATA_LOW (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the lower 32 bits of the data output of the calendar\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc104UL: // (DBG_REG_CPU_DEBUG_DATA: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_CPU_DEBUG_DATA (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: debug only:                            These bits indicate\n     debug data that arrives from the CPU client; if ~dbg_registers_immediate_ack\n    =1 then ack is returned upon the following cycle of write command. In \n    this case ~dbg_registers_cpu_data_taken_intr interrupt anounce that the\n     data slot was taken; If ~dbg_registers_immediate_ack=0 then ack is returned\n     only after the data slot was taken. in this case if the CPU is not configured\n     in the slots then an ack signal will never be returned by the calendar\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc108UL: // (DBG_REG_FULL_ON_EXT_BUFFER: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FULL_ON_EXT_BUFFER (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicates\n     that the external buffer was filled; Relevant only when ~dbg_registers_full_mode\n    =0 (one shot)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc10cUL: // (DBG_REG_FULL_ON_INT_BUFFER: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_FULL_ON_INT_BUFFER (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicates\n     that the internal buffer was filled\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc110UL: // (DBG_REG_INTR_BUFFER_RD_PTR: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nDBG_REG_INTR_BUFFER_RD_PTR (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the read pointer for the internal buffer; The read pointer\n     describes the next address to be read from the internal buffer\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc114UL: // (DBG_REG_INTR_BUFFER_WR_PTR: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nDBG_REG_INTR_BUFFER_WR_PTR (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the write pointer for the internal buffer; The write pointer\n     describes the last address that was written to the internal buffer. An\n     exception exists after reset when #dbg_registers_intr_buffer_wr_ptr is\n     0 until first data is written\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc118UL: // (DBG_REG_OVL_ON_EXT_BUFFER: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_OVL_ON_EXT_BUFFER (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicates\n     that the external buffer was overflowed (newest data was thrown); Relevant\n     only for                   (a) ~dbg_registers_debug_target=2 (PCI) & ~\n    dbg_registers_full_mode=0 (one shot); or   (b) ~dbg_registers_debug_target\n    =1 (NIG) & ~dbg_registers_full_mode=0 (one shot)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc11cUL: // (DBG_REG_OVL_ON_INT_BUFFER: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_OVL_ON_INT_BUFFER (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicates\n     that the internal buffer was overflowed (newest data was thrown); Not\n     relevant if ~dbg_registers_debug_target=0 (internal buffer) & ~dbg_registers_full_mode\n    =1 (wrap);\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc120UL: // (DBG_REG_PCI_REQ_CREDIT: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDBG_REG_PCI_REQ_CREDIT (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: debug only:                            These bits indicate\n     the credit  for PCI request type 4 interface; MUST be configured AFTER\n     pci_ext_buffer_strt_addr_lsb/msb are configured\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc124UL: // (DBG_REG_WRAP_ON_EXT_BUFFER: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_WRAP_ON_EXT_BUFFER (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicates\n     wheter  indicates that external buffer was wrapped (oldest data was thrown\n    ); Relevant only when ~dbg_registers_debug_target=2 (PCI) & ~dbg_registers_full_mode\n    =1 (wrap);\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc128UL: // (DBG_REG_WRAP_ON_INT_BUFFER: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_WRAP_ON_INT_BUFFER (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: debug only:                            This bit indicates\n     wheter the internal buffer was wrapped (oldest data was thrown) Relevant\n     only when ~dbg_registers_debug_target=0 (internal buffer)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc130UL: // (DBG_REG_EXT_BUFFER_RD_PTR: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nDBG_REG_EXT_BUFFER_RD_PTR (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the read pointer for the external pci buffer; relevant only\n     when ~dbg_registers_debug_target=2 (PCI); The read pointer describes \n    the next address to be read from the external buffer; WB Read Only (write\n     request will not be acknowledged); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc140UL: // (DBG_REG_EXT_BUFFER_WR_PTR: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nDBG_REG_EXT_BUFFER_WR_PTR (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the write pointer for the external pci buffer when ~dbg_registers_debug_target\n    =2 (PCI). It describes the next address to write to the external buffer\n    ; 1024 Byte Chunks counter when ~dbg_registers_debug_target=1 (NIG) and\n     ~dbg_registers_full_mode=0 (one-shot); WB Read Only (write request will\n     not be acknowledged); \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc160UL: // (DBG_REG_CALENDAR_OUT_DATA: ACCESS:WB_R  DataWidth:0x47 ) 
			sprintf(disp_str, "\nDBG_REG_CALENDAR_OUT_DATA (ACCESS:WB_R  DataWidth:0x47 )" \
				"\nDescription: debug only:                            These bits indicate\n     the value of the data frame and valid output of the calendar; The concatenation\n     is done as follows: bits 63:0 - data; bits 66:64 - frame; bits 70:68 - \n    valid. The reference for the valid/frame is according to the ~dbg_registers_framing_mode\n     as follows:                            (a) tdm32 mode - bit 68 for valid\n     bit 64 for frame for data bits 31:0;                                   (\n    b) tdm64 mode - bit 68 for valid bit 64 for frame for data bits 31:0; \n    bit 69 for valid; bit 65 for frame for data bits 63:32;       (c) 64 bit\n     mode - bit 68 for valid bit 64 for frame for data bits 63:0;                                    (\n    d) 24 bit mode - bit 68 for valid bit 64 for frame for data bits 7:0; \n    bit 69 for valid bit 65 for frame for data bits 15:8; bit 70 for valid\n     bit 66 for frame for data bits 23:16;                          WB Read\n     Only (write request will not be acknowledged);\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc180UL: // (DBG_REG_EXPECTED_PATTERN: ACCESS:WB  DataWidth:0x46 ) 
			sprintf(disp_str, "\nDBG_REG_EXPECTED_PATTERN (ACCESS:WB  DataWidth:0x46 )" \
				"\nDescription: debug only:                            For pattern recognition\n     usage:  These bits represent the pattern to be compared with the vector\n     {slot_num[2:0]; frame[2:0]; data[63:0]}; This vector represent the debug\n     data it's slot number and it's frame signals that are going to stored\n     in the internal buffer;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc1a0UL: // (DBG_REG_EXPECTED_PATTERN_BIT_MASK: ACCESS:WB  DataWidth:0x46 ) 
			sprintf(disp_str, "\nDBG_REG_EXPECTED_PATTERN_BIT_MASK (ACCESS:WB  DataWidth:0x46 )" \
				"\nDescription: debug only:                            For pattern recognition\n     usage: These bits represent a mask bit vector that refers to the ~dbg_registers_expected_pattern\n     vector as follows:                  (a) 1 - bit is masked. This bit won\n    't be compared with the ~dbg_registers_expected_pattern referred bit;                          (\n    b) 0 - bit is enabled. This bit will be compared with the ~dbg_registers_expected_pattern\n     reffered bit;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc800UL: // (DBG_REG_INTR_BUFFER: ACCESS:WB  DataWidth:0x80 ) 
			sprintf(disp_str, "\nDBG_REG_INTR_BUFFER (ACCESS:WB  DataWidth:0x80 )" \
				"\nDescription: debug only:                            Internal buffer of\n     2KByte buffer\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd000UL: // (DBG_REG_INT_BUFFER_WRAP_COUNTER: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_INT_BUFFER_WRAP_COUNTER (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Number of wraps on internal buffer;                                \n    NOTE: valid only when debug_target=0 (internal buffer) and full_mode=1\n     (wrap)    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd004UL: // (DBG_REG_TRIGGER_EVENT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_EVENT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Configured messages sequencing was identified.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd008UL: // (DBG_REG_TRIGGER_INDIRECT0_RECORDED_DATA: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT0_RECORDED_DATA (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The data that was recorded trigger_indirect0_offset cycles\n     after start of message (during triggering machine operation in state \n    trigger_indirect0_state); NOTE: CID recording for filtering purpose within\n     the sem must use this register (and NOT trigger_indirect1_recorded_data\n     register)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd00cUL: // (DBG_REG_TRIGGER_INDIRECT1_RECORDED_DATA: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDBG_REG_TRIGGER_INDIRECT1_RECORDED_DATA (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The data that was recorded trigger_indirect1_offset cycles\n     after start of message (during triggering machine operation in state \n    trigger_indirect0_state);\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc0acUL: // (DBG_REG_DBG_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDBG_REG_DBG_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc12cUL: // (DBG_REG_DBG_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDBG_REG_DBG_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc150UL: // (DBG_REG_DBG_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDBG_REG_DBG_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc1c0UL: // (DBG_REG_DBG_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDBG_REG_DBG_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xd5d8UL: // (DBG_REG_DBG_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDBG_REG_DBG_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102000UL: // (DMAE_REG_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. Initialises specific states and statuses. To \n    initialise the state - write 1 into register; to enable working after \n    that - write 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102004UL: // (DMAE_REG_PCI_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_PCI_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: DMAE PCI Interface (Request;Read;Write) enable. If 0 - the\n     acknowledge input is disregarded; valid is deasserted; full is asserted\n    ; all other signals are treated as usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102008UL: // (DMAE_REG_GRC_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GRC_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: DMAE GRC Interface (Target;Master) enable. If 0 - the acknowledge\n     input is disregarded; valid is deasserted; all other signals are treated\n     as usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10200cUL: // (DMAE_REG_RLXD_ORDR: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_RLXD_ORDR (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Relaxed ordering. 0-strict PCI ordering is used;1-PCI-X relaxed\n     ordering is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102010UL: // (DMAE_REG_NO_SNOOP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_NO_SNOOP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0-PCI type cache snoop protection is required;1-system isn\n    't required to cause processor cache snoop for coherency.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102014UL: // (DMAE_REG_CRC16I_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_CRC16I_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - the CRC-16 initial value is all zeroes; if 1 - the\n     CRC-16 initial value is all ones.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102018UL: // (DMAE_REG_CRC16_BSWAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_CRC16_BSWAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - the CRC-16 final calculation result isn't byte swapped\n    ; if 1 - the CRC-16 final calculation result is byte swapped (byte [7:\n    0] goes to location [31:24];etc).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10201cUL: // (DMAE_REG_CRC16C_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_CRC16C_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - the CRC-16c initial value is all zeroes; if 1 - the\n     CRC-16c initial value is all ones.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102020UL: // (DMAE_REG_CRC16T10_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_CRC16T10_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - the CRC-16 T10 initial value is all zeroes; if 1 - \n    the CRC-16 T10 initial value is all ones.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102024UL: // (DMAE_REG_CRC32I_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_CRC32I_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - the CRC-32 initial value is all zeroes; if 1 - the\n     CRC-32 initial value is all ones.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102028UL: // (DMAE_REG_CRC32I_BSWAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_CRC32I_BSWAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - the CRC-32 final calculation result isn't byte swapped\n    ; if 1 - the CRC-32 final calculation result is byte swapped (byte [7:\n    0] goes to location [31:24];etc).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10202cUL: // (DMAE_REG_CRC32C_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_CRC32C_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - the CRC-32c initial value is all zeroes; if 1 - the\n     CRC-32c initial value is all ones.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102030UL: // (DMAE_REG_CRC32C_BSWAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_CRC32C_BSWAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - the CRC-32c final calculation result isn't byte swapped\n    ; if 1 - the CRC-32c final calculation result is byte swapped (byte [7\n    :0] goes to location [31:24];etc).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102034UL: // (DMAE_REG_CHKSUM0_FIX: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_CHKSUM0_FIX (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - the final checksum equal 0 won't be changed;if 1 - \n    the final checksum equal 0 will be fixed to all ones.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102038UL: // (DMAE_REG_CMD_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_CMD_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of Command memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10203cUL: // (DMAE_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDMAE_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from DMAE\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102040UL: // (DMAE_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDMAE_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from DMAE\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select; before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102044UL: // (DMAE_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDMAE_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from DMAE\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102048UL: // (DMAE_REG_DMAE_INT_STS: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_INT_STS (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPCI_RD_BUF_ERR :"
				, (u32_t)((data & DMAE_DMAE_INT_STS_REG_PCI_RD_BUF_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DMAE_DMAE_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x10204cUL: // (DMAE_REG_DMAE_INT_STS_CLR: ACCESS:RC  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_INT_STS_CLR (ACCESS:RC  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPCI_RD_BUF_ERR :"
				, (u32_t)((data & DMAE_DMAE_INT_STS_CLR_REG_PCI_RD_BUF_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DMAE_DMAE_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x102050UL: // (DMAE_REG_DMAE_INT_STS_WR: ACCESS:WR  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_INT_STS_WR (ACCESS:WR  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPCI_RD_BUF_ERR :"
				, (u32_t)((data & DMAE_DMAE_INT_STS_WR_REG_PCI_RD_BUF_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DMAE_DMAE_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x102054UL: // (DMAE_REG_DMAE_INT_MASK: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_INT_MASK (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPCI_RD_BUF_ERR :"
				, (u32_t)((data & DMAE_DMAE_INT_MASK_REG_PCI_RD_BUF_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DMAE_DMAE_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x102058UL: // (DMAE_REG_DMAE_PRTY_STS: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_PRTY_STS (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCMD_MEM_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_REG_CMD_MEM_PRTY_ERR) >> 3)
				, "\n\tPCI_BUF_HIGH_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_REG_PCI_BUF_HIGH_PRTY_ERR) >> 2)
				, "\n\tPCI_BUF_LOW_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_REG_PCI_BUF_LOW_PRTY_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x10205cUL: // (DMAE_REG_DMAE_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x4 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCMD_MEM_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_CLR_REG_CMD_MEM_PRTY_ERR) >> 3)
				, "\n\tPCI_BUF_HIGH_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_CLR_REG_PCI_BUF_HIGH_PRTY_ERR) >> 2)
				, "\n\tPCI_BUF_LOW_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_CLR_REG_PCI_BUF_LOW_PRTY_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x102060UL: // (DMAE_REG_DMAE_PRTY_STS_WR: ACCESS:WR  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_PRTY_STS_WR (ACCESS:WR  DataWidth:0x4 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCMD_MEM_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_WR_REG_CMD_MEM_PRTY_ERR) >> 3)
				, "\n\tPCI_BUF_HIGH_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_WR_REG_PCI_BUF_HIGH_PRTY_ERR) >> 2)
				, "\n\tPCI_BUF_LOW_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_WR_REG_PCI_BUF_LOW_PRTY_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & DMAE_DMAE_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x102064UL: // (DMAE_REG_DMAE_PRTY_MASK: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_PRTY_MASK (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCMD_MEM_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_MASK_REG_CMD_MEM_PRTY_ERR) >> 3)
				, "\n\tPCI_BUF_HIGH_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_MASK_REG_PCI_BUF_HIGH_PRTY_ERR) >> 2)
				, "\n\tPCI_BUF_LOW_PRTY_ERR :"
				, (u32_t)((data & DMAE_DMAE_PRTY_MASK_REG_PCI_BUF_LOW_PRTY_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & DMAE_DMAE_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x102068UL: // (DMAE_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDMAE_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: chicken bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10206cUL: // (DMAE_REG_RD_ATC_FLAGS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDMAE_REG_RD_ATC_FLAGS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Read request ATC Flags[1:0]: 00 - Do nothing; 01 - Search\n     only; 10 - Search & Cache; 11 - Search & Release; ATC Flags[2]:0 - Low\n     Priority;  - High Priority\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102070UL: // (DMAE_REG_WR_ATC_FLAGS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDMAE_REG_WR_ATC_FLAGS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Write request ATC Flags[1:0]: 00 - Do nothing; 01 - Search\n     only; 10 - Search & Cache; 11 - Search & Release; ATC Flags[2]:0 - Low\n     Priority;  - High Priority\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102074UL: // (DMAE_REG_PCI_ERR_DISCARD_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_PCI_ERR_DISCARD_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When set discards 1- or 2-Dword PCI transaction read in case\n     there is PCI error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102078UL: // (DMAE_REG_PCI_ERR_DISCARD_ADDR: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nDMAE_REG_PCI_ERR_DISCARD_ADDR (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: GRC address in case 1- or 2-Dword PCI transaction is discardd\n     due to PCI error and dmae.pci_err_discard set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10207cUL: // (DMAE_REG_BACKWARD_COMP_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_BACKWARD_COMP_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When set the DMAE will process the commands as in E1.5. 1\n    .The function that is used is always SRC-PCI; 2.VF_Valid = 0; 3.VFID=0\n    ; 4.Completion function=0; 5.Error handling=0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020c8UL: // (DMAE_REG_DMAE_CPT_MBIST1_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_CPT_MBIST1_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: N/A\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020ccUL: // (DMAE_REG_CPT_CLK_MBIST1_CNTRL_DMAE_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPT_CLK_MBIST1_CNTRL_DMAE_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: N/A\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020d0UL: // (DMAE_REG_CPT_CLK_MBIST1_CNTRL_DMAE_STATUS_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPT_CLK_MBIST1_CNTRL_DMAE_STATUS_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: N/A\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020d4UL: // (DMAE_REG_CPT_CLK_PCI_MBIST1_CNTRL_DMAE_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPT_CLK_PCI_MBIST1_CNTRL_DMAE_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: N/A\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020d8UL: // (DMAE_REG_CPT_CLK_PCI_MBIST1_CNTRL_DMAE_STATUS_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPT_CLK_PCI_MBIST1_CNTRL_DMAE_STATUS_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: N/A\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020dcUL: // (DMAE_REG_CPT_CLK_PCI_MBIST1_CNTRL_DMAE_STATUS_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPT_CLK_PCI_MBIST1_CNTRL_DMAE_STATUS_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: N/A\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020e0UL: // (DMAE_REG_CPT_CLK_CAM_CID_CAM_MBIST1_CNTRL_DMAE_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPT_CLK_CAM_CID_CAM_MBIST1_CNTRL_DMAE_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: N/A\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020e4UL: // (DMAE_REG_CPT_CLK_CAM_STRING_CAM_MBIST1_CNTRL_DMAE_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPT_CLK_CAM_STRING_CAM_MBIST1_CNTRL_DMAE_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: N/A\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020e8UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020ecUL: // (DMAE_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020f0UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020f4UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_3_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_3_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020f8UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_4_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_4_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020fcUL: // (DMAE_REG_CPU_MBIST_MEMCTRL_5_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_5_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102100UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_6_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_6_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102104UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_7_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_7_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102108UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_8_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_8_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10210cUL: // (DMAE_REG_CPU_MBIST_MEMCTRL_0_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_0_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102110UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_1_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_1_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102114UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_2_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_2_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102118UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_3_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_3_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10211cUL: // (DMAE_REG_CPU_MBIST_MEMCTRL_4_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_4_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102120UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_5_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_5_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102124UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_6_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_6_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102128UL: // (DMAE_REG_CPU_MBIST_MEMCTRL_7_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_7_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10212cUL: // (DMAE_REG_CPU_MBIST_MEMCTRL_8_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CPU_MBIST_MEMCTRL_8_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102080UL: // (DMAE_REG_GO_C0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 0 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102084UL: // (DMAE_REG_GO_C1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 1 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102088UL: // (DMAE_REG_GO_C10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 10 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10208cUL: // (DMAE_REG_GO_C11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 11 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102090UL: // (DMAE_REG_GO_C12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 12 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102094UL: // (DMAE_REG_GO_C13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 13 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102098UL: // (DMAE_REG_GO_C14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 14 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10209cUL: // (DMAE_REG_GO_C15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 15 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020a0UL: // (DMAE_REG_GO_C2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 2 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020a4UL: // (DMAE_REG_GO_C3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 3 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020a8UL: // (DMAE_REG_GO_C4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 4 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020acUL: // (DMAE_REG_GO_C5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 5 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020b0UL: // (DMAE_REG_GO_C6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 6 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020b4UL: // (DMAE_REG_GO_C7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 7 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020b8UL: // (DMAE_REG_GO_C8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 8 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020bcUL: // (DMAE_REG_GO_C9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDMAE_REG_GO_C9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Command 9 go.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020c0UL: // (DMAE_REG_PXP_REQ_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDMAE_REG_PXP_REQ_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: DMAE- PCI Request Interface initial credit. Write writes \n    the initial value to the credit counter; related to the address. Read \n    returns the current value of the counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102400UL: // (DMAE_REG_CMD_MEM: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDMAE_REG_CMD_MEM (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Commands memory. The address to command X; row Y is to calculated\n     as 14*X+Y.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020c4UL: // (DMAE_REG_DMAE_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102130UL: // (DMAE_REG_DMAE_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102800UL: // (DMAE_REG_DMAE_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDMAE_REG_DMAE_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170000UL: // (DORQ_REG_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. Initialises specific states and statuses. To \n    initialise the state - write 1 into register; to enable working after \n    that - write 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170004UL: // (DORQ_REG_IF_EN: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nDORQ_REG_IF_EN (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDPM_IFEN :"
				, (u32_t)((data & DORQ_IF_EN_REG_DPM_IFEN) >> 3)
				, "\n\tRSP_IFEN :"
				, (u32_t)((data & DORQ_IF_EN_REG_RSP_IFEN) >> 2)
				, "\n\tCF_IFEN :"
				, (u32_t)((data & DORQ_IF_EN_REG_CF_IFEN) >> 1)
				, "\n\tRQ_IFEN :"
				, (u32_t)((data & DORQ_IF_EN_REG_RQ_IFEN) >> 0));
			break;
		case 0x170008UL: // (DORQ_REG_MODE_ACT: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nDORQ_REG_MODE_ACT (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDPM_MODE_ACT :"
				, (u32_t)((data & DORQ_MODE_ACT_REG_DPM_MODE_ACT) >> 1)
				, "\n\tNORM_MODE_ACT :"
				, (u32_t)((data & DORQ_MODE_ACT_REG_NORM_MODE_ACT) >> 0));
			break;
		case 0x17000cUL: // (DORQ_REG_NORM_ADDR_MASK: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_NORM_ADDR_MASK (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The normal mode address mask. Filters a certain amount of\n     MSBits of the address to distinguish the normal mode.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170010UL: // (DORQ_REG_DPM_ADDR_MASK: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_DPM_ADDR_MASK (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The DPM mode address mask. Filters a certain amount of MSBits\n     of the address to distinguish the DPM mode.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170014UL: // (DORQ_REG_NORM_ADDR_BASE: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_NORM_ADDR_BASE (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The normal mode base address.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170018UL: // (DORQ_REG_DPM_ADDR_BASE: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_DPM_ADDR_BASE (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The DPM mode base address.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17001cUL: // (DORQ_REG_NORM_CID_MASK: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nDORQ_REG_NORM_CID_MASK (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: The normal mode CID extraction mask.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170020UL: // (DORQ_REG_DPM_CID_MASK: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nDORQ_REG_DPM_CID_MASK (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: The DPM mode CID extraction mask.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170024UL: // (DORQ_REG_NORM_CID_BASE: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nDORQ_REG_NORM_CID_BASE (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: The normal mode CID base.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170028UL: // (DORQ_REG_DPM_CID_BASE: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nDORQ_REG_DPM_CID_BASE (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: The DPM mode CID base.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17002cUL: // (DORQ_REG_NORM_CID_OFST: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDORQ_REG_NORM_CID_OFST (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The normal mode CID extraction offset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170030UL: // (DORQ_REG_DPM_CID_OFST: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDORQ_REG_DPM_CID_OFST (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The DPM mode CID extraction offset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170034UL: // (DORQ_REG_QM_AEMPTY_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_QM_AEMPTY_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - QM almost empty is disregarded; if 1 - QM almost empty\n     is taken into consideration.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170038UL: // (DORQ_REG_REGN: ACCESS:RW  DataWidth:0x18  Multi Field Register  ) 
			sprintf(disp_str, "\nDORQ_REG_REGN (ACCESS:RW  DataWidth:0x18  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSHRT_REGN_TX :"
				, (u32_t)((data & DORQ_REGN_REG_SHRT_REGN_TX) >> 16)
				, "\n\tREGN_RX :"
				, (u32_t)((data & DORQ_REGN_REG_REGN_RX) >> 8)
				, "\n\tNORM_REGN_TX :"
				, (u32_t)((data & DORQ_REGN_REG_NORM_REGN_TX) >> 0));
			break;
		case 0x17003cUL: // (DORQ_REG_OUTST_REQ: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDORQ_REG_OUTST_REQ (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of simultaneous outstanding requests to Context\n     Fetch Interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170040UL: // (DORQ_REG_DPM_WR_TYPE_LOC: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDORQ_REG_DPM_WR_TYPE_LOC (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: DPM write type location. Selects one of bits of address to\n     differentiate between DPM data and DPM trigger types.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170044UL: // (DORQ_REG_DPM_CID_ADDR: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_DPM_CID_ADDR (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The address to write the DPM CID to STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170048UL: // (DORQ_REG_RSP_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDORQ_REG_RSP_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial credit at the Doorbell Response Interface. The\n     write writes the same initial credit to the rspa_crd_cnt and rspb_crd_cnt\n    . The read reads this written value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17004cUL: // (DORQ_REG_NORM_CMHEAD_TX: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nDORQ_REG_NORM_CMHEAD_TX (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: TCM Header when only TCP context is loaded.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170050UL: // (DORQ_REG_CMHEAD_RX: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nDORQ_REG_CMHEAD_RX (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: UCM Header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170054UL: // (DORQ_REG_SHRT_CMHEAD: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nDORQ_REG_SHRT_CMHEAD (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: TCM Header when both ULP and TCP context is loaded.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170058UL: // (DORQ_REG_ERR_CMHEAD: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nDORQ_REG_ERR_CMHEAD (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The value sent to CM header in the case of CFC load error\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17005cUL: // (DORQ_REG_ERR_EVENT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_ERR_EVENT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Event ID sent to TCM/UCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170070UL: // (DORQ_REG_SHRT_ACT_CNT: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDORQ_REG_SHRT_ACT_CNT (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Initial activity counter value on the load request; when \n    the shortcut is done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170074UL: // (DORQ_REG_M_SP_RAM_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nDORQ_REG_M_SP_RAM_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of DORQ memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170078UL: // (DORQ_REG_DQ_FIFO_FULL_TH: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FIFO_FULL_TH (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: The threshold of the DQ FIFO to send the full interrupt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17007cUL: // (DORQ_REG_DQ_FIFO_AFULL_TH: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FIFO_AFULL_TH (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: The threshold of the DQ FIFO to send the almost full interrupt\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170080UL: // (DORQ_REG_DQ_FREEZE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FREEZE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When set; the DQ will serve the doorbells; already existing\n     in the queue and will block the Response interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170084UL: // (DORQ_REG_AUTO_FREEZE_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_AUTO_FREEZE_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When set; the DQ will automatically stop sending CFC load\n     requests when a doorbell discard interrupt is generated. The freeze mode\n     will remain until the auto_freeze_rel register is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170088UL: // (DORQ_REG_AUTO_DISCARD_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_AUTO_DISCARD_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If this register is equal to 1 then the DQ will enter auto\n     discard mode when a doorbell discard interrupt is generated. In this \n    mode all incoming doorbells will be dropped even if the FIFO is not full\n     anymore. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17008cUL: // (DORQ_REG_DB_ADDR0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_DB_ADDR0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Doorbell address for RBC doorbells (function 0).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170090UL: // (DORQ_REG_DB_ADDR1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_DB_ADDR1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Doorbell address for RBC doorbells (function 1).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170094UL: // (DORQ_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from DORQ\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170098UL: // (DORQ_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from DORQ\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select; before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17009cUL: // (DORQ_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nDORQ_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from DORQ\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700a0UL: // (DORQ_REG_DQ_FILL_LVLL: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FILL_LVLL (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Current value of the  DQ FIFO fill level according to leading\n     pointer. The range is 0 - 256 FIFO rows; where each row stands for the\n     doorbell.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700a4UL: // (DORQ_REG_DQ_FILL_LVLF: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FILL_LVLF (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Current value of the  DQ FIFO fill level according to following\n     pointer. The range is 0 - 256 FIFO rows; where each row stands for the\n     doorbell.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700a8UL: // (DORQ_REG_OUTST_REQ_CNT: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDORQ_REG_OUTST_REQ_CNT (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Current value of outstanding requests counter credit.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700acUL: // (DORQ_REG_RSPA_CRD_CNT: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDORQ_REG_RSPA_CRD_CNT (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Current value of response A counter credit. Initial credit\n     is configured through write to ~dorq_registers_rsp_init_crd.rsp_init_crd\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700b0UL: // (DORQ_REG_RSPB_CRD_CNT: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nDORQ_REG_RSPB_CRD_CNT (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Current value of response B counter credit.  Initial credit\n     is configured through write to ~dorq_registers_rsp_init_crd.rsp_init_crd\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700b4UL: // (DORQ_REG_DQ_FIFO_RDL_PTR: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FIFO_RDL_PTR (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: DQ FIFO read leading pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700b8UL: // (DORQ_REG_DQ_FIFO_RDF_PTR: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FIFO_RDF_PTR (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: DQ FIFO read following pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700bcUL: // (DORQ_REG_DQ_FIFO_WR_PTR: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FIFO_WR_PTR (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: DQ FIFO write pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700c0UL: // (DORQ_REG_DQ_FULL_ST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FULL_ST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DQ FIFO full status. Is set; when FIFO filling level is more\n     or equal to full threshold; reset on full clear.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700c4UL: // (DORQ_REG_DB_CNTR: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_DB_CNTR (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Counts the total number of received doorbells.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700c8UL: // (DORQ_REG_DB_MIS_CNTR: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_DB_MIS_CNTR (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Counts the total number of discarded doorbells in PF mode\n     due to mode or range mismatch\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700ccUL: // (DORQ_REG_CFC_ERR_CNTR: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_CFC_ERR_CNTR (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Counts the number of CFC load errors.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700d0UL: // (DORQ_REG_DPM_FULL_CNTR: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_DPM_FULL_CNTR (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Counts the number DPM tried to wite to STORM when it was \n    full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700d4UL: // (DORQ_REG_DB_DIS_CNTR0: ACCESS:ST  DataWidth:0x10 ) 
			sprintf(disp_str, "\nDORQ_REG_DB_DIS_CNTR0 (ACCESS:ST  DataWidth:0x10 )" \
				"\nDescription: Doorbell disable counter (function 0).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700d8UL: // (DORQ_REG_DB_DIS_CNTR1: ACCESS:ST  DataWidth:0x10 ) 
			sprintf(disp_str, "\nDORQ_REG_DB_DIS_CNTR1 (ACCESS:ST  DataWidth:0x10 )" \
				"\nDescription: Doorbell disable counter (function 1).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700dcUL: // (DORQ_REG_TYPE_VAL_ERR_ADDR: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_VAL_ERR_ADDR (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Keeps the address of the first doorbell; caused the type \n    validation error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700e0UL: // (DORQ_REG_PRIV_MIN_ADDR0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_PRIV_MIN_ADDR0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700e4UL: // (DORQ_REG_PRIV_MIN_ADDR1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_PRIV_MIN_ADDR1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700e8UL: // (DORQ_REG_PRIV_MAX_ADDR0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_PRIV_MAX_ADDR0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700ecUL: // (DORQ_REG_PRIV_MAX_ADDR1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_PRIV_MAX_ADDR1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700f0UL: // (DORQ_REG_TYPE_MIN_ADDR00: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR00 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     0 conection type 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700f4UL: // (DORQ_REG_TYPE_MIN_ADDR01: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR01 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     0 conection type 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700f8UL: // (DORQ_REG_TYPE_MIN_ADDR02: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR02 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     0 conection type 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1700fcUL: // (DORQ_REG_TYPE_MIN_ADDR03: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR03 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     0 conection type 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170100UL: // (DORQ_REG_TYPE_MIN_ADDR04: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR04 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     0 conection type 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170104UL: // (DORQ_REG_TYPE_MIN_ADDR05: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR05 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     0 conection type 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170108UL: // (DORQ_REG_TYPE_MIN_ADDR06: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR06 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     0 conection type 6.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17010cUL: // (DORQ_REG_TYPE_MIN_ADDR07: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR07 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     0 conection type 7.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170110UL: // (DORQ_REG_TYPE_MIN_ADDR10: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR10 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     1 conection type 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170114UL: // (DORQ_REG_TYPE_MIN_ADDR11: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR11 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     1 conection type 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170118UL: // (DORQ_REG_TYPE_MIN_ADDR12: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR12 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     1 conection type 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17011cUL: // (DORQ_REG_TYPE_MIN_ADDR13: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR13 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     1 conection type 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170120UL: // (DORQ_REG_TYPE_MIN_ADDR14: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR14 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     1 conection type 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170124UL: // (DORQ_REG_TYPE_MIN_ADDR15: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR15 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     1 conection type 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170128UL: // (DORQ_REG_TYPE_MIN_ADDR16: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR16 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     1 conection type 6.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17012cUL: // (DORQ_REG_TYPE_MIN_ADDR17: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MIN_ADDR17 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The minimum level for the type address validation function\n     1 conection type 7.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170130UL: // (DORQ_REG_TYPE_MAX_ADDR00: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR00 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     0 connection type 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170134UL: // (DORQ_REG_TYPE_MAX_ADDR01: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR01 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     0 connection type 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170138UL: // (DORQ_REG_TYPE_MAX_ADDR02: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR02 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     0 connection type 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17013cUL: // (DORQ_REG_TYPE_MAX_ADDR03: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR03 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     0 connection type 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170140UL: // (DORQ_REG_TYPE_MAX_ADDR04: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR04 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     0 connection type 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170144UL: // (DORQ_REG_TYPE_MAX_ADDR05: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR05 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     0 connection type 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170148UL: // (DORQ_REG_TYPE_MAX_ADDR06: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR06 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     0 connection type 6.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17014cUL: // (DORQ_REG_TYPE_MAX_ADDR07: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR07 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     0 connection type 7.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170150UL: // (DORQ_REG_TYPE_MAX_ADDR10: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR10 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     1 connection type 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170154UL: // (DORQ_REG_TYPE_MAX_ADDR11: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR11 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     1 connection type 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170158UL: // (DORQ_REG_TYPE_MAX_ADDR12: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR12 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     1 connection type 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17015cUL: // (DORQ_REG_TYPE_MAX_ADDR13: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR13 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     1 connection type 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170160UL: // (DORQ_REG_TYPE_MAX_ADDR14: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR14 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     1 connection type 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170164UL: // (DORQ_REG_TYPE_MAX_ADDR15: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR15 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     1 connection type 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170168UL: // (DORQ_REG_TYPE_MAX_ADDR16: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR16 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     1 connection type 6.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17016cUL: // (DORQ_REG_TYPE_MAX_ADDR17: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_MAX_ADDR17 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The maximum level for the type address validation function\n     1 connection type 7.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170170UL: // (DORQ_REG_TYPE_VAL_ERR_CNTR: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_TYPE_VAL_ERR_CNTR (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: The number of doorbells which failed the PF type validation\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170174UL: // (DORQ_REG_DORQ_INT_STS: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_INT_STS (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVF_TYPE_VAL_ERR :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_REG_VF_TYPE_VAL_ERR) >> 5)
				, "\n\tDQ_AFULL :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_REG_DQ_AFULL) >> 4)
				, "\n\tDB_COLLISION :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_REG_DB_COLLISION) >> 3)
				, "\n\tTYPE_VAL_ERR :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_REG_TYPE_VAL_ERR) >> 2)
				, "\n\tDB_DISCARD :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_REG_DB_DISCARD) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x170178UL: // (DORQ_REG_DORQ_INT_STS_CLR: ACCESS:RC  DataWidth:0x6 ) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_INT_STS_CLR (ACCESS:RC  DataWidth:0x6 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVF_TYPE_VAL_ERR :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_CLR_REG_VF_TYPE_VAL_ERR) >> 5)
				, "\n\tDQ_AFULL :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_CLR_REG_DQ_AFULL) >> 4)
				, "\n\tDB_COLLISION :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_CLR_REG_DB_COLLISION) >> 3)
				, "\n\tTYPE_VAL_ERR :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_CLR_REG_TYPE_VAL_ERR) >> 2)
				, "\n\tDB_DISCARD :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_CLR_REG_DB_DISCARD) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x17017cUL: // (DORQ_REG_DORQ_INT_STS_WR: ACCESS:WR  DataWidth:0x6 ) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_INT_STS_WR (ACCESS:WR  DataWidth:0x6 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVF_TYPE_VAL_ERR :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_WR_REG_VF_TYPE_VAL_ERR) >> 5)
				, "\n\tDQ_AFULL :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_WR_REG_DQ_AFULL) >> 4)
				, "\n\tDB_COLLISION :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_WR_REG_DB_COLLISION) >> 3)
				, "\n\tTYPE_VAL_ERR :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_WR_REG_TYPE_VAL_ERR) >> 2)
				, "\n\tDB_DISCARD :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_WR_REG_DB_DISCARD) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DORQ_DORQ_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x170180UL: // (DORQ_REG_DORQ_INT_MASK: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_INT_MASK (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVF_TYPE_VAL_ERR :"
				, (u32_t)((data & DORQ_DORQ_INT_MASK_REG_VF_TYPE_VAL_ERR) >> 5)
				, "\n\tDQ_AFULL :"
				, (u32_t)((data & DORQ_DORQ_INT_MASK_REG_DQ_AFULL) >> 4)
				, "\n\tDB_COLLISION :"
				, (u32_t)((data & DORQ_DORQ_INT_MASK_REG_DB_COLLISION) >> 3)
				, "\n\tTYPE_VAL_ERR :"
				, (u32_t)((data & DORQ_DORQ_INT_MASK_REG_TYPE_VAL_ERR) >> 2)
				, "\n\tDB_DISCARD :"
				, (u32_t)((data & DORQ_DORQ_INT_MASK_REG_DB_DISCARD) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & DORQ_DORQ_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x170184UL: // (DORQ_REG_DORQ_PRTY_STS: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_PRTY_STS (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & DORQ_DORQ_PRTY_STS_REG_DORQ_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & DORQ_DORQ_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x170188UL: // (DORQ_REG_DORQ_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x2 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & DORQ_DORQ_PRTY_STS_CLR_REG_DORQ_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & DORQ_DORQ_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x17018cUL: // (DORQ_REG_DORQ_PRTY_STS_WR: ACCESS:WR  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_PRTY_STS_WR (ACCESS:WR  DataWidth:0x2 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & DORQ_DORQ_PRTY_STS_WR_REG_DORQ_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & DORQ_DORQ_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x170190UL: // (DORQ_REG_DORQ_PRTY_MASK: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_PRTY_MASK (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & DORQ_DORQ_PRTY_MASK_REG_DORQ_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & DORQ_DORQ_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x170194UL: // (DORQ_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: chicken bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701acUL: // (DORQ_REG_DB_RSP_CNTR: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_DB_RSP_CNTR (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Counts the number of doorbells sent to XCM or UCM on the \n    response interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701b0UL: // (DORQ_REG_DB_FID0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_DB_FID0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: FID for RBC doorbells (function 0).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701b4UL: // (DORQ_REG_DB_FID1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_DB_FID1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: FID for RBC doorbells (function 1).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701c4UL: // (DORQ_REG_CANCEL_CMHEAD: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nDORQ_REG_CANCEL_CMHEAD (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The value sent to CM header in the case of CFC load cancellation\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701ccUL: // (DORQ_REG_CFC_CANCEL_CNTR: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_CFC_CANCEL_CNTR (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Counts the number of CFC load cancellations.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701d4UL: // (DORQ_REG_VF_DB_MIS_CNTR: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_DB_MIS_CNTR (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Counts the total number of discarded doorbells in VF mode\n     due to range mismatch\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701d8UL: // (DORQ_REG_VF_TYPE_VAL_ERR_FID: ACCESS:RC  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VAL_ERR_FID (ACCESS:RC  DataWidth:0xa )" \
				"\nDescription: Keeps the FID of the first doorbell that caused a VF type\n     validation error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701dcUL: // (DORQ_REG_VF_TYPE_VAL_ERR_MCID: ACCESS:RC  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VAL_ERR_MCID (ACCESS:RC  DataWidth:0x11 )" \
				"\nDescription: Keeps the MCID of the first doorbell that caused a VF type\n     validation error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701e0UL: // (DORQ_REG_VF_TYPE_VAL_ERR_CNTR: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VAL_ERR_CNTR (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: The number of doorbells which failed the VF type validation\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701e8UL: // (DORQ_REG_VF_INDEX_FIX_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_INDEX_FIX_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Chicken bit for VF index fix. If high rf_dorq_func[9:4] maps\n     to VF registers.  Otherwise rf_dorq_func[5:0] maps to VF registers.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170218UL: // (DORQ_REG_VF_TYPE_MASK_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17021cUL: // (DORQ_REG_VF_TYPE_MASK_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170220UL: // (DORQ_REG_VF_TYPE_MASK_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170224UL: // (DORQ_REG_VF_TYPE_MASK_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170228UL: // (DORQ_REG_VF_TYPE_MASK_4: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_4 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17022cUL: // (DORQ_REG_VF_TYPE_MASK_5: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_5 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170230UL: // (DORQ_REG_VF_TYPE_MASK_6: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_6 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170234UL: // (DORQ_REG_VF_TYPE_MASK_7: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_7 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170238UL: // (DORQ_REG_VF_TYPE_MASK_8: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_8 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17023cUL: // (DORQ_REG_VF_TYPE_MASK_9: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_9 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170240UL: // (DORQ_REG_VF_TYPE_MASK_10: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_10 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170244UL: // (DORQ_REG_VF_TYPE_MASK_11: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_11 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170248UL: // (DORQ_REG_VF_TYPE_MASK_12: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_12 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17024cUL: // (DORQ_REG_VF_TYPE_MASK_13: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_13 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170250UL: // (DORQ_REG_VF_TYPE_MASK_14: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_14 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170254UL: // (DORQ_REG_VF_TYPE_MASK_15: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MASK_15 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation mask value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170258UL: // (DORQ_REG_VF_TYPE_VALUE_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17025cUL: // (DORQ_REG_VF_TYPE_VALUE_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170260UL: // (DORQ_REG_VF_TYPE_VALUE_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170264UL: // (DORQ_REG_VF_TYPE_VALUE_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170268UL: // (DORQ_REG_VF_TYPE_VALUE_4: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_4 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17026cUL: // (DORQ_REG_VF_TYPE_VALUE_5: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_5 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170270UL: // (DORQ_REG_VF_TYPE_VALUE_6: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_6 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170274UL: // (DORQ_REG_VF_TYPE_VALUE_7: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_7 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170278UL: // (DORQ_REG_VF_TYPE_VALUE_8: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_8 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17027cUL: // (DORQ_REG_VF_TYPE_VALUE_9: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_9 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170280UL: // (DORQ_REG_VF_TYPE_VALUE_10: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_10 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170284UL: // (DORQ_REG_VF_TYPE_VALUE_11: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_11 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170288UL: // (DORQ_REG_VF_TYPE_VALUE_12: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_12 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17028cUL: // (DORQ_REG_VF_TYPE_VALUE_13: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_13 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170290UL: // (DORQ_REG_VF_TYPE_VALUE_14: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_14 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170294UL: // (DORQ_REG_VF_TYPE_VALUE_15: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_VALUE_15 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: VF type validation comp value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170298UL: // (DORQ_REG_VF_TYPE_MIN_MCID_0: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_0 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17029cUL: // (DORQ_REG_VF_TYPE_MIN_MCID_1: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_1 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702a0UL: // (DORQ_REG_VF_TYPE_MIN_MCID_2: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_2 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702a4UL: // (DORQ_REG_VF_TYPE_MIN_MCID_3: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_3 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702a8UL: // (DORQ_REG_VF_TYPE_MIN_MCID_4: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_4 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702acUL: // (DORQ_REG_VF_TYPE_MIN_MCID_5: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_5 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702b0UL: // (DORQ_REG_VF_TYPE_MIN_MCID_6: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_6 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702b4UL: // (DORQ_REG_VF_TYPE_MIN_MCID_7: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_7 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702b8UL: // (DORQ_REG_VF_TYPE_MIN_MCID_8: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_8 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702bcUL: // (DORQ_REG_VF_TYPE_MIN_MCID_9: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_9 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702c0UL: // (DORQ_REG_VF_TYPE_MIN_MCID_10: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_10 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702c4UL: // (DORQ_REG_VF_TYPE_MIN_MCID_11: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_11 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702c8UL: // (DORQ_REG_VF_TYPE_MIN_MCID_12: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_12 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702ccUL: // (DORQ_REG_VF_TYPE_MIN_MCID_13: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_13 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702d0UL: // (DORQ_REG_VF_TYPE_MIN_MCID_14: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_14 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702d4UL: // (DORQ_REG_VF_TYPE_MIN_MCID_15: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MIN_MCID_15 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Max MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702d8UL: // (DORQ_REG_VF_TYPE_MAX_MCID_0: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_0 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702dcUL: // (DORQ_REG_VF_TYPE_MAX_MCID_1: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_1 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702e0UL: // (DORQ_REG_VF_TYPE_MAX_MCID_2: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_2 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702e4UL: // (DORQ_REG_VF_TYPE_MAX_MCID_3: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_3 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702e8UL: // (DORQ_REG_VF_TYPE_MAX_MCID_4: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_4 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702ecUL: // (DORQ_REG_VF_TYPE_MAX_MCID_5: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_5 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702f0UL: // (DORQ_REG_VF_TYPE_MAX_MCID_6: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_6 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702f4UL: // (DORQ_REG_VF_TYPE_MAX_MCID_7: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_7 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702f8UL: // (DORQ_REG_VF_TYPE_MAX_MCID_8: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_8 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1702fcUL: // (DORQ_REG_VF_TYPE_MAX_MCID_9: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_9 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170300UL: // (DORQ_REG_VF_TYPE_MAX_MCID_10: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_10 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170304UL: // (DORQ_REG_VF_TYPE_MAX_MCID_11: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_11 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170308UL: // (DORQ_REG_VF_TYPE_MAX_MCID_12: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_12 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17030cUL: // (DORQ_REG_VF_TYPE_MAX_MCID_13: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_13 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170310UL: // (DORQ_REG_VF_TYPE_MAX_MCID_14: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_14 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170314UL: // (DORQ_REG_VF_TYPE_MAX_MCID_15: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_TYPE_MAX_MCID_15 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: VF type validation Min MCID value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170318UL: // (DORQ_REG_AUTO_FREEZE_ST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_AUTO_FREEZE_ST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When high auto freeze is active and doorbells are not being\n     drained from the FIFO.  Cleared when auto_freeze_rel is written.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17031cUL: // (DORQ_REG_AUTO_DISCARD_ST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_AUTO_DISCARD_ST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When high auto discard is active and all doorbells are dropped\n     before going into the FIFO.  Cleared when auto_discard_rel is written\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17032cUL: // (DORQ_REG_PF_DISCARD_STAT_POLL: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_PF_DISCARD_STAT_POLL (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Allows all the pf_discard_status bits to be read at once.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170338UL: // (DORQ_REG_VF_DISCARD_STAT_POLL0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_DISCARD_STAT_POLL0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Allows the upper 32 bits of the vf_discard_status bits to\n     be read at once.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17033cUL: // (DORQ_REG_VF_DISCARD_STAT_POLL1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_DISCARD_STAT_POLL1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Allows the lower 32 bits of the vf_discard_status bits to\n     be read at once.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170344UL: // (DORQ_REG_CM_T_FLAG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_CM_T_FLAG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Thread Required bit for error indicating CM messages\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170400UL: // (DORQ_REG_AGG_CMD_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170404UL: // (DORQ_REG_AGG_CMD_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170408UL: // (DORQ_REG_AGG_CMD_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17040cUL: // (DORQ_REG_AGG_CMD_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170410UL: // (DORQ_REG_AGG_CMD_4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170414UL: // (DORQ_REG_AGG_CMD_5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170418UL: // (DORQ_REG_AGG_CMD_6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17041cUL: // (DORQ_REG_AGG_CMD_7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170420UL: // (DORQ_REG_AGG_CMD_8: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_8 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170424UL: // (DORQ_REG_AGG_CMD_9: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_9 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170428UL: // (DORQ_REG_AGG_CMD_10: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_10 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17042cUL: // (DORQ_REG_AGG_CMD_11: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_11 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170430UL: // (DORQ_REG_AGG_CMD_12: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_12 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170434UL: // (DORQ_REG_AGG_CMD_13: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_13 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170438UL: // (DORQ_REG_AGG_CMD_14: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_14 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17043cUL: // (DORQ_REG_AGG_CMD_15: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_15 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170440UL: // (DORQ_REG_AGG_CMD_16: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_16 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170444UL: // (DORQ_REG_AGG_CMD_17: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_17 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170448UL: // (DORQ_REG_AGG_CMD_18: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_18 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17044cUL: // (DORQ_REG_AGG_CMD_19: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_19 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170450UL: // (DORQ_REG_AGG_CMD_20: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_20 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170454UL: // (DORQ_REG_AGG_CMD_21: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_21 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170458UL: // (DORQ_REG_AGG_CMD_22: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_22 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17045cUL: // (DORQ_REG_AGG_CMD_23: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_23 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170460UL: // (DORQ_REG_AGG_CMD_24: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_24 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170464UL: // (DORQ_REG_AGG_CMD_25: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_25 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170468UL: // (DORQ_REG_AGG_CMD_26: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_26 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17046cUL: // (DORQ_REG_AGG_CMD_27: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_27 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170470UL: // (DORQ_REG_AGG_CMD_28: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_28 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170474UL: // (DORQ_REG_AGG_CMD_29: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_29 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170478UL: // (DORQ_REG_AGG_CMD_30: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_30 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17047cUL: // (DORQ_REG_AGG_CMD_31: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_31 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170480UL: // (DORQ_REG_AGG_CMD_32: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_32 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170484UL: // (DORQ_REG_AGG_CMD_33: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_33 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170488UL: // (DORQ_REG_AGG_CMD_34: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_34 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17048cUL: // (DORQ_REG_AGG_CMD_35: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_35 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170490UL: // (DORQ_REG_AGG_CMD_36: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_36 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170494UL: // (DORQ_REG_AGG_CMD_37: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_37 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170498UL: // (DORQ_REG_AGG_CMD_38: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_38 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17049cUL: // (DORQ_REG_AGG_CMD_39: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_39 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704a0UL: // (DORQ_REG_AGG_CMD_40: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_40 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704a4UL: // (DORQ_REG_AGG_CMD_41: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_41 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704a8UL: // (DORQ_REG_AGG_CMD_42: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_42 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704acUL: // (DORQ_REG_AGG_CMD_43: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_43 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704b0UL: // (DORQ_REG_AGG_CMD_44: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_44 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704b4UL: // (DORQ_REG_AGG_CMD_45: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_45 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704b8UL: // (DORQ_REG_AGG_CMD_46: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_46 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704bcUL: // (DORQ_REG_AGG_CMD_47: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_47 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704c0UL: // (DORQ_REG_AGG_CMD_48: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_48 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704c4UL: // (DORQ_REG_AGG_CMD_49: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_49 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704c8UL: // (DORQ_REG_AGG_CMD_50: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_50 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704ccUL: // (DORQ_REG_AGG_CMD_51: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_51 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704d0UL: // (DORQ_REG_AGG_CMD_52: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_52 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704d4UL: // (DORQ_REG_AGG_CMD_53: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_53 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704d8UL: // (DORQ_REG_AGG_CMD_54: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_54 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704dcUL: // (DORQ_REG_AGG_CMD_55: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_55 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704e0UL: // (DORQ_REG_AGG_CMD_56: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_56 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704e4UL: // (DORQ_REG_AGG_CMD_57: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_57 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704e8UL: // (DORQ_REG_AGG_CMD_58: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_58 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704ecUL: // (DORQ_REG_AGG_CMD_59: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_59 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704f0UL: // (DORQ_REG_AGG_CMD_60: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_60 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704f4UL: // (DORQ_REG_AGG_CMD_61: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_61 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704f8UL: // (DORQ_REG_AGG_CMD_62: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_62 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1704fcUL: // (DORQ_REG_AGG_CMD_63: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD_63 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Aggregation cmd per  {vf_valid; conn type; A; R}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170198UL: // (DORQ_REG_VF_NORM_ADDR_MASK: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_NORM_ADDR_MASK (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: The normal mode address mask for VF doorbells. Filters a \n    certain amount of MSBits of the address to distinguish the normal mode\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17019cUL: // (DORQ_REG_VF_NORM_ADDR_BASE: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_NORM_ADDR_BASE (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: The normal mode base address for VF doorbells.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701a0UL: // (DORQ_REG_VF_NORM_CID_BASE: ACCESS:RW  DataWidth:0x11  SPLIT:8 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_NORM_CID_BASE (ACCESS:RW  DataWidth:0x11  SPLIT:8 )" \
				"\nDescription: The per-VNIC normal mode CID base for VF doorbells.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701a8UL: // (DORQ_REG_VF_NORM_VF_BASE: ACCESS:RW  DataWidth:0x3  SPLIT:8 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_NORM_VF_BASE (ACCESS:RW  DataWidth:0x3  SPLIT:8 )" \
				"\nDescription: Starting absolute VFID for each PF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701d0UL: // (DORQ_REG_PF_USAGE_CNT: ACCESS:R  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nDORQ_REG_PF_USAGE_CNT (ACCESS:R  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: Per-PF usage count.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701f4UL: // (DORQ_REG_VF_NORM_CID_OFST: ACCESS:RW  DataWidth:0x5  SPLIT:8 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_NORM_CID_OFST (ACCESS:RW  DataWidth:0x5  SPLIT:8 )" \
				"\nDescription: The normal mode CID extraction offset for VF doorbells.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170200UL: // (DORQ_REG_AUTO_DISCARD_REL: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_AUTO_DISCARD_REL (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: Release the discard mode. Write only.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170204UL: // (DORQ_REG_AUTO_FREEZE_REL: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_AUTO_FREEZE_REL (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: Release the freeze mode set by auto freeze. Write only.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170208UL: // (DORQ_REG_DB_DATA0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_DB_DATA0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Doorbell data for RBC doorbells (function 0).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17020cUL: // (DORQ_REG_DB_DATA1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_DB_DATA1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Doorbell data for RBC doorbells (function 1).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170210UL: // (DORQ_REG_DQ_FIFO_FULL_CLR: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FIFO_FULL_CLR (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: Clears the full interrupt. Functionally write only. The read\n     access is allowed with don't care data.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170214UL: // (DORQ_REG_DQ_FILL_LVL_MAX: ACCESS:RW  DataWidth:0xd ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FILL_LVL_MAX (ACCESS:RW  DataWidth:0xd )" \
				"\nDescription: Sticky value of the maximal DQ FIFO fill level. The range\n     is 0 - 2560 FIFO rows; where each row stands for the doorbell.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170320UL: // (DORQ_REG_VF_USAGE_CNT: ACCESS:R  DataWidth:0x7  SPLIT:64 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_USAGE_CNT (ACCESS:R  DataWidth:0x7  SPLIT:64 )" \
				"\nDescription: Per-VF usage count\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170324UL: // (DORQ_REG_PF_DISCARD_STATUS: ACCESS:R  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nDORQ_REG_PF_DISCARD_STATUS (ACCESS:R  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Per-PF register indicating that a doorbell has been dropped\n     for this PF.  Cleared when corresponding pf_discard_rel register is written\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170328UL: // (DORQ_REG_PF_DISCARD_REL: ACCESS:W  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nDORQ_REG_PF_DISCARD_REL (ACCESS:W  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: When this register is written the corresponding pf_discard_status\n     register is cleared.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170330UL: // (DORQ_REG_VF_DISCARD_STATUS: ACCESS:R  DataWidth:0x1  SPLIT:64 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_DISCARD_STATUS (ACCESS:R  DataWidth:0x1  SPLIT:64 )" \
				"\nDescription: Per-VF register indicating that a doorbell has been dropped\n     for this VF.  Cleared when corresponding vf_discard_rel register is written\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170334UL: // (DORQ_REG_VF_DISCARD_REL: ACCESS:W  DataWidth:0x1  SPLIT:64 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_DISCARD_REL (ACCESS:W  DataWidth:0x1  SPLIT:64 )" \
				"\nDescription: When this register is written the corresponding vf_discard_status\n     register is cleared.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170340UL: // (DORQ_REG_VF_USAGE_CT_LIMIT: ACCESS:RW  DataWidth:0x7  SPLIT:8 ) 
			sprintf(disp_str, "\nDORQ_REG_VF_USAGE_CT_LIMIT (ACCESS:RW  DataWidth:0x7  SPLIT:8 )" \
				"\nDescription: Per-PF VF usage count limit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170348UL: // (DORQ_REG_FIFO_CNTR: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_FIFO_CNTR (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Counts all doorbells that enter the FIFO.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x178000UL: // (DORQ_REG_DQ_FIFO: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nDORQ_REG_DQ_FIFO (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Read access to DQ FIFO. The address will be composed of row\n     number; standing for a specific doorbell - 12 Lsbits and 1 Msbit; which\n     will be 1 to read the wait for done and CID  indication and 0 - for all\n     the other indications.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170060UL: // (DORQ_REG_AGG_CMD0: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD0 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Aggregation command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170064UL: // (DORQ_REG_AGG_CMD1: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD1 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Aggregation command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170068UL: // (DORQ_REG_AGG_CMD2: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD2 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Aggregation command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x17006cUL: // (DORQ_REG_AGG_CMD3: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nDORQ_REG_AGG_CMD3 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Aggregation command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701c0UL: // (DORQ_REG_DORQ_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701c8UL: // (DORQ_REG_DORQ_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701f0UL: // (DORQ_REG_DORQ_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1701f8UL: // (DORQ_REG_DORQ_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170350UL: // (DORQ_REG_DORQ_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x170500UL: // (DORQ_REG_DORQ_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nDORQ_REG_DORQ_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108000UL: // (HC_REG_CONFIG_0: ACCESS:RW  DataWidth:0xd  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_CONFIG_0 (ACCESS:RW  DataWidth:0xd  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMSI_MSIX_MEMORY_EN_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_MSI_MSIX_MEMORY_EN_0) >> 12)
				, "\n\tSTATISTIC_COUNTER_EN_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_STATISTIC_COUNTER_EN_0) >> 11)
				, "\n\tMAILBOX_COUNTER_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_MAILBOX_COUNTER_0) >> 9)
				, "\n\tMSIX_ATTN_EN_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_MSIX_ATTN_EN_0) >> 8)
				, "\n\tMSI_ATTN_EN_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_MSI_ATTN_EN_0) >> 7)
				, "\n\tCOALESCE_NOW_EN_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_COALESCE_NOW_EN_0) >> 6)
				, "\n\tNOT_DURING_INT_EN_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_NOT_DURING_INT_EN_0) >> 5)
				, "\n\tATTN_BIT_EN_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_ATTN_BIT_EN_0) >> 4)
				, "\n\tINT_LINE_EN_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_INT_LINE_EN_0) >> 3)
				, "\n\tMSI_MSIX_INT_EN_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_MSI_MSIX_INT_EN_0) >> 2)
				, "\n\tSINGLE_ISR_EN_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_SINGLE_ISR_EN_0) >> 1)
				, "\n\tBLOCK_DISABLE_0 :"
				, (u32_t)((data & HC_CONFIG_0_REG_BLOCK_DISABLE_0) >> 0));
			break;
		case 0x108004UL: // (HC_REG_CONFIG_1: ACCESS:RW  DataWidth:0xd  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_CONFIG_1 (ACCESS:RW  DataWidth:0xd  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMSI_MSIX_MEMORY_EN_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_MSI_MSIX_MEMORY_EN_1) >> 12)
				, "\n\tSTATISTIC_COUNTER_EN_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_STATISTIC_COUNTER_EN_1) >> 11)
				, "\n\tMAILBOX_COUNTER_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_MAILBOX_COUNTER_1) >> 9)
				, "\n\tMSIX_ATTN_EN_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_MSIX_ATTN_EN_1) >> 8)
				, "\n\tMSI_ATTN_EN_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_MSI_ATTN_EN_1) >> 7)
				, "\n\tCOALESCE_NOW_EN_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_COALESCE_NOW_EN_1) >> 6)
				, "\n\tNOT_DURING_INT_EN_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_NOT_DURING_INT_EN_1) >> 5)
				, "\n\tATTN_BIT_EN_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_ATTN_BIT_EN_1) >> 4)
				, "\n\tINT_LINE_EN_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_INT_LINE_EN_1) >> 3)
				, "\n\tMSI_MSIX_INT_EN_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_MSI_MSIX_INT_EN_1) >> 2)
				, "\n\tSINGLE_ISR_EN_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_SINGLE_ISR_EN_1) >> 1)
				, "\n\tBLOCK_DISABLE_1 :"
				, (u32_t)((data & HC_CONFIG_1_REG_BLOCK_DISABLE_1) >> 0));
			break;
		case 0x108008UL: // (HC_REG_VQID_0: ACCESS:RW  DataWidth:0xa  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_VQID_0 (ACCESS:RW  DataWidth:0xa  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVQID_ATTN_MSG_0 :"
				, (u32_t)((data & HC_VQID_0_REG_VQID_ATTN_MSG_0) >> 5)
				, "\n\tVQID_MSI_MSIX_0 :"
				, (u32_t)((data & HC_VQID_0_REG_VQID_MSI_MSIX_0) >> 0));
			break;
		case 0x10800cUL: // (HC_REG_VQID_1: ACCESS:RW  DataWidth:0xa  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_VQID_1 (ACCESS:RW  DataWidth:0xa  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVQID_ATTN_MSG_1 :"
				, (u32_t)((data & HC_VQID_1_REG_VQID_ATTN_MSG_1) >> 5)
				, "\n\tVQID_MSI_MSIX_1 :"
				, (u32_t)((data & HC_VQID_1_REG_VQID_MSI_MSIX_1) >> 0));
			break;
		case 0x108010UL: // (HC_REG_PCI_CONFIG_0: ACCESS:RW  DataWidth:0x4  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_PCI_CONFIG_0 (ACCESS:RW  DataWidth:0x4  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATTN_NO_SNOOP_0 :"
				, (u32_t)((data & HC_PCI_CONFIG_0_REG_ATTN_NO_SNOOP_0) >> 3)
				, "\n\tATTN_RELAX_0 :"
				, (u32_t)((data & HC_PCI_CONFIG_0_REG_ATTN_RELAX_0) >> 2)
				, "\n\tMSI_NO_SNOOP_0 :"
				, (u32_t)((data & HC_PCI_CONFIG_0_REG_MSI_NO_SNOOP_0) >> 1)
				, "\n\tMSI_RELAX_0 :"
				, (u32_t)((data & HC_PCI_CONFIG_0_REG_MSI_RELAX_0) >> 0));
			break;
		case 0x108014UL: // (HC_REG_PCI_CONFIG_1: ACCESS:RW  DataWidth:0x4  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_PCI_CONFIG_1 (ACCESS:RW  DataWidth:0x4  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATTN_NO_SNOOP_1 :"
				, (u32_t)((data & HC_PCI_CONFIG_1_REG_ATTN_NO_SNOOP_1) >> 3)
				, "\n\tATTN_RELAX_1 :"
				, (u32_t)((data & HC_PCI_CONFIG_1_REG_ATTN_RELAX_1) >> 2)
				, "\n\tMSI_NO_SNOOP_1 :"
				, (u32_t)((data & HC_PCI_CONFIG_1_REG_MSI_NO_SNOOP_1) >> 1)
				, "\n\tMSI_RELAX_1 :"
				, (u32_t)((data & HC_PCI_CONFIG_1_REG_MSI_RELAX_1) >> 0));
			break;
		case 0x108018UL: // (HC_REG_ATTN_MSG0_ADDR_L: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_ATTN_MSG0_ADDR_L (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: port 0 lower 32 bits address field for attn messag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10801cUL: // (HC_REG_ATTN_MSG0_ADDR_H: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_ATTN_MSG0_ADDR_H (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: port 0 higher 32 bits address field for attn messag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108020UL: // (HC_REG_ATTN_MSG1_ADDR_L: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_ATTN_MSG1_ADDR_L (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: port 1 lower 32 bits address field for attn messag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108024UL: // (HC_REG_ATTN_MSG1_ADDR_H: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_ATTN_MSG1_ADDR_H (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: port 1 higher 32 bits address field for attn messag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108028UL: // (HC_REG_UC_RAM_ADDR_0: ACCESS:RW  DataWidth:0x20  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_UC_RAM_ADDR_0 (ACCESS:RW  DataWidth:0x20  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCSTORM_0_ADDR :"
				, (u32_t)((data & HC_UC_RAM_ADDR_0_REG_CSTORM_0_ADDR) >> 16)
				, "\n\tUSTORM_0_ADDR :"
				, (u32_t)((data & HC_UC_RAM_ADDR_0_REG_USTORM_0_ADDR) >> 0));
			break;
		case 0x10802cUL: // (HC_REG_XT_RAM_ADDR_0: ACCESS:RW  DataWidth:0x20  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_XT_RAM_ADDR_0 (ACCESS:RW  DataWidth:0x20  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTSTORM_0_ADDR :"
				, (u32_t)((data & HC_XT_RAM_ADDR_0_REG_TSTORM_0_ADDR) >> 16)
				, "\n\tXSTORM_0_ADDR :"
				, (u32_t)((data & HC_XT_RAM_ADDR_0_REG_XSTORM_0_ADDR) >> 0));
			break;
		case 0x108030UL: // (HC_REG_UC_RAM_ADDR_1: ACCESS:RW  DataWidth:0x20  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_UC_RAM_ADDR_1 (ACCESS:RW  DataWidth:0x20  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCSTORM_1_ADDR :"
				, (u32_t)((data & HC_UC_RAM_ADDR_1_REG_CSTORM_1_ADDR) >> 16)
				, "\n\tUSTORM_1_ADDR :"
				, (u32_t)((data & HC_UC_RAM_ADDR_1_REG_USTORM_1_ADDR) >> 0));
			break;
		case 0x108034UL: // (HC_REG_XT_RAM_ADDR_1: ACCESS:RW  DataWidth:0x20  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_XT_RAM_ADDR_1 (ACCESS:RW  DataWidth:0x20  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTSTORM_1_ADDR :"
				, (u32_t)((data & HC_XT_RAM_ADDR_1_REG_TSTORM_1_ADDR) >> 16)
				, "\n\tXSTORM_1_ADDR :"
				, (u32_t)((data & HC_XT_RAM_ADDR_1_REG_XSTORM_1_ADDR) >> 0));
			break;
		case 0x108038UL: // (HC_REG_ATTN_NUM_P0: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_ATTN_NUM_P0 (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: status block number for attn bit msg - function 0;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10803cUL: // (HC_REG_ATTN_NUM_P1: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_ATTN_NUM_P1 (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: status block number for attn bit msg - function 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108040UL: // (HC_REG_LEADING_EDGE_0: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_LEADING_EDGE_0 (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: port 0 attn bit condition monitoring; each bit that is set\n     will lock a change fron 0 to 1 in the corresponding attention signals\n     that comes from the AEU\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108044UL: // (HC_REG_TRAILING_EDGE_0: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_TRAILING_EDGE_0 (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: port 0 attn bit condition monitoring; each bit that is set\n     will lock a change fron 1 to 0 in the corresponding attention signals\n     that comes from the AEU\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108048UL: // (HC_REG_LEADING_EDGE_1: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_LEADING_EDGE_1 (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: port 1 attn bit condition monitoring; each bit that is set\n     will lock a change fron 0 to 1 in the corresponding attention signals\n     that comes from the AEU\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10804cUL: // (HC_REG_TRAILING_EDGE_1: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_TRAILING_EDGE_1 (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: port 1 attn bit condition monitoring; each bit that is set\n     will lock a change fron 1 to 0 in the corresponding attention signals\n     that comes from the AEU\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108050UL: // (HC_REG_AGG_INT_0: ACCESS:RW  DataWidth:0xa  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_AGG_INT_0 (ACCESS:RW  DataWidth:0xa  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tAGG_INT_CONSUMER_UPD_0 :"
				, (u32_t)((data & HC_AGG_INT_0_REG_AGG_INT_CONSUMER_UPD_0) >> 5)
				, "\n\tAGG_INT_COALESCE_NOW_0 :"
				, (u32_t)((data & HC_AGG_INT_0_REG_AGG_INT_COALESCE_NOW_0) >> 0));
			break;
		case 0x108054UL: // (HC_REG_AGG_INT_1: ACCESS:RW  DataWidth:0xa  SPLIT:4  Multi Field Register  ) 
			sprintf(disp_str, "\nHC_REG_AGG_INT_1 (ACCESS:RW  DataWidth:0xa  SPLIT:4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tAGG_INT_CONSUMER_UPD_1 :"
				, (u32_t)((data & HC_AGG_INT_1_REG_AGG_INT_CONSUMER_UPD_1) >> 5)
				, "\n\tAGG_INT_COALESCE_NOW_1 :"
				, (u32_t)((data & HC_AGG_INT_1_REG_AGG_INT_COALESCE_NOW_1) >> 0));
			break;
		case 0x108058UL: // (HC_REG_INTERRUPT_A: ACCESS:R  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_INTERRUPT_A (ACCESS:R  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: read the interrupt b line value; 0 = asserted; 1= deasserted\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10805cUL: // (HC_REG_INTERRUPT_B: ACCESS:R  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_INTERRUPT_B (ACCESS:R  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: read the interrupt a line value; 0 = asserted; 1= deasserted\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108060UL: // (HC_REG_STATISTIC_DIVIDER: ACCESS:RW  DataWidth:0x14  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_STATISTIC_DIVIDER (ACCESS:RW  DataWidth:0x14  SPLIT:4 )" \
				"\nDescription: Define the time statistic counter timer resolution (period\n     rate of HW and SW intack counters update); minimuv value = 2500\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108064UL: // (HC_REG_COUNERTS_MEM_TM: ACCESS:RW  DataWidth:0x4  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_COUNERTS_MEM_TM (ACCESS:RW  DataWidth:0x4  SPLIT:4 )" \
				"\nDescription: tm bits for statistic counters memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108068UL: // (HC_REG_USTORM_ADDR_FOR_COALESCE: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_USTORM_ADDR_FOR_COALESCE (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: UNUSED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10806cUL: // (HC_REG_CSTORM_ADDR_FOR_COALESCE: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_CSTORM_ADDR_FOR_COALESCE (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: UNUSED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108070UL: // (HC_REG_XSTORM_ADDR_FOR_COALESCE: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_XSTORM_ADDR_FOR_COALESCE (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: UNUSED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108074UL: // (HC_REG_TSTORM_ADDR_FOR_COALESCE: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_TSTORM_ADDR_FOR_COALESCE (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: UNUSED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108078UL: // (HC_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from HC\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10807cUL: // (HC_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from HC\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108080UL: // (HC_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3  SPLIT:4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from HC\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108084UL: // (HC_REG_HC_INT_STS: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nHC_REG_HC_INT_STS (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCONS_OR_PROD_IDX_TOO_BIG :"
				, (u32_t)((data & HC_HC_INT_STS_REG_CONS_OR_PROD_IDX_TOO_BIG) >> 6)
				, "\n\tSTATISTIC_COUNTER_MEMORY :"
				, (u32_t)((data & HC_HC_INT_STS_REG_STATISTIC_COUNTER_MEMORY) >> 5)
				, "\n\tMAIN_MEMORY :"
				, (u32_t)((data & HC_HC_INT_STS_REG_MAIN_MEMORY) >> 4)
				, "\n\tTMP_STORM_ID_NOT_MATCH :"
				, (u32_t)((data & HC_HC_INT_STS_REG_TMP_STORM_ID_NOT_MATCH) >> 3)
				, "\n\tMME_IS_BIGGER_THEN_5 :"
				, (u32_t)((data & HC_HC_INT_STS_REG_MME_IS_BIGGER_THEN_5) >> 2)
				, "\n\tFIFO_ERROR :"
				, (u32_t)((data & HC_HC_INT_STS_REG_FIFO_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & HC_HC_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x108088UL: // (HC_REG_HC_INT_STS_CLR: ACCESS:RC  DataWidth:0x7 ) 
			sprintf(disp_str, "\nHC_REG_HC_INT_STS_CLR (ACCESS:RC  DataWidth:0x7 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCONS_OR_PROD_IDX_TOO_BIG :"
				, (u32_t)((data & HC_HC_INT_STS_CLR_REG_CONS_OR_PROD_IDX_TOO_BIG) >> 6)
				, "\n\tSTATISTIC_COUNTER_MEMORY :"
				, (u32_t)((data & HC_HC_INT_STS_CLR_REG_STATISTIC_COUNTER_MEMORY) >> 5)
				, "\n\tMAIN_MEMORY :"
				, (u32_t)((data & HC_HC_INT_STS_CLR_REG_MAIN_MEMORY) >> 4)
				, "\n\tTMP_STORM_ID_NOT_MATCH :"
				, (u32_t)((data & HC_HC_INT_STS_CLR_REG_TMP_STORM_ID_NOT_MATCH) >> 3)
				, "\n\tMME_IS_BIGGER_THEN_5 :"
				, (u32_t)((data & HC_HC_INT_STS_CLR_REG_MME_IS_BIGGER_THEN_5) >> 2)
				, "\n\tFIFO_ERROR :"
				, (u32_t)((data & HC_HC_INT_STS_CLR_REG_FIFO_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & HC_HC_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x10808cUL: // (HC_REG_HC_INT_STS_WR: ACCESS:WR  DataWidth:0x7 ) 
			sprintf(disp_str, "\nHC_REG_HC_INT_STS_WR (ACCESS:WR  DataWidth:0x7 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCONS_OR_PROD_IDX_TOO_BIG :"
				, (u32_t)((data & HC_HC_INT_STS_WR_REG_CONS_OR_PROD_IDX_TOO_BIG) >> 6)
				, "\n\tSTATISTIC_COUNTER_MEMORY :"
				, (u32_t)((data & HC_HC_INT_STS_WR_REG_STATISTIC_COUNTER_MEMORY) >> 5)
				, "\n\tMAIN_MEMORY :"
				, (u32_t)((data & HC_HC_INT_STS_WR_REG_MAIN_MEMORY) >> 4)
				, "\n\tTMP_STORM_ID_NOT_MATCH :"
				, (u32_t)((data & HC_HC_INT_STS_WR_REG_TMP_STORM_ID_NOT_MATCH) >> 3)
				, "\n\tMME_IS_BIGGER_THEN_5 :"
				, (u32_t)((data & HC_HC_INT_STS_WR_REG_MME_IS_BIGGER_THEN_5) >> 2)
				, "\n\tFIFO_ERROR :"
				, (u32_t)((data & HC_HC_INT_STS_WR_REG_FIFO_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & HC_HC_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x108090UL: // (HC_REG_HC_INT_MASK: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nHC_REG_HC_INT_MASK (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCONS_OR_PROD_IDX_TOO_BIG :"
				, (u32_t)((data & HC_HC_INT_MASK_REG_CONS_OR_PROD_IDX_TOO_BIG) >> 6)
				, "\n\tSTATISTIC_COUNTER_MEMORY :"
				, (u32_t)((data & HC_HC_INT_MASK_REG_STATISTIC_COUNTER_MEMORY) >> 5)
				, "\n\tMAIN_MEMORY :"
				, (u32_t)((data & HC_HC_INT_MASK_REG_MAIN_MEMORY) >> 4)
				, "\n\tTMP_STORM_ID_NOT_MATCH :"
				, (u32_t)((data & HC_HC_INT_MASK_REG_TMP_STORM_ID_NOT_MATCH) >> 3)
				, "\n\tMME_IS_BIGGER_THEN_5 :"
				, (u32_t)((data & HC_HC_INT_MASK_REG_MME_IS_BIGGER_THEN_5) >> 2)
				, "\n\tFIFO_ERROR :"
				, (u32_t)((data & HC_HC_INT_MASK_REG_FIFO_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & HC_HC_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x108094UL: // (HC_REG_HC_PRTY_STS: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nHC_REG_HC_PRTY_STS (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCOUNTERS_MEMORY :"
				, (u32_t)((data & HC_HC_PRTY_STS_REG_COUNTERS_MEMORY) >> 2)
				, "\n\tMAIN_MEMORY :"
				, (u32_t)((data & HC_HC_PRTY_STS_REG_MAIN_MEMORY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & HC_HC_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x108098UL: // (HC_REG_HC_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x3 ) 
			sprintf(disp_str, "\nHC_REG_HC_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x3 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCOUNTERS_MEMORY :"
				, (u32_t)((data & HC_HC_PRTY_STS_CLR_REG_COUNTERS_MEMORY) >> 2)
				, "\n\tMAIN_MEMORY :"
				, (u32_t)((data & HC_HC_PRTY_STS_CLR_REG_MAIN_MEMORY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & HC_HC_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x10809cUL: // (HC_REG_HC_PRTY_STS_WR: ACCESS:WR  DataWidth:0x3 ) 
			sprintf(disp_str, "\nHC_REG_HC_PRTY_STS_WR (ACCESS:WR  DataWidth:0x3 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCOUNTERS_MEMORY :"
				, (u32_t)((data & HC_HC_PRTY_STS_WR_REG_COUNTERS_MEMORY) >> 2)
				, "\n\tMAIN_MEMORY :"
				, (u32_t)((data & HC_HC_PRTY_STS_WR_REG_MAIN_MEMORY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & HC_HC_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x1080a0UL: // (HC_REG_HC_PRTY_MASK: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nHC_REG_HC_PRTY_MASK (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCOUNTERS_MEMORY :"
				, (u32_t)((data & HC_HC_PRTY_MASK_REG_COUNTERS_MEMORY) >> 2)
				, "\n\tMAIN_MEMORY :"
				, (u32_t)((data & HC_HC_PRTY_MASK_REG_MAIN_MEMORY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & HC_HC_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x1080acUL: // (HC_REG_FUNC_NUM_P0: ACCESS:RW  DataWidth:0x3  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_FUNC_NUM_P0 (ACCESS:RW  DataWidth:0x3  SPLIT:4 )" \
				"\nDescription: function number for MSI MSIX & Attn message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1080b0UL: // (HC_REG_FUNC_NUM_P1: ACCESS:RW  DataWidth:0x3  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_FUNC_NUM_P1 (ACCESS:RW  DataWidth:0x3  SPLIT:4 )" \
				"\nDescription: function number for MSI MSIX & Attn message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1080b4UL: // (HC_REG_AEU_MSI_ATTN_P0: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nHC_REG_AEU_MSI_ATTN_P0 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: read hc_misc_aeu_msi_msix_attn_p0 of all the cores. [0] HC_0\n    ; [1] HC_1; [2] HC_2; [3] HC_3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1080b8UL: // (HC_REG_AEU_MSI_ATTN_P1: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nHC_REG_AEU_MSI_ATTN_P1 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: read hc_misc_aeu_msi_msix_attn_p1 of all the cores. [0] HC_0\n    ; [1] HC_1; [2] HC_2; [3] HC_4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1080bcUL: // (HC_REG_PXP_REQUESTER_CREDIT: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nHC_REG_PXP_REQUESTER_CREDIT (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: the credit for the PXP requester interface. This register\n     is common for all the HC cores. Only 0ne and two are valid values.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108100UL: // (HC_REG_ATTN_IDX: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_ATTN_IDX (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: attn bits status index for attn bit msg; addr 0 - function\n     0; addr 1 - functin 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108108UL: // (HC_REG_INT_MASK: ACCESS:RW  DataWidth:0x11  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_INT_MASK (ACCESS:RW  DataWidth:0x11  SPLIT:4 )" \
				"\nDescription: status block interrupt mask; one in each bit means unmask\n    ; zerow in each bit means mask; bit 0 - default SB; bit 1 - SB_0; bit \n    2 - SB_1 to bit 16- SB_15; addr 0 - port 0; addr 1 - port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108110UL: // (HC_REG_PBA_BIT: ACCESS:RW  DataWidth:0x11  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_PBA_BIT (ACCESS:RW  DataWidth:0x11  SPLIT:4 )" \
				"\nDescription: PBA 17 bits adress 0 - port0; address 1 - port 1; bit0 = \n    default SB; bit1 = SB_0; bit2 = SB_1 . . . bit16 = SB_15; Each bit in \n    the vector is updated according with scan procedure for MSI/MSI-X message\n     build and posting according to the following rules: If the pending is\n     set AND For the relevant port MSI-X is enabled AND Block is not disabled\n     (per port indication) AND No full indication from the PCI command transmission\n     interface logic AND (relevant MSI-X function mask bit is set OR relevant\n     MSI-X vector mask bit is set) Then the analogous PBA bit is set; The \n    bit is cleared upon posting an MSI-X message for the analogous pending\n     bit.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108120UL: // (HC_REG_ATTN_BIT: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_ATTN_BIT (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: attention bit and attention acknowledge bits status for port\n     0 and 1 according to the following address map: addr 0 - attn_bit_0; \n    addr 1 - attn_ack_bit_0; addr 2 - attn_bit_1; addr 3 - attn_ack_bit_1;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108130UL: // (HC_REG_DEC_CEILING: ACCESS:W  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_DEC_CEILING (ACCESS:W  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: This command register infuence the MSI configuration state\n     machine. This register is write only and has 4 addresses as follow: 0\n     = dec port 0; 1 = ceiling port 0; 2 = dec port 1; 3 = ceiling port 1;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108140UL: // (HC_REG_PBA_COMMAND: ACCESS:W  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_PBA_COMMAND (ACCESS:W  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: This register is write only and has 4 addresses as follow\n    : 0 = clear all PBA bits port 0; 1 = clear all pending interrupts request\n     port0; 2 = clear all PBA bits port 1; 3 = clear all pending interrupts\n     request port1;there is no meaning for the data in this register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108180UL: // (HC_REG_COMMAND_REG: ACCESS:RW  DataWidth:0x1b  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_COMMAND_REG (ACCESS:RW  DataWidth:0x1b  SPLIT:4 )" \
				"\nDescription: This command register has 16 addresses. Write to these registers\n     active command according to the address. For port 0 the address mapped\n     as follow: 0-interrupt acknowledge ; 1-producer update ; 2-attntion bits\n     update; 3-attntion bits set; 4-attntion bits clear; 5-coalesce now; 6\n    -single_isr_multi_dpc with mask; 7-single isr without mask; For port 1\n     the address mapped as follow: 8-interrupt acknowledge ; 9-producer update\n     ; 10-attntion bits update; 11-attntion bits set; 12-attntion bits clear\n    ; 13-coalesce now; 14-single_isr_multi_dpc with mask; 15-single isr without\n     mask; Read from addresses 0-5 and 8-13 is ignored; Write to addresses\n     6-7 and 14-15 is ignored; the data bits are mapped as follow: tmp_dis_enable_cmd\n     = data[26:25]; tmp_upd_index_cmd = data[24]; tmp_cmd_storm_index = data\n    [23:21] // according to storm table; tmp_cmd_status_id = data[20:16] // \n    according to status block idx table; tmp_index_val = data[15:0]; tmp_function\n     = according to MSB address. if address match \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108200UL: // (HC_REG_P0_PROD_CONS: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_P0_PROD_CONS (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: all producer and consumer of port 0 according to the following\n     addresses; U_prod: 0-15; C_prod: 16-31; U_cons: 32-47; C_cons:48-63; \n    Defoult_prod: U/C/X/T/Attn-64/65/66/67/68; Defoult_cons: U/C/X/T/Attn-\n    69/70/71/72/73\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108400UL: // (HC_REG_P1_PROD_CONS: ACCESS:RW  DataWidth:0x10  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_P1_PROD_CONS (ACCESS:RW  DataWidth:0x10  SPLIT:4 )" \
				"\nDescription: all producer and consumer of port 1according to the following\n     addresses;  U_prod: 0-15; C_prod: 16-31; U_cons: 32-47; C_cons:48-63; \n    Defoult_prod: U/C/X/T/Attn-64/65/66/67/68; Defoult_cons: U/C/X/T/Attn-\n    69/70/71/72/73\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108800UL: // (HC_REG_MAIN_MEMORY: ACCESS:WB  DataWidth:0x40  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_MAIN_MEMORY (ACCESS:WB  DataWidth:0x40  SPLIT:4 )" \
				"\nDescription: The MSI and MSIX memory; each address has 64 bits of data\n    ; the address map is as follows; Port 0: addr 0-33 - MSIX_vector (acording\n     to the following distribution: 0 - default SB MSIX addr; 1 - default \n    SB msix data and mask (mask[32];data[31:0]); 2 - SB_0 MSIX addr; 3 - SB_0\n     msix data and mask (mask[32];data[31:0]); 4 - SB_1 MSIX addr; 5 - SB_1\n     msix data and mask (mask[32];data[31:0]);... 32 - SB_16 MSIX addr; 33\n     - SB_16 msix data and mask (mask[32];data[31:0])); addr 34 - MSI_msg_ctrl\n    ; addr 35 - MSI_addr_low; addr 36 - MSI_addr_high ; addr 37 - [63:32] - \n    MSI_mask_bit [31:0] - MSI_data. Port 1: addr 38-71 - MSIX_vector (acording\n     to the following distribution: 38 - default SB MSIX addr; 39 - default\n     SB msix data and mask (mask[32];data[31:0]); 40 - SB_0 MSIX addr; 41 - \n    SB_0 msix data and mask (mask[32];data[31:0]); 42 - SB_1 MSIX addr; 43\n     - SB_1 msix data and mask (mask[32];data[31:0]);... 70 - SB_16 MSIX addr\n    ; 71 - SB_16 msix data and mask (mask[32];data[31:0])); addr 72 - \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x109000UL: // (HC_REG_STATISTIC_COUNTERS: ACCESS:RW  DataWidth:0x18  SPLIT:4 ) 
			sprintf(disp_str, "\nHC_REG_STATISTIC_COUNTERS (ACCESS:RW  DataWidth:0x18  SPLIT:4 )" \
				"\nDescription: all counters acording to the following address: LSB: 0=read\n    ; 1= read_clear; 0-71 = HW counters (the inside order is the same as the\n     interrupt table in the spec); 72-219 = SW counters 1 (stops after first\n     consumer upd) the inside order  is: 72-103 - U_non_default_p0; 104-135\n     C_non_defaul_p0;136-145 U/C/X/T/Attn_default_p0; 146-177 U_non_default_p1\n    ; 178-209 C_non_defaul_p1;210-219 U/C/X/T/Attn_default_p1 ; 220-367 = \n    SW counters 2 (stops when prod=cons) the inside order is: 220-251 - U_non_default_p0\n    ; 252-283 C_non_defaul_p0;284-293 U/C/X/T/Attn_default_p0; 294-325 U_non_default_p1\n    ; 326-357 C_non_defaul_p1;358-367 U/C/X/T/Attn_default_p1 ; 368-515 = \n    mailbox counters; (the inside order of the mailbox counter is 368-431 \n    U and C non_default_p0; 432-441 U/C//X/T/Attn_default_p0; 442-505 U and\n     C non_default_p1; 506-515 U/C//X/T/Attn_default_p1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1080a4UL: // (HC_REG_HC_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nHC_REG_HC_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1080c0UL: // (HC_REG_HC_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nHC_REG_HC_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108118UL: // (HC_REG_HC_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nHC_REG_HC_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108150UL: // (HC_REG_HC_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nHC_REG_HC_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1081c0UL: // (HC_REG_HC_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nHC_REG_HC_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108600UL: // (HC_REG_HC_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nHC_REG_HC_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x108c00UL: // (HC_REG_HC_UNUSED_EMPTY_6: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nHC_REG_HC_UNUSED_EMPTY_6 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10a000UL: // (HC_REG_HC_UNUSED_EMPTY_7: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nHC_REG_HC_UNUSED_EMPTY_7 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130000UL: // (IGU_REG_BLOCK_CONFIGURATION: ACCESS:RW  DataWidth:0x5  Multi Field Register  ) 
			sprintf(disp_str, "\nIGU_REG_BLOCK_CONFIGURATION (ACCESS:RW  DataWidth:0x5  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTIMER_MASK_EN :"
				, (u32_t)((data & IGU_BLOCK_CONFIGURATION_REG_TIMER_MASK_EN) >> 4)
				, "\n\tPORT4MODE_EN :"
				, (u32_t)((data & IGU_BLOCK_CONFIGURATION_REG_PORT4MODE_EN) >> 3)
				, "\n\tBUS_DWORD_SELECT :"
				, (u32_t)((data & IGU_BLOCK_CONFIGURATION_REG_BUS_DWORD_SELECT) >> 2)
				, "\n\tBACKWARD_COMP_EN :"
				, (u32_t)((data & IGU_BLOCK_CONFIGURATION_REG_BACKWARD_COMP_EN) >> 1)
				, "\n\tBLOCK_ENABLE :"
				, (u32_t)((data & IGU_BLOCK_CONFIGURATION_REG_BLOCK_ENABLE) >> 0));
			break;
		case 0x130004UL: // (IGU_REG_MESSAGE_FIELDS: ACCESS:RW  DataWidth:0x1a  Multi Field Register  ) 
			sprintf(disp_str, "\nIGU_REG_MESSAGE_FIELDS (ACCESS:RW  DataWidth:0x1a  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATTN_WRITE_DONE_TYPE :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_ATTN_WRITE_DONE_TYPE) >> 25)
				, "\n\tATTN_NS :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_ATTN_NS) >> 24)
				, "\n\tATTN_RO :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_ATTN_RO) >> 23)
				, "\n\tATTN_ATC :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_ATTN_ATC) >> 20)
				, "\n\tATTN_VQID :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_ATTN_VQID) >> 15)
				, "\n\tMESSAGE_FIELDS_RESEVED :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_MESSAGE_FIELDS_RESEVED) >> 11)
				, "\n\tMSIX_WRITE_DONE_TYPE :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_MSIX_WRITE_DONE_TYPE) >> 10)
				, "\n\tMSI_MSIX_NS :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_MSI_MSIX_NS) >> 9)
				, "\n\tMSI_MSIX_RO :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_MSI_MSIX_RO) >> 8)
				, "\n\tMSI_MSIX_ATC :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_MSI_MSIX_ATC) >> 5)
				, "\n\tMSI_MSIX_VQID :"
				, (u32_t)((data & IGU_MESSAGE_FIELDS_REG_MSI_MSIX_VQID) >> 0));
			break;
		case 0x130008UL: // (IGU_REG_FUNC_WITH_MORE_16_SB_0: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nIGU_REG_FUNC_WITH_MORE_16_SB_0 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: The first function numbers that has more then 16 SBs. [6:\n    0] - function number (if VF - [6] = 0; [5:0] = VFID if PF - [6] = 1; [\n    5:3] = 0; [2:0] = PFID); . [7] reserved. [8]  - valid bit. In backward\n     compatible mode - not used. The reset value match 2ports mode operation\n    . in 4ports mode it should be reconfigured.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13000cUL: // (IGU_REG_FUNC_WITH_MORE_16_SB_1: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nIGU_REG_FUNC_WITH_MORE_16_SB_1 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: The second function numbers that has more then 16 SBs. [6\n    :0] - function number (if VF - [6] = 0; [5:0] = VFID if PF - [6] = 1; [\n    5:3] = 0; [2:0] = PFID); . [7] reserved. [8]  - valid bit. In backward\n     compatible mode - not used. The reset value match 2ports mode operation\n    . in 4ports mode it should be reconfigured.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130010UL: // (IGU_REG_FUNC_WITH_MORE_16_SB_2: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nIGU_REG_FUNC_WITH_MORE_16_SB_2 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: The third function numbers that has more then 16 SBs. [6:\n    0] - function number (if VF - [6] = 0; [5:0] = VFID if PF - [6] = 1; [\n    5:3] = 0; [2:0] = PFID); . [7] reserved. [8]  - valid bit. In backward\n     compatible mode - not used. The reset value match 2ports mode operation\n    . in 4ports mode it should be reconfigured.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130014UL: // (IGU_REG_FUNC_WITH_MORE_16_SB_3: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nIGU_REG_FUNC_WITH_MORE_16_SB_3 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: The forth function numbers that has more then 16 SBs. [6:\n    0] - function number (if VF - [6] = 0; [5:0] = VFID if PF - [6] = 1; [\n    5:3] = 0; [2:0] = PFID); . [7] reserved. [8]  - valid bit. In backward\n     compatible mode - not used. The reset value match 2ports mode operation\n    . in 4ports mode it should be reconfigured.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130018UL: // (IGU_REG_FUNC_WITH_MORE_16_SB_4: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nIGU_REG_FUNC_WITH_MORE_16_SB_4 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: The fifth function numbers that has more then 16 SBs. [6:\n    0] - function number (if VF - [6] = 0; [5:0] = VFID if PF - [6] = 1; [\n    5:3] = 0; [2:0] = PFID); . [7] reserved. [8]  - valid bit. In backward\n     compatible mode - not used. The reset value match 2ports mode operation\n    . in 4ports mode it should be reconfigured.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13001cUL: // (IGU_REG_FUNC_WITH_MORE_16_SB_5: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nIGU_REG_FUNC_WITH_MORE_16_SB_5 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: The sixth function numbers that has more then 16 SBs. [6:\n    0] - function number (if VF - [6] = 0; [5:0] = VFID if PF - [6] = 1; [\n    5:3] = 0; [2:0] = PFID); . [7] reserved. [8]  - valid bit. In backward\n     compatible mode - not used. The reset value match 2ports mode operation\n    . in 4ports mode it should be reconfigured.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130020UL: // (IGU_REG_FUNC_WITH_MORE_16_SB_6: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nIGU_REG_FUNC_WITH_MORE_16_SB_6 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: The seventh function numbers that has more then 16 SBs. [\n    6:0] - function number (if VF - [6] = 0; [5:0] = VFID if PF - [6] = 1; [\n    5:3] = 0; [2:0] = PFID); . [7] reserved. [8]  - valid bit. In backward\n     compatible mode - not used. The reset value match 2ports mode operation\n    . in 4ports mode it should be reconfigured.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130024UL: // (IGU_REG_FUNC_WITH_MORE_16_SB_7: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nIGU_REG_FUNC_WITH_MORE_16_SB_7 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: The eight function numbers that has more then 16 SBs. [6:\n    0] - function number (if VF - [6] = 0; [5:0] = VFID if PF - [6] = 1; [\n    5:3] = 0; [2:0] = PFID); . [7] reserved. [8]  - valid bit. In backward\n     compatible mode - not used. The reset value match 2ports mode operation\n    . in 4ports mode it should be reconfigured.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130028UL: // (IGU_REG_PXP_REQUESTER_INITIAL_CREDIT: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nIGU_REG_PXP_REQUESTER_INITIAL_CREDIT (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: PXP req credit. The max number of outstanding messages to\n     the PXP request. The value can be one or two only.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13002cUL: // (IGU_REG_ATTN_MSG_PENDING: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nIGU_REG_ATTN_MSG_PENDING (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Debug: messages that wait to be sent; but didnt were sent\n     yet. One bit for each PFID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130030UL: // (IGU_REG_ATTN_WRITE_DONE_PENDING: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nIGU_REG_ATTN_WRITE_DONE_PENDING (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug: [3] - attention write done message is pending (0-no\n     pending; 1-pending). [2:0] = PFID. Pending means attention message was\n     sent; but write done didnt receive.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130034UL: // (IGU_REG_COMMAND_DEBUG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nIGU_REG_COMMAND_DEBUG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only: 0 - FIFO collects eight first error messages; \n    1 - FIFO collects eight last incoming command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130038UL: // (IGU_REG_STATISTIC_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nIGU_REG_STATISTIC_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: enabling to collect data in the statistic_num_message_sent\n     memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13003cUL: // (IGU_REG_TIMER_MASKING_VALUE: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nIGU_REG_TIMER_MASKING_VALUE (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Number of cycles the timer mask masking the IGU interrupt\n     when a timer mask command arrives. Value must be bigger than 100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130040UL: // (IGU_REG_TIMER_MASK_ACTIVE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nIGU_REG_TIMER_MASK_ACTIVE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: when set the timer mask is active and the IGU does not send\n     interrupts. When clear the timer mask is inactive.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130044UL: // (IGU_REG_ERROR_HANDLING_FILTER: ACCESS:RW  DataWidth:0x9  Multi Field Register  ) 
			sprintf(disp_str, "\nIGU_REG_ERROR_HANDLING_FILTER (ACCESS:RW  DataWidth:0x9  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tERROR_HANDLING_FILTER_EN :"
				, (u32_t)((data & IGU_ERROR_HANDLING_FILTER_REG_ERROR_HANDLING_FILTER_EN) >> 8)
				, "\n\tERROR_HANDLING_RESERVED :"
				, (u32_t)((data & IGU_ERROR_HANDLING_FILTER_REG_ERROR_HANDLING_RESERVED) >> 7)
				, "\n\tERROR_HANDLING_FILTER_FID :"
				, (u32_t)((data & IGU_ERROR_HANDLING_FILTER_REG_ERROR_HANDLING_FILTER_FID) >> 0));
			break;
		case 0x130048UL: // (IGU_REG_MAPPING_FSM: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nIGU_REG_MAPPING_FSM (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug: mapping_fsm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13004cUL: // (IGU_REG_SB_CTRL_FSM: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nIGU_REG_SB_CTRL_FSM (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug: sb_ctrl_fsm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130050UL: // (IGU_REG_INT_HANDLE_FSM: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nIGU_REG_INT_HANDLE_FSM (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug: int_handle_fsm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130054UL: // (IGU_REG_ATTN_FSM: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nIGU_REG_ATTN_FSM (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug: attn_fsm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130058UL: // (IGU_REG_PBA_FSM: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nIGU_REG_PBA_FSM (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug: pba_fsm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13005cUL: // (IGU_REG_MSIX_MSG_BUILDER_FSM: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nIGU_REG_MSIX_MSG_BUILDER_FSM (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Debug: msix_msg_builder_fsm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130060UL: // (IGU_REG_MSIX_MEM_FSM: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nIGU_REG_MSIX_MEM_FSM (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Debug: msix_mem_fsm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130064UL: // (IGU_REG_CTRL_FSM: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nIGU_REG_CTRL_FSM (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Debug: ctrl_fsm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130068UL: // (IGU_REG_PXP_ARB_FSM: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nIGU_REG_PXP_ARB_FSM (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Debug: pxp_arb_fsm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13006cUL: // (IGU_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nIGU_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from HC\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130070UL: // (IGU_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nIGU_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from HC\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130074UL: // (IGU_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nIGU_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from HC\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130078UL: // (IGU_REG_TM: ACCESS:RW  DataWidth:0x18  Multi Field Register  ) 
			sprintf(disp_str, "\nIGU_REG_TM (ACCESS:RW  DataWidth:0x18  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMSIX_MEM_TM :"
				, (u32_t)((data & IGU_TM_REG_MSIX_MEM_TM) >> 19)
				, "\n\tSB_MEM_TM :"
				, (u32_t)((data & IGU_TM_REG_SB_MEM_TM) >> 14)
				, "\n\tCAM_TM :"
				, (u32_t)((data & IGU_TM_REG_CAM_TM) >> 0));
			break;
		case 0x13007cUL: // (IGU_REG_CAM_BIST_BDG_DATA: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nIGU_REG_CAM_BIST_BDG_DATA (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug: for CAM bist uses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130080UL: // (IGU_REG_CAM_BIST_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nIGU_REG_CAM_BIST_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug: for CAM bist uses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130084UL: // (IGU_REG_CAM_BIST_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nIGU_REG_CAM_BIST_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug: for CAM bist uses\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130088UL: // (IGU_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nIGU_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: reserved for ECO if needed\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13008cUL: // (IGU_REG_IGU_INT_STS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nIGU_REG_IGU_INT_STS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATTN_PROD_ACC :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_ATTN_PROD_ACC) >> 10)
				, "\n\tSEGMENT_ACCESS_INVALID :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_SEGMENT_ACCESS_INVALID) >> 9)
				, "\n\tCMD_FID_NOT_MATCH :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_CMD_FID_NOT_MATCH) >> 8)
				, "\n\tDURIN_INT_READ_WITH_SIMD_DIS :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_DURIN_INT_READ_WITH_SIMD_DIS) >> 7)
				, "\n\tSB_INDEX_IS_NOT_VALID :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_SB_INDEX_IS_NOT_VALID) >> 6)
				, "\n\tMME_BIGGER_THEN_5 :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_MME_BIGGER_THEN_5) >> 5)
				, "\n\tVF_TRIES2ACC_ATTN_CMD :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_VF_TRIES2ACC_ATTN_CMD) >> 4)
				, "\n\tHOST_TRIES2ACCESS_PROD_UPD :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_HOST_TRIES2ACCESS_PROD_UPD) >> 3)
				, "\n\tPXP_REQ_LENGTH_TOO_BIG :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_PXP_REQ_LENGTH_TOO_BIG) >> 2)
				, "\n\tCTRL_FIFO_ERROR_ERR :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_CTRL_FIFO_ERROR_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & IGU_IGU_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x130090UL: // (IGU_REG_IGU_INT_STS_CLR: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nIGU_REG_IGU_INT_STS_CLR (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATTN_PROD_ACC :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_ATTN_PROD_ACC) >> 10)
				, "\n\tSEGMENT_ACCESS_INVALID :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_SEGMENT_ACCESS_INVALID) >> 9)
				, "\n\tCMD_FID_NOT_MATCH :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_CMD_FID_NOT_MATCH) >> 8)
				, "\n\tDURIN_INT_READ_WITH_SIMD_DIS :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_DURIN_INT_READ_WITH_SIMD_DIS) >> 7)
				, "\n\tSB_INDEX_IS_NOT_VALID :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_SB_INDEX_IS_NOT_VALID) >> 6)
				, "\n\tMME_BIGGER_THEN_5 :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_MME_BIGGER_THEN_5) >> 5)
				, "\n\tVF_TRIES2ACC_ATTN_CMD :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_VF_TRIES2ACC_ATTN_CMD) >> 4)
				, "\n\tHOST_TRIES2ACCESS_PROD_UPD :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_HOST_TRIES2ACCESS_PROD_UPD) >> 3)
				, "\n\tPXP_REQ_LENGTH_TOO_BIG :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_PXP_REQ_LENGTH_TOO_BIG) >> 2)
				, "\n\tCTRL_FIFO_ERROR_ERR :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_CTRL_FIFO_ERROR_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & IGU_IGU_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x130094UL: // (IGU_REG_IGU_INT_STS_WR: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nIGU_REG_IGU_INT_STS_WR (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATTN_PROD_ACC :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_ATTN_PROD_ACC) >> 10)
				, "\n\tSEGMENT_ACCESS_INVALID :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_SEGMENT_ACCESS_INVALID) >> 9)
				, "\n\tCMD_FID_NOT_MATCH :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_CMD_FID_NOT_MATCH) >> 8)
				, "\n\tDURIN_INT_READ_WITH_SIMD_DIS :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_DURIN_INT_READ_WITH_SIMD_DIS) >> 7)
				, "\n\tSB_INDEX_IS_NOT_VALID :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_SB_INDEX_IS_NOT_VALID) >> 6)
				, "\n\tMME_BIGGER_THEN_5 :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_MME_BIGGER_THEN_5) >> 5)
				, "\n\tVF_TRIES2ACC_ATTN_CMD :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_VF_TRIES2ACC_ATTN_CMD) >> 4)
				, "\n\tHOST_TRIES2ACCESS_PROD_UPD :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_HOST_TRIES2ACCESS_PROD_UPD) >> 3)
				, "\n\tPXP_REQ_LENGTH_TOO_BIG :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_PXP_REQ_LENGTH_TOO_BIG) >> 2)
				, "\n\tCTRL_FIFO_ERROR_ERR :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_CTRL_FIFO_ERROR_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & IGU_IGU_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x130098UL: // (IGU_REG_IGU_INT_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nIGU_REG_IGU_INT_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tATTN_PROD_ACC :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_ATTN_PROD_ACC) >> 10)
				, "\n\tSEGMENT_ACCESS_INVALID :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_SEGMENT_ACCESS_INVALID) >> 9)
				, "\n\tCMD_FID_NOT_MATCH :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_CMD_FID_NOT_MATCH) >> 8)
				, "\n\tDURIN_INT_READ_WITH_SIMD_DIS :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_DURIN_INT_READ_WITH_SIMD_DIS) >> 7)
				, "\n\tSB_INDEX_IS_NOT_VALID :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_SB_INDEX_IS_NOT_VALID) >> 6)
				, "\n\tMME_BIGGER_THEN_5 :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_MME_BIGGER_THEN_5) >> 5)
				, "\n\tVF_TRIES2ACC_ATTN_CMD :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_VF_TRIES2ACC_ATTN_CMD) >> 4)
				, "\n\tHOST_TRIES2ACCESS_PROD_UPD :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_HOST_TRIES2ACCESS_PROD_UPD) >> 3)
				, "\n\tPXP_REQ_LENGTH_TOO_BIG :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_PXP_REQ_LENGTH_TOO_BIG) >> 2)
				, "\n\tCTRL_FIFO_ERROR_ERR :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_CTRL_FIFO_ERROR_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & IGU_IGU_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x13009cUL: // (IGU_REG_IGU_PRTY_STS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nIGU_REG_IGU_PRTY_STS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTATISTIC_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_STATISTIC_PARITY) >> 10)
				, "\n\tATTN_ADDR_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_ATTN_ADDR_PARITY) >> 9)
				, "\n\tMSI_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_MSI_PARITY) >> 8)
				, "\n\tMSIX_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_MSIX_PARITY) >> 7)
				, "\n\tPBA_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_PBA_LOW_PARITY) >> 6)
				, "\n\tMASK_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_MASK_LOW_PARITY) >> 5)
				, "\n\tSB_BEFORE_INT_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_SB_BEFORE_INT_LOW_PARITY) >> 4)
				, "\n\tSB_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_SB_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_CAM_PARITY) >> 2)
				, "\n\tCTRL_FIFO_ERROR_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_CTRL_FIFO_ERROR_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x1300a0UL: // (IGU_REG_IGU_PRTY_STS_CLR: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nIGU_REG_IGU_PRTY_STS_CLR (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTATISTIC_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_STATISTIC_PARITY) >> 10)
				, "\n\tATTN_ADDR_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_ATTN_ADDR_PARITY) >> 9)
				, "\n\tMSI_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_MSI_PARITY) >> 8)
				, "\n\tMSIX_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_MSIX_PARITY) >> 7)
				, "\n\tPBA_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_PBA_LOW_PARITY) >> 6)
				, "\n\tMASK_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_MASK_LOW_PARITY) >> 5)
				, "\n\tSB_BEFORE_INT_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_SB_BEFORE_INT_LOW_PARITY) >> 4)
				, "\n\tSB_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_SB_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_CAM_PARITY) >> 2)
				, "\n\tCTRL_FIFO_ERROR_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_CTRL_FIFO_ERROR_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x1300a4UL: // (IGU_REG_IGU_PRTY_STS_WR: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nIGU_REG_IGU_PRTY_STS_WR (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTATISTIC_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_STATISTIC_PARITY) >> 10)
				, "\n\tATTN_ADDR_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_ATTN_ADDR_PARITY) >> 9)
				, "\n\tMSI_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_MSI_PARITY) >> 8)
				, "\n\tMSIX_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_MSIX_PARITY) >> 7)
				, "\n\tPBA_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_PBA_LOW_PARITY) >> 6)
				, "\n\tMASK_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_MASK_LOW_PARITY) >> 5)
				, "\n\tSB_BEFORE_INT_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_SB_BEFORE_INT_LOW_PARITY) >> 4)
				, "\n\tSB_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_SB_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_CAM_PARITY) >> 2)
				, "\n\tCTRL_FIFO_ERROR_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_CTRL_FIFO_ERROR_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x1300a8UL: // (IGU_REG_IGU_PRTY_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nIGU_REG_IGU_PRTY_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTATISTIC_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_STATISTIC_PARITY) >> 10)
				, "\n\tATTN_ADDR_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_ATTN_ADDR_PARITY) >> 9)
				, "\n\tMSI_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_MSI_PARITY) >> 8)
				, "\n\tMSIX_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_MSIX_PARITY) >> 7)
				, "\n\tPBA_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_PBA_LOW_PARITY) >> 6)
				, "\n\tMASK_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_MASK_LOW_PARITY) >> 5)
				, "\n\tSB_BEFORE_INT_LOW_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_SB_BEFORE_INT_LOW_PARITY) >> 4)
				, "\n\tSB_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_SB_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_CAM_PARITY) >> 2)
				, "\n\tCTRL_FIFO_ERROR_PARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_CTRL_FIFO_ERROR_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & IGU_IGU_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x130100UL: // (IGU_REG_INTERRUPT_STATUS: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nIGU_REG_INTERRUPT_STATUS (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug: Interrupt status (active high). PF0 to PF3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130104UL: // (IGU_REG_TRAILING_EDGE_LATCH: ACCESS:RW  DataWidth:0x10  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_TRAILING_EDGE_LATCH (ACCESS:RW  DataWidth:0x10  SPLIT:8 )" \
				"\nDescription: Attention signals trailing edge. attn bit condition monitoring\n    ; each bit that is set will lock a change from 1 to 0 in the corresponding\n     attention signals that comes from the AEU\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130108UL: // (IGU_REG_ATTENTION_ACK_BITS: ACCESS:RW  DataWidth:0x10  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_ATTENTION_ACK_BITS (ACCESS:RW  DataWidth:0x10  SPLIT:8 )" \
				"\nDescription: 16 bit register with the attention ACK values.These are the\n     same bits as in the attention message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13010cUL: // (IGU_REG_ATTENTION_BIT_STATUS_INDEX: ACCESS:RW  DataWidth:0x10  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_ATTENTION_BIT_STATUS_INDEX (ACCESS:RW  DataWidth:0x10  SPLIT:8 )" \
				"\nDescription: Value of attention bit status index (posted toward the driver\n     as attention bit status index). This are the same value as in the attention\n     message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130110UL: // (IGU_REG_ATTENTION_BITS: ACCESS:RW  DataWidth:0x10  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_ATTENTION_BITS (ACCESS:RW  DataWidth:0x10  SPLIT:8 )" \
				"\nDescription: 16 bit register with the latched attention values. These \n    are the same bits as in the attention message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130114UL: // (IGU_REG_ATTENTION_SIGNAL_P0_STATUS: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nIGU_REG_ATTENTION_SIGNAL_P0_STATUS (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: attention signal status. Reflects the current value of the\n     attention signals from the MISC-AEU port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130118UL: // (IGU_REG_ATTENTION_SIGNAL_P1_STATUS: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nIGU_REG_ATTENTION_SIGNAL_P1_STATUS (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: attention signal status. Reflects the current value of the\n     attention signals from the MISC-AEU port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13011cUL: // (IGU_REG_ATTN_MSG_ADDR_H: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_ATTN_MSG_ADDR_H (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: For attention message: Attention bit destination address \n    32 MSB\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130120UL: // (IGU_REG_ATTN_MSG_ADDR_L: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_ATTN_MSG_ADDR_L (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: For attention message: Attention bit destination address \n    32 LSB. Two Lsbit must be zero.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130124UL: // (IGU_REG_COMMAND_REG_32LSB_DATA: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_COMMAND_REG_32LSB_DATA (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: If block is disabled this register will return zero. If the\n     last command sent to the command_reg_ctrl the 32LSB read value will be\n     written here. If address is PBA: 32 LSB of PBA register (one in each \n    bit means PBA message wasnt sent due to mask). If address = SIMD with \n    mask 64b/32LSB: 32 LSB of the during interrupt register (one in each bit\n     means the appropriate SB is asserted. Every bit that is set will be masked\n     in the mask bit register). If address = SIMD with mask 32MSB: this register\n     will return zero. If address = SIMD without mask 64b: 32 LSB of the during\n     interrupt register (one in each bit means the appropriate SB is asserted\n    ). If the command sent to the command_reg_ctrl is write command the data\n     in this register will be used as follows: If address = interrupt acknowledge\n     register: d31 - d27- Reserved; d26:d25 - enable/disable interrupt for\n     status block (0=Disable interrupt for status block; 1=Enable interrupt\n     for status block; 2 or 3 - NOP); d24 - update/nop statu\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130128UL: // (IGU_REG_COMMAND_REG_32MSB_DATA: ACCESS:WB_R  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_COMMAND_REG_32MSB_DATA (ACCESS:WB_R  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: Read only register. If block is disable this register will\n     return zero. If the last command sent to the command_reg_ctrl the 32MSB\n     read value will be written here. If address is PBA: 32 MSB of PBA register\n     (one in each bit means PBA message wasnt sent due to mask). If address\n     = SIMD with mask 64b/32MSB: 32 MSB of the during interrupt register (\n    one in each bit means the appropriate SB is asserted. Every bit that is\n     set will be masked in the mask bit register). If address = SIMD with \n    mask 32LSB: this register will return zero. If address = SIMD without \n    mask 64b: 32 MSB of the during interrupt register (one in each bit means\n     the appropriate SB is asserted).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13012cUL: // (IGU_REG_COMMAND_REG_CTRL: ACCESS:W  DataWidth:0x15  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_COMMAND_REG_CTRL (ACCESS:W  DataWidth:0x15  SPLIT:8 )" \
				"\nDescription: [11:0] - PXP BAR address; [18:12] - FID (if VF - [18] = 0\n    ; [17:12] = VF number; if PF - [18] = 1; [17:15] = 0; [14:12] = PFID); [\n    19] - reserved; [20] command type - 0-read; 1-wr. When writing to this\n     register the command will be executed. On write command the 32 LSB command\n     should be written first (to the command_reg_32lsb_data register) and \n    only then this register. PXP BAR address field: 0x000 - 0x1FF - reserved\n    ; 0x200 - PBA ; 0x201 - 0x3FF  - reserved; 0x400 - 0x487 - interrupt ack\n     register; 0x488-0x4FF - reserved; 0x500 - 0x587 - producer update; 0x588\n    -0x59F - reserved; 0x5A0 - Attention bits update register; 0x5A1 - Attention\n     bits set register; 0x5A2 - Attention bits clear register; 0x5A3 -  SIMD\n     with mask 64b; 0x5A4 -  SIMD with mask 32 LSB; 0x5A5 -  SIMD with mask\n     32 MSB; 0x5A6 -  SIMD without mask 64b; 0x5A7 - 0x5FF - reserved. The\n     following addresses are write only: interrupt ack register; producer \n    update; Attention bits update register; Attention bits set register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130130UL: // (IGU_REG_ERROR_HANDLING_DATA_VALID: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nIGU_REG_ERROR_HANDLING_DATA_VALID (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: data availble for error memory. If this bit is clear do not\n     red from error_handling_memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130134UL: // (IGU_REG_LEADING_EDGE_LATCH: ACCESS:RW  DataWidth:0x10  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_LEADING_EDGE_LATCH (ACCESS:RW  DataWidth:0x10  SPLIT:8 )" \
				"\nDescription: Attention signals leading edge. attn bit condition monitoring\n    ; each bit that is set will lock a change from 0 to 1 in the corresponding\n     attention signals that comes from the AEU\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130138UL: // (IGU_REG_PBA_STATUS_LSB: ACCESS:RW  DataWidth:0x20  SPLIT:72 ) 
			sprintf(disp_str, "\nIGU_REG_PBA_STATUS_LSB (ACCESS:RW  DataWidth:0x20  SPLIT:72 )" \
				"\nDescription: 32 lsb of PBA register. 0 - PBA clear 1 - PBA set - the appropriate\n     MSIX message was not set due to mask bit (function or vector). The bits\n     order is according to the vector number of each SB in that function. \n    For PF bit 0 is the default SB. bits 31:16 are valid only for functions\n     that appears in func_with_more_16_sb_0..7 registers.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13013cUL: // (IGU_REG_PBA_STATUS_MSB: ACCESS:RW  DataWidth:0x20  SPLIT:72 ) 
			sprintf(disp_str, "\nIGU_REG_PBA_STATUS_MSB (ACCESS:RW  DataWidth:0x20  SPLIT:72 )" \
				"\nDescription: 32 msb of PBA register. 0 - PBA clear 1 - PBA set - the appropriate\n     MSIX message was not set due to mask bit (function or vector). The bits\n     order is according to the vector number of each SB in that function. \n    For PF bit 0 is the default SB.This register is valid only for functions\n     that appears in func_with_more_16_sb_0..7 registers\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130140UL: // (IGU_REG_PCI_PF_MSI_EN: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_PCI_PF_MSI_EN (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: PF MSI enable status. shadow of PCI config register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130144UL: // (IGU_REG_PCI_PF_MSIX_EN: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_PCI_PF_MSIX_EN (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: PF MSIX enable status. shadow of PCI config register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130148UL: // (IGU_REG_PCI_PF_MSIX_FUNC_MASK: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_PCI_PF_MSIX_FUNC_MASK (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: PF MSIX function mask status. shadow of PCI config register\n    . 0 - unmasked; 1 - masked\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13014cUL: // (IGU_REG_PCI_VF_MSIX_EN: ACCESS:RW  DataWidth:0x1  SPLIT:64 ) 
			sprintf(disp_str, "\nIGU_REG_PCI_VF_MSIX_EN (ACCESS:RW  DataWidth:0x1  SPLIT:64 )" \
				"\nDescription: VF MSIX enable status. shadow of PCI config register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130150UL: // (IGU_REG_PCI_VF_MSIX_FUNC_MASK: ACCESS:RW  DataWidth:0x1  SPLIT:64 ) 
			sprintf(disp_str, "\nIGU_REG_PCI_VF_MSIX_FUNC_MASK (ACCESS:RW  DataWidth:0x1  SPLIT:64 )" \
				"\nDescription: VF MSIX function mask status  shadow of PCI config register\n    . 0 - unmasked; 1 - masked\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130154UL: // (IGU_REG_PF_CONFIGURATION: ACCESS:RW  DataWidth:0x6  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_PF_CONFIGURATION (ACCESS:RW  DataWidth:0x6  SPLIT:8 )" \
				"\nDescription: d0 - function enable; d1 - MSI/MSIX enable; d2 - INT enable\n    ; d3 - attention enabe; d4 - single ISR mode enable; d5 simd all ones \n    mode - If clear (reset value):If the result of SB_before_mask & ~mask \n    is 0xFFFF_FFFF then the read result will be 0x7FFF_FFFF and the mask will\n     be also 0x7FFF_FFFF. Therefore the interrupt is not de-asserted (the \n    MSB SB is asserted and unmasked). And on the next read from SIMD with \n    mask the result will be 0x8000_0000 and only now the interrupt will be\n     de-asserted. If set: If the result of SB_before_mask & ~mask is 0xFFFF_FFFF\n     then the read result will be 0x7FFF_FFFF but the mask will be 0xFFFF_FFFF\n    . Therefore the interrupt is de-asserted. And on the next read from SIMD\n     with mask the result will be 0x0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130158UL: // (IGU_REG_RESET_MEMORIES: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nIGU_REG_RESET_MEMORIES (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Write one for each bit will reset the appropriate memory. \n    When the memory reset finished the appropriate bit will be clear. Bit \n    0 - mapping memory; Bit 1 - SB memory; Bit 2 - SB interrupt and mask register\n    ; Bit 3 - MSIX memory; Bit 4 - PBA memory; Bit 5 - statistics; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13015cUL: // (IGU_REG_SB_INT_BEFORE_MASK_LSB: ACCESS:RW  DataWidth:0x20  SPLIT:72 ) 
			sprintf(disp_str, "\nIGU_REG_SB_INT_BEFORE_MASK_LSB (ACCESS:RW  DataWidth:0x20  SPLIT:72 )" \
				"\nDescription: 32 lsb of SB interrupt before mask. 0 - prod equal cons. \n    1 - prod not equal cons or last command for this SB was prod update. The\n     bits order is according to the vector number of each SB in that function\n    . For PF bit 0 is the default SB. bits 31:16 are valid only for functions\n     that appears in func_with_more_16_sb_0..7 registers.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130160UL: // (IGU_REG_SB_INT_BEFORE_MASK_MSB: ACCESS:RW  DataWidth:0x20  SPLIT:72 ) 
			sprintf(disp_str, "\nIGU_REG_SB_INT_BEFORE_MASK_MSB (ACCESS:RW  DataWidth:0x20  SPLIT:72 )" \
				"\nDescription: 32 msb of SB interrupt before mask. 0 - prod equal cons. \n    1 - prod not equal cons or last command for this SB was prod update. The\n     bits order is according to the vector number of each SB in that function\n    . For PF bit 0 is the default SB. This register is valid only for functions\n     that appears in func_with_more_16_sb_0..7 registers\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130164UL: // (IGU_REG_SB_MASK_LSB: ACCESS:RW  DataWidth:0x20  SPLIT:72 ) 
			sprintf(disp_str, "\nIGU_REG_SB_MASK_LSB (ACCESS:RW  DataWidth:0x20  SPLIT:72 )" \
				"\nDescription: 32 lsb of SB mask register. 0 - unmased. 1 - masked. The \n    bits order is according to the vector number of each SB in that function\n    . For PF bit 0 is the default SB. bits 31:16 are valid only for functions\n     that appears in func_with_more_16_sb_0..7 registers.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130168UL: // (IGU_REG_SB_MASK_MSB: ACCESS:RW  DataWidth:0x20  SPLIT:72 ) 
			sprintf(disp_str, "\nIGU_REG_SB_MASK_MSB (ACCESS:RW  DataWidth:0x20  SPLIT:72 )" \
				"\nDescription: 32 msb of SB mask register.  0 - unmased. 1 - masked. The\n     bits order is according to the vector number of each SB in that function\n    . For PF bit 0 is the default SB. This register is valid only for functions\n     that appears in func_with_more_16_sb_0..7 registers\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x13016cUL: // (IGU_REG_SILENT_DROP: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nIGU_REG_SILENT_DROP (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Number of command that were dropped without causing an interrupt\n     due to: read access for WO BAR address; or write access for RO BAR address\n     or any access for reserved address or PCI function error is set and address\n     is not MSIX; PBA or cleanup\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130170UL: // (IGU_REG_VF_CONFIGURATION: ACCESS:RW  DataWidth:0x8  SPLIT:64 ) 
			sprintf(disp_str, "\nIGU_REG_VF_CONFIGURATION (ACCESS:RW  DataWidth:0x8  SPLIT:64 )" \
				"\nDescription: d0 - function enable; d1 - MSI/MSIX enable; d3:d2 reserved\n    ; d4 - single ISR mode enable; d7:d5 parent PF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130178UL: // (IGU_REG_ERROR_HANDLING_MEMORY: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nIGU_REG_ERROR_HANDLING_MEMORY (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: Do not read from this memory if error_handling_data_valid\n     register is zero. The data is collected in according to the command_debug\n     value. if command_debug is clear it holds the first 8 error commands \n    as follows: {wr_data[31:0]; cmd_addr[12:0]; error_type_d[3:0]; cmd_fid\n    [6:0]; cmd_source; error_source[1:0]; cmd_func_err; cmd_length[2:0]; cmd_wr_n_rd\n    } . if command_debug is set it holds the last 8 commands to the IGU (with\n     and without error) as follows: {wr_data[31:0]; cmd_addr[15:0]; 1b0; cmd_fid\n    [6:0]; cmd_source; error_source[1:0]; cmd_func_err; cmd_length[2:0]; cmd_wr_n_rd\n    }. cmd_source - the source field in PXP command. VFID is encoded: PF? {\n    4b1000; PFID} : {1b0; VFID}. error_source[1:0] - pxp - 0; grc - 1; attn\n     - 2. cmd_func_err - func_err field in PXP command. cmd_length[2:0] - \n    length in PXP command. cmd_wr_n_rd: 0 - read; 1 - write. error_type_d[\n    3:0] :1 - length >1; 2 - command to disable function; 3 - VF command to\n     attn bits. 4 - host tries to update prod. 5 - during \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130180UL: // (IGU_REG_MSI_MEMORY: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nIGU_REG_MSI_MEMORY (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: addres 0 - MSI address low (two Lsbit are zero). Address \n    1 - MSI address high. Address 2 - [15:0] - MSI data; [18:16] MME; [31:\n    19] Reserved\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130200UL: // (IGU_REG_CSTORM_TYPE_0_SB_CLEANUP: ACCESS:WB_R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nIGU_REG_CSTORM_TYPE_0_SB_CLEANUP (ACCESS:WB_R  DataWidth:0x20 )" \
				"\nDescription: Cleanup bit status per SB. 1 = cleanup is set. 0 = cleanup\n     bit is clear. The bits in this registers are set and clear via the producer\n     command. Data valid only in addresses 0-4. all the rest are zero.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130280UL: // (IGU_REG_CSTORM_TYPE_1_SB_CLEANUP: ACCESS:WB_R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nIGU_REG_CSTORM_TYPE_1_SB_CLEANUP (ACCESS:WB_R  DataWidth:0x20 )" \
				"\nDescription: Cleanup bit status per SB. 1 = cleanup is set. 0 = cleanup\n     bit is clear. The bits in this registers are set and clear via the producer\n     command. Data valid only in addresses 0-4. all the rest are zero.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130300UL: // (IGU_REG_PENDING_BITS_STATUS: ACCESS:WB_R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nIGU_REG_PENDING_BITS_STATUS (ACCESS:WB_R  DataWidth:0x20 )" \
				"\nDescription: Each bit represent the pending bits status for that SB. 0\n     = no pending; 1 = pending. Pendings means interrupt was asserted; and\n     write done was not received. Data valid only in addresses 0-4. all the\n     rest are zero.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130380UL: // (IGU_REG_USTORM_TYPE_0_SB_CLEANUP: ACCESS:WB_R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nIGU_REG_USTORM_TYPE_0_SB_CLEANUP (ACCESS:WB_R  DataWidth:0x20 )" \
				"\nDescription: Cleanup bit status per SB. 1 = cleanup is set. 0 = cleanup\n     bit is clear. The bits in this registers are set and clear via the producer\n     command. Data valid only in addresses 0-4. all the rest are zero.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130400UL: // (IGU_REG_USTORM_TYPE_1_SB_CLEANUP: ACCESS:WB_R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nIGU_REG_USTORM_TYPE_1_SB_CLEANUP (ACCESS:WB_R  DataWidth:0x20 )" \
				"\nDescription: Cleanup bit status per SB. 1 = cleanup is set. 0 = cleanup\n     bit is clear. The bits in this registers are set and clear via the producer\n     command. Data valid only in addresses 0-4. all the rest are zero.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130480UL: // (IGU_REG_WRITE_DONE_PENDING: ACCESS:WB_R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nIGU_REG_WRITE_DONE_PENDING (ACCESS:WB_R  DataWidth:0x20 )" \
				"\nDescription: Each bit represent write done pending bits status for that\n     SB (MSI/MSIX message was sent and write done was not received yet). 0\n     = clear; 1 = set.  Data valid only in addresses 0-4. all the rest are\n     zero.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130800UL: // (IGU_REG_STATISTIC_NUM_MESSAGE_SENT: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nIGU_REG_STATISTIC_NUM_MESSAGE_SENT (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Number of MSI/MSIX/ATTN messages sent for the function: 0\n    -63 - number of MSIX messages per VF; 64-67 - number of MSI/MSIX messages\n     per PF; 68-71 number of ATTN messages per PF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x131000UL: // (IGU_REG_MAPPING_MEMORY: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nIGU_REG_MAPPING_MEMORY (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: mapping CAM; relevant for E2 operating mode only.  [0] - \n    valid. [6:1] - vector number; [13:7] - FID (if VF - [13] = 0; [12:7] = \n    VF number; if PF - [13] = 1; [12:10] = 0; [9:7] = PF number);\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x132000UL: // (IGU_REG_PROD_CONS_MEMORY: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nIGU_REG_PROD_CONS_MEMORY (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: producers only. E2 mode: address 0-135 match to the mapping\n     memory; 136 - PF0 default prod; 137 PF1 default prod; 138 - PF2 default\n     prod; 139 PF3 default prod; 140 - PF0 - ATTN prod; 141 - PF1 - ATTN prod\n    ; 142 - PF2 - ATTN prod; 143 - PF3 - ATTN prod; 144-147 reserved. E1.5\n     mode - In backward compatible mode; for non default SB; each even line\n     in the memory holds the U producer and each odd line hold the C producer\n    . The first 128 producer are for NDSB (PF0 - 0-31; PF1 - 32-63 and so \n    on). The last 20 producers are for the DSB for each PF. each PF has five\n     segments (the order inside each segment is PF0; PF1; PF2; PF3) - 128-\n    131 U prods; 132-135 C prods; 136-139 X prods; 140-143 T prods; 144-147\n     ATTN prods; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x134000UL: // (IGU_REG_MSIX_MEMORY: ACCESS:WB  DataWidth:0x61 ) 
			sprintf(disp_str, "\nIGU_REG_MSIX_MEMORY (ACCESS:WB  DataWidth:0x61 )" \
				"\nDescription: [63:0] - MSIX message address (bit [1:0] are always zero); [\n    95:64] - MSIX message data; [96] - MSIX mask bit (0 - unmasked; 1 - masked\n    ).Reset value (after reset_memories was set) is MSIX address = 0; MSIX\n     data = 0; MSIX mask bit=1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1300acUL: // (IGU_REG_IGU_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nIGU_REG_IGU_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130174UL: // (IGU_REG_IGU_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nIGU_REG_IGU_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130190UL: // (IGU_REG_IGU_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nIGU_REG_IGU_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130500UL: // (IGU_REG_IGU_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nIGU_REG_IGU_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x130c00UL: // (IGU_REG_IGU_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nIGU_REG_IGU_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x133000UL: // (IGU_REG_IGU_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nIGU_REG_IGU_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa000UL: // (MISC_REG_AEU_GENERAL_ATTN_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 0; this will set/clr bit 94 in \n    the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa004UL: // (MISC_REG_AEU_GENERAL_ATTN_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention  1; this will set/clr bit 95 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa008UL: // (MISC_REG_AEU_GENERAL_ATTN_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention  2; this will set/clr bit 96 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa00cUL: // (MISC_REG_AEU_GENERAL_ATTN_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention  3; this will set/clr bit 97 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa010UL: // (MISC_REG_AEU_GENERAL_ATTN_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 4; this will set/clr bit 98 in \n    the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa014UL: // (MISC_REG_AEU_GENERAL_ATTN_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 5; this will set/clr bit 99 in \n    the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa018UL: // (MISC_REG_AEU_GENERAL_ATTN_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 6; this will set/clr bit 100 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa01cUL: // (MISC_REG_AEU_GENERAL_ATTN_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 7; this will set/clr bit 101 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa020UL: // (MISC_REG_AEU_GENERAL_ATTN_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 8; this will set/clr bit 102 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa024UL: // (MISC_REG_AEU_GENERAL_ATTN_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 9; this will set/clr bit 103 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa028UL: // (MISC_REG_AEU_GENERAL_ATTN_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 10; this will set/clr bit 104 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa02cUL: // (MISC_REG_AEU_GENERAL_ATTN_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 11; this will set/clr bit 105 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa030UL: // (MISC_REG_AEU_GENERAL_ATTN_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 12; this will set/clr bit 106 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa034UL: // (MISC_REG_AEU_GENERAL_ATTN_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 13; this will set/clr bit 107 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa038UL: // (MISC_REG_AEU_GENERAL_ATTN_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 14; this will set/clr bit 108 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa03cUL: // (MISC_REG_AEU_GENERAL_ATTN_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 15; this will set/clr bit 109 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa040UL: // (MISC_REG_AEU_GENERAL_ATTN_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 16; this will set/clr bit 110 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa044UL: // (MISC_REG_AEU_GENERAL_ATTN_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 17; this will set/clr bit 111 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa048UL: // (MISC_REG_AEU_GENERAL_ATTN_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 18; this will set/clr bit 112 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa04cUL: // (MISC_REG_AEU_GENERAL_ATTN_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 19; this will set/clr bit 113 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa050UL: // (MISC_REG_AEU_GENERAL_ATTN_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 20; this will set/clr bit 114 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa054UL: // (MISC_REG_AEU_GENERAL_ATTN_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_ATTN_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: set/clr general attention 21; this will set/clr bit 115 in\n     the aeu 128 bit vector\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa058UL: // (MISC_REG_AEU_EVENT_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_EVENT_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Event_enable control; when this bit is clear the event enable\n     toward the MCP is masked.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa060UL: // (MISC_REG_AEU_MASK_ATTN_FUNC_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_AEU_MASK_ATTN_FUNC_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: [7:0] = mask 8 attention output signals toward IGU function0\n    ; [9:8] = reserved. 0 = mask; 1 = unmask\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa064UL: // (MISC_REG_AEU_MASK_ATTN_FUNC_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_AEU_MASK_ATTN_FUNC_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: [7:0] = mask 8 attention output signals toward IGU function1\n    ; [9:8] = reserved. 0 = mask; 1 = unmask\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa068UL: // (MISC_REG_AEU_MASK_ATTN_MCP: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_MASK_ATTN_MCP (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Masks 8 attention output signals toward MCP. Zero = mask; \n    one = unmask\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa06cUL: // (MISC_REG_AEU_ENABLE1_FUNC_0_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_0_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 0 output0. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 function\n     0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa070UL: // (MISC_REG_AEU_ENABLE2_FUNC_0_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_0_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 0 output0\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa074UL: // (MISC_REG_AEU_ENABLE3_FUNC_0_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_0_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 0 output0. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa078UL: // (MISC_REG_AEU_ENABLE4_FUNC_0_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_0_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 0 output0\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa07cUL: // (MISC_REG_AEU_ENABLE1_FUNC_0_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_0_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 0 output1. \n    Mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 function\n     0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa080UL: // (MISC_REG_AEU_ENABLE2_FUNC_0_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_0_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 0 output1\n    . Mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa084UL: // (MISC_REG_AEU_ENABLE3_FUNC_0_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_0_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 0 output1. \n    Mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa088UL: // (MISC_REG_AEU_ENABLE4_FUNC_0_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_0_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 0 output1\n    .Mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa08cUL: // (MISC_REG_AEU_ENABLE1_FUNC_0_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_0_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 0 output2. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 function\n     0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa090UL: // (MISC_REG_AEU_ENABLE2_FUNC_0_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_0_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 0 output2\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa094UL: // (MISC_REG_AEU_ENABLE3_FUNC_0_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_0_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 0 output2. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa098UL: // (MISC_REG_AEU_ENABLE4_FUNC_0_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_0_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 0 output2\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa09cUL: // (MISC_REG_AEU_ENABLE1_FUNC_0_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_0_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 0 output3. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 function\n     0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0a0UL: // (MISC_REG_AEU_ENABLE2_FUNC_0_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_0_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 0 output3\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0a4UL: // (MISC_REG_AEU_ENABLE3_FUNC_0_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_0_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 0 output3. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0a8UL: // (MISC_REG_AEU_ENABLE4_FUNC_0_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_0_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 0 output3\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0acUL: // (MISC_REG_AEU_ENABLE1_FUNC_0_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_0_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 0 output4. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 function\n     0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0b0UL: // (MISC_REG_AEU_ENABLE2_FUNC_0_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_0_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 0 output4\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0b4UL: // (MISC_REG_AEU_ENABLE3_FUNC_0_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_0_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 0 output4. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0b8UL: // (MISC_REG_AEU_ENABLE4_FUNC_0_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_0_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 0 output4\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0bcUL: // (MISC_REG_AEU_ENABLE1_FUNC_0_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_0_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 0 output5. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 function\n     0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0c0UL: // (MISC_REG_AEU_ENABLE2_FUNC_0_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_0_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 0 output5\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0c4UL: // (MISC_REG_AEU_ENABLE3_FUNC_0_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_0_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 0 output5. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0c8UL: // (MISC_REG_AEU_ENABLE4_FUNC_0_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_0_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 0 output5\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0ccUL: // (MISC_REG_AEU_ENABLE1_FUNC_0_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_0_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 0 output6. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 function\n     0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0d0UL: // (MISC_REG_AEU_ENABLE2_FUNC_0_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_0_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 0 output6\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0d4UL: // (MISC_REG_AEU_ENABLE3_FUNC_0_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_0_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 0 output6. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0d8UL: // (MISC_REG_AEU_ENABLE4_FUNC_0_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_0_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 0 output6\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0dcUL: // (MISC_REG_AEU_ENABLE1_FUNC_0_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_0_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 0 output7. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 function\n     0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0e0UL: // (MISC_REG_AEU_ENABLE2_FUNC_0_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_0_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 0 output7\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0e4UL: // (MISC_REG_AEU_ENABLE3_FUNC_0_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_0_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 0 output7. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0e8UL: // (MISC_REG_AEU_ENABLE4_FUNC_0_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_0_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 0 output7\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0ecUL: // (MISC_REG_AEU_ENABLE1_NIG_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_NIG_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for close the gate nig. \n    Mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 function\n     0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0f0UL: // (MISC_REG_AEU_ENABLE2_NIG_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_NIG_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for close the gate nig\n    . Mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0f4UL: // (MISC_REG_AEU_ENABLE3_NIG_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_NIG_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for close the gate nig. \n    Mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0f8UL: // (MISC_REG_AEU_ENABLE4_NIG_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_NIG_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for close the gate nig\n    . Mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0fcUL: // (MISC_REG_AEU_ENABLE1_PXP_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_PXP_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for close the gate pxp. \n    Mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 function\n     0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa100UL: // (MISC_REG_AEU_ENABLE2_PXP_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_PXP_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for close the gate pxp\n    . Mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa104UL: // (MISC_REG_AEU_ENABLE3_PXP_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_PXP_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for close the gate pxp. \n    Mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa108UL: // (MISC_REG_AEU_ENABLE4_PXP_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_PXP_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for close the gate pxp\n    . Mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa10cUL: // (MISC_REG_AEU_ENABLE1_FUNC_1_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_1_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 1 output0. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 1; [3] GPIO2 function 1; [4] GPIO3 function\n     1; [5] GPIO4 function 1; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 1; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa110UL: // (MISC_REG_AEU_ENABLE2_FUNC_1_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_1_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 1 output0\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa114UL: // (MISC_REG_AEU_ENABLE3_FUNC_1_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_1_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 1 output0. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa118UL: // (MISC_REG_AEU_ENABLE4_FUNC_1_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_1_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 1 output0\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa11cUL: // (MISC_REG_AEU_ENABLE1_FUNC_1_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_1_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 1 output1. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 1; [3] GPIO2 function 1; [4] GPIO3 function\n     1; [5] GPIO4 function 1; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 1; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa120UL: // (MISC_REG_AEU_ENABLE2_FUNC_1_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_1_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 1 output1\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa124UL: // (MISC_REG_AEU_ENABLE3_FUNC_1_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_1_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 1 output1. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa128UL: // (MISC_REG_AEU_ENABLE4_FUNC_1_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_1_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 1 output1\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa12cUL: // (MISC_REG_AEU_ENABLE1_FUNC_1_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_1_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 1 output2. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 1; [3] GPIO2 function 1; [4] GPIO3 function\n     1; [5] GPIO4 function 1; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 1; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa130UL: // (MISC_REG_AEU_ENABLE2_FUNC_1_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_1_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 1 output2\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa134UL: // (MISC_REG_AEU_ENABLE3_FUNC_1_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_1_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 1 output2. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa138UL: // (MISC_REG_AEU_ENABLE4_FUNC_1_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_1_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 1 output2\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa13cUL: // (MISC_REG_AEU_ENABLE1_FUNC_1_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_1_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 1 output3. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 1; [3] GPIO2 function 1; [4] GPIO3 function\n     1; [5] GPIO4 function 1; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 1; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa140UL: // (MISC_REG_AEU_ENABLE2_FUNC_1_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_1_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 1 output3\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa144UL: // (MISC_REG_AEU_ENABLE3_FUNC_1_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_1_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 1 output3. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa148UL: // (MISC_REG_AEU_ENABLE4_FUNC_1_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_1_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 1 output3\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa14cUL: // (MISC_REG_AEU_ENABLE1_FUNC_1_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_1_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 1 output4. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 1; [3] GPIO2 function 1; [4] GPIO3 function\n     1; [5] GPIO4 function 1; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 1; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa150UL: // (MISC_REG_AEU_ENABLE2_FUNC_1_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_1_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 1 output4\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa154UL: // (MISC_REG_AEU_ENABLE3_FUNC_1_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_1_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 1 output4. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa158UL: // (MISC_REG_AEU_ENABLE4_FUNC_1_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_1_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 1 output4\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa15cUL: // (MISC_REG_AEU_ENABLE1_FUNC_1_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_1_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 1 output5. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 1; [3] GPIO2 function 1; [4] GPIO3 function\n     1; [5] GPIO4 function 1; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 1; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa160UL: // (MISC_REG_AEU_ENABLE2_FUNC_1_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_1_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 1 output5\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa164UL: // (MISC_REG_AEU_ENABLE3_FUNC_1_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_1_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 1 output5. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa168UL: // (MISC_REG_AEU_ENABLE4_FUNC_1_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_1_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 1 output5\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa16cUL: // (MISC_REG_AEU_ENABLE1_FUNC_1_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_1_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 1 output6. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 1; [3] GPIO2 function 1; [4] GPIO3 function\n     1; [5] GPIO4 function 1; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 1; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa170UL: // (MISC_REG_AEU_ENABLE2_FUNC_1_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_1_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 1 output6\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa174UL: // (MISC_REG_AEU_ENABLE3_FUNC_1_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_1_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 1 output6. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa178UL: // (MISC_REG_AEU_ENABLE4_FUNC_1_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_1_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 1 output6\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa17cUL: // (MISC_REG_AEU_ENABLE1_FUNC_1_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_FUNC_1_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for function 1 output7. \n    mapped as follows: [0] NIG attention for function0; [1] NIG attention \n    for function1; [2] GPIO1 function 1; [3] GPIO2 function 1; [4] GPIO3 function\n     1; [5] GPIO4 function 1; [6] GPIO1 function 1; [7] GPIO2 function 1; [\n    8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD event\n     function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue/PXP\n     Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14] SPIO4\n    ; [15] SPIO5; [16] MSI/X indication for function 1; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa180UL: // (MISC_REG_AEU_ENABLE2_FUNC_1_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_FUNC_1_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for function 1 output7\n    . mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa184UL: // (MISC_REG_AEU_ENABLE3_FUNC_1_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_FUNC_1_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for function 1 output7. \n    mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] \n    PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error\n    ; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw\n     interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity\n     error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (\n    HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt; [16] \n    pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0; [\n    20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa188UL: // (MISC_REG_AEU_ENABLE4_FUNC_1_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_FUNC_1_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for function 0 output7\n    .mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa18cUL: // (MISC_REG_AEU_ENABLE1_NIG_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_NIG_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: unused \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa190UL: // (MISC_REG_AEU_ENABLE2_NIG_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_NIG_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: unused \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa194UL: // (MISC_REG_AEU_ENABLE3_NIG_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_NIG_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: unused \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa198UL: // (MISC_REG_AEU_ENABLE4_NIG_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_NIG_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: unused \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa19cUL: // (MISC_REG_AEU_ENABLE1_PXP_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_PXP_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for close the gate system\n     kill. mapped as follows: [0] NIG attention for function0; [1] NIG attention\n     for function1; [2] GPIO1 function 0; [3] GPIO2 function 0; [4] GPIO3 \n    function 0; [5] GPIO4 function 0; [6] GPIO1 function 1; [7] GPIO2 function\n     1; [8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD\n     event function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue\n    /PXP Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14\n    ] SPIO4; [15] SPIO5; [16] MSI/X indication for function 0; [17] MSI/X \n    indication for function 1; [18] BRB Parity error; [19] BRB Hw interrupt\n    ; [20] PRS Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [\n    23] SRC Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [\n    26] TCM Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [\n    29] TSEMI Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1a0UL: // (MISC_REG_AEU_ENABLE2_PXP_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_PXP_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for close the gate system\n     kill. mapped as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt\n    ; [2] QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [\n    5] Timers Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [\n    8] XCM Parity error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11\n    ] XSEMI Hw interrupt; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt\n    ; [14] NIG Parity error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity\n     error; [17] Vaux PCI core Hw interrupt; [18] Debug Parity error; [19] \n    Debug Hw interrupt; [20] USDM Parity error; [21] USDM Hw interrupt; [22\n    ] UCM Parity error; [23] UCM Hw interrupt; [24] USEMI Parity error; [25\n    ] USEMI Hw interrupt; [26] UPB Parity error; [27] UPB Hw interrupt; [28\n    ] CSDM Parity error; [29] CSDM Hw interrupt; [30] CCM Parity error; [31\n    ] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1a4UL: // (MISC_REG_AEU_ENABLE3_PXP_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_PXP_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for close the gate system\n     kill. mapped as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt\n    ; [2] PXP Parity error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity\n     error; [5] PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] \n    CFC Hw interrupt; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE\n     Parity error; [11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13\n    ] IGU (HC) Hw interrupt; [14] MISC Parity error; [15] MISC Hw interrupt\n    ; [16] pxp_misc_mps_attn; [17] Flash event; [18] SMB event; [19] MCP attn0\n    ; [20] MCP attn1; [21] SW timers attn_1 func0; [22] SW timers attn_2 func0\n    ; [23] SW timers attn_3 func0; [24] SW timers attn_4 func0; [25] PERST\n    ; [26] SW timers attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers\n     attn_3 func1; [29] SW timers attn_4 func1; [30] General attn0; [31] General\n     attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1a8UL: // (MISC_REG_AEU_ENABLE4_PXP_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_PXP_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for close the gate system\n     kill.mapped as follows:  [0] General attn2; [1] General attn3; [2] General\n     attn4; [3] General attn5; [4] General attn6; [5] General attn7; [6] General\n     attn8; [7] General attn9; [8] General attn10; [9] General attn11; [10\n    ] General attn12; [11] General attn13; [12] General attn14; [13] General\n     attn15; [14] General attn16; [15] General attn17; [16] General attn18\n    ; [17] General attn19; [18] General attn20; [19] General attn21; [20] \n    Main power interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [\n    23] RBCN Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [\n    26] GRC Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1acUL: // (MISC_REG_AEU_ENABLE1_MCP_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_MCP_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for mcp output0. mapped\n     as follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1b0UL: // (MISC_REG_AEU_ENABLE2_MCP_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_MCP_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for mcp output0. mapped\n     as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] \n    QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1b4UL: // (MISC_REG_AEU_ENABLE3_MCP_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_MCP_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for mcp output0. mapped\n     as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1b8UL: // (MISC_REG_AEU_ENABLE4_MCP_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_MCP_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for mcp output0.mapped\n     as follows:  [0] General attn2; [1] General attn3; [2] General attn4; [\n    3] General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1bcUL: // (MISC_REG_AEU_ENABLE1_MCP_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_MCP_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for mcp output1. mapped\n     as follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1c0UL: // (MISC_REG_AEU_ENABLE2_MCP_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_MCP_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for mcp output1. mapped\n     as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] \n    QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1c4UL: // (MISC_REG_AEU_ENABLE3_MCP_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_MCP_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for mcp output1. mapped\n     as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1c8UL: // (MISC_REG_AEU_ENABLE4_MCP_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_MCP_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for mcp output1.mapped\n     as follows:  [0] General attn2; [1] General attn3; [2] General attn4; [\n    3] General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1ccUL: // (MISC_REG_AEU_ENABLE1_MCP_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_MCP_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for mcp output2. mapped\n     as follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1d0UL: // (MISC_REG_AEU_ENABLE2_MCP_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_MCP_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for mcp output2. mapped\n     as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] \n    QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1d4UL: // (MISC_REG_AEU_ENABLE3_MCP_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_MCP_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for mcp output2. mapped\n     as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1d8UL: // (MISC_REG_AEU_ENABLE4_MCP_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_MCP_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for mcp output2.mapped\n     as follows:  [0] General attn2; [1] General attn3; [2] General attn4; [\n    3] General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1dcUL: // (MISC_REG_AEU_ENABLE1_MCP_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_MCP_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for mcp output3. mapped\n     as follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1e0UL: // (MISC_REG_AEU_ENABLE2_MCP_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_MCP_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for mcp output3. mapped\n     as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] \n    QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1e4UL: // (MISC_REG_AEU_ENABLE3_MCP_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_MCP_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for mcp output3. mapped\n     as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1e8UL: // (MISC_REG_AEU_ENABLE4_MCP_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_MCP_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for mcp output3.mapped\n     as follows:  [0] General attn2; [1] General attn3; [2] General attn4; [\n    3] General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1ecUL: // (MISC_REG_AEU_ENABLE1_MCP_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_MCP_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for mcp output4. mapped\n     as follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1f0UL: // (MISC_REG_AEU_ENABLE2_MCP_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_MCP_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for mcp output4. mapped\n     as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] \n    QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1f4UL: // (MISC_REG_AEU_ENABLE3_MCP_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_MCP_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for mcp output4. mapped\n     as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1f8UL: // (MISC_REG_AEU_ENABLE4_MCP_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_MCP_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for mcp output4.mapped\n     as follows:  [0] General attn2; [1] General attn3; [2] General attn4; [\n    3] General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa1fcUL: // (MISC_REG_AEU_ENABLE1_MCP_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_MCP_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for mcp output5. mapped\n     as follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa200UL: // (MISC_REG_AEU_ENABLE2_MCP_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_MCP_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for mcp output5. mapped\n     as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] \n    QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa204UL: // (MISC_REG_AEU_ENABLE3_MCP_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_MCP_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for mcp output5. mapped\n     as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa208UL: // (MISC_REG_AEU_ENABLE4_MCP_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_MCP_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for mcp output5.mapped\n     as follows:  [0] General attn2; [1] General attn3; [2] General attn4; [\n    3] General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa20cUL: // (MISC_REG_AEU_ENABLE1_MCP_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_MCP_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for mcp output6. mapped\n     as follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa210UL: // (MISC_REG_AEU_ENABLE2_MCP_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_MCP_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for mcp output6. mapped\n     as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] \n    QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa214UL: // (MISC_REG_AEU_ENABLE3_MCP_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_MCP_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for mcp output6. mapped\n     as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa218UL: // (MISC_REG_AEU_ENABLE4_MCP_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_MCP_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for mcp output6.mapped\n     as follows:  [0] General attn2; [1] General attn3; [2] General attn4; [\n    3] General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa21cUL: // (MISC_REG_AEU_ENABLE1_MCP_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE1_MCP_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for enabling the output for mcp output7. mapped\n     as follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa220UL: // (MISC_REG_AEU_ENABLE2_MCP_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE2_MCP_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for enabling the output for mcp output7. mapped\n     as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] \n    QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa224UL: // (MISC_REG_AEU_ENABLE3_MCP_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE3_MCP_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for enabling the output for mcp output7. mapped\n     as follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa228UL: // (MISC_REG_AEU_ENABLE4_MCP_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE4_MCP_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for enabling the output for mcp output7.mapped\n     as follows:  [0] General attn2; [1] General attn3; [2] General attn4; [\n    3] General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa22cUL: // (MISC_REG_AEU_INVERTER_1_FUNC_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_1_FUNC_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for inverting the input for function 0; for each\n     bit: 0= do not invert; 1= invert; mapped as follows: [0] NIG attention\n     for function0; [1] NIG attention for function1; [2] GPIO1 mcp; [3] GPIO2\n     mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1 function 1; [7] GPIO2 function\n     1; [8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD\n     event function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue\n    /PXP Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14\n    ] SPIO4; [15] SPIO5; [16] MSI/X indication for mcp; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa230UL: // (MISC_REG_AEU_INVERTER_2_FUNC_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_2_FUNC_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for inverting the input for function 0; for each\n     bit: 0= do not invert; 1= invert. mapped as follows: [0] PBClient Parity\n     error; [1] PBClient Hw interrupt; [2] QM Parity error; [3] QM Hw interrupt\n    ; [4] Timers Parity error; [5] Timers Hw interrupt; [6] XSDM Parity error\n    ; [7] XSDM Hw interrupt; [8] XCM Parity error; [9] XCM Hw interrupt; [\n    10] XSEMI Parity error; [11] XSEMI Hw interrupt; [12] DoorbellQ Parity\n     error; [13] DoorbellQ Hw interrupt; [14] NIG Parity error; [15] NIG Hw\n     interrupt; [16] Vaux PCI core Parity error; [17] Vaux PCI core Hw interrupt\n    ; [18] Debug Parity error; [19] Debug Hw interrupt; [20] USDM Parity error\n    ; [21] USDM Hw interrupt; [22] UCM Parity error; [23] UCM Hw interrupt\n    ; [24] USEMI Parity error; [25] USEMI Hw interrupt; [26] UPB Parity error\n    ; [27] UPB Hw interrupt; [28] CSDM Parity error; [29] CSDM Hw interrupt\n    ; [30] CCM Parity error; [31] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa234UL: // (MISC_REG_AEU_INVERTER_3_FUNC_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_3_FUNC_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for inverting the input for function 0; for each\n     bit: 0= do not invert; 1= invert. mapped as follows: [0] CSEMI Parity\n     error; [1] CSEMI Hw interrupt; [2] PXP Parity error; [3] PXP Hw interrupt\n    ; [4] PXPpciClockClient Parity error; [5] PXPpciClockClient Hw interrupt\n    ; [6] CFC Parity error; [7] CFC Hw interrupt; [8] CDU Parity error; [9\n    ] CDU Hw interrupt; [10] DMAE Parity error; [11] DMAE Hw interrupt; [12\n    ] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt; [14] MISC Parity \n    error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn; [17] Flash event\n    ; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21] SW timers attn_1\n     func0; [22] SW timers attn_2 func0; [23] SW timers attn_3 func0; [24] \n    SW timers attn_4 func0; [25] PERST; [26] SW timers attn_1 func1; [27] \n    SW timers attn_2 func1; [28] SW timers attn_3 func1; [29] SW timers attn_4\n     func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa238UL: // (MISC_REG_AEU_INVERTER_4_FUNC_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_4_FUNC_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for inverting the input for function 0; for each\n     bit: 0= do not invert; 1= invert.mapped as follows:  [0] General attn2\n    ; [1] General attn3; [2] General attn4; [3] General attn5; [4] General\n     attn6; [5] General attn7; [6] General attn8; [7] General attn9; [8] General\n     attn10; [9] General attn11; [10] General attn12; [11] General attn13; [\n    12] General attn14; [13] General attn15; [14] General attn16; [15] General\n     attn17; [16] General attn18; [17] General attn19; [18] General attn20\n    ; [19] General attn21; [20] Main power interrupt; [21] RBCR Latched attn\n    ; [22] RBCT Latched attn; [23] RBCN Latched attn; [24] RBCU Latched attn\n    ; [25] RBCP Latched attn; [26] GRC Latched timeout attention; [27] GRC\n     Latched reserved access attention; [28] MCP Latched rom_parity; [29] \n    MCP Latched ump_rx_parity; [30] MCP Latched ump_tx_parity; [31] MCP Latched\n     scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa23cUL: // (MISC_REG_AEU_INVERTER_1_FUNC_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_1_FUNC_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for inverting the input for function 1; for each\n     bit: 0= do not invert; 1= invert. mapped as follows: [0] NIG attention\n     for function0; [1] NIG attention for function1; [2] GPIO1 mcp; [3] GPIO2\n     mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1 function 1; [7] GPIO2 function\n     1; [8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD\n     event function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue\n    /PXP Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14\n    ] SPIO4; [15] SPIO5; [16] MSI/X indication for mcp; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa240UL: // (MISC_REG_AEU_INVERTER_2_FUNC_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_2_FUNC_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for inverting the input for function 1; for each\n     bit: 0= do not invert; 1= invert. mapped as follows: [0] PBClient Parity\n     error; [1] PBClient Hw interrupt; [2] QM Parity error; [3] QM Hw interrupt\n    ; [4] Timers Parity error; [5] Timers Hw interrupt; [6] XSDM Parity error\n    ; [7] XSDM Hw interrupt; [8] XCM Parity error; [9] XCM Hw interrupt; [\n    10] XSEMI Parity error; [11] XSEMI Hw interrupt; [12] DoorbellQ Parity\n     error; [13] DoorbellQ Hw interrupt; [14] NIG Parity error; [15] NIG Hw\n     interrupt; [16] Vaux PCI core Parity error; [17] Vaux PCI core Hw interrupt\n    ; [18] Debug Parity error; [19] Debug Hw interrupt; [20] USDM Parity error\n    ; [21] USDM Hw interrupt; [22] UCM Parity error; [23] UCM Hw interrupt\n    ; [24] USEMI Parity error; [25] USEMI Hw interrupt; [26] UPB Parity error\n    ; [27] UPB Hw interrupt; [28] CSDM Parity error; [29] CSDM Hw interrupt\n    ; [30] CCM Parity error; [31] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa244UL: // (MISC_REG_AEU_INVERTER_3_FUNC_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_3_FUNC_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for inverting the input for function 1; for each\n     bit: 0= do not invert; 1= invert. mapped as follows: [0] CSEMI Parity\n     error; [1] CSEMI Hw interrupt; [2] PXP Parity error; [3] PXP Hw interrupt\n    ; [4] PXPpciClockClient Parity error; [5] PXPpciClockClient Hw interrupt\n    ; [6] CFC Parity error; [7] CFC Hw interrupt; [8] CDU Parity error; [9\n    ] CDU Hw interrupt; [10] DMAE Parity error; [11] DMAE Hw interrupt; [12\n    ] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt; [14] MISC Parity \n    error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn; [17] Flash event\n    ; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21] SW timers attn_1\n     func0; [22] SW timers attn_2 func0; [23] SW timers attn_3 func0; [24] \n    SW timers attn_4 func0; [25] PERST; [26] SW timers attn_1 func1; [27] \n    SW timers attn_2 func1; [28] SW timers attn_3 func1; [29] SW timers attn_4\n     func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa248UL: // (MISC_REG_AEU_INVERTER_4_FUNC_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_4_FUNC_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for inverting the input for function 1; for each\n     bit: 0= do not invert; 1= invert.mapped as follows:  [0] General attn2\n    ; [1] General attn3; [2] General attn4; [3] General attn5; [4] General\n     attn6; [5] General attn7; [6] General attn8; [7] General attn9; [8] General\n     attn10; [9] General attn11; [10] General attn12; [11] General attn13; [\n    12] General attn14; [13] General attn15; [14] General attn16; [15] General\n     attn17; [16] General attn18; [17] General attn19; [18] General attn20\n    ; [19] General attn21; [20] Main power interrupt; [21] RBCR Latched attn\n    ; [22] RBCT Latched attn; [23] RBCN Latched attn; [24] RBCU Latched attn\n    ; [25] RBCP Latched attn; [26] GRC Latched timeout attention; [27] GRC\n     Latched reserved access attention; [28] MCP Latched rom_parity; [29] \n    MCP Latched ump_rx_parity; [30] MCP Latched ump_tx_parity; [31] MCP Latched\n     scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa24cUL: // (MISC_REG_AEU_INVERTER_1_MCP: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_1_MCP (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first 32b for inverting the input for mcp; for each bit: \n    0= do not invert; 1= invert. mapped as follows: [0] NIG attention for \n    function0; [1] NIG attention for function1; [2] GPIO1 mcp; [3] GPIO2 mcp\n    ; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1 function 1; [7] GPIO2 function\n     1; [8] GPIO3 function 1; [9] GPIO4 function 1; [10] PCIE glue/PXP VPD\n     event function0; [11] PCIE glue/PXP VPD event function1; [12] PCIE glue\n    /PXP Expansion ROM event0; [13] PCIE glue/PXP Expansion ROM event1; [14\n    ] SPIO4; [15] SPIO5; [16] MSI/X indication for mcp; [17] MSI/X indication\n     for function 1; [18] BRB Parity error; [19] BRB Hw interrupt; [20] PRS\n     Parity error; [21] PRS Hw interrupt; [22] SRC Parity error; [23] SRC \n    Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt; [26] TCM\n     Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error; [29] TSEMI\n     Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa250UL: // (MISC_REG_AEU_INVERTER_2_MCP: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_2_MCP (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second 32b for inverting the input for mcp; for each bit: \n    0= do not invert; 1= invert. mapped as follows: [0] PBClient Parity error\n    ; [1] PBClient Hw interrupt; [2] QM Parity error; [3] QM Hw interrupt; [\n    4] Timers Parity error; [5] Timers Hw interrupt; [6] XSDM Parity error\n    ; [7] XSDM Hw interrupt; [8] XCM Parity error; [9] XCM Hw interrupt; [\n    10] XSEMI Parity error; [11] XSEMI Hw interrupt; [12] DoorbellQ Parity\n     error; [13] DoorbellQ Hw interrupt; [14] NIG Parity error; [15] NIG Hw\n     interrupt; [16] Vaux PCI core Parity error; [17] Vaux PCI core Hw interrupt\n    ; [18] Debug Parity error; [19] Debug Hw interrupt; [20] USDM Parity error\n    ; [21] USDM Hw interrupt; [22] UCM Parity error; [23] UCM Hw interrupt\n    ; [24] USEMI Parity error; [25] USEMI Hw interrupt; [26] UPB Parity error\n    ; [27] UPB Hw interrupt; [28] CSDM Parity error; [29] CSDM Hw interrupt\n    ; [30] CCM Parity error; [31] CCM Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa254UL: // (MISC_REG_AEU_INVERTER_3_MCP: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_3_MCP (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third 32b for inverting the input for mcp; for each bit: \n    0= do not invert; 1= invert. mapped as follows: [0] CSEMI Parity error\n    ; [1] CSEMI Hw interrupt; [2] PXP Parity error; [3] PXP Hw interrupt; [\n    4] PXPpciClockClient Parity error; [5] PXPpciClockClient Hw interrupt; [\n    6] CFC Parity error; [7] CFC Hw interrupt; [8] CDU Parity error; [9] CDU\n     Hw interrupt; [10] DMAE Parity error; [11] DMAE Hw interrupt; [12] IGU\n     (HC) Parity error; [13] IGU (HC) Hw interrupt; [14] MISC Parity error\n    ; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn; [17] Flash event; [18\n    ] SMB event; [19] MCP attn0; [20] MCP attn1; [21] SW timers attn_1 func0\n    ; [22] SW timers attn_2 func0; [23] SW timers attn_3 func0; [24] SW timers\n     attn_4 func0; [25] PERST; [26] SW timers attn_1 func1; [27] SW timers\n     attn_2 func1; [28] SW timers attn_3 func1; [29] SW timers attn_4 func1\n    ; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa258UL: // (MISC_REG_AEU_INVERTER_4_MCP: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_4_MCP (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fourth 32b for inverting the input for mcp; for each bit: \n    0= do not invert; 1= invert. mapped as follows:  [0] General attn2; [1\n    ] General attn3; [2] General attn4; [3] General attn5; [4] General attn6\n    ; [5] General attn7; [6] General attn8; [7] General attn9; [8] General\n     attn10; [9] General attn11; [10] General attn12; [11] General attn13; [\n    12] General attn14; [13] General attn15; [14] General attn16; [15] General\n     attn17; [16] General attn18; [17] General attn19; [18] General attn20\n    ; [19] General attn21; [20] Main power interrupt; [21] RBCR Latched attn\n    ; [22] RBCT Latched attn; [23] RBCN Latched attn; [24] RBCU Latched attn\n    ; [25] RBCP Latched attn; [26] GRC Latched timeout attention; [27] GRC\n     Latched reserved access attention; [28] MCP Latched rom_parity; [29] \n    MCP Latched ump_rx_parity; [30] MCP Latched ump_tx_parity; [31] MCP Latched\n     scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa25cUL: // (MISC_REG_WAIT_P: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WAIT_P (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: when set to one stops all storms\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa278UL: // (MISC_REG_EMAC0: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nMISC_REG_EMAC0 (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEMAC0_PARITY_MODE :"
				, (u32_t)((data & MISC_EMAC0_REG_EMAC0_PARITY_MODE) >> 1)
				, "\n\tEMAC0_SHUTDOWN :"
				, (u32_t)((data & MISC_EMAC0_REG_EMAC0_SHUTDOWN) >> 0));
			break;
		case 0xa27cUL: // (MISC_REG_EMAC1: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nMISC_REG_EMAC1 (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEMAC1_PARITY_MODE :"
				, (u32_t)((data & MISC_EMAC1_REG_EMAC1_PARITY_MODE) >> 1)
				, "\n\tEMAC1_SHUTDOWN :"
				, (u32_t)((data & MISC_EMAC1_REG_EMAC1_SHUTDOWN) >> 0));
			break;
		case 0xa280UL: // (MISC_REG_GRC_TIMEOUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_GRC_TIMEOUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Setting this bit enables a timer in the GRC block to timeout\n     any access that does not finish within ~misc_registers_grc_timout_val\n    .grc_timeout_val cycles. When this bit is cleared; this timeout is disabled\n    . If this timeout occurs; the GRC shall assert it attention output.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa284UL: // (MISC_REG_GRC_TIMEOUT_VAL: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_GRC_TIMEOUT_VAL (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: The count value for the timer in the GRC block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa288UL: // (MISC_REG_GRC_DBG_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_GRC_DBG_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only: enable the debug mux \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa294UL: // (MISC_REG_PLL_STORM_CTRL_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_CTRL_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa298UL: // (MISC_REG_PLL_STORM_CTRL_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_CTRL_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa29cUL: // (MISC_REG_PLL_STORM_CTRL_3: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_CTRL_3 (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPLL_STORM_CTRL_3_RESERVE :"
				, (u32_t)((data & MISC_PLL_STORM_CTRL_3_REG_PLL_STORM_CTRL_3_RESERVE) >> 25)
				, "\n\tPLL_STORM_CTRL_M2DIV :"
				, (u32_t)((data & MISC_PLL_STORM_CTRL_3_REG_PLL_STORM_CTRL_M2DIV) >> 17)
				, "\n\tPLL_STORM_CTRL_M1DIV :"
				, (u32_t)((data & MISC_PLL_STORM_CTRL_3_REG_PLL_STORM_CTRL_M1DIV) >> 9)
				, "\n\tPLL_STORM_CTRL_N :"
				, (u32_t)((data & MISC_PLL_STORM_CTRL_3_REG_PLL_STORM_CTRL_N) >> 0));
			break;
		case 0xa2b0UL: // (MISC_REG_RINGOSC: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tIFMUX_SEM_RINGOSC_MUX_SEL :"
				, (u32_t)((data & MISC_RINGOSC_REG_IFMUX_SEM_RINGOSC_MUX_SEL) >> 12)
				, "\n\tUSEM_RINGOSC_SEL1 :"
				, (u32_t)((data & MISC_RINGOSC_REG_USEM_RINGOSC_SEL1) >> 11)
				, "\n\tUSEM_RINGOSC_SEL0 :"
				, (u32_t)((data & MISC_RINGOSC_REG_USEM_RINGOSC_SEL0) >> 10)
				, "\n\tUSEM_RINGOSC_ENABLE :"
				, (u32_t)((data & MISC_RINGOSC_REG_USEM_RINGOSC_ENABLE) >> 9)
				, "\n\tTSEM_RINGOSC_SEL1 :"
				, (u32_t)((data & MISC_RINGOSC_REG_TSEM_RINGOSC_SEL1) >> 8)
				, "\n\tTSEM_RINGOSC_SEL0 :"
				, (u32_t)((data & MISC_RINGOSC_REG_TSEM_RINGOSC_SEL0) >> 7)
				, "\n\tTSEM_RINGOSC_ENABLE :"
				, (u32_t)((data & MISC_RINGOSC_REG_TSEM_RINGOSC_ENABLE) >> 6)
				, "\n\tXSEM_RINGOSC_SEL1 :"
				, (u32_t)((data & MISC_RINGOSC_REG_XSEM_RINGOSC_SEL1) >> 5)
				, "\n\tXSEM_RINGOSC_SEL0 :"
				, (u32_t)((data & MISC_RINGOSC_REG_XSEM_RINGOSC_SEL0) >> 4)
				, "\n\tXSEM_RINGOSC_ENABLE :"
				, (u32_t)((data & MISC_RINGOSC_REG_XSEM_RINGOSC_ENABLE) >> 3)
				, "\n\tCSEM_RINGOSC_SEL1 :"
				, (u32_t)((data & MISC_RINGOSC_REG_CSEM_RINGOSC_SEL1) >> 2)
				, "\n\tCSEM_RINGOSC_SEL0 :"
				, (u32_t)((data & MISC_RINGOSC_REG_CSEM_RINGOSC_SEL0) >> 1)
				, "\n\tCSEM_RINGOSC_ENABLE :"
				, (u32_t)((data & MISC_RINGOSC_REG_CSEM_RINGOSC_ENABLE) >> 0));
			break;
		case 0xa2b4UL: // (MISC_REG_SHARED_MEM_ADDR: ACCESS:RW  DataWidth:0x16 ) 
			sprintf(disp_str, "\nMISC_REG_SHARED_MEM_ADDR (ACCESS:RW  DataWidth:0x16 )" \
				"\nDescription: 22 bit GRC address where the scratch-pad of the MCP that \n    is shared with the driver resides\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2c8UL: // (MISC_REG_OTP_CTRL_REG_0: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nMISC_REG_OTP_CTRL_REG_0 (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMISC_OTP_CTRL_REG_0_REG_DEBUG :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_DEBUG) >> 31)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_SADBYP :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_SADBYP) >> 30)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_TM :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_TM) >> 27)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_UNUSED_2 :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_UNUSED_2) >> 24)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_VSEL :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_VSEL) >> 20)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_PBYP :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_PBYP) >> 19)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_PCOUNT :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_PCOUNT) >> 16)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_UNUSED_1 :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_UNUSED_1) >> 11)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_PROGSTART :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_PROGSTART) >> 10)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_PROGSEL :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_PROGSEL) >> 9)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_USEPINS :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_USEPINS) >> 8)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_UNUSED_0 :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_UNUSED_0) >> 6)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_ENABLE_OTP_2 :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_ENABLE_OTP_2) >> 5)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_ENABLE_OTP_1 :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_ENABLE_OTP_1) >> 4)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_ENABLE_OTP_0 :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_ENABLE_OTP_0) >> 3)
				, "\n\tMISC_OTP_CTRL_REG_0_REG_FMODE :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_0_REG_FMODE) >> 0));
			break;
		case 0xa2ccUL: // (MISC_REG_OTP_CTRL_REG_1: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nMISC_REG_OTP_CTRL_REG_1 (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMISC_OTP_CTRL_REG_1_REG_UNUSED_4 :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_1_REG_UNUSED_4) >> 25)
				, "\n\tMISC_OTP_CTRL_REG_1_REG_DOSEL :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_1_REG_DOSEL) >> 16)
				, "\n\tMISC_OTP_CTRL_REG_1_REG_UNUSED_3 :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_1_REG_UNUSED_3) >> 10)
				, "\n\tMISC_OTP_CTRL_REG_1_REG_OTP_ADDR :"
				, (u32_t)((data & MISC_OTP_CTRL_REG_1_REG_OTP_ADDR) >> 0));
			break;
		case 0xa2d0UL: // (MISC_REG_SW_TIMER_CFG_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_CFG_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: SW timer 1 configuration register. Maped as follows: bit0\n    =en (0-stop counting; 1-counting); bit1=reload (0-no reload; 1-when ~misc_registers_sw_timer_val\n    .sw_timer_val of timer 1 reach zero the register will be reload with ~\n    misc_registers_sw_timer_reload_val_1.sw_timer_reload_val_1)  ; bit2=attn\n     (0-no attention; 1-attention signal will be send to the AEU when the ~\n    misc_registers_sw_timer_val.sw_timer_val of timer 1 reach zero); bit4-\n    5=resolution (00-10us; 01-100us; 10-1ms; 11- 1us)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2d4UL: // (MISC_REG_SW_TIMER_CFG_2: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_CFG_2 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: SW timer 1 configuration register. Maped as follows: bit0\n    =en (0-stop counting; 1-counting); bit1=reload (0-no reload; 1-when ~misc_registers_sw_timer_val\n    .sw_timer_val of timer 2 reach zero the register will be reload with ~\n    misc_registers_sw_timer_reload_val_2.sw_timer_reload_val_2)  ; bit2=attn\n     (0-no attention; 1-attention signal will be send to the AEU when the ~\n    misc_registers_sw_timer_val.sw_timer_val of timer 2 reach zero); bit4-\n    5=resolution (00-10us; 01-100us; 10-1ms; 11-1us\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2d8UL: // (MISC_REG_SW_TIMER_CFG_3: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_CFG_3 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: SW timer 1 configuration register. Maped as follows: bit0\n    =en (0-stop counting; 1-counting); bit1=reload (0-no reload; 1-when ~misc_registers_sw_timer_val\n    .sw_timer_val of timer 3 reach zero the register will be reload with ~\n    misc_registers_sw_timer_reload_val_3.sw_timer_reload_val_3)  ; bit2=attn\n     (0-no attention; 1-attention signal will be send to the AEU when the ~\n    misc_registers_sw_timer_val.sw_timer_val of timer 3 reach zero); bit4-\n    5=resolution (00-10us; 01-100us; 10-1ms; 11-1us)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2dcUL: // (MISC_REG_SW_TIMER_CFG_4: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_CFG_4 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: SW timer 1 configuration register. Maped as follows: bit0\n    =en (0-stop counting; 1-counting); bit1=reload (0-no reload; 1-when ~misc_registers_sw_timer_val\n    .sw_timer_val of timer 4 reach zero the register will be reload with ~\n    misc_registers_sw_timer_reload_val_4.sw_timer_reload_val_4)  ; bit2=attn\n     (0-no attention; 1-attention signal will be send to the AEU when the ~\n    misc_registers_sw_timer_val.sw_timer_val of timer 4 reach zero); bit4-\n    5=resolution (00-10us; 01-100us; 10-1ms; 11-1us)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2e0UL: // (MISC_REG_SW_TIMER_CFG_5: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_CFG_5 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: SW timer 1 configuration register. Maped as follows: bit0\n    =en (0-stop counting; 1-counting); bit1=reload (0-no reload; 1-when ~misc_registers_sw_timer_val\n    .sw_timer_val of timer 5 reach zero the register will be reload with ~\n    misc_registers_sw_timer_reload_val_5.sw_timer_reload_val_5)  ; bit2=attn\n     (0-no attention; 1-attention signal will be send to the AEU when the ~\n    misc_registers_sw_timer_val.sw_timer_val of timer 5 reach zero); bit4-\n    5=resolution (00-10us; 01-100us; 10-1ms; 11-1us)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2e4UL: // (MISC_REG_SW_TIMER_CFG_6: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_CFG_6 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: SW timer 1 configuration register. Maped as follows: bit0\n    =en (0-stop counting; 1-counting); bit1=reload (0-no reload; 1-when ~misc_registers_sw_timer_val\n    .sw_timer_val of timer 6 reach zero the register will be reload with ~\n    misc_registers_sw_timer_reload_val_6.sw_timer_reload_val_6)  ; bit2=attn\n     (0-no attention; 1-attention signal will be send to the AEU when the ~\n    misc_registers_sw_timer_val.sw_timer_val of timer 6 reach zero); bit4-\n    5=resolution (00-10us; 01-100us; 10-1ms; 11-1us)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2e8UL: // (MISC_REG_SW_TIMER_CFG_7: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_CFG_7 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: SW timer 1 configuration register. Maped as follows: bit0\n    =en (0-stop counting; 1-counting); bit1=reload (0-no reload; 1-when ~misc_registers_sw_timer_val\n    .sw_timer_val of timer 7 reach zero the register will be reload with ~\n    misc_registers_sw_timer_reload_val_7.sw_timer_reload_val_7)  ; bit2=attn\n     (0-no attention; 1-attention signal will be send to the AEU when the ~\n    misc_registers_sw_timer_val.sw_timer_val of timer 7 reach zero); bit4-\n    5=resolution (00-10us; 01-100us; 10-1ms; 11-1us)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2ecUL: // (MISC_REG_SW_TIMER_CFG_8: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_CFG_8 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: SW timer 1 configuration register. Maped as follows: bit0\n    =en (0-stop counting; 1-counting); bit1=reload (0-no reload; 1-when ~misc_registers_sw_timer_val\n    .sw_timer_val of timer 8 reach zero the register will be reload with ~\n    misc_registers_sw_timer_reload_val_8.sw_timer_reload_val_8)  ; bit2=attn\n     (0-no attention; 1-attention signal will be send to the AEU when the ~\n    misc_registers_sw_timer_val.sw_timer_val of timer 8 reach zero); bit4-\n    5=resolution (00-10us; 01-100us; 10-1ms; 11-1us)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2f0UL: // (MISC_REG_SW_TIMER_RELOAD_VAL_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_RELOAD_VAL_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reload value for counter 1 if reload; the value will be reload\n     if the counter reached zero and the reload bit (~misc_registers_sw_timer_cfg_1\n    .sw_timer_cfg_1[1] ) is set\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2f4UL: // (MISC_REG_SW_TIMER_RELOAD_VAL_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_RELOAD_VAL_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reload value for counter 2 if reload; the value will be reload\n     if the counter reached zero and the reload bit (~misc_registers_sw_timer_cfg_2\n    .sw_timer_cfg_2[1]  ) is set\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2f8UL: // (MISC_REG_SW_TIMER_RELOAD_VAL_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_RELOAD_VAL_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reload value for counter 3 if reload; the value will be reload\n     if the counter reached zero and the reload bit (~misc_registers_sw_timer_cfg_3\n    .sw_timer_cfg_3[1]  ) is set\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2fcUL: // (MISC_REG_SW_TIMER_RELOAD_VAL_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_RELOAD_VAL_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reload value for counter 4 if reload; the value will be reload\n     if the counter reached zero and the reload bit (~misc_registers_sw_timer_cfg_4\n    .sw_timer_cfg_4[1]  ) is set\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa300UL: // (MISC_REG_SW_TIMER_RELOAD_VAL_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_RELOAD_VAL_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reload value for counter 5 if reload; the value will be reload\n     if the counter reached zero and the reload bit (~misc_registers_sw_timer_cfg_1\n    .sw_timer_cfg_5[1]  ) is set\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa304UL: // (MISC_REG_SW_TIMER_RELOAD_VAL_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_RELOAD_VAL_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reload value for counter 6 if reload; the value will be reload\n     if the counter reached zero and the reload bit (~misc_registers_sw_timer_cfg_1\n    .sw_timer_cfg_6[1]  ) is set\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa308UL: // (MISC_REG_SW_TIMER_RELOAD_VAL_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_RELOAD_VAL_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reload value for counter 1 if reload; the value will be reload\n     if the counter reached zero and the reload bit (~misc_registers_sw_timer_cfg_1\n    .sw_timer_cfg_7[1] ) is set\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa30cUL: // (MISC_REG_SW_TIMER_RELOAD_VAL_8: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_RELOAD_VAL_8 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reload value for counter 8 if reload; the value will be reload\n     if the counter reached zero and the reload bit (~misc_registers_sw_timer_cfg_1\n    .sw_timer_cfg_8[1] ) is set\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa310UL: // (MISC_REG_SW_TIMER_10US_RESOLUTION: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_10US_RESOLUTION (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: fine tuning for sw 10us timer; max value=3000; min value=\n    2000. the 10us timer is the base counter for all the timers.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa314UL: // (MISC_REG_GRC_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_GRC_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from GRC\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa318UL: // (MISC_REG_GRC_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_GRC_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug Only: For dbgmux usage (debug data that goes from GRC\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select; before the shift)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa31cUL: // (MISC_REG_GRC_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_GRC_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug Only: For dbgmux usage (debug data that goes from GRC\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa320UL: // (MISC_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from MISC\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa324UL: // (MISC_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from MISC\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select; before the shift)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa328UL: // (MISC_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from MISC\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa37cUL: // (MISC_REG_MISC_INT_STS: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_MISC_INT_STS (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTX_LPI_REQ_P1 :"
				, (u32_t)((data & MISC_MISC_INT_STS_REG_TX_LPI_REQ_P1) >> 7)
				, "\n\tTX_LPI_REQ_P0 :"
				, (u32_t)((data & MISC_MISC_INT_STS_REG_TX_LPI_REQ_P0) >> 6)
				, "\n\tRX_LPI_P1 :"
				, (u32_t)((data & MISC_MISC_INT_STS_REG_RX_LPI_P1) >> 5)
				, "\n\tRX_LPI_P0 :"
				, (u32_t)((data & MISC_MISC_INT_STS_REG_RX_LPI_P0) >> 4)
				, "\n\tGENERIC_SW :"
				, (u32_t)((data & MISC_MISC_INT_STS_REG_GENERIC_SW) >> 3)
				, "\n\tTIMEOUT_ATTN :"
				, (u32_t)((data & MISC_MISC_INT_STS_REG_TIMEOUT_ATTN) >> 2)
				, "\n\tRSV_ACCESS_ATTN :"
				, (u32_t)((data & MISC_MISC_INT_STS_REG_RSV_ACCESS_ATTN) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & MISC_MISC_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xa380UL: // (MISC_REG_MISC_INT_STS_CLR: ACCESS:RC  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_MISC_INT_STS_CLR (ACCESS:RC  DataWidth:0x8 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTX_LPI_REQ_P1 :"
				, (u32_t)((data & MISC_MISC_INT_STS_CLR_REG_TX_LPI_REQ_P1) >> 7)
				, "\n\tTX_LPI_REQ_P0 :"
				, (u32_t)((data & MISC_MISC_INT_STS_CLR_REG_TX_LPI_REQ_P0) >> 6)
				, "\n\tRX_LPI_P1 :"
				, (u32_t)((data & MISC_MISC_INT_STS_CLR_REG_RX_LPI_P1) >> 5)
				, "\n\tRX_LPI_P0 :"
				, (u32_t)((data & MISC_MISC_INT_STS_CLR_REG_RX_LPI_P0) >> 4)
				, "\n\tGENERIC_SW :"
				, (u32_t)((data & MISC_MISC_INT_STS_CLR_REG_GENERIC_SW) >> 3)
				, "\n\tTIMEOUT_ATTN :"
				, (u32_t)((data & MISC_MISC_INT_STS_CLR_REG_TIMEOUT_ATTN) >> 2)
				, "\n\tRSV_ACCESS_ATTN :"
				, (u32_t)((data & MISC_MISC_INT_STS_CLR_REG_RSV_ACCESS_ATTN) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & MISC_MISC_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xa384UL: // (MISC_REG_MISC_INT_STS_WR: ACCESS:WR  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_MISC_INT_STS_WR (ACCESS:WR  DataWidth:0x8 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTX_LPI_REQ_P1 :"
				, (u32_t)((data & MISC_MISC_INT_STS_WR_REG_TX_LPI_REQ_P1) >> 7)
				, "\n\tTX_LPI_REQ_P0 :"
				, (u32_t)((data & MISC_MISC_INT_STS_WR_REG_TX_LPI_REQ_P0) >> 6)
				, "\n\tRX_LPI_P1 :"
				, (u32_t)((data & MISC_MISC_INT_STS_WR_REG_RX_LPI_P1) >> 5)
				, "\n\tRX_LPI_P0 :"
				, (u32_t)((data & MISC_MISC_INT_STS_WR_REG_RX_LPI_P0) >> 4)
				, "\n\tGENERIC_SW :"
				, (u32_t)((data & MISC_MISC_INT_STS_WR_REG_GENERIC_SW) >> 3)
				, "\n\tTIMEOUT_ATTN :"
				, (u32_t)((data & MISC_MISC_INT_STS_WR_REG_TIMEOUT_ATTN) >> 2)
				, "\n\tRSV_ACCESS_ATTN :"
				, (u32_t)((data & MISC_MISC_INT_STS_WR_REG_RSV_ACCESS_ATTN) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & MISC_MISC_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xa388UL: // (MISC_REG_MISC_INT_MASK: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_MISC_INT_MASK (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTX_LPI_REQ_P1 :"
				, (u32_t)((data & MISC_MISC_INT_MASK_REG_TX_LPI_REQ_P1) >> 7)
				, "\n\tTX_LPI_REQ_P0 :"
				, (u32_t)((data & MISC_MISC_INT_MASK_REG_TX_LPI_REQ_P0) >> 6)
				, "\n\tRX_LPI_P1 :"
				, (u32_t)((data & MISC_MISC_INT_MASK_REG_RX_LPI_P1) >> 5)
				, "\n\tRX_LPI_P0 :"
				, (u32_t)((data & MISC_MISC_INT_MASK_REG_RX_LPI_P0) >> 4)
				, "\n\tGENERIC_SW :"
				, (u32_t)((data & MISC_MISC_INT_MASK_REG_GENERIC_SW) >> 3)
				, "\n\tTIMEOUT_ATTN :"
				, (u32_t)((data & MISC_MISC_INT_MASK_REG_TIMEOUT_ATTN) >> 2)
				, "\n\tRSV_ACCESS_ATTN :"
				, (u32_t)((data & MISC_MISC_INT_MASK_REG_RSV_ACCESS_ATTN) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & MISC_MISC_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xa38cUL: // (MISC_REG_MISC_PRTY_STS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_MISC_PRTY_STS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & MISC_MISC_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0xa390UL: // (MISC_REG_MISC_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_MISC_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & MISC_MISC_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0xa394UL: // (MISC_REG_MISC_PRTY_STS_WR: ACCESS:WR  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_MISC_PRTY_STS_WR (ACCESS:WR  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & MISC_MISC_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0xa398UL: // (MISC_REG_MISC_PRTY_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_MISC_PRTY_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & MISC_MISC_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0xa3a0UL: // (MISC_REG_AEU_MCP_SCPAD_PERR: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_MCP_SCPAD_PERR (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: One in each bit shows that the appropriate MCP scpad parity\n     error occurs.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3acUL: // (MISC_REG_AEU_MASK_ATTN_FUNC_0_MSB: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_MASK_ATTN_FUNC_0_MSB (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: mask 8 MSB attention output signals toward IGU function0; \n    Zero = mask; one = unmask\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3b0UL: // (MISC_REG_AEU_MASK_ATTN_FUNC_1_MSB: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_MASK_ATTN_FUNC_1_MSB (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: mask 8 MSB attention output signals toward IGU function1; \n    Zero = mask; one = unmask\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3c0UL: // (MISC_REG_GRC_RSV_ATTN: ACCESS:R  DataWidth:0x1c ) 
			sprintf(disp_str, "\nMISC_REG_GRC_RSV_ATTN (ACCESS:R  DataWidth:0x1c )" \
				"\nDescription: this field hold the last information that caused reserved\n     attention. bits [19:0] - address; [22:20] function; [23] reserved; [27\n    :24] the master that caused the attention -  according to the following\n     encodeing:1 = pxp; 2 = mcp; 3 = usdm; 4 = tsdm; 5 = xsdm; 6 = csdm; 7\n     = dbu; 8 = dmae\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3c4UL: // (MISC_REG_GRC_TIMEOUT_ATTN: ACCESS:R  DataWidth:0x1c ) 
			sprintf(disp_str, "\nMISC_REG_GRC_TIMEOUT_ATTN (ACCESS:R  DataWidth:0x1c )" \
				"\nDescription: this field hold the last information that caused timeout \n    attention. bits [19:0] - address; [22:20] function; [23] reserved; [27\n    :24] the master that caused the attention -  according to the following\n     encodeing:1 = pxp; 2 = mcp; 3 = usdm; 4 = tsdm; 5 = xsdm; 6 = csdm; 7\n     = dbu; 8 = dmae\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa400UL: // (MISC_REG_BOND_ID: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_BOND_ID (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: This field indicates the type of the device. '0' - 2 Ports\n    ; '1' - 1 Port. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa404UL: // (MISC_REG_CHIP_METAL: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_CHIP_METAL (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: These bits indicate the metal revision of the chip. This \n    value starts at 0x00 for each all-layer tape-out and increments by one\n     for each tape-out. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa408UL: // (MISC_REG_CHIP_NUM: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nMISC_REG_CHIP_NUM (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: These bits indicate the part number for the chip. Global \n    register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa40cUL: // (MISC_REG_CHIP_REV: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_CHIP_REV (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: These bits indicate the base revision of the chip. This value\n     starts at 0x0 for the A0 tape-out and increments by one for each all-\n    layer tape-out.  Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa410UL: // (MISC_REG_CHIP_TEST_REG: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_CHIP_TEST_REG (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: These bits indicate the silent revision of the chip. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa418UL: // (MISC_REG_LINK_HOLDOFF_SUCCESS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_LINK_HOLDOFF_SUCCESS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: This bit indicates the PCIE link is successfully being held\n     from starting training. Used in conjunction with ~MISC_REGISTERS_LINK_HOLDOFF_REQ\n    .LINK_HOLDOFF_REQ. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa41cUL: // (MISC_REG_LINK_IN_L23: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_LINK_IN_L23 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When this bit is 1 it indicates that the link is down and\n     PCIE is prepared for operation off of VAUX. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa420UL: // (MISC_REG_PCIE_DIS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PCIE_DIS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: This bit reports the current state of the PCIE_DIS pin. If\n     this bit is 1 it means that the LOM design has been strapped to support\n     management only. The PCI power will always read as '0' in this state; \n    as if the chip is in Out-Of-Box WOL mode. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa428UL: // (MISC_REG_VAUX_PRESENT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_VAUX_PRESENT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: 0 - VAUX is not present; 1 - VAUX is present. Global register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa498UL: // (MISC_REG_ISOLATION_LOGIC: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_ISOLATION_LOGIC (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: the isolation between Vaux and Vmain read value. Global register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa49cUL: // (MISC_REG_LCPLL_MISC_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_MISC_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: lcpll lock signals. 0-unlocked; 1-locked. Global register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4b0UL: // (MISC_REG_P0_SERDES_PLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_P0_SERDES_PLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: serdes port 0 pll lock signals. 0-unlocked; 1-locked\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4b4UL: // (MISC_REG_P0_XGXS_PLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_P0_XGXS_PLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: xgxs port 0 pll lock signals. 0-unlocked; 1-locked\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4b8UL: // (MISC_REG_P1_SERDES_PLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_P1_SERDES_PLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: serdes port 1 pll lock signals. 0-unlocked; 1-locked\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4bcUL: // (MISC_REG_P1_XGXS_PLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_P1_XGXS_PLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: xgxs port 1 pll lock signals. 0-unlocked; 1-locked\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4c0UL: // (MISC_REG_PIPE_MISC_PLL_LOCK: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PIPE_MISC_PLL_LOCK (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: PCIE SERDES pll lock signals. 0-unlocked; 1-locked. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa548UL: // (MISC_REG_PLL_MAIN_CTRL_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_CTRL_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa550UL: // (MISC_REG_PLL_MAIN_CTRL_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_CTRL_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa558UL: // (MISC_REG_PLL_MAIN_CTRL_3_M1DIV: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_CTRL_3_M1DIV (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa560UL: // (MISC_REG_PLL_MAIN_CTRL_3_M2DIV: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_CTRL_3_M2DIV (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa568UL: // (MISC_REG_PLL_MAIN_CTRL_3_N: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_CTRL_3_N (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa5fcUL: // (MISC_REG_AEU_VPD_LATCH_STATUS: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_VPD_LATCH_STATUS (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: represent the status of pxpv_misc_vpd0_attn (bit 0) - pxpv_misc_vpd7_attn\n     (bit 7) after latching.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa618UL: // (MISC_REG_PCIE_HOT_RESET: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PCIE_HOT_RESET (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: If set indicate that the pcie_rst_b was asserted without \n    perst assertion. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa61cUL: // (MISC_REG_AEU_GENERAL_MASK: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nMISC_REG_AEU_GENERAL_MASK (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tAEU_SYS_KILL_MASK :"
				, (u32_t)((data & MISC_AEU_GENERAL_MASK_REG_AEU_SYS_KILL_MASK) >> 2)
				, "\n\tAEU_NIG_CLOSE_MASK :"
				, (u32_t)((data & MISC_AEU_GENERAL_MASK_REG_AEU_NIG_CLOSE_MASK) >> 1)
				, "\n\tAEU_PXP_CLOSE_MASK :"
				, (u32_t)((data & MISC_AEU_GENERAL_MASK_REG_AEU_PXP_CLOSE_MASK) >> 0));
			break;
		case 0xa620UL: // (MISC_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: eco reserved. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa62cUL: // (MISC_REG_CPU_OTP_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPU_OTP_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: [0]: command_done: This bit is set when the state machine\n     has returned to IDLE. This will not be set until the last bit of a WORD\n     program is complete; [1]: wrp_data_ready: Wrapper indicates data is ready\n    ; [2]: wrp_bit_dout: single bit otp_dout pointed to by wrp_addr; [3]: \n    wrp_busy: The otp_busy was seen - even if it is done. Useful to tell if\n     the state machine ever left IDLE. For a valid command the otp_wrapper\n     state machine asserts this signal indicating that it is busy in executing\n     the command; [4]: wrp_fail: A FAIL occurred on 1 or more commands; [5\n    ]: invalid_prog_req: A program request was sent with the incorrect access\n     mode; [6]: prog_blocked: Programming attempted on a locked row was blocked\n    ; [7]: wrp_fdone: OTP is ready to take command. Wait for this signal to\n     go high before issuing any command after POR or auto_reload command; [\n    8]: otp_stby_reg: Indicates that the OTP cell is in standby mode. The \n    only command that is valid during this time is a WAKEUP command; [9]:\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa634UL: // (MISC_REG_CPU_OTP_READ_DATA: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPU_OTP_READ_DATA (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: SHARED:   Data output from the OTP read data command. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa688UL: // (MISC_REG_AEU_ENABLE5_FUNC_0_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_0_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fifth 32b for enabling the output for function 0 output0. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa68cUL: // (MISC_REG_AEU_ENABLE5_FUNC_0_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_0_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fifth 32b for enabling the output for function 0 output1. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa690UL: // (MISC_REG_AEU_ENABLE5_FUNC_0_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_0_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fifth 32b for enabling the output for function 0 output2. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [[6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] \n    mstat1 parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa694UL: // (MISC_REG_AEU_ENABLE5_FUNC_0_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_0_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fifth 32b for enabling the output for function 0 output3. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [6] mstat0\n     attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1 parity\n    ; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa698UL: // (MISC_REG_AEU_ENABLE5_FUNC_0_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_0_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fifth 32b for enabling the output for function 0 output4. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ;[6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa69cUL: // (MISC_REG_AEU_ENABLE5_FUNC_0_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_0_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fifth 32b for enabling the output for function 0 output5. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6a0UL: // (MISC_REG_AEU_ENABLE5_FUNC_0_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_0_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: fifth 32b for enabling the output for function 0 output6. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6a4UL: // (MISC_REG_AEU_ENABLE5_FUNC_0_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_0_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for function 0 output7. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6a8UL: // (MISC_REG_AEU_ENABLE5_NIG_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_NIG_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for close the gate nig. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6acUL: // (MISC_REG_AEU_ENABLE5_PXP_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_PXP_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for close the gate pxp. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ;[6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6b0UL: // (MISC_REG_AEU_ENABLE5_FUNC_1_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_1_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for function 1 output0. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6b4UL: // (MISC_REG_AEU_ENABLE5_FUNC_1_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_1_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for function 1 output1. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6b8UL: // (MISC_REG_AEU_ENABLE5_FUNC_1_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_1_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for function 1 output2. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6bcUL: // (MISC_REG_AEU_ENABLE5_FUNC_1_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_1_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for function 1 output3. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6c0UL: // (MISC_REG_AEU_ENABLE5_FUNC_1_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_1_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for function 1 output4. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6c4UL: // (MISC_REG_AEU_ENABLE5_FUNC_1_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_1_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for function 1 output5. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6c8UL: // (MISC_REG_AEU_ENABLE5_FUNC_1_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_1_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for function 1 output6. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ;[6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6ccUL: // (MISC_REG_AEU_ENABLE5_FUNC_1_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_FUNC_1_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for function 1 output7. \n    Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE\n     B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ;[6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6d0UL: // (MISC_REG_AEU_ENABLE5_PXP_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_PXP_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for close the gate system\n     kill. Mapped as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [\n    2] PGLUE B RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5\n    ] ATC parity;[6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention\n    ; [9] mstat1 parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6d4UL: // (MISC_REG_AEU_ENABLE5_MCP_OUT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_MCP_OUT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for mcp output0. Mapped\n     as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B \n    RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6d8UL: // (MISC_REG_AEU_ENABLE5_MCP_OUT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_MCP_OUT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for mcp output1. Mapped\n     as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B \n    RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6dcUL: // (MISC_REG_AEU_ENABLE5_MCP_OUT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_MCP_OUT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for mcp output2. Mapped\n     as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B \n    RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6e0UL: // (MISC_REG_AEU_ENABLE5_MCP_OUT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_MCP_OUT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for mcp output3. Mapped\n     as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B \n    RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6e4UL: // (MISC_REG_AEU_ENABLE5_MCP_OUT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_MCP_OUT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for mcp output4. Mapped\n     as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B \n    RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6e8UL: // (MISC_REG_AEU_ENABLE5_MCP_OUT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_MCP_OUT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for mcp output5. Mapped\n     as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B \n    RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ;[6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6ecUL: // (MISC_REG_AEU_ENABLE5_MCP_OUT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_MCP_OUT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for mcp output6. Mapped\n     as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B \n    RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6f0UL: // (MISC_REG_AEU_ENABLE5_MCP_OUT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_ENABLE5_MCP_OUT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for enabling the output for mcp output7. Mapped\n     as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B \n    RBC attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity\n    ; [6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6f4UL: // (MISC_REG_AEU_INVERTER_5_FUNC_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_5_FUNC_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for inverting the input for function 0; Mapped \n    as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B RBC\n     attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity; [\n    6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6f8UL: // (MISC_REG_AEU_INVERTER_5_FUNC_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_5_FUNC_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for inverting the input for function 1; Mapped \n    as follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B RBC\n     attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity; [\n    6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa6fcUL: // (MISC_REG_AEU_INVERTER_5_MCP: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_INVERTER_5_MCP (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Fifth 32b for inverting the input for mcp; Mapped as follows\n    :  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B RBC attention\n     [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity; [6] mstat0\n     attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1 parity\n    ; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa710UL: // (MISC_REG_GRC_RSV_ATTN_FULL_FID: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_GRC_RSV_ATTN_FULL_FID (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: Holds the last FID that caused reserved attention. Need to\n     be used in conjunction with ~misc_registers_grc_rsv_attn; where 3 bits\n     of function (3 lsb) are also represented. Bit[2:0] - PFID; bit[3] - VFID\n     valid; bit[9:4] - VFID. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa714UL: // (MISC_REG_GRC_TIMEOUT_ATTN_FULL_FID: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_GRC_TIMEOUT_ATTN_FULL_FID (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: Holds the last FID that caused timeout attention. Need to\n     be used in conjunction with ~misc_registers_timeout_attn; where 3 bits\n     of function (3 lsb) are also represented. Bit[2:0] - PFID; bit[3] - VFID\n     valid; bit[9:4] - VFID. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa718UL: // (MISC_REG_FUNC_HIDE_PIN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_FUNC_HIDE_PIN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Synchronised value of ifmux_misc_func_hide. Global register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa750UL: // (MISC_REG_PORT4MODE_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PORT4MODE_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Status of 4 port mode enable input pin.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa754UL: // (MISC_REG_FOUR_PORT_PORT_SWAP: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_FOUR_PORT_PORT_SWAP (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Status of 4 port mode port swap input pin.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa758UL: // (MISC_REG_TWO_PORT_PATH_SWAP: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_TWO_PORT_PATH_SWAP (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Status of two port mode path swap input pin.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa75cUL: // (MISC_REG_FOUR_PORT_PATH_SWAP: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_FOUR_PORT_PATH_SWAP (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Status of four port mode path swap input pin.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa784UL: // (MISC_REG_EXT_PHY_MODE_STRAPS: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_EXT_PHY_MODE_STRAPS (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Status of PHY mode straps {TEST_IN[0];TEST_IN[1];EJTAG_TMS\n    ;EJTAG_TDI} input pin (ifmux_misc_ext_phy_mode_straps).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7e0UL: // (MISC_REG_PLL_MAIN_E40_OTP_PDIV: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_OTP_PDIV (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Input reference clock pre-divider control (code = divider\n     ratio). 000: divide-by-8; 001: divide-by-1; 010: divide-by-2; 011: divide\n    -by-3; 100: divide-by-4; 101: divide-by-5; 110: divide-by-6; 111: divide\n    -by-7. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7e4UL: // (MISC_REG_PLL_MAIN_E40_OTP_NDIV_INT: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_OTP_NDIV_INT (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: Feedback divider control (Code = divider ratio). 0000000000\n    : divide-by-1024; 0000000001: XXX; 0000000010: XXX; 0000000111: XXX; 0000001000\n    : divide-by-8; 0000001001: divide-by-9; 0000001010: divide-by-10; 1111111110\n    : divide-by-1022; 1111111111: divide-by-1023. Global register. Reset on\n     POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7e8UL: // (MISC_REG_PLL_MAIN_E40_OTP_NDIV_FRAC: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_OTP_NDIV_FRAC (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Fractional feedback divider control. Resolution= 1/(2^20). \n    Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7ecUL: // (MISC_REG_PLL_MAIN_E40_OTP_CH0_MDIV: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_OTP_CH0_MDIV (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Post-divider ratio for channel-0 (divider ratio = code value\n    ). 00000000: 256; 00000001: 1; 00000010: 2; 00000011: 3; : :; 11111101\n    : 253; 11111110: 254; 11111111: 255. Global register. Reset on POR reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7f0UL: // (MISC_REG_PLL_MAIN_E40_OTP_CTRL_1: ACCESS:R  DataWidth:0x1e ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_OTP_CTRL_1 (ACCESS:R  DataWidth:0x1e )" \
				"\nDescription: PLL configuration control register. [11:0]: dco_ctrl_bypass\n    [11:0]: direct programming of DAC: 000000000000 = MIN VCO clock frequency\n    ; : ; 111111111111 = MAX VCO clock frequency; [12]: dco_ctrl_bypass_enable\n    : enable of direct programming of DAC: 0 =normal mode; 1 =DAC programming\n     mode; [13]: stat_reset: reset of phase error measurement: 0 =normal mode\n    ; 1 =reset; [16:14]: stat_select[2:0]: select of test output: 000 = 000000000000\n    ; 001 = minimum phase error; 010 = maximum phase error; 011 = mean-square\n     phase error; 100 = dac control word; 101 = 000000000000; 110 = 000000000000\n    ; 111 = 000000000000; [17]: stat_update: On the synchronized rising edge\n     of this control signal the value selected by stat_select[2:0] is clocked\n     into o_statout; [18]: refclkout: Enable for o_fref output: 0 = o_fref\n     at logic 0; 1 = o_fref enabled (pre-divider output clock); [19]: AUX_CTRL\n    : ICO current: 0 = normal mode; 1 = Ivco increased by 130 ?A; [20]: VCOdiv2_Post\n    : Enables divide-by-2 for the clock t\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7f4UL: // (MISC_REG_PLL_MAIN_E40_OTP_CTRL_2: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_OTP_CTRL_2 (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: Configuration control register. [2:0]: i_ka[2:0]: Loop gain\n     in frequency acquisition mode; [5:3]: i_ki[2:0]: Gain of P/I loop filter\n     integrator path during fine phase acquisition mode; [9:6]: i_kp[3:0]: \n    Gain of P/I loop filter proportional path during fine phase acquisition\n     mode. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8d4UL: // (MISC_REG_RX_LPI_P0_STAT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RX_LPI_P0_STAT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: local port 0 SERDES is receiving (Rx) LPI status reference\n     value. If local port 0 SERDES is receiving (Rx) LPI status is different\n     from this value the corresponding interrupt is issued.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8d8UL: // (MISC_REG_RX_LPI_P1_STAT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RX_LPI_P1_STAT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: local port 1 SERDES is receiving (Rx) LPI status reference\n     value. If local port 1 SERDES is receiving (Rx) LPI status is different\n     from this value the corresponding interrupt is issued.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8dcUL: // (MISC_REG_TX_LPI_REQ_P0_STAT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_TX_LPI_REQ_P0_STAT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: local port 0 SERDES should enter (Tx) LPI mode reference \n    value. If local port 0 SERDES should enter (Tx) LPI mode is different \n    from this value the corresponding interrupt is issued.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8e0UL: // (MISC_REG_TX_LPI_REQ_P1_STAT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_TX_LPI_REQ_P1_STAT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: local port 1 SERDES should enter (Tx) LPI mode reference \n    value. If local port 1 SERDES should enter (Tx) LPI mode is different \n    from this value the corresponding interrupt is issued.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa974UL: // (MISC_REG_PORT_LANE_MODE: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_PORT_LANE_MODE (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Status of swap mode input pin.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa20UL: // (MISC_REG_WC0_PLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC0_PLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: WC0 pll lock signals. 0-unlocked; 1-locked.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa24UL: // (MISC_REG_WC1_PLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC1_PLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: WC1 pll lock signals. 0-unlocked; 1-locked.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa28UL: // (MISC_REG_WC2_PLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC2_PLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: WC2 pll lock signals. 0-unlocked; 1-locked.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xac60UL: // (MISC_REG_CHIP_TYPE: ACCESS:R  DataWidth:0xe ) 
			sprintf(disp_str, "\nMISC_REG_CHIP_TYPE (ACCESS:R  DataWidth:0xe )" \
				"\nDescription: otp_misc_do[100:0] spare bits collection: 13:11- otp_misc_do\n    [100:98]; 10:7 - otp_misc_do[87:84]; 6:3 - otp_misc_do[75:72]; 2:1 - otp_misc_do\n    [51:50]; 0 - otp_misc_do[1].\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa05cUL: // (MISC_REG_AEU_PRESET_REF: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_PRESET_REF (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The ref value for PERST signal change detection. Default \n    value is 1 (attention on PERST assertion).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa260UL: // (MISC_REG_SCPAD_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_SCPAD_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tm bits for mcp_scpad memory. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa264UL: // (MISC_REG_RX_TM: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_RX_TM (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: tm bits for ump_rx memory. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa268UL: // (MISC_REG_TX_TM: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_TX_TM (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: tm bits for ump_tx memory. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa26cUL: // (MISC_REG_PARITY_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PARITY_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only : parity mode to MCP. Setting this bit changes\n     the parity checking on the RAMs from even to odd parity. Global register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa270UL: // (MISC_REG_NIG_WOL_P0: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nMISC_REG_NIG_WOL_P0 (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: when set to one indicates WOL is detected by the MCP FW on\n     port 0. this register is multiply per function.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa274UL: // (MISC_REG_NIG_WOL_P1: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nMISC_REG_NIG_WOL_P1 (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: when set to one indicates WOL is detected by the MCP FW on\n     port 1. this register is multiply per function.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa28cUL: // (MISC_REG_UNCOND_ENTER_PLAY_DEAD: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_UNCOND_ENTER_PLAY_DEAD (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Writing to this register result with resetting entire chip\n     via the play dead mechanism.  Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa290UL: // (MISC_REG_COND_ENTER_PLAY_DEAD: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_COND_ENTER_PLAY_DEAD (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Writing to this register result with resetting entire chip\n     via the play dead mechanism if PERST is asserted.  Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2a0UL: // (MISC_REG_PLL_STORM_CTRL_4: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_CTRL_4 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: [0]clock storm bypass: 0-select Storm SPLL clock; 1-select\n     external clock; [1]PLL storm reset- Analog reset toward the Storm PLL\n    ; [2]PLL storm digital reset- Digital reset toward the Storm PLL. Global\n     register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2a4UL: // (MISC_REG_LCPLL_CTRL_1: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_CTRL_1 (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: 28 LSB of LCPLL first register; reset val = 521. inside order\n     of the bits is: [2:0] OAC reset value 001) CML output buffer bias control\n    ; 111 for +40%%; 011 for +20%%; 001 for 0%%; 000 for -20%%. [5:3] Icp_ctrl\n     (reset value 001) Charge pump current control; 111 for 720u; 011 for \n    600u; 001 for 480u and 000 for 360u. [7:6] Bias_ctrl (reset value 00) \n    Global bias control; When bit 7 is high bias current will be 10 0gh; When\n     bit 6 is high bias will be 100w; Valid values are 00; 10; 01. [10:8] \n    Pll_observe (reset value 010) Bits to control observability. bit 10 is\n     for test bias; bit 9 is for test CK; bit 8 is test Vc. [12:11] Vth_ctrl\n     (reset value 00) Comparator threshold control. 00 for 0.6V; 01 for 0.\n    54V and 10 for 0.66V. [13] pllSeqStart (reset value 0) Enables VCO tuning\n     sequencer: 1= sequencer disabled; 0= sequencer enabled (inverted internally\n    ). [14] reserved (reset value 0) Reset for VCO sequencer is connected \n    to RESET input directly. [15] capRetry_en (reset value 0) enable ret\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2acUL: // (MISC_REG_OSCILLATOR: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_OSCILLATOR (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Debug only. [0] - VMAIN oscillator control enable [1] - VMAIN\n     oscillator control select 0 [2] - VMAIN oscillator control select 1 [\n    3] - VAUX oscillator control enable [4] - VAUX oscillator control select\n     0 [5] - VAUX oscillator control select 1 [6] - PAD oscillator control\n     enable [7] - PAD oscillator control select 0 [8] - PAD oscillator control\n     select 1 [9] - PAD mux select. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2b8UL: // (MISC_REG_SPIO_EVENT_EN: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_SPIO_EVENT_EN (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: These bits enable the SPIO_INTs to signals event to the IGU\n    /MC. according to the following map: [3:0] reserved; [4] spio_4 [5] spio_5\n    ; [7:6] reserved. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2bcUL: // (MISC_REG_GPIO_EVENT_EN: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_GPIO_EVENT_EN (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: These bits enable the GPIO_INTs to signals event to the IGU\n    /MCP.according to the following map: [0] p0_gpio_0; [1] p0_gpio_1; [2] \n    p0_gpio_2; [3] p0_gpio_3; [4] p1_gpio_0; [5] p1_gpio_1; [6] p1_gpio_2; [\n    7] p1_gpio_3;  Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2c0UL: // (MISC_REG_NVM_WR_EN: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_NVM_WR_EN (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: These bits control how the write-disable to the NVM block\n     is generated. '0' - PROTECT: This value protects the NVM from any writes\n    . '1' - PCI: This value allows writes only when PCI\\_RST\\ is high.' \n    2' - ALLOW: This value allows writes to the NVM using normal NVM commands\n    . '3' - ALLOW2: This value allows writes to the NVM using normal NVM commands\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2c4UL: // (MISC_REG_LINK_HOLDOFF_REQ: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_LINK_HOLDOFF_REQ (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This bit is written to a '1' to request that the PCIE link\n     not begin training yet. Software should set this bit; and then check \n    the ~MISC_REGISTERS_LINK_HOLDOFF_SUCCESS.LINK_HOLDOFF_SUCCESS  bit. If\n     ~MISC_REGISTERS_LINK_HOLDOFF_SUCCESS.LINK_HOLDOFF_SUCCESS  is set; configure\n     the PCIE link and then clear this bit. If ~MISC_REGISTERS_LINK_HOLDOFF_SUCCESS\n    .LINK_HOLDOFF_SUCCESS  is not set; the PCIE link has already begun training\n     so it's too late to do any configuration. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa32cUL: // (MISC_REG_VMAIN_POR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_VMAIN_POR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0-bypass the Vmain PORBG. for Vmain POR; if sel=1 the output\n     wil be #misc_registers_vmain_por.vmain_por[1]; 1- bypass the Vmain PORBG\n    . If #misc_registers_vmain_por.vmain_por[0] is '1' the output of Vmain\n     POR will be this field. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa330UL: // (MISC_REG_RINGOSC_TOP_0_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_0_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control enable. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa334UL: // (MISC_REG_RINGOSC_TOP_0_SEL0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_0_SEL0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 0. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa338UL: // (MISC_REG_RINGOSC_TOP_0_SEL1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_0_SEL1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 1 . Global register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa33cUL: // (MISC_REG_RINGOSC_TOP_1_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_1_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control enable. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa340UL: // (MISC_REG_RINGOSC_TOP_1_SEL0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_1_SEL0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 0. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa344UL: // (MISC_REG_RINGOSC_TOP_1_SEL1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_1_SEL1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 1. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa348UL: // (MISC_REG_RINGOSC_TOP_2_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_2_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control enable. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa34cUL: // (MISC_REG_RINGOSC_TOP_2_SEL0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_2_SEL0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 0. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa350UL: // (MISC_REG_RINGOSC_TOP_2_SEL1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_2_SEL1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 1. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa354UL: // (MISC_REG_RINGOSC_TOP_3_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_3_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control enable. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa358UL: // (MISC_REG_RINGOSC_TOP_3_SEL0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_3_SEL0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 0. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa35cUL: // (MISC_REG_RINGOSC_TOP_3_SEL1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_3_SEL1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 1. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa360UL: // (MISC_REG_RINGOSC_TOP_4_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_4_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control enable. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa364UL: // (MISC_REG_RINGOSC_TOP_4_SEL0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_4_SEL0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 0. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa368UL: // (MISC_REG_RINGOSC_TOP_4_SEL1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_4_SEL1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 1. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa36cUL: // (MISC_REG_RINGOSC_TOP_5_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_5_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control enable. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa370UL: // (MISC_REG_RINGOSC_TOP_5_SEL0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_5_SEL0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 0. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa374UL: // (MISC_REG_RINGOSC_TOP_5_SEL1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_RINGOSC_TOP_5_SEL1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. oscillator control select 1. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa378UL: // (MISC_REG_TOP_RINGOSC_MUX_SEL: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_TOP_RINGOSC_MUX_SEL (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: top ringosc mux select. In debug mode maps the\n     following SEM ring oscillators to TDO output: [0] - CSEM0; [1] - XSEM0\n    ; [2] - TSEM0; [3] - USEM0; [4] - CSEM1; [5] - XSEM1; [6] - TSEM1; [7] - \n    USEM1. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa39cUL: // (MISC_REG_PORT_SWAP_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PORT_SWAP_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set the port swap feature for the GPIOs is controlled \n    by the port swap IO (or by override register). If clear port swap feature\n     for the GPIOs is disabled. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3a4UL: // (MISC_REG_SMBIO_ENABLE_GLITCH_FILTER: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_SMBIO_ENABLE_GLITCH_FILTER (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When set enables the deglitching circuit for the SMBus inputs\n     per I2C requirement. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3a8UL: // (MISC_REG_SCPAD_EXT_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_SCPAD_EXT_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tm bits for mcp_scpad_ext memory. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3b4UL: // (MISC_REG_AEU_CLR_VPD_LATCH_SIGNAL: ACCESS:W  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_CLR_VPD_LATCH_SIGNAL (ACCESS:W  DataWidth:0x8 )" \
				"\nDescription: write to this register results with the clear of the latched\n     signals; one in d0 clears pxpv_misc_vpd0_attn; one in d1 clears pxpv_misc_vpd1_attn\n    ; one in d2 clears pxpv_misc_vpd2_attn; one in d3 clears pxpv_misc_vpd3_attn\n    ; one in d4 clears pxpv_misc_vpd4_attn; one in d5 clears pxpv_misc_vpd5_attn\n    ; one in d6 clears pxpv_misc_vpd6_attn; one in d7 clears pxpv_misc_vpd7_attn\n    ; read from this register return zero\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3b8UL: // (MISC_REG_MISC_OSC40_CONTROL: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nMISC_REG_MISC_OSC40_CONTROL (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Differential Crystal Oscillator control. [5:0] - Enabling\n     o_cmos_p[5:0]. Once the clocks are running; user can turn off unused \n    clock to save power: 0: pd: 1: enable; [6] - Output enabling for o_cmos_p\n    [5:0]: 0: depends on i_resetb &; i_osccntrl[5:0]; 1: output is active; [\n    7] - i_sel_test; Monitor XTAL CMOS output; 0: disable monitor; 1: enable\n     monitor. [8] - i_LPG_sw; Control common mode voltage of xtal core input\n     swing; 0: 600mV; 1: 650mV. [9] - i_sel_cur; Adjust CML current by loading\n     distance; 0: 22mA; 1: 12mA. [12:10] - i_d2c_bias; Adjust the d2c block\n     bias current; 000: 76 uA; 001: 240 uA; 010: 376 uA; 011: 450 uA; 100: \n    872 uA; 111:1.15 mA; (set 000 for low current mode only). [15:13] - i_bias\n    ; Adjust the bias master current; 000: 23 uA; 001: 30 uA; 010: 35 uA; \n    100: 40 uA; 111: 55 uA. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3bcUL: // (MISC_REG_PLL_MAIN_E40_CH0_MDIV: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_CH0_MDIV (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Post-divider ratio for channel-0 (divider ratio = code value\n    ). 00000000: 256; 00000001: 1; 00000010: 2; 00000011: 3; : :; 11111101\n    : 253; 11111110: 254; 11111111: 255. Global register. Reset on POR reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3c8UL: // (MISC_REG_DRIVER_CONTROL_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3d0UL: // (MISC_REG_DRIVER_CONTROL_8: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_8 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3d8UL: // (MISC_REG_DRIVER_CONTROL_9: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_9 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3e0UL: // (MISC_REG_DRIVER_CONTROL_10: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_10 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3e8UL: // (MISC_REG_DRIVER_CONTROL_11: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_11 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3f0UL: // (MISC_REG_DRIVER_CONTROL_12: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_12 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa3f8UL: // (MISC_REG_DRIVER_CONTROL_13: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_13 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa414UL: // (MISC_REG_IPOR_CMD_REG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_IPOR_CMD_REG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Writing this bit as a '1' will cause the chip to do an internal\n     reset exactly like a power-up reset. There is not protection for this\n     request and it may cause any current PCI cycle to lock up. Global register\n    . Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa424UL: // (MISC_REG_UNPREPARED: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_UNPREPARED (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set by the MCP to remember if one or more of the drivers \n    is/are loaded; 0-prepare;1-unprepare. Global register. Reset on hard reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa42cUL: // (MISC_REG_AEU_AFTER_INVERT_1_FUNC_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_1_FUNC_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read first 32 bit after inversion of function 0. mapped as\n     follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa430UL: // (MISC_REG_AEU_AFTER_INVERT_1_FUNC_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_1_FUNC_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read first 32 bit after inversion of function 1. mapped as\n     follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa434UL: // (MISC_REG_AEU_AFTER_INVERT_1_MCP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_1_MCP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read first 32 bit after inversion of mcp. mapped as follows\n    : [0] NIG attention for function0; [1] NIG attention for function1; [2\n    ] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1 function\n     1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function 1; [\n    10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event function1\n    ; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP Expansion\n     ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for mcp; [17\n    ] MSI/X indication for function 1; [18] BRB Parity error; [19] BRB Hw \n    interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] SRC Parity\n     error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] TSDM Hw interrupt\n    ; [26] TCM Parity error; [27] TCM Hw interrupt; [28] TSEMI Parity error\n    ; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31] PBF Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa438UL: // (MISC_REG_AEU_AFTER_INVERT_2_FUNC_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_2_FUNC_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read second 32 bit after inversion of function 0. mapped \n    as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] \n    QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa43cUL: // (MISC_REG_AEU_AFTER_INVERT_2_FUNC_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_2_FUNC_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read second 32 bit after inversion of function 1. mapped \n    as follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] \n    QM Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa440UL: // (MISC_REG_AEU_AFTER_INVERT_2_MCP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_2_MCP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read second 32 bit after inversion of mcp. mapped as follows\n    : [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] QM Parity \n    error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers Hw interrupt\n    ; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity error; [\n    9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt; [\n    12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    ; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa444UL: // (MISC_REG_AEU_AFTER_INVERT_3_FUNC_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_3_FUNC_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read third 32 bit after inversion of function 0. mapped as\n     follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa448UL: // (MISC_REG_AEU_AFTER_INVERT_3_FUNC_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_3_FUNC_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read third 32 bit after inversion of function 1. mapped as\n     follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa44cUL: // (MISC_REG_AEU_AFTER_INVERT_3_MCP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_3_MCP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read third 32 bit after inversion of mcp. mapped as follows\n    : [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity error\n    ; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] PXPpciClockClient\n     Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt; [8] CDU Parity\n     error; [9] CDU Hw interrupt; [10] DMAE Parity error; [11] DMAE Hw interrupt\n    ; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt; [14] MISC Parity\n     error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn; [17] Flash event\n    ; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21] SW timers attn_1\n     func0; [22] SW timers attn_2 func0; [23] SW timers attn_3 func0; [24] \n    SW timers attn_4 func0; [25] PERST; [26] SW timers attn_1 func1; [27] \n    SW timers attn_2 func1; [28] SW timers attn_3 func1; [29] SW timers attn_4\n     func1; [30] General attn0; [31] General attn1; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa450UL: // (MISC_REG_AEU_AFTER_INVERT_4_FUNC_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_4_FUNC_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read fourth 32 bit after inversion of function 0. mapped \n    as follows:  [0] General attn2; [1] General attn3; [2] General attn4; [\n    3] General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa454UL: // (MISC_REG_AEU_AFTER_INVERT_4_FUNC_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_4_FUNC_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read fourth 32 bit after inversion of function 1. mapped \n    as follows:  [0] General attn2; [1] General attn3; [2] General attn4; [\n    3] General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa458UL: // (MISC_REG_AEU_AFTER_INVERT_4_MCP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_4_MCP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: read fourth 32 bit after inversion of mcp. mapped as follows\n    :  [0] General attn2; [1] General attn3; [2] General attn4; [3] General\n     attn5; [4] General attn6; [5] General attn7; [6] General attn8; [7] General\n     attn9; [8] General attn10; [9] General attn11; [10] General attn12; [\n    11] General attn13; [12] General attn14; [13] General attn15; [14] General\n     attn16; [15] General attn17; [16] General attn18; [17] General attn19\n    ; [18] General attn20; [19] General attn21; [20] Main power interrupt; [\n    21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN Latched attn; [\n    24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC Latched timeout\n     attention; [27] GRC Latched reserved access attention; [28] MCP Latched\n     rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP Latched ump_tx_parity\n    ; [31] MCP Latched scpad_parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa45cUL: // (MISC_REG_AEU_CLR_LATCH_SIGNAL: ACCESS:W  DataWidth:0xe ) 
			sprintf(disp_str, "\nMISC_REG_AEU_CLR_LATCH_SIGNAL (ACCESS:W  DataWidth:0xe )" \
				"\nDescription: write to this register results with the clear of the latched\n     signals; one in d0 clears RBCR latch; one in d1 clears RBCT latch; one\n     in d2 clears RBCN latch; one in d3 clears RBCU latch; one in d4 clears\n     RBCP latch; one in d5 clears GRC Latched timeout attention; one in d6\n     clears GRC Latched reserved access attention; one in d7 clears Latched\n     rom_parity; one in d8 clears Latched ump_rx_parity; one in d9 clears \n    Latched ump_tx_parity; one in d10 clears Latched scpad_parity (both ports\n    ); one in d11 clears pxpv_misc_mps_attn; one in d12 clears pxp_misc_exp_rom_attn0\n    ;  one in d13 clears pxp_misc_exp_rom_attn1;  read from this register \n    return zero\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa460UL: // (MISC_REG_GENERIC_CR_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_CR_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by core reset. Global\n     register. Reset on core reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa464UL: // (MISC_REG_GENERIC_CR_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_CR_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by core reset. Global\n     register. Reset on core reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa468UL: // (MISC_REG_GENERIC_HW_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_HW_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by hard reset. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa46cUL: // (MISC_REG_GENERIC_HW_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_HW_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by hard reset. Global\n     register. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa470UL: // (MISC_REG_GENERIC_POR_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_POR_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by por reset. bit 0 is\n     used for Vmain state machine system kill reset. If clear will not reset\n     all the Vmain sm (backward compatible); if set will reset all the Vmain\n     state machine. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa474UL: // (MISC_REG_GENERIC_POR_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_POR_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by por reset. Global \n    register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa478UL: // (MISC_REG_GENERIC_RO_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_RO_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RO register reset by hard reset. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa47cUL: // (MISC_REG_GENERIC_RO_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_RO_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RO register reset by hard reset. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa480UL: // (MISC_REG_GENERIC_RO_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_RO_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RO register reset by core reset. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa484UL: // (MISC_REG_GENERIC_RO_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_RO_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RO register reset by core reset. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa488UL: // (MISC_REG_GENERIC_RO_4: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_RO_4 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RO register reset by por reset. Global \n    register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa48cUL: // (MISC_REG_GENERIC_RO_5: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GENERIC_RO_5 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RO register reset by por reset. Global \n    register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa490UL: // (MISC_REG_GPIO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GPIO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: GPIO. [31-28]  FLOAT port 0; [27-24]  FLOAT port 0; When \n    any of these bits is written as a '1'; the corresponding GPIO bit will\n     turn off it's drivers and become an input. This is the reset state of\n     all GPIO pins. The read value of these bits will be a '1' if that last\n     command (#SET; #CLR; or #FLOAT) for this bit was a #FLOAT. (reset value\n     0xff). [23-20]  CLR port 1;[19-16]  CLR port 0; When any of these bits\n     is written as a '1'; the corresponding GPIO bit will drive low. The read\n     value of these bits will be a '1' if that last command (#SET; #CLR; or\n     #FLOAT) for this bit was a #CLR. (reset value 0). [15-12]  SET port 1\n    ;[11-8] port 0; SET When any of these bits is written as a '1'; the corresponding\n     GPIO bit will drive high (if it has that capability). The read value \n    of these bits will be a '1' if that last command (#SET; #CLR; or #FLOAT\n    ) for this bit was a #SET.  (reset value 0). [7-4]  VALUE port 1; [3-0\n    ]  VALUE port 0; RO; These bits indicate the read value of each of the\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa494UL: // (MISC_REG_GPIO_INT: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GPIO_INT (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: GPIO INT. [31-28]  OLD_CLR port1; [27-24]  OLD_CLR port0; \n    Writing a '1' to these bit clears the corresponding bit in the #OLD_VALUE\n     register. This will acknowledge an interrupt on the falling edge of corresponding\n     GPIO input (reset value 0). [23-16]  OLD_SET [23-16] port1; OLD_SET port0\n    ; Writing a '1' to these bit sets the corresponding bit in the #OLD_VALUE\n     register. This will acknowledge an interrupt on the rising edge of corresponding\n     GPIO input (reset value 0). [15-12]  OLD_VALUE [11-8] port1; OLD_VALUE\n     port0; RO; These bits indicate the old value of the GPIO input value. \n    When the ~INT_STATE bit is set; this bit indicates the OLD value of the\n     pin such that if ~INT_STATE is set and this bit is '0'; then the interrupt\n     is due to a low to high edge. If ~INT_STATE is set and this bit is '1\n    '; then the interrupt is due to a high to low edge (reset value 0). [7\n    -4]  INT_STATE port1; [3-0]  INT_STATE  RO port0; These bits indicate \n    the current GPIO interrupt state for each GPIO p\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4a4UL: // (MISC_REG_MAIN_SEQ_BYP_SEL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_MAIN_SEQ_BYP_SEL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Debug only. main_sequencer_bypass select. For each bit; when\n     set; the compatible bit in the ~misc_registers_main_seq_byp_val.main_seq_byp_val\n     affects the controls; when reset; the SM affects the controls. Bit 0 - \n    Vmain OTP reset; Bit 1 - isolation_logic_b; Bit 2 - unprepared_power_down_detection\n    ; Bit 3 - PCIE_reset_b; Bit 4 - sel_vaux_b. Global register. Reset on \n    hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4a8UL: // (MISC_REG_MAIN_SEQ_BYP_VAL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_MAIN_SEQ_BYP_VAL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Debug only. main_sequencer_bypass values. For each bit; the\n     written value affects the control only if the compatible bit in the ~\n    misc_registers_main_seq_byp_sel.main_seq_byp_val is set; when reset; the\n     SM affects the controls. Bit 0 - Vmain OTP reset; Bit 1 - isolation_logic_b\n    ; Bit 2 - uprepared_power_down_detection; Bit 3 - PCIE_reset_b; Bit 4 - \n    sel_vaux_b. Global register. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4c4UL: // (MISC_REG_PLL_MAIN_CTRL_4: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_CTRL_4 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: bit0 = Controls the glitch-less mux control source: 0-management\n     power sequencer output; 1-glich-less mux manual setting (bit#1 in this\n     regitser); reset (to 0) with hard_rst_b. bit1 =Glichless mux manual setting\n     has affect when bit#0 = 1: 0-select USPLL clock; 1-select 25Mhz (ref \n    clock); Reset (to 0) with hard_rst_b. bit 2 = Analog reset toward the \n    USPLL; Controls the USPLL analog reset control source: 0-management power\n     sequencer output; 1- Analog reset manual setting (bit#3 in this regitser\n    ); reset (to 0) with hard_rst_b. bit 3 = Analog reset toward the USPLL\n    ; Controls the USPLL analog reset control source: 0-reset de-asserted; \n    1-reset assertion; reset (to 1) with hard_rst_b. E65 oly:bit 4 = Digital\n     reset toward the USPLL; Controls the USPLL digital reset control source\n    : 0-management power sequencer output; 1- Digital reset manual setting\n     (bit#5 in this regitser); reset (to 0) with hard_rst_b. bit 5 = Digital\n     reset toward the USPLL; Controls the USPLL digital reset contr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4c8UL: // (MISC_REG_PLL_MAIN_MISC_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_MISC_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: main pll lock signals. 0-unlocked; 1-locked. Global register\n    . Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4ccUL: // (MISC_REG_PLL_MAIN_OTP_CTRL_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_OTP_CTRL_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4d0UL: // (MISC_REG_PLL_MAIN_OTP_CTRL_2: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_OTP_CTRL_2 (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4d4UL: // (MISC_REG_PLL_MAIN_OTP_CTRL_3_M1DIV: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_OTP_CTRL_3_M1DIV (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4d8UL: // (MISC_REG_PLL_MAIN_OTP_CTRL_3_M2DIV: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_OTP_CTRL_3_M2DIV (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4dcUL: // (MISC_REG_PLL_MAIN_OTP_CTRL_3_N: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_OTP_CTRL_3_N (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4e0UL: // (MISC_REG_PLL_STORM_MISC_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_MISC_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: storm pll lock signals. 0-unlocked; 1-locked. Global register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4e4UL: // (MISC_REG_PWR_ATTN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PWR_ATTN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This bit indicates that a Vmain powerdown event occurred. \n    Write 0 to clear the event. Global register. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4e8UL: // (MISC_REG_RESET_CONFIG: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_RESET_CONFIG (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reset configuration register. inside order of the bits is\n    : [0-1] reserved; [2] rst_pxp_rq_rd_wr_auto_mode (0- no auto deassertion\n    ; 1 - auto deassertion);[3] rst_pxp_auto_mode (0- no auto deassertion; \n    1 - auto deassertion); [4] rst_pxpv_auto_mode (0- no auto deassertion; \n    1 - auto deassertion); [5] rst_rbcp_auto_mode (0- no auto deassertion; \n    1 - auto deassertion); [6] reserved; [7] rst_mcp_n_reset_reg_hard_core_auto_mode\n     (0- no auto deassertion; 1 - auto deassertion); [8] rst_mcp_n_hard_core_rst_b_auto_mode\n     (0- no auto deassertion; 1 - auto deassertion); [9] rst_mcp_n_reset_cmn_cpu_auto_mode\n     (0- no auto deassertion; 1 - auto deassertion); [10] rst_mcp_n_reset_cmn_core_auto_mode\n     (0- no auto deassertion; 1 - auto deassertion); [11-12] reserved; [13\n    ] rst_dbg_auto_mode (0- no auto deassertion; 1 - auto deassertion); [14\n    ] rst_misc_core_auto_mode (0- no auto deassertion; 1 - auto deassertion\n    ); [15] rst_dbue_auto_mode (0- no auto deassertion; 1 - auto deassertion\n    ); [16] grc_reset_a\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4ecUL: // (MISC_REG_SHIFT1_CMD: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SHIFT1_CMD (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 31:13 Unused  RO;12:8 SHIFT_SELECT Selects the data to be\n     loaded from the OTP into SHIFT_DATA. RW 7:4 Unused  RO 3 LOAD_DATA Load\n     a 32-bit word from the 1024-bit parallel bus to SHIFT_DATA as selected\n     by the SHIFT_SELECT bits. RW 2 SHIFT_START Shift 32-bit redundancy data\n     bits into shift chain. RW 1 SHIFT_DONE Indicate SHIFT_START has completed\n    . RW 0 RESET_MODE_N Indicate the shift controller is in RESET_MODE; 0=\n    RESET_MODE; 1=not in RESET_MODE. RO. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4f0UL: // (MISC_REG_SHIFT1_DATA: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SHIFT1_DATA (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: In RESET_MODE when SHIFT_START is asserted; this register\n     loads the 32-bit OTP data selected with SHIFT_SELECT. When written to\n     this register this loads wr_data which is used for the next shift. When\n     LOAD_DATA is asserted; this register loads the 32-bit data selected by\n     SHIFT_SELECT. When shift is requested; this data is shifted data out \n    while shifting serial data in. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4f4UL: // (MISC_REG_SHIFT2_CMD: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SHIFT2_CMD (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 31:13 Unused  RO 12:8 SHIFT_SELECT Selects the data to be\n     loaded from the OTP into SHIFT_DATA. RW 7:4 Unused  RO  3 LOAD_DATA Load\n     a 32-bit word from the 1024-bit parallel bus to SHIFT_DATA as selected\n     by the SHIFT_SELECT bits. RW 2 SHIFT_START Shift 32-bit redundancy data\n     bits into shift chain. RW 1 SHIFT_DONE Indicate SHIFT_START has completed\n    . RW 0 RESET_MODE_N Indicate the shift controller is in RESET_MODE; 0=\n    RESET_MODE; 1=not in  RESET_MODE. RO. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4f8UL: // (MISC_REG_SHIFT2_DATA: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SHIFT2_DATA (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: In RESET_MODE when SHIFT_START is asserted; this register\n     loads the 32-bit OTP data selected with SHIFT_SELECT. When written to\n     this register this loads wr_data which is used for the next shift. When\n     LOAD_DATA is asserted; this register loads the 32-bit data selected by\n     SHIFT_SELECT. When shift is requested; this data is shifted data out \n    while shifting serial data in. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4fcUL: // (MISC_REG_SPIO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SPIO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: SPIO. [31-24]  FLOAT  When any of these bits is written as\n     a '1'; the corresponding SPIO bit will turn off it's drivers and become\n     an input. This is the reset state of all SPIO pins. The read value of\n     these bits will be a '1' if that last command (#SET; #CL; or #FLOAT) \n    for this bit was a #FLOAT. (reset value 0xff). [23-16]  CLR  When any \n    of these bits is written as a '1'; the corresponding SPIO bit will drive\n     low. The read value of these bits will be a '1' if that last command (#\n    SET; #CLR; or #FLOAT) for this bit was a #CLR. (reset value 0). [15-8]  \n    SET  When any of these bits is written as a '1'; the corresponding SPIO\n     bit will drive high (if it has that capability). The read value of these\n     bits will be a '1' if that last command (#SET; #CLR; or #FLOAT) for this\n     bit was a #SET.  (reset value 0). [7-0]  VALUE  RO; These bits indicate\n     the read value of each of the eight SPIO pins. This is the result value\n     of the pin; not the drive value. Writing these bits will have not eff\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa500UL: // (MISC_REG_SPIO_INT: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SPIO_INT (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: SPIO INT. [31-24]  OLD_CLR  Writing a '1' to these bit clears\n     the corresponding bit in the #OLD_VALUE register. This will acknowledge\n     an interrupt on the falling edge of corresponding SPIO input (reset value\n     0). [23-16]  OLD_SET  Writing a '1' to these bit sets the corresponding\n     bit in the #OLD_VALUE register. This will acknowledge an interrupt on\n     the rising edge of corresponding SPIO input (reset value 0). [15-8]  \n    OLD_VALUE  RO; These bits indicate the old value of the SPIO input value\n    . When the ~INT_STATE bit is set; this bit indicates the OLD value of \n    the pin such that if ~INT_STATE is set and this bit is '0'; then the interrupt\n     is due to a low to high edge. If ~INT_STATE is set and this bit is '1\n    '; then the interrupt is due to a high to low edge (reset value 0). [7\n    -0]  INT_STATE  RO; These bits indicate the current SPIO interrupt state\n     for each SPIO pin. This bit is cleared when the appropriate #OLD_SET \n    or #OLD_CLR command bit is written. This bit is set when the SPIO \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa504UL: // (MISC_REG_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: UNUSED for E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa508UL: // (MISC_REG_SW_TIMER_EVENT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_EVENT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: the appropriate timer had reach to zero. [0] timer1; [1] \n    timer2; [2] timer3; [3] timer4; [4] timer5; [5] timer6; [6] timer7; [7\n    ] timer8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa50cUL: // (MISC_REG_VOLTAGE_REGISTER: ACCESS:RW  DataWidth:0xd ) 
			sprintf(disp_str, "\nMISC_REG_VOLTAGE_REGISTER (ACCESS:RW  DataWidth:0xd )" \
				"\nDescription: bit 0 MDIO_VOLTAGE_SEL; bit 1-4 MAIN_VREG_1_0_SEL; bit 5-\n    8 MNG_VREG_1_0_SEL; bit 9-12 VREG_2_5_SEL;ECO32: bits 1-3 and 5-7 and \n    9-11 are inverted on the output to Vreg sel (Ex: if bits 1-4 = 0000 the\n     output MAIN_VREG_1_0_SEL = 0111). Reset on POR. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa510UL: // (MISC_REG_DRIVER_CONTROL_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa518UL: // (MISC_REG_DRIVER_CONTROL_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa520UL: // (MISC_REG_DRIVER_CONTROL_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa528UL: // (MISC_REG_DRIVER_CONTROL_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa530UL: // (MISC_REG_DRIVER_CONTROL_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa538UL: // (MISC_REG_DRIVER_CONTROL_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa540UL: // (MISC_REG_FUNCTION_HIDE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_FUNCTION_HIDE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Bypass to the FUNC_HIDE pin. Bit 0 - bypass select; Bits[\n    7:1] - bypass value per function (1 - function 1; 2 -function 2; etc.). \n    When bypass select is 0 => the value is selcted depending on FUNC_HIDE\n     pin and 4 port/2 port mode; when bypass select = 1; bypass value is selected\n    . Global register. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa570UL: // (MISC_REG_SW_TIMER_EVENT_CLR: ACCESS:W  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_EVENT_CLR (ACCESS:W  DataWidth:0x8 )" \
				"\nDescription: write one for the appropriate bit will clear the appropriate\n     event to the AEU (if the attn bit (bit 2) in the ~misc_registers_sw_timer_cfg_1\n    .sw_timer_cfg_1[2]; ~misc_registers_sw_timer_cfg_2.sw_timer_cfg_2[2]; ~\n    misc_registers_sw_timer_cfg_3.sw_timer_cfg_3[2]; ~misc_registers_sw_timer_cfg_4\n    .sw_timer_cfg_4[2]; ~misc_registers_sw_timer_cfg_5.sw_timer_cfg_5[2]; ~\n    misc_registers_sw_timer_cfg_6.sw_timer_cfg_6[2]; ~misc_registers_sw_timer_cfg_7\n    .sw_timer_cfg_7[2]; ~misc_registers_sw_timer_cfg_8.sw_timer_cfg_8[2] is\n     set). [0] timer1; [1] timer2; [2] timer3; [3] timer4; [4] timer5; [5] \n    timer6; [6] timer7; [7] timer8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa580UL: // (MISC_REG_RESET_REG_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_RESET_REG_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reset reg#1;write/read one = the specific block is out of\n     reset; write/read zero = the specific block is in reset; addr 0-wr- the\n     write value will be written to the register; addr 1-set - one will be\n     written to all the bits that have the value of one in the data written\n     (bits that have the value of zero will not be change) ; addr 2-clear - \n    zero will be written to all the bits that have the value of one in the\n     data written (bits that have the value of zero will not be change); addr\n     3-ignore; read ignore from all addr except addr 00; inside order of the\n     bits is: [0] rst_brb1; [1] rst_prs; [2] rst_src; [3] rst_tsdm; [4] rst_tsem\n    ; [5] rst_tcm; [6] rst_rbcr; [7] rst_nig; [8] rst_usdm; [9] rst_ucm; [\n    10] rst_usem; [11] rst_upb; [12] rst_ccm; [13] rst_csem; [14] rst_csdm\n    ; [15] rst_rbcu; [16] rst_pbf; [17] rst_qm; [18] rst_tm; [19] rst_dorq\n    ; [20] rst_xcm; [21] rst_xsdm; [22] rst_xsem; [23] rst_rbct; [24] rst_cdu\n    ; [25] rst_cfc; [26] rst_pxp_hst; [27] rst_pxpv (global register); [28\n    ]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa590UL: // (MISC_REG_RESET_REG_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_RESET_REG_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: reset reg#2;write/read one = the specific block is out of\n     reset; write/read zero = the specific block is in reset; addr 0-wr- the\n     write value will be written to the register; addr 1-set - one will be\n     written to all the bits that have the value of one in the data written\n     (bits that have the value of zero will not be change) ; addr 2-clear - \n    zero will be written to all the bits that have the value of one in the\n     data written (bits that have the value of zero will not be change); addr\n     3-ignore; read ignore from all addr except addr 00; inside order of the\n     bits is: [0] rst_bmac0; [1] rst_bmac1; [2] rst_emac0; [3] rst_emac1; [\n    4] rst_grc(global register); [5] rst_mcp_n_reset_reg_hard_core (global\n     register); [6] rst_mcp_n_hard_core_rst_b(global register); [7] rst_mcp_n_reset_cmn_cpu\n    (global register); [8] rst_mcp_n_reset_cmn_core(global register); [9] \n    rst_rbcn; [10] rst_dbg; [11] rst_misc_core(global register); [12] rst_dbue\n     (UART)(global register); [13] Pci_resetmdio_n(global regi\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa5c0UL: // (MISC_REG_SW_TIMER_VAL: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SW_TIMER_VAL (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: the value of the counter for sw timers1-8. there are 8 addresses\n     in this register. addres 0 - timer 1; address 1 - timer 2; etc ; address\n     7 - timer 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa5e0UL: // (MISC_REG_DRIVER_CONTROL_14: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_14 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa5e8UL: // (MISC_REG_DRIVER_CONTROL_15: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_15 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa5f0UL: // (MISC_REG_DRIVER_CONTROL_16: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_DRIVER_CONTROL_16 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The following driver registers(1...16) represent 16 drivers\n     and 32 clients. Each client can be controlled by one driver only. One\n     in each bit represent that this driver control the appropriate client\n     (Ex: bit 5 is set means this driver control client number 5). addr1 = \n    set; addr0 = clear; read from both addresses will give the same result\n     = status. write to address 1 will set a request to control all the clients\n     that their appropriate bit (in the write command)  is set. if the client\n     is free (the appropriate bit in all the other drivers is clear) one will\n     be written to that driver register; if the client isn't free the bit \n    will remain zero. if the appropriate bit is set (the driver request to\n     gain control on a client it already controls the ~MISC_REGISTERS_INT_STS\n    .GENERIC_SW interrupt will be asserted). write to address 0 will set a\n     request to free all the clients that their appropriate bit (in the write\n     command) is set. if the appropriate bit is clear (the driver request \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa600UL: // (MISC_REG_AEU_SYS_KILL_STATUS_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_SYS_KILL_STATUS_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Represent the status of the input vector to the AEU when \n    a system kill occurred. The register is reset in por reset. Mapped as \n    follows: [0] NIG attention for function0; [1] NIG attention for function1\n    ; [2] GPIO1 mcp; [3] GPIO2 mcp; [4] GPIO3 mcp; [5] GPIO4 mcp; [6] GPIO1\n     function 1; [7] GPIO2 function 1; [8] GPIO3 function 1; [9] GPIO4 function\n     1; [10] PCIE glue/PXP VPD event function0; [11] PCIE glue/PXP VPD event\n     function1; [12] PCIE glue/PXP Expansion ROM event0; [13] PCIE glue/PXP\n     Expansion ROM event1; [14] SPIO4; [15] SPIO5; [16] MSI/X indication for\n     mcp; [17] MSI/X indication for function 1; [18] BRB Parity error; [19\n    ] BRB Hw interrupt; [20] PRS Parity error; [21] PRS Hw interrupt; [22] \n    SRC Parity error; [23] SRC Hw interrupt; [24] TSDM Parity error; [25] \n    TSDM Hw interrupt; [26] TCM Parity error; [27] TCM Hw interrupt; [28] \n    TSEMI Parity error; [29] TSEMI Hw interrupt; [30] PBF Parity error; [31\n    ] PBF Hw interrupt. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa604UL: // (MISC_REG_AEU_SYS_KILL_STATUS_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_SYS_KILL_STATUS_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Represent the status of the input vector to the AEU when \n    a system kill occurred. The register is reset in por reset. Mapped as \n    follows: [0] PBClient Parity error; [1] PBClient Hw interrupt; [2] QM \n    Parity error; [3] QM Hw interrupt; [4] Timers Parity error; [5] Timers\n     Hw interrupt; [6] XSDM Parity error; [7] XSDM Hw interrupt; [8] XCM Parity\n     error; [9] XCM Hw interrupt; [10] XSEMI Parity error; [11] XSEMI Hw interrupt\n    ; [12] DoorbellQ Parity error; [13] DoorbellQ Hw interrupt; [14] NIG Parity\n     error; [15] NIG Hw interrupt; [16] Vaux PCI core Parity error; [17] Vaux\n     PCI core Hw interrupt; [18] Debug Parity error; [19] Debug Hw interrupt\n    ; [20] USDM Parity error; [21] USDM Hw interrupt; [22] UCM Parity error\n    ; [23] UCM Hw interrupt; [24] USEMI Parity error; [25] USEMI Hw interrupt\n    ; [26] UPB Parity error; [27] UPB Hw interrupt; [28] CSDM Parity error\n    ; [29] CSDM Hw interrupt; [30] CCM Parity error; [31] CCM Hw interrupt\n    . Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa608UL: // (MISC_REG_AEU_SYS_KILL_STATUS_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_SYS_KILL_STATUS_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Represent the status of the input vector to the AEU when \n    a system kill occurred. The register is reset in por reset. Mapped as \n    follows: [0] CSEMI Parity error; [1] CSEMI Hw interrupt; [2] PXP Parity\n     error; [3] PXP Hw interrupt; [4] PXPpciClockClient Parity error; [5] \n    PXPpciClockClient Hw interrupt; [6] CFC Parity error; [7] CFC Hw interrupt\n    ; [8] CDU Parity error; [9] CDU Hw interrupt; [10] DMAE Parity error; [\n    11] DMAE Hw interrupt; [12] IGU (HC) Parity error; [13] IGU (HC) Hw interrupt\n    ; [14] MISC Parity error; [15] MISC Hw interrupt; [16] pxp_misc_mps_attn\n    ; [17] Flash event; [18] SMB event; [19] MCP attn0; [20] MCP attn1; [21\n    ] SW timers attn_1 func0; [22] SW timers attn_2 func0; [23] SW timers \n    attn_3 func0; [24] SW timers attn_4 func0; [25] PERST; [26] SW timers \n    attn_1 func1; [27] SW timers attn_2 func1; [28] SW timers attn_3 func1\n    ; [29] SW timers attn_4 func1; [30] General attn0; [31] General attn1. \n    Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa60cUL: // (MISC_REG_AEU_SYS_KILL_STATUS_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_SYS_KILL_STATUS_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Represent the status of the input vector to the AEU when \n    a system kill occurred. The register is reset in por reset. Mapped as \n    follows:  [0] General attn2; [1] General attn3; [2] General attn4; [3] \n    General attn5; [4] General attn6; [5] General attn7; [6] General attn8\n    ; [7] General attn9; [8] General attn10; [9] General attn11; [10] General\n     attn12; [11] General attn13; [12] General attn14; [13] General attn15\n    ; [14] General attn16; [15] General attn17; [16] General attn18; [17] \n    General attn19; [18] General attn20; [19] General attn21; [20] Main power\n     interrupt; [21] RBCR Latched attn; [22] RBCT Latched attn; [23] RBCN \n    Latched attn; [24] RBCU Latched attn; [25] RBCP Latched attn; [26] GRC\n     Latched timeout attention; [27] GRC Latched reserved access attention\n    ; [28] MCP Latched rom_parity; [29] MCP Latched ump_rx_parity; [30] MCP\n     Latched ump_tx_parity; [31] MCP Latched scpad_parity. Reset on POR reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa610UL: // (MISC_REG_AEU_SYS_KILL_OCCURRED: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_SYS_KILL_OCCURRED (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set a system kill occurred. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa614UL: // (MISC_REG_AEU_SYS_KILL_BEHAVIOR: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_SYS_KILL_BEHAVIOR (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The System Kill enable: 0 - none; 1 - hard reset. Reset on\n     POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa624UL: // (MISC_REG_CPU_OTP_CTRL1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPU_OTP_CTRL1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: [0]: start: Rising edge of the signal will execute OTP command\n    . This bit should be kept high during the execution of the command (until\n     the command_done goes high). This bit should be set to Low and high again\n     for the next command execution to start; [5:1]: Command: 0: Read; 1: \n    OTP_ProgEnable (OTP must be put in ProgEnable mode by writing 0xF; 0x4\n    ; 0x8; 0xD in sequence with OTP_ProgEnable command before you do any actual\n      write to OTP. Sequence Data is taken from bitsel bus and therefore word_address\n     and wdata do not play any role during this authentication process; 2: \n    OTP_ProgDisable (Disable OTP with this command once you are done with \n    programming); 3: Verify( vsel and tm are used from control bits); 4: Init\n     (vsel and tm are used from strap module); 5: lock_cmd. used to program\n     the lock bits that can not be programmed by using  regular program bit\n     and program Word cmd. OTP word address 6 and 7 are allocated for lock\n     bits and to program these bits lock command must be use\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa628UL: // (MISC_REG_CPU_OTP_CTRL2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPU_OTP_CTRL2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: [8:0] - wrp_tm (debug_mode): OTP timing margin control ; [\n    16:9] - wrp_vsel (debug_mode): OTP verify select level; [20:17] - wrp_cpc_sel\n     (debug_mode): Charge pump select levels that control the strength (voltage\n     and current) of the OTP internal charge pump; [24:21] - wrp_pcount (debug_mode\n    ): Program Pulse count; [25] - Wrp_sadbyp (debug_mode): SenseAmp Delay\n     bypass for OTP; [26] - Wrp_pbyp (debug_mode): OTP cell clock control \n    in debug mode; [27] - Wrp_fuselsel0: Fusesel0 in debugmode; [28] - Cpu_disbale_otp\n    : Disables any command execution through CPU interface; [29] - Unused; [\n    30] - Hw_reader_restart: Restart enable for HW OTP reader block. If it\n     is set then HW OTP reader block starts read from OTP from the beginning\n    . It is applicable only with Otp_select_en = 0; [31] - Otp_select_en: \n    Select enable to OTP block: when set then MISC access to OTP; when reset\n     then HW OTP reader block access to OTP is enabled. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa630UL: // (MISC_REG_CPU_OTP_WRITE_DATA: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPU_OTP_WRITE_DATA (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: SHARED:   Used to provide write data with burst write command\n     from CPU side.  Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa700UL: // (MISC_REG_AEU_AFTER_INVERT_5_FUNC_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_5_FUNC_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Read fifth 32 bit after inversion of function 0. Mapped as\n     follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B RBC\n     attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity; [\n    6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa704UL: // (MISC_REG_AEU_AFTER_INVERT_5_FUNC_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_5_FUNC_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Read fifth 32 bit after inversion of function 1. Mapped as\n     follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B RBC\n     attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity; [\n    6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa708UL: // (MISC_REG_AEU_AFTER_INVERT_5_MCP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_AFTER_INVERT_5_MCP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Read fifth 32 bit after inversion of mcp. Mapped as follows\n    :  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B RBC attention\n     [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity; [6] mstat0\n     attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1 parity\n    ; [31-10] Reserved;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa70cUL: // (MISC_REG_AEU_SYS_KILL_STATUS_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_AEU_SYS_KILL_STATUS_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Represent the status of the input vector to the AEU when \n    a system kill occurred. The register is reset in por reset. Mapped as \n    follows:  [0] PGLUE config_space; [1] PGLUE misc_flr; [2] PGLUE B RBC \n    attention [3] PGLUE B RBC parity; [4] ATC attention; [5] ATC parity; [\n    6] mstat0 attention; [7] mstat0 parity; [8] mstat1 attention; [9] mstat1\n     parity; [31-10] Reserved; Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa71cUL: // (MISC_REG_VOLTAGE_REG_OVWR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_VOLTAGE_REG_OVWR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: ~misc_registers_volatge_register overwrite. 00 - voltage \n    register defines the value of MDIO_VOLTAGE_SEL (path 0 and path1); 01 - \n    MDIO_VOLTAGE_SEL is forced to 0; 10 - MDIO_VOLTAGE_SEL is forced to 1. \n    Global register. Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa720UL: // (MISC_REG_PORT4MODE_EN_OVWR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_PORT4MODE_EN_OVWR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 4 port mode enable overwrite.[0] - Overwrite control; if \n    it is 0 - the port4mode_en output is equal to 4 port mode input pin; if\n     it is 1 - the port4mode_en output is equal to bit[1] of this register\n    ; [1] - Overwrite value. If bit[0] of this register is 1 this is the value\n     that receives the port4mode_en output. Reset on Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa724UL: // (MISC_REG_SWREG_MGMT_PWRDN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_SWREG_MGMT_PWRDN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: SW Regulator power down. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa728UL: // (MISC_REG_SWREG_MGMT_CTRL: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_SWREG_MGMT_CTRL (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: SW Regulator management control: [3:0] - ADJ (Softstart oscillator\n     adjustment); [7:4] - ADJ1P2 (Integrator adjustment); [11:8] - NOVL_DELAY1P2\n     (Non Overlap Delay for external switch drivers on 1.2V regulator); [13\n    :12] - OSC_FREQ (Switching oscillator frequency control); [15:14] rsvd\n    ; [17:16] - OVCUR_PROT (Over-current Protection threshold); [19:18] - \n    LDO_CTRL (LDO Control); [22:20] - RAMP1P2 (Ramp current control for 1.\n    2V regulator); [23] - rsvd3; [27:24] - SEL2P5 (selects the voltage of \n    the LDO); [31:28] - VSELECT(Voltage output selection). Reset on POR reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa72cUL: // (MISC_REG_TWO_PORT_PATH_SWAP_OVWR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_TWO_PORT_PATH_SWAP_OVWR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 2 port swap overwrite.[0] - Overwrite control; if it is 0\n     - the path_swap output is equal to 2 port mode path swap input pin; if\n     it is 1 - the path_swap output is equal to bit[1] of this register; [\n    1] - Overwrite value. If bit[0] of this register is 1 this is the value\n     that receives the path_swap output. Reset on Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa734UL: // (MISC_REG_FOUR_PORT_PORT_SWAP_OVWR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_FOUR_PORT_PORT_SWAP_OVWR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 4 port port swap overwrite.[0] - Overwrite control; if it\n     is 0 - the port_swap output is equal to 4 port mode port swap input pin\n    ; if it is 1 - the port_swap output is equal to bit[1] of this register\n    ; [1] - Overwrite value. If bit[0] of this register is 1 this is the value\n     that receives the port_swap output. Reset on Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa738UL: // (MISC_REG_FOUR_PORT_PATH_SWAP_OVWR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_FOUR_PORT_PATH_SWAP_OVWR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 4 port path swap overwrite.[0] - Overwrite control; if it\n     is 0 - the path_swap output is equal to 4 port mode path swap input pin\n    ; if it is 1 - the path_swap output is equal to bit[1] of this register\n    ; [1] - Overwrite value. If bit[0] of this register is 1 this is the value\n     that receives the path_swap output. Reset on Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa74cUL: // (MISC_REG_NIG_DBG_VECTOR: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_NIG_DBG_VECTOR (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: NIG debug mux vector control. 0 - NIG0 debug vector is output\n     to IFMUX; 1 -  NIG1 debug vector is output to IFMUX.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa774UL: // (MISC_REG_CLK_ENABLE: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nMISC_REG_CLK_ENABLE (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Clock enable. [0] - clk_nm_e0: If 1 the clock of NM engine0\n     sub-chip is enabled; if 0 the clock of  NM engine0 sub-chip is disabled\n    . [1] - clk_nm_e1: If 1 the clock of NM engine1 sub-chip is enabled; if\n     0 the clock of NM engine1 sub-chip is disabled. [2] - clk_host: If 1 \n    the clock iof HOST sub-chip is enabled; if 0 the clock of HOST sub-chip\n     is disabled. [3] - clk_e0: If 1 the clock of CPT engine0 sub-chip is \n    enabled; if 0 the clock of CPT engine0 sub-chip is disabled. [4] - clk_e1\n    : If 1 the clock of CPT engine1 sub-chip is enabled; if 0 the clock of\n     CPT engine1 sub-chip is disabled. [5] - clk_st_e0: If 1 the storm clock\n     of engine0 is enabled; if 0 the storm clock of engine0 is disabled. [\n    6] - clk_st_e1: If 1 the storm clock of engine1 is enabled; if 0 the storm\n     clock of engine1 is disabled. [7] - clk_pci_host: If 1 the PCI clock \n    of HOST sub-chip is enabled; if 0 the PCI clock of HOST sub-chip is disabled\n    . [8] - clk_pci_e0: If 1 the PCI clock of CPT engine0 sub-chip is ena\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa778UL: // (MISC_REG_FOUR_PORT_SHARED_MDIO_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_FOUR_PORT_SHARED_MDIO_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When set this will allow any of the four emacs MDIO masters\n     to initiate MDIO transactions to access XGXS0 or the four external GPHYs\n    . Drives misc_cnig_mux_4port_shared_mdio_en output. Applicable both in\n     2-port and 4-port mode. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa77cUL: // (MISC_REG_SEL_DBG_IFMUX_TEST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_SEL_DBG_IFMUX_TEST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: NIG EMAC debug source selector. If 0 - path0 gmii/mii emac\n     debug outputs are selected by NIG; If 1 - path1 gmii/mii emac debug outputs\n     are selected by NIG. Drives output misc_cnig_sel_dbg_ifmux_test.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa780UL: // (MISC_REG_EXT_PHY_MODE_STRAPS_OVWR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_EXT_PHY_MODE_STRAPS_OVWR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: PHY mode straps overwrite.[0] - Overwrite control; if it \n    is 0 - the PHY mode straps output (misc_cnig_ext_phy_mode_straps) is equal\n     to {TEST_IN[0];TEST_IN[1];EJTAG_TMS;EJTAG_TDI} input pin (ifmux_misc_ext_phy_mode_straps\n    ); if it is 1 - the PHY mode straps output (misc_cnig_ext_phy_mode_straps\n    ) is equal to bits[4:1] of this register; [4:1] - Overwrite value. If \n    bit[0] of this register is 1 this is the value that receives the PHY mode\n     straps output (misc_cnig_ext_phy_mode_straps). Global register. Reset\n     on Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa788UL: // (MISC_REG_UNPREPARED_FW: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_UNPREPARED_FW (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set by the MCP to remember if one or more of the drivers \n    is/are loaded; 0-prepare;1-unprepare. Global register. Reset on hard reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa78cUL: // (MISC_REG_UNPREPARED_DR: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nMISC_REG_UNPREPARED_DR (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Set by the Driver to remember if one or more of the drivers\n     is/are loaded; 0-prepare;1-unprepare. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7ccUL: // (MISC_REG_CPMU_LP_MCP_EARLY_EXIT_P0: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MCP_EARLY_EXIT_P0 (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: MCP (FW) Early EEE LPI Exit. When writing 1 indicates that\n     the MCP (FW) expecting activity in the near future and the SM should \n    begin exiting EEE LPI mode. Clock 25MHz.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7d0UL: // (MISC_REG_CPMU_LP_MCP_EARLY_EXIT_P1: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MCP_EARLY_EXIT_P1 (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: MCP (FW) Early EEE LPI Exit. When writing 1 indicates that\n     the MCP (FW) expecting activity in the near future and the SM should \n    begin exiting EEE LPI mode. Clock 25MHz.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7d4UL: // (MISC_REG_CPMU_LP_MCP_EARLY_EXIT_L1: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MCP_EARLY_EXIT_L1 (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: MCP (FW) Early L1 Exit. When writing 1 indicates that the\n     MCP (FW) expecting activity in the near future and the SM should begin\n     exiting L1 mode. Clock 25MHz. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7d8UL: // (MISC_REG_ALT_CLK_SELECT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_ALT_CLK_SELECT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: PCI SERDES alternate clock selector. When 0 - 250MHz. When\n     1 -25MHz. Global register. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa848UL: // (MISC_REG_MCP_ROM_TM: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_MCP_ROM_TM (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: mcp_rom_tm. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa84cUL: // (MISC_REG_CPMU_LP_FW_ENABLE_P0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_FW_ENABLE_P0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: FW EEE LPI Enable. When 1 indicates that EEE LPI mode is \n    enabled by FW. When 0 indicates that the EEE LPI mode is disabled by FW\n    . Clk 25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa850UL: // (MISC_REG_CPMU_LP_FW_ENABLE_P1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_FW_ENABLE_P1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: FW EEE LPI Enable. When 1 indicates that EEE LPI mode is \n    enabled by FW. When 0 indicates that the EEE LPI mode is disabled by FW\n    . Clk 25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa854UL: // (MISC_REG_CPMU_LP_FW_ENABLE_L1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_FW_ENABLE_L1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: FW L1 Enable. When 1 indicates that L1 mode is enabled by\n     FW. When 0 indicates that the L1 mode is disabled by FW. Clock 25MHz. \n    Global register. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa858UL: // (MISC_REG_CPMU_LP_DR_ENABLE: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_DR_ENABLE (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Driver EEE LPI Enable. When 1 indicates that EEE LPI mode\n     is enabled by driver. When 0 indicates that the EEE LPI mode is disabled\n     by driver. Clock 25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa85cUL: // (MISC_REG_CPMU_LP_DR_ENABLE_L1: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_DR_ENABLE_L1 (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Driver L1 Enable. When 1 indicates that L1 mode is enabled\n     by driver. When 0 indicates that the L1 mode is disabled by driver. Clock\n     25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa860UL: // (MISC_REG_CPMU_LP_DR_EARLY_EXIT: ACCESS:W  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_DR_EARLY_EXIT (ACCESS:W  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Driver Early EEE LPI Exit. When 1 indicates that the driver\n     expecting activity in the near future and the SM should begin exiting\n     EEE LPI mode. Clock 25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa864UL: // (MISC_REG_CPMU_LP_DR_EARLY_EXIT_L1: ACCESS:W  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_DR_EARLY_EXIT_L1 (ACCESS:W  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Driver Early L1 Exit. When 1 indicates that the driver expecting\n     activity in the near future and the SM should begin exiting L1 mode. \n    Clock 25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa868UL: // (MISC_REG_CPMU_LP_FORCE_REQ_P0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_FORCE_REQ_P0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Force EEE LPI. When 1 indicates to force EEE LPI request \n    to the MACs. When 0 the EEE LPI request is defined based on the normal\n     conditions. Clock 25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa86cUL: // (MISC_REG_CPMU_LP_FORCE_REQ_P1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_FORCE_REQ_P1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Force EEE LPI. When 1 indicates to force EEE LPI request \n    to the MACs. When 0 the EEE LPI request is defined based on the normal\n     conditions. Clock 25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa870UL: // (MISC_REG_CPMU_LP_FORCE_REQ_L1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_FORCE_REQ_L1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Force L1. When 1 indicates to force L1 request to the MACs\n    . When 0 the L1 request is defined based on the normal conditions. Clock\n     25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa874UL: // (MISC_REG_CPMU_LP_OVERRIDE_SM_P0: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_OVERRIDE_SM_P0 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: EEE LPI Override. [0] - EEE LPI Override Control. When 1 \n    the SM EEE LPI request is overrode by bit 1. When 0 the SM EEE LPI request\n     is driven by the SM. [1] - EEE LPI Override Value. Indicates the value\n     of EEE LPI request when bit 0 is asserted. Clock 25MHz. Reset on hard\n     reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa878UL: // (MISC_REG_CPMU_LP_OVERRIDE_SM_P1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_OVERRIDE_SM_P1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: EEE LPI Override. [0] - EEE LPI Override Control. When 1 \n    the SM EEE LPI request is overrode by bit 1. When 0 the SM EEE LPI request\n     is driven by the SM. [1] - EEE LPI Override Value. Indicates the value\n     of EEE LPI request when bit 0 is asserted. Clock 25MHz. Reset on hard\n     reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa87cUL: // (MISC_REG_CPMU_LP_OVERRIDE_SM_L1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_OVERRIDE_SM_L1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: L1 Override. [0] - L1 Early Exit Override Control. When =\n    1 the SM L1 Early Exit request is overrode by bit 1.When =0 the SM L1 \n    early Exit request is driven by the SM or by the Force bit. [1] - L1 Early\n     Exit Override Value. Indicates the value of L1 Early Exit request when\n     bit 0 is asserted: 1 indicates low power state - L1 early request is \n    driven to 0 (indication to the PCIE CORE that it is ok to go to L1 ASPM\n    ). 0 indicates normal power state - L1 early request is driven to 1 (indication\n     to the PCIE CORE to get out L1 ASPM). Clock 25MHz. Global register. Reset\n     on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa880UL: // (MISC_REG_CPMU_LP_MASK_ENT_P0: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MASK_ENT_P0 (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: LPI entry events mask. [0] - Vmain SM Mask. When 1 indicates\n     that the Vmain SM end state is disabled. When 0 indicates that the Vmain\n     SM end state is enabled. [1] - FW Queues Empty Mask. When 1 indicates\n     that the FW command that all Queues are empty is disabled. When 0 indicates\n     that the FW command that all Queues are empty is enabled. [2] - FW Early\n     Exit Mask / Reserved (Entry mask). When 1 indicates that the FW Early\n     Exit command is disabled. When 0 indicates that the FW Early Exit command\n     is enabled. This bit applicable only in the EXIT Events Mask registers\n    . [3] - PBF Request Mask. When 1 indicates that the PBF Request indication\n     is disabled. When 0 indicates that the PBF Request indication is enabled\n    . [4] - Tx Request Mask. When =1 indicates that the Tx other Than PBF \n    Request indication is disabled. When 0 indicates that the Tx Other Than\n     PBF Request indication is enabled. [5] - Rx EEE LPI Status Mask. When\n     1 indicates that the RX EEE LPI Status indication is disabled\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa884UL: // (MISC_REG_CPMU_LP_MASK_ENT_P1: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MASK_ENT_P1 (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: EEE LPI entry events mask. [0] - Vmain SM Mask. When 1 indicates\n     that the Vmain SM end state is disabled. When 0 indicates that the Vmain\n     SM end state is enabled. [1] - FW Queues Empty Mask. When 1 indicates\n     that the FW command that all Queues are empty is disabled. When 0 indicates\n     that the FW command that all Queues are empty is enabled. [2] - FW Early\n     Exit Mask / Reserved (Entry mask). When 1 indicates that the FW Early\n     Exit command is disabled. When 0 indicates that the FW Early Exit command\n     is enabled. This bit applicable only in the EXIT Events Mask registers\n    . [3] - PBF Request Mask. When 1 indicates that the PBF Request indication\n     is disabled. When 0 indicates that the PBF Request indication is enabled\n    . [4] - Tx Request Mask. When =1 indicates that the Tx other Than PBF \n    Request indication is disabled. When 0 indicates that the Tx Other Than\n     PBF Request indication is enabled. [5] - Rx EEE LPI Status Mask. When\n     1 indicates that the RX EEE LPI Status indication is disa\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa888UL: // (MISC_REG_CPMU_LP_MASK_EXT_P0: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MASK_EXT_P0 (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: EEE LPI exit events mask. [0] - Vmain SM Mask. When 1 indicates\n     that the Vmain SM end state is disabled. When 0 indicates that the Vmain\n     SM end state is enabled. [1] - FW Queues Empty Mask. When 1 indicates\n     that the FW command that all Queues are empty is disabled. When 0 indicates\n     that the FW command that all Queues are empty is enabled. [2] - FW Early\n     Exit Mask / Reserved (Entry mask). When 1 indicates that the FW Early\n     Exit command is disabled. When 0 indicates that the FW Early Exit command\n     is enabled. This bit applicable only in the EXIT Events Mask registers\n    . [3] - PBF Request Mask. When 1 indicates that the PBF Request indication\n     is disabled. When 0 indicates that the PBF Request indication is enabled\n    . [4] - Tx Request Mask. When =1 indicates that the Tx other Than PBF \n    Request indication is disabled. When 0 indicates that the Tx Other Than\n     PBF Request indication is enabled. [5] - Rx EEE LPI Status Mask. When\n     1 indicates that the RX EEE LPI Status indication is disab\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa88cUL: // (MISC_REG_CPMU_LP_MASK_EXT_P1: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MASK_EXT_P1 (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: EEE LPI exit events mask. [0] - Vmain SM Mask. When 1 indicates\n     that the Vmain SM end state is disabled. When 0 indicates that the Vmain\n     SM end state is enabled. [1] - FW Queues Empty Mask. When 1 indicates\n     that the FW command that all Queues are empty is disabled. When 0 indicates\n     that the FW command that all Queues are empty is enabled. [2] - FW Early\n     Exit Mask / Reserved (Entry mask). When 1 indicates that the FW Early\n     Exit command is disabled. When 0 indicates that the FW Early Exit command\n     is enabled. This bit applicable only in the EXIT Events Mask registers\n    . [3] - PBF Request Mask. When 1 indicates that the PBF Request indication\n     is disabled. When 0 indicates that the PBF Request indication is enabled\n    . [4] - Tx Request Mask. When =1 indicates that the Tx other Than PBF \n    Request indication is disabled. When 0 indicates that the Tx Other Than\n     PBF Request indication is enabled. [5] - Rx EEE LPI Status Mask. When\n     1 indicates that the RX EEE LPI Status indication is disab\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa890UL: // (MISC_REG_CPMU_LP_MASK_ENT_P0_L1: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MASK_ENT_P0_L1 (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: L1 entry events mask. [0] - Vmain SM Mask. When 1 indicates\n     that the Vmain SM end state is disabled. When 0 indicates that the Vmain\n     SM end state is enabled. [1] - FW Queues Empty Mask. When 1 indicates\n     that the FW command that all Queues are empty is disabled. When 0 indicates\n     that the FW command that all Queues are empty is enabled. [2] - FW Early\n     Exit Mask / Reserved (Entry mask). When 1 indicates that the FW Early\n     Exit command is disabled. When 0 indicates that the FW Early Exit command\n     is enabled. This bit applicable only in the EXIT Events Mask registers\n    . [3] - PBF Request Mask. When 1 indicates that the PBF Request indication\n     is disabled. When 0 indicates that the PBF Request indication is enabled\n    . [4] - Tx Request Mask. When =1 indicates that the Tx other Than PBF \n    Request indication is disabled. When 0 indicates that the Tx Other Than\n     PBF Request indication is enabled. [5] - Rx EEE LPI Status Mask. When\n     1 indicates that the RX EEE LPI Status indication is disabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa894UL: // (MISC_REG_CPMU_LP_MASK_ENT_P1_L1: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MASK_ENT_P1_L1 (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: L1 entry events mask. [0] - Vmain SM Mask. When 1 indicates\n     that the Vmain SM end state is disabled. When 0 indicates that the Vmain\n     SM end state is enabled. [1] - FW Queues Empty Mask. When 1 indicates\n     that the FW command that all Queues are empty is disabled. When 0 indicates\n     that the FW command that all Queues are empty is enabled. [2] - FW Early\n     Exit Mask / Reserved (Entry mask). When 1 indicates that the FW Early\n     Exit command is disabled. When 0 indicates that the FW Early Exit command\n     is enabled. This bit applicable only in the EXIT Events Mask registers\n    . [3] - PBF Request Mask. When 1 indicates that the PBF Request indication\n     is disabled. When 0 indicates that the PBF Request indication is enabled\n    . [4] - Tx Request Mask. When =1 indicates that the Tx other Than PBF \n    Request indication is disabled. When 0 indicates that the Tx Other Than\n     PBF Request indication is enabled. [5] - Rx EEE LPI Status Mask. When\n     1 indicates that the RX EEE LPI Status indication is disabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa898UL: // (MISC_REG_CPMU_LP_MASK_EXT_P0_L1: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MASK_EXT_P0_L1 (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: L1 exit events mask. [0] - Vmain SM Mask. When 1 indicates\n     that the Vmain SM end state is disabled. When 0 indicates that the Vmain\n     SM end state is enabled. [1] - FW Queues Empty Mask. When 1 indicates\n     that the FW command that all Queues are empty is disabled. When 0 indicates\n     that the FW command that all Queues are empty is enabled. [2] - FW Early\n     Exit Mask / Reserved (Entry mask). When 1 indicates that the FW Early\n     Exit command is disabled. When 0 indicates that the FW Early Exit command\n     is enabled. This bit applicable only in the EXIT Events Mask registers\n    . [3] - PBF Request Mask. When 1 indicates that the PBF Request indication\n     is disabled. When 0 indicates that the PBF Request indication is enabled\n    . [4] - Tx Request Mask. When =1 indicates that the Tx other Than PBF \n    Request indication is disabled. When 0 indicates that the Tx Other Than\n     PBF Request indication is enabled. [5] - Rx EEE LPI Status Mask. When\n     1 indicates that the RX EEE LPI Status indication is disabled. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa89cUL: // (MISC_REG_CPMU_LP_MASK_EXT_P1_L1: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_MASK_EXT_P1_L1 (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: L1 exit events mask. [0] - Vmain SM Mask. When 1 indicates\n     that the Vmain SM end state is disabled. When 0 indicates that the Vmain\n     SM end state is enabled. [1] - FW Queues Empty Mask. When 1 indicates\n     that the FW command that all Queues are empty is disabled. When 0 indicates\n     that the FW command that all Queues are empty is enabled. [2] - FW Early\n     Exit Mask / Reserved (Entry mask). When 1 indicates that the FW Early\n     Exit command is disabled. When 0 indicates that the FW Early Exit command\n     is enabled. This bit applicable only in the EXIT Events Mask registers\n    . [3] - PBF Request Mask. When 1 indicates that the PBF Request indication\n     is disabled. When 0 indicates that the PBF Request indication is enabled\n    . [4] - Tx Request Mask. When =1 indicates that the Tx other Than PBF \n    Request indication is disabled. When 0 indicates that the Tx Other Than\n     PBF Request indication is enabled. [5] - Rx EEE LPI Status Mask. When\n     1 indicates that the RX EEE LPI Status indication is disabled. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8a0UL: // (MISC_REG_CPMU_LP_IDLE_THR_P0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_IDLE_THR_P0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: EEE LPI Idle Threshold. The threshold value for the idle \n    EEE LPI counter. Timer tick is 1 us. Clock 25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8a4UL: // (MISC_REG_CPMU_LP_IDLE_THR_P1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_IDLE_THR_P1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: EEE LPI Idle Threshold. The threshold value for the idle \n    EEE LPI counter. Timer tick is 1 us. Clock 25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8a8UL: // (MISC_REG_CPMU_LP_IDLE_THR_L1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_IDLE_THR_L1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: L1 Idle Threshold. The threshold value for the idle L1 counter\n    . Timer tick is 1 us. Clock 25MHz. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8acUL: // (MISC_REG_CPMU_LP_SM_ST_STAT_P0: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_SM_ST_STAT_P0 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: EEE LPI status. [0] - EEE LPI SM Request Status. When 0 the\n     SM is not in EEE LPI state. When 1 the SM is in EEE LPI state. [1] - \n    EEE LPI Control Request Status. This is the status of the control which\n     is function of the EEE LPI SM; EEE LPI Override register and the EEE \n    LPI Force register. When 0 the Control is not in EEE LPI state. When 1\n     the Control is in EEE LPI state. Clock 25MHz. Read only register. Reset\n     on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8b0UL: // (MISC_REG_CPMU_LP_SM_ST_STAT_P1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_SM_ST_STAT_P1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: EEE LPI status. [0] - EEE LPI SM Request Status. When 0 the\n     SM is not in EEE LPI state. When 1 the SM is in EEE LPI state. [1] - \n    EEE LPI Control Request Status. This is the status of the control which\n     is function of the EEE LPI SM; EEE LPI Override register and the EEE \n    LPI Force register. When 0 the Control is not in EEE LPI state. When 1\n     the Control is in EEE LPI state. Clock 25MHz. Read only register. Reset\n     on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8b4UL: // (MISC_REG_CPMU_LP_SM_ST_STAT_L1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_SM_ST_STAT_L1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: L1 status. [0] - L1 SM Request Status. When 0 the SM is not\n     in EEE LPI state. When 1 the SM is in L1 state. [1] - EEE LPI Control\n     Request Status. This is the status of the control which is function of\n     the L1 SM; L1 Override register and the L1 Force register. When 0 the\n     Control is not in L1 state. When 1 the Control is in L1 state. Clock \n    25MHz. Read only register. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8b8UL: // (MISC_REG_CPMU_LP_SM_ENT_CNT_P0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_SM_ENT_CNT_P0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: EEE LPI Entry Events Counter. A statistic counter with the\n     number of counts that the SM entered the EEE LPI state. Clock 25MHz. \n    Read only register. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8bcUL: // (MISC_REG_CPMU_LP_SM_ENT_CNT_P1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_SM_ENT_CNT_P1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: EEE LPI Entry Events Counter. A statistic counter with the\n     number of counts that the SM entered the EEE LPI state. Clock 25MHz. \n    Read only register. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8c0UL: // (MISC_REG_CPMU_LP_SM_ENT_CNT_L1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_SM_ENT_CNT_L1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: L1 Entry Events Counter. A statistic counter with the number\n     of counts that the SM entered the L1 state. Clock 25MHz. Read only register\n    . Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8c4UL: // (MISC_REG_ERSTCLK_CLKSWITCH_BYPASS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_ERSTCLK_CLKSWITCH_BYPASS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: eclk switches bypass. When =0 the eclk clkswitchs are functional\n    . When =1 the eclk clkswitchs are bypassed. Bit[0] - port 0 path0;Bit[\n    1] - port 0 path1;Bit[2] - port 1 path0;Bit[3] - port 1 path1. Global \n    register. Reset on PLL reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8c8UL: // (MISC_REG_CPMU_LP_OVERRIDE_SM_LTR2_L1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_LP_OVERRIDE_SM_LTR2_L1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: L1 LTR2 Override. [0] - L1 LTR2 Override Control. When =1\n     the SM LTR2 request is overriden by bit[1].When =0 the SM LTR2 request\n     is driven by the SM or by the Force bit. [1] - LTR Override Value. Indicates\n     the value of the LTR2 request when bit 0 is asserted: 1 indicates low\n     power state - LTR2 request is driven to 1 (indication to the CPU (through\n     PCIE CORE) that it is ok to go to low power state). 0 indicates normal\n     power state - LTR2 request is driven to 0 (indication to the CPU (through\n     PCIE CORE) to come out of low power state). Clock 25MHz. Global register\n    . Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8ccUL: // (MISC_REG_VTMON_CTRL: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nMISC_REG_VTMON_CTRL (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Volatge/Temperature Monitor control. [15:7] - reserved. [\n    6:3] - 1111: Output code =1023; 1110: Output code =960; 1101: Output code\n     =896; 1100: Output code =832; 1011: Output code =768; 1010: Output code\n     =704; 1001: Output code =640; 1000: Output code =576; 0111: Output code\n     =512; 0110: Output code =448; 0101: Output code =384; 0100: Output code\n     =320; 0011: Output code =256; 0010: Output code =192; 0001: Output code\n     =128; 0000: Output code =64. [2:0] - 111: 1.232 V; 110: 1.226 V; 101: \n    1.220 V; 100: 1.214 V; 011: 1.208 V; 010: 1.202 V; 001: 1.196 V; 000: \n    1.190 V. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8d0UL: // (MISC_REG_VTMON_DATA: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_VTMON_DATA (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: Voltage/Temperature Monitor output.Global register. Reset\n     on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8e4UL: // (MISC_REG_VTMON_HOLD: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_VTMON_HOLD (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Voltage/Temperature Monitor hold. 0 - update; 1 - hold on\n     to the value forever. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8e8UL: // (MISC_REG_CPMU_CONN_CNT_P0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_CONN_CNT_P0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Number of Connections. Contains the value of the counter \n    that counts the number of connections.The read order is obligatory: first\n     address 0 read; then address 1 read. When address is 0 the 32 lsbits \n    is read; latch the upper 8 msbits. When the address is 1 the 8 msbits \n    is read; the read is done from the latched register. Read only register\n    . Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8f0UL: // (MISC_REG_CPMU_CONN_CNT_P1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPMU_CONN_CNT_P1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Number of Connections. Contains the value of the counter \n    that counts the number of connections. The read order is obligatory: first\n     address 0 read; then address 1 read. When address is 0 the 32 lsbits \n    is read; latch the upper 8 msbits. When the address is 1 the 8 msbits \n    is read; the read is done from the latched register. Read only register\n    . Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8f8UL: // (MISC_REG_VTMON_PWR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_VTMON_PWR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Voltage/Temperature Monitor power control. [0] - Voltage/\n    Temperature monitor reset (0 - asserted; 1 - de-asserted); [1] - Voltage\n    /Temperature monitor power down (0 - power down; 1 - power up). Global\n     register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa8fcUL: // (MISC_REG_VTMON_SEL: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_VTMON_SEL (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Voltage/Temperature Monitor select. [2:0] - 000: Temperature\n     Monitor Enable; 001: Voltage Monitor input; i_VTMON_0p9V; 010: Voltage\n     Monitor input; i_VTMON_2V:0; 011: Voltage Monitor input; i_VTMON_2V:1\n    ; 100: Voltage Monitor input; i_VTMON_2V:2; 101: Voltage Monitor input\n    ; i_VTMON_4V:0; 110: Voltage Monitor input; i_VTMON_4V:1; 111: Test Mode\n    . Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa900UL: // (MISC_REG_PLL_MAIN_E40_CTRL_1: ACCESS:RW  DataWidth:0x1e ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_CTRL_1 (ACCESS:RW  DataWidth:0x1e )" \
				"\nDescription: PLL configuration control register. [11:0]: dco_ctrl_bypass\n    [11:0]: direct programming of DAC: 000000000000 = MIN VCO clock frequency\n    ; : ; 111111111111 = MAX VCO clock frequency; [12]: dco_ctrl_bypass_enable\n    : enable of direct programming of DAC: 0 =normal mode; 1 =DAC programming\n     mode; [13]: stat_reset: reset of phase error measurement: 0 =normal mode\n    ; 1 =reset; [16:14]: stat_select[2:0]: select of test output: 000 = 000000000000\n    ; 001 = minimum phase error; 010 = maximum phase error; 011 = mean-square\n     phase error; 100 = dac control word; 101 = 000000000000; 110 = 000000000000\n    ; 111 = 000000000000; [17]: stat_update: On the synchronized rising edge\n     of this control signal the value selected by stat_select[2:0] is clocked\n     into o_statout; [18]: refclkout: Enable for o_fref output: 0 = o_fref\n     at logic 0; 1 = o_fref enabled (pre-divider output clock); [19]: AUX_CTRL\n    : ICO current: 0 = normal mode; 1 = Ivco increased by 130 ?A; [20]: VCOdiv2_Post\n    : Enables divide-by-2 for the clock t\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa904UL: // (MISC_REG_PLL_MAIN_E40_CTRL_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_CTRL_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Configuration control register. [2:0]: i_ka[2:0]: Loop gain\n     in frequency acquisition mode; [5:3]: i_ki[2:0]: Gain of P/I loop filter\n     integrator path during fine phase acquisition mode; [9:6]: i_kp[3:0]: \n    Gain of P/I loop filter proportional path during fine phase acquisition\n     mode. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa908UL: // (MISC_REG_PLL_MAIN_E40_NDIV_FRAC: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_NDIV_FRAC (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Fractional feedback divider control. Resolution= 1/(2^20). \n    Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa90cUL: // (MISC_REG_PLL_MAIN_E40_NDIV_INT: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_NDIV_INT (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Feedback divider control (Code = divider ratio). 0000000000\n    : divide-by-1024; 0000000001: XXX; 0000000010: XXX; 0000000111: XXX; 0000001000\n    : divide-by-8; 0000001001: divide-by-9; 0000001010: divide-by-10; 1111111110\n    : divide-by-1022; 1111111111: divide-by-1023. Global register. Reset on\n     POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa910UL: // (MISC_REG_PLL_MAIN_E40_PDIV: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_PDIV (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Input reference clock pre-divider control (code = divider\n     ratio). 000: divide-by-8; 001: divide-by-1; 010: divide-by-2; 011: divide\n    -by-3; 100: divide-by-4; 101: divide-by-5; 110: divide-by-6; 111: divide\n    -by-7. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa914UL: // (MISC_REG_PLL_MAIN_E40_MISC_STATUS: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_MISC_STATUS (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: STATUS (PLL test) output BUS. Global register. Reset on POR\n     reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa918UL: // (MISC_REG_PLL_STORM_E40_MISC_STATUS: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E40_MISC_STATUS (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: STATUS (PLL test) output BUS. Global register. Reset on POR\n     reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa91cUL: // (MISC_REG_PLL_STORM_E40_CH0_MDIV: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E40_CH0_MDIV (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Post-divider ratio for channel-0 (divider ratio = code value\n    ). 00000000: 256; 00000001: 1; 00000010: 2; 00000011: 3; : :; 11111101\n    : 253; 11111110: 254; 11111111: 255. Global register. Reset on POR reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa920UL: // (MISC_REG_PLL_STORM_E40_CTRL_1: ACCESS:RW  DataWidth:0x1e ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E40_CTRL_1 (ACCESS:RW  DataWidth:0x1e )" \
				"\nDescription: PLL configuration control register. [11:0]: dco_ctrl_bypass\n    [11:0]: direct programming of DAC: 000000000000 = MIN VCO clock frequency\n    ; : ; 111111111111 = MAX VCO clock frequency; [12]: dco_ctrl_bypass_enable\n    : enable of direct programming of DAC: 0 =normal mode; 1 =DAC programming\n     mode; [13]: stat_reset: reset of phase error measurement: 0 =normal mode\n    ; 1 =reset; [16:14]: stat_select[2:0]: select of test output: 000 = 000000000000\n    ; 001 = minimum phase error; 010 = maximum phase error; 011 = mean-square\n     phase error; 100 = dac control word; 101 = 000000000000; 110 = 000000000000\n    ; 111 = 000000000000; [17]: stat_update: On the synchronized rising edge\n     of this control signal the value selected by stat_select[2:0] is clocked\n     into o_statout; [18]: refclkout: Enable for o_fref output: 0 = o_fref\n     at logic 0; 1 = o_fref enabled (pre-divider output clock); [19]: AUX_CTRL\n    : ICO current: 0 = normal mode; 1 = Ivco increased by 130 ?A; [20]: VCOdiv2_Post\n    : Enables divide-by-2 for the clock t\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa924UL: // (MISC_REG_PLL_STORM_E40_CTRL_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E40_CTRL_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Configuration control register. [2:0]: i_ka[2:0]: Loop gain\n     in frequency acquisition mode; [5:3]: i_ki[2:0]: Gain of P/I loop filter\n     integrator path during fine phase acquisition mode; [9:6]: i_kp[3:0]: \n    Gain of P/I loop filter proportional path during fine phase acquisition\n     mode. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa928UL: // (MISC_REG_PLL_STORM_E40_NDIV_FRAC: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E40_NDIV_FRAC (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Fractional feedback divider control. Resolution= 1/(2^20). \n    Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa92cUL: // (MISC_REG_PLL_STORM_E40_NDIV_INT: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E40_NDIV_INT (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Feedback divider control (Code = divider ratio). 0000000000\n    : divide-by-1024; 0000000001: XXX; 0000000010: XXX; 0000000111: XXX; 0000001000\n    : divide-by-8; 0000001001: divide-by-9; 0000001010: divide-by-10; 1111111110\n    : divide-by-1022; 1111111111: divide-by-1023. Global register. Reset on\n     POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa930UL: // (MISC_REG_PLL_STORM_E40_PDIV: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E40_PDIV (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Input reference clock pre-divider control (code = divider\n     ratio). 000: divide-by-8; 001: divide-by-1; 010: divide-by-2; 011: divide\n    -by-3; 100: divide-by-4; 101: divide-by-5; 110: divide-by-6; 111: divide\n    -by-7. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa934UL: // (MISC_REG_PWRWDOG_E40_ACCU_RUN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_E40_ACCU_RUN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Power watchdog. When 0 all registers and states are of power\n     watchdog accu sub-module are reset. Global register. Reset on POR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa938UL: // (MISC_REG_PWRWDOG_E40_ACCU_SKIP: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_E40_ACCU_SKIP (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Power watchdog. Allows to decimate the good signal to measure\n     for beats. Global register. Reset on POR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa93cUL: // (MISC_REG_PWRWDOG_E40_ACCU_SKIP_START: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_E40_ACCU_SKIP_START (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Power watchdog. Start value of skip counter. Global register\n    . Reset on POR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa940UL: // (MISC_REG_PWRWDOG_E40_CGFG: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_E40_CGFG (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Power watchdog configuration. Adjustment of integer delay\n     cgfg[7:4] and fractional delay cgfg[3:0]; valid range [0:191]. Global\n     register. Reset on POR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa944UL: // (MISC_REG_PWRWDOG_E40_CLEARCFG: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_E40_CLEARCFG (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Power watchdog selection of delay chain for the reset pulse\n    ; recommended setting is 3b100 for longest pulse; 3b010 will shorten the\n     reset pulse by one inverter+inverting mux delay; 3b001 will shorten the\n     reset pulse by two inverter+inverting mux delays; set clearcfg more or\n     equal than rsel to ensure the ring gets flushed from the previous cycle\n    ; reducing the reset pulse length is needed to reach the highest possible\n     operating frequencies.Global register. Reset on POR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa948UL: // (MISC_REG_PWRWDOG_E40_DATA: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_E40_DATA (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: Power watchdog data.Global register. Reset on POR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa94cUL: // (MISC_REG_PWRWDOG_E40_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_E40_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Power watchdog done. Global register. Reset on POR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa950UL: // (MISC_REG_PWRWDOG_E40_PWRDN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_E40_PWRDN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Power watchdog power down. 1 - power down; 0 - power up.Global\n     register.  Reset on POR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa954UL: // (MISC_REG_PWRWDOG_E40_RSEL: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_E40_RSEL (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Power watchdog selection of delay setting of ring delay element\n    ; recommended setting is 3b100 for longest ring delay; 3b010 will shorten\n     the ring by one inverter+inverting mux; 3b001 will shorten the ring by\n     two inverter+inverting mux cells;Global register. Reset on POR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa958UL: // (MISC_REG_PWRWDOG_E40_START: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_E40_START (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Power watchdog. Sets the number of trips through the ring\n     delay element to start+1; range for start is [-1:31].Global register. \n    Reset on POR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa95cUL: // (MISC_REG_CPU_OTP_CTRL3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_CPU_OTP_CTRL3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: CPU OTP Control Register. [10:0]: Otp_rom_address: OTP ROM\n     Address; [30:11]: reserved; [31]: Jtag_cpu_mode: This bit must be set\n     to enable access to the OTP ROM interface via GRC registers. If this \n    bit is cleared; access is allowed only through the JTAG interface. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa960UL: // (MISC_REG_XMAC_PHY_PORT_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_XMAC_PHY_PORT_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: XMAC PHY port mode. Indicates the number of ports on the \n    Warp Core. This is a strap input for the XMAC_MP core. 00 - Single Port\n     Mode; 01 - Dual Port Mode; 1x - Quad Port Mode; This is a strap input\n     for the XMAC_MP core; and should be changed only while reset is held \n    low. Reset on Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa964UL: // (MISC_REG_XMAC_CORE_PORT_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_XMAC_CORE_PORT_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: XMAC Core port mode. Indicates the number of ports on the\n     system side. This should be less than or equal to phy_port_mode; if some\n     of the ports are not used. This enables reduction of frequency on the\n     core side. This is a strap input for the XMAC_MP core. 00 - Single Port\n     Mode; 01 - Dual Port Mode; 10 - Tri Port Mode; 11 - Quad Port Mode. This\n     is a strap input for the XMAC_MP core; and should be changed only while\n     reset is held low. Reset on Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa968UL: // (MISC_REG_UMAC0_LPI_TX_DETECT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_UMAC0_LPI_TX_DETECT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: UMAC0 transmit LPI (Low Power Indication) state. This per\n     port signal is set to 1; whenever the transmit interface is in LPI (Low\n     Power Indication) state and LPI_IDLES are being sent out on the TX Warpcore\n     interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa96cUL: // (MISC_REG_UMAC1_LPI_TX_DETECT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_UMAC1_LPI_TX_DETECT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: UMAC1 transmit LPI (Low Power Indication) state. This per\n     port signal is set to 1; whenever the transmit interface is in LPI (Low\n     Power Indication) state and LPI_IDLES are being sent out on the TX Warpcore\n     interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa978UL: // (MISC_REG_PORT_LANE_MODE_OVWR: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PORT_LANE_MODE_OVWR (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Swap mode. This control is provided by the MISC block through\n     straps and can also changed by F/W override. It is used to select the\n     basic swap modes. The following table defines the swap modes. [0] - Overwrite\n     control; if it is 0 - the swap_mode output is equal to ifmux_swap_mode\n     input pin; if it is 1 - the swap_mode output is equal to bit[2:1] of \n    this register; [2:1] - Overwrite value. If bit[0] of this register is \n    1 this is the value that receives the swap_mode output. These two bits\n     are: 00 - generic; 01 - special mode 0; 10 - special mode 1; 11 - rsvd\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa97cUL: // (MISC_REG_GEN_PURP_CR0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_CR0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by core reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa980UL: // (MISC_REG_GEN_PURP_CR1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_CR1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by core reset. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa984UL: // (MISC_REG_GEN_PURP_CR2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_CR2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by core reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa988UL: // (MISC_REG_GEN_PURP_CR3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_CR3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by core reset. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa98cUL: // (MISC_REG_GEN_PURP_CRG: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_CRG (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by core reset. Bit[0]: \n    used for VCCMIN control to select 25MHz clock on XMAC; UMAC and PCIE Serdes\n    . Global register. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa990UL: // (MISC_REG_GEN_PURP_HW0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_HW0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by hard reset. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa994UL: // (MISC_REG_GEN_PURP_HW1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_HW1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by hard reset. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa998UL: // (MISC_REG_GEN_PURP_HW2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_HW2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by hard reset. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa99cUL: // (MISC_REG_GEN_PURP_HW3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_HW3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by hard reset. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9a0UL: // (MISC_REG_GEN_PURP_HWG: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_HWG (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Bit[0]: EPIO MODE SEL: Setting this bit to 1 will allow SW\n    /FW to use all of the 32 Extended GPIO pins. Without setting this bit; \n    an EPIO can not be configured as an output. Each output has its output\n     enable in the MCP register space; but this bit needs to be set to make\n     use of that. Bit[3:1] spare. Bit[4]: WCVTMON_PWRDN: Powerdown for Warpcore\n     VTMON. When set to 1 - Powerdown. Bit[5]: WCVTMON_RESETB: Reset for Warpcore\n     VTMON. When set to 0 -  vTMON is in reset. Bit[6]: setting this bit will\n     change the i/o to an output and will drive the TimeSync output. Bit[31\n    :7]: spare. Global register. Reset by hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9a4UL: // (MISC_REG_GEN_PURP_POR0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_POR0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by por reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9a8UL: // (MISC_REG_GEN_PURP_POR1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_POR1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by por reset. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9acUL: // (MISC_REG_GEN_PURP_POR2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_POR2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by por reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9b0UL: // (MISC_REG_GEN_PURP_POR3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_POR3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by por reset. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9b4UL: // (MISC_REG_GEN_PURP_PORG: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_GEN_PURP_PORG (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only: spare RW register reset by por reset. Global \n    register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9b8UL: // (MISC_REG_MDIO_OVERRIDE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_MDIO_OVERRIDE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: MDIO Override. Enables the values on ~misc_registers_mdio_subscription\n    .mdio_subscription to override the hardware mode defined defaults. Global\n     register. Reset on Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9bcUL: // (MISC_REG_MDIO_SUBSCRIPTION: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_MDIO_SUBSCRIPTION (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: MDIO Subscription. Is used to configure the subscriptions\n     of on-chip PHY devices and MAC ports to the four MDIO domains. It is \n    only used when ~misc_registers_mdio_override.mdio_override is set. [3:\n    0] - ch0_rr; [7:4] - ch1_rr; [11:8] - ch2_rr; [15:12] - ch3_rr; [19:16\n    ] - ch0_phy; [23:20] - ch1_phy; [27:24] - ch2_phy; [31:28] - ch3_phy. \n    Global register. Reset on Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9c0UL: // (MISC_REG_WC0_CTRL_EXTREMOTEMDIOST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC0_CTRL_EXTREMOTEMDIOST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Remote MDIO strap (BRCM mode rarely used); tie to 0. Drives\n     output misc_xgxs0_extremotemdiost. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9c4UL: // (MISC_REG_WC0_CTRL_MD_DEVAD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_WC0_CTRL_MD_DEVAD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Reserved for future XGXS cores which require an md_devad \n    input. Drives output misc_xgxs0_md_devad. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9c8UL: // (MISC_REG_WC0_CTRL_MD_ST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC0_CTRL_MD_ST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: MDIO State. This bit defines the LSB of the MDIO state machine\n     in the WarpCore. A logic-1 enables clause 22 and a logic-0 enables clause\n     45. Drives output misc_xgxs0_md_st. Global register. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9ccUL: // (MISC_REG_WC0_CTRL_PHY_ADDR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_WC0_CTRL_PHY_ADDR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: MDIO PHY Address. The WC uses this address to determine whether\n     or not it is the recipient of the message on the MDIO interface. The \n    value is compared to the value on ctrl_md_devad. Drives output misc_xgxs0_phy_addr\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9d0UL: // (MISC_REG_WC0_CTRL_PLL_BYPASS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC0_CTRL_PLL_BYPASS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Phase-locked-loop Bypass.  Drives output misc_xgxs0_pll_bypass\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9d4UL: // (MISC_REG_WC0_CTRL_REMOTEMDIOEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC0_CTRL_REMOTEMDIOEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Remote MDIO enable. Drives output misc_xgxs0_remotemdioen\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9d8UL: // (MISC_REG_WC1_CTRL_EXTREMOTEMDIOST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC1_CTRL_EXTREMOTEMDIOST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Remote MDIO strap (BRCM mode rarely used); tie to 0. Drives\n     output misc_xgxs1_extremotemdiost. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9dcUL: // (MISC_REG_WC1_CTRL_MD_DEVAD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_WC1_CTRL_MD_DEVAD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Reserved for future XGXS cores which require an md_devad \n    input. Drives output misc_xgxs1_md_devad. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9e0UL: // (MISC_REG_WC1_CTRL_MD_ST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC1_CTRL_MD_ST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: MDIO State. This bit defines the LSB of the MDIO state machine\n     in the WarpCore. A logic-1 enables clause 22 and a logic-0 enables clause\n     45. Drives output misc_xgxs1_md_st. Global register. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9e4UL: // (MISC_REG_WC1_CTRL_PHY_ADDR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_WC1_CTRL_PHY_ADDR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: MDIO PHY Address. The W C uses this address to determine \n    whether or not it is the recipient of the message on the MDIO interface\n    . The value is compared to the value on ctrl_md_devad. Drives output misc_xgxs1_phy_addr\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9e8UL: // (MISC_REG_WC1_CTRL_PLL_BYPASS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC1_CTRL_PLL_BYPASS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Phase-locked-loop Bypass.  Drives output misc_xgxs1_pll_bypass\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9ecUL: // (MISC_REG_WC1_CTRL_REMOTEMDIOEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC1_CTRL_REMOTEMDIOEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Remote MDIO enable. Drives output misc_xgxs1_remotemdioen\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9f0UL: // (MISC_REG_WC2_CTRL_EXTREMOTEMDIOST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC2_CTRL_EXTREMOTEMDIOST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Remote MDIO strap (BRCM mode rarely used); tie to 0. Drives\n     output misc_serdes0_extremotemdiost. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9f4UL: // (MISC_REG_WC2_CTRL_MD_DEVAD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_WC2_CTRL_MD_DEVAD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Reserved for future XGXS cores which require an md_devad \n    input. Drives output misc_serdes0_md_devad. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9f8UL: // (MISC_REG_WC2_CTRL_MD_ST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC2_CTRL_MD_ST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: MDIO State. This bit defines the LSB of the MDIO state machine\n     in the WarpCore. A logic-1 enables clause 22 and a logic-0 enables clause\n     45. Drives output misc_serdes0_md_st. Global register. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa9fcUL: // (MISC_REG_WC2_CTRL_PHY_ADDR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_WC2_CTRL_PHY_ADDR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: MDIO PHY Address. The WC uses this address to determine whether\n     or not it is the recipient of the message on the MDIO interface. The \n    value is compared to the value on ctrl_md_devad. Drives output misc_serdes0_phy_addr\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa00UL: // (MISC_REG_WC2_CTRL_PLL_BYPASS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC2_CTRL_PLL_BYPASS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Phase-locked-loop Bypass.  Drives output misc_serdes0_pll_bypass\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa04UL: // (MISC_REG_WC2_CTRL_REMOTEMDIOEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_WC2_CTRL_REMOTEMDIOEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Remote MDIO enable. Drives output misc_serdes0_remotemdioen\n    . Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa08UL: // (MISC_REG_WC0_MODE_STRAP: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_WC0_MODE_STRAP (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: 4 - os8: Independent channel mode w/ vco set to 10.3125G ; \n    5 - os5: Independent channel mode w/ vco set to 6.25G; 12 - combo Single\n     port mode. In Independent Channel modes the core supports multiple ports\n    . This can be four single lane ports; two dual lane ports; or 1 dual lane\n     port with two singles. Lanes 0 and 2 can be configured as either a single\n     lane or dual lane port; lanes 1 and  3 can only be programmed as a single\n     lane port. When lane 0 or 2 is programmed into a dual lane mode then \n    lanes 1 and 3 are configured as the second lane of the dual lane port \n    and therefore are not avaiable to be configured as an individual port. \n    In os8 mode each of the lanes within the core are capable of supporting\n     rates 10M; 100M (FX; SGMII); 1G (Fiber; SGMII); 2.5G; 10G; and 20G (DXGXS\n    ). In 0s5 each of these modes are supported except for 10G/20G. In combo\n     mode the core operates as a single port capable of supporting rates from\n     10M to 40G. Global register. Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa0cUL: // (MISC_REG_WC1_MODE_STRAP: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_WC1_MODE_STRAP (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: 4 - os8: Independent channel mode w/ vco set to 10.3125G ; \n    5 - os5: Independent channel mode w/ vco set to 6.25G; 12 - combo Single\n     port mode. In Independent Channel modes the core supports multiple ports\n    . This can be four single lane ports; two dual lane ports; or 1 dual lane\n     port with two singles. Lanes 0 and 2 can be configured as either a single\n     lane or dual lane port; lanes 1 and  3 can only be programmed as a single\n     lane port. When lane 0 or 2 is programmed into a dual lane mode then \n    lanes 1 and 3 are configured as the second lane of the dual lane port \n    and therefore are not avaiable to be configured as an individual port. \n    In os8 mode each of the lanes within the core are capable of supporting\n     rates 10M; 100M (FX; SGMII); 1G (Fiber; SGMII); 2.5G; 10G; and 20G (DXGXS\n    ). In 0s5 each of these modes are supported except for 10G/20G. In combo\n     mode the core operates as a single port capable of supporting rates from\n     10M to 40G. Global register. Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa10UL: // (MISC_REG_WC2_MODE_STRAP: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_WC2_MODE_STRAP (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: 4 - os8: Independent channel mode w/ vco set to 10.3125G ; \n    5 - os5: Independent channel mode w/ vco set to 6.25G; 12 - combo Single\n     port mode. In Independent Channel modes the core supports multiple ports\n    . This can be four single lane ports; two dual lane ports; or 1 dual lane\n     port with two singles. Lanes 0 and 2 can be configured as either a single\n     lane or dual lane port; lanes 1 and  3 can only be programmed as a single\n     lane port. When lane 0 or 2 is programmed into a dual lane mode then \n    lanes 1 and 3 are configured as the second lane of the dual lane port \n    and therefore are not avaiable to be configured as an individual port. \n    In os8 mode each of the lanes within the core are capable of supporting\n     rates 10M; 100M (FX; SGMII); 1G (Fiber; SGMII); 2.5G; 10G; and 20G (DXGXS\n    ). In 0s5 each of these modes are supported except for 10G/20G. In combo\n     mode the core operates as a single port capable of supporting rates from\n     10M to 40G. Global register. Hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa14UL: // (MISC_REG_XGXS_SERDES_MODE_SEL_OVWR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_XGXS_SERDES_MODE_SEL_OVWR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: WC0/WC1 to WC2 Mode select. [0] - Override Control. When \n    1 the logic based on port4mode_en and swap_mode is overrode by bit 1 value\n    . When 0 the output is driven by logic based on port4mode_en and swap_mode\n    . [1] - Override Value. Indicates the value of Mode select when bit 0 \n    is asserted.This control is used to select between the WarpCore0/1 and\n     the WarpCore2 for transmitting and receiving data. When Mode select is\n     0 the WarpCore2 is selected to override the WarpCore0/1. This is only\n     valid in specified special mode 0. Global register. Reset on hard reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa18UL: // (MISC_REG_XMAC_LPI_TX_DETECT_P0: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_XMAC_LPI_TX_DETECT_P0 (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: XMAC transmit LPI (Low Power Indication) state. This per \n    port signal is set to 1; whenever the transmit interface is in LPI state\n     and LPI_IDLES are being sent out on the TX Warpcore interface. Only bit\n     0 is applicable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa1cUL: // (MISC_REG_XMAC_LPI_TX_DETECT_P1: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_XMAC_LPI_TX_DETECT_P1 (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: XMAC transmit LPI state. This per port signal is set to 1\n    ; whenever the transmit interface is in LPI state and LPI_IDLES are being\n     sent out on the TX Warpcore interface. Only bit 0 is applicable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa2cUL: // (MISC_REG_PLL_MAIN_E40_CH1_MDIV: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_CH1_MDIV (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Post-divider ratio for channel-1 (divider ratio = code value\n    ). 00000000: 256; 00000001: 1; 00000010: 2; 00000011: 3; : :; 11111101\n    : 253; 11111110: 254; 11111111: 255. Global register. Reset on POR reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa30UL: // (MISC_REG_PLL_MAIN_E40_CH_LOAD_EN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_CH_LOAD_EN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Allows glitch free load capability. Global register. Reset\n     on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa34UL: // (MISC_REG_PLL_MAIN_E40_CH_ENABLEB: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E40_CH_ENABLEB (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Channel enable (only enables channel 0). Global register. \n    Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa38UL: // (MISC_REG_PLL_STORM_E40_CH1_MDIV: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E40_CH1_MDIV (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Post-divider ratio for channel-1 (divider ratio = code value\n    ). 00000000: 256; 00000001: 1; 00000010: 2; 00000011: 3; : :; 11111101\n    : 253; 11111110: 254; 11111111: 255. Global register. Reset on POR reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa3cUL: // (MISC_REG_PLL_STORM_E40_CH_ENABLEB: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E40_CH_ENABLEB (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Channel enable (only enables channel 0). Global register. \n    Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa40UL: // (MISC_REG_PLL_STORM_E40_CH_LOAD_EN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E40_CH_LOAD_EN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Allows glitch free load capability. Global register. Reset\n     on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa44UL: // (MISC_REG_LCPLL_E40_BYPASS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_BYPASS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: LCPLL Bypass. Drives the 156.25mhz applied to the balls of\n     the LCPLL directly to warpcore. This will allow to continue testing of\n     E3. By doing this we can use the same CML traces from LCPLL to Warpcore\n    . Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa48UL: // (MISC_REG_LCPLL_E40_CH0_MDIV: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_CH0_MDIV (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: LCPLL Post-divider ratio for channel-0 (divider ratio = code\n     value). 00000000= 256; 00000001= 1; 00000010= 2; 00000011= 3;-;11111101\n    = 253; 11111110= 254;11111111= 255. Divide by 20 to get 156.25MHz. Global\n     register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa4cUL: // (MISC_REG_LCPLL_E40_CH4_MDIV: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_CH4_MDIV (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: LCPLL Post-divider ratio for channel-4 (divider ratio = code\n     value). 00000000= 256; 00000001= 1; 00000010= 2; 00000011= 3;-;11111101\n    = 253; 11111110= 254;11111111= 255. Divide by 50 to get 62.5MHz. Global\n     register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa50UL: // (MISC_REG_LCPLL_E40_CH5_MDIV: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_CH5_MDIV (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: LCPLL Post-divider ratio for channel-5 (divider ratio = code\n     value). 00000000= 256; 00000001= 1; 00000010= 2; 00000011= 3;-;11111101\n    = 253; 11111110= 254;11111111= 255. Divide by 25 to get 125Mhz. Global\n     register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa54UL: // (MISC_REG_LCPLL_E40_CH_ENABLEB: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_CH_ENABLEB (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: LCPLL Enable (ACTIVE LOW) for post-divider channels 0-5. \n    0= channel ENABLED; 1= channel DISABLED (powered OFF); Bit<0>= post-divider\n     channel-0; Bit<1>= post-divider channel-1; Bit<2>= post-divider channel\n    -2; Bit<3>= post-divider channel-3; Bit<4>= post-divider channel-4; Bit\n    <5>= post-divider channel-5. Enable channels 0;5;6. Global register. Reset\n     on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa58UL: // (MISC_REG_LCPLL_E40_CH_LOAD_EN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_CH_LOAD_EN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: LCPLL LOAD ENABLE for post-divider channel 0-5. Allows on\n    -the-fly divider re-programming (glitch free). 0= HOLD present divider\n     setting; 1= LOAD in new divider setting; Bit<0>= LOAD ENABLE for post\n    -divider channel-0; Bit<1>= LOAD ENABLE for post-divider channel-1; Bit\n    <2>= LOAD ENABLE for post-divider channel-2; Bit<3>= LOAD ENABLE for post\n    -divider channel-3; Bit<4>= LOAD ENABLE for post-divider channel-4; Bit\n    <5>= LOAD ENABLE for post-divider channel-5. Global register. Reset on\n     POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa5cUL: // (MISC_REG_LCPLL_E40_CTRL_1: ACCESS:RW  DataWidth:0x1e ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_CTRL_1 (ACCESS:RW  DataWidth:0x1e )" \
				"\nDescription: LCPLL configuration control register. Global register. Reset\n     on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa60UL: // (MISC_REG_LCPLL_E40_CTRL_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_CTRL_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: [2:0]: Loop gain in frequency acquisition mode. [5:3]: Gain\n     of P/I loop filter integrator path during fine phase acquisition mode\n    . [9:6]: Gain of P/I loop filter proportional path during fine phase acquisition\n     mode. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa64UL: // (MISC_REG_LCPLL_E40_FREF_SEL: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_FREF_SEL (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: LCPLL 25MHz select. If 1 - selects the 25Mhz clock from the\n     balls or the pins of the LCPLL; if 0 - select 25Mhz from OSC. Global \n    register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa68UL: // (MISC_REG_LCPLL_E40_NDIV_FRAC: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_NDIV_FRAC (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: LCPLL Fractional feedback divider control.Resolution= 1/(\n    2^20). Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa6cUL: // (MISC_REG_LCPLL_E40_NDIV_INT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_NDIV_INT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: LCPLL Feedback divider control (Code = divider ratio). 00000000\n    = divide-by-256; 00000001= XXX; 00000010= XXX;...; 00000111= XXX; 00001000\n    = divide-by-8; 00001001= divide-by-9; 00001010= divide-by-10;...; 11111110\n    = divide-by-254; 11111111= divide-by-255. Global register. Reset on POR\n     reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa70UL: // (MISC_REG_LCPLL_E40_PDIV: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_PDIV (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: LCPLL input reference clock pre-divider control (code = divider\n     ratio). 000= divide-by-8; 001= divide-by-1; 010= divide-by-2; 011= divide\n    -by-3; 100= divide-by-4; 101= divide-by-5; 110= divide-by-6; 111= divide\n    -by-7. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa74UL: // (MISC_REG_LCPLL_E40_PWRDWN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_PWRDWN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: LCPLL power down. Global register. Active High. Reset on \n    POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa78UL: // (MISC_REG_LCPLL_E40_RESETB_ANA: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_RESETB_ANA (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: LCPLL VCO reset. Global register. Active Low Reset on POR\n     reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa7cUL: // (MISC_REG_LCPLL_E40_RESETB_DIG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_RESETB_DIG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: LCPLL post-divider reset. Global register. Active Low Reset\n     on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa80UL: // (MISC_REG_LCPLL_E40_STATUS: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E40_STATUS (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: LCPLL state RO register. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa84UL: // (MISC_REG_E1HMF_MODE_P0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_E1HMF_MODE_P0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Multifunction for WOL port0. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa88UL: // (MISC_REG_E1HMF_MODE_P1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_E1HMF_MODE_P1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Multifunction for WOL port1. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa8cUL: // (MISC_REG_WC0_PRTAD_BCST: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_WC0_PRTAD_BCST (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Broadcast port address. MDIO port address that the core respond\n     to in addition to prtad. Allow multiple cores to be written to during\n     a single MDIO frame (broadcast). Only works for MDIO writes. Drives output\n     misc_xgxs0_prtad_bcst. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa90UL: // (MISC_REG_WC1_PRTAD_BCST: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_WC1_PRTAD_BCST (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Broadcast port address. MDIO port address that the core respond\n     to in addition to prtad. Allow multiple cores to be written to during\n     a single MDIO frame (broadcast). Only works for MDIO writes. Drives output\n     misc_xgxs1_prtad_bcst. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa94UL: // (MISC_REG_WC2_PRTAD_BCST: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_WC2_PRTAD_BCST (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Broadcast port address. MDIO port address that the core respond\n     to in addition to prtad. Allow multiple cores to be written to during\n     a single MDIO frame (broadcast). Only works for MDIO writes. Drives output\n     misc_serdes0_prtad_bcst. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xac30UL: // (MISC_REG_WC0_RESET: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_WC0_RESET (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: reset reg#3;write/read one = the specific block is out of\n     reset; write/read zero = the specific block is in reset; addr 0-wr- the\n     write value will be written to the register; addr 1-set - one will be\n     written to all the bits that have the value of one in the data written\n     (bits that have the value of zero will not be change) ; addr 2-clear - \n    zero will be written to all the bits that have the value of one in the\n     data written (bits that have the value of zero will not be change); addr\n     3-ignore; read ignore from all addr except addr 00. [0]: rstb_hw: Active\n     low reset which when asserted drives entire WC into the reset state.  \n    All flops which within WC are driven into an initial state; as well as\n     the analog core. Output clocks txck_out; rxck0_10g; and clk_25 will be\n     driven to 0 upon its assertion. [1]: iddq. Enables iddq testing where\n     the supply current (Idd) is measured in the quiescent state. [2]: pwrdwn\n    : Active high control which forces the analog core of the WC into power\n    -do\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xac40UL: // (MISC_REG_WC1_RESET: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_WC1_RESET (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: reset reg#3;write/read one = the specific block is out of\n     reset; write/read zero = the specific block is in reset; addr 0-wr- the\n     write value will be written to the register; addr 1-set - one will be\n     written to all the bits that have the value of one in the data written\n     (bits that have the value of zero will not be change) ; addr 2-clear - \n    zero will be written to all the bits that have the value of one in the\n     data written (bits that have the value of zero will not be change); addr\n     3-ignore; read ignore from all addr except addr 00. [0]: rstb_hw: Active\n     low reset which when asserted drives entire WC into the reset state.  \n    All flops which within WC are driven into an initial state; as well as\n     the analog core. Output clocks txck_out; rxck0_10g; and clk_25 will be\n     driven to 0 upon its assertion. [1]: iddq. Enables iddq testing where\n     the supply current (Idd) is measured in the quiescent state. [2]: pwrdwn\n    : Active high control which forces the analog core of the WC into power\n    -do\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xac50UL: // (MISC_REG_WC2_RESET: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_WC2_RESET (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: reset reg#3;write/read one = the specific block is out of\n     reset; write/read zero = the specific block is in reset; addr 0-wr- the\n     write value will be written to the register; addr 1-set - one will be\n     written to all the bits that have the value of one in the data written\n     (bits that have the value of zero will not be change) ; addr 2-clear - \n    zero will be written to all the bits that have the value of one in the\n     data written (bits that have the value of zero will not be change); addr\n     3-ignore; read ignore from all addr except addr 00. [0]: rstb_hw: Active\n     low reset which when asserted drives entire WC into the reset state.  \n    All flops which within WC are driven into an initial state; as well as\n     the analog core. Output clocks txck_out; rxck0_10g; and clk_25 will be\n     driven to 0 upon its assertion. [1]: iddq. Enables iddq testing where\n     the supply current (Idd) is measured in the quiescent state. [2]: pwrdwn\n    : Active high control which forces the analog core of the WC into power\n    -do\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa2a8UL: // (MISC_REG_LCPLL_CTRL_REG_2: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_CTRL_REG_2 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: [7:0] 8 MSB of LCPLL second register. Inside order of bits\n    : [0-4] capSelectM[4:0] (reset value 0) bits to vco cap value; 00000 for\n     most cap and lowest frequency. [5] capForceSlowdown_en (reset value 0\n    ) bit to enable capForceSlowdown into capSeq. [6] capForceSlowdown (reset\n     value 0) bit to force slowdown. [7] Slowdn_xor (reset value 0) bit to\n     invert slowdn (added for test chip); [8] LCPLL RESET (reset value 0) \n    RESET pin of the LCPLL: Reset for VCO tuning sequencer. (active high; \n    longer than 30 usec); [9] LCPLL BYPASS_CML (reset value 0) BYPASS_CMD \n    pin of the LCPLL: Bypass mode enable (active high): 0= Normal mode; 1= \n    Bypass mode (CMOS bypass clock fed thru BYPASS MUX to CML output buffers\n    ); [10] LCPLL  BYPEN_CMOS (reset value 0) Bypass mode clock selection: \n    0= CMOS clock derived from REFP/N fed to bypass MUX; 1= CMOS clock from\n     Byp_CLK156 is fed to bypass MUX; [11] LCPLL PWRDN (reset value 1) PLL\n     power down (active high): 0 = Normal mode; 1 = Power down. Global register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4a0UL: // (MISC_REG_LCPLL_STATE: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_STATE (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: LCPLL state RO register. Mapped as follows: [0] freqDone_SM\n     bit indicating freq det is done. [1] freqPass_SM bit indicating frequency\n     lock. [2] pllSeqDone bit indicating pll sequencer is done. [3] pllSeqPass\n     bit indicating pll sequencer finised successfully. [6:4] pllState pll\n     sequence state variable. [9:7] capState cap sequencer state variable; \n    for test purposes. [14:10] capSelect cap selection to VCO; 00000 for most\n     cap and lowest vco frequency. [15] Slowdn indicator (PLL_LOW) VCO comparator\n     output. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa4acUL: // (MISC_REG_OSCCNTRL: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nMISC_REG_OSCCNTRL (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: OSCFUNDS control word. Inside order map: [11:10] Iamp_adj\n      adjust the current of the gain amplifier.(00:  150 uA; 01:  200 uA; \n    10:  250 uA; 11:  300 uA (reset value 10). [9:8] Icbuf_adj adjust the \n    current to the CMOS to CML buffer (00:  225 uA; 01:  300 uA; 10:  375 \n    uA; 11:  450 uA (reset value 10). [7:6] xtal_adjCM adjust the common-mode\n     of the oscillator first stage (01:  1.45 V; 00:  1.54 V; 10:  1.62 V; \n    11:  1.66 V (reset value 00). [5] Frequency-monitor output enable (0 - \n    off ; 1 - on) (reset value 0). [4] XCcml_6P; XCcml_6N Removed on latest\n     revision of the IP. (0-on; 1-off) (reset value 1). [3] XCcml_5P; XCcml_5N\n     Removed on latest revision of the IP. (0-on; 1-off) (reset value 1). [\n    2] XCcml_4P; XCcml_4N (0-on; 1-off) (reset value 1). [1] XCcml_3P; XCcml_3N\n     (0-on; 1-off) (reset value 0). [0] XCcml_2P; XCcml_2N (0-on; 1-off) (\n    reset value 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa5a0UL: // (MISC_REG_RESET_REG_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_RESET_REG_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: reset reg#3;write/read one = the specific block is out of\n     reset; write/read zero = the specific block is in reset; addr 0-wr- the\n     write value will be written to the register; addr 1-set - one will be\n     written to all the bits that have the value of one in the data written\n     (bits that have the value of zero will not be change) ; addr 2-clear - \n    zero will be written to all the bits that have the value of one in the\n     data written (bits that have the value of zero will not be change); addr\n     3-ignore; read ignore from all addr except addr 00; inside order of the\n     bits is: [0] misc_nig_mux_serdes0_rstb_hw (NIG - Reset Controls to SERDES0\n    ); [1] misc_nig_mux_serdes0_iddq (NIG - Reset Controls to SERDES0); [2\n    ] misc_nig_mux_serdes0_pwrdwn (NIG - Reset Controls to SERDES0); [3] misc_nig_mux_serdes0_pwrdwn_sd\n     (NIG - Reset Controls to SERDES0); [4] misc_nig_mux_xgxs0_rstb_hw (NIG\n     - Reset Controls to XGXS 0) (2-port mode: per-path; 4-port-mode: global\n    ); [5] misc_nig_mux_xgxs0_iddq (NIG - Reset\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa5f8UL: // (MISC_REG_E1HMF_MODE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_E1HMF_MODE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: multifunction for WOL. If clr WOL signal of the PXP will \n    be send on bit 0 only. Global register. Reset on hard reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa638UL: // (MISC_REG_PLL_STORM_E65_TEST_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_TEST_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Test mode output enable (active HIGH): ;0= testout in High\n    -Z state; 1= testout active. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa63cUL: // (MISC_REG_PLL_STORM_E65_P1_DIV: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_P1_DIV (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Input reference clock pre-divider control: 0000= divide-by\n    -16; 0001= divide-by-1; 0010= divide-by-2 ; 0011= divide-by-3; 0100= divide\n    -by-4; 0101= divide-by-5; 0110= divide-by-6; 0111= divide-by-7; 1000= \n    divide-by-8; 1001= divide-by-9; 1010= divide-by-10; 1011= divide-by-11\n    ; 1100= divide-by-12 ;1101= divide-by-13; 1110= divide-by-14; 1111= divide\n    -by-15. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa640UL: // (MISC_REG_PLL_STORM_E65_P2_DIV: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_P2_DIV (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Feedback pre-divider control: 0000= divide-by-16; 0001= divide\n    -by-1; 0010= divide-by-2 ; 0011= divide-by-3; 0100= divide-by-4; 0101= \n    divide-by-5; 0110= divide-by-6; 0111= divide-by-7; 1000= divide-by-8; \n    1001= divide-by-9; 1010= divide-by-10; 1011= divide-by-11; 1100= divide\n    -by-12 ;1101= divide-by-13; 1110= divide-by-14; 1111= divide-by-15. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa644UL: // (MISC_REG_PLL_STORM_E65_NDIV_MODE: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_NDIV_MODE (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Feedback Divider Mode Control: (Note: If the fractional part\n     is n/8; it is basically still interger-N mode; the modulator is not working\n    . So the loop filter setting should still follow integer-N modes setting\n    ); 000= Integer-N Mode (DEFAULT); 001= MASH ??? ; odulator Unit Mode; \n    010= MFB ??? Modulator Unit Mode (default for frac); th011= MASH ??? Modulator\n     1/8 Mode; th100= MFB ??? Modulator 1/8 Mode; 101= unused; 110= unused\n    ; 111= unused. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa648UL: // (MISC_REG_PLL_STORM_E65_NDIV_FRAC: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_NDIV_FRAC (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: Fractional control for feedback divider: i_ndiv_frac[23:0\n    ]; 24 Divide fraction= i_ndiv_frac [23:0] / 2. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa64cUL: // (MISC_REG_PLL_STORM_E65_M1_DIV: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_M1_DIV (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Clock channel-1 post divider control: (divide ratio= code\n     value); 00000000= divide-by-256; 00000001= divide-by-1; 00000010= divide\n    -by-2; 00000011= divide-by-3; 00000100= divide-by-4; 00000101= divide-\n    by-5; 00000110= divide-by-6 ... 11111011= divide-by-251; 11111100= divide\n    -by-252; 11111101= divide-by-253; 11111110= divide-by-254; 11111111= divide\n    -by-255. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa650UL: // (MISC_REG_PLL_STORM_E65_VCO_RNG: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_VCO_RNG (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: VCO range control register: (bit [1] unused at present); \n    x0: [ 800MHz ; 1600 MHz ]; x1: (1600MHz; 3200 MHz]. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa654UL: // (MISC_REG_PLL_STORM_E65_PLL_TEST_SE: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_PLL_TEST_SE (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Test mode clock select: 0000= VCO Vcontrol (see section 6\n    ; i_pll_ctrl[31:30]); 0001= VCO/8 clock; 0010= frefi; 0011= fdbki; 0100\n    = watchdog timer reset output; 0101= clkout1; 0110= clkout2; 0111= clkout3\n    ; 1000=clkout4**; 1001=clkout5**; 1010=clkout6** Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa658UL: // (MISC_REG_PLL_STORM_E65_CTRL_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_CTRL_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Configuration control register block. Bits ; Name ; Default\n     Setting ; Description. [3:0] -  Icp_off[3:0] ;  reset value - 0 ; Description\n     (Offset down current: Icp_off= Icp_off[4:0] /31 x 0.5LSB of Icp ). [4\n    ] -  freq_det_dis ;  reset value - 0 ; Description (Watchdog disable. \n    freq_det_dis=0 watchdog works. freq_det_dis=1 disable watchdog to prevent\n     frequency comparison).[9:5] -  Icpx[4:0] ;  reset value - 1110 ; Description\n     (Up/Down current (Icp): Icp= (1 + Icpx[4:0]) x IUNIT; I= (1.6 x (Vc-V\n    ) /14 kohm )/10; ( for Fvco<1600MHz)). [12:10] -  Rz[2:0] ;  reset value\n     - 1 ; Description (Rz= (1 + Rz[2:0]) x 7 kohm)). [14:13] -  Cz[1:0] ;  \n    reset value - 0 ; Description (Cz= (1 + Cz[1:0]) x 36.48 pF)). [16:15] -  \n    Cp[1:0] ;  reset value - 0 ; Description (Cp= 2Cp[1:0] x 0.57 pF). [18\n    :17] -  Rn[1:0] ;  reset value - 0 ; Description (3 pole resistor: Rn= (\n    4 + Rn[1:0]) x 7 kohm). [20:19] -  Cn[1:0] ;  reset value - 0 ; Description\n     (3 pole cap: Cn=2Cn[1:0]x 0.57 pF). [21] -  LF_order ;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa65cUL: // (MISC_REG_PLL_STORM_E65_CTRL_2: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_CTRL_2 (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Configuration control register block. 0] -  ; biasin_en ;  \n    reset value - 0 ; Description (CML bias reference enable: 0= disable;  \n    1= enable). [1] -  ; Lowcur_en ;  reset value - 0 ; Description (CML output\n     buffer drive: 0= 400-? output impedance (1.2 mA); 1= 800-? output impedance\n     (0.6 mA)). [3:2] -  ; ctap_adj[1:0] ;  reset value - 1 ; Description (\n    CTAT current I variation:  ;00=-6.25%%; 01/10=0%%; 11=6.25%%). [5:4] -  ; \n    ptap_adj[1:0] ;  reset value - 1 ; Description (PTAT current I variation\n    : 00=-6.25%%; 01/10=0%%; 11=6.25%%). Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa660UL: // (MISC_REG_PLL_MAIN_E65_OTP_TEST_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_TEST_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Test mode output enable (active HIGH): ;0= testout in High\n    -Z state; 1= testout active. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa664UL: // (MISC_REG_PLL_MAIN_E65_OTP_P1_DIV: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_P1_DIV (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Input reference clock pre-divider control: 0000= divide-by\n    -16; 0001= divide-by-1; 0010= divide-by-2 ; 0011= divide-by-3; 0100= divide\n    -by-4; 0101= divide-by-5; 0110= divide-by-6; 0111= divide-by-7; 1000= \n    divide-by-8; 1001= divide-by-9; 1010= divide-by-10; 1011= divide-by-11\n    ; 1100= divide-by-12 ;1101= divide-by-13; 1110= divide-by-14; 1111= divide\n    -by-15. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa668UL: // (MISC_REG_PLL_MAIN_E65_OTP_P2_DIV: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_P2_DIV (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Feedback pre-divider control: 0000= divide-by-16; 0001= divide\n    -by-1; 0010= divide-by-2 ; 0011= divide-by-3; 0100= divide-by-4; 0101= \n    divide-by-5; 0110= divide-by-6; 0111= divide-by-7; 1000= divide-by-8; \n    1001= divide-by-9; 1010= divide-by-10; 1011= divide-by-11; 1100= divide\n    -by-12 ;1101= divide-by-13; 1110= divide-by-14; 1111= divide-by-15. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa66cUL: // (MISC_REG_PLL_MAIN_E65_OTP_NDIV_MODE: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_NDIV_MODE (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Feedback Divider Mode Control: (Note: If the fractional part\n     is n/8; it is basically still interger-N mode; the modulator is not working\n    . So the loop filter setting should still follow integer-N modes setting\n    ); 000= Integer-N Mode (DEFAULT); 001= MASH ??? ; odulator Unit Mode; \n    010= MFB ??? Modulator Unit Mode (default for frac); th011= MASH ??? Modulator\n     1/8 Mode; th100= MFB ??? Modulator 1/8 Mode; 101= unused; 110= unused\n    ; 111= unused. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa670UL: // (MISC_REG_PLL_MAIN_E65_OTP_NDIV_FRAC: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_NDIV_FRAC (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: Fractional control for feedback divider: i_ndiv_frac[23:0\n    ]; 24 Divide fraction= i_ndiv_frac [23:0] / 2. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa674UL: // (MISC_REG_PLL_MAIN_E65_OTP_M1_DIV: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_M1_DIV (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Clock channel-1 post divider control: (divide ratio= code\n     value); 00000000= divide-by-256; 00000001= divide-by-1; 00000010= divide\n    -by-2; 00000011= divide-by-3; 00000100= divide-by-4; 00000101= divide-\n    by-5; 00000110= divide-by-6 ... 11111011= divide-by-251; 11111100= divide\n    -by-252; 11111101= divide-by-253; 11111110= divide-by-254; 11111111= divide\n    -by-255. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa678UL: // (MISC_REG_PLL_MAIN_E65_OTP_VCO_RNG: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_VCO_RNG (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: VCO range control register: (bit [1] unused at present); \n    x0: [ 800MHz ; 1600 MHz ]; x1: (1600MHz; 3200 MHz]. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa67cUL: // (MISC_REG_PLL_MAIN_E65_OTP_PLL_TEST_SE: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_PLL_TEST_SE (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Test mode clock select: 0000= VCO Vcontrol (see section 6\n    ; i_pll_ctrl[31:30]); 0001= VCO/8 clock; 0010= frefi; 0011= fdbki; 0100\n    = watchdog timer reset output; 0101= clkout1; 0110= clkout2; 0111= clkout3\n    ; 1000=clkout4**; 1001=clkout5**; 1010=clkout6**. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa680UL: // (MISC_REG_PLL_MAIN_E65_OTP_CTRL_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_CTRL_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Configuration control register block. Bits ; Name ; Default\n     Setting ; Description. [3:0] -  Icp_off[3:0] ;  reset value - 0 ; Description\n     (Offset down current: Icp_off= Icp_off[4:0] /31 x 0.5LSB of Icp ). [4\n    ] -  freq_det_dis ;  reset value - 0 ; Description (Watchdog disable. \n    freq_det_dis=0 watchdog works. freq_det_dis=1 disable watchdog to prevent\n     frequency comparison).[9:5] -  Icpx[4:0] ;  reset value - 1110 ; Description\n     (Up/Down current (Icp): Icp= (1 + Icpx[4:0]) x IUNIT; I= (1.6 x (Vc-V\n    ) /14 kohm )/10; ( for Fvco<1600MHz)). [12:10] -  Rz[2:0] ;  reset value\n     - 1 ; Description (Rz= (1 + Rz[2:0]) x 7 kohm)). [14:13] -  Cz[1:0] ;  \n    reset value - 0 ; Description (Cz= (1 + Cz[1:0]) x 36.48 pF)). [16:15] -  \n    Cp[1:0] ;  reset value - 0 ; Description (Cp= 2Cp[1:0] x 0.57 pF). [18\n    :17] -  Rn[1:0] ;  reset value - 0 ; Description (3 pole resistor: Rn= (\n    4 + Rn[1:0]) x 7 kohm). [20:19] -  Cn[1:0] ;  reset value - 0 ; Description\n     (3 pole cap: Cn=2Cn[1:0]x 0.57 pF). [21] -  LF_order ;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa684UL: // (MISC_REG_PLL_MAIN_E65_OTP_CTRL_2: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_CTRL_2 (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Configuration control register block. 0] -  ; biasin_en ;  \n    reset value - 0 ; Description (CML bias reference enable: 0= disable;  \n    1= enable). [1] -  ; Lowcur_en ;  reset value - 0 ; Description (CML output\n     buffer drive: 0= 400-? output impedance (1.2 mA); 1= 800-? output impedance\n     (0.6 mA)). [3:2] -  ; ctap_adj[1:0] ;  reset value - 1 ; Description (\n    CTAT current I variation:  ;00=-6.25%%; 01/10=0%%; 11=6.25%%). [5:4] -  ; \n    ptap_adj[1:0] ;  reset value - 1 ; Description (PTAT current I variation\n    : 00=-6.25%%; 01/10=0%%; 11=6.25%%). Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa730UL: // (MISC_REG_XGXS0_CTRL_MD_ST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_XGXS0_CTRL_MD_ST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: 4-port mode control to XGXS; Hardwired ST bits LSB: 0 - clause\n     45; 1 - clause 22. Drives output misc_xgxs0_mux_md_st. Global register\n    . \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa73cUL: // (MISC_REG_TEMPMON_CTRL: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_TEMPMON_CTRL (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Temperature Monitor control. [7:1] - Analog bias adjustments\n    ; [0] - ADC test mode enable (when 0 - SAR ADC test mode; when 1 - Temp\n     monitor normal mode).Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa740UL: // (MISC_REG_TEMPMON_PWR: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_TEMPMON_PWR (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Temperature Monitor power control. [0] - Temperature monitor\n     reset (0 - asserted; 1 - de-asserted); [1] - Temperature monitor power\n     down (0 - power down; 1 - power up).Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa744UL: // (MISC_REG_TEMPMON_HOLD: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_TEMPMON_HOLD (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Temperature Monitor hold. 0 - update every 400us; 1 - hold\n     on to the value forever.Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa748UL: // (MISC_REG_TEMPMON_DATA: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nMISC_REG_TEMPMON_DATA (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Temperature Monitor output.Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa760UL: // (MISC_REG_XGXS0_CTRL_MD_DEVAD: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_XGXS0_CTRL_MD_DEVAD (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: 4-port mode control to XGXS; Hardwired Device Address bits\n     CL45 DEVAD. Drives output misc_xgxs0_mux_md_devad. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa764UL: // (MISC_REG_XGXS0_CTRL_PHY_ADDR: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nMISC_REG_XGXS0_CTRL_PHY_ADDR (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: 4-port mode control to XGXS; Hardwired port address - CL22\n     PHY_ADD and CL45 PRTAD. Drives output misc_xgxs0_mux_phy_addr. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa768UL: // (MISC_REG_XGXS0_CTRL_REMOTEMDIOEN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_XGXS0_CTRL_REMOTEMDIOEN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: 4-port mode control to XGXS; remote PHY in-band MDIO. Drives\n     output misc_xgxs0_mux_remotemdioen. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa76cUL: // (MISC_REG_XGXS0_CTRL_EXTREMOTEMDIOST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_XGXS0_CTRL_EXTREMOTEMDIOST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: 4-port mode control to XGXS; remote PHY in-band MDIO. When\n     set causes the remote MDIO arbitration to be performed via pins instead\n     of mdio register bits. Drives output misc_xgxs0_mux_extremotemdiost. \n    Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa770UL: // (MISC_REG_XGXS0_CTRL_PLL_BYPASS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_XGXS0_CTRL_PLL_BYPASS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to xgxs - PLL bypass\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa790UL: // (MISC_REG_VOLTMON_CTRL: ACCESS:R  DataWidth:0x1c ) 
			sprintf(disp_str, "\nMISC_REG_VOLTMON_CTRL (ACCESS:R  DataWidth:0x1c )" \
				"\nDescription: Volatge Monitor control. [5:0] - 6 bit trim for the PTAT \n    Resistor (Nominal +/- 7 0n 64 steps) - denoted as i_bg_ptat[5:0] internally\n    . 2s complement logic. 000000 - Nominal (Default); 000001 - Nominal + \n    0.22%% (i.e. +7%%/32); 111111 - Nominal - 0.22%% (i.e. -7%%/32; [7:6] - \n    Enable measurement of different voltages. Denoted as i_bg_out_tempco[1\n    :0] internally. 00 - Reference voltage measurement (sum of PTAT and CTAT\n    ) (Default); 01 - PTAT voltage measurement; 10 - CTAT voltage measurement\n    ; 11 - same as 00 setting; [13:8] - 6 bit trim for the CTAT Resistor (\n    Nominal +/- 10 0n 64 steps) - denoted as i_bg_ctat[5:0] internally. 2s\n     complement logic. 000000 - Nominal (Default) ; 000001 - Nominal + 0.3\n    %% (i.e. +10%%/32); 111111 - Nominal - 0.3%% (i.e. -10%%/32); [19:14] - \n    6 bit trim for the vtemp Resistor (Nominal +/- 4 0n 64 steps) - denoted\n     as i_vtemp_trim[5:0] internally. 2s complement logic. 000000 - Nominal\n     (Default); 000001 - Nominal + 0.12%% (i.e. + 4%%/32); 111111 - Nominal\n     - 0.12%% (i.e\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa794UL: // (MISC_REG_VOLTMON_DATA: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nMISC_REG_VOLTMON_DATA (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Voltage Monitor output.Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa798UL: // (MISC_REG_VOLTMON_HOLD: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_VOLTMON_HOLD (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Voltage Monitor hold. 0 - update; 1 - hold on to the value\n     forever.Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa79cUL: // (MISC_REG_VOLTMON_PWR: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_VOLTMON_PWR (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Voltage Monitor power control. [0] - Voltage monitor reset\n     (0 - asserted; 1 - de-asserted); [1] - Voltage monitor power down (0 - \n    power down; 1 - power up).Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7a0UL: // (MISC_REG_VOLTMON_SEL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_VOLTMON_SEL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Voltage Monitor select. 1 = choose 3.3V; 0 = choose 1.2V.\n    Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7a4UL: // (MISC_REG_PWRWDOG_PWRDN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_PWRDN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Power watchdog power down. 1 - power down; 0 - power up.Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7a8UL: // (MISC_REG_PWRWDOG_CGFG: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_CGFG (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Power watchdog configuration. Adjustment of integer delay\n     cgfg[7:4] and fractional delay cgfg[3:0]; valid range [0:191]. Global\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7acUL: // (MISC_REG_PWRWDOG_RSEL: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_RSEL (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Power watchdog selection of delay setting of ring delay element\n    ; recommended setting is 3b100 for longest ring delay; 3b010 will shorten\n     the ring by one inverter+inverting mux; 3b001 will shorten the ring by\n     two inverter+inverting mux cells;Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7b0UL: // (MISC_REG_PWRWDOG_CLEARCFG: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_CLEARCFG (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Power watchdog selection of delay chain for the reset pulse\n    ; recommended setting is 3b100 for longest pulse; 3b010 will shorten the\n     reset pulse by one inverter+inverting mux delay; 3b001 will shorten the\n     reset pulse by two inverter+inverting mux delays; set clearcfg more or\n     equal than rsel to ensure the ring gets flushed from the previous cycle\n    ; reducing the reset pulse length is needed to reach the highest possible\n     operating frequencies.Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7b4UL: // (MISC_REG_PWRWDOG_START: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_START (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Power watchdog. Sets the number of trips through the ring\n     delay element to start+1; range for start is [-1:31].Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7b8UL: // (MISC_REG_PWRWDOG_ACCU_RUN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_ACCU_RUN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Power watchdog. When 0 all registers and states are of power\n     watchdog accu sub-module are reset. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7bcUL: // (MISC_REG_PWRWDOG_ACCU_SKIP: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_ACCU_SKIP (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Power watchdog. Allows to decimate the good signal to measure\n     for beats. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7c0UL: // (MISC_REG_PWRWDOG_ACCU_SKIP_START: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_ACCU_SKIP_START (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Power watchdog. Start value of skip counter. Global register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7c4UL: // (MISC_REG_PWRWDOG_DATA: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_DATA (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: Power watchdog data.Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7c8UL: // (MISC_REG_PWRWDOG_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PWRWDOG_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Power watchdog done. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa800UL: // (MISC_REG_PLL_MAIN_E65_CTRL_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_CTRL_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Configuration control register block. Bits ; Name ; Default\n     Setting ; Description. [3:0] -  Icp_off[3:0] ;  reset value - 0 ; Description\n     (Offset down current: Icp_off= Icp_off[4:0] /31 x 0.5LSB of Icp ). [4\n    ] -  freq_det_dis ;  reset value - 0 ; Description (Watchdog disable. \n    freq_det_dis=0 watchdog works. freq_det_dis=1 disable watchdog to prevent\n     frequency comparison).[9:5] -  Icpx[4:0] ;  reset value - 1110 ; Description\n     (Up/Down current (Icp): Icp= (1 + Icpx[4:0]) x IUNIT; I= (1.6 x (Vc-V\n    ) /14 kohm )/10; ( for Fvco<1600MHz)). [12:10] -  Rz[2:0] ;  reset value\n     - 1 ; Description (Rz= (1 + Rz[2:0]) x 7 kohm)). [14:13] -  Cz[1:0] ;  \n    reset value - 0 ; Description (Cz= (1 + Cz[1:0]) x 36.48 pF)). [16:15] -  \n    Cp[1:0] ;  reset value - 0 ; Description (Cp= 2Cp[1:0] x 0.57 pF). [18\n    :17] -  Rn[1:0] ;  reset value - 0 ; Description (3 pole resistor: Rn= (\n    4 + Rn[1:0]) x 7 kohm). [20:19] -  Cn[1:0] ;  reset value - 0 ; Description\n     (3 pole cap: Cn=2Cn[1:0]x 0.57 pF). [21] -  LF_order ;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa804UL: // (MISC_REG_PLL_MAIN_E65_CTRL_2: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_CTRL_2 (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Configuration control register block. 0] -  ; biasin_en ;  \n    reset value - 0 ; Description (CML bias reference enable: 0= disable;  \n    1= enable). [1] -  ; Lowcur_en ;  reset value - 0 ; Description (CML output\n     buffer drive: 0= 400-? output impedance (1.2 mA); 1= 800-? output impedance\n     (0.6 mA)). [3:2] -  ; ctap_adj[1:0] ;  reset value - 1 ; Description (\n    CTAT current I variation:  ;00=-6.25%%; 01/10=0%%; 11=6.25%%). [5:4] -  ; \n    ptap_adj[1:0] ;  reset value - 1 ; Description (PTAT current I variation\n    : 00=-6.25%%; 01/10=0%%; 11=6.25%%). Global register. Reset on POR reset\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa808UL: // (MISC_REG_PLL_MAIN_E65_M1_DIV: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_M1_DIV (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Clock channel-1 post divider control: (divide ratio= code\n     value); 00000000= divide-by-256; 00000001= divide-by-1; 00000010= divide\n    -by-2; 00000011= divide-by-3; 00000100= divide-by-4; 00000101= divide-\n    by-5; 00000110= divide-by-6 ... 11111011= divide-by-251; 11111100= divide\n    -by-252; 11111101= divide-by-253; 11111110= divide-by-254; 11111111= divide\n    -by-255. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa80cUL: // (MISC_REG_PLL_MAIN_E65_NDIV_FRAC: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_NDIV_FRAC (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: Fractional control for feedback divider: i_ndiv_frac[23:0\n    ]; 24 Divide fraction= i_ndiv_frac [23:0] / 2. Global register. Reset \n    on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa810UL: // (MISC_REG_PLL_MAIN_E65_NDIV_MODE: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_NDIV_MODE (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Feedback Divider Mode Control: (Note: If the fractional part\n     is n/8; it is basically still interger-N mode; the modulator is not working\n    . So the loop filter setting should still follow integer-N modes setting\n    ); 000= Integer-N Mode (DEFAULT); 001= MASH ??? ; odulator Unit Mode; \n    010= MFB ??? Modulator Unit Mode (default for frac); th011= MASH ??? Modulator\n     1/8 Mode; th100= MFB ??? Modulator 1/8 Mode; 101= unused; 110= unused\n    ; 111= unused. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa814UL: // (MISC_REG_PLL_MAIN_E65_P1_DIV: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_P1_DIV (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Input reference clock pre-divider control: 0000= divide-by\n    -16; 0001= divide-by-1; 0010= divide-by-2 ; 0011= divide-by-3; 0100= divide\n    -by-4; 0101= divide-by-5; 0110= divide-by-6; 0111= divide-by-7; 1000= \n    divide-by-8; 1001= divide-by-9; 1010= divide-by-10; 1011= divide-by-11\n    ; 1100= divide-by-12 ;1101= divide-by-13; 1110= divide-by-14; 1111= divide\n    -by-15. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa818UL: // (MISC_REG_PLL_MAIN_E65_P2_DIV: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_P2_DIV (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Feedback pre-divider control: 0000= divide-by-16; 0001= divide\n    -by-1; 0010= divide-by-2 ; 0011= divide-by-3; 0100= divide-by-4; 0101= \n    divide-by-5; 0110= divide-by-6; 0111= divide-by-7; 1000= divide-by-8; \n    1001= divide-by-9; 1010= divide-by-10; 1011= divide-by-11; 1100= divide\n    -by-12 ;1101= divide-by-13; 1110= divide-by-14; 1111= divide-by-15. Global\n     register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa81cUL: // (MISC_REG_PLL_MAIN_E65_PLL_TEST_SE: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_PLL_TEST_SE (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Test mode clock select: 0000= VCO Vcontrol (see section 6\n    ; i_pll_ctrl[31:30]); 0001= VCO/8 clock; 0010= frefi; 0011= fdbki; 0100\n    = watchdog timer reset output; 0101= clkout1; 0110= clkout2; 0111= clkout3\n    ; 1000=clkout4**; 1001=clkout5**; 1010=clkout6**. Global register. Reset\n     on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa820UL: // (MISC_REG_PLL_MAIN_E65_TEST_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_TEST_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Test mode output enable (active HIGH): ;0= testout in High\n    -Z state; 1= testout active. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa824UL: // (MISC_REG_PLL_MAIN_E65_VCO_RNG: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_VCO_RNG (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: VCO range control register: (bit [1] unused at present); \n    x0: [ 800MHz ; 1600 MHz ]; x1: (1600MHz; 3200 MHz]. Global register. Reset\n     on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa828UL: // (MISC_REG_PLL_MAIN_E65_NDIV_INT: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_NDIV_INT (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Clock channel-1 post divider control:(divide ratio= code \n    value);00000000= divide-by-256; 00000001= divide-by-1; 00000010= divide\n    -by-2; 00000011= divide-by-3; 00000100= divide-by-4; 00000101= divide-\n    by-5; 00000110= divide-by-6...11111011= divide-by-251; 11111100= divide\n    -by-252; 11111101= divide-by-253; 11111110= divide-by-254; 11111111= divide\n    -by-255. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa82cUL: // (MISC_REG_MISC_OSC65_CONTROL: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nMISC_REG_MISC_OSC65_CONTROL (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: osccntrl[15:14] Iamp_adj [1:0] adjust the current of the \n    gain amplifier; osccntrl[13:12] Icbuf_adj [1:0] adjust the current to \n    the CMOS to CML buffer; osccntrl[11] IbFD_adj adjust the current to the\n     Frequency Doubler Block; osccntrl[10] Activate the amplitude limiter \n    function. Constrains the amplitude on XP and XN to  =1.5 Vpp single-ended\n     regardless of crystal type or load capacitors. This limiter is automatically\n      disabled in bypass mode. osccntrl[9:8] xtal_adjCM [1:0] adjust the common\n    -mode of the oscillator first stage osccntrl[7] bypass frequency ; osccntrl\n    [6] cml_6p cml_6n 0: output is active; osccntrl[5] cml_5p cml_5n 0: output\n     is active ;osccntrl[4] cml_4p cml_4n 0: output is active; osccntrl[3] \n    cml_3p cml_3n 0: output is active; osccntrl[2] cml_2p cml_2n 0: output\n     is active; osccntrl[1] Frequency-monitor output enable; osccntrl[0] Frequency\n     Doubler power down. Global register. Reset on POR reset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa830UL: // (MISC_REG_PLL_STORM_E65_NDIV_INT: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_STORM_E65_NDIV_INT (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Clock channel-1 post divider control:(divide ratio= code \n    value);00000000= divide-by-256; 00000001= divide-by-1; 00000010= divide\n    -by-2; 00000011= divide-by-3; 00000100= divide-by-4; 00000101= divide-\n    by-5; 00000110= divide-by-6...11111011= divide-by-251; 11111100= divide\n    -by-252; 11111101= divide-by-253; 11111110= divide-by-254; 11111111= divide\n    -by-255. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa834UL: // (MISC_REG_PLL_MAIN_E65_OTP_NDIV_INT: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nMISC_REG_PLL_MAIN_E65_OTP_NDIV_INT (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Clock channel-1 post divider control:(divide ratio= code \n    value);00000000= divide-by-256; 00000001= divide-by-1; 00000010= divide\n    -by-2; 00000011= divide-by-3; 00000100= divide-by-4; 00000101= divide-\n    by-5; 00000110= divide-by-6...11111011= divide-by-251; 11111100= divide\n    -by-252; 11111101= divide-by-253; 11111110= divide-by-254; 11111111= divide\n    -by-255. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa838UL: // (MISC_REG_LCPLL_E65_CTRL_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E65_CTRL_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit [0]  reserved ;Bits [4:1] OC [3:0] ;Bits [8:5] Icp_ctrl\n     [3:0] ;Bits[10:9] FD_ctrl[1:0] Bits [14:11] Bias_ctrl[3:0] ;Bits [20:\n    15] Pll_observe[5:0] ;Bits [26:21] LF_ctrl[5:0] ;Bit [31:27] PLL_ctrl[\n    5:0]. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa83cUL: // (MISC_REG_LCPLL_E65_CTRL_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E65_CTRL_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit [43:32] pll_val[11:0] ;Bit [45:44] ref_val[1:0] ;Bit [\n    46] TAL_enable_b ;Bit [47] cal_enable_b ;Bit [50:48] refresh_cap_cnt[2\n    :0] ;Bit [51] refresh_cal ;Bit [54:52] numCapChange[2:0] ;Bit [63:55] \n    force_caps_val[8:0]. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa840UL: // (MISC_REG_LCPLL_E65_CTRL_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E65_CTRL_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit [71:64] delayBeforeOpenLoop[7:0] ;Bit [79:72] delayAfterRefresh\n    [7:0] Bit [87:80] delayAfterOpenLoop[7:0] ;Bit [95:88] delayAfterCloseLoop\n    [7:0]. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa844UL: // (MISC_REG_LCPLL_E65_CTRL_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMISC_REG_LCPLL_E65_CTRL_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit [107:96] calSetCount[11:0] ;Bits [108] force_caps ;Bit\n     [109] clk_gate_ovr ;Bit [110] cal_mode ;Bit [111] calSetCountOvr ;Bit\n     [119:112] cal_ref_timeout[7:0] ;Bit[124:120] enableTimeOut ;Bit[127:125\n    ] reserved. Global register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa54cUL: // (MISC_REG_MISC_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa554UL: // (MISC_REG_MISC_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa55cUL: // (MISC_REG_MISC_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa564UL: // (MISC_REG_MISC_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa56cUL: // (MISC_REG_MISC_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa578UL: // (MISC_REG_MISC_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa5b0UL: // (MISC_REG_MISC_UNUSED_EMPTY_6: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_6 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7dcUL: // (MISC_REG_MISC_UNUSED_EMPTY_7: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_7 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa7f8UL: // (MISC_REG_MISC_UNUSED_EMPTY_8: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_8 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa970UL: // (MISC_REG_MISC_UNUSED_EMPTY_9: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_9 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xaa98UL: // (MISC_REG_MISC_UNUSED_EMPTY_10: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_10 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xac64UL: // (MISC_REG_MISC_UNUSED_EMPTY_11: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMISC_REG_MISC_UNUSED_EMPTY_11 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x7f0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x7f0UL): // (MSTAT_REG_MSTAT_INT_STS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_MSTAT_INT_STS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & MSTAT_MSTAT_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_MSTAT0 + 0x7f4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x7f4UL): // (MSTAT_REG_MSTAT_INT_STS_CLR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_MSTAT_INT_STS_CLR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & MSTAT_MSTAT_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_MSTAT0 + 0x7f8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x7f8UL): // (MSTAT_REG_MSTAT_INT_STS_WR: ACCESS:WR  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_MSTAT_INT_STS_WR (ACCESS:WR  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & MSTAT_MSTAT_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_MSTAT0 + 0x7fcUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x7fcUL): // (MSTAT_REG_MSTAT_INT_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_MSTAT_INT_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & MSTAT_MSTAT_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_MSTAT0 + 0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0UL): // (MSTAT_REG_TX_STAT_GTXPOK_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTXPOK_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [00] Tx Good Packet Count Write to this register write \n    bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x4UL): // (MSTAT_REG_TX_STAT_GTXPOK_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTXPOK_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GTXPOK statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x8UL): // (MSTAT_REG_TX_STAT_GTXPF_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTXPF_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [01] Tx Pause Packet Counter Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xcUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xcUL): // (MSTAT_REG_TX_STAT_GTXPF_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTXPF_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GTXPF statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x10UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x10UL): // (MSTAT_REG_TX_STAT_GTXPP_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTXPP_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [02] Tx PFC Packet Counter Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x14UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x14UL): // (MSTAT_REG_TX_STAT_GTXPP_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTXPP_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GTXPP statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x18UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x18UL): // (MSTAT_REG_TX_STAT_GTFCS_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTFCS_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [03] Tx FCS Error Packet Counter Write to this register\n     write bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x1cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x1cUL): // (MSTAT_REG_TX_STAT_GTFCS_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTFCS_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GTFCS statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x20UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x20UL): // (MSTAT_REG_TX_STAT_GTUCA_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTUCA_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [04] Tx Unicast Packet Counter Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x24UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x24UL): // (MSTAT_REG_TX_STAT_GTUCA_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTUCA_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GTUCA statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x28UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x28UL): // (MSTAT_REG_TX_STAT_GTMCA_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTMCA_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [05] Tx Multicast Packet Counter Write to this register\n     write bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2cUL): // (MSTAT_REG_TX_STAT_GTMCA_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTMCA_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GTMCA statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x30UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x30UL): // (MSTAT_REG_TX_STAT_GTBCA_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTBCA_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [06] Tx Broadcast Packet Counter Write to this register\n     write bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x34UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x34UL): // (MSTAT_REG_TX_STAT_GTBCA_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTBCA_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GTBCA statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x38UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x38UL): // (MSTAT_REG_TX_STAT_GTPKT_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTPKT_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [07] Tx Total Packet Count Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x3cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x3cUL): // (MSTAT_REG_TX_STAT_GTPKT_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTPKT_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GTPKT statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x40UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x40UL): // (MSTAT_REG_TX_STAT_GT64_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT64_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [08] Packet Size = 64 Write to this register write bits\n     31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x44UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x44UL): // (MSTAT_REG_TX_STAT_GT64_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT64_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GT64 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x48UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x48UL): // (MSTAT_REG_TX_STAT_GT127_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT127_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [09] Packet Size = 65-127 Write to this register write \n    bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x4cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x4cUL): // (MSTAT_REG_TX_STAT_GT127_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT127_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GT127 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x50UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x50UL): // (MSTAT_REG_TX_STAT_GT255_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT255_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [10] Packet Size = 128-255 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x54UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x54UL): // (MSTAT_REG_TX_STAT_GT255_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT255_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GT255 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x58UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x58UL): // (MSTAT_REG_TX_STAT_GT511_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT511_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [11] Packet Size = 256-511 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x5cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x5cUL): // (MSTAT_REG_TX_STAT_GT511_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT511_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GT511 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x60UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x60UL): // (MSTAT_REG_TX_STAT_GT1023_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT1023_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [12] Packet Size = 512-1023 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x64UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x64UL): // (MSTAT_REG_TX_STAT_GT1023_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT1023_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GT1023 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x68UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x68UL): // (MSTAT_REG_TX_STAT_GT1518_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT1518_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [13] Packet Size = 1024-1522 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6cUL): // (MSTAT_REG_TX_STAT_GT1518_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT1518_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GT1518 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x70UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x70UL): // (MSTAT_REG_TX_STAT_GT2047_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT2047_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [14] Packet Size = 1523-2047 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x74UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x74UL): // (MSTAT_REG_TX_STAT_GT2047_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT2047_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GT2047 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x78UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x78UL): // (MSTAT_REG_TX_STAT_GT4095_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT4095_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [15] Packet Size = 2048-4095 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x7cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x7cUL): // (MSTAT_REG_TX_STAT_GT4095_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT4095_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GT4095 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x80UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x80UL): // (MSTAT_REG_TX_STAT_GT9216_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT9216_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [16] Packet Size = 4096-9216 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x84UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x84UL): // (MSTAT_REG_TX_STAT_GT9216_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT9216_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GT9216 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x88UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x88UL): // (MSTAT_REG_TX_STAT_GT16383_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT16383_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [17] Packet Size = 9217-16383 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x8cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x8cUL): // (MSTAT_REG_TX_STAT_GT16383_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT16383_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GT16383 statistic. \n    Write to this register write bits 6:0. Reads from this register will clear\n     bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x90UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x90UL): // (MSTAT_REG_TX_STAT_GTUFL_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTUFL_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [18] Tx FIFO Underflow Write to this register write bits\n     31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x94UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x94UL): // (MSTAT_REG_TX_STAT_GTUFL_HI: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTUFL_HI (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This is the upper half of the TX_STAT_GTUFL statistic. Write\n     to this register write bits 0:0. Reads from this register will clear \n    bits 0:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x98UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x98UL): // (MSTAT_REG_TX_STAT_GTERR_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTERR_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [19] Tx Errored Packet Count Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x9cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x9cUL): // (MSTAT_REG_TX_STAT_GTERR_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTERR_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the TX_STAT_GTERR statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xa0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xa0UL): // (MSTAT_REG_TX_STAT_GTBYT_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTBYT_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 16 [36:21] Tx Byte Count ([20] = Enable) Write to this register\n     write bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xa4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xa4UL): // (MSTAT_REG_TX_STAT_GTBYT_HI: ACCESS:RW  DataWidth:0xd ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTBYT_HI (ACCESS:RW  DataWidth:0xd )" \
				"\nDescription: This is the upper half of the TX_STAT_GTBYT statistic. Write\n     to this register write bits 12:0. Reads from this register will clear\n     bits 12:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xa8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xa8UL): // (MSTAT_REG_ETHERSTATSCOLLISIONS_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_ETHERSTATSCOLLISIONS_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 4 [41:38] Total number of collisions detected ([37] = Enable\n    ) Write to this register write bits 31:0. Reads from this register will\n     clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xacUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xacUL): // (MSTAT_REG_ETHERSTATSCOLLISIONS_HI: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_ETHERSTATSCOLLISIONS_HI (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This is the upper half of the ETHERSTATSCOLLISIONS statistic\n    . Write to this register write bits 0:0. Reads from this register will\n     clear bits 0:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xb0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xb0UL): // (MSTAT_REG_DOT3STATSSINGLECOLLISIONFRAMES_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSSINGLECOLLISIONFRAMES_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [42] Number of packets sent after one collision Write to\n     this register write bits 31:0. Reads from this register will clear bits\n     31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xb4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xb4UL): // (MSTAT_REG_DOT3STATSSINGLECOLLISIONFRAMES_HI: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSSINGLECOLLISIONFRAMES_HI (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This is the upper half of the DOT3STATSSINGLECOLLISIONFRAMES\n     statistic. Write to this register write bits 0:0. Reads from this register\n     will clear bits 0:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xb8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xb8UL): // (MSTAT_REG_DOT3STATSMULTIPLECOLLISIONFRAMES_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSMULTIPLECOLLISIONFRAMES_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [43] Number of packets sent after more than one collision\n     Write to this register write bits 31:0. Reads from this register will\n     clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xbcUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xbcUL): // (MSTAT_REG_DOT3STATSMULTIPLECOLLISIONFRAMES_HI: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSMULTIPLECOLLISIONFRAMES_HI (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This is the upper half of the DOT3STATSMULTIPLECOLLISIONFRAMES\n     statistic. Write to this register write bits 0:0. Reads from this register\n     will clear bits 0:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xc0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xc0UL): // (MSTAT_REG_DOT3STATSDEFERREDTRANSMISSIONS_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSDEFERREDTRANSMISSIONS_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [44] Number of transmissions delayed due to an Rx packet\n     on the wire Write to this register write bits 31:0. Reads from this register\n     will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xc4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xc4UL): // (MSTAT_REG_DOT3STATSDEFERREDTRANSMISSIONS_HI: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSDEFERREDTRANSMISSIONS_HI (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This is the upper half of the DOT3STATSDEFERREDTRANSMISSIONS\n     statistic. Write to this register write bits 0:0. Reads from this register\n     will clear bits 0:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xc8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xc8UL): // (MSTAT_REG_DOT3STATSEXCESSIVECOLLISIONS_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSEXCESSIVECOLLISIONS_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [45] Number of packets dropped due to 16 collisions Write\n     to this register write bits 31:0. Reads from this register will clear\n     bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xccUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xccUL): // (MSTAT_REG_DOT3STATSEXCESSIVECOLLISIONS_HI: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSEXCESSIVECOLLISIONS_HI (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This is the upper half of the DOT3STATSEXCESSIVECOLLISIONS\n     statistic. Write to this register write bits 0:0. Reads from this register\n     will clear bits 0:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xd0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xd0UL): // (MSTAT_REG_DOT3STATSLATECOLLISIONS_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSLATECOLLISIONS_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [46] Number of packets dropped due to late collisions Write\n     to this register write bits 31:0. Reads from this register will clear\n     bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xd4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xd4UL): // (MSTAT_REG_DOT3STATSLATECOLLISIONS_HI: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSLATECOLLISIONS_HI (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This is the upper half of the DOT3STATSLATECOLLISIONS statistic\n    . Write to this register write bits 0:0. Reads from this register will\n     clear bits 0:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x200UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x200UL): // (MSTAT_REG_RX_STAT_GR64_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR64_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [47] Packet Size = 64 Write to this register write bits\n     31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x204UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x204UL): // (MSTAT_REG_RX_STAT_GR64_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR64_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GR64 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x208UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x208UL): // (MSTAT_REG_RX_STAT_GR127_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR127_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [48] Packet Size = 65-127 Write to this register write \n    bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x20cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x20cUL): // (MSTAT_REG_RX_STAT_GR127_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR127_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GR127 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x210UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x210UL): // (MSTAT_REG_RX_STAT_GR255_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR255_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [49] Packet Size = 128-255 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x214UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x214UL): // (MSTAT_REG_RX_STAT_GR255_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR255_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GR255 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x218UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x218UL): // (MSTAT_REG_RX_STAT_GR511_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR511_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [50] Packet Size = 256-511 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x21cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x21cUL): // (MSTAT_REG_RX_STAT_GR511_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR511_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GR511 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x220UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x220UL): // (MSTAT_REG_RX_STAT_GR1023_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR1023_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [51] Packet Size = 512-1023 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x224UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x224UL): // (MSTAT_REG_RX_STAT_GR1023_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR1023_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GR1023 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x228UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x228UL): // (MSTAT_REG_RX_STAT_GR1518_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR1518_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [52] Packet Size = 1024-1522 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x22cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x22cUL): // (MSTAT_REG_RX_STAT_GR1518_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR1518_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GR1518 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x230UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x230UL): // (MSTAT_REG_RX_STAT_GR2047_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR2047_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [53] Packet Size = 1523-2047 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x234UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x234UL): // (MSTAT_REG_RX_STAT_GR2047_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR2047_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GR2047 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x238UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x238UL): // (MSTAT_REG_RX_STAT_GR4095_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR4095_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [54] Packet Size = 2048-4095 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x23cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x23cUL): // (MSTAT_REG_RX_STAT_GR4095_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR4095_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GR4095 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x240UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x240UL): // (MSTAT_REG_RX_STAT_GR9216_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR9216_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [55] Packet Size = 4096-9216 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x244UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x244UL): // (MSTAT_REG_RX_STAT_GR9216_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR9216_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GR9216 statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x248UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x248UL): // (MSTAT_REG_RX_STAT_GR16383_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR16383_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [56] Packet Size = 9217-16383 Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x24cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x24cUL): // (MSTAT_REG_RX_STAT_GR16383_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR16383_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GR16383 statistic. \n    Write to this register write bits 6:0. Reads from this register will clear\n     bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x250UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x250UL): // (MSTAT_REG_RX_STAT_GRPKT_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRPKT_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [57] Valid Rx Packet Count Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x254UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x254UL): // (MSTAT_REG_RX_STAT_GRPKT_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRPKT_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRPKT statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x258UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x258UL): // (MSTAT_REG_RX_STAT_GRFCS_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFCS_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [58] FCS Error Count Write to this register write bits \n    31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x25cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x25cUL): // (MSTAT_REG_RX_STAT_GRFCS_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFCS_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRFCS statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x260UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x260UL): // (MSTAT_REG_RX_STAT_GRUCA_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRUCA_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [59] Unicast Packet Count Write to this register write \n    bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x264UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x264UL): // (MSTAT_REG_RX_STAT_GRUCA_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRUCA_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRUCA statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x268UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x268UL): // (MSTAT_REG_RX_STAT_GRMCA_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRMCA_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [60] Mulitcast Packet Count Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x26cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x26cUL): // (MSTAT_REG_RX_STAT_GRMCA_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRMCA_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRMCA statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x270UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x270UL): // (MSTAT_REG_RX_STAT_GRBCA_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRBCA_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [61] Broadcast Packet Count Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x274UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x274UL): // (MSTAT_REG_RX_STAT_GRBCA_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRBCA_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRBCA statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x278UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x278UL): // (MSTAT_REG_RX_STAT_GRXPF_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXPF_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [62] Pause Frame Count Write to this register write bits\n     31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x27cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x27cUL): // (MSTAT_REG_RX_STAT_GRXPF_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXPF_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRXPF statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x280UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x280UL): // (MSTAT_REG_RX_STAT_GRXPP_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXPP_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [63] PFC Frame Count Write to this register write bits \n    31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x284UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x284UL): // (MSTAT_REG_RX_STAT_GRXPP_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXPP_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRXPP statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x288UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x288UL): // (MSTAT_REG_RX_STAT_GRXUO_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXUO_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [64] Control Frame w/ Unknown Opcode Count Write to this\n     register write bits 31:0. Reads from this register will clear bits 31\n    :0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x28cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x28cUL): // (MSTAT_REG_RX_STAT_GRXUO_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXUO_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRXUO statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x290UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x290UL): // (MSTAT_REG_RX_STAT_GROVR_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GROVR_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [65] Oversized Packet Write to this register write bits\n     31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x294UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x294UL): // (MSTAT_REG_RX_STAT_GROVR_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GROVR_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GROVR statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x298UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x298UL): // (MSTAT_REG_RX_STAT_GRXCF_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXCF_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [66] Control Frame Count Write to this register write bits\n     31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x29cUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x29cUL): // (MSTAT_REG_RX_STAT_GRXCF_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXCF_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRXCF statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2a0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2a0UL): // (MSTAT_REG_RX_STAT_GRFLR_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFLR_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [67] Frame Length Error. Write to this register write bits\n     31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2a4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2a4UL): // (MSTAT_REG_RX_STAT_GRFLR_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFLR_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRFLR statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2a8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2a8UL): // (MSTAT_REG_RX_STAT_GRPOK_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRPOK_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [68] Ok Packet Count (UC or MC or BC) Write to this register\n     write bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2acUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2acUL): // (MSTAT_REG_RX_STAT_GRPOK_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRPOK_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRPOK statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2b0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2b0UL): // (MSTAT_REG_RX_STAT_GRBYT_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRBYT_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 16 [85:70] Total Rx Byte Count ([69] = Enable) Write to this\n     register write bits 31:0. Reads from this register will clear bits 31\n    :0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2b4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2b4UL): // (MSTAT_REG_RX_STAT_GRBYT_HI: ACCESS:RW  DataWidth:0xd ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRBYT_HI (ACCESS:RW  DataWidth:0xd )" \
				"\nDescription: This is the upper half of the RX_STAT_GRBYT statistic. Write\n     to this register write bits 12:0. Reads from this register will clear\n     bits 12:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2b8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2b8UL): // (MSTAT_REG_RX_STAT_GRUND_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRUND_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [86] Undersized Packet Count. < 64; Good FCS Write to this\n     register write bits 31:0. Reads from this register will clear bits 31\n    :0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2bcUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2bcUL): // (MSTAT_REG_RX_STAT_GRUND_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRUND_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRUND statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2c0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2c0UL): // (MSTAT_REG_RX_STAT_GRFRG_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFRG_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [87] Fragment Count. < 64; Bad FCS Write to this register\n     write bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2c4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2c4UL): // (MSTAT_REG_RX_STAT_GRFRG_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFRG_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRFRG statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2c8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2c8UL): // (MSTAT_REG_RX_STAT_GRERB_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRERB_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 16 [104:89] Error Packet Byte Count ([88] = Enable) Write\n     to this register write bits 31:0. Reads from this register will clear\n     bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2ccUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2ccUL): // (MSTAT_REG_RX_STAT_GRERB_HI: ACCESS:RW  DataWidth:0xd ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRERB_HI (ACCESS:RW  DataWidth:0xd )" \
				"\nDescription: This is the upper half of the RX_STAT_GRERB statistic. Write\n     to this register write bits 12:0. Reads from this register will clear\n     bits 12:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2d0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2d0UL): // (MSTAT_REG_RX_STAT_GRFRE_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFRE_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [105] Frame Error Packet Count Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2d4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2d4UL): // (MSTAT_REG_RX_STAT_GRFRE_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFRE_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RX_STAT_GRFRE statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2d8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2d8UL): // (MSTAT_REG_DOT3STATSALIGNMENTERRORS_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSALIGNMENTERRORS_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [106] Number of packets with invalid size (extra nibble\n    ) Write to this register write bits 31:0. Reads from this register will\n     clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2dcUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2dcUL): // (MSTAT_REG_DOT3STATSALIGNMENTERRORS_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSALIGNMENTERRORS_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the DOT3STATSALIGNMENTERRORS statistic\n    . Write to this register write bits 6:0. Reads from this register will\n     clear bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2e0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2e0UL): // (MSTAT_REG_FALSECARRIERERRORS_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_FALSECARRIERERRORS_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [107] Number of False Carrier Events detected. Write to\n     this register write bits 31:0. Reads from this register will clear bits\n     31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2e4UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2e4UL): // (MSTAT_REG_FALSECARRIERERRORS_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_FALSECARRIERERRORS_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the FALSECARRIERERRORS statistic\n    . Write to this register write bits 6:0. Reads from this register will\n     clear bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2e8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2e8UL): // (MSTAT_REG_RXLLFCMSGCNT_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nMSTAT_REG_RXLLFCMSGCNT_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 1 [108] Rx LLFC Packet Count Write to this register write\n     bits 31:0. Reads from this register will clear bits 31:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2ecUL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2ecUL): // (MSTAT_REG_RXLLFCMSGCNT_HI: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nMSTAT_REG_RXLLFCMSGCNT_HI (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: This is the upper half of the RxLLFCMSGCNT statistic. Write\n     to this register write bits 6:0. Reads from this register will clear \n    bits 6:0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x400UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x400UL): // (MSTAT_REG_TX_STAT_GTXPOK_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTXPOK_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTXPOK\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x408UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x408UL): // (MSTAT_REG_TX_STAT_GTXPF_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTXPF_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTXPF \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x410UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x410UL): // (MSTAT_REG_TX_STAT_GTXPP_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTXPP_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTXPP \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x418UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x418UL): // (MSTAT_REG_TX_STAT_GTFCS_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTFCS_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTFCS \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x420UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x420UL): // (MSTAT_REG_TX_STAT_GTUCA_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTUCA_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTUCA \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x428UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x428UL): // (MSTAT_REG_TX_STAT_GTMCA_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTMCA_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTMCA \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x430UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x430UL): // (MSTAT_REG_TX_STAT_GTBCA_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTBCA_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTBCA \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x438UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x438UL): // (MSTAT_REG_TX_STAT_GTPKT_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTPKT_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTPKT \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x440UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x440UL): // (MSTAT_REG_TX_STAT_GT64_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT64_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GT64 statitic\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x448UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x448UL): // (MSTAT_REG_TX_STAT_GT127_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT127_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GT127 \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x450UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x450UL): // (MSTAT_REG_TX_STAT_GT255_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT255_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GT255 \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x458UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x458UL): // (MSTAT_REG_TX_STAT_GT511_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT511_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GT511 \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x460UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x460UL): // (MSTAT_REG_TX_STAT_GT1023_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT1023_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GT1023\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x468UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x468UL): // (MSTAT_REG_TX_STAT_GT1518_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT1518_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GT1518\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x470UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x470UL): // (MSTAT_REG_TX_STAT_GT2047_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT2047_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GT2047\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x478UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x478UL): // (MSTAT_REG_TX_STAT_GT4095_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT4095_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GT4095\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x480UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x480UL): // (MSTAT_REG_TX_STAT_GT9216_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT9216_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GT9216\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x488UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x488UL): // (MSTAT_REG_TX_STAT_GT16383_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GT16383_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GT16383\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x490UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x490UL): // (MSTAT_REG_TX_STAT_GTUFL_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTUFL_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTUFL \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x498UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x498UL): // (MSTAT_REG_TX_STAT_GTERR_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTERR_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTERR \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x4a0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x4a0UL): // (MSTAT_REG_TX_STAT_GTBYT_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_TX_STAT_GTBYT_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the TX_STAT_GTBYT \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x4a8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x4a8UL): // (MSTAT_REG_ETHERSTATSCOLLISIONS_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_ETHERSTATSCOLLISIONS_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the ETHERSTATSCOLLISIONS\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x4b0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x4b0UL): // (MSTAT_REG_DOT3STATSSINGLECOLLISIONFRAMES_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSSINGLECOLLISIONFRAMES_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the DOT3STATSSINGLECOLLISIONFRAMES\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x4b8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x4b8UL): // (MSTAT_REG_DOT3STATSMULTIPLECOLLISIONFRAMES_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSMULTIPLECOLLISIONFRAMES_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the DOT3STATSMULTIPLECOLLISIONFRAMES\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x4c0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x4c0UL): // (MSTAT_REG_DOT3STATSDEFERREDTRANSMISSIONS_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSDEFERREDTRANSMISSIONS_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the DOT3STATSDEFERREDTRANSMISSIONS\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x4c8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x4c8UL): // (MSTAT_REG_DOT3STATSEXCESSIVECOLLISIONS_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSEXCESSIVECOLLISIONS_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the DOT3STATSEXCESSIVECOLLISIONS\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x4d0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x4d0UL): // (MSTAT_REG_DOT3STATSLATECOLLISIONS_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSLATECOLLISIONS_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the DOT3STATSLATECOLLISIONS\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x600UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x600UL): // (MSTAT_REG_RX_STAT_GR64_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR64_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GR64 statitic\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x608UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x608UL): // (MSTAT_REG_RX_STAT_GR127_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR127_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GR127 \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x610UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x610UL): // (MSTAT_REG_RX_STAT_GR255_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR255_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GR255 \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x618UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x618UL): // (MSTAT_REG_RX_STAT_GR511_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR511_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GR511 \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x620UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x620UL): // (MSTAT_REG_RX_STAT_GR1023_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR1023_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GR1023\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x628UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x628UL): // (MSTAT_REG_RX_STAT_GR1518_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR1518_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GR1518\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x630UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x630UL): // (MSTAT_REG_RX_STAT_GR2047_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR2047_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GR2047\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x638UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x638UL): // (MSTAT_REG_RX_STAT_GR4095_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR4095_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GR4095\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x640UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x640UL): // (MSTAT_REG_RX_STAT_GR9216_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR9216_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GR9216\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x648UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x648UL): // (MSTAT_REG_RX_STAT_GR16383_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GR16383_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GR16383\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x650UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x650UL): // (MSTAT_REG_RX_STAT_GRPKT_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRPKT_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRPKT \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x658UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x658UL): // (MSTAT_REG_RX_STAT_GRFCS_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFCS_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRFCS \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x660UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x660UL): // (MSTAT_REG_RX_STAT_GRUCA_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRUCA_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRUCA \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x668UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x668UL): // (MSTAT_REG_RX_STAT_GRMCA_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRMCA_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRMCA \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x670UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x670UL): // (MSTAT_REG_RX_STAT_GRBCA_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRBCA_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRBCA \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x678UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x678UL): // (MSTAT_REG_RX_STAT_GRXPF_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXPF_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRXPF \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x680UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x680UL): // (MSTAT_REG_RX_STAT_GRXPP_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXPP_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRXPP \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x688UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x688UL): // (MSTAT_REG_RX_STAT_GRXUO_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXUO_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRXUO \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x690UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x690UL): // (MSTAT_REG_RX_STAT_GROVR_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GROVR_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GROVR \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x698UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x698UL): // (MSTAT_REG_RX_STAT_GRXCF_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRXCF_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRXCF \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6a0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6a0UL): // (MSTAT_REG_RX_STAT_GRFLR_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFLR_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRFLR \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6a8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6a8UL): // (MSTAT_REG_RX_STAT_GRPOK_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRPOK_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRPOK \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6b0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6b0UL): // (MSTAT_REG_RX_STAT_GRBYT_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRBYT_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRBYT \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6b8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6b8UL): // (MSTAT_REG_RX_STAT_GRUND_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRUND_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRUND \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6c0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6c0UL): // (MSTAT_REG_RX_STAT_GRFRG_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFRG_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRFRG \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6c8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6c8UL): // (MSTAT_REG_RX_STAT_GRERB_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRERB_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRERB \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6d0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6d0UL): // (MSTAT_REG_RX_STAT_GRFRE_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RX_STAT_GRFRE_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RX_STAT_GRFRE \n    statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6d8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6d8UL): // (MSTAT_REG_DOT3STATSALIGNMENTERRORS_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_DOT3STATSALIGNMENTERRORS_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the DOT3STATSALIGNMENTERRORS\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6e0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6e0UL): // (MSTAT_REG_FALSECARRIERERRORS_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_FALSECARRIERERRORS_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the FALSECARRIERERRORS\n     statitic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6e8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6e8UL): // (MSTAT_REG_RXLLFCMSGCNT_WB: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nMSTAT_REG_RXLLFCMSGCNT_WB (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This register provides WB-RO access of the RxLLFCMSGCNT statitic\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0xd8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0xd8UL): // (MSTAT_REG_MSTAT_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMSTAT_REG_MSTAT_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x2f0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x2f0UL): // (MSTAT_REG_MSTAT_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMSTAT_REG_MSTAT_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x4d8UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x4d8UL): // (MSTAT_REG_MSTAT_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMSTAT_REG_MSTAT_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_MSTAT0 + 0x6f0UL): // (2-port) 
		case (GRCBASE_MSTAT1 + 0x6f0UL): // (MSTAT_REG_MSTAT_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nMSTAT_REG_MSTAT_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10000UL: // (NIG_REG_NIG_INGRESS_PORT_RMP_FIFO_0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INGRESS_PORT_RMP_FIFO_0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for management fifo\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10004UL: // (NIG_REG_NIG_INGRESS_PORT_RMP_FIFO_1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INGRESS_PORT_RMP_FIFO_1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for management fifo\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10008UL: // (NIG_REG_NIG_INGRESS_EOP_FIFO_0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INGRESS_EOP_FIFO_0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for eop fifo of port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1000cUL: // (NIG_REG_NIG_INGRESS_EOP_FIFO_1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INGRESS_EOP_FIFO_1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for eop fifo of port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10010UL: // (NIG_REG_NIG_INGRESS_EOP_FIFO_2_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INGRESS_EOP_FIFO_2_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for eop fifo of LB\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10014UL: // (NIG_REG_NIG_EGRESS_RMP_FIFO_0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_EGRESS_RMP_FIFO_0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for rmp fifo in TX\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10018UL: // (NIG_REG_NIG_EGRESS_RMP_FIFO_1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_EGRESS_RMP_FIFO_1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for rmp fifo in TX\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1001cUL: // (NIG_REG_EGRESS_DEBUG_ALM_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DEBUG_ALM_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almoust full for debug FIFO in NIG_TX_DBG\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10020UL: // (NIG_REG_EGRESS_DELAY_ALM_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY_ALM_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almoust full for delay PBF FIFO in NIG_TX_PORT0 and NIG_TX_PORT1\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10028UL: // (NIG_REG_INGRESS_PBF_LB_BUFFER_ALM_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_PBF_LB_BUFFER_ALM_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almoust full for Delay FIFO from PBF in NIG_RX_lb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1002cUL: // (NIG_REG_TIMER0_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_TIMER0_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Maximum value for timer ~nig_registers_timer0_counter.timer0_counter\n     in NIG_TX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10030UL: // (NIG_REG_TIMER1_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_TIMER1_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Maximum value for timer ~nig_registers_timer1_counter.timer1_counter\n     in NIG_TX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10034UL: // (NIG_REG_TIMER0_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_TIMER0_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 1- ~nig_registers_timer0_counter.timer0_counter counts\n     from 0 after it gets maximum value  = ~nig_registers_timer0_max.timer0_max\n    . In this case interrupt to RBC may be sent some times: each time when\n     counter gets maximum. Other way if 0 - it stops to count for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10038UL: // (NIG_REG_TIMER1_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_TIMER1_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 1- ~nig_registers_timer1_counter.timer1_counter counts\n     from 0 after it gets maximum value ~nig_registers_timer1_max.timer1_max\n    . In this case interrupt to RBC may be sent some times: each time when\n     counter gets maximum. Other way if 0 - it stops to count for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10054UL: // (NIG_REG_EGRESS_DEBUG_PORT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DEBUG_PORT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Port configuration for packet from debug IF. 1 - debug packet\n     for port 1; other way for port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10058UL: // (NIG_REG_EGRESS_EMAC0_PORT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC0_PORT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: MAC configuration for packets of port0. If 1 - all packet\n     outputs to emac for port0; other way to bmac for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1005cUL: // (NIG_REG_EGRESS_EMAC1_PORT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC1_PORT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: MAC configuration for packets of port1. If 1 - all packet\n     outputs to emac for port1; other way to bmac for port1.  This bit is \n    not used in E2 since port 1 only has EMAC.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10060UL: // (NIG_REG_EGRESS_DRAIN0_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DRAIN0_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 1 - egress drain mode for port0 is active. In this mode\n     all packets from PBFare not forwarded to the MAC and just deleted from\n     FIFO. First packet may be deleted from the middle. And last packet will\n     be always deleted till the end.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10064UL: // (NIG_REG_EGRESS_DRAIN1_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DRAIN1_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 1 - egress drain mode for port1 is active. In this mode\n     all packets from PBFare not forwarded to the MAC and just deleted from\n     FIFO. First packet may be deleted from the middle. And last packet will\n     be always deleted till the end.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10068UL: // (NIG_REG_EGRESS_PARITY_ERR_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_PARITY_ERR_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: if 1 - egress drain mode  is enabled when parity error input\n     Is active ti NIG. Otherway always masks parity error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1006cUL: // (NIG_REG_INGRESS_MNG0_PKT_END: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG0_PKT_END (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When UMP and RBC end to work with RX management packet of\n     port0; RBC has to write to this register. It will cause to increment \n    consumer pointer in RX management FIFO and NIG  will pass to work with\n     a next packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10070UL: // (NIG_REG_INGRESS_MNG1_PKT_END: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG1_PKT_END (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When UMP and RBC end to work with RX management packet of\n     port1; RBC has to write to this register. It will cause to increment \n    consumer pointer in RX management FIFO and NIG  will pass to work with\n     a next packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10074UL: // (NIG_REG_LLH0_T_BIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_T_BIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: t bit for llh0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10078UL: // (NIG_REG_LLH1_T_BIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_T_BIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: t bit for llh1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1007cUL: // (NIG_REG_LLH0_CM_HEADER: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_CM_HEADER (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: cm header for llh0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10080UL: // (NIG_REG_LLH1_CM_HEADER: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_CM_HEADER (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: cm header for llh1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10084UL: // (NIG_REG_LLH0_EVENT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_EVENT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: event id for llh0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10088UL: // (NIG_REG_LLH1_EVENT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_EVENT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: event id for llh1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1008cUL: // (NIG_REG_LLH0_ERROR_MASK: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ERROR_MASK (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH0_ERROR_MASK_MAC_ERR :"
				, (u32_t)((data & NIG_LLH0_ERROR_MASK_REG_LLH0_ERROR_MASK_MAC_ERR) >> 5)
				, "\n\tLLH0_ERROR_MASK_UDP_LEN :"
				, (u32_t)((data & NIG_LLH0_ERROR_MASK_REG_LLH0_ERROR_MASK_UDP_LEN) >> 4)
				, "\n\tLLH0_ERROR_MASK_HEADER_BIG_5 :"
				, (u32_t)((data & NIG_LLH0_ERROR_MASK_REG_LLH0_ERROR_MASK_HEADER_BIG_5) >> 3)
				, "\n\tLLH0_ERROR_MASK_HEADER_LESS_5 :"
				, (u32_t)((data & NIG_LLH0_ERROR_MASK_REG_LLH0_ERROR_MASK_HEADER_LESS_5) >> 2)
				, "\n\tLLH0_ERROR_MASK_VERSION_NOT_6 :"
				, (u32_t)((data & NIG_LLH0_ERROR_MASK_REG_LLH0_ERROR_MASK_VERSION_NOT_6) >> 1)
				, "\n\tLLH0_ERROR_MASK_VERSION_NOT_4 :"
				, (u32_t)((data & NIG_LLH0_ERROR_MASK_REG_LLH0_ERROR_MASK_VERSION_NOT_4) >> 0));
			break;
		case 0x10090UL: // (NIG_REG_LLH1_ERROR_MASK: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ERROR_MASK (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH1_ERROR_MASK_MAC_ERR :"
				, (u32_t)((data & NIG_LLH1_ERROR_MASK_REG_LLH1_ERROR_MASK_MAC_ERR) >> 5)
				, "\n\tLLH1_ERROR_MASK_UDP_LEN :"
				, (u32_t)((data & NIG_LLH1_ERROR_MASK_REG_LLH1_ERROR_MASK_UDP_LEN) >> 4)
				, "\n\tLLH1_ERROR_MASK_HEADER_BIG_5 :"
				, (u32_t)((data & NIG_LLH1_ERROR_MASK_REG_LLH1_ERROR_MASK_HEADER_BIG_5) >> 3)
				, "\n\tLLH1_ERROR_MASK_HEADER_LESS_5 :"
				, (u32_t)((data & NIG_LLH1_ERROR_MASK_REG_LLH1_ERROR_MASK_HEADER_LESS_5) >> 2)
				, "\n\tLLH1_ERROR_MASK_VERSION_NOT_6 :"
				, (u32_t)((data & NIG_LLH1_ERROR_MASK_REG_LLH1_ERROR_MASK_VERSION_NOT_6) >> 1)
				, "\n\tLLH1_ERROR_MASK_VERSION_NOT_4 :"
				, (u32_t)((data & NIG_LLH1_ERROR_MASK_REG_LLH1_ERROR_MASK_VERSION_NOT_4) >> 0));
			break;
		case 0x10094UL: // (NIG_REG_STAT0_MR_P: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_MR_P (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: RX statistics for port 0 of xgxs or serdes: number of mr_page\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10098UL: // (NIG_REG_STAT1_MR_P: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_MR_P (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: RX statistics for port1 of xgxs or serdes: number of mr_page\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1009cUL: // (NIG_REG_STAT0_CL73_MR_P: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_CL73_MR_P (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: RX statistics for port 0 of xgxs or serdes: number of cl73\n     mr_page\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100a0UL: // (NIG_REG_STAT1_CL73_MR_P: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_CL73_MR_P (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: RX statistics for port1 of xgxs or serdes: number of cl73\n     mr_page\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100b4UL: // (NIG_REG_PBF_LB_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PBF_LB_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for RX PBF LP IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100b8UL: // (NIG_REG_PRS_REQ_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PRS_REQ_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for RX parser request IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100bcUL: // (NIG_REG_INGRESS_UMP0_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_UMP0_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for RX UMP port 0 request IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100c0UL: // (NIG_REG_INGRESS_UMP1_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_UMP1_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for RX UMP port 1 request IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100c4UL: // (NIG_REG_BRB0_PAUSE_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BRB0_PAUSE_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for TX BRB1 pause port 0 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100c8UL: // (NIG_REG_BRB1_PAUSE_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BRB1_PAUSE_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for TX BRB1 pause port 1 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100ccUL: // (NIG_REG_EGRESS_PBF0_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_PBF0_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for TX PBF user packet from IF0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100d0UL: // (NIG_REG_EGRESS_PBF1_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_PBF1_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for TX PBF user packet from IF1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100d4UL: // (NIG_REG_EGRESS_UMP0_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_UMP0_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for TX UMP management packet port0 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100d8UL: // (NIG_REG_EGRESS_UMP1_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_UMP1_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for TX UMP management packet port1 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100dcUL: // (NIG_REG_EGRESS_DEBUG_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DEBUG_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for TX Debug packet \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100f0UL: // (NIG_REG_XCM0_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XCM0_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: output enable for RX_XCM0 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100f4UL: // (NIG_REG_XCM1_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XCM1_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: output enable for RX_XCM1 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100f8UL: // (NIG_REG_BRB0_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BRB0_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: output enable for RX BRB1 port0 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100fcUL: // (NIG_REG_BRB1_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BRB1_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: output enable for RX BRB1 port1 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10100UL: // (NIG_REG_BRB_LB_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BRB_LB_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: output enable for RX BRB1 LP IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10104UL: // (NIG_REG_PRS_EOP_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PRS_EOP_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: output enable for RX parser descriptor IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10108UL: // (NIG_REG_INGRESS_UMP0_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_UMP0_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: output enable for RX UMP port 0 response IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1010cUL: // (NIG_REG_INGRESS_UMP1_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_UMP1_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: output enable for RX UMP port 1 response IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10128UL: // (NIG_REG_LLH0_ACPI_UPON_MGMT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_UPON_MGMT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: for llh0 : 1 = if match on both WoL & mangement - set power_on\n    ; 0 - if match on both WoL & mangement - do not set power_on\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1012cUL: // (NIG_REG_LLH1_ACPI_UPON_MGMT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_UPON_MGMT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: for llh1 : 1 = if match on both WoL & mangement - set power_on\n    ; 0 - if match on both WoL & mangement - do not set power_on\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10130UL: // (NIG_REG_LLH0_XCM_MASK: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_XCM_MASK (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH0_XCM_MASK_LLFC :"
				, (u32_t)((data & NIG_LLH0_XCM_MASK_REG_LLH0_XCM_MASK_LLFC) >> 2)
				, "\n\tLLH0_XCM_MASK_MAC_FRAME :"
				, (u32_t)((data & NIG_LLH0_XCM_MASK_REG_LLH0_XCM_MASK_MAC_FRAME) >> 1)
				, "\n\tLLH0_XCM_MASK_BCN :"
				, (u32_t)((data & NIG_LLH0_XCM_MASK_REG_LLH0_XCM_MASK_BCN) >> 0));
			break;
		case 0x10134UL: // (NIG_REG_LLH1_XCM_MASK: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_XCM_MASK (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH1_XCM_MASK_LLFC :"
				, (u32_t)((data & NIG_LLH1_XCM_MASK_REG_LLH1_XCM_MASK_LLFC) >> 2)
				, "\n\tLLH1_XCM_MASK_MAC_FRAME :"
				, (u32_t)((data & NIG_LLH1_XCM_MASK_REG_LLH1_XCM_MASK_MAC_FRAME) >> 1)
				, "\n\tLLH1_XCM_MASK_BCN :"
				, (u32_t)((data & NIG_LLH1_XCM_MASK_REG_LLH1_XCM_MASK_BCN) >> 0));
			break;
		case 0x101c0UL: // (NIG_REG_LLH0_DEST_MAC_0_0: ACCESS:RW  DataWidth:0x20  Register llh0_dest_mac_0: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_MAC_0_0 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_mac_0: 1 out of 2)" \
				"\nDescription: destination MAC address 1; The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101c4UL: // (NIG_REG_LLH0_DEST_MAC_0_1: ACCESS:RW  DataWidth:0x10  Register llh0_dest_mac_0: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_MAC_0_1 (ACCESS:RW  DataWidth:0x10  Register llh0_dest_mac_0: 2 out of 2)" \
				"\nDescription: destination MAC address 1; The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101c8UL: // (NIG_REG_LLH0_DEST_MAC_1_0: ACCESS:RW  DataWidth:0x20  Register llh0_dest_mac_1: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_MAC_1_0 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_mac_1: 1 out of 2)" \
				"\nDescription: destination MAC address 2;The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101ccUL: // (NIG_REG_LLH0_DEST_MAC_1_1: ACCESS:RW  DataWidth:0x10  Register llh0_dest_mac_1: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_MAC_1_1 (ACCESS:RW  DataWidth:0x10  Register llh0_dest_mac_1: 2 out of 2)" \
				"\nDescription: destination MAC address 2;The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101d0UL: // (NIG_REG_LLH0_DEST_MAC_2_0: ACCESS:RW  DataWidth:0x20  Register llh0_dest_mac_2: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_MAC_2_0 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_mac_2: 1 out of 2)" \
				"\nDescription: destination MAC address 3;The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101d4UL: // (NIG_REG_LLH0_DEST_MAC_2_1: ACCESS:RW  DataWidth:0x10  Register llh0_dest_mac_2: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_MAC_2_1 (ACCESS:RW  DataWidth:0x10  Register llh0_dest_mac_2: 2 out of 2)" \
				"\nDescription: destination MAC address 3;The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101d8UL: // (NIG_REG_LLH0_DEST_IP_0_0: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_0: 1 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_0_0 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_0: 1 out of 4)" \
				"\nDescription: destination IP address 1;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101dcUL: // (NIG_REG_LLH0_DEST_IP_0_1: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_0: 2 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_0_1 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_0: 2 out of 4)" \
				"\nDescription: destination IP address 1;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101e0UL: // (NIG_REG_LLH0_DEST_IP_0_2: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_0: 3 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_0_2 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_0: 3 out of 4)" \
				"\nDescription: destination IP address 1;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101e4UL: // (NIG_REG_LLH0_DEST_IP_0_3: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_0: 4 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_0_3 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_0: 4 out of 4)" \
				"\nDescription: destination IP address 1;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101e8UL: // (NIG_REG_LLH0_DEST_IP_1_0: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_1: 1 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_1_0 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_1: 1 out of 4)" \
				"\nDescription: destination IP address 2;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101ecUL: // (NIG_REG_LLH0_DEST_IP_1_1: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_1: 2 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_1_1 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_1: 2 out of 4)" \
				"\nDescription: destination IP address 2;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101f0UL: // (NIG_REG_LLH0_DEST_IP_1_2: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_1: 3 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_1_2 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_1: 3 out of 4)" \
				"\nDescription: destination IP address 2;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101f4UL: // (NIG_REG_LLH0_DEST_IP_1_3: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_1: 4 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_1_3 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_1: 4 out of 4)" \
				"\nDescription: destination IP address 2;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101f8UL: // (NIG_REG_LLH0_DEST_IP_2_0: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_2: 1 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_2_0 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_2: 1 out of 4)" \
				"\nDescription: destination IP address 3;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101fcUL: // (NIG_REG_LLH0_DEST_IP_2_1: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_2: 2 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_2_1 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_2: 2 out of 4)" \
				"\nDescription: destination IP address 3;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10200UL: // (NIG_REG_LLH0_DEST_IP_2_2: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_2: 3 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_2_2 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_2: 3 out of 4)" \
				"\nDescription: destination IP address 3;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10204UL: // (NIG_REG_LLH0_DEST_IP_2_3: ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_2: 4 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_IP_2_3 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_ip_2: 4 out of 4)" \
				"\nDescription: destination IP address 3;The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10208UL: // (NIG_REG_LLH0_IPV4_IPV6_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_IPV4_IPV6_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Determine the IP version to look for in ~nig_registers_llh0_dest_ip_0\n    .llh0_dest_ip_0.  0 - IPv6; 1-IPv4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1020cUL: // (NIG_REG_LLH0_IPV4_IPV6_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_IPV4_IPV6_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Determine the IP version to look for in ~nig_registers_llh0_dest_ip_1\n    .llh0_dest_ip_1.  0 - IPv6; 1-IPv4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10210UL: // (NIG_REG_LLH0_IPV4_IPV6_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_IPV4_IPV6_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Determine the IP version to look for in ~nig_registers_llh0_dest_ip_2\n    .llh0_dest_ip_2.  0 - IPv6; 1-IPv4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10214UL: // (NIG_REG_LLH0_DEST_UDP_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_UDP_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination UDP address  1 The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10218UL: // (NIG_REG_LLH0_DEST_UDP_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_UDP_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination UDP address  2  The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1021cUL: // (NIG_REG_LLH0_DEST_UDP_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_UDP_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination UDP address  3  The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10220UL: // (NIG_REG_LLH0_DEST_TCP_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_TCP_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination TCP address  1. The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10224UL: // (NIG_REG_LLH0_DEST_TCP_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_TCP_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination TCP address  2. The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10228UL: // (NIG_REG_LLH0_DEST_TCP_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_TCP_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination TCP address  3. The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1022cUL: // (NIG_REG_LLH0_VLAN_ID_0: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_VLAN_ID_0 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: VLAN ID  1. In case of VLAN packet  the LLH will look for\n     this ID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10230UL: // (NIG_REG_LLH0_VLAN_ID_1: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_VLAN_ID_1 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: VLAN ID  2. In case of VLAN packet  the LLH will look for\n     this ID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10234UL: // (NIG_REG_LLH0_VLAN_ID_2: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_VLAN_ID_2 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: VLAN ID  3. In case of VLAN packet  the LLH will look for\n     this ID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10238UL: // (NIG_REG_LLH0_BCN_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_BCN_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: BCN EtherType. The LLH will look for this Ethertype as a \n    BCN identifier.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1023cUL: // (NIG_REG_LLH0_MCP_MASK: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_MCP_MASK (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH0_MCP_MASK_NO_VLAN :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_NO_VLAN) >> 26)
				, "\n\tLLH0_MCP_MASK_VLAN :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_VLAN) >> 25)
				, "\n\tLLH0_MCP_MASK_VLAN_ID2 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_VLAN_ID2) >> 24)
				, "\n\tLLH0_MCP_MASK_VLAN_ID1 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_VLAN_ID1) >> 23)
				, "\n\tLLH0_MCP_MASK_VLAN_ID0 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_VLAN_ID0) >> 22)
				, "\n\tLLH0_MCP_MASK_TCP2 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_TCP2) >> 21)
				, "\n\tLLH0_MCP_MASK_TCP1 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_TCP1) >> 20)
				, "\n\tLLH0_MCP_MASK_TCP0 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_TCP0) >> 19)
				, "\n\tLLH0_MCP_MASK_NTBS_T_DST :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_NTBS_T_DST) >> 18)
				, "\n\tLLH0_MCP_MASK_UDP2 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_UDP2) >> 17)
				, "\n\tLLH0_MCP_MASK_UDP1 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_UDP1) >> 16)
				, "\n\tLLH0_MCP_MASK_UDP0 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_UDP0) >> 15)
				, "\n\tLLH0_MCP_MASK_NTBS_U_DST :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_NTBS_U_DST) >> 14)
				, "\n\tLLH0_MCP_MASK_DHCP :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_DHCP) >> 13)
				, "\n\tLLH0_MCP_MASK_RMCP :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_RMCP) >> 12)
				, "\n\tLLH0_MCP_MASK_NTBS_T_SRC :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_NTBS_T_SRC) >> 11)
				, "\n\tLLH0_MCP_MASK_NTBS_U_SRC :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_NTBS_U_SRC) >> 10)
				, "\n\tLLH0_MCP_MASK_IP2 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_IP2) >> 9)
				, "\n\tLLH0_MCP_MASK_IP1 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_IP1) >> 8)
				, "\n\tLLH0_MCP_MASK_IP0 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_IP0) >> 7)
				, "\n\tLLH0_MCP_MASK_ARP :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_ARP) >> 6)
				, "\n\tLLH0_MCP_MASK_MAC2 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_MAC2) >> 5)
				, "\n\tLLH0_MCP_MASK_MAC1 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_MAC1) >> 4)
				, "\n\tLLH0_MCP_MASK_MAC0 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_MAC0) >> 3)
				, "\n\tLLH0_MCP_MASK_UNCST :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_UNCST) >> 2)
				, "\n\tLLH0_MCP_MASK_MLCST :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_MLCST) >> 1)
				, "\n\tLLH0_MCP_MASK_BRCST :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_REG_LLH0_MCP_MASK_BRCST) >> 0));
			break;
		case 0x10240UL: // (NIG_REG_LLH1_MCP_MASK: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_MCP_MASK (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH1_MCP_MASK_NO_VLAN :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_NO_VLAN) >> 26)
				, "\n\tLLH1_MCP_MASK_VLAN :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_VLAN) >> 25)
				, "\n\tLLH1_MCP_MASK_VLAN_ID2 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_VLAN_ID2) >> 24)
				, "\n\tLLH1_MCP_MASK_VLAN_ID1 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_VLAN_ID1) >> 23)
				, "\n\tLLH1_MCP_MASK_VLAN_ID0 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_VLAN_ID0) >> 22)
				, "\n\tLLH1_MCP_MASK_TCP2 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_TCP2) >> 21)
				, "\n\tLLH1_MCP_MASK_TCP1 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_TCP1) >> 20)
				, "\n\tLLH1_MCP_MASK_TCP0 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_TCP0) >> 19)
				, "\n\tLLH1_MCP_MASK_NTBS_T_DST :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_NTBS_T_DST) >> 18)
				, "\n\tLLH1_MCP_MASK_UDP2 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_UDP2) >> 17)
				, "\n\tLLH1_MCP_MASK_UDP1 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_UDP1) >> 16)
				, "\n\tLLH1_MCP_MASK_UDP0 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_UDP0) >> 15)
				, "\n\tLLH1_MCP_MASK_NTBS_U_DST :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_NTBS_U_DST) >> 14)
				, "\n\tLLH1_MCP_MASK_DHCP :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_DHCP) >> 13)
				, "\n\tLLH1_MCP_MASK_RMCP :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_RMCP) >> 12)
				, "\n\tLLH1_MCP_MASK_NTBS_T_SRC :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_NTBS_T_SRC) >> 11)
				, "\n\tLLH1_MCP_MASK_NTBS_U_SRC :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_NTBS_U_SRC) >> 10)
				, "\n\tLLH1_MCP_MASK_IP2 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_IP2) >> 9)
				, "\n\tLLH1_MCP_MASK_IP1 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_IP1) >> 8)
				, "\n\tLLH1_MCP_MASK_IP0 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_IP0) >> 7)
				, "\n\tLLH1_MCP_MASK_ARP :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_ARP) >> 6)
				, "\n\tLLH1_MCP_MASK_MAC2 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_MAC2) >> 5)
				, "\n\tLLH1_MCP_MASK_MAC1 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_MAC1) >> 4)
				, "\n\tLLH1_MCP_MASK_MAC0 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_MAC0) >> 3)
				, "\n\tLLH1_MCP_MASK_UNCST :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_UNCST) >> 2)
				, "\n\tLLH1_MCP_MASK_MLCST :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_MLCST) >> 1)
				, "\n\tLLH1_MCP_MASK_BRCST :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_REG_LLH1_MCP_MASK_BRCST) >> 0));
			break;
		case 0x10244UL: // (NIG_REG_LLH0_BRB1_DRV_MASK: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_BRB1_DRV_MASK (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_LLH_BRB1_DRV_MASK_ALLMLCST :"
				, (u32_t)((data & NIG_LLH0_BRB1_DRV_MASK_REG_P0_LLH_BRB1_DRV_MASK_ALLMLCST) >> 5)
				, "\n\tLLH0_BRB1_DRV_MASK_NO_VLAN :"
				, (u32_t)((data & NIG_LLH0_BRB1_DRV_MASK_REG_LLH0_BRB1_DRV_MASK_NO_VLAN) >> 4)
				, "\n\tLLH0_BRB1_DRV_MASK_VLAN :"
				, (u32_t)((data & NIG_LLH0_BRB1_DRV_MASK_REG_LLH0_BRB1_DRV_MASK_VLAN) >> 3)
				, "\n\tLLH0_BRB1_DRV_MASK_UNCST :"
				, (u32_t)((data & NIG_LLH0_BRB1_DRV_MASK_REG_LLH0_BRB1_DRV_MASK_UNCST) >> 2)
				, "\n\tLLH0_BRB1_DRV_MASK_MLCST :"
				, (u32_t)((data & NIG_LLH0_BRB1_DRV_MASK_REG_LLH0_BRB1_DRV_MASK_MLCST) >> 1)
				, "\n\tLLH0_BRB1_DRV_MASK_BRCST :"
				, (u32_t)((data & NIG_LLH0_BRB1_DRV_MASK_REG_LLH0_BRB1_DRV_MASK_BRCST) >> 0));
			break;
		case 0x10248UL: // (NIG_REG_LLH1_BRB1_DRV_MASK: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_BRB1_DRV_MASK (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_LLH_BRB1_DRV_MASK_ALLMLCST :"
				, (u32_t)((data & NIG_LLH1_BRB1_DRV_MASK_REG_P1_LLH_BRB1_DRV_MASK_ALLMLCST) >> 5)
				, "\n\tLLH1_BRB1_DRV_MASK_NO_VLAN :"
				, (u32_t)((data & NIG_LLH1_BRB1_DRV_MASK_REG_LLH1_BRB1_DRV_MASK_NO_VLAN) >> 4)
				, "\n\tLLH1_BRB1_DRV_MASK_VLAN :"
				, (u32_t)((data & NIG_LLH1_BRB1_DRV_MASK_REG_LLH1_BRB1_DRV_MASK_VLAN) >> 3)
				, "\n\tLLH1_BRB1_DRV_MASK_UNCST :"
				, (u32_t)((data & NIG_LLH1_BRB1_DRV_MASK_REG_LLH1_BRB1_DRV_MASK_UNCST) >> 2)
				, "\n\tLLH1_BRB1_DRV_MASK_MLCST :"
				, (u32_t)((data & NIG_LLH1_BRB1_DRV_MASK_REG_LLH1_BRB1_DRV_MASK_MLCST) >> 1)
				, "\n\tLLH1_BRB1_DRV_MASK_BRCST :"
				, (u32_t)((data & NIG_LLH1_BRB1_DRV_MASK_REG_LLH1_BRB1_DRV_MASK_BRCST) >> 0));
			break;
		case 0x1024cUL: // (NIG_REG_LLH0_BRB1_MCP_MASK: ACCESS:RW  DataWidth:0x16  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_BRB1_MCP_MASK (ACCESS:RW  DataWidth:0x16  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH0_BRB1_MCP_MASK_VLAN_ID2 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_VLAN_ID2) >> 21)
				, "\n\tLLH0_BRB1_MCP_MASK_VLAN_ID1 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_VLAN_ID1) >> 20)
				, "\n\tLLH0_BRB1_MCP_MASK_VLAN_ID0 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_VLAN_ID0) >> 19)
				, "\n\tLLH0_BRB1_MCP_MASK_TCP2 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_TCP2) >> 18)
				, "\n\tLLH0_BRB1_MCP_MASK_TCP1 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_TCP1) >> 17)
				, "\n\tLLH0_BRB1_MCP_MASK_TCP0 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_TCP0) >> 16)
				, "\n\tLLH0_BRB1_MCP_MASK_NTBIOS_TCP_DST :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_NTBIOS_TCP_DST) >> 15)
				, "\n\tLLH0_BRB1_MCP_MASK_UDP2 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_UDP2) >> 14)
				, "\n\tLLH0_BRB1_MCP_MASK_UDP1 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_UDP1) >> 13)
				, "\n\tLLH0_BRB1_MCP_MASK_UDP0 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_UDP0) >> 12)
				, "\n\tLLH0_BRB1_MCP_MASK_NTBIOS_UDP_DST :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_NTBIOS_UDP_DST) >> 11)
				, "\n\tLLH0_BRB1_MCP_MASK_DHCP :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_DHCP) >> 10)
				, "\n\tLLH0_BRB1_MCP_MASK_RMCP :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_RMCP) >> 9)
				, "\n\tLLH0_BRB1_MCP_MASK_NTBIOS_TCP_SRC :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_NTBIOS_TCP_SRC) >> 8)
				, "\n\tLLH0_BRB1_MCP_MASK_NTBIOS_UDP_SRC :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_NTBIOS_UDP_SRC) >> 7)
				, "\n\tLLH0_BRB1_MCP_MASK_IP2 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_IP2) >> 6)
				, "\n\tLLH0_BRB1_MCP_MASK_IP1 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_IP1) >> 5)
				, "\n\tLLH0_BRB1_MCP_MASK_IP0 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_IP0) >> 4)
				, "\n\tLLH0_BRB1_MCP_MASK_ARP :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_ARP) >> 3)
				, "\n\tLLH0_BRB1_MCP_MASK_MAC2 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_MAC2) >> 2)
				, "\n\tLLH0_BRB1_MCP_MASK_MAC1 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_MAC1) >> 1)
				, "\n\tLLH0_BRB1_MCP_MASK_MAC0 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_REG_LLH0_BRB1_MCP_MASK_MAC0) >> 0));
			break;
		case 0x10250UL: // (NIG_REG_LLH1_BRB1_MCP_MASK: ACCESS:RW  DataWidth:0x16  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_BRB1_MCP_MASK (ACCESS:RW  DataWidth:0x16  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH1_BRB1_MCP_MASK_VLAN_ID2 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_VLAN_ID2) >> 21)
				, "\n\tLLH1_BRB1_MCP_MASK_VLAN_ID1 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_VLAN_ID1) >> 20)
				, "\n\tLLH1_BRB1_MCP_MASK_VLAN_ID0 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_VLAN_ID0) >> 19)
				, "\n\tLLH1_BRB1_MCP_MASK_TCP2 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_TCP2) >> 18)
				, "\n\tLLH1_BRB1_MCP_MASK_TCP1 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_TCP1) >> 17)
				, "\n\tLLH1_BRB1_MCP_MASK_TCP0 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_TCP0) >> 16)
				, "\n\tLLH1_BRB1_MCP_MASK_NTBIOS_TCP_DST :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_NTBIOS_TCP_DST) >> 15)
				, "\n\tLLH1_BRB1_MCP_MASK_UDP2 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_UDP2) >> 14)
				, "\n\tLLH1_BRB1_MCP_MASK_UDP1 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_UDP1) >> 13)
				, "\n\tLLH1_BRB1_MCP_MASK_UDP0 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_UDP0) >> 12)
				, "\n\tLLH1_BRB1_MCP_MASK_NTBIOS_UDP_DST :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_NTBIOS_UDP_DST) >> 11)
				, "\n\tLLH1_BRB1_MCP_MASK_DHCP :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_DHCP) >> 10)
				, "\n\tLLH1_BRB1_MCP_MASK_RMCP :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_RMCP) >> 9)
				, "\n\tLLH1_BRB1_MCP_MASK_NTBIOS_TCP_SRC :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_NTBIOS_TCP_SRC) >> 8)
				, "\n\tLLH1_BRB1_MCP_MASK_NTBIOS_UDP_SRC :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_NTBIOS_UDP_SRC) >> 7)
				, "\n\tLLH1_BRB1_MCP_MASK_IP2 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_IP2) >> 6)
				, "\n\tLLH1_BRB1_MCP_MASK_IP1 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_IP1) >> 5)
				, "\n\tLLH1_BRB1_MCP_MASK_IP0 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_IP0) >> 4)
				, "\n\tLLH1_BRB1_MCP_MASK_ARP :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_ARP) >> 3)
				, "\n\tLLH1_BRB1_MCP_MASK_MAC2 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_MAC2) >> 2)
				, "\n\tLLH1_BRB1_MCP_MASK_MAC1 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_MAC1) >> 1)
				, "\n\tLLH1_BRB1_MCP_MASK_MAC0 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_REG_LLH1_BRB1_MCP_MASK_MAC0) >> 0));
			break;
		case 0x10254UL: // (NIG_REG_LLH0_ACPI_VLAN_STRIP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_VLAN_STRIP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Remove VLAN  before calculating ACPI pattern.  This bit is\n     replaced by llh_acpi_tag_rm in E2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10258UL: // (NIG_REG_LLH1_ACPI_VLAN_STRIP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_VLAN_STRIP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: remove VLAN  before calculating ACPI pattern.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1025cUL: // (NIG_REG_LLH0_BRB1_NOT_MCP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_BRB1_NOT_MCP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: send to BRB1 if no match on any of RMP rules.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10260UL: // (NIG_REG_LLH1_DEST_MAC_0_0: ACCESS:RW  DataWidth:0x20  Register llh1_dest_mac_0: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_MAC_0_0 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_mac_0: 1 out of 2)" \
				"\nDescription: destination MAC address 1; The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10264UL: // (NIG_REG_LLH1_DEST_MAC_0_1: ACCESS:RW  DataWidth:0x10  Register llh1_dest_mac_0: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_MAC_0_1 (ACCESS:RW  DataWidth:0x10  Register llh1_dest_mac_0: 2 out of 2)" \
				"\nDescription: destination MAC address 1; The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10268UL: // (NIG_REG_LLH1_DEST_MAC_1_0: ACCESS:RW  DataWidth:0x20  Register llh1_dest_mac_1: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_MAC_1_0 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_mac_1: 1 out of 2)" \
				"\nDescription: destination MAC address 2; The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1026cUL: // (NIG_REG_LLH1_DEST_MAC_1_1: ACCESS:RW  DataWidth:0x10  Register llh1_dest_mac_1: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_MAC_1_1 (ACCESS:RW  DataWidth:0x10  Register llh1_dest_mac_1: 2 out of 2)" \
				"\nDescription: destination MAC address 2; The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10270UL: // (NIG_REG_LLH1_DEST_MAC_2_0: ACCESS:RW  DataWidth:0x20  Register llh1_dest_mac_2: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_MAC_2_0 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_mac_2: 1 out of 2)" \
				"\nDescription: destination MAC address 3; The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10274UL: // (NIG_REG_LLH1_DEST_MAC_2_1: ACCESS:RW  DataWidth:0x10  Register llh1_dest_mac_2: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_MAC_2_1 (ACCESS:RW  DataWidth:0x10  Register llh1_dest_mac_2: 2 out of 2)" \
				"\nDescription: destination MAC address 3; The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10278UL: // (NIG_REG_LLH1_DEST_IP_0_0: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_0: 1 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_0_0 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_0: 1 out of 4)" \
				"\nDescription: destination IP address 1; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1027cUL: // (NIG_REG_LLH1_DEST_IP_0_1: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_0: 2 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_0_1 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_0: 2 out of 4)" \
				"\nDescription: destination IP address 1; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10280UL: // (NIG_REG_LLH1_DEST_IP_0_2: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_0: 3 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_0_2 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_0: 3 out of 4)" \
				"\nDescription: destination IP address 1; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10284UL: // (NIG_REG_LLH1_DEST_IP_0_3: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_0: 4 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_0_3 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_0: 4 out of 4)" \
				"\nDescription: destination IP address 1; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10288UL: // (NIG_REG_LLH1_DEST_IP_1_0: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_1: 1 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_1_0 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_1: 1 out of 4)" \
				"\nDescription: destination IP address 2; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1028cUL: // (NIG_REG_LLH1_DEST_IP_1_1: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_1: 2 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_1_1 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_1: 2 out of 4)" \
				"\nDescription: destination IP address 2; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10290UL: // (NIG_REG_LLH1_DEST_IP_1_2: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_1: 3 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_1_2 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_1: 3 out of 4)" \
				"\nDescription: destination IP address 2; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10294UL: // (NIG_REG_LLH1_DEST_IP_1_3: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_1: 4 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_1_3 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_1: 4 out of 4)" \
				"\nDescription: destination IP address 2; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10298UL: // (NIG_REG_LLH1_DEST_IP_2_0: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_2: 1 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_2_0 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_2: 1 out of 4)" \
				"\nDescription: destination IP address 3; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1029cUL: // (NIG_REG_LLH1_DEST_IP_2_1: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_2: 2 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_2_1 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_2: 2 out of 4)" \
				"\nDescription: destination IP address 3; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102a0UL: // (NIG_REG_LLH1_DEST_IP_2_2: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_2: 3 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_2_2 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_2: 3 out of 4)" \
				"\nDescription: destination IP address 3; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102a4UL: // (NIG_REG_LLH1_DEST_IP_2_3: ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_2: 4 out of 4) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_IP_2_3 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_ip_2: 4 out of 4)" \
				"\nDescription: destination IP address 3; The LLH will look for this address\n     in all incoming packets. In case of IPv4 use only 32lsb.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102a8UL: // (NIG_REG_LLH1_IPV4_IPV6_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_IPV4_IPV6_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Determine the IP version to look for in ~nig_registers_llh0_dest_ip_0\n    .llh0_dest_ip_0.  0 - IPv6; 1-IPv4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102acUL: // (NIG_REG_LLH1_IPV4_IPV6_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_IPV4_IPV6_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Determine the IP version to look for in ~nig_registers_llh0_dest_ip_1\n    .llh0_dest_ip_1.  0 - IPv6; 1-IPv4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102b0UL: // (NIG_REG_LLH1_IPV4_IPV6_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_IPV4_IPV6_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Determine the IP version to look for in ~nig_registers_llh0_dest_ip_2\n    .llh0_dest_ip_2.  0 - IPv6; 1-IPv4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102b4UL: // (NIG_REG_LLH1_DEST_UDP_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_UDP_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination UDP address  1;The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102b8UL: // (NIG_REG_LLH1_DEST_UDP_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_UDP_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination UDP address  2 ;The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102bcUL: // (NIG_REG_LLH1_DEST_UDP_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_UDP_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination UDP address  3 ;The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102c0UL: // (NIG_REG_LLH1_DEST_TCP_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_TCP_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination TCP address  1. The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102c4UL: // (NIG_REG_LLH1_DEST_TCP_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_TCP_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination TCP address  2. The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102c8UL: // (NIG_REG_LLH1_DEST_TCP_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_TCP_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: destination TCP address  3. The LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102ccUL: // (NIG_REG_LLH1_VLAN_ID_0: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_VLAN_ID_0 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: VLAN ID  1. In case of VLAN packet; the LLH will look for\n     this ID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102d0UL: // (NIG_REG_LLH1_VLAN_ID_1: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_VLAN_ID_1 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: VLAN ID  2. In case of VLAN packet; the LLH will look for\n     this ID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102d4UL: // (NIG_REG_LLH1_VLAN_ID_2: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_VLAN_ID_2 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: VLAN ID  3. In case of VLAN packet; the LLH will look for\n     this ID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102d8UL: // (NIG_REG_LLH1_BCN_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_BCN_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: BCN EtherType. The LLH will look for this Ethertype as a \n    BCN identifier.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102dcUL: // (NIG_REG_LLH1_BRB1_NOT_MCP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_BRB1_NOT_MCP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: send to BRB1 if no match on any of RMP rules.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102f0UL: // (NIG_REG_LED_MODE_P0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_LED_MODE_P0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: led mode for port0: 0 MAC; 1-3 PHY1; 4 MAC2; 5-7 PHY4; 8-\n    MAC3; 9-11PHY7; 12 MAC4; 13-15 PHY10;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102f4UL: // (NIG_REG_LED_MODE_P1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_LED_MODE_P1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: led mode for port1: 0 MAC; 1-3 PHY1; 4 MAC2; 5-7 PHY4; 8-\n    MAC3; 9-11PHY7; 12 MAC4; 13-15 PHY10;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102f8UL: // (NIG_REG_LED_CONTROL_OVERRIDE_TRAFFIC_P0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_CONTROL_OVERRIDE_TRAFFIC_P0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Port0: If set overrides hardware control of the Traffic LED\n    . The Traffic LED will then be controlled via bit ~nig_registers_ led_control_traffic_p0\n    .led_control_traffic_p0 and bit ~nig_registers_led_control_blink_traffic_p0\n    .led_control_blink_traffic_p0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102fcUL: // (NIG_REG_LED_CONTROL_OVERRIDE_TRAFFIC_P1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_CONTROL_OVERRIDE_TRAFFIC_P1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Port1: If set overrides hardware control of the Traffic LED\n    . The Traffic LED will then be controlled via bit ~nig_registers_led_control_traffic_p1\n    .led_control_traffic_p1 and bit ~nig_registers_led_control_blink_traffic_p1\n    .led_control_blink_traffic_p1. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10300UL: // (NIG_REG_LED_CONTROL_TRAFFIC_P0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_CONTROL_TRAFFIC_P0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Port0: If set along with the led_control_override_trafic_p0\n     bit; turns on the Traffic LED. If the led_control_blink_traffic_p0 bit\n     is also set; the LED will blink with blink rate specified in ~nig_registers_led_control_blink_rate_p0\n    .led_control_blink_rate_p0 and ~nig_regsters_led_control_blink_rate_ena_p0\n    .led_control_blink_rate_ena_p0 fields. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10304UL: // (NIG_REG_LED_CONTROL_TRAFFIC_P1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_CONTROL_TRAFFIC_P1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Port1: If set along with the ~nig_registers_led_control_override_trafic_p0\n    .led_control_override_trafic_p0 bit; turns on the Traffic LED. If the ~\n    nig_registers_led_control_blink_traffic_p0.led_control_blink_traffic_p0\n     bit is also set; the LED will blink with blink rate specified in ~nig_registers_led_control_blink_rate_p1\n    .led_control_blink_rate_p1 and ~nig_registers_led_control_blink_rate_ena_p1\n    .led_control_blink_rate_ena_p1 fields. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10308UL: // (NIG_REG_LED_CONTROL_BLINK_TRAFFIC_P0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_CONTROL_BLINK_TRAFFIC_P0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Port0: If set along with the ~nig_registers_led_control_override_traffic_p0\n    .led_control_override_traffic_p0 bit and ~nig_registers_led_control_traffic_p0\n    .led_control_traffic_p0 LED bit; the Traffic LED will blink with the blink\n     rate specified in ~nig_registers_led_control_blink_rate_p0.led_control_blink_rate_p0\n     and ~nig_registers_led_control_blink_rate_ena_p0.led_control_blink_rate_ena_p0\n     fields. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030cUL: // (NIG_REG_LED_CONTROL_BLINK_TRAFFIC_P1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_CONTROL_BLINK_TRAFFIC_P1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Port1: If set along with the ~nig_registers_led_control_override_traffic_p0\n    .led_control_override_traffic_p0 bit and ~nig_registers_led_control_traffic_p0\n    .led_control_traffic_p0 LED bit; the Traffic LED will blink with the blink\n     rate specified in ~nig_registers_led_control_blink_rate_p1.led_control_blink_rate_p1\n     and ~nig_registers_led_control_blink_rate_ena_p1. led_control_blink_rate_ena_p1\n     fields. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10310UL: // (NIG_REG_LED_CONTROL_BLINK_RATE_P0: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_LED_CONTROL_BLINK_RATE_P0 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Port0: Specifies the period of each blink cycle (on + off\n    ) for Traffic LED in milliseconds. Must be a non-zero value. This 12-bit\n     field is reset to 0x080; giving a default blink period of approximately\n     8Hz. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10314UL: // (NIG_REG_LED_CONTROL_BLINK_RATE_P1: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_LED_CONTROL_BLINK_RATE_P1 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Port1: Specifies the period of each blink cycle (on + off\n    ) for Traffic LED in milliseconds. Must be a non-zero value. This 12-bit\n     field is reset to 0x080; giving a default blink period of approximately\n     8Hz. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10318UL: // (NIG_REG_LED_CONTROL_BLINK_RATE_ENA_P0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_CONTROL_BLINK_RATE_ENA_P0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Port0: This bit is set to enable the use of the ~nig_registers_led_control_blink_rate_p0\n    .led_control_blink_rate_p0 field defined below. If this bit is cleared\n    ; then the blink rate will be about 8Hz. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1031cUL: // (NIG_REG_LED_CONTROL_BLINK_RATE_ENA_P1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_CONTROL_BLINK_RATE_ENA_P1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Port1: This bit is set to enable the use of the ~nig_registers_led_control_blink_rate_p1\n    .led_control_blink_rate_p1 field defined below. If this bit is cleared\n    ; then the blink rate will be about 8Hz. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10320UL: // (NIG_REG_LED_10G_P0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_10G_P0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: led 10g for port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10324UL: // (NIG_REG_LED_10G_P1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_10G_P1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: led 10g for port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10328UL: // (NIG_REG_STATUS_INTERRUPT_PORT0: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_STATUS_INTERRUPT_PORT0 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTATUS_XGXS0_MAC_CRS :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_XGXS0_MAC_CRS) >> 26)
				, "\n\tSTATUS_XGXS0_RX_SIGDET :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_XGXS0_RX_SIGDET) >> 25)
				, "\n\tSTATUS_XGXS0_CL73_MR_PAGE_RX :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_XGXS0_CL73_MR_PAGE_RX) >> 24)
				, "\n\tSTATUS_XGXS0_CL73_AN_COMPLETE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_XGXS0_CL73_AN_COMPLETE) >> 23)
				, "\n\tSTATUS_XGXS0_MR_PAGE_RX :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_XGXS0_MR_PAGE_RX) >> 22)
				, "\n\tSTATUS_XGXS0_LINK_STATUS :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_XGXS0_LINK_STATUS) >> 18)
				, "\n\tSTATUS_XGXS0_FIBER_RXACT :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_XGXS0_FIBER_RXACT) >> 17)
				, "\n\tSTATUS_XGXS0_AUTONEG_COMPLETE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_XGXS0_AUTONEG_COMPLETE) >> 16)
				, "\n\tSTATUS_XGXS0_LINK10G :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_XGXS0_LINK10G) >> 15)
				, "\n\tSTATUS_XGXS0_REMOTEMDIOREQ :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_XGXS0_REMOTEMDIOREQ) >> 14)
				, "\n\tSTATUS_SERDES0_RX_SIGDET :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_SERDES0_RX_SIGDET) >> 13)
				, "\n\tSTATUS_SERDES0_CL73_MR_PAGE_RX :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_SERDES0_CL73_MR_PAGE_RX) >> 12)
				, "\n\tSTATUS_SERDES0_CL73_AN_COMPLETE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_SERDES0_CL73_AN_COMPLETE) >> 11)
				, "\n\tSTATUS_SERDES0_MR_PAGE_RX :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_SERDES0_MR_PAGE_RX) >> 10)
				, "\n\tSTATUS_SERDES0_LINK_STATUS :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_SERDES0_LINK_STATUS) >> 9)
				, "\n\tSTATUS_SERDES0_FIBER_RXACT :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_SERDES0_FIBER_RXACT) >> 8)
				, "\n\tSTATUS_SERDES0_AUTONEG_COMPLETE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_SERDES0_AUTONEG_COMPLETE) >> 7)
				, "\n\tSTATUS_SERDES0_MAC_CRS :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_SERDES0_MAC_CRS) >> 6)
				, "\n\tSTATUS_EMAC0_MISC_ATTN :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_EMAC0_MISC_ATTN) >> 5)
				, "\n\tSTATUS_EMAC0_MISC_LINK_CHANGE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_EMAC0_MISC_LINK_CHANGE) >> 4)
				, "\n\tSTATUS_EMAC0_MISC_LINK_STATUS :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_EMAC0_MISC_LINK_STATUS) >> 3)
				, "\n\tSTATUS_EMAC0_MISC_CFG_CHANGE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_EMAC0_MISC_CFG_CHANGE) >> 2)
				, "\n\tSTATUS_EMAC0_MISC_MI_COMPLETE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_EMAC0_MISC_MI_COMPLETE) >> 1)
				, "\n\tSTATUS_EMAC0_MISC_MI_INT :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT0_REG_STATUS_EMAC0_MISC_MI_INT) >> 0));
			break;
		case 0x1032cUL: // (NIG_REG_STATUS_INTERRUPT_PORT1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_STATUS_INTERRUPT_PORT1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTATUS_XGXS1_MAC_CRS :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_XGXS1_MAC_CRS) >> 26)
				, "\n\tSTATUS_XGXS1_RX_SIGDET :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_XGXS1_RX_SIGDET) >> 25)
				, "\n\tSTATUS_XGXS1_CL73_MR_PAGE_RX :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_XGXS1_CL73_MR_PAGE_RX) >> 24)
				, "\n\tSTATUS_XGXS1_CL73_AN_COMPLETE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_XGXS1_CL73_AN_COMPLETE) >> 23)
				, "\n\tSTATUS_XGXS1_MR_PAGE_RX :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_XGXS1_MR_PAGE_RX) >> 22)
				, "\n\tSTATUS_XGXS1_LINK_STATUS :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_XGXS1_LINK_STATUS) >> 18)
				, "\n\tSTATUS_XGXS1_FIBER_RXACT :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_XGXS1_FIBER_RXACT) >> 17)
				, "\n\tSTATUS_XGXS1_AUTONEG_COMPLETE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_XGXS1_AUTONEG_COMPLETE) >> 16)
				, "\n\tSTATUS_XGXS1_LINK10G :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_XGXS1_LINK10G) >> 15)
				, "\n\tSTATUS_XGXS1_REMOTEMDIOREQ :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_XGXS1_REMOTEMDIOREQ) >> 14)
				, "\n\tSTATUS_SERDES1_RX_SIGDET :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_SERDES1_RX_SIGDET) >> 13)
				, "\n\tSTATUS_SERDES1_CL73_MR_PAGE_RX :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_SERDES1_CL73_MR_PAGE_RX) >> 12)
				, "\n\tSTATUS_SERDES1_CL73_AN_COMPLETE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_SERDES1_CL73_AN_COMPLETE) >> 11)
				, "\n\tSTATUS_SERDES1_MR_PAGE_RX :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_SERDES1_MR_PAGE_RX) >> 10)
				, "\n\tSTATUS_SERDES1_LINK_STATUS :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_SERDES1_LINK_STATUS) >> 9)
				, "\n\tSTATUS_SERDES1_FIBER_RXACT :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_SERDES1_FIBER_RXACT) >> 8)
				, "\n\tSTATUS_SERDES1_AUTONEG_COMPLETE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_SERDES1_AUTONEG_COMPLETE) >> 7)
				, "\n\tSTATUS_SERDES1_MAC_CRS :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_SERDES1_MAC_CRS) >> 6)
				, "\n\tSTATUS_EMAC1_MISC_ATTN :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_EMAC1_MISC_ATTN) >> 5)
				, "\n\tSTATUS_EMAC1_MISC_LINK_CHANGE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_EMAC1_MISC_LINK_CHANGE) >> 4)
				, "\n\tSTATUS_EMAC1_MISC_LINK_STATUS :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_EMAC1_MISC_LINK_STATUS) >> 3)
				, "\n\tSTATUS_EMAC1_MISC_CFG_CHANGE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_EMAC1_MISC_CFG_CHANGE) >> 2)
				, "\n\tSTATUS_EMAC1_MISC_MI_COMPLETE :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_EMAC1_MISC_MI_COMPLETE) >> 1)
				, "\n\tSTATUS_EMAC1_MISC_MI_INT :"
				, (u32_t)((data & NIG_STATUS_INTERRUPT_PORT1_REG_STATUS_EMAC1_MISC_MI_INT) >> 0));
			break;
		case 0x10330UL: // (NIG_REG_MASK_INTERRUPT_PORT0: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_MASK_INTERRUPT_PORT0 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMASK_XGXS0_MAC_CRS :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_XGXS0_MAC_CRS) >> 26)
				, "\n\tMASK_XGXS0_RX_SIGDET :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_XGXS0_RX_SIGDET) >> 25)
				, "\n\tMASK_XGXS0_CL73_MR_PAGE_RX :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_XGXS0_CL73_MR_PAGE_RX) >> 24)
				, "\n\tMASK_XGXS0_CL73_AN_COMPLETE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_XGXS0_CL73_AN_COMPLETE) >> 23)
				, "\n\tMASK_XGXS0_MR_PAGE_RX :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_XGXS0_MR_PAGE_RX) >> 22)
				, "\n\tMASK_XGXS0_LINK_STATUS :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_XGXS0_LINK_STATUS) >> 18)
				, "\n\tMASK_XGXS0_FIBER_RXACT :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_XGXS0_FIBER_RXACT) >> 17)
				, "\n\tMASK_XGXS0_AUTONEG_COMPLETE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_XGXS0_AUTONEG_COMPLETE) >> 16)
				, "\n\tMASK_XGXS0_LINK10G :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_XGXS0_LINK10G) >> 15)
				, "\n\tMASK_XGXS0_REMOTEMDIOREQ :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_XGXS0_REMOTEMDIOREQ) >> 14)
				, "\n\tMASK_SERDES0_RX_SIGDET :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_SERDES0_RX_SIGDET) >> 13)
				, "\n\tMASK_SERDES0_CL73_MR_PAGE_RX :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_SERDES0_CL73_MR_PAGE_RX) >> 12)
				, "\n\tMASK_SERDES0_CL73_AN_COMPLETE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_SERDES0_CL73_AN_COMPLETE) >> 11)
				, "\n\tMASK_SERDES0_MR_PAGE_RX :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_SERDES0_MR_PAGE_RX) >> 10)
				, "\n\tMASK_SERDES0_LINK_STATUS :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_SERDES0_LINK_STATUS) >> 9)
				, "\n\tMASK_SERDES0_FIBER_RXACT :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_SERDES0_FIBER_RXACT) >> 8)
				, "\n\tMASK_SERDES0_AUTONEG_COMPLETE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_SERDES0_AUTONEG_COMPLETE) >> 7)
				, "\n\tMASK_SERDES0_MAC_CRS :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_SERDES0_MAC_CRS) >> 6)
				, "\n\tMASK_EMAC0_MISC_ATTN :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_EMAC0_MISC_ATTN) >> 5)
				, "\n\tMASK_EMAC0_MISC_LINK_CHANGE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_EMAC0_MISC_LINK_CHANGE) >> 4)
				, "\n\tMASK_EMAC0_MISC_LINK_STATUS :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_EMAC0_MISC_LINK_STATUS) >> 3)
				, "\n\tMASK_EMAC0_MISC_CFG_CHANGE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_EMAC0_MISC_CFG_CHANGE) >> 2)
				, "\n\tMASK_EMAC0_MISC_MI_COMPLETE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_EMAC0_MISC_MI_COMPLETE) >> 1)
				, "\n\tMASK_EMAC0_MISC_MI_INT :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT0_REG_MASK_EMAC0_MISC_MI_INT) >> 0));
			break;
		case 0x10334UL: // (NIG_REG_MASK_INTERRUPT_PORT1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_MASK_INTERRUPT_PORT1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMASK_XGXS1_MAC_CRS :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_XGXS1_MAC_CRS) >> 26)
				, "\n\tMASK_XGXS1_RX_SIGDET :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_XGXS1_RX_SIGDET) >> 25)
				, "\n\tMASK_XGXS1_CL73_MR_PAGE_RX :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_XGXS1_CL73_MR_PAGE_RX) >> 24)
				, "\n\tMASK_XGXS1_CL73_AN_COMPLETE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_XGXS1_CL73_AN_COMPLETE) >> 23)
				, "\n\tMASK_XGXS1_MR_PAGE_RX :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_XGXS1_MR_PAGE_RX) >> 22)
				, "\n\tMASK_XGXS1_LINK_STATUS :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_XGXS1_LINK_STATUS) >> 18)
				, "\n\tMASK_XGXS1_FIBER_RXACT :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_XGXS1_FIBER_RXACT) >> 17)
				, "\n\tMASK_XGXS1_AUTONEG_COMPLETE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_XGXS1_AUTONEG_COMPLETE) >> 16)
				, "\n\tMASK_XGXS1_LINK10G :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_XGXS1_LINK10G) >> 15)
				, "\n\tMASK_XGXS1_REMOTEMDIOREQ :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_XGXS1_REMOTEMDIOREQ) >> 14)
				, "\n\tMASK_SERDES1_RX_SIGDET :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_SERDES1_RX_SIGDET) >> 13)
				, "\n\tMASK_SERDES1_CL73_MR_PAGE_RX :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_SERDES1_CL73_MR_PAGE_RX) >> 12)
				, "\n\tMASK_SERDES1_CL73_AN_COMPLETE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_SERDES1_CL73_AN_COMPLETE) >> 11)
				, "\n\tMASK_SERDES1_MR_PAGE_RX :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_SERDES1_MR_PAGE_RX) >> 10)
				, "\n\tMASK_SERDES1_LINK_STATUS :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_SERDES1_LINK_STATUS) >> 9)
				, "\n\tMASK_SERDES1_FIBER_RXACT :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_SERDES1_FIBER_RXACT) >> 8)
				, "\n\tMASK_SERDES1_AUTONEG_COMPLETE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_SERDES1_AUTONEG_COMPLETE) >> 7)
				, "\n\tMASK_SERDES1_MAC_CRS :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_SERDES1_MAC_CRS) >> 6)
				, "\n\tMASK_EMAC1_MISC_ATTN :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_EMAC1_MISC_ATTN) >> 5)
				, "\n\tMASK_EMAC1_MISC_LINK_CHANGE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_EMAC1_MISC_LINK_CHANGE) >> 4)
				, "\n\tMASK_EMAC1_MISC_LINK_STATUS :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_EMAC1_MISC_LINK_STATUS) >> 3)
				, "\n\tMASK_EMAC1_MISC_CFG_CHANGE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_EMAC1_MISC_CFG_CHANGE) >> 2)
				, "\n\tMASK_EMAC1_MISC_MI_COMPLETE :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_EMAC1_MISC_MI_COMPLETE) >> 1)
				, "\n\tMASK_EMAC1_MISC_MI_INT :"
				, (u32_t)((data & NIG_MASK_INTERRUPT_PORT1_REG_MASK_EMAC1_MISC_MI_INT) >> 0));
			break;
		case 0x10388UL: // (NIG_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from NIG\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1038cUL: // (NIG_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from NIG\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10390UL: // (NIG_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nNIG_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from NIG\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10394UL: // (NIG_REG_PORT_SWAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PORT_SWAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Value of this register will be transmitted to port swap when\n     ~nig_registers_strap_override.strap_override =1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10398UL: // (NIG_REG_STRAP_OVERRIDE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_STRAP_OVERRIDE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: port swap mux selection. If  this register equal to 0  then\n     port swap is equal to SPIO pin that inputs from ifmux_serdes_swap. If\n      1 then ort swap is equal to ~nig_registers_port_swap.port_swap\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1039cUL: // (NIG_REG_SEL_DBG_IFMUX_TEST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SEL_DBG_IFMUX_TEST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: selection to mux from dbg block for output to ifmux.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103a0UL: // (NIG_REG_SEL_MUX_DBG_VECTOR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_SEL_MUX_DBG_VECTOR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: selection of vector: 0 - from emac0; 1-from emac1; 2 - from\n     msp; 3 - NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103a4UL: // (NIG_REG_SEL_MUX_DBG_VECTOR_NUM: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SEL_MUX_DBG_VECTOR_NUM (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 0 - selection of vector: from port 0; 1 - from port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103a8UL: // (NIG_REG_DEBUG_SEL_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_DEBUG_SEL_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Debug only. This control is outputted to EMAC0  EMAC1 and\n     MCP. It is used as a select for the debug mux sel_1 logic which exists\n     in these blocks; it controls which signals are to be outputted on the\n     Debug 1 interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103acUL: // (NIG_REG_DEBUG_SEL_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_DEBUG_SEL_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Debug only. This control is outputted to EMAC0  EMAC1 and\n     MCP. It is used as a select for the debug mux sel_2 logic which exists\n     in these blocks; it controls which signals are to be outputted on the\n     Debug 2 interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103b0UL: // (NIG_REG_NIG_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLFC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_LLFC1_POP_ERROR) >> 31)
				, "\n\tLLFC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_LLFC0_POP_ERROR) >> 30)
				, "\n\tLLH1_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_LLH1_FIFO_ERROR) >> 29)
				, "\n\tLLH0_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_LLH0_FIFO_ERROR) >> 28)
				, "\n\tTIMER1_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_TIMER1_MAX_INT) >> 27)
				, "\n\tTIMER0_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_TIMER0_MAX_INT) >> 26)
				, "\n\tEGRESS_BMAC1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_BMAC1_ERROR) >> 25)
				, "\n\tEGRESS_BMAC0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_BMAC0_ERROR) >> 24)
				, "\n\tEGRESS_EMAC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_EMAC1_POP_ERROR) >> 23)
				, "\n\tEGRESS_EMAC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_EMAC0_POP_ERROR) >> 22)
				, "\n\tEGRESS_EMAC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_EMAC1_PUSH_ERROR) >> 21)
				, "\n\tEGRESS_EMAC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_EMAC0_PUSH_ERROR) >> 20)
				, "\n\tEGRESS_DELAY1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_DELAY1_ERROR) >> 19)
				, "\n\tEGRESS_DELAY0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_DELAY0_ERROR) >> 18)
				, "\n\tEGRESS_DEBUG_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_DEBUG_FIFO_ERROR) >> 17)
				, "\n\tEGRESS_MNG1_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_MNG1_FIFO_ERROR) >> 16)
				, "\n\tEGRESS_MNG0_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_EGRESS_MNG0_FIFO_ERROR) >> 15)
				, "\n\tINGRESS_LB_PBF_DELAY_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_LB_PBF_DELAY_ERROR) >> 14)
				, "\n\tINGRESS_EMAC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_EMAC1_PUSH_ERROR) >> 13)
				, "\n\tINGRESS_EMAC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_EMAC0_PUSH_ERROR) >> 12)
				, "\n\tINGRESS_EMAC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_EMAC1_POP_ERROR) >> 11)
				, "\n\tINGRESS_EMAC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_EMAC0_POP_ERROR) >> 10)
				, "\n\tINGRESS_BMAC1_REGS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_BMAC1_REGS_ERROR) >> 9)
				, "\n\tINGRESS_BMAC0_REGS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_BMAC0_REGS_ERROR) >> 8)
				, "\n\tINGRESS_BMAC1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_BMAC1_ERROR) >> 7)
				, "\n\tINGRESS_BMAC0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_BMAC0_ERROR) >> 6)
				, "\n\tINGRESS_RMP1_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_RMP1_DSCR_FIFO_ERROR) >> 5)
				, "\n\tINGRESS_RMP0_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_RMP0_DSCR_FIFO_ERROR) >> 4)
				, "\n\tINGRESS_EOP_LB_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_EOP_LB_ERROR) >> 3)
				, "\n\tINGRESS_EOP_PORT1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_EOP_PORT1_ERROR) >> 2)
				, "\n\tINGRESS_EOP_PORT0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_INGRESS_EOP_PORT0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x103b4UL: // (NIG_REG_NIG_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLFC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_LLFC1_POP_ERROR) >> 31)
				, "\n\tLLFC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_LLFC0_POP_ERROR) >> 30)
				, "\n\tLLH1_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_LLH1_FIFO_ERROR) >> 29)
				, "\n\tLLH0_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_LLH0_FIFO_ERROR) >> 28)
				, "\n\tTIMER1_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_TIMER1_MAX_INT) >> 27)
				, "\n\tTIMER0_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_TIMER0_MAX_INT) >> 26)
				, "\n\tEGRESS_BMAC1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_BMAC1_ERROR) >> 25)
				, "\n\tEGRESS_BMAC0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_BMAC0_ERROR) >> 24)
				, "\n\tEGRESS_EMAC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_EMAC1_POP_ERROR) >> 23)
				, "\n\tEGRESS_EMAC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_EMAC0_POP_ERROR) >> 22)
				, "\n\tEGRESS_EMAC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_EMAC1_PUSH_ERROR) >> 21)
				, "\n\tEGRESS_EMAC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_EMAC0_PUSH_ERROR) >> 20)
				, "\n\tEGRESS_DELAY1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_DELAY1_ERROR) >> 19)
				, "\n\tEGRESS_DELAY0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_DELAY0_ERROR) >> 18)
				, "\n\tEGRESS_DEBUG_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_DEBUG_FIFO_ERROR) >> 17)
				, "\n\tEGRESS_MNG1_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_MNG1_FIFO_ERROR) >> 16)
				, "\n\tEGRESS_MNG0_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_EGRESS_MNG0_FIFO_ERROR) >> 15)
				, "\n\tINGRESS_LB_PBF_DELAY_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_LB_PBF_DELAY_ERROR) >> 14)
				, "\n\tINGRESS_EMAC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_EMAC1_PUSH_ERROR) >> 13)
				, "\n\tINGRESS_EMAC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_EMAC0_PUSH_ERROR) >> 12)
				, "\n\tINGRESS_EMAC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_EMAC1_POP_ERROR) >> 11)
				, "\n\tINGRESS_EMAC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_EMAC0_POP_ERROR) >> 10)
				, "\n\tINGRESS_BMAC1_REGS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_BMAC1_REGS_ERROR) >> 9)
				, "\n\tINGRESS_BMAC0_REGS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_BMAC0_REGS_ERROR) >> 8)
				, "\n\tINGRESS_BMAC1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_BMAC1_ERROR) >> 7)
				, "\n\tINGRESS_BMAC0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_BMAC0_ERROR) >> 6)
				, "\n\tINGRESS_RMP1_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_RMP1_DSCR_FIFO_ERROR) >> 5)
				, "\n\tINGRESS_RMP0_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_RMP0_DSCR_FIFO_ERROR) >> 4)
				, "\n\tINGRESS_EOP_LB_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_EOP_LB_ERROR) >> 3)
				, "\n\tINGRESS_EOP_PORT1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_EOP_PORT1_ERROR) >> 2)
				, "\n\tINGRESS_EOP_PORT0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_INGRESS_EOP_PORT0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x103b8UL: // (NIG_REG_NIG_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLFC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_LLFC1_POP_ERROR) >> 31)
				, "\n\tLLFC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_LLFC0_POP_ERROR) >> 30)
				, "\n\tLLH1_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_LLH1_FIFO_ERROR) >> 29)
				, "\n\tLLH0_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_LLH0_FIFO_ERROR) >> 28)
				, "\n\tTIMER1_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_TIMER1_MAX_INT) >> 27)
				, "\n\tTIMER0_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_TIMER0_MAX_INT) >> 26)
				, "\n\tEGRESS_BMAC1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_BMAC1_ERROR) >> 25)
				, "\n\tEGRESS_BMAC0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_BMAC0_ERROR) >> 24)
				, "\n\tEGRESS_EMAC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_EMAC1_POP_ERROR) >> 23)
				, "\n\tEGRESS_EMAC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_EMAC0_POP_ERROR) >> 22)
				, "\n\tEGRESS_EMAC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_EMAC1_PUSH_ERROR) >> 21)
				, "\n\tEGRESS_EMAC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_EMAC0_PUSH_ERROR) >> 20)
				, "\n\tEGRESS_DELAY1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_DELAY1_ERROR) >> 19)
				, "\n\tEGRESS_DELAY0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_DELAY0_ERROR) >> 18)
				, "\n\tEGRESS_DEBUG_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_DEBUG_FIFO_ERROR) >> 17)
				, "\n\tEGRESS_MNG1_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_MNG1_FIFO_ERROR) >> 16)
				, "\n\tEGRESS_MNG0_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_EGRESS_MNG0_FIFO_ERROR) >> 15)
				, "\n\tINGRESS_LB_PBF_DELAY_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_LB_PBF_DELAY_ERROR) >> 14)
				, "\n\tINGRESS_EMAC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_EMAC1_PUSH_ERROR) >> 13)
				, "\n\tINGRESS_EMAC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_EMAC0_PUSH_ERROR) >> 12)
				, "\n\tINGRESS_EMAC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_EMAC1_POP_ERROR) >> 11)
				, "\n\tINGRESS_EMAC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_EMAC0_POP_ERROR) >> 10)
				, "\n\tINGRESS_BMAC1_REGS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_BMAC1_REGS_ERROR) >> 9)
				, "\n\tINGRESS_BMAC0_REGS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_BMAC0_REGS_ERROR) >> 8)
				, "\n\tINGRESS_BMAC1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_BMAC1_ERROR) >> 7)
				, "\n\tINGRESS_BMAC0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_BMAC0_ERROR) >> 6)
				, "\n\tINGRESS_RMP1_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_RMP1_DSCR_FIFO_ERROR) >> 5)
				, "\n\tINGRESS_RMP0_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_RMP0_DSCR_FIFO_ERROR) >> 4)
				, "\n\tINGRESS_EOP_LB_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_EOP_LB_ERROR) >> 3)
				, "\n\tINGRESS_EOP_PORT1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_EOP_PORT1_ERROR) >> 2)
				, "\n\tINGRESS_EOP_PORT0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_INGRESS_EOP_PORT0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x103bcUL: // (NIG_REG_NIG_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLFC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_LLFC1_POP_ERROR) >> 31)
				, "\n\tLLFC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_LLFC0_POP_ERROR) >> 30)
				, "\n\tLLH1_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_LLH1_FIFO_ERROR) >> 29)
				, "\n\tLLH0_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_LLH0_FIFO_ERROR) >> 28)
				, "\n\tTIMER1_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_TIMER1_MAX_INT) >> 27)
				, "\n\tTIMER0_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_TIMER0_MAX_INT) >> 26)
				, "\n\tEGRESS_BMAC1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_BMAC1_ERROR) >> 25)
				, "\n\tEGRESS_BMAC0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_BMAC0_ERROR) >> 24)
				, "\n\tEGRESS_EMAC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_EMAC1_POP_ERROR) >> 23)
				, "\n\tEGRESS_EMAC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_EMAC0_POP_ERROR) >> 22)
				, "\n\tEGRESS_EMAC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_EMAC1_PUSH_ERROR) >> 21)
				, "\n\tEGRESS_EMAC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_EMAC0_PUSH_ERROR) >> 20)
				, "\n\tEGRESS_DELAY1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_DELAY1_ERROR) >> 19)
				, "\n\tEGRESS_DELAY0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_DELAY0_ERROR) >> 18)
				, "\n\tEGRESS_DEBUG_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_DEBUG_FIFO_ERROR) >> 17)
				, "\n\tEGRESS_MNG1_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_MNG1_FIFO_ERROR) >> 16)
				, "\n\tEGRESS_MNG0_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_EGRESS_MNG0_FIFO_ERROR) >> 15)
				, "\n\tINGRESS_LB_PBF_DELAY_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_LB_PBF_DELAY_ERROR) >> 14)
				, "\n\tINGRESS_EMAC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_EMAC1_PUSH_ERROR) >> 13)
				, "\n\tINGRESS_EMAC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_EMAC0_PUSH_ERROR) >> 12)
				, "\n\tINGRESS_EMAC1_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_EMAC1_POP_ERROR) >> 11)
				, "\n\tINGRESS_EMAC0_POP_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_EMAC0_POP_ERROR) >> 10)
				, "\n\tINGRESS_BMAC1_REGS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_BMAC1_REGS_ERROR) >> 9)
				, "\n\tINGRESS_BMAC0_REGS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_BMAC0_REGS_ERROR) >> 8)
				, "\n\tINGRESS_BMAC1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_BMAC1_ERROR) >> 7)
				, "\n\tINGRESS_BMAC0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_BMAC0_ERROR) >> 6)
				, "\n\tINGRESS_RMP1_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_RMP1_DSCR_FIFO_ERROR) >> 5)
				, "\n\tINGRESS_RMP0_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_RMP0_DSCR_FIFO_ERROR) >> 4)
				, "\n\tINGRESS_EOP_LB_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_EOP_LB_ERROR) >> 3)
				, "\n\tINGRESS_EOP_PORT1_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_EOP_PORT1_ERROR) >> 2)
				, "\n\tINGRESS_EOP_PORT0_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_INGRESS_EOP_PORT0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x103c0UL: // (NIG_REG_NIG_INT_STS_1: ACCESS:R  DataWidth:0x1b ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INT_STS_1 (ACCESS:R  DataWidth:0x1b )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEGRESS_DELAY5_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_EGRESS_DELAY5_ERROR) >> 26)
				, "\n\tEGRESS_DELAY4_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_EGRESS_DELAY4_ERROR) >> 25)
				, "\n\tEGRESS_DELAY3_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_EGRESS_DELAY3_ERROR) >> 24)
				, "\n\tEGRESS_DELAY2_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_EGRESS_DELAY2_ERROR) >> 23)
				, "\n\tP1_RX_COS2_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P1_RX_COS2_TIMER_MAX_INT) >> 22)
				, "\n\tP0_RX_COS5_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_RX_COS5_TIMER_MAX_INT) >> 21)
				, "\n\tP0_RX_COS4_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_RX_COS4_TIMER_MAX_INT) >> 20)
				, "\n\tP0_RX_COS3_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_RX_COS3_TIMER_MAX_INT) >> 19)
				, "\n\tP0_RX_COS2_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_RX_COS2_TIMER_MAX_INT) >> 18)
				, "\n\tP1_TX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P1_TX_MACFIFO_ERROR) >> 17)
				, "\n\tP0_TX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_TX_MACFIFO_ERROR) >> 16)
				, "\n\tP1_RX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P1_RX_MACFIFO_ERROR) >> 15)
				, "\n\tP0_RX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_RX_MACFIFO_ERROR) >> 14)
				, "\n\tP1_TLLH_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P1_TLLH_FIFO_ERROR) >> 13)
				, "\n\tP0_TLLH_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_TLLH_FIFO_ERROR) >> 12)
				, "\n\tP1_HBUF_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P1_HBUF_DSCR_FIFO_ERROR) >> 11)
				, "\n\tP0_HBUF_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_HBUF_DSCR_FIFO_ERROR) >> 10)
				, "\n\tP1_TX_MNG_HOST_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P1_TX_MNG_HOST_FIFO_ERROR) >> 9)
				, "\n\tP0_TX_MNG_HOST_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_TX_MNG_HOST_FIFO_ERROR) >> 8)
				, "\n\tP1_RX_COS1_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P1_RX_COS1_TIMER_MAX_INT) >> 7)
				, "\n\tP1_RX_COS0_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P1_RX_COS0_TIMER_MAX_INT) >> 6)
				, "\n\tP0_RX_COS1_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_RX_COS1_TIMER_MAX_INT) >> 5)
				, "\n\tP0_RX_COS0_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_P0_RX_COS0_TIMER_MAX_INT) >> 4)
				, "\n\tLLH1_MF_ON_AND_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_LLH1_MF_ON_AND_NO_OUTER_VLAN) >> 3)
				, "\n\tLLH0_MF_ON_AND_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_LLH0_MF_ON_AND_NO_OUTER_VLAN) >> 2)
				, "\n\tLLFC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_LLFC1_PUSH_ERROR) >> 1)
				, "\n\tLLFC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_1_REG_LLFC0_PUSH_ERROR) >> 0));
			break;
		case 0x103c4UL: // (NIG_REG_NIG_INT_STS_CLR_1: ACCESS:RC  DataWidth:0x1b ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0x1b )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEGRESS_DELAY5_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_EGRESS_DELAY5_ERROR) >> 26)
				, "\n\tEGRESS_DELAY4_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_EGRESS_DELAY4_ERROR) >> 25)
				, "\n\tEGRESS_DELAY3_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_EGRESS_DELAY3_ERROR) >> 24)
				, "\n\tEGRESS_DELAY2_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_EGRESS_DELAY2_ERROR) >> 23)
				, "\n\tP1_RX_COS2_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P1_RX_COS2_TIMER_MAX_INT) >> 22)
				, "\n\tP0_RX_COS5_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_RX_COS5_TIMER_MAX_INT) >> 21)
				, "\n\tP0_RX_COS4_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_RX_COS4_TIMER_MAX_INT) >> 20)
				, "\n\tP0_RX_COS3_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_RX_COS3_TIMER_MAX_INT) >> 19)
				, "\n\tP0_RX_COS2_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_RX_COS2_TIMER_MAX_INT) >> 18)
				, "\n\tP1_TX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P1_TX_MACFIFO_ERROR) >> 17)
				, "\n\tP0_TX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_TX_MACFIFO_ERROR) >> 16)
				, "\n\tP1_RX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P1_RX_MACFIFO_ERROR) >> 15)
				, "\n\tP0_RX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_RX_MACFIFO_ERROR) >> 14)
				, "\n\tP1_TLLH_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P1_TLLH_FIFO_ERROR) >> 13)
				, "\n\tP0_TLLH_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_TLLH_FIFO_ERROR) >> 12)
				, "\n\tP1_HBUF_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P1_HBUF_DSCR_FIFO_ERROR) >> 11)
				, "\n\tP0_HBUF_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_HBUF_DSCR_FIFO_ERROR) >> 10)
				, "\n\tP1_TX_MNG_HOST_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P1_TX_MNG_HOST_FIFO_ERROR) >> 9)
				, "\n\tP0_TX_MNG_HOST_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_TX_MNG_HOST_FIFO_ERROR) >> 8)
				, "\n\tP1_RX_COS1_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P1_RX_COS1_TIMER_MAX_INT) >> 7)
				, "\n\tP1_RX_COS0_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P1_RX_COS0_TIMER_MAX_INT) >> 6)
				, "\n\tP0_RX_COS1_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_RX_COS1_TIMER_MAX_INT) >> 5)
				, "\n\tP0_RX_COS0_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_P0_RX_COS0_TIMER_MAX_INT) >> 4)
				, "\n\tLLH1_MF_ON_AND_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_LLH1_MF_ON_AND_NO_OUTER_VLAN) >> 3)
				, "\n\tLLH0_MF_ON_AND_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_LLH0_MF_ON_AND_NO_OUTER_VLAN) >> 2)
				, "\n\tLLFC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_LLFC1_PUSH_ERROR) >> 1)
				, "\n\tLLFC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_CLR_1_REG_LLFC0_PUSH_ERROR) >> 0));
			break;
		case 0x103c8UL: // (NIG_REG_NIG_INT_STS_WR_1: ACCESS:WR  DataWidth:0x1b ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INT_STS_WR_1 (ACCESS:WR  DataWidth:0x1b )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEGRESS_DELAY5_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_EGRESS_DELAY5_ERROR) >> 26)
				, "\n\tEGRESS_DELAY4_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_EGRESS_DELAY4_ERROR) >> 25)
				, "\n\tEGRESS_DELAY3_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_EGRESS_DELAY3_ERROR) >> 24)
				, "\n\tEGRESS_DELAY2_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_EGRESS_DELAY2_ERROR) >> 23)
				, "\n\tP1_RX_COS2_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P1_RX_COS2_TIMER_MAX_INT) >> 22)
				, "\n\tP0_RX_COS5_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_RX_COS5_TIMER_MAX_INT) >> 21)
				, "\n\tP0_RX_COS4_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_RX_COS4_TIMER_MAX_INT) >> 20)
				, "\n\tP0_RX_COS3_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_RX_COS3_TIMER_MAX_INT) >> 19)
				, "\n\tP0_RX_COS2_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_RX_COS2_TIMER_MAX_INT) >> 18)
				, "\n\tP1_TX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P1_TX_MACFIFO_ERROR) >> 17)
				, "\n\tP0_TX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_TX_MACFIFO_ERROR) >> 16)
				, "\n\tP1_RX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P1_RX_MACFIFO_ERROR) >> 15)
				, "\n\tP0_RX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_RX_MACFIFO_ERROR) >> 14)
				, "\n\tP1_TLLH_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P1_TLLH_FIFO_ERROR) >> 13)
				, "\n\tP0_TLLH_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_TLLH_FIFO_ERROR) >> 12)
				, "\n\tP1_HBUF_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P1_HBUF_DSCR_FIFO_ERROR) >> 11)
				, "\n\tP0_HBUF_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_HBUF_DSCR_FIFO_ERROR) >> 10)
				, "\n\tP1_TX_MNG_HOST_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P1_TX_MNG_HOST_FIFO_ERROR) >> 9)
				, "\n\tP0_TX_MNG_HOST_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_TX_MNG_HOST_FIFO_ERROR) >> 8)
				, "\n\tP1_RX_COS1_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P1_RX_COS1_TIMER_MAX_INT) >> 7)
				, "\n\tP1_RX_COS0_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P1_RX_COS0_TIMER_MAX_INT) >> 6)
				, "\n\tP0_RX_COS1_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_RX_COS1_TIMER_MAX_INT) >> 5)
				, "\n\tP0_RX_COS0_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_P0_RX_COS0_TIMER_MAX_INT) >> 4)
				, "\n\tLLH1_MF_ON_AND_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_LLH1_MF_ON_AND_NO_OUTER_VLAN) >> 3)
				, "\n\tLLH0_MF_ON_AND_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_LLH0_MF_ON_AND_NO_OUTER_VLAN) >> 2)
				, "\n\tLLFC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_LLFC1_PUSH_ERROR) >> 1)
				, "\n\tLLFC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_STS_WR_1_REG_LLFC0_PUSH_ERROR) >> 0));
			break;
		case 0x103ccUL: // (NIG_REG_NIG_INT_MASK_1: ACCESS:RW  DataWidth:0x1b ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INT_MASK_1 (ACCESS:RW  DataWidth:0x1b )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEGRESS_DELAY5_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_EGRESS_DELAY5_ERROR) >> 26)
				, "\n\tEGRESS_DELAY4_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_EGRESS_DELAY4_ERROR) >> 25)
				, "\n\tEGRESS_DELAY3_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_EGRESS_DELAY3_ERROR) >> 24)
				, "\n\tEGRESS_DELAY2_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_EGRESS_DELAY2_ERROR) >> 23)
				, "\n\tP1_RX_COS2_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P1_RX_COS2_TIMER_MAX_INT) >> 22)
				, "\n\tP0_RX_COS5_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_RX_COS5_TIMER_MAX_INT) >> 21)
				, "\n\tP0_RX_COS4_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_RX_COS4_TIMER_MAX_INT) >> 20)
				, "\n\tP0_RX_COS3_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_RX_COS3_TIMER_MAX_INT) >> 19)
				, "\n\tP0_RX_COS2_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_RX_COS2_TIMER_MAX_INT) >> 18)
				, "\n\tP1_TX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P1_TX_MACFIFO_ERROR) >> 17)
				, "\n\tP0_TX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_TX_MACFIFO_ERROR) >> 16)
				, "\n\tP1_RX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P1_RX_MACFIFO_ERROR) >> 15)
				, "\n\tP0_RX_MACFIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_RX_MACFIFO_ERROR) >> 14)
				, "\n\tP1_TLLH_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P1_TLLH_FIFO_ERROR) >> 13)
				, "\n\tP0_TLLH_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_TLLH_FIFO_ERROR) >> 12)
				, "\n\tP1_HBUF_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P1_HBUF_DSCR_FIFO_ERROR) >> 11)
				, "\n\tP0_HBUF_DSCR_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_HBUF_DSCR_FIFO_ERROR) >> 10)
				, "\n\tP1_TX_MNG_HOST_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P1_TX_MNG_HOST_FIFO_ERROR) >> 9)
				, "\n\tP0_TX_MNG_HOST_FIFO_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_TX_MNG_HOST_FIFO_ERROR) >> 8)
				, "\n\tP1_RX_COS1_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P1_RX_COS1_TIMER_MAX_INT) >> 7)
				, "\n\tP1_RX_COS0_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P1_RX_COS0_TIMER_MAX_INT) >> 6)
				, "\n\tP0_RX_COS1_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_RX_COS1_TIMER_MAX_INT) >> 5)
				, "\n\tP0_RX_COS0_TIMER_MAX_INT :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_P0_RX_COS0_TIMER_MAX_INT) >> 4)
				, "\n\tLLH1_MF_ON_AND_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_LLH1_MF_ON_AND_NO_OUTER_VLAN) >> 3)
				, "\n\tLLH0_MF_ON_AND_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_LLH0_MF_ON_AND_NO_OUTER_VLAN) >> 2)
				, "\n\tLLFC1_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_LLFC1_PUSH_ERROR) >> 1)
				, "\n\tLLFC0_PUSH_ERROR :"
				, (u32_t)((data & NIG_NIG_INT_MASK_1_REG_LLFC0_PUSH_ERROR) >> 0));
			break;
		case 0x16004UL: // (NIG_REG_LLH0_DEST_MAC_3_0: ACCESS:RW  DataWidth:0x20  Register llh0_dest_mac_3: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_MAC_3_0 (ACCESS:RW  DataWidth:0x20  Register llh0_dest_mac_3: 1 out of 2)" \
				"\nDescription: Destination MAC address 3.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16008UL: // (NIG_REG_LLH0_DEST_MAC_3_1: ACCESS:RW  DataWidth:0x10  Register llh0_dest_mac_3: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH0_DEST_MAC_3_1 (ACCESS:RW  DataWidth:0x10  Register llh0_dest_mac_3: 2 out of 2)" \
				"\nDescription: Destination MAC address 3.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1600cUL: // (NIG_REG_LLH1_DEST_MAC_3_0: ACCESS:RW  DataWidth:0x20  Register llh1_dest_mac_3: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_MAC_3_0 (ACCESS:RW  DataWidth:0x20  Register llh1_dest_mac_3: 1 out of 2)" \
				"\nDescription: Destination MAC address 3.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16010UL: // (NIG_REG_LLH1_DEST_MAC_3_1: ACCESS:RW  DataWidth:0x10  Register llh1_dest_mac_3: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_LLH1_DEST_MAC_3_1 (ACCESS:RW  DataWidth:0x10  Register llh1_dest_mac_3: 2 out of 2)" \
				"\nDescription: Destination MAC address 3.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16014UL: // (NIG_REG_LLH0_OUTER_VLAN_ID: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_OUTER_VLAN_ID (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: VLAN ID of the outer VLAN on port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16018UL: // (NIG_REG_LLH1_OUTER_VLAN_ID: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_OUTER_VLAN_ID (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: VLAN ID of the outer VLAN on port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1601cUL: // (NIG_REG_LLH0_MCP_MASK_MF: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_MCP_MASK_MF (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_LLH_MCP_MASK_PF_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_MF_REG_P0_LLH_MCP_MASK_PF_OUTER_VLAN) >> 5)
				, "\n\tLLH0_MCP_MASK_IPV6_MLCST :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_MF_REG_LLH0_MCP_MASK_IPV6_MLCST) >> 4)
				, "\n\tLLH0_MCP_MASK_MAC3 :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_MF_REG_LLH0_MCP_MASK_MAC3) >> 3)
				, "\n\tLLH0_MCP_MASK_OUTER_VLAN_ID :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_MF_REG_LLH0_MCP_MASK_OUTER_VLAN_ID) >> 2)
				, "\n\tLLH0_MCP_MASK_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_MF_REG_LLH0_MCP_MASK_NO_OUTER_VLAN) >> 1)
				, "\n\tLLH0_MCP_MASK_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH0_MCP_MASK_MF_REG_LLH0_MCP_MASK_OUTER_VLAN) >> 0));
			break;
		case 0x16020UL: // (NIG_REG_LLH1_MCP_MASK_MF: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_MCP_MASK_MF (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_LLH_MCP_MASK_PF_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_MF_REG_P1_LLH_MCP_MASK_PF_OUTER_VLAN) >> 5)
				, "\n\tLLH1_MCP_MASK_IPV6_MLCST :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_MF_REG_LLH1_MCP_MASK_IPV6_MLCST) >> 4)
				, "\n\tLLH1_MCP_MASK_MAC3 :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_MF_REG_LLH1_MCP_MASK_MAC3) >> 3)
				, "\n\tLLH1_MCP_MASK_OUTER_VLAN_ID :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_MF_REG_LLH1_MCP_MASK_OUTER_VLAN_ID) >> 2)
				, "\n\tLLH1_MCP_MASK_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_MF_REG_LLH1_MCP_MASK_NO_OUTER_VLAN) >> 1)
				, "\n\tLLH1_MCP_MASK_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH1_MCP_MASK_MF_REG_LLH1_MCP_MASK_OUTER_VLAN) >> 0));
			break;
		case 0x16024UL: // (NIG_REG_LLH_MF_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH_MF_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When this bit is set; the LLH will classify the packet before\n     sending it to the BRB or calculating WoL on it.  This bit is applicable\n     to both ports 0 and 1 for E2.  This bit only controls port 0 in E3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16028UL: // (NIG_REG_LLH_E1HOV_TYPE_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH_E1HOV_TYPE_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Outer VLAN type identifier for multi-function mode. In non\n     multi-function mode; it will hold the inner VLAN type. Typically 0x8100\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1602cUL: // (NIG_REG_LLH_E1HOV_TYPE_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH_E1HOV_TYPE_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Outer VLAN type identifier for multi-function mode. In non\n     multi-function mode; it will hold the inner VLAN type. Typically 0x8100\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16030UL: // (NIG_REG_LLH_E1HOV_TYPE_3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH_E1HOV_TYPE_3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Outer VLAN type identifier for multi-function mode. In non\n     multi-function mode; it will hold the inner VLAN type. Typically 0x8100\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16034UL: // (NIG_REG_LLH_E1HOV_TYPE_4: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH_E1HOV_TYPE_4 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Outer VLAN type identifier for multi-function mode. In non\n     multi-function mode; it will hold the inner VLAN type. Typically 0x8100\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16038UL: // (NIG_REG_LLH_E1HOV_TYPE_5: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH_E1HOV_TYPE_5 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Outer VLAN type identifier for multi-function mode. In non\n     multi-function mode; it will hold the inner VLAN type. Typically 0x8100\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1603cUL: // (NIG_REG_LLH_E1HOV_TYPE_6: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH_E1HOV_TYPE_6 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Outer VLAN type identifier for multi-function mode. In non\n     multi-function mode; it will hold the inner VLAN type. Typically 0x8100\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16040UL: // (NIG_REG_LLH_E1HOV_TYPE_7: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH_E1HOV_TYPE_7 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Outer VLAN type identifier for multi-function mode. In non\n     multi-function mode; it will hold the inner VLAN type. Typically 0x8100\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16044UL: // (NIG_REG_LLH_E1HOV_TYPE_8: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH_E1HOV_TYPE_8 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Outer VLAN type identifier for multi-function mode. In non\n     multi-function mode; it will hold the inner VLAN type. Typically 0x8100\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16048UL: // (NIG_REG_LLH0_BRB1_DRV_MASK_MF: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_BRB1_DRV_MASK_MF (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_LLH_BRB1_DRV_MASK_PF_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH0_BRB1_DRV_MASK_MF_REG_P0_LLH_BRB1_DRV_MASK_PF_OUTER_VLAN) >> 2)
				, "\n\tLLH0_BRB1_DRV_MASK_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH0_BRB1_DRV_MASK_MF_REG_LLH0_BRB1_DRV_MASK_NO_OUTER_VLAN) >> 1)
				, "\n\tLLH0_BRB1_DRV_MASK_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH0_BRB1_DRV_MASK_MF_REG_LLH0_BRB1_DRV_MASK_OUTER_VLAN) >> 0));
			break;
		case 0x1604cUL: // (NIG_REG_LLH1_BRB1_DRV_MASK_MF: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_BRB1_DRV_MASK_MF (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_LLH_BRB1_DRV_MASK_PF_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH1_BRB1_DRV_MASK_MF_REG_P1_LLH_BRB1_DRV_MASK_PF_OUTER_VLAN) >> 2)
				, "\n\tLLH1_BRB1_DRV_MASK_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH1_BRB1_DRV_MASK_MF_REG_LLH1_BRB1_DRV_MASK_NO_OUTER_VLAN) >> 1)
				, "\n\tLLH1_BRB1_DRV_MASK_OUTER_VLAN :"
				, (u32_t)((data & NIG_LLH1_BRB1_DRV_MASK_MF_REG_LLH1_BRB1_DRV_MASK_OUTER_VLAN) >> 0));
			break;
		case 0x16050UL: // (NIG_REG_LLH0_BRB1_MCP_MASK_MF: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_BRB1_MCP_MASK_MF (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH0_BRB1_MCP_MASK_IPV6_MLCST :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_MF_REG_LLH0_BRB1_MCP_MASK_IPV6_MLCST) >> 2)
				, "\n\tLLH0_BRB1_MCP_MASK_OUTER_VLAN_ID :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_MF_REG_LLH0_BRB1_MCP_MASK_OUTER_VLAN_ID) >> 1)
				, "\n\tLLH0_BRB1_MCP_MASK_MAC3 :"
				, (u32_t)((data & NIG_LLH0_BRB1_MCP_MASK_MF_REG_LLH0_BRB1_MCP_MASK_MAC3) >> 0));
			break;
		case 0x16054UL: // (NIG_REG_LLH1_BRB1_MCP_MASK_MF: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_BRB1_MCP_MASK_MF (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH1_BRB1_MCP_MASK_IPV6_MLCST :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_MF_REG_LLH1_BRB1_MCP_MASK_IPV6_MLCST) >> 2)
				, "\n\tLLH1_BRB1_MCP_MASK_OUTER_VLAN_ID :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_MF_REG_LLH1_BRB1_MCP_MASK_OUTER_VLAN_ID) >> 1)
				, "\n\tLLH1_BRB1_MCP_MASK_MAC3 :"
				, (u32_t)((data & NIG_LLH1_BRB1_MCP_MASK_MF_REG_LLH1_BRB1_MCP_MASK_MAC3) >> 0));
			break;
		case 0x16058UL: // (NIG_REG_LLFC_HIGH_PRIORITY_CLASSES_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_HIGH_PRIORITY_CLASSES_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: classes are high-priority for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1605cUL: // (NIG_REG_LLFC_HIGH_PRIORITY_CLASSES_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_HIGH_PRIORITY_CLASSES_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: classes are high-priority for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16060UL: // (NIG_REG_LLFC_LOW_PRIORITY_CLASSES_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_LOW_PRIORITY_CLASSES_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: classes are low-priority for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16064UL: // (NIG_REG_LLFC_LOW_PRIORITY_CLASSES_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_LOW_PRIORITY_CLASSES_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: classes are low-priority for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16070UL: // (NIG_REG_LLFC_EGRESS_SRC_ENABLE_0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_EGRESS_SRC_ENABLE_0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: for port0 enable for llfc ppp and pause. b0 - brb1 enable\n    ; b1- tsdm enable; b2- usdm enable \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16074UL: // (NIG_REG_LLFC_EGRESS_SRC_ENABLE_1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_EGRESS_SRC_ENABLE_1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: for port1 enable for llfc ppp and pause. b0 - brb1 enable\n    ; b1- tsdm enable; b2- usdm enable\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16078UL: // (NIG_REG_LLFC_EGRESS_CYCLE_NUM_0: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_EGRESS_CYCLE_NUM_0 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: number of cycles between 2 LLFC request to BMAC; The minimum\n     value of this register must be 16. The value of this register must be\n     bigger then ~BIGMAC_REGISTERS_TX_LLFC_CTRLl.llfc_img register *BMAC_clk\n    /sys_clk for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1607cUL: // (NIG_REG_LLFC_EGRESS_CYCLE_NUM_1: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_EGRESS_CYCLE_NUM_1 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: number of cycles between 2 LLFC request to BMAC; The minimum\n     value of this register must be 16. The value of this register must be\n     bigger then ~BIGMAC_REGISTERS_TX_LLFC_CTRLl.llfc_img register *BMAC_clk\n    /sys_clk for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16080UL: // (NIG_REG_LLH0_CLS_TYPE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_CLS_TYPE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Determine the classification participants. 0: no classification\n    .1: classification upon VLAN id. 2: classification upon MAC address. 3\n    : classification upon both VLAN id & MAC addr.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16084UL: // (NIG_REG_LLH1_CLS_TYPE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_CLS_TYPE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Determine the classification participants. 0: no classification\n    .1: classification upon VLAN id. 2: classification upon MAC address. 3\n    : classification upon both VLAN id & MAC addr. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16088UL: // (NIG_REG_LLH0_MNG_OUTER_VLAN_STRIP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_MNG_OUTER_VLAN_STRIP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Remove outer VLAN before sending the packet to the RMP.  \n    This configuration is ORed with llh_mng_tag_rm - the new E2 tag removal\n     configuration.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1608cUL: // (NIG_REG_LLH1_MNG_OUTER_VLAN_STRIP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_MNG_OUTER_VLAN_STRIP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: remove outer VLAN before sending the packet to the RMP.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160a0UL: // (NIG_REG_PPP_STORM_ID: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_PPP_STORM_ID (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: STORM ID for header in PP message to PXP internal write IF\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160a4UL: // (NIG_REG_PPP_ADDRESS: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_PPP_ADDRESS (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: ADDRESS for header in PP message to PXP internal write IF\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160a8UL: // (NIG_REG_PPP_TRIG: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_PPP_TRIG (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Trigger for header in PP message to PXP internal write IF\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160acUL: // (NIG_REG_PPP_T124PARAM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_PPP_T124PARAM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: T124param for header in PP message to PXP internal write \n    IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160b0UL: // (NIG_REG_PPP_ENABLE_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PPP_ENABLE_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: PPP enable for port0. This register may get 1 only when ~\n    safc_enable.safc_enable = 0 and pause_enable.pause_enable =0 for the same\n     port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160b4UL: // (NIG_REG_PPP_ENABLE_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PPP_ENABLE_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: PPP enable for port1. This register may get 1 only when ~\n    safc_enable.safc_enable = 0 and pause_enable.pause_enable =0 for the same\n     port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160c0UL: // (NIG_REG_PAUSE_ENABLE_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PAUSE_ENABLE_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Pause enable for port0. This register may get 1 only when\n     ~safc_enable.safc_enable = 0 and ppp_enable.ppp_enable =0 for the same\n     port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160c4UL: // (NIG_REG_PAUSE_ENABLE_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PAUSE_ENABLE_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Pause enable for port1. This register may get 1 only when\n     ~safc_enable.safc_enable = 0 and ppp_enable.ppp_enable =0 for the same\n     port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160c8UL: // (NIG_REG_LLFC_OUT_EN_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_OUT_EN_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Output enable of message to LLFC BMAC IF for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160ccUL: // (NIG_REG_LLFC_OUT_EN_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_OUT_EN_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Output enable of message to LLFC BMAC IF for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160d0UL: // (NIG_REG_LLH0_MAC_CF_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_MAC_CF_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: MAC control frame EtherType. The LLH will look for this Ethertype\n     as a MAC CF identifier. Default is IEEE 0x8808.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160d4UL: // (NIG_REG_LLH1_MAC_CF_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_MAC_CF_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: MAC control frame EtherType. The LLH will look for this Ethertype\n     as a MAC CF identifier. Default is IEEE 0x8808.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160d8UL: // (NIG_REG_LLH_E1HOV_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH_E1HOV_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When this bit is set; the LLH will expect all packets to \n    be with outer VLAN.  This is not applicable to E2. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16204UL: // (NIG_REG_PPP_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PPP_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Output enable of message to PXP IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16208UL: // (NIG_REG_LLFC_ENABLE_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_ENABLE_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: SAFC enable for port0. This register may get 1 only when ~\n    ppp_enable.ppp_enable = 0 and pause_enable.pause_enable =0 for the same\n     port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1620cUL: // (NIG_REG_LLFC_ENABLE_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_ENABLE_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: SAFC enable for port1. This register may get 1 only when ~\n    ppp_enable.ppp_enable = 0 and pause_enable.pause_enable =0 for the same\n     port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16210UL: // (NIG_REG_LATCH_BC_0: ACCESS:RW  DataWidth:0x1b ) 
			sprintf(disp_str, "\nNIG_REG_LATCH_BC_0 (ACCESS:RW  DataWidth:0x1b )" \
				"\nDescription: 0 - must be active for Everest A0; 1- for Everest B0 when\n     latch logic for interrupts must be used. Enable per bit of interrupt \n    of ~latch_status.latch_status\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16214UL: // (NIG_REG_LATCH_BC_1: ACCESS:RW  DataWidth:0x1b ) 
			sprintf(disp_str, "\nNIG_REG_LATCH_BC_1 (ACCESS:RW  DataWidth:0x1b )" \
				"\nDescription: 0 - must be active for Everest A0; 1- for Everest B0 when\n     latch logic for interrupts must be used. Enable per bit of interrupt \n    of ~latch_status.latch_status\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18010UL: // (NIG_REG_LLH0_SYNC_LLFC_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_SYNC_LLFC_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM for LLH_SYNC_FIFO for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18014UL: // (NIG_REG_LLH1_SYNC_LLFC_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_SYNC_LLFC_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM for LLH_SYNC_FIFO for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18018UL: // (NIG_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1801cUL: // (NIG_REG_FIFOS_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_FIFOS_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM for fifos: LLH0 fifo [1:0]; LLH1 fifo[3:2]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18020UL: // (NIG_REG_P0_TAG_ETHERTYPE_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TAG_ETHERTYPE_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 0.  This tag is used for\n     VNTAG classification and filtering in E3 B0 - when enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18024UL: // (NIG_REG_P0_TAG_ETHERTYPE_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TAG_ETHERTYPE_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18028UL: // (NIG_REG_P0_TAG_ETHERTYPE_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TAG_ETHERTYPE_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802cUL: // (NIG_REG_P0_TAG_LEN_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TAG_LEN_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 0 - in bytes.  The\n     length is between 2B and 14B; in 2B granularity.  This length does not\n     include the Ethertype field.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18030UL: // (NIG_REG_P0_TAG_LEN_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TAG_LEN_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 1 - in bytes.  The\n     length is between 2B and 14B; in 2B granularity.  This length does not\n     include the Ethertype field.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18034UL: // (NIG_REG_P0_TAG_LEN_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TAG_LEN_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 2 - in bytes.  The\n     length is between 2B and 14B; in 2B granularity.  This length does not\n     include the Ethertype field.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18038UL: // (NIG_REG_P0_HDRS_AFTER_BASIC: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HDRS_AFTER_BASIC (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the basic\n     Ethernet header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1803cUL: // (NIG_REG_P0_HDRS_AFTER_OUTER_VLAN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HDRS_AFTER_OUTER_VLAN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the outer\n     VLAN header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18040UL: // (NIG_REG_P0_HDRS_AFTER_INNER_VLAN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HDRS_AFTER_INNER_VLAN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the inner\n     VLAN header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18044UL: // (NIG_REG_P0_HDRS_AFTER_LLC: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HDRS_AFTER_LLC (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the LLC\n     header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18048UL: // (NIG_REG_P0_HDRS_AFTER_TAG_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HDRS_AFTER_TAG_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1804cUL: // (NIG_REG_P0_HDRS_AFTER_TAG_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HDRS_AFTER_TAG_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18050UL: // (NIG_REG_P0_HDRS_AFTER_TAG_2: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HDRS_AFTER_TAG_2 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18054UL: // (NIG_REG_P0_PKT_PRIORITY_TO_COS: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_PKT_PRIORITY_TO_COS (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Eight 4-bit configurations for specifying which COS (0-15\n     for future expansion) each priorty is to be mapped to.  Bits 3:0 specify\n     the COS for priority 0.  Bits 31:28 specify the COS for priority 7.  \n    The 3-bit priority field is extracted from the outer-most VLAN in receive\n     packet.    Only COS 0 and COS 1 are supported in E2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18058UL: // (NIG_REG_P0_RX_COS0_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS0_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to COS\n     0.  A priority is mapped to COS 0 when the corresponding mask bit is \n    1.  More than one bit may be set; allowing multiple priorities to be mapped\n     to one COS.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1805cUL: // (NIG_REG_P0_RX_COS1_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS1_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to COS\n     1.  A priority is mapped to COS 1 when the corresponding mask bit is \n    1.  More than one bit may be set; allowing multiple priorities to be mapped\n     to one COS.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18060UL: // (NIG_REG_P0_RX_COS0_TIMER_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS0_TIMER_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the RX COS 0 timer to start over when\n     it reaches the configured maximum time.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18064UL: // (NIG_REG_P0_RX_COS1_TIMER_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS1_TIMER_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the RX COS 1 timer to start over when\n     it reaches the configured maximum time.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18068UL: // (NIG_REG_P0_RX_COS0_TIMER_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS0_TIMER_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This register specifies the period of time (in term of the\n     number of core clocks) RX COS0 is paused before asserting rx_cos0_timer_max_int\n     interrupt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1806cUL: // (NIG_REG_P0_RX_COS1_TIMER_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS1_TIMER_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This register specifies the period of time (in term of the\n     number of core clocks) RX COS1 is paused before asserting rx_cos1_timer_max_int\n     interrupt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18070UL: // (NIG_REG_P0_LLFC_XOFF_TIMER_MAX: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLFC_XOFF_TIMER_MAX (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Timeout value for LLFC XOFF timer in the TX direction for\n     sending refresh LLFC messages to the BMAC.  The value is in term of the\n     number of core clocks.  The timer starts whenever an LLFC request is \n    sent to the BMAC with at least one priority in the XOFF state.  An update\n     LLFC request is sent when the timer expires.  The value should be less\n     than LLFC XOFF TIME * 8 clk per incr * core clk freq / MAC RX clk freq\n    .  The default value is set with XOFF time of 0x8000; RX clock frequency\n     at 156.25 Mhz; and core clock frequency at 250 Mhz.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18074UL: // (NIG_REG_P0_HWLLFC_RX_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HWLLFC_RX_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: HW LLFC receive enable bit.  Set this bit to enable the HW\n     LLFC/SAFC functionality in the NIG in the RX path.  The NIG block determines\n     the XOFF/XON state of each class according to the BigMAC output SAFC \n    signals and the mapping configuration.  If a class is in XOFF state then\n     stop the corresponding PBF queue.  Other HW flow control modes such as\n     PAUSE and PFC should be disabled when this bit is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18078UL: // (NIG_REG_P0_HWPFC_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HWPFC_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: HW PFC enable bit.  Set this bit to enable the PFC functionality\n     in the NIG.  Other flow control modes such as PAUSE and SAFC/LLFC should\n     be disabled when this bit is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1807cUL: // (NIG_REG_P0_ETHERTYPE0_FOR_XCM: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_ETHERTYPE0_FOR_XCM (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for filtering packets to XCM for future flow control\n     and congestion management protocols (QFC/QCN). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18080UL: // (NIG_REG_P0_ETHERTYPE1_FOR_XCM: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_ETHERTYPE1_FOR_XCM (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for filtering packets to XCM for future flow control\n     and congestion management protocols (QFC/QCN). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18084UL: // (NIG_REG_P0_ETHERTYPE0_FOR_XCM_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_ETHERTYPE0_FOR_XCM_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable packets with Ethertype matching ethertype0_for_xcm\n     to be forwarded to XCM. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18088UL: // (NIG_REG_P0_ETHERTYPE1_FOR_XCM_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_ETHERTYPE1_FOR_XCM_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable packets with Ethertype matching ethertype1_for_xcm\n     to be forwarded to XCM. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1808cUL: // (NIG_REG_P0_LLH_DEST_MAC_4_0: ACCESS:RW  DataWidth:0x20  Register p0_llh_dest_mac_4: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_DEST_MAC_4_0 (ACCESS:RW  DataWidth:0x20  Register p0_llh_dest_mac_4: 1 out of 2)" \
				"\nDescription: Destination MAC address 4.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18090UL: // (NIG_REG_P0_LLH_DEST_MAC_4_1: ACCESS:RW  DataWidth:0x10  Register p0_llh_dest_mac_4: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_DEST_MAC_4_1 (ACCESS:RW  DataWidth:0x10  Register p0_llh_dest_mac_4: 2 out of 2)" \
				"\nDescription: Destination MAC address 4.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18094UL: // (NIG_REG_P0_LLH_DEST_MAC_5_0: ACCESS:RW  DataWidth:0x20  Register p0_llh_dest_mac_5: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_DEST_MAC_5_0 (ACCESS:RW  DataWidth:0x20  Register p0_llh_dest_mac_5: 1 out of 2)" \
				"\nDescription: Destination MAC address 5.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18098UL: // (NIG_REG_P0_LLH_DEST_MAC_5_1: ACCESS:RW  DataWidth:0x10  Register p0_llh_dest_mac_5: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_DEST_MAC_5_1 (ACCESS:RW  DataWidth:0x10  Register p0_llh_dest_mac_5: 2 out of 2)" \
				"\nDescription: Destination MAC address 5.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180a4UL: // (NIG_REG_P0_LLH_ETHERTYPE0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_ETHERTYPE0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for filtering packets.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180a8UL: // (NIG_REG_P0_LLH_ETHERTYPE1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_ETHERTYPE1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for filtering packets.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180c0UL: // (NIG_REG_P0_FRAME_CRACKER_SOFT_RST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_FRAME_CRACKER_SOFT_RST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Soft reset bit for frame cracker - not expected to be used\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180c4UL: // (NIG_REG_P0_FCOE_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_FCOE_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: FCOE Ethertype - default is 0x8906.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180ccUL: // (NIG_REG_P0_LLH_MNG_TAG_RM: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_MNG_TAG_RM (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: L2 tag removal configuration for RX management traffic.  \n    Bit mapped as follow: bit 0 - outer VLAN; bit 1 - inner VLAN; bit 2 - \n    LLC/SNAP; bit 3 - tag 0; bit 4 - tag 1; bit 5 - tag 2.  Set these bits\n     to 1's to enable the removal of the corresponding L2 tags when they are\n     present in the receive packet.  Clear the bit to keep the tag in the \n    receive packet.  This configuration is ORed with {5'd0;llh0_mng_outer_vlan_strip\n    } for backward compatibility.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180e8UL: // (NIG_REG_P0_TX_ARB_CLIENT_IS_STRICT: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CLIENT_IS_STRICT (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: Specify whether the client competes directly in the strict\n     priority arbiter.  The bits are mapped according to client ID  (client\n     IDs are defined in tx_arb_priority_client2):  0-management; 1-debug traffic\n     from this port; 2-debug traffic from other port; 3-COS0 traffic; 4-COS1\n     traffic; 5-COS2 traffic; 6-COS3 traffic; 7-COS4 traffic; 8-COS5 traffic\n    .  Default value is set to enable strict priorities for all clients.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180ecUL: // (NIG_REG_P0_TX_ARB_CLIENT_IS_SUBJECT2WFQ: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CLIENT_IS_SUBJECT2WFQ (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: Specify whether the client is subject to WFQ credit blocking\n    .  The bits are mapped according to client ID (client IDs are defined \n    in tx_arb_priority_client2):  0-management; 1-debug traffic from this \n    port; 2-debug traffic from other port; 3-COS0 traffic; 4-COS1 traffic; \n    5-COS2 traffic; 6-COS3 traffic; 7-COS4 traffic; 8-COS5 traffic.  Default\n     value is 0 for not using WFQ credit blocking.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180f4UL: // (NIG_REG_P0_TX_ARB_NUM_STRICT_ARB_SLOTS: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_NUM_STRICT_ARB_SLOTS (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Specify the number of strict priority arbitration slots between\n     two round-robin arbitration slots to avoid starvation.  A value of 0 \n    means no strict priority cycles - the strict priority with anti-starvation\n     arbiter becomes a round-robin arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180f8UL: // (NIG_REG_P0_TX_ARB_CREDIT_WEIGHT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_WEIGHT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     0 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180fcUL: // (NIG_REG_P0_TX_ARB_CREDIT_WEIGHT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_WEIGHT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     1 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18100UL: // (NIG_REG_P0_TX_ARB_CREDIT_WEIGHT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_WEIGHT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     2 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18104UL: // (NIG_REG_P0_TX_ARB_CREDIT_WEIGHT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_WEIGHT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     3 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18108UL: // (NIG_REG_P0_TX_ARB_CREDIT_WEIGHT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_WEIGHT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     4 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1810cUL: // (NIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 0 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18110UL: // (NIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 1 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18114UL: // (NIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 2 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18118UL: // (NIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 3 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1811cUL: // (NIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 4 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18120UL: // (NIG_REG_P0_COS_DRAIN_EN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_COS_DRAIN_EN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Set these bits to enable the drain mode for COS0 and COS5\n    .  Bit 0 is for COS0 flow.  Bit 5 is for COS5 flow.  When enabled -- draining\n     of the corrresponding PBF COS FIFO starts immediately - packet data are\n     dropped and not forwarded to the MAC.  When disabled--draining stops \n    at the next packet boundary. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18124UL: // (NIG_REG_P0_RX_MCPCOS_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_MCPCOS_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to MCP\n     COS.  A priority is mapped to MCP COS when the corresponding mask bit\n     is 1.  More than one bit may be set; allowing multiple priorities to \n    be mapped to one COS.  MCP COS is used to flow control TX management traffic\n     from MCP.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18128UL: // (NIG_REG_P0_LLH_ARP_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_ARP_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for ARP (filtering rules B).  Defaut is 0x0806.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1812cUL: // (NIG_REG_P0_LLH_ICMPV4_NXTHDR: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_ICMPV4_NXTHDR (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: IPv4 protocol field for ICMPv4 - defaults to 0x01.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18130UL: // (NIG_REG_P0_LLH_ICMPV6_NXTHDR: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_ICMPV6_NXTHDR (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: IPv6 next header field for ICMPv6 - defaults to 0x3A.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18134UL: // (NIG_REG_P0_LLH_LLDP_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_LLDP_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for LLDP (filtering rules C).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18138UL: // (NIG_REG_P0_LLH_MCP_MASK_A: ACCESS:RW  DataWidth:0x5  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_MCP_MASK_A (ACCESS:RW  DataWidth:0x5  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_LLH_MCP_MASK_A_ALLMLCST :"
				, (u32_t)((data & NIG_P0_LLH_MCP_MASK_A_REG_P0_LLH_MCP_MASK_A_ALLMLCST) >> 4)
				, "\n\tP0_LLH_MCP_MASK_A_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P0_LLH_MCP_MASK_A_REG_P0_LLH_MCP_MASK_A_ETHERTYPE1) >> 3)
				, "\n\tP0_LLH_MCP_MASK_A_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P0_LLH_MCP_MASK_A_REG_P0_LLH_MCP_MASK_A_ETHERTYPE0) >> 2)
				, "\n\tP0_LLH_MCP_MASK_A_MAC5 :"
				, (u32_t)((data & NIG_P0_LLH_MCP_MASK_A_REG_P0_LLH_MCP_MASK_A_MAC5) >> 1)
				, "\n\tP0_LLH_MCP_MASK_A_MAC4 :"
				, (u32_t)((data & NIG_P0_LLH_MCP_MASK_A_REG_P0_LLH_MCP_MASK_A_MAC4) >> 0));
			break;
		case 0x1813cUL: // (NIG_REG_P0_LLH_MCP_MASK_B: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_MCP_MASK_B (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_LLH_MCP_MASK_B_ICMPV6 :"
				, (u32_t)((data & NIG_P0_LLH_MCP_MASK_B_REG_P0_LLH_MCP_MASK_B_ICMPV6) >> 2)
				, "\n\tP0_LLH_MCP_MASK_B_ICMPV4 :"
				, (u32_t)((data & NIG_P0_LLH_MCP_MASK_B_REG_P0_LLH_MCP_MASK_B_ICMPV4) >> 1)
				, "\n\tP0_LLH_MCP_MASK_B_ARP :"
				, (u32_t)((data & NIG_P0_LLH_MCP_MASK_B_REG_P0_LLH_MCP_MASK_B_ARP) >> 0));
			break;
		case 0x18140UL: // (NIG_REG_P0_LLH_MCP_MASK_C_LLDP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_MCP_MASK_C_LLDP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable packets with Ethertype matching llh_lldp_type\n     to be forwarded to MCP. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18144UL: // (NIG_REG_P0_LLH_BRB1_MCP_MASK_A: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_BRB1_MCP_MASK_A (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_LLH_BRB1_MCP_MASK_A_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P0_LLH_BRB1_MCP_MASK_A_REG_P0_LLH_BRB1_MCP_MASK_A_ETHERTYPE1) >> 3)
				, "\n\tP0_LLH_BRB1_MCP_MASK_A_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P0_LLH_BRB1_MCP_MASK_A_REG_P0_LLH_BRB1_MCP_MASK_A_ETHERTYPE0) >> 2)
				, "\n\tP0_LLH_BRB1_MCP_MASK_A_MAC5 :"
				, (u32_t)((data & NIG_P0_LLH_BRB1_MCP_MASK_A_REG_P0_LLH_BRB1_MCP_MASK_A_MAC5) >> 1)
				, "\n\tP0_LLH_BRB1_MCP_MASK_A_MAC4 :"
				, (u32_t)((data & NIG_P0_LLH_BRB1_MCP_MASK_A_REG_P0_LLH_BRB1_MCP_MASK_A_MAC4) >> 0));
			break;
		case 0x18148UL: // (NIG_REG_P0_LLH_MCP_OVMASK_B: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_MCP_OVMASK_B (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_LLH_MCP_OVMASK_B_PF_OUTER_VLAN :"
				, (u32_t)((data & NIG_P0_LLH_MCP_OVMASK_B_REG_P0_LLH_MCP_OVMASK_B_PF_OUTER_VLAN) >> 3)
				, "\n\tP0_LLH_MCP_OVMASK_B_OUTER_VLAN_ID :"
				, (u32_t)((data & NIG_P0_LLH_MCP_OVMASK_B_REG_P0_LLH_MCP_OVMASK_B_OUTER_VLAN_ID) >> 2)
				, "\n\tP0_LLH_MCP_OVMASK_B_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_P0_LLH_MCP_OVMASK_B_REG_P0_LLH_MCP_OVMASK_B_NO_OUTER_VLAN) >> 1)
				, "\n\tP0_LLH_MCP_OVMASK_B_OUTER_VLAN :"
				, (u32_t)((data & NIG_P0_LLH_MCP_OVMASK_B_REG_P0_LLH_MCP_OVMASK_B_OUTER_VLAN) >> 0));
			break;
		case 0x1814cUL: // (NIG_REG_P0_LLH_MCP_OVMASK_C: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_MCP_OVMASK_C (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_LLH_MCP_OVMASK_C_PF_OUTER_VLAN :"
				, (u32_t)((data & NIG_P0_LLH_MCP_OVMASK_C_REG_P0_LLH_MCP_OVMASK_C_PF_OUTER_VLAN) >> 3)
				, "\n\tP0_LLH_MCP_OVMASK_C_OUTER_VLAN_ID :"
				, (u32_t)((data & NIG_P0_LLH_MCP_OVMASK_C_REG_P0_LLH_MCP_OVMASK_C_OUTER_VLAN_ID) >> 2)
				, "\n\tP0_LLH_MCP_OVMASK_C_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_P0_LLH_MCP_OVMASK_C_REG_P0_LLH_MCP_OVMASK_C_NO_OUTER_VLAN) >> 1)
				, "\n\tP0_LLH_MCP_OVMASK_C_OUTER_VLAN :"
				, (u32_t)((data & NIG_P0_LLH_MCP_OVMASK_C_REG_P0_LLH_MCP_OVMASK_C_OUTER_VLAN) >> 0));
			break;
		case 0x18150UL: // (NIG_REG_P0_XCM_TYPE_VALID: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_XCM_TYPE_VALID (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_XCM_TYPE_VALID_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P0_XCM_TYPE_VALID_REG_P0_XCM_TYPE_VALID_ETHERTYPE1) >> 3)
				, "\n\tP0_XCM_TYPE_VALID_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P0_XCM_TYPE_VALID_REG_P0_XCM_TYPE_VALID_ETHERTYPE0) >> 2)
				, "\n\tP0_XCM_TYPE_VALID_BCN :"
				, (u32_t)((data & NIG_P0_XCM_TYPE_VALID_REG_P0_XCM_TYPE_VALID_BCN) >> 1)
				, "\n\tP0_XCM_TYPE_VALID_MCF :"
				, (u32_t)((data & NIG_P0_XCM_TYPE_VALID_REG_P0_XCM_TYPE_VALID_MCF) >> 0));
			break;
		case 0x1816cUL: // (NIG_REG_P0_TX_ARB_DBG_CLIENT_DISABLE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_DBG_CLIENT_DISABLE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Set these bits to disable debug traffic at the inputs to \n    the ETS arbiter.  Bit 0 is for debug traffic from the current path.  Bit\n     1 is for debug traffic from the other path.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18170UL: // (NIG_REG_EGRESS_DEBUG_PATH: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DEBUG_PATH (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to select the path for sending debug traffic\n    .  0 selects the current path.  1 selects the other path.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18174UL: // (NIG_REG_P1_TAG_ETHERTYPE_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TAG_ETHERTYPE_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 0.  This tag is used for\n     VNTAG classification and filtering in E3 B0 - when enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18178UL: // (NIG_REG_P1_TAG_ETHERTYPE_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TAG_ETHERTYPE_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1817cUL: // (NIG_REG_P1_TAG_ETHERTYPE_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TAG_ETHERTYPE_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18180UL: // (NIG_REG_P1_TAG_LEN_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TAG_LEN_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 0 - in bytes.  The\n     length is between 2B and 14B; in 2B granularity.  This length does not\n     include the Ethertype field.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18184UL: // (NIG_REG_P1_TAG_LEN_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TAG_LEN_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 1 - in bytes.  The\n     length is between 2B and 14B; in 2B granularity.  This length does not\n     include the Ethertype field.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18188UL: // (NIG_REG_P1_TAG_LEN_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TAG_LEN_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 2 - in bytes.  The\n     length is between 2B and 14B; in 2B granularity.  This length does not\n     include the Ethertype field.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1818cUL: // (NIG_REG_P1_HDRS_AFTER_BASIC: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HDRS_AFTER_BASIC (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the basic\n     Ethernet header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18190UL: // (NIG_REG_P1_HDRS_AFTER_OUTER_VLAN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HDRS_AFTER_OUTER_VLAN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the outer\n     VLAN header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18194UL: // (NIG_REG_P1_HDRS_AFTER_INNER_VLAN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HDRS_AFTER_INNER_VLAN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the inner\n     VLAN header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18198UL: // (NIG_REG_P1_HDRS_AFTER_LLC: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HDRS_AFTER_LLC (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the LLC\n     header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1819cUL: // (NIG_REG_P1_HDRS_AFTER_TAG_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HDRS_AFTER_TAG_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181a0UL: // (NIG_REG_P1_HDRS_AFTER_TAG_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HDRS_AFTER_TAG_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181a4UL: // (NIG_REG_P1_HDRS_AFTER_TAG_2: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HDRS_AFTER_TAG_2 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181a8UL: // (NIG_REG_P1_PKT_PRIORITY_TO_COS: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_PKT_PRIORITY_TO_COS (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Eight 4-bit configurations for specifying which COS (0-15\n     for future expansion) each priorty is to be mapped to.  Bits 3:0 specify\n     the COS for priority 0.  Bits 31:28 specify the COS for priority 7.  \n    The 3-bit priority field is extracted from the outer-most VLAN in receive\n     packet.    Only COS 0 and COS 1 are supported in E2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181acUL: // (NIG_REG_P1_RX_COS0_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_COS0_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to COS\n     0.  A priority is mapped to COS 0 when the corresponding mask bit is \n    1.  More than one bit may be set; allowing multiple priorities to be mapped\n     to one COS.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181b0UL: // (NIG_REG_P1_RX_COS1_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_COS1_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to COS\n     1.  A priority is mapped to COS 1 when the corresponding mask bit is \n    1.  More than one bit may be set; allowing multiple priorities to be mapped\n     to one COS. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181b4UL: // (NIG_REG_P1_RX_MCPCOS_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_MCPCOS_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to MCP\n     COS.  A priority is mapped to MCP COS when the corresponding mask bit\n     is 1.  More than one bit may be set; allowing multiple priorities to \n    be mapped to one COS.  MCP COS is used to flow control TX management traffic\n     from MCP.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181b8UL: // (NIG_REG_P1_RX_COS0_TIMER_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_COS0_TIMER_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the RX COS 0 timer to start over when\n     it reaches the configured maximum time.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181bcUL: // (NIG_REG_P1_RX_COS1_TIMER_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_COS1_TIMER_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the RX COS 1 timer to start over when\n     it reaches the configured maximum time.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181c0UL: // (NIG_REG_P1_RX_COS0_TIMER_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_COS0_TIMER_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This register specifies the period of time (in term of the\n     number of core clocks) RX COS0 is paused before asserting rx_cos0_timer_max_int\n     interrupt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181c4UL: // (NIG_REG_P1_RX_COS1_TIMER_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_COS1_TIMER_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This register specifies the period of time (in term of the\n     number of core clocks) RX COS1 is paused before asserting rx_cos1_timer_max_int\n     interrupt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181c8UL: // (NIG_REG_P1_LLFC_XOFF_TIMER_MAX: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLFC_XOFF_TIMER_MAX (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Timeout value for LLFC XOFF timer in the TX direction for\n     sending refresh LLFC messages to the BMAC.  The value is in term of the\n     number of core clocks.  The timer starts whenever an LLFC request is \n    sent to the BMAC with at least one priority in the XOFF state.  An update\n     LLFC request is sent when the timer expires.  The value should be less\n     than LLFC XOFF TIME * 8 clk per incr * core clk freq / MAC RX clk freq\n    .  The default value is set with XOFF time of 0x8000; RX clock frequency\n     at 156.25 Mhz; and core clock frequency at 250 Mhz.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181ccUL: // (NIG_REG_P1_HWLLFC_RX_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HWLLFC_RX_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: HW LLFC receive enable bit.  Set this bit to enable the HW\n     LLFC/SAFC functionality in the NIG in the RX path.  The NIG block determines\n     the XOFF/XON state of each class according to the BigMAC output SAFC \n    signals and the mapping configuration.  If a class is in XOFF state then\n     stop the corresponding PBF queue.  Other HW flow control modes such as\n     PAUSE and PFC should be disabled when this bit is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181d0UL: // (NIG_REG_P1_HWPFC_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HWPFC_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: HW PFC enable bit.  Set this bit to enable the PFC functionality\n     in the NIG.  Other flow control modes such as PAUSE and SAFC/LLFC should\n     be disabled when this bit is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181d4UL: // (NIG_REG_P1_ETHERTYPE0_FOR_XCM: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_ETHERTYPE0_FOR_XCM (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for filtering packets to XCM for future flow control\n     and congestion management protocols (QFC/QCN). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181d8UL: // (NIG_REG_P1_ETHERTYPE1_FOR_XCM: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_ETHERTYPE1_FOR_XCM (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for filtering packets to XCM for future flow control\n     and congestion management protocols (QFC/QCN). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181dcUL: // (NIG_REG_P1_ETHERTYPE0_FOR_XCM_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_ETHERTYPE0_FOR_XCM_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable packets with Ethertype matching ethertype0_for_xcm\n     to be forwarded to XCM. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181e0UL: // (NIG_REG_P1_ETHERTYPE1_FOR_XCM_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_ETHERTYPE1_FOR_XCM_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable packets with Ethertype matching ethertype1_for_xcm\n     to be forwarded to XCM. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181e4UL: // (NIG_REG_P1_LLH_DEST_MAC_4_0: ACCESS:RW  DataWidth:0x20  Register p1_llh_dest_mac_4: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_DEST_MAC_4_0 (ACCESS:RW  DataWidth:0x20  Register p1_llh_dest_mac_4: 1 out of 2)" \
				"\nDescription: Destination MAC address 4.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181e8UL: // (NIG_REG_P1_LLH_DEST_MAC_4_1: ACCESS:RW  DataWidth:0x10  Register p1_llh_dest_mac_4: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_DEST_MAC_4_1 (ACCESS:RW  DataWidth:0x10  Register p1_llh_dest_mac_4: 2 out of 2)" \
				"\nDescription: Destination MAC address 4.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181ecUL: // (NIG_REG_P1_LLH_DEST_MAC_5_0: ACCESS:RW  DataWidth:0x20  Register p1_llh_dest_mac_5: 1 out of 2) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_DEST_MAC_5_0 (ACCESS:RW  DataWidth:0x20  Register p1_llh_dest_mac_5: 1 out of 2)" \
				"\nDescription: Destination MAC address 5.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181f0UL: // (NIG_REG_P1_LLH_DEST_MAC_5_1: ACCESS:RW  DataWidth:0x10  Register p1_llh_dest_mac_5: 2 out of 2) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_DEST_MAC_5_1 (ACCESS:RW  DataWidth:0x10  Register p1_llh_dest_mac_5: 2 out of 2)" \
				"\nDescription: Destination MAC address 5.  LLH will look for this address\n     in all incoming packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181f4UL: // (NIG_REG_P1_LLH_ETHERTYPE0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_ETHERTYPE0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for filtering packets.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181f8UL: // (NIG_REG_P1_LLH_ETHERTYPE1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_ETHERTYPE1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for filtering packets.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181fcUL: // (NIG_REG_P1_LLH_ARP_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_ARP_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for ARP (filtering rules B).  Defaut is 0x0806.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18200UL: // (NIG_REG_P1_LLH_ICMPV4_NXTHDR: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_ICMPV4_NXTHDR (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: IPv4 protocol field for ICMPv4 - defaults to 0x01.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18204UL: // (NIG_REG_P1_LLH_ICMPV6_NXTHDR: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_ICMPV6_NXTHDR (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: IPv6 next header field for ICMPv6 - defaults to 0x3A.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18208UL: // (NIG_REG_P1_LLH_LLDP_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_LLDP_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Ethertype for LLDP (filtering rules C).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1820cUL: // (NIG_REG_P1_LLH_MCP_MASK_A: ACCESS:RW  DataWidth:0x5  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_MCP_MASK_A (ACCESS:RW  DataWidth:0x5  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_LLH_MCP_MASK_A_ALLMLCST :"
				, (u32_t)((data & NIG_P1_LLH_MCP_MASK_A_REG_P1_LLH_MCP_MASK_A_ALLMLCST) >> 4)
				, "\n\tP1_LLH_MCP_MASK_A_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P1_LLH_MCP_MASK_A_REG_P1_LLH_MCP_MASK_A_ETHERTYPE1) >> 3)
				, "\n\tP1_LLH_MCP_MASK_A_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P1_LLH_MCP_MASK_A_REG_P1_LLH_MCP_MASK_A_ETHERTYPE0) >> 2)
				, "\n\tP1_LLH_MCP_MASK_A_MAC5 :"
				, (u32_t)((data & NIG_P1_LLH_MCP_MASK_A_REG_P1_LLH_MCP_MASK_A_MAC5) >> 1)
				, "\n\tP1_LLH_MCP_MASK_A_MAC4 :"
				, (u32_t)((data & NIG_P1_LLH_MCP_MASK_A_REG_P1_LLH_MCP_MASK_A_MAC4) >> 0));
			break;
		case 0x18210UL: // (NIG_REG_P1_LLH_MCP_MASK_B: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_MCP_MASK_B (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_LLH_MCP_MASK_B_ICMPV6 :"
				, (u32_t)((data & NIG_P1_LLH_MCP_MASK_B_REG_P1_LLH_MCP_MASK_B_ICMPV6) >> 2)
				, "\n\tP1_LLH_MCP_MASK_B_ICMPV4 :"
				, (u32_t)((data & NIG_P1_LLH_MCP_MASK_B_REG_P1_LLH_MCP_MASK_B_ICMPV4) >> 1)
				, "\n\tP1_LLH_MCP_MASK_B_ARP :"
				, (u32_t)((data & NIG_P1_LLH_MCP_MASK_B_REG_P1_LLH_MCP_MASK_B_ARP) >> 0));
			break;
		case 0x18214UL: // (NIG_REG_P1_LLH_MCP_MASK_C_LLDP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_MCP_MASK_C_LLDP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable packets with Ethertype matching llh_lldp_type\n     to be forwarded to MCP. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18218UL: // (NIG_REG_P1_LLH_BRB1_MCP_MASK_A: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_BRB1_MCP_MASK_A (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_LLH_BRB1_MCP_MASK_A_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P1_LLH_BRB1_MCP_MASK_A_REG_P1_LLH_BRB1_MCP_MASK_A_ETHERTYPE1) >> 3)
				, "\n\tP1_LLH_BRB1_MCP_MASK_A_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P1_LLH_BRB1_MCP_MASK_A_REG_P1_LLH_BRB1_MCP_MASK_A_ETHERTYPE0) >> 2)
				, "\n\tP1_LLH_BRB1_MCP_MASK_A_MAC5 :"
				, (u32_t)((data & NIG_P1_LLH_BRB1_MCP_MASK_A_REG_P1_LLH_BRB1_MCP_MASK_A_MAC5) >> 1)
				, "\n\tP1_LLH_BRB1_MCP_MASK_A_MAC4 :"
				, (u32_t)((data & NIG_P1_LLH_BRB1_MCP_MASK_A_REG_P1_LLH_BRB1_MCP_MASK_A_MAC4) >> 0));
			break;
		case 0x1821cUL: // (NIG_REG_P1_LLH_MCP_OVMASK_B: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_MCP_OVMASK_B (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_LLH_MCP_OVMASK_B_PF_OUTER_VLAN :"
				, (u32_t)((data & NIG_P1_LLH_MCP_OVMASK_B_REG_P1_LLH_MCP_OVMASK_B_PF_OUTER_VLAN) >> 3)
				, "\n\tP1_LLH_MCP_OVMASK_B_OUTER_VLAN_ID :"
				, (u32_t)((data & NIG_P1_LLH_MCP_OVMASK_B_REG_P1_LLH_MCP_OVMASK_B_OUTER_VLAN_ID) >> 2)
				, "\n\tP1_LLH_MCP_OVMASK_B_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_P1_LLH_MCP_OVMASK_B_REG_P1_LLH_MCP_OVMASK_B_NO_OUTER_VLAN) >> 1)
				, "\n\tP1_LLH_MCP_OVMASK_B_OUTER_VLAN :"
				, (u32_t)((data & NIG_P1_LLH_MCP_OVMASK_B_REG_P1_LLH_MCP_OVMASK_B_OUTER_VLAN) >> 0));
			break;
		case 0x18220UL: // (NIG_REG_P1_LLH_MCP_OVMASK_C: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_MCP_OVMASK_C (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_LLH_MCP_OVMASK_C_PF_OUTER_VLAN :"
				, (u32_t)((data & NIG_P1_LLH_MCP_OVMASK_C_REG_P1_LLH_MCP_OVMASK_C_PF_OUTER_VLAN) >> 3)
				, "\n\tP1_LLH_MCP_OVMASK_C_OUTER_VLAN_ID :"
				, (u32_t)((data & NIG_P1_LLH_MCP_OVMASK_C_REG_P1_LLH_MCP_OVMASK_C_OUTER_VLAN_ID) >> 2)
				, "\n\tP1_LLH_MCP_OVMASK_C_NO_OUTER_VLAN :"
				, (u32_t)((data & NIG_P1_LLH_MCP_OVMASK_C_REG_P1_LLH_MCP_OVMASK_C_NO_OUTER_VLAN) >> 1)
				, "\n\tP1_LLH_MCP_OVMASK_C_OUTER_VLAN :"
				, (u32_t)((data & NIG_P1_LLH_MCP_OVMASK_C_REG_P1_LLH_MCP_OVMASK_C_OUTER_VLAN) >> 0));
			break;
		case 0x18224UL: // (NIG_REG_P1_FRAME_CRACKER_SOFT_RST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_FRAME_CRACKER_SOFT_RST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Soft reset bit for frame cracker - not expected to be used\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18228UL: // (NIG_REG_P1_FCOE_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_FCOE_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: FCOE Ethertype - default is 0x8906.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1822cUL: // (NIG_REG_P1_LLH_MNG_TAG_RM: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_MNG_TAG_RM (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: L2 tag removal configuration for RX management traffic.  \n    Bit mapped as follow: bit 0 - outer VLAN; bit 1 - inner VLAN; bit 2 - \n    LLC/SNAP; bit 3 - tag 0; bit 4 - tag 1; bit 5 - tag 2.  Set these bits\n     to 1's to enable the removal of the corresponding L2 tags when they are\n     present in the receive packet.  Clear the bit to keep the tag in the \n    receive packet.  This configuration is ORed with {5'd0;llh0_mng_outer_vlan_strip\n    } for backward compatibility.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18234UL: // (NIG_REG_P1_TX_ARB_CLIENT_IS_STRICT: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CLIENT_IS_STRICT (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: Specify whether the client competes directly in the strict\n     priority arbiter.  The bits are mapped according to client ID  (client\n     IDs are defined in tx_arb_priority_client2):  0-management; 1-debug traffic\n     from this port; 2-debug traffic from other port; 3-COS0 traffic; 4-COS1\n     traffic; 5-COS2 traffic; 6-COS3 traffic; 7-COS4 traffic; 8-COS5 traffic\n    .  Default value is set to enable strict priorities for all clients.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18238UL: // (NIG_REG_P1_TX_ARB_CLIENT_IS_SUBJECT2WFQ: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CLIENT_IS_SUBJECT2WFQ (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: Specify whether the client is subject to WFQ credit blocking\n    .  The bits are mapped according to client ID (client IDs are defined \n    in tx_arb_priority_client2):  0-management; 1-debug traffic from this \n    port; 2-debug traffic from other port; 3-COS0 traffic; 4-COS1 traffic; \n    5-COS2 traffic; 6-COS3 traffic; 7-COS4 traffic; 8-COS5 traffic.  Default\n     value is 0 for not using WFQ credit blocking.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18240UL: // (NIG_REG_P1_TX_ARB_NUM_STRICT_ARB_SLOTS: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_NUM_STRICT_ARB_SLOTS (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Specify the number of strict priority arbitration slots between\n     two round-robin arbitration slots to avoid starvation.  A value of 0 \n    means no strict priority cycles - the strict priority with anti-starvation\n     arbiter becomes a round-robin arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18244UL: // (NIG_REG_P1_TX_ARB_CREDIT_WEIGHT_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_WEIGHT_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     0 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18248UL: // (NIG_REG_P1_TX_ARB_CREDIT_WEIGHT_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_WEIGHT_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     1 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1824cUL: // (NIG_REG_P1_TX_ARB_CREDIT_WEIGHT_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_WEIGHT_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     2 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18250UL: // (NIG_REG_P1_TX_ARB_CREDIT_WEIGHT_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_WEIGHT_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     3 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18254UL: // (NIG_REG_P1_TX_ARB_CREDIT_WEIGHT_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_WEIGHT_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     4 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18258UL: // (NIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 0 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1825cUL: // (NIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 1 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18260UL: // (NIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 2 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18264UL: // (NIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 3 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18268UL: // (NIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 4 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1826cUL: // (NIG_REG_P1_XCM_TYPE_VALID: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_XCM_TYPE_VALID (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_XCM_TYPE_VALID_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P1_XCM_TYPE_VALID_REG_P1_XCM_TYPE_VALID_ETHERTYPE1) >> 3)
				, "\n\tP1_XCM_TYPE_VALID_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P1_XCM_TYPE_VALID_REG_P1_XCM_TYPE_VALID_ETHERTYPE0) >> 2)
				, "\n\tP1_XCM_TYPE_VALID_BCN :"
				, (u32_t)((data & NIG_P1_XCM_TYPE_VALID_REG_P1_XCM_TYPE_VALID_BCN) >> 1)
				, "\n\tP1_XCM_TYPE_VALID_MCF :"
				, (u32_t)((data & NIG_P1_XCM_TYPE_VALID_REG_P1_XCM_TYPE_VALID_MCF) >> 0));
			break;
		case 0x18270UL: // (NIG_REG_P1_TX_ARB_DBG_CLIENT_DISABLE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_DBG_CLIENT_DISABLE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Set these bits to disable debug traffic at the inputs to \n    the ETS arbiter.  Bit 0 is for debug traffic from the current path.  Bit\n     1 is for debug traffic from the other path.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18288UL: // (NIG_REG_P0_LLH_GLOBAL_ERROR_MASK: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_GLOBAL_ERROR_MASK (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_LLH_BRB1_ERROR_MASK :"
				, (u32_t)((data & NIG_P0_LLH_GLOBAL_ERROR_MASK_REG_P0_LLH_BRB1_ERROR_MASK) >> 2)
				, "\n\tP0_LLH_MCP_ERROR_MASK :"
				, (u32_t)((data & NIG_P0_LLH_GLOBAL_ERROR_MASK_REG_P0_LLH_MCP_ERROR_MASK) >> 1)
				, "\n\tP0_LLH_XCM_ERROR_MASK :"
				, (u32_t)((data & NIG_P0_LLH_GLOBAL_ERROR_MASK_REG_P0_LLH_XCM_ERROR_MASK) >> 0));
			break;
		case 0x1828cUL: // (NIG_REG_P1_COS_DRAIN_EN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_COS_DRAIN_EN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Set these bits to enable the drain mode for COS0 and COS1\n    .  Bit 0 is for COS0 flow.  Bit 1 is for COS1 flow.  When enabled -- draining\n     of the corrresponding PBF COS FIFO starts immediately - packet data are\n     dropped and not forwarded to the MAC.  When disabled--draining stops \n    at the next packet boundary. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18290UL: // (NIG_REG_P1_LLH_GLOBAL_ERROR_MASK: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_GLOBAL_ERROR_MASK (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_LLH_BRB1_ERROR_MASK :"
				, (u32_t)((data & NIG_P1_LLH_GLOBAL_ERROR_MASK_REG_P1_LLH_BRB1_ERROR_MASK) >> 2)
				, "\n\tP1_LLH_MCP_ERROR_MASK :"
				, (u32_t)((data & NIG_P1_LLH_GLOBAL_ERROR_MASK_REG_P1_LLH_MCP_ERROR_MASK) >> 1)
				, "\n\tP1_LLH_XCM_ERROR_MASK :"
				, (u32_t)((data & NIG_P1_LLH_GLOBAL_ERROR_MASK_REG_P1_LLH_XCM_ERROR_MASK) >> 0));
			break;
		case 0x182c4UL: // (NIG_REG_P0_TX_MNG_DESTINATION: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MNG_DESTINATION (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Destination of the packet that will be loaded through the\n     UMP-to-NIG interface.  Value of 1 indicates to send to the network.  \n    Value of 2 indicates to send to the host.  Value of 3 indicates to send\n     to both the network and host.  Value of 0 is invalid and causes the packet\n     to be not loaded into the Management TX buffers.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182c8UL: // (NIG_REG_P0_TX_MNG_HOST_CTRL: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MNG_HOST_CTRL (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_TX_MNG_HOST_OUTER_VLAN_HEADER_INSERT :"
				, (u32_t)((data & NIG_P0_TX_MNG_HOST_CTRL_REG_P0_TX_MNG_HOST_OUTER_VLAN_HEADER_INSERT) >> 19)
				, "\n\tP0_TX_MNG_HOST_VNIC_NUM :"
				, (u32_t)((data & NIG_P0_TX_MNG_HOST_CTRL_REG_P0_TX_MNG_HOST_VNIC_NUM) >> 17)
				, "\n\tP0_TX_MNG_HOST_PORT_ID_TAG :"
				, (u32_t)((data & NIG_P0_TX_MNG_HOST_CTRL_REG_P0_TX_MNG_HOST_PORT_ID_TAG) >> 16)
				, "\n\tP0_TX_MNG_HOST_SIDEBAND_INFO :"
				, (u32_t)((data & NIG_P0_TX_MNG_HOST_CTRL_REG_P0_TX_MNG_HOST_SIDEBAND_INFO) >> 0));
			break;
		case 0x182ccUL: // (NIG_REG_P0_TX_MNG_HOST_OUTER_VLAN_MSB: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MNG_HOST_OUTER_VLAN_MSB (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: The 20 most significant bits of the outer VLAN header to \n    be inserted into MCP packets that are forwarded to the host.  These bits\n     compose of 16-bit Ethertype; 3-bit priority; and 1-bit canonical format\n     indicator.  The 12 LSB bits of the inserted VLAN header is selected from\n     the per-PF VLAN IDs based on the configured VNIC number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182d0UL: // (NIG_REG_P0_TX_MNG_HOST_FIFO_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MNG_HOST_FIFO_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for MCP TX FIFO used for fowarding packets to the host\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182d4UL: // (NIG_REG_P1_TX_MNG_DESTINATION: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MNG_DESTINATION (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Destination of the packet that will be loaded through the\n     UMP-to-NIG interface.  Value of 1 indicates to send to the network.  \n    Value of 2 indicates to send to the host.  Value of 3 indicates to send\n     to both the network and host.  Value of 0 is invalid and causes the packet\n     to be not loaded into the Management TX buffers.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182d8UL: // (NIG_REG_P1_TX_MNG_HOST_CTRL: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MNG_HOST_CTRL (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_TX_MNG_HOST_OUTER_VLAN_HEADER_INSERT :"
				, (u32_t)((data & NIG_P1_TX_MNG_HOST_CTRL_REG_P1_TX_MNG_HOST_OUTER_VLAN_HEADER_INSERT) >> 19)
				, "\n\tP1_TX_MNG_HOST_VNIC_NUM :"
				, (u32_t)((data & NIG_P1_TX_MNG_HOST_CTRL_REG_P1_TX_MNG_HOST_VNIC_NUM) >> 17)
				, "\n\tP1_TX_MNG_HOST_PORT_ID_TAG :"
				, (u32_t)((data & NIG_P1_TX_MNG_HOST_CTRL_REG_P1_TX_MNG_HOST_PORT_ID_TAG) >> 16)
				, "\n\tP1_TX_MNG_HOST_SIDEBAND_INFO :"
				, (u32_t)((data & NIG_P1_TX_MNG_HOST_CTRL_REG_P1_TX_MNG_HOST_SIDEBAND_INFO) >> 0));
			break;
		case 0x182dcUL: // (NIG_REG_P1_TX_MNG_HOST_OUTER_VLAN_MSB: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MNG_HOST_OUTER_VLAN_MSB (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: The 20 most significant bits of the outer VLAN header to \n    be inserted into MCP packets that are forwarded to the host.  These bits\n     compose of 16-bit Ethertype; 3-bit priority; and 1-bit canonical format\n     indicator.  The 12 LSB bits of the inserted VLAN header is selected from\n     the per-PF VLAN IDs based on the configured VNIC number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182e0UL: // (NIG_REG_P1_TX_MNG_HOST_FIFO_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MNG_HOST_FIFO_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for MCP TX FIFO used for fowarding packets to the host\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182f4UL: // (NIG_REG_P0_TX_MNG_HOST_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MNG_HOST_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: MCP-to-host path enable.  Set this bit to enable the routing\n     of MCP packets to BRB LB interface to forward the packet to the host.  \n    All packets from MCP are forwarded to the network when this bit is cleared\n     - regardless of the configured destination in tx_mng_destination register\n    .  When MCP-to-host paths for both ports 0 and 1 are disabled - the arbiter\n     for BRB LB interface is bypassed and PBF LB traffic is always selected\n     to send to BRB LB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182f8UL: // (NIG_REG_P1_TX_MNG_HOST_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MNG_HOST_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: MCP-to-host path enable.  Set this bit to enable the routing\n     of MCP packets to BRB LB interface to forward the packet to the host.  \n    All packets from MCP are forwarded to the network when this bit is cleared\n     - regardless of the configured destination in tx_mng_destination register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1835cUL: // (NIG_REG_P0_TX_HOST_MNG_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_HOST_MNG_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Host-to-MCP path enable.  Set this bit to enable the routing\n     of management packets from PBF interface toward MCP when the criteria\n     for the MCP filters are met.  All packets from PBF are forwarded to the\n     network when this bit is cleared.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18360UL: // (NIG_REG_P0_TLLH_MCP_MASK: ACCESS:RW  DataWidth:0x1d  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_MCP_MASK (ACCESS:RW  DataWidth:0x1d  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_TLLH_MCP_MASK_IPV6_MLCST :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_IPV6_MLCST) >> 28)
				, "\n\tP0_TLLH_MCP_MASK_MAC3 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_MAC3) >> 27)
				, "\n\tP0_TLLH_MCP_MASK_NO_VLAN :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_NO_VLAN) >> 26)
				, "\n\tP0_TLLH_MCP_MASK_VLAN :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_VLAN) >> 25)
				, "\n\tP0_TLLH_MCP_MASK_VLAN_ID2 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_VLAN_ID2) >> 24)
				, "\n\tP0_TLLH_MCP_MASK_VLAN_ID1 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_VLAN_ID1) >> 23)
				, "\n\tP0_TLLH_MCP_MASK_VLAN_ID0 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_VLAN_ID0) >> 22)
				, "\n\tP0_TLLH_MCP_MASK_TCP2 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_TCP2) >> 21)
				, "\n\tP0_TLLH_MCP_MASK_TCP1 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_TCP1) >> 20)
				, "\n\tP0_TLLH_MCP_MASK_TCP0 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_TCP0) >> 19)
				, "\n\tP0_TLLH_MCP_MASK_NTBS_T_DST :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_NTBS_T_DST) >> 18)
				, "\n\tP0_TLLH_MCP_MASK_UDP2 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_UDP2) >> 17)
				, "\n\tP0_TLLH_MCP_MASK_UDP1 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_UDP1) >> 16)
				, "\n\tP0_TLLH_MCP_MASK_UDP0 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_UDP0) >> 15)
				, "\n\tP0_TLLH_MCP_MASK_NTBS_U_DST :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_NTBS_U_DST) >> 14)
				, "\n\tP0_TLLH_MCP_MASK_DHCP :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_DHCP) >> 13)
				, "\n\tP0_TLLH_MCP_MASK_RMCP :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_RMCP) >> 12)
				, "\n\tP0_TLLH_MCP_MASK_NTBS_T_SRC :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_NTBS_T_SRC) >> 11)
				, "\n\tP0_TLLH_MCP_MASK_NTBS_U_SRC :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_NTBS_U_SRC) >> 10)
				, "\n\tP0_TLLH_MCP_MASK_IP2 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_IP2) >> 9)
				, "\n\tP0_TLLH_MCP_MASK_IP1 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_IP1) >> 8)
				, "\n\tP0_TLLH_MCP_MASK_IP0 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_IP0) >> 7)
				, "\n\tP0_TLLH_MCP_MASK_ARP :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_ARP) >> 6)
				, "\n\tP0_TLLH_MCP_MASK_MAC2 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_MAC2) >> 5)
				, "\n\tP0_TLLH_MCP_MASK_MAC1 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_MAC1) >> 4)
				, "\n\tP0_TLLH_MCP_MASK_MAC0 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_MAC0) >> 3)
				, "\n\tP0_TLLH_MCP_MASK_UNCST :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_UNCST) >> 2)
				, "\n\tP0_TLLH_MCP_MASK_MLCST :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_MLCST) >> 1)
				, "\n\tP0_TLLH_MCP_MASK_BRCST :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_REG_P0_TLLH_MCP_MASK_BRCST) >> 0));
			break;
		case 0x18364UL: // (NIG_REG_P0_TLLH_MCP_MASK_EXT: ACCESS:RW  DataWidth:0x5  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_MCP_MASK_EXT (ACCESS:RW  DataWidth:0x5  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_TLLH_MCP_MASK_EXT_ALLMLCST :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_EXT_REG_P0_TLLH_MCP_MASK_EXT_ALLMLCST) >> 4)
				, "\n\tP0_TLLH_MCP_MASK_EXT_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_EXT_REG_P0_TLLH_MCP_MASK_EXT_ETHERTYPE1) >> 3)
				, "\n\tP0_TLLH_MCP_MASK_EXT_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_EXT_REG_P0_TLLH_MCP_MASK_EXT_ETHERTYPE0) >> 2)
				, "\n\tP0_TLLH_MCP_MASK_EXT_MAC5 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_EXT_REG_P0_TLLH_MCP_MASK_EXT_MAC5) >> 1)
				, "\n\tP0_TLLH_MCP_MASK_EXT_MAC4 :"
				, (u32_t)((data & NIG_P0_TLLH_MCP_MASK_EXT_REG_P0_TLLH_MCP_MASK_EXT_MAC4) >> 0));
			break;
		case 0x18368UL: // (NIG_REG_P0_TLLH_NTWK_DRV_MASK: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_NTWK_DRV_MASK (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_TLLH_NTWK_DRV_MASK_ALLMLCST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DRV_MASK_REG_P0_TLLH_NTWK_DRV_MASK_ALLMLCST) >> 5)
				, "\n\tP0_TLLH_NTWK_DRV_MASK_NO_VLAN :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DRV_MASK_REG_P0_TLLH_NTWK_DRV_MASK_NO_VLAN) >> 4)
				, "\n\tP0_TLLH_NTWK_DRV_MASK_VLAN :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DRV_MASK_REG_P0_TLLH_NTWK_DRV_MASK_VLAN) >> 3)
				, "\n\tP0_TLLH_NTWK_DRV_MASK_UNCST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DRV_MASK_REG_P0_TLLH_NTWK_DRV_MASK_UNCST) >> 2)
				, "\n\tP0_TLLH_NTWK_DRV_MASK_MLCST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DRV_MASK_REG_P0_TLLH_NTWK_DRV_MASK_MLCST) >> 1)
				, "\n\tP0_TLLH_NTWK_DRV_MASK_BRCST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DRV_MASK_REG_P0_TLLH_NTWK_DRV_MASK_BRCST) >> 0));
			break;
		case 0x1836cUL: // (NIG_REG_P0_TLLH_NTWK_MCP_MASK: ACCESS:RW  DataWidth:0x1c  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_NTWK_MCP_MASK (ACCESS:RW  DataWidth:0x1c  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_TLLH_NTWK_MCP_MASK_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_ETHERTYPE1) >> 27)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_ETHERTYPE0) >> 26)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_MAC5 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_MAC5) >> 25)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_MAC4 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_MAC4) >> 24)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_IPV6_MLCST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_IPV6_MLCST) >> 23)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_MAC3 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_MAC3) >> 22)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_VLAN_ID2 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_VLAN_ID2) >> 21)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_VLAN_ID1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_VLAN_ID1) >> 20)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_VLAN_ID0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_VLAN_ID0) >> 19)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_TCP2 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_TCP2) >> 18)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_TCP1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_TCP1) >> 17)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_TCP0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_TCP0) >> 16)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_NTBIOS_TCP_DST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_NTBIOS_TCP_DST) >> 15)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_UDP2 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_UDP2) >> 14)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_UDP1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_UDP1) >> 13)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_UDP0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_UDP0) >> 12)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_NTBIOS_UDP_DST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_NTBIOS_UDP_DST) >> 11)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_DHCP :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_DHCP) >> 10)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_RMCP :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_RMCP) >> 9)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_NTBIOS_TCP_SRC :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_NTBIOS_TCP_SRC) >> 8)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_NTBIOS_UDP_SRC :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_NTBIOS_UDP_SRC) >> 7)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_IP2 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_IP2) >> 6)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_IP1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_IP1) >> 5)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_IP0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_IP0) >> 4)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_ARP :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_ARP) >> 3)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_MAC2 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_MAC2) >> 2)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_MAC1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_MAC1) >> 1)
				, "\n\tP0_TLLH_NTWK_MCP_MASK_MAC0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_MCP_MASK_REG_P0_TLLH_NTWK_MCP_MASK_MAC0) >> 0));
			break;
		case 0x18370UL: // (NIG_REG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_TLLH_NTWK_DONT_FWD_DRV_MASK_ALLMLCST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_ALLMLCST) >> 5)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_DRV_MASK_NO_VLAN :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_NO_VLAN) >> 4)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_DRV_MASK_VLAN :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_VLAN) >> 3)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_DRV_MASK_UNCST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_UNCST) >> 2)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_DRV_MASK_MLCST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_MLCST) >> 1)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_DRV_MASK_BRCST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P0_TLLH_NTWK_DONT_FWD_DRV_MASK_BRCST) >> 0));
			break;
		case 0x18374UL: // (NIG_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK: ACCESS:RW  DataWidth:0x1c  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK (ACCESS:RW  DataWidth:0x1c  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_ETHERTYPE1) >> 27)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_ETHERTYPE0) >> 26)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC5 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC5) >> 25)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC4 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC4) >> 24)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_IPV6_MLCST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_IPV6_MLCST) >> 23)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC3 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC3) >> 22)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID2 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID2) >> 21)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID1) >> 20)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID0) >> 19)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP2 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP2) >> 18)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP1) >> 17)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP0) >> 16)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_TCP_DST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_TCP_DST) >> 15)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP2 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP2) >> 14)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP1) >> 13)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP0) >> 12)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_UDP_DST :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_UDP_DST) >> 11)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_DHCP :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_DHCP) >> 10)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_RMCP :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_RMCP) >> 9)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_TCP_SRC :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_TCP_SRC) >> 8)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_UDP_SRC :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_UDP_SRC) >> 7)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_IP2 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_IP2) >> 6)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_IP1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_IP1) >> 5)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_IP0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_IP0) >> 4)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_ARP :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_ARP) >> 3)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC2 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC2) >> 2)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC1 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC1) >> 1)
				, "\n\tP0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC0 :"
				, (u32_t)((data & NIG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P0_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC0) >> 0));
			break;
		case 0x18378UL: // (NIG_REG_P0_TLLH_GLOBAL_ERROR_MASK: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_GLOBAL_ERROR_MASK (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP0_TLLH_NTWK_ERROR_MASK :"
				, (u32_t)((data & NIG_P0_TLLH_GLOBAL_ERROR_MASK_REG_P0_TLLH_NTWK_ERROR_MASK) >> 1)
				, "\n\tP0_TLLH_MCP_ERROR_MASK :"
				, (u32_t)((data & NIG_P0_TLLH_GLOBAL_ERROR_MASK_REG_P0_TLLH_MCP_ERROR_MASK) >> 0));
			break;
		case 0x1837cUL: // (NIG_REG_P0_HBUF_MEM_RBC_ACCESS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HBUF_MEM_RBC_ACCESS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. This bit is for selecting the RX management memory\n     for register read/write access through ingress_mng*_fifo register space\n    .  Set to 1 to select the Host-to-BMC buffer (HBUF).  Set to 0 to select\n     the MCP RX buffer:w.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18380UL: // (NIG_REG_P0_TLLH_FIFO_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_FIFO_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM for TLLH FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18384UL: // (NIG_REG_P0_HBUF_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HBUF_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for Host-to-BMC buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18388UL: // (NIG_REG_P0_TLLH_FRAME_CRACKER_SOFT_RST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_FRAME_CRACKER_SOFT_RST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Soft reset bit for frame cracker - not expected to be used\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1838cUL: // (NIG_REG_P1_TX_HOST_MNG_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_HOST_MNG_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Host-to-MCP path enable.  Set this bit to enable the routing\n     of management packets from PBF interface toward MCP when the criteria\n     for the MCP filters are met.  All packets from PBF are forwarded to the\n     network when this bit is cleared.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18390UL: // (NIG_REG_P1_TLLH_MCP_MASK: ACCESS:RW  DataWidth:0x1d  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_MCP_MASK (ACCESS:RW  DataWidth:0x1d  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_TLLH_MCP_MASK_IPV6_MLCST :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_IPV6_MLCST) >> 28)
				, "\n\tP1_TLLH_MCP_MASK_MAC3 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_MAC3) >> 27)
				, "\n\tP1_TLLH_MCP_MASK_NO_VLAN :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_NO_VLAN) >> 26)
				, "\n\tP1_TLLH_MCP_MASK_VLAN :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_VLAN) >> 25)
				, "\n\tP1_TLLH_MCP_MASK_VLAN_ID2 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_VLAN_ID2) >> 24)
				, "\n\tP1_TLLH_MCP_MASK_VLAN_ID1 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_VLAN_ID1) >> 23)
				, "\n\tP1_TLLH_MCP_MASK_VLAN_ID0 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_VLAN_ID0) >> 22)
				, "\n\tP1_TLLH_MCP_MASK_TCP2 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_TCP2) >> 21)
				, "\n\tP1_TLLH_MCP_MASK_TCP1 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_TCP1) >> 20)
				, "\n\tP1_TLLH_MCP_MASK_TCP0 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_TCP0) >> 19)
				, "\n\tP1_TLLH_MCP_MASK_NTBS_T_DST :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_NTBS_T_DST) >> 18)
				, "\n\tP1_TLLH_MCP_MASK_UDP2 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_UDP2) >> 17)
				, "\n\tP1_TLLH_MCP_MASK_UDP1 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_UDP1) >> 16)
				, "\n\tP1_TLLH_MCP_MASK_UDP0 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_UDP0) >> 15)
				, "\n\tP1_TLLH_MCP_MASK_NTBS_U_DST :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_NTBS_U_DST) >> 14)
				, "\n\tP1_TLLH_MCP_MASK_DHCP :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_DHCP) >> 13)
				, "\n\tP1_TLLH_MCP_MASK_RMCP :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_RMCP) >> 12)
				, "\n\tP1_TLLH_MCP_MASK_NTBS_T_SRC :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_NTBS_T_SRC) >> 11)
				, "\n\tP1_TLLH_MCP_MASK_NTBS_U_SRC :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_NTBS_U_SRC) >> 10)
				, "\n\tP1_TLLH_MCP_MASK_IP2 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_IP2) >> 9)
				, "\n\tP1_TLLH_MCP_MASK_IP1 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_IP1) >> 8)
				, "\n\tP1_TLLH_MCP_MASK_IP0 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_IP0) >> 7)
				, "\n\tP1_TLLH_MCP_MASK_ARP :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_ARP) >> 6)
				, "\n\tP1_TLLH_MCP_MASK_MAC2 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_MAC2) >> 5)
				, "\n\tP1_TLLH_MCP_MASK_MAC1 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_MAC1) >> 4)
				, "\n\tP1_TLLH_MCP_MASK_MAC0 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_MAC0) >> 3)
				, "\n\tP1_TLLH_MCP_MASK_UNCST :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_UNCST) >> 2)
				, "\n\tP1_TLLH_MCP_MASK_MLCST :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_MLCST) >> 1)
				, "\n\tP1_TLLH_MCP_MASK_BRCST :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_REG_P1_TLLH_MCP_MASK_BRCST) >> 0));
			break;
		case 0x18394UL: // (NIG_REG_P1_TLLH_MCP_MASK_EXT: ACCESS:RW  DataWidth:0x5  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_MCP_MASK_EXT (ACCESS:RW  DataWidth:0x5  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_TLLH_MCP_MASK_EXT_ALLMLCST :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_EXT_REG_P1_TLLH_MCP_MASK_EXT_ALLMLCST) >> 4)
				, "\n\tP1_TLLH_MCP_MASK_EXT_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_EXT_REG_P1_TLLH_MCP_MASK_EXT_ETHERTYPE1) >> 3)
				, "\n\tP1_TLLH_MCP_MASK_EXT_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_EXT_REG_P1_TLLH_MCP_MASK_EXT_ETHERTYPE0) >> 2)
				, "\n\tP1_TLLH_MCP_MASK_EXT_MAC5 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_EXT_REG_P1_TLLH_MCP_MASK_EXT_MAC5) >> 1)
				, "\n\tP1_TLLH_MCP_MASK_EXT_MAC4 :"
				, (u32_t)((data & NIG_P1_TLLH_MCP_MASK_EXT_REG_P1_TLLH_MCP_MASK_EXT_MAC4) >> 0));
			break;
		case 0x18398UL: // (NIG_REG_P1_TLLH_NTWK_DRV_MASK: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_NTWK_DRV_MASK (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_TLLH_NTWK_DRV_MASK_ALLMLCST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DRV_MASK_REG_P1_TLLH_NTWK_DRV_MASK_ALLMLCST) >> 5)
				, "\n\tP1_TLLH_NTWK_DRV_MASK_NO_VLAN :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DRV_MASK_REG_P1_TLLH_NTWK_DRV_MASK_NO_VLAN) >> 4)
				, "\n\tP1_TLLH_NTWK_DRV_MASK_VLAN :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DRV_MASK_REG_P1_TLLH_NTWK_DRV_MASK_VLAN) >> 3)
				, "\n\tP1_TLLH_NTWK_DRV_MASK_UNCST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DRV_MASK_REG_P1_TLLH_NTWK_DRV_MASK_UNCST) >> 2)
				, "\n\tP1_TLLH_NTWK_DRV_MASK_MLCST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DRV_MASK_REG_P1_TLLH_NTWK_DRV_MASK_MLCST) >> 1)
				, "\n\tP1_TLLH_NTWK_DRV_MASK_BRCST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DRV_MASK_REG_P1_TLLH_NTWK_DRV_MASK_BRCST) >> 0));
			break;
		case 0x1839cUL: // (NIG_REG_P1_TLLH_NTWK_MCP_MASK: ACCESS:RW  DataWidth:0x1c  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_NTWK_MCP_MASK (ACCESS:RW  DataWidth:0x1c  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_TLLH_NTWK_MCP_MASK_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_ETHERTYPE1) >> 27)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_ETHERTYPE0) >> 26)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_MAC5 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_MAC5) >> 25)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_MAC4 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_MAC4) >> 24)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_IPV6_MLCST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_IPV6_MLCST) >> 23)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_MAC3 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_MAC3) >> 22)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_VLAN_ID2 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_VLAN_ID2) >> 21)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_VLAN_ID1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_VLAN_ID1) >> 20)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_VLAN_ID0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_VLAN_ID0) >> 19)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_TCP2 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_TCP2) >> 18)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_TCP1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_TCP1) >> 17)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_TCP0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_TCP0) >> 16)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_NTBIOS_TCP_DST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_NTBIOS_TCP_DST) >> 15)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_UDP2 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_UDP2) >> 14)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_UDP1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_UDP1) >> 13)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_UDP0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_UDP0) >> 12)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_NTBIOS_UDP_DST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_NTBIOS_UDP_DST) >> 11)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_DHCP :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_DHCP) >> 10)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_RMCP :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_RMCP) >> 9)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_NTBIOS_TCP_SRC :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_NTBIOS_TCP_SRC) >> 8)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_NTBIOS_UDP_SRC :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_NTBIOS_UDP_SRC) >> 7)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_IP2 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_IP2) >> 6)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_IP1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_IP1) >> 5)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_IP0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_IP0) >> 4)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_ARP :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_ARP) >> 3)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_MAC2 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_MAC2) >> 2)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_MAC1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_MAC1) >> 1)
				, "\n\tP1_TLLH_NTWK_MCP_MASK_MAC0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_MCP_MASK_REG_P1_TLLH_NTWK_MCP_MASK_MAC0) >> 0));
			break;
		case 0x183a0UL: // (NIG_REG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_TLLH_NTWK_DONT_FWD_DRV_MASK_ALLMLCST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_ALLMLCST) >> 5)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_DRV_MASK_NO_VLAN :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_NO_VLAN) >> 4)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_DRV_MASK_VLAN :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_VLAN) >> 3)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_DRV_MASK_UNCST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_UNCST) >> 2)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_DRV_MASK_MLCST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_MLCST) >> 1)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_DRV_MASK_BRCST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_REG_P1_TLLH_NTWK_DONT_FWD_DRV_MASK_BRCST) >> 0));
			break;
		case 0x183a4UL: // (NIG_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK: ACCESS:RW  DataWidth:0x1c  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK (ACCESS:RW  DataWidth:0x1c  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_ETHERTYPE1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_ETHERTYPE1) >> 27)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_ETHERTYPE0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_ETHERTYPE0) >> 26)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC5 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC5) >> 25)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC4 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC4) >> 24)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_IPV6_MLCST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_IPV6_MLCST) >> 23)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC3 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC3) >> 22)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID2 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID2) >> 21)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID1) >> 20)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_VLAN_ID0) >> 19)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP2 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP2) >> 18)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP1) >> 17)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_TCP0) >> 16)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_TCP_DST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_TCP_DST) >> 15)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP2 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP2) >> 14)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP1) >> 13)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_UDP0) >> 12)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_UDP_DST :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_UDP_DST) >> 11)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_DHCP :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_DHCP) >> 10)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_RMCP :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_RMCP) >> 9)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_TCP_SRC :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_TCP_SRC) >> 8)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_UDP_SRC :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_NTBIOS_UDP_SRC) >> 7)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_IP2 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_IP2) >> 6)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_IP1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_IP1) >> 5)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_IP0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_IP0) >> 4)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_ARP :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_ARP) >> 3)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC2 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC2) >> 2)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC1 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC1) >> 1)
				, "\n\tP1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC0 :"
				, (u32_t)((data & NIG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_REG_P1_TLLH_NTWK_DONT_FWD_MCP_MASK_MAC0) >> 0));
			break;
		case 0x183a8UL: // (NIG_REG_P1_TLLH_GLOBAL_ERROR_MASK: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_GLOBAL_ERROR_MASK (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tP1_TLLH_NTWK_ERROR_MASK :"
				, (u32_t)((data & NIG_P1_TLLH_GLOBAL_ERROR_MASK_REG_P1_TLLH_NTWK_ERROR_MASK) >> 1)
				, "\n\tP1_TLLH_MCP_ERROR_MASK :"
				, (u32_t)((data & NIG_P1_TLLH_GLOBAL_ERROR_MASK_REG_P1_TLLH_MCP_ERROR_MASK) >> 0));
			break;
		case 0x183acUL: // (NIG_REG_P1_HBUF_MEM_RBC_ACCESS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HBUF_MEM_RBC_ACCESS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. This bit is for selecting the RX management memory\n     for register read/write access through ingress_mng*_fifo register space\n    .  Set to 1 to select the Host-to-BMC buffer (HBUF).  Set to 0 to select\n     the MCP RX buffer:w.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x183b0UL: // (NIG_REG_P1_TLLH_FIFO_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_FIFO_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM for TLLH FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x183b4UL: // (NIG_REG_P1_HBUF_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HBUF_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for Host-to-BMC buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x183b8UL: // (NIG_REG_P1_TLLH_FRAME_CRACKER_SOFT_RST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_FRAME_CRACKER_SOFT_RST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Soft reset bit for frame cracker - not expected to be used\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x183bcUL: // (NIG_REG_NIG_PRTY_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH1_FUNC3_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_LLH1_FUNC3_ACPI_RF_PARITY_ERR) >> 31)
				, "\n\tLLH1_FUNC2_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_LLH1_FUNC2_ACPI_RF_PARITY_ERR) >> 30)
				, "\n\tLLH1_FUNC1_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_LLH1_FUNC1_ACPI_RF_PARITY_ERR) >> 29)
				, "\n\tLLH0_FUNC3_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_LLH0_FUNC3_ACPI_RF_PARITY_ERR) >> 28)
				, "\n\tLLH0_FUNC2_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_LLH0_FUNC2_ACPI_RF_PARITY_ERR) >> 27)
				, "\n\tLLH0_FUNC1_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_LLH0_FUNC1_ACPI_RF_PARITY_ERR) >> 26)
				, "\n\tLLH1_ACPI_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_LLH1_ACPI_PARITY) >> 25)
				, "\n\tLLH0_ACPI_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_LLH0_ACPI_PARITY) >> 24)
				, "\n\tLLH1_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_LLH1_FIFO_PARITY) >> 23)
				, "\n\tLLH0_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_LLH0_FIFO_PARITY) >> 22)
				, "\n\tEGRESS_EMAC1_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_EGRESS_EMAC1_RDDATA_PARITY) >> 21)
				, "\n\tEGRESS_EMAC0_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_EGRESS_EMAC0_RDDATA_PARITY) >> 20)
				, "\n\tEGRESS_BMAC1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_EGRESS_BMAC1_PARITY) >> 19)
				, "\n\tEGRESS_BMAC0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_EGRESS_BMAC0_PARITY) >> 18)
				, "\n\tEGRESS_DELAY1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_EGRESS_DELAY1_PARITY) >> 17)
				, "\n\tEGRESS_DELAY0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_EGRESS_DELAY0_PARITY) >> 16)
				, "\n\tEGRESS_DEBUG_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_EGRESS_DEBUG_FIFO_PARITY) >> 15)
				, "\n\tEGRESS_MNG1_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_EGRESS_MNG1_FIFO_PARITY) >> 14)
				, "\n\tEGRESS_MNG0_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_EGRESS_MNG0_FIFO_PARITY) >> 13)
				, "\n\tINGRESS_LB_PBF_DELAY_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_LB_PBF_DELAY_PARITY) >> 12)
				, "\n\tINGRESS_EMAC1_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_EMAC1_RDDATA_PARITY) >> 11)
				, "\n\tINGRESS_EMAC0_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_EMAC0_RDDATA_PARITY) >> 10)
				, "\n\tINGRESS_BMAC1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_BMAC1_PARITY) >> 9)
				, "\n\tINGRESS_BMAC0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_BMAC0_PARITY) >> 8)
				, "\n\tINGRESS_MNG1_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_MNG1_PARITY1) >> 7)
				, "\n\tINGRESS_MNG0_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_MNG0_PARITY1) >> 6)
				, "\n\tINGRESS_MNG1_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_MNG1_PARITY0) >> 5)
				, "\n\tINGRESS_MNG0_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_MNG0_PARITY0) >> 4)
				, "\n\tINGRESS_EOP_LB_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_EOP_LB_PARITY) >> 3)
				, "\n\tINGRESS_EOP1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_EOP1_PARITY) >> 2)
				, "\n\tINGRESS_EOP0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_INGRESS_EOP0_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_0_REG_PARITY) >> 0));
			break;
		case 0x183c0UL: // (NIG_REG_NIG_PRTY_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH1_FUNC3_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_LLH1_FUNC3_ACPI_RF_PARITY_ERR) >> 31)
				, "\n\tLLH1_FUNC2_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_LLH1_FUNC2_ACPI_RF_PARITY_ERR) >> 30)
				, "\n\tLLH1_FUNC1_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_LLH1_FUNC1_ACPI_RF_PARITY_ERR) >> 29)
				, "\n\tLLH0_FUNC3_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_LLH0_FUNC3_ACPI_RF_PARITY_ERR) >> 28)
				, "\n\tLLH0_FUNC2_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_LLH0_FUNC2_ACPI_RF_PARITY_ERR) >> 27)
				, "\n\tLLH0_FUNC1_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_LLH0_FUNC1_ACPI_RF_PARITY_ERR) >> 26)
				, "\n\tLLH1_ACPI_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_LLH1_ACPI_PARITY) >> 25)
				, "\n\tLLH0_ACPI_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_LLH0_ACPI_PARITY) >> 24)
				, "\n\tLLH1_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_LLH1_FIFO_PARITY) >> 23)
				, "\n\tLLH0_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_LLH0_FIFO_PARITY) >> 22)
				, "\n\tEGRESS_EMAC1_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_EGRESS_EMAC1_RDDATA_PARITY) >> 21)
				, "\n\tEGRESS_EMAC0_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_EGRESS_EMAC0_RDDATA_PARITY) >> 20)
				, "\n\tEGRESS_BMAC1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_EGRESS_BMAC1_PARITY) >> 19)
				, "\n\tEGRESS_BMAC0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_EGRESS_BMAC0_PARITY) >> 18)
				, "\n\tEGRESS_DELAY1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_EGRESS_DELAY1_PARITY) >> 17)
				, "\n\tEGRESS_DELAY0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_EGRESS_DELAY0_PARITY) >> 16)
				, "\n\tEGRESS_DEBUG_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_EGRESS_DEBUG_FIFO_PARITY) >> 15)
				, "\n\tEGRESS_MNG1_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_EGRESS_MNG1_FIFO_PARITY) >> 14)
				, "\n\tEGRESS_MNG0_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_EGRESS_MNG0_FIFO_PARITY) >> 13)
				, "\n\tINGRESS_LB_PBF_DELAY_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_LB_PBF_DELAY_PARITY) >> 12)
				, "\n\tINGRESS_EMAC1_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_EMAC1_RDDATA_PARITY) >> 11)
				, "\n\tINGRESS_EMAC0_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_EMAC0_RDDATA_PARITY) >> 10)
				, "\n\tINGRESS_BMAC1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_BMAC1_PARITY) >> 9)
				, "\n\tINGRESS_BMAC0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_BMAC0_PARITY) >> 8)
				, "\n\tINGRESS_MNG1_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_MNG1_PARITY1) >> 7)
				, "\n\tINGRESS_MNG0_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_MNG0_PARITY1) >> 6)
				, "\n\tINGRESS_MNG1_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_MNG1_PARITY0) >> 5)
				, "\n\tINGRESS_MNG0_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_MNG0_PARITY0) >> 4)
				, "\n\tINGRESS_EOP_LB_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_EOP_LB_PARITY) >> 3)
				, "\n\tINGRESS_EOP1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_EOP1_PARITY) >> 2)
				, "\n\tINGRESS_EOP0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_INGRESS_EOP0_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_0_REG_PARITY) >> 0));
			break;
		case 0x183c4UL: // (NIG_REG_NIG_PRTY_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH1_FUNC3_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_LLH1_FUNC3_ACPI_RF_PARITY_ERR) >> 31)
				, "\n\tLLH1_FUNC2_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_LLH1_FUNC2_ACPI_RF_PARITY_ERR) >> 30)
				, "\n\tLLH1_FUNC1_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_LLH1_FUNC1_ACPI_RF_PARITY_ERR) >> 29)
				, "\n\tLLH0_FUNC3_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_LLH0_FUNC3_ACPI_RF_PARITY_ERR) >> 28)
				, "\n\tLLH0_FUNC2_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_LLH0_FUNC2_ACPI_RF_PARITY_ERR) >> 27)
				, "\n\tLLH0_FUNC1_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_LLH0_FUNC1_ACPI_RF_PARITY_ERR) >> 26)
				, "\n\tLLH1_ACPI_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_LLH1_ACPI_PARITY) >> 25)
				, "\n\tLLH0_ACPI_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_LLH0_ACPI_PARITY) >> 24)
				, "\n\tLLH1_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_LLH1_FIFO_PARITY) >> 23)
				, "\n\tLLH0_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_LLH0_FIFO_PARITY) >> 22)
				, "\n\tEGRESS_EMAC1_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_EGRESS_EMAC1_RDDATA_PARITY) >> 21)
				, "\n\tEGRESS_EMAC0_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_EGRESS_EMAC0_RDDATA_PARITY) >> 20)
				, "\n\tEGRESS_BMAC1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_EGRESS_BMAC1_PARITY) >> 19)
				, "\n\tEGRESS_BMAC0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_EGRESS_BMAC0_PARITY) >> 18)
				, "\n\tEGRESS_DELAY1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_EGRESS_DELAY1_PARITY) >> 17)
				, "\n\tEGRESS_DELAY0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_EGRESS_DELAY0_PARITY) >> 16)
				, "\n\tEGRESS_DEBUG_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_EGRESS_DEBUG_FIFO_PARITY) >> 15)
				, "\n\tEGRESS_MNG1_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_EGRESS_MNG1_FIFO_PARITY) >> 14)
				, "\n\tEGRESS_MNG0_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_EGRESS_MNG0_FIFO_PARITY) >> 13)
				, "\n\tINGRESS_LB_PBF_DELAY_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_LB_PBF_DELAY_PARITY) >> 12)
				, "\n\tINGRESS_EMAC1_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_EMAC1_RDDATA_PARITY) >> 11)
				, "\n\tINGRESS_EMAC0_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_EMAC0_RDDATA_PARITY) >> 10)
				, "\n\tINGRESS_BMAC1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_BMAC1_PARITY) >> 9)
				, "\n\tINGRESS_BMAC0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_BMAC0_PARITY) >> 8)
				, "\n\tINGRESS_MNG1_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_MNG1_PARITY1) >> 7)
				, "\n\tINGRESS_MNG0_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_MNG0_PARITY1) >> 6)
				, "\n\tINGRESS_MNG1_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_MNG1_PARITY0) >> 5)
				, "\n\tINGRESS_MNG0_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_MNG0_PARITY0) >> 4)
				, "\n\tINGRESS_EOP_LB_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_EOP_LB_PARITY) >> 3)
				, "\n\tINGRESS_EOP1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_EOP1_PARITY) >> 2)
				, "\n\tINGRESS_EOP0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_INGRESS_EOP0_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_0_REG_PARITY) >> 0));
			break;
		case 0x183c8UL: // (NIG_REG_NIG_PRTY_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLH1_FUNC3_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_LLH1_FUNC3_ACPI_RF_PARITY_ERR) >> 31)
				, "\n\tLLH1_FUNC2_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_LLH1_FUNC2_ACPI_RF_PARITY_ERR) >> 30)
				, "\n\tLLH1_FUNC1_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_LLH1_FUNC1_ACPI_RF_PARITY_ERR) >> 29)
				, "\n\tLLH0_FUNC3_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_LLH0_FUNC3_ACPI_RF_PARITY_ERR) >> 28)
				, "\n\tLLH0_FUNC2_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_LLH0_FUNC2_ACPI_RF_PARITY_ERR) >> 27)
				, "\n\tLLH0_FUNC1_ACPI_RF_PARITY_ERR :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_LLH0_FUNC1_ACPI_RF_PARITY_ERR) >> 26)
				, "\n\tLLH1_ACPI_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_LLH1_ACPI_PARITY) >> 25)
				, "\n\tLLH0_ACPI_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_LLH0_ACPI_PARITY) >> 24)
				, "\n\tLLH1_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_LLH1_FIFO_PARITY) >> 23)
				, "\n\tLLH0_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_LLH0_FIFO_PARITY) >> 22)
				, "\n\tEGRESS_EMAC1_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_EGRESS_EMAC1_RDDATA_PARITY) >> 21)
				, "\n\tEGRESS_EMAC0_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_EGRESS_EMAC0_RDDATA_PARITY) >> 20)
				, "\n\tEGRESS_BMAC1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_EGRESS_BMAC1_PARITY) >> 19)
				, "\n\tEGRESS_BMAC0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_EGRESS_BMAC0_PARITY) >> 18)
				, "\n\tEGRESS_DELAY1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_EGRESS_DELAY1_PARITY) >> 17)
				, "\n\tEGRESS_DELAY0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_EGRESS_DELAY0_PARITY) >> 16)
				, "\n\tEGRESS_DEBUG_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_EGRESS_DEBUG_FIFO_PARITY) >> 15)
				, "\n\tEGRESS_MNG1_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_EGRESS_MNG1_FIFO_PARITY) >> 14)
				, "\n\tEGRESS_MNG0_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_EGRESS_MNG0_FIFO_PARITY) >> 13)
				, "\n\tINGRESS_LB_PBF_DELAY_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_LB_PBF_DELAY_PARITY) >> 12)
				, "\n\tINGRESS_EMAC1_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_EMAC1_RDDATA_PARITY) >> 11)
				, "\n\tINGRESS_EMAC0_RDDATA_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_EMAC0_RDDATA_PARITY) >> 10)
				, "\n\tINGRESS_BMAC1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_BMAC1_PARITY) >> 9)
				, "\n\tINGRESS_BMAC0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_BMAC0_PARITY) >> 8)
				, "\n\tINGRESS_MNG1_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_MNG1_PARITY1) >> 7)
				, "\n\tINGRESS_MNG0_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_MNG0_PARITY1) >> 6)
				, "\n\tINGRESS_MNG1_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_MNG1_PARITY0) >> 5)
				, "\n\tINGRESS_MNG0_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_MNG0_PARITY0) >> 4)
				, "\n\tINGRESS_EOP_LB_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_EOP_LB_PARITY) >> 3)
				, "\n\tINGRESS_EOP1_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_EOP1_PARITY) >> 2)
				, "\n\tINGRESS_EOP0_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_INGRESS_EOP0_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_0_REG_PARITY) >> 0));
			break;
		case 0x183ccUL: // (NIG_REG_NIG_PRTY_STS_1: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_STS_1 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Parity register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEGRESS_DELAY5_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_EGRESS_DELAY5_PARITY) >> 15)
				, "\n\tEGRESS_DELAY4_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_EGRESS_DELAY4_PARITY) >> 14)
				, "\n\tEGRESS_DELAY3_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_EGRESS_DELAY3_PARITY) >> 13)
				, "\n\tEGRESS_DELAY2_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_EGRESS_DELAY2_PARITY) >> 12)
				, "\n\tP1_TX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P1_TX_MACFIFO_PARITY) >> 11)
				, "\n\tP0_TX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P0_TX_MACFIFO_PARITY) >> 10)
				, "\n\tP1_RX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P1_RX_MACFIFO_PARITY) >> 9)
				, "\n\tP0_RX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P0_RX_MACFIFO_PARITY) >> 8)
				, "\n\tP1_TLLH_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P1_TLLH_FIFO_PARITY) >> 7)
				, "\n\tP0_TLLH_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P0_TLLH_FIFO_PARITY) >> 6)
				, "\n\tP1_HBUF_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P1_HBUF_PARITY1) >> 5)
				, "\n\tP1_HBUF_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P1_HBUF_PARITY0) >> 4)
				, "\n\tP0_HBUF_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P0_HBUF_PARITY1) >> 3)
				, "\n\tP0_HBUF_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P0_HBUF_PARITY0) >> 2)
				, "\n\tP1_TX_MNG_HOST_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P1_TX_MNG_HOST_FIFO_PARITY) >> 1)
				, "\n\tP0_TX_MNG_HOST_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_1_REG_P0_TX_MNG_HOST_FIFO_PARITY) >> 0));
			break;
		case 0x183d0UL: // (NIG_REG_NIG_PRTY_STS_CLR_1: ACCESS:RC  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_STS_CLR_1 (ACCESS:RC  DataWidth:0x10 )" \
				"\nDescription: Parity register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEGRESS_DELAY5_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_EGRESS_DELAY5_PARITY) >> 15)
				, "\n\tEGRESS_DELAY4_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_EGRESS_DELAY4_PARITY) >> 14)
				, "\n\tEGRESS_DELAY3_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_EGRESS_DELAY3_PARITY) >> 13)
				, "\n\tEGRESS_DELAY2_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_EGRESS_DELAY2_PARITY) >> 12)
				, "\n\tP1_TX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P1_TX_MACFIFO_PARITY) >> 11)
				, "\n\tP0_TX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P0_TX_MACFIFO_PARITY) >> 10)
				, "\n\tP1_RX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P1_RX_MACFIFO_PARITY) >> 9)
				, "\n\tP0_RX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P0_RX_MACFIFO_PARITY) >> 8)
				, "\n\tP1_TLLH_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P1_TLLH_FIFO_PARITY) >> 7)
				, "\n\tP0_TLLH_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P0_TLLH_FIFO_PARITY) >> 6)
				, "\n\tP1_HBUF_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P1_HBUF_PARITY1) >> 5)
				, "\n\tP1_HBUF_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P1_HBUF_PARITY0) >> 4)
				, "\n\tP0_HBUF_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P0_HBUF_PARITY1) >> 3)
				, "\n\tP0_HBUF_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P0_HBUF_PARITY0) >> 2)
				, "\n\tP1_TX_MNG_HOST_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P1_TX_MNG_HOST_FIFO_PARITY) >> 1)
				, "\n\tP0_TX_MNG_HOST_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_CLR_1_REG_P0_TX_MNG_HOST_FIFO_PARITY) >> 0));
			break;
		case 0x183d4UL: // (NIG_REG_NIG_PRTY_STS_WR_1: ACCESS:WR  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_STS_WR_1 (ACCESS:WR  DataWidth:0x10 )" \
				"\nDescription: Parity register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEGRESS_DELAY5_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_EGRESS_DELAY5_PARITY) >> 15)
				, "\n\tEGRESS_DELAY4_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_EGRESS_DELAY4_PARITY) >> 14)
				, "\n\tEGRESS_DELAY3_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_EGRESS_DELAY3_PARITY) >> 13)
				, "\n\tEGRESS_DELAY2_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_EGRESS_DELAY2_PARITY) >> 12)
				, "\n\tP1_TX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P1_TX_MACFIFO_PARITY) >> 11)
				, "\n\tP0_TX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P0_TX_MACFIFO_PARITY) >> 10)
				, "\n\tP1_RX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P1_RX_MACFIFO_PARITY) >> 9)
				, "\n\tP0_RX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P0_RX_MACFIFO_PARITY) >> 8)
				, "\n\tP1_TLLH_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P1_TLLH_FIFO_PARITY) >> 7)
				, "\n\tP0_TLLH_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P0_TLLH_FIFO_PARITY) >> 6)
				, "\n\tP1_HBUF_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P1_HBUF_PARITY1) >> 5)
				, "\n\tP1_HBUF_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P1_HBUF_PARITY0) >> 4)
				, "\n\tP0_HBUF_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P0_HBUF_PARITY1) >> 3)
				, "\n\tP0_HBUF_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P0_HBUF_PARITY0) >> 2)
				, "\n\tP1_TX_MNG_HOST_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P1_TX_MNG_HOST_FIFO_PARITY) >> 1)
				, "\n\tP0_TX_MNG_HOST_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_STS_WR_1_REG_P0_TX_MNG_HOST_FIFO_PARITY) >> 0));
			break;
		case 0x183d8UL: // (NIG_REG_NIG_PRTY_MASK_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_MASK_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Parity mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEGRESS_DELAY5_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_EGRESS_DELAY5_PARITY) >> 15)
				, "\n\tEGRESS_DELAY4_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_EGRESS_DELAY4_PARITY) >> 14)
				, "\n\tEGRESS_DELAY3_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_EGRESS_DELAY3_PARITY) >> 13)
				, "\n\tEGRESS_DELAY2_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_EGRESS_DELAY2_PARITY) >> 12)
				, "\n\tP1_TX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P1_TX_MACFIFO_PARITY) >> 11)
				, "\n\tP0_TX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P0_TX_MACFIFO_PARITY) >> 10)
				, "\n\tP1_RX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P1_RX_MACFIFO_PARITY) >> 9)
				, "\n\tP0_RX_MACFIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P0_RX_MACFIFO_PARITY) >> 8)
				, "\n\tP1_TLLH_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P1_TLLH_FIFO_PARITY) >> 7)
				, "\n\tP0_TLLH_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P0_TLLH_FIFO_PARITY) >> 6)
				, "\n\tP1_HBUF_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P1_HBUF_PARITY1) >> 5)
				, "\n\tP1_HBUF_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P1_HBUF_PARITY0) >> 4)
				, "\n\tP0_HBUF_PARITY1 :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P0_HBUF_PARITY1) >> 3)
				, "\n\tP0_HBUF_PARITY0 :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P0_HBUF_PARITY0) >> 2)
				, "\n\tP1_TX_MNG_HOST_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P1_TX_MNG_HOST_FIFO_PARITY) >> 1)
				, "\n\tP0_TX_MNG_HOST_FIFO_PARITY :"
				, (u32_t)((data & NIG_NIG_PRTY_MASK_1_REG_P0_TX_MNG_HOST_FIFO_PARITY) >> 0));
			break;
		case 0x18500UL: // (NIG_REG_P0_ACPI_MF_GLOBAL_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_ACPI_MF_GLOBAL_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable ACPI pattern matching in multi-function\n     mode even when the per-function outer VLAN matching fails.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18504UL: // (NIG_REG_P0_LLH_CLS_TYPE_IVLAN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_CLS_TYPE_IVLAN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable VNIC classification based on Inner\n     VLAN instead of the default Outer VLAN.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18508UL: // (NIG_REG_P0_LLH_CLS_TYPE_PERFUNC_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_CLS_TYPE_PERFUNC_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the use of separate classification\n     type configuration for each function.  Set to 0 to use registers p*_llh_cls_type_ivlan\n     and llh*_cls_type as the global classification type registers.  Set to\n     1 to use the p*_llh_cls_type_func* registers as the classification type\n     registers - one register for each function.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1850cUL: // (NIG_REG_P0_LLH_CLS_TYPE_FUNC0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_CLS_TYPE_FUNC0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: This register selects the classification type for function\n     0.  Bits 1:0 specify the classication type - similar to llh*_cls_type\n    .  Bit 2 specifies whether the classification is based on Inner VLAN or\n     Outer VLAN - similar to p*_llh_cls_type_ivlan.  Bit 3 specifies whether\n     the classification is based on VNTAG - similar to p*_llh_cls_type_vntag\n     register.  This register is used only when p*_llh_cls_type_perfunc_en\n     is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18510UL: // (NIG_REG_P0_LLH_CLS_TYPE_FUNC1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_CLS_TYPE_FUNC1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: This register selects the classification type for function\n     1.  Bits 1:0 specify the classication type - similar to llh*_cls_type\n    .  Bit 2 specifies whether the classification is based on Inner VLAN or\n     Outer VLAN - similar to p*_llh_cls_type_ivlan.  Bit 3 specifies whether\n     the classification is based on VNTAG - similar to p*_llh_cls_type_vntag\n     register.  This register is used only when p*_llh_cls_type_perfunc_en\n     is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18514UL: // (NIG_REG_P0_LLH_CLS_TYPE_FUNC2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_CLS_TYPE_FUNC2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: This register selects the classification type for function\n     2.  Bits 1:0 specify the classication type - similar to llh*_cls_type\n    .  Bit 2 specifies whether the classification is based on Inner VLAN or\n     Outer VLAN - similar to p*_llh_cls_type_ivlan.  Bit 3 specifies whether\n     the classification is based on VNTAG - similar to p*_llh_cls_type_vntag\n     register.  This register is used only when p*_llh_cls_type_perfunc_en\n     is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18518UL: // (NIG_REG_P0_LLH_CLS_TYPE_FUNC3: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_CLS_TYPE_FUNC3 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: This register selects the classification type for function\n     3.  Bits 1:0 specify the classication type - similar to llh*_cls_type\n    .  Bit 2 specifies whether the classification is based on Inner VLAN or\n     Outer VLAN - similar to p*_llh_cls_type_ivlan.  Bit 3 specifies whether\n     the classification is based on VNTAG - similar to p*_llh_cls_type_vntag\n     register.  This register is used only when p*_llh_cls_type_perfunc_en\n     is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1851cUL: // (NIG_REG_P1_ACPI_MF_GLOBAL_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_ACPI_MF_GLOBAL_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable ACPI pattern matching in multi-function\n     mode even when the per-function outer VLAN matching fails.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18520UL: // (NIG_REG_P1_LLH_CLS_TYPE_IVLAN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_CLS_TYPE_IVLAN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable VNIC classification based on Inner\n     VLAN instead of the default Outer VLAN.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18524UL: // (NIG_REG_P1_LLH_CLS_TYPE_PERFUNC_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_CLS_TYPE_PERFUNC_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the use separate classification type\n     configuration for each function.  Set to 0 to use registers p*_llh_cls_type_ivlan\n     and llh*_cls_type as the global classification type registers.  Set to\n     1 to use the p*_llh_cls_type_func* registers as the classification type\n     registers - one register for each function.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18528UL: // (NIG_REG_P1_LLH_CLS_TYPE_FUNC0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_CLS_TYPE_FUNC0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: This register selects the classification type for function\n     0.  Bits 1:0 specify the classication type - similar to llh*_cls_type\n    .  Bit 2 specifies whether the classification is based on Inner VLAN or\n     Outer VLAN - similar to p*_llh_cls_type_ivlan.  Bit 3 specifies whether\n     the classification is based on VNTAG - similar to p*_llh_cls_type_vntag\n     register.  This register is used only when p*_llh_cls_type_perfunc_en\n     is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1852cUL: // (NIG_REG_P1_LLH_CLS_TYPE_FUNC1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_CLS_TYPE_FUNC1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: This register selects the classification type for function\n     1.  Bits 1:0 specify the classication type - similar to llh*_cls_type\n    .  Bit 2 specifies whether the classification is based on Inner VLAN or\n     Outer VLAN - similar to p*_llh_cls_type_ivlan.  Bit 3 specifies whether\n     the classification is based on VNTAG - similar to p*_llh_cls_type_vntag\n     register.  This register is used only when p*_llh_cls_type_perfunc_en\n     is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18530UL: // (NIG_REG_P1_LLH_CLS_TYPE_FUNC2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_CLS_TYPE_FUNC2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: This register selects the classification type for function\n     2.  Bits 1:0 specify the classication type - similar to llh*_cls_type\n    .  Bit 2 specifies whether the classification is based on Inner VLAN or\n     Outer VLAN - similar to p*_llh_cls_type_ivlan.  Bit 3 specifies whether\n     the classification is based on VNTAG - similar to p*_llh_cls_type_vntag\n     register.  This register is used only when p*_llh_cls_type_perfunc_en\n     is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18534UL: // (NIG_REG_P1_LLH_CLS_TYPE_FUNC3: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_CLS_TYPE_FUNC3 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: This register selects the classification type for function\n     3.  Bits 1:0 specify the classication type - similar to llh*_cls_type\n    .  Bit 2 specifies whether the classification is based on Inner VLAN or\n     Outer VLAN - similar to p*_llh_cls_type_ivlan.  Bit 3 specifies whether\n     the classification is based on VNTAG - similar to p*_llh_cls_type_vntag\n     register.  This register is used only when p*_llh_cls_type_perfunc_en\n     is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18538UL: // (NIG_REG_BRB_LB_CLS_FAIL_SEL: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BRB_LB_CLS_FAIL_SEL (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This bit selects the value for the BRB LB 'classification\n     failed' signal.  Clear this bit to select the hard-wired zero value.  \n    Set this bit to take the value from bit 11 of the sideband_info[15:0] \n    signal.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185a8UL: // (NIG_REG_P0_LED_20G: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LED_20G (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This bit controls the 20G LED output to CNIG.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185acUL: // (NIG_REG_P0_MAC_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_MAC_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for RX MAC interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185b0UL: // (NIG_REG_P0_MAC_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_MAC_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Output enable for TX MAC interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185b4UL: // (NIG_REG_P0_MAC_PAUSE_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_MAC_PAUSE_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Output enable for TX PAUSE signal to the MAC.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185b8UL: // (NIG_REG_P0_MAC_RSV_ERR_MASK: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nNIG_REG_P0_MAC_RSV_ERR_MASK (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: This register directly controls the MAC input mask to enable\n    /disable error conditions from triggering RX error output.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185bcUL: // (NIG_REG_P0_TX_MACFIFO_ALM_FULL_THR: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MACFIFO_ALM_FULL_THR (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full threshold for TX MAC FIFO.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185c0UL: // (NIG_REG_P1_MAC_IN_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_MAC_IN_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input enable for RX MAC interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185c4UL: // (NIG_REG_P1_MAC_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_MAC_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Output enable for TX MAC interface\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185c8UL: // (NIG_REG_P1_MAC_PAUSE_OUT_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_MAC_PAUSE_OUT_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Output enable for TX PAUSE signal to the MAC.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185ccUL: // (NIG_REG_P1_MAC_RSV_ERR_MASK: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nNIG_REG_P1_MAC_RSV_ERR_MASK (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: This register directly controls the MAC input mask to enable\n    /disable error conditions from triggering RX error output.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185d0UL: // (NIG_REG_P1_TX_MACFIFO_ALM_FULL_THR: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MACFIFO_ALM_FULL_THR (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full threshold for TX MAC FIFO.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18600UL: // (NIG_REG_P0_TXUMP_W_HIGIG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TXUMP_W_HIGIG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to 1 when HiGig mode is enabled (p*_higig_hdr_size\n     > 0).  Clear this bit to 0 when HiGig mode is disabled (p*_higig_hdr_size\n     == 0).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18608UL: // (NIG_REG_P1_TXUMP_W_HIGIG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TXUMP_W_HIGIG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to 1 when HiGig mode is enabled (p*_higig_hdr_size\n     > 0).  Clear this bit to 0 when HiGig mode is disabled (p*_higig_hdr_size\n     == 0).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18614UL: // (NIG_REG_LLH1_MF_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_MF_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When this bit is set; the LLH will classify the packet before\n     sending it to the BRB or calculating WoL on it.  This bit controls port\n     1 only.  The legacy llh_multi_function_mode bit controls port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18680UL: // (NIG_REG_P0_TX_ARB_PRIORITY_CLIENT2_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_PRIORITY_CLIENT2_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the client number to be assigned to each priority\n     of the strict priority arbiter.  This register specifies bits 31:0 of\n     the 36-bit value.  Priority 0 is the highest priority.  Bits [3:0] are\n     for priority 0 client; bits [35-32] are for priority 8 client.  The clients\n     are assigned the following IDs:  0-management; 1-debug traffic from this\n     port; 2-debug traffic from other port; 3-COS0 traffic; 4-COS1 traffic\n    ; 5-COS2 traffic; 6-COS3 traffic; 7-COS4 traffic; 8-COS5 traffic.  The\n     reset value[35:0] is set to 0x345678021.  This is a new register (with\n     2_) added in E3 B0 to accommodate the 9 input clients to ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18684UL: // (NIG_REG_P0_TX_ARB_PRIORITY_CLIENT2_MSB: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_PRIORITY_CLIENT2_MSB (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Specify the client number to be assigned to each priority\n     of the strict priority arbiter.  This register specifies bits 35:32 of\n     the 36-bit value.  Priority 0 is the highest priority.  Bits [3:0] are\n     for priority 0 client; bits [35-32] are for priority 8 client.  The clients\n     are assigned the following IDs:  0-management; 1-debug traffic from this\n     port; 2-debug traffic from other port; 3-COS0 traffic; 4-COS1 traffic\n    ; 5-COS2 traffic; 6-COS3 traffic; 7-COS4 traffic; 8-COS5 traffic.  The\n     reset value[35:0] is set to 0x345678021.  This is a new register (with\n     2_) added in E3 B0 to accommodate the 9 input clients to ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18688UL: // (NIG_REG_P0_TX_ARB_CLIENT_CREDIT_MAP2_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CLIENT_CREDIT_MAP2_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify which of the credit registers the client is to be\n     mapped to.  This register specifies bits 31:0 of the 36-bit value.  Bits\n    [3:0] are for client 0; bits [35:32] are for client 8.  For clients that\n     are not subject to WFQ credit blocking - their specifications here are\n     not used.  This is a new register (with 2_) added in E3 B0 to accommodate\n     the 9 input clients to ETS arbiter.  The reset default is set for management\n     and debug to use credit registers 6, 7, and 8, respectively, and COSes\n     0-5 to use credit registers 0-5 respectively (0x543210876).  Note that\n     credit registers can not be shared between clients.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1868cUL: // (NIG_REG_P0_TX_ARB_CLIENT_CREDIT_MAP2_MSB: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CLIENT_CREDIT_MAP2_MSB (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Specify which of the credit registers the client is to be\n     mapped to.  This register specifies bits 35:32 of the 36-bit value.  \n    Bits[3:0] are for client 0; bits [35:32] are for client 8.  For clients\n     that are not subject to WFQ credit blocking - their specifications here\n     are not used.  This is a new register (with 2_) added in E3 B0 to accommodate\n     the 9 input clients to ETS arbiter.  The reset default is set for management\n     and debug to use credit registers 6, 7, and 8, respectively, and COSes\n     0-5 to use credit registers 0-5 respectively (0x543210876).  Note that\n     credit registers can not be shared between clients.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18690UL: // (NIG_REG_P0_TX_ARB_CREDIT_WEIGHT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_WEIGHT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     5 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18694UL: // (NIG_REG_P0_TX_ARB_CREDIT_WEIGHT_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_WEIGHT_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     6 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18698UL: // (NIG_REG_P0_TX_ARB_CREDIT_WEIGHT_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_WEIGHT_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     7 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1869cUL: // (NIG_REG_P0_TX_ARB_CREDIT_WEIGHT_8: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_WEIGHT_8 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     8 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186a0UL: // (NIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 5 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186a4UL: // (NIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 6 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186a8UL: // (NIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 7 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186acUL: // (NIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_8: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CREDIT_UPPER_BOUND_8 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 8 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186b0UL: // (NIG_REG_P0_RX_COS2_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS2_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to COS\n     2.  A priority is mapped to COS 2 when the corresponding mask bit is \n    1.  More than one bit may be set; allowing multiple priorities to be mapped\n     to one COS.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186b4UL: // (NIG_REG_P0_RX_COS3_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS3_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to COS\n     3.  A priority is mapped to COS 3 when the corresponding mask bit is \n    1.  More than one bit may be set; allowing multiple priorities to be mapped\n     to one COS.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186b8UL: // (NIG_REG_P0_RX_COS4_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS4_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to COS\n     4.  A priority is mapped to COS 4 when the corresponding mask bit is \n    1.  More than one bit may be set; allowing multiple priorities to be mapped\n     to one COS.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186bcUL: // (NIG_REG_P0_RX_COS5_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS5_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to COS\n     5.  A priority is mapped to COS 5 when the corresponding mask bit is \n    1.  More than one bit may be set; allowing multiple priorities to be mapped\n     to one COS.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186c0UL: // (NIG_REG_P0_RX_COS2_TIMER_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS2_TIMER_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the RX COS 2 timer to start over when\n     it reaches the configured maximum time.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186c4UL: // (NIG_REG_P0_RX_COS3_TIMER_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS3_TIMER_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the RX COS 3 timer to start over when\n     it reaches the configured maximum time.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186c8UL: // (NIG_REG_P0_RX_COS4_TIMER_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS4_TIMER_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the RX COS 4 timer to start over when\n     it reaches the configured maximum time.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186ccUL: // (NIG_REG_P0_RX_COS5_TIMER_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS5_TIMER_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the RX COS 5 timer to start over when\n     it reaches the configured maximum time.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186d0UL: // (NIG_REG_P0_RX_COS2_TIMER_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS2_TIMER_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This register specifies the period of time (in term of the\n     number of core clocks) RX COS2 is paused before asserting rx_cos2_timer_max_int\n     interrupt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186d4UL: // (NIG_REG_P0_RX_COS3_TIMER_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS3_TIMER_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This register specifies the period of time (in term of the\n     number of core clocks) RX COS3 is paused before asserting rx_cos3_timer_max_int\n     interrupt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186d8UL: // (NIG_REG_P0_RX_COS4_TIMER_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS4_TIMER_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This register specifies the period of time (in term of the\n     number of core clocks) RX COS4 is paused before asserting rx_cos4_timer_max_int\n     interrupt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186dcUL: // (NIG_REG_P0_RX_COS5_TIMER_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_COS5_TIMER_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This register specifies the period of time (in term of the\n     number of core clocks) RX COS5 is paused before asserting rx_cos5_timer_max_int\n     interrupt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186e0UL: // (NIG_REG_P1_TX_ARB_PRIORITY_CLIENT2_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_PRIORITY_CLIENT2_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the client number to be assigned to each priority\n     of the strict priority arbiter.  This register specifies bits 31:0 of\n     the 36-bit value.  Priority 0 is the highest priority.  Bits [3:0] are\n     for priority 0 client; bits [35-32] are for priority 8 client.  The clients\n     are assigned the following IDs:  0-management; 1-debug traffic from this\n     port; 2-debug traffic from other port; 3-COS0 traffic; 4-COS1 traffic\n    ; 5-COS2 traffic; 6-COS3 traffic; 7-COS4 traffic; 8-COS5 traffic.  The\n     reset value[35:0] is set to 0x345678021.  This is a new register (with\n     2_) added in E3 B0 to accommodate the 9 input clients to ETS arbiter. \n    Note that this register is the same as the one for port 0, except that\n     port 1 only has COS 0-2 traffic.  There is no traffic for COS 3-5 of \n    port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186e4UL: // (NIG_REG_P1_TX_ARB_PRIORITY_CLIENT2_MSB: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_PRIORITY_CLIENT2_MSB (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Specify the client number to be assigned to each priority\n     of the strict priority arbiter.  This register specifies bits 35:32 of\n     the 36-bit value.  Priority 0 is the highest priority.  Bits [3:0] are\n     for priority 0 client; bits [35-32] are for priority 8 client.  The clients\n     are assigned the following IDs:  0-management; 1-debug traffic from this\n     port; 2-debug traffic from other port; 3-COS0 traffic; 4-COS1 traffic\n    ; 5-COS2 traffic; 6-COS3 traffic; 7-COS4 traffic; 8-COS5 traffic.  The\n     reset value[35:0] is set to 0x345678021.  This is a new register (with\n     2_) added in E3 B0 to accommodate the 9 input clients to ETS arbiter. \n    Note that this register is the same as the one for port 0, except that\n     port 1 only has COS 0-2 traffic.  There is no traffic for COS 3-5 of \n    port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186e8UL: // (NIG_REG_P1_TX_ARB_CLIENT_CREDIT_MAP2_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CLIENT_CREDIT_MAP2_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify which of the credit registers the client is to be\n     mapped to.  This register specifies bits 31:0 of the 36-bit value.  Bits\n    [3:0] are for client 0; bits [35:32] are for client 8.  For clients that\n     are not subject to WFQ credit blocking - their specifications here are\n     not used.  This is a new register (with 2_) added in E3 B0 to accommodate\n     the 9 input clients to ETS arbiter.  The reset default is set for management\n     and debug to use credit registers 6, 7, and 8, respectively, and COSes\n     0-5 to use credit registers 0-5 respectively (0x543210876).  Note that\n     credit registers can not be shared between clients.  Note also that there\n     are only COS0-2 in port 1- there is a total of 6 clients in port 1.  \n    Only credit registers 0-5 are valid.  This register should be configured\n     appropriately before enabling WFQ.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186ecUL: // (NIG_REG_P1_TX_ARB_CLIENT_CREDIT_MAP2_MSB: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CLIENT_CREDIT_MAP2_MSB (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Specify which of the credit registers the client is to be\n     mapped to.  This register specifies bits 35:32 of the 36-bit value.  \n    Bits[3:0] are for client 0; bits [35:32] are for client 8.  For clients\n     that are not subject to WFQ credit blocking - their specifications here\n     are not used.  This is a new register (with 2_) added in E3 B0 to accommodate\n     the 9 input clients to ETS arbiter.  The reset default is set for management\n     and debug to use credit registers 6, 7, and 8, respectively, and COSes\n     0-5 to use credit registers 0-5 respectively (0x543210876).  Note that\n     credit registers can not be shared between clients.  Note also that there\n     are only COS0-2 in port 1- there is a total of 6 clients in port 1.  \n    Only credit registers 0-5 are valid.  This register should be configured\n     appropriately before enabling WFQ.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186f0UL: // (NIG_REG_P1_TX_ARB_CREDIT_WEIGHT_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_WEIGHT_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the weight (in bytes) to be added to credit register\n     5 when it is time to increment.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186f4UL: // (NIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CREDIT_UPPER_BOUND_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Specify the upper bound that credit register 5 is allowed\n     to reach.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186f8UL: // (NIG_REG_P1_RX_COS2_PRIORITY_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_COS2_PRIORITY_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Bit-map indicating which SAFC/PFC priorities to map to COS\n     2.  A priority is mapped to COS 2 when the corresponding mask bit is \n    1.  More than one bit may be set; allowing multiple priorities to be mapped\n     to one COS.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x186fcUL: // (NIG_REG_P1_RX_COS2_TIMER_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_COS2_TIMER_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable the RX COS 2 timer to start over when\n     it reaches the configured maximum time.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18700UL: // (NIG_REG_P1_RX_COS2_TIMER_MAX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_COS2_TIMER_MAX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This register specifies the period of time (in term of the\n     number of core clocks) RX COS2 is paused before asserting rx_cos2_timer_max_int\n     interrupt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18704UL: // (NIG_REG_EGRESS_PBF2TO5_IN_EN: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_PBF2TO5_IN_EN (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Input enable for TX PBF interfaces 2 to 5.  Bit 0 is for \n    interface 2.  Bit 3 is for interface 5. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18708UL: // (NIG_REG_P0_LLH_CLS_TYPE_VNTAG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_CLS_TYPE_VNTAG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable VNIC classification based on VNTAG\n    .  Note that classifications based on VLAN and MAC addresses are disabled\n     when VNTAG classification is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1870cUL: // (NIG_REG_P1_LLH_CLS_TYPE_VNTAG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_CLS_TYPE_VNTAG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable VNIC classification based on VNTAG\n    .  Note that classifications based on VLAN and MAC addresses are disabled\n     when VNTAG classification is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18710UL: // (NIG_REG_P0_LLH_DST_VIF0: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_DST_VIF0 (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Destination VIF to be compared with the value in VNTAG header\n     (for MCP filter rule C).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18714UL: // (NIG_REG_P0_LLH_DST_VIF1: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_DST_VIF1 (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Destination VIF to be compared with the value in VNTAG header\n     (for MCP filter rule C).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18718UL: // (NIG_REG_P0_LLH_MCP_MASK_C_DST_VIF0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_MCP_MASK_C_DST_VIF0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable packets with VNTAG matching p*_llh_dst_vif0\n     to be forwarded to MCP. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1871cUL: // (NIG_REG_P0_LLH_MCP_MASK_C_DST_VIF1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_MCP_MASK_C_DST_VIF1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable packets with VNTAG matching p*_llh_dst_vif1\n     to be forwarded to MCP. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18720UL: // (NIG_REG_P1_LLH_DST_VIF0: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_DST_VIF0 (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Destination VIF to be compared with the value in VNTAG header\n     (for MCP filter rule C).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18724UL: // (NIG_REG_P1_LLH_DST_VIF1: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_DST_VIF1 (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Destination VIF to be compared with the value in VNTAG header\n     (for MCP filter rule C).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18728UL: // (NIG_REG_P1_LLH_MCP_MASK_C_DST_VIF0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_MCP_MASK_C_DST_VIF0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable packets with VNTAG matching p*_llh_dst_vif0\n     to be forwarded to MCP. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1872cUL: // (NIG_REG_P1_LLH_MCP_MASK_C_DST_VIF1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_MCP_MASK_C_DST_VIF1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to enable packets with VNTAG matching p*_llh_dst_vif1\n     to be forwarded to MCP. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18788UL: // (NIG_REG_P0_PTP_EN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_PTP_EN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Enable for TimeSync feature.  Bits [2:0] are for RX side.  \n    Bits [5:3] are for TX side.  Bit 0 enables TimeSync on RX side.  Bit 1\n     enables V1 frame format in timesync event detection on RX side.  Bit \n    2 enables V2 frame format in timesync event detection on RX side.  Bit\n     3 enables TimeSync on TX side.  Bit 4 enables V1 frame format in timesync\n     event detection on TX side.  Bit 5 enables V2 frame format in timesync\n     event detection on TX side.  Note that for HW to detect PTP packet and\n     extract data from the packet, at least one of the version bits of that\n     traffic direction has to be enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1879cUL: // (NIG_REG_P0_LLH_PTP_ETHERTYPE_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_ETHERTYPE_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: MAC Ethertype 1 for PTP packet detection.  Ethertype 0 is\n     fixed at 0x88F7.  This register defaults to 0x88f7.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187a0UL: // (NIG_REG_P0_LLH_PTP_PARAM_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_PARAM_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Mask register for the various parameters used in determining\n     PTP packet presence.  Set each bit to 1 to mask out the particular parameter\n    .  0-IPv4 DA 0 of 224.0.1.129.   1-IPv4 DA 1 of 224.0.0.107.   2-IPv6 \n    DA 0 of 0xFF0*:0:0:0:0:0:0:181.   3-IPv6 DA 1 of 0xFF02:0:0:0:0:0:0:6B\n    .   4-UDP destination port 0 of 319.   5-UDP destination port 1 of 320\n    .  6-MAC Ethertype 0 of 0x88F7.   7-configurable MAC Ethertype 1.   8-\n    MAC DA 0 of 0x01-1B-19-00-00-00.   9-MAC DA 1 of 0x01-80-C2-00-00-0E.  \n    10-configurable MAC DA 2.  The reset default is set to mask out all parameters\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187a4UL: // (NIG_REG_P0_LLH_PTP_RULE_MASK: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_RULE_MASK (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Mask regiser for the rules used in detecting PTP packets.  \n    Set each bit to 1 to mask out that particular rule.   0-{IPv4 DA 0; UDP\n     DP 0} .   1-{IPv4 DA 0; UDP DP 1} .  2-{IPv4 DA 1; UDP DP 0} .  3-{IPv4\n     DA 1; UDP DP 1} .  4-{IPv6 DA 0; UDP DP 0} .  5-{IPv6 DA 0; UDP DP 1} .  \n    6-{IPv6 DA 1; UDP DP 0} .  7-{IPv6 DA 1; UDP DP 1} .  8-{MAC DA 0; Ethertype\n     0} .  9-{MAC DA 1; Ethertype 0} .  10-{MAC DA 0; Ethertype 1} .  11-{\n    MAC DA 1; Ethertype 1} .  12-{MAC DA 2; Ethertype 0} .  13-{MAC DA 2; \n    Ethertype 1} .  The reset default is to mask out all of the rules.  Note\n     that rules 0-3 are for IPv4 packets only and require that the packet \n    is IPv4 for the rules to match.  Note that rules 4-7 are for IPv6 packets\n     only and require that the packet is IPv6 for the rules to match.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187a8UL: // (NIG_REG_P0_LLH_PTP_TO_MCP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_TO_MCP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set to 1 to enable PTP packets to be forwarded to MCP.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187acUL: // (NIG_REG_P0_LLH_PTP_TO_HOST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_TO_HOST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set to 1 to enable PTP packets to be forwarded to the host\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187b0UL: // (NIG_REG_P1_PTP_EN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_PTP_EN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Enable for TimeSync feature.  Bits [2:0] are for RX side.  \n    Bits [5:3] are for TX side.  Bit 0 enables TimeSync on RX side.  Bit 1\n     enables V1 frame format in timesync event detection on RX side.  Bit \n    2 enables V2 frame format in timesync event detection on RX side.  Bit\n     3 enables TimeSync on TX side.  Bit 4 enables V1 frame format in timesync\n     event detection on TX side.  Bit 5 enables V2 frame format in timesync\n     event detection on TX side.  Note that for HW to detect PTP packet and\n     extract data from the packet, at least one of the version bits of that\n     traffic direction has to be enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187c4UL: // (NIG_REG_P1_LLH_PTP_ETHERTYPE_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_ETHERTYPE_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: MAC Ethertype 1 for PTP packet detection.  Ethertype 0 is\n     fixed at 0x88F7.  This register defaults to 0x88f7.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187c8UL: // (NIG_REG_P1_LLH_PTP_PARAM_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_PARAM_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Mask register for the various parameters used in determining\n     PTP packet presence.  Set each bit to 1 to mask out the particular parameter\n    .  0-IPv4 DA 0 of 224.0.1.129.   1-IPv4 DA 1 of 224.0.0.107.   2-IPv6 \n    DA 0 of 0xFF0*:0:0:0:0:0:0:181.   3-IPv6 DA 1 of 0xFF02:0:0:0:0:0:0:6B\n    .   4-UDP destination port 0 of 319.   5-UDP destination port 1 of 320\n    .  6-MAC Ethertype 0 of 0x88F7.   7-configurable MAC Ethertype 1.   8-\n    MAC DA 0 of 0x01-1B-19-00-00-00.   9-MAC DA 1 of 0x01-80-C2-00-00-0E.  \n    10-configurable MAC DA 2.  The reset default is set to mask out all parameters\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187ccUL: // (NIG_REG_P1_LLH_PTP_RULE_MASK: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_RULE_MASK (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Mask regiser for the rules used in detecting PTP packets.  \n    Set each bit to 1 to mask out that particular rule.   0-{IPv4 DA 0; UDP\n     DP 0} .   1-{IPv4 DA 0; UDP DP 1} .  2-{IPv4 DA 1; UDP DP 0} .  3-{IPv4\n     DA 1; UDP DP 1} .  4-{IPv6 DA 0; UDP DP 0} .  5-{IPv6 DA 0; UDP DP 1} .  \n    6-{IPv6 DA 1; UDP DP 0} .  7-{IPv6 DA 1; UDP DP 1} .  8-{MAC DA 0; Ethertype\n     0} .  9-{MAC DA 1; Ethertype 0} .  10-{MAC DA 0; Ethertype 1} .  11-{\n    MAC DA 1; Ethertype 1} .  12-{MAC DA 2; Ethertype 0} .  13-{MAC DA 2; \n    Ethertype 1} .  The reset default is to mask out all of the rules.  Note\n     that rules 0-3 are for IPv4 packets only and require that the packet \n    is IPv4 for the rules to match.  Note that rules 4-7 are for IPv6 packets\n     only and require that the packet is IPv6 for the rules to match.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187d0UL: // (NIG_REG_P1_LLH_PTP_TO_MCP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_TO_MCP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set to 1 to enable PTP packets to be forwarded to MCP.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187d4UL: // (NIG_REG_P1_LLH_PTP_TO_HOST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_TO_HOST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set to 1 to enable PTP packets to be forwarded to the host\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187f0UL: // (NIG_REG_P0_TLLH_PTP_PARAM_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_PTP_PARAM_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Mask register for the various parameters used in determining\n     PTP packet presence.  Set each bit to 1 to mask out the particular parameter\n    .  0-IPv4 DA 0 of 224.0.1.129.   1-IPv4 DA 1 of 224.0.0.107.   2-IPv6 \n    DA 0 of 0xFF0*:0:0:0:0:0:0:181.   3-IPv6 DA 1 of 0xFF02:0:0:0:0:0:0:6B\n    .   4-UDP destination port 0 of 319.   5-UDP destination port 1 of 320\n    .  6-MAC Ethertype 0 of 0x88F7.   7-configurable MAC Ethertype 1.   8-\n    MAC DA 0 of 0x01-1B-19-00-00-00.   9-MAC DA 1 of 0x01-80-C2-00-00-0E.  \n    10-configurable MAC DA 2.  The reset default is set to mask out all parameters\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187f4UL: // (NIG_REG_P0_TLLH_PTP_RULE_MASK: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_PTP_RULE_MASK (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Mask regiser for the rules used in detecting PTP packets.  \n    Set each bit to 1 to mask out that particular rule.   0-{IPv4 DA 0; UDP\n     DP 0} .   1-{IPv4 DA 0; UDP DP 1} .  2-{IPv4 DA 1; UDP DP 0} .  3-{IPv4\n     DA 1; UDP DP 1} .  4-{IPv6 DA 0; UDP DP 0} .  5-{IPv6 DA 0; UDP DP 1} .  \n    6-{IPv6 DA 1; UDP DP 0} .  7-{IPv6 DA 1; UDP DP 1} .  8-{MAC DA 0; Ethertype\n     0} .  9-{MAC DA 1; Ethertype 0} .  10-{MAC DA 0; Ethertype 1} .  11-{\n    MAC DA 1; Ethertype 1} .  12-{MAC DA 2; Ethertype 0} .  13-{MAC DA 2; \n    Ethertype 1} .  The reset default is to mask out all of the rules.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187f8UL: // (NIG_REG_P1_TLLH_PTP_PARAM_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_PTP_PARAM_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Mask register for the various parameters used in determining\n     PTP packet presence.  Set each bit to 1 to mask out the particular parameter\n    .  0-IPv4 DA 0 of 224.0.1.129.   1-IPv4 DA 1 of 224.0.0.107.   2-IPv6 \n    DA 0 of 0xFF0*:0:0:0:0:0:0:181.   3-IPv6 DA 1 of 0xFF02:0:0:0:0:0:0:6B\n    .   4-UDP destination port 0 of 319.   5-UDP destination port 1 of 320\n    .  6-MAC Ethertype 0 of 0x88F7.   7-configurable MAC Ethertype 1.   8-\n    MAC DA 0 of 0x01-1B-19-00-00-00.   9-MAC DA 1 of 0x01-80-C2-00-00-0E.  \n    10-configurable MAC DA 2.  The reset default is set to mask out all parameters\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187fcUL: // (NIG_REG_P1_TLLH_PTP_RULE_MASK: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_PTP_RULE_MASK (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Mask regiser for the rules used in detecting PTP packets.  \n    Set each bit to 1 to mask out that particular rule.   0-{IPv4 DA 0; UDP\n     DP 0} .   1-{IPv4 DA 0; UDP DP 1} .  2-{IPv4 DA 1; UDP DP 0} .  3-{IPv4\n     DA 1; UDP DP 1} .  4-{IPv6 DA 0; UDP DP 0} .  5-{IPv6 DA 0; UDP DP 1} .  \n    6-{IPv6 DA 1; UDP DP 0} .  7-{IPv6 DA 1; UDP DP 1} .  8-{MAC DA 0; Ethertype\n     0} .  9-{MAC DA 1; Ethertype 0} .  10-{MAC DA 0; Ethertype 1} .  11-{\n    MAC DA 1; Ethertype 1} .  12-{MAC DA 2; Ethertype 0} .  13-{MAC DA 2; \n    Ethertype 1} .  The reset default is to mask out all of the rules.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a18UL: // (NIG_REG_P0_TX_TAG_INSERT_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_TAG_INSERT_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Set this field to select the tag insertion mode for TX management\n     traffic to be sent to the network. A value of 1 selects the 4-byte insertion\n     mode.  A value of 2 selects the 6-byte insertion mode.  A value of 3 \n    selects the 10-byte insertion mode.  Set this register to 0 to disable\n     tag insertion.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a1cUL: // (NIG_REG_P0_TX_4BYTE_TAG: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_4BYTE_TAG (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 4-byte tag for used in TX tag insertion for management traffic\n     from the UMP interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a20UL: // (NIG_REG_P0_TX_6BYTE_TAG_4LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_6BYTE_TAG_4LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 6-byte tag for used in TX tag insertion for management traffic\n     from the UMP interface.  This register holds the lower 4 bytes of the\n     6-byte tag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a24UL: // (NIG_REG_P0_TX_6BYTE_TAG_2MSB: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_6BYTE_TAG_2MSB (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: 6-byte tag for used in TX tag insertion for management traffic\n     from the UMP interface.  This register holds the upper 2 bytes of the\n     6-byte tag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a28UL: // (NIG_REG_P1_TX_TAG_INSERT_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_TAG_INSERT_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Set this field to select the tag insertion mode for TX management\n     traffic to be sent to the network. A value of 1 selects the 4-byte insertion\n     mode.  A value of 2 selects the 6-byte insertion mode.  A value of 3 \n    selects the 10-byte insertion mode.  Set this register to 0 to disable\n     tag insertion.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a2cUL: // (NIG_REG_P1_TX_4BYTE_TAG: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_4BYTE_TAG (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 4-byte tag for used in TX tag insertion for management traffic\n     from the UMP interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a30UL: // (NIG_REG_P1_TX_6BYTE_TAG_4LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_6BYTE_TAG_4LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 6-byte tag for used in TX tag insertion for management traffic\n     from the UMP interface.  This register holds the lower 4 bytes of the\n     6-byte tag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a34UL: // (NIG_REG_P1_TX_6BYTE_TAG_2MSB: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_6BYTE_TAG_2MSB (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: 6-byte tag for used in TX tag insertion for management traffic\n     from the UMP interface.  This register holds the upper 2 bytes of the\n     6-byte tag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a38UL: // (NIG_REG_P0_LLH_PTP_MAC_DA_2_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_MAC_DA_2_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: MAC destination address 2 for PTP packet detection.  This\n     register holds the lower 4 bytes of the address.  MAC destination address\n     0 is fixed at 0x011B_1900_0000.  MAC destination address 1 is fixed at\n     0x0180_C200_000E.  This register defaults to 0x011B_1900_0000.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a3cUL: // (NIG_REG_P0_LLH_PTP_MAC_DA_2_MSB: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_MAC_DA_2_MSB (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: MAC destination address 2 for PTP packet detection.  This\n     register holds the lower 4 bytes of the address.  MAC destination address\n     0 is fixed at 0x011B_1900_0000.  MAC destination address 1 is fixed at\n     0x0180_C200_000E.  This register defaults to 0x011B_1900_0000.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a40UL: // (NIG_REG_P1_LLH_PTP_MAC_DA_2_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_MAC_DA_2_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: MAC destination address 2 for PTP packet detection.  This\n     register holds the lower 4 bytes of the address.  MAC destination address\n     0 is fixed at 0x011B_1900_0000.  MAC destination address 1 is fixed at\n     0x0180_C200_000E.  This register defaults to 0x011B_1900_0000.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a44UL: // (NIG_REG_P1_LLH_PTP_MAC_DA_2_MSB: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_MAC_DA_2_MSB (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: MAC destination address 2 for PTP packet detection.  This\n     register holds the lower 4 bytes of the address.  MAC destination address\n     0 is fixed at 0x011B_1900_0000.  MAC destination address 1 is fixed at\n     0x0180_C200_000E.  This register defaults to 0x011B_1900_0000.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a48UL: // (NIG_REG_P0_PTP_SW_TXTSEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_PTP_SW_TXTSEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for SW-specified packet timestamp mode.  NIG will \n    capture the timestamp value of the packet that SW indicated through PBF\n     interface for host traffic or through the p*_tx_mng_timestamp_pkt bit\n     for TX management packet.  Note that the p*_ptp_en[3] bit has to be set\n     to enable TimeSync on TX side for this mode to work.  NIG will extract\n     and capture the sequence ID if one of the version bits is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a4cUL: // (NIG_REG_P0_TX_MNG_TIMESTAMP_PKT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MNG_TIMESTAMP_PKT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicate to timestamp the packet from MCP to network when\n     *_ptp_sw_txtsen is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a50UL: // (NIG_REG_P1_PTP_SW_TXTSEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_PTP_SW_TXTSEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for SW-specified packet timestamp mode.  NIG will \n    capture the timestamp value of the packet that SW indicated through PBF\n     interface for host traffic or through the p*_tx_mng_timestamp_pkt bit\n     for TX management packet.  Note that the p*_ptp_en[3] bit has to be set\n     to enable TimeSync on TX side for this mode to work.  NIG will extract\n     and capture the sequence ID if one of the version bits is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a54UL: // (NIG_REG_P1_TX_MNG_TIMESTAMP_PKT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MNG_TIMESTAMP_PKT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicate to timestamp the packet from MCP to network when\n     *_ptp_sw_txtsen is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a58UL: // (NIG_REG_P0_RX_FC_STATUS_CLEAR: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_FC_STATUS_CLEAR (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to clear the current flow control (PFC and LLFC\n    ) latched status.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a5cUL: // (NIG_REG_P1_RX_FC_STATUS_CLEAR: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_FC_STATUS_CLEAR (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Set this bit to clear the current flow control (PFC and LLFC\n    ) latched status.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10138UL: // (NIG_REG_LLH0_ACPI_ENABLE: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_ENABLE (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: When this bit is set  ACPI packet recognition will be enabled\n    . This bit must not be enabled until after after all other acpi registers\n     were configured.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1013cUL: // (NIG_REG_LLH0_ACPI_PAT_0_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_0_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10140UL: // (NIG_REG_LLH0_ACPI_PAT_1_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_1_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10144UL: // (NIG_REG_LLH0_ACPI_PAT_2_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_2_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10148UL: // (NIG_REG_LLH0_ACPI_PAT_3_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_3_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1014cUL: // (NIG_REG_LLH0_ACPI_PAT_4_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_4_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10150UL: // (NIG_REG_LLH0_ACPI_PAT_5_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_5_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10154UL: // (NIG_REG_LLH0_ACPI_PAT_6_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_6_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10158UL: // (NIG_REG_LLH0_ACPI_PAT_7_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_7_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1015cUL: // (NIG_REG_LLH0_ACPI_PAT_0_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_0_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10160UL: // (NIG_REG_LLH0_ACPI_PAT_1_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_1_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10164UL: // (NIG_REG_LLH0_ACPI_PAT_2_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_2_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10168UL: // (NIG_REG_LLH0_ACPI_PAT_3_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_3_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1016cUL: // (NIG_REG_LLH0_ACPI_PAT_4_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_4_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10170UL: // (NIG_REG_LLH0_ACPI_PAT_5_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_5_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10174UL: // (NIG_REG_LLH0_ACPI_PAT_6_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_6_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10178UL: // (NIG_REG_LLH0_ACPI_PAT_7_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_PAT_7_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1017cUL: // (NIG_REG_LLH1_ACPI_ENABLE: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_ENABLE (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: When this bit is set  ACPI packet recognition will be enabled\n    . This bit must not be enabled until after after all other acpi registers\n     were configured.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10180UL: // (NIG_REG_LLH1_ACPI_PAT_0_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_0_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10184UL: // (NIG_REG_LLH1_ACPI_PAT_1_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_1_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10188UL: // (NIG_REG_LLH1_ACPI_PAT_2_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_2_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1018cUL: // (NIG_REG_LLH1_ACPI_PAT_3_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_3_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10190UL: // (NIG_REG_LLH1_ACPI_PAT_4_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_4_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10194UL: // (NIG_REG_LLH1_ACPI_PAT_5_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_5_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10198UL: // (NIG_REG_LLH1_ACPI_PAT_6_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_6_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1019cUL: // (NIG_REG_LLH1_ACPI_PAT_7_LEN: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_7_LEN (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: Length in bytes of ACPI pattern  7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101a0UL: // (NIG_REG_LLH1_ACPI_PAT_0_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_0_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101a4UL: // (NIG_REG_LLH1_ACPI_PAT_1_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_1_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101a8UL: // (NIG_REG_LLH1_ACPI_PAT_2_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_2_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101acUL: // (NIG_REG_LLH1_ACPI_PAT_3_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_3_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101b0UL: // (NIG_REG_LLH1_ACPI_PAT_4_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_4_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101b4UL: // (NIG_REG_LLH1_ACPI_PAT_5_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_5_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101b8UL: // (NIG_REG_LLH1_ACPI_PAT_6_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_6_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x101bcUL: // (NIG_REG_LLH1_ACPI_PAT_7_CRC: ACCESS:RW  DataWidth:0x20  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_PAT_7_CRC (ACCESS:RW  DataWidth:0x20  SPLIT:4 )" \
				"\nDescription: CRC32C for pattern 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10418UL: // (NIG_REG_EGRESS_DEBUG_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DEBUG_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in DEBUG_FIFO in NIG_TX_DBG\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1041cUL: // (NIG_REG_EGRESS_DEBUG_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DEBUG_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in DEBUG_FIFO in NIG_TX_DBG\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10420UL: // (NIG_REG_EGRESS_DELAY0_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY0_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in DELAY_PBF_FIFO in NIG_RX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10424UL: // (NIG_REG_EGRESS_DELAY0_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY0_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in DELAY_PBF_FIFO in NIG_RX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10428UL: // (NIG_REG_EGRESS_DELAY1_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY1_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in DELAY_PBF_FIFO in NIG_RX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1042cUL: // (NIG_REG_EGRESS_DELAY1_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY1_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in DELAY_PBF_FIFO in NIG_RX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10458UL: // (NIG_REG_EGRESS_MNG0_BYTE_VALID: ACCESS:W  DataWidth:0x3 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_MNG0_BYTE_VALID (ACCESS:W  DataWidth:0x3 )" \
				"\nDescription: Byte valid for egress management packet from MCP. Write only\n     for the last cycle of the management packet; Value = 0 means all 64 bits\n     are valid; Value = 1 means bits[63:56] are vali;Value = 7 means bits[\n    63:8] are valid.;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1045cUL: // (NIG_REG_EGRESS_MNG0_FIFO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_MNG0_FIFO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: TX_MNG_FIFO in NIG_TX_PORT0; data[31:0] written in FIFO order\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10460UL: // (NIG_REG_EGRESS_MNG0_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_MNG0_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in MNG_FIFO in NIG_TX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10464UL: // (NIG_REG_EGRESS_MNG0_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_MNG0_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in MNG_FIFO in NIG_TX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10468UL: // (NIG_REG_EGRESS_MNG0_FIFO_MSB: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_MNG0_FIFO_MSB (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Debug only.MSB of egress_MNG_FIFO in NIG_egress_PORT0. Empty\n    [1]; parity[0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1046cUL: // (NIG_REG_EGRESS_MNG1_BYTE_VALID: ACCESS:W  DataWidth:0x3 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_MNG1_BYTE_VALID (ACCESS:W  DataWidth:0x3 )" \
				"\nDescription: Byte valid for egress management packet from MCP. Write only\n     for the last cycle of the management packet; Value = 0 means all 64 bits\n     are valid; Value = 1 means bits[63:56] are vali;Value = 7 means bits[\n    63:8] are valid.;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10470UL: // (NIG_REG_EGRESS_MNG1_FIFO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_MNG1_FIFO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: TX_MNG_FIFO in NIG_TX_PORT1; data[31:0] written in FIFO order\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10474UL: // (NIG_REG_EGRESS_MNG1_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_MNG1_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in MNG_FIFO in NIG_TX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10478UL: // (NIG_REG_EGRESS_MNG1_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_MNG1_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in MNG_FIFO in NIG_TX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1047cUL: // (NIG_REG_EGRESS_MNG1_FIFO_MSB: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_MNG1_FIFO_MSB (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Debug only. MSB of egress_MNG_FIFO in NIG_egress_PORT0. Empty\n    [1]; parity[0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10480UL: // (NIG_REG_EMAC0_STATUS_MISC_ATTN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC0_STATUS_MISC_ATTN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0. A value of 1 indicates that 1 of the attentions\n     occurred. (enabled inside the ~emac_reg_emac_attention_ena.emac_attention_ena\n     fields). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10484UL: // (NIG_REG_EMAC0_STATUS_MISC_CFG_CHANGE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC0_STATUS_MISC_CFG_CHANGE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0. This register is not applicable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10488UL: // (NIG_REG_EMAC0_STATUS_MISC_LINK_CHANGE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC0_STATUS_MISC_LINK_CHANGE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0. This bit is set on each change of the \n    link. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1048cUL: // (NIG_REG_EMAC0_STATUS_MISC_LINK_STATUS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC0_STATUS_MISC_LINK_STATUS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0. A value of 1 indicates that the physical\n     interface is linked (when all is configured correctly). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10490UL: // (NIG_REG_EMAC0_STATUS_MISC_MI_COMPLETE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC0_STATUS_MISC_MI_COMPLETE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0 This bit is set each time the Management\n     Interface transaction has completed. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10494UL: // (NIG_REG_EMAC0_STATUS_MISC_MI_INT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC0_STATUS_MISC_MI_INT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0. This bit is set when MDINT from either\n     the EXT_MDINT pin or from the Copper PHY is driven low. This condition\n     must be cleared in the attached PHY device that is driving the MINT pin\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10498UL: // (NIG_REG_EMAC1_STATUS_MISC_ATTN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC1_STATUS_MISC_ATTN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0. A value of 1 indicates that 1 of the attentions\n     occurred. (enabled inside the ~emac_reg_emac_attention_ena.emac_attention_ena\n     fields). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1049cUL: // (NIG_REG_EMAC1_STATUS_MISC_CFG_CHANGE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC1_STATUS_MISC_CFG_CHANGE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0. This register is not applicable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104a0UL: // (NIG_REG_EMAC1_STATUS_MISC_LINK_CHANGE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC1_STATUS_MISC_LINK_CHANGE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0. This bit is set on each change of the \n    link. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104a4UL: // (NIG_REG_EMAC1_STATUS_MISC_LINK_STATUS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC1_STATUS_MISC_LINK_STATUS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0. A value of 1 indicates that the physical\n     interface is linked (when all is configured correctly). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104a8UL: // (NIG_REG_EMAC1_STATUS_MISC_MI_COMPLETE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC1_STATUS_MISC_MI_COMPLETE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0 This bit is set each time the Management\n     Interface transaction has completed. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104acUL: // (NIG_REG_EMAC1_STATUS_MISC_MI_INT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC1_STATUS_MISC_MI_INT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from emac0. This bit is set when MDINT from either\n     the EXT_MDINT pin or from the Copper PHY is driven low. This condition\n     must be cleared in the attached PHY device that is driving the MINT pin\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104e0UL: // (NIG_REG_INGRESS_EOP_LB_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EOP_LB_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EOP descriptor FIFO of LP in NIG_RX_EOP\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104e4UL: // (NIG_REG_INGRESS_EOP_LB_FIFO: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EOP_LB_FIFO (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: Debug only. RX_EOP_DSCR_lb_FIFO in NIG_RX_EOP. Data packet_length\n    [13:0]; mac_error[14]; trunc_error[15]; parity[16]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104e8UL: // (NIG_REG_INGRESS_EOP_LB_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EOP_LB_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EOP descriptor FIFO of LP in NIG_RX_EOP\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104ecUL: // (NIG_REG_INGRESS_EOP_PORT0_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EOP_PORT0_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EOP descriptor FIFO of port 0 in NIG_RX_EOP\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104f0UL: // (NIG_REG_INGRESS_EOP_PORT0_FIFO: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EOP_PORT0_FIFO (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: Debug only. RX_EOP_DSCR_PORT0_FIFO in NIG_RX_EOP. Data packet_length\n    [13:0]; mac_error[14]; trunc_error[15]; parity[16]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104f4UL: // (NIG_REG_INGRESS_EOP_PORT0_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EOP_PORT0_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EOP descriptor FIFO of port 0 in NIG_RX_EOP\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104f8UL: // (NIG_REG_INGRESS_EOP_PORT1_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EOP_PORT1_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EOP descriptor FIFO of port 1 in NIG_RX_EOP\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104fcUL: // (NIG_REG_INGRESS_EOP_PORT1_FIFO: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EOP_PORT1_FIFO (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: Debug only. RX_EOP_DSCR_port1_FIFO in NIG_RX_EOP. Data packet_length\n    [13:0]; mac_error[14]; trunc_error[15]; parity[16]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10500UL: // (NIG_REG_INGRESS_EOP_PORT1_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EOP_PORT1_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EOP descriptor FIFO of port 1 in NIG_RX_EOP\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10504UL: // (NIG_REG_INGRESS_LB_PBF_DELAY_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_LB_PBF_DELAY_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO almost full in PBF_DELAY_lb_FIFO in NIG_RX_lb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10508UL: // (NIG_REG_INGRESS_LB_PBF_DELAY_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_LB_PBF_DELAY_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in PBF_DELAY_lb_FIFO in NIG_RX_lb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1050cUL: // (NIG_REG_INGRESS_LB_PBF_DELAY_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_LB_PBF_DELAY_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in PBF_DELAY_lb_FIFO in NIG_RX_lb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10510UL: // (NIG_REG_INGRESS_MNG0_CONTEXT0: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG0_CONTEXT0 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet in rmp0 block for packet started\n     at addr 0 {error[11]; eop_nyte_valid[10:8]; last cycle addr[7:0]} \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10514UL: // (NIG_REG_INGRESS_MNG0_CONTEXT1: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG0_CONTEXT1 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet in rmp0 block for packet started\n     at addr 192 {error[11]; eop_nyte_valid[10:8]; last cycle addr[7:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10518UL: // (NIG_REG_INGRESS_MNG0_CONTEXT2: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG0_CONTEXT2 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet in rmp0 block for packet started\n     at addr 384 {error[11]; eop_nyte_valid[10:8]; last cycle addr[7:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1051cUL: // (NIG_REG_INGRESS_MNG0_CONTEXT3: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG0_CONTEXT3 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet in rmp0 block for packet started\n     at addr 576 {error[11]; eop_nyte_valid[10:8]; last cycle addr[7:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10520UL: // (NIG_REG_INGRESS_MNG1_CONTEXT0: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG1_CONTEXT0 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet in rmp1 block for packet started\n     at addr 0 {error[11]; eop_nyte_valid[10:8]; last cycle addr[7:0]} \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10524UL: // (NIG_REG_INGRESS_MNG1_CONTEXT1: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG1_CONTEXT1 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet in rmp1 block for packet started\n     at addr 192 {error[11]; eop_nyte_valid[10:8]; last cycle addr[7:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10528UL: // (NIG_REG_INGRESS_MNG1_CONTEXT2: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG1_CONTEXT2 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet in rmp1 block for packet started\n     at addr 384 {error[11]; eop_nyte_valid[10:8]; last cycle addr[7:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1052cUL: // (NIG_REG_INGRESS_MNG1_CONTEXT3: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG1_CONTEXT3 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet in rmp1 block for packet started\n     at addr 576 {error[11]; eop_nyte_valid[10:8]; last cycle addr[7:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10530UL: // (NIG_REG_INGRESS_RMP0_DSCR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_RMP0_DSCR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in dscr_fifo in NIG_RX_RMP block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10534UL: // (NIG_REG_INGRESS_RMP0_DSCR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_RMP0_DSCR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in dscr_fifo in NIG_RX_RMP block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10538UL: // (NIG_REG_INGRESS_RMP1_DSCR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_RMP1_DSCR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in dscr_fifo in NIG_RX_RMP block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1053cUL: // (NIG_REG_INGRESS_RMP1_DSCR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_RMP1_DSCR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in dscr_fifo in NIG_RX_RMP block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10540UL: // (NIG_REG_LED_STATUS_ACTIVE_P0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_STATUS_ACTIVE_P0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status of led active for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10544UL: // (NIG_REG_LED_STATUS_ACTIVE_P1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LED_STATUS_ACTIVE_P1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status of led active for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10548UL: // (NIG_REG_LLH0_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in LLH port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1054cUL: // (NIG_REG_LLH0_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in LLH port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10550UL: // (NIG_REG_LLH0_PAT_IND: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_PAT_IND (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: pattern index for debugging ofLLH0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10554UL: // (NIG_REG_LLH0_XCM_INIT_CREDIT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_XCM_INIT_CREDIT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: init credit counter for port0 in LLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10558UL: // (NIG_REG_LLH1_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in LLH port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1055cUL: // (NIG_REG_LLH1_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in LLH port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10560UL: // (NIG_REG_LLH1_PAT_IND: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_PAT_IND (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: pattern index for debugging ofLLH0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10564UL: // (NIG_REG_LLH1_XCM_INIT_CREDIT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_XCM_INIT_CREDIT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: init credit counter for port1 in LLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10568UL: // (NIG_REG_SERDES0_STATUS_AUTONEG_COMPLETE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_AUTONEG_COMPLETE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes0 that inputs to interrupt logic of CL37\n     AN complete.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1056cUL: // (NIG_REG_SERDES0_STATUS_CL73_AN_COMPLETE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_CL73_AN_COMPLETE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes0 that inputs to interrupt logic of CL73\n     AN complete\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10570UL: // (NIG_REG_SERDES0_STATUS_CL73_MR_PAGE_RX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_CL73_MR_PAGE_RX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes0 that inputs to interrupt logic of received\n     CL73 AN page.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10574UL: // (NIG_REG_SERDES0_STATUS_FIBER_RXACT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_FIBER_RXACT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Detection of Fiber RX activity. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10578UL: // (NIG_REG_SERDES0_STATUS_LINK_STATUS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_LINK_STATUS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes0 that inputs to interrupt logic of link\n     status\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1057cUL: // (NIG_REG_SERDES0_STATUS_MAC_COL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_MAC_COL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication of MAC collision. Valid upon half duplex link.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10580UL: // (NIG_REG_SERDES0_STATUS_MAC_CRS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_MAC_CRS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication of MAC carrier sense event.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10584UL: // (NIG_REG_SERDES0_STATUS_MR_PAGE_RX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_MR_PAGE_RX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes0 that inputs to interrupt logic of received\n     CL37 AN page.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10588UL: // (NIG_REG_SERDES0_STATUS_RX_SIGDET: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_RX_SIGDET (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes0 that inputs to interrupt logic of Rx \n    signal detected on its pins.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1058cUL: // (NIG_REG_SERDES0_STATUS_SPEED_10: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_SPEED_10 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  SERDES0 link speed is 10M\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10590UL: // (NIG_REG_SERDES0_STATUS_SPEED_100: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_SPEED_100 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  SERDES0 link speed is 100M\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10594UL: // (NIG_REG_SERDES0_STATUS_SPEED_1000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_SPEED_1000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  SERDES0 link speed is 1G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10598UL: // (NIG_REG_SERDES0_STATUS_SPEED_1000_KX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_SPEED_1000_KX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  SERDES0 link speed is 1G-KX\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1059cUL: // (NIG_REG_SERDES0_STATUS_SPEED_2500: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_SPEED_2500 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  SERDES0 link speed is 2.5G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105a0UL: // (NIG_REG_SERDES0_STATUS_TXPLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_STATUS_TXPLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: when set  SERDES0 TX PLL is locked.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105a4UL: // (NIG_REG_SERDES1_STATUS_AUTONEG_COMPLETE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_AUTONEG_COMPLETE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes1 that inputs to interrupt logic of CL37\n     AN complete.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105a8UL: // (NIG_REG_SERDES1_STATUS_CL73_AN_COMPLETE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_CL73_AN_COMPLETE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes1 that inputs to interrupt logic of CL73\n     AN complete.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105acUL: // (NIG_REG_SERDES1_STATUS_CL73_MR_PAGE_RX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_CL73_MR_PAGE_RX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes1 that inputs to interrupt logic of received\n     CL73 AN page.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105b0UL: // (NIG_REG_SERDES1_STATUS_FIBER_RXACT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_FIBER_RXACT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Detection of Fiber RX activity. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105b4UL: // (NIG_REG_SERDES1_STATUS_LINK_STATUS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_LINK_STATUS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes1 that inputs to interrupt logic of link\n     status\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105b8UL: // (NIG_REG_SERDES1_STATUS_MAC_COL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_MAC_COL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication of MAC collision. Valid upon half duplex link.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105bcUL: // (NIG_REG_SERDES1_STATUS_MAC_CRS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_MAC_CRS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication of MAC carrier sense event.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105c0UL: // (NIG_REG_SERDES1_STATUS_MR_PAGE_RX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_MR_PAGE_RX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes1 that inputs to interrupt logic of received\n     CL37 AN page.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105c4UL: // (NIG_REG_SERDES1_STATUS_RX_SIGDET: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_RX_SIGDET (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from serdes1 that inputs to interrupt logic of Rx \n    signal detected on its pins.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105c8UL: // (NIG_REG_SERDES1_STATUS_SPEED_10: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_SPEED_10 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  SERDES1 link speed is 10M\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105ccUL: // (NIG_REG_SERDES1_STATUS_SPEED_100: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_SPEED_100 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  SERDES1 link speed is 100M\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105d0UL: // (NIG_REG_SERDES1_STATUS_SPEED_1000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_SPEED_1000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  SERDES1 link speed is 1G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105d4UL: // (NIG_REG_SERDES1_STATUS_SPEED_1000_KX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_SPEED_1000_KX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  SERDES1 link speed is 1G-KX\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105d8UL: // (NIG_REG_SERDES1_STATUS_SPEED_2500: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_SPEED_2500 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  SERDES1 link speed is 2.5G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105dcUL: // (NIG_REG_SERDES1_STATUS_TXPLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_STATUS_TXPLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: when set  SERDES1 TX PLL is locked.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105e0UL: // (NIG_REG_SLOW_LLFC0_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SLOW_LLFC0_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in LLFC0 push block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105e4UL: // (NIG_REG_SLOW_LLFC0_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SLOW_LLFC0_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in LLFC0 push block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105e8UL: // (NIG_REG_SLOW_LLFC1_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SLOW_LLFC1_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in LLFC1 push block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105ecUL: // (NIG_REG_SLOW_LLFC1_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SLOW_LLFC1_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in LLFC1 push block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105f0UL: // (NIG_REG_STAT0_BRB_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_BRB_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : In user packets discarded due to BRB backpressure\n     for port 0 COS0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105f4UL: // (NIG_REG_STAT0_BRB_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_BRB_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : User packets received (transferred to BRB\n    )  for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105f8UL: // (NIG_REG_STAT0_BRB_TRUNCATE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_BRB_TRUNCATE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : In user packets truncated due to BRB backpressure\n     for port 0 COS0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x105fcUL: // (NIG_REG_STAT0_FLOW_CTRL_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_FLOW_CTRL_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Flow control messages to XCM discarded due\n     to backpressure from XCM  for port0 from LLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10600UL: // (NIG_REG_STAT0_FLOW_CTRL_OCTETS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_FLOW_CTRL_OCTETS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Flow control octets received  for port0 from\n     LLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10604UL: // (NIG_REG_STAT0_FLOW_CTRL_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_FLOW_CTRL_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Flow control packets received (MAC control\n     frames with opcode != 1; BCN; SAFC)  for port0 from LLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10608UL: // (NIG_REG_STAT0_MNG_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_MNG_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Management packets discarded due to lack \n    of buffer space  for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1060cUL: // (NIG_REG_STAT0_MNG_OCTET_INP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_MNG_OCTET_INP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Management octets received  for port0 from\n     LLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10610UL: // (NIG_REG_STAT0_MNG_OCTET_OUT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_MNG_OCTET_OUT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : Management octets transmitted for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10614UL: // (NIG_REG_STAT0_MNG_PACKET_INP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_MNG_PACKET_INP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Management packets received  for port0 from\n     LLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10618UL: // (NIG_REG_STAT0_MNG_PACKET_OUT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_MNG_PACKET_OUT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : Management packets transmitted for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1061cUL: // (NIG_REG_STAT0_PBF_OCTETS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_PBF_OCTETS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User octets transmitted from PBF for port0\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10620UL: // (NIG_REG_STAT0_PBF_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_PBF_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User packets transmitted from PBF for port0\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10624UL: // (NIG_REG_STAT0_SAFC_INP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_SAFC_INP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : SAFC frames received for port0 from LLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10628UL: // (NIG_REG_STAT1_BRB_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_BRB_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : In user packets discarded due to BRB backpressure\n     for port 1 COS0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1062cUL: // (NIG_REG_STAT1_BRB_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_BRB_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : User packets received (transferred to BRB\n    )  for port1 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10630UL: // (NIG_REG_STAT1_BRB_TRUNCATE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_BRB_TRUNCATE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : In user packets truncated due to BRB backpressure\n     for port 1 COS0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10634UL: // (NIG_REG_STAT1_FLOW_CTRL_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_FLOW_CTRL_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Flow control messages to XCM discarded due\n     to backpressure from XCM  for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10638UL: // (NIG_REG_STAT1_FLOW_CTRL_OCTETS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_FLOW_CTRL_OCTETS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Flow control octets received  for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1063cUL: // (NIG_REG_STAT1_FLOW_CTRL_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_FLOW_CTRL_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Flow control packets received (MAC control\n     frames with opcode != 1; BCN; SAFC)  for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10640UL: // (NIG_REG_STAT1_MNG_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_MNG_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Management packets discarded due to lack \n    of buffer space  for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10644UL: // (NIG_REG_STAT1_MNG_OCTET_INP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_MNG_OCTET_INP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Management octets received  for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10648UL: // (NIG_REG_STAT1_MNG_OCTET_OUT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_MNG_OCTET_OUT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : Management octets transmitted for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1064cUL: // (NIG_REG_STAT1_MNG_PACKET_INP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_MNG_PACKET_INP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Management packets received  for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10650UL: // (NIG_REG_STAT1_MNG_PACKET_OUT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_MNG_PACKET_OUT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : Management packets transmitted for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10654UL: // (NIG_REG_STAT1_PBF_OCTETS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_PBF_OCTETS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User octets transmitted from PBF for port1\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10658UL: // (NIG_REG_STAT1_PBF_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_PBF_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User packets transmitted from PBF for port1\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1065cUL: // (NIG_REG_STAT1_SAFC_INP: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_SAFC_INP (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : SAFC frames received for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10660UL: // (NIG_REG_STAT2_BRB_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT2_BRB_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : User packets received (transferred to BRB\n    )  for LP\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10664UL: // (NIG_REG_STAT_PBF_LB_FULL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT_PBF_LB_FULL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : Number of cycles that the full signal was\n     asserted on the loopback port towards the PBF (due to BRB backpressure\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10668UL: // (NIG_REG_TIMER0_COUNTER: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_TIMER0_COUNTER (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Value of timer counter in NIG_TX_PORT0. This timer counts\n     when there is no read request from MAC when data to it is ready. Timeout\n     event will generate an interrupt to the host when this timer equal to\n     ~nig_registers_timer0_max.timer0_max\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1066cUL: // (NIG_REG_TIMER1_COUNTER: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_TIMER1_COUNTER (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Value of timer counter in NIG_TX_PORT1. This timer counts\n     when there is no read request from MAC when data to it is ready. Timeout\n     event will generate an interrupt to the host when this timer equal to\n     ~nig_registers_timer1_max.timer1_max\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10670UL: // (NIG_REG_XGXS0_STATUS_AUTONEG_COMPLETE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_AUTONEG_COMPLETE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs0 that inputs to interrupt logic of CL37 \n    AN complete.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10674UL: // (NIG_REG_XGXS0_STATUS_CL73_AN_COMPLETE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_CL73_AN_COMPLETE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs0 that inputs to interrupt logic of CL73 \n    AN complete\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10678UL: // (NIG_REG_XGXS0_STATUS_CL73_MR_PAGE_RX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_CL73_MR_PAGE_RX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs0 that inputs to interrupt logic of received\n     CL73 AN page.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1067cUL: // (NIG_REG_XGXS0_STATUS_FIBER_RXACT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_FIBER_RXACT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Detection of Fiber RX activity. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10680UL: // (NIG_REG_XGXS0_STATUS_LINK10G: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_LINK10G (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs0 that inputs to interrupt logic of link10g\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10684UL: // (NIG_REG_XGXS0_STATUS_LINK_STATUS: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_LINK_STATUS (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: status from xgxs0 that inputs to interrupt logic of link \n    status\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10688UL: // (NIG_REG_XGXS0_STATUS_MAC_COL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_MAC_COL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication of MAC collision. Valid upon half duplex link.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1068cUL: // (NIG_REG_XGXS0_STATUS_MAC_CRS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_MAC_CRS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs0 that inputs to interrupt logic of mac crs\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10690UL: // (NIG_REG_XGXS0_STATUS_MR_PAGE_RX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_MR_PAGE_RX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs0 that inputs to interrupt logic of received\n     CL37 AN page.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10694UL: // (NIG_REG_XGXS0_STATUS_REMOTEMDIOREQ: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_REMOTEMDIOREQ (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs port 0 of remote mdio request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10698UL: // (NIG_REG_XGXS0_STATUS_RX_SIGDET: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_RX_SIGDET (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs0 that inputs to interrupt logic of Rx signal\n     detected on its pins.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1069cUL: // (NIG_REG_XGXS0_STATUS_SPEED_10: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_10 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS0 link speed is 10M\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106a0UL: // (NIG_REG_XGXS0_STATUS_SPEED_100: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_100 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS0 link speed is 100M\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106a4UL: // (NIG_REG_XGXS0_STATUS_SPEED_1000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_1000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS0 link speed is 1G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106a8UL: // (NIG_REG_XGXS0_STATUS_SPEED_10000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_10000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS0 link speed is 10GHiG\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106acUL: // (NIG_REG_XGXS0_STATUS_SPEED_10000_CX4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_10000_CX4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS0 link speed is 10G-CX4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106b0UL: // (NIG_REG_XGXS0_STATUS_SPEED_10000_KR: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_10000_KR (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS0 link speed is 10G-KR\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106b4UL: // (NIG_REG_XGXS0_STATUS_SPEED_10000_KX4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_10000_KX4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS0 link speed is 10G-KX4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106b8UL: // (NIG_REG_XGXS0_STATUS_SPEED_1000_KX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_1000_KX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS0 link speed is 1G-KX\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106bcUL: // (NIG_REG_XGXS0_STATUS_SPEED_12000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_12000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS0 link speed is 12G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106c0UL: // (NIG_REG_XGXS0_STATUS_SPEED_12500: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_12500 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS0 link speed is 12.5GHiG\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106c4UL: // (NIG_REG_XGXS0_STATUS_SPEED_13000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_13000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS0 link speed is 13G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106c8UL: // (NIG_REG_XGXS0_STATUS_SPEED_15000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_15000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS0 link speed is 15G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106ccUL: // (NIG_REG_XGXS0_STATUS_SPEED_16000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_16000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS0 link speed is 16G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106d0UL: // (NIG_REG_XGXS0_STATUS_SPEED_2500: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_SPEED_2500 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS0 link speed is 2.5G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106d4UL: // (NIG_REG_XGXS0_STATUS_TXPLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_STATUS_TXPLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: when set; XGXS0 TX PLL is locked.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106d8UL: // (NIG_REG_XGXS1_STATUS_AUTONEG_COMPLETE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_AUTONEG_COMPLETE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs1 that inputs to interrupt logic of CL37 \n    AN complete.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106dcUL: // (NIG_REG_XGXS1_STATUS_CL73_AN_COMPLETE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_CL73_AN_COMPLETE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs1 that inputs to interrupt logic of CL73 \n    AN complete\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106e0UL: // (NIG_REG_XGXS1_STATUS_CL73_MR_PAGE_RX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_CL73_MR_PAGE_RX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs1 that inputs to interrupt logic of received\n     CL73 AN page.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106e4UL: // (NIG_REG_XGXS1_STATUS_FIBER_RXACT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_FIBER_RXACT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Detection of Fiber RX activity. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106e8UL: // (NIG_REG_XGXS1_STATUS_LINK10G: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_LINK10G (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs1 that inputs to interrupt logic of link10g\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106ecUL: // (NIG_REG_XGXS1_STATUS_LINK_STATUS: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_LINK_STATUS (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: status from xgxs1 that inputs to interrupt logic of link \n    status\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106f0UL: // (NIG_REG_XGXS1_STATUS_MAC_COL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_MAC_COL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication of MAC collision. Valid upon half duplex link.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106f4UL: // (NIG_REG_XGXS1_STATUS_MAC_CRS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_MAC_CRS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs1 that inputs to interrupt logic of mac crs\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106f8UL: // (NIG_REG_XGXS1_STATUS_MR_PAGE_RX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_MR_PAGE_RX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs1 that inputs to interrupt logic of received\n     CL37 AN page.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x106fcUL: // (NIG_REG_XGXS1_STATUS_REMOTEMDIOREQ: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_REMOTEMDIOREQ (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs port 1 of remote mdio request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10700UL: // (NIG_REG_XGXS1_STATUS_RX_SIGDET: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_RX_SIGDET (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: status from xgxs1 that inputs to interrupt logic of Rx signal\n     detected on its pins.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10704UL: // (NIG_REG_XGXS1_STATUS_SPEED_10: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_10 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 10M\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10708UL: // (NIG_REG_XGXS1_STATUS_SPEED_100: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_100 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 100M\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1070cUL: // (NIG_REG_XGXS1_STATUS_SPEED_1000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_1000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 1G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10710UL: // (NIG_REG_XGXS1_STATUS_SPEED_10000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_10000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 10GHiG\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10714UL: // (NIG_REG_XGXS1_STATUS_SPEED_10000_CX4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_10000_CX4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 10G-CX4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10718UL: // (NIG_REG_XGXS1_STATUS_SPEED_10000_KR: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_10000_KR (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 10G-KR\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1071cUL: // (NIG_REG_XGXS1_STATUS_SPEED_10000_KX4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_10000_KX4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 10G-KX4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10720UL: // (NIG_REG_XGXS1_STATUS_SPEED_1000_KX: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_1000_KX (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 1G-KX\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10724UL: // (NIG_REG_XGXS1_STATUS_SPEED_12000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_12000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 12G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10728UL: // (NIG_REG_XGXS1_STATUS_SPEED_12500: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_12500 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 12.5GHiG\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1072cUL: // (NIG_REG_XGXS1_STATUS_SPEED_13000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_13000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 13G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10730UL: // (NIG_REG_XGXS1_STATUS_SPEED_15000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_15000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 15G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10734UL: // (NIG_REG_XGXS1_STATUS_SPEED_16000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_16000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 16G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10738UL: // (NIG_REG_XGXS1_STATUS_SPEED_2500: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_SPEED_2500 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set  XGXS1 link speed is 2.5G\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1073cUL: // (NIG_REG_XGXS1_STATUS_TXPLL_LOCK: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_STATUS_TXPLL_LOCK (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: when set  XGXS0 TX PLL is locked.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10740UL: // (NIG_REG_STAT0_BRB_OCTET: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_BRB_OCTET (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: Rx statistics : User octets received for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10790UL: // (NIG_REG_STAT1_BRB_OCTET: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_BRB_OCTET (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: Rx statistics : User octets received for port1 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107e0UL: // (NIG_REG_STAT2_BRB_OCTET: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_REG_STAT2_BRB_OCTET (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: Rx statistics : User octets received for LP\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10800UL: // (NIG_REG_DEBUG_PACKET_LB: ACCESS:WB_W  DataWidth:0x5a ) 
			sprintf(disp_str, "\nNIG_REG_DEBUG_PACKET_LB (ACCESS:WB_W  DataWidth:0x5a )" \
				"\nDescription: Debug packet to LP from RBC; Data spelling:[63:0] data;[64\n    ] error; [67:65]eop_bvalid; [68]eop; [69]sop; [70]port_id;[71]flush;[72\n    :73]-vnic_num;[89:74]-sideband_info\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10820UL: // (NIG_REG_DEBUG_PACKET_PORT0: ACCESS:WB_W  DataWidth:0x47 ) 
			sprintf(disp_str, "\nNIG_REG_DEBUG_PACKET_PORT0 (ACCESS:WB_W  DataWidth:0x47 )" \
				"\nDescription: Debug packet to port0 from RBC; Data spelling: [63:0] data\n    ; [64]error; [67:65]eop_bvalid - number of valid bytes in the last cycle\n     (0=all bytes are valid); [68]eop - active on the last cycle of the packet\n    ; [69]sop - active on the first cycle of the packet; [70]parity - parity\n     is active for all bits [69:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10840UL: // (NIG_REG_DEBUG_PACKET_PORT1: ACCESS:WB_W  DataWidth:0x47 ) 
			sprintf(disp_str, "\nNIG_REG_DEBUG_PACKET_PORT1 (ACCESS:WB_W  DataWidth:0x47 )" \
				"\nDescription: Debug packet to port1 from RBC; Data spelling: [63:0] data\n    ; [64]error; [67:65]eop_bvalid - number of valid bytes in the last cycle\n     (0=all bytes are valid); [68]eop - active on the last cycle of the packet\n    ; [69]sop - active on the first cycle of the packet; [70]parity - parity\n     is active for all bits [69:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10880UL: // (NIG_REG_LLH0_ACPI_BE_MEM_DATA: ACCESS:WB  DataWidth:0x40  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_ACPI_BE_MEM_DATA (ACCESS:WB  DataWidth:0x40  SPLIT:4 )" \
				"\nDescription: byte enable memory for 8 ACPI patterns. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10900UL: // (NIG_REG_LLH1_ACPI_BE_MEM_DATA: ACCESS:WB  DataWidth:0x40  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_ACPI_BE_MEM_DATA (ACCESS:WB  DataWidth:0x40  SPLIT:4 )" \
				"\nDescription: byte enable memory for 8 ACPI patterns. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12000UL: // (NIG_REG_INGRESS_MNG0_FIFO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG0_FIFO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only. RX_MNG_PORT0_FIFO in NIG_RX_RMP.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14000UL: // (NIG_REG_INGRESS_MNG1_FIFO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_MNG1_FIFO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only.RX_MNG_PORT1_FIFO in NIG_RX_RMP.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160e4UL: // (NIG_REG_LLFC_STATUS_BRB1_0: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_STATUS_BRB1_0 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: LLFC status for BRB1 input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160e8UL: // (NIG_REG_LLFC_STATUS_BRB1_1: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_STATUS_BRB1_1 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: LLFC status for BRB1 input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160ecUL: // (NIG_REG_LLFC_STATUS_TSDM_0: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_STATUS_TSDM_0 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: LLFC status for TSDM input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160f0UL: // (NIG_REG_LLFC_STATUS_TSDM_1: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_STATUS_TSDM_1 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: LLFC status for TSDM input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160f4UL: // (NIG_REG_LLFC_STATUS_USDM_0: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_STATUS_USDM_0 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: LLFC status for USDM input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160f8UL: // (NIG_REG_LLFC_STATUS_USDM_1: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_STATUS_USDM_1 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: LLFC status for USDM input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160fcUL: // (NIG_REG_LLH0_FUNC_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_FUNC_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: VLAN ID enabler. This is the per-function enabler for VNIC\n     classification based on VLAN.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16100UL: // (NIG_REG_LLH0_FUNC_VLAN_ID: ACCESS:RW  DataWidth:0xc  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_FUNC_VLAN_ID (ACCESS:RW  DataWidth:0xc  SPLIT:4 )" \
				"\nDescription: VLAN ID for classification before ACPI search.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16104UL: // (NIG_REG_LLH1_FUNC_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_FUNC_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: VLAN ID enabler. This is the per-function enabler for VNIC\n     classification based on VLAN.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16108UL: // (NIG_REG_LLH1_FUNC_VLAN_ID: ACCESS:RW  DataWidth:0xc  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_FUNC_VLAN_ID (ACCESS:RW  DataWidth:0xc  SPLIT:4 )" \
				"\nDescription: VLAN ID for classification before ACPI search.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16114UL: // (NIG_REG_PAUSE_STATUS_BRB1_0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PAUSE_STATUS_BRB1_0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pause status for BRB1 input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16118UL: // (NIG_REG_PAUSE_STATUS_BRB1_1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PAUSE_STATUS_BRB1_1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pause status for BRB1 input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1611cUL: // (NIG_REG_PAUSE_STATUS_TSDM_0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PAUSE_STATUS_TSDM_0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pause status for TSDM input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16120UL: // (NIG_REG_PAUSE_STATUS_TSDM_1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PAUSE_STATUS_TSDM_1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pause status for TSDM input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16124UL: // (NIG_REG_PAUSE_STATUS_USDM_0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PAUSE_STATUS_USDM_0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pause status for USDM input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16128UL: // (NIG_REG_PAUSE_STATUS_USDM_1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PAUSE_STATUS_USDM_1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pause status for USDM input per port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16140UL: // (NIG_REG_LLH0_FUNC_MEM_ENABLE: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_FUNC_MEM_ENABLE (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Enable for MAC addresses to be matched with the VLAN ID for\n     function identification. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16160UL: // (NIG_REG_LLH1_FUNC_MEM_ENABLE: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_FUNC_MEM_ENABLE (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Enable for MAC addresses to be matched with the VLAN ID for\n     function identification. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16180UL: // (NIG_REG_LLH0_FUNC_MEM: ACCESS:WB  DataWidth:0x30  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH0_FUNC_MEM (ACCESS:WB  DataWidth:0x30  SPLIT:4 )" \
				"\nDescription:  MAC addresses to be matched with the VLAN ID for function\n     identification. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x161c0UL: // (NIG_REG_LLH1_FUNC_MEM: ACCESS:WB  DataWidth:0x30  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_LLH1_FUNC_MEM (ACCESS:WB  DataWidth:0x30  SPLIT:4 )" \
				"\nDescription:  MAC addresses to be matched with the VLAN ID for function\n     identification. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18000UL: // (NIG_REG_LATCH_STATUS_0: ACCESS:RW  DataWidth:0x1b ) 
			sprintf(disp_str, "\nNIG_REG_LATCH_STATUS_0 (ACCESS:RW  DataWidth:0x1b )" \
				"\nDescription: Latch for each interrupt from Unicore.b[0] status_emac0_misc_mi_int\n    ; b[1] status_emac0_misc_mi_complete; b[2]status_emac0_misc_cfg_change\n    ; b[3]status_emac0_misc_link_status; b[4]status_emac0_misc_link_change\n    ; b[5]status_emac0_misc_attn; b[6]status_serdes0_mac_crs; b[7]status_serdes0_autoneg_complete\n    ; b[8]status_serdes0_fiber_rxact; b[9]status_serdes0_link_status; b[10\n    ]status_serdes0_mr_page_rx; b[11]status_serdes0_cl73_an_complete; b[12\n    ]status_serdes0_cl73_mr_page_rx; b[13]status_serdes0_rx_sigdet; b[14]status_xgxs0_remotemdioreq\n    ; b[15]status_xgxs0_link10g; b[16]status_xgxs0_autoneg_complete; b[17]\n    status_xgxs0_fiber_rxact; b[21:18]status_xgxs0_link_status; b[22]status_xgxs0_mr_page_rx\n    ; b[23]status_xgxs0_cl73_an_complete; b[24]status_xgxs0_cl73_mr_page_rx\n    ; b[25]status_xgxs0_rx_sigdet; b[26]status_xgxs0_mac_crs\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18008UL: // (NIG_REG_LATCH_STATUS_1: ACCESS:RW  DataWidth:0x1b ) 
			sprintf(disp_str, "\nNIG_REG_LATCH_STATUS_1 (ACCESS:RW  DataWidth:0x1b )" \
				"\nDescription: Latch for each interrupt from Unicore.b[0] status_emac0_misc_mi_int\n    ; b[1] status_emac0_misc_mi_complete; b[2]status_emac0_misc_cfg_change\n    ; b[3]status_emac0_misc_link_status; b[4]status_emac0_misc_link_change\n    ; b[5]status_emac0_misc_attn; b[6]status_serdes0_mac_crs; b[7]status_serdes0_autoneg_complete\n    ; b[8]status_serdes0_fiber_rxact; b[9]status_serdes0_link_status; b[10\n    ]status_serdes0_mr_page_rx; b[11]status_serdes0_cl73_an_complete; b[12\n    ]status_serdes0_cl73_mr_page_rx; b[13]status_serdes0_rx_sigdet; b[14]status_xgxs0_remotemdioreq\n    ; b[15]status_xgxs0_link10g; b[16]status_xgxs0_autoneg_complete; b[17]\n    status_xgxs0_fiber_rxact; b[21:18]status_xgxs0_link_status; b[22]status_xgxs0_mr_page_rx\n    ; b[23]status_xgxs0_cl73_an_complete; b[24]status_xgxs0_cl73_mr_page_rx\n    ; b[25]status_xgxs0_rx_sigdet; b[26]status_xgxs0_mac_crs\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180c8UL: // (NIG_REG_P0_LLH_ACPI_TAG_RM: ACCESS:RW  DataWidth:0x6  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_ACPI_TAG_RM (ACCESS:RW  DataWidth:0x6  SPLIT:4 )" \
				"\nDescription: L2 tag removal configuration for ACPI.  Bit mapped as follow\n    : bit 0 - outer VLAN; bit 1 - inner VLAN; bit 2 - LLC/SNAP; bit 3 - tag\n     0; bit 4 - tag 1; bit 5 - tag 2.  Set these bits to 1's to enable the\n     removal of the corresponding L2 tags when they are present in the receive\n     packet.  Clear the bit to keep the tag in the receive packet.  This configuration\n     is per function.  This configuration replaces llh0_acpi_tag_strip.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180d0UL: // (NIG_REG_P0_TX_ARB_CURRENT_CREDIT_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CURRENT_CREDIT_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180d4UL: // (NIG_REG_P0_TX_ARB_CURRENT_CREDIT_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CURRENT_CREDIT_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180d8UL: // (NIG_REG_P0_TX_ARB_CURRENT_CREDIT_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CURRENT_CREDIT_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180dcUL: // (NIG_REG_P0_TX_ARB_CURRENT_CREDIT_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CURRENT_CREDIT_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180e0UL: // (NIG_REG_P0_TX_ARB_CURRENT_CREDIT_4: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CURRENT_CREDIT_4 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18154UL: // (NIG_REG_P1_LLH_ACPI_TAG_RM: ACCESS:RW  DataWidth:0x6  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_ACPI_TAG_RM (ACCESS:RW  DataWidth:0x6  SPLIT:4 )" \
				"\nDescription: L2 tag removal configuration for ACPI.  Bit mapped as follow\n    : bit 0 - outer VLAN; bit 1 - inner VLAN; bit 2 - LLC/SNAP; bit 3 - tag\n     0; bit 4 - tag 1; bit 5 - tag 2.  Set these bits to 1's to enable the\n     removal of the corresponding L2 tags when they are present in the receive\n     packet.  Clear the bit to keep the tag in the receive packet.  This configuration\n     is per function.  This configuration replaces llh0_acpi_tag_strip.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18158UL: // (NIG_REG_P1_TX_ARB_CURRENT_CREDIT_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CURRENT_CREDIT_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1815cUL: // (NIG_REG_P1_TX_ARB_CURRENT_CREDIT_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CURRENT_CREDIT_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18160UL: // (NIG_REG_P1_TX_ARB_CURRENT_CREDIT_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CURRENT_CREDIT_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18164UL: // (NIG_REG_P1_TX_ARB_CURRENT_CREDIT_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CURRENT_CREDIT_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18168UL: // (NIG_REG_P1_TX_ARB_CURRENT_CREDIT_4: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CURRENT_CREDIT_4 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18278UL: // (NIG_REG_STAT0_COS1_BRB_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_COS1_BRB_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : In user packets discarded due to BRB backpressure\n     for port 0 COS1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1827cUL: // (NIG_REG_STAT0_COS1_BRB_TRUNCATE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_COS1_BRB_TRUNCATE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : In user packets truncated due to BRB backpressure\n     for port 0 COS1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18280UL: // (NIG_REG_STAT1_COS1_BRB_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_COS1_BRB_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : In user packets discarded due to BRB backpressure\n     for port 1 COS1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18284UL: // (NIG_REG_STAT1_COS1_BRB_TRUNCATE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_COS1_BRB_TRUNCATE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Rx statistics : In user packets truncated due to BRB backpressure\n     for port 1 COS1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18294UL: // (NIG_REG_DBG_OUT_DATA_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_DBG_OUT_DATA_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only:  The 32 lower bits of nig_rbc_dbg_data output\n     from nig_dbgmux to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18298UL: // (NIG_REG_DBG_OUT_DATA_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_DBG_OUT_DATA_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only:  The 32 upper bits of nig_rbc_dbg_data output\n     from nig_dbgmux to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1829cUL: // (NIG_REG_DBG_OUT_FRAME: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_DBG_OUT_FRAME (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: The 4 frame bits that go to the DBG block.  Bit\n     0 is the frame of data byte 0.  Bit 3 is the frame of data byte 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182a0UL: // (NIG_REG_DBG_OUT_VALID: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_DBG_OUT_VALID (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only:  The 4 valid bits that go to the DBG block.  \n    Bit 0 validates data byte 0.  Bit 3 validates data byte 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182a4UL: // (NIG_REG_P0_TX_MNG_HOST_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MNG_HOST_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full status of the MCP TX FIFO used for storing MCP \n    packets forwarded to the host.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182a8UL: // (NIG_REG_P0_TX_MNG_HOST_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MNG_HOST_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty status of the MCP TX FIFO used for storing MCP\n     packets forwarded to the host.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182acUL: // (NIG_REG_P0_STAT_MNG_HOST_PACKET_OUT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_STAT_MNG_HOST_PACKET_OUT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : number of packets loaded into MCP TX FIFO\n     for forwarding to the host.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182b0UL: // (NIG_REG_P0_STAT_MNG_HOST_OCTET_OUT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_STAT_MNG_HOST_OCTET_OUT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : number of bytes loaded into MCP TX FIFO for\n     forwarding to the host.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182b4UL: // (NIG_REG_P1_TX_MNG_HOST_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MNG_HOST_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full status of the MCP TX FIFO used for storing MCP \n    packets forwarded to the host.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182b8UL: // (NIG_REG_P1_TX_MNG_HOST_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MNG_HOST_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty status of the MCP TX FIFO used for storing MCP\n     packets forwarded to the host.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182bcUL: // (NIG_REG_P1_STAT_MNG_HOST_PACKET_OUT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_STAT_MNG_HOST_PACKET_OUT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : number of packets loaded into MCP TX FIFO\n     for forwarding to the host.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182c0UL: // (NIG_REG_P1_STAT_MNG_HOST_OCTET_OUT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_STAT_MNG_HOST_OCTET_OUT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : number of bytes loaded into MCP TX FIFO for\n     forwarding to the host.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182e4UL: // (NIG_REG_P0_STAT_MNG_HOST_PACKET_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_STAT_MNG_HOST_PACKET_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : number of packets from MCP to be forwarded\n     to the host that got dropped due to BRB LB full backpressure.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182e8UL: // (NIG_REG_P0_STAT_MNG_HOST_PACKET_TRUNCATE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_STAT_MNG_HOST_PACKET_TRUNCATE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : number of packets from MCP to be forwarded\n     to the host that got truncated due to BRB LB full backpressure.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182ecUL: // (NIG_REG_P1_STAT_MNG_HOST_PACKET_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_STAT_MNG_HOST_PACKET_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : number of packets from MCP to be forwarded\n     to the host that got dropped due to BRB LB full backpressure.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182f0UL: // (NIG_REG_P1_STAT_MNG_HOST_PACKET_TRUNCATE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_STAT_MNG_HOST_PACKET_TRUNCATE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : number of packets from MCP to be forwarded\n     to the host that got truncated due to BRB LB full backpressure.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182fcUL: // (NIG_REG_P0_STAT_TLLH_MNG_PKT_RCVD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_STAT_TLLH_MNG_PKT_RCVD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : Management packets received  in TLLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18300UL: // (NIG_REG_P0_STAT_TLLH_MNG_OCTETS_RCVD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_STAT_TLLH_MNG_OCTETS_RCVD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : Management octets received  in TLLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18304UL: // (NIG_REG_P0_STAT_HBUF_PACKET_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_STAT_HBUF_PACKET_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : Management packets discarded due to lack \n    of buffer space in HBUF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18308UL: // (NIG_REG_P0_TLLH_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TLLH FIFO is empty.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1830cUL: // (NIG_REG_P0_TLLH_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TLLH FIFO is full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18310UL: // (NIG_REG_P0_TLLH_RSLT_BUF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_RSLT_BUF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TLLH local parsing result buffer is full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18314UL: // (NIG_REG_P0_HBUF_DSCR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HBUF_DSCR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication that HBUF descriptor FIFO is full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18318UL: // (NIG_REG_P0_HBUF_DSCR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_HBUF_DSCR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication that HBUF descriptor FIFO is empty.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1831cUL: // (NIG_REG_P0_HBUF_CONTEXT0: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_P0_HBUF_CONTEXT0 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet buffer 0 of HBUF: {error[11]; \n    eop_byte_valid[10:8]; last cycle addr[7:0]} \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18320UL: // (NIG_REG_P0_HBUF_CONTEXT1: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_P0_HBUF_CONTEXT1 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet buffer 1 of HBUF: {error[11]; \n    eop_byte_valid[10:8]; last cycle addr[7:0]} \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18324UL: // (NIG_REG_P0_HBUF_CONTEXT2: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_P0_HBUF_CONTEXT2 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet buffer 2 of HBUF: {error[11]; \n    eop_byte_valid[10:8]; last cycle addr[7:0]} \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18328UL: // (NIG_REG_P0_HBUF_CONTEXT3: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_P0_HBUF_CONTEXT3 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet buffer 3 of HBUF: {error[11]; \n    eop_byte_valid[10:8]; last cycle addr[7:0]} \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1832cUL: // (NIG_REG_P1_STAT_TLLH_MNG_PKT_RCVD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_STAT_TLLH_MNG_PKT_RCVD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : Management packets received  in TLLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18330UL: // (NIG_REG_P1_STAT_TLLH_MNG_OCTETS_RCVD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_STAT_TLLH_MNG_OCTETS_RCVD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : Management octets received  in TLLH\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18334UL: // (NIG_REG_P1_STAT_HBUF_PACKET_DISCARD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_STAT_HBUF_PACKET_DISCARD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : Management packets discarded due to lack \n    of buffer space in HBUF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18338UL: // (NIG_REG_P1_TLLH_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TLLH FIFO is empty.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1833cUL: // (NIG_REG_P1_TLLH_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TLLH FIFO is full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18340UL: // (NIG_REG_P1_TLLH_RSLT_BUF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_RSLT_BUF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TLLH local parsing result buffer is full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18344UL: // (NIG_REG_P1_HBUF_DSCR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HBUF_DSCR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication that HBUF descriptor FIFO is full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18348UL: // (NIG_REG_P1_HBUF_DSCR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_HBUF_DSCR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indication that HBUF descriptor FIFO is empty.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1834cUL: // (NIG_REG_P1_HBUF_CONTEXT0: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_P1_HBUF_CONTEXT0 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet buffer 0 of HBUF: {error[11]; \n    eop_byte_valid[10:8]; last cycle addr[7:0]} \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18350UL: // (NIG_REG_P1_HBUF_CONTEXT1: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_P1_HBUF_CONTEXT1 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet buffer 1 of HBUF: {error[11]; \n    eop_byte_valid[10:8]; last cycle addr[7:0]} \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18354UL: // (NIG_REG_P1_HBUF_CONTEXT2: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_P1_HBUF_CONTEXT2 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet buffer 2 of HBUF: {error[11]; \n    eop_byte_valid[10:8]; last cycle addr[7:0]} \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18358UL: // (NIG_REG_P1_HBUF_CONTEXT3: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nNIG_REG_P1_HBUF_CONTEXT3 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Context of management packet buffer 3 of HBUF: {error[11]; \n    eop_byte_valid[10:8]; last cycle addr[7:0]} \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18440UL: // (NIG_REG_P0_LLH_FUNC_MEM2_ENABLE: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_MEM2_ENABLE (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Enable for MAC addresses to be used for VNIC classification\n    .  These are the enable bits for addresses 8-15 of the 16 MAC addresses\n     of each function.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18460UL: // (NIG_REG_P1_LLH_FUNC_MEM2_ENABLE: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_MEM2_ENABLE (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Enable for MAC addresses to be used for VNIC classification\n    .  These are the enable bits for addresses 8-15 of the 16 MAC addresses\n     of each function.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18480UL: // (NIG_REG_P0_LLH_FUNC_MEM2: ACCESS:WB  DataWidth:0x30  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_MEM2 (ACCESS:WB  DataWidth:0x30  SPLIT:4 )" \
				"\nDescription:  MAC addresses to be used for VNIC classification.  These\n     are addresses 8-15 of the 16 MAC addresses of each function.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x184c0UL: // (NIG_REG_P1_LLH_FUNC_MEM2: ACCESS:WB  DataWidth:0x30  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_MEM2 (ACCESS:WB  DataWidth:0x30  SPLIT:4 )" \
				"\nDescription:  MAC addresses to be used for VNIC classification.  These\n     are addresses 8-15 of the 16 MAC addresses of each function.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18570UL: // (NIG_REG_P0_RX_MACFIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_MACFIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: RX FIFO for receiving data from MAC is empty.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18574UL: // (NIG_REG_P0_RX_MACFIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_MACFIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: RX FIFO for receiving data from MAC is full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18578UL: // (NIG_REG_P0_TX_MACFIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MACFIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TX FIFO for transmitting data to MAC is empty.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1857cUL: // (NIG_REG_P0_TX_MACFIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MACFIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TX FIFO for transmitting data to MAC is full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18580UL: // (NIG_REG_P0_TX_MACFIFO_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_MACFIFO_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TX FIFO for transmitting data to MAC is almost full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18584UL: // (NIG_REG_P0_LLFC_STATUS_TO_MAC: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLFC_STATUS_TO_MAC (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Current value of LLFC signal sent to MAC.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18588UL: // (NIG_REG_P0_PAUSE_STATUS_TO_MAC: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P0_PAUSE_STATUS_TO_MAC (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Current value of PAUSE signal sent to MAC.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1858cUL: // (NIG_REG_P1_RX_MACFIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_MACFIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: RX FIFO for receiving data from MAC is empty.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18590UL: // (NIG_REG_P1_RX_MACFIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_MACFIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: RX FIFO for receiving data from MAC is full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18594UL: // (NIG_REG_P1_TX_MACFIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MACFIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TX FIFO for transmitting data to MAC is empty.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18598UL: // (NIG_REG_P1_TX_MACFIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MACFIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TX FIFO for transmitting data to MAC is full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1859cUL: // (NIG_REG_P1_TX_MACFIFO_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_MACFIFO_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: TX FIFO for transmitting data to MAC is almost full.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185a0UL: // (NIG_REG_P1_LLFC_STATUS_TO_MAC: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLFC_STATUS_TO_MAC (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Current value of LLFC signal sent to MAC.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185a4UL: // (NIG_REG_P1_PAUSE_STATUS_TO_MAC: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_P1_PAUSE_STATUS_TO_MAC (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Current value of PAUSE signal sent to MAC.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185d4UL: // (NIG_REG_XGXS0_SPEED_10000_LR_SR: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_SPEED_10000_LR_SR (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS link speed is 10G-LR/SR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185d8UL: // (NIG_REG_XGXS0_SPEED_10000_XFI: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_SPEED_10000_XFI (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS link speed is 10G-XFI.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185dcUL: // (NIG_REG_XGXS0_SPEED_20000_CX4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_SPEED_20000_CX4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS link speed is 20G-CX4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185e0UL: // (NIG_REG_XGXS0_SPEED_20000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_SPEED_20000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS link speed is 20G.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185e4UL: // (NIG_REG_XGXS0_FIBER_TXACT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_FIBER_TXACT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indicate TX traffic activity for the link.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185e8UL: // (NIG_REG_XGXS1_SPEED_10000_LR_SR: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_SPEED_10000_LR_SR (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS link speed is 10G-LR/SR.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185ecUL: // (NIG_REG_XGXS1_SPEED_10000_XFI: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_SPEED_10000_XFI (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS link speed is 10G-XFI.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185f0UL: // (NIG_REG_XGXS1_SPEED_20000_CX4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_SPEED_20000_CX4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS link speed is 20G-CX4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185f4UL: // (NIG_REG_XGXS1_SPEED_20000: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_SPEED_20000 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: When set XGXS link speed is 20G.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x185f8UL: // (NIG_REG_XGXS1_FIBER_TXACT: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_FIBER_TXACT (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Indicate TX traffic activity for the link.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1860cUL: // (NIG_REG_XGXS0_LINK10G_DXGXS1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_LINK10G_DXGXS1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: A '1' indicates that the second DXGXS has acquired link.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18610UL: // (NIG_REG_XGXS1_LINK10G_DXGXS1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_LINK10G_DXGXS1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: A '1' indicates that the second DXGXS has acquired link.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18618UL: // (NIG_REG_P1_TX_ARB_CURRENT_CREDIT_5: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CURRENT_CREDIT_5 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1861cUL: // (NIG_REG_P0_TX_ARB_CURRENT_CREDIT_5: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CURRENT_CREDIT_5 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18620UL: // (NIG_REG_P0_TX_ARB_CURRENT_CREDIT_6: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CURRENT_CREDIT_6 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 6.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18624UL: // (NIG_REG_P0_TX_ARB_CURRENT_CREDIT_7: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CURRENT_CREDIT_7 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 7.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18628UL: // (NIG_REG_P0_TX_ARB_CURRENT_CREDIT_8: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CURRENT_CREDIT_8 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current upper 32 bits of the 33-bit value in TX arbiter credit\n     register 8.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1862cUL: // (NIG_REG_EGRESS_DELAY2_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY2_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF FIFO empty flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18630UL: // (NIG_REG_EGRESS_DELAY3_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY3_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF FIFO empty flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18634UL: // (NIG_REG_EGRESS_DELAY4_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY4_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF FIFO empty flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18638UL: // (NIG_REG_EGRESS_DELAY5_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY5_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF FIFO empty flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1863cUL: // (NIG_REG_EGRESS_DELAY2_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY2_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF FIFO full flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18640UL: // (NIG_REG_EGRESS_DELAY3_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY3_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF FIFO full flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18644UL: // (NIG_REG_EGRESS_DELAY4_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY4_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF FIFO full flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18648UL: // (NIG_REG_EGRESS_DELAY5_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_DELAY5_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF FIFO full flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1864cUL: // (NIG_REG_STAT2_PBF_OCTETS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT2_PBF_OCTETS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User octets transmitted from PBF \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18650UL: // (NIG_REG_STAT3_PBF_OCTETS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT3_PBF_OCTETS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User octets transmitted from PBF \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18654UL: // (NIG_REG_STAT4_PBF_OCTETS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT4_PBF_OCTETS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User octets transmitted from PBF \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18658UL: // (NIG_REG_STAT5_PBF_OCTETS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT5_PBF_OCTETS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User octets transmitted from PBF \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1865cUL: // (NIG_REG_STAT2_PBF_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT2_PBF_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User packets transmitted from PBF \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18660UL: // (NIG_REG_STAT3_PBF_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT3_PBF_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User packets transmitted from PBF \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18664UL: // (NIG_REG_STAT4_PBF_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT4_PBF_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User packets transmitted from PBF \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18668UL: // (NIG_REG_STAT5_PBF_PACKET: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_STAT5_PBF_PACKET (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Tx statistics : User packets transmitted from PBF \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1866cUL: // (NIG_REG_P0_LLH_FUNC_DST_VIF0: ACCESS:RW  DataWidth:0xe  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_DST_VIF0 (ACCESS:RW  DataWidth:0xe  SPLIT:4 )" \
				"\nDescription: Destination virtual interface value to be used for VNTAG \n    classification.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18670UL: // (NIG_REG_P0_LLH_FUNC_DST_VIF1: ACCESS:RW  DataWidth:0xe  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_DST_VIF1 (ACCESS:RW  DataWidth:0xe  SPLIT:4 )" \
				"\nDescription: Destination virtual interface value to be used for VNTAG \n    classification.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18674UL: // (NIG_REG_P1_LLH_FUNC_DST_VIF0: ACCESS:RW  DataWidth:0xe  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_DST_VIF0 (ACCESS:RW  DataWidth:0xe  SPLIT:4 )" \
				"\nDescription: Destination virtual interface value to be used for VNTAG \n    classification.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18678UL: // (NIG_REG_P1_LLH_FUNC_DST_VIF1: ACCESS:RW  DataWidth:0xe  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_DST_VIF1 (ACCESS:RW  DataWidth:0xe  SPLIT:4 )" \
				"\nDescription: Destination virtual interface value to be used for VNTAG \n    classification.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18748UL: // (NIG_REG_P0_LLH_PTP_MCP_BUF_TS_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_MCP_BUF_TS_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for MCP.  This location returns the lower 32 bits of timestamp value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1874cUL: // (NIG_REG_P0_LLH_PTP_MCP_BUF_TS_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_MCP_BUF_TS_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for MCP.  This location returns the upper 32 bits of timestamp value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18750UL: // (NIG_REG_P0_LLH_PTP_MCP_BUF_SEQID: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_MCP_BUF_SEQID (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for MCP.  Bits [15:0] return the sequence ID of the packet.  Bit 16 indicates\n     the validity of the data in the buffer.  Writing a 1 to bit 16 will clear\n     the buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18754UL: // (NIG_REG_P0_LLH_PTP_HOST_BUF_TS_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_HOST_BUF_TS_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for the host.  This location returns the lower 32 bits of timestamp value\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18758UL: // (NIG_REG_P0_LLH_PTP_HOST_BUF_TS_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_HOST_BUF_TS_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for the host.  This location returns the upper 32 bits of timestamp value\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1875cUL: // (NIG_REG_P0_LLH_PTP_HOST_BUF_SEQID: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_PTP_HOST_BUF_SEQID (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for the host.  Bits [15:0] return the sequence ID of the packet.  Bit\n     16 indicates the validity of the data in the buffer.  Writing a 1 to \n    bit 16 will clear the buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18760UL: // (NIG_REG_P1_LLH_PTP_MCP_BUF_TS_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_MCP_BUF_TS_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for MCP.  This location returns the lower 32 bits of timestamp value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18764UL: // (NIG_REG_P1_LLH_PTP_MCP_BUF_TS_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_MCP_BUF_TS_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for MCP.  This location returns the upper 32 bits of timestamp value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18768UL: // (NIG_REG_P1_LLH_PTP_MCP_BUF_SEQID: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_MCP_BUF_SEQID (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for MCP.  Bits [15:0] return the sequence ID of the packet.  Bit 16 indicates\n     the validity of the data in the buffer.  Writing a 1 to bit 16 will clear\n     the buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1876cUL: // (NIG_REG_P1_LLH_PTP_HOST_BUF_TS_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_HOST_BUF_TS_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for the host.  This location returns the lower 32 bits of timestamp value\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18770UL: // (NIG_REG_P1_LLH_PTP_HOST_BUF_TS_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_HOST_BUF_TS_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for the host.  This location returns the upper 32 bits of timestamp value\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18774UL: // (NIG_REG_P1_LLH_PTP_HOST_BUF_SEQID: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_PTP_HOST_BUF_SEQID (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for the host.  Bits [15:0] return the sequence ID of the packet.  Bit\n     16 indicates the validity of the data in the buffer.  Writing a 1 to \n    bit 16 will clear the buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187d8UL: // (NIG_REG_P0_TLLH_PTP_BUF_TS_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_PTP_BUF_TS_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for MCP.  This location returns the lower 32 bits of timestamp value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187dcUL: // (NIG_REG_P0_TLLH_PTP_BUF_TS_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_PTP_BUF_TS_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for MCP.  This location returns the upper 32 bits of timestamp value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187e0UL: // (NIG_REG_P0_TLLH_PTP_BUF_SEQID: ACCESS:RW  DataWidth:0x13 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TLLH_PTP_BUF_SEQID (ACCESS:RW  DataWidth:0x13 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for TX side.  Bits [15:0] reflect the sequence ID of the packet.  Bit\n     16 indicates the validity of the data in the buffer.  Bit 17 indicates\n     that the sequence ID is valid and it is waiting for the TX timestamp \n    value.  Bit 18 indicates whether the timestamp is from a SW request (value\n     of 1) or HW request (value of 0).  Writing a 1 to bit 16 will clear the\n     buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187e4UL: // (NIG_REG_P1_TLLH_PTP_BUF_TS_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_PTP_BUF_TS_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for MCP.  This location returns the lower 32 bits of timestamp value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187e8UL: // (NIG_REG_P1_TLLH_PTP_BUF_TS_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_PTP_BUF_TS_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for MCP.  This location returns the upper 32 bits of timestamp value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187ecUL: // (NIG_REG_P1_TLLH_PTP_BUF_SEQID: ACCESS:RW  DataWidth:0x13 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TLLH_PTP_BUF_SEQID (ACCESS:RW  DataWidth:0x13 )" \
				"\nDescription: Packet TimeSync information that is buffered in 1-deep FIFOs\n     for TX side.  Bits [15:0] reflect the sequence ID of the packet.  Bit\n     16 indicates the validity of the data in the buffer.  Bit 17 indicates\n     that the sequence ID is valid and it is waiting for the TX timestamp \n    value.  Bit 18 indicates whether the timestamp is from a SW request (value\n     of 1) or HW request (value of 0).  Writing a 1 to bit 16 will clear the\n     buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		// Ignore line: NIG_REG_TIMESYNC_GEN_REG                               0x18800UL //ACCESS:WB  DataWidth:0x40  Description: Addresses for TimeSync related registers in the timesync generator sub-module.
		case 0x18a00UL: // (NIG_REG_P0_RX_FC_STATUS: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_FC_STATUS (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Current latched flow control (PFC/LLFC)  priorities from \n    the MAC.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a04UL: // (NIG_REG_P1_RX_FC_STATUS: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_FC_STATUS (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Current latched flow control (PFC/LLFC)  priorities from \n    the MAC.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a08UL: // (NIG_REG_P0_LLH_FUNC_DST_VIF0_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_DST_VIF0_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Enable for the destination virtual interface value to be \n    used for VNTAG classification.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a0cUL: // (NIG_REG_P0_LLH_FUNC_DST_VIF1_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_DST_VIF1_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Enable for the destination virtual interface value to be \n    used for VNTAG classification.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a10UL: // (NIG_REG_P1_LLH_FUNC_DST_VIF0_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_DST_VIF0_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Enable for the destination virtual interface value to be \n    used for VNTAG classification.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a14UL: // (NIG_REG_P1_LLH_FUNC_DST_VIF1_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_DST_VIF1_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Enable for the destination virtual interface value to be \n    used for VNTAG classification.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a60UL: // (NIG_REG_P0_RX_PTP_TS_MSB_ERR: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_RX_PTP_TS_MSB_ERR (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Error detected in adjustment of the upper 32-bit time for\n     the 64-bit timestamp value.  Error occurs when bits [31:30] of the MAC\n     timestamp value and the current free-running time are different by 2.  \n    Bits 1:0 reflects MAC timestamp value 31:30.  Bits 3:2 reflects current\n     time value 31:30.  Bit 4 indicates that no adjustment is made to the \n    upper 32-bit time from the current time.  Bit 5 indicates that the upper\n     32-bit time is the decremented value from the current time.  The error\n     status is latched until cleared by writing a '1' to bit 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a64UL: // (NIG_REG_P0_TX_PTP_TS_MSB_ERR: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_PTP_TS_MSB_ERR (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Error detected in adjustment of the upper 32-bit time for\n     the 64-bit timestamp value.  Error occurs when bits [31:30] of the MAC\n     timestamp value and the current free-running time are different by 2.  \n    Bits 1:0 reflects MAC timestamp value 31:30.  Bits 3:2 reflects current\n     time value 31:30.  Bit 4 indicates that no adjustment is made to the \n    upper 32-bit time from the current time.  Bit 5 indicates that the upper\n     32-bit time is the decremented value from the current time.  The error\n     status is latched until cleared by writing a '1' to bit 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a68UL: // (NIG_REG_P1_RX_PTP_TS_MSB_ERR: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_RX_PTP_TS_MSB_ERR (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Error detected in adjustment of the upper 32-bit time for\n     the 64-bit timestamp value.  Error occurs when bits [31:30] of the MAC\n     timestamp value and the current free-running time are different by 2.  \n    Bits 1:0 reflects MAC timestamp value 31:30.  Bits 3:2 reflects current\n     time value 31:30.  Bit 4 indicates that no adjustment is made to the \n    upper 32-bit time from the current time.  Bit 5 indicates that the upper\n     32-bit time is the decremented value from the current time.  The error\n     status is latched until cleared by writing a '1' to bit 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a6cUL: // (NIG_REG_P1_TX_PTP_TS_MSB_ERR: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_PTP_TS_MSB_ERR (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Error detected in adjustment of the upper 32-bit time for\n     the 64-bit timestamp value.  Error occurs when bits [31:30] of the MAC\n     timestamp value and the current free-running time are different by 2.  \n    Bits 1:0 reflects MAC timestamp value 31:30.  Bits 3:2 reflects current\n     time value 31:30.  Bit 4 indicates that no adjustment is made to the \n    upper 32-bit time from the current time.  Bit 5 indicates that the upper\n     32-bit time is the decremented value from the current time.  The error\n     status is latched until cleared by writing a '1' to bit 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a70UL: // (NIG_REG_P0_LLH_FUNC_VLAN_ID1: ACCESS:RW  DataWidth:0xc  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_VLAN_ID1 (ACCESS:RW  DataWidth:0xc  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID to be used in VNIC classification\n    .  There are 4 VLAN IDs for each function.  The legacy llh0/1_func_vlan_id\n     register is used to specify VLAN ID 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a74UL: // (NIG_REG_P0_LLH_FUNC_VLAN_ID2: ACCESS:RW  DataWidth:0xc  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_VLAN_ID2 (ACCESS:RW  DataWidth:0xc  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID to be used in VNIC classification\n    .  There are 4 VLAN IDs for each function.  The legacy llh0/1_func_vlan_id\n     register is used to specify VLAN ID 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a78UL: // (NIG_REG_P0_LLH_FUNC_VLAN_ID3: ACCESS:RW  DataWidth:0xc  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_VLAN_ID3 (ACCESS:RW  DataWidth:0xc  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID to be used in VNIC classification\n    .  There are 4 VLAN IDs for each function.  The legacy llh0/1_func_vlan_id\n     register is used to specify VLAN ID 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a7cUL: // (NIG_REG_P0_LLH_FUNC_VLAN_ID1_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_VLAN_ID1_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID filter enable.  Set this bit to\n     1 to enable the filter.  VLAN ID 0 is enabled when llh0/1_func_en is \n    set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a80UL: // (NIG_REG_P0_LLH_FUNC_VLAN_ID2_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_VLAN_ID2_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID filter enable.  Set this bit to\n     1 to enable the filter.  VLAN ID 0 is enabled when llh0/1_func_en is \n    set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a84UL: // (NIG_REG_P0_LLH_FUNC_VLAN_ID3_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_VLAN_ID3_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID filter enable.  Set this bit to\n     1 to enable the filter.  VLAN ID 0 is enabled when llh0/1_func_en is \n    set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a88UL: // (NIG_REG_P0_LLH_FUNC_VLAN_ID0_NOVLAN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_VLAN_ID0_NOVLAN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function no-VLAN select bit.  Set this bit to 1 to select\n     the no-outer-VLAN-present filter.  Set this bit to 0 to select the perfect\n    -match-of-outer-VLAN-ID filter.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a8cUL: // (NIG_REG_P0_LLH_FUNC_VLAN_ID1_NOVLAN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_VLAN_ID1_NOVLAN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function no-VLAN select bit.  Set this bit to 1 to select\n     the no-outer-VLAN-present filter.  Set this bit to 0 to select the perfect\n    -match-of-outer-VLAN-ID filter.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a90UL: // (NIG_REG_P0_LLH_FUNC_VLAN_ID2_NOVLAN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_VLAN_ID2_NOVLAN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function no-VLAN select bit.  Set this bit to 1 to select\n     the no-outer-VLAN-present filter.  Set this bit to 0 to select the perfect\n    -match-of-outer-VLAN-ID filter.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a94UL: // (NIG_REG_P0_LLH_FUNC_VLAN_ID3_NOVLAN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P0_LLH_FUNC_VLAN_ID3_NOVLAN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function no-VLAN select bit.  Set this bit to 1 to select\n     the no-outer-VLAN-present filter.  Set this bit to 0 to select the perfect\n    -match-of-outer-VLAN-ID filter.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a98UL: // (NIG_REG_P1_LLH_FUNC_VLAN_ID1: ACCESS:RW  DataWidth:0xc  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_VLAN_ID1 (ACCESS:RW  DataWidth:0xc  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID to be used in VNIC classification\n    .  There are 4 VLAN IDs for each function.  The legacy llh0/1_func_vlan_id\n     register is used to specify VLAN ID 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18a9cUL: // (NIG_REG_P1_LLH_FUNC_VLAN_ID2: ACCESS:RW  DataWidth:0xc  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_VLAN_ID2 (ACCESS:RW  DataWidth:0xc  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID to be used in VNIC classification\n    .  There are 4 VLAN IDs for each function.  The legacy llh0/1_func_vlan_id\n     register is used to specify VLAN ID 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18aa0UL: // (NIG_REG_P1_LLH_FUNC_VLAN_ID3: ACCESS:RW  DataWidth:0xc  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_VLAN_ID3 (ACCESS:RW  DataWidth:0xc  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID to be used in VNIC classification\n    .  There are 4 VLAN IDs for each function.  The legacy llh0/1_func_vlan_id\n     register is used to specify VLAN ID 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18aa4UL: // (NIG_REG_P1_LLH_FUNC_VLAN_ID1_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_VLAN_ID1_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID filter enable.  Set this bit to\n     1 to enable the filter.  VLAN ID 0 is enabled when llh0/1_func_en is \n    set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18aa8UL: // (NIG_REG_P1_LLH_FUNC_VLAN_ID2_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_VLAN_ID2_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID filter enable.  Set this bit to\n     1 to enable the filter.  VLAN ID 0 is enabled when llh0/1_func_en is \n    set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18aacUL: // (NIG_REG_P1_LLH_FUNC_VLAN_ID3_EN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_VLAN_ID3_EN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function outer VLAN ID filter enable.  Set this bit to\n     1 to enable the filter.  VLAN ID 0 is enabled when llh0/1_func_en is \n    set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18ab0UL: // (NIG_REG_P1_LLH_FUNC_VLAN_ID0_NOVLAN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_VLAN_ID0_NOVLAN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function no-VLAN select bit.  Set this bit to 1 to select\n     the no-outer-VLAN-present filter.  Set this bit to 0 to select the perfect\n    -match-of-outer-VLAN-ID filter.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18ab4UL: // (NIG_REG_P1_LLH_FUNC_VLAN_ID1_NOVLAN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_VLAN_ID1_NOVLAN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function no-VLAN select bit.  Set this bit to 1 to select\n     the no-outer-VLAN-present filter.  Set this bit to 0 to select the perfect\n    -match-of-outer-VLAN-ID filter.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18ab8UL: // (NIG_REG_P1_LLH_FUNC_VLAN_ID2_NOVLAN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_VLAN_ID2_NOVLAN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function no-VLAN select bit.  Set this bit to 1 to select\n     the no-outer-VLAN-present filter.  Set this bit to 0 to select the perfect\n    -match-of-outer-VLAN-ID filter.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18abcUL: // (NIG_REG_P1_LLH_FUNC_VLAN_ID3_NOVLAN: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nNIG_REG_P1_LLH_FUNC_VLAN_ID3_NOVLAN (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Per-function no-VLAN select bit.  Set this bit to 1 to select\n     the no-outer-VLAN-present filter.  Set this bit to 0 to select the perfect\n    -match-of-outer-VLAN-ID filter.  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10024UL: // (NIG_REG_EGRESS_BMAC_PUSH_ALM_FULL: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_BMAC_PUSH_ALM_FULL (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Almoust full for BMAC FIFO in NIG_TX_PORT0 and NIG_TX_PORT1\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1003cUL: // (NIG_REG_NIG_EMAC0_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_EMAC0_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Output signal from NIG to EMAC0. When set enables the EMAC0\n     block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10040UL: // (NIG_REG_NIG_EMAC1_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_EMAC1_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Output signal from NIG to EMAC1. When set enables the EMAC1\n     block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10044UL: // (NIG_REG_NIG_INGRESS_EMAC0_NO_CRC: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INGRESS_EMAC0_NO_CRC (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Output signal from NIG to TX_EMAC0. When set indicates to\n     the EMAC0 to strip the CRC from the ingress packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10048UL: // (NIG_REG_NIG_INGRESS_EMAC1_NO_CRC: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_INGRESS_EMAC1_NO_CRC (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Output signal from NIG to TX_EMAC1. When set indicates to\n     the EMAC1 to strip the CRC from the ingress packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1004cUL: // (NIG_REG_NIG_EGRESS_EMAC0_NO_CRC: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_EGRESS_EMAC0_NO_CRC (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Output signal from NIG to TX_EMAC0. When set indicates to\n     the EMAC0 not to append CRC to the egress packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10050UL: // (NIG_REG_NIG_EGRESS_EMAC1_NO_CRC: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_EGRESS_EMAC1_NO_CRC (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Output signal from NIG to TX_EMAC1. When set indicates to\n     the EMAC1 not to append CRC to the egress packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100a4UL: // (NIG_REG_EMAC0_IN_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC0_IN_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Input enable for RX_EMAC0 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100a8UL: // (NIG_REG_EMAC1_IN_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC1_IN_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Input enable for RX_EMAC1 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100acUL: // (NIG_REG_BMAC0_IN_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BMAC0_IN_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Input enable for RX_BMAC0 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100b0UL: // (NIG_REG_BMAC1_IN_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BMAC1_IN_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Input enable for RX_BMAC1 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100e0UL: // (NIG_REG_BMAC0_OUT_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BMAC0_OUT_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: output enable for TX_BMAC0 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100e4UL: // (NIG_REG_BMAC1_OUT_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BMAC1_OUT_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: output enable for TX_BMAC1 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100e8UL: // (NIG_REG_BMAC0_REGS_OUT_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BMAC0_REGS_OUT_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: output enable for RX_BMAC0_REGS IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x100ecUL: // (NIG_REG_BMAC1_REGS_OUT_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BMAC1_REGS_OUT_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: output enable for RX_BMAC1_REGS IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10110UL: // (NIG_REG_BMAC0_PAUSE_OUT_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BMAC0_PAUSE_OUT_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: output enable for TX BMAC pause port 0 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10114UL: // (NIG_REG_BMAC1_PAUSE_OUT_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_BMAC1_PAUSE_OUT_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: output enable for TX BMAC pause port 1 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10118UL: // (NIG_REG_EMAC0_PAUSE_OUT_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC0_PAUSE_OUT_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: output enable for TX EMAC pause port 0 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1011cUL: // (NIG_REG_EMAC1_PAUSE_OUT_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EMAC1_PAUSE_OUT_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: output enable for TX EMAC pause port 1 IF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10120UL: // (NIG_REG_EGRESS_EMAC0_OUT_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC0_OUT_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Output enable to EMAC0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10124UL: // (NIG_REG_EGRESS_EMAC1_OUT_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC1_OUT_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Output enable to EMAC1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102e0UL: // (NIG_REG_XGXS_SERDES0_MODE_SEL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS_SERDES0_MODE_SEL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: selection for port0 for NIG_MUX block : 0 = SerDes; 1 = XGXS\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102e4UL: // (NIG_REG_XGXS_SERDES1_MODE_SEL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS_SERDES1_MODE_SEL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: selection for port1 for NIG_MUX block : 0 = SerDes; 1 = XGXS\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102e8UL: // (NIG_REG_XGXS_LANE_SEL_P0: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS_LANE_SEL_P0 (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: selection for  XGXS lane of port 0 in NIG_MUX block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x102ecUL: // (NIG_REG_XGXS_LANE_SEL_P1: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS_LANE_SEL_P1 (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: selection for  XGXS lane of port 1 in NIG_MUX block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10338UL: // (NIG_REG_XGXS0_CTRL_MD_ST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_CTRL_MD_ST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to xgxs; 0 - clause 45; 1 - clause 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1033cUL: // (NIG_REG_XGXS0_CTRL_MD_DEVAD: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_CTRL_MD_DEVAD (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: control to xgxs - CL45 DEVAD\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10340UL: // (NIG_REG_XGXS0_CTRL_PHY_ADDR: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_CTRL_PHY_ADDR (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: control to xgxs - CL22 PHY_ADD and CL45 PRTAD\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10344UL: // (NIG_REG_XGXS0_CTRL_REMOTEMDIOEN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_CTRL_REMOTEMDIOEN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to xgxs - remote PHY in-band MDIO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10348UL: // (NIG_REG_XGXS0_CTRL_EXTREMOTEMDIOST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_CTRL_EXTREMOTEMDIOST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to xgxs - remote PHY in-band MDIO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1034cUL: // (NIG_REG_XGXS0_CTRL_PLL_BYPASS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS0_CTRL_PLL_BYPASS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to xgxs - PLL bypass\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10350UL: // (NIG_REG_XGXS1_CTRL_MD_ST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_CTRL_MD_ST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to xgxs; 0 - clause 45; 1 - clause 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10354UL: // (NIG_REG_XGXS1_CTRL_MD_DEVAD: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_CTRL_MD_DEVAD (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: control to xgxs - CL45 DEVAD\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10358UL: // (NIG_REG_XGXS1_CTRL_PHY_ADDR: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_CTRL_PHY_ADDR (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: control to xgxs - CL22 PHY_ADD and CL45 PRTAD\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1035cUL: // (NIG_REG_XGXS1_CTRL_REMOTEMDIOEN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_CTRL_REMOTEMDIOEN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to xgxs - remote PHY in-band MDIO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10360UL: // (NIG_REG_XGXS1_CTRL_EXTREMOTEMDIOST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_CTRL_EXTREMOTEMDIOST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to xgxs - remote PHY in-band MDIO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10364UL: // (NIG_REG_XGXS1_CTRL_PLL_BYPASS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_XGXS1_CTRL_PLL_BYPASS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to xgxs - PLL bypass\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10368UL: // (NIG_REG_SERDES0_CTRL_PLL_BYPASS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_CTRL_PLL_BYPASS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to serdes - PLL bypass\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1036cUL: // (NIG_REG_SERDES0_CTRL_MD_ST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_CTRL_MD_ST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to serdes; 0 - clause 45; 1 - clause 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10370UL: // (NIG_REG_SERDES0_CTRL_MD_DEVAD: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_CTRL_MD_DEVAD (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: control to serdes - CL45 DEVAD\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10374UL: // (NIG_REG_SERDES0_CTRL_PHY_ADDR: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES0_CTRL_PHY_ADDR (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: control to serdes - CL22 PHY_ADD and CL45 PRTAD\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10378UL: // (NIG_REG_SERDES1_CTRL_PLL_BYPASS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_CTRL_PLL_BYPASS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to serdes - PLL bypass\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1037cUL: // (NIG_REG_SERDES1_CTRL_MD_ST: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_CTRL_MD_ST (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: control to serdes; 0 - clause 45; 1 - clause 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10380UL: // (NIG_REG_SERDES1_CTRL_MD_DEVAD: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_CTRL_MD_DEVAD (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: control to serdes - CL45 DEVAD\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10384UL: // (NIG_REG_SERDES1_CTRL_PHY_ADDR: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nNIG_REG_SERDES1_CTRL_PHY_ADDR (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: control to serdes - CL22 PHY_ADD and CL45 PRTAD\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103d0UL: // (NIG_REG_NIG_PRTY_STS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_STS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Legacy E1 and E1H location for parity error status register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103d4UL: // (NIG_REG_NIG_PRTY_STS_CLR: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_STS_CLR (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Legacy E1 and E1H location for parity error status clear \n    register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103d8UL: // (NIG_REG_NIG_PRTY_STS_WR: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_STS_WR (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Legacy E1 and E1H location for parity error status write \n    register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103dcUL: // (NIG_REG_NIG_PRTY_MASK: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_NIG_PRTY_MASK (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Legacy E1 and E1H location for parity error mask register\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10400UL: // (NIG_REG_EGRESS_BMAC0_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_BMAC0_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO almost full in BMAC_FIFO in NIG_TX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10404UL: // (NIG_REG_EGRESS_BMAC0_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_BMAC0_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in BMAC_FIFO in NIG_TX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10408UL: // (NIG_REG_EGRESS_BMAC0_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_BMAC0_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in BMAC_FIFO in NIG_TX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1040cUL: // (NIG_REG_EGRESS_BMAC1_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_BMAC1_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO almost full in BMAC_FIFO in NIG_TX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10410UL: // (NIG_REG_EGRESS_BMAC1_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_BMAC1_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in BMAC_FIFO in NIG_TX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10414UL: // (NIG_REG_EGRESS_BMAC1_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_BMAC1_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in BMAC_FIFO in NIG_TX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10430UL: // (NIG_REG_EGRESS_EMAC0_POP_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC0_POP_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EMAC_POP_FIFO in NIG_TX_EMAC0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10434UL: // (NIG_REG_EGRESS_EMAC0_POP_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC0_POP_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EMAC_POP_FIFO in NIG_TX_EMAC0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10438UL: // (NIG_REG_EGRESS_EMAC0_PUSH_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC0_PUSH_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO almost full in emac_FIFO in NIG_TX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1043cUL: // (NIG_REG_EGRESS_EMAC0_PUSH_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC0_PUSH_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EMAC_PUSH_FIFO in NIG_TX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10440UL: // (NIG_REG_EGRESS_EMAC0_PUSH_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC0_PUSH_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EMAC_PUSH_FIFO in NIG_TX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10444UL: // (NIG_REG_EGRESS_EMAC1_POP_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC1_POP_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EMAC_POP_FIFO in NIG_TX_EMAC1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10448UL: // (NIG_REG_EGRESS_EMAC1_POP_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC1_POP_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EMAC_POP_FIFO in NIG_TX_EMAC1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1044cUL: // (NIG_REG_EGRESS_EMAC1_PUSH_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC1_PUSH_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO almost full in emac_FIFO in NIG_TX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10450UL: // (NIG_REG_EGRESS_EMAC1_PUSH_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC1_PUSH_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EMAC_PUSH_FIFO in NIG_TX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10454UL: // (NIG_REG_EGRESS_EMAC1_PUSH_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC1_PUSH_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EMAC_PUSH_FIFO in NIG_TX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104b0UL: // (NIG_REG_INGRESS_BMAC0_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_BMAC0_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in BMAC_FIFO in NIG_RX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104b4UL: // (NIG_REG_INGRESS_BMAC0_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_BMAC0_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in BMAC_FIFO in NIG_RX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104b8UL: // (NIG_REG_INGRESS_BMAC1_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_BMAC1_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in BMAC_FIFO in NIG_RX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104bcUL: // (NIG_REG_INGRESS_BMAC1_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_BMAC1_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in BMAC_FIFO in NIG_RX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104c0UL: // (NIG_REG_INGRESS_EMAC0_POP_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EMAC0_POP_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EMAC_POP_FIFO in NIG_RX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104c4UL: // (NIG_REG_INGRESS_EMAC0_POP_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EMAC0_POP_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EMAC_POP_FIFO in NIG_RX_PORT0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104c8UL: // (NIG_REG_INGRESS_EMAC0_PUSH_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EMAC0_PUSH_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EMAC_PUSH_FIFO in NIG_RX_EMAC0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104ccUL: // (NIG_REG_INGRESS_EMAC0_PUSH_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EMAC0_PUSH_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EMAC_PUSH_FIFO in NIG_RX_EMAC0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104d0UL: // (NIG_REG_INGRESS_EMAC1_POP_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EMAC1_POP_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EMAC_POP_FIFO in NIG_RX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104d4UL: // (NIG_REG_INGRESS_EMAC1_POP_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EMAC1_POP_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EMAC_POP_FIFO in NIG_RX_PORT1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104d8UL: // (NIG_REG_INGRESS_EMAC1_PUSH_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EMAC1_PUSH_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO empty in EMAC_PUSH_FIFO in NIG_RX_EMAC1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x104dcUL: // (NIG_REG_INGRESS_EMAC1_PUSH_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_EMAC1_PUSH_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIFO full in EMAC_PUSH_FIFO in NIG_RX_EMAC1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10750UL: // (NIG_REG_STAT0_EGRESS_MAC_PKT0: ACCESS:WB_R  DataWidth:0x24 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_EGRESS_MAC_PKT0 (ACCESS:WB_R  DataWidth:0x24 )" \
				"\nDescription: Tx statistics : Number of packets from emac0 or bmac0 that\n     between 1024 and 1522 bytes for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10760UL: // (NIG_REG_STAT0_EGRESS_MAC_PKT1: ACCESS:WB_R  DataWidth:0x24 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_EGRESS_MAC_PKT1 (ACCESS:WB_R  DataWidth:0x24 )" \
				"\nDescription: Tx statistics : Number of packets from emac0 or bmac0 that\n     between 1523 bytes and above for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10770UL: // (NIG_REG_STAT0_INGRESS_MAC_PKT0: ACCESS:WB_R  DataWidth:0x24 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_INGRESS_MAC_PKT0 (ACCESS:WB_R  DataWidth:0x24 )" \
				"\nDescription: Rx statistics : Number of packets from emac0 or bmac0 that\n     between 1024 and 1522 bytes for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10780UL: // (NIG_REG_STAT0_INGRESS_MAC_PKT1: ACCESS:WB_R  DataWidth:0x24 ) 
			sprintf(disp_str, "\nNIG_REG_STAT0_INGRESS_MAC_PKT1 (ACCESS:WB_R  DataWidth:0x24 )" \
				"\nDescription: Rx statistics : Number of packets from emac0 or bmac0 that\n     between 1523 bytes and above for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107a0UL: // (NIG_REG_STAT1_EGRESS_MAC_PKT0: ACCESS:WB_R  DataWidth:0x24 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_EGRESS_MAC_PKT0 (ACCESS:WB_R  DataWidth:0x24 )" \
				"\nDescription: Tx statistics : Number of packets from emac1 or bmac1 that\n     between 1024 and 1522 bytes for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107b0UL: // (NIG_REG_STAT1_EGRESS_MAC_PKT1: ACCESS:WB_R  DataWidth:0x24 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_EGRESS_MAC_PKT1 (ACCESS:WB_R  DataWidth:0x24 )" \
				"\nDescription: Tx statistics : Number of packets from emac1 or bmac1 that\n     between 1523 bytes and above for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107c0UL: // (NIG_REG_STAT1_INGRESS_MAC_PKT0: ACCESS:WB_R  DataWidth:0x24 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_INGRESS_MAC_PKT0 (ACCESS:WB_R  DataWidth:0x24 )" \
				"\nDescription: Rx statistics : Number of packets from emac1 or bmac1 that\n     between 1024 and 1522 bytes for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107d0UL: // (NIG_REG_STAT1_INGRESS_MAC_PKT1: ACCESS:WB_R  DataWidth:0x24 ) 
			sprintf(disp_str, "\nNIG_REG_STAT1_INGRESS_MAC_PKT1 (ACCESS:WB_R  DataWidth:0x24 )" \
				"\nDescription: Rx statistics : Number of packets from emac1 or bmac1 that\n     between 1523 bytes and above for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10c00UL: // (NIG_REG_INGRESS_BMAC0_MEM: ACCESS:R  DataWidth:0x30 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_BMAC0_MEM (ACCESS:R  DataWidth:0x30 )" \
				"\nDescription: This address space contains BMAC0 registers. The BMAC registers\n     are described in appendix A. In order to access the BMAC0 registers; \n    the base address; NIG_REGISTERS_INGRESS_BMAC0_MEM; Offset: 0x10c00; should\n     be added to each BMAC register offset\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11000UL: // (NIG_REG_INGRESS_BMAC1_MEM: ACCESS:R  DataWidth:0x30 ) 
			sprintf(disp_str, "\nNIG_REG_INGRESS_BMAC1_MEM (ACCESS:R  DataWidth:0x30 )" \
				"\nDescription: This address space contains BMAC1 registers. The BMAC registers\n     are described in appendix A. In order to access the BMAC0 registers; \n    the base address; NIG_REGISTERS_INGRESS_BMAC1_MEM; Offset: 0x11000; should\n     be added to each BMAC register offset\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16000UL: // (NIG_REG_EGRESS_EMAC_PUSH_ALM_FULL: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_EMAC_PUSH_ALM_FULL (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Almoust full for EMAC SYNC FIFO in NIG_TX_PORT0 and NIG_TX_PORT1\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16090UL: // (NIG_REG_EGRESS_OUTER_VLAN_HEADER_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_OUTER_VLAN_HEADER_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Ouetr vlan header for management packets in TX for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16094UL: // (NIG_REG_EGRESS_OUTER_VLAN_HEADER_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_OUTER_VLAN_HEADER_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Ouetr vlan header for management packets in TX for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16098UL: // (NIG_REG_EGRESS_OUTER_VLAN_HEADER_INSERT_0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_OUTER_VLAN_HEADER_INSERT_0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Enable for insert outer vlan header ~egress_outer_vlan_header\n    .egress_outer_vlan_header for port0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1609cUL: // (NIG_REG_EGRESS_OUTER_VLAN_HEADER_INSERT_1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_EGRESS_OUTER_VLAN_HEADER_INSERT_1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Enable for insert outer vlan header ~egress_outer_vlan_header\n    .egress_outer_vlan_header for port1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160dcUL: // (NIG_REG_LLFC_STATUS_BMAC_0: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_STATUS_BMAC_0 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: LLFC status for BMAC output port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160e0UL: // (NIG_REG_LLFC_STATUS_BMAC_1: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nNIG_REG_LLFC_STATUS_BMAC_1 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: LLFC status for BMAC output port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1610cUL: // (NIG_REG_PAUSE_STATUS_BMAC_0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PAUSE_STATUS_BMAC_0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pause status for BMAC output port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16110UL: // (NIG_REG_PAUSE_STATUS_BMAC_1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PAUSE_STATUS_BMAC_1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pause status for BMAC output port\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180e4UL: // (NIG_REG_P0_TX_ARB_PRIORITY_CLIENT: ACCESS:R  DataWidth:0xf ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_PRIORITY_CLIENT (ACCESS:R  DataWidth:0xf )" \
				"\nDescription: Specify the client number to be assigned to each priority\n     of the strict priority arbiter.  Priority 0 is the highest priority.  \n    Bits [2:0] are for priority 0 client; bits [14:12] are for priority 4 \n    client.  The clients are assigned the following IDs:  0-management; 1-\n    debug traffic from this port; 2-debug traffic from other port; 3-COS0 \n    traffic; 4-COS1 traffic.  The reset value[14:0] is set to 0x4688 (15'b100_011_010_001_000\n    ) for management at priority 0; debug traffic at priorities 1 and 2; COS0\n     traffic at priority 3; and COS1 traffic at priority 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180f0UL: // (NIG_REG_P0_TX_ARB_CLIENT_CREDIT_MAP: ACCESS:R  DataWidth:0xf ) 
			sprintf(disp_str, "\nNIG_REG_P0_TX_ARB_CLIENT_CREDIT_MAP (ACCESS:R  DataWidth:0xf )" \
				"\nDescription: Specify which of the credit registers the client is to be\n     mapped to.  Bits[2:0] are for client 0; bits [14:12] are for client 4\n    .  For clients that are not subject to WFQ credit blocking - their specifications\n     here are not used.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18230UL: // (NIG_REG_P1_TX_ARB_PRIORITY_CLIENT: ACCESS:R  DataWidth:0xf ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_PRIORITY_CLIENT (ACCESS:R  DataWidth:0xf )" \
				"\nDescription: Specify the client number to be assigned to each priority\n     of the strict priority arbiter.  Priority 0 is the highest priority.  \n    Bits [2:0] are for priority 0 client; bits [14:12] are for priority 4 \n    client.  The clients are assigned the following IDs:  0-management; 1-\n    debug traffic from this port; 2-debug traffic from other port; 3-COS0 \n    traffic; 4-COS1 traffic.  The reset value[14:0] is set to 0x4688 (15'b100_011_010_001_000\n    ) for management at priority 0; debug traffic at priorities 1 and 2; COS0\n     traffic at priority 3; and COS1 traffic at priority 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1823cUL: // (NIG_REG_P1_TX_ARB_CLIENT_CREDIT_MAP: ACCESS:R  DataWidth:0xf ) 
			sprintf(disp_str, "\nNIG_REG_P1_TX_ARB_CLIENT_CREDIT_MAP (ACCESS:R  DataWidth:0xf )" \
				"\nDescription: Specify which of the credit registers the client is to be\n     mapped to.  Bits[2:0] are for client 0; bits [14:12] are for client 4\n    .  For clients that are not subject to WFQ credit blocking - their specifications\n     here are not used.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18274UL: // (NIG_REG_PORT4MODE_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_REG_PORT4MODE_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Set this bit to indicate that this path is used for 4-port\n     mode.  Clear this bit to select 2-port mode.  This bit is used to decide\n     how th route the two traffic flows from PBF to ports 0 and 1 of this \n    path.  In 2-port mode - both flows goes to port 0.  In 4-port mode - IF0\n     flow goes to port 0; IF1 flow goes to port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103e0UL: // (NIG_REG_NIG_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x107f0UL: // (NIG_REG_NIG_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10860UL: // (NIG_REG_NIG_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10980UL: // (NIG_REG_NIG_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x11400UL: // (NIG_REG_NIG_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16068UL: // (NIG_REG_NIG_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x160b8UL: // (NIG_REG_NIG_UNUSED_EMPTY_6: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_6 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1612cUL: // (NIG_REG_NIG_UNUSED_EMPTY_7: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_7 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16200UL: // (NIG_REG_NIG_UNUSED_EMPTY_8: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_8 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16218UL: // (NIG_REG_NIG_UNUSED_EMPTY_9: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_9 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1809cUL: // (NIG_REG_NIG_UNUSED_EMPTY_10: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_10 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180acUL: // (NIG_REG_NIG_UNUSED_EMPTY_11: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_11 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x183dcUL: // (NIG_REG_NIG_UNUSED_EMPTY_12: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_12 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1853cUL: // (NIG_REG_NIG_UNUSED_EMPTY_13: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_13 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1867cUL: // (NIG_REG_NIG_UNUSED_EMPTY_14: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_14 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18730UL: // (NIG_REG_NIG_UNUSED_EMPTY_15: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_15 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18778UL: // (NIG_REG_NIG_UNUSED_EMPTY_16: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_16 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1878cUL: // (NIG_REG_NIG_UNUSED_EMPTY_17: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_17 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x187b4UL: // (NIG_REG_NIG_UNUSED_EMPTY_18: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_18 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18900UL: // (NIG_REG_NIG_UNUSED_EMPTY_19: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_19 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18ac0UL: // (NIG_REG_NIG_UNUSED_EMPTY_20: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_REG_NIG_UNUSED_EMPTY_20 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0xc0UL): // (NIG_TSGEN_REG_NIG_TSGEN_INT_STS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_NIG_TSGEN_INT_STS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & NIG_TSGEN_NIG_TSGEN_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_NIG + 0x8800 + 0xc4UL): // (NIG_TSGEN_REG_NIG_TSGEN_INT_STS_CLR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_NIG_TSGEN_INT_STS_CLR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & NIG_TSGEN_NIG_TSGEN_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_NIG + 0x8800 + 0xc8UL): // (NIG_TSGEN_REG_NIG_TSGEN_INT_STS_WR: ACCESS:WR  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_NIG_TSGEN_INT_STS_WR (ACCESS:WR  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & NIG_TSGEN_NIG_TSGEN_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_NIG + 0x8800 + 0xccUL): // (NIG_TSGEN_REG_NIG_TSGEN_INT_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_NIG_TSGEN_INT_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & NIG_TSGEN_NIG_TSGEN_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_NIG + 0x8800 + 0UL): // (NIG_TSGEN_REG_TSGEN_CTRL: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_CTRL (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Control register.  Bit 0 resets the Free Running Counter \n    to 0.  This is a self-clearing bit.  Bit 1 pauses the Free Running Counter\n    .  Bit 2 writes the Free Running Counter with the value stored in TSGEN_WRITEVAL\n    .  This is a self-clearing bit.  Bits 63:3 are not used and are read-only\n     bits (read as zeroes).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x8UL): // (NIG_TSGEN_REG_TSGEN_WRITEVAL: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_WRITEVAL (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: This value is written to the Free Running Counter when FreeCount_Write\n     in TSGEN_CTRL is asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x10UL): // (NIG_TSGEN_REG_TSGEN_FREECOUNT: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_FREECOUNT (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This is the current value of the Free Running Counter.  This\n     is a read-only register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x18UL): // (NIG_TSGEN_REG_TSGEN_TSIO_OUTPUT_CONTROL: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_TSIO_OUTPUT_CONTROL (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: TSIO Output Control register.  Bits 3:0 are the active-high\n     output enables for the TSIO Output pins 3:0.  Bits 7:4 are not used and\n     are read-only bits (read as zeroes).  Bits 11:8 reflect the current value\n     of the TSIO Output Signals.  These bits are read-only.  Bits 15:12 are\n     not used and are read-only bits (read as zeroes).  Bits 19:16 are for\n     setting the corresponding TSIO output signals to 1.  Bits 23:20 are not\n     used and are read-only bits (read as zeroes).  Bits 27:24 are for clearing\n     the corresponding TSIO output signals to 0's  when these bits are set\n     to 1's.  Bits 63:28 are not used and are read-only bits (read as zeroes\n    ).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x20UL): // (NIG_TSGEN_REG_TSGEN_SYNCTIME_T0: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_SYNCTIME_T0 (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This is the synchronized time for Timer 0.  This is equal\n     to the Free Running Counter +/- the Offset value.  This is a read-only\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x28UL): // (NIG_TSGEN_REG_TSGEN_OFFSET_T0: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_OFFSET_T0 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: This is the Offset for Timer 0.  It is added or subtracted\n     from the Free Running Counter to create the synchronized time.  It is\n     updated by the Drift Compenstation logic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x30UL): // (NIG_TSGEN_REG_TSGEN_DRIFT_T0: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_DRIFT_T0 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Drift register.  Bits 24:0 specify how many microseconds \n    to wait before making a Drift adjustment to the TSGEN_OFFSET_T0 register\n    .  Bits 29:25 specify how many ns to add or subtract from the TSGEN_OFFSET_T0\n     register when making a Drift adjustment.  Bit 30 controls whether the\n     Adjustment_Value is added (1'b1) or subtracted (1'b0) from the TSGEN_OFFSET_T0\n     register when making a Drift adjustment.  Bit 31 controls whether the\n     offset in TSGEN_OFFSET_T0 is added (1'b1) or subtracted (1'b0) from the\n     Free Running Counter to create TSGEN_SYNCTIME_T0.  Bits 63:32 are not\n     used and are read-only bits (read as zeroes).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x38UL): // (NIG_TSGEN_REG_TSGEN_TSIO_INCTL_T0: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_TSIO_INCTL_T0 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: TSIO Input Control register.  Bits 3:0 are used to select\n     which TSIO Input signals are used to latch the synchronized time for \n    Timer 0 in TSGEN_TSIO_INTIMEVAL_T0.  Bits 7:4 are not used and are read\n    -only bits (read as zeroes).  Bit 8 is set if a positive edge on the TSIO\n     Input Signals should be used to capture the synchronized time.  Bit 9\n     is set if a negative edge on the TSIO Input Signals should be used to\n     capture the synchronized time.  Bits 15:10 are not used and are read-\n    only bits (read as zeroes).  Bits 19:16 are set by HW when a TSIO Input\n     Signal has caused the Timer Value to be captured in TSGEN_TSIO_INTIMEVAL_T0\n    .  Write 1's to these bits to clear the corresponding event.  Bits 23:\n    20 are not used and are read-only bits (read as zeroes).  Bits 27:24 allow\n     the user to read the current value on the TSIO Input Signals.  These \n    bits are read-only.  Bits 63:28 are not used and are read-only bits (read\n     as zeroes).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x40UL): // (NIG_TSGEN_REG_TSGEN_TSIO_INTIMEVAL_T0: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_TSIO_INTIMEVAL_T0 (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This is the synchronized time that was captured by an event\n     on the TSIO Input Signals.These bits are read-only.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x48UL): // (NIG_TSGEN_REG_TSGEN_TSIO_OUTCTL_T0: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_TSIO_OUTCTL_T0 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: TSIO Output Control register.  Bits 3:0 are used to select\n     which TSIO Output Signals will be toggled when the syncronized timer \n    increments above TSGEN_TSIO_OUTTIMEVAL_T0.  Bits 15:4 are not used and\n     are read-only bits (read as zeroes).  Bit 16 is set when the synchronized\n     timer has incremented above TSGEN_TSIO_OUTTIMEVAL_T0.  Writing 1'b1 to\n     this bit will clear the event.  Bits 63:17 are not used and are read-\n    only bits (read as zeroes).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x50UL): // (NIG_TSGEN_REG_TSGEN_TSIO_OUTTIMEVAL_T0: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_TSIO_OUTTIMEVAL_T0 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: When the T0 synchronized timer increments above this value\n    , the selected TSIO Output Signals are toggled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x58UL): // (NIG_TSGEN_REG_TSGEN_SYNCTIME_T1: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_SYNCTIME_T1 (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This is the synchronized time for Timer 0.  This is equal\n     to the Free Running Counter +/- the Offset value.  This is a read-only\n     register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x60UL): // (NIG_TSGEN_REG_TSGEN_OFFSET_T1: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_OFFSET_T1 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: This is the Offset for Timer 0.  It is added or subtracted\n     from the Free Running Counter to create the synchronized time.  It is\n     updated by the Drift Compenstation logic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x68UL): // (NIG_TSGEN_REG_TSGEN_DRIFT_T1: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_DRIFT_T1 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Drift register.  Bits 24:0 specify how many microseconds \n    to wait before making a Drift adjustment to the TSGEN_OFFSET_T0 register\n    .  Bits 29:25 specify how many ns to add or subtract from the TSGEN_OFFSET_T0\n     register when making a Drift adjustment.  Bit 30 controls whether the\n     Adjustment_Value is added (1'b1) or subtracted (1'b0) from the TSGEN_OFFSET_T0\n     register when making a Drift adjustment.  Bit 31 controls whether the\n     offset in TSGEN_OFFSET_T0 is added (1'b1) or subtracted (1'b0) from the\n     Free Running Counter to create TSGEN_SYNCTIME_T0.  Bits 63:32 are not\n     used and are read-only bits (read as zeroes).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x70UL): // (NIG_TSGEN_REG_TSGEN_TSIO_INCTL_T1: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_TSIO_INCTL_T1 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: TSIO Input Control register.  Bits 3:0 are used to select\n     which TSIO Input signals are used to latch the synchronized time for \n    Timer 0 in TSGEN_TSIO_INTIMEVAL_T0.  Bits 7:4 are not used and are read\n    -only bits (read as zeroes).  Bit 8 is set if a positive edge on the TSIO\n     Input Signals should be used to capture the synchronized time.  Bit 9\n     is set if a negative edge on the TSIO Input Signals should be used to\n     capture the synchronized time.  Bits 15:10 are not used and are read-\n    only bits (read as zeroes).  Bits 19:16 are set by HW when a TSIO Input\n     Signal has caused the Timer Value to be captured in TSGEN_TSIO_INTIMEVAL_T0\n    .  Write 1's to these bits to clear the corresponding event.  Bits 23:\n    20 are not used and are read-only bits (read as zeroes).  Bits 27:24 allow\n     the user to read the current value on the TSIO Input Signals.  These \n    bits are read-only.  Bits 63:28 are not used and are read-only bits (read\n     as zeroes).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x78UL): // (NIG_TSGEN_REG_TSGEN_TSIO_INTIMEVAL_T1: ACCESS:WB_R  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_TSIO_INTIMEVAL_T1 (ACCESS:WB_R  DataWidth:0x40 )" \
				"\nDescription: This is the synchronized time that was captured by an event\n     on the TSIO Input Signals.These bits are read-only.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x80UL): // (NIG_TSGEN_REG_TSGEN_TSIO_OUTCTL_T1: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_TSIO_OUTCTL_T1 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: TSIO Output Control register.  Bits 3:0 are used to select\n     which TSIO Output Signals will be toggled when the syncronized timer \n    increments above TSGEN_TSIO_OUTTIMEVAL_T0.  Bits 15:4 are not used and\n     are read-only bits (read as zeroes).  Bit 16 is set when the synchronized\n     timer has incremented above TSGEN_TSIO_OUTTIMEVAL_T0.  Writing 1'b1 to\n     this bit will clear the event.  Bits 63:17 are not used and are read-\n    only bits (read as zeroes).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x88UL): // (NIG_TSGEN_REG_TSGEN_TSIO_OUTTIMEVAL_T1: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_TSGEN_TSIO_OUTTIMEVAL_T1 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: When the T0 synchronized timer increments above this value\n    , the selected TSIO Output Signals are toggled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0x90UL): // (NIG_TSGEN_REG_NIG_TSGEN_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_NIG_TSGEN_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_NIG + 0x8800 + 0xd0UL): // (NIG_TSGEN_REG_NIG_TSGEN_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nNIG_TSGEN_REG_NIG_TSGEN_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140000UL: // (PBF_REG_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Init bit. When set the initial credits are copied to the \n    credit registers (except the port credits). Should be set and then reset\n     after the configuration of the block has ended.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140028UL: // (PBF_REG_CRC_BYTE_REVERSE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_CRC_BYTE_REVERSE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set the CRC result will be byte reversed before inserting\n     to the packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14002cUL: // (PBF_REG_DBG_CMP_SEND_MAX: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DBG_CMP_SEND_MAX (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set then SendMax check for debug will be enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140030UL: // (PBF_REG_MAC_IF0_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_MAC_IF0_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for mac interface 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140034UL: // (PBF_REG_MAC_IF1_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_MAC_IF1_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for mac interface 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140038UL: // (PBF_REG_MAC_IF2_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_MAC_IF2_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for mac interface 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14003cUL: // (PBF_REG_MAC_IF3_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_MAC_IF3_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for mac interface 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140040UL: // (PBF_REG_MAC_LB_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_MAC_LB_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for the loopback interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140044UL: // (PBF_REG_IF_ENABLE_REG: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPBF_REG_IF_ENABLE_REG (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPCI_INTER_WR_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_PCI_INTER_WR_IF_ENABLE) >> 19)
				, "\n\tPCI_DATA_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_PCI_DATA_IF_ENABLE) >> 18)
				, "\n\tPCI_REQ_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_PCI_REQ_IF_ENABLE) >> 17)
				, "\n\tXQM_BYTE_CRD_LB_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_XQM_BYTE_CRD_LB_ENABLE) >> 16)
				, "\n\tXQM_BYTE_CRD_IF5_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_XQM_BYTE_CRD_IF5_ENABLE) >> 15)
				, "\n\tXQM_BYTE_CRD_IF4_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_XQM_BYTE_CRD_IF4_ENABLE) >> 14)
				, "\n\tXQM_BYTE_CRD_IF3_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_XQM_BYTE_CRD_IF3_ENABLE) >> 13)
				, "\n\tXQM_BYTE_CRD_IF2_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_XQM_BYTE_CRD_IF2_ENABLE) >> 12)
				, "\n\tXQM_BYTE_CRD_IF1_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_XQM_BYTE_CRD_IF1_ENABLE) >> 11)
				, "\n\tXQM_BYTE_CRD_IF0_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_XQM_BYTE_CRD_IF0_ENABLE) >> 10)
				, "\n\tXQM_CMD_CREDIT_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_XQM_CMD_CREDIT_IF_ENABLE) >> 9)
				, "\n\tTM_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_TM_IF_ENABLE) >> 8)
				, "\n\tCFC_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_CFC_IF_ENABLE) >> 7)
				, "\n\tTCM_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_TCM_IF_ENABLE) >> 6)
				, "\n\tXCM_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_XCM_IF_ENABLE) >> 5)
				, "\n\tCCM_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_CCM_IF_ENABLE) >> 4)
				, "\n\tDATA2PB_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_DATA2PB_IF_ENABLE) >> 3)
				, "\n\tDATA_FROM_PB_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_DATA_FROM_PB_IF_ENABLE) >> 2)
				, "\n\tPB_REQ_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_PB_REQ_IF_ENABLE) >> 1)
				, "\n\tXSDM_IF_ENABLE :"
				, (u32_t)((data & PBF_IF_ENABLE_REG_REG_XSDM_IF_ENABLE) >> 0));
			break;
		case 0x140070UL: // (PBF_REG_SOFT_RESET: ACCESS:RW  DataWidth:0x5  Multi Field Register  ) 
			sprintf(disp_str, "\nPBF_REG_SOFT_RESET (ACCESS:RW  DataWidth:0x5  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCRACKER_INFO_FIFO_INIT :"
				, (u32_t)((data & PBF_SOFT_RESET_REG_CRACKER_INFO_FIFO_INIT) >> 4)
				, "\n\tHEADER_BUFFER_INIT :"
				, (u32_t)((data & PBF_SOFT_RESET_REG_HEADER_BUFFER_INIT) >> 3)
				, "\n\tSDM_IN_Q_INIT :"
				, (u32_t)((data & PBF_SOFT_RESET_REG_SDM_IN_Q_INIT) >> 2)
				, "\n\tHA_CMD_Q_INIT :"
				, (u32_t)((data & PBF_SOFT_RESET_REG_HA_CMD_Q_INIT) >> 1)
				, "\n\tPB_OUT_FIFO_INIT :"
				, (u32_t)((data & PBF_SOFT_RESET_REG_PB_OUT_FIFO_INIT) >> 0));
			break;
		case 0x14009cUL: // (PBF_REG_SDM_IN_Q_THRSH: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_SDM_IN_Q_THRSH (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full threshold for the SDM input command Q. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400b4UL: // (PBF_REG_PCI_VOQ_ID: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_PCI_VOQ_ID (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: PCI VOQ ID used in read request to PCI.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400b8UL: // (PBF_REG_PCI_REQ_INIT_CRD: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_PCI_REQ_INIT_CRD (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: PCI read request interafce initial credit - transoriented\n    . Default is 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400bcUL: // (PBF_REG_XQM_INIT_CRD: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_XQM_INIT_CRD (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: XQM interafce initial credit. Default is 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400c0UL: // (PBF_REG_CCM_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_CCM_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CCM initial credit.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400c4UL: // (PBF_REG_TCM_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TCM_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TCM initial credit.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400c8UL: // (PBF_REG_XCM_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_XCM_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CCM initial credit.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400ccUL: // (PBF_REG_TM_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TM_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Timer initial credit.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400f8UL: // (PBF_REG_PB_OUT_FIFO_THRSH: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_PB_OUT_FIFO_THRSH (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full threshold for the fifo located at the output \n    of the PB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400fcUL: // (PBF_REG_HA_CMD_Q_THRSH: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_HA_CMD_Q_THRSH (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full threshold for the HA command Q.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140128UL: // (PBF_REG_TCM_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nPBF_REG_TCM_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: Header in TCM messages.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14012cUL: // (PBF_REG_CCM_HDR: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_CCM_HDR (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Header in CCM messages.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140130UL: // (PBF_REG_CCM_AGG_DEC_TYPE: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_CCM_AGG_DEC_TYPE (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CCM aggregation dec type. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140134UL: // (PBF_REG_XCM_HDR: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_XCM_HDR (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Header in XCM messages.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140138UL: // (PBF_REG_XCM_AGG_DEC_TYPE: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_XCM_AGG_DEC_TYPE (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: XCM aggregation dec type. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14013cUL: // (PBF_REG_LAST_FLAG_MASK: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_LAST_FLAG_MASK (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The bit mask value used to set the last flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140140UL: // (PBF_REG_IP_ID_MASK: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_IP_ID_MASK (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: 1st bit mask used to control the rollover when increasing\n     the IP ID field in the packet. Selected per command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140168UL: // (PBF_REG_XCM_MSG_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_XCM_MSG_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of messages sent to XCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14016cUL: // (PBF_REG_CCM_MSG_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CCM_MSG_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of messages sent to CCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140170UL: // (PBF_REG_TCM_MSG_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_TCM_MSG_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of messages sent to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140174UL: // (PBF_REG_TIMER_MSG_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_TIMER_MSG_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of messages sent to timer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140178UL: // (PBF_REG_PCI_INT_MSG_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_PCI_INT_MSG_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of messages sent to PCI internal write interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14017cUL: // (PBF_REG_PCI_AGGR_CNT: ACCESS:ST  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_PCI_AGGR_CNT (ACCESS:ST  DataWidth:0x10 )" \
				"\nDescription: The number of messages which were overwritten because of \n    the aggregation mechanism on the PCI internal write interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140180UL: // (PBF_REG_PCI_RD_REQ_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_PCI_RD_REQ_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number PCI read requests sent.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140184UL: // (PBF_REG_NUM_FULL_CYCLES_2PCI: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_FULL_CYCLES_2PCI (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of cycles in which the full signal towards the PCI\n     was high.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140194UL: // (PBF_REG_NUM_FULL_CYCLES_2SDM: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_FULL_CYCLES_2SDM (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of cycles in which the full signal towards the SDM\n     was high.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140198UL: // (PBF_REG_SEQ_ERR_CID: ACCESS:RC  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_SEQ_ERR_CID (ACCESS:RC  DataWidth:0x18 )" \
				"\nDescription: Holds the CID of the first command that generated a sequence\n     error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14019cUL: // (PBF_REG_TX_MEM_A_EVEN_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_A_EVEN_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401a0UL: // (PBF_REG_TX_MEM_A_ODD_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_A_ODD_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401a4UL: // (PBF_REG_TX_MEM_B_EVEN_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_B_EVEN_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401a8UL: // (PBF_REG_TX_MEM_B_ODD_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_B_ODD_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401acUL: // (PBF_REG_TX_MEM_C_EVEN_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_C_EVEN_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401b0UL: // (PBF_REG_TX_MEM_C_ODD_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_C_ODD_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401b4UL: // (PBF_REG_TQ_MEM_EVEN_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_MEM_EVEN_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401b8UL: // (PBF_REG_TQ_MEM_ODD_TM: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPBF_REG_TQ_MEM_ODD_TM (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSMG_CMD_Q_TM :"
				, (u32_t)((data & PBF_TQ_MEM_ODD_TM_REG_SMG_CMD_Q_TM) >> 18)
				, "\n\tHD_HEADERS_MEM_TM :"
				, (u32_t)((data & PBF_TQ_MEM_ODD_TM_REG_HD_HEADERS_MEM_TM) >> 13)
				, "\n\tLRA_HA_CMD_Q_TM :"
				, (u32_t)((data & PBF_TQ_MEM_ODD_TM_REG_LRA_HA_CMD_Q_TM) >> 5)
				, "\n\tTQ_MEM_ODD_MEM_TM :"
				, (u32_t)((data & PBF_TQ_MEM_ODD_TM_REG_TQ_MEM_ODD_MEM_TM) >> 0));
			break;
		case 0x1401bcUL: // (PBF_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PBF\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401c0UL: // (PBF_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PBF\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401c4UL: // (PBF_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PBF\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401c8UL: // (PBF_REG_PBF_INT_STS: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPBF_REG_PBF_INT_STS (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCRACKER_INFO_FIFO_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_REG_CRACKER_INFO_FIFO_ERROR) >> 6)
				, "\n\tHEADER_BUFFER_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_REG_HEADER_BUFFER_ERROR) >> 5)
				, "\n\tHA_CMD_Q_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_REG_HA_CMD_Q_ERROR) >> 4)
				, "\n\tMCU_DATA_FIFO_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_REG_MCU_DATA_FIFO_ERROR) >> 3)
				, "\n\tSDM_IN_Q_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_REG_SDM_IN_Q_ERROR) >> 2)
				, "\n\tSEQ_CHECK_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_REG_SEQ_CHECK_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1401ccUL: // (PBF_REG_PBF_INT_STS_CLR: ACCESS:RC  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPBF_REG_PBF_INT_STS_CLR (ACCESS:RC  DataWidth:0x7 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCRACKER_INFO_FIFO_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_CLR_REG_CRACKER_INFO_FIFO_ERROR) >> 6)
				, "\n\tHEADER_BUFFER_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_CLR_REG_HEADER_BUFFER_ERROR) >> 5)
				, "\n\tHA_CMD_Q_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_CLR_REG_HA_CMD_Q_ERROR) >> 4)
				, "\n\tMCU_DATA_FIFO_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_CLR_REG_MCU_DATA_FIFO_ERROR) >> 3)
				, "\n\tSDM_IN_Q_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_CLR_REG_SDM_IN_Q_ERROR) >> 2)
				, "\n\tSEQ_CHECK_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_CLR_REG_SEQ_CHECK_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1401d0UL: // (PBF_REG_PBF_INT_STS_WR: ACCESS:WR  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPBF_REG_PBF_INT_STS_WR (ACCESS:WR  DataWidth:0x7 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCRACKER_INFO_FIFO_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_WR_REG_CRACKER_INFO_FIFO_ERROR) >> 6)
				, "\n\tHEADER_BUFFER_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_WR_REG_HEADER_BUFFER_ERROR) >> 5)
				, "\n\tHA_CMD_Q_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_WR_REG_HA_CMD_Q_ERROR) >> 4)
				, "\n\tMCU_DATA_FIFO_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_WR_REG_MCU_DATA_FIFO_ERROR) >> 3)
				, "\n\tSDM_IN_Q_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_WR_REG_SDM_IN_Q_ERROR) >> 2)
				, "\n\tSEQ_CHECK_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_WR_REG_SEQ_CHECK_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1401d4UL: // (PBF_REG_PBF_INT_MASK: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPBF_REG_PBF_INT_MASK (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCRACKER_INFO_FIFO_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_MASK_REG_CRACKER_INFO_FIFO_ERROR) >> 6)
				, "\n\tHEADER_BUFFER_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_MASK_REG_HEADER_BUFFER_ERROR) >> 5)
				, "\n\tHA_CMD_Q_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_MASK_REG_HA_CMD_Q_ERROR) >> 4)
				, "\n\tMCU_DATA_FIFO_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_MASK_REG_MCU_DATA_FIFO_ERROR) >> 3)
				, "\n\tSDM_IN_Q_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_MASK_REG_SDM_IN_Q_ERROR) >> 2)
				, "\n\tSEQ_CHECK_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_MASK_REG_SEQ_CHECK_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PBF_PBF_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1401d8UL: // (PBF_REG_PBF_PRTY_STS: ACCESS:R  DataWidth:0x1c ) 
			sprintf(disp_str, "\nPBF_REG_PBF_PRTY_STS (ACCESS:R  DataWidth:0x1c )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPBUF_G_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_G_ODD) >> 27)
				, "\n\tPBUF_G_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_G_EVEN) >> 26)
				, "\n\tPBUF_F_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_F_ODD) >> 25)
				, "\n\tPBUF_F_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_F_EVEN) >> 24)
				, "\n\tPBUF_E_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_E_ODD) >> 23)
				, "\n\tPBUF_E_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_E_EVEN) >> 22)
				, "\n\tPBUF_D_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_D_ODD) >> 21)
				, "\n\tPBUF_D_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_D_EVEN) >> 20)
				, "\n\tCRACKER_INFO_FIFO :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_CRACKER_INFO_FIFO) >> 19)
				, "\n\tHEADER_BUFFER :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_HEADER_BUFFER) >> 18)
				, "\n\tHM_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_HM_CMD_Q) >> 17)
				, "\n\tSDM_IN_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_SDM_IN_Q) >> 16)
				, "\n\tHD_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_HD_CMD_Q) >> 15)
				, "\n\tHD_HDRS_MEM :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_HD_HDRS_MEM) >> 14)
				, "\n\tHA_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_HA_CMD_Q) >> 13)
				, "\n\tPB_OUT_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PB_OUT_Q) >> 12)
				, "\n\tSMG_CURR_CMD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_SMG_CURR_CMD) >> 11)
				, "\n\tSMG_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_SMG_CMD_Q) >> 10)
				, "\n\tSMG_IN_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_SMG_IN_Q) >> 9)
				, "\n\tPBUF_C_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_C_ODD) >> 8)
				, "\n\tPBUF_C_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_C_EVEN) >> 7)
				, "\n\tPBUF_B_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_B_ODD) >> 6)
				, "\n\tPBUF_B_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_B_EVEN) >> 5)
				, "\n\tPBUF_A_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_A_ODD) >> 4)
				, "\n\tPBUF_A_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PBUF_A_EVEN) >> 3)
				, "\n\tTQ_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_TQ_ODD) >> 2)
				, "\n\tTQ_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_TQ_EVEN) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x1401dcUL: // (PBF_REG_PBF_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x1c ) 
			sprintf(disp_str, "\nPBF_REG_PBF_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x1c )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPBUF_G_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_G_ODD) >> 27)
				, "\n\tPBUF_G_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_G_EVEN) >> 26)
				, "\n\tPBUF_F_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_F_ODD) >> 25)
				, "\n\tPBUF_F_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_F_EVEN) >> 24)
				, "\n\tPBUF_E_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_E_ODD) >> 23)
				, "\n\tPBUF_E_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_E_EVEN) >> 22)
				, "\n\tPBUF_D_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_D_ODD) >> 21)
				, "\n\tPBUF_D_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_D_EVEN) >> 20)
				, "\n\tCRACKER_INFO_FIFO :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_CRACKER_INFO_FIFO) >> 19)
				, "\n\tHEADER_BUFFER :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_HEADER_BUFFER) >> 18)
				, "\n\tHM_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_HM_CMD_Q) >> 17)
				, "\n\tSDM_IN_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_SDM_IN_Q) >> 16)
				, "\n\tHD_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_HD_CMD_Q) >> 15)
				, "\n\tHD_HDRS_MEM :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_HD_HDRS_MEM) >> 14)
				, "\n\tHA_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_HA_CMD_Q) >> 13)
				, "\n\tPB_OUT_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PB_OUT_Q) >> 12)
				, "\n\tSMG_CURR_CMD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_SMG_CURR_CMD) >> 11)
				, "\n\tSMG_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_SMG_CMD_Q) >> 10)
				, "\n\tSMG_IN_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_SMG_IN_Q) >> 9)
				, "\n\tPBUF_C_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_C_ODD) >> 8)
				, "\n\tPBUF_C_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_C_EVEN) >> 7)
				, "\n\tPBUF_B_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_B_ODD) >> 6)
				, "\n\tPBUF_B_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_B_EVEN) >> 5)
				, "\n\tPBUF_A_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_A_ODD) >> 4)
				, "\n\tPBUF_A_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PBUF_A_EVEN) >> 3)
				, "\n\tTQ_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_TQ_ODD) >> 2)
				, "\n\tTQ_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_TQ_EVEN) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x1401e0UL: // (PBF_REG_PBF_PRTY_STS_WR: ACCESS:WR  DataWidth:0x1c ) 
			sprintf(disp_str, "\nPBF_REG_PBF_PRTY_STS_WR (ACCESS:WR  DataWidth:0x1c )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPBUF_G_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_G_ODD) >> 27)
				, "\n\tPBUF_G_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_G_EVEN) >> 26)
				, "\n\tPBUF_F_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_F_ODD) >> 25)
				, "\n\tPBUF_F_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_F_EVEN) >> 24)
				, "\n\tPBUF_E_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_E_ODD) >> 23)
				, "\n\tPBUF_E_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_E_EVEN) >> 22)
				, "\n\tPBUF_D_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_D_ODD) >> 21)
				, "\n\tPBUF_D_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_D_EVEN) >> 20)
				, "\n\tCRACKER_INFO_FIFO :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_CRACKER_INFO_FIFO) >> 19)
				, "\n\tHEADER_BUFFER :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_HEADER_BUFFER) >> 18)
				, "\n\tHM_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_HM_CMD_Q) >> 17)
				, "\n\tSDM_IN_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_SDM_IN_Q) >> 16)
				, "\n\tHD_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_HD_CMD_Q) >> 15)
				, "\n\tHD_HDRS_MEM :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_HD_HDRS_MEM) >> 14)
				, "\n\tHA_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_HA_CMD_Q) >> 13)
				, "\n\tPB_OUT_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PB_OUT_Q) >> 12)
				, "\n\tSMG_CURR_CMD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_SMG_CURR_CMD) >> 11)
				, "\n\tSMG_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_SMG_CMD_Q) >> 10)
				, "\n\tSMG_IN_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_SMG_IN_Q) >> 9)
				, "\n\tPBUF_C_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_C_ODD) >> 8)
				, "\n\tPBUF_C_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_C_EVEN) >> 7)
				, "\n\tPBUF_B_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_B_ODD) >> 6)
				, "\n\tPBUF_B_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_B_EVEN) >> 5)
				, "\n\tPBUF_A_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_A_ODD) >> 4)
				, "\n\tPBUF_A_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PBUF_A_EVEN) >> 3)
				, "\n\tTQ_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_TQ_ODD) >> 2)
				, "\n\tTQ_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_TQ_EVEN) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PBF_PBF_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x1401e4UL: // (PBF_REG_PBF_PRTY_MASK: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nPBF_REG_PBF_PRTY_MASK (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPBUF_G_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_G_ODD) >> 27)
				, "\n\tPBUF_G_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_G_EVEN) >> 26)
				, "\n\tPBUF_F_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_F_ODD) >> 25)
				, "\n\tPBUF_F_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_F_EVEN) >> 24)
				, "\n\tPBUF_E_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_E_ODD) >> 23)
				, "\n\tPBUF_E_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_E_EVEN) >> 22)
				, "\n\tPBUF_D_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_D_ODD) >> 21)
				, "\n\tPBUF_D_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_D_EVEN) >> 20)
				, "\n\tCRACKER_INFO_FIFO :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_CRACKER_INFO_FIFO) >> 19)
				, "\n\tHEADER_BUFFER :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_HEADER_BUFFER) >> 18)
				, "\n\tHM_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_HM_CMD_Q) >> 17)
				, "\n\tSDM_IN_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_SDM_IN_Q) >> 16)
				, "\n\tHD_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_HD_CMD_Q) >> 15)
				, "\n\tHD_HDRS_MEM :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_HD_HDRS_MEM) >> 14)
				, "\n\tHA_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_HA_CMD_Q) >> 13)
				, "\n\tPB_OUT_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PB_OUT_Q) >> 12)
				, "\n\tSMG_CURR_CMD :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_SMG_CURR_CMD) >> 11)
				, "\n\tSMG_CMD_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_SMG_CMD_Q) >> 10)
				, "\n\tSMG_IN_Q :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_SMG_IN_Q) >> 9)
				, "\n\tPBUF_C_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_C_ODD) >> 8)
				, "\n\tPBUF_C_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_C_EVEN) >> 7)
				, "\n\tPBUF_B_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_B_ODD) >> 6)
				, "\n\tPBUF_B_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_B_EVEN) >> 5)
				, "\n\tPBUF_A_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_A_ODD) >> 4)
				, "\n\tPBUF_A_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PBUF_A_EVEN) >> 3)
				, "\n\tTQ_ODD :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_TQ_ODD) >> 2)
				, "\n\tTQ_EVEN :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_TQ_EVEN) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PBF_PBF_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x15c000UL: // (PBF_REG_LLC_JUMBO_THRSH: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nPBF_REG_LLC_JUMBO_THRSH (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The threshold used for deciding whether the type/length field\n     in an LLC SNAP packet is equal to the length or to llc_jumbo_type.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c004UL: // (PBF_REG_LLC_JUMBO_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_LLC_JUMBO_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The value of the type/length in the case of a LLC SNAP packet\n     which is a jumbo packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c020UL: // (PBF_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved for ECO future implementations.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c024UL: // (PBF_REG_E15_FW_COMPAT_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_E15_FW_COMPAT_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: E1.5 FW backward compatibility mode. If set then backward\n     compatible to E1.5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c028UL: // (PBF_REG_LB_TRANSMIT_WINDOW: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_LB_TRANSMIT_WINDOW (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Number of cycles in a transmit window for pacing the LB traffic\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c02cUL: // (PBF_REG_LB_IDLE_WINDOW: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_LB_IDLE_WINDOW (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Number of cycles in a non-transmit window for pacing the \n    LB traffic.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c030UL: // (PBF_REG_HEADER_BUFFER_THRSH: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPBF_REG_HEADER_BUFFER_THRSH (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Almost full threshold for the header buffer fifo.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c034UL: // (PBF_REG_CRACKER_INFO_FIFO_THRSH: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_CRACKER_INFO_FIFO_THRSH (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full threshold for the cracker info fifo. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c038UL: // (PBF_REG_LIMIT_OUTSTANDING_READ_DATA: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_LIMIT_OUTSTANDING_READ_DATA (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set enables limitting the outstanding read data from the\n     PCI.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c03cUL: // (PBF_REG_MAX_OUTSTANDING_READ_DATA: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nPBF_REG_MAX_OUTSTANDING_READ_DATA (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: If limit_outstanding_read_data is set; limits the outstanding\n     PCI read data to this value which is in 16B lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c044UL: // (PBF_REG_LB_PACKETS_WRR_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_LB_PACKETS_WRR_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: WRR weight of commands destined to the LB in the command \n    arbiter\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c048UL: // (PBF_REG_WRR_BURST_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_WRR_BURST_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set; in the command queue arbiter; the WRR arbiters pointer\n     will stay on the winning input; instead of moving to the next one. This\n     allows burstiness. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c06cUL: // (PBF_REG_VLAN_TYPE_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_VLAN_TYPE_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c070UL: // (PBF_REG_VLAN_TYPE_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_VLAN_TYPE_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c074UL: // (PBF_REG_VLAN_TYPE_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_VLAN_TYPE_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c078UL: // (PBF_REG_VLAN_TYPE_3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_VLAN_TYPE_3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c07cUL: // (PBF_REG_VLAN_TYPE_4: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_VLAN_TYPE_4 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c080UL: // (PBF_REG_VLAN_TYPE_5: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_VLAN_TYPE_5 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c084UL: // (PBF_REG_VLAN_TYPE_6: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_VLAN_TYPE_6 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c088UL: // (PBF_REG_VLAN_TYPE_7: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_VLAN_TYPE_7 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c08cUL: // (PBF_REG_FCOE_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_FCOE_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for first FCoE type\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c090UL: // (PBF_REG_TAG_ETHERTYPE_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_TAG_ETHERTYPE_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c094UL: // (PBF_REG_TAG_ETHERTYPE_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_TAG_ETHERTYPE_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c098UL: // (PBF_REG_TAG_ETHERTYPE_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_TAG_ETHERTYPE_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c09cUL: // (PBF_REG_TAG_LEN_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TAG_LEN_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 0.  The length is\n     between 2B and 14B; in 2B granularity\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0a0UL: // (PBF_REG_TAG_LEN_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TAG_LEN_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 1.  The length is\n     between 2B and 14B; in 2B granularity\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0a4UL: // (PBF_REG_TAG_LEN_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TAG_LEN_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 2.  The length is\n     between 2B and 14B; in 2B granularity\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0a8UL: // (PBF_REG_HDRS_AFTER_BASIC: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPBF_REG_HDRS_AFTER_BASIC (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the basic\n     Ethernet header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0acUL: // (PBF_REG_HDRS_AFTER_OUTER_VLAN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPBF_REG_HDRS_AFTER_OUTER_VLAN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the outer\n     VLAN header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0b0UL: // (PBF_REG_HDRS_AFTER_INNER_VLAN: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPBF_REG_HDRS_AFTER_INNER_VLAN (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the inner\n     VLAN header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0b4UL: // (PBF_REG_HDRS_AFTER_LLC: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPBF_REG_HDRS_AFTER_LLC (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the LLC\n     header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0b8UL: // (PBF_REG_HDRS_AFTER_TAG_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPBF_REG_HDRS_AFTER_TAG_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0bcUL: // (PBF_REG_HDRS_AFTER_TAG_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPBF_REG_HDRS_AFTER_TAG_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0c0UL: // (PBF_REG_HDRS_AFTER_TAG_2: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPBF_REG_HDRS_AFTER_TAG_2 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0c4UL: // (PBF_REG_MUST_HAVE_HDRS: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPBF_REG_MUST_HAVE_HDRS (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which headers must appear in the packet\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0c8UL: // (PBF_REG_IP_ID_MASK_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_IP_ID_MASK_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: 2nd bit mask used to control the rollover when increasing\n     the IP ID field in the packet. Selected per command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0ccUL: // (PBF_REG_IP_ID_MASK_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_IP_ID_MASK_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: 3rd bit mask used to control the rollover when increasing\n     the IP ID field in the packet. Selected per command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0d0UL: // (PBF_REG_IP_ID_MASK_3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_IP_ID_MASK_3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: 4th bit mask used to control the rollover when increasing\n     the IP ID field in the packet. Selected per command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0d4UL: // (PBF_REG_ACK_FLG_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_ACK_FLG_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Update mode for the ACK flag\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0d8UL: // (PBF_REG_CWR_FLG_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_CWR_FLG_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Update mode for the CWR flag\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0dcUL: // (PBF_REG_ECE_FLG_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_ECE_FLG_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Update mode for the ECE flag\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0e0UL: // (PBF_REG_FIN_FLG_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_FIN_FLG_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Update mode for the FIN flag\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0e4UL: // (PBF_REG_NS_FLG_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_NS_FLG_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Update mode for the NS flag\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0e8UL: // (PBF_REG_PUSH_FLG_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_PUSH_FLG_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Update mode for the PUSH flag\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0ecUL: // (PBF_REG_RST_FLG_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_RST_FLG_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Update mode for the RST flag\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0f0UL: // (PBF_REG_SYN_FLG_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_SYN_FLG_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Update mode for the SYN flag\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0f4UL: // (PBF_REG_URG_FLG_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_URG_FLG_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Update mode for the URG flag\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c0f8UL: // (PBF_REG_NO_ERR_ON_PRS_ERR: ACCESS:RW  DataWidth:0x8  Multi Field Register  ) 
			sprintf(disp_str, "\nPBF_REG_NO_ERR_ON_PRS_ERR (ACCESS:RW  DataWidth:0x8  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tNO_ERR_ON_TCP_UDP_CHK_PTR_BEYOND_PKT_ERR :"
				, (u32_t)((data & PBF_NO_ERR_ON_PRS_ERR_REG_NO_ERR_ON_TCP_UDP_CHK_PTR_BEYOND_PKT_ERR) >> 7)
				, "\n\tNO_ERR_ON_OVERIP_HM_4NON_1ST_FRAG_ERR :"
				, (u32_t)((data & PBF_NO_ERR_ON_PRS_ERR_REG_NO_ERR_ON_OVERIP_HM_4NON_1ST_FRAG_ERR) >> 6)
				, "\n\tNO_ERR_ON_UNKNOWN_OVER_ETH_ERR :"
				, (u32_t)((data & PBF_NO_ERR_ON_PRS_ERR_REG_NO_ERR_ON_UNKNOWN_OVER_ETH_ERR) >> 5)
				, "\n\tNO_ERR_ON_UNKNOWN_OVER_IPV4_ERR :"
				, (u32_t)((data & PBF_NO_ERR_ON_PRS_ERR_REG_NO_ERR_ON_UNKNOWN_OVER_IPV4_ERR) >> 4)
				, "\n\tNO_ERR_ON_TCP_HDR_PTR_INV_ERR :"
				, (u32_t)((data & PBF_NO_ERR_ON_PRS_ERR_REG_NO_ERR_ON_TCP_HDR_PTR_INV_ERR) >> 3)
				, "\n\tNO_ERR_ON_PRS_PKT_TOO_SMALL_ERR :"
				, (u32_t)((data & PBF_NO_ERR_ON_PRS_ERR_REG_NO_ERR_ON_PRS_PKT_TOO_SMALL_ERR) >> 2)
				, "\n\tNO_ERR_ON_PRS_BAD_IP_HDR_LEN_ERR :"
				, (u32_t)((data & PBF_NO_ERR_ON_PRS_ERR_REG_NO_ERR_ON_PRS_BAD_IP_HDR_LEN_ERR) >> 1)
				, "\n\tNO_ERR_ON_PRS_BAD_IPV_ERR :"
				, (u32_t)((data & PBF_NO_ERR_ON_PRS_ERR_REG_NO_ERR_ON_PRS_BAD_IPV_ERR) >> 0));
			break;
		case 0x15c0fcUL: // (PBF_REG_PRS_ERRORS_MASK: ACCESS:RW  DataWidth:0x8  Multi Field Register  ) 
			sprintf(disp_str, "\nPBF_REG_PRS_ERRORS_MASK (ACCESS:RW  DataWidth:0x8  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMASK_TCP_UDP_CHK_PTR_BEYOND_PKT_ERR :"
				, (u32_t)((data & PBF_PRS_ERRORS_MASK_REG_MASK_TCP_UDP_CHK_PTR_BEYOND_PKT_ERR) >> 7)
				, "\n\tMASK_OVERIP_HM_4NON_1ST_FRAG_ERR :"
				, (u32_t)((data & PBF_PRS_ERRORS_MASK_REG_MASK_OVERIP_HM_4NON_1ST_FRAG_ERR) >> 6)
				, "\n\tMASK_UNKNOWN_OVER_ETH_ERR :"
				, (u32_t)((data & PBF_PRS_ERRORS_MASK_REG_MASK_UNKNOWN_OVER_ETH_ERR) >> 5)
				, "\n\tMASK_UNKNOWN_OVER_IPV4_ERR :"
				, (u32_t)((data & PBF_PRS_ERRORS_MASK_REG_MASK_UNKNOWN_OVER_IPV4_ERR) >> 4)
				, "\n\tMASK_TCP_HDR_PTR_INV_ERR :"
				, (u32_t)((data & PBF_PRS_ERRORS_MASK_REG_MASK_TCP_HDR_PTR_INV_ERR) >> 3)
				, "\n\tMASK_PRS_PKT_TOO_SMALL_ERR :"
				, (u32_t)((data & PBF_PRS_ERRORS_MASK_REG_MASK_PRS_PKT_TOO_SMALL_ERR) >> 2)
				, "\n\tMASK_PRS_BAD_IP_HDR_LEN_ERR :"
				, (u32_t)((data & PBF_PRS_ERRORS_MASK_REG_MASK_PRS_BAD_IP_HDR_LEN_ERR) >> 1)
				, "\n\tMASK_PRS_BAD_IPV_ERR :"
				, (u32_t)((data & PBF_PRS_ERRORS_MASK_REG_MASK_PRS_BAD_IPV_ERR) >> 0));
			break;
		case 0x15c100UL: // (PBF_REG_DROP_PKT_UPON_ERR: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DROP_PKT_UPON_ERR (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: if set; packets with error will be dropped; otherwise transmitted\n     with error to the NIG (FCS).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c104UL: // (PBF_REG_NO_PAD_IN_PKT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_NO_PAD_IN_PKT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: if set; there is no expected L2 padding in the packet.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c108UL: // (PBF_REG_NUM_PRS_BAD_IPV_ERRORS: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_PRS_BAD_IPV_ERRORS (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with bad IPV error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c10cUL: // (PBF_REG_NUM_PRS_BAD_IP_HDR_LEN_ERRORS: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_PRS_BAD_IP_HDR_LEN_ERRORS (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with bad IP header length.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c110UL: // (PBF_REG_NUM_PRS_PKT_TOO_SMALL_ERRORS: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_PRS_PKT_TOO_SMALL_ERRORS (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with packet too small error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c114UL: // (PBF_REG_NUM_TCP_HDR_PTR_INV_ERRORS: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_TCP_HDR_PTR_INV_ERRORS (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with an invalid TCP header pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c118UL: // (PBF_REG_NUM_UNKNOWN_OVER_IPV4_ERRORS: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_UNKNOWN_OVER_IPV4_ERRORS (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets which were identified to have an unknown\n     over IPV4 protocol error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c11cUL: // (PBF_REG_NUM_UNKNOWN_OVER_ETH_ERRORS: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_UNKNOWN_OVER_ETH_ERRORS (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: if set; a packet which was identified to have an unknown \n    over ethernet protocol error will not be transmitted with an error/dropped\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c120UL: // (PBF_REG_NUM_OVERIP_HM_4NON_1ST_FRAG_ERRORS: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_OVERIP_HM_4NON_1ST_FRAG_ERRORS (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets which were identified to have a non-first\n     fragment error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c124UL: // (PBF_REG_NUM_TCP_UDP_CHK_PTR_BEYOND_PKT_ERRORS: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_TCP_UDP_CHK_PTR_BEYOND_PKT_ERRORS (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets which were identified to have a TCP/UDP\n     checksum pointer beyond the packet error.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c130UL: // (PBF_REG_NUM_PCI_ERRORS: ACCESS:ST  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_PCI_ERRORS (ACCESS:ST  DataWidth:0x10 )" \
				"\nDescription: Number of errors from the PCI read interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c134UL: // (PBF_REG_TXP_MEM_ADDR: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TXP_MEM_ADDR (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Address for accessing the TXP memories. Should be written\n     before reading/writing through the WB registers.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c138UL: // (PBF_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c13cUL: // (PBF_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c140UL: // (PBF_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c144UL: // (PBF_REG_CPU_MBIST_MEMCTRL_0_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_CPU_MBIST_MEMCTRL_0_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c148UL: // (PBF_REG_CPU_MBIST_MEMCTRL_1_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_CPU_MBIST_MEMCTRL_1_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c14cUL: // (PBF_REG_CPU_MBIST_MEMCTRL_2_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_CPU_MBIST_MEMCTRL_2_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -  mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 -  mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c150UL: // (PBF_REG_HIGIG_HDR_SIZE_P0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_HIGIG_HDR_SIZE_P0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: HiGiG header size in DWORDS for PBF port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c154UL: // (PBF_REG_HIGIG_HDR_SIZE_P1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_HIGIG_HDR_SIZE_P1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: HiGiG header size in DWORDS for PBF port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c160UL: // (PBF_REG_INIT_Q0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT_Q0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Init bit for queue 0. When set the initial credit of queue\n     0 is copied to the credit register. Should be set and then reset after\n     the configuration of the port has ended. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c164UL: // (PBF_REG_INIT_Q1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT_Q1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Init bit for queue 1. When set the initial credit of queue\n     1 is copied to the credit register. Should be set and then reset after\n     the configuration of the port has ended. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c168UL: // (PBF_REG_INIT_Q2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT_Q2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Init bit for queue 2. When set the initial credit of queue\n     2 is copied to the credit register. Should be set and then reset after\n     the configuration of the port has ended. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c16cUL: // (PBF_REG_INIT_Q3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT_Q3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Init bit for queue 3. When set the initial credit of queue\n     3 is copied to the credit register. Should be set and then reset after\n     the configuration of the port has ended. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c170UL: // (PBF_REG_INIT_Q4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT_Q4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Init bit for queue 4. When set the initial credit of queue\n     4 is copied to the credit register. Should be set and then reset after\n     the configuration of the port has ended. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c174UL: // (PBF_REG_INIT_Q5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT_Q5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Init bit for queue 5. When set the initial credit of queue\n     5 is copied to the credit register. Should be set and then reset after\n     the configuration of the port has ended. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c178UL: // (PBF_REG_INIT_LB_Q: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT_LB_Q (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Init bit for LB queue. When set the initial credit of the\n     LB queue is copied to the credit register. Should be set and then reset\n     after the configuration of the port has ended. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c17cUL: // (PBF_REG_PAUSE_ENABLE_Q0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_PAUSE_ENABLE_Q0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indication that pause is enabled for queue 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c180UL: // (PBF_REG_PAUSE_ENABLE_Q1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_PAUSE_ENABLE_Q1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indication that pause is enabled for queue 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c184UL: // (PBF_REG_PAUSE_ENABLE_Q2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_PAUSE_ENABLE_Q2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indication that pause is enabled for queue 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c188UL: // (PBF_REG_PAUSE_ENABLE_Q3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_PAUSE_ENABLE_Q3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indication that pause is enabled for queue 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c18cUL: // (PBF_REG_PAUSE_ENABLE_Q4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_PAUSE_ENABLE_Q4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indication that pause is enabled for queue 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c190UL: // (PBF_REG_PAUSE_ENABLE_Q5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_PAUSE_ENABLE_Q5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indication that pause is enabled for queue 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c194UL: // (PBF_REG_PAUSE_ENABLE_LB_Q: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_PAUSE_ENABLE_LB_Q (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indication that pause is enabled for queue 6.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c198UL: // (PBF_REG_MAC_IF4_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_MAC_IF4_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for mac interface 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c19cUL: // (PBF_REG_MAC_IF5_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_MAC_IF5_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for mac interface 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1a0UL: // (PBF_REG_DISABLE_WR2TQ_Q0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_Q0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable the STORM writing to queue 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1a4UL: // (PBF_REG_DISABLE_WR2TQ_Q1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_Q1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable the STORM writing to queue 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1a8UL: // (PBF_REG_DISABLE_WR2TQ_Q2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_Q2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable the STORM writing to queue 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1acUL: // (PBF_REG_DISABLE_WR2TQ_Q3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_Q3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable the STORM writing to queue 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1b0UL: // (PBF_REG_DISABLE_WR2TQ_Q4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_Q4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable the STORM writing to queue 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1b4UL: // (PBF_REG_DISABLE_WR2TQ_Q5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_Q5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable the STORM writing to queue 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1b8UL: // (PBF_REG_DISABLE_WR2TQ_LB_Q: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_LB_Q (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable the STORM writing to the LB queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1bcUL: // (PBF_REG_DISABLE_NEW_TASK_PROC_Q0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_Q0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable processing further tasks from port 0 (after ending\n     the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1c0UL: // (PBF_REG_DISABLE_NEW_TASK_PROC_Q1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_Q1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable processing further tasks from port 0 (after ending\n     the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1c4UL: // (PBF_REG_DISABLE_NEW_TASK_PROC_Q2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_Q2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable processing further tasks from port 0 (after ending\n     the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1c8UL: // (PBF_REG_DISABLE_NEW_TASK_PROC_Q3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_Q3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable processing further tasks from port 0 (after ending\n     the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1ccUL: // (PBF_REG_DISABLE_NEW_TASK_PROC_Q4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_Q4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable processing further tasks from port 0 (after ending\n     the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1d0UL: // (PBF_REG_DISABLE_NEW_TASK_PROC_Q5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_Q5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable processing further tasks from port 0 (after ending\n     the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1d4UL: // (PBF_REG_DISABLE_NEW_TASK_PROC_LB_Q: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_LB_Q (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable processing further tasks from port 0 (after ending\n     the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1d8UL: // (PBF_REG_TQ_NUM_LINES_Q0: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TQ_NUM_LINES_Q0 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Number of 16 byte lines in the task Q reserved for queue \n    0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1dcUL: // (PBF_REG_TQ_NUM_LINES_Q1: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TQ_NUM_LINES_Q1 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Number of 16 byte lines in the task Q reserved for queue \n    1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1e0UL: // (PBF_REG_TQ_NUM_LINES_Q2: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TQ_NUM_LINES_Q2 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Number of 16 byte lines in the task Q reserved for queue \n    2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1e4UL: // (PBF_REG_TQ_NUM_LINES_Q3: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TQ_NUM_LINES_Q3 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Number of 16 byte lines in the task Q reserved for queue \n    3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1e8UL: // (PBF_REG_TQ_NUM_LINES_Q4: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TQ_NUM_LINES_Q4 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Number of 16 byte lines in the task Q reserved for queue \n    4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1ecUL: // (PBF_REG_TQ_NUM_LINES_Q5: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TQ_NUM_LINES_Q5 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Number of 16 byte lines in the task Q reserved for queue \n    5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1f0UL: // (PBF_REG_TQ_NUM_LINES_LB_Q: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TQ_NUM_LINES_LB_Q (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Number of 16 byte lines in the task Q reserved for the LB\n     queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1f4UL: // (PBF_REG_XSDM_ADDR_0_Q_NUM: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_0_Q_NUM (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The queue number to which address 0 from XSDM mapps to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1f8UL: // (PBF_REG_XSDM_ADDR_1_Q_NUM: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_1_Q_NUM (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The queue number to which address 1 from XSDM mapps to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c1fcUL: // (PBF_REG_XSDM_ADDR_2_Q_NUM: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_2_Q_NUM (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The queue number to which address 2 from XSDM mapps to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c200UL: // (PBF_REG_XSDM_ADDR_3_Q_NUM: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_3_Q_NUM (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The queue number to which address 3 from XSDM mapps to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c204UL: // (PBF_REG_XSDM_ADDR_4_Q_NUM: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_4_Q_NUM (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The queue number to which address 4 from XSDM mapps to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c208UL: // (PBF_REG_XSDM_ADDR_5_Q_NUM: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_5_Q_NUM (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The queue number to which address 5 from XSDM mapps to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c20cUL: // (PBF_REG_XSDM_ADDR_6_Q_NUM: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_6_Q_NUM (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The queue number to which address 6 from XSDM mapps to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c210UL: // (PBF_REG_XSDM_ADDR_7_Q_NUM: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_7_Q_NUM (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The queue number to which address 7 from XSDM mapps to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c214UL: // (PBF_REG_TQ_THRSH_Q0: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_THRSH_Q0 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full threshold for queue 0 in task Q in 16 byte lines\n    . \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c218UL: // (PBF_REG_TQ_THRSH_Q1: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_THRSH_Q1 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full threshold for queue 1 in task Q in 16 byte lines\n    . \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c21cUL: // (PBF_REG_TQ_THRSH_Q2: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_THRSH_Q2 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full threshold for queue 2 in task Q in 16 byte lines\n    . \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c220UL: // (PBF_REG_TQ_THRSH_Q3: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_THRSH_Q3 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full threshold for queue 3 in task Q in 16 byte lines\n    . \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c224UL: // (PBF_REG_TQ_THRSH_Q4: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_THRSH_Q4 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full threshold for queue 4 in task Q in 16 byte lines\n    . \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c228UL: // (PBF_REG_TQ_THRSH_Q5: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_THRSH_Q5 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full threshold for queue 5 in task Q in 16 byte lines\n    . \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c22cUL: // (PBF_REG_TQ_THRSH_LB_Q: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_THRSH_LB_Q (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full threshold for the LB queue in task Q in 16 byte\n     lines. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c230UL: // (PBF_REG_INIT_CRD_Q0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_INIT_CRD_Q0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Initial credit for queue 0 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c234UL: // (PBF_REG_INIT_CRD_Q1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_INIT_CRD_Q1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Initial credit for queue 1 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c238UL: // (PBF_REG_INIT_CRD_Q2: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_INIT_CRD_Q2 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Initial credit for queue 2 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c23cUL: // (PBF_REG_INIT_CRD_Q3: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_INIT_CRD_Q3 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Initial credit for queue 3 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c240UL: // (PBF_REG_INIT_CRD_Q4: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_INIT_CRD_Q4 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Initial credit for queue 4 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c244UL: // (PBF_REG_INIT_CRD_Q5: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_INIT_CRD_Q5 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Initial credit for queue 5 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c248UL: // (PBF_REG_INIT_CRD_LB_Q: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_INIT_CRD_LB_Q (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Initial credit for the LB queue in the tx port buffers in\n     16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c24cUL: // (PBF_REG_ARB_THRSH_Q0: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_ARB_THRSH_Q0 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Queue 0 threshold used by arbiter in 16 byte lines used when\n     pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c250UL: // (PBF_REG_ARB_THRSH_Q1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_ARB_THRSH_Q1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Queue 1 threshold used by arbiter in 16 byte lines used when\n     pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c254UL: // (PBF_REG_ARB_THRSH_Q2: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_ARB_THRSH_Q2 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Queue 2 threshold used by arbiter in 16 byte lines used when\n     pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c258UL: // (PBF_REG_ARB_THRSH_Q3: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_ARB_THRSH_Q3 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Queue 3 threshold used by arbiter in 16 byte lines used when\n     pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c25cUL: // (PBF_REG_ARB_THRSH_Q4: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_ARB_THRSH_Q4 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Queue 4 threshold used by arbiter in 16 byte lines used when\n     pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c260UL: // (PBF_REG_ARB_THRSH_Q5: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_ARB_THRSH_Q5 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Queue 5 threshold used by arbiter in 16 byte lines used when\n     pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c264UL: // (PBF_REG_ARB_THRSH_LB_Q: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_ARB_THRSH_LB_Q (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: LB Queue threshold used by arbiter in 16 byte lines used \n    when pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c268UL: // (PBF_REG_P0_PACKETS_WRR_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_P0_PACKETS_WRR_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: WRR weight of commands destined to port 0 in the command \n    arbiter\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c26cUL: // (PBF_REG_P1_PACKETS_WRR_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_P1_PACKETS_WRR_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: WRR weight of commands destined to port 1 in the command \n    arbiter\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c270UL: // (PBF_REG_ETS_ARB_PRIORITY_CLIENT_P0: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_PRIORITY_CLIENT_P0 (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: For port 0: Indicates which client is connected to each priority\n     in the strict-priority arbiter. Priority 0 is the highest priority, and\n     priority 5 is the lowest; to which the RR output is connected to (this\n     is not configurable).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c274UL: // (PBF_REG_ETS_ARB_PRIORITY_CLIENT_P1: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_PRIORITY_CLIENT_P1 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: For port 1: Indicates which client is connected to each priority\n     in the strict-priority arbiter. Priority 0 is the highest priority, and\n     priority 5 is the lowest; to which the RR output is connected to (this\n     is not configurable).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c278UL: // (PBF_REG_ETS_ARB_CLIENT_IS_STRICT_P0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_CLIENT_IS_STRICT_P0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: For port 0: Bit per client to indicate if the client competes\n     in the strict priority arbiter directly (corresponding bit = 1); or first\n     goes to the RR arbiter (corresponding bit = 0); and then competes in \n    the lowest priority in the strict-priority arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c27cUL: // (PBF_REG_ETS_ARB_CLIENT_IS_STRICT_P1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_CLIENT_IS_STRICT_P1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: For port 1: Bit per client to indicate if the client competes\n     in the strict priority arbiter directly (corresponding bit = 1); or first\n     goes to the RR arbiter (corresponding bit = 0); and then competes in \n    the lowest priority in the strict-priority arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c280UL: // (PBF_REG_ETS_ARB_CLIENT_IS_SUBJECT2WFQ_P0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_CLIENT_IS_SUBJECT2WFQ_P0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: For port 0: Bit per client to indicate if the client is subject\n     to WFQ credit blocking (corresponding bit = 1).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c284UL: // (PBF_REG_ETS_ARB_CLIENT_IS_SUBJECT2WFQ_P1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_CLIENT_IS_SUBJECT2WFQ_P1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: For port 0: Bit per client to indicate if the client is subject\n     to WFQ credit blocking (corresponding bit = 1).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c288UL: // (PBF_REG_ETS_ARB_CLIENT_CREDIT_MAP_P0: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_CLIENT_CREDIT_MAP_P0 (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: For port 0: For each client that is subject to WFQ (the corresponding\n     bit is 1); indicates to which of the credit registers this client is \n    mapped. For clients which are not credit blocked; their mapping is dont\n     care.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c28cUL: // (PBF_REG_ETS_ARB_CLIENT_CREDIT_MAP_P1: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_CLIENT_CREDIT_MAP_P1 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: For port 1: For each client that is subject to WFQ (the corresponding\n     bit is 1); indicates to which of the credit registers this client is \n    mapped. For clients which are not credit blocked; their mapping is dont\n     care.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c290UL: // (PBF_REG_ETS_ARB_RR_BURST_MODE_P0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_RR_BURST_MODE_P0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For port 0: If set; the round robin arbiters pointer will\n     stay on the winning input; instead of moving to the next one. This allows\n     burstiness. This mode should be carefully used; since in certain arbitration\n     configurations it can cause starvation.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c294UL: // (PBF_REG_ETS_ARB_RR_BURST_MODE_P1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_RR_BURST_MODE_P1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For port 1: If set; the round robin arbiters pointer will\n     stay on the winning input; instead of moving to the next one. This allows\n     burstiness. This mode should be carefully used; since in certain arbitration\n     configurations it can cause starvation.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c298UL: // (PBF_REG_ETS_ARB_SPWAS_BURST_MODE_P0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_SPWAS_BURST_MODE_P0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For port 0: If set; the round robin arbiter within the strict\n     priority w/ anti-starvation arbiter; will stay on the winning input; \n    instead of moving to the next one. This allows burstiness. This mode should\n     be carefully used; since in certain arbitration configurations it can\n     cause starvation.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c29cUL: // (PBF_REG_ETS_ARB_SPWAS_BURST_MODE_P1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_SPWAS_BURST_MODE_P1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For port 1: If set; the round robin arbiter within the strict\n     priority w/ anti-starvation arbiter; will stay on the winning input; \n    instead of moving to the next one. This allows burstiness. This mode should\n     be carefully used; since in certain arbitration configurations it can\n     cause starvation.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2a0UL: // (PBF_REG_ETS_ARB_NUM_STRICT_ARB_SLOTS_P0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_NUM_STRICT_ARB_SLOTS_P0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: For port 0: The number of strict priority arbitration slots\n     between 2 RR arbitration slots. A value of 0 means no strict priority\n     cycles; i.e. the strict-priority w/ anti-starvation arbiter is a RR arbiter\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2a4UL: // (PBF_REG_ETS_ARB_NUM_STRICT_ARB_SLOTS_P1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ARB_NUM_STRICT_ARB_SLOTS_P1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: For port 1: The number of strict priority arbitration slots\n     between 2 RR arbitration slots. A value of 0 means no strict priority\n     cycles; i.e. the strict-priority w/ anti-starvation arbiter is a RR arbiter\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2a8UL: // (PBF_REG_COS0_WEIGHT_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS0_WEIGHT_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The weight of COS0 in port 0 ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2acUL: // (PBF_REG_COS1_WEIGHT_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS1_WEIGHT_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The weight of COS1 in port 0 ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2b0UL: // (PBF_REG_COS2_WEIGHT_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS2_WEIGHT_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The weight of COS2 in port 0 ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2b4UL: // (PBF_REG_COS3_WEIGHT_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS3_WEIGHT_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The weight of COS3 in port 0 ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2b8UL: // (PBF_REG_COS4_WEIGHT_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS4_WEIGHT_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The weight of COS4 in port 0 ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2bcUL: // (PBF_REG_COS5_WEIGHT_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS5_WEIGHT_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The weight of COS5 in port 0 ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2c0UL: // (PBF_REG_COS0_WEIGHT_P1: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS0_WEIGHT_P1 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The weight of COS0 in port 1 ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2c4UL: // (PBF_REG_COS1_WEIGHT_P1: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS1_WEIGHT_P1 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The weight of COS1 in port 1 ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2c8UL: // (PBF_REG_COS2_WEIGHT_P1: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS2_WEIGHT_P1 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The weight of COS2 in port 1 ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2ccUL: // (PBF_REG_COS0_UPPER_BOUND_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS0_UPPER_BOUND_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The upper bound of the weight of COS0 in the ETS command \n    arbiter of port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2d0UL: // (PBF_REG_COS1_UPPER_BOUND_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS1_UPPER_BOUND_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The upper bound of the weight of COS1 in the ETS command \n    arbiter of port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2d4UL: // (PBF_REG_COS2_UPPER_BOUND_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS2_UPPER_BOUND_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The upper bound of the weight of COS2 in the ETS command \n    arbiter of port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2d8UL: // (PBF_REG_COS3_UPPER_BOUND_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS3_UPPER_BOUND_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The upper bound of the weight of COS3 in the ETS command \n    arbiter of port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2dcUL: // (PBF_REG_COS4_UPPER_BOUND_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS4_UPPER_BOUND_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The upper bound of the weight of COS4 in the ETS command \n    arbiter of port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2e0UL: // (PBF_REG_COS5_UPPER_BOUND_P0: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS5_UPPER_BOUND_P0 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The upper bound of the weight of COS5 in the ETS command \n    arbiter of port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2e4UL: // (PBF_REG_COS0_UPPER_BOUND_P1: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS0_UPPER_BOUND_P1 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The upper bound of the weight of COS0 in the ETS command \n    arbiter of port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2e8UL: // (PBF_REG_COS1_UPPER_BOUND_P1: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS1_UPPER_BOUND_P1 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The upper bound of the weight of COS1 in the ETS command \n    arbiter of port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2ecUL: // (PBF_REG_COS2_UPPER_BOUND_P1: ACCESS:RW  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS2_UPPER_BOUND_P1 (ACCESS:RW  DataWidth:0x1f )" \
				"\nDescription: The upper bound of the weight of COS2 in the ETS command \n    arbiter of port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2f0UL: // (PBF_REG_TXP_THRSH_Q0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TXP_THRSH_Q0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Tx queue 0 almost full threshold in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2f4UL: // (PBF_REG_TXP_THRSH_Q1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TXP_THRSH_Q1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Tx queue 1 almost full threshold in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2f8UL: // (PBF_REG_TXP_THRSH_Q2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TXP_THRSH_Q2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Tx queue 2 almost full threshold in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c2fcUL: // (PBF_REG_TXP_THRSH_Q3: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TXP_THRSH_Q3 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Tx queue 3 almost full threshold in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c300UL: // (PBF_REG_TXP_THRSH_Q4: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TXP_THRSH_Q4 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Tx queue 4 almost full threshold in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c304UL: // (PBF_REG_TXP_THRSH_Q5: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TXP_THRSH_Q5 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Tx queue 5 almost full threshold in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c308UL: // (PBF_REG_TXP_LB_Q: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_TXP_LB_Q (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Tx LB queue almost full threshold in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c30cUL: // (PBF_REG_NUM_SENT_BYTES_Q0: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_BYTES_Q0 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of bytes transmitted on queue 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c310UL: // (PBF_REG_NUM_SENT_BYTES_Q1: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_BYTES_Q1 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of bytes transmitted on queue 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c314UL: // (PBF_REG_NUM_SENT_BYTES_Q2: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_BYTES_Q2 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of bytes transmitted on queue 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c318UL: // (PBF_REG_NUM_SENT_BYTES_Q3: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_BYTES_Q3 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of bytes transmitted on queue 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c31cUL: // (PBF_REG_NUM_SENT_BYTES_Q4: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_BYTES_Q4 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of bytes transmitted on queue 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c320UL: // (PBF_REG_NUM_SENT_BYTES_Q5: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_BYTES_Q5 (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of bytes transmitted on queue 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c324UL: // (PBF_REG_NUM_SENT_BYTES_LB_Q: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_BYTES_LB_Q (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Number of bytes transmitted on the LB queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c328UL: // (PBF_REG_NUM_SENT_PKTS_Q0: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_PKTS_Q0 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of packets transmitted on queue 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c32cUL: // (PBF_REG_NUM_SENT_PKTS_Q1: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_PKTS_Q1 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of packets transmitted on queue 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c330UL: // (PBF_REG_NUM_SENT_PKTS_Q2: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_PKTS_Q2 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of packets transmitted on queue 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c334UL: // (PBF_REG_NUM_SENT_PKTS_Q3: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_PKTS_Q3 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of packets transmitted on queue 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c338UL: // (PBF_REG_NUM_SENT_PKTS_Q4: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_PKTS_Q4 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of packets transmitted on queue 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c33cUL: // (PBF_REG_NUM_SENT_PKTS_Q5: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_PKTS_Q5 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of packets transmitted on queue 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c340UL: // (PBF_REG_NUM_SENT_PKTS_LB_Q: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_PKTS_LB_Q (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of packets transmitted on the LB queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c344UL: // (PBF_REG_NUM_ERR_SENT_PKTS_Q0: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_ERR_SENT_PKTS_Q0 (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with error transmitted on queue 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c348UL: // (PBF_REG_NUM_ERR_SENT_PKTS_Q1: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_ERR_SENT_PKTS_Q1 (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with error transmitted on queue 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c34cUL: // (PBF_REG_NUM_ERR_SENT_PKTS_Q2: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_ERR_SENT_PKTS_Q2 (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with error transmitted on queue 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c350UL: // (PBF_REG_NUM_ERR_SENT_PKTS_Q3: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_ERR_SENT_PKTS_Q3 (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with error transmitted on queue 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c354UL: // (PBF_REG_NUM_ERR_SENT_PKTS_Q4: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_ERR_SENT_PKTS_Q4 (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with error transmitted on queue 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c358UL: // (PBF_REG_NUM_ERR_SENT_PKTS_Q5: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_ERR_SENT_PKTS_Q5 (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with error transmitted on queue 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c35cUL: // (PBF_REG_NUM_ERR_SENT_PKTS_LB_Q: ACCESS:ST  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_ERR_SENT_PKTS_LB_Q (ACCESS:ST  DataWidth:0x8 )" \
				"\nDescription: Number of packets with error transmitted on the LB queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c360UL: // (PBF_REG_LAST_CID_W_ERROR_Q0: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nPBF_REG_LAST_CID_W_ERROR_Q0 (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: last CID with either parsing or PCI error which was transmitted\n     with error or dropped on queue 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c364UL: // (PBF_REG_LAST_CID_W_ERROR_Q1: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nPBF_REG_LAST_CID_W_ERROR_Q1 (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: last CID with either parsing or PCI error which was transmitted\n     with error or dropped on queue 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c368UL: // (PBF_REG_LAST_CID_W_ERROR_Q2: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nPBF_REG_LAST_CID_W_ERROR_Q2 (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: last CID with either parsing or PCI error which was transmitted\n     with error or dropped on queue 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c36cUL: // (PBF_REG_LAST_CID_W_ERROR_Q3: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nPBF_REG_LAST_CID_W_ERROR_Q3 (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: last CID with either parsing or PCI error which was transmitted\n     with error or dropped on queue 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c370UL: // (PBF_REG_LAST_CID_W_ERROR_Q4: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nPBF_REG_LAST_CID_W_ERROR_Q4 (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: last CID with either parsing or PCI error which was transmitted\n     with error or dropped on queue 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c374UL: // (PBF_REG_LAST_CID_W_ERROR_Q5: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nPBF_REG_LAST_CID_W_ERROR_Q5 (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: last CID with either parsing or PCI error which was transmitted\n     with error or dropped on queue 5.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c378UL: // (PBF_REG_CMDS_RCVD_ON_Q0: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CMDS_RCVD_ON_Q0 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of commands received on queue 0 from STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c37cUL: // (PBF_REG_CMDS_RCVD_ON_Q1: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CMDS_RCVD_ON_Q1 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of commands received on queue 1 from STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c380UL: // (PBF_REG_CMDS_RCVD_ON_Q2: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CMDS_RCVD_ON_Q2 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of commands received on queue 2 from STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c384UL: // (PBF_REG_CMDS_RCVD_ON_Q3: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CMDS_RCVD_ON_Q3 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of commands received on queue 3 from STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c388UL: // (PBF_REG_CMDS_RCVD_ON_Q4: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CMDS_RCVD_ON_Q4 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of commands received on queue 4 from STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c38cUL: // (PBF_REG_CMDS_RCVD_ON_Q5: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CMDS_RCVD_ON_Q5 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of commands received on queue 5 from STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c390UL: // (PBF_REG_CMDS_RCVD_ON_LB_Q: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CMDS_RCVD_ON_LB_Q (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Number of commands received on the LB queue from STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c394UL: // (PBF_REG_TX_MEM_D_EVEN_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_D_EVEN_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c398UL: // (PBF_REG_TX_MEM_D_ODD_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_D_ODD_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c39cUL: // (PBF_REG_TX_MEM_E_EVEN_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_E_EVEN_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c3a0UL: // (PBF_REG_TX_MEM_E_ODD_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_E_ODD_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c3a4UL: // (PBF_REG_TX_MEM_F_EVEN_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_F_EVEN_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c3a8UL: // (PBF_REG_TX_MEM_F_ODD_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_F_ODD_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c3acUL: // (PBF_REG_TX_MEM_G_EVEN_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_G_EVEN_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c3b0UL: // (PBF_REG_TX_MEM_G_ODD_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_TX_MEM_G_ODD_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Test mode input port for memory instance.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402e8UL: // (PBF_REG_DISABLE_PF: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_PF (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Disable a PF from transmitting.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402ecUL: // (PBF_REG_DISABLE_VF: ACCESS:RW  DataWidth:0x1  SPLIT:64 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_VF (ACCESS:RW  DataWidth:0x1  SPLIT:64 )" \
				"\nDescription: Disable a VF from transmitting.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140314UL: // (PBF_REG_COS0_WFQ_CREDIT_P0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS0_WFQ_CREDIT_P0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current WFQ credit of COS0 in port 0 ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140318UL: // (PBF_REG_COS0_WFQ_CREDIT_P1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS0_WFQ_CREDIT_P1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current WFQ credit of COS0 in port 1 ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14031cUL: // (PBF_REG_COS1_WFQ_CREDIT_P0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS1_WFQ_CREDIT_P0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current WFQ credit of COS1 in port 0 ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140320UL: // (PBF_REG_COS1_WFQ_CREDIT_P1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS1_WFQ_CREDIT_P1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current WFQ credit of COS1 in port 1 ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140324UL: // (PBF_REG_COS2_WFQ_CREDIT_P0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS2_WFQ_CREDIT_P0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current WFQ credit of COS2 in port 0 ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140328UL: // (PBF_REG_COS2_WFQ_CREDIT_P1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS2_WFQ_CREDIT_P1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current WFQ credit of COS2 in port 1 ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14032cUL: // (PBF_REG_COS3_WFQ_CREDIT_P0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS3_WFQ_CREDIT_P0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current WFQ credit of COS3 in port 0 ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140330UL: // (PBF_REG_COS4_WFQ_CREDIT_P0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS4_WFQ_CREDIT_P0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current WFQ credit of COS4 in port 0 ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140334UL: // (PBF_REG_COS5_WFQ_CREDIT_P0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS5_WFQ_CREDIT_P0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current WFQ credit of COS5 in port 0 ETS arbiter. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140338UL: // (PBF_REG_CREDIT_LB_Q: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_CREDIT_LB_Q (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Current credit for the LB queue in the tx port buffers in\n     16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14033cUL: // (PBF_REG_CREDIT_Q0: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_CREDIT_Q0 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Current credit for queue 0 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140340UL: // (PBF_REG_CREDIT_Q1: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_CREDIT_Q1 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Current credit for queue 1 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140344UL: // (PBF_REG_CREDIT_Q2: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_CREDIT_Q2 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Current credit for queue 2 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140348UL: // (PBF_REG_CREDIT_Q3: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_CREDIT_Q3 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Current credit for queue 3 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14034cUL: // (PBF_REG_CREDIT_Q4: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_CREDIT_Q4 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Current credit for queue 4 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140350UL: // (PBF_REG_CREDIT_Q5: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_CREDIT_Q5 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Current credit for queue 5 in the tx port buffers in 16 byte\n     lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140354UL: // (PBF_REG_INTERNAL_CRD_FREED_CNT_LB_Q: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_INTERNAL_CRD_FREED_CNT_LB_Q (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for the amount credits in 16 bytes lines added\n     for the LB queue. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140358UL: // (PBF_REG_INTERNAL_CRD_FREED_CNT_Q0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_INTERNAL_CRD_FREED_CNT_Q0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for the amount credits in 16 bytes lines added\n     for queue 0. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14035cUL: // (PBF_REG_INTERNAL_CRD_FREED_CNT_Q1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_INTERNAL_CRD_FREED_CNT_Q1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for the amount credits in 16 bytes lines added\n     for queue 1. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140360UL: // (PBF_REG_INTERNAL_CRD_FREED_CNT_Q2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_INTERNAL_CRD_FREED_CNT_Q2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for the amount credits in 16 bytes lines added\n     for queue 2. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140364UL: // (PBF_REG_INTERNAL_CRD_FREED_CNT_Q3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_INTERNAL_CRD_FREED_CNT_Q3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for the amount credits in 16 bytes lines added\n     for queue 3. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140368UL: // (PBF_REG_INTERNAL_CRD_FREED_CNT_Q4: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_INTERNAL_CRD_FREED_CNT_Q4 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for the amount credits in 16 bytes lines added\n     for queue 4. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14036cUL: // (PBF_REG_INTERNAL_CRD_FREED_CNT_Q5: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_INTERNAL_CRD_FREED_CNT_Q5 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for the amount credits in 16 bytes lines added\n     for queue 5. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140370UL: // (PBF_REG_TASK_CNT_LB_Q: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_TASK_CNT_LB_Q (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of tasks in queue 0 task queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140374UL: // (PBF_REG_TASK_CNT_Q0: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_TASK_CNT_Q0 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of tasks in queue 0 task queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140378UL: // (PBF_REG_TASK_CNT_Q1: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_TASK_CNT_Q1 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of tasks in queue 0 task queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14037cUL: // (PBF_REG_TASK_CNT_Q2: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_TASK_CNT_Q2 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of tasks in queue 0 task queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140380UL: // (PBF_REG_TASK_CNT_Q3: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_TASK_CNT_Q3 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of tasks in queue 0 task queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140384UL: // (PBF_REG_TASK_CNT_Q4: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_TASK_CNT_Q4 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of tasks in queue 0 task queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140388UL: // (PBF_REG_TASK_CNT_Q5: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_TASK_CNT_Q5 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of tasks in queue 0 task queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14038cUL: // (PBF_REG_TQ_LINES_FREED_CNT_LB_Q: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_LINES_FREED_CNT_LB_Q (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for number of 8 byte lines freed from the \n    LB task queue. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140390UL: // (PBF_REG_TQ_LINES_FREED_CNT_Q0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_LINES_FREED_CNT_Q0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for number of 8 byte lines freed from the \n    task queue 0. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140394UL: // (PBF_REG_TQ_LINES_FREED_CNT_Q1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_LINES_FREED_CNT_Q1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for number of 8 byte lines freed from task\n     queue 1. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140398UL: // (PBF_REG_TQ_LINES_FREED_CNT_Q2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_LINES_FREED_CNT_Q2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for number of 8 byte lines freed from task\n     queue 2. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14039cUL: // (PBF_REG_TQ_LINES_FREED_CNT_Q3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_LINES_FREED_CNT_Q3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for number of 8 byte lines freed from task\n     queue 3. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403a0UL: // (PBF_REG_TQ_LINES_FREED_CNT_Q4: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_LINES_FREED_CNT_Q4 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for number of 8 byte lines freed from task\n     queue 4. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403a4UL: // (PBF_REG_TQ_LINES_FREED_CNT_Q5: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_TQ_LINES_FREED_CNT_Q5 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for number of 8 byte lines freed from task\n     queue 5. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403a8UL: // (PBF_REG_TQ_OCCUPANCY_LB_Q: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nPBF_REG_TQ_OCCUPANCY_LB_Q (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Number of 8 bytes lines occupied in the task queue of the\n     LB queue. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403acUL: // (PBF_REG_TQ_OCCUPANCY_Q0: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nPBF_REG_TQ_OCCUPANCY_Q0 (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Number of 8 bytes lines occupied in the task queue of queue\n     0. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403b0UL: // (PBF_REG_TQ_OCCUPANCY_Q1: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nPBF_REG_TQ_OCCUPANCY_Q1 (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Number of 8 bytes lines occupied in the task queue of queue\n     1. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403b4UL: // (PBF_REG_TQ_OCCUPANCY_Q2: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nPBF_REG_TQ_OCCUPANCY_Q2 (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Number of 8 bytes lines occupied in the task queue of queue\n     2. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403b8UL: // (PBF_REG_TQ_OCCUPANCY_Q3: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nPBF_REG_TQ_OCCUPANCY_Q3 (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Number of 8 bytes lines occupied in the task queue of queue\n     3. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403bcUL: // (PBF_REG_TQ_OCCUPANCY_Q4: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nPBF_REG_TQ_OCCUPANCY_Q4 (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Number of 8 bytes lines occupied in the task queue of queue\n     4. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403c0UL: // (PBF_REG_TQ_OCCUPANCY_Q5: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nPBF_REG_TQ_OCCUPANCY_Q5 (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Number of 8 bytes lines occupied in the task queue of queue\n     5. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403d0UL: // (PBF_REG_EVEN_TXP_MEM_LB_Q: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_EVEN_TXP_MEM_LB_Q (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403e0UL: // (PBF_REG_EVEN_TXP_MEM_Q0: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_EVEN_TXP_MEM_Q0 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403f0UL: // (PBF_REG_EVEN_TXP_MEM_Q1: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_EVEN_TXP_MEM_Q1 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140400UL: // (PBF_REG_EVEN_TXP_MEM_Q2: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_EVEN_TXP_MEM_Q2 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140410UL: // (PBF_REG_EVEN_TXP_MEM_Q3: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_EVEN_TXP_MEM_Q3 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140420UL: // (PBF_REG_EVEN_TXP_MEM_Q4: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_EVEN_TXP_MEM_Q4 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140430UL: // (PBF_REG_EVEN_TXP_MEM_Q5: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_EVEN_TXP_MEM_Q5 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140440UL: // (PBF_REG_ODD_TXP_MEM_LB_Q: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_ODD_TXP_MEM_LB_Q (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140450UL: // (PBF_REG_ODD_TXP_MEM_Q0: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_ODD_TXP_MEM_Q0 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140460UL: // (PBF_REG_ODD_TXP_MEM_Q1: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_ODD_TXP_MEM_Q1 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140470UL: // (PBF_REG_ODD_TXP_MEM_Q2: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_ODD_TXP_MEM_Q2 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140480UL: // (PBF_REG_ODD_TXP_MEM_Q3: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_ODD_TXP_MEM_Q3 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140490UL: // (PBF_REG_ODD_TXP_MEM_Q4: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_ODD_TXP_MEM_Q4 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404a0UL: // (PBF_REG_ODD_TXP_MEM_Q5: ACCESS:WB  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_ODD_TXP_MEM_Q5 (ACCESS:WB  DataWidth:0x41 )" \
				"\nDescription: Read & write access to even tx port memory; for test accessibility\n    . Only when the PBF is idle. There are 800 addresses pointing to 65 bits\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404b0UL: // (PBF_REG_TXP_OCCUPANCY_Q0: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TXP_OCCUPANCY_Q0 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Number of 8 bytes lines occupied in the TX port buffer of\n     queue 0. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404b4UL: // (PBF_REG_TXP_OCCUPANCY_Q1: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TXP_OCCUPANCY_Q1 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Number of 8 bytes lines occupied in the TX port buffer of\n     queue 1. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404b8UL: // (PBF_REG_TXP_OCCUPANCY_Q2: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TXP_OCCUPANCY_Q2 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Number of 8 bytes lines occupied in the TX port buffer of\n     queue 2. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404bcUL: // (PBF_REG_TXP_OCCUPANCY_Q3: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TXP_OCCUPANCY_Q3 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Number of 8 bytes lines occupied in the TX port buffer of\n     queue 3. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404c0UL: // (PBF_REG_TXP_OCCUPANCY_Q4: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TXP_OCCUPANCY_Q4 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Number of 8 bytes lines occupied in the TX port buffer of\n     queue 4. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404c4UL: // (PBF_REG_TXP_OCCUPANCY_Q5: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TXP_OCCUPANCY_Q5 (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Number of 8 bytes lines occupied in the TX port buffer of\n     queue 5. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404c8UL: // (PBF_REG_TXP_OCCUPANCY_LB_Q: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_TXP_OCCUPANCY_LB_Q (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Number of 8 bytes lines occupied in the TX port buffer of\n     the LB queue. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404ccUL: // (PBF_REG_TXP_NUM_PACKETS_Q0: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TXP_NUM_PACKETS_Q0 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Number packets in the TX port buffer of queue 0. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404d0UL: // (PBF_REG_TXP_NUM_PACKETS_Q1: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TXP_NUM_PACKETS_Q1 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Number packets in the TX port buffer of queue 1. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404d4UL: // (PBF_REG_TXP_NUM_PACKETS_Q2: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TXP_NUM_PACKETS_Q2 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Number packets in the TX port buffer of queue 2. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404d8UL: // (PBF_REG_TXP_NUM_PACKETS_Q3: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TXP_NUM_PACKETS_Q3 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Number packets in the TX port buffer of queue 3. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404dcUL: // (PBF_REG_TXP_NUM_PACKETS_Q4: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TXP_NUM_PACKETS_Q4 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Number packets in the TX port buffer of queue 4. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404e0UL: // (PBF_REG_TXP_NUM_PACKETS_Q5: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TXP_NUM_PACKETS_Q5 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Number packets in the TX port buffer of queue 5. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404e4UL: // (PBF_REG_TXP_NUM_PACKETS_LB_Q: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TXP_NUM_PACKETS_LB_Q (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Number packets in the TX port buffer of the LB queue. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140004UL: // (PBF_REG_INIT_P0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT_P0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Init bit for port 0. When set the initial\n     credit of port 0 is copied to the credit register. Should be set and \n    then reset after the configuration of the port has ended. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140008UL: // (PBF_REG_INIT_P1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT_P1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Init bit for port 1. When set the initial\n     credit of port 1 is copied to the credit register. Should be set and \n    then reset after the configuration of the port has ended. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14000cUL: // (PBF_REG_INIT_P4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_INIT_P4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Init bit for port 4. When set the initial\n     credit of port 4 is copied to the credit register. Should be set and \n    then reset after the configuration of the port has ended. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140010UL: // (PBF_REG_PORT_MODE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_PORT_MODE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - 0 - the device is in 2 port mode; 1 - \n    4 port mode.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140014UL: // (PBF_REG_P0_PAUSE_ENABLE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_P0_PAUSE_ENABLE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Indication that pause is enabled for \n    port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140018UL: // (PBF_REG_P1_PAUSE_ENABLE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_P1_PAUSE_ENABLE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Indication that pause is enabled for \n    port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14001cUL: // (PBF_REG_P2_PAUSE_ENABLE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_P2_PAUSE_ENABLE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Indication that pause is enabled for \n    port 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140020UL: // (PBF_REG_P3_PAUSE_ENABLE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_P3_PAUSE_ENABLE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Indication that pause is enabled for \n    port 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140024UL: // (PBF_REG_P4_PAUSE_ENABLE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_P4_PAUSE_ENABLE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Indication that pause is enabled for \n    port 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140048UL: // (PBF_REG_DISABLE_WR2TQ_P0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_P0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Disable bit per port 0 for the write \n    interface of the STORM to that port task Q.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14004cUL: // (PBF_REG_DISABLE_WR2TQ_P1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_P1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Disable bit per port 1 for the write \n    interface of the STORM to that port task Q.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140050UL: // (PBF_REG_DISABLE_WR2TQ_P2: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_P2 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Disable bit per port 2 for the write \n    interface of the STORM to that port task Q.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140054UL: // (PBF_REG_DISABLE_WR2TQ_P3: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_P3 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Disable bit per port 3 for the write \n    interface of the STORM to that port task Q.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140058UL: // (PBF_REG_DISABLE_WR2TQ_P4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_WR2TQ_P4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Disable bit per port 4 for the write \n    interface of the STORM to that port task Q.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14005cUL: // (PBF_REG_DISABLE_NEW_TASK_PROC_P0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_P0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Disable processing further tasks from\n     port 0 (after ending the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140060UL: // (PBF_REG_DISABLE_NEW_TASK_PROC_P1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_P1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Disable processing further tasks from\n     port 1 (after ending the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140064UL: // (PBF_REG_DISABLE_NEW_TASK_PROC_P2: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_P2 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Disable processing further tasks from\n     port 2 (after ending the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140068UL: // (PBF_REG_DISABLE_NEW_TASK_PROC_P3: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_P3 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Disable processing further tasks from\n     port 3 (after ending the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14006cUL: // (PBF_REG_DISABLE_NEW_TASK_PROC_P4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_DISABLE_NEW_TASK_PROC_P4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Disable processing further tasks from\n     port 4 (after ending the current task in process).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140074UL: // (PBF_REG_TQ_P0_NUM_LINES: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TQ_P0_NUM_LINES (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Number of 16 byte lines in the task Q\n     reserved for port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140078UL: // (PBF_REG_TQ_P1_NUM_LINES: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TQ_P1_NUM_LINES (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Number of 16 byte lines in the task Q\n     reserved for port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14007cUL: // (PBF_REG_TQ_P2_NUM_LINES: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TQ_P2_NUM_LINES (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Number of 16 byte lines in the task Q\n     reserved for port 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140080UL: // (PBF_REG_TQ_P3_NUM_LINES: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TQ_P3_NUM_LINES (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Number of 16 byte lines in the task Q\n     reserved for port 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140084UL: // (PBF_REG_TQ_P4_NUM_LINES: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_TQ_P4_NUM_LINES (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Number of 16 byte lines in the task Q\n     reserved for port 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140088UL: // (PBF_REG_XSDM_ADDR_P0: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_P0 (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Removed for E3 B0 - Xsdm hi address when sending a task for\n     port 0. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14008cUL: // (PBF_REG_XSDM_ADDR_P1: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_P1 (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Removed for E3 B0 - Xsdm hi address when sending a task for\n     port 0. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140090UL: // (PBF_REG_XSDM_ADDR_P2: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_P2 (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Removed for E3 B0 - Xsdm hi address when sending a task for\n     port 0. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140094UL: // (PBF_REG_XSDM_ADDR_P3: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_P3 (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Removed for E3 B0 - Xsdm hi address when sending a task for\n     port 0. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140098UL: // (PBF_REG_XSDM_ADDR_P4: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_XSDM_ADDR_P4 (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Removed for E3 B0 - Xsdm hi address when sending a task for\n     port 0. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400a0UL: // (PBF_REG_P0_TQ_THRSH: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_P0_TQ_THRSH (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Removed for E3 B0 - Almost full threshold for port 0 in task\n     Q in 16 byte lines. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400a4UL: // (PBF_REG_P1_TQ_THRSH: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_P1_TQ_THRSH (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Removed for E3 B0 - Almost full threshold for port 1 in task\n     Q in 16 byte lines. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400a8UL: // (PBF_REG_P2_TQ_THRSH: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_P2_TQ_THRSH (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Removed for E3 B0 - Almost full threshold for port 2 in task\n     Q in 16 byte lines. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400acUL: // (PBF_REG_P3_TQ_THRSH: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_P3_TQ_THRSH (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Removed for E3 B0 - Almost full threshold for port 3 in task\n     Q in 16 byte lines. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400b0UL: // (PBF_REG_P4_TQ_THRSH: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_P4_TQ_THRSH (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Removed for E3 B0 - Almost full threshold for port 4 in task\n     Q in 16 byte lines. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400d0UL: // (PBF_REG_P0_INIT_CRD: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P0_INIT_CRD (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Initial credit for port 0 in the tx port\n     buffers in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400d4UL: // (PBF_REG_P1_INIT_CRD: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P1_INIT_CRD (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Initial credit for port 0 in the tx port\n     buffers in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400d8UL: // (PBF_REG_P2_INIT_CRD: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P2_INIT_CRD (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Initial credit for port 0 in the tx port\n     buffers in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400dcUL: // (PBF_REG_P3_INIT_CRD: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P3_INIT_CRD (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Initial credit for port 0 in the tx port\n     buffers in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400e0UL: // (PBF_REG_P4_INIT_CRD: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P4_INIT_CRD (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Initial credit for port 0 in the tx port\n     buffers in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400e4UL: // (PBF_REG_P0_ARB_THRSH: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P0_ARB_THRSH (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Port 0 threshold used by arbiter in 16\n     byte lines used when pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400e8UL: // (PBF_REG_P1_ARB_THRSH: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P1_ARB_THRSH (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Port 1 threshold used by arbiter in 16\n     byte lines used when pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400ecUL: // (PBF_REG_P2_ARB_THRSH: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P2_ARB_THRSH (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Port 2 threshold used by arbiter in 16\n     byte lines used when pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400f0UL: // (PBF_REG_P3_ARB_THRSH: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P3_ARB_THRSH (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Port 3 threshold used by arbiter in 16\n     byte lines used when pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1400f4UL: // (PBF_REG_P4_ARB_THRSH: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P4_ARB_THRSH (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Port 04threshold used by arbiter in 16\n     byte lines used when pause not suppoterd.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140100UL: // (PBF_REG_P0_WEIGHT: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_P0_WEIGHT (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Removed for E3 B0 - Weight of port 0 in the arbitration.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140104UL: // (PBF_REG_P1_WEIGHT: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_P1_WEIGHT (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Removed for E3 B0 - Weight of port 1 in the arbitration.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140108UL: // (PBF_REG_P2_WEIGHT: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_P2_WEIGHT (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Removed for E3 B0 - Weight of port 2 in the arbitration.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14010cUL: // (PBF_REG_P3_WEIGHT: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_P3_WEIGHT (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Removed for E3 B0 -Weight of port 3 in the arbitration.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140110UL: // (PBF_REG_P4_WEIGHT: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPBF_REG_P4_WEIGHT (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Removed for E3 B0 - Weight of port 4 in the arbitration.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140114UL: // (PBF_REG_P0_TXP_THRSH: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_P0_TXP_THRSH (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Removed for E3 B0 - Tx port 0 almost full threshold in 16\n     byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140118UL: // (PBF_REG_P1_TXP_THRSH: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_P1_TXP_THRSH (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Removed for E3 B0 - Tx port 1 almost full threshold in 16\n     byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14011cUL: // (PBF_REG_P2_TXP_THRSH: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_P2_TXP_THRSH (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Removed for E3 B0 - Tx port 2 almost full threshold in 16\n     byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140120UL: // (PBF_REG_P3_TXP_THRSH: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_P3_TXP_THRSH (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Removed for E3 B0 - Tx port 3 almost full threshold in 16\n     byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140124UL: // (PBF_REG_P4_TXP_THRSH: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPBF_REG_P4_TXP_THRSH (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Removed for E3 B0 - Tx port 4 almost full threshold in 16\n     byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140144UL: // (PBF_REG_NUM_SENT_BYTES_P0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_BYTES_P0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Removed for E3 B0 - Number of bytes transmitted on port 0\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140148UL: // (PBF_REG_NUM_SENT_BYTES_P1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_BYTES_P1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Removed for E3 B0 - Number of bytes transmitted on port 1\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14014cUL: // (PBF_REG_NUM_SENT_BYTES_P4: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_BYTES_P4 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Removed for E3 B0 - Number of bytes transmitted on port 4\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140150UL: // (PBF_REG_NUM_SENT_PKTS_P0: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_PKTS_P0 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: Removed for E3 B0 - Number of packets transmitted on port\n     0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140154UL: // (PBF_REG_NUM_SENT_PKTS_P1: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_PKTS_P1 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: Removed for E3 B0 - Number of packets transmitted on port\n     1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140158UL: // (PBF_REG_NUM_SENT_PKTS_P4: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_SENT_PKTS_P4 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: Removed for E3 B0 - Number of packets transmitted on port\n     4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14015cUL: // (PBF_REG_NUM_ERR_SENT_PKTS_P0: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_ERR_SENT_PKTS_P0 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Removed for E3 B0 - Number of packets with error transmitted\n     on port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140160UL: // (PBF_REG_NUM_ERR_SENT_PKTS_P1: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_ERR_SENT_PKTS_P1 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Removed for E3 B0 - Number of packets with error transmitted\n     on port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140164UL: // (PBF_REG_NUM_ERR_SENT_PKTS_P4: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_ERR_SENT_PKTS_P4 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Removed for E3 B0 - Number of packets with error transmitted\n     on port 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140188UL: // (PBF_REG_CMDS_RCVD_ON_P0: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CMDS_RCVD_ON_P0 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: Removed for E3 B0 - Number of commands received on port 0\n     from STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14018cUL: // (PBF_REG_CMDS_RCVD_ON_P1: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CMDS_RCVD_ON_P1 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: Removed for E3 B0 - Number of commands received on port 1\n     from STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140190UL: // (PBF_REG_CMDS_RCVD_ON_P4: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPBF_REG_CMDS_RCVD_ON_P4 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: Removed for E3 B0 - Number of commands received on port 4\n     from STORM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140200UL: // (PBF_REG_P0_CREDIT: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P0_CREDIT (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Current credit for port 0 in the tx port\n     buffers in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140204UL: // (PBF_REG_P0_TASK_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_P0_TASK_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Removed for E3 B0 - Number of tasks in port 0 task queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140208UL: // (PBF_REG_P1_CREDIT: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P1_CREDIT (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Current credit for port 1 in the tx port\n     buffers in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14020cUL: // (PBF_REG_P1_TASK_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_P1_TASK_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Removed for E3 B0 - Number of tasks in port 1 task queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140210UL: // (PBF_REG_P4_CREDIT: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPBF_REG_P4_CREDIT (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Removed for E3 B0 - Current credit for port 4 in the tx port\n     buffers in 16 byte lines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140214UL: // (PBF_REG_P4_TASK_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPBF_REG_P4_TASK_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Removed for E3 B0 - Number of tasks in port 4 task queue.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140220UL: // (PBF_REG_P0_EVEN_TXP_MEM: ACCESS:R  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_P0_EVEN_TXP_MEM (ACCESS:R  DataWidth:0x41 )" \
				"\nDescription: Removed for E3 B0 - Read & write access to even tx port memory\n    ; for test accessibility. Only when the PBF is idle. There are 800 addresses\n     pointing to 65 bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140240UL: // (PBF_REG_P0_ODD_TXP_MEM: ACCESS:R  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_P0_ODD_TXP_MEM (ACCESS:R  DataWidth:0x41 )" \
				"\nDescription: Removed for E3 B0 - Read & write access to odd tx port 0 \n    memory; for test accessibility. Only when the PBf is idle. There are 800\n     addresses pointing to 65 bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140260UL: // (PBF_REG_P1_EVEN_TXP_MEM: ACCESS:R  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_P1_EVEN_TXP_MEM (ACCESS:R  DataWidth:0x41 )" \
				"\nDescription: Removed for E3 B0 - Read & write access to even tx port 1\n     memory; for test accessibility. Only when the PBF is idle. There are \n    800 addresses pointing to 65 bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140280UL: // (PBF_REG_P1_ODD_TXP_MEM: ACCESS:R  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_P1_ODD_TXP_MEM (ACCESS:R  DataWidth:0x41 )" \
				"\nDescription: Removed for E3 B0 - Read & write access to odd tx port 1 \n    memory; for test accessibility. Only when the PBF is idle. There are 800\n     addresses pointing to 65 bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402a0UL: // (PBF_REG_P4_EVEN_TXP_MEM: ACCESS:R  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_P4_EVEN_TXP_MEM (ACCESS:R  DataWidth:0x41 )" \
				"\nDescription: Removed for E3 B0 - Read & write access to even tx port 4\n     memory; for test accessibility. Only when the PBF is idle. There are \n    672 addresses pointing to 65 bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402c0UL: // (PBF_REG_P4_ODD_TXP_MEM: ACCESS:R  DataWidth:0x41 ) 
			sprintf(disp_str, "\nPBF_REG_P4_ODD_TXP_MEM (ACCESS:R  DataWidth:0x41 )" \
				"\nDescription: Removed for E3 B0 - Read & write access to odd tx port 4 \n    memory; for test accessibility. Only when the PBF is idle. There are 672\n     addresses pointing to 65 bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402e0UL: // (PBF_REG_COS0_WFQ_CREDIT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS0_WFQ_CREDIT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Removed for E3 B0 - current WFQ credit of COS0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402e4UL: // (PBF_REG_COS1_WFQ_CREDIT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_COS1_WFQ_CREDIT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Removed for E3 B0 - current WFQ credit of COS1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402f0UL: // (PBF_REG_P0_TQ_LINES_FREED_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_P0_TQ_LINES_FREED_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Removed for E3 B0 - Cyclic counter for number of 8 byte lines\n     freed from the task queue of port 0. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402f4UL: // (PBF_REG_P1_TQ_LINES_FREED_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_P1_TQ_LINES_FREED_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Removed for E3 B0 - Cyclic counter for number of 8 byte lines\n     freed from the task queue of port 1. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402f8UL: // (PBF_REG_P4_TQ_LINES_FREED_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_P4_TQ_LINES_FREED_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Removed for E3 B0 - Cyclic counter for number of 8 byte lines\n     freed from the task queue of port 4. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402fcUL: // (PBF_REG_P0_TQ_OCCUPANCY: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_P0_TQ_OCCUPANCY (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Number of 8 bytes lines occupied in the task queue of port\n     0. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140300UL: // (PBF_REG_P1_TQ_OCCUPANCY: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_P1_TQ_OCCUPANCY (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Number of 8 bytes lines occupied in the task queue of port\n     1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140304UL: // (PBF_REG_P4_TQ_OCCUPANCY: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPBF_REG_P4_TQ_OCCUPANCY (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Number of 8 bytes lines occupied in the task queue of port\n     4. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140308UL: // (PBF_REG_P0_INTERNAL_CRD_FREED_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_P0_INTERNAL_CRD_FREED_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for the amount credits in 16 bytes lines added\n     for port 0. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x14030cUL: // (PBF_REG_P1_INTERNAL_CRD_FREED_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_P1_INTERNAL_CRD_FREED_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for the amount credits in 16 bytes lines added\n     for port 1. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140310UL: // (PBF_REG_P4_INTERNAL_CRD_FREED_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_P4_INTERNAL_CRD_FREED_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Cyclic counter for the amount credits in 16 bytes lines added\n     for port 4. Reset upon init.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c008UL: // (PBF_REG_PBF_TX_MBIST1_CNTRL_CMD: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPBF_REG_PBF_TX_MBIST1_CNTRL_CMD (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Bit 0 -      bist_override Bit 1 -      mbist_en Bit 2 -      \n    mbist_async_reset Bits 4:3 -  bist_setup[1:0] \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c00cUL: // (PBF_REG_TX_CLK_MBIST1_CNTRL_PBF_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_TX_CLK_MBIST1_CNTRL_PBF_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 -  mbist_done Bit 1 -  mbist_go Bits 31:2 -  mbist_go0\n    [29:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c010UL: // (PBF_REG_TX_CLK_MBIST1_CNTRL_PBF_STATUS_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_TX_CLK_MBIST1_CNTRL_PBF_STATUS_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bits 2:0 - mbist_go0[32:30]; Bits 31:3 - mbist_go1[28:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c014UL: // (PBF_REG_XSEM_PD_CLK_ST_MBIST1_CNTRL_PBF_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_XSEM_PD_CLK_ST_MBIST1_CNTRL_PBF_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 -  mbist_done ; Bit 1 -  mbist_go ; Bits 8:2 - mbist_go0\n    [6:0] ; Bits 12:9 - mbist_go1[3:0] ; Bits [31:13] - reserved\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c018UL: // (PBF_REG_XSEM_PD_CLK_CAM_ST_MBIST1_CNTRL_PBF_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_XSEM_PD_CLK_CAM_ST_MBIST1_CNTRL_PBF_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 -  mbist_done ; Bit 1 -  mbist_go ; Bit 2 -  mbist_go0\n     ; Bits 31:3 - reserved\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c01cUL: // (PBF_REG_TX_CLK_MBIST1_CNTRL_PBF_STATUS_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPBF_REG_TX_CLK_MBIST1_CNTRL_PBF_STATUS_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bits 1:0 - mbist_go1[30:29] ; Bits 31:2 - reserved\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c040UL: // (PBF_REG_NETWORK_PACKETS_WRR_WEIGHT: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPBF_REG_NETWORK_PACKETS_WRR_WEIGHT (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Removed for E3 B0 - WRR weight of commands destined to the\n     network in the command arbiter\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c04cUL: // (PBF_REG_HIGH_PRIORITY_COS_NUM: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_HIGH_PRIORITY_COS_NUM (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Indicates which COS is conncted to the\n     highest priority in the command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c050UL: // (PBF_REG_ETS_ENABLED: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_ENABLED (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - Indicates that ETS is performed between\n     the COSes in the command arbiter. If reset strict priority w/ anti-starvation\n     will be performed w/o WFQ.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c054UL: // (PBF_REG_COS0_WEIGHT: ACCESS:R  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS0_WEIGHT (ACCESS:R  DataWidth:0x1f )" \
				"\nDescription: Removed for E3 B0 - The weight of COS0 in the ETS command\n     arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c058UL: // (PBF_REG_COS1_WEIGHT: ACCESS:R  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS1_WEIGHT (ACCESS:R  DataWidth:0x1f )" \
				"\nDescription: Removed for E3 B0 - The weight of COS1 in the ETS command\n     arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c05cUL: // (PBF_REG_COS0_UPPER_BOUND: ACCESS:R  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS0_UPPER_BOUND (ACCESS:R  DataWidth:0x1f )" \
				"\nDescription: Removed for E3 B0 -The upper bound of the weight of COS0 \n    in the ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c060UL: // (PBF_REG_COS1_UPPER_BOUND: ACCESS:R  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPBF_REG_COS1_UPPER_BOUND (ACCESS:R  DataWidth:0x1f )" \
				"\nDescription: Removed for E3 B0 -The upper bound of the weight of COS1 \n    in the ETS command arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c064UL: // (PBF_REG_NUM_STRICT_ARB_SLOTS: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPBF_REG_NUM_STRICT_ARB_SLOTS (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Removed for E3 B0 - The number of strict priority arbitration\n     slots between 2 RR arbitration slots. A value of 0 means no strict priority\n     cycles; i.e. the strict-priority w/ anti-starvation arbiter is a RR arbiter\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c068UL: // (PBF_REG_ETS_BURST_MODE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPBF_REG_ETS_BURST_MODE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Removed for E3 B0 - If set; in the command queue arbiter; \n    the round robin arbiters pointer will stay on the winning input; instead\n     of moving to the next one. This allows burstiness. This bit can be set\n     only when ETS is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c128UL: // (PBF_REG_LAST_CID_W_ERROR_P0: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nPBF_REG_LAST_CID_W_ERROR_P0 (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: last CID with either parsing or PCI error which was transmitted\n     with error or dropped on port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c12cUL: // (PBF_REG_LAST_CID_W_ERROR_P1: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nPBF_REG_LAST_CID_W_ERROR_P1 (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: last CID with either parsing or PCI error which was transmitted\n     with error or dropped on port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c158UL: // (PBF_REG_HDR_DUP_QWORDS_P0: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPBF_REG_HDR_DUP_QWORDS_P0 (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: Removed for E3 B0 - The number of QWORDS allocated in the\n     header duplication memory for PBF port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c15cUL: // (PBF_REG_HDR_DUP_QWORDS_P1: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPBF_REG_HDR_DUP_QWORDS_P1 (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: Removed for E3 B0 - The number of QWORDS allocated in the\n     header duplication memory for PBF port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1401e8UL: // (PBF_REG_PBF_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140218UL: // (PBF_REG_PBF_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140230UL: // (PBF_REG_PBF_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140250UL: // (PBF_REG_PBF_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140270UL: // (PBF_REG_PBF_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x140290UL: // (PBF_REG_PBF_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402b0UL: // (PBF_REG_PBF_UNUSED_EMPTY_6: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_6 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1402d0UL: // (PBF_REG_PBF_UNUSED_EMPTY_7: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_7 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1403c4UL: // (PBF_REG_PBF_UNUSED_EMPTY_8: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_8 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1404e8UL: // (PBF_REG_PBF_UNUSED_EMPTY_9: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_9 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x15c3b4UL: // (PBF_REG_PBF_UNUSED_EMPTY_10: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPBF_REG_PBF_UNUSED_EMPTY_10 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XPB + 0UL): // (PB_REG_CONTROL: ACCESS:RW  DataWidth:0xc  Multi Field Register  ) 
			sprintf(disp_str, "\nPB_REG_CONTROL (ACCESS:RW  DataWidth:0xc  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRELAX_TH :"
				, (u32_t)((data & PB_CONTROL_REG_RELAX_TH) >> 11)
				, "\n\tDEBUG_SELECT :"
				, (u32_t)((data & PB_CONTROL_REG_DEBUG_SELECT) >> 7)
				, "\n\tDISABLE_PB :"
				, (u32_t)((data & PB_CONTROL_REG_DISABLE_PB) >> 6)
				, "\n\tEN_INPUTS :"
				, (u32_t)((data & PB_CONTROL_REG_EN_INPUTS) >> 5)
				, "\n\tCRC_COMPARE_DISABLE :"
				, (u32_t)((data & PB_CONTROL_REG_CRC_COMPARE_DISABLE) >> 4)
				, "\n\tEOP_CHECK_DISABLE :"
				, (u32_t)((data & PB_CONTROL_REG_EOP_CHECK_DISABLE) >> 3)
				, "\n\tDONT_PASS_ERROR :"
				, (u32_t)((data & PB_CONTROL_REG_DONT_PASS_ERROR) >> 2)
				, "\n\tDB_IGNORE_ERROR :"
				, (u32_t)((data & PB_CONTROL_REG_DB_IGNORE_ERROR) >> 1)
				, "\n\tBYTE_ORDER_SWITCH :"
				, (u32_t)((data & PB_CONTROL_REG_BYTE_ORDER_SWITCH) >> 0));
			break;
		case (GRCBASE_XPB + 0x4UL): // (PB_REG_THRESHOLDS: ACCESS:RW  DataWidth:0xf  Multi Field Register  ) 
			sprintf(disp_str, "\nPB_REG_THRESHOLDS (ACCESS:RW  DataWidth:0xf  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDB_AF_THRESH :"
				, (u32_t)((data & PB_THRESHOLDS_REG_DB_AF_THRESH) >> 9)
				, "\n\tTQ_AF_THRESH :"
				, (u32_t)((data & PB_THRESHOLDS_REG_TQ_AF_THRESH) >> 0));
			break;
		case (GRCBASE_XPB + 0x8UL): // (PB_REG_L1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPB_REG_L1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of L1 memory; Not used in E65\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XPB + 0xcUL): // (PB_REG_TQ_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPB_REG_TQ_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits of TQ memory (relevant for PPB only)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XPB + 0x10UL): // (PB_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPB_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: For dbgmux usage (debug data that goes from PB to the DBG\n     block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XPB + 0x14UL): // (PB_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPB_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: For dbgmux usage (debug data that goes from PB to the DBG\n     block) - for enabling bytes in the selected line (after the select and\n     before the shift)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XPB + 0x18UL): // (PB_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPB_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: For dbgmux usage (debug data that goes from PB to the DBG\n     block) - for circular right shifting of the selected line (after the \n    enabling)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XPB + 0x1cUL): // (PB_REG_PB_INT_STS: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPB_REG_PB_INT_STS (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEOP_ERROR :"
				, (u32_t)((data & PB_PB_INT_STS_REG_EOP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PB_PB_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_XPB + 0x20UL): // (PB_REG_PB_INT_STS_CLR: ACCESS:RC  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPB_REG_PB_INT_STS_CLR (ACCESS:RC  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEOP_ERROR :"
				, (u32_t)((data & PB_PB_INT_STS_CLR_REG_EOP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PB_PB_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_XPB + 0x24UL): // (PB_REG_PB_INT_STS_WR: ACCESS:WR  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPB_REG_PB_INT_STS_WR (ACCESS:WR  DataWidth:0x2 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEOP_ERROR :"
				, (u32_t)((data & PB_PB_INT_STS_WR_REG_EOP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PB_PB_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_XPB + 0x28UL): // (PB_REG_PB_INT_MASK: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPB_REG_PB_INT_MASK (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEOP_ERROR :"
				, (u32_t)((data & PB_PB_INT_MASK_REG_EOP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PB_PB_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case (GRCBASE_XPB + 0x2cUL): // (PB_REG_PB_PRTY_STS: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPB_REG_PB_PRTY_STS (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tL1 :"
				, (u32_t)((data & PB_PB_PRTY_STS_REG_L1) >> 3)
				, "\n\tTQ :"
				, (u32_t)((data & PB_PB_PRTY_STS_REG_TQ) >> 2)
				, "\n\tDB :"
				, (u32_t)((data & PB_PB_PRTY_STS_REG_DB) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PB_PB_PRTY_STS_REG_PARITY) >> 0));
			break;
		case (GRCBASE_XPB + 0x30UL): // (PB_REG_PB_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPB_REG_PB_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x4 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tL1 :"
				, (u32_t)((data & PB_PB_PRTY_STS_CLR_REG_L1) >> 3)
				, "\n\tTQ :"
				, (u32_t)((data & PB_PB_PRTY_STS_CLR_REG_TQ) >> 2)
				, "\n\tDB :"
				, (u32_t)((data & PB_PB_PRTY_STS_CLR_REG_DB) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PB_PB_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case (GRCBASE_XPB + 0x34UL): // (PB_REG_PB_PRTY_STS_WR: ACCESS:WR  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPB_REG_PB_PRTY_STS_WR (ACCESS:WR  DataWidth:0x4 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tL1 :"
				, (u32_t)((data & PB_PB_PRTY_STS_WR_REG_L1) >> 3)
				, "\n\tTQ :"
				, (u32_t)((data & PB_PB_PRTY_STS_WR_REG_TQ) >> 2)
				, "\n\tDB :"
				, (u32_t)((data & PB_PB_PRTY_STS_WR_REG_DB) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PB_PB_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case (GRCBASE_XPB + 0x38UL): // (PB_REG_PB_PRTY_MASK: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPB_REG_PB_PRTY_MASK (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tL1 :"
				, (u32_t)((data & PB_PB_PRTY_MASK_REG_L1) >> 3)
				, "\n\tTQ :"
				, (u32_t)((data & PB_PB_PRTY_MASK_REG_TQ) >> 2)
				, "\n\tDB :"
				, (u32_t)((data & PB_PB_PRTY_MASK_REG_DB) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PB_PB_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case (GRCBASE_XPB + 0x3cUL): // (PB_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPB_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: for future eco\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XPB + 0x40UL): // (PB_REG_DB_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPB_REG_DB_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of DB memory (relevant for PPB only)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XPB + 0x800UL): // (PB_REG_L1: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nPB_REG_L1 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: L1 CRC memory access\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XPB + 0x44UL): // (PB_REG_PB_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPB_REG_PB_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9000UL: // (PGLUE_B_REG_CFG_SPACE_A_ADDRESS: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CFG_SPACE_A_ADDRESS (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Address[12:7] in PCI configuration space of the first register\n     on which config space A attention is generated. Note that this register\n     is in 128-byte units.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9004UL: // (PGLUE_B_REG_CFG_SPACE_A_ENABLE: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CFG_SPACE_A_ENABLE (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indicates which of the 32 registers starting in address cfg_space_a_address\n     generates an attention. If bit N is set - a CSSNOOP cycle with address\n     {cfg_space_a_address; 7b0}+4*N will generate a config space A attention\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9008UL: // (PGLUE_B_REG_CFG_SPACE_B_ADDRESS: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CFG_SPACE_B_ADDRESS (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Address[12:7] in PCI configuration space of the first register\n     on which config space B attention is generated. Note that this register\n     is in 128-byte units.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x900cUL: // (PGLUE_B_REG_CFG_SPACE_B_ENABLE: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CFG_SPACE_B_ENABLE (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indicates which of the 32 registers starting in address cfg_space_b_address\n     generates an attention. If bit N is set - a CSSNOOP cycle with address\n     {cfg_space_b_address; 7b0}+4*N will generate a config space B attention\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9010UL: // (PGLUE_B_REG_CFG_SPACE_A_REQUEST: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CFG_SPACE_A_REQUEST (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Config space A attention dirty bits. Each bit indicates that\n     the corresponding PF generates config space A attention. Set by PXP. \n    Reset by MCP writing 1 to icfg_space_a_request_clr. Note: register contains\n     bits from both paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9014UL: // (PGLUE_B_REG_CFG_SPACE_B_REQUEST: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CFG_SPACE_B_REQUEST (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Config space B attention dirty bits. Each bit indicates that\n     the corresponding PF generates config space B attention. Set by PXP. \n    Reset by MCP writing 1 to icfg_space_b_request_clr. Note: register contains\n     bits from both paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9018UL: // (PGLUE_B_REG_FLR_REQUEST_VF_31_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_FLR_REQUEST_VF_31_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: FLR request attention dirty bits for VFs 0 to 31. Each bit\n     indicates that the FLR register of the corresponding VF was set. Set \n    by PXP. Reset by MCP writing 1 to flr_request_vf_31_0_clr.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x901cUL: // (PGLUE_B_REG_FLR_REQUEST_VF_63_32: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_FLR_REQUEST_VF_63_32 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: FLR request attention dirty bits for VFs 32 to 63. Each bit\n     indicates that the FLR register of the corresponding VF was set. Set \n    by PXP. Reset by MCP writing 1 to flr_request_vf_63_32_clr.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9020UL: // (PGLUE_B_REG_FLR_REQUEST_VF_95_64: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_FLR_REQUEST_VF_95_64 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: FLR request attention dirty bits for VFs 64 to 95. Each bit\n     indicates that the FLR register of the corresponding VF was set. Set \n    by PXP. Reset by MCP writing 1 to flr_request_vf_95_64_clr.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9024UL: // (PGLUE_B_REG_FLR_REQUEST_VF_127_96: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_FLR_REQUEST_VF_127_96 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: FLR request attention dirty bits for VFs 96 to 127. Each \n    bit indicates that the FLR register of the corresponding VF was set. Set\n     by PXP. Reset by MCP writing 1 to flr_request_vf_127_96_clr.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9028UL: // (PGLUE_B_REG_FLR_REQUEST_PF_7_0: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_FLR_REQUEST_PF_7_0 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: FLR request attention dirty bits for PFs 0 to 7. Each bit\n     indicates that the FLR register of the corresponding PF was set. Set \n    by PXP. Reset by MCP writing 1 to flr_request_pf_7_0_clr. Note: register\n     contains bits from both paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x902cUL: // (PGLUE_B_REG_DISABLE_FLR_SRIOV_DISABLED: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DISABLE_FLR_SRIOV_DISABLED (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDISABLE_SRIOV_DISABLED_REQUEST :"
				, (u32_t)((data & PGLUE_B_DISABLE_FLR_SRIOV_DISABLED_REG_DISABLE_SRIOV_DISABLED_REQUEST) >> 1)
				, "\n\tDISABLE_FLR_REQUEST :"
				, (u32_t)((data & PGLUE_B_DISABLE_FLR_SRIOV_DISABLED_REG_DISABLE_FLR_REQUEST) >> 0));
			break;
		case 0x9030UL: // (PGLUE_B_REG_SR_IOV_DISABLED_REQUEST: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SR_IOV_DISABLED_REQUEST (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: SR IOV disabled attention dirty bits. Each bit indicates \n    that the VF enable register of the corresponding PF is written to 0 and\n     was previously 1. Set by PXP. Reset by MCP writing 1 to sr_iov_disabled_request_clr\n    . Note: register contains bits from both paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9034UL: // (PGLUE_B_REG_SHADOW_BME_VF_31_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_BME_VF_31_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Shadow BME register for VFs 0 to 31. Each bit indicates if\n     the corresponding VF is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9038UL: // (PGLUE_B_REG_SHADOW_BME_VF_63_32: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_BME_VF_63_32 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Shadow BME register for VFs 32 to 63. Each bit indicates \n    if the corresponding VF is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x903cUL: // (PGLUE_B_REG_SHADOW_BME_VF_95_64: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_BME_VF_95_64 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Shadow BME register for VFs 64 to 95. Each bit indicates \n    if the corresponding VF is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9040UL: // (PGLUE_B_REG_SHADOW_BME_VF_127_96: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_BME_VF_127_96 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Shadow BME register for VFs 96 to 127. Each bit indicates\n     if the corresponding VF is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9044UL: // (PGLUE_B_REG_SHADOW_BME_PF_7_0: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_BME_PF_7_0 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Shadow BME register for PFs 0 to 7. Each bit indicates if\n     the corresponding PF is enabled. Note: register contains bits from both\n     paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9048UL: // (PGLUE_B_REG_SHADOW_ATS_ENABLE_VF_31_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_ENABLE_VF_31_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Shadow ats_enable register for VFs 0 to 31. Each bit indicates\n     if ATS for the corresponding VF is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x904cUL: // (PGLUE_B_REG_SHADOW_ATS_ENABLE_VF_63_32: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_ENABLE_VF_63_32 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Shadow ats_enable register for VFs 32 to 63. Each bit indicates\n     if ATS for the corresponding VF is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9050UL: // (PGLUE_B_REG_SHADOW_ATS_ENABLE_VF_95_64: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_ENABLE_VF_95_64 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Shadow ats_enable register for VFs 64 to 95. Each bit indicates\n     if ATS for the corresponding VF is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9054UL: // (PGLUE_B_REG_SHADOW_ATS_ENABLE_VF_127_96: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_ENABLE_VF_127_96 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Shadow ats_enable register for VFs 96 to 127. Each bit indicates\n     if ATS for the corresponding VF is enabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9058UL: // (PGLUE_B_REG_SHADOW_ATS_ENABLE_PF_7_0: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_ENABLE_PF_7_0 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Shadow ats_enable register for PFs 0 to 7. Each bit indicates\n     if ATS for the corresponding PF is enabled. Note: register contains bits\n     from both paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x905cUL: // (PGLUE_B_REG_SHADOW_VF_ENABLE_PF_7_0: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_VF_ENABLE_PF_7_0 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Shadow vf_enable register for PFs 0 to 7. Each bit indicates\n     if SR-IOV for the corresponding PF is enabled. Note: register contains\n     bits from both paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9060UL: // (PGLUE_B_REG_SHADOW_IDO_BITS: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_IDO_BITS (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Shadow ido bits register for PFs 0 to 7. [7:0] : Each bit\n     indicates if IDO_REQ_ENABLE bit for the corresponding PF is set. [15:\n    8] : Each bit indicates if IDO_CPL_ENABLE bit for the corresponding PF\n     is set. Note: register contains bits from both paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9064UL: // (PGLUE_B_REG_DISABLE_ATS_EN_CLEARING: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DISABLE_ATS_EN_CLEARING (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only: PGLUE automatically clears ATC enable for a function\n     if a TCPL arrived for that function with Unsupported Request error. Setting\n     this register to 1 disables this automatic clearing.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9068UL: // (PGLUE_B_REG_INCORRECT_RCV_DETAILS: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_INCORRECT_RCV_DETAILS (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Each bit indicates an incorrect behavior in user RX interface\n    . Bit 0 - Target memory read arrived with a correctable error. Bit 1 - \n    Target memory read arrived with an uncorrectable error. Bit 2 - Configuration\n     RW arrived with a correctable error. Bit 3 - Configuration RW arrived\n     with an uncorrectable error. Bit 4 - Completion with Configuration Request\n     Retry Status. Bit 5 - Expansion ROM access received with a write request\n    . Bit 6 - Completion with pcie_rx_err of 0000; CMPL_STATUS of non-zero\n    ; and pcie_rx_last not asserted. Bit 7 - Completion with pcie_rx_err of\n     1010; and pcie_rx_last not asserted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x906cUL: // (PGLUE_B_REG_WAS_ERROR_VF_31_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_WAS_ERROR_VF_31_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Was_error indication dirty bits for VFs 0 to 31.  Each bit\n     indicates that there was a completion with uncorrectable error for the\n     corresponding VF. Set by PXP. Reset by MCP writing 1 to was_error_vf_31_0_clr\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9070UL: // (PGLUE_B_REG_WAS_ERROR_VF_63_32: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_WAS_ERROR_VF_63_32 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Was_error indication dirty bits for VFs 32 to 63.  Each bit\n     indicates that there was a completion with uncorrectable error for the\n     corresponding VF. Set by PXP. Reset by MCP writing 1 to was_error_vf_63_32_clr\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9074UL: // (PGLUE_B_REG_WAS_ERROR_VF_95_64: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_WAS_ERROR_VF_95_64 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Was_error indication dirty bits for VFs 64 to 95.  Each bit\n     indicates that there was a completion with uncorrectable error for the\n     corresponding VF. Set by PXP. Reset by MCP writing 1 to was_error_vf_95_64_clr\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9078UL: // (PGLUE_B_REG_WAS_ERROR_VF_127_96: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_WAS_ERROR_VF_127_96 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Was_error indication dirty bits for VFs 96 to 127.  Each \n    bit indicates that there was a completion with uncorrectable error for\n     the corresponding VF. Set by PXP. Reset by MCP writing 1 to was_error_vf_127_96_clr\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x907cUL: // (PGLUE_B_REG_WAS_ERROR_PF_7_0: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_WAS_ERROR_PF_7_0 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Was_error indication dirty bits for PFs 0 to 7.  Each bit\n     indicates that there was a completion with uncorrectable error for the\n     corresponding PF. Set by PXP. Reset by MCP writing 1 to was_error_pf_7_0_clr\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9080UL: // (PGLUE_B_REG_RX_ERR_DETAILS: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nPGLUE_B_REG_RX_ERR_DETAILS (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Details of first request received with error. [2:0] - PFID\n    . [3] - VF_VALID. [9:4] - VFID. [11:10] - Error Code - 0 - Indicates Completion\n     Timeout of a User Tx non-posted request. 1 - unsupported request. 2 - \n    completer abort. 3 - Illegal value for this field. [12] valid - indicates\n     if there was a completion error since the last time this register was\n     cleared. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9084UL: // (PGLUE_B_REG_RX_TCPL_ERR_DETAILS: ACCESS:R  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_RX_TCPL_ERR_DETAILS (ACCESS:R  DataWidth:0x12 )" \
				"\nDescription: Details of first ATS Translation Completion request received\n     with error. [2:0] - PFID. [3] - VF_VALID. [9:4] - VFID. [11:10] - Error\n     Code - 0 - Indicates Completion Timeout of a User Tx non-posted request\n    . 1 - unsupported request. 2 - completer abort. 3 - Illegal value for \n    this field. [16:12] - ATC OTB EntryID. [17] valid - indicates if there\n     was a completion error since the last time this register was cleared. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9088UL: // (PGLUE_B_REG_TX_ERR_WR_ADD_31_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TX_ERR_WR_ADD_31_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Address [31:0] of first write request not submitted due to\n     error\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x908cUL: // (PGLUE_B_REG_TX_ERR_WR_ADD_63_32: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TX_ERR_WR_ADD_63_32 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Address [63:32] of first write request not submitted due \n    to error\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9090UL: // (PGLUE_B_REG_TX_ERR_WR_DETAILS: ACCESS:R  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TX_ERR_WR_DETAILS (ACCESS:R  DataWidth:0x1f )" \
				"\nDescription: Details of first write request not submitted due to error\n    . [4:0] VQID. [20:8] - Length in bytes. [23:21] - PFID. [24] - VF_VALID\n    . [30:25] - VFID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9094UL: // (PGLUE_B_REG_TX_ERR_WR_DETAILS2: ACCESS:R  DataWidth:0x1a ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TX_ERR_WR_DETAILS2 (ACCESS:R  DataWidth:0x1a )" \
				"\nDescription: Details of first write request not submitted due to error\n    . [15:0] Request ID. [19:16] client ID. [20] - last SR. [24:21] - Error\n     type - [21] - Indicates was_error was set; [22] - Indicates BME was cleared\n    ; [23] - Indicates FID_enable was cleared; [24] - Indicates VF with parent\n     PF FLR_request or IOV_disable_request dirty bit is set. [25] valid - \n    indicates if there was a request not submitted due to error since the \n    last time this register was cleared.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9098UL: // (PGLUE_B_REG_TX_ERR_RD_ADD_31_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TX_ERR_RD_ADD_31_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Address [31:0] of first read request not submitted due to\n     error\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x909cUL: // (PGLUE_B_REG_TX_ERR_RD_ADD_63_32: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TX_ERR_RD_ADD_63_32 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Address [63:32] of first read request not submitted due to\n     error\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90a0UL: // (PGLUE_B_REG_TX_ERR_RD_DETAILS: ACCESS:R  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TX_ERR_RD_DETAILS (ACCESS:R  DataWidth:0x1f )" \
				"\nDescription: Details of first read request not submitted due to error. [\n    4:0] VQID. [5] TREQ. 1 - Indicates the request is a Translation Request\n    . [20:8] - Length in bytes. [23:21] - PFID. [24] - VF_VALID. [30:25] - \n    VFID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90a4UL: // (PGLUE_B_REG_TX_ERR_RD_DETAILS2: ACCESS:R  DataWidth:0x1a ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TX_ERR_RD_DETAILS2 (ACCESS:R  DataWidth:0x1a )" \
				"\nDescription: Details of first read request not submitted due to error. [\n    15:0] Request ID. [19:16] client ID. [20] - last SR. [24:21] - Error type\n     - [21] - Indicates was_error was set; [22] - Indicates BME was cleared\n    ; [23] - Indicates FID_enable was cleared; [24] - Indicates VF with parent\n     PF FLR_request or IOV_disable_request dirty bit is set. [25] valid - \n    indicates if there was a request not submitted due to error since the \n    last time this register was cleared.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90a8UL: // (PGLUE_B_REG_TX_ERR_WR_DETAILS_ICPL: ACCESS:R  DataWidth:0x15 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TX_ERR_WR_DETAILS_ICPL (ACCESS:R  DataWidth:0x15 )" \
				"\nDescription: Details of first Invalidation Completion message submitted\n     during a TX error condition. [2:0] - PFID. [3] - VF_VALID. [9:4] - VFID\n    . [14:10] - ITAG Index.  [19:16] - Error type - [16] - Indicates was_error\n     was set; [17] - Indicates BME was cleared; [18] - Indicates FID_enable\n     was cleared; [19] - Indicates VF with parent PF FLR_request or IOV_disable_request\n     dirty bit is set. [20] valid - indicates if there was an Invalidation\n     Completion message submitted during a TX error condition since the last\n     time this register was cleared.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90acUL: // (PGLUE_B_REG_TX_ERR_E15_MODE: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TX_ERR_E15_MODE (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTX_ERR_E15_MODE_VF :"
				, (u32_t)((data & PGLUE_B_TX_ERR_E15_MODE_REG_TX_ERR_E15_MODE_VF) >> 1)
				, "\n\tTX_ERR_E15_MODE_PF :"
				, (u32_t)((data & PGLUE_B_TX_ERR_E15_MODE_REG_TX_ERR_E15_MODE_PF) >> 0));
			break;
		case 0x90b0UL: // (PGLUE_B_REG_INTERNAL_VFID_ENABLE_31_0_VALUE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_INTERNAL_VFID_ENABLE_31_0_VALUE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: A global view of internal_vfid_enable register for VFs 0 \n    to 31.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90b4UL: // (PGLUE_B_REG_INTERNAL_VFID_ENABLE_63_32_VALUE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_INTERNAL_VFID_ENABLE_63_32_VALUE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: A global view of internal_vfid_enable register for VFs 32\n     to 63.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90b8UL: // (PGLUE_B_REG_INTERNAL_VFID_ENABLE_95_64_VALUE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_INTERNAL_VFID_ENABLE_95_64_VALUE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: A global view of internal_vfid_enable register for VFs 64\n     to 95.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90bcUL: // (PGLUE_B_REG_INTERNAL_VFID_ENABLE_127_96_VALUE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_INTERNAL_VFID_ENABLE_127_96_VALUE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: A global view of internal_vfid_enable register for VFs 96\n     to 127.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90c0UL: // (PGLUE_B_REG_INTERNAL_PFID_ENABLE_VALUE: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_INTERNAL_PFID_ENABLE_VALUE (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: A global view of internal_pfid_enable registers. Bits [7:\n    0] - internal_pfid_enable_master; Bits [15:8] = internal_pfid_enable_target_write\n    ; Bits [23:16] = internal_pfid_enable_target_read\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90c4UL: // (PGLUE_B_REG_TSDM_START_OFFSET_A: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_START_OFFSET_A (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Start offset of TSDM zone A (queue zone) in the internal \n    RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90c8UL: // (PGLUE_B_REG_TSDM_OFFSET_MASK_A: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_OFFSET_MASK_A (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Offset mask of TSDM zone A (queue zone) in the internal RAM\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90ccUL: // (PGLUE_B_REG_TSDM_START_OFFSET_B: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_START_OFFSET_B (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Start offset of TSDM zone B (legacy zone) in the internal\n     RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90d0UL: // (PGLUE_B_REG_TSDM_OFFSET_MASK_B: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_OFFSET_MASK_B (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: Offset mask of TSDM zone B (legacy zone) in the internal \n    RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90d4UL: // (PGLUE_B_REG_TSDM_VF_SHIFT_B: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_VF_SHIFT_B (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: VF Shift of TSDM zone B (legacy zone) in the internal RAM\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90d8UL: // (PGLUE_B_REG_USDM_START_OFFSET_A: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_START_OFFSET_A (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Start offset of USDM zone A (queue zone) in the internal \n    RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90dcUL: // (PGLUE_B_REG_USDM_OFFSET_MASK_A: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_OFFSET_MASK_A (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Offset mask of USDM zone A (queue zone) in the internal RAM\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90e0UL: // (PGLUE_B_REG_USDM_START_OFFSET_B: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_START_OFFSET_B (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Start offset of USDM zone B (legacy zone) in the internal\n     RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90e4UL: // (PGLUE_B_REG_USDM_OFFSET_MASK_B: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_OFFSET_MASK_B (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: Offset mask of USDM zone B (legacy zone) in the internal \n    RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90e8UL: // (PGLUE_B_REG_USDM_VF_SHIFT_B: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_VF_SHIFT_B (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: VF Shift of USDM zone B (legacy zone) in the internal RAM\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90ecUL: // (PGLUE_B_REG_XSDM_START_OFFSET_A: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_START_OFFSET_A (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Start offset of XSDM zone A (queue zone) in the internal \n    RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90f0UL: // (PGLUE_B_REG_XSDM_OFFSET_MASK_A: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_OFFSET_MASK_A (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Offset mask of XSDM zone A (queue zone) in the internal RAM\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90f4UL: // (PGLUE_B_REG_XSDM_START_OFFSET_B: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_START_OFFSET_B (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Start offset of XSDM zone B (legacy zone) in the internal\n     RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90f8UL: // (PGLUE_B_REG_XSDM_OFFSET_MASK_B: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_OFFSET_MASK_B (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: Offset mask of XSDM zone B (legacy zone) in the internal \n    RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90fcUL: // (PGLUE_B_REG_XSDM_VF_SHIFT_B: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_VF_SHIFT_B (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: VF Shift of XSDM zone B (legacy zone) in the internal RAM\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9100UL: // (PGLUE_B_REG_CSDM_START_OFFSET_A: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_START_OFFSET_A (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Start offset of CSDM zone A (queue zone) in the internal \n    RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9104UL: // (PGLUE_B_REG_CSDM_OFFSET_MASK_A: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_OFFSET_MASK_A (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Offset mask of CSDM zone A (queue zone) in the internal RAM\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9108UL: // (PGLUE_B_REG_CSDM_START_OFFSET_B: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_START_OFFSET_B (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Start offset of CSDM zone B (legacy zone) in the internal\n     RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x910cUL: // (PGLUE_B_REG_CSDM_OFFSET_MASK_B: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_OFFSET_MASK_B (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: Offset mask of CSDM zone B (legacy zone) in the internal \n    RAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9110UL: // (PGLUE_B_REG_CSDM_VF_SHIFT_B: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_VF_SHIFT_B (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: VF Shift of CSDM zone B (legacy zone) in the internal RAM\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9114UL: // (PGLUE_B_REG_TSDM_INB_INT_A_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_INB_INT_A_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for TSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9118UL: // (PGLUE_B_REG_TSDM_INB_INT_A_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_INB_INT_A_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for TSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x911cUL: // (PGLUE_B_REG_TSDM_INB_INT_A_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_INB_INT_A_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for TSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9120UL: // (PGLUE_B_REG_TSDM_INB_INT_A_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_INB_INT_A_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for TSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9124UL: // (PGLUE_B_REG_TSDM_INB_INT_B_VF: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_INB_INT_B_VF (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: Type B VF inbound interrupt table for TSDM: bits[17:9]-mask\n    ;bits[8:0]-address. Bits [1:0] must be zero (DW resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9128UL: // (PGLUE_B_REG_USDM_INB_INT_A_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_A_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for USDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x912cUL: // (PGLUE_B_REG_USDM_INB_INT_A_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_A_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for USDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9130UL: // (PGLUE_B_REG_USDM_INB_INT_A_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_A_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for USDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9134UL: // (PGLUE_B_REG_USDM_INB_INT_A_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_A_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for USDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9138UL: // (PGLUE_B_REG_USDM_INB_INT_A_4: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_A_4 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for USDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x913cUL: // (PGLUE_B_REG_USDM_INB_INT_A_5: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_A_5 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for USDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9140UL: // (PGLUE_B_REG_USDM_INB_INT_A_6: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_A_6 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for USDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9144UL: // (PGLUE_B_REG_USDM_INB_INT_B_VF: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_B_VF (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: Type B VF inbound interrupt table for USDM: bits[17:9]-mask\n    ;bits[8:0]-address. Bits [1:0] must be zero (DW resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9148UL: // (PGLUE_B_REG_XSDM_INB_INT_A_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_INB_INT_A_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for XSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x914cUL: // (PGLUE_B_REG_XSDM_INB_INT_A_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_INB_INT_A_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for XSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9150UL: // (PGLUE_B_REG_XSDM_INB_INT_A_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_INB_INT_A_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for XSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9154UL: // (PGLUE_B_REG_XSDM_INB_INT_A_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_INB_INT_A_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for XSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9158UL: // (PGLUE_B_REG_XSDM_INB_INT_B_VF: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_INB_INT_B_VF (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: Type B VF inbound interrupt table for XSDM: bits[17:9]-mask\n    ;bits[8:0]-address. Bits [1:0] must be zero (DW resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x915cUL: // (PGLUE_B_REG_CSDM_INB_INT_A_0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_INB_INT_A_0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for CSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9160UL: // (PGLUE_B_REG_CSDM_INB_INT_A_1: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_INB_INT_A_1 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for CSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9164UL: // (PGLUE_B_REG_CSDM_INB_INT_A_2: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_INB_INT_A_2 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for CSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9168UL: // (PGLUE_B_REG_CSDM_INB_INT_A_3: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_INB_INT_A_3 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Type A PF/VF inbound interrupt table for CSDM: bits[9:5]-\n    mask;bits[4:0]-address relative to start_offset_a. Bits [1:0] can have\n     any value (Byte resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x916cUL: // (PGLUE_B_REG_CSDM_INB_INT_B_VF: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_INB_INT_B_VF (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: Type B VF inbound interrupt table for CSDM: bits[17:9]-mask\n    ;bits[8:0]-address. Bits [1:0] must be zero (DW resolution address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9170UL: // (PGLUE_B_REG_TSDM_INB_INT_A_PF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_INB_INT_A_PF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type A PF enable inbound interrupt table for TSDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9174UL: // (PGLUE_B_REG_TSDM_INB_INT_A_VF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_INB_INT_A_VF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type A VF enable inbound interrupt table for TSDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9178UL: // (PGLUE_B_REG_TSDM_INB_INT_B_VF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_INB_INT_B_VF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type B VF enable inbound interrupt table for TSDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x917cUL: // (PGLUE_B_REG_USDM_INB_INT_A_PF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_A_PF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type A PF enable inbound interrupt table for USDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9180UL: // (PGLUE_B_REG_USDM_INB_INT_A_VF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_A_VF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type A VF enable inbound interrupt table for USDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9184UL: // (PGLUE_B_REG_USDM_INB_INT_B_VF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_INB_INT_B_VF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type B VF enable inbound interrupt table for USDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9188UL: // (PGLUE_B_REG_XSDM_INB_INT_A_PF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_INB_INT_A_PF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type A PF enable inbound interrupt table for XSDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x918cUL: // (PGLUE_B_REG_XSDM_INB_INT_A_VF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_INB_INT_A_VF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type A VF enable inbound interrupt table for XSDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9190UL: // (PGLUE_B_REG_XSDM_INB_INT_B_VF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_INB_INT_B_VF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type B VF enable inbound interrupt table for XSDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9194UL: // (PGLUE_B_REG_CSDM_INB_INT_A_PF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_INB_INT_A_PF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type A PF enable inbound interrupt table for CSDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9198UL: // (PGLUE_B_REG_CSDM_INB_INT_A_VF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_INB_INT_A_VF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type A VF enable inbound interrupt table for CSDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x919cUL: // (PGLUE_B_REG_CSDM_INB_INT_B_VF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_INB_INT_B_VF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Type B VF enable inbound interrupt table for CSDM. 0 - disable\n    ; 1 - enable.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91a0UL: // (PGLUE_B_REG_TSDM_ZONE_A_SIZE_PF: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TSDM_ZONE_A_SIZE_PF (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0 - Zone A size is 136x32B; 1 - Zone A size is 152x32B.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91a4UL: // (PGLUE_B_REG_USDM_ZONE_A_SIZE_PF: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_USDM_ZONE_A_SIZE_PF (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0 - Zone A size is 136x32B; 1 - Zone A size is 152x32B.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91a8UL: // (PGLUE_B_REG_XSDM_ZONE_A_SIZE_PF: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_XSDM_ZONE_A_SIZE_PF (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0 - Zone A size is 136x32B; 1 - Zone A size is 152x32B.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91acUL: // (PGLUE_B_REG_CSDM_ZONE_A_SIZE_PF: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSDM_ZONE_A_SIZE_PF (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0 - Zone A size is 136x32B; 1 - Zone A size is 152x32B.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91b0UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_0: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_0 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91b4UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_0: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_0 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91b8UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_1: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_1 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91bcUL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_1: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_1 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91c0UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_2: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_2 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91c4UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_2: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_2 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91c8UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_3: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_3 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91ccUL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_3: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_3 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91d0UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_4: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_4 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91d4UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_4: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_4 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91d8UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_5: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_5 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91dcUL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_5: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_5 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91e0UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_6: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_6 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91e4UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_6: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_6 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91e8UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_7: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_7 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91ecUL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_7: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_7 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91f0UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_8: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_8 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91f4UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_8: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_8 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91f8UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_9: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_9 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x91fcUL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_9: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_9 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9200UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_10: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_10 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9204UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_10: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_10 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9208UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_11: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_11 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x920cUL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_11: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_11 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9210UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_12: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_12 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9214UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_12: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_12 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9218UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_13: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_13 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x921cUL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_13: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_13 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9220UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_14: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_14 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9224UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_14: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_14 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9228UL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART1_15: ACCESS:RW  DataWidth:0x17 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART1_15 (ACCESS:RW  DataWidth:0x17 )" \
				"\nDescription: Part1 of VF GRC access register. Bits [22:20] - GRC range\n    . The GRC range in DWORDS that is allowed to be accessed (starting from\n     GRC base).  0 means that only the GRC base can be accessed. Bits [19:\n    0] - GRC base. The GRC base address that this packet is accessing (in \n    DWORS). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x922cUL: // (PGLUE_B_REG_VF_GRC_ACCESS_PART2_15: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_ACCESS_PART2_15 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Part2 of VF GRC access register. Bits [19:16] - Function \n    offset. This fields allows different functions to access GRC locations\n     in distance 2^Func_offset from GRC_base. Bits [15] - Absolute func index\n    . 0 - Path-relative func index should be used - a number between 0 and\n     63. 1 - Absolute func index should be used - a number between 0 and 127\n    . Bits [14] - Add func index. 1 - Function index should be added to the\n     GRC address.Bits [13] - GRC func. 1 - Access will be with the parent \n    PFID. 0 - Access will be with the VFID. Bits [12:4] - Allowed func ID. [\n    12:10] - PFID. [9:4] - VFID. Bits [3] - R/W. 0 - Read only access; 1 - \n    read and write access. Bits [2:0] - Permission. 0 - no one. 1 - all VFs\n    . 2 - All VFs within port. 3 - All VFs within PF. 4 - Specific VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9230UL: // (PGLUE_B_REG_VF_LENGTH_VIOLATION_DETAILS: ACCESS:R  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_LENGTH_VIOLATION_DETAILS (ACCESS:R  DataWidth:0x1f )" \
				"\nDescription: Details of first target VF request with length violation (\n    too many DWs) accessing BAR0. [12:0] Address in DWs (bits [14:2] of byte\n     address). [14:13] BAR. [20:15] VFID. [23:21] - PFID. [29:24] - Length\n     in DWs. [30] valid - indicates if there was a request with length violation\n     since the last time this register was cleared. Length violations: length\n     of more than 2DWs; length of 2DWs and address not QW aligned; window \n    is GRC and length is more than 1 DW.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9234UL: // (PGLUE_B_REG_VF_GRC_SPACE_VIOLATION_DETAILS: ACCESS:R  DataWidth:0x1a ) 
			sprintf(disp_str, "\nPGLUE_B_REG_VF_GRC_SPACE_VIOLATION_DETAILS (ACCESS:R  DataWidth:0x1a )" \
				"\nDescription: Details of first target VF request accessing VF GRC space\n     that failed permission check. [14:0] Address. [15] w_nr: 0 - Read; 1 - \n    Write. [21:16] VFID. [24:22] - PFID. [25] valid - indicates if there was\n     a request accessing VF GRC space that failed permission check since the\n     last time this register was cleared. Permission checks are: function \n    permission; R/W permission; address range permission.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9238UL: // (PGLUE_B_REG_IDO_ENABLE_MASTER_RW: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_IDO_ENABLE_MASTER_RW (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Each bit when set indicates that IDO bit towards PGLUE should\n     be set for this VQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x923cUL: // (PGLUE_B_REG_IDO_ENABLE_MASTER_RW2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_IDO_ENABLE_MASTER_RW2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: bit 0 - when set indicates that IDO bit towards PGLUE should\n     be set for Translation Requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9240UL: // (PGLUE_B_REG_IDO_ENABLE_TARGET_CPL: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_IDO_ENABLE_TARGET_CPL (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: bit 0 - when set indicates that IDO bit towards PGLUE should\n     be set for Target Completions\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9244UL: // (PGLUE_B_REG_TAGS_63_32: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TAGS_63_32 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Indicates the status of tags 32-63. 0 - tags is used - read\n     completion did not return yet. 1 - tag is unused. Same functionality \n    as pxp2_registers_pgl_exp_rom_data2 for tags 0-31.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9248UL: // (PGLUE_B_REG_IGU_BYPASS_ON_ERR: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_IGU_BYPASS_ON_ERR (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 1 - Do not discard IGU master transactions for PF when the\n     corresponding was_error bit is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9250UL: // (PGLUE_B_REG_ALLOW_MSIX_ACCESS_IN_BAR0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_ALLOW_MSIX_ACCESS_IN_BAR0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0 - Accesses to the first 8KB of IGU in BAR0 (MSIX table \n    and PBA) are not allowed. When this value is configured; BAR2 size for\n     PFs and VFs should be configured to 8KB to allow ONLY MSIX table and \n    PBA access. 1 - All IGU space in BAR 0 is accessible; including the first\n     8KB. When this value is configured; BAR2 size for PFs can be configured\n     to 64KB and for VFs to 16KB to allow all IGU space to be accessed in \n    BAR2 as well.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9254UL: // (PGLUE_B_REG_DISABLE_HIGHER_BW: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DISABLE_HIGHER_BW (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDISABLE_TWO_PENDING_WR_REQUESTS :"
				, (u32_t)((data & PGLUE_B_DISABLE_HIGHER_BW_REG_DISABLE_TWO_PENDING_WR_REQUESTS) >> 2)
				, "\n\tDISABLE_TWO_PENDING_REQUESTS :"
				, (u32_t)((data & PGLUE_B_DISABLE_HIGHER_BW_REG_DISABLE_TWO_PENDING_REQUESTS) >> 1)
				, "\n\tDISABLE_HIGHER_BW_WAW :"
				, (u32_t)((data & PGLUE_B_DISABLE_HIGHER_BW_REG_DISABLE_HIGHER_BW_WAW) >> 0));
			break;
		case 0x9258UL: // (PGLUE_B_REG_DISABLE_RAM1_BAR_ACCESS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DISABLE_RAM1_BAR_ACCESS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0 - RAM1 PF BAR accesses are enabled. 1 - RAM1 PF BAR accesses\n     are disabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x925cUL: // (PGLUE_B_REG_TCPL_IN_TWO_RCBS_DETAILS: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_TCPL_IN_TWO_RCBS_DETAILS (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Details of first ATS Translation Completion received in two\n     rcbs (packets). Logging is triggered by a Translation Completion with\n     length different than 2 DWs. Such a case is unsupported and the Translation\n     completion is considered erroneous. [2:0] - PFID. [3] - VF_VALID. [9:\n    4] - VFID. [14:10] - OTB EntryID. [15] valid - indicates if there was \n    a Translation Completion received in two rcbs since the last time this\n     register was cleared. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9260UL: // (PGLUE_B_REG_PCIE_ERR_STATUS: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PCIE_ERR_STATUS (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Details of PCIe core error status. Valid when pgl_pcie_attn\n     in pxp2 is set. 0 - Unsupported Request or Completer Abort on User RX\n     Interface. 1 - Reception of a poisoned TLP on RX Lanes. 2 - Completion\n     timeout. 3 - Unexpected Completion on RX Lanes. 4 - Detected Unsupported\n     Request on RX Lanes. 5 - ECRC error on RX Lanes. 6 - Reserved. 7 - Reserved\n    . 8 - Illegal operation size on User TX Interface. 9 - Detected Unsupported\n     Request on User TX Interface (Bridge Forwarding Error). 10 - Unsupported\n     header type on User TX Interface. 11 - Reserved. 12 - NP TAG value on\n     User TX Interface already in use. 13 - Completion RTAG value on User \n    TX Interface unexpected. 14 - User TX Interface Overflow Error (Too many\n     req wo/ack). 15 - reserved.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9264UL: // (PGLUE_B_REG_CSSNOOP_ALMOST_FULL_THR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CSSNOOP_ALMOST_FULL_THR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Debug only: If more than this Number of entries are occupied\n     in the cssnoop clock synchronization FIFO; it does not enable writing\n     to the fifo. This value is based on implementation and should not be \n    changed.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9268UL: // (PGLUE_B_REG_PGL_TM0: ACCESS:RW  DataWidth:0x8  Multi Field Register  ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGL_TM0 (ACCESS:RW  DataWidth:0x8  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPGL_PCIE_REPLAY_ADDR_TM :"
				, (u32_t)((data & PGLUE_B_PGL_TM0_REG_PGL_PCIE_REPLAY_ADDR_TM) >> 6)
				, "\n\tPGL_TAG_DB_TM :"
				, (u32_t)((data & PGLUE_B_PGL_TM0_REG_PGL_TAG_DB_TM) >> 4)
				, "\n\tPGL_CPL2_TM :"
				, (u32_t)((data & PGLUE_B_PGL_TM0_REG_PGL_CPL2_TM) >> 2)
				, "\n\tPGL_CPL1_TM :"
				, (u32_t)((data & PGLUE_B_PGL_TM0_REG_PGL_CPL1_TM) >> 0));
			break;
		case 0x926cUL: // (PGLUE_B_REG_PGL_ECO_RESERVED: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGL_ECO_RESERVED (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: debug only: Reserved bits for ECO. Bit 0: Used as chicken\n     bit for CQ43407 fix. Bit 1: Used as chicken bit for CQ43411 fix.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9270UL: // (PGLUE_B_REG_PGL_ECO_RESERVED2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGL_ECO_RESERVED2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: debug only: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9274UL: // (PGLUE_B_REG_DBGSYN_ALMOST_FULL_THR: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DBGSYN_ALMOST_FULL_THR (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Debug only: If more than this Number of entries are occupied\n     in the dbgsyn clock synchronization FIFO; it does not enable writing \n    to the fifo. This value is based on implementation and should not be changed\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9278UL: // (PGLUE_B_REG_DBGBUS_PATH_SELECT: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DBGBUS_PATH_SELECT (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDBGBUS_PATH_SELECT_E1 :"
				, (u32_t)((data & PGLUE_B_DBGBUS_PATH_SELECT_REG_DBGBUS_PATH_SELECT_E1) >> 1)
				, "\n\tDBGBUS_PATH_SELECT_E0 :"
				, (u32_t)((data & PGLUE_B_DBGBUS_PATH_SELECT_REG_DBGBUS_PATH_SELECT_E0) >> 0));
			break;
		case 0x927cUL: // (PGLUE_B_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PGLUE\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9280UL: // (PGLUE_B_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PGLUE\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select and before the shift)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9284UL: // (PGLUE_B_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PGLUE\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9288UL: // (PGLUE_B_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x928cUL: // (PGLUE_B_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9290UL: // (PGLUE_B_REG_CPU_MBIST_MEMCTRL_0_STATUS_0: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CPU_MBIST_MEMCTRL_0_STATUS_0 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9294UL: // (PGLUE_B_REG_CPU_MBIST_MEMCTRL_1_STATUS_0: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CPU_MBIST_MEMCTRL_1_STATUS_0 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9298UL: // (PGLUE_B_REG_PGLUE_B_INT_STS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGLUE_B_INT_STS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPCIE_RX_L0S_TIMEOUT :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_PCIE_RX_L0S_TIMEOUT) >> 10)
				, "\n\tTCPL_TRANSLATION_SIZE_DIFFERENT :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_TCPL_TRANSLATION_SIZE_DIFFERENT) >> 9)
				, "\n\tCSSNOOP_FIFO_OVERFLOW :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_CSSNOOP_FIFO_OVERFLOW) >> 8)
				, "\n\tTCPL_IN_TWO_RCBS_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_TCPL_IN_TWO_RCBS_ATTN) >> 7)
				, "\n\tTCPL_ERROR_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_TCPL_ERROR_ATTN) >> 6)
				, "\n\tVF_MSIX_BAR_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_VF_MSIX_BAR_VIOLATION_ATTN) >> 5)
				, "\n\tVF_GRC_SPACE_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_VF_GRC_SPACE_VIOLATION_ATTN) >> 4)
				, "\n\tVF_LENGTH_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_VF_LENGTH_VIOLATION_ATTN) >> 3)
				, "\n\tWAS_ERROR_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_WAS_ERROR_ATTN) >> 2)
				, "\n\tINCORRECT_RCV_BEHAVIOR :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_INCORRECT_RCV_BEHAVIOR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x929cUL: // (PGLUE_B_REG_PGLUE_B_INT_STS_CLR: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGLUE_B_INT_STS_CLR (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPCIE_RX_L0S_TIMEOUT :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_PCIE_RX_L0S_TIMEOUT) >> 10)
				, "\n\tTCPL_TRANSLATION_SIZE_DIFFERENT :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_TCPL_TRANSLATION_SIZE_DIFFERENT) >> 9)
				, "\n\tCSSNOOP_FIFO_OVERFLOW :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_CSSNOOP_FIFO_OVERFLOW) >> 8)
				, "\n\tTCPL_IN_TWO_RCBS_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_TCPL_IN_TWO_RCBS_ATTN) >> 7)
				, "\n\tTCPL_ERROR_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_TCPL_ERROR_ATTN) >> 6)
				, "\n\tVF_MSIX_BAR_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_VF_MSIX_BAR_VIOLATION_ATTN) >> 5)
				, "\n\tVF_GRC_SPACE_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_VF_GRC_SPACE_VIOLATION_ATTN) >> 4)
				, "\n\tVF_LENGTH_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_VF_LENGTH_VIOLATION_ATTN) >> 3)
				, "\n\tWAS_ERROR_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_WAS_ERROR_ATTN) >> 2)
				, "\n\tINCORRECT_RCV_BEHAVIOR :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_INCORRECT_RCV_BEHAVIOR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x92a0UL: // (PGLUE_B_REG_PGLUE_B_INT_STS_WR: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGLUE_B_INT_STS_WR (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPCIE_RX_L0S_TIMEOUT :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_PCIE_RX_L0S_TIMEOUT) >> 10)
				, "\n\tTCPL_TRANSLATION_SIZE_DIFFERENT :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_TCPL_TRANSLATION_SIZE_DIFFERENT) >> 9)
				, "\n\tCSSNOOP_FIFO_OVERFLOW :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_CSSNOOP_FIFO_OVERFLOW) >> 8)
				, "\n\tTCPL_IN_TWO_RCBS_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_TCPL_IN_TWO_RCBS_ATTN) >> 7)
				, "\n\tTCPL_ERROR_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_TCPL_ERROR_ATTN) >> 6)
				, "\n\tVF_MSIX_BAR_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_VF_MSIX_BAR_VIOLATION_ATTN) >> 5)
				, "\n\tVF_GRC_SPACE_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_VF_GRC_SPACE_VIOLATION_ATTN) >> 4)
				, "\n\tVF_LENGTH_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_VF_LENGTH_VIOLATION_ATTN) >> 3)
				, "\n\tWAS_ERROR_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_WAS_ERROR_ATTN) >> 2)
				, "\n\tINCORRECT_RCV_BEHAVIOR :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_INCORRECT_RCV_BEHAVIOR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x92a4UL: // (PGLUE_B_REG_PGLUE_B_INT_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGLUE_B_INT_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPCIE_RX_L0S_TIMEOUT :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_PCIE_RX_L0S_TIMEOUT) >> 10)
				, "\n\tTCPL_TRANSLATION_SIZE_DIFFERENT :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_TCPL_TRANSLATION_SIZE_DIFFERENT) >> 9)
				, "\n\tCSSNOOP_FIFO_OVERFLOW :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_CSSNOOP_FIFO_OVERFLOW) >> 8)
				, "\n\tTCPL_IN_TWO_RCBS_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_TCPL_IN_TWO_RCBS_ATTN) >> 7)
				, "\n\tTCPL_ERROR_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_TCPL_ERROR_ATTN) >> 6)
				, "\n\tVF_MSIX_BAR_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_VF_MSIX_BAR_VIOLATION_ATTN) >> 5)
				, "\n\tVF_GRC_SPACE_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_VF_GRC_SPACE_VIOLATION_ATTN) >> 4)
				, "\n\tVF_LENGTH_VIOLATION_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_VF_LENGTH_VIOLATION_ATTN) >> 3)
				, "\n\tWAS_ERROR_ATTN :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_WAS_ERROR_ATTN) >> 2)
				, "\n\tINCORRECT_RCV_BEHAVIOR :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_INCORRECT_RCV_BEHAVIOR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x92a8UL: // (PGLUE_B_REG_PGLUE_B_PRTY_STS: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGLUE_B_PRTY_STS (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTAG_DB :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_PRTY_STS_REG_TAG_DB) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x92acUL: // (PGLUE_B_REG_PGLUE_B_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGLUE_B_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x2 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTAG_DB :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_PRTY_STS_CLR_REG_TAG_DB) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x92b0UL: // (PGLUE_B_REG_PGLUE_B_PRTY_STS_WR: ACCESS:WR  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGLUE_B_PRTY_STS_WR (ACCESS:WR  DataWidth:0x2 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTAG_DB :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_PRTY_STS_WR_REG_TAG_DB) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x92b4UL: // (PGLUE_B_REG_PGLUE_B_PRTY_MASK: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGLUE_B_PRTY_MASK (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTAG_DB :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_PRTY_MASK_REG_TAG_DB) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PGLUE_B_PGLUE_B_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x9484UL: // (PGLUE_B_REG_DISABLE_TCPL_TRANSLATION_SIZE_CHECK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DISABLE_TCPL_TRANSLATION_SIZE_CHECK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only: 0 - Enable the fix for CQ45220. If a Function\n     receives a Translation Completion with a Translation Size field smaller\n     than the Function programmed STU value; clear the ATS_en shadow bit and\n     send UR to the ATC. 1 - Disable the fix for CQ45220.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9488UL: // (PGLUE_B_REG_PGL_TM1_E3: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGL_TM1_E3 (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPGL_PCIE_DLP2TLP_BUF_TMD :"
				, (u32_t)((data & PGLUE_B_PGL_TM1_E3_REG_PGL_PCIE_DLP2TLP_BUF_TMD) >> 24)
				, "\n\tPGL_PCIE_DLP2TLP_BUF_TMC :"
				, (u32_t)((data & PGLUE_B_PGL_TM1_E3_REG_PGL_PCIE_DLP2TLP_BUF_TMC) >> 16)
				, "\n\tPGL_PCIE_DLP2TLP_BUF_TMB :"
				, (u32_t)((data & PGLUE_B_PGL_TM1_E3_REG_PGL_PCIE_DLP2TLP_BUF_TMB) >> 8)
				, "\n\tPGL_PCIE_DLP2TLP_BUF_TMA :"
				, (u32_t)((data & PGLUE_B_PGL_TM1_E3_REG_PGL_PCIE_DLP2TLP_BUF_TMA) >> 0));
			break;
		case 0x948cUL: // (PGLUE_B_REG_PGL_TM2_E3: ACCESS:RW  DataWidth:0x18  Multi Field Register  ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGL_TM2_E3 (ACCESS:RW  DataWidth:0x18  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPGL_PCIE_TLP2DLP_BUF_TMC :"
				, (u32_t)((data & PGLUE_B_PGL_TM2_E3_REG_PGL_PCIE_TLP2DLP_BUF_TMC) >> 16)
				, "\n\tPGL_PCIE_TLP2DLP_BUF_TMB :"
				, (u32_t)((data & PGLUE_B_PGL_TM2_E3_REG_PGL_PCIE_TLP2DLP_BUF_TMB) >> 8)
				, "\n\tPGL_PCIE_TLP2DLP_BUF_TMA :"
				, (u32_t)((data & PGLUE_B_PGL_TM2_E3_REG_PGL_PCIE_TLP2DLP_BUF_TMA) >> 0));
			break;
		case 0x9490UL: // (PGLUE_B_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9494UL: // (PGLUE_B_REG_CPU_MBIST_MEMCTRL_3_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CPU_MBIST_MEMCTRL_3_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9498UL: // (PGLUE_B_REG_CPU_MBIST_MEMCTRL_2_STATUS_0: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CPU_MBIST_MEMCTRL_2_STATUS_0 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x949cUL: // (PGLUE_B_REG_CPU_MBIST_MEMCTRL_3_STATUS_0: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CPU_MBIST_MEMCTRL_3_STATUS_0 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9400UL: // (PGLUE_B_REG_CFG_SPACE_A_REQUEST_CLR: ACCESS:W  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CFG_SPACE_A_REQUEST_CLR (ACCESS:W  DataWidth:0x8 )" \
				"\nDescription: Config space A attention dirty bits clear. MCP writes 1 to\n     a bit in this register in order to clear the corresponding bit in cfg_space_a_request\n     register. Note: register contains bits from both paths. Note: Need to\n     re-read the enabled registers after clearing the dirty bit and then check\n     the dirty bit is still clear since they may have been written again during\n     the scan.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9404UL: // (PGLUE_B_REG_CFG_SPACE_B_REQUEST_CLR: ACCESS:W  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_CFG_SPACE_B_REQUEST_CLR (ACCESS:W  DataWidth:0x8 )" \
				"\nDescription: Config space B attention dirty bits clear. MCP writes 1 to\n     a bit in this register in order to clear the corresponding bit in cfg_space_b_request\n     register. Note: register contains bits from both paths. Note: Need to\n     re-read the enabled registers after clearing the dirty bit and then check\n     the dirty bit is still clear since they may have been written again during\n     the scan.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9408UL: // (PGLUE_B_REG_DBG_OUT_DATA_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DBG_OUT_DATA_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PGLUE_B\n     to the DBG block) - The 32 lsb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x940cUL: // (PGLUE_B_REG_DBG_OUT_DATA_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DBG_OUT_DATA_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PGLUE_B\n     to the DBG block) - The 32 msb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9410UL: // (PGLUE_B_REG_DBG_OUT_FRAME: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DBG_OUT_FRAME (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PGLUE_B\n     to the DBG block) - The 4 frame bits that goes to the DBG block. Bit0\n     is the frame of data byte0; Bit1 is the frame of data byte1; Bit2 is \n    the frame of data byte2; Bit3 is the frame of data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9414UL: // (PGLUE_B_REG_DBG_OUT_VALID: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_DBG_OUT_VALID (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PGLUE_B\n     to the DBG block) - The 4 valid bits that goes to the DBG block. Bit0\n     validtes data byte0; Bit1 validates data byte1; Bit2 validates data byte2\n    ; Bit3 validates data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9418UL: // (PGLUE_B_REG_FLR_REQUEST_PF_7_0_CLR: ACCESS:W  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_FLR_REQUEST_PF_7_0_CLR (ACCESS:W  DataWidth:0x8 )" \
				"\nDescription: FLR request attention dirty bits clear for PFs 0 to 7.  MCP\n     writes 1 to a bit in this register in order to clear the corresponding\n     bit in flr_request_pf_7_0 register. Note: register contains bits from\n     both paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x941cUL: // (PGLUE_B_REG_FLR_REQUEST_VF_127_96_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_FLR_REQUEST_VF_127_96_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: FLR request attention dirty bits clear for VFs 96 to 127.  \n    MCP writes 1 to a bit in this register in order to clear the corresponding\n     bit in flr_request_vf_127_96 register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9420UL: // (PGLUE_B_REG_FLR_REQUEST_VF_31_0_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_FLR_REQUEST_VF_31_0_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: FLR request attention dirty bits clear for VFs 0 to 31.  \n    MCP writes 1 to a bit in this register in order to clear the corresponding\n     bit in flr_request_vf_31_0 register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9424UL: // (PGLUE_B_REG_FLR_REQUEST_VF_63_32_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_FLR_REQUEST_VF_63_32_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: FLR request attention dirty bits clear for VFs 32 to 63.  \n    MCP writes 1 to a bit in this register in order to clear the corresponding\n     bit in flr_request_vf_63_32 register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9428UL: // (PGLUE_B_REG_FLR_REQUEST_VF_95_64_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_FLR_REQUEST_VF_95_64_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: FLR request attention dirty bits clear for VFs 64 to 95.  \n    MCP writes 1 to a bit in this register in order to clear the corresponding\n     bit in flr_request_vf_95_64 register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x942cUL: // (PGLUE_B_REG_INTERNAL_PFID_ENABLE_MASTER: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_INTERNAL_PFID_ENABLE_MASTER (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Internal FID_enable configuration per-PF for master transactions\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9430UL: // (PGLUE_B_REG_INTERNAL_PFID_ENABLE_TARGET_READ: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_INTERNAL_PFID_ENABLE_TARGET_READ (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Internal FID_enable configuration per-PF for target read \n    transactions.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9434UL: // (PGLUE_B_REG_INTERNAL_PFID_ENABLE_TARGET_WRITE: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_INTERNAL_PFID_ENABLE_TARGET_WRITE (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: Internal FID_enable configuration per-PF for target write\n     transactions.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9438UL: // (PGLUE_B_REG_INTERNAL_VFID_ENABLE: ACCESS:RW  DataWidth:0x1  SPLIT:128 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_INTERNAL_VFID_ENABLE (ACCESS:RW  DataWidth:0x1  SPLIT:128 )" \
				"\nDescription: Internal FID_enable configuration per-VF for master and target\n     transactions.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x943cUL: // (PGLUE_B_REG_LATCHED_ERRORS_CLR: ACCESS:W  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_LATCHED_ERRORS_CLR (ACCESS:W  DataWidth:0x7 )" \
				"\nDescription: Writing 1 to each bit in this register clears a corresponding\n     error details register and enables logging new error details. Bit 0 - \n    clears INCORRECT_RCV_DETAILS; Bit 1 - clears RX_ERR_DETAILS; Bit 2 - clears\n     TX_ERR_WR_ADD_31_0 TX_ERR_WR_ADD_63_32 TX_ERR_WR_DETAILS TX_ERR_WR_DETAILS2\n     TX_ERR_RD_ADD_31_0 TX_ERR_RD_ADD_63_32 TX_ERR_RD_DETAILS TX_ERR_RD_DETAILS2\n     TX_ERR_WR_DETAILS_ICPL; Bit 3 - clears VF_LENGTH_VIOLATION_DETAILS. Bit\n     4 - clears VF_GRC_SPACE_VIOLATION_DETAILS. Bit 5 - clears RX_TCPL_ERR_DETAILS\n    . Bit 6 - clears TCPL_IN_TWO_RCBS_DETAILS.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9440UL: // (PGLUE_B_REG_SHADOW_ATS_ENABLE_PF_7_0_CLR: ACCESS:W  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_ENABLE_PF_7_0_CLR (ACCESS:W  DataWidth:0x8 )" \
				"\nDescription: Debug only - Shadow ATS_ENABLE bits clear for PFs 0 to 7.  \n    MCP writes 1 to a bit in this register in order to clear the corresponding\n     bit in shadow_ats_enable_pf_7_0 register. MCP should never use this unless\n     a work-around is needed. Note: register contains bits from both paths\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9444UL: // (PGLUE_B_REG_SHADOW_ATS_ENABLE_VF_127_96_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_ENABLE_VF_127_96_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Shadow ATS_ENABLE bits clear for VFs 96 to 127. MCP writes\n     1 to a bit in this register in order to clear the corresponding bit in\n     shadow_ats_enable_vf_127_95 register. MCP should do this in PF-FLR and\n     SR-IOV-disabled events.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9448UL: // (PGLUE_B_REG_SHADOW_ATS_ENABLE_VF_31_0_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_ENABLE_VF_31_0_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Shadow ATS_ENABLE bits clear for VFs 0 to 31.  MCP writes\n     1 to a bit in this register in order to clear the corresponding bit in\n     shadow_ats_enable_vf_31_0 register. MCP should do this in PF-FLR and \n    SR-IOV-disabled events.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x944cUL: // (PGLUE_B_REG_SHADOW_ATS_ENABLE_VF_63_32_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_ENABLE_VF_63_32_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Shadow ATS_ENABLE bits clear for VFs 32 to 63.  MCP writes\n     1 to a bit in this register in order to clear the corresponding bit in\n     shadow_ats_enable_vf_63_32 register. MCP should do this in PF-FLR and\n     SR-IOV-disabled events.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9450UL: // (PGLUE_B_REG_SHADOW_ATS_ENABLE_VF_95_64_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_ENABLE_VF_95_64_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Shadow ATS_ENABLE bits clear for VFs64 to 95.  MCP writes\n     1 to a bit in this register in order to clear the corresponding bit in\n     shadow_ats_enable_vf_95_64 register. MCP should do this in PF-FLR and\n     SR-IOV-disabled events.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9454UL: // (PGLUE_B_REG_SHADOW_ATS_STU: ACCESS:WB_R  DataWidth:0x5  SPLIT:8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_ATS_STU (ACCESS:WB_R  DataWidth:0x5  SPLIT:8 )" \
				"\nDescription: Read only. Shadow ATS_STU register. (2^ATS_STU)*4KB is ATC\n     translation address granularity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9458UL: // (PGLUE_B_REG_SHADOW_BME_PF_7_0_CLR: ACCESS:W  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_BME_PF_7_0_CLR (ACCESS:W  DataWidth:0x8 )" \
				"\nDescription: Debug only - Shadow BME bits clear for PFs 0 to 7.  MCP writes\n     1 to a bit in this register in order to clear the corresponding bit in\n     shadow_bme_pf_7_0 register. MCP should never use this unless a work-around\n     is needed. Note: register contains bits from both paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x945cUL: // (PGLUE_B_REG_SHADOW_BME_VF_127_96_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_BME_VF_127_96_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Shadow BME bits clear for VFs 96 to 127. MCP writes 1 to \n    a bit in this register in order to clear the corresponding bit in shadow_bme_vf_127_95\n     register. MCP should do this in PF-FLR and SR-IOV-disabled events.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9460UL: // (PGLUE_B_REG_SHADOW_BME_VF_31_0_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_BME_VF_31_0_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Shadow BME bits clear for VFs 0 to 31.  MCP writes 1 to a\n     bit in this register in order to clear the corresponding bit in shadow_bme_vf_31_0\n     register. MCP should do this in PF-FLR and SR-IOV-disabled events.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9464UL: // (PGLUE_B_REG_SHADOW_BME_VF_63_32_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_BME_VF_63_32_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Shadow BME bits clear for VFs 32 to 63.  MCP writes 1 to \n    a bit in this register in order to clear the corresponding bit in shadow_bme_vf_63_32\n     register. MCP should do this in PF-FLR and SR-IOV-disabled events.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9468UL: // (PGLUE_B_REG_SHADOW_BME_VF_95_64_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SHADOW_BME_VF_95_64_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Shadow BME bits clear for VFs64 to 95.  MCP writes 1 to a\n     bit in this register in order to clear the corresponding bit in shadow_bme_vf_95_64\n     register. MCP should do this in PF-FLR and SR-IOV-disabled events.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x946cUL: // (PGLUE_B_REG_SR_IOV_DISABLED_REQUEST_CLR: ACCESS:W  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_SR_IOV_DISABLED_REQUEST_CLR (ACCESS:W  DataWidth:0x8 )" \
				"\nDescription: SR IOV disabled attention dirty bits clear. MCP writes 1 \n    to a bit in this register in order to clear the corresponding bit in sr_iov_disabled_request\n     register. Note: register contains bits from both paths.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9470UL: // (PGLUE_B_REG_WAS_ERROR_PF_7_0_CLR: ACCESS:W  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_WAS_ERROR_PF_7_0_CLR (ACCESS:W  DataWidth:0x8 )" \
				"\nDescription: Was_error indication dirty bits clear for PFs 0 to 7.  MCP\n     writes 1 to a bit in this register in order to clear the corresponding\n     bit in flr_request_pf_7_0 register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9474UL: // (PGLUE_B_REG_WAS_ERROR_VF_127_96_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_WAS_ERROR_VF_127_96_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Was_error indication dirty bits clear for VFs 96 to 127.  \n    MCP writes 1 to a bit in this register in order to clear the corresponding\n     bit in was_error_vf_127_96 register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9478UL: // (PGLUE_B_REG_WAS_ERROR_VF_31_0_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_WAS_ERROR_VF_31_0_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Was_error indication dirty bits clear for VFs 0 to 31.  MCP\n     writes 1 to a bit in this register in order to clear the corresponding\n     bit in was_error_vf_31_0 register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x947cUL: // (PGLUE_B_REG_WAS_ERROR_VF_63_32_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_WAS_ERROR_VF_63_32_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Was_error indication dirty bits clear for VFs 32 to 63.  \n    MCP writes 1 to a bit in this register in order to clear the corresponding\n     bit in was_error_vf_63_32 register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9480UL: // (PGLUE_B_REG_WAS_ERROR_VF_95_64_CLR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_WAS_ERROR_VF_95_64_CLR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Was_error indication dirty bits clear for VFs 64 to 95.  \n    MCP writes 1 to a bit in this register in order to clear the corresponding\n     bit in was_error_vf_95_64 register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x924cUL: // (PGLUE_B_REG_PORT4MODE_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPGLUE_B_REG_PORT4MODE_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: 0 - Work in two ports mode (one port per engine). 1 - Work\n     in 4 port mode (2 ports per engine). This affects PFID translation. This\n     register is removed for E3; where this information is received as an \n    input from MISC block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x92b8UL: // (PGLUE_B_REG_PGLUE_B_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGLUE_B_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x94a0UL: // (PGLUE_B_REG_PGLUE_B_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPGLUE_B_REG_PGLUE_B_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40000UL: // (PRS_REG_TSDM_INITIAL_CREDIT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_TSDM_INITIAL_CREDIT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The initial credit in the packet end message to the TSDM \n    interface. Credit is transaction based.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40004UL: // (PRS_REG_FLUSH_REGIONS_TYPE_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_FLUSH_REGIONS_TYPE_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for flush packet with packet type 0. Used \n    in CFC load request message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40008UL: // (PRS_REG_FLUSH_REGIONS_TYPE_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_FLUSH_REGIONS_TYPE_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for flush packet with packet type 1. Used \n    in CFC load request message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4000cUL: // (PRS_REG_FLUSH_REGIONS_TYPE_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_FLUSH_REGIONS_TYPE_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for flush packet with packet type 2. Used \n    in CFC load request message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40010UL: // (PRS_REG_FLUSH_REGIONS_TYPE_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_FLUSH_REGIONS_TYPE_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for flush packet with packet type 3. Used \n    in CFC load request message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40014UL: // (PRS_REG_FLUSH_REGIONS_TYPE_4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_FLUSH_REGIONS_TYPE_4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for flush packet with packet type4. Used in\n     CFC load request message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40018UL: // (PRS_REG_FLUSH_REGIONS_TYPE_5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_FLUSH_REGIONS_TYPE_5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for flush packet with packet type 5. Used \n    in CFC load request message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4001cUL: // (PRS_REG_FLUSH_REGIONS_TYPE_6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_FLUSH_REGIONS_TYPE_6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for flush packet with packet type 6. Used \n    in CFC load request message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40020UL: // (PRS_REG_FLUSH_REGIONS_TYPE_7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_FLUSH_REGIONS_TYPE_7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for flush packet with packet type7. Used in\n     CFC load request message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40024UL: // (PRS_REG_PURE_REGIONS: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PURE_REGIONS (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for pure acknowledge packets. Used in CFC \n    load request message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40028UL: // (PRS_REG_PACKET_REGIONS_TYPE_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PACKET_REGIONS_TYPE_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for received Ethernet packet with a match \n    and packet type 0. Used in CFC load request message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4002cUL: // (PRS_REG_PACKET_REGIONS_TYPE_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PACKET_REGIONS_TYPE_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for received Ethernet packet with a match \n    and packet type 1. Used in CFC load request message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40030UL: // (PRS_REG_PACKET_REGIONS_TYPE_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PACKET_REGIONS_TYPE_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for received Ethernet packet with a match \n    and packet type 2. Used in CFC load request message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40034UL: // (PRS_REG_PACKET_REGIONS_TYPE_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PACKET_REGIONS_TYPE_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for received Ethernet packet with a match \n    and packet type 3. Used in CFC load request message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40038UL: // (PRS_REG_PACKET_REGIONS_TYPE_4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PACKET_REGIONS_TYPE_4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for received Ethernet packet with a match \n    and packet type 4. Used in CFC load request message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4003cUL: // (PRS_REG_PACKET_REGIONS_TYPE_5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PACKET_REGIONS_TYPE_5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for received Ethernet packet with a match \n    and packet type 5. Used in CFC load request message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40040UL: // (PRS_REG_PACKET_REGIONS_TYPE_6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PACKET_REGIONS_TYPE_6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for received Ethernet packet with a match \n    and packet type 6. Used in CFC load request message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40044UL: // (PRS_REG_PACKET_REGIONS_TYPE_7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PACKET_REGIONS_TYPE_7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Context region for received Ethernet packet with a match \n    and packet type 7. Used in CFC load request message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40048UL: // (PRS_REG_INC_VALUE: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPRS_REG_INC_VALUE (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The increment value to send in the CFC load request message\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4004cUL: // (PRS_REG_CFC_LOAD_INITIAL_CREDIT: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_CFC_LOAD_INITIAL_CREDIT (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The initial credit for the load message to the CFC interface\n    . Credit is transaction based.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40050UL: // (PRS_REG_EVENT_ID_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_EVENT_ID_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The 8-bit event ID for a match and packet type 0. Used in\n     packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40054UL: // (PRS_REG_EVENT_ID_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_EVENT_ID_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The 8-bit event ID for a match and packet type 1. Used in\n     packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40058UL: // (PRS_REG_EVENT_ID_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_EVENT_ID_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The 8-bit event ID for a match and packet type 2. Used in\n     packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4005cUL: // (PRS_REG_EVENT_ID_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_EVENT_ID_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The 8-bit event ID for a match and packet type 3. Used in\n     packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40060UL: // (PRS_REG_EVENT_ID_4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_EVENT_ID_4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The 8-bit event ID for a match and packet type 4. Used in\n     packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40064UL: // (PRS_REG_EVENT_ID_5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_EVENT_ID_5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The 8-bit event ID for a match and packet type 5. Used in\n     packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40068UL: // (PRS_REG_EVENT_ID_6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_EVENT_ID_6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The 8-bit event ID for a match and packet type 6. Used in\n     packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4006cUL: // (PRS_REG_EVENT_ID_7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_EVENT_ID_7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The 8-bit event ID for a match and packet type 7. Used in\n     packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40070UL: // (PRS_REG_NO_MATCH_EVENT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_NO_MATCH_EVENT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The 8-bit event ID for cases where there is no match on the\n     connection. Used in packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40074UL: // (PRS_REG_TRCM_INITIAL_CREDIT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_TRCM_INITIAL_CREDIT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The initial credit in the packet start message to the TCM\n     interface (message to STORM). Credit is cycle based.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40078UL: // (PRS_REG_CM_HDR_TYPE_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_TYPE_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 0. Used in packet\n     start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4007cUL: // (PRS_REG_CM_HDR_TYPE_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_TYPE_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 1. Used in packet\n     start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40080UL: // (PRS_REG_CM_HDR_TYPE_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_TYPE_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 2. Used in packet\n     start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40084UL: // (PRS_REG_CM_HDR_TYPE_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_TYPE_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 3. Used in packet\n     start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40088UL: // (PRS_REG_CM_HDR_TYPE_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_TYPE_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 4. Used in packet\n     start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4008cUL: // (PRS_REG_CM_HDR_TYPE_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_TYPE_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 5. Used in packet\n     start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40090UL: // (PRS_REG_CM_HDR_TYPE_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_TYPE_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 6. Used in packet\n     start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40094UL: // (PRS_REG_CM_HDR_TYPE_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_TYPE_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 7. Used in packet\n     start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40098UL: // (PRS_REG_CM_HDR_LOOPBACK_TYPE_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_LOOPBACK_TYPE_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 0 for loopback \n    port. Used in packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4009cUL: // (PRS_REG_CM_HDR_LOOPBACK_TYPE_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_LOOPBACK_TYPE_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 1 for loopback \n    port. Used in packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400a0UL: // (PRS_REG_CM_HDR_LOOPBACK_TYPE_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_LOOPBACK_TYPE_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 2 for loopback \n    port. Used in packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400a4UL: // (PRS_REG_CM_HDR_LOOPBACK_TYPE_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_LOOPBACK_TYPE_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 3 for loopback \n    port. Used in packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400a8UL: // (PRS_REG_CM_HDR_LOOPBACK_TYPE_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_LOOPBACK_TYPE_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 4 for loopback \n    port. Used in packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400acUL: // (PRS_REG_CM_HDR_LOOPBACK_TYPE_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_LOOPBACK_TYPE_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 5 for loopback \n    port. Used in packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400b0UL: // (PRS_REG_CM_HDR_LOOPBACK_TYPE_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_LOOPBACK_TYPE_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 6 for loopback \n    port. Used in packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400b4UL: // (PRS_REG_CM_HDR_LOOPBACK_TYPE_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_LOOPBACK_TYPE_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for a match and packet type 7 for loopback \n    port. Used in packet start message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400b8UL: // (PRS_REG_CM_NO_MATCH_HDR: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_NO_MATCH_HDR (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header in case there was not a match on the connection\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400bcUL: // (PRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is set and packet type is 0. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400c0UL: // (PRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is set and packet type is 1. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400c4UL: // (PRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is set and packet type is 2. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400c8UL: // (PRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is set and packet type is 3. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400ccUL: // (PRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is set and packet type is 4. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400d0UL: // (PRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is set and packet type is 5. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400d4UL: // (PRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is set and packet type is 6. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400d8UL: // (PRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_NO_LOAD_TYPE_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is set and packet type is 7. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400dcUL: // (PRS_REG_CM_HDR_FLUSH_LOAD_TYPE_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_LOAD_TYPE_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is reset and packet type is 0. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400e0UL: // (PRS_REG_CM_HDR_FLUSH_LOAD_TYPE_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_LOAD_TYPE_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is reset and packet type is 1. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400e4UL: // (PRS_REG_CM_HDR_FLUSH_LOAD_TYPE_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_LOAD_TYPE_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is reset and packet type is 2. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400e8UL: // (PRS_REG_CM_HDR_FLUSH_LOAD_TYPE_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_LOAD_TYPE_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is reset and packet type is 3. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400ecUL: // (PRS_REG_CM_HDR_FLUSH_LOAD_TYPE_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_LOAD_TYPE_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is reset and packet type is 4. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400f0UL: // (PRS_REG_CM_HDR_FLUSH_LOAD_TYPE_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_LOAD_TYPE_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is reset and packet type is 5. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400f4UL: // (PRS_REG_CM_HDR_FLUSH_LOAD_TYPE_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_LOAD_TYPE_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is reset and packet type is 6. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400f8UL: // (PRS_REG_CM_HDR_FLUSH_LOAD_TYPE_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_HDR_FLUSH_LOAD_TYPE_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The CM header for flush message where 'load existed' bit \n    in CFC load response is reset and packet type is 7. Used in packet start\n     message to TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x400fcUL: // (PRS_REG_CID_PORT_0: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPRS_REG_CID_PORT_0 (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: CID for port 0 if no match\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40100UL: // (PRS_REG_CID_PORT_1: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPRS_REG_CID_PORT_1 (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: CID for port 1 if no match\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40104UL: // (PRS_REG_LCID_PORT_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_LCID_PORT_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: LCID for port 0 if no match\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40108UL: // (PRS_REG_LCID_PORT_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_LCID_PORT_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: LCID for port 1 if no match\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40114UL: // (PRS_REG_FLEXIBILITYPOSITION: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_FLEXIBILITYPOSITION (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Defines the position in bytes from the beginning of the packet\n     from which to extract a 32-bit field.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40118UL: // (PRS_REG_A_PRSU_12: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nPRS_REG_A_PRSU_12 (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTCP_SEARCH_TCP_DST :"
				, (u32_t)((data & PRS_A_PRSU_12_REG_TCP_SEARCH_TCP_DST) >> 3)
				, "\n\tTCP_SEARCH_TCP_SRC :"
				, (u32_t)((data & PRS_A_PRSU_12_REG_TCP_SEARCH_TCP_SRC) >> 2)
				, "\n\tTCP_SEARCH_IP_DST :"
				, (u32_t)((data & PRS_A_PRSU_12_REG_TCP_SEARCH_IP_DST) >> 1)
				, "\n\tTCP_SEARCH_IP_SRC :"
				, (u32_t)((data & PRS_A_PRSU_12_REG_TCP_SEARCH_IP_SRC) >> 0));
			break;
		case 0x4011cUL: // (PRS_REG_CFC_SEARCH_INITIAL_CREDIT: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_CFC_SEARCH_INITIAL_CREDIT (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The initial credit for the search message to the CFC interface\n    . Credit is transaction based.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40120UL: // (PRS_REG_SRC_INITIAL_CREDIT: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_SRC_INITIAL_CREDIT (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The initial credit in the message to the Searcher interface\n    . Credit is transaction based.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40124UL: // (PRS_REG_NUM_OF_PACKETS: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPRS_REG_NUM_OF_PACKETS (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: The number of input packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40128UL: // (PRS_REG_NUM_OF_CFC_FLUSH_MESSAGES: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPRS_REG_NUM_OF_CFC_FLUSH_MESSAGES (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: The number of input CFC flush packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4012cUL: // (PRS_REG_NUM_OF_TRANSPARENT_FLUSH_MESSAGES: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPRS_REG_NUM_OF_TRANSPARENT_FLUSH_MESSAGES (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: The number of input transparent flush packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40130UL: // (PRS_REG_NUM_OF_DEAD_CYCLES: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_NUM_OF_DEAD_CYCLES (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of cycles the Parser halted its operation since\n     it could not allocate the next serial number\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40134UL: // (PRS_REG_A_PRSU_20: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nPRS_REG_A_PRSU_20 (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tHASH_TCP_IPV6_PORT_1 :"
				, (u32_t)((data & PRS_A_PRSU_20_REG_HASH_TCP_IPV6_PORT_1) >> 3)
				, "\n\tHASH_TCP_IPV6_PORT_0 :"
				, (u32_t)((data & PRS_A_PRSU_20_REG_HASH_TCP_IPV6_PORT_0) >> 2)
				, "\n\tHASH_TCP_IPV4_PORT_1 :"
				, (u32_t)((data & PRS_A_PRSU_20_REG_HASH_TCP_IPV4_PORT_1) >> 1)
				, "\n\tHASH_TCP_IPV4_PORT_0 :"
				, (u32_t)((data & PRS_A_PRSU_20_REG_HASH_TCP_IPV4_PORT_0) >> 0));
			break;
		case 0x4013cUL: // (PRS_REG_SOFT_RST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPRS_REG_SOFT_RST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Soft reset - reset all FSM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40140UL: // (PRS_REG_IFIFO_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPRS_REG_IFIFO_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM Bits of 512x35 IFIFO Memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40144UL: // (PRS_REG_IFIFO_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPRS_REG_IFIFO_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM Bits of 512x35 IFIFO and 400x65 PRSF Memories\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40148UL: // (PRS_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PRS\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4014cUL: // (PRS_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PRS\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select; before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40150UL: // (PRS_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPRS_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PRS\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40154UL: // (PRS_REG_SERIAL_NUM_STATUS_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_SERIAL_NUM_STATUS_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: debug only: Serial number status lsb 32 bits. '1' indicates\n     this serail number was released by SDM but cannot be used because a previous\n     serial number was not released.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40158UL: // (PRS_REG_SERIAL_NUM_STATUS_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_SERIAL_NUM_STATUS_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: debug only: Serial number status msb 32 bits. '1' indicates\n     this serail number was released by SDM but cannot be used because a previous\n     serial number was not released.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4015cUL: // (PRS_REG_TSDM_CURRENT_CREDIT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_TSDM_CURRENT_CREDIT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: debug only: TSDM current credit. Transaction based.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40160UL: // (PRS_REG_TCM_CURRENT_CREDIT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_TCM_CURRENT_CREDIT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: debug only: TCM current credit. Cycle based.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40164UL: // (PRS_REG_CFC_LD_CURRENT_CREDIT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_CFC_LD_CURRENT_CREDIT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: debug only: CFC load request current credit. Transaction \n    based.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40168UL: // (PRS_REG_CFC_SEARCH_CURRENT_CREDIT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_CFC_SEARCH_CURRENT_CREDIT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: debug only: CFC search request current credit. Transaction\n     based.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4016cUL: // (PRS_REG_SRC_CURRENT_CREDIT: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPRS_REG_SRC_CURRENT_CREDIT (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: debug only: SRC current credit. Transaction based.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40170UL: // (PRS_REG_PENDING_BRB_PRS_RQ: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPRS_REG_PENDING_BRB_PRS_RQ (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: debug only: Number of pending requests for header parsing\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40174UL: // (PRS_REG_PENDING_BRB_CAC0_RQ: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPRS_REG_PENDING_BRB_CAC0_RQ (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: debug only: Number of pending requests for CAC on port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40178UL: // (PRS_REG_PENDING_BRB_CAC1_RQ: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPRS_REG_PENDING_BRB_CAC1_RQ (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: debug only: Number of pending requests for CAC on port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4017cUL: // (PRS_REG_PENDING_BRB_CAC2_RQ: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPRS_REG_PENDING_BRB_CAC2_RQ (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: debug only: Number of pending requests for CAC on port 2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40180UL: // (PRS_REG_PENDING_BRB_CAC3_RQ: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPRS_REG_PENDING_BRB_CAC3_RQ (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: debug only: Number of pending requests for CAC on port 3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40184UL: // (PRS_REG_PENDING_BRB_CAC4_RQ: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPRS_REG_PENDING_BRB_CAC4_RQ (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: debug only: Number of pending requests for CAC on port 4.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40188UL: // (PRS_REG_PRS_INT_STS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPRS_REG_PRS_INT_STS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PRS_PRS_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x4018cUL: // (PRS_REG_PRS_INT_STS_CLR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPRS_REG_PRS_INT_STS_CLR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PRS_PRS_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x40190UL: // (PRS_REG_PRS_INT_STS_WR: ACCESS:WR  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPRS_REG_PRS_INT_STS_WR (ACCESS:WR  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PRS_PRS_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x40194UL: // (PRS_REG_PRS_INT_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPRS_REG_PRS_INT_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PRS_PRS_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x40198UL: // (PRS_REG_PRS_PRTY_STS: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PRS_PRTY_STS (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRSF :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_REG_PRSF) >> 7)
				, "\n\tSRCF :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_REG_SRCF) >> 6)
				, "\n\tCFCF :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_REG_CFCF) >> 5)
				, "\n\tCFCQ :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_REG_CFCQ) >> 4)
				, "\n\tCFCR :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_REG_CFCR) >> 3)
				, "\n\tCACU :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_REG_CACU) >> 2)
				, "\n\tIFIFO :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_REG_IFIFO) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x4019cUL: // (PRS_REG_PRS_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PRS_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x8 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRSF :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_CLR_REG_PRSF) >> 7)
				, "\n\tSRCF :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_CLR_REG_SRCF) >> 6)
				, "\n\tCFCF :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_CLR_REG_CFCF) >> 5)
				, "\n\tCFCQ :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_CLR_REG_CFCQ) >> 4)
				, "\n\tCFCR :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_CLR_REG_CFCR) >> 3)
				, "\n\tCACU :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_CLR_REG_CACU) >> 2)
				, "\n\tIFIFO :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_CLR_REG_IFIFO) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x401a0UL: // (PRS_REG_PRS_PRTY_STS_WR: ACCESS:WR  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PRS_PRTY_STS_WR (ACCESS:WR  DataWidth:0x8 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRSF :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_WR_REG_PRSF) >> 7)
				, "\n\tSRCF :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_WR_REG_SRCF) >> 6)
				, "\n\tCFCF :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_WR_REG_CFCF) >> 5)
				, "\n\tCFCQ :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_WR_REG_CFCQ) >> 4)
				, "\n\tCFCR :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_WR_REG_CFCR) >> 3)
				, "\n\tCACU :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_WR_REG_CACU) >> 2)
				, "\n\tIFIFO :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_WR_REG_IFIFO) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PRS_PRS_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x401a4UL: // (PRS_REG_PRS_PRTY_MASK: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_PRS_PRTY_MASK (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRSF :"
				, (u32_t)((data & PRS_PRS_PRTY_MASK_REG_PRSF) >> 7)
				, "\n\tSRCF :"
				, (u32_t)((data & PRS_PRS_PRTY_MASK_REG_SRCF) >> 6)
				, "\n\tCFCF :"
				, (u32_t)((data & PRS_PRS_PRTY_MASK_REG_CFCF) >> 5)
				, "\n\tCFCQ :"
				, (u32_t)((data & PRS_PRS_PRTY_MASK_REG_CFCQ) >> 4)
				, "\n\tCFCR :"
				, (u32_t)((data & PRS_PRS_PRTY_MASK_REG_CFCR) >> 3)
				, "\n\tCACU :"
				, (u32_t)((data & PRS_PRS_PRTY_MASK_REG_CACU) >> 2)
				, "\n\tIFIFO :"
				, (u32_t)((data & PRS_PRS_PRTY_MASK_REG_IFIFO) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PRS_PRS_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x401a8UL: // (PRS_REG_VLAN_TYPE_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_VLAN_TYPE_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401acUL: // (PRS_REG_VLAN_TYPE_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_VLAN_TYPE_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401b0UL: // (PRS_REG_VLAN_TYPE_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_VLAN_TYPE_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401b4UL: // (PRS_REG_VLAN_TYPE_3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_VLAN_TYPE_3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401b8UL: // (PRS_REG_VLAN_TYPE_4: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_VLAN_TYPE_4 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401bcUL: // (PRS_REG_VLAN_TYPE_5: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_VLAN_TYPE_5 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401c0UL: // (PRS_REG_VLAN_TYPE_6: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_VLAN_TYPE_6 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401c4UL: // (PRS_REG_VLAN_TYPE_7: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_VLAN_TYPE_7 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: One of 8 values that should be compared to type in Ethernet\n     parsing. If there is a match; the field after Ethernet is the first VLAN\n    . Reset value is 0x8100 which is the standard VLAN type. Note that when\n     checking second VLAN; type is compared only to 0x8100.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401c8UL: // (PRS_REG_E1HOV_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPRS_REG_E1HOV_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates if in outer vlan mode. 0=non-outer-vlan mode; 1\n    =outer vlan mode.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401ccUL: // (PRS_REG_HASH_5TH_TUPLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPRS_REG_HASH_5TH_TUPLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates if to include the protocol field in the hash request\n    . 0=do not include; 1=include\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401d0UL: // (PRS_REG_FCOE_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_FCOE_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for first FCoE type\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401d4UL: // (PRS_REG_TAG_ETHERTYPE_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_TAG_ETHERTYPE_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401d8UL: // (PRS_REG_TCP_SEARCH_VLAN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPRS_REG_TCP_SEARCH_VLAN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates whether to include inner VLAN (if present) in the\n     TCP search.  If not present, a value of 0 will be sent in its place.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40220UL: // (PRS_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: debug only: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40224UL: // (PRS_REG_TAG_ETHERTYPE_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_TAG_ETHERTYPE_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40228UL: // (PRS_REG_TAG_ETHERTYPE_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPRS_REG_TAG_ETHERTYPE_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The Ethernet type value for L2 tag 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4022cUL: // (PRS_REG_TAG_LEN_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPRS_REG_TAG_LEN_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 0.  The length is\n     between 2B and 14B; in 2B granularity\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40230UL: // (PRS_REG_TAG_LEN_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPRS_REG_TAG_LEN_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 1.  The length is\n     between 2B and 14B; in 2B granularity\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40234UL: // (PRS_REG_TAG_LEN_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPRS_REG_TAG_LEN_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The length of the info field for L2 tag 2.  The length is\n     between 2B and 14B; in 2B granularity\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40258UL: // (PRS_REG_LOAD_CANCELLED_EVENT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_LOAD_CANCELLED_EVENT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Event ID for the load cancelled case\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4025cUL: // (PRS_REG_CONTEXT_ERROR_EVENT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPRS_REG_CONTEXT_ERROR_EVENT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Event ID for the context error case\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40260UL: // (PRS_REG_CM_LOAD_CANCELLED_HDR: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_LOAD_CANCELLED_HDR (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: CM header for the load cancelled case\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40264UL: // (PRS_REG_CM_CONTEXT_ERROR_HDR: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPRS_REG_CM_CONTEXT_ERROR_HDR (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: CM header for the context error case\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40268UL: // (PRS_REG_HIGIG_HDR_SIZE_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPRS_REG_HIGIG_HDR_SIZE_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Size of the HiGig header for port 0 (in 4B increments).  \n    If HiGig is disabled this value should be 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4026cUL: // (PRS_REG_HIGIG_HDR_SIZE_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPRS_REG_HIGIG_HDR_SIZE_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Size of the HiGig header for port 1 (in 4B increments).  \n    If HiGig is disabled this value should be 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40270UL: // (PRS_REG_HDRS_AFTER_BASIC_PORT_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_BASIC_PORT_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the basic\n     Ethernet header for port 0 packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40274UL: // (PRS_REG_HDRS_AFTER_OUTER_VLAN_PORT_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_OUTER_VLAN_PORT_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the outer\n     VLAN header for port 0 packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40278UL: // (PRS_REG_HDRS_AFTER_INNER_VLAN_PORT_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_INNER_VLAN_PORT_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the inner\n     VLAN header for port 0 packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4027cUL: // (PRS_REG_HDRS_AFTER_LLC_PORT_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_LLC_PORT_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the LLC\n     header for port 0 packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40280UL: // (PRS_REG_HDRS_AFTER_TAG_0_PORT_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_TAG_0_PORT_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    0 for port 0 packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40284UL: // (PRS_REG_HDRS_AFTER_TAG_1_PORT_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_TAG_1_PORT_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    1 for port 0 packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40288UL: // (PRS_REG_HDRS_AFTER_TAG_2_PORT_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_TAG_2_PORT_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    2 for port 0 packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4028cUL: // (PRS_REG_MUST_HAVE_HDRS_PORT_0: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_MUST_HAVE_HDRS_PORT_0 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which headers must appear in the packet\n     for port 0 packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40290UL: // (PRS_REG_HDRS_AFTER_BASIC_PORT_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_BASIC_PORT_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the basic\n     Ethernet header for port 1 packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40294UL: // (PRS_REG_HDRS_AFTER_OUTER_VLAN_PORT_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_OUTER_VLAN_PORT_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the outer\n     VLAN header for port 1 packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40298UL: // (PRS_REG_HDRS_AFTER_INNER_VLAN_PORT_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_INNER_VLAN_PORT_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the inner\n     VLAN header for port 1 packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4029cUL: // (PRS_REG_HDRS_AFTER_LLC_PORT_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_LLC_PORT_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the LLC\n     header for port 1 packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x402a0UL: // (PRS_REG_HDRS_AFTER_TAG_0_PORT_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_TAG_0_PORT_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    0 for port 1 packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x402a4UL: // (PRS_REG_HDRS_AFTER_TAG_1_PORT_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_TAG_1_PORT_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    1 for port 1 packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x402a8UL: // (PRS_REG_HDRS_AFTER_TAG_2_PORT_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_TAG_2_PORT_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    2 for port 1 packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x402acUL: // (PRS_REG_MUST_HAVE_HDRS_PORT_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_MUST_HAVE_HDRS_PORT_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which headers must appear in the packet\n     for port 1 packets\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x402b0UL: // (PRS_REG_TCP_SEARCH_OVLAN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPRS_REG_TCP_SEARCH_OVLAN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates whether to include outer VLAN (if present) in the\n     TCP search.  If not present, a value of 1 will be sent in its place.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40138UL: // (PRS_REG_NIC_MODE: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nPRS_REG_NIC_MODE (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: If set indicates not to send messages to CFC on received \n    packets - one bit per PF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40200UL: // (PRS_REG_INFO_VECTOR: ACCESS:WB_R  DataWidth:0x70 ) 
			sprintf(disp_str, "\nPRS_REG_INFO_VECTOR (ACCESS:WB_R  DataWidth:0x70 )" \
				"\nDescription: The value of PARAM0; PARAM1; PARAM2 and PARAM3 [15:0] from\n     the last message sent to the TCP-Rx STORM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40238UL: // (PRS_REG_HDRS_AFTER_BASIC: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_BASIC (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the basic\n     Ethernet header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4023cUL: // (PRS_REG_HDRS_AFTER_OUTER_VLAN: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_OUTER_VLAN (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the outer\n     VLAN header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40240UL: // (PRS_REG_HDRS_AFTER_INNER_VLAN: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_INNER_VLAN (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the inner\n     VLAN header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40244UL: // (PRS_REG_HDRS_AFTER_LLC: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_LLC (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after the LLC\n     header.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40248UL: // (PRS_REG_HDRS_AFTER_TAG_0: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_TAG_0 (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4024cUL: // (PRS_REG_HDRS_AFTER_TAG_1: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_TAG_1 (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40250UL: // (PRS_REG_HDRS_AFTER_TAG_2: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_HDRS_AFTER_TAG_2 (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which L2 hdrs may appear after L2 tag \n    2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40254UL: // (PRS_REG_MUST_HAVE_HDRS: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPRS_REG_MUST_HAVE_HDRS (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Bit-map indicating which headers must appear in the packet\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x401dcUL: // (PRS_REG_PRS_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPRS_REG_PRS_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x402b4UL: // (PRS_REG_PRS_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPRS_REG_PRS_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120000UL: // (PXP2_REG_PSWRQ_CDU0_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_CDU0_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_CDU0_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_CDU0_L2P_REG_RQ_CDU0_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_CDU0_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_CDU0_L2P_REG_RQ_CDU0_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120004UL: // (PXP2_REG_PSWRQ_CDU1_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_CDU1_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_CDU1_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_CDU1_L2P_REG_RQ_CDU1_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_CDU1_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_CDU1_L2P_REG_RQ_CDU1_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120008UL: // (PXP2_REG_RQ_CDU0_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CDU0_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for CDU module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12000cUL: // (PXP2_REG_RQ_CDU0_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CDU0_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for CDU module \n    port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120010UL: // (PXP2_REG_RQ_CDU1_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CDU1_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for CDU module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120014UL: // (PXP2_REG_RQ_CDU1_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CDU1_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for CDU module \n    port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12001cUL: // (PXP2_REG_PSWRQ_TM0_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_TM0_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_TM0_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_TM0_L2P_REG_RQ_TM0_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_TM0_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_TM0_L2P_REG_RQ_TM0_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120020UL: // (PXP2_REG_PSWRQ_TM1_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_TM1_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_TM1_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_TM1_L2P_REG_RQ_TM1_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_TM1_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_TM1_L2P_REG_RQ_TM1_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120024UL: // (PXP2_REG_RQ_TM0_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TM0_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for TM module \n    port 0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120028UL: // (PXP2_REG_RQ_TM0_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TM0_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for TM module port\n     0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12002cUL: // (PXP2_REG_RQ_TM1_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TM1_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for TM module \n    port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120030UL: // (PXP2_REG_RQ_TM1_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TM1_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for TM module port\n     1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120038UL: // (PXP2_REG_PSWRQ_QM0_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_QM0_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_QM0_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_QM0_L2P_REG_RQ_QM0_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_QM0_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_QM0_L2P_REG_RQ_QM0_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x12003cUL: // (PXP2_REG_PSWRQ_QM1_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_QM1_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_QM1_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_QM1_L2P_REG_RQ_QM1_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_QM1_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_QM1_L2P_REG_RQ_QM1_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120040UL: // (PXP2_REG_RQ_QM0_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QM0_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for QM module \n    port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120044UL: // (PXP2_REG_RQ_QM0_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QM0_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for QM module port\n     0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120048UL: // (PXP2_REG_RQ_QM1_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QM1_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for QM module \n    port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12004cUL: // (PXP2_REG_RQ_QM1_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QM1_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for QM module port\n     1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120054UL: // (PXP2_REG_PSWRQ_SRC0_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_SRC0_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_SRC0_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_SRC0_L2P_REG_RQ_SRC0_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_SRC0_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_SRC0_L2P_REG_RQ_SRC0_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120058UL: // (PXP2_REG_PSWRQ_SRC1_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_SRC1_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_SRC1_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_SRC1_L2P_REG_RQ_SRC1_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_SRC1_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_SRC1_L2P_REG_RQ_SRC1_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x12005cUL: // (PXP2_REG_RQ_SRC0_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SRC0_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for SRC module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120060UL: // (PXP2_REG_RQ_SRC0_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SRC0_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for SRC module \n    port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120064UL: // (PXP2_REG_RQ_SRC1_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SRC1_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for SRC module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120068UL: // (PXP2_REG_RQ_SRC1_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SRC1_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for SRC module \n    port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120070UL: // (PXP2_REG_PSWRQ_XSDM0_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_XSDM0_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_XSDM0_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_XSDM0_L2P_REG_RQ_XSDM0_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_XSDM0_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_XSDM0_L2P_REG_RQ_XSDM0_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120074UL: // (PXP2_REG_PSWRQ_XSDM1_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_XSDM1_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_XSDM1_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_XSDM1_L2P_REG_RQ_XSDM1_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_XSDM1_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_XSDM1_L2P_REG_RQ_XSDM1_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120078UL: // (PXP2_REG_RQ_XSDM0_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_XSDM0_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for XSDM module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12007cUL: // (PXP2_REG_RQ_XSDM0_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_XSDM0_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for XSDM module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120080UL: // (PXP2_REG_RQ_XSDM1_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_XSDM1_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for XSDM module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120084UL: // (PXP2_REG_RQ_XSDM1_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_XSDM1_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for XSDM module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12008cUL: // (PXP2_REG_PSWRQ_USDM0_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_USDM0_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_USDM0_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_USDM0_L2P_REG_RQ_USDM0_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_USDM0_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_USDM0_L2P_REG_RQ_USDM0_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120090UL: // (PXP2_REG_PSWRQ_USDM1_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_USDM1_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_USDM1_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_USDM1_L2P_REG_RQ_USDM1_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_USDM1_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_USDM1_L2P_REG_RQ_USDM1_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120094UL: // (PXP2_REG_RQ_USDM0_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDM0_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for USDM module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120098UL: // (PXP2_REG_RQ_USDM0_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDM0_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for USDM module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12009cUL: // (PXP2_REG_RQ_USDM1_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDM1_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for USDM module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200a0UL: // (PXP2_REG_RQ_USDM1_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDM1_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for USDM module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200a8UL: // (PXP2_REG_PSWRQ_USDMDP0_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_USDMDP0_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_USDMDP0_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_USDMDP0_L2P_REG_RQ_USDMDP0_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_USDMDP0_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_USDMDP0_L2P_REG_RQ_USDMDP0_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x1200acUL: // (PXP2_REG_PSWRQ_USDMDP1_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_USDMDP1_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_USDMDP1_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_USDMDP1_L2P_REG_RQ_USDMDP1_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_USDMDP1_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_USDMDP1_L2P_REG_RQ_USDMDP1_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x1200b0UL: // (PXP2_REG_RQ_USDMDP0_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDMDP0_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for USDM DP module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200b4UL: // (PXP2_REG_RQ_USDMDP0_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDMDP0_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for USDM DP module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200b8UL: // (PXP2_REG_RQ_USDMDP1_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDMDP1_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for USDM DP module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200bcUL: // (PXP2_REG_RQ_USDMDP1_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDMDP1_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for USDM DP module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200c4UL: // (PXP2_REG_PSWRQ_CSDM0_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_CSDM0_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_CSDM0_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_CSDM0_L2P_REG_RQ_CSDM0_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_CSDM0_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_CSDM0_L2P_REG_RQ_CSDM0_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x1200c8UL: // (PXP2_REG_PSWRQ_CSDM1_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_CSDM1_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_CSDM1_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_CSDM1_L2P_REG_RQ_CSDM1_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_CSDM1_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_CSDM1_L2P_REG_RQ_CSDM1_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x1200ccUL: // (PXP2_REG_RQ_CSDM0_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CSDM0_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for CSDM module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200d0UL: // (PXP2_REG_RQ_CSDM0_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CSDM0_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for CSDM module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200d4UL: // (PXP2_REG_RQ_CSDM1_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CSDM1_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for CSDM module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200d8UL: // (PXP2_REG_RQ_CSDM1_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CSDM1_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for CSDM module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200e0UL: // (PXP2_REG_PSWRQ_TSDM0_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_TSDM0_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_TSDM0_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_TSDM0_L2P_REG_RQ_TSDM0_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_TSDM0_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_TSDM0_L2P_REG_RQ_TSDM0_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x1200e4UL: // (PXP2_REG_PSWRQ_TSDM1_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_TSDM1_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_TSDM1_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_TSDM1_L2P_REG_RQ_TSDM1_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_TSDM1_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_TSDM1_L2P_REG_RQ_TSDM1_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x1200e8UL: // (PXP2_REG_RQ_TSDM0_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TSDM0_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for TSDM module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200ecUL: // (PXP2_REG_RQ_TSDM0_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TSDM0_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for TSDM module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200f0UL: // (PXP2_REG_RQ_TSDM1_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TSDM1_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for TSDM module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200f4UL: // (PXP2_REG_RQ_TSDM1_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TSDM1_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for TSDM module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1200fcUL: // (PXP2_REG_PSWRQ_DBG0_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_DBG0_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_DBG0_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_DBG0_L2P_REG_RQ_DBG0_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_DBG0_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_DBG0_L2P_REG_RQ_DBG0_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120100UL: // (PXP2_REG_PSWRQ_DBG1_L2P: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_DBG1_L2P (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_DBG1_LAST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_DBG1_L2P_REG_RQ_DBG1_LAST_MEM_ADDR) >> 10)
				, "\n\tRQ_DBG1_FIRST_MEM_ADDR :"
				, (u32_t)((data & PXP2_PSWRQ_DBG1_L2P_REG_RQ_DBG1_FIRST_MEM_ADDR) >> 0));
			break;
		case 0x120104UL: // (PXP2_REG_RQ_DBG0_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DBG0_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for dbg module\n     port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120108UL: // (PXP2_REG_RQ_DBG0_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DBG0_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for dbg module \n    port 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12010cUL: // (PXP2_REG_RQ_DBG1_EFIRST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DBG1_EFIRST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external first_mem_addr field in L2P table for dbg module\n     port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120110UL: // (PXP2_REG_RQ_DBG1_ELAST_MEM_ADDR: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DBG1_ELAST_MEM_ADDR (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: external last_mem_addr field in L2P table for dbg module \n    port 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120118UL: // (PXP2_REG_PSWRQ_HOST_L2P: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_HOST_L2P (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12011cUL: // (PXP2_REG_RQ_CSDM_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CSDM_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    csdm in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120120UL: // (PXP2_REG_RQ_USDM_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDM_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    usdm in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120124UL: // (PXP2_REG_RQ_USDMDP_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDMDP_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    usdmdp in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120128UL: // (PXP2_REG_RQ_TSDM_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TSDM_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    tsdm in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12012cUL: // (PXP2_REG_RQ_XSDM_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_XSDM_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    xsdm in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120130UL: // (PXP2_REG_RQ_DMAE_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DMAE_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    rwh in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120134UL: // (PXP2_REG_RQ_CDUWR_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CDUWR_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    cduwr in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120138UL: // (PXP2_REG_RQ_CDURD_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CDURD_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    cdurd in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12013cUL: // (PXP2_REG_RQ_PBF_ENTRY_TH: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_PBF_ENTRY_TH (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    pbf in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120140UL: // (PXP2_REG_RQ_QM_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QM_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    qm in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120144UL: // (PXP2_REG_RQ_TM_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TM_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    tm in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120148UL: // (PXP2_REG_RQ_SRC_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SRC_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    src in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12014cUL: // (PXP2_REG_RQ_DBG_ENTRY_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DBG_ENTRY_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    debug in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120150UL: // (PXP2_REG_RQ_HC_ENTRY_TH: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_HC_ENTRY_TH (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: This number indicates how many entries are guaranteed to \n    hc in the queues\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120154UL: // (PXP2_REG_RQ_GC_INIT_VAL: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_GC_INIT_VAL (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Initial value of global counter; This value MUST be 256 - \n    sum of all clients thresholds\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120158UL: // (PXP2_REG_PSWRQ_UFIFO: ACCESS:RW  DataWidth:0x8  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_UFIFO (ACCESS:RW  DataWidth:0x8  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_UFIFO_HIGH_TH :"
				, (u32_t)((data & PXP2_PSWRQ_UFIFO_REG_RQ_UFIFO_HIGH_TH) >> 4)
				, "\n\tRQ_UFIFO_LOW_TH :"
				, (u32_t)((data & PXP2_PSWRQ_UFIFO_REG_RQ_UFIFO_LOW_TH) >> 0));
			break;
		case 0x12015cUL: // (PXP2_REG_RQ_WR_MBS0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_WR_MBS0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Max burst size filed for write requests port 0; 000 - 128B\n    ; 001:256B; 010: 512B;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120160UL: // (PXP2_REG_RQ_RD_MBS0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_RD_MBS0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Max burst size filed for read requests port 0; 000 - 128B\n    ; 001:256B; 010: 512B;011:1K:100:2K;101:4K\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120164UL: // (PXP2_REG_RQ_WR_MBS1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_WR_MBS1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Max burst size filed for write requests port 1; 000 - 128B\n    ; 001:256B; 010: 512B;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120168UL: // (PXP2_REG_RQ_RD_MBS1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_RD_MBS1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Max burst size filed for read requests port 1; 000 - 128B\n    ; 001:256B; 010: 512B;011:1K:100:2K;101:4K\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12016cUL: // (PXP2_REG_RQ_IGNORE_BLK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_IGNORE_BLK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: if '1'; requester will ignore the reader blk_per_client indication\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120170UL: // (PXP2_REG_RQ_FUNCNUM0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_FUNCNUM0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120174UL: // (PXP2_REG_RQ_FUNCNUM1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_FUNCNUM1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120178UL: // (PXP2_REG_PSWRQ_QM_PCI_ATTR: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_QM_PCI_ATTR (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_QM_NOSNOOP :"
				, (u32_t)((data & PXP2_PSWRQ_QM_PCI_ATTR_REG_RQ_QM_NOSNOOP) >> 1)
				, "\n\tRQ_QM_RELAXED :"
				, (u32_t)((data & PXP2_PSWRQ_QM_PCI_ATTR_REG_RQ_QM_RELAXED) >> 0));
			break;
		case 0x12017cUL: // (PXP2_REG_PSWRQ_TM_PCI_ATTR: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_TM_PCI_ATTR (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_TM_NOSNOOP :"
				, (u32_t)((data & PXP2_PSWRQ_TM_PCI_ATTR_REG_RQ_TM_NOSNOOP) >> 1)
				, "\n\tRQ_TM_RELAXED :"
				, (u32_t)((data & PXP2_PSWRQ_TM_PCI_ATTR_REG_RQ_TM_RELAXED) >> 0));
			break;
		case 0x120180UL: // (PXP2_REG_PSWRQ_SRC_PCI_ATTR: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_SRC_PCI_ATTR (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_SRC_NOSNOOP :"
				, (u32_t)((data & PXP2_PSWRQ_SRC_PCI_ATTR_REG_RQ_SRC_NOSNOOP) >> 1)
				, "\n\tRQ_SRC_RELAXED :"
				, (u32_t)((data & PXP2_PSWRQ_SRC_PCI_ATTR_REG_RQ_SRC_RELAXED) >> 0));
			break;
		case 0x120184UL: // (PXP2_REG_PSWRQ_CDU_PCI_ATTR: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_CDU_PCI_ATTR (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_CDU_NOSNOOP :"
				, (u32_t)((data & PXP2_PSWRQ_CDU_PCI_ATTR_REG_RQ_CDU_NOSNOOP) >> 1)
				, "\n\tRQ_CDU_RELAXED :"
				, (u32_t)((data & PXP2_PSWRQ_CDU_PCI_ATTR_REG_RQ_CDU_RELAXED) >> 0));
			break;
		case 0x120188UL: // (PXP2_REG_PSWRQ_DBG_PCI_ATTR: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_DBG_PCI_ATTR (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_DBG_NOSNOOP :"
				, (u32_t)((data & PXP2_PSWRQ_DBG_PCI_ATTR_REG_RQ_DBG_NOSNOOP) >> 1)
				, "\n\tRQ_DBG_RELAXED :"
				, (u32_t)((data & PXP2_PSWRQ_DBG_PCI_ATTR_REG_RQ_DBG_RELAXED) >> 0));
			break;
		case 0x12018cUL: // (PXP2_REG_PSWRQ_HC_PCI_ATTR: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_HC_PCI_ATTR (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_HC_NOSNOOP :"
				, (u32_t)((data & PXP2_PSWRQ_HC_PCI_ATTR_REG_RQ_HC_NOSNOOP) >> 1)
				, "\n\tRQ_HC_RELAXED :"
				, (u32_t)((data & PXP2_PSWRQ_HC_PCI_ATTR_REG_RQ_HC_RELAXED) >> 0));
			break;
		case 0x120190UL: // (PXP2_REG_PSWRQ_DMAE_PCI_ATTR: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_DMAE_PCI_ATTR (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_DMAE_NOSNOOP :"
				, (u32_t)((data & PXP2_PSWRQ_DMAE_PCI_ATTR_REG_RQ_DMAE_NOSNOOP) >> 1)
				, "\n\tRQ_DMAE_RELAXED :"
				, (u32_t)((data & PXP2_PSWRQ_DMAE_PCI_ATTR_REG_RQ_DMAE_RELAXED) >> 0));
			break;
		case 0x120194UL: // (PXP2_REG_RQ_QM_ENDIAN_M: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QM_ENDIAN_M (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Endian mode for qm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120198UL: // (PXP2_REG_RQ_TM_ENDIAN_M: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TM_ENDIAN_M (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Endian mode for tm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12019cUL: // (PXP2_REG_RQ_SRC_ENDIAN_M: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SRC_ENDIAN_M (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Endian mode for src\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201a0UL: // (PXP2_REG_RQ_CDU_ENDIAN_M: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CDU_ENDIAN_M (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Endian mode for cdu\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201a4UL: // (PXP2_REG_RQ_DBG_ENDIAN_M: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DBG_ENDIAN_M (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Endian mode for debug\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201a8UL: // (PXP2_REG_RQ_HC_ENDIAN_M: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_HC_ENDIAN_M (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Endian mode for hc\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201acUL: // (PXP2_REG_RQ_PBF_ENDIAN_M: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_PBF_ENDIAN_M (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Endian mode for pbf\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201b0UL: // (PXP2_REG_RQ_RBC_DONE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_RBC_DONE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 1' indicates that the RBC has finished configuring the PSWRQ\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201b4UL: // (PXP2_REG_RQ_CFG_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CFG_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: 1' indicates that the requester has finished its internal\n     configuration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201b8UL: // (PXP2_REG_RQ_DONE_FIFO_TH: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DONE_FIFO_TH (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Write Done fifo threshold; this fifo has write done indications\n    ;this threshold would not be reached unless there is a bug\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201bcUL: // (PXP2_REG_RQ_BW_RD_ADD0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ0 write requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201c0UL: // (PXP2_REG_PSWRQ_BW_ADD1: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_ADD1 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_ADD1 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD1_REG_RQ_BW_WR_ADD1) >> 10)
				, "\n\tRQ_BW_RD_ADD1 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD1_REG_RQ_BW_RD_ADD1) >> 0));
			break;
		case 0x1201c4UL: // (PXP2_REG_PSWRQ_BW_ADD2: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_ADD2 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_ADD2 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD2_REG_RQ_BW_WR_ADD2) >> 10)
				, "\n\tRQ_BW_RD_ADD2 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD2_REG_RQ_BW_RD_ADD2) >> 0));
			break;
		case 0x1201c8UL: // (PXP2_REG_PSWRQ_BW_ADD3: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_ADD3 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_ADD3 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD3_REG_RQ_BW_WR_ADD3) >> 10)
				, "\n\tRQ_BW_RD_ADD3 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD3_REG_RQ_BW_RD_ADD3) >> 0));
			break;
		case 0x1201ccUL: // (PXP2_REG_RQ_BW_RD_ADD4: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD4 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ4 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201d0UL: // (PXP2_REG_RQ_BW_RD_ADD5: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD5 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ5 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201d4UL: // (PXP2_REG_PSWRQ_BW_ADD6: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_ADD6 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_ADD6 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD6_REG_RQ_BW_WR_ADD6) >> 10)
				, "\n\tRQ_BW_RD_ADD6 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD6_REG_RQ_BW_RD_ADD6) >> 0));
			break;
		case 0x1201d8UL: // (PXP2_REG_PSWRQ_BW_ADD7: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_ADD7 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_ADD7 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD7_REG_RQ_BW_WR_ADD7) >> 10)
				, "\n\tRQ_BW_RD_ADD7 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD7_REG_RQ_BW_RD_ADD7) >> 0));
			break;
		case 0x1201dcUL: // (PXP2_REG_PSWRQ_BW_ADD8: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_ADD8 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_ADD8 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD8_REG_RQ_BW_WR_ADD8) >> 10)
				, "\n\tRQ_BW_RD_ADD8 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD8_REG_RQ_BW_RD_ADD8) >> 0));
			break;
		case 0x1201e0UL: // (PXP2_REG_PSWRQ_BW_ADD9: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_ADD9 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_ADD9 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD9_REG_RQ_BW_WR_ADD9) >> 10)
				, "\n\tRQ_BW_RD_ADD9 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD9_REG_RQ_BW_RD_ADD9) >> 0));
			break;
		case 0x1201e4UL: // (PXP2_REG_PSWRQ_BW_ADD10: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_ADD10 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_ADD10 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD10_REG_RQ_BW_WR_ADD10) >> 10)
				, "\n\tRQ_BW_RD_ADD10 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD10_REG_RQ_BW_RD_ADD10) >> 0));
			break;
		case 0x1201e8UL: // (PXP2_REG_PSWRQ_BW_ADD11: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_ADD11 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_ADD11 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD11_REG_RQ_BW_WR_ADD11) >> 10)
				, "\n\tRQ_BW_RD_ADD11 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD11_REG_RQ_BW_RD_ADD11) >> 0));
			break;
		case 0x1201ecUL: // (PXP2_REG_RQ_BW_RD_ADD12: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD12 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ12 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201f0UL: // (PXP2_REG_RQ_BW_RD_ADD13: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD13 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ13 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201f4UL: // (PXP2_REG_RQ_BW_RD_ADD14: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD14 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ14 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201f8UL: // (PXP2_REG_RQ_BW_RD_ADD15: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD15 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ15 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1201fcUL: // (PXP2_REG_RQ_BW_RD_ADD16: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD16 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ16 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120200UL: // (PXP2_REG_RQ_BW_RD_ADD17: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD17 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ17 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120204UL: // (PXP2_REG_RQ_BW_RD_ADD18: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD18 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ18 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120208UL: // (PXP2_REG_RQ_BW_RD_ADD19: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD19 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ19 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12020cUL: // (PXP2_REG_RQ_BW_RD_ADD20: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD20 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ20 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120210UL: // (PXP2_REG_RQ_BW_RD_ADD22: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD22 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ22 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120214UL: // (PXP2_REG_RQ_BW_RD_ADD23: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD23 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ23 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120218UL: // (PXP2_REG_RQ_BW_RD_ADD24: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD24 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ24 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12021cUL: // (PXP2_REG_RQ_BW_RD_ADD25: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD25 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ25 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120220UL: // (PXP2_REG_RQ_BW_RD_ADD26: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD26 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ26 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120224UL: // (PXP2_REG_RQ_BW_RD_ADD27: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD27 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ27 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120228UL: // (PXP2_REG_PSWRQ_BW_ADD28: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_ADD28 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_ADD28 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD28_REG_RQ_BW_WR_ADD28) >> 10)
				, "\n\tRQ_BW_RD_ADD28 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_ADD28_REG_RQ_BW_RD_ADD28) >> 0));
			break;
		case 0x12022cUL: // (PXP2_REG_RQ_BW_WR_ADD29: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_WR_ADD29 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ29 write requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120230UL: // (PXP2_REG_RQ_BW_WR_ADD30: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_WR_ADD30 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ30 write requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120234UL: // (PXP2_REG_RQ_BW_RD_UBOUND0: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND0 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ0 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120238UL: // (PXP2_REG_PSWRQ_BW_UB1: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_UB1 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_UBOUND1 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB1_REG_RQ_BW_WR_UBOUND1) >> 7)
				, "\n\tRQ_BW_RD_UBOUND1 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB1_REG_RQ_BW_RD_UBOUND1) >> 0));
			break;
		case 0x12023cUL: // (PXP2_REG_PSWRQ_BW_UB2: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_UB2 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_UBOUND2 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB2_REG_RQ_BW_WR_UBOUND2) >> 7)
				, "\n\tRQ_BW_RD_UBOUND2 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB2_REG_RQ_BW_RD_UBOUND2) >> 0));
			break;
		case 0x120240UL: // (PXP2_REG_PSWRQ_BW_UB3: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_UB3 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_UBOUND3 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB3_REG_RQ_BW_WR_UBOUND3) >> 7)
				, "\n\tRQ_BW_RD_UBOUND3 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB3_REG_RQ_BW_RD_UBOUND3) >> 0));
			break;
		case 0x120244UL: // (PXP2_REG_RQ_BW_RD_UBOUND4: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND4 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ4 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120248UL: // (PXP2_REG_RQ_BW_RD_UBOUND5: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND5 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ5 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12024cUL: // (PXP2_REG_PSWRQ_BW_UB6: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_UB6 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_UBOUND6 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB6_REG_RQ_BW_WR_UBOUND6) >> 7)
				, "\n\tRQ_BW_RD_UBOUND6 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB6_REG_RQ_BW_RD_UBOUND6) >> 0));
			break;
		case 0x120250UL: // (PXP2_REG_PSWRQ_BW_UB7: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_UB7 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_UBOUND7 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB7_REG_RQ_BW_WR_UBOUND7) >> 7)
				, "\n\tRQ_BW_RD_UBOUND7 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB7_REG_RQ_BW_RD_UBOUND7) >> 0));
			break;
		case 0x120254UL: // (PXP2_REG_PSWRQ_BW_UB8: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_UB8 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_UBOUND8 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB8_REG_RQ_BW_WR_UBOUND8) >> 7)
				, "\n\tRQ_BW_RD_UBOUND8 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB8_REG_RQ_BW_RD_UBOUND8) >> 0));
			break;
		case 0x120258UL: // (PXP2_REG_PSWRQ_BW_UB9: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_UB9 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_UBOUND9 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB9_REG_RQ_BW_WR_UBOUND9) >> 7)
				, "\n\tRQ_BW_RD_UBOUND9 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB9_REG_RQ_BW_RD_UBOUND9) >> 0));
			break;
		case 0x12025cUL: // (PXP2_REG_PSWRQ_BW_UB10: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_UB10 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_UBOUND10 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB10_REG_RQ_BW_WR_UBOUND10) >> 7)
				, "\n\tRQ_BW_RD_UBOUND10 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB10_REG_RQ_BW_RD_UBOUND10) >> 0));
			break;
		case 0x120260UL: // (PXP2_REG_PSWRQ_BW_UB11: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_UB11 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_UBOUND11 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB11_REG_RQ_BW_WR_UBOUND11) >> 7)
				, "\n\tRQ_BW_RD_UBOUND11 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB11_REG_RQ_BW_RD_UBOUND11) >> 0));
			break;
		case 0x120264UL: // (PXP2_REG_RQ_BW_RD_UBOUND12: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND12 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ12 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120268UL: // (PXP2_REG_RQ_BW_RD_UBOUND13: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND13 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ13 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12026cUL: // (PXP2_REG_RQ_BW_RD_UBOUND14: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND14 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ14 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120270UL: // (PXP2_REG_RQ_BW_RD_UBOUND15: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND15 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ15 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120274UL: // (PXP2_REG_RQ_BW_RD_UBOUND16: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND16 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ16 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120278UL: // (PXP2_REG_RQ_BW_RD_UBOUND17: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND17 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ17 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12027cUL: // (PXP2_REG_RQ_BW_RD_UBOUND18: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND18 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ18 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120280UL: // (PXP2_REG_RQ_BW_RD_UBOUND19: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND19 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ19 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120284UL: // (PXP2_REG_RQ_BW_RD_UBOUND20: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND20 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ20 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120288UL: // (PXP2_REG_RQ_BW_RD_UBOUND22: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND22 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ22 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12028cUL: // (PXP2_REG_RQ_BW_RD_UBOUND23: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND23 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ23 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120290UL: // (PXP2_REG_RQ_BW_RD_UBOUND24: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND24 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ24 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120294UL: // (PXP2_REG_RQ_BW_RD_UBOUND25: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND25 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ25 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120298UL: // (PXP2_REG_RQ_BW_RD_UBOUND26: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND26 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ26 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12029cUL: // (PXP2_REG_RQ_BW_RD_UBOUND27: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND27 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ27 read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202a0UL: // (PXP2_REG_PSWRQ_BW_UB28: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_UB28 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_UBOUND28 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB28_REG_RQ_BW_WR_UBOUND28) >> 7)
				, "\n\tRQ_BW_RD_UBOUND28 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_UB28_REG_RQ_BW_RD_UBOUND28) >> 0));
			break;
		case 0x1202a4UL: // (PXP2_REG_RQ_BW_WR_UBOUND29: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_WR_UBOUND29 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202a8UL: // (PXP2_REG_RQ_BW_WR_UBOUND30: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_WR_UBOUND30 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound for VQ30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202acUL: // (PXP2_REG_RQ_BW_RD_L0: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L0 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ0 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202b0UL: // (PXP2_REG_PSWRQ_BW_L1: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_L1 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_RD_L1 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L1_REG_RQ_BW_RD_L1) >> 10)
				, "\n\tRQ_BW_WR_L1 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L1_REG_RQ_BW_WR_L1) >> 0));
			break;
		case 0x1202b4UL: // (PXP2_REG_PSWRQ_BW_L2: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_L2 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_RD_L2 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L2_REG_RQ_BW_RD_L2) >> 10)
				, "\n\tRQ_BW_WR_L2 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L2_REG_RQ_BW_WR_L2) >> 0));
			break;
		case 0x1202b8UL: // (PXP2_REG_PSWRQ_BW_L3: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_L3 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_RD_L3 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L3_REG_RQ_BW_RD_L3) >> 10)
				, "\n\tRQ_BW_WR_L3 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L3_REG_RQ_BW_WR_L3) >> 0));
			break;
		case 0x1202bcUL: // (PXP2_REG_RQ_BW_RD_L4: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L4 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ4 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202c0UL: // (PXP2_REG_RQ_BW_RD_L5: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L5 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ5 Read- currently not used\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202c4UL: // (PXP2_REG_PSWRQ_BW_L6: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_L6 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_L6 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L6_REG_RQ_BW_WR_L6) >> 10)
				, "\n\tRQ_BW_RD_L6 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L6_REG_RQ_BW_RD_L6) >> 0));
			break;
		case 0x1202c8UL: // (PXP2_REG_PSWRQ_BW_L7: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_L7 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_L7 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L7_REG_RQ_BW_WR_L7) >> 10)
				, "\n\tRQ_BW_RD_L7 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L7_REG_RQ_BW_RD_L7) >> 0));
			break;
		case 0x1202ccUL: // (PXP2_REG_PSWRQ_BW_L8: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_L8 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_L8 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L8_REG_RQ_BW_WR_L8) >> 10)
				, "\n\tRQ_BW_RD_L8 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L8_REG_RQ_BW_RD_L8) >> 0));
			break;
		case 0x1202d0UL: // (PXP2_REG_PSWRQ_BW_L9: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_L9 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_L9 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L9_REG_RQ_BW_WR_L9) >> 10)
				, "\n\tRQ_BW_RD_L9 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L9_REG_RQ_BW_RD_L9) >> 0));
			break;
		case 0x1202d4UL: // (PXP2_REG_PSWRQ_BW_L10: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_L10 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_L10 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L10_REG_RQ_BW_WR_L10) >> 10)
				, "\n\tRQ_BW_RD_L10 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L10_REG_RQ_BW_RD_L10) >> 0));
			break;
		case 0x1202d8UL: // (PXP2_REG_PSWRQ_BW_L11: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_L11 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_L11 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L11_REG_RQ_BW_WR_L11) >> 10)
				, "\n\tRQ_BW_RD_L11 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L11_REG_RQ_BW_RD_L11) >> 0));
			break;
		case 0x1202dcUL: // (PXP2_REG_RQ_BW_RD_L12: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L12 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ12 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202e0UL: // (PXP2_REG_RQ_BW_RD_L13: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L13 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ13 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202e4UL: // (PXP2_REG_RQ_BW_RD_L14: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L14 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ14 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202e8UL: // (PXP2_REG_RQ_BW_RD_L15: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L15 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ15 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202ecUL: // (PXP2_REG_RQ_BW_RD_L16: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L16 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ16 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202f0UL: // (PXP2_REG_RQ_BW_RD_L17: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L17 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ17 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202f4UL: // (PXP2_REG_RQ_BW_RD_L18: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L18 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ18 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202f8UL: // (PXP2_REG_RQ_BW_RD_L19: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L19 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ19 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1202fcUL: // (PXP2_REG_RQ_BW_RD_L20: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L20 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ20 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120300UL: // (PXP2_REG_RQ_BW_RD_L22: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L22 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ22 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120304UL: // (PXP2_REG_RQ_BW_RD_L23: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L23 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ23 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120308UL: // (PXP2_REG_RQ_BW_RD_L24: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L24 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ24 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12030cUL: // (PXP2_REG_RQ_BW_RD_L25: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L25 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ25 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120310UL: // (PXP2_REG_RQ_BW_RD_L26: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L26 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ26 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120314UL: // (PXP2_REG_RQ_BW_RD_L27: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L27 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ27 Read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120318UL: // (PXP2_REG_PSWRQ_BW_L28: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_L28 (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_BW_WR_L28 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L28_REG_RQ_BW_WR_L28) >> 10)
				, "\n\tRQ_BW_RD_L28 :"
				, (u32_t)((data & PXP2_PSWRQ_BW_L28_REG_RQ_BW_RD_L28) >> 0));
			break;
		case 0x12031cUL: // (PXP2_REG_RQ_BW_WR_L29: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_WR_L29 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ29 Write requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120320UL: // (PXP2_REG_RQ_BW_WR_L30: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_WR_L30 (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L for VQ30 Write requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120324UL: // (PXP2_REG_PSWRQ_BW_RD: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_RD (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_RD_BW_L :"
				, (u32_t)((data & PXP2_PSWRQ_BW_RD_REG_RQ_RD_BW_L) >> 17)
				, "\n\tRQ_RD_BW_UBOUND :"
				, (u32_t)((data & PXP2_PSWRQ_BW_RD_REG_RQ_RD_BW_UBOUND) >> 10)
				, "\n\tRQ_RD_BW_ADD :"
				, (u32_t)((data & PXP2_PSWRQ_BW_RD_REG_RQ_RD_BW_ADD) >> 0));
			break;
		case 0x120328UL: // (PXP2_REG_PSWRQ_BW_WR: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_WR (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_WR_BW_L :"
				, (u32_t)((data & PXP2_PSWRQ_BW_WR_REG_RQ_WR_BW_L) >> 17)
				, "\n\tRQ_WR_BW_UBOUND :"
				, (u32_t)((data & PXP2_PSWRQ_BW_WR_REG_RQ_WR_BW_UBOUND) >> 10)
				, "\n\tRQ_WR_BW_ADD :"
				, (u32_t)((data & PXP2_PSWRQ_BW_WR_REG_RQ_WR_BW_ADD) >> 0));
			break;
		case 0x12032cUL: // (PXP2_REG_PSWRQ_BW_CREDIT: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWRQ_BW_CREDIT (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_WRITE_CREDIT :"
				, (u32_t)((data & PXP2_PSWRQ_BW_CREDIT_REG_RQ_WRITE_CREDIT) >> 3)
				, "\n\tRQ_READ_CREDIT :"
				, (u32_t)((data & PXP2_PSWRQ_BW_CREDIT_REG_RQ_READ_CREDIT) >> 0));
			break;
		case 0x120330UL: // (PXP2_REG_RQ_DISABLE_INPUTS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DISABLE_INPUTS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When '1'; requests will enter input buffers but wont get \n    out towards the glue\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120334UL: // (PXP2_REG_RQ_PSWRQ_L2P_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_PSWRQ_L2P_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tm input for l2p memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120338UL: // (PXP2_REG_RQ_SLOW_TH: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SLOW_TH (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: when number of free entries in the context ram will be lower\n     than this;the input clients arbiter will work in a slower pace\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12033cUL: // (PXP2_REG_RQ_PDR_LIMIT: ACCESS:RW  DataWidth:0xd ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_PDR_LIMIT (ACCESS:RW  DataWidth:0xd )" \
				"\nDescription: Pending read limiter threshold; in Dwords\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120340UL: // (PXP2_REG_RQ_DBG_HEAD_MUX_SEL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DBG_HEAD_MUX_SEL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: sets which vq head pointer to see out of queues 0-31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120344UL: // (PXP2_REG_RQ_DBG_TAIL_MUX_SEL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DBG_TAIL_MUX_SEL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: sets which vq tail pointer to see out of queues 0-31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120348UL: // (PXP2_REG_WR_USDMDP_TH: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_USDMDP_TH (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: a. When pxp2.wr_th_mode_usdmdp=0 (E1.5-65 mode) should be\n     initialized to (MPS/32); b. When pxp2.wr_th_mode_usdmdp=1 (E1.5-90; enhanced\n     mode) and pxp2.wr_usdmdp_outst_req is different than default (3) should\n     be initialized to (pxp2.wr_usdmdp_outst_req x MPS/32); when pxp2.wr_usdmdp_outst_req\n     is 3 the reset value is the correct configuration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12034cUL: // (PXP2_REG_WR_DMAE_FULL_TH2: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_DMAE_FULL_TH2 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: if Number of entries in the dmae internal fifo is bigger \n    than this number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120350UL: // (PXP2_REG_WR_CDU_FULL_TH2: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_CDU_FULL_TH2 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: if Number of entries in the cdu internal fifo is bigger than\n     this number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120354UL: // (PXP2_REG_WR_USDMDP_FULL_TH2: ACCESS:RW  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_USDMDP_FULL_TH2 (ACCESS:RW  DataWidth:0x9 )" \
				"\nDescription: if Number of entries in the usdmdp internal fifo is bigger\n     than this number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120358UL: // (PXP2_REG_WR_USDMDP_FIFO0_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_USDMDP_FIFO0_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tma input for fifo 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12035cUL: // (PXP2_REG_WR_USDMDP_FIFO0_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_USDMDP_FIFO0_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tmb input for fifo 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120360UL: // (PXP2_REG_WR_USDMDP_FIFO1_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_USDMDP_FIFO1_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tma input for fifo 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120364UL: // (PXP2_REG_WR_USDMDP_FIFO1_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_USDMDP_FIFO1_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tmb input for fifo 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120368UL: // (PXP2_REG_WR_DMAE_TH: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_WR_DMAE_TH (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: This register is not used when pxp2.wr_rev_mode is 1. This\n     register should have value of 63 when pxp2.wr_rev_mode is 0 to disable\n     the threshold mechanism for DMAE. Therefore; 63 is the init value for\n     all modes.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12036cUL: // (PXP2_REG_RD_START_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_START_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Signals the PSWRD block to start initializing internal memories\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120370UL: // (PXP2_REG_RD_INIT_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_INIT_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PSWRD internal memories initialization is done\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120374UL: // (PXP2_REG_RD_DISABLE_INPUTS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_DISABLE_INPUTS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When '1'; inputs to the PSWRD block are ignored\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120378UL: // (PXP2_REG_RD_MAX_BLKS_VQ0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12037cUL: // (PXP2_REG_RD_MAX_BLKS_VQ1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120380UL: // (PXP2_REG_RD_MAX_BLKS_VQ2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120384UL: // (PXP2_REG_RD_MAX_BLKS_VQ3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120388UL: // (PXP2_REG_RD_MAX_BLKS_VQ4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12038cUL: // (PXP2_REG_RD_MAX_BLKS_VQ5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Not used. VQ5 is not used for read\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120390UL: // (PXP2_REG_RD_MAX_BLKS_VQ6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120394UL: // (PXP2_REG_RD_MAX_BLKS_VQ7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120398UL: // (PXP2_REG_RD_MAX_BLKS_VQ8: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ8 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12039cUL: // (PXP2_REG_RD_MAX_BLKS_VQ9: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ9 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203a0UL: // (PXP2_REG_RD_MAX_BLKS_VQ10: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ10 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203a4UL: // (PXP2_REG_RD_MAX_BLKS_VQ11: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ11 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203a8UL: // (PXP2_REG_RD_MAX_BLKS_VQ12: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ12 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203acUL: // (PXP2_REG_RD_MAX_BLKS_VQ13: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ13 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203b0UL: // (PXP2_REG_RD_MAX_BLKS_VQ14: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ14 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203b4UL: // (PXP2_REG_RD_MAX_BLKS_VQ15: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ15 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203b8UL: // (PXP2_REG_RD_MAX_BLKS_VQ16: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ16 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203bcUL: // (PXP2_REG_RD_MAX_BLKS_VQ17: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ17 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203c0UL: // (PXP2_REG_RD_MAX_BLKS_VQ18: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ18 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203c4UL: // (PXP2_REG_RD_MAX_BLKS_VQ19: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ19 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203c8UL: // (PXP2_REG_RD_MAX_BLKS_VQ20: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ20 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203ccUL: // (PXP2_REG_RD_MAX_BLKS_VQ21: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ21 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Not used. VQ21 is not used for read\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203d0UL: // (PXP2_REG_RD_MAX_BLKS_VQ22: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ22 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203d4UL: // (PXP2_REG_RD_MAX_BLKS_VQ23: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ23 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203d8UL: // (PXP2_REG_RD_MAX_BLKS_VQ24: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ24 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203dcUL: // (PXP2_REG_RD_MAX_BLKS_VQ25: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ25 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203e0UL: // (PXP2_REG_RD_MAX_BLKS_VQ26: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ26 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203e4UL: // (PXP2_REG_RD_MAX_BLKS_VQ27: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ27 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203e8UL: // (PXP2_REG_RD_MAX_BLKS_VQ28: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MAX_BLKS_VQ28 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The maximum number of blocks in Tetris Buffer that can be\n     allocated for vq28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203ecUL: // (PXP2_REG_RD_ARB_DELAY: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ARB_DELAY (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Debug only: The arbiter delay. The delivery port waits ARB_DELAY\n     cycles before asserting 'port_is_idle'. This value is based on implementation\n     and should not be changed.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203f0UL: // (PXP2_REG_RD_PBF_IN_SEPARATE_VQ: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_PBF_IN_SEPARATE_VQ (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 1' indicates that the PBF has a separate VQ and uses VQ4. '\n    0' indicates it shares VQ9 with SDM clients. This field should be consistent\n     with ~pbf_registers_pci_voq_id.pci_voq_id  \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203f4UL: // (PXP2_REG_RD_PBF_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_PBF_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: PBF byte swapping mode configuration for master read requests\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203f8UL: // (PXP2_REG_RD_QM_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_QM_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: QM byte swapping mode configuration for master read requests\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1203fcUL: // (PXP2_REG_RD_TM_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TM_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM byte swapping mode configuration for master read requests\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120400UL: // (PXP2_REG_RD_SRC_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_SRC_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: SRC byte swapping mode configuration for master read requests\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120404UL: // (PXP2_REG_RD_CDURD_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_CDURD_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: CDU byte swapping mode configuration for master read requests\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120408UL: // (PXP2_REG_RD_SR_NUM_CFG: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_SR_NUM_CFG (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Debug only: Total number of available PCI read sub-requests\n    . Must be bigger than 1. Normally should not be changed.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12040cUL: // (PXP2_REG_RD_BLK_NUM_CFG: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_BLK_NUM_CFG (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: Total number of available blocks in Tetris Buffer\n    . Must be bigger than 6. Normally should not be changed.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120410UL: // (PXP2_REG_RD_ALMOST_FULL_THR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_THR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Debug only: If more than this Number of entries are used \n    in the clock synchronization FIFO; it asserts the 'almost full' bit. This\n     number is common to all clock synchronization FIFOs. This value is based\n     on implementation and should not be changed.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120414UL: // (PXP2_REG_RD_SR_CNT: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_SR_CNT (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: Debug only: The SR counter - number of unused sub request\n     ids\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120418UL: // (PXP2_REG_RD_BLK_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_BLK_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Debug only: The blocks counter - number of unused block ids\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12041cUL: // (PXP2_REG_RD_PORT_IS_IDLE_0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_PORT_IS_IDLE_0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: Indication if delivery ports are idle\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120420UL: // (PXP2_REG_RD_PORT_IS_IDLE_1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_PORT_IS_IDLE_1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: Indication if delivery ports are idle\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120424UL: // (PXP2_REG_RD_ALMOST_FULL_0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120428UL: // (PXP2_REG_RD_ALMOST_FULL_1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12042cUL: // (PXP2_REG_RD_ALMOST_FULL_2: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_2 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120430UL: // (PXP2_REG_RD_ALMOST_FULL_3: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_3 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120434UL: // (PXP2_REG_RD_ALMOST_FULL_4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120438UL: // (PXP2_REG_RD_ALMOST_FULL_5: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_5 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12043cUL: // (PXP2_REG_RD_ALMOST_FULL_6: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_6 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120440UL: // (PXP2_REG_RD_ALMOST_FULL_7: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_7 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120444UL: // (PXP2_REG_RD_ALMOST_FULL_8: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_8 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120448UL: // (PXP2_REG_RD_ALMOST_FULL_9: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_9 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12044cUL: // (PXP2_REG_RD_ALMOST_FULL_10: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ALMOST_FULL_10 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Debug only: The 'almost full' indication from each fifo (\n    gives indication about backpressure)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120450UL: // (PXP2_REG_RD_SR_MEMD_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_SR_MEMD_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of rd_sr_memd memory. Not used in E65. Increased \n    to 5 bits in E3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120454UL: // (PXP2_REG_RD_TETRIS0_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TETRIS0_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of Tetris Buffer memory. Increased to 5 bits in E3\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120458UL: // (PXP2_REG_RD_TETRIS0_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TETRIS0_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of Tetris Buffer memory. Increased to 5 bits in E3\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12045cUL: // (PXP2_REG_RD_TETRIS1_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TETRIS1_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of Tetris Buffer memory. Increased to 5 bits in E3\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120460UL: // (PXP2_REG_RD_TETRIS1_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TETRIS1_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of Tetris Buffer memory. Increased to 5 bits in E3\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120464UL: // (PXP2_REG_RD_TETRIS2_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TETRIS2_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of Tetris Buffer memory. Increased to 5 bits in E3\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120468UL: // (PXP2_REG_RD_TETRIS2_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TETRIS2_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of Tetris Buffer memory. Increased to 5 bits in E3\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12046cUL: // (PXP2_REG_RD_TETRIS3_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TETRIS3_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of Tetris Buffer memory. Increased to 5 bits in E3\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120470UL: // (PXP2_REG_RD_TETRIS3_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TETRIS3_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of Tetris Buffer memory. Increased to 5 bits in E3\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120474UL: // (PXP2_REG_HST_DISABLE_INPUTS_PCI: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_HST_DISABLE_INPUTS_PCI (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only: When '1'; inputs to the PSWHST block in clk_pci\n     domain are ignored\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120478UL: // (PXP2_REG_HST_HEADER_FIFO_STATUS: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_HST_HEADER_FIFO_STATUS (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: Debug only: Number of used entries in the header FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12047cUL: // (PXP2_REG_HST_DATA_FIFO_STATUS: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_HST_DATA_FIFO_STATUS (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: Debug only: Number of used entries in the data FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120480UL: // (PXP2_REG_DBGSYN_ALMOST_FULL_THR: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_DBGSYN_ALMOST_FULL_THR (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Debug only: If more than this Number of entries are occupied\n     in the dbgsyn clock synchronization FIFO; it does not enable writing \n    to the fifo. This value is based on implementation and should not be changed\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120484UL: // (PXP2_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PXP2\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120488UL: // (PXP2_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PXP2\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select and before the shift)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12048cUL: // (PXP2_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PXP2\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120490UL: // (PXP2_REG_PGL_CONTROL0: ACCESS:RW  DataWidth:0x1a  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_CONTROL0 (ACCESS:RW  DataWidth:0x1a  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPGL_TXW_DP_AFT :"
				, (u32_t)((data & PXP2_PGL_CONTROL0_REG_PGL_TXW_DP_AFT) >> 20)
				, "\n\tPGL_TXW_CC_THRESH :"
				, (u32_t)((data & PXP2_PGL_CONTROL0_REG_PGL_TXW_CC_THRESH) >> 15)
				, "\n\tPGL_TXW_TAG :"
				, (u32_t)((data & PXP2_PGL_CONTROL0_REG_PGL_TXW_TAG) >> 10)
				, "\n\tPGL_L2P_SWAP_MODE :"
				, (u32_t)((data & PXP2_PGL_CONTROL0_REG_PGL_L2P_SWAP_MODE) >> 8)
				, "\n\tPGL_DISABLE_INPUTS :"
				, (u32_t)((data & PXP2_PGL_CONTROL0_REG_PGL_DISABLE_INPUTS) >> 7)
				, "\n\tPGL_CPL_AFT :"
				, (u32_t)((data & PXP2_PGL_CONTROL0_REG_PGL_CPL_AFT) >> 0));
			break;
		case 0x120494UL: // (PXP2_REG_PGL_INT_TSDM_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_TSDM_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for TSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120498UL: // (PXP2_REG_PGL_INT_TSDM_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_TSDM_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for TSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12049cUL: // (PXP2_REG_PGL_INT_TSDM_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_TSDM_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for TSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204a0UL: // (PXP2_REG_PGL_INT_TSDM_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_TSDM_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for TSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204a4UL: // (PXP2_REG_PGL_INT_TSDM_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_TSDM_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for TSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204a8UL: // (PXP2_REG_PGL_INT_TSDM_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_TSDM_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for TSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204acUL: // (PXP2_REG_PGL_INT_TSDM_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_TSDM_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for TSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204b0UL: // (PXP2_REG_PGL_INT_TSDM_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_TSDM_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for TSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204b4UL: // (PXP2_REG_PGL_INT_USDM_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_USDM_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for USDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204b8UL: // (PXP2_REG_PGL_INT_USDM_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_USDM_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for USDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204bcUL: // (PXP2_REG_PGL_INT_USDM_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_USDM_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for USDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204c0UL: // (PXP2_REG_PGL_INT_USDM_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_USDM_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for USDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204c4UL: // (PXP2_REG_PGL_INT_USDM_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_USDM_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for USDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204c8UL: // (PXP2_REG_PGL_INT_USDM_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_USDM_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for USDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204ccUL: // (PXP2_REG_PGL_INT_USDM_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_USDM_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for USDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204d0UL: // (PXP2_REG_PGL_INT_USDM_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_USDM_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for USDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204d4UL: // (PXP2_REG_PGL_INT_XSDM_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_XSDM_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for XSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204d8UL: // (PXP2_REG_PGL_INT_XSDM_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_XSDM_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for XSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204dcUL: // (PXP2_REG_PGL_INT_XSDM_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_XSDM_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for XSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204e0UL: // (PXP2_REG_PGL_INT_XSDM_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_XSDM_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for XSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204e4UL: // (PXP2_REG_PGL_INT_XSDM_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_XSDM_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for XSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204e8UL: // (PXP2_REG_PGL_INT_XSDM_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_XSDM_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for XSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204ecUL: // (PXP2_REG_PGL_INT_XSDM_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_XSDM_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for XSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204f0UL: // (PXP2_REG_PGL_INT_XSDM_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_XSDM_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for XSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204f4UL: // (PXP2_REG_PGL_INT_CSDM_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_CSDM_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for CSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204f8UL: // (PXP2_REG_PGL_INT_CSDM_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_CSDM_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for CSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1204fcUL: // (PXP2_REG_PGL_INT_CSDM_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_CSDM_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for CSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120500UL: // (PXP2_REG_PGL_INT_CSDM_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_CSDM_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for CSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120504UL: // (PXP2_REG_PGL_INT_CSDM_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_CSDM_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for CSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120508UL: // (PXP2_REG_PGL_INT_CSDM_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_CSDM_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for CSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12050cUL: // (PXP2_REG_PGL_INT_CSDM_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_CSDM_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for CSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120510UL: // (PXP2_REG_PGL_INT_CSDM_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_INT_CSDM_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Legacy (type B) PF inbound interrupt table for CSDM: bits\n    [31:16]-mask;bits[15:0]-address. Bits [1:0] must be zero (DW resolution\n     address).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120514UL: // (PXP2_REG_PGL_CONTROL1: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_CONTROL1 (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPGL_EN_INT_CSDM :"
				, (u32_t)((data & PXP2_PGL_CONTROL1_REG_PGL_EN_INT_CSDM) >> 13)
				, "\n\tPGL_EN_INT_XSDM :"
				, (u32_t)((data & PXP2_PGL_CONTROL1_REG_PGL_EN_INT_XSDM) >> 12)
				, "\n\tPGL_EN_INT_USDM :"
				, (u32_t)((data & PXP2_PGL_CONTROL1_REG_PGL_EN_INT_USDM) >> 11)
				, "\n\tPGL_EN_INT_TSDM :"
				, (u32_t)((data & PXP2_PGL_CONTROL1_REG_PGL_EN_INT_TSDM) >> 10)
				, "\n\tPGL_TGTWR_MLENGTH :"
				, (u32_t)((data & PXP2_PGL_CONTROL1_REG_PGL_TGTWR_MLENGTH) >> 0));
			break;
		case 0x120518UL: // (PXP2_REG_PGL_DQ_PREFIX2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_DQ_PREFIX2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: debug only\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12051cUL: // (PXP2_REG_PGL_PCIE_REPLAY_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PCIE_REPLAY_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of pcie replay memory. Increased to 5 bits in E3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120520UL: // (PXP2_REG_PGL_DEBUG: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_DEBUG (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPGL_DISABLE :"
				, (u32_t)((data & PXP2_PGL_DEBUG_REG_PGL_DISABLE) >> 2)
				, "\n\tPGL_TXW_RELAX :"
				, (u32_t)((data & PXP2_PGL_DEBUG_REG_PGL_TXW_RELAX) >> 1)
				, "\n\tPGL_TXR_RELAX :"
				, (u32_t)((data & PXP2_PGL_DEBUG_REG_PGL_TXR_RELAX) >> 0));
			break;
		case 0x120524UL: // (PXP2_REG_PGL_PCIE: ACCESS:RW  DataWidth:0x8  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PCIE (ACCESS:RW  DataWidth:0x8  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120528UL: // (PXP2_REG_PGL_TXR_CDTS: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_TXR_CDTS (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: debug only\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12052cUL: // (PXP2_REG_PGL_TXW_CDTS: ACCESS:R  DataWidth:0x15 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_TXW_CDTS (ACCESS:R  DataWidth:0x15 )" \
				"\nDescription: debug only\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120530UL: // (PXP2_REG_PGL_MOT: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_MOT (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: debug only\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120554UL: // (PXP2_REG_PGL_EXP_ROM_ADDR: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_EXP_ROM_ADDR (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: the address to be read from expansion rom (address is in \n    bytes according to read packet from host)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12055cUL: // (PXP2_REG_PGL_CORE_DEBUG: ACCESS:RW  DataWidth:0xb  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_CORE_DEBUG (ACCESS:RW  DataWidth:0xb  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPGL_TXARB_SP :"
				, (u32_t)((data & PXP2_PGL_CORE_DEBUG_REG_PGL_TXARB_SP) >> 10)
				, "\n\tPGL_PARITY_MODE :"
				, (u32_t)((data & PXP2_PGL_CORE_DEBUG_REG_PGL_PARITY_MODE) >> 9)
				, "\n\tPGL_CORE_DEBUG_SEL_2 :"
				, (u32_t)((data & PXP2_PGL_CORE_DEBUG_REG_PGL_CORE_DEBUG_SEL_2) >> 5)
				, "\n\tPGL_CORE_DEBUG_SEL_1 :"
				, (u32_t)((data & PXP2_PGL_CORE_DEBUG_REG_PGL_CORE_DEBUG_SEL_1) >> 1)
				, "\n\tPGL_1US_DISABLE :"
				, (u32_t)((data & PXP2_PGL_CORE_DEBUG_REG_PGL_1US_DISABLE) >> 0));
			break;
		case 0x120560UL: // (PXP2_REG_PGL_PM_STATUS: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PM_STATUS (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: this field provides the following debug information: {Link_L23\n    ;pm_state1[1:0];pm_state0[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120564UL: // (PXP2_REG_PGL_WRITE_BLOCKED: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_WRITE_BLOCKED (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: this bit indicates that a write request was blocked because\n     of bus_master_en was deasserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120568UL: // (PXP2_REG_PGL_READ_BLOCKED: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_READ_BLOCKED (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: this bit indicates that a read request was blocked because\n     of bus_master_en was deasserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12056cUL: // (PXP2_REG_PXP2_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_XSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_WR_XSDM_UNDERFLOW) >> 31)
				, "\n\tWR_CSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_WR_CSDM_UNDERFLOW) >> 30)
				, "\n\tWR_TSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_WR_TSDM_UNDERFLOW) >> 29)
				, "\n\tWR_USDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_WR_USDM_UNDERFLOW) >> 28)
				, "\n\tWR_SRC_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_WR_SRC_UNDERFLOW) >> 27)
				, "\n\tWR_QM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_WR_QM_UNDERFLOW) >> 26)
				, "\n\tWR_TM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_WR_TM_UNDERFLOW) >> 25)
				, "\n\tPGL_WRITE_BLOCKED :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_PGL_WRITE_BLOCKED) >> 24)
				, "\n\tPGL_READ_BLOCKED :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_PGL_READ_BLOCKED) >> 23)
				, "\n\tPGL_PCIE_ATTN :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_PGL_PCIE_ATTN) >> 22)
				, "\n\tPGL_CPL_ECRC :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_PGL_CPL_ECRC) >> 21)
				, "\n\tPGL_CPL_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_PGL_CPL_OF) >> 20)
				, "\n\tPGL_CPL_AFT :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_PGL_CPL_AFT) >> 19)
				, "\n\tPGL_TXW_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_PGL_TXW_OF) >> 18)
				, "\n\tPGL_CPL_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_PGL_CPL_ERR) >> 17)
				, "\n\tHST_CPL_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_HST_CPL_FIFO_ERR) >> 16)
				, "\n\tHST_DATA_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_HST_DATA_FIFO_ERR) >> 15)
				, "\n\tHST_HEADER_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_HST_HEADER_FIFO_ERR) >> 14)
				, "\n\tRD_COMPLETION_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RD_COMPLETION_ERR) >> 13)
				, "\n\tRD_PUSH_PBF_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RD_PUSH_PBF_ERROR) >> 12)
				, "\n\tRD_PUSH_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RD_PUSH_ERROR) >> 11)
				, "\n\tRD_BLK_FIFO_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RD_BLK_FIFO_ERROR) >> 10)
				, "\n\tRD_SR_FIFO_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RD_SR_FIFO_ERROR) >> 9)
				, "\n\tWR_PGLUE_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_WR_PGLUE_UNDERFLOW) >> 8)
				, "\n\tWR_PGLUE_EOP_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_WR_PGLUE_EOP_ERROR) >> 7)
				, "\n\tRQ_FREE_LIST_EMPTY :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RQ_FREE_LIST_EMPTY) >> 6)
				, "\n\tRQ_L2P_VIOLATION_2 :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RQ_L2P_VIOLATION_2) >> 5)
				, "\n\tRQ_L2P_VIOLATION_1 :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RQ_L2P_VIOLATION_1) >> 4)
				, "\n\tRQ_PHYADDR_FIFO_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RQ_PHYADDR_FIFO_OF) >> 3)
				, "\n\tRQ_WDFIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RQ_WDFIFO_OVERFLOW) >> 2)
				, "\n\tRQ_L2P_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_RQ_L2P_FIFO_OVERFLOW) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x120570UL: // (PXP2_REG_PXP2_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_XSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_WR_XSDM_UNDERFLOW) >> 31)
				, "\n\tWR_CSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_WR_CSDM_UNDERFLOW) >> 30)
				, "\n\tWR_TSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_WR_TSDM_UNDERFLOW) >> 29)
				, "\n\tWR_USDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_WR_USDM_UNDERFLOW) >> 28)
				, "\n\tWR_SRC_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_WR_SRC_UNDERFLOW) >> 27)
				, "\n\tWR_QM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_WR_QM_UNDERFLOW) >> 26)
				, "\n\tWR_TM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_WR_TM_UNDERFLOW) >> 25)
				, "\n\tPGL_WRITE_BLOCKED :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_PGL_WRITE_BLOCKED) >> 24)
				, "\n\tPGL_READ_BLOCKED :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_PGL_READ_BLOCKED) >> 23)
				, "\n\tPGL_PCIE_ATTN :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_PGL_PCIE_ATTN) >> 22)
				, "\n\tPGL_CPL_ECRC :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_PGL_CPL_ECRC) >> 21)
				, "\n\tPGL_CPL_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_PGL_CPL_OF) >> 20)
				, "\n\tPGL_CPL_AFT :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_PGL_CPL_AFT) >> 19)
				, "\n\tPGL_TXW_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_PGL_TXW_OF) >> 18)
				, "\n\tPGL_CPL_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_PGL_CPL_ERR) >> 17)
				, "\n\tHST_CPL_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_HST_CPL_FIFO_ERR) >> 16)
				, "\n\tHST_DATA_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_HST_DATA_FIFO_ERR) >> 15)
				, "\n\tHST_HEADER_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_HST_HEADER_FIFO_ERR) >> 14)
				, "\n\tRD_COMPLETION_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RD_COMPLETION_ERR) >> 13)
				, "\n\tRD_PUSH_PBF_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RD_PUSH_PBF_ERROR) >> 12)
				, "\n\tRD_PUSH_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RD_PUSH_ERROR) >> 11)
				, "\n\tRD_BLK_FIFO_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RD_BLK_FIFO_ERROR) >> 10)
				, "\n\tRD_SR_FIFO_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RD_SR_FIFO_ERROR) >> 9)
				, "\n\tWR_PGLUE_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_WR_PGLUE_UNDERFLOW) >> 8)
				, "\n\tWR_PGLUE_EOP_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_WR_PGLUE_EOP_ERROR) >> 7)
				, "\n\tRQ_FREE_LIST_EMPTY :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RQ_FREE_LIST_EMPTY) >> 6)
				, "\n\tRQ_L2P_VIOLATION_2 :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RQ_L2P_VIOLATION_2) >> 5)
				, "\n\tRQ_L2P_VIOLATION_1 :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RQ_L2P_VIOLATION_1) >> 4)
				, "\n\tRQ_PHYADDR_FIFO_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RQ_PHYADDR_FIFO_OF) >> 3)
				, "\n\tRQ_WDFIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RQ_WDFIFO_OVERFLOW) >> 2)
				, "\n\tRQ_L2P_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_RQ_L2P_FIFO_OVERFLOW) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x120574UL: // (PXP2_REG_PXP2_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_XSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_WR_XSDM_UNDERFLOW) >> 31)
				, "\n\tWR_CSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_WR_CSDM_UNDERFLOW) >> 30)
				, "\n\tWR_TSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_WR_TSDM_UNDERFLOW) >> 29)
				, "\n\tWR_USDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_WR_USDM_UNDERFLOW) >> 28)
				, "\n\tWR_SRC_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_WR_SRC_UNDERFLOW) >> 27)
				, "\n\tWR_QM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_WR_QM_UNDERFLOW) >> 26)
				, "\n\tWR_TM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_WR_TM_UNDERFLOW) >> 25)
				, "\n\tPGL_WRITE_BLOCKED :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_PGL_WRITE_BLOCKED) >> 24)
				, "\n\tPGL_READ_BLOCKED :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_PGL_READ_BLOCKED) >> 23)
				, "\n\tPGL_PCIE_ATTN :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_PGL_PCIE_ATTN) >> 22)
				, "\n\tPGL_CPL_ECRC :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_PGL_CPL_ECRC) >> 21)
				, "\n\tPGL_CPL_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_PGL_CPL_OF) >> 20)
				, "\n\tPGL_CPL_AFT :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_PGL_CPL_AFT) >> 19)
				, "\n\tPGL_TXW_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_PGL_TXW_OF) >> 18)
				, "\n\tPGL_CPL_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_PGL_CPL_ERR) >> 17)
				, "\n\tHST_CPL_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_HST_CPL_FIFO_ERR) >> 16)
				, "\n\tHST_DATA_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_HST_DATA_FIFO_ERR) >> 15)
				, "\n\tHST_HEADER_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_HST_HEADER_FIFO_ERR) >> 14)
				, "\n\tRD_COMPLETION_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RD_COMPLETION_ERR) >> 13)
				, "\n\tRD_PUSH_PBF_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RD_PUSH_PBF_ERROR) >> 12)
				, "\n\tRD_PUSH_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RD_PUSH_ERROR) >> 11)
				, "\n\tRD_BLK_FIFO_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RD_BLK_FIFO_ERROR) >> 10)
				, "\n\tRD_SR_FIFO_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RD_SR_FIFO_ERROR) >> 9)
				, "\n\tWR_PGLUE_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_WR_PGLUE_UNDERFLOW) >> 8)
				, "\n\tWR_PGLUE_EOP_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_WR_PGLUE_EOP_ERROR) >> 7)
				, "\n\tRQ_FREE_LIST_EMPTY :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RQ_FREE_LIST_EMPTY) >> 6)
				, "\n\tRQ_L2P_VIOLATION_2 :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RQ_L2P_VIOLATION_2) >> 5)
				, "\n\tRQ_L2P_VIOLATION_1 :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RQ_L2P_VIOLATION_1) >> 4)
				, "\n\tRQ_PHYADDR_FIFO_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RQ_PHYADDR_FIFO_OF) >> 3)
				, "\n\tRQ_WDFIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RQ_WDFIFO_OVERFLOW) >> 2)
				, "\n\tRQ_L2P_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_RQ_L2P_FIFO_OVERFLOW) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x120578UL: // (PXP2_REG_PXP2_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_XSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_WR_XSDM_UNDERFLOW) >> 31)
				, "\n\tWR_CSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_WR_CSDM_UNDERFLOW) >> 30)
				, "\n\tWR_TSDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_WR_TSDM_UNDERFLOW) >> 29)
				, "\n\tWR_USDM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_WR_USDM_UNDERFLOW) >> 28)
				, "\n\tWR_SRC_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_WR_SRC_UNDERFLOW) >> 27)
				, "\n\tWR_QM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_WR_QM_UNDERFLOW) >> 26)
				, "\n\tWR_TM_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_WR_TM_UNDERFLOW) >> 25)
				, "\n\tPGL_WRITE_BLOCKED :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_PGL_WRITE_BLOCKED) >> 24)
				, "\n\tPGL_READ_BLOCKED :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_PGL_READ_BLOCKED) >> 23)
				, "\n\tPGL_PCIE_ATTN :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_PGL_PCIE_ATTN) >> 22)
				, "\n\tPGL_CPL_ECRC :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_PGL_CPL_ECRC) >> 21)
				, "\n\tPGL_CPL_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_PGL_CPL_OF) >> 20)
				, "\n\tPGL_CPL_AFT :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_PGL_CPL_AFT) >> 19)
				, "\n\tPGL_TXW_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_PGL_TXW_OF) >> 18)
				, "\n\tPGL_CPL_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_PGL_CPL_ERR) >> 17)
				, "\n\tHST_CPL_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_HST_CPL_FIFO_ERR) >> 16)
				, "\n\tHST_DATA_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_HST_DATA_FIFO_ERR) >> 15)
				, "\n\tHST_HEADER_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_HST_HEADER_FIFO_ERR) >> 14)
				, "\n\tRD_COMPLETION_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RD_COMPLETION_ERR) >> 13)
				, "\n\tRD_PUSH_PBF_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RD_PUSH_PBF_ERROR) >> 12)
				, "\n\tRD_PUSH_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RD_PUSH_ERROR) >> 11)
				, "\n\tRD_BLK_FIFO_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RD_BLK_FIFO_ERROR) >> 10)
				, "\n\tRD_SR_FIFO_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RD_SR_FIFO_ERROR) >> 9)
				, "\n\tWR_PGLUE_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_WR_PGLUE_UNDERFLOW) >> 8)
				, "\n\tWR_PGLUE_EOP_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_WR_PGLUE_EOP_ERROR) >> 7)
				, "\n\tRQ_FREE_LIST_EMPTY :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RQ_FREE_LIST_EMPTY) >> 6)
				, "\n\tRQ_L2P_VIOLATION_2 :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RQ_L2P_VIOLATION_2) >> 5)
				, "\n\tRQ_L2P_VIOLATION_1 :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RQ_L2P_VIOLATION_1) >> 4)
				, "\n\tRQ_PHYADDR_FIFO_OF :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RQ_PHYADDR_FIFO_OF) >> 3)
				, "\n\tRQ_WDFIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RQ_WDFIFO_OVERFLOW) >> 2)
				, "\n\tRQ_L2P_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_RQ_L2P_FIFO_OVERFLOW) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x12057cUL: // (PXP2_REG_PXP2_PRTY_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_PRTY_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_FIRST_BLK :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_FIRST_BLK) >> 31)
				, "\n\tRD_TETRIS_BUFFER8 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_TETRIS_BUFFER8) >> 30)
				, "\n\tRD_TETRIS_BUFFER7 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_TETRIS_BUFFER7) >> 29)
				, "\n\tRD_TETRIS_BUFFER6 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_TETRIS_BUFFER6) >> 28)
				, "\n\tRD_TETRIS_BUFFER5 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_TETRIS_BUFFER5) >> 27)
				, "\n\tRD_TETRIS_BUFFER4 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_TETRIS_BUFFER4) >> 26)
				, "\n\tRD_TETRIS_BUFFER3 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_TETRIS_BUFFER3) >> 25)
				, "\n\tRD_TETRIS_BUFFER2 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_TETRIS_BUFFER2) >> 24)
				, "\n\tRD_TETRIS_BUFFER1 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_TETRIS_BUFFER1) >> 23)
				, "\n\tRD_SR_DELIVERY2 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_SR_DELIVERY2) >> 22)
				, "\n\tRD_SR_DELIVERY1 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_SR_DELIVERY1) >> 21)
				, "\n\tRD_COMP_CTX :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_COMP_CTX) >> 20)
				, "\n\tRD_SR_COMPLETION :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_SR_COMPLETION) >> 19)
				, "\n\tRD_VQ_HEAD :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RD_VQ_HEAD) >> 18)
				, "\n\tWR_HC_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_HC_FIFO_PRTY) >> 17)
				, "\n\tWR_DBG_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_DBG_FIFO_PRTY) >> 16)
				, "\n\tWR_DMAE_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_DMAE_FIFO_PRTY) >> 15)
				, "\n\tWR_CDUWR_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_CDUWR_FIFO_PRTY) >> 14)
				, "\n\tWR_CSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_CSDM_FIFO_PRTY) >> 13)
				, "\n\tWR_TSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_TSDM_FIFO_PRTY) >> 12)
				, "\n\tWR_XSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_XSDM_FIFO_PRTY) >> 11)
				, "\n\tWR_USDMDP_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_USDMDP_FIFO_PRTY) >> 10)
				, "\n\tWR_USDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_USDM_FIFO_PRTY) >> 9)
				, "\n\tWR_TM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_TM_FIFO_PRTY) >> 8)
				, "\n\tWR_QM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_QM_FIFO_PRTY) >> 7)
				, "\n\tWR_SRC_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_WR_SRC_FIFO_PRTY) >> 6)
				, "\n\tRQ_OFFCHIP_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RQ_OFFCHIP_PARITY_ERR) >> 5)
				, "\n\tRQ_ONCHIP_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RQ_ONCHIP_PARITY_ERR) >> 4)
				, "\n\tRQ_UFIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RQ_UFIFO_PARITY_ERR) >> 3)
				, "\n\tRQ_HOQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RQ_HOQ_PARITY_ERR) >> 2)
				, "\n\tRQ_CXR_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_RQ_CXR_PARITY_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_0_REG_PARITY) >> 0));
			break;
		case 0x120580UL: // (PXP2_REG_PXP2_PRTY_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_PRTY_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_FIRST_BLK :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_FIRST_BLK) >> 31)
				, "\n\tRD_TETRIS_BUFFER8 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_TETRIS_BUFFER8) >> 30)
				, "\n\tRD_TETRIS_BUFFER7 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_TETRIS_BUFFER7) >> 29)
				, "\n\tRD_TETRIS_BUFFER6 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_TETRIS_BUFFER6) >> 28)
				, "\n\tRD_TETRIS_BUFFER5 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_TETRIS_BUFFER5) >> 27)
				, "\n\tRD_TETRIS_BUFFER4 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_TETRIS_BUFFER4) >> 26)
				, "\n\tRD_TETRIS_BUFFER3 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_TETRIS_BUFFER3) >> 25)
				, "\n\tRD_TETRIS_BUFFER2 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_TETRIS_BUFFER2) >> 24)
				, "\n\tRD_TETRIS_BUFFER1 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_TETRIS_BUFFER1) >> 23)
				, "\n\tRD_SR_DELIVERY2 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_SR_DELIVERY2) >> 22)
				, "\n\tRD_SR_DELIVERY1 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_SR_DELIVERY1) >> 21)
				, "\n\tRD_COMP_CTX :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_COMP_CTX) >> 20)
				, "\n\tRD_SR_COMPLETION :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_SR_COMPLETION) >> 19)
				, "\n\tRD_VQ_HEAD :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RD_VQ_HEAD) >> 18)
				, "\n\tWR_HC_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_HC_FIFO_PRTY) >> 17)
				, "\n\tWR_DBG_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_DBG_FIFO_PRTY) >> 16)
				, "\n\tWR_DMAE_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_DMAE_FIFO_PRTY) >> 15)
				, "\n\tWR_CDUWR_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_CDUWR_FIFO_PRTY) >> 14)
				, "\n\tWR_CSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_CSDM_FIFO_PRTY) >> 13)
				, "\n\tWR_TSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_TSDM_FIFO_PRTY) >> 12)
				, "\n\tWR_XSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_XSDM_FIFO_PRTY) >> 11)
				, "\n\tWR_USDMDP_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_USDMDP_FIFO_PRTY) >> 10)
				, "\n\tWR_USDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_USDM_FIFO_PRTY) >> 9)
				, "\n\tWR_TM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_TM_FIFO_PRTY) >> 8)
				, "\n\tWR_QM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_QM_FIFO_PRTY) >> 7)
				, "\n\tWR_SRC_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_WR_SRC_FIFO_PRTY) >> 6)
				, "\n\tRQ_OFFCHIP_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RQ_OFFCHIP_PARITY_ERR) >> 5)
				, "\n\tRQ_ONCHIP_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RQ_ONCHIP_PARITY_ERR) >> 4)
				, "\n\tRQ_UFIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RQ_UFIFO_PARITY_ERR) >> 3)
				, "\n\tRQ_HOQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RQ_HOQ_PARITY_ERR) >> 2)
				, "\n\tRQ_CXR_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_RQ_CXR_PARITY_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_0_REG_PARITY) >> 0));
			break;
		case 0x120584UL: // (PXP2_REG_PXP2_PRTY_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_PRTY_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_FIRST_BLK :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_FIRST_BLK) >> 31)
				, "\n\tRD_TETRIS_BUFFER8 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_TETRIS_BUFFER8) >> 30)
				, "\n\tRD_TETRIS_BUFFER7 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_TETRIS_BUFFER7) >> 29)
				, "\n\tRD_TETRIS_BUFFER6 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_TETRIS_BUFFER6) >> 28)
				, "\n\tRD_TETRIS_BUFFER5 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_TETRIS_BUFFER5) >> 27)
				, "\n\tRD_TETRIS_BUFFER4 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_TETRIS_BUFFER4) >> 26)
				, "\n\tRD_TETRIS_BUFFER3 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_TETRIS_BUFFER3) >> 25)
				, "\n\tRD_TETRIS_BUFFER2 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_TETRIS_BUFFER2) >> 24)
				, "\n\tRD_TETRIS_BUFFER1 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_TETRIS_BUFFER1) >> 23)
				, "\n\tRD_SR_DELIVERY2 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_SR_DELIVERY2) >> 22)
				, "\n\tRD_SR_DELIVERY1 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_SR_DELIVERY1) >> 21)
				, "\n\tRD_COMP_CTX :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_COMP_CTX) >> 20)
				, "\n\tRD_SR_COMPLETION :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_SR_COMPLETION) >> 19)
				, "\n\tRD_VQ_HEAD :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RD_VQ_HEAD) >> 18)
				, "\n\tWR_HC_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_HC_FIFO_PRTY) >> 17)
				, "\n\tWR_DBG_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_DBG_FIFO_PRTY) >> 16)
				, "\n\tWR_DMAE_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_DMAE_FIFO_PRTY) >> 15)
				, "\n\tWR_CDUWR_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_CDUWR_FIFO_PRTY) >> 14)
				, "\n\tWR_CSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_CSDM_FIFO_PRTY) >> 13)
				, "\n\tWR_TSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_TSDM_FIFO_PRTY) >> 12)
				, "\n\tWR_XSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_XSDM_FIFO_PRTY) >> 11)
				, "\n\tWR_USDMDP_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_USDMDP_FIFO_PRTY) >> 10)
				, "\n\tWR_USDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_USDM_FIFO_PRTY) >> 9)
				, "\n\tWR_TM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_TM_FIFO_PRTY) >> 8)
				, "\n\tWR_QM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_QM_FIFO_PRTY) >> 7)
				, "\n\tWR_SRC_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_WR_SRC_FIFO_PRTY) >> 6)
				, "\n\tRQ_OFFCHIP_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RQ_OFFCHIP_PARITY_ERR) >> 5)
				, "\n\tRQ_ONCHIP_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RQ_ONCHIP_PARITY_ERR) >> 4)
				, "\n\tRQ_UFIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RQ_UFIFO_PARITY_ERR) >> 3)
				, "\n\tRQ_HOQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RQ_HOQ_PARITY_ERR) >> 2)
				, "\n\tRQ_CXR_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_RQ_CXR_PARITY_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_0_REG_PARITY) >> 0));
			break;
		case 0x120588UL: // (PXP2_REG_PXP2_PRTY_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_PRTY_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_FIRST_BLK :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_FIRST_BLK) >> 31)
				, "\n\tRD_TETRIS_BUFFER8 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_TETRIS_BUFFER8) >> 30)
				, "\n\tRD_TETRIS_BUFFER7 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_TETRIS_BUFFER7) >> 29)
				, "\n\tRD_TETRIS_BUFFER6 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_TETRIS_BUFFER6) >> 28)
				, "\n\tRD_TETRIS_BUFFER5 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_TETRIS_BUFFER5) >> 27)
				, "\n\tRD_TETRIS_BUFFER4 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_TETRIS_BUFFER4) >> 26)
				, "\n\tRD_TETRIS_BUFFER3 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_TETRIS_BUFFER3) >> 25)
				, "\n\tRD_TETRIS_BUFFER2 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_TETRIS_BUFFER2) >> 24)
				, "\n\tRD_TETRIS_BUFFER1 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_TETRIS_BUFFER1) >> 23)
				, "\n\tRD_SR_DELIVERY2 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_SR_DELIVERY2) >> 22)
				, "\n\tRD_SR_DELIVERY1 :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_SR_DELIVERY1) >> 21)
				, "\n\tRD_COMP_CTX :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_COMP_CTX) >> 20)
				, "\n\tRD_SR_COMPLETION :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_SR_COMPLETION) >> 19)
				, "\n\tRD_VQ_HEAD :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RD_VQ_HEAD) >> 18)
				, "\n\tWR_HC_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_HC_FIFO_PRTY) >> 17)
				, "\n\tWR_DBG_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_DBG_FIFO_PRTY) >> 16)
				, "\n\tWR_DMAE_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_DMAE_FIFO_PRTY) >> 15)
				, "\n\tWR_CDUWR_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_CDUWR_FIFO_PRTY) >> 14)
				, "\n\tWR_CSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_CSDM_FIFO_PRTY) >> 13)
				, "\n\tWR_TSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_TSDM_FIFO_PRTY) >> 12)
				, "\n\tWR_XSDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_XSDM_FIFO_PRTY) >> 11)
				, "\n\tWR_USDMDP_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_USDMDP_FIFO_PRTY) >> 10)
				, "\n\tWR_USDM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_USDM_FIFO_PRTY) >> 9)
				, "\n\tWR_TM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_TM_FIFO_PRTY) >> 8)
				, "\n\tWR_QM_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_QM_FIFO_PRTY) >> 7)
				, "\n\tWR_SRC_FIFO_PRTY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_WR_SRC_FIFO_PRTY) >> 6)
				, "\n\tRQ_OFFCHIP_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RQ_OFFCHIP_PARITY_ERR) >> 5)
				, "\n\tRQ_ONCHIP_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RQ_ONCHIP_PARITY_ERR) >> 4)
				, "\n\tRQ_UFIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RQ_UFIFO_PARITY_ERR) >> 3)
				, "\n\tRQ_HOQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RQ_HOQ_PARITY_ERR) >> 2)
				, "\n\tRQ_CXR_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_RQ_CXR_PARITY_ERR) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_0_REG_PARITY) >> 0));
			break;
		case 0x12058cUL: // (PXP2_REG_PXP2_PRTY_STS_1: ACCESS:R  DataWidth:0x19 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_PRTY_STS_1 (ACCESS:R  DataWidth:0x19 )" \
				"\nDescription: Parity register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_DBG_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_DBG_REQ_PARITY_ERR) >> 24)
				, "\n\tRQ_HC_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_HC_REQ_PARITY_ERR) >> 23)
				, "\n\tRQ_XSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_XSDM_REQ_PARITY_ERR) >> 22)
				, "\n\tRQ_DMAE_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_DMAE_REQ_PARITY_ERR) >> 21)
				, "\n\tRQ_CDURD_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_CDURD_REQ_PARITY_ERR) >> 20)
				, "\n\tRQ_CDUWR_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_CDUWR_REQ_PARITY_ERR) >> 19)
				, "\n\tRQ_CSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_CSDM_REQ_PARITY_ERR) >> 18)
				, "\n\tRQ_TSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_TSDM_REQ_PARITY_ERR) >> 17)
				, "\n\tRQ_USDMDP_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_USDMDP_REQ_PARITY_ERR) >> 16)
				, "\n\tRQ_USDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_USDM_REQ_PARITY_ERR) >> 15)
				, "\n\tRQ_SRC_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_SRC_REQ_PARITY_ERR) >> 14)
				, "\n\tRQ_TM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_TM_REQ_PARITY_ERR) >> 13)
				, "\n\tRQ_QM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_QM_REQ_PARITY_ERR) >> 12)
				, "\n\tRQ_PBF_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_PBF_REQ_PARITY_ERR) >> 11)
				, "\n\tRQ_ICPL_FIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_ICPL_FIFO_PARITY_ERR) >> 10)
				, "\n\tRQ_TREQ_FIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RQ_TREQ_FIFO_PARITY_ERR) >> 9)
				, "\n\tHST_IREQ_SYNC_FIFO :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_HST_IREQ_SYNC_FIFO) >> 8)
				, "\n\tRD_ATC_ENTRY_ID :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RD_ATC_ENTRY_ID) >> 7)
				, "\n\tPGL_REPLAY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_PGL_REPLAY) >> 6)
				, "\n\tPGL_TXW :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_PGL_TXW) >> 5)
				, "\n\tPGL_CPL :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_PGL_CPL) >> 4)
				, "\n\tHST_CPL_SYNC_FIFO :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_HST_CPL_SYNC_FIFO) >> 3)
				, "\n\tRD_BLOCK_FREE_LIST :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RD_BLOCK_FREE_LIST) >> 2)
				, "\n\tRD_SR_FREE_LIST :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RD_SR_FREE_LIST) >> 1)
				, "\n\tRD_NEXT_BLK_PTRS :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_1_REG_RD_NEXT_BLK_PTRS) >> 0));
			break;
		case 0x120590UL: // (PXP2_REG_PXP2_PRTY_STS_CLR_1: ACCESS:RC  DataWidth:0x19 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_PRTY_STS_CLR_1 (ACCESS:RC  DataWidth:0x19 )" \
				"\nDescription: Parity register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_DBG_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_DBG_REQ_PARITY_ERR) >> 24)
				, "\n\tRQ_HC_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_HC_REQ_PARITY_ERR) >> 23)
				, "\n\tRQ_XSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_XSDM_REQ_PARITY_ERR) >> 22)
				, "\n\tRQ_DMAE_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_DMAE_REQ_PARITY_ERR) >> 21)
				, "\n\tRQ_CDURD_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_CDURD_REQ_PARITY_ERR) >> 20)
				, "\n\tRQ_CDUWR_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_CDUWR_REQ_PARITY_ERR) >> 19)
				, "\n\tRQ_CSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_CSDM_REQ_PARITY_ERR) >> 18)
				, "\n\tRQ_TSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_TSDM_REQ_PARITY_ERR) >> 17)
				, "\n\tRQ_USDMDP_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_USDMDP_REQ_PARITY_ERR) >> 16)
				, "\n\tRQ_USDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_USDM_REQ_PARITY_ERR) >> 15)
				, "\n\tRQ_SRC_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_SRC_REQ_PARITY_ERR) >> 14)
				, "\n\tRQ_TM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_TM_REQ_PARITY_ERR) >> 13)
				, "\n\tRQ_QM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_QM_REQ_PARITY_ERR) >> 12)
				, "\n\tRQ_PBF_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_PBF_REQ_PARITY_ERR) >> 11)
				, "\n\tRQ_ICPL_FIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_ICPL_FIFO_PARITY_ERR) >> 10)
				, "\n\tRQ_TREQ_FIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RQ_TREQ_FIFO_PARITY_ERR) >> 9)
				, "\n\tHST_IREQ_SYNC_FIFO :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_HST_IREQ_SYNC_FIFO) >> 8)
				, "\n\tRD_ATC_ENTRY_ID :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RD_ATC_ENTRY_ID) >> 7)
				, "\n\tPGL_REPLAY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_PGL_REPLAY) >> 6)
				, "\n\tPGL_TXW :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_PGL_TXW) >> 5)
				, "\n\tPGL_CPL :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_PGL_CPL) >> 4)
				, "\n\tHST_CPL_SYNC_FIFO :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_HST_CPL_SYNC_FIFO) >> 3)
				, "\n\tRD_BLOCK_FREE_LIST :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RD_BLOCK_FREE_LIST) >> 2)
				, "\n\tRD_SR_FREE_LIST :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RD_SR_FREE_LIST) >> 1)
				, "\n\tRD_NEXT_BLK_PTRS :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_CLR_1_REG_RD_NEXT_BLK_PTRS) >> 0));
			break;
		case 0x120594UL: // (PXP2_REG_PXP2_PRTY_STS_WR_1: ACCESS:WR  DataWidth:0x19 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_PRTY_STS_WR_1 (ACCESS:WR  DataWidth:0x19 )" \
				"\nDescription: Parity register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_DBG_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_DBG_REQ_PARITY_ERR) >> 24)
				, "\n\tRQ_HC_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_HC_REQ_PARITY_ERR) >> 23)
				, "\n\tRQ_XSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_XSDM_REQ_PARITY_ERR) >> 22)
				, "\n\tRQ_DMAE_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_DMAE_REQ_PARITY_ERR) >> 21)
				, "\n\tRQ_CDURD_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_CDURD_REQ_PARITY_ERR) >> 20)
				, "\n\tRQ_CDUWR_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_CDUWR_REQ_PARITY_ERR) >> 19)
				, "\n\tRQ_CSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_CSDM_REQ_PARITY_ERR) >> 18)
				, "\n\tRQ_TSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_TSDM_REQ_PARITY_ERR) >> 17)
				, "\n\tRQ_USDMDP_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_USDMDP_REQ_PARITY_ERR) >> 16)
				, "\n\tRQ_USDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_USDM_REQ_PARITY_ERR) >> 15)
				, "\n\tRQ_SRC_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_SRC_REQ_PARITY_ERR) >> 14)
				, "\n\tRQ_TM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_TM_REQ_PARITY_ERR) >> 13)
				, "\n\tRQ_QM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_QM_REQ_PARITY_ERR) >> 12)
				, "\n\tRQ_PBF_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_PBF_REQ_PARITY_ERR) >> 11)
				, "\n\tRQ_ICPL_FIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_ICPL_FIFO_PARITY_ERR) >> 10)
				, "\n\tRQ_TREQ_FIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RQ_TREQ_FIFO_PARITY_ERR) >> 9)
				, "\n\tHST_IREQ_SYNC_FIFO :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_HST_IREQ_SYNC_FIFO) >> 8)
				, "\n\tRD_ATC_ENTRY_ID :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RD_ATC_ENTRY_ID) >> 7)
				, "\n\tPGL_REPLAY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_PGL_REPLAY) >> 6)
				, "\n\tPGL_TXW :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_PGL_TXW) >> 5)
				, "\n\tPGL_CPL :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_PGL_CPL) >> 4)
				, "\n\tHST_CPL_SYNC_FIFO :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_HST_CPL_SYNC_FIFO) >> 3)
				, "\n\tRD_BLOCK_FREE_LIST :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RD_BLOCK_FREE_LIST) >> 2)
				, "\n\tRD_SR_FREE_LIST :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RD_SR_FREE_LIST) >> 1)
				, "\n\tRD_NEXT_BLK_PTRS :"
				, (u32_t)((data & PXP2_PXP2_PRTY_STS_WR_1_REG_RD_NEXT_BLK_PTRS) >> 0));
			break;
		case 0x120598UL: // (PXP2_REG_PXP2_PRTY_MASK_1: ACCESS:RW  DataWidth:0x19 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_PRTY_MASK_1 (ACCESS:RW  DataWidth:0x19 )" \
				"\nDescription: Parity mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRQ_DBG_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_DBG_REQ_PARITY_ERR) >> 24)
				, "\n\tRQ_HC_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_HC_REQ_PARITY_ERR) >> 23)
				, "\n\tRQ_XSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_XSDM_REQ_PARITY_ERR) >> 22)
				, "\n\tRQ_DMAE_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_DMAE_REQ_PARITY_ERR) >> 21)
				, "\n\tRQ_CDURD_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_CDURD_REQ_PARITY_ERR) >> 20)
				, "\n\tRQ_CDUWR_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_CDUWR_REQ_PARITY_ERR) >> 19)
				, "\n\tRQ_CSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_CSDM_REQ_PARITY_ERR) >> 18)
				, "\n\tRQ_TSDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_TSDM_REQ_PARITY_ERR) >> 17)
				, "\n\tRQ_USDMDP_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_USDMDP_REQ_PARITY_ERR) >> 16)
				, "\n\tRQ_USDM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_USDM_REQ_PARITY_ERR) >> 15)
				, "\n\tRQ_SRC_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_SRC_REQ_PARITY_ERR) >> 14)
				, "\n\tRQ_TM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_TM_REQ_PARITY_ERR) >> 13)
				, "\n\tRQ_QM_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_QM_REQ_PARITY_ERR) >> 12)
				, "\n\tRQ_PBF_REQ_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_PBF_REQ_PARITY_ERR) >> 11)
				, "\n\tRQ_ICPL_FIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_ICPL_FIFO_PARITY_ERR) >> 10)
				, "\n\tRQ_TREQ_FIFO_PARITY_ERR :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RQ_TREQ_FIFO_PARITY_ERR) >> 9)
				, "\n\tHST_IREQ_SYNC_FIFO :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_HST_IREQ_SYNC_FIFO) >> 8)
				, "\n\tRD_ATC_ENTRY_ID :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RD_ATC_ENTRY_ID) >> 7)
				, "\n\tPGL_REPLAY :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_PGL_REPLAY) >> 6)
				, "\n\tPGL_TXW :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_PGL_TXW) >> 5)
				, "\n\tPGL_CPL :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_PGL_CPL) >> 4)
				, "\n\tHST_CPL_SYNC_FIFO :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_HST_CPL_SYNC_FIFO) >> 3)
				, "\n\tRD_BLOCK_FREE_LIST :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RD_BLOCK_FREE_LIST) >> 2)
				, "\n\tRD_SR_FREE_LIST :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RD_SR_FREE_LIST) >> 1)
				, "\n\tRD_NEXT_BLK_PTRS :"
				, (u32_t)((data & PXP2_PXP2_PRTY_MASK_1_REG_RD_NEXT_BLK_PTRS) >> 0));
			break;
		case 0x12059cUL: // (PXP2_REG_RD_INIT_MEMS_WITH_ZERO: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_INIT_MEMS_WITH_ZERO (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only: '1' indicates that the following memories should\n     be initialized with zeros or NULL values after 'start_init': tetris buffer\n    ; completion context memory; vq head memory; SR memory for delivery; first\n     block pointers memory. This is a fix done in E1.5 to fix a parity error\n     bug on these memories. '0' indicates not to initialize these memories\n    ; so 'start_init' will behave as in E1A0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205a0UL: // (PXP2_REG_PGL_EXP_ROM_FUNC: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_EXP_ROM_FUNC (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: the function number of the expansion rom that is being accessed\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205a4UL: // (PXP2_REG_PGL_VPD_FUNC: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_VPD_FUNC (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Unused.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205a8UL: // (PXP2_REG_PGL_TAGS_LIMIT: ACCESS:RW  DataWidth:0x7  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_TAGS_LIMIT (ACCESS:RW  DataWidth:0x7  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPGL_MAX_TAGS_DISABLE :"
				, (u32_t)((data & PXP2_PGL_TAGS_LIMIT_REG_PGL_MAX_TAGS_DISABLE) >> 6)
				, "\n\tPGL_MAX_TAGS :"
				, (u32_t)((data & PXP2_PGL_TAGS_LIMIT_REG_PGL_MAX_TAGS) >> 0));
			break;
		case 0x1205acUL: // (PXP2_REG_RQ_L2P_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_L2P_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: will determine how the logical address is calculated; 0: \n    as in E1; 1:with new algorithm\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205b0UL: // (PXP2_REG_RQ_DRAM_ALIGN: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DRAM_ALIGN (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Determines alignment of write SRs when a request is split\n     into several SRs. 0 - 8B aligned. 1 - 64B aligned. 2 - 128B aligned. \n    3 - 256B aligned. 4 - 512B aligned.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205b4UL: // (PXP2_REG_RQ_ILT_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ILT_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: when '0' ILT logic will work as in A0; otherwise B0; for \n    back compatibility needs; Note that different registers are used per mode\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205b8UL: // (PXP2_REG_RQ_ENDIANITY_00: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ENDIANITY_00 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Requests from all SDM's and DMAE with endian mode 0 will \n    receive the endian mode indicated here\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205bcUL: // (PXP2_REG_RQ_ENDIANITY_01: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ENDIANITY_01 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Requests from all SDM's and DMAE with endian mode 1 will \n    receive the endian mode indicated here\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205c0UL: // (PXP2_REG_RQ_ENDIANITY_02: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ENDIANITY_02 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Requests from all SDM's and DMAE with endian mode 2 will \n    receive the endian mode indicated here\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205c4UL: // (PXP2_REG_RQ_ENDIANITY_03: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ENDIANITY_03 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Requests from all SDM's and DMAE with endian mode 3 will \n    receive the endian mode indicated here\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205c8UL: // (PXP2_REG_WR_HC_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_HC_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205ccUL: // (PXP2_REG_WR_USDM_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_USDM_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205d0UL: // (PXP2_REG_WR_CSDM_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_CSDM_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205d4UL: // (PXP2_REG_WR_TSDM_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_TSDM_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205d8UL: // (PXP2_REG_WR_XSDM_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_XSDM_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205dcUL: // (PXP2_REG_WR_QM_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_QM_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205e0UL: // (PXP2_REG_WR_TM_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_TM_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205e4UL: // (PXP2_REG_WR_SRC_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_SRC_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205e8UL: // (PXP2_REG_WR_DBG_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_DBG_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205ecUL: // (PXP2_REG_WR_DMAE_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_DMAE_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205f0UL: // (PXP2_REG_WR_CDU_MPS: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_CDU_MPS (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: 0 - 128B;1 - 256B;2 - 512B;3 - 1024B; when the payload in\n     the buffer reaches this number has_payload will be asserted. 1024B is\n     not a real MPS; it is a way of indicating that the client needs to wait\n     for EOP before asserting has_payload. Register should be initialized \n    according to has_payload value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205f4UL: // (PXP2_REG_WR_DBG_FULL_TH2: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_DBG_FULL_TH2 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: if Number of entries in the dbg internal fifo is bigger than\n     this number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205f8UL: // (PXP2_REG_RQ_BW_RD_ADD_TREQ: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_ADD_TREQ (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth addition to VQ TREQ read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1205fcUL: // (PXP2_REG_RQ_BW_RD_UBOUND_TREQ: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_UBOUND_TREQ (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bandwidth upper bound to VQ TREQ read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120600UL: // (PXP2_REG_RQ_BW_RD_L_TREQ: ACCESS:RW  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_BW_RD_L_TREQ (ACCESS:RW  DataWidth:0xa )" \
				"\nDescription: Bandwidth Typical L to VQ TREQ read requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120604UL: // (PXP2_REG_WR_TH_MODE_USDMDP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_TH_MODE_USDMDP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For USDMDP - 0 - B0 mode (E1.5-65nm) - asserts has_payload\n     when MPS bytes are in the buffer and no outstanding sub-request grants\n     or EOP arrived; 1 - enhanced mode (E1.5-90nm) - assert has_payload when\n     (~pxp2.wr_usdmdp_th x 32) bytes are in the buffer or EOP arrived. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120608UL: // (PXP2_REG_PXP2_INT_STS_1: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_INT_STS_1 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_PGLUE_LSR_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_WR_PGLUE_LSR_ERROR) >> 16)
				, "\n\tRQ_BACK2BACK_ATC_RESPONSE :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_RQ_BACK2BACK_ATC_RESPONSE) >> 15)
				, "\n\tRQ_ICPL_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_RQ_ICPL_FIFO_OVERFLOW) >> 14)
				, "\n\tRQ_ICPL_FIFO_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_RQ_ICPL_FIFO_UNDERFLOW) >> 13)
				, "\n\tRQ_TREQ_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_RQ_TREQ_FIFO_OVERFLOW) >> 12)
				, "\n\tRQ_TREQ_FIFO_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_RQ_TREQ_FIFO_UNDERFLOW) >> 11)
				, "\n\tRD_BLK_CNT_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_RD_BLK_CNT_UNDERFLOW) >> 10)
				, "\n\tRD_SR_CNT_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_RD_SR_CNT_UNDERFLOW) >> 9)
				, "\n\tHST_IREQ_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_HST_IREQ_FIFO_ERR) >> 8)
				, "\n\tRQ_CORE_WDONE_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_RQ_CORE_WDONE_OVERFLOW) >> 7)
				, "\n\tRQ_L2P_VF_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_RQ_L2P_VF_ERR) >> 6)
				, "\n\tRQ_ELT_ADDR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_RQ_ELT_ADDR) >> 5)
				, "\n\tWR_HC_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_WR_HC_UNDERFLOW) >> 4)
				, "\n\tWR_DMAE_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_WR_DMAE_UNDERFLOW) >> 3)
				, "\n\tWR_DBG_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_WR_DBG_UNDERFLOW) >> 2)
				, "\n\tWR_CDU_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_WR_CDU_UNDERFLOW) >> 1)
				, "\n\tWR_USDMDP_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_1_REG_WR_USDMDP_UNDERFLOW) >> 0));
			break;
		case 0x12060cUL: // (PXP2_REG_PXP2_INT_STS_CLR_1: ACCESS:RC  DataWidth:0x11 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0x11 )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_PGLUE_LSR_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_WR_PGLUE_LSR_ERROR) >> 16)
				, "\n\tRQ_BACK2BACK_ATC_RESPONSE :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_RQ_BACK2BACK_ATC_RESPONSE) >> 15)
				, "\n\tRQ_ICPL_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_RQ_ICPL_FIFO_OVERFLOW) >> 14)
				, "\n\tRQ_ICPL_FIFO_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_RQ_ICPL_FIFO_UNDERFLOW) >> 13)
				, "\n\tRQ_TREQ_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_RQ_TREQ_FIFO_OVERFLOW) >> 12)
				, "\n\tRQ_TREQ_FIFO_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_RQ_TREQ_FIFO_UNDERFLOW) >> 11)
				, "\n\tRD_BLK_CNT_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_RD_BLK_CNT_UNDERFLOW) >> 10)
				, "\n\tRD_SR_CNT_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_RD_SR_CNT_UNDERFLOW) >> 9)
				, "\n\tHST_IREQ_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_HST_IREQ_FIFO_ERR) >> 8)
				, "\n\tRQ_CORE_WDONE_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_RQ_CORE_WDONE_OVERFLOW) >> 7)
				, "\n\tRQ_L2P_VF_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_RQ_L2P_VF_ERR) >> 6)
				, "\n\tRQ_ELT_ADDR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_RQ_ELT_ADDR) >> 5)
				, "\n\tWR_HC_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_WR_HC_UNDERFLOW) >> 4)
				, "\n\tWR_DMAE_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_WR_DMAE_UNDERFLOW) >> 3)
				, "\n\tWR_DBG_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_WR_DBG_UNDERFLOW) >> 2)
				, "\n\tWR_CDU_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_WR_CDU_UNDERFLOW) >> 1)
				, "\n\tWR_USDMDP_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_CLR_1_REG_WR_USDMDP_UNDERFLOW) >> 0));
			break;
		case 0x120610UL: // (PXP2_REG_PXP2_INT_STS_WR_1: ACCESS:WR  DataWidth:0x11 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_INT_STS_WR_1 (ACCESS:WR  DataWidth:0x11 )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_PGLUE_LSR_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_WR_PGLUE_LSR_ERROR) >> 16)
				, "\n\tRQ_BACK2BACK_ATC_RESPONSE :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_RQ_BACK2BACK_ATC_RESPONSE) >> 15)
				, "\n\tRQ_ICPL_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_RQ_ICPL_FIFO_OVERFLOW) >> 14)
				, "\n\tRQ_ICPL_FIFO_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_RQ_ICPL_FIFO_UNDERFLOW) >> 13)
				, "\n\tRQ_TREQ_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_RQ_TREQ_FIFO_OVERFLOW) >> 12)
				, "\n\tRQ_TREQ_FIFO_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_RQ_TREQ_FIFO_UNDERFLOW) >> 11)
				, "\n\tRD_BLK_CNT_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_RD_BLK_CNT_UNDERFLOW) >> 10)
				, "\n\tRD_SR_CNT_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_RD_SR_CNT_UNDERFLOW) >> 9)
				, "\n\tHST_IREQ_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_HST_IREQ_FIFO_ERR) >> 8)
				, "\n\tRQ_CORE_WDONE_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_RQ_CORE_WDONE_OVERFLOW) >> 7)
				, "\n\tRQ_L2P_VF_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_RQ_L2P_VF_ERR) >> 6)
				, "\n\tRQ_ELT_ADDR :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_RQ_ELT_ADDR) >> 5)
				, "\n\tWR_HC_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_WR_HC_UNDERFLOW) >> 4)
				, "\n\tWR_DMAE_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_WR_DMAE_UNDERFLOW) >> 3)
				, "\n\tWR_DBG_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_WR_DBG_UNDERFLOW) >> 2)
				, "\n\tWR_CDU_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_WR_CDU_UNDERFLOW) >> 1)
				, "\n\tWR_USDMDP_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_STS_WR_1_REG_WR_USDMDP_UNDERFLOW) >> 0));
			break;
		case 0x120614UL: // (PXP2_REG_PXP2_INT_MASK_1: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_INT_MASK_1 (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_PGLUE_LSR_ERROR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_WR_PGLUE_LSR_ERROR) >> 16)
				, "\n\tRQ_BACK2BACK_ATC_RESPONSE :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_RQ_BACK2BACK_ATC_RESPONSE) >> 15)
				, "\n\tRQ_ICPL_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_RQ_ICPL_FIFO_OVERFLOW) >> 14)
				, "\n\tRQ_ICPL_FIFO_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_RQ_ICPL_FIFO_UNDERFLOW) >> 13)
				, "\n\tRQ_TREQ_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_RQ_TREQ_FIFO_OVERFLOW) >> 12)
				, "\n\tRQ_TREQ_FIFO_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_RQ_TREQ_FIFO_UNDERFLOW) >> 11)
				, "\n\tRD_BLK_CNT_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_RD_BLK_CNT_UNDERFLOW) >> 10)
				, "\n\tRD_SR_CNT_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_RD_SR_CNT_UNDERFLOW) >> 9)
				, "\n\tHST_IREQ_FIFO_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_HST_IREQ_FIFO_ERR) >> 8)
				, "\n\tRQ_CORE_WDONE_OVERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_RQ_CORE_WDONE_OVERFLOW) >> 7)
				, "\n\tRQ_L2P_VF_ERR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_RQ_L2P_VF_ERR) >> 6)
				, "\n\tRQ_ELT_ADDR :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_RQ_ELT_ADDR) >> 5)
				, "\n\tWR_HC_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_WR_HC_UNDERFLOW) >> 4)
				, "\n\tWR_DMAE_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_WR_DMAE_UNDERFLOW) >> 3)
				, "\n\tWR_DBG_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_WR_DBG_UNDERFLOW) >> 2)
				, "\n\tWR_CDU_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_WR_CDU_UNDERFLOW) >> 1)
				, "\n\tWR_USDMDP_UNDERFLOW :"
				, (u32_t)((data & PXP2_PXP2_INT_MASK_1_REG_WR_USDMDP_UNDERFLOW) >> 0));
			break;
		case 0x120618UL: // (PXP2_REG_WR_USDMDP_OUTST_REQ: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_USDMDP_OUTST_REQ (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Number of USDMDP PSWRQ outstanding sub-request grants that\n     weren't yet read by PGLUE.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12066cUL: // (PXP2_REG_RQ_ELT_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ELT_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If 1 ILT failiue will not result in ELT access; An interrupt\n     will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120670UL: // (PXP2_REG_WR_REV_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_REV_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For non-USDMDP clients. 0 - working in A0 mode - assert has_payload\n     only when EOP arrived;1 - working in B0 mode - assert has_payload according\n     to pxp2.wr_th_usdmdp or to EOP arrived.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120674UL: // (PXP2_REG_PGL_PRETEND_FUNC_F0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PRETEND_FUNC_F0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: this field allows one function to pretend being another function\n     when accessing any BAR mapped resource within the device. the value of\n     the field is the number of the function that will be accessed effectively\n    . after software write to this bit it must read it in order to know that\n     the new value is updated. Bits [15] - force. Bits [14] - path ID. Bits\n     [13:10] - Reserved.  Bits [9:4] - VFID. Bits [3] - VF valid. Bits [2:\n    0] - PFID. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120678UL: // (PXP2_REG_PGL_PRETEND_FUNC_F1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PRETEND_FUNC_F1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: this field allows one function to pretend being another function\n     when accessing any BAR mapped resource within the device. the value of\n     the field is the number of the function that will be accessed effectively\n    . after software write to this bit it must read it in order to know that\n     the new value is updated. Bits [15] - force. Bits [14] - path ID. Bits\n     [13:10] - Reserved.  Bits [9:4] - VFID. Bits [3] - VF valid. Bits [2:\n    0] - PFID. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12067cUL: // (PXP2_REG_PGL_PRETEND_FUNC_F2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PRETEND_FUNC_F2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: this field allows one function to pretend being another function\n     when accessing any BAR mapped resource within the device. the value of\n     the field is the number of the function that will be accessed effectively\n    . after software write to this bit it must read it in order to know that\n     the new value is updated. Bits [15] - force. Bits [14] - path ID. Bits\n     [13:10] - Reserved.  Bits [9:4] - VFID. Bits [3] - VF valid. Bits [2:\n    0] - PFID. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120680UL: // (PXP2_REG_PGL_PRETEND_FUNC_F3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PRETEND_FUNC_F3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: this field allows one function to pretend being another function\n     when accessing any BAR mapped resource within the device. the value of\n     the field is the number of the function that will be accessed effectively\n    . after software write to this bit it must read it in order to know that\n     the new value is updated. Bits [15] - force. Bits [14] - path ID. Bits\n     [13:10] - Reserved.  Bits [9:4] - VFID. Bits [3] - VF valid. Bits [2:\n    0] - PFID. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120684UL: // (PXP2_REG_PGL_PRETEND_FUNC_F4: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PRETEND_FUNC_F4 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: this field allows one function to pretend being another function\n     when accessing any BAR mapped resource within the device. the value of\n     the field is the number of the function that will be accessed effectively\n    . after software write to this bit it must read it in order to know that\n     the new value is updated. Bits [15] - force. Bits [14] - path ID. Bits\n     [13:10] - Reserved.  Bits [9:4] - VFID. Bits [3] - VF valid. Bits [2:\n    0] - PFID. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120688UL: // (PXP2_REG_PGL_PRETEND_FUNC_F5: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PRETEND_FUNC_F5 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: this field allows one function to pretend being another function\n     when accessing any BAR mapped resource within the device. the value of\n     the field is the number of the function that will be accessed effectively\n    . after software write to this bit it must read it in order to know that\n     the new value is updated. Bits [15] - force. Bits [14] - path ID. Bits\n     [13:10] - Reserved.  Bits [9:4] - VFID. Bits [3] - VF valid. Bits [2:\n    0] - PFID. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12068cUL: // (PXP2_REG_PGL_PRETEND_FUNC_F6: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PRETEND_FUNC_F6 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: this field allows one function to pretend being another function\n     when accessing any BAR mapped resource within the device. the value of\n     the field is the number of the function that will be accessed effectively\n    . after software write to this bit it must read it in order to know that\n     the new value is updated. Bits [15] - force. Bits [14] - path ID. Bits\n     [13:10] - Reserved.  Bits [9:4] - VFID. Bits [3] - VF valid. Bits [2:\n    0] - PFID. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120690UL: // (PXP2_REG_PGL_PRETEND_FUNC_F7: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_PRETEND_FUNC_F7 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: this field allows one function to pretend being another function\n     when accessing any BAR mapped resource within the device. the value of\n     the field is the number of the function that will be accessed effectively\n    . after software write to this bit it must read it in order to know that\n     the new value is updated. Bits [15] - force. Bits [14] - path ID. Bits\n     [13:10] - Reserved.  Bits [9:4] - VFID. Bits [3] - VF valid. Bits [2:\n    0] - PFID. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120694UL: // (PXP2_REG_RQ_LOW_FREE_BYP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LOW_FREE_BYP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: when 1;there cannot be more than 3 pswrq_garb_gnt for reads\n     within 8 cycles; this can prevent the low free blocks bug\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120698UL: // (PXP2_REG_RQ_CXR_RAM0_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CXR_RAM0_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits for cxr ram0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12069cUL: // (PXP2_REG_RQ_CXR_RAM1_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CXR_RAM1_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits for cxr ram1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206a0UL: // (PXP2_REG_RQ_CLOSE_GATE_RD_CLIENT_EN: ACCESS:RW  DataWidth:0x1d ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CLOSE_GATE_RD_CLIENT_EN (ACCESS:RW  DataWidth:0x1d )" \
				"\nDescription: BW counter rd clients that are enabled during close the gates\n     flow (bit per rd vq and bit for treq). In E2 only the msbit should be\n     set (treq)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206a8UL: // (PXP2_REG_RQ_ATC_CSDM_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_CSDM_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206acUL: // (PXP2_REG_WR_DMAE_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_DMAE_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Bits [1:0] are for fifo 0;Bits [3:2] are for fifo 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206b0UL: // (PXP2_REG_PXP2_ECO_RSRV: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_ECO_RSRV (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_ECO_RESERVED :"
				, (u32_t)((data & PXP2_PXP2_ECO_RSRV_REG_RD_ECO_RESERVED) >> 12)
				, "\n\tWR_ECO_RESERVED :"
				, (u32_t)((data & PXP2_PXP2_ECO_RSRV_REG_WR_ECO_RESERVED) >> 6)
				, "\n\tRQ_ECO_RESERVED :"
				, (u32_t)((data & PXP2_PXP2_ECO_RSRV_REG_RQ_ECO_RESERVED) >> 0));
			break;
		case 0x1206b4UL: // (PXP2_REG_WR_HAS_PAYLOAD: ACCESS:R  DataWidth:0xe ) 
			sprintf(disp_str, "\nPXP2_REG_WR_HAS_PAYLOAD (ACCESS:R  DataWidth:0xe )" \
				"\nDescription: has payload indication for all clients; bit decoding - 0 \n    USDM; 1 CSDM; 2 XSDM; 3 TSDM; 4 PBF always 0; 5 QM; 6 TM; 7 SRC; 8 CDURD\n     always 0; 9 DMAE; 10 USDMDP; 11 HC; 12 CDUWR; 13 DBG\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206b8UL: // (PXP2_REG_RQ_VQ_RD_DISABLE: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ_RD_DISABLE (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: vq read disable as wdone was not received yet for the wr \n    request that was sent {vq1 ; vq2 ; vq3 ; vq6 ; vq7 ; vq8 ; vq9 ; vq10 ; \n    vq11 ; vq28}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206bcUL: // (PXP2_REG_RQ_QC_REG1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_REG1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206c0UL: // (PXP2_REG_RQ_QC_REG2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_REG2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206c4UL: // (PXP2_REG_RQ_QC_VIQ_1ENTRY: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_1ENTRY (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206c8UL: // (PXP2_REG_RQ_QC_HOQ_IS_LOGICAL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_HOQ_IS_LOGICAL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206ccUL: // (PXP2_REG_RQ_QC_VIQ_TAIL_V: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_TAIL_V (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206d0UL: // (PXP2_REG_RQ_QC_VIQ_HEAD_V: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_HEAD_V (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206d4UL: // (PXP2_REG_RQ_QC_VIQ_31_28_TAIL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_31_28_TAIL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206d8UL: // (PXP2_REG_RQ_QC_VIQ_27_24_TAIL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_27_24_TAIL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206dcUL: // (PXP2_REG_RQ_QC_VIQ_23_20_TAIL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_23_20_TAIL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206e0UL: // (PXP2_REG_RQ_QC_VIQ_19_16_TAIL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_19_16_TAIL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206e4UL: // (PXP2_REG_RQ_QC_VIQ_15_12_TAIL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_15_12_TAIL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206e8UL: // (PXP2_REG_RQ_QC_VIQ_11_8_TAIL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_11_8_TAIL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206ecUL: // (PXP2_REG_RQ_QC_VIQ_7_4_TAIL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_7_4_TAIL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206f0UL: // (PXP2_REG_RQ_QC_VIQ_3_0_TAIL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_3_0_TAIL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206f4UL: // (PXP2_REG_RQ_QC_VIQ_31_28_HEAD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_31_28_HEAD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206f8UL: // (PXP2_REG_RQ_QC_VIQ_27_24_HEAD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_27_24_HEAD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206fcUL: // (PXP2_REG_RQ_QC_VIQ_23_20_HEAD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_23_20_HEAD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120700UL: // (PXP2_REG_RQ_QC_VIQ_19_16_HEAD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_19_16_HEAD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120704UL: // (PXP2_REG_RQ_QC_VIQ_15_12_HEAD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_15_12_HEAD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120708UL: // (PXP2_REG_RQ_QC_VIQ_11_8_HEAD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_11_8_HEAD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12070cUL: // (PXP2_REG_RQ_QC_VIQ_7_4_HEAD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_7_4_HEAD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120710UL: // (PXP2_REG_RQ_QC_VIQ_3_0_HEAD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_VIQ_3_0_HEAD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120714UL: // (PXP2_REG_RQ_QC_CMG_VQ_31_24_FUNC: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_CMG_VQ_31_24_FUNC (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120718UL: // (PXP2_REG_RQ_QC_CMG_VQ_23_16_FUNC: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_CMG_VQ_23_16_FUNC (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12071cUL: // (PXP2_REG_RQ_QC_CMG_VQ_15_8_FUNC: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_CMG_VQ_15_8_FUNC (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120720UL: // (PXP2_REG_RQ_QC_CMG_VQ_7_0_FUNC: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QC_CMG_VQ_7_0_FUNC (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120724UL: // (PXP2_REG_RQ_CMG_NEED_4K_ALIGN: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG_NEED_4K_ALIGN (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: need 4k alignment; bit per VQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120728UL: // (PXP2_REG_RQ_CMG: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Status signals in pswrq_cmg module\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12072cUL: // (PXP2_REG_RQ_CMG_HAS_PAYLOAD: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG_HAS_PAYLOAD (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: has payload indication for the VQs with more than one client\n    : {vq11 ; vq10 ; vq9 ; vq8 ; vq7 ; vq6}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120730UL: // (PXP2_REG_RQ_CMG_S2_VQ_VAL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG_S2_VQ_VAL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: bit per VQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120734UL: // (PXP2_REG_RQ_CMG_S1_VQ_VAL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG_S1_VQ_VAL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: bit per VQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120738UL: // (PXP2_REG_RQ_CMG_VQ_RDWR_N: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG_VQ_RDWR_N (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: {vq28 ; vq11 ; vq10 ; vq9 ; vq8 ; vq7 ; vq6 ; vq5 ; vq3 ; \n    vq2 ; vq1}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12073cUL: // (PXP2_REG_RQ_CMG_VQ_CLIENT_ID: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG_VQ_CLIENT_ID (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: 4 bits per client: {vq11 client_id ; vq10 client_id; vq9 \n    client_id; vq8 client_id; vq7 client_id; vq6 client_id}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120740UL: // (PXP2_REG_RQ_CMG_VQ_1ST_SR: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG_VQ_1ST_SR (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current sub request is the 1st in the request (bit per vq\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120744UL: // (PXP2_REG_RQ_CMG_VQ_LAST_SR: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG_VQ_LAST_SR (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: current sub request is the last in the request (bit per vq\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120748UL: // (PXP2_REG_RQ_GARB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_GARB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Status signals in pswrq_garb module\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12074cUL: // (PXP2_REG_RQ_GARB_S1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_GARB_S1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Status signals in pswrq_garb_s1 module: bits 15:0 - i_rd_arbiter\n     instance ; Bits 31:16 - i_wr_arbiter instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120750UL: // (PXP2_REG_RQ_GARB_S1_BW_RD_STS: ACCESS:R  DataWidth:0x1c ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_GARB_S1_BW_RD_STS (ACCESS:R  DataWidth:0x1c )" \
				"\nDescription: bandwidth status for rd arbiter in s1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120754UL: // (PXP2_REG_RQ_GARB_S1_BW_WR_STS: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_GARB_S1_BW_WR_STS (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: bandwidth status for wr arbiter in s1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120758UL: // (PXP2_REG_RQ_CMG_GARB_HOQ_RDWR_N: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG_GARB_HOQ_RDWR_N (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: bit per VQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12075cUL: // (PXP2_REG_RQ_CMG_GARB_HOQ_STS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CMG_GARB_HOQ_STS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: bit per VQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120760UL: // (PXP2_REG_RQ_GARB_CTRL: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_GARB_CTRL (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Status signals in pswrq_garb_ctrl module:\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120764UL: // (PXP2_REG_RQ_GARB_SBMT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_GARB_SBMT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Status signals in pswrq_garb_sbmt module: {3b0 ; rd_is_pending\n     ; 3b0 ; rd_p_th_ok ; 3b0 ; pending_rd_arr_en ; 3b0 ; rd_gnt_disable ; \n    3b0 ; blk_mbs2_th_n_ok ; 2b0 ; rd_gnt_cnt[1:0] ; pswrd_pswrq_blk_cnt[7\n    :0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120768UL: // (PXP2_REG_RQ_PSWRD_PSWRQ_BLK_PER_VQ: ACCESS:R  DataWidth:0x1d ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_PSWRD_PSWRQ_BLK_PER_VQ (ACCESS:R  DataWidth:0x1d )" \
				"\nDescription: Available blocks in pswrd reader buffer (per vq)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12076cUL: // (PXP2_REG_RD_TM_65: ACCESS:RW  DataWidth:0x8  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TM_65 (ACCESS:RW  DataWidth:0x8  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_COMP_CTX_TM :"
				, (u32_t)((data & PXP2_RD_TM_65_REG_RD_COMP_CTX_TM) >> 6)
				, "\n\tRD_SR_MEMC_TM :"
				, (u32_t)((data & PXP2_RD_TM_65_REG_RD_SR_MEMC_TM) >> 4)
				, "\n\tRD_NEXT_BLK_PTR_TM :"
				, (u32_t)((data & PXP2_RD_TM_65_REG_RD_NEXT_BLK_PTR_TM) >> 2)
				, "\n\tRD_BLK_FREE_LIST_TM :"
				, (u32_t)((data & PXP2_RD_TM_65_REG_RD_BLK_FREE_LIST_TM) >> 0));
			break;
		case 0x120770UL: // (PXP2_REG_RD_MASK_ERROR_TO_CLIENTS: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPXP2_REG_RD_MASK_ERROR_TO_CLIENTS (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Debug only: '1' indicates that error indication is masked\n     towards the corresponding client.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120774UL: // (PXP2_REG_RD_CONF11: ACCESS:RW  DataWidth:0x12  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_RD_CONF11 (ACCESS:RW  DataWidth:0x12  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_OVERRIDE_LAST_CYCLE_ONLY :"
				, (u32_t)((data & PXP2_RD_CONF11_REG_RD_OVERRIDE_LAST_CYCLE_ONLY) >> 17)
				, "\n\tRD_OVERRIDE_DATA_WHEN_ERROR :"
				, (u32_t)((data & PXP2_RD_CONF11_REG_RD_OVERRIDE_DATA_WHEN_ERROR) >> 16)
				, "\n\tRD_ERROR_PATTERN :"
				, (u32_t)((data & PXP2_RD_CONF11_REG_RD_ERROR_PATTERN) >> 0));
			break;
		case 0x120778UL: // (PXP2_REG_RD_CPL_ERR_DETAILS: ACCESS:R  DataWidth:0x1d ) 
			sprintf(disp_str, "\nPXP2_REG_RD_CPL_ERR_DETAILS (ACCESS:R  DataWidth:0x1d )" \
				"\nDescription: Details of first request with error on receive side: [15:\n    0] - Echo ID. [28:16] - sub-request length plus start_offset_2_0 minus\n     1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12077cUL: // (PXP2_REG_RD_CPL_ERR_DETAILS2: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nPXP2_REG_RD_CPL_ERR_DETAILS2 (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: Details of first request with error on receive side: [4:0\n    ] - VQ ID. [8:5] - client ID. [9] - valid - indicates if there was a completion\n     error since the last time this register was read. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120780UL: // (PXP2_REG_RQ_ATC_USDM_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_USDM_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120784UL: // (PXP2_REG_RQ_ATC_USDMDP_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_USDMDP_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120788UL: // (PXP2_REG_RQ_ATC_TSDM_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_TSDM_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12078cUL: // (PXP2_REG_RQ_ATC_XSDM_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_XSDM_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120790UL: // (PXP2_REG_RQ_ATC_DMAE_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_DMAE_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120794UL: // (PXP2_REG_RQ_ATC_CDUWR_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_CDUWR_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120798UL: // (PXP2_REG_RQ_ATC_CDURD_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_CDURD_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12079cUL: // (PXP2_REG_RQ_ATC_PBF_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_PBF_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207a0UL: // (PXP2_REG_RQ_ATC_QM_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_QM_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207a4UL: // (PXP2_REG_RQ_ATC_TM_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_TM_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207a8UL: // (PXP2_REG_RQ_ATC_SRC_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_SRC_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207acUL: // (PXP2_REG_RQ_ATC_DBG_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_DBG_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207b0UL: // (PXP2_REG_RQ_ATC_HC_FLAGS: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_HC_FLAGS (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: ATC flags for that client as follows: b2-b0: atc flags; b3\n     - if set use this configuration (b2-b0) otherwise use the atc flags from\n     the request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207b4UL: // (PXP2_REG_RQ_ATC_VQ_ENABLE: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_VQ_ENABLE (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: ATC VQ enable bits. When set - SR from the VQ can send ATC\n     lookup request to the ATC (assuming all other conditions are met). When\n     reset - all SR-s from the VQ will NOT go through the ATC. b0 - VQ0; b1\n     - VQ1; b30 - VQ30; b31 - reserved (should be filled with zeroes).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207b8UL: // (PXP2_REG_RQ_CLOSE_GATE_WR_CLIENT_EN: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CLOSE_GATE_WR_CLIENT_EN (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: BW counter wr clients that are enabled during close the gates\n     flow (bit per wr vq). In E2 no clients operate during close the gates\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207bcUL: // (PXP2_REG_RQ_ATC_INTERNAL_ATS_ENABLE_ALL: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_INTERNAL_ATS_ENABLE_ALL (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Concatenated values of rq_atc_internal_ats_enable as follows\n    : b0 - PF0; b1 - VF0; b2 - PF1; b3 - VF1; b14 - PF7; b15 - VF7;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207c0UL: // (PXP2_REG_RQ_ATC_VQ_GO_TRANSLATED: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_VQ_GO_TRANSLATED (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: DEBUG ONLY. bit per VQ. go translated set means that SR of\n     the matched VQ will be always sent to the GLUE with the at_valid=1 indication\n     (see atc_code in PSWRQ-PGLUE interface for more details). In that case\n     the address will be delivered by the chip (and not by the ATC). This \n    mode will be used mainly for debug and the other configurations must make\n     sure that ATC will never be used for that VQ while the go_translated \n    bit for that VQ is set. when reset means that the at_valid indication \n    will be determined according to the ATC.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207c4UL: // (PXP2_REG_RQ_ATC_GLOBAL_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_GLOBAL_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Global ATC enable bit. when reset all ATC logic is disabled\n     within the PSWRQ. The value of this register must be the same as RD_ATC_GLOBAL_ENABLE\n    . This value must be '1' when ATC capability is enabled in PCIe core.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207c8UL: // (PXP2_REG_RQ_L2P_VF_ERR_VQ_ID: ACCESS:RC  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_L2P_VF_ERR_VQ_ID (ACCESS:RC  DataWidth:0x5 )" \
				"\nDescription: The VQ id of the request with rq_l2p_vf_err interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207ccUL: // (PXP2_REG_RD_ATC_GLOBAL_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ATC_GLOBAL_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Global ATC enable bit. When reset all ATC logic is disabled\n     within the PSWRD. 'ATC entry ID' interface from PSWRQ is ignored and '\n    ATC RCPL Done' interface to ATC is not generated. The value of this register\n     must be the same as PSWRQ_ATC_GLOBAL_ENABLE. This value must be '1' when\n     ATC capability is enabled in PCIe core.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207d4UL: // (PXP2_REG_RD_ATC_ENTRY_ID_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_ATC_ENTRY_ID_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of ATC entry ID memory - unused in E3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207fcUL: // (PXP2_REG_RQ_TREQ_FIFO_FILL_LVL: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TREQ_FIFO_FILL_LVL (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: The fill level of the TREQ fifo\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120910UL: // (PXP2_REG_RQ_ICPL_FIFO_FILL_LVL: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ICPL_FIFO_FILL_LVL (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: The fill level of the ICPL fifo\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120914UL: // (PXP2_REG_RQ_ATC_TREQ_FIFO_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_TREQ_FIFO_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120918UL: // (PXP2_REG_RQ_ASSERT_IF_ILT_FAIL: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ASSERT_IF_ILT_FAIL (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: when set - assert ilt fail interrupt (rq_elt_addr) in case\n     working in ilt mode and onchip translation fail due to overflow on vah_plus_1st\n     signal (Cont00041628). If reset - interrupt will not assert\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12092cUL: // (PXP2_REG_RQ_DRAM_ALIGN_RD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DRAM_ALIGN_RD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Determines alignment of read SRs when a request is split \n    into several SRs. 0 - 8B aligned. 1 - 64B aligned. 2 - 128B aligned. 3\n     - 256B aligned. 4 - 512B aligned.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120930UL: // (PXP2_REG_RQ_DRAM_ALIGN_SEL: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DRAM_ALIGN_SEL (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: when set the new alignment method (E2) will be applied; when\n     reset the original alignment method (E1 E1H) will be applied\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120934UL: // (PXP2_REG_WR_HP_FORCE_DEASSERT_E2: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_HP_FORCE_DEASSERT_E2 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: has_payload forced de-assertion after grant from PSWRQ. 0\n     - as in B0; 1 - forced to 2 cycles de-assertion after grant. Bit[0] - \n    all clients but USDM-DP; bit[1] - USDM-DP client.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120938UL: // (PXP2_REG_RQ_HOQ_RAM_RD_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_HOQ_RAM_RD_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: FOR DBG: enable reading from the hoq ram; when set hoq rbc\n     read is enabled; when reset hoq rbc read is disabled (i.e. rq_hoq_ram_rd_req\n     will not have any affect)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12095cUL: // (PXP2_REG_RD_CONTINUE_SERVING_PBF: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_CONTINUE_SERVING_PBF (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This register defines the delivery port behavior when finishing\n     delivering a request to the PBF and the data for the next request is \n    already in the Tetris buffer. 0 - The delivery port continues delivering\n     the next PBF request only if the second delivery port is idle. This is\n     the behavior in E1 E1H and E2. 1 - The delivery port always continues\n     delivering the next PBF request. This is more efficient since about 11\n     arbitration cycles are not wasted.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120960UL: // (PXP2_REG_RD_TM1_E3: ACCESS:RW  DataWidth:0x1e  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TM1_E3 (ACCESS:RW  DataWidth:0x1e  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_TETRIS6_TMB :"
				, (u32_t)((data & PXP2_RD_TM1_E3_REG_RD_TETRIS6_TMB) >> 25)
				, "\n\tRD_TETRIS6_TMA :"
				, (u32_t)((data & PXP2_RD_TM1_E3_REG_RD_TETRIS6_TMA) >> 20)
				, "\n\tRD_TETRIS5_TMB :"
				, (u32_t)((data & PXP2_RD_TM1_E3_REG_RD_TETRIS5_TMB) >> 15)
				, "\n\tRD_TETRIS5_TMA :"
				, (u32_t)((data & PXP2_RD_TM1_E3_REG_RD_TETRIS5_TMA) >> 10)
				, "\n\tRD_TETRIS4_TMB :"
				, (u32_t)((data & PXP2_RD_TM1_E3_REG_RD_TETRIS4_TMB) >> 5)
				, "\n\tRD_TETRIS4_TMA :"
				, (u32_t)((data & PXP2_RD_TM1_E3_REG_RD_TETRIS4_TMA) >> 0));
			break;
		case 0x120964UL: // (PXP2_REG_RD_TM2_E3: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_RD_TM2_E3 (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRD_TETRIS7_TMB :"
				, (u32_t)((data & PXP2_RD_TM2_E3_REG_RD_TETRIS7_TMB) >> 5)
				, "\n\tRD_TETRIS7_TMA :"
				, (u32_t)((data & PXP2_RD_TM2_E3_REG_RD_TETRIS7_TMA) >> 0));
			break;
		case 0x120968UL: // (PXP2_REG_PSWWR_USDMDP_TM: ACCESS:RW  DataWidth:0x14  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP2_REG_PSWWR_USDMDP_TM (ACCESS:RW  DataWidth:0x14  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_USDMDP_FIFO3_TMB :"
				, (u32_t)((data & PXP2_PSWWR_USDMDP_TM_REG_WR_USDMDP_FIFO3_TMB) >> 15)
				, "\n\tWR_USDMDP_FIFO3_TMA :"
				, (u32_t)((data & PXP2_PSWWR_USDMDP_TM_REG_WR_USDMDP_FIFO3_TMA) >> 10)
				, "\n\tWR_USDMDP_FIFO2_TMB :"
				, (u32_t)((data & PXP2_PSWWR_USDMDP_TM_REG_WR_USDMDP_FIFO2_TMB) >> 5)
				, "\n\tWR_USDMDP_FIFO2_TMA :"
				, (u32_t)((data & PXP2_PSWWR_USDMDP_TM_REG_WR_USDMDP_FIFO2_TMA) >> 0));
			break;
		case 0x1209a4UL: // (PXP2_REG_RQ_SR_CNT_WINDOW_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_WINDOW_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Counting window mode. 0 - manual window: counting is manually\n     being initiated & stopped by the user through GRC. 1 - configured window\n    : counting occurs according to configured window size.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209b8UL: // (PXP2_REG_RQ_SR_CNT_START_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_START_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Determines the trigger for start counting (for both SR counters\n     & global window counter). 0 - start counting upon any first SR that is\n     sent to the PGLUE. 1 - start counting upon first PBF/USDM-DP SR that \n    is sent to the PGLUE.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209bcUL: // (PXP2_REG_RQ_SR_CNT_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enables the SR counting mechanism.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209c0UL: // (PXP2_REG_RQ_SR_CNT_CLK_TICK: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_CLK_TICK (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The number of clk_pci ticks minus 1 between each increment\n     of the global window counter (i.e. 0 is for 1 clk_pci cycle; 1 is for\n     2 clk_pci cycles; 7 is for 8 clk_pci cycles)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120534UL: // (PXP2_REG_PGL_ADDR_88_F0: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_ADDR_88_F0 (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: GRC address for configuration access to PCIE config address\n     0x88. any write to this PCIE address will cause a GRC write access to\n     the address that's in t this register \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120538UL: // (PXP2_REG_PGL_ADDR_8C_F0: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_ADDR_8C_F0 (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: GRC address for configuration access to PCIE config address\n     0x8c.  any write to this PCIE address will cause a GRC write access to\n     the address that's in t this register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12053cUL: // (PXP2_REG_PGL_ADDR_90_F0: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_ADDR_90_F0 (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: GRC address for configuration access to PCIE config address\n     0x90.  any write to this PCIE address will cause a GRC write access to\n     the address that's in t this register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120540UL: // (PXP2_REG_PGL_ADDR_94_F0: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_ADDR_94_F0 (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: GRC address for configuration access to PCIE config address\n     0x94.  any write to this PCIE address will cause a GRC write access to\n     the address that's in t this register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12061cUL: // (PXP2_REG_RQ_CDU_FIRST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CDU_FIRST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: first memory address base for cdu in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120620UL: // (PXP2_REG_RQ_CDU_LAST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CDU_LAST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: last memory address base for cdu in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120624UL: // (PXP2_REG_RQ_CSDM_FIRST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CSDM_FIRST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: first memory address base for csdm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120628UL: // (PXP2_REG_RQ_CSDM_LAST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_CSDM_LAST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: last memory address base for csdm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12062cUL: // (PXP2_REG_RQ_DBG_FIRST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DBG_FIRST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: first memory address base for dbg in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120630UL: // (PXP2_REG_RQ_DBG_LAST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_DBG_LAST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: last memory address base for dbg in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120634UL: // (PXP2_REG_RQ_QM_FIRST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QM_FIRST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: first memory address base for qm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120638UL: // (PXP2_REG_RQ_QM_LAST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_QM_LAST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: last memory address base for qm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12063cUL: // (PXP2_REG_RQ_SRC_FIRST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SRC_FIRST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: first memory address base for src in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120640UL: // (PXP2_REG_RQ_SRC_LAST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SRC_LAST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: last memory address base for src in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120644UL: // (PXP2_REG_RQ_TM_FIRST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TM_FIRST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: first memory address base for tm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120648UL: // (PXP2_REG_RQ_TM_LAST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TM_LAST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: last memory address base for tm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12064cUL: // (PXP2_REG_RQ_TSDM_FIRST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TSDM_FIRST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: first memory address base for tsdm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120650UL: // (PXP2_REG_RQ_TSDM_LAST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_TSDM_LAST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: last memory address base for tsdm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120654UL: // (PXP2_REG_RQ_USDM_FIRST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDM_FIRST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: first memory address base for usdm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120658UL: // (PXP2_REG_RQ_USDM_LAST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDM_LAST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: last memory address base for usdm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12065cUL: // (PXP2_REG_RQ_USDMDP_FIRST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDMDP_FIRST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: first memory address base for usdmdp in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120660UL: // (PXP2_REG_RQ_USDMDP_LAST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_USDMDP_LAST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: last memory address base for usdmdp in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120664UL: // (PXP2_REG_RQ_XSDM_FIRST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_XSDM_FIRST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: first memory address base for xsdm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120668UL: // (PXP2_REG_RQ_XSDM_LAST_ILT: ACCESS:RW  DataWidth:0xc  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_XSDM_LAST_ILT (ACCESS:RW  DataWidth:0xc  SPLIT:8 )" \
				"\nDescription: last memory address base for xsdm in ILT\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207d0UL: // (PXP2_REG_RD_CPL_ERR_DETAILS_CLR: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RD_CPL_ERR_DETAILS_CLR (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: Writing to this register clears rd_cpl_err_details and rd_cpl_err_details2\n     and enables logging new error details\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207d8UL: // (PXP2_REG_RQ_ATC_INTERNAL_ATS_ENABLE: ACCESS:RW  DataWidth:0x2  SPLIT:8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ATC_INTERNAL_ATS_ENABLE (ACCESS:RW  DataWidth:0x2  SPLIT:8 )" \
				"\nDescription: ATC enable values per PF as follows: b0 - PF enable; b1 - \n    VF enable; PF enable bit is relevant when VF_Valid (in the request) bit\n     is 0; VF enable bit is relevant when VF_Valid bit is 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120800UL: // (PXP2_REG_PGL_EXP_ROM0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_EXP_ROM0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: first dword data of expansion rom request. When this register\n     is written a completion is sent to the pcie core. When the expansion \n    rom request contains more than one dword this register should be written\n     last. Writing to this register when there is not pending expansion rom\n     request should not be done!\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120804UL: // (PXP2_REG_PGL_EXP_ROM1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_EXP_ROM1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: second dword data of expansion rom request.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120808UL: // (PXP2_REG_PGL_EXP_ROM2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_EXP_ROM2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: third dword data of expansion rom request. this register \n    is special. reading from it provides a vector outstanding read requests\n    . if a bit is zero it means that a read request on the corresponding tag\n     did not finish yet (not all completions have arrived for it)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12080cUL: // (PXP2_REG_RQ_UFIFO_NUM_OF_ENTRY: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_UFIFO_NUM_OF_ENTRY (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Number of entries in the ufifo;This fifo has l2p completions\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120810UL: // (PXP2_REG_RQ_VQ0_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ0_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 0 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120814UL: // (PXP2_REG_RQ_VQ0_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ0_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120818UL: // (PXP2_REG_RQ_VQ10_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ10_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 10 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12081cUL: // (PXP2_REG_RQ_VQ10_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ10_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120820UL: // (PXP2_REG_RQ_VQ11_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ11_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 11 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120824UL: // (PXP2_REG_RQ_VQ11_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ11_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120828UL: // (PXP2_REG_RQ_VQ12_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ12_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 12 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12082cUL: // (PXP2_REG_RQ_VQ12_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ12_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120830UL: // (PXP2_REG_RQ_VQ13_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ13_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 13 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120834UL: // (PXP2_REG_RQ_VQ13_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ13_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120838UL: // (PXP2_REG_RQ_VQ14_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ14_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 14  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12083cUL: // (PXP2_REG_RQ_VQ14_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ14_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120840UL: // (PXP2_REG_RQ_VQ15_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ15_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 15  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120844UL: // (PXP2_REG_RQ_VQ15_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ15_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120848UL: // (PXP2_REG_RQ_VQ16_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ16_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 16 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12084cUL: // (PXP2_REG_RQ_VQ16_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ16_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120850UL: // (PXP2_REG_RQ_VQ17_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ17_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 17 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120854UL: // (PXP2_REG_RQ_VQ17_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ17_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120858UL: // (PXP2_REG_RQ_VQ18_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ18_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 18  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12085cUL: // (PXP2_REG_RQ_VQ18_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ18_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120860UL: // (PXP2_REG_RQ_VQ19_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ19_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 19 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120864UL: // (PXP2_REG_RQ_VQ19_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ19_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120868UL: // (PXP2_REG_RQ_VQ1_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ1_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 1 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12086cUL: // (PXP2_REG_RQ_VQ1_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ1_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120870UL: // (PXP2_REG_RQ_VQ20_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ20_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 20  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120874UL: // (PXP2_REG_RQ_VQ20_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ20_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120878UL: // (PXP2_REG_RQ_VQ21_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ21_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 21 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12087cUL: // (PXP2_REG_RQ_VQ21_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ21_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120880UL: // (PXP2_REG_RQ_VQ22_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ22_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 22 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120884UL: // (PXP2_REG_RQ_VQ22_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ22_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120888UL: // (PXP2_REG_RQ_VQ23_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ23_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 23 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12088cUL: // (PXP2_REG_RQ_VQ23_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ23_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120890UL: // (PXP2_REG_RQ_VQ24_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ24_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 24  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120894UL: // (PXP2_REG_RQ_VQ24_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ24_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120898UL: // (PXP2_REG_RQ_VQ25_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ25_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 25  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12089cUL: // (PXP2_REG_RQ_VQ25_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ25_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208a0UL: // (PXP2_REG_RQ_VQ26_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ26_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 26 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208a4UL: // (PXP2_REG_RQ_VQ26_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ26_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208a8UL: // (PXP2_REG_RQ_VQ27_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ27_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 27 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208acUL: // (PXP2_REG_RQ_VQ27_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ27_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208b0UL: // (PXP2_REG_RQ_VQ28_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ28_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 28  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208b4UL: // (PXP2_REG_RQ_VQ28_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ28_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208b8UL: // (PXP2_REG_RQ_VQ29_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ29_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 29 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208bcUL: // (PXP2_REG_RQ_VQ29_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ29_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208c0UL: // (PXP2_REG_RQ_VQ2_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ2_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 2 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208c4UL: // (PXP2_REG_RQ_VQ2_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ2_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208c8UL: // (PXP2_REG_RQ_VQ30_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ30_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 30  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208ccUL: // (PXP2_REG_RQ_VQ30_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ30_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208d0UL: // (PXP2_REG_RQ_VQ31_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ31_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 31 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208d4UL: // (PXP2_REG_RQ_VQ31_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ31_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208d8UL: // (PXP2_REG_RQ_VQ3_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ3_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 3 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208dcUL: // (PXP2_REG_RQ_VQ3_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ3_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208e0UL: // (PXP2_REG_RQ_VQ4_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ4_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 4  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208e4UL: // (PXP2_REG_RQ_VQ4_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ4_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208e8UL: // (PXP2_REG_RQ_VQ5_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ5_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 5  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208ecUL: // (PXP2_REG_RQ_VQ5_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ5_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208f0UL: // (PXP2_REG_RQ_VQ6_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ6_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 6 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208f4UL: // (PXP2_REG_RQ_VQ6_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ6_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208f8UL: // (PXP2_REG_RQ_VQ7_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ7_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 7 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1208fcUL: // (PXP2_REG_RQ_VQ7_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ7_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120900UL: // (PXP2_REG_RQ_VQ8_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ8_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 8  in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120904UL: // (PXP2_REG_RQ_VQ8_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ8_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120908UL: // (PXP2_REG_RQ_VQ9_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ9_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Number of entries occupied by vq 9 in pswrq memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12090cUL: // (PXP2_REG_RQ_VQ9_MAX_ENTRY_CNT: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_VQ9_MAX_ENTRY_CNT (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Maximum Number of entries occupied by vq 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12091cUL: // (PXP2_REG_DBG_OUT_DATA_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_DBG_OUT_DATA_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 32 lsb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120920UL: // (PXP2_REG_DBG_OUT_DATA_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_DBG_OUT_DATA_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 32 msb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120924UL: // (PXP2_REG_DBG_OUT_FRAME: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_DBG_OUT_FRAME (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 4 frame bits that goes to the DBG block. Bit0\n     is the frame of data byte0; Bit1 is the frame of data byte1; Bit2 is \n    the frame of data byte2; Bit3 is the frame of data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120928UL: // (PXP2_REG_DBG_OUT_VALID: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_DBG_OUT_VALID (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 4 valid bits that goes to the DBG block. Bit0\n     validtes data byte0; Bit1 validates data byte1; Bit2 validates data byte2\n    ; Bit3 validates data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12093cUL: // (PXP2_REG_RQ_HOQ_RAM_RD_REQ: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_HOQ_RAM_RD_REQ (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: FOR DBG: read request from the hoq ram; the write data represents\n     the address which is the vqid; in order to read from the hoq ram the \n    read enable register should be set as well (rq_hoq_ram_rd_en); upon read\n     completion (rq_hoq_ram_rd_status =1) data_rd_0 data_rd_1 data_rd_2 and\n     data_rd_3 are ready with the valid values\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120940UL: // (PXP2_REG_RQ_HOQ_RAM_RD_STATUS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_HOQ_RAM_RD_STATUS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FOR DBG: when set - data rd from hoq ram is completed (i.\n    e. data is ready in data_rd_0 data_rd_1 data_rd2 and data_rd_3); when \n    reset - still waiting for hoq ram read request to be completed)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120944UL: // (PXP2_REG_RQ_HOQ_RAM_DATA_RD_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_HOQ_RAM_DATA_RD_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: FOR DBG: bits 15:0 length; bits 31:16 request id\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120948UL: // (PXP2_REG_RQ_HOQ_RAM_DATA_RD_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_HOQ_RAM_DATA_RD_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: FOR DBG: address (32 lsb)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12094cUL: // (PXP2_REG_RQ_HOQ_RAM_DATA_RD_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_HOQ_RAM_DATA_RD_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: FOR DBG: address (32 msb)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120950UL: // (PXP2_REG_RQ_HOQ_RAM_DATA_RD_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_HOQ_RAM_DATA_RD_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: FOR DBG: bit 0 relaxed ordering; bit 1 no-snoop; bits 5:2\n     client id; bit 6 done type; bit 7 resevred; bit 10:8 pfid; bit 11 vf_valid\n    ; bit 17:12 vfid; bits 20:18 atc flags; bits 31:21 reserved\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12096cUL: // (PXP2_REG_RQ_SR_CNT_WR_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_WR_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The total number of WR SR-s that were sent to the PGLUE. \n    Valid when Sr_cnt_status is done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120970UL: // (PXP2_REG_RQ_SR_CNT_RD_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_RD_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The total number of RD SR-s that were sent to the PGLUE. \n    Valid when Sr_cnt_status is done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120974UL: // (PXP2_REG_RQ_SR_CNT_PBF_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_PBF_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The number of PBF RD SR-s that were sent to the PGLUE. Valid\n     when Sr_cnt_status is done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120978UL: // (PXP2_REG_RQ_SR_CNT_USDMDP_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_USDMDP_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The number of USDM-DP WR SR-s that were sent to the PGLUE\n    . Valid when Sr_cnt_status is done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12097cUL: // (PXP2_REG_RQ_SR_CNT_TREQ_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_TREQ_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The number of TREQ SR-s that were sent to the PGLUE. Valid\n     when Sr_cnt_status is done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120980UL: // (PXP2_REG_RQ_SR_CNT_ICPL_CNT: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_ICPL_CNT (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The number of ICPL SR-s that were sent to the PGLUE. Valid\n     when Sr_cnt_status is done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120984UL: // (PXP2_REG_RQ_SR_CNT_WR_BYTE_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_WR_BYTE_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The total number of bytes for WR SR-s that were sent to the\n     PGLUE. Valid when Sr_cnt_status is done - lsb value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120988UL: // (PXP2_REG_RQ_SR_CNT_WR_BYTE_MSB: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_WR_BYTE_MSB (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: The total number of bytes for WR SR-s that were sent to the\n     PGLUE. Valid when Sr_cnt_status is done - msb value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12098cUL: // (PXP2_REG_RQ_SR_CNT_RD_BYTE_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_RD_BYTE_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The total number of bytes for RD SR-s that were sent to the\n     PGLUE. Valid when Sr_cnt_status is done - lsb value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120990UL: // (PXP2_REG_RQ_SR_CNT_RD_BYTE_MSB: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_RD_BYTE_MSB (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: The total number of bytes for RD SR-s that were sent to the\n     PGLUE. Valid when Sr_cnt_status is done - msb value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120994UL: // (PXP2_REG_RQ_SR_CNT_PBF_BYTE_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_PBF_BYTE_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The number of bytes for PBF RD SR-s that were sent to the\n     PGLUE. Valid when Sr_cnt_status is done - lsb value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120998UL: // (PXP2_REG_RQ_SR_CNT_PBF_BYTE_MSB: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_PBF_BYTE_MSB (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: The number of bytes for PBF RD SR-s that were sent to the\n     PGLUE. Valid when Sr_cnt_status is done - msb value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12099cUL: // (PXP2_REG_RQ_SR_CNT_USDMDP_BYTE_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_USDMDP_BYTE_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The number of bytes for USDM-DP WR SR-s that were sent to\n     the PGLUE. Valid when Sr_cnt_status is done - lsb value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209a0UL: // (PXP2_REG_RQ_SR_CNT_USDMDP_BYTE_MSB: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_USDMDP_BYTE_MSB (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: The number of bytes for USDM-DP WR SR-s that were sent to\n     the PGLUE. Valid when Sr_cnt_status is done - msb value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209acUL: // (PXP2_REG_RQ_SR_CNT_WINDOW_VALUE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_WINDOW_VALUE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Global window counter for the current value of the recorded\n     window. Represents the number of sr_cnt_clk_tickxclk_pci cycles from \n    the beginning of counting. NOTE: beginning of counting is determined according\n     to Sr_cnt_start_mode\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209b0UL: // (PXP2_REG_RQ_SR_CNT_MANUAL_CMD: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_MANUAL_CMD (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: Write Only register. The manual window command sent by the\n     user. Valid when working in manual window mode (i.e. Sr_cnt_window_mode\n     = 0). 0 - stop counting. 1 - start counting.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209b4UL: // (PXP2_REG_RQ_SR_CNT_RST: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_RST (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: Write Only register. RBC write command to this reg (any value\n    ) will reset the SR counters & the global window counter. In addition \n    it'll move the Sr_cnt_status to idle state.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209c4UL: // (PXP2_REG_RQ_SR_CNT_STATUS: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_SR_CNT_STATUS (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: The status of the SR count mechanism: 0 - idle: ready to \n    start new counting. 1 - ongoing: counting is currently ongoing. 2 - done\n    : counting is completed. SR counters & global window counter are valid\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209c8UL: // (PXP2_REG_RQ_LAST_RD_SR_LOG_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LAST_RD_SR_LOG_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: SR address - 32 lsb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209ccUL: // (PXP2_REG_RQ_LAST_RD_SR_LOG_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LAST_RD_SR_LOG_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: SR address - 32 msb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209d0UL: // (PXP2_REG_RQ_LAST_RD_SR_LOG_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LAST_RD_SR_LOG_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: b15-0: reqid; b28-16: SR length; b29 - reserved; b31-30: \n    attributes\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209d4UL: // (PXP2_REG_RQ_LAST_RD_SR_LOG_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LAST_RD_SR_LOG_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: b2-0: PFID; b3: vf_valid; b9-b4: VFID; b10: first SR; b11\n    : last SR; b15-12: client id; b20-16: vq; b22-21: atc code; b29-23: srid\n    ; b31-30: endianity\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209d8UL: // (PXP2_REG_RQ_LAST_RD_SR_LOG_4: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LAST_RD_SR_LOG_4 (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: treq otb entry\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209dcUL: // (PXP2_REG_RQ_LAST_WR_SR_LOG_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LAST_WR_SR_LOG_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: SR address - 32 lsb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209e0UL: // (PXP2_REG_RQ_LAST_WR_SR_LOG_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LAST_WR_SR_LOG_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: SR address - 32 msb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209e4UL: // (PXP2_REG_RQ_LAST_WR_SR_LOG_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LAST_WR_SR_LOG_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: b15-0: reqid; b28-16: SR length; b29 - reserved; b31-30: \n    attributes\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209e8UL: // (PXP2_REG_RQ_LAST_WR_SR_LOG_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LAST_WR_SR_LOG_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: b2-0: PFID; b3: vf_valid; b9-b4: VFID; b10: first SR; b11\n    : last SR; b15-12: client id; b20-16: vq; b22-21: atc code; b27-23: start\n     offset; b28: usdm err; b29: wdone type; b31-30: endianity\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209ecUL: // (PXP2_REG_RQ_LAST_WR_SR_LOG_4: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_LAST_WR_SR_LOG_4 (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: icpl itag index\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120b00UL: // (PXP2_REG_RD_FIRST_SR_NODES: ACCESS:WB_R  DataWidth:0x1c ) 
			sprintf(disp_str, "\nPXP2_REG_RD_FIRST_SR_NODES (ACCESS:WB_R  DataWidth:0x1c )" \
				"\nDescription: Debug only and read only: Each entry provides the first sub\n     request ID in 4 VQs. SR ID of 0x7f is NULL and means there is no sub \n    request in this VQ in PSWRD. The reset value is the one expected in idle\n     check except for the Timers VQ (VQ3). E2 note: due to a bug (CQ44496) \n    in the implementation only the first out of the 8 addresses (for VQs 0\n     through 3) is accessible.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x122000UL: // (PXP2_REG_RQ_ONCHIP_AT: ACCESS:WB  DataWidth:0x35 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ONCHIP_AT (ACCESS:WB  DataWidth:0x35 )" \
				"\nDescription: Onchip address table\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x128000UL: // (PXP2_REG_RQ_ONCHIP_AT_B0: ACCESS:WB  DataWidth:0x35 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_ONCHIP_AT_B0 (ACCESS:WB  DataWidth:0x35 )" \
				"\nDescription: Onchip address table - B0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120544UL: // (PXP2_REG_PGL_ADDR_88_F1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_ADDR_88_F1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: GRC address for configuration access to PCIE config address\n     0x88.  any write to this PCIE address will cause a GRC write access to\n     the address that's in t this register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120548UL: // (PXP2_REG_PGL_ADDR_8C_F1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_ADDR_8C_F1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: GRC address for configuration access to PCIE config address\n     0x8c.  any write to this PCIE address will cause a GRC write access to\n     the address that's in t this register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x12054cUL: // (PXP2_REG_PGL_ADDR_90_F1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_ADDR_90_F1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: GRC address for configuration access to PCIE config address\n     0x90.  any write to this PCIE address will cause a GRC write access to\n     the address that's in t this register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120550UL: // (PXP2_REG_PGL_ADDR_94_F1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP2_REG_PGL_ADDR_94_F1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: GRC address for configuration access to PCIE config address\n     0x94.  any write to this PCIE address will cause a GRC write access to\n     the address that's in t this register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1206a4UL: // (PXP2_REG_WR_CDU_TM: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP2_REG_WR_CDU_TM (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Bits [1:0] are for fifo 0;Bits [3:2] are for fifo 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120a00UL: // (PXP2_REG_RQ_OFFCHIP_AT: ACCESS:R  DataWidth:0x34 ) 
			sprintf(disp_str, "\nPXP2_REG_RQ_OFFCHIP_AT (ACCESS:R  DataWidth:0x34 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1207dcUL: // (PXP2_REG_PXP2_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120954UL: // (PXP2_REG_PXP2_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1209f0UL: // (PXP2_REG_PXP2_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x120b20UL: // (PXP2_REG_PXP2_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x124000UL: // (PXP2_REG_PXP2_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPXP2_REG_PXP2_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103000UL: // (PXP_REG_HST_DISABLE_INPUTS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_DISABLE_INPUTS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug only: When '1'; inputs to the PSWHST block in clk domain\n     are ignored\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103004UL: // (PXP_REG_HST_ARB_IS_IDLE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_ARB_IS_IDLE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: debug only: Indication if PSWHST arbiter is idle\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103008UL: // (PXP_REG_HST_CLIENTS_WAITING_TO_ARB: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP_REG_HST_CLIENTS_WAITING_TO_ARB (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: debug only: A bit mask for all PSWHST arbiter clients. '1\n    ' means this client is waiting for the arbiter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10300cUL: // (PXP_REG_HST_USDM_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP_REG_HST_USDM_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: USDM byte swapping mode configuration for host read and write\n     requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103010UL: // (PXP_REG_HST_CSDM_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP_REG_HST_CSDM_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: CSDM byte swapping mode configuration for host read and write\n     requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103014UL: // (PXP_REG_HST_XSDM_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP_REG_HST_XSDM_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: XSDM byte swapping mode configuration for host read and write\n     requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103018UL: // (PXP_REG_HST_TSDM_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP_REG_HST_TSDM_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TSDM byte swapping mode configuration for host read and write\n     requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10301cUL: // (PXP_REG_HST_HC_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP_REG_HST_HC_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: HC byte swapping mode configuration for host read and write\n     requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103020UL: // (PXP_REG_HST_GRC_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP_REG_HST_GRC_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: GRC byte swapping mode configuration for host read and write\n     requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103024UL: // (PXP_REG_HST_DQ_SWAP_MODE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP_REG_HST_DQ_SWAP_MODE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: DORQ byte swapping mode configuration for host read and write\n     requests\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103028UL: // (PXP_REG_WR_USDM_FULL_TH: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP_REG_WR_USDM_FULL_TH (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: if number of entries in the usdm fifo is bigger than this\n     number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10302cUL: // (PXP_REG_WR_CSDM_FULL_TH: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP_REG_WR_CSDM_FULL_TH (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: if number of entries in the csdm fifo is bigger than this\n     number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103030UL: // (PXP_REG_WR_XSDM_FULL_TH: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP_REG_WR_XSDM_FULL_TH (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: if number of entries in the xsdm fifo is bigger than this\n     number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103034UL: // (PXP_REG_WR_TSDM_FULL_TH: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP_REG_WR_TSDM_FULL_TH (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: if number of entries in the tsdm fifo is bigger than this\n     number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103038UL: // (PXP_REG_WR_QM_FULL_TH: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP_REG_WR_QM_FULL_TH (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: if number of entries in the qm fifo is bigger than this number\n     than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10303cUL: // (PXP_REG_WR_TM_FULL_TH: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP_REG_WR_TM_FULL_TH (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: if number of entries in the tm fifo is bigger than this number\n     than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103040UL: // (PXP_REG_WR_SRC_FULL_TH: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP_REG_WR_SRC_FULL_TH (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: if number of entries in the src fifo is bigger than this \n    number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103044UL: // (PXP_REG_WR_DBG_FULL_TH: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP_REG_WR_DBG_FULL_TH (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: if number of entries in the dbg fifo is bigger than this \n    number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103048UL: // (PXP_REG_WR_HC_FULL_TH: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP_REG_WR_HC_FULL_TH (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: if number of entries in the hc fifo is bigger than this number\n     than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10304cUL: // (PXP_REG_WR_DMAE_FULL_TH: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP_REG_WR_DMAE_FULL_TH (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: if number of entries in the dmae input fifo is bigger than\n     this number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103050UL: // (PXP_REG_WR_CDU_FULL_TH: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP_REG_WR_CDU_FULL_TH (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: if number of entries in the cdu input fifo is bigger than\n     this number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103054UL: // (PXP_REG_WR_USDMDP_FULL_TH: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP_REG_WR_USDMDP_FULL_TH (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: if number of entries in the usdmdp input fifo is bigger than\n     this number than full will be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103058UL: // (PXP_REG_DBGSYN_STATUS: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nPXP_REG_DBGSYN_STATUS (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: debug only: Number of used entries in debug bus sync FIFO\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10305cUL: // (PXP_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PXP\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103060UL: // (PXP_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PXP\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select and before the shift)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103064UL: // (PXP_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nPXP_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from PXP\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103068UL: // (PXP_REG_PXP_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP_REG_PXP_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_TSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_WR_TSDM_FIFO_OVERFLOW) >> 31)
				, "\n\tWR_XSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_WR_XSDM_FIFO_OVERFLOW) >> 30)
				, "\n\tWR_USDMDP_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_WR_USDMDP_FIFO_OVERFLOW) >> 29)
				, "\n\tWR_USDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_WR_USDM_FIFO_OVERFLOW) >> 28)
				, "\n\tWR_TM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_WR_TM_FIFO_OVERFLOW) >> 27)
				, "\n\tWR_QM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_WR_QM_FIFO_OVERFLOW) >> 26)
				, "\n\tWR_SRC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_WR_SRC_FIFO_OVERFLOW) >> 25)
				, "\n\tRQ_DBG_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_DBG_FIFO_OVERFLOW) >> 24)
				, "\n\tRQ_HC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_HC_FIFO_OVERFLOW) >> 23)
				, "\n\tRQ_DMAE_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_DMAE_FIFO_OVERFLOW) >> 22)
				, "\n\tRQ_CDURD_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_CDURD_FIFO_OVERFLOW) >> 21)
				, "\n\tRQ_CDUWR_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_CDUWR_FIFO_OVERFLOW) >> 20)
				, "\n\tRQ_CSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_CSDM_FIFO_OVERFLOW) >> 19)
				, "\n\tRQ_TSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_TSDM_FIFO_OVERFLOW) >> 18)
				, "\n\tRQ_XSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_XSDM_FIFO_OVERFLOW) >> 17)
				, "\n\tRQ_USDMDP_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_USDMDP_FIFO_OVERFLOW) >> 16)
				, "\n\tRQ_USDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_USDM_FIFO_OVERFLOW) >> 15)
				, "\n\tRQ_TM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_TM_FIFO_OVERFLOW) >> 14)
				, "\n\tRQ_QM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_QM_FIFO_OVERFLOW) >> 13)
				, "\n\tRQ_SRC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_SRC_FIFO_OVERFLOW) >> 12)
				, "\n\tRQ_PBF_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RQ_PBF_FIFO_OVERFLOW) >> 11)
				, "\n\tHST_CPL_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_HST_CPL_SYNC_FIFO_ERR) >> 10)
				, "\n\tHST_DATA_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_HST_DATA_SYNC_FIFO_ERR) >> 9)
				, "\n\tHST_HDR_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_HST_HDR_SYNC_FIFO_ERR) >> 8)
				, "\n\tHST_SRC_FIFO5_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_HST_SRC_FIFO5_ERR) >> 7)
				, "\n\tHST_SRC_FIFO4_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_HST_SRC_FIFO4_ERR) >> 6)
				, "\n\tHST_SRC_FIFO3_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_HST_SRC_FIFO3_ERR) >> 5)
				, "\n\tHST_SRC_FIFO2_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_HST_SRC_FIFO2_ERR) >> 4)
				, "\n\tHST_SRC_FIFO1_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_HST_SRC_FIFO1_ERR) >> 3)
				, "\n\tRD_POP_PBF_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RD_POP_PBF_ERROR) >> 2)
				, "\n\tRD_POP_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_RD_POP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x10306cUL: // (PXP_REG_PXP_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP_REG_PXP_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_TSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_WR_TSDM_FIFO_OVERFLOW) >> 31)
				, "\n\tWR_XSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_WR_XSDM_FIFO_OVERFLOW) >> 30)
				, "\n\tWR_USDMDP_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_WR_USDMDP_FIFO_OVERFLOW) >> 29)
				, "\n\tWR_USDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_WR_USDM_FIFO_OVERFLOW) >> 28)
				, "\n\tWR_TM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_WR_TM_FIFO_OVERFLOW) >> 27)
				, "\n\tWR_QM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_WR_QM_FIFO_OVERFLOW) >> 26)
				, "\n\tWR_SRC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_WR_SRC_FIFO_OVERFLOW) >> 25)
				, "\n\tRQ_DBG_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_DBG_FIFO_OVERFLOW) >> 24)
				, "\n\tRQ_HC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_HC_FIFO_OVERFLOW) >> 23)
				, "\n\tRQ_DMAE_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_DMAE_FIFO_OVERFLOW) >> 22)
				, "\n\tRQ_CDURD_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_CDURD_FIFO_OVERFLOW) >> 21)
				, "\n\tRQ_CDUWR_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_CDUWR_FIFO_OVERFLOW) >> 20)
				, "\n\tRQ_CSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_CSDM_FIFO_OVERFLOW) >> 19)
				, "\n\tRQ_TSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_TSDM_FIFO_OVERFLOW) >> 18)
				, "\n\tRQ_XSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_XSDM_FIFO_OVERFLOW) >> 17)
				, "\n\tRQ_USDMDP_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_USDMDP_FIFO_OVERFLOW) >> 16)
				, "\n\tRQ_USDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_USDM_FIFO_OVERFLOW) >> 15)
				, "\n\tRQ_TM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_TM_FIFO_OVERFLOW) >> 14)
				, "\n\tRQ_QM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_QM_FIFO_OVERFLOW) >> 13)
				, "\n\tRQ_SRC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_SRC_FIFO_OVERFLOW) >> 12)
				, "\n\tRQ_PBF_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RQ_PBF_FIFO_OVERFLOW) >> 11)
				, "\n\tHST_CPL_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_HST_CPL_SYNC_FIFO_ERR) >> 10)
				, "\n\tHST_DATA_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_HST_DATA_SYNC_FIFO_ERR) >> 9)
				, "\n\tHST_HDR_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_HST_HDR_SYNC_FIFO_ERR) >> 8)
				, "\n\tHST_SRC_FIFO5_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_HST_SRC_FIFO5_ERR) >> 7)
				, "\n\tHST_SRC_FIFO4_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_HST_SRC_FIFO4_ERR) >> 6)
				, "\n\tHST_SRC_FIFO3_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_HST_SRC_FIFO3_ERR) >> 5)
				, "\n\tHST_SRC_FIFO2_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_HST_SRC_FIFO2_ERR) >> 4)
				, "\n\tHST_SRC_FIFO1_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_HST_SRC_FIFO1_ERR) >> 3)
				, "\n\tRD_POP_PBF_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RD_POP_PBF_ERROR) >> 2)
				, "\n\tRD_POP_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_RD_POP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x103070UL: // (PXP_REG_PXP_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP_REG_PXP_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_TSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_WR_TSDM_FIFO_OVERFLOW) >> 31)
				, "\n\tWR_XSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_WR_XSDM_FIFO_OVERFLOW) >> 30)
				, "\n\tWR_USDMDP_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_WR_USDMDP_FIFO_OVERFLOW) >> 29)
				, "\n\tWR_USDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_WR_USDM_FIFO_OVERFLOW) >> 28)
				, "\n\tWR_TM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_WR_TM_FIFO_OVERFLOW) >> 27)
				, "\n\tWR_QM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_WR_QM_FIFO_OVERFLOW) >> 26)
				, "\n\tWR_SRC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_WR_SRC_FIFO_OVERFLOW) >> 25)
				, "\n\tRQ_DBG_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_DBG_FIFO_OVERFLOW) >> 24)
				, "\n\tRQ_HC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_HC_FIFO_OVERFLOW) >> 23)
				, "\n\tRQ_DMAE_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_DMAE_FIFO_OVERFLOW) >> 22)
				, "\n\tRQ_CDURD_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_CDURD_FIFO_OVERFLOW) >> 21)
				, "\n\tRQ_CDUWR_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_CDUWR_FIFO_OVERFLOW) >> 20)
				, "\n\tRQ_CSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_CSDM_FIFO_OVERFLOW) >> 19)
				, "\n\tRQ_TSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_TSDM_FIFO_OVERFLOW) >> 18)
				, "\n\tRQ_XSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_XSDM_FIFO_OVERFLOW) >> 17)
				, "\n\tRQ_USDMDP_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_USDMDP_FIFO_OVERFLOW) >> 16)
				, "\n\tRQ_USDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_USDM_FIFO_OVERFLOW) >> 15)
				, "\n\tRQ_TM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_TM_FIFO_OVERFLOW) >> 14)
				, "\n\tRQ_QM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_QM_FIFO_OVERFLOW) >> 13)
				, "\n\tRQ_SRC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_SRC_FIFO_OVERFLOW) >> 12)
				, "\n\tRQ_PBF_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RQ_PBF_FIFO_OVERFLOW) >> 11)
				, "\n\tHST_CPL_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_HST_CPL_SYNC_FIFO_ERR) >> 10)
				, "\n\tHST_DATA_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_HST_DATA_SYNC_FIFO_ERR) >> 9)
				, "\n\tHST_HDR_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_HST_HDR_SYNC_FIFO_ERR) >> 8)
				, "\n\tHST_SRC_FIFO5_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_HST_SRC_FIFO5_ERR) >> 7)
				, "\n\tHST_SRC_FIFO4_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_HST_SRC_FIFO4_ERR) >> 6)
				, "\n\tHST_SRC_FIFO3_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_HST_SRC_FIFO3_ERR) >> 5)
				, "\n\tHST_SRC_FIFO2_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_HST_SRC_FIFO2_ERR) >> 4)
				, "\n\tHST_SRC_FIFO1_ERR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_HST_SRC_FIFO1_ERR) >> 3)
				, "\n\tRD_POP_PBF_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RD_POP_PBF_ERROR) >> 2)
				, "\n\tRD_POP_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_RD_POP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x103074UL: // (PXP_REG_PXP_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP_REG_PXP_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_TSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_WR_TSDM_FIFO_OVERFLOW) >> 31)
				, "\n\tWR_XSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_WR_XSDM_FIFO_OVERFLOW) >> 30)
				, "\n\tWR_USDMDP_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_WR_USDMDP_FIFO_OVERFLOW) >> 29)
				, "\n\tWR_USDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_WR_USDM_FIFO_OVERFLOW) >> 28)
				, "\n\tWR_TM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_WR_TM_FIFO_OVERFLOW) >> 27)
				, "\n\tWR_QM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_WR_QM_FIFO_OVERFLOW) >> 26)
				, "\n\tWR_SRC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_WR_SRC_FIFO_OVERFLOW) >> 25)
				, "\n\tRQ_DBG_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_DBG_FIFO_OVERFLOW) >> 24)
				, "\n\tRQ_HC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_HC_FIFO_OVERFLOW) >> 23)
				, "\n\tRQ_DMAE_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_DMAE_FIFO_OVERFLOW) >> 22)
				, "\n\tRQ_CDURD_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_CDURD_FIFO_OVERFLOW) >> 21)
				, "\n\tRQ_CDUWR_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_CDUWR_FIFO_OVERFLOW) >> 20)
				, "\n\tRQ_CSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_CSDM_FIFO_OVERFLOW) >> 19)
				, "\n\tRQ_TSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_TSDM_FIFO_OVERFLOW) >> 18)
				, "\n\tRQ_XSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_XSDM_FIFO_OVERFLOW) >> 17)
				, "\n\tRQ_USDMDP_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_USDMDP_FIFO_OVERFLOW) >> 16)
				, "\n\tRQ_USDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_USDM_FIFO_OVERFLOW) >> 15)
				, "\n\tRQ_TM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_TM_FIFO_OVERFLOW) >> 14)
				, "\n\tRQ_QM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_QM_FIFO_OVERFLOW) >> 13)
				, "\n\tRQ_SRC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_SRC_FIFO_OVERFLOW) >> 12)
				, "\n\tRQ_PBF_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RQ_PBF_FIFO_OVERFLOW) >> 11)
				, "\n\tHST_CPL_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_HST_CPL_SYNC_FIFO_ERR) >> 10)
				, "\n\tHST_DATA_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_HST_DATA_SYNC_FIFO_ERR) >> 9)
				, "\n\tHST_HDR_SYNC_FIFO_ERR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_HST_HDR_SYNC_FIFO_ERR) >> 8)
				, "\n\tHST_SRC_FIFO5_ERR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_HST_SRC_FIFO5_ERR) >> 7)
				, "\n\tHST_SRC_FIFO4_ERR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_HST_SRC_FIFO4_ERR) >> 6)
				, "\n\tHST_SRC_FIFO3_ERR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_HST_SRC_FIFO3_ERR) >> 5)
				, "\n\tHST_SRC_FIFO2_ERR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_HST_SRC_FIFO2_ERR) >> 4)
				, "\n\tHST_SRC_FIFO1_ERR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_HST_SRC_FIFO1_ERR) >> 3)
				, "\n\tRD_POP_PBF_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RD_POP_PBF_ERROR) >> 2)
				, "\n\tRD_POP_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_RD_POP_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & PXP_PXP_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x103078UL: // (PXP_REG_PXP_INT_STS_1: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP_REG_PXP_INT_STS_1 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tHST_PERMISSION_VIOLATION :"
				, (u32_t)((data & PXP_PXP_INT_STS_1_REG_HST_PERMISSION_VIOLATION) >> 7)
				, "\n\tHST_INCORRECT_ACCESS :"
				, (u32_t)((data & PXP_PXP_INT_STS_1_REG_HST_INCORRECT_ACCESS) >> 6)
				, "\n\tHST_VF_DISABLED_ACCESS :"
				, (u32_t)((data & PXP_PXP_INT_STS_1_REG_HST_VF_DISABLED_ACCESS) >> 5)
				, "\n\tWR_HC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_1_REG_WR_HC_FIFO_OVERFLOW) >> 4)
				, "\n\tWR_DMAE_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_1_REG_WR_DMAE_FIFO_OVERFLOW) >> 3)
				, "\n\tWR_DBG_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_1_REG_WR_DBG_FIFO_OVERFLOW) >> 2)
				, "\n\tWR_CDUWR_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_1_REG_WR_CDUWR_FIFO_OVERFLOW) >> 1)
				, "\n\tWR_CSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_1_REG_WR_CSDM_FIFO_OVERFLOW) >> 0));
			break;
		case 0x10307cUL: // (PXP_REG_PXP_INT_STS_CLR_1: ACCESS:RC  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP_REG_PXP_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0x8 )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tHST_PERMISSION_VIOLATION :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_1_REG_HST_PERMISSION_VIOLATION) >> 7)
				, "\n\tHST_INCORRECT_ACCESS :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_1_REG_HST_INCORRECT_ACCESS) >> 6)
				, "\n\tHST_VF_DISABLED_ACCESS :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_1_REG_HST_VF_DISABLED_ACCESS) >> 5)
				, "\n\tWR_HC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_1_REG_WR_HC_FIFO_OVERFLOW) >> 4)
				, "\n\tWR_DMAE_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_1_REG_WR_DMAE_FIFO_OVERFLOW) >> 3)
				, "\n\tWR_DBG_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_1_REG_WR_DBG_FIFO_OVERFLOW) >> 2)
				, "\n\tWR_CDUWR_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_1_REG_WR_CDUWR_FIFO_OVERFLOW) >> 1)
				, "\n\tWR_CSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_CLR_1_REG_WR_CSDM_FIFO_OVERFLOW) >> 0));
			break;
		case 0x103080UL: // (PXP_REG_PXP_INT_STS_WR_1: ACCESS:WR  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP_REG_PXP_INT_STS_WR_1 (ACCESS:WR  DataWidth:0x8 )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tHST_PERMISSION_VIOLATION :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_1_REG_HST_PERMISSION_VIOLATION) >> 7)
				, "\n\tHST_INCORRECT_ACCESS :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_1_REG_HST_INCORRECT_ACCESS) >> 6)
				, "\n\tHST_VF_DISABLED_ACCESS :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_1_REG_HST_VF_DISABLED_ACCESS) >> 5)
				, "\n\tWR_HC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_1_REG_WR_HC_FIFO_OVERFLOW) >> 4)
				, "\n\tWR_DMAE_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_1_REG_WR_DMAE_FIFO_OVERFLOW) >> 3)
				, "\n\tWR_DBG_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_1_REG_WR_DBG_FIFO_OVERFLOW) >> 2)
				, "\n\tWR_CDUWR_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_1_REG_WR_CDUWR_FIFO_OVERFLOW) >> 1)
				, "\n\tWR_CSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_STS_WR_1_REG_WR_CSDM_FIFO_OVERFLOW) >> 0));
			break;
		case 0x103084UL: // (PXP_REG_PXP_INT_MASK_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nPXP_REG_PXP_INT_MASK_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tHST_PERMISSION_VIOLATION :"
				, (u32_t)((data & PXP_PXP_INT_MASK_1_REG_HST_PERMISSION_VIOLATION) >> 7)
				, "\n\tHST_INCORRECT_ACCESS :"
				, (u32_t)((data & PXP_PXP_INT_MASK_1_REG_HST_INCORRECT_ACCESS) >> 6)
				, "\n\tHST_VF_DISABLED_ACCESS :"
				, (u32_t)((data & PXP_PXP_INT_MASK_1_REG_HST_VF_DISABLED_ACCESS) >> 5)
				, "\n\tWR_HC_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_1_REG_WR_HC_FIFO_OVERFLOW) >> 4)
				, "\n\tWR_DMAE_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_1_REG_WR_DMAE_FIFO_OVERFLOW) >> 3)
				, "\n\tWR_DBG_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_1_REG_WR_DBG_FIFO_OVERFLOW) >> 2)
				, "\n\tWR_CDUWR_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_1_REG_WR_CDUWR_FIFO_OVERFLOW) >> 1)
				, "\n\tWR_CSDM_FIFO_OVERFLOW :"
				, (u32_t)((data & PXP_PXP_INT_MASK_1_REG_WR_CSDM_FIFO_OVERFLOW) >> 0));
			break;
		case 0x103088UL: // (PXP_REG_PXP_PRTY_STS: ACCESS:R  DataWidth:0x1b ) 
			sprintf(disp_str, "\nPXP_REG_PXP_PRTY_STS (ACCESS:R  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPERM_TABLE :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_PERM_TABLE) >> 26)
				, "\n\tDBGSYN_FIFO :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_DBGSYN_FIFO) >> 25)
				, "\n\tRQ_PHY_FIFO_PRTY :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RQ_PHY_FIFO_PRTY) >> 24)
				, "\n\tRQ_WDONE_FIFO_PRTY :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RQ_WDONE_FIFO_PRTY) >> 23)
				, "\n\tHST_INBND_INT_MEM3 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_INBND_INT_MEM3) >> 22)
				, "\n\tHST_INBND_INT_MEM2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_INBND_INT_MEM2) >> 21)
				, "\n\tHST_INBND_INT_MEM1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_INBND_INT_MEM1) >> 20)
				, "\n\tHST_SRC_FIFO5 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_SRC_FIFO5) >> 19)
				, "\n\tHST_SRC_FIFO4 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_SRC_FIFO4) >> 18)
				, "\n\tHST_SRC_FIFO3 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_SRC_FIFO3) >> 17)
				, "\n\tHST_SRC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_SRC_FIFO2) >> 16)
				, "\n\tHST_SRC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_SRC_FIFO1) >> 15)
				, "\n\tHST_DATA_SYNC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_DATA_SYNC_FIFO2) >> 14)
				, "\n\tHST_DATA_SYNC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_DATA_SYNC_FIFO1) >> 13)
				, "\n\tHST_HDR_SYNC_FIFO :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_HST_HDR_SYNC_FIFO) >> 12)
				, "\n\tRD_SYNC_FIFO10 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO10) >> 11)
				, "\n\tRD_SYNC_FIFO9 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO9) >> 10)
				, "\n\tRD_SYNC_FIFO8 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO8) >> 9)
				, "\n\tRD_SYNC_FIFO7 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO7) >> 8)
				, "\n\tRD_SYNC_FIFO6 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO6) >> 7)
				, "\n\tRD_SYNC_FIFO5 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO5) >> 6)
				, "\n\tRD_SYNC_FIFO4 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO4) >> 5)
				, "\n\tRD_SYNC_FIFO3 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO3) >> 4)
				, "\n\tRD_SYNC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO2) >> 3)
				, "\n\tRD_SYNC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO1) >> 2)
				, "\n\tRD_SYNC_FIFO0 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_RD_SYNC_FIFO0) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x10308cUL: // (PXP_REG_PXP_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x1b ) 
			sprintf(disp_str, "\nPXP_REG_PXP_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPERM_TABLE :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_PERM_TABLE) >> 26)
				, "\n\tDBGSYN_FIFO :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_DBGSYN_FIFO) >> 25)
				, "\n\tRQ_PHY_FIFO_PRTY :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RQ_PHY_FIFO_PRTY) >> 24)
				, "\n\tRQ_WDONE_FIFO_PRTY :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RQ_WDONE_FIFO_PRTY) >> 23)
				, "\n\tHST_INBND_INT_MEM3 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_INBND_INT_MEM3) >> 22)
				, "\n\tHST_INBND_INT_MEM2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_INBND_INT_MEM2) >> 21)
				, "\n\tHST_INBND_INT_MEM1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_INBND_INT_MEM1) >> 20)
				, "\n\tHST_SRC_FIFO5 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_SRC_FIFO5) >> 19)
				, "\n\tHST_SRC_FIFO4 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_SRC_FIFO4) >> 18)
				, "\n\tHST_SRC_FIFO3 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_SRC_FIFO3) >> 17)
				, "\n\tHST_SRC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_SRC_FIFO2) >> 16)
				, "\n\tHST_SRC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_SRC_FIFO1) >> 15)
				, "\n\tHST_DATA_SYNC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_DATA_SYNC_FIFO2) >> 14)
				, "\n\tHST_DATA_SYNC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_DATA_SYNC_FIFO1) >> 13)
				, "\n\tHST_HDR_SYNC_FIFO :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_HST_HDR_SYNC_FIFO) >> 12)
				, "\n\tRD_SYNC_FIFO10 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO10) >> 11)
				, "\n\tRD_SYNC_FIFO9 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO9) >> 10)
				, "\n\tRD_SYNC_FIFO8 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO8) >> 9)
				, "\n\tRD_SYNC_FIFO7 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO7) >> 8)
				, "\n\tRD_SYNC_FIFO6 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO6) >> 7)
				, "\n\tRD_SYNC_FIFO5 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO5) >> 6)
				, "\n\tRD_SYNC_FIFO4 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO4) >> 5)
				, "\n\tRD_SYNC_FIFO3 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO3) >> 4)
				, "\n\tRD_SYNC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO2) >> 3)
				, "\n\tRD_SYNC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO1) >> 2)
				, "\n\tRD_SYNC_FIFO0 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_RD_SYNC_FIFO0) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x103090UL: // (PXP_REG_PXP_PRTY_STS_WR: ACCESS:WR  DataWidth:0x1b ) 
			sprintf(disp_str, "\nPXP_REG_PXP_PRTY_STS_WR (ACCESS:WR  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPERM_TABLE :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_PERM_TABLE) >> 26)
				, "\n\tDBGSYN_FIFO :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_DBGSYN_FIFO) >> 25)
				, "\n\tRQ_PHY_FIFO_PRTY :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RQ_PHY_FIFO_PRTY) >> 24)
				, "\n\tRQ_WDONE_FIFO_PRTY :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RQ_WDONE_FIFO_PRTY) >> 23)
				, "\n\tHST_INBND_INT_MEM3 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_INBND_INT_MEM3) >> 22)
				, "\n\tHST_INBND_INT_MEM2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_INBND_INT_MEM2) >> 21)
				, "\n\tHST_INBND_INT_MEM1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_INBND_INT_MEM1) >> 20)
				, "\n\tHST_SRC_FIFO5 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_SRC_FIFO5) >> 19)
				, "\n\tHST_SRC_FIFO4 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_SRC_FIFO4) >> 18)
				, "\n\tHST_SRC_FIFO3 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_SRC_FIFO3) >> 17)
				, "\n\tHST_SRC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_SRC_FIFO2) >> 16)
				, "\n\tHST_SRC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_SRC_FIFO1) >> 15)
				, "\n\tHST_DATA_SYNC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_DATA_SYNC_FIFO2) >> 14)
				, "\n\tHST_DATA_SYNC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_DATA_SYNC_FIFO1) >> 13)
				, "\n\tHST_HDR_SYNC_FIFO :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_HST_HDR_SYNC_FIFO) >> 12)
				, "\n\tRD_SYNC_FIFO10 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO10) >> 11)
				, "\n\tRD_SYNC_FIFO9 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO9) >> 10)
				, "\n\tRD_SYNC_FIFO8 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO8) >> 9)
				, "\n\tRD_SYNC_FIFO7 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO7) >> 8)
				, "\n\tRD_SYNC_FIFO6 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO6) >> 7)
				, "\n\tRD_SYNC_FIFO5 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO5) >> 6)
				, "\n\tRD_SYNC_FIFO4 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO4) >> 5)
				, "\n\tRD_SYNC_FIFO3 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO3) >> 4)
				, "\n\tRD_SYNC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO2) >> 3)
				, "\n\tRD_SYNC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO1) >> 2)
				, "\n\tRD_SYNC_FIFO0 :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_RD_SYNC_FIFO0) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PXP_PXP_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x103094UL: // (PXP_REG_PXP_PRTY_MASK: ACCESS:RW  DataWidth:0x1b ) 
			sprintf(disp_str, "\nPXP_REG_PXP_PRTY_MASK (ACCESS:RW  DataWidth:0x1b )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPERM_TABLE :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_PERM_TABLE) >> 26)
				, "\n\tDBGSYN_FIFO :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_DBGSYN_FIFO) >> 25)
				, "\n\tRQ_PHY_FIFO_PRTY :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RQ_PHY_FIFO_PRTY) >> 24)
				, "\n\tRQ_WDONE_FIFO_PRTY :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RQ_WDONE_FIFO_PRTY) >> 23)
				, "\n\tHST_INBND_INT_MEM3 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_INBND_INT_MEM3) >> 22)
				, "\n\tHST_INBND_INT_MEM2 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_INBND_INT_MEM2) >> 21)
				, "\n\tHST_INBND_INT_MEM1 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_INBND_INT_MEM1) >> 20)
				, "\n\tHST_SRC_FIFO5 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_SRC_FIFO5) >> 19)
				, "\n\tHST_SRC_FIFO4 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_SRC_FIFO4) >> 18)
				, "\n\tHST_SRC_FIFO3 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_SRC_FIFO3) >> 17)
				, "\n\tHST_SRC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_SRC_FIFO2) >> 16)
				, "\n\tHST_SRC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_SRC_FIFO1) >> 15)
				, "\n\tHST_DATA_SYNC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_DATA_SYNC_FIFO2) >> 14)
				, "\n\tHST_DATA_SYNC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_DATA_SYNC_FIFO1) >> 13)
				, "\n\tHST_HDR_SYNC_FIFO :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_HST_HDR_SYNC_FIFO) >> 12)
				, "\n\tRD_SYNC_FIFO10 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO10) >> 11)
				, "\n\tRD_SYNC_FIFO9 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO9) >> 10)
				, "\n\tRD_SYNC_FIFO8 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO8) >> 9)
				, "\n\tRD_SYNC_FIFO7 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO7) >> 8)
				, "\n\tRD_SYNC_FIFO6 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO6) >> 7)
				, "\n\tRD_SYNC_FIFO5 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO5) >> 6)
				, "\n\tRD_SYNC_FIFO4 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO4) >> 5)
				, "\n\tRD_SYNC_FIFO3 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO3) >> 4)
				, "\n\tRD_SYNC_FIFO2 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO2) >> 3)
				, "\n\tRD_SYNC_FIFO1 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO1) >> 2)
				, "\n\tRD_SYNC_FIFO0 :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_RD_SYNC_FIFO0) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & PXP_PXP_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x103098UL: // (PXP_REG_HST_HOST_STRICT_PRIORITY: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_HOST_STRICT_PRIORITY (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When 1; host requests have strict priority on internal write\n     requests; as in A0. When 0; arbiter alternately chooses host requests\n     and internal write requests.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x10309cUL: // (PXP_REG_HST_DISCARD_INTERNAL_WRITES_STATUS: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPXP_REG_HST_DISCARD_INTERNAL_WRITES_STATUS (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: debug only: A bit mask for all PSWHST internal write clients\n    . '1' means this PSWHST is discarding inputs from this client. Each bit\n     should update accoring to 'hst_discard_internal_writes' register when\n     the state machine is idle.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030a0UL: // (PXP_REG_HST_DISCARD_DOORBELLS_STATUS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_DISCARD_DOORBELLS_STATUS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: debug only: '1' means this PSWHST is discarding doorbells\n    . This bit should update accoring to 'hst_discard_doorbells' register \n    when the state machine is idle\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030acUL: // (PXP_REG_HST_HDR_MEM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nPXP_REG_HST_HDR_MEM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of hst_hdr_mem memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030b0UL: // (PXP_REG_PXP_ECO_RSRV: ACCESS:RW  DataWidth:0xc  Multi Field Register  ) 
			sprintf(disp_str, "\nPXP_REG_PXP_ECO_RSRV (ACCESS:RW  DataWidth:0xc  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_ECO_RESERVED :"
				, (u32_t)((data & PXP_PXP_ECO_RSRV_REG_WR_ECO_RESERVED) >> 6)
				, "\n\tRQ_ECO_RESERVED :"
				, (u32_t)((data & PXP_PXP_ECO_RSRV_REG_RQ_ECO_RESERVED) >> 0));
			break;
		case 0x1030b8UL: // (PXP_REG_HST_VF_DISABLED_ERROR_DATA: ACCESS:R  DataWidth:0xf ) 
			sprintf(disp_str, "\nPXP_REG_HST_VF_DISABLED_ERROR_DATA (ACCESS:R  DataWidth:0xf )" \
				"\nDescription: The FID of the first access to a disabled VF; the format \n    is [14:12] - pfid; [11:6] - vfid; [5] - vf_valid; [4:1] - client (0 USDM\n    ; 1 CSDM; 2 XSDM; 3 TSDM; 4 HC; 5 GRC; 6 DQ; 7 RESERVED SPACE; 8 ATC); [\n    0] - w_nr(0-read req; 1- write req). The data is written only when the\n     valid bit is reset. and it is stays stable until it is reset by the read\n     from interrupt_clr register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030bcUL: // (PXP_REG_HST_VF_DISABLED_ERROR_VALID: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_VF_DISABLED_ERROR_VALID (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: 1 - An error request is logged and wasn't handled yet. The\n     valid bit is reset when the relevant interrupt register is read (PXP_REG_INT_STS_CLR_1\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030c0UL: // (PXP_REG_HST_VF_DISABLED_ERROR_ADDRESS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP_REG_HST_VF_DISABLED_ERROR_ADDRESS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The address of the first access to a disabled VF.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030c4UL: // (PXP_REG_HST_SDM_MAX_LENGTH: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nPXP_REG_HST_SDM_MAX_LENGTH (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Maximum write transaction data in DWs that is sent to SDMs\n     and IGU. Write requests with bigger length are discarded in PSWHST. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030c8UL: // (PXP_REG_HST_INCORRECT_ACCESS_DATA: ACCESS:R  DataWidth:0x1f ) 
			sprintf(disp_str, "\nPXP_REG_HST_INCORRECT_ACCESS_DATA (ACCESS:R  DataWidth:0x1f )" \
				"\nDescription: The data of the first incorrect access. the format is: [30\n    :23] - length; [22:15] - byte enable; [14:12] - pfid; [11:6] - vfid; [\n    5] - vf_valid; [4:1] - client (0 USDM; 1 CSDM; 2 XSDM; 3 TSDM; 4 HC; 5\n     GRC; 6 DQ; 7 RESERVED SPACE; 8 ATC); [0] - w_nr(0-read req; 1- write \n    req). The data is written only when the valid bit is reset. and it is \n    stays stable until it is reset by the read from interrupt_clr register\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030ccUL: // (PXP_REG_HST_INCORRECT_ACCESS_VALID: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_INCORRECT_ACCESS_VALID (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: 1 - An incorrect access is logged. The valid bit is reset\n     when the relevant interrupt register is read (PXP_REG_INT_STS_CLR_1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030d0UL: // (PXP_REG_HST_INCORRECT_ACCESS_ADDRESS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP_REG_HST_INCORRECT_ACCESS_ADDRESS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: The address of the first incorrect access (length and alignement\n     combination).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030d8UL: // (PXP_REG_HST_ZONE_PERM_TABLE_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_ZONE_PERM_TABLE_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Start the Init sequence for the zone permission table\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030dcUL: // (PXP_REG_HST_ZONE_PERM_TABLE_INIT_DONE: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_ZONE_PERM_TABLE_INIT_DONE (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Done indication for the permission table's init sequence\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030e0UL: // (PXP_REG_HST_PER_VIOLATION_VALID: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_PER_VIOLATION_VALID (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: 1- permission violation data is logged. The valid bit is \n    reset when the relevant interrupt register is read (PXP_REG_INT_STS_CLR_1\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030e4UL: // (PXP_REG_HST_PER_VIOLATION_DATA: ACCESS:R  DataWidth:0xe ) 
			sprintf(disp_str, "\nPXP_REG_HST_PER_VIOLATION_DATA (ACCESS:R  DataWidth:0xe )" \
				"\nDescription: Log of the permission violation: {QID[7:0];VFID[5:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030a4UL: // (PXP_REG_HST_DISCARD_DOORBELLS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_DISCARD_DOORBELLS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When 1; doorbells are discarded and not passed to doorbell\n     queue block. Should be used for close the gates.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030a8UL: // (PXP_REG_HST_DISCARD_INTERNAL_WRITES: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_DISCARD_INTERNAL_WRITES (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When 1; new internal writes arriving to the block are discarded\n    . Should be used for close the gates.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030e8UL: // (PXP_REG_DBG_OUT_DATA_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP_REG_DBG_OUT_DATA_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 32 lsb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030ecUL: // (PXP_REG_DBG_OUT_DATA_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nPXP_REG_DBG_OUT_DATA_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 32 msb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030f0UL: // (PXP_REG_DBG_OUT_FRAME: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP_REG_DBG_OUT_FRAME (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 4 frame bits that goes to the DBG block. Bit0\n     is the frame of  data byte0; Bit1 is the frame of data byte1; Bit2 is\n     the frame of data byte2; Bit3 is the frame of data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030f4UL: // (PXP_REG_DBG_OUT_VALID: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nPXP_REG_DBG_OUT_VALID (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 4 valid bits that goes to the DBG block. Bit0\n     validtes data byte0; Bit1 validates data byte1; Bit2 validates data byte2\n    ; Bit3 validates data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103400UL: // (PXP_REG_HST_ZONE_PERMISSION_TABLE: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nPXP_REG_HST_ZONE_PERMISSION_TABLE (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Indirect access to the permission table. The fields are : {\n    Valid; VFID[5:0]}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x103800UL: // (PXP_REG_HST_INBOUND_INT: ACCESS:WB  DataWidth:0xa0 ) 
			sprintf(disp_str, "\nPXP_REG_HST_INBOUND_INT (ACCESS:WB  DataWidth:0xa0 )" \
				"\nDescription: Used for initialization of the inbound interrupts memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030b4UL: // (PXP_REG_HST_PORT4MODE_EN: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nPXP_REG_HST_PORT4MODE_EN (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: 4 Port mode enable bit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030d4UL: // (PXP_REG_PXP_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPXP_REG_PXP_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1030f8UL: // (PXP_REG_PXP_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nPXP_REG_PXP_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168000UL: // (QM_REG_XQM_WRC_FIFOLVL: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nQM_REG_XQM_WRC_FIFOLVL (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Keep the fill level of the fifo from write client 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168004UL: // (QM_REG_XQM_WRC_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_XQM_WRC_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable the write client 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168008UL: // (QM_REG_UQM_WRC_FIFOLVL: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nQM_REG_UQM_WRC_FIFOLVL (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Keep the fill level of the fifo from write client 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16800cUL: // (QM_REG_UQM_WRC_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_UQM_WRC_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable the write client 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168010UL: // (QM_REG_TQM_WRC_FIFOLVL: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nQM_REG_TQM_WRC_FIFOLVL (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Keep the fill level of the fifo from write client 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168014UL: // (QM_REG_TQM_WRC_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_TQM_WRC_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable the write client 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168018UL: // (QM_REG_CQM_WRC_FIFOLVL: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nQM_REG_CQM_WRC_FIFOLVL (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Keep the fill level of the fifo from write client 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16801cUL: // (QM_REG_CQM_WRC_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_CQM_WRC_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable the write client 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168020UL: // (QM_REG_CONNNUM_0: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nQM_REG_CONNNUM_0 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: The number of connections divided by 16 which dictates the\n     size of each queue which belongs to even function number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168024UL: // (QM_REG_CONNNUM_1: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nQM_REG_CONNNUM_1 (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: The number of connections divided by 16 which dictates the\n     size of each queue which belongs to odd function number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168028UL: // (QM_REG_QSTATUS_LOW: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_QSTATUS_LOW (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current queues in pipeline: Queues from 0 to 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16802cUL: // (QM_REG_QSTATUS_HIGH: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_QSTATUS_HIGH (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Current queues in pipeline: Queues from 32 to 63\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168030UL: // (QM_REG_CTXREG_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_CTXREG_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The context regions sent in the CFC load request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168034UL: // (QM_REG_CTXREG_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_CTXREG_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The context regions sent in the CFC load request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168038UL: // (QM_REG_CTXREG_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_CTXREG_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The context regions sent in the CFC load request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16803cUL: // (QM_REG_CTXREG_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_CTXREG_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The context regions sent in the CFC load request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168040UL: // (QM_REG_ACTCTRINITVAL_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_ACTCTRINITVAL_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The activity counter initial increment value sent in the \n    load request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168044UL: // (QM_REG_ACTCTRINITVAL_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_ACTCTRINITVAL_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The activity counter initial increment value sent in the \n    load request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168048UL: // (QM_REG_ACTCTRINITVAL_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_ACTCTRINITVAL_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The activity counter initial increment value sent in the \n    load request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16804cUL: // (QM_REG_ACTCTRINITVAL_3: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_ACTCTRINITVAL_3 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The activity counter initial increment value sent in the \n    load request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168050UL: // (QM_REG_PCIREQQID: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nQM_REG_PCIREQQID (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The virtual Queue ID used in the PCI request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168054UL: // (QM_REG_PCIREQAT: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nQM_REG_PCIREQAT (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The PCI attributes field used in the PCI request. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168058UL: // (QM_REG_OVFQNUM: ACCESS:RC  DataWidth:0x6 ) 
			sprintf(disp_str, "\nQM_REG_OVFQNUM (ACCESS:RC  DataWidth:0x6 )" \
				"\nDescription: the Q were the qverflow occurs\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16805cUL: // (QM_REG_OVFERROR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_OVFERROR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: A flag to indicate that overflow error occurred in one of\n     the queues.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168060UL: // (QM_REG_VOQINITCREDIT_0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168064UL: // (QM_REG_VOQINITCREDIT_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168068UL: // (QM_REG_VOQINITCREDIT_2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16806cUL: // (QM_REG_VOQINITCREDIT_3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168070UL: // (QM_REG_VOQINITCREDIT_4: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_4 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168074UL: // (QM_REG_VOQINITCREDIT_5: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_5 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168078UL: // (QM_REG_VOQINITCREDIT_6: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_6 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16807cUL: // (QM_REG_VOQINITCREDIT_7: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_7 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168080UL: // (QM_REG_VOQINITCREDIT_8: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_8 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168084UL: // (QM_REG_VOQINITCREDIT_9: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_9 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168088UL: // (QM_REG_VOQINITCREDIT_10: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_10 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16808cUL: // (QM_REG_VOQINITCREDIT_11: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQINITCREDIT_11 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The init and maximum credit for each VoQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168090UL: // (QM_REG_VOQCREDITAFULLTHR: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDITAFULLTHR (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The credit value that if above the QM is considered almost\n     full\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168094UL: // (QM_REG_BYTECREDITAFULLTHR: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_BYTECREDITAFULLTHR (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: The byte credit value that if above the QM is considered \n    almost full\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168098UL: // (QM_REG_AEMPTYTHR: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_AEMPTYTHR (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: Queue almost empty threshold\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16809cUL: // (QM_REG_TASKCRDCOST_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680a0UL: // (QM_REG_TASKCRDCOST_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680a4UL: // (QM_REG_TASKCRDCOST_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680a8UL: // (QM_REG_TASKCRDCOST_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680acUL: // (QM_REG_TASKCRDCOST_4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680b0UL: // (QM_REG_TASKCRDCOST_5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680b4UL: // (QM_REG_TASKCRDCOST_6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680b8UL: // (QM_REG_TASKCRDCOST_7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680bcUL: // (QM_REG_TASKCRDCOST_8: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_8 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680c0UL: // (QM_REG_TASKCRDCOST_9: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_9 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680c4UL: // (QM_REG_TASKCRDCOST_10: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_10 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680c8UL: // (QM_REG_TASKCRDCOST_11: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_TASKCRDCOST_11 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The credit cost per every task in the QM. A value per each\n     VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680ccUL: // (QM_REG_CMINITCRD_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_CMINITCRD_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial credit for interface \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680d0UL: // (QM_REG_CMINITCRD_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_CMINITCRD_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial credit for interface \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680d4UL: // (QM_REG_CMINITCRD_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_CMINITCRD_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial credit for interface \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680d8UL: // (QM_REG_CMINITCRD_3: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_CMINITCRD_3 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial credit for interface \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680dcUL: // (QM_REG_CMINITCRD_4: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_CMINITCRD_4 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial credit for interface \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680e0UL: // (QM_REG_CMINITCRD_5: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_CMINITCRD_5 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial credit for interface \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680e4UL: // (QM_REG_CMINITCRD_6: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_CMINITCRD_6 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial credit for interface \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680e8UL: // (QM_REG_CMINITCRD_7: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_CMINITCRD_7 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial credit for interface \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680ecUL: // (QM_REG_CMINTEN: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_CMINTEN (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: A mask bit per CM interface. If this bit is 0 then this interface\n     is masked\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680f0UL: // (QM_REG_ENSEC: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_ENSEC (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: If cleared then the secondary interface will not be served\n     by the RR arbiter\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680f4UL: // (QM_REG_QVOQIDX_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680f8UL: // (QM_REG_QVOQIDX_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1680fcUL: // (QM_REG_QVOQIDX_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168100UL: // (QM_REG_QVOQIDX_3: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_3 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168104UL: // (QM_REG_QVOQIDX_4: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_4 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168108UL: // (QM_REG_QVOQIDX_5: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_5 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16810cUL: // (QM_REG_QVOQIDX_6: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_6 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168110UL: // (QM_REG_QVOQIDX_7: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_7 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168114UL: // (QM_REG_QVOQIDX_8: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_8 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168118UL: // (QM_REG_QVOQIDX_9: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_9 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16811cUL: // (QM_REG_QVOQIDX_10: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_10 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168120UL: // (QM_REG_QVOQIDX_11: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_11 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168124UL: // (QM_REG_QVOQIDX_12: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_12 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168128UL: // (QM_REG_QVOQIDX_13: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_13 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16812cUL: // (QM_REG_QVOQIDX_14: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_14 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168130UL: // (QM_REG_QVOQIDX_15: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_15 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168134UL: // (QM_REG_QVOQIDX_16: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_16 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168138UL: // (QM_REG_QVOQIDX_17: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_17 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16813cUL: // (QM_REG_QVOQIDX_18: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_18 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168140UL: // (QM_REG_QVOQIDX_19: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_19 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168144UL: // (QM_REG_QVOQIDX_20: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_20 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168148UL: // (QM_REG_QVOQIDX_21: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_21 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16814cUL: // (QM_REG_QVOQIDX_22: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_22 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168150UL: // (QM_REG_QVOQIDX_23: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_23 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168154UL: // (QM_REG_QVOQIDX_24: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_24 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168158UL: // (QM_REG_QVOQIDX_25: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_25 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16815cUL: // (QM_REG_QVOQIDX_26: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_26 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168160UL: // (QM_REG_QVOQIDX_27: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_27 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168164UL: // (QM_REG_QVOQIDX_28: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_28 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168168UL: // (QM_REG_QVOQIDX_29: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_29 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16816cUL: // (QM_REG_QVOQIDX_30: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_30 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168170UL: // (QM_REG_QVOQIDX_31: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_31 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168174UL: // (QM_REG_QVOQIDX_32: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_32 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168178UL: // (QM_REG_QVOQIDX_33: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_33 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16817cUL: // (QM_REG_QVOQIDX_34: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_34 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168180UL: // (QM_REG_QVOQIDX_35: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_35 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168184UL: // (QM_REG_QVOQIDX_36: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_36 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168188UL: // (QM_REG_QVOQIDX_37: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_37 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16818cUL: // (QM_REG_QVOQIDX_38: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_38 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168190UL: // (QM_REG_QVOQIDX_39: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_39 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168194UL: // (QM_REG_QVOQIDX_40: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_40 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168198UL: // (QM_REG_QVOQIDX_41: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_41 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16819cUL: // (QM_REG_QVOQIDX_42: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_42 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681a0UL: // (QM_REG_QVOQIDX_43: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_43 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681a4UL: // (QM_REG_QVOQIDX_44: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_44 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681a8UL: // (QM_REG_QVOQIDX_45: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_45 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681acUL: // (QM_REG_QVOQIDX_46: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_46 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681b0UL: // (QM_REG_QVOQIDX_47: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_47 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681b4UL: // (QM_REG_QVOQIDX_48: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_48 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681b8UL: // (QM_REG_QVOQIDX_49: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_49 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681bcUL: // (QM_REG_QVOQIDX_50: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_50 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681c0UL: // (QM_REG_QVOQIDX_51: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_51 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681c4UL: // (QM_REG_QVOQIDX_52: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_52 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681c8UL: // (QM_REG_QVOQIDX_53: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_53 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681ccUL: // (QM_REG_QVOQIDX_54: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_54 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681d0UL: // (QM_REG_QVOQIDX_55: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_55 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681d4UL: // (QM_REG_QVOQIDX_56: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_56 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681d8UL: // (QM_REG_QVOQIDX_57: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_57 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681dcUL: // (QM_REG_QVOQIDX_58: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_58 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681e0UL: // (QM_REG_QVOQIDX_59: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_59 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681e4UL: // (QM_REG_QVOQIDX_60: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_60 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681e8UL: // (QM_REG_QVOQIDX_61: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_61 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681ecUL: // (QM_REG_QVOQIDX_62: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_62 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681f0UL: // (QM_REG_QVOQIDX_63: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_63 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Queue tied to VOQ\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681f4UL: // (QM_REG_CMINTVOQMASK_0: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_CMINTVOQMASK_0 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: A bit vector which indicates which one of the queues are \n    tied to interface 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681f8UL: // (QM_REG_CMINTVOQMASK_1: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_CMINTVOQMASK_1 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: A bit vector which indicates which one of the queues are \n    tied to interface 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1681fcUL: // (QM_REG_CMINTVOQMASK_2: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_CMINTVOQMASK_2 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: A bit vector which indicates which one of the queues are \n    tied to interface 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168200UL: // (QM_REG_CMINTVOQMASK_3: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_CMINTVOQMASK_3 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: A bit vector which indicates which one of the queues are \n    tied to interface 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168204UL: // (QM_REG_CMINTVOQMASK_4: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_CMINTVOQMASK_4 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: A bit vector which indicates which one of the queues are \n    tied to interface 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168208UL: // (QM_REG_CMINTVOQMASK_5: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_CMINTVOQMASK_5 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: A bit vector which indicates which one of the queues are \n    tied to interface 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16820cUL: // (QM_REG_CMINTVOQMASK_6: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_CMINTVOQMASK_6 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: A bit vector which indicates which one of the queues are \n    tied to interface 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168210UL: // (QM_REG_CMINTVOQMASK_7: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_CMINTVOQMASK_7 (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: A bit vector which indicates which one of the queues are \n    tied to interface 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168214UL: // (QM_REG_HWAEMPTYMASK_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_HWAEMPTYMASK_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: A mask register to mask the Almost empty signals which will\n     not be use for the almost empty indication to the HW block; queues 63\n    :32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168218UL: // (QM_REG_HWAEMPTYMASK_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_HWAEMPTYMASK_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: A mask register to mask the Almost empty signals which will\n     not be use for the almost empty indication to the HW block; queues 31\n    :0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16821cUL: // (QM_REG_ENBYTECRD_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_ENBYTECRD_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: A bit mask per each physical queue. If a bit is set then \n    the physical queue uses the byte credit; queues 63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168220UL: // (QM_REG_ENBYTECRD_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_ENBYTECRD_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: A bit mask per each physical queue. If a bit is set then \n    the physical queue uses the byte credit; queues 31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16822cUL: // (QM_REG_FUNCNUMSEL_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_FUNCNUMSEL_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168230UL: // (QM_REG_FUNCNUMSEL_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_FUNCNUMSEL_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: NA\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168234UL: // (QM_REG_BYTECRDCOST: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDCOST (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The byte credit cost for each task. This value is for all\n     byte credit counters\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168238UL: // (QM_REG_BYTECRDINITVAL: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDINITVAL (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: The initial byte credit value for all counters\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16823cUL: // (QM_REG_ENBYPVOQMASK: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_ENBYPVOQMASK (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: The VOQ mask used to select the VOQs which needs to be full\n     for bypass enable\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168240UL: // (QM_REG_VOQQMASK_0_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_0_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168244UL: // (QM_REG_VOQQMASK_0_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_0_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168248UL: // (QM_REG_VOQQMASK_1_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_1_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16824cUL: // (QM_REG_VOQQMASK_1_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_1_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168250UL: // (QM_REG_VOQQMASK_2_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_2_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168254UL: // (QM_REG_VOQQMASK_2_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_2_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168258UL: // (QM_REG_VOQQMASK_3_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_3_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16825cUL: // (QM_REG_VOQQMASK_3_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_3_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168260UL: // (QM_REG_VOQQMASK_4_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_4_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168264UL: // (QM_REG_VOQQMASK_4_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_4_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168268UL: // (QM_REG_VOQQMASK_5_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_5_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16826cUL: // (QM_REG_VOQQMASK_5_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_5_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168270UL: // (QM_REG_VOQQMASK_6_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_6_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168274UL: // (QM_REG_VOQQMASK_6_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_6_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168278UL: // (QM_REG_VOQQMASK_7_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_7_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16827cUL: // (QM_REG_VOQQMASK_7_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_7_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168280UL: // (QM_REG_VOQQMASK_8_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_8_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168284UL: // (QM_REG_VOQQMASK_8_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_8_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168288UL: // (QM_REG_VOQQMASK_9_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_9_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16828cUL: // (QM_REG_VOQQMASK_9_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_9_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168290UL: // (QM_REG_VOQQMASK_10_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_10_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168294UL: // (QM_REG_VOQQMASK_10_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_10_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168298UL: // (QM_REG_VOQQMASK_11_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_11_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     31-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16829cUL: // (QM_REG_VOQQMASK_11_MSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_11_MSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The physical queue number associated with each VOQ; queues\n     63-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682d0UL: // (QM_REG_VOQCREDIT_0: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_0 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682d4UL: // (QM_REG_VOQCREDIT_1: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_1 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682d8UL: // (QM_REG_VOQCREDIT_2: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_2 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682dcUL: // (QM_REG_VOQCREDIT_3: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_3 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682e0UL: // (QM_REG_VOQCREDIT_4: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_4 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682e4UL: // (QM_REG_VOQCREDIT_5: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_5 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682e8UL: // (QM_REG_VOQCREDIT_6: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_6 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682ecUL: // (QM_REG_VOQCREDIT_7: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_7 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682f0UL: // (QM_REG_VOQCREDIT_8: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_8 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682f4UL: // (QM_REG_VOQCREDIT_9: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_9 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682f8UL: // (QM_REG_VOQCREDIT_10: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_10 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682fcUL: // (QM_REG_VOQCREDIT_11: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nQM_REG_VOQCREDIT_11 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: The credit value for each VOQ. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168308UL: // (QM_REG_QTASKCTR_0: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_0 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16830cUL: // (QM_REG_QTASKCTR_1: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_1 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168310UL: // (QM_REG_QTASKCTR_2: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_2 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168314UL: // (QM_REG_QTASKCTR_3: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_3 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168318UL: // (QM_REG_QTASKCTR_4: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_4 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16831cUL: // (QM_REG_QTASKCTR_5: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_5 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168320UL: // (QM_REG_QTASKCTR_6: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_6 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168324UL: // (QM_REG_QTASKCTR_7: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_7 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168328UL: // (QM_REG_QTASKCTR_8: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_8 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16832cUL: // (QM_REG_QTASKCTR_9: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_9 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168330UL: // (QM_REG_QTASKCTR_10: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_10 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168334UL: // (QM_REG_QTASKCTR_11: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_11 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168338UL: // (QM_REG_QTASKCTR_12: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_12 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16833cUL: // (QM_REG_QTASKCTR_13: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_13 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168340UL: // (QM_REG_QTASKCTR_14: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_14 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168344UL: // (QM_REG_QTASKCTR_15: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_15 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168348UL: // (QM_REG_QTASKCTR_16: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_16 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16834cUL: // (QM_REG_QTASKCTR_17: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_17 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168350UL: // (QM_REG_QTASKCTR_18: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_18 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168354UL: // (QM_REG_QTASKCTR_19: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_19 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168358UL: // (QM_REG_QTASKCTR_20: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_20 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16835cUL: // (QM_REG_QTASKCTR_21: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_21 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168360UL: // (QM_REG_QTASKCTR_22: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_22 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168364UL: // (QM_REG_QTASKCTR_23: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_23 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168368UL: // (QM_REG_QTASKCTR_24: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_24 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16836cUL: // (QM_REG_QTASKCTR_25: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_25 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168370UL: // (QM_REG_QTASKCTR_26: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_26 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168374UL: // (QM_REG_QTASKCTR_27: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_27 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168378UL: // (QM_REG_QTASKCTR_28: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_28 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16837cUL: // (QM_REG_QTASKCTR_29: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_29 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168380UL: // (QM_REG_QTASKCTR_30: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_30 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168384UL: // (QM_REG_QTASKCTR_31: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_31 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168388UL: // (QM_REG_QTASKCTR_32: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_32 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16838cUL: // (QM_REG_QTASKCTR_33: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_33 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168390UL: // (QM_REG_QTASKCTR_34: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_34 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168394UL: // (QM_REG_QTASKCTR_35: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_35 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168398UL: // (QM_REG_QTASKCTR_36: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_36 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16839cUL: // (QM_REG_QTASKCTR_37: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_37 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683a0UL: // (QM_REG_QTASKCTR_38: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_38 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683a4UL: // (QM_REG_QTASKCTR_39: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_39 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683a8UL: // (QM_REG_QTASKCTR_40: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_40 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683acUL: // (QM_REG_QTASKCTR_41: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_41 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683b0UL: // (QM_REG_QTASKCTR_42: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_42 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683b4UL: // (QM_REG_QTASKCTR_43: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_43 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683b8UL: // (QM_REG_QTASKCTR_44: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_44 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683bcUL: // (QM_REG_QTASKCTR_45: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_45 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683c0UL: // (QM_REG_QTASKCTR_46: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_46 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683c4UL: // (QM_REG_QTASKCTR_47: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_47 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683c8UL: // (QM_REG_QTASKCTR_48: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_48 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683ccUL: // (QM_REG_QTASKCTR_49: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_49 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683d0UL: // (QM_REG_QTASKCTR_50: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_50 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683d4UL: // (QM_REG_QTASKCTR_51: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_51 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683d8UL: // (QM_REG_QTASKCTR_52: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_52 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683dcUL: // (QM_REG_QTASKCTR_53: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_53 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683e0UL: // (QM_REG_QTASKCTR_54: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_54 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683e4UL: // (QM_REG_QTASKCTR_55: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_55 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683e8UL: // (QM_REG_QTASKCTR_56: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_56 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683ecUL: // (QM_REG_QTASKCTR_57: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_57 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683f0UL: // (QM_REG_QTASKCTR_58: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_58 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683f4UL: // (QM_REG_QTASKCTR_59: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_59 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683f8UL: // (QM_REG_QTASKCTR_60: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_60 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1683fcUL: // (QM_REG_QTASKCTR_61: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_61 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168400UL: // (QM_REG_QTASKCTR_62: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_62 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168404UL: // (QM_REG_QTASKCTR_63: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_63 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks queued for each queue; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168408UL: // (QM_REG_VOQCRDERRREG: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQCRDERRREG (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: VOQ credit update error register; b3-b0: voq id (pbf error\n    ); b7-b4: voq id (storm increment error); b11-b8: voq id (storm decrement\n     error); b12: pbf error valid; b13: storm increment error valid; b14: \n    storm decrement error valid; b15: reserved; b27-b16: voq warning (warning\n    =decremented below zero). mask bit per voq counter; b31-b28: reserved; \n    NOTE: VOQ id-s represent HW VOQ id\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16840cUL: // (QM_REG_QLEVELMHVAL: ACCESS:RC  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QLEVELMHVAL (ACCESS:RC  DataWidth:0x18 )" \
				"\nDescription: The MAX hold value of the fill level of the physical queue\n     \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168410UL: // (QM_REG_PAUSESTATE0: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_PAUSESTATE0 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Pause state for physical queues 15-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168414UL: // (QM_REG_PAUSESTATE1: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_PAUSESTATE1 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Pause state for physical queues 31-16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168420UL: // (QM_REG_BIGRAMLOW_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nQM_REG_BIGRAMLOW_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM Bits of bigram low memory; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168424UL: // (QM_REG_BIGRAMHIGH_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nQM_REG_BIGRAMHIGH_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM Bits of bigram high memory; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168428UL: // (QM_REG_SOFT_RESET: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_SOFT_RESET (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Initialization bit command\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16842cUL: // (QM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168430UL: // (QM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168434UL: // (QM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168438UL: // (QM_REG_QM_INT_STS: ACCESS:R  DataWidth:0xe ) 
			sprintf(disp_str, "\nQM_REG_QM_INT_STS (ACCESS:R  DataWidth:0xe )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tBYTE_CRD_DEC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_BYTE_CRD_DEC_ERR) >> 13)
				, "\n\tBYTE_CRD_INC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_BYTE_CRD_INC_ERR) >> 12)
				, "\n\tVOQ_CRD_DEC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_VOQ_CRD_DEC_ERR) >> 11)
				, "\n\tVOQ_CRD_INC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_VOQ_CRD_INC_ERR) >> 10)
				, "\n\tPF_USG_CNT_7_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_PF_USG_CNT_7_ERR) >> 9)
				, "\n\tPF_USG_CNT_6_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_PF_USG_CNT_6_ERR) >> 8)
				, "\n\tPF_USG_CNT_5_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_PF_USG_CNT_5_ERR) >> 7)
				, "\n\tPF_USG_CNT_4_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_PF_USG_CNT_4_ERR) >> 6)
				, "\n\tPF_USG_CNT_3_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_PF_USG_CNT_3_ERR) >> 5)
				, "\n\tPF_USG_CNT_2_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_PF_USG_CNT_2_ERR) >> 4)
				, "\n\tPF_USG_CNT_1_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_PF_USG_CNT_1_ERR) >> 3)
				, "\n\tPF_USG_CNT_0_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_PF_USG_CNT_0_ERR) >> 2)
				, "\n\tOVF_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_OVF_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & QM_QM_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x16843cUL: // (QM_REG_QM_INT_STS_CLR: ACCESS:RC  DataWidth:0xe ) 
			sprintf(disp_str, "\nQM_REG_QM_INT_STS_CLR (ACCESS:RC  DataWidth:0xe )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tBYTE_CRD_DEC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_BYTE_CRD_DEC_ERR) >> 13)
				, "\n\tBYTE_CRD_INC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_BYTE_CRD_INC_ERR) >> 12)
				, "\n\tVOQ_CRD_DEC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_VOQ_CRD_DEC_ERR) >> 11)
				, "\n\tVOQ_CRD_INC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_VOQ_CRD_INC_ERR) >> 10)
				, "\n\tPF_USG_CNT_7_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_PF_USG_CNT_7_ERR) >> 9)
				, "\n\tPF_USG_CNT_6_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_PF_USG_CNT_6_ERR) >> 8)
				, "\n\tPF_USG_CNT_5_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_PF_USG_CNT_5_ERR) >> 7)
				, "\n\tPF_USG_CNT_4_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_PF_USG_CNT_4_ERR) >> 6)
				, "\n\tPF_USG_CNT_3_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_PF_USG_CNT_3_ERR) >> 5)
				, "\n\tPF_USG_CNT_2_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_PF_USG_CNT_2_ERR) >> 4)
				, "\n\tPF_USG_CNT_1_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_PF_USG_CNT_1_ERR) >> 3)
				, "\n\tPF_USG_CNT_0_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_PF_USG_CNT_0_ERR) >> 2)
				, "\n\tOVF_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_OVF_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & QM_QM_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x168440UL: // (QM_REG_QM_INT_STS_WR: ACCESS:WR  DataWidth:0xe ) 
			sprintf(disp_str, "\nQM_REG_QM_INT_STS_WR (ACCESS:WR  DataWidth:0xe )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tBYTE_CRD_DEC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_BYTE_CRD_DEC_ERR) >> 13)
				, "\n\tBYTE_CRD_INC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_BYTE_CRD_INC_ERR) >> 12)
				, "\n\tVOQ_CRD_DEC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_VOQ_CRD_DEC_ERR) >> 11)
				, "\n\tVOQ_CRD_INC_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_VOQ_CRD_INC_ERR) >> 10)
				, "\n\tPF_USG_CNT_7_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_PF_USG_CNT_7_ERR) >> 9)
				, "\n\tPF_USG_CNT_6_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_PF_USG_CNT_6_ERR) >> 8)
				, "\n\tPF_USG_CNT_5_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_PF_USG_CNT_5_ERR) >> 7)
				, "\n\tPF_USG_CNT_4_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_PF_USG_CNT_4_ERR) >> 6)
				, "\n\tPF_USG_CNT_3_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_PF_USG_CNT_3_ERR) >> 5)
				, "\n\tPF_USG_CNT_2_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_PF_USG_CNT_2_ERR) >> 4)
				, "\n\tPF_USG_CNT_1_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_PF_USG_CNT_1_ERR) >> 3)
				, "\n\tPF_USG_CNT_0_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_PF_USG_CNT_0_ERR) >> 2)
				, "\n\tOVF_ERR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_OVF_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & QM_QM_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x168444UL: // (QM_REG_QM_INT_MASK: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nQM_REG_QM_INT_MASK (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tBYTE_CRD_DEC_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_BYTE_CRD_DEC_ERR) >> 13)
				, "\n\tBYTE_CRD_INC_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_BYTE_CRD_INC_ERR) >> 12)
				, "\n\tVOQ_CRD_DEC_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_VOQ_CRD_DEC_ERR) >> 11)
				, "\n\tVOQ_CRD_INC_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_VOQ_CRD_INC_ERR) >> 10)
				, "\n\tPF_USG_CNT_7_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_PF_USG_CNT_7_ERR) >> 9)
				, "\n\tPF_USG_CNT_6_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_PF_USG_CNT_6_ERR) >> 8)
				, "\n\tPF_USG_CNT_5_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_PF_USG_CNT_5_ERR) >> 7)
				, "\n\tPF_USG_CNT_4_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_PF_USG_CNT_4_ERR) >> 6)
				, "\n\tPF_USG_CNT_3_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_PF_USG_CNT_3_ERR) >> 5)
				, "\n\tPF_USG_CNT_2_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_PF_USG_CNT_2_ERR) >> 4)
				, "\n\tPF_USG_CNT_1_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_PF_USG_CNT_1_ERR) >> 3)
				, "\n\tPF_USG_CNT_0_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_PF_USG_CNT_0_ERR) >> 2)
				, "\n\tOVF_ERR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_OVF_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & QM_QM_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x168448UL: // (QM_REG_QM_PRTY_STS: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_QM_PRTY_STS (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tBASE_ADDRESS_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_BASE_ADDRESS_EXT_A) >> 11)
				, "\n\tBIGRAMLOW_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_BIGRAMLOW_EXT_A) >> 10)
				, "\n\tBIGRAMHIGH_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_BIGRAMHIGH_EXT_A) >> 9)
				, "\n\tWRBUFF :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_WRBUFF) >> 8)
				, "\n\tBASE_ADDRESS :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_BASE_ADDRESS) >> 7)
				, "\n\tBIGRAMLOW :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_BIGRAMLOW) >> 6)
				, "\n\tBIGRAMHIGH :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_BIGRAMHIGH) >> 5)
				, "\n\tCCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_CCM_WRC_FIFO) >> 4)
				, "\n\tTCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_TCM_WRC_FIFO) >> 3)
				, "\n\tUCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_UCM_WRC_FIFO) >> 2)
				, "\n\tXCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_XCM_WRC_FIFO) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & QM_QM_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x16844cUL: // (QM_REG_QM_PRTY_STS_CLR: ACCESS:RC  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_QM_PRTY_STS_CLR (ACCESS:RC  DataWidth:0xc )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tBASE_ADDRESS_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_BASE_ADDRESS_EXT_A) >> 11)
				, "\n\tBIGRAMLOW_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_BIGRAMLOW_EXT_A) >> 10)
				, "\n\tBIGRAMHIGH_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_BIGRAMHIGH_EXT_A) >> 9)
				, "\n\tWRBUFF :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_WRBUFF) >> 8)
				, "\n\tBASE_ADDRESS :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_BASE_ADDRESS) >> 7)
				, "\n\tBIGRAMLOW :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_BIGRAMLOW) >> 6)
				, "\n\tBIGRAMHIGH :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_BIGRAMHIGH) >> 5)
				, "\n\tCCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_CCM_WRC_FIFO) >> 4)
				, "\n\tTCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_TCM_WRC_FIFO) >> 3)
				, "\n\tUCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_UCM_WRC_FIFO) >> 2)
				, "\n\tXCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_XCM_WRC_FIFO) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & QM_QM_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x168450UL: // (QM_REG_QM_PRTY_STS_WR: ACCESS:WR  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_QM_PRTY_STS_WR (ACCESS:WR  DataWidth:0xc )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tBASE_ADDRESS_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_BASE_ADDRESS_EXT_A) >> 11)
				, "\n\tBIGRAMLOW_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_BIGRAMLOW_EXT_A) >> 10)
				, "\n\tBIGRAMHIGH_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_BIGRAMHIGH_EXT_A) >> 9)
				, "\n\tWRBUFF :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_WRBUFF) >> 8)
				, "\n\tBASE_ADDRESS :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_BASE_ADDRESS) >> 7)
				, "\n\tBIGRAMLOW :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_BIGRAMLOW) >> 6)
				, "\n\tBIGRAMHIGH :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_BIGRAMHIGH) >> 5)
				, "\n\tCCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_CCM_WRC_FIFO) >> 4)
				, "\n\tTCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_TCM_WRC_FIFO) >> 3)
				, "\n\tUCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_UCM_WRC_FIFO) >> 2)
				, "\n\tXCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_XCM_WRC_FIFO) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & QM_QM_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x168454UL: // (QM_REG_QM_PRTY_MASK: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nQM_REG_QM_PRTY_MASK (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tBASE_ADDRESS_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_BASE_ADDRESS_EXT_A) >> 11)
				, "\n\tBIGRAMLOW_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_BIGRAMLOW_EXT_A) >> 10)
				, "\n\tBIGRAMHIGH_EXT_A :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_BIGRAMHIGH_EXT_A) >> 9)
				, "\n\tWRBUFF :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_WRBUFF) >> 8)
				, "\n\tBASE_ADDRESS :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_BASE_ADDRESS) >> 7)
				, "\n\tBIGRAMLOW :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_BIGRAMLOW) >> 6)
				, "\n\tBIGRAMHIGH :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_BIGRAMHIGH) >> 5)
				, "\n\tCCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_CCM_WRC_FIFO) >> 4)
				, "\n\tTCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_TCM_WRC_FIFO) >> 3)
				, "\n\tUCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_UCM_WRC_FIFO) >> 2)
				, "\n\tXCM_WRC_FIFO :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_XCM_WRC_FIFO) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & QM_QM_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x16e684UL: // (QM_REG_PAUSESTATE2: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_PAUSESTATE2 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Pause state for physical queues 47-32\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e688UL: // (QM_REG_PAUSESTATE3: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_PAUSESTATE3 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: Pause state for physical queues 63-48\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6bcUL: // (QM_REG_PQ2PCIFUNC_0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_PQ2PCIFUNC_0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: pci function number of queues 15-0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6c0UL: // (QM_REG_PQ2PCIFUNC_1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_PQ2PCIFUNC_1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: pci function number of queues 31-16 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6c4UL: // (QM_REG_PQ2PCIFUNC_2: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_PQ2PCIFUNC_2 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: pci function number of queues 47-32 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6c8UL: // (QM_REG_PQ2PCIFUNC_3: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_PQ2PCIFUNC_3 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: pci function number of queues 63-48 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6e0UL: // (QM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nQM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: eco reserved register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6e4UL: // (QM_REG_PCIREQATC: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_PCIREQATC (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: The PCI ATC flags used in the PCI request. b2-b0: rd first\n     bank in page; b3: reserved (zero); b6-b4: wr first bank in page; b7: \n    reserved (zero); b10-b8: rd middle bank in page; b11: reserved (zero); \n    b14-b12: wr middle bank in page; b15: reserved (zero); b18-b16: rd last\n     bank in page; b19: reserved (zero); b22-b20: wr last bank in page; b23\n    : reserved (zero);\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6fcUL: // (QM_REG_BYTECRD0: ACCESS:R  DataWidth:0x12 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRD0 (ACCESS:R  DataWidth:0x12 )" \
				"\nDescription: The credit value for byte credit 0. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e700UL: // (QM_REG_BYTECRD1: ACCESS:R  DataWidth:0x12 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRD1 (ACCESS:R  DataWidth:0x12 )" \
				"\nDescription: The credit value for byte credit 1. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e704UL: // (QM_REG_BYTECRD2: ACCESS:R  DataWidth:0x12 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRD2 (ACCESS:R  DataWidth:0x12 )" \
				"\nDescription: The credit value for byte credit 2. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e708UL: // (QM_REG_BYTECRDERRREG: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDERRREG (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: byte credit update error register; b2-b0: byte credit id (\n    pbf error); b3 - reserved (zero filled); b6-b4: byte credit id (storm \n    increment error); b7 - reserved (zero filled); b10-b8: byte credit id (\n    storm decrement error); b11 - reserved (zero filled); b12: pbf error valid\n    ; b13: storm increment error valid; b14: storm decrement error valid; \n    b15: reserved; b22-b16: byte credit warning (warning=decremented below\n     zero). mask bit per voq counter; b31-b23: reserved; NOTE: VOQ id-s represent\n     HW \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e788UL: // (QM_REG_PCI_RD_ERR_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_PCI_RD_ERR_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: enable pci rd error usage. When set pci rd error indication\n     coming from the pci will set the bank with error (within pci_rd_err reg\n    ). When reset pci_rd_reg will be always 0 (i.e. not affected by the error\n     coming from the pci). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e790UL: // (QM_REG_QMPAGESIZE: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nQM_REG_QMPAGESIZE (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The STU size; this should be configured according to the \n    minimal STU within the PXP (there is STU per PF). 0-4k;1-8k;2-16k;3-32k\n    ;4-64k;5-128k;6-256k;7-512k;8-1M;9-2M;10-4M\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e794UL: // (QM_REG_PQ_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_PQ_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: This register affects the way the QM looks on the interfaces\n     that involve physical queue logic (push; pop; xsdm command; xcm bypass\n    ) and takes care of the required physical queue mapping logic. the QM \n    will map IPQN[4:0] = EPQN[4:0]. In addition when set the QM will map IPQN\n    [5]=EPQN[6]. when reset IPQN[5]=EPQN[5].\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7a8UL: // (QM_REG_EEE_BYTE_CRD_EN: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nQM_REG_EEE_BYTE_CRD_EN (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: when set eee idle state requires both the byte credit counter\n     and voq counter to be full  (i.e. equal to their init values). When reset\n     only the voq counter needs to be full. Bit per byte credit counter as\n     follows: bit0 - byte credit0; bit 1 - byte credit1; . . .; bit 6 - byte\n     credit6; \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7acUL: // (QM_REG_BYTECRD3: ACCESS:R  DataWidth:0x12 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRD3 (ACCESS:R  DataWidth:0x12 )" \
				"\nDescription: The credit value for byte credit 3. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7b0UL: // (QM_REG_BYTECRD4: ACCESS:R  DataWidth:0x12 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRD4 (ACCESS:R  DataWidth:0x12 )" \
				"\nDescription: The credit value for byte credit 4. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7b4UL: // (QM_REG_BYTECRD5: ACCESS:R  DataWidth:0x12 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRD5 (ACCESS:R  DataWidth:0x12 )" \
				"\nDescription: The credit value for byte credit 5. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7b8UL: // (QM_REG_BYTECRD6: ACCESS:R  DataWidth:0x12 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRD6 (ACCESS:R  DataWidth:0x12 )" \
				"\nDescription: The credit value for byte credit 6. The value is 2s complement\n     value (i.e. msb is used for the sign).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7bcUL: // (QM_REG_FWVOQ0TOHWVOQ: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_FWVOQ0TOHWVOQ (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Describes the HW (real) VOQ id (id-s 0-6 used for HW TX VOQ\n    -s) of FW (virtual) VOQ0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7c0UL: // (QM_REG_FWVOQ1TOHWVOQ: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_FWVOQ1TOHWVOQ (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Describes the HW (real) VOQ id (id-s 0-6 used for HW TX VOQ\n    -s) of FW (virtual) VOQ1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7c4UL: // (QM_REG_FWVOQ2TOHWVOQ: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_FWVOQ2TOHWVOQ (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Describes the HW (real) VOQ id (id-s 0-6 used for HW TX VOQ\n    -s) of FW (virtual) VOQ2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7c8UL: // (QM_REG_FWVOQ3TOHWVOQ: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_FWVOQ3TOHWVOQ (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Describes the HW (real) VOQ id (id-s 0-6 used for HW TX VOQ\n    -s) of FW (virtual) VOQ3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7ccUL: // (QM_REG_FWVOQ4TOHWVOQ: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_FWVOQ4TOHWVOQ (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Describes the HW (real) VOQ id (id-s 0-6 used for HW TX VOQ\n    -s) of FW (virtual) VOQ4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7d0UL: // (QM_REG_FWVOQ5TOHWVOQ: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_FWVOQ5TOHWVOQ (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Describes the HW (real) VOQ id (id-s 0-6 used for HW TX VOQ\n    -s) of FW (virtual) VOQ5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7d4UL: // (QM_REG_FWVOQ6TOHWVOQ: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_FWVOQ6TOHWVOQ (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Describes the HW (real) VOQ id (id-s 0-6 used for HW TX VOQ\n    -s) of FW (virtual) VOQ6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7d8UL: // (QM_REG_FWVOQ7TOHWVOQ: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_FWVOQ7TOHWVOQ (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Describes the HW (real) VOQ id (id-s 0-6 used for HW TX VOQ\n    -s) of FW (virtual) VOQ7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168800UL: // (QM_REG_MHQNUMSEL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nQM_REG_MHQNUMSEL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The physical queue number for the MAX hold queue fill level\n     statistics\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168804UL: // (QM_REG_OUTLDREQ: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_OUTLDREQ (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of outstanding request to CFC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168808UL: // (QM_REG_UPDATECREDIT: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_UPDATECREDIT (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Credit update command interface. NOTE: unlike the command\n     from the XSDM (that updates credit based on the FW VOQ id) the RBC command\n     will update credit based on HW VOQ id. Or in other words the VOQ id field\n     represent HW VOQ id.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16880cUL: // (QM_REG_WRRWEIGHTS_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168810UL: // (QM_REG_WRRWEIGHTS_1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168814UL: // (QM_REG_WRRWEIGHTS_10: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_10 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168818UL: // (QM_REG_WRRWEIGHTS_11: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_11 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16881cUL: // (QM_REG_WRRWEIGHTS_12: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_12 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168820UL: // (QM_REG_WRRWEIGHTS_13: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_13 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168824UL: // (QM_REG_WRRWEIGHTS_14: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_14 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168828UL: // (QM_REG_WRRWEIGHTS_15: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_15 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16882cUL: // (QM_REG_WRRWEIGHTS_2: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_2 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168830UL: // (QM_REG_WRRWEIGHTS_3: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_3 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168834UL: // (QM_REG_WRRWEIGHTS_4: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_4 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168838UL: // (QM_REG_WRRWEIGHTS_5: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_5 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16883cUL: // (QM_REG_WRRWEIGHTS_6: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_6 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168840UL: // (QM_REG_WRRWEIGHTS_7: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_7 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168844UL: // (QM_REG_WRRWEIGHTS_8: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_8 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168848UL: // (QM_REG_WRRWEIGHTS_9: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_9 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Wrr weights. NOTE: weight update is allowed only to queues\n     which are either empty or paused\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168900UL: // (QM_REG_BASEADDR: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BASEADDR (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The base logical address (in bytes) of each physical queue\n    . The index I represents the physical queue number. The 12 lsbs are ignore\n     and considered zero so practically there are only 20 bits in this register\n    ; queues 63-0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168a00UL: // (QM_REG_PTRTBL: ACCESS:WB  DataWidth:0x36 ) 
			sprintf(disp_str, "\nQM_REG_PTRTBL (ACCESS:WB  DataWidth:0x36 )" \
				"\nDescription: Pointer Table Memory for queues 63-0; The mapping is as follow\n    : ptrtbl[53:30] read pointer; ptrtbl[29:6] write pointer; ptrtbl[5:4] \n    read bank0; ptrtbl[3:2] read bank 1; ptrtbl[1:0] write bank;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16a000UL: // (QM_REG_BIGRAMHIGH: ACCESS:RW  DataWidth:0x1e ) 
			sprintf(disp_str, "\nQM_REG_BIGRAMHIGH (ACCESS:RW  DataWidth:0x1e )" \
				"\nDescription: High part of BigRAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16c000UL: // (QM_REG_BIGRAMLOW: ACCESS:RW  DataWidth:0x1e ) 
			sprintf(disp_str, "\nQM_REG_BIGRAMLOW (ACCESS:RW  DataWidth:0x1e )" \
				"\nDescription: Low part of BigRAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e040UL: // (QM_REG_PF_USG_CNT_0: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_PF_USG_CNT_0 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks stored in the QM for the PF. only even\n     functions are valid in E2 (odd I registers will be hard wired to 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e044UL: // (QM_REG_PF_USG_CNT_1: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_PF_USG_CNT_1 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks stored in the QM for the PF. only even\n     functions are valid in E2 (odd I registers will be hard wired to 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e048UL: // (QM_REG_PF_USG_CNT_2: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_PF_USG_CNT_2 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks stored in the QM for the PF. only even\n     functions are valid in E2 (odd I registers will be hard wired to 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e04cUL: // (QM_REG_PF_USG_CNT_3: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_PF_USG_CNT_3 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks stored in the QM for the PF. only even\n     functions are valid in E2 (odd I registers will be hard wired to 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e050UL: // (QM_REG_PF_USG_CNT_4: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_PF_USG_CNT_4 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks stored in the QM for the PF. only even\n     functions are valid in E2 (odd I registers will be hard wired to 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e054UL: // (QM_REG_PF_USG_CNT_5: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_PF_USG_CNT_5 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks stored in the QM for the PF. only even\n     functions are valid in E2 (odd I registers will be hard wired to 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e058UL: // (QM_REG_PF_USG_CNT_6: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_PF_USG_CNT_6 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks stored in the QM for the PF. only even\n     functions are valid in E2 (odd I registers will be hard wired to 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e05cUL: // (QM_REG_PF_USG_CNT_7: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_PF_USG_CNT_7 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: The number of tasks stored in the QM for the PF. only even\n     functions are valid in E2 (odd I registers will be hard wired to 0)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e70cUL: // (QM_REG_PF_EN: ACCESS:RW  DataWidth:0x1  SPLIT:8 ) 
			sprintf(disp_str, "\nQM_REG_PF_EN (ACCESS:RW  DataWidth:0x1  SPLIT:8 )" \
				"\nDescription: PF enable vector. Bit per PF. If set the PF is enabled\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e768UL: // (QM_REG_PCI_RD_ERR_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_PCI_RD_ERR_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: pci read error that indicates for each bank if the data in\n     the bank has error. The bits allocation is as follows: bit0 - bank0 pq0\n    ; bit1 - bank1 pq0; bit2 - bank2 pq0; bit3 - reserved (should be written\n     with 0); bit4 - bank0 pq1; bit5 - bank1 pq1; bit6 - bank2 pq1; bit7 - \n    reserved (should be written with 0); etc. there are 8 registers - 8 physical\n     queues exist in each register as follows: register 0 pq 7-0; register\n     1 pq 15-8; etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e76cUL: // (QM_REG_PCI_RD_ERR_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_PCI_RD_ERR_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: pci read error that indicates for each bank if the data in\n     the bank has error. The bits allocation is as follows: bit0 - bank0 pq0\n    ; bit1 - bank1 pq0; bit2 - bank2 pq0; bit3 - reserved (should be written\n     with 0); bit4 - bank0 pq1; bit5 - bank1 pq1; bit6 - bank2 pq1; bit7 - \n    reserved (should be written with 0); etc. there are 8 registers - 8 physical\n     queues exist in each register as follows: register 0 pq 7-0; register\n     1 pq 15-8; etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e770UL: // (QM_REG_PCI_RD_ERR_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_PCI_RD_ERR_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: pci read error that indicates for each bank if the data in\n     the bank has error. The bits allocation is as follows: bit0 - bank0 pq0\n    ; bit1 - bank1 pq0; bit2 - bank2 pq0; bit3 - reserved (should be written\n     with 0); bit4 - bank0 pq1; bit5 - bank1 pq1; bit6 - bank2 pq1; bit7 - \n    reserved (should be written with 0); etc. there are 8 registers - 8 physical\n     queues exist in each register as follows: register 0 pq 7-0; register\n     1 pq 15-8; etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e774UL: // (QM_REG_PCI_RD_ERR_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_PCI_RD_ERR_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: pci read error that indicates for each bank if the data in\n     the bank has error. The bits allocation is as follows: bit0 - bank0 pq0\n    ; bit1 - bank1 pq0; bit2 - bank2 pq0; bit3 - reserved (should be written\n     with 0); bit4 - bank0 pq1; bit5 - bank1 pq1; bit6 - bank2 pq1; bit7 - \n    reserved (should be written with 0); etc. there are 8 registers - 8 physical\n     queues exist in each register as follows: register 0 pq 7-0; register\n     1 pq 15-8; etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e778UL: // (QM_REG_PCI_RD_ERR_4: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_PCI_RD_ERR_4 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: pci read error that indicates for each bank if the data in\n     the bank has error. The bits allocation is as follows: bit0 - bank0 pq0\n    ; bit1 - bank1 pq0; bit2 - bank2 pq0; bit3 - reserved (should be written\n     with 0); bit4 - bank0 pq1; bit5 - bank1 pq1; bit6 - bank2 pq1; bit7 - \n    reserved (should be written with 0); etc. there are 8 registers - 8 physical\n     queues exist in each register as follows: register 0 pq 7-0; register\n     1 pq 15-8; etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e77cUL: // (QM_REG_PCI_RD_ERR_5: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_PCI_RD_ERR_5 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: pci read error that indicates for each bank if the data in\n     the bank has error. The bits allocation is as follows: bit0 - bank0 pq0\n    ; bit1 - bank1 pq0; bit2 - bank2 pq0; bit3 - reserved (should be written\n     with 0); bit4 - bank0 pq1; bit5 - bank1 pq1; bit6 - bank2 pq1; bit7 - \n    reserved (should be written with 0); etc. there are 8 registers - 8 physical\n     queues exist in each register as follows: register 0 pq 7-0; register\n     1 pq 15-8; etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e780UL: // (QM_REG_PCI_RD_ERR_6: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_PCI_RD_ERR_6 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: pci read error that indicates for each bank if the data in\n     the bank has error. The bits allocation is as follows: bit0 - bank0 pq0\n    ; bit1 - bank1 pq0; bit2 - bank2 pq0; bit3 - reserved (should be written\n     with 0); bit4 - bank0 pq1; bit5 - bank1 pq1; bit6 - bank2 pq1; bit7 - \n    reserved (should be written with 0); etc. there are 8 registers - 8 physical\n     queues exist in each register as follows: register 0 pq 7-0; register\n     1 pq 15-8; etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e784UL: // (QM_REG_PCI_RD_ERR_7: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_PCI_RD_ERR_7 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: pci read error that indicates for each bank if the data in\n     the bank has error. The bits allocation is as follows: bit0 - bank0 pq0\n    ; bit1 - bank1 pq0; bit2 - bank2 pq0; bit3 - reserved (should be written\n     with 0); bit4 - bank0 pq1; bit5 - bank1 pq1; bit6 - bank2 pq1; bit7 - \n    reserved (should be written with 0); etc. there are 8 registers - 8 physical\n     queues exist in each register as follows: register 0 pq 7-0; register\n     1 pq 15-8; etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e78cUL: // (QM_REG_PCI_RD_ERR_RST: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nQM_REG_PCI_RD_ERR_RST (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: WRITE ONLY command register. Command for reseting the pci\n     error bits for the banks of the pq in the command. upon reset the pci_rd_err\n     register should be updated accordingly with the new values.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e798UL: // (QM_REG_DBG_OUT_DATA_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_DBG_OUT_DATA_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 32 lsb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e79cUL: // (QM_REG_DBG_OUT_DATA_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_DBG_OUT_DATA_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 32 msb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7a0UL: // (QM_REG_DBG_OUT_FRAME: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_DBG_OUT_FRAME (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 4 frame bits that goes to the DBG block. Bit0\n     is the frame of data byte0; Bit1 is the frame of  data byte1; Bit2 is\n     the frame of data byte2; Bit3 is the frame of  data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7a4UL: // (QM_REG_DBG_OUT_VALID: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_DBG_OUT_VALID (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from QM\n     to the DBG block) - The 4 valid bits that goes to the DBG block. Bit0\n     validtes data byte0; Bit1 validates data byte1; Bit2 validates data byte2\n    ; Bit3 validates data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168224UL: // (QM_REG_BYTECRDPORT_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDPORT_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168228UL: // (QM_REG_BYTECRDPORT_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDPORT_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682a0UL: // (QM_REG_VOQPORT_0: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_0 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682a4UL: // (QM_REG_VOQPORT_1: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_1 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682a8UL: // (QM_REG_VOQPORT_2: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_2 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682acUL: // (QM_REG_VOQPORT_3: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_3 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682b0UL: // (QM_REG_VOQPORT_4: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_4 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682b4UL: // (QM_REG_VOQPORT_5: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_5 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682b8UL: // (QM_REG_VOQPORT_6: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_6 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682bcUL: // (QM_REG_VOQPORT_7: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_7 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682c0UL: // (QM_REG_VOQPORT_8: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_8 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682c4UL: // (QM_REG_VOQPORT_9: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_9 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682c8UL: // (QM_REG_VOQPORT_10: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_10 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1682ccUL: // (QM_REG_VOQPORT_11: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_VOQPORT_11 (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168300UL: // (QM_REG_PORT0BYTECRD: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_PORT0BYTECRD (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168304UL: // (QM_REG_PORT1BYTECRD: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_PORT1BYTECRD (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168418UL: // (QM_REG_REMAINPAUSETM0: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_REMAINPAUSETM0 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16841cUL: // (QM_REG_REMAINPAUSETM1: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_REMAINPAUSETM1 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e000UL: // (QM_REG_WRRWEIGHTS_16: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_16 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e004UL: // (QM_REG_WRRWEIGHTS_17: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_17 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e008UL: // (QM_REG_WRRWEIGHTS_18: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_18 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e00cUL: // (QM_REG_WRRWEIGHTS_19: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_19 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e010UL: // (QM_REG_WRRWEIGHTS_20: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_20 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e014UL: // (QM_REG_WRRWEIGHTS_21: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_21 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e018UL: // (QM_REG_WRRWEIGHTS_22: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_22 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e01cUL: // (QM_REG_WRRWEIGHTS_23: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_23 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e020UL: // (QM_REG_WRRWEIGHTS_24: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_24 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e024UL: // (QM_REG_WRRWEIGHTS_25: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_25 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e028UL: // (QM_REG_WRRWEIGHTS_26: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_26 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e02cUL: // (QM_REG_WRRWEIGHTS_27: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_27 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e030UL: // (QM_REG_WRRWEIGHTS_28: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_28 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e034UL: // (QM_REG_WRRWEIGHTS_29: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_29 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e038UL: // (QM_REG_WRRWEIGHTS_30: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_30 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e03cUL: // (QM_REG_WRRWEIGHTS_31: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_WRRWEIGHTS_31 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e100UL: // (QM_REG_BASEADDR_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BASEADDR_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e200UL: // (QM_REG_PTRTBL_EXT_A: ACCESS:R  DataWidth:0x36 ) 
			sprintf(disp_str, "\nQM_REG_PTRTBL_EXT_A (ACCESS:R  DataWidth:0x36 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e400UL: // (QM_REG_BIGRAMSELUPPERQUEUES: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nQM_REG_BIGRAMSELUPPERQUEUES (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e404UL: // (QM_REG_QSTATUS_LOW_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_QSTATUS_LOW_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e408UL: // (QM_REG_QSTATUS_HIGH_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_QSTATUS_HIGH_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e40cUL: // (QM_REG_QVOQIDX_64: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_64 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e410UL: // (QM_REG_QVOQIDX_65: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_65 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e414UL: // (QM_REG_QVOQIDX_66: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_66 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e418UL: // (QM_REG_QVOQIDX_67: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_67 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e41cUL: // (QM_REG_QVOQIDX_68: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_68 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e420UL: // (QM_REG_QVOQIDX_69: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_69 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e424UL: // (QM_REG_QVOQIDX_70: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_70 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e428UL: // (QM_REG_QVOQIDX_71: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_71 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e42cUL: // (QM_REG_QVOQIDX_72: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_72 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e430UL: // (QM_REG_QVOQIDX_73: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_73 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e434UL: // (QM_REG_QVOQIDX_74: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_74 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e438UL: // (QM_REG_QVOQIDX_75: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_75 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e43cUL: // (QM_REG_QVOQIDX_76: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_76 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e440UL: // (QM_REG_QVOQIDX_77: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_77 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e444UL: // (QM_REG_QVOQIDX_78: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_78 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e448UL: // (QM_REG_QVOQIDX_79: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_79 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e44cUL: // (QM_REG_QVOQIDX_80: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_80 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e450UL: // (QM_REG_QVOQIDX_81: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_81 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e454UL: // (QM_REG_QVOQIDX_82: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_82 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e458UL: // (QM_REG_QVOQIDX_83: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_83 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e45cUL: // (QM_REG_QVOQIDX_84: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_84 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e460UL: // (QM_REG_QVOQIDX_85: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_85 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e464UL: // (QM_REG_QVOQIDX_86: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_86 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e468UL: // (QM_REG_QVOQIDX_87: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_87 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e46cUL: // (QM_REG_QVOQIDX_88: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_88 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e470UL: // (QM_REG_QVOQIDX_89: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_89 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e474UL: // (QM_REG_QVOQIDX_90: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_90 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e478UL: // (QM_REG_QVOQIDX_91: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_91 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e47cUL: // (QM_REG_QVOQIDX_92: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_92 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e480UL: // (QM_REG_QVOQIDX_93: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_93 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e484UL: // (QM_REG_QVOQIDX_94: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_94 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e488UL: // (QM_REG_QVOQIDX_95: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_95 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e48cUL: // (QM_REG_QVOQIDX_96: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_96 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e490UL: // (QM_REG_QVOQIDX_97: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_97 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e494UL: // (QM_REG_QVOQIDX_98: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_98 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e498UL: // (QM_REG_QVOQIDX_99: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_99 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e49cUL: // (QM_REG_QVOQIDX_100: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_100 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4a0UL: // (QM_REG_QVOQIDX_101: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_101 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4a4UL: // (QM_REG_QVOQIDX_102: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_102 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4a8UL: // (QM_REG_QVOQIDX_103: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_103 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4acUL: // (QM_REG_QVOQIDX_104: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_104 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4b0UL: // (QM_REG_QVOQIDX_105: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_105 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4b4UL: // (QM_REG_QVOQIDX_106: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_106 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4b8UL: // (QM_REG_QVOQIDX_107: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_107 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4bcUL: // (QM_REG_QVOQIDX_108: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_108 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4c0UL: // (QM_REG_QVOQIDX_109: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_109 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4c4UL: // (QM_REG_QVOQIDX_110: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_110 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4c8UL: // (QM_REG_QVOQIDX_111: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_111 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4ccUL: // (QM_REG_QVOQIDX_112: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_112 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4d0UL: // (QM_REG_QVOQIDX_113: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_113 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4d4UL: // (QM_REG_QVOQIDX_114: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_114 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4d8UL: // (QM_REG_QVOQIDX_115: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_115 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4dcUL: // (QM_REG_QVOQIDX_116: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_116 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4e0UL: // (QM_REG_QVOQIDX_117: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_117 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4e4UL: // (QM_REG_QVOQIDX_118: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_118 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4e8UL: // (QM_REG_QVOQIDX_119: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_119 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4ecUL: // (QM_REG_QVOQIDX_120: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_120 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4f0UL: // (QM_REG_QVOQIDX_121: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_121 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4f4UL: // (QM_REG_QVOQIDX_122: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_122 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4f8UL: // (QM_REG_QVOQIDX_123: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_123 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e4fcUL: // (QM_REG_QVOQIDX_124: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_124 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e500UL: // (QM_REG_QVOQIDX_125: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_125 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e504UL: // (QM_REG_QVOQIDX_126: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_126 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e508UL: // (QM_REG_QVOQIDX_127: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_QVOQIDX_127 (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e50cUL: // (QM_REG_HWAEMPTYMASK_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_HWAEMPTYMASK_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e510UL: // (QM_REG_HWAEMPTYMASK_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_HWAEMPTYMASK_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e514UL: // (QM_REG_ENBYTECRD_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_ENBYTECRD_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e518UL: // (QM_REG_ENBYTECRD_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_ENBYTECRD_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e51cUL: // (QM_REG_BYTECRDPORT_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDPORT_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e520UL: // (QM_REG_BYTECRDPORT_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDPORT_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e524UL: // (QM_REG_VOQQMASK_0_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_0_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e528UL: // (QM_REG_VOQQMASK_0_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_0_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e52cUL: // (QM_REG_VOQQMASK_1_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_1_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e530UL: // (QM_REG_VOQQMASK_1_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_1_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e534UL: // (QM_REG_VOQQMASK_2_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_2_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e538UL: // (QM_REG_VOQQMASK_2_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_2_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e53cUL: // (QM_REG_VOQQMASK_3_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_3_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e540UL: // (QM_REG_VOQQMASK_3_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_3_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e544UL: // (QM_REG_VOQQMASK_4_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_4_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e548UL: // (QM_REG_VOQQMASK_4_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_4_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e54cUL: // (QM_REG_VOQQMASK_5_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_5_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e550UL: // (QM_REG_VOQQMASK_5_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_5_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e554UL: // (QM_REG_VOQQMASK_6_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_6_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e558UL: // (QM_REG_VOQQMASK_6_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_6_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e55cUL: // (QM_REG_VOQQMASK_7_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_7_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e560UL: // (QM_REG_VOQQMASK_7_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_7_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e564UL: // (QM_REG_VOQQMASK_8_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_8_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e568UL: // (QM_REG_VOQQMASK_8_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_8_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e56cUL: // (QM_REG_VOQQMASK_9_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_9_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e570UL: // (QM_REG_VOQQMASK_9_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_9_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e574UL: // (QM_REG_VOQQMASK_10_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_10_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e578UL: // (QM_REG_VOQQMASK_10_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_10_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e57cUL: // (QM_REG_VOQQMASK_11_LSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_11_LSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e580UL: // (QM_REG_VOQQMASK_11_MSB_EXT_A: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_VOQQMASK_11_MSB_EXT_A (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e584UL: // (QM_REG_QTASKCTR_EXT_A_0: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_0 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e588UL: // (QM_REG_QTASKCTR_EXT_A_1: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_1 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e58cUL: // (QM_REG_QTASKCTR_EXT_A_2: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_2 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e590UL: // (QM_REG_QTASKCTR_EXT_A_3: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_3 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e594UL: // (QM_REG_QTASKCTR_EXT_A_4: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_4 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e598UL: // (QM_REG_QTASKCTR_EXT_A_5: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_5 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e59cUL: // (QM_REG_QTASKCTR_EXT_A_6: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_6 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5a0UL: // (QM_REG_QTASKCTR_EXT_A_7: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_7 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5a4UL: // (QM_REG_QTASKCTR_EXT_A_8: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_8 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5a8UL: // (QM_REG_QTASKCTR_EXT_A_9: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_9 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5acUL: // (QM_REG_QTASKCTR_EXT_A_10: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_10 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5b0UL: // (QM_REG_QTASKCTR_EXT_A_11: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_11 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5b4UL: // (QM_REG_QTASKCTR_EXT_A_12: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_12 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5b8UL: // (QM_REG_QTASKCTR_EXT_A_13: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_13 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5bcUL: // (QM_REG_QTASKCTR_EXT_A_14: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_14 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5c0UL: // (QM_REG_QTASKCTR_EXT_A_15: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_15 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5c4UL: // (QM_REG_QTASKCTR_EXT_A_16: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_16 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5c8UL: // (QM_REG_QTASKCTR_EXT_A_17: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_17 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5ccUL: // (QM_REG_QTASKCTR_EXT_A_18: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_18 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5d0UL: // (QM_REG_QTASKCTR_EXT_A_19: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_19 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5d4UL: // (QM_REG_QTASKCTR_EXT_A_20: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_20 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5d8UL: // (QM_REG_QTASKCTR_EXT_A_21: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_21 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5dcUL: // (QM_REG_QTASKCTR_EXT_A_22: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_22 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5e0UL: // (QM_REG_QTASKCTR_EXT_A_23: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_23 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5e4UL: // (QM_REG_QTASKCTR_EXT_A_24: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_24 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5e8UL: // (QM_REG_QTASKCTR_EXT_A_25: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_25 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5ecUL: // (QM_REG_QTASKCTR_EXT_A_26: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_26 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5f0UL: // (QM_REG_QTASKCTR_EXT_A_27: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_27 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5f4UL: // (QM_REG_QTASKCTR_EXT_A_28: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_28 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5f8UL: // (QM_REG_QTASKCTR_EXT_A_29: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_29 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e5fcUL: // (QM_REG_QTASKCTR_EXT_A_30: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_30 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e600UL: // (QM_REG_QTASKCTR_EXT_A_31: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_31 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e604UL: // (QM_REG_QTASKCTR_EXT_A_32: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_32 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e608UL: // (QM_REG_QTASKCTR_EXT_A_33: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_33 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e60cUL: // (QM_REG_QTASKCTR_EXT_A_34: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_34 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e610UL: // (QM_REG_QTASKCTR_EXT_A_35: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_35 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e614UL: // (QM_REG_QTASKCTR_EXT_A_36: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_36 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e618UL: // (QM_REG_QTASKCTR_EXT_A_37: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_37 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e61cUL: // (QM_REG_QTASKCTR_EXT_A_38: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_38 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e620UL: // (QM_REG_QTASKCTR_EXT_A_39: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_39 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e624UL: // (QM_REG_QTASKCTR_EXT_A_40: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_40 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e628UL: // (QM_REG_QTASKCTR_EXT_A_41: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_41 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e62cUL: // (QM_REG_QTASKCTR_EXT_A_42: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_42 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e630UL: // (QM_REG_QTASKCTR_EXT_A_43: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_43 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e634UL: // (QM_REG_QTASKCTR_EXT_A_44: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_44 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e638UL: // (QM_REG_QTASKCTR_EXT_A_45: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_45 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e63cUL: // (QM_REG_QTASKCTR_EXT_A_46: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_46 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e640UL: // (QM_REG_QTASKCTR_EXT_A_47: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_47 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e644UL: // (QM_REG_QTASKCTR_EXT_A_48: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_48 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e648UL: // (QM_REG_QTASKCTR_EXT_A_49: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_49 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e64cUL: // (QM_REG_QTASKCTR_EXT_A_50: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_50 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e650UL: // (QM_REG_QTASKCTR_EXT_A_51: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_51 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e654UL: // (QM_REG_QTASKCTR_EXT_A_52: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_52 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e658UL: // (QM_REG_QTASKCTR_EXT_A_53: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_53 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e65cUL: // (QM_REG_QTASKCTR_EXT_A_54: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_54 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e660UL: // (QM_REG_QTASKCTR_EXT_A_55: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_55 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e664UL: // (QM_REG_QTASKCTR_EXT_A_56: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_56 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e668UL: // (QM_REG_QTASKCTR_EXT_A_57: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_57 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e66cUL: // (QM_REG_QTASKCTR_EXT_A_58: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_58 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e670UL: // (QM_REG_QTASKCTR_EXT_A_59: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_59 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e674UL: // (QM_REG_QTASKCTR_EXT_A_60: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_60 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e678UL: // (QM_REG_QTASKCTR_EXT_A_61: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_61 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e67cUL: // (QM_REG_QTASKCTR_EXT_A_62: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_62 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e680UL: // (QM_REG_QTASKCTR_EXT_A_63: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_QTASKCTR_EXT_A_63 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e68cUL: // (QM_REG_PAUSESTATE4: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_PAUSESTATE4 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e690UL: // (QM_REG_PAUSESTATE5: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_PAUSESTATE5 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e694UL: // (QM_REG_PAUSESTATE6: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_PAUSESTATE6 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e698UL: // (QM_REG_PAUSESTATE7: ACCESS:R  DataWidth:0x10 ) 
			sprintf(disp_str, "\nQM_REG_PAUSESTATE7 (ACCESS:R  DataWidth:0x10 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e69cUL: // (QM_REG_REMAINPAUSETM2: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_REMAINPAUSETM2 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6a0UL: // (QM_REG_REMAINPAUSETM3: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_REMAINPAUSETM3 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6a4UL: // (QM_REG_REMAINPAUSETM4: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_REMAINPAUSETM4 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6a8UL: // (QM_REG_REMAINPAUSETM5: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_REMAINPAUSETM5 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6acUL: // (QM_REG_REMAINPAUSETM6: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_REMAINPAUSETM6 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6b0UL: // (QM_REG_REMAINPAUSETM7: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_REMAINPAUSETM7 (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6b4UL: // (QM_REG_BIGRAMLOW_EXT_A_TM: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_BIGRAMLOW_EXT_A_TM (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6b8UL: // (QM_REG_BIGRAMHIGH_EXT_A_TM: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nQM_REG_BIGRAMHIGH_EXT_A_TM (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6ccUL: // (QM_REG_PQ2PCIFUNC_4: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_PQ2PCIFUNC_4 (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6d0UL: // (QM_REG_PQ2PCIFUNC_5: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_PQ2PCIFUNC_5 (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6d4UL: // (QM_REG_PQ2PCIFUNC_6: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_PQ2PCIFUNC_6 (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6d8UL: // (QM_REG_PQ2PCIFUNC_7: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nQM_REG_PQ2PCIFUNC_7 (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6dcUL: // (QM_REG_WRBUFFER_TM: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nQM_REG_WRBUFFER_TM (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: NOT USED\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6e8UL: // (QM_REG_BYTECRDCMDQ_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDCMDQ_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED - removed for E3 B0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6ecUL: // (QM_REG_BYTECRDCMDQ_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDCMDQ_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED - removed for E3 B0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6f0UL: // (QM_REG_BYTECRDCMDQ_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDCMDQ_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED - removed for E3 B0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6f4UL: // (QM_REG_BYTECRDCMDQ_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nQM_REG_BYTECRDCMDQ_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: NOT USED - removed for E3 B0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e6f8UL: // (QM_REG_VOQCMDQ: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nQM_REG_VOQCMDQ (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: NOT USED - removed for E3 B0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168458UL: // (QM_REG_QM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nQM_REG_QM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16884cUL: // (QM_REG_QM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nQM_REG_QM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x168c00UL: // (QM_REG_QM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nQM_REG_QM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e060UL: // (QM_REG_QM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nQM_REG_QM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e710UL: // (QM_REG_QM_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nQM_REG_QM_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16e7dcUL: // (QM_REG_QM_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nQM_REG_QM_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18608UL): // (SEM_FAST_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18f00UL): // (SEM_FAST_REG_FILTER_ID: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nSEM_FAST_REG_FILTER_ID (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tFILTER_EVENT_ID :"
				, (u32_t)((data & SEM_FAST_FILTER_ID_REG_FILTER_EVENT_ID) >> 24)
				, "\n\tFILTER_CID :"
				, (u32_t)((data & SEM_FAST_FILTER_ID_REG_FILTER_CID) >> 0));
			break;
		case (0x1A0000 + 0x18f40UL): // (SEM_FAST_REG_FILTER_ENABLE: ACCESS:RW  DataWidth:0x7  Multi Field Register  ) 
			sprintf(disp_str, "\nSEM_FAST_REG_FILTER_ENABLE (ACCESS:RW  DataWidth:0x7  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tFILTER_DRA_SRC_EN :"
				, (u32_t)((data & SEM_FAST_FILTER_ENABLE_REG_FILTER_DRA_SRC_EN) >> 6)
				, "\n\tFILTER_DRA_SRC :"
				, (u32_t)((data & SEM_FAST_FILTER_ENABLE_REG_FILTER_DRA_SRC) >> 5)
				, "\n\tFILTER_EVNT_ID_EN :"
				, (u32_t)((data & SEM_FAST_FILTER_ENABLE_REG_FILTER_EVNT_ID_EN) >> 4)
				, "\n\tFILTER_CID_EN :"
				, (u32_t)((data & SEM_FAST_FILTER_ENABLE_REG_FILTER_CID_EN) >> 3)
				, "\n\tFILTER_CID_USE_RCRD :"
				, (u32_t)((data & SEM_FAST_FILTER_ENABLE_REG_FILTER_CID_USE_RCRD) >> 2)
				, "\n\tFILTER_EN :"
				, (u32_t)((data & SEM_FAST_FILTER_ENABLE_REG_FILTER_EN) >> 0));
			break;
		case (0x1A0000 + 0x19040UL): // (SEM_FAST_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19044UL): // (SEM_FAST_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19048UL): // (SEM_FAST_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x1904cUL): // (SEM_FAST_REG_CPU_MBIST_MEMCTRL_0_STATUS_0: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CPU_MBIST_MEMCTRL_0_STATUS_0 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19050UL): // (SEM_FAST_REG_CPU_MBIST_MEMCTRL_1_STATUS_0: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CPU_MBIST_MEMCTRL_1_STATUS_0 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19054UL): // (SEM_FAST_REG_CPU_MBIST_MEMCTRL_2_STATUS_0: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CPU_MBIST_MEMCTRL_2_STATUS_0 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19058UL): // (SEM_FAST_REG_CPU_MBIST_MEMCTRL_3_STATUS_0: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CPU_MBIST_MEMCTRL_3_STATUS_0 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x1905cUL): // (SEM_FAST_REG_CPU_MBIST_MEMCTRL_3_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CPU_MBIST_MEMCTRL_3_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19060UL): // (SEM_FAST_REG_CPU_MBIST_MEMCTRL_4_STATUS_0: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CPU_MBIST_MEMCTRL_4_STATUS_0 (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19064UL): // (SEM_FAST_REG_CPU_MBIST_MEMCTRL_4_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CPU_MBIST_MEMCTRL_4_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19080UL): // (SEM_FAST_REG_PWRWDOG_E40_PWRDN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PWRWDOG_E40_PWRDN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Power watchdog power down. 1 - power down; 0 - power up.Global\n     register. Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19084UL): // (SEM_FAST_REG_PWRWDOG_E40_CGFG: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PWRWDOG_E40_CGFG (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Power watchdog configuration. Adjustment of integer delay\n     cgfg[7:4] and fractional delay cgfg[3:0]; valid range [0:191]. Global\n     register. Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19088UL): // (SEM_FAST_REG_PWRWDOG_E40_RSEL: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PWRWDOG_E40_RSEL (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Power watchdog selection of delay setting of ring delay element\n    ; recommended setting is 3b100 for longest ring delay; 3b010 will shorten\n     the ring by one inverter+inverting mux; 3b001 will shorten the ring by\n     two inverter+inverting mux cells;Global register. Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x1908cUL): // (SEM_FAST_REG_PWRWDOG_E40_CLEARCFG: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PWRWDOG_E40_CLEARCFG (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Power watchdog selection of delay chain for the reset pulse\n    ; recommended setting is 3b100 for longest pulse; 3b010 will shorten the\n     reset pulse by one inverter+inverting mux delay; 3b001 will shorten the\n     reset pulse by two inverter+inverting mux delays; set clearcfg more or\n     equal than rsel to ensure the ring gets flushed from the previous cycle\n    ; reducing the reset pulse length is needed to reach the highest possible\n     operating frequencies.Global register. Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19090UL): // (SEM_FAST_REG_PWRWDOG_E40_START: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PWRWDOG_E40_START (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Power watchdog. Sets the number of trips through the ring\n     delay element to start+1; range for start is [-1:31].Global register. \n    Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19094UL): // (SEM_FAST_REG_PWRWDOG_E40_ACCU_RUN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PWRWDOG_E40_ACCU_RUN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Power watchdog. When 0 all registers and states are of power\n     watchdog accu sub-module are reset. Global register. Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19098UL): // (SEM_FAST_REG_PWRWDOG_E40_ACCU_SKIP: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PWRWDOG_E40_ACCU_SKIP (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Power watchdog. Allows to decimate the good signal to measure\n     for beats. Global register. Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x1909cUL): // (SEM_FAST_REG_PWRWDOG_E40_ACCU_SKIP_START: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PWRWDOG_E40_ACCU_SKIP_START (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Power watchdog. Start value of skip counter. Global register\n    . Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x190a0UL): // (SEM_FAST_REG_PWRWDOG_E40_DATA: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PWRWDOG_E40_DATA (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: Power watchdog data.Global register. Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x190a4UL): // (SEM_FAST_REG_PWRWDOG_E40_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PWRWDOG_E40_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Power watchdog done. Global register. Reset on POR. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x1ffe0UL): // (SEM_FAST_REG_SEM_FAST_PRTY_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_PRTY_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & SEM_FAST_SEM_FAST_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case (0x1A0000 + 0x1ffe4UL): // (SEM_FAST_REG_SEM_FAST_PRTY_STS_WR: ACCESS:WR  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_PRTY_STS_WR (ACCESS:WR  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & SEM_FAST_SEM_FAST_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case (0x1A0000 + 0x1ffe8UL): // (SEM_FAST_REG_SEM_FAST_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & SEM_FAST_SEM_FAST_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case (0x1A0000 + 0x1ffecUL): // (SEM_FAST_REG_SEM_FAST_PRTY_STS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_PRTY_STS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & SEM_FAST_SEM_FAST_PRTY_STS_REG_PARITY) >> 0));
			break;
		case (0x1A0000 + 0x1fff0UL): // (SEM_FAST_REG_SEM_FAST_INT_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_INT_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & SEM_FAST_SEM_FAST_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case (0x1A0000 + 0x1fff4UL): // (SEM_FAST_REG_SEM_FAST_INT_STS_WR: ACCESS:WR  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_INT_STS_WR (ACCESS:WR  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & SEM_FAST_SEM_FAST_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case (0x1A0000 + 0x1fff8UL): // (SEM_FAST_REG_SEM_FAST_INT_STS_CLR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_INT_STS_CLR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & SEM_FAST_SEM_FAST_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case (0x1A0000 + 0x1fffcUL): // (SEM_FAST_REG_SEM_FAST_INT_STS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_INT_STS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & SEM_FAST_SEM_FAST_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case (0x1A0000 + 0UL): // (SEM_FAST_REG_INT_RAM0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_INT_RAM0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Internal RAM0 (if bit lsb of addr =0 => write to bits[31:\n    0; otherway to [63:32)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10000UL): // (SEM_FAST_REG_CAM_MASK_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_MASK_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: cam_mask [31:0] only for search command\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10040UL): // (SEM_FAST_REG_CAM_MASK_MIDDLE: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_MASK_MIDDLE (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: cam mask [59:32] middle bits only for cam_wdth=80 bit  for\n     search) when cam width is 80 bit;  for cam width = 52 bit is not used\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10400UL): // (SEM_FAST_REG_CAM_MASK_MSB: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_MASK_MSB (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: cam_mask [51:32] only for search command for cam width =52\n    ; or cam_mask [79:60] only for search command for cam width =81\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10800UL): // (SEM_FAST_REG_CAM_VALUE_LSB: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_VALUE_LSB (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: cam_value [31:0]; only for search; add; invalidate commands\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10840UL): // (SEM_FAST_REG_CAM_VALUE_MIDDLE: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_VALUE_MIDDLE (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: cam_value [59:32] only for search; add; invalidate commands\n     when cam width is 80 bit; for cam width = 52 bit is not used\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10c00UL): // (SEM_FAST_REG_CAM_VALUE_MSB: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_VALUE_MSB (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: write or read to this offset will execute CAM command:b19\n    :0-cam_value[51:32]; b29:20-offset only for search command; b30=1-add;\n    0-invalidate; b31=1-search; 0-write\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11000UL): // (SEM_FAST_REG_CAM_RD_DATA_MSB: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_RD_DATA_MSB (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Read data from CAM: b20- validity; b[19:0] - cam_read_data\n    [51:32] when cam width is 52 bit; or b20- validity; b[19:0] - cam_read_data\n    [79:60] when cam width is 80 bit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11040UL): // (SEM_FAST_REG_CAM_RD_DATA_MIDDLE: ACCESS:R  DataWidth:0x1c ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_RD_DATA_MIDDLE (ACCESS:R  DataWidth:0x1c )" \
				"\nDescription: Read data from CAM : b59:32 -cam read data only when cam \n    width = 80 bit; it is not used for cam width = 52 bit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11400UL): // (SEM_FAST_REG_CAM_SEARCH: ACCESS:R  DataWidth:0xa ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_SEARCH (ACCESS:R  DataWidth:0xa )" \
				"\nDescription: cam search result: b8:0-search result; b9-match\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11440UL): // (SEM_FAST_REG_CAM_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Cam mode - 0: cam width =52 bit; 1=cam width=80 bit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11480UL): // (SEM_FAST_REG_CAM_PRTY_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_PRTY_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Cam parity enable - 0: cam read mechanism for parity check\n     is disabled; 1- enabled\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x114c0UL): // (SEM_FAST_REG_CAM_MATCH_VECTOR: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_MATCH_VECTOR (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: cam match vector 128 bit\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11800UL): // (SEM_FAST_REG_PAS_REG_SET0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PAS_REG_SET0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: passive register set0. If address lsb=0=>read from bits 31\n    :0; otherway from bits 63:32. Used only for debugging. STORm must get \n    waitp in time of reading this memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11c00UL): // (SEM_FAST_REG_PAS_REG_SET1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PAS_REG_SET1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: passive register set1. If address lsb=0=>read from bits 31\n    :0; otherway from bits 63:32. Used only for debugging. STORm must get \n    waitp in time of reading this memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x12000UL): // (SEM_FAST_REG_ACTIVE_REG_SET: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_ACTIVE_REG_SET (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: active register set of STORM. If 1 then passive register \n    set 0 is active; other way passive register set 1 is active.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x12400UL): // (SEM_FAST_REG_INT_RAM1: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_INT_RAM1 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Internal RAM1 (if bit lsb of addr =0 => write to bits[31:\n    0; otherway to [63:32)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18000UL): // (SEM_FAST_REG_FOC0_INIT_CREDIT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_FOC0_INIT_CREDIT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Init credit counter for FOC0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18040UL): // (SEM_FAST_REG_FOC1_INIT_CREDIT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_FOC1_INIT_CREDIT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Init credit counter for FOC1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18080UL): // (SEM_FAST_REG_FOC2_INIT_CREDIT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_FOC2_INIT_CREDIT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Init credit counter for FOC2.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x180c0UL): // (SEM_FAST_REG_FOC3_INIT_CREDIT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_FOC3_INIT_CREDIT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Init credit counter for FOC3.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18100UL): // (SEM_FAST_REG_SYNC_DRA_RD_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SYNC_DRA_RD_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full for DRA_RD SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18140UL): // (SEM_FAST_REG_SYNC_RAM_RD_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SYNC_RAM_RD_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full for RAM_RD SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18180UL): // (SEM_FAST_REG_SYNC_EXT_STORE_ALM_FULL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_SYNC_EXT_STORE_ALM_FULL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Almost full for EXT_STORE SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x181c0UL): // (SEM_FAST_REG_MAX_HANDLER_CYCLES: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_MAX_HANDLER_CYCLES (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: maximal number of cycles when STORM is idle. After this number\n     interrupt will be sent.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18200UL): // (SEM_FAST_REG_IDLE_CYCLES: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_IDLE_CYCLES (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: STATISTICS - number of idle cycles of STORM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18240UL): // (SEM_FAST_REG_STORE_HALT_CYCLES: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_STORE_HALT_CYCLES (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: STATISTICS - number of cycles when STORM is idle because \n    of waitp from EXT_STORE FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18280UL): // (SEM_FAST_REG_FIN_HALT_CYCLES: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_FIN_HALT_CYCLES (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: STATISTICS - number of cycles when STORM is idle because \n    of waitp from FIN FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x182c0UL): // (SEM_FAST_REG_PARTIAL_HALT_CYCLES: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PARTIAL_HALT_CYCLES (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: STATISTICS - number of cycles when STORM is idle because \n    of waitp from partial FIN\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18300UL): // (SEM_FAST_REG_RT_CLK_TICK1: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RT_CLK_TICK1 (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: Tick for real time counter1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18340UL): // (SEM_FAST_REG_RT_CLK_TICK2: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RT_CLK_TICK2 (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: Tick for real time counter2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18380UL): // (SEM_FAST_REG_RT_CLK_TICK3: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RT_CLK_TICK3 (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: Tick for real time counter3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x183c0UL): // (SEM_FAST_REG_RT_CLK_TICK4: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RT_CLK_TICK4 (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: Tick for real time counter4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18400UL): // (SEM_FAST_REG_REAL_TIME_CNT1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_REAL_TIME_CNT1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: real_time_cnt1 - reset value is changed next clock after \n    reset\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18440UL): // (SEM_FAST_REG_REAL_TIME_CNT2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_REAL_TIME_CNT2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: real_time_cnt2 - reset value is changed next clock after \n    reset\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18480UL): // (SEM_FAST_REG_REAL_TIME_CNT3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_REAL_TIME_CNT3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: real_time_cnt3 - reset value is changed next clock after \n    reset\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x184c0UL): // (SEM_FAST_REG_REAL_TIME_CNT4: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_REAL_TIME_CNT4 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: real_time_cnt4 - reset value is changed next clock after \n    reset\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18500UL): // (SEM_FAST_REG_FOC_WITHOUT_CREDIT: ACCESS:R  DataWidth:0x16 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_FOC_WITHOUT_CREDIT (ACCESS:R  DataWidth:0x16 )" \
				"\nDescription: Status of FIN command: if message can to send with not enough\n     credit: fin[21];foc0[20];foc1[19];foc2[18];foc3[17];pas[16];highest_rp\n    [15:10;lowest_rp[9:4];foc3_rdy[3];foc2_rdy[2];foc1_rdy[1];foc0_rdy[0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18540UL): // (SEM_FAST_REG_PC_BREAKPOINT: ACCESS:RW  DataWidth:0xf ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PC_BREAKPOINT (ACCESS:RW  DataWidth:0xf )" \
				"\nDescription: PC breakpoint: address of PRAM. If STORM will read from this\n     address. It will get waitp. It is active when ~breakpoint_waitp_disable\n     = 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18580UL): // (SEM_FAST_REG_DBG_ALM_FULL: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DBG_ALM_FULL (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Almost full for DBG SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185c0UL): // (SEM_FAST_REG_STORM_RF0_TM0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_STORM_RF0_TM0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM for STORM_RF0 instaqnce 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185c4UL): // (SEM_FAST_REG_STORM_RF0_TM1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_STORM_RF0_TM1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM for STORM_RF0 instance 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185c8UL): // (SEM_FAST_REG_STORM_RF1_TM0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_STORM_RF1_TM0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM for STORM_RF1 instaqnce 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185ccUL): // (SEM_FAST_REG_STORM_RF1_TM1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_STORM_RF1_TM1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM for STORM_RF1 instance 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185d0UL): // (SEM_FAST_REG_RAM0_RD_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RAM0_RD_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for RAM0_RD SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185d4UL): // (SEM_FAST_REG_RAM1_RD_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RAM1_RD_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM for RAM1_RD SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185d8UL): // (SEM_FAST_REG_RAM0_WR_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RAM0_WR_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for RAM0_WR SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185dcUL): // (SEM_FAST_REG_RAM1_WR_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RAM1_WR_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM for RAM1_WR SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185e0UL): // (SEM_FAST_REG_EXT_STORE_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_EXT_STORE_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for EXT_STORE SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185e4UL): // (SEM_FAST_REG_DBG_LSB_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DBG_LSB_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for DBG_LSB SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185e8UL): // (SEM_FAST_REG_DBG_MSB_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DBG_MSB_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for DBG_MSB SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185ecUL): // (SEM_FAST_REG_DRA_WR_LSB_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DRA_WR_LSB_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for DRA_WR_LSB SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185f0UL): // (SEM_FAST_REG_DRA_WR_MSB_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DRA_WR_MSB_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for DRA_WR_MSB SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185f4UL): // (SEM_FAST_REG_INT_TABLE_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_INT_TABLE_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for NT_TABLE SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185f8UL): // (SEM_FAST_REG_FIN_FIFO_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_FIN_FIFO_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for FIN SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x185fcUL): // (SEM_FAST_REG_DRA_RD_LSB_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DRA_RD_LSB_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for DRA_RD_LSB SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18600UL): // (SEM_FAST_REG_DRA_RD_MSB_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DRA_RD_MSB_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM for DRA_RD_MSB SYNC FIFO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18604UL): // (SEM_FAST_REG_RAM1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RAM1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM for INT_RAM1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18800UL): // (SEM_FAST_REG_ERROR_RST: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_ERROR_RST (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: reset to error interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18840UL): // (SEM_FAST_REG_PARITY_RST: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PARITY_RST (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: reset to parity interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18880UL): // (SEM_FAST_REG_RAM0_EXT_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RAM0_EXT_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: disable for SDM write to int_ram0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x188c0UL): // (SEM_FAST_REG_RAM1_EXT_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RAM1_EXT_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: disable for SDM write to int_ram1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18900UL): // (SEM_FAST_REG_FULL: ACCESS:R  DataWidth:0x12 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_FULL (ACCESS:R  DataWidth:0x12 )" \
				"\nDescription: Full data spelling : {vfc_out_fifo_full; cam_inp_msb_full\n    ; ext_load_rdy; sync_dbg_full; rd_fin_full; wr_data_full; inter_full; \n    sync_wr_pop_full; sync_rd_push_full; sync_fin_full; cam_inp_lsb_full ; \n    cam_inp_msb_full; cam_out_full; sync_ram0_rd_full; sync_ram1_rd_full; \n    sync_ram0_wr_full; sync_ram1_wr_full; sync_ext_store_full}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18940UL): // (SEM_FAST_REG_EMPTY: ACCESS:R  DataWidth:0x11 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_EMPTY (ACCESS:R  DataWidth:0x11 )" \
				"\nDescription: Empty data spelling;{vfc_out_fifo_empty; cam_inp_msb2_empty\n    ; sync_dbg_empty; rd_fin_empty; wr_data_empty; inter_empty; sync_wr_pop_empty\n    ; sync_rd_push_empty; sync_fin_empty; cam_inp_lsb_empty; cam_inp_msb_empty\n    ; cam_out_empty; sync_ram0_rd_empty; sync_ram1_rd_empty; sync_ram0_wr_empty\n    ; sync_ram1_wr_empty; sync_ext_store_empty}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18980UL): // (SEM_FAST_REG_ALM_FULL: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_ALM_FULL (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: alm_full data spelling; {ram0_alm_full; ram1_alm_full; ext_alm_full\n    }\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x189c0UL): // (SEM_FAST_REG_STATE_MACHINE: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_STATE_MACHINE (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: state machine bus spelling [vfc_ou_fifo_counter[11:8]; 2'\n    b0; int_state[5:4]; 1'b0; wr_state[2:0}]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18a00UL): // (SEM_FAST_REG_PRAM_LAST_ADDR: ACCESS:R  DataWidth:0x1e ) 
			sprintf(disp_str, "\nSEM_FAST_REG_PRAM_LAST_ADDR (ACCESS:R  DataWidth:0x1e )" \
				"\nDescription: last read address from STORM to pram {add_p_out_high; 2'b0\n    ; add_p_out_low}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18a40UL): // (SEM_FAST_REG_RBC_CAM_CLK_DIV: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_RBC_CAM_CLK_DIV (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: cam clock divider : (may be equal to 2; 4; or 8 only) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18a80UL): // (SEM_FAST_REG_WAITP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_WAITP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: waitp to STORM(b0=1-set; b0=0-reset) \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18ac0UL): // (SEM_FAST_REG_DEBUG_ACTIVE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DEBUG_ACTIVE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug active register. If set then output to debug FIFO is\n     open\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18b00UL): // (SEM_FAST_REG_DEBUG_MODE: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DEBUG_MODE (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: debug mode (0-store to 0x7000; 1-pram_addr; 2-foc ready; \n    3-(b[31int; b[30:16-int_add;b[15-dra_rd; b[14-dra_rd_set; b[13:8-dra_rd_addr\n    ; b[7dra_wr; b[6dra_wr_set; b[5:0dra_wr_addr); 4- (dmem_wr[17; dmem_rd\n    [16; dmem_add[15:0); 5-(int_rbc_cur_state[5:4; 1'b0; wr_rbc_cur_state[\n    2:0); 6-recording handlers. This register is not applicable when ~debug_active\n     =0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18b40UL): // (SEM_FAST_REG_INT_WAITP_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_INT_WAITP_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: disable waitp by STORM to itself. If this register equal \n    to 1then STORE from microcode to address 0x6015 is disabled.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18b80UL): // (SEM_FAST_REG_EXT_WAITP_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_EXT_WAITP_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: disable waitp from other 3 STORM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18bc0UL): // (SEM_FAST_REG_BREAKPOINT_WAITP_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_BREAKPOINT_WAITP_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: disable waitp by address is equal to the PcBreakpoint \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18c00UL): // (SEM_FAST_REG_DBG_WAITP_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DBG_WAITP_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: disable waitp by full debug fifo for debug mode 0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18c40UL): // (SEM_FAST_REG_DBG_MODE3_SRC_DISABLE: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DBG_MODE3_SRC_DISABLE (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Disable register for each output of ~debug_mode=3 : b0-dra_wr\n     disable; b1-dra_rd disable; b2-int disable\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18c80UL): // (SEM_FAST_REG_DBG_MODE4_SRC_DISABLE: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DBG_MODE4_SRC_DISABLE (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Disable register for each output of  ~debug_mode=4 : b0-dmem_wr\n     disable; b1-dmem_rd disable\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18cc0UL): // (SEM_FAST_REG_CAM_BIST_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_BIST_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Bist enable bit for Cam\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18cc4UL): // (SEM_FAST_REG_CAM_BIST_DBG_SEL: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_BIST_DBG_SEL (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: This registers selects the type of data present on bist_status\n     bus \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18d00UL): // (SEM_FAST_REG_CAM_BIST_DONE: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_BIST_DONE (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Bist done bit from CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18d40UL): // (SEM_FAST_REG_CAM_BIST_GO: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_BIST_GO (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Bist go bit from CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18d80UL): // (SEM_FAST_REG_CAM_S2_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_S2_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: S2 status from CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18dc0UL): // (SEM_FAST_REG_CAM_S3_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_S3_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: S3 status from CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18e00UL): // (SEM_FAST_REG_CAM_S5_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_S5_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: S5 status from CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18e40UL): // (SEM_FAST_REG_CAM_S6_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_S6_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: S6 status from CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18e80UL): // (SEM_FAST_REG_CAM_S8_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_S8_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: S8 status from CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18e84UL): // (SEM_FAST_REG_CAM_BIST_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_CAM_BIST_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bist status from CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18ec0UL): // (SEM_FAST_REG_DBG_MODE6_SRC_DISABLE: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DBG_MODE6_SRC_DISABLE (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Disable register for each output of  ~debug_mode=6 : b0-dra_in\n     disable; b1-fin disable; b2-load disable;b3-pram disable;b4-ext storedisable\n    ; b5-other store disable; ; b6-vfc store disable)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18f80UL): // (SEM_FAST_REG_DBG_HALT_CYCLES: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_DBG_HALT_CYCLES (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: STATISTICS - number of cycles when STORM is idle because \n    of waitp from debug block (recorded handlers or waitp mode)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fc0UL): // (SEM_FAST_REG_EVENT_ID_MASK: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_EVENT_ID_MASK (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: mask for event id. 1- specified bit is ignored; 0 - specified\n     bit is checked\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fc4UL): // (SEM_FAST_REG_VFC_DATA_WR: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_VFC_DATA_WR (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Command data for VFC. VFC will accumulate all writing to \n    this register till will be done write to vfc_wr_addr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fc8UL): // (SEM_FAST_REG_VFC_ADDR: ACCESS:W  DataWidth:0xc ) 
			sprintf(disp_str, "\nSEM_FAST_REG_VFC_ADDR (ACCESS:W  DataWidth:0xc )" \
				"\nDescription: Command address for VFC. Write to it should be done when \n    all command data was already written to vfc_data_wr register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fccUL): // (SEM_FAST_REG_VFC_DATA_RD: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_VFC_DATA_RD (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Read data from VFC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fd0UL): // (SEM_FAST_REG_VFC_STATUS: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_VFC_STATUS (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: B0 - response is ready. It is set when response cycle of \n    32 bit is ready from VFC block. It is reset when read is done from vfc_data_rd\n     register; B1 - vfc is busy. It is set when was done write to vfc_addr\n     register. It is reset when last from VFC was received. B2 - sending command\n     is on going. It will be set when was done write to vfc_data_wr register\n    . It will be reset when it was done write to vfc_addr register. New command\n     may be sent from RBC when all 3 bits of this register is reset. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fd4UL): // (SEM_FAST_REG_GPRE0: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_GPRE0 (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: GPRE0 bits {2'b0; message is ready in passive register file\n    ; dra_wr path is not empty; vfc ready;ext_load_rdy; cam ready; dra-empty\n    ; thread_err}\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fd8UL): // (SEM_FAST_REG_IDLE_CNT0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_IDLE_CNT0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Statistics - number of STORM idle cycles when whole SEM block\n     is empty - no sleeping threads and no messages from FIC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fdcUL): // (SEM_FAST_REG_IDLE_CNT1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_IDLE_CNT1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Statistics - number of STORM idle cycles when all threads\n     are sleeping\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fe0UL): // (SEM_FAST_REG_IDLE_CNT2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_IDLE_CNT2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Statistics - number of STORM idle cycles when dra write is\n     slower but not because of missed credits from foc interfaces\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fe4UL): // (SEM_FAST_REG_IDLE_CNT3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_IDLE_CNT3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Statistics - number of STORM idle cycles when dra write is\n     slower because of missed credits from foc interfaces\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fe8UL): // (SEM_FAST_REG_IDLE_CNT4: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_IDLE_CNT4 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Statistics - number of STORM idle cycles when at least one\n     thread is allocated\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18fecUL): // (SEM_FAST_REG_MAX_THREAD_NUM: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_MAX_THREAD_NUM (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Statistics - number of STORM idle cycles when dra write is\n     slower because of missed credits from foc interfaces\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19000UL): // (SEM_FAST_REG_GPRE: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSEM_FAST_REG_GPRE (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: 16 GPRE registers from STORM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10004UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10044UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10404UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10804UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x10844UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11004UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11044UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_6: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_6 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11404UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_7: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_7 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11444UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_8: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_8 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11484UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_9: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_9 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x114c4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_10: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_10 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11804UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_11: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_11 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x11c04UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_12: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_12 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x12004UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_13: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_13 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x12800UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_14: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_14 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18004UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_15: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_15 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18044UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_16: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_16 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18084UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_17: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_17 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x180c4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_18: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_18 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18104UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_19: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_19 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18144UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_20: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_20 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18184UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_21: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_21 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x181c4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_22: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_22 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18204UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_23: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_23 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18244UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_24: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_24 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18284UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_25: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_25 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x182c4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_26: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_26 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18304UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_27: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_27 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18344UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_28: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_28 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18384UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_29: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_29 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x183c4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_30: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_30 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18404UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_31: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_31 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18444UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_32: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_32 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18484UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_33: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_33 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x184c4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_34: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_34 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18504UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_35: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_35 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18544UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_36: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_36 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18584UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_37: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_37 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x1860cUL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_38: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_38 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18804UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_39: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_39 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18844UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_40: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_40 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18884UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_41: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_41 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x188c4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_42: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_42 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18904UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_43: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_43 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18944UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_44: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_44 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18984UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_45: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_45 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x189c4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_46: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_46 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18a04UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_47: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_47 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18a44UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_48: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_48 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18a84UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_49: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_49 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18ac4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_50: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_50 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18b04UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_51: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_51 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18b44UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_52: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_52 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18b84UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_53: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_53 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18bc4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_54: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_54 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18c04UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_55: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_55 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18c44UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_56: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_56 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18c84UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_57: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_57 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18cc8UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_58: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_58 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18d04UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_59: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_59 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18d44UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_60: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_60 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18d84UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_61: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_61 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18dc4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_62: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_62 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18e04UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_63: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_63 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18e44UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_64: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_64 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18e88UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_65: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_65 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18ec4UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_66: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_66 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18f04UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_67: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_67 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18f44UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_68: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_68 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18f84UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_69: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_69 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x18ff0UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_70: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_70 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19004UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_71: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_71 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x19068UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_72: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_72 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (0x1A0000 + 0x190a8UL): // (SEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_73: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSEM_FAST_REG_SEM_FAST_UNUSED_EMPTY_73 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40408UL: // (SRC_REG_KEYRSS0_0: ACCESS:RW  DataWidth:0x20  Register KeyRss0: 1 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_0 (ACCESS:RW  DataWidth:0x20  Register KeyRss0: 1 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4040cUL: // (SRC_REG_KEYRSS0_1: ACCESS:RW  DataWidth:0x20  Register KeyRss0: 2 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_1 (ACCESS:RW  DataWidth:0x20  Register KeyRss0: 2 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40410UL: // (SRC_REG_KEYRSS0_2: ACCESS:RW  DataWidth:0x20  Register KeyRss0: 3 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_2 (ACCESS:RW  DataWidth:0x20  Register KeyRss0: 3 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40414UL: // (SRC_REG_KEYRSS0_3: ACCESS:RW  DataWidth:0x20  Register KeyRss0: 4 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_3 (ACCESS:RW  DataWidth:0x20  Register KeyRss0: 4 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40418UL: // (SRC_REG_KEYRSS0_4: ACCESS:RW  DataWidth:0x20  Register KeyRss0: 5 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_4 (ACCESS:RW  DataWidth:0x20  Register KeyRss0: 5 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4041cUL: // (SRC_REG_KEYRSS0_5: ACCESS:RW  DataWidth:0x20  Register KeyRss0: 6 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_5 (ACCESS:RW  DataWidth:0x20  Register KeyRss0: 6 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40420UL: // (SRC_REG_KEYRSS0_6: ACCESS:RW  DataWidth:0x20  Register KeyRss0: 7 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_6 (ACCESS:RW  DataWidth:0x20  Register KeyRss0: 7 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40424UL: // (SRC_REG_KEYRSS0_7: ACCESS:RW  DataWidth:0x20  Register KeyRss0: 8 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_7 (ACCESS:RW  DataWidth:0x20  Register KeyRss0: 8 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40428UL: // (SRC_REG_KEYRSS0_8: ACCESS:RW  DataWidth:0x20  Register KeyRss0: 9 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_8 (ACCESS:RW  DataWidth:0x20  Register KeyRss0: 9 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4042cUL: // (SRC_REG_KEYRSS0_9: ACCESS:RW  DataWidth:0x20  Register KeyRss0: 10 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_9 (ACCESS:RW  DataWidth:0x20  Register KeyRss0: 10 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40430UL: // (SRC_REG_KEYRSS1_0: ACCESS:RW  DataWidth:0x20  Register KeyRss1: 1 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_0 (ACCESS:RW  DataWidth:0x20  Register KeyRss1: 1 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40434UL: // (SRC_REG_KEYRSS1_1: ACCESS:RW  DataWidth:0x20  Register KeyRss1: 2 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_1 (ACCESS:RW  DataWidth:0x20  Register KeyRss1: 2 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40438UL: // (SRC_REG_KEYRSS1_2: ACCESS:RW  DataWidth:0x20  Register KeyRss1: 3 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_2 (ACCESS:RW  DataWidth:0x20  Register KeyRss1: 3 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4043cUL: // (SRC_REG_KEYRSS1_3: ACCESS:RW  DataWidth:0x20  Register KeyRss1: 4 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_3 (ACCESS:RW  DataWidth:0x20  Register KeyRss1: 4 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40440UL: // (SRC_REG_KEYRSS1_4: ACCESS:RW  DataWidth:0x20  Register KeyRss1: 5 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_4 (ACCESS:RW  DataWidth:0x20  Register KeyRss1: 5 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40444UL: // (SRC_REG_KEYRSS1_5: ACCESS:RW  DataWidth:0x20  Register KeyRss1: 6 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_5 (ACCESS:RW  DataWidth:0x20  Register KeyRss1: 6 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40448UL: // (SRC_REG_KEYRSS1_6: ACCESS:RW  DataWidth:0x20  Register KeyRss1: 7 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_6 (ACCESS:RW  DataWidth:0x20  Register KeyRss1: 7 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4044cUL: // (SRC_REG_KEYRSS1_7: ACCESS:RW  DataWidth:0x20  Register KeyRss1: 8 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_7 (ACCESS:RW  DataWidth:0x20  Register KeyRss1: 8 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40450UL: // (SRC_REG_KEYRSS1_8: ACCESS:RW  DataWidth:0x20  Register KeyRss1: 9 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_8 (ACCESS:RW  DataWidth:0x20  Register KeyRss1: 9 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40454UL: // (SRC_REG_KEYRSS1_9: ACCESS:RW  DataWidth:0x20  Register KeyRss1: 10 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_9 (ACCESS:RW  DataWidth:0x20  Register KeyRss1: 10 out of 10)" \
				"\nDescription: Key for parser RSS hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40458UL: // (SRC_REG_KEYSEARCH_0: ACCESS:RW  DataWidth:0x20  Register KeySearch: 1 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_0 (ACCESS:RW  DataWidth:0x20  Register KeySearch: 1 out of 10)" \
				"\nDescription: Key for searcher hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4045cUL: // (SRC_REG_KEYSEARCH_1: ACCESS:RW  DataWidth:0x20  Register KeySearch: 2 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_1 (ACCESS:RW  DataWidth:0x20  Register KeySearch: 2 out of 10)" \
				"\nDescription: Key for searcher hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40460UL: // (SRC_REG_KEYSEARCH_2: ACCESS:RW  DataWidth:0x20  Register KeySearch: 3 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_2 (ACCESS:RW  DataWidth:0x20  Register KeySearch: 3 out of 10)" \
				"\nDescription: Key for searcher hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40464UL: // (SRC_REG_KEYSEARCH_3: ACCESS:RW  DataWidth:0x20  Register KeySearch: 4 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_3 (ACCESS:RW  DataWidth:0x20  Register KeySearch: 4 out of 10)" \
				"\nDescription: Key for searcher hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40468UL: // (SRC_REG_KEYSEARCH_4: ACCESS:RW  DataWidth:0x20  Register KeySearch: 5 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_4 (ACCESS:RW  DataWidth:0x20  Register KeySearch: 5 out of 10)" \
				"\nDescription: Key for searcher hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4046cUL: // (SRC_REG_KEYSEARCH_5: ACCESS:RW  DataWidth:0x20  Register KeySearch: 6 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_5 (ACCESS:RW  DataWidth:0x20  Register KeySearch: 6 out of 10)" \
				"\nDescription: Key for searcher hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40470UL: // (SRC_REG_KEYSEARCH_6: ACCESS:RW  DataWidth:0x20  Register KeySearch: 7 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_6 (ACCESS:RW  DataWidth:0x20  Register KeySearch: 7 out of 10)" \
				"\nDescription: Key for searcher hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40474UL: // (SRC_REG_KEYSEARCH_7: ACCESS:RW  DataWidth:0x20  Register KeySearch: 8 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_7 (ACCESS:RW  DataWidth:0x20  Register KeySearch: 8 out of 10)" \
				"\nDescription: Key for searcher hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40478UL: // (SRC_REG_KEYSEARCH_8: ACCESS:RW  DataWidth:0x20  Register KeySearch: 9 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_8 (ACCESS:RW  DataWidth:0x20  Register KeySearch: 9 out of 10)" \
				"\nDescription: Key for searcher hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4047cUL: // (SRC_REG_KEYSEARCH_9: ACCESS:RW  DataWidth:0x20  Register KeySearch: 10 out of 10) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_9 (ACCESS:RW  DataWidth:0x20  Register KeySearch: 10 out of 10)" \
				"\nDescription: Key for searcher hash function - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40480UL: // (SRC_REG_NUM_CONCURRENT_TASKS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_NUM_CONCURRENT_TASKS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: NOT IMPLEMENTED (Number of concurrent tasks; Values can be\n     1-4.)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40494UL: // (SRC_REG_ALLOWSHORTCUT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSRC_REG_ALLOWSHORTCUT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set; same search shortcut is allowed.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40498UL: // (SRC_REG_MAXNUMHOPS: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nSRC_REG_MAXNUMHOPS (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The maximum allowed HOP to search.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4049cUL: // (SRC_REG_SOFT_RST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSRC_REG_SOFT_RST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Reset internal state machines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x404a0UL: // (SRC_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSRC_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from SRC\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x404a4UL: // (SRC_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSRC_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from SRC\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x404a8UL: // (SRC_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from SRC\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x404acUL: // (SRC_REG_SRC_INT_STS: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_SRC_INT_STS (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_FIFO_ERR :"
				, (u32_t)((data & SRC_SRC_INT_STS_REG_WR_FIFO_ERR) >> 2)
				, "\n\tMMURQ_FIFO_ERR :"
				, (u32_t)((data & SRC_SRC_INT_STS_REG_MMURQ_FIFO_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & SRC_SRC_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x404b0UL: // (SRC_REG_SRC_INT_STS_CLR: ACCESS:RC  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_SRC_INT_STS_CLR (ACCESS:RC  DataWidth:0x3 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_FIFO_ERR :"
				, (u32_t)((data & SRC_SRC_INT_STS_CLR_REG_WR_FIFO_ERR) >> 2)
				, "\n\tMMURQ_FIFO_ERR :"
				, (u32_t)((data & SRC_SRC_INT_STS_CLR_REG_MMURQ_FIFO_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & SRC_SRC_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x404b4UL: // (SRC_REG_SRC_INT_STS_WR: ACCESS:WR  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_SRC_INT_STS_WR (ACCESS:WR  DataWidth:0x3 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_FIFO_ERR :"
				, (u32_t)((data & SRC_SRC_INT_STS_WR_REG_WR_FIFO_ERR) >> 2)
				, "\n\tMMURQ_FIFO_ERR :"
				, (u32_t)((data & SRC_SRC_INT_STS_WR_REG_MMURQ_FIFO_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & SRC_SRC_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x404b8UL: // (SRC_REG_SRC_INT_MASK: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_SRC_INT_MASK (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWR_FIFO_ERR :"
				, (u32_t)((data & SRC_SRC_INT_MASK_REG_WR_FIFO_ERR) >> 2)
				, "\n\tMMURQ_FIFO_ERR :"
				, (u32_t)((data & SRC_SRC_INT_MASK_REG_MMURQ_FIFO_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & SRC_SRC_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x404bcUL: // (SRC_REG_SRC_PRTY_STS: ACCESS:R  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_SRC_PRTY_STS (ACCESS:R  DataWidth:0x3 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWDATA :"
				, (u32_t)((data & SRC_SRC_PRTY_STS_REG_WDATA) >> 2)
				, "\n\tMEM_RQ :"
				, (u32_t)((data & SRC_SRC_PRTY_STS_REG_MEM_RQ) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & SRC_SRC_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x404c0UL: // (SRC_REG_SRC_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_SRC_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x3 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWDATA :"
				, (u32_t)((data & SRC_SRC_PRTY_STS_CLR_REG_WDATA) >> 2)
				, "\n\tMEM_RQ :"
				, (u32_t)((data & SRC_SRC_PRTY_STS_CLR_REG_MEM_RQ) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & SRC_SRC_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x404c4UL: // (SRC_REG_SRC_PRTY_STS_WR: ACCESS:WR  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_SRC_PRTY_STS_WR (ACCESS:WR  DataWidth:0x3 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWDATA :"
				, (u32_t)((data & SRC_SRC_PRTY_STS_WR_REG_WDATA) >> 2)
				, "\n\tMEM_RQ :"
				, (u32_t)((data & SRC_SRC_PRTY_STS_WR_REG_MEM_RQ) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & SRC_SRC_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x404c8UL: // (SRC_REG_SRC_PRTY_MASK: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_SRC_PRTY_MASK (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWDATA :"
				, (u32_t)((data & SRC_SRC_PRTY_MASK_REG_WDATA) >> 2)
				, "\n\tMEM_RQ :"
				, (u32_t)((data & SRC_SRC_PRTY_MASK_REG_MEM_RQ) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & SRC_SRC_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x404ccUL: // (SRC_REG_E1HMF_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSRC_REG_E1HMF_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If clr the searcher is compatible to E1 A0 - support only\n     two ports. If set the searcher support 8 functions.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x404d0UL: // (SRC_REG_KEYRSS0_EXT5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS0_EXT5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Key for parser RSS 5 tuple extantion hash - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x404d4UL: // (SRC_REG_KEYRSS1_EXT5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSRC_REG_KEYRSS1_EXT5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Key for parser RSS 5 tuple extantion hash - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40550UL: // (SRC_REG_TM_RAM_36X66: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nSRC_REG_TM_RAM_36X66 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: tm bits for ram_36x66\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40554UL: // (SRC_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nSRC_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: eco reserved\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40558UL: // (SRC_REG_ATCTBL1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_ATCTBL1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Controls PXP Request ATC field when accessing Table 1 (logical\n     addressing)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4055cUL: // (SRC_REG_ATCTBL2: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nSRC_REG_ATCTBL2 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Controls PXP Request ATC field when accessing Table 2 (Physical\n     addressing)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40560UL: // (SRC_REG_KEYSEARCH_VLAN: ACCESS:RW  DataWidth:0xc ) 
			sprintf(disp_str, "\nSRC_REG_KEYSEARCH_VLAN (ACCESS:RW  DataWidth:0xc )" \
				"\nDescription: Key for searcher hash function vlan field\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40564UL: // (SRC_REG_VLAN_HASH_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSRC_REG_VLAN_HASH_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for VLAN in Hash Address\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40610UL: // (SRC_REG_ALLOWEMPTYSHORTCUT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSRC_REG_ALLOWEMPTYSHORTCUT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set; search return no match on empty shortcut is allowed\n    . \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40614UL: // (SRC_REG_VLAN_MATCH_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nSRC_REG_VLAN_MATCH_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disable VLAN and VLAN Promiscuous Mode (vpf) matching logic\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40400UL: // (SRC_REG_NUMBER_HASH_BITS0: ACCESS:RW  DataWidth:0x5  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_NUMBER_HASH_BITS0 (ACCESS:RW  DataWidth:0x5  SPLIT:4 )" \
				"\nDescription: The number of hash bits used for the search (h); Values can\n     be 8 to 24.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40404UL: // (SRC_REG_NUMBER_HASH_BITS1: ACCESS:RW  DataWidth:0x5  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_NUMBER_HASH_BITS1 (ACCESS:RW  DataWidth:0x5  SPLIT:4 )" \
				"\nDescription: The number of hash bits used for the search (h); Values can\n     be 8 to 24.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40484UL: // (SRC_REG_NUMIPV4CONN0: ACCESS:RW  DataWidth:0x1a  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_NUMIPV4CONN0 (ACCESS:RW  DataWidth:0x1a  SPLIT:4 )" \
				"\nDescription: Number of Ipv4 connections - port 0 (statistics).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40488UL: // (SRC_REG_NUMIPV4CONN1: ACCESS:RW  DataWidth:0x1a  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_NUMIPV4CONN1 (ACCESS:RW  DataWidth:0x1a  SPLIT:4 )" \
				"\nDescription: Number of Ipv4 connections (statistics) - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4048cUL: // (SRC_REG_NUMIPV6CONN0: ACCESS:RW  DataWidth:0x1a  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_NUMIPV6CONN0 (ACCESS:RW  DataWidth:0x1a  SPLIT:4 )" \
				"\nDescription: Number of Ipv6 connections (statistics) - port 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40490UL: // (SRC_REG_NUMIPV6CONN1: ACCESS:RW  DataWidth:0x1a  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_NUMIPV6CONN1 (ACCESS:RW  DataWidth:0x1a  SPLIT:4 )" \
				"\nDescription: Number of Ipv6 connections (statistics) - port 1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x404e0UL: // (SRC_REG_EMPTY_PF: ACCESS:RW  DataWidth:0x20  SPLIT:8 ) 
			sprintf(disp_str, "\nSRC_REG_EMPTY_PF (ACCESS:RW  DataWidth:0x20  SPLIT:8 )" \
				"\nDescription: empty bit per bin 256 bins per PF\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40500UL: // (SRC_REG_COUNTFREE0: ACCESS:RW  DataWidth:0x16  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_COUNTFREE0 (ACCESS:RW  DataWidth:0x16  SPLIT:4 )" \
				"\nDescription: Number of free element in the free list of T2 entries - port\n     0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40504UL: // (SRC_REG_COUNTFREE1: ACCESS:RW  DataWidth:0x16  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_COUNTFREE1 (ACCESS:RW  DataWidth:0x16  SPLIT:4 )" \
				"\nDescription: Number of free element in the free list of T2 entries - port\n     1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40510UL: // (SRC_REG_FIRSTFREE0: ACCESS:WB  DataWidth:0x40  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_FIRSTFREE0 (ACCESS:WB  DataWidth:0x40  SPLIT:4 )" \
				"\nDescription: First free element in the free list of T2 entries - port \n    0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40520UL: // (SRC_REG_FIRSTFREE1: ACCESS:WB  DataWidth:0x40  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_FIRSTFREE1 (ACCESS:WB  DataWidth:0x40  SPLIT:4 )" \
				"\nDescription: First free element in the free list of T2 entries - port \n    1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40530UL: // (SRC_REG_LASTFREE0: ACCESS:WB  DataWidth:0x40  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_LASTFREE0 (ACCESS:WB  DataWidth:0x40  SPLIT:4 )" \
				"\nDescription: Last free element in the free list of T2 entries - port 0\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40540UL: // (SRC_REG_LASTFREE1: ACCESS:WB  DataWidth:0x40  SPLIT:4 ) 
			sprintf(disp_str, "\nSRC_REG_LASTFREE1 (ACCESS:WB  DataWidth:0x40  SPLIT:4 )" \
				"\nDescription: Last free element in the free list of T2 entries - port 1\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40568UL: // (SRC_REG_DBG_OUT_DATA_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSRC_REG_DBG_OUT_DATA_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from SRC\n     to the DBG block) - The 32 lsb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4056cUL: // (SRC_REG_DBG_OUT_DATA_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nSRC_REG_DBG_OUT_DATA_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from SRC\n     to the DBG block) - The 32 msb data that goes to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40570UL: // (SRC_REG_DBG_OUT_FRAME: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nSRC_REG_DBG_OUT_FRAME (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from SRC\n     to the DBG block) - The 4 frame bits that goes to the DBG block. Bit0\n     is the frame of data byte0; Bit1 is the frame of  data byte1; Bit2 is\n     the frame of data byte2; Bit3 is the frame of  data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40574UL: // (SRC_REG_DBG_OUT_VALID: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nSRC_REG_DBG_OUT_VALID (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from SRC\n     to the DBG block) - The 4 valid bits that goes to the DBG block. Bit0\n     validtes data byte0; Bit1 validates data byte1; Bit2 validates data byte2\n    ; Bit3 validates data byte4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x404d8UL: // (SRC_REG_SRC_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSRC_REG_SRC_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40508UL: // (SRC_REG_SRC_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSRC_REG_SRC_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40578UL: // (SRC_REG_SRC_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSRC_REG_SRC_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x40618UL: // (SRC_REG_SRC_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nSRC_REG_SRC_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50000UL: // (TCM_REG_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. Initialises specific states and statuses. To \n    initialise the state - write 1 into register; to enable working after \n    that - write 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50004UL: // (TCM_REG_TCM_STORM0_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_TCM_STORM0_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - STORM 0 Interface enable. If 0 - the acknowledge input\n     is disregarded; valid is deasserted; all other signals are treated as\n     usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50008UL: // (TCM_REG_TCM_STORM1_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_TCM_STORM1_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - STORM 1 Interface enable. If 0 - the acknowledge input\n     is disregarded; valid is deasserted; all other signals are treated as\n     usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5000cUL: // (TCM_REG_TCM_TQM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_TCM_TQM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - QM Interface enable. If 0 - the acknowledge input is\n     disregarded; valid is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50010UL: // (TCM_REG_STORM_TCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_STORM_TCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: STORM - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50014UL: // (TCM_REG_TQM_TCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_TQM_TCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: QM - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50018UL: // (TCM_REG_TSDM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_TSDM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input SDM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5001cUL: // (TCM_REG_TM_TCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_TM_TCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Timers - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50020UL: // (TCM_REG_PRS_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_PRS_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input prs Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50024UL: // (TCM_REG_PBF_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_PBF_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input pbf Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50028UL: // (TCM_REG_USEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_USEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input usem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5002cUL: // (TCM_REG_CSEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_CSEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input csem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50030UL: // (TCM_REG_CDU_AG_WR_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_CDU_AG_WR_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU AG write Interface enable. If 0 - the request and valid\n     input are disregarded; all other signals are treated as usual; if 1 - \n    normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50034UL: // (TCM_REG_CDU_AG_RD_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_CDU_AG_RD_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU AG read Interface enable. If 0 - the request input is\n     disregarded; valid output is deasserted; all other signals are treated\n     as usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50038UL: // (TCM_REG_CDU_SM_WR_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_CDU_SM_WR_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU STORM write Interface enable. If 0 - the request and \n    valid input is disregarded; all other signals are treated as usual; if\n     1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5003cUL: // (TCM_REG_CDU_SM_RD_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_CDU_SM_RD_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU STORM read Interface enable. If 0 - the request input\n     is disregarded; valid output is deasserted; all other signals are treated\n     as usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50040UL: // (TCM_REG_TCM_CFC_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_TCM_CFC_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - CFC Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50044UL: // (TCM_REG_XX_MAX_LL_SZ: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTCM_REG_XX_MAX_LL_SZ (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Maximum link list size (messages locked) per connection in\n     the XX protection.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50048UL: // (TCM_REG_XX_OVFL_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTCM_REG_XX_OVFL_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID; sent to the STORM in case of XX overflow.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5004cUL: // (TCM_REG_XX_MAX_NUM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_XX_MAX_NUM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The maximum number of connections in the XX protection.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50050UL: // (TCM_REG_N_SM_CTX_LD_0: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_0 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50054UL: // (TCM_REG_N_SM_CTX_LD_1: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_1 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50058UL: // (TCM_REG_N_SM_CTX_LD_2: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_2 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5005cUL: // (TCM_REG_N_SM_CTX_LD_3: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_3 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50060UL: // (TCM_REG_N_SM_CTX_LD_4: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_4 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50064UL: // (TCM_REG_N_SM_CTX_LD_5: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_5 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50068UL: // (TCM_REG_N_SM_CTX_LD_6: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_6 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5006cUL: // (TCM_REG_N_SM_CTX_LD_7: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_7 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50070UL: // (TCM_REG_N_SM_CTX_LD_8: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_8 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50074UL: // (TCM_REG_N_SM_CTX_LD_9: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_9 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50078UL: // (TCM_REG_N_SM_CTX_LD_10: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_10 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5007cUL: // (TCM_REG_N_SM_CTX_LD_11: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_11 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50080UL: // (TCM_REG_N_SM_CTX_LD_12: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_12 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50084UL: // (TCM_REG_N_SM_CTX_LD_13: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_13 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50088UL: // (TCM_REG_N_SM_CTX_LD_14: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_14 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5008cUL: // (TCM_REG_N_SM_CTX_LD_15: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_N_SM_CTX_LD_15 (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used to align to STORM context row size of 128 bits. The offset of\n     these data in the STORM context is always 0. Index _i stands for the \n    connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50090UL: // (TCM_REG_TQM_TCM_HDR_P: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nTCM_REG_TQM_TCM_HDR_P (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header value for QM request (primary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50094UL: // (TCM_REG_TQM_TCM_HDR_S: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nTCM_REG_TQM_TCM_HDR_S (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header value for QM request (secondary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50098UL: // (TCM_REG_TM_TCM_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nTCM_REG_TM_TCM_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header for Timers expiration command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5009cUL: // (TCM_REG_ERR_TCM_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nTCM_REG_ERR_TCM_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM erroneous header for QM and Timers formatting.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500a0UL: // (TCM_REG_ERR_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTCM_REG_ERR_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID in case of ErrorFlg is set in the input message\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500a4UL: // (TCM_REG_EXPR_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTCM_REG_EXPR_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID for Timers expiration.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500a8UL: // (TCM_REG_STOP_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTCM_REG_STOP_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID for Timers formatting in case of stop done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500acUL: // (TCM_REG_STORM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_STORM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the STORM input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500b0UL: // (TCM_REG_PRS_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_PRS_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input prs in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500b4UL: // (TCM_REG_PBF_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_PBF_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input pbf in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500b8UL: // (TCM_REG_USEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_USEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input usem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500bcUL: // (TCM_REG_CSEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_CSEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input csem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500c0UL: // (TCM_REG_CP_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_CP_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the CP input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500c4UL: // (TCM_REG_TSDM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_TSDM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the SDM input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500c8UL: // (TCM_REG_TQM_P_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_TQM_P_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the QM (primary) input in the WRR mechanism\n    . 0 stands for weight 8 (the most prioritised); 1 stands for weight 1(\n    least prioritised); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500ccUL: // (TCM_REG_TQM_S_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_TQM_S_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the QM (secondary) input in the WRR mechanism\n    . 0 stands for weight 8 (the most prioritised); 1 stands for weight 1(\n    least prioritised); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500d0UL: // (TCM_REG_TM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_TM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the Timers input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500d4UL: // (TCM_REG_TCM_TQM_USE_Q: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_TCM_TQM_USE_Q (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set the Q index; received from the QM is inserted to event\n     ID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500d8UL: // (TCM_REG_TCM_REG0_SZ: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_TCM_REG0_SZ (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The size of AG context region 0 in REG-pairs. Designates \n    the MS REG-pair number (e.g. if region 0 is 6 REG-pairs; the value should\n     be 5). Is used to determine the number of the AG context REG-pairs written\n     back; when the input message Reg1WbFlg isn't set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500dcUL: // (TCM_REG_STOP_DONE_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nTCM_REG_STOP_DONE_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header for Timers expiration command; when Stop Done\n     input bit is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50100UL: // (TCM_REG_PM_RAM_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTCM_REG_PM_RAM_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of pending message RAM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50104UL: // (TCM_REG_SM_CTX0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTCM_REG_SM_CTX0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of STORM context. LSB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50108UL: // (TCM_REG_AG_CTX0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTCM_REG_AG_CTX0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5010cUL: // (TCM_REG_AG_CTX1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTCM_REG_AG_CTX1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50110UL: // (TCM_REG_AG_CTX2_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTCM_REG_AG_CTX2_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50114UL: // (TCM_REG_GR_ARB_TYPE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_GR_ARB_TYPE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Arbitration between Input Arbiter groups: 0 - fair Round-\n    Robin; 1 - strict priority defined by ~tcm_registers_gr_ag_pr.gr_ag_pr\n    ; ~tcm_registers_gr_ld0_pr.gr_ld0_pr and ~tcm_registers_gr_ld1_pr.gr_ld1_pr\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50118UL: // (TCM_REG_GR_AG_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_GR_AG_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: AG channel group priority. The lowest priority is 0; the \n    highest priority is 3. It is supposed that the Store channel is the compliment\n     of the other 3 groups.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5011cUL: // (TCM_REG_GR_LD0_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_GR_LD0_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Load (FIC0) channel group priority. The lowest priority is\n     0; the highest priority is 3. It is supposed that the Store channel is\n     the compliment of the other 3 groups.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50120UL: // (TCM_REG_GR_LD1_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_GR_LD1_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Load (FIC1) channel group priority. The lowest priority is\n     0; the highest priority is 3. It is supposed that the Store channel is\n     the compliment of the other 3 groups.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50124UL: // (TCM_REG_WND_UPD_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_WND_UPD_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Queue index for WndUpdCntFlg.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50128UL: // (TCM_REG_TIMEOUT_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_TIMEOUT_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Queue index for TimeoutCntFlg.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5012cUL: // (TCM_REG_RXMIT_DEC_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_RXMIT_DEC_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Queue index for RxmitSeqDec and SndNxtDec (Rxmit decision\n     rules).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50130UL: // (TCM_REG_AUX1_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX1_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50134UL: // (TCM_REG_AUX2_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX2_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50138UL: // (TCM_REG_AUX3_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX3_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5013cUL: // (TCM_REG_AUX4_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX4_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50140UL: // (TCM_REG_AUX5_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX5_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50144UL: // (TCM_REG_AUX6_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX6_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50148UL: // (TCM_REG_AUX7_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX7_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5014cUL: // (TCM_REG_AUX8_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX8_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50150UL: // (TCM_REG_AUX9_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX9_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50154UL: // (TCM_REG_AUX10_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX10_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50158UL: // (TCM_REG_AUX11_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX11_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5015cUL: // (TCM_REG_AUX12_CNT_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_AUX12_CNT_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index to register to per each counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50160UL: // (TCM_REG_STORM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_STORM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Message length mismatch (relative to last indication) at \n    the STORM interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50164UL: // (TCM_REG_TSDM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_TSDM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Message length mismatch (relative to last indication) at \n    the SDM interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50168UL: // (TCM_REG_PRS_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_PRS_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Message length mismatch (relative to last indication) at \n    the In#6 interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5016cUL: // (TCM_REG_PBF_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_PBF_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Message length mismatch (relative to last indication) at \n    the In#7 interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50170UL: // (TCM_REG_USEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_USEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Message length mismatch (relative to last indication) at \n    the In#8 interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50174UL: // (TCM_REG_CSEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_CSEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Message length mismatch (relative to last indication) at \n    the In#9 interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50178UL: // (TCM_REG_XX_FREE: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTCM_REG_XX_FREE (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Use to read the value of XX protection Free counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5017cUL: // (TCM_REG_CAM_OCCUP: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_CAM_OCCUP (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: Used to read the value of the XX protection CAM occupancy\n     counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50180UL: // (TCM_REG_UNLOCK_MISS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_UNLOCK_MISS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when an error; indicating the LCID to be unlocked doesn\n    't exist in LCID CAM; happens. Is reset on read.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50184UL: // (TCM_REG_TQM_GLB_USE_CNTR: ACCESS:R  DataWidth:0x1a ) 
			sprintf(disp_str, "\nTCM_REG_TQM_GLB_USE_CNTR (ACCESS:R  DataWidth:0x1a )" \
				"\nDescription: Used to read the QM global usage counter - the counter of\n     the balance between QM requests sent and received by the TCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50188UL: // (TCM_REG_CP_BUF_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_CP_BUF_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: CP buffer is empty status.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5018cUL: // (TCM_REG_CP_BUF_STATUS: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTCM_REG_CP_BUF_STATUS (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: CP buffer status.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50190UL: // (TCM_REG_XX_OVFL_CNTR: ACCESS:ST  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTCM_REG_XX_OVFL_CNTR (ACCESS:ST  DataWidth:0x10 )" \
				"\nDescription: Counter of XX 0verflow occurencies.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50194UL: // (TCM_REG_STORM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_STORM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the STORM input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50198UL: // (TCM_REG_TSDM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_TSDM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input SDM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5019cUL: // (TCM_REG_PRS_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_PRS_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input prs.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501a0UL: // (TCM_REG_PBF_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_PBF_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input pbf.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501a4UL: // (TCM_REG_USEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_USEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input usem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501a8UL: // (TCM_REG_CSEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_CSEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input csem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501acUL: // (TCM_REG_CP_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_CP_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the CP input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501b0UL: // (TCM_REG_TQM_P_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_TQM_P_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the QM input (primary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501b4UL: // (TCM_REG_TQM_S_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_TQM_S_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the QM input (secondary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501b8UL: // (TCM_REG_TM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_TM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the Timers input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501bcUL: // (TCM_REG_STORM_OUT_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_STORM_OUT_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the output messages at FIC0 and FIC1 interfaces\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501c0UL: // (TCM_REG_TQM_OUT_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTCM_REG_TQM_OUT_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the output QM commands.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501c4UL: // (TCM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTCM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from TCM\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501c8UL: // (TCM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTCM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from TCM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select; before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501ccUL: // (TCM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTCM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from TCM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501d0UL: // (TCM_REG_TCM_INT_STS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nTCM_REG_TCM_INT_STS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_CP0_ERR) >> 8)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_CSEM_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_USEM_ERR) >> 6)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_PBF_ERR) >> 5)
				, "\n\tPRS_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_PRS_ERR) >> 4)
				, "\n\tTSDM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_TSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TCM_TCM_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x501d4UL: // (TCM_REG_TCM_INT_STS_CLR: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nTCM_REG_TCM_INT_STS_CLR (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_CP0_ERR) >> 8)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_CSEM_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_USEM_ERR) >> 6)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_PBF_ERR) >> 5)
				, "\n\tPRS_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_PRS_ERR) >> 4)
				, "\n\tTSDM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_TSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TCM_TCM_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x501d8UL: // (TCM_REG_TCM_INT_STS_WR: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nTCM_REG_TCM_INT_STS_WR (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_CP0_ERR) >> 8)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_CSEM_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_USEM_ERR) >> 6)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_PBF_ERR) >> 5)
				, "\n\tPRS_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_PRS_ERR) >> 4)
				, "\n\tTSDM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_TSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TCM_TCM_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x501dcUL: // (TCM_REG_TCM_INT_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nTCM_REG_TCM_INT_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_CP0_ERR) >> 8)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_CSEM_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_USEM_ERR) >> 6)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_PBF_ERR) >> 5)
				, "\n\tPRS_ERR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_PRS_ERR) >> 4)
				, "\n\tTSDM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_TSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TCM_TCM_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x501e0UL: // (TCM_REG_TCM_PRTY_STS: ACCESS:R  DataWidth:0x1b ) 
			sprintf(disp_str, "\nTCM_REG_TCM_PRTY_STS (ACCESS:R  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_CP0_PRTY) >> 24)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_CSEM_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_USEM_PRTY) >> 22)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_PBF_PRTY) >> 21)
				, "\n\tPRS_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_PRS_PRTY) >> 20)
				, "\n\tTSDM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_TSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x501e4UL: // (TCM_REG_TCM_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x1b ) 
			sprintf(disp_str, "\nTCM_REG_TCM_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_CP0_PRTY) >> 24)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_CSEM_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_USEM_PRTY) >> 22)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_PBF_PRTY) >> 21)
				, "\n\tPRS_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_PRS_PRTY) >> 20)
				, "\n\tTSDM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_TSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x501e8UL: // (TCM_REG_TCM_PRTY_STS_WR: ACCESS:WR  DataWidth:0x1b ) 
			sprintf(disp_str, "\nTCM_REG_TCM_PRTY_STS_WR (ACCESS:WR  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_CP0_PRTY) >> 24)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_CSEM_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_USEM_PRTY) >> 22)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_PBF_PRTY) >> 21)
				, "\n\tPRS_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_PRS_PRTY) >> 20)
				, "\n\tTSDM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_TSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TCM_TCM_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x501ecUL: // (TCM_REG_TCM_PRTY_MASK: ACCESS:RW  DataWidth:0x1b ) 
			sprintf(disp_str, "\nTCM_REG_TCM_PRTY_MASK (ACCESS:RW  DataWidth:0x1b )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_CP0_PRTY) >> 24)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_CSEM_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_USEM_PRTY) >> 22)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_PBF_PRTY) >> 21)
				, "\n\tPRS_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_PRS_PRTY) >> 20)
				, "\n\tTSDM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_TSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TCM_TCM_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x50300UL: // (TCM_REG_SM_CTX1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTCM_REG_SM_CTX1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of STORM context. MSB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50304UL: // (TCM_REG_IS_UM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_IS_UM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of UM input stage buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50308UL: // (TCM_REG_IS_STORM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_IS_STORM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of STORM input stage buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5030cUL: // (TCM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTCM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: chicken bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50310UL: // (TCM_REG_IS_PRS_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_IS_PRS_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of Parser input stage buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50314UL: // (TCM_REG_IS_USEM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTCM_REG_IS_USEM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of Parser input stage buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50318UL: // (TCM_REG_L1ST_PAGE_MODE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_L1ST_PAGE_MODE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: L1 Storm context page mode enable. If 0 - the whole context\n     of 256 LCIDs is visible. Possible only in legacy mode; looking only into\n     the old per-LCID addresses and not seeing the new per-LCID addresses. \n    If 1- page mode; when looking into all per-LCID addresses is possible \n    but for only one page of the whole context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5031cUL: // (TCM_REG_L1ST_PAGE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTCM_REG_L1ST_PAGE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: L1 Storm context page to access to. Browses between the windows\n    ; the size of which is defined by ~tcm_registers_l1st_page_size.l1st_page_size\n    . In E2 TCM it browses between 2 windows each of which contains context\n     for 256/2=128 LCIDs. Only bits from 7 down to log(~tcm_registers_l1st_page_size\n    .l1st_page_size;2) are meaningful. Meaning  for ~tcm_registers_l1st_page_size\n    .l1st_page_size=8h80; when 8h00 - LCIDs from 0 to 127 are visible; when\n     8h80 - LCIDs from 128 to 255. The SW can write exact start LCID into \n    the register and only its msb will be used to select the appropriate page\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50330UL: // (TCM_REG_UM_FIC1_FORCE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTCM_REG_UM_FIC1_FORCE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0-messages unlocked from Pending messages RAM go to the FIC\n     for which they were designated in input message; 1-messages unlocked \n    from Pending messages RAM are forced to FIC1 whether they were destined\n     to FIC0 or FIC1 in original message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500e0UL: // (TCM_REG_PHYS_QNUM0_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nTCM_REG_PHYS_QNUM0_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 0 per port index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500e4UL: // (TCM_REG_PHYS_QNUM0_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nTCM_REG_PHYS_QNUM0_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 0 per port index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500e8UL: // (TCM_REG_PHYS_QNUM1_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nTCM_REG_PHYS_QNUM1_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 1 per port index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500ecUL: // (TCM_REG_PHYS_QNUM1_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nTCM_REG_PHYS_QNUM1_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 1 per port index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500f0UL: // (TCM_REG_PHYS_QNUM2_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nTCM_REG_PHYS_QNUM2_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 2 per port index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500f4UL: // (TCM_REG_PHYS_QNUM2_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nTCM_REG_PHYS_QNUM2_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 2 per port index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500f8UL: // (TCM_REG_PHYS_QNUM3_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nTCM_REG_PHYS_QNUM3_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 3 per port index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x500fcUL: // (TCM_REG_PHYS_QNUM3_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nTCM_REG_PHYS_QNUM3_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 3 per port index.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501f0UL: // (TCM_REG_LCID_CAM_0: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nTCM_REG_LCID_CAM_0 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501f4UL: // (TCM_REG_LCID_CAM_1: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nTCM_REG_LCID_CAM_1 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501f8UL: // (TCM_REG_LCID_CAM_2: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nTCM_REG_LCID_CAM_2 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x501fcUL: // (TCM_REG_LCID_CAM_3: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nTCM_REG_LCID_CAM_3 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50200UL: // (TCM_REG_CAM_OCCUP_ST: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_CAM_OCCUP_ST (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CAM occupancy sticky status. The write to the register is\n     performed by the XX internal circuitry. In order to comply with the SW\n     scripts the write by GRC is also available for debug purposes.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50204UL: // (TCM_REG_CFC_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTCM_REG_CFC_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CFC output initial credit. Max credit available - 15.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 1 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50208UL: // (TCM_REG_CP_MSG: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTCM_REG_CP_MSG (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Use to write a CP message. Write only register.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5020cUL: // (TCM_REG_FIC0_INIT_CRD: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTCM_REG_FIC0_INIT_CRD (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: FIC0 output initial credit. Max credit available - 255.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 64 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50210UL: // (TCM_REG_FIC1_INIT_CRD: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTCM_REG_FIC1_INIT_CRD (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: FIC1 output initial credit. Max credit available - 255.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 64 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50214UL: // (TCM_REG_GLB_CNT_STICKY: ACCESS:RW  DataWidth:0x1a ) 
			sprintf(disp_str, "\nTCM_REG_GLB_CNT_STICKY (ACCESS:RW  DataWidth:0x1a )" \
				"\nDescription: QM global usage counter maximum sticky value.  In order to\n     comply with the SW scripts the write by GRC is also available for debug\n     purposes.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50218UL: // (TCM_REG_LL_SZ_STICKY: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTCM_REG_LL_SZ_STICKY (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: XX LL maximum value ever reached sticky value for any connection\n    . The write to the register is performed by the XX internal circuitry. \n    In order to comply with the SW scripts the write by GRC is also available\n     for debug purposes.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5021cUL: // (TCM_REG_TQM_INIT_CRD: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTCM_REG_TQM_INIT_CRD (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: QM output initial credit. Max credit available - 32.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 32 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50220UL: // (TCM_REG_XX_INIT_CRD: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTCM_REG_XX_INIT_CRD (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Initial value for the credit counter; responsible for fulfilling\n     of the Input Stage XX protection buffer by the XX protection pending \n    messages. Max credit available - 127.Write writes the initial credit value\n    ; read returns the current value of the credit counter. Must be initialized\n     to 19 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50224UL: // (TCM_REG_XX_MSG_NUM: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTCM_REG_XX_MSG_NUM (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The maximum number of pending messages; which may be stored\n     in XX protection. ~tcm_registers_xx_free.xx_free is also written on write\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50228UL: // (TCM_REG_LCID_CAM_4: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nTCM_REG_LCID_CAM_4 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5022cUL: // (TCM_REG_LCID_CAM_5: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nTCM_REG_LCID_CAM_5 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50230UL: // (TCM_REG_LCID_CAM_6: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nTCM_REG_LCID_CAM_6 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50234UL: // (TCM_REG_LCID_CAM_7: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nTCM_REG_LCID_CAM_7 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50238UL: // (TCM_REG_LCID_CAM_8: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nTCM_REG_LCID_CAM_8 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x5023cUL: // (TCM_REG_LCID_CAM_9: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nTCM_REG_LCID_CAM_9 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50240UL: // (TCM_REG_XX_TABLE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTCM_REG_XX_TABLE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Indirect access to the XX table of the XX protection mechanism\n    . The fields are:[4:0] - tail pointer; [10:5] - Link List size;[15:11] - \n    header pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50280UL: // (TCM_REG_XX_DESCR_TABLE: ACCESS:RW  DataWidth:0x15 ) 
			sprintf(disp_str, "\nTCM_REG_XX_DESCR_TABLE (ACCESS:RW  DataWidth:0x15 )" \
				"\nDescription: Indirect access to the descriptor table of the XX protection\n     mechanism. The fields are: [5:0] - length of the message;[15:6] - message\n     pointer;[20:16] - next pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x52000UL: // (TCM_REG_XX_PEND_MSG: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTCM_REG_XX_PEND_MSG (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Debug only. Indirect access to the Pending messages RAM of\n     the XX protection mechanism. Bit [0] designates the REG in the 2-REGs\n     memory row. The bits [10:1] designate the row.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x54000UL: // (TCM_REG_AG_CTX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTCM_REG_AG_CTX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indirect access to AG context with 32-bits granularity. The\n     bits [11:8] of the address should be the offset within the accessed LCID\n     context; the bits [7:0] are the accessed LCID.Example: to write to REG10\n     LCID100. The RBC address should be 12'ha64.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x58000UL: // (TCM_REG_STORM_CTX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTCM_REG_STORM_CTX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indirect access to STORM context with 32-bits granularity\n    . In page mode the bits [12:7] of the address should be the offset within\n     the accessed LCID context; the bits [6:0] are the accessed LCID[6:0]. \n    In non-page mode the bits [12:8] of the address should be the offset within\n     the accessed LCID context; the bits [7:0] are the accessed LCID.Example\n    : to write to REG10 LCID100. The RBC address should be 13'ha64. The actual\n     size of the memory is 9216.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50328UL: // (TCM_REG_TCM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTCM_REG_TCM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x50334UL: // (TCM_REG_TCM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTCM_REG_TCM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164000UL: // (TM_REG_EN_TIMERS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_EN_TIMERS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for Timers state machines.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164004UL: // (TM_REG_TIMER_SOFT_RST: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_TIMER_SOFT_RST (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Timer software reset - active high.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164008UL: // (TM_REG_EN_CL0_INPUT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_EN_CL0_INPUT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable client0 input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16400cUL: // (TM_REG_EN_CL1_INPUT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_EN_CL1_INPUT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable client1 input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164010UL: // (TM_REG_EN_CL2_INPUT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_EN_CL2_INPUT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable client2 input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164020UL: // (TM_REG_CLIN_ARB_TYPE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_CLIN_ARB_TYPE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Clin arbiter type  0=roundrobbin 1=priority.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164024UL: // (TM_REG_CLIN_PRIOR0_CLIENT: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTM_REG_CLIN_PRIOR0_CLIENT (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Client in High priority client number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164028UL: // (TM_REG_CLIN_PRIOR1_CLIENT: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTM_REG_CLIN_PRIOR1_CLIENT (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Client in Medium priority client number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16402cUL: // (TM_REG_CLIN_PRIOR2_CLIENT: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTM_REG_CLIN_PRIOR2_CLIENT (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Client in Low  priority client number.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164030UL: // (TM_REG_CL0_CONT_REGION: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTM_REG_CL0_CONT_REGION (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Client0 context region.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164034UL: // (TM_REG_CL1_CONT_REGION: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTM_REG_CL1_CONT_REGION (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Client1 context region.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164038UL: // (TM_REG_CL2_CONT_REGION: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTM_REG_CL2_CONT_REGION (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Client2 context region.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16403cUL: // (TM_REG_LIN0_SCAN_TIME: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_LIN0_SCAN_TIME (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: Linear0 array scan timeout.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164040UL: // (TM_REG_LIN1_SCAN_TIME: ACCESS:RW  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_LIN1_SCAN_TIME (ACCESS:RW  DataWidth:0x18 )" \
				"\nDescription: Linear1 array scan timeout.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164044UL: // (TM_REG_TM_CONTEXT_REGION: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTM_REG_TM_CONTEXT_REGION (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Timers Context region.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164048UL: // (TM_REG_LIN0_MAX_ACTIVE_CID: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nTM_REG_LIN0_MAX_ACTIVE_CID (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: Linear0 Max active cid (in banks of 32 entries).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16404cUL: // (TM_REG_LIN1_MAX_ACTIVE_CID: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nTM_REG_LIN1_MAX_ACTIVE_CID (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: Linear1 Max active cid (in banks of 32 entries).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164050UL: // (TM_REG_LIN0_MIN_ACTIVE_CID: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nTM_REG_LIN0_MIN_ACTIVE_CID (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: Linear0 Min active cid (in banks of 32 entries).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164054UL: // (TM_REG_LIN1_MIN_ACTIVE_CID: ACCESS:RW  DataWidth:0x12 ) 
			sprintf(disp_str, "\nTM_REG_LIN1_MIN_ACTIVE_CID (ACCESS:RW  DataWidth:0x12 )" \
				"\nDescription: Linear1 Min active cid (in banks of 32 entries).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164058UL: // (TM_REG_PCI_NS_FLAG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_PCI_NS_FLAG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: NS flag for pci requests.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16405cUL: // (TM_REG_PCI_RO_FLAG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_PCI_RO_FLAG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: RO flag for pci requests.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164060UL: // (TM_REG_PCI_FLOW_ID: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTM_REG_PCI_FLOW_ID (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Pci flow ID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164064UL: // (TM_REG_FALSE_EXPIR_CLIENT: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTM_REG_FALSE_EXPIR_CLIENT (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Client number for false expirations.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164068UL: // (TM_REG_LIN0_LOGIC_ADD_MASK: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTM_REG_LIN0_LOGIC_ADD_MASK (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Linear0 Address mask.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16406cUL: // (TM_REG_LIN1_LOGIC_ADD_MASK: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTM_REG_LIN1_LOGIC_ADD_MASK (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Linear1 Address mask.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164070UL: // (TM_REG_LIN_SETCLR_FIFO_ALFULL_THR: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTM_REG_LIN_SETCLR_FIFO_ALFULL_THR (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Linear timer set_clear fifo threshold.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164074UL: // (TM_REG_LIN_EXP_FIFO_ALFULL_THR: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTM_REG_LIN_EXP_FIFO_ALFULL_THR (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Linear expiration fifo threshold.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164078UL: // (TM_REG_LIN_EXTA_FIFO_ALFULL_THR: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_LIN_EXTA_FIFO_ALFULL_THR (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Linear exta fifo threshold.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16407cUL: // (TM_REG_EXP_CFCRES_FIFO_ALFULL_THR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTM_REG_EXP_CFCRES_FIFO_ALFULL_THR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Expiration cfc result fifo threshold.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164080UL: // (TM_REG_CFCREQ_AC_FIFO_ALFULL_THR: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_CFCREQ_AC_FIFO_ALFULL_THR (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CFC Request - Act cnt fifo threshold.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164084UL: // (TM_REG_CFCREQ_EXP_FIFO_ALFULL_THR: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_CFCREQ_EXP_FIFO_ALFULL_THR (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CFC req - expiration fifo threshold.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164088UL: // (TM_REG_CFCREQ_CLOUT_FIFO_ALFULL_THR: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_CFCREQ_CLOUT_FIFO_ALFULL_THR (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CFC req - clout fifo threshold.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16408cUL: // (TM_REG_CLOUT_DIRECT_FIFO_ALFULL_THR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTM_REG_CLOUT_DIRECT_FIFO_ALFULL_THR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Client Out - direct fifo threshold. FIFO is not used.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164090UL: // (TM_REG_CLOUT_TYP4_FIFO_ALFULL_THR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTM_REG_CLOUT_TYP4_FIFO_ALFULL_THR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Client Out type4 fifo threshold.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164094UL: // (TM_REG_SET_ERR_FLAG: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_SET_ERR_FLAG (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set error flag from cmd handler.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164098UL: // (TM_REG_CLEAR_ERR_FLAG: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_CLEAR_ERR_FLAG (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Clear error flag from cmd handler.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16409cUL: // (TM_REG_STOPALL_ERR_FLAG: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_STOPALL_ERR_FLAG (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Stopall error flag from cmd handler.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640a0UL: // (TM_REG_LIN0_NUM_SCANS: ACCESS:ST  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTM_REG_LIN0_NUM_SCANS (ACCESS:ST  DataWidth:0x10 )" \
				"\nDescription: Linear0 Number of scans counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640a4UL: // (TM_REG_LIN1_NUM_SCANS: ACCESS:ST  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTM_REG_LIN1_NUM_SCANS (ACCESS:ST  DataWidth:0x10 )" \
				"\nDescription: Linear1 Number of scans counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640a8UL: // (TM_REG_CLIENT0_EXPIR_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_CLIENT0_EXPIR_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Client0 Expiration counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640acUL: // (TM_REG_CLIENT1_EXPIR_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_CLIENT1_EXPIR_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Client1 Expiration counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640b0UL: // (TM_REG_CLIENT2_EXPIR_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_CLIENT2_EXPIR_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Client2 Expiration counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640b4UL: // (TM_REG_CLIENT0_REWIND_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_CLIENT0_REWIND_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Client0 Rewind counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640b8UL: // (TM_REG_CLIENT1_REWIND_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_CLIENT1_REWIND_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Client1 Rewind counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640bcUL: // (TM_REG_CLIENT2_REWIND_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_CLIENT2_REWIND_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Client2 Rewind counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640c0UL: // (TM_REG_CLIENT_ALL_REWIND_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_CLIENT_ALL_REWIND_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: All clients rewind counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640c4UL: // (TM_REG_CLIENT_ALL_CLEAR_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_CLIENT_ALL_CLEAR_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: All clients clear counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640c8UL: // (TM_REG_CLIENT_ALL_SET_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_CLIENT_ALL_SET_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: All clients set  counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640ccUL: // (TM_REG_CLIENT_ALL_STOPALL_CNT: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTM_REG_CLIENT_ALL_STOPALL_CNT (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: All clients stopall  counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640d8UL: // (TM_REG_EN_REAL_TIME_CNT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_EN_REAL_TIME_CNT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable real time counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640dcUL: // (TM_REG_CFCREQ_ECHO_FIFO_ALFULL_THR: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTM_REG_CFCREQ_ECHO_FIFO_ALFULL_THR (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Cfcreq echo fifofull  threshold.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640e0UL: // (TM_REG_TM_L1_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTM_REG_TM_L1_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM Bits of context ram0 Memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640e4UL: // (TM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from TM\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640e8UL: // (TM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from TM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640ecUL: // (TM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only: For dbgmux usage (debug data that goes from TM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640f0UL: // (TM_REG_TM_INT_STS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_TM_INT_STS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TM_TM_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1640f4UL: // (TM_REG_TM_INT_STS_CLR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_TM_INT_STS_CLR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TM_TM_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1640f8UL: // (TM_REG_TM_INT_STS_WR: ACCESS:WR  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_TM_INT_STS_WR (ACCESS:WR  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TM_TM_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1640fcUL: // (TM_REG_TM_INT_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_TM_INT_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TM_TM_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x164100UL: // (TM_REG_TM_PRTY_STS: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nTM_REG_TM_PRTY_STS (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTU_SETCLR1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_REG_TU_SETCLR1_PARITY) >> 6)
				, "\n\tTU_SETCLR0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_REG_TU_SETCLR0_PARITY) >> 5)
				, "\n\tTU_BANK1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_REG_TU_BANK1_PARITY) >> 4)
				, "\n\tTU_BANK0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_REG_TU_BANK0_PARITY) >> 3)
				, "\n\tTM_L1_1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_REG_TM_L1_1_PARITY) >> 2)
				, "\n\tTM_L1_0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_REG_TM_L1_0_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x164104UL: // (TM_REG_TM_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x7 ) 
			sprintf(disp_str, "\nTM_REG_TM_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x7 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTU_SETCLR1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_CLR_REG_TU_SETCLR1_PARITY) >> 6)
				, "\n\tTU_SETCLR0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_CLR_REG_TU_SETCLR0_PARITY) >> 5)
				, "\n\tTU_BANK1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_CLR_REG_TU_BANK1_PARITY) >> 4)
				, "\n\tTU_BANK0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_CLR_REG_TU_BANK0_PARITY) >> 3)
				, "\n\tTM_L1_1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_CLR_REG_TM_L1_1_PARITY) >> 2)
				, "\n\tTM_L1_0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_CLR_REG_TM_L1_0_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x164108UL: // (TM_REG_TM_PRTY_STS_WR: ACCESS:WR  DataWidth:0x7 ) 
			sprintf(disp_str, "\nTM_REG_TM_PRTY_STS_WR (ACCESS:WR  DataWidth:0x7 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTU_SETCLR1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_WR_REG_TU_SETCLR1_PARITY) >> 6)
				, "\n\tTU_SETCLR0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_WR_REG_TU_SETCLR0_PARITY) >> 5)
				, "\n\tTU_BANK1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_WR_REG_TU_BANK1_PARITY) >> 4)
				, "\n\tTU_BANK0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_WR_REG_TU_BANK0_PARITY) >> 3)
				, "\n\tTM_L1_1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_WR_REG_TM_L1_1_PARITY) >> 2)
				, "\n\tTM_L1_0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_WR_REG_TM_L1_0_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TM_TM_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x16410cUL: // (TM_REG_TM_PRTY_MASK: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nTM_REG_TM_PRTY_MASK (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTU_SETCLR1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_MASK_REG_TU_SETCLR1_PARITY) >> 6)
				, "\n\tTU_SETCLR0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_MASK_REG_TU_SETCLR0_PARITY) >> 5)
				, "\n\tTU_BANK1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_MASK_REG_TU_BANK1_PARITY) >> 4)
				, "\n\tTU_BANK0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_MASK_REG_TU_BANK0_PARITY) >> 3)
				, "\n\tTM_L1_1_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_MASK_REG_TM_L1_1_PARITY) >> 2)
				, "\n\tTM_L1_0_PARITY :"
				, (u32_t)((data & TM_TM_PRTY_MASK_REG_TM_L1_0_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TM_TM_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x164110UL: // (TM_REG_EN_PHY0_ADDR_CACHE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_EN_PHY0_ADDR_CACHE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable physical addres caching Port0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164114UL: // (TM_REG_EN_PHY1_ADDR_CACHE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_EN_PHY1_ADDR_CACHE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable physical addres caching Port1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164118UL: // (TM_REG_TM_L1_1_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTM_REG_TM_L1_1_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM Bits of context ram1 Memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16411cUL: // (TM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: chicken bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164120UL: // (TM_REG_SETCLR0_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTM_REG_SETCLR0_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM Bits of setclr fifo0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164124UL: // (TM_REG_SETCLR1_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTM_REG_SETCLR1_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM Bits of setclr fifo1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164134UL: // (TM_REG_ATC_PAGE_1ST_BNK_RD: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTM_REG_ATC_PAGE_1ST_BNK_RD (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: ATC flag for reading first bank in a page; bit2: 0->low priority\n     1->high priority; bit[1:0]: 00->do nothing 01->search only 10->search\n     & cache 11->search & release\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164138UL: // (TM_REG_ATC_PAGE_1ST_BNK_WR: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTM_REG_ATC_PAGE_1ST_BNK_WR (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: ATC flag for writing first bank in a page; bit2: 0->low priority\n     1->high priority; bit[1:0]: 00->do nothing 01->search only 10->search\n     & cache 11->search & release\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16413cUL: // (TM_REG_ATC_PAGE_LAS_BNK_RD: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTM_REG_ATC_PAGE_LAS_BNK_RD (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: ATC flag for reading last bank in a page; bit2: 0->low priority\n     1->high priority; bit[1:0]: 00->do nothing 01->search only 10->search\n     & cache 11->search & release\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164140UL: // (TM_REG_ATC_PAGE_LAS_BNK_WR: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTM_REG_ATC_PAGE_LAS_BNK_WR (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: ATC flag for writing last bank in a page; bit2: 0->low priority\n     1->high priority; bit[1:0]: 00->do nothing 01->search only 10->search\n     & cache 11->search & release\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164144UL: // (TM_REG_ATC_PAGE_MID_BNK_RD: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTM_REG_ATC_PAGE_MID_BNK_RD (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: ATC flag for reading middle banks in a page; bit2: 0->low\n     priority 1->high priority; bit[1:0]: 00->do nothing 01->search only 10\n    ->search & cache 11->search & release\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164148UL: // (TM_REG_ATC_PAGE_MID_BNK_WR: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTM_REG_ATC_PAGE_MID_BNK_WR (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: ATC flag for writing middle banks in a page; bit2: 0->low\n     priority 1->high priority; bit[1:0]: 00->do nothing 01->search only 10\n    ->search & cache 11->search & release\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16414cUL: // (TM_REG_ATC_SET_CLR_WR: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTM_REG_ATC_SET_CLR_WR (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: ATC flag for writes from set or clear (new) commands; bit2\n    : 0->low priority 1->high priority; bit[1:0]: 00->do nothing 01->search\n     only 10->search & cache 11->search & release\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164150UL: // (TM_REG_AC_UPD_CHANGE_DIS: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_AC_UPD_CHANGE_DIS (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables ac updt change; ac updt change prevents cmd handler\n     from pre-incr ac for expiration msg clients &  incr ac as part of client\n     out cfc load request\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164014UL: // (TM_REG_EN_LINEAR0_TIMER: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nTM_REG_EN_LINEAR0_TIMER (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Enable Linear Engine Port0; function 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164018UL: // (TM_REG_EN_LINEAR1_TIMER: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nTM_REG_EN_LINEAR1_TIMER (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Enable Linear Engine Port1; function 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640d0UL: // (TM_REG_LIN0_SCAN_ON: ACCESS:R  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nTM_REG_LIN0_SCAN_ON (ACCESS:R  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Scan-on sig from lin engine 0. Active during bank-rd; scan\n    ; bank-wr; expr cfc load.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1640d4UL: // (TM_REG_LIN1_SCAN_ON: ACCESS:R  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nTM_REG_LIN1_SCAN_ON (ACCESS:R  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: Scan-on sig from lin engine 1. Active during bank-rd; scan\n    ; bank-wr; expr cfc load.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164128UL: // (TM_REG_LIN0_VNIC_UC: ACCESS:R  DataWidth:0x9  SPLIT:4 ) 
			sprintf(disp_str, "\nTM_REG_LIN0_VNIC_UC (ACCESS:R  DataWidth:0x9  SPLIT:4 )" \
				"\nDescription: vnic usage counter value for port 0 accurate only after bank\n     write response is received.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16412cUL: // (TM_REG_LIN1_VNIC_UC: ACCESS:R  DataWidth:0x9  SPLIT:4 ) 
			sprintf(disp_str, "\nTM_REG_LIN1_VNIC_UC (ACCESS:R  DataWidth:0x9  SPLIT:4 )" \
				"\nDescription: vnic usage counter value for port 1 accurate only after bank\n     write response is received.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164200UL: // (TM_REG_CURRENT_TIME: ACCESS:R  DataWidth:0x1e ) 
			sprintf(disp_str, "\nTM_REG_CURRENT_TIME (ACCESS:R  DataWidth:0x1e )" \
				"\nDescription: Current Time read from Command Handler counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164208UL: // (TM_REG_CFC_AC_CRDCNT_VAL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_CFC_AC_CRDCNT_VAL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Load value for for cfc ac credit cnt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164210UL: // (TM_REG_CFC_CLD_CRDCNT_VAL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_CFC_CLD_CRDCNT_VAL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Load value for cfc cld credit cnt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164218UL: // (TM_REG_CFC_LOADREQ_MAX_NUM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_CFC_LOADREQ_MAX_NUM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CFC max number of outstanding  load requests for clients \n    context loading.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164220UL: // (TM_REG_CLOUT_CRDCNT0_VAL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_CLOUT_CRDCNT0_VAL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Load value for clout0 cred cnt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164228UL: // (TM_REG_CLOUT_CRDCNT1_VAL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_CLOUT_CRDCNT1_VAL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Load value for clout1 cred cnt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164230UL: // (TM_REG_CLOUT_CRDCNT2_VAL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_CLOUT_CRDCNT2_VAL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Load value for clout2 cred cnt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164238UL: // (TM_REG_EXP_CRDCNT_VAL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTM_REG_EXP_CRDCNT_VAL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Load value for expiration credit cnt. CFC max number of outstanding\n      load requests for timers (expiration) context loading.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164240UL: // (TM_REG_LIN0_LOGIC_ADDR: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTM_REG_LIN0_LOGIC_ADDR (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Linear0 logic address.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164248UL: // (TM_REG_LIN0_PHY_ADDR_VALID: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_LIN0_PHY_ADDR_VALID (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Linear0 physical address valid.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164250UL: // (TM_REG_LIN1_LOGIC_ADDR: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTM_REG_LIN1_LOGIC_ADDR (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Linear1 logic address.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164258UL: // (TM_REG_LIN1_PHY_ADDR_VALID: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTM_REG_LIN1_PHY_ADDR_VALID (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Linear1 physical address valid.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164260UL: // (TM_REG_PCIARB_CRDCNT_VAL: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTM_REG_PCIARB_CRDCNT_VAL (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Load value for pci arbiter credit cnt.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164270UL: // (TM_REG_LIN0_PHY_ADDR: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nTM_REG_LIN0_PHY_ADDR (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Linear0 phy address.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164280UL: // (TM_REG_LIN1_PHY_ADDR: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nTM_REG_LIN1_PHY_ADDR (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Linear1 phy address.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164800UL: // (TM_REG_CONTEXT_RAM0: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nTM_REG_CONTEXT_RAM0 (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Context ram0 wide bus read-write.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x165000UL: // (TM_REG_CONTEXT_RAM1: ACCESS:WB  DataWidth:0x23 ) 
			sprintf(disp_str, "\nTM_REG_CONTEXT_RAM1 (ACCESS:WB  DataWidth:0x23 )" \
				"\nDescription: Context ram1 wide bus read-write.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164154UL: // (TM_REG_TM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTM_REG_TM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164204UL: // (TM_REG_TM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTM_REG_TM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164268UL: // (TM_REG_TM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTM_REG_TM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x164290UL: // (TM_REG_TM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTM_REG_TM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x165800UL: // (TM_REG_TM_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTM_REG_TM_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42000UL: // (TSDM_REG_TIMER_TICK: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_TIMER_TICK (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Tick for timer counter. Applicable only when ~tsdm_registers_timer_tick_enable\n    .timer_tick_enable =1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42004UL: // (TSDM_REG_TIMERS_TICK_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_TIMERS_TICK_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for tick counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42008UL: // (TSDM_REG_CFC_RSP_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nTSDM_REG_CFC_RSP_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the  internal RAM for the cfc_rsp lcid\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4200cUL: // (TSDM_REG_CMP_COUNTER_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nTSDM_REG_CMP_COUNTER_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal  RAM for the completion\n     counters.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42010UL: // (TSDM_REG_Q_COUNTER_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nTSDM_REG_Q_COUNTER_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal RAM for queue counters\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42014UL: // (TSDM_REG_PCK_END_MSG_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nTSDM_REG_PCK_END_MSG_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal RAM for the packet end \n    message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42018UL: // (TSDM_REG_COUNTERS_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_COUNTERS_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates if the 204 statistics counters should stop counting\n     when reaching an all-ones value or should wrap-around 0=stop counting\n     1=wrap-around.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4201cUL: // (TSDM_REG_CMP_COUNTER_MAX0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTSDM_REG_CMP_COUNTER_MAX0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42020UL: // (TSDM_REG_CMP_COUNTER_MAX1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTSDM_REG_CMP_COUNTER_MAX1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42024UL: // (TSDM_REG_CMP_COUNTER_MAX2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTSDM_REG_CMP_COUNTER_MAX2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42028UL: // (TSDM_REG_CMP_COUNTER_MAX3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTSDM_REG_CMP_COUNTER_MAX3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4202cUL: // (TSDM_REG_BRB1_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTSDM_REG_BRB1_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from BRB1 in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42030UL: // (TSDM_REG_PXP_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTSDM_REG_PXP_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from pxp in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42034UL: // (TSDM_REG_PB_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTSDM_REG_PB_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from PB in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42038UL: // (TSDM_REG_AGG_INT_EVENT_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4203cUL: // (TSDM_REG_AGG_INT_EVENT_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42040UL: // (TSDM_REG_AGG_INT_EVENT_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42044UL: // (TSDM_REG_AGG_INT_EVENT_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42048UL: // (TSDM_REG_AGG_INT_EVENT_4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4204cUL: // (TSDM_REG_AGG_INT_EVENT_5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42050UL: // (TSDM_REG_AGG_INT_EVENT_6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42054UL: // (TSDM_REG_AGG_INT_EVENT_7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42058UL: // (TSDM_REG_AGG_INT_EVENT_8: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_8 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4205cUL: // (TSDM_REG_AGG_INT_EVENT_9: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_9 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42060UL: // (TSDM_REG_AGG_INT_EVENT_10: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_10 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42064UL: // (TSDM_REG_AGG_INT_EVENT_11: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_11 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42068UL: // (TSDM_REG_AGG_INT_EVENT_12: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_12 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4206cUL: // (TSDM_REG_AGG_INT_EVENT_13: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_13 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42070UL: // (TSDM_REG_AGG_INT_EVENT_14: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_14 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42074UL: // (TSDM_REG_AGG_INT_EVENT_15: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_15 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42078UL: // (TSDM_REG_AGG_INT_EVENT_16: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_16 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4207cUL: // (TSDM_REG_AGG_INT_EVENT_17: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_17 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42080UL: // (TSDM_REG_AGG_INT_EVENT_18: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_18 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42084UL: // (TSDM_REG_AGG_INT_EVENT_19: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_19 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42088UL: // (TSDM_REG_AGG_INT_EVENT_20: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_20 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4208cUL: // (TSDM_REG_AGG_INT_EVENT_21: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_21 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42090UL: // (TSDM_REG_AGG_INT_EVENT_22: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_22 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42094UL: // (TSDM_REG_AGG_INT_EVENT_23: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_23 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42098UL: // (TSDM_REG_AGG_INT_EVENT_24: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_24 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4209cUL: // (TSDM_REG_AGG_INT_EVENT_25: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_25 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420a0UL: // (TSDM_REG_AGG_INT_EVENT_26: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_26 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420a4UL: // (TSDM_REG_AGG_INT_EVENT_27: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_27 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420a8UL: // (TSDM_REG_AGG_INT_EVENT_28: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_28 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420acUL: // (TSDM_REG_AGG_INT_EVENT_29: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_29 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420b0UL: // (TSDM_REG_AGG_INT_EVENT_30: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_30 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420b4UL: // (TSDM_REG_AGG_INT_EVENT_31: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_EVENT_31 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420b8UL: // (TSDM_REG_AGG_INT_T_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420bcUL: // (TSDM_REG_AGG_INT_T_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420c0UL: // (TSDM_REG_AGG_INT_T_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420c4UL: // (TSDM_REG_AGG_INT_T_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420c8UL: // (TSDM_REG_AGG_INT_T_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420ccUL: // (TSDM_REG_AGG_INT_T_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420d0UL: // (TSDM_REG_AGG_INT_T_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420d4UL: // (TSDM_REG_AGG_INT_T_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420d8UL: // (TSDM_REG_AGG_INT_T_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420dcUL: // (TSDM_REG_AGG_INT_T_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420e0UL: // (TSDM_REG_AGG_INT_T_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420e4UL: // (TSDM_REG_AGG_INT_T_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420e8UL: // (TSDM_REG_AGG_INT_T_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420ecUL: // (TSDM_REG_AGG_INT_T_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420f0UL: // (TSDM_REG_AGG_INT_T_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420f4UL: // (TSDM_REG_AGG_INT_T_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420f8UL: // (TSDM_REG_AGG_INT_T_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x420fcUL: // (TSDM_REG_AGG_INT_T_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42100UL: // (TSDM_REG_AGG_INT_T_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42104UL: // (TSDM_REG_AGG_INT_T_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42108UL: // (TSDM_REG_AGG_INT_T_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4210cUL: // (TSDM_REG_AGG_INT_T_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42110UL: // (TSDM_REG_AGG_INT_T_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42114UL: // (TSDM_REG_AGG_INT_T_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42118UL: // (TSDM_REG_AGG_INT_T_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4211cUL: // (TSDM_REG_AGG_INT_T_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42120UL: // (TSDM_REG_AGG_INT_T_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42124UL: // (TSDM_REG_AGG_INT_T_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42128UL: // (TSDM_REG_AGG_INT_T_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4212cUL: // (TSDM_REG_AGG_INT_T_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42130UL: // (TSDM_REG_AGG_INT_T_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42134UL: // (TSDM_REG_AGG_INT_T_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_T_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42138UL: // (TSDM_REG_AGG_INT_FIC_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4213cUL: // (TSDM_REG_AGG_INT_FIC_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42140UL: // (TSDM_REG_AGG_INT_FIC_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42144UL: // (TSDM_REG_AGG_INT_FIC_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42148UL: // (TSDM_REG_AGG_INT_FIC_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4214cUL: // (TSDM_REG_AGG_INT_FIC_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42150UL: // (TSDM_REG_AGG_INT_FIC_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42154UL: // (TSDM_REG_AGG_INT_FIC_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42158UL: // (TSDM_REG_AGG_INT_FIC_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4215cUL: // (TSDM_REG_AGG_INT_FIC_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42160UL: // (TSDM_REG_AGG_INT_FIC_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42164UL: // (TSDM_REG_AGG_INT_FIC_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42168UL: // (TSDM_REG_AGG_INT_FIC_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4216cUL: // (TSDM_REG_AGG_INT_FIC_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42170UL: // (TSDM_REG_AGG_INT_FIC_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42174UL: // (TSDM_REG_AGG_INT_FIC_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42178UL: // (TSDM_REG_AGG_INT_FIC_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4217cUL: // (TSDM_REG_AGG_INT_FIC_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42180UL: // (TSDM_REG_AGG_INT_FIC_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42184UL: // (TSDM_REG_AGG_INT_FIC_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42188UL: // (TSDM_REG_AGG_INT_FIC_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4218cUL: // (TSDM_REG_AGG_INT_FIC_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42190UL: // (TSDM_REG_AGG_INT_FIC_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42194UL: // (TSDM_REG_AGG_INT_FIC_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42198UL: // (TSDM_REG_AGG_INT_FIC_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4219cUL: // (TSDM_REG_AGG_INT_FIC_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421a0UL: // (TSDM_REG_AGG_INT_FIC_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421a4UL: // (TSDM_REG_AGG_INT_FIC_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421a8UL: // (TSDM_REG_AGG_INT_FIC_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421acUL: // (TSDM_REG_AGG_INT_FIC_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421b0UL: // (TSDM_REG_AGG_INT_FIC_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421b4UL: // (TSDM_REG_AGG_INT_FIC_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_FIC_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421b8UL: // (TSDM_REG_AGG_INT_MODE_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421bcUL: // (TSDM_REG_AGG_INT_MODE_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421c0UL: // (TSDM_REG_AGG_INT_MODE_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421c4UL: // (TSDM_REG_AGG_INT_MODE_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421c8UL: // (TSDM_REG_AGG_INT_MODE_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421ccUL: // (TSDM_REG_AGG_INT_MODE_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421d0UL: // (TSDM_REG_AGG_INT_MODE_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421d4UL: // (TSDM_REG_AGG_INT_MODE_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421d8UL: // (TSDM_REG_AGG_INT_MODE_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421dcUL: // (TSDM_REG_AGG_INT_MODE_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421e0UL: // (TSDM_REG_AGG_INT_MODE_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421e4UL: // (TSDM_REG_AGG_INT_MODE_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421e8UL: // (TSDM_REG_AGG_INT_MODE_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421ecUL: // (TSDM_REG_AGG_INT_MODE_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421f0UL: // (TSDM_REG_AGG_INT_MODE_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421f4UL: // (TSDM_REG_AGG_INT_MODE_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421f8UL: // (TSDM_REG_AGG_INT_MODE_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (16) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x421fcUL: // (TSDM_REG_AGG_INT_MODE_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (17)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42200UL: // (TSDM_REG_AGG_INT_MODE_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42204UL: // (TSDM_REG_AGG_INT_MODE_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42208UL: // (TSDM_REG_AGG_INT_MODE_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4220cUL: // (TSDM_REG_AGG_INT_MODE_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42210UL: // (TSDM_REG_AGG_INT_MODE_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42214UL: // (TSDM_REG_AGG_INT_MODE_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42218UL: // (TSDM_REG_AGG_INT_MODE_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4221cUL: // (TSDM_REG_AGG_INT_MODE_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42220UL: // (TSDM_REG_AGG_INT_MODE_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42224UL: // (TSDM_REG_AGG_INT_MODE_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42228UL: // (TSDM_REG_AGG_INT_MODE_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4222cUL: // (TSDM_REG_AGG_INT_MODE_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42230UL: // (TSDM_REG_AGG_INT_MODE_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42234UL: // (TSDM_REG_AGG_INT_MODE_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_AGG_INT_MODE_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42238UL: // (TSDM_REG_ENABLE_IN1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nTSDM_REG_ENABLE_IN1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tQM_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_QM_EXT_WR_FULL_IN_EN) >> 26)
				, "\n\tCFC_PB_ACK_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_CFC_PB_ACK_IN_EN) >> 25)
				, "\n\tCFC_ACDEC_ACK_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_CFC_ACDEC_ACK_IN_EN) >> 24)
				, "\n\tCFC_ACINC_ACK_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_CFC_ACINC_ACK_IN_EN) >> 23)
				, "\n\tCFC_LOAD_RSP_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_CFC_LOAD_RSP_IN_EN) >> 22)
				, "\n\tCFC_LOAD_ACK_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_CFC_LOAD_ACK_IN_EN) >> 21)
				, "\n\tPXP_REQ_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PXP_REQ_IN_EN) >> 20)
				, "\n\tSDM_WAKE_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_SDM_WAKE_IN_EN) >> 19)
				, "\n\tPRS_MSG_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PRS_MSG_IN_EN) >> 18)
				, "\n\tPB_DATA_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PB_DATA_IN_EN) >> 17)
				, "\n\tBRB1_DP_DATA_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_BRB1_DP_DATA_IN_EN) >> 16)
				, "\n\tBRB1_CTRL_DATA_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_BRB1_CTRL_DATA_IN_EN) >> 15)
				, "\n\tPXP_DP_ACK_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PXP_DP_ACK_IN_EN) >> 14)
				, "\n\tPXP_CTRL_ACK_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PXP_CTRL_ACK_IN_EN) >> 13)
				, "\n\tPXP_DP_DATA_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PXP_DP_DATA_IN_EN) >> 12)
				, "\n\tPXP_INT_DONE_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PXP_INT_DONE_IN_EN) >> 11)
				, "\n\tPXP_CTRL_DATA_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PXP_CTRL_DATA_IN_EN) >> 10)
				, "\n\tPXP_DP_FULL_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PXP_DP_FULL_IN_EN) >> 9)
				, "\n\tPXP_CTRL_FULL_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PXP_CTRL_FULL_IN_EN) >> 8)
				, "\n\tPXP_DP_DONE_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PXP_DP_DONE_IN_EN) >> 7)
				, "\n\tPXP_CTRL_DONE_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PXP_CTRL_DONE_IN_EN) >> 6)
				, "\n\tPAS_BUF_FULL_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PAS_BUF_FULL_IN_EN) >> 5)
				, "\n\tPAS_BUF_DONE_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_PAS_BUF_DONE_IN_EN) >> 4)
				, "\n\tINT_RAM_FULL_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_INT_RAM_FULL_IN_EN) >> 3)
				, "\n\tINT_RAM_DONE_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_INT_RAM_DONE_IN_EN) >> 2)
				, "\n\tINT_RAM_DATA_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_INT_RAM_DATA_IN_EN) >> 1)
				, "\n\tEXT_STORE_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN1_REG_EXT_STORE_IN_EN) >> 0));
			break;
		case 0x4223cUL: // (TSDM_REG_ENABLE_IN2: ACCESS:RW  DataWidth:0x7  Multi Field Register  ) 
			sprintf(disp_str, "\nTSDM_REG_ENABLE_IN2 (ACCESS:RW  DataWidth:0x7  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_REQ_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN2_REG_DORQ_REQ_IN_EN) >> 6)
				, "\n\tPB_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN2_REG_PB_EXT_WR_FULL_IN_EN) >> 5)
				, "\n\tPBF_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN2_REG_PBF_EXT_WR_FULL_IN_EN) >> 4)
				, "\n\tPB_FULL_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN2_REG_PB_FULL_IN_EN) >> 3)
				, "\n\tPB_STATUS_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN2_REG_PB_STATUS_IN_EN) >> 2)
				, "\n\tCM_ACK_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN2_REG_CM_ACK_IN_EN) >> 1)
				, "\n\tSDM_ACK_IN_EN :"
				, (u32_t)((data & TSDM_ENABLE_IN2_REG_SDM_ACK_IN_EN) >> 0));
			break;
		case 0x42240UL: // (TSDM_REG_ENABLE_OUT1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nTSDM_REG_ENABLE_OUT1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPBF_EXT_WR_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PBF_EXT_WR_OUT_EN) >> 26)
				, "\n\tPB_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PB_OUT_EN) >> 25)
				, "\n\tCFC_SDM_ACK_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_CFC_SDM_ACK_OUT_EN) >> 24)
				, "\n\tCM_MSG_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_CM_MSG_OUT_EN) >> 23)
				, "\n\tPXP_REQ_DONE_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PXP_REQ_DONE_OUT_EN) >> 22)
				, "\n\tEXT_FULL_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_EXT_FULL_OUT_EN) >> 21)
				, "\n\tPXP_CTRL_FULL_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PXP_CTRL_FULL_OUT_EN) >> 20)
				, "\n\tPB_FULL_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PB_FULL_OUT_EN) >> 19)
				, "\n\tBRB1_DP_FULL_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_BRB1_DP_FULL_OUT_EN) >> 18)
				, "\n\tBRB1_CTRL_FULL_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_BRB1_CTRL_FULL_OUT_EN) >> 17)
				, "\n\tPXP_DP_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PXP_DP_OUT_EN) >> 16)
				, "\n\tPXP_CTRL_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PXP_CTRL_OUT_EN) >> 15)
				, "\n\tPXP_ASYNC_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PXP_ASYNC_OUT_EN) >> 14)
				, "\n\tPAS_BUF_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PAS_BUF_OUT_EN) >> 13)
				, "\n\tINT_RAM_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_INT_RAM_OUT_EN) >> 12)
				, "\n\tPRS_ACK_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PRS_ACK_OUT_EN) >> 11)
				, "\n\tPRS_SYNC_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PRS_SYNC_OUT_EN) >> 10)
				, "\n\tBRB1_DP_REQ_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_BRB1_DP_REQ_OUT_EN) >> 9)
				, "\n\tBRB1_CTRL_REQ_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_BRB1_CTRL_REQ_OUT_EN) >> 8)
				, "\n\tPXP_DP_REQ_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PXP_DP_REQ_OUT_EN) >> 7)
				, "\n\tPXP_CTRL_REQ_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PXP_CTRL_REQ_OUT_EN) >> 6)
				, "\n\tCFC_PB_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_CFC_PB_OUT_EN) >> 5)
				, "\n\tCFC_ACDEC_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_CFC_ACDEC_OUT_EN) >> 4)
				, "\n\tCFC_ACINC_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_CFC_ACINC_OUT_EN) >> 3)
				, "\n\tCFC_LOAD_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_CFC_LOAD_OUT_EN) >> 2)
				, "\n\tTHREADREADY_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_THREADREADY_OUT_EN) >> 1)
				, "\n\tPXP_INT_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT1_REG_PXP_INT_OUT_EN) >> 0));
			break;
		case 0x42244UL: // (TSDM_REG_ENABLE_OUT2: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nTSDM_REG_ENABLE_OUT2 (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_REQ_DONE_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT2_REG_DORQ_REQ_DONE_OUT_EN) >> 5)
				, "\n\tVFPF_ERR_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT2_REG_VFPF_ERR_OUT_EN) >> 4)
				, "\n\tSDM_EXT_WR_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT2_REG_SDM_EXT_WR_OUT_EN) >> 3)
				, "\n\tQM_EXT_WR_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT2_REG_QM_EXT_WR_OUT_EN) >> 2)
				, "\n\tDQ_EXT_WR_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT2_REG_DQ_EXT_WR_OUT_EN) >> 1)
				, "\n\tPB_EXT_WR_OUT_EN :"
				, (u32_t)((data & TSDM_ENABLE_OUT2_REG_PB_EXT_WR_OUT_EN) >> 0));
			break;
		case 0x42248UL: // (TSDM_REG_NUM_OF_Q0_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q0_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4224cUL: // (TSDM_REG_NUM_OF_Q1_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q1_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42250UL: // (TSDM_REG_NUM_OF_Q3_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q3_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42254UL: // (TSDM_REG_NUM_OF_Q4_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q4_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42258UL: // (TSDM_REG_NUM_OF_Q5_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q5_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4225cUL: // (TSDM_REG_NUM_OF_Q6_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q6_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42260UL: // (TSDM_REG_NUM_OF_Q7_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q7_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42264UL: // (TSDM_REG_NUM_OF_Q8_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q8_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42268UL: // (TSDM_REG_NUM_OF_Q9_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q9_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4226cUL: // (TSDM_REG_NUM_OF_Q10_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q10_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42270UL: // (TSDM_REG_NUM_OF_Q11_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_Q11_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42274UL: // (TSDM_REG_NUM_OF_PKT_END_MSG: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_PKT_END_MSG (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of packet end messages received from the parser\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42278UL: // (TSDM_REG_NUM_OF_PXP_ASYNC_REQ: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_PXP_ASYNC_REQ (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of requests received from the pxp async if\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4227cUL: // (TSDM_REG_NUM_OF_ACK_AFTER_PLACE: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_NUM_OF_ACK_AFTER_PLACE (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of ACK after placement messages received \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42280UL: // (TSDM_REG_STATISTICS_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSDM_REG_STATISTICS_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits for statistics sram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42284UL: // (TSDM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from TSDM\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42288UL: // (TSDM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from TSDM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4228cUL: // (TSDM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSDM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from TSDM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42290UL: // (TSDM_REG_TSDM_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x42294UL: // (TSDM_REG_TSDM_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x42298UL: // (TSDM_REG_TSDM_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x4229cUL: // (TSDM_REG_TSDM_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x422a0UL: // (TSDM_REG_TSDM_INT_STS_1: ACCESS:R  DataWidth:0xe ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_INT_STS_1 (ACCESS:R  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0x422a4UL: // (TSDM_REG_TSDM_INT_STS_CLR_1: ACCESS:RC  DataWidth:0xe ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_CLR_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0x422a8UL: // (TSDM_REG_TSDM_INT_STS_WR_1: ACCESS:WR  DataWidth:0xe ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_INT_STS_WR_1 (ACCESS:WR  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_STS_WR_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0x422acUL: // (TSDM_REG_TSDM_INT_MASK_1: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_INT_MASK_1 (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & TSDM_TSDM_INT_MASK_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0x422b0UL: // (TSDM_REG_TSDM_PRTY_STS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_PRTY_STS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x422b4UL: // (TSDM_REG_TSDM_PRTY_STS_CLR: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_PRTY_STS_CLR (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x422b8UL: // (TSDM_REG_TSDM_PRTY_STS_WR: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_PRTY_STS_WR (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TSDM_TSDM_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x422bcUL: // (TSDM_REG_TSDM_PRTY_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_PRTY_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TSDM_TSDM_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x422c0UL: // (TSDM_REG_CMP_COUNTER_MAX4: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTSDM_REG_CMP_COUNTER_MAX4 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422c4UL: // (TSDM_REG_CMP_COUNTER_MAX5: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTSDM_REG_CMP_COUNTER_MAX5 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422c8UL: // (TSDM_REG_CMP_COUNTER_MAX6: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTSDM_REG_CMP_COUNTER_MAX6 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422ccUL: // (TSDM_REG_CMP_COUNTER_MAX7: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTSDM_REG_CMP_COUNTER_MAX7 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422d4UL: // (TSDM_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSDM_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422d8UL: // (TSDM_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSDM_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422dcUL: // (TSDM_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSDM_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422e0UL: // (TSDM_REG_CPU_MBIST_MEMCTRL_0_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_CPU_MBIST_MEMCTRL_0_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422e4UL: // (TSDM_REG_CPU_MBIST_MEMCTRL_1_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_CPU_MBIST_MEMCTRL_1_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422e8UL: // (TSDM_REG_CPU_MBIST_MEMCTRL_2_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_CPU_MBIST_MEMCTRL_2_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42640UL: // (TSDM_REG_CM_QUEUE_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_CM_QUEUE_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits for cm queue\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42644UL: // (TSDM_REG_INP_QUEUE_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_INP_QUEUE_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits for inp queue\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42648UL: // (TSDM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSDM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4264cUL: // (TSDM_REG_FIFOS_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTSDM_REG_FIFOS_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM bits  fifos: BRB1_CTRL[1:0]; Reserved[3:2];Reserved[5:\n    4];PXP_CTRL[7:6]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42650UL: // (TSDM_REG_TIMERS_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTSDM_REG_TIMERS_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits for timers sram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42400UL: // (TSDM_REG_AGGREG_INTERRUPT_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_AGGREG_INTERRUPT_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: lsb register of aggregated interrupt in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42404UL: // (TSDM_REG_AGGREG_INTERRUPT_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_AGGREG_INTERRUPT_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: msb register of aggregated interrupt  in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42408UL: // (TSDM_REG_ASYNC_HOST_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_ASYNC_HOST_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: async fifo empty in sdm_async block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4240cUL: // (TSDM_REG_ASYNC_HOST_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_ASYNC_HOST_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: async fifo full in sdm_async block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42410UL: // (TSDM_REG_CFC_LOAD_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_CFC_LOAD_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42414UL: // (TSDM_REG_CFC_LOAD_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_CFC_LOAD_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load pending fifo full in sdm_cfc block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42418UL: // (TSDM_REG_CFC_LOAD_RSP_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_CFC_LOAD_RSP_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load rsp fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4241cUL: // (TSDM_REG_CFC_LOAD_RSP_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_CFC_LOAD_RSP_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load rsp fifo full in sdm_cfcblock\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42420UL: // (TSDM_REG_CM_DELAY_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_CM_DELAY_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm delay fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42424UL: // (TSDM_REG_CM_DELAY_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_CM_DELAY_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm delay fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42428UL: // (TSDM_REG_CM_QUEUE_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_CM_QUEUE_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm queue fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4242cUL: // (TSDM_REG_CM_QUEUE_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_CM_QUEUE_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm queue fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42430UL: // (TSDM_REG_DELAY_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DELAY_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: delay FIFO  empty in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42434UL: // (TSDM_REG_DELAY_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DELAY_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: delay FIFO  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42438UL: // (TSDM_REG_DST_BRB1_CTRL_SRC_ADDR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_BRB1_CTRL_SRC_ADDR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src address fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4243cUL: // (TSDM_REG_DST_BRB1_CTRL_SRC_ADDR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_BRB1_CTRL_SRC_ADDR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src address fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42440UL: // (TSDM_REG_DST_BRB1_CTRL_SRC_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_BRB1_CTRL_SRC_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42444UL: // (TSDM_REG_DST_BRB1_CTRL_SRC_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_BRB1_CTRL_SRC_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42448UL: // (TSDM_REG_DST_INT_RAM_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_INT_RAM_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4244cUL: // (TSDM_REG_DST_INT_RAM_WAIT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_INT_RAM_WAIT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram_wait fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42450UL: // (TSDM_REG_DST_INT_RAM_WAIT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_INT_RAM_WAIT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram_wait fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42454UL: // (TSDM_REG_DST_NONE_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_NONE_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: none pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42458UL: // (TSDM_REG_DST_NONE_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_NONE_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: none pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4245cUL: // (TSDM_REG_DST_PAS_BUF_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PAS_BUF_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42460UL: // (TSDM_REG_DST_PAS_BUF_WAIT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PAS_BUF_WAIT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf_wait fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42464UL: // (TSDM_REG_DST_PAS_BUF_WAIT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PAS_BUF_WAIT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf_wait fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42468UL: // (TSDM_REG_DST_PB_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PB_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4246cUL: // (TSDM_REG_DST_PB_IMMED_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PB_IMMED_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb immediate fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42470UL: // (TSDM_REG_DST_PB_IMMED_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PB_IMMED_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb immediate fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42474UL: // (TSDM_REG_DST_PXP_CTRL_DST_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_CTRL_DST_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_dst pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42478UL: // (TSDM_REG_DST_PXP_CTRL_DST_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_CTRL_DST_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_dst pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4247cUL: // (TSDM_REG_DST_PXP_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42480UL: // (TSDM_REG_DST_PXP_CTRL_IMMED_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_CTRL_IMMED_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl immediate fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42484UL: // (TSDM_REG_DST_PXP_CTRL_IMMED_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_CTRL_IMMED_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl immediate fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42488UL: // (TSDM_REG_DST_PXP_CTRL_LINK_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_CTRL_LINK_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl link list empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4248cUL: // (TSDM_REG_DST_PXP_CTRL_LINK_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_CTRL_LINK_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl link list full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42490UL: // (TSDM_REG_DST_PXP_CTRL_SRC_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_CTRL_SRC_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_src pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42494UL: // (TSDM_REG_DST_PXP_CTRL_SRC_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_CTRL_SRC_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_src pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42498UL: // (TSDM_REG_DST_PXP_DP_DST_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_DP_DST_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp_dst pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4249cUL: // (TSDM_REG_DST_PXP_DP_DST_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_DP_DST_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp_dst pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424a0UL: // (TSDM_REG_DST_PXP_DP_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_DP_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424a4UL: // (TSDM_REG_DST_PXP_DP_LINK_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_DP_LINK_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp link list empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424a8UL: // (TSDM_REG_DST_PXP_DP_LINK_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_DST_PXP_DP_LINK_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp link list full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424acUL: // (TSDM_REG_INIT_CREDIT_CFC_ACDEC: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTSDM_REG_INIT_CREDIT_CFC_ACDEC (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     activity counters interface without receiving any ACK\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424b0UL: // (TSDM_REG_INIT_CREDIT_CFC_ACINC: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTSDM_REG_INIT_CREDIT_CFC_ACINC (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     activity counters interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424b4UL: // (TSDM_REG_INIT_CREDIT_CFC_LOAD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTSDM_REG_INIT_CREDIT_CFC_LOAD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     load interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424b8UL: // (TSDM_REG_INIT_CREDIT_CM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTSDM_REG_INIT_CREDIT_CM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of cycles that can be sent to the CM  \n    interface without receiving any ACK in CM block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424bcUL: // (TSDM_REG_INIT_CREDIT_PXP_CTRL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nTSDM_REG_INIT_CREDIT_PXP_CTRL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the pxp\n     control interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424c0UL: // (TSDM_REG_INT_RAM_RR_REQ: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSDM_REG_INT_RAM_RR_REQ (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: round robin for int_ram arbiter: b0-pas_buf; b1-int_ram;b2\n    -pxp_dp;b3-pxp_ctrl;b4-brb1_ctrl;b5-brb1_dp;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424c4UL: // (TSDM_REG_OPERATION_GEN: ACCESS:W  DataWidth:0x11 ) 
			sprintf(disp_str, "\nTSDM_REG_OPERATION_GEN (ACCESS:W  DataWidth:0x11 )" \
				"\nDescription: Generate an operation after completion; bit-16 is AggVectIdx_valid\n    ; bits 15:8 are AggVectIdx; bits 7:5 are the TRIG and bits 4:0 are the\n     T124Param[4:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424c8UL: // (TSDM_REG_PB_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_PB_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: UPB IF  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424ccUL: // (TSDM_REG_PBF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_PBF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF if  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424d0UL: // (TSDM_REG_PXP_DELAY_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_PXP_DELAY_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp switch delay fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424d4UL: // (TSDM_REG_PXP_DELAY_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_PXP_DELAY_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp switch delay fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424d8UL: // (TSDM_REG_QM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_QM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: QM IF  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424dcUL: // (TSDM_REG_QUEUE_EMPTY: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nTSDM_REG_QUEUE_EMPTY (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Input queue fifo empty in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424e0UL: // (TSDM_REG_QUEUE_FULL: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nTSDM_REG_QUEUE_FULL (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Input queue fifo full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424e4UL: // (TSDM_REG_RR_CNT_COUNTERS_STATUS: ACCESS:R  DataWidth:0x15 ) 
			sprintf(disp_str, "\nTSDM_REG_RR_CNT_COUNTERS_STATUS (ACCESS:R  DataWidth:0x15 )" \
				"\nDescription: round robin for all completion counters\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424e8UL: // (TSDM_REG_RR_COMPLETE_REQ: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nTSDM_REG_RR_COMPLETE_REQ (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: round robin for all completion requests  in sdm_cm block: \n    b0-async b1-nop;b2-pxp_int; b3-timers;b4-dma;b5-grc;b6-rbc\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424ecUL: // (TSDM_REG_RR_PTR_REQ: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nTSDM_REG_RR_PTR_REQ (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: round robin for cm pointer: b0-async; b1-dma_dp; b2 - dma_ctrl\n    ; b3-cfc; b4-nop; b5-timers; b6-pxp_int\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424f0UL: // (TSDM_REG_RSP_BRB1_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424f4UL: // (TSDM_REG_RSP_BRB1_CTRL_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_CTRL_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424f8UL: // (TSDM_REG_RSP_BRB1_CTRL_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_CTRL_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x424fcUL: // (TSDM_REG_RSP_BRB1_CTRL_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_CTRL_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42500UL: // (TSDM_REG_RSP_BRB1_CTRL_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_CTRL_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42504UL: // (TSDM_REG_RSP_BRB1_DP_DST_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_DP_DST_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending dst fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42508UL: // (TSDM_REG_RSP_BRB1_DP_DST_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_DP_DST_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending dst fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4250cUL: // (TSDM_REG_RSP_BRB1_DP_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_DP_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42510UL: // (TSDM_REG_RSP_BRB1_DP_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_DP_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42514UL: // (TSDM_REG_RSP_BRB1_DP_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_DP_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42518UL: // (TSDM_REG_RSP_BRB1_DP_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_DP_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4251cUL: // (TSDM_REG_RSP_BRB1_DP_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_BRB1_DP_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42520UL: // (TSDM_REG_RSP_INT_RAM_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_INT_RAM_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42524UL: // (TSDM_REG_RSP_INT_RAM_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_INT_RAM_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42528UL: // (TSDM_REG_RSP_INT_RAM_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_INT_RAM_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4252cUL: // (TSDM_REG_RSP_INT_RAM_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_INT_RAM_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42530UL: // (TSDM_REG_RSP_PB_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_PB_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42534UL: // (TSDM_REG_RSP_PB_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_PB_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42538UL: // (TSDM_REG_RSP_PB_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_PB_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4253cUL: // (TSDM_REG_RSP_PB_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_PB_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42540UL: // (TSDM_REG_RSP_PB_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_PB_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42544UL: // (TSDM_REG_RSP_PXP_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_PXP_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42548UL: // (TSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4254cUL: // (TSDM_REG_RSP_PXP_CTRL_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_RSP_PXP_CTRL_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42550UL: // (TSDM_REG_SYNC_PARSER_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_SYNC_PARSER_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser fifo  empty in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42554UL: // (TSDM_REG_SYNC_PARSER_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_SYNC_PARSER_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser fifo  full in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42558UL: // (TSDM_REG_SYNC_SYNC_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_SYNC_SYNC_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser serial fifo  empty in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4255cUL: // (TSDM_REG_SYNC_SYNC_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_SYNC_SYNC_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser serial fifo  full in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42560UL: // (TSDM_REG_TIMERS_ADDR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_TIMERS_ADDR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: address FIFO  empty in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42564UL: // (TSDM_REG_TIMERS_ADDR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_TIMERS_ADDR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: address FIFO  full in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42568UL: // (TSDM_REG_TIMERS_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_TIMERS_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pending FIFO  empty in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x4256cUL: // (TSDM_REG_TIMERS_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSDM_REG_TIMERS_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pending FIFO  full in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42580UL: // (TSDM_REG_TIMERS: ACCESS:WB  DataWidth:0x34 ) 
			sprintf(disp_str, "\nTSDM_REG_TIMERS (ACCESS:WB  DataWidth:0x34 )" \
				"\nDescription: Debug only. Timers memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42800UL: // (TSDM_REG_STATISTICS: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSDM_REG_STATISTICS (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Statistics memory. Each read from RBC resets the corresponding\n     statistic counter\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x43000UL: // (TSDM_REG_CM_QUEUE: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nTSDM_REG_CM_QUEUE (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Debug only. CM queue memory. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x43800UL: // (TSDM_REG_INP_QUEUE: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nTSDM_REG_INP_QUEUE (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Debug only. Input queue memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422d0UL: // (TSDM_REG_TSDM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x422ecUL: // (TSDM_REG_TSDM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42570UL: // (TSDM_REG_TSDM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42600UL: // (TSDM_REG_TSDM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42654UL: // (TSDM_REG_TSDM_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x42c00UL: // (TSDM_REG_TSDM_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSDM_REG_TSDM_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180000UL: // (TSEM_REG_MSG_NUM_FIC0: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTSEM_REG_MSG_NUM_FIC0 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that entered \n    through FIC0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180004UL: // (TSEM_REG_MSG_NUM_FIC1: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTSEM_REG_MSG_NUM_FIC1 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that entered \n    through FIC1 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180008UL: // (TSEM_REG_MSG_NUM_FOC0: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTSEM_REG_MSG_NUM_FOC0 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18000cUL: // (TSEM_REG_MSG_NUM_FOC1: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTSEM_REG_MSG_NUM_FOC1 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC1 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180010UL: // (TSEM_REG_MSG_NUM_FOC2: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTSEM_REG_MSG_NUM_FOC2 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC2 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180014UL: // (TSEM_REG_MSG_NUM_FOC3: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTSEM_REG_MSG_NUM_FOC3 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180018UL: // (TSEM_REG_THREAD_INTER_CNT_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_THREAD_INTER_CNT_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for start count of counter ~tsem_registers_thread_inter_cnt\n    .thread_inter_cnt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18001cUL: // (TSEM_REG_THREAD_INTER_CNT: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nTSEM_REG_THREAD_INTER_CNT (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Maximum value of threads interrupt counter; when it gets \n    this value then interrupt to will be send if thread active from previous\n     maximum value of this counter. This register may be used only when ~tsem_registers_thread_inter_cnt_enable\n    .thread_inter_cnt_enable =1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180020UL: // (TSEM_REG_ARB_ELEMENT0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_ARB_ELEMENT0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 0. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180024UL: // (TSEM_REG_ARB_ELEMENT1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_ARB_ELEMENT1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 1. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~tsem_registers_arb_element0.arb_element0\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180028UL: // (TSEM_REG_ARB_ELEMENT2: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_ARB_ELEMENT2 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 2. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~tsem_registers_arb_element0.arb_element0\n     and ~tsem_registers_arb_element1.arb_element1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18002cUL: // (TSEM_REG_ARB_ELEMENT3: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_ARB_ELEMENT3 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 3. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2.Could not be equal to register ~tsem_registers_arb_element0.arb_element0\n     and ~tsem_registers_arb_element1.arb_element1 and ~tsem_registers_arb_element2\n    .arb_element2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180030UL: // (TSEM_REG_ARB_ELEMENT4: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_ARB_ELEMENT4 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 4. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~tsem_registers_arb_element0.arb_element0\n     and ~tsem_registers_arb_element1.arb_element1 and ~tsem_registers_arb_element2\n    .arb_element2 and ~tsem_registers_arb_element3.arb_element3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180038UL: // (TSEM_REG_TS_0_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_0_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18003cUL: // (TSEM_REG_TS_1_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_1_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180040UL: // (TSEM_REG_TS_2_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_2_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180044UL: // (TSEM_REG_TS_3_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_3_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180048UL: // (TSEM_REG_TS_4_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_4_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18004cUL: // (TSEM_REG_TS_5_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_5_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180050UL: // (TSEM_REG_TS_6_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_6_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180054UL: // (TSEM_REG_TS_7_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_7_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180058UL: // (TSEM_REG_TS_8_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_8_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18005cUL: // (TSEM_REG_TS_9_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_9_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180060UL: // (TSEM_REG_TS_10_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_10_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180064UL: // (TSEM_REG_TS_11_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_11_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180068UL: // (TSEM_REG_TS_12_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_12_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18006cUL: // (TSEM_REG_TS_13_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_13_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180070UL: // (TSEM_REG_TS_14_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_14_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180074UL: // (TSEM_REG_TS_15_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_15_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180078UL: // (TSEM_REG_TS_16_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_16_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18007cUL: // (TSEM_REG_TS_17_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_17_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180080UL: // (TSEM_REG_TS_18_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_18_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180084UL: // (TSEM_REG_TS_19_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_TS_19_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180088UL: // (TSEM_REG_FIC0_MIN_MSG_LINES: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC0_MIN_MSG_LINES (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The minimum number of cycles in a message from FIC0 interfaces\n     after which the message can be sent to the passive register_file. It \n    must be always equal to 2; other way it may cause to deadlock in a chip\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18008cUL: // (TSEM_REG_FIC1_MIN_MSG_LINES: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC1_MIN_MSG_LINES (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The minimum number of cycles in a message from FIC1 interfaces\n     after which the message can be sent to the passive register_file. It \n    must be always equal to 2; other way it may cause to deadlock in a chip\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180090UL: // (TSEM_REG_PASSIVE_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_PASSIVE_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of free entries in the sync FIFO between the external\n     HW and the passive buffer; below which the PassiveFull is asserted \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180094UL: // (TSEM_REG_SYNC_DRA_WR_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_SYNC_DRA_WR_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full for sync dra_wr fifo (data from DRA to STORM)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180098UL: // (TSEM_REG_SYNC_RAM_WR_ALM_FULL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSEM_REG_SYNC_RAM_WR_ALM_FULL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Almost full for sync ram_wr fifo (data from EXT_IF to STORM\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18009cUL: // (TSEM_REG_DBG_ALM_FULL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSEM_REG_DBG_ALM_FULL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Almost full for slow debug fifo \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800a0UL: // (TSEM_REG_EXCEPTION_INT: ACCESS:RW  DataWidth:0xf ) 
			sprintf(disp_str, "\nTSEM_REG_EXCEPTION_INT (ACCESS:RW  DataWidth:0xf )" \
				"\nDescription: The PRAM address for the interrupt in a case the event ID\n     is bigger then the INT table size. This register is always NA; because\n     this feature is removed\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800a4UL: // (TSEM_REG_ENABLE_IN: ACCESS:RW  DataWidth:0xf  Multi Field Register  ) 
			sprintf(disp_str, "\nTSEM_REG_ENABLE_IN (ACCESS:RW  DataWidth:0xf  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFPF_ERROR_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_VFPF_ERROR_ENABLE_IN) >> 14)
				, "\n\tWAITP_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_WAITP_ENABLE_IN) >> 13)
				, "\n\tFOC3_ACK_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_FOC3_ACK_ENABLE_IN) >> 12)
				, "\n\tFOC2_ACK_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_FOC2_ACK_ENABLE_IN) >> 11)
				, "\n\tFOC1_ACK_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_FOC1_ACK_ENABLE_IN) >> 10)
				, "\n\tFOC0_ACK_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_FOC0_ACK_ENABLE_IN) >> 9)
				, "\n\tRAM1_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_RAM1_ENABLE_IN) >> 8)
				, "\n\tRAM0_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_RAM0_ENABLE_IN) >> 7)
				, "\n\tEXT_FULL_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_EXT_FULL_ENABLE_IN) >> 6)
				, "\n\tEXT_RD_DATA_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_EXT_RD_DATA_ENABLE_IN) >> 5)
				, "\n\tTHREAD_RDY_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_THREAD_RDY_ENABLE_IN) >> 4)
				, "\n\tGENERAL_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_GENERAL_ENABLE_IN) >> 3)
				, "\n\tPASSIVE_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_PASSIVE_ENABLE_IN) >> 2)
				, "\n\tFIC1_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_FIC1_ENABLE_IN) >> 1)
				, "\n\tFIC0_ENABLE_IN :"
				, (u32_t)((data & TSEM_ENABLE_IN_REG_FIC0_ENABLE_IN) >> 0));
			break;
		case 0x1800a8UL: // (TSEM_REG_ENABLE_OUT: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nTSEM_REG_ENABLE_OUT (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWAITP_ENABLE_OUT :"
				, (u32_t)((data & TSEM_ENABLE_OUT_REG_WAITP_ENABLE_OUT) >> 9)
				, "\n\tRAM1_ENABLE_OUT :"
				, (u32_t)((data & TSEM_ENABLE_OUT_REG_RAM1_ENABLE_OUT) >> 8)
				, "\n\tRAM0_ENABLE_OUT :"
				, (u32_t)((data & TSEM_ENABLE_OUT_REG_RAM0_ENABLE_OUT) >> 7)
				, "\n\tPASSIVE_ENABLE_OUT :"
				, (u32_t)((data & TSEM_ENABLE_OUT_REG_PASSIVE_ENABLE_OUT) >> 6)
				, "\n\tFOC3_ENABLE_OUT :"
				, (u32_t)((data & TSEM_ENABLE_OUT_REG_FOC3_ENABLE_OUT) >> 5)
				, "\n\tFOC2_ENABLE_OUT :"
				, (u32_t)((data & TSEM_ENABLE_OUT_REG_FOC2_ENABLE_OUT) >> 4)
				, "\n\tFOC1_ENABLE_OUT :"
				, (u32_t)((data & TSEM_ENABLE_OUT_REG_FOC1_ENABLE_OUT) >> 3)
				, "\n\tFOC0_ENABLE_OUT :"
				, (u32_t)((data & TSEM_ENABLE_OUT_REG_FOC0_ENABLE_OUT) >> 2)
				, "\n\tEXT_WR_REQ_ENABLE_OUT :"
				, (u32_t)((data & TSEM_ENABLE_OUT_REG_EXT_WR_REQ_ENABLE_OUT) >> 1)
				, "\n\tEXT_RD_REQ_ENABLE_OUT :"
				, (u32_t)((data & TSEM_ENABLE_OUT_REG_EXT_RD_REQ_ENABLE_OUT) >> 0));
			break;
		case 0x1800acUL: // (TSEM_REG_STORM0_H_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_STORM0_H_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm0_h memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800b0UL: // (TSEM_REG_STORM1_H_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_STORM1_H_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm1_h memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800b4UL: // (TSEM_REG_STORM0_L_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_STORM0_L_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm0_l memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800b8UL: // (TSEM_REG_STORM1_L_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_STORM1_L_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm1_l memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800bcUL: // (TSEM_REG_CAM_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nTSEM_REG_CAM_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM bits  for cam\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800c0UL: // (TSEM_REG_RAM0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_RAM0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory ram0_0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800c4UL: // (TSEM_REG_INT_TABLE_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_INT_TABLE_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory int_table\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800c8UL: // (TSEM_REG_CLEAR_WAITP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_CLEAR_WAITP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Write 1 to this register will disable waitp from this storm\n     to other storms\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800ccUL: // (TSEM_REG_SLOW_DBG_MODE: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DBG_MODE (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: debug mode for slow debug bus. Applicable only when ~tsem_registers_slow_dbg_active\n    .slow_dbg_active =1. If mode =0 thread number; pram address and DRA WR\n     data selected; if mode =1 fin command and DRA RD ; if mode =2 pram address\n     and thread number and fin command and released thread from STORM; if \n    mode =3 STORE data to SDM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800d0UL: // (TSEM_REG_SLOW_DBG_ACTIVE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DBG_ACTIVE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug mode is active\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800d4UL: // (TSEM_REG_DBG_MSG_SRC: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_DBG_MSG_SRC (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~tsem_registers_slow_dbg_mode.slow_dbg_mode\n     =0. If =0only FIC-s output to debug bus; 1=both FIC-s and passive buffer\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800d8UL: // (TSEM_REG_DBG_MODE0_CFG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_DBG_MODE0_CFG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~tsem_registers_slow_dbg_mode.slow_dbg_mode\n     =0. If =0 all the message output to debug bus; 1=partial message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800dcUL: // (TSEM_REG_DBG_MODE0_CFG_CYCLE: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_DBG_MODE0_CFG_CYCLE (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Applicable only when ~tsem_registers_dbg_mode0_cfg.dbg_mode0_cfg\n     =1. If =1 the additional cycles to extract to the debug bus.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800e0UL: // (TSEM_REG_DBG_MODE1_CFG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_DBG_MODE1_CFG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~tsem_registers_slow_dbg_mode.slow_dbg_mode\n     =1. If=0 output to debug bus without the data; 1=with the data.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800e4UL: // (TSEM_REG_DBG_EACH_CYLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_DBG_EACH_CYLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If=0 output every cycle full indication or thread status; \n    1= output only when there is a change.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800e8UL: // (TSEM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSEM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from USEMI\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800ecUL: // (TSEM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSEM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only.For dbgmux usage (debug data that goes from USEM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800f0UL: // (TSEM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nTSEM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only.For dbgmux usage (debug data that goes from USEM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1800f4UL: // (TSEM_REG_TSEM_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1800f8UL: // (TSEM_REG_TSEM_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1800fcUL: // (TSEM_REG_TSEM_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x180100UL: // (TSEM_REG_TSEM_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x180104UL: // (TSEM_REG_TSEM_INT_STS_1: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_INT_STS_1 (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_OUT_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_VFC_OUT_FIFO_ERROR) >> 12)
				, "\n\tVFC_INTERRUPT :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_VFC_INTERRUPT) >> 11)
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x180108UL: // (TSEM_REG_TSEM_INT_STS_CLR_1: ACCESS:RC  DataWidth:0xd ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0xd )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_OUT_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_VFC_OUT_FIFO_ERROR) >> 12)
				, "\n\tVFC_INTERRUPT :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_VFC_INTERRUPT) >> 11)
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_CLR_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x18010cUL: // (TSEM_REG_TSEM_INT_STS_WR_1: ACCESS:WR  DataWidth:0xd ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_INT_STS_WR_1 (ACCESS:WR  DataWidth:0xd )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_OUT_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_VFC_OUT_FIFO_ERROR) >> 12)
				, "\n\tVFC_INTERRUPT :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_VFC_INTERRUPT) >> 11)
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_STS_WR_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x180110UL: // (TSEM_REG_TSEM_INT_MASK_1: ACCESS:RW  DataWidth:0xd ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_INT_MASK_1 (ACCESS:RW  DataWidth:0xd )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_OUT_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_VFC_OUT_FIFO_ERROR) >> 12)
				, "\n\tVFC_INTERRUPT :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_VFC_INTERRUPT) >> 11)
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & TSEM_TSEM_INT_MASK_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x180114UL: // (TSEM_REG_TSEM_PRTY_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_PRTY_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_0_REG_PARITY) >> 0));
			break;
		case 0x180118UL: // (TSEM_REG_TSEM_PRTY_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_PRTY_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_0_REG_PARITY) >> 0));
			break;
		case 0x18011cUL: // (TSEM_REG_TSEM_PRTY_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_PRTY_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_0_REG_PARITY) >> 0));
			break;
		case 0x180120UL: // (TSEM_REG_TSEM_PRTY_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_PRTY_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_0_REG_PARITY) >> 0));
			break;
		case 0x180124UL: // (TSEM_REG_TSEM_PRTY_STS_1: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_PRTY_STS_1 (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Parity register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_1_REG_VFC_PARITY) >> 5)
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x180128UL: // (TSEM_REG_TSEM_PRTY_STS_CLR_1: ACCESS:RC  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_PRTY_STS_CLR_1 (ACCESS:RC  DataWidth:0x6 )" \
				"\nDescription: Parity register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_1_REG_VFC_PARITY) >> 5)
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_CLR_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x18012cUL: // (TSEM_REG_TSEM_PRTY_STS_WR_1: ACCESS:WR  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_PRTY_STS_WR_1 (ACCESS:WR  DataWidth:0x6 )" \
				"\nDescription: Parity register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_1_REG_VFC_PARITY) >> 5)
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_STS_WR_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x180130UL: // (TSEM_REG_TSEM_PRTY_MASK_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_PRTY_MASK_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Parity mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_1_REG_VFC_PARITY) >> 5)
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & TSEM_TSEM_PRTY_MASK_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x18013cUL: // (TSEM_REG_RAM0_TM1: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_RAM0_TM1 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory ram0_1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180140UL: // (TSEM_REG_RAM0_TM2: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_RAM0_TM2 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory ram0_2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1803a0UL: // (TSEM_REG_PAS_BUF_LSB_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_PAS_BUF_LSB_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  PAS_BUF LSB\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1803a4UL: // (TSEM_REG_PAS_BUF_MSB_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_PAS_BUF_MSB_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  PAS_BUF MSB\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1803a8UL: // (TSEM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSEM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1803acUL: // (TSEM_REG_FIFOS_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nTSEM_REG_FIFOS_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM bits  for  FIC0_LSB [1:0]; FIC0_MSB[3:2]; FIC1_LSB[5:4\n    ]; FIC1_MSB[7:6]; DBG_LSB[9:8];DBG_MSB[11:10]; EXT_PAS[13:12]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1803b4UL: // (TSEM_REG_PAS_BUF_LSB_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_PAS_BUF_LSB_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_lsb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1803b8UL: // (TSEM_REG_PAS_BUF_MSB_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_PAS_BUF_MSB_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_msb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180200UL: // (TSEM_REG_ARBITER_REQUEST: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_ARBITER_REQUEST (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last request: 0-fic0; 1-fic1; 2-priority0; 3-\n    priority1; 4-priority2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180204UL: // (TSEM_REG_ARBITER_SELECT: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_ARBITER_SELECT (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last selection: 0-fic0; 1-fic1; 2-priority0; \n    3-priority1; 4-priority2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180208UL: // (TSEM_REG_ARBITER_SLOT: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_ARBITER_SLOT (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last slot\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18020cUL: // (TSEM_REG_DBG_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_DBG_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG IF is full in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180210UL: // (TSEM_REG_DRA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_DRA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: This register is active when FIN FIO  is empty and DRA RD\n     FIFO is empty\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180214UL: // (TSEM_REG_EXT_PAS_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_EXT_PAS_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_PAS FIFO empty in sem_slow\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180218UL: // (TSEM_REG_EXT_PAS_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_EXT_PAS_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_PAS FIFO Full in sem_slow\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18021cUL: // (TSEM_REG_EXT_STORE_FREE_ENTRIES: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSEM_REG_EXT_STORE_FREE_ENTRIES (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Number of free entries in the external STORE sync FIFO.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180220UL: // (TSEM_REG_EXT_STORE_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_EXT_STORE_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE IF is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180224UL: // (TSEM_REG_FIC0_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC0_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from FIC0  May be updated during \n    run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180228UL: // (TSEM_REG_FIC0_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC0_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO empty in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18022cUL: // (TSEM_REG_FIC0_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC0_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO Full in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180230UL: // (TSEM_REG_FIC0_LENGTH: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC0_LENGTH (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Length from FIC0. Active only with ~tsem_registers_fic0_length_error\n    .fic0_length_error interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180234UL: // (TSEM_REG_FIC1_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC1_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from FIC1  May be updated during \n    run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180238UL: // (TSEM_REG_FIC1_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC1_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO empty in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18023cUL: // (TSEM_REG_FIC1_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC1_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO Full in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180240UL: // (TSEM_REG_FIC1_LENGTH: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC1_LENGTH (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Length from FIC1. Active only with ~tsem_registers_fic1_length_error\n    .fic1_length_error interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180244UL: // (TSEM_REG_GPI_DATA: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nTSEM_REG_GPI_DATA (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: GPI signals that are inputs to SEMI\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180248UL: // (TSEM_REG_NUM_OF_THREADS: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nTSEM_REG_NUM_OF_THREADS (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: The number of threads currently active \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18024cUL: // (TSEM_REG_PAS_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_PAS_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from the passive buffer  May be updated\n     during run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180250UL: // (TSEM_REG_PAS_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_PAS_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Full from passive buffer asserted toward SDM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180254UL: // (TSEM_REG_RAM0_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_RAM0_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RAM0 IF is full in sem_slow_ls_ram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180258UL: // (TSEM_REG_RAM1_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_RAM1_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RAM1 IF is full in sem_slow_ls_ram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18025cUL: // (TSEM_REG_SET0_THREAD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SET0_THREAD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET0_THREAD fifo is empty in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180260UL: // (TSEM_REG_SET0_THREAD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SET0_THREAD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET0_THREAD fifo is full in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180264UL: // (TSEM_REG_SET1_THREAD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SET1_THREAD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET1_THREAD fifo is empty in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180268UL: // (TSEM_REG_SET1_THREAD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SET1_THREAD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET1_THREAD fifo is full in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18026cUL: // (TSEM_REG_SLEEP_THREADS_VALID: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nTSEM_REG_SLEEP_THREADS_VALID (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Valid sleeping threads indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180270UL: // (TSEM_REG_SLOW_DBG_ALM_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DBG_ALM_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is almost empty in sem_slow_ls_dbg (31 entry inside\n     fifo)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180274UL: // (TSEM_REG_SLOW_DBG_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DBG_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is almost empty in sem_slow_ls_dbg according to \n    configuration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180278UL: // (TSEM_REG_SLOW_DBG_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DBG_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is empty in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18027cUL: // (TSEM_REG_SLOW_DBG_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DBG_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is full in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180280UL: // (TSEM_REG_SLOW_DRA_FIN_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DRA_FIN_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIN fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180284UL: // (TSEM_REG_SLOW_DRA_FIN_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DRA_FIN_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIN fifo is full in sem_slow_dra_sync (never may be active\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180288UL: // (TSEM_REG_SLOW_DRA_INT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DRA_INT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18028cUL: // (TSEM_REG_SLOW_DRA_INT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DRA_INT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt fifo is full in sem_slow_dra_int\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180290UL: // (TSEM_REG_SLOW_DRA_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DRA_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_RD pop fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180294UL: // (TSEM_REG_SLOW_DRA_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DRA_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_RD pop fifo is full in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180298UL: // (TSEM_REG_SLOW_DRA_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DRA_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_WR push fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18029cUL: // (TSEM_REG_SLOW_DRA_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_DRA_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_WR push fifo is full in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802a0UL: // (TSEM_REG_SLOW_EXT_STORE_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_EXT_STORE_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802a4UL: // (TSEM_REG_SLOW_EXT_STORE_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_EXT_STORE_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802a8UL: // (TSEM_REG_SLOW_RAM0_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_RAM0_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM0 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802acUL: // (TSEM_REG_SLOW_RAM0_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_RAM0_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM0 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802b0UL: // (TSEM_REG_SLOW_RAM0_WR_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_RAM0_WR_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is almost full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802b4UL: // (TSEM_REG_SLOW_RAM0_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_RAM0_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM0 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802b8UL: // (TSEM_REG_SLOW_RAM0_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_RAM0_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM0 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802bcUL: // (TSEM_REG_SLOW_RAM1_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_RAM1_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM1 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802c0UL: // (TSEM_REG_SLOW_RAM1_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_RAM1_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM1 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802c4UL: // (TSEM_REG_SLOW_RAM1_WR_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_RAM1_WR_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is almost full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802c8UL: // (TSEM_REG_SLOW_RAM1_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_RAM1_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802ccUL: // (TSEM_REG_SLOW_RAM1_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SLOW_RAM1_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802d0UL: // (TSEM_REG_SYNC_DBG_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SYNC_DBG_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FAST SYNC FIFO is empty in sem_slow_ls_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802d4UL: // (TSEM_REG_SYNC_DBG_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nTSEM_REG_SYNC_DBG_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FAST SYNC FIFO is full in sem_slow_ls_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802d8UL: // (TSEM_REG_THREAD_ERROR: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nTSEM_REG_THREAD_ERROR (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Thread error indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802dcUL: // (TSEM_REG_THREAD_OVERRUN_NUM: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nTSEM_REG_THREAD_OVERRUN_NUM (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Threads are sleeping in passive buffer more than ~tsem_registers_thread_inter_cnt\n    .thread_inter_cnt number of cycles\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802e0UL: // (TSEM_REG_THREAD_RDY: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nTSEM_REG_THREAD_RDY (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Thread ready indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802e4UL: // (TSEM_REG_THREADS_LIST: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nTSEM_REG_THREADS_LIST (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: List of free threads . There is a bit per thread.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802e8UL: // (TSEM_REG_WB_MSB: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nTSEM_REG_WB_MSB (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Reset value of this register is right when was not read to\n     ~tsem_registers_fic0_fifo.fic0_fifo or ~tsem_registers_fic1_fifo.fic1_fifo\n     or ~tsem_registers_passive_buffer.passive_buffer. For read from ~tsem_registers_passive_buffer\n    .passive_buffer :b0- parity0; b1 parity1. For  read from ~tsem_registers_fic0_fifo\n    .fic0_fifo and ~tsem_registers_fic1_fifo.fic1_fifo :b1=0 data from ~tsem_registers_fic0_fifo\n    .fic0_fifo and ~tsem_registers_fic1_fifo.fic1_fifo is valid; b1 =1 ~tsem_registers_fic0_fifo\n    .fic0_fifo and ~tsem_registers_fic1_fifo.fic1_fifo is empty and data from\n     it must be equal to 0; b0 - parity from ~tsem_registers_fic0_fifo.fic0_fifo\n     and ~tsem_registers_fic1_fifo.fic1_fifo\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180300UL: // (TSEM_REG_FIC0_FIFO: ACCESS:WB_R  DataWidth:0x80 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC0_FIFO (ACCESS:WB_R  DataWidth:0x80 )" \
				"\nDescription: Debug only. FIC0_fifo: b[127:0] data; b128-parity;b129=1- \n    fifo empty;b129=0-data is valid\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180320UL: // (TSEM_REG_FIC1_FIFO: ACCESS:WB_R  DataWidth:0x80 ) 
			sprintf(disp_str, "\nTSEM_REG_FIC1_FIFO (ACCESS:WB_R  DataWidth:0x80 )" \
				"\nDescription: Debug only. FIC1_fifo read for debugging mode; b[127:0] data\n    ; b128-parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180340UL: // (TSEM_REG_FIN_COMMAND: ACCESS:WB_R  DataWidth:0x6d ) 
			sprintf(disp_str, "\nTSEM_REG_FIN_COMMAND (ACCESS:WB_R  DataWidth:0x6d )" \
				"\nDescription: last fin command that was read from fifo. Its spelling in\n     ~tsem_registers_fin_fifo.fin_fifo register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180360UL: // (TSEM_REG_FIN_FIFO: ACCESS:WB_R  DataWidth:0x6d ) 
			sprintf(disp_str, "\nTSEM_REG_FIN_FIFO (ACCESS:WB_R  DataWidth:0x6d )" \
				"\nDescription: Debug only. FIn FIFO.  [5:0]   start_rp_foc3; [11:6]    start_rp_foc2\n    ;  [17:12]   start_rp_foc1; [23:18]   start_rp_foc0;  [29:24]   end_rp_foc3\n    ; [35:30]   end_rp_foc2; [41:36]   end_rp_foc1; [47:42]   end_rp_foc0; [\n    53:48]   lowest rp; [59:54]   highest rp; [65:60]   store start rp; [71\n    :66]   store end rp; [77:72]   load start rp; [83:78]   load end rp; [\n    85:84]   priority; [101:86]  pram address; [102] pas; [103] foc3; [104\n    ] foc2; [105] foc1; [106] foc0; [107] release; [108] fin; [109]=1- fin\n     fifo is empty; 0 - data[108:0] is valid\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180380UL: // (TSEM_REG_VFPF_ERR_NUM: ACCESS:W  DataWidth:0x7 ) 
			sprintf(disp_str, "\nTSEM_REG_VFPF_ERR_NUM (ACCESS:W  DataWidth:0x7 )" \
				"\nDescription: VF or PF ID for reset error bit. Values 0-63 reset error \n    bit for 64 VF; values 64-67 reset error for 4 PF; values 68-127 are not\n     valid.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180388UL: // (TSEM_REG_VF_ERR_VECTOR_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSEM_REG_VF_ERR_VECTOR_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: VF/PF error bitmap vector [0-31]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180390UL: // (TSEM_REG_VF_ERR_VECTOR_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nTSEM_REG_VF_ERR_VECTOR_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: VF/PF error bitmap vector [32-63]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180398UL: // (TSEM_REG_PF_ERR_VECTOR: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nTSEM_REG_PF_ERR_VECTOR (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: VF/PF error bitmap vector [0-7]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1803b0UL: // (TSEM_REG_THREAD_SET_NUM: ACCESS:W  DataWidth:0x5 ) 
			sprintf(disp_str, "\nTSEM_REG_THREAD_SET_NUM (ACCESS:W  DataWidth:0x5 )" \
				"\nDescription: Thread ID. Write thread ID will set ready indication for \n    this thread ID\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180400UL: // (TSEM_REG_INT_TABLE: ACCESS:RW  DataWidth:0xf ) 
			sprintf(disp_str, "\nTSEM_REG_INT_TABLE (ACCESS:RW  DataWidth:0xf )" \
				"\nDescription: Interrupt table  Read and write access to it is not possible\n     in the middle of the work\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x181000UL: // (TSEM_REG_PASSIVE_BUFFER: ACCESS:WB  DataWidth:0x80 ) 
			sprintf(disp_str, "\nTSEM_REG_PASSIVE_BUFFER (ACCESS:WB  DataWidth:0x80 )" \
				"\nDescription: Debug only. Passive buffer memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x182000UL: // (TSEM_REG_PASSIVE_BUFFER_MSB: ACCESS:WB  DataWidth:0x80 ) 
			sprintf(disp_str, "\nTSEM_REG_PASSIVE_BUFFER_MSB (ACCESS:WB  DataWidth:0x80 )" \
				"\nDescription: Debug only. Passive buffer memory MSB that starts from row\n     256 of passive buffer till row 639\n(GRC %05x=%08x) "
				, addr, data);
			break;
		// Ignore line: TSEM_REG_FAST_MEMORY                                 0x1a0000UL //ACCESS:RW  DataWidth:0x20  Description: This address space contains all registers and memories that are placed in SEM_FAST block. The SEM_FAST registers are described in appendix B. In order to access the SEM_FAST registers the base address TSEM_REGISTERS_FAST_MEMORY (Offset: 0x1a0000) should be added to each SEM_FAST register offset.
		case 0x1c0000UL: // (TSEM_REG_PRAM: ACCESS:WB  DataWidth:0x2e ) 
			sprintf(disp_str, "\nTSEM_REG_PRAM (ACCESS:WB  DataWidth:0x2e )" \
				"\nDescription: pram memory. B45 is parity; b[44:0] - data.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180134UL: // (TSEM_REG_TSEM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180144UL: // (TSEM_REG_TSEM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1802ecUL: // (TSEM_REG_TSEM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180384UL: // (TSEM_REG_TSEM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18038cUL: // (TSEM_REG_TSEM_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180394UL: // (TSEM_REG_TSEM_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x18039cUL: // (TSEM_REG_TSEM_UNUSED_EMPTY_6: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_UNUSED_EMPTY_6 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1803bcUL: // (TSEM_REG_TSEM_UNUSED_EMPTY_7: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_UNUSED_EMPTY_7 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x180800UL: // (TSEM_REG_TSEM_UNUSED_EMPTY_8: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_UNUSED_EMPTY_8 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x184000UL: // (TSEM_REG_TSEM_UNUSED_EMPTY_9: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nTSEM_REG_TSEM_UNUSED_EMPTY_9 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0000UL: // (UCM_REG_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. Initialises specific states and statuses. To \n    initialise the state - write 1 into register; to enable working after \n    that - write 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0004UL: // (UCM_REG_UCM_STORM0_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_UCM_STORM0_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - STORM 0 Interface enable. If 0 - the acknowledge input\n     is disregarded; valid is deasserted; all other signals are treated as\n     usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0008UL: // (UCM_REG_UCM_STORM1_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_UCM_STORM1_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - STORM 1 Interface enable. If 0 - the acknowledge input\n     is disregarded; valid is deasserted; all other signals are treated as\n     usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe000cUL: // (UCM_REG_UCM_UQM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_UCM_UQM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - QM Interface enable. If 0 - the acknowledge input is\n     disregarded; valid is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0010UL: // (UCM_REG_STORM_UCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_STORM_UCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: STORM - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0014UL: // (UCM_REG_UQM_UCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_UQM_UCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: QM - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0018UL: // (UCM_REG_USDM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_USDM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input SDM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe001cUL: // (UCM_REG_TM_UCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_TM_UCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Timers - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0020UL: // (UCM_REG_UCM_TM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_UCM_TM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - Timers Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0024UL: // (UCM_REG_TSEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_TSEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input tsem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0028UL: // (UCM_REG_CSEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_CSEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input csem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe002cUL: // (UCM_REG_XSEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_XSEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input xsem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0030UL: // (UCM_REG_DORQ_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_DORQ_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input dorq Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0034UL: // (UCM_REG_CDU_AG_WR_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_CDU_AG_WR_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU AG write Interface enable. If 0 - the request and valid\n     input are disregarded; all other signals are treated as usual; if 1 - \n    normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0038UL: // (UCM_REG_CDU_AG_RD_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_CDU_AG_RD_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU AG read Interface enable. If 0 - the request input is\n     disregarded; valid output is deasserted; all other signals are treated\n     as usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe003cUL: // (UCM_REG_CDU_SM_WR_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_CDU_SM_WR_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU STORM write Interface enable. If 0 - the request and \n    valid input is disregarded; all other signals are treated as usual; if\n     1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0040UL: // (UCM_REG_CDU_SM_RD_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_CDU_SM_RD_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU STORM read Interface enable. If 0 - the request input\n     is disregarded; valid output is deasserted; all other signals are treated\n     as usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0044UL: // (UCM_REG_UCM_CFC_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_UCM_CFC_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - CFC Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0048UL: // (UCM_REG_XX_MAX_LL_SZ: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUCM_REG_XX_MAX_LL_SZ (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Maximum link list size (pending messages) per connection \n    in the XX protection.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe004cUL: // (UCM_REG_XX_OVFL_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUCM_REG_XX_OVFL_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID; sent to the STORM in case of XX overflow.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0050UL: // (UCM_REG_XX_MAX_NUM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_XX_MAX_NUM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The maximum number of connections in the XX protection.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0054UL: // (UCM_REG_N_SM_CTX_LD_0: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_0 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0058UL: // (UCM_REG_N_SM_CTX_LD_1: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_1 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe005cUL: // (UCM_REG_N_SM_CTX_LD_2: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_2 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0060UL: // (UCM_REG_N_SM_CTX_LD_3: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_3 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0064UL: // (UCM_REG_N_SM_CTX_LD_4: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_4 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0068UL: // (UCM_REG_N_SM_CTX_LD_5: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_5 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe006cUL: // (UCM_REG_N_SM_CTX_LD_6: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_6 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0070UL: // (UCM_REG_N_SM_CTX_LD_7: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_7 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0074UL: // (UCM_REG_N_SM_CTX_LD_8: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_8 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0078UL: // (UCM_REG_N_SM_CTX_LD_9: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_9 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe007cUL: // (UCM_REG_N_SM_CTX_LD_10: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_10 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0080UL: // (UCM_REG_N_SM_CTX_LD_11: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_11 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0084UL: // (UCM_REG_N_SM_CTX_LD_12: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_12 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0088UL: // (UCM_REG_N_SM_CTX_LD_13: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_13 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe008cUL: // (UCM_REG_N_SM_CTX_LD_14: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_14 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0090UL: // (UCM_REG_N_SM_CTX_LD_15: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_N_SM_CTX_LD_15 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. the double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0094UL: // (UCM_REG_UQM_UCM_HDR_P: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nUCM_REG_UQM_UCM_HDR_P (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header value for QM request (primary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0098UL: // (UCM_REG_UQM_UCM_HDR_S: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nUCM_REG_UQM_UCM_HDR_S (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header value for QM request (secondary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe009cUL: // (UCM_REG_TM_UCM_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nUCM_REG_TM_UCM_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header for Timers expiration command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00a0UL: // (UCM_REG_ERR_UCM_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nUCM_REG_ERR_UCM_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM erroneous header for QM and Timers formatting.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00a4UL: // (UCM_REG_ERR_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUCM_REG_ERR_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID in case ErrorFlg input message bit is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00a8UL: // (UCM_REG_EXPR_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUCM_REG_EXPR_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID for Timers expiration.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00acUL: // (UCM_REG_STOP_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUCM_REG_STOP_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID for Timers formatting in case of stop done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00b0UL: // (UCM_REG_STORM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_STORM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the STORM input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00b4UL: // (UCM_REG_TSEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_TSEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input tsem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00b8UL: // (UCM_REG_CSEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_CSEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input csem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00bcUL: // (UCM_REG_XSEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_XSEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input xsem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00c0UL: // (UCM_REG_DORQ_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_DORQ_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input dorq in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00c4UL: // (UCM_REG_CP_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_CP_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the CP input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00c8UL: // (UCM_REG_USDM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_USDM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the SDM input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00ccUL: // (UCM_REG_UQM_P_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_UQM_P_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the QM (primary) input in the WRR mechanism\n    . 0 stands for weight 8 (the most prioritised); 1 stands for weight 1(\n    least prioritised); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00d0UL: // (UCM_REG_UQM_S_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_UQM_S_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the QM (secondary) input in the WRR mechanism\n    . 0 stands for weight 8 (the most prioritised); 1 stands for weight 1(\n    least prioritised); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00d4UL: // (UCM_REG_TM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_TM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the Timers input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00d8UL: // (UCM_REG_UCM_UQM_USE_Q: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_UCM_UQM_USE_Q (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set the Q index; received from the QM is inserted to event\n     ID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00dcUL: // (UCM_REG_UCM_REG0_SZ: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_UCM_REG0_SZ (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The size of AG context region 0 in REG-pairs. Designates \n    the MS REG-pair number (e.g. if region 0 is 6 REG-pairs; the value should\n     be 5). Is used to determine the number of the AG context REG-pairs written\n     back; when the Reg1WbFlg isn't set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00e0UL: // (UCM_REG_STOP_DONE_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nUCM_REG_STOP_DONE_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header for Timers expiration command; when Stop Done\n     input bit is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00e4UL: // (UCM_REG_INV_CFLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_INV_CFLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for invalidate counter flag decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00e8UL: // (UCM_REG_CMP_CFLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_CMP_CFLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for comparison counter flag decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00ecUL: // (UCM_REG_TX_CFLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_TX_CFLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for tx counter flag decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00f0UL: // (UCM_REG_TIMER_CFLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_TIMER_CFLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for timer counter flag decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00f4UL: // (UCM_REG_AUX_CFLG1_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_AUX_CFLG1_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for auxillary counter flag 1 decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00f8UL: // (UCM_REG_AUX_CFLG2_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_AUX_CFLG2_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for auxillary counter flag 2 decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe00fcUL: // (UCM_REG_AUX_CFLG3_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_AUX_CFLG3_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for auxillary counter flag 3 decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0100UL: // (UCM_REG_AUX_CFLG4_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_AUX_CFLG4_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for auxillary counter flag 4 decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0104UL: // (UCM_REG_AGG_MISC4_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_AGG_MISC4_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for aggregative miscellaneous 4 decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0108UL: // (UCM_REG_AGG_VAL3_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_AGG_VAL3_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for aggregative value 3 decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe010cUL: // (UCM_REG_AGG_VAL2_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_AGG_VAL2_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for aggregative value 2 decision.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0130UL: // (UCM_REG_PM_RAM_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_PM_RAM_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of Pending messages RAM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0134UL: // (UCM_REG_SM_CTX0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_SM_CTX0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of STORM context. LSB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0138UL: // (UCM_REG_SM_CTX1_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUCM_REG_SM_CTX1_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM bits of STORM context. LSB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe013cUL: // (UCM_REG_AG_CTX0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_AG_CTX0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0140UL: // (UCM_REG_AG_CTX1_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_AG_CTX1_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0144UL: // (UCM_REG_GR_ARB_TYPE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_GR_ARB_TYPE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Arbitration between Input Arbiter groups: 0 - fair Round-\n    Robin; 1 - strict priority defined by ~ucm_registers_gr_ag_pr.gr_ag_pr\n    ; ~ucm_registers_gr_ld0_pr.gr_ld0_pr and ~ucm_registers_gr_ld1_pr.gr_ld1_pr\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0148UL: // (UCM_REG_GR_AG_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_GR_AG_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: AG channel group priority. The lowest priority is 0; the \n    highest priority is 3. It is supposed that the Store channel group is \n    compliment to the others.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe014cUL: // (UCM_REG_GR_LD0_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_GR_LD0_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Load (FIC0) channel group priority. The lowest priority is\n     0; the highest priority is 3. It is supposed that the Store channel group\n     is compliment to the others.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0150UL: // (UCM_REG_GR_LD1_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_GR_LD1_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Load (FIC1) channel group priority. The lowest priority is\n     0; the highest priority is 3. It is supposed that the Store channel group\n     is compliment to the others.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0154UL: // (UCM_REG_STORM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_STORM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the STORM interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0158UL: // (UCM_REG_USDM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_USDM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the SDM interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe015cUL: // (UCM_REG_TSEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_TSEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the tsem interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0160UL: // (UCM_REG_CSEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_CSEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the csem interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0164UL: // (UCM_REG_XSEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_XSEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the xsem interface isdetected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0168UL: // (UCM_REG_DORQ_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_DORQ_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the message length mismatch (relative to last indication\n    ) at the dorq interface is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe016cUL: // (UCM_REG_XX_FREE: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUCM_REG_XX_FREE (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Use to read the XX protection Free counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0170UL: // (UCM_REG_CAM_OCCUP: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_CAM_OCCUP (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Used to read the XX protection CAM occupancy counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0174UL: // (UCM_REG_UNLOCK_MISS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_UNLOCK_MISS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the error; indicating the LCID to be unlocked doesn\n    't exist in LCID CAM; is detected.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0178UL: // (UCM_REG_UQM_GLB_USE_CNTR: ACCESS:R  DataWidth:0x1a ) 
			sprintf(disp_str, "\nUCM_REG_UQM_GLB_USE_CNTR (ACCESS:R  DataWidth:0x1a )" \
				"\nDescription: QM global usage counter. Indicates the balance between QM\n     requests sent and received by the UCM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe017cUL: // (UCM_REG_CP_BUF_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_CP_BUF_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: CP buffer is empty indication.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0180UL: // (UCM_REG_CP_BUF_STATUS: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_CP_BUF_STATUS (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: CP buffer status.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0184UL: // (UCM_REG_XX_OVFL_CNTR: ACCESS:ST  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUCM_REG_XX_OVFL_CNTR (ACCESS:ST  DataWidth:0x10 )" \
				"\nDescription: Counter of XX 0verflow occurencies.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0188UL: // (UCM_REG_STORM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_STORM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the STORM input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe018cUL: // (UCM_REG_USDM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_USDM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input SDM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0190UL: // (UCM_REG_TSEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_TSEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input tsem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0194UL: // (UCM_REG_CSEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_CSEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input csem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0198UL: // (UCM_REG_XSEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_XSEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input xsem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe019cUL: // (UCM_REG_DORQ_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_DORQ_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input dorq.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01a0UL: // (UCM_REG_CP_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_CP_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the CP input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01a4UL: // (UCM_REG_UQM_P_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_UQM_P_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the QM input (primary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01a8UL: // (UCM_REG_UQM_S_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_UQM_S_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the QM input (secondary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01acUL: // (UCM_REG_TM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_TM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the Timers input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01b0UL: // (UCM_REG_STORM_OUT_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_STORM_OUT_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the output messages at FIC0 and FIC1 interfaces\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01b4UL: // (UCM_REG_UQM_OUT_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_UQM_OUT_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the output QM commands.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01b8UL: // (UCM_REG_TM_OUT_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUCM_REG_TM_OUT_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the output Timers commands.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01bcUL: // (UCM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUCM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from UCM\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01c0UL: // (UCM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUCM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from UCM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select; before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01c4UL: // (UCM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUCM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from UCM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01c8UL: // (UCM_REG_UCM_INT_STS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nUCM_REG_UCM_INT_STS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_CP0_ERR) >> 8)
				, "\n\tDORQ_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_DORQ_ERR) >> 7)
				, "\n\tXSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_XSEM_ERR) >> 6)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_CSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_TSEM_ERR) >> 4)
				, "\n\tUSDM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_USDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & UCM_UCM_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xe01ccUL: // (UCM_REG_UCM_INT_STS_CLR: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nUCM_REG_UCM_INT_STS_CLR (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_CP0_ERR) >> 8)
				, "\n\tDORQ_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_DORQ_ERR) >> 7)
				, "\n\tXSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_XSEM_ERR) >> 6)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_CSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_TSEM_ERR) >> 4)
				, "\n\tUSDM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_USDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & UCM_UCM_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xe01d0UL: // (UCM_REG_UCM_INT_STS_WR: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nUCM_REG_UCM_INT_STS_WR (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_CP0_ERR) >> 8)
				, "\n\tDORQ_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_DORQ_ERR) >> 7)
				, "\n\tXSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_XSEM_ERR) >> 6)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_CSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_TSEM_ERR) >> 4)
				, "\n\tUSDM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_USDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & UCM_UCM_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xe01d4UL: // (UCM_REG_UCM_INT_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nUCM_REG_UCM_INT_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_UM_ERR) >> 10)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_CP1_ERR) >> 9)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_CP0_ERR) >> 8)
				, "\n\tDORQ_ERR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_DORQ_ERR) >> 7)
				, "\n\tXSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_XSEM_ERR) >> 6)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_CSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_TSEM_ERR) >> 4)
				, "\n\tUSDM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_USDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & UCM_UCM_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xe01d8UL: // (UCM_REG_UCM_PRTY_STS: ACCESS:R  DataWidth:0x1b ) 
			sprintf(disp_str, "\nUCM_REG_UCM_PRTY_STS (ACCESS:R  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_CP0_PRTY) >> 24)
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_DORQ_PRTY) >> 23)
				, "\n\tXSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_XSEM_PRTY) >> 22)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_CSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_TSEM_PRTY) >> 20)
				, "\n\tUSDM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_USDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0xe01dcUL: // (UCM_REG_UCM_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x1b ) 
			sprintf(disp_str, "\nUCM_REG_UCM_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_CP0_PRTY) >> 24)
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_DORQ_PRTY) >> 23)
				, "\n\tXSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_XSEM_PRTY) >> 22)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_CSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_TSEM_PRTY) >> 20)
				, "\n\tUSDM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_USDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0xe01e0UL: // (UCM_REG_UCM_PRTY_STS_WR: ACCESS:WR  DataWidth:0x1b ) 
			sprintf(disp_str, "\nUCM_REG_UCM_PRTY_STS_WR (ACCESS:WR  DataWidth:0x1b )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_CP0_PRTY) >> 24)
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_DORQ_PRTY) >> 23)
				, "\n\tXSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_XSEM_PRTY) >> 22)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_CSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_TSEM_PRTY) >> 20)
				, "\n\tUSDM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_USDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & UCM_UCM_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0xe01e4UL: // (UCM_REG_UCM_PRTY_MASK: ACCESS:RW  DataWidth:0x1b ) 
			sprintf(disp_str, "\nUCM_REG_UCM_PRTY_MASK (ACCESS:RW  DataWidth:0x1b )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_UM_PRTY) >> 26)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_CP1_PRTY) >> 25)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_CP0_PRTY) >> 24)
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_DORQ_PRTY) >> 23)
				, "\n\tXSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_XSEM_PRTY) >> 22)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_CSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_TSEM_PRTY) >> 20)
				, "\n\tUSDM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_USDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & UCM_UCM_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0xe01e8UL: // (UCM_REG_SM_CTX01_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUCM_REG_SM_CTX01_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM bits of STORM context. MSB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01ecUL: // (UCM_REG_SM_CTX11_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUCM_REG_SM_CTX11_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM bits of STORM context. MSB.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01f0UL: // (UCM_REG_IS_UM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_IS_UM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of UM input stage buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01f4UL: // (UCM_REG_IS_STORM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_IS_STORM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of STORM input stage buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01f8UL: // (UCM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUCM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: chicken bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe01fcUL: // (UCM_REG_IS_CSEM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_IS_CSEM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of Csem input stage buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0274UL: // (UCM_REG_IS_TSEM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_IS_TSEM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of Csem input stage buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0278UL: // (UCM_REG_IS_XSEM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUCM_REG_IS_XSEM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of Csem input stage buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0400UL: // (UCM_REG_UM_FIC1_FORCE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUCM_REG_UM_FIC1_FORCE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0-messages unlocked from Pending messages RAM go to the FIC\n     for which they were designated in input message; 1-messages unlocked \n    from Pending messages RAM are forced to FIC1 whether they were destined\n     to FIC0 or FIC1 in original message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0110UL: // (UCM_REG_PHYS_QNUM0_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nUCM_REG_PHYS_QNUM0_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 0 per port index (CID[23])\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0114UL: // (UCM_REG_PHYS_QNUM0_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nUCM_REG_PHYS_QNUM0_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 0 per port index (CID[23])\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0118UL: // (UCM_REG_PHYS_QNUM1_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nUCM_REG_PHYS_QNUM1_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 1 per port index (CID[23])\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe011cUL: // (UCM_REG_PHYS_QNUM1_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nUCM_REG_PHYS_QNUM1_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 1 per port index (CID[23])\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0120UL: // (UCM_REG_PHYS_QNUM2_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nUCM_REG_PHYS_QNUM2_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 2 per port index (CID[23])\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0124UL: // (UCM_REG_PHYS_QNUM2_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nUCM_REG_PHYS_QNUM2_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 2 per port index (CID[23])\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0128UL: // (UCM_REG_PHYS_QNUM3_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nUCM_REG_PHYS_QNUM3_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 3 per port index (CID[23])\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe012cUL: // (UCM_REG_PHYS_QNUM3_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nUCM_REG_PHYS_QNUM3_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 3 per port index (CID[23])\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0200UL: // (UCM_REG_CAM_OCCUP_ST: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUCM_REG_CAM_OCCUP_ST (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: CAM occupancy sticky status. The write to the register is\n     performed by the XX internal circuitry.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0204UL: // (UCM_REG_CFC_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUCM_REG_CFC_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CFC output initial credit. Max credit available - 15.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 1 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0208UL: // (UCM_REG_CP_MSG: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUCM_REG_CP_MSG (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Used to write the CP message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe020cUL: // (UCM_REG_FIC0_INIT_CRD: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUCM_REG_FIC0_INIT_CRD (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: FIC0 output initial credit. Max credit available - 255.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 64 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0210UL: // (UCM_REG_FIC1_INIT_CRD: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUCM_REG_FIC1_INIT_CRD (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: FIC1 output initial credit. Max credit available - 255.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 64 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0214UL: // (UCM_REG_GLB_CNT_STICKY: ACCESS:RW  DataWidth:0x1a ) 
			sprintf(disp_str, "\nUCM_REG_GLB_CNT_STICKY (ACCESS:RW  DataWidth:0x1a )" \
				"\nDescription: QM global usage counter maximum sticky value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0218UL: // (UCM_REG_LL_SZ_STICKY: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUCM_REG_LL_SZ_STICKY (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: XX LL maximum value ever reached sticky value for any connection\n    . The write to the register is performed by the XX internal circuitry.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe021cUL: // (UCM_REG_TM_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUCM_REG_TM_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Timers output initial credit. Max credit available - 15.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 4 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0220UL: // (UCM_REG_UQM_INIT_CRD: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUCM_REG_UQM_INIT_CRD (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: QM output initial credit. Max credit available - 32.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 32 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0224UL: // (UCM_REG_XX_INIT_CRD: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUCM_REG_XX_INIT_CRD (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Initial value for the credit counter; responsible for fulfilling\n     of the Input Stage XX protection buffer by the XX protection pending \n    messages. Write writes the initial credit value; read returns the current\n     value of the credit counter. Must be initialized to 12 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0228UL: // (UCM_REG_XX_MSG_NUM: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUCM_REG_XX_MSG_NUM (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The maximum number of pending messages; which may be stored\n     in XX protection. ~ucm_registers_xx_free.xx_free is also written on write\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe022cUL: // (UCM_REG_LCID_CAM_0: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_0 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0230UL: // (UCM_REG_LCID_CAM_1: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_1 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0234UL: // (UCM_REG_LCID_CAM_2: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_2 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0238UL: // (UCM_REG_LCID_CAM_3: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_3 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe023cUL: // (UCM_REG_LCID_CAM_4: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_4 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0240UL: // (UCM_REG_LCID_CAM_5: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_5 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0244UL: // (UCM_REG_LCID_CAM_6: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_6 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0248UL: // (UCM_REG_LCID_CAM_7: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_7 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe024cUL: // (UCM_REG_LCID_CAM_8: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_8 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0250UL: // (UCM_REG_LCID_CAM_9: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_9 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0254UL: // (UCM_REG_LCID_CAM_10: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_10 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0258UL: // (UCM_REG_LCID_CAM_11: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_11 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe025cUL: // (UCM_REG_LCID_CAM_12: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_12 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0260UL: // (UCM_REG_LCID_CAM_13: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_13 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0264UL: // (UCM_REG_LCID_CAM_14: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_14 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0268UL: // (UCM_REG_LCID_CAM_15: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_15 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe026cUL: // (UCM_REG_LCID_CAM_16: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_16 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0270UL: // (UCM_REG_LCID_CAM_17: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nUCM_REG_LCID_CAM_17 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0280UL: // (UCM_REG_XX_DESCR_TABLE: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nUCM_REG_XX_DESCR_TABLE (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: Indirect access to the descriptor table of the XX protection\n     mechanism. The fields are:[5:0] - message length;[14:6] - message pointer\n    ;[19:15] - next pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0300UL: // (UCM_REG_XX_TABLE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUCM_REG_XX_TABLE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Indirect access to the XX table of the XX protection mechanism\n    . The fields are: [4:0] - tail pointer;[10:5] - Link List size;[15:11] - \n    header pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe1000UL: // (UCM_REG_XX_PEND_MSG: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUCM_REG_XX_PEND_MSG (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indirect access to the Pending messages RAM of the XX protection\n     mechanism. Bit [0] stands for one of two REGs in the row; bits [9:1] \n    stand for the row in the memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe2000UL: // (UCM_REG_AG_CTX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUCM_REG_AG_CTX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indirect access to AG context with 32-bits granularity. The\n     bits [10:8] of the address should be the offset within the accessed LCID\n     context; the bits [7:0] are the accessed LCID.Example: to write to REG10\n     LCID100. The RBC address should be 12'ha64.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xf0000UL: // (UCM_REG_STORM_CTX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUCM_REG_STORM_CTX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indirect access to STORM context with 32-bits granularity\n    . The bits [13:8] of the address should be the offset within the accessed\n     LCID context; the bits [7:0] are the accessed LCID.Example: to write \n    to REG10 LCID100. The RBC address should be 13'ha64.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe027cUL: // (UCM_REG_UCM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUCM_REG_UCM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0380UL: // (UCM_REG_UCM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUCM_REG_UCM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe0404UL: // (UCM_REG_UCM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUCM_REG_UCM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xe4000UL: // (UCM_REG_UCM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUCM_REG_UCM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x4UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x4UL): // (UMAC_REG_IPG_HD_BKP_CNTL: ACCESS:RW  DataWidth:0x7  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_IPG_HD_BKP_CNTL (ACCESS:RW  DataWidth:0x7  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tIPG_CONFIG_RX :"
				, (u32_t)((data & UMAC_IPG_HD_BKP_CNTL_REG_IPG_CONFIG_RX) >> 2)
				, "\n\tHD_FC_BKOFF_OK :"
				, (u32_t)((data & UMAC_IPG_HD_BKP_CNTL_REG_HD_FC_BKOFF_OK) >> 1)
				, "\n\tHD_FC_ENA :"
				, (u32_t)((data & UMAC_IPG_HD_BKP_CNTL_REG_HD_FC_ENA) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x8UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x8UL): // (UMAC_REG_COMMAND_CONFIG: ACCESS:RW  DataWidth:0x1f  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_COMMAND_CONFIG (ACCESS:RW  DataWidth:0x1f  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRUNT_FILTER_DIS :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_RUNT_FILTER_DIS) >> 30)
				, "\n\tUNUSED_1 :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_UNUSED_1) >> 29)
				, "\n\tIGNORE_TX_PAUSE :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_IGNORE_TX_PAUSE) >> 28)
				, "\n\tPRBL_ENA :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_PRBL_ENA) >> 27)
				, "\n\tRX_ERR_DISC :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_RX_ERR_DISC) >> 26)
				, "\n\tLINE_LOOPBACK :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_LINE_LOOPBACK) >> 25)
				, "\n\tNO_LGTH_CHECK :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_NO_LGTH_CHECK) >> 24)
				, "\n\tCNTL_FRM_ENA :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_CNTL_FRM_ENA) >> 23)
				, "\n\tENA_EXT_CONFIG :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_ENA_EXT_CONFIG) >> 22)
				, "\n\tEN_INTERNAL_TX_CRS :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_EN_INTERNAL_TX_CRS) >> 21)
				, "\n\tUNUSED_0 :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_UNUSED_0) >> 19)
				, "\n\tSW_OVERRIDE_RX :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_SW_OVERRIDE_RX) >> 18)
				, "\n\tSW_OVERRIDE_TX :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_SW_OVERRIDE_TX) >> 17)
				, "\n\tMAC_LOOP_CON :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_MAC_LOOP_CON) >> 16)
				, "\n\tLOOP_ENA :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_LOOP_ENA) >> 15)
				, "\n\tFCS_CORRUPT_URUN_EN :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_FCS_CORRUPT_URUN_EN) >> 14)
				, "\n\tSW_RESET :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_SW_RESET) >> 13)
				, "\n\tOVERFLOW_EN :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_OVERFLOW_EN) >> 12)
				, "\n\tRX_LOW_LATENCY_EN :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_RX_LOW_LATENCY_EN) >> 11)
				, "\n\tHD_ENA :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_HD_ENA) >> 10)
				, "\n\tTX_ADDR_INS :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_TX_ADDR_INS) >> 9)
				, "\n\tPAUSE_IGNORE :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_PAUSE_IGNORE) >> 8)
				, "\n\tPAUSE_FWD :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_PAUSE_FWD) >> 7)
				, "\n\tCRC_FWD :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_CRC_FWD) >> 6)
				, "\n\tPAD_EN :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_PAD_EN) >> 5)
				, "\n\tPROMIS_EN :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_PROMIS_EN) >> 4)
				, "\n\tETH_SPEED :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_ETH_SPEED) >> 2)
				, "\n\tRX_ENA :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_RX_ENA) >> 1)
				, "\n\tTX_ENA :"
				, (u32_t)((data & UMAC_COMMAND_CONFIG_REG_TX_ENA) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0xcUL): // (2-port) 
		case (GRCBASE_UMAC1 + 0xcUL): // (UMAC_REG_MAC_ADDR0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUMAC_REG_MAC_ADDR0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Register Bit 0 refers to Bit 16 of the MAC address; Bit 1\n     refers to bit 17 of the MAC address etc. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x10UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x10UL): // (UMAC_REG_MAC_ADDR1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUMAC_REG_MAC_ADDR1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Register Bit 0 refers to Bit 0 of the MAC address; Register\n     Bit 1 refers to Bit 1 of the MAC address etc. Bits 16 to 31 are reserved\n    . \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x14UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x14UL): // (UMAC_REG_MAXFR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nUMAC_REG_MAXFR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Defines a 14-Bit maximum frame length used by the MAC receive\n     logic to check frames. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x18UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x18UL): // (UMAC_REG_STAD2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUMAC_REG_STAD2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: 16-Bit value; sets; in increment of 512 Ethernet bit times\n    ; the pause quanta used in each Pause Frame sent to the remote Ethernet\n     device. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x3cUL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x3cUL): // (UMAC_REG_TX_TS_SEQ_ID: ACCESS:RW  DataWidth:0x11  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_TX_TS_SEQ_ID (ACCESS:RW  DataWidth:0x11  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTSTS_VALID :"
				, (u32_t)((data & UMAC_TX_TS_SEQ_ID_REG_TSTS_VALID) >> 16)
				, "\n\tTSTS_SEQ_ID :"
				, (u32_t)((data & UMAC_TX_TS_SEQ_ID_REG_TSTS_SEQ_ID) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x40UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x40UL): // (UMAC_REG_SFD_OFFSET: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUMAC_REG_SFD_OFFSET (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Defines the length of the EFM preamble between 5 and 15 Bytes\n    . When set to 0; 1; 2; 3 or 4; the Preamble EFM length is set to 5 Bytes\n    . \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x44UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x44UL): // (UMAC_REG_MAC_MODE: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_MAC_MODE (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLINK_STATUS :"
				, (u32_t)((data & UMAC_MAC_MODE_REG_LINK_STATUS) >> 5)
				, "\n\tMAC_TX_PAUSE :"
				, (u32_t)((data & UMAC_MAC_MODE_REG_MAC_TX_PAUSE) >> 4)
				, "\n\tMAC_RX_PAUSE :"
				, (u32_t)((data & UMAC_MAC_MODE_REG_MAC_RX_PAUSE) >> 3)
				, "\n\tMAC_DUPLEX :"
				, (u32_t)((data & UMAC_MAC_MODE_REG_MAC_DUPLEX) >> 2)
				, "\n\tMAC_SPEED :"
				, (u32_t)((data & UMAC_MAC_MODE_REG_MAC_SPEED) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x48UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x48UL): // (UMAC_REG_TAG_0: ACCESS:RW  DataWidth:0x11  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_TAG_0 (ACCESS:RW  DataWidth:0x11  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCONFIG_OUTER_TPID_ENABLE :"
				, (u32_t)((data & UMAC_TAG_0_REG_CONFIG_OUTER_TPID_ENABLE) >> 16)
				, "\n\tFRM_TAG_0 :"
				, (u32_t)((data & UMAC_TAG_0_REG_FRM_TAG_0) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x4cUL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x4cUL): // (UMAC_REG_TAG_1: ACCESS:RW  DataWidth:0x11  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_TAG_1 (ACCESS:RW  DataWidth:0x11  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCONFIG_INNER_TPID_ENABLE :"
				, (u32_t)((data & UMAC_TAG_1_REG_CONFIG_INNER_TPID_ENABLE) >> 16)
				, "\n\tFRM_TAG_1 :"
				, (u32_t)((data & UMAC_TAG_1_REG_FRM_TAG_1) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x50UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x50UL): // (UMAC_REG_RX_PAUSE_QUANTA_SCALE: ACCESS:RW  DataWidth:0x12  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_RX_PAUSE_QUANTA_SCALE (ACCESS:RW  DataWidth:0x12  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSCALE_FIX :"
				, (u32_t)((data & UMAC_RX_PAUSE_QUANTA_SCALE_REG_SCALE_FIX) >> 17)
				, "\n\tSCALE_CONTROL :"
				, (u32_t)((data & UMAC_RX_PAUSE_QUANTA_SCALE_REG_SCALE_CONTROL) >> 16)
				, "\n\tSCALE_VALUE :"
				, (u32_t)((data & UMAC_RX_PAUSE_QUANTA_SCALE_REG_SCALE_VALUE) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x54UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x54UL): // (UMAC_REG_TX_PREAMBLE: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUMAC_REG_TX_PREAMBLE (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Set the transmit preamble excluding SFD to be programmable\n     from min of 2 bytes to the max allowable of 7 bytes; with granularity\n     of 1 byte. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x5cUL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x5cUL): // (UMAC_REG_TX_IPG_LENGTH: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nUMAC_REG_TX_IPG_LENGTH (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Set the Transmit minimum IPG from 8 to 64 Byte-times. If \n    a value below 8 or above 64 is programmed; the minimum IPG is set to 12\n     byte-times. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x60UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x60UL): // (UMAC_REG_PFC_XOFF_TIMER: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUMAC_REG_PFC_XOFF_TIMER (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Time value sent in the Timer Field for classes in XOFF state\n     (Unit is 512 bit-times). \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x64UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x64UL): // (UMAC_REG_UMAC_EEE_CTRL: ACCESS:RW  DataWidth:0x8  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_UMAC_EEE_CTRL (ACCESS:RW  DataWidth:0x8  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLP_IDLE_PREDICTION_MODE :"
				, (u32_t)((data & UMAC_UMAC_EEE_CTRL_REG_LP_IDLE_PREDICTION_MODE) >> 7)
				, "\n\tDIS_EEE_10M :"
				, (u32_t)((data & UMAC_UMAC_EEE_CTRL_REG_DIS_EEE_10M) >> 6)
				, "\n\tEEE_TXCLK_DIS :"
				, (u32_t)((data & UMAC_UMAC_EEE_CTRL_REG_EEE_TXCLK_DIS) >> 5)
				, "\n\tRX_FIFO_CHECK :"
				, (u32_t)((data & UMAC_UMAC_EEE_CTRL_REG_RX_FIFO_CHECK) >> 4)
				, "\n\tEEE_EN :"
				, (u32_t)((data & UMAC_UMAC_EEE_CTRL_REG_EEE_EN) >> 3)
				, "\n\tUNUSED_2 :"
				, (u32_t)((data & UMAC_UMAC_EEE_CTRL_REG_UNUSED_2) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x68UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x68UL): // (UMAC_REG_EEE_LPI_TIMER: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUMAC_REG_EEE_LPI_TIMER (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This is the duration for which condition to move to LPI state\n     must be satisfied; at the end of which MAC transitions to LPI State. \n    The decrement unit is 1 micro-second. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x6cUL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x6cUL): // (UMAC_REG_EEE_WAKE_TIMER: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUMAC_REG_EEE_WAKE_TIMER (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: This is the duration for which MAC must wait to go back to\n     ACTIVE state from LPI state when it receives packet for transmission. \n    The decrement unit is 1 micro-second. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x74UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x74UL): // (UMAC_REG_UMAC_TIMESTAMP_ADJUST: ACCESS:RW  DataWidth:0xb  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_UMAC_TIMESTAMP_ADJUST (ACCESS:RW  DataWidth:0xb  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tAUTO_ADJUST :"
				, (u32_t)((data & UMAC_UMAC_TIMESTAMP_ADJUST_REG_AUTO_ADJUST) >> 10)
				, "\n\tEN_1588 :"
				, (u32_t)((data & UMAC_UMAC_TIMESTAMP_ADJUST_REG_EN_1588) >> 9)
				, "\n\tADJUST :"
				, (u32_t)((data & UMAC_UMAC_TIMESTAMP_ADJUST_REG_ADJUST) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x78UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x78UL): // (UMAC_REG_RX_IPG_INVAL: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUMAC_REG_RX_IPG_INVAL (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug status; set if MAC receives an IPG less than programmed\n     RX IPG or less than four bytes. Sticky bit. Clears when SW writes 0 into\n     the field or by sw_reset. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x7cUL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x7cUL): // (UMAC_REG_THRESHOLD_VALUE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUMAC_REG_THRESHOLD_VALUE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: If LPI_Prediction is enabled then this register defines the\n     number of IDLEs to be received by the UniMAC before allowing LP_IDLE \n    to be sent to Link Partner. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x300UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x300UL): // (UMAC_REG_PFC_ETH_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUMAC_REG_PFC_ETH_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: These 16 bits are for programmable ethertype in PFC. Since\n     PFC is not standardized yet; the ethertype must be programmable to make\n     sure that when it gets standardized; we can be standards compliant. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x304UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x304UL): // (UMAC_REG_PFC_OPCODE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUMAC_REG_PFC_OPCODE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: These 16 bits are for opcode. Since PFC is not standardized\n     yet; the opcode must be programmable to make sure that when it gets standardized\n    ; we can be standards compliant. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x308UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x308UL): // (UMAC_REG_PFC_MACDA_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUMAC_REG_PFC_MACDA_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Lower 32 bits of programmable DA for PPP. Since PPP is not\n     standardized yet; the DA must be programmable to make sure that when \n    it gets standardized; we can be standards compliant. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x30cUL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x30cUL): // (UMAC_REG_PFC_MACDA_1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUMAC_REG_PFC_MACDA_1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Upper 16 bits of programmable DA for PPP. Since PPP is not\n     standardized yet; the DA must be programmable to make sure that when \n    it gets standardized; we can be standards compliant. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x310UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x310UL): // (UMAC_REG_MACSEC_PROG_TX_CRC: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUMAC_REG_MACSEC_PROG_TX_CRC (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The transmitted CRC can be corrupted by replacing the FCS\n     of the transmitted frame by the FCS programmed in this register. This\n     is enabled and controlled by MACSEC_CNTRL register. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x314UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x314UL): // (UMAC_REG_MACSEC_CNTRL: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_MACSEC_CNTRL (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDIS_PAUSE_DATA_VAR_IPG :"
				, (u32_t)((data & UMAC_MACSEC_CNTRL_REG_DIS_PAUSE_DATA_VAR_IPG) >> 3)
				, "\n\tTX_CRC_PROGRAM :"
				, (u32_t)((data & UMAC_MACSEC_CNTRL_REG_TX_CRC_PROGRAM) >> 2)
				, "\n\tTX_CRC_CORUPT_EN :"
				, (u32_t)((data & UMAC_MACSEC_CNTRL_REG_TX_CRC_CORUPT_EN) >> 1)
				, "\n\tTX_LAUNCH_EN :"
				, (u32_t)((data & UMAC_MACSEC_CNTRL_REG_TX_LAUNCH_EN) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x318UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x318UL): // (UMAC_REG_TS_STATUS_CNTRL: ACCESS:RW  DataWidth:0x5  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_TS_STATUS_CNTRL (ACCESS:RW  DataWidth:0x5  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWORD_AVAIL :"
				, (u32_t)((data & UMAC_TS_STATUS_CNTRL_REG_WORD_AVAIL) >> 2)
				, "\n\tTX_TS_FIFO_EMPTY :"
				, (u32_t)((data & UMAC_TS_STATUS_CNTRL_REG_TX_TS_FIFO_EMPTY) >> 1)
				, "\n\tTX_TS_FIFO_FULL :"
				, (u32_t)((data & UMAC_TS_STATUS_CNTRL_REG_TX_TS_FIFO_FULL) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x31cUL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x31cUL): // (UMAC_REG_TX_TS_DATA: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUMAC_REG_TX_TS_DATA (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Every read of this register will fetch out one timestamp \n    value corresponding to the preceding seq_id read from the transmit FIFO\n    . Every 49 bit; val_bit + seq_id + timestamp is read in two steps; i.e\n    .; one read from 0x10f (val_bit + seq_id) followed by another read from\n     0x1c7 (timestamp). Timestamp read without a preceding seq_id read will\n     fetch stale timestamp value. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x330UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x330UL): // (UMAC_REG_PAUSE_CONTROL: ACCESS:RW  DataWidth:0x12  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_PAUSE_CONTROL (ACCESS:RW  DataWidth:0x12  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tENABLE :"
				, (u32_t)((data & UMAC_PAUSE_CONTROL_REG_ENABLE) >> 17)
				, "\n\tVALUE :"
				, (u32_t)((data & UMAC_PAUSE_CONTROL_REG_VALUE) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x334UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x334UL): // (UMAC_REG_FLUSH: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUMAC_REG_FLUSH (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Flush enable bit to drop out all packets in Tx FIFO without\n     egressing any packets when set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x338UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x338UL): // (UMAC_REG_RXFIFO_STAT: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_RXFIFO_STAT (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRXFIFO_OVERRUN :"
				, (u32_t)((data & UMAC_RXFIFO_STAT_REG_RXFIFO_OVERRUN) >> 1)
				, "\n\tRXFIFO_UNDERRUN :"
				, (u32_t)((data & UMAC_RXFIFO_STAT_REG_RXFIFO_UNDERRUN) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x33cUL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x33cUL): // (UMAC_REG_TXFIFO_STAT: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_TXFIFO_STAT (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTXFIFO_OVERRUN :"
				, (u32_t)((data & UMAC_TXFIFO_STAT_REG_TXFIFO_OVERRUN) >> 1)
				, "\n\tTXFIFO_UNDERRUN :"
				, (u32_t)((data & UMAC_TXFIFO_STAT_REG_TXFIFO_UNDERRUN) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x340UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x340UL): // (UMAC_REG_MAC_PFC_CTRL: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_MAC_PFC_CTRL (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPFC_STATS_EN :"
				, (u32_t)((data & UMAC_MAC_PFC_CTRL_REG_PFC_STATS_EN) >> 5)
				, "\n\tRX_PASS_PFC_FRM :"
				, (u32_t)((data & UMAC_MAC_PFC_CTRL_REG_RX_PASS_PFC_FRM) >> 4)
				, "\n\tUNUSED_3 :"
				, (u32_t)((data & UMAC_MAC_PFC_CTRL_REG_UNUSED_3) >> 3)
				, "\n\tFORCE_PFC_XON :"
				, (u32_t)((data & UMAC_MAC_PFC_CTRL_REG_FORCE_PFC_XON) >> 2)
				, "\n\tPFC_RX_ENBL :"
				, (u32_t)((data & UMAC_MAC_PFC_CTRL_REG_PFC_RX_ENBL) >> 1)
				, "\n\tPFC_TX_ENBL :"
				, (u32_t)((data & UMAC_MAC_PFC_CTRL_REG_PFC_TX_ENBL) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x344UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x344UL): // (UMAC_REG_MAC_PFC_REFRESH_CTRL: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nUMAC_REG_MAC_PFC_REFRESH_CTRL (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPFC_REFRESH_TIMER :"
				, (u32_t)((data & UMAC_MAC_PFC_REFRESH_CTRL_REG_PFC_REFRESH_TIMER) >> 16)
				, "\n\tUNUSED_4 :"
				, (u32_t)((data & UMAC_MAC_PFC_REFRESH_CTRL_REG_UNUSED_4) >> 1)
				, "\n\tPFC_REFRESH_EN :"
				, (u32_t)((data & UMAC_MAC_PFC_REFRESH_CTRL_REG_PFC_REFRESH_EN) >> 0));
			break;
		case (GRCBASE_UMAC0 + 0x1cUL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x1cUL): // (UMAC_REG_UMAC_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUMAC_REG_UMAC_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x58UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x58UL): // (UMAC_REG_UMAC_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUMAC_REG_UMAC_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x80UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x80UL): // (UMAC_REG_UMAC_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUMAC_REG_UMAC_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x320UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x320UL): // (UMAC_REG_UMAC_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUMAC_REG_UMAC_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_UMAC0 + 0x348UL): // (2-port) 
		case (GRCBASE_UMAC1 + 0x348UL): // (UMAC_REG_UMAC_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUMAC_REG_UMAC_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4000UL: // (USDM_REG_TIMER_TICK: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_TIMER_TICK (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Tick for timer counter. Applicable only when ~usdm_registers_timer_tick_enable\n    .timer_tick_enable =1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4004UL: // (USDM_REG_TIMERS_TICK_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_TIMERS_TICK_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for tick counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4008UL: // (USDM_REG_CFC_RSP_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nUSDM_REG_CFC_RSP_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the  internal RAM for the cfc_rsp lcid\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc400cUL: // (USDM_REG_CMP_COUNTER_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nUSDM_REG_CMP_COUNTER_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal  RAM for the completion\n     counters.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4010UL: // (USDM_REG_Q_COUNTER_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nUSDM_REG_Q_COUNTER_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal RAM for queue counters\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4014UL: // (USDM_REG_PCK_END_MSG_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nUSDM_REG_PCK_END_MSG_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal RAM for the packet end \n    message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4018UL: // (USDM_REG_COUNTERS_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_COUNTERS_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates if the 204 statistics counters should stop counting\n     when reaching an all-ones value or should wrap-around 0=stop counting\n     1=wrap-around.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc401cUL: // (USDM_REG_CMP_COUNTER_MAX0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUSDM_REG_CMP_COUNTER_MAX0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4020UL: // (USDM_REG_CMP_COUNTER_MAX1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUSDM_REG_CMP_COUNTER_MAX1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4024UL: // (USDM_REG_CMP_COUNTER_MAX2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUSDM_REG_CMP_COUNTER_MAX2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4028UL: // (USDM_REG_CMP_COUNTER_MAX3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUSDM_REG_CMP_COUNTER_MAX3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc402cUL: // (USDM_REG_BRB1_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_BRB1_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from BRB1 in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4030UL: // (USDM_REG_PXP_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_PXP_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from pxp in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4034UL: // (USDM_REG_PB_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_PB_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from PB in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4038UL: // (USDM_REG_AGG_INT_EVENT_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc403cUL: // (USDM_REG_AGG_INT_EVENT_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4040UL: // (USDM_REG_AGG_INT_EVENT_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4044UL: // (USDM_REG_AGG_INT_EVENT_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4048UL: // (USDM_REG_AGG_INT_EVENT_4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc404cUL: // (USDM_REG_AGG_INT_EVENT_5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4050UL: // (USDM_REG_AGG_INT_EVENT_6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4054UL: // (USDM_REG_AGG_INT_EVENT_7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4058UL: // (USDM_REG_AGG_INT_EVENT_8: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_8 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc405cUL: // (USDM_REG_AGG_INT_EVENT_9: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_9 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4060UL: // (USDM_REG_AGG_INT_EVENT_10: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_10 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4064UL: // (USDM_REG_AGG_INT_EVENT_11: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_11 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4068UL: // (USDM_REG_AGG_INT_EVENT_12: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_12 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc406cUL: // (USDM_REG_AGG_INT_EVENT_13: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_13 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4070UL: // (USDM_REG_AGG_INT_EVENT_14: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_14 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4074UL: // (USDM_REG_AGG_INT_EVENT_15: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_15 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4078UL: // (USDM_REG_AGG_INT_EVENT_16: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_16 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc407cUL: // (USDM_REG_AGG_INT_EVENT_17: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_17 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4080UL: // (USDM_REG_AGG_INT_EVENT_18: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_18 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4084UL: // (USDM_REG_AGG_INT_EVENT_19: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_19 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4088UL: // (USDM_REG_AGG_INT_EVENT_20: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_20 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc408cUL: // (USDM_REG_AGG_INT_EVENT_21: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_21 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4090UL: // (USDM_REG_AGG_INT_EVENT_22: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_22 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4094UL: // (USDM_REG_AGG_INT_EVENT_23: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_23 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4098UL: // (USDM_REG_AGG_INT_EVENT_24: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_24 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc409cUL: // (USDM_REG_AGG_INT_EVENT_25: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_25 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40a0UL: // (USDM_REG_AGG_INT_EVENT_26: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_26 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40a4UL: // (USDM_REG_AGG_INT_EVENT_27: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_27 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40a8UL: // (USDM_REG_AGG_INT_EVENT_28: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_28 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40acUL: // (USDM_REG_AGG_INT_EVENT_29: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_29 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40b0UL: // (USDM_REG_AGG_INT_EVENT_30: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_30 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40b4UL: // (USDM_REG_AGG_INT_EVENT_31: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_EVENT_31 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40b8UL: // (USDM_REG_AGG_INT_T_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40bcUL: // (USDM_REG_AGG_INT_T_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40c0UL: // (USDM_REG_AGG_INT_T_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40c4UL: // (USDM_REG_AGG_INT_T_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40c8UL: // (USDM_REG_AGG_INT_T_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40ccUL: // (USDM_REG_AGG_INT_T_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40d0UL: // (USDM_REG_AGG_INT_T_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40d4UL: // (USDM_REG_AGG_INT_T_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40d8UL: // (USDM_REG_AGG_INT_T_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40dcUL: // (USDM_REG_AGG_INT_T_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40e0UL: // (USDM_REG_AGG_INT_T_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40e4UL: // (USDM_REG_AGG_INT_T_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40e8UL: // (USDM_REG_AGG_INT_T_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40ecUL: // (USDM_REG_AGG_INT_T_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40f0UL: // (USDM_REG_AGG_INT_T_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40f4UL: // (USDM_REG_AGG_INT_T_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40f8UL: // (USDM_REG_AGG_INT_T_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc40fcUL: // (USDM_REG_AGG_INT_T_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4100UL: // (USDM_REG_AGG_INT_T_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4104UL: // (USDM_REG_AGG_INT_T_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4108UL: // (USDM_REG_AGG_INT_T_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc410cUL: // (USDM_REG_AGG_INT_T_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4110UL: // (USDM_REG_AGG_INT_T_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4114UL: // (USDM_REG_AGG_INT_T_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4118UL: // (USDM_REG_AGG_INT_T_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc411cUL: // (USDM_REG_AGG_INT_T_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4120UL: // (USDM_REG_AGG_INT_T_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4124UL: // (USDM_REG_AGG_INT_T_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4128UL: // (USDM_REG_AGG_INT_T_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc412cUL: // (USDM_REG_AGG_INT_T_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4130UL: // (USDM_REG_AGG_INT_T_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4134UL: // (USDM_REG_AGG_INT_T_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_T_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4138UL: // (USDM_REG_AGG_INT_FIC_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc413cUL: // (USDM_REG_AGG_INT_FIC_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4140UL: // (USDM_REG_AGG_INT_FIC_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4144UL: // (USDM_REG_AGG_INT_FIC_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4148UL: // (USDM_REG_AGG_INT_FIC_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc414cUL: // (USDM_REG_AGG_INT_FIC_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4150UL: // (USDM_REG_AGG_INT_FIC_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4154UL: // (USDM_REG_AGG_INT_FIC_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4158UL: // (USDM_REG_AGG_INT_FIC_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc415cUL: // (USDM_REG_AGG_INT_FIC_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4160UL: // (USDM_REG_AGG_INT_FIC_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4164UL: // (USDM_REG_AGG_INT_FIC_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4168UL: // (USDM_REG_AGG_INT_FIC_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc416cUL: // (USDM_REG_AGG_INT_FIC_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4170UL: // (USDM_REG_AGG_INT_FIC_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4174UL: // (USDM_REG_AGG_INT_FIC_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4178UL: // (USDM_REG_AGG_INT_FIC_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc417cUL: // (USDM_REG_AGG_INT_FIC_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4180UL: // (USDM_REG_AGG_INT_FIC_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4184UL: // (USDM_REG_AGG_INT_FIC_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4188UL: // (USDM_REG_AGG_INT_FIC_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc418cUL: // (USDM_REG_AGG_INT_FIC_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4190UL: // (USDM_REG_AGG_INT_FIC_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4194UL: // (USDM_REG_AGG_INT_FIC_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4198UL: // (USDM_REG_AGG_INT_FIC_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc419cUL: // (USDM_REG_AGG_INT_FIC_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41a0UL: // (USDM_REG_AGG_INT_FIC_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41a4UL: // (USDM_REG_AGG_INT_FIC_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41a8UL: // (USDM_REG_AGG_INT_FIC_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41acUL: // (USDM_REG_AGG_INT_FIC_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41b0UL: // (USDM_REG_AGG_INT_FIC_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41b4UL: // (USDM_REG_AGG_INT_FIC_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_FIC_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41b8UL: // (USDM_REG_AGG_INT_MODE_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41bcUL: // (USDM_REG_AGG_INT_MODE_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41c0UL: // (USDM_REG_AGG_INT_MODE_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41c4UL: // (USDM_REG_AGG_INT_MODE_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41c8UL: // (USDM_REG_AGG_INT_MODE_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41ccUL: // (USDM_REG_AGG_INT_MODE_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41d0UL: // (USDM_REG_AGG_INT_MODE_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41d4UL: // (USDM_REG_AGG_INT_MODE_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41d8UL: // (USDM_REG_AGG_INT_MODE_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41dcUL: // (USDM_REG_AGG_INT_MODE_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41e0UL: // (USDM_REG_AGG_INT_MODE_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41e4UL: // (USDM_REG_AGG_INT_MODE_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41e8UL: // (USDM_REG_AGG_INT_MODE_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41ecUL: // (USDM_REG_AGG_INT_MODE_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41f0UL: // (USDM_REG_AGG_INT_MODE_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41f4UL: // (USDM_REG_AGG_INT_MODE_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41f8UL: // (USDM_REG_AGG_INT_MODE_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (16) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc41fcUL: // (USDM_REG_AGG_INT_MODE_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (17)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4200UL: // (USDM_REG_AGG_INT_MODE_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4204UL: // (USDM_REG_AGG_INT_MODE_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4208UL: // (USDM_REG_AGG_INT_MODE_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc420cUL: // (USDM_REG_AGG_INT_MODE_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4210UL: // (USDM_REG_AGG_INT_MODE_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4214UL: // (USDM_REG_AGG_INT_MODE_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4218UL: // (USDM_REG_AGG_INT_MODE_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc421cUL: // (USDM_REG_AGG_INT_MODE_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4220UL: // (USDM_REG_AGG_INT_MODE_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4224UL: // (USDM_REG_AGG_INT_MODE_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4228UL: // (USDM_REG_AGG_INT_MODE_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc422cUL: // (USDM_REG_AGG_INT_MODE_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4230UL: // (USDM_REG_AGG_INT_MODE_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4234UL: // (USDM_REG_AGG_INT_MODE_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_AGG_INT_MODE_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4238UL: // (USDM_REG_ENABLE_IN1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nUSDM_REG_ENABLE_IN1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tQM_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_QM_EXT_WR_FULL_IN_EN) >> 26)
				, "\n\tCFC_PB_ACK_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_CFC_PB_ACK_IN_EN) >> 25)
				, "\n\tCFC_ACDEC_ACK_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_CFC_ACDEC_ACK_IN_EN) >> 24)
				, "\n\tCFC_ACINC_ACK_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_CFC_ACINC_ACK_IN_EN) >> 23)
				, "\n\tCFC_LOAD_RSP_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_CFC_LOAD_RSP_IN_EN) >> 22)
				, "\n\tCFC_LOAD_ACK_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_CFC_LOAD_ACK_IN_EN) >> 21)
				, "\n\tPXP_REQ_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PXP_REQ_IN_EN) >> 20)
				, "\n\tSDM_WAKE_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_SDM_WAKE_IN_EN) >> 19)
				, "\n\tPRS_MSG_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PRS_MSG_IN_EN) >> 18)
				, "\n\tPB_DATA_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PB_DATA_IN_EN) >> 17)
				, "\n\tBRB1_DP_DATA_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_BRB1_DP_DATA_IN_EN) >> 16)
				, "\n\tBRB1_CTRL_DATA_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_BRB1_CTRL_DATA_IN_EN) >> 15)
				, "\n\tPXP_DP_ACK_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PXP_DP_ACK_IN_EN) >> 14)
				, "\n\tPXP_CTRL_ACK_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PXP_CTRL_ACK_IN_EN) >> 13)
				, "\n\tPXP_DP_DATA_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PXP_DP_DATA_IN_EN) >> 12)
				, "\n\tPXP_INT_DONE_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PXP_INT_DONE_IN_EN) >> 11)
				, "\n\tPXP_CTRL_DATA_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PXP_CTRL_DATA_IN_EN) >> 10)
				, "\n\tPXP_DP_FULL_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PXP_DP_FULL_IN_EN) >> 9)
				, "\n\tPXP_CTRL_FULL_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PXP_CTRL_FULL_IN_EN) >> 8)
				, "\n\tPXP_DP_DONE_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PXP_DP_DONE_IN_EN) >> 7)
				, "\n\tPXP_CTRL_DONE_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PXP_CTRL_DONE_IN_EN) >> 6)
				, "\n\tPAS_BUF_FULL_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PAS_BUF_FULL_IN_EN) >> 5)
				, "\n\tPAS_BUF_DONE_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_PAS_BUF_DONE_IN_EN) >> 4)
				, "\n\tINT_RAM_FULL_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_INT_RAM_FULL_IN_EN) >> 3)
				, "\n\tINT_RAM_DONE_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_INT_RAM_DONE_IN_EN) >> 2)
				, "\n\tINT_RAM_DATA_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_INT_RAM_DATA_IN_EN) >> 1)
				, "\n\tEXT_STORE_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN1_REG_EXT_STORE_IN_EN) >> 0));
			break;
		case 0xc423cUL: // (USDM_REG_ENABLE_IN2: ACCESS:RW  DataWidth:0x7  Multi Field Register  ) 
			sprintf(disp_str, "\nUSDM_REG_ENABLE_IN2 (ACCESS:RW  DataWidth:0x7  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_REQ_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN2_REG_DORQ_REQ_IN_EN) >> 6)
				, "\n\tPB_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN2_REG_PB_EXT_WR_FULL_IN_EN) >> 5)
				, "\n\tPBF_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN2_REG_PBF_EXT_WR_FULL_IN_EN) >> 4)
				, "\n\tPB_FULL_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN2_REG_PB_FULL_IN_EN) >> 3)
				, "\n\tPB_STATUS_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN2_REG_PB_STATUS_IN_EN) >> 2)
				, "\n\tCM_ACK_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN2_REG_CM_ACK_IN_EN) >> 1)
				, "\n\tSDM_ACK_IN_EN :"
				, (u32_t)((data & USDM_ENABLE_IN2_REG_SDM_ACK_IN_EN) >> 0));
			break;
		case 0xc4240UL: // (USDM_REG_ENABLE_OUT1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nUSDM_REG_ENABLE_OUT1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPBF_EXT_WR_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PBF_EXT_WR_OUT_EN) >> 26)
				, "\n\tPB_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PB_OUT_EN) >> 25)
				, "\n\tCFC_SDM_ACK_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_CFC_SDM_ACK_OUT_EN) >> 24)
				, "\n\tCM_MSG_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_CM_MSG_OUT_EN) >> 23)
				, "\n\tPXP_REQ_DONE_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PXP_REQ_DONE_OUT_EN) >> 22)
				, "\n\tEXT_FULL_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_EXT_FULL_OUT_EN) >> 21)
				, "\n\tPXP_CTRL_FULL_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PXP_CTRL_FULL_OUT_EN) >> 20)
				, "\n\tPB_FULL_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PB_FULL_OUT_EN) >> 19)
				, "\n\tBRB1_DP_FULL_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_BRB1_DP_FULL_OUT_EN) >> 18)
				, "\n\tBRB1_CTRL_FULL_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_BRB1_CTRL_FULL_OUT_EN) >> 17)
				, "\n\tPXP_DP_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PXP_DP_OUT_EN) >> 16)
				, "\n\tPXP_CTRL_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PXP_CTRL_OUT_EN) >> 15)
				, "\n\tPXP_ASYNC_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PXP_ASYNC_OUT_EN) >> 14)
				, "\n\tPAS_BUF_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PAS_BUF_OUT_EN) >> 13)
				, "\n\tINT_RAM_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_INT_RAM_OUT_EN) >> 12)
				, "\n\tPRS_ACK_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PRS_ACK_OUT_EN) >> 11)
				, "\n\tPRS_SYNC_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PRS_SYNC_OUT_EN) >> 10)
				, "\n\tBRB1_DP_REQ_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_BRB1_DP_REQ_OUT_EN) >> 9)
				, "\n\tBRB1_CTRL_REQ_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_BRB1_CTRL_REQ_OUT_EN) >> 8)
				, "\n\tPXP_DP_REQ_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PXP_DP_REQ_OUT_EN) >> 7)
				, "\n\tPXP_CTRL_REQ_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PXP_CTRL_REQ_OUT_EN) >> 6)
				, "\n\tCFC_PB_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_CFC_PB_OUT_EN) >> 5)
				, "\n\tCFC_ACDEC_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_CFC_ACDEC_OUT_EN) >> 4)
				, "\n\tCFC_ACINC_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_CFC_ACINC_OUT_EN) >> 3)
				, "\n\tCFC_LOAD_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_CFC_LOAD_OUT_EN) >> 2)
				, "\n\tTHREADREADY_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_THREADREADY_OUT_EN) >> 1)
				, "\n\tPXP_INT_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT1_REG_PXP_INT_OUT_EN) >> 0));
			break;
		case 0xc4244UL: // (USDM_REG_ENABLE_OUT2: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nUSDM_REG_ENABLE_OUT2 (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_REQ_DONE_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT2_REG_DORQ_REQ_DONE_OUT_EN) >> 5)
				, "\n\tVFPF_ERR_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT2_REG_VFPF_ERR_OUT_EN) >> 4)
				, "\n\tSDM_EXT_WR_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT2_REG_SDM_EXT_WR_OUT_EN) >> 3)
				, "\n\tQM_EXT_WR_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT2_REG_QM_EXT_WR_OUT_EN) >> 2)
				, "\n\tDQ_EXT_WR_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT2_REG_DQ_EXT_WR_OUT_EN) >> 1)
				, "\n\tPB_EXT_WR_OUT_EN :"
				, (u32_t)((data & USDM_ENABLE_OUT2_REG_PB_EXT_WR_OUT_EN) >> 0));
			break;
		case 0xc4248UL: // (USDM_REG_NUM_OF_Q0_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q0_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc424cUL: // (USDM_REG_NUM_OF_Q1_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q1_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4250UL: // (USDM_REG_NUM_OF_Q2_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q2_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4254UL: // (USDM_REG_NUM_OF_Q3_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q3_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4258UL: // (USDM_REG_NUM_OF_Q4_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q4_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc425cUL: // (USDM_REG_NUM_OF_Q5_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q5_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4260UL: // (USDM_REG_NUM_OF_Q6_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q6_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4264UL: // (USDM_REG_NUM_OF_Q7_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q7_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4268UL: // (USDM_REG_NUM_OF_Q8_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q8_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc426cUL: // (USDM_REG_NUM_OF_Q9_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q9_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4270UL: // (USDM_REG_NUM_OF_Q10_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q10_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4274UL: // (USDM_REG_NUM_OF_Q11_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_Q11_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4278UL: // (USDM_REG_NUM_OF_PKT_END_MSG: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_PKT_END_MSG (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of packet end messages received from the parser\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc427cUL: // (USDM_REG_NUM_OF_PXP_ASYNC_REQ: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_PXP_ASYNC_REQ (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of requests received from the pxp async if\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4280UL: // (USDM_REG_NUM_OF_ACK_AFTER_PLACE: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_NUM_OF_ACK_AFTER_PLACE (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of ACK after placement messages received \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4284UL: // (USDM_REG_STATISTICS_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_STATISTICS_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM bits for statistics sram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4288UL: // (USDM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from USDM\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc428cUL: // (USDM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from USDM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4290UL: // (USDM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSDM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from USDM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4294UL: // (USDM_REG_USDM_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc4298UL: // (USDM_REG_USDM_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc429cUL: // (USDM_REG_USDM_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc42a0UL: // (USDM_REG_USDM_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0xc42a4UL: // (USDM_REG_USDM_INT_STS_1: ACCESS:R  DataWidth:0xe ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_INT_STS_1 (ACCESS:R  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0xc42a8UL: // (USDM_REG_USDM_INT_STS_CLR_1: ACCESS:RC  DataWidth:0xe ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_CLR_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0xc42acUL: // (USDM_REG_USDM_INT_STS_WR_1: ACCESS:WR  DataWidth:0xe ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_INT_STS_WR_1 (ACCESS:WR  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_STS_WR_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0xc42b0UL: // (USDM_REG_USDM_INT_MASK_1: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_INT_MASK_1 (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & USDM_USDM_INT_MASK_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0xc42b4UL: // (USDM_REG_USDM_PRTY_STS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_PRTY_STS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0xc42b8UL: // (USDM_REG_USDM_PRTY_STS_CLR: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_PRTY_STS_CLR (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0xc42bcUL: // (USDM_REG_USDM_PRTY_STS_WR: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_PRTY_STS_WR (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & USDM_USDM_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0xc42c0UL: // (USDM_REG_USDM_PRTY_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nUSDM_REG_USDM_PRTY_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & USDM_USDM_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0xc42d8UL: // (USDM_REG_CMP_COUNTER_MAX4: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUSDM_REG_CMP_COUNTER_MAX4 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc42dcUL: // (USDM_REG_CMP_COUNTER_MAX5: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUSDM_REG_CMP_COUNTER_MAX5 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc42e0UL: // (USDM_REG_CMP_COUNTER_MAX6: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUSDM_REG_CMP_COUNTER_MAX6 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc42e4UL: // (USDM_REG_CMP_COUNTER_MAX7: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUSDM_REG_CMP_COUNTER_MAX7 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc42e8UL: // (USDM_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSDM_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc42ecUL: // (USDM_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSDM_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc42f0UL: // (USDM_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSDM_REG_CPU_MBIST_MEMCTRL_2_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc42f4UL: // (USDM_REG_CPU_MBIST_MEMCTRL_0_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_CPU_MBIST_MEMCTRL_0_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc42f8UL: // (USDM_REG_CPU_MBIST_MEMCTRL_1_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_CPU_MBIST_MEMCTRL_1_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc42fcUL: // (USDM_REG_CPU_MBIST_MEMCTRL_2_STATUS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_CPU_MBIST_MEMCTRL_2_STATUS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 - mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4578UL: // (USDM_REG_CM_QUEUE_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_CM_QUEUE_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits  CM_QUEUE\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc457cUL: // (USDM_REG_INP_QUEUE_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_INP_QUEUE_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits  INP_QUEUE\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4580UL: // (USDM_REG_FIFOS_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_FIFOS_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits  fifos: BRB1_CTRL[1:0]; BRB1_DP[3:2];PB[5:4];PXP_CTRL\n    [7:6]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4584UL: // (USDM_REG_TIMERS_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUSDM_REG_TIMERS_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits for timers sram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4588UL: // (USDM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSDM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4400UL: // (USDM_REG_AGGREG_INTERRUPT_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_AGGREG_INTERRUPT_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: lsb register of aggregated interrupt in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4404UL: // (USDM_REG_AGGREG_INTERRUPT_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_AGGREG_INTERRUPT_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: msb register of aggregated interrupt  in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4408UL: // (USDM_REG_ASYNC_HOST_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_ASYNC_HOST_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: async fifo empty in sdm_async block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc440cUL: // (USDM_REG_ASYNC_HOST_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_ASYNC_HOST_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: async fifo full in sdm_async block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4410UL: // (USDM_REG_CFC_LOAD_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_CFC_LOAD_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4414UL: // (USDM_REG_CFC_LOAD_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_CFC_LOAD_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load pending fifo full in sdm_cfc block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4418UL: // (USDM_REG_CFC_LOAD_RSP_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_CFC_LOAD_RSP_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load rsp fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc441cUL: // (USDM_REG_CFC_LOAD_RSP_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_CFC_LOAD_RSP_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load rsp fifo full in sdm_cfcblock\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4420UL: // (USDM_REG_CM_DELAY_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_CM_DELAY_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm delay fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4424UL: // (USDM_REG_CM_DELAY_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_CM_DELAY_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm delay fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4428UL: // (USDM_REG_CM_QUEUE_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_CM_QUEUE_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm queue fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc442cUL: // (USDM_REG_CM_QUEUE_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_CM_QUEUE_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm queue fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4430UL: // (USDM_REG_DELAY_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DELAY_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: delay FIFO  empty in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4434UL: // (USDM_REG_DELAY_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DELAY_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: delay FIFO  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4438UL: // (USDM_REG_DST_BRB1_CTRL_SRC_ADDR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_BRB1_CTRL_SRC_ADDR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src address fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc443cUL: // (USDM_REG_DST_BRB1_CTRL_SRC_ADDR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_BRB1_CTRL_SRC_ADDR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src address fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4440UL: // (USDM_REG_DST_BRB1_CTRL_SRC_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_BRB1_CTRL_SRC_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4444UL: // (USDM_REG_DST_BRB1_CTRL_SRC_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_BRB1_CTRL_SRC_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4448UL: // (USDM_REG_DST_INT_RAM_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_INT_RAM_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc444cUL: // (USDM_REG_DST_INT_RAM_WAIT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_INT_RAM_WAIT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram_wait fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4450UL: // (USDM_REG_DST_INT_RAM_WAIT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_INT_RAM_WAIT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram_wait fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4454UL: // (USDM_REG_DST_NONE_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_NONE_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: none pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4458UL: // (USDM_REG_DST_NONE_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_NONE_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: none pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc445cUL: // (USDM_REG_DST_PAS_BUF_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PAS_BUF_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4460UL: // (USDM_REG_DST_PAS_BUF_WAIT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PAS_BUF_WAIT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf_wait fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4464UL: // (USDM_REG_DST_PAS_BUF_WAIT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PAS_BUF_WAIT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf_wait fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4468UL: // (USDM_REG_DST_PB_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PB_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc446cUL: // (USDM_REG_DST_PB_IMMED_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PB_IMMED_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb immediate fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4470UL: // (USDM_REG_DST_PB_IMMED_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PB_IMMED_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb immediate fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4474UL: // (USDM_REG_DST_PXP_CTRL_DST_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_CTRL_DST_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_dst pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4478UL: // (USDM_REG_DST_PXP_CTRL_DST_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_CTRL_DST_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_dst pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc447cUL: // (USDM_REG_DST_PXP_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4480UL: // (USDM_REG_DST_PXP_CTRL_IMMED_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_CTRL_IMMED_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl immediate fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4484UL: // (USDM_REG_DST_PXP_CTRL_IMMED_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_CTRL_IMMED_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl immediate fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4488UL: // (USDM_REG_DST_PXP_CTRL_LINK_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_CTRL_LINK_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl link list empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc448cUL: // (USDM_REG_DST_PXP_CTRL_LINK_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_CTRL_LINK_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl link list full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4490UL: // (USDM_REG_DST_PXP_CTRL_SRC_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_CTRL_SRC_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_src pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4494UL: // (USDM_REG_DST_PXP_CTRL_SRC_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_CTRL_SRC_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_src pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4498UL: // (USDM_REG_DST_PXP_DP_DST_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_DP_DST_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp_dst pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc449cUL: // (USDM_REG_DST_PXP_DP_DST_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_DP_DST_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp_dst pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44a0UL: // (USDM_REG_DST_PXP_DP_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_DP_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44a4UL: // (USDM_REG_DST_PXP_DP_LINK_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_DP_LINK_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp link list empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44a8UL: // (USDM_REG_DST_PXP_DP_LINK_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_DST_PXP_DP_LINK_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp link list full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44acUL: // (USDM_REG_INIT_CREDIT_CFC_ACDEC: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_INIT_CREDIT_CFC_ACDEC (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     activity counters interface without receiving any ACK\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44b0UL: // (USDM_REG_INIT_CREDIT_CFC_ACINC: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_INIT_CREDIT_CFC_ACINC (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     activity counters interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44b4UL: // (USDM_REG_INIT_CREDIT_CFC_LOAD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_INIT_CREDIT_CFC_LOAD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     load interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44b8UL: // (USDM_REG_INIT_CREDIT_CFC_PB: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_INIT_CREDIT_CFC_PB (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     activity counters interface 3 without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44bcUL: // (USDM_REG_INIT_CREDIT_CM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_INIT_CREDIT_CM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of cycles that can be sent to the CM  \n    interface without receiving any ACK in CM block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44c0UL: // (USDM_REG_INIT_CREDIT_PXP_CTRL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_INIT_CREDIT_PXP_CTRL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the pxp\n     control interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44c4UL: // (USDM_REG_INIT_CREDIT_PXP_DP: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nUSDM_REG_INIT_CREDIT_PXP_DP (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the pxp\n     dp interface without receiving any ACK\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44c8UL: // (USDM_REG_INT_RAM_RR_REQ: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUSDM_REG_INT_RAM_RR_REQ (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: round robin for int_ram arbiter: b0-pas_buf; b1-int_ram;b2\n    -pxp_dp;b3-pxp_ctrl;b4-brb1_ctrl;b5-brb1_dp;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44ccUL: // (USDM_REG_OPERATION_GEN: ACCESS:W  DataWidth:0x11 ) 
			sprintf(disp_str, "\nUSDM_REG_OPERATION_GEN (ACCESS:W  DataWidth:0x11 )" \
				"\nDescription: Generate an operation after completion; bit-16 is AggVectIdx_valid\n    ; bits 15:8 are AggVectIdx; bits 7:5 are the TRIG and bits 4:0 are the\n     T124Param[4:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44d0UL: // (USDM_REG_PB_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_PB_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: UPB IF  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44d4UL: // (USDM_REG_PBF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_PBF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF if  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44d8UL: // (USDM_REG_PXP_DELAY_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_PXP_DELAY_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp switch delay fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44dcUL: // (USDM_REG_PXP_DELAY_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_PXP_DELAY_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp switch delay fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44e0UL: // (USDM_REG_QM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_QM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: QM IF  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44e4UL: // (USDM_REG_QUEUE_EMPTY: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nUSDM_REG_QUEUE_EMPTY (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Input queue fifo empty in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44e8UL: // (USDM_REG_QUEUE_FULL: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nUSDM_REG_QUEUE_FULL (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Input queue fifo full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44ecUL: // (USDM_REG_RR_CNT_COUNTERS_STATUS: ACCESS:R  DataWidth:0x15 ) 
			sprintf(disp_str, "\nUSDM_REG_RR_CNT_COUNTERS_STATUS (ACCESS:R  DataWidth:0x15 )" \
				"\nDescription: round robin for all completion counters\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44f0UL: // (USDM_REG_RR_COMPLETE_REQ: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nUSDM_REG_RR_COMPLETE_REQ (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: round robin for all completion requests  in sdm_cm block: \n    b0-async b1-nop;b2-pxp_int; b3-timers;b4-dma;b5-grc;b6-rbc\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44f4UL: // (USDM_REG_RR_PTR_REQ: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nUSDM_REG_RR_PTR_REQ (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: round robin for cm pointer: b0-async; b1-dma_dp; b2 - dma_ctrl\n    ; b3-cfc; b4-nop; b5-timers; b6-pxp_int\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44f8UL: // (USDM_REG_RSP_BRB1_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc44fcUL: // (USDM_REG_RSP_BRB1_CTRL_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_CTRL_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4500UL: // (USDM_REG_RSP_BRB1_CTRL_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_CTRL_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4504UL: // (USDM_REG_RSP_BRB1_CTRL_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_CTRL_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4508UL: // (USDM_REG_RSP_BRB1_CTRL_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_CTRL_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc450cUL: // (USDM_REG_RSP_BRB1_DP_DST_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_DP_DST_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending dst fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4510UL: // (USDM_REG_RSP_BRB1_DP_DST_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_DP_DST_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending dst fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4514UL: // (USDM_REG_RSP_BRB1_DP_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_DP_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4518UL: // (USDM_REG_RSP_BRB1_DP_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_DP_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc451cUL: // (USDM_REG_RSP_BRB1_DP_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_DP_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4520UL: // (USDM_REG_RSP_BRB1_DP_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_DP_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4524UL: // (USDM_REG_RSP_BRB1_DP_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_BRB1_DP_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4528UL: // (USDM_REG_RSP_INT_RAM_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_INT_RAM_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc452cUL: // (USDM_REG_RSP_INT_RAM_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_INT_RAM_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4530UL: // (USDM_REG_RSP_INT_RAM_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_INT_RAM_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4534UL: // (USDM_REG_RSP_INT_RAM_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_INT_RAM_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4538UL: // (USDM_REG_RSP_PB_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_PB_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc453cUL: // (USDM_REG_RSP_PB_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_PB_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4540UL: // (USDM_REG_RSP_PB_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_PB_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4544UL: // (USDM_REG_RSP_PB_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_PB_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4548UL: // (USDM_REG_RSP_PB_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_PB_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc454cUL: // (USDM_REG_RSP_PXP_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_PXP_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4550UL: // (USDM_REG_RSP_PXP_CTRL_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4554UL: // (USDM_REG_RSP_PXP_CTRL_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_RSP_PXP_CTRL_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4558UL: // (USDM_REG_SYNC_PARSER_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_SYNC_PARSER_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser fifo  empty in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc455cUL: // (USDM_REG_SYNC_PARSER_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_SYNC_PARSER_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser fifo  full in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4560UL: // (USDM_REG_SYNC_SYNC_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_SYNC_SYNC_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser serial fifo  empty in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4564UL: // (USDM_REG_SYNC_SYNC_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_SYNC_SYNC_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser serial fifo  full in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4568UL: // (USDM_REG_TIMERS_ADDR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_TIMERS_ADDR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: address FIFO  empty in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc456cUL: // (USDM_REG_TIMERS_ADDR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_TIMERS_ADDR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: address FIFO  full in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4570UL: // (USDM_REG_TIMERS_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_TIMERS_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pending FIFO  empty in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4574UL: // (USDM_REG_TIMERS_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSDM_REG_TIMERS_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pending FIFO  full in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4600UL: // (USDM_REG_STATISTICS: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSDM_REG_STATISTICS (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Statistics memory. Each read from RBC resets the corresponding\n     statistic counter\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4800UL: // (USDM_REG_TIMERS: ACCESS:WB  DataWidth:0x34 ) 
			sprintf(disp_str, "\nUSDM_REG_TIMERS (ACCESS:WB  DataWidth:0x34 )" \
				"\nDescription: Debug only. Timers memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc5000UL: // (USDM_REG_CM_QUEUE: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nUSDM_REG_CM_QUEUE (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Debug only. CM queue memory. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc5800UL: // (USDM_REG_INP_QUEUE: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nUSDM_REG_INP_QUEUE (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Debug only. Input queue memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc42c4UL: // (USDM_REG_USDM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSDM_REG_USDM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4300UL: // (USDM_REG_USDM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSDM_REG_USDM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc458cUL: // (USDM_REG_USDM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSDM_REG_USDM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4700UL: // (USDM_REG_USDM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSDM_REG_USDM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xc4a00UL: // (USDM_REG_USDM_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSDM_REG_USDM_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300000UL: // (USEM_REG_MSG_NUM_FIC0: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUSEM_REG_MSG_NUM_FIC0 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that entered \n    through FIC0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300004UL: // (USEM_REG_MSG_NUM_FIC1: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUSEM_REG_MSG_NUM_FIC1 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that entered \n    through FIC1 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300008UL: // (USEM_REG_MSG_NUM_FOC0: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUSEM_REG_MSG_NUM_FOC0 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30000cUL: // (USEM_REG_MSG_NUM_FOC1: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUSEM_REG_MSG_NUM_FOC1 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC1 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300010UL: // (USEM_REG_MSG_NUM_FOC2: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUSEM_REG_MSG_NUM_FOC2 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC2 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300014UL: // (USEM_REG_MSG_NUM_FOC3: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUSEM_REG_MSG_NUM_FOC3 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300018UL: // (USEM_REG_THREAD_INTER_CNT_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_THREAD_INTER_CNT_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for start count of counter ~usem_registers_thread_inter_cnt\n    .thread_inter_cnt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30001cUL: // (USEM_REG_THREAD_INTER_CNT: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nUSEM_REG_THREAD_INTER_CNT (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Maximum value of threads interrupt counter; when it gets \n    this value then interrupt to will be send if thread active from previous\n     maximum value of this counter. This register may be used only when ~usem_registers_thread_inter_cnt_enable\n    .thread_inter_cnt_enable =1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300020UL: // (USEM_REG_ARB_ELEMENT0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_ARB_ELEMENT0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 0. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300024UL: // (USEM_REG_ARB_ELEMENT1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_ARB_ELEMENT1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 1. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~usem_registers_arb_element0.arb_element0\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300028UL: // (USEM_REG_ARB_ELEMENT2: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_ARB_ELEMENT2 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 2. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~usem_registers_arb_element0.arb_element0\n     and ~usem_registers_arb_element1.arb_element1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30002cUL: // (USEM_REG_ARB_ELEMENT3: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_ARB_ELEMENT3 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 3. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2.Could not be equal to register ~usem_registers_arb_element0.arb_element0\n     and ~usem_registers_arb_element1.arb_element1 and ~usem_registers_arb_element2\n    .arb_element2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300030UL: // (USEM_REG_ARB_ELEMENT4: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_ARB_ELEMENT4 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 4. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~usem_registers_arb_element0.arb_element0\n     and ~usem_registers_arb_element1.arb_element1 and ~usem_registers_arb_element2\n    .arb_element2 and ~usem_registers_arb_element3.arb_element3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300038UL: // (USEM_REG_TS_0_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_0_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30003cUL: // (USEM_REG_TS_1_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_1_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300040UL: // (USEM_REG_TS_2_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_2_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300044UL: // (USEM_REG_TS_3_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_3_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300048UL: // (USEM_REG_TS_4_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_4_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30004cUL: // (USEM_REG_TS_5_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_5_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300050UL: // (USEM_REG_TS_6_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_6_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300054UL: // (USEM_REG_TS_7_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_7_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300058UL: // (USEM_REG_TS_8_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_8_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30005cUL: // (USEM_REG_TS_9_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_9_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300060UL: // (USEM_REG_TS_10_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_10_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300064UL: // (USEM_REG_TS_11_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_11_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300068UL: // (USEM_REG_TS_12_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_12_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30006cUL: // (USEM_REG_TS_13_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_13_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300070UL: // (USEM_REG_TS_14_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_14_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300074UL: // (USEM_REG_TS_15_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_15_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300078UL: // (USEM_REG_TS_16_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_16_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30007cUL: // (USEM_REG_TS_17_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_17_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300080UL: // (USEM_REG_TS_18_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_18_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300084UL: // (USEM_REG_TS_19_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_TS_19_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300088UL: // (USEM_REG_FIC0_MIN_MSG_LINES: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC0_MIN_MSG_LINES (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The minimum number of cycles in a message from FIC0 interfaces\n     after which the message can be sent to the passive register_file. It \n    must be always equal to 2; other way it may cause to deadlock in a chip\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30008cUL: // (USEM_REG_FIC1_MIN_MSG_LINES: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC1_MIN_MSG_LINES (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The minimum number of cycles in a message from FIC1 interfaces\n     after which the message can be sent to the passive register_file. It \n    must be always equal to 2; other way it may cause to deadlock in a chip\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300090UL: // (USEM_REG_PASSIVE_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_PASSIVE_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of free entries in the sync FIFO between the external\n     HW and the passive buffer; below which the PassiveFull is asserted \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300094UL: // (USEM_REG_SYNC_DRA_WR_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_SYNC_DRA_WR_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full for sync dra_wr fifo (data from DRA to STORM)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300098UL: // (USEM_REG_SYNC_RAM_WR_ALM_FULL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUSEM_REG_SYNC_RAM_WR_ALM_FULL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Almost full for sync ram_wr fifo (data from EXT_IF to STORM\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30009cUL: // (USEM_REG_DBG_ALM_FULL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUSEM_REG_DBG_ALM_FULL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Almost full for slow debug fifo \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000a0UL: // (USEM_REG_EXCEPTION_INT: ACCESS:RW  DataWidth:0xf ) 
			sprintf(disp_str, "\nUSEM_REG_EXCEPTION_INT (ACCESS:RW  DataWidth:0xf )" \
				"\nDescription: The PRAM address for the interrupt in a case the event ID\n     is bigger then the INT table size. This register is always NA because\n     this feature is removed\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000a4UL: // (USEM_REG_ENABLE_IN: ACCESS:RW  DataWidth:0xf  Multi Field Register  ) 
			sprintf(disp_str, "\nUSEM_REG_ENABLE_IN (ACCESS:RW  DataWidth:0xf  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFPF_ERROR_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_VFPF_ERROR_ENABLE_IN) >> 14)
				, "\n\tWAITP_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_WAITP_ENABLE_IN) >> 13)
				, "\n\tFOC3_ACK_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_FOC3_ACK_ENABLE_IN) >> 12)
				, "\n\tFOC2_ACK_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_FOC2_ACK_ENABLE_IN) >> 11)
				, "\n\tFOC1_ACK_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_FOC1_ACK_ENABLE_IN) >> 10)
				, "\n\tFOC0_ACK_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_FOC0_ACK_ENABLE_IN) >> 9)
				, "\n\tRAM1_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_RAM1_ENABLE_IN) >> 8)
				, "\n\tRAM0_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_RAM0_ENABLE_IN) >> 7)
				, "\n\tEXT_FULL_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_EXT_FULL_ENABLE_IN) >> 6)
				, "\n\tEXT_RD_DATA_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_EXT_RD_DATA_ENABLE_IN) >> 5)
				, "\n\tTHREAD_RDY_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_THREAD_RDY_ENABLE_IN) >> 4)
				, "\n\tGENERAL_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_GENERAL_ENABLE_IN) >> 3)
				, "\n\tPASSIVE_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_PASSIVE_ENABLE_IN) >> 2)
				, "\n\tFIC1_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_FIC1_ENABLE_IN) >> 1)
				, "\n\tFIC0_ENABLE_IN :"
				, (u32_t)((data & USEM_ENABLE_IN_REG_FIC0_ENABLE_IN) >> 0));
			break;
		case 0x3000a8UL: // (USEM_REG_ENABLE_OUT: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nUSEM_REG_ENABLE_OUT (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWAITP_ENABLE_OUT :"
				, (u32_t)((data & USEM_ENABLE_OUT_REG_WAITP_ENABLE_OUT) >> 9)
				, "\n\tRAM1_ENABLE_OUT :"
				, (u32_t)((data & USEM_ENABLE_OUT_REG_RAM1_ENABLE_OUT) >> 8)
				, "\n\tRAM0_ENABLE_OUT :"
				, (u32_t)((data & USEM_ENABLE_OUT_REG_RAM0_ENABLE_OUT) >> 7)
				, "\n\tPASSIVE_ENABLE_OUT :"
				, (u32_t)((data & USEM_ENABLE_OUT_REG_PASSIVE_ENABLE_OUT) >> 6)
				, "\n\tFOC3_ENABLE_OUT :"
				, (u32_t)((data & USEM_ENABLE_OUT_REG_FOC3_ENABLE_OUT) >> 5)
				, "\n\tFOC2_ENABLE_OUT :"
				, (u32_t)((data & USEM_ENABLE_OUT_REG_FOC2_ENABLE_OUT) >> 4)
				, "\n\tFOC1_ENABLE_OUT :"
				, (u32_t)((data & USEM_ENABLE_OUT_REG_FOC1_ENABLE_OUT) >> 3)
				, "\n\tFOC0_ENABLE_OUT :"
				, (u32_t)((data & USEM_ENABLE_OUT_REG_FOC0_ENABLE_OUT) >> 2)
				, "\n\tEXT_WR_REQ_ENABLE_OUT :"
				, (u32_t)((data & USEM_ENABLE_OUT_REG_EXT_WR_REQ_ENABLE_OUT) >> 1)
				, "\n\tEXT_RD_REQ_ENABLE_OUT :"
				, (u32_t)((data & USEM_ENABLE_OUT_REG_EXT_RD_REQ_ENABLE_OUT) >> 0));
			break;
		case 0x3000acUL: // (USEM_REG_STORM0_H_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_STORM0_H_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm0_h memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000b0UL: // (USEM_REG_STORM1_H_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_STORM1_H_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm1_h memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000b4UL: // (USEM_REG_STORM0_L_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_STORM0_L_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm0_l memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000b8UL: // (USEM_REG_STORM1_L_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_STORM1_L_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm1_l memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000bcUL: // (USEM_REG_CAM_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nUSEM_REG_CAM_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM bits  for cam\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000c0UL: // (USEM_REG_RAM0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_RAM0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tm for ram0_0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000c4UL: // (USEM_REG_PAS_BUF_LSB_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_PAS_BUF_LSB_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_lsb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000c8UL: // (USEM_REG_PAS_BUF_LSB_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_PAS_BUF_LSB_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_lsb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000ccUL: // (USEM_REG_PAS_BUF_MSB_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_PAS_BUF_MSB_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_msb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000d0UL: // (USEM_REG_PAS_BUF_MSB_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_PAS_BUF_MSB_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_msb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000d4UL: // (USEM_REG_INT_TABLE_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_INT_TABLE_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory int_table\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000d8UL: // (USEM_REG_CLEAR_WAITP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_CLEAR_WAITP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Write 1 to this register will disable waitp from this storm\n     to other storms\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000dcUL: // (USEM_REG_SLOW_DBG_MODE: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DBG_MODE (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: debug mode for slow debug bus. Applicable only when ~usem_registers_slow_dbg_active\n    .slow_dbg_active =1. If mode =0 thread number; pram address and DRA WR\n     data selected; if mode =1 fin command and DRA RD ; if mode =2 pram address\n     and thread number and fin command and released thread from STORM; if \n    mode =3 STORE data to SDM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000e0UL: // (USEM_REG_SLOW_DBG_ACTIVE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DBG_ACTIVE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug mode is active\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000e4UL: // (USEM_REG_DBG_MSG_SRC: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_DBG_MSG_SRC (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~usem_registers_slow_dbg_mode.slow_dbg_mode\n     =0. If =0only FIC-s output to debug bus; 1=both FIC-s and passive buffer\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000e8UL: // (USEM_REG_DBG_MODE0_CFG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_DBG_MODE0_CFG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~usem_registers_slow_dbg_mode.slow_dbg_mode\n     =0. If =0 all the message output to debug bus; 1=partial message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000ecUL: // (USEM_REG_DBG_MODE0_CFG_CYCLE: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_DBG_MODE0_CFG_CYCLE (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Applicable only when ~usem_registers_dbg_mode0_cfg.dbg_mode0_cfg\n     =1. If =1 the additional cycles to extract to the debug bus.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000f0UL: // (USEM_REG_DBG_MODE1_CFG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_DBG_MODE1_CFG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~usem_registers_slow_dbg_mode.slow_dbg_mode\n     =1. If=0 output to debug bus without the data; 1=with the data.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000f4UL: // (USEM_REG_DBG_EACH_CYLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_DBG_EACH_CYLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If=0 output every cycle full indication or thread status; \n    1= output only when there is a change.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000f8UL: // (USEM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSEM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from USEMI\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3000fcUL: // (USEM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSEM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only.For dbgmux usage (debug data that goes from USEM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300100UL: // (USEM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nUSEM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only.For dbgmux usage (debug data that goes from USEM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300104UL: // (USEM_REG_USEM_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x300108UL: // (USEM_REG_USEM_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x30010cUL: // (USEM_REG_USEM_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x300110UL: // (USEM_REG_USEM_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x300114UL: // (USEM_REG_USEM_INT_STS_1: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_INT_STS_1 (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x300118UL: // (USEM_REG_USEM_INT_STS_CLR_1: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_CLR_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x30011cUL: // (USEM_REG_USEM_INT_STS_WR_1: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_INT_STS_WR_1 (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_STS_WR_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x300120UL: // (USEM_REG_USEM_INT_MASK_1: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_INT_MASK_1 (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & USEM_USEM_INT_MASK_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x300124UL: // (USEM_REG_USEM_PRTY_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_PRTY_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_0_REG_PARITY) >> 0));
			break;
		case 0x300128UL: // (USEM_REG_USEM_PRTY_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_PRTY_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_0_REG_PARITY) >> 0));
			break;
		case 0x30012cUL: // (USEM_REG_USEM_PRTY_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_PRTY_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_0_REG_PARITY) >> 0));
			break;
		case 0x300130UL: // (USEM_REG_USEM_PRTY_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_PRTY_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_0_REG_PARITY) >> 0));
			break;
		case 0x300134UL: // (USEM_REG_USEM_PRTY_STS_1: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_PRTY_STS_1 (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Parity register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x300138UL: // (USEM_REG_USEM_PRTY_STS_CLR_1: ACCESS:RC  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_PRTY_STS_CLR_1 (ACCESS:RC  DataWidth:0x5 )" \
				"\nDescription: Parity register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_CLR_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x30013cUL: // (USEM_REG_USEM_PRTY_STS_WR_1: ACCESS:WR  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_PRTY_STS_WR_1 (ACCESS:WR  DataWidth:0x5 )" \
				"\nDescription: Parity register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_STS_WR_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x300140UL: // (USEM_REG_USEM_PRTY_MASK_1: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_USEM_PRTY_MASK_1 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Parity mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & USEM_USEM_PRTY_MASK_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x30014cUL: // (USEM_REG_RAM0_TM1: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_RAM0_TM1 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tm for ram0_1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300150UL: // (USEM_REG_RAM0_TM2: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_RAM0_TM2 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tm for ram0_2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3003a8UL: // (USEM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSEM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3003acUL: // (USEM_REG_FIFOS_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nUSEM_REG_FIFOS_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM bits  for  FIC0_LSB [1:0]; FIC0_MSB[3:2]; FIC1_LSB[5:4\n    ]; FIC1_MSB[7:6]; DBG_LSB[9:8];DBG_MSB[11:10]; EXT_PAS[13:12]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300200UL: // (USEM_REG_ARBITER_REQUEST: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_ARBITER_REQUEST (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last request: 0-fic0; 1-fic1; 2-priority0; 3-\n    priority1; 4-priority2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300204UL: // (USEM_REG_ARBITER_SELECT: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_ARBITER_SELECT (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last selection: 0-fic0; 1-fic1; 2-priority0; \n    3-priority1; 4-priority2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300208UL: // (USEM_REG_ARBITER_SLOT: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_ARBITER_SLOT (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last slot\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30020cUL: // (USEM_REG_DBG_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_DBG_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG IF is full in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300210UL: // (USEM_REG_DRA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_DRA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: This register is active when FIN FIO  is empty and DRA RD\n     FIFO is empty\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300214UL: // (USEM_REG_EXT_PAS_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_EXT_PAS_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_PAS FIFO empty in sem_slow\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300218UL: // (USEM_REG_EXT_PAS_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_EXT_PAS_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_PAS FIFO Full in sem_slow\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30021cUL: // (USEM_REG_EXT_STORE_FREE_ENTRIES: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUSEM_REG_EXT_STORE_FREE_ENTRIES (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Number of free entries in the external STORE sync FIFO.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300220UL: // (USEM_REG_EXT_STORE_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_EXT_STORE_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE IF is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300224UL: // (USEM_REG_FIC0_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC0_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from FIC0  May be updated during \n    run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300228UL: // (USEM_REG_FIC0_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC0_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO empty in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30022cUL: // (USEM_REG_FIC0_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC0_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO Full in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300230UL: // (USEM_REG_FIC0_LENGTH: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC0_LENGTH (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Length from FIC0. Active only with ~usem_registers_fic0_length_error\n    .fic0_length_error interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300234UL: // (USEM_REG_FIC1_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC1_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from FIC1  May be updated during \n    run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300238UL: // (USEM_REG_FIC1_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC1_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO empty in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30023cUL: // (USEM_REG_FIC1_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC1_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO Full in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300240UL: // (USEM_REG_FIC1_LENGTH: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC1_LENGTH (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Length from FIC1. Active only with ~usem_registers_fic1_length_error\n    .fic1_length_error interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300244UL: // (USEM_REG_GPI_DATA: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nUSEM_REG_GPI_DATA (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: GPI signals that are inputs to SEMI\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300248UL: // (USEM_REG_NUM_OF_THREADS: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nUSEM_REG_NUM_OF_THREADS (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: The number of threads currently active \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30024cUL: // (USEM_REG_PAS_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_PAS_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from the passive buffer  May be updated\n     during run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300250UL: // (USEM_REG_PAS_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_PAS_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Full from passive buffer asserted toward SDM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300254UL: // (USEM_REG_RAM0_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_RAM0_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RAM0 IF is full in sem_slow_ls_ram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300258UL: // (USEM_REG_RAM1_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_RAM1_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RAM1 IF is full in sem_slow_ls_ram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30025cUL: // (USEM_REG_SET0_THREAD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SET0_THREAD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET0_THREAD fifo is empty in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300260UL: // (USEM_REG_SET0_THREAD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SET0_THREAD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET0_THREAD fifo is full in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300264UL: // (USEM_REG_SET1_THREAD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SET1_THREAD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET1_THREAD fifo is empty in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300268UL: // (USEM_REG_SET1_THREAD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SET1_THREAD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET1_THREAD fifo is full in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30026cUL: // (USEM_REG_SLEEP_THREADS_VALID: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nUSEM_REG_SLEEP_THREADS_VALID (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Valid sleeping threads indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300270UL: // (USEM_REG_SLOW_DBG_ALM_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DBG_ALM_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is almost empty in sem_slow_ls_dbg (31 entry inside\n     fifo)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300274UL: // (USEM_REG_SLOW_DBG_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DBG_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is almost empty in sem_slow_ls_dbg according to \n    configuration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300278UL: // (USEM_REG_SLOW_DBG_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DBG_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is empty in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30027cUL: // (USEM_REG_SLOW_DBG_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DBG_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is full in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300280UL: // (USEM_REG_SLOW_DRA_FIN_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DRA_FIN_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIN fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300284UL: // (USEM_REG_SLOW_DRA_FIN_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DRA_FIN_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIN fifo is full in sem_slow_dra_sync (never may be active\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300288UL: // (USEM_REG_SLOW_DRA_INT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DRA_INT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30028cUL: // (USEM_REG_SLOW_DRA_INT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DRA_INT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt fifo is full in sem_slow_dra_int\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300290UL: // (USEM_REG_SLOW_DRA_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DRA_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_RD pop fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300294UL: // (USEM_REG_SLOW_DRA_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DRA_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_RD pop fifo is full in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300298UL: // (USEM_REG_SLOW_DRA_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DRA_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_WR push fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30029cUL: // (USEM_REG_SLOW_DRA_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_DRA_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_WR push fifo is full in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002a0UL: // (USEM_REG_SLOW_EXT_STORE_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_EXT_STORE_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002a4UL: // (USEM_REG_SLOW_EXT_STORE_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_EXT_STORE_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002a8UL: // (USEM_REG_SLOW_RAM0_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_RAM0_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM0 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002acUL: // (USEM_REG_SLOW_RAM0_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_RAM0_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM0 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002b0UL: // (USEM_REG_SLOW_RAM0_WR_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_RAM0_WR_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is almost full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002b4UL: // (USEM_REG_SLOW_RAM0_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_RAM0_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM0 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002b8UL: // (USEM_REG_SLOW_RAM0_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_RAM0_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM0 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002bcUL: // (USEM_REG_SLOW_RAM1_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_RAM1_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM1 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002c0UL: // (USEM_REG_SLOW_RAM1_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_RAM1_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM1 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002c4UL: // (USEM_REG_SLOW_RAM1_WR_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_RAM1_WR_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is almost full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002c8UL: // (USEM_REG_SLOW_RAM1_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_RAM1_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002ccUL: // (USEM_REG_SLOW_RAM1_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SLOW_RAM1_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002d0UL: // (USEM_REG_SYNC_DBG_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SYNC_DBG_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FAST SYNC FIFO is empty in sem_slow_ls_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002d4UL: // (USEM_REG_SYNC_DBG_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nUSEM_REG_SYNC_DBG_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FAST SYNC FIFO is full in sem_slow_ls_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002d8UL: // (USEM_REG_THREAD_ERROR: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nUSEM_REG_THREAD_ERROR (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Thread error indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002dcUL: // (USEM_REG_THREAD_OVERRUN_NUM: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nUSEM_REG_THREAD_OVERRUN_NUM (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Threads are sleeping in passive buffer more than ~usem_registers_thread_inter_cnt\n    .thread_inter_cnt number of cycles\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002e0UL: // (USEM_REG_THREAD_RDY: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nUSEM_REG_THREAD_RDY (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Thread ready indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002e4UL: // (USEM_REG_THREADS_LIST: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nUSEM_REG_THREADS_LIST (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: List of free threads . There is a bit per thread.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002e8UL: // (USEM_REG_WB_MSB: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nUSEM_REG_WB_MSB (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Reset value of this register is right when was not read to\n     ~usem_registers_fic0_fifo.fic0_fifo or ~usem_registers_fic1_fifo.fic1_fifo\n     or ~usem_registers_passive_buffer.passive_buffer. For read from ~usem_registers_passive_buffer\n    .passive_buffer :b0- parity0; b1 parity1. For  read from ~usem_registers_fic0_fifo\n    .fic0_fifo and ~usem_registers_fic1_fifo.fic1_fifo :b1=0 data from ~usem_registers_fic0_fifo\n    .fic0_fifo and ~usem_registers_fic1_fifo.fic1_fifo is valid; b1 =1 ~usem_registers_fic0_fifo\n    .fic0_fifo and ~usem_registers_fic1_fifo.fic1_fifo is empty and data from\n     it must be equal to 0; b0 - parity from ~usem_registers_fic0_fifo.fic0_fifo\n     and ~usem_registers_fic1_fifo.fic1_fifo\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300300UL: // (USEM_REG_FIC0_FIFO: ACCESS:WB_R  DataWidth:0x80 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC0_FIFO (ACCESS:WB_R  DataWidth:0x80 )" \
				"\nDescription: Debug only. FIC0_fifo: b[127:0] data; b128-parity;b129=1- \n    fifo empty;b129=0-data is valid\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300320UL: // (USEM_REG_FIC1_FIFO: ACCESS:WB_R  DataWidth:0x80 ) 
			sprintf(disp_str, "\nUSEM_REG_FIC1_FIFO (ACCESS:WB_R  DataWidth:0x80 )" \
				"\nDescription: Debug only. FIC1_fifo read for debugging mode; b[127:0] data\n    ; b128-parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300340UL: // (USEM_REG_FIN_COMMAND: ACCESS:WB_R  DataWidth:0x6d ) 
			sprintf(disp_str, "\nUSEM_REG_FIN_COMMAND (ACCESS:WB_R  DataWidth:0x6d )" \
				"\nDescription: last fin command that was read from fifo. Its spelling in\n     ~usem_registers_fin_fifo.fin_fifo register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300360UL: // (USEM_REG_FIN_FIFO: ACCESS:WB_R  DataWidth:0x6d ) 
			sprintf(disp_str, "\nUSEM_REG_FIN_FIFO (ACCESS:WB_R  DataWidth:0x6d )" \
				"\nDescription: Debug only. FIn FIFO.  [5:0]   start_rp_foc3; [11:6]    start_rp_foc2\n    ;  [17:12]   start_rp_foc1; [23:18]   start_rp_foc0;  [29:24]   end_rp_foc3\n    ; [35:30]   end_rp_foc2; [41:36]   end_rp_foc1; [47:42]   end_rp_foc0; [\n    53:48]   lowest rp; [59:54]   highest rp; [65:60]   store start rp; [71\n    :66]   store end rp; [77:72]   load start rp; [83:78]   load end rp; [\n    85:84]   priority; [101:86]  pram address; [102] pas; [103] foc3; [104\n    ] foc2; [105] foc1; [106] foc0; [107] release; [108] fin; [109]=1- fin\n     fifo is empty; 0 - data[108:0] is valid\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300380UL: // (USEM_REG_VFPF_ERR_NUM: ACCESS:W  DataWidth:0x7 ) 
			sprintf(disp_str, "\nUSEM_REG_VFPF_ERR_NUM (ACCESS:W  DataWidth:0x7 )" \
				"\nDescription: VF or PF ID for reset error bit. Values 0-63 reset error \n    bit for 64 VF; values 64-67 reset error for 4 PF; values 68-127 are not\n     valid.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300388UL: // (USEM_REG_VF_ERR_VECTOR_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_VF_ERR_VECTOR_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: VF/PF error bitmap vector [0-31]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300390UL: // (USEM_REG_VF_ERR_VECTOR_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_VF_ERR_VECTOR_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: VF/PF error bitmap vector [32-63]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300398UL: // (USEM_REG_PF_ERR_VECTOR: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nUSEM_REG_PF_ERR_VECTOR (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: VF/PF error bitmap vector [0-7]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3003a0UL: // (USEM_REG_THREAD_SET_NUM: ACCESS:W  DataWidth:0x5 ) 
			sprintf(disp_str, "\nUSEM_REG_THREAD_SET_NUM (ACCESS:W  DataWidth:0x5 )" \
				"\nDescription: Thread ID. Write thread ID will set ready indication for \n    this thread ID\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300400UL: // (USEM_REG_INT_TABLE: ACCESS:RW  DataWidth:0xf ) 
			sprintf(disp_str, "\nUSEM_REG_INT_TABLE (ACCESS:RW  DataWidth:0xf )" \
				"\nDescription: Interrupt table  Read and write access to it is not possible\n     in the middle of the work\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x302000UL: // (USEM_REG_PASSIVE_BUFFER: ACCESS:WB  DataWidth:0x80 ) 
			sprintf(disp_str, "\nUSEM_REG_PASSIVE_BUFFER (ACCESS:WB  DataWidth:0x80 )" \
				"\nDescription: Debug only. Passive buffer memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x304000UL: // (USEM_REG_PASSIVE_BUFFER_MSB: ACCESS:WB  DataWidth:0x80 ) 
			sprintf(disp_str, "\nUSEM_REG_PASSIVE_BUFFER_MSB (ACCESS:WB  DataWidth:0x80 )" \
				"\nDescription: Debug only. Passive buffer memory MSB that starts from row\n     512 of passive buffer till row 639\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x320000UL: // (USEM_REG_FAST_MEMORY: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nUSEM_REG_FAST_MEMORY (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This address space contains all registers and memories that\n     are placed in SEM_FAST block. The SEM_FAST registers are described in\n     appendix B. In order to access the SEM_FAST registers the base address\n     USEM_REGISTERS_FAST_MEMORY (Offset: 0x320000) should be added to each\n     SEM_FAST register offset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x340000UL: // (USEM_REG_PRAM: ACCESS:WB  DataWidth:0x2e ) 
			sprintf(disp_str, "\nUSEM_REG_PRAM (ACCESS:WB  DataWidth:0x2e )" \
				"\nDescription: pram memory. B45 is parity; b[44:0] - data.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300144UL: // (USEM_REG_USEM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300154UL: // (USEM_REG_USEM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3002ecUL: // (USEM_REG_USEM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300384UL: // (USEM_REG_USEM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30038cUL: // (USEM_REG_USEM_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300394UL: // (USEM_REG_USEM_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30039cUL: // (USEM_REG_USEM_UNUSED_EMPTY_6: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_6 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3003a4UL: // (USEM_REG_USEM_UNUSED_EMPTY_7: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_7 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3003b0UL: // (USEM_REG_USEM_UNUSED_EMPTY_8: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_8 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x300800UL: // (USEM_REG_USEM_UNUSED_EMPTY_9: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_9 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x304800UL: // (USEM_REG_USEM_UNUSED_EMPTY_10: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nUSEM_REG_USEM_UNUSED_EMPTY_10 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19428UL: // (VFC_REG_INTERRUPT_IND: ACCESS:RW  DataWidth:0xb  Multi Field Register  ) 
			sprintf(disp_str, "\nVFC_REG_INTERRUPT_IND (ACCESS:RW  DataWidth:0xb  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDEADLOCK_INTERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_DEADLOCK_INTERRUPT) >> 10)
				, "\n\tRBC_WRITE_INTERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_RBC_WRITE_INTERRUPT) >> 9)
				, "\n\tRSS_KEY_MSB_INTERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_RSS_KEY_MSB_INTERRUPT) >> 8)
				, "\n\tRSS_KEY_LSB_INTERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_RSS_KEY_LSB_INTERRUPT) >> 7)
				, "\n\tRSS_INFO_INTERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_RSS_INFO_INTERRUPT) >> 6)
				, "\n\tRSS_IND_INTERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_RSS_IND_INTERRUPT) >> 5)
				, "\n\tOUT_BUF_INTERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_OUT_BUF_INTERRUPT) >> 4)
				, "\n\tINP_BUF_INTERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_INP_BUF_INTERRUPT) >> 3)
				, "\n\tLEN_FIFO_INTERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_LEN_FIFO_INTERRUPT) >> 2)
				, "\n\tINP_FIFO_ITERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_INP_FIFO_ITERRUPT) >> 1)
				, "\n\tADDRESS_INTERRUPT :"
				, (u32_t)((data & VFC_INTERRUPT_IND_REG_ADDRESS_INTERRUPT) >> 0));
			break;
		case 0x1942cUL: // (VFC_REG_PARITY_IND: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nVFC_REG_PARITY_IND (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tINP_FIFO_PARITY :"
				, (u32_t)((data & VFC_PARITY_IND_REG_INP_FIFO_PARITY) >> 2)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & VFC_PARITY_IND_REG_CAM_PARITY) >> 1)
				, "\n\tRSS_RAM_PARITY :"
				, (u32_t)((data & VFC_PARITY_IND_REG_RSS_RAM_PARITY) >> 0));
			break;
		case 0x19430UL: // (VFC_REG_INDICATIONS1: ACCESS:RW  DataWidth:0x12  Multi Field Register  ) 
			sprintf(disp_str, "\nVFC_REG_INDICATIONS1 (ACCESS:RW  DataWidth:0x12  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_WAITP :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_VFC_WAITP) >> 17)
				, "\n\tRBC_RSP_RDY :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_RBC_RSP_RDY) >> 16)
				, "\n\tRESERVED1_2 :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_RESERVED1_2) >> 13)
				, "\n\tSEM_FIFO_FULL :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_SEM_FIFO_FULL) >> 12)
				, "\n\tOUT_FIFO_FULL :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_OUT_FIFO_FULL) >> 11)
				, "\n\tINP_BUF_FULL :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_INP_BUF_FULL) >> 10)
				, "\n\tLEN_FIFO_FULL :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_LEN_FIFO_FULL) >> 9)
				, "\n\tINP_FIFO_FULL :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_INP_FIFO_FULL) >> 8)
				, "\n\tRESERVED1_1 :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_RESERVED1_1) >> 5)
				, "\n\tSEM_FIFO_EMPTY :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_SEM_FIFO_EMPTY) >> 4)
				, "\n\tOUT_FIFO_EMPTY :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_OUT_FIFO_EMPTY) >> 3)
				, "\n\tINP_BUF_EMPTY :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_INP_BUF_EMPTY) >> 2)
				, "\n\tLEN_FIFO_EMPTY :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_LEN_FIFO_EMPTY) >> 1)
				, "\n\tINP_FIFO_EMPTY :"
				, (u32_t)((data & VFC_INDICATIONS1_REG_INP_FIFO_EMPTY) >> 0));
			break;
		case 0x19434UL: // (VFC_REG_INDICATIONS2: ACCESS:RW  DataWidth:0x1c  Multi Field Register  ) 
			sprintf(disp_str, "\nVFC_REG_INDICATIONS2 (ACCESS:RW  DataWidth:0x1c  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSEM_FIFO_CNT :"
				, (u32_t)((data & VFC_INDICATIONS2_REG_SEM_FIFO_CNT) >> 24)
				, "\n\tOUT_BUF_CNT :"
				, (u32_t)((data & VFC_INDICATIONS2_REG_OUT_BUF_CNT) >> 20)
				, "\n\tINP_BUF_CNT :"
				, (u32_t)((data & VFC_INDICATIONS2_REG_INP_BUF_CNT) >> 16)
				, "\n\tRESERVED2_2 :"
				, (u32_t)((data & VFC_INDICATIONS2_REG_RESERVED2_2) >> 13)
				, "\n\tLEN_FIFO_CNT :"
				, (u32_t)((data & VFC_INDICATIONS2_REG_LEN_FIFO_CNT) >> 8)
				, "\n\tRESERVED2_1 :"
				, (u32_t)((data & VFC_INDICATIONS2_REG_RESERVED2_1) >> 5)
				, "\n\tINP_FIFO_CNT :"
				, (u32_t)((data & VFC_INDICATIONS2_REG_INP_FIFO_CNT) >> 0));
			break;
		case 0x1943cUL: // (VFC_REG_MEMORIES_RST: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nVFC_REG_MEMORIES_RST (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRAM_RST :"
				, (u32_t)((data & VFC_MEMORIES_RST_REG_RAM_RST) >> 1)
				, "\n\tCAM_RST :"
				, (u32_t)((data & VFC_MEMORIES_RST_REG_CAM_RST) >> 0));
			break;
		case 0x19450UL: // (VFC_REG_RSS_RAM_TM_0: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nVFC_REG_RSS_RAM_TM_0 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM indication for RSS RAM instance 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19454UL: // (VFC_REG_RSS_RAM_TM_1: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nVFC_REG_RSS_RAM_TM_1 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM indication for RSS RAM instance 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19458UL: // (VFC_REG_INP_FIFO_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nVFC_REG_INP_FIFO_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM indication for Input fifo\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1945cUL: // (VFC_REG_CAM_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nVFC_REG_CAM_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM indication for CAM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19474UL: // (VFC_REG_STORM_CMD_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_STORM_CMD_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: When set then it disables selecting of commands from STORM\n    . It will allow for RBC to configurate block. STORM command may be executed\n     when this bit will be deasserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19478UL: // (VFC_REG_WAITP_STAT: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_WAITP_STAT (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: Statistics for number of cycles when waitp was raised to \n    STORM as a result of full input FIFO. This vector will be reset after \n    reading from it. It is also possible to write to it.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1947cUL: // (VFC_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Unused bits for future eco \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19480UL: // (VFC_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nVFC_REG_CPU_MBIST_MEMCTRL_0_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19484UL: // (VFC_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nVFC_REG_CPU_MBIST_MEMCTRL_1_CNTRL_CMD (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Bit 0 - bist_override; Bit 1 - mbist_en; Bit 2 - mbist_async_reset\n    ; Bits 4:3 - bist_setup[1:0];\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19488UL: // (VFC_REG_CPU_MBIST_MEMCTRL_0_STATUS_0: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nVFC_REG_CPU_MBIST_MEMCTRL_0_STATUS_0 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1948cUL: // (VFC_REG_CPU_MBIST_MEMCTRL_1_STATUS_0: ACCESS:RW  DataWidth:0x7 ) 
			sprintf(disp_str, "\nVFC_REG_CPU_MBIST_MEMCTRL_1_STATUS_0 (ACCESS:RW  DataWidth:0x7 )" \
				"\nDescription: Bit 0 - mbist_done; Bit 1 -mbist_go (global go bit); Bits\n     NUM_OF_MEM+1:2 - mbist_go0 (per mem go bit); Bits NUM_OF_SPARE+NUM_OF_MEM\n    +1:NUM_OF_MEM+2 - spare bits \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19490UL: // (VFC_REG_DEBUG_DATA: ACCESS:RW  DataWidth:0x1e  Multi Field Register  ) 
			sprintf(disp_str, "\nVFC_REG_DEBUG_DATA (ACCESS:RW  DataWidth:0x1e  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLAST_MATCH_ADDR :"
				, (u32_t)((data & VFC_DEBUG_DATA_REG_LAST_MATCH_ADDR) >> 20)
				, "\n\tRESERVED2 :"
				, (u32_t)((data & VFC_DEBUG_DATA_REG_RESERVED2) >> 18)
				, "\n\tRBC_READY :"
				, (u32_t)((data & VFC_DEBUG_DATA_REG_RBC_READY) >> 17)
				, "\n\tSTORM_READY :"
				, (u32_t)((data & VFC_DEBUG_DATA_REG_STORM_READY) >> 16)
				, "\n\tRBC_CNT :"
				, (u32_t)((data & VFC_DEBUG_DATA_REG_RBC_CNT) >> 8)
				, "\n\tNEXT_MSG_EMTY :"
				, (u32_t)((data & VFC_DEBUG_DATA_REG_NEXT_MSG_EMTY) >> 7)
				, "\n\tMEXT_MSG_LEN :"
				, (u32_t)((data & VFC_DEBUG_DATA_REG_MEXT_MSG_LEN) >> 4)
				, "\n\tCUR_MSG_EMPTY :"
				, (u32_t)((data & VFC_DEBUG_DATA_REG_CUR_MSG_EMPTY) >> 3)
				, "\n\tCURRENT_MSG_LEN :"
				, (u32_t)((data & VFC_DEBUG_DATA_REG_CURRENT_MSG_LEN) >> 0));
			break;
		case 0x19494UL: // (VFC_REG_STORM_CMD_ADDR: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nVFC_REG_STORM_CMD_ADDR (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Address of command from STORM that is waiting for arbitration\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19498UL: // (VFC_REG_STORM_CMD_DATA_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_STORM_CMD_DATA_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Data bits[31:0] of VFC command from STORM that are waiting\n     for arbitration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1949cUL: // (VFC_REG_STORM_CMD_DATA_1: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_STORM_CMD_DATA_1 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Data bits[63:32] of VFC command from STORM that are waiting\n     for arbitration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x194a0UL: // (VFC_REG_STORM_CMD_DATA_2: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_STORM_CMD_DATA_2 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Data bits[95:64] of VFC command from STORM that are waiting\n     for arbitration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x194a4UL: // (VFC_REG_STORM_CMD_DATA_3: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_STORM_CMD_DATA_3 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Data bits[127:96] of VFC command from STORM that are waiting\n     for arbitration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x194a8UL: // (VFC_REG_STORM_CMD_DATA_4: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_STORM_CMD_DATA_4 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Data bits[159:128] of VFC command from STORM that are waiting\n     for arbitration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x194acUL: // (VFC_REG_STORM_CMD_DATA_5: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_STORM_CMD_DATA_5 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Data bits[191:160] of VFC command from STORM that are waiting\n     for arbitration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x194b0UL: // (VFC_REG_STORM_CMD_DATA_6: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_STORM_CMD_DATA_6 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Data bits[223:192] of VFC command from STORM that are waiting\n     for arbitration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x194b4UL: // (VFC_REG_STORM_CMD_DATA_7: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_STORM_CMD_DATA_7 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Data bits[255:224] of VFC command from STORM that are waiting\n     for arbitration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x194e0UL: // (VFC_REG_VFC_PRTY_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_PRTY_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & VFC_VFC_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x194e4UL: // (VFC_REG_VFC_PRTY_STS_WR: ACCESS:WR  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_PRTY_STS_WR (ACCESS:WR  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & VFC_VFC_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x194e8UL: // (VFC_REG_VFC_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & VFC_VFC_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x194ecUL: // (VFC_REG_VFC_PRTY_STS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_PRTY_STS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tPARITY :"
				, (u32_t)((data & VFC_VFC_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x194f0UL: // (VFC_REG_VFC_INT_MASK: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_INT_MASK (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & VFC_VFC_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x194f4UL: // (VFC_REG_VFC_INT_STS_WR: ACCESS:WR  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_INT_STS_WR (ACCESS:WR  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & VFC_VFC_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x194f8UL: // (VFC_REG_VFC_INT_STS_CLR: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_INT_STS_CLR (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & VFC_VFC_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x194fcUL: // (VFC_REG_VFC_INT_STS: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_INT_STS (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & VFC_VFC_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x19400UL: // (VFC_REG_MASK_LSB_0_LOW: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_MASK_LSB_0_LOW (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 64 LSB bits for vector CAM mask that are used for search \n    and add commands. 1 means the corresponding data bit should be compared\n     and 0 means it should be ignored.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19404UL: // (VFC_REG_MASK_LSB_0_HIGH: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_MASK_LSB_0_HIGH (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription:  64 LSB bits for vector CAM mask that are used for search\n     and add commands. 1 means the corresponding data bit should be compared\n     and 0 means it should be ignored.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19408UL: // (VFC_REG_MASK_LSB_1_LOW: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_MASK_LSB_1_LOW (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 64 LSB bits for vector CAM mask that are used for search \n    and add commands. 1 means the corresponding data bit should be compared\n     and 0 means it should be ignored. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1940cUL: // (VFC_REG_MASK_LSB_1_HIGH: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_MASK_LSB_1_HIGH (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 64 LSB bits for vector CAM mask that are used for search \n    and add commands. 1 means the corresponding data bit should be compared\n     and 0 means it should be ignored. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19410UL: // (VFC_REG_MASK_LSB_2_LOW: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_MASK_LSB_2_LOW (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 32 LSB bits [31:0] for vector CAM mask that are used for \n    search and add commands.1 means the corresponding data bit should be compared\n     and 0 means it should be ignored.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19414UL: // (VFC_REG_MASK_LSB_2_HIGH: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_MASK_LSB_2_HIGH (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 32 LSB bits [63:32] for vector CAM mask that are used for\n     search and add commands. 1 means the corresponding data bit should be\n     compared and 0 means it should be ignored.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19418UL: // (VFC_REG_MASK_LSB_3_LOW: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_MASK_LSB_3_LOW (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 32 LSB bits [31:0]  bits for vector CAM mask that are used\n     for search and add commands.1 means the corresponding data bit should\n     be compared and 0 means it should be ignored. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1941cUL: // (VFC_REG_MASK_LSB_3_HIGH: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_MASK_LSB_3_HIGH (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: 32 LSB bits [63:32]  for vector CAM mask that are used for\n     search and add commands. 1 means the corresponding data bit should be\n     compared and 0 means it should be ignored. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19420UL: // (VFC_REG_MASK_OPTIMIZED_LOW: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_MASK_OPTIMIZED_LOW (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This field contains 4 vectors of 8 bit [31:0]. Decoding of\n     each field:Bo- 1 search is done for MAC address (48 bit LSB in CAM); \n    0 - search for MAC address disabled. B1- 1 search is done for IVLAN (bits59\n    :48 in CAM); 0 - search for IVLAN disabled. B2- 1 search is done for b60\n     in CAM; 0 - search for b60 in CAM disabled. B3- 1 search is done for \n    b61 in CAM; 0 - search for b61 in CAM disabled. B4- 1 search is done for\n     b62 in CAM; 0 - search for b62 in CAM disabled. B5- 1 search is done \n    for b63 in CAM; 0 - search for b63 in CAM disabled. B7:6 - reserved\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19424UL: // (VFC_REG_MASK_OPTIMIZED_HIGH: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_MASK_OPTIMIZED_HIGH (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This field contains 4 vectors of 8 bit[63:32]. Decoding of\n     each field:Bo- 1 search is done for MAC address (48 bit LSB in CAM); \n    0 - search for MAC address disabled. B1- 1 search is done for IVLAN (bits59\n    :48 in CAM); 0 - search for IVLAN disabled. B2- 1 search is done for b60\n     in CAM; 0 - search for b60 in CAM disabled. B3- 1 search is done for \n    b61 in CAM; 0 - search for b61 in CAM disabled. B4- 1 search is done for\n     b62 in CAM; 0 - search for b62 in CAM disabled. B5- 1 search is done \n    for b63 in CAM; 0 - search for b63 in CAM disabled. B7:6 - reserved\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19438UL: // (VFC_REG_SW_RST: ACCESS:W  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_SW_RST (ACCESS:W  DataWidth:0x1 )" \
				"\nDescription: Write to this bit will cause to block reset\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19440UL: // (VFC_REG_CAM_PARITY_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_CAM_PARITY_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: REQUIRED -If this bit is set then background mechanism for\n     parity check will be enabled; 0 - disabled\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19444UL: // (VFC_REG_CAM_CLK_DIVIDER: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nVFC_REG_CAM_CLK_DIVIDER (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Cam clock divider : (may be equal to 2; 4; or 8 only)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19448UL: // (VFC_REG_PARITY_MASK: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nVFC_REG_PARITY_MASK (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: REQUIRED - 0 - parity is enabled;1 parity check is disabled\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1944cUL: // (VFC_REG_INTERRUPT_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nVFC_REG_INTERRUPT_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: REQUIRED - 0 - interrupt is enabled;1- interrupt check is\n     disabled\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19460UL: // (VFC_REG_VFC_CAM_BIST_EN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_CAM_BIST_EN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Bist enable bit for Cam\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19464UL: // (VFC_REG_VFC_CAM_BIST_DBG_SEL: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_CAM_BIST_DBG_SEL (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: This select the type of data present on bist_status bus (\n    slixe or status select)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19468UL: // (VFC_REG_VFC_CAM_BIST_STATUS: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nVFC_REG_VFC_CAM_BIST_STATUS (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: This returns the bist_status which can be done/go/sX_status\n     \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1946cUL: // (VFC_REG_KEY_RSS_EXT5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nVFC_REG_KEY_RSS_EXT5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Key extension for 5th tuple\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19470UL: // (VFC_REG_INP_FIFO_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nVFC_REG_INP_FIFO_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full for input FIFO. When number of entries inside\n     input FIFO is bigger or equal to this number then waitp to STORM will\n     be asserted\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x194b8UL: // (VFC_REG_VFC_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nVFC_REG_VFC_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x19500UL: // (VFC_REG_VFC_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nVFC_REG_VFC_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20000UL: // (XCM_REG_INIT: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_INIT (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Debug only. Initialises specific states and statuses. To \n    initialise the state - write 1 into register; to enable working after \n    that - write 0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20004UL: // (XCM_REG_XCM_STORM0_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_XCM_STORM0_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - STORM 0 Interface enable. If 0 - the acknowledge input\n     is disregarded; valid is deasserted; all other signals are treated as\n     usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20008UL: // (XCM_REG_XCM_STORM1_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_XCM_STORM1_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - STORM 1 Interface enable. If 0 - the acknowledge input\n     is disregarded; valid is deasserted; all other signals are treated as\n     usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2000cUL: // (XCM_REG_XCM_XQM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_XCM_XQM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - QM Interface enable. If 0 - the acknowledge input is\n     disregarded; valid is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20010UL: // (XCM_REG_STORM_XCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_STORM_XCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: STORM - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20014UL: // (XCM_REG_XQM_XCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_XQM_XCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: QM - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20018UL: // (XCM_REG_XSDM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_XSDM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input SDM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2001cUL: // (XCM_REG_TM_XCM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_TM_XCM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Timers - CM Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20020UL: // (XCM_REG_XCM_TM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_XCM_TM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - Timers Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20024UL: // (XCM_REG_TSEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_TSEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input tsem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20028UL: // (XCM_REG_CSEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_CSEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input csem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2002cUL: // (XCM_REG_USEM_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_USEM_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input usem Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20030UL: // (XCM_REG_DORQ_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_DORQ_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input dorq Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20034UL: // (XCM_REG_PBF_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_PBF_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input pbf Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20038UL: // (XCM_REG_NIG0_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_NIG0_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input nig0 Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2003cUL: // (XCM_REG_NIG1_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_NIG1_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Input nig1 Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20040UL: // (XCM_REG_CDU_AG_WR_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_CDU_AG_WR_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU AG write Interface enable. If 0 - the request and valid\n     input are disregarded; all other signals are treated as usual; if 1 - \n    normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20044UL: // (XCM_REG_CDU_AG_RD_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_CDU_AG_RD_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU AG read Interface enable. If 0 - the request input is\n     disregarded; valid output is deasserted; all other signals are treated\n     as usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20048UL: // (XCM_REG_CDU_SM_WR_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_CDU_SM_WR_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU STORM write Interface enable. If 0 - the request and \n    valid input is disregarded; all other signals are treated as usual; if\n     1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2004cUL: // (XCM_REG_CDU_SM_RD_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_CDU_SM_RD_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CDU STORM read Interface enable. If 0 - the request input\n     is disregarded; valid output is deasserted; all other signals are treated\n     as usual; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20050UL: // (XCM_REG_XCM_CFC_IFEN: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_XCM_CFC_IFEN (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: CM - CFC Interface enable. If 0 - the valid input is disregarded\n    ; acknowledge output is deasserted; all other signals are treated as usual\n    ; if 1 - normal activity.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20054UL: // (XCM_REG_XX_MAX_LL_SZ: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXCM_REG_XX_MAX_LL_SZ (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Maximum link list size (number of pending messages) per connection\n     in the XX protection.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20058UL: // (XCM_REG_XX_OVFL_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_XX_OVFL_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID; sent to the STORM in case of XX overflow.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2005cUL: // (XCM_REG_XX_MAX_NUM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_XX_MAX_NUM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The maximum number of connections in the XX protection.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20060UL: // (XCM_REG_N_SM_CTX_LD_0: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_0 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20064UL: // (XCM_REG_N_SM_CTX_LD_1: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_1 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20068UL: // (XCM_REG_N_SM_CTX_LD_2: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_2 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2006cUL: // (XCM_REG_N_SM_CTX_LD_3: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_3 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20070UL: // (XCM_REG_N_SM_CTX_LD_4: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_4 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20074UL: // (XCM_REG_N_SM_CTX_LD_5: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_5 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20078UL: // (XCM_REG_N_SM_CTX_LD_6: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_6 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2007cUL: // (XCM_REG_N_SM_CTX_LD_7: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_7 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20080UL: // (XCM_REG_N_SM_CTX_LD_8: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_8 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20084UL: // (XCM_REG_N_SM_CTX_LD_9: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_9 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20088UL: // (XCM_REG_N_SM_CTX_LD_10: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_10 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2008cUL: // (XCM_REG_N_SM_CTX_LD_11: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_11 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20090UL: // (XCM_REG_N_SM_CTX_LD_12: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_12 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20094UL: // (XCM_REG_N_SM_CTX_LD_13: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_13 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20098UL: // (XCM_REG_N_SM_CTX_LD_14: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_14 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2009cUL: // (XCM_REG_N_SM_CTX_LD_15: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_N_SM_CTX_LD_15 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of double REG-pairs; loaded from the STORM context\n     and sent to STORM; for a specific connection type. The double REG-pairs\n     are used in order to align to STORM context row size of 128 bits. The\n     offset of these data in the STORM context is always 0. Index _i stands\n     for the connection type (one of 16).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200a0UL: // (XCM_REG_XQM_XCM_HDR_P: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nXCM_REG_XQM_XCM_HDR_P (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header value for QM request (primary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200a4UL: // (XCM_REG_XQM_XCM_HDR_S: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nXCM_REG_XQM_XCM_HDR_S (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header value for QM request (secondary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200a8UL: // (XCM_REG_TM_XCM_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nXCM_REG_TM_XCM_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header for Timers expiration command.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200acUL: // (XCM_REG_ERR_XCM_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nXCM_REG_ERR_XCM_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM erroneous header for QM and Timers formatting.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200b0UL: // (XCM_REG_ERR_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_ERR_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID in case the ErrorFlg input message bit is set\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200b4UL: // (XCM_REG_EXPR_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_EXPR_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID for Timers expiration.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200b8UL: // (XCM_REG_STOP_EVNT_ID: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_STOP_EVNT_ID (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The Event ID for Timers formatting in case of stop done.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200bcUL: // (XCM_REG_STORM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_STORM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the STORM input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200c0UL: // (XCM_REG_TSEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_TSEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input tsem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200c4UL: // (XCM_REG_CSEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_CSEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input csem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200c8UL: // (XCM_REG_USEM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_USEM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input usem in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200ccUL: // (XCM_REG_DORQ_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_DORQ_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input dorq in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200d0UL: // (XCM_REG_PBF_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_PBF_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input pbf in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200d4UL: // (XCM_REG_NIG0_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_NIG0_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input nig0 in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200d8UL: // (XCM_REG_NIG1_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_NIG1_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the input nig1 in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200dcUL: // (XCM_REG_CP_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_CP_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the CP input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200e0UL: // (XCM_REG_XSDM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_XSDM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the SDM input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200e4UL: // (XCM_REG_XQM_P_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_XQM_P_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the QM (primary) input in the WRR mechanism\n    . 0 stands for weight 8 (the most prioritised); 1 stands for weight 1(\n    least prioritised); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200e8UL: // (XCM_REG_XQM_S_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_XQM_S_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the QM (secondary) input in the WRR mechanism\n    . 0 stands for weight 8 (the most prioritised); 1 stands for weight 1(\n    least prioritised); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200ecUL: // (XCM_REG_TM_WEIGHT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_TM_WEIGHT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The weight of the Timers input in the WRR mechanism. 0 stands\n     for weight 8 (the most prioritised); 1 stands for weight 1(least prioritised\n    ); 2 stands for weight 2;etc.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200f0UL: // (XCM_REG_XCM_XQM_USE_Q: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_XCM_XQM_USE_Q (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If set the Q index; received from the QM is inserted to event\n     ID.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200f4UL: // (XCM_REG_XCM_REG0_SZ: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXCM_REG_XCM_REG0_SZ (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The size of AG context region 0 in REG-pairs. Designates \n    the MS REG-pair number (e.g. if region 0 is 6 REG-pairs; the value should\n     be 5). Is used to determine the number of the AG context REG-pairs written\n     back; when the Reg1WbFlg isn't set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200f8UL: // (XCM_REG_STOP_DONE_HDR: ACCESS:RW  DataWidth:0x1c ) 
			sprintf(disp_str, "\nXCM_REG_STOP_DONE_HDR (ACCESS:RW  DataWidth:0x1c )" \
				"\nDescription: The CM header for Timers expiration command; when Stop Done\n     input bit is set.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x200fcUL: // (XCM_REG_XQM_BYP_ACT_UPD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXCM_REG_XQM_BYP_ACT_UPD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The value by which CFC updates the activity counter at QM\n     bypass.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20110UL: // (XCM_REG_PERSIST_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_PERSIST_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index on registration on Persist counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20114UL: // (XCM_REG_FORCE_PURE_ACK_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_FORCE_PURE_ACK_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on Force ACK counter flag\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20120UL: // (XCM_REG_UNA_GT_NXT_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_UNA_GT_NXT_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on UNA greater NXT decision\n     rule.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20124UL: // (XCM_REG_SEND_KA_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_SEND_KA_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on SendKa counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20128UL: // (XCM_REG_SET_KA_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_SET_KA_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on SetKa counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2012cUL: // (XCM_REG_DA_EXPR_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DA_EXPR_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on Da Timet expiration flag\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20130UL: // (XCM_REG_DA_CNT_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DA_CNT_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on Da counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20134UL: // (XCM_REG_AUX1_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX1_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on Aux1 counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20138UL: // (XCM_REG_AUX2_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX2_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on Aux2 counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2013cUL: // (XCM_REG_AUX3_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX3_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on Aux3 counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20140UL: // (XCM_REG_AUX4_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX4_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on Aux4 counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20144UL: // (XCM_REG_DEC_RULE_Q_1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DEC_RULE_Q_1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20148UL: // (XCM_REG_DEC_RULE_Q_2: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DEC_RULE_Q_2 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2014cUL: // (XCM_REG_DEC_RULE_Q_3: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DEC_RULE_Q_3 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20150UL: // (XCM_REG_DEC_RULE_Q_4: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DEC_RULE_Q_4 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20154UL: // (XCM_REG_DEC_RULE_Q_5: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DEC_RULE_Q_5 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20158UL: // (XCM_REG_DEC_RULE_Q_6: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DEC_RULE_Q_6 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2015cUL: // (XCM_REG_DEC_RULE_Q_7: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DEC_RULE_Q_7 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20160UL: // (XCM_REG_DEC_RULE_Q_8: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DEC_RULE_Q_8 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20164UL: // (XCM_REG_DEC_RULE_Q_10: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DEC_RULE_Q_10 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20168UL: // (XCM_REG_DEC_RULE_Q_11: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DEC_RULE_Q_11 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2016cUL: // (XCM_REG_GOTO_SS_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_GOTO_SS_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on GoToSs counter flag.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20170UL: // (XCM_REG_FAST_RXMIT_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_FAST_RXMIT_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queue index for registration on Fast retransmit.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20174UL: // (XCM_REG_MORE2_SEND_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_MORE2_SEND_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: The queu index for registration on More2Send rule.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20178UL: // (XCM_REG_AUX_CNT_FLG_Q_5: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_5 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2017cUL: // (XCM_REG_AUX_CNT_FLG_Q_6: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_6 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20180UL: // (XCM_REG_AUX_CNT_FLG_Q_7: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_7 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20184UL: // (XCM_REG_AUX_CNT_FLG_Q_8: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_8 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20188UL: // (XCM_REG_AUX_CNT_FLG_Q_9: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_9 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2018cUL: // (XCM_REG_AUX_CNT_FLG_Q_10: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_10 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20190UL: // (XCM_REG_AUX_CNT_FLG_Q_11: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_11 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20194UL: // (XCM_REG_AUX_CNT_FLG_Q_12: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_12 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20198UL: // (XCM_REG_AUX_CNT_FLG_Q_13: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_13 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2019cUL: // (XCM_REG_AUX_CNT_FLG_Q_14: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_14 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201a0UL: // (XCM_REG_AUX_CNT_FLG_Q_15: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_15 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201a4UL: // (XCM_REG_AUX_CNT_FLG_Q_16: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_16 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201a8UL: // (XCM_REG_AUX_CNT_FLG_Q_17: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_17 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201acUL: // (XCM_REG_AUX_CNT_FLG_Q_18: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_18 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201b0UL: // (XCM_REG_AUX_CNT_FLG_Q_19: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_19 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201b4UL: // (XCM_REG_AUX_CNT_FLG_Q_20: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_20 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201b8UL: // (XCM_REG_AUX_CNT_FLG_Q_21: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_AUX_CNT_FLG_Q_21 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Per each decision rule the queue index to register to.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201bcUL: // (XCM_REG_TX_FIN_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_TX_FIN_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Queue index for TxFin flag decision rule.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201c0UL: // (XCM_REG_DQ_SPR_FLG_Q: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_DQ_SPR_FLG_Q (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Queue index for TxSpare flag decision rule.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201f4UL: // (XCM_REG_SM_CTX0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_SM_CTX0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of STORM context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201f8UL: // (XCM_REG_SM_CTX1_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXCM_REG_SM_CTX1_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM bits of STORM context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201fcUL: // (XCM_REG_AG_CTX00_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_AG_CTX00_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20204UL: // (XCM_REG_AG_CTX10_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_AG_CTX10_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2020cUL: // (XCM_REG_GR_ARB_TYPE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_GR_ARB_TYPE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Arbitratiojn between Input Arbiter groups: 0 - fair Round\n    -Robin; 1 - strict priority defined by ~xcm_registers_gr_ag_pr.gr_ag_pr\n    ; ~xcm_registers_gr_ld0_pr.gr_ld0_pr and ~xcm_registers_gr_ld1_pr.gr_ld1_pr\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20210UL: // (XCM_REG_GR_AG_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_GR_AG_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: AG channel group priority. The lowest priority is 0; the \n    highest priority is 3. It is supposed that the Channel group is the compliment\n     of the other 3 groups.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20214UL: // (XCM_REG_GR_LD0_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_GR_LD0_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Load (FIC0) channel group priority. The lowest priority is\n     0; the highest priority is 3. It is supposed that the Channel group is\n     the compliment of the other 3 groups.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20218UL: // (XCM_REG_GR_LD1_PR: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_GR_LD1_PR (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: Load (FIC1) channel group priority. The lowest priority is\n     0; the highest priority is 3. It is supposed that the Channel group is\n     the compliment of the other 3 groups.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2021cUL: // (XCM_REG_STORM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_STORM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set at message length mismatch (relative to last indication\n    ) at the STORM interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20220UL: // (XCM_REG_XSDM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_XSDM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set at message length mismatch (relative to last indication\n    ) at the SDM interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20224UL: // (XCM_REG_TSEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_TSEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set at message length mismatch (relative to last indication\n    ) at the tsem interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20228UL: // (XCM_REG_CSEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_CSEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set at message length mismatch (relative to last indication\n    ) at the csem interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2022cUL: // (XCM_REG_USEM_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_USEM_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Message length mismatch (relative to last indication) at \n    the usem interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20230UL: // (XCM_REG_DORQ_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_DORQ_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set at message length mismatch (relative to last indication\n    ) at the dorq interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20234UL: // (XCM_REG_PBF_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_PBF_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set at message length mismatch (relative to last indication\n    ) at the pbf interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20238UL: // (XCM_REG_NIG0_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_NIG0_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set at message length mismatch (relative to last indication\n    ) at the nig0 interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2023cUL: // (XCM_REG_NIG1_LENGTH_MIS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_NIG1_LENGTH_MIS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set at message length mismatch (relative to last indication\n    ) at the nig1 interface.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20240UL: // (XCM_REG_XX_FREE: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXCM_REG_XX_FREE (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Used to read the XX protection Free counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20244UL: // (XCM_REG_CAM_OCCUP: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_CAM_OCCUP (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: Used to read the XX protection CAM occupancy counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20248UL: // (XCM_REG_UNLOCK_MISS: ACCESS:RC  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_UNLOCK_MISS (ACCESS:RC  DataWidth:0x1 )" \
				"\nDescription: Set when the error; indicating the LCID to be unlocked doesn\n    't exist in LCID CAM; happens.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2024cUL: // (XCM_REG_XQM_GLB_USE_CNTR: ACCESS:R  DataWidth:0x1a ) 
			sprintf(disp_str, "\nXCM_REG_XQM_GLB_USE_CNTR (ACCESS:R  DataWidth:0x1a )" \
				"\nDescription: Used to read the QM global usage counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20250UL: // (XCM_REG_CP_BUF_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_CP_BUF_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: CP buffer is empty indication.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20254UL: // (XCM_REG_CP_BUF_STATUS: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_CP_BUF_STATUS (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: CP buffer status.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20258UL: // (XCM_REG_XX_OVFL_CNTR: ACCESS:ST  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXCM_REG_XX_OVFL_CNTR (ACCESS:ST  DataWidth:0x10 )" \
				"\nDescription: Counter of XX 0verflow occurencies.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2025cUL: // (XCM_REG_STORM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_STORM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the STORM input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20260UL: // (XCM_REG_XSDM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_XSDM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input SDM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20264UL: // (XCM_REG_TSEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_TSEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input tsem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20268UL: // (XCM_REG_CSEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_CSEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input csem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2026cUL: // (XCM_REG_USEM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_USEM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input usem.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20270UL: // (XCM_REG_DORQ_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_DORQ_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input dorq.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20274UL: // (XCM_REG_PBF_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_PBF_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input pbf.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20278UL: // (XCM_REG_NIG0_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_NIG0_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input nig0.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2027cUL: // (XCM_REG_NIG1_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_NIG1_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the input nig1.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20280UL: // (XCM_REG_CP_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_CP_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the CP input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20284UL: // (XCM_REG_XQM_P_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_XQM_P_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the QM input (primary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20288UL: // (XCM_REG_XQM_S_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_XQM_S_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the QM input (secondary).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2028cUL: // (XCM_REG_TM_MSG_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_TM_MSG_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the input messages at the Timers input.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20290UL: // (XCM_REG_STORM_OUT_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_STORM_OUT_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the output messages at FIC0 and FIC1 interfaces\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20294UL: // (XCM_REG_XQM_OUT_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_XQM_OUT_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the output QM commands.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20298UL: // (XCM_REG_TM_OUT_CNTR: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXCM_REG_TM_OUT_CNTR (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Counter of the output Timers commands.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2029cUL: // (XCM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from XCM\n     to the DBG block) - for selecting a line to output to the DBG block.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202a0UL: // (XCM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from XCM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select; before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202a4UL: // (XCM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXCM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from XCM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202a8UL: // (XCM_REG_XCM_INT_STS: ACCESS:R  DataWidth:0xe ) 
			sprintf(disp_str, "\nXCM_REG_XCM_INT_STS (ACCESS:R  DataWidth:0xe )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_UM_ERR) >> 13)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_CP1_ERR) >> 12)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_CP0_ERR) >> 11)
				, "\n\tNIG1_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_NIG1_ERR) >> 10)
				, "\n\tNIG0_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_NIG0_ERR) >> 9)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_PBF_ERR) >> 8)
				, "\n\tDORQ_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_DORQ_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_USEM_ERR) >> 6)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_CSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_TSEM_ERR) >> 4)
				, "\n\tXSDM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_XSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XCM_XCM_INT_STS_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x202acUL: // (XCM_REG_XCM_INT_STS_CLR: ACCESS:RC  DataWidth:0xe ) 
			sprintf(disp_str, "\nXCM_REG_XCM_INT_STS_CLR (ACCESS:RC  DataWidth:0xe )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_UM_ERR) >> 13)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_CP1_ERR) >> 12)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_CP0_ERR) >> 11)
				, "\n\tNIG1_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_NIG1_ERR) >> 10)
				, "\n\tNIG0_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_NIG0_ERR) >> 9)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_PBF_ERR) >> 8)
				, "\n\tDORQ_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_DORQ_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_USEM_ERR) >> 6)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_CSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_TSEM_ERR) >> 4)
				, "\n\tXSDM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_XSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XCM_XCM_INT_STS_CLR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x202b0UL: // (XCM_REG_XCM_INT_STS_WR: ACCESS:WR  DataWidth:0xe ) 
			sprintf(disp_str, "\nXCM_REG_XCM_INT_STS_WR (ACCESS:WR  DataWidth:0xe )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_UM_ERR) >> 13)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_CP1_ERR) >> 12)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_CP0_ERR) >> 11)
				, "\n\tNIG1_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_NIG1_ERR) >> 10)
				, "\n\tNIG0_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_NIG0_ERR) >> 9)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_PBF_ERR) >> 8)
				, "\n\tDORQ_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_DORQ_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_USEM_ERR) >> 6)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_CSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_TSEM_ERR) >> 4)
				, "\n\tXSDM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_XSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XCM_XCM_INT_STS_WR_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x202b4UL: // (XCM_REG_XCM_INT_MASK: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nXCM_REG_XCM_INT_MASK (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_UM_ERR) >> 13)
				, "\n\tCP1_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_CP1_ERR) >> 12)
				, "\n\tCP0_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_CP0_ERR) >> 11)
				, "\n\tNIG1_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_NIG1_ERR) >> 10)
				, "\n\tNIG0_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_NIG0_ERR) >> 9)
				, "\n\tPBF_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_PBF_ERR) >> 8)
				, "\n\tDORQ_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_DORQ_ERR) >> 7)
				, "\n\tUSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_USEM_ERR) >> 6)
				, "\n\tCSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_CSEM_ERR) >> 5)
				, "\n\tTSEM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_TSEM_ERR) >> 4)
				, "\n\tXSDM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_XSDM_ERR) >> 3)
				, "\n\tSTORM_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_STORM_ERR) >> 2)
				, "\n\tXX_UQ_ERR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_XX_UQ_ERR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XCM_XCM_INT_MASK_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x202b8UL: // (XCM_REG_XCM_PRTY_STS: ACCESS:R  DataWidth:0x1e ) 
			sprintf(disp_str, "\nXCM_REG_XCM_PRTY_STS (ACCESS:R  DataWidth:0x1e )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_UM_PRTY) >> 29)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_CP1_PRTY) >> 28)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_CP0_PRTY) >> 27)
				, "\n\tNIG1_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_NIG1_PRTY) >> 26)
				, "\n\tNIG0_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_NIG0_PRTY) >> 25)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_PBF_PRTY) >> 24)
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_DORQ_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_USEM_PRTY) >> 22)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_CSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_TSEM_PRTY) >> 20)
				, "\n\tXSDM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_XSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x202bcUL: // (XCM_REG_XCM_PRTY_STS_CLR: ACCESS:RC  DataWidth:0x1e ) 
			sprintf(disp_str, "\nXCM_REG_XCM_PRTY_STS_CLR (ACCESS:RC  DataWidth:0x1e )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_UM_PRTY) >> 29)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_CP1_PRTY) >> 28)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_CP0_PRTY) >> 27)
				, "\n\tNIG1_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_NIG1_PRTY) >> 26)
				, "\n\tNIG0_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_NIG0_PRTY) >> 25)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_PBF_PRTY) >> 24)
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_DORQ_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_USEM_PRTY) >> 22)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_CSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_TSEM_PRTY) >> 20)
				, "\n\tXSDM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_XSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x202c0UL: // (XCM_REG_XCM_PRTY_STS_WR: ACCESS:WR  DataWidth:0x1e ) 
			sprintf(disp_str, "\nXCM_REG_XCM_PRTY_STS_WR (ACCESS:WR  DataWidth:0x1e )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_UM_PRTY) >> 29)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_CP1_PRTY) >> 28)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_CP0_PRTY) >> 27)
				, "\n\tNIG1_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_NIG1_PRTY) >> 26)
				, "\n\tNIG0_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_NIG0_PRTY) >> 25)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_PBF_PRTY) >> 24)
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_DORQ_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_USEM_PRTY) >> 22)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_CSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_TSEM_PRTY) >> 20)
				, "\n\tXSDM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_XSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XCM_XCM_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x202c4UL: // (XCM_REG_XCM_PRTY_MASK: ACCESS:RW  DataWidth:0x1e ) 
			sprintf(disp_str, "\nXCM_REG_XCM_PRTY_MASK (ACCESS:RW  DataWidth:0x1e )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_UM_PRTY) >> 29)
				, "\n\tCP1_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_CP1_PRTY) >> 28)
				, "\n\tCP0_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_CP0_PRTY) >> 27)
				, "\n\tNIG1_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_NIG1_PRTY) >> 26)
				, "\n\tNIG0_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_NIG0_PRTY) >> 25)
				, "\n\tPBF_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_PBF_PRTY) >> 24)
				, "\n\tDORQ_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_DORQ_PRTY) >> 23)
				, "\n\tUSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_USEM_PRTY) >> 22)
				, "\n\tCSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_CSEM_PRTY) >> 21)
				, "\n\tTSEM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_TSEM_PRTY) >> 20)
				, "\n\tXSDM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_XSDM_PRTY) >> 19)
				, "\n\tSTORM_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_STORM_PRTY) >> 18)
				, "\n\tSM_PRTY3 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_SM_PRTY3) >> 17)
				, "\n\tSM_PRTY2 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_SM_PRTY2) >> 16)
				, "\n\tSM_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_SM_PRTY1) >> 15)
				, "\n\tSM_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_SM_PRTY0) >> 14)
				, "\n\tAG_PRTY7 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_AG_PRTY7) >> 13)
				, "\n\tAG_PRTY6 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_AG_PRTY6) >> 12)
				, "\n\tAG_PRTY5 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_AG_PRTY5) >> 11)
				, "\n\tAG_PRTY4 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_AG_PRTY4) >> 10)
				, "\n\tAG_PRTY3 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_AG_PRTY3) >> 9)
				, "\n\tAG_PRTY2 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_AG_PRTY2) >> 8)
				, "\n\tAG_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_AG_PRTY1) >> 7)
				, "\n\tAG_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_AG_PRTY0) >> 6)
				, "\n\tUQ_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_UQ_PRTY) >> 5)
				, "\n\tPM_PRTY1 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_PM_PRTY1) >> 4)
				, "\n\tPM_PRTY0 :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_PM_PRTY0) >> 3)
				, "\n\tDT_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_DT_PRTY) >> 2)
				, "\n\tXT_PRTY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_XT_PRTY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XCM_XCM_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x202c8UL: // (XCM_REG_IS_STORM_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_IS_STORM_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits of input message STORM buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202ccUL: // (XCM_REG_SM_CTX01_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXCM_REG_SM_CTX01_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM bits of STORM context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202d0UL: // (XCM_REG_SM_CTX11_TM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXCM_REG_SM_CTX11_TM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: TM bits of STORM context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202d4UL: // (XCM_REG_IS_UM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_IS_UM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of input stage UM buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202d8UL: // (XCM_REG_PM_RAM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_PM_RAM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of Pending messages RAM.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202dcUL: // (XCM_REG_IS_NIG0_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_IS_NIG0_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits of input stage NIG0 buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202e0UL: // (XCM_REG_IS_NIG1_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_IS_NIG1_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits of input stage NIG1 buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202e4UL: // (XCM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: chicken bits\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202e8UL: // (XCM_REG_IS_USEM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_IS_USEM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of input stage USEM buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202ecUL: // (XCM_REG_IS_TSEM_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXCM_REG_IS_TSEM_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits of input stage TSEM buffer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202f0UL: // (XCM_REG_UM_FIC1_FORCE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXCM_REG_UM_FIC1_FORCE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: 0-messages unlocked from Pending messages RAM go to the FIC\n     for which they were designated in input message; 1-messages unlocked \n    from Pending messages RAM are forced to FIC1 whether they were destined\n     to FIC0 or FIC1 in original message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20100UL: // (XCM_REG_PHYS_QNUM3_0: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_PHYS_QNUM3_0 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 3 per port index (CID[23])\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20104UL: // (XCM_REG_PHYS_QNUM3_1: ACCESS:RW  DataWidth:0x7  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_PHYS_QNUM3_1 (ACCESS:RW  DataWidth:0x7  SPLIT:4 )" \
				"\nDescription: The physical queue number 3 per port index (CID[23])\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20108UL: // (XCM_REG_GLB_DEL_ACK_TMR_VAL_0: ACCESS:RW  DataWidth:0x1c  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_GLB_DEL_ACK_TMR_VAL_0 (ACCESS:RW  DataWidth:0x1c  SPLIT:4 )" \
				"\nDescription: The delayed ACK timeout in ticks. Per port value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2010cUL: // (XCM_REG_GLB_DEL_ACK_TMR_VAL_1: ACCESS:RW  DataWidth:0x1c  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_GLB_DEL_ACK_TMR_VAL_1 (ACCESS:RW  DataWidth:0x1c  SPLIT:4 )" \
				"\nDescription: The delayed ACK timeout in ticks. Per port value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20118UL: // (XCM_REG_GLB_DEL_ACK_MAX_CNT_0: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_GLB_DEL_ACK_MAX_CNT_0 (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: The maximum delayed ACK counter value.Must be at least 2. \n    Per port value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2011cUL: // (XCM_REG_GLB_DEL_ACK_MAX_CNT_1: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_GLB_DEL_ACK_MAX_CNT_1 (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: The maximum delayed ACK counter value.Must be at least 2. \n    Per port value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201c4UL: // (XCM_REG_WU_DA_SET_TMR_CNT_FLG_CMD00: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_SET_TMR_CNT_FLG_CMD00 (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: DA timer command; used in case of window update doorbell.\n    The first index stands for the value DaEnable of that connection. The \n    second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201c8UL: // (XCM_REG_WU_DA_SET_TMR_CNT_FLG_CMD01: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_SET_TMR_CNT_FLG_CMD01 (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: DA timer command; used in case of window update doorbell.\n    The first index stands for the value DaEnable of that connection. The \n    second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201ccUL: // (XCM_REG_WU_DA_SET_TMR_CNT_FLG_CMD10: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_SET_TMR_CNT_FLG_CMD10 (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: DA timer command; used in case of window update doorbell.\n    The first index stands for the value DaEnable of that connection. The \n    second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201d0UL: // (XCM_REG_WU_DA_SET_TMR_CNT_FLG_CMD11: ACCESS:RW  DataWidth:0x1  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_SET_TMR_CNT_FLG_CMD11 (ACCESS:RW  DataWidth:0x1  SPLIT:4 )" \
				"\nDescription: DA timer command; used in case of window update doorbell.\n    The first index stands for the value DaEnable of that connection. The \n    second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201d4UL: // (XCM_REG_WU_DA_CNT_CMD00: ACCESS:RW  DataWidth:0x2  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_CNT_CMD00 (ACCESS:RW  DataWidth:0x2  SPLIT:4 )" \
				"\nDescription: DA counter command; used in case of window update doorbell\n    .The first index stands for the value DaEnable of that connection. The\n     second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201d8UL: // (XCM_REG_WU_DA_CNT_CMD01: ACCESS:RW  DataWidth:0x2  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_CNT_CMD01 (ACCESS:RW  DataWidth:0x2  SPLIT:4 )" \
				"\nDescription: DA counter command; used in case of window update doorbell\n    .The first index stands for the value DaEnable of that connection. The\n     second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201dcUL: // (XCM_REG_WU_DA_CNT_CMD10: ACCESS:RW  DataWidth:0x2  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_CNT_CMD10 (ACCESS:RW  DataWidth:0x2  SPLIT:4 )" \
				"\nDescription: DA counter command; used in case of window update doorbell\n    .The first index stands for the value DaEnable of that connection. The\n     second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201e0UL: // (XCM_REG_WU_DA_CNT_CMD11: ACCESS:RW  DataWidth:0x2  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_CNT_CMD11 (ACCESS:RW  DataWidth:0x2  SPLIT:4 )" \
				"\nDescription: DA counter command; used in case of window update doorbell\n    .The first index stands for the value DaEnable of that connection. The\n     second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201e4UL: // (XCM_REG_WU_DA_CNT_UPD_VAL00: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_CNT_UPD_VAL00 (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: DA counter update value used in case of window update doorbell\n    .The first index stands for the value DaEnable of that connection. The\n     second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201e8UL: // (XCM_REG_WU_DA_CNT_UPD_VAL01: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_CNT_UPD_VAL01 (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: DA counter update value; used in case of window update doorbell\n    .The first index stands for the value DaEnable of that connection. The\n     second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201ecUL: // (XCM_REG_WU_DA_CNT_UPD_VAL10: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_CNT_UPD_VAL10 (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: DA counter update value; used in case of window update doorbell\n    .The first index stands for the value DaEnable of that connection. The\n     second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x201f0UL: // (XCM_REG_WU_DA_CNT_UPD_VAL11: ACCESS:RW  DataWidth:0x8  SPLIT:4 ) 
			sprintf(disp_str, "\nXCM_REG_WU_DA_CNT_UPD_VAL11 (ACCESS:RW  DataWidth:0x8  SPLIT:4 )" \
				"\nDescription: DA counter update value; used in case of window update doorbell\n    .The first index stands for the value DaEnable of that connection. The\n     second index stands for port number. The access to 4 registers corresponding\n     to 4 functions is arranged.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20400UL: // (XCM_REG_CAM_OCCUP_ST: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXCM_REG_CAM_OCCUP_ST (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: CAM occupancy sticky status. The write to the register is\n     performed by the XX internal circuitry.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20404UL: // (XCM_REG_CFC_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXCM_REG_CFC_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: CFC output initial credit. Max credit available - 15.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 1 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20408UL: // (XCM_REG_CP_MSG: ACCESS:W  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXCM_REG_CP_MSG (ACCESS:W  DataWidth:0x20 )" \
				"\nDescription: Used to write the CP message. Write only.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2040cUL: // (XCM_REG_FIC0_INIT_CRD: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_FIC0_INIT_CRD (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: FIC0 output initial credit. Max credit available - 255.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 64 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20410UL: // (XCM_REG_FIC1_INIT_CRD: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXCM_REG_FIC1_INIT_CRD (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: FIC1 output initial credit. Max credit available - 255.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 64 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20414UL: // (XCM_REG_GLB_CNT_STICKY: ACCESS:RW  DataWidth:0x1a ) 
			sprintf(disp_str, "\nXCM_REG_GLB_CNT_STICKY (ACCESS:RW  DataWidth:0x1a )" \
				"\nDescription: QM global usage counter maximum sticky value.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20418UL: // (XCM_REG_LL_SZ_STICKY: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXCM_REG_LL_SZ_STICKY (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: XX LL maximum value ever reached sticky value for any connection\n    . It has no write access. The write to the register is performed by the\n     XX internal circuitry.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2041cUL: // (XCM_REG_TM_INIT_CRD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXCM_REG_TM_INIT_CRD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Timers output initial credit. Max credit available - 15.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 4 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20420UL: // (XCM_REG_XQM_INIT_CRD: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXCM_REG_XQM_INIT_CRD (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: QM output initial credit. Max credit available - 32.Write\n     writes the initial credit value; read returns the current value of the\n     credit counter. Must be initialized to 32 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20424UL: // (XCM_REG_XX_INIT_CRD: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXCM_REG_XX_INIT_CRD (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Initial value for the credit counter; responsible for fulfilling\n     of the Input Stage XX protection buffer by the XX protection pending \n    messages. Max credit available - 3.Write writes the initial credit value\n    ; read returns the current value of the credit counter. Must be initialized\n     to 2 at start-up.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20428UL: // (XCM_REG_XX_MSG_NUM: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXCM_REG_XX_MSG_NUM (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The maximum number of pending messages; which may be stored\n     in XX protection. ~xcm_registers_xx_free.xx_free is also written on write\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2042cUL: // (XCM_REG_LCID_CAM_0: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_0 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20430UL: // (XCM_REG_LCID_CAM_1: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_1 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20434UL: // (XCM_REG_LCID_CAM_2: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_2 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20438UL: // (XCM_REG_LCID_CAM_3: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_3 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2043cUL: // (XCM_REG_LCID_CAM_4: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_4 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20440UL: // (XCM_REG_LCID_CAM_5: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_5 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20444UL: // (XCM_REG_LCID_CAM_6: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_6 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20448UL: // (XCM_REG_LCID_CAM_7: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_7 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2044cUL: // (XCM_REG_LCID_CAM_8: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_8 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20450UL: // (XCM_REG_LCID_CAM_9: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_9 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20454UL: // (XCM_REG_LCID_CAM_10: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_10 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20458UL: // (XCM_REG_LCID_CAM_11: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_11 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2045cUL: // (XCM_REG_LCID_CAM_12: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_12 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20460UL: // (XCM_REG_LCID_CAM_13: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_13 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20464UL: // (XCM_REG_LCID_CAM_14: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_14 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20468UL: // (XCM_REG_LCID_CAM_15: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_15 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2046cUL: // (XCM_REG_LCID_CAM_16: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_16 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20470UL: // (XCM_REG_LCID_CAM_17: ACCESS:R  DataWidth:0x9 ) 
			sprintf(disp_str, "\nXCM_REG_LCID_CAM_17 (ACCESS:R  DataWidth:0x9 )" \
				"\nDescription: Debug only. Read access to LCID CAM in XX protection mechanism\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20480UL: // (XCM_REG_XX_DESCR_TABLE: ACCESS:RW  DataWidth:0x11 ) 
			sprintf(disp_str, "\nXCM_REG_XX_DESCR_TABLE (ACCESS:RW  DataWidth:0x11 )" \
				"\nDescription: Indirect access to the descriptor table of the XX protection\n     mechanism. The fields are: [5:0] - message length;[11:6] - message pointer\n    ;[16:12] - next pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20500UL: // (XCM_REG_XX_TABLE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXCM_REG_XX_TABLE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Indirect access to the XX table of the XX protection mechanism\n    . The fields are:[4:0] - tail pointer;[10:5] - Link List size;[15:11] - \n    header pointer.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20600UL: // (XCM_REG_XX_PEND_MSG: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXCM_REG_XX_PEND_MSG (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indirect access to the Pending messages RAM of the XX protection\n     mechanism.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28000UL: // (XCM_REG_AG_CTX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXCM_REG_AG_CTX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indirect access to AG context with 32-bits granularity. The\n     bits [12:8] of the address should be the offset within the accessed LCID\n     context; the bits [7:0] are the accessed LCID.Example: to write to REG10\n     LCID100. The RBC address should be 13'ha64.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x30000UL: // (XCM_REG_STORM_CTX: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXCM_REG_STORM_CTX (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Indirect access to STORM context with 32-bits granularity\n    . The bits [13:8] of the address should be the offset within the accessed\n     LCID context; the bits [7:0] are the accessed LCID.Example: to write \n    to REG10 LCID100. The RBC address should be 14'ha64.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20200UL: // (XCM_REG_AG_CTX01_TM: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXCM_REG_AG_CTX01_TM (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20208UL: // (XCM_REG_AG_CTX11_TM: ACCESS:R  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXCM_REG_AG_CTX11_TM (ACCESS:R  DataWidth:0x4 )" \
				"\nDescription: TM bits of AG context.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x202f4UL: // (XCM_REG_XCM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXCM_REG_XCM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20474UL: // (XCM_REG_XCM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXCM_REG_XCM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20580UL: // (XCM_REG_XCM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXCM_REG_XCM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x20800UL: // (XCM_REG_XCM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXCM_REG_XCM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0UL): // (XMAC_REG_CTRL: ACCESS:RW  DataWidth:0xc  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_CTRL (ACCESS:RW  DataWidth:0xc  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tXGMII_IPG_CHECK_DISABLE :"
				, (u32_t)((data & XMAC_CTRL_REG_XGMII_IPG_CHECK_DISABLE) >> 11)
				, "\n\tRS_SOFT_RESET :"
				, (u32_t)((data & XMAC_CTRL_REG_RS_SOFT_RESET) >> 10)
				, "\n\tREMOTE_LPBK_LEAK_ENB :"
				, (u32_t)((data & XMAC_CTRL_REG_REMOTE_LPBK_LEAK_ENB) >> 9)
				, "\n\tLOCAL_LPBK_LEAK_ENB :"
				, (u32_t)((data & XMAC_CTRL_REG_LOCAL_LPBK_LEAK_ENB) >> 8)
				, "\n\tXLGMII_ALIGN_ENB :"
				, (u32_t)((data & XMAC_CTRL_REG_XLGMII_ALIGN_ENB) >> 7)
				, "\n\tSOFT_RESET :"
				, (u32_t)((data & XMAC_CTRL_REG_SOFT_RESET) >> 6)
				, "\n\tCORE_REMOTE_LPBK :"
				, (u32_t)((data & XMAC_CTRL_REG_CORE_REMOTE_LPBK) >> 5)
				, "\n\tLINE_REMOTE_LPBK :"
				, (u32_t)((data & XMAC_CTRL_REG_LINE_REMOTE_LPBK) >> 4)
				, "\n\tCORE_LOCAL_LPBK :"
				, (u32_t)((data & XMAC_CTRL_REG_CORE_LOCAL_LPBK) >> 3)
				, "\n\tLINE_LOCAL_LPBK :"
				, (u32_t)((data & XMAC_CTRL_REG_LINE_LOCAL_LPBK) >> 2)
				, "\n\tRX_EN :"
				, (u32_t)((data & XMAC_CTRL_REG_RX_EN) >> 1)
				, "\n\tTX_EN :"
				, (u32_t)((data & XMAC_CTRL_REG_TX_EN) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x8UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x8UL): // (XMAC_REG_MODE: ACCESS:RW  DataWidth:0x7  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_MODE (ACCESS:RW  DataWidth:0x7  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSPEED_MODE :"
				, (u32_t)((data & XMAC_MODE_REG_SPEED_MODE) >> 4)
				, "\n\tNO_SOP_FOR_CRC_HG :"
				, (u32_t)((data & XMAC_MODE_REG_NO_SOP_FOR_CRC_HG) >> 3)
				, "\n\tHDR_MODE :"
				, (u32_t)((data & XMAC_MODE_REG_HDR_MODE) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x10UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x10UL): // (XMAC_REG_XMAC_SPARE0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_SPARE0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: SPARE REGISTERS 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x18UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x18UL): // (XMAC_REG_XMAC_SPARE1: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_SPARE1 (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: SPARE REGISTERS 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x20UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x20UL): // (XMAC_REG_TX_CTRL: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_TX_CTRL (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTHROT_DENOM_LO :"
				, (u32_t)((data & XMAC_TX_CTRL_REG_THROT_DENOM_LO) >> 25)
				, "\n\tTHROT_NUM :"
				, (u32_t)((data & XMAC_TX_CTRL_REG_THROT_NUM) >> 19)
				, "\n\tAVERAGE_IPG :"
				, (u32_t)((data & XMAC_TX_CTRL_REG_AVERAGE_IPG) >> 12)
				, "\n\tPAD_THRESHOLD :"
				, (u32_t)((data & XMAC_TX_CTRL_REG_PAD_THRESHOLD) >> 5)
				, "\n\tPAD_EN :"
				, (u32_t)((data & XMAC_TX_CTRL_REG_PAD_EN) >> 4)
				, "\n\tTX_ANY_START :"
				, (u32_t)((data & XMAC_TX_CTRL_REG_TX_ANY_START) >> 3)
				, "\n\tDISCARD :"
				, (u32_t)((data & XMAC_TX_CTRL_REG_DISCARD) >> 2)
				, "\n\tCRC_MODE :"
				, (u32_t)((data & XMAC_TX_CTRL_REG_CRC_MODE) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x24UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x24UL): // (XMAC_REG_TX_CTRL_HI: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_TX_CTRL_HI (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTX_64BYTE_BUFFER_EN :"
				, (u32_t)((data & XMAC_TX_CTRL_HI_REG_TX_64BYTE_BUFFER_EN) >> 5)
				, "\n\tTX_PREAMBLE_LENGTH :"
				, (u32_t)((data & XMAC_TX_CTRL_HI_REG_TX_PREAMBLE_LENGTH) >> 1)
				, "\n\tTHROT_DENOM_HI :"
				, (u32_t)((data & XMAC_TX_CTRL_HI_REG_THROT_DENOM_HI) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x28UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x28UL): // (XMAC_REG_CTRL_SA_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXMAC_REG_CTRL_SA_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Lower 48 bits of ctrl_sa register. Used as the SA in PAUSE\n    /PFC packets transmitted by the MAC \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x2cUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x2cUL): // (XMAC_REG_CTRL_SA_HI: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXMAC_REG_CTRL_SA_HI (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Upper 48 bits of ctrl_sa register. Used as the SA in PAUSE\n    /PFC packets transmitted by the MAC \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x30UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x30UL): // (XMAC_REG_RX_CTRL: ACCESS:RW  DataWidth:0xd  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_RX_CTRL (ACCESS:RW  DataWidth:0xd  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPROCESS_VARIABLE_PREAMBLE :"
				, (u32_t)((data & XMAC_RX_CTRL_REG_PROCESS_VARIABLE_PREAMBLE) >> 12)
				, "\n\tRECEIVE_18_BYTE_PKTS :"
				, (u32_t)((data & XMAC_RX_CTRL_REG_RECEIVE_18_BYTE_PKTS) >> 11)
				, "\n\tRUNT_THRESHOLD :"
				, (u32_t)((data & XMAC_RX_CTRL_REG_RUNT_THRESHOLD) >> 4)
				, "\n\tSTRICT_PREAMBLE :"
				, (u32_t)((data & XMAC_RX_CTRL_REG_STRICT_PREAMBLE) >> 3)
				, "\n\tSTRIP_CRC :"
				, (u32_t)((data & XMAC_RX_CTRL_REG_STRIP_CRC) >> 2)
				, "\n\tRX_ANY_START :"
				, (u32_t)((data & XMAC_RX_CTRL_REG_RX_ANY_START) >> 1)
				, "\n\tRX_PASS_CTRL :"
				, (u32_t)((data & XMAC_RX_CTRL_REG_RX_PASS_CTRL) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x38UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x38UL): // (XMAC_REG_RX_SA_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXMAC_REG_RX_SA_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Lower 48 bits of rx_sa register. SA recognized for MAC control\n     packets in addition to the standard 0x0180C2000001 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x3cUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x3cUL): // (XMAC_REG_RX_SA_HI: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXMAC_REG_RX_SA_HI (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Upper 48 bits of rx_sa register. SA recognized for MAC control\n     packets in addition to the standard 0x0180C2000001 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x48UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x48UL): // (XMAC_REG_RX_VLAN_TAG: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_RX_VLAN_TAG (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tOUTER_VLAN_TAG :"
				, (u32_t)((data & XMAC_RX_VLAN_TAG_REG_OUTER_VLAN_TAG) >> 16)
				, "\n\tINNER_VLAN_TAG :"
				, (u32_t)((data & XMAC_RX_VLAN_TAG_REG_INNER_VLAN_TAG) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x4cUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x4cUL): // (XMAC_REG_RX_VLAN_TAG_HI: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_RX_VLAN_TAG_HI (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tOUTER_VLAN_TAG_ENABLE :"
				, (u32_t)((data & XMAC_RX_VLAN_TAG_HI_REG_OUTER_VLAN_TAG_ENABLE) >> 1)
				, "\n\tINNER_VLAN_TAG_ENABLE :"
				, (u32_t)((data & XMAC_RX_VLAN_TAG_HI_REG_INNER_VLAN_TAG_ENABLE) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x50UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x50UL): // (XMAC_REG_RX_LSS_CTRL: ACCESS:RW  DataWidth:0x3  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_RX_LSS_CTRL (ACCESS:RW  DataWidth:0x3  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tUSE_EXTERNAL_FAULTS_FOR_TX :"
				, (u32_t)((data & XMAC_RX_LSS_CTRL_REG_USE_EXTERNAL_FAULTS_FOR_TX) >> 2)
				, "\n\tREMOTE_FAULT_DISABLE :"
				, (u32_t)((data & XMAC_RX_LSS_CTRL_REG_REMOTE_FAULT_DISABLE) >> 1)
				, "\n\tLOCAL_FAULT_DISABLE :"
				, (u32_t)((data & XMAC_RX_LSS_CTRL_REG_LOCAL_FAULT_DISABLE) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x58UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x58UL): // (XMAC_REG_RX_LSS_STATUS: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_RX_LSS_STATUS (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tREMOTE_FAULT_STATUS :"
				, (u32_t)((data & XMAC_RX_LSS_STATUS_REG_REMOTE_FAULT_STATUS) >> 1)
				, "\n\tLOCAL_FAULT_STATUS :"
				, (u32_t)((data & XMAC_RX_LSS_STATUS_REG_LOCAL_FAULT_STATUS) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x60UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x60UL): // (XMAC_REG_CLEAR_RX_LSS_STATUS: ACCESS:RW  DataWidth:0x2  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_CLEAR_RX_LSS_STATUS (ACCESS:RW  DataWidth:0x2  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCLEAR_REMOTE_FAULT_STATUS :"
				, (u32_t)((data & XMAC_CLEAR_RX_LSS_STATUS_REG_CLEAR_REMOTE_FAULT_STATUS) >> 1)
				, "\n\tCLEAR_LOCAL_FAULT_STATUS :"
				, (u32_t)((data & XMAC_CLEAR_RX_LSS_STATUS_REG_CLEAR_LOCAL_FAULT_STATUS) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x68UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x68UL): // (XMAC_REG_PAUSE_CTRL: ACCESS:RW  DataWidth:0x15  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_PAUSE_CTRL (ACCESS:RW  DataWidth:0x15  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPAUSE_GMII_ON_TX_LINE_SIDE :"
				, (u32_t)((data & XMAC_PAUSE_CTRL_REG_PAUSE_GMII_ON_TX_LINE_SIDE) >> 20)
				, "\n\tRX_PASS_PAUSE :"
				, (u32_t)((data & XMAC_PAUSE_CTRL_REG_RX_PASS_PAUSE) >> 19)
				, "\n\tRX_PAUSE_EN :"
				, (u32_t)((data & XMAC_PAUSE_CTRL_REG_RX_PAUSE_EN) >> 18)
				, "\n\tTX_PAUSE_EN :"
				, (u32_t)((data & XMAC_PAUSE_CTRL_REG_TX_PAUSE_EN) >> 17)
				, "\n\tPAUSE_REFRESH_EN :"
				, (u32_t)((data & XMAC_PAUSE_CTRL_REG_PAUSE_REFRESH_EN) >> 16)
				, "\n\tPAUSE_REFRESH_TIMER :"
				, (u32_t)((data & XMAC_PAUSE_CTRL_REG_PAUSE_REFRESH_TIMER) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x70UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x70UL): // (XMAC_REG_PFC_CTRL: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_PFC_CTRL (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPFC_XOFF_TIMER :"
				, (u32_t)((data & XMAC_PFC_CTRL_REG_PFC_XOFF_TIMER) >> 16)
				, "\n\tPFC_REFRESH_TIMER :"
				, (u32_t)((data & XMAC_PFC_CTRL_REG_PFC_REFRESH_TIMER) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x74UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x74UL): // (XMAC_REG_PFC_CTRL_HI: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_PFC_CTRL_HI (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTX_PFC_EN :"
				, (u32_t)((data & XMAC_PFC_CTRL_HI_REG_TX_PFC_EN) >> 5)
				, "\n\tRX_PFC_EN :"
				, (u32_t)((data & XMAC_PFC_CTRL_HI_REG_RX_PFC_EN) >> 4)
				, "\n\tPFC_STATS_EN :"
				, (u32_t)((data & XMAC_PFC_CTRL_HI_REG_PFC_STATS_EN) >> 3)
				, "\n\tRX_PASS_PFC :"
				, (u32_t)((data & XMAC_PFC_CTRL_HI_REG_RX_PASS_PFC) >> 2)
				, "\n\tFORCE_PFC_XON :"
				, (u32_t)((data & XMAC_PFC_CTRL_HI_REG_FORCE_PFC_XON) >> 1)
				, "\n\tPFC_REFRESH_EN :"
				, (u32_t)((data & XMAC_PFC_CTRL_HI_REG_PFC_REFRESH_EN) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x78UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x78UL): // (XMAC_REG_PFC_ETH_TYPE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXMAC_REG_PFC_ETH_TYPE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The PFC packet generation and detection uses this Ethertype\n     value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x80UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x80UL): // (XMAC_REG_PFC_OPCODE: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXMAC_REG_PFC_OPCODE (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The PFC packet generation and detection uses this Ethertype\n     value\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x88UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x88UL): // (XMAC_REG_PFC_MACDA_LO: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXMAC_REG_PFC_MACDA_LO (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Lower 48 bits of pfc_macda register. Used as the DA in PFC\n     packets transmitted by the MAC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x8cUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x8cUL): // (XMAC_REG_PFC_MACDA_HI: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXMAC_REG_PFC_MACDA_HI (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Upper 48 bits of pfc_macda register. Used as the DA in PFC\n     packets transmitted by the MAC\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x90UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x90UL): // (XMAC_REG_LLFC_CTRL: ACCESS:RW  DataWidth:0xe  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_LLFC_CTRL (ACCESS:RW  DataWidth:0xe  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLFC_IMG :"
				, (u32_t)((data & XMAC_LLFC_CTRL_REG_LLFC_IMG) >> 6)
				, "\n\tNO_SOM_FOR_CRC_LLFC :"
				, (u32_t)((data & XMAC_LLFC_CTRL_REG_NO_SOM_FOR_CRC_LLFC) >> 5)
				, "\n\tLLFC_CRC_IGNORE :"
				, (u32_t)((data & XMAC_LLFC_CTRL_REG_LLFC_CRC_IGNORE) >> 4)
				, "\n\tLLFC_CUT_THROUGH_MODE :"
				, (u32_t)((data & XMAC_LLFC_CTRL_REG_LLFC_CUT_THROUGH_MODE) >> 3)
				, "\n\tLLFC_IN_IPG_ONLY :"
				, (u32_t)((data & XMAC_LLFC_CTRL_REG_LLFC_IN_IPG_ONLY) >> 2)
				, "\n\tRX_LLFC_EN :"
				, (u32_t)((data & XMAC_LLFC_CTRL_REG_RX_LLFC_EN) >> 1)
				, "\n\tTX_LLFC_EN :"
				, (u32_t)((data & XMAC_LLFC_CTRL_REG_TX_LLFC_EN) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x98UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x98UL): // (XMAC_REG_TX_LLFC_MSG_FIELDS: ACCESS:RW  DataWidth:0x1c  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_TX_LLFC_MSG_FIELDS (ACCESS:RW  DataWidth:0x1c  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tLLFC_XOFF_TIME :"
				, (u32_t)((data & XMAC_TX_LLFC_MSG_FIELDS_REG_LLFC_XOFF_TIME) >> 12)
				, "\n\tTX_LLFC_FC_OBJ_LOGICAL :"
				, (u32_t)((data & XMAC_TX_LLFC_MSG_FIELDS_REG_TX_LLFC_FC_OBJ_LOGICAL) >> 8)
				, "\n\tTX_LLFC_MSG_TYPE_LOGICAL :"
				, (u32_t)((data & XMAC_TX_LLFC_MSG_FIELDS_REG_TX_LLFC_MSG_TYPE_LOGICAL) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0xa0UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xa0UL): // (XMAC_REG_RX_LLFC_MSG_FIELDS: ACCESS:RW  DataWidth:0x18  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_RX_LLFC_MSG_FIELDS (ACCESS:RW  DataWidth:0x18  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRX_LLFC_FC_OBJ_PHYSICAL :"
				, (u32_t)((data & XMAC_RX_LLFC_MSG_FIELDS_REG_RX_LLFC_FC_OBJ_PHYSICAL) >> 20)
				, "\n\tRX_LLFC_MSG_TYPE_PHYSICAL :"
				, (u32_t)((data & XMAC_RX_LLFC_MSG_FIELDS_REG_RX_LLFC_MSG_TYPE_PHYSICAL) >> 12)
				, "\n\tRX_LLFC_FC_OBJ_LOGICAL :"
				, (u32_t)((data & XMAC_RX_LLFC_MSG_FIELDS_REG_RX_LLFC_FC_OBJ_LOGICAL) >> 8)
				, "\n\tRX_LLFC_MSG_TYPE_LOGICAL :"
				, (u32_t)((data & XMAC_RX_LLFC_MSG_FIELDS_REG_RX_LLFC_MSG_TYPE_LOGICAL) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0xa8UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xa8UL): // (XMAC_REG_HCFC_CTRL: ACCESS:RW  DataWidth:0x15  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_HCFC_CTRL (ACCESS:RW  DataWidth:0x15  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tHCFC_IMG :"
				, (u32_t)((data & XMAC_HCFC_CTRL_REG_HCFC_IMG) >> 13)
				, "\n\tHCFC_SOM :"
				, (u32_t)((data & XMAC_HCFC_CTRL_REG_HCFC_SOM) >> 5)
				, "\n\tHCFC_IN_IPG_ONLY :"
				, (u32_t)((data & XMAC_HCFC_CTRL_REG_HCFC_IN_IPG_ONLY) >> 4)
				, "\n\tNO_SOM_FOR_CRC_HCFC :"
				, (u32_t)((data & XMAC_HCFC_CTRL_REG_NO_SOM_FOR_CRC_HCFC) >> 3)
				, "\n\tHCFC_CRC_IGNORE :"
				, (u32_t)((data & XMAC_HCFC_CTRL_REG_HCFC_CRC_IGNORE) >> 2)
				, "\n\tRX_HCFC_EN :"
				, (u32_t)((data & XMAC_HCFC_CTRL_REG_RX_HCFC_EN) >> 1)
				, "\n\tTX_HCFC_EN :"
				, (u32_t)((data & XMAC_HCFC_CTRL_REG_TX_HCFC_EN) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0xb0UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xb0UL): // (XMAC_REG_TIME_STAMP: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXMAC_REG_TIME_STAMP (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: The TimeStamp value of the Tx packets.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xb4UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xb4UL): // (XMAC_REG_TX_TIMESTAMP_FIFO_DATA_HI: ACCESS:RW  DataWidth:0x11  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_TX_TIMESTAMP_FIFO_DATA_HI (ACCESS:RW  DataWidth:0x11  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTS_ENTRY_VALID :"
				, (u32_t)((data & XMAC_TX_TIMESTAMP_FIFO_DATA_HI_REG_TS_ENTRY_VALID) >> 16)
				, "\n\tSEQUENCE_ID :"
				, (u32_t)((data & XMAC_TX_TIMESTAMP_FIFO_DATA_HI_REG_SEQUENCE_ID) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0xc0UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xc0UL): // (XMAC_REG_FIFO_STATUS: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_FIFO_STATUS (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tTX_LLFC_MSG_OVERFLOW :"
				, (u32_t)((data & XMAC_FIFO_STATUS_REG_TX_LLFC_MSG_OVERFLOW) >> 5)
				, "\n\tTX_HCFC_MSG_OVERFLOW :"
				, (u32_t)((data & XMAC_FIFO_STATUS_REG_TX_HCFC_MSG_OVERFLOW) >> 4)
				, "\n\tTX_PKT_OVERFLOW :"
				, (u32_t)((data & XMAC_FIFO_STATUS_REG_TX_PKT_OVERFLOW) >> 3)
				, "\n\tTX_PKT_UNDERFLOW :"
				, (u32_t)((data & XMAC_FIFO_STATUS_REG_TX_PKT_UNDERFLOW) >> 2)
				, "\n\tRX_MSG_OVERFLOW :"
				, (u32_t)((data & XMAC_FIFO_STATUS_REG_RX_MSG_OVERFLOW) >> 1)
				, "\n\tRX_PKT_OVERFLOW :"
				, (u32_t)((data & XMAC_FIFO_STATUS_REG_RX_PKT_OVERFLOW) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0xc8UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xc8UL): // (XMAC_REG_CLEAR_FIFO_STATUS: ACCESS:RW  DataWidth:0x7  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_CLEAR_FIFO_STATUS (ACCESS:RW  DataWidth:0x7  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCLEAR_TX_TS_FIFO_OVERFLOW :"
				, (u32_t)((data & XMAC_CLEAR_FIFO_STATUS_REG_CLEAR_TX_TS_FIFO_OVERFLOW) >> 6)
				, "\n\tCLEAR_TX_LLFC_MSG_OVERFLOW :"
				, (u32_t)((data & XMAC_CLEAR_FIFO_STATUS_REG_CLEAR_TX_LLFC_MSG_OVERFLOW) >> 5)
				, "\n\tCLEAR_TX_HCFC_MSG_OVERFLOW :"
				, (u32_t)((data & XMAC_CLEAR_FIFO_STATUS_REG_CLEAR_TX_HCFC_MSG_OVERFLOW) >> 4)
				, "\n\tCLEAR_TX_PKT_OVERFLOW :"
				, (u32_t)((data & XMAC_CLEAR_FIFO_STATUS_REG_CLEAR_TX_PKT_OVERFLOW) >> 3)
				, "\n\tCLEAR_TX_PKT_UNDERFLOW :"
				, (u32_t)((data & XMAC_CLEAR_FIFO_STATUS_REG_CLEAR_TX_PKT_UNDERFLOW) >> 2)
				, "\n\tCLEAR_RX_MSG_OVERFLOW :"
				, (u32_t)((data & XMAC_CLEAR_FIFO_STATUS_REG_CLEAR_RX_MSG_OVERFLOW) >> 1)
				, "\n\tCLEAR_RX_PKT_OVERFLOW :"
				, (u32_t)((data & XMAC_CLEAR_FIFO_STATUS_REG_CLEAR_RX_PKT_OVERFLOW) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0xd0UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xd0UL): // (XMAC_REG_TX_FIFO_CREDITS: ACCESS:RW  DataWidth:0x12  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_TX_FIFO_CREDITS (ACCESS:RW  DataWidth:0x12  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSINGLE_PORT_TX_CREDITS :"
				, (u32_t)((data & XMAC_TX_FIFO_CREDITS_REG_SINGLE_PORT_TX_CREDITS) >> 12)
				, "\n\tDUAL_PORT_TX_CREDITS :"
				, (u32_t)((data & XMAC_TX_FIFO_CREDITS_REG_DUAL_PORT_TX_CREDITS) >> 6)
				, "\n\tQUAD_PORT_TX_CREDITS :"
				, (u32_t)((data & XMAC_TX_FIFO_CREDITS_REG_QUAD_PORT_TX_CREDITS) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0xd8UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xd8UL): // (XMAC_REG_EEE_CTRL: ACCESS:RW  DataWidth:0x4  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_EEE_CTRL (ACCESS:RW  DataWidth:0x4  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tEEE_DISABLE_RX_PAUSE_ACTIVE :"
				, (u32_t)((data & XMAC_EEE_CTRL_REG_EEE_DISABLE_RX_PAUSE_ACTIVE) >> 3)
				, "\n\tEEE_DISABLE_TX_PFC_XOFF :"
				, (u32_t)((data & XMAC_EEE_CTRL_REG_EEE_DISABLE_TX_PFC_XOFF) >> 2)
				, "\n\tEEE_DISABLE_TX_PAUSE_XOFF :"
				, (u32_t)((data & XMAC_EEE_CTRL_REG_EEE_DISABLE_TX_PAUSE_XOFF) >> 1)
				, "\n\tEEE_EN :"
				, (u32_t)((data & XMAC_EEE_CTRL_REG_EEE_EN) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0xe0UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xe0UL): // (XMAC_REG_EEE_DELAY_ENTRY_TIMER: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXMAC_REG_EEE_DELAY_ENTRY_TIMER (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This is the duration for which condition to move to LPI state\n     must be satisfied; at the end of which MAC transitions to LPI State. \n    This is in terms of micro seconds \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xe4UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xe4UL): // (XMAC_REG_EEE_TIMERS_HI: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_EEE_TIMERS_HI (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tEEE_WAKE_TIMER :"
				, (u32_t)((data & XMAC_EEE_TIMERS_HI_REG_EEE_WAKE_TIMER) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x110UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x110UL): // (XMAC_REG_GMII_EEE_CTRL: ACCESS:RW  DataWidth:0x12  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_GMII_EEE_CTRL (ACCESS:RW  DataWidth:0x12  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tGMII_TXCLK_DIS :"
				, (u32_t)((data & XMAC_GMII_EEE_CTRL_REG_GMII_TXCLK_DIS) >> 17)
				, "\n\tGMII_LPI_PREDICT_MODE_EN :"
				, (u32_t)((data & XMAC_GMII_EEE_CTRL_REG_GMII_LPI_PREDICT_MODE_EN) >> 16)
				, "\n\tGMII_LPI_PREDICT_THRESHOLD :"
				, (u32_t)((data & XMAC_GMII_EEE_CTRL_REG_GMII_LPI_PREDICT_THRESHOLD) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x120UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x120UL): // (XMAC_REG_MACSEC_CTRL: ACCESS:RW  DataWidth:0x20  Multi Field Register  ) 
			sprintf(disp_str, "\nXMAC_REG_MACSEC_CTRL (ACCESS:RW  DataWidth:0x20  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMACSEC_PROG_TX_CRC_LO :"
				, (u32_t)((data & XMAC_MACSEC_CTRL_REG_MACSEC_PROG_TX_CRC_LO) >> 3)
				, "\n\tMACSEC_TX_CRC_CORRUPTION_MODE :"
				, (u32_t)((data & XMAC_MACSEC_CTRL_REG_MACSEC_TX_CRC_CORRUPTION_MODE) >> 2)
				, "\n\tMACSEC_TX_CRC_CORRUPT_EN :"
				, (u32_t)((data & XMAC_MACSEC_CTRL_REG_MACSEC_TX_CRC_CORRUPT_EN) >> 1)
				, "\n\tMACSEC_TX_LAUNCH_EN :"
				, (u32_t)((data & XMAC_MACSEC_CTRL_REG_MACSEC_TX_LAUNCH_EN) >> 0));
			break;
		case (GRCBASE_XMAC0 + 0x124UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x124UL): // (XMAC_REG_MACSEC_PROG_TX_CRC_HI: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXMAC_REG_MACSEC_PROG_TX_CRC_HI (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Upper 32 bits of macsec_prog_tx_crc register. Programmable\n     CRC value to corrupt the Tx CRC to be used in MACSEC. The computed CRC\n     is replaced by this programmed CRC value \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x420UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x420UL): // (XMAC_REG_WB_TX_CTRL: ACCESS:WB  DataWidth:0x26 ) 
			sprintf(disp_str, "\nXMAC_REG_WB_TX_CTRL (ACCESS:WB  DataWidth:0x26 )" \
				"\nDescription: This is a WB access for version of the register at XMAC_TX_CTRL\n    . The register can be access in either this loation or at XMAC_TX_CTRL\n    ; but normal WB access methods mus be used at this location. The fields\n     within this WB register are:<br/>1:0=XMAC_CRC_MODE; 2:2=XMAC_DISCARD; \n    3:3=XMAC_TX_ANY_START; 4:4=XMAC_PAD_EN; 11:5=XMAC_PAD_THRESHOLD; 18:12\n    =XMAC_AVERAGE_IPG; 24:19=XMAC_THROT_NUM; 31:25=XMAC_THROT_DENOM_LO; 32\n    :32=XMAC_THROT_DENOM_HI; 36:33=XMAC_TX_PREAMBLE_LENGTH; 37:37=XMAC_TX_64BYTE_BUFFER_EN\n    ; 63:38=XMAC_RESERVED.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x428UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x428UL): // (XMAC_REG_WB_TX_MAC_SA: ACCESS:WB  DataWidth:0x30 ) 
			sprintf(disp_str, "\nXMAC_REG_WB_TX_MAC_SA (ACCESS:WB  DataWidth:0x30 )" \
				"\nDescription: This is a WB access for version of the register at XMAC_TX_MAC_SA\n    . The register can be access in either this loation or at XMAC_TX_MAC_SA\n    ; but normal WB access methods mus be used at this location. The fields\n     within this WB register are:<br/>31:0=XMAC_CTRL_SA_LO; 47:32=XMAC_CTRL_SA_HI\n    ; 63:48=XMAC_RESERVED.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x438UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x438UL): // (XMAC_REG_WB_RX_MAC_SA: ACCESS:WB  DataWidth:0x30 ) 
			sprintf(disp_str, "\nXMAC_REG_WB_RX_MAC_SA (ACCESS:WB  DataWidth:0x30 )" \
				"\nDescription: This is a WB access for version of the register at XMAC_RX_MAC_SA\n    . The register can be access in either this loation or at XMAC_RX_MAC_SA\n    ; but normal WB access methods mus be used at this location. The fields\n     within this WB register are:<br/>31:0=XMAC_RX_SA_LO; 47:32=XMAC_RX_SA_HI\n    ; 63:48=XMAC_RESERVED.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x488UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x488UL): // (XMAC_REG_WB_PFC_DA: ACCESS:WB  DataWidth:0x30 ) 
			sprintf(disp_str, "\nXMAC_REG_WB_PFC_DA (ACCESS:WB  DataWidth:0x30 )" \
				"\nDescription: This is a WB access for version of the register at XMAC_PFC_DA\n    . The register can be access in either this loation or at XMAC_PFC_DA; \n    but normal WB access methods mus be used at this location. The fields \n    within this WB register are:<br/>31:0=XMAC_PFC_MACDA_LO; 47:32=XMAC_PFC_MACDA_HI\n    ; 63:48=XMAC_RESERVED.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x520UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x520UL): // (XMAC_REG_WB_MACSEC_CTRL: ACCESS:WB  DataWidth:0x23 ) 
			sprintf(disp_str, "\nXMAC_REG_WB_MACSEC_CTRL (ACCESS:WB  DataWidth:0x23 )" \
				"\nDescription: This is a WB access for version of the register at XMAC_MACSEC_CTRL\n    . The register can be access in either this loation or at XMAC_MACSEC_CTRL\n    ; but normal WB access methods mus be used at this location. The fields\n     within this WB register are:<br/>0:0=XMAC_MACSEC_TX_LAUNCH_EN; 1:1=XMAC_MACSEC_TX_CRC_CORRUPT_EN\n    ; 2:2=XMAC_MACSEC_TX_CRC_CORRUPTION_MODE; 31:3=XMAC_MACSEC_PROG_TX_CRC_LO\n    ; 34:32=XMAC_MACSEC_PROG_TX_CRC_HI; 63:35=XMAC_RESERVED.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x4UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x4UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xcUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xcUL): // (XMAC_REG_XMAC_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x14UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x14UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x1cUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x1cUL): // (XMAC_REG_XMAC_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x34UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x34UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x44UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x44UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x54UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x54UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_6: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_6 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x5cUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x5cUL): // (XMAC_REG_XMAC_UNUSED_EMPTY_7: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_7 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x64UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x64UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_8: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_8 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x6cUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x6cUL): // (XMAC_REG_XMAC_UNUSED_EMPTY_9: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_9 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x7cUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x7cUL): // (XMAC_REG_XMAC_UNUSED_EMPTY_10: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_10 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x84UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x84UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_11: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_11 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x94UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x94UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_12: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_12 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x9cUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x9cUL): // (XMAC_REG_XMAC_UNUSED_EMPTY_13: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_13 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xa4UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xa4UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_14: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_14 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xacUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xacUL): // (XMAC_REG_XMAC_UNUSED_EMPTY_15: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_15 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xb8UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xb8UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_16: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_16 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xc4UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xc4UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_17: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_17 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xccUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xccUL): // (XMAC_REG_XMAC_UNUSED_EMPTY_18: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_18 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xd4UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xd4UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_19: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_19 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xdcUL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xdcUL): // (XMAC_REG_XMAC_UNUSED_EMPTY_20: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_20 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0xe8UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0xe8UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_21: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_21 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x114UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x114UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_22: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_22 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x128UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x128UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_23: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_23 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x430UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x430UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_24: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_24 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x440UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x440UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_25: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_25 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x490UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x490UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_26: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_26 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case (GRCBASE_XMAC0 + 0x528UL): // (2-port) 
		case (GRCBASE_XMAC1 + 0x528UL): // (XMAC_REG_XMAC_UNUSED_EMPTY_27: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXMAC_REG_XMAC_UNUSED_EMPTY_27 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166000UL: // (XSDM_REG_TIMER_TICK: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_TIMER_TICK (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Tick for timer counter. Applicable only when ~xsdm_registers_timer_tick_enable\n    .timer_tick_enable =1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166004UL: // (XSDM_REG_TIMERS_TICK_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_TIMERS_TICK_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for tick counter.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166008UL: // (XSDM_REG_CFC_RSP_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSDM_REG_CFC_RSP_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the  internal RAM for the cfc_rsp lcid\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16600cUL: // (XSDM_REG_CMP_COUNTER_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSDM_REG_CMP_COUNTER_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal  RAM for the completion\n     counters.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166010UL: // (XSDM_REG_Q_COUNTER_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSDM_REG_Q_COUNTER_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal RAM for queue counters\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166014UL: // (XSDM_REG_PCK_END_MSG_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSDM_REG_PCK_END_MSG_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal RAM for the packet end \n    message\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166018UL: // (XSDM_REG_COUNTERS_WRAP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_COUNTERS_WRAP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Indicates if the 204 statistics counters should stop counting\n     when reaching an all-ones value or should wrap-around 0=stop counting\n     1=wrap-around.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16601cUL: // (XSDM_REG_CMP_COUNTER_MAX0: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXSDM_REG_CMP_COUNTER_MAX0 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166020UL: // (XSDM_REG_CMP_COUNTER_MAX1: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXSDM_REG_CMP_COUNTER_MAX1 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166024UL: // (XSDM_REG_CMP_COUNTER_MAX2: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXSDM_REG_CMP_COUNTER_MAX2 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166028UL: // (XSDM_REG_CMP_COUNTER_MAX3: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXSDM_REG_CMP_COUNTER_MAX3 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16602cUL: // (XSDM_REG_BRB1_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXSDM_REG_BRB1_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from BRB1 in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166030UL: // (XSDM_REG_PXP_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXSDM_REG_PXP_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from pxp in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166034UL: // (XSDM_REG_PB_ALMOST_FULL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXSDM_REG_PB_ALMOST_FULL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: Almost full signal for read data from PB in DMA_RSP block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166038UL: // (XSDM_REG_AGG_INT_EVENT_0: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_0 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16603cUL: // (XSDM_REG_AGG_INT_EVENT_1: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_1 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166040UL: // (XSDM_REG_AGG_INT_EVENT_2: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_2 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166044UL: // (XSDM_REG_AGG_INT_EVENT_3: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_3 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166048UL: // (XSDM_REG_AGG_INT_EVENT_4: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_4 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16604cUL: // (XSDM_REG_AGG_INT_EVENT_5: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_5 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166050UL: // (XSDM_REG_AGG_INT_EVENT_6: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_6 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166054UL: // (XSDM_REG_AGG_INT_EVENT_7: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_7 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166058UL: // (XSDM_REG_AGG_INT_EVENT_8: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_8 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16605cUL: // (XSDM_REG_AGG_INT_EVENT_9: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_9 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166060UL: // (XSDM_REG_AGG_INT_EVENT_10: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_10 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166064UL: // (XSDM_REG_AGG_INT_EVENT_11: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_11 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166068UL: // (XSDM_REG_AGG_INT_EVENT_12: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_12 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16606cUL: // (XSDM_REG_AGG_INT_EVENT_13: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_13 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166070UL: // (XSDM_REG_AGG_INT_EVENT_14: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_14 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166074UL: // (XSDM_REG_AGG_INT_EVENT_15: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_15 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166078UL: // (XSDM_REG_AGG_INT_EVENT_16: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_16 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16607cUL: // (XSDM_REG_AGG_INT_EVENT_17: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_17 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166080UL: // (XSDM_REG_AGG_INT_EVENT_18: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_18 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166084UL: // (XSDM_REG_AGG_INT_EVENT_19: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_19 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166088UL: // (XSDM_REG_AGG_INT_EVENT_20: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_20 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16608cUL: // (XSDM_REG_AGG_INT_EVENT_21: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_21 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166090UL: // (XSDM_REG_AGG_INT_EVENT_22: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_22 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166094UL: // (XSDM_REG_AGG_INT_EVENT_23: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_23 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166098UL: // (XSDM_REG_AGG_INT_EVENT_24: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_24 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16609cUL: // (XSDM_REG_AGG_INT_EVENT_25: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_25 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660a0UL: // (XSDM_REG_AGG_INT_EVENT_26: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_26 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660a4UL: // (XSDM_REG_AGG_INT_EVENT_27: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_27 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660a8UL: // (XSDM_REG_AGG_INT_EVENT_28: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_28 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660acUL: // (XSDM_REG_AGG_INT_EVENT_29: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_29 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660b0UL: // (XSDM_REG_AGG_INT_EVENT_30: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_30 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660b4UL: // (XSDM_REG_AGG_INT_EVENT_31: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_EVENT_31 (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: The event id for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660b8UL: // (XSDM_REG_AGG_INT_T_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660bcUL: // (XSDM_REG_AGG_INT_T_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660c0UL: // (XSDM_REG_AGG_INT_T_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660c4UL: // (XSDM_REG_AGG_INT_T_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660c8UL: // (XSDM_REG_AGG_INT_T_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660ccUL: // (XSDM_REG_AGG_INT_T_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660d0UL: // (XSDM_REG_AGG_INT_T_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660d4UL: // (XSDM_REG_AGG_INT_T_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660d8UL: // (XSDM_REG_AGG_INT_T_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660dcUL: // (XSDM_REG_AGG_INT_T_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660e0UL: // (XSDM_REG_AGG_INT_T_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660e4UL: // (XSDM_REG_AGG_INT_T_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660e8UL: // (XSDM_REG_AGG_INT_T_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660ecUL: // (XSDM_REG_AGG_INT_T_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660f0UL: // (XSDM_REG_AGG_INT_T_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660f4UL: // (XSDM_REG_AGG_INT_T_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660f8UL: // (XSDM_REG_AGG_INT_T_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1660fcUL: // (XSDM_REG_AGG_INT_T_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166100UL: // (XSDM_REG_AGG_INT_T_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166104UL: // (XSDM_REG_AGG_INT_T_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166108UL: // (XSDM_REG_AGG_INT_T_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16610cUL: // (XSDM_REG_AGG_INT_T_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166110UL: // (XSDM_REG_AGG_INT_T_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166114UL: // (XSDM_REG_AGG_INT_T_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166118UL: // (XSDM_REG_AGG_INT_T_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16611cUL: // (XSDM_REG_AGG_INT_T_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166120UL: // (XSDM_REG_AGG_INT_T_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166124UL: // (XSDM_REG_AGG_INT_T_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166128UL: // (XSDM_REG_AGG_INT_T_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16612cUL: // (XSDM_REG_AGG_INT_T_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166130UL: // (XSDM_REG_AGG_INT_T_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166134UL: // (XSDM_REG_AGG_INT_T_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_T_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The T bit for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166138UL: // (XSDM_REG_AGG_INT_FIC_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16613cUL: // (XSDM_REG_AGG_INT_FIC_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166140UL: // (XSDM_REG_AGG_INT_FIC_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166144UL: // (XSDM_REG_AGG_INT_FIC_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166148UL: // (XSDM_REG_AGG_INT_FIC_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16614cUL: // (XSDM_REG_AGG_INT_FIC_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166150UL: // (XSDM_REG_AGG_INT_FIC_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166154UL: // (XSDM_REG_AGG_INT_FIC_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166158UL: // (XSDM_REG_AGG_INT_FIC_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16615cUL: // (XSDM_REG_AGG_INT_FIC_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166160UL: // (XSDM_REG_AGG_INT_FIC_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166164UL: // (XSDM_REG_AGG_INT_FIC_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166168UL: // (XSDM_REG_AGG_INT_FIC_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16616cUL: // (XSDM_REG_AGG_INT_FIC_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166170UL: // (XSDM_REG_AGG_INT_FIC_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166174UL: // (XSDM_REG_AGG_INT_FIC_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166178UL: // (XSDM_REG_AGG_INT_FIC_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16617cUL: // (XSDM_REG_AGG_INT_FIC_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166180UL: // (XSDM_REG_AGG_INT_FIC_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166184UL: // (XSDM_REG_AGG_INT_FIC_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166188UL: // (XSDM_REG_AGG_INT_FIC_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 20\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16618cUL: // (XSDM_REG_AGG_INT_FIC_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 21\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166190UL: // (XSDM_REG_AGG_INT_FIC_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 22\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166194UL: // (XSDM_REG_AGG_INT_FIC_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 23\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166198UL: // (XSDM_REG_AGG_INT_FIC_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 24\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16619cUL: // (XSDM_REG_AGG_INT_FIC_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 25\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661a0UL: // (XSDM_REG_AGG_INT_FIC_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 26\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661a4UL: // (XSDM_REG_AGG_INT_FIC_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 27\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661a8UL: // (XSDM_REG_AGG_INT_FIC_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 28\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661acUL: // (XSDM_REG_AGG_INT_FIC_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 29\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661b0UL: // (XSDM_REG_AGG_INT_FIC_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 30\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661b4UL: // (XSDM_REG_AGG_INT_FIC_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_FIC_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: The destination FIC for aggregated interrupt 31\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661b8UL: // (XSDM_REG_AGG_INT_MODE_0: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_0 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661bcUL: // (XSDM_REG_AGG_INT_MODE_1: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_1 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661c0UL: // (XSDM_REG_AGG_INT_MODE_2: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_2 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661c4UL: // (XSDM_REG_AGG_INT_MODE_3: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_3 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661c8UL: // (XSDM_REG_AGG_INT_MODE_4: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_4 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661ccUL: // (XSDM_REG_AGG_INT_MODE_5: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_5 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661d0UL: // (XSDM_REG_AGG_INT_MODE_6: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_6 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661d4UL: // (XSDM_REG_AGG_INT_MODE_7: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_7 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661d8UL: // (XSDM_REG_AGG_INT_MODE_8: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_8 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661dcUL: // (XSDM_REG_AGG_INT_MODE_9: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_9 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661e0UL: // (XSDM_REG_AGG_INT_MODE_10: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_10 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661e4UL: // (XSDM_REG_AGG_INT_MODE_11: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_11 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661e8UL: // (XSDM_REG_AGG_INT_MODE_12: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_12 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661ecUL: // (XSDM_REG_AGG_INT_MODE_13: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_13 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661f0UL: // (XSDM_REG_AGG_INT_MODE_14: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_14 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661f4UL: // (XSDM_REG_AGG_INT_MODE_15: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_15 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661f8UL: // (XSDM_REG_AGG_INT_MODE_16: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_16 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (16) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1661fcUL: // (XSDM_REG_AGG_INT_MODE_17: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_17 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (17)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166200UL: // (XSDM_REG_AGG_INT_MODE_18: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_18 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166204UL: // (XSDM_REG_AGG_INT_MODE_19: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_19 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166208UL: // (XSDM_REG_AGG_INT_MODE_20: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_20 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16620cUL: // (XSDM_REG_AGG_INT_MODE_21: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_21 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166210UL: // (XSDM_REG_AGG_INT_MODE_22: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_22 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166214UL: // (XSDM_REG_AGG_INT_MODE_23: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_23 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166218UL: // (XSDM_REG_AGG_INT_MODE_24: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_24 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16621cUL: // (XSDM_REG_AGG_INT_MODE_25: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_25 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166220UL: // (XSDM_REG_AGG_INT_MODE_26: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_26 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166224UL: // (XSDM_REG_AGG_INT_MODE_27: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_27 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166228UL: // (XSDM_REG_AGG_INT_MODE_28: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_28 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16622cUL: // (XSDM_REG_AGG_INT_MODE_29: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_29 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166230UL: // (XSDM_REG_AGG_INT_MODE_30: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_30 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166234UL: // (XSDM_REG_AGG_INT_MODE_31: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_AGG_INT_MODE_31 (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: For each aggregated interrupt index whether the mode is normal\n     (0) or auto-mask-mode (1)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166238UL: // (XSDM_REG_ENABLE_IN1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nXSDM_REG_ENABLE_IN1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tQM_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_QM_EXT_WR_FULL_IN_EN) >> 26)
				, "\n\tCFC_PB_ACK_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_CFC_PB_ACK_IN_EN) >> 25)
				, "\n\tCFC_ACDEC_ACK_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_CFC_ACDEC_ACK_IN_EN) >> 24)
				, "\n\tCFC_ACINC_ACK_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_CFC_ACINC_ACK_IN_EN) >> 23)
				, "\n\tCFC_LOAD_RSP_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_CFC_LOAD_RSP_IN_EN) >> 22)
				, "\n\tCFC_LOAD_ACK_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_CFC_LOAD_ACK_IN_EN) >> 21)
				, "\n\tPXP_REQ_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PXP_REQ_IN_EN) >> 20)
				, "\n\tSDM_WAKE_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_SDM_WAKE_IN_EN) >> 19)
				, "\n\tPRS_MSG_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PRS_MSG_IN_EN) >> 18)
				, "\n\tPB_DATA_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PB_DATA_IN_EN) >> 17)
				, "\n\tBRB1_DP_DATA_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_BRB1_DP_DATA_IN_EN) >> 16)
				, "\n\tBRB1_CTRL_DATA_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_BRB1_CTRL_DATA_IN_EN) >> 15)
				, "\n\tPXP_DP_ACK_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PXP_DP_ACK_IN_EN) >> 14)
				, "\n\tPXP_CTRL_ACK_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PXP_CTRL_ACK_IN_EN) >> 13)
				, "\n\tPXP_DP_DATA_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PXP_DP_DATA_IN_EN) >> 12)
				, "\n\tPXP_INT_DONE_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PXP_INT_DONE_IN_EN) >> 11)
				, "\n\tPXP_CTRL_DATA_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PXP_CTRL_DATA_IN_EN) >> 10)
				, "\n\tPXP_DP_FULL_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PXP_DP_FULL_IN_EN) >> 9)
				, "\n\tPXP_CTRL_FULL_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PXP_CTRL_FULL_IN_EN) >> 8)
				, "\n\tPXP_DP_DONE_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PXP_DP_DONE_IN_EN) >> 7)
				, "\n\tPXP_CTRL_DONE_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PXP_CTRL_DONE_IN_EN) >> 6)
				, "\n\tPAS_BUF_FULL_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PAS_BUF_FULL_IN_EN) >> 5)
				, "\n\tPAS_BUF_DONE_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_PAS_BUF_DONE_IN_EN) >> 4)
				, "\n\tINT_RAM_FULL_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_INT_RAM_FULL_IN_EN) >> 3)
				, "\n\tINT_RAM_DONE_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_INT_RAM_DONE_IN_EN) >> 2)
				, "\n\tINT_RAM_DATA_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_INT_RAM_DATA_IN_EN) >> 1)
				, "\n\tEXT_STORE_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN1_REG_EXT_STORE_IN_EN) >> 0));
			break;
		case 0x16623cUL: // (XSDM_REG_ENABLE_IN2: ACCESS:RW  DataWidth:0x7  Multi Field Register  ) 
			sprintf(disp_str, "\nXSDM_REG_ENABLE_IN2 (ACCESS:RW  DataWidth:0x7  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_REQ_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN2_REG_DORQ_REQ_IN_EN) >> 6)
				, "\n\tPB_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN2_REG_PB_EXT_WR_FULL_IN_EN) >> 5)
				, "\n\tPBF_EXT_WR_FULL_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN2_REG_PBF_EXT_WR_FULL_IN_EN) >> 4)
				, "\n\tPB_FULL_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN2_REG_PB_FULL_IN_EN) >> 3)
				, "\n\tPB_STATUS_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN2_REG_PB_STATUS_IN_EN) >> 2)
				, "\n\tCM_ACK_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN2_REG_CM_ACK_IN_EN) >> 1)
				, "\n\tSDM_ACK_IN_EN :"
				, (u32_t)((data & XSDM_ENABLE_IN2_REG_SDM_ACK_IN_EN) >> 0));
			break;
		case 0x166240UL: // (XSDM_REG_ENABLE_OUT1: ACCESS:RW  DataWidth:0x1b  Multi Field Register  ) 
			sprintf(disp_str, "\nXSDM_REG_ENABLE_OUT1 (ACCESS:RW  DataWidth:0x1b  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPBF_EXT_WR_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PBF_EXT_WR_OUT_EN) >> 26)
				, "\n\tPB_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PB_OUT_EN) >> 25)
				, "\n\tCFC_SDM_ACK_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_CFC_SDM_ACK_OUT_EN) >> 24)
				, "\n\tCM_MSG_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_CM_MSG_OUT_EN) >> 23)
				, "\n\tPXP_REQ_DONE_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PXP_REQ_DONE_OUT_EN) >> 22)
				, "\n\tEXT_FULL_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_EXT_FULL_OUT_EN) >> 21)
				, "\n\tPXP_CTRL_FULL_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PXP_CTRL_FULL_OUT_EN) >> 20)
				, "\n\tPB_FULL_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PB_FULL_OUT_EN) >> 19)
				, "\n\tBRB1_DP_FULL_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_BRB1_DP_FULL_OUT_EN) >> 18)
				, "\n\tBRB1_CTRL_FULL_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_BRB1_CTRL_FULL_OUT_EN) >> 17)
				, "\n\tPXP_DP_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PXP_DP_OUT_EN) >> 16)
				, "\n\tPXP_CTRL_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PXP_CTRL_OUT_EN) >> 15)
				, "\n\tPXP_ASYNC_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PXP_ASYNC_OUT_EN) >> 14)
				, "\n\tPAS_BUF_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PAS_BUF_OUT_EN) >> 13)
				, "\n\tINT_RAM_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_INT_RAM_OUT_EN) >> 12)
				, "\n\tPRS_ACK_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PRS_ACK_OUT_EN) >> 11)
				, "\n\tPRS_SYNC_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PRS_SYNC_OUT_EN) >> 10)
				, "\n\tBRB1_DP_REQ_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_BRB1_DP_REQ_OUT_EN) >> 9)
				, "\n\tBRB1_CTRL_REQ_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_BRB1_CTRL_REQ_OUT_EN) >> 8)
				, "\n\tPXP_DP_REQ_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PXP_DP_REQ_OUT_EN) >> 7)
				, "\n\tPXP_CTRL_REQ_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PXP_CTRL_REQ_OUT_EN) >> 6)
				, "\n\tCFC_PB_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_CFC_PB_OUT_EN) >> 5)
				, "\n\tCFC_ACDEC_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_CFC_ACDEC_OUT_EN) >> 4)
				, "\n\tCFC_ACINC_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_CFC_ACINC_OUT_EN) >> 3)
				, "\n\tCFC_LOAD_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_CFC_LOAD_OUT_EN) >> 2)
				, "\n\tTHREADREADY_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_THREADREADY_OUT_EN) >> 1)
				, "\n\tPXP_INT_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT1_REG_PXP_INT_OUT_EN) >> 0));
			break;
		case 0x166244UL: // (XSDM_REG_ENABLE_OUT2: ACCESS:RW  DataWidth:0x6  Multi Field Register  ) 
			sprintf(disp_str, "\nXSDM_REG_ENABLE_OUT2 (ACCESS:RW  DataWidth:0x6  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDORQ_REQ_DONE_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT2_REG_DORQ_REQ_DONE_OUT_EN) >> 5)
				, "\n\tVFPF_ERR_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT2_REG_VFPF_ERR_OUT_EN) >> 4)
				, "\n\tSDM_EXT_WR_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT2_REG_SDM_EXT_WR_OUT_EN) >> 3)
				, "\n\tQM_EXT_WR_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT2_REG_QM_EXT_WR_OUT_EN) >> 2)
				, "\n\tDQ_EXT_WR_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT2_REG_DQ_EXT_WR_OUT_EN) >> 1)
				, "\n\tPB_EXT_WR_OUT_EN :"
				, (u32_t)((data & XSDM_ENABLE_OUT2_REG_PB_EXT_WR_OUT_EN) >> 0));
			break;
		case 0x166248UL: // (XSDM_REG_NUM_OF_Q0_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q0_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16624cUL: // (XSDM_REG_NUM_OF_Q1_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q1_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166250UL: // (XSDM_REG_NUM_OF_Q3_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q3_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166254UL: // (XSDM_REG_NUM_OF_Q4_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q4_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166258UL: // (XSDM_REG_NUM_OF_Q5_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q5_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16625cUL: // (XSDM_REG_NUM_OF_Q6_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q6_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166260UL: // (XSDM_REG_NUM_OF_Q7_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q7_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166264UL: // (XSDM_REG_NUM_OF_Q8_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q8_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166268UL: // (XSDM_REG_NUM_OF_Q9_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q9_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16626cUL: // (XSDM_REG_NUM_OF_Q10_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q10_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166270UL: // (XSDM_REG_NUM_OF_Q11_CMD: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_Q11_CMD (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of commands received in queue 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166274UL: // (XSDM_REG_NUM_OF_PKT_END_MSG: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_PKT_END_MSG (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of packet end messages received from the parser\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166278UL: // (XSDM_REG_NUM_OF_PXP_ASYNC_REQ: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_PXP_ASYNC_REQ (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of requests received from the pxp async if\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16627cUL: // (XSDM_REG_NUM_OF_ACK_AFTER_PLACE: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_ACK_AFTER_PLACE (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of ACK after placement messages received \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166280UL: // (XSDM_REG_STATISTICS_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSDM_REG_STATISTICS_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits for statistics sram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166284UL: // (XSDM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from XSDM\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166288UL: // (XSDM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from XSDM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16628cUL: // (XSDM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSDM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from XSDM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166290UL: // (XSDM_REG_XSDM_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x166294UL: // (XSDM_REG_XSDM_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x166298UL: // (XSDM_REG_XSDM_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x16629cUL: // (XSDM_REG_XSDM_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tRSP_INT_RAM_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_RSP_INT_RAM_PEND_ERROR) >> 31)
				, "\n\tRSP_BRB1_DP_DST_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_RSP_BRB1_DP_DST_ERROR) >> 30)
				, "\n\tRSP_BRB1_DP_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_RSP_BRB1_DP_PEND_ERROR) >> 29)
				, "\n\tRSP_BRB1_CTRL_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_RSP_BRB1_CTRL_PEND_ERROR) >> 28)
				, "\n\tDST_NONE_DP_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_DST_NONE_DP_DST_PEND_ERROR) >> 27)
				, "\n\tDST_BRB1_SRC_ADDR_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_DST_BRB1_SRC_ADDR_ERROR) >> 26)
				, "\n\tDST_BRB1_SRC_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_DST_BRB1_SRC_PEND_ERROR) >> 25)
				, "\n\tDST_PXP_DP_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_DST_PXP_DP_DST_PEND_ERROR) >> 24)
				, "\n\tDST_PXP_CTRL_DST_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_DST_PXP_CTRL_DST_PEND_ERROR) >> 23)
				, "\n\tDST_PB_IMMED_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_DST_PB_IMMED_ERROR) >> 22)
				, "\n\tDST_PXP_CTRL_IMMED_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_DST_PXP_CTRL_IMMED_ERROR) >> 21)
				, "\n\tDST_PAS_BUF_WAIT_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_DST_PAS_BUF_WAIT_ERROR) >> 20)
				, "\n\tDST_INT_RAM_WAIT_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_DST_INT_RAM_WAIT_ERROR) >> 19)
				, "\n\tCFC_LOAD_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_CFC_LOAD_PEND_ERROR) >> 18)
				, "\n\tCFC_LOAD_RSP_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_CFC_LOAD_RSP_ERROR) >> 17)
				, "\n\tSYNC_SER_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_SYNC_SER_FIFO_ERROR) >> 16)
				, "\n\tSYNC_PRS_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_SYNC_PRS_FIFO_ERROR) >> 15)
				, "\n\tASYNC_HOST_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_ASYNC_HOST_ERROR) >> 14)
				, "\n\tDELAY_FIFO_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_DELAY_FIFO_ERROR) >> 13)
				, "\n\tINP_QUEUE11_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE11_ERROR) >> 12)
				, "\n\tINP_QUEUE10_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE10_ERROR) >> 11)
				, "\n\tINP_QUEUE9_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE9_ERROR) >> 10)
				, "\n\tINP_QUEUE8_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE8_ERROR) >> 9)
				, "\n\tINP_QUEUE7_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE7_ERROR) >> 8)
				, "\n\tINP_QUEUE6_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE6_ERROR) >> 7)
				, "\n\tINP_QUEUE5_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE5_ERROR) >> 6)
				, "\n\tINP_QUEUE4_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE4_ERROR) >> 5)
				, "\n\tINP_QUEUE3_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE3_ERROR) >> 4)
				, "\n\tINP_QUEUE2_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE2_ERROR) >> 3)
				, "\n\tINP_QUEUE1_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE1_ERROR) >> 2)
				, "\n\tINP_QUEUE0_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_INP_QUEUE0_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x1662a0UL: // (XSDM_REG_XSDM_INT_STS_1: ACCESS:R  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_INT_STS_1 (ACCESS:R  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0x1662a4UL: // (XSDM_REG_XSDM_INT_STS_CLR_1: ACCESS:RC  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_CLR_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0x1662a8UL: // (XSDM_REG_XSDM_INT_STS_WR_1: ACCESS:WR  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_INT_STS_WR_1 (ACCESS:WR  DataWidth:0xe )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_STS_WR_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0x1662acUL: // (XSDM_REG_XSDM_INT_MASK_1: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_INT_MASK_1 (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tDST_PXP_CTRL_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_DST_PXP_CTRL_DONE_ERROR) >> 13)
				, "\n\tDST_PXP_DP_DST_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_DST_PXP_DP_DST_DONE_ERROR) >> 12)
				, "\n\tDST_PXP_DP_SRC_DONE_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_DST_PXP_DP_SRC_DONE_ERROR) >> 11)
				, "\n\tDORQ_DPM_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_DORQ_DPM_ERROR) >> 10)
				, "\n\tTIMER_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_TIMER_PEND_ERROR) >> 9)
				, "\n\tTIMER_ADDR_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_TIMER_ADDR_ERROR) >> 8)
				, "\n\tPXP_DELAY_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_PXP_DELAY_ERROR) >> 7)
				, "\n\tCM_DELAY_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_CM_DELAY_ERROR) >> 6)
				, "\n\tRSP_PXP_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_RSP_PXP_CTRL_RD_DATA_ERROR) >> 5)
				, "\n\tRSP_INT_RAM_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_RSP_INT_RAM_RD_DATA_ERROR) >> 4)
				, "\n\tRSP_PB_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_RSP_PB_RD_DATA_ERROR) >> 3)
				, "\n\tRSP_BRB1_DP_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_RSP_BRB1_DP_RD_DATA_ERROR) >> 2)
				, "\n\tRSP_BRB1_CTRL_RD_DATA_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_RSP_BRB1_CTRL_RD_DATA_ERROR) >> 1)
				, "\n\tRSP_PB_PEND_ERROR :"
				, (u32_t)((data & XSDM_XSDM_INT_MASK_1_REG_RSP_PB_PEND_ERROR) >> 0));
			break;
		case 0x1662b0UL: // (XSDM_REG_XSDM_PRTY_STS: ACCESS:R  DataWidth:0xb ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_PRTY_STS (ACCESS:R  DataWidth:0xb )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_REG_PARITY) >> 0));
			break;
		case 0x1662b4UL: // (XSDM_REG_XSDM_PRTY_STS_CLR: ACCESS:RC  DataWidth:0xb ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_PRTY_STS_CLR (ACCESS:RC  DataWidth:0xb )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_CLR_REG_PARITY) >> 0));
			break;
		case 0x1662b8UL: // (XSDM_REG_XSDM_PRTY_STS_WR: ACCESS:WR  DataWidth:0xb ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_PRTY_STS_WR (ACCESS:WR  DataWidth:0xb )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XSDM_XSDM_PRTY_STS_WR_REG_PARITY) >> 0));
			break;
		case 0x1662bcUL: // (XSDM_REG_XSDM_PRTY_MASK: ACCESS:RW  DataWidth:0xb ) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_PRTY_MASK (ACCESS:RW  DataWidth:0xb )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tCM_QUEUE_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_CM_QUEUE_RD_DATA) >> 10)
				, "\n\tSTAT_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_STAT_RD_DATA) >> 9)
				, "\n\tINT_RAM_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_INT_RAM_RD_DATA) >> 8)
				, "\n\tPXP_CTRL_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_PXP_CTRL_RD_DATA) >> 7)
				, "\n\tPB_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_PB_RD_DATA) >> 6)
				, "\n\tBRB1_DP_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_BRB1_DP_RD_DATA) >> 5)
				, "\n\tBRB1_CTRL_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_BRB1_CTRL_RD_DATA) >> 4)
				, "\n\tASYNC_RD_DATA :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_ASYNC_RD_DATA) >> 3)
				, "\n\tINP_QUEUE :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_INP_QUEUE) >> 2)
				, "\n\tTIMERS :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_TIMERS) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XSDM_XSDM_PRTY_MASK_REG_PARITY) >> 0));
			break;
		case 0x1662ccUL: // (XSDM_REG_DORQ_DPM_START_ADDR: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSDM_REG_DORQ_DPM_START_ADDR (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: The start address in the internal RAM for DORQ DPM messages\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1662d0UL: // (XSDM_REG_DORQ_ALMOST_FULL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSDM_REG_DORQ_ALMOST_FULL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Almost full signal for read data from DORq in SDM_DORQ block\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1662d4UL: // (XSDM_REG_NUM_OF_DPM_REQ: ACCESS:ST  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_NUM_OF_DPM_REQ (ACCESS:ST  DataWidth:0x20 )" \
				"\nDescription: The number of DORQ DPM messages received \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1662d8UL: // (XSDM_REG_CMP_COUNTER_MAX4: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXSDM_REG_CMP_COUNTER_MAX4 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1662dcUL: // (XSDM_REG_CMP_COUNTER_MAX5: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXSDM_REG_CMP_COUNTER_MAX5 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1662e0UL: // (XSDM_REG_CMP_COUNTER_MAX6: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXSDM_REG_CMP_COUNTER_MAX6 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1662e4UL: // (XSDM_REG_CMP_COUNTER_MAX7: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXSDM_REG_CMP_COUNTER_MAX7 (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: The maximum value of the competion counter #7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166570UL: // (XSDM_REG_CM_QUEUE_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_CM_QUEUE_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits  CM_QUEUE\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166574UL: // (XSDM_REG_INP_QUEUE_TM: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_INP_QUEUE_TM (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: TM bits  INP_QUEUE\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166578UL: // (XSDM_REG_FIFOS_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXSDM_REG_FIFOS_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits  fifos: BRB1_CTRL[1:0]; Reserved[3:2];Reserved[5:\n    4];PXP_CTRL[7:6]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16657cUL: // (XSDM_REG_TIMERS_TM: ACCESS:RW  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXSDM_REG_TIMERS_TM (ACCESS:RW  DataWidth:0x2 )" \
				"\nDescription: TM bits for timers sram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166580UL: // (XSDM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSDM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166400UL: // (XSDM_REG_AGGREG_INTERRUPT_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_AGGREG_INTERRUPT_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: lsb register of aggregated interrupt in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166404UL: // (XSDM_REG_AGGREG_INTERRUPT_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_AGGREG_INTERRUPT_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: msb register of aggregated interrupt  in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166408UL: // (XSDM_REG_ASYNC_HOST_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_ASYNC_HOST_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: async fifo empty in sdm_async block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16640cUL: // (XSDM_REG_ASYNC_HOST_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_ASYNC_HOST_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: async fifo full in sdm_async block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166410UL: // (XSDM_REG_CFC_LOAD_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_CFC_LOAD_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166414UL: // (XSDM_REG_CFC_LOAD_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_CFC_LOAD_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load pending fifo full in sdm_cfc block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166418UL: // (XSDM_REG_CFC_LOAD_RSP_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_CFC_LOAD_RSP_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load rsp fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16641cUL: // (XSDM_REG_CFC_LOAD_RSP_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_CFC_LOAD_RSP_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cfc load rsp fifo full in sdm_cfcblock\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166420UL: // (XSDM_REG_CM_DELAY_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_CM_DELAY_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm delay fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166424UL: // (XSDM_REG_CM_DELAY_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_CM_DELAY_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm delay fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166428UL: // (XSDM_REG_CM_QUEUE_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_CM_QUEUE_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm queue fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16642cUL: // (XSDM_REG_CM_QUEUE_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_CM_QUEUE_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: cm queue fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166430UL: // (XSDM_REG_DELAY_FIFO_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DELAY_FIFO_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: delay FIFO  empty in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166434UL: // (XSDM_REG_DELAY_FIFO_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DELAY_FIFO_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: delay FIFO  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166438UL: // (XSDM_REG_DST_BRB1_CTRL_SRC_ADDR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_BRB1_CTRL_SRC_ADDR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src address fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16643cUL: // (XSDM_REG_DST_BRB1_CTRL_SRC_ADDR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_BRB1_CTRL_SRC_ADDR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src address fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166440UL: // (XSDM_REG_DST_BRB1_CTRL_SRC_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_BRB1_CTRL_SRC_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166444UL: // (XSDM_REG_DST_BRB1_CTRL_SRC_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_BRB1_CTRL_SRC_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl_src pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166448UL: // (XSDM_REG_DST_INT_RAM_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_INT_RAM_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16644cUL: // (XSDM_REG_DST_INT_RAM_WAIT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_INT_RAM_WAIT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram_wait fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166450UL: // (XSDM_REG_DST_INT_RAM_WAIT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_INT_RAM_WAIT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram_wait fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166454UL: // (XSDM_REG_DST_NONE_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_NONE_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: none pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166458UL: // (XSDM_REG_DST_NONE_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_NONE_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: none pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16645cUL: // (XSDM_REG_DST_PAS_BUF_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PAS_BUF_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166460UL: // (XSDM_REG_DST_PAS_BUF_WAIT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PAS_BUF_WAIT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf_wait fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166464UL: // (XSDM_REG_DST_PAS_BUF_WAIT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PAS_BUF_WAIT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pas_buf_wait fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166468UL: // (XSDM_REG_DST_PB_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PB_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16646cUL: // (XSDM_REG_DST_PB_IMMED_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PB_IMMED_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb immediate fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166470UL: // (XSDM_REG_DST_PB_IMMED_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PB_IMMED_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb immediate fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166474UL: // (XSDM_REG_DST_PXP_CTRL_DST_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_CTRL_DST_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_dst pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166478UL: // (XSDM_REG_DST_PXP_CTRL_DST_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_CTRL_DST_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_dst pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16647cUL: // (XSDM_REG_DST_PXP_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166480UL: // (XSDM_REG_DST_PXP_CTRL_IMMED_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_CTRL_IMMED_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl immediate fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166484UL: // (XSDM_REG_DST_PXP_CTRL_IMMED_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_CTRL_IMMED_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl immediate fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166488UL: // (XSDM_REG_DST_PXP_CTRL_LINK_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_CTRL_LINK_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl link list empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16648cUL: // (XSDM_REG_DST_PXP_CTRL_LINK_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_CTRL_LINK_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl link list full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166490UL: // (XSDM_REG_DST_PXP_CTRL_SRC_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_CTRL_SRC_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_src pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166494UL: // (XSDM_REG_DST_PXP_CTRL_SRC_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_CTRL_SRC_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl_src pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166498UL: // (XSDM_REG_DST_PXP_DP_DST_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_DP_DST_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp_dst pending fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16649cUL: // (XSDM_REG_DST_PXP_DP_DST_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_DP_DST_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp_dst pending fifo full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664a0UL: // (XSDM_REG_DST_PXP_DP_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_DP_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp if full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664a4UL: // (XSDM_REG_DST_PXP_DP_LINK_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_DP_LINK_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp link list empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664a8UL: // (XSDM_REG_DST_PXP_DP_LINK_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_DST_PXP_DP_LINK_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_dp link list full in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664acUL: // (XSDM_REG_INIT_CREDIT_CFC_ACDEC: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXSDM_REG_INIT_CREDIT_CFC_ACDEC (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     activity counters interface without receiving any ACK\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664b0UL: // (XSDM_REG_INIT_CREDIT_CFC_ACINC: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXSDM_REG_INIT_CREDIT_CFC_ACINC (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     activity counters interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664b4UL: // (XSDM_REG_INIT_CREDIT_CFC_LOAD: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXSDM_REG_INIT_CREDIT_CFC_LOAD (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the CFC\n     load interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664b8UL: // (XSDM_REG_INIT_CREDIT_CM: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXSDM_REG_INIT_CREDIT_CM (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of cycles that can be sent to the CM  \n    interface without receiving any ACK in CM block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664bcUL: // (XSDM_REG_INIT_CREDIT_PXP_CTRL: ACCESS:RW  DataWidth:0x4 ) 
			sprintf(disp_str, "\nXSDM_REG_INIT_CREDIT_PXP_CTRL (ACCESS:RW  DataWidth:0x4 )" \
				"\nDescription: The initial number of messages that can be sent to the pxp\n     control interface without receiving any ACK.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664c0UL: // (XSDM_REG_INT_RAM_RR_REQ: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSDM_REG_INT_RAM_RR_REQ (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: round robin for int_ram arbiter: b0-pas_buf; b1-int_ram;b2\n    -pxp_dp;b3-pxp_ctrl;b4-brb1_ctrl;b5-brb1_dp;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664c4UL: // (XSDM_REG_OPERATION_GEN: ACCESS:W  DataWidth:0x11 ) 
			sprintf(disp_str, "\nXSDM_REG_OPERATION_GEN (ACCESS:W  DataWidth:0x11 )" \
				"\nDescription: Generate an operation after completion; bit-16 is AggVectIdx_valid\n    ; bits 15:8 are AggVectIdx; bits 7:5 are the TRIG and bits 4:0 are the\n     T124Param[4:0]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664c8UL: // (XSDM_REG_PB_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_PB_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: UPB IF  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664ccUL: // (XSDM_REG_PBF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_PBF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: PBF if  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664d0UL: // (XSDM_REG_PXP_DELAY_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_PXP_DELAY_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp switch delay fifo empty in sdm_dma_dst block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664d4UL: // (XSDM_REG_PXP_DELAY_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_PXP_DELAY_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp switch delay fifo full in sdm_cm block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664d8UL: // (XSDM_REG_QM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_QM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: QM IF  full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664dcUL: // (XSDM_REG_QUEUE_EMPTY: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nXSDM_REG_QUEUE_EMPTY (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Input queue fifo empty in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664e0UL: // (XSDM_REG_QUEUE_FULL: ACCESS:R  DataWidth:0xc ) 
			sprintf(disp_str, "\nXSDM_REG_QUEUE_FULL (ACCESS:R  DataWidth:0xc )" \
				"\nDescription: Input queue fifo full in sdm_inp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664e4UL: // (XSDM_REG_RR_CNT_COUNTERS_STATUS: ACCESS:R  DataWidth:0x15 ) 
			sprintf(disp_str, "\nXSDM_REG_RR_CNT_COUNTERS_STATUS (ACCESS:R  DataWidth:0x15 )" \
				"\nDescription: round robin for all completion counters\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664e8UL: // (XSDM_REG_RR_COMPLETE_REQ: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nXSDM_REG_RR_COMPLETE_REQ (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: round robin for all completion requests  in sdm_cm block: \n    b0-async b1-nop;b2-pxp_int; b3-timers;b4-dma;b5-grc;b6-rbc\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664ecUL: // (XSDM_REG_RR_PTR_REQ: ACCESS:R  DataWidth:0x7 ) 
			sprintf(disp_str, "\nXSDM_REG_RR_PTR_REQ (ACCESS:R  DataWidth:0x7 )" \
				"\nDescription: round robin for cm pointer: b0-async; b1-dma_dp; b2 - dma_ctrl\n    ; b3-cfc; b4-nop; b5-timers; b6-pxp_int\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664f0UL: // (XSDM_REG_RSP_BRB1_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664f4UL: // (XSDM_REG_RSP_BRB1_CTRL_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_CTRL_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664f8UL: // (XSDM_REG_RSP_BRB1_CTRL_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_CTRL_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1664fcUL: // (XSDM_REG_RSP_BRB1_CTRL_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_CTRL_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166500UL: // (XSDM_REG_RSP_BRB1_CTRL_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_CTRL_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_ctrl rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166504UL: // (XSDM_REG_RSP_BRB1_DP_DST_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_DP_DST_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending dst fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166508UL: // (XSDM_REG_RSP_BRB1_DP_DST_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_DP_DST_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending dst fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16650cUL: // (XSDM_REG_RSP_BRB1_DP_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_DP_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166510UL: // (XSDM_REG_RSP_BRB1_DP_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_DP_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166514UL: // (XSDM_REG_RSP_BRB1_DP_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_DP_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166518UL: // (XSDM_REG_RSP_BRB1_DP_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_DP_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16651cUL: // (XSDM_REG_RSP_BRB1_DP_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_BRB1_DP_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: brb1_dp rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166520UL: // (XSDM_REG_RSP_INT_RAM_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_INT_RAM_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166524UL: // (XSDM_REG_RSP_INT_RAM_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_INT_RAM_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166528UL: // (XSDM_REG_RSP_INT_RAM_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_INT_RAM_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16652cUL: // (XSDM_REG_RSP_INT_RAM_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_INT_RAM_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: int_ram rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166530UL: // (XSDM_REG_RSP_PB_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_PB_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166534UL: // (XSDM_REG_RSP_PB_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_PB_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb pending fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166538UL: // (XSDM_REG_RSP_PB_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_PB_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb pending fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16653cUL: // (XSDM_REG_RSP_PB_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_PB_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166540UL: // (XSDM_REG_RSP_PB_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_PB_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166544UL: // (XSDM_REG_RSP_PXP_CTRL_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_PXP_CTRL_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pb if is  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166548UL: // (XSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_PXP_CTRL_RDATA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl rd_data fifo  empty in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16654cUL: // (XSDM_REG_RSP_PXP_CTRL_RDATA_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_RSP_PXP_CTRL_RDATA_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pxp_ctrl rd_data fifo  full in sdm_dma_rsp block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166550UL: // (XSDM_REG_SYNC_PARSER_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_SYNC_PARSER_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser fifo  empty in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166554UL: // (XSDM_REG_SYNC_PARSER_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_SYNC_PARSER_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser fifo  full in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166558UL: // (XSDM_REG_SYNC_SYNC_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_SYNC_SYNC_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser serial fifo  empty in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16655cUL: // (XSDM_REG_SYNC_SYNC_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_SYNC_SYNC_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: parser serial fifo  full in sdm_sync block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166560UL: // (XSDM_REG_TIMERS_ADDR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_TIMERS_ADDR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: address FIFO  empty in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166564UL: // (XSDM_REG_TIMERS_ADDR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_TIMERS_ADDR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: address FIFO  full in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166568UL: // (XSDM_REG_TIMERS_PEND_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_TIMERS_PEND_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pending FIFO  empty in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x16656cUL: // (XSDM_REG_TIMERS_PEND_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSDM_REG_TIMERS_PEND_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: pending FIFO  full in sdm_timers block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166600UL: // (XSDM_REG_TIMERS: ACCESS:WB  DataWidth:0x34 ) 
			sprintf(disp_str, "\nXSDM_REG_TIMERS (ACCESS:WB  DataWidth:0x34 )" \
				"\nDescription: Debug only. Timers memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166800UL: // (XSDM_REG_STATISTICS: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSDM_REG_STATISTICS (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Statistics memory. Each read from RBC resets the corresponding\n     statistic counter\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x167000UL: // (XSDM_REG_CM_QUEUE: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nXSDM_REG_CM_QUEUE (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Debug only. CM queue memory. \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x167800UL: // (XSDM_REG_INP_QUEUE: ACCESS:WB  DataWidth:0x40 ) 
			sprintf(disp_str, "\nXSDM_REG_INP_QUEUE (ACCESS:WB  DataWidth:0x40 )" \
				"\nDescription: Debug only. Input queue memory.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1662c0UL: // (XSDM_REG_XSDM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x1662e8UL: // (XSDM_REG_XSDM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166584UL: // (XSDM_REG_XSDM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x166c00UL: // (XSDM_REG_XSDM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSDM_REG_XSDM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280000UL: // (XSEM_REG_MSG_NUM_FIC0: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXSEM_REG_MSG_NUM_FIC0 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that entered \n    through FIC0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280004UL: // (XSEM_REG_MSG_NUM_FIC1: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXSEM_REG_MSG_NUM_FIC1 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that entered \n    through FIC1 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280008UL: // (XSEM_REG_MSG_NUM_FOC0: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXSEM_REG_MSG_NUM_FOC0 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC0 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28000cUL: // (XSEM_REG_MSG_NUM_FOC1: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXSEM_REG_MSG_NUM_FOC1 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC1 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280010UL: // (XSEM_REG_MSG_NUM_FOC2: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXSEM_REG_MSG_NUM_FOC2 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC2 \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280014UL: // (XSEM_REG_MSG_NUM_FOC3: ACCESS:ST  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXSEM_REG_MSG_NUM_FOC3 (ACCESS:ST  DataWidth:0x18 )" \
				"\nDescription: Statistics register. The number of messages that were sent\n     to FOC3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280018UL: // (XSEM_REG_THREAD_INTER_CNT_ENABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_THREAD_INTER_CNT_ENABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Enable for start count of counter ~xsem_registers_thread_inter_cnt\n    .thread_inter_cnt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28001cUL: // (XSEM_REG_THREAD_INTER_CNT: ACCESS:RW  DataWidth:0x10 ) 
			sprintf(disp_str, "\nXSEM_REG_THREAD_INTER_CNT (ACCESS:RW  DataWidth:0x10 )" \
				"\nDescription: Maximum value of threads interrupt counter; when it gets \n    this value then interrupt to will be send if thread active from previous\n     maximum value of this counter. This register may be used only when ~xsem_registers_thread_inter_cnt_enable\n    .thread_inter_cnt_enable =1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280020UL: // (XSEM_REG_ARB_ELEMENT0: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_ARB_ELEMENT0 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 0. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280024UL: // (XSEM_REG_ARB_ELEMENT1: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_ARB_ELEMENT1 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 1. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~xsem_registers_arb_element0.arb_element0\n    \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280028UL: // (XSEM_REG_ARB_ELEMENT2: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_ARB_ELEMENT2 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 2. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~xsem_registers_arb_element0.arb_element0\n     and ~xsem_registers_arb_element1.arb_element1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28002cUL: // (XSEM_REG_ARB_ELEMENT3: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_ARB_ELEMENT3 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 3. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2.Could not be equal to register ~xsem_registers_arb_element0.arb_element0\n     and ~xsem_registers_arb_element1.arb_element1 and ~xsem_registers_arb_element2\n    .arb_element2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280030UL: // (XSEM_REG_ARB_ELEMENT4: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_ARB_ELEMENT4 (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The source that is associated with arbitration element 4. \n    Source decoding is: 0- foc0; 1-fic1; 2-sleeping thread with priority 0\n    ; 3- sleeping thread with priority 1; 4- sleeping thread with priority\n     2. Could not be equal to register ~xsem_registers_arb_element0.arb_element0\n     and ~xsem_registers_arb_element1.arb_element1 and ~xsem_registers_arb_element2\n    .arb_element2 and ~xsem_registers_arb_element3.arb_element3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280038UL: // (XSEM_REG_TS_0_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_0_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28003cUL: // (XSEM_REG_TS_1_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_1_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280040UL: // (XSEM_REG_TS_2_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_2_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280044UL: // (XSEM_REG_TS_3_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_3_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 3\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280048UL: // (XSEM_REG_TS_4_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_4_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 4\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28004cUL: // (XSEM_REG_TS_5_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_5_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 5\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280050UL: // (XSEM_REG_TS_6_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_6_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 6\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280054UL: // (XSEM_REG_TS_7_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_7_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 7\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280058UL: // (XSEM_REG_TS_8_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_8_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 8\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28005cUL: // (XSEM_REG_TS_9_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_9_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 9\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280060UL: // (XSEM_REG_TS_10_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_10_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 10\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280064UL: // (XSEM_REG_TS_11_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_11_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 11\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280068UL: // (XSEM_REG_TS_12_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_12_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 12\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28006cUL: // (XSEM_REG_TS_13_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_13_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 13\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280070UL: // (XSEM_REG_TS_14_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_14_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 14\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280074UL: // (XSEM_REG_TS_15_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_15_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 15\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280078UL: // (XSEM_REG_TS_16_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_16_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 16\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28007cUL: // (XSEM_REG_TS_17_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_17_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 17\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280080UL: // (XSEM_REG_TS_18_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_18_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 18\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280084UL: // (XSEM_REG_TS_19_AS: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_TS_19_AS (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: The arbitration scheme of time_slot 19\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280088UL: // (XSEM_REG_FIC0_MIN_MSG_LINES: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC0_MIN_MSG_LINES (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The minimum number of cycles in a message from FIC0 interfaces\n     after which the message can be sent to the passive register_file. It \n    must be always equal to 2; other way it may cause to deadlock in a chip\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28008cUL: // (XSEM_REG_FIC1_MIN_MSG_LINES: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC1_MIN_MSG_LINES (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: The minimum number of cycles in a message from FIC1 interfaces\n     after which the message can be sent to the passive register_file. It \n    must be always equal to 2; other way it may cause to deadlock in a chip\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280090UL: // (XSEM_REG_PASSIVE_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_PASSIVE_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: The number of free entries in the sync FIFO between the external\n     HW and the passive buffer; below which the PassiveFull is asserted \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280094UL: // (XSEM_REG_SYNC_DRA_WR_ALM_FULL: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_SYNC_DRA_WR_ALM_FULL (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Almost full for sync dra_wr fifo (data from DRA to STORM)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280098UL: // (XSEM_REG_SYNC_RAM_WR_ALM_FULL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSEM_REG_SYNC_RAM_WR_ALM_FULL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Almost full for sync ram_wr fifo (data from EXT_IF to STORM\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28009cUL: // (XSEM_REG_DBG_ALM_FULL: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSEM_REG_DBG_ALM_FULL (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Almost full for slow debug fifo \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800a0UL: // (XSEM_REG_EXCEPTION_INT: ACCESS:RW  DataWidth:0xf ) 
			sprintf(disp_str, "\nXSEM_REG_EXCEPTION_INT (ACCESS:RW  DataWidth:0xf )" \
				"\nDescription: The PRAM address for the interrupt in a case the event ID\n     is bigger then the INT table size. This register is always NA because\n     this feature is removed\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800a4UL: // (XSEM_REG_ENABLE_IN: ACCESS:RW  DataWidth:0xf  Multi Field Register  ) 
			sprintf(disp_str, "\nXSEM_REG_ENABLE_IN (ACCESS:RW  DataWidth:0xf  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFPF_ERROR_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_VFPF_ERROR_ENABLE_IN) >> 14)
				, "\n\tWAITP_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_WAITP_ENABLE_IN) >> 13)
				, "\n\tFOC3_ACK_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_FOC3_ACK_ENABLE_IN) >> 12)
				, "\n\tFOC2_ACK_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_FOC2_ACK_ENABLE_IN) >> 11)
				, "\n\tFOC1_ACK_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_FOC1_ACK_ENABLE_IN) >> 10)
				, "\n\tFOC0_ACK_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_FOC0_ACK_ENABLE_IN) >> 9)
				, "\n\tRAM1_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_RAM1_ENABLE_IN) >> 8)
				, "\n\tRAM0_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_RAM0_ENABLE_IN) >> 7)
				, "\n\tEXT_FULL_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_EXT_FULL_ENABLE_IN) >> 6)
				, "\n\tEXT_RD_DATA_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_EXT_RD_DATA_ENABLE_IN) >> 5)
				, "\n\tTHREAD_RDY_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_THREAD_RDY_ENABLE_IN) >> 4)
				, "\n\tGENERAL_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_GENERAL_ENABLE_IN) >> 3)
				, "\n\tPASSIVE_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_PASSIVE_ENABLE_IN) >> 2)
				, "\n\tFIC1_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_FIC1_ENABLE_IN) >> 1)
				, "\n\tFIC0_ENABLE_IN :"
				, (u32_t)((data & XSEM_ENABLE_IN_REG_FIC0_ENABLE_IN) >> 0));
			break;
		case 0x2800a8UL: // (XSEM_REG_ENABLE_OUT: ACCESS:RW  DataWidth:0xa  Multi Field Register  ) 
			sprintf(disp_str, "\nXSEM_REG_ENABLE_OUT (ACCESS:RW  DataWidth:0xa  Multi Field Register  )" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tWAITP_ENABLE_OUT :"
				, (u32_t)((data & XSEM_ENABLE_OUT_REG_WAITP_ENABLE_OUT) >> 9)
				, "\n\tRAM1_ENABLE_OUT :"
				, (u32_t)((data & XSEM_ENABLE_OUT_REG_RAM1_ENABLE_OUT) >> 8)
				, "\n\tRAM0_ENABLE_OUT :"
				, (u32_t)((data & XSEM_ENABLE_OUT_REG_RAM0_ENABLE_OUT) >> 7)
				, "\n\tPASSIVE_ENABLE_OUT :"
				, (u32_t)((data & XSEM_ENABLE_OUT_REG_PASSIVE_ENABLE_OUT) >> 6)
				, "\n\tFOC3_ENABLE_OUT :"
				, (u32_t)((data & XSEM_ENABLE_OUT_REG_FOC3_ENABLE_OUT) >> 5)
				, "\n\tFOC2_ENABLE_OUT :"
				, (u32_t)((data & XSEM_ENABLE_OUT_REG_FOC2_ENABLE_OUT) >> 4)
				, "\n\tFOC1_ENABLE_OUT :"
				, (u32_t)((data & XSEM_ENABLE_OUT_REG_FOC1_ENABLE_OUT) >> 3)
				, "\n\tFOC0_ENABLE_OUT :"
				, (u32_t)((data & XSEM_ENABLE_OUT_REG_FOC0_ENABLE_OUT) >> 2)
				, "\n\tEXT_WR_REQ_ENABLE_OUT :"
				, (u32_t)((data & XSEM_ENABLE_OUT_REG_EXT_WR_REQ_ENABLE_OUT) >> 1)
				, "\n\tEXT_RD_REQ_ENABLE_OUT :"
				, (u32_t)((data & XSEM_ENABLE_OUT_REG_EXT_RD_REQ_ENABLE_OUT) >> 0));
			break;
		case 0x2800acUL: // (XSEM_REG_STORM0_H_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_STORM0_H_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm0_h memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800b0UL: // (XSEM_REG_STORM1_H_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_STORM1_H_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm1_h memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800b4UL: // (XSEM_REG_STORM0_L_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_STORM0_L_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm0_l memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800b8UL: // (XSEM_REG_STORM1_L_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_STORM1_L_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for storm1_l memory instance\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800bcUL: // (XSEM_REG_CAM_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSEM_REG_CAM_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM bits  for cam\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800c0UL: // (XSEM_REG_RAM0_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_RAM0_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tm for ram0_0\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800c4UL: // (XSEM_REG_PAS_BUF_LSB_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_PAS_BUF_LSB_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_lsb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800c8UL: // (XSEM_REG_PAS_BUF_LSB_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_PAS_BUF_LSB_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_lsb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800ccUL: // (XSEM_REG_PAS_BUF_MSB_TMA: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_PAS_BUF_MSB_TMA (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_msb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800d0UL: // (XSEM_REG_PAS_BUF_MSB_TMB: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_PAS_BUF_MSB_TMB (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory pas_buf_msb\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800d4UL: // (XSEM_REG_INT_TABLE_TM: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_INT_TABLE_TM (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: TM bits  for memory int_table\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800d8UL: // (XSEM_REG_CLEAR_WAITP: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_CLEAR_WAITP (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Write 1 to this register will disable waitp from this storm\n     to other storms\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800dcUL: // (XSEM_REG_SLOW_DBG_MODE: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DBG_MODE (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: debug mode for slow debug bus. Applicable only when ~xsem_registers_slow_dbg_active\n    .slow_dbg_active =1. If mode =0 thread number; pram address and DRA WR\n     data selected; if mode =1 fin command and DRA RD ; if mode =2 pram address\n     and thread number and fin command and released thread from STORM; if \n    mode =3 STORE data to SDM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800e0UL: // (XSEM_REG_SLOW_DBG_ACTIVE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DBG_ACTIVE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: debug mode is active\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800e4UL: // (XSEM_REG_DBG_MSG_SRC: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_DBG_MSG_SRC (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~xsem_registers_slow_dbg_mode.slow_dbg_mode\n     =0. If =0only FIC-s output to debug bus; 1=both FIC-s and passive buffer\n    .\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800e8UL: // (XSEM_REG_DBG_MODE0_CFG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_DBG_MODE0_CFG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~xsem_registers_slow_dbg_mode.slow_dbg_mode\n     =0. If =0 all the message output to debug bus; 1=partial message.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800ecUL: // (XSEM_REG_DBG_MODE0_CFG_CYCLE: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_DBG_MODE0_CFG_CYCLE (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: Applicable only when ~xsem_registers_dbg_mode0_cfg.dbg_mode0_cfg\n     =1. If =1 the additional cycles to extract to the debug bus.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800f0UL: // (XSEM_REG_DBG_MODE1_CFG: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_DBG_MODE1_CFG (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Applicable only when ~xsem_registers_slow_dbg_mode.slow_dbg_mode\n     =1. If=0 output to debug bus without the data; 1=with the data.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800f4UL: // (XSEM_REG_DBG_EACH_CYLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_DBG_EACH_CYLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: If=0 output every cycle full indication or thread status; \n    1= output only when there is a change.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800f8UL: // (XSEM_REG_DBG_SELECT: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSEM_REG_DBG_SELECT (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only. For dbgmux usage (debug data that goes from USEMI\n     to the DBG block) - for selecting a line to output to the DBG block\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2800fcUL: // (XSEM_REG_DBG_BYTE_ENABLE: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSEM_REG_DBG_BYTE_ENABLE (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Debug only.For dbgmux usage (debug data that goes from USEM\n     to the DBG block) - for enabling bytes in the selected line (after the\n     select before the shift).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280100UL: // (XSEM_REG_DBG_SHIFT: ACCESS:RW  DataWidth:0x3 ) 
			sprintf(disp_str, "\nXSEM_REG_DBG_SHIFT (ACCESS:RW  DataWidth:0x3 )" \
				"\nDescription: Debug only.For dbgmux usage (debug data that goes from USEM\n     to the DBG block) - for circular right shifting of the selected line (\n    after the enabling).\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280104UL: // (XSEM_REG_XSEM_INT_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_INT_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x280108UL: // (XSEM_REG_XSEM_INT_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_INT_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x28010cUL: // (XSEM_REG_XSEM_INT_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_INT_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Interrupt register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x280110UL: // (XSEM_REG_XSEM_INT_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_INT_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Interrupt mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tSYNC_RAM0_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_RAM0_RD_PUSH_ERROR) >> 31)
				, "\n\tSYNC_EXT_STORE_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_EXT_STORE_POP_ERROR) >> 30)
				, "\n\tSYNC_EXT_STORE_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_EXT_STORE_PUSH_ERROR) >> 29)
				, "\n\tTHREAD_OVERRUN :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_THREAD_OVERRUN) >> 28)
				, "\n\tSET1_THREAD_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SET1_THREAD_ERROR) >> 27)
				, "\n\tSET0_THREAD_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SET0_THREAD_ERROR) >> 26)
				, "\n\tFIN_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_FIN_FIFO) >> 25)
				, "\n\tCAM_OUT_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_CAM_OUT_FIFO) >> 24)
				, "\n\tCAM_MSB_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_CAM_MSB_INP_FIFO) >> 23)
				, "\n\tCAM_LSB_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_CAM_LSB_INP_FIFO) >> 22)
				, "\n\tWR_FULL_CAM :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_WR_FULL_CAM) >> 21)
				, "\n\tRD_EMPTY_CAM :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_RD_EMPTY_CAM) >> 20)
				, "\n\tWR_FULL_LOAD_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_WR_FULL_LOAD_FIFO) >> 19)
				, "\n\tRD_EMPTY_LOAD_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_RD_EMPTY_LOAD_FIFO) >> 18)
				, "\n\tDRA_DATA_WR_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_DRA_DATA_WR_ERROR) >> 17)
				, "\n\tMAX_HANDLER_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_MAX_HANDLER_ERROR) >> 16)
				, "\n\tSYNC_FIN_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_FIN_PUSH_ERROR) >> 15)
				, "\n\tSYNC_DRA_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_DRA_RD_POP_ERROR) >> 14)
				, "\n\tSYNC_DRA_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_DRA_RD_PUSH_ERROR) >> 13)
				, "\n\tSYNC_DRA_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_DRA_WR_POP_ERROR) >> 12)
				, "\n\tSYNC_DRA_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_DRA_WR_PUSH_ERROR) >> 11)
				, "\n\tSYNC_FIN_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_FIN_POP_ERROR) >> 10)
				, "\n\tPAS_BUF_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_PAS_BUF_FIFO_ERROR) >> 9)
				, "\n\tSYNC_INT_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_INT_POP_ERROR) >> 8)
				, "\n\tSYNC_INT_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_SYNC_INT_PUSH_ERROR) >> 7)
				, "\n\tFIC1_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_FIC1_FIFO_ERROR) >> 6)
				, "\n\tFIC0_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_FIC0_FIFO_ERROR) >> 5)
				, "\n\tFIC1_LENGTH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_FIC1_LENGTH_ERROR) >> 4)
				, "\n\tFIC0_LENGTH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_FIC0_LENGTH_ERROR) >> 3)
				, "\n\tFIC1_LAST_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_FIC1_LAST_ERROR) >> 2)
				, "\n\tFIC0_LAST_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_FIC0_LAST_ERROR) >> 1)
				, "\n\tADDRESS_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_0_REG_ADDRESS_ERROR) >> 0));
			break;
		case 0x280114UL: // (XSEM_REG_XSEM_INT_STS_1: ACCESS:R  DataWidth:0xd ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_INT_STS_1 (ACCESS:R  DataWidth:0xd )" \
				"\nDescription: Interrupt register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_OUT_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_VFC_OUT_FIFO_ERROR) >> 12)
				, "\n\tVFC_INTERRUPT :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_VFC_INTERRUPT) >> 11)
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x280118UL: // (XSEM_REG_XSEM_INT_STS_CLR_1: ACCESS:RC  DataWidth:0xd ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_INT_STS_CLR_1 (ACCESS:RC  DataWidth:0xd )" \
				"\nDescription: Interrupt register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_OUT_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_VFC_OUT_FIFO_ERROR) >> 12)
				, "\n\tVFC_INTERRUPT :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_VFC_INTERRUPT) >> 11)
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_CLR_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x28011cUL: // (XSEM_REG_XSEM_INT_STS_WR_1: ACCESS:WR  DataWidth:0xd ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_INT_STS_WR_1 (ACCESS:WR  DataWidth:0xd )" \
				"\nDescription: Interrupt register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_OUT_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_VFC_OUT_FIFO_ERROR) >> 12)
				, "\n\tVFC_INTERRUPT :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_VFC_INTERRUPT) >> 11)
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_STS_WR_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x280120UL: // (XSEM_REG_XSEM_INT_MASK_1: ACCESS:RW  DataWidth:0xd ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_INT_MASK_1 (ACCESS:RW  DataWidth:0xd )" \
				"\nDescription: Interrupt mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_OUT_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_VFC_OUT_FIFO_ERROR) >> 12)
				, "\n\tVFC_INTERRUPT :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_VFC_INTERRUPT) >> 11)
				, "\n\tCAM_MSB2_INP_FIFO :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_CAM_MSB2_INP_FIFO) >> 10)
				, "\n\tDBG_FIFO_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_DBG_FIFO_ERROR) >> 9)
				, "\n\tSYNC_DBG_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_SYNC_DBG_POP_ERROR) >> 8)
				, "\n\tSYNC_DBG_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_SYNC_DBG_PUSH_ERROR) >> 7)
				, "\n\tSYNC_RAM1_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_SYNC_RAM1_WR_PUSH_ERROR) >> 6)
				, "\n\tSYNC_RAM0_WR_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_SYNC_RAM0_WR_PUSH_ERROR) >> 5)
				, "\n\tSYNC_RAM1_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_SYNC_RAM1_WR_POP_ERROR) >> 4)
				, "\n\tSYNC_RAM0_WR_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_SYNC_RAM0_WR_POP_ERROR) >> 3)
				, "\n\tSYNC_RAM1_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_SYNC_RAM1_RD_POP_ERROR) >> 2)
				, "\n\tSYNC_RAM0_RD_POP_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_SYNC_RAM0_RD_POP_ERROR) >> 1)
				, "\n\tSYNC_RAM1_RD_PUSH_ERROR :"
				, (u32_t)((data & XSEM_XSEM_INT_MASK_1_REG_SYNC_RAM1_RD_PUSH_ERROR) >> 0));
			break;
		case 0x280124UL: // (XSEM_REG_XSEM_PRTY_STS_0: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_PRTY_STS_0 (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_0_REG_PARITY) >> 0));
			break;
		case 0x280128UL: // (XSEM_REG_XSEM_PRTY_STS_CLR_0: ACCESS:RC  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_PRTY_STS_CLR_0 (ACCESS:RC  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_0_REG_PARITY) >> 0));
			break;
		case 0x28012cUL: // (XSEM_REG_XSEM_PRTY_STS_WR_0: ACCESS:WR  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_PRTY_STS_WR_0 (ACCESS:WR  DataWidth:0x20 )" \
				"\nDescription: Parity register #0 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_0_REG_PARITY) >> 0));
			break;
		case 0x280130UL: // (XSEM_REG_XSEM_PRTY_MASK_0: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_PRTY_MASK_0 (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: Parity mask register #0 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tPRAM_HIGH_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_PRAM_HIGH_PARITY) >> 31)
				, "\n\tPRAM_LOW_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_PRAM_LOW_PARITY) >> 30)
				, "\n\tRAM1_PARITY7 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM1_PARITY7) >> 29)
				, "\n\tRAM1_PARITY6 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM1_PARITY6) >> 28)
				, "\n\tRAM1_PARITY5 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM1_PARITY5) >> 27)
				, "\n\tRAM1_PARITY4 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM1_PARITY4) >> 26)
				, "\n\tRAM1_PARITY3 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM1_PARITY3) >> 25)
				, "\n\tRAM1_PARITY2 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM1_PARITY2) >> 24)
				, "\n\tRAM1_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM1_PARITY1) >> 23)
				, "\n\tRAM1_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM1_PARITY0) >> 22)
				, "\n\tRAM0_PARITY7 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM0_PARITY7) >> 21)
				, "\n\tRAM0_PARITY6 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM0_PARITY6) >> 20)
				, "\n\tRAM0_PARITY5 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM0_PARITY5) >> 19)
				, "\n\tRAM0_PARITY4 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM0_PARITY4) >> 18)
				, "\n\tRAM0_PARITY3 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM0_PARITY3) >> 17)
				, "\n\tRAM0_PARITY2 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM0_PARITY2) >> 16)
				, "\n\tRAM0_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM0_PARITY1) >> 15)
				, "\n\tRAM0_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_RAM0_PARITY0) >> 14)
				, "\n\tINT_TABLE_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_INT_TABLE_PARITY) >> 13)
				, "\n\tPAS_PARITY1 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_PAS_PARITY1) >> 12)
				, "\n\tPAS_PARITY0 :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_PAS_PARITY0) >> 11)
				, "\n\tPAS_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_PAS_FIFO_PARITY) >> 10)
				, "\n\tFIC1_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_FIC1_FIFO_PARITY) >> 9)
				, "\n\tFIC0_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_FIC0_FIFO_PARITY) >> 8)
				, "\n\tSYNC_EXT_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_SYNC_EXT_FIFO_PARITY) >> 7)
				, "\n\tSYNC_RAM1_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_SYNC_RAM1_WR_FIFO_PARITY) >> 6)
				, "\n\tSYNC_RAM0_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_SYNC_RAM0_WR_FIFO_PARITY) >> 5)
				, "\n\tSYNC_RAM1_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_SYNC_RAM1_RD_FIFO_PARITY) >> 4)
				, "\n\tSYNC_RAM0_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_SYNC_RAM0_RD_FIFO_PARITY) >> 3)
				, "\n\tSYNC_DRA_RD_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_SYNC_DRA_RD_FIFO_PARITY) >> 2)
				, "\n\tSYNC_DRA_WR_FIFO_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_SYNC_DRA_WR_FIFO_PARITY) >> 1)
				, "\n\tPARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_0_REG_PARITY) >> 0));
			break;
		case 0x280134UL: // (XSEM_REG_XSEM_PRTY_STS_1: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_PRTY_STS_1 (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Parity register #1 read  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_1_REG_VFC_PARITY) >> 5)
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x280138UL: // (XSEM_REG_XSEM_PRTY_STS_CLR_1: ACCESS:RC  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_PRTY_STS_CLR_1 (ACCESS:RC  DataWidth:0x6 )" \
				"\nDescription: Parity register #1 read clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_1_REG_VFC_PARITY) >> 5)
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_CLR_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x28013cUL: // (XSEM_REG_XSEM_PRTY_STS_WR_1: ACCESS:WR  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_PRTY_STS_WR_1 (ACCESS:WR  DataWidth:0x6 )" \
				"\nDescription: Parity register #1 bit set or clear  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_1_REG_VFC_PARITY) >> 5)
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_STS_WR_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x280140UL: // (XSEM_REG_XSEM_PRTY_MASK_1: ACCESS:RW  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_PRTY_MASK_1 (ACCESS:RW  DataWidth:0x6 )" \
				"\nDescription: Parity mask register #1 read/write  \n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tVFC_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_1_REG_VFC_PARITY) >> 5)
				, "\n\tSTORM_RF1_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_1_REG_STORM_RF1_PARITY) >> 4)
				, "\n\tSTORM_RF0_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_1_REG_STORM_RF0_PARITY) >> 3)
				, "\n\tCAM_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_1_REG_CAM_PARITY) >> 2)
				, "\n\tSLOW_DBG_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_1_REG_SLOW_DBG_PARITY) >> 1)
				, "\n\tSYNC_DBG_PARITY :"
				, (u32_t)((data & XSEM_XSEM_PRTY_MASK_1_REG_SYNC_DBG_PARITY) >> 0));
			break;
		case 0x28014cUL: // (XSEM_REG_RAM0_TM1: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_RAM0_TM1 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tm for ram0_1\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280150UL: // (XSEM_REG_RAM0_TM2: ACCESS:RW  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_RAM0_TM2 (ACCESS:RW  DataWidth:0x5 )" \
				"\nDescription: tm for ram0_2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2803a8UL: // (XSEM_REG_ECO_RESERVED: ACCESS:RW  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSEM_REG_ECO_RESERVED (ACCESS:RW  DataWidth:0x8 )" \
				"\nDescription: Reserved bits for ECO\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2803acUL: // (XSEM_REG_FIFOS_TM: ACCESS:RW  DataWidth:0xe ) 
			sprintf(disp_str, "\nXSEM_REG_FIFOS_TM (ACCESS:RW  DataWidth:0xe )" \
				"\nDescription: TM bits  for  FIC0_LSB [1:0]; FIC0_MSB[3:2]; FIC1_LSB[5:4\n    ]; FIC1_MSB[7:6]; DBG_LSB[9:8];DBG_MSB[11:10]; EXT_PAS[13:12]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280200UL: // (XSEM_REG_ARBITER_REQUEST: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_ARBITER_REQUEST (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last request: 0-fic0; 1-fic1; 2-priority0; 3-\n    priority1; 4-priority2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280204UL: // (XSEM_REG_ARBITER_SELECT: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_ARBITER_SELECT (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last selection: 0-fic0; 1-fic1; 2-priority0; \n    3-priority1; 4-priority2\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280208UL: // (XSEM_REG_ARBITER_SLOT: ACCESS:R  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_ARBITER_SLOT (ACCESS:R  DataWidth:0x5 )" \
				"\nDescription: dra arbiter last slot\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28020cUL: // (XSEM_REG_DBG_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_DBG_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG IF is full in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280210UL: // (XSEM_REG_DRA_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_DRA_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: This register is active when FIN FIO  is empty and DRA RD\n     FIFO is empty\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280214UL: // (XSEM_REG_EXT_PAS_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_EXT_PAS_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_PAS FIFO empty in sem_slow\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280218UL: // (XSEM_REG_EXT_PAS_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_EXT_PAS_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_PAS FIFO Full in sem_slow\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28021cUL: // (XSEM_REG_EXT_STORE_FREE_ENTRIES: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSEM_REG_EXT_STORE_FREE_ENTRIES (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: Number of free entries in the external STORE sync FIFO.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280220UL: // (XSEM_REG_EXT_STORE_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_EXT_STORE_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE IF is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280224UL: // (XSEM_REG_FIC0_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC0_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from FIC0  May be updated during \n    run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280228UL: // (XSEM_REG_FIC0_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC0_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO empty in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28022cUL: // (XSEM_REG_FIC0_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC0_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO Full in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280230UL: // (XSEM_REG_FIC0_LENGTH: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC0_LENGTH (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Length from FIC0. Active only with ~xsem_registers_fic0_length_error\n    .fic0_length_error interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280234UL: // (XSEM_REG_FIC1_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC1_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from FIC1  May be updated during \n    run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280238UL: // (XSEM_REG_FIC1_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC1_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO empty in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28023cUL: // (XSEM_REG_FIC1_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC1_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIC0 FIFO Full in sem_slow_fic\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280240UL: // (XSEM_REG_FIC1_LENGTH: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC1_LENGTH (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: Length from FIC1. Active only with ~xsem_registers_fic1_length_error\n    .fic1_length_error interrupt\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280244UL: // (XSEM_REG_GPI_DATA: ACCESS:R  DataWidth:0x18 ) 
			sprintf(disp_str, "\nXSEM_REG_GPI_DATA (ACCESS:R  DataWidth:0x18 )" \
				"\nDescription: GPI signals that are inputs to SEMI\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280248UL: // (XSEM_REG_NUM_OF_THREADS: ACCESS:R  DataWidth:0x6 ) 
			sprintf(disp_str, "\nXSEM_REG_NUM_OF_THREADS (ACCESS:R  DataWidth:0x6 )" \
				"\nDescription: The number of threads currently active \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28024cUL: // (XSEM_REG_PAS_DISABLE: ACCESS:RW  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_PAS_DISABLE (ACCESS:RW  DataWidth:0x1 )" \
				"\nDescription: Disables input messages from the passive buffer  May be updated\n     during run_time by the microcode \n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280250UL: // (XSEM_REG_PAS_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_PAS_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Full from passive buffer asserted toward SDM\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280254UL: // (XSEM_REG_RAM0_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_RAM0_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RAM0 IF is full in sem_slow_ls_ram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280258UL: // (XSEM_REG_RAM1_IF_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_RAM1_IF_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RAM1 IF is full in sem_slow_ls_ram\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28025cUL: // (XSEM_REG_SET0_THREAD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SET0_THREAD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET0_THREAD fifo is empty in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280260UL: // (XSEM_REG_SET0_THREAD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SET0_THREAD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET0_THREAD fifo is full in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280264UL: // (XSEM_REG_SET1_THREAD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SET1_THREAD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET1_THREAD fifo is empty in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280268UL: // (XSEM_REG_SET1_THREAD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SET1_THREAD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: SET1_THREAD fifo is full in sem_slow_dra_wr\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28026cUL: // (XSEM_REG_SLEEP_THREADS_VALID: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nXSEM_REG_SLEEP_THREADS_VALID (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Valid sleeping threads indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280270UL: // (XSEM_REG_SLOW_DBG_ALM_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DBG_ALM_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is almost empty in sem_slow_ls_dbg (31 entry inside\n     fifo)\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280274UL: // (XSEM_REG_SLOW_DBG_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DBG_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is almost empty in sem_slow_ls_dbg according to \n    configuration\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280278UL: // (XSEM_REG_SLOW_DBG_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DBG_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is empty in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28027cUL: // (XSEM_REG_SLOW_DBG_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DBG_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FIFO is full in sem_slow_ls_dbg\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280280UL: // (XSEM_REG_SLOW_DRA_FIN_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DRA_FIN_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIN fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280284UL: // (XSEM_REG_SLOW_DRA_FIN_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DRA_FIN_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: FIN fifo is full in sem_slow_dra_sync (never may be active\n    )\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280288UL: // (XSEM_REG_SLOW_DRA_INT_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DRA_INT_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28028cUL: // (XSEM_REG_SLOW_DRA_INT_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DRA_INT_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: Interrupt fifo is full in sem_slow_dra_int\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280290UL: // (XSEM_REG_SLOW_DRA_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DRA_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_RD pop fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280294UL: // (XSEM_REG_SLOW_DRA_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DRA_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_RD pop fifo is full in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280298UL: // (XSEM_REG_SLOW_DRA_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DRA_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_WR push fifo is empty in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28029cUL: // (XSEM_REG_SLOW_DRA_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_DRA_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DRA_WR push fifo is full in sem_slow_dra_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802a0UL: // (XSEM_REG_SLOW_EXT_STORE_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_EXT_STORE_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802a4UL: // (XSEM_REG_SLOW_EXT_STORE_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_EXT_STORE_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_STORE FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802a8UL: // (XSEM_REG_SLOW_RAM0_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_RAM0_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM0 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802acUL: // (XSEM_REG_SLOW_RAM0_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_RAM0_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM0 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802b0UL: // (XSEM_REG_SLOW_RAM0_WR_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_RAM0_WR_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is almost full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802b4UL: // (XSEM_REG_SLOW_RAM0_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_RAM0_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM0 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802b8UL: // (XSEM_REG_SLOW_RAM0_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_RAM0_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM0 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802bcUL: // (XSEM_REG_SLOW_RAM1_RD_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_RAM1_RD_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM1 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802c0UL: // (XSEM_REG_SLOW_RAM1_RD_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_RAM1_RD_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_RD_RAM1 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802c4UL: // (XSEM_REG_SLOW_RAM1_WR_ALM_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_RAM1_WR_ALM_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is almost full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802c8UL: // (XSEM_REG_SLOW_RAM1_WR_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_RAM1_WR_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is empty in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802ccUL: // (XSEM_REG_SLOW_RAM1_WR_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SLOW_RAM1_WR_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: EXT_WR_RAM1 FIFO is full in sem_slow_ls_ext\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802d0UL: // (XSEM_REG_SYNC_DBG_EMPTY: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SYNC_DBG_EMPTY (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FAST SYNC FIFO is empty in sem_slow_ls_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802d4UL: // (XSEM_REG_SYNC_DBG_FULL: ACCESS:R  DataWidth:0x1 ) 
			sprintf(disp_str, "\nXSEM_REG_SYNC_DBG_FULL (ACCESS:R  DataWidth:0x1 )" \
				"\nDescription: DBG FAST SYNC FIFO is full in sem_slow_ls_sync\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802d8UL: // (XSEM_REG_THREAD_ERROR: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nXSEM_REG_THREAD_ERROR (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Thread error indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802dcUL: // (XSEM_REG_THREAD_OVERRUN_NUM: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nXSEM_REG_THREAD_OVERRUN_NUM (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Threads are sleeping in passive buffer more than ~xsem_registers_thread_inter_cnt\n    .thread_inter_cnt number of cycles\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802e0UL: // (XSEM_REG_THREAD_RDY: ACCESS:R  DataWidth:0x14 ) 
			sprintf(disp_str, "\nXSEM_REG_THREAD_RDY (ACCESS:R  DataWidth:0x14 )" \
				"\nDescription: Thread ready indication have bit per thread\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802e4UL: // (XSEM_REG_THREADS_LIST: ACCESS:RW  DataWidth:0x14 ) 
			sprintf(disp_str, "\nXSEM_REG_THREADS_LIST (ACCESS:RW  DataWidth:0x14 )" \
				"\nDescription: List of free threads . There is a bit per thread.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802e8UL: // (XSEM_REG_WB_MSB: ACCESS:R  DataWidth:0x2 ) 
			sprintf(disp_str, "\nXSEM_REG_WB_MSB (ACCESS:R  DataWidth:0x2 )" \
				"\nDescription: Reset value of this register is right when was not read to\n     ~xsem_registers_fic0_fifo.fic0_fifo or ~xsem_registers_fic1_fifo.fic1_fifo\n     or ~xsem_registers_passive_buffer.passive_buffer. For read from ~xsem_registers_passive_buffer\n    .passive_buffer :b0- parity0; b1 parity1. For  read from ~xsem_registers_fic0_fifo\n    .fic0_fifo and ~xsem_registers_fic1_fifo.fic1_fifo :b1=0 data from ~xsem_registers_fic0_fifo\n    .fic0_fifo and ~xsem_registers_fic1_fifo.fic1_fifo is valid; b1 =1 ~xsem_registers_fic0_fifo\n    .fic0_fifo and ~xsem_registers_fic1_fifo.fic1_fifo is empty and data from\n     it must be equal to 0; b0 - parity from ~xsem_registers_fic0_fifo.fic0_fifo\n     and ~xsem_registers_fic1_fifo.fic1_fifo\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280300UL: // (XSEM_REG_FIC0_FIFO: ACCESS:WB_R  DataWidth:0x80 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC0_FIFO (ACCESS:WB_R  DataWidth:0x80 )" \
				"\nDescription: Debug only. FIC0_fifo: b[127:0] data; b128-parity;b129=1- \n    fifo empty;b129=0-data is valid\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280320UL: // (XSEM_REG_FIC1_FIFO: ACCESS:WB_R  DataWidth:0x80 ) 
			sprintf(disp_str, "\nXSEM_REG_FIC1_FIFO (ACCESS:WB_R  DataWidth:0x80 )" \
				"\nDescription: Debug only. FIC1_fifo read for debugging mode; b[127:0] data\n    ; b128-parity;\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280340UL: // (XSEM_REG_FIN_COMMAND: ACCESS:WB_R  DataWidth:0x6d ) 
			sprintf(disp_str, "\nXSEM_REG_FIN_COMMAND (ACCESS:WB_R  DataWidth:0x6d )" \
				"\nDescription: last fin command that was read from fifo. Its spelling in\n     ~xsem_registers_fin_fifo.fin_fifo register\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280360UL: // (XSEM_REG_FIN_FIFO: ACCESS:WB_R  DataWidth:0x6d ) 
			sprintf(disp_str, "\nXSEM_REG_FIN_FIFO (ACCESS:WB_R  DataWidth:0x6d )" \
				"\nDescription: Debug only. FIn FIFO.  [5:0]   start_rp_foc3; [11:6]    start_rp_foc2\n    ;  [17:12]   start_rp_foc1; [23:18]   start_rp_foc0;  [29:24]   end_rp_foc3\n    ; [35:30]   end_rp_foc2; [41:36]   end_rp_foc1; [47:42]   end_rp_foc0; [\n    53:48]   lowest rp; [59:54]   highest rp; [65:60]   store start rp; [71\n    :66]   store end rp; [77:72]   load start rp; [83:78]   load end rp; [\n    85:84]   priority; [101:86]  pram address; [102] pas; [103] foc3; [104\n    ] foc2; [105] foc1; [106] foc0; [107] release; [108] fin; [109]=1- fin\n     fifo is empty; 0 - data[108:0] is valid\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280380UL: // (XSEM_REG_VFPF_ERR_NUM: ACCESS:W  DataWidth:0x7 ) 
			sprintf(disp_str, "\nXSEM_REG_VFPF_ERR_NUM (ACCESS:W  DataWidth:0x7 )" \
				"\nDescription: VF or PF ID for reset error bit. Values 0-63 reset error \n    bit for 64 VF; values 64-67 reset error for 4 PF; values 68-127 are not\n     valid.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280388UL: // (XSEM_REG_VF_ERR_VECTOR_LSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_VF_ERR_VECTOR_LSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: VF/PF error bitmap vector [0-31]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280390UL: // (XSEM_REG_VF_ERR_VECTOR_MSB: ACCESS:R  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_VF_ERR_VECTOR_MSB (ACCESS:R  DataWidth:0x20 )" \
				"\nDescription: VF/PF error bitmap vector [32-63]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280398UL: // (XSEM_REG_PF_ERR_VECTOR: ACCESS:R  DataWidth:0x8 ) 
			sprintf(disp_str, "\nXSEM_REG_PF_ERR_VECTOR (ACCESS:R  DataWidth:0x8 )" \
				"\nDescription: VF/PF error bitmap vector [0-7]\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2803a0UL: // (XSEM_REG_THREAD_SET_NUM: ACCESS:W  DataWidth:0x5 ) 
			sprintf(disp_str, "\nXSEM_REG_THREAD_SET_NUM (ACCESS:W  DataWidth:0x5 )" \
				"\nDescription: Thread ID. Write thread ID will set ready indication for \n    this thread ID\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280400UL: // (XSEM_REG_INT_TABLE: ACCESS:RW  DataWidth:0xf ) 
			sprintf(disp_str, "\nXSEM_REG_INT_TABLE (ACCESS:RW  DataWidth:0xf )" \
				"\nDescription: Interrupt table  Read and write access to it is not possible\n     in the middle of the work\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x282000UL: // (XSEM_REG_PASSIVE_BUFFER: ACCESS:WB  DataWidth:0x80 ) 
			sprintf(disp_str, "\nXSEM_REG_PASSIVE_BUFFER (ACCESS:WB  DataWidth:0x80 )" \
				"\nDescription: Debug only. Passive buffer memory\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x284000UL: // (XSEM_REG_PASSIVE_BUFFER_MSB: ACCESS:WB  DataWidth:0x80 ) 
			sprintf(disp_str, "\nXSEM_REG_PASSIVE_BUFFER_MSB (ACCESS:WB  DataWidth:0x80 )" \
				"\nDescription: Debug only. Passive buffer memory MSB that starts from row\n     512 of passive buffer till row 639\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2a0000UL: // (XSEM_REG_FAST_MEMORY: ACCESS:RW  DataWidth:0x20 ) 
			sprintf(disp_str, "\nXSEM_REG_FAST_MEMORY (ACCESS:RW  DataWidth:0x20 )" \
				"\nDescription: This address space contains all registers and memories that\n     are placed in SEM_FAST block. The SEM_FAST registers are described in\n     appendix B. In order to access the SEM_FAST registers the base address\n     XSEM_REGISTERS_FAST_MEMORY (Offset: 0x2a0000) should be added to each\n     SEM_FAST register offset.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2c0000UL: // (XSEM_REG_PRAM: ACCESS:WB  DataWidth:0x2e ) 
			sprintf(disp_str, "\nXSEM_REG_PRAM (ACCESS:WB  DataWidth:0x2e )" \
				"\nDescription: pram memory. B45 is parity; b[44:0] - data.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280144UL: // (XSEM_REG_XSEM_UNUSED_EMPTY_0: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_0 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280154UL: // (XSEM_REG_XSEM_UNUSED_EMPTY_1: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_1 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2802ecUL: // (XSEM_REG_XSEM_UNUSED_EMPTY_2: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_2 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280384UL: // (XSEM_REG_XSEM_UNUSED_EMPTY_3: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_3 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28038cUL: // (XSEM_REG_XSEM_UNUSED_EMPTY_4: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_4 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280394UL: // (XSEM_REG_XSEM_UNUSED_EMPTY_5: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_5 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x28039cUL: // (XSEM_REG_XSEM_UNUSED_EMPTY_6: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_6 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2803a4UL: // (XSEM_REG_XSEM_UNUSED_EMPTY_7: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_7 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x2803b0UL: // (XSEM_REG_XSEM_UNUSED_EMPTY_8: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_8 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x280800UL: // (XSEM_REG_XSEM_UNUSED_EMPTY_9: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_9 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x284800UL: // (XSEM_REG_XSEM_UNUSED_EMPTY_10: ACCESS:R  DataWidth:0x20  Unused empty space) 
			sprintf(disp_str, "\nXSEM_REG_XSEM_UNUSED_EMPTY_10 (ACCESS:R  DataWidth:0x20  Unused empty space)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x80000: // (MCP_REG_MCPR_UNUSED_A: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED_A (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x80080: // (MCP_REG_MCPR_MCP_CONTROL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MCP_CONTROL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MCP_CONTROL_MCP_ISOLATE :"
				, (u32_t)((data & MCPR_MCP_CONTROL_MCP_ISOLATE) >> 31)
				, "\n\tMCPR_MCP_CONTROL_SMBUS_SEL :"
				, (u32_t)((data & MCPR_MCP_CONTROL_SMBUS_SEL) >> 30));
			break;
		case 0x80084: // (MCP_REG_MCPR_MCP_ATTENTION_STATUS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MCP_ATTENTION_STATUS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MCP_ATTENTION_STATUS_CPU_EVENT :"
				, (u32_t)((data & MCPR_MCP_ATTENTION_STATUS_CPU_EVENT) >> 31)
				, "\n\tMCPR_MCP_ATTENTION_STATUS_WATCHDOG_TIMEOUT :"
				, (u32_t)((data & MCPR_MCP_ATTENTION_STATUS_WATCHDOG_TIMEOUT) >> 30)
				, "\n\tMCPR_MCP_ATTENTION_STATUS_DRV_DOORBELL :"
				, (u32_t)((data & MCPR_MCP_ATTENTION_STATUS_DRV_DOORBELL) >> 29));
			break;
		case 0x80088: // (MCP_REG_MCPR_MCP_HEARTBEAT_CONTROL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MCP_HEARTBEAT_CONTROL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MCP_HEARTBEAT_CONTROL_MCP_HEARTBEAT_ENABLE :"
				, (u32_t)((data & MCPR_MCP_HEARTBEAT_CONTROL_MCP_HEARTBEAT_ENABLE) >> 31));
			break;
		case 0x8008c: // (MCP_REG_MCPR_MCP_HEARTBEAT_STATUS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MCP_HEARTBEAT_STATUS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MCP_HEARTBEAT_STATUS_VALID :"
				, (u32_t)((data & MCPR_MCP_HEARTBEAT_STATUS_VALID) >> 31)
				, "\n\tMCPR_MCP_HEARTBEAT_STATUS_MCP_HEARTBEAT_PERIOD :"
				, (u32_t)((data & MCPR_MCP_HEARTBEAT_STATUS_MCP_HEARTBEAT_PERIOD) >> 0));
			break;
		case 0x80090: // (MCP_REG_MCPR_MCP_HEARTBEAT: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MCP_HEARTBEAT (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MCP_HEARTBEAT_MCP_HEARTBEAT_RESET :"
				, (u32_t)((data & MCPR_MCP_HEARTBEAT_MCP_HEARTBEAT_RESET) >> 31)
				, "\n\tMCPR_MCP_HEARTBEAT_MCP_HEARTBEAT_INC :"
				, (u32_t)((data & MCPR_MCP_HEARTBEAT_MCP_HEARTBEAT_INC) >> 30));
			break;
		case 0x80094: // (MCP_REG_MCPR_WATCHDOG_RESET: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_WATCHDOG_RESET (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_WATCHDOG_RESET_WATCHDOG_RESET :"
				, (u32_t)((data & MCPR_WATCHDOG_RESET_WATCHDOG_RESET) >> 31));
			break;
		case 0x80098: // (MCP_REG_MCPR_WATCHDOG_CONTROL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_WATCHDOG_CONTROL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_WATCHDOG_CONTROL_WATCHDOG_ENABLE :"
				, (u32_t)((data & MCPR_WATCHDOG_CONTROL_WATCHDOG_ENABLE) >> 31)
				, "\n\tMCPR_WATCHDOG_CONTROL_MCP_RST_ENABLE :"
				, (u32_t)((data & MCPR_WATCHDOG_CONTROL_MCP_RST_ENABLE) >> 30)
				, "\n\tMCPR_WATCHDOG_CONTROL_WATCHDOG_ATTN :"
				, (u32_t)((data & MCPR_WATCHDOG_CONTROL_WATCHDOG_ATTN) >> 29)
				, "\n\tMCPR_WATCHDOG_CONTROL_WATCHDOG_TIMEOUT :"
				, (u32_t)((data & MCPR_WATCHDOG_CONTROL_WATCHDOG_TIMEOUT) >> 0));
			break;
		case 0x8009c: // (MCP_REG_MCPR_ACCESS_LOCK: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ACCESS_LOCK (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ACCESS_LOCK_LOCK :"
				, (u32_t)((data & MCPR_ACCESS_LOCK_LOCK) >> 31));
			break;
		case 0x800a0: // (MCP_REG_MCPR_TOE_ID: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_TOE_ID (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_TOE_ID_FUNCTION_ID :"
				, (u32_t)((data & MCPR_TOE_ID_FUNCTION_ID) >> 31));
			break;
		case 0x800a4: // (MCP_REG_MCPR_MAILBOX_CFG: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MAILBOX_CFG (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MAILBOX_CFG_MAILBOX_OFFSET :"
				, (u32_t)((data & MCPR_MAILBOX_CFG_MAILBOX_OFFSET) >> 0));
			break;
		case 0x800a8: // (MCP_REG_MCPR_MAILBOX_CFG_OTHER_FUNC: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MAILBOX_CFG_OTHER_FUNC (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MAILBOX_CFG_OTHER_FUNC_MAILBOX_OFFSET :"
				, (u32_t)((data & MCPR_MAILBOX_CFG_OTHER_FUNC_MAILBOX_OFFSET) >> 0));
			break;
		case 0x800ac: // (MCP_REG_MCPR_MCP_DOORBELL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MCP_DOORBELL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MCP_DOORBELL_MCP_DOORBELL :"
				, (u32_t)((data & MCPR_MCP_DOORBELL_MCP_DOORBELL) >> 31));
			break;
		case 0x800b0: // (MCP_REG_MCPR_DRIVER_DOORBELL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_DRIVER_DOORBELL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_DRIVER_DOORBELL_DRIVER_DOORBELL :"
				, (u32_t)((data & MCPR_DRIVER_DOORBELL_DRIVER_DOORBELL) >> 31));
			break;
		case 0x800b4: // (MCP_REG_MCPR_DRIVER_DOORBELL_OTHER_FUNC: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_DRIVER_DOORBELL_OTHER_FUNC (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_DRIVER_DOORBELL_OTHER_FUNC_DRIVER_DOORBELL :"
				, (u32_t)((data & MCPR_DRIVER_DOORBELL_OTHER_FUNC_DRIVER_DOORBELL) >> 31));
			break;
		case 0x800b8: // (MCP_REG_MCPR_PORT4MODE_EN: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_PORT4MODE_EN (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_PORT4MODE_EN_FOUR_PORT_MODE :"
				, (u32_t)((data & MCPR_PORT4MODE_EN_FOUR_PORT_MODE) >> 0));
			break;
		case 0x800bc: // (MCP_REG_MCPR_MCP_VFID: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MCP_VFID (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MCP_VFID_PATH_FORCE :"
				, (u32_t)((data & MCPR_MCP_VFID_PATH_FORCE) >> 31)
				, "\n\tMCPR_MCP_VFID_PATHID :"
				, (u32_t)((data & MCPR_MCP_VFID_PATHID) >> 20)
				, "\n\tMCPR_MCP_VFID_VFID_VALID :"
				, (u32_t)((data & MCPR_MCP_VFID_VFID_VALID) >> 16)
				, "\n\tMCPR_MCP_VFID_VFID :"
				, (u32_t)((data & MCPR_MCP_VFID_VFID) >> 0));
			break;
		case 0x800c0: // (MCP_REG_MCPR_GP_INPUTS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_GP_INPUTS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x800c4: // (MCP_REG_MCPR_GP_OUTPUTS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_GP_OUTPUTS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x800c8: // (MCP_REG_MCPR_GP_OENABLE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_GP_OENABLE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x800cc: // (MCP_REG_MCPR_GP_MASK_HI_TO_LO: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_GP_MASK_HI_TO_LO (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x800d0: // (MCP_REG_MCPR_GP_MASK_LO_TO_HI: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_GP_MASK_LO_TO_HI (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x800d4: // (MCP_REG_MCPR_GP_HI_TO_LO: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_GP_HI_TO_LO (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x800d8: // (MCP_REG_MCPR_GP_LO_TO_HI: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_GP_LO_TO_HI (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x800dc: // (MCP_REG_MCPR_GP_EVENT_VEC: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_GP_EVENT_VEC (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x800e0: // (MCP_REG_MCPR_UNUSED_B: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED_B (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85000: // (MCP_REG_MCPR_CPU_MODE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_MODE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_CPU_MODE_SPAD_UNDERFLOW_HALT_ENA :"
				, (u32_t)((data & MCPR_CPU_MODE_SPAD_UNDERFLOW_HALT_ENA) >> 15)
				, "\n\tMCPR_CPU_MODE_FIO_ABORT_HALT_ENA :"
				, (u32_t)((data & MCPR_CPU_MODE_FIO_ABORT_HALT_ENA) >> 13)
				, "\n\tMCPR_CPU_MODE_BAD_INST_HALT_ENA :"
				, (u32_t)((data & MCPR_CPU_MODE_BAD_INST_HALT_ENA) >> 12)
				, "\n\tMCPR_CPU_MODE_BAD_DATA_HALT_ENA :"
				, (u32_t)((data & MCPR_CPU_MODE_BAD_DATA_HALT_ENA) >> 11)
				, "\n\tMCPR_CPU_MODE_SOFT_HALT :"
				, (u32_t)((data & MCPR_CPU_MODE_SOFT_HALT) >> 10)
				, "\n\tMCPR_CPU_MODE_INTERRUPT_ENA :"
				, (u32_t)((data & MCPR_CPU_MODE_INTERRUPT_ENA) >> 7)
				, "\n\tMCPR_CPU_MODE_MSG_BIT1 :"
				, (u32_t)((data & MCPR_CPU_MODE_MSG_BIT1) >> 6)
				, "\n\tMCPR_CPU_MODE_PAGE_0_INST_ENA :"
				, (u32_t)((data & MCPR_CPU_MODE_PAGE_0_INST_ENA) >> 3)
				, "\n\tMCPR_CPU_MODE_PAGE_0_DATA_ENA :"
				, (u32_t)((data & MCPR_CPU_MODE_PAGE_0_DATA_ENA) >> 2)
				, "\n\tMCPR_CPU_MODE_STEP_ENA :"
				, (u32_t)((data & MCPR_CPU_MODE_STEP_ENA) >> 1)
				, "\n\tMCPR_CPU_MODE_LOCAL_RST :"
				, (u32_t)((data & MCPR_CPU_MODE_LOCAL_RST) >> 0));
			break;
		case 0x85004: // (MCP_REG_MCPR_CPU_STATE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_STATE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_CPU_STATE_BLOCKED_READ :"
				, (u32_t)((data & MCPR_CPU_STATE_BLOCKED_READ) >> 31)
				, "\n\tMCPR_CPU_STATE_INST_FETCH_STALL :"
				, (u32_t)((data & MCPR_CPU_STATE_INST_FETCH_STALL) >> 15)
				, "\n\tMCPR_CPU_STATE_DATA_ACCESS_STALL :"
				, (u32_t)((data & MCPR_CPU_STATE_DATA_ACCESS_STALL) >> 14)
				, "\n\tMCPR_CPU_STATE_INTERRRUPT :"
				, (u32_t)((data & MCPR_CPU_STATE_INTERRRUPT) >> 12)
				, "\n\tMCPR_CPU_STATE_SPAD_UNDERFLOW :"
				, (u32_t)((data & MCPR_CPU_STATE_SPAD_UNDERFLOW) >> 11)
				, "\n\tMCPR_CPU_STATE_SOFT_HALTED :"
				, (u32_t)((data & MCPR_CPU_STATE_SOFT_HALTED) >> 10)
				, "\n\tMCPR_CPU_STATE_FIO_ABORT_HALTED :"
				, (u32_t)((data & MCPR_CPU_STATE_FIO_ABORT_HALTED) >> 8)
				, "\n\tMCPR_CPU_STATE_ALIGN_HALTED :"
				, (u32_t)((data & MCPR_CPU_STATE_ALIGN_HALTED) >> 7)
				, "\n\tMCPR_CPU_STATE_BAD_PC_HALTED :"
				, (u32_t)((data & MCPR_CPU_STATE_BAD_PC_HALTED) >> 6)
				, "\n\tMCPR_CPU_STATE_BAD_DATA_ADDR_HALTED :"
				, (u32_t)((data & MCPR_CPU_STATE_BAD_DATA_ADDR_HALTED) >> 5)
				, "\n\tMCPR_CPU_STATE_PAGE_0_INST_HALTED :"
				, (u32_t)((data & MCPR_CPU_STATE_PAGE_0_INST_HALTED) >> 4)
				, "\n\tMCPR_CPU_STATE_PAGE_0_DATA_HALTED :"
				, (u32_t)((data & MCPR_CPU_STATE_PAGE_0_DATA_HALTED) >> 3)
				, "\n\tMCPR_CPU_STATE_BAD_INST_HALTED :"
				, (u32_t)((data & MCPR_CPU_STATE_BAD_INST_HALTED) >> 2)
				, "\n\tMCPR_CPU_STATE_BREAKPOINT :"
				, (u32_t)((data & MCPR_CPU_STATE_BREAKPOINT) >> 0));
			break;
		case 0x85008: // (MCP_REG_MCPR_CPU_EVENT_MASK: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_EVENT_MASK (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_CPU_EVENT_MASK_INTERRUPT_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_INTERRUPT_MASK) >> 12)
				, "\n\tMCPR_CPU_EVENT_MASK_SPAD_UNDERFLOW_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_SPAD_UNDERFLOW_MASK) >> 11)
				, "\n\tMCPR_CPU_EVENT_MASK_SOFT_HALTED_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_SOFT_HALTED_MASK) >> 10)
				, "\n\tMCPR_CPU_EVENT_MASK_FIO_ABORT_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_FIO_ABORT_MASK) >> 8)
				, "\n\tMCPR_CPU_EVENT_MASK_ALIGN_HALTED_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_ALIGN_HALTED_MASK) >> 7)
				, "\n\tMCPR_CPU_EVENT_MASK_BAD_PC_HALTED_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_BAD_PC_HALTED_MASK) >> 6)
				, "\n\tMCPR_CPU_EVENT_MASK_BAD_DATA_ADDR_HALTED_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_BAD_DATA_ADDR_HALTED_MASK) >> 5)
				, "\n\tMCPR_CPU_EVENT_MASK_PAGE_0_INST_HALTED_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_PAGE_0_INST_HALTED_MASK) >> 4)
				, "\n\tMCPR_CPU_EVENT_MASK_PAGE_0_DATA_HALTED_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_PAGE_0_DATA_HALTED_MASK) >> 3)
				, "\n\tMCPR_CPU_EVENT_MASK_BAD_INST_HALTED_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_BAD_INST_HALTED_MASK) >> 2)
				, "\n\tMCPR_CPU_EVENT_MASK_BREAKPOINT_MASK :"
				, (u32_t)((data & MCPR_CPU_EVENT_MASK_BREAKPOINT_MASK) >> 0));
			break;
		case 0x8500c: // (MCP_REG_MCPR_CPU_UNUSED_B: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_UNUSED_B (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x8501c: // (MCP_REG_MCPR_CPU_PROGRAM_COUNTER: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_PROGRAM_COUNTER (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85020: // (MCP_REG_MCPR_CPU_INSTRUCTION: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_INSTRUCTION (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85024: // (MCP_REG_MCPR_CPU_DATA_ACCESS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_DATA_ACCESS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85028: // (MCP_REG_MCPR_CPU_INTERRUPT_ENABLE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_INTERRUPT_ENABLE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x8502c: // (MCP_REG_MCPR_CPU_INTERRUPT_VECTOR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_INTERRUPT_VECTOR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85030: // (MCP_REG_MCPR_CPU_INTERRUPT_SAVED_PC: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_INTERRUPT_SAVED_PC (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85034: // (MCP_REG_MCPR_CPU_HW_BREAKPOINT: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_HW_BREAKPOINT (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_CPU_HW_BREAKPOINT_ADDRESS :"
				, (u32_t)((data & MCPR_CPU_HW_BREAKPOINT_ADDRESS) >> 2)
				, "\n\tMCPR_CPU_HW_BREAKPOINT_DISABLE :"
				, (u32_t)((data & MCPR_CPU_HW_BREAKPOINT_DISABLE) >> 0));
			break;
		case 0x85038: // (MCP_REG_MCPR_CPU_DEBUG_VECT_PEEK: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_DEBUG_VECT_PEEK (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_CPU_DEBUG_VECT_PEEK_2_SEL :"
				, (u32_t)((data & MCPR_CPU_DEBUG_VECT_PEEK_2_SEL) >> 28)
				, "\n\tMCPR_CPU_DEBUG_VECT_PEEK_2_PEEK_EN :"
				, (u32_t)((data & MCPR_CPU_DEBUG_VECT_PEEK_2_PEEK_EN) >> 27)
				, "\n\tMCPR_CPU_DEBUG_VECT_PEEK_2_VALUE :"
				, (u32_t)((data & MCPR_CPU_DEBUG_VECT_PEEK_2_VALUE) >> 16)
				, "\n\tMCPR_CPU_DEBUG_VECT_PEEK_1_SEL :"
				, (u32_t)((data & MCPR_CPU_DEBUG_VECT_PEEK_1_SEL) >> 12)
				, "\n\tMCPR_CPU_DEBUG_VECT_PEEK_1_PEEK_EN :"
				, (u32_t)((data & MCPR_CPU_DEBUG_VECT_PEEK_1_PEEK_EN) >> 11)
				, "\n\tMCPR_CPU_DEBUG_VECT_PEEK_1_VALUE :"
				, (u32_t)((data & MCPR_CPU_DEBUG_VECT_PEEK_1_VALUE) >> 0));
			break;
		case 0x8503c: // (MCP_REG_MCPR_CPU_UNUSED_D: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_UNUSED_D (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85048: // (MCP_REG_MCPR_CPU_LAST_BRANCH_ADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_LAST_BRANCH_ADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_CPU_LAST_BRANCH_ADDR_LBA :"
				, (u32_t)((data & MCPR_CPU_LAST_BRANCH_ADDR_LBA) >> 2)
				, "\n\tMCPR_CPU_LAST_BRANCH_ADDR_TYPE :"
				, (u32_t)((data & MCPR_CPU_LAST_BRANCH_ADDR_TYPE) >> 1),
				(u32_t)(data & MCPR_CPU_LAST_BRANCH_ADDR_TYPE) == MCPR_CPU_LAST_BRANCH_ADDR_TYPE_BRANCH  ? "BRANCH" :
				(u32_t)(data & MCPR_CPU_LAST_BRANCH_ADDR_TYPE) == MCPR_CPU_LAST_BRANCH_ADDR_TYPE_JUMP  ? "JUMP" :"INVALID");
			break;
		case 0x8504c: // (MCP_REG_MCPR_CPU_UNUSED_E: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_UNUSED_E (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85200: // (MCP_REG_MCPR_CPU_REG_FILE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_CPU_REG_FILE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85280: // (MCP_REG_MCPR_UNUSED_C: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED_C (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85380: // (MCP_REG_MCPR_HMCPQ: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_HMCPQ (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85390: // (MCP_REG_MCPR_UNUSED1: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED1 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x853b8: // (MCP_REG_MCPR_HMCPQ_FTQ_CMD: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_HMCPQ_FTQ_CMD (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x (%s)%-20s 0x%x "
				, addr, data
				, "\n\tMCPR_HMCPQ_FTQ_CMD_BUSY :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CMD_BUSY) >> 31)
				, "\n\tMCPR_HMCPQ_FTQ_CMD_POP :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CMD_POP) >> 30)
				, "\n\tMCPR_HMCPQ_FTQ_CMD_INTERVENE_CLR :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CMD_INTERVENE_CLR) >> 29)
				, "\n\tMCPR_HMCPQ_FTQ_CMD_ADD_DATA :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CMD_ADD_DATA) >> 28)
				, "\n\tMCPR_HMCPQ_FTQ_CMD_ADD_INTERVEN :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CMD_ADD_INTERVEN) >> 27)
				, "\n\tMCPR_HMCPQ_FTQ_CMD_RD_DATA :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CMD_RD_DATA) >> 26)
				, "\n\tMCPR_HMCPQ_FTQ_CMD_SFT_RESET :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CMD_SFT_RESET) >> 25)
				, "\n\tMCPR_HMCPQ_FTQ_CMD_WR_TOP :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CMD_WR_TOP) >> 10),
				(u32_t)(data & MCPR_HMCPQ_FTQ_CMD_WR_TOP) == MCPR_HMCPQ_FTQ_CMD_WR_TOP_1  ? "1" :
				(u32_t)(data & MCPR_HMCPQ_FTQ_CMD_WR_TOP) == MCPR_HMCPQ_FTQ_CMD_WR_TOP_0  ? "0" :"INVALID"
				, "\n\tMCPR_HMCPQ_FTQ_CMD_OFFSET :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CMD_OFFSET) >> 0));
			break;
		case 0x853bc: // (MCP_REG_MCPR_HMCPQ_FTQ_CTL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_HMCPQ_FTQ_CTL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_HMCPQ_FTQ_CTL_CUR_DEPTH :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CTL_CUR_DEPTH) >> 22)
				, "\n\tMCPR_HMCPQ_FTQ_CTL_MAX_DEPTH :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CTL_MAX_DEPTH) >> 12)
				, "\n\tMCPR_HMCPQ_FTQ_CTL_FORCE_INTERVENE :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CTL_FORCE_INTERVENE) >> 2)
				, "\n\tMCPR_HMCPQ_FTQ_CTL_OVERFLOW :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CTL_OVERFLOW) >> 1)
				, "\n\tMCPR_HMCPQ_FTQ_CTL_INTERVENE :"
				, (u32_t)((data & MCPR_HMCPQ_FTQ_CTL_INTERVENE) >> 0));
			break;
		case 0x853c0: // (MCP_REG_MCPR_MCPQ: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MCPQ (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x853d0: // (MCP_REG_MCPR_UNUSED2: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED2 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x853f8: // (MCP_REG_MCPR_MCPQ_FTQ_CMD: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MCPQ_FTQ_CMD (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x (%s)%-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MCPQ_FTQ_CMD_BUSY :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CMD_BUSY) >> 31)
				, "\n\tMCPR_MCPQ_FTQ_CMD_POP :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CMD_POP) >> 30)
				, "\n\tMCPR_MCPQ_FTQ_CMD_INTERVENE_CLR :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CMD_INTERVENE_CLR) >> 29)
				, "\n\tMCPR_MCPQ_FTQ_CMD_ADD_DATA :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CMD_ADD_DATA) >> 28)
				, "\n\tMCPR_MCPQ_FTQ_CMD_ADD_INTERVEN :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CMD_ADD_INTERVEN) >> 27)
				, "\n\tMCPR_MCPQ_FTQ_CMD_RD_DATA :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CMD_RD_DATA) >> 26)
				, "\n\tMCPR_MCPQ_FTQ_CMD_SFT_RESET :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CMD_SFT_RESET) >> 25)
				, "\n\tMCPR_MCPQ_FTQ_CMD_WR_TOP :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CMD_WR_TOP) >> 10),
				(u32_t)(data & MCPR_MCPQ_FTQ_CMD_WR_TOP) == MCPR_MCPQ_FTQ_CMD_WR_TOP_1  ? "1" :
				(u32_t)(data & MCPR_MCPQ_FTQ_CMD_WR_TOP) == MCPR_MCPQ_FTQ_CMD_WR_TOP_0  ? "0" :"INVALID"
				, "\n\tMCPR_MCPQ_FTQ_CMD_OFFSET :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CMD_OFFSET) >> 0));
			break;
		case 0x853fc: // (MCP_REG_MCPR_MCPQ_FTQ_CTL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MCPQ_FTQ_CTL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MCPQ_FTQ_CTL_CUR_DEPTH :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CTL_CUR_DEPTH) >> 22)
				, "\n\tMCPR_MCPQ_FTQ_CTL_MAX_DEPTH :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CTL_MAX_DEPTH) >> 12)
				, "\n\tMCPR_MCPQ_FTQ_CTL_FORCE_INTERVENE :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CTL_FORCE_INTERVENE) >> 2)
				, "\n\tMCPR_MCPQ_FTQ_CTL_OVERFLOW :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CTL_OVERFLOW) >> 1)
				, "\n\tMCPR_MCPQ_FTQ_CTL_INTERVENE :"
				, (u32_t)((data & MCPR_MCPQ_FTQ_CTL_INTERVENE) >> 0));
			break;
		case 0x85400: // (MCP_REG_MCPR_UNUSED_D: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED_D (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x854a8: // (MCP_REG_MCPR_MDIO_AUTO_POLL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MDIO_AUTO_POLL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MDIO_AUTO_POLL_REG_ADDR :"
				, (u32_t)((data & MCPR_MDIO_AUTO_POLL_REG_ADDR) >> 16)
				, "\n\tMCPR_MDIO_AUTO_POLL_DATA_MASK :"
				, (u32_t)((data & MCPR_MDIO_AUTO_POLL_DATA_MASK) >> 0));
			break;
		case 0x854ac: // (MCP_REG_MCPR_MDIO_COMM: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MDIO_COMM (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x (%s)%-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MDIO_COMM_START_BUSY :"
				, (u32_t)((data & MCPR_MDIO_COMM_START_BUSY) >> 29)
				, "\n\tMCPR_MDIO_COMM_FAIL :"
				, (u32_t)((data & MCPR_MDIO_COMM_FAIL) >> 28)
				, "\n\tMCPR_MDIO_COMM_COMMAND :"
				, (u32_t)((data & MCPR_MDIO_COMM_COMMAND) >> 26),
				(u32_t)(data & MCPR_MDIO_COMM_COMMAND) == MCPR_MDIO_COMM_COMMAND_READ_45  ? "READ_45" :
				(u32_t)(data & MCPR_MDIO_COMM_COMMAND) == MCPR_MDIO_COMM_COMMAND_UNDEFINED_3  ? "UNDEFINED_3" :
				(u32_t)(data & MCPR_MDIO_COMM_COMMAND) == MCPR_MDIO_COMM_COMMAND_READ_INC_45  ? "READ_INC_45" :
				(u32_t)(data & MCPR_MDIO_COMM_COMMAND) == MCPR_MDIO_COMM_COMMAND_READ_22  ? "READ_22" :
				(u32_t)(data & MCPR_MDIO_COMM_COMMAND) == MCPR_MDIO_COMM_COMMAND_WRITE_45  ? "WRITE_45" :
				(u32_t)(data & MCPR_MDIO_COMM_COMMAND) == MCPR_MDIO_COMM_COMMAND_WRITE_22  ? "WRITE_22" :
				(u32_t)(data & MCPR_MDIO_COMM_COMMAND) == MCPR_MDIO_COMM_COMMAND_ADDRESS  ? "ADDRESS" :
				(u32_t)(data & MCPR_MDIO_COMM_COMMAND) == MCPR_MDIO_COMM_COMMAND_UNDEFINED_0  ? "UNDEFINED_0" :"INVALID"
				, "\n\tMCPR_MDIO_COMM_PHY_ADDR :"
				, (u32_t)((data & MCPR_MDIO_COMM_PHY_ADDR) >> 21)
				, "\n\tMCPR_MDIO_COMM_REG_ADDR :"
				, (u32_t)((data & MCPR_MDIO_COMM_REG_ADDR) >> 16)
				, "\n\tMCPR_MDIO_COMM_DATA :"
				, (u32_t)((data & MCPR_MDIO_COMM_DATA) >> 0));
			break;
		case 0x854b0: // (MCP_REG_MCPR_MDIO_STATUS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MDIO_STATUS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MDIO_STATUS_10MB :"
				, (u32_t)((data & MCPR_MDIO_STATUS_10MB) >> 1)
				, "\n\tMCPR_MDIO_STATUS_LINK :"
				, (u32_t)((data & MCPR_MDIO_STATUS_LINK) >> 0));
			break;
		case 0x854b4: // (MCP_REG_MCPR_MDIO_MODE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MDIO_MODE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MDIO_MODE_CLAUSE_45 :"
				, (u32_t)((data & MCPR_MDIO_MODE_CLAUSE_45) >> 31)
				, "\n\tMCPR_MDIO_MODE_CLOCK_CNT :"
				, (u32_t)((data & MCPR_MDIO_MODE_CLOCK_CNT) >> 16)
				, "\n\tMCPR_MDIO_MODE_EXT_MDINT :"
				, (u32_t)((data & MCPR_MDIO_MODE_EXT_MDINT) >> 13)
				, "\n\tMCPR_MDIO_MODE_MDINT :"
				, (u32_t)((data & MCPR_MDIO_MODE_MDINT) >> 12)
				, "\n\tMCPR_MDIO_MODE_MDC :"
				, (u32_t)((data & MCPR_MDIO_MODE_MDC) >> 11)
				, "\n\tMCPR_MDIO_MODE_MDIO_OE :"
				, (u32_t)((data & MCPR_MDIO_MODE_MDIO_OE) >> 10)
				, "\n\tMCPR_MDIO_MODE_MDIO :"
				, (u32_t)((data & MCPR_MDIO_MODE_MDIO) >> 9)
				, "\n\tMCPR_MDIO_MODE_BIT_BANG :"
				, (u32_t)((data & MCPR_MDIO_MODE_BIT_BANG) >> 8)
				, "\n\tMCPR_MDIO_MODE_AUTO_POLL :"
				, (u32_t)((data & MCPR_MDIO_MODE_AUTO_POLL) >> 4)
				, "\n\tMCPR_MDIO_MODE_SHORT_PREAMBLE :"
				, (u32_t)((data & MCPR_MDIO_MODE_SHORT_PREAMBLE) >> 1));
			break;
		case 0x854b8: // (MCP_REG_MCPR_MDIO_AUTO_STATUS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_MDIO_AUTO_STATUS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_MDIO_AUTO_STATUS_AUTO_ERR :"
				, (u32_t)((data & MCPR_MDIO_AUTO_STATUS_AUTO_ERR) >> 0));
			break;
		case 0x854bc: // (MCP_REG_MCPR_UNUSED_F: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED_F (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85800: // (MCP_REG_MCPR_UCINT_WARP_MODE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UCINT_WARP_MODE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x (%s)%-20s 0x%x %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_UCINT_WARP_MODE_DUMMY_CYCLES :"
				, (u32_t)((data & MCPR_UCINT_WARP_MODE_DUMMY_CYCLES) >> 16)
				, "\n\tMCPR_UCINT_WARP_MODE_BYTE_SWAP :"
				, (u32_t)((data & MCPR_UCINT_WARP_MODE_BYTE_SWAP) >> 8),
				(u32_t)(data & MCPR_UCINT_WARP_MODE_BYTE_SWAP) == MCPR_UCINT_WARP_MODE_BYTE_SWAP_BIG_ENDIAN  ? "BIG_ENDIAN" :
				(u32_t)(data & MCPR_UCINT_WARP_MODE_BYTE_SWAP) == MCPR_UCINT_WARP_MODE_BYTE_SWAP_LITTLE_ENDIAN  ? "LITTLE_ENDIAN" :"INVALID"
				, "\n\tMCPR_UCINT_WARP_MODE_TARGET :"
				, (u32_t)((data & MCPR_UCINT_WARP_MODE_TARGET) >> 4)
				, "\n\tMCPR_UCINT_WARP_MODE_ACCESS_MODE :"
				, (u32_t)((data & MCPR_UCINT_WARP_MODE_ACCESS_MODE) >> 0),
				(u32_t)(data & MCPR_UCINT_WARP_MODE_ACCESS_MODE) == MCPR_UCINT_WARP_MODE_ACCESS_MODE_SPECIFIC_READ  ? "SPECIFIC_READ" :
				(u32_t)(data & MCPR_UCINT_WARP_MODE_ACCESS_MODE) == MCPR_UCINT_WARP_MODE_ACCESS_MODE_SPECIFIC_WRITE  ? "SPECIFIC_WRITE" :
				(u32_t)(data & MCPR_UCINT_WARP_MODE_ACCESS_MODE) == MCPR_UCINT_WARP_MODE_ACCESS_MODE_BROADCAST_WRITE  ? "BROADCAST_WRITE" :
				(u32_t)(data & MCPR_UCINT_WARP_MODE_ACCESS_MODE) == MCPR_UCINT_WARP_MODE_ACCESS_MODE_MDIO  ? "MDIO" :"INVALID");
			break;
		case 0x85804: // (MCP_REG_MCPR_UCINT_WARP_CLK_DIV: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UCINT_WARP_CLK_DIV (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_UCINT_WARP_CLK_DIV_CLOCK_RATE :"
				, (u32_t)((data & MCPR_UCINT_WARP_CLK_DIV_CLOCK_RATE) >> 0),
				(u32_t)(data & MCPR_UCINT_WARP_CLK_DIV_CLOCK_RATE) == MCPR_UCINT_WARP_CLK_DIV_CLOCK_RATE_DIVIDE_BY_16  ? "DIVIDE_BY_16" :
				(u32_t)(data & MCPR_UCINT_WARP_CLK_DIV_CLOCK_RATE) == MCPR_UCINT_WARP_CLK_DIV_CLOCK_RATE_DIVIDE_BY_8  ? "DIVIDE_BY_8" :
				(u32_t)(data & MCPR_UCINT_WARP_CLK_DIV_CLOCK_RATE) == MCPR_UCINT_WARP_CLK_DIV_CLOCK_RATE_DIVIDE_BY_4  ? "DIVIDE_BY_4" :"INVALID");
			break;
		case 0x85808: // (MCP_REG_MCPR_UCINT_WARP_ADDRESS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UCINT_WARP_ADDRESS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UCINT_WARP_ADDRESS_ADDRESS :"
				, (u32_t)((data & MCPR_UCINT_WARP_ADDRESS_ADDRESS) >> 0));
			break;
		case 0x8580c: // (MCP_REG_MCPR_UCINT_WARP_DATA: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UCINT_WARP_DATA (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85810: // (MCP_REG_MCPR_UCINT_WARP_TARGET_ENABLE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UCINT_WARP_TARGET_ENABLE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UCINT_WARP_TARGET_ENABLE_UC_ENABLE2 :"
				, (u32_t)((data & MCPR_UCINT_WARP_TARGET_ENABLE_UC_ENABLE2) >> 18)
				, "\n\tMCPR_UCINT_WARP_TARGET_ENABLE_UC_ENABLE1 :"
				, (u32_t)((data & MCPR_UCINT_WARP_TARGET_ENABLE_UC_ENABLE1) >> 17)
				, "\n\tMCPR_UCINT_WARP_TARGET_ENABLE_UC_ENABLE0 :"
				, (u32_t)((data & MCPR_UCINT_WARP_TARGET_ENABLE_UC_ENABLE0) >> 16)
				, "\n\tMCPR_UCINT_WARP_TARGET_ENABLE_EN_CLR2 :"
				, (u32_t)((data & MCPR_UCINT_WARP_TARGET_ENABLE_EN_CLR2) >> 10)
				, "\n\tMCPR_UCINT_WARP_TARGET_ENABLE_EN_CLR1 :"
				, (u32_t)((data & MCPR_UCINT_WARP_TARGET_ENABLE_EN_CLR1) >> 9)
				, "\n\tMCPR_UCINT_WARP_TARGET_ENABLE_EN_CLR0 :"
				, (u32_t)((data & MCPR_UCINT_WARP_TARGET_ENABLE_EN_CLR0) >> 8)
				, "\n\tMCPR_UCINT_WARP_TARGET_ENABLE_EN_SET2 :"
				, (u32_t)((data & MCPR_UCINT_WARP_TARGET_ENABLE_EN_SET2) >> 2)
				, "\n\tMCPR_UCINT_WARP_TARGET_ENABLE_EN_SET1 :"
				, (u32_t)((data & MCPR_UCINT_WARP_TARGET_ENABLE_EN_SET1) >> 1)
				, "\n\tMCPR_UCINT_WARP_TARGET_ENABLE_EN_SET0 :"
				, (u32_t)((data & MCPR_UCINT_WARP_TARGET_ENABLE_EN_SET0) >> 0));
			break;
		case 0x85814: // (MCP_REG_MCPR_UCINT_PCIE_MODE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UCINT_PCIE_MODE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x (%s)%-20s 0x%x %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_UCINT_PCIE_MODE_DUMMY_CYCLES :"
				, (u32_t)((data & MCPR_UCINT_PCIE_MODE_DUMMY_CYCLES) >> 16)
				, "\n\tMCPR_UCINT_PCIE_MODE_BYTE_SWAP :"
				, (u32_t)((data & MCPR_UCINT_PCIE_MODE_BYTE_SWAP) >> 8),
				(u32_t)(data & MCPR_UCINT_PCIE_MODE_BYTE_SWAP) == MCPR_UCINT_PCIE_MODE_BYTE_SWAP_BIG_ENDIAN  ? "BIG_ENDIAN" :
				(u32_t)(data & MCPR_UCINT_PCIE_MODE_BYTE_SWAP) == MCPR_UCINT_PCIE_MODE_BYTE_SWAP_LITTLE_ENDIAN  ? "LITTLE_ENDIAN" :"INVALID"
				, "\n\tMCPR_UCINT_PCIE_MODE_TARGET :"
				, (u32_t)((data & MCPR_UCINT_PCIE_MODE_TARGET) >> 4)
				, "\n\tMCPR_UCINT_PCIE_MODE_ACCESS_MODE :"
				, (u32_t)((data & MCPR_UCINT_PCIE_MODE_ACCESS_MODE) >> 0),
				(u32_t)(data & MCPR_UCINT_PCIE_MODE_ACCESS_MODE) == MCPR_UCINT_PCIE_MODE_ACCESS_MODE_SPECIFIC_READ  ? "SPECIFIC_READ" :
				(u32_t)(data & MCPR_UCINT_PCIE_MODE_ACCESS_MODE) == MCPR_UCINT_PCIE_MODE_ACCESS_MODE_SPECIFIC_WRITE  ? "SPECIFIC_WRITE" :
				(u32_t)(data & MCPR_UCINT_PCIE_MODE_ACCESS_MODE) == MCPR_UCINT_PCIE_MODE_ACCESS_MODE_BROADCAST_WRITE  ? "BROADCAST_WRITE" :
				(u32_t)(data & MCPR_UCINT_PCIE_MODE_ACCESS_MODE) == MCPR_UCINT_PCIE_MODE_ACCESS_MODE_MDIO  ? "MDIO" :"INVALID");
			break;
		case 0x85818: // (MCP_REG_MCPR_UCINT_PCIE_CLK_DIV: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UCINT_PCIE_CLK_DIV (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_UCINT_PCIE_CLK_DIV_CLOCK_RATE :"
				, (u32_t)((data & MCPR_UCINT_PCIE_CLK_DIV_CLOCK_RATE) >> 0),
				(u32_t)(data & MCPR_UCINT_PCIE_CLK_DIV_CLOCK_RATE) == MCPR_UCINT_PCIE_CLK_DIV_CLOCK_RATE_DIVIDE_BY_16  ? "DIVIDE_BY_16" :
				(u32_t)(data & MCPR_UCINT_PCIE_CLK_DIV_CLOCK_RATE) == MCPR_UCINT_PCIE_CLK_DIV_CLOCK_RATE_DIVIDE_BY_8  ? "DIVIDE_BY_8" :
				(u32_t)(data & MCPR_UCINT_PCIE_CLK_DIV_CLOCK_RATE) == MCPR_UCINT_PCIE_CLK_DIV_CLOCK_RATE_DIVIDE_BY_4  ? "DIVIDE_BY_4" :"INVALID");
			break;
		case 0x8581c: // (MCP_REG_MCPR_UCINT_PCIE_ADDRESS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UCINT_PCIE_ADDRESS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UCINT_PCIE_ADDRESS_ADDRESS :"
				, (u32_t)((data & MCPR_UCINT_PCIE_ADDRESS_ADDRESS) >> 0));
			break;
		case 0x85820: // (MCP_REG_MCPR_UCINT_PCIE_DATA: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UCINT_PCIE_DATA (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85824: // (MCP_REG_MCPR_UCINT_PCIE_TARGET_ENABLE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UCINT_PCIE_TARGET_ENABLE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UCINT_PCIE_TARGET_ENABLE_UC_ENABLE1 :"
				, (u32_t)((data & MCPR_UCINT_PCIE_TARGET_ENABLE_UC_ENABLE1) >> 17)
				, "\n\tMCPR_UCINT_PCIE_TARGET_ENABLE_UC_ENABLE0 :"
				, (u32_t)((data & MCPR_UCINT_PCIE_TARGET_ENABLE_UC_ENABLE0) >> 16)
				, "\n\tMCPR_UCINT_PCIE_TARGET_ENABLE_EN_CLR1 :"
				, (u32_t)((data & MCPR_UCINT_PCIE_TARGET_ENABLE_EN_CLR1) >> 9)
				, "\n\tMCPR_UCINT_PCIE_TARGET_ENABLE_EN_CLR0 :"
				, (u32_t)((data & MCPR_UCINT_PCIE_TARGET_ENABLE_EN_CLR0) >> 8)
				, "\n\tMCPR_UCINT_PCIE_TARGET_ENABLE_EN_SET1 :"
				, (u32_t)((data & MCPR_UCINT_PCIE_TARGET_ENABLE_EN_SET1) >> 1)
				, "\n\tMCPR_UCINT_PCIE_TARGET_ENABLE_EN_SET0 :"
				, (u32_t)((data & MCPR_UCINT_PCIE_TARGET_ENABLE_EN_SET0) >> 0));
			break;
		case 0x85828: // (MCP_REG_MCPR_UNUSED_G: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED_G (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85900: // (MCP_REG_MCPR_IMC_COMMAND: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_COMMAND (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_IMC_COMMAND_ENABLE :"
				, (u32_t)((data & MCPR_IMC_COMMAND_ENABLE) >> 31)
				, "\n\tMCPR_IMC_COMMAND_OPERATION :"
				, (u32_t)((data & MCPR_IMC_COMMAND_OPERATION) >> 28)
				, "\n\tMCPR_IMC_COMMAND_IMC_STATUS :"
				, (u32_t)((data & MCPR_IMC_COMMAND_IMC_STATUS) >> 16)
				, "\n\tMCPR_IMC_COMMAND_TRANSFER_ADDRESS :"
				, (u32_t)((data & MCPR_IMC_COMMAND_TRANSFER_ADDRESS) >> 8));
			break;
		case 0x85904: // (MCP_REG_MCPR_IMC_SLAVE_CONTROL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_SLAVE_CONTROL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_IMC_SLAVE_CONTROL_SLAVE_DEVICE_ID :"
				, (u32_t)((data & MCPR_IMC_SLAVE_CONTROL_SLAVE_DEVICE_ID) >> 17)
				, "\n\tMCPR_IMC_SLAVE_CONTROL_SLAVE_ADDRESS :"
				, (u32_t)((data & MCPR_IMC_SLAVE_CONTROL_SLAVE_ADDRESS) >> 0));
			break;
		case 0x85908: // (MCP_REG_MCPR_IMC_RESERVED0: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_RESERVED0 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x8590c: // (MCP_REG_MCPR_IMC_RESERVED1: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_RESERVED1 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85910: // (MCP_REG_MCPR_IMC_RESERVED2: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_RESERVED2 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85914: // (MCP_REG_MCPR_IMC_TIMING0: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_TIMING0 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_IMC_TIMING0_SCL_HIGH_PERIOD :"
				, (u32_t)((data & MCPR_IMC_TIMING0_SCL_HIGH_PERIOD) >> 16)
				, "\n\tMCPR_IMC_TIMING0_SCL_LOW_PERIOD :"
				, (u32_t)((data & MCPR_IMC_TIMING0_SCL_LOW_PERIOD) >> 0));
			break;
		case 0x85918: // (MCP_REG_MCPR_IMC_TIMING1: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_TIMING1 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_IMC_TIMING1_DATA_HOLD_TIME :"
				, (u32_t)((data & MCPR_IMC_TIMING1_DATA_HOLD_TIME) >> 16)
				, "\n\tMCPR_IMC_TIMING1_START_TO_SCL_LOW :"
				, (u32_t)((data & MCPR_IMC_TIMING1_START_TO_SCL_LOW) >> 0));
			break;
		case 0x8591c: // (MCP_REG_MCPR_IMC_TIMING2: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_TIMING2 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_IMC_TIMING2_STOP_TO_START :"
				, (u32_t)((data & MCPR_IMC_TIMING2_STOP_TO_START) >> 16)
				, "\n\tMCPR_IMC_TIMING2_STOP_TO_SDA_HIGH :"
				, (u32_t)((data & MCPR_IMC_TIMING2_STOP_TO_SDA_HIGH) >> 0));
			break;
		case 0x85920: // (MCP_REG_MCPR_IMC_DATAREG0: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_DATAREG0 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_IMC_DATAREG0_DATA :"
				, (u32_t)((data & MCPR_IMC_DATAREG0_DATA) >> 0));
			break;
		case 0x85924: // (MCP_REG_MCPR_IMC_DATAREG1: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_DATAREG1 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_IMC_DATAREG1_DATA :"
				, (u32_t)((data & MCPR_IMC_DATAREG1_DATA) >> 0));
			break;
		case 0x85928: // (MCP_REG_MCPR_IMC_DATAREG2: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_DATAREG2 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_IMC_DATAREG2_DATA :"
				, (u32_t)((data & MCPR_IMC_DATAREG2_DATA) >> 0));
			break;
		case 0x8592c: // (MCP_REG_MCPR_IMC_DATAREG3: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_DATAREG3 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_IMC_DATAREG3_DATA :"
				, (u32_t)((data & MCPR_IMC_DATAREG3_DATA) >> 0));
			break;
		case 0x85930: // (MCP_REG_MCPR_IMC_UNUSED: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_IMC_UNUSED (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x85940: // (MCP_REG_MCPR_UNUSED_H: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED_H (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x86400: // (MCP_REG_MCPR_NVM_COMMAND: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_COMMAND (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_NVM_COMMAND_MODE_256 :"
				, (u32_t)((data & MCPR_NVM_COMMAND_MODE_256) >> 22)
				, "\n\tMCPR_NVM_COMMAND_RD_STATUS :"
				, (u32_t)((data & MCPR_NVM_COMMAND_RD_STATUS) >> 21)
				, "\n\tMCPR_NVM_COMMAND_RD_ID :"
				, (u32_t)((data & MCPR_NVM_COMMAND_RD_ID) >> 20)
				, "\n\tMCPR_NVM_COMMAND_WRDI :"
				, (u32_t)((data & MCPR_NVM_COMMAND_WRDI) >> 17)
				, "\n\tMCPR_NVM_COMMAND_WREN :"
				, (u32_t)((data & MCPR_NVM_COMMAND_WREN) >> 16)
				, "\n\tMCPR_NVM_COMMAND_LAST :"
				, (u32_t)((data & MCPR_NVM_COMMAND_LAST) >> 8)
				, "\n\tMCPR_NVM_COMMAND_FIRST :"
				, (u32_t)((data & MCPR_NVM_COMMAND_FIRST) >> 7)
				, "\n\tMCPR_NVM_COMMAND_ERASE :"
				, (u32_t)((data & MCPR_NVM_COMMAND_ERASE) >> 6)
				, "\n\tMCPR_NVM_COMMAND_WR :"
				, (u32_t)((data & MCPR_NVM_COMMAND_WR) >> 5)
				, "\n\tMCPR_NVM_COMMAND_DOIT :"
				, (u32_t)((data & MCPR_NVM_COMMAND_DOIT) >> 4)
				, "\n\tMCPR_NVM_COMMAND_DONE :"
				, (u32_t)((data & MCPR_NVM_COMMAND_DONE) >> 3)
				, "\n\tMCPR_NVM_COMMAND_RST :"
				, (u32_t)((data & MCPR_NVM_COMMAND_RST) >> 0));
			break;
		case 0x86404: // (MCP_REG_MCPR_NVM_STATUS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_STATUS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_NVM_STATUS_SPI_FSM_STATE :"
				, (u32_t)((data & MCPR_NVM_STATUS_SPI_FSM_STATE) >> 0),
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_WAIT  ? "SPI_WAIT" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_ST_WREN  ? "SPI_ST_WREN" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_CHECK_BUSY0  ? "SPI_CHECK_BUSY0" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_READ_STATUS_RDID4  ? "SPI_READ_STATUS_RDID4" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_READ_STATUS_RDID3  ? "SPI_READ_STATUS_RDID3" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_READ_STATUS_RDID2  ? "SPI_READ_STATUS_RDID2" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_READ_STATUS_RDID1  ? "SPI_READ_STATUS_RDID1" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_READ_STATUS_RDID0  ? "SPI_READ_STATUS_RDID0" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_READ_DATA2  ? "SPI_READ_DATA2" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_READ_DATA1  ? "SPI_READ_DATA1" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_READ_DATA0  ? "SPI_READ_DATA0" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_WRITE_DATA2  ? "SPI_WRITE_DATA2" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_WRITE_DATA1  ? "SPI_WRITE_DATA1" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_WRITE_DATA0  ? "SPI_WRITE_DATA0" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_ADDR0  ? "SPI_ADDR0" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_CMD_FINISH1  ? "SPI_CMD_FINISH1" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_CMD_FINISH0  ? "SPI_CMD_FINISH0" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_CMD1  ? "SPI_CMD1" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_CMD0  ? "SPI_CMD0" :
				(u32_t)(data & MCPR_NVM_STATUS_SPI_FSM_STATE) == MCPR_NVM_STATUS_SPI_FSM_STATE_SPI_IDLE  ? "SPI_IDLE" :"INVALID");
			break;
		case 0x86408: // (MCP_REG_MCPR_NVM_WRITE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_WRITE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_NVM_WRITE_NVM_WRITE_VALUE :"
				, (u32_t)((data & MCPR_NVM_WRITE_NVM_WRITE_VALUE) >> 0),
				(u32_t)(data & MCPR_NVM_WRITE_NVM_WRITE_VALUE) == MCPR_NVM_WRITE_NVM_WRITE_VALUE_SCLK  ? "SCLK" :
				(u32_t)(data & MCPR_NVM_WRITE_NVM_WRITE_VALUE) == MCPR_NVM_WRITE_NVM_WRITE_VALUE_CS_B  ? "CS_B" :
				(u32_t)(data & MCPR_NVM_WRITE_NVM_WRITE_VALUE) == MCPR_NVM_WRITE_NVM_WRITE_VALUE_SO  ? "SO" :
				(u32_t)(data & MCPR_NVM_WRITE_NVM_WRITE_VALUE) == MCPR_NVM_WRITE_NVM_WRITE_VALUE_SI  ? "SI" :
				(u32_t)(data & MCPR_NVM_WRITE_NVM_WRITE_VALUE) == MCPR_NVM_WRITE_NVM_WRITE_VALUE_BIT_BANG  ? "BIT_BANG" :"INVALID");
			break;
		case 0x8640c: // (MCP_REG_MCPR_NVM_ADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_ADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_NVM_ADDR_NVM_ADDR_VALUE :"
				, (u32_t)((data & MCPR_NVM_ADDR_NVM_ADDR_VALUE) >> 0),
				(u32_t)(data & MCPR_NVM_ADDR_NVM_ADDR_VALUE) == MCPR_NVM_ADDR_NVM_ADDR_VALUE_SCLK  ? "SCLK" :
				(u32_t)(data & MCPR_NVM_ADDR_NVM_ADDR_VALUE) == MCPR_NVM_ADDR_NVM_ADDR_VALUE_CS_B  ? "CS_B" :
				(u32_t)(data & MCPR_NVM_ADDR_NVM_ADDR_VALUE) == MCPR_NVM_ADDR_NVM_ADDR_VALUE_SO  ? "SO" :
				(u32_t)(data & MCPR_NVM_ADDR_NVM_ADDR_VALUE) == MCPR_NVM_ADDR_NVM_ADDR_VALUE_SI  ? "SI" :
				(u32_t)(data & MCPR_NVM_ADDR_NVM_ADDR_VALUE) == MCPR_NVM_ADDR_NVM_ADDR_VALUE_BIT_BANG  ? "BIT_BANG" :"INVALID");
			break;
		case 0x86410: // (MCP_REG_MCPR_NVM_READ: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_READ (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_NVM_READ_NVM_READ_VALUE :"
				, (u32_t)((data & MCPR_NVM_READ_NVM_READ_VALUE) >> 0),
				(u32_t)(data & MCPR_NVM_READ_NVM_READ_VALUE) == MCPR_NVM_READ_NVM_READ_VALUE_SCLK  ? "SCLK" :
				(u32_t)(data & MCPR_NVM_READ_NVM_READ_VALUE) == MCPR_NVM_READ_NVM_READ_VALUE_CS_B  ? "CS_B" :
				(u32_t)(data & MCPR_NVM_READ_NVM_READ_VALUE) == MCPR_NVM_READ_NVM_READ_VALUE_SO  ? "SO" :
				(u32_t)(data & MCPR_NVM_READ_NVM_READ_VALUE) == MCPR_NVM_READ_NVM_READ_VALUE_SI  ? "SI" :
				(u32_t)(data & MCPR_NVM_READ_NVM_READ_VALUE) == MCPR_NVM_READ_NVM_READ_VALUE_BIT_BANG  ? "BIT_BANG" :"INVALID");
			break;
		case 0x86414: // (MCP_REG_MCPR_NVM_CFG1: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_CFG1 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_NVM_CFG1_COMPAT_BYPASSS :"
				, (u32_t)((data & MCPR_NVM_CFG1_COMPAT_BYPASSS) >> 31)
				, "\n\tMCPR_NVM_CFG1_FW_FLASH_TYPE_EN :"
				, (u32_t)((data & MCPR_NVM_CFG1_FW_FLASH_TYPE_EN) >> 30)
				, "\n\tMCPR_NVM_CFG1_FW_USTRAP_3 :"
				, (u32_t)((data & MCPR_NVM_CFG1_FW_USTRAP_3) >> 29)
				, "\n\tMCPR_NVM_CFG1_FW_USTRAP_2 :"
				, (u32_t)((data & MCPR_NVM_CFG1_FW_USTRAP_2) >> 28)
				, "\n\tMCPR_NVM_CFG1_FW_USTRAP_0 :"
				, (u32_t)((data & MCPR_NVM_CFG1_FW_USTRAP_0) >> 27)
				, "\n\tMCPR_NVM_CFG1_FW_USTRAP_1 :"
				, (u32_t)((data & MCPR_NVM_CFG1_FW_USTRAP_1) >> 26)
				, "\n\tMCPR_NVM_CFG1_PROTECT_MODE :"
				, (u32_t)((data & MCPR_NVM_CFG1_PROTECT_MODE) >> 24)
				, "\n\tMCPR_NVM_CFG1_STRAP_CONTROL_0 :"
				, (u32_t)((data & MCPR_NVM_CFG1_STRAP_CONTROL_0) >> 23)
				, "\n\tMCPR_NVM_CFG1_SEE_CLK_DIV :"
				, (u32_t)((data & MCPR_NVM_CFG1_SEE_CLK_DIV) >> 11)
				, "\n\tMCPR_NVM_CFG1_SPI_CLK_DIV :"
				, (u32_t)((data & MCPR_NVM_CFG1_SPI_CLK_DIV) >> 7)
				, "\n\tMCPR_NVM_CFG1_STATUS_BIT :"
				, (u32_t)((data & MCPR_NVM_CFG1_STATUS_BIT) >> 4)
				, "\n\tMCPR_NVM_CFG1_BITBANG_MODE :"
				, (u32_t)((data & MCPR_NVM_CFG1_BITBANG_MODE) >> 3)
				, "\n\tMCPR_NVM_CFG1_PASS_MODE :"
				, (u32_t)((data & MCPR_NVM_CFG1_PASS_MODE) >> 2)
				, "\n\tMCPR_NVM_CFG1_BUFFER_MODE :"
				, (u32_t)((data & MCPR_NVM_CFG1_BUFFER_MODE) >> 1)
				, "\n\tMCPR_NVM_CFG1_FLASH_MODE :"
				, (u32_t)((data & MCPR_NVM_CFG1_FLASH_MODE) >> 0));
			break;
		case 0x86418: // (MCP_REG_MCPR_NVM_CFG2: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_CFG2 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_NVM_CFG2_READ_ID :"
				, (u32_t)((data & MCPR_NVM_CFG2_READ_ID) >> 24)
				, "\n\tMCPR_NVM_CFG2_STATUS_CMD :"
				, (u32_t)((data & MCPR_NVM_CFG2_STATUS_CMD) >> 16)
				, "\n\tMCPR_NVM_CFG2_CSB_W :"
				, (u32_t)((data & MCPR_NVM_CFG2_CSB_W) >> 8)
				, "\n\tMCPR_NVM_CFG2_ERASE_CMD :"
				, (u32_t)((data & MCPR_NVM_CFG2_ERASE_CMD) >> 0));
			break;
		case 0x8641c: // (MCP_REG_MCPR_NVM_CFG3: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_CFG3 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_NVM_CFG3_READ_CMD :"
				, (u32_t)((data & MCPR_NVM_CFG3_READ_CMD) >> 24)
				, "\n\tMCPR_NVM_CFG3_FAST_READ_CMD :"
				, (u32_t)((data & MCPR_NVM_CFG3_FAST_READ_CMD) >> 16)
				, "\n\tMCPR_NVM_CFG3_WRITE_CMD :"
				, (u32_t)((data & MCPR_NVM_CFG3_WRITE_CMD) >> 8)
				, "\n\tMCPR_NVM_CFG3_BUFFER_RD_CMD :"
				, (u32_t)((data & MCPR_NVM_CFG3_BUFFER_RD_CMD) >> 0));
			break;
		case 0x86420: // (MCP_REG_MCPR_NVM_SW_ARB: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_SW_ARB (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_NVM_SW_ARB_REQ3 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_REQ3) >> 15)
				, "\n\tMCPR_NVM_SW_ARB_REQ2 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_REQ2) >> 14)
				, "\n\tMCPR_NVM_SW_ARB_REQ1 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_REQ1) >> 13)
				, "\n\tMCPR_NVM_SW_ARB_REQ0 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_REQ0) >> 12)
				, "\n\tMCPR_NVM_SW_ARB_ARB_ARB3 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_ARB3) >> 11)
				, "\n\tMCPR_NVM_SW_ARB_ARB_ARB2 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_ARB2) >> 10)
				, "\n\tMCPR_NVM_SW_ARB_ARB_ARB1 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_ARB1) >> 9)
				, "\n\tMCPR_NVM_SW_ARB_ARB_ARB0 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_ARB0) >> 8)
				, "\n\tMCPR_NVM_SW_ARB_ARB_REQ_CLR3 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_REQ_CLR3) >> 7)
				, "\n\tMCPR_NVM_SW_ARB_ARB_REQ_CLR2 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_REQ_CLR2) >> 6)
				, "\n\tMCPR_NVM_SW_ARB_ARB_REQ_CLR1 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_REQ_CLR1) >> 5)
				, "\n\tMCPR_NVM_SW_ARB_ARB_REQ_CLR0 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_REQ_CLR0) >> 4)
				, "\n\tMCPR_NVM_SW_ARB_ARB_REQ_SET3 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_REQ_SET3) >> 3)
				, "\n\tMCPR_NVM_SW_ARB_ARB_REQ_SET2 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_REQ_SET2) >> 2)
				, "\n\tMCPR_NVM_SW_ARB_ARB_REQ_SET1 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_REQ_SET1) >> 1)
				, "\n\tMCPR_NVM_SW_ARB_ARB_REQ_SET0 :"
				, (u32_t)((data & MCPR_NVM_SW_ARB_ARB_REQ_SET0) >> 0));
			break;
		case 0x86424: // (MCP_REG_MCPR_NVM_ACCESS_ENABLE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_ACCESS_ENABLE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_NVM_ACCESS_ENABLE_WR_EN :"
				, (u32_t)((data & MCPR_NVM_ACCESS_ENABLE_WR_EN) >> 1)
				, "\n\tMCPR_NVM_ACCESS_ENABLE_EN :"
				, (u32_t)((data & MCPR_NVM_ACCESS_ENABLE_EN) >> 0));
			break;
		case 0x86428: // (MCP_REG_MCPR_NVM_WRITE1: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_WRITE1 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_NVM_WRITE1_WRDI_CMD :"
				, (u32_t)((data & MCPR_NVM_WRITE1_WRDI_CMD) >> 8)
				, "\n\tMCPR_NVM_WRITE1_WREN_CMD :"
				, (u32_t)((data & MCPR_NVM_WRITE1_WREN_CMD) >> 0));
			break;
		case 0x8642c: // (MCP_REG_MCPR_NVM_CFG4: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_CFG4 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x (%s)%-20s 0x%x (%s)(%s)"
				, addr, data
				, "\n\tMCPR_NVM_CFG4_RESERVED :"
				, (u32_t)((data & MCPR_NVM_CFG4_RESERVED) >> 10)
				, "\n\tMCPR_NVM_CFG4_PASS_MODE_RELAXED_TIMING :"
				, (u32_t)((data & MCPR_NVM_CFG4_PASS_MODE_RELAXED_TIMING) >> 9)
				, "\n\tMCPR_NVM_CFG4_SI_INPUT_RELAXED_TIMING :"
				, (u32_t)((data & MCPR_NVM_CFG4_SI_INPUT_RELAXED_TIMING) >> 8)
				, "\n\tMCPR_NVM_CFG4_FAST :"
				, (u32_t)((data & MCPR_NVM_CFG4_FAST) >> 7)
				, "\n\tMCPR_NVM_CFG4_STATUS_BIT_POLARITY :"
				, (u32_t)((data & MCPR_NVM_CFG4_STATUS_BIT_POLARITY) >> 6)
				, "\n\tMCPR_NVM_CFG4_MODE_256_EMPTY_BIT_LOC :"
				, (u32_t)((data & MCPR_NVM_CFG4_MODE_256_EMPTY_BIT_LOC) >> 4),
				(u32_t)(data & MCPR_NVM_CFG4_MODE_256_EMPTY_BIT_LOC) == MCPR_NVM_CFG4_MODE_256_EMPTY_BIT_LOC_BIT11  ? "BIT11" :
				(u32_t)(data & MCPR_NVM_CFG4_MODE_256_EMPTY_BIT_LOC) == MCPR_NVM_CFG4_MODE_256_EMPTY_BIT_LOC_BIT10  ? "BIT10" :
				(u32_t)(data & MCPR_NVM_CFG4_MODE_256_EMPTY_BIT_LOC) == MCPR_NVM_CFG4_MODE_256_EMPTY_BIT_LOC_BIT9  ? "BIT9" :
				(u32_t)(data & MCPR_NVM_CFG4_MODE_256_EMPTY_BIT_LOC) == MCPR_NVM_CFG4_MODE_256_EMPTY_BIT_LOC_BIT8  ? "BIT8" :"INVALID"
				, "\n\tMCPR_NVM_CFG4_FLASH_VENDOR :"
				, (u32_t)((data & MCPR_NVM_CFG4_FLASH_VENDOR) >> 3),
				(u32_t)(data & MCPR_NVM_CFG4_FLASH_VENDOR) == MCPR_NVM_CFG4_FLASH_VENDOR_ATMEL  ? "ATMEL" :
				(u32_t)(data & MCPR_NVM_CFG4_FLASH_VENDOR) == MCPR_NVM_CFG4_FLASH_VENDOR_ST  ? "ST" :"INVALID",
				(u32_t)(data & MCPR_NVM_WRITE1_WRDI_CMD) == MCPR_NVM_CFG4_FLASH_SIZE_128MBIT  ? "MCPRNVM_CFG4_FLASH_SIZE_128MBIT" :
				(u32_t)(data & MCPR_NVM_WRITE1_WRDI_CMD) == MCPR_NVM_CFG4_FLASH_SIZE_64MBIT  ? "MCPRNVM_CFG4_FLASH_SIZE_64MBIT" :
				(u32_t)(data & MCPR_NVM_WRITE1_WRDI_CMD) == MCPR_NVM_CFG4_FLASH_SIZE_32MBIT  ? "MCPRNVM_CFG4_FLASH_SIZE_32MBIT" :
				(u32_t)(data & MCPR_NVM_WRITE1_WRDI_CMD) == MCPR_NVM_CFG4_FLASH_SIZE_16MBIT  ? "MCPRNVM_CFG4_FLASH_SIZE_16MBIT" :
				(u32_t)(data & MCPR_NVM_WRITE1_WRDI_CMD) == MCPR_NVM_CFG4_FLASH_SIZE_8MBIT  ? "MCPRNVM_CFG4_FLASH_SIZE_8MBIT" :
				(u32_t)(data & MCPR_NVM_WRITE1_WRDI_CMD) == MCPR_NVM_CFG4_FLASH_SIZE_4MBIT  ? "MCPRNVM_CFG4_FLASH_SIZE_4MBIT" :
				(u32_t)(data & MCPR_NVM_WRITE1_WRDI_CMD) == MCPR_NVM_CFG4_FLASH_SIZE_2MBIT  ? "MCPRNVM_CFG4_FLASH_SIZE_2MBIT" :
				(u32_t)(data & MCPR_NVM_WRITE1_WRDI_CMD) == MCPR_NVM_CFG4_FLASH_SIZE_1MBIT  ? "MCPRNVM_CFG4_FLASH_SIZE_1MBIT" :"INVALID");
			break;
		case 0x86430: // (MCP_REG_MCPR_NVM_RECONFIG: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_NVM_RECONFIG (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_NVM_RECONFIG_RECONFIG_DONE :"
				, (u32_t)((data & MCPR_NVM_RECONFIG_RECONFIG_DONE) >> 31)
				, "\n\tMCPR_NVM_RECONFIG_RESERVED :"
				, (u32_t)((data & MCPR_NVM_RECONFIG_RESERVED) >> 8)
				, "\n\tMCPR_NVM_RECONFIG_RECONFIG_STRAP_VALUE :"
				, (u32_t)((data & MCPR_NVM_RECONFIG_RECONFIG_STRAP_VALUE) >> 4)
				, "\n\tMCPR_NVM_RECONFIG_ORIG_STRAP_VALUE :"
				, (u32_t)((data & MCPR_NVM_RECONFIG_ORIG_STRAP_VALUE) >> 0),
				(u32_t)(data & MCPR_NVM_RECONFIG_ORIG_STRAP_VALUE) == MCPR_NVM_RECONFIG_ORIG_STRAP_VALUE_ATMEL  ? "ATMEL" :
				(u32_t)(data & MCPR_NVM_RECONFIG_ORIG_STRAP_VALUE) == MCPR_NVM_RECONFIG_ORIG_STRAP_VALUE_ST  ? "ST" :"INVALID");
			break;
		case 0x86434: // (MCP_REG_MCPR_UNUSED3: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED3 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x86800: // (MCP_REG_MCPR_ERNGN_EXP_ROM_CTRL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_EXP_ROM_CTRL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_EXP_ROM_CTRL_ACTIVE :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_CTRL_ACTIVE) >> 31)
				, "\n\tMCPR_ERNGN_EXP_ROM_CTRL_READ_TIMEOUT :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_CTRL_READ_TIMEOUT) >> 30)
				, "\n\tMCPR_ERNGN_EXP_ROM_CTRL_ARB_TIMEOUT :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_CTRL_ARB_TIMEOUT) >> 29)
				, "\n\tMCPR_ERNGN_EXP_ROM_CTRL_CACHE_VALID :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_CTRL_CACHE_VALID) >> 28)
				, "\n\tMCPR_ERNGN_EXP_ROM_CTRL_STATE :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_CTRL_STATE) >> 16)
				, "\n\tMCPR_ERNGN_EXP_ROM_CTRL_ARB_NUM :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_CTRL_ARB_NUM) >> 4)
				, "\n\tMCPR_ERNGN_EXP_ROM_CTRL_BFRD :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_CTRL_BFRD) >> 1)
				, "\n\tMCPR_ERNGN_EXP_ROM_CTRL_ENA :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_CTRL_ENA) >> 0));
			break;
		case 0x86804: // (MCP_REG_MCPR_ERNGN_EXP_ROM_BADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_EXP_ROM_BADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_EXP_ROM_BADDR_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_BADDR_VALUE) >> 2));
			break;
		case 0x86808: // (MCP_REG_MCPR_ERNGN_EXP_ROM_CFG: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_EXP_ROM_CFG (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_EXP_ROM_CFG_READ_TIMEOUT_SHFT :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_CFG_READ_TIMEOUT_SHFT) >> 4)
				, "\n\tMCPR_ERNGN_EXP_ROM_CFG_ARB_TIMEOUT_SHFT :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_CFG_ARB_TIMEOUT_SHFT) >> 0));
			break;
		case 0x8680c: // (MCP_REG_MCPR_ERNGN_EXP_ROM_ADR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_EXP_ROM_ADR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_EXP_ROM_ADR_REQ :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_ADR_REQ) >> 31)
				, "\n\tMCPR_ERNGN_EXP_ROM_ADR_ACT_FUNC :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_ADR_ACT_FUNC) >> 26)
				, "\n\tMCPR_ERNGN_EXP_ROM_ADR_ADDRESS :"
				, (u32_t)((data & MCPR_ERNGN_EXP_ROM_ADR_ADDRESS) >> 2));
			break;
		case 0x86810: // (MCP_REG_MCPR_ERNGN_EXP_ROM_DATA0: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_EXP_ROM_DATA0 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x86814: // (MCP_REG_MCPR_ERNGN_EXP_ROM_DATA1: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_EXP_ROM_DATA1 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x86818: // (MCP_REG_MCPR_ERNGN_EXP_ROM_DATA2: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_EXP_ROM_DATA2 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x8681c: // (MCP_REG_MCPR_ERNGN_IMG_LOADER0_BADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER0_BADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER0_BADDR_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER0_BADDR_VALUE) >> 2));
			break;
		case 0x86820: // (MCP_REG_MCPR_ERNGN_IMG_LOADER0_GADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER0_GADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER0_GADDR_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER0_GADDR_VALUE) >> 2));
			break;
		case 0x86824: // (MCP_REG_MCPR_ERNGN_IMG_LOADER0_CADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER0_CADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER0_CADDR_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER0_CADDR_VALUE) >> 2));
			break;
		case 0x86828: // (MCP_REG_MCPR_ERNGN_IMG_LOADER0_CDATA: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER0_CDATA (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER0_CDATA_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER0_CDATA_VALUE) >> 0));
			break;
		case 0x8682c: // (MCP_REG_MCPR_ERNGN_IMG_LOADER0_CFG: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER0_CFG (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER0_CFG_ACTIVE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER0_CFG_ACTIVE) >> 31)
				, "\n\tMCPR_ERNGN_IMG_LOADER0_CFG_BUSY :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER0_CFG_BUSY) >> 30)
				, "\n\tMCPR_ERNGN_IMG_LOADER0_CFG_READ_TO :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER0_CFG_READ_TO) >> 29)
				, "\n\tMCPR_ERNGN_IMG_LOADER0_CFG_ARB_TO :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER0_CFG_ARB_TO) >> 28)
				, "\n\tMCPR_ERNGN_IMG_LOADER0_CFG_AUTO_INC :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER0_CFG_AUTO_INC) >> 24),
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER0_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER0_CFG_BURST_SIZE_B4  ? "MCPRERNGN_IMG_LOADER0_CFG_BURST_SIZE_B4" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER0_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER0_CFG_BURST_SIZE_B3  ? "MCPRERNGN_IMG_LOADER0_CFG_BURST_SIZE_B3" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER0_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER0_CFG_BURST_SIZE_B2  ? "MCPRERNGN_IMG_LOADER0_CFG_BURST_SIZE_B2" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER0_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER0_CFG_BURST_SIZE_B1  ? "MCPRERNGN_IMG_LOADER0_CFG_BURST_SIZE_B1" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER0_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER0_CFG_BURST_SIZE_B0  ? "MCPRERNGN_IMG_LOADER0_CFG_BURST_SIZE_B0" :"INVALID");
			break;
		case 0x86830: // (MCP_REG_MCPR_ERNGN_IMG_LOADER1_BADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER1_BADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER1_BADDR_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER1_BADDR_VALUE) >> 2));
			break;
		case 0x86834: // (MCP_REG_MCPR_ERNGN_IMG_LOADER1_GADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER1_GADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER1_GADDR_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER1_GADDR_VALUE) >> 2));
			break;
		case 0x86838: // (MCP_REG_MCPR_ERNGN_IMG_LOADER1_CADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER1_CADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER1_CADDR_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER1_CADDR_VALUE) >> 2));
			break;
		case 0x8683c: // (MCP_REG_MCPR_ERNGN_IMG_LOADER1_CDATA: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER1_CDATA (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER1_CDATA_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER1_CDATA_VALUE) >> 0));
			break;
		case 0x86840: // (MCP_REG_MCPR_ERNGN_IMG_LOADER1_CFG: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER1_CFG (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER1_CFG_ACTIVE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER1_CFG_ACTIVE) >> 31)
				, "\n\tMCPR_ERNGN_IMG_LOADER1_CFG_BUSY :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER1_CFG_BUSY) >> 30)
				, "\n\tMCPR_ERNGN_IMG_LOADER1_CFG_READ_TO :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER1_CFG_READ_TO) >> 29)
				, "\n\tMCPR_ERNGN_IMG_LOADER1_CFG_ARB_TO :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER1_CFG_ARB_TO) >> 28)
				, "\n\tMCPR_ERNGN_IMG_LOADER1_CFG_AUTO_INC :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER1_CFG_AUTO_INC) >> 24),
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER1_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER1_CFG_BURST_SIZE_B4  ? "MCPRERNGN_IMG_LOADER1_CFG_BURST_SIZE_B4" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER1_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER1_CFG_BURST_SIZE_B3  ? "MCPRERNGN_IMG_LOADER1_CFG_BURST_SIZE_B3" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER1_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER1_CFG_BURST_SIZE_B2  ? "MCPRERNGN_IMG_LOADER1_CFG_BURST_SIZE_B2" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER1_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER1_CFG_BURST_SIZE_B1  ? "MCPRERNGN_IMG_LOADER1_CFG_BURST_SIZE_B1" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER1_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER1_CFG_BURST_SIZE_B0  ? "MCPRERNGN_IMG_LOADER1_CFG_BURST_SIZE_B0" :"INVALID");
			break;
		case 0x86844: // (MCP_REG_MCPR_ERNGN_IMG_LOADER2_BADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER2_BADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER2_BADDR_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER2_BADDR_VALUE) >> 2));
			break;
		case 0x86848: // (MCP_REG_MCPR_ERNGN_IMG_LOADER2_GADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER2_GADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER2_GADDR_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER2_GADDR_VALUE) >> 2));
			break;
		case 0x8684c: // (MCP_REG_MCPR_ERNGN_IMG_LOADER2_CADDR: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER2_CADDR (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER2_CADDR_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER2_CADDR_VALUE) >> 2));
			break;
		case 0x86850: // (MCP_REG_MCPR_ERNGN_IMG_LOADER2_CDATA: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER2_CDATA (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER2_CDATA_VALUE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER2_CDATA_VALUE) >> 0));
			break;
		case 0x86854: // (MCP_REG_MCPR_ERNGN_IMG_LOADER2_CFG: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_IMG_LOADER2_CFG (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_ERNGN_IMG_LOADER2_CFG_ACTIVE :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER2_CFG_ACTIVE) >> 31)
				, "\n\tMCPR_ERNGN_IMG_LOADER2_CFG_BUSY :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER2_CFG_BUSY) >> 30)
				, "\n\tMCPR_ERNGN_IMG_LOADER2_CFG_READ_TO :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER2_CFG_READ_TO) >> 29)
				, "\n\tMCPR_ERNGN_IMG_LOADER2_CFG_ARB_TO :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER2_CFG_ARB_TO) >> 28)
				, "\n\tMCPR_ERNGN_IMG_LOADER2_CFG_AUTO_INC :"
				, (u32_t)((data & MCPR_ERNGN_IMG_LOADER2_CFG_AUTO_INC) >> 24),
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER2_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER2_CFG_BURST_SIZE_B4  ? "MCPRERNGN_IMG_LOADER2_CFG_BURST_SIZE_B4" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER2_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER2_CFG_BURST_SIZE_B3  ? "MCPRERNGN_IMG_LOADER2_CFG_BURST_SIZE_B3" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER2_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER2_CFG_BURST_SIZE_B2  ? "MCPRERNGN_IMG_LOADER2_CFG_BURST_SIZE_B2" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER2_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER2_CFG_BURST_SIZE_B1  ? "MCPRERNGN_IMG_LOADER2_CFG_BURST_SIZE_B1" :
				(u32_t)(data & MCPR_ERNGN_IMG_LOADER2_CDATA_VALUE) == MCPR_ERNGN_IMG_LOADER2_CFG_BURST_SIZE_B0  ? "MCPRERNGN_IMG_LOADER2_CFG_BURST_SIZE_B0" :"INVALID");
			break;
		case 0x86858: // (MCP_REG_MCPR_ERNGN_UNUSED_ERNGN: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ERNGN_UNUSED_ERNGN (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x86880: // (MCP_REG_MCPR_UNUSED_E: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED_E (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x88000: // (MCP_REG_MCPR_SMBUS_CONFIG: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_CONFIG (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_CONFIG_RESET :"
				, (u32_t)((data & MCPR_SMBUS_CONFIG_RESET) >> 31)
				, "\n\tMCPR_SMBUS_CONFIG_SMB_EN :"
				, (u32_t)((data & MCPR_SMBUS_CONFIG_SMB_EN) >> 30)
				, "\n\tMCPR_SMBUS_CONFIG_BIT_BANG_EN :"
				, (u32_t)((data & MCPR_SMBUS_CONFIG_BIT_BANG_EN) >> 29)
				, "\n\tMCPR_SMBUS_CONFIG_EN_NIC_SMB_ADDR_0 :"
				, (u32_t)((data & MCPR_SMBUS_CONFIG_EN_NIC_SMB_ADDR_0) >> 28)
				, "\n\tMCPR_SMBUS_CONFIG_PROMISCOUS_MODE :"
				, (u32_t)((data & MCPR_SMBUS_CONFIG_PROMISCOUS_MODE) >> 27)
				, "\n\tMCPR_SMBUS_CONFIG_TIMESTAMP_CNT_EN :"
				, (u32_t)((data & MCPR_SMBUS_CONFIG_TIMESTAMP_CNT_EN) >> 26)
				, "\n\tMCPR_SMBUS_CONFIG_MASTER_RTRY_CNT :"
				, (u32_t)((data & MCPR_SMBUS_CONFIG_MASTER_RTRY_CNT) >> 16)
				, "\n\tMCPR_SMBUS_CONFIG_ARP_EN1 :"
				, (u32_t)((data & MCPR_SMBUS_CONFIG_ARP_EN1) >> 9)
				, "\n\tMCPR_SMBUS_CONFIG_ARP_EN0 :"
				, (u32_t)((data & MCPR_SMBUS_CONFIG_ARP_EN0) >> 8)
				, "\n\tMCPR_SMBUS_CONFIG_HW_ARP_ASSIGN_ADDR :"
				, (u32_t)((data & MCPR_SMBUS_CONFIG_HW_ARP_ASSIGN_ADDR) >> 7));
			break;
		case 0x88004: // (MCP_REG_MCPR_SMBUS_TIMING_CONFIG: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_TIMING_CONFIG (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_TIMING_CONFIG_MODE_400 :"
				, (u32_t)((data & MCPR_SMBUS_TIMING_CONFIG_MODE_400) >> 31)
				, "\n\tMCPR_SMBUS_TIMING_CONFIG_RANDOM_SLAVE_STRETCH :"
				, (u32_t)((data & MCPR_SMBUS_TIMING_CONFIG_RANDOM_SLAVE_STRETCH) >> 24)
				, "\n\tMCPR_SMBUS_TIMING_CONFIG_PERIODIC_SLAVE_STRETCH :"
				, (u32_t)((data & MCPR_SMBUS_TIMING_CONFIG_PERIODIC_SLAVE_STRETCH) >> 16)
				, "\n\tMCPR_SMBUS_TIMING_CONFIG_SMBUS_IDLE_TIME :"
				, (u32_t)((data & MCPR_SMBUS_TIMING_CONFIG_SMBUS_IDLE_TIME) >> 8));
			break;
		case 0x88008: // (MCP_REG_MCPR_SMBUS_ADDRESS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_ADDRESS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_ADDRESS_EN_NIC_SMB_ADDR3 :"
				, (u32_t)((data & MCPR_SMBUS_ADDRESS_EN_NIC_SMB_ADDR3) >> 31)
				, "\n\tMCPR_SMBUS_ADDRESS_NIC_SMB_ADDR3 :"
				, (u32_t)((data & MCPR_SMBUS_ADDRESS_NIC_SMB_ADDR3) >> 24)
				, "\n\tMCPR_SMBUS_ADDRESS_EN_NIC_SMB_ADDR2 :"
				, (u32_t)((data & MCPR_SMBUS_ADDRESS_EN_NIC_SMB_ADDR2) >> 23)
				, "\n\tMCPR_SMBUS_ADDRESS_NIC_SMB_ADDR2 :"
				, (u32_t)((data & MCPR_SMBUS_ADDRESS_NIC_SMB_ADDR2) >> 16)
				, "\n\tMCPR_SMBUS_ADDRESS_EN_NIC_SMB_ADDR1 :"
				, (u32_t)((data & MCPR_SMBUS_ADDRESS_EN_NIC_SMB_ADDR1) >> 15)
				, "\n\tMCPR_SMBUS_ADDRESS_NIC_SMB_ADDR1 :"
				, (u32_t)((data & MCPR_SMBUS_ADDRESS_NIC_SMB_ADDR1) >> 8)
				, "\n\tMCPR_SMBUS_ADDRESS_EN_NIC_SMB_ADDR0 :"
				, (u32_t)((data & MCPR_SMBUS_ADDRESS_EN_NIC_SMB_ADDR0) >> 7)
				, "\n\tMCPR_SMBUS_ADDRESS_NIC_SMB_ADDR0 :"
				, (u32_t)((data & MCPR_SMBUS_ADDRESS_NIC_SMB_ADDR0) >> 0));
			break;
		case 0x8800c: // (MCP_REG_MCPR_SMBUS_MASTER_FIFO_CONTROL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_MASTER_FIFO_CONTROL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_MASTER_FIFO_CONTROL_MASTER_RX_FIFO_FLUSH :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_FIFO_CONTROL_MASTER_RX_FIFO_FLUSH) >> 31)
				, "\n\tMCPR_SMBUS_MASTER_FIFO_CONTROL_MASTER_TX_FIFO_FLUSH :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_FIFO_CONTROL_MASTER_TX_FIFO_FLUSH) >> 30)
				, "\n\tMCPR_SMBUS_MASTER_FIFO_CONTROL_MASTER_RX_FIFO_THRESHOLD :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_FIFO_CONTROL_MASTER_RX_FIFO_THRESHOLD) >> 8));
			break;
		case 0x88010: // (MCP_REG_MCPR_SMBUS_SLAVE_FIFO_CONTROL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_SLAVE_FIFO_CONTROL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_SLAVE_FIFO_CONTROL_SLAVE_RX_FIFO_FLUSH :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_FIFO_CONTROL_SLAVE_RX_FIFO_FLUSH) >> 31)
				, "\n\tMCPR_SMBUS_SLAVE_FIFO_CONTROL_SLAVE_TX_FIFO_FLUSH :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_FIFO_CONTROL_SLAVE_TX_FIFO_FLUSH) >> 30)
				, "\n\tMCPR_SMBUS_SLAVE_FIFO_CONTROL_SLAVE_RX_FIFO_THRESHOLD :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_FIFO_CONTROL_SLAVE_RX_FIFO_THRESHOLD) >> 8));
			break;
		case 0x88014: // (MCP_REG_MCPR_SMBUS_BIT_BANG_CONTROL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_BIT_BANG_CONTROL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_BIT_BANG_CONTROL_SMBCLK_IN :"
				, (u32_t)((data & MCPR_SMBUS_BIT_BANG_CONTROL_SMBCLK_IN) >> 31)
				, "\n\tMCPR_SMBUS_BIT_BANG_CONTROL_SMBCLK_OUT_EN :"
				, (u32_t)((data & MCPR_SMBUS_BIT_BANG_CONTROL_SMBCLK_OUT_EN) >> 30)
				, "\n\tMCPR_SMBUS_BIT_BANG_CONTROL_SMBDAT_IN :"
				, (u32_t)((data & MCPR_SMBUS_BIT_BANG_CONTROL_SMBDAT_IN) >> 29)
				, "\n\tMCPR_SMBUS_BIT_BANG_CONTROL_SMBDAT_OUT_EN :"
				, (u32_t)((data & MCPR_SMBUS_BIT_BANG_CONTROL_SMBDAT_OUT_EN) >> 28));
			break;
		case 0x88018: // (MCP_REG_MCPR_SMBUS_WATCHDOG: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_WATCHDOG (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_WATCHDOG_WATCHDOG :"
				, (u32_t)((data & MCPR_SMBUS_WATCHDOG_WATCHDOG) >> 0));
			break;
		case 0x8801c: // (MCP_REG_MCPR_SMBUS_HEARTBEAT: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_HEARTBEAT (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_HEARTBEAT_HEARTBEAT :"
				, (u32_t)((data & MCPR_SMBUS_HEARTBEAT_HEARTBEAT) >> 0));
			break;
		case 0x88020: // (MCP_REG_MCPR_SMBUS_POLL_ASF: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_POLL_ASF (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_POLL_ASF_POLL_ASF :"
				, (u32_t)((data & MCPR_SMBUS_POLL_ASF_POLL_ASF) >> 0));
			break;
		case 0x88024: // (MCP_REG_MCPR_SMBUS_POLL_LEGACY: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_POLL_LEGACY (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_POLL_LEGACY_POLL_LEGACY :"
				, (u32_t)((data & MCPR_SMBUS_POLL_LEGACY_POLL_LEGACY) >> 0));
			break;
		case 0x88028: // (MCP_REG_MCPR_SMBUS_RETRAN: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_RETRAN (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_RETRAN_RETRAN :"
				, (u32_t)((data & MCPR_SMBUS_RETRAN_RETRAN) >> 0));
			break;
		case 0x8802c: // (MCP_REG_MCPR_SMBUS_TIMESTAMP: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_TIMESTAMP (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_TIMESTAMP_TIMESTAMP :"
				, (u32_t)((data & MCPR_SMBUS_TIMESTAMP_TIMESTAMP) >> 0));
			break;
		case 0x88030: // (MCP_REG_MCPR_SMBUS_MASTER_COMMAND: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_MASTER_COMMAND (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x (%s)%-20s 0x%x (%s)%-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_MASTER_COMMAND_START_BUSY :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_COMMAND_START_BUSY) >> 31)
				, "\n\tMCPR_SMBUS_MASTER_COMMAND_ABORT :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_COMMAND_ABORT) >> 30)
				, "\n\tMCPR_SMBUS_MASTER_COMMAND_STATUS :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_COMMAND_STATUS) >> 25),
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_STATUS) == MCPR_SMBUS_MASTER_COMMAND_STATUS_111  ? "111" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_STATUS) == MCPR_SMBUS_MASTER_COMMAND_STATUS_110  ? "110" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_STATUS) == MCPR_SMBUS_MASTER_COMMAND_STATUS_101  ? "101" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_STATUS) == MCPR_SMBUS_MASTER_COMMAND_STATUS_100  ? "100" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_STATUS) == MCPR_SMBUS_MASTER_COMMAND_STATUS_011  ? "011" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_STATUS) == MCPR_SMBUS_MASTER_COMMAND_STATUS_010  ? "010" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_STATUS) == MCPR_SMBUS_MASTER_COMMAND_STATUS_001  ? "001" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_STATUS) == MCPR_SMBUS_MASTER_COMMAND_STATUS_000  ? "000" :"INVALID"
				, "\n\tMCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) >> 9),
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_1011  ? "1011" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_1010  ? "1010" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_1001  ? "1001" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_1000  ? "1000" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_0111  ? "0111" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_0110  ? "0110" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_0101  ? "0101" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_0100  ? "0100" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_0011  ? "0011" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_0010  ? "0010" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_0001  ? "0001" :
				(u32_t)(data & MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL) == MCPR_SMBUS_MASTER_COMMAND_SMBUS_PROTOCOL_0000  ? "0000" :"INVALID"
				, "\n\tMCPR_SMBUS_MASTER_COMMAND_PEC :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_COMMAND_PEC) >> 8));
			break;
		case 0x88034: // (MCP_REG_MCPR_SMBUS_SLAVE_COMMAND: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_SLAVE_COMMAND (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x (%s)%-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_SLAVE_COMMAND_START :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_COMMAND_START) >> 31)
				, "\n\tMCPR_SMBUS_SLAVE_COMMAND_ABORT :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_COMMAND_ABORT) >> 30)
				, "\n\tMCPR_SMBUS_SLAVE_COMMAND_STATUS :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_COMMAND_STATUS) >> 23),
				(u32_t)(data & MCPR_SMBUS_SLAVE_COMMAND_STATUS) == MCPR_SMBUS_SLAVE_COMMAND_STATUS_111  ? "111" :
				(u32_t)(data & MCPR_SMBUS_SLAVE_COMMAND_STATUS) == MCPR_SMBUS_SLAVE_COMMAND_STATUS_101  ? "101" :
				(u32_t)(data & MCPR_SMBUS_SLAVE_COMMAND_STATUS) == MCPR_SMBUS_SLAVE_COMMAND_STATUS_000  ? "000" :"INVALID"
				, "\n\tMCPR_SMBUS_SLAVE_COMMAND_PEC :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_COMMAND_PEC) >> 8));
			break;
		case 0x88038: // (MCP_REG_MCPR_SMBUS_EVENT_ENABLE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_EVENT_ENABLE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_MASTER_RX_FIFO_FULL_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_MASTER_RX_FIFO_FULL_EN) >> 31)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_MASTER_RX_THRESHOLD_HIT_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_MASTER_RX_THRESHOLD_HIT_EN) >> 30)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_MASTER_RX_EVENT_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_MASTER_RX_EVENT_EN) >> 29)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_MASTER_START_BUSY_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_MASTER_START_BUSY_EN) >> 28)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_MASTER_TX_UNDERRUN_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_MASTER_TX_UNDERRUN_EN) >> 27)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_SLAVE_RX_FIFO_FULL_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_SLAVE_RX_FIFO_FULL_EN) >> 26)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_SLAVE_RX_THRESHOLD_HIT_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_SLAVE_RX_THRESHOLD_HIT_EN) >> 25)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_SLAVE_RX_EVENT_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_SLAVE_RX_EVENT_EN) >> 24)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_SLAVE_START_BUSY_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_SLAVE_START_BUSY_EN) >> 23)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_SLAVE_TX_UNDERRUN_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_SLAVE_TX_UNDERRUN_EN) >> 22)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_SLAVE_RD_EVENT_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_SLAVE_RD_EVENT_EN) >> 21)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_SLAVE_ARP_EVENT_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_SLAVE_ARP_EVENT_EN) >> 20)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_RETRANSMIT_TO_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_RETRANSMIT_TO_EN) >> 4)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_POLL_LEGACY_TO_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_POLL_LEGACY_TO_EN) >> 3)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_POLL_ASF_TO_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_POLL_ASF_TO_EN) >> 2)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_HEARTBEAT_TO_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_HEARTBEAT_TO_EN) >> 1)
				, "\n\tMCPR_SMBUS_EVENT_ENABLE_WATCHDOG_TO_EN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_ENABLE_WATCHDOG_TO_EN) >> 0));
			break;
		case 0x8803c: // (MCP_REG_MCPR_SMBUS_EVENT_STATUS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_EVENT_STATUS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_EVENT_STATUS_MASTER_RX_FIFO_FULL :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_MASTER_RX_FIFO_FULL) >> 31)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_MASTER_RX_THRESHOLD_HIT :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_MASTER_RX_THRESHOLD_HIT) >> 30)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_MASTER_RX_EVENT :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_MASTER_RX_EVENT) >> 29)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_MASTER_START_BUSY :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_MASTER_START_BUSY) >> 28)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_MASTER_TX_UNDERRUN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_MASTER_TX_UNDERRUN) >> 27)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_SLAVE_RX_FIFO_FULL :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_SLAVE_RX_FIFO_FULL) >> 26)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_SLAVE_RX_THRESHOLD_HIT :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_SLAVE_RX_THRESHOLD_HIT) >> 25)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_SLAVE_RX_EVENT :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_SLAVE_RX_EVENT) >> 24)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_SLAVE_START_BUSY :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_SLAVE_START_BUSY) >> 23)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_SLAVE_TX_UNDERRUN :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_SLAVE_TX_UNDERRUN) >> 22)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_SLAVE_RD_EVENT :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_SLAVE_RD_EVENT) >> 21)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_SLAVE_ARP_EVENT :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_SLAVE_ARP_EVENT) >> 20)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_RETRANSMIT_TO :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_RETRANSMIT_TO) >> 4)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_POLL_LEGACY_TO :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_POLL_LEGACY_TO) >> 3)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_POLL_ASF_TO :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_POLL_ASF_TO) >> 2)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_HEARTBEAT_TO :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_HEARTBEAT_TO) >> 1)
				, "\n\tMCPR_SMBUS_EVENT_STATUS_WATCHDOG_TO :"
				, (u32_t)((data & MCPR_SMBUS_EVENT_STATUS_WATCHDOG_TO) >> 0));
			break;
		case 0x88040: // (MCP_REG_MCPR_SMBUS_MASTER_DATA_WRITE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_MASTER_DATA_WRITE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_MASTER_DATA_WRITE_WR_STATUS :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_DATA_WRITE_WR_STATUS) >> 31)
				, "\n\tMCPR_SMBUS_MASTER_DATA_WRITE_MASTER_SMBUS_WR_DATA :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_DATA_WRITE_MASTER_SMBUS_WR_DATA) >> 0));
			break;
		case 0x88044: // (MCP_REG_MCPR_SMBUS_MASTER_DATA_READ: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_MASTER_DATA_READ (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)%-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_MASTER_DATA_READ_RD_STATUS :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_DATA_READ_RD_STATUS) >> 30),
				(u32_t)(data & MCPR_SMBUS_MASTER_DATA_READ_RD_STATUS) == MCPR_SMBUS_MASTER_DATA_READ_RD_STATUS_11  ? "11" :
				(u32_t)(data & MCPR_SMBUS_MASTER_DATA_READ_RD_STATUS) == MCPR_SMBUS_MASTER_DATA_READ_RD_STATUS_10  ? "10" :
				(u32_t)(data & MCPR_SMBUS_MASTER_DATA_READ_RD_STATUS) == MCPR_SMBUS_MASTER_DATA_READ_RD_STATUS_01  ? "01" :
				(u32_t)(data & MCPR_SMBUS_MASTER_DATA_READ_RD_STATUS) == MCPR_SMBUS_MASTER_DATA_READ_RD_STATUS_00  ? "00" :"INVALID"
				, "\n\tMCPR_SMBUS_MASTER_DATA_READ_PEC_ERR :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_DATA_READ_PEC_ERR) >> 29)
				, "\n\tMCPR_SMBUS_MASTER_DATA_READ_MASTER_SMBUS_RD_DATA :"
				, (u32_t)((data & MCPR_SMBUS_MASTER_DATA_READ_MASTER_SMBUS_RD_DATA) >> 0));
			break;
		case 0x88048: // (MCP_REG_MCPR_SMBUS_SLAVE_DATA_WRITE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_SLAVE_DATA_WRITE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)%-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_SLAVE_DATA_WRITE_WR_STATUS :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_DATA_WRITE_WR_STATUS) >> 31),
				(u32_t)(data & MCPR_SMBUS_SLAVE_DATA_WRITE_WR_STATUS) == MCPR_SMBUS_SLAVE_DATA_WRITE_WR_STATUS_1  ? "1" :
				(u32_t)(data & MCPR_SMBUS_SLAVE_DATA_WRITE_WR_STATUS) == MCPR_SMBUS_SLAVE_DATA_WRITE_WR_STATUS_0  ? "0" :"INVALID"
				, "\n\tMCPR_SMBUS_SLAVE_DATA_WRITE_SLAVE_SMBUS_WR_DATA :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_DATA_WRITE_SLAVE_SMBUS_WR_DATA) >> 0));
			break;
		case 0x8804c: // (MCP_REG_MCPR_SMBUS_SLAVE_DATA_READ: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_SLAVE_DATA_READ (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)%-20s 0x%x (%s)%-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_SLAVE_DATA_READ_RD_STATUS :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_DATA_READ_RD_STATUS) >> 30),
				(u32_t)(data & MCPR_SMBUS_SLAVE_DATA_READ_RD_STATUS) == MCPR_SMBUS_SLAVE_DATA_READ_RD_STATUS_11  ? "11" :
				(u32_t)(data & MCPR_SMBUS_SLAVE_DATA_READ_RD_STATUS) == MCPR_SMBUS_SLAVE_DATA_READ_RD_STATUS_10  ? "10" :
				(u32_t)(data & MCPR_SMBUS_SLAVE_DATA_READ_RD_STATUS) == MCPR_SMBUS_SLAVE_DATA_READ_RD_STATUS_01  ? "01" :
				(u32_t)(data & MCPR_SMBUS_SLAVE_DATA_READ_RD_STATUS) == MCPR_SMBUS_SLAVE_DATA_READ_RD_STATUS_00  ? "00" :"INVALID"
				, "\n\tMCPR_SMBUS_SLAVE_DATA_READ_ERR_STATUS :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_DATA_READ_ERR_STATUS) >> 28),
				(u32_t)(data & MCPR_SMBUS_SLAVE_DATA_READ_ERR_STATUS) == MCPR_SMBUS_SLAVE_DATA_READ_ERR_STATUS_11  ? "11" :
				(u32_t)(data & MCPR_SMBUS_SLAVE_DATA_READ_ERR_STATUS) == MCPR_SMBUS_SLAVE_DATA_READ_ERR_STATUS_10  ? "10" :
				(u32_t)(data & MCPR_SMBUS_SLAVE_DATA_READ_ERR_STATUS) == MCPR_SMBUS_SLAVE_DATA_READ_ERR_STATUS_01  ? "01" :
				(u32_t)(data & MCPR_SMBUS_SLAVE_DATA_READ_ERR_STATUS) == MCPR_SMBUS_SLAVE_DATA_READ_ERR_STATUS_00  ? "00" :"INVALID"
				, "\n\tMCPR_SMBUS_SLAVE_DATA_READ_SLAVE_SMBUS_RD_DATA :"
				, (u32_t)((data & MCPR_SMBUS_SLAVE_DATA_READ_SLAVE_SMBUS_RD_DATA) >> 0));
			break;
		case 0x88050: // (MCP_REG_MCPR_UNUSED4: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED4 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x88080: // (MCP_REG_MCPR_SMBUS_ARP_STATE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_ARP_STATE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_ARP_STATE_AR_FLAG1 :"
				, (u32_t)((data & MCPR_SMBUS_ARP_STATE_AR_FLAG1) >> 5)
				, "\n\tMCPR_SMBUS_ARP_STATE_AV_FLAG1 :"
				, (u32_t)((data & MCPR_SMBUS_ARP_STATE_AV_FLAG1) >> 4)
				, "\n\tMCPR_SMBUS_ARP_STATE_AR_FLAG0 :"
				, (u32_t)((data & MCPR_SMBUS_ARP_STATE_AR_FLAG0) >> 1)
				, "\n\tMCPR_SMBUS_ARP_STATE_AV_FLAG0 :"
				, (u32_t)((data & MCPR_SMBUS_ARP_STATE_AV_FLAG0) >> 0));
			break;
		case 0x88084: // (MCP_REG_MCPR_UNUSED5: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED5 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x88090: // (MCP_REG_MCPR_SMBUS_UDID0_3: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_UDID0_3 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_UDID0_3_BYTE_15 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_3_BYTE_15) >> 24)
				, "\n\tMCPR_SMBUS_UDID0_3_BYTE_14 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_3_BYTE_14) >> 16)
				, "\n\tMCPR_SMBUS_UDID0_3_BYTE_13 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_3_BYTE_13) >> 8)
				, "\n\tMCPR_SMBUS_UDID0_3_BYTE_12 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_3_BYTE_12) >> 0));
			break;
		case 0x88094: // (MCP_REG_MCPR_SMBUS_UDID0_2: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_UDID0_2 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_UDID0_2_BYTE_11 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_2_BYTE_11) >> 24)
				, "\n\tMCPR_SMBUS_UDID0_2_BYTE_10 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_2_BYTE_10) >> 16)
				, "\n\tMCPR_SMBUS_UDID0_2_BYTE_9 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_2_BYTE_9) >> 8)
				, "\n\tMCPR_SMBUS_UDID0_2_BYTE_8 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_2_BYTE_8) >> 0));
			break;
		case 0x88098: // (MCP_REG_MCPR_SMBUS_UDID0_1: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_UDID0_1 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_UDID0_1_BYTE_7 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_1_BYTE_7) >> 24)
				, "\n\tMCPR_SMBUS_UDID0_1_BYTE_6 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_1_BYTE_6) >> 16)
				, "\n\tMCPR_SMBUS_UDID0_1_BYTE_5 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_1_BYTE_5) >> 8)
				, "\n\tMCPR_SMBUS_UDID0_1_BYTE_4 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_1_BYTE_4) >> 0));
			break;
		case 0x8809c: // (MCP_REG_MCPR_SMBUS_UDID0_0: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_UDID0_0 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_UDID0_0_BYTE_3 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_0_BYTE_3) >> 24)
				, "\n\tMCPR_SMBUS_UDID0_0_BYTE_2 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_0_BYTE_2) >> 16)
				, "\n\tMCPR_SMBUS_UDID0_0_BYTE_1 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_0_BYTE_1) >> 8)
				, "\n\tMCPR_SMBUS_UDID0_0_BYTE_0 :"
				, (u32_t)((data & MCPR_SMBUS_UDID0_0_BYTE_0) >> 0));
			break;
		case 0x880a0: // (MCP_REG_MCPR_SMBUS_UDID1_3: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_UDID1_3 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_UDID1_3_BYTE_15 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_3_BYTE_15) >> 24)
				, "\n\tMCPR_SMBUS_UDID1_3_BYTE_14 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_3_BYTE_14) >> 16)
				, "\n\tMCPR_SMBUS_UDID1_3_BYTE_13 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_3_BYTE_13) >> 8)
				, "\n\tMCPR_SMBUS_UDID1_3_BYTE_12 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_3_BYTE_12) >> 0));
			break;
		case 0x880a4: // (MCP_REG_MCPR_SMBUS_UDID1_2: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_UDID1_2 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_UDID1_2_BYTE_11 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_2_BYTE_11) >> 24)
				, "\n\tMCPR_SMBUS_UDID1_2_BYTE_10 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_2_BYTE_10) >> 16)
				, "\n\tMCPR_SMBUS_UDID1_2_BYTE_9 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_2_BYTE_9) >> 8)
				, "\n\tMCPR_SMBUS_UDID1_2_BYTE_8 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_2_BYTE_8) >> 0));
			break;
		case 0x880a8: // (MCP_REG_MCPR_SMBUS_UDID1_1: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_UDID1_1 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_UDID1_1_BYTE_7 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_1_BYTE_7) >> 24)
				, "\n\tMCPR_SMBUS_UDID1_1_BYTE_6 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_1_BYTE_6) >> 16)
				, "\n\tMCPR_SMBUS_UDID1_1_BYTE_5 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_1_BYTE_5) >> 8)
				, "\n\tMCPR_SMBUS_UDID1_1_BYTE_4 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_1_BYTE_4) >> 0));
			break;
		case 0x880ac: // (MCP_REG_MCPR_SMBUS_UDID1_0: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SMBUS_UDID1_0 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_SMBUS_UDID1_0_BYTE_3 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_0_BYTE_3) >> 24)
				, "\n\tMCPR_SMBUS_UDID1_0_BYTE_2 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_0_BYTE_2) >> 16)
				, "\n\tMCPR_SMBUS_UDID1_0_BYTE_1 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_0_BYTE_1) >> 8)
				, "\n\tMCPR_SMBUS_UDID1_0_BYTE_0 :"
				, (u32_t)((data & MCPR_SMBUS_UDID1_0_BYTE_0) >> 0));
			break;
		case 0x880b0: // (MCP_REG_MCPR_UNUSED6: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED6 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x88400: // (MCP_REG_MCPR_UNUSED7: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED7 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x88800: // (MCP_REG_MCPR_LEGACY_UNUSED_LEGACY_SMB: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_LEGACY_UNUSED_LEGACY_SMB (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x88824: // (MCP_REG_MCPR_UNUSED8: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED8 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x88c00: // (MCP_REG_MCPR_UNUSED9: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED9 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90000: // (MCP_REG_MCPR_ROM: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_ROM (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x90500: // (MCP_REG_MCPR_UNUSED10: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED10 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x98000: // (MCP_REG_MCPR_UMP_UMP_CMD: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_CMD (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_CMD_INGRESS_MAC_RESET :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_INGRESS_MAC_RESET) >> 31)
				, "\n\tMCPR_UMP_UMP_CMD_EGRESS_MAC_RESET :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_EGRESS_MAC_RESET) >> 30)
				, "\n\tMCPR_UMP_UMP_CMD_INGRESS_FIO_RESET :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_INGRESS_FIO_RESET) >> 29)
				, "\n\tMCPR_UMP_UMP_CMD_EGRESS_FIO_RESET :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_EGRESS_FIO_RESET) >> 28)
				, "\n\tMCPR_UMP_UMP_CMD_CMD_IPG :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_CMD_IPG) >> 17)
				, "\n\tMCPR_UMP_UMP_CMD_EGRESS_PKT_FLUSH :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_EGRESS_PKT_FLUSH) >> 16)
				, "\n\tMCPR_UMP_UMP_CMD_NO_PLUS_TWO :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_NO_PLUS_TWO) >> 15)
				, "\n\tMCPR_UMP_UMP_CMD_INGRESS_RESET :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_INGRESS_RESET) >> 14)
				, "\n\tMCPR_UMP_UMP_CMD_AUTO_DRIVE :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_AUTO_DRIVE) >> 13)
				, "\n\tMCPR_UMP_UMP_CMD_SW_PAUSE :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_SW_PAUSE) >> 9)
				, "\n\tMCPR_UMP_UMP_CMD_INGRESS_DRIVE :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_INGRESS_DRIVE) >> 8)
				, "\n\tMCPR_UMP_UMP_CMD_INGRESS_MAC_DISABLE :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_INGRESS_MAC_DISABLE) >> 6)
				, "\n\tMCPR_UMP_UMP_CMD_EGRESS_MAC_DISABLE :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_EGRESS_MAC_DISABLE) >> 5)
				, "\n\tMCPR_UMP_UMP_CMD_MAC_LOOPBACK :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_MAC_LOOPBACK) >> 3)
				, "\n\tMCPR_UMP_UMP_CMD_FC_EN :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_FC_EN) >> 2)
				, "\n\tMCPR_UMP_UMP_CMD_INGRESS_FIFO_ENABLED :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_INGRESS_FIFO_ENABLED) >> 1)
				, "\n\tMCPR_UMP_UMP_CMD_EGRESS_FIFO_ENABLED :"
				, (u32_t)((data & MCPR_UMP_UMP_CMD_EGRESS_FIFO_ENABLED) >> 0));
			break;
		case 0x98004: // (MCP_REG_MCPR_UMP_UMP_CONFIG: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_CONFIG (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_CONFIG_OLD_BCNT_RDY :"
				, (u32_t)((data & MCPR_UMP_UMP_CONFIG_OLD_BCNT_RDY) >> 24)
				, "\n\tMCPR_UMP_UMP_CONFIG_INGRESS_WORD_ACCM :"
				, (u32_t)((data & MCPR_UMP_UMP_CONFIG_INGRESS_WORD_ACCM) >> 8)
				, "\n\tMCPR_UMP_UMP_CONFIG_INGRESS_MODE :"
				, (u32_t)((data & MCPR_UMP_UMP_CONFIG_INGRESS_MODE) >> 7)
				, "\n\tMCPR_UMP_UMP_CONFIG_RVMII_MODE :"
				, (u32_t)((data & MCPR_UMP_UMP_CONFIG_RVMII_MODE) >> 6)
				, "\n\tMCPR_UMP_UMP_CONFIG_RMII_MODE :"
				, (u32_t)((data & MCPR_UMP_UMP_CONFIG_RMII_MODE) >> 4));
			break;
		case 0x98008: // (MCP_REG_MCPR_UMP_UMP_FC_TRIP: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_FC_TRIP (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_FC_TRIP_XOFF_TRIP :"
				, (u32_t)((data & MCPR_UMP_UMP_FC_TRIP_XOFF_TRIP) >> 16)
				, "\n\tMCPR_UMP_UMP_FC_TRIP_XON_TRIP :"
				, (u32_t)((data & MCPR_UMP_UMP_FC_TRIP_XON_TRIP) >> 0));
			break;
		case 0x9800c: // (MCP_REG_MCPR_UMP_UNUSED_E: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UNUSED_E (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x98090: // (MCP_REG_MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)%-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE) >> 27),
				(u32_t)(data & MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE) == MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE_LATCH_IP_HDR  ? "LATCH_IP_HDR" :
				(u32_t)(data & MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE) == MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE_EXTRA_RD  ? "EXTRA_RD" :
				(u32_t)(data & MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE) == MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE_BUSY  ? "BUSY" :
				(u32_t)(data & MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE) == MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE_READY  ? "READY" :
				(u32_t)(data & MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE) == MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_EGRESS_FIFO_STATE_IDLE  ? "IDLE" :"INVALID"
				, "\n\tMCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_BCNT :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_BCNT) >> 3)
				, "\n\tMCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_FIFO_EMPTY :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_FIFO_EMPTY) >> 2)
				, "\n\tMCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_FRM_IN_PRO :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_FRM_IN_PRO) >> 1)
				, "\n\tMCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_NEW_FRM :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_FRM_RD_STATUS_NEW_FRM) >> 0));
			break;
		case 0x98094: // (MCP_REG_MCPR_UMP_UMP_EGRESS_FRM_RD_DATA: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_FRM_RD_DATA (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x98098: // (MCP_REG_MCPR_UMP_UMP_INGRESS_FRM_WR_CTL: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_INGRESS_FRM_WR_CTL (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)%-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_INGRESS_FRM_WR_CTL_INGRESS_FIFO_STATE :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_INGRESS_FIFO_STATE) >> 30),
				(u32_t)(data & MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_INGRESS_FIFO_STATE) == MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_INGRESS_FIFO_STATE_EXTRA_WR  ? "EXTRA_WR" :
				(u32_t)(data & MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_INGRESS_FIFO_STATE) == MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_INGRESS_FIFO_STATE_BUSY  ? "BUSY" :
				(u32_t)(data & MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_INGRESS_FIFO_STATE) == MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_INGRESS_FIFO_STATE_WAIT  ? "WAIT" :
				(u32_t)(data & MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_INGRESS_FIFO_STATE) == MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_INGRESS_FIFO_STATE_IDLE  ? "IDLE" :"INVALID"
				, "\n\tMCPR_UMP_UMP_INGRESS_FRM_WR_CTL_BCNT :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_BCNT) >> 3)
				, "\n\tMCPR_UMP_UMP_INGRESS_FRM_WR_CTL_BCNT_RDY :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_BCNT_RDY) >> 2)
				, "\n\tMCPR_UMP_UMP_INGRESS_FRM_WR_CTL_FIFO_RDY :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_FIFO_RDY) >> 1)
				, "\n\tMCPR_UMP_UMP_INGRESS_FRM_WR_CTL_NEW_FRM :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_FRM_WR_CTL_NEW_FRM) >> 0));
			break;
		case 0x9809c: // (MCP_REG_MCPR_UMP_UMP_INGRESS_FRM_WR_DATA: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_INGRESS_FRM_WR_DATA (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x980a0: // (MCP_REG_MCPR_UMP_UMP_EGRESS_FRAME_TYPE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_FRAME_TYPE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x980a4: // (MCP_REG_MCPR_UMP_UMP_FIFO_REMAINING_WORDS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_FIFO_REMAINING_WORDS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_FIFO_REMAINING_WORDS_INGRESS_FIFO_OVERFLOW :"
				, (u32_t)((data & MCPR_UMP_UMP_FIFO_REMAINING_WORDS_INGRESS_FIFO_OVERFLOW) >> 31)
				, "\n\tMCPR_UMP_UMP_FIFO_REMAINING_WORDS_INGRESS_FIFO_UNDERFLOW :"
				, (u32_t)((data & MCPR_UMP_UMP_FIFO_REMAINING_WORDS_INGRESS_FIFO_UNDERFLOW) >> 30)
				, "\n\tMCPR_UMP_UMP_FIFO_REMAINING_WORDS_INGRESS_FIFO_DEPTH :"
				, (u32_t)((data & MCPR_UMP_UMP_FIFO_REMAINING_WORDS_INGRESS_FIFO_DEPTH) >> 16)
				, "\n\tMCPR_UMP_UMP_FIFO_REMAINING_WORDS_EGRESS_FIFO_OVERFLOW :"
				, (u32_t)((data & MCPR_UMP_UMP_FIFO_REMAINING_WORDS_EGRESS_FIFO_OVERFLOW) >> 15)
				, "\n\tMCPR_UMP_UMP_FIFO_REMAINING_WORDS_EGRESS_FIFO_UNDERFLOW :"
				, (u32_t)((data & MCPR_UMP_UMP_FIFO_REMAINING_WORDS_EGRESS_FIFO_UNDERFLOW) >> 14)
				, "\n\tMCPR_UMP_UMP_FIFO_REMAINING_WORDS_EGRESS_FIFO_DEPTH :"
				, (u32_t)((data & MCPR_UMP_UMP_FIFO_REMAINING_WORDS_EGRESS_FIFO_DEPTH) >> 0));
			break;
		case 0x980a8: // (MCP_REG_MCPR_UMP_UMP_EGRESS_FIFO_PTRS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_FIFO_PTRS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_EGRESS_FIFO_PTRS_UPDATE_WRPTR :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_FIFO_PTRS_UPDATE_WRPTR) >> 31)
				, "\n\tMCPR_UMP_UMP_EGRESS_FIFO_PTRS_EGRESS_FIFO_WR_PTR :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_FIFO_PTRS_EGRESS_FIFO_WR_PTR) >> 16)
				, "\n\tMCPR_UMP_UMP_EGRESS_FIFO_PTRS_UPDATE_RDPTR :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_FIFO_PTRS_UPDATE_RDPTR) >> 15)
				, "\n\tMCPR_UMP_UMP_EGRESS_FIFO_PTRS_EGRESS_FIFO_RD_PTR :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_FIFO_PTRS_EGRESS_FIFO_RD_PTR) >> 0));
			break;
		case 0x980ac: // (MCP_REG_MCPR_UMP_UMP_INGRESS_FIFO_PTRS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_INGRESS_FIFO_PTRS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_INGRESS_FIFO_PTRS_UPDATE_WRPTR :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_FIFO_PTRS_UPDATE_WRPTR) >> 31)
				, "\n\tMCPR_UMP_UMP_INGRESS_FIFO_PTRS_INGRESS_FIFO_WR_PTR :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_FIFO_PTRS_INGRESS_FIFO_WR_PTR) >> 16)
				, "\n\tMCPR_UMP_UMP_INGRESS_FIFO_PTRS_UPDATE_RDPTR :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_FIFO_PTRS_UPDATE_RDPTR) >> 15)
				, "\n\tMCPR_UMP_UMP_INGRESS_FIFO_PTRS_INGRESS_FIFO_RD_PTR :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_FIFO_PTRS_INGRESS_FIFO_RD_PTR) >> 0));
			break;
		case 0x980b0: // (MCP_REG_MCPR_UMP_UNUSED_Z: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UNUSED_Z (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x980b4: // (MCP_REG_MCPR_UMP_UMP_EGRESS_PACKET_SA_0: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_PACKET_SA_0 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_EGRESS_PACKET_SA_0_EGRESS_SA :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_PACKET_SA_0_EGRESS_SA) >> 0));
			break;
		case 0x980b8: // (MCP_REG_MCPR_UMP_UMP_EGRESS_PACKET_SA_1: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_PACKET_SA_1 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_EGRESS_PACKET_SA_1_EGRESS_SA :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_PACKET_SA_1_EGRESS_SA) >> 0));
			break;
		case 0x980bc: // (MCP_REG_MCPR_UMP_UMP_INGRESS_BURST_COMMAND: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_INGRESS_BURST_COMMAND (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_INGRESS_BURST_COMMAND_RBUF_OFFSET :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_BURST_COMMAND_RBUF_OFFSET) >> 16)
				, "\n\tMCPR_UMP_UMP_INGRESS_BURST_COMMAND_INGRESS_PORT_EXT :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_BURST_COMMAND_INGRESS_PORT_EXT) >> 13)
				, "\n\tMCPR_UMP_UMP_INGRESS_BURST_COMMAND_DMA_LENGTH :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_BURST_COMMAND_DMA_LENGTH) >> 2)
				, "\n\tMCPR_UMP_UMP_INGRESS_BURST_COMMAND_INGRESS_PORT :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_BURST_COMMAND_INGRESS_PORT) >> 1)
				, "\n\tMCPR_UMP_UMP_INGRESS_BURST_COMMAND_INGRESS_DMA_START :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_BURST_COMMAND_INGRESS_DMA_START) >> 0));
			break;
		case 0x980c0: // (MCP_REG_MCPR_UMP_UMP_INGRESS_RBUF_CLUSTER: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_INGRESS_RBUF_CLUSTER (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_INGRESS_RBUF_CLUSTER_RBUF_CLUSTER :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_RBUF_CLUSTER_RBUF_CLUSTER) >> 0));
			break;
		case 0x980c4: // (MCP_REG_MCPR_UMP_UMP_INGRESS_VLAN: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_INGRESS_VLAN (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_INGRESS_VLAN_VLAN_DEL :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_VLAN_VLAN_DEL) >> 17)
				, "\n\tMCPR_UMP_UMP_INGRESS_VLAN_VLAN_INS :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_VLAN_VLAN_INS) >> 16)
				, "\n\tMCPR_UMP_UMP_INGRESS_VLAN_INGRESS_VLAN_TAG :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_VLAN_INGRESS_VLAN_TAG) >> 0));
			break;
		case 0x980c8: // (MCP_REG_MCPR_UMP_UMP_INGRESS_BURST_STATUS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_INGRESS_BURST_STATUS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_UMP_UMP_INGRESS_BURST_STATUS_RESULT :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_BURST_STATUS_RESULT) >> 0),
				(u32_t)(data & MCPR_UMP_UMP_INGRESS_BURST_STATUS_RESULT) == MCPR_UMP_UMP_INGRESS_BURST_STATUS_RESULT_ERR1  ? "ERR1" :
				(u32_t)(data & MCPR_UMP_UMP_INGRESS_BURST_STATUS_RESULT) == MCPR_UMP_UMP_INGRESS_BURST_STATUS_RESULT_ERR  ? "ERR" :
				(u32_t)(data & MCPR_UMP_UMP_INGRESS_BURST_STATUS_RESULT) == MCPR_UMP_UMP_INGRESS_BURST_STATUS_RESULT_DONE  ? "DONE" :
				(u32_t)(data & MCPR_UMP_UMP_INGRESS_BURST_STATUS_RESULT) == MCPR_UMP_UMP_INGRESS_BURST_STATUS_RESULT_BUSY  ? "BUSY" :"INVALID");
			break;
		case 0x980cc: // (MCP_REG_MCPR_UMP_UMP_EGRESS_BURST_COMMAND: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_BURST_COMMAND (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_EGRESS_BURST_COMMAND_TPBUF_OFFSET :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_BURST_COMMAND_TPBUF_OFFSET) >> 16)
				, "\n\tMCPR_UMP_UMP_EGRESS_BURST_COMMAND_EGRESS_PORT_EXT :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_BURST_COMMAND_EGRESS_PORT_EXT) >> 13)
				, "\n\tMCPR_UMP_UMP_EGRESS_BURST_COMMAND_DMA_LENGTH :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_BURST_COMMAND_DMA_LENGTH) >> 2)
				, "\n\tMCPR_UMP_UMP_EGRESS_BURST_COMMAND_EGRESS_PORT :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_BURST_COMMAND_EGRESS_PORT) >> 1)
				, "\n\tMCPR_UMP_UMP_EGRESS_BURST_COMMAND_EGRESS_DMA_START :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_BURST_COMMAND_EGRESS_DMA_START) >> 0));
			break;
		case 0x980d0: // (MCP_REG_MCPR_UMP_UMP_EGRESS_VLAN: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_VLAN (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_EGRESS_VLAN_VLAN_DEL :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_VLAN_VLAN_DEL) >> 17)
				, "\n\tMCPR_UMP_UMP_EGRESS_VLAN_VLAN_INS :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_VLAN_VLAN_INS) >> 16)
				, "\n\tMCPR_UMP_UMP_EGRESS_VLAN_EGRESS_VLAN_TAG :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_VLAN_EGRESS_VLAN_TAG) >> 0));
			break;
		case 0x980d4: // (MCP_REG_MCPR_UMP_UMP_EGRESS_BURST_STATUS: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_BURST_STATUS (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x (%s)"
				, addr, data
				, "\n\tMCPR_UMP_UMP_EGRESS_BURST_STATUS_RESULT :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_BURST_STATUS_RESULT) >> 0),
				(u32_t)(data & MCPR_UMP_UMP_EGRESS_BURST_STATUS_RESULT) == MCPR_UMP_UMP_EGRESS_BURST_STATUS_RESULT_RSVD  ? "RSVD" :
				(u32_t)(data & MCPR_UMP_UMP_EGRESS_BURST_STATUS_RESULT) == MCPR_UMP_UMP_EGRESS_BURST_STATUS_RESULT_ERR0  ? "ERR0" :
				(u32_t)(data & MCPR_UMP_UMP_EGRESS_BURST_STATUS_RESULT) == MCPR_UMP_UMP_EGRESS_BURST_STATUS_RESULT_DONE  ? "DONE" :
				(u32_t)(data & MCPR_UMP_UMP_EGRESS_BURST_STATUS_RESULT) == MCPR_UMP_UMP_EGRESS_BURST_STATUS_RESULT_BUSY  ? "BUSY" :"INVALID");
			break;
		case 0x980d8: // (MCP_REG_MCPR_UMP_UMP_EGRESS_STATISTIC: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_STATISTIC (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_EGRESS_STATISTIC_EGRESS_DROP_CNT :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_STATISTIC_EGRESS_DROP_CNT) >> 24)
				, "\n\tMCPR_UMP_UMP_EGRESS_STATISTIC_EGRESS_ERROR_CNT :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_STATISTIC_EGRESS_ERROR_CNT) >> 16)
				, "\n\tMCPR_UMP_UMP_EGRESS_STATISTIC_EGRESS_GOOD_CNT :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_STATISTIC_EGRESS_GOOD_CNT) >> 0));
			break;
		case 0x980dc: // (MCP_REG_MCPR_UMP_UMP_INGRESS_STATISTIC: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_INGRESS_STATISTIC (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_INGRESS_STATISTIC_INGRESS_PKT_CNT :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_STATISTIC_INGRESS_PKT_CNT) >> 0));
			break;
		case 0x980e0: // (MCP_REG_MCPR_UMP_UMP_ARB_CMD: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_ARB_CMD (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_ARB_CMD_UMP_ARB_TIMEOUT :"
				, (u32_t)((data & MCPR_UMP_UMP_ARB_CMD_UMP_ARB_TIMEOUT) >> 16)
				, "\n\tMCPR_UMP_UMP_ARB_CMD_UMP_ARB_FC_DISABLE :"
				, (u32_t)((data & MCPR_UMP_UMP_ARB_CMD_UMP_ARB_FC_DISABLE) >> 15)
				, "\n\tMCPR_UMP_UMP_ARB_CMD_UMP_ARB_TOKEN_VALID :"
				, (u32_t)((data & MCPR_UMP_UMP_ARB_CMD_UMP_ARB_TOKEN_VALID) >> 13)
				, "\n\tMCPR_UMP_UMP_ARB_CMD_UMP_ARB_TOKEN_IPG :"
				, (u32_t)((data & MCPR_UMP_UMP_ARB_CMD_UMP_ARB_TOKEN_IPG) >> 8)
				, "\n\tMCPR_UMP_UMP_ARB_CMD_UMP_ARB_AUTOBYPASS :"
				, (u32_t)((data & MCPR_UMP_UMP_ARB_CMD_UMP_ARB_AUTOBYPASS) >> 7)
				, "\n\tMCPR_UMP_UMP_ARB_CMD_UMP_ARB_BYPASS :"
				, (u32_t)((data & MCPR_UMP_UMP_ARB_CMD_UMP_ARB_BYPASS) >> 6)
				, "\n\tMCPR_UMP_UMP_ARB_CMD_UMP_ARB_START :"
				, (u32_t)((data & MCPR_UMP_UMP_ARB_CMD_UMP_ARB_START) >> 5)
				, "\n\tMCPR_UMP_UMP_ARB_CMD_UMP_ARB_DISABLE :"
				, (u32_t)((data & MCPR_UMP_UMP_ARB_CMD_UMP_ARB_DISABLE) >> 4)
				, "\n\tMCPR_UMP_UMP_ARB_CMD_UMP_ID :"
				, (u32_t)((data & MCPR_UMP_UMP_ARB_CMD_UMP_ID) >> 0));
			break;
		case 0x980e4: // (MCP_REG_MCPR_UMP_UNUSED_F: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UNUSED_F (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x980f0: // (MCP_REG_MCPR_UMP_UMP_EGRESS_STATISTIC_AC: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_STATISTIC_AC (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_EGRESS_STATISTIC_AC_EGRESS_DROP_CNT :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_STATISTIC_AC_EGRESS_DROP_CNT) >> 24)
				, "\n\tMCPR_UMP_UMP_EGRESS_STATISTIC_AC_EGRESS_ERROR_CNT :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_STATISTIC_AC_EGRESS_ERROR_CNT) >> 16)
				, "\n\tMCPR_UMP_UMP_EGRESS_STATISTIC_AC_EGRESS_GOOD_CNT :"
				, (u32_t)((data & MCPR_UMP_UMP_EGRESS_STATISTIC_AC_EGRESS_GOOD_CNT) >> 0));
			break;
		case 0x980f4: // (MCP_REG_MCPR_UMP_UMP_INGRESS_STATISTIC_AC: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_INGRESS_STATISTIC_AC (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_INGRESS_STATISTIC_AC_INGRESS_PKT_CNT :"
				, (u32_t)((data & MCPR_UMP_UMP_INGRESS_STATISTIC_AC_INGRESS_PKT_CNT) >> 0));
			break;
		case 0x980f8: // (MCP_REG_MCPR_UMP_UMP_EVENT: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EVENT (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x %-20s 0x%x "
				, addr, data
				, "\n\tMCPR_UMP_UMP_EVENT_EGRESS_FRAME_DROP_EVENT_EN :"
				, (u32_t)((data & MCPR_UMP_UMP_EVENT_EGRESS_FRAME_DROP_EVENT_EN) >> 20)
				, "\n\tMCPR_UMP_UMP_EVENT_EGRESSBURST_DONE_EVENT_EN :"
				, (u32_t)((data & MCPR_UMP_UMP_EVENT_EGRESSBURST_DONE_EVENT_EN) >> 19)
				, "\n\tMCPR_UMP_UMP_EVENT_INGRESSBURST_DONE_EVENT_EN :"
				, (u32_t)((data & MCPR_UMP_UMP_EVENT_INGRESSBURST_DONE_EVENT_EN) >> 18)
				, "\n\tMCPR_UMP_UMP_EVENT_EGRESS_RDY_EVENT_EN :"
				, (u32_t)((data & MCPR_UMP_UMP_EVENT_EGRESS_RDY_EVENT_EN) >> 17)
				, "\n\tMCPR_UMP_UMP_EVENT_INGRESS_RDY_EVENT_EN :"
				, (u32_t)((data & MCPR_UMP_UMP_EVENT_INGRESS_RDY_EVENT_EN) >> 16)
				, "\n\tMCPR_UMP_UMP_EVENT_EGRESS_FRAME_DROP_EVENT :"
				, (u32_t)((data & MCPR_UMP_UMP_EVENT_EGRESS_FRAME_DROP_EVENT) >> 4)
				, "\n\tMCPR_UMP_UMP_EVENT_EGRESSBURST_DONE_EVENT :"
				, (u32_t)((data & MCPR_UMP_UMP_EVENT_EGRESSBURST_DONE_EVENT) >> 3)
				, "\n\tMCPR_UMP_UMP_EVENT_INGRESSBURST_DONE_EVENT :"
				, (u32_t)((data & MCPR_UMP_UMP_EVENT_INGRESSBURST_DONE_EVENT) >> 2)
				, "\n\tMCPR_UMP_UMP_EVENT_EGRESS_RDY_EVENT :"
				, (u32_t)((data & MCPR_UMP_UMP_EVENT_EGRESS_RDY_EVENT) >> 1)
				, "\n\tMCPR_UMP_UMP_EVENT_INGRESS_RDY_EVENT :"
				, (u32_t)((data & MCPR_UMP_UMP_EVENT_INGRESS_RDY_EVENT) >> 0));
			break;
		case 0x980fc: // (MCP_REG_MCPR_UNUSED11: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED11 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9c000: // (MCP_REG_MCPR_UMP_UMP_EGRESS_FIFO_FLAT_SPACE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_EGRESS_FIFO_FLAT_SPACE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9de00: // (MCP_REG_MCPR_UNUSED12: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED12 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9e000: // (MCP_REG_MCPR_UMP_UMP_INGRESS_FIFO_FLAT_SPACE: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UMP_UMP_INGRESS_FIFO_FLAT_SPACE (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x9ec00: // (MCP_REG_MCPR_UNUSED13: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_UNUSED13 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0xa0000: // (MCP_REG_MCPR_SCRATCH: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_REG_MCPR_SCRATCH (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3a0000: // (MCP_A_REG_MCPR_SCRATCH: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_A_REG_MCPR_SCRATCH (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		case 0x3c8000: // (MCP_A_REG_MCPR_UNUSED14: ACCESS:??  DataWidth:0x20) 
			sprintf(disp_str, "\nMCP_A_REG_MCPR_UNUSED14 (ACCESS:??  DataWidth:0x20)" \
				"\nDescription: none.\n(GRC %05x=%08x) "
				, addr, data);
			break;
		default:
			sprintf(disp_str, "\n(Unknown GRC%05x=%08x)", addr, data);
			break;
	}
	Tcl_AppendStringsToObj( rc_ptr, disp_str, NULL);

} // DecodeGrc
